
//-------------------------------------------------------------------------
//   Copyright 2002-2022 National Technology & Engineering Solutions of
//   Sandia, LLC (NTESS).  Under the terms of Contract DE-NA0003525 with
//   NTESS, the U.S. Government retains certain rights in this software.
//
//   This file is part of the Xyce(TM) Parallel Electrical Simulator.
//
//   Xyce(TM) is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Xyce(TM) is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Xyce(TM).
//   If not, see <http://www.gnu.org/licenses/>.
//-------------------------------------------------------------------------


//-------------------------------------------------------------------------
//
// Purpose        :
//
// Special Notes  : Generated from verilog file bsimsoi.va with ADMS
//                  interface for Xyce 7.6.0
//                  DO NOT EDIT THIS FILE DIRECTLY!  It may be overwritten!
//
// Creator        : admsXml-2.3.7
//
// Creation Date  : Tue, 18 Oct 2022 17:52:28
//
//-------------------------------------------------------------------------
// Shut up clang's warnings about extraneous parentheses
#ifdef __clang__
#  pragma clang diagnostic ignored "-Wparentheses-equality"
#endif

#include <Xyce_config.h>
#include <N_DEV_ADMSbsimsoi.h>


#include <N_DEV_Const.h>
#include <N_DEV_DeviceOptions.h>
#include <N_DEV_DeviceMaster.h>
#include <N_DEV_ExternData.h>
#include <N_DEV_SolverState.h>
#include <N_DEV_Message.h>

#include <N_LAS_Matrix.h>
#include <N_LAS_Vector.h>

  #include <N_ANP_NoiseData.h>

#include <N_UTL_FeatureTest.h>
#include <unordered_map>
using std::unordered_map;

#include <algorithm>

// Xyce_config.h contains a VERSION macro from autoconf, and some
// Verilog-A models like to define a variable of that name.  This can be
// a serious problem, and we don't need the VERSION macro.  Get rid of it.
// This must happen *after* all includes of Xyce headers, each of which
// includes Xyce_config.h!
#ifdef VERSION
#undef VERSION
#endif

namespace Xyce {
namespace Device {
namespace ADMSbsimsoi {
namespace AnalogFunctions {
// Derivative of Analog Function lln
double d_lln(double x  , double d_x )
{
// Function return variable and total deriv
double lln;
double d_lln;
// Derivatives of return value w.r.t input vars
double d_lln_d_x;
{
d_lln_d_x=((x>1.0e-38)?(1.0/x):0.0);
lln=((x>1.0e-38)?log(x):(-87.498233533773742465));
}
d_lln = d_lln_d_x*d_x;
return(d_lln);
}

// Evaluator class implementations for Analog Function lln
  // Constructor
  llnEvaluator::llnEvaluator(double x)
  {
    llnReturn_ = evaluator_(x);
  }
  // method to get precomputed values into double vars 
  double llnEvaluator::getValues(double  x)
  {
    // Silence unused argument warnings
    (void) x;
    // Copy all precomputed values into corresponding output
    return(llnReturn_.value);
  }
  // method to get total deriv w.r.t some variable via chain rule 
  // given precomputed derivs of function w.r.t. args and derivs of args
  // w.r.t desired vars
  double llnEvaluator::getDerivs(double x  , double d_x)
  {
    // Function total deriv
    double d_lln;
    // Silence unused argument warnings
    (void) x;
    d_lln = llnReturn_.deriv_WRT_x*d_x;
    return(d_lln);
  }
  // method that actually performs our computations.
  llnEvaluator::returnType llnEvaluator::evaluator_(double x)
  {
  // Function value and derivs variables, and a returnType to store everything
  double lln;
  llnEvaluator::returnType llnReturn;
  // Derivatives of return value w.r.t input vars
  double d_lln_d_x;
  // declared local variables
{
d_lln_d_x=((x>1.0e-38)?(1.0/x):0.0);
lln=((x>1.0e-38)?log(x):(-87.498233533773742465));
}
  // now save outputs and derivs into appropriate return vars
  llnReturn.value=lln;
  llnReturn.deriv_WRT_x = d_lln_d_x;
  return(llnReturn);
  }

// Derivative of Analog Function hypsmooth
double d_hypsmooth(double x , double c  , double d_x , double d_c )
{
// Function return variable and total deriv
double hypsmooth;
double d_hypsmooth;
// Derivatives of return value w.r.t input vars
double d_hypsmooth_d_x;
double d_hypsmooth_d_c;
{
d_hypsmooth_d_x=(0.5*(1.0+(0.5/sqrt(((x*x)+((4.0*c)*c))))*(x+x)));
d_hypsmooth_d_c=(0.5*(0.5/sqrt(((x*x)+((4.0*c)*c))))*((4.0*c)+c*(4.0)));
hypsmooth=(0.5*(x+sqrt(((x*x)+((4.0*c)*c)))));
}
d_hypsmooth = d_hypsmooth_d_x*d_x+d_hypsmooth_d_c*d_c;
return(d_hypsmooth);
}

// Evaluator class implementations for Analog Function hypsmooth
  // Constructor
  hypsmoothEvaluator::hypsmoothEvaluator(double x, double c)
  {
    hypsmoothReturn_ = evaluator_(x, c);
  }
  // method to get precomputed values into double vars 
  double hypsmoothEvaluator::getValues(double  x, double  c)
  {
    // Silence unused argument warnings
    (void) x;
    (void) c;
    // Copy all precomputed values into corresponding output
    return(hypsmoothReturn_.value);
  }
  // method to get total deriv w.r.t some variable via chain rule 
  // given precomputed derivs of function w.r.t. args and derivs of args
  // w.r.t desired vars
  double hypsmoothEvaluator::getDerivs(double x , double c  , double d_x, double d_c)
  {
    // Function total deriv
    double d_hypsmooth;
    // Silence unused argument warnings
    (void) x;
    (void) c;
    d_hypsmooth = hypsmoothReturn_.deriv_WRT_x*d_x+hypsmoothReturn_.deriv_WRT_c*d_c;
    return(d_hypsmooth);
  }
  // method that actually performs our computations.
  hypsmoothEvaluator::returnType hypsmoothEvaluator::evaluator_(double x, double c)
  {
  // Function value and derivs variables, and a returnType to store everything
  double hypsmooth;
  hypsmoothEvaluator::returnType hypsmoothReturn;
  // Derivatives of return value w.r.t input vars
  double d_hypsmooth_d_x;
  double d_hypsmooth_d_c;
  // declared local variables
{
d_hypsmooth_d_x=(0.5*(1.0+(0.5/sqrt(((x*x)+((4.0*c)*c))))*(x+x)));
d_hypsmooth_d_c=(0.5*(0.5/sqrt(((x*x)+((4.0*c)*c))))*((4.0*c)+c*(4.0)));
hypsmooth=(0.5*(x+sqrt(((x*x)+((4.0*c)*c)))));
}
  // now save outputs and derivs into appropriate return vars
  hypsmoothReturn.value=hypsmooth;
  hypsmoothReturn.deriv_WRT_x = d_hypsmooth_d_x;
  hypsmoothReturn.deriv_WRT_c = d_hypsmooth_d_c;
  return(hypsmoothReturn);
  }

} // namepace AnalogFunctions


//This is necessary because the IntPair usage can trip undefined references
    const int Instance::admsNodeID_d;
    const int Instance::admsNodeID_g;
    const int Instance::admsNodeID_s;
    const int Instance::admsNodeID_e;
    const int Instance::admsNodeID_p;
    const int Instance::admsNodeID_b;
    const int Instance::admsNodeID_t;
    const int Instance::admsNodeID_di;
    const int Instance::admsNodeID_si;
    const int Instance::admsNodeID_gi;
    const int Instance::admsNodeID_gm;
    const int Instance::admsNodeID_sb;
    const int Instance::admsNodeID_db;
     const int Instance::admsNodeID_GND;
   // Additional IDs for branch equations


void
Traits::loadInstanceParameters(ParametricData<ADMSbsimsoi::Instance> &p)
{
  // Set up each parameter directly, using the up-cased variable name
  // as the tag.

  // This kludge is to force us always to have an instance parameter
  // that the device manager can set to the temperature, even if we have
  // no "TEMP".
  p.addPar("XYCEADMSINSTTEMP", 0.0, &ADMSbsimsoi::Instance::admsInstTemp)
    .setExpressionAccess(NO_DOC)
    .setUnit(U_DEGK)
    .setCategory(CAT_TEMP)
    .setDescription("Internal-use parameter for setting device instance temperature");
  p.addPar("DTEMP", static_cast<double>(0.0), &ADMSbsimsoi::Instance::DTEMP)
    .setUnit(U_UNKNOWN)
    .setDescription("device temperature offset from ambient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("L", static_cast<double>(5e-6), &ADMSbsimsoi::Instance::L)
    .setUnit(U_METER)
    .setDescription("Length")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("W", static_cast<double>(5e-6), &ADMSbsimsoi::Instance::W)
    .setUnit(U_METER)
    .setDescription("Width")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NF", static_cast<int>(1), &ADMSbsimsoi::Instance::NF)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of fingers");
  p.addPar("SA", static_cast<double>(0.0), &ADMSbsimsoi::Instance::SA)
    .setUnit(U_METER)
    .setDescription("distance between  OD edge to poly of one side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SB", static_cast<double>(0.0), &ADMSbsimsoi::Instance::SB)
    .setUnit(U_METER)
    .setDescription("distance between  OD edge to poly of the other side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SD", static_cast<double>(0.0), &ADMSbsimsoi::Instance::SD)
    .setUnit(U_METER)
    .setDescription("distance between neighbor fingers")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AD", static_cast<double>(0.0), &ADMSbsimsoi::Instance::AD)
    .setUnit(U_METER2)
    .setDescription("Drain area")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AS", static_cast<double>(0.0), &ADMSbsimsoi::Instance::AS)
    .setUnit(U_METER2)
    .setDescription("Source area")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PD", static_cast<double>(0.0), &ADMSbsimsoi::Instance::PD)
    .setUnit(U_METER)
    .setDescription("Drain perimeter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PS", static_cast<double>(0.0), &ADMSbsimsoi::Instance::PS)
    .setUnit(U_METER)
    .setDescription("Source perimeter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRD", static_cast<double>(1.0), &ADMSbsimsoi::Instance::NRD)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of squares in drain")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRS", static_cast<double>(1.0), &ADMSbsimsoi::Instance::NRS)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of squares in source")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("OFF", static_cast<int>(0), &ADMSbsimsoi::Instance::OFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Device is initially off");
  p.addPar("BJTOFF", static_cast<int>(0), &ADMSbsimsoi::Instance::BJTOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("BJT on/off flag");
  p.addPar("DEBUG", static_cast<int>(0), &ADMSbsimsoi::Instance::DEBUG)
    .setUnit(U_UNKNOWN)
    .setDescription("DEBUG on/off flag");
  p.addPar("RTH0", static_cast<double>(0.0), &ADMSbsimsoi::Instance::RTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Instance Thermal Resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CTH0", static_cast<double>(1e-5), &ADMSbsimsoi::Instance::CTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Instance Thermal Capacitance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRB", static_cast<double>(1.0), &ADMSbsimsoi::Instance::NRB)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of squares in body")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FRBODY", static_cast<double>(1.0), &ADMSbsimsoi::Instance::FRBODY)
    .setUnit(U_UNKNOWN)
    .setDescription("layout dependent body-resistance coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RBDB", static_cast<double>(50.0), &ADMSbsimsoi::Instance::RBDB)
    .setUnit(U_UNKNOWN)
    .setDescription("Body resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RBSB", static_cast<double>(50.0), &ADMSbsimsoi::Instance::RBSB)
    .setUnit(U_UNKNOWN)
    .setDescription("Body resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DELVTO", static_cast<double>(0.0), &ADMSbsimsoi::Instance::DELVTO)
    .setUnit(U_VOLT)
    .setDescription("Zero bias threshold voltage variation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SOIMOD", static_cast<int>(0), &ADMSbsimsoi::Instance::SOIMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Instance model selector for PD/FD operation /* v3.2 ");
  p.addPar("NBC", static_cast<double>(0.0), &ADMSbsimsoi::Instance::NBC)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of body contact isolation edge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NSEG", static_cast<double>(1), &ADMSbsimsoi::Instance::NSEG)
    .setUnit(U_UNKNOWN)
    .setDescription("Number segments for width partitioning")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDBCP", static_cast<double>(0.0), &ADMSbsimsoi::Instance::PDBCP)
    .setUnit(U_METER)
    .setDescription("Perimeter length for bc parasitics at drain side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PSBCP", static_cast<double>(0.0), &ADMSbsimsoi::Instance::PSBCP)
    .setUnit(U_METER)
    .setDescription("Perimeter length for bc parasitics at source side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBCP", static_cast<double>(0.0), &ADMSbsimsoi::Instance::AGBCP)
    .setUnit(U_METER2)
    .setDescription("Gate to body overlap area for bc parasitics")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Instance::AGBCP2)
    .setUnit(U_METER2)
    .setDescription("Parasitic Gate to body overlap area for bc parasitics /* v4.1 improvement on BC ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBCPD", 0.0, &ADMSbsimsoi::Instance::AGBCPD)
    .setUnit(U_METER2)
    .setDescription("Gate to body overlap area for bc parasitics in DC")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AEBCP", static_cast<double>(0.0), &ADMSbsimsoi::Instance::AEBCP)
    .setUnit(U_METER2)
    .setDescription("Substrate to body overlap area for bc parasitics")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TNODEOUT", static_cast<int>(0), &ADMSbsimsoi::Instance::TNODEOUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Flag indicating external temp node");
  p.addPar("SHMOD", static_cast<int>(0), &ADMSbsimsoi::Instance::SHMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Self heating mode selector");
  p.addPar("M", static_cast<double>(1.0), &ADMSbsimsoi::Instance::m)
    .setUnit(U_UNKNOWN)
    .setDescription("multiplicity factor")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&instSens)
#endif // Xyce_ADMS_SENSITIVITIES
;


}

void Traits::loadModelParameters(ParametricData<ADMSbsimsoi::Model> &p)
{
  // Set up each parameter directly, using the up-cased variable name
  // as the tag.

  // This kludge is to force us always to have a model parameter
  // that the device manager can set to the temperature, even if we have
  // no "TEMP".
  p.addPar("XYCEADMSMODTEMP", 0.0, &ADMSbsimsoi::Model::admsModTemp)
    .setExpressionAccess(NO_DOC)
    .setUnit(U_DEGK)
    .setCategory(CAT_TEMP)
    .setDescription("Internal-use parameter for setting device model temperature");
  p.addPar("L", static_cast<double>(5e-6), &ADMSbsimsoi::Model::L)
    .setUnit(U_METER)
    .setDescription("Length")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("W", static_cast<double>(5e-6), &ADMSbsimsoi::Model::W)
    .setUnit(U_METER)
    .setDescription("Width")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NF", static_cast<int>(1), &ADMSbsimsoi::Model::NF)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of fingers");
  p.addPar("SA", static_cast<double>(0.0), &ADMSbsimsoi::Model::SA)
    .setUnit(U_METER)
    .setDescription("distance between  OD edge to poly of one side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SB", static_cast<double>(0.0), &ADMSbsimsoi::Model::SB)
    .setUnit(U_METER)
    .setDescription("distance between  OD edge to poly of the other side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SD", static_cast<double>(0.0), &ADMSbsimsoi::Model::SD)
    .setUnit(U_METER)
    .setDescription("distance between neighbor fingers")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AD", static_cast<double>(0.0), &ADMSbsimsoi::Model::AD)
    .setUnit(U_METER2)
    .setDescription("Drain area")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AS", static_cast<double>(0.0), &ADMSbsimsoi::Model::AS)
    .setUnit(U_METER2)
    .setDescription("Source area")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PD)
    .setUnit(U_METER)
    .setDescription("Drain perimeter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PS", static_cast<double>(0.0), &ADMSbsimsoi::Model::PS)
    .setUnit(U_METER)
    .setDescription("Source perimeter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRD", static_cast<double>(1.0), &ADMSbsimsoi::Model::NRD)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of squares in drain")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRS", static_cast<double>(1.0), &ADMSbsimsoi::Model::NRS)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of squares in source")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("OFF", static_cast<int>(0), &ADMSbsimsoi::Model::OFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Device is initially off");
  p.addPar("BJTOFF", static_cast<int>(0), &ADMSbsimsoi::Model::BJTOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("BJT on/off flag");
  p.addPar("RTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::RTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Instance Thermal Resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CTH0", static_cast<double>(1e-5), &ADMSbsimsoi::Model::CTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Instance Thermal Capacitance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRB", static_cast<double>(1.0), &ADMSbsimsoi::Model::NRB)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of squares in body")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FRBODY", static_cast<double>(1.0), &ADMSbsimsoi::Model::FRBODY)
    .setUnit(U_UNKNOWN)
    .setDescription("layout dependent body-resistance coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RBDB", static_cast<double>(50.0), &ADMSbsimsoi::Model::RBDB)
    .setUnit(U_UNKNOWN)
    .setDescription("Body resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RBSB", static_cast<double>(50.0), &ADMSbsimsoi::Model::RBSB)
    .setUnit(U_UNKNOWN)
    .setDescription("Body resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DELVTO", static_cast<double>(0.0), &ADMSbsimsoi::Model::DELVTO)
    .setUnit(U_VOLT)
    .setDescription("Zero bias threshold voltage variation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SOIMOD", static_cast<int>(0), &ADMSbsimsoi::Model::SOIMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Instance model selector for PD/FD operation /* v3.2 ");
  p.addPar("NBC", static_cast<double>(0.0), &ADMSbsimsoi::Model::NBC)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of body contact isolation edge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NSEG", static_cast<double>(1), &ADMSbsimsoi::Model::NSEG)
    .setUnit(U_UNKNOWN)
    .setDescription("Number segments for width partitioning")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDBCP", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDBCP)
    .setUnit(U_METER)
    .setDescription("Perimeter length for bc parasitics at drain side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PSBCP", static_cast<double>(0.0), &ADMSbsimsoi::Model::PSBCP)
    .setUnit(U_METER)
    .setDescription("Perimeter length for bc parasitics at source side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBCP", static_cast<double>(0.0), &ADMSbsimsoi::Model::AGBCP)
    .setUnit(U_METER2)
    .setDescription("Gate to body overlap area for bc parasitics")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::AGBCP2)
    .setUnit(U_METER2)
    .setDescription("Parasitic Gate to body overlap area for bc parasitics /* v4.1 improvement on BC ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBCPD", 0.0, &ADMSbsimsoi::Model::AGBCPD)
    .setUnit(U_METER2)
    .setDescription("Gate to body overlap area for bc parasitics in DC")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AEBCP", static_cast<double>(0.0), &ADMSbsimsoi::Model::AEBCP)
    .setUnit(U_METER2)
    .setDescription("Substrate to body overlap area for bc parasitics")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SHMOD", static_cast<int>(0), &ADMSbsimsoi::Model::SHMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Self heating mode selector");
  p.addPar("TYPE", static_cast<int>(1), &ADMSbsimsoi::Model::TYPE)
    .setUnit(U_UNKNOWN)
    .setDescription("+1 = NMOS, -1 = PMOS");
  p.addPar("VERSION", static_cast<double>(4.60), &ADMSbsimsoi::Model::VERSION)
    .setUnit(U_UNKNOWN)
    .setDescription("parameter for model version")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VBSUSR", static_cast<double>(0.0), &ADMSbsimsoi::Model::VBSUSR)
    .setUnit(U_VOLT)
    .setDescription("Vbs specified by user")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RGATEMOD", static_cast<int>(0), &ADMSbsimsoi::Model::RGATEMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate resistance model selector");
  p.addPar("RBODYMOD", static_cast<int>(0), &ADMSbsimsoi::Model::RBODYMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Body R model selector /* v4.0 ");
  p.addPar("MTRLMOD", static_cast<int>(0), &ADMSbsimsoi::Model::MTRLMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("parameter for non-silicon substrate or metal gate selector");
  p.addPar("VGSTCVMOD", 0, &ADMSbsimsoi::Model::VGSTCVMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Improved VgsteffCV selector");
  p.addPar("GIDLMOD", static_cast<int>(0), &ADMSbsimsoi::Model::GIDLMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("parameter for GIDL selector");
  p.addPar("IIIMOD", static_cast<int>(0), &ADMSbsimsoi::Model::IIIMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("parameter for III selector");
  p.addPar("EOT", static_cast<double>(100.0e-10), &ADMSbsimsoi::Model::EOT)
    .setUnit(U_METER)
    .setDescription("Effective SiO2 thickness")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EPSROX", static_cast<double>(3.9), &ADMSbsimsoi::Model::EPSROX)
    .setUnit(U_UNKNOWN)
    .setDescription("Dielectric constant of the gate oxide relative to vacuum")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EPSRSUB", static_cast<double>(11.7), &ADMSbsimsoi::Model::EPSRSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Dielectric constant of substrate relative to vacuum")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NI0SUB", static_cast<double>(1.45e10), &ADMSbsimsoi::Model::NI0SUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Intrinsic carrier concentration of substrate at Tnom")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BG0SUB", static_cast<double>(1.16), &ADMSbsimsoi::Model::BG0SUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Band-gap of substrate at T=0K")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TBGASUB", static_cast<double>(7.02e-4), &ADMSbsimsoi::Model::TBGASUB)
    .setUnit(U_UNKNOWN)
    .setDescription("First parameter of band-gap change due to temperature")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TBGBSUB", static_cast<double>(1108.0), &ADMSbsimsoi::Model::TBGBSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Second parameter of band-gap change due to temperature")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PHIG", static_cast<double>(4.05), &ADMSbsimsoi::Model::PHIG)
    .setUnit(U_UNKNOWN)
    .setDescription("Work function of gate")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EASUB", static_cast<double>(4.05), &ADMSbsimsoi::Model::EASUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Electron affinity of substrate")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LEFFEOT", static_cast<double>(1.0), &ADMSbsimsoi::Model::LEFFEOT)
    .setUnit(U_UNKNOWN)
    .setDescription("Effective length for extraction of EOT")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WEFFEOT", static_cast<double>(10.0), &ADMSbsimsoi::Model::WEFFEOT)
    .setUnit(U_UNKNOWN)
    .setDescription("Effective width for extraction of EOT")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VDDEOT", 0.0, &ADMSbsimsoi::Model::VDDEOT)
    .setUnit(U_UNKNOWN)
    .setDescription("Voltage for extraction of EOT")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TEMPEOT", static_cast<double>(300.15), &ADMSbsimsoi::Model::TEMPEOT)
    .setUnit(U_DEGK)
    .setDescription("Temperature for extraction of EOT")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ADOS", static_cast<double>(1.0), &ADMSbsimsoi::Model::ADOS)
    .setUnit(U_UNKNOWN)
    .setDescription("Charge centroid parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BDOS", static_cast<double>(1.0), &ADMSbsimsoi::Model::BDOS)
    .setUnit(U_UNKNOWN)
    .setDescription("Charge centroid parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EPSRGATE", static_cast<double>(11.7), &ADMSbsimsoi::Model::EPSRGATE)
    .setUnit(U_UNKNOWN)
    .setDescription("Dielectric constant of gate relative to vacuum")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CAPMOD", static_cast<int>(2), &ADMSbsimsoi::Model::CAPMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Capacitance model selector");
  p.addPar("MOBMOD", static_cast<int>(1), &ADMSbsimsoi::Model::MOBMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Mobility model selector");
  p.addPar("PARAMCHK", static_cast<int>(0), &ADMSbsimsoi::Model::PARAMCHK)
    .setUnit(U_UNKNOWN)
    .setDescription("Model parameter checking selector");
  p.addPar("NODECHK", static_cast<int>(1), &ADMSbsimsoi::Model::NODECHK)
    .setUnit(U_UNKNOWN)
    .setDescription("NODE checking flag");
  p.addPar("BINUNIT", static_cast<int>(1), &ADMSbsimsoi::Model::BINUNIT)
    .setUnit(U_UNKNOWN)
    .setDescription("Bin  unit  selector");
  p.addPar("TOX", static_cast<double>(100.0e-10), &ADMSbsimsoi::Model::TOX)
    .setUnit(U_METER)
    .setDescription("Gate oxide thickness in meters")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TOXP", 0.0, &ADMSbsimsoi::Model::TOXP)
    .setUnit(U_METER)
    .setDescription("Physical gate oxide thickness")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TOXM", 0.0, &ADMSbsimsoi::Model::TOXM)
    .setUnit(U_METER)
    .setDescription("Gate oxide thickness used in extraction /* v3.2 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DTOXCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::DTOXCV)
    .setUnit(U_METER)
    .setDescription("Delta oxide thickness in meters in CapMod3 /* v2.2.3 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CDSC", static_cast<double>(2.4e-4), &ADMSbsimsoi::Model::CDSC)
    .setUnit(U_UNKNOWN)
    .setDescription("Drain/Source and channel coupling capacitance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CDSCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::CDSCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Body-bias dependence of cdsc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CDSCD", static_cast<double>(0.0), &ADMSbsimsoi::Model::CDSCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Drain-bias dependence of cdsc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CIT", static_cast<double>(0.0), &ADMSbsimsoi::Model::CIT)
    .setUnit(U_UNKNOWN)
    .setDescription("Interface state capacitance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NFACTOR", static_cast<double>(1), &ADMSbsimsoi::Model::NFACTOR)
    .setUnit(U_UNKNOWN)
    .setDescription("Subthreshold swing Coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VSAT", static_cast<double>(8.0e4), &ADMSbsimsoi::Model::VSAT)
    .setUnit(U_UNKNOWN)
    .setDescription("Saturation velocity at tnom")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AT", static_cast<double>(3.3e4), &ADMSbsimsoi::Model::AT)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of vsat")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("A0", static_cast<double>(1.0), &ADMSbsimsoi::Model::A0)
    .setUnit(U_UNKNOWN)
    .setDescription("Non-uniform depletion width effect coefficient.")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGS", static_cast<double>(0.0), &ADMSbsimsoi::Model::AGS)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate bias  coefficient of Abulk.")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("A1", static_cast<double>(0.0), &ADMSbsimsoi::Model::A1)
    .setUnit(U_UNKNOWN)
    .setDescription("Non-saturation effect coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("A2", static_cast<double>(1.0), &ADMSbsimsoi::Model::A2)
    .setUnit(U_UNKNOWN)
    .setDescription("Non-saturation effect coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KETA", static_cast<double>((-0.6)), &ADMSbsimsoi::Model::KETA)
    .setUnit(U_UNKNOWN)
    .setDescription("Body-bias coefficient of non-uniform depletion width effect.")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NSUB", static_cast<double>(6.0e16), &ADMSbsimsoi::Model::NSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Substrate doping concentration with polarity")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NCH", static_cast<double>(1.7e17), &ADMSbsimsoi::Model::NCH)
    .setUnit(U_UNKNOWN)
    .setDescription("Channel doping concentration")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NGATE", static_cast<double>(0.0), &ADMSbsimsoi::Model::NGATE)
    .setUnit(U_UNKNOWN)
    .setDescription("Poly-gate doping concentration")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NSD", static_cast<double>(1.0e20), &ADMSbsimsoi::Model::NSD)
    .setUnit(U_UNKNOWN)
    .setDescription("S/D doping concentration")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("GAMMA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::GAMMA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Vth body coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("GAMMA2", static_cast<double>(0.0), &ADMSbsimsoi::Model::GAMMA2)
    .setUnit(U_UNKNOWN)
    .setDescription("Vth body coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VBX", static_cast<double>(0.0), &ADMSbsimsoi::Model::VBX)
    .setUnit(U_UNKNOWN)
    .setDescription("Vth transition body Voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VBM", static_cast<double>((-3.0)), &ADMSbsimsoi::Model::VBM)
    .setUnit(U_VOLT)
    .setDescription("Maximum body voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XT", static_cast<double>(1.55e-7), &ADMSbsimsoi::Model::XT)
    .setUnit(U_METER)
    .setDescription("Doping depth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K1", static_cast<double>(0.53), &ADMSbsimsoi::Model::K1)
    .setUnit(U_UNKNOWN)
    .setDescription("Bulk effect coefficient 1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KT1", static_cast<double>((-0.11)), &ADMSbsimsoi::Model::KT1)
    .setUnit(U_VOLT)
    .setDescription("Temperature coefficient of Vth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KT1L", static_cast<double>(0.0), &ADMSbsimsoi::Model::KT1L)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of Vth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KT2", static_cast<double>(0.022), &ADMSbsimsoi::Model::KT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Body-coefficient of kt1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K2", static_cast<double>((-0.0186)), &ADMSbsimsoi::Model::K2)
    .setUnit(U_UNKNOWN)
    .setDescription("Bulk effect coefficient 2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K3", static_cast<double>(0.0), &ADMSbsimsoi::Model::K3)
    .setUnit(U_UNKNOWN)
    .setDescription("Narrow width effect coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K3B", static_cast<double>(0.0), &ADMSbsimsoi::Model::K3B)
    .setUnit(U_UNKNOWN)
    .setDescription("Body effect coefficient of k3")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("W0", static_cast<double>(2.5e-6), &ADMSbsimsoi::Model::W0)
    .setUnit(U_METER)
    .setDescription("Narrow width effect parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPEB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPEB)
    .setUnit(U_METER)
    .setDescription("Lateral non-uniform doping effect for body bias")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVT0", static_cast<double>(2.2), &ADMSbsimsoi::Model::DVT0)
    .setUnit(U_UNKNOWN)
    .setDescription("Short channel effect coeff. 0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVT1", static_cast<double>(0.53), &ADMSbsimsoi::Model::DVT1)
    .setUnit(U_UNKNOWN)
    .setDescription("Short channel effect coeff. 1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVT2", static_cast<double>((-0.032)), &ADMSbsimsoi::Model::DVT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Short channel effect coeff. 2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVT0W", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVT0W)
    .setUnit(U_UNKNOWN)
    .setDescription("Narrow Width coeff. 0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVT1W", static_cast<double>(5.3e6), &ADMSbsimsoi::Model::DVT1W)
    .setUnit(U_UNKNOWN)
    .setDescription("Narrow Width effect coeff. 1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVT2W", static_cast<double>((-0.032)), &ADMSbsimsoi::Model::DVT2W)
    .setUnit(U_UNKNOWN)
    .setDescription("Narrow Width effect coeff. 2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DROUT", static_cast<double>(0.56), &ADMSbsimsoi::Model::DROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("DIBL coefficient of output resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DSUB", 0.0, &ADMSbsimsoi::Model::DSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("DIBL coefficient in the subthreshold region")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VTHO", 0.0, &ADMSbsimsoi::Model::VTHO)
    .setUnit(U_VOLT)
    .setDescription("Threshold voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VTH0", 0.0, &ADMSbsimsoi::Model::VTH0)
    .setUnit(U_VOLT)
    .setDescription("Threshold voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VFB", static_cast<double>((-1.0)), &ADMSbsimsoi::Model::VFB)
    .setUnit(U_VOLT)
    .setDescription("Flat Band Voltage /* v4.1 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UA", static_cast<double>(2.25e-9), &ADMSbsimsoi::Model::UA)
    .setUnit(U_UNKNOWN)
    .setDescription("Linear gate dependence of mobility")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UA1", static_cast<double>(4.31e-9), &ADMSbsimsoi::Model::UA1)
    .setUnit(U_MVM1)
    .setDescription("Temperature coefficient of ua")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UB", static_cast<double>(5.87e-19), &ADMSbsimsoi::Model::UB)
    .setUnit(U_UNKNOWN)
    .setDescription("Quadratic gate dependence of mobility")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UB1", static_cast<double>((-7.61e-18)), &ADMSbsimsoi::Model::UB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of ub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UC", 0.0, &ADMSbsimsoi::Model::UC)
    .setUnit(U_UNKNOWN)
    .setDescription("Body-bias dependence of mobility")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UC1", 0.0, &ADMSbsimsoi::Model::UC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of uc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("U0", 0.0, &ADMSbsimsoi::Model::U0)
    .setUnit(U_UNKNOWN)
    .setDescription("Low-field mobility at Tnom")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EU", 0.0, &ADMSbsimsoi::Model::EU)
    .setUnit(U_UNKNOWN)
    .setDescription("Mobility exponent")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UTE", static_cast<double>((-1.5)), &ADMSbsimsoi::Model::UTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of mobility")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UCS", 0.0, &ADMSbsimsoi::Model::UCS)
    .setUnit(U_UNKNOWN)
    .setDescription("Mobility exponent")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UCSTE", static_cast<double>((-4.775e-3)), &ADMSbsimsoi::Model::UCSTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of UCS")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UD", static_cast<double>(0.0), &ADMSbsimsoi::Model::UD)
    .setUnit(U_UNKNOWN)
    .setDescription("Coulomb scattering factor of mobility")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("UD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::UD1)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of ud")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VOFF", static_cast<double>((-0.08)), &ADMSbsimsoi::Model::VOFF)
    .setUnit(U_VOLT)
    .setDescription("Threshold voltage offset")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TNOM", static_cast<double>(27.0), &ADMSbsimsoi::Model::TNOM)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter measurement temperature")
   .setAutoConvertTemperature(false)
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CGSO", static_cast<double>(0.0), &ADMSbsimsoi::Model::CGSO)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate-source overlap capacitance per width")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CGDO", static_cast<double>(0.0), &ADMSbsimsoi::Model::CGDO)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate-drain overlap capacitance per width")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XPART", static_cast<double>(0.0), &ADMSbsimsoi::Model::XPART)
    .setUnit(U_UNKNOWN)
    .setDescription("Channel charge partitioning")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DELTA", static_cast<double>(0.01), &ADMSbsimsoi::Model::DELTA)
    .setUnit(U_UNKNOWN)
    .setDescription("Effective Vds parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RSH", static_cast<double>(0.0), &ADMSbsimsoi::Model::RSH)
    .setUnit(U_UNKNOWN)
    .setDescription("Source-drain sheet resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RDSW", static_cast<double>(100), &ADMSbsimsoi::Model::RDSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Source-drain resistance per width")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RSW", static_cast<double>(50), &ADMSbsimsoi::Model::RSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Source resistance per width /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RDW", static_cast<double>(50), &ADMSbsimsoi::Model::RDW)
    .setUnit(U_UNKNOWN)
    .setDescription("Drain resistance per width /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RSWMIN", static_cast<double>(0.0), &ADMSbsimsoi::Model::RSWMIN)
    .setUnit(U_UNKNOWN)
    .setDescription("Source resistance per width at high Vg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RDWMIN", static_cast<double>(0.0), &ADMSbsimsoi::Model::RDWMIN)
    .setUnit(U_UNKNOWN)
    .setDescription("Drain resistance per width at hight Vg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::PRWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate-bias effect on parasitic resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PRWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Body-effect on parasitic resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PRT)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of parasitic resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ETA0", static_cast<double>(0.08), &ADMSbsimsoi::Model::ETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Subthreshold region DIBL coefficient for I-V")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ETAB", static_cast<double>((-0.07)), &ADMSbsimsoi::Model::ETAB)
    .setUnit(U_UNKNOWN)
    .setDescription("Subthreshold region DIBL coefficient for I-V")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ETA0CV", 0.0, &ADMSbsimsoi::Model::ETA0CV)
    .setUnit(U_UNKNOWN)
    .setDescription("Subthreshold region DIBL coefficient for C-V")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ETABCV", 0.0, &ADMSbsimsoi::Model::ETABCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Subthreshold region DIBL coefficient for C-V")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCLM", static_cast<double>(1.3), &ADMSbsimsoi::Model::PCLM)
    .setUnit(U_UNKNOWN)
    .setDescription("Channel length modulation Coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDIBLC1", static_cast<double>(0.39), &ADMSbsimsoi::Model::PDIBLC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Drain-induced barrier lowering coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDIBLC2", static_cast<double>(0.0086), &ADMSbsimsoi::Model::PDIBLC2)
    .setUnit(U_UNKNOWN)
    .setDescription("Drain-induced barrier lowering coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDIBLCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDIBLCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Body-effect on drain-induced barrier lowering")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVAG", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVAG)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate dependence of output resistance parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TBOX", static_cast<double>(3e-7), &ADMSbsimsoi::Model::TBOX)
    .setUnit(U_METER)
    .setDescription("Back gate oxide thickness in meters")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TSI", static_cast<double>(1e-7), &ADMSbsimsoi::Model::TSI)
    .setUnit(U_METER)
    .setDescription("Silicon-on-insulator thickness in meters")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ETSI", static_cast<double>(1e-7), &ADMSbsimsoi::Model::ETSI)
    .setUnit(U_METER)
    .setDescription("Effective Silicon-on-insulator thickness in meters")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XJ", 0.0, &ADMSbsimsoi::Model::XJ)
    .setUnit(U_METER)
    .setDescription("Junction Depth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::AGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("GIDL second parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BGIDL", static_cast<double>(2.3e9), &ADMSbsimsoi::Model::BGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("GIDL third parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CGIDL", static_cast<double>(0.5), &ADMSbsimsoi::Model::CGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("GIDL vb parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RGIDL", static_cast<double>(1.0), &ADMSbsimsoi::Model::RGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("GIDL vg parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::KGIDL)
    .setUnit(U_VOLT)
    .setDescription("GIDL vb parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::FGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("GIDL vb parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGISL", 0.0, &ADMSbsimsoi::Model::AGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("GISL second parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BGISL", 0.0, &ADMSbsimsoi::Model::BGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("GISL third parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CGISL", 0.0, &ADMSbsimsoi::Model::CGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("GISL vb parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RGISL", 0.0, &ADMSbsimsoi::Model::RGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("GISL vg parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KGISL", 0.0, &ADMSbsimsoi::Model::KGISL)
    .setUnit(U_VOLT)
    .setDescription("GISL vb parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FGISL", 0.0, &ADMSbsimsoi::Model::FGISL)
    .setUnit(U_VOLT)
    .setDescription("GISL vb parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NDIODE", static_cast<double>(1.0), &ADMSbsimsoi::Model::NDIODE)
    .setUnit(U_UNKNOWN)
    .setDescription("Diode non-ideality factor /*v4.0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NDIODED", 0.0, &ADMSbsimsoi::Model::NDIODED)
    .setUnit(U_UNKNOWN)
    .setDescription("Diode non-ideality factor /*v4.0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XBJT", static_cast<double>(1.0), &ADMSbsimsoi::Model::XBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Isbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XDIF", 0.0, &ADMSbsimsoi::Model::XDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Isdif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XREC", static_cast<double>(1.0), &ADMSbsimsoi::Model::XREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Isrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::XTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Istun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XDIFD", 0.0, &ADMSbsimsoi::Model::XDIFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Iddif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XRECD", 0.0, &ADMSbsimsoi::Model::XRECD)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Idrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XTUND", 0.0, &ADMSbsimsoi::Model::XTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Idtun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBSWG", static_cast<double>(0.7), &ADMSbsimsoi::Model::PBSWG)
    .setUnit(U_VOLT)
    .setDescription("Source(gate side) sidewall junction capacitance built in potential /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBSWGD", 0.0, &ADMSbsimsoi::Model::PBSWGD)
    .setUnit(U_VOLT)
    .setDescription("Drain(gate side) sidewall junction capacitance built in potential /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("MJSWG", static_cast<double>(0.5), &ADMSbsimsoi::Model::MJSWG)
    .setUnit(U_VOLT)
    .setDescription("Source (gate side) sidewall junction capacitance grading coefficient /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("MJSWGD", 0.0, &ADMSbsimsoi::Model::MJSWGD)
    .setUnit(U_VOLT)
    .setDescription("Drain (gate side) sidewall junction capacitance grading coefficient /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CJSWG", static_cast<double>(1e-10), &ADMSbsimsoi::Model::CJSWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Source(gate side) sidewall junction capacitance per unit width /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CJSWGD", 0.0, &ADMSbsimsoi::Model::CJSWGD)
    .setUnit(U_UNKNOWN)
    .setDescription("Drain (gate side) sidewall junction capacitance per unit width /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LINT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LINT)
    .setUnit(U_METER)
    .setDescription("Length reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LL)
    .setUnit(U_METER)
    .setDescription("Length reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LLC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length reduction parameter /* v2.2.3 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLN", static_cast<double>(1.0), &ADMSbsimsoi::Model::LLN)
    .setUnit(U_UNKNOWN)
    .setDescription("Length reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LW", static_cast<double>(0.0), &ADMSbsimsoi::Model::LW)
    .setUnit(U_METER)
    .setDescription("Length reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LWC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LWC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length reduction parameter /* v2.2.3 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LWN", static_cast<double>(1.0), &ADMSbsimsoi::Model::LWN)
    .setUnit(U_UNKNOWN)
    .setDescription("Length reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LWL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LWL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LWLC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LWLC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length reduction parameter /* v2.2.3 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WR", static_cast<double>(1.0), &ADMSbsimsoi::Model::WR)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of rds")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WINT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WINT)
    .setUnit(U_METER)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::DWG)
    .setUnit(U_MVM1)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::DWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WL)
    .setUnit(U_METER)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WLC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width reduction parameter /* v2.2.3 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLN", static_cast<double>(1.0), &ADMSbsimsoi::Model::WLN)
    .setUnit(U_UNKNOWN)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WW", static_cast<double>(0.0), &ADMSbsimsoi::Model::WW)
    .setUnit(U_METER)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WWC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WWC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width reduction parameter /* v2.2.3 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WWN", static_cast<double>(1.0), &ADMSbsimsoi::Model::WWN)
    .setUnit(U_UNKNOWN)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WWL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WWL)
    .setUnit(U_METER)
    .setDescription("Width reduction parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WWLC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WWLC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width reduction parameter /* v2.2.3 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("B0", static_cast<double>(0.0), &ADMSbsimsoi::Model::B0)
    .setUnit(U_METER)
    .setDescription("Abulk narrow width parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("B1", static_cast<double>(0.0), &ADMSbsimsoi::Model::B1)
    .setUnit(U_METER)
    .setDescription("Abulk narrow width parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CGSL", static_cast<double>(0.0), &ADMSbsimsoi::Model::CGSL)
    .setUnit(U_UNKNOWN)
    .setDescription("New C-V model parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CGDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::CGDL)
    .setUnit(U_UNKNOWN)
    .setDescription("New C-V model parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CKAPPA", static_cast<double>(0.6), &ADMSbsimsoi::Model::CKAPPA)
    .setUnit(U_UNKNOWN)
    .setDescription("New C-V model parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CF", static_cast<double>(0.0), &ADMSbsimsoi::Model::CF)
    .setUnit(U_UNKNOWN)
    .setDescription("Fringe capacitance parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CLC", static_cast<double>(0.1e-7), &ADMSbsimsoi::Model::CLC)
    .setUnit(U_METER)
    .setDescription("Vdsat parameter for C-V model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CLE", static_cast<double>(0.0), &ADMSbsimsoi::Model::CLE)
    .setUnit(U_UNKNOWN)
    .setDescription("Vdsat parameter for C-V model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DWC", 0.0, &ADMSbsimsoi::Model::DWC)
    .setUnit(U_METER)
    .setDescription("Delta W for C-V model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DLC", 0.0, &ADMSbsimsoi::Model::DLC)
    .setUnit(U_METER)
    .setDescription("Delta L for C-V model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ALPHA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::ALPHA0)
    .setUnit(U_MVM1)
    .setDescription("substrate current model parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NOIA", 0.0, &ADMSbsimsoi::Model::NOIA)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NOIB", 0.0, &ADMSbsimsoi::Model::NOIB)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NOIC", static_cast<double>(8.75e9), &ADMSbsimsoi::Model::NOIC)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FNOIMOD", static_cast<int>(1), &ADMSbsimsoi::Model::FNOIMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise model selector");
  p.addPar("TNOIMOD", static_cast<int>(0), &ADMSbsimsoi::Model::TNOIMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Thermal noise model selector");
  p.addPar("TNOIA", static_cast<double>(1.5), &ADMSbsimsoi::Model::TNOIA)
    .setUnit(U_UNKNOWN)
    .setDescription("Thermal noise parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TNOIB", static_cast<double>(3.5), &ADMSbsimsoi::Model::TNOIB)
    .setUnit(U_UNKNOWN)
    .setDescription("Thermal noise parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RNOIA", static_cast<double>(0.577), &ADMSbsimsoi::Model::RNOIA)
    .setUnit(U_UNKNOWN)
    .setDescription("Thermal noise coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RNOIB", static_cast<double>(0.37), &ADMSbsimsoi::Model::RNOIB)
    .setUnit(U_UNKNOWN)
    .setDescription("Thermal noise coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NTNOI", static_cast<double>(1.0), &ADMSbsimsoi::Model::NTNOI)
    .setUnit(U_UNKNOWN)
    .setDescription("Thermal noise parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SAREF", static_cast<double>(1e-6), &ADMSbsimsoi::Model::SAREF)
    .setUnit(U_METER)
    .setDescription("Reference distance between OD edge to poly of one side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SBREF", static_cast<double>(1e-6), &ADMSbsimsoi::Model::SBREF)
    .setUnit(U_METER)
    .setDescription("Reference distance between OD edge to poly of the other side")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLOD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WLOD)
    .setUnit(U_METER)
    .setDescription("Width parameter for stress effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::KU0)
    .setUnit(U_METER)
    .setDescription("Mobility degradation/enhancement coefficient for LOD")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KVSAT", static_cast<double>(0.0), &ADMSbsimsoi::Model::KVSAT)
    .setUnit(U_METER)
    .setDescription("Saturation velocity degradation/enhancement parameter for LOD")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KVTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::KVTH0)
    .setUnit(U_VM)
    .setDescription("Threshold degradation/enhancement parameter for LOD")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TKU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::TKU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient of KU0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLODKU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LLODKU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length parameter for u0 LOD effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLODKU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WLODKU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width parameter for u0 LOD effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLODVTH", static_cast<double>(0.0), &ADMSbsimsoi::Model::LLODVTH)
    .setUnit(U_UNKNOWN)
    .setDescription("Length parameter for vth LOD effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLODVTH", static_cast<double>(0.0), &ADMSbsimsoi::Model::WLODVTH)
    .setUnit(U_UNKNOWN)
    .setDescription("Width parameter for vth LOD effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ku0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ku0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ku0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKVTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKVTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of kvth0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKVTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKVTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of kvth0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKVTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKVTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of kvth0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("STK2", static_cast<double>(0.0), &ADMSbsimsoi::Model::STK2)
    .setUnit(U_METER)
    .setDescription("K2 shift factor related to stress effect on vth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LODK2", static_cast<double>(1.0), &ADMSbsimsoi::Model::LODK2)
    .setUnit(U_UNKNOWN)
    .setDescription("K2 shift modification factor for stress effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("STETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::STETA0)
    .setUnit(U_METER)
    .setDescription("eta0 shift factor related to stress effect on vth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LODETA0", static_cast<double>(1.0), &ADMSbsimsoi::Model::LODETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("eta0 shift modification factor for stress effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("STETA0CV", 0.0, &ADMSbsimsoi::Model::STETA0CV)
    .setUnit(U_UNKNOWN)
    .setDescription("eta0cv shift factor related to stress effect on vth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LODETA0CV", 0.0, &ADMSbsimsoi::Model::LODETA0CV)
    .setUnit(U_UNKNOWN)
    .setDescription("eta0cv shift modification factor for stress effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("GBMIN", static_cast<double>(1.0e-12), &ADMSbsimsoi::Model::GBMIN)
    .setUnit(U_UNKNOWN)
    .setDescription("Minimum body conductance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BF", static_cast<double>(2.0), &ADMSbsimsoi::Model::BF)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise length dependence exponent")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("W0FLK", static_cast<double>(10.0e-6), &ADMSbsimsoi::Model::W0FLK)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise width dependence")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVTP0", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVTP0)
    .setUnit(U_METER)
    .setDescription("First parameter for Vth shift due to pocket")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVTP0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVTP0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvtp0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVTP0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVTP0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvtp0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVTP0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVTP0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvtp0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVTP1", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVTP1)
    .setUnit(U_UNKNOWN)
    .setDescription("Second parameter for Vth shift due to pocket")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVTP1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVTP1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvtp1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVTP1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVTP1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvtp1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVTP1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVTP1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvtp1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVTP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVTP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Third parameter for Vth shift due to pocket")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVTP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVTP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvtp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVTP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVTP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvtp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVTP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVTP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvtp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVTP3", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVTP3)
    .setUnit(U_UNKNOWN)
    .setDescription("Third parameter for Vth shift due to pocket")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVTP3", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVTP3)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvtp3")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVTP3", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVTP3)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvtp3")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVTP3", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVTP3)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvtp3")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVTP4", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVTP4)
    .setUnit(U_UNKNOWN)
    .setDescription("Forth parameter for Vth shift due to pocket")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVTP4", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVTP4)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvtp4")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVTP4", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVTP4)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvtp4")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVTP4", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVTP4)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvtp4")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("MINV", static_cast<double>(0.0), &ADMSbsimsoi::Model::MINV)
    .setUnit(U_UNKNOWN)
    .setDescription("For moderate inversion in Vgsteff")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LMINV", static_cast<double>(0.0), &ADMSbsimsoi::Model::LMINV)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of minv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WMINV", static_cast<double>(0.0), &ADMSbsimsoi::Model::WMINV)
    .setUnit(U_UNKNOWN)
    .setDescription("width dependence of minv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PMINV", static_cast<double>(0.0), &ADMSbsimsoi::Model::PMINV)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of minv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDITS", static_cast<double>(1e-20), &ADMSbsimsoi::Model::PDITS)
    .setUnit(U_UNKNOWN)
    .setDescription("Coefficient for drain-induced Vth shifts")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDITSL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDITSL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of drain-induced Vth shifts")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDITSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDITSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Vds dependence of drain-induced Vth shifts")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FPROUT", static_cast<double>(0.0), &ADMSbsimsoi::Model::FPROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Rout degradation coefficient for pocket devices")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LFPROUT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LFPROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pdiblcb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPDITS", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPDITS)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pdits")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPDITSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPDITSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pditsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WFPROUT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WFPROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pdiblcb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPDITS", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPDITS)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pdits")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPDITSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPDITSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pditsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PFPROUT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PFPROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pdiblcb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPDITS", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPDITS)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pdits")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPDITSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPDITSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pditsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EM", static_cast<double>(4.1e7), &ADMSbsimsoi::Model::EM)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EF", static_cast<double>(1.0), &ADMSbsimsoi::Model::EF)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise frequency exponent")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AF", static_cast<double>(1.0), &ADMSbsimsoi::Model::AF)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise exponent")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KF", static_cast<double>(0.0), &ADMSbsimsoi::Model::KF)
    .setUnit(U_UNKNOWN)
    .setDescription("Flicker noise coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NOIF", static_cast<double>(1.0), &ADMSbsimsoi::Model::NOIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Floating body excess noise ideality factor")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K1W1", static_cast<double>(0.0), &ADMSbsimsoi::Model::K1W1)
    .setUnit(U_METER)
    .setDescription("First Body effect width dependent parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K1W2", static_cast<double>(0.0), &ADMSbsimsoi::Model::K1W2)
    .setUnit(U_METER)
    .setDescription("Second Body effect width dependent parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KETAS", static_cast<double>(0.0), &ADMSbsimsoi::Model::KETAS)
    .setUnit(U_VOLT)
    .setDescription("Surface potential adjustment for bulk charge effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DWBC", static_cast<double>(0.0), &ADMSbsimsoi::Model::DWBC)
    .setUnit(U_METER)
    .setDescription("Width offset for body contact isolation edge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::BETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("First Vds dependent parameter of impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BETA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::BETA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Second Vds dependent parameter of impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BETA2", static_cast<double>(0.1), &ADMSbsimsoi::Model::BETA2)
    .setUnit(U_VOLT)
    .setDescription("Third Vds dependent parameter of impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VDSATII0", static_cast<double>(0.9), &ADMSbsimsoi::Model::VDSATII0)
    .setUnit(U_UNKNOWN)
    .setDescription("Nominal drain saturation voltage at threshold for impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TII", static_cast<double>(0.0), &ADMSbsimsoi::Model::TII)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature dependent parameter for impact ionization")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LII)
    .setUnit(U_UNKNOWN)
    .setDescription("Channel length dependent parameter at threshold for impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SII0", static_cast<double>(0.5), &ADMSbsimsoi::Model::SII0)
    .setUnit(U_UNKNOWN)
    .setDescription("First Vgs dependent parameter for impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SII1", static_cast<double>(0.1), &ADMSbsimsoi::Model::SII1)
    .setUnit(U_UNKNOWN)
    .setDescription("Second Vgs dependent parameter for impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SII2", static_cast<double>(0.0), &ADMSbsimsoi::Model::SII2)
    .setUnit(U_UNKNOWN)
    .setDescription("Third Vgs dependent parameter for impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("SIID", static_cast<double>(0.0), &ADMSbsimsoi::Model::SIID)
    .setUnit(U_UNKNOWN)
    .setDescription("Vds dependent parameter of drain saturation voltage for impact ionization current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::FBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Fraction of bipolar current affecting the impact ionization")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::EBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Impact ionization parameter for BJT part")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::CBJTII)
    .setUnit(U_METER)
    .setDescription("Length scaling parameter for II BJT part")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VBCI", static_cast<double>(0.0), &ADMSbsimsoi::Model::VBCI)
    .setUnit(U_VOLT)
    .setDescription("Internal B-C built-in potential")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ABJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::ABJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Exponent factor for avalanche current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("MBJTII", static_cast<double>(0.4), &ADMSbsimsoi::Model::MBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Internal B-C grading coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TVBCI", static_cast<double>(0.0), &ADMSbsimsoi::Model::TVBCI)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for VBCI")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ESATII", static_cast<double>(1e7), &ADMSbsimsoi::Model::ESATII)
    .setUnit(U_UNKNOWN)
    .setDescription("Saturation electric field for impact ionization")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NTUN", static_cast<double>(10.0), &ADMSbsimsoi::Model::NTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Reverse tunneling non-ideality factor")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NTUND", 0.0, &ADMSbsimsoi::Model::NTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Reverse tunneling non-ideality factor")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRECF0", static_cast<double>(2.0), &ADMSbsimsoi::Model::NRECF0)
    .setUnit(U_UNKNOWN)
    .setDescription("Recombination non-ideality factor at forward bias")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRECF0D", 0.0, &ADMSbsimsoi::Model::NRECF0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Recombination non-ideality factor at forward bias")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRECR0", static_cast<double>(10.0), &ADMSbsimsoi::Model::NRECR0)
    .setUnit(U_UNKNOWN)
    .setDescription("Recombination non-ideality factor at reversed bias")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NRECR0D", 0.0, &ADMSbsimsoi::Model::NRECR0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Recombination non-ideality factor at reversed bias")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ISBJT", static_cast<double>(1e-6), &ADMSbsimsoi::Model::ISBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("BJT injection saturation current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("IDBJT", 0.0, &ADMSbsimsoi::Model::IDBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("BJT injection saturation current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ISDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::ISDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Body to source/drain injection saturation current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("IDDIF", 0.0, &ADMSbsimsoi::Model::IDDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Body to source/drain injection saturation current /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ISREC", static_cast<double>(1e-5), &ADMSbsimsoi::Model::ISREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Recombination in depletion saturation current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("IDREC", 0.0, &ADMSbsimsoi::Model::IDREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Recombination in depletion saturation current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ISTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::ISTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Reverse tunneling saturation current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("IDTUN", 0.0, &ADMSbsimsoi::Model::IDTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Reverse tunneling saturation current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LN", static_cast<double>(2e-6), &ADMSbsimsoi::Model::LN)
    .setUnit(U_METER)
    .setDescription("Electron/hole diffusion length")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VREC0", static_cast<double>(0.0), &ADMSbsimsoi::Model::VREC0)
    .setUnit(U_VOLT)
    .setDescription("Voltage dependent parameter for recombination current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VREC0D", 0.0, &ADMSbsimsoi::Model::VREC0D)
    .setUnit(U_VOLT)
    .setDescription("Voltage dependent parameter for recombination current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VTUN0", static_cast<double>(0.0), &ADMSbsimsoi::Model::VTUN0)
    .setUnit(U_VOLT)
    .setDescription("Voltage dependent parameter for tunneling current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VTUN0D", 0.0, &ADMSbsimsoi::Model::VTUN0D)
    .setUnit(U_VOLT)
    .setDescription("Voltage dependent parameter for tunneling current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NBJT", static_cast<double>(1.0), &ADMSbsimsoi::Model::NBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Power coefficient of channel length dependency for bipolar current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBJT0", static_cast<double>(0.20e-6), &ADMSbsimsoi::Model::LBJT0)
    .setUnit(U_METER)
    .setDescription("Reference channel length for bipolar current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDIF0", static_cast<double>(1.0), &ADMSbsimsoi::Model::LDIF0)
    .setUnit(U_UNKNOWN)
    .setDescription("Channel-length dependency coefficient of diffusion cap")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VABJT", static_cast<double>(10.0), &ADMSbsimsoi::Model::VABJT)
    .setUnit(U_VOLT)
    .setDescription("Early voltage for bipolar current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AELY", static_cast<double>(0.0), &ADMSbsimsoi::Model::AELY)
    .setUnit(U_UNKNOWN)
    .setDescription("Channel length dependency of early voltage for bipolar current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AHLI", static_cast<double>(0.0), &ADMSbsimsoi::Model::AHLI)
    .setUnit(U_UNKNOWN)
    .setDescription("High level injection parameter for bipolar current /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AHLID", 0.0, &ADMSbsimsoi::Model::AHLID)
    .setUnit(U_UNKNOWN)
    .setDescription("High level injection parameter for bipolar current /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RBODY", static_cast<double>(0.0), &ADMSbsimsoi::Model::RBODY)
    .setUnit(U_UNKNOWN)
    .setDescription("Intrinsic body contact sheet resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RBSH", static_cast<double>(0.0), &ADMSbsimsoi::Model::RBSH)
    .setUnit(U_UNKNOWN)
    .setDescription("Extrinsic body contact sheet resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CGEO", static_cast<double>(0.0), &ADMSbsimsoi::Model::CGEO)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate substrate overlap capacitance per unit channel length")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TT", static_cast<double>(1e-12), &ADMSbsimsoi::Model::TT)
    .setUnit(U_SECOND)
    .setDescription("Diffusion capacitance transit time coefficient")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NDIF", static_cast<double>((-1.0)), &ADMSbsimsoi::Model::NDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Power coefficient of channel length dependency for diffusion capacitance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VSDFB", static_cast<double>(0.0), &ADMSbsimsoi::Model::VSDFB)
    .setUnit(U_VOLT)
    .setDescription("Source/drain bottom diffusion capacitance flatband voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VSDTH", static_cast<double>(0.0), &ADMSbsimsoi::Model::VSDTH)
    .setUnit(U_VOLT)
    .setDescription("Source/drain bottom diffusion capacitance threshold voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CSDMIN", static_cast<double>(0.0), &ADMSbsimsoi::Model::CSDMIN)
    .setUnit(U_VOLT)
    .setDescription("Source/drain bottom diffusion minimum capacitance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ASD", static_cast<double>(0.3), &ADMSbsimsoi::Model::ASD)
    .setUnit(U_UNKNOWN)
    .setDescription("Source/drain bottom diffusion smoothing parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CSDESW", static_cast<double>(0.0), &ADMSbsimsoi::Model::CSDESW)
    .setUnit(U_UNKNOWN)
    .setDescription("Source/drain sidewall fringing capacitance per unit length")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NTRECF", static_cast<double>(0.0), &ADMSbsimsoi::Model::NTRECF)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Nrecf")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NTRECR", static_cast<double>(0.0), &ADMSbsimsoi::Model::NTRECR)
    .setUnit(U_UNKNOWN)
    .setDescription("Temperature coefficient for Nrecr")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DLCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::DLCB)
    .setUnit(U_METER)
    .setDescription("Length offset fitting parameter for body charge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("FBODY", static_cast<double>(1.0), &ADMSbsimsoi::Model::FBODY)
    .setUnit(U_UNKNOWN)
    .setDescription("Scaling factor for body charge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TCJSWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::TCJSWG)
    .setUnit(U_DEGKM1)
    .setDescription("Temperature coefficient of Cjswgs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TPBSWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::TPBSWG)
    .setUnit(U_VKM1)
    .setDescription("Temperature coefficient of Pbswgs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TCJSWGD", 0.0, &ADMSbsimsoi::Model::TCJSWGD)
    .setUnit(U_DEGKM1)
    .setDescription("Temperature coefficient of Cjswgd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TPBSWGD", 0.0, &ADMSbsimsoi::Model::TPBSWGD)
    .setUnit(U_VKM1)
    .setDescription("Temperature coefficient of Pbswgd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ACDE", static_cast<double>(1.0), &ADMSbsimsoi::Model::ACDE)
    .setUnit(U_MVM1)
    .setDescription("Exponential coefficient for charge thickness in capMod=3 for accumulation and depletion regions")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("MOIN", static_cast<double>(15.0), &ADMSbsimsoi::Model::MOIN)
    .setUnit(U_UNKNOWN)
    .setDescription("Coefficient for the gate-bias dependent surface potential")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NOFF", static_cast<double>(1.0), &ADMSbsimsoi::Model::NOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("C-V turn-on/off parameter /* v3.2 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NOFF2", 0.0, &ADMSbsimsoi::Model::NOFF2)
    .setUnit(U_UNKNOWN)
    .setDescription("C-V turn-on/off parameter /* v4.6 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DELVT", static_cast<double>(0.0), &ADMSbsimsoi::Model::DELVT)
    .setUnit(U_VOLT)
    .setDescription("Threshold voltage adjust for CV")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("KB1", static_cast<double>(1.0), &ADMSbsimsoi::Model::KB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Scaling factor for backgate charge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DLBG", static_cast<double>(0.0), &ADMSbsimsoi::Model::DLBG)
    .setUnit(U_METER)
    .setDescription("Length offset fitting parameter for backgate charge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CFRCOEFF", static_cast<double>(1.0), &ADMSbsimsoi::Model::CFRCOEFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Fringe Cap parameter /* v4.4 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("IGBMOD", static_cast<int>(0), &ADMSbsimsoi::Model::IGBMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("gate-body tunneling current model selector /* v3.0 ");
  p.addPar("IGMOD", 0, &ADMSbsimsoi::Model::IGMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("gate-body tunneling current model selector /* v3.1.1 ");
  p.addPar("IGCMOD", static_cast<int>(0), &ADMSbsimsoi::Model::IGCMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("gate-channel tunneling current model selector /* v3.0 ");
  p.addPar("TOXQM", 0.0, &ADMSbsimsoi::Model::TOXQM)
    .setUnit(U_METER)
    .setDescription("effective oxide thickness considering quantum effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WTH0)
    .setUnit(U_METER)
    .setDescription("Minimum width for thermal resistance calculation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RHALO", static_cast<double>(1e15), &ADMSbsimsoi::Model::RHALO)
    .setUnit(U_UNKNOWN)
    .setDescription("body halo sheet resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NTOX", static_cast<double>(1), &ADMSbsimsoi::Model::NTOX)
    .setUnit(U_UNKNOWN)
    .setDescription("power term of gate current")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("TOXREF", static_cast<double>(2.5e-9), &ADMSbsimsoi::Model::TOXREF)
    .setUnit(U_METER)
    .setDescription("target oxide thickness")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EBG", static_cast<double>(1.2), &ADMSbsimsoi::Model::EBG)
    .setUnit(U_UNKNOWN)
    .setDescription("effective bandgap in gate current calculation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VEVB", static_cast<double>(0.075), &ADMSbsimsoi::Model::VEVB)
    .setUnit(U_UNKNOWN)
    .setDescription("Vaux parameter for valence-band electron tunneling")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ALPHAGB1", static_cast<double>(0.35), &ADMSbsimsoi::Model::ALPHAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("First Vox dependent parameter for gate current in inversion")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BETAGB1", static_cast<double>(0.03), &ADMSbsimsoi::Model::BETAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Second Vox dependent parameter for gate current in inversion")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VGB1", static_cast<double>(300), &ADMSbsimsoi::Model::VGB1)
    .setUnit(U_VOLT)
    .setDescription("Third Vox dependent parameter for gate current in inversion")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VECB", static_cast<double>(0.026), &ADMSbsimsoi::Model::VECB)
    .setUnit(U_UNKNOWN)
    .setDescription("Vaux parameter for conduction-band electron tunneling")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("ALPHAGB2", static_cast<double>(0.43), &ADMSbsimsoi::Model::ALPHAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("First Vox dependent parameter for gate current in accumulation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BETAGB2", static_cast<double>(0.05), &ADMSbsimsoi::Model::BETAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("Second Vox dependent parameter for gate current in accumulation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VGB2", static_cast<double>(17), &ADMSbsimsoi::Model::VGB2)
    .setUnit(U_VOLT)
    .setDescription("Third Vox dependent parameter for gate current in accumulation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AIGBCP2", static_cast<double>(0.043), &ADMSbsimsoi::Model::AIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("First Vgp dependent parameter for gate current in accumulation in AGBCP2 region")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BIGBCP2", static_cast<double>(0.0054), &ADMSbsimsoi::Model::BIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Second Vgp dependent parameter for gate current in accumulation in AGBCP2 region")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CIGBCP2", static_cast<double>(0.0075), &ADMSbsimsoi::Model::CIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Third Vgp dependent parameter for gate current in accumulation in AGBCP2 region")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VOXH", static_cast<double>(5.0), &ADMSbsimsoi::Model::VOXH)
    .setUnit(U_VOLT)
    .setDescription("the limit of Vox in gate current calculation")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DELTAVOX", static_cast<double>(0.005), &ADMSbsimsoi::Model::DELTAVOX)
    .setUnit(U_UNKNOWN)
    .setDescription("the smoothing parameter in the Vox smoothing function")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AIGC", 0.0, &ADMSbsimsoi::Model::AIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BIGC", 0.0, &ADMSbsimsoi::Model::BIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CIGC", 0.0, &ADMSbsimsoi::Model::CIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AIGSD", 0.0, &ADMSbsimsoi::Model::AIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igs,d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BIGSD", 0.0, &ADMSbsimsoi::Model::BIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igs,d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CIGSD", 0.0, &ADMSbsimsoi::Model::CIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igs,d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NIGC", static_cast<double>(1.0), &ADMSbsimsoi::Model::NIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igc slope")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PIGCD", static_cast<double>(1.0), &ADMSbsimsoi::Model::PIGCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Parameter for Igc partition")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("POXEDGE", static_cast<double>(1.0), &ADMSbsimsoi::Model::POXEDGE)
    .setUnit(U_UNKNOWN)
    .setDescription("Factor for the gate edge Tox")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DLCIG", 0.0, &ADMSbsimsoi::Model::DLCIG)
    .setUnit(U_METER)
    .setDescription("Delta L for Ig model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VBS0PD", static_cast<double>(0.0), &ADMSbsimsoi::Model::VBS0PD)
    .setUnit(U_VOLT)
    .setDescription("Upper bound of built-in potential lowering for PD operation /* v3.2 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VBS0FD", static_cast<double>(0.5), &ADMSbsimsoi::Model::VBS0FD)
    .setUnit(U_VOLT)
    .setDescription("Lower bound of built-in potential lowering for FD operation /* v3.2 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VBSA", static_cast<double>(0.0), &ADMSbsimsoi::Model::VBSA)
    .setUnit(U_VOLT)
    .setDescription("vbsa offset voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NOFFFD", static_cast<double>(1.0), &ADMSbsimsoi::Model::NOFFFD)
    .setUnit(U_UNKNOWN)
    .setDescription("smoothing parameter in FD module")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VOFFFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::VOFFFD)
    .setUnit(U_VOLT)
    .setDescription("smoothing parameter in FD module")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K1B", static_cast<double>(1.0), &ADMSbsimsoi::Model::K1B)
    .setUnit(U_UNKNOWN)
    .setDescription("first backgate body effect parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("K2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::K2B)
    .setUnit(U_UNKNOWN)
    .setDescription("second backgate body effect parameter for short channel effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DK2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::DK2B)
    .setUnit(U_UNKNOWN)
    .setDescription("third backgate body effect parameter for short channel effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVBD0", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVBD0)
    .setUnit(U_UNKNOWN)
    .setDescription("first short-channel effect parameter in FD module")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("DVBD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::DVBD1)
    .setUnit(U_UNKNOWN)
    .setDescription("second short-channel effect parameter in FD module")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("MOINFD", static_cast<double>(1e3), &ADMSbsimsoi::Model::MOINFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Coefficient for the gate-bias dependent surface potential in FD")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XRCRG1", static_cast<double>(12.0), &ADMSbsimsoi::Model::XRCRG1)
    .setUnit(U_UNKNOWN)
    .setDescription("First fitting parameter the bias-dependent Rg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XRCRG2", static_cast<double>(1.0), &ADMSbsimsoi::Model::XRCRG2)
    .setUnit(U_UNKNOWN)
    .setDescription("Second fitting parameter the bias-dependent Rg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RSHG", static_cast<double>(0.1), &ADMSbsimsoi::Model::RSHG)
    .setUnit(U_UNKNOWN)
    .setDescription("Gate sheet resistance")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NGCON", static_cast<double>(1.0), &ADMSbsimsoi::Model::NGCON)
    .setUnit(U_UNKNOWN)
    .setDescription("Number of gate contacts")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XGW", static_cast<double>(0.0), &ADMSbsimsoi::Model::XGW)
    .setUnit(U_METER)
    .setDescription("Distance from gate contact center to device edge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("XGL", static_cast<double>(0.0), &ADMSbsimsoi::Model::XGL)
    .setUnit(U_METER)
    .setDescription("Variation in Ldrawn")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("RDSMOD", static_cast<int>(0), &ADMSbsimsoi::Model::RDSMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Bias-dependent S/D resistance model selector /* v4.0 ");
  p.addPar("FDMOD", static_cast<int>(0), &ADMSbsimsoi::Model::FDMOD)
    .setUnit(U_UNKNOWN)
    .setDescription("Improved dVbi model selector");
  p.addPar("VSCE", static_cast<double>(0.0), &ADMSbsimsoi::Model::VSCE)
    .setUnit(U_UNKNOWN)
    .setDescription("SCE parameter for improved dVbi model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("CDSBS", static_cast<double>(0.0), &ADMSbsimsoi::Model::CDSBS)
    .setUnit(U_UNKNOWN)
    .setDescription("coupling from Vd to Vbs for improved dVbi model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("MINVCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::MINVCV)
    .setUnit(U_UNKNOWN)
    .setDescription("For moderate inversion in VgsteffCV")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LMINVCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::LMINVCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of minvcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WMINVCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::WMINVCV)
    .setUnit(U_UNKNOWN)
    .setDescription("width dependence of minvcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PMINVCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::PMINVCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of minvcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VOFFCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::VOFFCV)
    .setUnit(U_VOLT)
    .setDescription("CV Threshold voltage offset // NOT -0.08 for backwards-compatibility")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVOFFCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVOFFCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of voffcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVOFFCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVOFFCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of voffcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVOFFCV", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVOFFCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of voffcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXJ", static_cast<double>(0.0), &ADMSbsimsoi::Model::LXJ)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xj")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LALPHAGB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LALPHAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of alphagb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBETAGB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBETAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of betagb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LALPHAGB2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LALPHAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of alphagb2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBETAGB2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBETAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of betagb2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of aigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of bigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCGSL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCGSL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cgsl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCGDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCGDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cgdl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCKAPPA", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCKAPPA)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ckappa")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ndif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUTE", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ute")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKT1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKT1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of kt1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKT1L", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKT1L)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of kt1l")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKT2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of kt2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ua1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ub1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUC1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of uc1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of at")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPRT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPRT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of prt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNTRECF", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNTRECF)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ntrecf")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNTRECR", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNTRECR)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ntrecr")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LXBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::LXDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xdif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXREC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LXREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::LXTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xtun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXDIFD", 0.0, &ADMSbsimsoi::Model::LXDIFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xdifd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXRECD", 0.0, &ADMSbsimsoi::Model::LXRECD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xrecd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXTUND", 0.0, &ADMSbsimsoi::Model::LXTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xtund")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of aigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of bigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of aigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of bigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPIGCD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPIGCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence for pigcd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPOXEDGE", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPOXEDGE)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence for poxedge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNCH", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNCH)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nch")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNSUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nsub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNGATE", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNGATE)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ngate")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vto")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVFB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVFB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vfb /* v4.1 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK1W1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK1W1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k1w1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK1W2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK1W2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k1w2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK3", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK3)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k3")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK3B", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK3B)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k3b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of kb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LW0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LW0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of w0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLPEB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LLPEB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of lpeb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVT0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVT0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvt0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVT1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVT1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvt1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVT2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvt2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVT0W", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVT0W)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvt0w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVT1W", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVT1W)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvt1w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVT2W", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVT2W)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvt2w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of u0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LEU", static_cast<double>(0.0), &ADMSbsimsoi::Model::LEU)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of eu")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUA", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUA)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ua")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of uc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ud")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUD1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ud1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUCSTE", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUCSTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ucste")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LUCS", static_cast<double>(0.0), &ADMSbsimsoi::Model::LUCS)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of lucs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVSAT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVSAT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vsat")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of a0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAGS", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAGS)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ags")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LB0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LB0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of b0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of b1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKETA", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKETA)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of keta")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKETAS", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKETAS)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ketas")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of a1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LA2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LA2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of a2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LRDSW", static_cast<double>(0.0), &ADMSbsimsoi::Model::LRDSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of rdsw")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LRSW", static_cast<double>(0.0), &ADMSbsimsoi::Model::LRSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of rsw /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LRDW", static_cast<double>(0.0), &ADMSbsimsoi::Model::LRDW)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of rdw /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPRWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPRWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of prwb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPRWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPRWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of prwg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LWR", static_cast<double>(0.0), &ADMSbsimsoi::Model::LWR)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of wr")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNFACTOR", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNFACTOR)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nfactor")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dwg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dwb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVOFF", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of voff")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of eta0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LETAB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LETAB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of etab")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LETA0CV", 0.0, &ADMSbsimsoi::Model::LETA0CV)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of eta0cv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LETABCV", 0.0, &ADMSbsimsoi::Model::LETABCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of etabcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDSUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dsub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCIT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCIT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cit")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCDSC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCDSC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cdsc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCDSCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCDSCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cdscb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCDSCD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCDSCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cdscd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPCLM", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPCLM)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pclm")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPDIBLC1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPDIBLC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pdiblc1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPDIBLC2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPDIBLC2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pdiblc2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPDIBLCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPDIBLCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pdiblcb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDROUT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of drout")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPVAG", static_cast<double>(0.0), &ADMSbsimsoi::Model::LPVAG)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of pvag")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDELTA", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDELTA)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of delta")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LALPHA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LALPHA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of alpha0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LFBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LFBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of fbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LABJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LABJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of abjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LEBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LEBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ebjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LMBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LMBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of mbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVBCI", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVBCI)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vbci")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of beta0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBETA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBETA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of beta1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBETA2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBETA2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of beta2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVDSATII0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVDSATII0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vdsatii0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LLII)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of lii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LESATII", static_cast<double>(0.0), &ADMSbsimsoi::Model::LESATII)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of esatii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LSII0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LSII0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of sii0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LSII1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LSII1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of sii1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LSII2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LSII2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of sii2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LSIID", static_cast<double>(0.0), &ADMSbsimsoi::Model::LSIID)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of siid")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of agidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LBGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of bgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LRGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LRGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of rgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LKGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of kgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LFGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LFGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of fgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAGISL", 0.0, &ADMSbsimsoi::Model::LAGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of agisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LBGISL", 0.0, &ADMSbsimsoi::Model::LBGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of bgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCGISL", 0.0, &ADMSbsimsoi::Model::LCGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LRGISL", 0.0, &ADMSbsimsoi::Model::LRGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of rgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LKGISL", 0.0, &ADMSbsimsoi::Model::LKGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of kgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LFGISL", 0.0, &ADMSbsimsoi::Model::LFGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of fgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ntun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNTUND", 0.0, &ADMSbsimsoi::Model::LNTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ntund")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNDIODE", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNDIODE)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ndiode")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNDIODED", 0.0, &ADMSbsimsoi::Model::LNDIODED)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ndioded")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNRECF0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNRECF0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nrecf0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNRECF0D", 0.0, &ADMSbsimsoi::Model::LNRECF0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nrecf0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNRECR0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNRECR0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nrecr0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNRECR0D", 0.0, &ADMSbsimsoi::Model::LNRECR0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nrecr0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LISBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LISBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of isbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LIDBJT", 0.0, &ADMSbsimsoi::Model::LIDBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of idbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LISDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::LISDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of isdif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LIDDIF", 0.0, &ADMSbsimsoi::Model::LIDDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of iddif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LISREC", static_cast<double>(0.0), &ADMSbsimsoi::Model::LISREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of isrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LIDREC", 0.0, &ADMSbsimsoi::Model::LIDREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of idrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LISTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::LISTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of istun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LIDTUN", 0.0, &ADMSbsimsoi::Model::LIDTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of idtun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVREC0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVREC0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vrec0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVREC0D", 0.0, &ADMSbsimsoi::Model::LVREC0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vrec0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVTUN0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVTUN0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vtun0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVTUN0D", 0.0, &ADMSbsimsoi::Model::LVTUN0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vtun0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLBJT0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LLBJT0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of lbjt0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVABJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVABJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vabjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAELY", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAELY)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of aely")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAHLI", static_cast<double>(0.0), &ADMSbsimsoi::Model::LAHLI)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ahli /*v4.0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LAHLID", 0.0, &ADMSbsimsoi::Model::LAHLID)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ahlid /*v4.0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVSDFB", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVSDFB)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vsdfb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVSDTH", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVSDTH)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vsdth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDELVT", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDELVT)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of delvt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LACDE", static_cast<double>(0.0), &ADMSbsimsoi::Model::LACDE)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of acde")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LMOIN", static_cast<double>(0.0), &ADMSbsimsoi::Model::LMOIN)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of moin")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNOFF", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of noff /* v3.2 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNOFF2", 0.0, &ADMSbsimsoi::Model::LNOFF2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of noff2 /* v4.6 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXRCRG1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LXRCRG1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xrcrg1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LXRCRG2", static_cast<double>(0.0), &ADMSbsimsoi::Model::LXRCRG2)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of xrcrg2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVBSA", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVBSA)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vbsa")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVSCE", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVSCE)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vsce")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LCDSBS", static_cast<double>(0.0), &ADMSbsimsoi::Model::LCDSBS)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of cdsbs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNOFFFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNOFFFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nofffd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVOFFFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVOFFFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vofffd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK1B", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK1B)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k1b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LK2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::LK2B)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of k2b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDK2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDK2B)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dk2b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVBD0", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVBD0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvbd0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LDVBD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::LDVBD1)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of dvbd1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LMOINFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LMOINFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of moinfd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVBS0PD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVBS0PD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vbs0pd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LVBS0FD", static_cast<double>(0.0), &ADMSbsimsoi::Model::LVBS0FD)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of vbs0fd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXJ", static_cast<double>(0.0), &ADMSbsimsoi::Model::WXJ)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xj")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WALPHAGB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WALPHAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of alphagb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBETAGB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBETAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of betagb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WALPHAGB2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WALPHAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of alphagb2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBETAGB2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBETAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of betagb2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of aigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of bigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCGSL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCGSL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cgsl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCGDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCGDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cgdl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCKAPPA", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCKAPPA)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ckappa")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ndif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUTE", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ute")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKT1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKT1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of kt1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKT1L", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKT1L)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of kt1l")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKT2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of kt2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ua1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ub1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUC1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of uc1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of at")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPRT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPRT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of prt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNTRECF", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNTRECF)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ntrecf")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNTRECR", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNTRECR)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ntrecr")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WXBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::WXDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xdif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXREC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WXREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::WXTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xtun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXDIFD", 0.0, &ADMSbsimsoi::Model::WXDIFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xdifd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXRECD", 0.0, &ADMSbsimsoi::Model::WXRECD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xrecd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXTUND", 0.0, &ADMSbsimsoi::Model::WXTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xtund")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of aigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of bigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of aigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of bigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPIGCD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPIGCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence for pigcd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPOXEDGE", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPOXEDGE)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence for poxedge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNCH", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNCH)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nch")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNSUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nsub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNGATE", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNGATE)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ngate")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vto")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVFB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVFB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vfb /* v4.1 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK1W1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK1W1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k1w1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK1W2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK1W2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k1w2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK3", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK3)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k3")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK3B", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK3B)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k3b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of kb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WW0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WW0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of w0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLPEB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WLPEB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of lpeb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVT0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVT0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvt0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVT1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVT1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvt1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVT2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvt2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVT0W", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVT0W)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvt0w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVT1W", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVT1W)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvt1w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVT2W", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVT2W)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvt2w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of u0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WEU", static_cast<double>(0.0), &ADMSbsimsoi::Model::WEU)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of eu")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUA", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUA)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ua")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of uc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ud")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUD1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ud1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUCSTE", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUCSTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ucste")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WUCS", static_cast<double>(0.0), &ADMSbsimsoi::Model::WUCS)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ucs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVSAT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVSAT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vsat")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of a0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAGS", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAGS)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ags")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WB0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WB0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of b0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of b1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKETA", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKETA)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of keta")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKETAS", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKETAS)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ketas")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of a1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WA2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WA2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of a2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WRDSW", static_cast<double>(0.0), &ADMSbsimsoi::Model::WRDSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of rdsw")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WRSW", static_cast<double>(0.0), &ADMSbsimsoi::Model::WRSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of rsw /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WRDW", static_cast<double>(0.0), &ADMSbsimsoi::Model::WRDW)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of rdw /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPRWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPRWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of prwb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPRWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPRWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of prwg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WWR", static_cast<double>(0.0), &ADMSbsimsoi::Model::WWR)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of wr")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNFACTOR", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNFACTOR)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nfactor")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dwg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dwb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVOFF", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of voff")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of eta0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WETAB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WETAB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of etab")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WETA0CV", 0.0, &ADMSbsimsoi::Model::WETA0CV)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of eta0cv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WETABCV", 0.0, &ADMSbsimsoi::Model::WETABCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of etabcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDSUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dsub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCIT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCIT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cit")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCDSC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCDSC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cdsc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCDSCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCDSCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cdscb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCDSCD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCDSCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cdscd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPCLM", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPCLM)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pclm")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPDIBLC1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPDIBLC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pdiblc1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPDIBLC2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPDIBLC2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pdiblc2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPDIBLCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPDIBLCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pdiblcb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDROUT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of drout")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WPVAG", static_cast<double>(0.0), &ADMSbsimsoi::Model::WPVAG)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of pvag")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDELTA", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDELTA)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of delta")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WALPHA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WALPHA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of alpha0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WFBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::WFBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of fbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WABJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::WABJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of abjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WEBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::WEBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ebjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WMBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::WMBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of mbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVBCI", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVBCI)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vbci")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of beta0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBETA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBETA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of beta1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBETA2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBETA2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of beta2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVDSATII0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVDSATII0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vdsatii0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLII", static_cast<double>(0.0), &ADMSbsimsoi::Model::WLII)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of lii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WESATII", static_cast<double>(0.0), &ADMSbsimsoi::Model::WESATII)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of esatii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WSII0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WSII0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of sii0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WSII1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WSII1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of sii1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WSII2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WSII2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of sii2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WSIID", static_cast<double>(0.0), &ADMSbsimsoi::Model::WSIID)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of siid")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of agidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WBGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of bgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WRGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WRGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of rgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WKGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of kgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WFGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WFGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of fgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAGISL", 0.0, &ADMSbsimsoi::Model::WAGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of agisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WBGISL", 0.0, &ADMSbsimsoi::Model::WBGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of bgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCGISL", 0.0, &ADMSbsimsoi::Model::WCGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WRGISL", 0.0, &ADMSbsimsoi::Model::WRGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of rgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WKGISL", 0.0, &ADMSbsimsoi::Model::WKGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of kgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WFGISL", 0.0, &ADMSbsimsoi::Model::WFGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of fgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ntun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNTUND", 0.0, &ADMSbsimsoi::Model::WNTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ntund")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNDIODE", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNDIODE)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ndiode")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNDIODED", 0.0, &ADMSbsimsoi::Model::WNDIODED)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ndioded")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNRECF0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNRECF0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nrecf0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNRECF0D", 0.0, &ADMSbsimsoi::Model::WNRECF0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nrecf0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNRECR0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNRECR0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nrecr0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNRECR0D", 0.0, &ADMSbsimsoi::Model::WNRECR0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nrecr0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WISBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WISBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of isbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WIDBJT", 0.0, &ADMSbsimsoi::Model::WIDBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of idbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WISDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::WISDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of isdif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WIDDIF", 0.0, &ADMSbsimsoi::Model::WIDDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of iddif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WISREC", static_cast<double>(0.0), &ADMSbsimsoi::Model::WISREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of isrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WIDREC", 0.0, &ADMSbsimsoi::Model::WIDREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of idrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WISTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::WISTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of istun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WIDTUN", 0.0, &ADMSbsimsoi::Model::WIDTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of idtun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVREC0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVREC0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vrec0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVREC0D", 0.0, &ADMSbsimsoi::Model::WVREC0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vrec0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVTUN0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVTUN0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vtun0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVTUN0D", 0.0, &ADMSbsimsoi::Model::WVTUN0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vtun0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLBJT0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WLBJT0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of lbjt0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVABJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVABJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vabjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAELY", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAELY)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of aely")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAHLI", static_cast<double>(0.0), &ADMSbsimsoi::Model::WAHLI)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ahli /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WAHLID", 0.0, &ADMSbsimsoi::Model::WAHLID)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ahlid /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVSDFB", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVSDFB)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vsdfb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVSDTH", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVSDTH)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vsdth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDELVT", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDELVT)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of delvt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WACDE", static_cast<double>(0.0), &ADMSbsimsoi::Model::WACDE)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of acde")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WMOIN", static_cast<double>(0.0), &ADMSbsimsoi::Model::WMOIN)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of moin")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNOFF", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of noff /* v3.2 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNOFF2", 0.0, &ADMSbsimsoi::Model::WNOFF2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of noff2 /* v4.6 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXRCRG1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WXRCRG1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xrcrg1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WXRCRG2", static_cast<double>(0.0), &ADMSbsimsoi::Model::WXRCRG2)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of xrcrg2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVBSA", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVBSA)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vbsa")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVSCE", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVSCE)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vsce")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WCDSBS", static_cast<double>(0.0), &ADMSbsimsoi::Model::WCDSBS)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of cdsbs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNOFFFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNOFFFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nofffd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVOFFFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVOFFFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vofffd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK1B", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK1B)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k1b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WK2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::WK2B)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of k2b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDK2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDK2B)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dk2b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVBD0", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVBD0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvbd0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WDVBD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::WDVBD1)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of dvbd1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WMOINFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WMOINFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of moinfd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVBS0PD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVBS0PD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vbs0pd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WVBS0FD", static_cast<double>(0.0), &ADMSbsimsoi::Model::WVBS0FD)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of vbs0fd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXJ", static_cast<double>(0.0), &ADMSbsimsoi::Model::PXJ)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xj")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PALPHAGB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PALPHAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of alphagb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBETAGB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBETAGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of betagb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PALPHAGB2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PALPHAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of alphagb2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBETAGB2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBETAGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of betagb2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of aigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of bigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCIGBCP2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCIGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cigbcp2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCGSL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCGSL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cgsl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCGDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCGDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cgdl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCKAPPA", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCKAPPA)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ckappa")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ndif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUTE", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ute")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKT1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKT1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of kt1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKT1L", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKT1L)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of kt1l")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKT2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of kt2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ua1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ub1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUC1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of uc1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of at")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPRT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPRT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of prt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNTRECF", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNTRECF)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ntrecf")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNTRECR", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNTRECR)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ntrecr")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PXBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::PXDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xdif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXREC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PXREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::PXTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xtun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXDIFD", 0.0, &ADMSbsimsoi::Model::PXDIFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xdifd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXRECD", 0.0, &ADMSbsimsoi::Model::PXRECD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xrecd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXTUND", 0.0, &ADMSbsimsoi::Model::PXTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xtund")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of aigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of bigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of aigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of bigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCIGSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCIGSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cigsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNIGC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNIGC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nigc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPIGCD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPIGCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence for pigcd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPOXEDGE", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPOXEDGE)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence for poxedge")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNCH", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNCH)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nch")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNSUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nsub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNSD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNSD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nsd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNGATE", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNGATE)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ngate")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVTH0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVTH0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vto")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVFB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVFB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vfb /* v4.1 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK1W1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK1W1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k1w1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK1W2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK1W2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k1w2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK3", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK3)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k3")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK3B", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK3B)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k3b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of kb1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PW0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PW0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of w0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PLPEB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PLPEB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of lpeb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVT0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVT0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvt0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVT1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVT1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvt1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVT2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVT2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvt2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVT0W", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVT0W)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvt0w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVT1W", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVT1W)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvt1w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVT2W", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVT2W)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvt2w")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PU0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PU0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of u0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PEU", static_cast<double>(0.0), &ADMSbsimsoi::Model::PEU)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of eu")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUA", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUA)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ua")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of uc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ud")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUD1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ud1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUCSTE", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUCSTE)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ucste")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PUCS", static_cast<double>(0.0), &ADMSbsimsoi::Model::PUCS)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ucs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVSAT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVSAT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vsat")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of a0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAGS", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAGS)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ags")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PB0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PB0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of b0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PB1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PB1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of b1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKETA", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKETA)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of keta")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKETAS", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKETAS)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ketas")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of a1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PA2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PA2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of a2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRDSW", static_cast<double>(0.0), &ADMSbsimsoi::Model::PRDSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of rdsw")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRSW", static_cast<double>(0.0), &ADMSbsimsoi::Model::PRSW)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of rsw /*v4.0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRDW", static_cast<double>(0.0), &ADMSbsimsoi::Model::PRDW)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of rdw /*v4.0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPRWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPRWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of prwb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPRWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPRWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of prwg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PWR", static_cast<double>(0.0), &ADMSbsimsoi::Model::PWR)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of wr")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNFACTOR", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNFACTOR)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nfactor")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDWG", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDWG)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dwg")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDWB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDWB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dwb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVOFF", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of voff")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of eta0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PETAB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PETAB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of etab")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PETA0CV", 0.0, &ADMSbsimsoi::Model::PETA0CV)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of eta0cv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PETABCV", 0.0, &ADMSbsimsoi::Model::PETABCV)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of etabcv")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDSUB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDSUB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dsub")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCIT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCIT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cit")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCDSC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCDSC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cdsc")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCDSCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCDSCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cdscb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCDSCD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCDSCD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cdscd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPCLM", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPCLM)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pclm")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPDIBLC1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPDIBLC1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pdiblc1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPDIBLC2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPDIBLC2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pdiblc2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPDIBLCB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPDIBLCB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pdiblcb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDROUT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDROUT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of drout")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PPVAG", static_cast<double>(0.0), &ADMSbsimsoi::Model::PPVAG)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of pvag")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDELTA", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDELTA)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of delta")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PALPHA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PALPHA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of alpha0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PFBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::PFBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of fbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PABJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::PABJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of abjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PEBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::PEBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ebjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PMBJTII", static_cast<double>(0.0), &ADMSbsimsoi::Model::PMBJTII)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of mbjtii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVBCI", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVBCI)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vbci")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBETA0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBETA0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of beta0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBETA1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBETA1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of beta1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBETA2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBETA2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of beta2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVDSATII0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVDSATII0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vdsatii0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PLII", static_cast<double>(0.0), &ADMSbsimsoi::Model::PLII)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of lii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PESATII", static_cast<double>(0.0), &ADMSbsimsoi::Model::PESATII)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of esatii")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PSII0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PSII0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of sii0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PSII1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PSII1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of sii1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PSII2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PSII2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of sii2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PSIID", static_cast<double>(0.0), &ADMSbsimsoi::Model::PSIID)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of siid")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of agidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PBGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of bgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PRGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of rgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PKGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of kgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PFGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PFGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of fgidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAGISL", 0.0, &ADMSbsimsoi::Model::PAGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of agisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PBGISL", 0.0, &ADMSbsimsoi::Model::PBGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of bgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCGISL", 0.0, &ADMSbsimsoi::Model::PCGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PRGISL", 0.0, &ADMSbsimsoi::Model::PRGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of rgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PKGISL", 0.0, &ADMSbsimsoi::Model::PKGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of kgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PFGISL", 0.0, &ADMSbsimsoi::Model::PFGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of fgisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ntun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNTUND", 0.0, &ADMSbsimsoi::Model::PNTUND)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ntund")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNDIODE", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNDIODE)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ndiode")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNDIODED", 0.0, &ADMSbsimsoi::Model::PNDIODED)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ndiode")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNRECF0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNRECF0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nrecf0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNRECF0D", 0.0, &ADMSbsimsoi::Model::PNRECF0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nrecf0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNRECR0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNRECR0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nrecr0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNRECR0D", 0.0, &ADMSbsimsoi::Model::PNRECR0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nrecr0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PISBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PISBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of isbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PIDBJT", 0.0, &ADMSbsimsoi::Model::PIDBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of idbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PISDIF", static_cast<double>(0.0), &ADMSbsimsoi::Model::PISDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of isdif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PIDDIF", 0.0, &ADMSbsimsoi::Model::PIDDIF)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of iddif")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PISREC", static_cast<double>(0.0), &ADMSbsimsoi::Model::PISREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of isrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PIDREC", 0.0, &ADMSbsimsoi::Model::PIDREC)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of idrec")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PISTUN", static_cast<double>(0.0), &ADMSbsimsoi::Model::PISTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of istun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PIDTUN", 0.0, &ADMSbsimsoi::Model::PIDTUN)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of idtun")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVREC0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVREC0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vrec0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVREC0D", 0.0, &ADMSbsimsoi::Model::PVREC0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vrec0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVTUN0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVTUN0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vtun0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVTUN0D", 0.0, &ADMSbsimsoi::Model::PVTUN0D)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vtun0d")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNBJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNBJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nbjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PLBJT0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PLBJT0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of lbjt0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVABJT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVABJT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vabjt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAELY", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAELY)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of aely")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAHLI", static_cast<double>(0.0), &ADMSbsimsoi::Model::PAHLI)
    .setUnit(U_UNKNOWN)
    .setDescription("X-term dependence of ahli /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PAHLID", 0.0, &ADMSbsimsoi::Model::PAHLID)
    .setUnit(U_UNKNOWN)
    .setDescription("X-term dependence of ahlid /* v4.0 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVSDFB", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVSDFB)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vsdfb")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVSDTH", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVSDTH)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vsdth")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDELVT", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDELVT)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of delvt")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PACDE", static_cast<double>(0.0), &ADMSbsimsoi::Model::PACDE)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of acde")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PMOIN", static_cast<double>(0.0), &ADMSbsimsoi::Model::PMOIN)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of moin")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNOFF", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNOFF)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of noff /* v3.2 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNOFF2", 0.0, &ADMSbsimsoi::Model::PNOFF2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of noff2 /* v4.6 ")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXRCRG1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PXRCRG1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xrcrg1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PXRCRG2", static_cast<double>(0.0), &ADMSbsimsoi::Model::PXRCRG2)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of xrcrg2")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVBSA", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVBSA)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vbsa")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVSCE", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVSCE)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vsce")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PCDSBS", static_cast<double>(0.0), &ADMSbsimsoi::Model::PCDSBS)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of cdsbs")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNOFFFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNOFFFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nofffd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVOFFFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVOFFFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vofffd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK1B", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK1B)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k1b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PK2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::PK2B)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of k2b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDK2B", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDK2B)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dk2b")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVBD0", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVBD0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvbd0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PDVBD1", static_cast<double>(0.0), &ADMSbsimsoi::Model::PDVBD1)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of dvbd1")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PMOINFD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PMOINFD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of moinfd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVBS0PD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVBS0PD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vbs0pd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PVBS0FD", static_cast<double>(0.0), &ADMSbsimsoi::Model::PVBS0FD)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of vbs0fd")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NLX", static_cast<double>(1.74e-7), &ADMSbsimsoi::Model::NLX)
    .setUnit(U_METER)
    .setDescription("Lateral non-uniform doping effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNLX", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNLX)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of nlx")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNLX", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNLX)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of nlx")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNLX", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNLX)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of nlx")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("NGIDL", static_cast<double>(1.2), &ADMSbsimsoi::Model::NGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("GIDL first parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LNGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::LNGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of ngidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WNGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::WNGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of ngidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PNGIDL", static_cast<double>(0.0), &ADMSbsimsoi::Model::PNGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of ngidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LPE0", 0.0, &ADMSbsimsoi::Model::LPE0)
    .setUnit(U_METER)
    .setDescription("Lateral non-uniform doping effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EGIDL", 0.0, &ADMSbsimsoi::Model::EGIDL)
    .setUnit(U_VOLT)
    .setDescription("GIDL first parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EGISL", 0.0, &ADMSbsimsoi::Model::EGISL)
    .setUnit(U_VOLT)
    .setDescription("GISL first parameter")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LLPE0", 0.0, &ADMSbsimsoi::Model::LLPE0)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of lpe0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LEGIDL", 0.0, &ADMSbsimsoi::Model::LEGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of egidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("LEGISL", 0.0, &ADMSbsimsoi::Model::LEGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Length dependence of egisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WLPE0", 0.0, &ADMSbsimsoi::Model::WLPE0)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of lpe0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WEGIDL", 0.0, &ADMSbsimsoi::Model::WEGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of egidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("WEGISL", 0.0, &ADMSbsimsoi::Model::WEGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Width dependence of egisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PLPE0", 0.0, &ADMSbsimsoi::Model::PLPE0)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of lpe0")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PEGIDL", 0.0, &ADMSbsimsoi::Model::PEGIDL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of egidl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("PEGISL", 0.0, &ADMSbsimsoi::Model::PEGISL)
    .setUnit(U_UNKNOWN)
    .setDescription("Cross-term dependence of egisl")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EGGBCP2", static_cast<double>(1.12), &ADMSbsimsoi::Model::EGGBCP2)
    .setUnit(U_UNKNOWN)
    .setDescription("Bandgap in Agbcp2 region")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("EGGDEP", static_cast<double>(1.12), &ADMSbsimsoi::Model::EGGDEP)
    .setUnit(U_UNKNOWN)
    .setDescription("Bandgap for gate depletion effect")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGB1", static_cast<double>(3.7622e-7), &ADMSbsimsoi::Model::AGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("'A' for Igb1 Tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BGB1", static_cast<double>((-3.1051e10)), &ADMSbsimsoi::Model::BGB1)
    .setUnit(U_UNKNOWN)
    .setDescription("'B' for Igb1 Tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGB2", static_cast<double>(4.9758e-7), &ADMSbsimsoi::Model::AGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("'A' for Igb2 Tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BGB2", static_cast<double>((-2.357e10)), &ADMSbsimsoi::Model::BGB2)
    .setUnit(U_UNKNOWN)
    .setDescription("'B' for Igb2 Tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBC2N", static_cast<double>(3.4254e-7), &ADMSbsimsoi::Model::AGBC2N)
    .setUnit(U_UNKNOWN)
    .setDescription("NMOS 'A' for tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("AGBC2P", static_cast<double>(4.9723e-7), &ADMSbsimsoi::Model::AGBC2P)
    .setUnit(U_UNKNOWN)
    .setDescription("PMOS 'A' for tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BGBC2N", static_cast<double>(1.1665e12), &ADMSbsimsoi::Model::BGBC2N)
    .setUnit(U_UNKNOWN)
    .setDescription("NMOS 'B' for tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("BGBC2P", static_cast<double>(7.4567e11), &ADMSbsimsoi::Model::BGBC2P)
    .setUnit(U_UNKNOWN)
    .setDescription("PMOS 'B' for tunneling current model")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;
  p.addPar("VTM00", static_cast<double>(0.026), &ADMSbsimsoi::Model::VTM00)
    .setUnit(U_VOLT)
    .setDescription("Hard coded 25 degC thermal voltage")
#ifdef Xyce_ADMS_SENSITIVITIES
    .setAnalyticSensitivityAvailable(true)
    .setSensitivityFunctor(&modSens)
#endif // Xyce_ADMS_SENSITIVITIES
;

}

//-----------------------------------------------------------------------------
// Function      : Instance::processParams
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::processParams()
{
  // set any non-constant parameter defaults

  // copy any model variables that have associated instance variables, but
  // are only given in the model card:
   if (!(given("L")))
   {
      L = model_.L;
   }
   if (!(given("W")))
   {
      W = model_.W;
   }
   if (!(given("NF")))
   {
      NF = model_.NF;
   }
   if (!(given("SA")))
   {
      SA = model_.SA;
   }
   if (!(given("SB")))
   {
      SB = model_.SB;
   }
   if (!(given("SD")))
   {
      SD = model_.SD;
   }
   if (!(given("AD")))
   {
      AD = model_.AD;
   }
   if (!(given("AS")))
   {
      AS = model_.AS;
   }
   if (!(given("PD")))
   {
      PD = model_.PD;
   }
   if (!(given("PS")))
   {
      PS = model_.PS;
   }
   if (!(given("NRD")))
   {
      NRD = model_.NRD;
   }
   if (!(given("NRS")))
   {
      NRS = model_.NRS;
   }
   if (!(given("OFF")))
   {
      OFF = model_.OFF;
   }
   if (!(given("BJTOFF")))
   {
      BJTOFF = model_.BJTOFF;
   }
   if (!(given("RTH0")))
   {
      RTH0 = model_.RTH0;
   }
   if (!(given("CTH0")))
   {
      CTH0 = model_.CTH0;
   }
   if (!(given("NRB")))
   {
      NRB = model_.NRB;
   }
   if (!(given("FRBODY")))
   {
      FRBODY = model_.FRBODY;
   }
   if (!(given("RBDB")))
   {
      RBDB = model_.RBDB;
   }
   if (!(given("RBSB")))
   {
      RBSB = model_.RBSB;
   }
   if (!(given("DELVTO")))
   {
      DELVTO = model_.DELVTO;
   }
   if (!(given("SOIMOD")))
   {
      SOIMOD = model_.SOIMOD;
   }
   if (!(given("NBC")))
   {
      NBC = model_.NBC;
   }
   if (!(given("NSEG")))
   {
      NSEG = model_.NSEG;
   }
   if (!(given("PDBCP")))
   {
      PDBCP = model_.PDBCP;
   }
   if (!(given("PSBCP")))
   {
      PSBCP = model_.PSBCP;
   }
   if (!(given("AGBCP")))
   {
      AGBCP = model_.AGBCP;
   }
   if (!(given("AGBCP2")))
   {
      AGBCP2 = model_.AGBCP2;
   }
   if (!(given("AGBCPD")))
   {
      AGBCPD = model_.AGBCPD;
   }
   if (!(given("AEBCP")))
   {
      AEBCP = model_.AEBCP;
   }
   if (!(given("SHMOD")))
   {
      SHMOD = model_.SHMOD;
   }



  // Set any parameters that were not given and whose defaults depend on other
  // parameters:
  if (!given("AGBCPD"))
  {
  AGBCPD = AGBCP;
  }


// Now we need to check that any parameters are within their ranges as
  // specified in the verilog:

//    Parameter L : ] 0,  (+inf) [
  if ( (!((L >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter L value " << L << " out of range ] 0,  (+inf) [";
  }

//    Parameter W : ] 0,  (+inf) [
  if ( (!((W >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter W value " << W << " out of range ] 0,  (+inf) [";
  }

//    Parameter NF : [ 1,  (+inf) [
  if ( (!((NF >=1))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NF value " << NF << " out of range [ 1,  (+inf) [";
  }

//    Parameter SA : [ 0,  (+inf) [
  if ( (!((SA >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SA value " << SA << " out of range [ 0,  (+inf) [";
  }

//    Parameter SB : [ 0,  (+inf) [
  if ( (!((SB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SB value " << SB << " out of range [ 0,  (+inf) [";
  }

//    Parameter SD : [ 0,  (+inf) [
  if ( (!((SD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SD value " << SD << " out of range [ 0,  (+inf) [";
  }

//    Parameter AD : [ 0,  (+inf) [
  if ( (!((AD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AD value " << AD << " out of range [ 0,  (+inf) [";
  }

//    Parameter AS : [ 0,  (+inf) [
  if ( (!((AS >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AS value " << AS << " out of range [ 0,  (+inf) [";
  }

//    Parameter PD : [ 0,  (+inf) [
  if ( (!((PD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PD value " << PD << " out of range [ 0,  (+inf) [";
  }

//    Parameter PS : [ 0,  (+inf) [
  if ( (!((PS >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PS value " << PS << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRD : [ 0,  (+inf) [
  if ( (!((NRD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRD value " << NRD << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRS : [ 0,  (+inf) [
  if ( (!((NRS >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRS value " << NRS << " out of range [ 0,  (+inf) [";
  }

//    Parameter OFF : [ 0, 1 ]
  if ( (!((OFF >=0 && OFF <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter OFF value " << OFF << " out of range [ 0, 1 ]";
  }

//    Parameter BJTOFF : [ 0, 1 ]
  if ( (!((BJTOFF >=0 && BJTOFF <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter BJTOFF value " << BJTOFF << " out of range [ 0, 1 ]";
  }

//    Parameter DEBUG : [ 0, 1 ]
  if ( (!((DEBUG >=0 && DEBUG <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter DEBUG value " << DEBUG << " out of range [ 0, 1 ]";
  }

//    Parameter RTH0 : [ 0,  (+inf) [
  if ( (!((RTH0 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RTH0 value " << RTH0 << " out of range [ 0,  (+inf) [";
  }

//    Parameter CTH0 : [ 0,  (+inf) [
  if ( (!((CTH0 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CTH0 value " << CTH0 << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRB : [ 0,  (+inf) [
  if ( (!((NRB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRB value " << NRB << " out of range [ 0,  (+inf) [";
  }

//    Parameter RBDB : [ 0,  (+inf) [
  if ( (!((RBDB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RBDB value " << RBDB << " out of range [ 0,  (+inf) [";
  }

//    Parameter RBSB : [ 0,  (+inf) [
  if ( (!((RBSB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RBSB value " << RBSB << " out of range [ 0,  (+inf) [";
  }

//    Parameter SOIMOD : [ 0, 3 ]
  if ( (!((SOIMOD >=0 && SOIMOD <=3 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SOIMOD value " << SOIMOD << " out of range [ 0, 3 ]";
  }

//    Parameter NBC : [ 0,  (+inf) [
  if ( (!((NBC >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NBC value " << NBC << " out of range [ 0,  (+inf) [";
  }

//    Parameter NSEG : [ 1,  (+inf) [
  if ( (!((NSEG >=1))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NSEG value " << NSEG << " out of range [ 1,  (+inf) [";
  }

//    Parameter PDBCP : [ 0,  (+inf) [
  if ( (!((PDBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PDBCP value " << PDBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter PSBCP : [ 0,  (+inf) [
  if ( (!((PSBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PSBCP value " << PSBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter AGBCP : [ 0,  (+inf) [
  if ( (!((AGBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AGBCP value " << AGBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter AGBCP2 : [ 0,  (+inf) [
  if ( (!((AGBCP2 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AGBCP2 value " << AGBCP2 << " out of range [ 0,  (+inf) [";
  }

//    Parameter AGBCPD : [ 0,  (+inf) [
  if ( (!((AGBCPD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AGBCPD value " << AGBCPD << " out of range [ 0,  (+inf) [";
  }

//    Parameter AEBCP : [ 0,  (+inf) [
  if ( (!((AEBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AEBCP value " << AEBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter TNODEOUT : [ 0, 1 ]
  if ( (!((TNODEOUT >=0 && TNODEOUT <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TNODEOUT value " << TNODEOUT << " out of range [ 0, 1 ]";
  }

//    Parameter SHMOD : [ 0, 1 ]
  if ( (!((SHMOD >=0 && SHMOD <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SHMOD value " << SHMOD << " out of range [ 0, 1 ]";
  }

//    Parameter m : ] 0.0,  (+inf) [
  if ( (!((m >0.0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter m value " << m << " out of range ] 0.0,  (+inf) [";
  }


  // this seems a little stupid, but verilog models that use $temperature
  // don't also use a defined parameter "Temp", and count on $temperature
  // to be the one the simulator wants:

  updateTemperature(admsInstTemp);

  // and of course, this routine is where we should put the initial_instance
  // stuff
  double pParam_B4SOIdl=0.0;
  double pParam_B4SOIdlc=0.0;
  double pParam_B4SOIdlcig=0.0;
  double pParam_B4SOIdw=0.0;
  double pParam_B4SOIdwc=0.0;
  double pParam_B4SOIcf=0.0;
  double pParam_B4SOIcle=0.0;
  double pParam_B4SOInsd=0.0;
  double pParam_B4SOIvth0=0.0;
  double pParam_B4SOIwr=0.0;
  double pParam_B4SOInbjt=0.0;
  double pParam_B4SOIlbjt0=0.0;
  double pParam_B4SOIvabjt=0.0;
  double pParam_B4SOIaely=0.0;
  double pParam_B4SOIndif=0.0;
  double pParam_B4SOIvbs0pd=0.0;
  double pParam_B4SOIvbs0fd=0.0;
  double pParam_B4SOIdvtp2=0.0;
  double pParam_B4SOIdvtp3=0.0;
  double pParam_B4SOIminv=0.0;
  double pParam_B4SOIminvcv=0.0;
  double NchMax=0.0;
  double Qsi=0.0;
  double Vbs0t=0.0;
  double pParam_B4SOIsqrtXdep0=0.0;
  double pParam_B4SOIToxRatio=0.0;
  double pParam_B4SOIToxRatioEdge=0.0;
  double pParam_B4SOIkvth0=0.0;
  double pParam_B4SOIku0temp=0.0;
  double pParam_B4SOIrho_ref=0.0;
  double pParam_B4SOIvfbzb=0.0;
  double B4SOIcgso=0.0;
  double B4SOIcgdo=0.0;
  double B4SOIcf=0.0;
  double B4SOIVTM00=0.0;
  double B4SOIBGBC2P=0.0;
  double B4SOIBGBC2N=0.0;
  double B4SOIAGBC2P=0.0;
  double B4SOIAGBC2N=0.0;
  double B4SOIBGB2=0.0;
  double B4SOIAGB2=0.0;
  double B4SOIBGB1=0.0;
  double B4SOIAGB1=0.0;
  double B4SOIEGGDEP=0.0;
  double B4SOIEGGBCP2=0.0;
  double B4SOIvoffcv=0.0;
  double B4SOIminvcv=0.0;
  double B4SOIcdsbs=0.0;
  double B4SOIvsce=0.0;
  double B4SOIxgl=0.0;
  double B4SOIxgw=0.0;
  double B4SOIngcon=0.0;
  double B4SOIrshg=0.0;
  double B4SOIxrcrg2=0.0;
  double B4SOIxrcrg1=0.0;
  double B4SOIvbs0fd=0.0;
  double B4SOIvbs0pd=0.0;
  double B4SOImoinFD=0.0;
  double B4SOIdvbd1=0.0;
  double B4SOIdvbd0=0.0;
  double B4SOIdk2b=0.0;
  double B4SOIk2b=0.0;
  double B4SOIk1b=0.0;
  double B4SOIvofffd=0.0;
  double B4SOInofffd=0.0;
  double B4SOIvbsa=0.0;
  double B4SOIdlcig=0.0;
  double B4SOIpoxedge=0.0;
  double B4SOIpigcd=0.0;
  double B4SOInigc=0.0;
  double B4SOIcigsd=0.0;
  double B4SOIbigsd=0.0;
  double B4SOIaigsd=0.0;
  double B4SOIcigc=0.0;
  double B4SOIbigc=0.0;
  double B4SOIaigc=0.0;
  double B4SOIcigbcp2=0.0;
  double B4SOIbigbcp2=0.0;
  double B4SOIaigbcp2=0.0;
  double B4SOIbetaGB2=0.0;
  double B4SOIalphaGB2=0.0;
  double B4SOIbetaGB1=0.0;
  double B4SOIalphaGB1=0.0;
  double B4SOIcfrcoeff=0.0;
  double B4SOIkb1=0.0;
  double B4SOIdelvt=0.0;
  double B4SOInoff2=0.0;
  double B4SOInoff=0.0;
  double B4SOIdlcb=0.0;
  double B4SOIntrecr=0.0;
  double B4SOIntrecf=0.0;
  double B4SOIvsdth=0.0;
  double B4SOIvsdfb=0.0;
  double B4SOIndif=0.0;
  double B4SOIcgeo=0.0;
  double B4SOIvabjt=0.0;
  double B4SOIldif0=0.0;
  double B4SOIlbjt0=0.0;
  double B4SOIln=0.0;
  double B4SOInrecr0d=0.0;
  double B4SOInrecr0=0.0;
  double B4SOInrecf0d=0.0;
  double B4SOInrecf0=0.0;
  double B4SOImbjtii=0.0;
  double B4SOIabjtii=0.0;
  double B4SOIvbci=0.0;
  double B4SOIcbjtii=0.0;
  double B4SOIebjtii=0.0;
  double B4SOIsii0=0.0;
  double B4SOIvdsatii0=0.0;
  double B4SOIfprout=0.0;
  double B4SOIpditsd=0.0;
  double B4SOIpdits=0.0;
  double B4SOIminv=0.0;
  double B4SOIdvtp4=0.0;
  double B4SOIdvtp3=0.0;
  double B4SOIdvtp2=0.0;
  double B4SOIdvtp1=0.0;
  double B4SOIdvtp0=0.0;
  double B4SOIgbmin=0.0;
  double B4SOIsteta0cv=0.0;
  double B4SOIsteta0=0.0;
  double B4SOIstk2=0.0;
  double B4SOIwlodvth=0.0;
  double B4SOIllodvth=0.0;
  double B4SOIwlodku0=0.0;
  double B4SOIllodku0=0.0;
  double B4SOIkvth0=0.0;
  double B4SOIwlod=0.0;
  double B4SOIalpha0=0.0;
  double B4SOIdlc=0.0;
  double B4SOIdwc=0.0;
  double B4SOIcle=0.0;
  double B4SOIclc=0.0;
  double B4SOIckappa=0.0;
  double B4SOIcgdl=0.0;
  double B4SOIcgsl=0.0;
  double B4SOIb1=0.0;
  double B4SOIb0=0.0;
  double B4SOIWwlc=0.0;
  double B4SOIWwl=0.0;
  double B4SOIWwn=0.0;
  double B4SOIWwc=0.0;
  double B4SOIWw=0.0;
  double B4SOIWln=0.0;
  double B4SOIWlc=0.0;
  double B4SOIWl=0.0;
  double B4SOIdwb=0.0;
  double B4SOIdwg=0.0;
  double B4SOIWint=0.0;
  double B4SOIwr=0.0;
  double B4SOILwlc=0.0;
  double B4SOILwl=0.0;
  double B4SOILwn=0.0;
  double B4SOILwc=0.0;
  double B4SOILw=0.0;
  double B4SOILln=0.0;
  double B4SOILlc=0.0;
  double B4SOILl=0.0;
  double B4SOILint=0.0;
  double B4SOIxtund=0.0;
  double B4SOIxrecd=0.0;
  double B4SOIxdifd=0.0;
  double B4SOIxtun=0.0;
  double B4SOIxrec=0.0;
  double B4SOIxdif=0.0;
  double B4SOIxbjt=0.0;
  double B4SOIfgisl=0.0;
  double B4SOIkgisl=0.0;
  double B4SOIrgisl=0.0;
  double B4SOIfgidl=0.0;
  double B4SOIkgidl=0.0;
  double B4SOIrgidl=0.0;
  double B4SOIxj=0.0;
  double B4SOIetsi=0.0;
  double B4SOIpvag=0.0;
  double B4SOIpdiblb=0.0;
  double B4SOIpdibl2=0.0;
  double B4SOIpdibl1=0.0;
  double B4SOIpclm=0.0;
  double B4SOIetabcv=0.0;
  double B4SOIeta0cv=0.0;
  double B4SOIetab=0.0;
  double B4SOIeta0=0.0;
  double B4SOIprt=0.0;
  double B4SOIprwb=0.0;
  double B4SOIprwg=0.0;
  double B4SOIrdw=0.0;
  double B4SOIrsw=0.0;
  double B4SOIrdsw=0.0;
  double B4SOIsheetResistance=0.0;
  double B4SOIdelta=0.0;
  double B4SOIvoff=0.0;
  double B4SOIud1=0.0;
  double B4SOIud=0.0;
  double B4SOIucste=0.0;
  double B4SOIucs=0.0;
  double B4SOIute=0.0;
  double B4SOIeu=0.0;
  double B4SOIu0=0.0;
  double B4SOIuc1=0.0;
  double B4SOIuc=0.0;
  double B4SOIub1=0.0;
  double B4SOIub=0.0;
  double B4SOIua1=0.0;
  double B4SOIua=0.0;
  double B4SOIvfb=0.0;
  double B4SOIvth0=0.0;
  double B4SOIdsub=0.0;
  double B4SOIdrout=0.0;
  double B4SOIdvt2w=0.0;
  double B4SOIdvt1w=0.0;
  double B4SOIdvt0w=0.0;
  double B4SOIdvt2=0.0;
  double B4SOIdvt1=0.0;
  double B4SOIdvt0=0.0;
  double B4SOIlpeb=0.0;
  double B4SOIlpe0=0.0;
  double B4SOIw0=0.0;
  double B4SOIk3b=0.0;
  double B4SOIk3=0.0;
  double B4SOIk2=0.0;
  double B4SOIkt2=0.0;
  double B4SOIkt1l=0.0;
  double B4SOIkt1=0.0;
  double B4SOIk1=0.0;
  double B4SOIxt=0.0;
  double B4SOIvbm=0.0;
  double B4SOIvbx=0.0;
  double B4SOIgamma2=0.0;
  double B4SOIgamma1=0.0;
  double B4SOInpeak=0.0;
  double B4SOInsub=0.0;
  double B4SOIketa=0.0;
  double B4SOIa2=0.0;
  double B4SOIa1=0.0;
  double B4SOIags=0.0;
  double B4SOIa0=0.0;
  double B4SOIat=0.0;
  double B4SOIvsat=0.0;
  double B4SOInfactor=0.0;
  double B4SOIcit=0.0;
  double B4SOIcdscd=0.0;
  double B4SOIcdscb=0.0;
  double B4SOIcdsc=0.0;
  int B4SOIbinUnit=0.0;
  double B4SOItempeot=0.0;
  double B4SOIvddeot=0.0;
  double B4SOIepsrox=0.0;
  double B4SOIpsbcp=0.0;
  double B4SOIpdbcp=0.0;
  double B4SOIdelvto=0.0;
  double B4SOIrbsb=0.0;
  double B4SOIrbdb=0.0;
  double B4SOIfrbody=0.0;
  double B4SOIbodySquares=0.0;
  double B4SOIsourceSquares=0.0;
  double B4SOIdrainSquares=0.0;
  double B4SOIsourcePerimeter=0.0;
  double B4SOIdrainPerimeter=0.0;
  double B4SOIsourceArea=0.0;
  double B4SOIdrainArea=0.0;
  double B4SOIw=0.0;
  double B4SOIl=0.0;
//Begin block initial_instance
{

d_DevTemp_dTemp_t_GND = d_DevTemp_dV_p_GND = d_DevTemp_dV_b_GND =  0.0;
DevTemp = (admsTemperature+DTEMP);
B4SOItype = (model_.TYPE);
B4SOIl = L;
B4SOIw = W;
B4SOInf = NF;
B4SOIsa = SA;
B4SOIsb = SB;
B4SOIsd = SD;
B4SOIdrainArea = AD;
B4SOIsourceArea = AS;
B4SOIdrainPerimeter = PD;
B4SOIsourcePerimeter = PS;
B4SOIdrainSquares = NRD;
B4SOIsourceSquares = NRS;
B4SOIbjtoff = BJTOFF;
B4SOIrth0 = RTH0;
B4SOIcth0 = CTH0;
B4SOIbodySquares = NRB;
B4SOIfrbody = FRBODY;
B4SOIrbdb = RBDB;
B4SOIrbsb = RBSB;
B4SOIdelvto = DELVTO;
B4SOIsoiMod = SOIMOD;
B4SOInbc = NBC;
B4SOInseg = NSEG;
B4SOIpdbcp = PDBCP;
B4SOIpsbcp = PSBCP;
B4SOIagbcp = AGBCP;
B4SOIagbcp2 = AGBCP2;
B4SOIagbcpd = AGBCPD;
B4SOIaebcp = AEBCP;
B4SOIrgateMod = (model_.RGATEMOD);
B4SOIrbodyMod = (model_.RBODYMOD);
B4SOImtrlMod = (model_.MTRLMOD);
B4SOIvgstcvMod = (model_.VGSTCVMOD);
B4SOIgidlMod = (model_.GIDLMOD);
B4SOIiiiMod = (model_.IIIMOD);
B4SOIeot = (model_.EOT);
B4SOIepsrox = (model_.EPSROX);
B4SOIepsrsub = (model_.EPSRSUB);
B4SOIni0sub = (model_.NI0SUB);
B4SOIbg0sub = (model_.BG0SUB);
B4SOItbgasub = (model_.TBGASUB);
B4SOItbgbsub = (model_.TBGBSUB);
B4SOIphig = (model_.PHIG);
B4SOIeasub = (model_.EASUB);
B4SOIleffeot = (model_.LEFFEOT);
B4SOIweffeot = (model_.WEFFEOT);
B4SOIvddeot = (model_.VDDEOT);
B4SOItempeot = (model_.TEMPEOT);
B4SOIados = (model_.ADOS);
B4SOIbdos = (model_.BDOS);
B4SOIepsrgate = (model_.EPSRGATE);
B4SOIcapMod = (model_.CAPMOD);
B4SOImobMod = (model_.MOBMOD);
B4SOIbinUnit = (model_.BINUNIT);
B4SOItox = (model_.TOX);
B4SOItoxm = (model_.TOXM);
B4SOIdtoxcv = (model_.DTOXCV);
B4SOIcdsc = (model_.CDSC);
B4SOIcdscb = (model_.CDSCB);
B4SOIcdscd = (model_.CDSCD);
B4SOIcit = (model_.CIT);
B4SOInfactor = (model_.NFACTOR);
B4SOIvsat = (model_.VSAT);
B4SOIat = (model_.AT);
B4SOIa0 = (model_.A0);
B4SOIags = (model_.AGS);
B4SOIa1 = (model_.A1);
B4SOIa2 = (model_.A2);
B4SOIketa = (model_.KETA);
B4SOInsub = (model_.NSUB);
B4SOInpeak = (model_.NCH);
B4SOIngate = (model_.NGATE);
B4SOInsd = (model_.NSD);
B4SOIgamma1 = (model_.GAMMA1);
B4SOIgamma2 = (model_.GAMMA2);
B4SOIvbx = (model_.VBX);
B4SOIvbm = (model_.VBM);
B4SOIxt = (model_.XT);
B4SOIk1 = (model_.K1);
B4SOIkt1 = (model_.KT1);
B4SOIkt1l = (model_.KT1L);
B4SOIkt2 = (model_.KT2);
B4SOIk2 = (model_.K2);
B4SOIk3 = (model_.K3);
B4SOIk3b = (model_.K3B);
B4SOIw0 = (model_.W0);
B4SOIlpe0 = (model_.LPE0);
B4SOIlpeb = (model_.LPEB);
B4SOIdvt0 = (model_.DVT0);
B4SOIdvt1 = (model_.DVT1);
B4SOIdvt2 = (model_.DVT2);
B4SOIdvt0w = (model_.DVT0W);
B4SOIdvt1w = (model_.DVT1W);
B4SOIdvt2w = (model_.DVT2W);
B4SOIdrout = (model_.DROUT);
B4SOIdsub = (model_.DSUB);
B4SOIvth0 = (model_.VTH0);
B4SOIvfb = (model_.VFB);
B4SOIua = (model_.UA);
B4SOIua1 = (model_.UA1);
B4SOIub = (model_.UB);
B4SOIub1 = (model_.UB1);
B4SOIuc = (model_.UC);
B4SOIuc1 = (model_.UC1);
B4SOIu0 = (model_.U0);
B4SOIeu = (model_.EU);
B4SOIute = (model_.UTE);
B4SOIucs = (model_.UCS);
B4SOIucste = (model_.UCSTE);
B4SOIud = (model_.UD);
B4SOIud1 = (model_.UD1);
B4SOIvoff = (model_.VOFF);
B4SOItnom = ((model_.TNOM)+273.15);
B4SOIxpart = (model_.XPART);
B4SOIdelta = (model_.DELTA);
B4SOIsheetResistance = (model_.RSH);
B4SOIrdsw = (model_.RDSW);
B4SOIrsw = (model_.RSW);
B4SOIrdw = (model_.RDW);
B4SOIrswmin = (model_.RSWMIN);
B4SOIrdwmin = (model_.RDWMIN);
B4SOIprwg = (model_.PRWG);
B4SOIprwb = (model_.PRWB);
B4SOIprt = (model_.PRT);
B4SOIeta0 = (model_.ETA0);
B4SOIetab = (model_.ETAB);
B4SOIeta0cv = (model_.ETA0CV);
B4SOIetabcv = (model_.ETABCV);
B4SOIpclm = (model_.PCLM);
B4SOIpdibl1 = (model_.PDIBLC1);
B4SOIpdibl2 = (model_.PDIBLC2);
B4SOIpdiblb = (model_.PDIBLCB);
B4SOIpvag = (model_.PVAG);
B4SOItbox = (model_.TBOX);
B4SOItsi = (model_.TSI);
B4SOIetsi = (model_.ETSI);
B4SOIxj = (model_.XJ);
B4SOIegidl = (model_.EGIDL);
B4SOIagidl = (model_.AGIDL);
B4SOIbgidl = (model_.BGIDL);
B4SOIcgidl = (model_.CGIDL);
B4SOIrgidl = (model_.RGIDL);
B4SOIkgidl = (model_.KGIDL);
B4SOIfgidl = (model_.FGIDL);
B4SOIegisl = (model_.EGISL);
B4SOIagisl = (model_.AGISL);
B4SOIbgisl = (model_.BGISL);
B4SOIcgisl = (model_.CGISL);
B4SOIrgisl = (model_.RGISL);
B4SOIkgisl = (model_.KGISL);
B4SOIfgisl = (model_.FGISL);
B4SOIndiode = (model_.NDIODE);
B4SOIndioded = (model_.NDIODED);
B4SOIxbjt = (model_.XBJT);
B4SOIxdif = (model_.XDIF);
B4SOIxrec = (model_.XREC);
B4SOIxtun = (model_.XTUN);
B4SOIxdifd = (model_.XDIFD);
B4SOIxrecd = (model_.XRECD);
B4SOIxtund = (model_.XTUND);
B4SOIGatesidewallJctSPotential = (model_.PBSWG);
B4SOIGatesidewallJctDPotential = (model_.PBSWGD);
B4SOIbodyJctGateSideSGradingCoeff = (model_.MJSWG);
B4SOIbodyJctGateSideDGradingCoeff = (model_.MJSWGD);
B4SOIunitLengthGateSidewallJctCapS = (model_.CJSWG);
B4SOIunitLengthGateSidewallJctCapD = (model_.CJSWGD);
B4SOILint = (model_.LINT);
B4SOILl = (model_.LL);
B4SOILlc = (model_.LLC);
B4SOILln = (model_.LLN);
B4SOILw = (model_.LW);
B4SOILwc = (model_.LWC);
B4SOILwn = (model_.LWN);
B4SOILwl = (model_.LWL);
B4SOILwlc = (model_.LWLC);
B4SOIwr = (model_.WR);
B4SOIWint = (model_.WINT);
B4SOIdwg = (model_.DWG);
B4SOIdwb = (model_.DWB);
B4SOIWl = (model_.WL);
B4SOIWlc = (model_.WLC);
B4SOIWln = (model_.WLN);
B4SOIWw = (model_.WW);
B4SOIWwc = (model_.WWC);
B4SOIWwn = (model_.WWN);
B4SOIWwl = (model_.WWL);
B4SOIWwlc = (model_.WWLC);
B4SOIb0 = (model_.B0);
B4SOIb1 = (model_.B1);
B4SOIcgsl = (model_.CGSL);
B4SOIcgdl = (model_.CGDL);
B4SOIckappa = (model_.CKAPPA);
B4SOIclc = (model_.CLC);
B4SOIcle = (model_.CLE);
B4SOIdwc = (model_.DWC);
B4SOIdlc = (model_.DLC);
B4SOIalpha0 = (model_.ALPHA0);
B4SOIoxideTrapDensityA = (model_.NOIA);
B4SOIoxideTrapDensityB = (model_.NOIB);
B4SOIoxideTrapDensityC = (model_.NOIC);
B4SOItnoia = (model_.TNOIA);
B4SOItnoib = (model_.TNOIB);
B4SOIrnoia = (model_.RNOIA);
B4SOIrnoib = (model_.RNOIB);
B4SOIntnoi = (model_.NTNOI);
B4SOIsaref = (model_.SAREF);
B4SOIsbref = (model_.SBREF);
B4SOIwlod = (model_.WLOD);
B4SOIku0 = (model_.KU0);
B4SOIkvsat = (model_.KVSAT);
B4SOIkvth0 = (model_.KVTH0);
B4SOItku0 = (model_.TKU0);
B4SOIllodku0 = (model_.LLODKU0);
B4SOIwlodku0 = (model_.WLODKU0);
B4SOIllodvth = (model_.LLODVTH);
B4SOIwlodvth = (model_.WLODVTH);
B4SOIstk2 = (model_.STK2);
B4SOIlodk2 = (model_.LODK2);
B4SOIsteta0 = (model_.STETA0);
B4SOIlodeta0 = (model_.LODETA0);
B4SOIsteta0cv = (model_.STETA0CV);
B4SOIlodeta0cv = (model_.LODETA0CV);
B4SOIgbmin = (model_.GBMIN);
B4SOIbf = (model_.BF);
B4SOIw0flk = (model_.W0FLK);
B4SOIdvtp0 = (model_.DVTP0);
B4SOIdvtp1 = (model_.DVTP1);
B4SOIdvtp2 = (model_.DVTP2);
B4SOIdvtp3 = (model_.DVTP3);
B4SOIdvtp4 = (model_.DVTP4);
B4SOIminv = (model_.MINV);
B4SOIpdits = (model_.PDITS);
B4SOIpditsl = (model_.PDITSL);
B4SOIpditsd = (model_.PDITSD);
B4SOIfprout = (model_.FPROUT);
B4SOIem = (model_.EM);
B4SOIef = (model_.EF);
B4SOIaf = (model_.AF);
B4SOIkf = (model_.KF);
B4SOInoif = (model_.NOIF);
B4SOIk1w1 = (model_.K1W1);
B4SOIk1w2 = (model_.K1W2);
B4SOIketas = (model_.KETAS);
B4SOIdwbc = (model_.DWBC);
B4SOIbeta0 = (model_.BETA0);
B4SOIbeta1 = (model_.BETA1);
B4SOIbeta2 = (model_.BETA2);
B4SOIvdsatii0 = (model_.VDSATII0);
B4SOItii = (model_.TII);
B4SOIlii = (model_.LII);
B4SOIsii0 = (model_.SII0);
B4SOIsii1 = (model_.SII1);
B4SOIsii2 = (model_.SII2);
B4SOIsiid = (model_.SIID);
B4SOIfbjtii = (model_.FBJTII);
B4SOIebjtii = (model_.EBJTII);
B4SOIcbjtii = (model_.CBJTII);
B4SOIvbci = (model_.VBCI);
B4SOIabjtii = (model_.ABJTII);
B4SOImbjtii = (model_.MBJTII);
B4SOItvbci = (model_.TVBCI);
B4SOIesatii = (model_.ESATII);
B4SOIntun = (model_.NTUN);
B4SOIntund = (model_.NTUND);
B4SOInrecf0 = (model_.NRECF0);
B4SOInrecf0d = (model_.NRECF0D);
B4SOInrecr0 = (model_.NRECR0);
B4SOInrecr0d = (model_.NRECR0D);
B4SOIisbjt = (model_.ISBJT);
B4SOIidbjt = (model_.IDBJT);
B4SOIisdif = (model_.ISDIF);
B4SOIiddif = (model_.IDDIF);
B4SOIisrec = (model_.ISREC);
B4SOIidrec = (model_.IDREC);
B4SOIistun = (model_.ISTUN);
B4SOIidtun = (model_.IDTUN);
B4SOIln = (model_.LN);
B4SOIvrec0 = (model_.VREC0);
B4SOIvrec0d = (model_.VREC0D);
B4SOIvtun0 = (model_.VTUN0);
B4SOIvtun0d = (model_.VTUN0D);
B4SOInbjt = (model_.NBJT);
B4SOIlbjt0 = (model_.LBJT0);
B4SOIldif0 = (model_.LDIF0);
B4SOIvabjt = (model_.VABJT);
B4SOIaely = (model_.AELY);
B4SOIahli = (model_.AHLI);
B4SOIahlid = (model_.AHLID);
B4SOIrbody = (model_.RBODY);
B4SOIrbsh = (model_.RBSH);
B4SOIcgeo = (model_.CGEO);
B4SOItt = (model_.TT);
B4SOIndif = (model_.NDIF);
B4SOIvsdfb = (model_.VSDFB);
B4SOIvsdth = (model_.VSDTH);
B4SOIcsdmin = (model_.CSDMIN);
B4SOIasd = (model_.ASD);
B4SOIcsdesw = (model_.CSDESW);
B4SOIntrecf = (model_.NTRECF);
B4SOIntrecr = (model_.NTRECR);
B4SOIdlcb = (model_.DLCB);
B4SOIfbody = (model_.FBODY);
B4SOItcjswg = (model_.TCJSWG);
B4SOItpbswg = (model_.TPBSWG);
B4SOItcjswgd = (model_.TCJSWGD);
B4SOItpbswgd = (model_.TPBSWGD);
B4SOIacde = (model_.ACDE);
B4SOImoin = (model_.MOIN);
B4SOInoff = (model_.NOFF);
B4SOInoff2 = (model_.NOFF2);
B4SOIdelvt = (model_.DELVT);
B4SOIkb1 = (model_.KB1);
B4SOIdlbg = (model_.DLBG);
B4SOIcfrcoeff = (model_.CFRCOEFF);
B4SOIigbMod = (model_.IGMOD);
B4SOIigcMod = (model_.IGCMOD);
B4SOItoxqm = (model_.TOXQM);
B4SOIwth0 = (model_.WTH0);
B4SOIrhalo = (model_.RHALO);
B4SOIntox = (model_.NTOX);
B4SOItoxref = (model_.TOXREF);
B4SOIebg = (model_.EBG);
B4SOIvevb = (model_.VEVB);
B4SOIalphaGB1 = (model_.ALPHAGB1);
B4SOIbetaGB1 = (model_.BETAGB1);
B4SOIvgb1 = (model_.VGB1);
B4SOIvecb = (model_.VECB);
B4SOIalphaGB2 = (model_.ALPHAGB2);
B4SOIbetaGB2 = (model_.BETAGB2);
B4SOIvgb2 = (model_.VGB2);
B4SOIaigbcp2 = (model_.AIGBCP2);
B4SOIbigbcp2 = (model_.BIGBCP2);
B4SOIcigbcp2 = (model_.CIGBCP2);
B4SOIvoxh = (model_.VOXH);
B4SOIdeltavox = (model_.DELTAVOX);
B4SOIaigc = (model_.AIGC);
B4SOIbigc = (model_.BIGC);
B4SOIcigc = (model_.CIGC);
B4SOIaigsd = (model_.AIGSD);
B4SOIbigsd = (model_.BIGSD);
B4SOIcigsd = (model_.CIGSD);
B4SOInigc = (model_.NIGC);
B4SOIpigcd = (model_.PIGCD);
B4SOIpoxedge = (model_.POXEDGE);
B4SOIdlcig = (model_.DLCIG);
B4SOIvbsa = (model_.VBSA);
B4SOInofffd = (model_.NOFFFD);
B4SOIvofffd = (model_.VOFFFD);
B4SOIk1b = (model_.K1B);
B4SOIk2b = (model_.K2B);
B4SOIdk2b = (model_.DK2B);
B4SOIdvbd0 = (model_.DVBD0);
B4SOIdvbd1 = (model_.DVBD1);
B4SOImoinFD = (model_.MOINFD);
B4SOIvbs0pd = (model_.VBS0PD);
B4SOIvbs0fd = (model_.VBS0FD);
B4SOIxrcrg1 = (model_.XRCRG1);
B4SOIxrcrg2 = (model_.XRCRG2);
B4SOIrshg = (model_.RSHG);
B4SOIngcon = (model_.NGCON);
B4SOIxgw = (model_.XGW);
B4SOIxgl = (model_.XGL);
B4SOIrdsMod = (model_.RDSMOD);
B4SOIfdMod = (model_.FDMOD);
B4SOIvsce = (model_.VSCE);
B4SOIcdsbs = (model_.CDSBS);
B4SOIminvcv = (model_.MINVCV);
B4SOIvoffcv = (model_.VOFFCV);
B4SOIEGGBCP2 = (model_.EGGBCP2);
B4SOIEGGDEP = (model_.EGGDEP);
B4SOIAGB1 = (model_.AGB1);
B4SOIBGB1 = (model_.BGB1);
B4SOIAGB2 = (model_.AGB2);
B4SOIBGB2 = (model_.BGB2);
B4SOIAGBC2N = (model_.AGBC2N);
B4SOIAGBC2P = (model_.AGBC2P);
B4SOIBGBC2N = (model_.BGBC2N);
B4SOIBGBC2P = (model_.BGBC2P);
B4SOIVTM00 = (model_.VTM00);
if (((model_.given("LPE0"))&&(model_.given("NLX"))))
{
UserInfo(*this)  << "Warning: parameter LPE0 overrides NLX." << " " <<  std::endl; 
}
if (((model_.given("LLPE0"))&&(model_.given("LNLX"))))
{
UserInfo(*this)  << "Warning: parameter LLPE0 overrides LNLX." << " " <<  std::endl; 
}
if (((model_.given("WLPE0"))&&(model_.given("WNLX"))))
{
UserInfo(*this)  << "Warning: parameter WLPE0 overrides WNLX." << " " <<  std::endl; 
}
if (((model_.given("PLPE0"))&&(model_.given("PNLX"))))
{
UserInfo(*this)  << "Warning: parameter PLPE0 overrides PNLX." << " " <<  std::endl; 
}
if (((model_.given("EGIDL"))&&(model_.given("NGIDL"))))
{
UserInfo(*this)  << "Warning: parameter EGIDL overrides NGIDL." << " " <<  std::endl; 
}
if (((model_.given("LEGIDL"))&&(model_.given("LNGIDL"))))
{
UserInfo(*this)  << "Warning: parameter LEGIDL overrides LNGIDL." << " " <<  std::endl; 
}
if (((model_.given("WEGIDL"))&&(model_.given("WNGIDL"))))
{
UserInfo(*this)  << "Warning: parameter WEGIDL overrides WNGIDL." << " " <<  std::endl; 
}
if (((model_.given("PEGIDL"))&&(model_.given("PNGIDL"))))
{
UserInfo(*this)  << "Warning: parameter PEGIDL overrides PNGIDL." << " " <<  std::endl; 
}
if (B4SOImtrlMod)
{
epsrox = 3.9;
toxe = B4SOIeot;
epssub = (8.85418e-12*B4SOIepsrsub);
{
double value_sqrt_0 = sqrt(((2.0e6*1.60219e-19)*epssub));
sqrt2qeps = value_sqrt_0;
}
B4SOIcox = ((epsrox*8.85418e-12)/toxe);
eggbcp2 = B4SOIEGGBCP2;
eggdep = B4SOIEGGDEP;
agb1 = B4SOIAGB1;
bgb1 = B4SOIBGB1;
agb2 = B4SOIAGB2;
bgb2 = B4SOIBGB2;
agbc2n = B4SOIAGBC2N;
agbc2p = B4SOIAGBC2P;
bgbc2n = B4SOIBGBC2N;
bgbc2p = B4SOIBGBC2P;
}
else
{
epsrox = B4SOIepsrox;
toxe = B4SOItox;
epssub = 1.03594e-10;
sqrt2qeps = 5.753e-12;
B4SOIcox = (3.453133e-11/B4SOItox);
eggbcp2 = B4SOIEGGBCP2;
eggdep = B4SOIEGGDEP;
agb1 = B4SOIAGB1;
bgb1 = B4SOIBGB1;
agb2 = B4SOIAGB2;
bgb2 = B4SOIBGB2;
agbc2n = B4SOIAGBC2N;
agbc2p = B4SOIAGBC2P;
bgbc2n = B4SOIBGBC2N;
bgbc2p = B4SOIBGBC2P;
}
if ((B4SOIsoiMod==2))
{
if ((SHMOD==0))
{
if ((TNODEOUT==0))
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(*this)  << "Error:  you have specified a P and B node (5th and 6th node) when soimod=2 and TNODEOUT=0.  The P and B nodes are not used in this mode and Xyce cannot deal with them being externally specified.  Please remove the 5th and 6th nodes from the instance line." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(*this)  << "Error:  you have specified a P node (5th node) when soimod=2 and TNODEOUT=0.  P node is not used in this mode and Xyce cannot deal with it as an externally specified node.  Please remove the 5th node." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(*this)  << "Error:  you have specified a P and B node (5th and 6th node) when soimod=2 and TNODEOUT=1, and a seventh node for temperature. The P and B nodes are not used in this mode and Xyce cannot deal with them being externally specified.  Please remove the P and B nodes. With TNODEOUT=1, the fifth node should be the last one and will be used for temperature." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
}
}
else
{
if ((TNODEOUT==0))
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(*this)  << "Error:  you have specified a P and B node (fifth and 6th nodes) when soimod=2, and a temperature node (7th) with TNODEOUT=0.  None of these nodes are used in this mode and Xyce cannot deal with them being externally specified.  Please remove the 6th node." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(*this)  << "Error:  you have specified P and B nodes (5th and 6th node) when soimod=2 and TNODEOUT=0.  These nodes are not used in this mode and Xyce cannot deal with them being externally specified.  Please remove the 5th and 6th nodes." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(*this)  << "Error:  you have specified P node (5th node) when soimod=2 and TNODEOUT=0.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 5th node." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(*this)  << "Error:  you have specified a B node (6th node) when soimod=2 and TNODEOUT=1, and a seventh node for temperature. The B node is not used in this mode and Xyce cannot deal with it as an externally specified node.  Please remove the B node. With TNODEOUT=1, the 6th node should be the last one and will be used for temperature." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
}
}
if ((model_.NODECHK))
{
if ((SHMOD==0))
{
if ((TNODEOUT==0))
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(*this)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(*this)  << "Warning: A 'P' node was specified, however 'SOIMOD = 2' does not support a 'P' node. Internally this node is tied to global ground." << " " <<  std::endl; 
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(*this)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(*this)  << "Warning: A 'P' node was specified, however 'SOIMOD = 2' does not support a 'P' node. Internally this node is tied to global ground." << " " <<  std::endl; 
}
}
}
}
else
{
if ((TNODEOUT==0))
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(*this)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(*this)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(*this)  << "Warning: A 'P' node was specified however 'SOIMOD = 2' does not support a 'P' node. Internally this node is tied to global ground." << " " <<  std::endl; 
}
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(*this)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(*this)  << "Warning: Fifth and sixth nodes were specified when TNODEOUT = 1. The fifth node will be treated as the temperature node 'T', and internally the sixth node will be tied to global ground." << " " <<  std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(*this)  << "Warning: A fifth node has been specified with TNODEOUT = 1. The fifth node will be treated as the temperature node 'T'." << " " <<  std::endl; 
}
else
{
if ((!((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t])))
{
UserInfo(*this)  << "Warning: TNODEOUT = 1 requesting an external temperature node, but a fifth node for temperature is not supplied." << " " <<  std::endl; 
}
}
}
}
}
}
}
B4SOIbodyMod = 0;
}
else
{
if ((portsConnected_[admsNodeID_p]==0))
{
B4SOIbodyMod = 0;
if (portsConnected_[admsNodeID_b])
{
UserInfo(*this)  << "Warning: cannot connect to 'b' terminal without connecting 'p' terminal." << " " <<  std::endl; 
}
}
else
{
if (((portsConnected_[admsNodeID_t]==0)&&(TNODEOUT==1)))
{
UserInfo(*this)  << "Xyce does not implement the BSIM-SOI 4 model with TNODEOUT and fewer than 7 nodes.  Ignore any subsequent messages regarding node collapse, they are meaningless." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
if ((portsConnected_[admsNodeID_b]==0))
{
if (((B4SOIrbody==0.0)&&(B4SOIrbsh==0.0)))
{
B4SOIbodyMod = 2;
}
else
{
B4SOIbodyMod = 1;
}
}
else
{
if (((B4SOIrbody==0.0)&&(B4SOIrbsh==0.0)))
{
UserInfo(*this)  << "Warning: model parameter rbody=0!" << " " <<  std::endl; 
B4SOIrbody = 1e0;
B4SOIbodyMod = 1;
}
else
{
B4SOIbodyMod = 1;
}
}
}
}
if ((model_.given("CF")))
{
B4SOIcf = (model_.CF);
}
else
{
{
double value_log_0 = log((1.0+(0.4e-6/B4SOItox)));
B4SOIcf = (((2.0*3.453133e-11)/3.14159265358979323846)*value_log_0);
}
}
if ((model_.given("CGDO")))
{
B4SOIcgdo = (model_.CGDO);
}
else
{
if (((model_.given("DLC"))&&(B4SOIdlc>0.0)))
{
B4SOIcgdo = ((B4SOIdlc*B4SOIcox)-B4SOIcgdl);
}
else
{
B4SOIcgdo = ((0.6*B4SOIxj)*B4SOIcox);
}
}
if ((model_.given("CGSO")))
{
B4SOIcgso = (model_.CGSO);
}
else
{
if (((model_.given("DLC"))&&(B4SOIdlc>0.0)))
{
B4SOIcgso = ((B4SOIdlc*B4SOIcox)-B4SOIcgsl);
}
else
{
B4SOIcgso = ((0.6*B4SOIxj)*B4SOIcox);
}
}
//Begin block b4soi_temp
{
//Block-local variables for block b4soi_temp
double tmp=0.0;
double tmp1=0.0;
double tmp2=0.0;
double Eg=0.0;
double Eg0=0.0;
double ni=0.0;
double T0=0.0;
double T1=0.0;
double T2=0.0;
double T3=0.0;
double T4=0.0;
double T5=0.0;
double T6=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Ldrn=0.0;
double Wdrn=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double TempRatio=0.0;
double Inv_L=0.0;
double Inv_W=0.0;
double Inv_LW=0.0;
double Vtm0=0.0;
double Tnom=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double SDphi=0.0;
double SDgamma=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Inv_saref=0.0;
double Inv_sbref=0.0;
double Inv_sa=0.0;
double Inv_sb=0.0;
double rho=0.0;
double dvth0_lod=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double W_tmp=0.0;
double Inv_ODeff=0.0;
double OD_offset=0.0;
double dk2_lod=0.0;
double deta0_lod=0.0;
double deta0cv_lod=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
int i=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double PowWeffWr=0.0;
double T10=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Vtm0eot=0.0;
double Vtmeot=0.0;
double vbieot=0.0;
double phieot=0.0;
double sqrtphieot=0.0;
double vddeot=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Vgs_eff=0.0;
double Vgsteff=0.0;
double V0=0.0;
double Vth=0.0;
double Vgst=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double lt1=0.0;
double ltw=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double TempRatioMinus1=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double n=0.0;
double VgstNVt=0.0;
double ExpArg=0.0;
double sqrtPhisExt=0.0;
double ExpVgst=0.0;
double vtfbphi2eot=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double DIBL_Sft=0.0;
double Lpe_Vb=0.0;
double DITS_Sft=0.0;
double DeltVthtemp=0.0;
double DITS_Sft2=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Theta0=0.0;
double Delt_vth=0.0;
double DeltVthw=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double niter=0.0;
double toxpf=0.0;
double toxpi=0.0;
double Tcen=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double n0=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double tmp3=0.0;
double T7=0.0;
//End of Block-local variables
if ((B4SOIGatesidewallJctSPotential<0.1))
{
B4SOIGatesidewallJctSPotential = 0.1;
}
if ((B4SOIGatesidewallJctDPotential<0.1))
{
B4SOIGatesidewallJctDPotential = 0.1;
}
Tnom = B4SOItnom;
TempRatio = (DevTemp/Tnom);
if (B4SOImtrlMod)
{
{
double value_sqrt_0 = sqrt(((epssub/(epsrox*8.85418e-12))*toxe));
B4SOIfactor1 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((1.03594e-10/3.453133e-11)*B4SOItox));
B4SOIfactor1 = value_sqrt_0;
}
}
Vtm00 = B4SOIVTM00;
if ((B4SOImtrlMod==0))
{
Vtm0 = (8.617087e-5*Tnom);
Eg0 = (1.16-(((7.02e-4*Tnom)*Tnom)/(Tnom+1108.0)));
B4SOIeg0 = Eg0;

d_B4SOIvtm_dTemp_t_GND = d_B4SOIvtm_dV_p_GND = d_B4SOIvtm_dV_b_GND =  0.0;
B4SOIvtm = (8.617087e-5*DevTemp);
Eg = (1.16-(((7.02e-4*DevTemp)*DevTemp)/(DevTemp+1108.0)));
B4SOIeg = Eg;
{
double value_sqrt_0 = sqrt((DevTemp/300.15));
double value_exp_1 = exp((21.5565981-(Eg/(2.0*B4SOIvtm))));
ni = (((1.45e10*(DevTemp/300.15))*value_sqrt_0)*value_exp_1);
}
}
else
{
Vtm0 = (8.617087e-5*Tnom);
Eg0 = (B4SOIbg0sub-(((B4SOItbgasub*Tnom)*Tnom)/(Tnom+B4SOItbgbsub)));
B4SOIeg0 = Eg0;

d_B4SOIvtm_dTemp_t_GND = d_B4SOIvtm_dV_p_GND = d_B4SOIvtm_dV_b_GND =  0.0;
B4SOIvtm = (8.617087e-5*DevTemp);
Eg = (B4SOIbg0sub-(((B4SOItbgasub*DevTemp)*DevTemp)/(DevTemp+B4SOItbgbsub)));
B4SOIeg = Eg;
{
double value_sqrt_0 = sqrt((DevTemp/Tnom));
double value_exp_1 = exp(((Eg0/(2.0*Vtm0))-(Eg/(2.0*B4SOIvtm))));
ni = (((B4SOIni0sub*(DevTemp/Tnom))*value_sqrt_0)*value_exp_1);
}
}
B4SOIrbodyext = (B4SOIbodySquares*B4SOIrbsh);
Ldrn = B4SOIl;
Wdrn = (B4SOIw/B4SOInf);
{
double value_pow_0 = pow(Ldrn,B4SOILln);
T0 = value_pow_0;
}
{
double value_pow_0 = pow(Wdrn,B4SOILwn);
T1 = value_pow_0;
}
tmp1 = (((B4SOILl/T0)+(B4SOILw/T1))+(B4SOILwl/(T0*T1)));
pParam_B4SOIdl = (B4SOILint+tmp1);
tmp1 = (((B4SOILlc/T0)+(B4SOILwc/T1))+(B4SOILwlc/(T0*T1)));
pParam_B4SOIdlc = (B4SOIdlc+tmp1);
pParam_B4SOIdlcig = (B4SOIdlcig+tmp1);
if ((pParam_B4SOIdlcig<0.0))
{
UserInfo(*this)  << "Warning: pParam_B4SOIdlcig = %g is negative. Set to zero." << " " << pParam_B4SOIdlcig << " " <<  std::endl; 
pParam_B4SOIdlcig = 0;
}
{
double value_pow_0 = pow(Ldrn,B4SOIWln);
T2 = value_pow_0;
}
{
double value_pow_0 = pow(Wdrn,B4SOIWwn);
T3 = value_pow_0;
}
tmp2 = (((B4SOIWl/T2)+(B4SOIWw/T3))+(B4SOIWwl/(T2*T3)));
pParam_B4SOIdw = (B4SOIWint+tmp2);
tmp2 = (((B4SOIWlc/T2)+(B4SOIWwc/T3))+(B4SOIWwlc/(T2*T3)));
pParam_B4SOIdwc = (B4SOIdwc+tmp2);
pParam_B4SOIleff = (B4SOIl-(2.0*pParam_B4SOIdl));
if ((pParam_B4SOIleff<=0.0))
{
UserInfo(*this)  << "Effective channel length <= 0" << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
pParam_B4SOIweff = (((B4SOIw/B4SOInf)-(B4SOInbc*B4SOIdwbc))-((2.0-B4SOInbc)*pParam_B4SOIdw));
if ((pParam_B4SOIweff<=0.0))
{
UserInfo(*this)  << "Effective channel width <= 0" << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
pParam_B4SOIwdiod = ((pParam_B4SOIweff/B4SOInseg)+B4SOIpdbcp);
pParam_B4SOIwdios = ((pParam_B4SOIweff/B4SOInseg)+B4SOIpsbcp);
pParam_B4SOIleffCV = (B4SOIl-(2.0*pParam_B4SOIdlc));
if ((pParam_B4SOIleffCV<=0.0))
{
UserInfo(*this)  << "Effective channel length for C-V <= 0" << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
pParam_B4SOIweffCV = (((B4SOIw/B4SOInf)-(B4SOInbc*B4SOIdwbc))-((2.0-B4SOInbc)*pParam_B4SOIdwc));
if ((pParam_B4SOIweffCV<=0.0))
{
UserInfo(*this)  << "Effective channel width for C-V <= 0" << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
pParam_B4SOIwdiodCV = ((pParam_B4SOIweffCV/B4SOInseg)+B4SOIpdbcp);
pParam_B4SOIwdiosCV = ((pParam_B4SOIweffCV/B4SOInseg)+B4SOIpsbcp);
pParam_B4SOIleffCVb = ((B4SOIl-(2.0*pParam_B4SOIdlc))-B4SOIdlcb);
if ((pParam_B4SOIleffCVb<=0.0))
{
UserInfo(*this)  << "Effective channel length for C-V (body) <= 0" << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
pParam_B4SOIleffCVbg = (pParam_B4SOIleffCVb+(2*B4SOIdlbg));
if ((pParam_B4SOIleffCVbg<=0.0))
{
UserInfo(*this)  << "Effective channel length for C-V (backgate) <= 0" << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
pParam_B4SOIgamma1 = B4SOIgamma1;
pParam_B4SOIgamma2 = B4SOIgamma2;

d_pParam_B4SOIvbx_dTemp_t_GND = d_pParam_B4SOIvbx_dV_p_GND = d_pParam_B4SOIvbx_dV_b_GND =  0.0;
pParam_B4SOIvbx = B4SOIvbx;
pParam_B4SOIvbm = B4SOIvbm;
pParam_B4SOIxt = B4SOIxt;
pParam_B4SOIcf = B4SOIcf;
pParam_B4SOIclc = B4SOIclc;
pParam_B4SOIcle = B4SOIcle;
{
double value_pow_0 = pow((pParam_B4SOIclc/pParam_B4SOIleff),pParam_B4SOIcle);
pParam_B4SOIabulkCVfactor = (1.0+value_pow_0);
}
if ((B4SOIbinUnit==1))
{
Inv_L = (1.0e-6/pParam_B4SOIleff);
Inv_W = (1.0e-6/pParam_B4SOIweff);
Inv_LW = (1.0e-12/(pParam_B4SOIleff*pParam_B4SOIweff));
}
else
{
Inv_L = (1.0/pParam_B4SOIleff);
Inv_W = (1.0/pParam_B4SOIweff);
Inv_LW = (1.0/(pParam_B4SOIleff*pParam_B4SOIweff));
}
pParam_B4SOInpeak = (((B4SOInpeak+((model_.LNCH)*Inv_L))+((model_.WNCH)*Inv_W))+((model_.PNCH)*Inv_LW));
pParam_B4SOInsub = (((B4SOInsub+((model_.LNSUB)*Inv_L))+((model_.WNSUB)*Inv_W))+((model_.PNSUB)*Inv_LW));
pParam_B4SOIngate = (((B4SOIngate+((model_.LNGATE)*Inv_L))+((model_.WNGATE)*Inv_W))+((model_.PNGATE)*Inv_LW));
pParam_B4SOInsd = (((B4SOInsd+((model_.LNSD)*Inv_L))+((model_.WNSD)*Inv_W))+((model_.PNSD)*Inv_LW));
pParam_B4SOIvth0 = (((B4SOIvth0+((model_.LVTH0)*Inv_L))+((model_.WVTH0)*Inv_W))+((model_.PVTH0)*Inv_LW));
pParam_B4SOIvfb = (((B4SOIvfb+((model_.LVFB)*Inv_L))+((model_.WVFB)*Inv_W))+((model_.PVFB)*Inv_LW));

d_pParam_B4SOIk1_dTemp_t_GND = d_pParam_B4SOIk1_dV_p_GND = d_pParam_B4SOIk1_dV_b_GND =  0.0;
pParam_B4SOIk1 = (((B4SOIk1+((model_.LK1)*Inv_L))+((model_.WK1)*Inv_W))+((model_.PK1)*Inv_LW));
pParam_B4SOIk2 = (((B4SOIk2+((model_.LK2)*Inv_L))+((model_.WK2)*Inv_W))+((model_.PK2)*Inv_LW));
pParam_B4SOIk1w1 = (((B4SOIk1w1+((model_.LK1W1)*Inv_L))+((model_.WK1W1)*Inv_W))+((model_.PK1W1)*Inv_LW));
pParam_B4SOIk1w2 = (((B4SOIk1w2+((model_.LK1W2)*Inv_L))+((model_.WK1W2)*Inv_W))+((model_.PK1W2)*Inv_LW));
pParam_B4SOIk3 = (((B4SOIk3+((model_.LK3)*Inv_L))+((model_.WK3)*Inv_W))+((model_.PK3)*Inv_LW));
pParam_B4SOIk3b = (((B4SOIk3b+((model_.LK3B)*Inv_L))+((model_.WK3B)*Inv_W))+((model_.PK3B)*Inv_LW));
pParam_B4SOIkb1 = (((B4SOIkb1+((model_.LKB1)*Inv_L))+((model_.WKB1)*Inv_W))+((model_.PKB1)*Inv_LW));
pParam_B4SOIw0 = (((B4SOIw0+((model_.LW0)*Inv_L))+((model_.WW0)*Inv_W))+((model_.PW0)*Inv_LW));
pParam_B4SOIlpe0 = (((B4SOIlpe0+((model_.LLPE0)*Inv_L))+((model_.WLPE0)*Inv_W))+((model_.PLPE0)*Inv_LW));
pParam_B4SOIlpeb = (((B4SOIlpeb+((model_.LLPEB)*Inv_L))+((model_.WLPEB)*Inv_W))+((model_.PLPEB)*Inv_LW));
pParam_B4SOIdvt0 = (((B4SOIdvt0+((model_.LDVT0)*Inv_L))+((model_.WDVT0)*Inv_W))+((model_.PDVT0)*Inv_LW));
pParam_B4SOIdvt1 = (((B4SOIdvt1+((model_.LDVT1)*Inv_L))+((model_.WDVT1)*Inv_W))+((model_.PDVT1)*Inv_LW));
pParam_B4SOIdvt2 = (((B4SOIdvt2+((model_.LDVT2)*Inv_L))+((model_.WDVT2)*Inv_W))+((model_.PDVT2)*Inv_LW));
pParam_B4SOIdvt0w = (((B4SOIdvt0w+((model_.LDVT0W)*Inv_L))+((model_.WDVT0W)*Inv_W))+((model_.PDVT0W)*Inv_LW));
pParam_B4SOIdvt1w = (((B4SOIdvt1w+((model_.LDVT1W)*Inv_L))+((model_.WDVT1W)*Inv_W))+((model_.PDVT1W)*Inv_LW));
pParam_B4SOIdvt2w = (((B4SOIdvt2w+((model_.LDVT2W)*Inv_L))+((model_.WDVT2W)*Inv_W))+((model_.PDVT2W)*Inv_LW));
pParam_B4SOIu0 = (((B4SOIu0+((model_.LU0)*Inv_L))+((model_.WU0)*Inv_W))+((model_.PU0)*Inv_LW));
pParam_B4SOIua = (((B4SOIua+((model_.LUA)*Inv_L))+((model_.WUA)*Inv_W))+((model_.PUA)*Inv_LW));
pParam_B4SOIub = (((B4SOIub+((model_.LUB)*Inv_L))+((model_.WUB)*Inv_W))+((model_.PUB)*Inv_LW));
pParam_B4SOIuc = (((B4SOIuc+((model_.LUC)*Inv_L))+((model_.WUC)*Inv_W))+((model_.PUC)*Inv_LW));
pParam_B4SOIvsat = (((B4SOIvsat+((model_.LVSAT)*Inv_L))+((model_.WVSAT)*Inv_W))+((model_.PVSAT)*Inv_LW));
pParam_B4SOIa0 = (((B4SOIa0+((model_.LA0)*Inv_L))+((model_.WA0)*Inv_W))+((model_.PA0)*Inv_LW));
pParam_B4SOIags = (((B4SOIags+((model_.LAGS)*Inv_L))+((model_.WAGS)*Inv_W))+((model_.PAGS)*Inv_LW));
pParam_B4SOIb0 = (((B4SOIb0+((model_.LB0)*Inv_L))+((model_.WB0)*Inv_W))+((model_.PB0)*Inv_LW));
pParam_B4SOIb1 = (((B4SOIb1+((model_.LB1)*Inv_L))+((model_.WB1)*Inv_W))+((model_.PB1)*Inv_LW));
pParam_B4SOIketa = (((B4SOIketa+((model_.LKETA)*Inv_L))+((model_.WKETA)*Inv_W))+((model_.PKETA)*Inv_LW));
pParam_B4SOIketas = (((B4SOIketas+((model_.LKETAS)*Inv_L))+((model_.WKETAS)*Inv_W))+((model_.PKETAS)*Inv_LW));
pParam_B4SOIa1 = (((B4SOIa1+((model_.LA1)*Inv_L))+((model_.WA1)*Inv_W))+((model_.PA1)*Inv_LW));
pParam_B4SOIa2 = (((B4SOIa2+((model_.LA2)*Inv_L))+((model_.WA2)*Inv_W))+((model_.PA2)*Inv_LW));
pParam_B4SOIrdsw = (((B4SOIrdsw+((model_.LRDSW)*Inv_L))+((model_.WRDSW)*Inv_W))+((model_.PRDSW)*Inv_LW));
pParam_B4SOIrsw = (((B4SOIrsw+((model_.LRSW)*Inv_L))+((model_.WRSW)*Inv_W))+((model_.PRSW)*Inv_LW));
pParam_B4SOIrdw = (((B4SOIrdw+((model_.LRDW)*Inv_L))+((model_.WRDW)*Inv_W))+((model_.PRDW)*Inv_LW));
pParam_B4SOIprwb = (((B4SOIprwb+((model_.LPRWB)*Inv_L))+((model_.WPRWB)*Inv_W))+((model_.PPRWB)*Inv_LW));
pParam_B4SOIprwg = (((B4SOIprwg+((model_.LPRWG)*Inv_L))+((model_.WPRWG)*Inv_W))+((model_.PPRWG)*Inv_LW));
pParam_B4SOIwr = (((B4SOIwr+((model_.LWR)*Inv_L))+((model_.WWR)*Inv_W))+((model_.PWR)*Inv_LW));
pParam_B4SOInfactor = (((B4SOInfactor+((model_.LNFACTOR)*Inv_L))+((model_.WNFACTOR)*Inv_W))+((model_.PNFACTOR)*Inv_LW));
pParam_B4SOIdwg = (((B4SOIdwg+((model_.LDWG)*Inv_L))+((model_.WDWG)*Inv_W))+((model_.PDWG)*Inv_LW));
pParam_B4SOIdwb = (((B4SOIdwb+((model_.LDWB)*Inv_L))+((model_.WDWB)*Inv_W))+((model_.PDWB)*Inv_LW));
pParam_B4SOIvoff = (((B4SOIvoff+((model_.LVOFF)*Inv_L))+((model_.WVOFF)*Inv_W))+((model_.PVOFF)*Inv_LW));
pParam_B4SOIeta0 = (((B4SOIeta0+((model_.LETA0)*Inv_L))+((model_.WETA0)*Inv_W))+((model_.PETA0)*Inv_LW));
pParam_B4SOIetab = (((B4SOIetab+((model_.LETAB)*Inv_L))+((model_.WETAB)*Inv_W))+((model_.PETAB)*Inv_LW));
pParam_B4SOIeta0cv = (((B4SOIeta0cv+((model_.LETA0CV)*Inv_L))+((model_.WETA0CV)*Inv_W))+((model_.PETA0CV)*Inv_LW));
pParam_B4SOIetabcv = (((B4SOIetabcv+((model_.LETABCV)*Inv_L))+((model_.WETABCV)*Inv_W))+((model_.PETABCV)*Inv_LW));
pParam_B4SOIdsub = (((B4SOIdsub+((model_.LDSUB)*Inv_L))+((model_.WDSUB)*Inv_W))+((model_.PDSUB)*Inv_LW));
pParam_B4SOIcit = (((B4SOIcit+((model_.LCIT)*Inv_L))+((model_.WCIT)*Inv_W))+((model_.PCIT)*Inv_LW));
pParam_B4SOIcdsc = (((B4SOIcdsc+((model_.LCDSC)*Inv_L))+((model_.WCDSC)*Inv_W))+((model_.PCDSC)*Inv_LW));
pParam_B4SOIcdscb = (((B4SOIcdscb+((model_.LCDSCB)*Inv_L))+((model_.WCDSCB)*Inv_W))+((model_.PCDSCB)*Inv_LW));
pParam_B4SOIcdscd = (((B4SOIcdscd+((model_.LCDSCD)*Inv_L))+((model_.WCDSCD)*Inv_W))+((model_.PCDSCD)*Inv_LW));
pParam_B4SOIpclm = (((B4SOIpclm+((model_.LPCLM)*Inv_L))+((model_.WPCLM)*Inv_W))+((model_.PPCLM)*Inv_LW));
pParam_B4SOIpdibl1 = (((B4SOIpdibl1+((model_.LPDIBLC1)*Inv_L))+((model_.WPDIBLC1)*Inv_W))+((model_.PPDIBLC1)*Inv_LW));
pParam_B4SOIpdibl2 = (((B4SOIpdibl2+((model_.LPDIBLC2)*Inv_L))+((model_.WPDIBLC2)*Inv_W))+((model_.PPDIBLC2)*Inv_LW));
pParam_B4SOIpdiblb = (((B4SOIpdiblb+((model_.LPDIBLCB)*Inv_L))+((model_.WPDIBLCB)*Inv_W))+((model_.PPDIBLCB)*Inv_LW));
pParam_B4SOIdrout = (((B4SOIdrout+((model_.LDROUT)*Inv_L))+((model_.WDROUT)*Inv_W))+((model_.PDROUT)*Inv_LW));
pParam_B4SOIpvag = (((B4SOIpvag+((model_.LPVAG)*Inv_L))+((model_.WPVAG)*Inv_W))+((model_.PPVAG)*Inv_LW));
pParam_B4SOIdelta = (((B4SOIdelta+((model_.LDELTA)*Inv_L))+((model_.WDELTA)*Inv_W))+((model_.PDELTA)*Inv_LW));
pParam_B4SOIalpha0 = (((B4SOIalpha0+((model_.LALPHA0)*Inv_L))+((model_.WALPHA0)*Inv_W))+((model_.PALPHA0)*Inv_LW));
pParam_B4SOIfbjtii = (((B4SOIfbjtii+((model_.LFBJTII)*Inv_L))+((model_.WFBJTII)*Inv_W))+((model_.PFBJTII)*Inv_LW));
pParam_B4SOIebjtii = (((B4SOIebjtii+((model_.LEBJTII)*Inv_L))+((model_.WEBJTII)*Inv_W))+((model_.PEBJTII)*Inv_LW));
pParam_B4SOIcbjtii = (((B4SOIcbjtii+((model_.LCBJTII)*Inv_L))+((model_.WCBJTII)*Inv_W))+((model_.PCBJTII)*Inv_LW));
pParam_B4SOIvbci = (((B4SOIvbci+((model_.LVBCI)*Inv_L))+((model_.WVBCI)*Inv_W))+((model_.PVBCI)*Inv_LW));
pParam_B4SOIabjtii = (((B4SOIabjtii+((model_.LABJTII)*Inv_L))+((model_.WABJTII)*Inv_W))+((model_.PABJTII)*Inv_LW));
pParam_B4SOImbjtii = (((B4SOImbjtii+((model_.LMBJTII)*Inv_L))+((model_.WMBJTII)*Inv_W))+((model_.PMBJTII)*Inv_LW));
pParam_B4SOIbeta0 = (((B4SOIbeta0+((model_.LBETA0)*Inv_L))+((model_.WBETA0)*Inv_W))+((model_.PBETA0)*Inv_LW));
pParam_B4SOIbeta1 = (((B4SOIbeta1+((model_.LBETA1)*Inv_L))+((model_.WBETA1)*Inv_W))+((model_.PBETA1)*Inv_LW));
pParam_B4SOIbeta2 = (((B4SOIbeta2+((model_.LBETA2)*Inv_L))+((model_.WBETA2)*Inv_W))+((model_.PBETA2)*Inv_LW));
pParam_B4SOIvdsatii0 = (((B4SOIvdsatii0+((model_.LVDSATII0)*Inv_L))+((model_.WVDSATII0)*Inv_W))+((model_.PVDSATII0)*Inv_LW));
pParam_B4SOIlii = (((B4SOIlii+((model_.LLII)*Inv_L))+((model_.WLII)*Inv_W))+((model_.PLII)*Inv_LW));
pParam_B4SOIesatii = (((B4SOIesatii+((model_.LESATII)*Inv_L))+((model_.WESATII)*Inv_W))+((model_.PESATII)*Inv_LW));
pParam_B4SOIsii0 = (((B4SOIsii0+((model_.LSII0)*Inv_L))+((model_.WSII0)*Inv_W))+((model_.PSII0)*Inv_LW));
pParam_B4SOIsii1 = (((B4SOIsii1+((model_.LSII1)*Inv_L))+((model_.WSII1)*Inv_W))+((model_.PSII1)*Inv_LW));
pParam_B4SOIsii2 = (((B4SOIsii2+((model_.LSII2)*Inv_L))+((model_.WSII2)*Inv_W))+((model_.PSII2)*Inv_LW));
pParam_B4SOIsiid = (((B4SOIsiid+((model_.LSIID)*Inv_L))+((model_.WSIID)*Inv_W))+((model_.PSIID)*Inv_LW));
pParam_B4SOIagidl = (((B4SOIagidl+((model_.LAGIDL)*Inv_L))+((model_.WAGIDL)*Inv_W))+((model_.PAGIDL)*Inv_LW));
pParam_B4SOIbgidl = (((B4SOIbgidl+((model_.LBGIDL)*Inv_L))+((model_.WBGIDL)*Inv_W))+((model_.PBGIDL)*Inv_LW));
pParam_B4SOIcgidl = (((B4SOIcgidl+((model_.LCGIDL)*Inv_L))+((model_.WCGIDL)*Inv_W))+((model_.PCGIDL)*Inv_LW));
pParam_B4SOIegidl = (((B4SOIegidl+((model_.LEGIDL)*Inv_L))+((model_.WEGIDL)*Inv_W))+((model_.PEGIDL)*Inv_LW));
pParam_B4SOIrgidl = (((B4SOIrgidl+((model_.LRGIDL)*Inv_L))+((model_.WRGIDL)*Inv_W))+((model_.PRGIDL)*Inv_LW));
pParam_B4SOIkgidl = (((B4SOIkgidl+((model_.LKGIDL)*Inv_L))+((model_.WKGIDL)*Inv_W))+((model_.PKGIDL)*Inv_LW));
pParam_B4SOIfgidl = (((B4SOIfgidl+((model_.LFGIDL)*Inv_L))+((model_.WFGIDL)*Inv_W))+((model_.PFGIDL)*Inv_LW));
pParam_B4SOIagisl = (((B4SOIagisl+((model_.LAGISL)*Inv_L))+((model_.WAGISL)*Inv_W))+((model_.PAGISL)*Inv_LW));
pParam_B4SOIbgisl = (((B4SOIbgisl+((model_.LBGISL)*Inv_L))+((model_.WBGISL)*Inv_W))+((model_.PBGISL)*Inv_LW));
pParam_B4SOIcgisl = (((B4SOIcgisl+((model_.LCGISL)*Inv_L))+((model_.WCGISL)*Inv_W))+((model_.PCGISL)*Inv_LW));
pParam_B4SOIegisl = (((B4SOIegisl+((model_.LEGISL)*Inv_L))+((model_.WEGISL)*Inv_W))+((model_.PEGISL)*Inv_LW));
pParam_B4SOIrgisl = (((B4SOIrgisl+((model_.LRGISL)*Inv_L))+((model_.WRGISL)*Inv_W))+((model_.PRGISL)*Inv_LW));
pParam_B4SOIkgisl = (((B4SOIkgisl+((model_.LKGISL)*Inv_L))+((model_.WKGISL)*Inv_W))+((model_.PKGISL)*Inv_LW));
pParam_B4SOIfgisl = (((B4SOIfgisl+((model_.LFGISL)*Inv_L))+((model_.WFGISL)*Inv_W))+((model_.PFGISL)*Inv_LW));
pParam_B4SOIntun = (((B4SOIntun+((model_.LNTUN)*Inv_L))+((model_.WNTUN)*Inv_W))+((model_.PNTUN)*Inv_LW));
pParam_B4SOIntund = (((B4SOIntund+((model_.LNTUND)*Inv_L))+((model_.WNTUND)*Inv_W))+((model_.PNTUND)*Inv_LW));
pParam_B4SOIndiode = (((B4SOIndiode+((model_.LNDIODE)*Inv_L))+((model_.WNDIODE)*Inv_W))+((model_.PNDIODE)*Inv_LW));
pParam_B4SOIndioded = (((B4SOIndioded+((model_.LNDIODED)*Inv_L))+((model_.WNDIODED)*Inv_W))+((model_.PNDIODED)*Inv_LW));
pParam_B4SOInrecf0 = (((B4SOInrecf0+((model_.LNRECF0)*Inv_L))+((model_.WNRECF0)*Inv_W))+((model_.PNRECF0)*Inv_LW));
pParam_B4SOInrecf0d = (((B4SOInrecf0d+((model_.LNRECF0D)*Inv_L))+((model_.WNRECF0D)*Inv_W))+((model_.PNRECF0D)*Inv_LW));
pParam_B4SOInrecr0 = (((B4SOInrecr0+((model_.LNRECR0)*Inv_L))+((model_.WNRECR0)*Inv_W))+((model_.PNRECR0)*Inv_LW));
pParam_B4SOInrecr0d = (((B4SOInrecr0d+((model_.LNRECR0D)*Inv_L))+((model_.WNRECR0D)*Inv_W))+((model_.PNRECR0D)*Inv_LW));
pParam_B4SOIisbjt = (((B4SOIisbjt+((model_.LISBJT)*Inv_L))+((model_.WISBJT)*Inv_W))+((model_.PISBJT)*Inv_LW));
pParam_B4SOIidbjt = (((B4SOIidbjt+((model_.LIDBJT)*Inv_L))+((model_.WIDBJT)*Inv_W))+((model_.PIDBJT)*Inv_LW));
pParam_B4SOIisdif = (((B4SOIisdif+((model_.LISDIF)*Inv_L))+((model_.WISDIF)*Inv_W))+((model_.PISDIF)*Inv_LW));
pParam_B4SOIiddif = (((B4SOIiddif+((model_.LIDDIF)*Inv_L))+((model_.WIDDIF)*Inv_W))+((model_.PIDDIF)*Inv_LW));
pParam_B4SOIisrec = (((B4SOIisrec+((model_.LISREC)*Inv_L))+((model_.WISREC)*Inv_W))+((model_.PISREC)*Inv_LW));
pParam_B4SOIistun = (((B4SOIistun+((model_.LISTUN)*Inv_L))+((model_.WISTUN)*Inv_W))+((model_.PISTUN)*Inv_LW));
pParam_B4SOIidrec = (((B4SOIidrec+((model_.LIDREC)*Inv_L))+((model_.WIDREC)*Inv_W))+((model_.PIDREC)*Inv_LW));
pParam_B4SOIidtun = (((B4SOIidtun+((model_.LIDTUN)*Inv_L))+((model_.WIDTUN)*Inv_W))+((model_.PIDTUN)*Inv_LW));
pParam_B4SOIvrec0 = (((B4SOIvrec0+((model_.LVREC0)*Inv_L))+((model_.WVREC0)*Inv_W))+((model_.PVREC0)*Inv_LW));
pParam_B4SOIvrec0d = (((B4SOIvrec0d+((model_.LVREC0D)*Inv_L))+((model_.WVREC0D)*Inv_W))+((model_.PVREC0D)*Inv_LW));
pParam_B4SOIvtun0 = (((B4SOIvtun0+((model_.LVTUN0)*Inv_L))+((model_.WVTUN0)*Inv_W))+((model_.PVTUN0)*Inv_LW));
pParam_B4SOIvtun0d = (((B4SOIvtun0d+((model_.LVTUN0D)*Inv_L))+((model_.WVTUN0D)*Inv_W))+((model_.PVTUN0D)*Inv_LW));
pParam_B4SOInbjt = (((B4SOInbjt+((model_.LNBJT)*Inv_L))+((model_.WNBJT)*Inv_W))+((model_.PNBJT)*Inv_LW));
pParam_B4SOIlbjt0 = (((B4SOIlbjt0+((model_.LLBJT0)*Inv_L))+((model_.WLBJT0)*Inv_W))+((model_.PLBJT0)*Inv_LW));
pParam_B4SOIvabjt = (((B4SOIvabjt+((model_.LVABJT)*Inv_L))+((model_.WVABJT)*Inv_W))+((model_.PVABJT)*Inv_LW));
pParam_B4SOIaely = (((B4SOIaely+((model_.LAELY)*Inv_L))+((model_.WAELY)*Inv_W))+((model_.PAELY)*Inv_LW));
pParam_B4SOIahli = (((B4SOIahli+((model_.LAHLI)*Inv_L))+((model_.WAHLI)*Inv_W))+((model_.PAHLI)*Inv_LW));
pParam_B4SOIahlid = (((B4SOIahlid+((model_.LAHLID)*Inv_L))+((model_.WAHLID)*Inv_W))+((model_.PAHLID)*Inv_LW));
pParam_B4SOIxj = (((B4SOIxj+((model_.LXJ)*Inv_L))+((model_.WXJ)*Inv_W))+((model_.PXJ)*Inv_LW));
pParam_B4SOIalphaGB1 = (((B4SOIalphaGB1+((model_.LALPHAGB1)*Inv_L))+((model_.WALPHAGB1)*Inv_W))+((model_.PALPHAGB1)*Inv_LW));
pParam_B4SOIalphaGB2 = (((B4SOIalphaGB2+((model_.LALPHAGB2)*Inv_L))+((model_.WALPHAGB2)*Inv_W))+((model_.PALPHAGB2)*Inv_LW));
pParam_B4SOIbetaGB1 = (((B4SOIbetaGB1+((model_.LBETAGB1)*Inv_L))+((model_.WBETAGB1)*Inv_W))+((model_.PBETAGB1)*Inv_LW));
pParam_B4SOIbetaGB2 = (((B4SOIbetaGB2+((model_.LBETAGB2)*Inv_L))+((model_.WBETAGB2)*Inv_W))+((model_.PBETAGB2)*Inv_LW));
pParam_B4SOIndif = (((B4SOIndif+((model_.LNDIF)*Inv_L))+((model_.WNDIF)*Inv_W))+((model_.PNDIF)*Inv_LW));
pParam_B4SOIntrecf = (((B4SOIntrecf+((model_.LNTRECF)*Inv_L))+((model_.WNTRECF)*Inv_W))+((model_.PNTRECF)*Inv_LW));
pParam_B4SOIntrecr = (((B4SOIntrecr+((model_.LNTRECR)*Inv_L))+((model_.WNTRECR)*Inv_W))+((model_.PNTRECR)*Inv_LW));
pParam_B4SOIxbjt = (((B4SOIxbjt+((model_.LXBJT)*Inv_L))+((model_.WXBJT)*Inv_W))+((model_.PXBJT)*Inv_LW));
pParam_B4SOIxdif = (((B4SOIxdif+((model_.LXDIF)*Inv_L))+((model_.WXDIF)*Inv_W))+((model_.PXDIF)*Inv_LW));
pParam_B4SOIxrec = (((B4SOIxrec+((model_.LXREC)*Inv_L))+((model_.WXREC)*Inv_W))+((model_.PXREC)*Inv_LW));
pParam_B4SOIxtun = (((B4SOIxtun+((model_.LXTUN)*Inv_L))+((model_.WXTUN)*Inv_W))+((model_.PXTUN)*Inv_LW));
pParam_B4SOIxdifd = (((B4SOIxdifd+((model_.LXDIFD)*Inv_L))+((model_.WXDIFD)*Inv_W))+((model_.PXDIFD)*Inv_LW));
pParam_B4SOIxrecd = (((B4SOIxrecd+((model_.LXRECD)*Inv_L))+((model_.WXRECD)*Inv_W))+((model_.PXRECD)*Inv_LW));
pParam_B4SOIxtund = (((B4SOIxtund+((model_.LXTUND)*Inv_L))+((model_.WXTUND)*Inv_W))+((model_.PXTUND)*Inv_LW));
pParam_B4SOIcgdl = (((B4SOIcgdl+((model_.LCGDL)*Inv_L))+((model_.WCGDL)*Inv_W))+((model_.PCGDL)*Inv_LW));
pParam_B4SOIcgsl = (((B4SOIcgsl+((model_.LCGSL)*Inv_L))+((model_.WCGSL)*Inv_W))+((model_.PCGSL)*Inv_LW));
pParam_B4SOIckappa = (((B4SOIckappa+((model_.LCKAPPA)*Inv_L))+((model_.WCKAPPA)*Inv_W))+((model_.PCKAPPA)*Inv_LW));
pParam_B4SOIute = (((B4SOIute+((model_.LUTE)*Inv_L))+((model_.WUTE)*Inv_W))+((model_.PUTE)*Inv_LW));
pParam_B4SOIud = (((B4SOIud+((model_.LUD)*Inv_L))+((model_.WUD)*Inv_W))+((model_.PUD)*Inv_LW));
pParam_B4SOIud1 = (((B4SOIud1+((model_.LUD1)*Inv_L))+((model_.WUD1)*Inv_W))+((model_.PUD1)*Inv_LW));
pParam_B4SOIeu = (((B4SOIeu+((model_.LEU)*Inv_L))+((model_.WEU)*Inv_W))+((model_.PEU)*Inv_LW));
pParam_B4SOIucs = (((B4SOIucs+((model_.LUCS)*Inv_L))+((model_.WUCS)*Inv_W))+((model_.PUCS)*Inv_LW));
pParam_B4SOIucste = (((B4SOIucste+((model_.LUCSTE)*Inv_L))+((model_.WUCSTE)*Inv_W))+((model_.PUCSTE)*Inv_LW));
pParam_B4SOIkt1 = (((B4SOIkt1+((model_.LKT1)*Inv_L))+((model_.WKT1)*Inv_W))+((model_.PKT1)*Inv_LW));
pParam_B4SOIkt2 = (((B4SOIkt2+((model_.LKT2)*Inv_L))+((model_.WKT2)*Inv_W))+((model_.PKT2)*Inv_LW));
pParam_B4SOIkt1l = (((B4SOIkt1l+((model_.LKT1L)*Inv_L))+((model_.WKT1L)*Inv_W))+((model_.PKT1L)*Inv_LW));
pParam_B4SOIua1 = (((B4SOIua1+((model_.LUA1)*Inv_L))+((model_.WUA1)*Inv_W))+((model_.PUA1)*Inv_LW));
pParam_B4SOIub1 = (((B4SOIub1+((model_.LUB1)*Inv_L))+((model_.WUB1)*Inv_W))+((model_.PUB1)*Inv_LW));
pParam_B4SOIuc1 = (((B4SOIuc1+((model_.LUC1)*Inv_L))+((model_.WUC1)*Inv_W))+((model_.PUC1)*Inv_LW));
pParam_B4SOIat = (((B4SOIat+((model_.LAT)*Inv_L))+((model_.WAT)*Inv_W))+((model_.PAT)*Inv_LW));
pParam_B4SOIprt = (((B4SOIprt+((model_.LPRT)*Inv_L))+((model_.WPRT)*Inv_W))+((model_.PPRT)*Inv_LW));
pParam_B4SOInigc = (((B4SOInigc+((model_.LNIGC)*Inv_L))+((model_.WNIGC)*Inv_W))+((model_.PNIGC)*Inv_LW));
pParam_B4SOIaigc = (((B4SOIaigc+((model_.LAIGC)*Inv_L))+((model_.WAIGC)*Inv_W))+((model_.PAIGC)*Inv_LW));
pParam_B4SOIbigc = (((B4SOIbigc+((model_.LBIGC)*Inv_L))+((model_.WBIGC)*Inv_W))+((model_.PBIGC)*Inv_LW));
pParam_B4SOIcigc = (((B4SOIcigc+((model_.LCIGC)*Inv_L))+((model_.WCIGC)*Inv_W))+((model_.PCIGC)*Inv_LW));
pParam_B4SOIaigsd = (((B4SOIaigsd+((model_.LAIGSD)*Inv_L))+((model_.WAIGSD)*Inv_W))+((model_.PAIGSD)*Inv_LW));
pParam_B4SOIbigsd = (((B4SOIbigsd+((model_.LBIGSD)*Inv_L))+((model_.WBIGSD)*Inv_W))+((model_.PBIGSD)*Inv_LW));
pParam_B4SOIcigsd = (((B4SOIcigsd+((model_.LCIGSD)*Inv_L))+((model_.WCIGSD)*Inv_W))+((model_.PCIGSD)*Inv_LW));
pParam_B4SOIpigcd = (((B4SOIpigcd+((model_.LPIGCD)*Inv_L))+((model_.WPIGCD)*Inv_W))+((model_.PPIGCD)*Inv_LW));
pParam_B4SOIpoxedge = (((B4SOIpoxedge+((model_.LPOXEDGE)*Inv_L))+((model_.WPOXEDGE)*Inv_W))+((model_.PPOXEDGE)*Inv_LW));
pParam_B4SOIxrcrg1 = (((B4SOIxrcrg1+((model_.LXRCRG1)*Inv_L))+((model_.WXRCRG1)*Inv_W))+((model_.PXRCRG1)*Inv_LW));
pParam_B4SOIxrcrg2 = (((B4SOIxrcrg2+((model_.LXRCRG2)*Inv_L))+((model_.WXRCRG2)*Inv_W))+((model_.PXRCRG2)*Inv_LW));
pParam_B4SOIvbsa = (((B4SOIvbsa+((model_.LVBSA)*Inv_L))+((model_.WVBSA)*Inv_W))+((model_.PVBSA)*Inv_LW));
pParam_B4SOIvsce = (((B4SOIvsce+((model_.LVSCE)*Inv_L))+((model_.WVSCE)*Inv_W))+((model_.PVSCE)*Inv_LW));
pParam_B4SOIcdsbs = (((B4SOIcdsbs+((model_.LCDSBS)*Inv_L))+((model_.WCDSBS)*Inv_W))+((model_.PCDSBS)*Inv_LW));
pParam_B4SOInofffd = (((B4SOInofffd+((model_.LNOFFFD)*Inv_L))+((model_.WNOFFFD)*Inv_W))+((model_.PNOFFFD)*Inv_LW));
pParam_B4SOIvofffd = (((B4SOIvofffd+((model_.LVOFFFD)*Inv_L))+((model_.WVOFFFD)*Inv_W))+((model_.PVOFFFD)*Inv_LW));
pParam_B4SOIk1b = (((B4SOIk1b+((model_.LK1B)*Inv_L))+((model_.WK1B)*Inv_W))+((model_.PK1B)*Inv_LW));
pParam_B4SOIk2b = (((B4SOIk2b+((model_.LK2B)*Inv_L))+((model_.WK2B)*Inv_W))+((model_.PK2B)*Inv_LW));
pParam_B4SOIdk2b = (((B4SOIdk2b+((model_.LDK2B)*Inv_L))+((model_.WDK2B)*Inv_W))+((model_.PDK2B)*Inv_LW));
pParam_B4SOIdvbd0 = (((B4SOIdvbd0+((model_.LDVBD0)*Inv_L))+((model_.WDVBD0)*Inv_W))+((model_.PDVBD0)*Inv_LW));
pParam_B4SOIdvbd1 = (((B4SOIdvbd1+((model_.LDVBD1)*Inv_L))+((model_.WDVBD1)*Inv_W))+((model_.PDVBD1)*Inv_LW));
pParam_B4SOImoinfd = (((B4SOImoinFD+((model_.LMOINFD)*Inv_L))+((model_.WMOINFD)*Inv_W))+((model_.PMOINFD)*Inv_LW));
pParam_B4SOIvbs0pd = (((B4SOIvbs0pd+((model_.LVBS0PD)*Inv_L))+((model_.WVBS0PD)*Inv_W))+((model_.PVBS0PD)*Inv_LW));
pParam_B4SOIvbs0fd = (((B4SOIvbs0fd+((model_.LVBS0FD)*Inv_L))+((model_.WVBS0FD)*Inv_W))+((model_.PVBS0FD)*Inv_LW));
pParam_B4SOIvsdfb = (((B4SOIvsdfb+((model_.LVSDFB)*Inv_L))+((model_.WVSDFB)*Inv_W))+((model_.PVSDFB)*Inv_LW));
pParam_B4SOIvsdth = (((B4SOIvsdth+((model_.LVSDTH)*Inv_L))+((model_.WVSDTH)*Inv_W))+((model_.PVSDTH)*Inv_LW));
pParam_B4SOIdelvt = (((B4SOIdelvt+((model_.LDELVT)*Inv_L))+((model_.WDELVT)*Inv_W))+((model_.PDELVT)*Inv_LW));
pParam_B4SOIacde = (((B4SOIacde+((model_.LACDE)*Inv_L))+((model_.WACDE)*Inv_W))+((model_.PACDE)*Inv_LW));
{
double value_pow_0 = pow((pParam_B4SOInpeak/2.0e16),(-0.25));
pParam_B4SOIacde = (pParam_B4SOIacde*value_pow_0);
}
pParam_B4SOImoin = (((B4SOImoin+((model_.LMOIN)*Inv_L))+((model_.WMOIN)*Inv_W))+((model_.PMOIN)*Inv_LW));
pParam_B4SOInoff = (((B4SOInoff+((model_.LNOFF)*Inv_L))+((model_.WNOFF)*Inv_W))+((model_.PNOFF)*Inv_LW));
pParam_B4SOInoff2 = (((B4SOInoff2+((model_.LNOFF2)*Inv_L))+((model_.WNOFF2)*Inv_W))+((model_.PNOFF2)*Inv_LW));
pParam_B4SOIdvtp0 = (((B4SOIdvtp0+((model_.LDVTP0)*Inv_L))+((model_.WDVTP0)*Inv_W))+((model_.PDVTP0)*Inv_LW));
pParam_B4SOIdvtp1 = (((B4SOIdvtp1+((model_.LDVTP1)*Inv_L))+((model_.WDVTP1)*Inv_W))+((model_.PDVTP1)*Inv_LW));
pParam_B4SOIdvtp2 = (((B4SOIdvtp2+((model_.LDVTP2)*Inv_L))+((model_.WDVTP2)*Inv_W))+((model_.PDVTP2)*Inv_LW));
pParam_B4SOIdvtp3 = (((B4SOIdvtp3+((model_.LDVTP3)*Inv_L))+((model_.WDVTP3)*Inv_W))+((model_.PDVTP3)*Inv_LW));
pParam_B4SOIdvtp4 = (((B4SOIdvtp4+((model_.LDVTP4)*Inv_L))+((model_.WDVTP4)*Inv_W))+((model_.PDVTP4)*Inv_LW));
pParam_B4SOIminv = (((B4SOIminv+((model_.LMINV)*Inv_L))+((model_.WMINV)*Inv_W))+((model_.PMINV)*Inv_LW));
pParam_B4SOIminvcv = (((B4SOIminvcv+((model_.LMINVCV)*Inv_L))+((model_.WMINVCV)*Inv_W))+((model_.PMINVCV)*Inv_LW));
pParam_B4SOIvoffcv = (((B4SOIvoffcv+((model_.LVOFFCV)*Inv_L))+((model_.WVOFFCV)*Inv_W))+((model_.PVOFFCV)*Inv_LW));
pParam_B4SOIfprout = (((B4SOIfprout+((model_.LFPROUT)*Inv_L))+((model_.WFPROUT)*Inv_W))+((model_.PFPROUT)*Inv_LW));
pParam_B4SOIpdits = (((B4SOIpdits+((model_.LPDITS)*Inv_L))+((model_.WPDITS)*Inv_W))+((model_.PPDITS)*Inv_LW));
pParam_B4SOIpditsd = (((B4SOIpditsd+((model_.LPDITSD)*Inv_L))+((model_.WPDITSD)*Inv_W))+((model_.PPDITSD)*Inv_LW));
pParam_B4SOIaigbcp2 = (((B4SOIaigbcp2+((model_.LAIGBCP2)*Inv_L))+((model_.WAIGBCP2)*Inv_W))+((model_.PAIGBCP2)*Inv_LW));
pParam_B4SOIbigbcp2 = (((B4SOIbigbcp2+((model_.LBIGBCP2)*Inv_L))+((model_.WBIGBCP2)*Inv_W))+((model_.PBIGBCP2)*Inv_LW));
pParam_B4SOIcigbcp2 = (((B4SOIcigbcp2+((model_.LCIGBCP2)*Inv_L))+((model_.WCIGBCP2)*Inv_W))+((model_.PCIGBCP2)*Inv_LW));
{
double value_atan_0 = atan(pParam_B4SOIminv);
pParam_B4SOImstar = (0.5+(value_atan_0/3.14159265358979323846));
}
if (((B4SOIvgstcvMod==0)&&((model_.VERSION)>=4.1)))
{
UserInfo(*this)  << "vgstcvMod = 0, users are suggested to use vgstcvMod = 1 or 2." << " " <<  std::endl; 
}
{
double value_atan_0 = atan(pParam_B4SOIminvcv);
pParam_B4SOImstarcv = (0.5+(value_atan_0/3.14159265358979323846));
}
T0 = (TempRatio-1.0);
pParam_B4SOIuatemp = pParam_B4SOIua;
pParam_B4SOIubtemp = pParam_B4SOIub;
pParam_B4SOIuctemp = pParam_B4SOIuc;
{
double value_pow_0 = pow((pParam_B4SOIweff*1E6),pParam_B4SOIwr);
pParam_B4SOIrds0denom = value_pow_0;
}
pParam_B4SOIrth = ((B4SOIrth0/(B4SOInf*(pParam_B4SOIweff+B4SOIwth0)))*B4SOInseg);
pParam_B4SOIcth = ((B4SOIcth0*(B4SOInf*(pParam_B4SOIweff+B4SOIwth0)))/B4SOInseg);
if ((B4SOIrbody==0))
{
pParam_B4SOIrbody = 0.0;
}
else
{
pParam_B4SOIrbody = ((((((B4SOIfrbody*B4SOIrbody)*B4SOIrhalo)/((2*B4SOIrbody)+(B4SOIrhalo*pParam_B4SOIleff)))*pParam_B4SOIweff)/B4SOInseg)/B4SOInf);
}
{
double value_pow_0 = pow((B4SOItoxref/B4SOItoxqm),B4SOIntox);
pParam_B4SOIoxideRatio = ((value_pow_0/B4SOItoxqm)/B4SOItoxqm);
}
pParam_B4SOIua = (pParam_B4SOIua+(pParam_B4SOIua1*T0));
pParam_B4SOIub = (pParam_B4SOIub+(pParam_B4SOIub1*T0));
pParam_B4SOIuc = (pParam_B4SOIuc+(pParam_B4SOIuc1*T0));
if ((pParam_B4SOIu0>1.0))
{
pParam_B4SOIu0 = (pParam_B4SOIu0/1.0e4);
}
{
double value_pow_0 = pow(TempRatio,pParam_B4SOIute);
pParam_B4SOIu0temp = (pParam_B4SOIu0*value_pow_0);
}
pParam_B4SOIvsattemp = (pParam_B4SOIvsat-(pParam_B4SOIat*T0));
pParam_B4SOIrds0 = ((pParam_B4SOIrdsw+(pParam_B4SOIprt*T0))/pParam_B4SOIrds0denom);
if ((B4SOIrdsMod==1))
{
PowWeffWr = (pParam_B4SOIrds0denom*B4SOInf);
T10 = (pParam_B4SOIprt*T0);
T1 = (pParam_B4SOIrdw+T10);
T2 = (B4SOIrdwmin+T10);
if ((T1<0.0))
{
T1 = 0.0;
UserInfo(*this)  << "Warning: Rdw at current temperature is negative; set to 0." << " " <<  std::endl; 
}
if ((T2<0.0))
{
T2 = 0.0;
UserInfo(*this)  << "Warning: Rdwmin at current temperature is negative; set to 0." << " " <<  std::endl; 
}
pParam_B4SOIrd0 = (T1/PowWeffWr);
pParam_B4SOIrdwmin = (T2/PowWeffWr);
T3 = (pParam_B4SOIrsw+T10);
T4 = (B4SOIrswmin+T10);
if ((T3<0.0))
{
T3 = 0.0;
UserInfo(*this)  << "Warning: Rsw at current temperature is negative; set to 0." << " " <<  std::endl; 
}
if ((T4<0.0))
{
T4 = 0.0;
UserInfo(*this)  << "Warning: Rswmin at current temperature is negative; set to 0." << " " <<  std::endl; 
}
pParam_B4SOIrs0 = (T3/PowWeffWr);
pParam_B4SOIrswmin = (T4/PowWeffWr);
}
else
{
pParam_B4SOIrd0 = 0;
pParam_B4SOIrdwmin = 0;
pParam_B4SOIrs0 = 0;
pParam_B4SOIrswmin = 0;
}
if ((B4SOIcgdo<0.0))
{
UserInfo(*this)  << "Warning: cgdo = %g is negative. Set to zero." << " " << B4SOIcgdo << " " <<  std::endl; 
B4SOIcgdo = 0.0;
}
if ((B4SOIcgso<0.0))
{
UserInfo(*this)  << "Warning: cgso = %g is negative. Set to zero." << " " << B4SOIcgso << " " <<  std::endl; 
B4SOIcgso = 0.0;
}
if ((B4SOIcgeo<0.0))
{
UserInfo(*this)  << "Warning: cgeo = %g is negative. Set to zero." << " " << B4SOIcgeo << " " <<  std::endl; 
B4SOIcgeo = 0.0;
}
pParam_B4SOIcgdo = ((B4SOIcgdo+pParam_B4SOIcf)*pParam_B4SOIwdiodCV);
pParam_B4SOIcgso = ((B4SOIcgso+pParam_B4SOIcf)*pParam_B4SOIwdiosCV);
pParam_B4SOIcgeo = ((B4SOIcgeo*pParam_B4SOIleffCV)*B4SOInf);
if (((!(model_.given("NCH")))&&(model_.given("GAMMA1"))))
{
T0 = (pParam_B4SOIgamma1*B4SOIcox);
pParam_B4SOInpeak = ((3.021E22*T0)*T0);
}
if ((B4SOIsoiMod==2))
{
if (B4SOImtrlMod)
{
NchMax = (((((B4SOIbg0sub-0.1)/1.60219e-19)*2e-6)*epssub)/(B4SOIetsi*B4SOIetsi));
if ((pParam_B4SOInpeak>NchMax))
{
UserInfo(*this)  << "SOIMOD=2 cannot support given Nch=%g cm^-3 and Etsi=%g m " << " " << pParam_B4SOInpeak << " " << B4SOIetsi << " " <<  std::endl; 
UserInfo(*this)  << "Exceeds maximum allowed band bending of (Eg-0.1)eV" << " " <<  std::endl; 
UserInfo(*this)  << "Nch is set to  %g cm^-3" << " " << NchMax << " " <<  std::endl; 
pParam_B4SOInpeak = NchMax;
}
}
else
{
NchMax = (((((1.12-0.1)/1.60219e-19)*2e-6)*epssub)/(B4SOItsi*B4SOItsi));
if ((pParam_B4SOInpeak>NchMax))
{
UserInfo(*this)  << "SOIMOD=2 cannot support given Nch=%g cm^-3 and Tsi=%g m " << " " << pParam_B4SOInpeak << " " << B4SOItsi << " " <<  std::endl; 
UserInfo(*this)  << "Exceeds maximum allowed band bending of (Eg-0.1)eV" << " " <<  std::endl; 
UserInfo(*this)  << "Nch is set to  %g cm^-3" << " " << NchMax << " " <<  std::endl; 
pParam_B4SOInpeak = NchMax;
}
}
}
B4SOIcbox = (3.453133e-11/B4SOItbox);
if (B4SOImtrlMod)
{
B4SOIcsi = (1.03594e-10/B4SOIetsi);
}
else
{
B4SOIcsi = (1.03594e-10/B4SOItsi);
}
if (B4SOImtrlMod)
{
Qsi = ((((1.60219e-19*pParam_B4SOInpeak)*(1.0+(B4SOIlpe0/B4SOIl)))*1e6)*B4SOIetsi);
}
else
{
Qsi = ((((1.60219e-19*pParam_B4SOInpeak)*(1.0+(B4SOIlpe0/B4SOIl)))*1e6)*B4SOItsi);
}
Vbs0t = ((0.8-((0.5*Qsi)/B4SOIcsi))+pParam_B4SOIvbsa);
if ((B4SOIsoiMod==3))
{
if ((Vbs0t>pParam_B4SOIvbs0fd))
{
B4SOIsoiMod = 2;
}
else
{
if ((Vbs0t<pParam_B4SOIvbs0pd))
{
B4SOIsoiMod = 0;
}
else
{
B4SOIsoiMod = 1;
}
}
}
T4 = ((1.115/B4SOIvtm)*(TempRatio-1.0));
T7 = ((pParam_B4SOIxbjt*T4)/pParam_B4SOIndiode);
if ((T7>100.0))
{
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T0 = value_exp_0;
}
}
}
T7 = ((pParam_B4SOIxdif*T4)/pParam_B4SOIndiode);
if ((T7>100.0))
{
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T1 = value_exp_0;
}
}
}
T7 = ((pParam_B4SOIxrec*T4)/pParam_B4SOInrecf0);
if ((T7>100.0))
{
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T2 = value_exp_0;
}
}
}
pParam_B4SOIahli0s = (pParam_B4SOIahli*T0);
pParam_B4SOIjbjts = (pParam_B4SOIisbjt*T0);
pParam_B4SOIjdifs = (pParam_B4SOIisdif*T1);
pParam_B4SOIjrecs = (pParam_B4SOIisrec*T2);
T7 = (pParam_B4SOIxtun*(TempRatio-1));
if ((T7>100.0))
{
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T0 = value_exp_0;
}
}
}
pParam_B4SOIjtuns = (pParam_B4SOIistun*T0);
T7 = ((pParam_B4SOIxbjt*T4)/pParam_B4SOIndioded);
if ((T7>100.0))
{
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T0 = value_exp_0;
}
}
}
T7 = ((pParam_B4SOIxdifd*T4)/pParam_B4SOIndioded);
if ((T7>100.0))
{
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T1 = value_exp_0;
}
}
}
T7 = ((pParam_B4SOIxrecd*T4)/pParam_B4SOInrecf0d);
if ((T7>100.0))
{
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T2 = value_exp_0;
}
}
}
pParam_B4SOIahli0d = (pParam_B4SOIahlid*T0);
pParam_B4SOIjbjtd = (pParam_B4SOIidbjt*T0);
pParam_B4SOIjdifd = (pParam_B4SOIiddif*T1);
pParam_B4SOIjrecd = (pParam_B4SOIidrec*T2);
T7 = (pParam_B4SOIxtund*(TempRatio-1));
if ((T7>100.0))
{
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
T0 = value_exp_0;
}
}
}
pParam_B4SOIjtund = (pParam_B4SOIidtun*T0);
if ((pParam_B4SOInsub>0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOInpeak/pParam_B4SOInsub));
pParam_B4SOIvfbb = (((-B4SOItype)*B4SOIvtm)*evaluator_lln_0.getValues((pParam_B4SOInpeak/pParam_B4SOInsub)));
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)/ni));
pParam_B4SOIvfbb = (((-B4SOItype)*B4SOIvtm)*evaluator_lln_0.getValues(((((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)/ni)));
}
}
if ((!(model_.given("VSDFB"))))
{
if ((pParam_B4SOInsub>0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((((1e20*pParam_B4SOInsub)/ni)/ni));
pParam_B4SOIvsdfb = ((-B4SOItype)*((B4SOIvtm*evaluator_lln_0.getValues((((1e20*pParam_B4SOInsub)/ni)/ni)))-0.3));
}
}
else
{
if ((pParam_B4SOInsub<0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((-1e20)/pParam_B4SOInsub));
pParam_B4SOIvsdfb = ((-B4SOItype)*((B4SOIvtm*evaluator_lln_0.getValues(((-1e20)/pParam_B4SOInsub)))+0.3));
}
}
}
}
{
double value_fabs_0 = fabs(pParam_B4SOInsub);
AnalogFunctions::llnEvaluator evaluator_lln_1((value_fabs_0/ni));
SDphi = ((2.0*B4SOIvtm)*evaluator_lln_1.getValues((value_fabs_0/ni)));
}
{
double value_fabs_0 = fabs(pParam_B4SOInsub);
double value_sqrt_1 = sqrt(value_fabs_0);
SDgamma = ((sqrt2qeps*value_sqrt_1)/B4SOIcbox);
}
if ((!(model_.given("VSDTH"))))
{
if ((((pParam_B4SOInsub>0)&&(B4SOItype>0))||((pParam_B4SOInsub<0)&&(B4SOItype<0))))
{
{
double value_sqrt_0 = sqrt(SDphi);
pParam_B4SOIvsdth = ((pParam_B4SOIvsdfb+SDphi)+(SDgamma*value_sqrt_0));
}
}
else
{
{
double value_sqrt_0 = sqrt(SDphi);
pParam_B4SOIvsdth = ((pParam_B4SOIvsdfb-SDphi)-(SDgamma*value_sqrt_0));
}
}
}
if ((!(model_.given("CSDMIN"))))
{
{
double value_fabs_0 = fabs(pParam_B4SOInsub);
double value_sqrt_1 = sqrt((((2.0*epssub)*SDphi)/((1.60219e-19*value_fabs_0)*1.0e6)));
tmp = value_sqrt_1;
}
tmp1 = (epssub/tmp);
B4SOIcsdmin = ((tmp1*B4SOIcbox)/(tmp1+B4SOIcbox));
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOInpeak/ni));
pParam_B4SOIphi = ((2.0*B4SOIvtm)*evaluator_lln_0.getValues((pParam_B4SOInpeak/ni)));
}
{
double value_sqrt_0 = sqrt(pParam_B4SOIphi);
pParam_B4SOIsqrtPhi = value_sqrt_0;
}
{
double value_sqrt_0 = sqrt(((2.0*epssub)/((1.60219e-19*pParam_B4SOInpeak)*1.0e6)));
pParam_B4SOIXdep0 = (value_sqrt_0*pParam_B4SOIsqrtPhi);
}
{
double value_sqrt_0 = sqrt(pParam_B4SOIXdep0);
pParam_B4SOIsqrtXdep0 = value_sqrt_0;
}
if ((B4SOImtrlMod==0))
{
{
double value_sqrt_0 = sqrt(((((3.0*3.9)/epsrox)*pParam_B4SOIxj)*B4SOItox));
pParam_B4SOIlitl = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt((((epssub*pParam_B4SOIxj)*toxe)/(epsrox*8.85418e-12)));
pParam_B4SOIlitl = value_sqrt_0;
}
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*pParam_B4SOInpeak)/(ni*ni)));
pParam_B4SOIvbi = (B4SOIvtm*evaluator_lln_0.getValues(((1.0e20*pParam_B4SOInpeak)/(ni*ni))));
}
{
double value_sqrt_0 = sqrt((((((1.60219e-19*epssub)*pParam_B4SOInpeak)*1.0e6)/2.0)/pParam_B4SOIphi));
pParam_B4SOIcdep0 = value_sqrt_0;
}
if ((B4SOImtrlMod==0))
{
if ((pParam_B4SOIngate>0.0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOIngate/1.0e20));
pParam_B4SOIvfbsd = (Vtm0*evaluator_lln_0.getValues((pParam_B4SOIngate/1.0e20)));
}
}
else
{
pParam_B4SOIvfbsd = 0.0;
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOInsd/ni));
T0 = (Vtm0*evaluator_lln_0.getValues((pParam_B4SOInsd/ni)));
}
T1 = (0.5*Eg0);
if ((T0>T1))
{
T0 = T1;
}
T2 = ((B4SOIeasub+T1)-(B4SOItype*T0));
pParam_B4SOIvfbsd = (B4SOIphig-T2);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((B4SOItoxref/B4SOItoxqm));
double value_exp_1 = exp((B4SOIntox*evaluator_lln_0.getValues((B4SOItoxref/B4SOItoxqm))));
pParam_B4SOIToxRatio = ((value_exp_1/B4SOItoxqm)/B4SOItoxqm);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((B4SOItoxref/(B4SOItoxqm*pParam_B4SOIpoxedge)));
double value_exp_1 = exp((B4SOIntox*evaluator_lln_0.getValues((B4SOItoxref/(B4SOItoxqm*pParam_B4SOIpoxedge)))));
pParam_B4SOIToxRatioEdge = ((((value_exp_1/B4SOItoxqm)/B4SOItoxqm)/pParam_B4SOIpoxedge)/pParam_B4SOIpoxedge);
}
pParam_B4SOIAechvb = ((B4SOItype==1)?agbc2p:agbc2n);
pParam_B4SOIBechvb = ((B4SOItype==1)?bgbc2p:bgbc2n);
pParam_B4SOIAechvbEdges = (((pParam_B4SOIAechvb*((pParam_B4SOIweff/B4SOInseg)+B4SOIpsbcp))*pParam_B4SOIdlcig)*pParam_B4SOIToxRatioEdge);
pParam_B4SOIAechvbEdged = (((pParam_B4SOIAechvb*((pParam_B4SOIweff/B4SOInseg)+B4SOIpdbcp))*pParam_B4SOIdlcig)*pParam_B4SOIToxRatioEdge);
pParam_B4SOIBechvbEdge = (((-pParam_B4SOIBechvb)*B4SOItoxqm)*pParam_B4SOIpoxedge);
pParam_B4SOIAechvb = ((pParam_B4SOIAechvb*pParam_B4SOIToxRatio)*(((pParam_B4SOIweff/B4SOInseg)*pParam_B4SOIleff)+(B4SOIagbcpd/B4SOInf)));
pParam_B4SOIBechvb = (pParam_B4SOIBechvb*(-B4SOItoxqm));
if (((model_.given("K1"))||(model_.given("K2"))))
{
if ((!(model_.given("K1"))))
{
UserInfo(*this)  << "Warning: k1 should be specified with k2." << " " <<  std::endl; 

d_pParam_B4SOIk1_dTemp_t_GND = d_pParam_B4SOIk1_dV_p_GND = d_pParam_B4SOIk1_dV_b_GND =  0.0;
pParam_B4SOIk1 = 0.53;
}
if ((!(model_.given("K2"))))
{
UserInfo(*this)  << "Warning: k2 should be specified with k1." << " " <<  std::endl; 
pParam_B4SOIk2 = (-0.0186);
}
if ((model_.given("XT")))
{
UserInfo(*this)  << "Warning: xt is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("VBX")))
{
UserInfo(*this)  << "Warning: vbx is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("VBM")))
{
UserInfo(*this)  << "Warning: vbm is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("GAMMA1")))
{
UserInfo(*this)  << "Warning: gamma1 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("GAMMA2")))
{
UserInfo(*this)  << "Warning: gamma2 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
}
else
{
if ((!(model_.given("VBX"))))
{
if (B4SOImtrlMod)
{
T0 = ((1.60219e-19/(2.0*epssub))*1.0e6);
}
else
{
T0 = 7.7348e-4;
}

d_pParam_B4SOIvbx_dTemp_t_GND = d_pParam_B4SOIvbx_dV_p_GND = d_pParam_B4SOIvbx_dV_b_GND =  0.0;
pParam_B4SOIvbx = (pParam_B4SOIphi-(((T0*pParam_B4SOInpeak)*pParam_B4SOIxt)*pParam_B4SOIxt));
}
if ((pParam_B4SOIvbx>0.0))
{

d_pParam_B4SOIvbx_dTemp_t_GND = d_pParam_B4SOIvbx_dV_p_GND = d_pParam_B4SOIvbx_dV_b_GND =  0.0;
pParam_B4SOIvbx = (-pParam_B4SOIvbx);
}
if ((pParam_B4SOIvbm>0.0))
{
pParam_B4SOIvbm = (-pParam_B4SOIvbm);
}
if ((!(model_.given("GAMMA1"))))
{
{
double value_sqrt_0 = sqrt(pParam_B4SOInpeak);
pParam_B4SOIgamma1 = ((sqrt2qeps*value_sqrt_0)/B4SOIcox);
}
}
if ((!(model_.given("GAMMA2"))))
{
{
double value_sqrt_0 = sqrt(pParam_B4SOInsub);
pParam_B4SOIgamma2 = ((sqrt2qeps*value_sqrt_0)/B4SOIcox);
}
}
T0 = (pParam_B4SOIgamma1-pParam_B4SOIgamma2);
{
double value_sqrt_0 = sqrt((pParam_B4SOIphi-pParam_B4SOIvbx));
T1 = (value_sqrt_0-pParam_B4SOIsqrtPhi);
}
{
double value_sqrt_0 = sqrt((pParam_B4SOIphi-pParam_B4SOIvbm));
T2 = (pParam_B4SOIsqrtPhi*(value_sqrt_0-pParam_B4SOIsqrtPhi));
}
pParam_B4SOIk2 = ((T0*T1)/((2.0*T2)+pParam_B4SOIvbm));
{
double value_sqrt_0 = sqrt((pParam_B4SOIphi-pParam_B4SOIvbm));

d_pParam_B4SOIk1_dTemp_t_GND = d_pParam_B4SOIk1_dV_p_GND = d_pParam_B4SOIk1_dV_b_GND =  0.0;
pParam_B4SOIk1 = (pParam_B4SOIgamma2-((2.0*pParam_B4SOIk2)*value_sqrt_0));
}
}
T0 = (pParam_B4SOIweff+pParam_B4SOIk1w2);
if ((T0<1e-8))
{
T0 = 1e-8;
}

d_pParam_B4SOIk1eff_dTemp_t_GND = d_pParam_B4SOIk1eff_dV_p_GND = d_pParam_B4SOIk1eff_dV_b_GND =  0.0;
pParam_B4SOIk1eff = (pParam_B4SOIk1*(1+(pParam_B4SOIk1w1/T0)));
if ((!(model_.given("VFB"))))
{
if (((model_.given("VTH0"))||(model_.given("VTHO"))))
{
pParam_B4SOIvfb = (((B4SOItype*pParam_B4SOIvth0)-pParam_B4SOIphi)-(pParam_B4SOIk1eff*pParam_B4SOIsqrtPhi));
}
else
{
pParam_B4SOIvfb = (-1.0);
}
}
if ((!(model_.given("VTH0"))))
{
pParam_B4SOIvth0 = (B4SOItype*((pParam_B4SOIvfb+pParam_B4SOIphi)+(pParam_B4SOIk1eff*pParam_B4SOIsqrtPhi)));
}
pParam_B4SOIk1ox = ((pParam_B4SOIk1eff*B4SOItox)/B4SOItoxm);
T1 = (B4SOIfactor1*pParam_B4SOIsqrtXdep0);
{
double value_exp_0 = exp(((((-0.5)*pParam_B4SOIdsub)*pParam_B4SOIleff)/T1));
T0 = value_exp_0;
}
pParam_B4SOItheta0vb0 = (T0+((2.0*T0)*T0));
{
double value_exp_0 = exp(((((-0.5)*pParam_B4SOIdrout)*pParam_B4SOIleff)/T1));
T0 = value_exp_0;
}
T2 = (T0+((2.0*T0)*T0));
pParam_B4SOIthetaRout = ((pParam_B4SOIpdibl1*T2)+pParam_B4SOIpdibl2);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(pParam_B4SOIleff);
double value_exp_1 = exp((pParam_B4SOIdvtp3*evaluator_lln_0.getValues(pParam_B4SOIleff)));
pParam_B4SOIdvtp2factor = (pParam_B4SOIdvtp2/value_exp_1);
}
if ((B4SOIwlod<0.0))
{
UserInfo(*this)  << "Warning: WLOD = %g is less than 0. Set to 0.0" << " " << B4SOIwlod << " " <<  std::endl; 
B4SOIwlod = 0.0;
}
{
double value_pow_0 = pow(Ldrn,B4SOIllodku0);
T0 = value_pow_0;
}
W_tmp = (Wdrn+B4SOIwlod);
{
double value_pow_0 = pow(W_tmp,B4SOIwlodku0);
T1 = value_pow_0;
}
tmp1 = ((((model_.LKU0)/T0)+((model_.WKU0)/T1))+((model_.PKU0)/(T0*T1)));
pParam_B4SOIku0 = (1.0+tmp1);
{
double value_pow_0 = pow(Ldrn,B4SOIllodvth);
T0 = value_pow_0;
}
{
double value_pow_0 = pow(W_tmp,B4SOIwlodvth);
T1 = value_pow_0;
}
tmp1 = ((((model_.LKVTH0)/T0)+((model_.WKVTH0)/T1))+((model_.PKVTH0)/(T0*T1)));
pParam_B4SOIkvth0 = (1.0+tmp1);
{
double value_sqrt_0 = sqrt(((pParam_B4SOIkvth0*pParam_B4SOIkvth0)+1.0E-9));
pParam_B4SOIkvth0 = value_sqrt_0;
}
T0 = (TempRatio-1.0);
pParam_B4SOIku0temp = ((pParam_B4SOIku0*(1.0+(B4SOItku0*T0)))+1.0E-9);
Inv_saref = (1.0/(B4SOIsaref+(0.5*Ldrn)));
Inv_sbref = (1.0/(B4SOIsbref+(0.5*Ldrn)));
pParam_B4SOIinv_od_ref = (Inv_saref+Inv_sbref);
pParam_B4SOIrho_ref = ((B4SOIku0/pParam_B4SOIku0temp)*pParam_B4SOIinv_od_ref);
if ((((B4SOIsa>0.0)&&(B4SOIsb>0.0))&&((B4SOInf==1.0)||((B4SOInf>1.0)&&(B4SOIsd>0.0)))))
{
Inv_sa = 0;
Inv_sb = 0;
if ((B4SOIkvsat<(-1.0)))
{
UserInfo(*this)  << "Warning: KVSAT = %g is too small; Reset to -1.0." << " " << B4SOIkvsat << " " <<  std::endl; 
B4SOIkvsat = (-1.0);
}
else
{
if ((B4SOIkvsat>1.0))
{
UserInfo(*this)  << "Warning: KVSAT = %g is too big; Reset to 1.0." << " " << B4SOIkvsat << " " <<  std::endl; 
B4SOIkvsat = 1.0;
}
else
{
}
}
for (i=0;(i<B4SOInf);i=(i+1) )//Begin block forloop
{
//Block-local variables for block forloop
double T0=0.0;
double T1=0.0;
//End of Block-local variables
T0 = ((1.0/B4SOInf)/((B4SOIsa+(0.5*Ldrn))+(i*(B4SOIsd+Ldrn))));
T1 = ((1.0/B4SOInf)/((B4SOIsb+(0.5*Ldrn))+(i*(B4SOIsd+Ldrn))));
Inv_sa = (Inv_sa+T0);
Inv_sb = (Inv_sb+T1);
}
// End block forloop
Inv_ODeff = (Inv_sa+Inv_sb);
B4SOIInv_ODeff = Inv_ODeff;
rho = ((B4SOIku0/pParam_B4SOIku0temp)*Inv_ODeff);
T0 = ((1.0+rho)/(1.0+pParam_B4SOIrho_ref));
here_B4SOIu0temp = (pParam_B4SOIu0temp*T0);
T1 = ((1.0+(B4SOIkvsat*rho))/(1.0+(B4SOIkvsat*pParam_B4SOIrho_ref)));
here_B4SOIvsattemp = (pParam_B4SOIvsattemp*T1);
OD_offset = (Inv_ODeff-pParam_B4SOIinv_od_ref);
dvth0_lod = ((B4SOIkvth0/pParam_B4SOIkvth0)*OD_offset);
{
double value_pow_0 = pow(pParam_B4SOIkvth0,B4SOIlodk2);
dk2_lod = ((B4SOIstk2/value_pow_0)*OD_offset);
}
{
double value_pow_0 = pow(pParam_B4SOIkvth0,B4SOIlodeta0);
deta0_lod = ((B4SOIsteta0/value_pow_0)*OD_offset);
}
{
double value_pow_0 = pow(pParam_B4SOIkvth0,B4SOIlodeta0cv);
deta0cv_lod = ((B4SOIsteta0cv/value_pow_0)*OD_offset);
}

d_here_B4SOIvth0_dTemp_t_GND = d_here_B4SOIvth0_dV_p_GND = d_here_B4SOIvth0_dV_b_GND =  0.0;
here_B4SOIvth0 = (pParam_B4SOIvth0+dvth0_lod);

d_here_B4SOIk2_dTemp_t_GND = d_here_B4SOIk2_dV_p_GND = d_here_B4SOIk2_dV_b_GND =  0.0;
here_B4SOIk2 = (pParam_B4SOIk2+dk2_lod);
here_B4SOIeta0 = (pParam_B4SOIeta0+deta0_lod);
here_B4SOIeta0cv = (pParam_B4SOIeta0cv+deta0cv_lod);
}
else
{
here_B4SOIu0temp = pParam_B4SOIu0temp;

d_here_B4SOIvth0_dTemp_t_GND = d_here_B4SOIvth0_dV_p_GND = d_here_B4SOIvth0_dV_b_GND =  0.0;
here_B4SOIvth0 = pParam_B4SOIvth0;
here_B4SOIvsattemp = pParam_B4SOIvsattemp;

d_here_B4SOIk2_dTemp_t_GND = d_here_B4SOIk2_dV_p_GND = d_here_B4SOIk2_dV_b_GND =  0.0;
here_B4SOIk2 = pParam_B4SOIk2;
here_B4SOIeta0 = pParam_B4SOIeta0;
here_B4SOIeta0cv = pParam_B4SOIeta0cv;
B4SOIInv_ODeff = 0;
pParam_B4SOIinv_od_ref = 0;
B4SOIkvsat = 0;
}
here_B4SOIk2ox = ((here_B4SOIk2*B4SOItox)/B4SOItoxm);

d_here_B4SOIvth0_dTemp_t_GND = d_here_B4SOIvth0_dV_p_GND = d_here_B4SOIvth0_dV_b_GND =  0.0;
here_B4SOIvth0 = (here_B4SOIvth0+B4SOIdelvto);

d_here_B4SOIvfb_dTemp_t_GND = d_here_B4SOIvfb_dV_p_GND = d_here_B4SOIvfb_dV_b_GND =  0.0;
here_B4SOIvfb = (pParam_B4SOIvfb+(B4SOItype*B4SOIdelvto));
B4SOIcsbox = (B4SOIcbox*B4SOIsourceArea);
B4SOIcsmin = (B4SOIcsdmin*B4SOIsourceArea);
B4SOIcdbox = (B4SOIcbox*B4SOIdrainArea);
B4SOIcdmin = (B4SOIcsdmin*B4SOIdrainArea);
if ((B4SOIcsdmin>0))
{
if ((((pParam_B4SOInsub>0)&&(B4SOItype>0))||((pParam_B4SOInsub<0)&&(B4SOItype<0))))
{
T0 = (pParam_B4SOIvsdth-pParam_B4SOIvsdfb);
pParam_B4SOIsdt1 = (pParam_B4SOIvsdfb+(B4SOIasd*T0));
T1 = (B4SOIcsbox-B4SOIcsmin);
T2 = ((T1/T0)/T0);
pParam_B4SOIst2 = (T2/B4SOIasd);
pParam_B4SOIst3 = (T2/(1-B4SOIasd));
B4SOIst4 = ((((T0*T1)*(1+B4SOIasd))/3)-(B4SOIcsmin*pParam_B4SOIvsdfb));
T1 = (B4SOIcdbox-B4SOIcdmin);
T2 = ((T1/T0)/T0);
pParam_B4SOIdt2 = (T2/B4SOIasd);
pParam_B4SOIdt3 = (T2/(1-B4SOIasd));
B4SOIdt4 = ((((T0*T1)*(1+B4SOIasd))/3)-(B4SOIcdmin*pParam_B4SOIvsdfb));
}
else
{
T0 = (pParam_B4SOIvsdfb-pParam_B4SOIvsdth);
pParam_B4SOIsdt1 = (pParam_B4SOIvsdth+(B4SOIasd*T0));
T1 = (B4SOIcsmin-B4SOIcsbox);
T2 = ((T1/T0)/T0);
pParam_B4SOIst2 = (T2/B4SOIasd);
pParam_B4SOIst3 = (T2/(1-B4SOIasd));
B4SOIst4 = ((((T0*T1)*(1+B4SOIasd))/3)-(B4SOIcsbox*pParam_B4SOIvsdth));
T1 = (B4SOIcdmin-B4SOIcdbox);
T2 = ((T1/T0)/T0);
pParam_B4SOIdt2 = (T2/B4SOIasd);
pParam_B4SOIdt3 = (T2/(1-B4SOIasd));
B4SOIdt4 = ((((T0*T1)*(1+B4SOIasd))/3)-(B4SOIcdbox*pParam_B4SOIvsdth));
}
}
else
{
pParam_B4SOIsdt1 = 0;
pParam_B4SOIst2 = 0;
pParam_B4SOIst3 = 0;
B4SOIst4 = 0;
pParam_B4SOIdt2 = 0;
pParam_B4SOIdt3 = 0;
B4SOIdt4 = 0;
}
if (((B4SOIcfrcoeff<1.0)||(B4SOIcfrcoeff>2.0)))
{
UserInfo(*this)  << "Warning: CfrCoeff = %g is outside valid range [1,2], set to 1. \n" << " " << B4SOIcfrcoeff << " " <<  std::endl; 
B4SOIcfrcoeff = 1.0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((B4SOIcfrcoeff*(1+(B4SOItsi/B4SOItbox))));
T0 = (B4SOIcsdesw*evaluator_lln_0.getValues((B4SOIcfrcoeff*(1+(B4SOItsi/B4SOItbox)))));
}
T1 = (B4SOIsourcePerimeter-B4SOIw);
if ((T1>0.0))
{
B4SOIcsesw = (T0*T1);
}
else
{
B4SOIcsesw = 0.0;
}
T1 = (B4SOIdrainPerimeter-B4SOIw);
if ((T1>0.0))
{
B4SOIcdesw = (T0*T1);
}
else
{
B4SOIcdesw = 0.0;
}
B4SOIdrainResistance = (B4SOIsheetResistance*B4SOIdrainSquares);
if ((B4SOIdrainResistance<=1.0e-3))
{
B4SOIdrainResistance = 1.0e-3;
}
B4SOIsourceResistance = (B4SOIsheetResistance*B4SOIsourceSquares);
if ((B4SOIsourceResistance<=1.0e-3))
{
B4SOIsourceResistance = 1.0e-3;
}
if ((B4SOIln<1e-15))
{
B4SOIln = 1e-15;
}
T0 = (((((-0.5)*pParam_B4SOIleff)*pParam_B4SOIleff)/B4SOIln)/B4SOIln);
if ((T0>100.0))
{
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
T1 = value_exp_0;
}
}
}
pParam_B4SOIarfabjt = T1;
T0 = (pParam_B4SOIlbjt0*((1.0/pParam_B4SOIleff)+(1.0/B4SOIln)));
{
double value_pow_0 = pow(T0,pParam_B4SOInbjt);
pParam_B4SOIlratio = value_pow_0;
}
{
double value_pow_0 = pow(T0,pParam_B4SOIndif);
pParam_B4SOIlratiodif = (1.0+(B4SOIldif0*value_pow_0));
}
pParam_B4SOIvearly = (pParam_B4SOIvabjt+(pParam_B4SOIaely*pParam_B4SOIleff));
if ((pParam_B4SOIvearly<1))
{
pParam_B4SOIvearly = 1;
}
if ((B4SOImtrlMod==0))
{
B4SOItoxp = (B4SOItox-B4SOIdtoxcv);
}
else
{
Vtm0eot = (8.617087e-5*B4SOItempeot);
Vtmeot = Vtm0eot;
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*pParam_B4SOInpeak)/(ni*ni)));
vbieot = (Vtm0eot*evaluator_lln_0.getValues(((1.0e20*pParam_B4SOInpeak)/(ni*ni))));
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOInpeak/ni));
phieot = ((2.0*Vtm0eot)*evaluator_lln_0.getValues((pParam_B4SOInpeak/ni)));
}
{
double value_sqrt_0 = sqrt(phieot);
sqrtphieot = value_sqrt_0;
}
tmp2 = (here_B4SOIvfb+phieot);
vddeot = (B4SOItype*B4SOIvddeot);
T0 = (B4SOIepsrgate*8.85418e-12);
if (((((pParam_B4SOIngate>1.0e18)&&(pParam_B4SOIngate<1.0e25))&&(vddeot>tmp2))&&(T0!=0)))
{
T1 = ((((1.0e6*1.60219e-19)*epssub)*pParam_B4SOIngate)/(B4SOIcox*B4SOIcox));
{
double value_sqrt_0 = sqrt((1.0+((2.0*(vddeot-T0))/T1)));
T4 = value_sqrt_0;
}
T2 = (T1*(T4-1.0));
T3 = (((0.5*T2)*T2)/T1);
T7 = ((eggdep-T3)-0.05);
{
double value_sqrt_0 = sqrt(((T7*T7)+0.224));
T6 = value_sqrt_0;
}
T5 = (eggdep-(0.5*(T7+T6)));
Vgs_eff = (vddeot-T5);
}
else
{
Vgs_eff = vddeot;
}
V0 = (vbieot-phieot);
T3 = pParam_B4SOIsqrtXdep0;
lt1 = (B4SOIfactor1*T3);
ltw = (B4SOIfactor1*T3);
T0 = ((((-0.5)*pParam_B4SOIdvt1)*B4SOIleffeot)/lt1);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
T1 = value_exp_0;
}
Theta0 = (T1*(1.0+(2.0*T1)));
}
else
{
T1 = 3.720075976e-44;
Theta0 = (T1*(1.0+(2.0*T1)));
}
T2 = ((pParam_B4SOInfactor*epssub)/pParam_B4SOIXdep0);
T3 = pParam_B4SOIcdsc;
T4 = (((T2+(T3*Theta0))+pParam_B4SOIcit)/B4SOIcox);
if ((T4>=(-0.5)))
{
n = (1.0+T4);
}
else
{
T0 = (1.0/(3.0+(8.0*T4)));
n = ((1.0+(3.0*T4))*T0);
}
if ((pParam_B4SOIdvtp0>0.0))
{
T3 = (B4SOIleffeot+(2.0*pParam_B4SOIdvtp0));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((B4SOIleffeot/T3));
T4 = (Vtmeot*evaluator_lln_0.getValues((B4SOIleffeot/T3)));
}
DITS_Sft = (n*T4);
}
else
{
DITS_Sft = 0.0;
}

d_B4SOIthetavth_dV_b_si = d_B4SOIthetavth_dV_e_si = d_B4SOIthetavth_dV_di_si = d_B4SOIthetavth_dV_gi_si = d_B4SOIthetavth_dTemp_t_GND = d_B4SOIthetavth_dV_p_GND = d_B4SOIthetavth_dV_b_GND =  0.0;
B4SOIthetavth = (pParam_B4SOIdvt0*Theta0);
Delt_vth = (B4SOIthetavth*V0);
T0 = (((((-0.5)*pParam_B4SOIdvt1w)*B4SOIweffeot)*B4SOIleffeot)/ltw);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
T1 = value_exp_0;
}
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
T1 = 3.720075976e-44;
T2 = (T1*(1.0+(2.0*T1)));
}
T0 = (pParam_B4SOIdvt0w*T2);
DeltVthw = (T0*V0);
TempRatioMinus1 = ((B4SOItempeot/B4SOItnom)-1.0);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpe0/B4SOIleffeot)));
T0 = value_sqrt_0;
}
T1 = (pParam_B4SOIkt1+(pParam_B4SOIkt1l/B4SOIleffeot));
DeltVthtemp = (((pParam_B4SOIk1ox*(T0-1.0))*sqrtphieot)+(T1*TempRatioMinus1));
tmp2 = ((toxe*phieot)/(B4SOIweffeot+pParam_B4SOIw0));
DIBL_Sft = 0.0;
DITS_Sft2 = 0.0;
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpeb/B4SOIleffeot)));
Lpe_Vb = value_sqrt_0;
}
sqrtPhisExt = sqrtphieot;
Vth = (((((((((B4SOItype*here_B4SOIvth0)+(((pParam_B4SOIk1ox*sqrtPhisExt)-(pParam_B4SOIk1eff*sqrtphieot))*Lpe_Vb))-Delt_vth)-DeltVthw)+(pParam_B4SOIk3*tmp2))+DeltVthtemp)-DIBL_Sft)-DITS_Sft)-DITS_Sft2);
Vgst = (Vgs_eff-Vth);
T10 = (n*Vtmeot);
VgstNVt = ((pParam_B4SOImstar*Vgst)/T10);
ExpArg = ((pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))/T10);
if ((VgstNVt>100.0))
{
Vgsteff = Vgst;
}
else
{
if ((ExpArg>100.0))
{
T0 = ((Vgst-pParam_B4SOIvoff)/(n*Vtmeot));
{
double value_exp_0 = exp(T0);
ExpVgst = value_exp_0;
}
Vgsteff = (((Vtmeot*pParam_B4SOIcdep0)/B4SOIcox)*ExpVgst);
}
else
{
{
double value_exp_0 = exp(VgstNVt);
ExpVgst = value_exp_0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));
T1 = (T10*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
{
double value_exp_0 = exp(ExpArg);
T4 = ((((-B4SOIcox)/(Vtm0eot*pParam_B4SOIcdep0))*value_exp_0)*(1-pParam_B4SOImstar));
}
T2 = (pParam_B4SOImstar-((T10*T4)/(1.0-pParam_B4SOImstar)));
Vgsteff = (T1/T2);
}
}
T3 = (((B4SOItype*here_B4SOIvth0)-here_B4SOIvfb)-phieot);
vtfbphi2eot = (4.0*T3);
if ((vtfbphi2eot<0.0))
{
vtfbphi2eot = 0.0;
}
niter = 0;
toxpf = toxe;
toxpi = 1e6;
while (((niter<=4)&&(fabs((toxpf-toxpi))>1e-12)))
//Begin block whileloop
{
//Block-local variables for block whileloop
double T0=0.0;
double T1=0.0;
//End of Block-local variables
toxpi = toxpf;
tmp2 = (2.0e8*toxpf);
T0 = ((Vgsteff+vtfbphi2eot)/tmp2);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp(((B4SOIbdos*0.7)*evaluator_lln_0.getValues(T0)));
T1 = (1.0+value_exp_1);
}
Tcen = ((B4SOIados*1.9e-9)/T1);
toxpf = (toxe-((epsrox/B4SOIepsrsub)*Tcen));
niter = (niter+1);
}
// End block whileloop
B4SOItoxp = toxpf;
}
tmp = pParam_B4SOIsqrtXdep0;
tmp1 = (pParam_B4SOIvbi-pParam_B4SOIphi);
tmp2 = (B4SOIfactor1*tmp);
T0 = (((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*pParam_B4SOIleff)/tmp2);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
T1 = value_exp_0;
}
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
T1 = 3.720075976e-44;
T2 = (T1*(1.0+(2.0*T1)));
}
T0 = (pParam_B4SOIdvt0w*T2);
T2 = (T0*tmp1);
T0 = ((((-0.5)*pParam_B4SOIdvt1)*pParam_B4SOIleff)/tmp2);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
T1 = value_exp_0;
}
T3 = (T1*(1.0+(2.0*T1)));
}
else
{
T1 = 3.720075976e-44;
T3 = (T1*(1.0+(2.0*T1)));
}
T3 = ((pParam_B4SOIdvt0*T3)*tmp1);
T4 = ((B4SOItoxp*pParam_B4SOIphi)/(pParam_B4SOIweff+pParam_B4SOIw0));
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpe0/pParam_B4SOIleff)));
T0 = value_sqrt_0;
}
T5 = (((pParam_B4SOIk1ox*(T0-1.0))*pParam_B4SOIsqrtPhi)+((pParam_B4SOIkt1+(pParam_B4SOIkt1l/pParam_B4SOIleff))*(TempRatio-1.0)));
tmp3 = (((((B4SOItype*pParam_B4SOIvth0)-T2)-T3)+(pParam_B4SOIk3*T4))+T5);
pParam_B4SOIvfbzb = ((tmp3-pParam_B4SOIphi)-(pParam_B4SOIk1*pParam_B4SOIsqrtPhi));
pParam_B4SOIqsi = ((((1.60219e-19*pParam_B4SOInpeak)*(1.0+(pParam_B4SOIlpe0/pParam_B4SOIleff)))*1e6)*B4SOItsi);
B4SOIgrgeltd = ((B4SOIrshg*(B4SOIxgw+(((pParam_B4SOIweff/B4SOInseg)/3.0)/B4SOIngcon)))/((B4SOIngcon*B4SOInf)*(B4SOIl-B4SOIxgl)));
if ((B4SOIgrgeltd>0.0))
{
B4SOIgrgeltd = (1.0/B4SOIgrgeltd);
}
else
{
B4SOIgrgeltd = 1.0e3;
if ((B4SOIrgateMod!=0))
{
UserInfo(*this)  << "Warning: The gate conductance reset to 1.0e3 mho." << " " <<  std::endl; 
}
}
if (B4SOIrbodyMod)
{
if ((B4SOIrbdb<1.0e-3))
{
B4SOIgrbdb = 1.0e3;
}
else
{
B4SOIgrbdb = (B4SOIgbmin+(1.0/B4SOIrbdb));
}
if ((B4SOIrbsb<1.0e-3))
{
B4SOIgrbsb = 1.0e3;
}
else
{
B4SOIgrbsb = (B4SOIgbmin+(1.0/B4SOIrbsb));
}
}
else
{
B4SOIgrbdb = 0;
B4SOIgrbsb = 0;
}
B4SOIvfbzb = (pParam_B4SOIvfbzb+(B4SOItype*B4SOIdelvto));
{
double value_sqrt_0 = sqrt(((epssub*Vtm0)/((1.60219e-19*pParam_B4SOInpeak)*1.0e6)));
pParam_B4SOIldeb = (value_sqrt_0/3.0);
}
T1 = (((B4SOItype*here_B4SOIvth0)-here_B4SOIvfb)-pParam_B4SOIphi);
T2 = (T1+T1);
T3 = (2.5*T1);
B4SOIvtfbphi1 = ((B4SOItype==1)?T2:T3);
if ((B4SOIvtfbphi1<0.0))
{
B4SOIvtfbphi1 = 0.0;
}
if ((B4SOImobMod==4))
{
lt1 = (B4SOIfactor1*pParam_B4SOIsqrtXdep0);
T0 = ((pParam_B4SOIdvt1*pParam_B4SOIleff)/lt1);
if ((T0<100.0))
{
{
double value_exp_0 = exp(T0);
T1 = value_exp_0;
}
T2 = (T1-1.0);
T3 = (T2*T2);
T4 = (T3+((2.0*T1)*3.720075976e-44));
Theta0 = (T1/T4);
}
else
{
Theta0 = (1.0/(2.688117142e+43-2.0));
}
tmp1 = (epssub/pParam_B4SOIXdep0);
tmp2 = (pParam_B4SOInfactor*tmp1);
tmp3 = (((tmp2+(pParam_B4SOIcdsc*Theta0))+pParam_B4SOIcit)/B4SOIcox);
if ((tmp3>=(-0.5)))
{
n0 = (1.0+tmp3);
}
else
{
T0 = (1.0/(3.0+(8.0*tmp3)));
n0 = ((1.0+(3.0*tmp3))*T0);
}
T0 = (n0*Vtm0);
T1 = pParam_B4SOIvoff;
T2 = (T1/T0);
if ((T2<(-100.0)))
{
T3 = ((B4SOIcox*3.720075976e-44)/pParam_B4SOIcdep0);
T4 = (pParam_B4SOImstar+(T3*n0));
}
else
{
if ((T2>100.0))
{
T3 = ((B4SOIcox*2.688117142e+43)/pParam_B4SOIcdep0);
T4 = (pParam_B4SOImstar+(T3*n0));
}
else
{
{
double value_exp_0 = exp(T2);
T3 = ((value_exp_0*B4SOIcox)/pParam_B4SOIcdep0);
}
T4 = (pParam_B4SOImstar+(T3*n0));
}
}
B4SOIvgsteffvth = ((T0*.69314718055994530941)/T4);
}
else
{
B4SOIvgsteffvth = 0;
}
}
// End block b4soi_temp
}
// End block initial_instance


  return true;
}

//-----------------------------------------------------------------------------
// Function      : Instance::Instance
// Purpose       : "instance block" constructor
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------

Instance::Instance(
  const Configuration & configuration,
  const InstanceBlock & instance_block,
  Model &               model,
  const FactoryBlock &  factory_block)
  : DeviceInstance(instance_block, configuration.getInstanceParameters(), factory_block),
    model_(model),
    DTEMP(0.0),
    L(5e-6),
    W(5e-6),
    NF(1),
    SA(0.0),
    SB(0.0),
    SD(0.0),
    AD(0.0),
    AS(0.0),
    PD(0.0),
    PS(0.0),
    NRD(1.0),
    NRS(1.0),
    OFF(0),
    BJTOFF(0),
    DEBUG(0),
    RTH0(0.0),
    CTH0(1e-5),
    NRB(1.0),
    FRBODY(1.0),
    RBDB(50.0),
    RBSB(50.0),
    DELVTO(0.0),
    SOIMOD(0),
    NBC(0.0),
    NSEG(1),
    PDBCP(0.0),
    PSBCP(0.0),
    AGBCP(0.0),
    AGBCP2(0.0),
    AGBCPD(0.0),
    AEBCP(0.0),
    TNODEOUT(0),
    SHMOD(0),
    m(1.0),
    VDSAT(0.0),
    VTH(0.0),
    IDS(0.0),
    GM(0.0),
    GDS(0.0),
    GMBS(0.0),
    IC(0.0),
    IBD(0.0),
    IBS(0.0),
    IGIDL(0.0),
    IGISL(0.0),
    IGS(0.0),
    IGD(0.0),
    IGB(0.0),
    IGCS(0.0),
    IGCD(0.0),
    CGG(0.0),
    CGS(0.0),
    CGD(0.0),
    CBG(0.0),
    CBS(0.0),
    CBD(0.0),
    CDG(0.0),
    CDD(0.0),
    CDS(0.0),
    CAPBD(0.0),
    CAPBS(0.0),
    QG(0.0),
d_QG_dV_gm_si(0.0),
d_QG_dV_gi_p(0.0),
d_QG_dV_db_di(0.0),
d_QG_dV_sb_si(0.0),
d_QG_dV_di_si(0.0),
d_QG_dV_gi_si(0.0),
d_QG_dTemp_t_GND(0.0),
d_QG_dV_p_GND(0.0),
d_QG_dV_b_GND(0.0),
d_QG_dV_b_si(0.0),
d_QG_dV_e_si(0.0),
    QB(0.0),
d_QB_dV_b_si(0.0),
d_QB_dV_e_si(0.0),
d_QB_dV_di_si(0.0),
d_QB_dV_gi_si(0.0),
d_QB_dTemp_t_GND(0.0),
d_QB_dV_p_GND(0.0),
d_QB_dV_b_GND(0.0),
d_QB_dV_gi_p(0.0),
d_QB_dV_db_di(0.0),
d_QB_dV_sb_si(0.0),
    QD(0.0),
d_QD_dV_gi_p(0.0),
d_QD_dV_db_di(0.0),
d_QD_dV_sb_si(0.0),
d_QD_dV_di_si(0.0),
d_QD_dV_gi_si(0.0),
d_QD_dTemp_t_GND(0.0),
d_QD_dV_p_GND(0.0),
d_QD_dV_b_GND(0.0),
d_QD_dV_b_si(0.0),
d_QD_dV_e_si(0.0),
    QS(0.0),
    QJD(0.0),
d_QJD_dTemp_t_GND(0.0),
d_QJD_dV_p_GND(0.0),
d_QJD_dV_b_GND(0.0),
d_QJD_dV_gi_p(0.0),
d_QJD_dV_db_di(0.0),
d_QJD_dV_sb_si(0.0),
d_QJD_dV_b_si(0.0),
d_QJD_dV_e_si(0.0),
d_QJD_dV_di_si(0.0),
d_QJD_dV_gi_si(0.0),
    QJS(0.0),
d_QJS_dTemp_t_GND(0.0),
d_QJS_dV_p_GND(0.0),
d_QJS_dV_b_GND(0.0),
d_QJS_dV_gi_p(0.0),
d_QJS_dV_db_di(0.0),
d_QJS_dV_sb_si(0.0),
d_QJS_dV_b_si(0.0),
d_QJS_dV_e_si(0.0),
d_QJS_dV_di_si(0.0),
d_QJS_dV_gi_si(0.0),
    T_TOTAL_K(0.0),
    T_TOTAL_C(0.0),
    T_DELTA_SH(0.0),
    B4SOItype(0),
    B4SOInf(0.0),
    B4SOIsa(0.0),
    B4SOIsb(0.0),
    B4SOIsd(0.0),
    B4SOIbjtoff(0),
    B4SOIrth0(0.0),
    B4SOIcth0(0.0),
    B4SOIsoiMod(0),
    B4SOInbc(0.0),
    B4SOInseg(0.0),
    B4SOIagbcp(0.0),
    B4SOIagbcp2(0.0),
    B4SOIagbcpd(0.0),
    B4SOIaebcp(0.0),
    B4SOIrgateMod(0),
    B4SOIrbodyMod(0),
    B4SOImtrlMod(0),
    B4SOIvgstcvMod(0),
    B4SOIgidlMod(0),
    B4SOIiiiMod(0),
    B4SOIeot(0.0),
    B4SOIepsrsub(0.0),
    B4SOIni0sub(0.0),
    B4SOIbg0sub(0.0),
    B4SOItbgasub(0.0),
    B4SOItbgbsub(0.0),
    B4SOIphig(0.0),
    B4SOIeasub(0.0),
    B4SOIleffeot(0.0),
    B4SOIweffeot(0.0),
    B4SOIados(0.0),
    B4SOIbdos(0.0),
    B4SOIepsrgate(0.0),
    B4SOIcapMod(0),
    B4SOImobMod(0),
    B4SOItox(0.0),
    B4SOItoxp(0.0),
    B4SOItoxm(0.0),
    B4SOIdtoxcv(0.0),
    B4SOIngate(0.0),
    B4SOInsd(0.0),
    B4SOItnom(0.0),
    B4SOIxpart(0.0),
    B4SOIrswmin(0.0),
    B4SOIrdwmin(0.0),
    B4SOItbox(0.0),
    B4SOItsi(0.0),
    B4SOIegidl(0.0),
    B4SOIagidl(0.0),
    B4SOIbgidl(0.0),
    B4SOIcgidl(0.0),
    B4SOIegisl(0.0),
    B4SOIagisl(0.0),
    B4SOIbgisl(0.0),
    B4SOIcgisl(0.0),
    B4SOIndiode(0.0),
    B4SOIndioded(0.0),
    B4SOIGatesidewallJctSPotential(0.0),
    B4SOIGatesidewallJctDPotential(0.0),
    B4SOIbodyJctGateSideSGradingCoeff(0.0),
    B4SOIbodyJctGateSideDGradingCoeff(0.0),
    B4SOIunitLengthGateSidewallJctCapS(0.0),
    B4SOIunitLengthGateSidewallJctCapD(0.0),
    B4SOIoxideTrapDensityA(0.0),
    B4SOIoxideTrapDensityB(0.0),
    B4SOIoxideTrapDensityC(0.0),
    B4SOItnoia(0.0),
    B4SOItnoib(0.0),
    B4SOIrnoia(0.0),
    B4SOIrnoib(0.0),
    B4SOIntnoi(0.0),
    B4SOIsaref(0.0),
    B4SOIsbref(0.0),
    B4SOIku0(0.0),
    B4SOIkvsat(0.0),
    B4SOItku0(0.0),
    B4SOIlodk2(0.0),
    B4SOIlodeta0(0.0),
    B4SOIlodeta0cv(0.0),
    B4SOIbf(0.0),
    B4SOIw0flk(0.0),
    B4SOIpditsl(0.0),
    B4SOIem(0.0),
    B4SOIef(0.0),
    B4SOIaf(0.0),
    B4SOIkf(0.0),
    B4SOInoif(0.0),
    B4SOIk1w1(0.0),
    B4SOIk1w2(0.0),
    B4SOIketas(0.0),
    B4SOIdwbc(0.0),
    B4SOIbeta0(0.0),
    B4SOIbeta1(0.0),
    B4SOIbeta2(0.0),
    B4SOItii(0.0),
    B4SOIlii(0.0),
    B4SOIsii1(0.0),
    B4SOIsii2(0.0),
    B4SOIsiid(0.0),
    B4SOIfbjtii(0.0),
    B4SOItvbci(0.0),
    B4SOIesatii(0.0),
    B4SOIntun(0.0),
    B4SOIntund(0.0),
    B4SOIisbjt(0.0),
    B4SOIidbjt(0.0),
    B4SOIisdif(0.0),
    B4SOIiddif(0.0),
    B4SOIisrec(0.0),
    B4SOIidrec(0.0),
    B4SOIistun(0.0),
    B4SOIidtun(0.0),
    B4SOIvrec0(0.0),
    B4SOIvrec0d(0.0),
    B4SOIvtun0(0.0),
    B4SOIvtun0d(0.0),
    B4SOInbjt(0.0),
    B4SOIaely(0.0),
    B4SOIahli(0.0),
    B4SOIahlid(0.0),
    B4SOIrbody(0.0),
    B4SOIrbsh(0.0),
    B4SOItt(0.0),
    B4SOIcsdmin(0.0),
    B4SOIasd(0.0),
    B4SOIcsdesw(0.0),
    B4SOIfbody(0.0),
    B4SOItcjswg(0.0),
    B4SOItpbswg(0.0),
    B4SOItcjswgd(0.0),
    B4SOItpbswgd(0.0),
    B4SOIacde(0.0),
    B4SOImoin(0.0),
    B4SOIdlbg(0.0),
    B4SOIigbMod(0),
    B4SOIigcMod(0),
    B4SOItoxqm(0.0),
    B4SOIwth0(0.0),
    B4SOIrhalo(0.0),
    B4SOIntox(0.0),
    B4SOItoxref(0.0),
    B4SOIebg(0.0),
    B4SOIvevb(0.0),
    B4SOIvgb1(0.0),
    B4SOIvecb(0.0),
    B4SOIvgb2(0.0),
    B4SOIvoxh(0.0),
    B4SOIdeltavox(0.0),
    B4SOIrdsMod(0),
    B4SOIfdMod(0),
    B4SOIvtm(0.0),
d_B4SOIvtm_dTemp_t_GND(0.0),
d_B4SOIvtm_dV_p_GND(0.0),
d_B4SOIvtm_dV_b_GND(0.0),
    B4SOIueff(0.0),
    B4SOIthetavth(0.0),
d_B4SOIthetavth_dV_b_si(0.0),
d_B4SOIthetavth_dV_e_si(0.0),
d_B4SOIthetavth_dV_di_si(0.0),
d_B4SOIthetavth_dV_gi_si(0.0),
d_B4SOIthetavth_dTemp_t_GND(0.0),
d_B4SOIthetavth_dV_p_GND(0.0),
d_B4SOIthetavth_dV_b_GND(0.0),
    B4SOIInv_ODeff(0.0),
    B4SOIgrbsb(0.0),
    B4SOIgrbdb(0.0),
    B4SOITempSH(0.0),
    B4SOIgrgeltd(0.0),
    B4SOIsourceResistance(0.0),
    B4SOIdrainResistance(0.0),
    B4SOIvfbzb(0.0),
    B4SOIvtfbphi1(0.0),
    B4SOIvgsteffvth(0.0),
    B4SOIrbodyext(0.0),
    B4SOIcsesw(0.0),
    B4SOIcdesw(0.0),
    B4SOIcsbox(0.0),
    B4SOIcdbox(0.0),
    B4SOIcsmin(0.0),
    B4SOIcdmin(0.0),
    B4SOIst4(0.0),
    B4SOIdt4(0.0),
    B4SOInstar(0.0),
    B4SOIAbulk(0.0),
    B4SOIqinv(0.0),
    B4SOIgm(0.0),
    B4SOIgds(0.0),
    B4SOIgmbs(0.0),
    B4SOIrds(0.0),
    B4SOIVgsteff(0.0),
    B4SOIVdseff(0.0),
    B4SOIAbovVgst2Vtm(0.0),
    B4SOIidovVds(0.0),
    pParam_B4SOIcdsc(0.0),
    pParam_B4SOIcdscb(0.0),
    pParam_B4SOIcdscd(0.0),
    pParam_B4SOIcit(0.0),
    pParam_B4SOInfactor(0.0),
    pParam_B4SOIvsat(0.0),
    pParam_B4SOIat(0.0),
    pParam_B4SOIa0(0.0),
    pParam_B4SOIags(0.0),
    pParam_B4SOIa1(0.0),
    pParam_B4SOIa2(0.0),
    pParam_B4SOIketa(0.0),
    pParam_B4SOInpeak(0.0),
    pParam_B4SOInsub(0.0),
    pParam_B4SOIngate(0.0),
    pParam_B4SOIgamma1(0.0),
    pParam_B4SOIgamma2(0.0),
    pParam_B4SOIvbx(0.0),
d_pParam_B4SOIvbx_dTemp_t_GND(0.0),
d_pParam_B4SOIvbx_dV_p_GND(0.0),
d_pParam_B4SOIvbx_dV_b_GND(0.0),
    pParam_B4SOIvbi(0.0),
    pParam_B4SOIvbm(0.0),
    pParam_B4SOIxt(0.0),
    pParam_B4SOIphi(0.0),
    pParam_B4SOIlitl(0.0),
    pParam_B4SOIk1(0.0),
d_pParam_B4SOIk1_dTemp_t_GND(0.0),
d_pParam_B4SOIk1_dV_p_GND(0.0),
d_pParam_B4SOIk1_dV_b_GND(0.0),
    pParam_B4SOIkt1(0.0),
    pParam_B4SOIkt1l(0.0),
    pParam_B4SOIkt2(0.0),
    pParam_B4SOIk2(0.0),
    pParam_B4SOIk3(0.0),
    pParam_B4SOIk3b(0.0),
    pParam_B4SOIw0(0.0),
    pParam_B4SOIlpe0(0.0),
    pParam_B4SOIdvt0(0.0),
    pParam_B4SOIdvt1(0.0),
    pParam_B4SOIdvt2(0.0),
    pParam_B4SOIdvt0w(0.0),
    pParam_B4SOIdvt1w(0.0),
    pParam_B4SOIdvt2w(0.0),
    pParam_B4SOIdrout(0.0),
    pParam_B4SOIdsub(0.0),
    pParam_B4SOIua(0.0),
    pParam_B4SOIua1(0.0),
    pParam_B4SOIub(0.0),
    pParam_B4SOIub1(0.0),
    pParam_B4SOIuc(0.0),
    pParam_B4SOIuc1(0.0),
    pParam_B4SOIu0(0.0),
    pParam_B4SOIute(0.0),
    pParam_B4SOIud(0.0),
    pParam_B4SOIud1(0.0),
    pParam_B4SOIeu(0.0),
    pParam_B4SOIucs(0.0),
    pParam_B4SOIucste(0.0),
    pParam_B4SOIvoff(0.0),
    pParam_B4SOIvfb(0.0),
    pParam_B4SOIuatemp(0.0),
    pParam_B4SOIubtemp(0.0),
    pParam_B4SOIuctemp(0.0),
    pParam_B4SOIrbody(0.0),
    pParam_B4SOIrth(0.0),
    pParam_B4SOIcth(0.0),
    pParam_B4SOIrds0denom(0.0),
    pParam_B4SOIvfbb(0.0),
    pParam_B4SOIjbjts(0.0),
    pParam_B4SOIjbjtd(0.0),
    pParam_B4SOIjdifs(0.0),
    pParam_B4SOIjdifd(0.0),
    pParam_B4SOIjrecs(0.0),
    pParam_B4SOIjrecd(0.0),
    pParam_B4SOIjtuns(0.0),
    pParam_B4SOIjtund(0.0),
    pParam_B4SOIrdw(0.0),
    pParam_B4SOIrsw(0.0),
    pParam_B4SOIrdwmin(0.0),
    pParam_B4SOIrswmin(0.0),
    pParam_B4SOIrd0(0.0),
    pParam_B4SOIrs0(0.0),
    pParam_B4SOIsdt1(0.0),
    pParam_B4SOIst2(0.0),
    pParam_B4SOIst3(0.0),
    pParam_B4SOIdt2(0.0),
    pParam_B4SOIdt3(0.0),
    pParam_B4SOIdelta(0.0),
    pParam_B4SOIrdsw(0.0),
    pParam_B4SOIrds0(0.0),
    pParam_B4SOIprwg(0.0),
    pParam_B4SOIprwb(0.0),
    pParam_B4SOIprt(0.0),
    pParam_B4SOIeta0(0.0),
    pParam_B4SOIetab(0.0),
    pParam_B4SOIeta0cv(0.0),
    pParam_B4SOIetabcv(0.0),
    pParam_B4SOIpclm(0.0),
    pParam_B4SOIpdibl1(0.0),
    pParam_B4SOIpdibl2(0.0),
    pParam_B4SOIpdiblb(0.0),
    pParam_B4SOIpvag(0.0),
    pParam_B4SOIdwg(0.0),
    pParam_B4SOIdwb(0.0),
    pParam_B4SOIb0(0.0),
    pParam_B4SOIb1(0.0),
    pParam_B4SOIalpha0(0.0),
    pParam_B4SOIbeta0(0.0),
    pParam_B4SOIaigc(0.0),
    pParam_B4SOIbigc(0.0),
    pParam_B4SOIcigc(0.0),
    pParam_B4SOIaigsd(0.0),
    pParam_B4SOIbigsd(0.0),
    pParam_B4SOIcigsd(0.0),
    pParam_B4SOInigc(0.0),
    pParam_B4SOIpigcd(0.0),
    pParam_B4SOIpoxedge(0.0),
    pParam_B4SOIvbsa(0.0),
    pParam_B4SOIvsce(0.0),
    pParam_B4SOIcdsbs(0.0),
    pParam_B4SOInofffd(0.0),
    pParam_B4SOIvofffd(0.0),
    pParam_B4SOIk1b(0.0),
    pParam_B4SOIk2b(0.0),
    pParam_B4SOIdk2b(0.0),
    pParam_B4SOIdvbd0(0.0),
    pParam_B4SOIdvbd1(0.0),
    pParam_B4SOImoinfd(0.0),
    pParam_B4SOIxrcrg1(0.0),
    pParam_B4SOIxrcrg2(0.0),
    pParam_B4SOIku0(0.0),
    pParam_B4SOIinv_od_ref(0.0),
    pParam_B4SOIcgsl(0.0),
    pParam_B4SOIcgdl(0.0),
    pParam_B4SOIckappa(0.0),
    pParam_B4SOIclc(0.0),
    pParam_B4SOIxj(0.0),
    pParam_B4SOIalphaGB1(0.0),
    pParam_B4SOIbetaGB1(0.0),
    pParam_B4SOIalphaGB2(0.0),
    pParam_B4SOIbetaGB2(0.0),
    pParam_B4SOIaigbcp2(0.0),
    pParam_B4SOIbigbcp2(0.0),
    pParam_B4SOIcigbcp2(0.0),
    pParam_B4SOIntrecf(0.0),
    pParam_B4SOIntrecr(0.0),
    pParam_B4SOIxbjt(0.0),
    pParam_B4SOIxdif(0.0),
    pParam_B4SOIxrec(0.0),
    pParam_B4SOIxtun(0.0),
    pParam_B4SOIxdifd(0.0),
    pParam_B4SOIxrecd(0.0),
    pParam_B4SOIxtund(0.0),
    pParam_B4SOIkb1(0.0),
    pParam_B4SOIk1w1(0.0),
    pParam_B4SOIk1w2(0.0),
    pParam_B4SOIketas(0.0),
    pParam_B4SOIfbjtii(0.0),
    pParam_B4SOIebjtii(0.0),
    pParam_B4SOIcbjtii(0.0),
    pParam_B4SOIvbci(0.0),
    pParam_B4SOIabjtii(0.0),
    pParam_B4SOImbjtii(0.0),
    pParam_B4SOIbeta1(0.0),
    pParam_B4SOIbeta2(0.0),
    pParam_B4SOIvdsatii0(0.0),
    pParam_B4SOIlii(0.0),
    pParam_B4SOIesatii(0.0),
    pParam_B4SOIsii0(0.0),
    pParam_B4SOIsii1(0.0),
    pParam_B4SOIsii2(0.0),
    pParam_B4SOIsiid(0.0),
    pParam_B4SOIagidl(0.0),
    pParam_B4SOIbgidl(0.0),
    pParam_B4SOIcgidl(0.0),
    pParam_B4SOIegidl(0.0),
    pParam_B4SOIrgidl(0.0),
    pParam_B4SOIkgidl(0.0),
    pParam_B4SOIfgidl(0.0),
    pParam_B4SOIagisl(0.0),
    pParam_B4SOIbgisl(0.0),
    pParam_B4SOIcgisl(0.0),
    pParam_B4SOIegisl(0.0),
    pParam_B4SOIrgisl(0.0),
    pParam_B4SOIkgisl(0.0),
    pParam_B4SOIfgisl(0.0),
    pParam_B4SOIntun(0.0),
    pParam_B4SOIntund(0.0),
    pParam_B4SOIndiode(0.0),
    pParam_B4SOIndioded(0.0),
    pParam_B4SOInrecf0(0.0),
    pParam_B4SOInrecf0d(0.0),
    pParam_B4SOInrecr0(0.0),
    pParam_B4SOInrecr0d(0.0),
    pParam_B4SOIisbjt(0.0),
    pParam_B4SOIidbjt(0.0),
    pParam_B4SOIisdif(0.0),
    pParam_B4SOIiddif(0.0),
    pParam_B4SOIisrec(0.0),
    pParam_B4SOIidrec(0.0),
    pParam_B4SOIistun(0.0),
    pParam_B4SOIidtun(0.0),
    pParam_B4SOIvrec0(0.0),
    pParam_B4SOIvrec0d(0.0),
    pParam_B4SOIvtun0(0.0),
    pParam_B4SOIvtun0d(0.0),
    pParam_B4SOIvsdfb(0.0),
    pParam_B4SOIvsdth(0.0),
    pParam_B4SOIdelvt(0.0),
    pParam_B4SOIleff(0.0),
    pParam_B4SOIweff(0.0),
    pParam_B4SOIleffCV(0.0),
    pParam_B4SOIweffCV(0.0),
    pParam_B4SOIabulkCVfactor(0.0),
    pParam_B4SOIcgso(0.0),
    pParam_B4SOIcgdo(0.0),
    pParam_B4SOIcgeo(0.0),
    pParam_B4SOIu0temp(0.0),
    pParam_B4SOIvsattemp(0.0),
    pParam_B4SOIsqrtPhi(0.0),
    pParam_B4SOIXdep0(0.0),
    pParam_B4SOItheta0vb0(0.0),
    pParam_B4SOIthetaRout(0.0),
    pParam_B4SOIqsi(0.0),
    pParam_B4SOIoxideRatio(0.0),
    pParam_B4SOIk1eff(0.0),
d_pParam_B4SOIk1eff_dTemp_t_GND(0.0),
d_pParam_B4SOIk1eff_dV_p_GND(0.0),
d_pParam_B4SOIk1eff_dV_b_GND(0.0),
    pParam_B4SOIwdios(0.0),
    pParam_B4SOIwdiod(0.0),
    pParam_B4SOIwdiodCV(0.0),
    pParam_B4SOIwdiosCV(0.0),
    pParam_B4SOIarfabjt(0.0),
    pParam_B4SOIlratio(0.0),
    pParam_B4SOIlratiodif(0.0),
    pParam_B4SOIvearly(0.0),
    pParam_B4SOIahli(0.0),
    pParam_B4SOIahlid(0.0),
    pParam_B4SOIahli0s(0.0),
    pParam_B4SOIahli0d(0.0),
    pParam_B4SOIldeb(0.0),
    pParam_B4SOIacde(0.0),
    pParam_B4SOImoin(0.0),
    pParam_B4SOInoff(0.0),
    pParam_B4SOInoff2(0.0),
    pParam_B4SOIleffCVb(0.0),
    pParam_B4SOIleffCVbg(0.0),
    pParam_B4SOIcdep0(0.0),
    pParam_B4SOIAechvb(0.0),
    pParam_B4SOIBechvb(0.0),
    pParam_B4SOIAechvbEdges(0.0),
    pParam_B4SOIAechvbEdged(0.0),
    pParam_B4SOIBechvbEdge(0.0),
    pParam_B4SOIvfbsd(0.0),
    pParam_B4SOIk1ox(0.0),
    pParam_B4SOIlpeb(0.0),
    pParam_B4SOIdvtp0(0.0),
    pParam_B4SOIdvtp1(0.0),
    pParam_B4SOIdvtp4(0.0),
    pParam_B4SOImstar(0.0),
    pParam_B4SOIfprout(0.0),
    pParam_B4SOIpdits(0.0),
    pParam_B4SOIpditsd(0.0),
    pParam_B4SOImstarcv(0.0),
    pParam_B4SOIvoffcv(0.0),
    pParam_B4SOIdvtp2factor(0.0),
    B4SOIcbox(0.0),
    B4SOIcsi(0.0),
    B4SOIeg0(0.0),
    B4SOIeg(0.0),
    B4SOIcox(0.0),
    B4SOIfactor1(0.0),
    B4SOImode(0),
    B4SOIbodyMod(0),
    here_B4SOIeta0(0.0),
    here_B4SOIeta0cv(0.0),
    here_B4SOIk2(0.0),
d_here_B4SOIk2_dTemp_t_GND(0.0),
d_here_B4SOIk2_dV_p_GND(0.0),
d_here_B4SOIk2_dV_b_GND(0.0),
    here_B4SOIk2ox(0.0),
    here_B4SOIu0temp(0.0),
    here_B4SOIvfb(0.0),
d_here_B4SOIvfb_dTemp_t_GND(0.0),
d_here_B4SOIvfb_dV_p_GND(0.0),
d_here_B4SOIvfb_dV_b_GND(0.0),
    here_B4SOIvsattemp(0.0),
    here_B4SOIvth0(0.0),
d_here_B4SOIvth0_dTemp_t_GND(0.0),
d_here_B4SOIvth0_dV_p_GND(0.0),
d_here_B4SOIvth0_dV_b_GND(0.0),
    DevTemp(0.0),
d_DevTemp_dTemp_t_GND(0.0),
d_DevTemp_dV_p_GND(0.0),
d_DevTemp_dV_b_GND(0.0),
    fourkt(0.0),
    gdnoise(0.0),
    gsnoise(0.0),
    toxe(0.0),
    epsrox(0.0),
    epssub(0.0),
    sqrt2qeps(0.0),
    eggbcp2(0.0),
    eggdep(0.0),
    Vtm00(0.0),
    agb1(0.0),
    bgb1(0.0),
    agb2(0.0),
    bgb2(0.0),
    agbc2n(0.0),
    agbc2p(0.0),
    bgbc2n(0.0),
    bgbc2p(0.0),
    li_d(-1),
    li_g(-1),
    li_s(-1),
    li_e(-1),
    li_p(-1),
    li_b(-1),
    li_t(-1),
    li_di(-1),
    li_si(-1),
    li_gi(-1),
    li_gm(-1),
    li_sb(-1),
    li_db(-1),
    li_branch_id(-1),
    li_branch_ig(-1),
    li_branch_is(-1),
    li_branch_ie(-1),
    li_branch_ip(-1),
    li_branch_ib(-1),
    li_branch_it(-1),
    f_di_Equ_sb_Node_Ptr(0),
    f_di_Equ_si_Node_Ptr(0),
    f_si_Equ_sb_Node_Ptr(0),
    f_si_Equ_si_Node_Ptr(0),
    f_di_Equ_db_Node_Ptr(0),
    f_di_Equ_di_Node_Ptr(0),
    f_si_Equ_db_Node_Ptr(0),
    f_si_Equ_di_Node_Ptr(0),
    f_di_Equ_gi_Node_Ptr(0),
    f_di_Equ_p_Node_Ptr(0),
    f_si_Equ_gi_Node_Ptr(0),
    f_si_Equ_p_Node_Ptr(0),
    f_di_Equ_e_Node_Ptr(0),
    f_si_Equ_e_Node_Ptr(0),
    f_di_Equ_b_Node_Ptr(0),
    f_si_Equ_b_Node_Ptr(0),
    f_di_Equ_t_Node_Ptr(0),
    f_si_Equ_t_Node_Ptr(0),
    f_di_Equ_gm_Node_Ptr(0),
    f_si_Equ_gm_Node_Ptr(0),
    f_d_Equ_gi_Node_Ptr(0),
    f_d_Equ_si_Node_Ptr(0),
    f_d_Equ_di_Node_Ptr(0),
    f_d_Equ_e_Node_Ptr(0),
    f_d_Equ_b_Node_Ptr(0),
    f_d_Equ_sb_Node_Ptr(0),
    f_d_Equ_db_Node_Ptr(0),
    f_d_Equ_p_Node_Ptr(0),
    f_d_Equ_t_Node_Ptr(0),
    f_d_Equ_d_Node_Ptr(0),
    f_di_Equ_d_Node_Ptr(0),
    f_s_Equ_gi_Node_Ptr(0),
    f_s_Equ_si_Node_Ptr(0),
    f_s_Equ_di_Node_Ptr(0),
    f_s_Equ_e_Node_Ptr(0),
    f_s_Equ_b_Node_Ptr(0),
    f_s_Equ_sb_Node_Ptr(0),
    f_s_Equ_db_Node_Ptr(0),
    f_s_Equ_p_Node_Ptr(0),
    f_s_Equ_t_Node_Ptr(0),
    f_s_Equ_s_Node_Ptr(0),
    f_si_Equ_s_Node_Ptr(0),
    f_b_Equ_gi_Node_Ptr(0),
    f_b_Equ_si_Node_Ptr(0),
    f_b_Equ_e_Node_Ptr(0),
    f_b_Equ_b_Node_Ptr(0),
    f_b_Equ_sb_Node_Ptr(0),
    f_b_Equ_db_Node_Ptr(0),
    f_b_Equ_di_Node_Ptr(0),
    f_b_Equ_p_Node_Ptr(0),
    f_b_Equ_t_Node_Ptr(0),
    f_db_Equ_b_Node_Ptr(0),
    f_db_Equ_p_Node_Ptr(0),
    f_db_Equ_t_Node_Ptr(0),
    f_db_Equ_e_Node_Ptr(0),
    f_db_Equ_si_Node_Ptr(0),
    f_db_Equ_gi_Node_Ptr(0),
    f_db_Equ_di_Node_Ptr(0),
    f_db_Equ_sb_Node_Ptr(0),
    f_db_Equ_db_Node_Ptr(0),
    f_sb_Equ_b_Node_Ptr(0),
    f_sb_Equ_p_Node_Ptr(0),
    f_sb_Equ_t_Node_Ptr(0),
    f_sb_Equ_e_Node_Ptr(0),
    f_sb_Equ_si_Node_Ptr(0),
    f_sb_Equ_gi_Node_Ptr(0),
    f_sb_Equ_di_Node_Ptr(0),
    f_sb_Equ_sb_Node_Ptr(0),
    f_sb_Equ_db_Node_Ptr(0),
    f_gi_Equ_sb_Node_Ptr(0),
    f_gi_Equ_si_Node_Ptr(0),
    f_gi_Equ_db_Node_Ptr(0),
    f_gi_Equ_di_Node_Ptr(0),
    f_gi_Equ_b_Node_Ptr(0),
    f_gi_Equ_p_Node_Ptr(0),
    f_gi_Equ_t_Node_Ptr(0),
    f_gi_Equ_e_Node_Ptr(0),
    f_gi_Equ_gi_Node_Ptr(0),
    f_p_Equ_gi_Node_Ptr(0),
    f_p_Equ_p_Node_Ptr(0),
    f_p_Equ_b_Node_Ptr(0),
    f_p_Equ_t_Node_Ptr(0),
    f_p_Equ_si_Node_Ptr(0),
    f_p_Equ_di_Node_Ptr(0),
    f_p_Equ_e_Node_Ptr(0),
    f_p_Equ_sb_Node_Ptr(0),
    f_p_Equ_db_Node_Ptr(0),
    f_gi_Equ_gm_Node_Ptr(0),
    f_b_Equ_gm_Node_Ptr(0),
    f_e_Equ_b_Node_Ptr(0),
    f_e_Equ_si_Node_Ptr(0),
    f_e_Equ_gi_Node_Ptr(0),
    f_e_Equ_p_Node_Ptr(0),
    f_e_Equ_t_Node_Ptr(0),
    f_e_Equ_e_Node_Ptr(0),
    f_e_Equ_di_Node_Ptr(0),
    f_gm_Equ_gm_Node_Ptr(0),
    f_gm_Equ_si_Node_Ptr(0),
    f_gm_Equ_b_Node_Ptr(0),
    f_gm_Equ_p_Node_Ptr(0),
    f_gm_Equ_t_Node_Ptr(0),
    f_gm_Equ_gi_Node_Ptr(0),
    f_gm_Equ_di_Node_Ptr(0),
    f_gm_Equ_e_Node_Ptr(0),
    f_gm_Equ_sb_Node_Ptr(0),
    f_gm_Equ_db_Node_Ptr(0),
    f_e_Equ_gm_Node_Ptr(0),
    f_e_Equ_sb_Node_Ptr(0),
    f_e_Equ_db_Node_Ptr(0),
    f_g_Equ_g_Node_Ptr(0),
    f_g_Equ_gm_Node_Ptr(0),
    f_gm_Equ_g_Node_Ptr(0),
    f_g_Equ_b_Node_Ptr(0),
    f_g_Equ_p_Node_Ptr(0),
    f_g_Equ_t_Node_Ptr(0),
    f_t_Equ_gi_Node_Ptr(0),
    f_t_Equ_si_Node_Ptr(0),
    f_t_Equ_di_Node_Ptr(0),
    f_t_Equ_e_Node_Ptr(0),
    f_t_Equ_b_Node_Ptr(0),
    f_t_Equ_p_Node_Ptr(0),
    f_t_Equ_t_Node_Ptr(0),
    q_di_Equ_sb_Node_Ptr(0),
    q_di_Equ_si_Node_Ptr(0),
    q_si_Equ_sb_Node_Ptr(0),
    q_si_Equ_si_Node_Ptr(0),
    q_di_Equ_db_Node_Ptr(0),
    q_di_Equ_di_Node_Ptr(0),
    q_si_Equ_db_Node_Ptr(0),
    q_si_Equ_di_Node_Ptr(0),
    q_di_Equ_gi_Node_Ptr(0),
    q_di_Equ_p_Node_Ptr(0),
    q_si_Equ_gi_Node_Ptr(0),
    q_si_Equ_p_Node_Ptr(0),
    q_di_Equ_e_Node_Ptr(0),
    q_si_Equ_e_Node_Ptr(0),
    q_di_Equ_b_Node_Ptr(0),
    q_si_Equ_b_Node_Ptr(0),
    q_di_Equ_t_Node_Ptr(0),
    q_si_Equ_t_Node_Ptr(0),
    q_di_Equ_gm_Node_Ptr(0),
    q_si_Equ_gm_Node_Ptr(0),
    q_d_Equ_gi_Node_Ptr(0),
    q_d_Equ_si_Node_Ptr(0),
    q_d_Equ_di_Node_Ptr(0),
    q_d_Equ_e_Node_Ptr(0),
    q_d_Equ_b_Node_Ptr(0),
    q_d_Equ_sb_Node_Ptr(0),
    q_d_Equ_db_Node_Ptr(0),
    q_d_Equ_p_Node_Ptr(0),
    q_d_Equ_t_Node_Ptr(0),
    q_d_Equ_d_Node_Ptr(0),
    q_di_Equ_d_Node_Ptr(0),
    q_s_Equ_gi_Node_Ptr(0),
    q_s_Equ_si_Node_Ptr(0),
    q_s_Equ_di_Node_Ptr(0),
    q_s_Equ_e_Node_Ptr(0),
    q_s_Equ_b_Node_Ptr(0),
    q_s_Equ_sb_Node_Ptr(0),
    q_s_Equ_db_Node_Ptr(0),
    q_s_Equ_p_Node_Ptr(0),
    q_s_Equ_t_Node_Ptr(0),
    q_s_Equ_s_Node_Ptr(0),
    q_si_Equ_s_Node_Ptr(0),
    q_b_Equ_gi_Node_Ptr(0),
    q_b_Equ_si_Node_Ptr(0),
    q_b_Equ_e_Node_Ptr(0),
    q_b_Equ_b_Node_Ptr(0),
    q_b_Equ_sb_Node_Ptr(0),
    q_b_Equ_db_Node_Ptr(0),
    q_b_Equ_di_Node_Ptr(0),
    q_b_Equ_p_Node_Ptr(0),
    q_b_Equ_t_Node_Ptr(0),
    q_db_Equ_b_Node_Ptr(0),
    q_db_Equ_p_Node_Ptr(0),
    q_db_Equ_t_Node_Ptr(0),
    q_db_Equ_e_Node_Ptr(0),
    q_db_Equ_si_Node_Ptr(0),
    q_db_Equ_gi_Node_Ptr(0),
    q_db_Equ_di_Node_Ptr(0),
    q_db_Equ_sb_Node_Ptr(0),
    q_db_Equ_db_Node_Ptr(0),
    q_sb_Equ_b_Node_Ptr(0),
    q_sb_Equ_p_Node_Ptr(0),
    q_sb_Equ_t_Node_Ptr(0),
    q_sb_Equ_e_Node_Ptr(0),
    q_sb_Equ_si_Node_Ptr(0),
    q_sb_Equ_gi_Node_Ptr(0),
    q_sb_Equ_di_Node_Ptr(0),
    q_sb_Equ_sb_Node_Ptr(0),
    q_sb_Equ_db_Node_Ptr(0),
    q_gi_Equ_sb_Node_Ptr(0),
    q_gi_Equ_si_Node_Ptr(0),
    q_gi_Equ_db_Node_Ptr(0),
    q_gi_Equ_di_Node_Ptr(0),
    q_gi_Equ_b_Node_Ptr(0),
    q_gi_Equ_p_Node_Ptr(0),
    q_gi_Equ_t_Node_Ptr(0),
    q_gi_Equ_e_Node_Ptr(0),
    q_gi_Equ_gi_Node_Ptr(0),
    q_p_Equ_gi_Node_Ptr(0),
    q_p_Equ_p_Node_Ptr(0),
    q_p_Equ_b_Node_Ptr(0),
    q_p_Equ_t_Node_Ptr(0),
    q_p_Equ_si_Node_Ptr(0),
    q_p_Equ_di_Node_Ptr(0),
    q_p_Equ_e_Node_Ptr(0),
    q_p_Equ_sb_Node_Ptr(0),
    q_p_Equ_db_Node_Ptr(0),
    q_gi_Equ_gm_Node_Ptr(0),
    q_b_Equ_gm_Node_Ptr(0),
    q_e_Equ_b_Node_Ptr(0),
    q_e_Equ_si_Node_Ptr(0),
    q_e_Equ_gi_Node_Ptr(0),
    q_e_Equ_p_Node_Ptr(0),
    q_e_Equ_t_Node_Ptr(0),
    q_e_Equ_e_Node_Ptr(0),
    q_e_Equ_di_Node_Ptr(0),
    q_gm_Equ_gm_Node_Ptr(0),
    q_gm_Equ_si_Node_Ptr(0),
    q_gm_Equ_b_Node_Ptr(0),
    q_gm_Equ_p_Node_Ptr(0),
    q_gm_Equ_t_Node_Ptr(0),
    q_gm_Equ_gi_Node_Ptr(0),
    q_gm_Equ_di_Node_Ptr(0),
    q_gm_Equ_e_Node_Ptr(0),
    q_gm_Equ_sb_Node_Ptr(0),
    q_gm_Equ_db_Node_Ptr(0),
    q_e_Equ_gm_Node_Ptr(0),
    q_e_Equ_sb_Node_Ptr(0),
    q_e_Equ_db_Node_Ptr(0),
    q_g_Equ_g_Node_Ptr(0),
    q_g_Equ_gm_Node_Ptr(0),
    q_gm_Equ_g_Node_Ptr(0),
    q_g_Equ_b_Node_Ptr(0),
    q_g_Equ_p_Node_Ptr(0),
    q_g_Equ_t_Node_Ptr(0),
    q_t_Equ_gi_Node_Ptr(0),
    q_t_Equ_si_Node_Ptr(0),
    q_t_Equ_di_Node_Ptr(0),
    q_t_Equ_e_Node_Ptr(0),
    q_t_Equ_b_Node_Ptr(0),
    q_t_Equ_p_Node_Ptr(0),
    q_t_Equ_t_Node_Ptr(0),
    A_di_Equ_sb_NodeOffset(-1),
    A_di_Equ_si_NodeOffset(-1),
    A_si_Equ_sb_NodeOffset(-1),
    A_si_Equ_si_NodeOffset(-1),
    A_di_Equ_db_NodeOffset(-1),
    A_di_Equ_di_NodeOffset(-1),
    A_si_Equ_db_NodeOffset(-1),
    A_si_Equ_di_NodeOffset(-1),
    A_di_Equ_gi_NodeOffset(-1),
    A_di_Equ_p_NodeOffset(-1),
    A_si_Equ_gi_NodeOffset(-1),
    A_si_Equ_p_NodeOffset(-1),
    A_di_Equ_e_NodeOffset(-1),
    A_si_Equ_e_NodeOffset(-1),
    A_di_Equ_b_NodeOffset(-1),
    A_si_Equ_b_NodeOffset(-1),
    A_di_Equ_t_NodeOffset(-1),
    A_si_Equ_t_NodeOffset(-1),
    A_di_Equ_gm_NodeOffset(-1),
    A_si_Equ_gm_NodeOffset(-1),
    A_d_Equ_gi_NodeOffset(-1),
    A_d_Equ_si_NodeOffset(-1),
    A_d_Equ_di_NodeOffset(-1),
    A_d_Equ_e_NodeOffset(-1),
    A_d_Equ_b_NodeOffset(-1),
    A_d_Equ_sb_NodeOffset(-1),
    A_d_Equ_db_NodeOffset(-1),
    A_d_Equ_p_NodeOffset(-1),
    A_d_Equ_t_NodeOffset(-1),
    A_d_Equ_d_NodeOffset(-1),
    A_di_Equ_d_NodeOffset(-1),
    A_s_Equ_gi_NodeOffset(-1),
    A_s_Equ_si_NodeOffset(-1),
    A_s_Equ_di_NodeOffset(-1),
    A_s_Equ_e_NodeOffset(-1),
    A_s_Equ_b_NodeOffset(-1),
    A_s_Equ_sb_NodeOffset(-1),
    A_s_Equ_db_NodeOffset(-1),
    A_s_Equ_p_NodeOffset(-1),
    A_s_Equ_t_NodeOffset(-1),
    A_s_Equ_s_NodeOffset(-1),
    A_si_Equ_s_NodeOffset(-1),
    A_b_Equ_gi_NodeOffset(-1),
    A_b_Equ_si_NodeOffset(-1),
    A_b_Equ_e_NodeOffset(-1),
    A_b_Equ_b_NodeOffset(-1),
    A_b_Equ_sb_NodeOffset(-1),
    A_b_Equ_db_NodeOffset(-1),
    A_b_Equ_di_NodeOffset(-1),
    A_b_Equ_p_NodeOffset(-1),
    A_b_Equ_t_NodeOffset(-1),
    A_db_Equ_b_NodeOffset(-1),
    A_db_Equ_p_NodeOffset(-1),
    A_db_Equ_t_NodeOffset(-1),
    A_db_Equ_e_NodeOffset(-1),
    A_db_Equ_si_NodeOffset(-1),
    A_db_Equ_gi_NodeOffset(-1),
    A_db_Equ_di_NodeOffset(-1),
    A_db_Equ_sb_NodeOffset(-1),
    A_db_Equ_db_NodeOffset(-1),
    A_sb_Equ_b_NodeOffset(-1),
    A_sb_Equ_p_NodeOffset(-1),
    A_sb_Equ_t_NodeOffset(-1),
    A_sb_Equ_e_NodeOffset(-1),
    A_sb_Equ_si_NodeOffset(-1),
    A_sb_Equ_gi_NodeOffset(-1),
    A_sb_Equ_di_NodeOffset(-1),
    A_sb_Equ_sb_NodeOffset(-1),
    A_sb_Equ_db_NodeOffset(-1),
    A_gi_Equ_sb_NodeOffset(-1),
    A_gi_Equ_si_NodeOffset(-1),
    A_gi_Equ_db_NodeOffset(-1),
    A_gi_Equ_di_NodeOffset(-1),
    A_gi_Equ_b_NodeOffset(-1),
    A_gi_Equ_p_NodeOffset(-1),
    A_gi_Equ_t_NodeOffset(-1),
    A_gi_Equ_e_NodeOffset(-1),
    A_gi_Equ_gi_NodeOffset(-1),
    A_p_Equ_gi_NodeOffset(-1),
    A_p_Equ_p_NodeOffset(-1),
    A_p_Equ_b_NodeOffset(-1),
    A_p_Equ_t_NodeOffset(-1),
    A_p_Equ_si_NodeOffset(-1),
    A_p_Equ_di_NodeOffset(-1),
    A_p_Equ_e_NodeOffset(-1),
    A_p_Equ_sb_NodeOffset(-1),
    A_p_Equ_db_NodeOffset(-1),
    A_gi_Equ_gm_NodeOffset(-1),
    A_b_Equ_gm_NodeOffset(-1),
    A_e_Equ_b_NodeOffset(-1),
    A_e_Equ_si_NodeOffset(-1),
    A_e_Equ_gi_NodeOffset(-1),
    A_e_Equ_p_NodeOffset(-1),
    A_e_Equ_t_NodeOffset(-1),
    A_e_Equ_e_NodeOffset(-1),
    A_e_Equ_di_NodeOffset(-1),
    A_gm_Equ_gm_NodeOffset(-1),
    A_gm_Equ_si_NodeOffset(-1),
    A_gm_Equ_b_NodeOffset(-1),
    A_gm_Equ_p_NodeOffset(-1),
    A_gm_Equ_t_NodeOffset(-1),
    A_gm_Equ_gi_NodeOffset(-1),
    A_gm_Equ_di_NodeOffset(-1),
    A_gm_Equ_e_NodeOffset(-1),
    A_gm_Equ_sb_NodeOffset(-1),
    A_gm_Equ_db_NodeOffset(-1),
    A_e_Equ_gm_NodeOffset(-1),
    A_e_Equ_sb_NodeOffset(-1),
    A_e_Equ_db_NodeOffset(-1),
    A_g_Equ_g_NodeOffset(-1),
    A_g_Equ_gm_NodeOffset(-1),
    A_gm_Equ_g_NodeOffset(-1),
    A_g_Equ_b_NodeOffset(-1),
    A_g_Equ_p_NodeOffset(-1),
    A_g_Equ_t_NodeOffset(-1),
    A_t_Equ_gi_NodeOffset(-1),
    A_t_Equ_si_NodeOffset(-1),
    A_t_Equ_di_NodeOffset(-1),
    A_t_Equ_e_NodeOffset(-1),
    A_t_Equ_b_NodeOffset(-1),
    A_t_Equ_p_NodeOffset(-1),
    A_t_Equ_t_NodeOffset(-1),
    li_store_VDSAT(-1),
    li_store_VTH(-1),
    li_store_IDS(-1),
    li_store_GM(-1),
    li_store_GDS(-1),
    li_store_GMBS(-1),
    li_store_IC(-1),
    li_store_IBD(-1),
    li_store_IBS(-1),
    li_store_IGIDL(-1),
    li_store_IGISL(-1),
    li_store_IGS(-1),
    li_store_IGD(-1),
    li_store_IGB(-1),
    li_store_IGCS(-1),
    li_store_IGCD(-1),
    li_store_CGG(-1),
    li_store_CGS(-1),
    li_store_CGD(-1),
    li_store_CBG(-1),
    li_store_CBS(-1),
    li_store_CBD(-1),
    li_store_CDG(-1),
    li_store_CDD(-1),
    li_store_CDS(-1),
    li_store_CAPBD(-1),
    li_store_CAPBS(-1),
    li_store_QG(-1),
    li_store_QB(-1),
    li_store_QD(-1),
    li_store_QS(-1),
    li_store_QJD(-1),
    li_store_QJS(-1),
    li_store_T_TOTAL_K(-1),
    li_store_T_TOTAL_C(-1),
    li_store_T_DELTA_SH(-1),
    collapseNode_p(false),
    collapseNode_b(false),
    collapseNode_t(false),
    collapseNode_di(false),
    collapseNode_si(false),
    collapseNode_gi(false),
    collapseNode_gm(false),
    collapseNode_sb(false),
    collapseNode_db(false),
    admsTemperature(getDeviceOptions().temp.getImmutableValue<double>())

{
    numIntVars = 6 + 0;
    numExtVars = 7;

    portsConnected_.resize(7,true);
    if (instance_block.numExtVars < 7)
    {
       // some of our optional nodes were not given...
       for (int i=instance_block.numExtVars ; i<numExtVars; i++)
       {
         portsConnected_[i]=false;
       }
       numIntVars += (7-instance_block.numExtVars);
       numExtVars = instance_block.numExtVars;
     }
  // Right now, we only have store for limited probes and output vars...
  setNumStoreVars(0+36);

  // Do not allocate "branch" (lead current) vectors by default
  setNumBranchDataVars(0);
  numBranchDataVarsIfAllocated = std::min(7,numExtVars);

  leadCurrentF.resize(7);
  leadCurrentQ.resize(7);

  // Set up jacobian stamp:
  // Create a vector of the non-zero elements of the stamp
PairVector jacobianElements;
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_d,admsNodeID_d));
    jacobianElements.push_back(IntPair(admsNodeID_g,admsNodeID_g));
    jacobianElements.push_back(IntPair(admsNodeID_g,admsNodeID_gm));
    jacobianElements.push_back(IntPair(admsNodeID_g,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_g,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_g,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_s,admsNodeID_s));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_gm));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_e,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_p,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_b,admsNodeID_gm));
    jacobianElements.push_back(IntPair(admsNodeID_t,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_t,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_t,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_t,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_t,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_t,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_t,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_gm));
    jacobianElements.push_back(IntPair(admsNodeID_di,admsNodeID_d));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_gm));
    jacobianElements.push_back(IntPair(admsNodeID_si,admsNodeID_s));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_gi,admsNodeID_gm));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_gm));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_gm,admsNodeID_g));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_sb,admsNodeID_db));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_b));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_p));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_t));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_e));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_si));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_gi));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_di));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_sb));
    jacobianElements.push_back(IntPair(admsNodeID_db,admsNodeID_db));



  setDefaultParams();
  setParams(instance_block.params);

  // Real bogosity here...
  if (!given("XYCEADMSINSTTEMP"))
    admsInstTemp=getDeviceOptions().temp.getImmutableValue<double>();

  //calculate any parameters specified as expressions
  updateDependentParameters();

  // calculate dependent (i.e. computed params) and check for errors.
  processParams();

  PairVector collapsedNodes;
  collapseNodes();

  if (collapseNode_p)
  {
  //this node is optional, we only *literally* collapse it if it's not
  // connected
  if(!portsConnected_[admsNodeID_p])
  {
  collapsedNodes.push_back(IntPair(admsNodeID_p,admsNodeID_b));
  }
  else
  {
  UserWarning(*this) << "Collapsing optional external node p (node " << admsNodeID_p+1 << ") to b, but this is not possible, so we're not actually doing it.  This can lead to singular matrices and convergence problems.  Consider removing this optional node from the instance line if convergence problems result." << std::endl;
  }
  }
  if (collapseNode_b)
  {
  //this node is optional, we only *literally* collapse it if it's not
  // connected
  if(!portsConnected_[admsNodeID_b])
  {
  collapsedNodes.push_back(IntPair(admsNodeID_b,admsNodeID_si));
  }
  else
  {
  UserWarning(*this) << "Collapsing optional external node b (node " << admsNodeID_b+1 << ") to si, but this is not possible, so we're not actually doing it.  This can lead to singular matrices and convergence problems.  Consider removing this optional node from the instance line if convergence problems result." << std::endl;
  }
  }
  if (collapseNode_t)
  {
  //this node is optional, we only *literally* collapse it if it's not
  // connected
  if(!portsConnected_[admsNodeID_t])
  {
  collapsedNodes.push_back(IntPair(admsNodeID_t,admsNodeID_GND));
  }
  else
  {
  UserWarning(*this) << "Collapsing optional external node t (node " << admsNodeID_t+1 << ") to ground.  This can lead to singular matrices and convergence problems.  Consider removing this optional node from the instance line if convergence problems result." << std::endl;
  }
  }
  if (collapseNode_di)
  {
  collapsedNodes.push_back(IntPair(admsNodeID_di,admsNodeID_d));
  }
  if (collapseNode_si)
  {
  collapsedNodes.push_back(IntPair(admsNodeID_si,admsNodeID_s));
  }
  if (collapseNode_gi)
  {
  collapsedNodes.push_back(IntPair(admsNodeID_gi,admsNodeID_gm));
  }
  if (collapseNode_gm)
  {
  collapsedNodes.push_back(IntPair(admsNodeID_gm,admsNodeID_g));
  }
  if (collapseNode_sb)
  {
  collapsedNodes.push_back(IntPair(admsNodeID_sb,admsNodeID_b));
  }
  if (collapseNode_db)
  {
  collapsedNodes.push_back(IntPair(admsNodeID_db,admsNodeID_b));
  }

// Now generate the jacstamp from what we already have.
// This jacstamp will have all the correct node mapping.  map will be the nodal mapping of original
// node IDs to row/column ids in the reduced (non-sparse) representation of the jacobian.
// (for devices that have no collapsibles, this will be static, so check that it hasn't already
// been filled in)
if (jacStamp.empty())
{
int originalSize = 13+0;
computeJacStampAndMaps(jacobianElements,collapsedNodes,jacStamp,nodeMap,pairToJacStampMap,originalSize);
}

}

//-----------------------------------------------------------------------------
// Function      : Instance::collapseNodes
// Purpose       : function to handle collapsing of nodes
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::collapseNodes()
{
collapseNode_p = false;
collapseNode_b = false;
collapseNode_t = false;
collapseNode_di = false;
collapseNode_si = false;
collapseNode_gi = false;
collapseNode_gm = false;
collapseNode_sb = false;
collapseNode_db = false;
// begin block named load

if (!((B4SOIrdsMod!=2)))
{

collapseNode_di = true;

collapseNode_si = true;

}

if (((B4SOIbodyMod==0)||(B4SOIbodyMod==2)))
{
collapseNode_p = true;

}

if ((((model_.RGATEMOD)==0)||((model_.RGATEMOD)==2)))
{

collapseNode_gm = true;

}

if ((((model_.RGATEMOD)==0)||((model_.RGATEMOD)==1)))
{

collapseNode_gi = true;

}

if (!(B4SOIrbodyMod))
{

collapseNode_db = true;

collapseNode_sb = true;

}

if ((B4SOIsoiMod==2))
{
collapseNode_b = true;

}

if (!(((SHMOD==1)&&(RTH0!=0.0))))
{

if (!((TNODEOUT&&(!portsConnected_[admsNodeID_t]))))
{

collapseNode_t = true;

}

}

// end block  named load


  if (collapseNode_p)
  {
  //this node is optional, and is external if it's connected --- so don't
  // decrement the internal node count!
  if(!portsConnected_[admsNodeID_p])
  {
    numIntVars--;
  }
  }
  if (collapseNode_b)
  {
  //this node is optional, and is external if it's connected --- so don't
  // decrement the internal node count!
  if(!portsConnected_[admsNodeID_b])
  {
    numIntVars--;
  }
  }
  if (collapseNode_t)
  {
  //this node is optional, and is external if it's connected --- so don't
  // decrement the internal node count!
  if(!portsConnected_[admsNodeID_t])
  {
    numIntVars--;
  }
  }
  if (collapseNode_di)
  {
    numIntVars--;
  }
  if (collapseNode_si)
  {
    numIntVars--;
  }
  if (collapseNode_gi)
  {
    numIntVars--;
  }
  if (collapseNode_gm)
  {
    numIntVars--;
  }
  if (collapseNode_sb)
  {
    numIntVars--;
  }
  if (collapseNode_db)
  {
    numIntVars--;
  }
}

//-----------------------------------------------------------------------------
// Function      : Instance::~Instance
// Purpose       : destructor
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
Instance::~Instance()
{
}

//-----------------------------------------------------------------------------
// Function      : Instance::registerLIDs
// Purpose       : function for registering, and setting up, local IDs.
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::registerLIDs( const LocalIdVector & intLIDVecRef,
                             const LocalIdVector & extLIDVecRef)
{
  AssertLIDs(intLIDVecRef.size() == numIntVars);
  AssertLIDs(extLIDVecRef.size() == numExtVars);

  LocalIdVector localLIDVec;

  if (DEBUG_DEVICE && isActive(Diag::DEVICE_PARAMETERS))
  {
    Xyce::dout() << std::endl << section_divider << std::endl
                 << "In ADMSbsimsoi::Instance::register LIDs\n\n"
                 << "name             = " << getName() << std::endl
                 << "number of internal variables: " << numIntVars << std::endl
                 << "number of external variables: " << numExtVars << std::endl;
  }

  // copy over the global ID lists into a local array.
  // The end result of this is an array of LIDs corresponding to all the
  // nodes we actually have, in the order that topology thinks of them
  intLIDVec = intLIDVecRef;
  extLIDVec = extLIDVecRef;
  localLIDVec.resize(numExtVars+numIntVars);
  for (int localNodeIndex=0;localNodeIndex<numExtVars;++localNodeIndex)
  {
    localLIDVec[localNodeIndex]=extLIDVec[localNodeIndex];
  }
  for (int localNodeIndex=numExtVars;localNodeIndex<numExtVars+numIntVars;++localNodeIndex)
  {
    localLIDVec[localNodeIndex]=intLIDVec[localNodeIndex-numExtVars];
    }

  // Now pull the LIDs for each of our nodes out of the local array.
  // Use the node mapping created by createJacStampAndMaps to handle
  // all the node collapse complications.
  li_d = localLIDVec[nodeMap[admsNodeID_d]];
  li_g = localLIDVec[nodeMap[admsNodeID_g]];
  li_s = localLIDVec[nodeMap[admsNodeID_s]];
  li_e = localLIDVec[nodeMap[admsNodeID_e]];
  li_p = localLIDVec[nodeMap[admsNodeID_p]];
  li_b = localLIDVec[nodeMap[admsNodeID_b]];
  if (collapseNode_t)
  {
    li_t = -1;
  }
  else
  {
  li_t = localLIDVec[nodeMap[admsNodeID_t]];
  }
  li_di = localLIDVec[nodeMap[admsNodeID_di]];
  li_si = localLIDVec[nodeMap[admsNodeID_si]];
  li_gi = localLIDVec[nodeMap[admsNodeID_gi]];
  li_gm = localLIDVec[nodeMap[admsNodeID_gm]];
  li_sb = localLIDVec[nodeMap[admsNodeID_sb]];
  li_db = localLIDVec[nodeMap[admsNodeID_db]];


  if (DEBUG_DEVICE && isActive(Diag::DEVICE_PARAMETERS))
  {
    Xyce::dout() << "\nSolution and RHS variables:\n";
      Xyce::dout() << "\nli_d = " << li_d << std::endl;
      Xyce::dout() << "\nli_g = " << li_g << std::endl;
      Xyce::dout() << "\nli_s = " << li_s << std::endl;
      Xyce::dout() << "\nli_e = " << li_e << std::endl;
      Xyce::dout() << "\nli_p = " << li_p << std::endl;
      Xyce::dout() << "\nli_b = " << li_b << std::endl;
      Xyce::dout() << "\nli_t = " << li_t << std::endl;
      Xyce::dout() << "\nli_di = " << li_di << std::endl;
      Xyce::dout() << "\nli_si = " << li_si << std::endl;
      Xyce::dout() << "\nli_gi = " << li_gi << std::endl;
      Xyce::dout() << "\nli_gm = " << li_gm << std::endl;
      Xyce::dout() << "\nli_sb = " << li_sb << std::endl;
      Xyce::dout() << "\nli_db = " << li_db << std::endl;

    Xyce::dout() << "\nEnd of ADMSbsimsoi::Instance::register LIDs\n";
    Xyce::dout() << section_divider << std::endl;
  }
}

//-----------------------------------------------------------------------------
// Function      : Instance::loadNodeSymbols
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::loadNodeSymbols(Util::SymbolTable &symbol_table) const
{
    if (!portsConnected_[admsNodeID_p])
    {
    if (!collapseNode_p)
    {
    addInternalNode(symbol_table, li_p, getName(), "p");
    }
    }
    if (!portsConnected_[admsNodeID_b])
    {
    if (!collapseNode_b)
    {
    addInternalNode(symbol_table, li_b, getName(), "b");
    }
    }
    if (!portsConnected_[admsNodeID_t])
    {
    if (!collapseNode_t)
    {
    addInternalNode(symbol_table, li_t, getName(), "t");
    }
    }
    if (!collapseNode_di)
    {
    addInternalNode(symbol_table, li_di, getName(), "di");
    }
    if (!collapseNode_si)
    {
    addInternalNode(symbol_table, li_si, getName(), "si");
    }
    if (!collapseNode_gi)
    {
    addInternalNode(symbol_table, li_gi, getName(), "gi");
    }
    if (!collapseNode_gm)
    {
    addInternalNode(symbol_table, li_gm, getName(), "gm");
    }
    if (!collapseNode_sb)
    {
    addInternalNode(symbol_table, li_sb, getName(), "sb");
    }
    if (!collapseNode_db)
    {
    addInternalNode(symbol_table, li_db, getName(), "db");
    }
    addStoreNode(symbol_table, li_store_VDSAT, getName().getEncodedName() + ":VDSAT");
    addStoreNode(symbol_table, li_store_VTH, getName().getEncodedName() + ":VTH");
    addStoreNode(symbol_table, li_store_IDS, getName().getEncodedName() + ":IDS");
    addStoreNode(symbol_table, li_store_GM, getName().getEncodedName() + ":GM");
    addStoreNode(symbol_table, li_store_GDS, getName().getEncodedName() + ":GDS");
    addStoreNode(symbol_table, li_store_GMBS, getName().getEncodedName() + ":GMBS");
    addStoreNode(symbol_table, li_store_IC, getName().getEncodedName() + ":IC");
    addStoreNode(symbol_table, li_store_IBD, getName().getEncodedName() + ":IBD");
    addStoreNode(symbol_table, li_store_IBS, getName().getEncodedName() + ":IBS");
    addStoreNode(symbol_table, li_store_IGIDL, getName().getEncodedName() + ":IGIDL");
    addStoreNode(symbol_table, li_store_IGISL, getName().getEncodedName() + ":IGISL");
    addStoreNode(symbol_table, li_store_IGS, getName().getEncodedName() + ":IGS");
    addStoreNode(symbol_table, li_store_IGD, getName().getEncodedName() + ":IGD");
    addStoreNode(symbol_table, li_store_IGB, getName().getEncodedName() + ":IGB");
    addStoreNode(symbol_table, li_store_IGCS, getName().getEncodedName() + ":IGCS");
    addStoreNode(symbol_table, li_store_IGCD, getName().getEncodedName() + ":IGCD");
    addStoreNode(symbol_table, li_store_CGG, getName().getEncodedName() + ":CGG");
    addStoreNode(symbol_table, li_store_CGS, getName().getEncodedName() + ":CGS");
    addStoreNode(symbol_table, li_store_CGD, getName().getEncodedName() + ":CGD");
    addStoreNode(symbol_table, li_store_CBG, getName().getEncodedName() + ":CBG");
    addStoreNode(symbol_table, li_store_CBS, getName().getEncodedName() + ":CBS");
    addStoreNode(symbol_table, li_store_CBD, getName().getEncodedName() + ":CBD");
    addStoreNode(symbol_table, li_store_CDG, getName().getEncodedName() + ":CDG");
    addStoreNode(symbol_table, li_store_CDD, getName().getEncodedName() + ":CDD");
    addStoreNode(symbol_table, li_store_CDS, getName().getEncodedName() + ":CDS");
    addStoreNode(symbol_table, li_store_CAPBD, getName().getEncodedName() + ":CAPBD");
    addStoreNode(symbol_table, li_store_CAPBS, getName().getEncodedName() + ":CAPBS");
    addStoreNode(symbol_table, li_store_QG, getName().getEncodedName() + ":QG");
    addStoreNode(symbol_table, li_store_QB, getName().getEncodedName() + ":QB");
    addStoreNode(symbol_table, li_store_QD, getName().getEncodedName() + ":QD");
    addStoreNode(symbol_table, li_store_QS, getName().getEncodedName() + ":QS");
    addStoreNode(symbol_table, li_store_QJD, getName().getEncodedName() + ":QJD");
    addStoreNode(symbol_table, li_store_QJS, getName().getEncodedName() + ":QJS");
    addStoreNode(symbol_table, li_store_T_TOTAL_K, getName().getEncodedName() + ":T_TOTAL_K");
    addStoreNode(symbol_table, li_store_T_TOTAL_C, getName().getEncodedName() + ":T_TOTAL_C");
    addStoreNode(symbol_table, li_store_T_DELTA_SH, getName().getEncodedName() + ":T_DELTA_SH");

  if (loadLeadCurrent)
  {
    addBranchDataNode( symbol_table, li_branch_id, getName(), "BRANCH_DD");
    addBranchDataNode( symbol_table, li_branch_ig, getName(), "BRANCH_DG");
    addBranchDataNode( symbol_table, li_branch_is, getName(), "BRANCH_DS");
    addBranchDataNode( symbol_table, li_branch_ie, getName(), "BRANCH_DE");
    if (portsConnected_[admsNodeID_p])
    {
    addBranchDataNode( symbol_table, li_branch_ip, getName(), "BRANCH_D5");
    }
    if (portsConnected_[admsNodeID_b])
    {
    addBranchDataNode( symbol_table, li_branch_ib, getName(), "BRANCH_DB");
    }
    if (portsConnected_[admsNodeID_t])
    {
    addBranchDataNode( symbol_table, li_branch_it, getName(), "BRANCH_D7");
    }


  }
}

//-----------------------------------------------------------------------------
// Function      : Instance::registerStoreLIDs
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::registerStoreLIDs( const LocalIdVector & stoLIDVecRef)
{
  AssertLIDs(stoLIDVecRef.size() == getNumStoreVars());

  int numSto = stoLIDVecRef.size();
  if (numSto > 0)
  {
    int i=0;
    stoLIDVec = stoLIDVecRef;

      li_store_VDSAT= stoLIDVec[i++];
      li_store_VTH= stoLIDVec[i++];
      li_store_IDS= stoLIDVec[i++];
      li_store_GM= stoLIDVec[i++];
      li_store_GDS= stoLIDVec[i++];
      li_store_GMBS= stoLIDVec[i++];
      li_store_IC= stoLIDVec[i++];
      li_store_IBD= stoLIDVec[i++];
      li_store_IBS= stoLIDVec[i++];
      li_store_IGIDL= stoLIDVec[i++];
      li_store_IGISL= stoLIDVec[i++];
      li_store_IGS= stoLIDVec[i++];
      li_store_IGD= stoLIDVec[i++];
      li_store_IGB= stoLIDVec[i++];
      li_store_IGCS= stoLIDVec[i++];
      li_store_IGCD= stoLIDVec[i++];
      li_store_CGG= stoLIDVec[i++];
      li_store_CGS= stoLIDVec[i++];
      li_store_CGD= stoLIDVec[i++];
      li_store_CBG= stoLIDVec[i++];
      li_store_CBS= stoLIDVec[i++];
      li_store_CBD= stoLIDVec[i++];
      li_store_CDG= stoLIDVec[i++];
      li_store_CDD= stoLIDVec[i++];
      li_store_CDS= stoLIDVec[i++];
      li_store_CAPBD= stoLIDVec[i++];
      li_store_CAPBS= stoLIDVec[i++];
      li_store_QG= stoLIDVec[i++];
      li_store_QB= stoLIDVec[i++];
      li_store_QD= stoLIDVec[i++];
      li_store_QS= stoLIDVec[i++];
      li_store_QJD= stoLIDVec[i++];
      li_store_QJS= stoLIDVec[i++];
      li_store_T_TOTAL_K= stoLIDVec[i++];
      li_store_T_TOTAL_C= stoLIDVec[i++];
      li_store_T_DELTA_SH= stoLIDVec[i++];
  }
}

//-----------------------------------------------------------------------------
// Function      : Instance::registerBranchDataLIDs
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : Eric Keiter, SNL
// Creation Date : 12/09/11
//-----------------------------------------------------------------------------
void Instance::registerBranchDataLIDs(const std::vector<int> & branchLIDVecRef)
{
  AssertLIDs(branchLIDVecRef.size() == getNumBranchDataVars());

  if (loadLeadCurrent)
  {
    int i = 0;
    li_branch_id = branchLIDVecRef[i++];
    li_branch_ig = branchLIDVecRef[i++];
    li_branch_is = branchLIDVecRef[i++];
    li_branch_ie = branchLIDVecRef[i++];
    if (portsConnected_[admsNodeID_p])
    {
    li_branch_ip = branchLIDVecRef[i++];
    }
    if (portsConnected_[admsNodeID_b])
    {
    li_branch_ib = branchLIDVecRef[i++];
    }
    if (portsConnected_[admsNodeID_t])
    {
    li_branch_it = branchLIDVecRef[i++];
    }

  }
}

//-----------------------------------------------------------------------------
// Function      : Instance::jacobianStamp
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
const JacobianStamp & Instance::jacobianStamp() const
{
    return jacStamp;
}

//-----------------------------------------------------------------------------
// Function      : Instance::registerJacLIDs
// Purpose       : Create Offset variables used for referencing jacobian
//                 elements
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::registerJacLIDs( const JacobianStamp & jacLIDVec)
{
  DeviceInstance::registerJacLIDs(jacLIDVec);  IntPair jacLoc;
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_gi)];
    A_d_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_si)];
    A_d_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_di)];
    A_d_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_e)];
    A_d_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_b)];
    A_d_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_sb)];
    A_d_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_db)];
    A_d_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_p)];
    A_d_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_t)];
    A_d_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_d,admsNodeID_d)];
    A_d_Equ_d_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_g,admsNodeID_g)];
    A_g_Equ_g_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_g,admsNodeID_gm)];
    A_g_Equ_gm_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_g,admsNodeID_b)];
    A_g_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_g,admsNodeID_p)];
    A_g_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_g,admsNodeID_t)];
    A_g_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_gi)];
    A_s_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_si)];
    A_s_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_di)];
    A_s_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_e)];
    A_s_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_b)];
    A_s_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_sb)];
    A_s_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_db)];
    A_s_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_p)];
    A_s_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_t)];
    A_s_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_s,admsNodeID_s)];
    A_s_Equ_s_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_b)];
    A_e_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_si)];
    A_e_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_gi)];
    A_e_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_p)];
    A_e_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_t)];
    A_e_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_e)];
    A_e_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_di)];
    A_e_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_gm)];
    A_e_Equ_gm_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_sb)];
    A_e_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_e,admsNodeID_db)];
    A_e_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_gi)];
    A_p_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_p)];
    A_p_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_b)];
    A_p_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_t)];
    A_p_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_si)];
    A_p_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_di)];
    A_p_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_e)];
    A_p_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_sb)];
    A_p_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_p,admsNodeID_db)];
    A_p_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_gi)];
    A_b_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_si)];
    A_b_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_e)];
    A_b_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_b)];
    A_b_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_sb)];
    A_b_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_db)];
    A_b_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_di)];
    A_b_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_p)];
    A_b_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_t)];
    A_b_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_b,admsNodeID_gm)];
    A_b_Equ_gm_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_t,admsNodeID_gi)];
    A_t_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_t,admsNodeID_si)];
    A_t_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_t,admsNodeID_di)];
    A_t_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_t,admsNodeID_e)];
    A_t_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_t,admsNodeID_b)];
    A_t_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_t,admsNodeID_p)];
    A_t_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_t,admsNodeID_t)];
    A_t_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_sb)];
    A_di_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_si)];
    A_di_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_db)];
    A_di_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_di)];
    A_di_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_gi)];
    A_di_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_p)];
    A_di_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_e)];
    A_di_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_b)];
    A_di_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_t)];
    A_di_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_gm)];
    A_di_Equ_gm_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_di,admsNodeID_d)];
    A_di_Equ_d_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_sb)];
    A_si_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_si)];
    A_si_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_db)];
    A_si_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_di)];
    A_si_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_gi)];
    A_si_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_p)];
    A_si_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_e)];
    A_si_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_b)];
    A_si_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_t)];
    A_si_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_gm)];
    A_si_Equ_gm_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_si,admsNodeID_s)];
    A_si_Equ_s_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_sb)];
    A_gi_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_si)];
    A_gi_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_db)];
    A_gi_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_di)];
    A_gi_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_b)];
    A_gi_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_p)];
    A_gi_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_t)];
    A_gi_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_e)];
    A_gi_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_gi)];
    A_gi_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gi,admsNodeID_gm)];
    A_gi_Equ_gm_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_gm)];
    A_gm_Equ_gm_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_si)];
    A_gm_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_b)];
    A_gm_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_p)];
    A_gm_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_t)];
    A_gm_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_gi)];
    A_gm_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_di)];
    A_gm_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_e)];
    A_gm_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_sb)];
    A_gm_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_db)];
    A_gm_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_gm,admsNodeID_g)];
    A_gm_Equ_g_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_b)];
    A_sb_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_p)];
    A_sb_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_t)];
    A_sb_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_e)];
    A_sb_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_si)];
    A_sb_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_gi)];
    A_sb_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_di)];
    A_sb_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_sb)];
    A_sb_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_sb,admsNodeID_db)];
    A_sb_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_b)];
    A_db_Equ_b_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_p)];
    A_db_Equ_p_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
if (!collapseNode_t)
{
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_t)];
    A_db_Equ_t_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
}
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_e)];
    A_db_Equ_e_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_si)];
    A_db_Equ_si_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_gi)];
    A_db_Equ_gi_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_di)];
    A_db_Equ_di_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_sb)];
    A_db_Equ_sb_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];
    jacLoc = pairToJacStampMap[IntPair(admsNodeID_db,admsNodeID_db)];
    A_db_Equ_db_NodeOffset = jacLIDVec[jacLoc.first][jacLoc.second];


}

//-----------------------------------------------------------------------------
// Function      : Instance::setupPointers
// Purpose       : Create pointer variables used for referencing jacobian
//                 elements
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::setupPointers( )
{

  Linear::Matrix & dFdx = *(extData.dFdxMatrixPtr);
  Linear::Matrix & dQdx = *(extData.dQdxMatrixPtr);
    f_d_Equ_gi_Node_Ptr = &(dFdx[li_d][A_d_Equ_gi_NodeOffset]);
    q_d_Equ_gi_Node_Ptr = &(dQdx[li_d][A_d_Equ_gi_NodeOffset]);
    f_d_Equ_si_Node_Ptr = &(dFdx[li_d][A_d_Equ_si_NodeOffset]);
    q_d_Equ_si_Node_Ptr = &(dQdx[li_d][A_d_Equ_si_NodeOffset]);
    f_d_Equ_di_Node_Ptr = &(dFdx[li_d][A_d_Equ_di_NodeOffset]);
    q_d_Equ_di_Node_Ptr = &(dQdx[li_d][A_d_Equ_di_NodeOffset]);
    f_d_Equ_e_Node_Ptr = &(dFdx[li_d][A_d_Equ_e_NodeOffset]);
    q_d_Equ_e_Node_Ptr = &(dQdx[li_d][A_d_Equ_e_NodeOffset]);
    f_d_Equ_b_Node_Ptr = &(dFdx[li_d][A_d_Equ_b_NodeOffset]);
    q_d_Equ_b_Node_Ptr = &(dQdx[li_d][A_d_Equ_b_NodeOffset]);
    f_d_Equ_sb_Node_Ptr = &(dFdx[li_d][A_d_Equ_sb_NodeOffset]);
    q_d_Equ_sb_Node_Ptr = &(dQdx[li_d][A_d_Equ_sb_NodeOffset]);
    f_d_Equ_db_Node_Ptr = &(dFdx[li_d][A_d_Equ_db_NodeOffset]);
    q_d_Equ_db_Node_Ptr = &(dQdx[li_d][A_d_Equ_db_NodeOffset]);
    f_d_Equ_p_Node_Ptr = &(dFdx[li_d][A_d_Equ_p_NodeOffset]);
    q_d_Equ_p_Node_Ptr = &(dQdx[li_d][A_d_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_d_Equ_t_Node_Ptr = &(dFdx[li_d][A_d_Equ_t_NodeOffset]);
    q_d_Equ_t_Node_Ptr = &(dQdx[li_d][A_d_Equ_t_NodeOffset]);
}
    f_d_Equ_d_Node_Ptr = &(dFdx[li_d][A_d_Equ_d_NodeOffset]);
    q_d_Equ_d_Node_Ptr = &(dQdx[li_d][A_d_Equ_d_NodeOffset]);
    f_g_Equ_g_Node_Ptr = &(dFdx[li_g][A_g_Equ_g_NodeOffset]);
    q_g_Equ_g_Node_Ptr = &(dQdx[li_g][A_g_Equ_g_NodeOffset]);
    f_g_Equ_gm_Node_Ptr = &(dFdx[li_g][A_g_Equ_gm_NodeOffset]);
    q_g_Equ_gm_Node_Ptr = &(dQdx[li_g][A_g_Equ_gm_NodeOffset]);
    f_g_Equ_b_Node_Ptr = &(dFdx[li_g][A_g_Equ_b_NodeOffset]);
    q_g_Equ_b_Node_Ptr = &(dQdx[li_g][A_g_Equ_b_NodeOffset]);
    f_g_Equ_p_Node_Ptr = &(dFdx[li_g][A_g_Equ_p_NodeOffset]);
    q_g_Equ_p_Node_Ptr = &(dQdx[li_g][A_g_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_g_Equ_t_Node_Ptr = &(dFdx[li_g][A_g_Equ_t_NodeOffset]);
    q_g_Equ_t_Node_Ptr = &(dQdx[li_g][A_g_Equ_t_NodeOffset]);
}
    f_s_Equ_gi_Node_Ptr = &(dFdx[li_s][A_s_Equ_gi_NodeOffset]);
    q_s_Equ_gi_Node_Ptr = &(dQdx[li_s][A_s_Equ_gi_NodeOffset]);
    f_s_Equ_si_Node_Ptr = &(dFdx[li_s][A_s_Equ_si_NodeOffset]);
    q_s_Equ_si_Node_Ptr = &(dQdx[li_s][A_s_Equ_si_NodeOffset]);
    f_s_Equ_di_Node_Ptr = &(dFdx[li_s][A_s_Equ_di_NodeOffset]);
    q_s_Equ_di_Node_Ptr = &(dQdx[li_s][A_s_Equ_di_NodeOffset]);
    f_s_Equ_e_Node_Ptr = &(dFdx[li_s][A_s_Equ_e_NodeOffset]);
    q_s_Equ_e_Node_Ptr = &(dQdx[li_s][A_s_Equ_e_NodeOffset]);
    f_s_Equ_b_Node_Ptr = &(dFdx[li_s][A_s_Equ_b_NodeOffset]);
    q_s_Equ_b_Node_Ptr = &(dQdx[li_s][A_s_Equ_b_NodeOffset]);
    f_s_Equ_sb_Node_Ptr = &(dFdx[li_s][A_s_Equ_sb_NodeOffset]);
    q_s_Equ_sb_Node_Ptr = &(dQdx[li_s][A_s_Equ_sb_NodeOffset]);
    f_s_Equ_db_Node_Ptr = &(dFdx[li_s][A_s_Equ_db_NodeOffset]);
    q_s_Equ_db_Node_Ptr = &(dQdx[li_s][A_s_Equ_db_NodeOffset]);
    f_s_Equ_p_Node_Ptr = &(dFdx[li_s][A_s_Equ_p_NodeOffset]);
    q_s_Equ_p_Node_Ptr = &(dQdx[li_s][A_s_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_s_Equ_t_Node_Ptr = &(dFdx[li_s][A_s_Equ_t_NodeOffset]);
    q_s_Equ_t_Node_Ptr = &(dQdx[li_s][A_s_Equ_t_NodeOffset]);
}
    f_s_Equ_s_Node_Ptr = &(dFdx[li_s][A_s_Equ_s_NodeOffset]);
    q_s_Equ_s_Node_Ptr = &(dQdx[li_s][A_s_Equ_s_NodeOffset]);
    f_e_Equ_b_Node_Ptr = &(dFdx[li_e][A_e_Equ_b_NodeOffset]);
    q_e_Equ_b_Node_Ptr = &(dQdx[li_e][A_e_Equ_b_NodeOffset]);
    f_e_Equ_si_Node_Ptr = &(dFdx[li_e][A_e_Equ_si_NodeOffset]);
    q_e_Equ_si_Node_Ptr = &(dQdx[li_e][A_e_Equ_si_NodeOffset]);
    f_e_Equ_gi_Node_Ptr = &(dFdx[li_e][A_e_Equ_gi_NodeOffset]);
    q_e_Equ_gi_Node_Ptr = &(dQdx[li_e][A_e_Equ_gi_NodeOffset]);
    f_e_Equ_p_Node_Ptr = &(dFdx[li_e][A_e_Equ_p_NodeOffset]);
    q_e_Equ_p_Node_Ptr = &(dQdx[li_e][A_e_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_e_Equ_t_Node_Ptr = &(dFdx[li_e][A_e_Equ_t_NodeOffset]);
    q_e_Equ_t_Node_Ptr = &(dQdx[li_e][A_e_Equ_t_NodeOffset]);
}
    f_e_Equ_e_Node_Ptr = &(dFdx[li_e][A_e_Equ_e_NodeOffset]);
    q_e_Equ_e_Node_Ptr = &(dQdx[li_e][A_e_Equ_e_NodeOffset]);
    f_e_Equ_di_Node_Ptr = &(dFdx[li_e][A_e_Equ_di_NodeOffset]);
    q_e_Equ_di_Node_Ptr = &(dQdx[li_e][A_e_Equ_di_NodeOffset]);
    f_e_Equ_gm_Node_Ptr = &(dFdx[li_e][A_e_Equ_gm_NodeOffset]);
    q_e_Equ_gm_Node_Ptr = &(dQdx[li_e][A_e_Equ_gm_NodeOffset]);
    f_e_Equ_sb_Node_Ptr = &(dFdx[li_e][A_e_Equ_sb_NodeOffset]);
    q_e_Equ_sb_Node_Ptr = &(dQdx[li_e][A_e_Equ_sb_NodeOffset]);
    f_e_Equ_db_Node_Ptr = &(dFdx[li_e][A_e_Equ_db_NodeOffset]);
    q_e_Equ_db_Node_Ptr = &(dQdx[li_e][A_e_Equ_db_NodeOffset]);
    f_p_Equ_gi_Node_Ptr = &(dFdx[li_p][A_p_Equ_gi_NodeOffset]);
    q_p_Equ_gi_Node_Ptr = &(dQdx[li_p][A_p_Equ_gi_NodeOffset]);
    f_p_Equ_p_Node_Ptr = &(dFdx[li_p][A_p_Equ_p_NodeOffset]);
    q_p_Equ_p_Node_Ptr = &(dQdx[li_p][A_p_Equ_p_NodeOffset]);
    f_p_Equ_b_Node_Ptr = &(dFdx[li_p][A_p_Equ_b_NodeOffset]);
    q_p_Equ_b_Node_Ptr = &(dQdx[li_p][A_p_Equ_b_NodeOffset]);
if (!collapseNode_t)
{
    f_p_Equ_t_Node_Ptr = &(dFdx[li_p][A_p_Equ_t_NodeOffset]);
    q_p_Equ_t_Node_Ptr = &(dQdx[li_p][A_p_Equ_t_NodeOffset]);
}
    f_p_Equ_si_Node_Ptr = &(dFdx[li_p][A_p_Equ_si_NodeOffset]);
    q_p_Equ_si_Node_Ptr = &(dQdx[li_p][A_p_Equ_si_NodeOffset]);
    f_p_Equ_di_Node_Ptr = &(dFdx[li_p][A_p_Equ_di_NodeOffset]);
    q_p_Equ_di_Node_Ptr = &(dQdx[li_p][A_p_Equ_di_NodeOffset]);
    f_p_Equ_e_Node_Ptr = &(dFdx[li_p][A_p_Equ_e_NodeOffset]);
    q_p_Equ_e_Node_Ptr = &(dQdx[li_p][A_p_Equ_e_NodeOffset]);
    f_p_Equ_sb_Node_Ptr = &(dFdx[li_p][A_p_Equ_sb_NodeOffset]);
    q_p_Equ_sb_Node_Ptr = &(dQdx[li_p][A_p_Equ_sb_NodeOffset]);
    f_p_Equ_db_Node_Ptr = &(dFdx[li_p][A_p_Equ_db_NodeOffset]);
    q_p_Equ_db_Node_Ptr = &(dQdx[li_p][A_p_Equ_db_NodeOffset]);
    f_b_Equ_gi_Node_Ptr = &(dFdx[li_b][A_b_Equ_gi_NodeOffset]);
    q_b_Equ_gi_Node_Ptr = &(dQdx[li_b][A_b_Equ_gi_NodeOffset]);
    f_b_Equ_si_Node_Ptr = &(dFdx[li_b][A_b_Equ_si_NodeOffset]);
    q_b_Equ_si_Node_Ptr = &(dQdx[li_b][A_b_Equ_si_NodeOffset]);
    f_b_Equ_e_Node_Ptr = &(dFdx[li_b][A_b_Equ_e_NodeOffset]);
    q_b_Equ_e_Node_Ptr = &(dQdx[li_b][A_b_Equ_e_NodeOffset]);
    f_b_Equ_b_Node_Ptr = &(dFdx[li_b][A_b_Equ_b_NodeOffset]);
    q_b_Equ_b_Node_Ptr = &(dQdx[li_b][A_b_Equ_b_NodeOffset]);
    f_b_Equ_sb_Node_Ptr = &(dFdx[li_b][A_b_Equ_sb_NodeOffset]);
    q_b_Equ_sb_Node_Ptr = &(dQdx[li_b][A_b_Equ_sb_NodeOffset]);
    f_b_Equ_db_Node_Ptr = &(dFdx[li_b][A_b_Equ_db_NodeOffset]);
    q_b_Equ_db_Node_Ptr = &(dQdx[li_b][A_b_Equ_db_NodeOffset]);
    f_b_Equ_di_Node_Ptr = &(dFdx[li_b][A_b_Equ_di_NodeOffset]);
    q_b_Equ_di_Node_Ptr = &(dQdx[li_b][A_b_Equ_di_NodeOffset]);
    f_b_Equ_p_Node_Ptr = &(dFdx[li_b][A_b_Equ_p_NodeOffset]);
    q_b_Equ_p_Node_Ptr = &(dQdx[li_b][A_b_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_b_Equ_t_Node_Ptr = &(dFdx[li_b][A_b_Equ_t_NodeOffset]);
    q_b_Equ_t_Node_Ptr = &(dQdx[li_b][A_b_Equ_t_NodeOffset]);
}
    f_b_Equ_gm_Node_Ptr = &(dFdx[li_b][A_b_Equ_gm_NodeOffset]);
    q_b_Equ_gm_Node_Ptr = &(dQdx[li_b][A_b_Equ_gm_NodeOffset]);
if (!collapseNode_t)
{
    f_t_Equ_gi_Node_Ptr = &(dFdx[li_t][A_t_Equ_gi_NodeOffset]);
    q_t_Equ_gi_Node_Ptr = &(dQdx[li_t][A_t_Equ_gi_NodeOffset]);
}
if (!collapseNode_t)
{
    f_t_Equ_si_Node_Ptr = &(dFdx[li_t][A_t_Equ_si_NodeOffset]);
    q_t_Equ_si_Node_Ptr = &(dQdx[li_t][A_t_Equ_si_NodeOffset]);
}
if (!collapseNode_t)
{
    f_t_Equ_di_Node_Ptr = &(dFdx[li_t][A_t_Equ_di_NodeOffset]);
    q_t_Equ_di_Node_Ptr = &(dQdx[li_t][A_t_Equ_di_NodeOffset]);
}
if (!collapseNode_t)
{
    f_t_Equ_e_Node_Ptr = &(dFdx[li_t][A_t_Equ_e_NodeOffset]);
    q_t_Equ_e_Node_Ptr = &(dQdx[li_t][A_t_Equ_e_NodeOffset]);
}
if (!collapseNode_t)
{
    f_t_Equ_b_Node_Ptr = &(dFdx[li_t][A_t_Equ_b_NodeOffset]);
    q_t_Equ_b_Node_Ptr = &(dQdx[li_t][A_t_Equ_b_NodeOffset]);
}
if (!collapseNode_t)
{
    f_t_Equ_p_Node_Ptr = &(dFdx[li_t][A_t_Equ_p_NodeOffset]);
    q_t_Equ_p_Node_Ptr = &(dQdx[li_t][A_t_Equ_p_NodeOffset]);
}
if (!collapseNode_t)
{
    f_t_Equ_t_Node_Ptr = &(dFdx[li_t][A_t_Equ_t_NodeOffset]);
    q_t_Equ_t_Node_Ptr = &(dQdx[li_t][A_t_Equ_t_NodeOffset]);
}
    f_di_Equ_sb_Node_Ptr = &(dFdx[li_di][A_di_Equ_sb_NodeOffset]);
    q_di_Equ_sb_Node_Ptr = &(dQdx[li_di][A_di_Equ_sb_NodeOffset]);
    f_di_Equ_si_Node_Ptr = &(dFdx[li_di][A_di_Equ_si_NodeOffset]);
    q_di_Equ_si_Node_Ptr = &(dQdx[li_di][A_di_Equ_si_NodeOffset]);
    f_di_Equ_db_Node_Ptr = &(dFdx[li_di][A_di_Equ_db_NodeOffset]);
    q_di_Equ_db_Node_Ptr = &(dQdx[li_di][A_di_Equ_db_NodeOffset]);
    f_di_Equ_di_Node_Ptr = &(dFdx[li_di][A_di_Equ_di_NodeOffset]);
    q_di_Equ_di_Node_Ptr = &(dQdx[li_di][A_di_Equ_di_NodeOffset]);
    f_di_Equ_gi_Node_Ptr = &(dFdx[li_di][A_di_Equ_gi_NodeOffset]);
    q_di_Equ_gi_Node_Ptr = &(dQdx[li_di][A_di_Equ_gi_NodeOffset]);
    f_di_Equ_p_Node_Ptr = &(dFdx[li_di][A_di_Equ_p_NodeOffset]);
    q_di_Equ_p_Node_Ptr = &(dQdx[li_di][A_di_Equ_p_NodeOffset]);
    f_di_Equ_e_Node_Ptr = &(dFdx[li_di][A_di_Equ_e_NodeOffset]);
    q_di_Equ_e_Node_Ptr = &(dQdx[li_di][A_di_Equ_e_NodeOffset]);
    f_di_Equ_b_Node_Ptr = &(dFdx[li_di][A_di_Equ_b_NodeOffset]);
    q_di_Equ_b_Node_Ptr = &(dQdx[li_di][A_di_Equ_b_NodeOffset]);
if (!collapseNode_t)
{
    f_di_Equ_t_Node_Ptr = &(dFdx[li_di][A_di_Equ_t_NodeOffset]);
    q_di_Equ_t_Node_Ptr = &(dQdx[li_di][A_di_Equ_t_NodeOffset]);
}
    f_di_Equ_gm_Node_Ptr = &(dFdx[li_di][A_di_Equ_gm_NodeOffset]);
    q_di_Equ_gm_Node_Ptr = &(dQdx[li_di][A_di_Equ_gm_NodeOffset]);
    f_di_Equ_d_Node_Ptr = &(dFdx[li_di][A_di_Equ_d_NodeOffset]);
    q_di_Equ_d_Node_Ptr = &(dQdx[li_di][A_di_Equ_d_NodeOffset]);
    f_si_Equ_sb_Node_Ptr = &(dFdx[li_si][A_si_Equ_sb_NodeOffset]);
    q_si_Equ_sb_Node_Ptr = &(dQdx[li_si][A_si_Equ_sb_NodeOffset]);
    f_si_Equ_si_Node_Ptr = &(dFdx[li_si][A_si_Equ_si_NodeOffset]);
    q_si_Equ_si_Node_Ptr = &(dQdx[li_si][A_si_Equ_si_NodeOffset]);
    f_si_Equ_db_Node_Ptr = &(dFdx[li_si][A_si_Equ_db_NodeOffset]);
    q_si_Equ_db_Node_Ptr = &(dQdx[li_si][A_si_Equ_db_NodeOffset]);
    f_si_Equ_di_Node_Ptr = &(dFdx[li_si][A_si_Equ_di_NodeOffset]);
    q_si_Equ_di_Node_Ptr = &(dQdx[li_si][A_si_Equ_di_NodeOffset]);
    f_si_Equ_gi_Node_Ptr = &(dFdx[li_si][A_si_Equ_gi_NodeOffset]);
    q_si_Equ_gi_Node_Ptr = &(dQdx[li_si][A_si_Equ_gi_NodeOffset]);
    f_si_Equ_p_Node_Ptr = &(dFdx[li_si][A_si_Equ_p_NodeOffset]);
    q_si_Equ_p_Node_Ptr = &(dQdx[li_si][A_si_Equ_p_NodeOffset]);
    f_si_Equ_e_Node_Ptr = &(dFdx[li_si][A_si_Equ_e_NodeOffset]);
    q_si_Equ_e_Node_Ptr = &(dQdx[li_si][A_si_Equ_e_NodeOffset]);
    f_si_Equ_b_Node_Ptr = &(dFdx[li_si][A_si_Equ_b_NodeOffset]);
    q_si_Equ_b_Node_Ptr = &(dQdx[li_si][A_si_Equ_b_NodeOffset]);
if (!collapseNode_t)
{
    f_si_Equ_t_Node_Ptr = &(dFdx[li_si][A_si_Equ_t_NodeOffset]);
    q_si_Equ_t_Node_Ptr = &(dQdx[li_si][A_si_Equ_t_NodeOffset]);
}
    f_si_Equ_gm_Node_Ptr = &(dFdx[li_si][A_si_Equ_gm_NodeOffset]);
    q_si_Equ_gm_Node_Ptr = &(dQdx[li_si][A_si_Equ_gm_NodeOffset]);
    f_si_Equ_s_Node_Ptr = &(dFdx[li_si][A_si_Equ_s_NodeOffset]);
    q_si_Equ_s_Node_Ptr = &(dQdx[li_si][A_si_Equ_s_NodeOffset]);
    f_gi_Equ_sb_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_sb_NodeOffset]);
    q_gi_Equ_sb_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_sb_NodeOffset]);
    f_gi_Equ_si_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_si_NodeOffset]);
    q_gi_Equ_si_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_si_NodeOffset]);
    f_gi_Equ_db_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_db_NodeOffset]);
    q_gi_Equ_db_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_db_NodeOffset]);
    f_gi_Equ_di_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_di_NodeOffset]);
    q_gi_Equ_di_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_di_NodeOffset]);
    f_gi_Equ_b_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_b_NodeOffset]);
    q_gi_Equ_b_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_b_NodeOffset]);
    f_gi_Equ_p_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_p_NodeOffset]);
    q_gi_Equ_p_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_gi_Equ_t_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_t_NodeOffset]);
    q_gi_Equ_t_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_t_NodeOffset]);
}
    f_gi_Equ_e_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_e_NodeOffset]);
    q_gi_Equ_e_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_e_NodeOffset]);
    f_gi_Equ_gi_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_gi_NodeOffset]);
    q_gi_Equ_gi_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_gi_NodeOffset]);
    f_gi_Equ_gm_Node_Ptr = &(dFdx[li_gi][A_gi_Equ_gm_NodeOffset]);
    q_gi_Equ_gm_Node_Ptr = &(dQdx[li_gi][A_gi_Equ_gm_NodeOffset]);
    f_gm_Equ_gm_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_gm_NodeOffset]);
    q_gm_Equ_gm_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_gm_NodeOffset]);
    f_gm_Equ_si_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_si_NodeOffset]);
    q_gm_Equ_si_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_si_NodeOffset]);
    f_gm_Equ_b_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_b_NodeOffset]);
    q_gm_Equ_b_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_b_NodeOffset]);
    f_gm_Equ_p_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_p_NodeOffset]);
    q_gm_Equ_p_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_gm_Equ_t_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_t_NodeOffset]);
    q_gm_Equ_t_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_t_NodeOffset]);
}
    f_gm_Equ_gi_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_gi_NodeOffset]);
    q_gm_Equ_gi_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_gi_NodeOffset]);
    f_gm_Equ_di_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_di_NodeOffset]);
    q_gm_Equ_di_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_di_NodeOffset]);
    f_gm_Equ_e_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_e_NodeOffset]);
    q_gm_Equ_e_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_e_NodeOffset]);
    f_gm_Equ_sb_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_sb_NodeOffset]);
    q_gm_Equ_sb_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_sb_NodeOffset]);
    f_gm_Equ_db_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_db_NodeOffset]);
    q_gm_Equ_db_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_db_NodeOffset]);
    f_gm_Equ_g_Node_Ptr = &(dFdx[li_gm][A_gm_Equ_g_NodeOffset]);
    q_gm_Equ_g_Node_Ptr = &(dQdx[li_gm][A_gm_Equ_g_NodeOffset]);
    f_sb_Equ_b_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_b_NodeOffset]);
    q_sb_Equ_b_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_b_NodeOffset]);
    f_sb_Equ_p_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_p_NodeOffset]);
    q_sb_Equ_p_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_sb_Equ_t_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_t_NodeOffset]);
    q_sb_Equ_t_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_t_NodeOffset]);
}
    f_sb_Equ_e_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_e_NodeOffset]);
    q_sb_Equ_e_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_e_NodeOffset]);
    f_sb_Equ_si_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_si_NodeOffset]);
    q_sb_Equ_si_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_si_NodeOffset]);
    f_sb_Equ_gi_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_gi_NodeOffset]);
    q_sb_Equ_gi_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_gi_NodeOffset]);
    f_sb_Equ_di_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_di_NodeOffset]);
    q_sb_Equ_di_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_di_NodeOffset]);
    f_sb_Equ_sb_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_sb_NodeOffset]);
    q_sb_Equ_sb_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_sb_NodeOffset]);
    f_sb_Equ_db_Node_Ptr = &(dFdx[li_sb][A_sb_Equ_db_NodeOffset]);
    q_sb_Equ_db_Node_Ptr = &(dQdx[li_sb][A_sb_Equ_db_NodeOffset]);
    f_db_Equ_b_Node_Ptr = &(dFdx[li_db][A_db_Equ_b_NodeOffset]);
    q_db_Equ_b_Node_Ptr = &(dQdx[li_db][A_db_Equ_b_NodeOffset]);
    f_db_Equ_p_Node_Ptr = &(dFdx[li_db][A_db_Equ_p_NodeOffset]);
    q_db_Equ_p_Node_Ptr = &(dQdx[li_db][A_db_Equ_p_NodeOffset]);
if (!collapseNode_t)
{
    f_db_Equ_t_Node_Ptr = &(dFdx[li_db][A_db_Equ_t_NodeOffset]);
    q_db_Equ_t_Node_Ptr = &(dQdx[li_db][A_db_Equ_t_NodeOffset]);
}
    f_db_Equ_e_Node_Ptr = &(dFdx[li_db][A_db_Equ_e_NodeOffset]);
    q_db_Equ_e_Node_Ptr = &(dQdx[li_db][A_db_Equ_e_NodeOffset]);
    f_db_Equ_si_Node_Ptr = &(dFdx[li_db][A_db_Equ_si_NodeOffset]);
    q_db_Equ_si_Node_Ptr = &(dQdx[li_db][A_db_Equ_si_NodeOffset]);
    f_db_Equ_gi_Node_Ptr = &(dFdx[li_db][A_db_Equ_gi_NodeOffset]);
    q_db_Equ_gi_Node_Ptr = &(dQdx[li_db][A_db_Equ_gi_NodeOffset]);
    f_db_Equ_di_Node_Ptr = &(dFdx[li_db][A_db_Equ_di_NodeOffset]);
    q_db_Equ_di_Node_Ptr = &(dQdx[li_db][A_db_Equ_di_NodeOffset]);
    f_db_Equ_sb_Node_Ptr = &(dFdx[li_db][A_db_Equ_sb_NodeOffset]);
    q_db_Equ_sb_Node_Ptr = &(dQdx[li_db][A_db_Equ_sb_NodeOffset]);
    f_db_Equ_db_Node_Ptr = &(dFdx[li_db][A_db_Equ_db_NodeOffset]);
    q_db_Equ_db_Node_Ptr = &(dQdx[li_db][A_db_Equ_db_NodeOffset]);


}

// RHS load functions

//-----------------------------------------------------------------------------
// Function      : Instance::loadDAEFVector
// Purpose       : load F vector (static contributions) for one instance for
//                 NEW DAE formulation
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::loadDAEFVector()
{

  bool bsuccess=true;
(*extData.daeFVectorPtr)[li_d] += staticContributions[admsNodeID_d];
(*extData.daeFVectorPtr)[li_g] += staticContributions[admsNodeID_g];
(*extData.daeFVectorPtr)[li_s] += staticContributions[admsNodeID_s];
(*extData.daeFVectorPtr)[li_e] += staticContributions[admsNodeID_e];
(*extData.daeFVectorPtr)[li_p] += staticContributions[admsNodeID_p];
(*extData.daeFVectorPtr)[li_b] += staticContributions[admsNodeID_b];
if (!collapseNode_t)
  (*extData.daeFVectorPtr)[li_t] += staticContributions[admsNodeID_t];
(*extData.daeFVectorPtr)[li_di] += staticContributions[admsNodeID_di];
(*extData.daeFVectorPtr)[li_si] += staticContributions[admsNodeID_si];
(*extData.daeFVectorPtr)[li_gi] += staticContributions[admsNodeID_gi];
(*extData.daeFVectorPtr)[li_gm] += staticContributions[admsNodeID_gm];
(*extData.daeFVectorPtr)[li_sb] += staticContributions[admsNodeID_sb];
(*extData.daeFVectorPtr)[li_db] += staticContributions[admsNodeID_db];


  if (loadLeadCurrent)
  {
    double * leadF = extData.nextLeadCurrFCompRawPtr;
    leadF[li_branch_id] = leadCurrentF[admsNodeID_d];
    leadF[li_branch_ig] = leadCurrentF[admsNodeID_g];
    leadF[li_branch_is] = leadCurrentF[admsNodeID_s];
    leadF[li_branch_ie] = leadCurrentF[admsNodeID_e];
    if (portsConnected_[admsNodeID_p])
    {
    leadF[li_branch_ip] = leadCurrentF[admsNodeID_p];
    }
    if (portsConnected_[admsNodeID_b])
    {
    leadF[li_branch_ib] = leadCurrentF[admsNodeID_b];
    }
    if (portsConnected_[admsNodeID_t])
    {
    leadF[li_branch_it] = leadCurrentF[admsNodeID_t];
    }



    // here we have to do special things for BJTs, MOSFETs and 2-terminal
    // devices for power computation.

        double * solVec = extData.nextSolVectorRawPtr;
        double * junctionV = extData.nextJunctionVCompRawPtr;
        junctionV[li_branch_id] = solVec[li_d] - solVec[li_s];
        junctionV[li_branch_ig] = solVec[li_g] - solVec[li_s];

  }
  return bsuccess;
}

//-----------------------------------------------------------------------------
// Function      : Instance::loadDAEQVector
// Purpose       : load Q vector (dynamic contributions) for one instance for
//                 NEW DAE formulation
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::loadDAEQVector()
{
(*extData.daeQVectorPtr)[li_d] += dynamicContributions[admsNodeID_d];
(*extData.daeQVectorPtr)[li_g] += dynamicContributions[admsNodeID_g];
(*extData.daeQVectorPtr)[li_s] += dynamicContributions[admsNodeID_s];
(*extData.daeQVectorPtr)[li_e] += dynamicContributions[admsNodeID_e];
(*extData.daeQVectorPtr)[li_p] += dynamicContributions[admsNodeID_p];
(*extData.daeQVectorPtr)[li_b] += dynamicContributions[admsNodeID_b];
if (!collapseNode_t)
  (*extData.daeQVectorPtr)[li_t] += dynamicContributions[admsNodeID_t];
(*extData.daeQVectorPtr)[li_di] += dynamicContributions[admsNodeID_di];
(*extData.daeQVectorPtr)[li_si] += dynamicContributions[admsNodeID_si];
(*extData.daeQVectorPtr)[li_gi] += dynamicContributions[admsNodeID_gi];
(*extData.daeQVectorPtr)[li_gm] += dynamicContributions[admsNodeID_gm];
(*extData.daeQVectorPtr)[li_sb] += dynamicContributions[admsNodeID_sb];
(*extData.daeQVectorPtr)[li_db] += dynamicContributions[admsNodeID_db];


  if (loadLeadCurrent)
  {
    double * leadQ = extData.nextLeadCurrQCompRawPtr;
    leadQ[li_branch_id] = leadCurrentQ[admsNodeID_d];
    leadQ[li_branch_ig] = leadCurrentQ[admsNodeID_g];
    leadQ[li_branch_is] = leadCurrentQ[admsNodeID_s];
    leadQ[li_branch_ie] = leadCurrentQ[admsNodeID_e];
    if (portsConnected_[admsNodeID_p])
    {
    leadQ[li_branch_ip] = leadCurrentQ[admsNodeID_p];
    }
    if (portsConnected_[admsNodeID_b])
    {
    leadQ[li_branch_ib] = leadCurrentQ[admsNodeID_b];
    }
    if (portsConnected_[admsNodeID_t])
    {
    leadQ[li_branch_it] = leadCurrentQ[admsNodeID_t];
    }


  }
  return true;
}


//-----------------------------------------------------------------------------
// Function      : Instance::updatePrimaryState
// Purpose       :
// Special Notes : In this initial implementation, does little more than
//                 call updateIntermediateVars()
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::updatePrimaryState()
{
  bool bsuccess = true;

  bsuccess = updateIntermediateVars();

  // if old DAE were implemented, we'd save dynamic contributions as state
  // here.

  double * stoVec = extData.nextStoVectorRawPtr;
  // Also need to save limited voltage drops and output vars
  // This formulation assumes that we have *always* written the
  // limited voltages back into the probeVars[] array.

  stoVec[li_store_VDSAT] = VDSAT;
  stoVec[li_store_VTH] = VTH;
  stoVec[li_store_IDS] = IDS;
  stoVec[li_store_GM] = GM;
  stoVec[li_store_GDS] = GDS;
  stoVec[li_store_GMBS] = GMBS;
  stoVec[li_store_IC] = IC;
  stoVec[li_store_IBD] = IBD;
  stoVec[li_store_IBS] = IBS;
  stoVec[li_store_IGIDL] = IGIDL;
  stoVec[li_store_IGISL] = IGISL;
  stoVec[li_store_IGS] = IGS;
  stoVec[li_store_IGD] = IGD;
  stoVec[li_store_IGB] = IGB;
  stoVec[li_store_IGCS] = IGCS;
  stoVec[li_store_IGCD] = IGCD;
  stoVec[li_store_CGG] = CGG;
  stoVec[li_store_CGS] = CGS;
  stoVec[li_store_CGD] = CGD;
  stoVec[li_store_CBG] = CBG;
  stoVec[li_store_CBS] = CBS;
  stoVec[li_store_CBD] = CBD;
  stoVec[li_store_CDG] = CDG;
  stoVec[li_store_CDD] = CDD;
  stoVec[li_store_CDS] = CDS;
  stoVec[li_store_CAPBD] = CAPBD;
  stoVec[li_store_CAPBS] = CAPBS;
  stoVec[li_store_QG] = QG;
  stoVec[li_store_QB] = QB;
  stoVec[li_store_QD] = QD;
  stoVec[li_store_QS] = QS;
  stoVec[li_store_QJD] = QJD;
  stoVec[li_store_QJS] = QJS;
  stoVec[li_store_T_TOTAL_K] = T_TOTAL_K;
  stoVec[li_store_T_TOTAL_C] = T_TOTAL_C;
  stoVec[li_store_T_DELTA_SH] = T_DELTA_SH;

  return bsuccess;
}

//-----------------------------------------------------------------------------
// Function      : Instance::updateSecondaryState
// Purpose       :
// Special Notes : In this initial implementation, does nothing
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::updateSecondaryState()
{
  bool bsuccess = true;

  // were old DAE implemented, we'd pull dynamic contribution derivatives
  // out of state.

  return bsuccess;
}

//-----------------------------------------------------------------------------
// Function      : Instance::updateIntermediateVars
// Purpose       : update intermediate variables for one bsimsoi instance
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::updateIntermediateVars()
{

  bool bsuccess=true;
  Linear::Vector * solVectorPtr = extData.nextSolVectorPtr;

  if (DEBUG_DEVICE && isActive(Diag::DEVICE_PARAMETERS) && getSolverState().debugTimeFlag)
  {
    Xyce::dout() << std::endl << subsection_divider << std::endl;
    Xyce::dout() << "  In ADMSbsimsoi::Instance::updateIntermediateVars\n\n";
    Xyce::dout() << "  name = " << getName() << std::endl;
  }

  // Local variables
  double epsgate=0.0;
  double B4SOIig=0.0;
     double d_B4SOIig_dV_db_di=0.0;
     double d_B4SOIig_dV_sb_si=0.0;
     double d_B4SOIig_dV_b_si=0.0;
     double d_B4SOIig_dV_e_si=0.0;
     double d_B4SOIig_dV_di_si=0.0;
     double d_B4SOIig_dV_gi_si=0.0;
     double d_B4SOIig_dTemp_t_GND=0.0;
     double d_B4SOIig_dV_p_GND=0.0;
     double d_B4SOIig_dV_b_GND=0.0;
  double B4SOIigp=0.0;
     double d_B4SOIigp_dV_db_di=0.0;
     double d_B4SOIigp_dV_sb_si=0.0;
     double d_B4SOIigp_dV_b_si=0.0;
     double d_B4SOIigp_dV_e_si=0.0;
     double d_B4SOIigp_dV_di_si=0.0;
     double d_B4SOIigp_dV_gi_si=0.0;
     double d_B4SOIigp_dTemp_t_GND=0.0;
     double d_B4SOIigp_dV_p_GND=0.0;
     double d_B4SOIigp_dV_b_GND=0.0;
     double d_B4SOIigp_dV_gi_p=0.0;
  double B4SOIgcrg=0.0;
     double d_B4SOIgcrg_dV_gi_p=0.0;
     double d_B4SOIgcrg_dV_db_di=0.0;
     double d_B4SOIgcrg_dV_sb_si=0.0;
     double d_B4SOIgcrg_dV_b_si=0.0;
     double d_B4SOIgcrg_dV_di_si=0.0;
     double d_B4SOIgcrg_dV_gi_si=0.0;
     double d_B4SOIgcrg_dV_e_si=0.0;
     double d_B4SOIgcrg_dTemp_t_GND=0.0;
     double d_B4SOIgcrg_dV_p_GND=0.0;
     double d_B4SOIgcrg_dV_b_GND=0.0;
  double B4SOIqse=0.0;
     double d_B4SOIqse_dV_gi_p=0.0;
     double d_B4SOIqse_dV_db_di=0.0;
     double d_B4SOIqse_dV_sb_si=0.0;
     double d_B4SOIqse_dV_b_si=0.0;
     double d_B4SOIqse_dV_e_si=0.0;
     double d_B4SOIqse_dV_di_si=0.0;
     double d_B4SOIqse_dV_gi_si=0.0;
     double d_B4SOIqse_dTemp_t_GND=0.0;
     double d_B4SOIqse_dV_p_GND=0.0;
     double d_B4SOIqse_dV_b_GND=0.0;
  double B4SOIqde=0.0;
     double d_B4SOIqde_dV_gi_p=0.0;
     double d_B4SOIqde_dV_db_di=0.0;
     double d_B4SOIqde_dV_sb_si=0.0;
     double d_B4SOIqde_dV_b_si=0.0;
     double d_B4SOIqde_dV_e_si=0.0;
     double d_B4SOIqde_dV_di_si=0.0;
     double d_B4SOIqde_dV_gi_si=0.0;
     double d_B4SOIqde_dTemp_t_GND=0.0;
     double d_B4SOIqde_dV_p_GND=0.0;
     double d_B4SOIqde_dV_b_GND=0.0;
  double qgi=0.0;
     double d_qgi_dV_gi_p=0.0;
     double d_qgi_dV_db_di=0.0;
     double d_qgi_dV_sb_si=0.0;
     double d_qgi_dV_di_si=0.0;
     double d_qgi_dV_gi_si=0.0;
     double d_qgi_dTemp_t_GND=0.0;
     double d_qgi_dV_p_GND=0.0;
     double d_qgi_dV_b_GND=0.0;
     double d_qgi_dV_b_si=0.0;
     double d_qgi_dV_e_si=0.0;
  double qov=0.0;
     double d_qov_dV_gi_p=0.0;
     double d_qov_dV_db_di=0.0;
     double d_qov_dV_sb_si=0.0;
     double d_qov_dV_b_si=0.0;
     double d_qov_dV_e_si=0.0;
     double d_qov_dV_di_si=0.0;
     double d_qov_dV_gi_si=0.0;
     double d_qov_dTemp_t_GND=0.0;
     double d_qov_dV_p_GND=0.0;
     double d_qov_dV_b_GND=0.0;
     double d_qov_dV_gm_si=0.0;
  double B4SOIigidl=0.0;
     double d_B4SOIigidl_dV_b_si=0.0;
     double d_B4SOIigidl_dV_e_si=0.0;
     double d_B4SOIigidl_dV_di_si=0.0;
     double d_B4SOIigidl_dV_gi_si=0.0;
     double d_B4SOIigidl_dTemp_t_GND=0.0;
     double d_B4SOIigidl_dV_p_GND=0.0;
     double d_B4SOIigidl_dV_b_GND=0.0;
  double B4SOIigisl=0.0;
     double d_B4SOIigisl_dV_b_si=0.0;
     double d_B4SOIigisl_dV_e_si=0.0;
     double d_B4SOIigisl_dV_di_si=0.0;
     double d_B4SOIigisl_dV_gi_si=0.0;
     double d_B4SOIigisl_dTemp_t_GND=0.0;
     double d_B4SOIigisl_dV_p_GND=0.0;
     double d_B4SOIigisl_dV_b_GND=0.0;
  double B4SOIIgcd=0.0;
     double d_B4SOIIgcd_dV_db_di=0.0;
     double d_B4SOIIgcd_dV_sb_si=0.0;
     double d_B4SOIIgcd_dV_b_si=0.0;
     double d_B4SOIIgcd_dV_e_si=0.0;
     double d_B4SOIIgcd_dV_di_si=0.0;
     double d_B4SOIIgcd_dV_gi_si=0.0;
     double d_B4SOIIgcd_dTemp_t_GND=0.0;
     double d_B4SOIIgcd_dV_p_GND=0.0;
     double d_B4SOIIgcd_dV_b_GND=0.0;
  double B4SOIIgcs=0.0;
     double d_B4SOIIgcs_dV_db_di=0.0;
     double d_B4SOIIgcs_dV_sb_si=0.0;
     double d_B4SOIIgcs_dV_b_si=0.0;
     double d_B4SOIIgcs_dV_e_si=0.0;
     double d_B4SOIIgcs_dV_di_si=0.0;
     double d_B4SOIIgcs_dV_gi_si=0.0;
     double d_B4SOIIgcs_dTemp_t_GND=0.0;
     double d_B4SOIIgcs_dV_p_GND=0.0;
     double d_B4SOIIgcs_dV_b_GND=0.0;
  double B4SOIqdrn=0.0;
     double d_B4SOIqdrn_dV_gi_p=0.0;
     double d_B4SOIqdrn_dV_db_di=0.0;
     double d_B4SOIqdrn_dV_sb_si=0.0;
     double d_B4SOIqdrn_dV_di_si=0.0;
     double d_B4SOIqdrn_dV_gi_si=0.0;
     double d_B4SOIqdrn_dTemp_t_GND=0.0;
     double d_B4SOIqdrn_dV_p_GND=0.0;
     double d_B4SOIqdrn_dV_b_GND=0.0;
     double d_B4SOIqdrn_dV_b_si=0.0;
     double d_B4SOIqdrn_dV_e_si=0.0;
  double B4SOIqsrc=0.0;
     double d_B4SOIqsrc_dV_gi_p=0.0;
     double d_B4SOIqsrc_dV_db_di=0.0;
     double d_B4SOIqsrc_dV_sb_si=0.0;
     double d_B4SOIqsrc_dV_di_si=0.0;
     double d_B4SOIqsrc_dV_gi_si=0.0;
     double d_B4SOIqsrc_dTemp_t_GND=0.0;
     double d_B4SOIqsrc_dV_p_GND=0.0;
     double d_B4SOIqsrc_dV_b_GND=0.0;
     double d_B4SOIqsrc_dV_b_si=0.0;
     double d_B4SOIqsrc_dV_e_si=0.0;
  double B4SOIIgd=0.0;
     double d_B4SOIIgd_dV_b_si=0.0;
     double d_B4SOIIgd_dV_di_si=0.0;
     double d_B4SOIIgd_dV_gi_si=0.0;
     double d_B4SOIIgd_dV_e_si=0.0;
     double d_B4SOIIgd_dTemp_t_GND=0.0;
     double d_B4SOIIgd_dV_p_GND=0.0;
     double d_B4SOIIgd_dV_b_GND=0.0;
     double d_B4SOIIgd_dV_db_di=0.0;
     double d_B4SOIIgd_dV_sb_si=0.0;
  double B4SOIIgs=0.0;
     double d_B4SOIIgs_dV_b_si=0.0;
     double d_B4SOIIgs_dV_di_si=0.0;
     double d_B4SOIIgs_dV_gi_si=0.0;
     double d_B4SOIIgs_dV_e_si=0.0;
     double d_B4SOIIgs_dTemp_t_GND=0.0;
     double d_B4SOIIgs_dV_p_GND=0.0;
     double d_B4SOIIgs_dV_b_GND=0.0;
     double d_B4SOIIgs_dV_db_di=0.0;
     double d_B4SOIIgs_dV_sb_si=0.0;
  double QGI=0.0;
     double d_QGI_dV_gi_p=0.0;
     double d_QGI_dV_db_di=0.0;
     double d_QGI_dV_sb_si=0.0;
     double d_QGI_dV_di_si=0.0;
     double d_QGI_dV_gi_si=0.0;
     double d_QGI_dTemp_t_GND=0.0;
     double d_QGI_dV_p_GND=0.0;
     double d_QGI_dV_b_GND=0.0;
     double d_QGI_dV_b_si=0.0;
     double d_QGI_dV_e_si=0.0;
  double QOV=0.0;
     double d_QOV_dV_gi_p=0.0;
     double d_QOV_dV_db_di=0.0;
     double d_QOV_dV_sb_si=0.0;
     double d_QOV_dV_b_si=0.0;
     double d_QOV_dV_e_si=0.0;
     double d_QOV_dV_di_si=0.0;
     double d_QOV_dV_gi_si=0.0;
     double d_QOV_dTemp_t_GND=0.0;
     double d_QOV_dV_p_GND=0.0;
     double d_QOV_dV_b_GND=0.0;
     double d_QOV_dV_gm_si=0.0;
  double TempRatioMinus1=0.0;
     double d_TempRatioMinus1_dTemp_t_GND=0.0;
     double d_TempRatioMinus1_dV_p_GND=0.0;
     double d_TempRatioMinus1_dV_b_GND=0.0;
  double TempRatio=0.0;
     double d_TempRatio_dTemp_t_GND=0.0;
     double d_TempRatio_dV_p_GND=0.0;
     double d_TempRatio_dV_b_GND=0.0;
  double delTemp=0.0;
     double d_delTemp_dTemp_t_GND=0.0;
     double d_delTemp_dV_p_GND=0.0;
     double d_delTemp_dV_b_GND=0.0;


  // set the sizes of the Fad arrays:

  if (probeVars.size() != (26))
  {
    probeVars.resize(26);
    d_probeVars.resize(26);
    for (int i=0;i<26;i++)
    {
      d_probeVars[i].resize(26);
      for (int j=0;j<26;j++)
        d_probeVars[i][j]=0.0;
    }
    staticContributions.resize(13+0);
    d_staticContributions.resize(13+0);
    for (int i=0;i<13+0;i++)
      d_staticContributions[i].resize(26);
    dynamicContributions.resize(13+0);
    d_dynamicContributions.resize(13+0);
    for (int i=0;i<13+0;i++)
      d_dynamicContributions[i].resize(26);
  }

    noiseContribsPower.resize(16);
    noiseContribsExponent.resize(16);

  // initialize contributions to zero (automatically sets derivatives to zero)
  for (int i=0; i < 13+0 ; ++i)
  {
     staticContributions[i]=0;
     for (int j=0;j<26;j++)
        d_staticContributions[i][j] = 0.0;
     dynamicContributions[i]=0;
     for (int j=0;j<26;j++)
        d_dynamicContributions[i][j] = 0.0;
  }

  // extract solution variables and set as Fad independent variables.
probeVars[admsProbeID_V_sb_GND] = (*solVectorPtr)[li_sb];
d_probeVars[admsProbeID_V_sb_GND][admsProbeID_V_sb_GND] = 1.0;
probeVars[admsProbeID_V_db_GND] = (*solVectorPtr)[li_db];
d_probeVars[admsProbeID_V_db_GND][admsProbeID_V_db_GND] = 1.0;
probeVars[admsProbeID_V_gm_GND] = (*solVectorPtr)[li_gm];
d_probeVars[admsProbeID_V_gm_GND][admsProbeID_V_gm_GND] = 1.0;
probeVars[admsProbeID_V_b_sb] = (*solVectorPtr)[li_b] - (*solVectorPtr)[li_sb];
d_probeVars[admsProbeID_V_b_sb][admsProbeID_V_b_sb] = 1.0;
probeVars[admsProbeID_V_b_db] = (*solVectorPtr)[li_b] - (*solVectorPtr)[li_db];
d_probeVars[admsProbeID_V_b_db][admsProbeID_V_b_db] = 1.0;
probeVars[admsProbeID_V_gm_gi] = (*solVectorPtr)[li_gm] - (*solVectorPtr)[li_gi];
d_probeVars[admsProbeID_V_gm_gi][admsProbeID_V_gm_gi] = 1.0;
probeVars[admsProbeID_V_g_gm] = (*solVectorPtr)[li_g] - (*solVectorPtr)[li_gm];
d_probeVars[admsProbeID_V_g_gm][admsProbeID_V_g_gm] = 1.0;
probeVars[admsProbeID_V_gi_e] = (*solVectorPtr)[li_gi] - (*solVectorPtr)[li_e];
d_probeVars[admsProbeID_V_gi_e][admsProbeID_V_gi_e] = 1.0;
probeVars[admsProbeID_V_gm_e] = (*solVectorPtr)[li_gm] - (*solVectorPtr)[li_e];
d_probeVars[admsProbeID_V_gm_e][admsProbeID_V_gm_e] = 1.0;
probeVars[admsProbeID_V_s_si] = (*solVectorPtr)[li_s] - (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_s_si][admsProbeID_V_s_si] = 1.0;
probeVars[admsProbeID_V_d_di] = (*solVectorPtr)[li_d] - (*solVectorPtr)[li_di];
d_probeVars[admsProbeID_V_d_di][admsProbeID_V_d_di] = 1.0;
probeVars[admsProbeID_V_si_GND] = (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_si_GND][admsProbeID_V_si_GND] = 1.0;
probeVars[admsProbeID_V_di_GND] = (*solVectorPtr)[li_di];
d_probeVars[admsProbeID_V_di_GND][admsProbeID_V_di_GND] = 1.0;
probeVars[admsProbeID_V_gi_GND] = (*solVectorPtr)[li_gi];
d_probeVars[admsProbeID_V_gi_GND][admsProbeID_V_gi_GND] = 1.0;
probeVars[admsProbeID_V_gm_si] = (*solVectorPtr)[li_gm] - (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_gm_si][admsProbeID_V_gm_si] = 1.0;
probeVars[admsProbeID_V_db_di] = (*solVectorPtr)[li_db] - (*solVectorPtr)[li_di];
d_probeVars[admsProbeID_V_db_di][admsProbeID_V_db_di] = 1.0;
probeVars[admsProbeID_V_sb_si] = (*solVectorPtr)[li_sb] - (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_sb_si][admsProbeID_V_sb_si] = 1.0;
probeVars[admsProbeID_V_gi_p] = (*solVectorPtr)[li_gi] - (*solVectorPtr)[li_p];
d_probeVars[admsProbeID_V_gi_p][admsProbeID_V_gi_p] = 1.0;
probeVars[admsProbeID_V_b_p] = (*solVectorPtr)[li_b] - (*solVectorPtr)[li_p];
d_probeVars[admsProbeID_V_b_p][admsProbeID_V_b_p] = 1.0;
probeVars[admsProbeID_V_e_si] = (*solVectorPtr)[li_e] - (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_e_si][admsProbeID_V_e_si] = 1.0;
probeVars[admsProbeID_V_gi_si] = (*solVectorPtr)[li_gi] - (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_gi_si][admsProbeID_V_gi_si] = 1.0;
probeVars[admsProbeID_V_b_si] = (*solVectorPtr)[li_b] - (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_b_si][admsProbeID_V_b_si] = 1.0;
probeVars[admsProbeID_V_di_si] = (*solVectorPtr)[li_di] - (*solVectorPtr)[li_si];
d_probeVars[admsProbeID_V_di_si][admsProbeID_V_di_si] = 1.0;
probeVars[admsProbeID_Temp_t_GND] = ((collapseNode_t)?(0):((*solVectorPtr)[li_t]));
d_probeVars[admsProbeID_Temp_t_GND][admsProbeID_Temp_t_GND] = 1.0;
probeVars[admsProbeID_V_p_GND] = (*solVectorPtr)[li_p];
d_probeVars[admsProbeID_V_p_GND][admsProbeID_V_p_GND] = 1.0;
probeVars[admsProbeID_V_b_GND] = (*solVectorPtr)[li_b];
d_probeVars[admsProbeID_V_b_GND][admsProbeID_V_b_GND] = 1.0;
// -- code converted from analog/code block
//Begin block b4soi_check
{
//Block-local variables for block b4soi_check
int Fatal_Flag=0.0;
//End of Block-local variables
Fatal_Flag = 0;
if ((pParam_B4SOIlpe0<(-pParam_B4SOIleff)))
{
UserInfo(*this)  << "Fatal: Lpe0 = %g is less than -Leff." << " " << pParam_B4SOIlpe0 << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((((B4SOIsa>0.0)&&(B4SOIsb>0.0))&&((B4SOInf==1.0)||((B4SOInf>1.0)&&(B4SOIsd>0.0)))))
{
if ((B4SOIsaref<=0.0))
{
UserInfo(*this)  << "Fatal: SAref = %g is not positive." << " " << B4SOIsaref << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOIsbref<=0.0))
{
UserInfo(*this)  << "Fatal: SBref = %g is not positive." << " " << B4SOIsbref << " " <<  std::endl; 
Fatal_Flag = 1;
}
}
if ((pParam_B4SOIlpeb<(-pParam_B4SOIleff)))
{
UserInfo(*this)  << "Fatal: Lpeb = %g is less than -Leff." << " " << pParam_B4SOIlpeb << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIfprout<0.0))
{
UserInfo(*this)  << "Fatal: fprout = %g is negative." << " " << pParam_B4SOIfprout << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIpdits<0.0))
{
UserInfo(*this)  << "Fatal: pdits = %g is negative." << " " << pParam_B4SOIpdits << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOIpditsl<0.0))
{
UserInfo(*this)  << "Fatal: pditsl = %g is negative." << " " << B4SOIpditsl << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOItox<=0.0))
{
UserInfo(*this)  << "Fatal: Tox = %g is not positive." << " " << B4SOItox << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOIleffeot<=0.0))
{
UserInfo(*this)  << "Fatal: Leffeot = %g is not positive." << " " << B4SOIleffeot << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOIweffeot<=0.0))
{
UserInfo(*this)  << "Fatal: Weffeot = %g is not positive." << " " << B4SOIweffeot << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOItoxp<=0.0))
{
UserInfo(*this)  << "Fatal: Toxp = %g is not positive." << " " << B4SOItoxp << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOIepsrgate<0.0))
{
UserInfo(*this)  << "Fatal: Epsrgate = %g is not positive." << " " << B4SOIepsrgate << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOItoxm<=0.0))
{
UserInfo(*this)  << "Fatal: Toxm = %g is not positive." << " " << B4SOItoxm << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOInf<1.0))
{
UserInfo(*this)  << "Fatal: Number of finger = %g is smaller than one." << " " << B4SOInf << " " <<  std::endl; 
Fatal_Flag = 1;
}
if (((B4SOItox-B4SOIdtoxcv)<=0.0))
{
UserInfo(*this)  << "Fatal: Tox - dtoxcv = %g is not positive." << " " << (B4SOItox-B4SOIdtoxcv) << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOItbox<=0.0))
{
UserInfo(*this)  << "Fatal: Tbox = %g is not positive." << " " << B4SOItbox << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOInpeak<=0.0))
{
UserInfo(*this)  << "Fatal: Nch = %g is not positive." << " " << pParam_B4SOInpeak << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIngate<0.0))
{
UserInfo(*this)  << "Fatal: Ngate = %g Ngate is not positive." << " " << pParam_B4SOIngate << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIngate>1.0e25))
{
UserInfo(*this)  << "Fatal: Ngate = %g Ngate is too high" << " " << pParam_B4SOIngate << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIdvt1<0.0))
{
UserInfo(*this)  << "Fatal: Dvt1 = %g is negative." << " " << pParam_B4SOIdvt1 << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIdvt1w<0.0))
{
UserInfo(*this)  << "Fatal: Dvt1w = %g is negative." << " " << pParam_B4SOIdvt1w << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIw0==(-pParam_B4SOIweff)))
{
UserInfo(*this)  << "Fatal: (W0 + Weff) = 0 causing divided-by-zero." << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIdsub<0.0))
{
UserInfo(*this)  << "Fatal: Dsub = %g is negative." << " " << pParam_B4SOIdsub << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIb1==(-pParam_B4SOIweff)))
{
UserInfo(*this)  << "Fatal: (B1 + Weff) = 0 causing divided-by-zero." << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIu0temp<=0.0))
{
UserInfo(*this)  << "Fatal: u0 at current temperature = %g is not positive." << " " << pParam_B4SOIu0temp << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIdelta<0.0))
{
UserInfo(*this)  << "Fatal: Delta = %g is less than zero." << " " << pParam_B4SOIdelta << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIvsattemp<=0.0))
{
UserInfo(*this)  << "Fatal: Vsat at current temperature = %g is not positive." << " " << pParam_B4SOIvsattemp << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIpclm<=0.0))
{
UserInfo(*this)  << "Fatal: Pclm = %g is not positive." << " " << pParam_B4SOIpclm << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIdrout<0.0))
{
UserInfo(*this)  << "Fatal: Drout = %g is negative." << " " << pParam_B4SOIdrout << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIclc<0.0))
{
UserInfo(*this)  << "Fatal: Clc = %g is negative." << " " << pParam_B4SOIclc << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOInoff<0.1))
{
UserInfo(*this)  << "Warning: Noff = %g is too small." << " " << pParam_B4SOInoff << " " <<  std::endl; 
}
else
{
if ((pParam_B4SOInoff>4.0))
{
UserInfo(*this)  << "Warning: Noff = %g is too large." << " " << pParam_B4SOInoff << " " <<  std::endl; 
}
}
if ((pParam_B4SOInoff2<0.1))
{
UserInfo(*this)  << "Warning: Noff2 = %g is too small." << " " << pParam_B4SOInoff2 << " " <<  std::endl; 
}
else
{
if ((pParam_B4SOInoff2>4.0))
{
UserInfo(*this)  << "Warning: Noff2 = %g is too large." << " " << pParam_B4SOInoff2 << " " <<  std::endl; 
}
}
if ((((B4SOIsa>0.0)&&(B4SOIsb>0.0))&&((B4SOInf==1.0)||((B4SOInf>1.0)&&(B4SOIsd>0.0)))))
{
if ((B4SOIlodk2<=0.0))
{
UserInfo(*this)  << "Warning: LODK2 = %g is not positive." << " " << B4SOIlodk2 << " " <<  std::endl; 
}
if ((B4SOIlodeta0<=0.0))
{
UserInfo(*this)  << "Warning: LODETA0 = %g is not positive." << " " << B4SOIlodeta0 << " " <<  std::endl; 
}
if ((B4SOIlodeta0cv<=0.0))
{
UserInfo(*this)  << "Warning: LODETA0CV = %g is not positive." << " " << B4SOIlodeta0cv << " " <<  std::endl; 
}
}
if ((pParam_B4SOImoin<5.0))
{
UserInfo(*this)  << "Warning: Moin = %g is too small." << " " << pParam_B4SOImoin << " " <<  std::endl; 
}
if ((pParam_B4SOImoin>25.0))
{
UserInfo(*this)  << "Warning: Moin = %g is too large." << " " << pParam_B4SOImoin << " " <<  std::endl; 
}
if ((pParam_B4SOImoinfd<5.0))
{
UserInfo(*this)  << "Warning: MoinFD = %g is too small." << " " << pParam_B4SOImoinfd << " " <<  std::endl; 
}
if ((B4SOIcapMod==3))
{
if ((pParam_B4SOIacde<0.1))
{
UserInfo(*this)  << "Warning: Acde = %g is too small." << " " << pParam_B4SOIacde << " " <<  std::endl; 
}
else
{
if ((pParam_B4SOIacde>1.6))
{
UserInfo(*this)  << "Warning: Acde = %g is too large." << " " << pParam_B4SOIacde << " " <<  std::endl; 
}
}
}
if (((B4SOIcsdmin>0)&&((B4SOIasd<=0.0)||(B4SOIasd>=1.0))))
{
UserInfo(*this)  << "Fatal: Asd = %g should be within (0, 1)." << " " << B4SOIasd << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOInigc<=0.0))
{
UserInfo(*this)  << "Fatal: nigc = %g is non-positive." << " " << pParam_B4SOInigc << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIpoxedge<=0.0))
{
UserInfo(*this)  << "Fatal: poxedge = %g is non-positive." << " " << pParam_B4SOIpoxedge << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((pParam_B4SOIpigcd<=0.0))
{
UserInfo(*this)  << "Fatal: pigcd = %g is non-positive." << " " << pParam_B4SOIpigcd << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOItoxref<0.0))
{
UserInfo(*this)  << "Warning:  Toxref = %g is negative." << " " << B4SOItoxref << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((B4SOItoxqm<=0.0))
{
UserInfo(*this)  << "Fatal: Toxqm = %g is not positive." << " " << B4SOItoxqm << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((((model_.VERSION)>=4.4)||(model_.PARAMCHK)))
{
if ((pParam_B4SOIa2<0.01))
{
UserInfo(*this)  << "Warning: A2 = %g is too small. Set to 0.01." << " " << pParam_B4SOIa2 << " " <<  std::endl; 
pParam_B4SOIa2 = 0.01;
}
else
{
if ((pParam_B4SOIa2>1.0))
{
UserInfo(*this)  << "Warning: A2 = %g is larger than 1. A2 is set to 1 and A1 is set to 0." << " " << pParam_B4SOIa2 << " " <<  std::endl; 
pParam_B4SOIa2 = 1.0;
pParam_B4SOIa1 = 0.0;
}
}
}
if ((pParam_B4SOIrdsw<0.0))
{
UserInfo(*this)  << "Warning: Rdsw = %g is negative. Set to zero." << " " << pParam_B4SOIrdsw << " " <<  std::endl; 
pParam_B4SOIrdsw = 0.0;
pParam_B4SOIrds0 = 0.0;
}
else
{
if (((pParam_B4SOIrds0<0.001)&&(pParam_B4SOIrds0!=0.0)))
{
UserInfo(*this)  << "Warning: Rds at current temperature = %g is less than 0.001 ohm. Set to zero." << " " << pParam_B4SOIrds0 << " " <<  std::endl; 
pParam_B4SOIrds0 = 0.0;
}
}
if ((model_.PARAMCHK))
{
if ((pParam_B4SOIleff<=5.0e-8))
{
UserInfo(*this)  << "Warning: Leff = %g may be too small." << " " << pParam_B4SOIleff << " " <<  std::endl; 
}
if ((pParam_B4SOIleffCV<=5.0e-8))
{
UserInfo(*this)  << "Warning: Leff for CV = %g may be too small." << " " << pParam_B4SOIleffCV << " " <<  std::endl; 
}
if ((pParam_B4SOIweff<=1.0e-7))
{
UserInfo(*this)  << "Warning: Weff = %g may be too small." << " " << pParam_B4SOIweff << " " <<  std::endl; 
}
if ((pParam_B4SOIweffCV<=1.0e-7))
{
UserInfo(*this)  << "Warning: Weff for CV = %g may be too small." << " " << pParam_B4SOIweffCV << " " <<  std::endl; 
}
if ((pParam_B4SOIlpe0<0.0))
{
UserInfo(*this)  << "Warning: Lpe0 = %g is negative." << " " << pParam_B4SOIlpe0 << " " <<  std::endl; 
}
if ((B4SOItox<1.0e-9))
{
UserInfo(*this)  << "Warning: Tox = %g is less than 10A." << " " << B4SOItox << " " <<  std::endl; 
}
if ((pParam_B4SOInpeak<=1.0e15))
{
UserInfo(*this)  << "Warning: Nch = %g may be too small." << " " << pParam_B4SOInpeak << " " <<  std::endl; 
}
else
{
if ((pParam_B4SOInpeak>=1.0e21))
{
UserInfo(*this)  << "Warning: Nch = %g may be too large." << " " << pParam_B4SOInpeak << " " <<  std::endl; 
}
}
if ((fabs(pParam_B4SOInsub)>=1.0e21))
{
UserInfo(*this)  << "Warning: Nsub = %g may be too large." << " " << pParam_B4SOInsub << " " <<  std::endl; 
}
if (((pParam_B4SOIngate>0.0)&&(pParam_B4SOIngate<=1.0e18)))
{
UserInfo(*this)  << "Warning: Ngate = %g is less than 1.E18cm^-3." << " " << pParam_B4SOIngate << " " <<  std::endl; 
}
if ((pParam_B4SOIdvt0<0.0))
{
UserInfo(*this)  << "Warning: Dvt0 = %g is negative." << " " << pParam_B4SOIdvt0 << " " <<  std::endl; 
}
if ((fabs((1.0e-6/(pParam_B4SOIw0+pParam_B4SOIweff)))>10.0))
{
UserInfo(*this)  << "Warning: (W0 + Weff) may be too small." << " " <<  std::endl; 
}
if ((B4SOInsd>1.0e23))
{
UserInfo(*this)  << "Warning: Nsd = %g is too large, should be specified in cm^-3." << " " << B4SOInsd << " " <<  std::endl; 
}
if ((B4SOIngate>1.0e23))
{
UserInfo(*this)  << "Warning: Ngate = %g is too large, should be specified in cm^-3." << " " << B4SOIngate << " " <<  std::endl; 
}
if ((pParam_B4SOInfactor<0.0))
{
UserInfo(*this)  << "Warning: Nfactor = %g is negative." << " " << pParam_B4SOInfactor << " " <<  std::endl; 
}
if ((pParam_B4SOIcdsc<0.0))
{
UserInfo(*this)  << "Warning: Cdsc = %g is negative." << " " << pParam_B4SOIcdsc << " " <<  std::endl; 
}
if ((pParam_B4SOIcdscd<0.0))
{
UserInfo(*this)  << "Warning: Cdscd = %g is negative." << " " << pParam_B4SOIcdscd << " " <<  std::endl; 
}
if ((pParam_B4SOIeta0<0.0))
{
UserInfo(*this)  << "Warning: Eta0 = %g is negative." << " " << pParam_B4SOIeta0 << " " <<  std::endl; 
}
if ((pParam_B4SOIeta0cv<0.0))
{
UserInfo(*this)  << "Warning: Eta0CV = %g is negative." << " " << pParam_B4SOIeta0cv << " " <<  std::endl; 
}
if ((fabs((1.0e-6/(pParam_B4SOIb1+pParam_B4SOIweff)))>10.0))
{
UserInfo(*this)  << "Warning: (B1 + Weff) may be too small." << " " <<  std::endl; 
}
if ((pParam_B4SOIvsattemp<1.0e3))
{
UserInfo(*this)  << "Warning: Vsat at current temperature = %g may be too small." << " " << pParam_B4SOIvsattemp << " " <<  std::endl; 
}
if ((pParam_B4SOIpdibl1<0.0))
{
UserInfo(*this)  << "Warning: Pdibl1 = %g is negative." << " " << pParam_B4SOIpdibl1 << " " <<  std::endl; 
}
if ((pParam_B4SOIpdibl2<0.0))
{
UserInfo(*this)  << "Warning: Pdibl2 = %g is negative." << " " << pParam_B4SOIpdibl2 << " " <<  std::endl; 
}
if ((B4SOIntun<0.0))
{
UserInfo(*this)  << "Warning: Ntuns = %g is negative." << " " << B4SOIntun << " " <<  std::endl; 
}
if ((B4SOIntund<0.0))
{
UserInfo(*this)  << "Warning: Ntund = %g is negative." << " " << B4SOIntund << " " <<  std::endl; 
}
if ((B4SOIndiode<0.0))
{
UserInfo(*this)  << "Warning: Ndiode = %g is negative." << " " << B4SOIndiode << " " <<  std::endl; 
}
if ((B4SOIndioded<0.0))
{
UserInfo(*this)  << "Warning: Ndioded = %g is negative." << " " << B4SOIndioded << " " <<  std::endl; 
}
if ((B4SOIisbjt<0.0))
{
UserInfo(*this)  << "Warning: Isbjt = %g is negative." << " " << B4SOIisbjt << " " <<  std::endl; 
}
if ((B4SOIidbjt<0.0))
{
UserInfo(*this)  << "Warning: Idbjt = %g is negative." << " " << B4SOIidbjt << " " <<  std::endl; 
}
if ((B4SOIisdif<0.0))
{
UserInfo(*this)  << "Warning: Isdif = %g is negative." << " " << B4SOIisdif << " " <<  std::endl; 
}
if ((B4SOIiddif<0.0))
{
UserInfo(*this)  << "Warning: Iddif = %g is negative." << " " << B4SOIiddif << " " <<  std::endl; 
}
if ((B4SOIisrec<0.0))
{
UserInfo(*this)  << "Warning: Isrec = %g is negative." << " " << B4SOIisrec << " " <<  std::endl; 
}
if ((B4SOIidrec<0.0))
{
UserInfo(*this)  << "Warning: Idrec = %g is negative." << " " << B4SOIidrec << " " <<  std::endl; 
}
if ((B4SOIistun<0.0))
{
UserInfo(*this)  << "Warning: Istun = %g is negative." << " " << B4SOIistun << " " <<  std::endl; 
}
if ((B4SOIidtun<0.0))
{
UserInfo(*this)  << "Warning: Idtun = %g is negative." << " " << B4SOIidtun << " " <<  std::endl; 
}
if ((B4SOItt<0.0))
{
UserInfo(*this)  << "Warning: Tt = %g is negative." << " " << B4SOItt << " " <<  std::endl; 
}
if ((B4SOIcsdmin<0.0))
{
UserInfo(*this)  << "Warning: Csdmin = %g is negative." << " " << B4SOIcsdmin << " " <<  std::endl; 
}
if ((B4SOIcsdesw<0.0))
{
UserInfo(*this)  << "Warning: Csdesw = %g is negative." << " " << B4SOIcsdesw << " " <<  std::endl; 
}
if ((B4SOIrth0<0.0))
{
UserInfo(*this)  << "Warning: Rth0 = %g is negative." << " " << B4SOIrth0 << " " <<  std::endl; 
}
if ((B4SOIcth0<0.0))
{
UserInfo(*this)  << "Warning: Cth0 = %g is negative." << " " << B4SOIcth0 << " " <<  std::endl; 
}
if ((B4SOIwth0<0.0))
{
UserInfo(*this)  << "Warning:  Wth0 = %g is negative." << " " << B4SOIwth0 << " " <<  std::endl; 
}
if ((B4SOIrbody<0.0))
{
UserInfo(*this)  << "Warning: Rbody = %g is negative." << " " << B4SOIrbody << " " <<  std::endl; 
}
if ((B4SOIrbsh<0.0))
{
UserInfo(*this)  << "Warning: Rbsh = %g is negative." << " " << B4SOIrbsh << " " <<  std::endl; 
}
if ((B4SOIrhalo<0.0))
{
UserInfo(*this)  << "Warning:  Rhalo = %g is negative." << " " << B4SOIrhalo << " " <<  std::endl; 
}
if ((B4SOIntox<0.0))
{
UserInfo(*this)  << "Warning:  Ntox = %g is negative." << " " << B4SOIntox << " " <<  std::endl; 
}
if ((B4SOIebg<0.0))
{
UserInfo(*this)  << "Warning:  Ebg = %g is negative." << " " << B4SOIebg << " " <<  std::endl; 
}
if ((B4SOIvevb<0.0))
{
UserInfo(*this)  << "Warning:  Vevb = %g is negative." << " " << B4SOIvevb << " " <<  std::endl; 
}
if ((pParam_B4SOIalphaGB1<0.0))
{
UserInfo(*this)  << "Warning:  AlphaGB1 = %g is negative." << " " << pParam_B4SOIalphaGB1 << " " <<  std::endl; 
}
if ((pParam_B4SOIbetaGB1<0.0))
{
UserInfo(*this)  << "Warning:  BetaGB1 = %g is negative." << " " << pParam_B4SOIbetaGB1 << " " <<  std::endl; 
}
if ((B4SOIvgb1<0.0))
{
UserInfo(*this)  << "Warning:  Vgb1 = %g is negative." << " " << B4SOIvgb1 << " " <<  std::endl; 
}
if ((B4SOIvecb<0.0))
{
UserInfo(*this)  << "Warning:  Vecb = %g is negative." << " " << B4SOIvecb << " " <<  std::endl; 
}
if ((pParam_B4SOIalphaGB2<0.0))
{
UserInfo(*this)  << "Warning:  AlphaGB2 = %g is negative." << " " << pParam_B4SOIalphaGB2 << " " <<  std::endl; 
}
if ((pParam_B4SOIbetaGB2<0.0))
{
UserInfo(*this)  << "Warning:  BetaGB2 = %g is negative." << " " << pParam_B4SOIbetaGB2 << " " <<  std::endl; 
}
if ((B4SOIvgb2<0.0))
{
UserInfo(*this)  << "Warning:  Vgb2 = %g is negative." << " " << B4SOIvgb2 << " " <<  std::endl; 
}
if ((B4SOIvoxh<0.0))
{
UserInfo(*this)  << "Warning:  Voxh = %g is negative." << " " << B4SOIvoxh << " " <<  std::endl; 
}
if ((B4SOIdeltavox<=0.0))
{
UserInfo(*this)  << "Fatal: Deltavox = %g is not positive." << " " << B4SOIdeltavox << " " <<  std::endl; 
}
if ((B4SOIk1w1<0.0))
{
UserInfo(*this)  << "Warning:  K1w1 = %g is negative." << " " << B4SOIk1w1 << " " <<  std::endl; 
}
if ((B4SOIk1w2<0.0))
{
UserInfo(*this)  << "Warning:  K1w2 = %g is negative." << " " << B4SOIk1w2 << " " <<  std::endl; 
}
if ((B4SOIketas<0.0))
{
UserInfo(*this)  << "Warning:  Ketas = %g is negative." << " " << B4SOIketas << " " <<  std::endl; 
}
if ((B4SOIdwbc<0.0))
{
UserInfo(*this)  << "Warning:  Dwbc = %g is negative." << " " << B4SOIdwbc << " " <<  std::endl; 
}
if ((B4SOIbeta0<0.0))
{
UserInfo(*this)  << "Warning:  Beta0 = %g is negative." << " " << B4SOIbeta0 << " " <<  std::endl; 
}
if ((B4SOIbeta1<0.0))
{
UserInfo(*this)  << "Warning:  Beta1 = %g is negative." << " " << B4SOIbeta1 << " " <<  std::endl; 
}
if ((B4SOIbeta2<0.0))
{
UserInfo(*this)  << "Warning:  Beta2 = %g is negative." << " " << B4SOIbeta2 << " " <<  std::endl; 
}
if ((B4SOIlii<0.0))
{
UserInfo(*this)  << "Warning:  Lii = %g is negative." << " " << B4SOIlii << " " <<  std::endl; 
}
if ((B4SOIsii1<0.0))
{
UserInfo(*this)  << "Warning:  Sii1 = %g is negative." << " " << B4SOIsii1 << " " <<  std::endl; 
}
if ((B4SOIsii2<0.0))
{
UserInfo(*this)  << "Warning:  Sii2 = %g is negative." << " " << B4SOIsii2 << " " <<  std::endl; 
}
if ((B4SOIsiid<0.0))
{
UserInfo(*this)  << "Warning:  Siid = %g is negative." << " " << B4SOIsiid << " " <<  std::endl; 
}
if ((B4SOIfbjtii<0.0))
{
UserInfo(*this)  << "Warning:  fbjtii = %g is negative." << " " << B4SOIfbjtii << " " <<  std::endl; 
}
if ((B4SOIvrec0<0.0))
{
UserInfo(*this)  << "Warning:  Vrec0s = %g is negative." << " " << B4SOIvrec0 << " " <<  std::endl; 
}
if ((B4SOIvrec0d<0.0))
{
UserInfo(*this)  << "Warning:  Vrec0d = %g is negative." << " " << B4SOIvrec0d << " " <<  std::endl; 
}
if ((B4SOIvtun0<0.0))
{
UserInfo(*this)  << "Warning:  Vtun0s = %g is negative." << " " << B4SOIvtun0 << " " <<  std::endl; 
}
if ((B4SOIvtun0d<0.0))
{
UserInfo(*this)  << "Warning:  Vtun0d = %g is negative." << " " << B4SOIvtun0d << " " <<  std::endl; 
}
if ((B4SOInbjt<0.0))
{
UserInfo(*this)  << "Warning:  Nbjt = %g is negative." << " " << B4SOInbjt << " " <<  std::endl; 
}
if ((B4SOIaely<0.0))
{
UserInfo(*this)  << "Warning:  Aely = %g is negative." << " " << B4SOIaely << " " <<  std::endl; 
}
if ((B4SOIahli<0.0))
{
UserInfo(*this)  << "Warning:  Ahlis = %g is negative." << " " << B4SOIahli << " " <<  std::endl; 
}
if ((B4SOIahlid<0.0))
{
UserInfo(*this)  << "Warning:  Ahlid = %g is negative." << " " << B4SOIahlid << " " <<  std::endl; 
}
if (((B4SOIacde<0.1)||(B4SOIacde>1.6)))
{
UserInfo(*this)  << "Warning:  Acde = %g is out of range." << " " << B4SOIacde << " " <<  std::endl; 
}
if (((B4SOImoin<5.0)||(B4SOImoin>25.0)))
{
UserInfo(*this)  << "Warning:  Moin = %g is out of range." << " " << B4SOImoin << " " <<  std::endl; 
}
if ((B4SOIdlbg<0.0))
{
UserInfo(*this)  << "Warning:  dlbg = %g is negative." << " " << B4SOIdlbg << " " <<  std::endl; 
}
if ((B4SOIagidl<0.0))
{
UserInfo(*this)  << "Warning:  Agidl = %g is negative." << " " << B4SOIagidl << " " <<  std::endl; 
}
if ((B4SOIbgidl<0.0))
{
UserInfo(*this)  << "Warning:  Bgidl = %g is negative." << " " << B4SOIbgidl << " " <<  std::endl; 
}
if ((fabs(B4SOIcgidl)<1e-9))
{
UserInfo(*this)  << "Warning:  Cgidl = %g is smaller than 1e-9." << " " << B4SOIcgidl << " " <<  std::endl; 
}
if ((B4SOIegidl<0.0))
{
UserInfo(*this)  << "Warning:  Egidl = %g is negative." << " " << B4SOIegidl << " " <<  std::endl; 
}
if ((B4SOIagisl<0.0))
{
UserInfo(*this)  << "Warning:  Agidl = %g is negative." << " " << B4SOIagidl << " " <<  std::endl; 
}
if ((B4SOIbgisl<0.0))
{
UserInfo(*this)  << "Warning:  Bgisl = %g is negative." << " " << B4SOIbgisl << " " <<  std::endl; 
}
if ((fabs(B4SOIcgisl)<1e-9))
{
UserInfo(*this)  << "Warning:  Cgisl = %g is smaller than 1e-9." << " " << B4SOIcgisl << " " <<  std::endl; 
}
if ((B4SOIegisl<0.0))
{
UserInfo(*this)  << "Warning:  Egisl = %g is negative." << " " << B4SOIegisl << " " <<  std::endl; 
}
if ((B4SOIesatii<0.0))
{
UserInfo(*this)  << "Warning: Esatii = %g is negative." << " " << B4SOIesatii << " " <<  std::endl; 
}
if ((pParam_B4SOIxj>B4SOItsi))
{
UserInfo(*this)  << "Warning: Xj = %g is thicker than Tsi = %g." << " " << pParam_B4SOIxj << " " << B4SOItsi << " " <<  std::endl; 
}
}
if (Fatal_Flag)
{
 UserError(*this) << "$finish called." << std::endl; 
}
}
// End block b4soi_check
if (((SHMOD==1)&&(RTH0!=0.0)))
{
if ((TNODEOUT&&(!portsConnected_[admsNodeID_t])))
{
if (portsConnected_[admsNodeID_b])
{

d_delTemp_dTemp_t_GND = d_delTemp_dV_p_GND =  0.0;
d_delTemp_dV_b_GND = d_probeVars[admsProbeID_V_b_GND][admsProbeID_V_b_GND];
delTemp = (probeVars[admsProbeID_V_b_GND]);
}
else
{
if (portsConnected_[admsNodeID_p])
{

d_delTemp_dTemp_t_GND = d_delTemp_dV_b_GND =  0.0;
d_delTemp_dV_p_GND = d_probeVars[admsProbeID_V_p_GND][admsProbeID_V_p_GND];
delTemp = (probeVars[admsProbeID_V_p_GND]);
}
else
{

d_delTemp_dV_p_GND = d_delTemp_dV_b_GND =  0.0;
d_delTemp_dTemp_t_GND = d_probeVars[admsProbeID_Temp_t_GND][admsProbeID_Temp_t_GND];
delTemp = (probeVars[admsProbeID_Temp_t_GND]);
}
}
}
else
{

d_delTemp_dV_p_GND = d_delTemp_dV_b_GND =  0.0;
d_delTemp_dTemp_t_GND = d_probeVars[admsProbeID_Temp_t_GND][admsProbeID_Temp_t_GND];
delTemp = (probeVars[admsProbeID_Temp_t_GND]);
}
}
else
{

d_delTemp_dTemp_t_GND = d_delTemp_dV_p_GND = d_delTemp_dV_b_GND =  0.0;
delTemp = 0;
}

d_DevTemp_dTemp_t_GND = d_DevTemp_dV_p_GND = d_DevTemp_dV_b_GND =  0.0;
DevTemp = (admsTemperature+DTEMP);

d_DevTemp_dV_b_GND = (d_delTemp_dV_b_GND+d_DevTemp_dV_b_GND);
d_DevTemp_dV_p_GND = (d_delTemp_dV_p_GND+d_DevTemp_dV_p_GND);
d_DevTemp_dTemp_t_GND = (d_delTemp_dTemp_t_GND+d_DevTemp_dTemp_t_GND);
DevTemp = (delTemp+DevTemp);

d_TempRatio_dV_b_GND = (d_DevTemp_dV_b_GND/B4SOItnom);
d_TempRatio_dV_p_GND = (d_DevTemp_dV_p_GND/B4SOItnom);
d_TempRatio_dTemp_t_GND = (d_DevTemp_dTemp_t_GND/B4SOItnom);
TempRatio = (DevTemp/B4SOItnom);

d_TempRatioMinus1_dV_b_GND = (d_DevTemp_dV_b_GND/B4SOItnom);
d_TempRatioMinus1_dV_p_GND = (d_DevTemp_dV_p_GND/B4SOItnom);
d_TempRatioMinus1_dTemp_t_GND = (d_DevTemp_dTemp_t_GND/B4SOItnom);
TempRatioMinus1 = ((DevTemp/B4SOItnom)-1.0);
B4SOITempSH = DevTemp;
T_TOTAL_K = DevTemp;
T_TOTAL_C = (DevTemp-273.15);
T_DELTA_SH = (probeVars[admsProbeID_Temp_t_GND]);
//Begin block load
{
//Block-local variables for block load
double VgstNVt=0.0;
     double d_VgstNVt_dV_db_di=0.0;
     double d_VgstNVt_dV_sb_si=0.0;
     double d_VgstNVt_dV_di_si=0.0;
     double d_VgstNVt_dV_gi_si=0.0;
     double d_VgstNVt_dTemp_t_GND=0.0;
     double d_VgstNVt_dV_p_GND=0.0;
     double d_VgstNVt_dV_b_GND=0.0;
     double d_VgstNVt_dV_b_si=0.0;
     double d_VgstNVt_dV_e_si=0.0;
double ExpVgst=0.0;
     double d_ExpVgst_dV_gi_p=0.0;
     double d_ExpVgst_dV_db_di=0.0;
     double d_ExpVgst_dV_sb_si=0.0;
     double d_ExpVgst_dV_b_si=0.0;
     double d_ExpVgst_dV_di_si=0.0;
     double d_ExpVgst_dV_gi_si=0.0;
     double d_ExpVgst_dV_e_si=0.0;
     double d_ExpVgst_dTemp_t_GND=0.0;
     double d_ExpVgst_dV_p_GND=0.0;
     double d_ExpVgst_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double arg=0.0;
     double d_arg_dV_db_di=0.0;
     double d_arg_dV_sb_si=0.0;
     double d_arg_dTemp_t_GND=0.0;
     double d_arg_dV_p_GND=0.0;
     double d_arg_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vfbeff=0.0;
     double d_Vfbeff_dV_gi_p=0.0;
     double d_Vfbeff_dTemp_t_GND=0.0;
     double d_Vfbeff_dV_p_GND=0.0;
     double d_Vfbeff_dV_b_GND=0.0;
     double d_Vfbeff_dV_db_di=0.0;
     double d_Vfbeff_dV_sb_si=0.0;
     double d_Vfbeff_dV_b_si=0.0;
     double d_Vfbeff_dV_e_si=0.0;
     double d_Vfbeff_dV_di_si=0.0;
     double d_Vfbeff_dV_gi_si=0.0;
double V3=0.0;
     double d_V3_dV_gi_p=0.0;
     double d_V3_dV_db_di=0.0;
     double d_V3_dV_sb_si=0.0;
     double d_V3_dV_b_si=0.0;
     double d_V3_dV_di_si=0.0;
     double d_V3_dV_gi_si=0.0;
     double d_V3_dV_e_si=0.0;
     double d_V3_dTemp_t_GND=0.0;
     double d_V3_dV_p_GND=0.0;
     double d_V3_dV_b_GND=0.0;
double V4=0.0;
     double d_V4_dV_gi_p=0.0;
     double d_V4_dV_db_di=0.0;
     double d_V4_dV_sb_si=0.0;
     double d_V4_dV_di_si=0.0;
     double d_V4_dV_gi_si=0.0;
     double d_V4_dTemp_t_GND=0.0;
     double d_V4_dV_p_GND=0.0;
     double d_V4_dV_b_GND=0.0;
     double d_V4_dV_b_si=0.0;
     double d_V4_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double PhiBSWG=0.0;
     double d_PhiBSWG_dTemp_t_GND=0.0;
     double d_PhiBSWG_dV_p_GND=0.0;
     double d_PhiBSWG_dV_b_GND=0.0;
double MJSWG=0.0;
//End of Block-local variables
//Block-local variables for block load
double vbd=0.0;
     double d_vbd_dV_b_si=0.0;
     double d_vbd_dV_di_si=0.0;
double vbs=0.0;
     double d_vbs_dV_b_si=0.0;
double vds=0.0;
     double d_vds_dV_di_si=0.0;
double vgd=0.0;
     double d_vgd_dV_gi_si=0.0;
     double d_vgd_dV_di_si=0.0;
double vgs=0.0;
     double d_vgs_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vds=0.0;
     double d_Vds_dV_di_si=0.0;
double Vgs=0.0;
     double d_Vgs_dV_di_si=0.0;
     double d_Vgs_dV_gi_si=0.0;
double Vbs=0.0;
     double d_Vbs_dV_gi_si=0.0;
     double d_Vbs_dTemp_t_GND=0.0;
     double d_Vbs_dV_p_GND=0.0;
     double d_Vbs_dV_b_GND=0.0;
     double d_Vbs_dV_e_si=0.0;
     double d_Vbs_dV_di_si=0.0;
     double d_Vbs_dV_b_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgs_eff=0.0;
     double d_Vgs_eff_dV_di_si=0.0;
     double d_Vgs_eff_dV_gi_si=0.0;
     double d_Vgs_eff_dTemp_t_GND=0.0;
     double d_Vgs_eff_dV_p_GND=0.0;
     double d_Vgs_eff_dV_b_GND=0.0;
double Vfb=0.0;
     double d_Vfb_dV_b_si=0.0;
     double d_Vfb_dV_di_si=0.0;
     double d_Vfb_dV_gi_si=0.0;
     double d_Vfb_dV_e_si=0.0;
     double d_Vfb_dTemp_t_GND=0.0;
     double d_Vfb_dV_p_GND=0.0;
     double d_Vfb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Phis=0.0;
     double d_Phis_dV_b_si=0.0;
     double d_Phis_dTemp_t_GND=0.0;
     double d_Phis_dV_p_GND=0.0;
     double d_Phis_dV_b_GND=0.0;
     double d_Phis_dV_di_si=0.0;
     double d_Phis_dV_gi_si=0.0;
     double d_Phis_dV_e_si=0.0;
double sqrtPhis=0.0;
     double d_sqrtPhis_dV_b_si=0.0;
     double d_sqrtPhis_dTemp_t_GND=0.0;
     double d_sqrtPhis_dV_p_GND=0.0;
     double d_sqrtPhis_dV_b_GND=0.0;
     double d_sqrtPhis_dV_di_si=0.0;
     double d_sqrtPhis_dV_gi_si=0.0;
     double d_sqrtPhis_dV_e_si=0.0;
double Vth=0.0;
     double d_Vth_dTemp_t_GND=0.0;
     double d_Vth_dV_p_GND=0.0;
     double d_Vth_dV_b_GND=0.0;
     double d_Vth_dV_b_si=0.0;
     double d_Vth_dV_di_si=0.0;
     double d_Vth_dV_gi_si=0.0;
     double d_Vth_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgst=0.0;
     double d_Vgst_dV_di_si=0.0;
     double d_Vgst_dV_gi_si=0.0;
     double d_Vgst_dTemp_t_GND=0.0;
     double d_Vgst_dV_p_GND=0.0;
     double d_Vgst_dV_b_GND=0.0;
     double d_Vgst_dV_b_si=0.0;
     double d_Vgst_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double n=0.0;
     double d_n_dV_b_si=0.0;
     double d_n_dV_e_si=0.0;
     double d_n_dV_di_si=0.0;
     double d_n_dV_gi_si=0.0;
     double d_n_dTemp_t_GND=0.0;
     double d_n_dV_p_GND=0.0;
     double d_n_dV_b_GND=0.0;
double Vtm=0.0;
     double d_Vtm_dTemp_t_GND=0.0;
     double d_Vtm_dV_p_GND=0.0;
     double d_Vtm_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double ExpArg=0.0;
     double d_ExpArg_dV_di_si=0.0;
     double d_ExpArg_dV_gi_si=0.0;
     double d_ExpArg_dTemp_t_GND=0.0;
     double d_ExpArg_dV_p_GND=0.0;
     double d_ExpArg_dV_b_GND=0.0;
     double d_ExpArg_dV_b_si=0.0;
     double d_ExpArg_dV_e_si=0.0;
double V0=0.0;
     double d_V0_dTemp_t_GND=0.0;
     double d_V0_dV_p_GND=0.0;
     double d_V0_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double ueff=0.0;
     double d_ueff_dTemp_t_GND=0.0;
     double d_ueff_dV_p_GND=0.0;
     double d_ueff_dV_b_GND=0.0;
     double d_ueff_dV_b_si=0.0;
     double d_ueff_dV_e_si=0.0;
     double d_ueff_dV_di_si=0.0;
     double d_ueff_dV_gi_si=0.0;
double Esat=0.0;
     double d_Esat_dTemp_t_GND=0.0;
     double d_Esat_dV_p_GND=0.0;
     double d_Esat_dV_b_GND=0.0;
     double d_Esat_dV_b_si=0.0;
     double d_Esat_dV_e_si=0.0;
     double d_Esat_dV_di_si=0.0;
     double d_Esat_dV_gi_si=0.0;
double Vdsat=0.0;
     double d_Vdsat_dV_b_si=0.0;
     double d_Vdsat_dV_e_si=0.0;
     double d_Vdsat_dV_di_si=0.0;
     double d_Vdsat_dV_gi_si=0.0;
     double d_Vdsat_dTemp_t_GND=0.0;
     double d_Vdsat_dV_p_GND=0.0;
     double d_Vdsat_dV_b_GND=0.0;
double EsatL=0.0;
     double d_EsatL_dTemp_t_GND=0.0;
     double d_EsatL_dV_p_GND=0.0;
     double d_EsatL_dV_b_GND=0.0;
     double d_EsatL_dV_b_si=0.0;
     double d_EsatL_dV_e_si=0.0;
     double d_EsatL_dV_di_si=0.0;
     double d_EsatL_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Va=0.0;
     double d_Va_dV_b_si=0.0;
     double d_Va_dV_di_si=0.0;
     double d_Va_dV_gi_si=0.0;
     double d_Va_dV_e_si=0.0;
     double d_Va_dTemp_t_GND=0.0;
     double d_Va_dV_p_GND=0.0;
     double d_Va_dV_b_GND=0.0;
double Vasat=0.0;
     double d_Vasat_dV_b_si=0.0;
     double d_Vasat_dV_di_si=0.0;
     double d_Vasat_dV_gi_si=0.0;
     double d_Vasat_dV_e_si=0.0;
     double d_Vasat_dTemp_t_GND=0.0;
     double d_Vasat_dV_p_GND=0.0;
     double d_Vasat_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbseff=0.0;
     double d_Vbseff_dV_b_si=0.0;
     double d_Vbseff_dV_di_si=0.0;
     double d_Vbseff_dV_gi_si=0.0;
     double d_Vbseff_dV_e_si=0.0;
     double d_Vbseff_dTemp_t_GND=0.0;
     double d_Vbseff_dV_p_GND=0.0;
     double d_Vbseff_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWL=0.0;
//End of Block-local variables
//Block-local variables for block load
double T0=0.0;
     double d_T0_dV_gm_si=0.0;
     double d_T0_dV_gi_p=0.0;
     double d_T0_dV_db_di=0.0;
     double d_T0_dV_sb_si=0.0;
     double d_T0_dV_b_si=0.0;
     double d_T0_dV_di_si=0.0;
     double d_T0_dV_gi_si=0.0;
     double d_T0_dV_e_si=0.0;
     double d_T0_dTemp_t_GND=0.0;
     double d_T0_dV_p_GND=0.0;
     double d_T0_dV_b_GND=0.0;
double T1=0.0;
     double d_T1_dV_gm_si=0.0;
     double d_T1_dV_gi_p=0.0;
     double d_T1_dV_db_di=0.0;
     double d_T1_dV_sb_si=0.0;
     double d_T1_dV_b_si=0.0;
     double d_T1_dV_e_si=0.0;
     double d_T1_dV_di_si=0.0;
     double d_T1_dV_gi_si=0.0;
     double d_T1_dTemp_t_GND=0.0;
     double d_T1_dV_p_GND=0.0;
     double d_T1_dV_b_GND=0.0;
double T2=0.0;
     double d_T2_dV_gm_si=0.0;
     double d_T2_dV_gi_p=0.0;
     double d_T2_dV_db_di=0.0;
     double d_T2_dV_sb_si=0.0;
     double d_T2_dV_b_si=0.0;
     double d_T2_dV_e_si=0.0;
     double d_T2_dV_di_si=0.0;
     double d_T2_dV_gi_si=0.0;
     double d_T2_dTemp_t_GND=0.0;
     double d_T2_dV_p_GND=0.0;
     double d_T2_dV_b_GND=0.0;
double T3=0.0;
     double d_T3_dV_gi_p=0.0;
     double d_T3_dV_db_di=0.0;
     double d_T3_dV_sb_si=0.0;
     double d_T3_dV_b_si=0.0;
     double d_T3_dV_e_si=0.0;
     double d_T3_dV_di_si=0.0;
     double d_T3_dV_gi_si=0.0;
     double d_T3_dTemp_t_GND=0.0;
     double d_T3_dV_p_GND=0.0;
     double d_T3_dV_b_GND=0.0;
double T4=0.0;
     double d_T4_dV_gm_si=0.0;
     double d_T4_dV_gi_p=0.0;
     double d_T4_dV_db_di=0.0;
     double d_T4_dV_sb_si=0.0;
     double d_T4_dV_b_si=0.0;
     double d_T4_dV_e_si=0.0;
     double d_T4_dV_di_si=0.0;
     double d_T4_dV_gi_si=0.0;
     double d_T4_dTemp_t_GND=0.0;
     double d_T4_dV_p_GND=0.0;
     double d_T4_dV_b_GND=0.0;
double T5=0.0;
     double d_T5_dV_gi_p=0.0;
     double d_T5_dV_db_di=0.0;
     double d_T5_dV_sb_si=0.0;
     double d_T5_dV_b_si=0.0;
     double d_T5_dV_e_si=0.0;
     double d_T5_dV_di_si=0.0;
     double d_T5_dV_gi_si=0.0;
     double d_T5_dTemp_t_GND=0.0;
     double d_T5_dV_p_GND=0.0;
     double d_T5_dV_b_GND=0.0;
double T6=0.0;
     double d_T6_dV_gi_p=0.0;
     double d_T6_dV_db_di=0.0;
     double d_T6_dV_sb_si=0.0;
     double d_T6_dV_b_si=0.0;
     double d_T6_dV_e_si=0.0;
     double d_T6_dV_di_si=0.0;
     double d_T6_dV_gi_si=0.0;
     double d_T6_dTemp_t_GND=0.0;
     double d_T6_dV_p_GND=0.0;
     double d_T6_dV_b_GND=0.0;
double T7=0.0;
     double d_T7_dV_gi_p=0.0;
     double d_T7_dV_db_di=0.0;
     double d_T7_dV_sb_si=0.0;
     double d_T7_dV_b_si=0.0;
     double d_T7_dV_e_si=0.0;
     double d_T7_dV_di_si=0.0;
     double d_T7_dV_gi_si=0.0;
     double d_T7_dTemp_t_GND=0.0;
     double d_T7_dV_p_GND=0.0;
     double d_T7_dV_b_GND=0.0;
double T8=0.0;
     double d_T8_dV_b_si=0.0;
     double d_T8_dV_e_si=0.0;
     double d_T8_dV_di_si=0.0;
     double d_T8_dV_gi_si=0.0;
     double d_T8_dTemp_t_GND=0.0;
     double d_T8_dV_p_GND=0.0;
     double d_T8_dV_b_GND=0.0;
double T9=0.0;
     double d_T9_dV_b_si=0.0;
     double d_T9_dV_e_si=0.0;
     double d_T9_dV_di_si=0.0;
     double d_T9_dV_gi_si=0.0;
     double d_T9_dTemp_t_GND=0.0;
     double d_T9_dV_p_GND=0.0;
     double d_T9_dV_b_GND=0.0;
double T10=0.0;
     double d_T10_dV_db_di=0.0;
     double d_T10_dV_sb_si=0.0;
     double d_T10_dV_b_si=0.0;
     double d_T10_dV_e_si=0.0;
     double d_T10_dV_di_si=0.0;
     double d_T10_dV_gi_si=0.0;
     double d_T10_dTemp_t_GND=0.0;
     double d_T10_dV_p_GND=0.0;
     double d_T10_dV_b_GND=0.0;
double T11=0.0;
     double d_T11_dV_gi_p=0.0;
     double d_T11_dV_db_di=0.0;
     double d_T11_dV_sb_si=0.0;
     double d_T11_dV_b_si=0.0;
     double d_T11_dV_e_si=0.0;
     double d_T11_dV_di_si=0.0;
     double d_T11_dV_gi_si=0.0;
     double d_T11_dTemp_t_GND=0.0;
     double d_T11_dV_p_GND=0.0;
     double d_T11_dV_b_GND=0.0;
double T12=0.0;
     double d_T12_dV_gi_p=0.0;
     double d_T12_dV_db_di=0.0;
     double d_T12_dV_sb_si=0.0;
     double d_T12_dV_b_si=0.0;
     double d_T12_dV_e_si=0.0;
     double d_T12_dV_di_si=0.0;
     double d_T12_dV_gi_si=0.0;
     double d_T12_dTemp_t_GND=0.0;
     double d_T12_dV_p_GND=0.0;
     double d_T12_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double dT2_dVg=0.0;
     double d_dT2_dVg_dTemp_t_GND=0.0;
     double d_dT2_dVg_dV_p_GND=0.0;
     double d_dT2_dVg_dV_b_GND=0.0;
     double d_dT2_dVg_dV_di_si=0.0;
     double d_dT2_dVg_dV_gi_si=0.0;
     double d_dT2_dVg_dV_b_si=0.0;
     double d_dT2_dVg_dV_e_si=0.0;
double dT3_dVb=0.0;
     double d_dT3_dVb_dV_db_di=0.0;
     double d_dT3_dVb_dV_sb_si=0.0;
     double d_dT3_dVb_dTemp_t_GND=0.0;
     double d_dT3_dVb_dV_p_GND=0.0;
     double d_dT3_dVb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double tmp=0.0;
     double d_tmp_dV_gi_p=0.0;
     double d_tmp_dV_db_di=0.0;
     double d_tmp_dV_sb_si=0.0;
     double d_tmp_dV_b_si=0.0;
     double d_tmp_dV_di_si=0.0;
     double d_tmp_dV_gi_si=0.0;
     double d_tmp_dV_e_si=0.0;
     double d_tmp_dTemp_t_GND=0.0;
     double d_tmp_dV_p_GND=0.0;
     double d_tmp_dV_b_GND=0.0;
double Abulk=0.0;
     double d_Abulk_dV_b_si=0.0;
     double d_Abulk_dV_e_si=0.0;
     double d_Abulk_dV_di_si=0.0;
     double d_Abulk_dV_gi_si=0.0;
     double d_Abulk_dTemp_t_GND=0.0;
     double d_Abulk_dV_p_GND=0.0;
     double d_Abulk_dV_b_GND=0.0;
double Abulk0=0.0;
     double d_Abulk0_dV_b_si=0.0;
     double d_Abulk0_dV_e_si=0.0;
     double d_Abulk0_dV_di_si=0.0;
     double d_Abulk0_dV_gi_si=0.0;
     double d_Abulk0_dTemp_t_GND=0.0;
     double d_Abulk0_dV_p_GND=0.0;
     double d_Abulk0_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double VACLM=0.0;
     double d_VACLM_dV_b_si=0.0;
     double d_VACLM_dV_e_si=0.0;
     double d_VACLM_dV_di_si=0.0;
     double d_VACLM_dV_gi_si=0.0;
     double d_VACLM_dTemp_t_GND=0.0;
     double d_VACLM_dV_p_GND=0.0;
     double d_VACLM_dV_b_GND=0.0;
double VADIBL=0.0;
     double d_VADIBL_dV_di_si=0.0;
     double d_VADIBL_dV_gi_si=0.0;
     double d_VADIBL_dTemp_t_GND=0.0;
     double d_VADIBL_dV_p_GND=0.0;
     double d_VADIBL_dV_b_GND=0.0;
     double d_VADIBL_dV_b_si=0.0;
     double d_VADIBL_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Xdep=0.0;
     double d_Xdep_dV_b_si=0.0;
     double d_Xdep_dTemp_t_GND=0.0;
     double d_Xdep_dV_p_GND=0.0;
     double d_Xdep_dV_b_GND=0.0;
     double d_Xdep_dV_di_si=0.0;
     double d_Xdep_dV_gi_si=0.0;
     double d_Xdep_dV_e_si=0.0;
double lt1=0.0;
     double d_lt1_dV_b_si=0.0;
     double d_lt1_dV_e_si=0.0;
     double d_lt1_dV_di_si=0.0;
     double d_lt1_dV_gi_si=0.0;
     double d_lt1_dTemp_t_GND=0.0;
     double d_lt1_dV_p_GND=0.0;
     double d_lt1_dV_b_GND=0.0;
double ltw=0.0;
     double d_ltw_dV_b_si=0.0;
     double d_ltw_dV_e_si=0.0;
     double d_ltw_dV_di_si=0.0;
     double d_ltw_dV_gi_si=0.0;
     double d_ltw_dTemp_t_GND=0.0;
     double d_ltw_dV_p_GND=0.0;
     double d_ltw_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Delt_vth=0.0;
     double d_Delt_vth_dV_b_si=0.0;
     double d_Delt_vth_dV_e_si=0.0;
     double d_Delt_vth_dV_di_si=0.0;
     double d_Delt_vth_dV_gi_si=0.0;
     double d_Delt_vth_dTemp_t_GND=0.0;
     double d_Delt_vth_dV_p_GND=0.0;
     double d_Delt_vth_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Theta0=0.0;
     double d_Theta0_dV_b_si=0.0;
     double d_Theta0_dV_e_si=0.0;
     double d_Theta0_dV_di_si=0.0;
     double d_Theta0_dV_gi_si=0.0;
     double d_Theta0_dTemp_t_GND=0.0;
     double d_Theta0_dV_p_GND=0.0;
     double d_Theta0_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double tmp1=0.0;
     double d_tmp1_dV_b_si=0.0;
     double d_tmp1_dV_e_si=0.0;
     double d_tmp1_dV_di_si=0.0;
     double d_tmp1_dV_gi_si=0.0;
     double d_tmp1_dTemp_t_GND=0.0;
     double d_tmp1_dV_p_GND=0.0;
     double d_tmp1_dV_b_GND=0.0;
double tmp2=0.0;
     double d_tmp2_dV_b_si=0.0;
     double d_tmp2_dV_e_si=0.0;
     double d_tmp2_dV_di_si=0.0;
     double d_tmp2_dV_gi_si=0.0;
     double d_tmp2_dTemp_t_GND=0.0;
     double d_tmp2_dV_p_GND=0.0;
     double d_tmp2_dV_b_GND=0.0;
double tmp3=0.0;
     double d_tmp3_dV_b_si=0.0;
     double d_tmp3_dV_e_si=0.0;
     double d_tmp3_dV_di_si=0.0;
     double d_tmp3_dV_gi_si=0.0;
     double d_tmp3_dTemp_t_GND=0.0;
     double d_tmp3_dV_p_GND=0.0;
     double d_tmp3_dV_b_GND=0.0;
double tmp4=0.0;
     double d_tmp4_dV_b_si=0.0;
     double d_tmp4_dV_e_si=0.0;
     double d_tmp4_dV_di_si=0.0;
     double d_tmp4_dV_gi_si=0.0;
     double d_tmp4_dTemp_t_GND=0.0;
     double d_tmp4_dV_p_GND=0.0;
     double d_tmp4_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double DIBL_Sft=0.0;
     double d_DIBL_Sft_dV_b_si=0.0;
     double d_DIBL_Sft_dV_e_si=0.0;
     double d_DIBL_Sft_dV_di_si=0.0;
     double d_DIBL_Sft_dV_gi_si=0.0;
     double d_DIBL_Sft_dTemp_t_GND=0.0;
     double d_DIBL_Sft_dV_p_GND=0.0;
     double d_DIBL_Sft_dV_b_GND=0.0;
double Lambda=0.0;
     double d_Lambda_dV_b_si=0.0;
     double d_Lambda_dV_di_si=0.0;
     double d_Lambda_dV_gi_si=0.0;
     double d_Lambda_dV_e_si=0.0;
     double d_Lambda_dTemp_t_GND=0.0;
     double d_Lambda_dV_p_GND=0.0;
     double d_Lambda_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgsteff=0.0;
     double d_Vgsteff_dV_gi_p=0.0;
     double d_Vgsteff_dV_db_di=0.0;
     double d_Vgsteff_dV_sb_si=0.0;
     double d_Vgsteff_dV_di_si=0.0;
     double d_Vgsteff_dV_gi_si=0.0;
     double d_Vgsteff_dTemp_t_GND=0.0;
     double d_Vgsteff_dV_p_GND=0.0;
     double d_Vgsteff_dV_b_GND=0.0;
     double d_Vgsteff_dV_b_si=0.0;
     double d_Vgsteff_dV_e_si=0.0;
double Vdseff=0.0;
     double d_Vdseff_dV_b_si=0.0;
     double d_Vdseff_dV_e_si=0.0;
     double d_Vdseff_dV_di_si=0.0;
     double d_Vdseff_dV_gi_si=0.0;
     double d_Vdseff_dTemp_t_GND=0.0;
     double d_Vdseff_dV_p_GND=0.0;
     double d_Vdseff_dV_b_GND=0.0;
double VdseffCV=0.0;
     double d_VdseffCV_dV_gi_p=0.0;
     double d_VdseffCV_dV_db_di=0.0;
     double d_VdseffCV_dV_sb_si=0.0;
     double d_VdseffCV_dV_di_si=0.0;
     double d_VdseffCV_dV_gi_si=0.0;
     double d_VdseffCV_dTemp_t_GND=0.0;
     double d_VdseffCV_dV_p_GND=0.0;
     double d_VdseffCV_dV_b_GND=0.0;
     double d_VdseffCV_dV_b_si=0.0;
     double d_VdseffCV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double diffVds=0.0;
     double d_diffVds_dV_di_si=0.0;
     double d_diffVds_dV_b_si=0.0;
     double d_diffVds_dV_e_si=0.0;
     double d_diffVds_dV_gi_si=0.0;
     double d_diffVds_dTemp_t_GND=0.0;
     double d_diffVds_dV_p_GND=0.0;
     double d_diffVds_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double dAbulk_dVg=0.0;
     double d_dAbulk_dVg_dV_b_si=0.0;
     double d_dAbulk_dVg_dV_e_si=0.0;
     double d_dAbulk_dVg_dV_di_si=0.0;
     double d_dAbulk_dVg_dV_gi_si=0.0;
     double d_dAbulk_dVg_dTemp_t_GND=0.0;
     double d_dAbulk_dVg_dV_p_GND=0.0;
     double d_dAbulk_dVg_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double beta=0.0;
     double d_beta_dTemp_t_GND=0.0;
     double d_beta_dV_p_GND=0.0;
     double d_beta_dV_b_GND=0.0;
     double d_beta_dV_b_si=0.0;
     double d_beta_dV_e_si=0.0;
     double d_beta_dV_di_si=0.0;
     double d_beta_dV_gi_si=0.0;
double gche=0.0;
     double d_gche_dTemp_t_GND=0.0;
     double d_gche_dV_p_GND=0.0;
     double d_gche_dV_b_GND=0.0;
     double d_gche_dV_b_si=0.0;
     double d_gche_dV_e_si=0.0;
     double d_gche_dV_di_si=0.0;
     double d_gche_dV_gi_si=0.0;
double fgche1=0.0;
     double d_fgche1_dV_di_si=0.0;
     double d_fgche1_dV_gi_si=0.0;
     double d_fgche1_dTemp_t_GND=0.0;
     double d_fgche1_dV_p_GND=0.0;
     double d_fgche1_dV_b_GND=0.0;
     double d_fgche1_dV_b_si=0.0;
     double d_fgche1_dV_e_si=0.0;
double fgche2=0.0;
     double d_fgche2_dV_b_si=0.0;
     double d_fgche2_dV_e_si=0.0;
     double d_fgche2_dV_di_si=0.0;
     double d_fgche2_dV_gi_si=0.0;
     double d_fgche2_dTemp_t_GND=0.0;
     double d_fgche2_dV_p_GND=0.0;
     double d_fgche2_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Idl=0.0;
     double d_Idl_dTemp_t_GND=0.0;
     double d_Idl_dV_p_GND=0.0;
     double d_Idl_dV_b_GND=0.0;
     double d_Idl_dV_b_si=0.0;
     double d_Idl_dV_e_si=0.0;
     double d_Idl_dV_di_si=0.0;
     double d_Idl_dV_gi_si=0.0;
double Ids=0.0;
     double d_Ids_dTemp_t_GND=0.0;
     double d_Ids_dV_p_GND=0.0;
     double d_Ids_dV_b_GND=0.0;
     double d_Ids_dV_b_si=0.0;
     double d_Ids_dV_e_si=0.0;
     double d_Ids_dV_di_si=0.0;
     double d_Ids_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWovL=0.0;
     double d_CoxWovL_dV_di_si=0.0;
     double d_CoxWovL_dV_gi_si=0.0;
     double d_CoxWovL_dTemp_t_GND=0.0;
     double d_CoxWovL_dV_p_GND=0.0;
     double d_CoxWovL_dV_b_GND=0.0;
     double d_CoxWovL_dV_b_si=0.0;
     double d_CoxWovL_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Rds=0.0;
     double d_Rds_dTemp_t_GND=0.0;
     double d_Rds_dV_p_GND=0.0;
     double d_Rds_dV_b_GND=0.0;
     double d_Rds_dV_b_si=0.0;
     double d_Rds_dV_di_si=0.0;
     double d_Rds_dV_gi_si=0.0;
     double d_Rds_dV_e_si=0.0;
double WVCox=0.0;
     double d_WVCox_dV_di_si=0.0;
     double d_WVCox_dV_gi_si=0.0;
     double d_WVCox_dTemp_t_GND=0.0;
     double d_WVCox_dV_p_GND=0.0;
     double d_WVCox_dV_b_GND=0.0;
     double d_WVCox_dV_b_si=0.0;
     double d_WVCox_dV_e_si=0.0;
double WVCoxRds=0.0;
     double d_WVCoxRds_dV_di_si=0.0;
     double d_WVCoxRds_dV_gi_si=0.0;
     double d_WVCoxRds_dTemp_t_GND=0.0;
     double d_WVCoxRds_dV_p_GND=0.0;
     double d_WVCoxRds_dV_b_GND=0.0;
     double d_WVCoxRds_dV_b_si=0.0;
     double d_WVCoxRds_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgst2Vtm=0.0;
     double d_Vgst2Vtm_dV_di_si=0.0;
     double d_Vgst2Vtm_dV_gi_si=0.0;
     double d_Vgst2Vtm_dTemp_t_GND=0.0;
     double d_Vgst2Vtm_dV_p_GND=0.0;
     double d_Vgst2Vtm_dV_b_GND=0.0;
     double d_Vgst2Vtm_dV_b_si=0.0;
     double d_Vgst2Vtm_dV_e_si=0.0;
double VdsatCV=0.0;
     double d_VdsatCV_dV_gi_p=0.0;
     double d_VdsatCV_dV_db_di=0.0;
     double d_VdsatCV_dV_sb_si=0.0;
     double d_VdsatCV_dV_di_si=0.0;
     double d_VdsatCV_dV_gi_si=0.0;
     double d_VdsatCV_dTemp_t_GND=0.0;
     double d_VdsatCV_dV_p_GND=0.0;
     double d_VdsatCV_dV_b_GND=0.0;
     double d_VdsatCV_dV_b_si=0.0;
     double d_VdsatCV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Leff=0.0;
double Weff=0.0;
     double d_Weff_dV_di_si=0.0;
     double d_Weff_dV_gi_si=0.0;
     double d_Weff_dTemp_t_GND=0.0;
     double d_Weff_dV_p_GND=0.0;
     double d_Weff_dV_b_GND=0.0;
     double d_Weff_dV_b_si=0.0;
     double d_Weff_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double AbulkCV=0.0;
     double d_AbulkCV_dV_b_si=0.0;
     double d_AbulkCV_dV_e_si=0.0;
     double d_AbulkCV_dV_di_si=0.0;
     double d_AbulkCV_dV_gi_si=0.0;
     double d_AbulkCV_dTemp_t_GND=0.0;
     double d_AbulkCV_dV_p_GND=0.0;
     double d_AbulkCV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double qgdo=0.0;
     double d_qgdo_dV_gi_p=0.0;
     double d_qgdo_dV_db_di=0.0;
     double d_qgdo_dV_sb_si=0.0;
     double d_qgdo_dV_b_si=0.0;
     double d_qgdo_dV_e_si=0.0;
     double d_qgdo_dV_di_si=0.0;
     double d_qgdo_dV_gi_si=0.0;
     double d_qgdo_dTemp_t_GND=0.0;
     double d_qgdo_dV_p_GND=0.0;
     double d_qgdo_dV_b_GND=0.0;
     double d_qgdo_dV_gm_si=0.0;
double qgso=0.0;
     double d_qgso_dV_gi_p=0.0;
     double d_qgso_dV_db_di=0.0;
     double d_qgso_dV_sb_si=0.0;
     double d_qgso_dV_b_si=0.0;
     double d_qgso_dV_e_si=0.0;
     double d_qgso_dV_di_si=0.0;
     double d_qgso_dV_gi_si=0.0;
     double d_qgso_dTemp_t_GND=0.0;
     double d_qgso_dV_p_GND=0.0;
     double d_qgso_dV_b_GND=0.0;
     double d_qgso_dV_gm_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double ves=0.0;
     double d_ves_dV_e_si=0.0;
double ved=0.0;
     double d_ved_dV_e_si=0.0;
     double d_ved_dV_di_si=0.0;
double vbp=0.0;
     double d_vbp_dV_b_p=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbd=0.0;
     double d_Vbd_dV_b_si=0.0;
     double d_Vbd_dV_di_si=0.0;
double Ves=0.0;
     double d_Ves_dV_di_si=0.0;
     double d_Ves_dV_e_si=0.0;
double Vesfb=0.0;
     double d_Vesfb_dV_di_si=0.0;
     double d_Vesfb_dV_e_si=0.0;
     double d_Vesfb_dTemp_t_GND=0.0;
     double d_Vesfb_dV_p_GND=0.0;
     double d_Vesfb_dV_b_GND=0.0;
double DeltVthtemp=0.0;
     double d_DeltVthtemp_dV_b_si=0.0;
     double d_DeltVthtemp_dV_di_si=0.0;
     double d_DeltVthtemp_dV_gi_si=0.0;
     double d_DeltVthtemp_dV_e_si=0.0;
     double d_DeltVthtemp_dTemp_t_GND=0.0;
     double d_DeltVthtemp_dV_p_GND=0.0;
     double d_DeltVthtemp_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltVthw=0.0;
     double d_DeltVthw_dV_b_si=0.0;
     double d_DeltVthw_dV_di_si=0.0;
     double d_DeltVthw_dV_gi_si=0.0;
     double d_DeltVthw_dV_e_si=0.0;
     double d_DeltVthw_dTemp_t_GND=0.0;
     double d_DeltVthw_dV_p_GND=0.0;
     double d_DeltVthw_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igidl=0.0;
     double d_Igidl_dV_b_si=0.0;
     double d_Igidl_dV_e_si=0.0;
     double d_Igidl_dV_di_si=0.0;
     double d_Igidl_dV_gi_si=0.0;
     double d_Igidl_dTemp_t_GND=0.0;
     double d_Igidl_dV_p_GND=0.0;
     double d_Igidl_dV_b_GND=0.0;
double Igisl=0.0;
     double d_Igisl_dV_b_si=0.0;
     double d_Igisl_dV_e_si=0.0;
     double d_Igisl_dV_di_si=0.0;
     double d_Igisl_dV_gi_si=0.0;
     double d_Igisl_dTemp_t_GND=0.0;
     double d_Igisl_dV_p_GND=0.0;
     double d_Igisl_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibp=0.0;
     double d_Ibp_dV_b_p=0.0;
     double d_Ibp_dV_gi_p=0.0;
     double d_Ibp_dV_db_di=0.0;
     double d_Ibp_dV_sb_si=0.0;
     double d_Ibp_dV_b_si=0.0;
     double d_Ibp_dV_di_si=0.0;
     double d_Ibp_dV_gi_si=0.0;
     double d_Ibp_dV_e_si=0.0;
     double d_Ibp_dTemp_t_GND=0.0;
     double d_Ibp_dV_p_GND=0.0;
     double d_Ibp_dV_b_GND=0.0;
double Iii=0.0;
     double d_Iii_dV_di_si=0.0;
     double d_Iii_dTemp_t_GND=0.0;
     double d_Iii_dV_p_GND=0.0;
     double d_Iii_dV_b_GND=0.0;
     double d_Iii_dV_gi_p=0.0;
     double d_Iii_dV_db_di=0.0;
     double d_Iii_dV_sb_si=0.0;
     double d_Iii_dV_b_si=0.0;
     double d_Iii_dV_e_si=0.0;
     double d_Iii_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double qjd=0.0;
     double d_qjd_dTemp_t_GND=0.0;
     double d_qjd_dV_p_GND=0.0;
     double d_qjd_dV_b_GND=0.0;
     double d_qjd_dV_gi_p=0.0;
     double d_qjd_dV_db_di=0.0;
     double d_qjd_dV_sb_si=0.0;
     double d_qjd_dV_b_si=0.0;
     double d_qjd_dV_e_si=0.0;
     double d_qjd_dV_di_si=0.0;
     double d_qjd_dV_gi_si=0.0;
double qjs=0.0;
     double d_qjs_dTemp_t_GND=0.0;
     double d_qjs_dV_p_GND=0.0;
     double d_qjs_dV_b_GND=0.0;
     double d_qjs_dV_gi_p=0.0;
     double d_qjs_dV_db_di=0.0;
     double d_qjs_dV_sb_si=0.0;
     double d_qjs_dV_b_si=0.0;
     double d_qjs_dV_e_si=0.0;
     double d_qjs_dV_di_si=0.0;
     double d_qjs_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double ni=0.0;
     double d_ni_dTemp_t_GND=0.0;
     double d_ni_dV_p_GND=0.0;
     double d_ni_dV_b_GND=0.0;
double Eg=0.0;
     double d_Eg_dTemp_t_GND=0.0;
     double d_Eg_dV_p_GND=0.0;
     double d_Eg_dV_b_GND=0.0;
double Cbox=0.0;
double CboxWL=0.0;
//End of Block-local variables
//Block-local variables for block load
double qinv=0.0;
     double d_qinv_dV_gi_p=0.0;
     double d_qinv_dV_db_di=0.0;
     double d_qinv_dV_sb_si=0.0;
     double d_qinv_dV_di_si=0.0;
     double d_qinv_dV_gi_si=0.0;
     double d_qinv_dTemp_t_GND=0.0;
     double d_qinv_dV_p_GND=0.0;
     double d_qinv_dV_b_GND=0.0;
     double d_qinv_dV_b_si=0.0;
     double d_qinv_dV_e_si=0.0;
double qgate=0.0;
     double d_qgate_dV_gm_si=0.0;
     double d_qgate_dV_gi_p=0.0;
     double d_qgate_dV_db_di=0.0;
     double d_qgate_dV_sb_si=0.0;
     double d_qgate_dV_di_si=0.0;
     double d_qgate_dV_gi_si=0.0;
     double d_qgate_dTemp_t_GND=0.0;
     double d_qgate_dV_p_GND=0.0;
     double d_qgate_dV_b_GND=0.0;
     double d_qgate_dV_b_si=0.0;
     double d_qgate_dV_e_si=0.0;
double qbody=0.0;
     double d_qbody_dV_b_si=0.0;
     double d_qbody_dV_e_si=0.0;
     double d_qbody_dV_di_si=0.0;
     double d_qbody_dV_gi_si=0.0;
     double d_qbody_dTemp_t_GND=0.0;
     double d_qbody_dV_p_GND=0.0;
     double d_qbody_dV_b_GND=0.0;
     double d_qbody_dV_gi_p=0.0;
     double d_qbody_dV_db_di=0.0;
     double d_qbody_dV_sb_si=0.0;
double qdrn=0.0;
     double d_qdrn_dV_gi_p=0.0;
     double d_qdrn_dV_db_di=0.0;
     double d_qdrn_dV_sb_si=0.0;
     double d_qdrn_dV_di_si=0.0;
     double d_qdrn_dV_gi_si=0.0;
     double d_qdrn_dTemp_t_GND=0.0;
     double d_qdrn_dV_p_GND=0.0;
     double d_qdrn_dV_b_GND=0.0;
     double d_qdrn_dV_b_si=0.0;
     double d_qdrn_dV_e_si=0.0;
double qsrc=0.0;
     double d_qsrc_dV_gi_p=0.0;
     double d_qsrc_dV_db_di=0.0;
     double d_qsrc_dV_sb_si=0.0;
     double d_qsrc_dV_di_si=0.0;
     double d_qsrc_dV_gi_si=0.0;
     double d_qsrc_dTemp_t_GND=0.0;
     double d_qsrc_dV_p_GND=0.0;
     double d_qsrc_dV_b_GND=0.0;
     double d_qsrc_dV_b_si=0.0;
     double d_qsrc_dV_e_si=0.0;
double qsub=0.0;
     double d_qsub_dV_di_si=0.0;
     double d_qsub_dV_e_si=0.0;
     double d_qsub_dTemp_t_GND=0.0;
     double d_qsub_dV_p_GND=0.0;
     double d_qsub_dV_b_GND=0.0;
     double d_qsub_dV_gi_si=0.0;
     double d_qsub_dV_b_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vdsatii=0.0;
     double d_Vdsatii_dTemp_t_GND=0.0;
     double d_Vdsatii_dV_p_GND=0.0;
     double d_Vdsatii_dV_b_GND=0.0;
     double d_Vdsatii_dV_gi_p=0.0;
     double d_Vdsatii_dV_db_di=0.0;
     double d_Vdsatii_dV_sb_si=0.0;
     double d_Vdsatii_dV_b_si=0.0;
     double d_Vdsatii_dV_e_si=0.0;
     double d_Vdsatii_dV_di_si=0.0;
     double d_Vdsatii_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibd1=0.0;
     double d_Ibd1_dV_db_di=0.0;
     double d_Ibd1_dV_sb_si=0.0;
     double d_Ibd1_dV_b_si=0.0;
     double d_Ibd1_dV_di_si=0.0;
     double d_Ibd1_dV_gi_si=0.0;
     double d_Ibd1_dV_e_si=0.0;
     double d_Ibd1_dTemp_t_GND=0.0;
     double d_Ibd1_dV_p_GND=0.0;
     double d_Ibd1_dV_b_GND=0.0;
double Ibd2=0.0;
     double d_Ibd2_dV_b_si=0.0;
     double d_Ibd2_dV_e_si=0.0;
     double d_Ibd2_dV_di_si=0.0;
     double d_Ibd2_dV_gi_si=0.0;
     double d_Ibd2_dTemp_t_GND=0.0;
     double d_Ibd2_dV_p_GND=0.0;
     double d_Ibd2_dV_b_GND=0.0;
     double d_Ibd2_dV_db_di=0.0;
     double d_Ibd2_dV_sb_si=0.0;
double Ibd3=0.0;
     double d_Ibd3_dV_db_di=0.0;
     double d_Ibd3_dV_sb_si=0.0;
     double d_Ibd3_dV_b_si=0.0;
     double d_Ibd3_dV_e_si=0.0;
     double d_Ibd3_dV_di_si=0.0;
     double d_Ibd3_dV_gi_si=0.0;
     double d_Ibd3_dTemp_t_GND=0.0;
     double d_Ibd3_dV_p_GND=0.0;
     double d_Ibd3_dV_b_GND=0.0;
double Ibd4=0.0;
     double d_Ibd4_dV_db_di=0.0;
     double d_Ibd4_dV_sb_si=0.0;
     double d_Ibd4_dV_b_si=0.0;
     double d_Ibd4_dV_e_si=0.0;
     double d_Ibd4_dV_di_si=0.0;
     double d_Ibd4_dV_gi_si=0.0;
     double d_Ibd4_dTemp_t_GND=0.0;
     double d_Ibd4_dV_p_GND=0.0;
     double d_Ibd4_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibs1=0.0;
     double d_Ibs1_dV_db_di=0.0;
     double d_Ibs1_dV_sb_si=0.0;
     double d_Ibs1_dV_b_si=0.0;
     double d_Ibs1_dV_di_si=0.0;
     double d_Ibs1_dV_gi_si=0.0;
     double d_Ibs1_dV_e_si=0.0;
     double d_Ibs1_dTemp_t_GND=0.0;
     double d_Ibs1_dV_p_GND=0.0;
     double d_Ibs1_dV_b_GND=0.0;
double Ibs2=0.0;
     double d_Ibs2_dV_b_si=0.0;
     double d_Ibs2_dV_e_si=0.0;
     double d_Ibs2_dV_di_si=0.0;
     double d_Ibs2_dV_gi_si=0.0;
     double d_Ibs2_dTemp_t_GND=0.0;
     double d_Ibs2_dV_p_GND=0.0;
     double d_Ibs2_dV_b_GND=0.0;
     double d_Ibs2_dV_db_di=0.0;
     double d_Ibs2_dV_sb_si=0.0;
double Ibs3=0.0;
     double d_Ibs3_dV_db_di=0.0;
     double d_Ibs3_dV_sb_si=0.0;
     double d_Ibs3_dV_b_si=0.0;
     double d_Ibs3_dV_e_si=0.0;
     double d_Ibs3_dV_di_si=0.0;
     double d_Ibs3_dV_gi_si=0.0;
     double d_Ibs3_dTemp_t_GND=0.0;
     double d_Ibs3_dV_p_GND=0.0;
     double d_Ibs3_dV_b_GND=0.0;
double Ibs4=0.0;
     double d_Ibs4_dV_db_di=0.0;
     double d_Ibs4_dV_sb_si=0.0;
     double d_Ibs4_dV_b_si=0.0;
     double d_Ibs4_dV_e_si=0.0;
     double d_Ibs4_dV_di_si=0.0;
     double d_Ibs4_dV_gi_si=0.0;
     double d_Ibs4_dTemp_t_GND=0.0;
     double d_Ibs4_dV_p_GND=0.0;
     double d_Ibs4_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double WTsi=0.0;
double NVtm1=0.0;
     double d_NVtm1_dTemp_t_GND=0.0;
     double d_NVtm1_dV_p_GND=0.0;
     double d_NVtm1_dV_b_GND=0.0;
double NVtm2=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ic=0.0;
     double d_Ic_dV_db_di=0.0;
     double d_Ic_dV_sb_si=0.0;
     double d_Ic_dV_b_si=0.0;
     double d_Ic_dV_di_si=0.0;
     double d_Ic_dV_gi_si=0.0;
     double d_Ic_dV_e_si=0.0;
     double d_Ic_dTemp_t_GND=0.0;
     double d_Ic_dV_p_GND=0.0;
     double d_Ic_dV_b_GND=0.0;
double Ibs=0.0;
     double d_Ibs_dV_db_di=0.0;
     double d_Ibs_dV_sb_si=0.0;
     double d_Ibs_dV_b_si=0.0;
     double d_Ibs_dV_di_si=0.0;
     double d_Ibs_dV_gi_si=0.0;
     double d_Ibs_dV_e_si=0.0;
     double d_Ibs_dTemp_t_GND=0.0;
     double d_Ibs_dV_p_GND=0.0;
     double d_Ibs_dV_b_GND=0.0;
double Ibd=0.0;
     double d_Ibd_dV_db_di=0.0;
     double d_Ibd_dV_sb_si=0.0;
     double d_Ibd_dV_b_si=0.0;
     double d_Ibd_dV_di_si=0.0;
     double d_Ibd_dV_gi_si=0.0;
     double d_Ibd_dV_e_si=0.0;
     double d_Ibd_dTemp_t_GND=0.0;
     double d_Ibd_dV_p_GND=0.0;
     double d_Ibd_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Denomi=0.0;
     double d_Denomi_dV_b_si=0.0;
     double d_Denomi_dV_e_si=0.0;
     double d_Denomi_dV_di_si=0.0;
     double d_Denomi_dV_gi_si=0.0;
     double d_Denomi_dTemp_t_GND=0.0;
     double d_Denomi_dV_p_GND=0.0;
     double d_Denomi_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Qsub0=0.0;
     double d_Qsub0_dV_gi_p=0.0;
     double d_Qsub0_dV_db_di=0.0;
     double d_Qsub0_dV_sb_si=0.0;
     double d_Qsub0_dV_b_si=0.0;
     double d_Qsub0_dV_e_si=0.0;
     double d_Qsub0_dV_di_si=0.0;
     double d_Qsub0_dV_gi_si=0.0;
     double d_Qsub0_dTemp_t_GND=0.0;
     double d_Qsub0_dV_p_GND=0.0;
     double d_Qsub0_dV_b_GND=0.0;
double Qac0=0.0;
     double d_Qac0_dV_gi_p=0.0;
     double d_Qac0_dTemp_t_GND=0.0;
     double d_Qac0_dV_p_GND=0.0;
     double d_Qac0_dV_b_GND=0.0;
     double d_Qac0_dV_db_di=0.0;
     double d_Qac0_dV_sb_si=0.0;
     double d_Qac0_dV_b_si=0.0;
     double d_Qac0_dV_e_si=0.0;
     double d_Qac0_dV_di_si=0.0;
     double d_Qac0_dV_gi_si=0.0;
double Qe1=0.0;
     double d_Qe1_dV_di_si=0.0;
     double d_Qe1_dV_e_si=0.0;
     double d_Qe1_dTemp_t_GND=0.0;
     double d_Qe1_dV_p_GND=0.0;
     double d_Qe1_dV_b_GND=0.0;
     double d_Qe1_dV_gi_si=0.0;
     double d_Qe1_dV_b_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double vbi=0.0;
     double d_vbi_dTemp_t_GND=0.0;
     double d_vbi_dV_p_GND=0.0;
     double d_vbi_dV_b_GND=0.0;
double vfbb=0.0;
     double d_vfbb_dTemp_t_GND=0.0;
     double d_vfbb_dV_p_GND=0.0;
     double d_vfbb_dV_b_GND=0.0;
double phi=0.0;
     double d_phi_dTemp_t_GND=0.0;
     double d_phi_dV_p_GND=0.0;
     double d_phi_dV_b_GND=0.0;
double sqrtPhi=0.0;
     double d_sqrtPhi_dTemp_t_GND=0.0;
     double d_sqrtPhi_dV_p_GND=0.0;
     double d_sqrtPhi_dV_b_GND=0.0;
double Xdep0=0.0;
     double d_Xdep0_dTemp_t_GND=0.0;
     double d_Xdep0_dV_p_GND=0.0;
     double d_Xdep0_dV_b_GND=0.0;
double u0temp=0.0;
     double d_u0temp_dTemp_t_GND=0.0;
     double d_u0temp_dV_p_GND=0.0;
     double d_u0temp_dV_b_GND=0.0;
double vsattemp=0.0;
     double d_vsattemp_dTemp_t_GND=0.0;
     double d_vsattemp_dV_p_GND=0.0;
     double d_vsattemp_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double jdifs=0.0;
     double d_jdifs_dTemp_t_GND=0.0;
     double d_jdifs_dV_p_GND=0.0;
     double d_jdifs_dV_b_GND=0.0;
double jdifd=0.0;
     double d_jdifd_dTemp_t_GND=0.0;
     double d_jdifd_dV_p_GND=0.0;
     double d_jdifd_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double jbjts=0.0;
     double d_jbjts_dTemp_t_GND=0.0;
     double d_jbjts_dV_p_GND=0.0;
     double d_jbjts_dV_b_GND=0.0;
double jbjtd=0.0;
     double d_jbjtd_dTemp_t_GND=0.0;
     double d_jbjtd_dV_p_GND=0.0;
     double d_jbjtd_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double jrecs=0.0;
     double d_jrecs_dTemp_t_GND=0.0;
     double d_jrecs_dV_p_GND=0.0;
     double d_jrecs_dV_b_GND=0.0;
double jrecd=0.0;
     double d_jrecd_dTemp_t_GND=0.0;
     double d_jrecd_dV_p_GND=0.0;
     double d_jrecd_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double jtuns=0.0;
     double d_jtuns_dTemp_t_GND=0.0;
     double d_jtuns_dV_p_GND=0.0;
     double d_jtuns_dV_b_GND=0.0;
double jtund=0.0;
     double d_jtund_dTemp_t_GND=0.0;
     double d_jtund_dV_p_GND=0.0;
     double d_jtund_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double rds0=0.0;
     double d_rds0_dTemp_t_GND=0.0;
     double d_rds0_dV_p_GND=0.0;
     double d_rds0_dV_b_GND=0.0;
double ua=0.0;
     double d_ua_dTemp_t_GND=0.0;
     double d_ua_dV_p_GND=0.0;
     double d_ua_dV_b_GND=0.0;
double ub=0.0;
     double d_ub_dTemp_t_GND=0.0;
     double d_ub_dV_p_GND=0.0;
     double d_ub_dV_b_GND=0.0;
double uc=0.0;
     double d_uc_dTemp_t_GND=0.0;
     double d_uc_dV_p_GND=0.0;
     double d_uc_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double rho=0.0;
     double d_rho_dTemp_t_GND=0.0;
     double d_rho_dV_p_GND=0.0;
     double d_rho_dV_b_GND=0.0;
double rho_ref=0.0;
     double d_rho_ref_dTemp_t_GND=0.0;
     double d_rho_ref_dV_p_GND=0.0;
     double d_rho_ref_dV_b_GND=0.0;
double ku0temp=0.0;
     double d_ku0temp_dTemp_t_GND=0.0;
     double d_ku0temp_dV_p_GND=0.0;
     double d_ku0temp_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsh=0.0;
     double d_Vbsh_dV_b_si=0.0;
     double d_Vbsh_dV_di_si=0.0;
     double d_Vbsh_dV_gi_si=0.0;
     double d_Vbsh_dV_e_si=0.0;
     double d_Vbsh_dTemp_t_GND=0.0;
     double d_Vbsh_dV_p_GND=0.0;
     double d_Vbsh_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double sqrtPhisExt=0.0;
     double d_sqrtPhisExt_dV_b_si=0.0;
     double d_sqrtPhisExt_dTemp_t_GND=0.0;
     double d_sqrtPhisExt_dV_p_GND=0.0;
     double d_sqrtPhisExt_dV_b_GND=0.0;
     double d_sqrtPhisExt_dV_di_si=0.0;
     double d_sqrtPhisExt_dV_gi_si=0.0;
     double d_sqrtPhisExt_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double T13=0.0;
     double d_T13_dV_b_si=0.0;
     double d_T13_dV_di_si=0.0;
     double d_T13_dV_gi_si=0.0;
     double d_T13_dV_e_si=0.0;
     double d_T13_dTemp_t_GND=0.0;
     double d_T13_dV_p_GND=0.0;
     double d_T13_dV_b_GND=0.0;
double T14=0.0;
     double d_T14_dV_b_si=0.0;
     double d_T14_dV_di_si=0.0;
     double d_T14_dV_gi_si=0.0;
     double d_T14_dV_e_si=0.0;
     double d_T14_dTemp_t_GND=0.0;
     double d_T14_dV_p_GND=0.0;
     double d_T14_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vdsatii0=0.0;
     double d_Vdsatii0_dTemp_t_GND=0.0;
     double d_Vdsatii0_dV_p_GND=0.0;
     double d_Vdsatii0_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double VgsStep=0.0;
     double d_VgsStep_dV_gi_p=0.0;
     double d_VgsStep_dV_db_di=0.0;
     double d_VgsStep_dV_sb_si=0.0;
     double d_VgsStep_dV_b_si=0.0;
     double d_VgsStep_dV_e_si=0.0;
     double d_VgsStep_dV_di_si=0.0;
     double d_VgsStep_dV_gi_si=0.0;
     double d_VgsStep_dTemp_t_GND=0.0;
     double d_VgsStep_dV_p_GND=0.0;
     double d_VgsStep_dV_b_GND=0.0;
double Ratio=0.0;
     double d_Ratio_dV_di_si=0.0;
     double d_Ratio_dTemp_t_GND=0.0;
     double d_Ratio_dV_p_GND=0.0;
     double d_Ratio_dV_b_GND=0.0;
     double d_Ratio_dV_gi_p=0.0;
     double d_Ratio_dV_db_di=0.0;
     double d_Ratio_dV_sb_si=0.0;
     double d_Ratio_dV_b_si=0.0;
     double d_Ratio_dV_e_si=0.0;
     double d_Ratio_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vdiff=0.0;
     double d_Vdiff_dV_di_si=0.0;
     double d_Vdiff_dTemp_t_GND=0.0;
     double d_Vdiff_dV_p_GND=0.0;
     double d_Vdiff_dV_b_GND=0.0;
     double d_Vdiff_dV_gi_p=0.0;
     double d_Vdiff_dV_db_di=0.0;
     double d_Vdiff_dV_sb_si=0.0;
     double d_Vdiff_dV_b_si=0.0;
     double d_Vdiff_dV_e_si=0.0;
     double d_Vdiff_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double NVtmf=0.0;
     double d_NVtmf_dTemp_t_GND=0.0;
     double d_NVtmf_dV_p_GND=0.0;
     double d_NVtmf_dV_b_GND=0.0;
double NVtmr=0.0;
     double d_NVtmr_dTemp_t_GND=0.0;
     double d_NVtmr_dV_p_GND=0.0;
     double d_NVtmr_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ahlis=0.0;
     double d_Ahlis_dTemp_t_GND=0.0;
     double d_Ahlis_dV_p_GND=0.0;
     double d_Ahlis_dV_b_GND=0.0;
double Ahlid=0.0;
     double d_Ahlid_dTemp_t_GND=0.0;
     double d_Ahlid_dV_p_GND=0.0;
     double d_Ahlid_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double WsTsi=0.0;
double WdTsi=0.0;
//End of Block-local variables
//Block-local variables for block load
double cjsbs=0.0;
     double d_cjsbs_dTemp_t_GND=0.0;
     double d_cjsbs_dV_p_GND=0.0;
     double d_cjsbs_dV_b_GND=0.0;
double cjdbs=0.0;
     double d_cjdbs_dTemp_t_GND=0.0;
     double d_cjdbs_dV_p_GND=0.0;
     double d_cjdbs_dV_b_GND=0.0;
double dcjdbs_dT=0.0;
double dcjsbs_dT=0.0;
double dPhiBSWG_dT=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWLb=0.0;
double CoxWLcenb=0.0;
     double d_CoxWLcenb_dV_gi_p=0.0;
     double d_CoxWLcenb_dV_db_di=0.0;
     double d_CoxWLcenb_dV_sb_si=0.0;
     double d_CoxWLcenb_dV_b_si=0.0;
     double d_CoxWLcenb_dV_e_si=0.0;
     double d_CoxWLcenb_dV_di_si=0.0;
     double d_CoxWLcenb_dV_gi_si=0.0;
     double d_CoxWLcenb_dTemp_t_GND=0.0;
     double d_CoxWLcenb_dV_p_GND=0.0;
     double d_CoxWLcenb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double ExpVbsNVtm=0.0;
     double d_ExpVbsNVtm_dV_sb_si=0.0;
     double d_ExpVbsNVtm_dV_b_si=0.0;
     double d_ExpVbsNVtm_dV_di_si=0.0;
     double d_ExpVbsNVtm_dV_gi_si=0.0;
     double d_ExpVbsNVtm_dV_e_si=0.0;
     double d_ExpVbsNVtm_dTemp_t_GND=0.0;
     double d_ExpVbsNVtm_dV_p_GND=0.0;
     double d_ExpVbsNVtm_dV_b_GND=0.0;
double ExpVbdNVtm=0.0;
     double d_ExpVbdNVtm_dV_db_di=0.0;
     double d_ExpVbdNVtm_dV_sb_si=0.0;
     double d_ExpVbdNVtm_dV_b_si=0.0;
     double d_ExpVbdNVtm_dV_di_si=0.0;
     double d_ExpVbdNVtm_dV_gi_si=0.0;
     double d_ExpVbdNVtm_dV_e_si=0.0;
     double d_ExpVbdNVtm_dTemp_t_GND=0.0;
     double d_ExpVbdNVtm_dV_p_GND=0.0;
     double d_ExpVbdNVtm_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ien=0.0;
     double d_Ien_dTemp_t_GND=0.0;
     double d_Ien_dV_p_GND=0.0;
     double d_Ien_dV_b_GND=0.0;
double Iendif=0.0;
     double d_Iendif_dTemp_t_GND=0.0;
     double d_Iendif_dV_p_GND=0.0;
     double d_Iendif_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibsdif=0.0;
     double d_Ibsdif_dTemp_t_GND=0.0;
     double d_Ibsdif_dV_p_GND=0.0;
     double d_Ibsdif_dV_b_GND=0.0;
     double d_Ibsdif_dV_sb_si=0.0;
     double d_Ibsdif_dV_b_si=0.0;
     double d_Ibsdif_dV_di_si=0.0;
     double d_Ibsdif_dV_gi_si=0.0;
     double d_Ibsdif_dV_e_si=0.0;
double Ibddif=0.0;
     double d_Ibddif_dTemp_t_GND=0.0;
     double d_Ibddif_dV_p_GND=0.0;
     double d_Ibddif_dV_b_GND=0.0;
     double d_Ibddif_dV_db_di=0.0;
     double d_Ibddif_dV_sb_si=0.0;
     double d_Ibddif_dV_b_si=0.0;
     double d_Ibddif_dV_di_si=0.0;
     double d_Ibddif_dV_gi_si=0.0;
     double d_Ibddif_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ehlis=0.0;
     double d_Ehlis_dTemp_t_GND=0.0;
     double d_Ehlis_dV_p_GND=0.0;
     double d_Ehlis_dV_b_GND=0.0;
     double d_Ehlis_dV_sb_si=0.0;
     double d_Ehlis_dV_b_si=0.0;
     double d_Ehlis_dV_di_si=0.0;
     double d_Ehlis_dV_gi_si=0.0;
     double d_Ehlis_dV_e_si=0.0;
double Ehlid=0.0;
     double d_Ehlid_dTemp_t_GND=0.0;
     double d_Ehlid_dV_p_GND=0.0;
     double d_Ehlid_dV_b_GND=0.0;
     double d_Ehlid_dV_db_di=0.0;
     double d_Ehlid_dV_sb_si=0.0;
     double d_Ehlid_dV_b_si=0.0;
     double d_Ehlid_dV_di_si=0.0;
     double d_Ehlid_dV_gi_si=0.0;
     double d_Ehlid_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double EhlisFactor=0.0;
     double d_EhlisFactor_dTemp_t_GND=0.0;
     double d_EhlisFactor_dV_p_GND=0.0;
     double d_EhlisFactor_dV_b_GND=0.0;
     double d_EhlisFactor_dV_sb_si=0.0;
     double d_EhlisFactor_dV_b_si=0.0;
     double d_EhlisFactor_dV_di_si=0.0;
     double d_EhlisFactor_dV_gi_si=0.0;
     double d_EhlisFactor_dV_e_si=0.0;
double EhlidFactor=0.0;
     double d_EhlidFactor_dTemp_t_GND=0.0;
     double d_EhlidFactor_dV_p_GND=0.0;
     double d_EhlidFactor_dV_b_GND=0.0;
     double d_EhlidFactor_dV_db_di=0.0;
     double d_EhlidFactor_dV_sb_si=0.0;
     double d_EhlidFactor_dV_b_si=0.0;
     double d_EhlidFactor_dV_di_si=0.0;
     double d_EhlidFactor_dV_gi_si=0.0;
     double d_EhlidFactor_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double E2ndFactor=0.0;
     double d_E2ndFactor_dV_db_di=0.0;
     double d_E2ndFactor_dV_sb_si=0.0;
     double d_E2ndFactor_dV_b_si=0.0;
     double d_E2ndFactor_dV_e_si=0.0;
     double d_E2ndFactor_dV_di_si=0.0;
     double d_E2ndFactor_dV_gi_si=0.0;
     double d_E2ndFactor_dTemp_t_GND=0.0;
     double d_E2ndFactor_dV_p_GND=0.0;
     double d_E2ndFactor_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double DioMax=0.0;
     double d_DioMax_dTemp_t_GND=0.0;
     double d_DioMax_dV_p_GND=0.0;
     double d_DioMax_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double wdios=0.0;
double wdiod=0.0;
//End of Block-local variables
//Block-local variables for block load
double Cox=0.0;
double Tox=0.0;
double Tcen=0.0;
     double d_Tcen_dV_gi_p=0.0;
     double d_Tcen_dV_db_di=0.0;
     double d_Tcen_dV_sb_si=0.0;
     double d_Tcen_dV_b_si=0.0;
     double d_Tcen_dV_di_si=0.0;
     double d_Tcen_dV_gi_si=0.0;
     double d_Tcen_dV_e_si=0.0;
     double d_Tcen_dTemp_t_GND=0.0;
     double d_Tcen_dV_p_GND=0.0;
     double d_Tcen_dV_b_GND=0.0;
double LINK=0.0;
double Ccen=0.0;
     double d_Ccen_dV_gi_p=0.0;
     double d_Ccen_dV_db_di=0.0;
     double d_Ccen_dV_sb_si=0.0;
     double d_Ccen_dV_b_si=0.0;
     double d_Ccen_dV_di_si=0.0;
     double d_Ccen_dV_gi_si=0.0;
     double d_Ccen_dV_e_si=0.0;
     double d_Ccen_dTemp_t_GND=0.0;
     double d_Ccen_dV_p_GND=0.0;
     double d_Ccen_dV_b_GND=0.0;
double Coxeff=0.0;
     double d_Coxeff_dV_gi_p=0.0;
     double d_Coxeff_dV_db_di=0.0;
     double d_Coxeff_dV_sb_si=0.0;
     double d_Coxeff_dV_b_si=0.0;
     double d_Coxeff_dV_e_si=0.0;
     double d_Coxeff_dV_di_si=0.0;
     double d_Coxeff_dV_gi_si=0.0;
     double d_Coxeff_dTemp_t_GND=0.0;
     double d_Coxeff_dV_p_GND=0.0;
     double d_Coxeff_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWLcen=0.0;
     double d_CoxWLcen_dV_gi_p=0.0;
     double d_CoxWLcen_dV_db_di=0.0;
     double d_CoxWLcen_dV_sb_si=0.0;
     double d_CoxWLcen_dV_b_si=0.0;
     double d_CoxWLcen_dV_e_si=0.0;
     double d_CoxWLcen_dV_di_si=0.0;
     double d_CoxWLcen_dV_gi_si=0.0;
     double d_CoxWLcen_dTemp_t_GND=0.0;
     double d_CoxWLcen_dV_p_GND=0.0;
     double d_CoxWLcen_dV_b_GND=0.0;
double DeltaPhi=0.0;
     double d_DeltaPhi_dTemp_t_GND=0.0;
     double d_DeltaPhi_dV_p_GND=0.0;
     double d_DeltaPhi_dV_b_GND=0.0;
     double d_DeltaPhi_dV_gi_p=0.0;
     double d_DeltaPhi_dV_db_di=0.0;
     double d_DeltaPhi_dV_sb_si=0.0;
     double d_DeltaPhi_dV_b_si=0.0;
     double d_DeltaPhi_dV_e_si=0.0;
     double d_DeltaPhi_dV_di_si=0.0;
     double d_DeltaPhi_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double qinoi=0.0;
double qbulk=0.0;
     double d_qbulk_dV_b_si=0.0;
     double d_qbulk_dV_e_si=0.0;
     double d_qbulk_dV_di_si=0.0;
     double d_qbulk_dV_gi_si=0.0;
     double d_qbulk_dTemp_t_GND=0.0;
     double d_qbulk_dV_p_GND=0.0;
     double d_qbulk_dV_b_GND=0.0;
     double d_qbulk_dV_gi_p=0.0;
     double d_qbulk_dV_db_di=0.0;
     double d_qbulk_dV_sb_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double T3zb=0.0;
     double d_T3zb_dTemp_t_GND=0.0;
     double d_T3zb_dV_p_GND=0.0;
     double d_T3zb_dV_b_GND=0.0;
double lt1zb=0.0;
     double d_lt1zb_dTemp_t_GND=0.0;
     double d_lt1zb_dV_p_GND=0.0;
     double d_lt1zb_dV_b_GND=0.0;
double ltwzb=0.0;
     double d_ltwzb_dTemp_t_GND=0.0;
     double d_ltwzb_dV_p_GND=0.0;
     double d_ltwzb_dV_b_GND=0.0;
double Theta0zb=0.0;
     double d_Theta0zb_dV_b_si=0.0;
     double d_Theta0zb_dV_e_si=0.0;
     double d_Theta0zb_dV_di_si=0.0;
     double d_Theta0zb_dV_gi_si=0.0;
     double d_Theta0zb_dTemp_t_GND=0.0;
     double d_Theta0zb_dV_p_GND=0.0;
     double d_Theta0zb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Delt_vthzb=0.0;
     double d_Delt_vthzb_dV_b_si=0.0;
     double d_Delt_vthzb_dV_e_si=0.0;
     double d_Delt_vthzb_dV_di_si=0.0;
     double d_Delt_vthzb_dV_gi_si=0.0;
     double d_Delt_vthzb_dTemp_t_GND=0.0;
     double d_Delt_vthzb_dV_p_GND=0.0;
     double d_Delt_vthzb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltVthwzb=0.0;
     double d_DeltVthwzb_dV_b_si=0.0;
     double d_DeltVthwzb_dV_di_si=0.0;
     double d_DeltVthwzb_dV_gi_si=0.0;
     double d_DeltVthwzb_dV_e_si=0.0;
     double d_DeltVthwzb_dTemp_t_GND=0.0;
     double d_DeltVthwzb_dV_p_GND=0.0;
     double d_DeltVthwzb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltVthtempzb=0.0;
     double d_DeltVthtempzb_dV_b_si=0.0;
     double d_DeltVthtempzb_dV_di_si=0.0;
     double d_DeltVthtempzb_dV_gi_si=0.0;
     double d_DeltVthtempzb_dV_e_si=0.0;
     double d_DeltVthtempzb_dTemp_t_GND=0.0;
     double d_DeltVthtempzb_dV_p_GND=0.0;
     double d_DeltVthtempzb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vthzb=0.0;
     double d_Vthzb_dTemp_t_GND=0.0;
     double d_Vthzb_dV_p_GND=0.0;
     double d_Vthzb_dV_b_GND=0.0;
     double d_Vthzb_dV_b_si=0.0;
     double d_Vthzb_dV_e_si=0.0;
     double d_Vthzb_dV_di_si=0.0;
     double d_Vthzb_dV_gi_si=0.0;
double Vfbzb=0.0;
     double d_Vfbzb_dTemp_t_GND=0.0;
     double d_Vfbzb_dV_p_GND=0.0;
     double d_Vfbzb_dV_b_GND=0.0;
     double d_Vfbzb_dV_b_si=0.0;
     double d_Vfbzb_dV_e_si=0.0;
     double d_Vfbzb_dV_di_si=0.0;
     double d_Vfbzb_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double noff=0.0;
     double d_noff_dV_b_si=0.0;
     double d_noff_dV_e_si=0.0;
     double d_noff_dV_di_si=0.0;
     double d_noff_dV_gi_si=0.0;
     double d_noff_dTemp_t_GND=0.0;
     double d_noff_dV_p_GND=0.0;
     double d_noff_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double noff2=0.0;
     double d_noff2_dV_b_si=0.0;
     double d_noff2_dV_e_si=0.0;
     double d_noff2_dV_di_si=0.0;
     double d_noff2_dV_gi_si=0.0;
     double d_noff2_dTemp_t_GND=0.0;
     double d_noff2_dV_p_GND=0.0;
     double d_noff2_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double vgms=0.0;
     double d_vgms_dV_gm_si=0.0;
double vgmd=0.0;
     double d_vgmd_dV_gm_si=0.0;
     double d_vgmd_dV_di_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igc=0.0;
     double d_Igc_dV_db_di=0.0;
     double d_Igc_dV_sb_si=0.0;
     double d_Igc_dV_b_si=0.0;
     double d_Igc_dV_e_si=0.0;
     double d_Igc_dV_di_si=0.0;
     double d_Igc_dV_gi_si=0.0;
     double d_Igc_dTemp_t_GND=0.0;
     double d_Igc_dV_p_GND=0.0;
     double d_Igc_dV_b_GND=0.0;
double Igs=0.0;
     double d_Igs_dV_b_si=0.0;
     double d_Igs_dV_di_si=0.0;
     double d_Igs_dV_gi_si=0.0;
     double d_Igs_dV_e_si=0.0;
     double d_Igs_dTemp_t_GND=0.0;
     double d_Igs_dV_p_GND=0.0;
     double d_Igs_dV_b_GND=0.0;
     double d_Igs_dV_db_di=0.0;
     double d_Igs_dV_sb_si=0.0;
double Igd=0.0;
     double d_Igd_dV_b_si=0.0;
     double d_Igd_dV_di_si=0.0;
     double d_Igd_dV_gi_si=0.0;
     double d_Igd_dV_e_si=0.0;
     double d_Igd_dTemp_t_GND=0.0;
     double d_Igd_dV_p_GND=0.0;
     double d_Igd_dV_b_GND=0.0;
     double d_Igd_dV_db_di=0.0;
     double d_Igd_dV_sb_si=0.0;
double Igcs=0.0;
     double d_Igcs_dV_db_di=0.0;
     double d_Igcs_dV_sb_si=0.0;
     double d_Igcs_dV_b_si=0.0;
     double d_Igcs_dV_e_si=0.0;
     double d_Igcs_dV_di_si=0.0;
     double d_Igcs_dV_gi_si=0.0;
     double d_Igcs_dTemp_t_GND=0.0;
     double d_Igcs_dV_p_GND=0.0;
     double d_Igcs_dV_b_GND=0.0;
double Igcd=0.0;
     double d_Igcd_dV_db_di=0.0;
     double d_Igcd_dV_sb_si=0.0;
     double d_Igcd_dV_b_si=0.0;
     double d_Igcd_dV_e_si=0.0;
     double d_Igcd_dV_di_si=0.0;
     double d_Igcd_dV_gi_si=0.0;
     double d_Igcd_dTemp_t_GND=0.0;
     double d_Igcd_dV_p_GND=0.0;
     double d_Igcd_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double vgs_eff=0.0;
     double d_vgs_eff_dV_gi_p=0.0;
     double d_vgs_eff_dV_db_di=0.0;
     double d_vgs_eff_dV_sb_si=0.0;
     double d_vgs_eff_dV_b_si=0.0;
     double d_vgs_eff_dV_di_si=0.0;
     double d_vgs_eff_dV_gi_si=0.0;
     double d_vgs_eff_dV_e_si=0.0;
     double d_vgs_eff_dTemp_t_GND=0.0;
     double d_vgs_eff_dV_p_GND=0.0;
     double d_vgs_eff_dV_b_GND=0.0;
double vgd_eff=0.0;
     double d_vgd_eff_dV_gi_p=0.0;
     double d_vgd_eff_dV_db_di=0.0;
     double d_vgd_eff_dV_sb_si=0.0;
     double d_vgd_eff_dV_b_si=0.0;
     double d_vgd_eff_dV_di_si=0.0;
     double d_vgd_eff_dV_gi_si=0.0;
     double d_vgd_eff_dV_e_si=0.0;
     double d_vgd_eff_dTemp_t_GND=0.0;
     double d_vgd_eff_dV_p_GND=0.0;
     double d_vgd_eff_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double VxNVt=0.0;
     double d_VxNVt_dV_di_si=0.0;
     double d_VxNVt_dV_gi_si=0.0;
     double d_VxNVt_dTemp_t_GND=0.0;
     double d_VxNVt_dV_p_GND=0.0;
     double d_VxNVt_dV_b_GND=0.0;
     double d_VxNVt_dV_db_di=0.0;
     double d_VxNVt_dV_sb_si=0.0;
     double d_VxNVt_dV_b_si=0.0;
     double d_VxNVt_dV_e_si=0.0;
double ExpVxNVt=0.0;
     double d_ExpVxNVt_dV_di_si=0.0;
     double d_ExpVxNVt_dV_gi_si=0.0;
     double d_ExpVxNVt_dTemp_t_GND=0.0;
     double d_ExpVxNVt_dV_p_GND=0.0;
     double d_ExpVxNVt_dV_b_GND=0.0;
     double d_ExpVxNVt_dV_db_di=0.0;
     double d_ExpVxNVt_dV_sb_si=0.0;
     double d_ExpVxNVt_dV_b_si=0.0;
     double d_ExpVxNVt_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsitf=0.0;
     double d_Vbsitf_dV_b_si=0.0;
     double d_Vbsitf_dV_di_si=0.0;
     double d_Vbsitf_dV_gi_si=0.0;
     double d_Vbsitf_dTemp_t_GND=0.0;
     double d_Vbsitf_dV_p_GND=0.0;
     double d_Vbsitf_dV_b_GND=0.0;
     double d_Vbsitf_dV_e_si=0.0;
double Vbs0=0.0;
     double d_Vbs0_dV_di_si=0.0;
     double d_Vbs0_dV_gi_si=0.0;
     double d_Vbs0_dTemp_t_GND=0.0;
     double d_Vbs0_dV_p_GND=0.0;
     double d_Vbs0_dV_b_GND=0.0;
     double d_Vbs0_dV_e_si=0.0;
double Vbs0mos=0.0;
     double d_Vbs0mos_dV_di_si=0.0;
     double d_Vbs0mos_dV_gi_si=0.0;
     double d_Vbs0mos_dTemp_t_GND=0.0;
     double d_Vbs0mos_dV_p_GND=0.0;
     double d_Vbs0mos_dV_b_GND=0.0;
     double d_Vbs0mos_dV_e_si=0.0;
double Vbsmos=0.0;
     double d_Vbsmos_dV_gi_si=0.0;
     double d_Vbsmos_dTemp_t_GND=0.0;
     double d_Vbsmos_dV_p_GND=0.0;
     double d_Vbsmos_dV_b_GND=0.0;
     double d_Vbsmos_dV_e_si=0.0;
     double d_Vbsmos_dV_di_si=0.0;
     double d_Vbsmos_dV_b_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double PhiON=0.0;
     double d_PhiON_dTemp_t_GND=0.0;
     double d_PhiON_dV_p_GND=0.0;
     double d_PhiON_dV_b_GND=0.0;
     double d_PhiON_dV_di_si=0.0;
     double d_PhiON_dV_gi_si=0.0;
     double d_PhiON_dV_e_si=0.0;
double PhiFD=0.0;
     double d_PhiFD_dTemp_t_GND=0.0;
     double d_PhiFD_dV_p_GND=0.0;
     double d_PhiFD_dV_b_GND=0.0;
     double d_PhiFD_dV_di_si=0.0;
     double d_PhiFD_dV_gi_si=0.0;
     double d_PhiFD_dV_e_si=0.0;
double Vbs0t=0.0;
     double d_Vbs0t_dV_e_si=0.0;
     double d_Vbs0t_dTemp_t_GND=0.0;
     double d_Vbs0t_dV_p_GND=0.0;
     double d_Vbs0t_dV_b_GND=0.0;
     double d_Vbs0t_dV_di_si=0.0;
     double d_Vbs0t_dV_gi_si=0.0;
double VthFD=0.0;
     double d_VthFD_dTemp_t_GND=0.0;
     double d_VthFD_dV_p_GND=0.0;
     double d_VthFD_dV_b_GND=0.0;
     double d_VthFD_dV_di_si=0.0;
     double d_VthFD_dV_gi_si=0.0;
     double d_VthFD_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgsFD=0.0;
     double d_VtgsFD_dTemp_t_GND=0.0;
     double d_VtgsFD_dV_p_GND=0.0;
     double d_VtgsFD_dV_b_GND=0.0;
     double d_VtgsFD_dV_di_si=0.0;
     double d_VtgsFD_dV_gi_si=0.0;
     double d_VtgsFD_dV_e_si=0.0;
double ExpVtgsFD=0.0;
     double d_ExpVtgsFD_dTemp_t_GND=0.0;
     double d_ExpVtgsFD_dV_p_GND=0.0;
     double d_ExpVtgsFD_dV_b_GND=0.0;
     double d_ExpVtgsFD_dV_di_si=0.0;
     double d_ExpVtgsFD_dV_gi_si=0.0;
     double d_ExpVtgsFD_dV_e_si=0.0;
double VgstFD=0.0;
     double d_VgstFD_dV_di_si=0.0;
     double d_VgstFD_dV_gi_si=0.0;
     double d_VgstFD_dTemp_t_GND=0.0;
     double d_VgstFD_dV_p_GND=0.0;
     double d_VgstFD_dV_b_GND=0.0;
     double d_VgstFD_dV_e_si=0.0;
double ExpVgstFD=0.0;
     double d_ExpVgstFD_dV_di_si=0.0;
     double d_ExpVgstFD_dV_gi_si=0.0;
     double d_ExpVgstFD_dTemp_t_GND=0.0;
     double d_ExpVgstFD_dV_p_GND=0.0;
     double d_ExpVgstFD_dV_b_GND=0.0;
     double d_ExpVgstFD_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgseffFD=0.0;
     double d_VtgseffFD_dTemp_t_GND=0.0;
     double d_VtgseffFD_dV_p_GND=0.0;
     double d_VtgseffFD_dV_b_GND=0.0;
     double d_VtgseffFD_dV_di_si=0.0;
     double d_VtgseffFD_dV_gi_si=0.0;
     double d_VtgseffFD_dV_e_si=0.0;
double VgsteffFD=0.0;
     double d_VgsteffFD_dTemp_t_GND=0.0;
     double d_VgsteffFD_dV_p_GND=0.0;
     double d_VgsteffFD_dV_b_GND=0.0;
     double d_VgsteffFD_dV_di_si=0.0;
     double d_VgsteffFD_dV_gi_si=0.0;
     double d_VgsteffFD_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbseff_CV=0.0;
     double d_Vbseff_CV_dV_b_si=0.0;
     double d_Vbseff_CV_dV_di_si=0.0;
     double d_Vbseff_CV_dV_gi_si=0.0;
     double d_Vbseff_CV_dV_e_si=0.0;
     double d_Vbseff_CV_dTemp_t_GND=0.0;
     double d_Vbseff_CV_dV_p_GND=0.0;
     double d_Vbseff_CV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsh_CV=0.0;
     double d_Vbsh_CV_dV_b_si=0.0;
     double d_Vbsh_CV_dV_di_si=0.0;
     double d_Vbsh_CV_dV_gi_si=0.0;
     double d_Vbsh_CV_dV_e_si=0.0;
     double d_Vbsh_CV_dTemp_t_GND=0.0;
     double d_Vbsh_CV_dV_p_GND=0.0;
     double d_Vbsh_CV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsitf_CV=0.0;
     double d_Vbsitf_CV_dV_b_si=0.0;
     double d_Vbsitf_CV_dV_di_si=0.0;
     double d_Vbsitf_CV_dV_gi_si=0.0;
     double d_Vbsitf_CV_dV_e_si=0.0;
     double d_Vbsitf_CV_dTemp_t_GND=0.0;
     double d_Vbsitf_CV_dV_p_GND=0.0;
     double d_Vbsitf_CV_dV_b_GND=0.0;
double Vbs0_CV=0.0;
     double d_Vbs0_CV_dV_b_si=0.0;
     double d_Vbs0_CV_dV_di_si=0.0;
     double d_Vbs0_CV_dV_gi_si=0.0;
     double d_Vbs0_CV_dV_e_si=0.0;
     double d_Vbs0_CV_dTemp_t_GND=0.0;
     double d_Vbs0_CV_dV_p_GND=0.0;
     double d_Vbs0_CV_dV_b_GND=0.0;
double Vbsmos_CV=0.0;
     double d_Vbsmos_CV_dV_gi_si=0.0;
     double d_Vbsmos_CV_dV_e_si=0.0;
     double d_Vbsmos_CV_dTemp_t_GND=0.0;
     double d_Vbsmos_CV_dV_p_GND=0.0;
     double d_Vbsmos_CV_dV_b_GND=0.0;
     double d_Vbsmos_CV_dV_di_si=0.0;
     double d_Vbsmos_CV_dV_b_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double PhiON_CV=0.0;
     double d_PhiON_CV_dTemp_t_GND=0.0;
     double d_PhiON_CV_dV_p_GND=0.0;
     double d_PhiON_CV_dV_b_GND=0.0;
     double d_PhiON_CV_dV_b_si=0.0;
     double d_PhiON_CV_dV_di_si=0.0;
     double d_PhiON_CV_dV_gi_si=0.0;
     double d_PhiON_CV_dV_e_si=0.0;
double PhiFD_CV=0.0;
     double d_PhiFD_CV_dTemp_t_GND=0.0;
     double d_PhiFD_CV_dV_p_GND=0.0;
     double d_PhiFD_CV_dV_b_GND=0.0;
     double d_PhiFD_CV_dV_b_si=0.0;
     double d_PhiFD_CV_dV_di_si=0.0;
     double d_PhiFD_CV_dV_gi_si=0.0;
     double d_PhiFD_CV_dV_e_si=0.0;
double Vbs0t_CV=0.0;
     double d_Vbs0t_CV_dTemp_t_GND=0.0;
     double d_Vbs0t_CV_dV_p_GND=0.0;
     double d_Vbs0t_CV_dV_b_GND=0.0;
     double d_Vbs0t_CV_dV_b_si=0.0;
     double d_Vbs0t_CV_dV_di_si=0.0;
     double d_Vbs0t_CV_dV_gi_si=0.0;
     double d_Vbs0t_CV_dV_e_si=0.0;
double VthFD_CV=0.0;
     double d_VthFD_CV_dTemp_t_GND=0.0;
     double d_VthFD_CV_dV_p_GND=0.0;
     double d_VthFD_CV_dV_b_GND=0.0;
     double d_VthFD_CV_dV_di_si=0.0;
     double d_VthFD_CV_dV_gi_si=0.0;
     double d_VthFD_CV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Phis_CV=0.0;
     double d_Phis_CV_dTemp_t_GND=0.0;
     double d_Phis_CV_dV_p_GND=0.0;
     double d_Phis_CV_dV_b_GND=0.0;
     double d_Phis_CV_dV_b_si=0.0;
     double d_Phis_CV_dV_di_si=0.0;
     double d_Phis_CV_dV_gi_si=0.0;
     double d_Phis_CV_dV_e_si=0.0;
double sqrtPhis_CV=0.0;
     double d_sqrtPhis_CV_dTemp_t_GND=0.0;
     double d_sqrtPhis_CV_dV_p_GND=0.0;
     double d_sqrtPhis_CV_dV_b_GND=0.0;
     double d_sqrtPhis_CV_dV_b_si=0.0;
     double d_sqrtPhis_CV_dV_di_si=0.0;
     double d_sqrtPhis_CV_dV_gi_si=0.0;
     double d_sqrtPhis_CV_dV_e_si=0.0;
double Xdep_CV=0.0;
     double d_Xdep_CV_dTemp_t_GND=0.0;
     double d_Xdep_CV_dV_p_GND=0.0;
     double d_Xdep_CV_dV_b_GND=0.0;
     double d_Xdep_CV_dV_b_si=0.0;
     double d_Xdep_CV_dV_di_si=0.0;
     double d_Xdep_CV_dV_gi_si=0.0;
     double d_Xdep_CV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double lt1_CV=0.0;
     double d_lt1_CV_dV_b_si=0.0;
     double d_lt1_CV_dV_e_si=0.0;
     double d_lt1_CV_dV_di_si=0.0;
     double d_lt1_CV_dV_gi_si=0.0;
     double d_lt1_CV_dTemp_t_GND=0.0;
     double d_lt1_CV_dV_p_GND=0.0;
     double d_lt1_CV_dV_b_GND=0.0;
double ltw_CV=0.0;
     double d_ltw_CV_dV_b_si=0.0;
     double d_ltw_CV_dV_e_si=0.0;
     double d_ltw_CV_dV_di_si=0.0;
     double d_ltw_CV_dV_gi_si=0.0;
     double d_ltw_CV_dTemp_t_GND=0.0;
     double d_ltw_CV_dV_p_GND=0.0;
     double d_ltw_CV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Theta0_CV=0.0;
     double d_Theta0_CV_dV_b_si=0.0;
     double d_Theta0_CV_dV_e_si=0.0;
     double d_Theta0_CV_dV_di_si=0.0;
     double d_Theta0_CV_dV_gi_si=0.0;
     double d_Theta0_CV_dTemp_t_GND=0.0;
     double d_Theta0_CV_dV_p_GND=0.0;
     double d_Theta0_CV_dV_b_GND=0.0;
double n_CV=0.0;
     double d_n_CV_dV_b_si=0.0;
     double d_n_CV_dV_e_si=0.0;
     double d_n_CV_dV_di_si=0.0;
     double d_n_CV_dV_gi_si=0.0;
     double d_n_CV_dTemp_t_GND=0.0;
     double d_n_CV_dV_p_GND=0.0;
     double d_n_CV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgsFD_CV=0.0;
     double d_VtgsFD_CV_dTemp_t_GND=0.0;
     double d_VtgsFD_CV_dV_p_GND=0.0;
     double d_VtgsFD_CV_dV_b_GND=0.0;
     double d_VtgsFD_CV_dV_di_si=0.0;
     double d_VtgsFD_CV_dV_gi_si=0.0;
     double d_VtgsFD_CV_dV_e_si=0.0;
double ExpVtgsFD_CV=0.0;
     double d_ExpVtgsFD_CV_dTemp_t_GND=0.0;
     double d_ExpVtgsFD_CV_dV_p_GND=0.0;
     double d_ExpVtgsFD_CV_dV_b_GND=0.0;
     double d_ExpVtgsFD_CV_dV_di_si=0.0;
     double d_ExpVtgsFD_CV_dV_gi_si=0.0;
     double d_ExpVtgsFD_CV_dV_e_si=0.0;
double VgstFD_CV=0.0;
     double d_VgstFD_CV_dV_di_si=0.0;
     double d_VgstFD_CV_dV_gi_si=0.0;
     double d_VgstFD_CV_dTemp_t_GND=0.0;
     double d_VgstFD_CV_dV_p_GND=0.0;
     double d_VgstFD_CV_dV_b_GND=0.0;
     double d_VgstFD_CV_dV_e_si=0.0;
double ExpVgstFD_CV=0.0;
     double d_ExpVgstFD_CV_dV_di_si=0.0;
     double d_ExpVgstFD_CV_dV_gi_si=0.0;
     double d_ExpVgstFD_CV_dTemp_t_GND=0.0;
     double d_ExpVgstFD_CV_dV_p_GND=0.0;
     double d_ExpVgstFD_CV_dV_b_GND=0.0;
     double d_ExpVgstFD_CV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgseffFD_CV=0.0;
     double d_VtgseffFD_CV_dTemp_t_GND=0.0;
     double d_VtgseffFD_CV_dV_p_GND=0.0;
     double d_VtgseffFD_CV_dV_b_GND=0.0;
     double d_VtgseffFD_CV_dV_di_si=0.0;
     double d_VtgseffFD_CV_dV_gi_si=0.0;
     double d_VtgseffFD_CV_dV_e_si=0.0;
double VgsteffFD_CV=0.0;
     double d_VgsteffFD_CV_dTemp_t_GND=0.0;
     double d_VgsteffFD_CV_dV_p_GND=0.0;
     double d_VgsteffFD_CV_dV_b_GND=0.0;
     double d_VgsteffFD_CV_dV_di_si=0.0;
     double d_VgsteffFD_CV_dV_gi_si=0.0;
     double d_VgsteffFD_CV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double tmp2_CV=0.0;
     double d_tmp2_CV_dTemp_t_GND=0.0;
     double d_tmp2_CV_dV_p_GND=0.0;
     double d_tmp2_CV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Delt_vth_CV=0.0;
     double d_Delt_vth_CV_dV_b_si=0.0;
     double d_Delt_vth_CV_dV_e_si=0.0;
     double d_Delt_vth_CV_dV_di_si=0.0;
     double d_Delt_vth_CV_dV_gi_si=0.0;
     double d_Delt_vth_CV_dTemp_t_GND=0.0;
     double d_Delt_vth_CV_dV_p_GND=0.0;
     double d_Delt_vth_CV_dV_b_GND=0.0;
double DeltVthw_CV=0.0;
     double d_DeltVthw_CV_dV_b_si=0.0;
     double d_DeltVthw_CV_dV_di_si=0.0;
     double d_DeltVthw_CV_dV_gi_si=0.0;
     double d_DeltVthw_CV_dV_e_si=0.0;
     double d_DeltVthw_CV_dTemp_t_GND=0.0;
     double d_DeltVthw_CV_dV_p_GND=0.0;
     double d_DeltVthw_CV_dV_b_GND=0.0;
double DeltVthtemp_CV=0.0;
     double d_DeltVthtemp_CV_dV_b_si=0.0;
     double d_DeltVthtemp_CV_dV_di_si=0.0;
     double d_DeltVthtemp_CV_dV_gi_si=0.0;
     double d_DeltVthtemp_CV_dV_e_si=0.0;
     double d_DeltVthtemp_CV_dTemp_t_GND=0.0;
     double d_DeltVthtemp_CV_dV_p_GND=0.0;
     double d_DeltVthtemp_CV_dV_b_GND=0.0;
double DIBL_Sft_CV=0.0;
     double d_DIBL_Sft_CV_dV_b_si=0.0;
     double d_DIBL_Sft_CV_dV_e_si=0.0;
     double d_DIBL_Sft_CV_dV_di_si=0.0;
     double d_DIBL_Sft_CV_dV_gi_si=0.0;
     double d_DIBL_Sft_CV_dTemp_t_GND=0.0;
     double d_DIBL_Sft_CV_dV_p_GND=0.0;
     double d_DIBL_Sft_CV_dV_b_GND=0.0;
double DITS_Sft_CV=0.0;
     double d_DITS_Sft_CV_dV_b_si=0.0;
     double d_DITS_Sft_CV_dV_e_si=0.0;
     double d_DITS_Sft_CV_dV_di_si=0.0;
     double d_DITS_Sft_CV_dV_gi_si=0.0;
     double d_DITS_Sft_CV_dTemp_t_GND=0.0;
     double d_DITS_Sft_CV_dV_p_GND=0.0;
     double d_DITS_Sft_CV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double sqrtPhisExt_CV=0.0;
     double d_sqrtPhisExt_CV_dTemp_t_GND=0.0;
     double d_sqrtPhisExt_CV_dV_p_GND=0.0;
     double d_sqrtPhisExt_CV_dV_b_GND=0.0;
     double d_sqrtPhisExt_CV_dV_b_si=0.0;
     double d_sqrtPhisExt_CV_dV_di_si=0.0;
     double d_sqrtPhisExt_CV_dV_gi_si=0.0;
     double d_sqrtPhisExt_CV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vth_CV=0.0;
     double d_Vth_CV_dTemp_t_GND=0.0;
     double d_Vth_CV_dV_p_GND=0.0;
     double d_Vth_CV_dV_b_GND=0.0;
     double d_Vth_CV_dV_b_si=0.0;
     double d_Vth_CV_dV_di_si=0.0;
     double d_Vth_CV_dV_gi_si=0.0;
     double d_Vth_CV_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Abulk0_CV=0.0;
     double d_Abulk0_CV_dV_b_si=0.0;
     double d_Abulk0_CV_dV_e_si=0.0;
     double d_Abulk0_CV_dV_di_si=0.0;
     double d_Abulk0_CV_dV_gi_si=0.0;
     double d_Abulk0_CV_dTemp_t_GND=0.0;
     double d_Abulk0_CV_dV_p_GND=0.0;
     double d_Abulk0_CV_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgb=0.0;
     double d_Vgb_dV_di_si=0.0;
     double d_Vgb_dV_gi_si=0.0;
     double d_Vgb_dTemp_t_GND=0.0;
     double d_Vgb_dV_p_GND=0.0;
     double d_Vgb_dV_b_GND=0.0;
     double d_Vgb_dV_e_si=0.0;
     double d_Vgb_dV_b_si=0.0;
double Vox=0.0;
     double d_Vox_dV_di_si=0.0;
     double d_Vox_dV_gi_si=0.0;
     double d_Vox_dTemp_t_GND=0.0;
     double d_Vox_dV_p_GND=0.0;
     double d_Vox_dV_b_GND=0.0;
     double d_Vox_dV_db_di=0.0;
     double d_Vox_dV_sb_si=0.0;
     double d_Vox_dV_b_si=0.0;
     double d_Vox_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double OxideRatio=0.0;
double Vaux=0.0;
     double d_Vaux_dV_db_di=0.0;
     double d_Vaux_dV_sb_si=0.0;
     double d_Vaux_dV_b_si=0.0;
     double d_Vaux_dV_e_si=0.0;
     double d_Vaux_dV_di_si=0.0;
     double d_Vaux_dV_gi_si=0.0;
     double d_Vaux_dTemp_t_GND=0.0;
     double d_Vaux_dV_p_GND=0.0;
     double d_Vaux_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igb=0.0;
     double d_Igb_dV_db_di=0.0;
     double d_Igb_dV_sb_si=0.0;
     double d_Igb_dV_b_si=0.0;
     double d_Igb_dV_e_si=0.0;
     double d_Igb_dV_di_si=0.0;
     double d_Igb_dV_gi_si=0.0;
     double d_Igb_dTemp_t_GND=0.0;
     double d_Igb_dV_p_GND=0.0;
     double d_Igb_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Voxeff=0.0;
     double d_Voxeff_dV_db_di=0.0;
     double d_Voxeff_dV_sb_si=0.0;
     double d_Voxeff_dV_b_si=0.0;
     double d_Voxeff_dV_di_si=0.0;
     double d_Voxeff_dV_gi_si=0.0;
     double d_Voxeff_dV_e_si=0.0;
     double d_Voxeff_dTemp_t_GND=0.0;
     double d_Voxeff_dV_p_GND=0.0;
     double d_Voxeff_dV_b_GND=0.0;
double Voxacc=0.0;
     double d_Voxacc_dTemp_t_GND=0.0;
     double d_Voxacc_dV_p_GND=0.0;
     double d_Voxacc_dV_b_GND=0.0;
     double d_Voxacc_dV_db_di=0.0;
     double d_Voxacc_dV_sb_si=0.0;
     double d_Voxacc_dV_b_si=0.0;
     double d_Voxacc_dV_e_si=0.0;
     double d_Voxacc_dV_di_si=0.0;
     double d_Voxacc_dV_gi_si=0.0;
double Voxdepinv=0.0;
     double d_Voxdepinv_dV_di_si=0.0;
     double d_Voxdepinv_dV_gi_si=0.0;
     double d_Voxdepinv_dTemp_t_GND=0.0;
     double d_Voxdepinv_dV_p_GND=0.0;
     double d_Voxdepinv_dV_b_GND=0.0;
     double d_Voxdepinv_dV_db_di=0.0;
     double d_Voxdepinv_dV_sb_si=0.0;
     double d_Voxdepinv_dV_b_si=0.0;
     double d_Voxdepinv_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igb1=0.0;
     double d_Igb1_dV_db_di=0.0;
     double d_Igb1_dV_sb_si=0.0;
     double d_Igb1_dV_b_si=0.0;
     double d_Igb1_dV_e_si=0.0;
     double d_Igb1_dV_di_si=0.0;
     double d_Igb1_dV_gi_si=0.0;
     double d_Igb1_dTemp_t_GND=0.0;
     double d_Igb1_dV_p_GND=0.0;
     double d_Igb1_dV_b_GND=0.0;
double Igb2=0.0;
     double d_Igb2_dV_db_di=0.0;
     double d_Igb2_dV_sb_si=0.0;
     double d_Igb2_dV_b_si=0.0;
     double d_Igb2_dV_e_si=0.0;
     double d_Igb2_dV_di_si=0.0;
     double d_Igb2_dV_gi_si=0.0;
     double d_Igb2_dTemp_t_GND=0.0;
     double d_Igb2_dV_p_GND=0.0;
     double d_Igb2_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double IdlovVdseff=0.0;
     double d_IdlovVdseff_dTemp_t_GND=0.0;
     double d_IdlovVdseff_dV_p_GND=0.0;
     double d_IdlovVdseff_dV_b_GND=0.0;
     double d_IdlovVdseff_dV_b_si=0.0;
     double d_IdlovVdseff_dV_e_si=0.0;
     double d_IdlovVdseff_dV_di_si=0.0;
     double d_IdlovVdseff_dV_gi_si=0.0;
double IdovVds=0.0;
     double d_IdovVds_dTemp_t_GND=0.0;
     double d_IdovVds_dV_p_GND=0.0;
     double d_IdovVds_dV_b_GND=0.0;
     double d_IdovVds_dV_b_si=0.0;
     double d_IdovVds_dV_e_si=0.0;
     double d_IdovVds_dV_di_si=0.0;
     double d_IdovVds_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double vsbs=0.0;
     double d_vsbs_dV_sb_si=0.0;
double vdbd=0.0;
     double d_vdbd_dV_db_di=0.0;
//End of Block-local variables
//Block-local variables for block load
double Lpe_Vb=0.0;
//End of Block-local variables
//Block-local variables for block load
double DITS_Sft=0.0;
     double d_DITS_Sft_dV_b_si=0.0;
     double d_DITS_Sft_dV_e_si=0.0;
     double d_DITS_Sft_dV_di_si=0.0;
     double d_DITS_Sft_dV_gi_si=0.0;
     double d_DITS_Sft_dTemp_t_GND=0.0;
     double d_DITS_Sft_dV_p_GND=0.0;
     double d_DITS_Sft_dV_b_GND=0.0;
double DITS_Sft2=0.0;
     double d_DITS_Sft2_dV_b_si=0.0;
     double d_DITS_Sft2_dV_di_si=0.0;
     double d_DITS_Sft2_dV_gi_si=0.0;
     double d_DITS_Sft2_dV_e_si=0.0;
     double d_DITS_Sft2_dTemp_t_GND=0.0;
     double d_DITS_Sft2_dV_p_GND=0.0;
     double d_DITS_Sft2_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double FP=0.0;
     double d_FP_dV_b_si=0.0;
     double d_FP_dV_e_si=0.0;
     double d_FP_dV_di_si=0.0;
     double d_FP_dV_gi_si=0.0;
     double d_FP_dTemp_t_GND=0.0;
     double d_FP_dV_p_GND=0.0;
     double d_FP_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double VADITS=0.0;
     double d_VADITS_dV_b_si=0.0;
     double d_VADITS_dV_e_si=0.0;
     double d_VADITS_dV_di_si=0.0;
     double d_VADITS_dV_gi_si=0.0;
     double d_VADITS_dTemp_t_GND=0.0;
     double d_VADITS_dV_p_GND=0.0;
     double d_VADITS_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double PowWeffWr=0.0;
double rd0=0.0;
     double d_rd0_dTemp_t_GND=0.0;
     double d_rd0_dV_p_GND=0.0;
     double d_rd0_dV_b_GND=0.0;
double rs0=0.0;
     double d_rs0_dTemp_t_GND=0.0;
     double d_rs0_dV_p_GND=0.0;
     double d_rs0_dV_b_GND=0.0;
double rdwmin=0.0;
     double d_rdwmin_dTemp_t_GND=0.0;
     double d_rdwmin_dV_p_GND=0.0;
     double d_rdwmin_dV_b_GND=0.0;
double rswmin=0.0;
     double d_rswmin_dTemp_t_GND=0.0;
     double d_rswmin_dV_p_GND=0.0;
     double d_rswmin_dV_b_GND=0.0;
double Rd=0.0;
     double d_Rd_dTemp_t_GND=0.0;
     double d_Rd_dV_p_GND=0.0;
     double d_Rd_dV_b_GND=0.0;
     double d_Rd_dV_gi_p=0.0;
     double d_Rd_dV_db_di=0.0;
     double d_Rd_dV_sb_si=0.0;
     double d_Rd_dV_b_si=0.0;
     double d_Rd_dV_e_si=0.0;
     double d_Rd_dV_di_si=0.0;
     double d_Rd_dV_gi_si=0.0;
double Rs=0.0;
     double d_Rs_dTemp_t_GND=0.0;
     double d_Rs_dV_p_GND=0.0;
     double d_Rs_dV_b_GND=0.0;
     double d_Rs_dV_gi_p=0.0;
     double d_Rs_dV_db_di=0.0;
     double d_Rs_dV_sb_si=0.0;
     double d_Rs_dV_b_si=0.0;
     double d_Rs_dV_e_si=0.0;
     double d_Rs_dV_di_si=0.0;
     double d_Rs_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Tnom=0.0;
double Eg0=0.0;
double Vtm0=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbci=0.0;
     double d_Vbci_dTemp_t_GND=0.0;
     double d_Vbci_dV_p_GND=0.0;
     double d_Vbci_dV_b_GND=0.0;
double Idsmosfet=0.0;
     double d_Idsmosfet_dV_di_si=0.0;
     double d_Idsmosfet_dTemp_t_GND=0.0;
     double d_Idsmosfet_dV_p_GND=0.0;
     double d_Idsmosfet_dV_b_GND=0.0;
     double d_Idsmosfet_dV_gi_p=0.0;
     double d_Idsmosfet_dV_db_di=0.0;
     double d_Idsmosfet_dV_sb_si=0.0;
     double d_Idsmosfet_dV_b_si=0.0;
     double d_Idsmosfet_dV_e_si=0.0;
     double d_Idsmosfet_dV_gi_si=0.0;
double Iiibjt=0.0;
     double d_Iiibjt_dV_gi_p=0.0;
     double d_Iiibjt_dV_db_di=0.0;
     double d_Iiibjt_dV_sb_si=0.0;
     double d_Iiibjt_dV_b_si=0.0;
     double d_Iiibjt_dV_di_si=0.0;
     double d_Iiibjt_dV_gi_si=0.0;
     double d_Iiibjt_dV_e_si=0.0;
     double d_Iiibjt_dTemp_t_GND=0.0;
     double d_Iiibjt_dV_p_GND=0.0;
     double d_Iiibjt_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double VgsteffVth=0.0;
//End of Block-local variables
//Block-local variables for block load
double toxe_mob=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgd=0.0;
     double d_Vgd_dV_gi_si=0.0;
     double d_Vgd_dV_di_si=0.0;
double Vgd_eff=0.0;
     double d_Vgd_eff_dV_gi_si=0.0;
     double d_Vgd_eff_dV_di_si=0.0;
     double d_Vgd_eff_dTemp_t_GND=0.0;
     double d_Vgd_eff_dV_p_GND=0.0;
     double d_Vgd_eff_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ig_agbcp2=0.0;
     double d_Ig_agbcp2_dV_db_di=0.0;
     double d_Ig_agbcp2_dV_sb_si=0.0;
     double d_Ig_agbcp2_dV_b_si=0.0;
     double d_Ig_agbcp2_dV_e_si=0.0;
     double d_Ig_agbcp2_dV_di_si=0.0;
     double d_Ig_agbcp2_dV_gi_si=0.0;
     double d_Ig_agbcp2_dTemp_t_GND=0.0;
     double d_Ig_agbcp2_dV_p_GND=0.0;
     double d_Ig_agbcp2_dV_b_GND=0.0;
     double d_Ig_agbcp2_dV_gi_p=0.0;
//End of Block-local variables
//Block-local variables for block load
double vgp_eff=0.0;
     double d_vgp_eff_dV_gi_p=0.0;
     double d_vgp_eff_dV_db_di=0.0;
     double d_vgp_eff_dV_sb_si=0.0;
     double d_vgp_eff_dV_b_si=0.0;
     double d_vgp_eff_dV_di_si=0.0;
     double d_vgp_eff_dV_gi_si=0.0;
     double d_vgp_eff_dV_e_si=0.0;
     double d_vgp_eff_dTemp_t_GND=0.0;
     double d_vgp_eff_dV_p_GND=0.0;
     double d_vgp_eff_dV_b_GND=0.0;
double vgp=0.0;
     double d_vgp_dV_gi_p=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWL2=0.0;
double CoxWLb2=0.0;
//End of Block-local variables
//Block-local variables for block load
double ExpVgst2=0.0;
     double d_ExpVgst2_dV_gi_p=0.0;
     double d_ExpVgst2_dV_db_di=0.0;
     double d_ExpVgst2_dV_sb_si=0.0;
     double d_ExpVgst2_dV_b_si=0.0;
     double d_ExpVgst2_dV_di_si=0.0;
     double d_ExpVgst2_dV_gi_si=0.0;
     double d_ExpVgst2_dV_e_si=0.0;
     double d_ExpVgst2_dTemp_t_GND=0.0;
     double d_ExpVgst2_dV_p_GND=0.0;
     double d_ExpVgst2_dV_b_GND=0.0;
double Vgsteff2=0.0;
     double d_Vgsteff2_dV_gi_p=0.0;
     double d_Vgsteff2_dV_b_si=0.0;
     double d_Vgsteff2_dV_e_si=0.0;
     double d_Vgsteff2_dV_di_si=0.0;
     double d_Vgsteff2_dV_gi_si=0.0;
     double d_Vgsteff2_dTemp_t_GND=0.0;
     double d_Vgsteff2_dV_p_GND=0.0;
     double d_Vgsteff2_dV_b_GND=0.0;
     double d_Vgsteff2_dV_db_di=0.0;
     double d_Vgsteff2_dV_sb_si=0.0;
double VgstNVt2=0.0;
     double d_VgstNVt2_dV_di_si=0.0;
     double d_VgstNVt2_dV_gi_si=0.0;
     double d_VgstNVt2_dTemp_t_GND=0.0;
     double d_VgstNVt2_dV_p_GND=0.0;
     double d_VgstNVt2_dV_b_GND=0.0;
     double d_VgstNVt2_dV_b_si=0.0;
     double d_VgstNVt2_dV_e_si=0.0;
double ExpArg2=0.0;
     double d_ExpArg2_dV_di_si=0.0;
     double d_ExpArg2_dV_gi_si=0.0;
     double d_ExpArg2_dTemp_t_GND=0.0;
     double d_ExpArg2_dV_p_GND=0.0;
     double d_ExpArg2_dV_b_GND=0.0;
     double d_ExpArg2_dV_b_si=0.0;
     double d_ExpArg2_dV_e_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double T02=0.0;
     double d_T02_dV_b_si=0.0;
     double d_T02_dV_e_si=0.0;
     double d_T02_dV_di_si=0.0;
     double d_T02_dV_gi_si=0.0;
     double d_T02_dTemp_t_GND=0.0;
     double d_T02_dV_p_GND=0.0;
     double d_T02_dV_b_GND=0.0;
     double d_T02_dV_gi_p=0.0;
     double d_T02_dV_db_di=0.0;
     double d_T02_dV_sb_si=0.0;
double T22=0.0;
     double d_T22_dV_gi_p=0.0;
     double d_T22_dV_db_di=0.0;
     double d_T22_dV_sb_si=0.0;
     double d_T22_dV_b_si=0.0;
     double d_T22_dV_e_si=0.0;
     double d_T22_dV_di_si=0.0;
     double d_T22_dV_gi_si=0.0;
     double d_T22_dTemp_t_GND=0.0;
     double d_T22_dV_p_GND=0.0;
     double d_T22_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Qac02=0.0;
     double d_Qac02_dV_gi_p=0.0;
     double d_Qac02_dV_db_di=0.0;
     double d_Qac02_dV_sb_si=0.0;
     double d_Qac02_dV_b_si=0.0;
     double d_Qac02_dV_e_si=0.0;
     double d_Qac02_dV_di_si=0.0;
     double d_Qac02_dV_gi_si=0.0;
     double d_Qac02_dTemp_t_GND=0.0;
     double d_Qac02_dV_p_GND=0.0;
     double d_Qac02_dV_b_GND=0.0;
double Qsub02=0.0;
     double d_Qsub02_dV_gi_p=0.0;
     double d_Qsub02_dV_db_di=0.0;
     double d_Qsub02_dV_sb_si=0.0;
     double d_Qsub02_dV_b_si=0.0;
     double d_Qsub02_dV_e_si=0.0;
     double d_Qsub02_dV_di_si=0.0;
     double d_Qsub02_dV_gi_si=0.0;
     double d_Qsub02_dTemp_t_GND=0.0;
     double d_Qsub02_dV_p_GND=0.0;
     double d_Qsub02_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgs_eff2=0.0;
     double d_Vgs_eff2_dV_di_si=0.0;
     double d_Vgs_eff2_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vfbzb2=0.0;
     double d_Vfbzb2_dTemp_t_GND=0.0;
     double d_Vfbzb2_dV_p_GND=0.0;
     double d_Vfbzb2_dV_b_GND=0.0;
     double d_Vfbzb2_dV_b_si=0.0;
     double d_Vfbzb2_dV_e_si=0.0;
     double d_Vfbzb2_dV_di_si=0.0;
     double d_Vfbzb2_dV_gi_si=0.0;
double Vfb2=0.0;
     double d_Vfb2_dV_b_si=0.0;
     double d_Vfb2_dV_di_si=0.0;
     double d_Vfb2_dV_gi_si=0.0;
     double d_Vfb2_dV_e_si=0.0;
     double d_Vfb2_dTemp_t_GND=0.0;
     double d_Vfb2_dV_p_GND=0.0;
     double d_Vfb2_dV_b_GND=0.0;
double Vfbeff2=0.0;
     double d_Vfbeff2_dV_b_si=0.0;
     double d_Vfbeff2_dV_di_si=0.0;
     double d_Vfbeff2_dV_gi_si=0.0;
     double d_Vfbeff2_dV_e_si=0.0;
     double d_Vfbeff2_dTemp_t_GND=0.0;
     double d_Vfbeff2_dV_p_GND=0.0;
     double d_Vfbeff2_dV_b_GND=0.0;
     double d_Vfbeff2_dV_gi_p=0.0;
     double d_Vfbeff2_dV_db_di=0.0;
     double d_Vfbeff2_dV_sb_si=0.0;
double VdsatCV2=0.0;
     double d_VdsatCV2_dV_gi_p=0.0;
     double d_VdsatCV2_dV_b_si=0.0;
     double d_VdsatCV2_dV_e_si=0.0;
     double d_VdsatCV2_dV_di_si=0.0;
     double d_VdsatCV2_dV_gi_si=0.0;
     double d_VdsatCV2_dTemp_t_GND=0.0;
     double d_VdsatCV2_dV_p_GND=0.0;
     double d_VdsatCV2_dV_b_GND=0.0;
     double d_VdsatCV2_dV_db_di=0.0;
     double d_VdsatCV2_dV_sb_si=0.0;
double VdseffCV2=0.0;
     double d_VdseffCV2_dV_gi_p=0.0;
     double d_VdseffCV2_dV_b_si=0.0;
     double d_VdseffCV2_dV_e_si=0.0;
     double d_VdseffCV2_dV_di_si=0.0;
     double d_VdseffCV2_dV_gi_si=0.0;
     double d_VdseffCV2_dTemp_t_GND=0.0;
     double d_VdseffCV2_dV_p_GND=0.0;
     double d_VdseffCV2_dV_b_GND=0.0;
     double d_VdseffCV2_dV_db_di=0.0;
     double d_VdseffCV2_dV_sb_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double Tcen2=0.0;
     double d_Tcen2_dV_gi_p=0.0;
     double d_Tcen2_dV_db_di=0.0;
     double d_Tcen2_dV_sb_si=0.0;
     double d_Tcen2_dV_b_si=0.0;
     double d_Tcen2_dV_di_si=0.0;
     double d_Tcen2_dV_gi_si=0.0;
     double d_Tcen2_dV_e_si=0.0;
     double d_Tcen2_dTemp_t_GND=0.0;
     double d_Tcen2_dV_p_GND=0.0;
     double d_Tcen2_dV_b_GND=0.0;
double Ccen2=0.0;
     double d_Ccen2_dV_gi_p=0.0;
     double d_Ccen2_dV_db_di=0.0;
     double d_Ccen2_dV_sb_si=0.0;
     double d_Ccen2_dV_b_si=0.0;
     double d_Ccen2_dV_di_si=0.0;
     double d_Ccen2_dV_gi_si=0.0;
     double d_Ccen2_dV_e_si=0.0;
     double d_Ccen2_dTemp_t_GND=0.0;
     double d_Ccen2_dV_p_GND=0.0;
     double d_Ccen2_dV_b_GND=0.0;
double Coxeff2=0.0;
     double d_Coxeff2_dV_gi_p=0.0;
     double d_Coxeff2_dV_db_di=0.0;
     double d_Coxeff2_dV_sb_si=0.0;
     double d_Coxeff2_dV_b_si=0.0;
     double d_Coxeff2_dV_e_si=0.0;
     double d_Coxeff2_dV_di_si=0.0;
     double d_Coxeff2_dV_gi_si=0.0;
     double d_Coxeff2_dTemp_t_GND=0.0;
     double d_Coxeff2_dV_p_GND=0.0;
     double d_Coxeff2_dV_b_GND=0.0;
double CoxWLcen2=0.0;
     double d_CoxWLcen2_dV_gi_p=0.0;
     double d_CoxWLcen2_dV_db_di=0.0;
     double d_CoxWLcen2_dV_sb_si=0.0;
     double d_CoxWLcen2_dV_b_si=0.0;
     double d_CoxWLcen2_dV_e_si=0.0;
     double d_CoxWLcen2_dV_di_si=0.0;
     double d_CoxWLcen2_dV_gi_si=0.0;
     double d_CoxWLcen2_dTemp_t_GND=0.0;
     double d_CoxWLcen2_dV_p_GND=0.0;
     double d_CoxWLcen2_dV_b_GND=0.0;
double CoxWLcenb2=0.0;
     double d_CoxWLcenb2_dV_gi_p=0.0;
     double d_CoxWLcenb2_dV_db_di=0.0;
     double d_CoxWLcenb2_dV_sb_si=0.0;
     double d_CoxWLcenb2_dV_b_si=0.0;
     double d_CoxWLcenb2_dV_e_si=0.0;
     double d_CoxWLcenb2_dV_di_si=0.0;
     double d_CoxWLcenb2_dV_gi_si=0.0;
     double d_CoxWLcenb2_dTemp_t_GND=0.0;
     double d_CoxWLcenb2_dV_p_GND=0.0;
     double d_CoxWLcenb2_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltaPhi2=0.0;
     double d_DeltaPhi2_dTemp_t_GND=0.0;
     double d_DeltaPhi2_dV_p_GND=0.0;
     double d_DeltaPhi2_dV_b_GND=0.0;
     double d_DeltaPhi2_dV_gi_p=0.0;
     double d_DeltaPhi2_dV_db_di=0.0;
     double d_DeltaPhi2_dV_sb_si=0.0;
     double d_DeltaPhi2_dV_b_si=0.0;
     double d_DeltaPhi2_dV_e_si=0.0;
     double d_DeltaPhi2_dV_di_si=0.0;
     double d_DeltaPhi2_dV_gi_si=0.0;
//End of Block-local variables
//Block-local variables for block load
double qsrc2=0.0;
     double d_qsrc2_dV_gi_p=0.0;
     double d_qsrc2_dV_db_di=0.0;
     double d_qsrc2_dV_sb_si=0.0;
     double d_qsrc2_dV_b_si=0.0;
     double d_qsrc2_dV_e_si=0.0;
     double d_qsrc2_dV_di_si=0.0;
     double d_qsrc2_dV_gi_si=0.0;
     double d_qsrc2_dTemp_t_GND=0.0;
     double d_qsrc2_dV_p_GND=0.0;
     double d_qsrc2_dV_b_GND=0.0;
double qbulk2=0.0;
     double d_qbulk2_dV_gi_p=0.0;
     double d_qbulk2_dV_db_di=0.0;
     double d_qbulk2_dV_sb_si=0.0;
     double d_qbulk2_dV_b_si=0.0;
     double d_qbulk2_dV_e_si=0.0;
     double d_qbulk2_dV_di_si=0.0;
     double d_qbulk2_dV_gi_si=0.0;
     double d_qbulk2_dTemp_t_GND=0.0;
     double d_qbulk2_dV_p_GND=0.0;
     double d_qbulk2_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double DELTA_3_SOI2=0.0;
//End of Block-local variables
//Block-local variables for block load
double cdep0=0.0;
     double d_cdep0_dTemp_t_GND=0.0;
     double d_cdep0_dV_p_GND=0.0;
     double d_cdep0_dV_b_GND=0.0;
double theta0vb0=0.0;
     double d_theta0vb0_dTemp_t_GND=0.0;
     double d_theta0vb0_dV_p_GND=0.0;
     double d_theta0vb0_dV_b_GND=0.0;
double thetaRout=0.0;
     double d_thetaRout_dTemp_t_GND=0.0;
     double d_thetaRout_dV_p_GND=0.0;
     double d_thetaRout_dV_b_GND=0.0;
//End of Block-local variables
//Block-local variables for block load
double agidl=0.0;
double bgidl=0.0;
double cgidl=0.0;
double egidl=0.0;
double rgidl=0.0;
double kgidl=0.0;
double fgidl=0.0;
//End of Block-local variables
//Block-local variables for block load
double agisl=0.0;
double bgisl=0.0;
double cgisl=0.0;
double egisl=0.0;
double rgisl=0.0;
double kgisl=0.0;
double fgisl=0.0;
//End of Block-local variables
//Block-local variables for block load
double ucs=0.0;
     double d_ucs_dTemp_t_GND=0.0;
     double d_ucs_dV_p_GND=0.0;
     double d_ucs_dV_b_GND=0.0;
double ud=0.0;
     double d_ud_dTemp_t_GND=0.0;
     double d_ud_dV_p_GND=0.0;
     double d_ud_dV_b_GND=0.0;
//End of Block-local variables

d_Coxeff2_dV_gi_p = d_Coxeff2_dV_db_di = d_Coxeff2_dV_sb_si = d_Coxeff2_dV_b_si = d_Coxeff2_dV_e_si = d_Coxeff2_dV_di_si = d_Coxeff2_dV_gi_si = d_Coxeff2_dTemp_t_GND = d_Coxeff2_dV_p_GND = d_Coxeff2_dV_b_GND =  0.0;
Coxeff2 = 0;

d_CoxWLcen2_dV_gi_p = d_CoxWLcen2_dV_db_di = d_CoxWLcen2_dV_sb_si = d_CoxWLcen2_dV_b_si = d_CoxWLcen2_dV_e_si = d_CoxWLcen2_dV_di_si = d_CoxWLcen2_dV_gi_si = d_CoxWLcen2_dTemp_t_GND = d_CoxWLcen2_dV_p_GND = d_CoxWLcen2_dV_b_GND =  0.0;
CoxWLcen2 = 0;

d_CoxWLcenb2_dV_gi_p = d_CoxWLcenb2_dV_db_di = d_CoxWLcenb2_dV_sb_si = d_CoxWLcenb2_dV_b_si = d_CoxWLcenb2_dV_e_si = d_CoxWLcenb2_dV_di_si = d_CoxWLcenb2_dV_gi_si = d_CoxWLcenb2_dTemp_t_GND = d_CoxWLcenb2_dV_p_GND = d_CoxWLcenb2_dV_b_GND =  0.0;
CoxWLcenb2 = 0;

d_DeltaPhi2_dTemp_t_GND = d_DeltaPhi2_dV_p_GND = d_DeltaPhi2_dV_b_GND = d_DeltaPhi2_dV_gi_p = d_DeltaPhi2_dV_db_di = d_DeltaPhi2_dV_sb_si = d_DeltaPhi2_dV_b_si = d_DeltaPhi2_dV_e_si = d_DeltaPhi2_dV_di_si = d_DeltaPhi2_dV_gi_si =  0.0;
DeltaPhi2 = 0;

d_Tcen2_dV_gi_p = d_Tcen2_dV_db_di = d_Tcen2_dV_sb_si = d_Tcen2_dV_b_si = d_Tcen2_dV_di_si = d_Tcen2_dV_gi_si = d_Tcen2_dV_e_si = d_Tcen2_dTemp_t_GND = d_Tcen2_dV_p_GND = d_Tcen2_dV_b_GND =  0.0;
Tcen2 = 0;

d_T02_dV_b_si = d_T02_dV_e_si = d_T02_dV_di_si = d_T02_dV_gi_si = d_T02_dTemp_t_GND = d_T02_dV_p_GND = d_T02_dV_b_GND = d_T02_dV_gi_p = d_T02_dV_db_di = d_T02_dV_sb_si =  0.0;
T02 = 0;

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si = d_T12_dV_b_si = d_T12_dV_e_si = d_T12_dV_di_si = d_T12_dV_gi_si = d_T12_dTemp_t_GND = d_T12_dV_p_GND = d_T12_dV_b_GND =  0.0;
T12 = 0;

d_T22_dV_gi_p = d_T22_dV_db_di = d_T22_dV_sb_si = d_T22_dV_b_si = d_T22_dV_e_si = d_T22_dV_di_si = d_T22_dV_gi_si = d_T22_dTemp_t_GND = d_T22_dV_p_GND = d_T22_dV_b_GND =  0.0;
T22 = 0;

d_VdseffCV2_dV_gi_p = d_VdseffCV2_dV_b_si = d_VdseffCV2_dV_e_si = d_VdseffCV2_dV_di_si = d_VdseffCV2_dV_gi_si = d_VdseffCV2_dTemp_t_GND = d_VdseffCV2_dV_p_GND = d_VdseffCV2_dV_b_GND = d_VdseffCV2_dV_db_di = d_VdseffCV2_dV_sb_si =  0.0;
VdseffCV2 = 0;

d_Vfb2_dV_b_si = d_Vfb2_dV_di_si = d_Vfb2_dV_gi_si = d_Vfb2_dV_e_si = d_Vfb2_dTemp_t_GND = d_Vfb2_dV_p_GND = d_Vfb2_dV_b_GND =  0.0;
Vfb2 = 0;

d_Vfbeff2_dV_b_si = d_Vfbeff2_dV_di_si = d_Vfbeff2_dV_gi_si = d_Vfbeff2_dV_e_si = d_Vfbeff2_dTemp_t_GND = d_Vfbeff2_dV_p_GND = d_Vfbeff2_dV_b_GND = d_Vfbeff2_dV_gi_p = d_Vfbeff2_dV_db_di = d_Vfbeff2_dV_sb_si =  0.0;
Vfbeff2 = 0;

d_Vfbzb2_dTemp_t_GND = d_Vfbzb2_dV_p_GND = d_Vfbzb2_dV_b_GND = d_Vfbzb2_dV_b_si = d_Vfbzb2_dV_e_si = d_Vfbzb2_dV_di_si = d_Vfbzb2_dV_gi_si =  0.0;
Vfbzb2 = 0;

d_Vgsteff2_dV_gi_p = d_Vgsteff2_dV_b_si = d_Vgsteff2_dV_e_si = d_Vgsteff2_dV_di_si = d_Vgsteff2_dV_gi_si = d_Vgsteff2_dTemp_t_GND = d_Vgsteff2_dV_p_GND = d_Vgsteff2_dV_b_GND = d_Vgsteff2_dV_db_di = d_Vgsteff2_dV_sb_si =  0.0;
Vgsteff2 = 0;

d_rds0_dTemp_t_GND = d_rds0_dV_p_GND = d_rds0_dV_b_GND =  0.0;
rds0 = pParam_B4SOIrds0;

d_rd0_dTemp_t_GND = d_rd0_dV_p_GND = d_rd0_dV_b_GND =  0.0;
rd0 = pParam_B4SOIrd0;

d_rs0_dTemp_t_GND = d_rs0_dV_p_GND = d_rs0_dV_b_GND =  0.0;
rs0 = pParam_B4SOIrs0;

d_rdwmin_dTemp_t_GND = d_rdwmin_dV_p_GND = d_rdwmin_dV_b_GND =  0.0;
rdwmin = pParam_B4SOIrdwmin;

d_rswmin_dTemp_t_GND = d_rswmin_dV_p_GND = d_rswmin_dV_b_GND =  0.0;
rswmin = pParam_B4SOIrswmin;
if (((SHMOD==1)&&(RTH0!=0.0)))
{
if ((B4SOImtrlMod==0))
{

d_Vtm_dV_b_GND = 8.617087e-5*d_DevTemp_dV_b_GND;
d_Vtm_dV_p_GND = 8.617087e-5*d_DevTemp_dV_p_GND;
d_Vtm_dTemp_t_GND = 8.617087e-5*d_DevTemp_dTemp_t_GND;
Vtm = (8.617087e-5*DevTemp);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = d_DevTemp_dV_b_GND;
d_T0_dV_p_GND = d_DevTemp_dV_p_GND;
d_T0_dTemp_t_GND = d_DevTemp_dTemp_t_GND;
T0 = (1108.0+DevTemp);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si = d_T5_dV_di_si = d_T5_dV_gi_si =  0.0;
d_T5_dV_b_GND = (DevTemp*d_DevTemp_dV_b_GND+d_DevTemp_dV_b_GND*DevTemp);
d_T5_dV_p_GND = (DevTemp*d_DevTemp_dV_p_GND+d_DevTemp_dV_p_GND*DevTemp);
d_T5_dTemp_t_GND = (DevTemp*d_DevTemp_dTemp_t_GND+d_DevTemp_dTemp_t_GND*DevTemp);
T5 = (DevTemp*DevTemp);

d_Eg_dV_b_GND = (-((T0*7.02e-4*d_T5_dV_b_GND-(7.02e-4*T5)*d_T0_dV_b_GND)/T0/T0));
d_Eg_dV_p_GND = (-((T0*7.02e-4*d_T5_dV_p_GND-(7.02e-4*T5)*d_T0_dV_p_GND)/T0/T0));
d_Eg_dTemp_t_GND = (-((T0*7.02e-4*d_T5_dTemp_t_GND-(7.02e-4*T5)*d_T0_dTemp_t_GND)/T0/T0));
Eg = (1.16-((7.02e-4*T5)/T0));

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = 1.9230584e-4;
{
double value_sqrt_0 = sqrt(DevTemp);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si = d_T5_dV_di_si = d_T5_dV_gi_si =  0.0;
d_T5_dV_b_GND = (deriv_sqrt_0_d0*(d_DevTemp_dV_b_GND));
d_T5_dV_p_GND = (deriv_sqrt_0_d0*(d_DevTemp_dV_p_GND));
d_T5_dTemp_t_GND = (deriv_sqrt_0_d0*(d_DevTemp_dTemp_t_GND));
T5 = value_sqrt_0;
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = (((1.45e10*DevTemp)*T5)*d_T2_dV_b_GND+((1.45e10*DevTemp)*d_T5_dV_b_GND+1.45e10*d_DevTemp_dV_b_GND*T5)*T2);
d_T3_dV_p_GND = (((1.45e10*DevTemp)*T5)*d_T2_dV_p_GND+((1.45e10*DevTemp)*d_T5_dV_p_GND+1.45e10*d_DevTemp_dV_p_GND*T5)*T2);
d_T3_dTemp_t_GND = (((1.45e10*DevTemp)*T5)*d_T2_dTemp_t_GND+((1.45e10*DevTemp)*d_T5_dTemp_t_GND+1.45e10*d_DevTemp_dTemp_t_GND*T5)*T2);
T3 = (((1.45e10*DevTemp)*T5)*T2);

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si =  0.0;
d_T6_dV_b_GND = (-(((2.0*Vtm)*d_Eg_dV_b_GND-Eg*2.0*d_Vtm_dV_b_GND)/(2.0*Vtm)/(2.0*Vtm)));
d_T6_dV_p_GND = (-(((2.0*Vtm)*d_Eg_dV_p_GND-Eg*2.0*d_Vtm_dV_p_GND)/(2.0*Vtm)/(2.0*Vtm)));
d_T6_dTemp_t_GND = (-(((2.0*Vtm)*d_Eg_dTemp_t_GND-Eg*2.0*d_Vtm_dTemp_t_GND)/(2.0*Vtm)/(2.0*Vtm)));
T6 = (21.5565981-(Eg/(2.0*Vtm)));
if ((T6>(-100.0)))
{
{
double value_exp_0 = exp(T6);
double  deriv_exp_0_d0 = value_exp_0;

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si =  0.0;
d_T4_dV_b_GND = (deriv_exp_0_d0*(d_T6_dV_b_GND));
d_T4_dV_p_GND = (deriv_exp_0_d0*(d_T6_dV_p_GND));
d_T4_dTemp_t_GND = (deriv_exp_0_d0*(d_T6_dTemp_t_GND));
T4 = value_exp_0;
}
}
else
{
{
double value_exp_0 = exp((-100.0));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = value_exp_0;
}
}

d_ni_dV_b_GND = (T3*d_T4_dV_b_GND+d_T3_dV_b_GND*T4);
d_ni_dV_p_GND = (T3*d_T4_dV_p_GND+d_T3_dV_p_GND*T4);
d_ni_dTemp_t_GND = (T3*d_T4_dTemp_t_GND+d_T3_dTemp_t_GND*T4);
ni = (T3*T4);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*pParam_B4SOInpeak)/(ni*ni)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = evaluator_lln_0.getDerivs(((1.0e20*pParam_B4SOInpeak)/(ni*ni)),(-(1.0e20*pParam_B4SOInpeak)*(ni*d_ni_dV_b_GND+d_ni_dV_b_GND*ni)/(ni*ni)/(ni*ni)));
d_T0_dV_p_GND = evaluator_lln_0.getDerivs(((1.0e20*pParam_B4SOInpeak)/(ni*ni)),(-(1.0e20*pParam_B4SOInpeak)*(ni*d_ni_dV_p_GND+d_ni_dV_p_GND*ni)/(ni*ni)/(ni*ni)));
d_T0_dTemp_t_GND = evaluator_lln_0.getDerivs(((1.0e20*pParam_B4SOInpeak)/(ni*ni)),(-(1.0e20*pParam_B4SOInpeak)*(ni*d_ni_dTemp_t_GND+d_ni_dTemp_t_GND*ni)/(ni*ni)/(ni*ni)));
T0 = evaluator_lln_0.getValues(((1.0e20*pParam_B4SOInpeak)/(ni*ni)));
}

d_vbi_dV_b_GND = (Vtm*d_T0_dV_b_GND+d_Vtm_dV_b_GND*T0);
d_vbi_dV_p_GND = (Vtm*d_T0_dV_p_GND+d_Vtm_dV_p_GND*T0);
d_vbi_dTemp_t_GND = (Vtm*d_T0_dTemp_t_GND+d_Vtm_dTemp_t_GND*T0);
vbi = (Vtm*T0);
}
else
{
Tnom = B4SOItnom;

d_Vtm_dV_b_GND = 8.617087e-5*d_DevTemp_dV_b_GND;
d_Vtm_dV_p_GND = 8.617087e-5*d_DevTemp_dV_p_GND;
d_Vtm_dTemp_t_GND = 8.617087e-5*d_DevTemp_dTemp_t_GND;
Vtm = (8.617087e-5*DevTemp);
Vtm0 = (8.617087e-5*Tnom);
Eg0 = B4SOIeg0;

d_Eg_dV_b_GND = (-(((DevTemp+B4SOItbgbsub)*((B4SOItbgasub*DevTemp)*d_DevTemp_dV_b_GND+B4SOItbgasub*d_DevTemp_dV_b_GND*DevTemp)-((B4SOItbgasub*DevTemp)*DevTemp)*d_DevTemp_dV_b_GND)/(DevTemp+B4SOItbgbsub)/(DevTemp+B4SOItbgbsub)));
d_Eg_dV_p_GND = (-(((DevTemp+B4SOItbgbsub)*((B4SOItbgasub*DevTemp)*d_DevTemp_dV_p_GND+B4SOItbgasub*d_DevTemp_dV_p_GND*DevTemp)-((B4SOItbgasub*DevTemp)*DevTemp)*d_DevTemp_dV_p_GND)/(DevTemp+B4SOItbgbsub)/(DevTemp+B4SOItbgbsub)));
d_Eg_dTemp_t_GND = (-(((DevTemp+B4SOItbgbsub)*((B4SOItbgasub*DevTemp)*d_DevTemp_dTemp_t_GND+B4SOItbgasub*d_DevTemp_dTemp_t_GND*DevTemp)-((B4SOItbgasub*DevTemp)*DevTemp)*d_DevTemp_dTemp_t_GND)/(DevTemp+B4SOItbgbsub)/(DevTemp+B4SOItbgbsub)));
Eg = (B4SOIbg0sub-(((B4SOItbgasub*DevTemp)*DevTemp)/(DevTemp+B4SOItbgbsub)));
{
double value_sqrt_0 = sqrt(((Tnom*Tnom)*Tnom));

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = (1/value_sqrt_0);
}
{
double value_sqrt_0 = sqrt(DevTemp);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si = d_T5_dV_di_si = d_T5_dV_gi_si =  0.0;
d_T5_dV_b_GND = (deriv_sqrt_0_d0*(d_DevTemp_dV_b_GND));
d_T5_dV_p_GND = (deriv_sqrt_0_d0*(d_DevTemp_dV_p_GND));
d_T5_dTemp_t_GND = (deriv_sqrt_0_d0*(d_DevTemp_dTemp_t_GND));
T5 = value_sqrt_0;
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = (((B4SOIni0sub*DevTemp)*T5)*d_T2_dV_b_GND+((B4SOIni0sub*DevTemp)*d_T5_dV_b_GND+B4SOIni0sub*d_DevTemp_dV_b_GND*T5)*T2);
d_T3_dV_p_GND = (((B4SOIni0sub*DevTemp)*T5)*d_T2_dV_p_GND+((B4SOIni0sub*DevTemp)*d_T5_dV_p_GND+B4SOIni0sub*d_DevTemp_dV_p_GND*T5)*T2);
d_T3_dTemp_t_GND = (((B4SOIni0sub*DevTemp)*T5)*d_T2_dTemp_t_GND+((B4SOIni0sub*DevTemp)*d_T5_dTemp_t_GND+B4SOIni0sub*d_DevTemp_dTemp_t_GND*T5)*T2);
T3 = (((B4SOIni0sub*DevTemp)*T5)*T2);
{
double value_exp_0 = exp(((Eg0/(2.0*Vtm0))-(Eg/(2.0*Vtm))));
double  deriv_exp_0_d0 = value_exp_0;

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si =  0.0;
d_T4_dV_b_GND = (deriv_exp_0_d0*((-(((2.0*Vtm)*d_Eg_dV_b_GND-Eg*2.0*d_Vtm_dV_b_GND)/(2.0*Vtm)/(2.0*Vtm)))));
d_T4_dV_p_GND = (deriv_exp_0_d0*((-(((2.0*Vtm)*d_Eg_dV_p_GND-Eg*2.0*d_Vtm_dV_p_GND)/(2.0*Vtm)/(2.0*Vtm)))));
d_T4_dTemp_t_GND = (deriv_exp_0_d0*((-(((2.0*Vtm)*d_Eg_dTemp_t_GND-Eg*2.0*d_Vtm_dTemp_t_GND)/(2.0*Vtm)/(2.0*Vtm)))));
T4 = value_exp_0;
}

d_ni_dV_b_GND = (T3*d_T4_dV_b_GND+d_T3_dV_b_GND*T4);
d_ni_dV_p_GND = (T3*d_T4_dV_p_GND+d_T3_dV_p_GND*T4);
d_ni_dTemp_t_GND = (T3*d_T4_dTemp_t_GND+d_T3_dTemp_t_GND*T4);
ni = (T3*T4);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*pParam_B4SOInpeak)/(ni*ni)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = evaluator_lln_0.getDerivs(((1.0e20*pParam_B4SOInpeak)/(ni*ni)),(-(1.0e20*pParam_B4SOInpeak)*(ni*d_ni_dV_b_GND+d_ni_dV_b_GND*ni)/(ni*ni)/(ni*ni)));
d_T0_dV_p_GND = evaluator_lln_0.getDerivs(((1.0e20*pParam_B4SOInpeak)/(ni*ni)),(-(1.0e20*pParam_B4SOInpeak)*(ni*d_ni_dV_p_GND+d_ni_dV_p_GND*ni)/(ni*ni)/(ni*ni)));
d_T0_dTemp_t_GND = evaluator_lln_0.getDerivs(((1.0e20*pParam_B4SOInpeak)/(ni*ni)),(-(1.0e20*pParam_B4SOInpeak)*(ni*d_ni_dTemp_t_GND+d_ni_dTemp_t_GND*ni)/(ni*ni)/(ni*ni)));
T0 = evaluator_lln_0.getValues(((1.0e20*pParam_B4SOInpeak)/(ni*ni)));
}

d_vbi_dV_b_GND = (Vtm*d_T0_dV_b_GND+d_Vtm_dV_b_GND*T0);
d_vbi_dV_p_GND = (Vtm*d_T0_dV_p_GND+d_Vtm_dV_p_GND*T0);
d_vbi_dTemp_t_GND = (Vtm*d_T0_dTemp_t_GND+d_Vtm_dTemp_t_GND*T0);
vbi = (Vtm*T0);
}
if ((pParam_B4SOInsub>0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOInpeak/pParam_B4SOInsub));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = evaluator_lln_0.getValues((pParam_B4SOInpeak/pParam_B4SOInsub));
}

d_vfbb_dV_b_GND = (((-B4SOItype)*Vtm)*d_T0_dV_b_GND+(-B4SOItype)*d_Vtm_dV_b_GND*T0);
d_vfbb_dV_p_GND = (((-B4SOItype)*Vtm)*d_T0_dV_p_GND+(-B4SOItype)*d_Vtm_dV_p_GND*T0);
d_vfbb_dTemp_t_GND = (((-B4SOItype)*Vtm)*d_T0_dTemp_t_GND+(-B4SOItype)*d_Vtm_dTemp_t_GND*T0);
vfbb = (((-B4SOItype)*Vtm)*T0);
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)/ni));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = evaluator_lln_0.getDerivs(((((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)/ni),((ni*(-((-pParam_B4SOInpeak)*pParam_B4SOInsub)*d_ni_dV_b_GND/ni/ni)-(((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)*d_ni_dV_b_GND)/ni/ni));
d_T0_dV_p_GND = evaluator_lln_0.getDerivs(((((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)/ni),((ni*(-((-pParam_B4SOInpeak)*pParam_B4SOInsub)*d_ni_dV_p_GND/ni/ni)-(((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)*d_ni_dV_p_GND)/ni/ni));
d_T0_dTemp_t_GND = evaluator_lln_0.getDerivs(((((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)/ni),((ni*(-((-pParam_B4SOInpeak)*pParam_B4SOInsub)*d_ni_dTemp_t_GND/ni/ni)-(((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)*d_ni_dTemp_t_GND)/ni/ni));
T0 = evaluator_lln_0.getValues(((((-pParam_B4SOInpeak)*pParam_B4SOInsub)/ni)/ni));
}

d_vfbb_dV_b_GND = (((-B4SOItype)*Vtm)*d_T0_dV_b_GND+(-B4SOItype)*d_Vtm_dV_b_GND*T0);
d_vfbb_dV_p_GND = (((-B4SOItype)*Vtm)*d_T0_dV_p_GND+(-B4SOItype)*d_Vtm_dV_p_GND*T0);
d_vfbb_dTemp_t_GND = (((-B4SOItype)*Vtm)*d_T0_dTemp_t_GND+(-B4SOItype)*d_Vtm_dTemp_t_GND*T0);
vfbb = (((-B4SOItype)*Vtm)*T0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOInpeak/ni));

d_phi_dV_b_GND = ((2.0*Vtm)*evaluator_lln_0.getDerivs((pParam_B4SOInpeak/ni),(-pParam_B4SOInpeak*d_ni_dV_b_GND/ni/ni))+2.0*d_Vtm_dV_b_GND*evaluator_lln_0.getValues((pParam_B4SOInpeak/ni)));
d_phi_dV_p_GND = ((2.0*Vtm)*evaluator_lln_0.getDerivs((pParam_B4SOInpeak/ni),(-pParam_B4SOInpeak*d_ni_dV_p_GND/ni/ni))+2.0*d_Vtm_dV_p_GND*evaluator_lln_0.getValues((pParam_B4SOInpeak/ni)));
d_phi_dTemp_t_GND = ((2.0*Vtm)*evaluator_lln_0.getDerivs((pParam_B4SOInpeak/ni),(-pParam_B4SOInpeak*d_ni_dTemp_t_GND/ni/ni))+2.0*d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues((pParam_B4SOInpeak/ni)));
phi = ((2.0*Vtm)*evaluator_lln_0.getValues((pParam_B4SOInpeak/ni)));
}
{
double value_sqrt_0 = sqrt(phi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_sqrtPhi_dV_b_GND = (deriv_sqrt_0_d0*(d_phi_dV_b_GND));
d_sqrtPhi_dV_p_GND = (deriv_sqrt_0_d0*(d_phi_dV_p_GND));
d_sqrtPhi_dTemp_t_GND = (deriv_sqrt_0_d0*(d_phi_dTemp_t_GND));
sqrtPhi = value_sqrt_0;
}
{
double value_sqrt_0 = sqrt(((2.0*epssub)/((1.60219e-19*pParam_B4SOInpeak)*1.0e6)));

d_Xdep0_dV_b_GND = value_sqrt_0*d_sqrtPhi_dV_b_GND;
d_Xdep0_dV_p_GND = value_sqrt_0*d_sqrtPhi_dV_p_GND;
d_Xdep0_dTemp_t_GND = value_sqrt_0*d_sqrtPhi_dTemp_t_GND;
Xdep0 = (value_sqrt_0*sqrtPhi);
}
{
double value_sqrt_0 = sqrt(((((1.60219e-19*epssub)*pParam_B4SOInpeak)*1.0e6)/2.0));

d_cdep0_dV_b_GND = (-value_sqrt_0*d_sqrtPhi_dV_b_GND/sqrtPhi/sqrtPhi);
d_cdep0_dV_p_GND = (-value_sqrt_0*d_sqrtPhi_dV_p_GND/sqrtPhi/sqrtPhi);
d_cdep0_dTemp_t_GND = (-value_sqrt_0*d_sqrtPhi_dTemp_t_GND/sqrtPhi/sqrtPhi);
cdep0 = (value_sqrt_0/sqrtPhi);
}
{
double value_sqrt_0 = sqrt((((epssub/(epsrox*8.85418e-12))*toxe)*Xdep0));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*(((epssub/(epsrox*8.85418e-12))*toxe)*d_Xdep0_dV_b_GND));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*(((epssub/(epsrox*8.85418e-12))*toxe)*d_Xdep0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*(((epssub/(epsrox*8.85418e-12))*toxe)*d_Xdep0_dTemp_t_GND));
T1 = value_sqrt_0;
}
{
double value_exp_0 = exp(((((-0.5)*pParam_B4SOIdsub)*pParam_B4SOIleff)/T1));
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (deriv_exp_0_d0*((-(((-0.5)*pParam_B4SOIdsub)*pParam_B4SOIleff)*d_T1_dV_b_GND/T1/T1)));
d_T0_dV_p_GND = (deriv_exp_0_d0*((-(((-0.5)*pParam_B4SOIdsub)*pParam_B4SOIleff)*d_T1_dV_p_GND/T1/T1)));
d_T0_dTemp_t_GND = (deriv_exp_0_d0*((-(((-0.5)*pParam_B4SOIdsub)*pParam_B4SOIleff)*d_T1_dTemp_t_GND/T1/T1)));
T0 = value_exp_0;
}

d_theta0vb0_dV_b_GND = (d_T0_dV_b_GND+((2.0*T0)*d_T0_dV_b_GND+2.0*d_T0_dV_b_GND*T0));
d_theta0vb0_dV_p_GND = (d_T0_dV_p_GND+((2.0*T0)*d_T0_dV_p_GND+2.0*d_T0_dV_p_GND*T0));
d_theta0vb0_dTemp_t_GND = (d_T0_dTemp_t_GND+((2.0*T0)*d_T0_dTemp_t_GND+2.0*d_T0_dTemp_t_GND*T0));
theta0vb0 = (T0+((2.0*T0)*T0));
{
double value_exp_0 = exp(((((-0.5)*pParam_B4SOIdrout)*pParam_B4SOIleff)/T1));
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (deriv_exp_0_d0*((-(((-0.5)*pParam_B4SOIdrout)*pParam_B4SOIleff)*d_T1_dV_b_GND/T1/T1)));
d_T0_dV_p_GND = (deriv_exp_0_d0*((-(((-0.5)*pParam_B4SOIdrout)*pParam_B4SOIleff)*d_T1_dV_p_GND/T1/T1)));
d_T0_dTemp_t_GND = (deriv_exp_0_d0*((-(((-0.5)*pParam_B4SOIdrout)*pParam_B4SOIleff)*d_T1_dTemp_t_GND/T1/T1)));
T0 = value_exp_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = (d_T0_dV_b_GND+((2.0*T0)*d_T0_dV_b_GND+2.0*d_T0_dV_b_GND*T0));
d_T2_dV_p_GND = (d_T0_dV_p_GND+((2.0*T0)*d_T0_dV_p_GND+2.0*d_T0_dV_p_GND*T0));
d_T2_dTemp_t_GND = (d_T0_dTemp_t_GND+((2.0*T0)*d_T0_dTemp_t_GND+2.0*d_T0_dTemp_t_GND*T0));
T2 = (T0+((2.0*T0)*T0));

d_thetaRout_dV_b_GND = pParam_B4SOIpdibl1*d_T2_dV_b_GND;
d_thetaRout_dV_p_GND = pParam_B4SOIpdibl1*d_T2_dV_p_GND;
d_thetaRout_dTemp_t_GND = pParam_B4SOIpdibl1*d_T2_dTemp_t_GND;
thetaRout = ((pParam_B4SOIpdibl1*T2)+pParam_B4SOIpdibl2);

d_B4SOIvtm_dV_b_GND = d_Vtm_dV_b_GND;
d_B4SOIvtm_dV_p_GND = d_Vtm_dV_p_GND;
d_B4SOIvtm_dTemp_t_GND = d_Vtm_dTemp_t_GND;
B4SOIvtm = Vtm;

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = d_TempRatio_dV_b_GND;
d_T3_dV_p_GND = d_TempRatio_dV_p_GND;
d_T3_dTemp_t_GND = d_TempRatio_dTemp_t_GND;
T3 = (TempRatio-1.0);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si =  0.0;
d_T4_dV_b_GND = ((1.115/Vtm)*d_T3_dV_b_GND+(-1.115*d_Vtm_dV_b_GND/Vtm/Vtm)*T3);
d_T4_dV_p_GND = ((1.115/Vtm)*d_T3_dV_p_GND+(-1.115*d_Vtm_dV_p_GND/Vtm/Vtm)*T3);
d_T4_dTemp_t_GND = ((1.115/Vtm)*d_T3_dTemp_t_GND+(-1.115*d_Vtm_dTemp_t_GND/Vtm/Vtm)*T3);
T4 = ((1.115/Vtm)*T3);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = (pParam_B4SOIxbjt*d_T4_dV_b_GND/pParam_B4SOIndiode);
d_T7_dV_p_GND = (pParam_B4SOIxbjt*d_T4_dV_p_GND/pParam_B4SOIndiode);
d_T7_dTemp_t_GND = (pParam_B4SOIxbjt*d_T4_dTemp_t_GND/pParam_B4SOIndiode);
T7 = ((pParam_B4SOIxbjt*T4)/pParam_B4SOIndiode);
if ((T7>100.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T0_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T0_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T0_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T0_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T0 = value_exp_0;
}
}
}
if ((pParam_B4SOIxbjt==pParam_B4SOIxdif))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
T1 = T0;
}
else
{

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = (pParam_B4SOIxdif*d_T4_dV_b_GND/pParam_B4SOIndiode);
d_T7_dV_p_GND = (pParam_B4SOIxdif*d_T4_dV_p_GND/pParam_B4SOIndiode);
d_T7_dTemp_t_GND = (pParam_B4SOIxdif*d_T4_dTemp_t_GND/pParam_B4SOIndiode);
T7 = ((pParam_B4SOIxdif*T4)/pParam_B4SOIndiode);
if ((T7>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T1 = value_exp_0;
}
}
}
}

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = (pParam_B4SOIxrec*d_T4_dV_b_GND/pParam_B4SOInrecf0);
d_T7_dV_p_GND = (pParam_B4SOIxrec*d_T4_dV_p_GND/pParam_B4SOInrecf0);
d_T7_dTemp_t_GND = (pParam_B4SOIxrec*d_T4_dTemp_t_GND/pParam_B4SOInrecf0);
T7 = ((pParam_B4SOIxrec*T4)/pParam_B4SOInrecf0);
if ((T7>100.0))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T2_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T2_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T2_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T2_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T2 = value_exp_0;
}
}
}

d_Ahlis_dV_b_GND = pParam_B4SOIahli*d_T0_dV_b_GND;
d_Ahlis_dV_p_GND = pParam_B4SOIahli*d_T0_dV_p_GND;
d_Ahlis_dTemp_t_GND = pParam_B4SOIahli*d_T0_dTemp_t_GND;
Ahlis = (pParam_B4SOIahli*T0);

d_jbjts_dV_b_GND = pParam_B4SOIisbjt*d_T0_dV_b_GND;
d_jbjts_dV_p_GND = pParam_B4SOIisbjt*d_T0_dV_p_GND;
d_jbjts_dTemp_t_GND = pParam_B4SOIisbjt*d_T0_dTemp_t_GND;
jbjts = (pParam_B4SOIisbjt*T0);

d_jdifs_dV_b_GND = pParam_B4SOIisdif*d_T1_dV_b_GND;
d_jdifs_dV_p_GND = pParam_B4SOIisdif*d_T1_dV_p_GND;
d_jdifs_dTemp_t_GND = pParam_B4SOIisdif*d_T1_dTemp_t_GND;
jdifs = (pParam_B4SOIisdif*T1);

d_jrecs_dV_b_GND = pParam_B4SOIisrec*d_T2_dV_b_GND;
d_jrecs_dV_p_GND = pParam_B4SOIisrec*d_T2_dV_p_GND;
d_jrecs_dTemp_t_GND = pParam_B4SOIisrec*d_T2_dTemp_t_GND;
jrecs = (pParam_B4SOIisrec*T2);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = pParam_B4SOIxtun*d_T3_dV_b_GND;
d_T7_dV_p_GND = pParam_B4SOIxtun*d_T3_dV_p_GND;
d_T7_dTemp_t_GND = pParam_B4SOIxtun*d_T3_dTemp_t_GND;
T7 = (pParam_B4SOIxtun*T3);
if ((T7>100.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T0_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T0_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T0_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T0_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T0 = value_exp_0;
}
}
}

d_jtuns_dV_b_GND = pParam_B4SOIistun*d_T0_dV_b_GND;
d_jtuns_dV_p_GND = pParam_B4SOIistun*d_T0_dV_p_GND;
d_jtuns_dTemp_t_GND = pParam_B4SOIistun*d_T0_dTemp_t_GND;
jtuns = (pParam_B4SOIistun*T0);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = (pParam_B4SOIxbjt*d_T4_dV_b_GND/pParam_B4SOIndioded);
d_T7_dV_p_GND = (pParam_B4SOIxbjt*d_T4_dV_p_GND/pParam_B4SOIndioded);
d_T7_dTemp_t_GND = (pParam_B4SOIxbjt*d_T4_dTemp_t_GND/pParam_B4SOIndioded);
T7 = ((pParam_B4SOIxbjt*T4)/pParam_B4SOIndioded);
if ((T7>100.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T0_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T0_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T0_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T0_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T0 = value_exp_0;
}
}
}
if ((pParam_B4SOIxbjt==pParam_B4SOIxdifd))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
T1 = T0;
}
else
{

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = (pParam_B4SOIxdifd*d_T4_dV_b_GND/pParam_B4SOIndioded);
d_T7_dV_p_GND = (pParam_B4SOIxdifd*d_T4_dV_p_GND/pParam_B4SOIndioded);
d_T7_dTemp_t_GND = (pParam_B4SOIxdifd*d_T4_dTemp_t_GND/pParam_B4SOIndioded);
T7 = ((pParam_B4SOIxdifd*T4)/pParam_B4SOIndioded);
if ((T7>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T1 = value_exp_0;
}
}
}
}

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = (pParam_B4SOIxrecd*d_T4_dV_b_GND/pParam_B4SOInrecf0d);
d_T7_dV_p_GND = (pParam_B4SOIxrecd*d_T4_dV_p_GND/pParam_B4SOInrecf0d);
d_T7_dTemp_t_GND = (pParam_B4SOIxrecd*d_T4_dTemp_t_GND/pParam_B4SOInrecf0d);
T7 = ((pParam_B4SOIxrecd*T4)/pParam_B4SOInrecf0d);
if ((T7>100.0))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T2_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T2_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T2_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T2_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T2 = value_exp_0;
}
}
}

d_Ahlid_dV_b_GND = pParam_B4SOIahlid*d_T0_dV_b_GND;
d_Ahlid_dV_p_GND = pParam_B4SOIahlid*d_T0_dV_p_GND;
d_Ahlid_dTemp_t_GND = pParam_B4SOIahlid*d_T0_dTemp_t_GND;
Ahlid = (pParam_B4SOIahlid*T0);

d_jbjtd_dV_b_GND = pParam_B4SOIidbjt*d_T0_dV_b_GND;
d_jbjtd_dV_p_GND = pParam_B4SOIidbjt*d_T0_dV_p_GND;
d_jbjtd_dTemp_t_GND = pParam_B4SOIidbjt*d_T0_dTemp_t_GND;
jbjtd = (pParam_B4SOIidbjt*T0);

d_jdifd_dV_b_GND = pParam_B4SOIiddif*d_T1_dV_b_GND;
d_jdifd_dV_p_GND = pParam_B4SOIiddif*d_T1_dV_p_GND;
d_jdifd_dTemp_t_GND = pParam_B4SOIiddif*d_T1_dTemp_t_GND;
jdifd = (pParam_B4SOIiddif*T1);

d_jrecd_dV_b_GND = pParam_B4SOIidrec*d_T2_dV_b_GND;
d_jrecd_dV_p_GND = pParam_B4SOIidrec*d_T2_dV_p_GND;
d_jrecd_dTemp_t_GND = pParam_B4SOIidrec*d_T2_dTemp_t_GND;
jrecd = (pParam_B4SOIidrec*T2);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = pParam_B4SOIxtund*d_T3_dV_b_GND;
d_T7_dV_p_GND = pParam_B4SOIxtund*d_T3_dV_p_GND;
d_T7_dTemp_t_GND = pParam_B4SOIxtund*d_T3_dTemp_t_GND;
T7 = (pParam_B4SOIxtund*T3);
if ((T7>100.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = 2.688117142e+43*d_T7_dV_b_GND;
d_T0_dV_p_GND = 2.688117142e+43*d_T7_dV_p_GND;
d_T0_dTemp_t_GND = 2.688117142e+43*d_T7_dTemp_t_GND;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T0_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T0_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
T0 = value_exp_0;
}
}
}

d_jtund_dV_b_GND = pParam_B4SOIidtun*d_T0_dV_b_GND;
d_jtund_dV_p_GND = pParam_B4SOIidtun*d_T0_dV_p_GND;
d_jtund_dTemp_t_GND = pParam_B4SOIidtun*d_T0_dTemp_t_GND;
jtund = (pParam_B4SOIidtun*T0);
{
double value_pow_0 = pow(TempRatio,pParam_B4SOIute);
double  deriv_pow_0_d0 = ((TempRatio == 0.0)?0.0:(value_pow_0*pParam_B4SOIute/TempRatio));

d_u0temp_dV_b_GND = pParam_B4SOIu0*(deriv_pow_0_d0*(d_TempRatio_dV_b_GND));
d_u0temp_dV_p_GND = pParam_B4SOIu0*(deriv_pow_0_d0*(d_TempRatio_dV_p_GND));
d_u0temp_dTemp_t_GND = pParam_B4SOIu0*(deriv_pow_0_d0*(d_TempRatio_dTemp_t_GND));
u0temp = (pParam_B4SOIu0*value_pow_0);
}
if (((model_.VERSION)<4.2))
{

d_ku0temp_dV_b_GND = pParam_B4SOIku0*B4SOItku0*d_TempRatio_dV_b_GND;
d_ku0temp_dV_p_GND = pParam_B4SOIku0*B4SOItku0*d_TempRatio_dV_p_GND;
d_ku0temp_dTemp_t_GND = pParam_B4SOIku0*B4SOItku0*d_TempRatio_dTemp_t_GND;
ku0temp = ((pParam_B4SOIku0*(1.0+(B4SOItku0*TempRatio)))+1.0E-9);
}
else
{

d_ku0temp_dV_b_GND = pParam_B4SOIku0*B4SOItku0*d_T3_dV_b_GND;
d_ku0temp_dV_p_GND = pParam_B4SOIku0*B4SOItku0*d_T3_dV_p_GND;
d_ku0temp_dTemp_t_GND = pParam_B4SOIku0*B4SOItku0*d_T3_dTemp_t_GND;
ku0temp = ((pParam_B4SOIku0*(1.0+(B4SOItku0*T3)))+1.0E-9);
}

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si = d_T7_dTemp_t_GND = d_T7_dV_p_GND = d_T7_dV_b_GND =  0.0;
T7 = (B4SOIku0*pParam_B4SOIinv_od_ref);

d_rho_ref_dV_b_GND = ((ku0temp*d_T7_dV_b_GND-T7*d_ku0temp_dV_b_GND)/ku0temp/ku0temp);
d_rho_ref_dV_p_GND = ((ku0temp*d_T7_dV_p_GND-T7*d_ku0temp_dV_p_GND)/ku0temp/ku0temp);
d_rho_ref_dTemp_t_GND = ((ku0temp*d_T7_dTemp_t_GND-T7*d_ku0temp_dTemp_t_GND)/ku0temp/ku0temp);
rho_ref = (T7/ku0temp);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = (B4SOIku0*B4SOIInv_ODeff);

d_rho_dV_b_GND = ((ku0temp*d_T4_dV_b_GND-T4*d_ku0temp_dV_b_GND)/ku0temp/ku0temp);
d_rho_dV_p_GND = ((ku0temp*d_T4_dV_p_GND-T4*d_ku0temp_dV_p_GND)/ku0temp/ku0temp);
d_rho_dTemp_t_GND = ((ku0temp*d_T4_dTemp_t_GND-T4*d_ku0temp_dTemp_t_GND)/ku0temp/ku0temp);
rho = (T4/ku0temp);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = d_rho_dV_b_GND;
d_T2_dV_p_GND = d_rho_dV_p_GND;
d_T2_dTemp_t_GND = d_rho_dTemp_t_GND;
T2 = (1.0+rho);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = d_rho_ref_dV_b_GND;
d_T7_dV_p_GND = d_rho_ref_dV_p_GND;
d_T7_dTemp_t_GND = d_rho_ref_dTemp_t_GND;
T7 = (1.0+rho_ref);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = ((T7*d_T2_dV_b_GND-T2*d_T7_dV_b_GND)/T7/T7);
d_T0_dV_p_GND = ((T7*d_T2_dV_p_GND-T2*d_T7_dV_p_GND)/T7/T7);
d_T0_dTemp_t_GND = ((T7*d_T2_dTemp_t_GND-T2*d_T7_dTemp_t_GND)/T7/T7);
T0 = (T2/T7);

d_u0temp_dV_b_GND = (u0temp*d_T0_dV_b_GND+d_u0temp_dV_b_GND*T0);
d_u0temp_dV_p_GND = (u0temp*d_T0_dV_p_GND+d_u0temp_dV_p_GND*T0);
d_u0temp_dTemp_t_GND = (u0temp*d_T0_dTemp_t_GND+d_u0temp_dTemp_t_GND*T0);
u0temp = (u0temp*T0);

d_vsattemp_dV_b_GND = (-pParam_B4SOIat*d_T3_dV_b_GND);
d_vsattemp_dV_p_GND = (-pParam_B4SOIat*d_T3_dV_p_GND);
d_vsattemp_dTemp_t_GND = (-pParam_B4SOIat*d_T3_dTemp_t_GND);
vsattemp = (pParam_B4SOIvsat-(pParam_B4SOIat*T3));

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = B4SOIkvsat*d_rho_dV_b_GND;
d_T2_dV_p_GND = B4SOIkvsat*d_rho_dV_p_GND;
d_T2_dTemp_t_GND = B4SOIkvsat*d_rho_dTemp_t_GND;
T2 = (1.0+(B4SOIkvsat*rho));

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = B4SOIkvsat*d_rho_ref_dV_b_GND;
d_T7_dV_p_GND = B4SOIkvsat*d_rho_ref_dV_p_GND;
d_T7_dTemp_t_GND = B4SOIkvsat*d_rho_ref_dTemp_t_GND;
T7 = (1.0+(B4SOIkvsat*rho_ref));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = ((T7*d_T2_dV_b_GND-T2*d_T7_dV_b_GND)/T7/T7);
d_T0_dV_p_GND = ((T7*d_T2_dV_p_GND-T2*d_T7_dV_p_GND)/T7/T7);
d_T0_dTemp_t_GND = ((T7*d_T2_dTemp_t_GND-T2*d_T7_dTemp_t_GND)/T7/T7);
T0 = (T2/T7);

d_vsattemp_dV_b_GND = (vsattemp*d_T0_dV_b_GND+d_vsattemp_dV_b_GND*T0);
d_vsattemp_dV_p_GND = (vsattemp*d_T0_dV_p_GND+d_vsattemp_dV_p_GND*T0);
d_vsattemp_dTemp_t_GND = (vsattemp*d_T0_dTemp_t_GND+d_vsattemp_dTemp_t_GND*T0);
vsattemp = (vsattemp*T0);
if ((B4SOIrdsMod!=1))
{

d_rds0_dV_b_GND = (pParam_B4SOIprt*d_T3_dV_b_GND/pParam_B4SOIrds0denom);
d_rds0_dV_p_GND = (pParam_B4SOIprt*d_T3_dV_p_GND/pParam_B4SOIrds0denom);
d_rds0_dTemp_t_GND = (pParam_B4SOIprt*d_T3_dTemp_t_GND/pParam_B4SOIrds0denom);
rds0 = ((pParam_B4SOIrdsw+(pParam_B4SOIprt*T3))/pParam_B4SOIrds0denom);

d_rd0_dTemp_t_GND = d_rd0_dV_p_GND = d_rd0_dV_b_GND =  0.0;
rd0 = 0;

d_rs0_dTemp_t_GND = d_rs0_dV_p_GND = d_rs0_dV_b_GND =  0.0;
rs0 = 0;
}
else
{

d_rds0_dTemp_t_GND = d_rds0_dV_p_GND = d_rds0_dV_b_GND =  0.0;
rds0 = 0;
PowWeffWr = (pParam_B4SOIrds0denom*B4SOInf);

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si =  0.0;
d_T10_dV_b_GND = pParam_B4SOIprt*d_T3_dV_b_GND;
d_T10_dV_p_GND = pParam_B4SOIprt*d_T3_dV_p_GND;
d_T10_dTemp_t_GND = pParam_B4SOIprt*d_T3_dTemp_t_GND;
T10 = (pParam_B4SOIprt*T3);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = d_T10_dV_b_GND;
d_T1_dV_p_GND = d_T10_dV_p_GND;
d_T1_dTemp_t_GND = d_T10_dTemp_t_GND;
T1 = (pParam_B4SOIrdw+T10);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = d_T10_dV_b_GND;
d_T2_dV_p_GND = d_T10_dV_p_GND;
d_T2_dTemp_t_GND = d_T10_dTemp_t_GND;
T2 = (B4SOIrdwmin+T10);

d_rd0_dV_b_GND = (d_T1_dV_b_GND/PowWeffWr);
d_rd0_dV_p_GND = (d_T1_dV_p_GND/PowWeffWr);
d_rd0_dTemp_t_GND = (d_T1_dTemp_t_GND/PowWeffWr);
rd0 = (T1/PowWeffWr);

d_rdwmin_dV_b_GND = (d_T2_dV_b_GND/PowWeffWr);
d_rdwmin_dV_p_GND = (d_T2_dV_p_GND/PowWeffWr);
d_rdwmin_dTemp_t_GND = (d_T2_dTemp_t_GND/PowWeffWr);
rdwmin = (T2/PowWeffWr);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_di_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_b_GND = d_T10_dV_b_GND;
d_T7_dV_p_GND = d_T10_dV_p_GND;
d_T7_dTemp_t_GND = d_T10_dTemp_t_GND;
T7 = (pParam_B4SOIrsw+T10);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si =  0.0;
d_T4_dV_b_GND = d_T10_dV_b_GND;
d_T4_dV_p_GND = d_T10_dV_p_GND;
d_T4_dTemp_t_GND = d_T10_dTemp_t_GND;
T4 = (B4SOIrswmin+T10);

d_rs0_dV_b_GND = (d_T7_dV_b_GND/PowWeffWr);
d_rs0_dV_p_GND = (d_T7_dV_p_GND/PowWeffWr);
d_rs0_dTemp_t_GND = (d_T7_dTemp_t_GND/PowWeffWr);
rs0 = (T7/PowWeffWr);

d_rswmin_dV_b_GND = (d_T4_dV_b_GND/PowWeffWr);
d_rswmin_dV_p_GND = (d_T4_dV_p_GND/PowWeffWr);
d_rswmin_dTemp_t_GND = (d_T4_dTemp_t_GND/PowWeffWr);
rswmin = (T4/PowWeffWr);
}

d_ua_dV_b_GND = pParam_B4SOIua1*d_T3_dV_b_GND;
d_ua_dV_p_GND = pParam_B4SOIua1*d_T3_dV_p_GND;
d_ua_dTemp_t_GND = pParam_B4SOIua1*d_T3_dTemp_t_GND;
ua = (pParam_B4SOIuatemp+(pParam_B4SOIua1*T3));

d_ub_dV_b_GND = pParam_B4SOIub1*d_T3_dV_b_GND;
d_ub_dV_p_GND = pParam_B4SOIub1*d_T3_dV_p_GND;
d_ub_dTemp_t_GND = pParam_B4SOIub1*d_T3_dTemp_t_GND;
ub = (pParam_B4SOIubtemp+(pParam_B4SOIub1*T3));

d_uc_dV_b_GND = pParam_B4SOIuc1*d_T3_dV_b_GND;
d_uc_dV_p_GND = pParam_B4SOIuc1*d_T3_dV_p_GND;
d_uc_dTemp_t_GND = pParam_B4SOIuc1*d_T3_dTemp_t_GND;
uc = (pParam_B4SOIuctemp+(pParam_B4SOIuc1*T3));
}
else
{

d_vbi_dTemp_t_GND = d_vbi_dV_p_GND = d_vbi_dV_b_GND =  0.0;
vbi = pParam_B4SOIvbi;

d_vfbb_dTemp_t_GND = d_vfbb_dV_p_GND = d_vfbb_dV_b_GND =  0.0;
vfbb = pParam_B4SOIvfbb;

d_phi_dTemp_t_GND = d_phi_dV_p_GND = d_phi_dV_b_GND =  0.0;
phi = pParam_B4SOIphi;

d_sqrtPhi_dTemp_t_GND = d_sqrtPhi_dV_p_GND = d_sqrtPhi_dV_b_GND =  0.0;
sqrtPhi = pParam_B4SOIsqrtPhi;

d_Xdep0_dTemp_t_GND = d_Xdep0_dV_p_GND = d_Xdep0_dV_b_GND =  0.0;
Xdep0 = pParam_B4SOIXdep0;

d_Eg_dTemp_t_GND = d_Eg_dV_p_GND = d_Eg_dV_b_GND =  0.0;
Eg = B4SOIeg;

d_cdep0_dTemp_t_GND = d_cdep0_dV_p_GND = d_cdep0_dV_b_GND =  0.0;
cdep0 = pParam_B4SOIcdep0;

d_theta0vb0_dTemp_t_GND = d_theta0vb0_dV_p_GND = d_theta0vb0_dV_b_GND =  0.0;
theta0vb0 = pParam_B4SOItheta0vb0;

d_thetaRout_dTemp_t_GND = d_thetaRout_dV_p_GND = d_thetaRout_dV_b_GND =  0.0;
thetaRout = pParam_B4SOIthetaRout;

d_jbjts_dTemp_t_GND = d_jbjts_dV_p_GND = d_jbjts_dV_b_GND =  0.0;
jbjts = pParam_B4SOIjbjts;

d_jbjtd_dTemp_t_GND = d_jbjtd_dV_p_GND = d_jbjtd_dV_b_GND =  0.0;
jbjtd = pParam_B4SOIjbjtd;

d_jdifs_dTemp_t_GND = d_jdifs_dV_p_GND = d_jdifs_dV_b_GND =  0.0;
jdifs = pParam_B4SOIjdifs;

d_jdifd_dTemp_t_GND = d_jdifd_dV_p_GND = d_jdifd_dV_b_GND =  0.0;
jdifd = pParam_B4SOIjdifd;

d_jrecs_dTemp_t_GND = d_jrecs_dV_p_GND = d_jrecs_dV_b_GND =  0.0;
jrecs = pParam_B4SOIjrecs;

d_jrecd_dTemp_t_GND = d_jrecd_dV_p_GND = d_jrecd_dV_b_GND =  0.0;
jrecd = pParam_B4SOIjrecd;

d_jtuns_dTemp_t_GND = d_jtuns_dV_p_GND = d_jtuns_dV_b_GND =  0.0;
jtuns = pParam_B4SOIjtuns;

d_jtund_dTemp_t_GND = d_jtund_dV_p_GND = d_jtund_dV_b_GND =  0.0;
jtund = pParam_B4SOIjtund;

d_Ahlis_dTemp_t_GND = d_Ahlis_dV_p_GND = d_Ahlis_dV_b_GND =  0.0;
Ahlis = pParam_B4SOIahli0s;

d_Ahlid_dTemp_t_GND = d_Ahlid_dV_p_GND = d_Ahlid_dV_b_GND =  0.0;
Ahlid = pParam_B4SOIahli0d;

d_u0temp_dTemp_t_GND = d_u0temp_dV_p_GND = d_u0temp_dV_b_GND =  0.0;
u0temp = here_B4SOIu0temp;

d_vsattemp_dTemp_t_GND = d_vsattemp_dV_p_GND = d_vsattemp_dV_b_GND =  0.0;
vsattemp = here_B4SOIvsattemp;

d_ua_dTemp_t_GND = d_ua_dV_p_GND = d_ua_dV_b_GND =  0.0;
ua = pParam_B4SOIua;

d_ub_dTemp_t_GND = d_ub_dV_p_GND = d_ub_dV_b_GND =  0.0;
ub = pParam_B4SOIub;

d_uc_dTemp_t_GND = d_uc_dV_p_GND = d_uc_dV_b_GND =  0.0;
uc = pParam_B4SOIuc;
}
if (((model_.given("K1"))||(model_.given("K2"))))
{
if ((!(model_.given("K1"))))
{
UserInfo(*this)  << "Warning: k1 should be specified with k2." << " " <<  std::endl; 

d_pParam_B4SOIk1_dTemp_t_GND = d_pParam_B4SOIk1_dV_p_GND = d_pParam_B4SOIk1_dV_b_GND =  0.0;
pParam_B4SOIk1 = 0.53;
}
if ((!(model_.given("K2"))))
{
UserInfo(*this)  << "Warning: k2 should be specified with k1." << " " <<  std::endl; 
pParam_B4SOIk2 = (-0.0186);
}
if ((model_.given("XT")))
{
UserInfo(*this)  << "Warning: xt is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("VBX")))
{
UserInfo(*this)  << "Warning: vbx is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("VBM")))
{
UserInfo(*this)  << "Warning: vbm is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("GAMMA1")))
{
UserInfo(*this)  << "Warning: gamma1 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if ((model_.given("GAMMA2")))
{
UserInfo(*this)  << "Warning: gamma2 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
}
else
{
if ((!(model_.given("VBX"))))
{
if (B4SOImtrlMod)
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = ((1.60219e-19/(2.0*epssub))*1.0e6);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 7.7348e-4;
}

d_pParam_B4SOIvbx_dV_b_GND = (d_phi_dV_b_GND-d_T0_dV_b_GND*pParam_B4SOInpeak*pParam_B4SOIxt*pParam_B4SOIxt);
d_pParam_B4SOIvbx_dV_p_GND = (d_phi_dV_p_GND-d_T0_dV_p_GND*pParam_B4SOInpeak*pParam_B4SOIxt*pParam_B4SOIxt);
d_pParam_B4SOIvbx_dTemp_t_GND = (d_phi_dTemp_t_GND-d_T0_dTemp_t_GND*pParam_B4SOInpeak*pParam_B4SOIxt*pParam_B4SOIxt);
pParam_B4SOIvbx = (phi-(((T0*pParam_B4SOInpeak)*pParam_B4SOIxt)*pParam_B4SOIxt));
}
if ((pParam_B4SOIvbx>0.0))
{

d_pParam_B4SOIvbx_dV_b_GND = (-d_pParam_B4SOIvbx_dV_b_GND);
d_pParam_B4SOIvbx_dV_p_GND = (-d_pParam_B4SOIvbx_dV_p_GND);
d_pParam_B4SOIvbx_dTemp_t_GND = (-d_pParam_B4SOIvbx_dTemp_t_GND);
pParam_B4SOIvbx = (-pParam_B4SOIvbx);
}
if ((pParam_B4SOIvbm>0.0))
{
pParam_B4SOIvbm = (-pParam_B4SOIvbm);
}
if ((!(model_.given("GAMMA1"))))
{
{
double value_sqrt_0 = sqrt(pParam_B4SOInpeak);
pParam_B4SOIgamma1 = ((sqrt2qeps*value_sqrt_0)/B4SOIcox);
}
}
if ((!(model_.given("GAMMA2"))))
{
{
double value_sqrt_0 = sqrt(pParam_B4SOInsub);
pParam_B4SOIgamma2 = ((sqrt2qeps*value_sqrt_0)/B4SOIcox);
}
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (pParam_B4SOIgamma1-pParam_B4SOIgamma2);
{
double value_sqrt_0 = sqrt((phi-pParam_B4SOIvbx));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = ((deriv_sqrt_0_d0*((d_phi_dV_b_GND-d_pParam_B4SOIvbx_dV_b_GND)))-d_sqrtPhi_dV_b_GND);
d_T1_dV_p_GND = ((deriv_sqrt_0_d0*((d_phi_dV_p_GND-d_pParam_B4SOIvbx_dV_p_GND)))-d_sqrtPhi_dV_p_GND);
d_T1_dTemp_t_GND = ((deriv_sqrt_0_d0*((d_phi_dTemp_t_GND-d_pParam_B4SOIvbx_dTemp_t_GND)))-d_sqrtPhi_dTemp_t_GND);
T1 = (value_sqrt_0-sqrtPhi);
}
{
double value_sqrt_0 = sqrt((phi-pParam_B4SOIvbm));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = (sqrtPhi*((deriv_sqrt_0_d0*(d_phi_dV_b_GND))-d_sqrtPhi_dV_b_GND)+d_sqrtPhi_dV_b_GND*(value_sqrt_0-sqrtPhi));
d_T2_dV_p_GND = (sqrtPhi*((deriv_sqrt_0_d0*(d_phi_dV_p_GND))-d_sqrtPhi_dV_p_GND)+d_sqrtPhi_dV_p_GND*(value_sqrt_0-sqrtPhi));
d_T2_dTemp_t_GND = (sqrtPhi*((deriv_sqrt_0_d0*(d_phi_dTemp_t_GND))-d_sqrtPhi_dTemp_t_GND)+d_sqrtPhi_dTemp_t_GND*(value_sqrt_0-sqrtPhi));
T2 = (sqrtPhi*(value_sqrt_0-sqrtPhi));
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = ((((2.0*T2)+pParam_B4SOIvbm)*(T0*d_T1_dV_b_GND+d_T0_dV_b_GND*T1)-(T0*T1)*2.0*d_T2_dV_b_GND)/((2.0*T2)+pParam_B4SOIvbm)/((2.0*T2)+pParam_B4SOIvbm));
d_T3_dV_p_GND = ((((2.0*T2)+pParam_B4SOIvbm)*(T0*d_T1_dV_p_GND+d_T0_dV_p_GND*T1)-(T0*T1)*2.0*d_T2_dV_p_GND)/((2.0*T2)+pParam_B4SOIvbm)/((2.0*T2)+pParam_B4SOIvbm));
d_T3_dTemp_t_GND = ((((2.0*T2)+pParam_B4SOIvbm)*(T0*d_T1_dTemp_t_GND+d_T0_dTemp_t_GND*T1)-(T0*T1)*2.0*d_T2_dTemp_t_GND)/((2.0*T2)+pParam_B4SOIvbm)/((2.0*T2)+pParam_B4SOIvbm));
T3 = ((T0*T1)/((2.0*T2)+pParam_B4SOIvbm));

d_here_B4SOIk2_dV_b_GND = (d_here_B4SOIk2_dV_b_GND+d_T3_dV_b_GND);
d_here_B4SOIk2_dV_p_GND = (d_here_B4SOIk2_dV_p_GND+d_T3_dV_p_GND);
d_here_B4SOIk2_dTemp_t_GND = (d_here_B4SOIk2_dTemp_t_GND+d_T3_dTemp_t_GND);
here_B4SOIk2 = ((here_B4SOIk2-pParam_B4SOIk2)+T3);
{
double value_sqrt_0 = sqrt((phi-pParam_B4SOIvbm));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_pParam_B4SOIk1_dV_b_GND = (-((2.0*here_B4SOIk2)*(deriv_sqrt_0_d0*(d_phi_dV_b_GND))+2.0*d_here_B4SOIk2_dV_b_GND*value_sqrt_0));
d_pParam_B4SOIk1_dV_p_GND = (-((2.0*here_B4SOIk2)*(deriv_sqrt_0_d0*(d_phi_dV_p_GND))+2.0*d_here_B4SOIk2_dV_p_GND*value_sqrt_0));
d_pParam_B4SOIk1_dTemp_t_GND = (-((2.0*here_B4SOIk2)*(deriv_sqrt_0_d0*(d_phi_dTemp_t_GND))+2.0*d_here_B4SOIk2_dTemp_t_GND*value_sqrt_0));
pParam_B4SOIk1 = (pParam_B4SOIgamma2-((2.0*here_B4SOIk2)*value_sqrt_0));
}
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (pParam_B4SOIweff+pParam_B4SOIk1w2);
if ((T0<1e-8))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 1e-8;
}

d_pParam_B4SOIk1eff_dV_b_GND = (pParam_B4SOIk1*(-pParam_B4SOIk1w1*d_T0_dV_b_GND/T0/T0)+d_pParam_B4SOIk1_dV_b_GND*(1+(pParam_B4SOIk1w1/T0)));
d_pParam_B4SOIk1eff_dV_p_GND = (pParam_B4SOIk1*(-pParam_B4SOIk1w1*d_T0_dV_p_GND/T0/T0)+d_pParam_B4SOIk1_dV_p_GND*(1+(pParam_B4SOIk1w1/T0)));
d_pParam_B4SOIk1eff_dTemp_t_GND = (pParam_B4SOIk1*(-pParam_B4SOIk1w1*d_T0_dTemp_t_GND/T0/T0)+d_pParam_B4SOIk1_dTemp_t_GND*(1+(pParam_B4SOIk1w1/T0)));
pParam_B4SOIk1eff = (pParam_B4SOIk1*(1+(pParam_B4SOIk1w1/T0)));
if ((!(model_.given("VFB"))))
{
if (((model_.given("VTH0"))||(model_.given("VTHO"))))
{

d_here_B4SOIvfb_dV_b_GND = (((d_here_B4SOIvfb_dV_b_GND+B4SOItype*d_here_B4SOIvth0_dV_b_GND)-d_phi_dV_b_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi));
d_here_B4SOIvfb_dV_p_GND = (((d_here_B4SOIvfb_dV_p_GND+B4SOItype*d_here_B4SOIvth0_dV_p_GND)-d_phi_dV_p_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi));
d_here_B4SOIvfb_dTemp_t_GND = (((d_here_B4SOIvfb_dTemp_t_GND+B4SOItype*d_here_B4SOIvth0_dTemp_t_GND)-d_phi_dTemp_t_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi));
here_B4SOIvfb = ((((here_B4SOIvfb-pParam_B4SOIvfb)+(B4SOItype*here_B4SOIvth0))-phi)-(pParam_B4SOIk1eff*sqrtPhi));
}
else
{

}
}
if ((!(model_.given("VTH0"))))
{

d_here_B4SOIvth0_dV_b_GND = B4SOItype*((d_here_B4SOIvfb_dV_b_GND+d_phi_dV_b_GND)+(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi));
d_here_B4SOIvth0_dV_p_GND = B4SOItype*((d_here_B4SOIvfb_dV_p_GND+d_phi_dV_p_GND)+(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi));
d_here_B4SOIvth0_dTemp_t_GND = B4SOItype*((d_here_B4SOIvfb_dTemp_t_GND+d_phi_dTemp_t_GND)+(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi));
here_B4SOIvth0 = (B4SOItype*((here_B4SOIvfb+phi)+(pParam_B4SOIk1eff*sqrtPhi)));
}
if (((model_.VERSION)<4.2))
{

d_rd0_dTemp_t_GND = d_rd0_dV_p_GND = d_rd0_dV_b_GND =  0.0;
rd0 = pParam_B4SOIrd0;

d_rdwmin_dTemp_t_GND = d_rdwmin_dV_p_GND = d_rdwmin_dV_b_GND =  0.0;
rdwmin = pParam_B4SOIrdwmin;

d_cdep0_dTemp_t_GND = d_cdep0_dV_p_GND = d_cdep0_dV_b_GND =  0.0;
cdep0 = pParam_B4SOIcdep0;

d_theta0vb0_dTemp_t_GND = d_theta0vb0_dV_p_GND = d_theta0vb0_dV_b_GND =  0.0;
theta0vb0 = pParam_B4SOItheta0vb0;

d_thetaRout_dTemp_t_GND = d_thetaRout_dV_p_GND = d_thetaRout_dV_b_GND =  0.0;
thetaRout = pParam_B4SOIthetaRout;
if ((B4SOImobMod==4))
{

d_ua_dTemp_t_GND = d_ua_dV_p_GND = d_ua_dV_b_GND =  0.0;
ua = pParam_B4SOIua;

d_uc_dTemp_t_GND = d_uc_dV_p_GND = d_uc_dV_b_GND =  0.0;
uc = pParam_B4SOIuc;
}
}

d_vds_dV_di_si = B4SOItype*d_probeVars[admsProbeID_V_di_si][admsProbeID_V_di_si];
vds = (B4SOItype*(probeVars[admsProbeID_V_di_si]));

d_vbs_dV_b_si = B4SOItype*d_probeVars[admsProbeID_V_b_si][admsProbeID_V_b_si];
vbs = (B4SOItype*(probeVars[admsProbeID_V_b_si]));

d_vgs_dV_gi_si = B4SOItype*d_probeVars[admsProbeID_V_gi_si][admsProbeID_V_gi_si];
vgs = (B4SOItype*(probeVars[admsProbeID_V_gi_si]));

d_ves_dV_e_si = B4SOItype*d_probeVars[admsProbeID_V_e_si][admsProbeID_V_e_si];
ves = (B4SOItype*(probeVars[admsProbeID_V_e_si]));

d_vbp_dV_b_p = B4SOItype*d_probeVars[admsProbeID_V_b_p][admsProbeID_V_b_p];
vbp = (B4SOItype*(probeVars[admsProbeID_V_b_p]));

d_vgp_dV_gi_p = B4SOItype*d_probeVars[admsProbeID_V_gi_p][admsProbeID_V_gi_p];
vgp = (B4SOItype*(probeVars[admsProbeID_V_gi_p]));

d_vsbs_dV_sb_si = B4SOItype*d_probeVars[admsProbeID_V_sb_si][admsProbeID_V_sb_si];
vsbs = (B4SOItype*(probeVars[admsProbeID_V_sb_si]));

d_vdbd_dV_db_di = B4SOItype*d_probeVars[admsProbeID_V_db_di][admsProbeID_V_db_di];
vdbd = (B4SOItype*(probeVars[admsProbeID_V_db_di]));

d_vgms_dV_gm_si = B4SOItype*d_probeVars[admsProbeID_V_gm_si][admsProbeID_V_gm_si];
vgms = (B4SOItype*(probeVars[admsProbeID_V_gm_si]));

d_vbd_dV_di_si = (-d_vds_dV_di_si);
d_vbd_dV_b_si = d_vbs_dV_b_si;
vbd = (vbs-vds);

d_vgd_dV_di_si = (-d_vds_dV_di_si);
d_vgd_dV_gi_si = d_vgs_dV_gi_si;
vgd = (vgs-vds);

d_ved_dV_di_si = (-d_vds_dV_di_si);
d_ved_dV_e_si = d_ves_dV_e_si;
ved = (ves-vds);

d_vgmd_dV_di_si = (-d_vds_dV_di_si);
d_vgmd_dV_gm_si = d_vgms_dV_gm_si;
vgmd = (vgms-vds);
if ((vds>=0.0))
{
B4SOImode = 1;

d_Vds_dV_di_si = d_vds_dV_di_si;
Vds = vds;

d_Vgs_dV_di_si =  0.0;
d_Vgs_dV_gi_si = d_vgs_dV_gi_si;
Vgs = vgs;

d_Vbs_dV_gi_si = d_Vbs_dTemp_t_GND = d_Vbs_dV_p_GND = d_Vbs_dV_b_GND = d_Vbs_dV_e_si = d_Vbs_dV_di_si =  0.0;
d_Vbs_dV_b_si = d_vbs_dV_b_si;
Vbs = vbs;

d_Vbd_dV_di_si = d_vbd_dV_di_si;
d_Vbd_dV_b_si = d_vbd_dV_b_si;
Vbd = vbd;

d_Ves_dV_di_si =  0.0;
d_Ves_dV_e_si = d_ves_dV_e_si;
Ves = ves;

d_Vgd_dV_di_si = d_vgd_dV_di_si;
d_Vgd_dV_gi_si = d_vgd_dV_gi_si;
Vgd = vgd;
wdios = pParam_B4SOIwdios;
wdiod = pParam_B4SOIwdiod;
agidl = pParam_B4SOIagidl;
bgidl = pParam_B4SOIbgidl;
cgidl = pParam_B4SOIcgidl;
egidl = pParam_B4SOIegidl;
rgidl = pParam_B4SOIrgidl;
kgidl = pParam_B4SOIkgidl;
fgidl = pParam_B4SOIfgidl;
agisl = pParam_B4SOIagisl;
bgisl = pParam_B4SOIbgisl;
cgisl = pParam_B4SOIcgisl;
egisl = pParam_B4SOIegisl;
rgisl = pParam_B4SOIrgisl;
kgisl = pParam_B4SOIkgisl;
fgisl = pParam_B4SOIfgisl;
}
else
{
B4SOImode = (-1);

d_Vds_dV_di_si = (-d_vds_dV_di_si);
Vds = (-vds);

d_Vgs_dV_di_si = d_vgd_dV_di_si;
d_Vgs_dV_gi_si = d_vgd_dV_gi_si;
Vgs = vgd;

d_Vbs_dV_gi_si = d_Vbs_dTemp_t_GND = d_Vbs_dV_p_GND = d_Vbs_dV_b_GND = d_Vbs_dV_e_si =  0.0;
d_Vbs_dV_di_si = d_vbd_dV_di_si;
d_Vbs_dV_b_si = d_vbd_dV_b_si;
Vbs = vbd;

d_Vbd_dV_di_si =  0.0;
d_Vbd_dV_b_si = d_vbs_dV_b_si;
Vbd = vbs;

d_Ves_dV_di_si = d_ved_dV_di_si;
d_Ves_dV_e_si = d_ved_dV_e_si;
Ves = ved;

d_Vgd_dV_di_si =  0.0;
d_Vgd_dV_gi_si = d_vgs_dV_gi_si;
Vgd = vgs;
wdios = pParam_B4SOIwdiod;
wdiod = pParam_B4SOIwdios;
agidl = pParam_B4SOIagisl;
bgidl = pParam_B4SOIbgisl;
cgidl = pParam_B4SOIcgisl;
egidl = pParam_B4SOIegisl;
rgidl = pParam_B4SOIrgisl;
kgidl = pParam_B4SOIkgisl;
fgidl = pParam_B4SOIfgisl;
agisl = pParam_B4SOIagidl;
bgisl = pParam_B4SOIbgidl;
cgisl = pParam_B4SOIcgidl;
egisl = pParam_B4SOIegidl;
rgisl = pParam_B4SOIrgidl;
kgisl = pParam_B4SOIkgidl;
fgisl = pParam_B4SOIfgidl;
}

d_Vesfb_dV_b_GND = (-d_vfbb_dV_b_GND);
d_Vesfb_dV_p_GND = (-d_vfbb_dV_p_GND);
d_Vesfb_dTemp_t_GND = (-d_vfbb_dTemp_t_GND);
d_Vesfb_dV_e_si = d_Ves_dV_e_si;
d_Vesfb_dV_di_si = d_Ves_dV_di_si;
Vesfb = (Ves-vfbb);
Cbox = B4SOIcbox;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (d_here_B4SOIvfb_dV_b_GND+d_phi_dV_b_GND);
d_T0_dV_p_GND = (d_here_B4SOIvfb_dV_p_GND+d_phi_dV_p_GND);
d_T0_dTemp_t_GND = (d_here_B4SOIvfb_dTemp_t_GND+d_phi_dTemp_t_GND);
T0 = (here_B4SOIvfb+phi);
if ((B4SOImtrlMod==0))
{
epsgate = epssub;
}
else
{
epsgate = (B4SOIepsrgate*8.85418e-12);
}
if (((((pParam_B4SOIngate>1.0e18)&&(pParam_B4SOIngate<1.0e25))&&(Vgs>T0))&&(epsgate!=0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = ((((1.0e6*1.60219e-19)*epsgate)*pParam_B4SOIngate)/(B4SOIcox*B4SOIcox));
{
double value_sqrt_0 = sqrt((1.0+((2.0*(Vgs-T0))/T1)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si =  0.0;
d_T4_dV_b_GND = (deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dV_b_GND)-(2.0*(Vgs-T0))*d_T1_dV_b_GND)/T1/T1)));
d_T4_dV_p_GND = (deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dV_p_GND)-(2.0*(Vgs-T0))*d_T1_dV_p_GND)/T1/T1)));
d_T4_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dTemp_t_GND)-(2.0*(Vgs-T0))*d_T1_dTemp_t_GND)/T1/T1)));
d_T4_dV_gi_si = (deriv_sqrt_0_d0*(((T1*2.0*(d_Vgs_dV_gi_si-d_T0_dV_gi_si)-(2.0*(Vgs-T0))*d_T1_dV_gi_si)/T1/T1)));
d_T4_dV_di_si = (deriv_sqrt_0_d0*(((T1*2.0*(d_Vgs_dV_di_si-d_T0_dV_di_si)-(2.0*(Vgs-T0))*d_T1_dV_di_si)/T1/T1)));
T4 = value_sqrt_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si =  0.0;
d_T2_dV_gi_si = (T1*d_T4_dV_gi_si+d_T1_dV_gi_si*(T4-1.0));
d_T2_dV_di_si = (T1*d_T4_dV_di_si+d_T1_dV_di_si*(T4-1.0));
d_T2_dV_b_GND = (T1*d_T4_dV_b_GND+d_T1_dV_b_GND*(T4-1.0));
d_T2_dV_p_GND = (T1*d_T4_dV_p_GND+d_T1_dV_p_GND*(T4-1.0));
d_T2_dTemp_t_GND = (T1*d_T4_dTemp_t_GND+d_T1_dTemp_t_GND*(T4-1.0));
T2 = (T1*(T4-1.0));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si =  0.0;
d_T3_dV_b_GND = ((T1*((0.5*T2)*d_T2_dV_b_GND+0.5*d_T2_dV_b_GND*T2)-((0.5*T2)*T2)*d_T1_dV_b_GND)/T1/T1);
d_T3_dV_p_GND = ((T1*((0.5*T2)*d_T2_dV_p_GND+0.5*d_T2_dV_p_GND*T2)-((0.5*T2)*T2)*d_T1_dV_p_GND)/T1/T1);
d_T3_dTemp_t_GND = ((T1*((0.5*T2)*d_T2_dTemp_t_GND+0.5*d_T2_dTemp_t_GND*T2)-((0.5*T2)*T2)*d_T1_dTemp_t_GND)/T1/T1);
d_T3_dV_gi_si = ((T1*((0.5*T2)*d_T2_dV_gi_si+0.5*d_T2_dV_gi_si*T2)-((0.5*T2)*T2)*d_T1_dV_gi_si)/T1/T1);
d_T3_dV_di_si = ((T1*((0.5*T2)*d_T2_dV_di_si+0.5*d_T2_dV_di_si*T2)-((0.5*T2)*T2)*d_T1_dV_di_si)/T1/T1);
T3 = (((0.5*T2)*T2)/T1);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si =  0.0;
d_T7_dV_b_GND = (-d_T3_dV_b_GND);
d_T7_dV_p_GND = (-d_T3_dV_p_GND);
d_T7_dTemp_t_GND = (-d_T3_dTemp_t_GND);
d_T7_dV_gi_si = (-d_T3_dV_gi_si);
d_T7_dV_di_si = (-d_T3_dV_di_si);
T7 = ((eggdep-T3)-0.05);
{
double value_sqrt_0 = sqrt(((T7*T7)+0.224));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si =  0.0;
d_T6_dV_b_GND = (deriv_sqrt_0_d0*((T7*d_T7_dV_b_GND+d_T7_dV_b_GND*T7)));
d_T6_dV_p_GND = (deriv_sqrt_0_d0*((T7*d_T7_dV_p_GND+d_T7_dV_p_GND*T7)));
d_T6_dTemp_t_GND = (deriv_sqrt_0_d0*((T7*d_T7_dTemp_t_GND+d_T7_dTemp_t_GND*T7)));
d_T6_dV_gi_si = (deriv_sqrt_0_d0*((T7*d_T7_dV_gi_si+d_T7_dV_gi_si*T7)));
d_T6_dV_di_si = (deriv_sqrt_0_d0*((T7*d_T7_dV_di_si+d_T7_dV_di_si*T7)));
T6 = value_sqrt_0;
}

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si =  0.0;
d_T5_dV_b_GND = (-0.5*(d_T7_dV_b_GND+d_T6_dV_b_GND));
d_T5_dV_p_GND = (-0.5*(d_T7_dV_p_GND+d_T6_dV_p_GND));
d_T5_dTemp_t_GND = (-0.5*(d_T7_dTemp_t_GND+d_T6_dTemp_t_GND));
d_T5_dV_gi_si = (-0.5*(d_T7_dV_gi_si+d_T6_dV_gi_si));
d_T5_dV_di_si = (-0.5*(d_T7_dV_di_si+d_T6_dV_di_si));
T5 = (eggdep-(0.5*(T7+T6)));

d_Vgs_eff_dV_b_GND = (-d_T5_dV_b_GND);
d_Vgs_eff_dV_p_GND = (-d_T5_dV_p_GND);
d_Vgs_eff_dTemp_t_GND = (-d_T5_dTemp_t_GND);
d_Vgs_eff_dV_gi_si = (d_Vgs_dV_gi_si-d_T5_dV_gi_si);
d_Vgs_eff_dV_di_si = (d_Vgs_dV_di_si-d_T5_dV_di_si);
Vgs_eff = (Vgs-T5);
}
else
{

d_Vgs_eff_dTemp_t_GND = d_Vgs_eff_dV_p_GND = d_Vgs_eff_dV_b_GND =  0.0;
d_Vgs_eff_dV_gi_si = d_Vgs_dV_gi_si;
d_Vgs_eff_dV_di_si = d_Vgs_dV_di_si;
Vgs_eff = Vgs;
}
if (((((pParam_B4SOIngate>1.0e18)&&(pParam_B4SOIngate<1.0e25))&&(Vgd>T0))&&(epsgate!=0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = ((((1.0e6*1.60219e-19)*epsgate)*pParam_B4SOIngate)/(B4SOIcox*B4SOIcox));
{
double value_sqrt_0 = sqrt((1.0+((2.0*(Vgd-T0))/T1)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si =  0.0;
d_T4_dV_b_GND = (deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dV_b_GND)-(2.0*(Vgd-T0))*d_T1_dV_b_GND)/T1/T1)));
d_T4_dV_p_GND = (deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dV_p_GND)-(2.0*(Vgd-T0))*d_T1_dV_p_GND)/T1/T1)));
d_T4_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dTemp_t_GND)-(2.0*(Vgd-T0))*d_T1_dTemp_t_GND)/T1/T1)));
d_T4_dV_di_si = (deriv_sqrt_0_d0*(((T1*2.0*(d_Vgd_dV_di_si-d_T0_dV_di_si)-(2.0*(Vgd-T0))*d_T1_dV_di_si)/T1/T1)));
d_T4_dV_gi_si = (deriv_sqrt_0_d0*(((T1*2.0*(d_Vgd_dV_gi_si-d_T0_dV_gi_si)-(2.0*(Vgd-T0))*d_T1_dV_gi_si)/T1/T1)));
T4 = value_sqrt_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si =  0.0;
d_T2_dV_gi_si = (T1*d_T4_dV_gi_si+d_T1_dV_gi_si*(T4-1.0));
d_T2_dV_di_si = (T1*d_T4_dV_di_si+d_T1_dV_di_si*(T4-1.0));
d_T2_dV_b_GND = (T1*d_T4_dV_b_GND+d_T1_dV_b_GND*(T4-1.0));
d_T2_dV_p_GND = (T1*d_T4_dV_p_GND+d_T1_dV_p_GND*(T4-1.0));
d_T2_dTemp_t_GND = (T1*d_T4_dTemp_t_GND+d_T1_dTemp_t_GND*(T4-1.0));
T2 = (T1*(T4-1.0));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si =  0.0;
d_T3_dV_b_GND = ((T1*((0.5*T2)*d_T2_dV_b_GND+0.5*d_T2_dV_b_GND*T2)-((0.5*T2)*T2)*d_T1_dV_b_GND)/T1/T1);
d_T3_dV_p_GND = ((T1*((0.5*T2)*d_T2_dV_p_GND+0.5*d_T2_dV_p_GND*T2)-((0.5*T2)*T2)*d_T1_dV_p_GND)/T1/T1);
d_T3_dTemp_t_GND = ((T1*((0.5*T2)*d_T2_dTemp_t_GND+0.5*d_T2_dTemp_t_GND*T2)-((0.5*T2)*T2)*d_T1_dTemp_t_GND)/T1/T1);
d_T3_dV_gi_si = ((T1*((0.5*T2)*d_T2_dV_gi_si+0.5*d_T2_dV_gi_si*T2)-((0.5*T2)*T2)*d_T1_dV_gi_si)/T1/T1);
d_T3_dV_di_si = ((T1*((0.5*T2)*d_T2_dV_di_si+0.5*d_T2_dV_di_si*T2)-((0.5*T2)*T2)*d_T1_dV_di_si)/T1/T1);
T3 = (((0.5*T2)*T2)/T1);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si =  0.0;
d_T7_dV_b_GND = (-d_T3_dV_b_GND);
d_T7_dV_p_GND = (-d_T3_dV_p_GND);
d_T7_dTemp_t_GND = (-d_T3_dTemp_t_GND);
d_T7_dV_gi_si = (-d_T3_dV_gi_si);
d_T7_dV_di_si = (-d_T3_dV_di_si);
T7 = ((eggdep-T3)-0.05);
{
double value_sqrt_0 = sqrt(((T7*T7)+0.224));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si =  0.0;
d_T6_dV_b_GND = (deriv_sqrt_0_d0*((T7*d_T7_dV_b_GND+d_T7_dV_b_GND*T7)));
d_T6_dV_p_GND = (deriv_sqrt_0_d0*((T7*d_T7_dV_p_GND+d_T7_dV_p_GND*T7)));
d_T6_dTemp_t_GND = (deriv_sqrt_0_d0*((T7*d_T7_dTemp_t_GND+d_T7_dTemp_t_GND*T7)));
d_T6_dV_gi_si = (deriv_sqrt_0_d0*((T7*d_T7_dV_gi_si+d_T7_dV_gi_si*T7)));
d_T6_dV_di_si = (deriv_sqrt_0_d0*((T7*d_T7_dV_di_si+d_T7_dV_di_si*T7)));
T6 = value_sqrt_0;
}

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si =  0.0;
d_T5_dV_b_GND = (-0.5*(d_T7_dV_b_GND+d_T6_dV_b_GND));
d_T5_dV_p_GND = (-0.5*(d_T7_dV_p_GND+d_T6_dV_p_GND));
d_T5_dTemp_t_GND = (-0.5*(d_T7_dTemp_t_GND+d_T6_dTemp_t_GND));
d_T5_dV_gi_si = (-0.5*(d_T7_dV_gi_si+d_T6_dV_gi_si));
d_T5_dV_di_si = (-0.5*(d_T7_dV_di_si+d_T6_dV_di_si));
T5 = (eggdep-(0.5*(T7+T6)));

d_Vgd_eff_dV_b_GND = (-d_T5_dV_b_GND);
d_Vgd_eff_dV_p_GND = (-d_T5_dV_p_GND);
d_Vgd_eff_dTemp_t_GND = (-d_T5_dTemp_t_GND);
d_Vgd_eff_dV_di_si = (d_Vgd_dV_di_si-d_T5_dV_di_si);
d_Vgd_eff_dV_gi_si = (d_Vgd_dV_gi_si-d_T5_dV_gi_si);
Vgd_eff = (Vgd-T5);
}
else
{

d_Vgd_eff_dTemp_t_GND = d_Vgd_eff_dV_p_GND = d_Vgd_eff_dV_b_GND =  0.0;
d_Vgd_eff_dV_di_si = d_Vgd_dV_di_si;
d_Vgd_eff_dV_gi_si = d_Vgd_dV_gi_si;
Vgd_eff = Vgd;
}

d_Vgs_eff2_dV_gi_si = d_Vgs_dV_gi_si;
d_Vgs_eff2_dV_di_si = d_Vgs_dV_di_si;
Vgs_eff2 = Vgs;
Leff = pParam_B4SOIleff;
if (((SHMOD==1)&&(RTH0!=0.0)))
{

d_Vtm_dV_b_GND = 8.617087e-5*d_DevTemp_dV_b_GND;
d_Vtm_dV_p_GND = 8.617087e-5*d_DevTemp_dV_p_GND;
d_Vtm_dTemp_t_GND = 8.617087e-5*d_DevTemp_dTemp_t_GND;
Vtm = (8.617087e-5*DevTemp);
}
else
{

d_Vtm_dV_b_GND = d_B4SOIvtm_dV_b_GND;
d_Vtm_dV_p_GND = d_B4SOIvtm_dV_p_GND;
d_Vtm_dTemp_t_GND = d_B4SOIvtm_dTemp_t_GND;
Vtm = B4SOIvtm;
}

d_V0_dV_b_GND = (d_vbi_dV_b_GND-d_phi_dV_b_GND);
d_V0_dV_p_GND = (d_vbi_dV_p_GND-d_phi_dV_p_GND);
d_V0_dTemp_t_GND = (d_vbi_dTemp_t_GND-d_phi_dTemp_t_GND);
V0 = (vbi-phi);
if ((B4SOIsoiMod==0))
{

d_Vbsmos_dV_gi_si = d_Vbsmos_dTemp_t_GND = d_Vbsmos_dV_p_GND = d_Vbsmos_dV_b_GND = d_Vbsmos_dV_e_si =  0.0;
d_Vbsmos_dV_b_si = d_Vbs_dV_b_si;
d_Vbsmos_dV_di_si = d_Vbs_dV_di_si;
Vbsmos = Vbs;

d_Vbsmos_CV_dV_gi_si = d_Vbsmos_CV_dV_e_si = d_Vbsmos_CV_dTemp_t_GND = d_Vbsmos_CV_dV_p_GND = d_Vbsmos_CV_dV_b_GND =  0.0;
d_Vbsmos_CV_dV_b_si = d_Vbs_dV_b_si;
d_Vbsmos_CV_dV_di_si = d_Vbs_dV_di_si;
Vbsmos_CV = Vbs;
}
else
{
if ((B4SOIfdMod==0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (((-pParam_B4SOIdvbd1)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T0));
double value_exp_1 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_b_GND))+2*(deriv_exp_1_d0*(d_T0_dV_b_GND)));
d_T1_dV_p_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_p_GND))+2*(deriv_exp_1_d0*(d_T0_dV_p_GND)));
d_T1_dTemp_t_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T0_dTemp_t_GND)));
T1 = (pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = (T1*(d_vbi_dV_b_GND-d_phi_dV_b_GND)+d_T1_dV_b_GND*(vbi-phi));
d_T2_dV_p_GND = (T1*(d_vbi_dV_p_GND-d_phi_dV_p_GND)+d_T1_dV_p_GND*(vbi-phi));
d_T2_dTemp_t_GND = (T1*(d_vbi_dTemp_t_GND-d_phi_dTemp_t_GND)+d_T1_dTemp_t_GND*(vbi-phi));
T2 = (T1*(vbi-phi));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = ((0.5*pParam_B4SOIqsi)/B4SOIcsi);

d_Vbs0t_dV_e_si =  0.0;
d_Vbs0t_dV_gi_si = ((-d_T3_dV_gi_si)+d_T2_dV_gi_si);
d_Vbs0t_dV_di_si = ((-d_T3_dV_di_si)+d_T2_dV_di_si);
d_Vbs0t_dV_b_GND = ((d_phi_dV_b_GND-d_T3_dV_b_GND)+d_T2_dV_b_GND);
d_Vbs0t_dV_p_GND = ((d_phi_dV_p_GND-d_T3_dV_p_GND)+d_T2_dV_p_GND);
d_Vbs0t_dTemp_t_GND = ((d_phi_dTemp_t_GND-d_T3_dTemp_t_GND)+d_T2_dTemp_t_GND);
Vbs0t = (((phi-T3)+pParam_B4SOIvbsa)+T2);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1+(B4SOIcsi/Cbox));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = (((-pParam_B4SOIdk2b)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T3));
double value_exp_1 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si =  0.0;
d_T5_dV_b_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_b_GND))+2*(deriv_exp_1_d0*(d_T3_dV_b_GND)));
d_T5_dV_p_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_p_GND))+2*(deriv_exp_1_d0*(d_T3_dV_p_GND)));
d_T5_dTemp_t_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T3_dTemp_t_GND)));
d_T5_dV_gi_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_gi_si))+2*(deriv_exp_1_d0*(d_T3_dV_gi_si)));
d_T5_dV_di_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_di_si))+2*(deriv_exp_1_d0*(d_T3_dV_di_si)));
T5 = (pParam_B4SOIk2b*(value_exp_0+(2*value_exp_1)));
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si =  0.0;
d_T1_dV_b_GND = ((T0*(-d_T5_dV_b_GND)-(pParam_B4SOIk1b-T5)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-d_T5_dV_p_GND)-(pParam_B4SOIk1b-T5)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-d_T5_dTemp_t_GND)-(pParam_B4SOIk1b-T5)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-d_T5_dV_gi_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*(-d_T5_dV_di_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_di_si)/T0/T0);
T1 = ((pParam_B4SOIk1b-T5)/T0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_e_si = (T1*d_Vesfb_dV_e_si+d_T1_dV_e_si*Vesfb);
d_T2_dV_b_GND = (T1*d_Vesfb_dV_b_GND+d_T1_dV_b_GND*Vesfb);
d_T2_dV_p_GND = (T1*d_Vesfb_dV_p_GND+d_T1_dV_p_GND*Vesfb);
d_T2_dTemp_t_GND = (T1*d_Vesfb_dTemp_t_GND+d_T1_dTemp_t_GND*Vesfb);
d_T2_dV_gi_si = d_T1_dV_gi_si*Vesfb;
d_T2_dV_di_si = (T1*d_Vesfb_dV_di_si+d_T1_dV_di_si*Vesfb);
T2 = (T1*Vesfb);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = (1.0/(1+(Cbox/B4SOIcsi)));

d_Vbs0_dV_e_si = ((T4*d_Vbs0t_dV_e_si+d_T4_dV_e_si*Vbs0t)+d_T2_dV_e_si);
d_Vbs0_dV_b_GND = ((T4*d_Vbs0t_dV_b_GND+d_T4_dV_b_GND*Vbs0t)+d_T2_dV_b_GND);
d_Vbs0_dV_p_GND = ((T4*d_Vbs0t_dV_p_GND+d_T4_dV_p_GND*Vbs0t)+d_T2_dV_p_GND);
d_Vbs0_dTemp_t_GND = ((T4*d_Vbs0t_dTemp_t_GND+d_T4_dTemp_t_GND*Vbs0t)+d_T2_dTemp_t_GND);
d_Vbs0_dV_gi_si = ((T4*d_Vbs0t_dV_gi_si+d_T4_dV_gi_si*Vbs0t)+d_T2_dV_gi_si);
d_Vbs0_dV_di_si = ((T4*d_Vbs0t_dV_di_si+d_T4_dV_di_si*Vbs0t)+d_T2_dV_di_si);
Vbs0 = ((T4*Vbs0t)+T2);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0/((B4SOIcsi+Cbox)+pParam_B4SOIcdsbs));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = (((-pParam_B4SOIdvbd1)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T1));
double value_exp_1 = exp(T1);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si =  0.0;
d_T2_dV_b_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_b_GND))+2*(deriv_exp_1_d0*(d_T1_dV_b_GND)));
d_T2_dV_p_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_p_GND))+2*(deriv_exp_1_d0*(d_T1_dV_p_GND)));
d_T2_dTemp_t_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T1_dTemp_t_GND)));
d_T2_dV_gi_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_gi_si))+2*(deriv_exp_1_d0*(d_T1_dV_gi_si)));
d_T2_dV_di_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_di_si))+2*(deriv_exp_1_d0*(d_T1_dV_di_si)));
T2 = (pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_b_GND = d_T2_dV_b_GND*(Vds+pParam_B4SOIvsce);
d_T3_dV_p_GND = d_T2_dV_p_GND*(Vds+pParam_B4SOIvsce);
d_T3_dTemp_t_GND = d_T2_dTemp_t_GND*(Vds+pParam_B4SOIvsce);
d_T3_dV_gi_si = d_T2_dV_gi_si*(Vds+pParam_B4SOIvsce);
d_T3_dV_di_si = (T2*d_Vds_dV_di_si+d_T2_dV_di_si*(Vds+pParam_B4SOIvsce));
d_T3_dV_e_si = d_T2_dV_e_si*(Vds+pParam_B4SOIvsce);
T3 = (T2*(Vds+pParam_B4SOIvsce));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = ((0.5*pParam_B4SOIqsi)/B4SOIcsi);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si =  0.0;
d_T5_dV_gi_si = ((B4SOIcsi*T0)*(-d_T4_dV_gi_si)+B4SOIcsi*d_T0_dV_gi_si*((phi-T4)+pParam_B4SOIvbsa));
d_T5_dV_di_si = ((B4SOIcsi*T0)*(-d_T4_dV_di_si)+B4SOIcsi*d_T0_dV_di_si*((phi-T4)+pParam_B4SOIvbsa));
d_T5_dV_b_GND = ((B4SOIcsi*T0)*(d_phi_dV_b_GND-d_T4_dV_b_GND)+B4SOIcsi*d_T0_dV_b_GND*((phi-T4)+pParam_B4SOIvbsa));
d_T5_dV_p_GND = ((B4SOIcsi*T0)*(d_phi_dV_p_GND-d_T4_dV_p_GND)+B4SOIcsi*d_T0_dV_p_GND*((phi-T4)+pParam_B4SOIvbsa));
d_T5_dTemp_t_GND = ((B4SOIcsi*T0)*(d_phi_dTemp_t_GND-d_T4_dTemp_t_GND)+B4SOIcsi*d_T0_dTemp_t_GND*((phi-T4)+pParam_B4SOIvbsa));
T5 = ((B4SOIcsi*T0)*((phi-T4)+pParam_B4SOIvbsa));

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si =  0.0;
d_T6_dV_gi_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_gi_si+pParam_B4SOIcdsbs*d_T0_dV_gi_si*T3);
d_T6_dV_di_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_di_si+pParam_B4SOIcdsbs*d_T0_dV_di_si*T3);
d_T6_dV_e_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_e_si+pParam_B4SOIcdsbs*d_T0_dV_e_si*T3);
d_T6_dV_b_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_b_GND+pParam_B4SOIcdsbs*d_T0_dV_b_GND*T3);
d_T6_dV_p_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_p_GND+pParam_B4SOIcdsbs*d_T0_dV_p_GND*T3);
d_T6_dTemp_t_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dTemp_t_GND+pParam_B4SOIcdsbs*d_T0_dTemp_t_GND*T3);
T6 = ((pParam_B4SOIcdsbs*T0)*T3);

d_Vbs0t_dV_e_si = (d_T5_dV_e_si+d_T6_dV_e_si);
d_Vbs0t_dV_b_GND = (d_T5_dV_b_GND+d_T6_dV_b_GND);
d_Vbs0t_dV_p_GND = (d_T5_dV_p_GND+d_T6_dV_p_GND);
d_Vbs0t_dTemp_t_GND = (d_T5_dTemp_t_GND+d_T6_dTemp_t_GND);
d_Vbs0t_dV_gi_si = (d_T5_dV_gi_si+d_T6_dV_gi_si);
d_Vbs0t_dV_di_si = (d_T5_dV_di_si+d_T6_dV_di_si);
Vbs0t = (T5+T6);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_gi_si =  0.0;
d_T7_dV_e_si = ((Cbox*T0)*d_Vesfb_dV_e_si+Cbox*d_T0_dV_e_si*Vesfb);
d_T7_dV_di_si = ((Cbox*T0)*d_Vesfb_dV_di_si+Cbox*d_T0_dV_di_si*Vesfb);
d_T7_dV_b_GND = ((Cbox*T0)*d_Vesfb_dV_b_GND+Cbox*d_T0_dV_b_GND*Vesfb);
d_T7_dV_p_GND = ((Cbox*T0)*d_Vesfb_dV_p_GND+Cbox*d_T0_dV_p_GND*Vesfb);
d_T7_dTemp_t_GND = ((Cbox*T0)*d_Vesfb_dTemp_t_GND+Cbox*d_T0_dTemp_t_GND*Vesfb);
T7 = ((Cbox*T0)*Vesfb);

d_Vbs0_dV_gi_si = (d_Vbs0t_dV_gi_si+d_T7_dV_gi_si);
d_Vbs0_dV_di_si = (d_Vbs0t_dV_di_si+d_T7_dV_di_si);
d_Vbs0_dV_b_GND = (d_Vbs0t_dV_b_GND+d_T7_dV_b_GND);
d_Vbs0_dV_p_GND = (d_Vbs0t_dV_p_GND+d_T7_dV_p_GND);
d_Vbs0_dTemp_t_GND = (d_Vbs0t_dTemp_t_GND+d_T7_dTemp_t_GND);
d_Vbs0_dV_e_si = (d_Vbs0t_dV_e_si+d_T7_dV_e_si);
Vbs0 = (Vbs0t+T7);
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_gi_si = (d_Vbs0t_dV_gi_si-d_Vbs0_dV_gi_si);
d_T1_dV_di_si = (d_Vbs0t_dV_di_si-d_Vbs0_dV_di_si);
d_T1_dV_b_GND = (d_Vbs0t_dV_b_GND-d_Vbs0_dV_b_GND);
d_T1_dV_p_GND = (d_Vbs0t_dV_p_GND-d_Vbs0_dV_p_GND);
d_T1_dTemp_t_GND = (d_Vbs0t_dTemp_t_GND-d_Vbs0_dTemp_t_GND);
d_T1_dV_e_si = (d_Vbs0t_dV_e_si-d_Vbs0_dV_e_si);
T1 = ((Vbs0t-Vbs0)-0.005);
{
double value_sqrt_0 = sqrt(((T1*T1)+2.5e-5));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)));
T2 = value_sqrt_0;
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_b_GND = 0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND);
d_T3_dV_p_GND = 0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND);
d_T3_dTemp_t_GND = 0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND);
d_T3_dV_gi_si = 0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si);
d_T3_dV_di_si = 0.5*(d_T1_dV_di_si+d_T2_dV_di_si);
d_T3_dV_e_si = 0.5*(d_T1_dV_e_si+d_T2_dV_e_si);
T3 = (0.5*(T1+T2));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si =  0.0;
d_T4_dV_b_GND = (d_T3_dV_b_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_p_GND = (d_T3_dV_p_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dTemp_t_GND = (d_T3_dTemp_t_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_gi_si = (d_T3_dV_gi_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_di_si = (d_T3_dV_di_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_e_si = (d_T3_dV_e_si*B4SOIcsi/pParam_B4SOIqsi);
T4 = ((T3*B4SOIcsi)/pParam_B4SOIqsi);

d_Vbs0mos_dV_e_si = (d_Vbs0_dV_e_si-((0.5*T3)*d_T4_dV_e_si+0.5*d_T3_dV_e_si*T4));
d_Vbs0mos_dV_b_GND = (d_Vbs0_dV_b_GND-((0.5*T3)*d_T4_dV_b_GND+0.5*d_T3_dV_b_GND*T4));
d_Vbs0mos_dV_p_GND = (d_Vbs0_dV_p_GND-((0.5*T3)*d_T4_dV_p_GND+0.5*d_T3_dV_p_GND*T4));
d_Vbs0mos_dTemp_t_GND = (d_Vbs0_dTemp_t_GND-((0.5*T3)*d_T4_dTemp_t_GND+0.5*d_T3_dTemp_t_GND*T4));
d_Vbs0mos_dV_gi_si = (d_Vbs0_dV_gi_si-((0.5*T3)*d_T4_dV_gi_si+0.5*d_T3_dV_gi_si*T4));
d_Vbs0mos_dV_di_si = (d_Vbs0_dV_di_si-((0.5*T3)*d_T4_dV_di_si+0.5*d_T3_dV_di_si*T4));
Vbs0mos = (Vbs0-((0.5*T3)*T4));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = d_phi_dV_b_GND;
d_T1_dV_p_GND = d_phi_dV_p_GND;
d_T1_dTemp_t_GND = d_phi_dTemp_t_GND;
T1 = (phi-0.02);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = (d_T1_dV_b_GND-d_Vbs0mos_dV_b_GND);
d_T2_dV_p_GND = (d_T1_dV_p_GND-d_Vbs0mos_dV_p_GND);
d_T2_dTemp_t_GND = (d_T1_dTemp_t_GND-d_Vbs0mos_dTemp_t_GND);
d_T2_dV_gi_si = (d_T1_dV_gi_si-d_Vbs0mos_dV_gi_si);
d_T2_dV_di_si = (d_T1_dV_di_si-d_Vbs0mos_dV_di_si);
d_T2_dV_e_si = (d_T1_dV_e_si-d_Vbs0mos_dV_e_si);
T2 = ((T1-Vbs0mos)-0.005);
{
double value_sqrt_0 = sqrt(((T2*T2)+(4.0*0.005)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_b_GND = (deriv_sqrt_0_d0*((T2*d_T2_dV_b_GND+d_T2_dV_b_GND*T2)));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*((T2*d_T2_dV_p_GND+d_T2_dV_p_GND*T2)));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*((T2*d_T2_dTemp_t_GND+d_T2_dTemp_t_GND*T2)));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*((T2*d_T2_dV_gi_si+d_T2_dV_gi_si*T2)));
d_T3_dV_di_si = (deriv_sqrt_0_d0*((T2*d_T2_dV_di_si+d_T2_dV_di_si*T2)));
d_T3_dV_e_si = (deriv_sqrt_0_d0*((T2*d_T2_dV_e_si+d_T2_dV_e_si*T2)));
T3 = value_sqrt_0;
}

d_Vbs0mos_dV_b_GND = (d_T1_dV_b_GND-0.5*(d_T2_dV_b_GND+d_T3_dV_b_GND));
d_Vbs0mos_dV_p_GND = (d_T1_dV_p_GND-0.5*(d_T2_dV_p_GND+d_T3_dV_p_GND));
d_Vbs0mos_dTemp_t_GND = (d_T1_dTemp_t_GND-0.5*(d_T2_dTemp_t_GND+d_T3_dTemp_t_GND));
d_Vbs0mos_dV_gi_si = (d_T1_dV_gi_si-0.5*(d_T2_dV_gi_si+d_T3_dV_gi_si));
d_Vbs0mos_dV_di_si = (d_T1_dV_di_si-0.5*(d_T2_dV_di_si+d_T3_dV_di_si));
d_Vbs0mos_dV_e_si = (d_T1_dV_e_si-0.5*(d_T2_dV_e_si+d_T3_dV_e_si));
Vbs0mos = (T1-(0.5*(T2+T3)));

d_Phis_dV_b_si =  0.0;
d_Phis_dV_e_si = (-d_Vbs0mos_dV_e_si);
d_Phis_dV_gi_si = (-d_Vbs0mos_dV_gi_si);
d_Phis_dV_di_si = (-d_Vbs0mos_dV_di_si);
d_Phis_dV_b_GND = (d_phi_dV_b_GND-d_Vbs0mos_dV_b_GND);
d_Phis_dV_p_GND = (d_phi_dV_p_GND-d_Vbs0mos_dV_p_GND);
d_Phis_dTemp_t_GND = (d_phi_dTemp_t_GND-d_Vbs0mos_dTemp_t_GND);
Phis = (phi-Vbs0mos);
{
double value_sqrt_0 = sqrt(Phis);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_sqrtPhis_dV_b_si =  0.0;
d_sqrtPhis_dV_e_si = (deriv_sqrt_0_d0*(d_Phis_dV_e_si));
d_sqrtPhis_dV_gi_si = (deriv_sqrt_0_d0*(d_Phis_dV_gi_si));
d_sqrtPhis_dV_di_si = (deriv_sqrt_0_d0*(d_Phis_dV_di_si));
d_sqrtPhis_dV_b_GND = (deriv_sqrt_0_d0*(d_Phis_dV_b_GND));
d_sqrtPhis_dV_p_GND = (deriv_sqrt_0_d0*(d_Phis_dV_p_GND));
d_sqrtPhis_dTemp_t_GND = (deriv_sqrt_0_d0*(d_Phis_dTemp_t_GND));
sqrtPhis = value_sqrt_0;
}

d_Xdep_dV_b_si =  0.0;
d_Xdep_dV_e_si = (Xdep0*d_sqrtPhis_dV_e_si/sqrtPhi);
d_Xdep_dV_gi_si = (Xdep0*d_sqrtPhis_dV_gi_si/sqrtPhi);
d_Xdep_dV_di_si = (Xdep0*d_sqrtPhis_dV_di_si/sqrtPhi);
d_Xdep_dV_b_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_dV_b_GND+d_Xdep0_dV_b_GND*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dV_b_GND)/sqrtPhi/sqrtPhi);
d_Xdep_dV_p_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_dV_p_GND+d_Xdep0_dV_p_GND*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dV_p_GND)/sqrtPhi/sqrtPhi);
d_Xdep_dTemp_t_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_dTemp_t_GND+d_Xdep0_dTemp_t_GND*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dTemp_t_GND)/sqrtPhi/sqrtPhi);
Xdep = ((Xdep0*sqrtPhis)/sqrtPhi);
{
double value_sqrt_0 = sqrt(Xdep);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_e_si = (deriv_sqrt_0_d0*(d_Xdep_dV_e_si));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(d_Xdep_dV_gi_si));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(d_Xdep_dV_di_si));
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(d_Xdep_dV_b_GND));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(d_Xdep_dV_p_GND));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(d_Xdep_dTemp_t_GND));
T3 = value_sqrt_0;
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si =  0.0;
d_T0_dV_e_si = pParam_B4SOIdvt2*d_Vbs0mos_dV_e_si;
d_T0_dV_b_GND = pParam_B4SOIdvt2*d_Vbs0mos_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt2*d_Vbs0mos_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt2*d_Vbs0mos_dTemp_t_GND;
d_T0_dV_gi_si = pParam_B4SOIdvt2*d_Vbs0mos_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt2*d_Vbs0mos_dV_di_si;
T0 = (pParam_B4SOIdvt2*Vbs0mos);
if ((T0>=(-0.5)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T1_dV_e_si = d_T0_dV_e_si;
d_T1_dV_gi_si = d_T0_dV_gi_si;
d_T1_dV_di_si = d_T0_dV_di_si;
T1 = (1.0+T0);
}
else
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si =  0.0;
d_T4_dV_b_GND = (-8.0*d_T0_dV_b_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_p_GND = (-8.0*d_T0_dV_p_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dTemp_t_GND = (-8.0*d_T0_dTemp_t_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_e_si = (-8.0*d_T0_dV_e_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_gi_si = (-8.0*d_T0_dV_gi_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_di_si = (-8.0*d_T0_dV_di_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = ((1.0+(3.0*T0))*d_T4_dV_b_GND+3.0*d_T0_dV_b_GND*T4);
d_T1_dV_p_GND = ((1.0+(3.0*T0))*d_T4_dV_p_GND+3.0*d_T0_dV_p_GND*T4);
d_T1_dTemp_t_GND = ((1.0+(3.0*T0))*d_T4_dTemp_t_GND+3.0*d_T0_dTemp_t_GND*T4);
d_T1_dV_e_si = ((1.0+(3.0*T0))*d_T4_dV_e_si+3.0*d_T0_dV_e_si*T4);
d_T1_dV_gi_si = ((1.0+(3.0*T0))*d_T4_dV_gi_si+3.0*d_T0_dV_gi_si*T4);
d_T1_dV_di_si = ((1.0+(3.0*T0))*d_T4_dV_di_si+3.0*d_T0_dV_di_si*T4);
T1 = ((1.0+(3.0*T0))*T4);
}

d_lt1_dV_b_si =  0.0;
d_lt1_dV_b_GND = ((B4SOIfactor1*T3)*d_T1_dV_b_GND+B4SOIfactor1*d_T3_dV_b_GND*T1);
d_lt1_dV_p_GND = ((B4SOIfactor1*T3)*d_T1_dV_p_GND+B4SOIfactor1*d_T3_dV_p_GND*T1);
d_lt1_dTemp_t_GND = ((B4SOIfactor1*T3)*d_T1_dTemp_t_GND+B4SOIfactor1*d_T3_dTemp_t_GND*T1);
d_lt1_dV_gi_si = ((B4SOIfactor1*T3)*d_T1_dV_gi_si+B4SOIfactor1*d_T3_dV_gi_si*T1);
d_lt1_dV_di_si = ((B4SOIfactor1*T3)*d_T1_dV_di_si+B4SOIfactor1*d_T3_dV_di_si*T1);
d_lt1_dV_e_si = ((B4SOIfactor1*T3)*d_T1_dV_e_si+B4SOIfactor1*d_T3_dV_e_si*T1);
lt1 = ((B4SOIfactor1*T3)*T1);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si =  0.0;
d_T0_dV_e_si = pParam_B4SOIdvt2w*d_Vbs0mos_dV_e_si;
d_T0_dV_b_GND = pParam_B4SOIdvt2w*d_Vbs0mos_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt2w*d_Vbs0mos_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt2w*d_Vbs0mos_dTemp_t_GND;
d_T0_dV_gi_si = pParam_B4SOIdvt2w*d_Vbs0mos_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt2w*d_Vbs0mos_dV_di_si;
T0 = (pParam_B4SOIdvt2w*Vbs0mos);
if ((T0>=(-0.5)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T1_dV_e_si = d_T0_dV_e_si;
d_T1_dV_gi_si = d_T0_dV_gi_si;
d_T1_dV_di_si = d_T0_dV_di_si;
T1 = (1.0+T0);
}
else
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si =  0.0;
d_T4_dV_b_GND = (-8.0*d_T0_dV_b_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_p_GND = (-8.0*d_T0_dV_p_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dTemp_t_GND = (-8.0*d_T0_dTemp_t_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_e_si = (-8.0*d_T0_dV_e_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_gi_si = (-8.0*d_T0_dV_gi_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_di_si = (-8.0*d_T0_dV_di_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = ((1.0+(3.0*T0))*d_T4_dV_b_GND+3.0*d_T0_dV_b_GND*T4);
d_T1_dV_p_GND = ((1.0+(3.0*T0))*d_T4_dV_p_GND+3.0*d_T0_dV_p_GND*T4);
d_T1_dTemp_t_GND = ((1.0+(3.0*T0))*d_T4_dTemp_t_GND+3.0*d_T0_dTemp_t_GND*T4);
d_T1_dV_e_si = ((1.0+(3.0*T0))*d_T4_dV_e_si+3.0*d_T0_dV_e_si*T4);
d_T1_dV_gi_si = ((1.0+(3.0*T0))*d_T4_dV_gi_si+3.0*d_T0_dV_gi_si*T4);
d_T1_dV_di_si = ((1.0+(3.0*T0))*d_T4_dV_di_si+3.0*d_T0_dV_di_si*T4);
T1 = ((1.0+(3.0*T0))*T4);
}

d_ltw_dV_b_si =  0.0;
d_ltw_dV_b_GND = ((B4SOIfactor1*T3)*d_T1_dV_b_GND+B4SOIfactor1*d_T3_dV_b_GND*T1);
d_ltw_dV_p_GND = ((B4SOIfactor1*T3)*d_T1_dV_p_GND+B4SOIfactor1*d_T3_dV_p_GND*T1);
d_ltw_dTemp_t_GND = ((B4SOIfactor1*T3)*d_T1_dTemp_t_GND+B4SOIfactor1*d_T3_dTemp_t_GND*T1);
d_ltw_dV_gi_si = ((B4SOIfactor1*T3)*d_T1_dV_gi_si+B4SOIfactor1*d_T3_dV_gi_si*T1);
d_ltw_dV_di_si = ((B4SOIfactor1*T3)*d_T1_dV_di_si+B4SOIfactor1*d_T3_dV_di_si*T1);
d_ltw_dV_e_si = ((B4SOIfactor1*T3)*d_T1_dV_e_si+B4SOIfactor1*d_T3_dV_e_si*T1);
ltw = ((B4SOIfactor1*T3)*T1);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si =  0.0;
d_T0_dV_b_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_b_GND/lt1/lt1);
d_T0_dV_p_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_p_GND/lt1/lt1);
d_T0_dTemp_t_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dTemp_t_GND/lt1/lt1);
d_T0_dV_gi_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_gi_si/lt1/lt1);
d_T0_dV_di_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_di_si/lt1/lt1);
d_T0_dV_e_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_e_si/lt1/lt1);
T0 = ((((-0.5)*pParam_B4SOIdvt1)*Leff)/lt1);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
T1 = value_exp_0;
}

d_Theta0_dV_b_si =  0.0;
d_Theta0_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_Theta0_dV_b_si =  0.0;
d_Theta0_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_e_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_e_si/Xdep/Xdep);
d_T2_dV_gi_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_gi_si/Xdep/Xdep);
d_T2_dV_di_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_di_si/Xdep/Xdep);
d_T2_dV_b_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_b_GND/Xdep/Xdep);
d_T2_dV_p_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_p_GND/Xdep/Xdep);
d_T2_dTemp_t_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dTemp_t_GND/Xdep/Xdep);
T2 = ((pParam_B4SOInfactor*epssub)/Xdep);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_e_si = pParam_B4SOIcdscb*d_Vbs0mos_dV_e_si;
d_T3_dV_b_GND = pParam_B4SOIcdscb*d_Vbs0mos_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIcdscb*d_Vbs0mos_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIcdscb*d_Vbs0mos_dTemp_t_GND;
d_T3_dV_gi_si = pParam_B4SOIcdscb*d_Vbs0mos_dV_gi_si;
d_T3_dV_di_si = (pParam_B4SOIcdscb*d_Vbs0mos_dV_di_si+pParam_B4SOIcdscd*d_Vds_dV_di_si);
T3 = ((pParam_B4SOIcdsc+(pParam_B4SOIcdscb*Vbs0mos))+(pParam_B4SOIcdscd*Vds));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si =  0.0;
d_T4_dV_b_GND = ((d_T2_dV_b_GND+(T3*d_Theta0_dV_b_GND+d_T3_dV_b_GND*Theta0))/B4SOIcox);
d_T4_dV_p_GND = ((d_T2_dV_p_GND+(T3*d_Theta0_dV_p_GND+d_T3_dV_p_GND*Theta0))/B4SOIcox);
d_T4_dTemp_t_GND = ((d_T2_dTemp_t_GND+(T3*d_Theta0_dTemp_t_GND+d_T3_dTemp_t_GND*Theta0))/B4SOIcox);
d_T4_dV_gi_si = ((d_T2_dV_gi_si+(T3*d_Theta0_dV_gi_si+d_T3_dV_gi_si*Theta0))/B4SOIcox);
d_T4_dV_di_si = ((d_T2_dV_di_si+(T3*d_Theta0_dV_di_si+d_T3_dV_di_si*Theta0))/B4SOIcox);
d_T4_dV_e_si = ((d_T2_dV_e_si+(T3*d_Theta0_dV_e_si+d_T3_dV_e_si*Theta0))/B4SOIcox);
T4 = (((T2+(T3*Theta0))+pParam_B4SOIcit)/B4SOIcox);
if ((T4>=(-0.5)))
{

d_n_dV_b_si =  0.0;
d_n_dV_b_GND = d_T4_dV_b_GND;
d_n_dV_p_GND = d_T4_dV_p_GND;
d_n_dTemp_t_GND = d_T4_dTemp_t_GND;
d_n_dV_gi_si = d_T4_dV_gi_si;
d_n_dV_di_si = d_T4_dV_di_si;
d_n_dV_e_si = d_T4_dV_e_si;
n = (1.0+T4);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si =  0.0;
d_T0_dV_b_GND = (-8.0*d_T4_dV_b_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_p_GND = (-8.0*d_T4_dV_p_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dTemp_t_GND = (-8.0*d_T4_dTemp_t_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_gi_si = (-8.0*d_T4_dV_gi_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_di_si = (-8.0*d_T4_dV_di_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_e_si = (-8.0*d_T4_dV_e_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
T0 = (1.0/(3.0+(8.0*T4)));

d_n_dV_b_si =  0.0;
d_n_dV_b_GND = ((1.0+(3.0*T4))*d_T0_dV_b_GND+3.0*d_T4_dV_b_GND*T0);
d_n_dV_p_GND = ((1.0+(3.0*T4))*d_T0_dV_p_GND+3.0*d_T4_dV_p_GND*T0);
d_n_dTemp_t_GND = ((1.0+(3.0*T4))*d_T0_dTemp_t_GND+3.0*d_T4_dTemp_t_GND*T0);
d_n_dV_gi_si = ((1.0+(3.0*T4))*d_T0_dV_gi_si+3.0*d_T4_dV_gi_si*T0);
d_n_dV_di_si = ((1.0+(3.0*T4))*d_T0_dV_di_si+3.0*d_T4_dV_di_si*T0);
d_n_dV_e_si = ((1.0+(3.0*T4))*d_T0_dV_e_si+3.0*d_T4_dV_e_si*T0);
n = ((1.0+(3.0*T4))*T0);
}
if ((pParam_B4SOIdvtp0>0.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = (-pParam_B4SOIdvtp1)*d_Vds_dV_di_si;
T0 = ((-pParam_B4SOIdvtp1)*Vds);
if ((T0<(-100.0)))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T2_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T2_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T2_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T2_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T2_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
T2 = value_exp_0;
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_b_GND = pParam_B4SOIdvtp0*d_T2_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIdvtp0*d_T2_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIdvtp0*d_T2_dTemp_t_GND;
d_T3_dV_gi_si = pParam_B4SOIdvtp0*d_T2_dV_gi_si;
d_T3_dV_di_si = pParam_B4SOIdvtp0*d_T2_dV_di_si;
d_T3_dV_e_si = pParam_B4SOIdvtp0*d_T2_dV_e_si;
T3 = (Leff+(pParam_B4SOIdvtp0*(1.0+T2)));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((Leff/T3));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si =  0.0;
d_T4_dV_gi_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_gi_si/T3/T3));
d_T4_dV_di_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_di_si/T3/T3));
d_T4_dV_e_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_e_si/T3/T3));
d_T4_dV_b_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_b_GND/T3/T3))+d_Vtm_dV_b_GND*evaluator_lln_0.getValues((Leff/T3)));
d_T4_dV_p_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_p_GND/T3/T3))+d_Vtm_dV_p_GND*evaluator_lln_0.getValues((Leff/T3)));
d_T4_dTemp_t_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dTemp_t_GND/T3/T3))+d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues((Leff/T3)));
T4 = (Vtm*evaluator_lln_0.getValues((Leff/T3)));
}

d_DITS_Sft_dV_b_si =  0.0;
d_DITS_Sft_dV_b_GND = (n*d_T4_dV_b_GND+d_n_dV_b_GND*T4);
d_DITS_Sft_dV_p_GND = (n*d_T4_dV_p_GND+d_n_dV_p_GND*T4);
d_DITS_Sft_dTemp_t_GND = (n*d_T4_dTemp_t_GND+d_n_dTemp_t_GND*T4);
d_DITS_Sft_dV_gi_si = (n*d_T4_dV_gi_si+d_n_dV_gi_si*T4);
d_DITS_Sft_dV_di_si = (n*d_T4_dV_di_si+d_n_dV_di_si*T4);
d_DITS_Sft_dV_e_si = (n*d_T4_dV_e_si+d_n_dV_e_si*T4);
DITS_Sft = (n*T4);
}
else
{

d_DITS_Sft_dV_b_si = d_DITS_Sft_dV_e_si = d_DITS_Sft_dV_di_si = d_DITS_Sft_dV_gi_si = d_DITS_Sft_dTemp_t_GND = d_DITS_Sft_dV_p_GND = d_DITS_Sft_dV_b_GND =  0.0;
DITS_Sft = 0.0;
}

d_B4SOIthetavth_dV_b_si =  0.0;
d_B4SOIthetavth_dV_b_GND = pParam_B4SOIdvt0*d_Theta0_dV_b_GND;
d_B4SOIthetavth_dV_p_GND = pParam_B4SOIdvt0*d_Theta0_dV_p_GND;
d_B4SOIthetavth_dTemp_t_GND = pParam_B4SOIdvt0*d_Theta0_dTemp_t_GND;
d_B4SOIthetavth_dV_gi_si = pParam_B4SOIdvt0*d_Theta0_dV_gi_si;
d_B4SOIthetavth_dV_di_si = pParam_B4SOIdvt0*d_Theta0_dV_di_si;
d_B4SOIthetavth_dV_e_si = pParam_B4SOIdvt0*d_Theta0_dV_e_si;
B4SOIthetavth = (pParam_B4SOIdvt0*Theta0);

d_Delt_vth_dV_b_si =  0.0;
d_Delt_vth_dV_b_GND = (B4SOIthetavth*d_V0_dV_b_GND+d_B4SOIthetavth_dV_b_GND*V0);
d_Delt_vth_dV_p_GND = (B4SOIthetavth*d_V0_dV_p_GND+d_B4SOIthetavth_dV_p_GND*V0);
d_Delt_vth_dTemp_t_GND = (B4SOIthetavth*d_V0_dTemp_t_GND+d_B4SOIthetavth_dTemp_t_GND*V0);
d_Delt_vth_dV_gi_si = d_B4SOIthetavth_dV_gi_si*V0;
d_Delt_vth_dV_di_si = d_B4SOIthetavth_dV_di_si*V0;
d_Delt_vth_dV_e_si = d_B4SOIthetavth_dV_e_si*V0;
Delt_vth = (B4SOIthetavth*V0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si =  0.0;
d_T0_dV_b_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_b_GND/ltw/ltw);
d_T0_dV_p_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_p_GND/ltw/ltw);
d_T0_dTemp_t_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dTemp_t_GND/ltw/ltw);
d_T0_dV_gi_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_gi_si/ltw/ltw);
d_T0_dV_di_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_di_si/ltw/ltw);
d_T0_dV_e_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_e_si/ltw/ltw);
T0 = (((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)/ltw);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
T1 = value_exp_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt0w*d_T2_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt0w*d_T2_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt0w*d_T2_dTemp_t_GND;
d_T0_dV_gi_si = pParam_B4SOIdvt0w*d_T2_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt0w*d_T2_dV_di_si;
d_T0_dV_e_si = pParam_B4SOIdvt0w*d_T2_dV_e_si;
T0 = (pParam_B4SOIdvt0w*T2);

d_DeltVthw_dV_b_si =  0.0;
d_DeltVthw_dV_b_GND = (T0*d_V0_dV_b_GND+d_T0_dV_b_GND*V0);
d_DeltVthw_dV_p_GND = (T0*d_V0_dV_p_GND+d_T0_dV_p_GND*V0);
d_DeltVthw_dTemp_t_GND = (T0*d_V0_dTemp_t_GND+d_T0_dTemp_t_GND*V0);
d_DeltVthw_dV_e_si = d_T0_dV_e_si*V0;
d_DeltVthw_dV_gi_si = d_T0_dV_gi_si*V0;
d_DeltVthw_dV_di_si = d_T0_dV_di_si*V0;
DeltVthw = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpe0/Leff)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = value_sqrt_0;
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_e_si = pParam_B4SOIkt2*d_Vbs0mos_dV_e_si;
d_T1_dV_b_GND = pParam_B4SOIkt2*d_Vbs0mos_dV_b_GND;
d_T1_dV_p_GND = pParam_B4SOIkt2*d_Vbs0mos_dV_p_GND;
d_T1_dTemp_t_GND = pParam_B4SOIkt2*d_Vbs0mos_dTemp_t_GND;
d_T1_dV_gi_si = pParam_B4SOIkt2*d_Vbs0mos_dV_gi_si;
d_T1_dV_di_si = pParam_B4SOIkt2*d_Vbs0mos_dV_di_si;
T1 = ((pParam_B4SOIkt1+(pParam_B4SOIkt1l/Leff))+(pParam_B4SOIkt2*Vbs0mos));

d_DeltVthtemp_dV_b_si =  0.0;
d_DeltVthtemp_dV_b_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_b_GND+pParam_B4SOIk1ox*d_T0_dV_b_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_b_GND+d_T1_dV_b_GND*TempRatioMinus1));
d_DeltVthtemp_dV_p_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_p_GND+pParam_B4SOIk1ox*d_T0_dV_p_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_p_GND+d_T1_dV_p_GND*TempRatioMinus1));
d_DeltVthtemp_dTemp_t_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dTemp_t_GND+pParam_B4SOIk1ox*d_T0_dTemp_t_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dTemp_t_GND+d_T1_dTemp_t_GND*TempRatioMinus1));
d_DeltVthtemp_dV_e_si = (pParam_B4SOIk1ox*d_T0_dV_e_si*sqrtPhi+d_T1_dV_e_si*TempRatioMinus1);
d_DeltVthtemp_dV_gi_si = (pParam_B4SOIk1ox*d_T0_dV_gi_si*sqrtPhi+d_T1_dV_gi_si*TempRatioMinus1);
d_DeltVthtemp_dV_di_si = (pParam_B4SOIk1ox*d_T0_dV_di_si*sqrtPhi+d_T1_dV_di_si*TempRatioMinus1);
DeltVthtemp = (((pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));

d_tmp2_dV_b_si = d_tmp2_dV_e_si = d_tmp2_dV_di_si = d_tmp2_dV_gi_si =  0.0;
d_tmp2_dV_b_GND = (toxe*d_phi_dV_b_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
d_tmp2_dV_p_GND = (toxe*d_phi_dV_p_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
d_tmp2_dTemp_t_GND = (toxe*d_phi_dTemp_t_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
tmp2 = ((toxe*phi)/(pParam_B4SOIweff+pParam_B4SOIw0));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_e_si = pParam_B4SOIetab*d_Vbs0mos_dV_e_si;
d_T3_dV_b_GND = pParam_B4SOIetab*d_Vbs0mos_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIetab*d_Vbs0mos_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIetab*d_Vbs0mos_dTemp_t_GND;
d_T3_dV_gi_si = pParam_B4SOIetab*d_Vbs0mos_dV_gi_si;
d_T3_dV_di_si = pParam_B4SOIetab*d_Vbs0mos_dV_di_si;
T3 = (here_B4SOIeta0+(pParam_B4SOIetab*Vbs0mos));
if ((T3<1.0e-4))
{

d_T9_dV_b_si =  0.0;
d_T9_dV_b_GND = (-(-2.0e4*d_T3_dV_b_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_p_GND = (-(-2.0e4*d_T3_dV_p_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dTemp_t_GND = (-(-2.0e4*d_T3_dTemp_t_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_gi_si = (-(-2.0e4*d_T3_dV_gi_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_di_si = (-(-2.0e4*d_T3_dV_di_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_e_si = (-(-2.0e4*d_T3_dV_e_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_b_GND = ((2.0e-4-T3)*d_T9_dV_b_GND+(-d_T3_dV_b_GND)*T9);
d_T3_dV_p_GND = ((2.0e-4-T3)*d_T9_dV_p_GND+(-d_T3_dV_p_GND)*T9);
d_T3_dTemp_t_GND = ((2.0e-4-T3)*d_T9_dTemp_t_GND+(-d_T3_dTemp_t_GND)*T9);
d_T3_dV_gi_si = ((2.0e-4-T3)*d_T9_dV_gi_si+(-d_T3_dV_gi_si)*T9);
d_T3_dV_di_si = ((2.0e-4-T3)*d_T9_dV_di_si+(-d_T3_dV_di_si)*T9);
d_T3_dV_e_si = ((2.0e-4-T3)*d_T9_dV_e_si+(-d_T3_dV_e_si)*T9);
T3 = ((2.0e-4-T3)*T9);
}

d_DIBL_Sft_dV_b_si =  0.0;
d_DIBL_Sft_dV_b_GND = (T3*d_theta0vb0_dV_b_GND+d_T3_dV_b_GND*theta0vb0)*Vds;
d_DIBL_Sft_dV_p_GND = (T3*d_theta0vb0_dV_p_GND+d_T3_dV_p_GND*theta0vb0)*Vds;
d_DIBL_Sft_dTemp_t_GND = (T3*d_theta0vb0_dTemp_t_GND+d_T3_dTemp_t_GND*theta0vb0)*Vds;
d_DIBL_Sft_dV_gi_si = d_T3_dV_gi_si*theta0vb0*Vds;
d_DIBL_Sft_dV_di_si = ((T3*theta0vb0)*d_Vds_dV_di_si+d_T3_dV_di_si*theta0vb0*Vds);
d_DIBL_Sft_dV_e_si = d_T3_dV_e_si*theta0vb0*Vds;
DIBL_Sft = ((T3*theta0vb0)*Vds);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_e_si = pParam_B4SOIetabcv*d_Vbs0mos_dV_e_si;
d_T3_dV_b_GND = pParam_B4SOIetabcv*d_Vbs0mos_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIetabcv*d_Vbs0mos_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIetabcv*d_Vbs0mos_dTemp_t_GND;
d_T3_dV_gi_si = pParam_B4SOIetabcv*d_Vbs0mos_dV_gi_si;
d_T3_dV_di_si = pParam_B4SOIetabcv*d_Vbs0mos_dV_di_si;
T3 = (here_B4SOIeta0cv+(pParam_B4SOIetabcv*Vbs0mos));
if ((T3<1.0e-4))
{

d_T9_dV_b_si =  0.0;
d_T9_dV_b_GND = (-(-2.0e4*d_T3_dV_b_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_p_GND = (-(-2.0e4*d_T3_dV_p_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dTemp_t_GND = (-(-2.0e4*d_T3_dTemp_t_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_gi_si = (-(-2.0e4*d_T3_dV_gi_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_di_si = (-(-2.0e4*d_T3_dV_di_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_e_si = (-(-2.0e4*d_T3_dV_e_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_b_GND = ((2.0e-4-T3)*d_T9_dV_b_GND+(-d_T3_dV_b_GND)*T9);
d_T3_dV_p_GND = ((2.0e-4-T3)*d_T9_dV_p_GND+(-d_T3_dV_p_GND)*T9);
d_T3_dTemp_t_GND = ((2.0e-4-T3)*d_T9_dTemp_t_GND+(-d_T3_dTemp_t_GND)*T9);
d_T3_dV_gi_si = ((2.0e-4-T3)*d_T9_dV_gi_si+(-d_T3_dV_gi_si)*T9);
d_T3_dV_di_si = ((2.0e-4-T3)*d_T9_dV_di_si+(-d_T3_dV_di_si)*T9);
d_T3_dV_e_si = ((2.0e-4-T3)*d_T9_dV_e_si+(-d_T3_dV_e_si)*T9);
T3 = ((2.0e-4-T3)*T9);
}

d_DIBL_Sft_CV_dV_b_si =  0.0;
d_DIBL_Sft_CV_dV_b_GND = (T3*d_theta0vb0_dV_b_GND+d_T3_dV_b_GND*theta0vb0)*Vds;
d_DIBL_Sft_CV_dV_p_GND = (T3*d_theta0vb0_dV_p_GND+d_T3_dV_p_GND*theta0vb0)*Vds;
d_DIBL_Sft_CV_dTemp_t_GND = (T3*d_theta0vb0_dTemp_t_GND+d_T3_dTemp_t_GND*theta0vb0)*Vds;
d_DIBL_Sft_CV_dV_gi_si = d_T3_dV_gi_si*theta0vb0*Vds;
d_DIBL_Sft_CV_dV_di_si = ((T3*theta0vb0)*d_Vds_dV_di_si+d_T3_dV_di_si*theta0vb0*Vds);
d_DIBL_Sft_CV_dV_e_si = d_T3_dV_e_si*theta0vb0*Vds;
DIBL_Sft_CV = ((T3*theta0vb0)*Vds);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpeb/Leff)));
Lpe_Vb = value_sqrt_0;
}
{
double value_exp_0 = exp(((2.0*pParam_B4SOIdvtp4)*Vds));
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = (deriv_exp_0_d0*((2.0*pParam_B4SOIdvtp4)*d_Vds_dV_di_si));
T0 = value_exp_0;
}

d_DITS_Sft2_dV_b_si =  0.0;
d_DITS_Sft2_dV_b_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_b_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_b_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_p_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_p_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_p_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dTemp_t_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dTemp_t_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dTemp_t_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_e_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_e_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_e_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_gi_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_gi_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_gi_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_di_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_di_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_di_si)/(T0+1)/(T0+1));
DITS_Sft2 = ((pParam_B4SOIdvtp2factor*(T0-1))/(T0+1));

d_VthFD_dV_e_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhis_dV_e_si*Lpe_Vb-here_B4SOIk2ox*d_Vbs0mos_dV_e_si)-d_Delt_vth_dV_e_si)-d_DeltVthw_dV_e_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_e_si+pParam_B4SOIk3b*d_Vbs0mos_dV_e_si*tmp2))+d_DeltVthtemp_dV_e_si)-d_DIBL_Sft_dV_e_si)-d_DITS_Sft_dV_e_si)-d_DITS_Sft2_dV_e_si);
d_VthFD_dV_gi_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhis_dV_gi_si*Lpe_Vb-here_B4SOIk2ox*d_Vbs0mos_dV_gi_si)-d_Delt_vth_dV_gi_si)-d_DeltVthw_dV_gi_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_gi_si+pParam_B4SOIk3b*d_Vbs0mos_dV_gi_si*tmp2))+d_DeltVthtemp_dV_gi_si)-d_DIBL_Sft_dV_gi_si)-d_DITS_Sft_dV_gi_si)-d_DITS_Sft2_dV_gi_si);
d_VthFD_dV_di_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhis_dV_di_si*Lpe_Vb-here_B4SOIk2ox*d_Vbs0mos_dV_di_si)-d_Delt_vth_dV_di_si)-d_DeltVthw_dV_di_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_di_si+pParam_B4SOIk3b*d_Vbs0mos_dV_di_si*tmp2))+d_DeltVthtemp_dV_di_si)-d_DIBL_Sft_dV_di_si)-d_DITS_Sft_dV_di_si)-d_DITS_Sft2_dV_di_si);
d_VthFD_dV_b_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_b_GND+(pParam_B4SOIk1ox*d_sqrtPhis_dV_b_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbs0mos_dV_b_GND)-d_Delt_vth_dV_b_GND)-d_DeltVthw_dV_b_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_b_GND+pParam_B4SOIk3b*d_Vbs0mos_dV_b_GND*tmp2))+d_DeltVthtemp_dV_b_GND)-d_DIBL_Sft_dV_b_GND)-d_DITS_Sft_dV_b_GND)-d_DITS_Sft2_dV_b_GND);
d_VthFD_dV_p_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_p_GND+(pParam_B4SOIk1ox*d_sqrtPhis_dV_p_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbs0mos_dV_p_GND)-d_Delt_vth_dV_p_GND)-d_DeltVthw_dV_p_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_p_GND+pParam_B4SOIk3b*d_Vbs0mos_dV_p_GND*tmp2))+d_DeltVthtemp_dV_p_GND)-d_DIBL_Sft_dV_p_GND)-d_DITS_Sft_dV_p_GND)-d_DITS_Sft2_dV_p_GND);
d_VthFD_dTemp_t_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dTemp_t_GND+(pParam_B4SOIk1ox*d_sqrtPhis_dTemp_t_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbs0mos_dTemp_t_GND)-d_Delt_vth_dTemp_t_GND)-d_DeltVthw_dTemp_t_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dTemp_t_GND+pParam_B4SOIk3b*d_Vbs0mos_dTemp_t_GND*tmp2))+d_DeltVthtemp_dTemp_t_GND)-d_DIBL_Sft_dTemp_t_GND)-d_DITS_Sft_dTemp_t_GND)-d_DITS_Sft2_dTemp_t_GND);
VthFD = ((((((((((B4SOItype*here_B4SOIvth0)+(((pParam_B4SOIk1ox*sqrtPhis)-(pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(here_B4SOIk2ox*Vbs0mos))-Delt_vth)-DeltVthw)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*tmp2))+DeltVthtemp)-DIBL_Sft)-DITS_Sft)-DITS_Sft2);

d_VthFD_CV_dV_e_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhis_dV_e_si*Lpe_Vb-here_B4SOIk2ox*d_Vbs0mos_dV_e_si)-d_Delt_vth_dV_e_si)-d_DeltVthw_dV_e_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_e_si+pParam_B4SOIk3b*d_Vbs0mos_dV_e_si*tmp2))+d_DeltVthtemp_dV_e_si)-d_DIBL_Sft_CV_dV_e_si)-d_DITS_Sft_dV_e_si)-d_DITS_Sft2_dV_e_si);
d_VthFD_CV_dV_gi_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhis_dV_gi_si*Lpe_Vb-here_B4SOIk2ox*d_Vbs0mos_dV_gi_si)-d_Delt_vth_dV_gi_si)-d_DeltVthw_dV_gi_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_gi_si+pParam_B4SOIk3b*d_Vbs0mos_dV_gi_si*tmp2))+d_DeltVthtemp_dV_gi_si)-d_DIBL_Sft_CV_dV_gi_si)-d_DITS_Sft_dV_gi_si)-d_DITS_Sft2_dV_gi_si);
d_VthFD_CV_dV_di_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhis_dV_di_si*Lpe_Vb-here_B4SOIk2ox*d_Vbs0mos_dV_di_si)-d_Delt_vth_dV_di_si)-d_DeltVthw_dV_di_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_di_si+pParam_B4SOIk3b*d_Vbs0mos_dV_di_si*tmp2))+d_DeltVthtemp_dV_di_si)-d_DIBL_Sft_CV_dV_di_si)-d_DITS_Sft_dV_di_si)-d_DITS_Sft2_dV_di_si);
d_VthFD_CV_dV_b_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_b_GND+(pParam_B4SOIk1ox*d_sqrtPhis_dV_b_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbs0mos_dV_b_GND)-d_Delt_vth_dV_b_GND)-d_DeltVthw_dV_b_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_b_GND+pParam_B4SOIk3b*d_Vbs0mos_dV_b_GND*tmp2))+d_DeltVthtemp_dV_b_GND)-d_DIBL_Sft_CV_dV_b_GND)-d_DITS_Sft_dV_b_GND)-d_DITS_Sft2_dV_b_GND);
d_VthFD_CV_dV_p_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_p_GND+(pParam_B4SOIk1ox*d_sqrtPhis_dV_p_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbs0mos_dV_p_GND)-d_Delt_vth_dV_p_GND)-d_DeltVthw_dV_p_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dV_p_GND+pParam_B4SOIk3b*d_Vbs0mos_dV_p_GND*tmp2))+d_DeltVthtemp_dV_p_GND)-d_DIBL_Sft_CV_dV_p_GND)-d_DITS_Sft_dV_p_GND)-d_DITS_Sft2_dV_p_GND);
d_VthFD_CV_dTemp_t_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dTemp_t_GND+(pParam_B4SOIk1ox*d_sqrtPhis_dTemp_t_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbs0mos_dTemp_t_GND)-d_Delt_vth_dTemp_t_GND)-d_DeltVthw_dTemp_t_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dTemp_t_GND+pParam_B4SOIk3b*d_Vbs0mos_dTemp_t_GND*tmp2))+d_DeltVthtemp_dTemp_t_GND)-d_DIBL_Sft_CV_dTemp_t_GND)-d_DITS_Sft_dTemp_t_GND)-d_DITS_Sft2_dTemp_t_GND);
VthFD_CV = ((((((((((B4SOItype*here_B4SOIvth0)+(((pParam_B4SOIk1ox*sqrtPhis)-(pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(here_B4SOIk2ox*Vbs0mos))-Delt_vth)-DeltVthw)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbs0mos))*tmp2))+DeltVthtemp)-DIBL_Sft_CV)-DITS_Sft)-DITS_Sft2);

d_VtgsFD_dV_e_si = d_VthFD_dV_e_si;
d_VtgsFD_dV_gi_si = (d_VthFD_dV_gi_si-d_Vgs_eff_dV_gi_si);
d_VtgsFD_dV_di_si = (d_VthFD_dV_di_si-d_Vgs_eff_dV_di_si);
d_VtgsFD_dV_b_GND = (d_VthFD_dV_b_GND-d_Vgs_eff_dV_b_GND);
d_VtgsFD_dV_p_GND = (d_VthFD_dV_p_GND-d_Vgs_eff_dV_p_GND);
d_VtgsFD_dTemp_t_GND = (d_VthFD_dTemp_t_GND-d_Vgs_eff_dTemp_t_GND);
VtgsFD = (VthFD-Vgs_eff);

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si =  0.0;
d_T10_dV_b_GND = pParam_B4SOInofffd*d_Vtm_dV_b_GND;
d_T10_dV_p_GND = pParam_B4SOInofffd*d_Vtm_dV_p_GND;
d_T10_dTemp_t_GND = pParam_B4SOInofffd*d_Vtm_dTemp_t_GND;
T10 = (pParam_B4SOInofffd*Vtm);
if ((((VtgsFD-pParam_B4SOIvofffd)/T10)>100.0))
{

d_ExpVtgsFD_dV_e_si = 2.688117142e+43*((T10*d_VtgsFD_dV_e_si-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10);
d_ExpVtgsFD_dV_gi_si = 2.688117142e+43*((T10*d_VtgsFD_dV_gi_si-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10);
d_ExpVtgsFD_dV_di_si = 2.688117142e+43*((T10*d_VtgsFD_dV_di_si-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10);
d_ExpVtgsFD_dV_b_GND = 2.688117142e+43*((T10*d_VtgsFD_dV_b_GND-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10);
d_ExpVtgsFD_dV_p_GND = 2.688117142e+43*((T10*d_VtgsFD_dV_p_GND-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10);
d_ExpVtgsFD_dTemp_t_GND = 2.688117142e+43*((T10*d_VtgsFD_dTemp_t_GND-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10);
ExpVtgsFD = (2.688117142e+43*((1.0+((VtgsFD-pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VtgsFD-pParam_B4SOIvofffd)/T10)<(-100.0)))
{

d_ExpVtgsFD_dTemp_t_GND = d_ExpVtgsFD_dV_p_GND = d_ExpVtgsFD_dV_b_GND = d_ExpVtgsFD_dV_di_si = d_ExpVtgsFD_dV_gi_si = d_ExpVtgsFD_dV_e_si =  0.0;
ExpVtgsFD = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VtgsFD-pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVtgsFD_dV_e_si = (deriv_exp_0_d0*(((T10*d_VtgsFD_dV_e_si-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10)));
d_ExpVtgsFD_dV_gi_si = (deriv_exp_0_d0*(((T10*d_VtgsFD_dV_gi_si-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10)));
d_ExpVtgsFD_dV_di_si = (deriv_exp_0_d0*(((T10*d_VtgsFD_dV_di_si-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10)));
d_ExpVtgsFD_dV_b_GND = (deriv_exp_0_d0*(((T10*d_VtgsFD_dV_b_GND-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10)));
d_ExpVtgsFD_dV_p_GND = (deriv_exp_0_d0*(((T10*d_VtgsFD_dV_p_GND-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10)));
d_ExpVtgsFD_dTemp_t_GND = (deriv_exp_0_d0*(((T10*d_VtgsFD_dTemp_t_GND-(VtgsFD-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10)));
ExpVtgsFD = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVtgsFD));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVtgsFD));

d_VtgseffFD_dV_e_si = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_dV_e_si))+d_T10_dV_e_si*value_log_0);
d_VtgseffFD_dV_gi_si = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_dV_gi_si))+d_T10_dV_gi_si*value_log_0);
d_VtgseffFD_dV_di_si = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_dV_di_si))+d_T10_dV_di_si*value_log_0);
d_VtgseffFD_dV_b_GND = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_dV_b_GND))+d_T10_dV_b_GND*value_log_0);
d_VtgseffFD_dV_p_GND = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_dV_p_GND))+d_T10_dV_p_GND*value_log_0);
d_VtgseffFD_dTemp_t_GND = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_dTemp_t_GND))+d_T10_dTemp_t_GND*value_log_0);
VtgseffFD = (T10*value_log_0);
}

d_VgstFD_dV_e_si = (-d_VthFD_dV_e_si);
d_VgstFD_dV_b_GND = (d_Vgs_eff_dV_b_GND-d_VthFD_dV_b_GND);
d_VgstFD_dV_p_GND = (d_Vgs_eff_dV_p_GND-d_VthFD_dV_p_GND);
d_VgstFD_dTemp_t_GND = (d_Vgs_eff_dTemp_t_GND-d_VthFD_dTemp_t_GND);
d_VgstFD_dV_gi_si = (d_Vgs_eff_dV_gi_si-d_VthFD_dV_gi_si);
d_VgstFD_dV_di_si = (d_Vgs_eff_dV_di_si-d_VthFD_dV_di_si);
VgstFD = (Vgs_eff-VthFD);
if ((((VgstFD-pParam_B4SOIvofffd)/T10)>100.0))
{

d_ExpVgstFD_dV_e_si = 2.688117142e+43*((T10*d_VgstFD_dV_e_si-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10);
d_ExpVgstFD_dV_b_GND = 2.688117142e+43*((T10*d_VgstFD_dV_b_GND-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10);
d_ExpVgstFD_dV_p_GND = 2.688117142e+43*((T10*d_VgstFD_dV_p_GND-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10);
d_ExpVgstFD_dTemp_t_GND = 2.688117142e+43*((T10*d_VgstFD_dTemp_t_GND-(VgstFD-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10);
d_ExpVgstFD_dV_gi_si = 2.688117142e+43*((T10*d_VgstFD_dV_gi_si-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10);
d_ExpVgstFD_dV_di_si = 2.688117142e+43*((T10*d_VgstFD_dV_di_si-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10);
ExpVgstFD = (2.688117142e+43*((1.0+((VgstFD-pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VgstFD-pParam_B4SOIvofffd)/T10)<(-100.0)))
{

d_ExpVgstFD_dV_di_si = d_ExpVgstFD_dV_gi_si = d_ExpVgstFD_dTemp_t_GND = d_ExpVgstFD_dV_p_GND = d_ExpVgstFD_dV_b_GND = d_ExpVgstFD_dV_e_si =  0.0;
ExpVgstFD = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VgstFD-pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgstFD_dV_e_si = (deriv_exp_0_d0*(((T10*d_VgstFD_dV_e_si-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10)));
d_ExpVgstFD_dV_b_GND = (deriv_exp_0_d0*(((T10*d_VgstFD_dV_b_GND-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10)));
d_ExpVgstFD_dV_p_GND = (deriv_exp_0_d0*(((T10*d_VgstFD_dV_p_GND-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10)));
d_ExpVgstFD_dTemp_t_GND = (deriv_exp_0_d0*(((T10*d_VgstFD_dTemp_t_GND-(VgstFD-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10)));
d_ExpVgstFD_dV_gi_si = (deriv_exp_0_d0*(((T10*d_VgstFD_dV_gi_si-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10)));
d_ExpVgstFD_dV_di_si = (deriv_exp_0_d0*(((T10*d_VgstFD_dV_di_si-(VgstFD-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10)));
ExpVgstFD = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVgstFD));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVgstFD));

d_VgsteffFD_dV_e_si = (T10*(deriv_log_0_d0*(d_ExpVgstFD_dV_e_si))+d_T10_dV_e_si*value_log_0);
d_VgsteffFD_dV_gi_si = (T10*(deriv_log_0_d0*(d_ExpVgstFD_dV_gi_si))+d_T10_dV_gi_si*value_log_0);
d_VgsteffFD_dV_di_si = (T10*(deriv_log_0_d0*(d_ExpVgstFD_dV_di_si))+d_T10_dV_di_si*value_log_0);
d_VgsteffFD_dV_b_GND = (T10*(deriv_log_0_d0*(d_ExpVgstFD_dV_b_GND))+d_T10_dV_b_GND*value_log_0);
d_VgsteffFD_dV_p_GND = (T10*(deriv_log_0_d0*(d_ExpVgstFD_dV_p_GND))+d_T10_dV_p_GND*value_log_0);
d_VgsteffFD_dTemp_t_GND = (T10*(deriv_log_0_d0*(d_ExpVgstFD_dTemp_t_GND))+d_T10_dTemp_t_GND*value_log_0);
VgsteffFD = (T10*value_log_0);
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*d_Vtm_dV_b_GND+(pParam_B4SOImoinfd*pParam_B4SOIk1ox)*d_Vtm_dV_b_GND*Vtm);
d_T1_dV_p_GND = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*d_Vtm_dV_p_GND+(pParam_B4SOImoinfd*pParam_B4SOIk1ox)*d_Vtm_dV_p_GND*Vtm);
d_T1_dTemp_t_GND = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*d_Vtm_dTemp_t_GND+(pParam_B4SOImoinfd*pParam_B4SOIk1ox)*d_Vtm_dTemp_t_GND*Vtm);
T1 = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*Vtm);
{
double value_sqrt_0 = sqrt(phi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_e_si = d_VgsteffFD_dV_e_si;
d_T2_dV_gi_si = d_VgsteffFD_dV_gi_si;
d_T2_dV_di_si = d_VgsteffFD_dV_di_si;
d_T2_dV_b_GND = (d_VgsteffFD_dV_b_GND+((2*pParam_B4SOIk1eff)*(deriv_sqrt_0_d0*(d_phi_dV_b_GND))+2*d_pParam_B4SOIk1eff_dV_b_GND*value_sqrt_0));
d_T2_dV_p_GND = (d_VgsteffFD_dV_p_GND+((2*pParam_B4SOIk1eff)*(deriv_sqrt_0_d0*(d_phi_dV_p_GND))+2*d_pParam_B4SOIk1eff_dV_p_GND*value_sqrt_0));
d_T2_dTemp_t_GND = (d_VgsteffFD_dTemp_t_GND+((2*pParam_B4SOIk1eff)*(deriv_sqrt_0_d0*(d_phi_dTemp_t_GND))+2*d_pParam_B4SOIk1eff_dTemp_t_GND*value_sqrt_0));
T2 = (VgsteffFD+((2*pParam_B4SOIk1eff)*value_sqrt_0));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si =  0.0;
d_T0_dV_e_si = ((T1*(VgsteffFD*d_T2_dV_e_si+d_VgsteffFD_dV_e_si*T2)-(VgsteffFD*T2)*d_T1_dV_e_si)/T1/T1);
d_T0_dV_gi_si = ((T1*(VgsteffFD*d_T2_dV_gi_si+d_VgsteffFD_dV_gi_si*T2)-(VgsteffFD*T2)*d_T1_dV_gi_si)/T1/T1);
d_T0_dV_di_si = ((T1*(VgsteffFD*d_T2_dV_di_si+d_VgsteffFD_dV_di_si*T2)-(VgsteffFD*T2)*d_T1_dV_di_si)/T1/T1);
d_T0_dV_b_GND = ((T1*(VgsteffFD*d_T2_dV_b_GND+d_VgsteffFD_dV_b_GND*T2)-(VgsteffFD*T2)*d_T1_dV_b_GND)/T1/T1);
d_T0_dV_p_GND = ((T1*(VgsteffFD*d_T2_dV_p_GND+d_VgsteffFD_dV_p_GND*T2)-(VgsteffFD*T2)*d_T1_dV_p_GND)/T1/T1);
d_T0_dTemp_t_GND = ((T1*(VgsteffFD*d_T2_dTemp_t_GND+d_VgsteffFD_dTemp_t_GND*T2)-(VgsteffFD*T2)*d_T1_dTemp_t_GND)/T1/T1);
T0 = (1+((VgsteffFD*T2)/T1));
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);

d_PhiON_dV_e_si = Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_e_si);
d_PhiON_dV_gi_si = Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_gi_si);
d_PhiON_dV_di_si = Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_di_si);
d_PhiON_dV_b_GND = (d_phi_dV_b_GND+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_GND)+d_Vtm_dV_b_GND*evaluator_lln_0.getValues(T0)));
d_PhiON_dV_p_GND = (d_phi_dV_p_GND+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_p_GND)+d_Vtm_dV_p_GND*evaluator_lln_0.getValues(T0)));
d_PhiON_dTemp_t_GND = (d_phi_dTemp_t_GND+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dTemp_t_GND)+d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues(T0)));
PhiON = (phi+(Vtm*evaluator_lln_0.getValues(T0)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (B4SOIcox/(B4SOIcox+(1.0/((1.0/B4SOIcsi)+(1.0/Cbox)))));

d_PhiFD_dV_e_si = (d_PhiON_dV_e_si-(T0*d_VtgseffFD_dV_e_si+d_T0_dV_e_si*VtgseffFD));
d_PhiFD_dV_gi_si = (d_PhiON_dV_gi_si-(T0*d_VtgseffFD_dV_gi_si+d_T0_dV_gi_si*VtgseffFD));
d_PhiFD_dV_di_si = (d_PhiON_dV_di_si-(T0*d_VtgseffFD_dV_di_si+d_T0_dV_di_si*VtgseffFD));
d_PhiFD_dV_b_GND = (d_PhiON_dV_b_GND-(T0*d_VtgseffFD_dV_b_GND+d_T0_dV_b_GND*VtgseffFD));
d_PhiFD_dV_p_GND = (d_PhiON_dV_p_GND-(T0*d_VtgseffFD_dV_p_GND+d_T0_dV_p_GND*VtgseffFD));
d_PhiFD_dTemp_t_GND = (d_PhiON_dTemp_t_GND-(T0*d_VtgseffFD_dTemp_t_GND+d_T0_dTemp_t_GND*VtgseffFD));
PhiFD = (PhiON-(T0*VtgseffFD));
if ((B4SOIfdMod==0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (((-pParam_B4SOIdvbd1)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T0));
double value_exp_1 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_b_GND))+2*(deriv_exp_1_d0*(d_T0_dV_b_GND)));
d_T1_dV_p_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_p_GND))+2*(deriv_exp_1_d0*(d_T0_dV_p_GND)));
d_T1_dTemp_t_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T0_dTemp_t_GND)));
d_T1_dV_e_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_e_si))+2*(deriv_exp_1_d0*(d_T0_dV_e_si)));
d_T1_dV_gi_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_gi_si))+2*(deriv_exp_1_d0*(d_T0_dV_gi_si)));
d_T1_dV_di_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_di_si))+2*(deriv_exp_1_d0*(d_T0_dV_di_si)));
T1 = (pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = (T1*(d_vbi_dV_b_GND-d_phi_dV_b_GND)+d_T1_dV_b_GND*(vbi-phi));
d_T2_dV_p_GND = (T1*(d_vbi_dV_p_GND-d_phi_dV_p_GND)+d_T1_dV_p_GND*(vbi-phi));
d_T2_dTemp_t_GND = (T1*(d_vbi_dTemp_t_GND-d_phi_dTemp_t_GND)+d_T1_dTemp_t_GND*(vbi-phi));
d_T2_dV_gi_si = d_T1_dV_gi_si*(vbi-phi);
d_T2_dV_di_si = d_T1_dV_di_si*(vbi-phi);
d_T2_dV_e_si = d_T1_dV_e_si*(vbi-phi);
T2 = (T1*(vbi-phi));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = ((0.5*pParam_B4SOIqsi)/B4SOIcsi);

d_Vbs0t_dV_e_si = ((d_PhiFD_dV_e_si-d_T3_dV_e_si)+d_T2_dV_e_si);
d_Vbs0t_dV_gi_si = ((d_PhiFD_dV_gi_si-d_T3_dV_gi_si)+d_T2_dV_gi_si);
d_Vbs0t_dV_di_si = ((d_PhiFD_dV_di_si-d_T3_dV_di_si)+d_T2_dV_di_si);
d_Vbs0t_dV_b_GND = ((d_PhiFD_dV_b_GND-d_T3_dV_b_GND)+d_T2_dV_b_GND);
d_Vbs0t_dV_p_GND = ((d_PhiFD_dV_p_GND-d_T3_dV_p_GND)+d_T2_dV_p_GND);
d_Vbs0t_dTemp_t_GND = ((d_PhiFD_dTemp_t_GND-d_T3_dTemp_t_GND)+d_T2_dTemp_t_GND);
Vbs0t = (((PhiFD-T3)+pParam_B4SOIvbsa)+T2);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1+(B4SOIcsi/Cbox));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = (((-pParam_B4SOIdk2b)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T3));
double value_exp_1 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si =  0.0;
d_T5_dV_b_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_b_GND))+2*(deriv_exp_1_d0*(d_T3_dV_b_GND)));
d_T5_dV_p_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_p_GND))+2*(deriv_exp_1_d0*(d_T3_dV_p_GND)));
d_T5_dTemp_t_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T3_dTemp_t_GND)));
d_T5_dV_gi_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_gi_si))+2*(deriv_exp_1_d0*(d_T3_dV_gi_si)));
d_T5_dV_di_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_di_si))+2*(deriv_exp_1_d0*(d_T3_dV_di_si)));
d_T5_dV_e_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_e_si))+2*(deriv_exp_1_d0*(d_T3_dV_e_si)));
T5 = (pParam_B4SOIk2b*(value_exp_0+(2*value_exp_1)));
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si =  0.0;
d_T1_dV_b_GND = ((T0*(-d_T5_dV_b_GND)-(pParam_B4SOIk1b-T5)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-d_T5_dV_p_GND)-(pParam_B4SOIk1b-T5)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-d_T5_dTemp_t_GND)-(pParam_B4SOIk1b-T5)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-d_T5_dV_gi_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*(-d_T5_dV_di_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_di_si)/T0/T0);
d_T1_dV_e_si = ((T0*(-d_T5_dV_e_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_e_si)/T0/T0);
T1 = ((pParam_B4SOIk1b-T5)/T0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = (T1*d_Vesfb_dV_b_GND+d_T1_dV_b_GND*Vesfb);
d_T2_dV_p_GND = (T1*d_Vesfb_dV_p_GND+d_T1_dV_p_GND*Vesfb);
d_T2_dTemp_t_GND = (T1*d_Vesfb_dTemp_t_GND+d_T1_dTemp_t_GND*Vesfb);
d_T2_dV_gi_si = d_T1_dV_gi_si*Vesfb;
d_T2_dV_di_si = (T1*d_Vesfb_dV_di_si+d_T1_dV_di_si*Vesfb);
d_T2_dV_e_si = (T1*d_Vesfb_dV_e_si+d_T1_dV_e_si*Vesfb);
T2 = (T1*Vesfb);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0/(1+(Cbox/B4SOIcsi)));

d_Vbs0_dV_b_GND = ((T0*d_Vbs0t_dV_b_GND+d_T0_dV_b_GND*Vbs0t)+d_T2_dV_b_GND);
d_Vbs0_dV_p_GND = ((T0*d_Vbs0t_dV_p_GND+d_T0_dV_p_GND*Vbs0t)+d_T2_dV_p_GND);
d_Vbs0_dTemp_t_GND = ((T0*d_Vbs0t_dTemp_t_GND+d_T0_dTemp_t_GND*Vbs0t)+d_T2_dTemp_t_GND);
d_Vbs0_dV_e_si = ((T0*d_Vbs0t_dV_e_si+d_T0_dV_e_si*Vbs0t)+d_T2_dV_e_si);
d_Vbs0_dV_gi_si = ((T0*d_Vbs0t_dV_gi_si+d_T0_dV_gi_si*Vbs0t)+d_T2_dV_gi_si);
d_Vbs0_dV_di_si = ((T0*d_Vbs0t_dV_di_si+d_T0_dV_di_si*Vbs0t)+d_T2_dV_di_si);
Vbs0 = ((T0*Vbs0t)+T2);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0/((B4SOIcsi+Cbox)+pParam_B4SOIcdsbs));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = (((-pParam_B4SOIdvbd1)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T1));
double value_exp_1 = exp(T1);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_b_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_b_GND))+2*(deriv_exp_1_d0*(d_T1_dV_b_GND)));
d_T2_dV_p_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_p_GND))+2*(deriv_exp_1_d0*(d_T1_dV_p_GND)));
d_T2_dTemp_t_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T1_dTemp_t_GND)));
d_T2_dV_gi_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_gi_si))+2*(deriv_exp_1_d0*(d_T1_dV_gi_si)));
d_T2_dV_di_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_di_si))+2*(deriv_exp_1_d0*(d_T1_dV_di_si)));
d_T2_dV_e_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_e_si))+2*(deriv_exp_1_d0*(d_T1_dV_e_si)));
T2 = (pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si =  0.0;
d_T3_dV_b_GND = d_T2_dV_b_GND*(Vds+pParam_B4SOIvsce);
d_T3_dV_p_GND = d_T2_dV_p_GND*(Vds+pParam_B4SOIvsce);
d_T3_dTemp_t_GND = d_T2_dTemp_t_GND*(Vds+pParam_B4SOIvsce);
d_T3_dV_gi_si = d_T2_dV_gi_si*(Vds+pParam_B4SOIvsce);
d_T3_dV_di_si = (T2*d_Vds_dV_di_si+d_T2_dV_di_si*(Vds+pParam_B4SOIvsce));
d_T3_dV_e_si = d_T2_dV_e_si*(Vds+pParam_B4SOIvsce);
T3 = (T2*(Vds+pParam_B4SOIvsce));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = ((0.5*pParam_B4SOIqsi)/B4SOIcsi);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si =  0.0;
d_T5_dV_b_GND = ((B4SOIcsi*T0)*(d_PhiFD_dV_b_GND-d_T4_dV_b_GND)+B4SOIcsi*d_T0_dV_b_GND*((PhiFD-T4)+pParam_B4SOIvbsa));
d_T5_dV_p_GND = ((B4SOIcsi*T0)*(d_PhiFD_dV_p_GND-d_T4_dV_p_GND)+B4SOIcsi*d_T0_dV_p_GND*((PhiFD-T4)+pParam_B4SOIvbsa));
d_T5_dTemp_t_GND = ((B4SOIcsi*T0)*(d_PhiFD_dTemp_t_GND-d_T4_dTemp_t_GND)+B4SOIcsi*d_T0_dTemp_t_GND*((PhiFD-T4)+pParam_B4SOIvbsa));
d_T5_dV_e_si = ((B4SOIcsi*T0)*(d_PhiFD_dV_e_si-d_T4_dV_e_si)+B4SOIcsi*d_T0_dV_e_si*((PhiFD-T4)+pParam_B4SOIvbsa));
d_T5_dV_gi_si = ((B4SOIcsi*T0)*(d_PhiFD_dV_gi_si-d_T4_dV_gi_si)+B4SOIcsi*d_T0_dV_gi_si*((PhiFD-T4)+pParam_B4SOIvbsa));
d_T5_dV_di_si = ((B4SOIcsi*T0)*(d_PhiFD_dV_di_si-d_T4_dV_di_si)+B4SOIcsi*d_T0_dV_di_si*((PhiFD-T4)+pParam_B4SOIvbsa));
T5 = ((B4SOIcsi*T0)*((PhiFD-T4)+pParam_B4SOIvbsa));

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si =  0.0;
d_T6_dV_b_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_b_GND+pParam_B4SOIcdsbs*d_T0_dV_b_GND*T3);
d_T6_dV_p_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_p_GND+pParam_B4SOIcdsbs*d_T0_dV_p_GND*T3);
d_T6_dTemp_t_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dTemp_t_GND+pParam_B4SOIcdsbs*d_T0_dTemp_t_GND*T3);
d_T6_dV_e_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_e_si+pParam_B4SOIcdsbs*d_T0_dV_e_si*T3);
d_T6_dV_gi_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_gi_si+pParam_B4SOIcdsbs*d_T0_dV_gi_si*T3);
d_T6_dV_di_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_di_si+pParam_B4SOIcdsbs*d_T0_dV_di_si*T3);
T6 = ((pParam_B4SOIcdsbs*T0)*T3);

d_Vbs0t_dV_b_GND = (d_T5_dV_b_GND+d_T6_dV_b_GND);
d_Vbs0t_dV_p_GND = (d_T5_dV_p_GND+d_T6_dV_p_GND);
d_Vbs0t_dTemp_t_GND = (d_T5_dTemp_t_GND+d_T6_dTemp_t_GND);
d_Vbs0t_dV_gi_si = (d_T5_dV_gi_si+d_T6_dV_gi_si);
d_Vbs0t_dV_di_si = (d_T5_dV_di_si+d_T6_dV_di_si);
d_Vbs0t_dV_e_si = (d_T5_dV_e_si+d_T6_dV_e_si);
Vbs0t = (T5+T6);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si =  0.0;
d_T7_dV_b_GND = ((Cbox*T0)*d_Vesfb_dV_b_GND+Cbox*d_T0_dV_b_GND*Vesfb);
d_T7_dV_p_GND = ((Cbox*T0)*d_Vesfb_dV_p_GND+Cbox*d_T0_dV_p_GND*Vesfb);
d_T7_dTemp_t_GND = ((Cbox*T0)*d_Vesfb_dTemp_t_GND+Cbox*d_T0_dTemp_t_GND*Vesfb);
d_T7_dV_e_si = ((Cbox*T0)*d_Vesfb_dV_e_si+Cbox*d_T0_dV_e_si*Vesfb);
d_T7_dV_gi_si = Cbox*d_T0_dV_gi_si*Vesfb;
d_T7_dV_di_si = ((Cbox*T0)*d_Vesfb_dV_di_si+Cbox*d_T0_dV_di_si*Vesfb);
T7 = ((Cbox*T0)*Vesfb);

d_Vbs0_dV_gi_si = (d_Vbs0t_dV_gi_si+d_T7_dV_gi_si);
d_Vbs0_dV_di_si = (d_Vbs0t_dV_di_si+d_T7_dV_di_si);
d_Vbs0_dV_b_GND = (d_Vbs0t_dV_b_GND+d_T7_dV_b_GND);
d_Vbs0_dV_p_GND = (d_Vbs0t_dV_p_GND+d_T7_dV_p_GND);
d_Vbs0_dTemp_t_GND = (d_Vbs0t_dTemp_t_GND+d_T7_dTemp_t_GND);
d_Vbs0_dV_e_si = (d_Vbs0t_dV_e_si+d_T7_dV_e_si);
Vbs0 = (Vbs0t+T7);
}
if ((B4SOIsoiMod==2))
{

d_Vbsitf_dV_b_si =  0.0;
d_Vbsitf_dV_e_si = d_Vbs0_dV_e_si;
d_Vbsitf_dV_b_GND = d_Vbs0_dV_b_GND;
d_Vbsitf_dV_p_GND = d_Vbs0_dV_p_GND;
d_Vbsitf_dTemp_t_GND = d_Vbs0_dTemp_t_GND;
d_Vbsitf_dV_gi_si = d_Vbs0_dV_gi_si;
d_Vbsitf_dV_di_si = d_Vbs0_dV_di_si;
Vbsitf = (Vbs0+0.02);

d_Vbs_dV_b_si =  0.0;
d_Vbs_dV_e_si = d_Vbs0_dV_e_si;
d_Vbs_dV_b_GND = d_Vbs0_dV_b_GND;
d_Vbs_dV_p_GND = d_Vbs0_dV_p_GND;
d_Vbs_dTemp_t_GND = d_Vbs0_dTemp_t_GND;
d_Vbs_dV_gi_si = d_Vbs0_dV_gi_si;
d_Vbs_dV_di_si = d_Vbs0_dV_di_si;
Vbs = (Vbs0+0.02);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_si = d_Vbs_dV_b_si;
d_T1_dV_di_si = (d_Vbs_dV_di_si-d_Vbs0_dV_di_si);
d_T1_dV_e_si = (d_Vbs_dV_e_si-d_Vbs0_dV_e_si);
d_T1_dV_b_GND = (d_Vbs_dV_b_GND-d_Vbs0_dV_b_GND);
d_T1_dV_p_GND = (d_Vbs_dV_p_GND-d_Vbs0_dV_p_GND);
d_T1_dTemp_t_GND = (d_Vbs_dTemp_t_GND-d_Vbs0_dTemp_t_GND);
d_T1_dV_gi_si = (d_Vbs_dV_gi_si-d_Vbs0_dV_gi_si);
T1 = ((Vbs-(Vbs0+0.02))-0.01);
{
double value_sqrt_0 = sqrt(((T1*T1)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)));
T2 = value_sqrt_0;
}

d_Vbsitf_dV_b_si = 0.5*(d_T1_dV_b_si+d_T2_dV_b_si);
d_Vbsitf_dV_e_si = (d_Vbs0_dV_e_si+0.5*(d_T1_dV_e_si+d_T2_dV_e_si));
d_Vbsitf_dV_b_GND = (d_Vbs0_dV_b_GND+0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND));
d_Vbsitf_dV_p_GND = (d_Vbs0_dV_p_GND+0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND));
d_Vbsitf_dTemp_t_GND = (d_Vbs0_dTemp_t_GND+0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND));
d_Vbsitf_dV_gi_si = (d_Vbs0_dV_gi_si+0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si));
d_Vbsitf_dV_di_si = (d_Vbs0_dV_di_si+0.5*(d_T1_dV_di_si+d_T2_dV_di_si));
Vbsitf = ((Vbs0+0.02)+(0.5*(T1+T2)));
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_si = (-d_Vbsitf_dV_b_si);
d_T1_dV_gi_si = (d_Vbs0t_dV_gi_si-d_Vbsitf_dV_gi_si);
d_T1_dV_di_si = (d_Vbs0t_dV_di_si-d_Vbsitf_dV_di_si);
d_T1_dV_b_GND = (d_Vbs0t_dV_b_GND-d_Vbsitf_dV_b_GND);
d_T1_dV_p_GND = (d_Vbs0t_dV_p_GND-d_Vbsitf_dV_p_GND);
d_T1_dTemp_t_GND = (d_Vbs0t_dTemp_t_GND-d_Vbsitf_dTemp_t_GND);
d_T1_dV_e_si = (d_Vbs0t_dV_e_si-d_Vbsitf_dV_e_si);
T1 = ((Vbs0t-Vbsitf)-0.005);
{
double value_sqrt_0 = sqrt(((T1*T1)+2.5e-5));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)));
T2 = value_sqrt_0;
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = 0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND);
d_T3_dV_p_GND = 0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND);
d_T3_dTemp_t_GND = 0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND);
d_T3_dV_gi_si = 0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si);
d_T3_dV_di_si = 0.5*(d_T1_dV_di_si+d_T2_dV_di_si);
d_T3_dV_e_si = 0.5*(d_T1_dV_e_si+d_T2_dV_e_si);
d_T3_dV_b_si = 0.5*(d_T1_dV_b_si+d_T2_dV_b_si);
T3 = (0.5*(T1+T2));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (d_T3_dV_b_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_p_GND = (d_T3_dV_p_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dTemp_t_GND = (d_T3_dTemp_t_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_gi_si = (d_T3_dV_gi_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_di_si = (d_T3_dV_di_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_e_si = (d_T3_dV_e_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_b_si = (d_T3_dV_b_si*B4SOIcsi/pParam_B4SOIqsi);
T4 = ((T3*B4SOIcsi)/pParam_B4SOIqsi);

d_Vbsmos_dV_e_si = (d_Vbsitf_dV_e_si-((0.5*T3)*d_T4_dV_e_si+0.5*d_T3_dV_e_si*T4));
d_Vbsmos_dV_b_GND = (d_Vbsitf_dV_b_GND-((0.5*T3)*d_T4_dV_b_GND+0.5*d_T3_dV_b_GND*T4));
d_Vbsmos_dV_p_GND = (d_Vbsitf_dV_p_GND-((0.5*T3)*d_T4_dV_p_GND+0.5*d_T3_dV_p_GND*T4));
d_Vbsmos_dTemp_t_GND = (d_Vbsitf_dTemp_t_GND-((0.5*T3)*d_T4_dTemp_t_GND+0.5*d_T3_dTemp_t_GND*T4));
d_Vbsmos_dV_gi_si = (d_Vbsitf_dV_gi_si-((0.5*T3)*d_T4_dV_gi_si+0.5*d_T3_dV_gi_si*T4));
d_Vbsmos_dV_di_si = (d_Vbsitf_dV_di_si-((0.5*T3)*d_T4_dV_di_si+0.5*d_T3_dV_di_si*T4));
d_Vbsmos_dV_b_si = (d_Vbsitf_dV_b_si-((0.5*T3)*d_T4_dV_b_si+0.5*d_T3_dV_b_si*T4));
Vbsmos = (Vbsitf-((0.5*T3)*T4));

d_VtgsFD_CV_dV_e_si = d_VthFD_CV_dV_e_si;
d_VtgsFD_CV_dV_gi_si = (d_VthFD_CV_dV_gi_si-d_Vgs_eff_dV_gi_si);
d_VtgsFD_CV_dV_di_si = (d_VthFD_CV_dV_di_si-d_Vgs_eff_dV_di_si);
d_VtgsFD_CV_dV_b_GND = (d_VthFD_CV_dV_b_GND-d_Vgs_eff_dV_b_GND);
d_VtgsFD_CV_dV_p_GND = (d_VthFD_CV_dV_p_GND-d_Vgs_eff_dV_p_GND);
d_VtgsFD_CV_dTemp_t_GND = (d_VthFD_CV_dTemp_t_GND-d_Vgs_eff_dTemp_t_GND);
VtgsFD_CV = (VthFD_CV-Vgs_eff);

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si =  0.0;
d_T10_dV_b_GND = pParam_B4SOInofffd*d_Vtm_dV_b_GND;
d_T10_dV_p_GND = pParam_B4SOInofffd*d_Vtm_dV_p_GND;
d_T10_dTemp_t_GND = pParam_B4SOInofffd*d_Vtm_dTemp_t_GND;
T10 = (pParam_B4SOInofffd*Vtm);
if ((((VtgsFD_CV-pParam_B4SOIvofffd)/T10)>100.0))
{

d_ExpVtgsFD_CV_dV_e_si = 2.688117142e+43*((T10*d_VtgsFD_CV_dV_e_si-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10);
d_ExpVtgsFD_CV_dV_gi_si = 2.688117142e+43*((T10*d_VtgsFD_CV_dV_gi_si-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10);
d_ExpVtgsFD_CV_dV_di_si = 2.688117142e+43*((T10*d_VtgsFD_CV_dV_di_si-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10);
d_ExpVtgsFD_CV_dV_b_GND = 2.688117142e+43*((T10*d_VtgsFD_CV_dV_b_GND-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10);
d_ExpVtgsFD_CV_dV_p_GND = 2.688117142e+43*((T10*d_VtgsFD_CV_dV_p_GND-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10);
d_ExpVtgsFD_CV_dTemp_t_GND = 2.688117142e+43*((T10*d_VtgsFD_CV_dTemp_t_GND-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10);
ExpVtgsFD_CV = (2.688117142e+43*((1.0+((VtgsFD_CV-pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VtgsFD_CV-pParam_B4SOIvofffd)/T10)<(-100.0)))
{

d_ExpVtgsFD_CV_dTemp_t_GND = d_ExpVtgsFD_CV_dV_p_GND = d_ExpVtgsFD_CV_dV_b_GND = d_ExpVtgsFD_CV_dV_di_si = d_ExpVtgsFD_CV_dV_gi_si = d_ExpVtgsFD_CV_dV_e_si =  0.0;
ExpVtgsFD_CV = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VtgsFD_CV-pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVtgsFD_CV_dV_e_si = (deriv_exp_0_d0*(((T10*d_VtgsFD_CV_dV_e_si-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10)));
d_ExpVtgsFD_CV_dV_gi_si = (deriv_exp_0_d0*(((T10*d_VtgsFD_CV_dV_gi_si-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10)));
d_ExpVtgsFD_CV_dV_di_si = (deriv_exp_0_d0*(((T10*d_VtgsFD_CV_dV_di_si-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10)));
d_ExpVtgsFD_CV_dV_b_GND = (deriv_exp_0_d0*(((T10*d_VtgsFD_CV_dV_b_GND-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10)));
d_ExpVtgsFD_CV_dV_p_GND = (deriv_exp_0_d0*(((T10*d_VtgsFD_CV_dV_p_GND-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10)));
d_ExpVtgsFD_CV_dTemp_t_GND = (deriv_exp_0_d0*(((T10*d_VtgsFD_CV_dTemp_t_GND-(VtgsFD_CV-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10)));
ExpVtgsFD_CV = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVtgsFD_CV));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVtgsFD_CV));

d_VtgseffFD_CV_dV_e_si = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_CV_dV_e_si))+d_T10_dV_e_si*value_log_0);
d_VtgseffFD_CV_dV_gi_si = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_CV_dV_gi_si))+d_T10_dV_gi_si*value_log_0);
d_VtgseffFD_CV_dV_di_si = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_CV_dV_di_si))+d_T10_dV_di_si*value_log_0);
d_VtgseffFD_CV_dV_b_GND = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_CV_dV_b_GND))+d_T10_dV_b_GND*value_log_0);
d_VtgseffFD_CV_dV_p_GND = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_CV_dV_p_GND))+d_T10_dV_p_GND*value_log_0);
d_VtgseffFD_CV_dTemp_t_GND = (T10*(deriv_log_0_d0*(d_ExpVtgsFD_CV_dTemp_t_GND))+d_T10_dTemp_t_GND*value_log_0);
VtgseffFD_CV = (T10*value_log_0);
}

d_VgstFD_CV_dV_e_si = (-d_VthFD_CV_dV_e_si);
d_VgstFD_CV_dV_b_GND = (d_Vgs_eff_dV_b_GND-d_VthFD_CV_dV_b_GND);
d_VgstFD_CV_dV_p_GND = (d_Vgs_eff_dV_p_GND-d_VthFD_CV_dV_p_GND);
d_VgstFD_CV_dTemp_t_GND = (d_Vgs_eff_dTemp_t_GND-d_VthFD_CV_dTemp_t_GND);
d_VgstFD_CV_dV_gi_si = (d_Vgs_eff_dV_gi_si-d_VthFD_CV_dV_gi_si);
d_VgstFD_CV_dV_di_si = (d_Vgs_eff_dV_di_si-d_VthFD_CV_dV_di_si);
VgstFD_CV = (Vgs_eff-VthFD_CV);
if ((((VgstFD_CV-pParam_B4SOIvofffd)/T10)>100.0))
{

d_ExpVgstFD_CV_dV_e_si = 2.688117142e+43*((T10*d_VgstFD_CV_dV_e_si-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10);
d_ExpVgstFD_CV_dV_b_GND = 2.688117142e+43*((T10*d_VgstFD_CV_dV_b_GND-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10);
d_ExpVgstFD_CV_dV_p_GND = 2.688117142e+43*((T10*d_VgstFD_CV_dV_p_GND-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10);
d_ExpVgstFD_CV_dTemp_t_GND = 2.688117142e+43*((T10*d_VgstFD_CV_dTemp_t_GND-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10);
d_ExpVgstFD_CV_dV_gi_si = 2.688117142e+43*((T10*d_VgstFD_CV_dV_gi_si-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10);
d_ExpVgstFD_CV_dV_di_si = 2.688117142e+43*((T10*d_VgstFD_CV_dV_di_si-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10);
ExpVgstFD_CV = (2.688117142e+43*((1.0+((VgstFD_CV-pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VgstFD_CV-pParam_B4SOIvofffd)/T10)<(-100.0)))
{

d_ExpVgstFD_CV_dV_di_si = d_ExpVgstFD_CV_dV_gi_si = d_ExpVgstFD_CV_dTemp_t_GND = d_ExpVgstFD_CV_dV_p_GND = d_ExpVgstFD_CV_dV_b_GND = d_ExpVgstFD_CV_dV_e_si =  0.0;
ExpVgstFD_CV = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VgstFD_CV-pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgstFD_CV_dV_e_si = (deriv_exp_0_d0*(((T10*d_VgstFD_CV_dV_e_si-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_e_si)/T10/T10)));
d_ExpVgstFD_CV_dV_b_GND = (deriv_exp_0_d0*(((T10*d_VgstFD_CV_dV_b_GND-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_b_GND)/T10/T10)));
d_ExpVgstFD_CV_dV_p_GND = (deriv_exp_0_d0*(((T10*d_VgstFD_CV_dV_p_GND-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_p_GND)/T10/T10)));
d_ExpVgstFD_CV_dTemp_t_GND = (deriv_exp_0_d0*(((T10*d_VgstFD_CV_dTemp_t_GND-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dTemp_t_GND)/T10/T10)));
d_ExpVgstFD_CV_dV_gi_si = (deriv_exp_0_d0*(((T10*d_VgstFD_CV_dV_gi_si-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_gi_si)/T10/T10)));
d_ExpVgstFD_CV_dV_di_si = (deriv_exp_0_d0*(((T10*d_VgstFD_CV_dV_di_si-(VgstFD_CV-pParam_B4SOIvofffd)*d_T10_dV_di_si)/T10/T10)));
ExpVgstFD_CV = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVgstFD_CV));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVgstFD_CV));

d_VgsteffFD_CV_dV_e_si = (T10*(deriv_log_0_d0*(d_ExpVgstFD_CV_dV_e_si))+d_T10_dV_e_si*value_log_0);
d_VgsteffFD_CV_dV_gi_si = (T10*(deriv_log_0_d0*(d_ExpVgstFD_CV_dV_gi_si))+d_T10_dV_gi_si*value_log_0);
d_VgsteffFD_CV_dV_di_si = (T10*(deriv_log_0_d0*(d_ExpVgstFD_CV_dV_di_si))+d_T10_dV_di_si*value_log_0);
d_VgsteffFD_CV_dV_b_GND = (T10*(deriv_log_0_d0*(d_ExpVgstFD_CV_dV_b_GND))+d_T10_dV_b_GND*value_log_0);
d_VgsteffFD_CV_dV_p_GND = (T10*(deriv_log_0_d0*(d_ExpVgstFD_CV_dV_p_GND))+d_T10_dV_p_GND*value_log_0);
d_VgsteffFD_CV_dTemp_t_GND = (T10*(deriv_log_0_d0*(d_ExpVgstFD_CV_dTemp_t_GND))+d_T10_dTemp_t_GND*value_log_0);
VgsteffFD_CV = (T10*value_log_0);
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_b_GND = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*d_Vtm_dV_b_GND+(pParam_B4SOImoinfd*pParam_B4SOIk1ox)*d_Vtm_dV_b_GND*Vtm);
d_T1_dV_p_GND = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*d_Vtm_dV_p_GND+(pParam_B4SOImoinfd*pParam_B4SOIk1ox)*d_Vtm_dV_p_GND*Vtm);
d_T1_dTemp_t_GND = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*d_Vtm_dTemp_t_GND+(pParam_B4SOImoinfd*pParam_B4SOIk1ox)*d_Vtm_dTemp_t_GND*Vtm);
T1 = (((pParam_B4SOImoinfd*pParam_B4SOIk1ox)*Vtm)*Vtm);
{
double value_sqrt_0 = sqrt(phi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si =  0.0;
d_T2_dV_e_si = d_VgsteffFD_CV_dV_e_si;
d_T2_dV_gi_si = d_VgsteffFD_CV_dV_gi_si;
d_T2_dV_di_si = d_VgsteffFD_CV_dV_di_si;
d_T2_dV_b_GND = (d_VgsteffFD_CV_dV_b_GND+((2*pParam_B4SOIk1eff)*(deriv_sqrt_0_d0*(d_phi_dV_b_GND))+2*d_pParam_B4SOIk1eff_dV_b_GND*value_sqrt_0));
d_T2_dV_p_GND = (d_VgsteffFD_CV_dV_p_GND+((2*pParam_B4SOIk1eff)*(deriv_sqrt_0_d0*(d_phi_dV_p_GND))+2*d_pParam_B4SOIk1eff_dV_p_GND*value_sqrt_0));
d_T2_dTemp_t_GND = (d_VgsteffFD_CV_dTemp_t_GND+((2*pParam_B4SOIk1eff)*(deriv_sqrt_0_d0*(d_phi_dTemp_t_GND))+2*d_pParam_B4SOIk1eff_dTemp_t_GND*value_sqrt_0));
T2 = (VgsteffFD_CV+((2*pParam_B4SOIk1eff)*value_sqrt_0));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_si = ((T1*VgsteffFD_CV*d_T2_dV_b_si-(VgsteffFD_CV*T2)*d_T1_dV_b_si)/T1/T1);
d_T0_dV_e_si = ((T1*(VgsteffFD_CV*d_T2_dV_e_si+d_VgsteffFD_CV_dV_e_si*T2)-(VgsteffFD_CV*T2)*d_T1_dV_e_si)/T1/T1);
d_T0_dV_gi_si = ((T1*(VgsteffFD_CV*d_T2_dV_gi_si+d_VgsteffFD_CV_dV_gi_si*T2)-(VgsteffFD_CV*T2)*d_T1_dV_gi_si)/T1/T1);
d_T0_dV_di_si = ((T1*(VgsteffFD_CV*d_T2_dV_di_si+d_VgsteffFD_CV_dV_di_si*T2)-(VgsteffFD_CV*T2)*d_T1_dV_di_si)/T1/T1);
d_T0_dV_b_GND = ((T1*(VgsteffFD_CV*d_T2_dV_b_GND+d_VgsteffFD_CV_dV_b_GND*T2)-(VgsteffFD_CV*T2)*d_T1_dV_b_GND)/T1/T1);
d_T0_dV_p_GND = ((T1*(VgsteffFD_CV*d_T2_dV_p_GND+d_VgsteffFD_CV_dV_p_GND*T2)-(VgsteffFD_CV*T2)*d_T1_dV_p_GND)/T1/T1);
d_T0_dTemp_t_GND = ((T1*(VgsteffFD_CV*d_T2_dTemp_t_GND+d_VgsteffFD_CV_dTemp_t_GND*T2)-(VgsteffFD_CV*T2)*d_T1_dTemp_t_GND)/T1/T1);
T0 = (1+((VgsteffFD_CV*T2)/T1));
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);

d_PhiON_CV_dV_e_si = Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_e_si);
d_PhiON_CV_dV_gi_si = Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_gi_si);
d_PhiON_CV_dV_di_si = Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_di_si);
d_PhiON_CV_dV_b_si = Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_si);
d_PhiON_CV_dV_b_GND = (d_phi_dV_b_GND+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_GND)+d_Vtm_dV_b_GND*evaluator_lln_0.getValues(T0)));
d_PhiON_CV_dV_p_GND = (d_phi_dV_p_GND+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dV_p_GND)+d_Vtm_dV_p_GND*evaluator_lln_0.getValues(T0)));
d_PhiON_CV_dTemp_t_GND = (d_phi_dTemp_t_GND+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dTemp_t_GND)+d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues(T0)));
PhiON_CV = (phi+(Vtm*evaluator_lln_0.getValues(T0)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (B4SOIcox/(B4SOIcox+(1.0/((1.0/B4SOIcsi)+(1.0/Cbox)))));

d_PhiFD_CV_dV_e_si = (d_PhiON_CV_dV_e_si-(T0*d_VtgseffFD_CV_dV_e_si+d_T0_dV_e_si*VtgseffFD_CV));
d_PhiFD_CV_dV_gi_si = (d_PhiON_CV_dV_gi_si-(T0*d_VtgseffFD_CV_dV_gi_si+d_T0_dV_gi_si*VtgseffFD_CV));
d_PhiFD_CV_dV_di_si = (d_PhiON_CV_dV_di_si-(T0*d_VtgseffFD_CV_dV_di_si+d_T0_dV_di_si*VtgseffFD_CV));
d_PhiFD_CV_dV_b_si = (d_PhiON_CV_dV_b_si-d_T0_dV_b_si*VtgseffFD_CV);
d_PhiFD_CV_dV_b_GND = (d_PhiON_CV_dV_b_GND-(T0*d_VtgseffFD_CV_dV_b_GND+d_T0_dV_b_GND*VtgseffFD_CV));
d_PhiFD_CV_dV_p_GND = (d_PhiON_CV_dV_p_GND-(T0*d_VtgseffFD_CV_dV_p_GND+d_T0_dV_p_GND*VtgseffFD_CV));
d_PhiFD_CV_dTemp_t_GND = (d_PhiON_CV_dTemp_t_GND-(T0*d_VtgseffFD_CV_dTemp_t_GND+d_T0_dTemp_t_GND*VtgseffFD_CV));
PhiFD_CV = (PhiON_CV-(T0*VtgseffFD_CV));
if ((B4SOIfdMod==0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (((-pParam_B4SOIdvbd1)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T0));
double value_exp_1 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_b_GND))+2*(deriv_exp_1_d0*(d_T0_dV_b_GND)));
d_T1_dV_p_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_p_GND))+2*(deriv_exp_1_d0*(d_T0_dV_p_GND)));
d_T1_dTemp_t_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T0_dTemp_t_GND)));
d_T1_dV_e_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_e_si))+2*(deriv_exp_1_d0*(d_T0_dV_e_si)));
d_T1_dV_gi_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_gi_si))+2*(deriv_exp_1_d0*(d_T0_dV_gi_si)));
d_T1_dV_di_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_di_si))+2*(deriv_exp_1_d0*(d_T0_dV_di_si)));
d_T1_dV_b_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T0_dV_b_si))+2*(deriv_exp_1_d0*(d_T0_dV_b_si)));
T1 = (pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*(d_vbi_dV_b_GND-d_phi_dV_b_GND)+d_T1_dV_b_GND*(vbi-phi));
d_T2_dV_p_GND = (T1*(d_vbi_dV_p_GND-d_phi_dV_p_GND)+d_T1_dV_p_GND*(vbi-phi));
d_T2_dTemp_t_GND = (T1*(d_vbi_dTemp_t_GND-d_phi_dTemp_t_GND)+d_T1_dTemp_t_GND*(vbi-phi));
d_T2_dV_gi_si = d_T1_dV_gi_si*(vbi-phi);
d_T2_dV_di_si = d_T1_dV_di_si*(vbi-phi);
d_T2_dV_e_si = d_T1_dV_e_si*(vbi-phi);
d_T2_dV_b_si = d_T1_dV_b_si*(vbi-phi);
T2 = (T1*(vbi-phi));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = ((0.5*pParam_B4SOIqsi)/B4SOIcsi);

d_Vbs0t_CV_dV_e_si = ((d_PhiFD_CV_dV_e_si-d_T3_dV_e_si)+d_T2_dV_e_si);
d_Vbs0t_CV_dV_gi_si = ((d_PhiFD_CV_dV_gi_si-d_T3_dV_gi_si)+d_T2_dV_gi_si);
d_Vbs0t_CV_dV_di_si = ((d_PhiFD_CV_dV_di_si-d_T3_dV_di_si)+d_T2_dV_di_si);
d_Vbs0t_CV_dV_b_si = ((d_PhiFD_CV_dV_b_si-d_T3_dV_b_si)+d_T2_dV_b_si);
d_Vbs0t_CV_dV_b_GND = ((d_PhiFD_CV_dV_b_GND-d_T3_dV_b_GND)+d_T2_dV_b_GND);
d_Vbs0t_CV_dV_p_GND = ((d_PhiFD_CV_dV_p_GND-d_T3_dV_p_GND)+d_T2_dV_p_GND);
d_Vbs0t_CV_dTemp_t_GND = ((d_PhiFD_CV_dTemp_t_GND-d_T3_dTemp_t_GND)+d_T2_dTemp_t_GND);
Vbs0t_CV = (((PhiFD_CV-T3)+pParam_B4SOIvbsa)+T2);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1+(B4SOIcsi/Cbox));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = (((-pParam_B4SOIdk2b)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T3));
double value_exp_1 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_b_GND))+2*(deriv_exp_1_d0*(d_T3_dV_b_GND)));
d_T5_dV_p_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_p_GND))+2*(deriv_exp_1_d0*(d_T3_dV_p_GND)));
d_T5_dTemp_t_GND = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T3_dTemp_t_GND)));
d_T5_dV_gi_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_gi_si))+2*(deriv_exp_1_d0*(d_T3_dV_gi_si)));
d_T5_dV_di_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_di_si))+2*(deriv_exp_1_d0*(d_T3_dV_di_si)));
d_T5_dV_e_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_e_si))+2*(deriv_exp_1_d0*(d_T3_dV_e_si)));
d_T5_dV_b_si = pParam_B4SOIk2b*((deriv_exp_0_d0*(0.5*d_T3_dV_b_si))+2*(deriv_exp_1_d0*(d_T3_dV_b_si)));
T5 = (pParam_B4SOIk2b*(value_exp_0+(2*value_exp_1)));
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = ((T0*(-d_T5_dV_b_GND)-(pParam_B4SOIk1b-T5)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-d_T5_dV_p_GND)-(pParam_B4SOIk1b-T5)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-d_T5_dTemp_t_GND)-(pParam_B4SOIk1b-T5)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-d_T5_dV_gi_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*(-d_T5_dV_di_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_di_si)/T0/T0);
d_T1_dV_e_si = ((T0*(-d_T5_dV_e_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_e_si)/T0/T0);
d_T1_dV_b_si = ((T0*(-d_T5_dV_b_si)-(pParam_B4SOIk1b-T5)*d_T0_dV_b_si)/T0/T0);
T1 = ((pParam_B4SOIk1b-T5)/T0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*d_Vesfb_dV_b_GND+d_T1_dV_b_GND*Vesfb);
d_T2_dV_p_GND = (T1*d_Vesfb_dV_p_GND+d_T1_dV_p_GND*Vesfb);
d_T2_dTemp_t_GND = (T1*d_Vesfb_dTemp_t_GND+d_T1_dTemp_t_GND*Vesfb);
d_T2_dV_gi_si = d_T1_dV_gi_si*Vesfb;
d_T2_dV_di_si = (T1*d_Vesfb_dV_di_si+d_T1_dV_di_si*Vesfb);
d_T2_dV_e_si = (T1*d_Vesfb_dV_e_si+d_T1_dV_e_si*Vesfb);
d_T2_dV_b_si = d_T1_dV_b_si*Vesfb;
T2 = (T1*Vesfb);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0/(1+(Cbox/B4SOIcsi)));

d_Vbs0_CV_dV_b_GND = ((T0*d_Vbs0t_CV_dV_b_GND+d_T0_dV_b_GND*Vbs0t_CV)+d_T2_dV_b_GND);
d_Vbs0_CV_dV_p_GND = ((T0*d_Vbs0t_CV_dV_p_GND+d_T0_dV_p_GND*Vbs0t_CV)+d_T2_dV_p_GND);
d_Vbs0_CV_dTemp_t_GND = ((T0*d_Vbs0t_CV_dTemp_t_GND+d_T0_dTemp_t_GND*Vbs0t_CV)+d_T2_dTemp_t_GND);
d_Vbs0_CV_dV_e_si = ((T0*d_Vbs0t_CV_dV_e_si+d_T0_dV_e_si*Vbs0t_CV)+d_T2_dV_e_si);
d_Vbs0_CV_dV_gi_si = ((T0*d_Vbs0t_CV_dV_gi_si+d_T0_dV_gi_si*Vbs0t_CV)+d_T2_dV_gi_si);
d_Vbs0_CV_dV_di_si = ((T0*d_Vbs0t_CV_dV_di_si+d_T0_dV_di_si*Vbs0t_CV)+d_T2_dV_di_si);
d_Vbs0_CV_dV_b_si = ((T0*d_Vbs0t_CV_dV_b_si+d_T0_dV_b_si*Vbs0t_CV)+d_T2_dV_b_si);
Vbs0_CV = ((T0*Vbs0t_CV)+T2);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0/((B4SOIcsi+Cbox)+pParam_B4SOIcdsbs));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = (((-pParam_B4SOIdvbd1)*pParam_B4SOIleff)/pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T1));
double value_exp_1 = exp(T1);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_b_GND))+2*(deriv_exp_1_d0*(d_T1_dV_b_GND)));
d_T2_dV_p_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_p_GND))+2*(deriv_exp_1_d0*(d_T1_dV_p_GND)));
d_T2_dTemp_t_GND = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dTemp_t_GND))+2*(deriv_exp_1_d0*(d_T1_dTemp_t_GND)));
d_T2_dV_gi_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_gi_si))+2*(deriv_exp_1_d0*(d_T1_dV_gi_si)));
d_T2_dV_di_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_di_si))+2*(deriv_exp_1_d0*(d_T1_dV_di_si)));
d_T2_dV_e_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_e_si))+2*(deriv_exp_1_d0*(d_T1_dV_e_si)));
d_T2_dV_b_si = pParam_B4SOIdvbd0*((deriv_exp_0_d0*(0.5*d_T1_dV_b_si))+2*(deriv_exp_1_d0*(d_T1_dV_b_si)));
T2 = (pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = d_T2_dV_b_GND*(Vds+pParam_B4SOIvsce);
d_T3_dV_p_GND = d_T2_dV_p_GND*(Vds+pParam_B4SOIvsce);
d_T3_dTemp_t_GND = d_T2_dTemp_t_GND*(Vds+pParam_B4SOIvsce);
d_T3_dV_gi_si = d_T2_dV_gi_si*(Vds+pParam_B4SOIvsce);
d_T3_dV_di_si = (T2*d_Vds_dV_di_si+d_T2_dV_di_si*(Vds+pParam_B4SOIvsce));
d_T3_dV_e_si = d_T2_dV_e_si*(Vds+pParam_B4SOIvsce);
d_T3_dV_b_si = d_T2_dV_b_si*(Vds+pParam_B4SOIvsce);
T3 = (T2*(Vds+pParam_B4SOIvsce));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = ((0.5*pParam_B4SOIqsi)/B4SOIcsi);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = ((B4SOIcsi*T0)*(d_PhiFD_CV_dV_b_GND-d_T4_dV_b_GND)+B4SOIcsi*d_T0_dV_b_GND*((PhiFD_CV-T4)+pParam_B4SOIvbsa));
d_T5_dV_p_GND = ((B4SOIcsi*T0)*(d_PhiFD_CV_dV_p_GND-d_T4_dV_p_GND)+B4SOIcsi*d_T0_dV_p_GND*((PhiFD_CV-T4)+pParam_B4SOIvbsa));
d_T5_dTemp_t_GND = ((B4SOIcsi*T0)*(d_PhiFD_CV_dTemp_t_GND-d_T4_dTemp_t_GND)+B4SOIcsi*d_T0_dTemp_t_GND*((PhiFD_CV-T4)+pParam_B4SOIvbsa));
d_T5_dV_e_si = ((B4SOIcsi*T0)*(d_PhiFD_CV_dV_e_si-d_T4_dV_e_si)+B4SOIcsi*d_T0_dV_e_si*((PhiFD_CV-T4)+pParam_B4SOIvbsa));
d_T5_dV_gi_si = ((B4SOIcsi*T0)*(d_PhiFD_CV_dV_gi_si-d_T4_dV_gi_si)+B4SOIcsi*d_T0_dV_gi_si*((PhiFD_CV-T4)+pParam_B4SOIvbsa));
d_T5_dV_di_si = ((B4SOIcsi*T0)*(d_PhiFD_CV_dV_di_si-d_T4_dV_di_si)+B4SOIcsi*d_T0_dV_di_si*((PhiFD_CV-T4)+pParam_B4SOIvbsa));
d_T5_dV_b_si = ((B4SOIcsi*T0)*(d_PhiFD_CV_dV_b_si-d_T4_dV_b_si)+B4SOIcsi*d_T0_dV_b_si*((PhiFD_CV-T4)+pParam_B4SOIvbsa));
T5 = ((B4SOIcsi*T0)*((PhiFD_CV-T4)+pParam_B4SOIvbsa));

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_b_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_b_GND+pParam_B4SOIcdsbs*d_T0_dV_b_GND*T3);
d_T6_dV_p_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_p_GND+pParam_B4SOIcdsbs*d_T0_dV_p_GND*T3);
d_T6_dTemp_t_GND = ((pParam_B4SOIcdsbs*T0)*d_T3_dTemp_t_GND+pParam_B4SOIcdsbs*d_T0_dTemp_t_GND*T3);
d_T6_dV_e_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_e_si+pParam_B4SOIcdsbs*d_T0_dV_e_si*T3);
d_T6_dV_gi_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_gi_si+pParam_B4SOIcdsbs*d_T0_dV_gi_si*T3);
d_T6_dV_di_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_di_si+pParam_B4SOIcdsbs*d_T0_dV_di_si*T3);
d_T6_dV_b_si = ((pParam_B4SOIcdsbs*T0)*d_T3_dV_b_si+pParam_B4SOIcdsbs*d_T0_dV_b_si*T3);
T6 = ((pParam_B4SOIcdsbs*T0)*T3);

d_Vbs0t_CV_dV_b_GND = (d_T5_dV_b_GND+d_T6_dV_b_GND);
d_Vbs0t_CV_dV_p_GND = (d_T5_dV_p_GND+d_T6_dV_p_GND);
d_Vbs0t_CV_dTemp_t_GND = (d_T5_dTemp_t_GND+d_T6_dTemp_t_GND);
d_Vbs0t_CV_dV_gi_si = (d_T5_dV_gi_si+d_T6_dV_gi_si);
d_Vbs0t_CV_dV_di_si = (d_T5_dV_di_si+d_T6_dV_di_si);
d_Vbs0t_CV_dV_e_si = (d_T5_dV_e_si+d_T6_dV_e_si);
d_Vbs0t_CV_dV_b_si = (d_T5_dV_b_si+d_T6_dV_b_si);
Vbs0t_CV = (T5+T6);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = ((Cbox*T0)*d_Vesfb_dV_b_GND+Cbox*d_T0_dV_b_GND*Vesfb);
d_T7_dV_p_GND = ((Cbox*T0)*d_Vesfb_dV_p_GND+Cbox*d_T0_dV_p_GND*Vesfb);
d_T7_dTemp_t_GND = ((Cbox*T0)*d_Vesfb_dTemp_t_GND+Cbox*d_T0_dTemp_t_GND*Vesfb);
d_T7_dV_e_si = ((Cbox*T0)*d_Vesfb_dV_e_si+Cbox*d_T0_dV_e_si*Vesfb);
d_T7_dV_gi_si = Cbox*d_T0_dV_gi_si*Vesfb;
d_T7_dV_di_si = ((Cbox*T0)*d_Vesfb_dV_di_si+Cbox*d_T0_dV_di_si*Vesfb);
d_T7_dV_b_si = Cbox*d_T0_dV_b_si*Vesfb;
T7 = ((Cbox*T0)*Vesfb);

d_Vbs0_CV_dV_e_si = (d_Vbs0t_CV_dV_e_si+d_T7_dV_e_si);
d_Vbs0_CV_dV_gi_si = (d_Vbs0t_CV_dV_gi_si+d_T7_dV_gi_si);
d_Vbs0_CV_dV_di_si = (d_Vbs0t_CV_dV_di_si+d_T7_dV_di_si);
d_Vbs0_CV_dV_b_si = (d_Vbs0t_CV_dV_b_si+d_T7_dV_b_si);
d_Vbs0_CV_dV_b_GND = (d_Vbs0t_CV_dV_b_GND+d_T7_dV_b_GND);
d_Vbs0_CV_dV_p_GND = (d_Vbs0t_CV_dV_p_GND+d_T7_dV_p_GND);
d_Vbs0_CV_dTemp_t_GND = (d_Vbs0t_CV_dTemp_t_GND+d_T7_dTemp_t_GND);
Vbs0_CV = (Vbs0t_CV+T7);
}
if ((B4SOIsoiMod==2))
{

d_Vbsitf_CV_dV_b_GND = d_Vbs0_CV_dV_b_GND;
d_Vbsitf_CV_dV_p_GND = d_Vbs0_CV_dV_p_GND;
d_Vbsitf_CV_dTemp_t_GND = d_Vbs0_CV_dTemp_t_GND;
d_Vbsitf_CV_dV_e_si = d_Vbs0_CV_dV_e_si;
d_Vbsitf_CV_dV_gi_si = d_Vbs0_CV_dV_gi_si;
d_Vbsitf_CV_dV_di_si = d_Vbs0_CV_dV_di_si;
d_Vbsitf_CV_dV_b_si = d_Vbs0_CV_dV_b_si;
Vbsitf_CV = (Vbs0_CV+0.02);

d_Vbs_dV_b_GND = d_Vbs0_CV_dV_b_GND;
d_Vbs_dV_p_GND = d_Vbs0_CV_dV_p_GND;
d_Vbs_dTemp_t_GND = d_Vbs0_CV_dTemp_t_GND;
d_Vbs_dV_e_si = d_Vbs0_CV_dV_e_si;
d_Vbs_dV_gi_si = d_Vbs0_CV_dV_gi_si;
d_Vbs_dV_di_si = d_Vbs0_CV_dV_di_si;
d_Vbs_dV_b_si = d_Vbs0_CV_dV_b_si;
Vbs = (Vbs0_CV+0.02);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_si = (d_Vbs_dV_b_si-d_Vbs0_CV_dV_b_si);
d_T1_dV_di_si = (d_Vbs_dV_di_si-d_Vbs0_CV_dV_di_si);
d_T1_dV_e_si = (d_Vbs_dV_e_si-d_Vbs0_CV_dV_e_si);
d_T1_dV_b_GND = (d_Vbs_dV_b_GND-d_Vbs0_CV_dV_b_GND);
d_T1_dV_p_GND = (d_Vbs_dV_p_GND-d_Vbs0_CV_dV_p_GND);
d_T1_dTemp_t_GND = (d_Vbs_dTemp_t_GND-d_Vbs0_CV_dTemp_t_GND);
d_T1_dV_gi_si = (d_Vbs_dV_gi_si-d_Vbs0_CV_dV_gi_si);
T1 = ((Vbs-(Vbs0_CV+0.02))-0.01);
{
double value_sqrt_0 = sqrt(((T1*T1)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)));
T2 = value_sqrt_0;
}

d_Vbsitf_CV_dV_b_GND = (d_Vbs0_CV_dV_b_GND+0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND));
d_Vbsitf_CV_dV_p_GND = (d_Vbs0_CV_dV_p_GND+0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND));
d_Vbsitf_CV_dTemp_t_GND = (d_Vbs0_CV_dTemp_t_GND+0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND));
d_Vbsitf_CV_dV_e_si = (d_Vbs0_CV_dV_e_si+0.5*(d_T1_dV_e_si+d_T2_dV_e_si));
d_Vbsitf_CV_dV_gi_si = (d_Vbs0_CV_dV_gi_si+0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si));
d_Vbsitf_CV_dV_di_si = (d_Vbs0_CV_dV_di_si+0.5*(d_T1_dV_di_si+d_T2_dV_di_si));
d_Vbsitf_CV_dV_b_si = (d_Vbs0_CV_dV_b_si+0.5*(d_T1_dV_b_si+d_T2_dV_b_si));
Vbsitf_CV = ((Vbs0_CV+0.02)+(0.5*(T1+T2)));
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (d_Vbs0t_CV_dV_e_si-d_Vbsitf_CV_dV_e_si);
d_T1_dV_gi_si = (d_Vbs0t_CV_dV_gi_si-d_Vbsitf_CV_dV_gi_si);
d_T1_dV_di_si = (d_Vbs0t_CV_dV_di_si-d_Vbsitf_CV_dV_di_si);
d_T1_dV_b_si = (d_Vbs0t_CV_dV_b_si-d_Vbsitf_CV_dV_b_si);
d_T1_dV_b_GND = (d_Vbs0t_CV_dV_b_GND-d_Vbsitf_CV_dV_b_GND);
d_T1_dV_p_GND = (d_Vbs0t_CV_dV_p_GND-d_Vbsitf_CV_dV_p_GND);
d_T1_dTemp_t_GND = (d_Vbs0t_CV_dTemp_t_GND-d_Vbsitf_CV_dTemp_t_GND);
T1 = ((Vbs0t_CV-Vbsitf_CV)-0.005);
{
double value_sqrt_0 = sqrt(((T1*T1)+2.5e-5));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)));
T2 = value_sqrt_0;
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = 0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND);
d_T3_dV_p_GND = 0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND);
d_T3_dTemp_t_GND = 0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND);
d_T3_dV_gi_si = 0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si);
d_T3_dV_di_si = 0.5*(d_T1_dV_di_si+d_T2_dV_di_si);
d_T3_dV_e_si = 0.5*(d_T1_dV_e_si+d_T2_dV_e_si);
d_T3_dV_b_si = 0.5*(d_T1_dV_b_si+d_T2_dV_b_si);
T3 = (0.5*(T1+T2));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (d_T3_dV_b_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_p_GND = (d_T3_dV_p_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dTemp_t_GND = (d_T3_dTemp_t_GND*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_gi_si = (d_T3_dV_gi_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_di_si = (d_T3_dV_di_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_e_si = (d_T3_dV_e_si*B4SOIcsi/pParam_B4SOIqsi);
d_T4_dV_b_si = (d_T3_dV_b_si*B4SOIcsi/pParam_B4SOIqsi);
T4 = ((T3*B4SOIcsi)/pParam_B4SOIqsi);

d_Vbsmos_CV_dV_b_GND = (d_Vbsitf_CV_dV_b_GND-((0.5*T3)*d_T4_dV_b_GND+0.5*d_T3_dV_b_GND*T4));
d_Vbsmos_CV_dV_p_GND = (d_Vbsitf_CV_dV_p_GND-((0.5*T3)*d_T4_dV_p_GND+0.5*d_T3_dV_p_GND*T4));
d_Vbsmos_CV_dTemp_t_GND = (d_Vbsitf_CV_dTemp_t_GND-((0.5*T3)*d_T4_dTemp_t_GND+0.5*d_T3_dTemp_t_GND*T4));
d_Vbsmos_CV_dV_e_si = (d_Vbsitf_CV_dV_e_si-((0.5*T3)*d_T4_dV_e_si+0.5*d_T3_dV_e_si*T4));
d_Vbsmos_CV_dV_gi_si = (d_Vbsitf_CV_dV_gi_si-((0.5*T3)*d_T4_dV_gi_si+0.5*d_T3_dV_gi_si*T4));
d_Vbsmos_CV_dV_di_si = (d_Vbsitf_CV_dV_di_si-((0.5*T3)*d_T4_dV_di_si+0.5*d_T3_dV_di_si*T4));
d_Vbsmos_CV_dV_b_si = (d_Vbsitf_CV_dV_b_si-((0.5*T3)*d_T4_dV_b_si+0.5*d_T3_dV_b_si*T4));
Vbsmos_CV = (Vbsitf_CV-((0.5*T3)*T4));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_si = d_Vbsmos_dV_b_si;
d_T0_dV_di_si = d_Vbsmos_dV_di_si;
d_T0_dV_e_si = d_Vbsmos_dV_e_si;
d_T0_dV_b_GND = d_Vbsmos_dV_b_GND;
d_T0_dV_p_GND = d_Vbsmos_dV_p_GND;
d_T0_dTemp_t_GND = d_Vbsmos_dTemp_t_GND;
d_T0_dV_gi_si = d_Vbsmos_dV_gi_si;
T0 = ((Vbsmos+5)-0.001);
{
double value_sqrt_0 = sqrt(((T0*T0)-(0.004*(-5))));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
T1 = value_sqrt_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = 0.5*(d_T0_dV_b_GND+d_T1_dV_b_GND);
d_T2_dV_p_GND = 0.5*(d_T0_dV_p_GND+d_T1_dV_p_GND);
d_T2_dTemp_t_GND = 0.5*(d_T0_dTemp_t_GND+d_T1_dTemp_t_GND);
d_T2_dV_e_si = 0.5*(d_T0_dV_e_si+d_T1_dV_e_si);
d_T2_dV_gi_si = 0.5*(d_T0_dV_gi_si+d_T1_dV_gi_si);
d_T2_dV_di_si = 0.5*(d_T0_dV_di_si+d_T1_dV_di_si);
d_T2_dV_b_si = 0.5*(d_T0_dV_b_si+d_T1_dV_b_si);
T2 = ((-5)+(0.5*(T0+T1)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 1.5;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND-d_T2_dV_b_GND);
d_T1_dV_p_GND = (d_T0_dV_p_GND-d_T2_dV_p_GND);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND-d_T2_dTemp_t_GND);
d_T1_dV_e_si = (d_T0_dV_e_si-d_T2_dV_e_si);
d_T1_dV_gi_si = (d_T0_dV_gi_si-d_T2_dV_gi_si);
d_T1_dV_di_si = (d_T0_dV_di_si-d_T2_dV_di_si);
d_T1_dV_b_si = (d_T0_dV_b_si-d_T2_dV_b_si);
T1 = ((T0-T2)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+0.008*d_T0_dV_b_GND)));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+0.008*d_T0_dV_p_GND)));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+0.008*d_T0_dTemp_t_GND)));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+0.008*d_T0_dV_gi_si)));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+0.008*d_T0_dV_di_si)));
d_T3_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+0.008*d_T0_dV_e_si)));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+0.008*d_T0_dV_b_si)));
T3 = value_sqrt_0;
}

d_Vbsh_dV_b_GND = (d_T0_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T3_dV_b_GND));
d_Vbsh_dV_p_GND = (d_T0_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T3_dV_p_GND));
d_Vbsh_dTemp_t_GND = (d_T0_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T3_dTemp_t_GND));
d_Vbsh_dV_e_si = (d_T0_dV_e_si-0.5*(d_T1_dV_e_si+d_T3_dV_e_si));
d_Vbsh_dV_gi_si = (d_T0_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T3_dV_gi_si));
d_Vbsh_dV_di_si = (d_T0_dV_di_si-0.5*(d_T1_dV_di_si+d_T3_dV_di_si));
d_Vbsh_dV_b_si = (d_T0_dV_b_si-0.5*(d_T1_dV_b_si+d_T3_dV_b_si));
Vbsh = (T0-(0.5*(T1+T3)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = 0.95*d_phi_dV_b_GND;
d_T0_dV_p_GND = 0.95*d_phi_dV_p_GND;
d_T0_dTemp_t_GND = 0.95*d_phi_dTemp_t_GND;
T0 = (0.95*phi);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND-d_Vbsh_dV_b_GND);
d_T1_dV_p_GND = (d_T0_dV_p_GND-d_Vbsh_dV_p_GND);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND-d_Vbsh_dTemp_t_GND);
d_T1_dV_e_si = (d_T0_dV_e_si-d_Vbsh_dV_e_si);
d_T1_dV_gi_si = (d_T0_dV_gi_si-d_Vbsh_dV_gi_si);
d_T1_dV_di_si = (d_T0_dV_di_si-d_Vbsh_dV_di_si);
d_T1_dV_b_si = (d_T0_dV_b_si-d_Vbsh_dV_b_si);
T1 = ((T0-Vbsh)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+0.008*d_T0_dV_b_GND)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+0.008*d_T0_dV_p_GND)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+0.008*d_T0_dTemp_t_GND)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+0.008*d_T0_dV_gi_si)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+0.008*d_T0_dV_di_si)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+0.008*d_T0_dV_e_si)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+0.008*d_T0_dV_b_si)));
T2 = value_sqrt_0;
}

d_Vbseff_dV_b_GND = (d_T0_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND));
d_Vbseff_dV_p_GND = (d_T0_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND));
d_Vbseff_dTemp_t_GND = (d_T0_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND));
d_Vbseff_dV_e_si = (d_T0_dV_e_si-0.5*(d_T1_dV_e_si+d_T2_dV_e_si));
d_Vbseff_dV_gi_si = (d_T0_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si));
d_Vbseff_dV_di_si = (d_T0_dV_di_si-0.5*(d_T1_dV_di_si+d_T2_dV_di_si));
d_Vbseff_dV_b_si = (d_T0_dV_b_si-0.5*(d_T1_dV_b_si+d_T2_dV_b_si));
Vbseff = (T0-(0.5*(T1+T2)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_si = d_Vbsmos_CV_dV_b_si;
d_T0_dV_di_si = d_Vbsmos_CV_dV_di_si;
d_T0_dV_b_GND = d_Vbsmos_CV_dV_b_GND;
d_T0_dV_p_GND = d_Vbsmos_CV_dV_p_GND;
d_T0_dTemp_t_GND = d_Vbsmos_CV_dTemp_t_GND;
d_T0_dV_e_si = d_Vbsmos_CV_dV_e_si;
d_T0_dV_gi_si = d_Vbsmos_CV_dV_gi_si;
T0 = ((Vbsmos_CV+5)-0.001);
{
double value_sqrt_0 = sqrt(((T0*T0)-(0.004*(-5))));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
T1 = value_sqrt_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = 0.5*(d_T0_dV_b_GND+d_T1_dV_b_GND);
d_T2_dV_p_GND = 0.5*(d_T0_dV_p_GND+d_T1_dV_p_GND);
d_T2_dTemp_t_GND = 0.5*(d_T0_dTemp_t_GND+d_T1_dTemp_t_GND);
d_T2_dV_e_si = 0.5*(d_T0_dV_e_si+d_T1_dV_e_si);
d_T2_dV_gi_si = 0.5*(d_T0_dV_gi_si+d_T1_dV_gi_si);
d_T2_dV_di_si = 0.5*(d_T0_dV_di_si+d_T1_dV_di_si);
d_T2_dV_b_si = 0.5*(d_T0_dV_b_si+d_T1_dV_b_si);
T2 = ((-5)+(0.5*(T0+T1)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 1.5;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND-d_T2_dV_b_GND);
d_T1_dV_p_GND = (d_T0_dV_p_GND-d_T2_dV_p_GND);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND-d_T2_dTemp_t_GND);
d_T1_dV_e_si = (d_T0_dV_e_si-d_T2_dV_e_si);
d_T1_dV_gi_si = (d_T0_dV_gi_si-d_T2_dV_gi_si);
d_T1_dV_di_si = (d_T0_dV_di_si-d_T2_dV_di_si);
d_T1_dV_b_si = (d_T0_dV_b_si-d_T2_dV_b_si);
T1 = ((T0-T2)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+0.008*d_T0_dV_b_GND)));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+0.008*d_T0_dV_p_GND)));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+0.008*d_T0_dTemp_t_GND)));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+0.008*d_T0_dV_gi_si)));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+0.008*d_T0_dV_di_si)));
d_T3_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+0.008*d_T0_dV_e_si)));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+0.008*d_T0_dV_b_si)));
T3 = value_sqrt_0;
}

d_Vbsh_CV_dV_b_GND = (d_T0_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T3_dV_b_GND));
d_Vbsh_CV_dV_p_GND = (d_T0_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T3_dV_p_GND));
d_Vbsh_CV_dTemp_t_GND = (d_T0_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T3_dTemp_t_GND));
d_Vbsh_CV_dV_e_si = (d_T0_dV_e_si-0.5*(d_T1_dV_e_si+d_T3_dV_e_si));
d_Vbsh_CV_dV_gi_si = (d_T0_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T3_dV_gi_si));
d_Vbsh_CV_dV_di_si = (d_T0_dV_di_si-0.5*(d_T1_dV_di_si+d_T3_dV_di_si));
d_Vbsh_CV_dV_b_si = (d_T0_dV_b_si-0.5*(d_T1_dV_b_si+d_T3_dV_b_si));
Vbsh_CV = (T0-(0.5*(T1+T3)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = 0.95*d_phi_dV_b_GND;
d_T0_dV_p_GND = 0.95*d_phi_dV_p_GND;
d_T0_dTemp_t_GND = 0.95*d_phi_dTemp_t_GND;
T0 = (0.95*phi);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND-d_Vbsh_CV_dV_b_GND);
d_T1_dV_p_GND = (d_T0_dV_p_GND-d_Vbsh_CV_dV_p_GND);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND-d_Vbsh_CV_dTemp_t_GND);
d_T1_dV_e_si = (d_T0_dV_e_si-d_Vbsh_CV_dV_e_si);
d_T1_dV_gi_si = (d_T0_dV_gi_si-d_Vbsh_CV_dV_gi_si);
d_T1_dV_di_si = (d_T0_dV_di_si-d_Vbsh_CV_dV_di_si);
d_T1_dV_b_si = (d_T0_dV_b_si-d_Vbsh_CV_dV_b_si);
T1 = ((T0-Vbsh_CV)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+0.008*d_T0_dV_b_GND)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+0.008*d_T0_dV_p_GND)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+0.008*d_T0_dTemp_t_GND)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+0.008*d_T0_dV_gi_si)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+0.008*d_T0_dV_di_si)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+0.008*d_T0_dV_e_si)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+0.008*d_T0_dV_b_si)));
T2 = value_sqrt_0;
}

d_Vbseff_CV_dV_b_GND = (d_T0_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND));
d_Vbseff_CV_dV_p_GND = (d_T0_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND));
d_Vbseff_CV_dTemp_t_GND = (d_T0_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND));
d_Vbseff_CV_dV_e_si = (d_T0_dV_e_si-0.5*(d_T1_dV_e_si+d_T2_dV_e_si));
d_Vbseff_CV_dV_gi_si = (d_T0_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si));
d_Vbseff_CV_dV_di_si = (d_T0_dV_di_si-0.5*(d_T1_dV_di_si+d_T2_dV_di_si));
d_Vbseff_CV_dV_b_si = (d_T0_dV_b_si-0.5*(d_T1_dV_b_si+d_T2_dV_b_si));
Vbseff_CV = (T0-(0.5*(T1+T2)));

d_Phis_dV_e_si = (-d_Vbseff_dV_e_si);
d_Phis_dV_gi_si = (-d_Vbseff_dV_gi_si);
d_Phis_dV_di_si = (-d_Vbseff_dV_di_si);
d_Phis_dV_b_si = (-d_Vbseff_dV_b_si);
d_Phis_dV_b_GND = (d_phi_dV_b_GND-d_Vbseff_dV_b_GND);
d_Phis_dV_p_GND = (d_phi_dV_p_GND-d_Vbseff_dV_p_GND);
d_Phis_dTemp_t_GND = (d_phi_dTemp_t_GND-d_Vbseff_dTemp_t_GND);
Phis = (phi-Vbseff);
{
double value_sqrt_0 = sqrt(Phis);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_sqrtPhis_dV_e_si = (deriv_sqrt_0_d0*(d_Phis_dV_e_si));
d_sqrtPhis_dV_gi_si = (deriv_sqrt_0_d0*(d_Phis_dV_gi_si));
d_sqrtPhis_dV_di_si = (deriv_sqrt_0_d0*(d_Phis_dV_di_si));
d_sqrtPhis_dV_b_GND = (deriv_sqrt_0_d0*(d_Phis_dV_b_GND));
d_sqrtPhis_dV_p_GND = (deriv_sqrt_0_d0*(d_Phis_dV_p_GND));
d_sqrtPhis_dTemp_t_GND = (deriv_sqrt_0_d0*(d_Phis_dTemp_t_GND));
d_sqrtPhis_dV_b_si = (deriv_sqrt_0_d0*(d_Phis_dV_b_si));
sqrtPhis = value_sqrt_0;
}

d_Xdep_dV_e_si = (Xdep0*d_sqrtPhis_dV_e_si/sqrtPhi);
d_Xdep_dV_gi_si = (Xdep0*d_sqrtPhis_dV_gi_si/sqrtPhi);
d_Xdep_dV_di_si = (Xdep0*d_sqrtPhis_dV_di_si/sqrtPhi);
d_Xdep_dV_b_si = (Xdep0*d_sqrtPhis_dV_b_si/sqrtPhi);
d_Xdep_dV_b_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_dV_b_GND+d_Xdep0_dV_b_GND*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dV_b_GND)/sqrtPhi/sqrtPhi);
d_Xdep_dV_p_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_dV_p_GND+d_Xdep0_dV_p_GND*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dV_p_GND)/sqrtPhi/sqrtPhi);
d_Xdep_dTemp_t_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_dTemp_t_GND+d_Xdep0_dTemp_t_GND*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dTemp_t_GND)/sqrtPhi/sqrtPhi);
Xdep = ((Xdep0*sqrtPhis)/sqrtPhi);
B4SOInstar = ((B4SOIvtm/1.60219e-19)*((B4SOIcox+(epssub/Xdep))+pParam_B4SOIcit));
{
double value_sqrt_0 = sqrt(Xdep);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_e_si = (deriv_sqrt_0_d0*(d_Xdep_dV_e_si));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(d_Xdep_dV_gi_si));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(d_Xdep_dV_di_si));
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(d_Xdep_dV_b_GND));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(d_Xdep_dV_p_GND));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(d_Xdep_dTemp_t_GND));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(d_Xdep_dV_b_si));
T3 = value_sqrt_0;
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt2*d_Vbseff_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt2*d_Vbseff_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt2*d_Vbseff_dTemp_t_GND;
d_T0_dV_e_si = pParam_B4SOIdvt2*d_Vbseff_dV_e_si;
d_T0_dV_gi_si = pParam_B4SOIdvt2*d_Vbseff_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt2*d_Vbseff_dV_di_si;
d_T0_dV_b_si = pParam_B4SOIdvt2*d_Vbseff_dV_b_si;
T0 = (pParam_B4SOIdvt2*Vbseff);
if ((T0>=(-0.5)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T1_dV_e_si = d_T0_dV_e_si;
d_T1_dV_gi_si = d_T0_dV_gi_si;
d_T1_dV_di_si = d_T0_dV_di_si;
d_T1_dV_b_si = d_T0_dV_b_si;
T1 = (1.0+T0);
}
else
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (-8.0*d_T0_dV_b_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_p_GND = (-8.0*d_T0_dV_p_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dTemp_t_GND = (-8.0*d_T0_dTemp_t_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_e_si = (-8.0*d_T0_dV_e_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_gi_si = (-8.0*d_T0_dV_gi_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_di_si = (-8.0*d_T0_dV_di_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_b_si = (-8.0*d_T0_dV_b_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = ((1.0+(3.0*T0))*d_T4_dV_b_GND+3.0*d_T0_dV_b_GND*T4);
d_T1_dV_p_GND = ((1.0+(3.0*T0))*d_T4_dV_p_GND+3.0*d_T0_dV_p_GND*T4);
d_T1_dTemp_t_GND = ((1.0+(3.0*T0))*d_T4_dTemp_t_GND+3.0*d_T0_dTemp_t_GND*T4);
d_T1_dV_e_si = ((1.0+(3.0*T0))*d_T4_dV_e_si+3.0*d_T0_dV_e_si*T4);
d_T1_dV_gi_si = ((1.0+(3.0*T0))*d_T4_dV_gi_si+3.0*d_T0_dV_gi_si*T4);
d_T1_dV_di_si = ((1.0+(3.0*T0))*d_T4_dV_di_si+3.0*d_T0_dV_di_si*T4);
d_T1_dV_b_si = ((1.0+(3.0*T0))*d_T4_dV_b_si+3.0*d_T0_dV_b_si*T4);
T1 = ((1.0+(3.0*T0))*T4);
}

d_lt1_dV_b_GND = ((B4SOIfactor1*T3)*d_T1_dV_b_GND+B4SOIfactor1*d_T3_dV_b_GND*T1);
d_lt1_dV_p_GND = ((B4SOIfactor1*T3)*d_T1_dV_p_GND+B4SOIfactor1*d_T3_dV_p_GND*T1);
d_lt1_dTemp_t_GND = ((B4SOIfactor1*T3)*d_T1_dTemp_t_GND+B4SOIfactor1*d_T3_dTemp_t_GND*T1);
d_lt1_dV_gi_si = ((B4SOIfactor1*T3)*d_T1_dV_gi_si+B4SOIfactor1*d_T3_dV_gi_si*T1);
d_lt1_dV_di_si = ((B4SOIfactor1*T3)*d_T1_dV_di_si+B4SOIfactor1*d_T3_dV_di_si*T1);
d_lt1_dV_e_si = ((B4SOIfactor1*T3)*d_T1_dV_e_si+B4SOIfactor1*d_T3_dV_e_si*T1);
d_lt1_dV_b_si = ((B4SOIfactor1*T3)*d_T1_dV_b_si+B4SOIfactor1*d_T3_dV_b_si*T1);
lt1 = ((B4SOIfactor1*T3)*T1);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt2w*d_Vbseff_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt2w*d_Vbseff_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt2w*d_Vbseff_dTemp_t_GND;
d_T0_dV_e_si = pParam_B4SOIdvt2w*d_Vbseff_dV_e_si;
d_T0_dV_gi_si = pParam_B4SOIdvt2w*d_Vbseff_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt2w*d_Vbseff_dV_di_si;
d_T0_dV_b_si = pParam_B4SOIdvt2w*d_Vbseff_dV_b_si;
T0 = (pParam_B4SOIdvt2w*Vbseff);
if ((T0>=(-0.5)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T1_dV_e_si = d_T0_dV_e_si;
d_T1_dV_gi_si = d_T0_dV_gi_si;
d_T1_dV_di_si = d_T0_dV_di_si;
d_T1_dV_b_si = d_T0_dV_b_si;
T1 = (1.0+T0);
}
else
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (-8.0*d_T0_dV_b_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_p_GND = (-8.0*d_T0_dV_p_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dTemp_t_GND = (-8.0*d_T0_dTemp_t_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_e_si = (-8.0*d_T0_dV_e_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_gi_si = (-8.0*d_T0_dV_gi_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_di_si = (-8.0*d_T0_dV_di_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_b_si = (-8.0*d_T0_dV_b_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = ((1.0+(3.0*T0))*d_T4_dV_b_GND+3.0*d_T0_dV_b_GND*T4);
d_T1_dV_p_GND = ((1.0+(3.0*T0))*d_T4_dV_p_GND+3.0*d_T0_dV_p_GND*T4);
d_T1_dTemp_t_GND = ((1.0+(3.0*T0))*d_T4_dTemp_t_GND+3.0*d_T0_dTemp_t_GND*T4);
d_T1_dV_e_si = ((1.0+(3.0*T0))*d_T4_dV_e_si+3.0*d_T0_dV_e_si*T4);
d_T1_dV_gi_si = ((1.0+(3.0*T0))*d_T4_dV_gi_si+3.0*d_T0_dV_gi_si*T4);
d_T1_dV_di_si = ((1.0+(3.0*T0))*d_T4_dV_di_si+3.0*d_T0_dV_di_si*T4);
d_T1_dV_b_si = ((1.0+(3.0*T0))*d_T4_dV_b_si+3.0*d_T0_dV_b_si*T4);
T1 = ((1.0+(3.0*T0))*T4);
}

d_ltw_dV_b_GND = ((B4SOIfactor1*T3)*d_T1_dV_b_GND+B4SOIfactor1*d_T3_dV_b_GND*T1);
d_ltw_dV_p_GND = ((B4SOIfactor1*T3)*d_T1_dV_p_GND+B4SOIfactor1*d_T3_dV_p_GND*T1);
d_ltw_dTemp_t_GND = ((B4SOIfactor1*T3)*d_T1_dTemp_t_GND+B4SOIfactor1*d_T3_dTemp_t_GND*T1);
d_ltw_dV_gi_si = ((B4SOIfactor1*T3)*d_T1_dV_gi_si+B4SOIfactor1*d_T3_dV_gi_si*T1);
d_ltw_dV_di_si = ((B4SOIfactor1*T3)*d_T1_dV_di_si+B4SOIfactor1*d_T3_dV_di_si*T1);
d_ltw_dV_e_si = ((B4SOIfactor1*T3)*d_T1_dV_e_si+B4SOIfactor1*d_T3_dV_e_si*T1);
d_ltw_dV_b_si = ((B4SOIfactor1*T3)*d_T1_dV_b_si+B4SOIfactor1*d_T3_dV_b_si*T1);
ltw = ((B4SOIfactor1*T3)*T1);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_b_GND/lt1/lt1);
d_T0_dV_p_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_p_GND/lt1/lt1);
d_T0_dTemp_t_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dTemp_t_GND/lt1/lt1);
d_T0_dV_gi_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_gi_si/lt1/lt1);
d_T0_dV_di_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_di_si/lt1/lt1);
d_T0_dV_e_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_e_si/lt1/lt1);
d_T0_dV_b_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_dV_b_si/lt1/lt1);
T0 = ((((-0.5)*pParam_B4SOIdvt1)*Leff)/lt1);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T1 = value_exp_0;
}

d_Theta0_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_Theta0_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_Theta0_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_Theta0_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_e_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_e_si/Xdep/Xdep);
d_T2_dV_gi_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_gi_si/Xdep/Xdep);
d_T2_dV_di_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_di_si/Xdep/Xdep);
d_T2_dV_b_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_b_GND/Xdep/Xdep);
d_T2_dV_p_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_p_GND/Xdep/Xdep);
d_T2_dTemp_t_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dTemp_t_GND/Xdep/Xdep);
d_T2_dV_b_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_dV_b_si/Xdep/Xdep);
T2 = ((pParam_B4SOInfactor*epssub)/Xdep);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = pParam_B4SOIcdscb*d_Vbseff_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIcdscb*d_Vbseff_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIcdscb*d_Vbseff_dTemp_t_GND;
d_T3_dV_e_si = pParam_B4SOIcdscb*d_Vbseff_dV_e_si;
d_T3_dV_gi_si = pParam_B4SOIcdscb*d_Vbseff_dV_gi_si;
d_T3_dV_di_si = (pParam_B4SOIcdscb*d_Vbseff_dV_di_si+pParam_B4SOIcdscd*d_Vds_dV_di_si);
d_T3_dV_b_si = pParam_B4SOIcdscb*d_Vbseff_dV_b_si;
T3 = ((pParam_B4SOIcdsc+(pParam_B4SOIcdscb*Vbseff))+(pParam_B4SOIcdscd*Vds));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = ((d_T2_dV_b_GND+(T3*d_Theta0_dV_b_GND+d_T3_dV_b_GND*Theta0))/B4SOIcox);
d_T4_dV_p_GND = ((d_T2_dV_p_GND+(T3*d_Theta0_dV_p_GND+d_T3_dV_p_GND*Theta0))/B4SOIcox);
d_T4_dTemp_t_GND = ((d_T2_dTemp_t_GND+(T3*d_Theta0_dTemp_t_GND+d_T3_dTemp_t_GND*Theta0))/B4SOIcox);
d_T4_dV_gi_si = ((d_T2_dV_gi_si+(T3*d_Theta0_dV_gi_si+d_T3_dV_gi_si*Theta0))/B4SOIcox);
d_T4_dV_di_si = ((d_T2_dV_di_si+(T3*d_Theta0_dV_di_si+d_T3_dV_di_si*Theta0))/B4SOIcox);
d_T4_dV_e_si = ((d_T2_dV_e_si+(T3*d_Theta0_dV_e_si+d_T3_dV_e_si*Theta0))/B4SOIcox);
d_T4_dV_b_si = ((d_T2_dV_b_si+(T3*d_Theta0_dV_b_si+d_T3_dV_b_si*Theta0))/B4SOIcox);
T4 = (((T2+(T3*Theta0))+pParam_B4SOIcit)/B4SOIcox);
if ((T4>=(-0.5)))
{

d_n_dV_b_GND = d_T4_dV_b_GND;
d_n_dV_p_GND = d_T4_dV_p_GND;
d_n_dTemp_t_GND = d_T4_dTemp_t_GND;
d_n_dV_gi_si = d_T4_dV_gi_si;
d_n_dV_di_si = d_T4_dV_di_si;
d_n_dV_e_si = d_T4_dV_e_si;
d_n_dV_b_si = d_T4_dV_b_si;
n = (1.0+T4);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-8.0*d_T4_dV_b_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_p_GND = (-8.0*d_T4_dV_p_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dTemp_t_GND = (-8.0*d_T4_dTemp_t_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_gi_si = (-8.0*d_T4_dV_gi_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_di_si = (-8.0*d_T4_dV_di_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_e_si = (-8.0*d_T4_dV_e_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_b_si = (-8.0*d_T4_dV_b_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
T0 = (1.0/(3.0+(8.0*T4)));

d_n_dV_b_GND = ((1.0+(3.0*T4))*d_T0_dV_b_GND+3.0*d_T4_dV_b_GND*T0);
d_n_dV_p_GND = ((1.0+(3.0*T4))*d_T0_dV_p_GND+3.0*d_T4_dV_p_GND*T0);
d_n_dTemp_t_GND = ((1.0+(3.0*T4))*d_T0_dTemp_t_GND+3.0*d_T4_dTemp_t_GND*T0);
d_n_dV_gi_si = ((1.0+(3.0*T4))*d_T0_dV_gi_si+3.0*d_T4_dV_gi_si*T0);
d_n_dV_di_si = ((1.0+(3.0*T4))*d_T0_dV_di_si+3.0*d_T4_dV_di_si*T0);
d_n_dV_e_si = ((1.0+(3.0*T4))*d_T0_dV_e_si+3.0*d_T4_dV_e_si*T0);
d_n_dV_b_si = ((1.0+(3.0*T4))*d_T0_dV_b_si+3.0*d_T4_dV_b_si*T0);
n = ((1.0+(3.0*T4))*T0);
}
if ((pParam_B4SOIdvtp0>0.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = (-pParam_B4SOIdvtp1)*d_Vds_dV_di_si;
T0 = ((-pParam_B4SOIdvtp1)*Vds);
if ((T0<(-100.0)))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T2_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T2_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T2_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T2_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T2_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T2_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T2 = value_exp_0;
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = pParam_B4SOIdvtp0*d_T2_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIdvtp0*d_T2_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIdvtp0*d_T2_dTemp_t_GND;
d_T3_dV_gi_si = pParam_B4SOIdvtp0*d_T2_dV_gi_si;
d_T3_dV_di_si = pParam_B4SOIdvtp0*d_T2_dV_di_si;
d_T3_dV_e_si = pParam_B4SOIdvtp0*d_T2_dV_e_si;
d_T3_dV_b_si = pParam_B4SOIdvtp0*d_T2_dV_b_si;
T3 = (Leff+(pParam_B4SOIdvtp0*(1.0+T2)));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((Leff/T3));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_gi_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_gi_si/T3/T3));
d_T4_dV_di_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_di_si/T3/T3));
d_T4_dV_e_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_e_si/T3/T3));
d_T4_dV_b_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_b_si/T3/T3));
d_T4_dV_b_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_b_GND/T3/T3))+d_Vtm_dV_b_GND*evaluator_lln_0.getValues((Leff/T3)));
d_T4_dV_p_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_p_GND/T3/T3))+d_Vtm_dV_p_GND*evaluator_lln_0.getValues((Leff/T3)));
d_T4_dTemp_t_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dTemp_t_GND/T3/T3))+d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues((Leff/T3)));
T4 = (Vtm*evaluator_lln_0.getValues((Leff/T3)));
}

d_DITS_Sft_dV_b_GND = (n*d_T4_dV_b_GND+d_n_dV_b_GND*T4);
d_DITS_Sft_dV_p_GND = (n*d_T4_dV_p_GND+d_n_dV_p_GND*T4);
d_DITS_Sft_dTemp_t_GND = (n*d_T4_dTemp_t_GND+d_n_dTemp_t_GND*T4);
d_DITS_Sft_dV_gi_si = (n*d_T4_dV_gi_si+d_n_dV_gi_si*T4);
d_DITS_Sft_dV_di_si = (n*d_T4_dV_di_si+d_n_dV_di_si*T4);
d_DITS_Sft_dV_e_si = (n*d_T4_dV_e_si+d_n_dV_e_si*T4);
d_DITS_Sft_dV_b_si = (n*d_T4_dV_b_si+d_n_dV_b_si*T4);
DITS_Sft = (n*T4);
}
else
{

d_DITS_Sft_dV_b_si = d_DITS_Sft_dV_e_si = d_DITS_Sft_dV_di_si = d_DITS_Sft_dV_gi_si = d_DITS_Sft_dTemp_t_GND = d_DITS_Sft_dV_p_GND = d_DITS_Sft_dV_b_GND =  0.0;
DITS_Sft = 0.0;
}

d_B4SOIthetavth_dV_b_GND = pParam_B4SOIdvt0*d_Theta0_dV_b_GND;
d_B4SOIthetavth_dV_p_GND = pParam_B4SOIdvt0*d_Theta0_dV_p_GND;
d_B4SOIthetavth_dTemp_t_GND = pParam_B4SOIdvt0*d_Theta0_dTemp_t_GND;
d_B4SOIthetavth_dV_gi_si = pParam_B4SOIdvt0*d_Theta0_dV_gi_si;
d_B4SOIthetavth_dV_di_si = pParam_B4SOIdvt0*d_Theta0_dV_di_si;
d_B4SOIthetavth_dV_e_si = pParam_B4SOIdvt0*d_Theta0_dV_e_si;
d_B4SOIthetavth_dV_b_si = pParam_B4SOIdvt0*d_Theta0_dV_b_si;
B4SOIthetavth = (pParam_B4SOIdvt0*Theta0);

d_Delt_vth_dV_b_GND = (B4SOIthetavth*d_V0_dV_b_GND+d_B4SOIthetavth_dV_b_GND*V0);
d_Delt_vth_dV_p_GND = (B4SOIthetavth*d_V0_dV_p_GND+d_B4SOIthetavth_dV_p_GND*V0);
d_Delt_vth_dTemp_t_GND = (B4SOIthetavth*d_V0_dTemp_t_GND+d_B4SOIthetavth_dTemp_t_GND*V0);
d_Delt_vth_dV_gi_si = d_B4SOIthetavth_dV_gi_si*V0;
d_Delt_vth_dV_di_si = d_B4SOIthetavth_dV_di_si*V0;
d_Delt_vth_dV_e_si = d_B4SOIthetavth_dV_e_si*V0;
d_Delt_vth_dV_b_si = d_B4SOIthetavth_dV_b_si*V0;
Delt_vth = (B4SOIthetavth*V0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_b_GND/ltw/ltw);
d_T0_dV_p_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_p_GND/ltw/ltw);
d_T0_dTemp_t_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dTemp_t_GND/ltw/ltw);
d_T0_dV_gi_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_gi_si/ltw/ltw);
d_T0_dV_di_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_di_si/ltw/ltw);
d_T0_dV_e_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_e_si/ltw/ltw);
d_T0_dV_b_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_dV_b_si/ltw/ltw);
T0 = (((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)/ltw);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T1 = value_exp_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_T2_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_T2_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt0w*d_T2_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt0w*d_T2_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt0w*d_T2_dTemp_t_GND;
d_T0_dV_gi_si = pParam_B4SOIdvt0w*d_T2_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt0w*d_T2_dV_di_si;
d_T0_dV_e_si = pParam_B4SOIdvt0w*d_T2_dV_e_si;
d_T0_dV_b_si = pParam_B4SOIdvt0w*d_T2_dV_b_si;
T0 = (pParam_B4SOIdvt0w*T2);

d_DeltVthw_dV_b_GND = (T0*d_V0_dV_b_GND+d_T0_dV_b_GND*V0);
d_DeltVthw_dV_p_GND = (T0*d_V0_dV_p_GND+d_T0_dV_p_GND*V0);
d_DeltVthw_dTemp_t_GND = (T0*d_V0_dTemp_t_GND+d_T0_dTemp_t_GND*V0);
d_DeltVthw_dV_e_si = d_T0_dV_e_si*V0;
d_DeltVthw_dV_gi_si = d_T0_dV_gi_si*V0;
d_DeltVthw_dV_di_si = d_T0_dV_di_si*V0;
d_DeltVthw_dV_b_si = d_T0_dV_b_si*V0;
DeltVthw = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpe0/Leff)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = value_sqrt_0;
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = pParam_B4SOIkt2*d_Vbseff_dV_b_GND;
d_T1_dV_p_GND = pParam_B4SOIkt2*d_Vbseff_dV_p_GND;
d_T1_dTemp_t_GND = pParam_B4SOIkt2*d_Vbseff_dTemp_t_GND;
d_T1_dV_e_si = pParam_B4SOIkt2*d_Vbseff_dV_e_si;
d_T1_dV_gi_si = pParam_B4SOIkt2*d_Vbseff_dV_gi_si;
d_T1_dV_di_si = pParam_B4SOIkt2*d_Vbseff_dV_di_si;
d_T1_dV_b_si = pParam_B4SOIkt2*d_Vbseff_dV_b_si;
T1 = ((pParam_B4SOIkt1+(pParam_B4SOIkt1l/Leff))+(pParam_B4SOIkt2*Vbseff));

d_DeltVthtemp_dV_b_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_b_GND+pParam_B4SOIk1ox*d_T0_dV_b_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_b_GND+d_T1_dV_b_GND*TempRatioMinus1));
d_DeltVthtemp_dV_p_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_p_GND+pParam_B4SOIk1ox*d_T0_dV_p_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_p_GND+d_T1_dV_p_GND*TempRatioMinus1));
d_DeltVthtemp_dTemp_t_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dTemp_t_GND+pParam_B4SOIk1ox*d_T0_dTemp_t_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dTemp_t_GND+d_T1_dTemp_t_GND*TempRatioMinus1));
d_DeltVthtemp_dV_e_si = (pParam_B4SOIk1ox*d_T0_dV_e_si*sqrtPhi+d_T1_dV_e_si*TempRatioMinus1);
d_DeltVthtemp_dV_gi_si = (pParam_B4SOIk1ox*d_T0_dV_gi_si*sqrtPhi+d_T1_dV_gi_si*TempRatioMinus1);
d_DeltVthtemp_dV_di_si = (pParam_B4SOIk1ox*d_T0_dV_di_si*sqrtPhi+d_T1_dV_di_si*TempRatioMinus1);
d_DeltVthtemp_dV_b_si = (pParam_B4SOIk1ox*d_T0_dV_b_si*sqrtPhi+d_T1_dV_b_si*TempRatioMinus1);
DeltVthtemp = (((pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));

d_tmp2_dV_b_si = d_tmp2_dV_e_si = d_tmp2_dV_di_si = d_tmp2_dV_gi_si =  0.0;
d_tmp2_dV_b_GND = (toxe*d_phi_dV_b_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
d_tmp2_dV_p_GND = (toxe*d_phi_dV_p_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
d_tmp2_dTemp_t_GND = (toxe*d_phi_dTemp_t_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
tmp2 = ((toxe*phi)/(pParam_B4SOIweff+pParam_B4SOIw0));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = pParam_B4SOIetab*d_Vbseff_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIetab*d_Vbseff_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIetab*d_Vbseff_dTemp_t_GND;
d_T3_dV_e_si = pParam_B4SOIetab*d_Vbseff_dV_e_si;
d_T3_dV_gi_si = pParam_B4SOIetab*d_Vbseff_dV_gi_si;
d_T3_dV_di_si = pParam_B4SOIetab*d_Vbseff_dV_di_si;
d_T3_dV_b_si = pParam_B4SOIetab*d_Vbseff_dV_b_si;
T3 = (here_B4SOIeta0+(pParam_B4SOIetab*Vbseff));
if ((T3<1.0e-4))
{

d_T9_dV_b_GND = (-(-2.0e4*d_T3_dV_b_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_p_GND = (-(-2.0e4*d_T3_dV_p_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dTemp_t_GND = (-(-2.0e4*d_T3_dTemp_t_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_gi_si = (-(-2.0e4*d_T3_dV_gi_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_di_si = (-(-2.0e4*d_T3_dV_di_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_e_si = (-(-2.0e4*d_T3_dV_e_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_b_si = (-(-2.0e4*d_T3_dV_b_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = ((2.0e-4-T3)*d_T9_dV_b_GND+(-d_T3_dV_b_GND)*T9);
d_T3_dV_p_GND = ((2.0e-4-T3)*d_T9_dV_p_GND+(-d_T3_dV_p_GND)*T9);
d_T3_dTemp_t_GND = ((2.0e-4-T3)*d_T9_dTemp_t_GND+(-d_T3_dTemp_t_GND)*T9);
d_T3_dV_gi_si = ((2.0e-4-T3)*d_T9_dV_gi_si+(-d_T3_dV_gi_si)*T9);
d_T3_dV_di_si = ((2.0e-4-T3)*d_T9_dV_di_si+(-d_T3_dV_di_si)*T9);
d_T3_dV_e_si = ((2.0e-4-T3)*d_T9_dV_e_si+(-d_T3_dV_e_si)*T9);
d_T3_dV_b_si = ((2.0e-4-T3)*d_T9_dV_b_si+(-d_T3_dV_b_si)*T9);
T3 = ((2.0e-4-T3)*T9);
}

d_DIBL_Sft_dV_b_GND = (T3*d_theta0vb0_dV_b_GND+d_T3_dV_b_GND*theta0vb0)*Vds;
d_DIBL_Sft_dV_p_GND = (T3*d_theta0vb0_dV_p_GND+d_T3_dV_p_GND*theta0vb0)*Vds;
d_DIBL_Sft_dTemp_t_GND = (T3*d_theta0vb0_dTemp_t_GND+d_T3_dTemp_t_GND*theta0vb0)*Vds;
d_DIBL_Sft_dV_gi_si = d_T3_dV_gi_si*theta0vb0*Vds;
d_DIBL_Sft_dV_di_si = ((T3*theta0vb0)*d_Vds_dV_di_si+d_T3_dV_di_si*theta0vb0*Vds);
d_DIBL_Sft_dV_e_si = d_T3_dV_e_si*theta0vb0*Vds;
d_DIBL_Sft_dV_b_si = d_T3_dV_b_si*theta0vb0*Vds;
DIBL_Sft = ((T3*theta0vb0)*Vds);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpeb/Leff)));
Lpe_Vb = value_sqrt_0;
}

d_T9_dV_b_si = d_T9_dV_e_si = d_T9_dV_di_si = d_T9_dV_gi_si =  0.0;
d_T9_dV_b_GND = (-2.2361*d_sqrtPhi_dV_b_GND/sqrtPhi/sqrtPhi);
d_T9_dV_p_GND = (-2.2361*d_sqrtPhi_dV_p_GND/sqrtPhi/sqrtPhi);
d_T9_dTemp_t_GND = (-2.2361*d_sqrtPhi_dTemp_t_GND/sqrtPhi/sqrtPhi);
T9 = (2.2361/sqrtPhi);

d_sqrtPhisExt_dV_e_si = (d_sqrtPhis_dV_e_si-(T9*(d_Vbsh_dV_e_si-d_Vbseff_dV_e_si)+d_T9_dV_e_si*(Vbsh-Vbseff)));
d_sqrtPhisExt_dV_gi_si = (d_sqrtPhis_dV_gi_si-(T9*(d_Vbsh_dV_gi_si-d_Vbseff_dV_gi_si)+d_T9_dV_gi_si*(Vbsh-Vbseff)));
d_sqrtPhisExt_dV_di_si = (d_sqrtPhis_dV_di_si-(T9*(d_Vbsh_dV_di_si-d_Vbseff_dV_di_si)+d_T9_dV_di_si*(Vbsh-Vbseff)));
d_sqrtPhisExt_dV_b_GND = (d_sqrtPhis_dV_b_GND-(T9*(d_Vbsh_dV_b_GND-d_Vbseff_dV_b_GND)+d_T9_dV_b_GND*(Vbsh-Vbseff)));
d_sqrtPhisExt_dV_p_GND = (d_sqrtPhis_dV_p_GND-(T9*(d_Vbsh_dV_p_GND-d_Vbseff_dV_p_GND)+d_T9_dV_p_GND*(Vbsh-Vbseff)));
d_sqrtPhisExt_dTemp_t_GND = (d_sqrtPhis_dTemp_t_GND-(T9*(d_Vbsh_dTemp_t_GND-d_Vbseff_dTemp_t_GND)+d_T9_dTemp_t_GND*(Vbsh-Vbseff)));
d_sqrtPhisExt_dV_b_si = (d_sqrtPhis_dV_b_si-(T9*(d_Vbsh_dV_b_si-d_Vbseff_dV_b_si)+d_T9_dV_b_si*(Vbsh-Vbseff)));
sqrtPhisExt = (sqrtPhis-(T9*(Vbsh-Vbseff)));
{
double value_exp_0 = exp(((2.0*pParam_B4SOIdvtp4)*Vds));
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = (deriv_exp_0_d0*((2.0*pParam_B4SOIdvtp4)*d_Vds_dV_di_si));
T0 = value_exp_0;
}

d_DITS_Sft2_dV_b_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_b_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_b_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_p_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_p_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_p_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dTemp_t_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dTemp_t_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dTemp_t_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_e_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_e_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_e_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_gi_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_gi_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_gi_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_di_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_di_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_di_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_b_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_b_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_b_si)/(T0+1)/(T0+1));
DITS_Sft2 = ((pParam_B4SOIdvtp2factor*(T0-1))/(T0+1));

d_Vth_dV_e_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_dV_e_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_dV_e_si)-d_Delt_vth_dV_e_si)-d_DeltVthw_dV_e_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*d_tmp2_dV_e_si+pParam_B4SOIk3b*d_Vbseff_dV_e_si*tmp2))+d_DeltVthtemp_dV_e_si)-d_DIBL_Sft_dV_e_si)-d_DITS_Sft_dV_e_si)-d_DITS_Sft2_dV_e_si);
d_Vth_dV_gi_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_dV_gi_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_dV_gi_si)-d_Delt_vth_dV_gi_si)-d_DeltVthw_dV_gi_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*d_tmp2_dV_gi_si+pParam_B4SOIk3b*d_Vbseff_dV_gi_si*tmp2))+d_DeltVthtemp_dV_gi_si)-d_DIBL_Sft_dV_gi_si)-d_DITS_Sft_dV_gi_si)-d_DITS_Sft2_dV_gi_si);
d_Vth_dV_di_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_dV_di_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_dV_di_si)-d_Delt_vth_dV_di_si)-d_DeltVthw_dV_di_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*d_tmp2_dV_di_si+pParam_B4SOIk3b*d_Vbseff_dV_di_si*tmp2))+d_DeltVthtemp_dV_di_si)-d_DIBL_Sft_dV_di_si)-d_DITS_Sft_dV_di_si)-d_DITS_Sft2_dV_di_si);
d_Vth_dV_b_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_dV_b_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_dV_b_si)-d_Delt_vth_dV_b_si)-d_DeltVthw_dV_b_si)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*d_tmp2_dV_b_si+pParam_B4SOIk3b*d_Vbseff_dV_b_si*tmp2))+d_DeltVthtemp_dV_b_si)-d_DIBL_Sft_dV_b_si)-d_DITS_Sft_dV_b_si)-d_DITS_Sft2_dV_b_si);
d_Vth_dV_b_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_b_GND+(pParam_B4SOIk1ox*d_sqrtPhisExt_dV_b_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbseff_dV_b_GND)-d_Delt_vth_dV_b_GND)-d_DeltVthw_dV_b_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*d_tmp2_dV_b_GND+pParam_B4SOIk3b*d_Vbseff_dV_b_GND*tmp2))+d_DeltVthtemp_dV_b_GND)-d_DIBL_Sft_dV_b_GND)-d_DITS_Sft_dV_b_GND)-d_DITS_Sft2_dV_b_GND);
d_Vth_dV_p_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_p_GND+(pParam_B4SOIk1ox*d_sqrtPhisExt_dV_p_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbseff_dV_p_GND)-d_Delt_vth_dV_p_GND)-d_DeltVthw_dV_p_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*d_tmp2_dV_p_GND+pParam_B4SOIk3b*d_Vbseff_dV_p_GND*tmp2))+d_DeltVthtemp_dV_p_GND)-d_DIBL_Sft_dV_p_GND)-d_DITS_Sft_dV_p_GND)-d_DITS_Sft2_dV_p_GND);
d_Vth_dTemp_t_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dTemp_t_GND+(pParam_B4SOIk1ox*d_sqrtPhisExt_dTemp_t_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbseff_dTemp_t_GND)-d_Delt_vth_dTemp_t_GND)-d_DeltVthw_dTemp_t_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*d_tmp2_dTemp_t_GND+pParam_B4SOIk3b*d_Vbseff_dTemp_t_GND*tmp2))+d_DeltVthtemp_dTemp_t_GND)-d_DIBL_Sft_dTemp_t_GND)-d_DITS_Sft_dTemp_t_GND)-d_DITS_Sft2_dTemp_t_GND);
Vth = ((((((((((B4SOItype*here_B4SOIvth0)+(((pParam_B4SOIk1ox*sqrtPhisExt)-(pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(here_B4SOIk2ox*Vbseff))-Delt_vth)-DeltVthw)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff))*tmp2))+DeltVthtemp)-DIBL_Sft)-DITS_Sft)-DITS_Sft2);
VTH = Vth;

d_Phis_CV_dV_e_si = (-d_Vbseff_CV_dV_e_si);
d_Phis_CV_dV_gi_si = (-d_Vbseff_CV_dV_gi_si);
d_Phis_CV_dV_di_si = (-d_Vbseff_CV_dV_di_si);
d_Phis_CV_dV_b_si = (-d_Vbseff_CV_dV_b_si);
d_Phis_CV_dV_b_GND = (d_phi_dV_b_GND-d_Vbseff_CV_dV_b_GND);
d_Phis_CV_dV_p_GND = (d_phi_dV_p_GND-d_Vbseff_CV_dV_p_GND);
d_Phis_CV_dTemp_t_GND = (d_phi_dTemp_t_GND-d_Vbseff_CV_dTemp_t_GND);
Phis_CV = (phi-Vbseff_CV);
{
double value_sqrt_0 = sqrt(Phis_CV);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_sqrtPhis_CV_dV_e_si = (deriv_sqrt_0_d0*(d_Phis_CV_dV_e_si));
d_sqrtPhis_CV_dV_gi_si = (deriv_sqrt_0_d0*(d_Phis_CV_dV_gi_si));
d_sqrtPhis_CV_dV_di_si = (deriv_sqrt_0_d0*(d_Phis_CV_dV_di_si));
d_sqrtPhis_CV_dV_b_si = (deriv_sqrt_0_d0*(d_Phis_CV_dV_b_si));
d_sqrtPhis_CV_dV_b_GND = (deriv_sqrt_0_d0*(d_Phis_CV_dV_b_GND));
d_sqrtPhis_CV_dV_p_GND = (deriv_sqrt_0_d0*(d_Phis_CV_dV_p_GND));
d_sqrtPhis_CV_dTemp_t_GND = (deriv_sqrt_0_d0*(d_Phis_CV_dTemp_t_GND));
sqrtPhis_CV = value_sqrt_0;
}

d_Xdep_CV_dV_e_si = (Xdep0*d_sqrtPhis_CV_dV_e_si/sqrtPhi);
d_Xdep_CV_dV_gi_si = (Xdep0*d_sqrtPhis_CV_dV_gi_si/sqrtPhi);
d_Xdep_CV_dV_di_si = (Xdep0*d_sqrtPhis_CV_dV_di_si/sqrtPhi);
d_Xdep_CV_dV_b_si = (Xdep0*d_sqrtPhis_CV_dV_b_si/sqrtPhi);
d_Xdep_CV_dV_b_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_CV_dV_b_GND+d_Xdep0_dV_b_GND*sqrtPhis_CV)-(Xdep0*sqrtPhis_CV)*d_sqrtPhi_dV_b_GND)/sqrtPhi/sqrtPhi);
d_Xdep_CV_dV_p_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_CV_dV_p_GND+d_Xdep0_dV_p_GND*sqrtPhis_CV)-(Xdep0*sqrtPhis_CV)*d_sqrtPhi_dV_p_GND)/sqrtPhi/sqrtPhi);
d_Xdep_CV_dTemp_t_GND = ((sqrtPhi*(Xdep0*d_sqrtPhis_CV_dTemp_t_GND+d_Xdep0_dTemp_t_GND*sqrtPhis_CV)-(Xdep0*sqrtPhis_CV)*d_sqrtPhi_dTemp_t_GND)/sqrtPhi/sqrtPhi);
Xdep_CV = ((Xdep0*sqrtPhis_CV)/sqrtPhi);
B4SOInstar = ((B4SOIvtm/1.60219e-19)*((B4SOIcox+(epssub/Xdep_CV))+pParam_B4SOIcit));
{
double value_sqrt_0 = sqrt(Xdep_CV);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_e_si = (deriv_sqrt_0_d0*(d_Xdep_CV_dV_e_si));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(d_Xdep_CV_dV_gi_si));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(d_Xdep_CV_dV_di_si));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(d_Xdep_CV_dV_b_si));
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(d_Xdep_CV_dV_b_GND));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(d_Xdep_CV_dV_p_GND));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(d_Xdep_CV_dTemp_t_GND));
T3 = value_sqrt_0;
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt2*d_Vbseff_CV_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt2*d_Vbseff_CV_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt2*d_Vbseff_CV_dTemp_t_GND;
d_T0_dV_e_si = pParam_B4SOIdvt2*d_Vbseff_CV_dV_e_si;
d_T0_dV_gi_si = pParam_B4SOIdvt2*d_Vbseff_CV_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt2*d_Vbseff_CV_dV_di_si;
d_T0_dV_b_si = pParam_B4SOIdvt2*d_Vbseff_CV_dV_b_si;
T0 = (pParam_B4SOIdvt2*Vbseff_CV);
if ((T0>=(-0.5)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T1_dV_e_si = d_T0_dV_e_si;
d_T1_dV_gi_si = d_T0_dV_gi_si;
d_T1_dV_di_si = d_T0_dV_di_si;
d_T1_dV_b_si = d_T0_dV_b_si;
T1 = (1.0+T0);
}
else
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (-8.0*d_T0_dV_b_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_p_GND = (-8.0*d_T0_dV_p_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dTemp_t_GND = (-8.0*d_T0_dTemp_t_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_e_si = (-8.0*d_T0_dV_e_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_gi_si = (-8.0*d_T0_dV_gi_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_di_si = (-8.0*d_T0_dV_di_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_b_si = (-8.0*d_T0_dV_b_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = ((1.0+(3.0*T0))*d_T4_dV_b_GND+3.0*d_T0_dV_b_GND*T4);
d_T1_dV_p_GND = ((1.0+(3.0*T0))*d_T4_dV_p_GND+3.0*d_T0_dV_p_GND*T4);
d_T1_dTemp_t_GND = ((1.0+(3.0*T0))*d_T4_dTemp_t_GND+3.0*d_T0_dTemp_t_GND*T4);
d_T1_dV_e_si = ((1.0+(3.0*T0))*d_T4_dV_e_si+3.0*d_T0_dV_e_si*T4);
d_T1_dV_gi_si = ((1.0+(3.0*T0))*d_T4_dV_gi_si+3.0*d_T0_dV_gi_si*T4);
d_T1_dV_di_si = ((1.0+(3.0*T0))*d_T4_dV_di_si+3.0*d_T0_dV_di_si*T4);
d_T1_dV_b_si = ((1.0+(3.0*T0))*d_T4_dV_b_si+3.0*d_T0_dV_b_si*T4);
T1 = ((1.0+(3.0*T0))*T4);
}

d_lt1_CV_dV_b_GND = ((B4SOIfactor1*T3)*d_T1_dV_b_GND+B4SOIfactor1*d_T3_dV_b_GND*T1);
d_lt1_CV_dV_p_GND = ((B4SOIfactor1*T3)*d_T1_dV_p_GND+B4SOIfactor1*d_T3_dV_p_GND*T1);
d_lt1_CV_dTemp_t_GND = ((B4SOIfactor1*T3)*d_T1_dTemp_t_GND+B4SOIfactor1*d_T3_dTemp_t_GND*T1);
d_lt1_CV_dV_gi_si = ((B4SOIfactor1*T3)*d_T1_dV_gi_si+B4SOIfactor1*d_T3_dV_gi_si*T1);
d_lt1_CV_dV_di_si = ((B4SOIfactor1*T3)*d_T1_dV_di_si+B4SOIfactor1*d_T3_dV_di_si*T1);
d_lt1_CV_dV_e_si = ((B4SOIfactor1*T3)*d_T1_dV_e_si+B4SOIfactor1*d_T3_dV_e_si*T1);
d_lt1_CV_dV_b_si = ((B4SOIfactor1*T3)*d_T1_dV_b_si+B4SOIfactor1*d_T3_dV_b_si*T1);
lt1_CV = ((B4SOIfactor1*T3)*T1);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt2w*d_Vbseff_CV_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt2w*d_Vbseff_CV_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt2w*d_Vbseff_CV_dTemp_t_GND;
d_T0_dV_e_si = pParam_B4SOIdvt2w*d_Vbseff_CV_dV_e_si;
d_T0_dV_gi_si = pParam_B4SOIdvt2w*d_Vbseff_CV_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt2w*d_Vbseff_CV_dV_di_si;
d_T0_dV_b_si = pParam_B4SOIdvt2w*d_Vbseff_CV_dV_b_si;
T0 = (pParam_B4SOIdvt2w*Vbseff_CV);
if ((T0>=(-0.5)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = d_T0_dV_b_GND;
d_T1_dV_p_GND = d_T0_dV_p_GND;
d_T1_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T1_dV_e_si = d_T0_dV_e_si;
d_T1_dV_gi_si = d_T0_dV_gi_si;
d_T1_dV_di_si = d_T0_dV_di_si;
d_T1_dV_b_si = d_T0_dV_b_si;
T1 = (1.0+T0);
}
else
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (-8.0*d_T0_dV_b_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_p_GND = (-8.0*d_T0_dV_p_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dTemp_t_GND = (-8.0*d_T0_dTemp_t_GND/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_e_si = (-8.0*d_T0_dV_e_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_gi_si = (-8.0*d_T0_dV_gi_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_di_si = (-8.0*d_T0_dV_di_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
d_T4_dV_b_si = (-8.0*d_T0_dV_b_si/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = ((1.0+(3.0*T0))*d_T4_dV_b_GND+3.0*d_T0_dV_b_GND*T4);
d_T1_dV_p_GND = ((1.0+(3.0*T0))*d_T4_dV_p_GND+3.0*d_T0_dV_p_GND*T4);
d_T1_dTemp_t_GND = ((1.0+(3.0*T0))*d_T4_dTemp_t_GND+3.0*d_T0_dTemp_t_GND*T4);
d_T1_dV_e_si = ((1.0+(3.0*T0))*d_T4_dV_e_si+3.0*d_T0_dV_e_si*T4);
d_T1_dV_gi_si = ((1.0+(3.0*T0))*d_T4_dV_gi_si+3.0*d_T0_dV_gi_si*T4);
d_T1_dV_di_si = ((1.0+(3.0*T0))*d_T4_dV_di_si+3.0*d_T0_dV_di_si*T4);
d_T1_dV_b_si = ((1.0+(3.0*T0))*d_T4_dV_b_si+3.0*d_T0_dV_b_si*T4);
T1 = ((1.0+(3.0*T0))*T4);
}

d_ltw_CV_dV_b_GND = ((B4SOIfactor1*T3)*d_T1_dV_b_GND+B4SOIfactor1*d_T3_dV_b_GND*T1);
d_ltw_CV_dV_p_GND = ((B4SOIfactor1*T3)*d_T1_dV_p_GND+B4SOIfactor1*d_T3_dV_p_GND*T1);
d_ltw_CV_dTemp_t_GND = ((B4SOIfactor1*T3)*d_T1_dTemp_t_GND+B4SOIfactor1*d_T3_dTemp_t_GND*T1);
d_ltw_CV_dV_gi_si = ((B4SOIfactor1*T3)*d_T1_dV_gi_si+B4SOIfactor1*d_T3_dV_gi_si*T1);
d_ltw_CV_dV_di_si = ((B4SOIfactor1*T3)*d_T1_dV_di_si+B4SOIfactor1*d_T3_dV_di_si*T1);
d_ltw_CV_dV_e_si = ((B4SOIfactor1*T3)*d_T1_dV_e_si+B4SOIfactor1*d_T3_dV_e_si*T1);
d_ltw_CV_dV_b_si = ((B4SOIfactor1*T3)*d_T1_dV_b_si+B4SOIfactor1*d_T3_dV_b_si*T1);
ltw_CV = ((B4SOIfactor1*T3)*T1);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dV_b_GND/lt1_CV/lt1_CV);
d_T0_dV_p_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dV_p_GND/lt1_CV/lt1_CV);
d_T0_dTemp_t_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dTemp_t_GND/lt1_CV/lt1_CV);
d_T0_dV_gi_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dV_gi_si/lt1_CV/lt1_CV);
d_T0_dV_di_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dV_di_si/lt1_CV/lt1_CV);
d_T0_dV_e_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dV_e_si/lt1_CV/lt1_CV);
d_T0_dV_b_si = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dV_b_si/lt1_CV/lt1_CV);
T0 = ((((-0.5)*pParam_B4SOIdvt1)*Leff)/lt1_CV);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T1 = value_exp_0;
}

d_Theta0_CV_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0_CV_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0_CV_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0_CV_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0_CV_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0_CV_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_Theta0_CV_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
Theta0_CV = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_Theta0_CV_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0_CV_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0_CV_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0_CV_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0_CV_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0_CV_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_Theta0_CV_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
Theta0_CV = (T1*(1.0+(2.0*T1)));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_e_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_CV_dV_e_si/Xdep_CV/Xdep_CV);
d_T2_dV_gi_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_CV_dV_gi_si/Xdep_CV/Xdep_CV);
d_T2_dV_di_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_CV_dV_di_si/Xdep_CV/Xdep_CV);
d_T2_dV_b_si = (-(pParam_B4SOInfactor*epssub)*d_Xdep_CV_dV_b_si/Xdep_CV/Xdep_CV);
d_T2_dV_b_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_CV_dV_b_GND/Xdep_CV/Xdep_CV);
d_T2_dV_p_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_CV_dV_p_GND/Xdep_CV/Xdep_CV);
d_T2_dTemp_t_GND = (-(pParam_B4SOInfactor*epssub)*d_Xdep_CV_dTemp_t_GND/Xdep_CV/Xdep_CV);
T2 = ((pParam_B4SOInfactor*epssub)/Xdep_CV);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = pParam_B4SOIcdscb*d_Vbseff_CV_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIcdscb*d_Vbseff_CV_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIcdscb*d_Vbseff_CV_dTemp_t_GND;
d_T3_dV_e_si = pParam_B4SOIcdscb*d_Vbseff_CV_dV_e_si;
d_T3_dV_gi_si = pParam_B4SOIcdscb*d_Vbseff_CV_dV_gi_si;
d_T3_dV_di_si = (pParam_B4SOIcdscb*d_Vbseff_CV_dV_di_si+pParam_B4SOIcdscd*d_Vds_dV_di_si);
d_T3_dV_b_si = pParam_B4SOIcdscb*d_Vbseff_CV_dV_b_si;
T3 = ((pParam_B4SOIcdsc+(pParam_B4SOIcdscb*Vbseff_CV))+(pParam_B4SOIcdscd*Vds));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = ((d_T2_dV_b_GND+(T3*d_Theta0_CV_dV_b_GND+d_T3_dV_b_GND*Theta0_CV))/B4SOIcox);
d_T4_dV_p_GND = ((d_T2_dV_p_GND+(T3*d_Theta0_CV_dV_p_GND+d_T3_dV_p_GND*Theta0_CV))/B4SOIcox);
d_T4_dTemp_t_GND = ((d_T2_dTemp_t_GND+(T3*d_Theta0_CV_dTemp_t_GND+d_T3_dTemp_t_GND*Theta0_CV))/B4SOIcox);
d_T4_dV_gi_si = ((d_T2_dV_gi_si+(T3*d_Theta0_CV_dV_gi_si+d_T3_dV_gi_si*Theta0_CV))/B4SOIcox);
d_T4_dV_di_si = ((d_T2_dV_di_si+(T3*d_Theta0_CV_dV_di_si+d_T3_dV_di_si*Theta0_CV))/B4SOIcox);
d_T4_dV_e_si = ((d_T2_dV_e_si+(T3*d_Theta0_CV_dV_e_si+d_T3_dV_e_si*Theta0_CV))/B4SOIcox);
d_T4_dV_b_si = ((d_T2_dV_b_si+(T3*d_Theta0_CV_dV_b_si+d_T3_dV_b_si*Theta0_CV))/B4SOIcox);
T4 = (((T2+(T3*Theta0_CV))+pParam_B4SOIcit)/B4SOIcox);
if ((T4>=(-0.5)))
{

d_n_CV_dV_b_GND = d_T4_dV_b_GND;
d_n_CV_dV_p_GND = d_T4_dV_p_GND;
d_n_CV_dTemp_t_GND = d_T4_dTemp_t_GND;
d_n_CV_dV_gi_si = d_T4_dV_gi_si;
d_n_CV_dV_di_si = d_T4_dV_di_si;
d_n_CV_dV_e_si = d_T4_dV_e_si;
d_n_CV_dV_b_si = d_T4_dV_b_si;
n_CV = (1.0+T4);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-8.0*d_T4_dV_b_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_p_GND = (-8.0*d_T4_dV_p_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dTemp_t_GND = (-8.0*d_T4_dTemp_t_GND/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_gi_si = (-8.0*d_T4_dV_gi_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_di_si = (-8.0*d_T4_dV_di_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_e_si = (-8.0*d_T4_dV_e_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
d_T0_dV_b_si = (-8.0*d_T4_dV_b_si/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
T0 = (1.0/(3.0+(8.0*T4)));

d_n_CV_dV_b_GND = ((1.0+(3.0*T4))*d_T0_dV_b_GND+3.0*d_T4_dV_b_GND*T0);
d_n_CV_dV_p_GND = ((1.0+(3.0*T4))*d_T0_dV_p_GND+3.0*d_T4_dV_p_GND*T0);
d_n_CV_dTemp_t_GND = ((1.0+(3.0*T4))*d_T0_dTemp_t_GND+3.0*d_T4_dTemp_t_GND*T0);
d_n_CV_dV_gi_si = ((1.0+(3.0*T4))*d_T0_dV_gi_si+3.0*d_T4_dV_gi_si*T0);
d_n_CV_dV_di_si = ((1.0+(3.0*T4))*d_T0_dV_di_si+3.0*d_T4_dV_di_si*T0);
d_n_CV_dV_e_si = ((1.0+(3.0*T4))*d_T0_dV_e_si+3.0*d_T4_dV_e_si*T0);
d_n_CV_dV_b_si = ((1.0+(3.0*T4))*d_T0_dV_b_si+3.0*d_T4_dV_b_si*T0);
n_CV = ((1.0+(3.0*T4))*T0);
}
if ((pParam_B4SOIdvtp0>0.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = (-pParam_B4SOIdvtp1)*d_Vds_dV_di_si;
T0 = ((-pParam_B4SOIdvtp1)*Vds);
if ((T0<(-100.0)))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T2_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T2_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T2_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T2_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T2_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T2_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T2 = value_exp_0;
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = pParam_B4SOIdvtp0*d_T2_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIdvtp0*d_T2_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIdvtp0*d_T2_dTemp_t_GND;
d_T3_dV_gi_si = pParam_B4SOIdvtp0*d_T2_dV_gi_si;
d_T3_dV_di_si = pParam_B4SOIdvtp0*d_T2_dV_di_si;
d_T3_dV_e_si = pParam_B4SOIdvtp0*d_T2_dV_e_si;
d_T3_dV_b_si = pParam_B4SOIdvtp0*d_T2_dV_b_si;
T3 = (Leff+(pParam_B4SOIdvtp0*(1.0+T2)));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((Leff/T3));

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_gi_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_gi_si/T3/T3));
d_T4_dV_di_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_di_si/T3/T3));
d_T4_dV_e_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_e_si/T3/T3));
d_T4_dV_b_si = Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_b_si/T3/T3));
d_T4_dV_b_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_b_GND/T3/T3))+d_Vtm_dV_b_GND*evaluator_lln_0.getValues((Leff/T3)));
d_T4_dV_p_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dV_p_GND/T3/T3))+d_Vtm_dV_p_GND*evaluator_lln_0.getValues((Leff/T3)));
d_T4_dTemp_t_GND = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),(-Leff*d_T3_dTemp_t_GND/T3/T3))+d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues((Leff/T3)));
T4 = (Vtm*evaluator_lln_0.getValues((Leff/T3)));
}

d_DITS_Sft_CV_dV_b_GND = (n_CV*d_T4_dV_b_GND+d_n_CV_dV_b_GND*T4);
d_DITS_Sft_CV_dV_p_GND = (n_CV*d_T4_dV_p_GND+d_n_CV_dV_p_GND*T4);
d_DITS_Sft_CV_dTemp_t_GND = (n_CV*d_T4_dTemp_t_GND+d_n_CV_dTemp_t_GND*T4);
d_DITS_Sft_CV_dV_gi_si = (n_CV*d_T4_dV_gi_si+d_n_CV_dV_gi_si*T4);
d_DITS_Sft_CV_dV_di_si = (n_CV*d_T4_dV_di_si+d_n_CV_dV_di_si*T4);
d_DITS_Sft_CV_dV_e_si = (n_CV*d_T4_dV_e_si+d_n_CV_dV_e_si*T4);
d_DITS_Sft_CV_dV_b_si = (n_CV*d_T4_dV_b_si+d_n_CV_dV_b_si*T4);
DITS_Sft_CV = (n_CV*T4);
}
else
{

d_DITS_Sft_CV_dV_b_si = d_DITS_Sft_CV_dV_e_si = d_DITS_Sft_CV_dV_di_si = d_DITS_Sft_CV_dV_gi_si = d_DITS_Sft_CV_dTemp_t_GND = d_DITS_Sft_CV_dV_p_GND = d_DITS_Sft_CV_dV_b_GND =  0.0;
DITS_Sft_CV = 0.0;
}

d_B4SOIthetavth_dV_b_GND = pParam_B4SOIdvt0*d_Theta0_CV_dV_b_GND;
d_B4SOIthetavth_dV_p_GND = pParam_B4SOIdvt0*d_Theta0_CV_dV_p_GND;
d_B4SOIthetavth_dTemp_t_GND = pParam_B4SOIdvt0*d_Theta0_CV_dTemp_t_GND;
d_B4SOIthetavth_dV_gi_si = pParam_B4SOIdvt0*d_Theta0_CV_dV_gi_si;
d_B4SOIthetavth_dV_di_si = pParam_B4SOIdvt0*d_Theta0_CV_dV_di_si;
d_B4SOIthetavth_dV_e_si = pParam_B4SOIdvt0*d_Theta0_CV_dV_e_si;
d_B4SOIthetavth_dV_b_si = pParam_B4SOIdvt0*d_Theta0_CV_dV_b_si;
B4SOIthetavth = (pParam_B4SOIdvt0*Theta0_CV);

d_Delt_vth_CV_dV_b_GND = (B4SOIthetavth*d_V0_dV_b_GND+d_B4SOIthetavth_dV_b_GND*V0);
d_Delt_vth_CV_dV_p_GND = (B4SOIthetavth*d_V0_dV_p_GND+d_B4SOIthetavth_dV_p_GND*V0);
d_Delt_vth_CV_dTemp_t_GND = (B4SOIthetavth*d_V0_dTemp_t_GND+d_B4SOIthetavth_dTemp_t_GND*V0);
d_Delt_vth_CV_dV_gi_si = d_B4SOIthetavth_dV_gi_si*V0;
d_Delt_vth_CV_dV_di_si = d_B4SOIthetavth_dV_di_si*V0;
d_Delt_vth_CV_dV_e_si = d_B4SOIthetavth_dV_e_si*V0;
d_Delt_vth_CV_dV_b_si = d_B4SOIthetavth_dV_b_si*V0;
Delt_vth_CV = (B4SOIthetavth*V0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_CV_dV_b_GND/ltw_CV/ltw_CV);
d_T0_dV_p_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_CV_dV_p_GND/ltw_CV/ltw_CV);
d_T0_dTemp_t_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_CV_dTemp_t_GND/ltw_CV/ltw_CV);
d_T0_dV_gi_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_CV_dV_gi_si/ltw_CV/ltw_CV);
d_T0_dV_di_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_CV_dV_di_si/ltw_CV/ltw_CV);
d_T0_dV_e_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_CV_dV_e_si/ltw_CV/ltw_CV);
d_T0_dV_b_si = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltw_CV_dV_b_si/ltw_CV/ltw_CV);
T0 = (((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)/ltw_CV);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T1 = value_exp_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_T2_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_T2_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt0w*d_T2_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt0w*d_T2_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt0w*d_T2_dTemp_t_GND;
d_T0_dV_gi_si = pParam_B4SOIdvt0w*d_T2_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt0w*d_T2_dV_di_si;
d_T0_dV_e_si = pParam_B4SOIdvt0w*d_T2_dV_e_si;
d_T0_dV_b_si = pParam_B4SOIdvt0w*d_T2_dV_b_si;
T0 = (pParam_B4SOIdvt0w*T2);

d_DeltVthw_CV_dV_b_GND = (T0*d_V0_dV_b_GND+d_T0_dV_b_GND*V0);
d_DeltVthw_CV_dV_p_GND = (T0*d_V0_dV_p_GND+d_T0_dV_p_GND*V0);
d_DeltVthw_CV_dTemp_t_GND = (T0*d_V0_dTemp_t_GND+d_T0_dTemp_t_GND*V0);
d_DeltVthw_CV_dV_e_si = d_T0_dV_e_si*V0;
d_DeltVthw_CV_dV_gi_si = d_T0_dV_gi_si*V0;
d_DeltVthw_CV_dV_di_si = d_T0_dV_di_si*V0;
d_DeltVthw_CV_dV_b_si = d_T0_dV_b_si*V0;
DeltVthw_CV = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpe0/Leff)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = value_sqrt_0;
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = pParam_B4SOIkt2*d_Vbseff_CV_dV_b_GND;
d_T1_dV_p_GND = pParam_B4SOIkt2*d_Vbseff_CV_dV_p_GND;
d_T1_dTemp_t_GND = pParam_B4SOIkt2*d_Vbseff_CV_dTemp_t_GND;
d_T1_dV_e_si = pParam_B4SOIkt2*d_Vbseff_CV_dV_e_si;
d_T1_dV_gi_si = pParam_B4SOIkt2*d_Vbseff_CV_dV_gi_si;
d_T1_dV_di_si = pParam_B4SOIkt2*d_Vbseff_CV_dV_di_si;
d_T1_dV_b_si = pParam_B4SOIkt2*d_Vbseff_CV_dV_b_si;
T1 = ((pParam_B4SOIkt1+(pParam_B4SOIkt1l/Leff))+(pParam_B4SOIkt2*Vbseff_CV));

d_DeltVthtemp_CV_dV_b_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_b_GND+pParam_B4SOIk1ox*d_T0_dV_b_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_b_GND+d_T1_dV_b_GND*TempRatioMinus1));
d_DeltVthtemp_CV_dV_p_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_p_GND+pParam_B4SOIk1ox*d_T0_dV_p_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_p_GND+d_T1_dV_p_GND*TempRatioMinus1));
d_DeltVthtemp_CV_dTemp_t_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dTemp_t_GND+pParam_B4SOIk1ox*d_T0_dTemp_t_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dTemp_t_GND+d_T1_dTemp_t_GND*TempRatioMinus1));
d_DeltVthtemp_CV_dV_e_si = (pParam_B4SOIk1ox*d_T0_dV_e_si*sqrtPhi+d_T1_dV_e_si*TempRatioMinus1);
d_DeltVthtemp_CV_dV_gi_si = (pParam_B4SOIk1ox*d_T0_dV_gi_si*sqrtPhi+d_T1_dV_gi_si*TempRatioMinus1);
d_DeltVthtemp_CV_dV_di_si = (pParam_B4SOIk1ox*d_T0_dV_di_si*sqrtPhi+d_T1_dV_di_si*TempRatioMinus1);
d_DeltVthtemp_CV_dV_b_si = (pParam_B4SOIk1ox*d_T0_dV_b_si*sqrtPhi+d_T1_dV_b_si*TempRatioMinus1);
DeltVthtemp_CV = (((pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));

d_tmp2_CV_dV_b_GND = (toxe*d_phi_dV_b_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
d_tmp2_CV_dV_p_GND = (toxe*d_phi_dV_p_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
d_tmp2_CV_dTemp_t_GND = (toxe*d_phi_dTemp_t_GND/(pParam_B4SOIweff+pParam_B4SOIw0));
tmp2_CV = ((toxe*phi)/(pParam_B4SOIweff+pParam_B4SOIw0));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = pParam_B4SOIetabcv*d_Vbseff_CV_dV_b_GND;
d_T3_dV_p_GND = pParam_B4SOIetabcv*d_Vbseff_CV_dV_p_GND;
d_T3_dTemp_t_GND = pParam_B4SOIetabcv*d_Vbseff_CV_dTemp_t_GND;
d_T3_dV_e_si = pParam_B4SOIetabcv*d_Vbseff_CV_dV_e_si;
d_T3_dV_gi_si = pParam_B4SOIetabcv*d_Vbseff_CV_dV_gi_si;
d_T3_dV_di_si = pParam_B4SOIetabcv*d_Vbseff_CV_dV_di_si;
d_T3_dV_b_si = pParam_B4SOIetabcv*d_Vbseff_CV_dV_b_si;
T3 = (here_B4SOIeta0cv+(pParam_B4SOIetabcv*Vbseff_CV));
if ((T3<1.0e-4))
{

d_T9_dV_b_GND = (-(-2.0e4*d_T3_dV_b_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_p_GND = (-(-2.0e4*d_T3_dV_p_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dTemp_t_GND = (-(-2.0e4*d_T3_dTemp_t_GND)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_gi_si = (-(-2.0e4*d_T3_dV_gi_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_di_si = (-(-2.0e4*d_T3_dV_di_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_e_si = (-(-2.0e4*d_T3_dV_e_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
d_T9_dV_b_si = (-(-2.0e4*d_T3_dV_b_si)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = ((2.0e-4-T3)*d_T9_dV_b_GND+(-d_T3_dV_b_GND)*T9);
d_T3_dV_p_GND = ((2.0e-4-T3)*d_T9_dV_p_GND+(-d_T3_dV_p_GND)*T9);
d_T3_dTemp_t_GND = ((2.0e-4-T3)*d_T9_dTemp_t_GND+(-d_T3_dTemp_t_GND)*T9);
d_T3_dV_gi_si = ((2.0e-4-T3)*d_T9_dV_gi_si+(-d_T3_dV_gi_si)*T9);
d_T3_dV_di_si = ((2.0e-4-T3)*d_T9_dV_di_si+(-d_T3_dV_di_si)*T9);
d_T3_dV_e_si = ((2.0e-4-T3)*d_T9_dV_e_si+(-d_T3_dV_e_si)*T9);
d_T3_dV_b_si = ((2.0e-4-T3)*d_T9_dV_b_si+(-d_T3_dV_b_si)*T9);
T3 = ((2.0e-4-T3)*T9);
}

d_DIBL_Sft_CV_dV_b_GND = (T3*d_theta0vb0_dV_b_GND+d_T3_dV_b_GND*theta0vb0)*Vds;
d_DIBL_Sft_CV_dV_p_GND = (T3*d_theta0vb0_dV_p_GND+d_T3_dV_p_GND*theta0vb0)*Vds;
d_DIBL_Sft_CV_dTemp_t_GND = (T3*d_theta0vb0_dTemp_t_GND+d_T3_dTemp_t_GND*theta0vb0)*Vds;
d_DIBL_Sft_CV_dV_gi_si = d_T3_dV_gi_si*theta0vb0*Vds;
d_DIBL_Sft_CV_dV_di_si = ((T3*theta0vb0)*d_Vds_dV_di_si+d_T3_dV_di_si*theta0vb0*Vds);
d_DIBL_Sft_CV_dV_e_si = d_T3_dV_e_si*theta0vb0*Vds;
d_DIBL_Sft_CV_dV_b_si = d_T3_dV_b_si*theta0vb0*Vds;
DIBL_Sft_CV = ((T3*theta0vb0)*Vds);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpeb/Leff)));
Lpe_Vb = value_sqrt_0;
}

d_T9_dV_b_si = d_T9_dV_e_si = d_T9_dV_di_si = d_T9_dV_gi_si =  0.0;
d_T9_dV_b_GND = (-2.2361*d_sqrtPhi_dV_b_GND/sqrtPhi/sqrtPhi);
d_T9_dV_p_GND = (-2.2361*d_sqrtPhi_dV_p_GND/sqrtPhi/sqrtPhi);
d_T9_dTemp_t_GND = (-2.2361*d_sqrtPhi_dTemp_t_GND/sqrtPhi/sqrtPhi);
T9 = (2.2361/sqrtPhi);

d_sqrtPhisExt_CV_dV_e_si = (d_sqrtPhis_CV_dV_e_si-(T9*(d_Vbsh_CV_dV_e_si-d_Vbseff_CV_dV_e_si)+d_T9_dV_e_si*(Vbsh_CV-Vbseff_CV)));
d_sqrtPhisExt_CV_dV_gi_si = (d_sqrtPhis_CV_dV_gi_si-(T9*(d_Vbsh_CV_dV_gi_si-d_Vbseff_CV_dV_gi_si)+d_T9_dV_gi_si*(Vbsh_CV-Vbseff_CV)));
d_sqrtPhisExt_CV_dV_di_si = (d_sqrtPhis_CV_dV_di_si-(T9*(d_Vbsh_CV_dV_di_si-d_Vbseff_CV_dV_di_si)+d_T9_dV_di_si*(Vbsh_CV-Vbseff_CV)));
d_sqrtPhisExt_CV_dV_b_si = (d_sqrtPhis_CV_dV_b_si-(T9*(d_Vbsh_CV_dV_b_si-d_Vbseff_CV_dV_b_si)+d_T9_dV_b_si*(Vbsh_CV-Vbseff_CV)));
d_sqrtPhisExt_CV_dV_b_GND = (d_sqrtPhis_CV_dV_b_GND-(T9*(d_Vbsh_CV_dV_b_GND-d_Vbseff_CV_dV_b_GND)+d_T9_dV_b_GND*(Vbsh_CV-Vbseff_CV)));
d_sqrtPhisExt_CV_dV_p_GND = (d_sqrtPhis_CV_dV_p_GND-(T9*(d_Vbsh_CV_dV_p_GND-d_Vbseff_CV_dV_p_GND)+d_T9_dV_p_GND*(Vbsh_CV-Vbseff_CV)));
d_sqrtPhisExt_CV_dTemp_t_GND = (d_sqrtPhis_CV_dTemp_t_GND-(T9*(d_Vbsh_CV_dTemp_t_GND-d_Vbseff_CV_dTemp_t_GND)+d_T9_dTemp_t_GND*(Vbsh_CV-Vbseff_CV)));
sqrtPhisExt_CV = (sqrtPhis_CV-(T9*(Vbsh_CV-Vbseff_CV)));
{
double value_exp_0 = exp(((2.0*pParam_B4SOIdvtp4)*Vds));
double  deriv_exp_0_d0 = value_exp_0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = (deriv_exp_0_d0*((2.0*pParam_B4SOIdvtp4)*d_Vds_dV_di_si));
T0 = value_exp_0;
}

d_DITS_Sft2_dV_b_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_b_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_b_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_p_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_p_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_p_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dTemp_t_GND = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dTemp_t_GND-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dTemp_t_GND)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_e_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_e_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_e_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_gi_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_gi_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_gi_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_di_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_di_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_di_si)/(T0+1)/(T0+1));
d_DITS_Sft2_dV_b_si = (((T0+1)*pParam_B4SOIdvtp2factor*d_T0_dV_b_si-(pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dV_b_si)/(T0+1)/(T0+1));
DITS_Sft2 = ((pParam_B4SOIdvtp2factor*(T0-1))/(T0+1));

d_Vth_CV_dV_e_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dV_e_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_CV_dV_e_si)-d_Delt_vth_CV_dV_e_si)-d_DeltVthw_CV_dV_e_si)+pParam_B4SOIk3b*d_Vbseff_CV_dV_e_si*tmp2_CV)+d_DeltVthtemp_CV_dV_e_si)-d_DIBL_Sft_CV_dV_e_si)-d_DITS_Sft_CV_dV_e_si)-d_DITS_Sft2_dV_e_si);
d_Vth_CV_dV_gi_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dV_gi_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_CV_dV_gi_si)-d_Delt_vth_CV_dV_gi_si)-d_DeltVthw_CV_dV_gi_si)+pParam_B4SOIk3b*d_Vbseff_CV_dV_gi_si*tmp2_CV)+d_DeltVthtemp_CV_dV_gi_si)-d_DIBL_Sft_CV_dV_gi_si)-d_DITS_Sft_CV_dV_gi_si)-d_DITS_Sft2_dV_gi_si);
d_Vth_CV_dV_di_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dV_di_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_CV_dV_di_si)-d_Delt_vth_CV_dV_di_si)-d_DeltVthw_CV_dV_di_si)+pParam_B4SOIk3b*d_Vbseff_CV_dV_di_si*tmp2_CV)+d_DeltVthtemp_CV_dV_di_si)-d_DIBL_Sft_CV_dV_di_si)-d_DITS_Sft_CV_dV_di_si)-d_DITS_Sft2_dV_di_si);
d_Vth_CV_dV_b_si = ((((((((pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dV_b_si*Lpe_Vb-here_B4SOIk2ox*d_Vbseff_CV_dV_b_si)-d_Delt_vth_CV_dV_b_si)-d_DeltVthw_CV_dV_b_si)+pParam_B4SOIk3b*d_Vbseff_CV_dV_b_si*tmp2_CV)+d_DeltVthtemp_CV_dV_b_si)-d_DIBL_Sft_CV_dV_b_si)-d_DITS_Sft_CV_dV_b_si)-d_DITS_Sft2_dV_b_si);
d_Vth_CV_dV_b_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_b_GND+(pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dV_b_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbseff_CV_dV_b_GND)-d_Delt_vth_CV_dV_b_GND)-d_DeltVthw_CV_dV_b_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff_CV))*d_tmp2_CV_dV_b_GND+pParam_B4SOIk3b*d_Vbseff_CV_dV_b_GND*tmp2_CV))+d_DeltVthtemp_CV_dV_b_GND)-d_DIBL_Sft_CV_dV_b_GND)-d_DITS_Sft_CV_dV_b_GND)-d_DITS_Sft2_dV_b_GND);
d_Vth_CV_dV_p_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dV_p_GND+(pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dV_p_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbseff_CV_dV_p_GND)-d_Delt_vth_CV_dV_p_GND)-d_DeltVthw_CV_dV_p_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff_CV))*d_tmp2_CV_dV_p_GND+pParam_B4SOIk3b*d_Vbseff_CV_dV_p_GND*tmp2_CV))+d_DeltVthtemp_CV_dV_p_GND)-d_DIBL_Sft_CV_dV_p_GND)-d_DITS_Sft_CV_dV_p_GND)-d_DITS_Sft2_dV_p_GND);
d_Vth_CV_dTemp_t_GND = (((((((((B4SOItype*d_here_B4SOIvth0_dTemp_t_GND+(pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dTemp_t_GND-(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi))*Lpe_Vb)-here_B4SOIk2ox*d_Vbseff_CV_dTemp_t_GND)-d_Delt_vth_CV_dTemp_t_GND)-d_DeltVthw_CV_dTemp_t_GND)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff_CV))*d_tmp2_CV_dTemp_t_GND+pParam_B4SOIk3b*d_Vbseff_CV_dTemp_t_GND*tmp2_CV))+d_DeltVthtemp_CV_dTemp_t_GND)-d_DIBL_Sft_CV_dTemp_t_GND)-d_DITS_Sft_CV_dTemp_t_GND)-d_DITS_Sft2_dTemp_t_GND);
Vth_CV = ((((((((((B4SOItype*here_B4SOIvth0)+(((pParam_B4SOIk1ox*sqrtPhisExt_CV)-(pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(here_B4SOIk2ox*Vbseff_CV))-Delt_vth_CV)-DeltVthw_CV)+((pParam_B4SOIk3+(pParam_B4SOIk3b*Vbseff_CV))*tmp2_CV))+DeltVthtemp_CV)-DIBL_Sft_CV)-DITS_Sft_CV)-DITS_Sft2);
if ((((B4SOIcapMod==3)&&(SHMOD==1))&&(RTH0!=0.0)))
{
{
double value_sqrt_0 = sqrt(Xdep0);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3zb_dV_b_GND = (deriv_sqrt_0_d0*(d_Xdep0_dV_b_GND));
d_T3zb_dV_p_GND = (deriv_sqrt_0_d0*(d_Xdep0_dV_p_GND));
d_T3zb_dTemp_t_GND = (deriv_sqrt_0_d0*(d_Xdep0_dTemp_t_GND));
T3zb = value_sqrt_0;
}

d_lt1zb_dV_b_GND = B4SOIfactor1*d_T3zb_dV_b_GND;
d_lt1zb_dV_p_GND = B4SOIfactor1*d_T3zb_dV_p_GND;
d_lt1zb_dTemp_t_GND = B4SOIfactor1*d_T3zb_dTemp_t_GND;
lt1zb = (B4SOIfactor1*T3zb);

d_ltwzb_dV_b_GND = B4SOIfactor1*d_T3zb_dV_b_GND;
d_ltwzb_dV_p_GND = B4SOIfactor1*d_T3zb_dV_p_GND;
d_ltwzb_dTemp_t_GND = B4SOIfactor1*d_T3zb_dTemp_t_GND;
ltwzb = (B4SOIfactor1*T3zb);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1zb_dV_b_GND/lt1zb/lt1zb);
d_T0_dV_p_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1zb_dV_p_GND/lt1zb/lt1zb);
d_T0_dTemp_t_GND = (-(((-0.5)*pParam_B4SOIdvt1)*Leff)*d_lt1zb_dTemp_t_GND/lt1zb/lt1zb);
T0 = ((((-0.5)*pParam_B4SOIdvt1)*Leff)/lt1zb);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T1 = value_exp_0;
}

d_Theta0zb_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0zb_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0zb_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0zb_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0zb_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0zb_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_Theta0zb_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
Theta0zb = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_Theta0zb_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_Theta0zb_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_Theta0zb_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_Theta0zb_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_Theta0zb_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_Theta0zb_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_Theta0zb_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
Theta0zb = (T1*(1.0+(2.0*T1)));
}

d_Delt_vthzb_dV_b_GND = ((pParam_B4SOIdvt0*Theta0zb)*d_V0_dV_b_GND+pParam_B4SOIdvt0*d_Theta0zb_dV_b_GND*V0);
d_Delt_vthzb_dV_p_GND = ((pParam_B4SOIdvt0*Theta0zb)*d_V0_dV_p_GND+pParam_B4SOIdvt0*d_Theta0zb_dV_p_GND*V0);
d_Delt_vthzb_dTemp_t_GND = ((pParam_B4SOIdvt0*Theta0zb)*d_V0_dTemp_t_GND+pParam_B4SOIdvt0*d_Theta0zb_dTemp_t_GND*V0);
d_Delt_vthzb_dV_gi_si = pParam_B4SOIdvt0*d_Theta0zb_dV_gi_si*V0;
d_Delt_vthzb_dV_di_si = pParam_B4SOIdvt0*d_Theta0zb_dV_di_si*V0;
d_Delt_vthzb_dV_e_si = pParam_B4SOIdvt0*d_Theta0zb_dV_e_si*V0;
d_Delt_vthzb_dV_b_si = pParam_B4SOIdvt0*d_Theta0zb_dV_b_si*V0;
Delt_vthzb = ((pParam_B4SOIdvt0*Theta0zb)*V0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltwzb_dV_b_GND/ltwzb/ltwzb);
d_T0_dV_p_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltwzb_dV_p_GND/ltwzb/ltwzb);
d_T0_dTemp_t_GND = (-((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)*d_ltwzb_dTemp_t_GND/ltwzb/ltwzb);
T0 = (((((-0.5)*pParam_B4SOIdvt1w)*pParam_B4SOIweff)*Leff)/ltwzb);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T1 = value_exp_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_T2_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (T1*2.0*d_T1_dV_b_GND+d_T1_dV_b_GND*(1.0+(2.0*T1)));
d_T2_dV_p_GND = (T1*2.0*d_T1_dV_p_GND+d_T1_dV_p_GND*(1.0+(2.0*T1)));
d_T2_dTemp_t_GND = (T1*2.0*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*(1.0+(2.0*T1)));
d_T2_dV_gi_si = (T1*2.0*d_T1_dV_gi_si+d_T1_dV_gi_si*(1.0+(2.0*T1)));
d_T2_dV_di_si = (T1*2.0*d_T1_dV_di_si+d_T1_dV_di_si*(1.0+(2.0*T1)));
d_T2_dV_e_si = (T1*2.0*d_T1_dV_e_si+d_T1_dV_e_si*(1.0+(2.0*T1)));
d_T2_dV_b_si = (T1*2.0*d_T1_dV_b_si+d_T1_dV_b_si*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIdvt0w*d_T2_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIdvt0w*d_T2_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIdvt0w*d_T2_dTemp_t_GND;
d_T0_dV_gi_si = pParam_B4SOIdvt0w*d_T2_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIdvt0w*d_T2_dV_di_si;
d_T0_dV_e_si = pParam_B4SOIdvt0w*d_T2_dV_e_si;
d_T0_dV_b_si = pParam_B4SOIdvt0w*d_T2_dV_b_si;
T0 = (pParam_B4SOIdvt0w*T2);

d_DeltVthwzb_dV_b_GND = (T0*d_V0_dV_b_GND+d_T0_dV_b_GND*V0);
d_DeltVthwzb_dV_p_GND = (T0*d_V0_dV_p_GND+d_T0_dV_p_GND*V0);
d_DeltVthwzb_dTemp_t_GND = (T0*d_V0_dTemp_t_GND+d_T0_dTemp_t_GND*V0);
d_DeltVthwzb_dV_e_si = d_T0_dV_e_si*V0;
d_DeltVthwzb_dV_gi_si = d_T0_dV_gi_si*V0;
d_DeltVthwzb_dV_di_si = d_T0_dV_di_si*V0;
d_DeltVthwzb_dV_b_si = d_T0_dV_b_si*V0;
DeltVthwzb = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(pParam_B4SOIlpe0/Leff)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = value_sqrt_0;
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = (pParam_B4SOIkt1+(pParam_B4SOIkt1l/Leff));

d_DeltVthtempzb_dV_b_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_b_GND+pParam_B4SOIk1ox*d_T0_dV_b_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_b_GND+d_T1_dV_b_GND*TempRatioMinus1));
d_DeltVthtempzb_dV_p_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dV_p_GND+pParam_B4SOIk1ox*d_T0_dV_p_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dV_p_GND+d_T1_dV_p_GND*TempRatioMinus1));
d_DeltVthtempzb_dTemp_t_GND = (((pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dTemp_t_GND+pParam_B4SOIk1ox*d_T0_dTemp_t_GND*sqrtPhi)+(T1*d_TempRatioMinus1_dTemp_t_GND+d_T1_dTemp_t_GND*TempRatioMinus1));
d_DeltVthtempzb_dV_e_si = (pParam_B4SOIk1ox*d_T0_dV_e_si*sqrtPhi+d_T1_dV_e_si*TempRatioMinus1);
d_DeltVthtempzb_dV_gi_si = (pParam_B4SOIk1ox*d_T0_dV_gi_si*sqrtPhi+d_T1_dV_gi_si*TempRatioMinus1);
d_DeltVthtempzb_dV_di_si = (pParam_B4SOIk1ox*d_T0_dV_di_si*sqrtPhi+d_T1_dV_di_si*TempRatioMinus1);
d_DeltVthtempzb_dV_b_si = (pParam_B4SOIk1ox*d_T0_dV_b_si*sqrtPhi+d_T1_dV_b_si*TempRatioMinus1);
DeltVthtempzb = (((pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));

d_Vthzb_dV_gi_si = (((-d_Delt_vthzb_dV_gi_si)-d_DeltVthwzb_dV_gi_si)+d_DeltVthtempzb_dV_gi_si);
d_Vthzb_dV_di_si = (((-d_Delt_vthzb_dV_di_si)-d_DeltVthwzb_dV_di_si)+d_DeltVthtempzb_dV_di_si);
d_Vthzb_dV_e_si = (((-d_Delt_vthzb_dV_e_si)-d_DeltVthwzb_dV_e_si)+d_DeltVthtempzb_dV_e_si);
d_Vthzb_dV_b_si = (((-d_Delt_vthzb_dV_b_si)-d_DeltVthwzb_dV_b_si)+d_DeltVthtempzb_dV_b_si);
d_Vthzb_dV_b_GND = ((((B4SOItype*d_here_B4SOIvth0_dV_b_GND-d_Delt_vthzb_dV_b_GND)-d_DeltVthwzb_dV_b_GND)+pParam_B4SOIk3*d_tmp2_CV_dV_b_GND)+d_DeltVthtempzb_dV_b_GND);
d_Vthzb_dV_p_GND = ((((B4SOItype*d_here_B4SOIvth0_dV_p_GND-d_Delt_vthzb_dV_p_GND)-d_DeltVthwzb_dV_p_GND)+pParam_B4SOIk3*d_tmp2_CV_dV_p_GND)+d_DeltVthtempzb_dV_p_GND);
d_Vthzb_dTemp_t_GND = ((((B4SOItype*d_here_B4SOIvth0_dTemp_t_GND-d_Delt_vthzb_dTemp_t_GND)-d_DeltVthwzb_dTemp_t_GND)+pParam_B4SOIk3*d_tmp2_CV_dTemp_t_GND)+d_DeltVthtempzb_dTemp_t_GND);
Vthzb = (((((B4SOItype*here_B4SOIvth0)-Delt_vthzb)-DeltVthwzb)+(pParam_B4SOIk3*tmp2_CV))+DeltVthtempzb);
}
else
{

d_Vthzb_dTemp_t_GND = d_Vthzb_dV_p_GND = d_Vthzb_dV_b_GND = d_Vthzb_dV_b_si = d_Vthzb_dV_e_si = d_Vthzb_dV_di_si = d_Vthzb_dV_gi_si =  0.0;
Vthzb = 0;
}

d_Vgst_dV_e_si = (-d_Vth_dV_e_si);
d_Vgst_dV_b_si = (-d_Vth_dV_b_si);
d_Vgst_dV_b_GND = (d_Vgs_eff_dV_b_GND-d_Vth_dV_b_GND);
d_Vgst_dV_p_GND = (d_Vgs_eff_dV_p_GND-d_Vth_dV_p_GND);
d_Vgst_dTemp_t_GND = (d_Vgs_eff_dTemp_t_GND-d_Vth_dTemp_t_GND);
d_Vgst_dV_gi_si = (d_Vgs_eff_dV_gi_si-d_Vth_dV_gi_si);
d_Vgst_dV_di_si = (d_Vgs_eff_dV_di_si-d_Vth_dV_di_si);
Vgst = (Vgs_eff-Vth);

d_T10_dV_db_di = d_T10_dV_sb_si =  0.0;
d_T10_dV_b_GND = (n*d_Vtm_dV_b_GND+d_n_dV_b_GND*Vtm);
d_T10_dV_p_GND = (n*d_Vtm_dV_p_GND+d_n_dV_p_GND*Vtm);
d_T10_dTemp_t_GND = (n*d_Vtm_dTemp_t_GND+d_n_dTemp_t_GND*Vtm);
d_T10_dV_gi_si = d_n_dV_gi_si*Vtm;
d_T10_dV_di_si = d_n_dV_di_si*Vtm;
d_T10_dV_e_si = d_n_dV_e_si*Vtm;
d_T10_dV_b_si = d_n_dV_b_si*Vtm;
T10 = (n*Vtm);

d_VgstNVt_dV_db_di = d_VgstNVt_dV_sb_si =  0.0;
d_VgstNVt_dV_e_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_e_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_e_si)/T10/T10);
d_VgstNVt_dV_b_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_b_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_b_si)/T10/T10);
d_VgstNVt_dV_b_GND = ((T10*pParam_B4SOImstar*d_Vgst_dV_b_GND-(pParam_B4SOImstar*Vgst)*d_T10_dV_b_GND)/T10/T10);
d_VgstNVt_dV_p_GND = ((T10*pParam_B4SOImstar*d_Vgst_dV_p_GND-(pParam_B4SOImstar*Vgst)*d_T10_dV_p_GND)/T10/T10);
d_VgstNVt_dTemp_t_GND = ((T10*pParam_B4SOImstar*d_Vgst_dTemp_t_GND-(pParam_B4SOImstar*Vgst)*d_T10_dTemp_t_GND)/T10/T10);
d_VgstNVt_dV_gi_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_gi_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_gi_si)/T10/T10);
d_VgstNVt_dV_di_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_di_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_di_si)/T10/T10);
VgstNVt = ((pParam_B4SOImstar*Vgst)/T10);

d_ExpArg_dV_e_si = ((T10*(-(1-pParam_B4SOImstar)*d_Vgst_dV_e_si)-(pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))*d_T10_dV_e_si)/T10/T10);
d_ExpArg_dV_b_si = ((T10*(-(1-pParam_B4SOImstar)*d_Vgst_dV_b_si)-(pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))*d_T10_dV_b_si)/T10/T10);
d_ExpArg_dV_b_GND = ((T10*(-(1-pParam_B4SOImstar)*d_Vgst_dV_b_GND)-(pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))*d_T10_dV_b_GND)/T10/T10);
d_ExpArg_dV_p_GND = ((T10*(-(1-pParam_B4SOImstar)*d_Vgst_dV_p_GND)-(pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))*d_T10_dV_p_GND)/T10/T10);
d_ExpArg_dTemp_t_GND = ((T10*(-(1-pParam_B4SOImstar)*d_Vgst_dTemp_t_GND)-(pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))*d_T10_dTemp_t_GND)/T10/T10);
d_ExpArg_dV_gi_si = ((T10*(-(1-pParam_B4SOImstar)*d_Vgst_dV_gi_si)-(pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))*d_T10_dV_gi_si)/T10/T10);
d_ExpArg_dV_di_si = ((T10*(-(1-pParam_B4SOImstar)*d_Vgst_dV_di_si)-(pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))*d_T10_dV_di_si)/T10/T10);
ExpArg = ((pParam_B4SOIvoff-((1-pParam_B4SOImstar)*Vgst))/T10);
if ((VgstNVt>100.0))
{

d_Vgsteff_dV_gi_p = d_Vgsteff_dV_db_di = d_Vgsteff_dV_sb_si =  0.0;
d_Vgsteff_dV_e_si = d_Vgst_dV_e_si;
d_Vgsteff_dV_b_si = d_Vgst_dV_b_si;
d_Vgsteff_dV_b_GND = d_Vgst_dV_b_GND;
d_Vgsteff_dV_p_GND = d_Vgst_dV_p_GND;
d_Vgsteff_dTemp_t_GND = d_Vgst_dTemp_t_GND;
d_Vgsteff_dV_gi_si = d_Vgst_dV_gi_si;
d_Vgsteff_dV_di_si = d_Vgst_dV_di_si;
Vgsteff = Vgst;

d_ExpVgst_dV_gi_p = d_ExpVgst_dV_db_di = d_ExpVgst_dV_sb_si = d_ExpVgst_dV_b_si = d_ExpVgst_dV_di_si = d_ExpVgst_dV_gi_si = d_ExpVgst_dV_e_si = d_ExpVgst_dTemp_t_GND = d_ExpVgst_dV_p_GND = d_ExpVgst_dV_b_GND =  0.0;
ExpVgst = 0;
}
else
{
if ((ExpArg>100.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (((n*Vtm)*d_Vgst_dV_e_si-(Vgst-pParam_B4SOIvoff)*d_n_dV_e_si*Vtm)/(n*Vtm)/(n*Vtm));
d_T0_dV_b_si = (((n*Vtm)*d_Vgst_dV_b_si-(Vgst-pParam_B4SOIvoff)*d_n_dV_b_si*Vtm)/(n*Vtm)/(n*Vtm));
d_T0_dV_b_GND = (((n*Vtm)*d_Vgst_dV_b_GND-(Vgst-pParam_B4SOIvoff)*(n*d_Vtm_dV_b_GND+d_n_dV_b_GND*Vtm))/(n*Vtm)/(n*Vtm));
d_T0_dV_p_GND = (((n*Vtm)*d_Vgst_dV_p_GND-(Vgst-pParam_B4SOIvoff)*(n*d_Vtm_dV_p_GND+d_n_dV_p_GND*Vtm))/(n*Vtm)/(n*Vtm));
d_T0_dTemp_t_GND = (((n*Vtm)*d_Vgst_dTemp_t_GND-(Vgst-pParam_B4SOIvoff)*(n*d_Vtm_dTemp_t_GND+d_n_dTemp_t_GND*Vtm))/(n*Vtm)/(n*Vtm));
d_T0_dV_gi_si = (((n*Vtm)*d_Vgst_dV_gi_si-(Vgst-pParam_B4SOIvoff)*d_n_dV_gi_si*Vtm)/(n*Vtm)/(n*Vtm));
d_T0_dV_di_si = (((n*Vtm)*d_Vgst_dV_di_si-(Vgst-pParam_B4SOIvoff)*d_n_dV_di_si*Vtm)/(n*Vtm)/(n*Vtm));
T0 = ((Vgst-pParam_B4SOIvoff)/(n*Vtm));
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst_dV_gi_p = d_ExpVgst_dV_db_di = d_ExpVgst_dV_sb_si =  0.0;
d_ExpVgst_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_ExpVgst_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_ExpVgst_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_ExpVgst_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_ExpVgst_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_ExpVgst_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_ExpVgst_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
ExpVgst = value_exp_0;
}

d_Vgsteff_dV_gi_p = d_Vgsteff_dV_db_di = d_Vgsteff_dV_sb_si =  0.0;
d_Vgsteff_dV_e_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_e_si;
d_Vgsteff_dV_gi_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_gi_si;
d_Vgsteff_dV_di_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_di_si;
d_Vgsteff_dV_b_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_b_si;
d_Vgsteff_dV_b_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_b_GND+((Vtm*d_cdep0_dV_b_GND+d_Vtm_dV_b_GND*cdep0)/B4SOIcox)*ExpVgst);
d_Vgsteff_dV_p_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_p_GND+((Vtm*d_cdep0_dV_p_GND+d_Vtm_dV_p_GND*cdep0)/B4SOIcox)*ExpVgst);
d_Vgsteff_dTemp_t_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dTemp_t_GND+((Vtm*d_cdep0_dTemp_t_GND+d_Vtm_dTemp_t_GND*cdep0)/B4SOIcox)*ExpVgst);
Vgsteff = (((Vtm*cdep0)/B4SOIcox)*ExpVgst);
}
else
{
{
double value_exp_0 = exp(VgstNVt);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst_dV_gi_p = d_ExpVgst_dV_db_di = d_ExpVgst_dV_sb_si =  0.0;
d_ExpVgst_dV_e_si = (deriv_exp_0_d0*(d_VgstNVt_dV_e_si));
d_ExpVgst_dV_b_si = (deriv_exp_0_d0*(d_VgstNVt_dV_b_si));
d_ExpVgst_dV_b_GND = (deriv_exp_0_d0*(d_VgstNVt_dV_b_GND));
d_ExpVgst_dV_p_GND = (deriv_exp_0_d0*(d_VgstNVt_dV_p_GND));
d_ExpVgst_dTemp_t_GND = (deriv_exp_0_d0*(d_VgstNVt_dTemp_t_GND));
d_ExpVgst_dV_gi_si = (deriv_exp_0_d0*(d_VgstNVt_dV_gi_si));
d_ExpVgst_dV_di_si = (deriv_exp_0_d0*(d_VgstNVt_dV_di_si));
ExpVgst = value_exp_0;
}
{
double value_log_0 = log((1.0+ExpVgst));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVgst));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (T10*(deriv_log_0_d0*(d_ExpVgst_dV_b_GND))+d_T10_dV_b_GND*value_log_0);
d_T1_dV_p_GND = (T10*(deriv_log_0_d0*(d_ExpVgst_dV_p_GND))+d_T10_dV_p_GND*value_log_0);
d_T1_dTemp_t_GND = (T10*(deriv_log_0_d0*(d_ExpVgst_dTemp_t_GND))+d_T10_dTemp_t_GND*value_log_0);
d_T1_dV_gi_si = (T10*(deriv_log_0_d0*(d_ExpVgst_dV_gi_si))+d_T10_dV_gi_si*value_log_0);
d_T1_dV_di_si = (T10*(deriv_log_0_d0*(d_ExpVgst_dV_di_si))+d_T10_dV_di_si*value_log_0);
d_T1_dV_e_si = (T10*(deriv_log_0_d0*(d_ExpVgst_dV_e_si))+d_T10_dV_e_si*value_log_0);
d_T1_dV_b_si = (T10*(deriv_log_0_d0*(d_ExpVgst_dV_b_si))+d_T10_dV_b_si*value_log_0);
T1 = (T10*value_log_0);
}
{
double value_exp_0 = exp(ExpArg);
double  deriv_exp_0_d0 = value_exp_0;

d_dT2_dVg_dV_e_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_e_si))*(1-pParam_B4SOImstar);
d_dT2_dVg_dV_b_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_b_si))*(1-pParam_B4SOImstar);
d_dT2_dVg_dV_gi_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_gi_si))*(1-pParam_B4SOImstar);
d_dT2_dVg_dV_di_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_di_si))*(1-pParam_B4SOImstar);
d_dT2_dVg_dV_b_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_b_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dV_b_GND+d_Vtm_dV_b_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstar);
d_dT2_dVg_dV_p_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_p_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dV_p_GND+d_Vtm_dV_p_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstar);
d_dT2_dVg_dTemp_t_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dTemp_t_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dTemp_t_GND+d_Vtm_dTemp_t_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstar);
dT2_dVg = ((((-B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstar));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (-((T10*d_dT2_dVg_dV_b_GND+d_T10_dV_b_GND*dT2_dVg)/(1.0-pParam_B4SOImstar)));
d_T2_dV_p_GND = (-((T10*d_dT2_dVg_dV_p_GND+d_T10_dV_p_GND*dT2_dVg)/(1.0-pParam_B4SOImstar)));
d_T2_dTemp_t_GND = (-((T10*d_dT2_dVg_dTemp_t_GND+d_T10_dTemp_t_GND*dT2_dVg)/(1.0-pParam_B4SOImstar)));
d_T2_dV_gi_si = (-((T10*d_dT2_dVg_dV_gi_si+d_T10_dV_gi_si*dT2_dVg)/(1.0-pParam_B4SOImstar)));
d_T2_dV_di_si = (-((T10*d_dT2_dVg_dV_di_si+d_T10_dV_di_si*dT2_dVg)/(1.0-pParam_B4SOImstar)));
d_T2_dV_e_si = (-((T10*d_dT2_dVg_dV_e_si+d_T10_dV_e_si*dT2_dVg)/(1.0-pParam_B4SOImstar)));
d_T2_dV_b_si = (-((T10*d_dT2_dVg_dV_b_si+d_T10_dV_b_si*dT2_dVg)/(1.0-pParam_B4SOImstar)));
T2 = (pParam_B4SOImstar-((T10*dT2_dVg)/(1.0-pParam_B4SOImstar)));

d_Vgsteff_dV_gi_p = d_Vgsteff_dV_db_di = d_Vgsteff_dV_sb_si =  0.0;
d_Vgsteff_dV_b_GND = ((T2*d_T1_dV_b_GND-T1*d_T2_dV_b_GND)/T2/T2);
d_Vgsteff_dV_p_GND = ((T2*d_T1_dV_p_GND-T1*d_T2_dV_p_GND)/T2/T2);
d_Vgsteff_dTemp_t_GND = ((T2*d_T1_dTemp_t_GND-T1*d_T2_dTemp_t_GND)/T2/T2);
d_Vgsteff_dV_gi_si = ((T2*d_T1_dV_gi_si-T1*d_T2_dV_gi_si)/T2/T2);
d_Vgsteff_dV_di_si = ((T2*d_T1_dV_di_si-T1*d_T2_dV_di_si)/T2/T2);
d_Vgsteff_dV_e_si = ((T2*d_T1_dV_e_si-T1*d_T2_dV_e_si)/T2/T2);
d_Vgsteff_dV_b_si = ((T2*d_T1_dV_b_si-T1*d_T2_dV_b_si)/T2/T2);
Vgsteff = (T1/T2);
}
}

d_Vgst2Vtm_dV_e_si = d_Vgsteff_dV_e_si;
d_Vgst2Vtm_dV_b_si = d_Vgsteff_dV_b_si;
d_Vgst2Vtm_dV_b_GND = (d_Vgsteff_dV_b_GND+2.0*d_Vtm_dV_b_GND);
d_Vgst2Vtm_dV_p_GND = (d_Vgsteff_dV_p_GND+2.0*d_Vtm_dV_p_GND);
d_Vgst2Vtm_dTemp_t_GND = (d_Vgsteff_dTemp_t_GND+2.0*d_Vtm_dTemp_t_GND);
d_Vgst2Vtm_dV_gi_si = d_Vgsteff_dV_gi_si;
d_Vgst2Vtm_dV_di_si = d_Vgsteff_dV_di_si;
Vgst2Vtm = (Vgsteff+(2.0*Vtm));
B4SOIVgsteff = Vgsteff;
if ((pParam_B4SOIfprout<=0.0))
{

d_FP_dV_b_si = d_FP_dV_e_si = d_FP_dV_di_si = d_FP_dV_gi_si = d_FP_dTemp_t_GND = d_FP_dV_p_GND = d_FP_dV_b_GND =  0.0;
FP = 1.0;
}
else
{
{
double value_sqrt_0 = sqrt(Leff);

d_T9_dV_e_si = (-(pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dV_e_si/Vgst2Vtm/Vgst2Vtm);
d_T9_dV_b_si = (-(pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dV_b_si/Vgst2Vtm/Vgst2Vtm);
d_T9_dV_b_GND = (-(pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dV_b_GND/Vgst2Vtm/Vgst2Vtm);
d_T9_dV_p_GND = (-(pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dV_p_GND/Vgst2Vtm/Vgst2Vtm);
d_T9_dTemp_t_GND = (-(pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dTemp_t_GND/Vgst2Vtm/Vgst2Vtm);
d_T9_dV_gi_si = (-(pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dV_gi_si/Vgst2Vtm/Vgst2Vtm);
d_T9_dV_di_si = (-(pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dV_di_si/Vgst2Vtm/Vgst2Vtm);
T9 = ((pParam_B4SOIfprout*value_sqrt_0)/Vgst2Vtm);
}

d_FP_dV_b_GND = (-d_T9_dV_b_GND/(1.0+T9)/(1.0+T9));
d_FP_dV_p_GND = (-d_T9_dV_p_GND/(1.0+T9)/(1.0+T9));
d_FP_dTemp_t_GND = (-d_T9_dTemp_t_GND/(1.0+T9)/(1.0+T9));
d_FP_dV_gi_si = (-d_T9_dV_gi_si/(1.0+T9)/(1.0+T9));
d_FP_dV_di_si = (-d_T9_dV_di_si/(1.0+T9)/(1.0+T9));
d_FP_dV_e_si = (-d_T9_dV_e_si/(1.0+T9)/(1.0+T9));
d_FP_dV_b_si = (-d_T9_dV_b_si/(1.0+T9)/(1.0+T9));
FP = (1.0/(1.0+T9));
}

d_T9_dV_e_si = d_sqrtPhis_dV_e_si;
d_T9_dV_gi_si = d_sqrtPhis_dV_gi_si;
d_T9_dV_di_si = d_sqrtPhis_dV_di_si;
d_T9_dV_b_GND = (d_sqrtPhis_dV_b_GND-d_sqrtPhi_dV_b_GND);
d_T9_dV_p_GND = (d_sqrtPhis_dV_p_GND-d_sqrtPhi_dV_p_GND);
d_T9_dTemp_t_GND = (d_sqrtPhis_dTemp_t_GND-d_sqrtPhi_dTemp_t_GND);
d_T9_dV_b_si = d_sqrtPhis_dV_b_si;
T9 = (sqrtPhis-sqrtPhi);

d_Weff_dV_e_si = (-(2.0-B4SOInbc)*(pParam_B4SOIdwg*d_Vgsteff_dV_e_si+pParam_B4SOIdwb*d_T9_dV_e_si));
d_Weff_dV_b_si = (-(2.0-B4SOInbc)*(pParam_B4SOIdwg*d_Vgsteff_dV_b_si+pParam_B4SOIdwb*d_T9_dV_b_si));
d_Weff_dV_b_GND = (-(2.0-B4SOInbc)*(pParam_B4SOIdwg*d_Vgsteff_dV_b_GND+pParam_B4SOIdwb*d_T9_dV_b_GND));
d_Weff_dV_p_GND = (-(2.0-B4SOInbc)*(pParam_B4SOIdwg*d_Vgsteff_dV_p_GND+pParam_B4SOIdwb*d_T9_dV_p_GND));
d_Weff_dTemp_t_GND = (-(2.0-B4SOInbc)*(pParam_B4SOIdwg*d_Vgsteff_dTemp_t_GND+pParam_B4SOIdwb*d_T9_dTemp_t_GND));
d_Weff_dV_gi_si = (-(2.0-B4SOInbc)*(pParam_B4SOIdwg*d_Vgsteff_dV_gi_si+pParam_B4SOIdwb*d_T9_dV_gi_si));
d_Weff_dV_di_si = (-(2.0-B4SOInbc)*(pParam_B4SOIdwg*d_Vgsteff_dV_di_si+pParam_B4SOIdwb*d_T9_dV_di_si));
Weff = (pParam_B4SOIweff-((2.0-B4SOInbc)*((pParam_B4SOIdwg*Vgsteff)+(pParam_B4SOIdwb*T9))));
if ((Weff<2.0e-8))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (-(-2.0*d_Weff_dV_e_si)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
d_T0_dV_b_si = (-(-2.0*d_Weff_dV_b_si)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
d_T0_dV_b_GND = (-(-2.0*d_Weff_dV_b_GND)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
d_T0_dV_p_GND = (-(-2.0*d_Weff_dV_p_GND)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
d_T0_dTemp_t_GND = (-(-2.0*d_Weff_dTemp_t_GND)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
d_T0_dV_gi_si = (-(-2.0*d_Weff_dV_gi_si)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
d_T0_dV_di_si = (-(-2.0*d_Weff_dV_di_si)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
T0 = (1.0/(6.0e-8-(2.0*Weff)));

d_Weff_dV_e_si = ((2.0e-8*(4.0e-8-Weff))*d_T0_dV_e_si+2.0e-8*(-d_Weff_dV_e_si)*T0);
d_Weff_dV_b_si = ((2.0e-8*(4.0e-8-Weff))*d_T0_dV_b_si+2.0e-8*(-d_Weff_dV_b_si)*T0);
d_Weff_dV_b_GND = ((2.0e-8*(4.0e-8-Weff))*d_T0_dV_b_GND+2.0e-8*(-d_Weff_dV_b_GND)*T0);
d_Weff_dV_p_GND = ((2.0e-8*(4.0e-8-Weff))*d_T0_dV_p_GND+2.0e-8*(-d_Weff_dV_p_GND)*T0);
d_Weff_dTemp_t_GND = ((2.0e-8*(4.0e-8-Weff))*d_T0_dTemp_t_GND+2.0e-8*(-d_Weff_dTemp_t_GND)*T0);
d_Weff_dV_gi_si = ((2.0e-8*(4.0e-8-Weff))*d_T0_dV_gi_si+2.0e-8*(-d_Weff_dV_gi_si)*T0);
d_Weff_dV_di_si = ((2.0e-8*(4.0e-8-Weff))*d_T0_dV_di_si+2.0e-8*(-d_Weff_dV_di_si)*T0);
Weff = ((2.0e-8*(4.0e-8-Weff))*T0);
}
if ((B4SOIrdsMod==1))
{

d_Rds_dTemp_t_GND = d_Rds_dV_p_GND = d_Rds_dV_b_GND = d_Rds_dV_b_si = d_Rds_dV_di_si = d_Rds_dV_gi_si = d_Rds_dV_e_si =  0.0;
Rds = 0.0;
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (pParam_B4SOIprwg*d_Vgsteff_dV_e_si+pParam_B4SOIprwb*d_T9_dV_e_si);
d_T0_dV_b_si = (pParam_B4SOIprwg*d_Vgsteff_dV_b_si+pParam_B4SOIprwb*d_T9_dV_b_si);
d_T0_dV_b_GND = (pParam_B4SOIprwg*d_Vgsteff_dV_b_GND+pParam_B4SOIprwb*d_T9_dV_b_GND);
d_T0_dV_p_GND = (pParam_B4SOIprwg*d_Vgsteff_dV_p_GND+pParam_B4SOIprwb*d_T9_dV_p_GND);
d_T0_dTemp_t_GND = (pParam_B4SOIprwg*d_Vgsteff_dTemp_t_GND+pParam_B4SOIprwb*d_T9_dTemp_t_GND);
d_T0_dV_gi_si = (pParam_B4SOIprwg*d_Vgsteff_dV_gi_si+pParam_B4SOIprwb*d_T9_dV_gi_si);
d_T0_dV_di_si = (pParam_B4SOIprwg*d_Vgsteff_dV_di_si+pParam_B4SOIprwb*d_T9_dV_di_si);
T0 = ((pParam_B4SOIprwg*Vgsteff)+(pParam_B4SOIprwb*T9));
if ((T0>=(-0.9)))
{

d_Rds_dV_e_si = rds0*d_T0_dV_e_si;
d_Rds_dV_gi_si = rds0*d_T0_dV_gi_si;
d_Rds_dV_di_si = rds0*d_T0_dV_di_si;
d_Rds_dV_b_si = rds0*d_T0_dV_b_si;
d_Rds_dV_b_GND = (rds0*d_T0_dV_b_GND+d_rds0_dV_b_GND*(1.0+T0));
d_Rds_dV_p_GND = (rds0*d_T0_dV_p_GND+d_rds0_dV_p_GND*(1.0+T0));
d_Rds_dTemp_t_GND = (rds0*d_T0_dTemp_t_GND+d_rds0_dTemp_t_GND*(1.0+T0));
Rds = (rds0*(1.0+T0));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (-20.0*d_T0_dV_b_GND/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
d_T1_dV_p_GND = (-20.0*d_T0_dV_p_GND/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
d_T1_dTemp_t_GND = (-20.0*d_T0_dTemp_t_GND/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
d_T1_dV_e_si = (-20.0*d_T0_dV_e_si/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
d_T1_dV_gi_si = (-20.0*d_T0_dV_gi_si/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
d_T1_dV_di_si = (-20.0*d_T0_dV_di_si/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
d_T1_dV_b_si = (-20.0*d_T0_dV_b_si/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
T1 = (1.0/(17.0+(20.0*T0)));

d_Rds_dV_e_si = ((rds0*(0.8+T0))*d_T1_dV_e_si+rds0*d_T0_dV_e_si*T1);
d_Rds_dV_gi_si = ((rds0*(0.8+T0))*d_T1_dV_gi_si+rds0*d_T0_dV_gi_si*T1);
d_Rds_dV_di_si = ((rds0*(0.8+T0))*d_T1_dV_di_si+rds0*d_T0_dV_di_si*T1);
d_Rds_dV_b_si = ((rds0*(0.8+T0))*d_T1_dV_b_si+rds0*d_T0_dV_b_si*T1);
d_Rds_dV_b_GND = ((rds0*(0.8+T0))*d_T1_dV_b_GND+(rds0*d_T0_dV_b_GND+d_rds0_dV_b_GND*(0.8+T0))*T1);
d_Rds_dV_p_GND = ((rds0*(0.8+T0))*d_T1_dV_p_GND+(rds0*d_T0_dV_p_GND+d_rds0_dV_p_GND*(0.8+T0))*T1);
d_Rds_dTemp_t_GND = ((rds0*(0.8+T0))*d_T1_dTemp_t_GND+(rds0*d_T0_dTemp_t_GND+d_rds0_dTemp_t_GND*(0.8+T0))*T1);
Rds = ((rds0*(0.8+T0))*T1);
}
}
if ((B4SOIrdsMod==2))
{

Rds = ((B4SOIdrainResistance+Rds)+B4SOIsourceResistance);
}
B4SOIrds = (Rds/B4SOInf);
if ((pParam_B4SOIa0==0.0))
{

d_Abulk_dV_b_si = d_Abulk_dV_e_si = d_Abulk_dV_di_si = d_Abulk_dV_gi_si = d_Abulk_dTemp_t_GND = d_Abulk_dV_p_GND = d_Abulk_dV_b_GND =  0.0;
Abulk = 1.0;

d_Abulk0_dV_b_si = d_Abulk0_dV_e_si = d_Abulk0_dV_di_si = d_Abulk0_dV_gi_si = d_Abulk0_dTemp_t_GND = d_Abulk0_dV_p_GND = d_Abulk0_dV_b_GND =  0.0;
Abulk0 = 1.0;
}
else
{

d_T10_dV_db_di = d_T10_dV_sb_si =  0.0;
d_T10_dV_b_GND = pParam_B4SOIketa*d_Vbsh_dV_b_GND;
d_T10_dV_p_GND = pParam_B4SOIketa*d_Vbsh_dV_p_GND;
d_T10_dTemp_t_GND = pParam_B4SOIketa*d_Vbsh_dTemp_t_GND;
d_T10_dV_e_si = pParam_B4SOIketa*d_Vbsh_dV_e_si;
d_T10_dV_gi_si = pParam_B4SOIketa*d_Vbsh_dV_gi_si;
d_T10_dV_di_si = pParam_B4SOIketa*d_Vbsh_dV_di_si;
d_T10_dV_b_si = pParam_B4SOIketa*d_Vbsh_dV_b_si;
T10 = (pParam_B4SOIketa*Vbsh);
if ((T10>=(-0.5)))
{

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si =  0.0;
d_T11_dV_b_GND = (-d_T10_dV_b_GND/(1.0+T10)/(1.0+T10));
d_T11_dV_p_GND = (-d_T10_dV_p_GND/(1.0+T10)/(1.0+T10));
d_T11_dTemp_t_GND = (-d_T10_dTemp_t_GND/(1.0+T10)/(1.0+T10));
d_T11_dV_gi_si = (-d_T10_dV_gi_si/(1.0+T10)/(1.0+T10));
d_T11_dV_di_si = (-d_T10_dV_di_si/(1.0+T10)/(1.0+T10));
d_T11_dV_e_si = (-d_T10_dV_e_si/(1.0+T10)/(1.0+T10));
d_T11_dV_b_si = (-d_T10_dV_b_si/(1.0+T10)/(1.0+T10));
T11 = (1.0/(1.0+T10));
}
else
{

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si = d_T12_dV_b_si = d_T12_dV_e_si = d_T12_dV_di_si = d_T12_dV_gi_si = d_T12_dTemp_t_GND = d_T12_dV_p_GND = d_T12_dV_b_GND =  0.0;
T12 = ((-1.0)/((1.0-0.5)*(1.0-0.5)));

d_T13_dV_b_si = d_T13_dV_di_si = d_T13_dV_gi_si = d_T13_dV_e_si = d_T13_dTemp_t_GND = d_T13_dV_p_GND = d_T13_dV_b_GND =  0.0;
T13 = ((1.0/(1.0-0.5))+(T12*0.5));

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si =  0.0;
d_T11_dV_b_GND = ((T12*d_T10_dV_b_GND+d_T12_dV_b_GND*T10)+d_T13_dV_b_GND);
d_T11_dV_p_GND = ((T12*d_T10_dV_p_GND+d_T12_dV_p_GND*T10)+d_T13_dV_p_GND);
d_T11_dTemp_t_GND = ((T12*d_T10_dTemp_t_GND+d_T12_dTemp_t_GND*T10)+d_T13_dTemp_t_GND);
d_T11_dV_gi_si = ((T12*d_T10_dV_gi_si+d_T12_dV_gi_si*T10)+d_T13_dV_gi_si);
d_T11_dV_di_si = ((T12*d_T10_dV_di_si+d_T12_dV_di_si*T10)+d_T13_dV_di_si);
d_T11_dV_e_si = ((T12*d_T10_dV_e_si+d_T12_dV_e_si*T10)+d_T13_dV_e_si);
d_T11_dV_b_si = ((T12*d_T10_dV_b_si+d_T12_dV_b_si*T10)+d_T13_dV_b_si);
T11 = ((T12*T10)+T13);
}

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si =  0.0;
d_T10_dV_b_GND = d_phi_dV_b_GND;
d_T10_dV_p_GND = d_phi_dV_p_GND;
d_T10_dTemp_t_GND = d_phi_dTemp_t_GND;
T10 = (phi+pParam_B4SOIketas);

d_T13_dV_b_GND = ((T10*(Vbsh*d_T11_dV_b_GND+d_Vbsh_dV_b_GND*T11)-(Vbsh*T11)*d_T10_dV_b_GND)/T10/T10);
d_T13_dV_p_GND = ((T10*(Vbsh*d_T11_dV_p_GND+d_Vbsh_dV_p_GND*T11)-(Vbsh*T11)*d_T10_dV_p_GND)/T10/T10);
d_T13_dTemp_t_GND = ((T10*(Vbsh*d_T11_dTemp_t_GND+d_Vbsh_dTemp_t_GND*T11)-(Vbsh*T11)*d_T10_dTemp_t_GND)/T10/T10);
d_T13_dV_e_si = ((T10*(Vbsh*d_T11_dV_e_si+d_Vbsh_dV_e_si*T11)-(Vbsh*T11)*d_T10_dV_e_si)/T10/T10);
d_T13_dV_gi_si = ((T10*(Vbsh*d_T11_dV_gi_si+d_Vbsh_dV_gi_si*T11)-(Vbsh*T11)*d_T10_dV_gi_si)/T10/T10);
d_T13_dV_di_si = ((T10*(Vbsh*d_T11_dV_di_si+d_Vbsh_dV_di_si*T11)-(Vbsh*T11)*d_T10_dV_di_si)/T10/T10);
d_T13_dV_b_si = ((T10*(Vbsh*d_T11_dV_b_si+d_Vbsh_dV_b_si*T11)-(Vbsh*T11)*d_T10_dV_b_si)/T10/T10);
T13 = ((Vbsh*T11)/T10);
if ((T13<0.50))
{
{
double value_sqrt_0 = sqrt((1.0-T13));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T14_dV_b_GND = (-(deriv_sqrt_0_d0*((-d_T13_dV_b_GND)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_p_GND = (-(deriv_sqrt_0_d0*((-d_T13_dV_p_GND)))/value_sqrt_0/value_sqrt_0);
d_T14_dTemp_t_GND = (-(deriv_sqrt_0_d0*((-d_T13_dTemp_t_GND)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_e_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_e_si)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_gi_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_gi_si)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_di_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_di_si)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_b_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_b_si)))/value_sqrt_0/value_sqrt_0);
T14 = (1.0/value_sqrt_0);
}
}
else
{
{
double value_sqrt_0 = sqrt((1-0.50));

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = (1.0/((2*(1-0.50))*value_sqrt_0));
}
{
double value_sqrt_0 = sqrt((1.0-0.50));

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si =  0.0;
d_T12_dV_b_GND = (-d_T11_dV_b_GND*0.50);
d_T12_dV_p_GND = (-d_T11_dV_p_GND*0.50);
d_T12_dTemp_t_GND = (-d_T11_dTemp_t_GND*0.50);
d_T12_dV_gi_si = (-d_T11_dV_gi_si*0.50);
d_T12_dV_di_si = (-d_T11_dV_di_si*0.50);
d_T12_dV_e_si = (-d_T11_dV_e_si*0.50);
d_T12_dV_b_si = (-d_T11_dV_b_si*0.50);
T12 = ((1/value_sqrt_0)-(T11*0.50));
}

d_T14_dV_b_GND = ((T11*d_T13_dV_b_GND+d_T11_dV_b_GND*T13)+d_T12_dV_b_GND);
d_T14_dV_p_GND = ((T11*d_T13_dV_p_GND+d_T11_dV_p_GND*T13)+d_T12_dV_p_GND);
d_T14_dTemp_t_GND = ((T11*d_T13_dTemp_t_GND+d_T11_dTemp_t_GND*T13)+d_T12_dTemp_t_GND);
d_T14_dV_gi_si = ((T11*d_T13_dV_gi_si+d_T11_dV_gi_si*T13)+d_T12_dV_gi_si);
d_T14_dV_di_si = ((T11*d_T13_dV_di_si+d_T11_dV_di_si*T13)+d_T12_dV_di_si);
d_T14_dV_e_si = ((T11*d_T13_dV_e_si+d_T11_dV_e_si*T13)+d_T12_dV_e_si);
d_T14_dV_b_si = ((T11*d_T13_dV_b_si+d_T11_dV_b_si*T13)+d_T12_dV_b_si);
T14 = ((T11*T13)+T12);
}
{
double value_sqrt_0 = sqrt((phi+pParam_B4SOIketas));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si =  0.0;
d_T10_dV_b_GND = (-((0.5*pParam_B4SOIk1ox)*Lpe_Vb)*(deriv_sqrt_0_d0*(d_phi_dV_b_GND))/value_sqrt_0/value_sqrt_0);
d_T10_dV_p_GND = (-((0.5*pParam_B4SOIk1ox)*Lpe_Vb)*(deriv_sqrt_0_d0*(d_phi_dV_p_GND))/value_sqrt_0/value_sqrt_0);
d_T10_dTemp_t_GND = (-((0.5*pParam_B4SOIk1ox)*Lpe_Vb)*(deriv_sqrt_0_d0*(d_phi_dTemp_t_GND))/value_sqrt_0/value_sqrt_0);
T10 = (((0.5*pParam_B4SOIk1ox)*Lpe_Vb)/value_sqrt_0);
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (T10*d_T14_dV_b_GND+d_T10_dV_b_GND*T14);
d_T1_dV_p_GND = (T10*d_T14_dV_p_GND+d_T10_dV_p_GND*T14);
d_T1_dTemp_t_GND = (T10*d_T14_dTemp_t_GND+d_T10_dTemp_t_GND*T14);
d_T1_dV_gi_si = (T10*d_T14_dV_gi_si+d_T10_dV_gi_si*T14);
d_T1_dV_di_si = (T10*d_T14_dV_di_si+d_T10_dV_di_si*T14);
d_T1_dV_e_si = (T10*d_T14_dV_e_si+d_T10_dV_e_si*T14);
d_T1_dV_b_si = (T10*d_T14_dV_b_si+d_T10_dV_b_si*T14);
T1 = (T10*T14);
{
double value_sqrt_0 = sqrt((pParam_B4SOIxj*Xdep));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T9_dV_e_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_dV_e_si));
d_T9_dV_gi_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_dV_gi_si));
d_T9_dV_di_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_dV_di_si));
d_T9_dV_b_GND = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_dV_b_GND));
d_T9_dV_p_GND = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_dV_p_GND));
d_T9_dTemp_t_GND = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_dTemp_t_GND));
d_T9_dV_b_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_dV_b_si));
T9 = value_sqrt_0;
}

d_tmp1_dV_b_GND = 2.0*d_T9_dV_b_GND;
d_tmp1_dV_p_GND = 2.0*d_T9_dV_p_GND;
d_tmp1_dTemp_t_GND = 2.0*d_T9_dTemp_t_GND;
d_tmp1_dV_gi_si = 2.0*d_T9_dV_gi_si;
d_tmp1_dV_di_si = 2.0*d_T9_dV_di_si;
d_tmp1_dV_e_si = 2.0*d_T9_dV_e_si;
d_tmp1_dV_b_si = 2.0*d_T9_dV_b_si;
tmp1 = (Leff+(2.0*T9));

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = (-Leff*d_tmp1_dV_b_GND/tmp1/tmp1);
d_T5_dV_p_GND = (-Leff*d_tmp1_dV_p_GND/tmp1/tmp1);
d_T5_dTemp_t_GND = (-Leff*d_tmp1_dTemp_t_GND/tmp1/tmp1);
d_T5_dV_gi_si = (-Leff*d_tmp1_dV_gi_si/tmp1/tmp1);
d_T5_dV_di_si = (-Leff*d_tmp1_dV_di_si/tmp1/tmp1);
d_T5_dV_e_si = (-Leff*d_tmp1_dV_e_si/tmp1/tmp1);
d_T5_dV_b_si = (-Leff*d_tmp1_dV_b_si/tmp1/tmp1);
T5 = (Leff/tmp1);

d_tmp2_dV_b_GND = pParam_B4SOIa0*d_T5_dV_b_GND;
d_tmp2_dV_p_GND = pParam_B4SOIa0*d_T5_dV_p_GND;
d_tmp2_dTemp_t_GND = pParam_B4SOIa0*d_T5_dTemp_t_GND;
d_tmp2_dV_gi_si = pParam_B4SOIa0*d_T5_dV_gi_si;
d_tmp2_dV_di_si = pParam_B4SOIa0*d_T5_dV_di_si;
d_tmp2_dV_e_si = pParam_B4SOIa0*d_T5_dV_e_si;
d_tmp2_dV_b_si = pParam_B4SOIa0*d_T5_dV_b_si;
tmp2 = (pParam_B4SOIa0*T5);

d_tmp3_dV_b_si = d_tmp3_dV_e_si = d_tmp3_dV_di_si = d_tmp3_dV_gi_si = d_tmp3_dTemp_t_GND = d_tmp3_dV_p_GND = d_tmp3_dV_b_GND =  0.0;
tmp3 = (pParam_B4SOIweff+pParam_B4SOIb1);

d_tmp4_dV_b_si = d_tmp4_dV_e_si = d_tmp4_dV_di_si = d_tmp4_dV_gi_si = d_tmp4_dTemp_t_GND = d_tmp4_dV_p_GND = d_tmp4_dV_b_GND =  0.0;
tmp4 = (pParam_B4SOIb0/tmp3);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (d_tmp2_dV_b_GND+d_tmp4_dV_b_GND);
d_T2_dV_p_GND = (d_tmp2_dV_p_GND+d_tmp4_dV_p_GND);
d_T2_dTemp_t_GND = (d_tmp2_dTemp_t_GND+d_tmp4_dTemp_t_GND);
d_T2_dV_gi_si = (d_tmp2_dV_gi_si+d_tmp4_dV_gi_si);
d_T2_dV_di_si = (d_tmp2_dV_di_si+d_tmp4_dV_di_si);
d_T2_dV_e_si = (d_tmp2_dV_e_si+d_tmp4_dV_e_si);
d_T2_dV_b_si = (d_tmp2_dV_b_si+d_tmp4_dV_b_si);
T2 = (tmp2+tmp4);

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_b_GND = (T5*d_T5_dV_b_GND+d_T5_dV_b_GND*T5);
d_T6_dV_p_GND = (T5*d_T5_dV_p_GND+d_T5_dV_p_GND*T5);
d_T6_dTemp_t_GND = (T5*d_T5_dTemp_t_GND+d_T5_dTemp_t_GND*T5);
d_T6_dV_gi_si = (T5*d_T5_dV_gi_si+d_T5_dV_gi_si*T5);
d_T6_dV_di_si = (T5*d_T5_dV_di_si+d_T5_dV_di_si*T5);
d_T6_dV_e_si = (T5*d_T5_dV_e_si+d_T5_dV_e_si*T5);
d_T6_dV_b_si = (T5*d_T5_dV_b_si+d_T5_dV_b_si*T5);
T6 = (T5*T5);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = (T5*d_T6_dV_b_GND+d_T5_dV_b_GND*T6);
d_T7_dV_p_GND = (T5*d_T6_dV_p_GND+d_T5_dV_p_GND*T6);
d_T7_dTemp_t_GND = (T5*d_T6_dTemp_t_GND+d_T5_dTemp_t_GND*T6);
d_T7_dV_gi_si = (T5*d_T6_dV_gi_si+d_T5_dV_gi_si*T6);
d_T7_dV_di_si = (T5*d_T6_dV_di_si+d_T5_dV_di_si*T6);
d_T7_dV_e_si = (T5*d_T6_dV_e_si+d_T5_dV_e_si*T6);
d_T7_dV_b_si = (T5*d_T6_dV_b_si+d_T5_dV_b_si*T6);
T7 = (T5*T6);

d_Abulk0_dV_b_GND = (T1*d_T2_dV_b_GND+d_T1_dV_b_GND*T2);
d_Abulk0_dV_p_GND = (T1*d_T2_dV_p_GND+d_T1_dV_p_GND*T2);
d_Abulk0_dTemp_t_GND = (T1*d_T2_dTemp_t_GND+d_T1_dTemp_t_GND*T2);
d_Abulk0_dV_gi_si = (T1*d_T2_dV_gi_si+d_T1_dV_gi_si*T2);
d_Abulk0_dV_di_si = (T1*d_T2_dV_di_si+d_T1_dV_di_si*T2);
d_Abulk0_dV_e_si = (T1*d_T2_dV_e_si+d_T1_dV_e_si*T2);
d_Abulk0_dV_b_si = (T1*d_T2_dV_b_si+d_T1_dV_b_si*T2);
Abulk0 = (1+(T1*T2));

d_T8_dV_b_GND = (pParam_B4SOIags*pParam_B4SOIa0)*d_T7_dV_b_GND;
d_T8_dV_p_GND = (pParam_B4SOIags*pParam_B4SOIa0)*d_T7_dV_p_GND;
d_T8_dTemp_t_GND = (pParam_B4SOIags*pParam_B4SOIa0)*d_T7_dTemp_t_GND;
d_T8_dV_gi_si = (pParam_B4SOIags*pParam_B4SOIa0)*d_T7_dV_gi_si;
d_T8_dV_di_si = (pParam_B4SOIags*pParam_B4SOIa0)*d_T7_dV_di_si;
d_T8_dV_e_si = (pParam_B4SOIags*pParam_B4SOIa0)*d_T7_dV_e_si;
d_T8_dV_b_si = (pParam_B4SOIags*pParam_B4SOIa0)*d_T7_dV_b_si;
T8 = ((pParam_B4SOIags*pParam_B4SOIa0)*T7);

d_dAbulk_dVg_dV_b_GND = ((-T1)*d_T8_dV_b_GND+(-d_T1_dV_b_GND)*T8);
d_dAbulk_dVg_dV_p_GND = ((-T1)*d_T8_dV_p_GND+(-d_T1_dV_p_GND)*T8);
d_dAbulk_dVg_dTemp_t_GND = ((-T1)*d_T8_dTemp_t_GND+(-d_T1_dTemp_t_GND)*T8);
d_dAbulk_dVg_dV_gi_si = ((-T1)*d_T8_dV_gi_si+(-d_T1_dV_gi_si)*T8);
d_dAbulk_dVg_dV_di_si = ((-T1)*d_T8_dV_di_si+(-d_T1_dV_di_si)*T8);
d_dAbulk_dVg_dV_e_si = ((-T1)*d_T8_dV_e_si+(-d_T1_dV_e_si)*T8);
d_dAbulk_dVg_dV_b_si = ((-T1)*d_T8_dV_b_si+(-d_T1_dV_b_si)*T8);
dAbulk_dVg = ((-T1)*T8);

d_Abulk_dV_b_GND = (d_Abulk0_dV_b_GND+(dAbulk_dVg*d_Vgsteff_dV_b_GND+d_dAbulk_dVg_dV_b_GND*Vgsteff));
d_Abulk_dV_p_GND = (d_Abulk0_dV_p_GND+(dAbulk_dVg*d_Vgsteff_dV_p_GND+d_dAbulk_dVg_dV_p_GND*Vgsteff));
d_Abulk_dTemp_t_GND = (d_Abulk0_dTemp_t_GND+(dAbulk_dVg*d_Vgsteff_dTemp_t_GND+d_dAbulk_dVg_dTemp_t_GND*Vgsteff));
d_Abulk_dV_gi_si = (d_Abulk0_dV_gi_si+(dAbulk_dVg*d_Vgsteff_dV_gi_si+d_dAbulk_dVg_dV_gi_si*Vgsteff));
d_Abulk_dV_di_si = (d_Abulk0_dV_di_si+(dAbulk_dVg*d_Vgsteff_dV_di_si+d_dAbulk_dVg_dV_di_si*Vgsteff));
d_Abulk_dV_e_si = (d_Abulk0_dV_e_si+(dAbulk_dVg*d_Vgsteff_dV_e_si+d_dAbulk_dVg_dV_e_si*Vgsteff));
d_Abulk_dV_b_si = (d_Abulk0_dV_b_si+(dAbulk_dVg*d_Vgsteff_dV_b_si+d_dAbulk_dVg_dV_b_si*Vgsteff));
Abulk = (Abulk0+(dAbulk_dVg*Vgsteff));
}
if ((Abulk0<0.01))
{

d_T9_dV_b_GND = (-(-200.0*d_Abulk0_dV_b_GND)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
d_T9_dV_p_GND = (-(-200.0*d_Abulk0_dV_p_GND)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
d_T9_dTemp_t_GND = (-(-200.0*d_Abulk0_dTemp_t_GND)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
d_T9_dV_gi_si = (-(-200.0*d_Abulk0_dV_gi_si)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
d_T9_dV_di_si = (-(-200.0*d_Abulk0_dV_di_si)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
d_T9_dV_e_si = (-(-200.0*d_Abulk0_dV_e_si)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
d_T9_dV_b_si = (-(-200.0*d_Abulk0_dV_b_si)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
T9 = (1.0/(3.0-(200.0*Abulk0)));

d_Abulk0_dV_b_GND = ((0.02-Abulk0)*d_T9_dV_b_GND+(-d_Abulk0_dV_b_GND)*T9);
d_Abulk0_dV_p_GND = ((0.02-Abulk0)*d_T9_dV_p_GND+(-d_Abulk0_dV_p_GND)*T9);
d_Abulk0_dTemp_t_GND = ((0.02-Abulk0)*d_T9_dTemp_t_GND+(-d_Abulk0_dTemp_t_GND)*T9);
d_Abulk0_dV_gi_si = ((0.02-Abulk0)*d_T9_dV_gi_si+(-d_Abulk0_dV_gi_si)*T9);
d_Abulk0_dV_di_si = ((0.02-Abulk0)*d_T9_dV_di_si+(-d_Abulk0_dV_di_si)*T9);
d_Abulk0_dV_e_si = ((0.02-Abulk0)*d_T9_dV_e_si+(-d_Abulk0_dV_e_si)*T9);
d_Abulk0_dV_b_si = ((0.02-Abulk0)*d_T9_dV_b_si+(-d_Abulk0_dV_b_si)*T9);
Abulk0 = ((0.02-Abulk0)*T9);
}
if ((Abulk<0.01))
{

d_T9_dV_b_GND = (-(-200.0*d_Abulk_dV_b_GND)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
d_T9_dV_p_GND = (-(-200.0*d_Abulk_dV_p_GND)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
d_T9_dTemp_t_GND = (-(-200.0*d_Abulk_dTemp_t_GND)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
d_T9_dV_gi_si = (-(-200.0*d_Abulk_dV_gi_si)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
d_T9_dV_di_si = (-(-200.0*d_Abulk_dV_di_si)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
d_T9_dV_e_si = (-(-200.0*d_Abulk_dV_e_si)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
d_T9_dV_b_si = (-(-200.0*d_Abulk_dV_b_si)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
T9 = (1.0/(3.0-(200.0*Abulk)));

d_Abulk_dV_b_GND = ((0.02-Abulk)*d_T9_dV_b_GND+(-d_Abulk_dV_b_GND)*T9);
d_Abulk_dV_p_GND = ((0.02-Abulk)*d_T9_dV_p_GND+(-d_Abulk_dV_p_GND)*T9);
d_Abulk_dTemp_t_GND = ((0.02-Abulk)*d_T9_dTemp_t_GND+(-d_Abulk_dTemp_t_GND)*T9);
d_Abulk_dV_gi_si = ((0.02-Abulk)*d_T9_dV_gi_si+(-d_Abulk_dV_gi_si)*T9);
d_Abulk_dV_di_si = ((0.02-Abulk)*d_T9_dV_di_si+(-d_Abulk_dV_di_si)*T9);
d_Abulk_dV_e_si = ((0.02-Abulk)*d_T9_dV_e_si+(-d_Abulk_dV_e_si)*T9);
d_Abulk_dV_b_si = ((0.02-Abulk)*d_T9_dV_b_si+(-d_Abulk_dV_b_si)*T9);
Abulk = ((0.02-Abulk)*T9);
}
B4SOIAbulk = Abulk;
if ((pParam_B4SOIa0==0.0))
{

d_Abulk0_CV_dV_b_si = d_Abulk0_CV_dV_e_si = d_Abulk0_CV_dV_di_si = d_Abulk0_CV_dV_gi_si = d_Abulk0_CV_dTemp_t_GND = d_Abulk0_CV_dV_p_GND = d_Abulk0_CV_dV_b_GND =  0.0;
Abulk0_CV = 1.0;
}
else
{

d_T10_dV_db_di = d_T10_dV_sb_si =  0.0;
d_T10_dV_b_GND = pParam_B4SOIketa*d_Vbsh_CV_dV_b_GND;
d_T10_dV_p_GND = pParam_B4SOIketa*d_Vbsh_CV_dV_p_GND;
d_T10_dTemp_t_GND = pParam_B4SOIketa*d_Vbsh_CV_dTemp_t_GND;
d_T10_dV_e_si = pParam_B4SOIketa*d_Vbsh_CV_dV_e_si;
d_T10_dV_gi_si = pParam_B4SOIketa*d_Vbsh_CV_dV_gi_si;
d_T10_dV_di_si = pParam_B4SOIketa*d_Vbsh_CV_dV_di_si;
d_T10_dV_b_si = pParam_B4SOIketa*d_Vbsh_CV_dV_b_si;
T10 = (pParam_B4SOIketa*Vbsh_CV);
if ((T10>=(-0.5)))
{

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si =  0.0;
d_T11_dV_b_GND = (-d_T10_dV_b_GND/(1.0+T10)/(1.0+T10));
d_T11_dV_p_GND = (-d_T10_dV_p_GND/(1.0+T10)/(1.0+T10));
d_T11_dTemp_t_GND = (-d_T10_dTemp_t_GND/(1.0+T10)/(1.0+T10));
d_T11_dV_gi_si = (-d_T10_dV_gi_si/(1.0+T10)/(1.0+T10));
d_T11_dV_di_si = (-d_T10_dV_di_si/(1.0+T10)/(1.0+T10));
d_T11_dV_e_si = (-d_T10_dV_e_si/(1.0+T10)/(1.0+T10));
d_T11_dV_b_si = (-d_T10_dV_b_si/(1.0+T10)/(1.0+T10));
T11 = (1.0/(1.0+T10));
}
else
{

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si = d_T12_dV_b_si = d_T12_dV_e_si = d_T12_dV_di_si = d_T12_dV_gi_si = d_T12_dTemp_t_GND = d_T12_dV_p_GND = d_T12_dV_b_GND =  0.0;
T12 = ((-1.0)/((1.0-0.5)*(1.0-0.5)));

d_T13_dV_b_GND = d_T12_dV_b_GND*0.5;
d_T13_dV_p_GND = d_T12_dV_p_GND*0.5;
d_T13_dTemp_t_GND = d_T12_dTemp_t_GND*0.5;
d_T13_dV_gi_si = d_T12_dV_gi_si*0.5;
d_T13_dV_di_si = d_T12_dV_di_si*0.5;
d_T13_dV_e_si = d_T12_dV_e_si*0.5;
d_T13_dV_b_si = d_T12_dV_b_si*0.5;
T13 = ((1.0/(1.0-0.5))+(T12*0.5));

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si =  0.0;
d_T11_dV_b_GND = ((T12*d_T10_dV_b_GND+d_T12_dV_b_GND*T10)+d_T13_dV_b_GND);
d_T11_dV_p_GND = ((T12*d_T10_dV_p_GND+d_T12_dV_p_GND*T10)+d_T13_dV_p_GND);
d_T11_dTemp_t_GND = ((T12*d_T10_dTemp_t_GND+d_T12_dTemp_t_GND*T10)+d_T13_dTemp_t_GND);
d_T11_dV_gi_si = ((T12*d_T10_dV_gi_si+d_T12_dV_gi_si*T10)+d_T13_dV_gi_si);
d_T11_dV_di_si = ((T12*d_T10_dV_di_si+d_T12_dV_di_si*T10)+d_T13_dV_di_si);
d_T11_dV_e_si = ((T12*d_T10_dV_e_si+d_T12_dV_e_si*T10)+d_T13_dV_e_si);
d_T11_dV_b_si = ((T12*d_T10_dV_b_si+d_T12_dV_b_si*T10)+d_T13_dV_b_si);
T11 = ((T12*T10)+T13);
}

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si =  0.0;
d_T10_dV_b_GND = d_phi_dV_b_GND;
d_T10_dV_p_GND = d_phi_dV_p_GND;
d_T10_dTemp_t_GND = d_phi_dTemp_t_GND;
T10 = (phi+pParam_B4SOIketas);

d_T13_dV_b_GND = ((T10*(Vbsh_CV*d_T11_dV_b_GND+d_Vbsh_CV_dV_b_GND*T11)-(Vbsh_CV*T11)*d_T10_dV_b_GND)/T10/T10);
d_T13_dV_p_GND = ((T10*(Vbsh_CV*d_T11_dV_p_GND+d_Vbsh_CV_dV_p_GND*T11)-(Vbsh_CV*T11)*d_T10_dV_p_GND)/T10/T10);
d_T13_dTemp_t_GND = ((T10*(Vbsh_CV*d_T11_dTemp_t_GND+d_Vbsh_CV_dTemp_t_GND*T11)-(Vbsh_CV*T11)*d_T10_dTemp_t_GND)/T10/T10);
d_T13_dV_e_si = ((T10*(Vbsh_CV*d_T11_dV_e_si+d_Vbsh_CV_dV_e_si*T11)-(Vbsh_CV*T11)*d_T10_dV_e_si)/T10/T10);
d_T13_dV_gi_si = ((T10*(Vbsh_CV*d_T11_dV_gi_si+d_Vbsh_CV_dV_gi_si*T11)-(Vbsh_CV*T11)*d_T10_dV_gi_si)/T10/T10);
d_T13_dV_di_si = ((T10*(Vbsh_CV*d_T11_dV_di_si+d_Vbsh_CV_dV_di_si*T11)-(Vbsh_CV*T11)*d_T10_dV_di_si)/T10/T10);
d_T13_dV_b_si = ((T10*(Vbsh_CV*d_T11_dV_b_si+d_Vbsh_CV_dV_b_si*T11)-(Vbsh_CV*T11)*d_T10_dV_b_si)/T10/T10);
T13 = ((Vbsh_CV*T11)/T10);
if ((T13<0.50))
{
{
double value_sqrt_0 = sqrt((1.0-T13));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T14_dV_b_GND = (-(deriv_sqrt_0_d0*((-d_T13_dV_b_GND)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_p_GND = (-(deriv_sqrt_0_d0*((-d_T13_dV_p_GND)))/value_sqrt_0/value_sqrt_0);
d_T14_dTemp_t_GND = (-(deriv_sqrt_0_d0*((-d_T13_dTemp_t_GND)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_e_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_e_si)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_gi_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_gi_si)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_di_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_di_si)))/value_sqrt_0/value_sqrt_0);
d_T14_dV_b_si = (-(deriv_sqrt_0_d0*((-d_T13_dV_b_si)))/value_sqrt_0/value_sqrt_0);
T14 = (1.0/value_sqrt_0);
}
}
else
{
{
double value_sqrt_0 = sqrt((1-0.50));

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = (1.0/((2*(1-0.50))*value_sqrt_0));
}
{
double value_sqrt_0 = sqrt((1.0-0.50));

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si =  0.0;
d_T12_dV_b_GND = (-d_T11_dV_b_GND*0.50);
d_T12_dV_p_GND = (-d_T11_dV_p_GND*0.50);
d_T12_dTemp_t_GND = (-d_T11_dTemp_t_GND*0.50);
d_T12_dV_gi_si = (-d_T11_dV_gi_si*0.50);
d_T12_dV_di_si = (-d_T11_dV_di_si*0.50);
d_T12_dV_e_si = (-d_T11_dV_e_si*0.50);
d_T12_dV_b_si = (-d_T11_dV_b_si*0.50);
T12 = ((1/value_sqrt_0)-(T11*0.50));
}

d_T14_dV_b_GND = ((T11*d_T13_dV_b_GND+d_T11_dV_b_GND*T13)+d_T12_dV_b_GND);
d_T14_dV_p_GND = ((T11*d_T13_dV_p_GND+d_T11_dV_p_GND*T13)+d_T12_dV_p_GND);
d_T14_dTemp_t_GND = ((T11*d_T13_dTemp_t_GND+d_T11_dTemp_t_GND*T13)+d_T12_dTemp_t_GND);
d_T14_dV_gi_si = ((T11*d_T13_dV_gi_si+d_T11_dV_gi_si*T13)+d_T12_dV_gi_si);
d_T14_dV_di_si = ((T11*d_T13_dV_di_si+d_T11_dV_di_si*T13)+d_T12_dV_di_si);
d_T14_dV_e_si = ((T11*d_T13_dV_e_si+d_T11_dV_e_si*T13)+d_T12_dV_e_si);
d_T14_dV_b_si = ((T11*d_T13_dV_b_si+d_T11_dV_b_si*T13)+d_T12_dV_b_si);
T14 = ((T11*T13)+T12);
}
{
double value_sqrt_0 = sqrt((phi+pParam_B4SOIketas));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si =  0.0;
d_T10_dV_b_GND = (-((0.5*pParam_B4SOIk1ox)*Lpe_Vb)*(deriv_sqrt_0_d0*(d_phi_dV_b_GND))/value_sqrt_0/value_sqrt_0);
d_T10_dV_p_GND = (-((0.5*pParam_B4SOIk1ox)*Lpe_Vb)*(deriv_sqrt_0_d0*(d_phi_dV_p_GND))/value_sqrt_0/value_sqrt_0);
d_T10_dTemp_t_GND = (-((0.5*pParam_B4SOIk1ox)*Lpe_Vb)*(deriv_sqrt_0_d0*(d_phi_dTemp_t_GND))/value_sqrt_0/value_sqrt_0);
T10 = (((0.5*pParam_B4SOIk1ox)*Lpe_Vb)/value_sqrt_0);
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (T10*d_T14_dV_b_GND+d_T10_dV_b_GND*T14);
d_T1_dV_p_GND = (T10*d_T14_dV_p_GND+d_T10_dV_p_GND*T14);
d_T1_dTemp_t_GND = (T10*d_T14_dTemp_t_GND+d_T10_dTemp_t_GND*T14);
d_T1_dV_gi_si = (T10*d_T14_dV_gi_si+d_T10_dV_gi_si*T14);
d_T1_dV_di_si = (T10*d_T14_dV_di_si+d_T10_dV_di_si*T14);
d_T1_dV_e_si = (T10*d_T14_dV_e_si+d_T10_dV_e_si*T14);
d_T1_dV_b_si = (T10*d_T14_dV_b_si+d_T10_dV_b_si*T14);
T1 = (T10*T14);
{
double value_sqrt_0 = sqrt((pParam_B4SOIxj*Xdep_CV));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T9_dV_e_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_CV_dV_e_si));
d_T9_dV_gi_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_CV_dV_gi_si));
d_T9_dV_di_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_CV_dV_di_si));
d_T9_dV_b_si = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_CV_dV_b_si));
d_T9_dV_b_GND = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_CV_dV_b_GND));
d_T9_dV_p_GND = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_CV_dV_p_GND));
d_T9_dTemp_t_GND = (deriv_sqrt_0_d0*(pParam_B4SOIxj*d_Xdep_CV_dTemp_t_GND));
T9 = value_sqrt_0;
}

d_tmp1_dV_b_GND = 2.0*d_T9_dV_b_GND;
d_tmp1_dV_p_GND = 2.0*d_T9_dV_p_GND;
d_tmp1_dTemp_t_GND = 2.0*d_T9_dTemp_t_GND;
d_tmp1_dV_gi_si = 2.0*d_T9_dV_gi_si;
d_tmp1_dV_di_si = 2.0*d_T9_dV_di_si;
d_tmp1_dV_e_si = 2.0*d_T9_dV_e_si;
d_tmp1_dV_b_si = 2.0*d_T9_dV_b_si;
tmp1 = (Leff+(2.0*T9));

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = (-Leff*d_tmp1_dV_b_GND/tmp1/tmp1);
d_T5_dV_p_GND = (-Leff*d_tmp1_dV_p_GND/tmp1/tmp1);
d_T5_dTemp_t_GND = (-Leff*d_tmp1_dTemp_t_GND/tmp1/tmp1);
d_T5_dV_gi_si = (-Leff*d_tmp1_dV_gi_si/tmp1/tmp1);
d_T5_dV_di_si = (-Leff*d_tmp1_dV_di_si/tmp1/tmp1);
d_T5_dV_e_si = (-Leff*d_tmp1_dV_e_si/tmp1/tmp1);
d_T5_dV_b_si = (-Leff*d_tmp1_dV_b_si/tmp1/tmp1);
T5 = (Leff/tmp1);

d_tmp2_dV_b_GND = pParam_B4SOIa0*d_T5_dV_b_GND;
d_tmp2_dV_p_GND = pParam_B4SOIa0*d_T5_dV_p_GND;
d_tmp2_dTemp_t_GND = pParam_B4SOIa0*d_T5_dTemp_t_GND;
d_tmp2_dV_gi_si = pParam_B4SOIa0*d_T5_dV_gi_si;
d_tmp2_dV_di_si = pParam_B4SOIa0*d_T5_dV_di_si;
d_tmp2_dV_e_si = pParam_B4SOIa0*d_T5_dV_e_si;
d_tmp2_dV_b_si = pParam_B4SOIa0*d_T5_dV_b_si;
tmp2 = (pParam_B4SOIa0*T5);

d_tmp3_dV_b_si = d_tmp3_dV_e_si = d_tmp3_dV_di_si = d_tmp3_dV_gi_si = d_tmp3_dTemp_t_GND = d_tmp3_dV_p_GND = d_tmp3_dV_b_GND =  0.0;
tmp3 = (pParam_B4SOIweff+pParam_B4SOIb1);

d_tmp4_dV_b_si = d_tmp4_dV_e_si = d_tmp4_dV_di_si = d_tmp4_dV_gi_si = d_tmp4_dTemp_t_GND = d_tmp4_dV_p_GND = d_tmp4_dV_b_GND =  0.0;
tmp4 = (pParam_B4SOIb0/tmp3);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (d_tmp2_dV_b_GND+d_tmp4_dV_b_GND);
d_T2_dV_p_GND = (d_tmp2_dV_p_GND+d_tmp4_dV_p_GND);
d_T2_dTemp_t_GND = (d_tmp2_dTemp_t_GND+d_tmp4_dTemp_t_GND);
d_T2_dV_gi_si = (d_tmp2_dV_gi_si+d_tmp4_dV_gi_si);
d_T2_dV_di_si = (d_tmp2_dV_di_si+d_tmp4_dV_di_si);
d_T2_dV_e_si = (d_tmp2_dV_e_si+d_tmp4_dV_e_si);
d_T2_dV_b_si = (d_tmp2_dV_b_si+d_tmp4_dV_b_si);
T2 = (tmp2+tmp4);

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_b_GND = (T5*d_T5_dV_b_GND+d_T5_dV_b_GND*T5);
d_T6_dV_p_GND = (T5*d_T5_dV_p_GND+d_T5_dV_p_GND*T5);
d_T6_dTemp_t_GND = (T5*d_T5_dTemp_t_GND+d_T5_dTemp_t_GND*T5);
d_T6_dV_gi_si = (T5*d_T5_dV_gi_si+d_T5_dV_gi_si*T5);
d_T6_dV_di_si = (T5*d_T5_dV_di_si+d_T5_dV_di_si*T5);
d_T6_dV_e_si = (T5*d_T5_dV_e_si+d_T5_dV_e_si*T5);
d_T6_dV_b_si = (T5*d_T5_dV_b_si+d_T5_dV_b_si*T5);
T6 = (T5*T5);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = (T5*d_T6_dV_b_GND+d_T5_dV_b_GND*T6);
d_T7_dV_p_GND = (T5*d_T6_dV_p_GND+d_T5_dV_p_GND*T6);
d_T7_dTemp_t_GND = (T5*d_T6_dTemp_t_GND+d_T5_dTemp_t_GND*T6);
d_T7_dV_gi_si = (T5*d_T6_dV_gi_si+d_T5_dV_gi_si*T6);
d_T7_dV_di_si = (T5*d_T6_dV_di_si+d_T5_dV_di_si*T6);
d_T7_dV_e_si = (T5*d_T6_dV_e_si+d_T5_dV_e_si*T6);
d_T7_dV_b_si = (T5*d_T6_dV_b_si+d_T5_dV_b_si*T6);
T7 = (T5*T6);

d_Abulk0_CV_dV_b_GND = (T1*d_T2_dV_b_GND+d_T1_dV_b_GND*T2);
d_Abulk0_CV_dV_p_GND = (T1*d_T2_dV_p_GND+d_T1_dV_p_GND*T2);
d_Abulk0_CV_dTemp_t_GND = (T1*d_T2_dTemp_t_GND+d_T1_dTemp_t_GND*T2);
d_Abulk0_CV_dV_gi_si = (T1*d_T2_dV_gi_si+d_T1_dV_gi_si*T2);
d_Abulk0_CV_dV_di_si = (T1*d_T2_dV_di_si+d_T1_dV_di_si*T2);
d_Abulk0_CV_dV_e_si = (T1*d_T2_dV_e_si+d_T1_dV_e_si*T2);
d_Abulk0_CV_dV_b_si = (T1*d_T2_dV_b_si+d_T1_dV_b_si*T2);
Abulk0_CV = (1+(T1*T2));
}
if ((Abulk0_CV<0.01))
{

d_T9_dV_b_GND = (-(-200.0*d_Abulk0_CV_dV_b_GND)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
d_T9_dV_p_GND = (-(-200.0*d_Abulk0_CV_dV_p_GND)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
d_T9_dTemp_t_GND = (-(-200.0*d_Abulk0_CV_dTemp_t_GND)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
d_T9_dV_gi_si = (-(-200.0*d_Abulk0_CV_dV_gi_si)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
d_T9_dV_di_si = (-(-200.0*d_Abulk0_CV_dV_di_si)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
d_T9_dV_e_si = (-(-200.0*d_Abulk0_CV_dV_e_si)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
d_T9_dV_b_si = (-(-200.0*d_Abulk0_CV_dV_b_si)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
T9 = (1.0/(3.0-(200.0*Abulk0_CV)));

d_Abulk0_CV_dV_b_GND = ((0.02-Abulk0_CV)*d_T9_dV_b_GND+(-d_Abulk0_CV_dV_b_GND)*T9);
d_Abulk0_CV_dV_p_GND = ((0.02-Abulk0_CV)*d_T9_dV_p_GND+(-d_Abulk0_CV_dV_p_GND)*T9);
d_Abulk0_CV_dTemp_t_GND = ((0.02-Abulk0_CV)*d_T9_dTemp_t_GND+(-d_Abulk0_CV_dTemp_t_GND)*T9);
d_Abulk0_CV_dV_gi_si = ((0.02-Abulk0_CV)*d_T9_dV_gi_si+(-d_Abulk0_CV_dV_gi_si)*T9);
d_Abulk0_CV_dV_di_si = ((0.02-Abulk0_CV)*d_T9_dV_di_si+(-d_Abulk0_CV_dV_di_si)*T9);
d_Abulk0_CV_dV_e_si = ((0.02-Abulk0_CV)*d_T9_dV_e_si+(-d_Abulk0_CV_dV_e_si)*T9);
d_Abulk0_CV_dV_b_si = ((0.02-Abulk0_CV)*d_T9_dV_b_si+(-d_Abulk0_CV_dV_b_si)*T9);
Abulk0_CV = ((0.02-Abulk0_CV)*T9);
}
if (B4SOImtrlMod)
{

d_T14_dV_b_si = d_T14_dV_di_si = d_T14_dV_gi_si = d_T14_dV_e_si =  0.0;
d_T14_dV_b_GND = (2.0*B4SOItype)*(-0.5*d_Eg_dV_b_GND);
d_T14_dV_p_GND = (2.0*B4SOItype)*(-0.5*d_Eg_dV_p_GND);
d_T14_dTemp_t_GND = (2.0*B4SOItype)*(-0.5*d_Eg_dTemp_t_GND);
T14 = ((2.0*B4SOItype)*(((B4SOIphig-B4SOIeasub)-(0.5*Eg))+0.45));
toxe_mob = ((B4SOIeot*B4SOIepsrsub)/3.9);
}
else
{

d_T14_dV_b_si = d_T14_dV_di_si = d_T14_dV_gi_si = d_T14_dV_e_si = d_T14_dTemp_t_GND = d_T14_dV_p_GND = d_T14_dV_b_GND =  0.0;
T14 = 0.0;
toxe_mob = B4SOItox;
}
if ((B4SOImobMod==1))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (((d_Vgsteff_dV_e_si+d_Vth_dV_e_si)+d_Vth_dV_e_si)-d_T14_dV_e_si);
d_T0_dV_b_si = (((d_Vgsteff_dV_b_si+d_Vth_dV_b_si)+d_Vth_dV_b_si)-d_T14_dV_b_si);
d_T0_dV_b_GND = (((d_Vgsteff_dV_b_GND+d_Vth_dV_b_GND)+d_Vth_dV_b_GND)-d_T14_dV_b_GND);
d_T0_dV_p_GND = (((d_Vgsteff_dV_p_GND+d_Vth_dV_p_GND)+d_Vth_dV_p_GND)-d_T14_dV_p_GND);
d_T0_dTemp_t_GND = (((d_Vgsteff_dTemp_t_GND+d_Vth_dTemp_t_GND)+d_Vth_dTemp_t_GND)-d_T14_dTemp_t_GND);
d_T0_dV_gi_si = (((d_Vgsteff_dV_gi_si+d_Vth_dV_gi_si)+d_Vth_dV_gi_si)-d_T14_dV_gi_si);
d_T0_dV_di_si = (((d_Vgsteff_dV_di_si+d_Vth_dV_di_si)+d_Vth_dV_di_si)-d_T14_dV_di_si);
T0 = (((Vgsteff+Vth)+Vth)-T14);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_e_si = uc*d_Vbseff_dV_e_si;
d_T2_dV_gi_si = uc*d_Vbseff_dV_gi_si;
d_T2_dV_di_si = uc*d_Vbseff_dV_di_si;
d_T2_dV_b_si = uc*d_Vbseff_dV_b_si;
d_T2_dV_b_GND = (d_ua_dV_b_GND+(uc*d_Vbseff_dV_b_GND+d_uc_dV_b_GND*Vbseff));
d_T2_dV_p_GND = (d_ua_dV_p_GND+(uc*d_Vbseff_dV_p_GND+d_uc_dV_p_GND*Vbseff));
d_T2_dTemp_t_GND = (d_ua_dTemp_t_GND+(uc*d_Vbseff_dTemp_t_GND+d_uc_dTemp_t_GND*Vbseff));
T2 = (ua+(uc*Vbseff));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = (d_T0_dV_b_GND/toxe_mob);
d_T3_dV_p_GND = (d_T0_dV_p_GND/toxe_mob);
d_T3_dTemp_t_GND = (d_T0_dTemp_t_GND/toxe_mob);
d_T3_dV_e_si = (d_T0_dV_e_si/toxe_mob);
d_T3_dV_gi_si = (d_T0_dV_gi_si/toxe_mob);
d_T3_dV_di_si = (d_T0_dV_di_si/toxe_mob);
d_T3_dV_b_si = (d_T0_dV_b_si/toxe_mob);
T3 = (T0/toxe_mob);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = (T3*(d_T2_dV_b_GND+(ub*d_T3_dV_b_GND+d_ub_dV_b_GND*T3))+d_T3_dV_b_GND*(T2+(ub*T3)));
d_T5_dV_p_GND = (T3*(d_T2_dV_p_GND+(ub*d_T3_dV_p_GND+d_ub_dV_p_GND*T3))+d_T3_dV_p_GND*(T2+(ub*T3)));
d_T5_dTemp_t_GND = (T3*(d_T2_dTemp_t_GND+(ub*d_T3_dTemp_t_GND+d_ub_dTemp_t_GND*T3))+d_T3_dTemp_t_GND*(T2+(ub*T3)));
d_T5_dV_gi_si = (T3*(d_T2_dV_gi_si+ub*d_T3_dV_gi_si)+d_T3_dV_gi_si*(T2+(ub*T3)));
d_T5_dV_di_si = (T3*(d_T2_dV_di_si+ub*d_T3_dV_di_si)+d_T3_dV_di_si*(T2+(ub*T3)));
d_T5_dV_e_si = (T3*(d_T2_dV_e_si+ub*d_T3_dV_e_si)+d_T3_dV_e_si*(T2+(ub*T3)));
d_T5_dV_b_si = (T3*(d_T2_dV_b_si+ub*d_T3_dV_b_si)+d_T3_dV_b_si*(T2+(ub*T3)));
T5 = (T3*(T2+(ub*T3)));
}
else
{
if ((B4SOImobMod==2))
{

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_e_si = (((Vgsteff-T14)/toxe)*(uc*d_Vbseff_dV_e_si+(ub*(d_Vgsteff_dV_e_si-d_T14_dV_e_si)/toxe))+((d_Vgsteff_dV_e_si-d_T14_dV_e_si)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
d_T5_dV_b_si = (((Vgsteff-T14)/toxe)*(uc*d_Vbseff_dV_b_si+(ub*(d_Vgsteff_dV_b_si-d_T14_dV_b_si)/toxe))+((d_Vgsteff_dV_b_si-d_T14_dV_b_si)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
d_T5_dV_b_GND = (((Vgsteff-T14)/toxe)*((d_ua_dV_b_GND+(uc*d_Vbseff_dV_b_GND+d_uc_dV_b_GND*Vbseff))+((ub*(d_Vgsteff_dV_b_GND-d_T14_dV_b_GND)+d_ub_dV_b_GND*(Vgsteff-T14))/toxe))+((d_Vgsteff_dV_b_GND-d_T14_dV_b_GND)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
d_T5_dV_p_GND = (((Vgsteff-T14)/toxe)*((d_ua_dV_p_GND+(uc*d_Vbseff_dV_p_GND+d_uc_dV_p_GND*Vbseff))+((ub*(d_Vgsteff_dV_p_GND-d_T14_dV_p_GND)+d_ub_dV_p_GND*(Vgsteff-T14))/toxe))+((d_Vgsteff_dV_p_GND-d_T14_dV_p_GND)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
d_T5_dTemp_t_GND = (((Vgsteff-T14)/toxe)*((d_ua_dTemp_t_GND+(uc*d_Vbseff_dTemp_t_GND+d_uc_dTemp_t_GND*Vbseff))+((ub*(d_Vgsteff_dTemp_t_GND-d_T14_dTemp_t_GND)+d_ub_dTemp_t_GND*(Vgsteff-T14))/toxe))+((d_Vgsteff_dTemp_t_GND-d_T14_dTemp_t_GND)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
d_T5_dV_gi_si = (((Vgsteff-T14)/toxe)*(uc*d_Vbseff_dV_gi_si+(ub*(d_Vgsteff_dV_gi_si-d_T14_dV_gi_si)/toxe))+((d_Vgsteff_dV_gi_si-d_T14_dV_gi_si)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
d_T5_dV_di_si = (((Vgsteff-T14)/toxe)*(uc*d_Vbseff_dV_di_si+(ub*(d_Vgsteff_dV_di_si-d_T14_dV_di_si)/toxe))+((d_Vgsteff_dV_di_si-d_T14_dV_di_si)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
T5 = (((Vgsteff-T14)/toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/toxe)));
}
else
{
if ((B4SOImobMod==3))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (((d_Vgsteff_dV_e_si+d_Vth_dV_e_si)+d_Vth_dV_e_si)-d_T14_dV_e_si);
d_T0_dV_b_si = (((d_Vgsteff_dV_b_si+d_Vth_dV_b_si)+d_Vth_dV_b_si)-d_T14_dV_b_si);
d_T0_dV_b_GND = (((d_Vgsteff_dV_b_GND+d_Vth_dV_b_GND)+d_Vth_dV_b_GND)-d_T14_dV_b_GND);
d_T0_dV_p_GND = (((d_Vgsteff_dV_p_GND+d_Vth_dV_p_GND)+d_Vth_dV_p_GND)-d_T14_dV_p_GND);
d_T0_dTemp_t_GND = (((d_Vgsteff_dTemp_t_GND+d_Vth_dTemp_t_GND)+d_Vth_dTemp_t_GND)-d_T14_dTemp_t_GND);
d_T0_dV_gi_si = (((d_Vgsteff_dV_gi_si+d_Vth_dV_gi_si)+d_Vth_dV_gi_si)-d_T14_dV_gi_si);
d_T0_dV_di_si = (((d_Vgsteff_dV_di_si+d_Vth_dV_di_si)+d_Vth_dV_di_si)-d_T14_dV_di_si);
T0 = (((Vgsteff+Vth)+Vth)-T14);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_e_si = uc*d_Vbseff_dV_e_si;
d_T2_dV_gi_si = uc*d_Vbseff_dV_gi_si;
d_T2_dV_di_si = uc*d_Vbseff_dV_di_si;
d_T2_dV_b_si = uc*d_Vbseff_dV_b_si;
d_T2_dV_b_GND = (uc*d_Vbseff_dV_b_GND+d_uc_dV_b_GND*Vbseff);
d_T2_dV_p_GND = (uc*d_Vbseff_dV_p_GND+d_uc_dV_p_GND*Vbseff);
d_T2_dTemp_t_GND = (uc*d_Vbseff_dTemp_t_GND+d_uc_dTemp_t_GND*Vbseff);
T2 = (1.0+(uc*Vbseff));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = (d_T0_dV_b_GND/toxe_mob);
d_T3_dV_p_GND = (d_T0_dV_p_GND/toxe_mob);
d_T3_dTemp_t_GND = (d_T0_dTemp_t_GND/toxe_mob);
d_T3_dV_e_si = (d_T0_dV_e_si/toxe_mob);
d_T3_dV_gi_si = (d_T0_dV_gi_si/toxe_mob);
d_T3_dV_di_si = (d_T0_dV_di_si/toxe_mob);
d_T3_dV_b_si = (d_T0_dV_b_si/toxe_mob);
T3 = (T0/toxe_mob);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (T3*(d_ua_dV_b_GND+(ub*d_T3_dV_b_GND+d_ub_dV_b_GND*T3))+d_T3_dV_b_GND*(ua+(ub*T3)));
d_T4_dV_p_GND = (T3*(d_ua_dV_p_GND+(ub*d_T3_dV_p_GND+d_ub_dV_p_GND*T3))+d_T3_dV_p_GND*(ua+(ub*T3)));
d_T4_dTemp_t_GND = (T3*(d_ua_dTemp_t_GND+(ub*d_T3_dTemp_t_GND+d_ub_dTemp_t_GND*T3))+d_T3_dTemp_t_GND*(ua+(ub*T3)));
d_T4_dV_gi_si = (T3*ub*d_T3_dV_gi_si+d_T3_dV_gi_si*(ua+(ub*T3)));
d_T4_dV_di_si = (T3*ub*d_T3_dV_di_si+d_T3_dV_di_si*(ua+(ub*T3)));
d_T4_dV_e_si = (T3*ub*d_T3_dV_e_si+d_T3_dV_e_si*(ua+(ub*T3)));
d_T4_dV_b_si = (T3*ub*d_T3_dV_b_si+d_T3_dV_b_si*(ua+(ub*T3)));
T4 = (T3*(ua+(ub*T3)));

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = (T4*d_T2_dV_b_GND+d_T4_dV_b_GND*T2);
d_T5_dV_p_GND = (T4*d_T2_dV_p_GND+d_T4_dV_p_GND*T2);
d_T5_dTemp_t_GND = (T4*d_T2_dTemp_t_GND+d_T4_dTemp_t_GND*T2);
d_T5_dV_gi_si = (T4*d_T2_dV_gi_si+d_T4_dV_gi_si*T2);
d_T5_dV_di_si = (T4*d_T2_dV_di_si+d_T4_dV_di_si*T2);
d_T5_dV_e_si = (T4*d_T2_dV_e_si+d_T4_dV_e_si*T2);
d_T5_dV_b_si = (T4*d_T2_dV_b_si+d_T4_dV_b_si*T2);
T5 = (T4*T2);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = ((d_Vgsteff_dV_e_si*1.0e-8/toxe)/6.0);
d_T0_dV_b_si = ((d_Vgsteff_dV_b_si*1.0e-8/toxe)/6.0);
d_T0_dV_b_GND = ((d_Vgsteff_dV_b_GND*1.0e-8/toxe)/6.0);
d_T0_dV_p_GND = ((d_Vgsteff_dV_p_GND*1.0e-8/toxe)/6.0);
d_T0_dTemp_t_GND = ((d_Vgsteff_dTemp_t_GND*1.0e-8/toxe)/6.0);
d_T0_dV_gi_si = ((d_Vgsteff_dV_gi_si*1.0e-8/toxe)/6.0);
d_T0_dV_di_si = ((d_Vgsteff_dV_di_si*1.0e-8/toxe)/6.0);
T0 = ((((Vgsteff+B4SOIvtfbphi1)*1.0e-8)/toxe)/6.0);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp((pParam_B4SOIeu*evaluator_lln_0.getValues(T0)));
double  deriv_exp_1_d0 = value_exp_1;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_1_d0*(pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_GND)));
d_T1_dV_p_GND = (deriv_exp_1_d0*(pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dV_p_GND)));
d_T1_dTemp_t_GND = (deriv_exp_1_d0*(pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dTemp_t_GND)));
d_T1_dV_e_si = (deriv_exp_1_d0*(pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dV_e_si)));
d_T1_dV_gi_si = (deriv_exp_1_d0*(pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dV_gi_si)));
d_T1_dV_di_si = (deriv_exp_1_d0*(pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dV_di_si)));
d_T1_dV_b_si = (deriv_exp_1_d0*(pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_si)));
T1 = value_exp_1;
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_e_si = uc*d_Vbseff_dV_e_si;
d_T2_dV_gi_si = uc*d_Vbseff_dV_gi_si;
d_T2_dV_di_si = uc*d_Vbseff_dV_di_si;
d_T2_dV_b_si = uc*d_Vbseff_dV_b_si;
d_T2_dV_b_GND = (d_ua_dV_b_GND+(uc*d_Vbseff_dV_b_GND+d_uc_dV_b_GND*Vbseff));
d_T2_dV_p_GND = (d_ua_dV_p_GND+(uc*d_Vbseff_dV_p_GND+d_uc_dV_p_GND*Vbseff));
d_T2_dTemp_t_GND = (d_ua_dTemp_t_GND+(uc*d_Vbseff_dTemp_t_GND+d_uc_dTemp_t_GND*Vbseff));
T2 = (ua+(uc*Vbseff));
{
double value_pow_0 = pow(TempRatio,pParam_B4SOIucste);
double  deriv_pow_0_d0 = ((TempRatio == 0.0)?0.0:(value_pow_0*pParam_B4SOIucste/TempRatio));

d_ucs_dV_b_GND = pParam_B4SOIucs*(deriv_pow_0_d0*(d_TempRatio_dV_b_GND));
d_ucs_dV_p_GND = pParam_B4SOIucs*(deriv_pow_0_d0*(d_TempRatio_dV_p_GND));
d_ucs_dTemp_t_GND = pParam_B4SOIucs*(deriv_pow_0_d0*(d_TempRatio_dTemp_t_GND));
ucs = (pParam_B4SOIucs*value_pow_0);
}
{
double value_pow_0 = pow(TempRatio,pParam_B4SOIud1);
double  deriv_pow_0_d0 = ((TempRatio == 0.0)?0.0:(value_pow_0*pParam_B4SOIud1/TempRatio));

d_ud_dV_b_GND = pParam_B4SOIud*(deriv_pow_0_d0*(d_TempRatio_dV_b_GND));
d_ud_dV_p_GND = pParam_B4SOIud*(deriv_pow_0_d0*(d_TempRatio_dV_p_GND));
d_ud_dTemp_t_GND = pParam_B4SOIud*(deriv_pow_0_d0*(d_TempRatio_dTemp_t_GND));
ud = (pParam_B4SOIud*value_pow_0);
}
VgsteffVth = B4SOIvgsteffvth;
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+(Vgsteff/VgsteffVth)));
double value_exp_1 = exp((ucs*evaluator_lln_0.getValues((1.0+(Vgsteff/VgsteffVth)))));
double  deriv_exp_1_d0 = value_exp_1;

d_T10_dV_db_di = d_T10_dV_sb_si =  0.0;
d_T10_dV_e_si = (deriv_exp_1_d0*(ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),(d_Vgsteff_dV_e_si/VgsteffVth))));
d_T10_dV_b_si = (deriv_exp_1_d0*(ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),(d_Vgsteff_dV_b_si/VgsteffVth))));
d_T10_dV_gi_si = (deriv_exp_1_d0*(ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),(d_Vgsteff_dV_gi_si/VgsteffVth))));
d_T10_dV_di_si = (deriv_exp_1_d0*(ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),(d_Vgsteff_dV_di_si/VgsteffVth))));
d_T10_dV_b_GND = (deriv_exp_1_d0*((ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),(d_Vgsteff_dV_b_GND/VgsteffVth))+d_ucs_dV_b_GND*evaluator_lln_0.getValues((1.0+(Vgsteff/VgsteffVth))))));
d_T10_dV_p_GND = (deriv_exp_1_d0*((ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),(d_Vgsteff_dV_p_GND/VgsteffVth))+d_ucs_dV_p_GND*evaluator_lln_0.getValues((1.0+(Vgsteff/VgsteffVth))))));
d_T10_dTemp_t_GND = (deriv_exp_1_d0*((ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),(d_Vgsteff_dTemp_t_GND/VgsteffVth))+d_ucs_dTemp_t_GND*evaluator_lln_0.getValues((1.0+(Vgsteff/VgsteffVth))))));
T10 = value_exp_1;
}

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si =  0.0;
d_T11_dV_gi_si = (-ud*d_T10_dV_gi_si/T10/T10);
d_T11_dV_di_si = (-ud*d_T10_dV_di_si/T10/T10);
d_T11_dV_e_si = (-ud*d_T10_dV_e_si/T10/T10);
d_T11_dV_b_si = (-ud*d_T10_dV_b_si/T10/T10);
d_T11_dV_b_GND = ((T10*d_ud_dV_b_GND-ud*d_T10_dV_b_GND)/T10/T10);
d_T11_dV_p_GND = ((T10*d_ud_dV_p_GND-ud*d_T10_dV_p_GND)/T10/T10);
d_T11_dTemp_t_GND = ((T10*d_ud_dTemp_t_GND-ud*d_T10_dTemp_t_GND)/T10/T10);
T11 = (ud/T10);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = ((T1*d_T2_dV_b_GND+d_T1_dV_b_GND*T2)+d_T11_dV_b_GND);
d_T5_dV_p_GND = ((T1*d_T2_dV_p_GND+d_T1_dV_p_GND*T2)+d_T11_dV_p_GND);
d_T5_dTemp_t_GND = ((T1*d_T2_dTemp_t_GND+d_T1_dTemp_t_GND*T2)+d_T11_dTemp_t_GND);
d_T5_dV_gi_si = ((T1*d_T2_dV_gi_si+d_T1_dV_gi_si*T2)+d_T11_dV_gi_si);
d_T5_dV_di_si = ((T1*d_T2_dV_di_si+d_T1_dV_di_si*T2)+d_T11_dV_di_si);
d_T5_dV_e_si = ((T1*d_T2_dV_e_si+d_T1_dV_e_si*T2)+d_T11_dV_e_si);
d_T5_dV_b_si = ((T1*d_T2_dV_b_si+d_T1_dV_b_si*T2)+d_T11_dV_b_si);
T5 = ((T1*T2)+T11);
}
}
}
if ((T5>=(-0.8)))
{

d_Denomi_dV_b_GND = d_T5_dV_b_GND;
d_Denomi_dV_p_GND = d_T5_dV_p_GND;
d_Denomi_dTemp_t_GND = d_T5_dTemp_t_GND;
d_Denomi_dV_gi_si = d_T5_dV_gi_si;
d_Denomi_dV_di_si = d_T5_dV_di_si;
d_Denomi_dV_e_si = d_T5_dV_e_si;
d_Denomi_dV_b_si = d_T5_dV_b_si;
Denomi = (1.0+T5);
}
else
{

d_T9_dV_b_GND = (-10.0*d_T5_dV_b_GND/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
d_T9_dV_p_GND = (-10.0*d_T5_dV_p_GND/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
d_T9_dTemp_t_GND = (-10.0*d_T5_dTemp_t_GND/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
d_T9_dV_gi_si = (-10.0*d_T5_dV_gi_si/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
d_T9_dV_di_si = (-10.0*d_T5_dV_di_si/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
d_T9_dV_e_si = (-10.0*d_T5_dV_e_si/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
d_T9_dV_b_si = (-10.0*d_T5_dV_b_si/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
T9 = (1.0/(7.0+(10.0*T5)));

d_Denomi_dV_b_GND = ((0.6+T5)*d_T9_dV_b_GND+d_T5_dV_b_GND*T9);
d_Denomi_dV_p_GND = ((0.6+T5)*d_T9_dV_p_GND+d_T5_dV_p_GND*T9);
d_Denomi_dTemp_t_GND = ((0.6+T5)*d_T9_dTemp_t_GND+d_T5_dTemp_t_GND*T9);
d_Denomi_dV_gi_si = ((0.6+T5)*d_T9_dV_gi_si+d_T5_dV_gi_si*T9);
d_Denomi_dV_di_si = ((0.6+T5)*d_T9_dV_di_si+d_T5_dV_di_si*T9);
d_Denomi_dV_e_si = ((0.6+T5)*d_T9_dV_e_si+d_T5_dV_e_si*T9);
d_Denomi_dV_b_si = ((0.6+T5)*d_T9_dV_b_si+d_T5_dV_b_si*T9);
Denomi = ((0.6+T5)*T9);
}

d_ueff_dV_gi_si = (-u0temp*d_Denomi_dV_gi_si/Denomi/Denomi);
d_ueff_dV_di_si = (-u0temp*d_Denomi_dV_di_si/Denomi/Denomi);
d_ueff_dV_e_si = (-u0temp*d_Denomi_dV_e_si/Denomi/Denomi);
d_ueff_dV_b_si = (-u0temp*d_Denomi_dV_b_si/Denomi/Denomi);
d_ueff_dV_b_GND = ((Denomi*d_u0temp_dV_b_GND-u0temp*d_Denomi_dV_b_GND)/Denomi/Denomi);
d_ueff_dV_p_GND = ((Denomi*d_u0temp_dV_p_GND-u0temp*d_Denomi_dV_p_GND)/Denomi/Denomi);
d_ueff_dTemp_t_GND = ((Denomi*d_u0temp_dTemp_t_GND-u0temp*d_Denomi_dTemp_t_GND)/Denomi/Denomi);
ueff = (u0temp/Denomi);
B4SOIueff = ueff;

d_WVCox_dV_e_si = d_Weff_dV_e_si*vsattemp*B4SOIcox;
d_WVCox_dV_b_si = d_Weff_dV_b_si*vsattemp*B4SOIcox;
d_WVCox_dV_b_GND = (Weff*d_vsattemp_dV_b_GND+d_Weff_dV_b_GND*vsattemp)*B4SOIcox;
d_WVCox_dV_p_GND = (Weff*d_vsattemp_dV_p_GND+d_Weff_dV_p_GND*vsattemp)*B4SOIcox;
d_WVCox_dTemp_t_GND = (Weff*d_vsattemp_dTemp_t_GND+d_Weff_dTemp_t_GND*vsattemp)*B4SOIcox;
d_WVCox_dV_gi_si = d_Weff_dV_gi_si*vsattemp*B4SOIcox;
d_WVCox_dV_di_si = d_Weff_dV_di_si*vsattemp*B4SOIcox;
WVCox = ((Weff*vsattemp)*B4SOIcox);

d_WVCoxRds_dV_e_si = (WVCox*d_Rds_dV_e_si+d_WVCox_dV_e_si*Rds);
d_WVCoxRds_dV_b_si = (WVCox*d_Rds_dV_b_si+d_WVCox_dV_b_si*Rds);
d_WVCoxRds_dV_b_GND = (WVCox*d_Rds_dV_b_GND+d_WVCox_dV_b_GND*Rds);
d_WVCoxRds_dV_p_GND = (WVCox*d_Rds_dV_p_GND+d_WVCox_dV_p_GND*Rds);
d_WVCoxRds_dTemp_t_GND = (WVCox*d_Rds_dTemp_t_GND+d_WVCox_dTemp_t_GND*Rds);
d_WVCoxRds_dV_gi_si = (WVCox*d_Rds_dV_gi_si+d_WVCox_dV_gi_si*Rds);
d_WVCoxRds_dV_di_si = (WVCox*d_Rds_dV_di_si+d_WVCox_dV_di_si*Rds);
WVCoxRds = (WVCox*Rds);

d_Esat_dV_gi_si = (-(2.0*vsattemp)*d_ueff_dV_gi_si/ueff/ueff);
d_Esat_dV_di_si = (-(2.0*vsattemp)*d_ueff_dV_di_si/ueff/ueff);
d_Esat_dV_e_si = (-(2.0*vsattemp)*d_ueff_dV_e_si/ueff/ueff);
d_Esat_dV_b_si = (-(2.0*vsattemp)*d_ueff_dV_b_si/ueff/ueff);
d_Esat_dV_b_GND = ((ueff*2.0*d_vsattemp_dV_b_GND-(2.0*vsattemp)*d_ueff_dV_b_GND)/ueff/ueff);
d_Esat_dV_p_GND = ((ueff*2.0*d_vsattemp_dV_p_GND-(2.0*vsattemp)*d_ueff_dV_p_GND)/ueff/ueff);
d_Esat_dTemp_t_GND = ((ueff*2.0*d_vsattemp_dTemp_t_GND-(2.0*vsattemp)*d_ueff_dTemp_t_GND)/ueff/ueff);
Esat = ((2.0*vsattemp)/ueff);

d_EsatL_dV_gi_si = d_Esat_dV_gi_si*Leff;
d_EsatL_dV_di_si = d_Esat_dV_di_si*Leff;
d_EsatL_dV_e_si = d_Esat_dV_e_si*Leff;
d_EsatL_dV_b_si = d_Esat_dV_b_si*Leff;
d_EsatL_dV_b_GND = d_Esat_dV_b_GND*Leff;
d_EsatL_dV_p_GND = d_Esat_dV_p_GND*Leff;
d_EsatL_dTemp_t_GND = d_Esat_dTemp_t_GND*Leff;
EsatL = (Esat*Leff);
if ((pParam_B4SOIa1==0.0))
{

d_Lambda_dV_b_si = d_Lambda_dV_di_si = d_Lambda_dV_gi_si = d_Lambda_dV_e_si = d_Lambda_dTemp_t_GND = d_Lambda_dV_p_GND = d_Lambda_dV_b_GND =  0.0;
Lambda = pParam_B4SOIa2;
}
else
{
if ((pParam_B4SOIa1>0.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0-pParam_B4SOIa2);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND-pParam_B4SOIa1*d_Vgsteff_dV_b_GND);
d_T1_dV_p_GND = (d_T0_dV_p_GND-pParam_B4SOIa1*d_Vgsteff_dV_p_GND);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND-pParam_B4SOIa1*d_Vgsteff_dTemp_t_GND);
d_T1_dV_e_si = (d_T0_dV_e_si-pParam_B4SOIa1*d_Vgsteff_dV_e_si);
d_T1_dV_gi_si = (d_T0_dV_gi_si-pParam_B4SOIa1*d_Vgsteff_dV_gi_si);
d_T1_dV_di_si = (d_T0_dV_di_si-pParam_B4SOIa1*d_Vgsteff_dV_di_si);
d_T1_dV_b_si = (d_T0_dV_b_si-pParam_B4SOIa1*d_Vgsteff_dV_b_si);
T1 = ((T0-(pParam_B4SOIa1*Vgsteff))-0.0001);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.0004*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+0.0004*d_T0_dV_b_GND)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+0.0004*d_T0_dV_p_GND)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+0.0004*d_T0_dTemp_t_GND)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+0.0004*d_T0_dV_gi_si)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+0.0004*d_T0_dV_di_si)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+0.0004*d_T0_dV_e_si)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+0.0004*d_T0_dV_b_si)));
T2 = value_sqrt_0;
}

d_Lambda_dV_b_GND = (d_T0_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND));
d_Lambda_dV_p_GND = (d_T0_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND));
d_Lambda_dTemp_t_GND = (d_T0_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND));
d_Lambda_dV_e_si = (d_T0_dV_e_si-0.5*(d_T1_dV_e_si+d_T2_dV_e_si));
d_Lambda_dV_gi_si = (d_T0_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si));
d_Lambda_dV_di_si = (d_T0_dV_di_si-0.5*(d_T1_dV_di_si+d_T2_dV_di_si));
d_Lambda_dV_b_si = (d_T0_dV_b_si-0.5*(d_T1_dV_b_si+d_T2_dV_b_si));
Lambda = ((pParam_B4SOIa2+T0)-(0.5*(T1+T2)));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = pParam_B4SOIa1*d_Vgsteff_dV_e_si;
d_T1_dV_b_si = pParam_B4SOIa1*d_Vgsteff_dV_b_si;
d_T1_dV_b_GND = pParam_B4SOIa1*d_Vgsteff_dV_b_GND;
d_T1_dV_p_GND = pParam_B4SOIa1*d_Vgsteff_dV_p_GND;
d_T1_dTemp_t_GND = pParam_B4SOIa1*d_Vgsteff_dTemp_t_GND;
d_T1_dV_gi_si = pParam_B4SOIa1*d_Vgsteff_dV_gi_si;
d_T1_dV_di_si = pParam_B4SOIa1*d_Vgsteff_dV_di_si;
T1 = ((pParam_B4SOIa2+(pParam_B4SOIa1*Vgsteff))-0.0001);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.0004*pParam_B4SOIa2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)));
T2 = value_sqrt_0;
}

d_Lambda_dV_b_GND = 0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND);
d_Lambda_dV_p_GND = 0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND);
d_Lambda_dTemp_t_GND = 0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND);
d_Lambda_dV_gi_si = 0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si);
d_Lambda_dV_di_si = 0.5*(d_T1_dV_di_si+d_T2_dV_di_si);
d_Lambda_dV_e_si = 0.5*(d_T1_dV_e_si+d_T2_dV_e_si);
d_Lambda_dV_b_si = 0.5*(d_T1_dV_b_si+d_T2_dV_b_si);
Lambda = (0.5*(T1+T2));
}
}
B4SOIAbovVgst2Vtm = (Abulk/Vgst2Vtm);
if (((Rds==0.0)&&(Lambda==1.0)))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-((Abulk*d_EsatL_dV_b_GND+d_Abulk_dV_b_GND*EsatL)+d_Vgst2Vtm_dV_b_GND)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
d_T0_dV_p_GND = (-((Abulk*d_EsatL_dV_p_GND+d_Abulk_dV_p_GND*EsatL)+d_Vgst2Vtm_dV_p_GND)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
d_T0_dTemp_t_GND = (-((Abulk*d_EsatL_dTemp_t_GND+d_Abulk_dTemp_t_GND*EsatL)+d_Vgst2Vtm_dTemp_t_GND)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
d_T0_dV_gi_si = (-((Abulk*d_EsatL_dV_gi_si+d_Abulk_dV_gi_si*EsatL)+d_Vgst2Vtm_dV_gi_si)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
d_T0_dV_di_si = (-((Abulk*d_EsatL_dV_di_si+d_Abulk_dV_di_si*EsatL)+d_Vgst2Vtm_dV_di_si)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
d_T0_dV_e_si = (-((Abulk*d_EsatL_dV_e_si+d_Abulk_dV_e_si*EsatL)+d_Vgst2Vtm_dV_e_si)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
d_T0_dV_b_si = (-((Abulk*d_EsatL_dV_b_si+d_Abulk_dV_b_si*EsatL)+d_Vgst2Vtm_dV_b_si)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
T0 = (1.0/((Abulk*EsatL)+Vgst2Vtm));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_gi_si = (EsatL*d_Vgst2Vtm_dV_gi_si+d_EsatL_dV_gi_si*Vgst2Vtm);
d_T3_dV_di_si = (EsatL*d_Vgst2Vtm_dV_di_si+d_EsatL_dV_di_si*Vgst2Vtm);
d_T3_dV_e_si = (EsatL*d_Vgst2Vtm_dV_e_si+d_EsatL_dV_e_si*Vgst2Vtm);
d_T3_dV_b_si = (EsatL*d_Vgst2Vtm_dV_b_si+d_EsatL_dV_b_si*Vgst2Vtm);
d_T3_dV_b_GND = (EsatL*d_Vgst2Vtm_dV_b_GND+d_EsatL_dV_b_GND*Vgst2Vtm);
d_T3_dV_p_GND = (EsatL*d_Vgst2Vtm_dV_p_GND+d_EsatL_dV_p_GND*Vgst2Vtm);
d_T3_dTemp_t_GND = (EsatL*d_Vgst2Vtm_dTemp_t_GND+d_EsatL_dTemp_t_GND*Vgst2Vtm);
T3 = (EsatL*Vgst2Vtm);

d_Vdsat_dV_b_GND = (T3*d_T0_dV_b_GND+d_T3_dV_b_GND*T0);
d_Vdsat_dV_p_GND = (T3*d_T0_dV_p_GND+d_T3_dV_p_GND*T0);
d_Vdsat_dTemp_t_GND = (T3*d_T0_dTemp_t_GND+d_T3_dTemp_t_GND*T0);
d_Vdsat_dV_gi_si = (T3*d_T0_dV_gi_si+d_T3_dV_gi_si*T0);
d_Vdsat_dV_di_si = (T3*d_T0_dV_di_si+d_T3_dV_di_si*T0);
d_Vdsat_dV_e_si = (T3*d_T0_dV_e_si+d_T3_dV_e_si*T0);
d_Vdsat_dV_b_si = (T3*d_T0_dV_b_si+d_T3_dV_b_si*T0);
Vdsat = (T3*T0);
}
else
{

d_T9_dV_b_GND = (Abulk*d_WVCoxRds_dV_b_GND+d_Abulk_dV_b_GND*WVCoxRds);
d_T9_dV_p_GND = (Abulk*d_WVCoxRds_dV_p_GND+d_Abulk_dV_p_GND*WVCoxRds);
d_T9_dTemp_t_GND = (Abulk*d_WVCoxRds_dTemp_t_GND+d_Abulk_dTemp_t_GND*WVCoxRds);
d_T9_dV_gi_si = (Abulk*d_WVCoxRds_dV_gi_si+d_Abulk_dV_gi_si*WVCoxRds);
d_T9_dV_di_si = (Abulk*d_WVCoxRds_dV_di_si+d_Abulk_dV_di_si*WVCoxRds);
d_T9_dV_e_si = (Abulk*d_WVCoxRds_dV_e_si+d_Abulk_dV_e_si*WVCoxRds);
d_T9_dV_b_si = (Abulk*d_WVCoxRds_dV_b_si+d_Abulk_dV_b_si*WVCoxRds);
T9 = (Abulk*WVCoxRds);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_e_si = (Vgst2Vtm*d_T9_dV_e_si+d_Vgst2Vtm_dV_e_si*T9);
d_T7_dV_b_si = (Vgst2Vtm*d_T9_dV_b_si+d_Vgst2Vtm_dV_b_si*T9);
d_T7_dV_b_GND = (Vgst2Vtm*d_T9_dV_b_GND+d_Vgst2Vtm_dV_b_GND*T9);
d_T7_dV_p_GND = (Vgst2Vtm*d_T9_dV_p_GND+d_Vgst2Vtm_dV_p_GND*T9);
d_T7_dTemp_t_GND = (Vgst2Vtm*d_T9_dTemp_t_GND+d_Vgst2Vtm_dTemp_t_GND*T9);
d_T7_dV_gi_si = (Vgst2Vtm*d_T9_dV_gi_si+d_Vgst2Vtm_dV_gi_si*T9);
d_T7_dV_di_si = (Vgst2Vtm*d_T9_dV_di_si+d_Vgst2Vtm_dV_di_si*T9);
T7 = (Vgst2Vtm*T9);

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_e_si = (Vgst2Vtm*d_WVCoxRds_dV_e_si+d_Vgst2Vtm_dV_e_si*WVCoxRds);
d_T6_dV_b_si = (Vgst2Vtm*d_WVCoxRds_dV_b_si+d_Vgst2Vtm_dV_b_si*WVCoxRds);
d_T6_dV_b_GND = (Vgst2Vtm*d_WVCoxRds_dV_b_GND+d_Vgst2Vtm_dV_b_GND*WVCoxRds);
d_T6_dV_p_GND = (Vgst2Vtm*d_WVCoxRds_dV_p_GND+d_Vgst2Vtm_dV_p_GND*WVCoxRds);
d_T6_dTemp_t_GND = (Vgst2Vtm*d_WVCoxRds_dTemp_t_GND+d_Vgst2Vtm_dTemp_t_GND*WVCoxRds);
d_T6_dV_gi_si = (Vgst2Vtm*d_WVCoxRds_dV_gi_si+d_Vgst2Vtm_dV_gi_si*WVCoxRds);
d_T6_dV_di_si = (Vgst2Vtm*d_WVCoxRds_dV_di_si+d_Vgst2Vtm_dV_di_si*WVCoxRds);
T6 = (Vgst2Vtm*WVCoxRds);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = ((2.0*Abulk)*(d_T9_dV_b_GND+(-d_Lambda_dV_b_GND/Lambda/Lambda))+2.0*d_Abulk_dV_b_GND*((T9-1.0)+(1.0/Lambda)));
d_T0_dV_p_GND = ((2.0*Abulk)*(d_T9_dV_p_GND+(-d_Lambda_dV_p_GND/Lambda/Lambda))+2.0*d_Abulk_dV_p_GND*((T9-1.0)+(1.0/Lambda)));
d_T0_dTemp_t_GND = ((2.0*Abulk)*(d_T9_dTemp_t_GND+(-d_Lambda_dTemp_t_GND/Lambda/Lambda))+2.0*d_Abulk_dTemp_t_GND*((T9-1.0)+(1.0/Lambda)));
d_T0_dV_gi_si = ((2.0*Abulk)*(d_T9_dV_gi_si+(-d_Lambda_dV_gi_si/Lambda/Lambda))+2.0*d_Abulk_dV_gi_si*((T9-1.0)+(1.0/Lambda)));
d_T0_dV_di_si = ((2.0*Abulk)*(d_T9_dV_di_si+(-d_Lambda_dV_di_si/Lambda/Lambda))+2.0*d_Abulk_dV_di_si*((T9-1.0)+(1.0/Lambda)));
d_T0_dV_e_si = ((2.0*Abulk)*(d_T9_dV_e_si+(-d_Lambda_dV_e_si/Lambda/Lambda))+2.0*d_Abulk_dV_e_si*((T9-1.0)+(1.0/Lambda)));
d_T0_dV_b_si = ((2.0*Abulk)*(d_T9_dV_b_si+(-d_Lambda_dV_b_si/Lambda/Lambda))+2.0*d_Abulk_dV_b_si*((T9-1.0)+(1.0/Lambda)));
T0 = ((2.0*Abulk)*((T9-1.0)+(1.0/Lambda)));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (((Vgst2Vtm*(-2.0*d_Lambda_dV_e_si/Lambda/Lambda)+d_Vgst2Vtm_dV_e_si*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dV_e_si+d_Abulk_dV_e_si*EsatL))+3.0*d_T7_dV_e_si);
d_T1_dV_b_si = (((Vgst2Vtm*(-2.0*d_Lambda_dV_b_si/Lambda/Lambda)+d_Vgst2Vtm_dV_b_si*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dV_b_si+d_Abulk_dV_b_si*EsatL))+3.0*d_T7_dV_b_si);
d_T1_dV_b_GND = (((Vgst2Vtm*(-2.0*d_Lambda_dV_b_GND/Lambda/Lambda)+d_Vgst2Vtm_dV_b_GND*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dV_b_GND+d_Abulk_dV_b_GND*EsatL))+3.0*d_T7_dV_b_GND);
d_T1_dV_p_GND = (((Vgst2Vtm*(-2.0*d_Lambda_dV_p_GND/Lambda/Lambda)+d_Vgst2Vtm_dV_p_GND*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dV_p_GND+d_Abulk_dV_p_GND*EsatL))+3.0*d_T7_dV_p_GND);
d_T1_dTemp_t_GND = (((Vgst2Vtm*(-2.0*d_Lambda_dTemp_t_GND/Lambda/Lambda)+d_Vgst2Vtm_dTemp_t_GND*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dTemp_t_GND+d_Abulk_dTemp_t_GND*EsatL))+3.0*d_T7_dTemp_t_GND);
d_T1_dV_gi_si = (((Vgst2Vtm*(-2.0*d_Lambda_dV_gi_si/Lambda/Lambda)+d_Vgst2Vtm_dV_gi_si*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dV_gi_si+d_Abulk_dV_gi_si*EsatL))+3.0*d_T7_dV_gi_si);
d_T1_dV_di_si = (((Vgst2Vtm*(-2.0*d_Lambda_dV_di_si/Lambda/Lambda)+d_Vgst2Vtm_dV_di_si*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dV_di_si+d_Abulk_dV_di_si*EsatL))+3.0*d_T7_dV_di_si);
T1 = (((Vgst2Vtm*((2.0/Lambda)-1.0))+(Abulk*EsatL))+(3.0*T7));

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_e_si = (Vgst2Vtm*(d_EsatL_dV_e_si+2.0*d_T6_dV_e_si)+d_Vgst2Vtm_dV_e_si*(EsatL+(2.0*T6)));
d_T2_dV_b_si = (Vgst2Vtm*(d_EsatL_dV_b_si+2.0*d_T6_dV_b_si)+d_Vgst2Vtm_dV_b_si*(EsatL+(2.0*T6)));
d_T2_dV_b_GND = (Vgst2Vtm*(d_EsatL_dV_b_GND+2.0*d_T6_dV_b_GND)+d_Vgst2Vtm_dV_b_GND*(EsatL+(2.0*T6)));
d_T2_dV_p_GND = (Vgst2Vtm*(d_EsatL_dV_p_GND+2.0*d_T6_dV_p_GND)+d_Vgst2Vtm_dV_p_GND*(EsatL+(2.0*T6)));
d_T2_dTemp_t_GND = (Vgst2Vtm*(d_EsatL_dTemp_t_GND+2.0*d_T6_dTemp_t_GND)+d_Vgst2Vtm_dTemp_t_GND*(EsatL+(2.0*T6)));
d_T2_dV_gi_si = (Vgst2Vtm*(d_EsatL_dV_gi_si+2.0*d_T6_dV_gi_si)+d_Vgst2Vtm_dV_gi_si*(EsatL+(2.0*T6)));
d_T2_dV_di_si = (Vgst2Vtm*(d_EsatL_dV_di_si+2.0*d_T6_dV_di_si)+d_Vgst2Vtm_dV_di_si*(EsatL+(2.0*T6)));
T2 = (Vgst2Vtm*(EsatL+(2.0*T6)));
{
double value_sqrt_0 = sqrt(((T1*T1)-((2.0*T0)*T2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)-((2.0*T0)*d_T2_dV_b_GND+2.0*d_T0_dV_b_GND*T2))));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)-((2.0*T0)*d_T2_dV_p_GND+2.0*d_T0_dV_p_GND*T2))));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)-((2.0*T0)*d_T2_dTemp_t_GND+2.0*d_T0_dTemp_t_GND*T2))));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)-((2.0*T0)*d_T2_dV_gi_si+2.0*d_T0_dV_gi_si*T2))));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)-((2.0*T0)*d_T2_dV_di_si+2.0*d_T0_dV_di_si*T2))));
d_T3_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)-((2.0*T0)*d_T2_dV_e_si+2.0*d_T0_dV_e_si*T2))));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)-((2.0*T0)*d_T2_dV_b_si+2.0*d_T0_dV_b_si*T2))));
T3 = value_sqrt_0;
}

d_Vdsat_dV_b_GND = ((T0*(d_T1_dV_b_GND-d_T3_dV_b_GND)-(T1-T3)*d_T0_dV_b_GND)/T0/T0);
d_Vdsat_dV_p_GND = ((T0*(d_T1_dV_p_GND-d_T3_dV_p_GND)-(T1-T3)*d_T0_dV_p_GND)/T0/T0);
d_Vdsat_dTemp_t_GND = ((T0*(d_T1_dTemp_t_GND-d_T3_dTemp_t_GND)-(T1-T3)*d_T0_dTemp_t_GND)/T0/T0);
d_Vdsat_dV_gi_si = ((T0*(d_T1_dV_gi_si-d_T3_dV_gi_si)-(T1-T3)*d_T0_dV_gi_si)/T0/T0);
d_Vdsat_dV_di_si = ((T0*(d_T1_dV_di_si-d_T3_dV_di_si)-(T1-T3)*d_T0_dV_di_si)/T0/T0);
d_Vdsat_dV_e_si = ((T0*(d_T1_dV_e_si-d_T3_dV_e_si)-(T1-T3)*d_T0_dV_e_si)/T0/T0);
d_Vdsat_dV_b_si = ((T0*(d_T1_dV_b_si-d_T3_dV_b_si)-(T1-T3)*d_T0_dV_b_si)/T0/T0);
Vdsat = ((T1-T3)/T0);
}
VDSAT = Vdsat;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = d_Vdsat_dV_b_GND;
d_T1_dV_p_GND = d_Vdsat_dV_p_GND;
d_T1_dTemp_t_GND = d_Vdsat_dTemp_t_GND;
d_T1_dV_gi_si = d_Vdsat_dV_gi_si;
d_T1_dV_di_si = (d_Vdsat_dV_di_si-d_Vds_dV_di_si);
d_T1_dV_e_si = d_Vdsat_dV_e_si;
d_T1_dV_b_si = d_Vdsat_dV_b_si;
T1 = ((Vdsat-Vds)-pParam_B4SOIdelta);
{
double value_sqrt_0 = sqrt(((T1*T1)+((4.0*pParam_B4SOIdelta)*Vdsat)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+(4.0*pParam_B4SOIdelta)*d_Vdsat_dV_b_GND)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+(4.0*pParam_B4SOIdelta)*d_Vdsat_dV_p_GND)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+(4.0*pParam_B4SOIdelta)*d_Vdsat_dTemp_t_GND)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+(4.0*pParam_B4SOIdelta)*d_Vdsat_dV_gi_si)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+(4.0*pParam_B4SOIdelta)*d_Vdsat_dV_di_si)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+(4.0*pParam_B4SOIdelta)*d_Vdsat_dV_e_si)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+(4.0*pParam_B4SOIdelta)*d_Vdsat_dV_b_si)));
T2 = value_sqrt_0;
}

d_Vdseff_dV_b_GND = (d_Vdsat_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T2_dV_b_GND));
d_Vdseff_dV_p_GND = (d_Vdsat_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T2_dV_p_GND));
d_Vdseff_dTemp_t_GND = (d_Vdsat_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T2_dTemp_t_GND));
d_Vdseff_dV_gi_si = (d_Vdsat_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T2_dV_gi_si));
d_Vdseff_dV_di_si = (d_Vdsat_dV_di_si-0.5*(d_T1_dV_di_si+d_T2_dV_di_si));
d_Vdseff_dV_e_si = (d_Vdsat_dV_e_si-0.5*(d_T1_dV_e_si+d_T2_dV_e_si));
d_Vdseff_dV_b_si = (d_Vdsat_dV_b_si-0.5*(d_T1_dV_b_si+d_T2_dV_b_si));
Vdseff = (Vdsat-(0.5*(T1+T2)));
if ((Vdseff>Vds))
{

d_Vdseff_dV_b_si = d_Vdseff_dV_e_si = d_Vdseff_dV_gi_si = d_Vdseff_dTemp_t_GND = d_Vdseff_dV_p_GND = d_Vdseff_dV_b_GND =  0.0;
d_Vdseff_dV_di_si = d_Vds_dV_di_si;
Vdseff = Vds;
}

d_diffVds_dV_b_GND = (-d_Vdseff_dV_b_GND);
d_diffVds_dV_p_GND = (-d_Vdseff_dV_p_GND);
d_diffVds_dTemp_t_GND = (-d_Vdseff_dTemp_t_GND);
d_diffVds_dV_gi_si = (-d_Vdseff_dV_gi_si);
d_diffVds_dV_e_si = (-d_Vdseff_dV_e_si);
d_diffVds_dV_b_si = (-d_Vdseff_dV_b_si);
d_diffVds_dV_di_si = (d_Vds_dV_di_si-d_Vdseff_dV_di_si);
diffVds = (Vds-Vdseff);
B4SOIVdseff = Vdseff;

d_tmp4_dV_b_GND = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dV_b_GND+0.5*d_Abulk_dV_b_GND*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dV_b_GND)/Vgst2Vtm/Vgst2Vtm));
d_tmp4_dV_p_GND = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dV_p_GND+0.5*d_Abulk_dV_p_GND*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dV_p_GND)/Vgst2Vtm/Vgst2Vtm));
d_tmp4_dTemp_t_GND = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dTemp_t_GND+0.5*d_Abulk_dTemp_t_GND*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dTemp_t_GND)/Vgst2Vtm/Vgst2Vtm));
d_tmp4_dV_gi_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dV_gi_si+0.5*d_Abulk_dV_gi_si*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dV_gi_si)/Vgst2Vtm/Vgst2Vtm));
d_tmp4_dV_di_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dV_di_si+0.5*d_Abulk_dV_di_si*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dV_di_si)/Vgst2Vtm/Vgst2Vtm));
d_tmp4_dV_e_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dV_e_si+0.5*d_Abulk_dV_e_si*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dV_e_si)/Vgst2Vtm/Vgst2Vtm));
d_tmp4_dV_b_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dV_b_si+0.5*d_Abulk_dV_b_si*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dV_b_si)/Vgst2Vtm/Vgst2Vtm));
tmp4 = (1.0-(((0.5*Abulk)*Vdsat)/Vgst2Vtm));

d_T9_dV_e_si = (WVCoxRds*d_Vgsteff_dV_e_si+d_WVCoxRds_dV_e_si*Vgsteff);
d_T9_dV_b_si = (WVCoxRds*d_Vgsteff_dV_b_si+d_WVCoxRds_dV_b_si*Vgsteff);
d_T9_dV_b_GND = (WVCoxRds*d_Vgsteff_dV_b_GND+d_WVCoxRds_dV_b_GND*Vgsteff);
d_T9_dV_p_GND = (WVCoxRds*d_Vgsteff_dV_p_GND+d_WVCoxRds_dV_p_GND*Vgsteff);
d_T9_dTemp_t_GND = (WVCoxRds*d_Vgsteff_dTemp_t_GND+d_WVCoxRds_dTemp_t_GND*Vgsteff);
d_T9_dV_gi_si = (WVCoxRds*d_Vgsteff_dV_gi_si+d_WVCoxRds_dV_gi_si*Vgsteff);
d_T9_dV_di_si = (WVCoxRds*d_Vgsteff_dV_di_si+d_WVCoxRds_dV_di_si*Vgsteff);
T9 = (WVCoxRds*Vgsteff);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_gi_si = ((d_EsatL_dV_gi_si+d_Vdsat_dV_gi_si)+((2.0*T9)*d_tmp4_dV_gi_si+2.0*d_T9_dV_gi_si*tmp4));
d_T0_dV_di_si = ((d_EsatL_dV_di_si+d_Vdsat_dV_di_si)+((2.0*T9)*d_tmp4_dV_di_si+2.0*d_T9_dV_di_si*tmp4));
d_T0_dV_e_si = ((d_EsatL_dV_e_si+d_Vdsat_dV_e_si)+((2.0*T9)*d_tmp4_dV_e_si+2.0*d_T9_dV_e_si*tmp4));
d_T0_dV_b_si = ((d_EsatL_dV_b_si+d_Vdsat_dV_b_si)+((2.0*T9)*d_tmp4_dV_b_si+2.0*d_T9_dV_b_si*tmp4));
d_T0_dV_b_GND = ((d_EsatL_dV_b_GND+d_Vdsat_dV_b_GND)+((2.0*T9)*d_tmp4_dV_b_GND+2.0*d_T9_dV_b_GND*tmp4));
d_T0_dV_p_GND = ((d_EsatL_dV_p_GND+d_Vdsat_dV_p_GND)+((2.0*T9)*d_tmp4_dV_p_GND+2.0*d_T9_dV_p_GND*tmp4));
d_T0_dTemp_t_GND = ((d_EsatL_dTemp_t_GND+d_Vdsat_dTemp_t_GND)+((2.0*T9)*d_tmp4_dTemp_t_GND+2.0*d_T9_dTemp_t_GND*tmp4));
T0 = ((EsatL+Vdsat)+((2.0*T9)*tmp4));

d_T9_dV_e_si = (WVCoxRds*d_Abulk_dV_e_si+d_WVCoxRds_dV_e_si*Abulk);
d_T9_dV_b_si = (WVCoxRds*d_Abulk_dV_b_si+d_WVCoxRds_dV_b_si*Abulk);
d_T9_dV_b_GND = (WVCoxRds*d_Abulk_dV_b_GND+d_WVCoxRds_dV_b_GND*Abulk);
d_T9_dV_p_GND = (WVCoxRds*d_Abulk_dV_p_GND+d_WVCoxRds_dV_p_GND*Abulk);
d_T9_dTemp_t_GND = (WVCoxRds*d_Abulk_dTemp_t_GND+d_WVCoxRds_dTemp_t_GND*Abulk);
d_T9_dV_gi_si = (WVCoxRds*d_Abulk_dV_gi_si+d_WVCoxRds_dV_gi_si*Abulk);
d_T9_dV_di_si = (WVCoxRds*d_Abulk_dV_di_si+d_WVCoxRds_dV_di_si*Abulk);
T9 = (WVCoxRds*Abulk);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = ((-2.0*d_Lambda_dV_b_GND/Lambda/Lambda)+d_T9_dV_b_GND);
d_T1_dV_p_GND = ((-2.0*d_Lambda_dV_p_GND/Lambda/Lambda)+d_T9_dV_p_GND);
d_T1_dTemp_t_GND = ((-2.0*d_Lambda_dTemp_t_GND/Lambda/Lambda)+d_T9_dTemp_t_GND);
d_T1_dV_e_si = ((-2.0*d_Lambda_dV_e_si/Lambda/Lambda)+d_T9_dV_e_si);
d_T1_dV_gi_si = ((-2.0*d_Lambda_dV_gi_si/Lambda/Lambda)+d_T9_dV_gi_si);
d_T1_dV_di_si = ((-2.0*d_Lambda_dV_di_si/Lambda/Lambda)+d_T9_dV_di_si);
d_T1_dV_b_si = ((-2.0*d_Lambda_dV_b_si/Lambda/Lambda)+d_T9_dV_b_si);
T1 = (((2.0/Lambda)-1.0)+T9);

d_Vasat_dV_b_GND = ((T1*d_T0_dV_b_GND-T0*d_T1_dV_b_GND)/T1/T1);
d_Vasat_dV_p_GND = ((T1*d_T0_dV_p_GND-T0*d_T1_dV_p_GND)/T1/T1);
d_Vasat_dTemp_t_GND = ((T1*d_T0_dTemp_t_GND-T0*d_T1_dTemp_t_GND)/T1/T1);
d_Vasat_dV_e_si = ((T1*d_T0_dV_e_si-T0*d_T1_dV_e_si)/T1/T1);
d_Vasat_dV_gi_si = ((T1*d_T0_dV_gi_si-T0*d_T1_dV_gi_si)/T1/T1);
d_Vasat_dV_di_si = ((T1*d_T0_dV_di_si-T0*d_T1_dV_di_si)/T1/T1);
d_Vasat_dV_b_si = ((T1*d_T0_dV_b_si-T0*d_T1_dV_b_si)/T1/T1);
Vasat = (T0/T1);
if (((pParam_B4SOIpclm>0.0)&&(diffVds>1.0e-10)))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-pParam_B4SOIpclm*d_Abulk_dV_b_GND*pParam_B4SOIlitl/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl)/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));
d_T0_dV_p_GND = (-pParam_B4SOIpclm*d_Abulk_dV_p_GND*pParam_B4SOIlitl/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl)/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));
d_T0_dTemp_t_GND = (-pParam_B4SOIpclm*d_Abulk_dTemp_t_GND*pParam_B4SOIlitl/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl)/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));
d_T0_dV_gi_si = (-pParam_B4SOIpclm*d_Abulk_dV_gi_si*pParam_B4SOIlitl/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl)/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));
d_T0_dV_di_si = (-pParam_B4SOIpclm*d_Abulk_dV_di_si*pParam_B4SOIlitl/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl)/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));
d_T0_dV_e_si = (-pParam_B4SOIpclm*d_Abulk_dV_e_si*pParam_B4SOIlitl/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl)/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));
d_T0_dV_b_si = (-pParam_B4SOIpclm*d_Abulk_dV_b_si*pParam_B4SOIlitl/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl)/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));
T0 = (1.0/((pParam_B4SOIpclm*Abulk)*pParam_B4SOIlitl));

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_e_si = ((EsatL*d_Vgsteff_dV_e_si-Vgsteff*d_EsatL_dV_e_si)/EsatL/EsatL);
d_T2_dV_b_si = ((EsatL*d_Vgsteff_dV_b_si-Vgsteff*d_EsatL_dV_b_si)/EsatL/EsatL);
d_T2_dV_b_GND = ((EsatL*d_Vgsteff_dV_b_GND-Vgsteff*d_EsatL_dV_b_GND)/EsatL/EsatL);
d_T2_dV_p_GND = ((EsatL*d_Vgsteff_dV_p_GND-Vgsteff*d_EsatL_dV_p_GND)/EsatL/EsatL);
d_T2_dTemp_t_GND = ((EsatL*d_Vgsteff_dTemp_t_GND-Vgsteff*d_EsatL_dTemp_t_GND)/EsatL/EsatL);
d_T2_dV_gi_si = ((EsatL*d_Vgsteff_dV_gi_si-Vgsteff*d_EsatL_dV_gi_si)/EsatL/EsatL);
d_T2_dV_di_si = ((EsatL*d_Vgsteff_dV_di_si-Vgsteff*d_EsatL_dV_di_si)/EsatL/EsatL);
T2 = (Vgsteff/EsatL);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = Leff*(d_Abulk_dV_b_GND+d_T2_dV_b_GND);
d_T1_dV_p_GND = Leff*(d_Abulk_dV_p_GND+d_T2_dV_p_GND);
d_T1_dTemp_t_GND = Leff*(d_Abulk_dTemp_t_GND+d_T2_dTemp_t_GND);
d_T1_dV_gi_si = Leff*(d_Abulk_dV_gi_si+d_T2_dV_gi_si);
d_T1_dV_di_si = Leff*(d_Abulk_dV_di_si+d_T2_dV_di_si);
d_T1_dV_e_si = Leff*(d_Abulk_dV_e_si+d_T2_dV_e_si);
d_T1_dV_b_si = Leff*(d_Abulk_dV_b_si+d_T2_dV_b_si);
T1 = (Leff*(Abulk+T2));

d_T9_dV_b_GND = (T0*d_T1_dV_b_GND+d_T0_dV_b_GND*T1);
d_T9_dV_p_GND = (T0*d_T1_dV_p_GND+d_T0_dV_p_GND*T1);
d_T9_dTemp_t_GND = (T0*d_T1_dTemp_t_GND+d_T0_dTemp_t_GND*T1);
d_T9_dV_e_si = (T0*d_T1_dV_e_si+d_T0_dV_e_si*T1);
d_T9_dV_gi_si = (T0*d_T1_dV_gi_si+d_T0_dV_gi_si*T1);
d_T9_dV_di_si = (T0*d_T1_dV_di_si+d_T0_dV_di_si*T1);
d_T9_dV_b_si = (T0*d_T1_dV_b_si+d_T0_dV_b_si*T1);
T9 = (T0*T1);

d_VACLM_dV_b_GND = (T9*d_diffVds_dV_b_GND+d_T9_dV_b_GND*diffVds);
d_VACLM_dV_p_GND = (T9*d_diffVds_dV_p_GND+d_T9_dV_p_GND*diffVds);
d_VACLM_dTemp_t_GND = (T9*d_diffVds_dTemp_t_GND+d_T9_dTemp_t_GND*diffVds);
d_VACLM_dV_gi_si = (T9*d_diffVds_dV_gi_si+d_T9_dV_gi_si*diffVds);
d_VACLM_dV_di_si = (T9*d_diffVds_dV_di_si+d_T9_dV_di_si*diffVds);
d_VACLM_dV_e_si = (T9*d_diffVds_dV_e_si+d_T9_dV_e_si*diffVds);
d_VACLM_dV_b_si = (T9*d_diffVds_dV_b_si+d_T9_dV_b_si*diffVds);
VACLM = (T9*diffVds);
}
else
{

d_VACLM_dV_b_si = d_VACLM_dV_e_si = d_VACLM_dV_di_si = d_VACLM_dV_gi_si = d_VACLM_dTemp_t_GND = d_VACLM_dV_p_GND = d_VACLM_dV_b_GND =  0.0;
VACLM = 2.688117142e+43;
}
if ((thetaRout>0.0))
{

d_T8_dV_b_GND = (Abulk*d_Vdsat_dV_b_GND+d_Abulk_dV_b_GND*Vdsat);
d_T8_dV_p_GND = (Abulk*d_Vdsat_dV_p_GND+d_Abulk_dV_p_GND*Vdsat);
d_T8_dTemp_t_GND = (Abulk*d_Vdsat_dTemp_t_GND+d_Abulk_dTemp_t_GND*Vdsat);
d_T8_dV_gi_si = (Abulk*d_Vdsat_dV_gi_si+d_Abulk_dV_gi_si*Vdsat);
d_T8_dV_di_si = (Abulk*d_Vdsat_dV_di_si+d_Abulk_dV_di_si*Vdsat);
d_T8_dV_e_si = (Abulk*d_Vdsat_dV_e_si+d_Abulk_dV_e_si*Vdsat);
d_T8_dV_b_si = (Abulk*d_Vdsat_dV_b_si+d_Abulk_dV_b_si*Vdsat);
T8 = (Abulk*Vdsat);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (Vgst2Vtm*d_T8_dV_e_si+d_Vgst2Vtm_dV_e_si*T8);
d_T0_dV_b_si = (Vgst2Vtm*d_T8_dV_b_si+d_Vgst2Vtm_dV_b_si*T8);
d_T0_dV_b_GND = (Vgst2Vtm*d_T8_dV_b_GND+d_Vgst2Vtm_dV_b_GND*T8);
d_T0_dV_p_GND = (Vgst2Vtm*d_T8_dV_p_GND+d_Vgst2Vtm_dV_p_GND*T8);
d_T0_dTemp_t_GND = (Vgst2Vtm*d_T8_dTemp_t_GND+d_Vgst2Vtm_dTemp_t_GND*T8);
d_T0_dV_gi_si = (Vgst2Vtm*d_T8_dV_gi_si+d_Vgst2Vtm_dV_gi_si*T8);
d_T0_dV_di_si = (Vgst2Vtm*d_T8_dV_di_si+d_Vgst2Vtm_dV_di_si*T8);
T0 = (Vgst2Vtm*T8);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (d_Vgst2Vtm_dV_e_si+d_T8_dV_e_si);
d_T1_dV_b_si = (d_Vgst2Vtm_dV_b_si+d_T8_dV_b_si);
d_T1_dV_b_GND = (d_Vgst2Vtm_dV_b_GND+d_T8_dV_b_GND);
d_T1_dV_p_GND = (d_Vgst2Vtm_dV_p_GND+d_T8_dV_p_GND);
d_T1_dTemp_t_GND = (d_Vgst2Vtm_dTemp_t_GND+d_T8_dTemp_t_GND);
d_T1_dV_gi_si = (d_Vgst2Vtm_dV_gi_si+d_T8_dV_gi_si);
d_T1_dV_di_si = (d_Vgst2Vtm_dV_di_si+d_T8_dV_di_si);
T1 = (Vgst2Vtm+T8);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si =  0.0;
d_T2_dV_b_GND = d_thetaRout_dV_b_GND;
d_T2_dV_p_GND = d_thetaRout_dV_p_GND;
d_T2_dTemp_t_GND = d_thetaRout_dTemp_t_GND;
T2 = thetaRout;

d_VADIBL_dV_e_si = ((T2*(d_Vgst2Vtm_dV_e_si-((T1*d_T0_dV_e_si-T0*d_T1_dV_e_si)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dV_e_si)/T2/T2);
d_VADIBL_dV_b_si = ((T2*(d_Vgst2Vtm_dV_b_si-((T1*d_T0_dV_b_si-T0*d_T1_dV_b_si)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dV_b_si)/T2/T2);
d_VADIBL_dV_b_GND = ((T2*(d_Vgst2Vtm_dV_b_GND-((T1*d_T0_dV_b_GND-T0*d_T1_dV_b_GND)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dV_b_GND)/T2/T2);
d_VADIBL_dV_p_GND = ((T2*(d_Vgst2Vtm_dV_p_GND-((T1*d_T0_dV_p_GND-T0*d_T1_dV_p_GND)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dV_p_GND)/T2/T2);
d_VADIBL_dTemp_t_GND = ((T2*(d_Vgst2Vtm_dTemp_t_GND-((T1*d_T0_dTemp_t_GND-T0*d_T1_dTemp_t_GND)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dTemp_t_GND)/T2/T2);
d_VADIBL_dV_gi_si = ((T2*(d_Vgst2Vtm_dV_gi_si-((T1*d_T0_dV_gi_si-T0*d_T1_dV_gi_si)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dV_gi_si)/T2/T2);
d_VADIBL_dV_di_si = ((T2*(d_Vgst2Vtm_dV_di_si-((T1*d_T0_dV_di_si-T0*d_T1_dV_di_si)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dV_di_si)/T2/T2);
VADIBL = ((Vgst2Vtm-(T0/T1))/T2);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = pParam_B4SOIpdiblb*d_Vbseff_dV_b_GND;
d_T7_dV_p_GND = pParam_B4SOIpdiblb*d_Vbseff_dV_p_GND;
d_T7_dTemp_t_GND = pParam_B4SOIpdiblb*d_Vbseff_dTemp_t_GND;
d_T7_dV_e_si = pParam_B4SOIpdiblb*d_Vbseff_dV_e_si;
d_T7_dV_gi_si = pParam_B4SOIpdiblb*d_Vbseff_dV_gi_si;
d_T7_dV_di_si = pParam_B4SOIpdiblb*d_Vbseff_dV_di_si;
d_T7_dV_b_si = pParam_B4SOIpdiblb*d_Vbseff_dV_b_si;
T7 = (pParam_B4SOIpdiblb*Vbseff);
if ((T7>=(-0.9)))
{

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = (-d_T7_dV_b_GND/(1.0+T7)/(1.0+T7));
d_T3_dV_p_GND = (-d_T7_dV_p_GND/(1.0+T7)/(1.0+T7));
d_T3_dTemp_t_GND = (-d_T7_dTemp_t_GND/(1.0+T7)/(1.0+T7));
d_T3_dV_gi_si = (-d_T7_dV_gi_si/(1.0+T7)/(1.0+T7));
d_T3_dV_di_si = (-d_T7_dV_di_si/(1.0+T7)/(1.0+T7));
d_T3_dV_e_si = (-d_T7_dV_e_si/(1.0+T7)/(1.0+T7));
d_T3_dV_b_si = (-d_T7_dV_b_si/(1.0+T7)/(1.0+T7));
T3 = (1.0/(1.0+T7));

d_VADIBL_dV_e_si = (VADIBL*d_T3_dV_e_si+d_VADIBL_dV_e_si*T3);
d_VADIBL_dV_b_si = (VADIBL*d_T3_dV_b_si+d_VADIBL_dV_b_si*T3);
d_VADIBL_dV_b_GND = (VADIBL*d_T3_dV_b_GND+d_VADIBL_dV_b_GND*T3);
d_VADIBL_dV_p_GND = (VADIBL*d_T3_dV_p_GND+d_VADIBL_dV_p_GND*T3);
d_VADIBL_dTemp_t_GND = (VADIBL*d_T3_dTemp_t_GND+d_VADIBL_dTemp_t_GND*T3);
d_VADIBL_dV_gi_si = (VADIBL*d_T3_dV_gi_si+d_VADIBL_dV_gi_si*T3);
d_VADIBL_dV_di_si = (VADIBL*d_T3_dV_di_si+d_VADIBL_dV_di_si*T3);
VADIBL = (VADIBL*T3);
}
else
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_GND = (-d_T7_dV_b_GND/(0.8+T7)/(0.8+T7));
d_T4_dV_p_GND = (-d_T7_dV_p_GND/(0.8+T7)/(0.8+T7));
d_T4_dTemp_t_GND = (-d_T7_dTemp_t_GND/(0.8+T7)/(0.8+T7));
d_T4_dV_gi_si = (-d_T7_dV_gi_si/(0.8+T7)/(0.8+T7));
d_T4_dV_di_si = (-d_T7_dV_di_si/(0.8+T7)/(0.8+T7));
d_T4_dV_e_si = (-d_T7_dV_e_si/(0.8+T7)/(0.8+T7));
d_T4_dV_b_si = (-d_T7_dV_b_si/(0.8+T7)/(0.8+T7));
T4 = (1.0/(0.8+T7));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_GND = ((17.0+(20.0*T7))*d_T4_dV_b_GND+20.0*d_T7_dV_b_GND*T4);
d_T3_dV_p_GND = ((17.0+(20.0*T7))*d_T4_dV_p_GND+20.0*d_T7_dV_p_GND*T4);
d_T3_dTemp_t_GND = ((17.0+(20.0*T7))*d_T4_dTemp_t_GND+20.0*d_T7_dTemp_t_GND*T4);
d_T3_dV_gi_si = ((17.0+(20.0*T7))*d_T4_dV_gi_si+20.0*d_T7_dV_gi_si*T4);
d_T3_dV_di_si = ((17.0+(20.0*T7))*d_T4_dV_di_si+20.0*d_T7_dV_di_si*T4);
d_T3_dV_e_si = ((17.0+(20.0*T7))*d_T4_dV_e_si+20.0*d_T7_dV_e_si*T4);
d_T3_dV_b_si = ((17.0+(20.0*T7))*d_T4_dV_b_si+20.0*d_T7_dV_b_si*T4);
T3 = ((17.0+(20.0*T7))*T4);

d_VADIBL_dV_e_si = (VADIBL*d_T3_dV_e_si+d_VADIBL_dV_e_si*T3);
d_VADIBL_dV_b_si = (VADIBL*d_T3_dV_b_si+d_VADIBL_dV_b_si*T3);
d_VADIBL_dV_b_GND = (VADIBL*d_T3_dV_b_GND+d_VADIBL_dV_b_GND*T3);
d_VADIBL_dV_p_GND = (VADIBL*d_T3_dV_p_GND+d_VADIBL_dV_p_GND*T3);
d_VADIBL_dTemp_t_GND = (VADIBL*d_T3_dTemp_t_GND+d_VADIBL_dTemp_t_GND*T3);
d_VADIBL_dV_gi_si = (VADIBL*d_T3_dV_gi_si+d_VADIBL_dV_gi_si*T3);
d_VADIBL_dV_di_si = (VADIBL*d_T3_dV_di_si+d_VADIBL_dV_di_si*T3);
VADIBL = (VADIBL*T3);
}
}
else
{

d_VADIBL_dV_di_si = d_VADIBL_dV_gi_si = d_VADIBL_dTemp_t_GND = d_VADIBL_dV_p_GND = d_VADIBL_dV_b_GND = d_VADIBL_dV_b_si = d_VADIBL_dV_e_si =  0.0;
VADIBL = 2.688117142e+43;
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = pParam_B4SOIpditsd*d_Vds_dV_di_si;
T0 = (pParam_B4SOIpditsd*Vds);
if ((T0>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 2.688117142e+43;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
T1 = value_exp_0;
}
}
if ((pParam_B4SOIpdits>3.720075976e-44))
{

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = (1.0+(B4SOIpditsl*Leff));

d_VADITS_dV_b_GND = ((T2*d_T1_dV_b_GND+d_T2_dV_b_GND*T1)/pParam_B4SOIpdits);
d_VADITS_dV_p_GND = ((T2*d_T1_dV_p_GND+d_T2_dV_p_GND*T1)/pParam_B4SOIpdits);
d_VADITS_dTemp_t_GND = ((T2*d_T1_dTemp_t_GND+d_T2_dTemp_t_GND*T1)/pParam_B4SOIpdits);
d_VADITS_dV_gi_si = ((T2*d_T1_dV_gi_si+d_T2_dV_gi_si*T1)/pParam_B4SOIpdits);
d_VADITS_dV_di_si = ((T2*d_T1_dV_di_si+d_T2_dV_di_si*T1)/pParam_B4SOIpdits);
d_VADITS_dV_e_si = ((T2*d_T1_dV_e_si+d_T2_dV_e_si*T1)/pParam_B4SOIpdits);
d_VADITS_dV_b_si = ((T2*d_T1_dV_b_si+d_T2_dV_b_si*T1)/pParam_B4SOIpdits);
VADITS = ((1.0+(T2*T1))/pParam_B4SOIpdits);

d_VADITS_dV_b_GND = (VADITS*d_FP_dV_b_GND+d_VADITS_dV_b_GND*FP);
d_VADITS_dV_p_GND = (VADITS*d_FP_dV_p_GND+d_VADITS_dV_p_GND*FP);
d_VADITS_dTemp_t_GND = (VADITS*d_FP_dTemp_t_GND+d_VADITS_dTemp_t_GND*FP);
d_VADITS_dV_gi_si = (VADITS*d_FP_dV_gi_si+d_VADITS_dV_gi_si*FP);
d_VADITS_dV_di_si = (VADITS*d_FP_dV_di_si+d_VADITS_dV_di_si*FP);
d_VADITS_dV_e_si = (VADITS*d_FP_dV_e_si+d_VADITS_dV_e_si*FP);
d_VADITS_dV_b_si = (VADITS*d_FP_dV_b_si+d_VADITS_dV_b_si*FP);
VADITS = (VADITS*FP);
}
else
{

d_VADITS_dV_b_si = d_VADITS_dV_e_si = d_VADITS_dV_di_si = d_VADITS_dV_gi_si = d_VADITS_dTemp_t_GND = d_VADITS_dV_p_GND = d_VADITS_dV_b_GND =  0.0;
VADITS = 2.688117142e+43;
}

d_T8_dV_gi_si = (-pParam_B4SOIpvag*d_EsatL_dV_gi_si/EsatL/EsatL);
d_T8_dV_di_si = (-pParam_B4SOIpvag*d_EsatL_dV_di_si/EsatL/EsatL);
d_T8_dV_e_si = (-pParam_B4SOIpvag*d_EsatL_dV_e_si/EsatL/EsatL);
d_T8_dV_b_si = (-pParam_B4SOIpvag*d_EsatL_dV_b_si/EsatL/EsatL);
d_T8_dV_b_GND = (-pParam_B4SOIpvag*d_EsatL_dV_b_GND/EsatL/EsatL);
d_T8_dV_p_GND = (-pParam_B4SOIpvag*d_EsatL_dV_p_GND/EsatL/EsatL);
d_T8_dTemp_t_GND = (-pParam_B4SOIpvag*d_EsatL_dTemp_t_GND/EsatL/EsatL);
T8 = (pParam_B4SOIpvag/EsatL);

d_T9_dV_b_GND = (T8*d_Vgsteff_dV_b_GND+d_T8_dV_b_GND*Vgsteff);
d_T9_dV_p_GND = (T8*d_Vgsteff_dV_p_GND+d_T8_dV_p_GND*Vgsteff);
d_T9_dTemp_t_GND = (T8*d_Vgsteff_dTemp_t_GND+d_T8_dTemp_t_GND*Vgsteff);
d_T9_dV_gi_si = (T8*d_Vgsteff_dV_gi_si+d_T8_dV_gi_si*Vgsteff);
d_T9_dV_di_si = (T8*d_Vgsteff_dV_di_si+d_T8_dV_di_si*Vgsteff);
d_T9_dV_e_si = (T8*d_Vgsteff_dV_e_si+d_T8_dV_e_si*Vgsteff);
d_T9_dV_b_si = (T8*d_Vgsteff_dV_b_si+d_T8_dV_b_si*Vgsteff);
T9 = (T8*Vgsteff);
if ((T9>(-0.9)))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = d_T9_dV_b_GND;
d_T0_dV_p_GND = d_T9_dV_p_GND;
d_T0_dTemp_t_GND = d_T9_dTemp_t_GND;
d_T0_dV_gi_si = d_T9_dV_gi_si;
d_T0_dV_di_si = d_T9_dV_di_si;
d_T0_dV_e_si = d_T9_dV_e_si;
d_T0_dV_b_si = d_T9_dV_b_si;
T0 = (1.0+T9);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = (-20.0*d_T9_dV_b_GND/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
d_T1_dV_p_GND = (-20.0*d_T9_dV_p_GND/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
d_T1_dTemp_t_GND = (-20.0*d_T9_dTemp_t_GND/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
d_T1_dV_gi_si = (-20.0*d_T9_dV_gi_si/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
d_T1_dV_di_si = (-20.0*d_T9_dV_di_si/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
d_T1_dV_e_si = (-20.0*d_T9_dV_e_si/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
d_T1_dV_b_si = (-20.0*d_T9_dV_b_si/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
T1 = (1.0/(17.0+(20.0*T9)));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = ((0.8+T9)*d_T1_dV_b_GND+d_T9_dV_b_GND*T1);
d_T0_dV_p_GND = ((0.8+T9)*d_T1_dV_p_GND+d_T9_dV_p_GND*T1);
d_T0_dTemp_t_GND = ((0.8+T9)*d_T1_dTemp_t_GND+d_T9_dTemp_t_GND*T1);
d_T0_dV_gi_si = ((0.8+T9)*d_T1_dV_gi_si+d_T9_dV_gi_si*T1);
d_T0_dV_di_si = ((0.8+T9)*d_T1_dV_di_si+d_T9_dV_di_si*T1);
d_T0_dV_e_si = ((0.8+T9)*d_T1_dV_e_si+d_T9_dV_e_si*T1);
d_T0_dV_b_si = ((0.8+T9)*d_T1_dV_b_si+d_T9_dV_b_si*T1);
T0 = ((0.8+T9)*T1);
}

d_tmp3_dV_b_GND = (d_VACLM_dV_b_GND+d_VADIBL_dV_b_GND);
d_tmp3_dV_p_GND = (d_VACLM_dV_p_GND+d_VADIBL_dV_p_GND);
d_tmp3_dTemp_t_GND = (d_VACLM_dTemp_t_GND+d_VADIBL_dTemp_t_GND);
d_tmp3_dV_gi_si = (d_VACLM_dV_gi_si+d_VADIBL_dV_gi_si);
d_tmp3_dV_di_si = (d_VACLM_dV_di_si+d_VADIBL_dV_di_si);
d_tmp3_dV_e_si = (d_VACLM_dV_e_si+d_VADIBL_dV_e_si);
d_tmp3_dV_b_si = (d_VACLM_dV_b_si+d_VADIBL_dV_b_si);
tmp3 = (VACLM+VADIBL);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = ((tmp3*(VACLM*d_VADIBL_dV_b_GND+d_VACLM_dV_b_GND*VADIBL)-(VACLM*VADIBL)*d_tmp3_dV_b_GND)/tmp3/tmp3);
d_T1_dV_p_GND = ((tmp3*(VACLM*d_VADIBL_dV_p_GND+d_VACLM_dV_p_GND*VADIBL)-(VACLM*VADIBL)*d_tmp3_dV_p_GND)/tmp3/tmp3);
d_T1_dTemp_t_GND = ((tmp3*(VACLM*d_VADIBL_dTemp_t_GND+d_VACLM_dTemp_t_GND*VADIBL)-(VACLM*VADIBL)*d_tmp3_dTemp_t_GND)/tmp3/tmp3);
d_T1_dV_gi_si = ((tmp3*(VACLM*d_VADIBL_dV_gi_si+d_VACLM_dV_gi_si*VADIBL)-(VACLM*VADIBL)*d_tmp3_dV_gi_si)/tmp3/tmp3);
d_T1_dV_di_si = ((tmp3*(VACLM*d_VADIBL_dV_di_si+d_VACLM_dV_di_si*VADIBL)-(VACLM*VADIBL)*d_tmp3_dV_di_si)/tmp3/tmp3);
d_T1_dV_e_si = ((tmp3*(VACLM*d_VADIBL_dV_e_si+d_VACLM_dV_e_si*VADIBL)-(VACLM*VADIBL)*d_tmp3_dV_e_si)/tmp3/tmp3);
d_T1_dV_b_si = ((tmp3*(VACLM*d_VADIBL_dV_b_si+d_VACLM_dV_b_si*VADIBL)-(VACLM*VADIBL)*d_tmp3_dV_b_si)/tmp3/tmp3);
T1 = ((VACLM*VADIBL)/tmp3);

d_tmp3_dV_b_GND = (d_T1_dV_b_GND+d_VADITS_dV_b_GND);
d_tmp3_dV_p_GND = (d_T1_dV_p_GND+d_VADITS_dV_p_GND);
d_tmp3_dTemp_t_GND = (d_T1_dTemp_t_GND+d_VADITS_dTemp_t_GND);
d_tmp3_dV_gi_si = (d_T1_dV_gi_si+d_VADITS_dV_gi_si);
d_tmp3_dV_di_si = (d_T1_dV_di_si+d_VADITS_dV_di_si);
d_tmp3_dV_e_si = (d_T1_dV_e_si+d_VADITS_dV_e_si);
d_tmp3_dV_b_si = (d_T1_dV_b_si+d_VADITS_dV_b_si);
tmp3 = (T1+VADITS);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = ((tmp3*(T1*d_VADITS_dV_b_GND+d_T1_dV_b_GND*VADITS)-(T1*VADITS)*d_tmp3_dV_b_GND)/tmp3/tmp3);
d_T2_dV_p_GND = ((tmp3*(T1*d_VADITS_dV_p_GND+d_T1_dV_p_GND*VADITS)-(T1*VADITS)*d_tmp3_dV_p_GND)/tmp3/tmp3);
d_T2_dTemp_t_GND = ((tmp3*(T1*d_VADITS_dTemp_t_GND+d_T1_dTemp_t_GND*VADITS)-(T1*VADITS)*d_tmp3_dTemp_t_GND)/tmp3/tmp3);
d_T2_dV_gi_si = ((tmp3*(T1*d_VADITS_dV_gi_si+d_T1_dV_gi_si*VADITS)-(T1*VADITS)*d_tmp3_dV_gi_si)/tmp3/tmp3);
d_T2_dV_di_si = ((tmp3*(T1*d_VADITS_dV_di_si+d_T1_dV_di_si*VADITS)-(T1*VADITS)*d_tmp3_dV_di_si)/tmp3/tmp3);
d_T2_dV_e_si = ((tmp3*(T1*d_VADITS_dV_e_si+d_T1_dV_e_si*VADITS)-(T1*VADITS)*d_tmp3_dV_e_si)/tmp3/tmp3);
d_T2_dV_b_si = ((tmp3*(T1*d_VADITS_dV_b_si+d_T1_dV_b_si*VADITS)-(T1*VADITS)*d_tmp3_dV_b_si)/tmp3/tmp3);
T2 = ((T1*VADITS)/tmp3);

d_Va_dV_b_GND = (d_Vasat_dV_b_GND+(T0*d_T2_dV_b_GND+d_T0_dV_b_GND*T2));
d_Va_dV_p_GND = (d_Vasat_dV_p_GND+(T0*d_T2_dV_p_GND+d_T0_dV_p_GND*T2));
d_Va_dTemp_t_GND = (d_Vasat_dTemp_t_GND+(T0*d_T2_dTemp_t_GND+d_T0_dTemp_t_GND*T2));
d_Va_dV_e_si = (d_Vasat_dV_e_si+(T0*d_T2_dV_e_si+d_T0_dV_e_si*T2));
d_Va_dV_gi_si = (d_Vasat_dV_gi_si+(T0*d_T2_dV_gi_si+d_T0_dV_gi_si*T2));
d_Va_dV_di_si = (d_Vasat_dV_di_si+(T0*d_T2_dV_di_si+d_T0_dV_di_si*T2));
d_Va_dV_b_si = (d_Vasat_dV_b_si+(T0*d_T2_dV_b_si+d_T0_dV_b_si*T2));
Va = (Vasat+(T0*T2));

d_CoxWovL_dV_e_si = (B4SOIcox*d_Weff_dV_e_si/Leff);
d_CoxWovL_dV_b_si = (B4SOIcox*d_Weff_dV_b_si/Leff);
d_CoxWovL_dV_b_GND = (B4SOIcox*d_Weff_dV_b_GND/Leff);
d_CoxWovL_dV_p_GND = (B4SOIcox*d_Weff_dV_p_GND/Leff);
d_CoxWovL_dTemp_t_GND = (B4SOIcox*d_Weff_dTemp_t_GND/Leff);
d_CoxWovL_dV_gi_si = (B4SOIcox*d_Weff_dV_gi_si/Leff);
d_CoxWovL_dV_di_si = (B4SOIcox*d_Weff_dV_di_si/Leff);
CoxWovL = ((B4SOIcox*Weff)/Leff);

d_beta_dV_gi_si = (ueff*d_CoxWovL_dV_gi_si+d_ueff_dV_gi_si*CoxWovL);
d_beta_dV_di_si = (ueff*d_CoxWovL_dV_di_si+d_ueff_dV_di_si*CoxWovL);
d_beta_dV_e_si = (ueff*d_CoxWovL_dV_e_si+d_ueff_dV_e_si*CoxWovL);
d_beta_dV_b_si = (ueff*d_CoxWovL_dV_b_si+d_ueff_dV_b_si*CoxWovL);
d_beta_dV_b_GND = (ueff*d_CoxWovL_dV_b_GND+d_ueff_dV_b_GND*CoxWovL);
d_beta_dV_p_GND = (ueff*d_CoxWovL_dV_p_GND+d_ueff_dV_p_GND*CoxWovL);
d_beta_dTemp_t_GND = (ueff*d_CoxWovL_dTemp_t_GND+d_ueff_dTemp_t_GND*CoxWovL);
beta = (ueff*CoxWovL);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_b_GND+0.5*d_Abulk_dV_b_GND*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_b_GND)/Vgst2Vtm/Vgst2Vtm));
d_T0_dV_p_GND = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_p_GND+0.5*d_Abulk_dV_p_GND*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_p_GND)/Vgst2Vtm/Vgst2Vtm));
d_T0_dTemp_t_GND = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dTemp_t_GND+0.5*d_Abulk_dTemp_t_GND*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dTemp_t_GND)/Vgst2Vtm/Vgst2Vtm));
d_T0_dV_gi_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_gi_si+0.5*d_Abulk_dV_gi_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_gi_si)/Vgst2Vtm/Vgst2Vtm));
d_T0_dV_di_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_di_si+0.5*d_Abulk_dV_di_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_di_si)/Vgst2Vtm/Vgst2Vtm));
d_T0_dV_e_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_e_si+0.5*d_Abulk_dV_e_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_e_si)/Vgst2Vtm/Vgst2Vtm));
d_T0_dV_b_si = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_b_si+0.5*d_Abulk_dV_b_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_b_si)/Vgst2Vtm/Vgst2Vtm));
T0 = (1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm));

d_fgche1_dV_e_si = (Vgsteff*d_T0_dV_e_si+d_Vgsteff_dV_e_si*T0);
d_fgche1_dV_b_si = (Vgsteff*d_T0_dV_b_si+d_Vgsteff_dV_b_si*T0);
d_fgche1_dV_b_GND = (Vgsteff*d_T0_dV_b_GND+d_Vgsteff_dV_b_GND*T0);
d_fgche1_dV_p_GND = (Vgsteff*d_T0_dV_p_GND+d_Vgsteff_dV_p_GND*T0);
d_fgche1_dTemp_t_GND = (Vgsteff*d_T0_dTemp_t_GND+d_Vgsteff_dTemp_t_GND*T0);
d_fgche1_dV_gi_si = (Vgsteff*d_T0_dV_gi_si+d_Vgsteff_dV_gi_si*T0);
d_fgche1_dV_di_si = (Vgsteff*d_T0_dV_di_si+d_Vgsteff_dV_di_si*T0);
fgche1 = (Vgsteff*T0);

d_T9_dV_b_GND = ((EsatL*d_Vdseff_dV_b_GND-Vdseff*d_EsatL_dV_b_GND)/EsatL/EsatL);
d_T9_dV_p_GND = ((EsatL*d_Vdseff_dV_p_GND-Vdseff*d_EsatL_dV_p_GND)/EsatL/EsatL);
d_T9_dTemp_t_GND = ((EsatL*d_Vdseff_dTemp_t_GND-Vdseff*d_EsatL_dTemp_t_GND)/EsatL/EsatL);
d_T9_dV_gi_si = ((EsatL*d_Vdseff_dV_gi_si-Vdseff*d_EsatL_dV_gi_si)/EsatL/EsatL);
d_T9_dV_di_si = ((EsatL*d_Vdseff_dV_di_si-Vdseff*d_EsatL_dV_di_si)/EsatL/EsatL);
d_T9_dV_e_si = ((EsatL*d_Vdseff_dV_e_si-Vdseff*d_EsatL_dV_e_si)/EsatL/EsatL);
d_T9_dV_b_si = ((EsatL*d_Vdseff_dV_b_si-Vdseff*d_EsatL_dV_b_si)/EsatL/EsatL);
T9 = (Vdseff/EsatL);

d_fgche2_dV_b_GND = d_T9_dV_b_GND;
d_fgche2_dV_p_GND = d_T9_dV_p_GND;
d_fgche2_dTemp_t_GND = d_T9_dTemp_t_GND;
d_fgche2_dV_gi_si = d_T9_dV_gi_si;
d_fgche2_dV_di_si = d_T9_dV_di_si;
d_fgche2_dV_e_si = d_T9_dV_e_si;
d_fgche2_dV_b_si = d_T9_dV_b_si;
fgche2 = (1.0+T9);

d_gche_dV_gi_si = ((fgche2*(beta*d_fgche1_dV_gi_si+d_beta_dV_gi_si*fgche1)-(beta*fgche1)*d_fgche2_dV_gi_si)/fgche2/fgche2);
d_gche_dV_di_si = ((fgche2*(beta*d_fgche1_dV_di_si+d_beta_dV_di_si*fgche1)-(beta*fgche1)*d_fgche2_dV_di_si)/fgche2/fgche2);
d_gche_dV_e_si = ((fgche2*(beta*d_fgche1_dV_e_si+d_beta_dV_e_si*fgche1)-(beta*fgche1)*d_fgche2_dV_e_si)/fgche2/fgche2);
d_gche_dV_b_si = ((fgche2*(beta*d_fgche1_dV_b_si+d_beta_dV_b_si*fgche1)-(beta*fgche1)*d_fgche2_dV_b_si)/fgche2/fgche2);
d_gche_dV_b_GND = ((fgche2*(beta*d_fgche1_dV_b_GND+d_beta_dV_b_GND*fgche1)-(beta*fgche1)*d_fgche2_dV_b_GND)/fgche2/fgche2);
d_gche_dV_p_GND = ((fgche2*(beta*d_fgche1_dV_p_GND+d_beta_dV_p_GND*fgche1)-(beta*fgche1)*d_fgche2_dV_p_GND)/fgche2/fgche2);
d_gche_dTemp_t_GND = ((fgche2*(beta*d_fgche1_dTemp_t_GND+d_beta_dTemp_t_GND*fgche1)-(beta*fgche1)*d_fgche2_dTemp_t_GND)/fgche2/fgche2);
gche = ((beta*fgche1)/fgche2);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_gi_si = (gche*d_Rds_dV_gi_si+d_gche_dV_gi_si*Rds);
d_T0_dV_di_si = (gche*d_Rds_dV_di_si+d_gche_dV_di_si*Rds);
d_T0_dV_e_si = (gche*d_Rds_dV_e_si+d_gche_dV_e_si*Rds);
d_T0_dV_b_si = (gche*d_Rds_dV_b_si+d_gche_dV_b_si*Rds);
d_T0_dV_b_GND = (gche*d_Rds_dV_b_GND+d_gche_dV_b_GND*Rds);
d_T0_dV_p_GND = (gche*d_Rds_dV_p_GND+d_gche_dV_p_GND*Rds);
d_T0_dTemp_t_GND = (gche*d_Rds_dTemp_t_GND+d_gche_dTemp_t_GND*Rds);
T0 = (1.0+(gche*Rds));

d_T9_dV_b_GND = ((T0*d_Vdseff_dV_b_GND-Vdseff*d_T0_dV_b_GND)/T0/T0);
d_T9_dV_p_GND = ((T0*d_Vdseff_dV_p_GND-Vdseff*d_T0_dV_p_GND)/T0/T0);
d_T9_dTemp_t_GND = ((T0*d_Vdseff_dTemp_t_GND-Vdseff*d_T0_dTemp_t_GND)/T0/T0);
d_T9_dV_gi_si = ((T0*d_Vdseff_dV_gi_si-Vdseff*d_T0_dV_gi_si)/T0/T0);
d_T9_dV_di_si = ((T0*d_Vdseff_dV_di_si-Vdseff*d_T0_dV_di_si)/T0/T0);
d_T9_dV_e_si = ((T0*d_Vdseff_dV_e_si-Vdseff*d_T0_dV_e_si)/T0/T0);
d_T9_dV_b_si = ((T0*d_Vdseff_dV_b_si-Vdseff*d_T0_dV_b_si)/T0/T0);
T9 = (Vdseff/T0);

d_Idl_dV_gi_si = (gche*d_T9_dV_gi_si+d_gche_dV_gi_si*T9);
d_Idl_dV_di_si = (gche*d_T9_dV_di_si+d_gche_dV_di_si*T9);
d_Idl_dV_e_si = (gche*d_T9_dV_e_si+d_gche_dV_e_si*T9);
d_Idl_dV_b_si = (gche*d_T9_dV_b_si+d_gche_dV_b_si*T9);
d_Idl_dV_b_GND = (gche*d_T9_dV_b_GND+d_gche_dV_b_GND*T9);
d_Idl_dV_p_GND = (gche*d_T9_dV_p_GND+d_gche_dV_p_GND*T9);
d_Idl_dTemp_t_GND = (gche*d_T9_dTemp_t_GND+d_gche_dTemp_t_GND*T9);
Idl = (gche*T9);

d_IdlovVdseff_dV_gi_si = ((T0*d_gche_dV_gi_si-gche*d_T0_dV_gi_si)/T0/T0);
d_IdlovVdseff_dV_di_si = ((T0*d_gche_dV_di_si-gche*d_T0_dV_di_si)/T0/T0);
d_IdlovVdseff_dV_e_si = ((T0*d_gche_dV_e_si-gche*d_T0_dV_e_si)/T0/T0);
d_IdlovVdseff_dV_b_si = ((T0*d_gche_dV_b_si-gche*d_T0_dV_b_si)/T0/T0);
d_IdlovVdseff_dV_b_GND = ((T0*d_gche_dV_b_GND-gche*d_T0_dV_b_GND)/T0/T0);
d_IdlovVdseff_dV_p_GND = ((T0*d_gche_dV_p_GND-gche*d_T0_dV_p_GND)/T0/T0);
d_IdlovVdseff_dTemp_t_GND = ((T0*d_gche_dTemp_t_GND-gche*d_T0_dTemp_t_GND)/T0/T0);
IdlovVdseff = (gche/T0);

d_T9_dV_b_GND = ((Va*d_diffVds_dV_b_GND-diffVds*d_Va_dV_b_GND)/Va/Va);
d_T9_dV_p_GND = ((Va*d_diffVds_dV_p_GND-diffVds*d_Va_dV_p_GND)/Va/Va);
d_T9_dTemp_t_GND = ((Va*d_diffVds_dTemp_t_GND-diffVds*d_Va_dTemp_t_GND)/Va/Va);
d_T9_dV_gi_si = ((Va*d_diffVds_dV_gi_si-diffVds*d_Va_dV_gi_si)/Va/Va);
d_T9_dV_e_si = ((Va*d_diffVds_dV_e_si-diffVds*d_Va_dV_e_si)/Va/Va);
d_T9_dV_b_si = ((Va*d_diffVds_dV_b_si-diffVds*d_Va_dV_b_si)/Va/Va);
d_T9_dV_di_si = ((Va*d_diffVds_dV_di_si-diffVds*d_Va_dV_di_si)/Va/Va);
T9 = (diffVds/Va);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = d_T9_dV_b_GND;
d_T0_dV_p_GND = d_T9_dV_p_GND;
d_T0_dTemp_t_GND = d_T9_dTemp_t_GND;
d_T0_dV_gi_si = d_T9_dV_gi_si;
d_T0_dV_di_si = d_T9_dV_di_si;
d_T0_dV_e_si = d_T9_dV_e_si;
d_T0_dV_b_si = d_T9_dV_b_si;
T0 = (1.0+T9);

d_Ids_dV_gi_si = ((Idl*d_T0_dV_gi_si+d_Idl_dV_gi_si*T0)/B4SOInseg);
d_Ids_dV_di_si = ((Idl*d_T0_dV_di_si+d_Idl_dV_di_si*T0)/B4SOInseg);
d_Ids_dV_e_si = ((Idl*d_T0_dV_e_si+d_Idl_dV_e_si*T0)/B4SOInseg);
d_Ids_dV_b_si = ((Idl*d_T0_dV_b_si+d_Idl_dV_b_si*T0)/B4SOInseg);
d_Ids_dV_b_GND = ((Idl*d_T0_dV_b_GND+d_Idl_dV_b_GND*T0)/B4SOInseg);
d_Ids_dV_p_GND = ((Idl*d_T0_dV_p_GND+d_Idl_dV_p_GND*T0)/B4SOInseg);
d_Ids_dTemp_t_GND = ((Idl*d_T0_dTemp_t_GND+d_Idl_dTemp_t_GND*T0)/B4SOInseg);
Ids = ((Idl*T0)/B4SOInseg);
B4SOIidovVds = ((IdlovVdseff*T0)/B4SOInseg);
if ((B4SOIidovVds<1e-9))
{
B4SOIidovVds = 1e-9;
}

d_IdovVds_dV_gi_si = ((IdlovVdseff*d_T0_dV_gi_si+d_IdlovVdseff_dV_gi_si*T0)/B4SOInseg);
d_IdovVds_dV_di_si = ((IdlovVdseff*d_T0_dV_di_si+d_IdlovVdseff_dV_di_si*T0)/B4SOInseg);
d_IdovVds_dV_e_si = ((IdlovVdseff*d_T0_dV_e_si+d_IdlovVdseff_dV_e_si*T0)/B4SOInseg);
d_IdovVds_dV_b_si = ((IdlovVdseff*d_T0_dV_b_si+d_IdlovVdseff_dV_b_si*T0)/B4SOInseg);
d_IdovVds_dV_b_GND = ((IdlovVdseff*d_T0_dV_b_GND+d_IdlovVdseff_dV_b_GND*T0)/B4SOInseg);
d_IdovVds_dV_p_GND = ((IdlovVdseff*d_T0_dV_p_GND+d_IdlovVdseff_dV_p_GND*T0)/B4SOInseg);
d_IdovVds_dTemp_t_GND = ((IdlovVdseff*d_T0_dTemp_t_GND+d_IdlovVdseff_dTemp_t_GND*T0)/B4SOInseg);
IdovVds = ((IdlovVdseff*T0)/B4SOInseg);
if ((B4SOIsoiMod!=2))
{
if ((B4SOImtrlMod==0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (((3.0*3.9)/epsrox)*toxe);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = ((B4SOIepsrsub*toxe)/epsrox);
}
if ((B4SOIgidlMod==0))
{
if ((B4SOImtrlMod==0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-(((-Vds)-Vgd_eff)-egisl)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-(((-Vds)-Vgd_eff)-egisl)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-d_Vgd_eff_dV_b_GND)-(((-Vds)-Vgd_eff)-egisl)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-d_Vgd_eff_dV_p_GND)-(((-Vds)-Vgd_eff)-egisl)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-d_Vgd_eff_dTemp_t_GND)-(((-Vds)-Vgd_eff)-egisl)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-d_Vgd_eff_dV_gi_si)-(((-Vds)-Vgd_eff)-egisl)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*((-d_Vds_dV_di_si)-d_Vgd_eff_dV_di_si)-(((-Vds)-Vgd_eff)-egisl)*d_T0_dV_di_si)/T0/T0);
T1 = ((((-Vds)-Vgd_eff)-egisl)/T0);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-d_Vgd_eff_dV_b_GND)-((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-d_Vgd_eff_dV_p_GND)-((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-d_Vgd_eff_dTemp_t_GND)-((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-d_Vgd_eff_dV_gi_si)-((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*((-d_Vds_dV_di_si)-d_Vgd_eff_dV_di_si)-((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_di_si)/T0/T0);
T1 = (((((-Vds)-Vgd_eff)-egisl)+pParam_B4SOIvfbsd)/T0);
}
if ((((agisl<=0.0)||(bgisl<=0.0))||(cgisl<0.0)))
{

d_Igisl_dV_b_si = d_Igisl_dV_e_si = d_Igisl_dV_di_si = d_Igisl_dV_gi_si = d_Igisl_dTemp_t_GND = d_Igisl_dV_p_GND = d_Igisl_dV_b_GND =  0.0;
Igisl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_GND,0.0);
d_T1_dV_p_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_p_GND,0.0);
d_T1_dTemp_t_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dTemp_t_GND,0.0);
d_T1_dV_gi_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_gi_si,0.0);
d_T1_dV_di_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_di_si,0.0);
d_T1_dV_e_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_e_si,0.0);
d_T1_dV_b_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_si,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (-bgisl*d_T1_dV_b_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_p_GND = (-bgisl*d_T1_dV_p_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dTemp_t_GND = (-bgisl*d_T1_dTemp_t_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_gi_si = (-bgisl*d_T1_dV_gi_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_di_si = (-bgisl*d_T1_dV_di_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_e_si = (-bgisl*d_T1_dV_e_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_b_si = (-bgisl*d_T1_dV_b_si/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgisl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;

d_Igisl_dV_b_GND = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_GND)))+(wdios*agisl)*d_T1_dV_b_GND*value_exp_0);
d_Igisl_dV_p_GND = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_p_GND)))+(wdios*agisl)*d_T1_dV_p_GND*value_exp_0);
d_Igisl_dTemp_t_GND = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dTemp_t_GND)))+(wdios*agisl)*d_T1_dTemp_t_GND*value_exp_0);
d_Igisl_dV_gi_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_gi_si)))+(wdios*agisl)*d_T1_dV_gi_si*value_exp_0);
d_Igisl_dV_di_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_di_si)))+(wdios*agisl)*d_T1_dV_di_si*value_exp_0);
d_Igisl_dV_e_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_e_si)))+(wdios*agisl)*d_T1_dV_e_si*value_exp_0);
d_Igisl_dV_b_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_si)))+(wdios*agisl)*d_T1_dV_b_si*value_exp_0);
Igisl = (((wdios*agisl)*T1)*value_exp_0);
}

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_si = (Vbs*d_Vbs_dV_b_si+d_Vbs_dV_b_si*Vbs);
d_T4_dV_di_si = (Vbs*d_Vbs_dV_di_si+d_Vbs_dV_di_si*Vbs);
d_T4_dV_e_si = (Vbs*d_Vbs_dV_e_si+d_Vbs_dV_e_si*Vbs);
d_T4_dV_b_GND = (Vbs*d_Vbs_dV_b_GND+d_Vbs_dV_b_GND*Vbs);
d_T4_dV_p_GND = (Vbs*d_Vbs_dV_p_GND+d_Vbs_dV_p_GND*Vbs);
d_T4_dTemp_t_GND = (Vbs*d_Vbs_dTemp_t_GND+d_Vbs_dTemp_t_GND*Vbs);
d_T4_dV_gi_si = (Vbs*d_Vbs_dV_gi_si+d_Vbs_dV_gi_si*Vbs);
T4 = (Vbs*Vbs);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_si = ((-Vbs)*d_T4_dV_b_si+(-d_Vbs_dV_b_si)*T4);
d_T5_dV_di_si = ((-Vbs)*d_T4_dV_di_si+(-d_Vbs_dV_di_si)*T4);
d_T5_dV_e_si = ((-Vbs)*d_T4_dV_e_si+(-d_Vbs_dV_e_si)*T4);
d_T5_dV_b_GND = ((-Vbs)*d_T4_dV_b_GND+(-d_Vbs_dV_b_GND)*T4);
d_T5_dV_p_GND = ((-Vbs)*d_T4_dV_p_GND+(-d_Vbs_dV_p_GND)*T4);
d_T5_dTemp_t_GND = ((-Vbs)*d_T4_dTemp_t_GND+(-d_Vbs_dTemp_t_GND)*T4);
d_T5_dV_gi_si = ((-Vbs)*d_T4_dV_gi_si+(-d_Vbs_dV_gi_si)*T4);
T5 = ((-Vbs)*T4);
{
double value_fabs_0 = fabs(T5);
double  deriv_fabs_0_d0 = (((T5)>=0)?(+1.0):(-1.0));

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_b_GND = (deriv_fabs_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_fabs_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_fabs_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_fabs_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_fabs_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_fabs_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_fabs_0_d0*(d_T5_dV_b_si));
T6 = ((cgisl+value_fabs_0)+1.0E-9);
}
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0((T5/T6),1.0E-6);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_b_GND-T5*d_T6_dV_b_GND)/T6/T6),0.0);
d_T7_dV_p_GND = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_p_GND-T5*d_T6_dV_p_GND)/T6/T6),0.0);
d_T7_dTemp_t_GND = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dTemp_t_GND-T5*d_T6_dTemp_t_GND)/T6/T6),0.0);
d_T7_dV_gi_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_gi_si-T5*d_T6_dV_gi_si)/T6/T6),0.0);
d_T7_dV_di_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_di_si-T5*d_T6_dV_di_si)/T6/T6),0.0);
d_T7_dV_e_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_e_si-T5*d_T6_dV_e_si)/T6/T6),0.0);
d_T7_dV_b_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_b_si-T5*d_T6_dV_b_si)/T6/T6),0.0);
T7 = (evaluator_hypsmooth_0.getValues((T5/T6),1.0E-6)-1.0E-6);
}

d_Igisl_dV_b_GND = (Igisl*d_T7_dV_b_GND+d_Igisl_dV_b_GND*T7);
d_Igisl_dV_p_GND = (Igisl*d_T7_dV_p_GND+d_Igisl_dV_p_GND*T7);
d_Igisl_dTemp_t_GND = (Igisl*d_T7_dTemp_t_GND+d_Igisl_dTemp_t_GND*T7);
d_Igisl_dV_gi_si = (Igisl*d_T7_dV_gi_si+d_Igisl_dV_gi_si*T7);
d_Igisl_dV_di_si = (Igisl*d_T7_dV_di_si+d_Igisl_dV_di_si*T7);
d_Igisl_dV_e_si = (Igisl*d_T7_dV_e_si+d_Igisl_dV_e_si*T7);
d_Igisl_dV_b_si = (Igisl*d_T7_dV_b_si+d_Igisl_dV_b_si*T7);
Igisl = (Igisl*T7);
}
if ((B4SOImtrlMod==0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-((Vds-Vgs_eff)-egidl)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-((Vds-Vgs_eff)-egidl)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-d_Vgs_eff_dV_b_GND)-((Vds-Vgs_eff)-egidl)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-d_Vgs_eff_dV_p_GND)-((Vds-Vgs_eff)-egidl)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-d_Vgs_eff_dTemp_t_GND)-((Vds-Vgs_eff)-egidl)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-d_Vgs_eff_dV_gi_si)-((Vds-Vgs_eff)-egidl)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*(d_Vds_dV_di_si-d_Vgs_eff_dV_di_si)-((Vds-Vgs_eff)-egidl)*d_T0_dV_di_si)/T0/T0);
T1 = (((Vds-Vgs_eff)-egidl)/T0);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-(((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-(((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-d_Vgs_eff_dV_b_GND)-(((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-d_Vgs_eff_dV_p_GND)-(((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-d_Vgs_eff_dTemp_t_GND)-(((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-d_Vgs_eff_dV_gi_si)-(((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*(d_Vds_dV_di_si-d_Vgs_eff_dV_di_si)-(((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_di_si)/T0/T0);
T1 = ((((Vds-Vgs_eff)-egidl)+pParam_B4SOIvfbsd)/T0);
}
if ((((agidl<=0.0)||(bgidl<=0.0))||(cgidl<0.0)))
{

d_Igidl_dV_b_si = d_Igidl_dV_e_si = d_Igidl_dV_di_si = d_Igidl_dV_gi_si = d_Igidl_dTemp_t_GND = d_Igidl_dV_p_GND = d_Igidl_dV_b_GND =  0.0;
Igidl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_GND,0.0);
d_T1_dV_p_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_p_GND,0.0);
d_T1_dTemp_t_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dTemp_t_GND,0.0);
d_T1_dV_gi_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_gi_si,0.0);
d_T1_dV_di_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_di_si,0.0);
d_T1_dV_e_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_e_si,0.0);
d_T1_dV_b_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_si,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (-bgidl*d_T1_dV_b_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_p_GND = (-bgidl*d_T1_dV_p_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dTemp_t_GND = (-bgidl*d_T1_dTemp_t_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_gi_si = (-bgidl*d_T1_dV_gi_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_di_si = (-bgidl*d_T1_dV_di_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_e_si = (-bgidl*d_T1_dV_e_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_b_si = (-bgidl*d_T1_dV_b_si/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgidl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;

d_Igidl_dV_b_GND = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_GND)))+(wdiod*agidl)*d_T1_dV_b_GND*value_exp_0);
d_Igidl_dV_p_GND = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_p_GND)))+(wdiod*agidl)*d_T1_dV_p_GND*value_exp_0);
d_Igidl_dTemp_t_GND = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dTemp_t_GND)))+(wdiod*agidl)*d_T1_dTemp_t_GND*value_exp_0);
d_Igidl_dV_gi_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_gi_si)))+(wdiod*agidl)*d_T1_dV_gi_si*value_exp_0);
d_Igidl_dV_di_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_di_si)))+(wdiod*agidl)*d_T1_dV_di_si*value_exp_0);
d_Igidl_dV_e_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_e_si)))+(wdiod*agidl)*d_T1_dV_e_si*value_exp_0);
d_Igidl_dV_b_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_si)))+(wdiod*agidl)*d_T1_dV_b_si*value_exp_0);
Igidl = (((wdiod*agidl)*T1)*value_exp_0);
}

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_e_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
d_T4_dV_di_si = (Vbd*d_Vbd_dV_di_si+d_Vbd_dV_di_si*Vbd);
d_T4_dV_b_si = (Vbd*d_Vbd_dV_b_si+d_Vbd_dV_b_si*Vbd);
T4 = (Vbd*Vbd);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = (-Vbd)*d_T4_dV_b_GND;
d_T5_dV_p_GND = (-Vbd)*d_T4_dV_p_GND;
d_T5_dTemp_t_GND = (-Vbd)*d_T4_dTemp_t_GND;
d_T5_dV_gi_si = (-Vbd)*d_T4_dV_gi_si;
d_T5_dV_e_si = (-Vbd)*d_T4_dV_e_si;
d_T5_dV_di_si = ((-Vbd)*d_T4_dV_di_si+(-d_Vbd_dV_di_si)*T4);
d_T5_dV_b_si = ((-Vbd)*d_T4_dV_b_si+(-d_Vbd_dV_b_si)*T4);
T5 = ((-Vbd)*T4);
{
double value_fabs_0 = fabs(T5);
double  deriv_fabs_0_d0 = (((T5)>=0)?(+1.0):(-1.0));

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_b_GND = (deriv_fabs_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_fabs_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_fabs_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_fabs_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_fabs_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_fabs_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_fabs_0_d0*(d_T5_dV_b_si));
T6 = ((cgidl+value_fabs_0)+1.0E-9);
}
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0((T5/T6),1.0E-6);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_b_GND-T5*d_T6_dV_b_GND)/T6/T6),0.0);
d_T7_dV_p_GND = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_p_GND-T5*d_T6_dV_p_GND)/T6/T6),0.0);
d_T7_dTemp_t_GND = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dTemp_t_GND-T5*d_T6_dTemp_t_GND)/T6/T6),0.0);
d_T7_dV_gi_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_gi_si-T5*d_T6_dV_gi_si)/T6/T6),0.0);
d_T7_dV_di_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_di_si-T5*d_T6_dV_di_si)/T6/T6),0.0);
d_T7_dV_e_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_e_si-T5*d_T6_dV_e_si)/T6/T6),0.0);
d_T7_dV_b_si = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dV_b_si-T5*d_T6_dV_b_si)/T6/T6),0.0);
T7 = (evaluator_hypsmooth_0.getValues((T5/T6),1.0E-6)-1.0E-6);
}

d_Igidl_dV_b_GND = (Igidl*d_T7_dV_b_GND+d_Igidl_dV_b_GND*T7);
d_Igidl_dV_p_GND = (Igidl*d_T7_dV_p_GND+d_Igidl_dV_p_GND*T7);
d_Igidl_dTemp_t_GND = (Igidl*d_T7_dTemp_t_GND+d_Igidl_dTemp_t_GND*T7);
d_Igidl_dV_gi_si = (Igidl*d_T7_dV_gi_si+d_Igidl_dV_gi_si*T7);
d_Igidl_dV_di_si = (Igidl*d_T7_dV_di_si+d_Igidl_dV_di_si*T7);
d_Igidl_dV_e_si = (Igidl*d_T7_dV_e_si+d_Igidl_dV_e_si*T7);
d_Igidl_dV_b_si = (Igidl*d_T7_dV_b_si+d_Igidl_dV_b_si*T7);
Igidl = (Igidl*T7);
}
}
else
{
if ((B4SOImtrlMod==0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-rgisl*d_Vgd_eff_dV_b_GND)-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-rgisl*d_Vgd_eff_dV_p_GND)-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-rgisl*d_Vgd_eff_dTemp_t_GND)-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-rgisl*d_Vgd_eff_dV_gi_si)-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*((-d_Vds_dV_di_si)-rgisl*d_Vgd_eff_dV_di_si)-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dV_di_si)/T0/T0);
T1 = ((((-Vds)-(rgisl*Vgd_eff))-egisl)/T0);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-rgisl*d_Vgd_eff_dV_b_GND)-((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-rgisl*d_Vgd_eff_dV_p_GND)-((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-rgisl*d_Vgd_eff_dTemp_t_GND)-((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-rgisl*d_Vgd_eff_dV_gi_si)-((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*((-d_Vds_dV_di_si)-rgisl*d_Vgd_eff_dV_di_si)-((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)*d_T0_dV_di_si)/T0/T0);
T1 = (((((-Vds)-(rgisl*Vgd_eff))-egisl)+pParam_B4SOIvfbsd)/T0);
}
if ((((agisl<=0.0)||(bgisl<=0.0))||(cgisl<0.0)))
{

d_Igisl_dV_b_si = d_Igisl_dV_e_si = d_Igisl_dV_di_si = d_Igisl_dV_gi_si = d_Igisl_dTemp_t_GND = d_Igisl_dV_p_GND = d_Igisl_dV_b_GND =  0.0;
Igisl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_GND,0.0);
d_T1_dV_p_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_p_GND,0.0);
d_T1_dTemp_t_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dTemp_t_GND,0.0);
d_T1_dV_gi_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_gi_si,0.0);
d_T1_dV_di_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_di_si,0.0);
d_T1_dV_e_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_e_si,0.0);
d_T1_dV_b_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_si,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (-bgisl*d_T1_dV_b_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_p_GND = (-bgisl*d_T1_dV_p_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dTemp_t_GND = (-bgisl*d_T1_dTemp_t_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_gi_si = (-bgisl*d_T1_dV_gi_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_di_si = (-bgisl*d_T1_dV_di_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_e_si = (-bgisl*d_T1_dV_e_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_b_si = (-bgisl*d_T1_dV_b_si/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgisl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;

d_Igisl_dV_b_GND = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_GND)))+(wdios*agisl)*d_T1_dV_b_GND*value_exp_0);
d_Igisl_dV_p_GND = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_p_GND)))+(wdios*agisl)*d_T1_dV_p_GND*value_exp_0);
d_Igisl_dTemp_t_GND = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dTemp_t_GND)))+(wdios*agisl)*d_T1_dTemp_t_GND*value_exp_0);
d_Igisl_dV_gi_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_gi_si)))+(wdios*agisl)*d_T1_dV_gi_si*value_exp_0);
d_Igisl_dV_di_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_di_si)))+(wdios*agisl)*d_T1_dV_di_si*value_exp_0);
d_Igisl_dV_e_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_e_si)))+(wdios*agisl)*d_T1_dV_e_si*value_exp_0);
d_Igisl_dV_b_si = (((wdios*agisl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_si)))+(wdios*agisl)*d_T1_dV_b_si*value_exp_0);
Igisl = (((wdios*agisl)*T1)*value_exp_0);
}

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si =  0.0;
d_T4_dV_b_si = d_Vbs_dV_b_si;
d_T4_dV_di_si = d_Vbs_dV_di_si;
d_T4_dV_e_si = d_Vbs_dV_e_si;
d_T4_dV_b_GND = d_Vbs_dV_b_GND;
d_T4_dV_p_GND = d_Vbs_dV_p_GND;
d_T4_dTemp_t_GND = d_Vbs_dTemp_t_GND;
d_T4_dV_gi_si = d_Vbs_dV_gi_si;
T4 = (Vbs-fgisl);
if ((T4>=((-1.0)/100.0)))
{

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si = d_T5_dV_di_si = d_T5_dV_gi_si = d_T5_dTemp_t_GND = d_T5_dV_p_GND = d_T5_dV_b_GND =  0.0;
T5 = ((-kgisl)*100.0);
}
else
{

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = (-kgisl*d_T4_dV_b_GND/T4/T4);
d_T5_dV_p_GND = (-kgisl*d_T4_dV_p_GND/T4/T4);
d_T5_dTemp_t_GND = (-kgisl*d_T4_dTemp_t_GND/T4/T4);
d_T5_dV_gi_si = (-kgisl*d_T4_dV_gi_si/T4/T4);
d_T5_dV_di_si = (-kgisl*d_T4_dV_di_si/T4/T4);
d_T5_dV_e_si = (-kgisl*d_T4_dV_e_si/T4/T4);
d_T5_dV_b_si = (-kgisl*d_T4_dV_b_si/T4/T4);
T5 = (kgisl/T4);
}
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_b_GND = (deriv_exp_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_exp_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_exp_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_exp_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_exp_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_exp_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_exp_0_d0*(d_T5_dV_b_si));
T6 = value_exp_0;
}

d_Igisl_dV_b_GND = (Igisl*d_T6_dV_b_GND+d_Igisl_dV_b_GND*T6);
d_Igisl_dV_p_GND = (Igisl*d_T6_dV_p_GND+d_Igisl_dV_p_GND*T6);
d_Igisl_dTemp_t_GND = (Igisl*d_T6_dTemp_t_GND+d_Igisl_dTemp_t_GND*T6);
d_Igisl_dV_gi_si = (Igisl*d_T6_dV_gi_si+d_Igisl_dV_gi_si*T6);
d_Igisl_dV_di_si = (Igisl*d_T6_dV_di_si+d_Igisl_dV_di_si*T6);
d_Igisl_dV_e_si = (Igisl*d_T6_dV_e_si+d_Igisl_dV_e_si*T6);
d_Igisl_dV_b_si = (Igisl*d_T6_dV_b_si+d_Igisl_dV_b_si*T6);
Igisl = (Igisl*T6);
}
if ((B4SOImtrlMod==0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-rgidl*d_Vgs_eff_dV_b_GND)-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-rgidl*d_Vgs_eff_dV_p_GND)-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-rgidl*d_Vgs_eff_dTemp_t_GND)-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-rgidl*d_Vgs_eff_dV_gi_si)-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*(d_Vds_dV_di_si-rgidl*d_Vgs_eff_dV_di_si)-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dV_di_si)/T0/T0);
T1 = (((Vds-(rgidl*Vgs_eff))-egidl)/T0);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (-(((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_e_si/T0/T0);
d_T1_dV_b_si = (-(((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_b_si/T0/T0);
d_T1_dV_b_GND = ((T0*(-rgidl*d_Vgs_eff_dV_b_GND)-(((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_b_GND)/T0/T0);
d_T1_dV_p_GND = ((T0*(-rgidl*d_Vgs_eff_dV_p_GND)-(((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_p_GND)/T0/T0);
d_T1_dTemp_t_GND = ((T0*(-rgidl*d_Vgs_eff_dTemp_t_GND)-(((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)*d_T0_dTemp_t_GND)/T0/T0);
d_T1_dV_gi_si = ((T0*(-rgidl*d_Vgs_eff_dV_gi_si)-(((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_gi_si)/T0/T0);
d_T1_dV_di_si = ((T0*(d_Vds_dV_di_si-rgidl*d_Vgs_eff_dV_di_si)-(((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)*d_T0_dV_di_si)/T0/T0);
T1 = ((((Vds-(rgidl*Vgs_eff))-egidl)+pParam_B4SOIvfbsd)/T0);
}
if ((((agidl<=0.0)||(bgidl<=0.0))||(cgidl<0.0)))
{

d_Igidl_dV_b_si = d_Igidl_dV_e_si = d_Igidl_dV_di_si = d_Igidl_dV_gi_si = d_Igidl_dTemp_t_GND = d_Igidl_dV_p_GND = d_Igidl_dV_b_GND =  0.0;
Igidl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_GND,0.0);
d_T1_dV_p_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_p_GND,0.0);
d_T1_dTemp_t_GND = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dTemp_t_GND,0.0);
d_T1_dV_gi_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_gi_si,0.0);
d_T1_dV_di_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_di_si,0.0);
d_T1_dV_e_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_e_si,0.0);
d_T1_dV_b_si = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dV_b_si,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (-bgidl*d_T1_dV_b_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_p_GND = (-bgidl*d_T1_dV_p_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dTemp_t_GND = (-bgidl*d_T1_dTemp_t_GND/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_gi_si = (-bgidl*d_T1_dV_gi_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_di_si = (-bgidl*d_T1_dV_di_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_e_si = (-bgidl*d_T1_dV_e_si/(T1+1.0E-3)/(T1+1.0E-3));
d_T2_dV_b_si = (-bgidl*d_T1_dV_b_si/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgidl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;

d_Igidl_dV_b_GND = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_GND)))+(wdiod*agidl)*d_T1_dV_b_GND*value_exp_0);
d_Igidl_dV_p_GND = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_p_GND)))+(wdiod*agidl)*d_T1_dV_p_GND*value_exp_0);
d_Igidl_dTemp_t_GND = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dTemp_t_GND)))+(wdiod*agidl)*d_T1_dTemp_t_GND*value_exp_0);
d_Igidl_dV_gi_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_gi_si)))+(wdiod*agidl)*d_T1_dV_gi_si*value_exp_0);
d_Igidl_dV_di_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_di_si)))+(wdiod*agidl)*d_T1_dV_di_si*value_exp_0);
d_Igidl_dV_e_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_e_si)))+(wdiod*agidl)*d_T1_dV_e_si*value_exp_0);
d_Igidl_dV_b_si = (((wdiod*agidl)*T1)*(deriv_exp_0_d0*((-d_T2_dV_b_si)))+(wdiod*agidl)*d_T1_dV_b_si*value_exp_0);
Igidl = (((wdiod*agidl)*T1)*value_exp_0);
}

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_e_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
d_T4_dV_di_si = d_Vbd_dV_di_si;
d_T4_dV_b_si = d_Vbd_dV_b_si;
T4 = (Vbd-fgidl);
if ((T4>=((-1.0)/100.0)))
{

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si = d_T5_dV_di_si = d_T5_dV_gi_si = d_T5_dTemp_t_GND = d_T5_dV_p_GND = d_T5_dV_b_GND =  0.0;
T5 = ((-kgidl)*100.0);
}
else
{

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_b_GND = (-kgidl*d_T4_dV_b_GND/T4/T4);
d_T5_dV_p_GND = (-kgidl*d_T4_dV_p_GND/T4/T4);
d_T5_dTemp_t_GND = (-kgidl*d_T4_dTemp_t_GND/T4/T4);
d_T5_dV_gi_si = (-kgidl*d_T4_dV_gi_si/T4/T4);
d_T5_dV_di_si = (-kgidl*d_T4_dV_di_si/T4/T4);
d_T5_dV_e_si = (-kgidl*d_T4_dV_e_si/T4/T4);
d_T5_dV_b_si = (-kgidl*d_T4_dV_b_si/T4/T4);
T5 = (kgidl/T4);
}
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si =  0.0;
d_T6_dV_b_GND = (deriv_exp_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_exp_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_exp_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_exp_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_exp_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_exp_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_exp_0_d0*(d_T5_dV_b_si));
T6 = value_exp_0;
}

d_Igidl_dV_b_GND = (Igidl*d_T6_dV_b_GND+d_Igidl_dV_b_GND*T6);
d_Igidl_dV_p_GND = (Igidl*d_T6_dV_p_GND+d_Igidl_dV_p_GND*T6);
d_Igidl_dTemp_t_GND = (Igidl*d_T6_dTemp_t_GND+d_Igidl_dTemp_t_GND*T6);
d_Igidl_dV_gi_si = (Igidl*d_T6_dV_gi_si+d_Igidl_dV_gi_si*T6);
d_Igidl_dV_di_si = (Igidl*d_T6_dV_di_si+d_Igidl_dV_di_si*T6);
d_Igidl_dV_e_si = (Igidl*d_T6_dV_e_si+d_Igidl_dV_e_si*T6);
d_Igidl_dV_b_si = (Igidl*d_T6_dV_b_si+d_Igidl_dV_b_si*T6);
Igidl = (Igidl*T6);
}
}
WsTsi = (pParam_B4SOIwdios*B4SOItsi);
WdTsi = (pParam_B4SOIwdiod*B4SOItsi);

d_NVtm1_dV_b_GND = d_Vtm_dV_b_GND*pParam_B4SOIndiode;
d_NVtm1_dV_p_GND = d_Vtm_dV_p_GND*pParam_B4SOIndiode;
d_NVtm1_dTemp_t_GND = d_Vtm_dTemp_t_GND*pParam_B4SOIndiode;
NVtm1 = (Vtm*pParam_B4SOIndiode);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (-vsbs*d_NVtm1_dV_b_GND/NVtm1/NVtm1);
d_T0_dV_p_GND = (-vsbs*d_NVtm1_dV_p_GND/NVtm1/NVtm1);
d_T0_dTemp_t_GND = (-vsbs*d_NVtm1_dTemp_t_GND/NVtm1/NVtm1);
d_T0_dV_sb_si = (d_vsbs_dV_sb_si/NVtm1);
T0 = (vsbs/NVtm1);
if ((T0>100.0))
{

d_ExpVbsNVtm_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_ExpVbsNVtm_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_ExpVbsNVtm_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_ExpVbsNVtm_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_ExpVbsNVtm_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_ExpVbsNVtm_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_ExpVbsNVtm_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_ExpVbsNVtm_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
ExpVbsNVtm = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_ExpVbsNVtm_dV_sb_si = d_ExpVbsNVtm_dV_b_si = d_ExpVbsNVtm_dV_di_si = d_ExpVbsNVtm_dV_gi_si = d_ExpVbsNVtm_dV_e_si = d_ExpVbsNVtm_dTemp_t_GND = d_ExpVbsNVtm_dV_p_GND = d_ExpVbsNVtm_dV_b_GND =  0.0;
ExpVbsNVtm = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVbsNVtm_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_ExpVbsNVtm_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_ExpVbsNVtm_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_ExpVbsNVtm_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_ExpVbsNVtm_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_ExpVbsNVtm_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_ExpVbsNVtm_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_ExpVbsNVtm_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
ExpVbsNVtm = value_exp_0;
}
}
}

d_NVtm1_dV_b_GND = d_Vtm_dV_b_GND*pParam_B4SOIndioded;
d_NVtm1_dV_p_GND = d_Vtm_dV_p_GND*pParam_B4SOIndioded;
d_NVtm1_dTemp_t_GND = d_Vtm_dTemp_t_GND*pParam_B4SOIndioded;
NVtm1 = (Vtm*pParam_B4SOIndioded);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (-vdbd*d_NVtm1_dV_b_GND/NVtm1/NVtm1);
d_T0_dV_p_GND = (-vdbd*d_NVtm1_dV_p_GND/NVtm1/NVtm1);
d_T0_dTemp_t_GND = (-vdbd*d_NVtm1_dTemp_t_GND/NVtm1/NVtm1);
d_T0_dV_db_di = (d_vdbd_dV_db_di/NVtm1);
T0 = (vdbd/NVtm1);
if ((T0>100.0))
{

d_ExpVbdNVtm_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_ExpVbdNVtm_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_ExpVbdNVtm_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_ExpVbdNVtm_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_ExpVbdNVtm_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_ExpVbdNVtm_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_ExpVbdNVtm_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_ExpVbdNVtm_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_ExpVbdNVtm_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
ExpVbdNVtm = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_ExpVbdNVtm_dV_db_di = d_ExpVbdNVtm_dV_sb_si = d_ExpVbdNVtm_dV_b_si = d_ExpVbdNVtm_dV_di_si = d_ExpVbdNVtm_dV_gi_si = d_ExpVbdNVtm_dV_e_si = d_ExpVbdNVtm_dTemp_t_GND = d_ExpVbdNVtm_dV_p_GND = d_ExpVbdNVtm_dV_b_GND =  0.0;
ExpVbdNVtm = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVbdNVtm_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_ExpVbdNVtm_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_ExpVbdNVtm_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_ExpVbdNVtm_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_ExpVbdNVtm_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_ExpVbdNVtm_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_ExpVbdNVtm_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_ExpVbdNVtm_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_ExpVbdNVtm_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
ExpVbdNVtm = value_exp_0;
}
}
}
if ((jdifs==0))
{

d_Ibs1_dV_db_di = d_Ibs1_dV_sb_si = d_Ibs1_dV_b_si = d_Ibs1_dV_di_si = d_Ibs1_dV_gi_si = d_Ibs1_dV_e_si = d_Ibs1_dTemp_t_GND = d_Ibs1_dV_p_GND = d_Ibs1_dV_b_GND =  0.0;
Ibs1 = 0;
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = WsTsi*d_jdifs_dV_b_GND;
d_T0_dV_p_GND = WsTsi*d_jdifs_dV_p_GND;
d_T0_dTemp_t_GND = WsTsi*d_jdifs_dTemp_t_GND;
T0 = (WsTsi*jdifs);

d_Ibs1_dV_b_GND = (T0*d_ExpVbsNVtm_dV_b_GND+d_T0_dV_b_GND*(ExpVbsNVtm-1));
d_Ibs1_dV_p_GND = (T0*d_ExpVbsNVtm_dV_p_GND+d_T0_dV_p_GND*(ExpVbsNVtm-1));
d_Ibs1_dTemp_t_GND = (T0*d_ExpVbsNVtm_dTemp_t_GND+d_T0_dTemp_t_GND*(ExpVbsNVtm-1));
d_Ibs1_dV_e_si = (T0*d_ExpVbsNVtm_dV_e_si+d_T0_dV_e_si*(ExpVbsNVtm-1));
d_Ibs1_dV_gi_si = (T0*d_ExpVbsNVtm_dV_gi_si+d_T0_dV_gi_si*(ExpVbsNVtm-1));
d_Ibs1_dV_di_si = (T0*d_ExpVbsNVtm_dV_di_si+d_T0_dV_di_si*(ExpVbsNVtm-1));
d_Ibs1_dV_b_si = (T0*d_ExpVbsNVtm_dV_b_si+d_T0_dV_b_si*(ExpVbsNVtm-1));
d_Ibs1_dV_sb_si = (T0*d_ExpVbsNVtm_dV_sb_si+d_T0_dV_sb_si*(ExpVbsNVtm-1));
d_Ibs1_dV_db_di = d_T0_dV_db_di*(ExpVbsNVtm-1);
Ibs1 = (T0*(ExpVbsNVtm-1));
}
if ((jdifd==0))
{

d_Ibd1_dV_db_di = d_Ibd1_dV_sb_si = d_Ibd1_dV_b_si = d_Ibd1_dV_di_si = d_Ibd1_dV_gi_si = d_Ibd1_dV_e_si = d_Ibd1_dTemp_t_GND = d_Ibd1_dV_p_GND = d_Ibd1_dV_b_GND =  0.0;
Ibd1 = 0;
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = WdTsi*d_jdifd_dV_b_GND;
d_T0_dV_p_GND = WdTsi*d_jdifd_dV_p_GND;
d_T0_dTemp_t_GND = WdTsi*d_jdifd_dTemp_t_GND;
T0 = (WdTsi*jdifd);

d_Ibd1_dV_b_GND = (T0*d_ExpVbdNVtm_dV_b_GND+d_T0_dV_b_GND*(ExpVbdNVtm-1));
d_Ibd1_dV_p_GND = (T0*d_ExpVbdNVtm_dV_p_GND+d_T0_dV_p_GND*(ExpVbdNVtm-1));
d_Ibd1_dTemp_t_GND = (T0*d_ExpVbdNVtm_dTemp_t_GND+d_T0_dTemp_t_GND*(ExpVbdNVtm-1));
d_Ibd1_dV_e_si = (T0*d_ExpVbdNVtm_dV_e_si+d_T0_dV_e_si*(ExpVbdNVtm-1));
d_Ibd1_dV_gi_si = (T0*d_ExpVbdNVtm_dV_gi_si+d_T0_dV_gi_si*(ExpVbdNVtm-1));
d_Ibd1_dV_di_si = (T0*d_ExpVbdNVtm_dV_di_si+d_T0_dV_di_si*(ExpVbdNVtm-1));
d_Ibd1_dV_b_si = (T0*d_ExpVbdNVtm_dV_b_si+d_T0_dV_b_si*(ExpVbdNVtm-1));
d_Ibd1_dV_sb_si = (T0*d_ExpVbdNVtm_dV_sb_si+d_T0_dV_sb_si*(ExpVbdNVtm-1));
d_Ibd1_dV_db_di = (T0*d_ExpVbdNVtm_dV_db_di+d_T0_dV_db_di*(ExpVbdNVtm-1));
Ibd1 = (T0*(ExpVbdNVtm-1));
}
if ((jrecs==0))
{

d_Ibs2_dV_b_si = d_Ibs2_dV_e_si = d_Ibs2_dV_di_si = d_Ibs2_dV_gi_si = d_Ibs2_dTemp_t_GND = d_Ibs2_dV_p_GND = d_Ibs2_dV_b_GND = d_Ibs2_dV_db_di = d_Ibs2_dV_sb_si =  0.0;
Ibs2 = 0;
}
else
{

d_NVtmf_dV_b_GND = (Vtm00*pParam_B4SOInrecf0)*pParam_B4SOIntrecf*d_TempRatio_dV_b_GND;
d_NVtmf_dV_p_GND = (Vtm00*pParam_B4SOInrecf0)*pParam_B4SOIntrecf*d_TempRatio_dV_p_GND;
d_NVtmf_dTemp_t_GND = (Vtm00*pParam_B4SOInrecf0)*pParam_B4SOIntrecf*d_TempRatio_dTemp_t_GND;
NVtmf = ((Vtm00*pParam_B4SOInrecf0)*(1+(pParam_B4SOIntrecf*(TempRatio-1))));

d_NVtmr_dV_b_GND = (Vtm00*pParam_B4SOInrecr0)*pParam_B4SOIntrecr*d_TempRatio_dV_b_GND;
d_NVtmr_dV_p_GND = (Vtm00*pParam_B4SOInrecr0)*pParam_B4SOIntrecr*d_TempRatio_dV_p_GND;
d_NVtmr_dTemp_t_GND = (Vtm00*pParam_B4SOInrecr0)*pParam_B4SOIntrecr*d_TempRatio_dTemp_t_GND;
NVtmr = ((Vtm00*pParam_B4SOInrecr0)*(1+(pParam_B4SOIntrecr*(TempRatio-1))));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (-vsbs*d_NVtmf_dV_b_GND/NVtmf/NVtmf);
d_T0_dV_p_GND = (-vsbs*d_NVtmf_dV_p_GND/NVtmf/NVtmf);
d_T0_dTemp_t_GND = (-vsbs*d_NVtmf_dTemp_t_GND/NVtmf/NVtmf);
d_T0_dV_sb_si = (d_vsbs_dV_sb_si/NVtmf);
T0 = (vsbs/NVtmf);
if ((T0>100.0))
{

d_T10_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T10_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T10_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T10_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T10_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T10_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T10_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T10_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T10_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T10 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si = d_T10_dTemp_t_GND = d_T10_dV_p_GND = d_T10_dV_b_GND =  0.0;
T10 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T10_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T10_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T10_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T10_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T10_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T10_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T10_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T10_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T10_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T10 = value_exp_0;
}
}
}
if (((pParam_B4SOIvrec0-vsbs)<1e-3))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 1e3;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di =  0.0;
d_T0_dV_gi_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_b_si;
d_T0_dV_b_GND = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_b_GND+(-(-vsbs)*d_NVtmr_dV_b_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0*T1);
d_T0_dV_p_GND = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_p_GND+(-(-vsbs)*d_NVtmr_dV_p_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0*T1);
d_T0_dTemp_t_GND = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dTemp_t_GND+(-(-vsbs)*d_NVtmr_dTemp_t_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0*T1);
d_T0_dV_sb_si = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_sb_si+((-d_vsbs_dV_sb_si)/NVtmr)*pParam_B4SOIvrec0*T1);
T0 = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*T1);
if ((T0>100.0))
{

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T11_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T11_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T11_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T11_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T11_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T11_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T11_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T11_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T11_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T11_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T11_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T11_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T11_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T11_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T11_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T11_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T11 = value_exp_0;
}
}
}

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (-d_T11_dV_b_GND);
d_T11_dV_p_GND = (-d_T11_dV_p_GND);
d_T11_dTemp_t_GND = (-d_T11_dTemp_t_GND);
d_T11_dV_gi_si = (-d_T11_dV_gi_si);
d_T11_dV_di_si = (-d_T11_dV_di_si);
d_T11_dV_e_si = (-d_T11_dV_e_si);
d_T11_dV_b_si = (-d_T11_dV_b_si);
d_T11_dV_sb_si = (-d_T11_dV_sb_si);
d_T11_dV_db_di = (-d_T11_dV_db_di);
T11 = (-T11);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
d_T1_dV_sb_si = (-1*(-d_vsbs_dV_sb_si)/(pParam_B4SOIvrec0-vsbs)/(pParam_B4SOIvrec0-vsbs));
T1 = (1/(pParam_B4SOIvrec0-vsbs));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di =  0.0;
d_T0_dV_gi_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_b_si;
d_T0_dV_b_GND = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_b_GND+(-(-vsbs)*d_NVtmr_dV_b_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0*T1);
d_T0_dV_p_GND = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_p_GND+(-(-vsbs)*d_NVtmr_dV_p_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0*T1);
d_T0_dTemp_t_GND = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dTemp_t_GND+(-(-vsbs)*d_NVtmr_dTemp_t_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0*T1);
d_T0_dV_sb_si = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*d_T1_dV_sb_si+((-d_vsbs_dV_sb_si)/NVtmr)*pParam_B4SOIvrec0*T1);
T0 = ((((-vsbs)/NVtmr)*pParam_B4SOIvrec0)*T1);
if ((T0>100.0))
{

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T11_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T11_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T11_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T11_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T11_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T11_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T11_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T11_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T11_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T11_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T11_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T11_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T11_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T11_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T11_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T11_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T11 = value_exp_0;
}
}
}

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (-d_T11_dV_b_GND);
d_T11_dV_p_GND = (-d_T11_dV_p_GND);
d_T11_dTemp_t_GND = (-d_T11_dTemp_t_GND);
d_T11_dV_gi_si = (-d_T11_dV_gi_si);
d_T11_dV_di_si = (-d_T11_dV_di_si);
d_T11_dV_e_si = (-d_T11_dV_e_si);
d_T11_dV_b_si = (-d_T11_dV_b_si);
d_T11_dV_sb_si = (-d_T11_dV_sb_si);
d_T11_dV_db_di = (-d_T11_dV_db_di);
T11 = (-T11);
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = WsTsi*d_jrecs_dV_b_GND;
d_T3_dV_p_GND = WsTsi*d_jrecs_dV_p_GND;
d_T3_dTemp_t_GND = WsTsi*d_jrecs_dTemp_t_GND;
T3 = (WsTsi*jrecs);

d_Ibs2_dV_sb_si = (T3*(d_T10_dV_sb_si+d_T11_dV_sb_si)+d_T3_dV_sb_si*(T10+T11));
d_Ibs2_dV_db_di = (T3*(d_T10_dV_db_di+d_T11_dV_db_di)+d_T3_dV_db_di*(T10+T11));
d_Ibs2_dV_b_GND = (T3*(d_T10_dV_b_GND+d_T11_dV_b_GND)+d_T3_dV_b_GND*(T10+T11));
d_Ibs2_dV_p_GND = (T3*(d_T10_dV_p_GND+d_T11_dV_p_GND)+d_T3_dV_p_GND*(T10+T11));
d_Ibs2_dTemp_t_GND = (T3*(d_T10_dTemp_t_GND+d_T11_dTemp_t_GND)+d_T3_dTemp_t_GND*(T10+T11));
d_Ibs2_dV_gi_si = (T3*(d_T10_dV_gi_si+d_T11_dV_gi_si)+d_T3_dV_gi_si*(T10+T11));
d_Ibs2_dV_di_si = (T3*(d_T10_dV_di_si+d_T11_dV_di_si)+d_T3_dV_di_si*(T10+T11));
d_Ibs2_dV_e_si = (T3*(d_T10_dV_e_si+d_T11_dV_e_si)+d_T3_dV_e_si*(T10+T11));
d_Ibs2_dV_b_si = (T3*(d_T10_dV_b_si+d_T11_dV_b_si)+d_T3_dV_b_si*(T10+T11));
Ibs2 = (T3*(T10+T11));
}
if ((jrecd==0))
{

d_Ibd2_dV_b_si = d_Ibd2_dV_e_si = d_Ibd2_dV_di_si = d_Ibd2_dV_gi_si = d_Ibd2_dTemp_t_GND = d_Ibd2_dV_p_GND = d_Ibd2_dV_b_GND = d_Ibd2_dV_db_di = d_Ibd2_dV_sb_si =  0.0;
Ibd2 = 0;
}
else
{

d_NVtmf_dV_b_GND = (Vtm00*pParam_B4SOInrecf0d)*pParam_B4SOIntrecf*d_TempRatio_dV_b_GND;
d_NVtmf_dV_p_GND = (Vtm00*pParam_B4SOInrecf0d)*pParam_B4SOIntrecf*d_TempRatio_dV_p_GND;
d_NVtmf_dTemp_t_GND = (Vtm00*pParam_B4SOInrecf0d)*pParam_B4SOIntrecf*d_TempRatio_dTemp_t_GND;
NVtmf = ((Vtm00*pParam_B4SOInrecf0d)*(1+(pParam_B4SOIntrecf*(TempRatio-1))));

d_NVtmr_dV_b_GND = (Vtm00*pParam_B4SOInrecr0d)*pParam_B4SOIntrecr*d_TempRatio_dV_b_GND;
d_NVtmr_dV_p_GND = (Vtm00*pParam_B4SOInrecr0d)*pParam_B4SOIntrecr*d_TempRatio_dV_p_GND;
d_NVtmr_dTemp_t_GND = (Vtm00*pParam_B4SOInrecr0d)*pParam_B4SOIntrecr*d_TempRatio_dTemp_t_GND;
NVtmr = ((Vtm00*pParam_B4SOInrecr0d)*(1+(pParam_B4SOIntrecr*(TempRatio-1))));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (-vdbd*d_NVtmf_dV_b_GND/NVtmf/NVtmf);
d_T0_dV_p_GND = (-vdbd*d_NVtmf_dV_p_GND/NVtmf/NVtmf);
d_T0_dTemp_t_GND = (-vdbd*d_NVtmf_dTemp_t_GND/NVtmf/NVtmf);
d_T0_dV_db_di = (d_vdbd_dV_db_di/NVtmf);
T0 = (vdbd/NVtmf);
if ((T0>100.0))
{

d_T10_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T10_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T10_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T10_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T10_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T10_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T10_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T10_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T10_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T10 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_e_si = d_T10_dV_di_si = d_T10_dV_gi_si = d_T10_dTemp_t_GND = d_T10_dV_p_GND = d_T10_dV_b_GND =  0.0;
T10 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T10_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T10_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T10_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T10_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T10_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T10_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T10_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T10_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T10_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T10 = value_exp_0;
}
}
}
if (((pParam_B4SOIvrec0d-vdbd)<1e-3))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 1e3;

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_gi_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_b_si;
d_T0_dV_sb_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_sb_si;
d_T0_dV_b_GND = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_b_GND+(-(-vdbd)*d_NVtmr_dV_b_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0d*T1);
d_T0_dV_p_GND = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_p_GND+(-(-vdbd)*d_NVtmr_dV_p_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0d*T1);
d_T0_dTemp_t_GND = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dTemp_t_GND+(-(-vdbd)*d_NVtmr_dTemp_t_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0d*T1);
d_T0_dV_db_di = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_db_di+((-d_vdbd_dV_db_di)/NVtmr)*pParam_B4SOIvrec0d*T1);
T0 = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*T1);
if ((T0>100.0))
{

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T11_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T11_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T11_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T11_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T11_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T11_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T11_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T11_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T11_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T11_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T11_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T11_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T11_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T11_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T11_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T11_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T11 = value_exp_0;
}
}
}

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (-d_T11_dV_b_GND);
d_T11_dV_p_GND = (-d_T11_dV_p_GND);
d_T11_dTemp_t_GND = (-d_T11_dTemp_t_GND);
d_T11_dV_gi_si = (-d_T11_dV_gi_si);
d_T11_dV_di_si = (-d_T11_dV_di_si);
d_T11_dV_e_si = (-d_T11_dV_e_si);
d_T11_dV_b_si = (-d_T11_dV_b_si);
d_T11_dV_sb_si = (-d_T11_dV_sb_si);
d_T11_dV_db_di = (-d_T11_dV_db_di);
T11 = (-T11);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
d_T1_dV_db_di = (-1*(-d_vdbd_dV_db_di)/(pParam_B4SOIvrec0d-vdbd)/(pParam_B4SOIvrec0d-vdbd));
T1 = (1/(pParam_B4SOIvrec0d-vdbd));

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_gi_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_b_si;
d_T0_dV_sb_si = (((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_sb_si;
d_T0_dV_b_GND = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_b_GND+(-(-vdbd)*d_NVtmr_dV_b_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0d*T1);
d_T0_dV_p_GND = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_p_GND+(-(-vdbd)*d_NVtmr_dV_p_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0d*T1);
d_T0_dTemp_t_GND = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dTemp_t_GND+(-(-vdbd)*d_NVtmr_dTemp_t_GND/NVtmr/NVtmr)*pParam_B4SOIvrec0d*T1);
d_T0_dV_db_di = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*d_T1_dV_db_di+((-d_vdbd_dV_db_di)/NVtmr)*pParam_B4SOIvrec0d*T1);
T0 = ((((-vdbd)/NVtmr)*pParam_B4SOIvrec0d)*T1);
if ((T0>100.0))
{

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T11_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T11_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T11_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T11_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T11_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T11_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T11_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T11_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T11_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T11_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T11_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T11_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T11_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T11_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T11_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T11_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T11 = value_exp_0;
}
}
}

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = (-d_T11_dV_b_GND);
d_T11_dV_p_GND = (-d_T11_dV_p_GND);
d_T11_dTemp_t_GND = (-d_T11_dTemp_t_GND);
d_T11_dV_gi_si = (-d_T11_dV_gi_si);
d_T11_dV_di_si = (-d_T11_dV_di_si);
d_T11_dV_e_si = (-d_T11_dV_e_si);
d_T11_dV_b_si = (-d_T11_dV_b_si);
d_T11_dV_sb_si = (-d_T11_dV_sb_si);
d_T11_dV_db_di = (-d_T11_dV_db_di);
T11 = (-T11);
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = WdTsi*d_jrecd_dV_b_GND;
d_T3_dV_p_GND = WdTsi*d_jrecd_dV_p_GND;
d_T3_dTemp_t_GND = WdTsi*d_jrecd_dTemp_t_GND;
T3 = (WdTsi*jrecd);

d_Ibd2_dV_sb_si = (T3*(d_T10_dV_sb_si+d_T11_dV_sb_si)+d_T3_dV_sb_si*(T10+T11));
d_Ibd2_dV_db_di = (T3*(d_T10_dV_db_di+d_T11_dV_db_di)+d_T3_dV_db_di*(T10+T11));
d_Ibd2_dV_b_GND = (T3*(d_T10_dV_b_GND+d_T11_dV_b_GND)+d_T3_dV_b_GND*(T10+T11));
d_Ibd2_dV_p_GND = (T3*(d_T10_dV_p_GND+d_T11_dV_p_GND)+d_T3_dV_p_GND*(T10+T11));
d_Ibd2_dTemp_t_GND = (T3*(d_T10_dTemp_t_GND+d_T11_dTemp_t_GND)+d_T3_dTemp_t_GND*(T10+T11));
d_Ibd2_dV_gi_si = (T3*(d_T10_dV_gi_si+d_T11_dV_gi_si)+d_T3_dV_gi_si*(T10+T11));
d_Ibd2_dV_di_si = (T3*(d_T10_dV_di_si+d_T11_dV_di_si)+d_T3_dV_di_si*(T10+T11));
d_Ibd2_dV_e_si = (T3*(d_T10_dV_e_si+d_T11_dV_e_si)+d_T3_dV_e_si*(T10+T11));
d_Ibd2_dV_b_si = (T3*(d_T10_dV_b_si+d_T11_dV_b_si)+d_T3_dV_b_si*(T10+T11));
Ibd2 = (T3*(T10+T11));
}
WTsi = ((pParam_B4SOIweff/B4SOInseg)*B4SOItsi);
if (((jbjts==0.0)&&(jbjtd==0.0)))
{

d_Ibs3_dV_db_di = d_Ibs3_dV_sb_si = d_Ibs3_dV_b_si = d_Ibs3_dV_e_si = d_Ibs3_dV_di_si = d_Ibs3_dV_gi_si = d_Ibs3_dTemp_t_GND = d_Ibs3_dV_p_GND = d_Ibs3_dV_b_GND =  0.0;
Ibs3 = 0.0;

d_Ibd3_dV_db_di = d_Ibd3_dV_sb_si = d_Ibd3_dV_b_si = d_Ibd3_dV_e_si = d_Ibd3_dV_di_si = d_Ibd3_dV_gi_si = d_Ibd3_dTemp_t_GND = d_Ibd3_dV_p_GND = d_Ibd3_dV_b_GND =  0.0;
Ibd3 = 0.0;

d_Ibsdif_dTemp_t_GND = d_Ibsdif_dV_p_GND = d_Ibsdif_dV_b_GND = d_Ibsdif_dV_sb_si = d_Ibsdif_dV_b_si = d_Ibsdif_dV_di_si = d_Ibsdif_dV_gi_si = d_Ibsdif_dV_e_si =  0.0;
Ibsdif = 0;

d_Ibddif_dTemp_t_GND = d_Ibddif_dV_p_GND = d_Ibddif_dV_b_GND = d_Ibddif_dV_db_di = d_Ibddif_dV_sb_si = d_Ibddif_dV_b_si = d_Ibddif_dV_di_si = d_Ibddif_dV_gi_si = d_Ibddif_dV_e_si =  0.0;
Ibddif = 0;

d_Ic_dV_db_di = d_Ic_dV_sb_si = d_Ic_dV_b_si = d_Ic_dV_di_si = d_Ic_dV_gi_si = d_Ic_dV_e_si = d_Ic_dTemp_t_GND = d_Ic_dV_p_GND = d_Ic_dV_b_GND =  0.0;
Ic = 0.0;
}
else
{

d_Ehlis_dV_e_si = Ahlis*d_ExpVbsNVtm_dV_e_si;
d_Ehlis_dV_gi_si = Ahlis*d_ExpVbsNVtm_dV_gi_si;
d_Ehlis_dV_di_si = Ahlis*d_ExpVbsNVtm_dV_di_si;
d_Ehlis_dV_b_si = Ahlis*d_ExpVbsNVtm_dV_b_si;
d_Ehlis_dV_sb_si = Ahlis*d_ExpVbsNVtm_dV_sb_si;
d_Ehlis_dV_b_GND = (Ahlis*d_ExpVbsNVtm_dV_b_GND+d_Ahlis_dV_b_GND*(ExpVbsNVtm-1));
d_Ehlis_dV_p_GND = (Ahlis*d_ExpVbsNVtm_dV_p_GND+d_Ahlis_dV_p_GND*(ExpVbsNVtm-1));
d_Ehlis_dTemp_t_GND = (Ahlis*d_ExpVbsNVtm_dTemp_t_GND+d_Ahlis_dTemp_t_GND*(ExpVbsNVtm-1));
Ehlis = (Ahlis*(ExpVbsNVtm-1));
if ((Ehlis<1e-5))
{

d_Ehlis_dTemp_t_GND = d_Ehlis_dV_p_GND = d_Ehlis_dV_b_GND = d_Ehlis_dV_sb_si = d_Ehlis_dV_b_si = d_Ehlis_dV_di_si = d_Ehlis_dV_gi_si = d_Ehlis_dV_e_si =  0.0;
Ehlis = 0;

d_EhlisFactor_dTemp_t_GND = d_EhlisFactor_dV_p_GND = d_EhlisFactor_dV_b_GND = d_EhlisFactor_dV_sb_si = d_EhlisFactor_dV_b_si = d_EhlisFactor_dV_di_si = d_EhlisFactor_dV_gi_si = d_EhlisFactor_dV_e_si =  0.0;
EhlisFactor = 1;
}
else
{
{
double value_sqrt_0 = sqrt((1+Ehlis));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_EhlisFactor_dV_e_si = (-(deriv_sqrt_0_d0*(d_Ehlis_dV_e_si))/value_sqrt_0/value_sqrt_0);
d_EhlisFactor_dV_gi_si = (-(deriv_sqrt_0_d0*(d_Ehlis_dV_gi_si))/value_sqrt_0/value_sqrt_0);
d_EhlisFactor_dV_di_si = (-(deriv_sqrt_0_d0*(d_Ehlis_dV_di_si))/value_sqrt_0/value_sqrt_0);
d_EhlisFactor_dV_b_si = (-(deriv_sqrt_0_d0*(d_Ehlis_dV_b_si))/value_sqrt_0/value_sqrt_0);
d_EhlisFactor_dV_sb_si = (-(deriv_sqrt_0_d0*(d_Ehlis_dV_sb_si))/value_sqrt_0/value_sqrt_0);
d_EhlisFactor_dV_b_GND = (-(deriv_sqrt_0_d0*(d_Ehlis_dV_b_GND))/value_sqrt_0/value_sqrt_0);
d_EhlisFactor_dV_p_GND = (-(deriv_sqrt_0_d0*(d_Ehlis_dV_p_GND))/value_sqrt_0/value_sqrt_0);
d_EhlisFactor_dTemp_t_GND = (-(deriv_sqrt_0_d0*(d_Ehlis_dTemp_t_GND))/value_sqrt_0/value_sqrt_0);
EhlisFactor = (1.0/value_sqrt_0);
}
}

d_Ehlid_dV_e_si = Ahlid*d_ExpVbdNVtm_dV_e_si;
d_Ehlid_dV_gi_si = Ahlid*d_ExpVbdNVtm_dV_gi_si;
d_Ehlid_dV_di_si = Ahlid*d_ExpVbdNVtm_dV_di_si;
d_Ehlid_dV_b_si = Ahlid*d_ExpVbdNVtm_dV_b_si;
d_Ehlid_dV_sb_si = Ahlid*d_ExpVbdNVtm_dV_sb_si;
d_Ehlid_dV_db_di = Ahlid*d_ExpVbdNVtm_dV_db_di;
d_Ehlid_dV_b_GND = (Ahlid*d_ExpVbdNVtm_dV_b_GND+d_Ahlid_dV_b_GND*(ExpVbdNVtm-1));
d_Ehlid_dV_p_GND = (Ahlid*d_ExpVbdNVtm_dV_p_GND+d_Ahlid_dV_p_GND*(ExpVbdNVtm-1));
d_Ehlid_dTemp_t_GND = (Ahlid*d_ExpVbdNVtm_dTemp_t_GND+d_Ahlid_dTemp_t_GND*(ExpVbdNVtm-1));
Ehlid = (Ahlid*(ExpVbdNVtm-1));
if ((Ehlid<1e-5))
{

d_Ehlid_dTemp_t_GND = d_Ehlid_dV_p_GND = d_Ehlid_dV_b_GND = d_Ehlid_dV_db_di = d_Ehlid_dV_sb_si = d_Ehlid_dV_b_si = d_Ehlid_dV_di_si = d_Ehlid_dV_gi_si = d_Ehlid_dV_e_si =  0.0;
Ehlid = 0;

d_EhlidFactor_dTemp_t_GND = d_EhlidFactor_dV_p_GND = d_EhlidFactor_dV_b_GND = d_EhlidFactor_dV_db_di = d_EhlidFactor_dV_sb_si = d_EhlidFactor_dV_b_si = d_EhlidFactor_dV_di_si = d_EhlidFactor_dV_gi_si = d_EhlidFactor_dV_e_si =  0.0;
EhlidFactor = 1;
}
else
{
{
double value_sqrt_0 = sqrt((1+Ehlid));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_EhlidFactor_dV_e_si = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_e_si))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dV_gi_si = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_gi_si))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dV_di_si = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_di_si))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dV_b_si = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_b_si))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dV_sb_si = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_sb_si))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dV_db_di = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_db_di))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dV_b_GND = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_b_GND))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dV_p_GND = (-(deriv_sqrt_0_d0*(d_Ehlid_dV_p_GND))/value_sqrt_0/value_sqrt_0);
d_EhlidFactor_dTemp_t_GND = (-(deriv_sqrt_0_d0*(d_Ehlid_dTemp_t_GND))/value_sqrt_0/value_sqrt_0);
EhlidFactor = (1.0/value_sqrt_0);
}
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1-pParam_B4SOIarfabjt);

d_Ien_dV_b_GND = WTsi*d_jbjts_dV_b_GND*pParam_B4SOIlratio;
d_Ien_dV_p_GND = WTsi*d_jbjts_dV_p_GND*pParam_B4SOIlratio;
d_Ien_dTemp_t_GND = WTsi*d_jbjts_dTemp_t_GND*pParam_B4SOIlratio;
Ien = ((WTsi*jbjts)*pParam_B4SOIlratio);

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (T0*d_Ien_dV_b_GND+d_T0_dV_b_GND*Ien);
d_T1_dV_p_GND = (T0*d_Ien_dV_p_GND+d_T0_dV_p_GND*Ien);
d_T1_dTemp_t_GND = (T0*d_Ien_dTemp_t_GND+d_T0_dTemp_t_GND*Ien);
d_T1_dV_e_si = d_T0_dV_e_si*Ien;
d_T1_dV_gi_si = d_T0_dV_gi_si*Ien;
d_T1_dV_di_si = d_T0_dV_di_si*Ien;
d_T1_dV_b_si = d_T0_dV_b_si*Ien;
d_T1_dV_sb_si = d_T0_dV_sb_si*Ien;
d_T1_dV_db_di = d_T0_dV_db_di*Ien;
T1 = (T0*Ien);

d_Ibs3_dV_b_GND = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dV_b_GND+(T1*d_ExpVbsNVtm_dV_b_GND+d_T1_dV_b_GND*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dV_p_GND = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dV_p_GND+(T1*d_ExpVbsNVtm_dV_p_GND+d_T1_dV_p_GND*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dTemp_t_GND = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dTemp_t_GND+(T1*d_ExpVbsNVtm_dTemp_t_GND+d_T1_dTemp_t_GND*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dV_gi_si = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dV_gi_si+(T1*d_ExpVbsNVtm_dV_gi_si+d_T1_dV_gi_si*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dV_di_si = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dV_di_si+(T1*d_ExpVbsNVtm_dV_di_si+d_T1_dV_di_si*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dV_e_si = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dV_e_si+(T1*d_ExpVbsNVtm_dV_e_si+d_T1_dV_e_si*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dV_b_si = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dV_b_si+(T1*d_ExpVbsNVtm_dV_b_si+d_T1_dV_b_si*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dV_sb_si = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dV_sb_si+(T1*d_ExpVbsNVtm_dV_sb_si+d_T1_dV_sb_si*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibs3_dV_db_di = d_T1_dV_db_di*(ExpVbsNVtm-1)*EhlisFactor;
Ibs3 = ((T1*(ExpVbsNVtm-1))*EhlisFactor);

d_Ien_dV_b_GND = WTsi*d_jbjtd_dV_b_GND*pParam_B4SOIlratio;
d_Ien_dV_p_GND = WTsi*d_jbjtd_dV_p_GND*pParam_B4SOIlratio;
d_Ien_dTemp_t_GND = WTsi*d_jbjtd_dTemp_t_GND*pParam_B4SOIlratio;
Ien = ((WTsi*jbjtd)*pParam_B4SOIlratio);

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (T0*d_Ien_dV_b_GND+d_T0_dV_b_GND*Ien);
d_T1_dV_p_GND = (T0*d_Ien_dV_p_GND+d_T0_dV_p_GND*Ien);
d_T1_dTemp_t_GND = (T0*d_Ien_dTemp_t_GND+d_T0_dTemp_t_GND*Ien);
d_T1_dV_e_si = d_T0_dV_e_si*Ien;
d_T1_dV_gi_si = d_T0_dV_gi_si*Ien;
d_T1_dV_di_si = d_T0_dV_di_si*Ien;
d_T1_dV_b_si = d_T0_dV_b_si*Ien;
d_T1_dV_sb_si = d_T0_dV_sb_si*Ien;
d_T1_dV_db_di = d_T0_dV_db_di*Ien;
T1 = (T0*Ien);

d_Ibd3_dV_b_GND = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_b_GND+(T1*d_ExpVbdNVtm_dV_b_GND+d_T1_dV_b_GND*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dV_p_GND = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_p_GND+(T1*d_ExpVbdNVtm_dV_p_GND+d_T1_dV_p_GND*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dTemp_t_GND = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dTemp_t_GND+(T1*d_ExpVbdNVtm_dTemp_t_GND+d_T1_dTemp_t_GND*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dV_gi_si = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_gi_si+(T1*d_ExpVbdNVtm_dV_gi_si+d_T1_dV_gi_si*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dV_di_si = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_di_si+(T1*d_ExpVbdNVtm_dV_di_si+d_T1_dV_di_si*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dV_e_si = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_e_si+(T1*d_ExpVbdNVtm_dV_e_si+d_T1_dV_e_si*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dV_b_si = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_b_si+(T1*d_ExpVbdNVtm_dV_b_si+d_T1_dV_b_si*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dV_sb_si = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_sb_si+(T1*d_ExpVbdNVtm_dV_sb_si+d_T1_dV_sb_si*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibd3_dV_db_di = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dV_db_di+(T1*d_ExpVbdNVtm_dV_db_di+d_T1_dV_db_di*(ExpVbdNVtm-1))*EhlidFactor);
Ibd3 = ((T1*(ExpVbdNVtm-1))*EhlidFactor);

d_Iendif_dV_b_GND = WTsi*d_jbjts_dV_b_GND*pParam_B4SOIlratiodif;
d_Iendif_dV_p_GND = WTsi*d_jbjts_dV_p_GND*pParam_B4SOIlratiodif;
d_Iendif_dTemp_t_GND = WTsi*d_jbjts_dTemp_t_GND*pParam_B4SOIlratiodif;
Iendif = ((WTsi*jbjts)*pParam_B4SOIlratiodif);

d_Ibsdif_dV_e_si = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dV_e_si+Iendif*d_ExpVbsNVtm_dV_e_si*EhlisFactor);
d_Ibsdif_dV_gi_si = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dV_gi_si+Iendif*d_ExpVbsNVtm_dV_gi_si*EhlisFactor);
d_Ibsdif_dV_di_si = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dV_di_si+Iendif*d_ExpVbsNVtm_dV_di_si*EhlisFactor);
d_Ibsdif_dV_b_si = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dV_b_si+Iendif*d_ExpVbsNVtm_dV_b_si*EhlisFactor);
d_Ibsdif_dV_sb_si = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dV_sb_si+Iendif*d_ExpVbsNVtm_dV_sb_si*EhlisFactor);
d_Ibsdif_dV_b_GND = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dV_b_GND+(Iendif*d_ExpVbsNVtm_dV_b_GND+d_Iendif_dV_b_GND*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibsdif_dV_p_GND = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dV_p_GND+(Iendif*d_ExpVbsNVtm_dV_p_GND+d_Iendif_dV_p_GND*(ExpVbsNVtm-1))*EhlisFactor);
d_Ibsdif_dTemp_t_GND = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dTemp_t_GND+(Iendif*d_ExpVbsNVtm_dTemp_t_GND+d_Iendif_dTemp_t_GND*(ExpVbsNVtm-1))*EhlisFactor);
Ibsdif = ((Iendif*(ExpVbsNVtm-1))*EhlisFactor);

d_Iendif_dV_b_GND = WTsi*d_jbjtd_dV_b_GND*pParam_B4SOIlratiodif;
d_Iendif_dV_p_GND = WTsi*d_jbjtd_dV_p_GND*pParam_B4SOIlratiodif;
d_Iendif_dTemp_t_GND = WTsi*d_jbjtd_dTemp_t_GND*pParam_B4SOIlratiodif;
Iendif = ((WTsi*jbjtd)*pParam_B4SOIlratiodif);

d_Ibddif_dV_e_si = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_e_si+Iendif*d_ExpVbdNVtm_dV_e_si*EhlidFactor);
d_Ibddif_dV_gi_si = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_gi_si+Iendif*d_ExpVbdNVtm_dV_gi_si*EhlidFactor);
d_Ibddif_dV_di_si = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_di_si+Iendif*d_ExpVbdNVtm_dV_di_si*EhlidFactor);
d_Ibddif_dV_b_si = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_b_si+Iendif*d_ExpVbdNVtm_dV_b_si*EhlidFactor);
d_Ibddif_dV_sb_si = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_sb_si+Iendif*d_ExpVbdNVtm_dV_sb_si*EhlidFactor);
d_Ibddif_dV_db_di = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_db_di+Iendif*d_ExpVbdNVtm_dV_db_di*EhlidFactor);
d_Ibddif_dV_b_GND = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_b_GND+(Iendif*d_ExpVbdNVtm_dV_b_GND+d_Iendif_dV_b_GND*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibddif_dV_p_GND = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dV_p_GND+(Iendif*d_ExpVbdNVtm_dV_p_GND+d_Iendif_dV_p_GND*(ExpVbdNVtm-1))*EhlidFactor);
d_Ibddif_dTemp_t_GND = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dTemp_t_GND+(Iendif*d_ExpVbdNVtm_dTemp_t_GND+d_Iendif_dTemp_t_GND*(ExpVbdNVtm-1))*EhlidFactor);
Ibddif = ((Iendif*(ExpVbdNVtm-1))*EhlidFactor);
if ((B4SOIbjtoff==1))
{

d_Ic_dV_db_di = d_Ic_dV_sb_si = d_Ic_dV_b_si = d_Ic_dV_di_si = d_Ic_dV_gi_si = d_Ic_dV_e_si = d_Ic_dTemp_t_GND = d_Ic_dV_p_GND = d_Ic_dV_b_GND =  0.0;
Ic = 0.0;
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_db_di = (d_vdbd_dV_db_di/pParam_B4SOIvearly);
d_T0_dV_sb_si = (d_vsbs_dV_sb_si/pParam_B4SOIvearly);
T0 = (1+((vsbs+vdbd)/pParam_B4SOIvearly));

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_db_di = d_Ehlid_dV_db_di;
d_T1_dV_e_si = (d_Ehlis_dV_e_si+d_Ehlid_dV_e_si);
d_T1_dV_gi_si = (d_Ehlis_dV_gi_si+d_Ehlid_dV_gi_si);
d_T1_dV_di_si = (d_Ehlis_dV_di_si+d_Ehlid_dV_di_si);
d_T1_dV_b_si = (d_Ehlis_dV_b_si+d_Ehlid_dV_b_si);
d_T1_dV_sb_si = (d_Ehlis_dV_sb_si+d_Ehlid_dV_sb_si);
d_T1_dV_b_GND = (d_Ehlis_dV_b_GND+d_Ehlid_dV_b_GND);
d_T1_dV_p_GND = (d_Ehlis_dV_p_GND+d_Ehlid_dV_p_GND);
d_T1_dTemp_t_GND = (d_Ehlis_dTemp_t_GND+d_Ehlid_dTemp_t_GND);
T1 = (Ehlis+Ehlid);
{
double value_sqrt_0 = sqrt(((T0*T0)+(4*T1)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p =  0.0;
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)+4*d_T1_dV_b_GND)));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)+4*d_T1_dV_p_GND)));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)+4*d_T1_dTemp_t_GND)));
d_T3_dV_e_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)+4*d_T1_dV_e_si)));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)+4*d_T1_dV_gi_si)));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)+4*d_T1_dV_di_si)));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)+4*d_T1_dV_b_si)));
d_T3_dV_sb_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)+4*d_T1_dV_sb_si)));
d_T3_dV_db_di = (deriv_sqrt_0_d0*(((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)+4*d_T1_dV_db_di)));
T3 = value_sqrt_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p =  0.0;
d_T2_dV_b_GND = ((d_T0_dV_b_GND+d_T3_dV_b_GND)/2.0);
d_T2_dV_p_GND = ((d_T0_dV_p_GND+d_T3_dV_p_GND)/2.0);
d_T2_dTemp_t_GND = ((d_T0_dTemp_t_GND+d_T3_dTemp_t_GND)/2.0);
d_T2_dV_e_si = ((d_T0_dV_e_si+d_T3_dV_e_si)/2.0);
d_T2_dV_gi_si = ((d_T0_dV_gi_si+d_T3_dV_gi_si)/2.0);
d_T2_dV_di_si = ((d_T0_dV_di_si+d_T3_dV_di_si)/2.0);
d_T2_dV_b_si = ((d_T0_dV_b_si+d_T3_dV_b_si)/2.0);
d_T2_dV_sb_si = ((d_T0_dV_sb_si+d_T3_dV_sb_si)/2.0);
d_T2_dV_db_di = ((d_T0_dV_db_di+d_T3_dV_db_di)/2.0);
T2 = ((T0+T3)/2.0);
if ((T2<0.1))
{

d_E2ndFactor_dV_db_di = d_E2ndFactor_dV_sb_si = d_E2ndFactor_dV_b_si = d_E2ndFactor_dV_e_si = d_E2ndFactor_dV_di_si = d_E2ndFactor_dV_gi_si = d_E2ndFactor_dTemp_t_GND = d_E2ndFactor_dV_p_GND = d_E2ndFactor_dV_b_GND =  0.0;
E2ndFactor = 10.0;
}
else
{

d_E2ndFactor_dV_b_GND = (-d_T2_dV_b_GND/T2/T2);
d_E2ndFactor_dV_p_GND = (-d_T2_dV_p_GND/T2/T2);
d_E2ndFactor_dTemp_t_GND = (-d_T2_dTemp_t_GND/T2/T2);
d_E2ndFactor_dV_gi_si = (-d_T2_dV_gi_si/T2/T2);
d_E2ndFactor_dV_di_si = (-d_T2_dV_di_si/T2/T2);
d_E2ndFactor_dV_e_si = (-d_T2_dV_e_si/T2/T2);
d_E2ndFactor_dV_b_si = (-d_T2_dV_b_si/T2/T2);
d_E2ndFactor_dV_sb_si = (-d_T2_dV_sb_si/T2/T2);
d_E2ndFactor_dV_db_di = (-d_T2_dV_db_di/T2/T2);
E2ndFactor = (1.0/T2);
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIarfabjt*d_Ien_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIarfabjt*d_Ien_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIarfabjt*d_Ien_dTemp_t_GND;
T0 = (pParam_B4SOIarfabjt*Ien);

d_Ic_dV_b_GND = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_b_GND+(T0*(d_ExpVbsNVtm_dV_b_GND-d_ExpVbdNVtm_dV_b_GND)+d_T0_dV_b_GND*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dV_p_GND = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_p_GND+(T0*(d_ExpVbsNVtm_dV_p_GND-d_ExpVbdNVtm_dV_p_GND)+d_T0_dV_p_GND*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dTemp_t_GND = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dTemp_t_GND+(T0*(d_ExpVbsNVtm_dTemp_t_GND-d_ExpVbdNVtm_dTemp_t_GND)+d_T0_dTemp_t_GND*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dV_e_si = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_e_si+(T0*(d_ExpVbsNVtm_dV_e_si-d_ExpVbdNVtm_dV_e_si)+d_T0_dV_e_si*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dV_gi_si = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_gi_si+(T0*(d_ExpVbsNVtm_dV_gi_si-d_ExpVbdNVtm_dV_gi_si)+d_T0_dV_gi_si*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dV_di_si = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_di_si+(T0*(d_ExpVbsNVtm_dV_di_si-d_ExpVbdNVtm_dV_di_si)+d_T0_dV_di_si*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dV_b_si = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_b_si+(T0*(d_ExpVbsNVtm_dV_b_si-d_ExpVbdNVtm_dV_b_si)+d_T0_dV_b_si*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dV_sb_si = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_sb_si+(T0*(d_ExpVbsNVtm_dV_sb_si-d_ExpVbdNVtm_dV_sb_si)+d_T0_dV_sb_si*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
d_Ic_dV_db_di = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dV_db_di+(T0*(-d_ExpVbdNVtm_dV_db_di)+d_T0_dV_db_di*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
Ic = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
}
}
if (((jtuns==0)&&(jtund==0)))
{

d_Ibd4_dV_db_di = d_Ibd4_dV_sb_si = d_Ibd4_dV_b_si = d_Ibd4_dV_e_si = d_Ibd4_dV_di_si = d_Ibd4_dV_gi_si = d_Ibd4_dTemp_t_GND = d_Ibd4_dV_p_GND = d_Ibd4_dV_b_GND =  0.0;
Ibd4 = 0;

d_Ibs4_dV_db_di = d_Ibs4_dV_sb_si = d_Ibs4_dV_b_si = d_Ibs4_dV_e_si = d_Ibs4_dV_di_si = d_Ibs4_dV_gi_si = d_Ibs4_dTemp_t_GND = d_Ibs4_dV_p_GND = d_Ibs4_dV_b_GND =  0.0;
Ibs4 = 0;
}
else
{
NVtm2 = (Vtm00*pParam_B4SOIntun);
if (((pParam_B4SOIvtun0-vsbs)<1e-3))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 1e3;

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_b_GND;
d_T0_dV_p_GND = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_p_GND;
d_T0_dTemp_t_GND = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dTemp_t_GND;
d_T0_dV_gi_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_b_si;
d_T0_dV_db_di = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_db_di;
d_T0_dV_sb_si = ((((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_sb_si+((-d_vsbs_dV_sb_si)/NVtm2)*pParam_B4SOIvtun0*T1);
T0 = ((((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*T1);
if ((T0>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T1_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T1_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T1_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T1_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T1_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T1_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T1_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T1_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T1 = value_exp_0;
}
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = WsTsi*d_jtuns_dV_b_GND;
d_T3_dV_p_GND = WsTsi*d_jtuns_dV_p_GND;
d_T3_dTemp_t_GND = WsTsi*d_jtuns_dTemp_t_GND;
T3 = (WsTsi*jtuns);

d_Ibs4_dV_b_GND = (T3*(-d_T1_dV_b_GND)+d_T3_dV_b_GND*(1-T1));
d_Ibs4_dV_p_GND = (T3*(-d_T1_dV_p_GND)+d_T3_dV_p_GND*(1-T1));
d_Ibs4_dTemp_t_GND = (T3*(-d_T1_dTemp_t_GND)+d_T3_dTemp_t_GND*(1-T1));
d_Ibs4_dV_gi_si = (T3*(-d_T1_dV_gi_si)+d_T3_dV_gi_si*(1-T1));
d_Ibs4_dV_di_si = (T3*(-d_T1_dV_di_si)+d_T3_dV_di_si*(1-T1));
d_Ibs4_dV_e_si = (T3*(-d_T1_dV_e_si)+d_T3_dV_e_si*(1-T1));
d_Ibs4_dV_b_si = (T3*(-d_T1_dV_b_si)+d_T3_dV_b_si*(1-T1));
d_Ibs4_dV_sb_si = (T3*(-d_T1_dV_sb_si)+d_T3_dV_sb_si*(1-T1));
d_Ibs4_dV_db_di = (T3*(-d_T1_dV_db_di)+d_T3_dV_db_di*(1-T1));
Ibs4 = (T3*(1-T1));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
d_T1_dV_sb_si = (-1*(-d_vsbs_dV_sb_si)/(pParam_B4SOIvtun0-vsbs)/(pParam_B4SOIvtun0-vsbs));
T1 = (1/(pParam_B4SOIvtun0-vsbs));

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_b_GND;
d_T0_dV_p_GND = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_p_GND;
d_T0_dTemp_t_GND = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dTemp_t_GND;
d_T0_dV_gi_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_b_si;
d_T0_dV_db_di = (((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_db_di;
d_T0_dV_sb_si = ((((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*d_T1_dV_sb_si+((-d_vsbs_dV_sb_si)/NVtm2)*pParam_B4SOIvtun0*T1);
T0 = ((((-vsbs)/NVtm2)*pParam_B4SOIvtun0)*T1);
if ((T0>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T1_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T1_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T1_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T1_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T1_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T1_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T1_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T1_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T1 = value_exp_0;
}
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = WsTsi*d_jtuns_dV_b_GND;
d_T3_dV_p_GND = WsTsi*d_jtuns_dV_p_GND;
d_T3_dTemp_t_GND = WsTsi*d_jtuns_dTemp_t_GND;
T3 = (WsTsi*jtuns);

d_Ibs4_dV_b_GND = (T3*(-d_T1_dV_b_GND)+d_T3_dV_b_GND*(1-T1));
d_Ibs4_dV_p_GND = (T3*(-d_T1_dV_p_GND)+d_T3_dV_p_GND*(1-T1));
d_Ibs4_dTemp_t_GND = (T3*(-d_T1_dTemp_t_GND)+d_T3_dTemp_t_GND*(1-T1));
d_Ibs4_dV_gi_si = (T3*(-d_T1_dV_gi_si)+d_T3_dV_gi_si*(1-T1));
d_Ibs4_dV_di_si = (T3*(-d_T1_dV_di_si)+d_T3_dV_di_si*(1-T1));
d_Ibs4_dV_e_si = (T3*(-d_T1_dV_e_si)+d_T3_dV_e_si*(1-T1));
d_Ibs4_dV_b_si = (T3*(-d_T1_dV_b_si)+d_T3_dV_b_si*(1-T1));
d_Ibs4_dV_sb_si = (T3*(-d_T1_dV_sb_si)+d_T3_dV_sb_si*(1-T1));
d_Ibs4_dV_db_di = (T3*(-d_T1_dV_db_di)+d_T3_dV_db_di*(1-T1));
Ibs4 = (T3*(1-T1));
}
NVtm2 = (Vtm00*pParam_B4SOIntund);
if (((pParam_B4SOIvtun0d-vdbd)<1e-3))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 1e3;

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_b_GND;
d_T0_dV_p_GND = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_p_GND;
d_T0_dTemp_t_GND = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dTemp_t_GND;
d_T0_dV_gi_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_b_si;
d_T0_dV_sb_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_sb_si;
d_T0_dV_db_di = ((((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_db_di+((-d_vdbd_dV_db_di)/NVtm2)*pParam_B4SOIvtun0d*T1);
T0 = ((((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*T1);
if ((T0>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T1_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T1_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T1_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T1_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T1_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T1_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T1_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T1_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T1 = value_exp_0;
}
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = WdTsi*d_jtund_dV_b_GND;
d_T3_dV_p_GND = WdTsi*d_jtund_dV_p_GND;
d_T3_dTemp_t_GND = WdTsi*d_jtund_dTemp_t_GND;
T3 = (WdTsi*jtund);

d_Ibd4_dV_b_GND = (T3*(-d_T1_dV_b_GND)+d_T3_dV_b_GND*(1-T1));
d_Ibd4_dV_p_GND = (T3*(-d_T1_dV_p_GND)+d_T3_dV_p_GND*(1-T1));
d_Ibd4_dTemp_t_GND = (T3*(-d_T1_dTemp_t_GND)+d_T3_dTemp_t_GND*(1-T1));
d_Ibd4_dV_gi_si = (T3*(-d_T1_dV_gi_si)+d_T3_dV_gi_si*(1-T1));
d_Ibd4_dV_di_si = (T3*(-d_T1_dV_di_si)+d_T3_dV_di_si*(1-T1));
d_Ibd4_dV_e_si = (T3*(-d_T1_dV_e_si)+d_T3_dV_e_si*(1-T1));
d_Ibd4_dV_b_si = (T3*(-d_T1_dV_b_si)+d_T3_dV_b_si*(1-T1));
d_Ibd4_dV_sb_si = (T3*(-d_T1_dV_sb_si)+d_T3_dV_sb_si*(1-T1));
d_Ibd4_dV_db_di = (T3*(-d_T1_dV_db_di)+d_T3_dV_db_di*(1-T1));
Ibd4 = (T3*(1-T1));
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
d_T1_dV_db_di = (-1*(-d_vdbd_dV_db_di)/(pParam_B4SOIvtun0d-vdbd)/(pParam_B4SOIvtun0d-vdbd));
T1 = (1/(pParam_B4SOIvtun0d-vdbd));

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_b_GND;
d_T0_dV_p_GND = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_p_GND;
d_T0_dTemp_t_GND = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dTemp_t_GND;
d_T0_dV_gi_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_gi_si;
d_T0_dV_di_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_di_si;
d_T0_dV_e_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_e_si;
d_T0_dV_b_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_b_si;
d_T0_dV_sb_si = (((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_sb_si;
d_T0_dV_db_di = ((((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*d_T1_dV_db_di+((-d_vdbd_dV_db_di)/NVtm2)*pParam_B4SOIvtun0d*T1);
T0 = ((((-vdbd)/NVtm2)*pParam_B4SOIvtun0d)*T1);
if ((T0>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T1_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T1_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T1_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T1_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T1_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T1_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T1_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T1_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T1 = value_exp_0;
}
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = WdTsi*d_jtund_dV_b_GND;
d_T3_dV_p_GND = WdTsi*d_jtund_dV_p_GND;
d_T3_dTemp_t_GND = WdTsi*d_jtund_dTemp_t_GND;
T3 = (WdTsi*jtund);

d_Ibd4_dV_b_GND = (T3*(-d_T1_dV_b_GND)+d_T3_dV_b_GND*(1-T1));
d_Ibd4_dV_p_GND = (T3*(-d_T1_dV_p_GND)+d_T3_dV_p_GND*(1-T1));
d_Ibd4_dTemp_t_GND = (T3*(-d_T1_dTemp_t_GND)+d_T3_dTemp_t_GND*(1-T1));
d_Ibd4_dV_gi_si = (T3*(-d_T1_dV_gi_si)+d_T3_dV_gi_si*(1-T1));
d_Ibd4_dV_di_si = (T3*(-d_T1_dV_di_si)+d_T3_dV_di_si*(1-T1));
d_Ibd4_dV_e_si = (T3*(-d_T1_dV_e_si)+d_T3_dV_e_si*(1-T1));
d_Ibd4_dV_b_si = (T3*(-d_T1_dV_b_si)+d_T3_dV_b_si*(1-T1));
d_Ibd4_dV_sb_si = (T3*(-d_T1_dV_sb_si)+d_T3_dV_sb_si*(1-T1));
d_Ibd4_dV_db_di = (T3*(-d_T1_dV_db_di)+d_T3_dV_db_di*(1-T1));
Ibd4 = (T3*(1-T1));
}
}

d_Ibs_dV_b_GND = (((d_Ibs1_dV_b_GND+d_Ibs2_dV_b_GND)+d_Ibs3_dV_b_GND)+d_Ibs4_dV_b_GND);
d_Ibs_dV_p_GND = (((d_Ibs1_dV_p_GND+d_Ibs2_dV_p_GND)+d_Ibs3_dV_p_GND)+d_Ibs4_dV_p_GND);
d_Ibs_dTemp_t_GND = (((d_Ibs1_dTemp_t_GND+d_Ibs2_dTemp_t_GND)+d_Ibs3_dTemp_t_GND)+d_Ibs4_dTemp_t_GND);
d_Ibs_dV_e_si = (((d_Ibs1_dV_e_si+d_Ibs2_dV_e_si)+d_Ibs3_dV_e_si)+d_Ibs4_dV_e_si);
d_Ibs_dV_gi_si = (((d_Ibs1_dV_gi_si+d_Ibs2_dV_gi_si)+d_Ibs3_dV_gi_si)+d_Ibs4_dV_gi_si);
d_Ibs_dV_di_si = (((d_Ibs1_dV_di_si+d_Ibs2_dV_di_si)+d_Ibs3_dV_di_si)+d_Ibs4_dV_di_si);
d_Ibs_dV_b_si = (((d_Ibs1_dV_b_si+d_Ibs2_dV_b_si)+d_Ibs3_dV_b_si)+d_Ibs4_dV_b_si);
d_Ibs_dV_sb_si = (((d_Ibs1_dV_sb_si+d_Ibs2_dV_sb_si)+d_Ibs3_dV_sb_si)+d_Ibs4_dV_sb_si);
d_Ibs_dV_db_di = (((d_Ibs1_dV_db_di+d_Ibs2_dV_db_di)+d_Ibs3_dV_db_di)+d_Ibs4_dV_db_di);
Ibs = (((Ibs1+Ibs2)+Ibs3)+Ibs4);

d_Ibd_dV_b_GND = (((d_Ibd1_dV_b_GND+d_Ibd2_dV_b_GND)+d_Ibd3_dV_b_GND)+d_Ibd4_dV_b_GND);
d_Ibd_dV_p_GND = (((d_Ibd1_dV_p_GND+d_Ibd2_dV_p_GND)+d_Ibd3_dV_p_GND)+d_Ibd4_dV_p_GND);
d_Ibd_dTemp_t_GND = (((d_Ibd1_dTemp_t_GND+d_Ibd2_dTemp_t_GND)+d_Ibd3_dTemp_t_GND)+d_Ibd4_dTemp_t_GND);
d_Ibd_dV_e_si = (((d_Ibd1_dV_e_si+d_Ibd2_dV_e_si)+d_Ibd3_dV_e_si)+d_Ibd4_dV_e_si);
d_Ibd_dV_gi_si = (((d_Ibd1_dV_gi_si+d_Ibd2_dV_gi_si)+d_Ibd3_dV_gi_si)+d_Ibd4_dV_gi_si);
d_Ibd_dV_di_si = (((d_Ibd1_dV_di_si+d_Ibd2_dV_di_si)+d_Ibd3_dV_di_si)+d_Ibd4_dV_di_si);
d_Ibd_dV_b_si = (((d_Ibd1_dV_b_si+d_Ibd2_dV_b_si)+d_Ibd3_dV_b_si)+d_Ibd4_dV_b_si);
d_Ibd_dV_sb_si = (((d_Ibd1_dV_sb_si+d_Ibd2_dV_sb_si)+d_Ibd3_dV_sb_si)+d_Ibd4_dV_sb_si);
d_Ibd_dV_db_di = (((d_Ibd1_dV_db_di+d_Ibd2_dV_db_di)+d_Ibd3_dV_db_di)+d_Ibd4_dV_db_di);
Ibd = (((Ibd1+Ibd2)+Ibd3)+Ibd4);
}
else
{

d_Igidl_dV_b_si = d_Igidl_dV_e_si = d_Igidl_dV_di_si = d_Igidl_dV_gi_si = d_Igidl_dTemp_t_GND = d_Igidl_dV_p_GND = d_Igidl_dV_b_GND =  0.0;
Igidl = 0.0;

d_Igisl_dV_b_si = d_Igisl_dV_e_si = d_Igisl_dV_di_si = d_Igisl_dV_gi_si = d_Igisl_dTemp_t_GND = d_Igisl_dV_p_GND = d_Igisl_dV_b_GND =  0.0;
Igisl = 0.0;

d_Ibs_dV_db_di = d_Ibs_dV_sb_si = d_Ibs_dV_b_si = d_Ibs_dV_di_si = d_Ibs_dV_gi_si = d_Ibs_dV_e_si = d_Ibs_dTemp_t_GND = d_Ibs_dV_p_GND = d_Ibs_dV_b_GND =  0.0;
Ibs = 0;

d_Ibd_dV_db_di = d_Ibd_dV_sb_si = d_Ibd_dV_b_si = d_Ibd_dV_di_si = d_Ibd_dV_gi_si = d_Ibd_dV_e_si = d_Ibd_dTemp_t_GND = d_Ibd_dV_p_GND = d_Ibd_dV_b_GND =  0.0;
Ibd = 0;

d_Ibsdif_dTemp_t_GND = d_Ibsdif_dV_p_GND = d_Ibsdif_dV_b_GND = d_Ibsdif_dV_sb_si = d_Ibsdif_dV_b_si = d_Ibsdif_dV_di_si = d_Ibsdif_dV_gi_si = d_Ibsdif_dV_e_si =  0.0;
Ibsdif = 0;

d_Ibddif_dTemp_t_GND = d_Ibddif_dV_p_GND = d_Ibddif_dV_b_GND = d_Ibddif_dV_db_di = d_Ibddif_dV_sb_si = d_Ibddif_dV_b_si = d_Ibddif_dV_di_si = d_Ibddif_dV_gi_si = d_Ibddif_dV_e_si =  0.0;
Ibddif = 0;

d_Ic_dV_db_di = d_Ic_dV_sb_si = d_Ic_dV_b_si = d_Ic_dV_di_si = d_Ic_dV_gi_si = d_Ic_dV_e_si = d_Ic_dTemp_t_GND = d_Ic_dV_p_GND = d_Ic_dV_b_GND =  0.0;
Ic = 0.0;
}
if (((B4SOIigbMod!=0)||(B4SOIigcMod!=0)))
{

d_Vgb_dV_b_si = (-d_Vbs_dV_b_si);
d_Vgb_dV_e_si = (-d_Vbs_dV_e_si);
d_Vgb_dV_b_GND = (d_Vgs_eff_dV_b_GND-d_Vbs_dV_b_GND);
d_Vgb_dV_p_GND = (d_Vgs_eff_dV_p_GND-d_Vbs_dV_p_GND);
d_Vgb_dTemp_t_GND = (d_Vgs_eff_dTemp_t_GND-d_Vbs_dTemp_t_GND);
d_Vgb_dV_gi_si = (d_Vgs_eff_dV_gi_si-d_Vbs_dV_gi_si);
d_Vgb_dV_di_si = (d_Vgs_eff_dV_di_si-d_Vbs_dV_di_si);
Vgb = (Vgs_eff-Vbs);

d_Vfb_dV_b_si = d_Vfb_dV_di_si = d_Vfb_dV_gi_si = d_Vfb_dV_e_si =  0.0;
d_Vfb_dV_b_GND = ((B4SOItype*d_here_B4SOIvth0_dV_b_GND-d_phi_dV_b_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi));
d_Vfb_dV_p_GND = ((B4SOItype*d_here_B4SOIvth0_dV_p_GND-d_phi_dV_p_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi));
d_Vfb_dTemp_t_GND = ((B4SOItype*d_here_B4SOIvth0_dTemp_t_GND-d_phi_dTemp_t_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi));
Vfb = (((B4SOItype*here_B4SOIvth0)-phi)-(pParam_B4SOIk1eff*sqrtPhi));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_b_si = (d_Vfb_dV_b_si+d_Vbs_dV_b_si);
d_T3_dV_e_si = (d_Vfb_dV_e_si+d_Vbs_dV_e_si);
d_T3_dV_gi_si = ((d_Vfb_dV_gi_si-d_Vgs_eff_dV_gi_si)+d_Vbs_dV_gi_si);
d_T3_dV_di_si = ((d_Vfb_dV_di_si-d_Vgs_eff_dV_di_si)+d_Vbs_dV_di_si);
d_T3_dV_b_GND = ((d_Vfb_dV_b_GND-d_Vgs_eff_dV_b_GND)+d_Vbs_dV_b_GND);
d_T3_dV_p_GND = ((d_Vfb_dV_p_GND-d_Vgs_eff_dV_p_GND)+d_Vbs_dV_p_GND);
d_T3_dTemp_t_GND = ((d_Vfb_dTemp_t_GND-d_Vgs_eff_dTemp_t_GND)+d_Vbs_dTemp_t_GND);
T3 = (((Vfb-Vgs_eff)+Vbs)-0.02);
if ((Vfb<=0.0))
{
{
double value_sqrt_0 = sqrt(((T3*T3)-((4.0*0.02)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((T3*d_T3_dV_b_GND+d_T3_dV_b_GND*T3)-(4.0*0.02)*d_Vfb_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((T3*d_T3_dV_p_GND+d_T3_dV_p_GND*T3)-(4.0*0.02)*d_Vfb_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((T3*d_T3_dTemp_t_GND+d_T3_dTemp_t_GND*T3)-(4.0*0.02)*d_Vfb_dTemp_t_GND)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_gi_si+d_T3_dV_gi_si*T3)-(4.0*0.02)*d_Vfb_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_di_si+d_T3_dV_di_si*T3)-(4.0*0.02)*d_Vfb_dV_di_si)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_e_si+d_T3_dV_e_si*T3)-(4.0*0.02)*d_Vfb_dV_e_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_b_si+d_T3_dV_b_si*T3)-(4.0*0.02)*d_Vfb_dV_b_si)));
d_T0_dV_sb_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_sb_si+d_T3_dV_sb_si*T3)));
d_T0_dV_db_di = (deriv_sqrt_0_d0*((T3*d_T3_dV_db_di+d_T3_dV_db_di*T3)));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((T3*T3)+((4.0*0.02)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((T3*d_T3_dV_b_GND+d_T3_dV_b_GND*T3)+(4.0*0.02)*d_Vfb_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((T3*d_T3_dV_p_GND+d_T3_dV_p_GND*T3)+(4.0*0.02)*d_Vfb_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((T3*d_T3_dTemp_t_GND+d_T3_dTemp_t_GND*T3)+(4.0*0.02)*d_Vfb_dTemp_t_GND)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_gi_si+d_T3_dV_gi_si*T3)+(4.0*0.02)*d_Vfb_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_di_si+d_T3_dV_di_si*T3)+(4.0*0.02)*d_Vfb_dV_di_si)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_e_si+d_T3_dV_e_si*T3)+(4.0*0.02)*d_Vfb_dV_e_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((T3*d_T3_dV_b_si+d_T3_dV_b_si*T3)+(4.0*0.02)*d_Vfb_dV_b_si)));
d_T0_dV_sb_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_sb_si+d_T3_dV_sb_si*T3)));
d_T0_dV_db_di = (deriv_sqrt_0_d0*((T3*d_T3_dV_db_di+d_T3_dV_db_di*T3)));
T0 = value_sqrt_0;
}
}

d_Vfbeff_dV_gi_p =  0.0;
d_Vfbeff_dV_gi_si = (d_Vfb_dV_gi_si-0.5*(d_T3_dV_gi_si+d_T0_dV_gi_si));
d_Vfbeff_dV_di_si = (d_Vfb_dV_di_si-0.5*(d_T3_dV_di_si+d_T0_dV_di_si));
d_Vfbeff_dV_e_si = (d_Vfb_dV_e_si-0.5*(d_T3_dV_e_si+d_T0_dV_e_si));
d_Vfbeff_dV_b_si = (d_Vfb_dV_b_si-0.5*(d_T3_dV_b_si+d_T0_dV_b_si));
d_Vfbeff_dV_sb_si = (-0.5*(d_T3_dV_sb_si+d_T0_dV_sb_si));
d_Vfbeff_dV_db_di = (-0.5*(d_T3_dV_db_di+d_T0_dV_db_di));
d_Vfbeff_dV_b_GND = (d_Vfb_dV_b_GND-0.5*(d_T3_dV_b_GND+d_T0_dV_b_GND));
d_Vfbeff_dV_p_GND = (d_Vfb_dV_p_GND-0.5*(d_T3_dV_p_GND+d_T0_dV_p_GND));
d_Vfbeff_dTemp_t_GND = (d_Vfb_dTemp_t_GND-0.5*(d_T3_dTemp_t_GND+d_T0_dTemp_t_GND));
Vfbeff = (Vfb-(0.5*(T3+T0)));

d_Voxacc_dV_gi_si = (d_Vfb_dV_gi_si-d_Vfbeff_dV_gi_si);
d_Voxacc_dV_di_si = (d_Vfb_dV_di_si-d_Vfbeff_dV_di_si);
d_Voxacc_dV_e_si = (d_Vfb_dV_e_si-d_Vfbeff_dV_e_si);
d_Voxacc_dV_b_si = (d_Vfb_dV_b_si-d_Vfbeff_dV_b_si);
d_Voxacc_dV_sb_si = (-d_Vfbeff_dV_sb_si);
d_Voxacc_dV_db_di = (-d_Vfbeff_dV_db_di);
d_Voxacc_dV_b_GND = (d_Vfb_dV_b_GND-d_Vfbeff_dV_b_GND);
d_Voxacc_dV_p_GND = (d_Vfb_dV_p_GND-d_Vfbeff_dV_p_GND);
d_Voxacc_dTemp_t_GND = (d_Vfb_dTemp_t_GND-d_Vfbeff_dTemp_t_GND);
Voxacc = (Vfb-Vfbeff);
if ((Voxacc<0.0))
{

d_Voxacc_dTemp_t_GND = d_Voxacc_dV_p_GND = d_Voxacc_dV_b_GND = d_Voxacc_dV_db_di = d_Voxacc_dV_sb_si = d_Voxacc_dV_b_si = d_Voxacc_dV_e_si = d_Voxacc_dV_di_si = d_Voxacc_dV_gi_si =  0.0;
Voxacc = 0.0;
}
if ((pParam_B4SOIk1ox==0.0))
{

d_Voxdepinv_dV_di_si = d_Voxdepinv_dV_gi_si = d_Voxdepinv_dTemp_t_GND = d_Voxdepinv_dV_p_GND = d_Voxdepinv_dV_b_GND = d_Voxdepinv_dV_db_di = d_Voxdepinv_dV_sb_si = d_Voxdepinv_dV_b_si = d_Voxdepinv_dV_e_si =  0.0;
Voxdepinv = 0.0;
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_sb_si = ((-d_Vgsteff_dV_sb_si)-d_Vfbeff_dV_sb_si);
d_T0_dV_db_di = ((-d_Vgsteff_dV_db_di)-d_Vfbeff_dV_db_di);
d_T0_dV_e_si = (((-d_Vgsteff_dV_e_si)-d_Vfbeff_dV_e_si)-d_Vbseff_dV_e_si);
d_T0_dV_b_si = (((-d_Vgsteff_dV_b_si)-d_Vfbeff_dV_b_si)-d_Vbseff_dV_b_si);
d_T0_dV_b_GND = (((d_Vgs_eff_dV_b_GND-d_Vgsteff_dV_b_GND)-d_Vfbeff_dV_b_GND)-d_Vbseff_dV_b_GND);
d_T0_dV_p_GND = (((d_Vgs_eff_dV_p_GND-d_Vgsteff_dV_p_GND)-d_Vfbeff_dV_p_GND)-d_Vbseff_dV_p_GND);
d_T0_dTemp_t_GND = (((d_Vgs_eff_dTemp_t_GND-d_Vgsteff_dTemp_t_GND)-d_Vfbeff_dTemp_t_GND)-d_Vbseff_dTemp_t_GND);
d_T0_dV_gi_si = (((d_Vgs_eff_dV_gi_si-d_Vgsteff_dV_gi_si)-d_Vfbeff_dV_gi_si)-d_Vbseff_dV_gi_si);
d_T0_dV_di_si = (((d_Vgs_eff_dV_di_si-d_Vgsteff_dV_di_si)-d_Vfbeff_dV_di_si)-d_Vbseff_dV_di_si);
T0 = (((Vgs_eff-Vgsteff)-Vfbeff)-Vbseff);
if ((T0<0.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND/pParam_B4SOIk1ox);
d_T1_dV_p_GND = (d_T0_dV_p_GND/pParam_B4SOIk1ox);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND/pParam_B4SOIk1ox);
d_T1_dV_e_si = (d_T0_dV_e_si/pParam_B4SOIk1ox);
d_T1_dV_gi_si = (d_T0_dV_gi_si/pParam_B4SOIk1ox);
d_T1_dV_di_si = (d_T0_dV_di_si/pParam_B4SOIk1ox);
d_T1_dV_b_si = (d_T0_dV_b_si/pParam_B4SOIk1ox);
d_T1_dV_sb_si = (d_T0_dV_sb_si/pParam_B4SOIk1ox);
d_T1_dV_db_di = (d_T0_dV_db_di/pParam_B4SOIk1ox);
T1 = (T0/pParam_B4SOIk1ox);
}
else
{
{
double value_sqrt_0 = sqrt((1+(((4*T0)/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_b_GND/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dV_p_GND = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_p_GND/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dTemp_t_GND = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dTemp_t_GND/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dV_e_si = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_e_si/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dV_gi_si = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_gi_si/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dV_di_si = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_di_si/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dV_b_si = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_b_si/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dV_sb_si = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_sb_si/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
d_T1_dV_db_di = (pParam_B4SOIk1ox/2)*(deriv_sqrt_0_d0*(((4*d_T0_dV_db_di/pParam_B4SOIk1ox)/pParam_B4SOIk1ox)));
T1 = ((pParam_B4SOIk1ox/2)*((-1)+value_sqrt_0));
}
}

d_Voxdepinv_dV_e_si = ((-((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+d_Vbs_dV_e_si))-d_Vfb_dV_e_si);
d_Voxdepinv_dV_b_si = ((-((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+d_Vbs_dV_b_si))-d_Vfb_dV_b_si);
d_Voxdepinv_dV_sb_si = (-(T1*d_T1_dV_sb_si+d_T1_dV_sb_si*T1));
d_Voxdepinv_dV_db_di = (-(T1*d_T1_dV_db_di+d_T1_dV_db_di*T1));
d_Voxdepinv_dV_b_GND = ((d_Vgs_eff_dV_b_GND-((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+d_Vbs_dV_b_GND))-d_Vfb_dV_b_GND);
d_Voxdepinv_dV_p_GND = ((d_Vgs_eff_dV_p_GND-((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+d_Vbs_dV_p_GND))-d_Vfb_dV_p_GND);
d_Voxdepinv_dTemp_t_GND = ((d_Vgs_eff_dTemp_t_GND-((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+d_Vbs_dTemp_t_GND))-d_Vfb_dTemp_t_GND);
d_Voxdepinv_dV_gi_si = ((d_Vgs_eff_dV_gi_si-((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+d_Vbs_dV_gi_si))-d_Vfb_dV_gi_si);
d_Voxdepinv_dV_di_si = ((d_Vgs_eff_dV_di_si-((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+d_Vbs_dV_di_si))-d_Vfb_dV_di_si);
Voxdepinv = ((Vgs_eff-((T1*T1)+Vbs))-Vfb);
}
}
else
{

d_Vfb_dV_b_si = d_Vfb_dV_di_si = d_Vfb_dV_gi_si = d_Vfb_dV_e_si = d_Vfb_dTemp_t_GND = d_Vfb_dV_p_GND = d_Vfb_dV_b_GND =  0.0;
Vfb = 0;

d_Vgb_dV_di_si = d_Vgb_dV_gi_si = d_Vgb_dTemp_t_GND = d_Vgb_dV_p_GND = d_Vgb_dV_b_GND = d_Vgb_dV_e_si = d_Vgb_dV_b_si =  0.0;
Vgb = 0;

d_Voxacc_dTemp_t_GND = d_Voxacc_dV_p_GND = d_Voxacc_dV_b_GND = d_Voxacc_dV_db_di = d_Voxacc_dV_sb_si = d_Voxacc_dV_b_si = d_Voxacc_dV_e_si = d_Voxacc_dV_di_si = d_Voxacc_dV_gi_si =  0.0;
Voxacc = 0;

d_Voxdepinv_dV_di_si = d_Voxdepinv_dV_gi_si = d_Voxdepinv_dTemp_t_GND = d_Voxdepinv_dV_p_GND = d_Voxdepinv_dV_b_GND = d_Voxdepinv_dV_db_di = d_Voxdepinv_dV_sb_si = d_Voxdepinv_dV_b_si = d_Voxdepinv_dV_e_si =  0.0;
Voxdepinv = 0;
}
if (B4SOIigcMod)
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = d_Vtm_dV_b_GND*pParam_B4SOInigc;
d_T0_dV_p_GND = d_Vtm_dV_p_GND*pParam_B4SOInigc;
d_T0_dTemp_t_GND = d_Vtm_dTemp_t_GND*pParam_B4SOInigc;
T0 = (Vtm*pParam_B4SOInigc);

d_VxNVt_dV_e_si = (-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_e_si/T0/T0);
d_VxNVt_dV_b_si = (-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_b_si/T0/T0);
d_VxNVt_dV_sb_si = (-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_sb_si/T0/T0);
d_VxNVt_dV_db_di = (-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_db_di/T0/T0);
d_VxNVt_dV_b_GND = ((T0*(d_Vgs_eff_dV_b_GND-B4SOItype*d_here_B4SOIvth0_dV_b_GND)-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_b_GND)/T0/T0);
d_VxNVt_dV_p_GND = ((T0*(d_Vgs_eff_dV_p_GND-B4SOItype*d_here_B4SOIvth0_dV_p_GND)-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_p_GND)/T0/T0);
d_VxNVt_dTemp_t_GND = ((T0*(d_Vgs_eff_dTemp_t_GND-B4SOItype*d_here_B4SOIvth0_dTemp_t_GND)-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dTemp_t_GND)/T0/T0);
d_VxNVt_dV_gi_si = ((T0*d_Vgs_eff_dV_gi_si-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_gi_si)/T0/T0);
d_VxNVt_dV_di_si = ((T0*d_Vgs_eff_dV_di_si-(Vgs_eff-(B4SOItype*here_B4SOIvth0))*d_T0_dV_di_si)/T0/T0);
VxNVt = ((Vgs_eff-(B4SOItype*here_B4SOIvth0))/T0);
if ((VxNVt>100.0))
{

d_Vaux_dV_db_di = d_Vaux_dV_sb_si = d_Vaux_dV_b_si = d_Vaux_dV_e_si =  0.0;
d_Vaux_dV_b_GND = (d_Vgs_eff_dV_b_GND-B4SOItype*d_here_B4SOIvth0_dV_b_GND);
d_Vaux_dV_p_GND = (d_Vgs_eff_dV_p_GND-B4SOItype*d_here_B4SOIvth0_dV_p_GND);
d_Vaux_dTemp_t_GND = (d_Vgs_eff_dTemp_t_GND-B4SOItype*d_here_B4SOIvth0_dTemp_t_GND);
d_Vaux_dV_gi_si = d_Vgs_eff_dV_gi_si;
d_Vaux_dV_di_si = d_Vgs_eff_dV_di_si;
Vaux = (Vgs_eff-(B4SOItype*here_B4SOIvth0));
}
else
{
if ((VxNVt<(-100.0)))
{
{
double value_log_0 = log((1.0+3.720075976e-44));

d_Vaux_dV_b_GND = d_T0_dV_b_GND*value_log_0;
d_Vaux_dV_p_GND = d_T0_dV_p_GND*value_log_0;
d_Vaux_dTemp_t_GND = d_T0_dTemp_t_GND*value_log_0;
d_Vaux_dV_e_si = d_T0_dV_e_si*value_log_0;
d_Vaux_dV_gi_si = d_T0_dV_gi_si*value_log_0;
d_Vaux_dV_di_si = d_T0_dV_di_si*value_log_0;
d_Vaux_dV_b_si = d_T0_dV_b_si*value_log_0;
d_Vaux_dV_sb_si = d_T0_dV_sb_si*value_log_0;
d_Vaux_dV_db_di = d_T0_dV_db_di*value_log_0;
Vaux = (T0*value_log_0);
}
}
else
{
{
double value_exp_0 = exp(VxNVt);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVxNVt_dV_e_si = (deriv_exp_0_d0*(d_VxNVt_dV_e_si));
d_ExpVxNVt_dV_b_si = (deriv_exp_0_d0*(d_VxNVt_dV_b_si));
d_ExpVxNVt_dV_sb_si = (deriv_exp_0_d0*(d_VxNVt_dV_sb_si));
d_ExpVxNVt_dV_db_di = (deriv_exp_0_d0*(d_VxNVt_dV_db_di));
d_ExpVxNVt_dV_b_GND = (deriv_exp_0_d0*(d_VxNVt_dV_b_GND));
d_ExpVxNVt_dV_p_GND = (deriv_exp_0_d0*(d_VxNVt_dV_p_GND));
d_ExpVxNVt_dTemp_t_GND = (deriv_exp_0_d0*(d_VxNVt_dTemp_t_GND));
d_ExpVxNVt_dV_gi_si = (deriv_exp_0_d0*(d_VxNVt_dV_gi_si));
d_ExpVxNVt_dV_di_si = (deriv_exp_0_d0*(d_VxNVt_dV_di_si));
ExpVxNVt = value_exp_0;
}
{
double value_log_0 = log((1.0+ExpVxNVt));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVxNVt));

d_Vaux_dV_b_GND = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_b_GND))+d_T0_dV_b_GND*value_log_0);
d_Vaux_dV_p_GND = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_p_GND))+d_T0_dV_p_GND*value_log_0);
d_Vaux_dTemp_t_GND = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dTemp_t_GND))+d_T0_dTemp_t_GND*value_log_0);
d_Vaux_dV_e_si = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_e_si))+d_T0_dV_e_si*value_log_0);
d_Vaux_dV_gi_si = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_gi_si))+d_T0_dV_gi_si*value_log_0);
d_Vaux_dV_di_si = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_di_si))+d_T0_dV_di_si*value_log_0);
d_Vaux_dV_b_si = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_b_si))+d_T0_dV_b_si*value_log_0);
d_Vaux_dV_sb_si = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_sb_si))+d_T0_dV_sb_si*value_log_0);
d_Vaux_dV_db_di = (T0*(deriv_log_0_d0*(d_ExpVxNVt_dV_db_di))+d_T0_dV_db_di*value_log_0);
Vaux = (T0*value_log_0);
}
}
}

d_T2_dV_gm_si = d_T2_dV_gi_p =  0.0;
d_T2_dV_e_si = Vgs_eff*d_Vaux_dV_e_si;
d_T2_dV_b_si = Vgs_eff*d_Vaux_dV_b_si;
d_T2_dV_sb_si = Vgs_eff*d_Vaux_dV_sb_si;
d_T2_dV_db_di = Vgs_eff*d_Vaux_dV_db_di;
d_T2_dV_b_GND = (Vgs_eff*d_Vaux_dV_b_GND+d_Vgs_eff_dV_b_GND*Vaux);
d_T2_dV_p_GND = (Vgs_eff*d_Vaux_dV_p_GND+d_Vgs_eff_dV_p_GND*Vaux);
d_T2_dTemp_t_GND = (Vgs_eff*d_Vaux_dTemp_t_GND+d_Vgs_eff_dTemp_t_GND*Vaux);
d_T2_dV_gi_si = (Vgs_eff*d_Vaux_dV_gi_si+d_Vgs_eff_dV_gi_si*Vaux);
d_T2_dV_di_si = (Vgs_eff*d_Vaux_dV_di_si+d_Vgs_eff_dV_di_si*Vaux);
T2 = (Vgs_eff*Vaux);

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = pParam_B4SOIAechvb;

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si = d_T12_dV_b_si = d_T12_dV_e_si = d_T12_dV_di_si = d_T12_dV_gi_si = d_T12_dTemp_t_GND = d_T12_dV_p_GND = d_T12_dV_b_GND =  0.0;
T12 = pParam_B4SOIBechvb;

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = ((pParam_B4SOIaigc*pParam_B4SOIcigc)-pParam_B4SOIbigc);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = (pParam_B4SOIbigc*pParam_B4SOIcigc);

d_T5_dV_gi_p =  0.0;
d_T5_dV_sb_si = (T12*((T3*d_Voxdepinv_dV_sb_si+d_T3_dV_sb_si*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_sb_si+(T4*d_Voxdepinv_dV_sb_si+d_T4_dV_sb_si*Voxdepinv)*Voxdepinv))+d_T12_dV_sb_si*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dV_db_di = (T12*((T3*d_Voxdepinv_dV_db_di+d_T3_dV_db_di*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_db_di+(T4*d_Voxdepinv_dV_db_di+d_T4_dV_db_di*Voxdepinv)*Voxdepinv))+d_T12_dV_db_di*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dV_b_GND = (T12*((T3*d_Voxdepinv_dV_b_GND+d_T3_dV_b_GND*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_b_GND+(T4*d_Voxdepinv_dV_b_GND+d_T4_dV_b_GND*Voxdepinv)*Voxdepinv))+d_T12_dV_b_GND*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dV_p_GND = (T12*((T3*d_Voxdepinv_dV_p_GND+d_T3_dV_p_GND*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_p_GND+(T4*d_Voxdepinv_dV_p_GND+d_T4_dV_p_GND*Voxdepinv)*Voxdepinv))+d_T12_dV_p_GND*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dTemp_t_GND = (T12*((T3*d_Voxdepinv_dTemp_t_GND+d_T3_dTemp_t_GND*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dTemp_t_GND+(T4*d_Voxdepinv_dTemp_t_GND+d_T4_dTemp_t_GND*Voxdepinv)*Voxdepinv))+d_T12_dTemp_t_GND*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dV_gi_si = (T12*((T3*d_Voxdepinv_dV_gi_si+d_T3_dV_gi_si*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_gi_si+(T4*d_Voxdepinv_dV_gi_si+d_T4_dV_gi_si*Voxdepinv)*Voxdepinv))+d_T12_dV_gi_si*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dV_di_si = (T12*((T3*d_Voxdepinv_dV_di_si+d_T3_dV_di_si*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_di_si+(T4*d_Voxdepinv_dV_di_si+d_T4_dV_di_si*Voxdepinv)*Voxdepinv))+d_T12_dV_di_si*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dV_e_si = (T12*((T3*d_Voxdepinv_dV_e_si+d_T3_dV_e_si*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_e_si+(T4*d_Voxdepinv_dV_e_si+d_T4_dV_e_si*Voxdepinv)*Voxdepinv))+d_T12_dV_e_si*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
d_T5_dV_b_si = (T12*((T3*d_Voxdepinv_dV_b_si+d_T3_dV_b_si*Voxdepinv)-((T4*Voxdepinv)*d_Voxdepinv_dV_b_si+(T4*d_Voxdepinv_dV_b_si+d_T4_dV_b_si*Voxdepinv)*Voxdepinv))+d_T12_dV_b_si*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
T5 = (T12*((pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
if ((T5>100.0))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;

d_T6_dV_gi_p =  0.0;
d_T6_dV_b_GND = (deriv_exp_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_exp_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_exp_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_exp_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_exp_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_exp_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_exp_0_d0*(d_T5_dV_b_si));
d_T6_dV_sb_si = (deriv_exp_0_d0*(d_T5_dV_sb_si));
d_T6_dV_db_di = (deriv_exp_0_d0*(d_T5_dV_db_di));
T6 = value_exp_0;
}
}
}

d_Igc_dV_b_GND = ((T11*T2)*d_T6_dV_b_GND+(T11*d_T2_dV_b_GND+d_T11_dV_b_GND*T2)*T6);
d_Igc_dV_p_GND = ((T11*T2)*d_T6_dV_p_GND+(T11*d_T2_dV_p_GND+d_T11_dV_p_GND*T2)*T6);
d_Igc_dTemp_t_GND = ((T11*T2)*d_T6_dTemp_t_GND+(T11*d_T2_dTemp_t_GND+d_T11_dTemp_t_GND*T2)*T6);
d_Igc_dV_gi_si = ((T11*T2)*d_T6_dV_gi_si+(T11*d_T2_dV_gi_si+d_T11_dV_gi_si*T2)*T6);
d_Igc_dV_di_si = ((T11*T2)*d_T6_dV_di_si+(T11*d_T2_dV_di_si+d_T11_dV_di_si*T2)*T6);
d_Igc_dV_e_si = ((T11*T2)*d_T6_dV_e_si+(T11*d_T2_dV_e_si+d_T11_dV_e_si*T2)*T6);
d_Igc_dV_b_si = ((T11*T2)*d_T6_dV_b_si+(T11*d_T2_dV_b_si+d_T11_dV_b_si*T2)*T6);
d_Igc_dV_sb_si = ((T11*T2)*d_T6_dV_sb_si+(T11*d_T2_dV_sb_si+d_T11_dV_sb_si*T2)*T6);
d_Igc_dV_db_di = ((T11*T2)*d_T6_dV_db_di+(T11*d_T2_dV_db_di+d_T11_dV_db_di*T2)*T6);
Igc = ((T11*T2)*T6);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si = d_T7_dV_b_si = d_T7_dV_e_si = d_T7_dV_gi_si = d_T7_dTemp_t_GND = d_T7_dV_p_GND = d_T7_dV_b_GND =  0.0;
d_T7_dV_di_si = (-pParam_B4SOIpigcd)*d_Vds_dV_di_si;
T7 = ((-pParam_B4SOIpigcd)*Vds);

d_T8_dV_b_GND = (T7*d_T7_dV_b_GND+d_T7_dV_b_GND*T7);
d_T8_dV_p_GND = (T7*d_T7_dV_p_GND+d_T7_dV_p_GND*T7);
d_T8_dTemp_t_GND = (T7*d_T7_dTemp_t_GND+d_T7_dTemp_t_GND*T7);
d_T8_dV_gi_si = (T7*d_T7_dV_gi_si+d_T7_dV_gi_si*T7);
d_T8_dV_di_si = (T7*d_T7_dV_di_si+d_T7_dV_di_si*T7);
d_T8_dV_e_si = (T7*d_T7_dV_e_si+d_T7_dV_e_si*T7);
d_T8_dV_b_si = (T7*d_T7_dV_b_si+d_T7_dV_b_si*T7);
T8 = ((T7*T7)+2.0e-4);
if ((T7>100.0))
{

d_T9_dV_b_si = d_T9_dV_e_si = d_T9_dV_di_si = d_T9_dV_gi_si = d_T9_dTemp_t_GND = d_T9_dV_p_GND = d_T9_dV_b_GND =  0.0;
T9 = 2.688117142e+43;
}
else
{
if ((T7<(-100.0)))
{

d_T9_dV_b_si = d_T9_dV_e_si = d_T9_dV_di_si = d_T9_dV_gi_si = d_T9_dTemp_t_GND = d_T9_dV_p_GND = d_T9_dV_b_GND =  0.0;
T9 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;

d_T9_dV_b_GND = (deriv_exp_0_d0*(d_T7_dV_b_GND));
d_T9_dV_p_GND = (deriv_exp_0_d0*(d_T7_dV_p_GND));
d_T9_dTemp_t_GND = (deriv_exp_0_d0*(d_T7_dTemp_t_GND));
d_T9_dV_gi_si = (deriv_exp_0_d0*(d_T7_dV_gi_si));
d_T9_dV_di_si = (deriv_exp_0_d0*(d_T7_dV_di_si));
d_T9_dV_e_si = (deriv_exp_0_d0*(d_T7_dV_e_si));
d_T9_dV_b_si = (deriv_exp_0_d0*(d_T7_dV_b_si));
T9 = value_exp_0;
}
}
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = d_T9_dV_b_GND;
d_T1_dV_p_GND = d_T9_dV_p_GND;
d_T1_dTemp_t_GND = d_T9_dTemp_t_GND;
d_T1_dV_gi_si = d_T9_dV_gi_si;
d_T1_dV_di_si = d_T9_dV_di_si;
d_T1_dV_e_si = d_T9_dV_e_si;
d_T1_dV_b_si = d_T9_dV_b_si;
T1 = ((T9-1.0)+1.0e-4);

d_T10_dV_b_GND = ((T8*(d_T1_dV_b_GND-d_T7_dV_b_GND)-(T1-T7)*d_T8_dV_b_GND)/T8/T8);
d_T10_dV_p_GND = ((T8*(d_T1_dV_p_GND-d_T7_dV_p_GND)-(T1-T7)*d_T8_dV_p_GND)/T8/T8);
d_T10_dTemp_t_GND = ((T8*(d_T1_dTemp_t_GND-d_T7_dTemp_t_GND)-(T1-T7)*d_T8_dTemp_t_GND)/T8/T8);
d_T10_dV_gi_si = ((T8*(d_T1_dV_gi_si-d_T7_dV_gi_si)-(T1-T7)*d_T8_dV_gi_si)/T8/T8);
d_T10_dV_di_si = ((T8*(d_T1_dV_di_si-d_T7_dV_di_si)-(T1-T7)*d_T8_dV_di_si)/T8/T8);
d_T10_dV_e_si = ((T8*(d_T1_dV_e_si-d_T7_dV_e_si)-(T1-T7)*d_T8_dV_e_si)/T8/T8);
d_T10_dV_b_si = ((T8*(d_T1_dV_b_si-d_T7_dV_b_si)-(T1-T7)*d_T8_dV_b_si)/T8/T8);
d_T10_dV_sb_si = ((d_T1_dV_sb_si-d_T7_dV_sb_si)/T8);
d_T10_dV_db_di = ((d_T1_dV_db_di-d_T7_dV_db_di)/T8);
T10 = ((T1-T7)/T8);

d_Igcs_dV_b_GND = (Igc*d_T10_dV_b_GND+d_Igc_dV_b_GND*T10);
d_Igcs_dV_p_GND = (Igc*d_T10_dV_p_GND+d_Igc_dV_p_GND*T10);
d_Igcs_dTemp_t_GND = (Igc*d_T10_dTemp_t_GND+d_Igc_dTemp_t_GND*T10);
d_Igcs_dV_gi_si = (Igc*d_T10_dV_gi_si+d_Igc_dV_gi_si*T10);
d_Igcs_dV_di_si = (Igc*d_T10_dV_di_si+d_Igc_dV_di_si*T10);
d_Igcs_dV_e_si = (Igc*d_T10_dV_e_si+d_Igc_dV_e_si*T10);
d_Igcs_dV_b_si = (Igc*d_T10_dV_b_si+d_Igc_dV_b_si*T10);
d_Igcs_dV_sb_si = (Igc*d_T10_dV_sb_si+d_Igc_dV_sb_si*T10);
d_Igcs_dV_db_di = (Igc*d_T10_dV_db_di+d_Igc_dV_db_di*T10);
Igcs = (Igc*T10);

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_b_GND = d_T9_dV_b_GND;
d_T1_dV_p_GND = d_T9_dV_p_GND;
d_T1_dTemp_t_GND = d_T9_dTemp_t_GND;
d_T1_dV_gi_si = d_T9_dV_gi_si;
d_T1_dV_di_si = d_T9_dV_di_si;
d_T1_dV_e_si = d_T9_dV_e_si;
d_T1_dV_b_si = d_T9_dV_b_si;
T1 = ((T9-1.0)-1.0e-4);

d_T10_dV_sb_si = ((d_T7_dV_sb_si*T9-d_T1_dV_sb_si)/T8);
d_T10_dV_db_di = ((d_T7_dV_db_di*T9-d_T1_dV_db_di)/T8);
d_T10_dV_b_GND = ((T8*((T7*d_T9_dV_b_GND+d_T7_dV_b_GND*T9)-d_T1_dV_b_GND)-((T7*T9)-T1)*d_T8_dV_b_GND)/T8/T8);
d_T10_dV_p_GND = ((T8*((T7*d_T9_dV_p_GND+d_T7_dV_p_GND*T9)-d_T1_dV_p_GND)-((T7*T9)-T1)*d_T8_dV_p_GND)/T8/T8);
d_T10_dTemp_t_GND = ((T8*((T7*d_T9_dTemp_t_GND+d_T7_dTemp_t_GND*T9)-d_T1_dTemp_t_GND)-((T7*T9)-T1)*d_T8_dTemp_t_GND)/T8/T8);
d_T10_dV_gi_si = ((T8*((T7*d_T9_dV_gi_si+d_T7_dV_gi_si*T9)-d_T1_dV_gi_si)-((T7*T9)-T1)*d_T8_dV_gi_si)/T8/T8);
d_T10_dV_di_si = ((T8*((T7*d_T9_dV_di_si+d_T7_dV_di_si*T9)-d_T1_dV_di_si)-((T7*T9)-T1)*d_T8_dV_di_si)/T8/T8);
d_T10_dV_e_si = ((T8*((T7*d_T9_dV_e_si+d_T7_dV_e_si*T9)-d_T1_dV_e_si)-((T7*T9)-T1)*d_T8_dV_e_si)/T8/T8);
d_T10_dV_b_si = ((T8*((T7*d_T9_dV_b_si+d_T7_dV_b_si*T9)-d_T1_dV_b_si)-((T7*T9)-T1)*d_T8_dV_b_si)/T8/T8);
T10 = (((T7*T9)-T1)/T8);

d_Igcd_dV_b_GND = (Igc*d_T10_dV_b_GND+d_Igc_dV_b_GND*T10);
d_Igcd_dV_p_GND = (Igc*d_T10_dV_p_GND+d_Igc_dV_p_GND*T10);
d_Igcd_dTemp_t_GND = (Igc*d_T10_dTemp_t_GND+d_Igc_dTemp_t_GND*T10);
d_Igcd_dV_gi_si = (Igc*d_T10_dV_gi_si+d_Igc_dV_gi_si*T10);
d_Igcd_dV_di_si = (Igc*d_T10_dV_di_si+d_Igc_dV_di_si*T10);
d_Igcd_dV_e_si = (Igc*d_T10_dV_e_si+d_Igc_dV_e_si*T10);
d_Igcd_dV_b_si = (Igc*d_T10_dV_b_si+d_Igc_dV_b_si*T10);
d_Igcd_dV_sb_si = (Igc*d_T10_dV_sb_si+d_Igc_dV_sb_si*T10);
d_Igcd_dV_db_di = (Igc*d_T10_dV_db_di+d_Igc_dV_db_di*T10);
Igcd = (Igc*T10);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_gi_si = d_vgs_dV_gi_si;
T0 = (vgs-pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_vgs_eff_dV_gi_p =  0.0;
d_vgs_eff_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_vgs_eff_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_vgs_eff_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_vgs_eff_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_vgs_eff_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_vgs_eff_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_vgs_eff_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
d_vgs_eff_dV_sb_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)));
d_vgs_eff_dV_db_di = (deriv_sqrt_0_d0*((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)));
vgs_eff = value_sqrt_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p =  0.0;
d_T2_dV_b_GND = vgs*d_vgs_eff_dV_b_GND;
d_T2_dV_p_GND = vgs*d_vgs_eff_dV_p_GND;
d_T2_dTemp_t_GND = vgs*d_vgs_eff_dTemp_t_GND;
d_T2_dV_e_si = vgs*d_vgs_eff_dV_e_si;
d_T2_dV_di_si = vgs*d_vgs_eff_dV_di_si;
d_T2_dV_b_si = vgs*d_vgs_eff_dV_b_si;
d_T2_dV_sb_si = vgs*d_vgs_eff_dV_sb_si;
d_T2_dV_db_di = vgs*d_vgs_eff_dV_db_di;
d_T2_dV_gi_si = (vgs*d_vgs_eff_dV_gi_si+d_vgs_dV_gi_si*vgs_eff);
T2 = (vgs*vgs_eff);

d_T13_dV_b_si = d_T13_dV_di_si = d_T13_dV_gi_si = d_T13_dV_e_si = d_T13_dTemp_t_GND = d_T13_dV_p_GND = d_T13_dV_b_GND =  0.0;
T13 = pParam_B4SOIAechvbEdges;

d_T14_dV_b_si = d_T14_dV_di_si = d_T14_dV_gi_si = d_T14_dV_e_si = d_T14_dTemp_t_GND = d_T14_dV_p_GND = d_T14_dV_b_GND =  0.0;
T14 = pParam_B4SOIAechvbEdged;

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si = d_T12_dV_b_si = d_T12_dV_e_si = d_T12_dV_di_si = d_T12_dV_gi_si = d_T12_dTemp_t_GND = d_T12_dV_p_GND = d_T12_dV_b_GND =  0.0;
T12 = pParam_B4SOIBechvbEdge;

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = ((pParam_B4SOIaigsd*pParam_B4SOIcigsd)-pParam_B4SOIbigsd);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = (pParam_B4SOIbigsd*pParam_B4SOIcigsd);

d_T5_dV_gi_p =  0.0;
d_T5_dV_sb_si = (T12*((T3*d_vgs_eff_dV_sb_si+d_T3_dV_sb_si*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_sb_si+(T4*d_vgs_eff_dV_sb_si+d_T4_dV_sb_si*vgs_eff)*vgs_eff))+d_T12_dV_sb_si*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dV_db_di = (T12*((T3*d_vgs_eff_dV_db_di+d_T3_dV_db_di*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_db_di+(T4*d_vgs_eff_dV_db_di+d_T4_dV_db_di*vgs_eff)*vgs_eff))+d_T12_dV_db_di*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dV_b_GND = (T12*((T3*d_vgs_eff_dV_b_GND+d_T3_dV_b_GND*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_b_GND+(T4*d_vgs_eff_dV_b_GND+d_T4_dV_b_GND*vgs_eff)*vgs_eff))+d_T12_dV_b_GND*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dV_p_GND = (T12*((T3*d_vgs_eff_dV_p_GND+d_T3_dV_p_GND*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_p_GND+(T4*d_vgs_eff_dV_p_GND+d_T4_dV_p_GND*vgs_eff)*vgs_eff))+d_T12_dV_p_GND*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dTemp_t_GND = (T12*((T3*d_vgs_eff_dTemp_t_GND+d_T3_dTemp_t_GND*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dTemp_t_GND+(T4*d_vgs_eff_dTemp_t_GND+d_T4_dTemp_t_GND*vgs_eff)*vgs_eff))+d_T12_dTemp_t_GND*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dV_gi_si = (T12*((T3*d_vgs_eff_dV_gi_si+d_T3_dV_gi_si*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_gi_si+(T4*d_vgs_eff_dV_gi_si+d_T4_dV_gi_si*vgs_eff)*vgs_eff))+d_T12_dV_gi_si*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dV_di_si = (T12*((T3*d_vgs_eff_dV_di_si+d_T3_dV_di_si*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_di_si+(T4*d_vgs_eff_dV_di_si+d_T4_dV_di_si*vgs_eff)*vgs_eff))+d_T12_dV_di_si*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dV_e_si = (T12*((T3*d_vgs_eff_dV_e_si+d_T3_dV_e_si*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_e_si+(T4*d_vgs_eff_dV_e_si+d_T4_dV_e_si*vgs_eff)*vgs_eff))+d_T12_dV_e_si*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
d_T5_dV_b_si = (T12*((T3*d_vgs_eff_dV_b_si+d_T3_dV_b_si*vgs_eff)-((T4*vgs_eff)*d_vgs_eff_dV_b_si+(T4*d_vgs_eff_dV_b_si+d_T4_dV_b_si*vgs_eff)*vgs_eff))+d_T12_dV_b_si*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
T5 = (T12*((pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
if ((T5>100.0))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;

d_T6_dV_gi_p =  0.0;
d_T6_dV_b_GND = (deriv_exp_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_exp_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_exp_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_exp_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_exp_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_exp_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_exp_0_d0*(d_T5_dV_b_si));
d_T6_dV_sb_si = (deriv_exp_0_d0*(d_T5_dV_sb_si));
d_T6_dV_db_di = (deriv_exp_0_d0*(d_T5_dV_db_di));
T6 = value_exp_0;
}
}
}

d_Igs_dV_sb_si = ((T13*T2)*d_T6_dV_sb_si+T13*d_T2_dV_sb_si*T6);
d_Igs_dV_db_di = ((T13*T2)*d_T6_dV_db_di+T13*d_T2_dV_db_di*T6);
d_Igs_dV_b_GND = ((T13*T2)*d_T6_dV_b_GND+(T13*d_T2_dV_b_GND+d_T13_dV_b_GND*T2)*T6);
d_Igs_dV_p_GND = ((T13*T2)*d_T6_dV_p_GND+(T13*d_T2_dV_p_GND+d_T13_dV_p_GND*T2)*T6);
d_Igs_dTemp_t_GND = ((T13*T2)*d_T6_dTemp_t_GND+(T13*d_T2_dTemp_t_GND+d_T13_dTemp_t_GND*T2)*T6);
d_Igs_dV_e_si = ((T13*T2)*d_T6_dV_e_si+(T13*d_T2_dV_e_si+d_T13_dV_e_si*T2)*T6);
d_Igs_dV_gi_si = ((T13*T2)*d_T6_dV_gi_si+(T13*d_T2_dV_gi_si+d_T13_dV_gi_si*T2)*T6);
d_Igs_dV_di_si = ((T13*T2)*d_T6_dV_di_si+(T13*d_T2_dV_di_si+d_T13_dV_di_si*T2)*T6);
d_Igs_dV_b_si = ((T13*T2)*d_T6_dV_b_si+(T13*d_T2_dV_b_si+d_T13_dV_b_si*T2)*T6);
Igs = ((T13*T2)*T6);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = d_vgd_dV_di_si;
d_T0_dV_gi_si = d_vgd_dV_gi_si;
T0 = (vgd-pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_vgd_eff_dV_gi_p =  0.0;
d_vgd_eff_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_vgd_eff_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_vgd_eff_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_vgd_eff_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_vgd_eff_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_vgd_eff_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_vgd_eff_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
d_vgd_eff_dV_sb_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)));
d_vgd_eff_dV_db_di = (deriv_sqrt_0_d0*((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)));
vgd_eff = value_sqrt_0;
}

d_T2_dV_gm_si = d_T2_dV_gi_p =  0.0;
d_T2_dV_b_GND = vgd*d_vgd_eff_dV_b_GND;
d_T2_dV_p_GND = vgd*d_vgd_eff_dV_p_GND;
d_T2_dTemp_t_GND = vgd*d_vgd_eff_dTemp_t_GND;
d_T2_dV_e_si = vgd*d_vgd_eff_dV_e_si;
d_T2_dV_b_si = vgd*d_vgd_eff_dV_b_si;
d_T2_dV_sb_si = vgd*d_vgd_eff_dV_sb_si;
d_T2_dV_db_di = vgd*d_vgd_eff_dV_db_di;
d_T2_dV_di_si = (vgd*d_vgd_eff_dV_di_si+d_vgd_dV_di_si*vgd_eff);
d_T2_dV_gi_si = (vgd*d_vgd_eff_dV_gi_si+d_vgd_dV_gi_si*vgd_eff);
T2 = (vgd*vgd_eff);

d_T5_dV_gi_p =  0.0;
d_T5_dV_sb_si = (T12*((T3*d_vgd_eff_dV_sb_si+d_T3_dV_sb_si*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_sb_si+(T4*d_vgd_eff_dV_sb_si+d_T4_dV_sb_si*vgd_eff)*vgd_eff))+d_T12_dV_sb_si*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dV_db_di = (T12*((T3*d_vgd_eff_dV_db_di+d_T3_dV_db_di*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_db_di+(T4*d_vgd_eff_dV_db_di+d_T4_dV_db_di*vgd_eff)*vgd_eff))+d_T12_dV_db_di*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dV_b_GND = (T12*((T3*d_vgd_eff_dV_b_GND+d_T3_dV_b_GND*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_b_GND+(T4*d_vgd_eff_dV_b_GND+d_T4_dV_b_GND*vgd_eff)*vgd_eff))+d_T12_dV_b_GND*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dV_p_GND = (T12*((T3*d_vgd_eff_dV_p_GND+d_T3_dV_p_GND*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_p_GND+(T4*d_vgd_eff_dV_p_GND+d_T4_dV_p_GND*vgd_eff)*vgd_eff))+d_T12_dV_p_GND*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dTemp_t_GND = (T12*((T3*d_vgd_eff_dTemp_t_GND+d_T3_dTemp_t_GND*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dTemp_t_GND+(T4*d_vgd_eff_dTemp_t_GND+d_T4_dTemp_t_GND*vgd_eff)*vgd_eff))+d_T12_dTemp_t_GND*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dV_gi_si = (T12*((T3*d_vgd_eff_dV_gi_si+d_T3_dV_gi_si*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_gi_si+(T4*d_vgd_eff_dV_gi_si+d_T4_dV_gi_si*vgd_eff)*vgd_eff))+d_T12_dV_gi_si*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dV_di_si = (T12*((T3*d_vgd_eff_dV_di_si+d_T3_dV_di_si*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_di_si+(T4*d_vgd_eff_dV_di_si+d_T4_dV_di_si*vgd_eff)*vgd_eff))+d_T12_dV_di_si*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dV_e_si = (T12*((T3*d_vgd_eff_dV_e_si+d_T3_dV_e_si*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_e_si+(T4*d_vgd_eff_dV_e_si+d_T4_dV_e_si*vgd_eff)*vgd_eff))+d_T12_dV_e_si*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
d_T5_dV_b_si = (T12*((T3*d_vgd_eff_dV_b_si+d_T3_dV_b_si*vgd_eff)-((T4*vgd_eff)*d_vgd_eff_dV_b_si+(T4*d_vgd_eff_dV_b_si+d_T4_dV_b_si*vgd_eff)*vgd_eff))+d_T12_dV_b_si*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
T5 = (T12*((pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
if ((T5>100.0))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;

d_T6_dV_gi_p =  0.0;
d_T6_dV_b_GND = (deriv_exp_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_exp_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_exp_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_exp_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_exp_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_exp_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_exp_0_d0*(d_T5_dV_b_si));
d_T6_dV_sb_si = (deriv_exp_0_d0*(d_T5_dV_sb_si));
d_T6_dV_db_di = (deriv_exp_0_d0*(d_T5_dV_db_di));
T6 = value_exp_0;
}
}
}

d_Igd_dV_sb_si = ((T14*T2)*d_T6_dV_sb_si+T14*d_T2_dV_sb_si*T6);
d_Igd_dV_db_di = ((T14*T2)*d_T6_dV_db_di+T14*d_T2_dV_db_di*T6);
d_Igd_dV_b_GND = ((T14*T2)*d_T6_dV_b_GND+(T14*d_T2_dV_b_GND+d_T14_dV_b_GND*T2)*T6);
d_Igd_dV_p_GND = ((T14*T2)*d_T6_dV_p_GND+(T14*d_T2_dV_p_GND+d_T14_dV_p_GND*T2)*T6);
d_Igd_dTemp_t_GND = ((T14*T2)*d_T6_dTemp_t_GND+(T14*d_T2_dTemp_t_GND+d_T14_dTemp_t_GND*T2)*T6);
d_Igd_dV_e_si = ((T14*T2)*d_T6_dV_e_si+(T14*d_T2_dV_e_si+d_T14_dV_e_si*T2)*T6);
d_Igd_dV_gi_si = ((T14*T2)*d_T6_dV_gi_si+(T14*d_T2_dV_gi_si+d_T14_dV_gi_si*T2)*T6);
d_Igd_dV_di_si = ((T14*T2)*d_T6_dV_di_si+(T14*d_T2_dV_di_si+d_T14_dV_di_si*T2)*T6);
d_Igd_dV_b_si = ((T14*T2)*d_T6_dV_b_si+(T14*d_T2_dV_b_si+d_T14_dV_b_si*T2)*T6);
Igd = ((T14*T2)*T6);
}
else
{

d_Igd_dV_b_si = d_Igd_dV_di_si = d_Igd_dV_gi_si = d_Igd_dV_e_si = d_Igd_dTemp_t_GND = d_Igd_dV_p_GND = d_Igd_dV_b_GND = d_Igd_dV_db_di = d_Igd_dV_sb_si =  0.0;
Igd = 0.0;

d_Igs_dV_b_si = d_Igs_dV_di_si = d_Igs_dV_gi_si = d_Igs_dV_e_si = d_Igs_dTemp_t_GND = d_Igs_dV_p_GND = d_Igs_dV_b_GND = d_Igs_dV_db_di = d_Igs_dV_sb_si =  0.0;
Igs = 0.0;

d_Igcd_dV_db_di = d_Igcd_dV_sb_si = d_Igcd_dV_b_si = d_Igcd_dV_e_si = d_Igcd_dV_di_si = d_Igcd_dV_gi_si = d_Igcd_dTemp_t_GND = d_Igcd_dV_p_GND = d_Igcd_dV_b_GND =  0.0;
Igcd = 0.0;

d_Igcs_dV_db_di = d_Igcs_dV_sb_si = d_Igcs_dV_b_si = d_Igcs_dV_e_si = d_Igcs_dV_di_si = d_Igcs_dV_gi_si = d_Igcs_dTemp_t_GND = d_Igcs_dV_p_GND = d_Igcs_dV_b_GND =  0.0;
Igcs = 0.0;
}
if (((B4SOIigbMod!=0)&&(B4SOIsoiMod!=2)))
{
OxideRatio = pParam_B4SOIoxideRatio;

d_Vox_dV_e_si = d_Voxdepinv_dV_e_si;
d_Vox_dV_b_si = d_Voxdepinv_dV_b_si;
d_Vox_dV_sb_si = d_Voxdepinv_dV_sb_si;
d_Vox_dV_db_di = d_Voxdepinv_dV_db_di;
d_Vox_dV_b_GND = d_Voxdepinv_dV_b_GND;
d_Vox_dV_p_GND = d_Voxdepinv_dV_p_GND;
d_Vox_dTemp_t_GND = d_Voxdepinv_dTemp_t_GND;
d_Vox_dV_gi_si = d_Voxdepinv_dV_gi_si;
d_Vox_dV_di_si = d_Voxdepinv_dV_di_si;
Vox = Voxdepinv;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = B4SOIvoxh;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND-d_Vox_dV_b_GND);
d_T1_dV_p_GND = (d_T0_dV_p_GND-d_Vox_dV_p_GND);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND-d_Vox_dTemp_t_GND);
d_T1_dV_e_si = (d_T0_dV_e_si-d_Vox_dV_e_si);
d_T1_dV_gi_si = (d_T0_dV_gi_si-d_Vox_dV_gi_si);
d_T1_dV_di_si = (d_T0_dV_di_si-d_Vox_dV_di_si);
d_T1_dV_b_si = (d_T0_dV_b_si-d_Vox_dV_b_si);
d_T1_dV_sb_si = (d_T0_dV_sb_si-d_Vox_dV_sb_si);
d_T1_dV_db_di = (d_T0_dV_db_di-d_Vox_dV_db_di);
T1 = ((T0-Vox)-B4SOIdeltavox);
{
double value_sqrt_0 = sqrt(((T1*T1)+((4*B4SOIdeltavox)*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p =  0.0;
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+(4*B4SOIdeltavox)*d_T0_dV_b_GND)));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+(4*B4SOIdeltavox)*d_T0_dV_p_GND)));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+(4*B4SOIdeltavox)*d_T0_dTemp_t_GND)));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_gi_si)));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_di_si)));
d_T3_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_e_si)));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_b_si)));
d_T3_dV_sb_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_sb_si+d_T1_dV_sb_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_sb_si)));
d_T3_dV_db_di = (deriv_sqrt_0_d0*(((T1*d_T1_dV_db_di+d_T1_dV_db_di*T1)+(4*B4SOIdeltavox)*d_T0_dV_db_di)));
T3 = value_sqrt_0;
}

d_Voxeff_dV_b_GND = (d_T0_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T3_dV_b_GND));
d_Voxeff_dV_p_GND = (d_T0_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T3_dV_p_GND));
d_Voxeff_dTemp_t_GND = (d_T0_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T3_dTemp_t_GND));
d_Voxeff_dV_e_si = (d_T0_dV_e_si-0.5*(d_T1_dV_e_si+d_T3_dV_e_si));
d_Voxeff_dV_gi_si = (d_T0_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T3_dV_gi_si));
d_Voxeff_dV_di_si = (d_T0_dV_di_si-0.5*(d_T1_dV_di_si+d_T3_dV_di_si));
d_Voxeff_dV_b_si = (d_T0_dV_b_si-0.5*(d_T1_dV_b_si+d_T3_dV_b_si));
d_Voxeff_dV_sb_si = (d_T0_dV_sb_si-0.5*(d_T1_dV_sb_si+d_T3_dV_sb_si));
d_Voxeff_dV_db_di = (d_T0_dV_db_di-0.5*(d_T1_dV_db_di+d_T3_dV_db_di));
Voxeff = (T0-(0.5*(T1+T3)));

d_Vox_dV_b_GND = d_Voxeff_dV_b_GND;
d_Vox_dV_p_GND = d_Voxeff_dV_p_GND;
d_Vox_dTemp_t_GND = d_Voxeff_dTemp_t_GND;
d_Vox_dV_e_si = d_Voxeff_dV_e_si;
d_Vox_dV_gi_si = d_Voxeff_dV_gi_si;
d_Vox_dV_di_si = d_Voxeff_dV_di_si;
d_Vox_dV_b_si = d_Voxeff_dV_b_si;
d_Vox_dV_sb_si = d_Voxeff_dV_sb_si;
d_Vox_dV_db_di = d_Voxeff_dV_db_di;
Vox = Voxeff;

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_e_si = (d_Vox_dV_e_si/B4SOIvevb);
d_T0_dV_b_si = (d_Vox_dV_b_si/B4SOIvevb);
d_T0_dV_sb_si = (d_Vox_dV_sb_si/B4SOIvevb);
d_T0_dV_db_di = (d_Vox_dV_db_di/B4SOIvevb);
d_T0_dV_b_GND = (d_Vox_dV_b_GND/B4SOIvevb);
d_T0_dV_p_GND = (d_Vox_dV_p_GND/B4SOIvevb);
d_T0_dTemp_t_GND = (d_Vox_dTemp_t_GND/B4SOIvevb);
d_T0_dV_gi_si = (d_Vox_dV_gi_si/B4SOIvevb);
d_T0_dV_di_si = (d_Vox_dV_di_si/B4SOIvevb);
T0 = ((Vox-B4SOIebg)/B4SOIvevb);
if ((T0>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T1_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T1_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T1_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T1_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T1_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T1_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T1_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T1_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T1 = value_exp_0;
}
}
}
{
double value_log_0 = log((1+T1));
double  deriv_log_0_d0 = (1.0/(1+T1));

d_Vaux_dV_b_GND = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_b_GND));
d_Vaux_dV_p_GND = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_p_GND));
d_Vaux_dTemp_t_GND = B4SOIvevb*(deriv_log_0_d0*(d_T1_dTemp_t_GND));
d_Vaux_dV_gi_si = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_gi_si));
d_Vaux_dV_di_si = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_di_si));
d_Vaux_dV_e_si = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_e_si));
d_Vaux_dV_b_si = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_b_si));
d_Vaux_dV_sb_si = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_sb_si));
d_Vaux_dV_db_di = B4SOIvevb*(deriv_log_0_d0*(d_T1_dV_db_di));
Vaux = (B4SOIvevb*value_log_0);
}
if ((B4SOIvgb1!=0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_e_si = (-(d_Vox_dV_e_si/B4SOIvgb1));
d_T0_dV_b_si = (-(d_Vox_dV_b_si/B4SOIvgb1));
d_T0_dV_sb_si = (-(d_Vox_dV_sb_si/B4SOIvgb1));
d_T0_dV_db_di = (-(d_Vox_dV_db_di/B4SOIvgb1));
d_T0_dV_b_GND = (-(d_Vox_dV_b_GND/B4SOIvgb1));
d_T0_dV_p_GND = (-(d_Vox_dV_p_GND/B4SOIvgb1));
d_T0_dTemp_t_GND = (-(d_Vox_dTemp_t_GND/B4SOIvgb1));
d_T0_dV_gi_si = (-(d_Vox_dV_gi_si/B4SOIvgb1));
d_T0_dV_di_si = (-(d_Vox_dV_di_si/B4SOIvgb1));
T0 = (1-(Vox/B4SOIvgb1));
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 1;
}
if ((T0<0.01))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 0.01;
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (Leff*d_Weff_dV_e_si/B4SOInseg)*agb1*OxideRatio;
d_T1_dV_b_si = (Leff*d_Weff_dV_b_si/B4SOInseg)*agb1*OxideRatio;
d_T1_dV_b_GND = (Leff*d_Weff_dV_b_GND/B4SOInseg)*agb1*OxideRatio;
d_T1_dV_p_GND = (Leff*d_Weff_dV_p_GND/B4SOInseg)*agb1*OxideRatio;
d_T1_dTemp_t_GND = (Leff*d_Weff_dTemp_t_GND/B4SOInseg)*agb1*OxideRatio;
d_T1_dV_gi_si = (Leff*d_Weff_dV_gi_si/B4SOInseg)*agb1*OxideRatio;
d_T1_dV_di_si = (Leff*d_Weff_dV_di_si/B4SOInseg)*agb1*OxideRatio;
T1 = (((((Leff*Weff)/B4SOInseg)+(B4SOIagbcpd/B4SOInf))*agb1)*OxideRatio);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = (bgb1*B4SOItoxqm);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = pParam_B4SOIalphaGB1;

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = pParam_B4SOIbetaGB1;

d_T6_dV_gi_p =  0.0;
d_T6_dV_b_GND = ((T0*(T2*(d_T3_dV_b_GND-(T4*d_Vox_dV_b_GND+d_T4_dV_b_GND*Vox))+d_T2_dV_b_GND*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_b_GND)/T0/T0);
d_T6_dV_p_GND = ((T0*(T2*(d_T3_dV_p_GND-(T4*d_Vox_dV_p_GND+d_T4_dV_p_GND*Vox))+d_T2_dV_p_GND*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_p_GND)/T0/T0);
d_T6_dTemp_t_GND = ((T0*(T2*(d_T3_dTemp_t_GND-(T4*d_Vox_dTemp_t_GND+d_T4_dTemp_t_GND*Vox))+d_T2_dTemp_t_GND*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dTemp_t_GND)/T0/T0);
d_T6_dV_gi_si = ((T0*(T2*(d_T3_dV_gi_si-(T4*d_Vox_dV_gi_si+d_T4_dV_gi_si*Vox))+d_T2_dV_gi_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_gi_si)/T0/T0);
d_T6_dV_di_si = ((T0*(T2*(d_T3_dV_di_si-(T4*d_Vox_dV_di_si+d_T4_dV_di_si*Vox))+d_T2_dV_di_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_di_si)/T0/T0);
d_T6_dV_e_si = ((T0*(T2*(d_T3_dV_e_si-(T4*d_Vox_dV_e_si+d_T4_dV_e_si*Vox))+d_T2_dV_e_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_e_si)/T0/T0);
d_T6_dV_b_si = ((T0*(T2*(d_T3_dV_b_si-(T4*d_Vox_dV_b_si+d_T4_dV_b_si*Vox))+d_T2_dV_b_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_b_si)/T0/T0);
d_T6_dV_sb_si = ((T0*(T2*(d_T3_dV_sb_si-(T4*d_Vox_dV_sb_si+d_T4_dV_sb_si*Vox))+d_T2_dV_sb_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_sb_si)/T0/T0);
d_T6_dV_db_di = ((T0*(T2*(d_T3_dV_db_di-(T4*d_Vox_dV_db_di+d_T4_dV_db_di*Vox))+d_T2_dV_db_di*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_db_di)/T0/T0);
T6 = ((T2*(T3-(T4*Vox)))/T0);
if ((T6>100.0))
{

d_T5_dV_gi_p =  0.0;
d_T5_dV_b_GND = 2.688117142e+43*d_T6_dV_b_GND;
d_T5_dV_p_GND = 2.688117142e+43*d_T6_dV_p_GND;
d_T5_dTemp_t_GND = 2.688117142e+43*d_T6_dTemp_t_GND;
d_T5_dV_gi_si = 2.688117142e+43*d_T6_dV_gi_si;
d_T5_dV_di_si = 2.688117142e+43*d_T6_dV_di_si;
d_T5_dV_e_si = 2.688117142e+43*d_T6_dV_e_si;
d_T5_dV_b_si = 2.688117142e+43*d_T6_dV_b_si;
d_T5_dV_sb_si = 2.688117142e+43*d_T6_dV_sb_si;
d_T5_dV_db_di = 2.688117142e+43*d_T6_dV_db_di;
T5 = (2.688117142e+43*((1.0+T6)-100.0));
}
else
{
if ((T6<(-100.0)))
{

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si = d_T5_dV_di_si = d_T5_dV_gi_si = d_T5_dTemp_t_GND = d_T5_dV_p_GND = d_T5_dV_b_GND =  0.0;
T5 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T6);
double  deriv_exp_0_d0 = value_exp_0;

d_T5_dV_gi_p =  0.0;
d_T5_dV_b_GND = (deriv_exp_0_d0*(d_T6_dV_b_GND));
d_T5_dV_p_GND = (deriv_exp_0_d0*(d_T6_dV_p_GND));
d_T5_dTemp_t_GND = (deriv_exp_0_d0*(d_T6_dTemp_t_GND));
d_T5_dV_gi_si = (deriv_exp_0_d0*(d_T6_dV_gi_si));
d_T5_dV_di_si = (deriv_exp_0_d0*(d_T6_dV_di_si));
d_T5_dV_e_si = (deriv_exp_0_d0*(d_T6_dV_e_si));
d_T5_dV_b_si = (deriv_exp_0_d0*(d_T6_dV_b_si));
d_T5_dV_sb_si = (deriv_exp_0_d0*(d_T6_dV_sb_si));
d_T5_dV_db_di = (deriv_exp_0_d0*(d_T6_dV_db_di));
T5 = value_exp_0;
}
}
}

d_Igb1_dV_b_GND = (((T1*Vgb)*Vaux)*d_T5_dV_b_GND+((T1*Vgb)*d_Vaux_dV_b_GND+(T1*d_Vgb_dV_b_GND+d_T1_dV_b_GND*Vgb)*Vaux)*T5);
d_Igb1_dV_p_GND = (((T1*Vgb)*Vaux)*d_T5_dV_p_GND+((T1*Vgb)*d_Vaux_dV_p_GND+(T1*d_Vgb_dV_p_GND+d_T1_dV_p_GND*Vgb)*Vaux)*T5);
d_Igb1_dTemp_t_GND = (((T1*Vgb)*Vaux)*d_T5_dTemp_t_GND+((T1*Vgb)*d_Vaux_dTemp_t_GND+(T1*d_Vgb_dTemp_t_GND+d_T1_dTemp_t_GND*Vgb)*Vaux)*T5);
d_Igb1_dV_gi_si = (((T1*Vgb)*Vaux)*d_T5_dV_gi_si+((T1*Vgb)*d_Vaux_dV_gi_si+(T1*d_Vgb_dV_gi_si+d_T1_dV_gi_si*Vgb)*Vaux)*T5);
d_Igb1_dV_di_si = (((T1*Vgb)*Vaux)*d_T5_dV_di_si+((T1*Vgb)*d_Vaux_dV_di_si+(T1*d_Vgb_dV_di_si+d_T1_dV_di_si*Vgb)*Vaux)*T5);
d_Igb1_dV_e_si = (((T1*Vgb)*Vaux)*d_T5_dV_e_si+((T1*Vgb)*d_Vaux_dV_e_si+(T1*d_Vgb_dV_e_si+d_T1_dV_e_si*Vgb)*Vaux)*T5);
d_Igb1_dV_b_si = (((T1*Vgb)*Vaux)*d_T5_dV_b_si+((T1*Vgb)*d_Vaux_dV_b_si+(T1*d_Vgb_dV_b_si+d_T1_dV_b_si*Vgb)*Vaux)*T5);
d_Igb1_dV_sb_si = (((T1*Vgb)*Vaux)*d_T5_dV_sb_si+((T1*Vgb)*d_Vaux_dV_sb_si+d_T1_dV_sb_si*Vgb*Vaux)*T5);
d_Igb1_dV_db_di = (((T1*Vgb)*Vaux)*d_T5_dV_db_di+((T1*Vgb)*d_Vaux_dV_db_di+d_T1_dV_db_di*Vgb*Vaux)*T5);
Igb1 = (((T1*Vgb)*Vaux)*T5);

d_Vox_dV_gi_si = d_Voxacc_dV_gi_si;
d_Vox_dV_di_si = d_Voxacc_dV_di_si;
d_Vox_dV_e_si = d_Voxacc_dV_e_si;
d_Vox_dV_b_si = d_Voxacc_dV_b_si;
d_Vox_dV_sb_si = d_Voxacc_dV_sb_si;
d_Vox_dV_db_di = d_Voxacc_dV_db_di;
d_Vox_dV_b_GND = d_Voxacc_dV_b_GND;
d_Vox_dV_p_GND = d_Voxacc_dV_p_GND;
d_Vox_dTemp_t_GND = d_Voxacc_dTemp_t_GND;
Vox = Voxacc;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = B4SOIvoxh;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND-d_Vox_dV_b_GND);
d_T1_dV_p_GND = (d_T0_dV_p_GND-d_Vox_dV_p_GND);
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND-d_Vox_dTemp_t_GND);
d_T1_dV_e_si = (d_T0_dV_e_si-d_Vox_dV_e_si);
d_T1_dV_gi_si = (d_T0_dV_gi_si-d_Vox_dV_gi_si);
d_T1_dV_di_si = (d_T0_dV_di_si-d_Vox_dV_di_si);
d_T1_dV_b_si = (d_T0_dV_b_si-d_Vox_dV_b_si);
d_T1_dV_sb_si = (d_T0_dV_sb_si-d_Vox_dV_sb_si);
d_T1_dV_db_di = (d_T0_dV_db_di-d_Vox_dV_db_di);
T1 = ((T0-Vox)-B4SOIdeltavox);
{
double value_sqrt_0 = sqrt(((T1*T1)+((4*B4SOIdeltavox)*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_gi_p =  0.0;
d_T3_dV_b_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)+(4*B4SOIdeltavox)*d_T0_dV_b_GND)));
d_T3_dV_p_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)+(4*B4SOIdeltavox)*d_T0_dV_p_GND)));
d_T3_dTemp_t_GND = (deriv_sqrt_0_d0*(((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)+(4*B4SOIdeltavox)*d_T0_dTemp_t_GND)));
d_T3_dV_gi_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_gi_si)));
d_T3_dV_di_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_di_si)));
d_T3_dV_e_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_e_si)));
d_T3_dV_b_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_b_si)));
d_T3_dV_sb_si = (deriv_sqrt_0_d0*(((T1*d_T1_dV_sb_si+d_T1_dV_sb_si*T1)+(4*B4SOIdeltavox)*d_T0_dV_sb_si)));
d_T3_dV_db_di = (deriv_sqrt_0_d0*(((T1*d_T1_dV_db_di+d_T1_dV_db_di*T1)+(4*B4SOIdeltavox)*d_T0_dV_db_di)));
T3 = value_sqrt_0;
}

d_Voxeff_dV_b_GND = (d_T0_dV_b_GND-0.5*(d_T1_dV_b_GND+d_T3_dV_b_GND));
d_Voxeff_dV_p_GND = (d_T0_dV_p_GND-0.5*(d_T1_dV_p_GND+d_T3_dV_p_GND));
d_Voxeff_dTemp_t_GND = (d_T0_dTemp_t_GND-0.5*(d_T1_dTemp_t_GND+d_T3_dTemp_t_GND));
d_Voxeff_dV_e_si = (d_T0_dV_e_si-0.5*(d_T1_dV_e_si+d_T3_dV_e_si));
d_Voxeff_dV_gi_si = (d_T0_dV_gi_si-0.5*(d_T1_dV_gi_si+d_T3_dV_gi_si));
d_Voxeff_dV_di_si = (d_T0_dV_di_si-0.5*(d_T1_dV_di_si+d_T3_dV_di_si));
d_Voxeff_dV_b_si = (d_T0_dV_b_si-0.5*(d_T1_dV_b_si+d_T3_dV_b_si));
d_Voxeff_dV_sb_si = (d_T0_dV_sb_si-0.5*(d_T1_dV_sb_si+d_T3_dV_sb_si));
d_Voxeff_dV_db_di = (d_T0_dV_db_di-0.5*(d_T1_dV_db_di+d_T3_dV_db_di));
Voxeff = (T0-(0.5*(T1+T3)));

d_Vox_dV_b_GND = d_Voxeff_dV_b_GND;
d_Vox_dV_p_GND = d_Voxeff_dV_p_GND;
d_Vox_dTemp_t_GND = d_Voxeff_dTemp_t_GND;
d_Vox_dV_e_si = d_Voxeff_dV_e_si;
d_Vox_dV_gi_si = d_Voxeff_dV_gi_si;
d_Vox_dV_di_si = d_Voxeff_dV_di_si;
d_Vox_dV_b_si = d_Voxeff_dV_b_si;
d_Vox_dV_sb_si = d_Voxeff_dV_sb_si;
d_Vox_dV_db_di = d_Voxeff_dV_db_di;
Vox = Voxeff;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_si = (((-d_Vgb_dV_b_si)+d_Vfb_dV_b_si)/B4SOIvecb);
d_T0_dV_e_si = (((-d_Vgb_dV_e_si)+d_Vfb_dV_e_si)/B4SOIvecb);
d_T0_dV_b_GND = (((-d_Vgb_dV_b_GND)+d_Vfb_dV_b_GND)/B4SOIvecb);
d_T0_dV_p_GND = (((-d_Vgb_dV_p_GND)+d_Vfb_dV_p_GND)/B4SOIvecb);
d_T0_dTemp_t_GND = (((-d_Vgb_dTemp_t_GND)+d_Vfb_dTemp_t_GND)/B4SOIvecb);
d_T0_dV_gi_si = (((-d_Vgb_dV_gi_si)+d_Vfb_dV_gi_si)/B4SOIvecb);
d_T0_dV_di_si = (((-d_Vgb_dV_di_si)+d_Vfb_dV_di_si)/B4SOIvecb);
T0 = (((-Vgb)+Vfb)/B4SOIvecb);
if ((T0>100.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = 2.688117142e+43*d_T0_dV_b_GND;
d_T1_dV_p_GND = 2.688117142e+43*d_T0_dV_p_GND;
d_T1_dTemp_t_GND = 2.688117142e+43*d_T0_dTemp_t_GND;
d_T1_dV_e_si = 2.688117142e+43*d_T0_dV_e_si;
d_T1_dV_gi_si = 2.688117142e+43*d_T0_dV_gi_si;
d_T1_dV_di_si = 2.688117142e+43*d_T0_dV_di_si;
d_T1_dV_b_si = 2.688117142e+43*d_T0_dV_b_si;
d_T1_dV_sb_si = 2.688117142e+43*d_T0_dV_sb_si;
d_T1_dV_db_di = 2.688117142e+43*d_T0_dV_db_di;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_T1_dV_gm_si = d_T1_dV_gi_p =  0.0;
d_T1_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_T1_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_T1_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_T1_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_T1_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_T1_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_T1_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_T1_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_T1_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
T1 = value_exp_0;
}
}
}
{
double value_log_0 = log((1+T1));
double  deriv_log_0_d0 = (1.0/(1+T1));

d_Vaux_dV_b_GND = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_b_GND));
d_Vaux_dV_p_GND = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_p_GND));
d_Vaux_dTemp_t_GND = B4SOIvecb*(deriv_log_0_d0*(d_T1_dTemp_t_GND));
d_Vaux_dV_gi_si = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_gi_si));
d_Vaux_dV_di_si = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_di_si));
d_Vaux_dV_e_si = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_e_si));
d_Vaux_dV_b_si = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_b_si));
d_Vaux_dV_sb_si = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_sb_si));
d_Vaux_dV_db_di = B4SOIvecb*(deriv_log_0_d0*(d_T1_dV_db_di));
Vaux = (B4SOIvecb*value_log_0);
}
if ((B4SOIvgb2!=0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_e_si = (-(d_Vox_dV_e_si/B4SOIvgb2));
d_T0_dV_b_si = (-(d_Vox_dV_b_si/B4SOIvgb2));
d_T0_dV_sb_si = (-(d_Vox_dV_sb_si/B4SOIvgb2));
d_T0_dV_db_di = (-(d_Vox_dV_db_di/B4SOIvgb2));
d_T0_dV_b_GND = (-(d_Vox_dV_b_GND/B4SOIvgb2));
d_T0_dV_p_GND = (-(d_Vox_dV_p_GND/B4SOIvgb2));
d_T0_dTemp_t_GND = (-(d_Vox_dTemp_t_GND/B4SOIvgb2));
d_T0_dV_gi_si = (-(d_Vox_dV_gi_si/B4SOIvgb2));
d_T0_dV_di_si = (-(d_Vox_dV_di_si/B4SOIvgb2));
T0 = (1-(Vox/B4SOIvgb2));
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 1;
}
if ((T0<0.01))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 0.01;
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (Leff*d_Weff_dV_e_si/B4SOInseg)*agb2*OxideRatio;
d_T1_dV_b_si = (Leff*d_Weff_dV_b_si/B4SOInseg)*agb2*OxideRatio;
d_T1_dV_b_GND = (Leff*d_Weff_dV_b_GND/B4SOInseg)*agb2*OxideRatio;
d_T1_dV_p_GND = (Leff*d_Weff_dV_p_GND/B4SOInseg)*agb2*OxideRatio;
d_T1_dTemp_t_GND = (Leff*d_Weff_dTemp_t_GND/B4SOInseg)*agb2*OxideRatio;
d_T1_dV_gi_si = (Leff*d_Weff_dV_gi_si/B4SOInseg)*agb2*OxideRatio;
d_T1_dV_di_si = (Leff*d_Weff_dV_di_si/B4SOInseg)*agb2*OxideRatio;
T1 = (((((Leff*Weff)/B4SOInseg)+(B4SOIagbcpd/B4SOInf))*agb2)*OxideRatio);

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = (bgb2*B4SOItoxqm);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = pParam_B4SOIalphaGB2;

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = pParam_B4SOIbetaGB2;

d_T6_dV_gi_p =  0.0;
d_T6_dV_b_GND = ((T0*(T2*(d_T3_dV_b_GND-(T4*d_Vox_dV_b_GND+d_T4_dV_b_GND*Vox))+d_T2_dV_b_GND*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_b_GND)/T0/T0);
d_T6_dV_p_GND = ((T0*(T2*(d_T3_dV_p_GND-(T4*d_Vox_dV_p_GND+d_T4_dV_p_GND*Vox))+d_T2_dV_p_GND*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_p_GND)/T0/T0);
d_T6_dTemp_t_GND = ((T0*(T2*(d_T3_dTemp_t_GND-(T4*d_Vox_dTemp_t_GND+d_T4_dTemp_t_GND*Vox))+d_T2_dTemp_t_GND*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dTemp_t_GND)/T0/T0);
d_T6_dV_gi_si = ((T0*(T2*(d_T3_dV_gi_si-(T4*d_Vox_dV_gi_si+d_T4_dV_gi_si*Vox))+d_T2_dV_gi_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_gi_si)/T0/T0);
d_T6_dV_di_si = ((T0*(T2*(d_T3_dV_di_si-(T4*d_Vox_dV_di_si+d_T4_dV_di_si*Vox))+d_T2_dV_di_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_di_si)/T0/T0);
d_T6_dV_e_si = ((T0*(T2*(d_T3_dV_e_si-(T4*d_Vox_dV_e_si+d_T4_dV_e_si*Vox))+d_T2_dV_e_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_e_si)/T0/T0);
d_T6_dV_b_si = ((T0*(T2*(d_T3_dV_b_si-(T4*d_Vox_dV_b_si+d_T4_dV_b_si*Vox))+d_T2_dV_b_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_b_si)/T0/T0);
d_T6_dV_sb_si = ((T0*(T2*(d_T3_dV_sb_si-(T4*d_Vox_dV_sb_si+d_T4_dV_sb_si*Vox))+d_T2_dV_sb_si*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_sb_si)/T0/T0);
d_T6_dV_db_di = ((T0*(T2*(d_T3_dV_db_di-(T4*d_Vox_dV_db_di+d_T4_dV_db_di*Vox))+d_T2_dV_db_di*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dV_db_di)/T0/T0);
T6 = ((T2*(T3-(T4*Vox)))/T0);
if ((T6>100.0))
{

d_T5_dV_gi_p =  0.0;
d_T5_dV_b_GND = 2.688117142e+43*d_T6_dV_b_GND;
d_T5_dV_p_GND = 2.688117142e+43*d_T6_dV_p_GND;
d_T5_dTemp_t_GND = 2.688117142e+43*d_T6_dTemp_t_GND;
d_T5_dV_gi_si = 2.688117142e+43*d_T6_dV_gi_si;
d_T5_dV_di_si = 2.688117142e+43*d_T6_dV_di_si;
d_T5_dV_e_si = 2.688117142e+43*d_T6_dV_e_si;
d_T5_dV_b_si = 2.688117142e+43*d_T6_dV_b_si;
d_T5_dV_sb_si = 2.688117142e+43*d_T6_dV_sb_si;
d_T5_dV_db_di = 2.688117142e+43*d_T6_dV_db_di;
T5 = (2.688117142e+43*((1.0+T6)-100.0));
}
else
{
if ((T6<(-100.0)))
{

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si = d_T5_dV_b_si = d_T5_dV_e_si = d_T5_dV_di_si = d_T5_dV_gi_si = d_T5_dTemp_t_GND = d_T5_dV_p_GND = d_T5_dV_b_GND =  0.0;
T5 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T6);
double  deriv_exp_0_d0 = value_exp_0;

d_T5_dV_gi_p =  0.0;
d_T5_dV_b_GND = (deriv_exp_0_d0*(d_T6_dV_b_GND));
d_T5_dV_p_GND = (deriv_exp_0_d0*(d_T6_dV_p_GND));
d_T5_dTemp_t_GND = (deriv_exp_0_d0*(d_T6_dTemp_t_GND));
d_T5_dV_gi_si = (deriv_exp_0_d0*(d_T6_dV_gi_si));
d_T5_dV_di_si = (deriv_exp_0_d0*(d_T6_dV_di_si));
d_T5_dV_e_si = (deriv_exp_0_d0*(d_T6_dV_e_si));
d_T5_dV_b_si = (deriv_exp_0_d0*(d_T6_dV_b_si));
d_T5_dV_sb_si = (deriv_exp_0_d0*(d_T6_dV_sb_si));
d_T5_dV_db_di = (deriv_exp_0_d0*(d_T6_dV_db_di));
T5 = value_exp_0;
}
}
}

d_Igb2_dV_b_GND = (((T1*Vgb)*Vaux)*d_T5_dV_b_GND+((T1*Vgb)*d_Vaux_dV_b_GND+(T1*d_Vgb_dV_b_GND+d_T1_dV_b_GND*Vgb)*Vaux)*T5);
d_Igb2_dV_p_GND = (((T1*Vgb)*Vaux)*d_T5_dV_p_GND+((T1*Vgb)*d_Vaux_dV_p_GND+(T1*d_Vgb_dV_p_GND+d_T1_dV_p_GND*Vgb)*Vaux)*T5);
d_Igb2_dTemp_t_GND = (((T1*Vgb)*Vaux)*d_T5_dTemp_t_GND+((T1*Vgb)*d_Vaux_dTemp_t_GND+(T1*d_Vgb_dTemp_t_GND+d_T1_dTemp_t_GND*Vgb)*Vaux)*T5);
d_Igb2_dV_gi_si = (((T1*Vgb)*Vaux)*d_T5_dV_gi_si+((T1*Vgb)*d_Vaux_dV_gi_si+(T1*d_Vgb_dV_gi_si+d_T1_dV_gi_si*Vgb)*Vaux)*T5);
d_Igb2_dV_di_si = (((T1*Vgb)*Vaux)*d_T5_dV_di_si+((T1*Vgb)*d_Vaux_dV_di_si+(T1*d_Vgb_dV_di_si+d_T1_dV_di_si*Vgb)*Vaux)*T5);
d_Igb2_dV_e_si = (((T1*Vgb)*Vaux)*d_T5_dV_e_si+((T1*Vgb)*d_Vaux_dV_e_si+(T1*d_Vgb_dV_e_si+d_T1_dV_e_si*Vgb)*Vaux)*T5);
d_Igb2_dV_b_si = (((T1*Vgb)*Vaux)*d_T5_dV_b_si+((T1*Vgb)*d_Vaux_dV_b_si+(T1*d_Vgb_dV_b_si+d_T1_dV_b_si*Vgb)*Vaux)*T5);
d_Igb2_dV_sb_si = (((T1*Vgb)*Vaux)*d_T5_dV_sb_si+((T1*Vgb)*d_Vaux_dV_sb_si+d_T1_dV_sb_si*Vgb*Vaux)*T5);
d_Igb2_dV_db_di = (((T1*Vgb)*Vaux)*d_T5_dV_db_di+((T1*Vgb)*d_Vaux_dV_db_di+d_T1_dV_db_di*Vgb*Vaux)*T5);
Igb2 = (((T1*Vgb)*Vaux)*T5);
if ((Vgb>=0))
{

d_Igb_dV_b_GND = d_Igb1_dV_b_GND;
d_Igb_dV_p_GND = d_Igb1_dV_p_GND;
d_Igb_dTemp_t_GND = d_Igb1_dTemp_t_GND;
d_Igb_dV_gi_si = d_Igb1_dV_gi_si;
d_Igb_dV_di_si = d_Igb1_dV_di_si;
d_Igb_dV_e_si = d_Igb1_dV_e_si;
d_Igb_dV_b_si = d_Igb1_dV_b_si;
d_Igb_dV_sb_si = d_Igb1_dV_sb_si;
d_Igb_dV_db_di = d_Igb1_dV_db_di;
Igb = Igb1;
}
else
{

d_Igb_dV_b_GND = d_Igb2_dV_b_GND;
d_Igb_dV_p_GND = d_Igb2_dV_p_GND;
d_Igb_dTemp_t_GND = d_Igb2_dTemp_t_GND;
d_Igb_dV_gi_si = d_Igb2_dV_gi_si;
d_Igb_dV_di_si = d_Igb2_dV_di_si;
d_Igb_dV_e_si = d_Igb2_dV_e_si;
d_Igb_dV_b_si = d_Igb2_dV_b_si;
d_Igb_dV_sb_si = d_Igb2_dV_sb_si;
d_Igb_dV_db_di = d_Igb2_dV_db_di;
Igb = Igb2;
}

d_Vfb2_dV_b_si = d_Vfb2_dV_di_si = d_Vfb2_dV_gi_si = d_Vfb2_dV_e_si =  0.0;
d_Vfb2_dV_b_GND = d_Vfb_dV_b_GND;
d_Vfb2_dV_p_GND = d_Vfb_dV_p_GND;
d_Vfb2_dTemp_t_GND = d_Vfb_dTemp_t_GND;
Vfb2 = (Vfb+eggbcp2);
}
else
{

d_Igb_dV_db_di = d_Igb_dV_sb_si = d_Igb_dV_b_si = d_Igb_dV_e_si = d_Igb_dV_di_si = d_Igb_dV_gi_si = d_Igb_dTemp_t_GND = d_Igb_dV_p_GND = d_Igb_dV_b_GND =  0.0;
Igb = 0.0;
}

d_B4SOIig_dV_b_GND = B4SOItype*d_Igb_dV_b_GND;
d_B4SOIig_dV_p_GND = B4SOItype*d_Igb_dV_p_GND;
d_B4SOIig_dTemp_t_GND = B4SOItype*d_Igb_dTemp_t_GND;
d_B4SOIig_dV_gi_si = B4SOItype*d_Igb_dV_gi_si;
d_B4SOIig_dV_di_si = B4SOItype*d_Igb_dV_di_si;
d_B4SOIig_dV_e_si = B4SOItype*d_Igb_dV_e_si;
d_B4SOIig_dV_b_si = B4SOItype*d_Igb_dV_b_si;
d_B4SOIig_dV_sb_si = B4SOItype*d_Igb_dV_sb_si;
d_B4SOIig_dV_db_di = B4SOItype*d_Igb_dV_db_di;
B4SOIig = (B4SOItype*Igb);
if ((((((B4SOIigbMod!=0)&&(B4SOIsoiMod!=2))&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0))&&(vgp<Vfb2)))
{

d_T0_dV_gm_si = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si =  0.0;
d_T0_dV_b_GND = (-d_Vfb2_dV_b_GND);
d_T0_dV_p_GND = (-d_Vfb2_dV_p_GND);
d_T0_dTemp_t_GND = (-d_Vfb2_dTemp_t_GND);
d_T0_dV_gi_p = d_vgp_dV_gi_p;
T0 = (vgp-Vfb2);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)));
T1 = value_sqrt_0;
}

d_vgp_eff_dV_b_GND = 0.5*((-d_T0_dV_b_GND)+d_T1_dV_b_GND);
d_vgp_eff_dV_p_GND = 0.5*((-d_T0_dV_p_GND)+d_T1_dV_p_GND);
d_vgp_eff_dTemp_t_GND = 0.5*((-d_T0_dTemp_t_GND)+d_T1_dTemp_t_GND);
d_vgp_eff_dV_e_si = 0.5*((-d_T0_dV_e_si)+d_T1_dV_e_si);
d_vgp_eff_dV_gi_si = 0.5*((-d_T0_dV_gi_si)+d_T1_dV_gi_si);
d_vgp_eff_dV_di_si = 0.5*((-d_T0_dV_di_si)+d_T1_dV_di_si);
d_vgp_eff_dV_b_si = 0.5*((-d_T0_dV_b_si)+d_T1_dV_b_si);
d_vgp_eff_dV_sb_si = 0.5*((-d_T0_dV_sb_si)+d_T1_dV_sb_si);
d_vgp_eff_dV_db_di = 0.5*((-d_T0_dV_db_di)+d_T1_dV_db_di);
d_vgp_eff_dV_gi_p = 0.5*((-d_T0_dV_gi_p)+d_T1_dV_gi_p);
vgp_eff = (0.5*(((-T0)+T1)-1.0e-2));

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_e_si = d_T11_dV_di_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
T11 = ((B4SOItype==1)?agbc2n:agbc2p);

d_T12_dV_gi_p = d_T12_dV_db_di = d_T12_dV_sb_si = d_T12_dV_b_si = d_T12_dV_e_si = d_T12_dV_di_si = d_T12_dV_gi_si = d_T12_dTemp_t_GND = d_T12_dV_p_GND = d_T12_dV_b_GND =  0.0;
T12 = ((B4SOItype==1)?bgbc2n:bgbc2p);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = vgp*d_vgp_eff_dV_b_GND;
d_T2_dV_p_GND = vgp*d_vgp_eff_dV_p_GND;
d_T2_dTemp_t_GND = vgp*d_vgp_eff_dTemp_t_GND;
d_T2_dV_e_si = vgp*d_vgp_eff_dV_e_si;
d_T2_dV_gi_si = vgp*d_vgp_eff_dV_gi_si;
d_T2_dV_di_si = vgp*d_vgp_eff_dV_di_si;
d_T2_dV_b_si = vgp*d_vgp_eff_dV_b_si;
d_T2_dV_sb_si = vgp*d_vgp_eff_dV_sb_si;
d_T2_dV_db_di = vgp*d_vgp_eff_dV_db_di;
d_T2_dV_gi_p = (vgp*d_vgp_eff_dV_gi_p+d_vgp_dV_gi_p*vgp_eff);
T2 = (vgp*vgp_eff);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = ((pParam_B4SOIaigbcp2*pParam_B4SOIcigbcp2)-pParam_B4SOIbigbcp2);

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = (pParam_B4SOIbigbcp2*pParam_B4SOIcigbcp2);

d_T5_dV_gi_p = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_gi_p+d_T3_dV_gi_p*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_gi_p+(T4*d_vgp_eff_dV_gi_p+d_T4_dV_gi_p*vgp_eff)*vgp_eff))+(-d_T12_dV_gi_p)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_sb_si = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_sb_si+d_T3_dV_sb_si*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_sb_si+(T4*d_vgp_eff_dV_sb_si+d_T4_dV_sb_si*vgp_eff)*vgp_eff))+(-d_T12_dV_sb_si)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_db_di = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_db_di+d_T3_dV_db_di*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_db_di+(T4*d_vgp_eff_dV_db_di+d_T4_dV_db_di*vgp_eff)*vgp_eff))+(-d_T12_dV_db_di)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_b_GND = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_b_GND+d_T3_dV_b_GND*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_b_GND+(T4*d_vgp_eff_dV_b_GND+d_T4_dV_b_GND*vgp_eff)*vgp_eff))+(-d_T12_dV_b_GND)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_p_GND = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_p_GND+d_T3_dV_p_GND*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_p_GND+(T4*d_vgp_eff_dV_p_GND+d_T4_dV_p_GND*vgp_eff)*vgp_eff))+(-d_T12_dV_p_GND)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dTemp_t_GND = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dTemp_t_GND+d_T3_dTemp_t_GND*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dTemp_t_GND+(T4*d_vgp_eff_dTemp_t_GND+d_T4_dTemp_t_GND*vgp_eff)*vgp_eff))+(-d_T12_dTemp_t_GND)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_gi_si = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_gi_si+d_T3_dV_gi_si*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_gi_si+(T4*d_vgp_eff_dV_gi_si+d_T4_dV_gi_si*vgp_eff)*vgp_eff))+(-d_T12_dV_gi_si)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_di_si = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_di_si+d_T3_dV_di_si*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_di_si+(T4*d_vgp_eff_dV_di_si+d_T4_dV_di_si*vgp_eff)*vgp_eff))+(-d_T12_dV_di_si)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_e_si = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_e_si+d_T3_dV_e_si*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_e_si+(T4*d_vgp_eff_dV_e_si+d_T4_dV_e_si*vgp_eff)*vgp_eff))+(-d_T12_dV_e_si)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
d_T5_dV_b_si = (((-T12)*B4SOItoxqm)*((T3*d_vgp_eff_dV_b_si+d_T3_dV_b_si*vgp_eff)-((T4*vgp_eff)*d_vgp_eff_dV_b_si+(T4*d_vgp_eff_dV_b_si+d_T4_dV_b_si*vgp_eff)*vgp_eff))+(-d_T12_dV_b_si)*B4SOItoxqm*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
T5 = (((-T12)*B4SOItoxqm)*((pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
if ((T5>100.0))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{

d_T6_dV_gi_p = d_T6_dV_db_di = d_T6_dV_sb_si = d_T6_dV_b_si = d_T6_dV_e_si = d_T6_dV_di_si = d_T6_dV_gi_si = d_T6_dTemp_t_GND = d_T6_dV_p_GND = d_T6_dV_b_GND =  0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;

d_T6_dV_b_GND = (deriv_exp_0_d0*(d_T5_dV_b_GND));
d_T6_dV_p_GND = (deriv_exp_0_d0*(d_T5_dV_p_GND));
d_T6_dTemp_t_GND = (deriv_exp_0_d0*(d_T5_dTemp_t_GND));
d_T6_dV_gi_si = (deriv_exp_0_d0*(d_T5_dV_gi_si));
d_T6_dV_di_si = (deriv_exp_0_d0*(d_T5_dV_di_si));
d_T6_dV_e_si = (deriv_exp_0_d0*(d_T5_dV_e_si));
d_T6_dV_b_si = (deriv_exp_0_d0*(d_T5_dV_b_si));
d_T6_dV_sb_si = (deriv_exp_0_d0*(d_T5_dV_sb_si));
d_T6_dV_db_di = (deriv_exp_0_d0*(d_T5_dV_db_di));
d_T6_dV_gi_p = (deriv_exp_0_d0*(d_T5_dV_gi_p));
T6 = value_exp_0;
}
}
}

d_T11_dV_gi_p =  0.0;
d_T11_dV_b_GND = d_T11_dV_b_GND*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dV_p_GND = d_T11_dV_p_GND*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dTemp_t_GND = d_T11_dTemp_t_GND*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dV_gi_si = d_T11_dV_gi_si*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dV_di_si = d_T11_dV_di_si*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dV_e_si = d_T11_dV_e_si*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dV_b_si = d_T11_dV_b_si*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dV_sb_si = d_T11_dV_sb_si*B4SOIagbcp2*pParam_B4SOIoxideRatio;
d_T11_dV_db_di = d_T11_dV_db_di*B4SOIagbcp2*pParam_B4SOIoxideRatio;
T11 = ((T11*B4SOIagbcp2)*pParam_B4SOIoxideRatio);

d_Ig_agbcp2_dV_gi_p = ((T11*T2)*d_T6_dV_gi_p+(T11*d_T2_dV_gi_p+d_T11_dV_gi_p*T2)*T6);
d_Ig_agbcp2_dV_b_GND = ((T11*T2)*d_T6_dV_b_GND+(T11*d_T2_dV_b_GND+d_T11_dV_b_GND*T2)*T6);
d_Ig_agbcp2_dV_p_GND = ((T11*T2)*d_T6_dV_p_GND+(T11*d_T2_dV_p_GND+d_T11_dV_p_GND*T2)*T6);
d_Ig_agbcp2_dTemp_t_GND = ((T11*T2)*d_T6_dTemp_t_GND+(T11*d_T2_dTemp_t_GND+d_T11_dTemp_t_GND*T2)*T6);
d_Ig_agbcp2_dV_gi_si = ((T11*T2)*d_T6_dV_gi_si+(T11*d_T2_dV_gi_si+d_T11_dV_gi_si*T2)*T6);
d_Ig_agbcp2_dV_di_si = ((T11*T2)*d_T6_dV_di_si+(T11*d_T2_dV_di_si+d_T11_dV_di_si*T2)*T6);
d_Ig_agbcp2_dV_e_si = ((T11*T2)*d_T6_dV_e_si+(T11*d_T2_dV_e_si+d_T11_dV_e_si*T2)*T6);
d_Ig_agbcp2_dV_b_si = ((T11*T2)*d_T6_dV_b_si+(T11*d_T2_dV_b_si+d_T11_dV_b_si*T2)*T6);
d_Ig_agbcp2_dV_sb_si = ((T11*T2)*d_T6_dV_sb_si+(T11*d_T2_dV_sb_si+d_T11_dV_sb_si*T2)*T6);
d_Ig_agbcp2_dV_db_di = ((T11*T2)*d_T6_dV_db_di+(T11*d_T2_dV_db_di+d_T11_dV_db_di*T2)*T6);
Ig_agbcp2 = ((T11*T2)*T6);
}
else
{

d_Ig_agbcp2_dV_db_di = d_Ig_agbcp2_dV_sb_si = d_Ig_agbcp2_dV_b_si = d_Ig_agbcp2_dV_e_si = d_Ig_agbcp2_dV_di_si = d_Ig_agbcp2_dV_gi_si = d_Ig_agbcp2_dTemp_t_GND = d_Ig_agbcp2_dV_p_GND = d_Ig_agbcp2_dV_b_GND = d_Ig_agbcp2_dV_gi_p =  0.0;
Ig_agbcp2 = 0.0;
}

d_B4SOIigp_dV_gi_p = B4SOItype*d_Ig_agbcp2_dV_gi_p;
d_B4SOIigp_dV_b_GND = B4SOItype*d_Ig_agbcp2_dV_b_GND;
d_B4SOIigp_dV_p_GND = B4SOItype*d_Ig_agbcp2_dV_p_GND;
d_B4SOIigp_dTemp_t_GND = B4SOItype*d_Ig_agbcp2_dTemp_t_GND;
d_B4SOIigp_dV_gi_si = B4SOItype*d_Ig_agbcp2_dV_gi_si;
d_B4SOIigp_dV_di_si = B4SOItype*d_Ig_agbcp2_dV_di_si;
d_B4SOIigp_dV_e_si = B4SOItype*d_Ig_agbcp2_dV_e_si;
d_B4SOIigp_dV_b_si = B4SOItype*d_Ig_agbcp2_dV_b_si;
d_B4SOIigp_dV_sb_si = B4SOItype*d_Ig_agbcp2_dV_sb_si;
d_B4SOIigp_dV_db_di = B4SOItype*d_Ig_agbcp2_dV_db_di;
B4SOIigp = (B4SOItype*Ig_agbcp2);
if ((B4SOIsoiMod!=2))
{
if ((B4SOIiiiMod==0))
{
if ((pParam_B4SOIalpha0<=0.0))
{

d_Iii_dV_di_si = d_Iii_dTemp_t_GND = d_Iii_dV_p_GND = d_Iii_dV_b_GND = d_Iii_dV_gi_p = d_Iii_dV_db_di = d_Iii_dV_sb_si = d_Iii_dV_b_si = d_Iii_dV_e_si = d_Iii_dV_gi_si =  0.0;
Iii = 0.0;
}
else
{

d_Vdsatii0_dV_b_GND = pParam_B4SOIvdsatii0*B4SOItii*d_TempRatio_dV_b_GND;
d_Vdsatii0_dV_p_GND = pParam_B4SOIvdsatii0*B4SOItii*d_TempRatio_dV_p_GND;
d_Vdsatii0_dTemp_t_GND = pParam_B4SOIvdsatii0*B4SOItii*d_TempRatio_dTemp_t_GND;
Vdsatii0 = ((pParam_B4SOIvdsatii0*(1+(B4SOItii*(TempRatio-1.0))))-(pParam_B4SOIlii/Leff));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (pParam_B4SOIesatii*Leff);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_b_GND-(pParam_B4SOIsii0*T0)*d_T0_dV_b_GND)/(1.0+T0)/(1.0+T0));
d_T1_dV_p_GND = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_p_GND-(pParam_B4SOIsii0*T0)*d_T0_dV_p_GND)/(1.0+T0)/(1.0+T0));
d_T1_dTemp_t_GND = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dTemp_t_GND-(pParam_B4SOIsii0*T0)*d_T0_dTemp_t_GND)/(1.0+T0)/(1.0+T0));
d_T1_dV_e_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_e_si-(pParam_B4SOIsii0*T0)*d_T0_dV_e_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_gi_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_gi_si-(pParam_B4SOIsii0*T0)*d_T0_dV_gi_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_di_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_di_si-(pParam_B4SOIsii0*T0)*d_T0_dV_di_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_b_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_b_si-(pParam_B4SOIsii0*T0)*d_T0_dV_b_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_sb_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_sb_si-(pParam_B4SOIsii0*T0)*d_T0_dV_sb_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_db_di = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_db_di-(pParam_B4SOIsii0*T0)*d_T0_dV_db_di)/(1.0+T0)/(1.0+T0));
d_T1_dV_gi_p = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_gi_p-(pParam_B4SOIsii0*T0)*d_T0_dV_gi_p)/(1.0+T0)/(1.0+T0));
T1 = ((pParam_B4SOIsii0*T0)/(1.0+T0));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_e_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_b_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_b_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_b_GND = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_b_GND/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_p_GND = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_p_GND/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dTemp_t_GND = (-1*pParam_B4SOIsii1*d_Vgsteff_dTemp_t_GND/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_gi_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_gi_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_di_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_di_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
T0 = (1/(1+(pParam_B4SOIsii1*Vgsteff)));

d_T3_dV_b_GND = d_T0_dV_b_GND;
d_T3_dV_p_GND = d_T0_dV_p_GND;
d_T3_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T3_dV_e_si = d_T0_dV_e_si;
d_T3_dV_gi_si = d_T0_dV_gi_si;
d_T3_dV_di_si = d_T0_dV_di_si;
d_T3_dV_b_si = d_T0_dV_b_si;
d_T3_dV_sb_si = d_T0_dV_sb_si;
d_T3_dV_db_di = d_T0_dV_db_di;
d_T3_dV_gi_p = d_T0_dV_gi_p;
T3 = (T0+pParam_B4SOIsii2);

d_T2_dV_gm_si =  0.0;
d_T2_dV_sb_si = Vgst*d_T3_dV_sb_si;
d_T2_dV_db_di = Vgst*d_T3_dV_db_di;
d_T2_dV_gi_p = Vgst*d_T3_dV_gi_p;
d_T2_dV_e_si = (Vgst*d_T3_dV_e_si+d_Vgst_dV_e_si*T3);
d_T2_dV_b_si = (Vgst*d_T3_dV_b_si+d_Vgst_dV_b_si*T3);
d_T2_dV_b_GND = (Vgst*d_T3_dV_b_GND+d_Vgst_dV_b_GND*T3);
d_T2_dV_p_GND = (Vgst*d_T3_dV_p_GND+d_Vgst_dV_p_GND*T3);
d_T2_dTemp_t_GND = (Vgst*d_T3_dTemp_t_GND+d_Vgst_dTemp_t_GND*T3);
d_T2_dV_gi_si = (Vgst*d_T3_dV_gi_si+d_Vgst_dV_gi_si*T3);
d_T2_dV_di_si = (Vgst*d_T3_dV_di_si+d_Vgst_dV_di_si*T3);
T2 = (Vgst*T3);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
d_T3_dV_di_si = (-1*pParam_B4SOIsiid*d_Vds_dV_di_si/(1+(pParam_B4SOIsiid*Vds))/(1+(pParam_B4SOIsiid*Vds)));
T3 = (1/(1+(pParam_B4SOIsiid*Vds)));

d_VgsStep_dV_b_GND = ((T1*T2)*d_T3_dV_b_GND+(T1*d_T2_dV_b_GND+d_T1_dV_b_GND*T2)*T3);
d_VgsStep_dV_p_GND = ((T1*T2)*d_T3_dV_p_GND+(T1*d_T2_dV_p_GND+d_T1_dV_p_GND*T2)*T3);
d_VgsStep_dTemp_t_GND = ((T1*T2)*d_T3_dTemp_t_GND+(T1*d_T2_dTemp_t_GND+d_T1_dTemp_t_GND*T2)*T3);
d_VgsStep_dV_gi_si = ((T1*T2)*d_T3_dV_gi_si+(T1*d_T2_dV_gi_si+d_T1_dV_gi_si*T2)*T3);
d_VgsStep_dV_di_si = ((T1*T2)*d_T3_dV_di_si+(T1*d_T2_dV_di_si+d_T1_dV_di_si*T2)*T3);
d_VgsStep_dV_e_si = ((T1*T2)*d_T3_dV_e_si+(T1*d_T2_dV_e_si+d_T1_dV_e_si*T2)*T3);
d_VgsStep_dV_b_si = ((T1*T2)*d_T3_dV_b_si+(T1*d_T2_dV_b_si+d_T1_dV_b_si*T2)*T3);
d_VgsStep_dV_sb_si = ((T1*T2)*d_T3_dV_sb_si+(T1*d_T2_dV_sb_si+d_T1_dV_sb_si*T2)*T3);
d_VgsStep_dV_db_di = ((T1*T2)*d_T3_dV_db_di+(T1*d_T2_dV_db_di+d_T1_dV_db_di*T2)*T3);
d_VgsStep_dV_gi_p = ((T1*T2)*d_T3_dV_gi_p+(T1*d_T2_dV_gi_p+d_T1_dV_gi_p*T2)*T3);
VgsStep = ((T1*T2)*T3);

d_Vdsatii_dV_gi_si = d_VgsStep_dV_gi_si;
d_Vdsatii_dV_di_si = d_VgsStep_dV_di_si;
d_Vdsatii_dV_e_si = d_VgsStep_dV_e_si;
d_Vdsatii_dV_b_si = d_VgsStep_dV_b_si;
d_Vdsatii_dV_sb_si = d_VgsStep_dV_sb_si;
d_Vdsatii_dV_db_di = d_VgsStep_dV_db_di;
d_Vdsatii_dV_gi_p = d_VgsStep_dV_gi_p;
d_Vdsatii_dV_b_GND = (d_Vdsatii0_dV_b_GND+d_VgsStep_dV_b_GND);
d_Vdsatii_dV_p_GND = (d_Vdsatii0_dV_p_GND+d_VgsStep_dV_p_GND);
d_Vdsatii_dTemp_t_GND = (d_Vdsatii0_dTemp_t_GND+d_VgsStep_dTemp_t_GND);
Vdsatii = (Vdsatii0+VgsStep);

d_Vdiff_dV_gi_si = (-d_Vdsatii_dV_gi_si);
d_Vdiff_dV_e_si = (-d_Vdsatii_dV_e_si);
d_Vdiff_dV_b_si = (-d_Vdsatii_dV_b_si);
d_Vdiff_dV_sb_si = (-d_Vdsatii_dV_sb_si);
d_Vdiff_dV_db_di = (-d_Vdsatii_dV_db_di);
d_Vdiff_dV_gi_p = (-d_Vdsatii_dV_gi_p);
d_Vdiff_dV_b_GND = (-d_Vdsatii_dV_b_GND);
d_Vdiff_dV_p_GND = (-d_Vdsatii_dV_p_GND);
d_Vdiff_dTemp_t_GND = (-d_Vdsatii_dTemp_t_GND);
d_Vdiff_dV_di_si = (d_Vds_dV_di_si-d_Vdsatii_dV_di_si);
Vdiff = (Vds-Vdsatii);

d_T0_dV_gm_si =  0.0;
d_T0_dV_gi_si = (pParam_B4SOIbeta1*d_Vdiff_dV_gi_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_gi_si+pParam_B4SOIbeta0*d_Vdiff_dV_gi_si*Vdiff));
d_T0_dV_e_si = (pParam_B4SOIbeta1*d_Vdiff_dV_e_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_e_si+pParam_B4SOIbeta0*d_Vdiff_dV_e_si*Vdiff));
d_T0_dV_b_si = (pParam_B4SOIbeta1*d_Vdiff_dV_b_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_b_si+pParam_B4SOIbeta0*d_Vdiff_dV_b_si*Vdiff));
d_T0_dV_sb_si = (pParam_B4SOIbeta1*d_Vdiff_dV_sb_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_sb_si+pParam_B4SOIbeta0*d_Vdiff_dV_sb_si*Vdiff));
d_T0_dV_db_di = (pParam_B4SOIbeta1*d_Vdiff_dV_db_di+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_db_di+pParam_B4SOIbeta0*d_Vdiff_dV_db_di*Vdiff));
d_T0_dV_gi_p = (pParam_B4SOIbeta1*d_Vdiff_dV_gi_p+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_gi_p+pParam_B4SOIbeta0*d_Vdiff_dV_gi_p*Vdiff));
d_T0_dV_b_GND = (pParam_B4SOIbeta1*d_Vdiff_dV_b_GND+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_b_GND+pParam_B4SOIbeta0*d_Vdiff_dV_b_GND*Vdiff));
d_T0_dV_p_GND = (pParam_B4SOIbeta1*d_Vdiff_dV_p_GND+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_p_GND+pParam_B4SOIbeta0*d_Vdiff_dV_p_GND*Vdiff));
d_T0_dTemp_t_GND = (pParam_B4SOIbeta1*d_Vdiff_dTemp_t_GND+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dTemp_t_GND+pParam_B4SOIbeta0*d_Vdiff_dTemp_t_GND*Vdiff));
d_T0_dV_di_si = (pParam_B4SOIbeta1*d_Vdiff_dV_di_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_di_si+pParam_B4SOIbeta0*d_Vdiff_dV_di_si*Vdiff));
T0 = ((pParam_B4SOIbeta2+(pParam_B4SOIbeta1*Vdiff))+((pParam_B4SOIbeta0*Vdiff)*Vdiff));
if ((T0<1e-5))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 1e-5;
}
if (((T0<(Vdiff/100.0))&&(Vdiff>0.0)))
{

d_Ratio_dV_di_si = d_Ratio_dTemp_t_GND = d_Ratio_dV_p_GND = d_Ratio_dV_b_GND = d_Ratio_dV_gi_p = d_Ratio_dV_db_di = d_Ratio_dV_sb_si = d_Ratio_dV_b_si = d_Ratio_dV_e_si = d_Ratio_dV_gi_si =  0.0;
Ratio = (pParam_B4SOIalpha0*2.688117142e+43);
}
else
{
if (((T0<((-Vdiff)/100.0))&&(Vdiff<0.0)))
{

d_Ratio_dV_di_si = d_Ratio_dTemp_t_GND = d_Ratio_dV_p_GND = d_Ratio_dV_b_GND = d_Ratio_dV_gi_p = d_Ratio_dV_db_di = d_Ratio_dV_sb_si = d_Ratio_dV_b_si = d_Ratio_dV_e_si = d_Ratio_dV_gi_si =  0.0;
Ratio = (pParam_B4SOIalpha0*3.720075976e-44);
}
else
{
{
double value_exp_0 = exp((Vdiff/T0));
double  deriv_exp_0_d0 = value_exp_0;

d_Ratio_dV_gi_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_gi_si-Vdiff*d_T0_dV_gi_si)/T0/T0)));
d_Ratio_dV_e_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_e_si-Vdiff*d_T0_dV_e_si)/T0/T0)));
d_Ratio_dV_b_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_b_si-Vdiff*d_T0_dV_b_si)/T0/T0)));
d_Ratio_dV_sb_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_sb_si-Vdiff*d_T0_dV_sb_si)/T0/T0)));
d_Ratio_dV_db_di = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_db_di-Vdiff*d_T0_dV_db_di)/T0/T0)));
d_Ratio_dV_gi_p = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_gi_p-Vdiff*d_T0_dV_gi_p)/T0/T0)));
d_Ratio_dV_b_GND = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_b_GND-Vdiff*d_T0_dV_b_GND)/T0/T0)));
d_Ratio_dV_p_GND = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_p_GND-Vdiff*d_T0_dV_p_GND)/T0/T0)));
d_Ratio_dTemp_t_GND = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dTemp_t_GND-Vdiff*d_T0_dTemp_t_GND)/T0/T0)));
d_Ratio_dV_di_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_di_si-Vdiff*d_T0_dV_di_si)/T0/T0)));
Ratio = (pParam_B4SOIalpha0*value_exp_0);
}
}
}
if ((Ratio>10.0))
{

d_Ratio_dV_di_si = d_Ratio_dTemp_t_GND = d_Ratio_dV_p_GND = d_Ratio_dV_b_GND = d_Ratio_dV_gi_p = d_Ratio_dV_db_di = d_Ratio_dV_sb_si = d_Ratio_dV_b_si = d_Ratio_dV_e_si = d_Ratio_dV_gi_si =  0.0;
Ratio = 10.0;
}

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_sb_si = (pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_sb_si;
d_T0_dV_db_di = (pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_db_di;
d_T0_dV_gi_si = (d_Ids_dV_gi_si+(pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_gi_si);
d_T0_dV_di_si = (d_Ids_dV_di_si+(pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_di_si);
d_T0_dV_e_si = (d_Ids_dV_e_si+(pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_e_si);
d_T0_dV_b_si = (d_Ids_dV_b_si+(pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_b_si);
d_T0_dV_b_GND = (d_Ids_dV_b_GND+(pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_b_GND);
d_T0_dV_p_GND = (d_Ids_dV_p_GND+(pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dV_p_GND);
d_T0_dTemp_t_GND = (d_Ids_dTemp_t_GND+(pParam_B4SOIfbjtii*B4SOImode)*d_Ic_dTemp_t_GND);
T0 = (Ids+((pParam_B4SOIfbjtii*B4SOImode)*Ic));

d_Iii_dV_gi_si = (Ratio*d_T0_dV_gi_si+d_Ratio_dV_gi_si*T0);
d_Iii_dV_e_si = (Ratio*d_T0_dV_e_si+d_Ratio_dV_e_si*T0);
d_Iii_dV_b_si = (Ratio*d_T0_dV_b_si+d_Ratio_dV_b_si*T0);
d_Iii_dV_sb_si = (Ratio*d_T0_dV_sb_si+d_Ratio_dV_sb_si*T0);
d_Iii_dV_db_di = (Ratio*d_T0_dV_db_di+d_Ratio_dV_db_di*T0);
d_Iii_dV_gi_p = (Ratio*d_T0_dV_gi_p+d_Ratio_dV_gi_p*T0);
d_Iii_dV_b_GND = (Ratio*d_T0_dV_b_GND+d_Ratio_dV_b_GND*T0);
d_Iii_dV_p_GND = (Ratio*d_T0_dV_p_GND+d_Ratio_dV_p_GND*T0);
d_Iii_dTemp_t_GND = (Ratio*d_T0_dTemp_t_GND+d_Ratio_dTemp_t_GND*T0);
d_Iii_dV_di_si = (Ratio*d_T0_dV_di_si+d_Ratio_dV_di_si*T0);
Iii = (Ratio*T0);
}
}
else
{
if ((pParam_B4SOIalpha0<=0.0))
{

d_Idsmosfet_dV_di_si = d_Idsmosfet_dTemp_t_GND = d_Idsmosfet_dV_p_GND = d_Idsmosfet_dV_b_GND = d_Idsmosfet_dV_gi_p = d_Idsmosfet_dV_db_di = d_Idsmosfet_dV_sb_si = d_Idsmosfet_dV_b_si = d_Idsmosfet_dV_e_si = d_Idsmosfet_dV_gi_si =  0.0;
Idsmosfet = 0.0;
}
else
{

d_Vdsatii0_dV_b_GND = pParam_B4SOIvdsatii0*B4SOItii*d_TempRatio_dV_b_GND;
d_Vdsatii0_dV_p_GND = pParam_B4SOIvdsatii0*B4SOItii*d_TempRatio_dV_p_GND;
d_Vdsatii0_dTemp_t_GND = pParam_B4SOIvdsatii0*B4SOItii*d_TempRatio_dTemp_t_GND;
Vdsatii0 = ((pParam_B4SOIvdsatii0*(1+(B4SOItii*(TempRatio-1.0))))-(pParam_B4SOIlii/Leff));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (pParam_B4SOIesatii*Leff);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_b_GND-(pParam_B4SOIsii0*T0)*d_T0_dV_b_GND)/(1.0+T0)/(1.0+T0));
d_T1_dV_p_GND = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_p_GND-(pParam_B4SOIsii0*T0)*d_T0_dV_p_GND)/(1.0+T0)/(1.0+T0));
d_T1_dTemp_t_GND = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dTemp_t_GND-(pParam_B4SOIsii0*T0)*d_T0_dTemp_t_GND)/(1.0+T0)/(1.0+T0));
d_T1_dV_e_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_e_si-(pParam_B4SOIsii0*T0)*d_T0_dV_e_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_gi_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_gi_si-(pParam_B4SOIsii0*T0)*d_T0_dV_gi_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_di_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_di_si-(pParam_B4SOIsii0*T0)*d_T0_dV_di_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_b_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_b_si-(pParam_B4SOIsii0*T0)*d_T0_dV_b_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_sb_si = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_sb_si-(pParam_B4SOIsii0*T0)*d_T0_dV_sb_si)/(1.0+T0)/(1.0+T0));
d_T1_dV_db_di = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_db_di-(pParam_B4SOIsii0*T0)*d_T0_dV_db_di)/(1.0+T0)/(1.0+T0));
d_T1_dV_gi_p = (((1.0+T0)*pParam_B4SOIsii0*d_T0_dV_gi_p-(pParam_B4SOIsii0*T0)*d_T0_dV_gi_p)/(1.0+T0)/(1.0+T0));
T1 = ((pParam_B4SOIsii0*T0)/(1.0+T0));

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_e_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_b_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_b_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_b_GND = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_b_GND/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_p_GND = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_p_GND/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dTemp_t_GND = (-1*pParam_B4SOIsii1*d_Vgsteff_dTemp_t_GND/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_gi_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_gi_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
d_T0_dV_di_si = (-1*pParam_B4SOIsii1*d_Vgsteff_dV_di_si/(1+(pParam_B4SOIsii1*Vgsteff))/(1+(pParam_B4SOIsii1*Vgsteff)));
T0 = (1/(1+(pParam_B4SOIsii1*Vgsteff)));

d_T3_dV_b_GND = d_T0_dV_b_GND;
d_T3_dV_p_GND = d_T0_dV_p_GND;
d_T3_dTemp_t_GND = d_T0_dTemp_t_GND;
d_T3_dV_e_si = d_T0_dV_e_si;
d_T3_dV_gi_si = d_T0_dV_gi_si;
d_T3_dV_di_si = d_T0_dV_di_si;
d_T3_dV_b_si = d_T0_dV_b_si;
d_T3_dV_sb_si = d_T0_dV_sb_si;
d_T3_dV_db_di = d_T0_dV_db_di;
d_T3_dV_gi_p = d_T0_dV_gi_p;
T3 = (T0+pParam_B4SOIsii2);

d_T2_dV_gm_si =  0.0;
d_T2_dV_sb_si = Vgst*d_T3_dV_sb_si;
d_T2_dV_db_di = Vgst*d_T3_dV_db_di;
d_T2_dV_gi_p = Vgst*d_T3_dV_gi_p;
d_T2_dV_e_si = (Vgst*d_T3_dV_e_si+d_Vgst_dV_e_si*T3);
d_T2_dV_b_si = (Vgst*d_T3_dV_b_si+d_Vgst_dV_b_si*T3);
d_T2_dV_b_GND = (Vgst*d_T3_dV_b_GND+d_Vgst_dV_b_GND*T3);
d_T2_dV_p_GND = (Vgst*d_T3_dV_p_GND+d_Vgst_dV_p_GND*T3);
d_T2_dTemp_t_GND = (Vgst*d_T3_dTemp_t_GND+d_Vgst_dTemp_t_GND*T3);
d_T2_dV_gi_si = (Vgst*d_T3_dV_gi_si+d_Vgst_dV_gi_si*T3);
d_T2_dV_di_si = (Vgst*d_T3_dV_di_si+d_Vgst_dV_di_si*T3);
T2 = (Vgst*T3);

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
d_T3_dV_di_si = (-1*pParam_B4SOIsiid*d_Vds_dV_di_si/(1+(pParam_B4SOIsiid*Vds))/(1+(pParam_B4SOIsiid*Vds)));
T3 = (1/(1+(pParam_B4SOIsiid*Vds)));

d_VgsStep_dV_b_GND = ((T1*T2)*d_T3_dV_b_GND+(T1*d_T2_dV_b_GND+d_T1_dV_b_GND*T2)*T3);
d_VgsStep_dV_p_GND = ((T1*T2)*d_T3_dV_p_GND+(T1*d_T2_dV_p_GND+d_T1_dV_p_GND*T2)*T3);
d_VgsStep_dTemp_t_GND = ((T1*T2)*d_T3_dTemp_t_GND+(T1*d_T2_dTemp_t_GND+d_T1_dTemp_t_GND*T2)*T3);
d_VgsStep_dV_gi_si = ((T1*T2)*d_T3_dV_gi_si+(T1*d_T2_dV_gi_si+d_T1_dV_gi_si*T2)*T3);
d_VgsStep_dV_di_si = ((T1*T2)*d_T3_dV_di_si+(T1*d_T2_dV_di_si+d_T1_dV_di_si*T2)*T3);
d_VgsStep_dV_e_si = ((T1*T2)*d_T3_dV_e_si+(T1*d_T2_dV_e_si+d_T1_dV_e_si*T2)*T3);
d_VgsStep_dV_b_si = ((T1*T2)*d_T3_dV_b_si+(T1*d_T2_dV_b_si+d_T1_dV_b_si*T2)*T3);
d_VgsStep_dV_sb_si = ((T1*T2)*d_T3_dV_sb_si+(T1*d_T2_dV_sb_si+d_T1_dV_sb_si*T2)*T3);
d_VgsStep_dV_db_di = ((T1*T2)*d_T3_dV_db_di+(T1*d_T2_dV_db_di+d_T1_dV_db_di*T2)*T3);
d_VgsStep_dV_gi_p = ((T1*T2)*d_T3_dV_gi_p+(T1*d_T2_dV_gi_p+d_T1_dV_gi_p*T2)*T3);
VgsStep = ((T1*T2)*T3);

d_Vdsatii_dV_gi_si = d_VgsStep_dV_gi_si;
d_Vdsatii_dV_di_si = d_VgsStep_dV_di_si;
d_Vdsatii_dV_e_si = d_VgsStep_dV_e_si;
d_Vdsatii_dV_b_si = d_VgsStep_dV_b_si;
d_Vdsatii_dV_sb_si = d_VgsStep_dV_sb_si;
d_Vdsatii_dV_db_di = d_VgsStep_dV_db_di;
d_Vdsatii_dV_gi_p = d_VgsStep_dV_gi_p;
d_Vdsatii_dV_b_GND = (d_Vdsatii0_dV_b_GND+d_VgsStep_dV_b_GND);
d_Vdsatii_dV_p_GND = (d_Vdsatii0_dV_p_GND+d_VgsStep_dV_p_GND);
d_Vdsatii_dTemp_t_GND = (d_Vdsatii0_dTemp_t_GND+d_VgsStep_dTemp_t_GND);
Vdsatii = (Vdsatii0+VgsStep);

d_Vdiff_dV_gi_si = (-d_Vdsatii_dV_gi_si);
d_Vdiff_dV_e_si = (-d_Vdsatii_dV_e_si);
d_Vdiff_dV_b_si = (-d_Vdsatii_dV_b_si);
d_Vdiff_dV_sb_si = (-d_Vdsatii_dV_sb_si);
d_Vdiff_dV_db_di = (-d_Vdsatii_dV_db_di);
d_Vdiff_dV_gi_p = (-d_Vdsatii_dV_gi_p);
d_Vdiff_dV_b_GND = (-d_Vdsatii_dV_b_GND);
d_Vdiff_dV_p_GND = (-d_Vdsatii_dV_p_GND);
d_Vdiff_dTemp_t_GND = (-d_Vdsatii_dTemp_t_GND);
d_Vdiff_dV_di_si = (d_Vds_dV_di_si-d_Vdsatii_dV_di_si);
Vdiff = (Vds-Vdsatii);

d_T0_dV_gm_si =  0.0;
d_T0_dV_gi_si = (pParam_B4SOIbeta1*d_Vdiff_dV_gi_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_gi_si+pParam_B4SOIbeta0*d_Vdiff_dV_gi_si*Vdiff));
d_T0_dV_e_si = (pParam_B4SOIbeta1*d_Vdiff_dV_e_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_e_si+pParam_B4SOIbeta0*d_Vdiff_dV_e_si*Vdiff));
d_T0_dV_b_si = (pParam_B4SOIbeta1*d_Vdiff_dV_b_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_b_si+pParam_B4SOIbeta0*d_Vdiff_dV_b_si*Vdiff));
d_T0_dV_sb_si = (pParam_B4SOIbeta1*d_Vdiff_dV_sb_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_sb_si+pParam_B4SOIbeta0*d_Vdiff_dV_sb_si*Vdiff));
d_T0_dV_db_di = (pParam_B4SOIbeta1*d_Vdiff_dV_db_di+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_db_di+pParam_B4SOIbeta0*d_Vdiff_dV_db_di*Vdiff));
d_T0_dV_gi_p = (pParam_B4SOIbeta1*d_Vdiff_dV_gi_p+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_gi_p+pParam_B4SOIbeta0*d_Vdiff_dV_gi_p*Vdiff));
d_T0_dV_b_GND = (pParam_B4SOIbeta1*d_Vdiff_dV_b_GND+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_b_GND+pParam_B4SOIbeta0*d_Vdiff_dV_b_GND*Vdiff));
d_T0_dV_p_GND = (pParam_B4SOIbeta1*d_Vdiff_dV_p_GND+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_p_GND+pParam_B4SOIbeta0*d_Vdiff_dV_p_GND*Vdiff));
d_T0_dTemp_t_GND = (pParam_B4SOIbeta1*d_Vdiff_dTemp_t_GND+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dTemp_t_GND+pParam_B4SOIbeta0*d_Vdiff_dTemp_t_GND*Vdiff));
d_T0_dV_di_si = (pParam_B4SOIbeta1*d_Vdiff_dV_di_si+((pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dV_di_si+pParam_B4SOIbeta0*d_Vdiff_dV_di_si*Vdiff));
T0 = ((pParam_B4SOIbeta2+(pParam_B4SOIbeta1*Vdiff))+((pParam_B4SOIbeta0*Vdiff)*Vdiff));
if ((T0<1e-5))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = 1e-5;
}
if (((T0<(Vdiff/100.0))&&(Vdiff>0.0)))
{

d_Ratio_dV_di_si = d_Ratio_dTemp_t_GND = d_Ratio_dV_p_GND = d_Ratio_dV_b_GND = d_Ratio_dV_gi_p = d_Ratio_dV_db_di = d_Ratio_dV_sb_si = d_Ratio_dV_b_si = d_Ratio_dV_e_si = d_Ratio_dV_gi_si =  0.0;
Ratio = (pParam_B4SOIalpha0*2.688117142e+43);
}
else
{
if (((T0<((-Vdiff)/100.0))&&(Vdiff<0.0)))
{

d_Ratio_dV_di_si = d_Ratio_dTemp_t_GND = d_Ratio_dV_p_GND = d_Ratio_dV_b_GND = d_Ratio_dV_gi_p = d_Ratio_dV_db_di = d_Ratio_dV_sb_si = d_Ratio_dV_b_si = d_Ratio_dV_e_si = d_Ratio_dV_gi_si =  0.0;
Ratio = (pParam_B4SOIalpha0*3.720075976e-44);
}
else
{
{
double value_exp_0 = exp((Vdiff/T0));
double  deriv_exp_0_d0 = value_exp_0;

d_Ratio_dV_gi_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_gi_si-Vdiff*d_T0_dV_gi_si)/T0/T0)));
d_Ratio_dV_e_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_e_si-Vdiff*d_T0_dV_e_si)/T0/T0)));
d_Ratio_dV_b_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_b_si-Vdiff*d_T0_dV_b_si)/T0/T0)));
d_Ratio_dV_sb_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_sb_si-Vdiff*d_T0_dV_sb_si)/T0/T0)));
d_Ratio_dV_db_di = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_db_di-Vdiff*d_T0_dV_db_di)/T0/T0)));
d_Ratio_dV_gi_p = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_gi_p-Vdiff*d_T0_dV_gi_p)/T0/T0)));
d_Ratio_dV_b_GND = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_b_GND-Vdiff*d_T0_dV_b_GND)/T0/T0)));
d_Ratio_dV_p_GND = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_p_GND-Vdiff*d_T0_dV_p_GND)/T0/T0)));
d_Ratio_dTemp_t_GND = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dTemp_t_GND-Vdiff*d_T0_dTemp_t_GND)/T0/T0)));
d_Ratio_dV_di_si = pParam_B4SOIalpha0*(deriv_exp_0_d0*(((T0*d_Vdiff_dV_di_si-Vdiff*d_T0_dV_di_si)/T0/T0)));
Ratio = (pParam_B4SOIalpha0*value_exp_0);
}
}
}
if ((Ratio>10.0))
{

d_Ratio_dV_di_si = d_Ratio_dTemp_t_GND = d_Ratio_dV_p_GND = d_Ratio_dV_b_GND = d_Ratio_dV_gi_p = d_Ratio_dV_db_di = d_Ratio_dV_sb_si = d_Ratio_dV_b_si = d_Ratio_dV_e_si = d_Ratio_dV_gi_si =  0.0;
Ratio = 10.0;
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_gi_si = d_Ids_dV_gi_si;
d_T0_dV_di_si = d_Ids_dV_di_si;
d_T0_dV_e_si = d_Ids_dV_e_si;
d_T0_dV_b_si = d_Ids_dV_b_si;
d_T0_dV_b_GND = d_Ids_dV_b_GND;
d_T0_dV_p_GND = d_Ids_dV_p_GND;
d_T0_dTemp_t_GND = d_Ids_dTemp_t_GND;
T0 = Ids;

d_Idsmosfet_dV_gi_si = (Ratio*d_T0_dV_gi_si+d_Ratio_dV_gi_si*T0);
d_Idsmosfet_dV_e_si = (Ratio*d_T0_dV_e_si+d_Ratio_dV_e_si*T0);
d_Idsmosfet_dV_b_si = (Ratio*d_T0_dV_b_si+d_Ratio_dV_b_si*T0);
d_Idsmosfet_dV_sb_si = (Ratio*d_T0_dV_sb_si+d_Ratio_dV_sb_si*T0);
d_Idsmosfet_dV_db_di = (Ratio*d_T0_dV_db_di+d_Ratio_dV_db_di*T0);
d_Idsmosfet_dV_gi_p = (Ratio*d_T0_dV_gi_p+d_Ratio_dV_gi_p*T0);
d_Idsmosfet_dV_b_GND = (Ratio*d_T0_dV_b_GND+d_Ratio_dV_b_GND*T0);
d_Idsmosfet_dV_p_GND = (Ratio*d_T0_dV_p_GND+d_Ratio_dV_p_GND*T0);
d_Idsmosfet_dTemp_t_GND = (Ratio*d_T0_dTemp_t_GND+d_Ratio_dTemp_t_GND*T0);
d_Idsmosfet_dV_di_si = (Ratio*d_T0_dV_di_si+d_Ratio_dV_di_si*T0);
Idsmosfet = (Ratio*T0);
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = ((pParam_B4SOIcbjtii+(pParam_B4SOIebjtii*Leff))/Leff);

d_Vbci_dV_b_GND = pParam_B4SOIvbci*B4SOItvbci*d_TempRatio_dV_b_GND;
d_Vbci_dV_p_GND = pParam_B4SOIvbci*B4SOItvbci*d_TempRatio_dV_p_GND;
d_Vbci_dTemp_t_GND = pParam_B4SOIvbci*B4SOItvbci*d_TempRatio_dTemp_t_GND;
Vbci = (pParam_B4SOIvbci*(1.0+(B4SOItvbci*(TempRatio-1.0))));
if ((B4SOImode>0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_db_di = (-d_vdbd_dV_db_di);
d_T1_dV_b_GND = d_Vbci_dV_b_GND;
d_T1_dV_p_GND = d_Vbci_dV_p_GND;
d_T1_dTemp_t_GND = d_Vbci_dTemp_t_GND;
T1 = (Vbci-vdbd);
}
else
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si =  0.0;
d_T1_dV_sb_si = (-d_vsbs_dV_sb_si);
d_T1_dV_b_GND = d_Vbci_dV_b_GND;
d_T1_dV_p_GND = d_Vbci_dV_p_GND;
d_T1_dTemp_t_GND = d_Vbci_dTemp_t_GND;
T1 = (Vbci-vsbs);
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si = d_T2_dV_b_si = d_T2_dV_e_si = d_T2_dV_di_si = d_T2_dV_gi_si = d_T2_dTemp_t_GND = d_T2_dV_p_GND = d_T2_dV_b_GND =  0.0;
T2 = (pParam_B4SOImbjtii-1.0);
if ((T1<=0.0))
{

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = 0.0;
}
else
{
{
double value_pow_0 = pow(T1,T2);
double  deriv_pow_0_d0 = ((T1 == 0.0)?0.0:(value_pow_0*T2/T1));
double  deriv_pow_0_d1 = (T1 == 0.0)?0.0:(log(T1)*value_pow_0);

d_T3_dV_b_GND = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_b_GND))+(deriv_pow_0_d1*(d_T2_dV_b_GND)));
d_T3_dV_p_GND = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_p_GND))+(deriv_pow_0_d1*(d_T2_dV_p_GND)));
d_T3_dTemp_t_GND = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dTemp_t_GND))+(deriv_pow_0_d1*(d_T2_dTemp_t_GND)));
d_T3_dV_gi_si = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_gi_si))+(deriv_pow_0_d1*(d_T2_dV_gi_si)));
d_T3_dV_di_si = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_di_si))+(deriv_pow_0_d1*(d_T2_dV_di_si)));
d_T3_dV_e_si = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_e_si))+(deriv_pow_0_d1*(d_T2_dV_e_si)));
d_T3_dV_b_si = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_b_si))+(deriv_pow_0_d1*(d_T2_dV_b_si)));
d_T3_dV_sb_si = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_sb_si))+(deriv_pow_0_d1*(d_T2_dV_sb_si)));
d_T3_dV_db_di = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_db_di))+(deriv_pow_0_d1*(d_T2_dV_db_di)));
d_T3_dV_gi_p = (-pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dV_gi_p))+(deriv_pow_0_d1*(d_T2_dV_gi_p)));
T3 = ((-pParam_B4SOIabjtii)*value_pow_0);
}
}
if ((T3>100.0))
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = 2.688117142e+43;
}
else
{
if ((T3<(-100.0)))
{

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si = d_T4_dTemp_t_GND = d_T4_dV_p_GND = d_T4_dV_b_GND =  0.0;
T4 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;

d_T4_dV_gm_si =  0.0;
d_T4_dV_b_GND = (deriv_exp_0_d0*(d_T3_dV_b_GND));
d_T4_dV_p_GND = (deriv_exp_0_d0*(d_T3_dV_p_GND));
d_T4_dTemp_t_GND = (deriv_exp_0_d0*(d_T3_dTemp_t_GND));
d_T4_dV_gi_si = (deriv_exp_0_d0*(d_T3_dV_gi_si));
d_T4_dV_di_si = (deriv_exp_0_d0*(d_T3_dV_di_si));
d_T4_dV_e_si = (deriv_exp_0_d0*(d_T3_dV_e_si));
d_T4_dV_b_si = (deriv_exp_0_d0*(d_T3_dV_b_si));
d_T4_dV_sb_si = (deriv_exp_0_d0*(d_T3_dV_sb_si));
d_T4_dV_db_di = (deriv_exp_0_d0*(d_T3_dV_db_di));
d_T4_dV_gi_p = (deriv_exp_0_d0*(d_T3_dV_gi_p));
T4 = value_exp_0;
}
}
}

d_Iiibjt_dV_b_GND = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_b_GND+(((T0*B4SOImode)*Ic)*d_T1_dV_b_GND+((T0*B4SOImode)*d_Ic_dV_b_GND+d_T0_dV_b_GND*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_p_GND = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_p_GND+(((T0*B4SOImode)*Ic)*d_T1_dV_p_GND+((T0*B4SOImode)*d_Ic_dV_p_GND+d_T0_dV_p_GND*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dTemp_t_GND = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dTemp_t_GND+(((T0*B4SOImode)*Ic)*d_T1_dTemp_t_GND+((T0*B4SOImode)*d_Ic_dTemp_t_GND+d_T0_dTemp_t_GND*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_e_si = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_e_si+(((T0*B4SOImode)*Ic)*d_T1_dV_e_si+((T0*B4SOImode)*d_Ic_dV_e_si+d_T0_dV_e_si*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_gi_si = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_gi_si+(((T0*B4SOImode)*Ic)*d_T1_dV_gi_si+((T0*B4SOImode)*d_Ic_dV_gi_si+d_T0_dV_gi_si*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_di_si = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_di_si+(((T0*B4SOImode)*Ic)*d_T1_dV_di_si+((T0*B4SOImode)*d_Ic_dV_di_si+d_T0_dV_di_si*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_b_si = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_b_si+(((T0*B4SOImode)*Ic)*d_T1_dV_b_si+((T0*B4SOImode)*d_Ic_dV_b_si+d_T0_dV_b_si*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_sb_si = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_sb_si+(((T0*B4SOImode)*Ic)*d_T1_dV_sb_si+((T0*B4SOImode)*d_Ic_dV_sb_si+d_T0_dV_sb_si*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_db_di = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_db_di+(((T0*B4SOImode)*Ic)*d_T1_dV_db_di+((T0*B4SOImode)*d_Ic_dV_db_di+d_T0_dV_db_di*B4SOImode*Ic)*T1)*T4);
d_Iiibjt_dV_gi_p = ((((T0*B4SOImode)*Ic)*T1)*d_T4_dV_gi_p+(((T0*B4SOImode)*Ic)*d_T1_dV_gi_p+d_T0_dV_gi_p*B4SOImode*Ic*T1)*T4);
Iiibjt = ((((T0*B4SOImode)*Ic)*T1)*T4);

d_Iii_dV_gi_si = (d_Idsmosfet_dV_gi_si+d_Iiibjt_dV_gi_si);
d_Iii_dV_e_si = (d_Idsmosfet_dV_e_si+d_Iiibjt_dV_e_si);
d_Iii_dV_b_si = (d_Idsmosfet_dV_b_si+d_Iiibjt_dV_b_si);
d_Iii_dV_sb_si = (d_Idsmosfet_dV_sb_si+d_Iiibjt_dV_sb_si);
d_Iii_dV_db_di = (d_Idsmosfet_dV_db_di+d_Iiibjt_dV_db_di);
d_Iii_dV_gi_p = (d_Idsmosfet_dV_gi_p+d_Iiibjt_dV_gi_p);
d_Iii_dV_b_GND = (d_Idsmosfet_dV_b_GND+d_Iiibjt_dV_b_GND);
d_Iii_dV_p_GND = (d_Idsmosfet_dV_p_GND+d_Iiibjt_dV_p_GND);
d_Iii_dTemp_t_GND = (d_Idsmosfet_dTemp_t_GND+d_Iiibjt_dTemp_t_GND);
d_Iii_dV_di_si = (d_Idsmosfet_dV_di_si+d_Iiibjt_dV_di_si);
Iii = (Idsmosfet+Iiibjt);
}
if (((B4SOIbodyMod==0)||(B4SOIbodyMod==2)))
{

d_Ibp_dV_b_p = d_Ibp_dV_gi_p = d_Ibp_dV_db_di = d_Ibp_dV_sb_si = d_Ibp_dV_b_si = d_Ibp_dV_di_si = d_Ibp_dV_gi_si = d_Ibp_dV_e_si = d_Ibp_dTemp_t_GND = d_Ibp_dV_p_GND = d_Ibp_dV_b_GND =  0.0;
Ibp = 0;
}
else
{
if ((pParam_B4SOIrbody<1e-3))
{
if ((B4SOIrbodyext<=1e-3))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0/1e-3);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (1.0/B4SOIrbodyext);
}

d_Ibp_dV_b_GND = vbp*d_T0_dV_b_GND;
d_Ibp_dV_p_GND = vbp*d_T0_dV_p_GND;
d_Ibp_dTemp_t_GND = vbp*d_T0_dTemp_t_GND;
d_Ibp_dV_e_si = vbp*d_T0_dV_e_si;
d_Ibp_dV_gi_si = vbp*d_T0_dV_gi_si;
d_Ibp_dV_di_si = vbp*d_T0_dV_di_si;
d_Ibp_dV_b_si = vbp*d_T0_dV_b_si;
d_Ibp_dV_sb_si = vbp*d_T0_dV_sb_si;
d_Ibp_dV_db_di = vbp*d_T0_dV_db_di;
d_Ibp_dV_gi_p = vbp*d_T0_dV_gi_p;
d_Ibp_dV_b_p = d_vbp_dV_b_p*T0;
Ibp = (vbp*T0);
}
else
{

d_Ibp_dV_gi_p = d_Ibp_dV_db_di = d_Ibp_dV_sb_si = d_Ibp_dV_b_si = d_Ibp_dV_di_si = d_Ibp_dV_gi_si = d_Ibp_dV_e_si = d_Ibp_dTemp_t_GND = d_Ibp_dV_p_GND = d_Ibp_dV_b_GND =  0.0;
d_Ibp_dV_b_p = (d_vbp_dV_b_p/(pParam_B4SOIrbody+B4SOIrbodyext));
Ibp = (vbp/(pParam_B4SOIrbody+B4SOIrbodyext));
}
}
}
else
{

d_Iii_dV_di_si = d_Iii_dTemp_t_GND = d_Iii_dV_p_GND = d_Iii_dV_b_GND = d_Iii_dV_gi_p = d_Iii_dV_db_di = d_Iii_dV_sb_si = d_Iii_dV_b_si = d_Iii_dV_e_si = d_Iii_dV_gi_si =  0.0;
Iii = 0.0;

d_Ibp_dV_b_p = d_Ibp_dV_gi_p = d_Ibp_dV_db_di = d_Ibp_dV_sb_si = d_Ibp_dV_b_si = d_Ibp_dV_di_si = d_Ibp_dV_gi_si = d_Ibp_dV_e_si = d_Ibp_dTemp_t_GND = d_Ibp_dV_p_GND = d_Ibp_dV_b_GND =  0.0;
Ibp = 0.0;
}
if ((B4SOIrgateMod>1))
{

d_T9_dV_b_si = d_T9_dV_e_si = d_T9_dV_di_si = d_T9_dV_gi_si =  0.0;
d_T9_dV_b_GND = pParam_B4SOIxrcrg2*d_B4SOIvtm_dV_b_GND;
d_T9_dV_p_GND = pParam_B4SOIxrcrg2*d_B4SOIvtm_dV_p_GND;
d_T9_dTemp_t_GND = pParam_B4SOIxrcrg2*d_B4SOIvtm_dTemp_t_GND;
T9 = (pParam_B4SOIxrcrg2*B4SOIvtm);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (T9*d_beta_dV_b_GND+d_T9_dV_b_GND*beta);
d_T0_dV_p_GND = (T9*d_beta_dV_p_GND+d_T9_dV_p_GND*beta);
d_T0_dTemp_t_GND = (T9*d_beta_dTemp_t_GND+d_T9_dTemp_t_GND*beta);
d_T0_dV_gi_si = (T9*d_beta_dV_gi_si+d_T9_dV_gi_si*beta);
d_T0_dV_di_si = (T9*d_beta_dV_di_si+d_T9_dV_di_si*beta);
d_T0_dV_e_si = (T9*d_beta_dV_e_si+d_T9_dV_e_si*beta);
d_T0_dV_b_si = (T9*d_beta_dV_b_si+d_T9_dV_b_si*beta);
T0 = (T9*beta);

d_B4SOIgcrg_dV_b_GND = pParam_B4SOIxrcrg1*(d_T0_dV_b_GND+d_IdovVds_dV_b_GND);
d_B4SOIgcrg_dV_p_GND = pParam_B4SOIxrcrg1*(d_T0_dV_p_GND+d_IdovVds_dV_p_GND);
d_B4SOIgcrg_dTemp_t_GND = pParam_B4SOIxrcrg1*(d_T0_dTemp_t_GND+d_IdovVds_dTemp_t_GND);
d_B4SOIgcrg_dV_e_si = pParam_B4SOIxrcrg1*(d_T0_dV_e_si+d_IdovVds_dV_e_si);
d_B4SOIgcrg_dV_gi_si = pParam_B4SOIxrcrg1*(d_T0_dV_gi_si+d_IdovVds_dV_gi_si);
d_B4SOIgcrg_dV_di_si = pParam_B4SOIxrcrg1*(d_T0_dV_di_si+d_IdovVds_dV_di_si);
d_B4SOIgcrg_dV_b_si = pParam_B4SOIxrcrg1*(d_T0_dV_b_si+d_IdovVds_dV_b_si);
d_B4SOIgcrg_dV_sb_si = pParam_B4SOIxrcrg1*d_T0_dV_sb_si;
d_B4SOIgcrg_dV_db_di = pParam_B4SOIxrcrg1*d_T0_dV_db_di;
d_B4SOIgcrg_dV_gi_p = pParam_B4SOIxrcrg1*d_T0_dV_gi_p;
B4SOIgcrg = (pParam_B4SOIxrcrg1*(T0+IdovVds));
if ((B4SOInf!=1.0))
{

d_B4SOIgcrg_dV_b_GND = d_B4SOIgcrg_dV_b_GND*B4SOInf;
d_B4SOIgcrg_dV_p_GND = d_B4SOIgcrg_dV_p_GND*B4SOInf;
d_B4SOIgcrg_dTemp_t_GND = d_B4SOIgcrg_dTemp_t_GND*B4SOInf;
d_B4SOIgcrg_dV_e_si = d_B4SOIgcrg_dV_e_si*B4SOInf;
d_B4SOIgcrg_dV_gi_si = d_B4SOIgcrg_dV_gi_si*B4SOInf;
d_B4SOIgcrg_dV_di_si = d_B4SOIgcrg_dV_di_si*B4SOInf;
d_B4SOIgcrg_dV_b_si = d_B4SOIgcrg_dV_b_si*B4SOInf;
d_B4SOIgcrg_dV_sb_si = d_B4SOIgcrg_dV_sb_si*B4SOInf;
d_B4SOIgcrg_dV_db_di = d_B4SOIgcrg_dV_db_di*B4SOInf;
d_B4SOIgcrg_dV_gi_p = d_B4SOIgcrg_dV_gi_p*B4SOInf;
B4SOIgcrg = (B4SOIgcrg*B4SOInf);
}
if ((B4SOIrgateMod==2))
{

d_T11_dV_b_GND = d_B4SOIgcrg_dV_b_GND;
d_T11_dV_p_GND = d_B4SOIgcrg_dV_p_GND;
d_T11_dTemp_t_GND = d_B4SOIgcrg_dTemp_t_GND;
d_T11_dV_e_si = d_B4SOIgcrg_dV_e_si;
d_T11_dV_gi_si = d_B4SOIgcrg_dV_gi_si;
d_T11_dV_di_si = d_B4SOIgcrg_dV_di_si;
d_T11_dV_b_si = d_B4SOIgcrg_dV_b_si;
d_T11_dV_sb_si = d_B4SOIgcrg_dV_sb_si;
d_T11_dV_db_di = d_B4SOIgcrg_dV_db_di;
d_T11_dV_gi_p = d_B4SOIgcrg_dV_gi_p;
T11 = (B4SOIgrgeltd+B4SOIgcrg);

d_B4SOIgcrg_dV_b_GND = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_b_GND-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_b_GND)/T11/T11);
d_B4SOIgcrg_dV_p_GND = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_p_GND-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_p_GND)/T11/T11);
d_B4SOIgcrg_dTemp_t_GND = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dTemp_t_GND-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dTemp_t_GND)/T11/T11);
d_B4SOIgcrg_dV_e_si = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_e_si-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_e_si)/T11/T11);
d_B4SOIgcrg_dV_gi_si = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_gi_si-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_gi_si)/T11/T11);
d_B4SOIgcrg_dV_di_si = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_di_si-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_di_si)/T11/T11);
d_B4SOIgcrg_dV_b_si = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_b_si-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_b_si)/T11/T11);
d_B4SOIgcrg_dV_sb_si = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_sb_si-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_sb_si)/T11/T11);
d_B4SOIgcrg_dV_db_di = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_db_di-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_db_di)/T11/T11);
d_B4SOIgcrg_dV_gi_p = ((T11*B4SOIgrgeltd*d_B4SOIgcrg_dV_gi_p-(B4SOIgrgeltd*B4SOIgcrg)*d_T11_dV_gi_p)/T11/T11);
B4SOIgcrg = ((B4SOIgrgeltd*B4SOIgcrg)/T11);
}
}
else
{

d_B4SOIgcrg_dV_gi_p = d_B4SOIgcrg_dV_db_di = d_B4SOIgcrg_dV_sb_si = d_B4SOIgcrg_dV_b_si = d_B4SOIgcrg_dV_di_si = d_B4SOIgcrg_dV_gi_si = d_B4SOIgcrg_dV_e_si = d_B4SOIgcrg_dTemp_t_GND = d_B4SOIgcrg_dV_p_GND = d_B4SOIgcrg_dV_b_GND =  0.0;
B4SOIgcrg = 0;
}
if ((B4SOIrdsMod==1))
{

d_Rds_dTemp_t_GND = d_Rds_dV_p_GND = d_Rds_dV_b_GND = d_Rds_dV_b_si = d_Rds_dV_di_si = d_Rds_dV_gi_si = d_Rds_dV_e_si =  0.0;
Rds = 0.0;

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_gi_si = d_vgs_dV_gi_si;
T0 = (vgs-pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)));
T1 = value_sqrt_0;
}

d_vgs_eff_dV_b_GND = 0.5*(d_T0_dV_b_GND+d_T1_dV_b_GND);
d_vgs_eff_dV_p_GND = 0.5*(d_T0_dV_p_GND+d_T1_dV_p_GND);
d_vgs_eff_dTemp_t_GND = 0.5*(d_T0_dTemp_t_GND+d_T1_dTemp_t_GND);
d_vgs_eff_dV_e_si = 0.5*(d_T0_dV_e_si+d_T1_dV_e_si);
d_vgs_eff_dV_gi_si = 0.5*(d_T0_dV_gi_si+d_T1_dV_gi_si);
d_vgs_eff_dV_di_si = 0.5*(d_T0_dV_di_si+d_T1_dV_di_si);
d_vgs_eff_dV_b_si = 0.5*(d_T0_dV_b_si+d_T1_dV_b_si);
d_vgs_eff_dV_sb_si = 0.5*(d_T0_dV_sb_si+d_T1_dV_sb_si);
d_vgs_eff_dV_db_di = 0.5*(d_T0_dV_db_di+d_T1_dV_db_di);
d_vgs_eff_dV_gi_p = 0.5*(d_T0_dV_gi_p+d_T1_dV_gi_p);
vgs_eff = (0.5*(T0+T1));

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIprwg*d_vgs_eff_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIprwg*d_vgs_eff_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIprwg*d_vgs_eff_dTemp_t_GND;
d_T0_dV_e_si = pParam_B4SOIprwg*d_vgs_eff_dV_e_si;
d_T0_dV_gi_si = pParam_B4SOIprwg*d_vgs_eff_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIprwg*d_vgs_eff_dV_di_si;
d_T0_dV_b_si = pParam_B4SOIprwg*d_vgs_eff_dV_b_si;
d_T0_dV_sb_si = pParam_B4SOIprwg*d_vgs_eff_dV_sb_si;
d_T0_dV_db_di = pParam_B4SOIprwg*d_vgs_eff_dV_db_di;
d_T0_dV_gi_p = pParam_B4SOIprwg*d_vgs_eff_dV_gi_p;
T0 = (1.0+(pParam_B4SOIprwg*vgs_eff));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
d_T1_dV_b_si = (-pParam_B4SOIprwb)*d_vbs_dV_b_si;
T1 = ((-pParam_B4SOIprwb)*vbs);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = ((-d_T0_dV_b_GND/T0/T0)+d_T1_dV_b_GND);
d_T2_dV_p_GND = ((-d_T0_dV_p_GND/T0/T0)+d_T1_dV_p_GND);
d_T2_dTemp_t_GND = ((-d_T0_dTemp_t_GND/T0/T0)+d_T1_dTemp_t_GND);
d_T2_dV_e_si = ((-d_T0_dV_e_si/T0/T0)+d_T1_dV_e_si);
d_T2_dV_gi_si = ((-d_T0_dV_gi_si/T0/T0)+d_T1_dV_gi_si);
d_T2_dV_di_si = ((-d_T0_dV_di_si/T0/T0)+d_T1_dV_di_si);
d_T2_dV_b_si = ((-d_T0_dV_b_si/T0/T0)+d_T1_dV_b_si);
d_T2_dV_sb_si = ((-d_T0_dV_sb_si/T0/T0)+d_T1_dV_sb_si);
d_T2_dV_db_di = ((-d_T0_dV_db_di/T0/T0)+d_T1_dV_db_di);
d_T2_dV_gi_p = ((-d_T0_dV_gi_p/T0/T0)+d_T1_dV_gi_p);
T2 = ((1.0/T0)+T1);
{
double value_sqrt_0 = sqrt(((T2*T2)+0.01));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_b_GND = (d_T2_dV_b_GND+(deriv_sqrt_0_d0*((T2*d_T2_dV_b_GND+d_T2_dV_b_GND*T2))));
d_T3_dV_p_GND = (d_T2_dV_p_GND+(deriv_sqrt_0_d0*((T2*d_T2_dV_p_GND+d_T2_dV_p_GND*T2))));
d_T3_dTemp_t_GND = (d_T2_dTemp_t_GND+(deriv_sqrt_0_d0*((T2*d_T2_dTemp_t_GND+d_T2_dTemp_t_GND*T2))));
d_T3_dV_gi_si = (d_T2_dV_gi_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_gi_si+d_T2_dV_gi_si*T2))));
d_T3_dV_di_si = (d_T2_dV_di_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_di_si+d_T2_dV_di_si*T2))));
d_T3_dV_e_si = (d_T2_dV_e_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_e_si+d_T2_dV_e_si*T2))));
d_T3_dV_b_si = (d_T2_dV_b_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_b_si+d_T2_dV_b_si*T2))));
d_T3_dV_sb_si = (d_T2_dV_sb_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_sb_si+d_T2_dV_sb_si*T2))));
d_T3_dV_db_di = (d_T2_dV_db_di+(deriv_sqrt_0_d0*((T2*d_T2_dV_db_di+d_T2_dV_db_di*T2))));
d_T3_dV_gi_p = (d_T2_dV_gi_p+(deriv_sqrt_0_d0*((T2*d_T2_dV_gi_p+d_T2_dV_gi_p*T2))));
T3 = (T2+value_sqrt_0);
}

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si =  0.0;
d_T4_dV_b_GND = d_rs0_dV_b_GND*0.5;
d_T4_dV_p_GND = d_rs0_dV_p_GND*0.5;
d_T4_dTemp_t_GND = d_rs0_dTemp_t_GND*0.5;
T4 = (rs0*0.5);

d_Rs_dV_gi_si = (T3*d_T4_dV_gi_si+d_T3_dV_gi_si*T4);
d_Rs_dV_di_si = (T3*d_T4_dV_di_si+d_T3_dV_di_si*T4);
d_Rs_dV_e_si = (T3*d_T4_dV_e_si+d_T3_dV_e_si*T4);
d_Rs_dV_b_si = (T3*d_T4_dV_b_si+d_T3_dV_b_si*T4);
d_Rs_dV_sb_si = (T3*d_T4_dV_sb_si+d_T3_dV_sb_si*T4);
d_Rs_dV_db_di = (T3*d_T4_dV_db_di+d_T3_dV_db_di*T4);
d_Rs_dV_gi_p = (T3*d_T4_dV_gi_p+d_T3_dV_gi_p*T4);
d_Rs_dV_b_GND = (d_rswmin_dV_b_GND+(T3*d_T4_dV_b_GND+d_T3_dV_b_GND*T4));
d_Rs_dV_p_GND = (d_rswmin_dV_p_GND+(T3*d_T4_dV_p_GND+d_T3_dV_p_GND*T4));
d_Rs_dTemp_t_GND = (d_rswmin_dTemp_t_GND+(T3*d_T4_dTemp_t_GND+d_T3_dTemp_t_GND*T4));
Rs = ((rswmin+(T3*T4))+B4SOIsourceResistance);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = d_vgd_dV_di_si;
d_T0_dV_gi_si = d_vgd_dV_gi_si;
T0 = (vgd-pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)));
T1 = value_sqrt_0;
}

d_vgd_eff_dV_b_GND = 0.5*(d_T0_dV_b_GND+d_T1_dV_b_GND);
d_vgd_eff_dV_p_GND = 0.5*(d_T0_dV_p_GND+d_T1_dV_p_GND);
d_vgd_eff_dTemp_t_GND = 0.5*(d_T0_dTemp_t_GND+d_T1_dTemp_t_GND);
d_vgd_eff_dV_e_si = 0.5*(d_T0_dV_e_si+d_T1_dV_e_si);
d_vgd_eff_dV_gi_si = 0.5*(d_T0_dV_gi_si+d_T1_dV_gi_si);
d_vgd_eff_dV_di_si = 0.5*(d_T0_dV_di_si+d_T1_dV_di_si);
d_vgd_eff_dV_b_si = 0.5*(d_T0_dV_b_si+d_T1_dV_b_si);
d_vgd_eff_dV_sb_si = 0.5*(d_T0_dV_sb_si+d_T1_dV_sb_si);
d_vgd_eff_dV_db_di = 0.5*(d_T0_dV_db_di+d_T1_dV_db_di);
d_vgd_eff_dV_gi_p = 0.5*(d_T0_dV_gi_p+d_T1_dV_gi_p);
vgd_eff = (0.5*(T0+T1));

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = pParam_B4SOIprwg*d_vgd_eff_dV_b_GND;
d_T0_dV_p_GND = pParam_B4SOIprwg*d_vgd_eff_dV_p_GND;
d_T0_dTemp_t_GND = pParam_B4SOIprwg*d_vgd_eff_dTemp_t_GND;
d_T0_dV_e_si = pParam_B4SOIprwg*d_vgd_eff_dV_e_si;
d_T0_dV_gi_si = pParam_B4SOIprwg*d_vgd_eff_dV_gi_si;
d_T0_dV_di_si = pParam_B4SOIprwg*d_vgd_eff_dV_di_si;
d_T0_dV_b_si = pParam_B4SOIprwg*d_vgd_eff_dV_b_si;
d_T0_dV_sb_si = pParam_B4SOIprwg*d_vgd_eff_dV_sb_si;
d_T0_dV_db_di = pParam_B4SOIprwg*d_vgd_eff_dV_db_di;
d_T0_dV_gi_p = pParam_B4SOIprwg*d_vgd_eff_dV_gi_p;
T0 = (1.0+(pParam_B4SOIprwg*vgd_eff));

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_e_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
d_T1_dV_di_si = (-pParam_B4SOIprwb)*d_vbd_dV_di_si;
d_T1_dV_b_si = (-pParam_B4SOIprwb)*d_vbd_dV_b_si;
T1 = ((-pParam_B4SOIprwb)*vbd);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = ((-d_T0_dV_b_GND/T0/T0)+d_T1_dV_b_GND);
d_T2_dV_p_GND = ((-d_T0_dV_p_GND/T0/T0)+d_T1_dV_p_GND);
d_T2_dTemp_t_GND = ((-d_T0_dTemp_t_GND/T0/T0)+d_T1_dTemp_t_GND);
d_T2_dV_e_si = ((-d_T0_dV_e_si/T0/T0)+d_T1_dV_e_si);
d_T2_dV_gi_si = ((-d_T0_dV_gi_si/T0/T0)+d_T1_dV_gi_si);
d_T2_dV_di_si = ((-d_T0_dV_di_si/T0/T0)+d_T1_dV_di_si);
d_T2_dV_b_si = ((-d_T0_dV_b_si/T0/T0)+d_T1_dV_b_si);
d_T2_dV_sb_si = ((-d_T0_dV_sb_si/T0/T0)+d_T1_dV_sb_si);
d_T2_dV_db_di = ((-d_T0_dV_db_di/T0/T0)+d_T1_dV_db_di);
d_T2_dV_gi_p = ((-d_T0_dV_gi_p/T0/T0)+d_T1_dV_gi_p);
T2 = ((1.0/T0)+T1);
{
double value_sqrt_0 = sqrt(((T2*T2)+0.01));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T3_dV_b_GND = (d_T2_dV_b_GND+(deriv_sqrt_0_d0*((T2*d_T2_dV_b_GND+d_T2_dV_b_GND*T2))));
d_T3_dV_p_GND = (d_T2_dV_p_GND+(deriv_sqrt_0_d0*((T2*d_T2_dV_p_GND+d_T2_dV_p_GND*T2))));
d_T3_dTemp_t_GND = (d_T2_dTemp_t_GND+(deriv_sqrt_0_d0*((T2*d_T2_dTemp_t_GND+d_T2_dTemp_t_GND*T2))));
d_T3_dV_gi_si = (d_T2_dV_gi_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_gi_si+d_T2_dV_gi_si*T2))));
d_T3_dV_di_si = (d_T2_dV_di_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_di_si+d_T2_dV_di_si*T2))));
d_T3_dV_e_si = (d_T2_dV_e_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_e_si+d_T2_dV_e_si*T2))));
d_T3_dV_b_si = (d_T2_dV_b_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_b_si+d_T2_dV_b_si*T2))));
d_T3_dV_sb_si = (d_T2_dV_sb_si+(deriv_sqrt_0_d0*((T2*d_T2_dV_sb_si+d_T2_dV_sb_si*T2))));
d_T3_dV_db_di = (d_T2_dV_db_di+(deriv_sqrt_0_d0*((T2*d_T2_dV_db_di+d_T2_dV_db_di*T2))));
d_T3_dV_gi_p = (d_T2_dV_gi_p+(deriv_sqrt_0_d0*((T2*d_T2_dV_gi_p+d_T2_dV_gi_p*T2))));
T3 = (T2+value_sqrt_0);
}

d_T4_dV_gm_si = d_T4_dV_gi_p = d_T4_dV_db_di = d_T4_dV_sb_si = d_T4_dV_b_si = d_T4_dV_e_si = d_T4_dV_di_si = d_T4_dV_gi_si =  0.0;
d_T4_dV_b_GND = d_rd0_dV_b_GND*0.5;
d_T4_dV_p_GND = d_rd0_dV_p_GND*0.5;
d_T4_dTemp_t_GND = d_rd0_dTemp_t_GND*0.5;
T4 = (rd0*0.5);

d_Rd_dV_gi_si = (T3*d_T4_dV_gi_si+d_T3_dV_gi_si*T4);
d_Rd_dV_di_si = (T3*d_T4_dV_di_si+d_T3_dV_di_si*T4);
d_Rd_dV_e_si = (T3*d_T4_dV_e_si+d_T3_dV_e_si*T4);
d_Rd_dV_b_si = (T3*d_T4_dV_b_si+d_T3_dV_b_si*T4);
d_Rd_dV_sb_si = (T3*d_T4_dV_sb_si+d_T3_dV_sb_si*T4);
d_Rd_dV_db_di = (T3*d_T4_dV_db_di+d_T3_dV_db_di*T4);
d_Rd_dV_gi_p = (T3*d_T4_dV_gi_p+d_T3_dV_gi_p*T4);
d_Rd_dV_b_GND = (d_rdwmin_dV_b_GND+(T3*d_T4_dV_b_GND+d_T3_dV_b_GND*T4));
d_Rd_dV_p_GND = (d_rdwmin_dV_p_GND+(T3*d_T4_dV_p_GND+d_T3_dV_p_GND*T4));
d_Rd_dTemp_t_GND = (d_rdwmin_dTemp_t_GND+(T3*d_T4_dTemp_t_GND+d_T3_dTemp_t_GND*T4));
Rd = ((rdwmin+(T3*T4))+B4SOIdrainResistance);
}
else
{

d_Rs_dTemp_t_GND = d_Rs_dV_p_GND = d_Rs_dV_b_GND = d_Rs_dV_gi_p = d_Rs_dV_db_di = d_Rs_dV_sb_si = d_Rs_dV_b_si = d_Rs_dV_e_si = d_Rs_dV_di_si = d_Rs_dV_gi_si =  0.0;
Rs = B4SOIsourceResistance;

d_Rd_dTemp_t_GND = d_Rd_dV_p_GND = d_Rd_dV_b_GND = d_Rd_dV_gi_p = d_Rd_dV_db_di = d_Rd_dV_sb_si = d_Rd_dV_b_si = d_Rd_dV_e_si = d_Rd_dV_di_si = d_Rd_dV_gi_si =  0.0;
Rd = B4SOIdrainResistance;
}
if ((B4SOIrdsMod==2))
{

d_Rs_dTemp_t_GND = d_Rs_dV_p_GND = d_Rs_dV_b_GND = d_Rs_dV_gi_p = d_Rs_dV_db_di = d_Rs_dV_sb_si = d_Rs_dV_b_si = d_Rs_dV_e_si = d_Rs_dV_di_si = d_Rs_dV_gi_si =  0.0;
Rs = 0;

d_Rd_dTemp_t_GND = d_Rd_dV_p_GND = d_Rd_dV_b_GND = d_Rd_dV_gi_p = d_Rd_dV_db_di = d_Rd_dV_sb_si = d_Rd_dV_b_si = d_Rd_dV_e_si = d_Rd_dV_di_si = d_Rd_dV_gi_si =  0.0;
Rd = 0;
}

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si =  0.0;
d_T1_dV_e_si = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_e_si+0.5*d_Abulk_dV_e_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_e_si)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dV_e_si*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
d_T1_dV_b_si = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_b_si+0.5*d_Abulk_dV_b_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_b_si)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dV_b_si*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
d_T1_dV_b_GND = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_b_GND+0.5*d_Abulk_dV_b_GND*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_b_GND)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dV_b_GND*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
d_T1_dV_p_GND = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_p_GND+0.5*d_Abulk_dV_p_GND*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_p_GND)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dV_p_GND*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
d_T1_dTemp_t_GND = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dTemp_t_GND+0.5*d_Abulk_dTemp_t_GND*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dTemp_t_GND)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dTemp_t_GND*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
d_T1_dV_gi_si = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_gi_si+0.5*d_Abulk_dV_gi_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_gi_si)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dV_gi_si*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
d_T1_dV_di_si = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dV_di_si+0.5*d_Abulk_dV_di_si*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dV_di_si)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dV_di_si*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
T1 = (Vgsteff*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
B4SOIqinv = (((((-B4SOIcox)*pParam_B4SOIweff)*B4SOInf)*Leff)*T1);
if ((B4SOInf!=1))
{

d_Ids_dV_gi_si = d_Ids_dV_gi_si*B4SOInf;
d_Ids_dV_di_si = d_Ids_dV_di_si*B4SOInf;
d_Ids_dV_e_si = d_Ids_dV_e_si*B4SOInf;
d_Ids_dV_b_si = d_Ids_dV_b_si*B4SOInf;
d_Ids_dV_b_GND = d_Ids_dV_b_GND*B4SOInf;
d_Ids_dV_p_GND = d_Ids_dV_p_GND*B4SOInf;
d_Ids_dTemp_t_GND = d_Ids_dTemp_t_GND*B4SOInf;
Ids = (Ids*B4SOInf);

d_Ic_dV_b_GND = d_Ic_dV_b_GND*B4SOInf;
d_Ic_dV_p_GND = d_Ic_dV_p_GND*B4SOInf;
d_Ic_dTemp_t_GND = d_Ic_dTemp_t_GND*B4SOInf;
d_Ic_dV_e_si = d_Ic_dV_e_si*B4SOInf;
d_Ic_dV_gi_si = d_Ic_dV_gi_si*B4SOInf;
d_Ic_dV_di_si = d_Ic_dV_di_si*B4SOInf;
d_Ic_dV_b_si = d_Ic_dV_b_si*B4SOInf;
d_Ic_dV_sb_si = d_Ic_dV_sb_si*B4SOInf;
d_Ic_dV_db_di = d_Ic_dV_db_di*B4SOInf;
Ic = (Ic*B4SOInf);
B4SOIidovVds = (B4SOIidovVds*B4SOInf);

d_Ibs_dV_b_GND = d_Ibs_dV_b_GND*B4SOInf;
d_Ibs_dV_p_GND = d_Ibs_dV_p_GND*B4SOInf;
d_Ibs_dTemp_t_GND = d_Ibs_dTemp_t_GND*B4SOInf;
d_Ibs_dV_e_si = d_Ibs_dV_e_si*B4SOInf;
d_Ibs_dV_gi_si = d_Ibs_dV_gi_si*B4SOInf;
d_Ibs_dV_di_si = d_Ibs_dV_di_si*B4SOInf;
d_Ibs_dV_b_si = d_Ibs_dV_b_si*B4SOInf;
d_Ibs_dV_sb_si = d_Ibs_dV_sb_si*B4SOInf;
d_Ibs_dV_db_di = d_Ibs_dV_db_di*B4SOInf;
Ibs = (Ibs*B4SOInf);

d_Ibd_dV_b_GND = d_Ibd_dV_b_GND*B4SOInf;
d_Ibd_dV_p_GND = d_Ibd_dV_p_GND*B4SOInf;
d_Ibd_dTemp_t_GND = d_Ibd_dTemp_t_GND*B4SOInf;
d_Ibd_dV_e_si = d_Ibd_dV_e_si*B4SOInf;
d_Ibd_dV_gi_si = d_Ibd_dV_gi_si*B4SOInf;
d_Ibd_dV_di_si = d_Ibd_dV_di_si*B4SOInf;
d_Ibd_dV_b_si = d_Ibd_dV_b_si*B4SOInf;
d_Ibd_dV_sb_si = d_Ibd_dV_sb_si*B4SOInf;
d_Ibd_dV_db_di = d_Ibd_dV_db_di*B4SOInf;
Ibd = (Ibd*B4SOInf);

d_Igcs_dV_b_GND = d_Igcs_dV_b_GND*B4SOInf;
d_Igcs_dV_p_GND = d_Igcs_dV_p_GND*B4SOInf;
d_Igcs_dTemp_t_GND = d_Igcs_dTemp_t_GND*B4SOInf;
d_Igcs_dV_gi_si = d_Igcs_dV_gi_si*B4SOInf;
d_Igcs_dV_di_si = d_Igcs_dV_di_si*B4SOInf;
d_Igcs_dV_e_si = d_Igcs_dV_e_si*B4SOInf;
d_Igcs_dV_b_si = d_Igcs_dV_b_si*B4SOInf;
d_Igcs_dV_sb_si = d_Igcs_dV_sb_si*B4SOInf;
d_Igcs_dV_db_di = d_Igcs_dV_db_di*B4SOInf;
Igcs = (Igcs*B4SOInf);

d_Igcd_dV_b_GND = d_Igcd_dV_b_GND*B4SOInf;
d_Igcd_dV_p_GND = d_Igcd_dV_p_GND*B4SOInf;
d_Igcd_dTemp_t_GND = d_Igcd_dTemp_t_GND*B4SOInf;
d_Igcd_dV_gi_si = d_Igcd_dV_gi_si*B4SOInf;
d_Igcd_dV_di_si = d_Igcd_dV_di_si*B4SOInf;
d_Igcd_dV_e_si = d_Igcd_dV_e_si*B4SOInf;
d_Igcd_dV_b_si = d_Igcd_dV_b_si*B4SOInf;
d_Igcd_dV_sb_si = d_Igcd_dV_sb_si*B4SOInf;
d_Igcd_dV_db_di = d_Igcd_dV_db_di*B4SOInf;
Igcd = (Igcd*B4SOInf);

d_Igs_dV_sb_si = d_Igs_dV_sb_si*B4SOInf;
d_Igs_dV_db_di = d_Igs_dV_db_di*B4SOInf;
d_Igs_dV_b_GND = d_Igs_dV_b_GND*B4SOInf;
d_Igs_dV_p_GND = d_Igs_dV_p_GND*B4SOInf;
d_Igs_dTemp_t_GND = d_Igs_dTemp_t_GND*B4SOInf;
d_Igs_dV_e_si = d_Igs_dV_e_si*B4SOInf;
d_Igs_dV_gi_si = d_Igs_dV_gi_si*B4SOInf;
d_Igs_dV_di_si = d_Igs_dV_di_si*B4SOInf;
d_Igs_dV_b_si = d_Igs_dV_b_si*B4SOInf;
Igs = (Igs*B4SOInf);

d_Igd_dV_sb_si = d_Igd_dV_sb_si*B4SOInf;
d_Igd_dV_db_di = d_Igd_dV_db_di*B4SOInf;
d_Igd_dV_b_GND = d_Igd_dV_b_GND*B4SOInf;
d_Igd_dV_p_GND = d_Igd_dV_p_GND*B4SOInf;
d_Igd_dTemp_t_GND = d_Igd_dTemp_t_GND*B4SOInf;
d_Igd_dV_e_si = d_Igd_dV_e_si*B4SOInf;
d_Igd_dV_gi_si = d_Igd_dV_gi_si*B4SOInf;
d_Igd_dV_di_si = d_Igd_dV_di_si*B4SOInf;
d_Igd_dV_b_si = d_Igd_dV_b_si*B4SOInf;
Igd = (Igd*B4SOInf);

d_Iii_dV_gi_si = d_Iii_dV_gi_si*B4SOInf;
d_Iii_dV_e_si = d_Iii_dV_e_si*B4SOInf;
d_Iii_dV_b_si = d_Iii_dV_b_si*B4SOInf;
d_Iii_dV_sb_si = d_Iii_dV_sb_si*B4SOInf;
d_Iii_dV_db_di = d_Iii_dV_db_di*B4SOInf;
d_Iii_dV_gi_p = d_Iii_dV_gi_p*B4SOInf;
d_Iii_dV_b_GND = d_Iii_dV_b_GND*B4SOInf;
d_Iii_dV_p_GND = d_Iii_dV_p_GND*B4SOInf;
d_Iii_dTemp_t_GND = d_Iii_dTemp_t_GND*B4SOInf;
d_Iii_dV_di_si = d_Iii_dV_di_si*B4SOInf;
Iii = (Iii*B4SOInf);

d_B4SOIig_dV_b_GND = d_B4SOIig_dV_b_GND*B4SOInf;
d_B4SOIig_dV_p_GND = d_B4SOIig_dV_p_GND*B4SOInf;
d_B4SOIig_dTemp_t_GND = d_B4SOIig_dTemp_t_GND*B4SOInf;
d_B4SOIig_dV_gi_si = d_B4SOIig_dV_gi_si*B4SOInf;
d_B4SOIig_dV_di_si = d_B4SOIig_dV_di_si*B4SOInf;
d_B4SOIig_dV_e_si = d_B4SOIig_dV_e_si*B4SOInf;
d_B4SOIig_dV_b_si = d_B4SOIig_dV_b_si*B4SOInf;
d_B4SOIig_dV_sb_si = d_B4SOIig_dV_sb_si*B4SOInf;
d_B4SOIig_dV_db_di = d_B4SOIig_dV_db_di*B4SOInf;
B4SOIig = (B4SOIig*B4SOInf);

d_Igidl_dV_b_GND = d_Igidl_dV_b_GND*B4SOInf;
d_Igidl_dV_p_GND = d_Igidl_dV_p_GND*B4SOInf;
d_Igidl_dTemp_t_GND = d_Igidl_dTemp_t_GND*B4SOInf;
d_Igidl_dV_gi_si = d_Igidl_dV_gi_si*B4SOInf;
d_Igidl_dV_di_si = d_Igidl_dV_di_si*B4SOInf;
d_Igidl_dV_e_si = d_Igidl_dV_e_si*B4SOInf;
d_Igidl_dV_b_si = d_Igidl_dV_b_si*B4SOInf;
Igidl = (Igidl*B4SOInf);

d_Igisl_dV_b_GND = d_Igisl_dV_b_GND*B4SOInf;
d_Igisl_dV_p_GND = d_Igisl_dV_p_GND*B4SOInf;
d_Igisl_dTemp_t_GND = d_Igisl_dTemp_t_GND*B4SOInf;
d_Igisl_dV_gi_si = d_Igisl_dV_gi_si*B4SOInf;
d_Igisl_dV_di_si = d_Igisl_dV_di_si*B4SOInf;
d_Igisl_dV_e_si = d_Igisl_dV_e_si*B4SOInf;
d_Igisl_dV_b_si = d_Igisl_dV_b_si*B4SOInf;
Igisl = (Igisl*B4SOInf);
}
B4SOIgm = (B4SOItype*(+d_Ids_dV_gi_si));
if ((B4SOImode>0))
{
B4SOIgds = (B4SOItype*(+d_Ids_dV_di_si));
}
else
{
B4SOIgds = (B4SOItype*(-d_Ids_dV_gi_si-d_Ids_dV_di_si-d_Ids_dV_e_si-d_Ids_dV_b_si));
}
B4SOIgmbs = (B4SOItype*(+d_Ids_dV_b_si+d_Ids_dV_b_GND));
CoxWL = (B4SOIcox*((((pParam_B4SOIweffCV/B4SOInseg)*B4SOInf)*pParam_B4SOIleffCV)+B4SOIagbcp));
CoxWLb = ((B4SOIfbody*B4SOIcox)*((((pParam_B4SOIweffCV/B4SOInseg)*B4SOInf)*pParam_B4SOIleffCVb)+B4SOIagbcp));
CoxWL2 = (B4SOIcox*B4SOIagbcp2);
CoxWLb2 = ((B4SOIfbody*B4SOIcox)*B4SOIagbcp2);

d_Vgst_dV_e_si = (-d_Vth_CV_dV_e_si);
d_Vgst_dV_b_si = (-d_Vth_CV_dV_b_si);
d_Vgst_dV_b_GND = (d_Vgs_eff_dV_b_GND-d_Vth_CV_dV_b_GND);
d_Vgst_dV_p_GND = (d_Vgs_eff_dV_p_GND-d_Vth_CV_dV_p_GND);
d_Vgst_dTemp_t_GND = (d_Vgs_eff_dTemp_t_GND-d_Vth_CV_dTemp_t_GND);
d_Vgst_dV_gi_si = (d_Vgs_eff_dV_gi_si-d_Vth_CV_dV_gi_si);
d_Vgst_dV_di_si = (d_Vgs_eff_dV_di_si-d_Vth_CV_dV_di_si);
Vgst = (Vgs_eff-Vth_CV);

d_T10_dV_db_di = d_T10_dV_sb_si =  0.0;
d_T10_dV_b_GND = (n_CV*d_Vtm_dV_b_GND+d_n_CV_dV_b_GND*Vtm);
d_T10_dV_p_GND = (n_CV*d_Vtm_dV_p_GND+d_n_CV_dV_p_GND*Vtm);
d_T10_dTemp_t_GND = (n_CV*d_Vtm_dTemp_t_GND+d_n_CV_dTemp_t_GND*Vtm);
d_T10_dV_gi_si = d_n_CV_dV_gi_si*Vtm;
d_T10_dV_di_si = d_n_CV_dV_di_si*Vtm;
d_T10_dV_e_si = d_n_CV_dV_e_si*Vtm;
d_T10_dV_b_si = d_n_CV_dV_b_si*Vtm;
T10 = (n_CV*Vtm);

d_VgstNVt_dV_sb_si = (-(pParam_B4SOImstar*Vgst)*d_T10_dV_sb_si/T10/T10);
d_VgstNVt_dV_db_di = (-(pParam_B4SOImstar*Vgst)*d_T10_dV_db_di/T10/T10);
d_VgstNVt_dV_e_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_e_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_e_si)/T10/T10);
d_VgstNVt_dV_b_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_b_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_b_si)/T10/T10);
d_VgstNVt_dV_b_GND = ((T10*pParam_B4SOImstar*d_Vgst_dV_b_GND-(pParam_B4SOImstar*Vgst)*d_T10_dV_b_GND)/T10/T10);
d_VgstNVt_dV_p_GND = ((T10*pParam_B4SOImstar*d_Vgst_dV_p_GND-(pParam_B4SOImstar*Vgst)*d_T10_dV_p_GND)/T10/T10);
d_VgstNVt_dTemp_t_GND = ((T10*pParam_B4SOImstar*d_Vgst_dTemp_t_GND-(pParam_B4SOImstar*Vgst)*d_T10_dTemp_t_GND)/T10/T10);
d_VgstNVt_dV_gi_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_gi_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_gi_si)/T10/T10);
d_VgstNVt_dV_di_si = ((T10*pParam_B4SOImstar*d_Vgst_dV_di_si-(pParam_B4SOImstar*Vgst)*d_T10_dV_di_si)/T10/T10);
VgstNVt = ((pParam_B4SOImstar*Vgst)/T10);

d_noff_dV_b_GND = ((n_CV*pParam_B4SOInoff)*d_Vtm_dV_b_GND+d_n_CV_dV_b_GND*pParam_B4SOInoff*Vtm);
d_noff_dV_p_GND = ((n_CV*pParam_B4SOInoff)*d_Vtm_dV_p_GND+d_n_CV_dV_p_GND*pParam_B4SOInoff*Vtm);
d_noff_dTemp_t_GND = ((n_CV*pParam_B4SOInoff)*d_Vtm_dTemp_t_GND+d_n_CV_dTemp_t_GND*pParam_B4SOInoff*Vtm);
d_noff_dV_gi_si = d_n_CV_dV_gi_si*pParam_B4SOInoff*Vtm;
d_noff_dV_di_si = d_n_CV_dV_di_si*pParam_B4SOInoff*Vtm;
d_noff_dV_e_si = d_n_CV_dV_e_si*pParam_B4SOInoff*Vtm;
d_noff_dV_b_si = d_n_CV_dV_b_si*pParam_B4SOInoff*Vtm;
noff = ((n_CV*pParam_B4SOInoff)*Vtm);

d_noff2_dV_b_GND = ((n_CV*pParam_B4SOInoff2)*d_Vtm_dV_b_GND+d_n_CV_dV_b_GND*pParam_B4SOInoff2*Vtm);
d_noff2_dV_p_GND = ((n_CV*pParam_B4SOInoff2)*d_Vtm_dV_p_GND+d_n_CV_dV_p_GND*pParam_B4SOInoff2*Vtm);
d_noff2_dTemp_t_GND = ((n_CV*pParam_B4SOInoff2)*d_Vtm_dTemp_t_GND+d_n_CV_dTemp_t_GND*pParam_B4SOInoff2*Vtm);
d_noff2_dV_gi_si = d_n_CV_dV_gi_si*pParam_B4SOInoff2*Vtm;
d_noff2_dV_di_si = d_n_CV_dV_di_si*pParam_B4SOInoff2*Vtm;
d_noff2_dV_e_si = d_n_CV_dV_e_si*pParam_B4SOInoff2*Vtm;
d_noff2_dV_b_si = d_n_CV_dV_b_si*pParam_B4SOInoff2*Vtm;
noff2 = ((n_CV*pParam_B4SOInoff2)*Vtm);
if ((B4SOIvgstcvMod==0))
{
if (((VgstNVt>(-100.0))&&(VgstNVt<100.0)))
{
{
double value_exp_0 = exp(VgstNVt);
double value_exp_1 = exp(VgstNVt);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;

d_ExpVgst_dV_gi_p =  0.0;
d_ExpVgst_dV_e_si = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_e_si))+(deriv_exp_0_d0*(d_VgstNVt_dV_e_si))*value_exp_1);
d_ExpVgst_dV_b_si = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_b_si))+(deriv_exp_0_d0*(d_VgstNVt_dV_b_si))*value_exp_1);
d_ExpVgst_dV_b_GND = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_b_GND))+(deriv_exp_0_d0*(d_VgstNVt_dV_b_GND))*value_exp_1);
d_ExpVgst_dV_p_GND = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_p_GND))+(deriv_exp_0_d0*(d_VgstNVt_dV_p_GND))*value_exp_1);
d_ExpVgst_dTemp_t_GND = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dTemp_t_GND))+(deriv_exp_0_d0*(d_VgstNVt_dTemp_t_GND))*value_exp_1);
d_ExpVgst_dV_gi_si = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_gi_si))+(deriv_exp_0_d0*(d_VgstNVt_dV_gi_si))*value_exp_1);
d_ExpVgst_dV_di_si = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_di_si))+(deriv_exp_0_d0*(d_VgstNVt_dV_di_si))*value_exp_1);
d_ExpVgst_dV_sb_si = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_sb_si))+(deriv_exp_0_d0*(d_VgstNVt_dV_sb_si))*value_exp_1);
d_ExpVgst_dV_db_di = (value_exp_0*(deriv_exp_1_d0*(d_VgstNVt_dV_db_di))+(deriv_exp_0_d0*(d_VgstNVt_dV_db_di))*value_exp_1);
ExpVgst = (value_exp_0*value_exp_1);
}
{
double value_exp_0 = exp((-(pParam_B4SOIdelvt/noff)));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst_dV_gi_p =  0.0;
d_ExpVgst_dV_b_GND = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_b_GND/noff/noff))))+d_ExpVgst_dV_b_GND*value_exp_0);
d_ExpVgst_dV_p_GND = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_p_GND/noff/noff))))+d_ExpVgst_dV_p_GND*value_exp_0);
d_ExpVgst_dTemp_t_GND = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dTemp_t_GND/noff/noff))))+d_ExpVgst_dTemp_t_GND*value_exp_0);
d_ExpVgst_dV_e_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_e_si/noff/noff))))+d_ExpVgst_dV_e_si*value_exp_0);
d_ExpVgst_dV_gi_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_gi_si/noff/noff))))+d_ExpVgst_dV_gi_si*value_exp_0);
d_ExpVgst_dV_di_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_di_si/noff/noff))))+d_ExpVgst_dV_di_si*value_exp_0);
d_ExpVgst_dV_b_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_b_si/noff/noff))))+d_ExpVgst_dV_b_si*value_exp_0);
d_ExpVgst_dV_sb_si = d_ExpVgst_dV_sb_si*value_exp_0;
d_ExpVgst_dV_db_di = d_ExpVgst_dV_db_di*value_exp_0;
ExpVgst = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));

d_Vgsteff_dV_gi_p =  0.0;
d_Vgsteff_dV_sb_si = noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_sb_si);
d_Vgsteff_dV_db_di = noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_db_di);
d_Vgsteff_dV_b_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_b_GND)+d_noff_dV_b_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_p_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_p_GND)+d_noff_dV_p_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dTemp_t_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dTemp_t_GND)+d_noff_dTemp_t_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_gi_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_gi_si)+d_noff_dV_gi_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_di_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_di_si)+d_noff_dV_di_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_e_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_e_si)+d_noff_dV_e_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_b_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_b_si)+d_noff_dV_b_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
Vgsteff = (noff*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
if ((B4SOIagbcp2>0))
{
{
double value_exp_0 = exp((((-eggbcp2)/noff2)/(Vtm*Vtm)));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst2_dV_gi_p =  0.0;
d_ExpVgst2_dV_b_GND = (ExpVgst*(deriv_exp_0_d0*((((Vtm*Vtm)*(-(-eggbcp2)*d_noff2_dV_b_GND/noff2/noff2)-((-eggbcp2)/noff2)*(Vtm*d_Vtm_dV_b_GND+d_Vtm_dV_b_GND*Vtm))/(Vtm*Vtm)/(Vtm*Vtm))))+d_ExpVgst_dV_b_GND*value_exp_0);
d_ExpVgst2_dV_p_GND = (ExpVgst*(deriv_exp_0_d0*((((Vtm*Vtm)*(-(-eggbcp2)*d_noff2_dV_p_GND/noff2/noff2)-((-eggbcp2)/noff2)*(Vtm*d_Vtm_dV_p_GND+d_Vtm_dV_p_GND*Vtm))/(Vtm*Vtm)/(Vtm*Vtm))))+d_ExpVgst_dV_p_GND*value_exp_0);
d_ExpVgst2_dTemp_t_GND = (ExpVgst*(deriv_exp_0_d0*((((Vtm*Vtm)*(-(-eggbcp2)*d_noff2_dTemp_t_GND/noff2/noff2)-((-eggbcp2)/noff2)*(Vtm*d_Vtm_dTemp_t_GND+d_Vtm_dTemp_t_GND*Vtm))/(Vtm*Vtm)/(Vtm*Vtm))))+d_ExpVgst_dTemp_t_GND*value_exp_0);
d_ExpVgst2_dV_e_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_e_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_e_si*value_exp_0);
d_ExpVgst2_dV_gi_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_gi_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_gi_si*value_exp_0);
d_ExpVgst2_dV_di_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_di_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_di_si*value_exp_0);
d_ExpVgst2_dV_b_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_b_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_b_si*value_exp_0);
d_ExpVgst2_dV_sb_si = d_ExpVgst_dV_sb_si*value_exp_0;
d_ExpVgst2_dV_db_di = d_ExpVgst_dV_db_di*value_exp_0;
ExpVgst2 = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst2));

d_Vgsteff2_dV_gi_p =  0.0;
d_Vgsteff2_dV_sb_si = noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_sb_si);
d_Vgsteff2_dV_db_di = noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_db_di);
d_Vgsteff2_dV_b_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_b_GND)+d_noff2_dV_b_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_p_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_p_GND)+d_noff2_dV_p_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dTemp_t_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dTemp_t_GND)+d_noff2_dTemp_t_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_gi_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_gi_si)+d_noff2_dV_gi_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_di_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_di_si)+d_noff2_dV_di_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_e_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_e_si)+d_noff2_dV_e_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_b_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_b_si)+d_noff2_dV_b_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
Vgsteff2 = (noff2*evaluator_lln_0.getValues((1.0+ExpVgst2)));
}
}
}
}
else
{
if ((B4SOIvgstcvMod==1))
{
if (((VgstNVt>(-100.0))&&(VgstNVt<100.0)))
{
{
double value_exp_0 = exp((VgstNVt/(pParam_B4SOImstar*pParam_B4SOInoff)));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst_dV_gi_p =  0.0;
d_ExpVgst_dV_e_si = (deriv_exp_0_d0*((d_VgstNVt_dV_e_si/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dV_b_si = (deriv_exp_0_d0*((d_VgstNVt_dV_b_si/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dV_b_GND = (deriv_exp_0_d0*((d_VgstNVt_dV_b_GND/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dV_p_GND = (deriv_exp_0_d0*((d_VgstNVt_dV_p_GND/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dTemp_t_GND = (deriv_exp_0_d0*((d_VgstNVt_dTemp_t_GND/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dV_gi_si = (deriv_exp_0_d0*((d_VgstNVt_dV_gi_si/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dV_di_si = (deriv_exp_0_d0*((d_VgstNVt_dV_di_si/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dV_sb_si = (deriv_exp_0_d0*((d_VgstNVt_dV_sb_si/(pParam_B4SOImstar*pParam_B4SOInoff))));
d_ExpVgst_dV_db_di = (deriv_exp_0_d0*((d_VgstNVt_dV_db_di/(pParam_B4SOImstar*pParam_B4SOInoff))));
ExpVgst = value_exp_0;
}
{
double value_exp_0 = exp((-(pParam_B4SOIdelvt/noff)));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst_dV_gi_p =  0.0;
d_ExpVgst_dV_b_GND = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_b_GND/noff/noff))))+d_ExpVgst_dV_b_GND*value_exp_0);
d_ExpVgst_dV_p_GND = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_p_GND/noff/noff))))+d_ExpVgst_dV_p_GND*value_exp_0);
d_ExpVgst_dTemp_t_GND = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dTemp_t_GND/noff/noff))))+d_ExpVgst_dTemp_t_GND*value_exp_0);
d_ExpVgst_dV_e_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_e_si/noff/noff))))+d_ExpVgst_dV_e_si*value_exp_0);
d_ExpVgst_dV_gi_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_gi_si/noff/noff))))+d_ExpVgst_dV_gi_si*value_exp_0);
d_ExpVgst_dV_di_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_di_si/noff/noff))))+d_ExpVgst_dV_di_si*value_exp_0);
d_ExpVgst_dV_b_si = (ExpVgst*(deriv_exp_0_d0*((-(-pParam_B4SOIdelvt*d_noff_dV_b_si/noff/noff))))+d_ExpVgst_dV_b_si*value_exp_0);
d_ExpVgst_dV_sb_si = d_ExpVgst_dV_sb_si*value_exp_0;
d_ExpVgst_dV_db_di = d_ExpVgst_dV_db_di*value_exp_0;
ExpVgst = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));

d_Vgsteff_dV_gi_p =  0.0;
d_Vgsteff_dV_sb_si = noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_sb_si);
d_Vgsteff_dV_db_di = noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_db_di);
d_Vgsteff_dV_b_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_b_GND)+d_noff_dV_b_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_p_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_p_GND)+d_noff_dV_p_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dTemp_t_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dTemp_t_GND)+d_noff_dTemp_t_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_gi_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_gi_si)+d_noff_dV_gi_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_di_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_di_si)+d_noff_dV_di_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_e_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_e_si)+d_noff_dV_e_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_Vgsteff_dV_b_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_b_si)+d_noff_dV_b_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
Vgsteff = (noff*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
if ((B4SOIagbcp2>0))
{
{
double value_exp_0 = exp((((-eggbcp2)/noff2)/(Vtm*Vtm)));
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst2_dV_gi_p =  0.0;
d_ExpVgst2_dV_b_GND = (ExpVgst*(deriv_exp_0_d0*((((Vtm*Vtm)*(-(-eggbcp2)*d_noff2_dV_b_GND/noff2/noff2)-((-eggbcp2)/noff2)*(Vtm*d_Vtm_dV_b_GND+d_Vtm_dV_b_GND*Vtm))/(Vtm*Vtm)/(Vtm*Vtm))))+d_ExpVgst_dV_b_GND*value_exp_0);
d_ExpVgst2_dV_p_GND = (ExpVgst*(deriv_exp_0_d0*((((Vtm*Vtm)*(-(-eggbcp2)*d_noff2_dV_p_GND/noff2/noff2)-((-eggbcp2)/noff2)*(Vtm*d_Vtm_dV_p_GND+d_Vtm_dV_p_GND*Vtm))/(Vtm*Vtm)/(Vtm*Vtm))))+d_ExpVgst_dV_p_GND*value_exp_0);
d_ExpVgst2_dTemp_t_GND = (ExpVgst*(deriv_exp_0_d0*((((Vtm*Vtm)*(-(-eggbcp2)*d_noff2_dTemp_t_GND/noff2/noff2)-((-eggbcp2)/noff2)*(Vtm*d_Vtm_dTemp_t_GND+d_Vtm_dTemp_t_GND*Vtm))/(Vtm*Vtm)/(Vtm*Vtm))))+d_ExpVgst_dTemp_t_GND*value_exp_0);
d_ExpVgst2_dV_e_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_e_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_e_si*value_exp_0);
d_ExpVgst2_dV_gi_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_gi_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_gi_si*value_exp_0);
d_ExpVgst2_dV_di_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_di_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_di_si*value_exp_0);
d_ExpVgst2_dV_b_si = (ExpVgst*(deriv_exp_0_d0*(((-(-eggbcp2)*d_noff2_dV_b_si/noff2/noff2)/(Vtm*Vtm))))+d_ExpVgst_dV_b_si*value_exp_0);
d_ExpVgst2_dV_sb_si = d_ExpVgst_dV_sb_si*value_exp_0;
d_ExpVgst2_dV_db_di = d_ExpVgst_dV_db_di*value_exp_0;
ExpVgst2 = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst2));

d_Vgsteff2_dV_gi_p =  0.0;
d_Vgsteff2_dV_sb_si = noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_sb_si);
d_Vgsteff2_dV_db_di = noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_db_di);
d_Vgsteff2_dV_b_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_b_GND)+d_noff2_dV_b_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_p_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_p_GND)+d_noff2_dV_p_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dTemp_t_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dTemp_t_GND)+d_noff2_dTemp_t_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_gi_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_gi_si)+d_noff2_dV_gi_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_di_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_di_si)+d_noff2_dV_di_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_e_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_e_si)+d_noff2_dV_e_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_Vgsteff2_dV_b_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_b_si)+d_noff2_dV_b_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
Vgsteff2 = (noff2*evaluator_lln_0.getValues((1.0+ExpVgst2)));
}
}
}
}
else
{

d_VgstNVt_dV_db_di = d_VgstNVt_dV_sb_si =  0.0;
d_VgstNVt_dV_e_si = ((noff*pParam_B4SOImstarcv*d_Vgst_dV_e_si-(pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))*d_noff_dV_e_si)/noff/noff);
d_VgstNVt_dV_b_si = ((noff*pParam_B4SOImstarcv*d_Vgst_dV_b_si-(pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))*d_noff_dV_b_si)/noff/noff);
d_VgstNVt_dV_b_GND = ((noff*pParam_B4SOImstarcv*d_Vgst_dV_b_GND-(pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))*d_noff_dV_b_GND)/noff/noff);
d_VgstNVt_dV_p_GND = ((noff*pParam_B4SOImstarcv*d_Vgst_dV_p_GND-(pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))*d_noff_dV_p_GND)/noff/noff);
d_VgstNVt_dTemp_t_GND = ((noff*pParam_B4SOImstarcv*d_Vgst_dTemp_t_GND-(pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))*d_noff_dTemp_t_GND)/noff/noff);
d_VgstNVt_dV_gi_si = ((noff*pParam_B4SOImstarcv*d_Vgst_dV_gi_si-(pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))*d_noff_dV_gi_si)/noff/noff);
d_VgstNVt_dV_di_si = ((noff*pParam_B4SOImstarcv*d_Vgst_dV_di_si-(pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))*d_noff_dV_di_si)/noff/noff);
VgstNVt = ((pParam_B4SOImstarcv*(Vgst-pParam_B4SOIdelvt))/noff);

d_ExpArg_dV_e_si = ((noff*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_e_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))*d_noff_dV_e_si)/noff/noff);
d_ExpArg_dV_b_si = ((noff*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_b_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))*d_noff_dV_b_si)/noff/noff);
d_ExpArg_dV_b_GND = ((noff*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_b_GND)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))*d_noff_dV_b_GND)/noff/noff);
d_ExpArg_dV_p_GND = ((noff*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_p_GND)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))*d_noff_dV_p_GND)/noff/noff);
d_ExpArg_dTemp_t_GND = ((noff*(-(1-pParam_B4SOImstarcv)*d_Vgst_dTemp_t_GND)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))*d_noff_dTemp_t_GND)/noff/noff);
d_ExpArg_dV_gi_si = ((noff*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_gi_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))*d_noff_dV_gi_si)/noff/noff);
d_ExpArg_dV_di_si = ((noff*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_di_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))*d_noff_dV_di_si)/noff/noff);
ExpArg = ((pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*(Vgst-pParam_B4SOIdelvt)))/noff);
if ((VgstNVt>100.0))
{

d_Vgsteff_dV_gi_p = d_Vgsteff_dV_db_di = d_Vgsteff_dV_sb_si =  0.0;
d_Vgsteff_dV_e_si = d_Vgst_dV_e_si;
d_Vgsteff_dV_b_si = d_Vgst_dV_b_si;
d_Vgsteff_dV_b_GND = d_Vgst_dV_b_GND;
d_Vgsteff_dV_p_GND = d_Vgst_dV_p_GND;
d_Vgsteff_dTemp_t_GND = d_Vgst_dTemp_t_GND;
d_Vgsteff_dV_gi_si = d_Vgst_dV_gi_si;
d_Vgsteff_dV_di_si = d_Vgst_dV_di_si;
Vgsteff = (Vgst-pParam_B4SOIdelvt);
}
else
{
if ((ExpArg>100.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = ((noff*d_Vgst_dV_e_si-((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)*d_noff_dV_e_si)/noff/noff);
d_T0_dV_b_si = ((noff*d_Vgst_dV_b_si-((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)*d_noff_dV_b_si)/noff/noff);
d_T0_dV_b_GND = ((noff*d_Vgst_dV_b_GND-((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)*d_noff_dV_b_GND)/noff/noff);
d_T0_dV_p_GND = ((noff*d_Vgst_dV_p_GND-((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)*d_noff_dV_p_GND)/noff/noff);
d_T0_dTemp_t_GND = ((noff*d_Vgst_dTemp_t_GND-((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)*d_noff_dTemp_t_GND)/noff/noff);
d_T0_dV_gi_si = ((noff*d_Vgst_dV_gi_si-((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)*d_noff_dV_gi_si)/noff/noff);
d_T0_dV_di_si = ((noff*d_Vgst_dV_di_si-((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)*d_noff_dV_di_si)/noff/noff);
T0 = (((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)/noff);
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_ExpVgst_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_ExpVgst_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_ExpVgst_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_ExpVgst_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_ExpVgst_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_ExpVgst_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_ExpVgst_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_ExpVgst_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
d_ExpVgst_dV_gi_p = (deriv_exp_0_d0*(d_T0_dV_gi_p));
ExpVgst = value_exp_0;
}

d_Vgsteff_dV_e_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_e_si;
d_Vgsteff_dV_gi_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_gi_si;
d_Vgsteff_dV_di_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_di_si;
d_Vgsteff_dV_b_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_b_si;
d_Vgsteff_dV_sb_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_sb_si;
d_Vgsteff_dV_db_di = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_db_di;
d_Vgsteff_dV_gi_p = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_gi_p;
d_Vgsteff_dV_b_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_b_GND+((Vtm*d_cdep0_dV_b_GND+d_Vtm_dV_b_GND*cdep0)/B4SOIcox)*ExpVgst);
d_Vgsteff_dV_p_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dV_p_GND+((Vtm*d_cdep0_dV_p_GND+d_Vtm_dV_p_GND*cdep0)/B4SOIcox)*ExpVgst);
d_Vgsteff_dTemp_t_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst_dTemp_t_GND+((Vtm*d_cdep0_dTemp_t_GND+d_Vtm_dTemp_t_GND*cdep0)/B4SOIcox)*ExpVgst);
Vgsteff = (((Vtm*cdep0)/B4SOIcox)*ExpVgst);
}
else
{
{
double value_exp_0 = exp(VgstNVt);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst_dV_gi_p =  0.0;
d_ExpVgst_dV_e_si = (deriv_exp_0_d0*(d_VgstNVt_dV_e_si));
d_ExpVgst_dV_b_si = (deriv_exp_0_d0*(d_VgstNVt_dV_b_si));
d_ExpVgst_dV_b_GND = (deriv_exp_0_d0*(d_VgstNVt_dV_b_GND));
d_ExpVgst_dV_p_GND = (deriv_exp_0_d0*(d_VgstNVt_dV_p_GND));
d_ExpVgst_dTemp_t_GND = (deriv_exp_0_d0*(d_VgstNVt_dTemp_t_GND));
d_ExpVgst_dV_gi_si = (deriv_exp_0_d0*(d_VgstNVt_dV_gi_si));
d_ExpVgst_dV_di_si = (deriv_exp_0_d0*(d_VgstNVt_dV_di_si));
d_ExpVgst_dV_sb_si = (deriv_exp_0_d0*(d_VgstNVt_dV_sb_si));
d_ExpVgst_dV_db_di = (deriv_exp_0_d0*(d_VgstNVt_dV_db_di));
ExpVgst = value_exp_0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));

d_T1_dV_gm_si =  0.0;
d_T1_dV_sb_si = noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_sb_si);
d_T1_dV_db_di = noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_db_di);
d_T1_dV_gi_p = noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_gi_p);
d_T1_dV_b_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_b_GND)+d_noff_dV_b_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_T1_dV_p_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_p_GND)+d_noff_dV_p_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_T1_dTemp_t_GND = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dTemp_t_GND)+d_noff_dTemp_t_GND*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_T1_dV_gi_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_gi_si)+d_noff_dV_gi_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_T1_dV_di_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_di_si)+d_noff_dV_di_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_T1_dV_e_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_e_si)+d_noff_dV_e_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
d_T1_dV_b_si = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dV_b_si)+d_noff_dV_b_si*evaluator_lln_0.getValues((1.0+ExpVgst)));
T1 = (noff*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
{
double value_exp_0 = exp(ExpArg);
double  deriv_exp_0_d0 = value_exp_0;

d_dT2_dVg_dV_e_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_e_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_b_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_b_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_gi_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_gi_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_di_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_di_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_b_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_b_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dV_b_GND+d_Vtm_dV_b_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_p_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dV_p_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dV_p_GND+d_Vtm_dV_p_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dTemp_t_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg_dTemp_t_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dTemp_t_GND+d_Vtm_dTemp_t_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv);
dT2_dVg = ((((-B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (-((noff*d_dT2_dVg_dV_b_GND+d_noff_dV_b_GND*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_p_GND = (-((noff*d_dT2_dVg_dV_p_GND+d_noff_dV_p_GND*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dTemp_t_GND = (-((noff*d_dT2_dVg_dTemp_t_GND+d_noff_dTemp_t_GND*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_gi_si = (-((noff*d_dT2_dVg_dV_gi_si+d_noff_dV_gi_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_di_si = (-((noff*d_dT2_dVg_dV_di_si+d_noff_dV_di_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_e_si = (-((noff*d_dT2_dVg_dV_e_si+d_noff_dV_e_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_b_si = (-((noff*d_dT2_dVg_dV_b_si+d_noff_dV_b_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
T2 = (pParam_B4SOImstarcv-((noff*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));

d_Vgsteff_dV_b_GND = ((T2*d_T1_dV_b_GND-T1*d_T2_dV_b_GND)/T2/T2);
d_Vgsteff_dV_p_GND = ((T2*d_T1_dV_p_GND-T1*d_T2_dV_p_GND)/T2/T2);
d_Vgsteff_dTemp_t_GND = ((T2*d_T1_dTemp_t_GND-T1*d_T2_dTemp_t_GND)/T2/T2);
d_Vgsteff_dV_gi_si = ((T2*d_T1_dV_gi_si-T1*d_T2_dV_gi_si)/T2/T2);
d_Vgsteff_dV_di_si = ((T2*d_T1_dV_di_si-T1*d_T2_dV_di_si)/T2/T2);
d_Vgsteff_dV_e_si = ((T2*d_T1_dV_e_si-T1*d_T2_dV_e_si)/T2/T2);
d_Vgsteff_dV_b_si = ((T2*d_T1_dV_b_si-T1*d_T2_dV_b_si)/T2/T2);
d_Vgsteff_dV_sb_si = ((T2*d_T1_dV_sb_si-T1*d_T2_dV_sb_si)/T2/T2);
d_Vgsteff_dV_db_di = ((T2*d_T1_dV_db_di-T1*d_T2_dV_db_di)/T2/T2);
d_Vgsteff_dV_gi_p = ((T2*d_T1_dV_gi_p-T1*d_T2_dV_gi_p)/T2/T2);
Vgsteff = (T1/T2);
}
}
if ((B4SOIagbcp2>0))
{

d_VgstNVt2_dV_e_si = ((noff2*pParam_B4SOImstarcv*d_Vgst_dV_e_si-(pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))*d_noff2_dV_e_si)/noff2/noff2);
d_VgstNVt2_dV_b_si = ((noff2*pParam_B4SOImstarcv*d_Vgst_dV_b_si-(pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))*d_noff2_dV_b_si)/noff2/noff2);
d_VgstNVt2_dV_b_GND = ((noff2*pParam_B4SOImstarcv*d_Vgst_dV_b_GND-(pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))*d_noff2_dV_b_GND)/noff2/noff2);
d_VgstNVt2_dV_p_GND = ((noff2*pParam_B4SOImstarcv*d_Vgst_dV_p_GND-(pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))*d_noff2_dV_p_GND)/noff2/noff2);
d_VgstNVt2_dTemp_t_GND = ((noff2*pParam_B4SOImstarcv*d_Vgst_dTemp_t_GND-(pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))*d_noff2_dTemp_t_GND)/noff2/noff2);
d_VgstNVt2_dV_gi_si = ((noff2*pParam_B4SOImstarcv*d_Vgst_dV_gi_si-(pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))*d_noff2_dV_gi_si)/noff2/noff2);
d_VgstNVt2_dV_di_si = ((noff2*pParam_B4SOImstarcv*d_Vgst_dV_di_si-(pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))*d_noff2_dV_di_si)/noff2/noff2);
VgstNVt2 = ((pParam_B4SOImstarcv*((Vgst-pParam_B4SOIdelvt)-eggbcp2))/noff2);

d_ExpArg2_dV_e_si = ((noff2*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_e_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))*d_noff2_dV_e_si)/noff2/noff2);
d_ExpArg2_dV_b_si = ((noff2*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_b_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))*d_noff2_dV_b_si)/noff2/noff2);
d_ExpArg2_dV_b_GND = ((noff2*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_b_GND)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))*d_noff2_dV_b_GND)/noff2/noff2);
d_ExpArg2_dV_p_GND = ((noff2*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_p_GND)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))*d_noff2_dV_p_GND)/noff2/noff2);
d_ExpArg2_dTemp_t_GND = ((noff2*(-(1-pParam_B4SOImstarcv)*d_Vgst_dTemp_t_GND)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))*d_noff2_dTemp_t_GND)/noff2/noff2);
d_ExpArg2_dV_gi_si = ((noff2*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_gi_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))*d_noff2_dV_gi_si)/noff2/noff2);
d_ExpArg2_dV_di_si = ((noff2*(-(1-pParam_B4SOImstarcv)*d_Vgst_dV_di_si)-(pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))*d_noff2_dV_di_si)/noff2/noff2);
ExpArg2 = ((pParam_B4SOIvoffcv-((1-pParam_B4SOImstarcv)*((Vgst-pParam_B4SOIdelvt)-eggbcp2)))/noff2);
if ((VgstNVt2>100.0))
{

d_Vgsteff2_dV_gi_p = d_Vgsteff2_dV_db_di = d_Vgsteff2_dV_sb_si =  0.0;
d_Vgsteff2_dV_e_si = d_Vgst_dV_e_si;
d_Vgsteff2_dV_b_si = d_Vgst_dV_b_si;
d_Vgsteff2_dV_b_GND = d_Vgst_dV_b_GND;
d_Vgsteff2_dV_p_GND = d_Vgst_dV_p_GND;
d_Vgsteff2_dTemp_t_GND = d_Vgst_dTemp_t_GND;
d_Vgsteff2_dV_gi_si = d_Vgst_dV_gi_si;
d_Vgsteff2_dV_di_si = d_Vgst_dV_di_si;
Vgsteff2 = ((Vgst-pParam_B4SOIdelvt)-eggbcp2);
}
else
{
if ((ExpArg2>100.0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = ((noff2*d_Vgst_dV_e_si-(((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)*d_noff2_dV_e_si)/noff2/noff2);
d_T0_dV_b_si = ((noff2*d_Vgst_dV_b_si-(((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)*d_noff2_dV_b_si)/noff2/noff2);
d_T0_dV_b_GND = ((noff2*d_Vgst_dV_b_GND-(((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)*d_noff2_dV_b_GND)/noff2/noff2);
d_T0_dV_p_GND = ((noff2*d_Vgst_dV_p_GND-(((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)*d_noff2_dV_p_GND)/noff2/noff2);
d_T0_dTemp_t_GND = ((noff2*d_Vgst_dTemp_t_GND-(((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)*d_noff2_dTemp_t_GND)/noff2/noff2);
d_T0_dV_gi_si = ((noff2*d_Vgst_dV_gi_si-(((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)*d_noff2_dV_gi_si)/noff2/noff2);
d_T0_dV_di_si = ((noff2*d_Vgst_dV_di_si-(((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)*d_noff2_dV_di_si)/noff2/noff2);
T0 = ((((Vgst-pParam_B4SOIdelvt)-pParam_B4SOIvoffcv)-eggbcp2)/noff2);
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst2_dV_b_GND = (deriv_exp_0_d0*(d_T0_dV_b_GND));
d_ExpVgst2_dV_p_GND = (deriv_exp_0_d0*(d_T0_dV_p_GND));
d_ExpVgst2_dTemp_t_GND = (deriv_exp_0_d0*(d_T0_dTemp_t_GND));
d_ExpVgst2_dV_e_si = (deriv_exp_0_d0*(d_T0_dV_e_si));
d_ExpVgst2_dV_gi_si = (deriv_exp_0_d0*(d_T0_dV_gi_si));
d_ExpVgst2_dV_di_si = (deriv_exp_0_d0*(d_T0_dV_di_si));
d_ExpVgst2_dV_b_si = (deriv_exp_0_d0*(d_T0_dV_b_si));
d_ExpVgst2_dV_sb_si = (deriv_exp_0_d0*(d_T0_dV_sb_si));
d_ExpVgst2_dV_db_di = (deriv_exp_0_d0*(d_T0_dV_db_di));
d_ExpVgst2_dV_gi_p = (deriv_exp_0_d0*(d_T0_dV_gi_p));
ExpVgst2 = value_exp_0;
}

d_Vgsteff2_dV_e_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_e_si;
d_Vgsteff2_dV_gi_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_gi_si;
d_Vgsteff2_dV_di_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_di_si;
d_Vgsteff2_dV_b_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_b_si;
d_Vgsteff2_dV_sb_si = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_sb_si;
d_Vgsteff2_dV_db_di = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_db_di;
d_Vgsteff2_dV_gi_p = ((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_gi_p;
d_Vgsteff2_dV_b_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_b_GND+((Vtm*d_cdep0_dV_b_GND+d_Vtm_dV_b_GND*cdep0)/B4SOIcox)*ExpVgst2);
d_Vgsteff2_dV_p_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dV_p_GND+((Vtm*d_cdep0_dV_p_GND+d_Vtm_dV_p_GND*cdep0)/B4SOIcox)*ExpVgst2);
d_Vgsteff2_dTemp_t_GND = (((Vtm*cdep0)/B4SOIcox)*d_ExpVgst2_dTemp_t_GND+((Vtm*d_cdep0_dTemp_t_GND+d_Vtm_dTemp_t_GND*cdep0)/B4SOIcox)*ExpVgst2);
Vgsteff2 = (((Vtm*cdep0)/B4SOIcox)*ExpVgst2);
}
else
{
{
double value_exp_0 = exp(VgstNVt2);
double  deriv_exp_0_d0 = value_exp_0;

d_ExpVgst2_dV_gi_p = d_ExpVgst2_dV_db_di = d_ExpVgst2_dV_sb_si =  0.0;
d_ExpVgst2_dV_e_si = (deriv_exp_0_d0*(d_VgstNVt2_dV_e_si));
d_ExpVgst2_dV_b_si = (deriv_exp_0_d0*(d_VgstNVt2_dV_b_si));
d_ExpVgst2_dV_b_GND = (deriv_exp_0_d0*(d_VgstNVt2_dV_b_GND));
d_ExpVgst2_dV_p_GND = (deriv_exp_0_d0*(d_VgstNVt2_dV_p_GND));
d_ExpVgst2_dTemp_t_GND = (deriv_exp_0_d0*(d_VgstNVt2_dTemp_t_GND));
d_ExpVgst2_dV_gi_si = (deriv_exp_0_d0*(d_VgstNVt2_dV_gi_si));
d_ExpVgst2_dV_di_si = (deriv_exp_0_d0*(d_VgstNVt2_dV_di_si));
ExpVgst2 = value_exp_0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst2));

d_T1_dV_gm_si =  0.0;
d_T1_dV_sb_si = noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_sb_si);
d_T1_dV_db_di = noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_db_di);
d_T1_dV_gi_p = noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_gi_p);
d_T1_dV_b_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_b_GND)+d_noff2_dV_b_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_T1_dV_p_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_p_GND)+d_noff2_dV_p_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_T1_dTemp_t_GND = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dTemp_t_GND)+d_noff2_dTemp_t_GND*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_T1_dV_gi_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_gi_si)+d_noff2_dV_gi_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_T1_dV_di_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_di_si)+d_noff2_dV_di_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_T1_dV_e_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_e_si)+d_noff2_dV_e_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
d_T1_dV_b_si = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dV_b_si)+d_noff2_dV_b_si*evaluator_lln_0.getValues((1.0+ExpVgst2)));
T1 = (noff2*evaluator_lln_0.getValues((1.0+ExpVgst2)));
}
{
double value_exp_0 = exp(ExpArg2);
double  deriv_exp_0_d0 = value_exp_0;

d_dT2_dVg_dV_e_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg2_dV_e_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_b_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg2_dV_b_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_gi_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg2_dV_gi_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_di_si = ((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg2_dV_di_si))*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_b_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg2_dV_b_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dV_b_GND+d_Vtm_dV_b_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dV_p_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg2_dV_p_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dV_p_GND+d_Vtm_dV_p_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv);
d_dT2_dVg_dTemp_t_GND = (((-B4SOIcox)/(Vtm*cdep0))*(deriv_exp_0_d0*(d_ExpArg2_dTemp_t_GND))+(-(-B4SOIcox)*(Vtm*d_cdep0_dTemp_t_GND+d_Vtm_dTemp_t_GND*cdep0)/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv);
dT2_dVg = ((((-B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(1-pParam_B4SOImstarcv));
}

d_T2_dV_gm_si = d_T2_dV_gi_p = d_T2_dV_db_di = d_T2_dV_sb_si =  0.0;
d_T2_dV_b_GND = (-((noff2*d_dT2_dVg_dV_b_GND+d_noff2_dV_b_GND*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_p_GND = (-((noff2*d_dT2_dVg_dV_p_GND+d_noff2_dV_p_GND*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dTemp_t_GND = (-((noff2*d_dT2_dVg_dTemp_t_GND+d_noff2_dTemp_t_GND*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_gi_si = (-((noff2*d_dT2_dVg_dV_gi_si+d_noff2_dV_gi_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_di_si = (-((noff2*d_dT2_dVg_dV_di_si+d_noff2_dV_di_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_e_si = (-((noff2*d_dT2_dVg_dV_e_si+d_noff2_dV_e_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
d_T2_dV_b_si = (-((noff2*d_dT2_dVg_dV_b_si+d_noff2_dV_b_si*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));
T2 = (pParam_B4SOImstarcv-((noff2*dT2_dVg)/(1.0-pParam_B4SOImstarcv)));

d_Vgsteff2_dV_b_GND = ((T2*d_T1_dV_b_GND-T1*d_T2_dV_b_GND)/T2/T2);
d_Vgsteff2_dV_p_GND = ((T2*d_T1_dV_p_GND-T1*d_T2_dV_p_GND)/T2/T2);
d_Vgsteff2_dTemp_t_GND = ((T2*d_T1_dTemp_t_GND-T1*d_T2_dTemp_t_GND)/T2/T2);
d_Vgsteff2_dV_gi_si = ((T2*d_T1_dV_gi_si-T1*d_T2_dV_gi_si)/T2/T2);
d_Vgsteff2_dV_di_si = ((T2*d_T1_dV_di_si-T1*d_T2_dV_di_si)/T2/T2);
d_Vgsteff2_dV_e_si = ((T2*d_T1_dV_e_si-T1*d_T2_dV_e_si)/T2/T2);
d_Vgsteff2_dV_b_si = ((T2*d_T1_dV_b_si-T1*d_T2_dV_b_si)/T2/T2);
d_Vgsteff2_dV_sb_si = ((T2*d_T1_dV_sb_si-T1*d_T2_dV_sb_si)/T2/T2);
d_Vgsteff2_dV_db_di = ((T2*d_T1_dV_db_di-T1*d_T2_dV_db_di)/T2/T2);
d_Vgsteff2_dV_gi_p = ((T2*d_T1_dV_gi_p-T1*d_T2_dV_gi_p)/T2/T2);
Vgsteff2 = (T1/T2);
}
}
}
}
}

d_Vth_dV_e_si = d_Vth_CV_dV_e_si;
d_Vth_dV_gi_si = d_Vth_CV_dV_gi_si;
d_Vth_dV_di_si = d_Vth_CV_dV_di_si;
d_Vth_dV_b_si = d_Vth_CV_dV_b_si;
d_Vth_dV_b_GND = d_Vth_CV_dV_b_GND;
d_Vth_dV_p_GND = d_Vth_CV_dV_p_GND;
d_Vth_dTemp_t_GND = d_Vth_CV_dTemp_t_GND;
Vth = Vth_CV;

d_sqrtPhis_dV_e_si = d_sqrtPhis_CV_dV_e_si;
d_sqrtPhis_dV_gi_si = d_sqrtPhis_CV_dV_gi_si;
d_sqrtPhis_dV_di_si = d_sqrtPhis_CV_dV_di_si;
d_sqrtPhis_dV_b_si = d_sqrtPhis_CV_dV_b_si;
d_sqrtPhis_dV_b_GND = d_sqrtPhis_CV_dV_b_GND;
d_sqrtPhis_dV_p_GND = d_sqrtPhis_CV_dV_p_GND;
d_sqrtPhis_dTemp_t_GND = d_sqrtPhis_CV_dTemp_t_GND;
sqrtPhis = sqrtPhis_CV;

d_Vbseff_dV_b_GND = d_Vbseff_CV_dV_b_GND;
d_Vbseff_dV_p_GND = d_Vbseff_CV_dV_p_GND;
d_Vbseff_dTemp_t_GND = d_Vbseff_CV_dTemp_t_GND;
d_Vbseff_dV_e_si = d_Vbseff_CV_dV_e_si;
d_Vbseff_dV_gi_si = d_Vbseff_CV_dV_gi_si;
d_Vbseff_dV_di_si = d_Vbseff_CV_dV_di_si;
d_Vbseff_dV_b_si = d_Vbseff_CV_dV_b_si;
Vbseff = Vbseff_CV;
if ((B4SOIcapMod==2))
{
if ((B4SOIsoiMod==2))
{

d_Qac0_dV_gi_p = d_Qac0_dTemp_t_GND = d_Qac0_dV_p_GND = d_Qac0_dV_b_GND = d_Qac0_dV_db_di = d_Qac0_dV_sb_si = d_Qac0_dV_b_si = d_Qac0_dV_e_si = d_Qac0_dV_di_si = d_Qac0_dV_gi_si =  0.0;
Qac0 = 0.0;

d_Qsub0_dV_gi_p = d_Qsub0_dV_db_di = d_Qsub0_dV_sb_si = d_Qsub0_dV_b_si = d_Qsub0_dV_e_si = d_Qsub0_dV_di_si = d_Qsub0_dV_gi_si = d_Qsub0_dTemp_t_GND = d_Qsub0_dV_p_GND = d_Qsub0_dV_b_GND =  0.0;
Qsub0 = 0.0;
}
else
{

d_Vfb_dV_e_si = (d_Vth_dV_e_si-pParam_B4SOIk1eff*d_sqrtPhis_dV_e_si);
d_Vfb_dV_gi_si = (d_Vth_dV_gi_si-pParam_B4SOIk1eff*d_sqrtPhis_dV_gi_si);
d_Vfb_dV_di_si = (d_Vth_dV_di_si-pParam_B4SOIk1eff*d_sqrtPhis_dV_di_si);
d_Vfb_dV_b_si = (d_Vth_dV_b_si-pParam_B4SOIk1eff*d_sqrtPhis_dV_b_si);
d_Vfb_dV_b_GND = ((d_Vth_dV_b_GND-d_phi_dV_b_GND)-(pParam_B4SOIk1eff*d_sqrtPhis_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhis));
d_Vfb_dV_p_GND = ((d_Vth_dV_p_GND-d_phi_dV_p_GND)-(pParam_B4SOIk1eff*d_sqrtPhis_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhis));
d_Vfb_dTemp_t_GND = ((d_Vth_dTemp_t_GND-d_phi_dTemp_t_GND)-(pParam_B4SOIk1eff*d_sqrtPhis_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhis));
Vfb = (((Vth-phi)-(pParam_B4SOIk1eff*sqrtPhis))+pParam_B4SOIdelvt);

d_V3_dV_gi_p = d_V3_dV_db_di = d_V3_dV_sb_si =  0.0;
d_V3_dV_b_GND = ((d_Vfb_dV_b_GND-d_Vgs_eff_dV_b_GND)+d_Vbseff_dV_b_GND);
d_V3_dV_p_GND = ((d_Vfb_dV_p_GND-d_Vgs_eff_dV_p_GND)+d_Vbseff_dV_p_GND);
d_V3_dTemp_t_GND = ((d_Vfb_dTemp_t_GND-d_Vgs_eff_dTemp_t_GND)+d_Vbseff_dTemp_t_GND);
d_V3_dV_e_si = (d_Vfb_dV_e_si+d_Vbseff_dV_e_si);
d_V3_dV_gi_si = ((d_Vfb_dV_gi_si-d_Vgs_eff_dV_gi_si)+d_Vbseff_dV_gi_si);
d_V3_dV_di_si = ((d_Vfb_dV_di_si-d_Vgs_eff_dV_di_si)+d_Vbseff_dV_di_si);
d_V3_dV_b_si = (d_Vfb_dV_b_si+d_Vbseff_dV_b_si);
V3 = (((Vfb-Vgs_eff)+Vbseff)-0.08);
if ((Vfb<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((4.0*0.08)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)-(4.0*0.08)*d_Vfb_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)-(4.0*0.08)*d_Vfb_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)-(4.0*0.08)*d_Vfb_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)-(4.0*0.08)*d_Vfb_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)-(4.0*0.08)*d_Vfb_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)-(4.0*0.08)*d_Vfb_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)-(4.0*0.08)*d_Vfb_dV_b_si)));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*0.08)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)+(4.0*0.08)*d_Vfb_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)+(4.0*0.08)*d_Vfb_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)+(4.0*0.08)*d_Vfb_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)+(4.0*0.08)*d_Vfb_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)+(4.0*0.08)*d_Vfb_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)+(4.0*0.08)*d_Vfb_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)+(4.0*0.08)*d_Vfb_dV_b_si)));
T0 = value_sqrt_0;
}
}

d_Vfbeff_dV_sb_si = (-0.5*(d_V3_dV_sb_si+d_T0_dV_sb_si));
d_Vfbeff_dV_db_di = (-0.5*(d_V3_dV_db_di+d_T0_dV_db_di));
d_Vfbeff_dV_gi_p = (-0.5*(d_V3_dV_gi_p+d_T0_dV_gi_p));
d_Vfbeff_dV_b_GND = (d_Vfb_dV_b_GND-0.5*(d_V3_dV_b_GND+d_T0_dV_b_GND));
d_Vfbeff_dV_p_GND = (d_Vfb_dV_p_GND-0.5*(d_V3_dV_p_GND+d_T0_dV_p_GND));
d_Vfbeff_dTemp_t_GND = (d_Vfb_dTemp_t_GND-0.5*(d_V3_dTemp_t_GND+d_T0_dTemp_t_GND));
d_Vfbeff_dV_e_si = (d_Vfb_dV_e_si-0.5*(d_V3_dV_e_si+d_T0_dV_e_si));
d_Vfbeff_dV_gi_si = (d_Vfb_dV_gi_si-0.5*(d_V3_dV_gi_si+d_T0_dV_gi_si));
d_Vfbeff_dV_di_si = (d_Vfb_dV_di_si-0.5*(d_V3_dV_di_si+d_T0_dV_di_si));
d_Vfbeff_dV_b_si = (d_Vfb_dV_b_si-0.5*(d_V3_dV_b_si+d_T0_dV_b_si));
Vfbeff = (Vfb-(0.5*(V3+T0)));

d_Qac0_dV_gi_si = CoxWLb*(d_Vfbeff_dV_gi_si-d_Vfb_dV_gi_si);
d_Qac0_dV_di_si = CoxWLb*(d_Vfbeff_dV_di_si-d_Vfb_dV_di_si);
d_Qac0_dV_e_si = CoxWLb*(d_Vfbeff_dV_e_si-d_Vfb_dV_e_si);
d_Qac0_dV_b_si = CoxWLb*(d_Vfbeff_dV_b_si-d_Vfb_dV_b_si);
d_Qac0_dV_sb_si = CoxWLb*d_Vfbeff_dV_sb_si;
d_Qac0_dV_db_di = CoxWLb*d_Vfbeff_dV_db_di;
d_Qac0_dV_b_GND = CoxWLb*(d_Vfbeff_dV_b_GND-d_Vfb_dV_b_GND);
d_Qac0_dV_p_GND = CoxWLb*(d_Vfbeff_dV_p_GND-d_Vfb_dV_p_GND);
d_Qac0_dTemp_t_GND = CoxWLb*(d_Vfbeff_dTemp_t_GND-d_Vfb_dTemp_t_GND);
d_Qac0_dV_gi_p = CoxWLb*d_Vfbeff_dV_gi_p;
Qac0 = (CoxWLb*(Vfbeff-Vfb));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_Vfb2_dV_b_GND = d_Vfb_dV_b_GND;
d_Vfb2_dV_p_GND = d_Vfb_dV_p_GND;
d_Vfb2_dTemp_t_GND = d_Vfb_dTemp_t_GND;
d_Vfb2_dV_e_si = d_Vfb_dV_e_si;
d_Vfb2_dV_gi_si = d_Vfb_dV_gi_si;
d_Vfb2_dV_di_si = d_Vfb_dV_di_si;
d_Vfb2_dV_b_si = d_Vfb_dV_b_si;
Vfb2 = (Vfb+eggbcp2);
DELTA_3_SOI2 = 0.08;

d_V3_dV_gi_p = d_V3_dV_db_di = d_V3_dV_sb_si =  0.0;
d_V3_dV_b_GND = (d_Vfb2_dV_b_GND+d_Vbseff_dV_b_GND);
d_V3_dV_p_GND = (d_Vfb2_dV_p_GND+d_Vbseff_dV_p_GND);
d_V3_dTemp_t_GND = (d_Vfb2_dTemp_t_GND+d_Vbseff_dTemp_t_GND);
d_V3_dV_e_si = (d_Vfb2_dV_e_si+d_Vbseff_dV_e_si);
d_V3_dV_gi_si = ((d_Vfb2_dV_gi_si-d_Vgs_eff2_dV_gi_si)+d_Vbseff_dV_gi_si);
d_V3_dV_di_si = ((d_Vfb2_dV_di_si-d_Vgs_eff2_dV_di_si)+d_Vbseff_dV_di_si);
d_V3_dV_b_si = (d_Vfb2_dV_b_si+d_Vbseff_dV_b_si);
V3 = (((Vfb2-Vgs_eff2)+Vbseff)-DELTA_3_SOI2);
if ((Vfb2<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((100.0*DELTA_3_SOI2)*Vfb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dV_b_si)));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((100.0*DELTA_3_SOI2)*Vfb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dV_b_si)));
T0 = value_sqrt_0;
}
}

d_Vfbeff2_dV_sb_si = (-0.5*(d_V3_dV_sb_si+d_T0_dV_sb_si));
d_Vfbeff2_dV_db_di = (-0.5*(d_V3_dV_db_di+d_T0_dV_db_di));
d_Vfbeff2_dV_gi_p = (-0.5*(d_V3_dV_gi_p+d_T0_dV_gi_p));
d_Vfbeff2_dV_b_GND = (d_Vfb2_dV_b_GND-0.5*(d_V3_dV_b_GND+d_T0_dV_b_GND));
d_Vfbeff2_dV_p_GND = (d_Vfb2_dV_p_GND-0.5*(d_V3_dV_p_GND+d_T0_dV_p_GND));
d_Vfbeff2_dTemp_t_GND = (d_Vfb2_dTemp_t_GND-0.5*(d_V3_dTemp_t_GND+d_T0_dTemp_t_GND));
d_Vfbeff2_dV_e_si = (d_Vfb2_dV_e_si-0.5*(d_V3_dV_e_si+d_T0_dV_e_si));
d_Vfbeff2_dV_gi_si = (d_Vfb2_dV_gi_si-0.5*(d_V3_dV_gi_si+d_T0_dV_gi_si));
d_Vfbeff2_dV_di_si = (d_Vfb2_dV_di_si-0.5*(d_V3_dV_di_si+d_T0_dV_di_si));
d_Vfbeff2_dV_b_si = (d_Vfb2_dV_b_si-0.5*(d_V3_dV_b_si+d_T0_dV_b_si));
Vfbeff2 = (Vfb2-(0.5*(V3+T0)));

d_Qac0_dV_gi_si = (d_Qac0_dV_gi_si+CoxWLb2*(d_Vfbeff2_dV_gi_si-d_Vfb2_dV_gi_si));
d_Qac0_dV_di_si = (d_Qac0_dV_di_si+CoxWLb2*(d_Vfbeff2_dV_di_si-d_Vfb2_dV_di_si));
d_Qac0_dV_e_si = (d_Qac0_dV_e_si+CoxWLb2*(d_Vfbeff2_dV_e_si-d_Vfb2_dV_e_si));
d_Qac0_dV_b_si = (d_Qac0_dV_b_si+CoxWLb2*(d_Vfbeff2_dV_b_si-d_Vfb2_dV_b_si));
d_Qac0_dV_sb_si = (d_Qac0_dV_sb_si+CoxWLb2*d_Vfbeff2_dV_sb_si);
d_Qac0_dV_db_di = (d_Qac0_dV_db_di+CoxWLb2*d_Vfbeff2_dV_db_di);
d_Qac0_dV_b_GND = (d_Qac0_dV_b_GND+CoxWLb2*(d_Vfbeff2_dV_b_GND-d_Vfb2_dV_b_GND));
d_Qac0_dV_p_GND = (d_Qac0_dV_p_GND+CoxWLb2*(d_Vfbeff2_dV_p_GND-d_Vfb2_dV_p_GND));
d_Qac0_dTemp_t_GND = (d_Qac0_dTemp_t_GND+CoxWLb2*(d_Vfbeff2_dTemp_t_GND-d_Vfb2_dTemp_t_GND));
d_Qac0_dV_gi_p = (d_Qac0_dV_gi_p+CoxWLb2*d_Vfbeff2_dV_gi_p);
Qac0 = (Qac0+(CoxWLb2*(Vfbeff2-Vfb2)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (0.5*pParam_B4SOIk1ox);

d_T3_dV_e_si = (((-d_Vfbeff_dV_e_si)-d_Vbseff_dV_e_si)-d_Vgsteff_dV_e_si);
d_T3_dV_b_si = (((-d_Vfbeff_dV_b_si)-d_Vbseff_dV_b_si)-d_Vgsteff_dV_b_si);
d_T3_dV_sb_si = ((-d_Vfbeff_dV_sb_si)-d_Vgsteff_dV_sb_si);
d_T3_dV_db_di = ((-d_Vfbeff_dV_db_di)-d_Vgsteff_dV_db_di);
d_T3_dV_gi_p = ((-d_Vfbeff_dV_gi_p)-d_Vgsteff_dV_gi_p);
d_T3_dV_b_GND = (((d_Vgs_eff_dV_b_GND-d_Vfbeff_dV_b_GND)-d_Vbseff_dV_b_GND)-d_Vgsteff_dV_b_GND);
d_T3_dV_p_GND = (((d_Vgs_eff_dV_p_GND-d_Vfbeff_dV_p_GND)-d_Vbseff_dV_p_GND)-d_Vgsteff_dV_p_GND);
d_T3_dTemp_t_GND = (((d_Vgs_eff_dTemp_t_GND-d_Vfbeff_dTemp_t_GND)-d_Vbseff_dTemp_t_GND)-d_Vgsteff_dTemp_t_GND);
d_T3_dV_gi_si = (((d_Vgs_eff_dV_gi_si-d_Vfbeff_dV_gi_si)-d_Vbseff_dV_gi_si)-d_Vgsteff_dV_gi_si);
d_T3_dV_di_si = (((d_Vgs_eff_dV_di_si-d_Vfbeff_dV_di_si)-d_Vbseff_dV_di_si)-d_Vgsteff_dV_di_si);
T3 = (((Vgs_eff-Vfbeff)-Vbseff)-Vgsteff);
if ((pParam_B4SOIk1ox==0.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 0.0;
}
else
{
if ((T3<0.0))
{

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND+(d_T3_dV_b_GND/pParam_B4SOIk1ox));
d_T1_dV_p_GND = (d_T0_dV_p_GND+(d_T3_dV_p_GND/pParam_B4SOIk1ox));
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND+(d_T3_dTemp_t_GND/pParam_B4SOIk1ox));
d_T1_dV_e_si = (d_T0_dV_e_si+(d_T3_dV_e_si/pParam_B4SOIk1ox));
d_T1_dV_gi_si = (d_T0_dV_gi_si+(d_T3_dV_gi_si/pParam_B4SOIk1ox));
d_T1_dV_di_si = (d_T0_dV_di_si+(d_T3_dV_di_si/pParam_B4SOIk1ox));
d_T1_dV_b_si = (d_T0_dV_b_si+(d_T3_dV_b_si/pParam_B4SOIk1ox));
d_T1_dV_sb_si = (d_T0_dV_sb_si+(d_T3_dV_sb_si/pParam_B4SOIk1ox));
d_T1_dV_db_di = (d_T0_dV_db_di+(d_T3_dV_db_di/pParam_B4SOIk1ox));
d_T1_dV_gi_p = (d_T0_dV_gi_p+(d_T3_dV_gi_p/pParam_B4SOIk1ox));
T1 = (T0+(T3/pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)+d_T3_dV_b_GND)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)+d_T3_dV_p_GND)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)+d_T3_dTemp_t_GND)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)+d_T3_dV_e_si)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)+d_T3_dV_gi_si)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)+d_T3_dV_di_si)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)+d_T3_dV_b_si)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)+d_T3_dV_sb_si)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*(((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)+d_T3_dV_db_di)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)+d_T3_dV_gi_p)));
T1 = value_sqrt_0;
}
}
}

d_Qsub0_dV_b_GND = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_b_GND-d_T0_dV_b_GND);
d_Qsub0_dV_p_GND = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_p_GND-d_T0_dV_p_GND);
d_Qsub0_dTemp_t_GND = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dTemp_t_GND-d_T0_dTemp_t_GND);
d_Qsub0_dV_gi_si = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_gi_si-d_T0_dV_gi_si);
d_Qsub0_dV_di_si = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_di_si-d_T0_dV_di_si);
d_Qsub0_dV_e_si = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_e_si-d_T0_dV_e_si);
d_Qsub0_dV_b_si = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_b_si-d_T0_dV_b_si);
d_Qsub0_dV_sb_si = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_sb_si-d_T0_dV_sb_si);
d_Qsub0_dV_db_di = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_db_di-d_T0_dV_db_di);
d_Qsub0_dV_gi_p = (CoxWLb*pParam_B4SOIk1ox)*(d_T1_dV_gi_p-d_T0_dV_gi_p);
Qsub0 = ((CoxWLb*pParam_B4SOIk1ox)*(T1-T0));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T3_dV_sb_si = ((-d_Vfbeff2_dV_sb_si)-d_Vgsteff2_dV_sb_si);
d_T3_dV_db_di = ((-d_Vfbeff2_dV_db_di)-d_Vgsteff2_dV_db_di);
d_T3_dV_gi_p = ((-d_Vfbeff2_dV_gi_p)-d_Vgsteff2_dV_gi_p);
d_T3_dV_b_GND = (((-d_Vfbeff2_dV_b_GND)-d_Vbseff_dV_b_GND)-d_Vgsteff2_dV_b_GND);
d_T3_dV_p_GND = (((-d_Vfbeff2_dV_p_GND)-d_Vbseff_dV_p_GND)-d_Vgsteff2_dV_p_GND);
d_T3_dTemp_t_GND = (((-d_Vfbeff2_dTemp_t_GND)-d_Vbseff_dTemp_t_GND)-d_Vgsteff2_dTemp_t_GND);
d_T3_dV_e_si = (((-d_Vfbeff2_dV_e_si)-d_Vbseff_dV_e_si)-d_Vgsteff2_dV_e_si);
d_T3_dV_b_si = (((-d_Vfbeff2_dV_b_si)-d_Vbseff_dV_b_si)-d_Vgsteff2_dV_b_si);
d_T3_dV_gi_si = (((d_Vgs_eff2_dV_gi_si-d_Vfbeff2_dV_gi_si)-d_Vbseff_dV_gi_si)-d_Vgsteff2_dV_gi_si);
d_T3_dV_di_si = (((d_Vgs_eff2_dV_di_si-d_Vfbeff2_dV_di_si)-d_Vbseff_dV_di_si)-d_Vgsteff2_dV_di_si);
T3 = (((Vgs_eff2-Vfbeff2)-Vbseff)-Vgsteff2);
if ((T3<0.0))
{

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND+(d_T3_dV_b_GND/pParam_B4SOIk1ox));
d_T1_dV_p_GND = (d_T0_dV_p_GND+(d_T3_dV_p_GND/pParam_B4SOIk1ox));
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND+(d_T3_dTemp_t_GND/pParam_B4SOIk1ox));
d_T1_dV_e_si = (d_T0_dV_e_si+(d_T3_dV_e_si/pParam_B4SOIk1ox));
d_T1_dV_gi_si = (d_T0_dV_gi_si+(d_T3_dV_gi_si/pParam_B4SOIk1ox));
d_T1_dV_di_si = (d_T0_dV_di_si+(d_T3_dV_di_si/pParam_B4SOIk1ox));
d_T1_dV_b_si = (d_T0_dV_b_si+(d_T3_dV_b_si/pParam_B4SOIk1ox));
d_T1_dV_sb_si = (d_T0_dV_sb_si+(d_T3_dV_sb_si/pParam_B4SOIk1ox));
d_T1_dV_db_di = (d_T0_dV_db_di+(d_T3_dV_db_di/pParam_B4SOIk1ox));
d_T1_dV_gi_p = (d_T0_dV_gi_p+(d_T3_dV_gi_p/pParam_B4SOIk1ox));
T1 = (T0+(T3/pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)+d_T3_dV_b_GND)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)+d_T3_dV_p_GND)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)+d_T3_dTemp_t_GND)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)+d_T3_dV_e_si)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)+d_T3_dV_gi_si)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)+d_T3_dV_di_si)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)+d_T3_dV_b_si)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)+d_T3_dV_sb_si)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*(((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)+d_T3_dV_db_di)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)+d_T3_dV_gi_p)));
T1 = value_sqrt_0;
}
}

d_Qsub0_dV_b_GND = (d_Qsub0_dV_b_GND+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_b_GND-d_T0_dV_b_GND));
d_Qsub0_dV_p_GND = (d_Qsub0_dV_p_GND+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_p_GND-d_T0_dV_p_GND));
d_Qsub0_dTemp_t_GND = (d_Qsub0_dTemp_t_GND+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dTemp_t_GND-d_T0_dTemp_t_GND));
d_Qsub0_dV_gi_si = (d_Qsub0_dV_gi_si+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_gi_si-d_T0_dV_gi_si));
d_Qsub0_dV_di_si = (d_Qsub0_dV_di_si+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_di_si-d_T0_dV_di_si));
d_Qsub0_dV_e_si = (d_Qsub0_dV_e_si+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_e_si-d_T0_dV_e_si));
d_Qsub0_dV_b_si = (d_Qsub0_dV_b_si+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_b_si-d_T0_dV_b_si));
d_Qsub0_dV_sb_si = (d_Qsub0_dV_sb_si+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_sb_si-d_T0_dV_sb_si));
d_Qsub0_dV_db_di = (d_Qsub0_dV_db_di+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_db_di-d_T0_dV_db_di));
d_Qsub0_dV_gi_p = (d_Qsub0_dV_gi_p+(CoxWLb2*pParam_B4SOIk1ox)*(d_T1_dV_gi_p-d_T0_dV_gi_p));
Qsub0 = (Qsub0+((CoxWLb2*pParam_B4SOIk1ox)*(T1-T0)));
}
}

d_AbulkCV_dV_b_GND = d_Abulk0_dV_b_GND*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_p_GND = d_Abulk0_dV_p_GND*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dTemp_t_GND = d_Abulk0_dTemp_t_GND*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_gi_si = d_Abulk0_dV_gi_si*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_di_si = d_Abulk0_dV_di_si*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_e_si = d_Abulk0_dV_e_si*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_b_si = d_Abulk0_dV_b_si*pParam_B4SOIabulkCVfactor;
AbulkCV = (Abulk0*pParam_B4SOIabulkCVfactor);

d_VdsatCV_dV_e_si = ((AbulkCV*d_Vgsteff_dV_e_si-Vgsteff*d_AbulkCV_dV_e_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_b_si = ((AbulkCV*d_Vgsteff_dV_b_si-Vgsteff*d_AbulkCV_dV_b_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_b_GND = ((AbulkCV*d_Vgsteff_dV_b_GND-Vgsteff*d_AbulkCV_dV_b_GND)/AbulkCV/AbulkCV);
d_VdsatCV_dV_p_GND = ((AbulkCV*d_Vgsteff_dV_p_GND-Vgsteff*d_AbulkCV_dV_p_GND)/AbulkCV/AbulkCV);
d_VdsatCV_dTemp_t_GND = ((AbulkCV*d_Vgsteff_dTemp_t_GND-Vgsteff*d_AbulkCV_dTemp_t_GND)/AbulkCV/AbulkCV);
d_VdsatCV_dV_gi_si = ((AbulkCV*d_Vgsteff_dV_gi_si-Vgsteff*d_AbulkCV_dV_gi_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_di_si = ((AbulkCV*d_Vgsteff_dV_di_si-Vgsteff*d_AbulkCV_dV_di_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_sb_si = (d_Vgsteff_dV_sb_si/AbulkCV);
d_VdsatCV_dV_db_di = (d_Vgsteff_dV_db_di/AbulkCV);
d_VdsatCV_dV_gi_p = (d_Vgsteff_dV_gi_p/AbulkCV);
VdsatCV = (Vgsteff/AbulkCV);

d_V4_dV_e_si = d_VdsatCV_dV_e_si;
d_V4_dV_b_si = d_VdsatCV_dV_b_si;
d_V4_dV_b_GND = d_VdsatCV_dV_b_GND;
d_V4_dV_p_GND = d_VdsatCV_dV_p_GND;
d_V4_dTemp_t_GND = d_VdsatCV_dTemp_t_GND;
d_V4_dV_gi_si = d_VdsatCV_dV_gi_si;
d_V4_dV_di_si = (d_VdsatCV_dV_di_si-d_Vds_dV_di_si);
d_V4_dV_sb_si = d_VdsatCV_dV_sb_si;
d_V4_dV_db_di = d_VdsatCV_dV_db_di;
d_V4_dV_gi_p = d_VdsatCV_dV_gi_p;
V4 = ((VdsatCV-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si =  0.0;
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_e_si+d_V4_dV_e_si*V4)+(4.0*0.02)*d_VdsatCV_dV_e_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_si+d_V4_dV_b_si*V4)+(4.0*0.02)*d_VdsatCV_dV_b_si)));
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_GND+d_V4_dV_b_GND*V4)+(4.0*0.02)*d_VdsatCV_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_p_GND+d_V4_dV_p_GND*V4)+(4.0*0.02)*d_VdsatCV_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dTemp_t_GND+d_V4_dTemp_t_GND*V4)+(4.0*0.02)*d_VdsatCV_dTemp_t_GND)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_si+d_V4_dV_gi_si*V4)+(4.0*0.02)*d_VdsatCV_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_di_si+d_V4_dV_di_si*V4)+(4.0*0.02)*d_VdsatCV_dV_di_si)));
d_T0_dV_sb_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_sb_si+d_V4_dV_sb_si*V4)+(4.0*0.02)*d_VdsatCV_dV_sb_si)));
d_T0_dV_db_di = (deriv_sqrt_0_d0*(((V4*d_V4_dV_db_di+d_V4_dV_db_di*V4)+(4.0*0.02)*d_VdsatCV_dV_db_di)));
d_T0_dV_gi_p = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_p+d_V4_dV_gi_p*V4)+(4.0*0.02)*d_VdsatCV_dV_gi_p)));
T0 = value_sqrt_0;
}

d_VdseffCV_dV_e_si = (d_VdsatCV_dV_e_si-0.5*(d_V4_dV_e_si+d_T0_dV_e_si));
d_VdseffCV_dV_b_si = (d_VdsatCV_dV_b_si-0.5*(d_V4_dV_b_si+d_T0_dV_b_si));
d_VdseffCV_dV_b_GND = (d_VdsatCV_dV_b_GND-0.5*(d_V4_dV_b_GND+d_T0_dV_b_GND));
d_VdseffCV_dV_p_GND = (d_VdsatCV_dV_p_GND-0.5*(d_V4_dV_p_GND+d_T0_dV_p_GND));
d_VdseffCV_dTemp_t_GND = (d_VdsatCV_dTemp_t_GND-0.5*(d_V4_dTemp_t_GND+d_T0_dTemp_t_GND));
d_VdseffCV_dV_gi_si = (d_VdsatCV_dV_gi_si-0.5*(d_V4_dV_gi_si+d_T0_dV_gi_si));
d_VdseffCV_dV_di_si = (d_VdsatCV_dV_di_si-0.5*(d_V4_dV_di_si+d_T0_dV_di_si));
d_VdseffCV_dV_sb_si = (d_VdsatCV_dV_sb_si-0.5*(d_V4_dV_sb_si+d_T0_dV_sb_si));
d_VdseffCV_dV_db_di = (d_VdsatCV_dV_db_di-0.5*(d_V4_dV_db_di+d_T0_dV_db_di));
d_VdseffCV_dV_gi_p = (d_VdsatCV_dV_gi_p-0.5*(d_V4_dV_gi_p+d_T0_dV_gi_p));
VdseffCV = (VdsatCV-(0.5*(V4+T0)));
if ((B4SOIagbcp2>0))
{

d_VdsatCV2_dV_sb_si = (d_Vgsteff2_dV_sb_si/AbulkCV);
d_VdsatCV2_dV_db_di = (d_Vgsteff2_dV_db_di/AbulkCV);
d_VdsatCV2_dV_b_GND = ((AbulkCV*d_Vgsteff2_dV_b_GND-Vgsteff2*d_AbulkCV_dV_b_GND)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_p_GND = ((AbulkCV*d_Vgsteff2_dV_p_GND-Vgsteff2*d_AbulkCV_dV_p_GND)/AbulkCV/AbulkCV);
d_VdsatCV2_dTemp_t_GND = ((AbulkCV*d_Vgsteff2_dTemp_t_GND-Vgsteff2*d_AbulkCV_dTemp_t_GND)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_gi_si = ((AbulkCV*d_Vgsteff2_dV_gi_si-Vgsteff2*d_AbulkCV_dV_gi_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_di_si = ((AbulkCV*d_Vgsteff2_dV_di_si-Vgsteff2*d_AbulkCV_dV_di_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_e_si = ((AbulkCV*d_Vgsteff2_dV_e_si-Vgsteff2*d_AbulkCV_dV_e_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_b_si = ((AbulkCV*d_Vgsteff2_dV_b_si-Vgsteff2*d_AbulkCV_dV_b_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_gi_p = (d_Vgsteff2_dV_gi_p/AbulkCV);
VdsatCV2 = (Vgsteff2/AbulkCV);

d_V4_dV_sb_si = d_VdsatCV2_dV_sb_si;
d_V4_dV_db_di = d_VdsatCV2_dV_db_di;
d_V4_dV_b_GND = d_VdsatCV2_dV_b_GND;
d_V4_dV_p_GND = d_VdsatCV2_dV_p_GND;
d_V4_dTemp_t_GND = d_VdsatCV2_dTemp_t_GND;
d_V4_dV_gi_si = d_VdsatCV2_dV_gi_si;
d_V4_dV_di_si = (d_VdsatCV2_dV_di_si-d_Vds_dV_di_si);
d_V4_dV_e_si = d_VdsatCV2_dV_e_si;
d_V4_dV_b_si = d_VdsatCV2_dV_b_si;
d_V4_dV_gi_p = d_VdsatCV2_dV_gi_p;
V4 = ((VdsatCV2-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si =  0.0;
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_e_si+d_V4_dV_e_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_e_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_si+d_V4_dV_b_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_b_si)));
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_GND+d_V4_dV_b_GND*V4)+(4.0*0.02)*d_VdsatCV2_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_p_GND+d_V4_dV_p_GND*V4)+(4.0*0.02)*d_VdsatCV2_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dTemp_t_GND+d_V4_dTemp_t_GND*V4)+(4.0*0.02)*d_VdsatCV2_dTemp_t_GND)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_si+d_V4_dV_gi_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_di_si+d_V4_dV_di_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_di_si)));
d_T0_dV_sb_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_sb_si+d_V4_dV_sb_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_sb_si)));
d_T0_dV_db_di = (deriv_sqrt_0_d0*(((V4*d_V4_dV_db_di+d_V4_dV_db_di*V4)+(4.0*0.02)*d_VdsatCV2_dV_db_di)));
d_T0_dV_gi_p = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_p+d_V4_dV_gi_p*V4)+(4.0*0.02)*d_VdsatCV2_dV_gi_p)));
T0 = value_sqrt_0;
}

d_VdseffCV2_dV_sb_si = (d_VdsatCV2_dV_sb_si-0.5*(d_V4_dV_sb_si+d_T0_dV_sb_si));
d_VdseffCV2_dV_db_di = (d_VdsatCV2_dV_db_di-0.5*(d_V4_dV_db_di+d_T0_dV_db_di));
d_VdseffCV2_dV_b_GND = (d_VdsatCV2_dV_b_GND-0.5*(d_V4_dV_b_GND+d_T0_dV_b_GND));
d_VdseffCV2_dV_p_GND = (d_VdsatCV2_dV_p_GND-0.5*(d_V4_dV_p_GND+d_T0_dV_p_GND));
d_VdseffCV2_dTemp_t_GND = (d_VdsatCV2_dTemp_t_GND-0.5*(d_V4_dTemp_t_GND+d_T0_dTemp_t_GND));
d_VdseffCV2_dV_gi_si = (d_VdsatCV2_dV_gi_si-0.5*(d_V4_dV_gi_si+d_T0_dV_gi_si));
d_VdseffCV2_dV_di_si = (d_VdsatCV2_dV_di_si-0.5*(d_V4_dV_di_si+d_T0_dV_di_si));
d_VdseffCV2_dV_e_si = (d_VdsatCV2_dV_e_si-0.5*(d_V4_dV_e_si+d_T0_dV_e_si));
d_VdseffCV2_dV_b_si = (d_VdsatCV2_dV_b_si-0.5*(d_V4_dV_b_si+d_T0_dV_b_si));
d_VdseffCV2_dV_gi_p = (d_VdsatCV2_dV_gi_p-0.5*(d_V4_dV_gi_p+d_T0_dV_gi_p));
VdseffCV2 = (VdsatCV2-(0.5*(V4+T0)));
}
if ((B4SOIsoiMod==2))
{

d_qbulk_dV_b_si = d_qbulk_dV_e_si = d_qbulk_dV_di_si = d_qbulk_dV_gi_si = d_qbulk_dTemp_t_GND = d_qbulk_dV_p_GND = d_qbulk_dV_b_GND = d_qbulk_dV_gi_p = d_qbulk_dV_db_di = d_qbulk_dV_sb_si =  0.0;
qbulk = 0;
}
else
{

d_T0_dV_gm_si =  0.0;
d_T0_dV_sb_si = AbulkCV*d_VdseffCV_dV_sb_si;
d_T0_dV_db_di = AbulkCV*d_VdseffCV_dV_db_di;
d_T0_dV_gi_p = AbulkCV*d_VdseffCV_dV_gi_p;
d_T0_dV_b_GND = (AbulkCV*d_VdseffCV_dV_b_GND+d_AbulkCV_dV_b_GND*VdseffCV);
d_T0_dV_p_GND = (AbulkCV*d_VdseffCV_dV_p_GND+d_AbulkCV_dV_p_GND*VdseffCV);
d_T0_dTemp_t_GND = (AbulkCV*d_VdseffCV_dTemp_t_GND+d_AbulkCV_dTemp_t_GND*VdseffCV);
d_T0_dV_gi_si = (AbulkCV*d_VdseffCV_dV_gi_si+d_AbulkCV_dV_gi_si*VdseffCV);
d_T0_dV_di_si = (AbulkCV*d_VdseffCV_dV_di_si+d_AbulkCV_dV_di_si*VdseffCV);
d_T0_dV_e_si = (AbulkCV*d_VdseffCV_dV_e_si+d_AbulkCV_dV_e_si*VdseffCV);
d_T0_dV_b_si = (AbulkCV*d_VdseffCV_dV_b_si+d_AbulkCV_dV_b_si*VdseffCV);
T0 = (AbulkCV*VdseffCV);

d_T1_dV_gm_si =  0.0;
d_T1_dV_e_si = 12.0*(d_Vgsteff_dV_e_si-0.5*d_T0_dV_e_si);
d_T1_dV_b_si = 12.0*(d_Vgsteff_dV_b_si-0.5*d_T0_dV_b_si);
d_T1_dV_b_GND = 12.0*(d_Vgsteff_dV_b_GND-0.5*d_T0_dV_b_GND);
d_T1_dV_p_GND = 12.0*(d_Vgsteff_dV_p_GND-0.5*d_T0_dV_p_GND);
d_T1_dTemp_t_GND = 12.0*(d_Vgsteff_dTemp_t_GND-0.5*d_T0_dTemp_t_GND);
d_T1_dV_gi_si = 12.0*(d_Vgsteff_dV_gi_si-0.5*d_T0_dV_gi_si);
d_T1_dV_di_si = 12.0*(d_Vgsteff_dV_di_si-0.5*d_T0_dV_di_si);
d_T1_dV_sb_si = 12.0*(d_Vgsteff_dV_sb_si-0.5*d_T0_dV_sb_si);
d_T1_dV_db_di = 12.0*(d_Vgsteff_dV_db_di-0.5*d_T0_dV_db_di);
d_T1_dV_gi_p = 12.0*(d_Vgsteff_dV_gi_p-0.5*d_T0_dV_gi_p);
T1 = (12.0*((Vgsteff-(0.5*T0))+1e-20));

d_T2_dV_gm_si =  0.0;
d_T2_dV_e_si = ((T1*d_VdseffCV_dV_e_si-VdseffCV*d_T1_dV_e_si)/T1/T1);
d_T2_dV_b_si = ((T1*d_VdseffCV_dV_b_si-VdseffCV*d_T1_dV_b_si)/T1/T1);
d_T2_dV_b_GND = ((T1*d_VdseffCV_dV_b_GND-VdseffCV*d_T1_dV_b_GND)/T1/T1);
d_T2_dV_p_GND = ((T1*d_VdseffCV_dV_p_GND-VdseffCV*d_T1_dV_p_GND)/T1/T1);
d_T2_dTemp_t_GND = ((T1*d_VdseffCV_dTemp_t_GND-VdseffCV*d_T1_dTemp_t_GND)/T1/T1);
d_T2_dV_gi_si = ((T1*d_VdseffCV_dV_gi_si-VdseffCV*d_T1_dV_gi_si)/T1/T1);
d_T2_dV_di_si = ((T1*d_VdseffCV_dV_di_si-VdseffCV*d_T1_dV_di_si)/T1/T1);
d_T2_dV_sb_si = ((T1*d_VdseffCV_dV_sb_si-VdseffCV*d_T1_dV_sb_si)/T1/T1);
d_T2_dV_db_di = ((T1*d_VdseffCV_dV_db_di-VdseffCV*d_T1_dV_db_di)/T1/T1);
d_T2_dV_gi_p = ((T1*d_VdseffCV_dV_gi_p-VdseffCV*d_T1_dV_gi_p)/T1/T1);
T2 = (VdseffCV/T1);

d_T3_dV_b_GND = (T0*d_T2_dV_b_GND+d_T0_dV_b_GND*T2);
d_T3_dV_p_GND = (T0*d_T2_dV_p_GND+d_T0_dV_p_GND*T2);
d_T3_dTemp_t_GND = (T0*d_T2_dTemp_t_GND+d_T0_dTemp_t_GND*T2);
d_T3_dV_e_si = (T0*d_T2_dV_e_si+d_T0_dV_e_si*T2);
d_T3_dV_gi_si = (T0*d_T2_dV_gi_si+d_T0_dV_gi_si*T2);
d_T3_dV_di_si = (T0*d_T2_dV_di_si+d_T0_dV_di_si*T2);
d_T3_dV_b_si = (T0*d_T2_dV_b_si+d_T0_dV_b_si*T2);
d_T3_dV_sb_si = (T0*d_T2_dV_sb_si+d_T0_dV_sb_si*T2);
d_T3_dV_db_di = (T0*d_T2_dV_db_di+d_T0_dV_db_di*T2);
d_T3_dV_gi_p = (T0*d_T2_dV_gi_p+d_T0_dV_gi_p*T2);
T3 = (T0*T2);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = (-d_AbulkCV_dV_b_GND);
d_T7_dV_p_GND = (-d_AbulkCV_dV_p_GND);
d_T7_dTemp_t_GND = (-d_AbulkCV_dTemp_t_GND);
d_T7_dV_gi_si = (-d_AbulkCV_dV_gi_si);
d_T7_dV_di_si = (-d_AbulkCV_dV_di_si);
d_T7_dV_e_si = (-d_AbulkCV_dV_e_si);
d_T7_dV_b_si = (-d_AbulkCV_dV_b_si);
T7 = (1.0-AbulkCV);

d_qbulk_dV_sb_si = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_sb_si-d_T3_dV_sb_si)+CoxWLb*d_T7_dV_sb_si*((0.5*VdseffCV)-T3));
d_qbulk_dV_db_di = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_db_di-d_T3_dV_db_di)+CoxWLb*d_T7_dV_db_di*((0.5*VdseffCV)-T3));
d_qbulk_dV_gi_p = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_gi_p-d_T3_dV_gi_p)+CoxWLb*d_T7_dV_gi_p*((0.5*VdseffCV)-T3));
d_qbulk_dV_b_GND = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_b_GND-d_T3_dV_b_GND)+CoxWLb*d_T7_dV_b_GND*((0.5*VdseffCV)-T3));
d_qbulk_dV_p_GND = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_p_GND-d_T3_dV_p_GND)+CoxWLb*d_T7_dV_p_GND*((0.5*VdseffCV)-T3));
d_qbulk_dTemp_t_GND = ((CoxWLb*T7)*(0.5*d_VdseffCV_dTemp_t_GND-d_T3_dTemp_t_GND)+CoxWLb*d_T7_dTemp_t_GND*((0.5*VdseffCV)-T3));
d_qbulk_dV_gi_si = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_gi_si-d_T3_dV_gi_si)+CoxWLb*d_T7_dV_gi_si*((0.5*VdseffCV)-T3));
d_qbulk_dV_di_si = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_di_si-d_T3_dV_di_si)+CoxWLb*d_T7_dV_di_si*((0.5*VdseffCV)-T3));
d_qbulk_dV_e_si = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_e_si-d_T3_dV_e_si)+CoxWLb*d_T7_dV_e_si*((0.5*VdseffCV)-T3));
d_qbulk_dV_b_si = ((CoxWLb*T7)*(0.5*d_VdseffCV_dV_b_si-d_T3_dV_b_si)+CoxWLb*d_T7_dV_b_si*((0.5*VdseffCV)-T3));
qbulk = ((CoxWLb*T7)*((0.5*VdseffCV)-T3));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T0_dV_gm_si =  0.0;
d_T0_dV_sb_si = AbulkCV*d_VdseffCV2_dV_sb_si;
d_T0_dV_db_di = AbulkCV*d_VdseffCV2_dV_db_di;
d_T0_dV_gi_p = AbulkCV*d_VdseffCV2_dV_gi_p;
d_T0_dV_b_GND = (AbulkCV*d_VdseffCV2_dV_b_GND+d_AbulkCV_dV_b_GND*VdseffCV2);
d_T0_dV_p_GND = (AbulkCV*d_VdseffCV2_dV_p_GND+d_AbulkCV_dV_p_GND*VdseffCV2);
d_T0_dTemp_t_GND = (AbulkCV*d_VdseffCV2_dTemp_t_GND+d_AbulkCV_dTemp_t_GND*VdseffCV2);
d_T0_dV_gi_si = (AbulkCV*d_VdseffCV2_dV_gi_si+d_AbulkCV_dV_gi_si*VdseffCV2);
d_T0_dV_di_si = (AbulkCV*d_VdseffCV2_dV_di_si+d_AbulkCV_dV_di_si*VdseffCV2);
d_T0_dV_e_si = (AbulkCV*d_VdseffCV2_dV_e_si+d_AbulkCV_dV_e_si*VdseffCV2);
d_T0_dV_b_si = (AbulkCV*d_VdseffCV2_dV_b_si+d_AbulkCV_dV_b_si*VdseffCV2);
T0 = (AbulkCV*VdseffCV2);

d_T1_dV_gm_si =  0.0;
d_T1_dV_sb_si = 12.0*(d_Vgsteff2_dV_sb_si-0.5*d_T0_dV_sb_si);
d_T1_dV_db_di = 12.0*(d_Vgsteff2_dV_db_di-0.5*d_T0_dV_db_di);
d_T1_dV_b_GND = 12.0*(d_Vgsteff2_dV_b_GND-0.5*d_T0_dV_b_GND);
d_T1_dV_p_GND = 12.0*(d_Vgsteff2_dV_p_GND-0.5*d_T0_dV_p_GND);
d_T1_dTemp_t_GND = 12.0*(d_Vgsteff2_dTemp_t_GND-0.5*d_T0_dTemp_t_GND);
d_T1_dV_gi_si = 12.0*(d_Vgsteff2_dV_gi_si-0.5*d_T0_dV_gi_si);
d_T1_dV_di_si = 12.0*(d_Vgsteff2_dV_di_si-0.5*d_T0_dV_di_si);
d_T1_dV_e_si = 12.0*(d_Vgsteff2_dV_e_si-0.5*d_T0_dV_e_si);
d_T1_dV_b_si = 12.0*(d_Vgsteff2_dV_b_si-0.5*d_T0_dV_b_si);
d_T1_dV_gi_p = 12.0*(d_Vgsteff2_dV_gi_p-0.5*d_T0_dV_gi_p);
T1 = (12.0*((Vgsteff2-(0.5*T0))+1e-20));

d_T2_dV_gm_si =  0.0;
d_T2_dV_sb_si = ((T1*d_VdseffCV2_dV_sb_si-VdseffCV2*d_T1_dV_sb_si)/T1/T1);
d_T2_dV_db_di = ((T1*d_VdseffCV2_dV_db_di-VdseffCV2*d_T1_dV_db_di)/T1/T1);
d_T2_dV_b_GND = ((T1*d_VdseffCV2_dV_b_GND-VdseffCV2*d_T1_dV_b_GND)/T1/T1);
d_T2_dV_p_GND = ((T1*d_VdseffCV2_dV_p_GND-VdseffCV2*d_T1_dV_p_GND)/T1/T1);
d_T2_dTemp_t_GND = ((T1*d_VdseffCV2_dTemp_t_GND-VdseffCV2*d_T1_dTemp_t_GND)/T1/T1);
d_T2_dV_gi_si = ((T1*d_VdseffCV2_dV_gi_si-VdseffCV2*d_T1_dV_gi_si)/T1/T1);
d_T2_dV_di_si = ((T1*d_VdseffCV2_dV_di_si-VdseffCV2*d_T1_dV_di_si)/T1/T1);
d_T2_dV_e_si = ((T1*d_VdseffCV2_dV_e_si-VdseffCV2*d_T1_dV_e_si)/T1/T1);
d_T2_dV_b_si = ((T1*d_VdseffCV2_dV_b_si-VdseffCV2*d_T1_dV_b_si)/T1/T1);
d_T2_dV_gi_p = ((T1*d_VdseffCV2_dV_gi_p-VdseffCV2*d_T1_dV_gi_p)/T1/T1);
T2 = (VdseffCV2/T1);

d_T3_dV_b_GND = (T0*d_T2_dV_b_GND+d_T0_dV_b_GND*T2);
d_T3_dV_p_GND = (T0*d_T2_dV_p_GND+d_T0_dV_p_GND*T2);
d_T3_dTemp_t_GND = (T0*d_T2_dTemp_t_GND+d_T0_dTemp_t_GND*T2);
d_T3_dV_e_si = (T0*d_T2_dV_e_si+d_T0_dV_e_si*T2);
d_T3_dV_gi_si = (T0*d_T2_dV_gi_si+d_T0_dV_gi_si*T2);
d_T3_dV_di_si = (T0*d_T2_dV_di_si+d_T0_dV_di_si*T2);
d_T3_dV_b_si = (T0*d_T2_dV_b_si+d_T0_dV_b_si*T2);
d_T3_dV_sb_si = (T0*d_T2_dV_sb_si+d_T0_dV_sb_si*T2);
d_T3_dV_db_di = (T0*d_T2_dV_db_di+d_T0_dV_db_di*T2);
d_T3_dV_gi_p = (T0*d_T2_dV_gi_p+d_T0_dV_gi_p*T2);
T3 = (T0*T2);

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = (-d_AbulkCV_dV_b_GND);
d_T7_dV_p_GND = (-d_AbulkCV_dV_p_GND);
d_T7_dTemp_t_GND = (-d_AbulkCV_dTemp_t_GND);
d_T7_dV_gi_si = (-d_AbulkCV_dV_gi_si);
d_T7_dV_di_si = (-d_AbulkCV_dV_di_si);
d_T7_dV_e_si = (-d_AbulkCV_dV_e_si);
d_T7_dV_b_si = (-d_AbulkCV_dV_b_si);
T7 = (1.0-AbulkCV);

d_qbulk_dV_sb_si = (d_qbulk_dV_sb_si+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_sb_si-d_T3_dV_sb_si)+CoxWLb2*d_T7_dV_sb_si*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_db_di = (d_qbulk_dV_db_di+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_db_di-d_T3_dV_db_di)+CoxWLb2*d_T7_dV_db_di*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_gi_p = (d_qbulk_dV_gi_p+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_gi_p-d_T3_dV_gi_p)+CoxWLb2*d_T7_dV_gi_p*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_b_GND = (d_qbulk_dV_b_GND+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_b_GND-d_T3_dV_b_GND)+CoxWLb2*d_T7_dV_b_GND*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_p_GND = (d_qbulk_dV_p_GND+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_p_GND-d_T3_dV_p_GND)+CoxWLb2*d_T7_dV_p_GND*((0.5*VdseffCV2)-T3)));
d_qbulk_dTemp_t_GND = (d_qbulk_dTemp_t_GND+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dTemp_t_GND-d_T3_dTemp_t_GND)+CoxWLb2*d_T7_dTemp_t_GND*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_gi_si = (d_qbulk_dV_gi_si+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_gi_si-d_T3_dV_gi_si)+CoxWLb2*d_T7_dV_gi_si*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_di_si = (d_qbulk_dV_di_si+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_di_si-d_T3_dV_di_si)+CoxWLb2*d_T7_dV_di_si*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_e_si = (d_qbulk_dV_e_si+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_e_si-d_T3_dV_e_si)+CoxWLb2*d_T7_dV_e_si*((0.5*VdseffCV2)-T3)));
d_qbulk_dV_b_si = (d_qbulk_dV_b_si+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dV_b_si-d_T3_dV_b_si)+CoxWLb2*d_T7_dV_b_si*((0.5*VdseffCV2)-T3)));
qbulk = (qbulk+((CoxWLb2*T7)*((0.5*VdseffCV2)-T3)));
}
}

d_T0_dV_gm_si =  0.0;
d_T0_dV_sb_si = AbulkCV*d_VdseffCV_dV_sb_si;
d_T0_dV_db_di = AbulkCV*d_VdseffCV_dV_db_di;
d_T0_dV_gi_p = AbulkCV*d_VdseffCV_dV_gi_p;
d_T0_dV_b_GND = (AbulkCV*d_VdseffCV_dV_b_GND+d_AbulkCV_dV_b_GND*VdseffCV);
d_T0_dV_p_GND = (AbulkCV*d_VdseffCV_dV_p_GND+d_AbulkCV_dV_p_GND*VdseffCV);
d_T0_dTemp_t_GND = (AbulkCV*d_VdseffCV_dTemp_t_GND+d_AbulkCV_dTemp_t_GND*VdseffCV);
d_T0_dV_gi_si = (AbulkCV*d_VdseffCV_dV_gi_si+d_AbulkCV_dV_gi_si*VdseffCV);
d_T0_dV_di_si = (AbulkCV*d_VdseffCV_dV_di_si+d_AbulkCV_dV_di_si*VdseffCV);
d_T0_dV_e_si = (AbulkCV*d_VdseffCV_dV_e_si+d_AbulkCV_dV_e_si*VdseffCV);
d_T0_dV_b_si = (AbulkCV*d_VdseffCV_dV_b_si+d_AbulkCV_dV_b_si*VdseffCV);
T0 = (AbulkCV*VdseffCV);

d_T1_dV_gm_si =  0.0;
d_T1_dV_e_si = 12.0*(d_Vgsteff_dV_e_si-0.5*d_T0_dV_e_si);
d_T1_dV_b_si = 12.0*(d_Vgsteff_dV_b_si-0.5*d_T0_dV_b_si);
d_T1_dV_b_GND = 12.0*(d_Vgsteff_dV_b_GND-0.5*d_T0_dV_b_GND);
d_T1_dV_p_GND = 12.0*(d_Vgsteff_dV_p_GND-0.5*d_T0_dV_p_GND);
d_T1_dTemp_t_GND = 12.0*(d_Vgsteff_dTemp_t_GND-0.5*d_T0_dTemp_t_GND);
d_T1_dV_gi_si = 12.0*(d_Vgsteff_dV_gi_si-0.5*d_T0_dV_gi_si);
d_T1_dV_di_si = 12.0*(d_Vgsteff_dV_di_si-0.5*d_T0_dV_di_si);
d_T1_dV_sb_si = 12.0*(d_Vgsteff_dV_sb_si-0.5*d_T0_dV_sb_si);
d_T1_dV_db_di = 12.0*(d_Vgsteff_dV_db_di-0.5*d_T0_dV_db_di);
d_T1_dV_gi_p = 12.0*(d_Vgsteff_dV_gi_p-0.5*d_T0_dV_gi_p);
T1 = (12.0*((Vgsteff-(0.5*T0))+1e-20));

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = ((T1*d_T0_dV_b_GND-T0*d_T1_dV_b_GND)/T1/T1);
d_T2_dV_p_GND = ((T1*d_T0_dV_p_GND-T0*d_T1_dV_p_GND)/T1/T1);
d_T2_dTemp_t_GND = ((T1*d_T0_dTemp_t_GND-T0*d_T1_dTemp_t_GND)/T1/T1);
d_T2_dV_e_si = ((T1*d_T0_dV_e_si-T0*d_T1_dV_e_si)/T1/T1);
d_T2_dV_gi_si = ((T1*d_T0_dV_gi_si-T0*d_T1_dV_gi_si)/T1/T1);
d_T2_dV_di_si = ((T1*d_T0_dV_di_si-T0*d_T1_dV_di_si)/T1/T1);
d_T2_dV_b_si = ((T1*d_T0_dV_b_si-T0*d_T1_dV_b_si)/T1/T1);
d_T2_dV_sb_si = ((T1*d_T0_dV_sb_si-T0*d_T1_dV_sb_si)/T1/T1);
d_T2_dV_db_di = ((T1*d_T0_dV_db_di-T0*d_T1_dV_db_di)/T1/T1);
d_T2_dV_gi_p = ((T1*d_T0_dV_gi_p-T0*d_T1_dV_gi_p)/T1/T1);
T2 = (T0/T1);

d_T3_dV_b_GND = (T0*d_T2_dV_b_GND+d_T0_dV_b_GND*T2);
d_T3_dV_p_GND = (T0*d_T2_dV_p_GND+d_T0_dV_p_GND*T2);
d_T3_dTemp_t_GND = (T0*d_T2_dTemp_t_GND+d_T0_dTemp_t_GND*T2);
d_T3_dV_e_si = (T0*d_T2_dV_e_si+d_T0_dV_e_si*T2);
d_T3_dV_gi_si = (T0*d_T2_dV_gi_si+d_T0_dV_gi_si*T2);
d_T3_dV_di_si = (T0*d_T2_dV_di_si+d_T0_dV_di_si*T2);
d_T3_dV_b_si = (T0*d_T2_dV_b_si+d_T0_dV_b_si*T2);
d_T3_dV_sb_si = (T0*d_T2_dV_sb_si+d_T0_dV_sb_si*T2);
d_T3_dV_db_di = (T0*d_T2_dV_db_di+d_T0_dV_db_di*T2);
d_T3_dV_gi_p = (T0*d_T2_dV_gi_p+d_T0_dV_gi_p*T2);
T3 = (T0*T2);

d_qinv_dV_e_si = CoxWL*((d_Vgsteff_dV_e_si-0.5*d_T0_dV_e_si)+d_T3_dV_e_si);
d_qinv_dV_b_si = CoxWL*((d_Vgsteff_dV_b_si-0.5*d_T0_dV_b_si)+d_T3_dV_b_si);
d_qinv_dV_b_GND = CoxWL*((d_Vgsteff_dV_b_GND-0.5*d_T0_dV_b_GND)+d_T3_dV_b_GND);
d_qinv_dV_p_GND = CoxWL*((d_Vgsteff_dV_p_GND-0.5*d_T0_dV_p_GND)+d_T3_dV_p_GND);
d_qinv_dTemp_t_GND = CoxWL*((d_Vgsteff_dTemp_t_GND-0.5*d_T0_dTemp_t_GND)+d_T3_dTemp_t_GND);
d_qinv_dV_gi_si = CoxWL*((d_Vgsteff_dV_gi_si-0.5*d_T0_dV_gi_si)+d_T3_dV_gi_si);
d_qinv_dV_di_si = CoxWL*((d_Vgsteff_dV_di_si-0.5*d_T0_dV_di_si)+d_T3_dV_di_si);
d_qinv_dV_sb_si = CoxWL*((d_Vgsteff_dV_sb_si-0.5*d_T0_dV_sb_si)+d_T3_dV_sb_si);
d_qinv_dV_db_di = CoxWL*((d_Vgsteff_dV_db_di-0.5*d_T0_dV_db_di)+d_T3_dV_db_di);
d_qinv_dV_gi_p = CoxWL*((d_Vgsteff_dV_gi_p-0.5*d_T0_dV_gi_p)+d_T3_dV_gi_p);
qinv = (CoxWL*((Vgsteff-(0.5*T0))+T3));
B4SOIqinv = (-qinv);
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T02_dV_sb_si = AbulkCV*d_VdseffCV2_dV_sb_si;
d_T02_dV_db_di = AbulkCV*d_VdseffCV2_dV_db_di;
d_T02_dV_gi_p = AbulkCV*d_VdseffCV2_dV_gi_p;
d_T02_dV_b_GND = (AbulkCV*d_VdseffCV2_dV_b_GND+d_AbulkCV_dV_b_GND*VdseffCV2);
d_T02_dV_p_GND = (AbulkCV*d_VdseffCV2_dV_p_GND+d_AbulkCV_dV_p_GND*VdseffCV2);
d_T02_dTemp_t_GND = (AbulkCV*d_VdseffCV2_dTemp_t_GND+d_AbulkCV_dTemp_t_GND*VdseffCV2);
d_T02_dV_gi_si = (AbulkCV*d_VdseffCV2_dV_gi_si+d_AbulkCV_dV_gi_si*VdseffCV2);
d_T02_dV_di_si = (AbulkCV*d_VdseffCV2_dV_di_si+d_AbulkCV_dV_di_si*VdseffCV2);
d_T02_dV_e_si = (AbulkCV*d_VdseffCV2_dV_e_si+d_AbulkCV_dV_e_si*VdseffCV2);
d_T02_dV_b_si = (AbulkCV*d_VdseffCV2_dV_b_si+d_AbulkCV_dV_b_si*VdseffCV2);
T02 = (AbulkCV*VdseffCV2);

d_T12_dV_sb_si = 12.0*(d_Vgsteff2_dV_sb_si-0.5*d_T02_dV_sb_si);
d_T12_dV_db_di = 12.0*(d_Vgsteff2_dV_db_di-0.5*d_T02_dV_db_di);
d_T12_dV_b_GND = 12.0*(d_Vgsteff2_dV_b_GND-0.5*d_T02_dV_b_GND);
d_T12_dV_p_GND = 12.0*(d_Vgsteff2_dV_p_GND-0.5*d_T02_dV_p_GND);
d_T12_dTemp_t_GND = 12.0*(d_Vgsteff2_dTemp_t_GND-0.5*d_T02_dTemp_t_GND);
d_T12_dV_gi_si = 12.0*(d_Vgsteff2_dV_gi_si-0.5*d_T02_dV_gi_si);
d_T12_dV_di_si = 12.0*(d_Vgsteff2_dV_di_si-0.5*d_T02_dV_di_si);
d_T12_dV_e_si = 12.0*(d_Vgsteff2_dV_e_si-0.5*d_T02_dV_e_si);
d_T12_dV_b_si = 12.0*(d_Vgsteff2_dV_b_si-0.5*d_T02_dV_b_si);
d_T12_dV_gi_p = 12.0*(d_Vgsteff2_dV_gi_p-0.5*d_T02_dV_gi_p);
T12 = (12.0*((Vgsteff2-(0.5*T02))+1e-20));

d_T2_dV_gm_si =  0.0;
d_T2_dV_sb_si = ((T12*d_T02_dV_sb_si-T02*d_T12_dV_sb_si)/T12/T12);
d_T2_dV_db_di = ((T12*d_T02_dV_db_di-T02*d_T12_dV_db_di)/T12/T12);
d_T2_dV_gi_p = ((T12*d_T02_dV_gi_p-T02*d_T12_dV_gi_p)/T12/T12);
d_T2_dV_b_GND = ((T12*d_T02_dV_b_GND-T02*d_T12_dV_b_GND)/T12/T12);
d_T2_dV_p_GND = ((T12*d_T02_dV_p_GND-T02*d_T12_dV_p_GND)/T12/T12);
d_T2_dTemp_t_GND = ((T12*d_T02_dTemp_t_GND-T02*d_T12_dTemp_t_GND)/T12/T12);
d_T2_dV_gi_si = ((T12*d_T02_dV_gi_si-T02*d_T12_dV_gi_si)/T12/T12);
d_T2_dV_di_si = ((T12*d_T02_dV_di_si-T02*d_T12_dV_di_si)/T12/T12);
d_T2_dV_e_si = ((T12*d_T02_dV_e_si-T02*d_T12_dV_e_si)/T12/T12);
d_T2_dV_b_si = ((T12*d_T02_dV_b_si-T02*d_T12_dV_b_si)/T12/T12);
T2 = (T02/T12);

d_T3_dV_sb_si = (T02*d_T2_dV_sb_si+d_T02_dV_sb_si*T2);
d_T3_dV_db_di = (T02*d_T2_dV_db_di+d_T02_dV_db_di*T2);
d_T3_dV_gi_p = (T02*d_T2_dV_gi_p+d_T02_dV_gi_p*T2);
d_T3_dV_b_GND = (T02*d_T2_dV_b_GND+d_T02_dV_b_GND*T2);
d_T3_dV_p_GND = (T02*d_T2_dV_p_GND+d_T02_dV_p_GND*T2);
d_T3_dTemp_t_GND = (T02*d_T2_dTemp_t_GND+d_T02_dTemp_t_GND*T2);
d_T3_dV_gi_si = (T02*d_T2_dV_gi_si+d_T02_dV_gi_si*T2);
d_T3_dV_di_si = (T02*d_T2_dV_di_si+d_T02_dV_di_si*T2);
d_T3_dV_e_si = (T02*d_T2_dV_e_si+d_T02_dV_e_si*T2);
d_T3_dV_b_si = (T02*d_T2_dV_b_si+d_T02_dV_b_si*T2);
T3 = (T02*T2);

d_qinv_dV_e_si = (d_qinv_dV_e_si+CoxWL2*((d_Vgsteff2_dV_e_si-0.5*d_T02_dV_e_si)+d_T3_dV_e_si));
d_qinv_dV_b_si = (d_qinv_dV_b_si+CoxWL2*((d_Vgsteff2_dV_b_si-0.5*d_T02_dV_b_si)+d_T3_dV_b_si));
d_qinv_dV_b_GND = (d_qinv_dV_b_GND+CoxWL2*((d_Vgsteff2_dV_b_GND-0.5*d_T02_dV_b_GND)+d_T3_dV_b_GND));
d_qinv_dV_p_GND = (d_qinv_dV_p_GND+CoxWL2*((d_Vgsteff2_dV_p_GND-0.5*d_T02_dV_p_GND)+d_T3_dV_p_GND));
d_qinv_dTemp_t_GND = (d_qinv_dTemp_t_GND+CoxWL2*((d_Vgsteff2_dTemp_t_GND-0.5*d_T02_dTemp_t_GND)+d_T3_dTemp_t_GND));
d_qinv_dV_gi_si = (d_qinv_dV_gi_si+CoxWL2*((d_Vgsteff2_dV_gi_si-0.5*d_T02_dV_gi_si)+d_T3_dV_gi_si));
d_qinv_dV_di_si = (d_qinv_dV_di_si+CoxWL2*((d_Vgsteff2_dV_di_si-0.5*d_T02_dV_di_si)+d_T3_dV_di_si));
d_qinv_dV_sb_si = (d_qinv_dV_sb_si+CoxWL2*((d_Vgsteff2_dV_sb_si-0.5*d_T02_dV_sb_si)+d_T3_dV_sb_si));
d_qinv_dV_db_di = (d_qinv_dV_db_di+CoxWL2*((d_Vgsteff2_dV_db_di-0.5*d_T02_dV_db_di)+d_T3_dV_db_di));
d_qinv_dV_gi_p = (d_qinv_dV_gi_p+CoxWL2*((d_Vgsteff2_dV_gi_p-0.5*d_T02_dV_gi_p)+d_T3_dV_gi_p));
qinv = (qinv+(CoxWL2*((Vgsteff2-(0.5*T02))+T3)));
B4SOIqinv = (-qinv);
}
if ((B4SOIxpart>0.5))
{

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (d_T1_dV_b_GND+d_T1_dV_b_GND);
d_T1_dV_p_GND = (d_T1_dV_p_GND+d_T1_dV_p_GND);
d_T1_dTemp_t_GND = (d_T1_dTemp_t_GND+d_T1_dTemp_t_GND);
d_T1_dV_gi_si = (d_T1_dV_gi_si+d_T1_dV_gi_si);
d_T1_dV_di_si = (d_T1_dV_di_si+d_T1_dV_di_si);
d_T1_dV_e_si = (d_T1_dV_e_si+d_T1_dV_e_si);
d_T1_dV_b_si = (d_T1_dV_b_si+d_T1_dV_b_si);
d_T1_dV_sb_si = (d_T1_dV_sb_si+d_T1_dV_sb_si);
d_T1_dV_db_di = (d_T1_dV_db_di+d_T1_dV_db_di);
d_T1_dV_gi_p = (d_T1_dV_gi_p+d_T1_dV_gi_p);
T1 = (T1+T1);

d_qsrc_dV_e_si = (-CoxWL)*((0.5*d_Vgsteff_dV_e_si+0.25*d_T0_dV_e_si)-((T1*(T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)-(T0*T0)*d_T1_dV_e_si)/T1/T1));
d_qsrc_dV_b_si = (-CoxWL)*((0.5*d_Vgsteff_dV_b_si+0.25*d_T0_dV_b_si)-((T1*(T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)-(T0*T0)*d_T1_dV_b_si)/T1/T1));
d_qsrc_dV_b_GND = (-CoxWL)*((0.5*d_Vgsteff_dV_b_GND+0.25*d_T0_dV_b_GND)-((T1*(T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)-(T0*T0)*d_T1_dV_b_GND)/T1/T1));
d_qsrc_dV_p_GND = (-CoxWL)*((0.5*d_Vgsteff_dV_p_GND+0.25*d_T0_dV_p_GND)-((T1*(T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)-(T0*T0)*d_T1_dV_p_GND)/T1/T1));
d_qsrc_dTemp_t_GND = (-CoxWL)*((0.5*d_Vgsteff_dTemp_t_GND+0.25*d_T0_dTemp_t_GND)-((T1*(T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)-(T0*T0)*d_T1_dTemp_t_GND)/T1/T1));
d_qsrc_dV_gi_si = (-CoxWL)*((0.5*d_Vgsteff_dV_gi_si+0.25*d_T0_dV_gi_si)-((T1*(T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)-(T0*T0)*d_T1_dV_gi_si)/T1/T1));
d_qsrc_dV_di_si = (-CoxWL)*((0.5*d_Vgsteff_dV_di_si+0.25*d_T0_dV_di_si)-((T1*(T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)-(T0*T0)*d_T1_dV_di_si)/T1/T1));
d_qsrc_dV_sb_si = (-CoxWL)*((0.5*d_Vgsteff_dV_sb_si+0.25*d_T0_dV_sb_si)-((T1*(T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)-(T0*T0)*d_T1_dV_sb_si)/T1/T1));
d_qsrc_dV_db_di = (-CoxWL)*((0.5*d_Vgsteff_dV_db_di+0.25*d_T0_dV_db_di)-((T1*(T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)-(T0*T0)*d_T1_dV_db_di)/T1/T1));
d_qsrc_dV_gi_p = (-CoxWL)*((0.5*d_Vgsteff_dV_gi_p+0.25*d_T0_dV_gi_p)-((T1*(T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)-(T0*T0)*d_T1_dV_gi_p)/T1/T1));
qsrc = ((-CoxWL)*(((0.5*Vgsteff)+(0.25*T0))-((T0*T0)/T1)));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T12_dV_b_GND = (d_T12_dV_b_GND+d_T12_dV_b_GND);
d_T12_dV_p_GND = (d_T12_dV_p_GND+d_T12_dV_p_GND);
d_T12_dTemp_t_GND = (d_T12_dTemp_t_GND+d_T12_dTemp_t_GND);
d_T12_dV_gi_si = (d_T12_dV_gi_si+d_T12_dV_gi_si);
d_T12_dV_di_si = (d_T12_dV_di_si+d_T12_dV_di_si);
d_T12_dV_e_si = (d_T12_dV_e_si+d_T12_dV_e_si);
d_T12_dV_b_si = (d_T12_dV_b_si+d_T12_dV_b_si);
d_T12_dV_sb_si = (d_T12_dV_sb_si+d_T12_dV_sb_si);
d_T12_dV_db_di = (d_T12_dV_db_di+d_T12_dV_db_di);
d_T12_dV_gi_p = (d_T12_dV_gi_p+d_T12_dV_gi_p);
T12 = (T12+T12);

d_qsrc_dV_e_si = (d_qsrc_dV_e_si-CoxWL2*((0.5*d_Vgsteff2_dV_e_si+0.25*d_T02_dV_e_si)-((T12*(T02*d_T02_dV_e_si+d_T02_dV_e_si*T02)-(T02*T02)*d_T12_dV_e_si)/T12/T12)));
d_qsrc_dV_b_si = (d_qsrc_dV_b_si-CoxWL2*((0.5*d_Vgsteff2_dV_b_si+0.25*d_T02_dV_b_si)-((T12*(T02*d_T02_dV_b_si+d_T02_dV_b_si*T02)-(T02*T02)*d_T12_dV_b_si)/T12/T12)));
d_qsrc_dV_b_GND = (d_qsrc_dV_b_GND-CoxWL2*((0.5*d_Vgsteff2_dV_b_GND+0.25*d_T02_dV_b_GND)-((T12*(T02*d_T02_dV_b_GND+d_T02_dV_b_GND*T02)-(T02*T02)*d_T12_dV_b_GND)/T12/T12)));
d_qsrc_dV_p_GND = (d_qsrc_dV_p_GND-CoxWL2*((0.5*d_Vgsteff2_dV_p_GND+0.25*d_T02_dV_p_GND)-((T12*(T02*d_T02_dV_p_GND+d_T02_dV_p_GND*T02)-(T02*T02)*d_T12_dV_p_GND)/T12/T12)));
d_qsrc_dTemp_t_GND = (d_qsrc_dTemp_t_GND-CoxWL2*((0.5*d_Vgsteff2_dTemp_t_GND+0.25*d_T02_dTemp_t_GND)-((T12*(T02*d_T02_dTemp_t_GND+d_T02_dTemp_t_GND*T02)-(T02*T02)*d_T12_dTemp_t_GND)/T12/T12)));
d_qsrc_dV_gi_si = (d_qsrc_dV_gi_si-CoxWL2*((0.5*d_Vgsteff2_dV_gi_si+0.25*d_T02_dV_gi_si)-((T12*(T02*d_T02_dV_gi_si+d_T02_dV_gi_si*T02)-(T02*T02)*d_T12_dV_gi_si)/T12/T12)));
d_qsrc_dV_di_si = (d_qsrc_dV_di_si-CoxWL2*((0.5*d_Vgsteff2_dV_di_si+0.25*d_T02_dV_di_si)-((T12*(T02*d_T02_dV_di_si+d_T02_dV_di_si*T02)-(T02*T02)*d_T12_dV_di_si)/T12/T12)));
d_qsrc_dV_sb_si = (d_qsrc_dV_sb_si-CoxWL2*((0.5*d_Vgsteff2_dV_sb_si+0.25*d_T02_dV_sb_si)-((T12*(T02*d_T02_dV_sb_si+d_T02_dV_sb_si*T02)-(T02*T02)*d_T12_dV_sb_si)/T12/T12)));
d_qsrc_dV_db_di = (d_qsrc_dV_db_di-CoxWL2*((0.5*d_Vgsteff2_dV_db_di+0.25*d_T02_dV_db_di)-((T12*(T02*d_T02_dV_db_di+d_T02_dV_db_di*T02)-(T02*T02)*d_T12_dV_db_di)/T12/T12)));
d_qsrc_dV_gi_p = (d_qsrc_dV_gi_p-CoxWL2*((0.5*d_Vgsteff2_dV_gi_p+0.25*d_T02_dV_gi_p)-((T12*(T02*d_T02_dV_gi_p+d_T02_dV_gi_p*T02)-(T02*T02)*d_T12_dV_gi_p)/T12/T12)));
qsrc = (qsrc-(CoxWL2*(((0.5*Vgsteff2)+(0.25*T02))-((T02*T02)/T12))));
}
}
else
{
if ((B4SOIxpart<0.5))
{

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (d_T1_dV_b_GND/12.0);
d_T1_dV_p_GND = (d_T1_dV_p_GND/12.0);
d_T1_dTemp_t_GND = (d_T1_dTemp_t_GND/12.0);
d_T1_dV_gi_si = (d_T1_dV_gi_si/12.0);
d_T1_dV_di_si = (d_T1_dV_di_si/12.0);
d_T1_dV_e_si = (d_T1_dV_e_si/12.0);
d_T1_dV_b_si = (d_T1_dV_b_si/12.0);
d_T1_dV_sb_si = (d_T1_dV_sb_si/12.0);
d_T1_dV_db_di = (d_T1_dV_db_di/12.0);
d_T1_dV_gi_p = (d_T1_dV_gi_p/12.0);
T1 = (T1/12.0);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = (-(0.5*CoxWL)*(T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)/(T1*T1)/(T1*T1));
d_T2_dV_p_GND = (-(0.5*CoxWL)*(T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)/(T1*T1)/(T1*T1));
d_T2_dTemp_t_GND = (-(0.5*CoxWL)*(T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)/(T1*T1)/(T1*T1));
d_T2_dV_gi_si = (-(0.5*CoxWL)*(T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)/(T1*T1)/(T1*T1));
d_T2_dV_di_si = (-(0.5*CoxWL)*(T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)/(T1*T1)/(T1*T1));
d_T2_dV_e_si = (-(0.5*CoxWL)*(T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)/(T1*T1)/(T1*T1));
d_T2_dV_b_si = (-(0.5*CoxWL)*(T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)/(T1*T1)/(T1*T1));
d_T2_dV_sb_si = (-(0.5*CoxWL)*(T1*d_T1_dV_sb_si+d_T1_dV_sb_si*T1)/(T1*T1)/(T1*T1));
d_T2_dV_db_di = (-(0.5*CoxWL)*(T1*d_T1_dV_db_di+d_T1_dV_db_di*T1)/(T1*T1)/(T1*T1));
d_T2_dV_gi_p = (-(0.5*CoxWL)*(T1*d_T1_dV_gi_p+d_T1_dV_gi_p*T1)/(T1*T1)/(T1*T1));
T2 = ((0.5*CoxWL)/(T1*T1));

d_T3_dV_e_si = ((Vgsteff*((((2.0*T0)*d_T0_dV_e_si+2.0*d_T0_dV_e_si*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_e_si-(4.0*d_T0_dV_e_si/3.0))+d_Vgsteff_dV_e_si*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_e_si*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_e_si+((2.0*T0)*d_T0_dV_e_si+2.0*d_T0_dV_e_si*T0)*T0)/15.0));
d_T3_dV_b_si = ((Vgsteff*((((2.0*T0)*d_T0_dV_b_si+2.0*d_T0_dV_b_si*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_b_si-(4.0*d_T0_dV_b_si/3.0))+d_Vgsteff_dV_b_si*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_b_si*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_b_si+((2.0*T0)*d_T0_dV_b_si+2.0*d_T0_dV_b_si*T0)*T0)/15.0));
d_T3_dV_b_GND = ((Vgsteff*((((2.0*T0)*d_T0_dV_b_GND+2.0*d_T0_dV_b_GND*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_b_GND-(4.0*d_T0_dV_b_GND/3.0))+d_Vgsteff_dV_b_GND*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_b_GND*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_b_GND+((2.0*T0)*d_T0_dV_b_GND+2.0*d_T0_dV_b_GND*T0)*T0)/15.0));
d_T3_dV_p_GND = ((Vgsteff*((((2.0*T0)*d_T0_dV_p_GND+2.0*d_T0_dV_p_GND*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_p_GND-(4.0*d_T0_dV_p_GND/3.0))+d_Vgsteff_dV_p_GND*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_p_GND*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_p_GND+((2.0*T0)*d_T0_dV_p_GND+2.0*d_T0_dV_p_GND*T0)*T0)/15.0));
d_T3_dTemp_t_GND = ((Vgsteff*((((2.0*T0)*d_T0_dTemp_t_GND+2.0*d_T0_dTemp_t_GND*T0)/3.0)+(Vgsteff*(d_Vgsteff_dTemp_t_GND-(4.0*d_T0_dTemp_t_GND/3.0))+d_Vgsteff_dTemp_t_GND*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dTemp_t_GND*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dTemp_t_GND+((2.0*T0)*d_T0_dTemp_t_GND+2.0*d_T0_dTemp_t_GND*T0)*T0)/15.0));
d_T3_dV_gi_si = ((Vgsteff*((((2.0*T0)*d_T0_dV_gi_si+2.0*d_T0_dV_gi_si*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_gi_si-(4.0*d_T0_dV_gi_si/3.0))+d_Vgsteff_dV_gi_si*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_gi_si*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_gi_si+((2.0*T0)*d_T0_dV_gi_si+2.0*d_T0_dV_gi_si*T0)*T0)/15.0));
d_T3_dV_di_si = ((Vgsteff*((((2.0*T0)*d_T0_dV_di_si+2.0*d_T0_dV_di_si*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_di_si-(4.0*d_T0_dV_di_si/3.0))+d_Vgsteff_dV_di_si*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_di_si*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_di_si+((2.0*T0)*d_T0_dV_di_si+2.0*d_T0_dV_di_si*T0)*T0)/15.0));
d_T3_dV_sb_si = ((Vgsteff*((((2.0*T0)*d_T0_dV_sb_si+2.0*d_T0_dV_sb_si*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_sb_si-(4.0*d_T0_dV_sb_si/3.0))+d_Vgsteff_dV_sb_si*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_sb_si*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_sb_si+((2.0*T0)*d_T0_dV_sb_si+2.0*d_T0_dV_sb_si*T0)*T0)/15.0));
d_T3_dV_db_di = ((Vgsteff*((((2.0*T0)*d_T0_dV_db_di+2.0*d_T0_dV_db_di*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_db_di-(4.0*d_T0_dV_db_di/3.0))+d_Vgsteff_dV_db_di*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_db_di*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_db_di+((2.0*T0)*d_T0_dV_db_di+2.0*d_T0_dV_db_di*T0)*T0)/15.0));
d_T3_dV_gi_p = ((Vgsteff*((((2.0*T0)*d_T0_dV_gi_p+2.0*d_T0_dV_gi_p*T0)/3.0)+(Vgsteff*(d_Vgsteff_dV_gi_p-(4.0*d_T0_dV_gi_p/3.0))+d_Vgsteff_dV_gi_p*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dV_gi_p*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_gi_p+((2.0*T0)*d_T0_dV_gi_p+2.0*d_T0_dV_gi_p*T0)*T0)/15.0));
T3 = ((Vgsteff*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*T0)/15.0));

d_qsrc_dV_b_GND = ((-T2)*d_T3_dV_b_GND+(-d_T2_dV_b_GND)*T3);
d_qsrc_dV_p_GND = ((-T2)*d_T3_dV_p_GND+(-d_T2_dV_p_GND)*T3);
d_qsrc_dTemp_t_GND = ((-T2)*d_T3_dTemp_t_GND+(-d_T2_dTemp_t_GND)*T3);
d_qsrc_dV_gi_si = ((-T2)*d_T3_dV_gi_si+(-d_T2_dV_gi_si)*T3);
d_qsrc_dV_di_si = ((-T2)*d_T3_dV_di_si+(-d_T2_dV_di_si)*T3);
d_qsrc_dV_e_si = ((-T2)*d_T3_dV_e_si+(-d_T2_dV_e_si)*T3);
d_qsrc_dV_b_si = ((-T2)*d_T3_dV_b_si+(-d_T2_dV_b_si)*T3);
d_qsrc_dV_sb_si = ((-T2)*d_T3_dV_sb_si+(-d_T2_dV_sb_si)*T3);
d_qsrc_dV_db_di = ((-T2)*d_T3_dV_db_di+(-d_T2_dV_db_di)*T3);
d_qsrc_dV_gi_p = ((-T2)*d_T3_dV_gi_p+(-d_T2_dV_gi_p)*T3);
qsrc = ((-T2)*T3);
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T12_dV_b_GND = (d_T12_dV_b_GND/12.0);
d_T12_dV_p_GND = (d_T12_dV_p_GND/12.0);
d_T12_dTemp_t_GND = (d_T12_dTemp_t_GND/12.0);
d_T12_dV_gi_si = (d_T12_dV_gi_si/12.0);
d_T12_dV_di_si = (d_T12_dV_di_si/12.0);
d_T12_dV_e_si = (d_T12_dV_e_si/12.0);
d_T12_dV_b_si = (d_T12_dV_b_si/12.0);
d_T12_dV_sb_si = (d_T12_dV_sb_si/12.0);
d_T12_dV_db_di = (d_T12_dV_db_di/12.0);
d_T12_dV_gi_p = (d_T12_dV_gi_p/12.0);
T12 = (T12/12.0);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = (-(0.5*CoxWL2)*(T12*d_T12_dV_b_GND+d_T12_dV_b_GND*T12)/(T12*T12)/(T12*T12));
d_T2_dV_p_GND = (-(0.5*CoxWL2)*(T12*d_T12_dV_p_GND+d_T12_dV_p_GND*T12)/(T12*T12)/(T12*T12));
d_T2_dTemp_t_GND = (-(0.5*CoxWL2)*(T12*d_T12_dTemp_t_GND+d_T12_dTemp_t_GND*T12)/(T12*T12)/(T12*T12));
d_T2_dV_gi_si = (-(0.5*CoxWL2)*(T12*d_T12_dV_gi_si+d_T12_dV_gi_si*T12)/(T12*T12)/(T12*T12));
d_T2_dV_di_si = (-(0.5*CoxWL2)*(T12*d_T12_dV_di_si+d_T12_dV_di_si*T12)/(T12*T12)/(T12*T12));
d_T2_dV_e_si = (-(0.5*CoxWL2)*(T12*d_T12_dV_e_si+d_T12_dV_e_si*T12)/(T12*T12)/(T12*T12));
d_T2_dV_b_si = (-(0.5*CoxWL2)*(T12*d_T12_dV_b_si+d_T12_dV_b_si*T12)/(T12*T12)/(T12*T12));
d_T2_dV_sb_si = (-(0.5*CoxWL2)*(T12*d_T12_dV_sb_si+d_T12_dV_sb_si*T12)/(T12*T12)/(T12*T12));
d_T2_dV_db_di = (-(0.5*CoxWL2)*(T12*d_T12_dV_db_di+d_T12_dV_db_di*T12)/(T12*T12)/(T12*T12));
d_T2_dV_gi_p = (-(0.5*CoxWL2)*(T12*d_T12_dV_gi_p+d_T12_dV_gi_p*T12)/(T12*T12)/(T12*T12));
T2 = ((0.5*CoxWL2)/(T12*T12));

d_T3_dV_sb_si = ((Vgsteff2*((((2.0*T02)*d_T02_dV_sb_si+2.0*d_T02_dV_sb_si*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_sb_si-(4.0*d_T02_dV_sb_si/3.0))+d_Vgsteff2_dV_sb_si*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_sb_si*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_sb_si+((2.0*T02)*d_T02_dV_sb_si+2.0*d_T02_dV_sb_si*T02)*T02)/15.0));
d_T3_dV_db_di = ((Vgsteff2*((((2.0*T02)*d_T02_dV_db_di+2.0*d_T02_dV_db_di*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_db_di-(4.0*d_T02_dV_db_di/3.0))+d_Vgsteff2_dV_db_di*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_db_di*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_db_di+((2.0*T02)*d_T02_dV_db_di+2.0*d_T02_dV_db_di*T02)*T02)/15.0));
d_T3_dV_b_GND = ((Vgsteff2*((((2.0*T02)*d_T02_dV_b_GND+2.0*d_T02_dV_b_GND*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_b_GND-(4.0*d_T02_dV_b_GND/3.0))+d_Vgsteff2_dV_b_GND*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_b_GND*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_b_GND+((2.0*T02)*d_T02_dV_b_GND+2.0*d_T02_dV_b_GND*T02)*T02)/15.0));
d_T3_dV_p_GND = ((Vgsteff2*((((2.0*T02)*d_T02_dV_p_GND+2.0*d_T02_dV_p_GND*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_p_GND-(4.0*d_T02_dV_p_GND/3.0))+d_Vgsteff2_dV_p_GND*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_p_GND*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_p_GND+((2.0*T02)*d_T02_dV_p_GND+2.0*d_T02_dV_p_GND*T02)*T02)/15.0));
d_T3_dTemp_t_GND = ((Vgsteff2*((((2.0*T02)*d_T02_dTemp_t_GND+2.0*d_T02_dTemp_t_GND*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dTemp_t_GND-(4.0*d_T02_dTemp_t_GND/3.0))+d_Vgsteff2_dTemp_t_GND*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dTemp_t_GND*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dTemp_t_GND+((2.0*T02)*d_T02_dTemp_t_GND+2.0*d_T02_dTemp_t_GND*T02)*T02)/15.0));
d_T3_dV_gi_si = ((Vgsteff2*((((2.0*T02)*d_T02_dV_gi_si+2.0*d_T02_dV_gi_si*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_gi_si-(4.0*d_T02_dV_gi_si/3.0))+d_Vgsteff2_dV_gi_si*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_gi_si*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_gi_si+((2.0*T02)*d_T02_dV_gi_si+2.0*d_T02_dV_gi_si*T02)*T02)/15.0));
d_T3_dV_di_si = ((Vgsteff2*((((2.0*T02)*d_T02_dV_di_si+2.0*d_T02_dV_di_si*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_di_si-(4.0*d_T02_dV_di_si/3.0))+d_Vgsteff2_dV_di_si*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_di_si*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_di_si+((2.0*T02)*d_T02_dV_di_si+2.0*d_T02_dV_di_si*T02)*T02)/15.0));
d_T3_dV_e_si = ((Vgsteff2*((((2.0*T02)*d_T02_dV_e_si+2.0*d_T02_dV_e_si*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_e_si-(4.0*d_T02_dV_e_si/3.0))+d_Vgsteff2_dV_e_si*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_e_si*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_e_si+((2.0*T02)*d_T02_dV_e_si+2.0*d_T02_dV_e_si*T02)*T02)/15.0));
d_T3_dV_b_si = ((Vgsteff2*((((2.0*T02)*d_T02_dV_b_si+2.0*d_T02_dV_b_si*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_b_si-(4.0*d_T02_dV_b_si/3.0))+d_Vgsteff2_dV_b_si*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_b_si*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_b_si+((2.0*T02)*d_T02_dV_b_si+2.0*d_T02_dV_b_si*T02)*T02)/15.0));
d_T3_dV_gi_p = ((Vgsteff2*((((2.0*T02)*d_T02_dV_gi_p+2.0*d_T02_dV_gi_p*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dV_gi_p-(4.0*d_T02_dV_gi_p/3.0))+d_Vgsteff2_dV_gi_p*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dV_gi_p*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_gi_p+((2.0*T02)*d_T02_dV_gi_p+2.0*d_T02_dV_gi_p*T02)*T02)/15.0));
T3 = ((Vgsteff2*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*T02)/15.0));

d_qsrc2_dV_b_GND = ((-T2)*d_T3_dV_b_GND+(-d_T2_dV_b_GND)*T3);
d_qsrc2_dV_p_GND = ((-T2)*d_T3_dV_p_GND+(-d_T2_dV_p_GND)*T3);
d_qsrc2_dTemp_t_GND = ((-T2)*d_T3_dTemp_t_GND+(-d_T2_dTemp_t_GND)*T3);
d_qsrc2_dV_gi_si = ((-T2)*d_T3_dV_gi_si+(-d_T2_dV_gi_si)*T3);
d_qsrc2_dV_di_si = ((-T2)*d_T3_dV_di_si+(-d_T2_dV_di_si)*T3);
d_qsrc2_dV_e_si = ((-T2)*d_T3_dV_e_si+(-d_T2_dV_e_si)*T3);
d_qsrc2_dV_b_si = ((-T2)*d_T3_dV_b_si+(-d_T2_dV_b_si)*T3);
d_qsrc2_dV_sb_si = ((-T2)*d_T3_dV_sb_si+(-d_T2_dV_sb_si)*T3);
d_qsrc2_dV_db_di = ((-T2)*d_T3_dV_db_di+(-d_T2_dV_db_di)*T3);
d_qsrc2_dV_gi_p = ((-T2)*d_T3_dV_gi_p+(-d_T2_dV_gi_p)*T3);
qsrc2 = ((-T2)*T3);

d_qsrc_dV_e_si = (d_qsrc_dV_e_si+d_qsrc2_dV_e_si);
d_qsrc_dV_b_si = (d_qsrc_dV_b_si+d_qsrc2_dV_b_si);
d_qsrc_dV_b_GND = (d_qsrc_dV_b_GND+d_qsrc2_dV_b_GND);
d_qsrc_dV_p_GND = (d_qsrc_dV_p_GND+d_qsrc2_dV_p_GND);
d_qsrc_dTemp_t_GND = (d_qsrc_dTemp_t_GND+d_qsrc2_dTemp_t_GND);
d_qsrc_dV_gi_si = (d_qsrc_dV_gi_si+d_qsrc2_dV_gi_si);
d_qsrc_dV_di_si = (d_qsrc_dV_di_si+d_qsrc2_dV_di_si);
d_qsrc_dV_sb_si = (d_qsrc_dV_sb_si+d_qsrc2_dV_sb_si);
d_qsrc_dV_db_di = (d_qsrc_dV_db_di+d_qsrc2_dV_db_di);
d_qsrc_dV_gi_p = (d_qsrc_dV_gi_p+d_qsrc2_dV_gi_p);
qsrc = (qsrc+qsrc2);
}
}
else
{

d_qsrc_dV_e_si = (-0.5)*(d_qinv_dV_e_si+d_qbulk_dV_e_si);
d_qsrc_dV_b_si = (-0.5)*(d_qinv_dV_b_si+d_qbulk_dV_b_si);
d_qsrc_dV_b_GND = (-0.5)*(d_qinv_dV_b_GND+d_qbulk_dV_b_GND);
d_qsrc_dV_p_GND = (-0.5)*(d_qinv_dV_p_GND+d_qbulk_dV_p_GND);
d_qsrc_dTemp_t_GND = (-0.5)*(d_qinv_dTemp_t_GND+d_qbulk_dTemp_t_GND);
d_qsrc_dV_gi_si = (-0.5)*(d_qinv_dV_gi_si+d_qbulk_dV_gi_si);
d_qsrc_dV_di_si = (-0.5)*(d_qinv_dV_di_si+d_qbulk_dV_di_si);
d_qsrc_dV_sb_si = (-0.5)*(d_qinv_dV_sb_si+d_qbulk_dV_sb_si);
d_qsrc_dV_db_di = (-0.5)*(d_qinv_dV_db_di+d_qbulk_dV_db_di);
d_qsrc_dV_gi_p = (-0.5)*(d_qinv_dV_gi_p+d_qbulk_dV_gi_p);
qsrc = ((-0.5)*(qinv+qbulk));
}
}
if ((B4SOIsoiMod==2))
{

d_Qe1_dV_di_si = d_Qe1_dV_e_si = d_Qe1_dTemp_t_GND = d_Qe1_dV_p_GND = d_Qe1_dV_b_GND = d_Qe1_dV_gi_si = d_Qe1_dV_b_si =  0.0;
Qe1 = 0;
}
else
{
CboxWL = (((pParam_B4SOIkb1*B4SOIfbody)*Cbox)*((((pParam_B4SOIweffCV/B4SOInseg)*B4SOInf)*pParam_B4SOIleffCVbg)+B4SOIaebcp));

d_Qe1_dV_b_si = CboxWL*(-d_Vbs_dV_b_si);
d_Qe1_dV_gi_si = CboxWL*(-d_Vbs_dV_gi_si);
d_Qe1_dV_b_GND = CboxWL*(d_Vesfb_dV_b_GND-d_Vbs_dV_b_GND);
d_Qe1_dV_p_GND = CboxWL*(d_Vesfb_dV_p_GND-d_Vbs_dV_p_GND);
d_Qe1_dTemp_t_GND = CboxWL*(d_Vesfb_dTemp_t_GND-d_Vbs_dTemp_t_GND);
d_Qe1_dV_e_si = CboxWL*(d_Vesfb_dV_e_si-d_Vbs_dV_e_si);
d_Qe1_dV_di_si = CboxWL*(d_Vesfb_dV_di_si-d_Vbs_dV_di_si);
Qe1 = (CboxWL*(Vesfb-Vbs));
}

d_qgate_dV_gm_si =  0.0;
d_qgate_dV_e_si = ((d_qinv_dV_e_si+d_Qac0_dV_e_si)+d_Qsub0_dV_e_si);
d_qgate_dV_b_si = ((d_qinv_dV_b_si+d_Qac0_dV_b_si)+d_Qsub0_dV_b_si);
d_qgate_dV_b_GND = ((d_qinv_dV_b_GND+d_Qac0_dV_b_GND)+d_Qsub0_dV_b_GND);
d_qgate_dV_p_GND = ((d_qinv_dV_p_GND+d_Qac0_dV_p_GND)+d_Qsub0_dV_p_GND);
d_qgate_dTemp_t_GND = ((d_qinv_dTemp_t_GND+d_Qac0_dTemp_t_GND)+d_Qsub0_dTemp_t_GND);
d_qgate_dV_gi_si = ((d_qinv_dV_gi_si+d_Qac0_dV_gi_si)+d_Qsub0_dV_gi_si);
d_qgate_dV_di_si = ((d_qinv_dV_di_si+d_Qac0_dV_di_si)+d_Qsub0_dV_di_si);
d_qgate_dV_sb_si = ((d_qinv_dV_sb_si+d_Qac0_dV_sb_si)+d_Qsub0_dV_sb_si);
d_qgate_dV_db_di = ((d_qinv_dV_db_di+d_Qac0_dV_db_di)+d_Qsub0_dV_db_di);
d_qgate_dV_gi_p = ((d_qinv_dV_gi_p+d_Qac0_dV_gi_p)+d_Qsub0_dV_gi_p);
qgate = ((qinv+Qac0)+Qsub0);

d_qbody_dV_sb_si = ((d_qbulk_dV_sb_si-d_Qac0_dV_sb_si)-d_Qsub0_dV_sb_si);
d_qbody_dV_db_di = ((d_qbulk_dV_db_di-d_Qac0_dV_db_di)-d_Qsub0_dV_db_di);
d_qbody_dV_gi_p = ((d_qbulk_dV_gi_p-d_Qac0_dV_gi_p)-d_Qsub0_dV_gi_p);
d_qbody_dV_b_GND = (((d_qbulk_dV_b_GND-d_Qac0_dV_b_GND)-d_Qsub0_dV_b_GND)-d_Qe1_dV_b_GND);
d_qbody_dV_p_GND = (((d_qbulk_dV_p_GND-d_Qac0_dV_p_GND)-d_Qsub0_dV_p_GND)-d_Qe1_dV_p_GND);
d_qbody_dTemp_t_GND = (((d_qbulk_dTemp_t_GND-d_Qac0_dTemp_t_GND)-d_Qsub0_dTemp_t_GND)-d_Qe1_dTemp_t_GND);
d_qbody_dV_gi_si = (((d_qbulk_dV_gi_si-d_Qac0_dV_gi_si)-d_Qsub0_dV_gi_si)-d_Qe1_dV_gi_si);
d_qbody_dV_di_si = (((d_qbulk_dV_di_si-d_Qac0_dV_di_si)-d_Qsub0_dV_di_si)-d_Qe1_dV_di_si);
d_qbody_dV_e_si = (((d_qbulk_dV_e_si-d_Qac0_dV_e_si)-d_Qsub0_dV_e_si)-d_Qe1_dV_e_si);
d_qbody_dV_b_si = (((d_qbulk_dV_b_si-d_Qac0_dV_b_si)-d_Qsub0_dV_b_si)-d_Qe1_dV_b_si);
qbody = (((qbulk-Qac0)-Qsub0)-Qe1);

d_qsub_dV_b_si = d_Qe1_dV_b_si;
d_qsub_dV_gi_si = d_Qe1_dV_gi_si;
d_qsub_dV_b_GND = d_Qe1_dV_b_GND;
d_qsub_dV_p_GND = d_Qe1_dV_p_GND;
d_qsub_dTemp_t_GND = d_Qe1_dTemp_t_GND;
d_qsub_dV_e_si = d_Qe1_dV_e_si;
d_qsub_dV_di_si = d_Qe1_dV_di_si;
qsub = Qe1;

d_qdrn_dV_e_si = (-(((d_qgate_dV_e_si+d_qsrc_dV_e_si)+d_qbody_dV_e_si)+d_qsub_dV_e_si));
d_qdrn_dV_b_si = (-(((d_qgate_dV_b_si+d_qsrc_dV_b_si)+d_qbody_dV_b_si)+d_qsub_dV_b_si));
d_qdrn_dV_b_GND = (-(((d_qgate_dV_b_GND+d_qsrc_dV_b_GND)+d_qbody_dV_b_GND)+d_qsub_dV_b_GND));
d_qdrn_dV_p_GND = (-(((d_qgate_dV_p_GND+d_qsrc_dV_p_GND)+d_qbody_dV_p_GND)+d_qsub_dV_p_GND));
d_qdrn_dTemp_t_GND = (-(((d_qgate_dTemp_t_GND+d_qsrc_dTemp_t_GND)+d_qbody_dTemp_t_GND)+d_qsub_dTemp_t_GND));
d_qdrn_dV_gi_si = (-(((d_qgate_dV_gi_si+d_qsrc_dV_gi_si)+d_qbody_dV_gi_si)+d_qsub_dV_gi_si));
d_qdrn_dV_di_si = (-(((d_qgate_dV_di_si+d_qsrc_dV_di_si)+d_qbody_dV_di_si)+d_qsub_dV_di_si));
d_qdrn_dV_sb_si = (-((d_qgate_dV_sb_si+d_qsrc_dV_sb_si)+d_qbody_dV_sb_si));
d_qdrn_dV_db_di = (-((d_qgate_dV_db_di+d_qsrc_dV_db_di)+d_qbody_dV_db_di));
d_qdrn_dV_gi_p = (-((d_qgate_dV_gi_p+d_qsrc_dV_gi_p)+d_qbody_dV_gi_p));
qdrn = (-(((qgate+qsrc)+qbody)+qsub));
}
else
{
if ((B4SOIcapMod==3))
{
if ((B4SOImtrlMod==0))
{
Cox = (3.453133e-11/B4SOItoxp);
}
else
{
Cox = ((epsrox*8.85418e-12)/B4SOItoxp);
}
CoxWL = ((CoxWL*toxe)/B4SOItoxp);
CoxWLb = ((CoxWLb*B4SOItox)/B4SOItoxp);
Tox = (1.0e8*B4SOItoxp);
if ((B4SOIagbcp2>0))
{
CoxWL2 = ((CoxWL2*B4SOItox)/B4SOItoxp);
CoxWLb2 = ((CoxWLb2*B4SOItox)/B4SOItoxp);
}
if ((B4SOIsoiMod==2))
{

d_Qac0_dV_gi_p = d_Qac0_dTemp_t_GND = d_Qac0_dV_p_GND = d_Qac0_dV_b_GND = d_Qac0_dV_db_di = d_Qac0_dV_sb_si = d_Qac0_dV_b_si = d_Qac0_dV_e_si = d_Qac0_dV_di_si = d_Qac0_dV_gi_si =  0.0;
Qac0 = 0.0;

d_Qsub0_dV_gi_p = d_Qsub0_dV_db_di = d_Qsub0_dV_sb_si = d_Qsub0_dV_b_si = d_Qsub0_dV_e_si = d_Qsub0_dV_di_si = d_Qsub0_dV_gi_si = d_Qsub0_dTemp_t_GND = d_Qsub0_dV_p_GND = d_Qsub0_dV_b_GND =  0.0;
Qsub0 = 0.0;

d_Vfbzb_dTemp_t_GND = d_Vfbzb_dV_p_GND = d_Vfbzb_dV_b_GND = d_Vfbzb_dV_b_si = d_Vfbzb_dV_e_si = d_Vfbzb_dV_di_si = d_Vfbzb_dV_gi_si =  0.0;
Vfbzb = 0;
}
else
{
if (((SHMOD==1)&&(RTH0!=0.0)))
{

d_Vfbzb_dV_gi_si = d_Vthzb_dV_gi_si;
d_Vfbzb_dV_di_si = d_Vthzb_dV_di_si;
d_Vfbzb_dV_e_si = d_Vthzb_dV_e_si;
d_Vfbzb_dV_b_si = d_Vthzb_dV_b_si;
d_Vfbzb_dV_b_GND = ((d_Vthzb_dV_b_GND-d_phi_dV_b_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dV_b_GND+d_pParam_B4SOIk1eff_dV_b_GND*sqrtPhi));
d_Vfbzb_dV_p_GND = ((d_Vthzb_dV_p_GND-d_phi_dV_p_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dV_p_GND+d_pParam_B4SOIk1eff_dV_p_GND*sqrtPhi));
d_Vfbzb_dTemp_t_GND = ((d_Vthzb_dTemp_t_GND-d_phi_dTemp_t_GND)-(pParam_B4SOIk1eff*d_sqrtPhi_dTemp_t_GND+d_pParam_B4SOIk1eff_dTemp_t_GND*sqrtPhi));
Vfbzb = (((Vthzb-phi)-(pParam_B4SOIk1eff*sqrtPhi))+pParam_B4SOIdelvt);
}
else
{

d_Vfbzb_dTemp_t_GND = d_Vfbzb_dV_p_GND = d_Vfbzb_dV_b_GND = d_Vfbzb_dV_b_si = d_Vfbzb_dV_e_si = d_Vfbzb_dV_di_si = d_Vfbzb_dV_gi_si =  0.0;
Vfbzb = (B4SOIvfbzb+pParam_B4SOIdelvt);
}

d_V3_dV_gi_p = d_V3_dV_db_di = d_V3_dV_sb_si =  0.0;
d_V3_dV_gi_si = ((d_Vfbzb_dV_gi_si-d_Vgs_eff_dV_gi_si)+d_Vbseff_dV_gi_si);
d_V3_dV_di_si = ((d_Vfbzb_dV_di_si-d_Vgs_eff_dV_di_si)+d_Vbseff_dV_di_si);
d_V3_dV_e_si = (d_Vfbzb_dV_e_si+d_Vbseff_dV_e_si);
d_V3_dV_b_si = (d_Vfbzb_dV_b_si+d_Vbseff_dV_b_si);
d_V3_dV_b_GND = ((d_Vfbzb_dV_b_GND-d_Vgs_eff_dV_b_GND)+d_Vbseff_dV_b_GND);
d_V3_dV_p_GND = ((d_Vfbzb_dV_p_GND-d_Vgs_eff_dV_p_GND)+d_Vbseff_dV_p_GND);
d_V3_dTemp_t_GND = ((d_Vfbzb_dTemp_t_GND-d_Vgs_eff_dTemp_t_GND)+d_Vbseff_dTemp_t_GND);
V3 = (((Vfbzb-Vgs_eff)+Vbseff)-0.02);
if ((Vfbzb<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((4.0*0.02)*Vfbzb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)-(4.0*0.02)*d_Vfbzb_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)-(4.0*0.02)*d_Vfbzb_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)-(4.0*0.02)*d_Vfbzb_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)-(4.0*0.02)*d_Vfbzb_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)-(4.0*0.02)*d_Vfbzb_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)-(4.0*0.02)*d_Vfbzb_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)-(4.0*0.02)*d_Vfbzb_dV_b_si)));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*0.02)*Vfbzb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)+(4.0*0.02)*d_Vfbzb_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)+(4.0*0.02)*d_Vfbzb_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)+(4.0*0.02)*d_Vfbzb_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)+(4.0*0.02)*d_Vfbzb_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)+(4.0*0.02)*d_Vfbzb_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)+(4.0*0.02)*d_Vfbzb_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)+(4.0*0.02)*d_Vfbzb_dV_b_si)));
T0 = value_sqrt_0;
}
}

d_Vfbeff_dV_sb_si = (-0.5*(d_V3_dV_sb_si+d_T0_dV_sb_si));
d_Vfbeff_dV_db_di = (-0.5*(d_V3_dV_db_di+d_T0_dV_db_di));
d_Vfbeff_dV_gi_p = (-0.5*(d_V3_dV_gi_p+d_T0_dV_gi_p));
d_Vfbeff_dV_gi_si = (d_Vfbzb_dV_gi_si-0.5*(d_V3_dV_gi_si+d_T0_dV_gi_si));
d_Vfbeff_dV_di_si = (d_Vfbzb_dV_di_si-0.5*(d_V3_dV_di_si+d_T0_dV_di_si));
d_Vfbeff_dV_e_si = (d_Vfbzb_dV_e_si-0.5*(d_V3_dV_e_si+d_T0_dV_e_si));
d_Vfbeff_dV_b_si = (d_Vfbzb_dV_b_si-0.5*(d_V3_dV_b_si+d_T0_dV_b_si));
d_Vfbeff_dV_b_GND = (d_Vfbzb_dV_b_GND-0.5*(d_V3_dV_b_GND+d_T0_dV_b_GND));
d_Vfbeff_dV_p_GND = (d_Vfbzb_dV_p_GND-0.5*(d_V3_dV_p_GND+d_T0_dV_p_GND));
d_Vfbeff_dTemp_t_GND = (d_Vfbzb_dTemp_t_GND-0.5*(d_V3_dTemp_t_GND+d_T0_dTemp_t_GND));
Vfbeff = (Vfbzb-(0.5*(V3+T0)));
if ((B4SOIagbcp2>0))
{

d_Vfbzb2_dV_gi_si = d_Vfbzb_dV_gi_si;
d_Vfbzb2_dV_di_si = d_Vfbzb_dV_di_si;
d_Vfbzb2_dV_e_si = d_Vfbzb_dV_e_si;
d_Vfbzb2_dV_b_si = d_Vfbzb_dV_b_si;
d_Vfbzb2_dV_b_GND = d_Vfbzb_dV_b_GND;
d_Vfbzb2_dV_p_GND = d_Vfbzb_dV_p_GND;
d_Vfbzb2_dTemp_t_GND = d_Vfbzb_dTemp_t_GND;
Vfbzb2 = (Vfbzb+eggbcp2);

d_V3_dV_gi_p = d_V3_dV_db_di = d_V3_dV_sb_si =  0.0;
d_V3_dV_gi_si = ((d_Vfbzb2_dV_gi_si-d_Vgs_eff2_dV_gi_si)+d_Vbseff_dV_gi_si);
d_V3_dV_di_si = ((d_Vfbzb2_dV_di_si-d_Vgs_eff2_dV_di_si)+d_Vbseff_dV_di_si);
d_V3_dV_e_si = (d_Vfbzb2_dV_e_si+d_Vbseff_dV_e_si);
d_V3_dV_b_si = (d_Vfbzb2_dV_b_si+d_Vbseff_dV_b_si);
d_V3_dV_b_GND = (d_Vfbzb2_dV_b_GND+d_Vbseff_dV_b_GND);
d_V3_dV_p_GND = (d_Vfbzb2_dV_p_GND+d_Vbseff_dV_p_GND);
d_V3_dTemp_t_GND = (d_Vfbzb2_dTemp_t_GND+d_Vbseff_dTemp_t_GND);
V3 = (((Vfbzb2-Vgs_eff2)+Vbseff)-0.02);
if ((Vfbzb2<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((100.0*0.02)*Vfbzb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)-(100.0*0.02)*d_Vfbzb2_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)-(100.0*0.02)*d_Vfbzb2_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)-(100.0*0.02)*d_Vfbzb2_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)-(100.0*0.02)*d_Vfbzb2_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)-(100.0*0.02)*d_Vfbzb2_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)-(100.0*0.02)*d_Vfbzb2_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)-(100.0*0.02)*d_Vfbzb2_dV_b_si)));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((100.0*0.02)*Vfbzb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)+(100.0*0.02)*d_Vfbzb2_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)+(100.0*0.02)*d_Vfbzb2_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)+(100.0*0.02)*d_Vfbzb2_dTemp_t_GND)));
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)+(100.0*0.02)*d_Vfbzb2_dV_e_si)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)+(100.0*0.02)*d_Vfbzb2_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)+(100.0*0.02)*d_Vfbzb2_dV_di_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)+(100.0*0.02)*d_Vfbzb2_dV_b_si)));
T0 = value_sqrt_0;
}
}

d_Vfbeff2_dV_sb_si = (-0.5*(d_V3_dV_sb_si+d_T0_dV_sb_si));
d_Vfbeff2_dV_db_di = (-0.5*(d_V3_dV_db_di+d_T0_dV_db_di));
d_Vfbeff2_dV_gi_p = (-0.5*(d_V3_dV_gi_p+d_T0_dV_gi_p));
d_Vfbeff2_dV_gi_si = (d_Vfbzb2_dV_gi_si-0.5*(d_V3_dV_gi_si+d_T0_dV_gi_si));
d_Vfbeff2_dV_di_si = (d_Vfbzb2_dV_di_si-0.5*(d_V3_dV_di_si+d_T0_dV_di_si));
d_Vfbeff2_dV_e_si = (d_Vfbzb2_dV_e_si-0.5*(d_V3_dV_e_si+d_T0_dV_e_si));
d_Vfbeff2_dV_b_si = (d_Vfbzb2_dV_b_si-0.5*(d_V3_dV_b_si+d_T0_dV_b_si));
d_Vfbeff2_dV_b_GND = (d_Vfbzb2_dV_b_GND-0.5*(d_V3_dV_b_GND+d_T0_dV_b_GND));
d_Vfbeff2_dV_p_GND = (d_Vfbzb2_dV_p_GND-0.5*(d_V3_dV_p_GND+d_T0_dV_p_GND));
d_Vfbeff2_dTemp_t_GND = (d_Vfbzb2_dTemp_t_GND-0.5*(d_V3_dTemp_t_GND+d_T0_dTemp_t_GND));
Vfbeff2 = (Vfbzb2-(0.5*(V3+T0)));
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_e_si = (((-d_Vbseff_dV_e_si)-d_Vfbzb_dV_e_si)/Tox);
d_T0_dV_b_si = (((-d_Vbseff_dV_b_si)-d_Vfbzb_dV_b_si)/Tox);
d_T0_dV_b_GND = (((d_Vgs_eff_dV_b_GND-d_Vbseff_dV_b_GND)-d_Vfbzb_dV_b_GND)/Tox);
d_T0_dV_p_GND = (((d_Vgs_eff_dV_p_GND-d_Vbseff_dV_p_GND)-d_Vfbzb_dV_p_GND)/Tox);
d_T0_dTemp_t_GND = (((d_Vgs_eff_dTemp_t_GND-d_Vbseff_dTemp_t_GND)-d_Vfbzb_dTemp_t_GND)/Tox);
d_T0_dV_gi_si = (((d_Vgs_eff_dV_gi_si-d_Vbseff_dV_gi_si)-d_Vfbzb_dV_gi_si)/Tox);
d_T0_dV_di_si = (((d_Vgs_eff_dV_di_si-d_Vbseff_dV_di_si)-d_Vfbzb_dV_di_si)/Tox);
T0 = (((Vgs_eff-Vbseff)-Vfbzb)/Tox);

d_tmp_dV_b_GND = d_T0_dV_b_GND*pParam_B4SOIacde;
d_tmp_dV_p_GND = d_T0_dV_p_GND*pParam_B4SOIacde;
d_tmp_dTemp_t_GND = d_T0_dTemp_t_GND*pParam_B4SOIacde;
d_tmp_dV_e_si = d_T0_dV_e_si*pParam_B4SOIacde;
d_tmp_dV_gi_si = d_T0_dV_gi_si*pParam_B4SOIacde;
d_tmp_dV_di_si = d_T0_dV_di_si*pParam_B4SOIacde;
d_tmp_dV_b_si = d_T0_dV_b_si*pParam_B4SOIacde;
d_tmp_dV_sb_si = d_T0_dV_sb_si*pParam_B4SOIacde;
d_tmp_dV_db_di = d_T0_dV_db_di*pParam_B4SOIacde;
d_tmp_dV_gi_p = d_T0_dV_gi_p*pParam_B4SOIacde;
tmp = (T0*pParam_B4SOIacde);
if ((((-100.0)<tmp)&&(tmp<100.0)))
{
{
double value_exp_0 = exp(tmp);
double  deriv_exp_0_d0 = value_exp_0;

d_Tcen_dV_b_GND = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_b_GND));
d_Tcen_dV_p_GND = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_p_GND));
d_Tcen_dTemp_t_GND = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dTemp_t_GND));
d_Tcen_dV_e_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_e_si));
d_Tcen_dV_gi_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_gi_si));
d_Tcen_dV_di_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_di_si));
d_Tcen_dV_b_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_b_si));
d_Tcen_dV_sb_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_sb_si));
d_Tcen_dV_db_di = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_db_di));
d_Tcen_dV_gi_p = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_gi_p));
Tcen = (pParam_B4SOIldeb*value_exp_0);
}
}
else
{
if ((tmp<=(-100.0)))
{

d_Tcen_dV_gi_p = d_Tcen_dV_db_di = d_Tcen_dV_sb_si = d_Tcen_dV_b_si = d_Tcen_dV_di_si = d_Tcen_dV_gi_si = d_Tcen_dV_e_si = d_Tcen_dTemp_t_GND = d_Tcen_dV_p_GND = d_Tcen_dV_b_GND =  0.0;
Tcen = (pParam_B4SOIldeb*3.720075976e-44);
}
else
{

d_Tcen_dV_gi_p = d_Tcen_dV_db_di = d_Tcen_dV_sb_si = d_Tcen_dV_b_si = d_Tcen_dV_di_si = d_Tcen_dV_gi_si = d_Tcen_dV_e_si = d_Tcen_dTemp_t_GND = d_Tcen_dV_p_GND = d_Tcen_dV_b_GND =  0.0;
Tcen = (pParam_B4SOIldeb*2.688117142e+43);
}
}
LINK = (1.0e-3*B4SOItoxp);

d_V3_dV_b_GND = (-d_Tcen_dV_b_GND);
d_V3_dV_p_GND = (-d_Tcen_dV_p_GND);
d_V3_dTemp_t_GND = (-d_Tcen_dTemp_t_GND);
d_V3_dV_e_si = (-d_Tcen_dV_e_si);
d_V3_dV_gi_si = (-d_Tcen_dV_gi_si);
d_V3_dV_di_si = (-d_Tcen_dV_di_si);
d_V3_dV_b_si = (-d_Tcen_dV_b_si);
d_V3_dV_sb_si = (-d_Tcen_dV_sb_si);
d_V3_dV_db_di = (-d_Tcen_dV_db_di);
d_V3_dV_gi_p = (-d_Tcen_dV_gi_p);
V3 = ((pParam_B4SOIldeb-Tcen)-LINK);
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*LINK)*pParam_B4SOIldeb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_V4_dV_b_GND = (deriv_sqrt_0_d0*((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)));
d_V4_dV_p_GND = (deriv_sqrt_0_d0*((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)));
d_V4_dTemp_t_GND = (deriv_sqrt_0_d0*((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)));
d_V4_dV_e_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)));
d_V4_dV_gi_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)));
d_V4_dV_di_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)));
d_V4_dV_b_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)));
d_V4_dV_sb_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_sb_si+d_V3_dV_sb_si*V3)));
d_V4_dV_db_di = (deriv_sqrt_0_d0*((V3*d_V3_dV_db_di+d_V3_dV_db_di*V3)));
d_V4_dV_gi_p = (deriv_sqrt_0_d0*((V3*d_V3_dV_gi_p+d_V3_dV_gi_p*V3)));
V4 = value_sqrt_0;
}

d_Tcen_dV_b_GND = (-0.5*(d_V3_dV_b_GND+d_V4_dV_b_GND));
d_Tcen_dV_p_GND = (-0.5*(d_V3_dV_p_GND+d_V4_dV_p_GND));
d_Tcen_dTemp_t_GND = (-0.5*(d_V3_dTemp_t_GND+d_V4_dTemp_t_GND));
d_Tcen_dV_e_si = (-0.5*(d_V3_dV_e_si+d_V4_dV_e_si));
d_Tcen_dV_gi_si = (-0.5*(d_V3_dV_gi_si+d_V4_dV_gi_si));
d_Tcen_dV_di_si = (-0.5*(d_V3_dV_di_si+d_V4_dV_di_si));
d_Tcen_dV_b_si = (-0.5*(d_V3_dV_b_si+d_V4_dV_b_si));
d_Tcen_dV_sb_si = (-0.5*(d_V3_dV_sb_si+d_V4_dV_sb_si));
d_Tcen_dV_db_di = (-0.5*(d_V3_dV_db_di+d_V4_dV_db_di));
d_Tcen_dV_gi_p = (-0.5*(d_V3_dV_gi_p+d_V4_dV_gi_p));
Tcen = (pParam_B4SOIldeb-(0.5*(V3+V4)));
if ((Tcen<1e-15))
{

d_Tcen_dV_gi_p = d_Tcen_dV_db_di = d_Tcen_dV_sb_si = d_Tcen_dV_b_si = d_Tcen_dV_di_si = d_Tcen_dV_gi_si = d_Tcen_dV_e_si = d_Tcen_dTemp_t_GND = d_Tcen_dV_p_GND = d_Tcen_dV_b_GND =  0.0;
Tcen = 1e-15;
}
if ((B4SOIagbcp2>0))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_b_GND = (((-d_Vbseff_dV_b_GND)-d_Vfbzb2_dV_b_GND)/Tox);
d_T0_dV_p_GND = (((-d_Vbseff_dV_p_GND)-d_Vfbzb2_dV_p_GND)/Tox);
d_T0_dTemp_t_GND = (((-d_Vbseff_dTemp_t_GND)-d_Vfbzb2_dTemp_t_GND)/Tox);
d_T0_dV_e_si = (((-d_Vbseff_dV_e_si)-d_Vfbzb2_dV_e_si)/Tox);
d_T0_dV_b_si = (((-d_Vbseff_dV_b_si)-d_Vfbzb2_dV_b_si)/Tox);
d_T0_dV_gi_si = (((d_Vgs_eff2_dV_gi_si-d_Vbseff_dV_gi_si)-d_Vfbzb2_dV_gi_si)/Tox);
d_T0_dV_di_si = (((d_Vgs_eff2_dV_di_si-d_Vbseff_dV_di_si)-d_Vfbzb2_dV_di_si)/Tox);
T0 = (((Vgs_eff2-Vbseff)-Vfbzb2)/Tox);

d_tmp_dV_b_GND = d_T0_dV_b_GND*pParam_B4SOIacde;
d_tmp_dV_p_GND = d_T0_dV_p_GND*pParam_B4SOIacde;
d_tmp_dTemp_t_GND = d_T0_dTemp_t_GND*pParam_B4SOIacde;
d_tmp_dV_e_si = d_T0_dV_e_si*pParam_B4SOIacde;
d_tmp_dV_gi_si = d_T0_dV_gi_si*pParam_B4SOIacde;
d_tmp_dV_di_si = d_T0_dV_di_si*pParam_B4SOIacde;
d_tmp_dV_b_si = d_T0_dV_b_si*pParam_B4SOIacde;
d_tmp_dV_sb_si = d_T0_dV_sb_si*pParam_B4SOIacde;
d_tmp_dV_db_di = d_T0_dV_db_di*pParam_B4SOIacde;
d_tmp_dV_gi_p = d_T0_dV_gi_p*pParam_B4SOIacde;
tmp = (T0*pParam_B4SOIacde);
if ((((-100.0)<tmp)&&(tmp<100.0)))
{
{
double value_exp_0 = exp(tmp);
double  deriv_exp_0_d0 = value_exp_0;

d_Tcen2_dV_b_GND = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_b_GND));
d_Tcen2_dV_p_GND = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_p_GND));
d_Tcen2_dTemp_t_GND = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dTemp_t_GND));
d_Tcen2_dV_e_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_e_si));
d_Tcen2_dV_gi_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_gi_si));
d_Tcen2_dV_di_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_di_si));
d_Tcen2_dV_b_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_b_si));
d_Tcen2_dV_sb_si = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_sb_si));
d_Tcen2_dV_db_di = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_db_di));
d_Tcen2_dV_gi_p = pParam_B4SOIldeb*(deriv_exp_0_d0*(d_tmp_dV_gi_p));
Tcen2 = (pParam_B4SOIldeb*value_exp_0);
}
}
else
{
if ((tmp<=(-100.0)))
{

d_Tcen2_dV_gi_p = d_Tcen2_dV_db_di = d_Tcen2_dV_sb_si = d_Tcen2_dV_b_si = d_Tcen2_dV_di_si = d_Tcen2_dV_gi_si = d_Tcen2_dV_e_si = d_Tcen2_dTemp_t_GND = d_Tcen2_dV_p_GND = d_Tcen2_dV_b_GND =  0.0;
Tcen2 = (pParam_B4SOIldeb*3.720075976e-44);
}
else
{

d_Tcen2_dV_gi_p = d_Tcen2_dV_db_di = d_Tcen2_dV_sb_si = d_Tcen2_dV_b_si = d_Tcen2_dV_di_si = d_Tcen2_dV_gi_si = d_Tcen2_dV_e_si = d_Tcen2_dTemp_t_GND = d_Tcen2_dV_p_GND = d_Tcen2_dV_b_GND =  0.0;
Tcen2 = (pParam_B4SOIldeb*2.688117142e+43);
}
}

d_V3_dV_b_GND = (-d_Tcen2_dV_b_GND);
d_V3_dV_p_GND = (-d_Tcen2_dV_p_GND);
d_V3_dTemp_t_GND = (-d_Tcen2_dTemp_t_GND);
d_V3_dV_e_si = (-d_Tcen2_dV_e_si);
d_V3_dV_gi_si = (-d_Tcen2_dV_gi_si);
d_V3_dV_di_si = (-d_Tcen2_dV_di_si);
d_V3_dV_b_si = (-d_Tcen2_dV_b_si);
d_V3_dV_sb_si = (-d_Tcen2_dV_sb_si);
d_V3_dV_db_di = (-d_Tcen2_dV_db_di);
d_V3_dV_gi_p = (-d_Tcen2_dV_gi_p);
V3 = ((pParam_B4SOIldeb-Tcen2)-LINK);
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*LINK)*pParam_B4SOIldeb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_V4_dV_b_GND = (deriv_sqrt_0_d0*((V3*d_V3_dV_b_GND+d_V3_dV_b_GND*V3)));
d_V4_dV_p_GND = (deriv_sqrt_0_d0*((V3*d_V3_dV_p_GND+d_V3_dV_p_GND*V3)));
d_V4_dTemp_t_GND = (deriv_sqrt_0_d0*((V3*d_V3_dTemp_t_GND+d_V3_dTemp_t_GND*V3)));
d_V4_dV_e_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_e_si+d_V3_dV_e_si*V3)));
d_V4_dV_gi_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_gi_si+d_V3_dV_gi_si*V3)));
d_V4_dV_di_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_di_si+d_V3_dV_di_si*V3)));
d_V4_dV_b_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_b_si+d_V3_dV_b_si*V3)));
d_V4_dV_sb_si = (deriv_sqrt_0_d0*((V3*d_V3_dV_sb_si+d_V3_dV_sb_si*V3)));
d_V4_dV_db_di = (deriv_sqrt_0_d0*((V3*d_V3_dV_db_di+d_V3_dV_db_di*V3)));
d_V4_dV_gi_p = (deriv_sqrt_0_d0*((V3*d_V3_dV_gi_p+d_V3_dV_gi_p*V3)));
V4 = value_sqrt_0;
}

d_Tcen2_dV_b_GND = (-0.5*(d_V3_dV_b_GND+d_V4_dV_b_GND));
d_Tcen2_dV_p_GND = (-0.5*(d_V3_dV_p_GND+d_V4_dV_p_GND));
d_Tcen2_dTemp_t_GND = (-0.5*(d_V3_dTemp_t_GND+d_V4_dTemp_t_GND));
d_Tcen2_dV_e_si = (-0.5*(d_V3_dV_e_si+d_V4_dV_e_si));
d_Tcen2_dV_gi_si = (-0.5*(d_V3_dV_gi_si+d_V4_dV_gi_si));
d_Tcen2_dV_di_si = (-0.5*(d_V3_dV_di_si+d_V4_dV_di_si));
d_Tcen2_dV_b_si = (-0.5*(d_V3_dV_b_si+d_V4_dV_b_si));
d_Tcen2_dV_sb_si = (-0.5*(d_V3_dV_sb_si+d_V4_dV_sb_si));
d_Tcen2_dV_db_di = (-0.5*(d_V3_dV_db_di+d_V4_dV_db_di));
d_Tcen2_dV_gi_p = (-0.5*(d_V3_dV_gi_p+d_V4_dV_gi_p));
Tcen2 = (pParam_B4SOIldeb-(0.5*(V3+V4)));
if ((Tcen2<1e-15))
{

d_Tcen2_dV_gi_p = d_Tcen2_dV_db_di = d_Tcen2_dV_sb_si = d_Tcen2_dV_b_si = d_Tcen2_dV_di_si = d_Tcen2_dV_gi_si = d_Tcen2_dV_e_si = d_Tcen2_dTemp_t_GND = d_Tcen2_dV_p_GND = d_Tcen2_dV_b_GND =  0.0;
Tcen2 = 1e-15;
}
}

d_Ccen_dV_b_GND = (-epssub*d_Tcen_dV_b_GND/Tcen/Tcen);
d_Ccen_dV_p_GND = (-epssub*d_Tcen_dV_p_GND/Tcen/Tcen);
d_Ccen_dTemp_t_GND = (-epssub*d_Tcen_dTemp_t_GND/Tcen/Tcen);
d_Ccen_dV_e_si = (-epssub*d_Tcen_dV_e_si/Tcen/Tcen);
d_Ccen_dV_gi_si = (-epssub*d_Tcen_dV_gi_si/Tcen/Tcen);
d_Ccen_dV_di_si = (-epssub*d_Tcen_dV_di_si/Tcen/Tcen);
d_Ccen_dV_b_si = (-epssub*d_Tcen_dV_b_si/Tcen/Tcen);
d_Ccen_dV_sb_si = (-epssub*d_Tcen_dV_sb_si/Tcen/Tcen);
d_Ccen_dV_db_di = (-epssub*d_Tcen_dV_db_di/Tcen/Tcen);
d_Ccen_dV_gi_p = (-epssub*d_Tcen_dV_gi_p/Tcen/Tcen);
Ccen = (epssub/Tcen);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = (-Cox*d_Ccen_dV_b_GND/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_p_GND = (-Cox*d_Ccen_dV_p_GND/(Cox+Ccen)/(Cox+Ccen));
d_T2_dTemp_t_GND = (-Cox*d_Ccen_dTemp_t_GND/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_e_si = (-Cox*d_Ccen_dV_e_si/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_gi_si = (-Cox*d_Ccen_dV_gi_si/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_di_si = (-Cox*d_Ccen_dV_di_si/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_b_si = (-Cox*d_Ccen_dV_b_si/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_sb_si = (-Cox*d_Ccen_dV_sb_si/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_db_di = (-Cox*d_Ccen_dV_db_di/(Cox+Ccen)/(Cox+Ccen));
d_T2_dV_gi_p = (-Cox*d_Ccen_dV_gi_p/(Cox+Ccen)/(Cox+Ccen));
T2 = (Cox/(Cox+Ccen));

d_Coxeff_dV_b_GND = (T2*d_Ccen_dV_b_GND+d_T2_dV_b_GND*Ccen);
d_Coxeff_dV_p_GND = (T2*d_Ccen_dV_p_GND+d_T2_dV_p_GND*Ccen);
d_Coxeff_dTemp_t_GND = (T2*d_Ccen_dTemp_t_GND+d_T2_dTemp_t_GND*Ccen);
d_Coxeff_dV_gi_si = (T2*d_Ccen_dV_gi_si+d_T2_dV_gi_si*Ccen);
d_Coxeff_dV_di_si = (T2*d_Ccen_dV_di_si+d_T2_dV_di_si*Ccen);
d_Coxeff_dV_e_si = (T2*d_Ccen_dV_e_si+d_T2_dV_e_si*Ccen);
d_Coxeff_dV_b_si = (T2*d_Ccen_dV_b_si+d_T2_dV_b_si*Ccen);
d_Coxeff_dV_sb_si = (T2*d_Ccen_dV_sb_si+d_T2_dV_sb_si*Ccen);
d_Coxeff_dV_db_di = (T2*d_Ccen_dV_db_di+d_T2_dV_db_di*Ccen);
d_Coxeff_dV_gi_p = (T2*d_Ccen_dV_gi_p+d_T2_dV_gi_p*Ccen);
Coxeff = (T2*Ccen);
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_Ccen2_dV_b_GND = (-epssub*d_Tcen2_dV_b_GND/Tcen2/Tcen2);
d_Ccen2_dV_p_GND = (-epssub*d_Tcen2_dV_p_GND/Tcen2/Tcen2);
d_Ccen2_dTemp_t_GND = (-epssub*d_Tcen2_dTemp_t_GND/Tcen2/Tcen2);
d_Ccen2_dV_e_si = (-epssub*d_Tcen2_dV_e_si/Tcen2/Tcen2);
d_Ccen2_dV_gi_si = (-epssub*d_Tcen2_dV_gi_si/Tcen2/Tcen2);
d_Ccen2_dV_di_si = (-epssub*d_Tcen2_dV_di_si/Tcen2/Tcen2);
d_Ccen2_dV_b_si = (-epssub*d_Tcen2_dV_b_si/Tcen2/Tcen2);
d_Ccen2_dV_sb_si = (-epssub*d_Tcen2_dV_sb_si/Tcen2/Tcen2);
d_Ccen2_dV_db_di = (-epssub*d_Tcen2_dV_db_di/Tcen2/Tcen2);
d_Ccen2_dV_gi_p = (-epssub*d_Tcen2_dV_gi_p/Tcen2/Tcen2);
Ccen2 = (epssub/Tcen2);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = (-Cox*d_Ccen2_dV_b_GND/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_p_GND = (-Cox*d_Ccen2_dV_p_GND/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dTemp_t_GND = (-Cox*d_Ccen2_dTemp_t_GND/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_e_si = (-Cox*d_Ccen2_dV_e_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_gi_si = (-Cox*d_Ccen2_dV_gi_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_di_si = (-Cox*d_Ccen2_dV_di_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_b_si = (-Cox*d_Ccen2_dV_b_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_sb_si = (-Cox*d_Ccen2_dV_sb_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_db_di = (-Cox*d_Ccen2_dV_db_di/(Cox+Ccen2)/(Cox+Ccen2));
d_T2_dV_gi_p = (-Cox*d_Ccen2_dV_gi_p/(Cox+Ccen2)/(Cox+Ccen2));
T2 = (Cox/(Cox+Ccen2));

d_Coxeff2_dV_b_GND = (T2*d_Ccen2_dV_b_GND+d_T2_dV_b_GND*Ccen2);
d_Coxeff2_dV_p_GND = (T2*d_Ccen2_dV_p_GND+d_T2_dV_p_GND*Ccen2);
d_Coxeff2_dTemp_t_GND = (T2*d_Ccen2_dTemp_t_GND+d_T2_dTemp_t_GND*Ccen2);
d_Coxeff2_dV_gi_si = (T2*d_Ccen2_dV_gi_si+d_T2_dV_gi_si*Ccen2);
d_Coxeff2_dV_di_si = (T2*d_Ccen2_dV_di_si+d_T2_dV_di_si*Ccen2);
d_Coxeff2_dV_e_si = (T2*d_Ccen2_dV_e_si+d_T2_dV_e_si*Ccen2);
d_Coxeff2_dV_b_si = (T2*d_Ccen2_dV_b_si+d_T2_dV_b_si*Ccen2);
d_Coxeff2_dV_sb_si = (T2*d_Ccen2_dV_sb_si+d_T2_dV_sb_si*Ccen2);
d_Coxeff2_dV_db_di = (T2*d_Ccen2_dV_db_di+d_T2_dV_db_di*Ccen2);
d_Coxeff2_dV_gi_p = (T2*d_Ccen2_dV_gi_p+d_T2_dV_gi_p*Ccen2);
Coxeff2 = (T2*Ccen2);
}

d_CoxWLcenb_dV_b_GND = (CoxWLb*d_Coxeff_dV_b_GND/Cox);
d_CoxWLcenb_dV_p_GND = (CoxWLb*d_Coxeff_dV_p_GND/Cox);
d_CoxWLcenb_dTemp_t_GND = (CoxWLb*d_Coxeff_dTemp_t_GND/Cox);
d_CoxWLcenb_dV_gi_si = (CoxWLb*d_Coxeff_dV_gi_si/Cox);
d_CoxWLcenb_dV_di_si = (CoxWLb*d_Coxeff_dV_di_si/Cox);
d_CoxWLcenb_dV_e_si = (CoxWLb*d_Coxeff_dV_e_si/Cox);
d_CoxWLcenb_dV_b_si = (CoxWLb*d_Coxeff_dV_b_si/Cox);
d_CoxWLcenb_dV_sb_si = (CoxWLb*d_Coxeff_dV_sb_si/Cox);
d_CoxWLcenb_dV_db_di = (CoxWLb*d_Coxeff_dV_db_di/Cox);
d_CoxWLcenb_dV_gi_p = (CoxWLb*d_Coxeff_dV_gi_p/Cox);
CoxWLcenb = ((CoxWLb*Coxeff)/Cox);
if ((B4SOIagbcp2>0))
{

d_CoxWLcenb2_dV_b_GND = (CoxWLb2*d_Coxeff2_dV_b_GND/Cox);
d_CoxWLcenb2_dV_p_GND = (CoxWLb2*d_Coxeff2_dV_p_GND/Cox);
d_CoxWLcenb2_dTemp_t_GND = (CoxWLb2*d_Coxeff2_dTemp_t_GND/Cox);
d_CoxWLcenb2_dV_gi_si = (CoxWLb2*d_Coxeff2_dV_gi_si/Cox);
d_CoxWLcenb2_dV_di_si = (CoxWLb2*d_Coxeff2_dV_di_si/Cox);
d_CoxWLcenb2_dV_e_si = (CoxWLb2*d_Coxeff2_dV_e_si/Cox);
d_CoxWLcenb2_dV_b_si = (CoxWLb2*d_Coxeff2_dV_b_si/Cox);
d_CoxWLcenb2_dV_sb_si = (CoxWLb2*d_Coxeff2_dV_sb_si/Cox);
d_CoxWLcenb2_dV_db_di = (CoxWLb2*d_Coxeff2_dV_db_di/Cox);
d_CoxWLcenb2_dV_gi_p = (CoxWLb2*d_Coxeff2_dV_gi_p/Cox);
CoxWLcenb2 = ((CoxWLb2*Coxeff2)/Cox);
}

d_Qac0_dV_b_GND = (CoxWLcenb*(d_Vfbeff_dV_b_GND-d_Vfbzb_dV_b_GND)+d_CoxWLcenb_dV_b_GND*(Vfbeff-Vfbzb));
d_Qac0_dV_p_GND = (CoxWLcenb*(d_Vfbeff_dV_p_GND-d_Vfbzb_dV_p_GND)+d_CoxWLcenb_dV_p_GND*(Vfbeff-Vfbzb));
d_Qac0_dTemp_t_GND = (CoxWLcenb*(d_Vfbeff_dTemp_t_GND-d_Vfbzb_dTemp_t_GND)+d_CoxWLcenb_dTemp_t_GND*(Vfbeff-Vfbzb));
d_Qac0_dV_gi_si = (CoxWLcenb*(d_Vfbeff_dV_gi_si-d_Vfbzb_dV_gi_si)+d_CoxWLcenb_dV_gi_si*(Vfbeff-Vfbzb));
d_Qac0_dV_di_si = (CoxWLcenb*(d_Vfbeff_dV_di_si-d_Vfbzb_dV_di_si)+d_CoxWLcenb_dV_di_si*(Vfbeff-Vfbzb));
d_Qac0_dV_e_si = (CoxWLcenb*(d_Vfbeff_dV_e_si-d_Vfbzb_dV_e_si)+d_CoxWLcenb_dV_e_si*(Vfbeff-Vfbzb));
d_Qac0_dV_b_si = (CoxWLcenb*(d_Vfbeff_dV_b_si-d_Vfbzb_dV_b_si)+d_CoxWLcenb_dV_b_si*(Vfbeff-Vfbzb));
d_Qac0_dV_sb_si = (CoxWLcenb*d_Vfbeff_dV_sb_si+d_CoxWLcenb_dV_sb_si*(Vfbeff-Vfbzb));
d_Qac0_dV_db_di = (CoxWLcenb*d_Vfbeff_dV_db_di+d_CoxWLcenb_dV_db_di*(Vfbeff-Vfbzb));
d_Qac0_dV_gi_p = (CoxWLcenb*d_Vfbeff_dV_gi_p+d_CoxWLcenb_dV_gi_p*(Vfbeff-Vfbzb));
Qac0 = (CoxWLcenb*(Vfbeff-Vfbzb));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_Qac02_dV_b_GND = (CoxWLcenb2*(d_Vfbeff2_dV_b_GND-d_Vfbzb2_dV_b_GND)+d_CoxWLcenb2_dV_b_GND*(Vfbeff2-Vfbzb2));
d_Qac02_dV_p_GND = (CoxWLcenb2*(d_Vfbeff2_dV_p_GND-d_Vfbzb2_dV_p_GND)+d_CoxWLcenb2_dV_p_GND*(Vfbeff2-Vfbzb2));
d_Qac02_dTemp_t_GND = (CoxWLcenb2*(d_Vfbeff2_dTemp_t_GND-d_Vfbzb2_dTemp_t_GND)+d_CoxWLcenb2_dTemp_t_GND*(Vfbeff2-Vfbzb2));
d_Qac02_dV_gi_si = (CoxWLcenb2*(d_Vfbeff2_dV_gi_si-d_Vfbzb2_dV_gi_si)+d_CoxWLcenb2_dV_gi_si*(Vfbeff2-Vfbzb2));
d_Qac02_dV_di_si = (CoxWLcenb2*(d_Vfbeff2_dV_di_si-d_Vfbzb2_dV_di_si)+d_CoxWLcenb2_dV_di_si*(Vfbeff2-Vfbzb2));
d_Qac02_dV_e_si = (CoxWLcenb2*(d_Vfbeff2_dV_e_si-d_Vfbzb2_dV_e_si)+d_CoxWLcenb2_dV_e_si*(Vfbeff2-Vfbzb2));
d_Qac02_dV_b_si = (CoxWLcenb2*(d_Vfbeff2_dV_b_si-d_Vfbzb2_dV_b_si)+d_CoxWLcenb2_dV_b_si*(Vfbeff2-Vfbzb2));
d_Qac02_dV_sb_si = (CoxWLcenb2*d_Vfbeff2_dV_sb_si+d_CoxWLcenb2_dV_sb_si*(Vfbeff2-Vfbzb2));
d_Qac02_dV_db_di = (CoxWLcenb2*d_Vfbeff2_dV_db_di+d_CoxWLcenb2_dV_db_di*(Vfbeff2-Vfbzb2));
d_Qac02_dV_gi_p = (CoxWLcenb2*d_Vfbeff2_dV_gi_p+d_CoxWLcenb2_dV_gi_p*(Vfbeff2-Vfbzb2));
Qac02 = (CoxWLcenb2*(Vfbeff2-Vfbzb2));

d_Qac0_dV_gi_si = (d_Qac0_dV_gi_si+d_Qac02_dV_gi_si);
d_Qac0_dV_di_si = (d_Qac0_dV_di_si+d_Qac02_dV_di_si);
d_Qac0_dV_e_si = (d_Qac0_dV_e_si+d_Qac02_dV_e_si);
d_Qac0_dV_b_si = (d_Qac0_dV_b_si+d_Qac02_dV_b_si);
d_Qac0_dV_sb_si = (d_Qac0_dV_sb_si+d_Qac02_dV_sb_si);
d_Qac0_dV_db_di = (d_Qac0_dV_db_di+d_Qac02_dV_db_di);
d_Qac0_dV_b_GND = (d_Qac0_dV_b_GND+d_Qac02_dV_b_GND);
d_Qac0_dV_p_GND = (d_Qac0_dV_p_GND+d_Qac02_dV_p_GND);
d_Qac0_dTemp_t_GND = (d_Qac0_dTemp_t_GND+d_Qac02_dTemp_t_GND);
d_Qac0_dV_gi_p = (d_Qac0_dV_gi_p+d_Qac02_dV_gi_p);
Qac0 = (Qac0+Qac02);
}

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (0.5*pParam_B4SOIk1ox);

d_T3_dV_e_si = (((-d_Vfbeff_dV_e_si)-d_Vbseff_dV_e_si)-d_Vgsteff_dV_e_si);
d_T3_dV_b_si = (((-d_Vfbeff_dV_b_si)-d_Vbseff_dV_b_si)-d_Vgsteff_dV_b_si);
d_T3_dV_sb_si = ((-d_Vfbeff_dV_sb_si)-d_Vgsteff_dV_sb_si);
d_T3_dV_db_di = ((-d_Vfbeff_dV_db_di)-d_Vgsteff_dV_db_di);
d_T3_dV_gi_p = ((-d_Vfbeff_dV_gi_p)-d_Vgsteff_dV_gi_p);
d_T3_dV_b_GND = (((d_Vgs_eff_dV_b_GND-d_Vfbeff_dV_b_GND)-d_Vbseff_dV_b_GND)-d_Vgsteff_dV_b_GND);
d_T3_dV_p_GND = (((d_Vgs_eff_dV_p_GND-d_Vfbeff_dV_p_GND)-d_Vbseff_dV_p_GND)-d_Vgsteff_dV_p_GND);
d_T3_dTemp_t_GND = (((d_Vgs_eff_dTemp_t_GND-d_Vfbeff_dTemp_t_GND)-d_Vbseff_dTemp_t_GND)-d_Vgsteff_dTemp_t_GND);
d_T3_dV_gi_si = (((d_Vgs_eff_dV_gi_si-d_Vfbeff_dV_gi_si)-d_Vbseff_dV_gi_si)-d_Vgsteff_dV_gi_si);
d_T3_dV_di_si = (((d_Vgs_eff_dV_di_si-d_Vfbeff_dV_di_si)-d_Vbseff_dV_di_si)-d_Vgsteff_dV_di_si);
T3 = (((Vgs_eff-Vfbeff)-Vbseff)-Vgsteff);
if ((pParam_B4SOIk1ox==0.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 0.0;
}
else
{
if ((T3<0.0))
{

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND+(d_T3_dV_b_GND/pParam_B4SOIk1ox));
d_T1_dV_p_GND = (d_T0_dV_p_GND+(d_T3_dV_p_GND/pParam_B4SOIk1ox));
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND+(d_T3_dTemp_t_GND/pParam_B4SOIk1ox));
d_T1_dV_e_si = (d_T0_dV_e_si+(d_T3_dV_e_si/pParam_B4SOIk1ox));
d_T1_dV_gi_si = (d_T0_dV_gi_si+(d_T3_dV_gi_si/pParam_B4SOIk1ox));
d_T1_dV_di_si = (d_T0_dV_di_si+(d_T3_dV_di_si/pParam_B4SOIk1ox));
d_T1_dV_b_si = (d_T0_dV_b_si+(d_T3_dV_b_si/pParam_B4SOIk1ox));
d_T1_dV_sb_si = (d_T0_dV_sb_si+(d_T3_dV_sb_si/pParam_B4SOIk1ox));
d_T1_dV_db_di = (d_T0_dV_db_di+(d_T3_dV_db_di/pParam_B4SOIk1ox));
d_T1_dV_gi_p = (d_T0_dV_gi_p+(d_T3_dV_gi_p/pParam_B4SOIk1ox));
T1 = (T0+(T3/pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)+d_T3_dV_b_GND)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)+d_T3_dV_p_GND)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)+d_T3_dTemp_t_GND)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)+d_T3_dV_e_si)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)+d_T3_dV_gi_si)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)+d_T3_dV_di_si)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)+d_T3_dV_b_si)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)+d_T3_dV_sb_si)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*(((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)+d_T3_dV_db_di)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)+d_T3_dV_gi_p)));
T1 = value_sqrt_0;
}
}
}

d_Qsub0_dV_b_GND = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_b_GND-d_T0_dV_b_GND)+d_CoxWLcenb_dV_b_GND*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_p_GND = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_p_GND-d_T0_dV_p_GND)+d_CoxWLcenb_dV_p_GND*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dTemp_t_GND = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dTemp_t_GND-d_T0_dTemp_t_GND)+d_CoxWLcenb_dTemp_t_GND*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_gi_si = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_gi_si-d_T0_dV_gi_si)+d_CoxWLcenb_dV_gi_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_di_si = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_di_si-d_T0_dV_di_si)+d_CoxWLcenb_dV_di_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_e_si = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_e_si-d_T0_dV_e_si)+d_CoxWLcenb_dV_e_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_b_si = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_b_si-d_T0_dV_b_si)+d_CoxWLcenb_dV_b_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_sb_si = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_sb_si-d_T0_dV_sb_si)+d_CoxWLcenb_dV_sb_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_db_di = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_db_di-d_T0_dV_db_di)+d_CoxWLcenb_dV_db_di*pParam_B4SOIk1ox*(T1-T0));
d_Qsub0_dV_gi_p = ((CoxWLcenb*pParam_B4SOIk1ox)*(d_T1_dV_gi_p-d_T0_dV_gi_p)+d_CoxWLcenb_dV_gi_p*pParam_B4SOIk1ox*(T1-T0));
Qsub0 = ((CoxWLcenb*pParam_B4SOIk1ox)*(T1-T0));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T3_dV_sb_si = ((-d_Vfbeff2_dV_sb_si)-d_Vgsteff2_dV_sb_si);
d_T3_dV_db_di = ((-d_Vfbeff2_dV_db_di)-d_Vgsteff2_dV_db_di);
d_T3_dV_gi_p = ((-d_Vfbeff2_dV_gi_p)-d_Vgsteff2_dV_gi_p);
d_T3_dV_b_GND = (((-d_Vfbeff2_dV_b_GND)-d_Vbseff_dV_b_GND)-d_Vgsteff2_dV_b_GND);
d_T3_dV_p_GND = (((-d_Vfbeff2_dV_p_GND)-d_Vbseff_dV_p_GND)-d_Vgsteff2_dV_p_GND);
d_T3_dTemp_t_GND = (((-d_Vfbeff2_dTemp_t_GND)-d_Vbseff_dTemp_t_GND)-d_Vgsteff2_dTemp_t_GND);
d_T3_dV_e_si = (((-d_Vfbeff2_dV_e_si)-d_Vbseff_dV_e_si)-d_Vgsteff2_dV_e_si);
d_T3_dV_b_si = (((-d_Vfbeff2_dV_b_si)-d_Vbseff_dV_b_si)-d_Vgsteff2_dV_b_si);
d_T3_dV_gi_si = (((d_Vgs_eff2_dV_gi_si-d_Vfbeff2_dV_gi_si)-d_Vbseff_dV_gi_si)-d_Vgsteff2_dV_gi_si);
d_T3_dV_di_si = (((d_Vgs_eff2_dV_di_si-d_Vfbeff2_dV_di_si)-d_Vbseff_dV_di_si)-d_Vgsteff2_dV_di_si);
T3 = (((Vgs_eff2-Vfbeff2)-Vbseff)-Vgsteff2);
if ((pParam_B4SOIk1ox==0.0))
{

d_T1_dV_gm_si = d_T1_dV_gi_p = d_T1_dV_db_di = d_T1_dV_sb_si = d_T1_dV_b_si = d_T1_dV_e_si = d_T1_dV_di_si = d_T1_dV_gi_si = d_T1_dTemp_t_GND = d_T1_dV_p_GND = d_T1_dV_b_GND =  0.0;
T1 = 0.0;
}
else
{
if ((T3<0.0))
{

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (d_T0_dV_b_GND+(d_T3_dV_b_GND/pParam_B4SOIk1ox));
d_T1_dV_p_GND = (d_T0_dV_p_GND+(d_T3_dV_p_GND/pParam_B4SOIk1ox));
d_T1_dTemp_t_GND = (d_T0_dTemp_t_GND+(d_T3_dTemp_t_GND/pParam_B4SOIk1ox));
d_T1_dV_e_si = (d_T0_dV_e_si+(d_T3_dV_e_si/pParam_B4SOIk1ox));
d_T1_dV_gi_si = (d_T0_dV_gi_si+(d_T3_dV_gi_si/pParam_B4SOIk1ox));
d_T1_dV_di_si = (d_T0_dV_di_si+(d_T3_dV_di_si/pParam_B4SOIk1ox));
d_T1_dV_b_si = (d_T0_dV_b_si+(d_T3_dV_b_si/pParam_B4SOIk1ox));
d_T1_dV_sb_si = (d_T0_dV_sb_si+(d_T3_dV_sb_si/pParam_B4SOIk1ox));
d_T1_dV_db_di = (d_T0_dV_db_di+(d_T3_dV_db_di/pParam_B4SOIk1ox));
d_T1_dV_gi_p = (d_T0_dV_gi_p+(d_T3_dV_gi_p/pParam_B4SOIk1ox));
T1 = (T0+(T3/pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)+d_T3_dV_b_GND)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)+d_T3_dV_p_GND)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*(((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)+d_T3_dTemp_t_GND)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)+d_T3_dV_e_si)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)+d_T3_dV_gi_si)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)+d_T3_dV_di_si)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)+d_T3_dV_b_si)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*(((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)+d_T3_dV_sb_si)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*(((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)+d_T3_dV_db_di)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*(((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)+d_T3_dV_gi_p)));
T1 = value_sqrt_0;
}
}
}

d_Qsub02_dV_b_GND = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_b_GND-d_T0_dV_b_GND)+d_CoxWLcenb2_dV_b_GND*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_p_GND = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_p_GND-d_T0_dV_p_GND)+d_CoxWLcenb2_dV_p_GND*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dTemp_t_GND = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dTemp_t_GND-d_T0_dTemp_t_GND)+d_CoxWLcenb2_dTemp_t_GND*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_gi_si = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_gi_si-d_T0_dV_gi_si)+d_CoxWLcenb2_dV_gi_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_di_si = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_di_si-d_T0_dV_di_si)+d_CoxWLcenb2_dV_di_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_e_si = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_e_si-d_T0_dV_e_si)+d_CoxWLcenb2_dV_e_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_b_si = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_b_si-d_T0_dV_b_si)+d_CoxWLcenb2_dV_b_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_sb_si = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_sb_si-d_T0_dV_sb_si)+d_CoxWLcenb2_dV_sb_si*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_db_di = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_db_di-d_T0_dV_db_di)+d_CoxWLcenb2_dV_db_di*pParam_B4SOIk1ox*(T1-T0));
d_Qsub02_dV_gi_p = ((CoxWLcenb2*pParam_B4SOIk1ox)*(d_T1_dV_gi_p-d_T0_dV_gi_p)+d_CoxWLcenb2_dV_gi_p*pParam_B4SOIk1ox*(T1-T0));
Qsub02 = ((CoxWLcenb2*pParam_B4SOIk1ox)*(T1-T0));

d_Qsub0_dV_b_GND = (d_Qsub0_dV_b_GND+d_Qsub02_dV_b_GND);
d_Qsub0_dV_p_GND = (d_Qsub0_dV_p_GND+d_Qsub02_dV_p_GND);
d_Qsub0_dTemp_t_GND = (d_Qsub0_dTemp_t_GND+d_Qsub02_dTemp_t_GND);
d_Qsub0_dV_gi_si = (d_Qsub0_dV_gi_si+d_Qsub02_dV_gi_si);
d_Qsub0_dV_di_si = (d_Qsub0_dV_di_si+d_Qsub02_dV_di_si);
d_Qsub0_dV_e_si = (d_Qsub0_dV_e_si+d_Qsub02_dV_e_si);
d_Qsub0_dV_b_si = (d_Qsub0_dV_b_si+d_Qsub02_dV_b_si);
d_Qsub0_dV_sb_si = (d_Qsub0_dV_sb_si+d_Qsub02_dV_sb_si);
d_Qsub0_dV_db_di = (d_Qsub0_dV_db_di+d_Qsub02_dV_db_di);
d_Qsub0_dV_gi_p = (d_Qsub0_dV_gi_p+d_Qsub02_dV_gi_p);
Qsub0 = (Qsub0+Qsub02);
}
}
if ((pParam_B4SOIk1ox<=0.0))
{

d_Denomi_dV_b_si = d_Denomi_dV_e_si = d_Denomi_dV_di_si = d_Denomi_dV_gi_si =  0.0;
d_Denomi_dV_b_GND = (0.25*pParam_B4SOImoin)*d_Vtm_dV_b_GND;
d_Denomi_dV_p_GND = (0.25*pParam_B4SOImoin)*d_Vtm_dV_p_GND;
d_Denomi_dTemp_t_GND = (0.25*pParam_B4SOImoin)*d_Vtm_dTemp_t_GND;
Denomi = ((0.25*pParam_B4SOImoin)*Vtm);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (0.5*pParam_B4SOIsqrtPhi);
}
else
{

d_Denomi_dV_b_si = d_Denomi_dV_e_si = d_Denomi_dV_di_si = d_Denomi_dV_gi_si =  0.0;
d_Denomi_dV_b_GND = pParam_B4SOImoin*d_Vtm_dV_b_GND*pParam_B4SOIk1ox*pParam_B4SOIk1ox;
d_Denomi_dV_p_GND = pParam_B4SOImoin*d_Vtm_dV_p_GND*pParam_B4SOIk1ox*pParam_B4SOIk1ox;
d_Denomi_dTemp_t_GND = pParam_B4SOImoin*d_Vtm_dTemp_t_GND*pParam_B4SOIk1ox*pParam_B4SOIk1ox;
Denomi = (((pParam_B4SOImoin*Vtm)*pParam_B4SOIk1ox)*pParam_B4SOIk1ox);

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
T0 = (pParam_B4SOIk1ox*pParam_B4SOIsqrtPhi);
}

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (2.0*d_T0_dV_b_GND+d_Vgsteff_dV_b_GND);
d_T1_dV_p_GND = (2.0*d_T0_dV_p_GND+d_Vgsteff_dV_p_GND);
d_T1_dTemp_t_GND = (2.0*d_T0_dTemp_t_GND+d_Vgsteff_dTemp_t_GND);
d_T1_dV_e_si = (2.0*d_T0_dV_e_si+d_Vgsteff_dV_e_si);
d_T1_dV_gi_si = (2.0*d_T0_dV_gi_si+d_Vgsteff_dV_gi_si);
d_T1_dV_di_si = (2.0*d_T0_dV_di_si+d_Vgsteff_dV_di_si);
d_T1_dV_b_si = (2.0*d_T0_dV_b_si+d_Vgsteff_dV_b_si);
d_T1_dV_sb_si = (2.0*d_T0_dV_sb_si+d_Vgsteff_dV_sb_si);
d_T1_dV_db_di = (2.0*d_T0_dV_db_di+d_Vgsteff_dV_db_di);
d_T1_dV_gi_p = (2.0*d_T0_dV_gi_p+d_Vgsteff_dV_gi_p);
T1 = ((2.0*T0)+Vgsteff);
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+((T1*Vgsteff)/Denomi)));

d_DeltaPhi_dV_gi_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dV_gi_si+d_T1_dV_gi_si*Vgsteff)-(T1*Vgsteff)*d_Denomi_dV_gi_si)/Denomi/Denomi));
d_DeltaPhi_dV_di_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dV_di_si+d_T1_dV_di_si*Vgsteff)-(T1*Vgsteff)*d_Denomi_dV_di_si)/Denomi/Denomi));
d_DeltaPhi_dV_e_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dV_e_si+d_T1_dV_e_si*Vgsteff)-(T1*Vgsteff)*d_Denomi_dV_e_si)/Denomi/Denomi));
d_DeltaPhi_dV_b_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dV_b_si+d_T1_dV_b_si*Vgsteff)-(T1*Vgsteff)*d_Denomi_dV_b_si)/Denomi/Denomi));
d_DeltaPhi_dV_sb_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((T1*d_Vgsteff_dV_sb_si+d_T1_dV_sb_si*Vgsteff)/Denomi));
d_DeltaPhi_dV_db_di = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((T1*d_Vgsteff_dV_db_di+d_T1_dV_db_di*Vgsteff)/Denomi));
d_DeltaPhi_dV_gi_p = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((T1*d_Vgsteff_dV_gi_p+d_T1_dV_gi_p*Vgsteff)/Denomi));
d_DeltaPhi_dV_b_GND = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dV_b_GND+d_T1_dV_b_GND*Vgsteff)-(T1*Vgsteff)*d_Denomi_dV_b_GND)/Denomi/Denomi))+d_Vtm_dV_b_GND*evaluator_lln_0.getValues((1.0+((T1*Vgsteff)/Denomi))));
d_DeltaPhi_dV_p_GND = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dV_p_GND+d_T1_dV_p_GND*Vgsteff)-(T1*Vgsteff)*d_Denomi_dV_p_GND)/Denomi/Denomi))+d_Vtm_dV_p_GND*evaluator_lln_0.getValues((1.0+((T1*Vgsteff)/Denomi))));
d_DeltaPhi_dTemp_t_GND = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dTemp_t_GND+d_T1_dTemp_t_GND*Vgsteff)-(T1*Vgsteff)*d_Denomi_dTemp_t_GND)/Denomi/Denomi))+d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues((1.0+((T1*Vgsteff)/Denomi))));
DeltaPhi = (Vtm*evaluator_lln_0.getValues((1.0+((T1*Vgsteff)/Denomi))));
}
if ((B4SOIagbcp2>0))
{

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (2.0*d_T0_dV_b_GND+d_Vgsteff2_dV_b_GND);
d_T1_dV_p_GND = (2.0*d_T0_dV_p_GND+d_Vgsteff2_dV_p_GND);
d_T1_dTemp_t_GND = (2.0*d_T0_dTemp_t_GND+d_Vgsteff2_dTemp_t_GND);
d_T1_dV_e_si = (2.0*d_T0_dV_e_si+d_Vgsteff2_dV_e_si);
d_T1_dV_gi_si = (2.0*d_T0_dV_gi_si+d_Vgsteff2_dV_gi_si);
d_T1_dV_di_si = (2.0*d_T0_dV_di_si+d_Vgsteff2_dV_di_si);
d_T1_dV_b_si = (2.0*d_T0_dV_b_si+d_Vgsteff2_dV_b_si);
d_T1_dV_sb_si = (2.0*d_T0_dV_sb_si+d_Vgsteff2_dV_sb_si);
d_T1_dV_db_di = (2.0*d_T0_dV_db_di+d_Vgsteff2_dV_db_di);
d_T1_dV_gi_p = (2.0*d_T0_dV_gi_p+d_Vgsteff2_dV_gi_p);
T1 = ((2.0*T0)+Vgsteff2);
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+((T1*Vgsteff2)/Denomi)));

d_DeltaPhi2_dV_gi_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dV_gi_si+d_T1_dV_gi_si*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dV_gi_si)/Denomi/Denomi));
d_DeltaPhi2_dV_di_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dV_di_si+d_T1_dV_di_si*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dV_di_si)/Denomi/Denomi));
d_DeltaPhi2_dV_e_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dV_e_si+d_T1_dV_e_si*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dV_e_si)/Denomi/Denomi));
d_DeltaPhi2_dV_b_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dV_b_si+d_T1_dV_b_si*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dV_b_si)/Denomi/Denomi));
d_DeltaPhi2_dV_sb_si = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((T1*d_Vgsteff2_dV_sb_si+d_T1_dV_sb_si*Vgsteff2)/Denomi));
d_DeltaPhi2_dV_db_di = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((T1*d_Vgsteff2_dV_db_di+d_T1_dV_db_di*Vgsteff2)/Denomi));
d_DeltaPhi2_dV_gi_p = Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((T1*d_Vgsteff2_dV_gi_p+d_T1_dV_gi_p*Vgsteff2)/Denomi));
d_DeltaPhi2_dV_b_GND = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dV_b_GND+d_T1_dV_b_GND*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dV_b_GND)/Denomi/Denomi))+d_Vtm_dV_b_GND*evaluator_lln_0.getValues((1.0+((T1*Vgsteff2)/Denomi))));
d_DeltaPhi2_dV_p_GND = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dV_p_GND+d_T1_dV_p_GND*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dV_p_GND)/Denomi/Denomi))+d_Vtm_dV_p_GND*evaluator_lln_0.getValues((1.0+((T1*Vgsteff2)/Denomi))));
d_DeltaPhi2_dTemp_t_GND = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dTemp_t_GND+d_T1_dTemp_t_GND*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dTemp_t_GND)/Denomi/Denomi))+d_Vtm_dTemp_t_GND*evaluator_lln_0.getValues((1.0+((T1*Vgsteff2)/Denomi))));
DeltaPhi2 = (Vtm*evaluator_lln_0.getValues((1.0+((T1*Vgsteff2)/Denomi))));
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_e_si = 4.0*(d_Vth_dV_e_si-d_Vfbzb_dV_e_si);
d_T3_dV_gi_si = 4.0*(d_Vth_dV_gi_si-d_Vfbzb_dV_gi_si);
d_T3_dV_di_si = 4.0*(d_Vth_dV_di_si-d_Vfbzb_dV_di_si);
d_T3_dV_b_si = 4.0*(d_Vth_dV_b_si-d_Vfbzb_dV_b_si);
d_T3_dV_b_GND = 4.0*((d_Vth_dV_b_GND-d_Vfbzb_dV_b_GND)-d_phi_dV_b_GND);
d_T3_dV_p_GND = 4.0*((d_Vth_dV_p_GND-d_Vfbzb_dV_p_GND)-d_phi_dV_p_GND);
d_T3_dTemp_t_GND = 4.0*((d_Vth_dTemp_t_GND-d_Vfbzb_dTemp_t_GND)-d_phi_dTemp_t_GND);
T3 = (4.0*((Vth-Vfbzb)-phi));
{
double value_sqrt_0 = sqrt(((T3*T3)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T3*d_T3_dV_b_GND+d_T3_dV_b_GND*T3)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T3*d_T3_dV_p_GND+d_T3_dV_p_GND*T3)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T3*d_T3_dTemp_t_GND+d_T3_dTemp_t_GND*T3)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_gi_si+d_T3_dV_gi_si*T3)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_di_si+d_T3_dV_di_si*T3)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_e_si+d_T3_dV_e_si*T3)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_b_si+d_T3_dV_b_si*T3)));
d_T2_dV_sb_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_sb_si+d_T3_dV_sb_si*T3)));
d_T2_dV_db_di = (deriv_sqrt_0_d0*((T3*d_T3_dV_db_di+d_T3_dV_db_di*T3)));
d_T2_dV_gi_p = (deriv_sqrt_0_d0*((T3*d_T3_dV_gi_p+d_T3_dV_gi_p*T3)));
T2 = value_sqrt_0;
}

d_T4_dV_gm_si =  0.0;
d_T4_dV_b_GND = 0.5*(d_T3_dV_b_GND+d_T2_dV_b_GND);
d_T4_dV_p_GND = 0.5*(d_T3_dV_p_GND+d_T2_dV_p_GND);
d_T4_dTemp_t_GND = 0.5*(d_T3_dTemp_t_GND+d_T2_dTemp_t_GND);
d_T4_dV_gi_si = 0.5*(d_T3_dV_gi_si+d_T2_dV_gi_si);
d_T4_dV_di_si = 0.5*(d_T3_dV_di_si+d_T2_dV_di_si);
d_T4_dV_e_si = 0.5*(d_T3_dV_e_si+d_T2_dV_e_si);
d_T4_dV_b_si = 0.5*(d_T3_dV_b_si+d_T2_dV_b_si);
d_T4_dV_sb_si = 0.5*(d_T3_dV_sb_si+d_T2_dV_sb_si);
d_T4_dV_db_di = 0.5*(d_T3_dV_db_di+d_T2_dV_db_di);
d_T4_dV_gi_p = 0.5*(d_T3_dV_gi_p+d_T2_dV_gi_p);
T4 = (0.5*(T3+T2));
Tox = (Tox+Tox);

d_T0_dV_gm_si =  0.0;
d_T0_dV_e_si = ((d_Vgsteff_dV_e_si+d_T4_dV_e_si)/Tox);
d_T0_dV_b_si = ((d_Vgsteff_dV_b_si+d_T4_dV_b_si)/Tox);
d_T0_dV_b_GND = ((d_Vgsteff_dV_b_GND+d_T4_dV_b_GND)/Tox);
d_T0_dV_p_GND = ((d_Vgsteff_dV_p_GND+d_T4_dV_p_GND)/Tox);
d_T0_dTemp_t_GND = ((d_Vgsteff_dTemp_t_GND+d_T4_dTemp_t_GND)/Tox);
d_T0_dV_gi_si = ((d_Vgsteff_dV_gi_si+d_T4_dV_gi_si)/Tox);
d_T0_dV_di_si = ((d_Vgsteff_dV_di_si+d_T4_dV_di_si)/Tox);
d_T0_dV_sb_si = ((d_Vgsteff_dV_sb_si+d_T4_dV_sb_si)/Tox);
d_T0_dV_db_di = ((d_Vgsteff_dV_db_di+d_T4_dV_db_di)/Tox);
d_T0_dV_gi_p = ((d_Vgsteff_dV_gi_p+d_T4_dV_gi_p)/Tox);
T0 = ((Vgsteff+T4)/Tox);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp(((B4SOIbdos*0.7)*evaluator_lln_0.getValues(T0)));
double  deriv_exp_1_d0 = value_exp_1;

d_tmp_dV_b_GND = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_GND)));
d_tmp_dV_p_GND = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_p_GND)));
d_tmp_dTemp_t_GND = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dTemp_t_GND)));
d_tmp_dV_e_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_e_si)));
d_tmp_dV_gi_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_gi_si)));
d_tmp_dV_di_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_di_si)));
d_tmp_dV_b_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_si)));
d_tmp_dV_sb_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_sb_si)));
d_tmp_dV_db_di = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_db_di)));
d_tmp_dV_gi_p = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_gi_p)));
tmp = value_exp_1;
}

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = d_tmp_dV_b_GND;
d_T1_dV_p_GND = d_tmp_dV_p_GND;
d_T1_dTemp_t_GND = d_tmp_dTemp_t_GND;
d_T1_dV_e_si = d_tmp_dV_e_si;
d_T1_dV_gi_si = d_tmp_dV_gi_si;
d_T1_dV_di_si = d_tmp_dV_di_si;
d_T1_dV_b_si = d_tmp_dV_b_si;
d_T1_dV_sb_si = d_tmp_dV_sb_si;
d_T1_dV_db_di = d_tmp_dV_db_di;
d_T1_dV_gi_p = d_tmp_dV_gi_p;
T1 = (1.0+tmp);

d_Tcen_dV_b_GND = (-(B4SOIados*1.9e-9)*d_T1_dV_b_GND/T1/T1);
d_Tcen_dV_p_GND = (-(B4SOIados*1.9e-9)*d_T1_dV_p_GND/T1/T1);
d_Tcen_dTemp_t_GND = (-(B4SOIados*1.9e-9)*d_T1_dTemp_t_GND/T1/T1);
d_Tcen_dV_gi_si = (-(B4SOIados*1.9e-9)*d_T1_dV_gi_si/T1/T1);
d_Tcen_dV_di_si = (-(B4SOIados*1.9e-9)*d_T1_dV_di_si/T1/T1);
d_Tcen_dV_e_si = (-(B4SOIados*1.9e-9)*d_T1_dV_e_si/T1/T1);
d_Tcen_dV_b_si = (-(B4SOIados*1.9e-9)*d_T1_dV_b_si/T1/T1);
d_Tcen_dV_sb_si = (-(B4SOIados*1.9e-9)*d_T1_dV_sb_si/T1/T1);
d_Tcen_dV_db_di = (-(B4SOIados*1.9e-9)*d_T1_dV_db_di/T1/T1);
d_Tcen_dV_gi_p = (-(B4SOIados*1.9e-9)*d_T1_dV_gi_p/T1/T1);
Tcen = ((B4SOIados*1.9e-9)/T1);

d_Ccen_dV_b_GND = (-epssub*d_Tcen_dV_b_GND/Tcen/Tcen);
d_Ccen_dV_p_GND = (-epssub*d_Tcen_dV_p_GND/Tcen/Tcen);
d_Ccen_dTemp_t_GND = (-epssub*d_Tcen_dTemp_t_GND/Tcen/Tcen);
d_Ccen_dV_e_si = (-epssub*d_Tcen_dV_e_si/Tcen/Tcen);
d_Ccen_dV_gi_si = (-epssub*d_Tcen_dV_gi_si/Tcen/Tcen);
d_Ccen_dV_di_si = (-epssub*d_Tcen_dV_di_si/Tcen/Tcen);
d_Ccen_dV_b_si = (-epssub*d_Tcen_dV_b_si/Tcen/Tcen);
d_Ccen_dV_sb_si = (-epssub*d_Tcen_dV_sb_si/Tcen/Tcen);
d_Ccen_dV_db_di = (-epssub*d_Tcen_dV_db_di/Tcen/Tcen);
d_Ccen_dV_gi_p = (-epssub*d_Tcen_dV_gi_p/Tcen/Tcen);
Ccen = (epssub/Tcen);

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = (-Cox*d_Ccen_dV_b_GND/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_p_GND = (-Cox*d_Ccen_dV_p_GND/(Cox+Ccen)/(Cox+Ccen));
d_T0_dTemp_t_GND = (-Cox*d_Ccen_dTemp_t_GND/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_e_si = (-Cox*d_Ccen_dV_e_si/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_gi_si = (-Cox*d_Ccen_dV_gi_si/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_di_si = (-Cox*d_Ccen_dV_di_si/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_b_si = (-Cox*d_Ccen_dV_b_si/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_sb_si = (-Cox*d_Ccen_dV_sb_si/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_db_di = (-Cox*d_Ccen_dV_db_di/(Cox+Ccen)/(Cox+Ccen));
d_T0_dV_gi_p = (-Cox*d_Ccen_dV_gi_p/(Cox+Ccen)/(Cox+Ccen));
T0 = (Cox/(Cox+Ccen));

d_Coxeff_dV_b_GND = (T0*d_Ccen_dV_b_GND+d_T0_dV_b_GND*Ccen);
d_Coxeff_dV_p_GND = (T0*d_Ccen_dV_p_GND+d_T0_dV_p_GND*Ccen);
d_Coxeff_dTemp_t_GND = (T0*d_Ccen_dTemp_t_GND+d_T0_dTemp_t_GND*Ccen);
d_Coxeff_dV_e_si = (T0*d_Ccen_dV_e_si+d_T0_dV_e_si*Ccen);
d_Coxeff_dV_gi_si = (T0*d_Ccen_dV_gi_si+d_T0_dV_gi_si*Ccen);
d_Coxeff_dV_di_si = (T0*d_Ccen_dV_di_si+d_T0_dV_di_si*Ccen);
d_Coxeff_dV_b_si = (T0*d_Ccen_dV_b_si+d_T0_dV_b_si*Ccen);
d_Coxeff_dV_sb_si = (T0*d_Ccen_dV_sb_si+d_T0_dV_sb_si*Ccen);
d_Coxeff_dV_db_di = (T0*d_Ccen_dV_db_di+d_T0_dV_db_di*Ccen);
d_Coxeff_dV_gi_p = (T0*d_Ccen_dV_gi_p+d_T0_dV_gi_p*Ccen);
Coxeff = (T0*Ccen);

d_CoxWLcen_dV_b_GND = (CoxWL*d_Coxeff_dV_b_GND/Cox);
d_CoxWLcen_dV_p_GND = (CoxWL*d_Coxeff_dV_p_GND/Cox);
d_CoxWLcen_dTemp_t_GND = (CoxWL*d_Coxeff_dTemp_t_GND/Cox);
d_CoxWLcen_dV_gi_si = (CoxWL*d_Coxeff_dV_gi_si/Cox);
d_CoxWLcen_dV_di_si = (CoxWL*d_Coxeff_dV_di_si/Cox);
d_CoxWLcen_dV_e_si = (CoxWL*d_Coxeff_dV_e_si/Cox);
d_CoxWLcen_dV_b_si = (CoxWL*d_Coxeff_dV_b_si/Cox);
d_CoxWLcen_dV_sb_si = (CoxWL*d_Coxeff_dV_sb_si/Cox);
d_CoxWLcen_dV_db_di = (CoxWL*d_Coxeff_dV_db_di/Cox);
d_CoxWLcen_dV_gi_p = (CoxWL*d_Coxeff_dV_gi_p/Cox);
CoxWLcen = ((CoxWL*Coxeff)/Cox);

d_CoxWLcenb_dV_b_GND = (CoxWLb*d_Coxeff_dV_b_GND/Cox);
d_CoxWLcenb_dV_p_GND = (CoxWLb*d_Coxeff_dV_p_GND/Cox);
d_CoxWLcenb_dTemp_t_GND = (CoxWLb*d_Coxeff_dTemp_t_GND/Cox);
d_CoxWLcenb_dV_gi_si = (CoxWLb*d_Coxeff_dV_gi_si/Cox);
d_CoxWLcenb_dV_di_si = (CoxWLb*d_Coxeff_dV_di_si/Cox);
d_CoxWLcenb_dV_e_si = (CoxWLb*d_Coxeff_dV_e_si/Cox);
d_CoxWLcenb_dV_b_si = (CoxWLb*d_Coxeff_dV_b_si/Cox);
d_CoxWLcenb_dV_sb_si = (CoxWLb*d_Coxeff_dV_sb_si/Cox);
d_CoxWLcenb_dV_db_di = (CoxWLb*d_Coxeff_dV_db_di/Cox);
d_CoxWLcenb_dV_gi_p = (CoxWLb*d_Coxeff_dV_gi_p/Cox);
CoxWLcenb = ((CoxWLb*Coxeff)/Cox);
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si =  0.0;
d_T3_dV_e_si = 4.0*(d_Vth_dV_e_si-d_Vfbzb2_dV_e_si);
d_T3_dV_gi_si = 4.0*(d_Vth_dV_gi_si-d_Vfbzb2_dV_gi_si);
d_T3_dV_di_si = 4.0*(d_Vth_dV_di_si-d_Vfbzb2_dV_di_si);
d_T3_dV_b_si = 4.0*(d_Vth_dV_b_si-d_Vfbzb2_dV_b_si);
d_T3_dV_b_GND = 4.0*((d_Vth_dV_b_GND-d_Vfbzb2_dV_b_GND)-d_phi_dV_b_GND);
d_T3_dV_p_GND = 4.0*((d_Vth_dV_p_GND-d_Vfbzb2_dV_p_GND)-d_phi_dV_p_GND);
d_T3_dTemp_t_GND = 4.0*((d_Vth_dTemp_t_GND-d_Vfbzb2_dTemp_t_GND)-d_phi_dTemp_t_GND);
T3 = (4.0*(((Vth+eggbcp2)-Vfbzb2)-phi));
{
double value_sqrt_0 = sqrt(((T3*T3)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = (deriv_sqrt_0_d0*((T3*d_T3_dV_b_GND+d_T3_dV_b_GND*T3)));
d_T2_dV_p_GND = (deriv_sqrt_0_d0*((T3*d_T3_dV_p_GND+d_T3_dV_p_GND*T3)));
d_T2_dTemp_t_GND = (deriv_sqrt_0_d0*((T3*d_T3_dTemp_t_GND+d_T3_dTemp_t_GND*T3)));
d_T2_dV_gi_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_gi_si+d_T3_dV_gi_si*T3)));
d_T2_dV_di_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_di_si+d_T3_dV_di_si*T3)));
d_T2_dV_e_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_e_si+d_T3_dV_e_si*T3)));
d_T2_dV_b_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_b_si+d_T3_dV_b_si*T3)));
d_T2_dV_sb_si = (deriv_sqrt_0_d0*((T3*d_T3_dV_sb_si+d_T3_dV_sb_si*T3)));
d_T2_dV_db_di = (deriv_sqrt_0_d0*((T3*d_T3_dV_db_di+d_T3_dV_db_di*T3)));
d_T2_dV_gi_p = (deriv_sqrt_0_d0*((T3*d_T3_dV_gi_p+d_T3_dV_gi_p*T3)));
T2 = value_sqrt_0;
}

d_T4_dV_gm_si =  0.0;
d_T4_dV_b_GND = 0.5*(d_T3_dV_b_GND+d_T2_dV_b_GND);
d_T4_dV_p_GND = 0.5*(d_T3_dV_p_GND+d_T2_dV_p_GND);
d_T4_dTemp_t_GND = 0.5*(d_T3_dTemp_t_GND+d_T2_dTemp_t_GND);
d_T4_dV_gi_si = 0.5*(d_T3_dV_gi_si+d_T2_dV_gi_si);
d_T4_dV_di_si = 0.5*(d_T3_dV_di_si+d_T2_dV_di_si);
d_T4_dV_e_si = 0.5*(d_T3_dV_e_si+d_T2_dV_e_si);
d_T4_dV_b_si = 0.5*(d_T3_dV_b_si+d_T2_dV_b_si);
d_T4_dV_sb_si = 0.5*(d_T3_dV_sb_si+d_T2_dV_sb_si);
d_T4_dV_db_di = 0.5*(d_T3_dV_db_di+d_T2_dV_db_di);
d_T4_dV_gi_p = 0.5*(d_T3_dV_gi_p+d_T2_dV_gi_p);
T4 = (0.5*(T3+T2));

d_T0_dV_gm_si =  0.0;
d_T0_dV_sb_si = ((d_Vgsteff2_dV_sb_si+d_T4_dV_sb_si)/Tox);
d_T0_dV_db_di = ((d_Vgsteff2_dV_db_di+d_T4_dV_db_di)/Tox);
d_T0_dV_b_GND = ((d_Vgsteff2_dV_b_GND+d_T4_dV_b_GND)/Tox);
d_T0_dV_p_GND = ((d_Vgsteff2_dV_p_GND+d_T4_dV_p_GND)/Tox);
d_T0_dTemp_t_GND = ((d_Vgsteff2_dTemp_t_GND+d_T4_dTemp_t_GND)/Tox);
d_T0_dV_gi_si = ((d_Vgsteff2_dV_gi_si+d_T4_dV_gi_si)/Tox);
d_T0_dV_di_si = ((d_Vgsteff2_dV_di_si+d_T4_dV_di_si)/Tox);
d_T0_dV_e_si = ((d_Vgsteff2_dV_e_si+d_T4_dV_e_si)/Tox);
d_T0_dV_b_si = ((d_Vgsteff2_dV_b_si+d_T4_dV_b_si)/Tox);
d_T0_dV_gi_p = ((d_Vgsteff2_dV_gi_p+d_T4_dV_gi_p)/Tox);
T0 = ((Vgsteff2+T4)/Tox);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp(((B4SOIbdos*0.7)*evaluator_lln_0.getValues(T0)));
double  deriv_exp_1_d0 = value_exp_1;

d_tmp_dV_b_GND = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_GND)));
d_tmp_dV_p_GND = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_p_GND)));
d_tmp_dTemp_t_GND = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dTemp_t_GND)));
d_tmp_dV_e_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_e_si)));
d_tmp_dV_gi_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_gi_si)));
d_tmp_dV_di_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_di_si)));
d_tmp_dV_b_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_b_si)));
d_tmp_dV_sb_si = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_sb_si)));
d_tmp_dV_db_di = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_db_di)));
d_tmp_dV_gi_p = (deriv_exp_1_d0*((B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dV_gi_p)));
tmp = value_exp_1;
}

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = d_tmp_dV_b_GND;
d_T1_dV_p_GND = d_tmp_dV_p_GND;
d_T1_dTemp_t_GND = d_tmp_dTemp_t_GND;
d_T1_dV_e_si = d_tmp_dV_e_si;
d_T1_dV_gi_si = d_tmp_dV_gi_si;
d_T1_dV_di_si = d_tmp_dV_di_si;
d_T1_dV_b_si = d_tmp_dV_b_si;
d_T1_dV_sb_si = d_tmp_dV_sb_si;
d_T1_dV_db_di = d_tmp_dV_db_di;
d_T1_dV_gi_p = d_tmp_dV_gi_p;
T1 = (1.0+tmp);

d_Tcen2_dV_b_GND = (-(B4SOIados*1.9e-9)*d_T1_dV_b_GND/T1/T1);
d_Tcen2_dV_p_GND = (-(B4SOIados*1.9e-9)*d_T1_dV_p_GND/T1/T1);
d_Tcen2_dTemp_t_GND = (-(B4SOIados*1.9e-9)*d_T1_dTemp_t_GND/T1/T1);
d_Tcen2_dV_gi_si = (-(B4SOIados*1.9e-9)*d_T1_dV_gi_si/T1/T1);
d_Tcen2_dV_di_si = (-(B4SOIados*1.9e-9)*d_T1_dV_di_si/T1/T1);
d_Tcen2_dV_e_si = (-(B4SOIados*1.9e-9)*d_T1_dV_e_si/T1/T1);
d_Tcen2_dV_b_si = (-(B4SOIados*1.9e-9)*d_T1_dV_b_si/T1/T1);
d_Tcen2_dV_sb_si = (-(B4SOIados*1.9e-9)*d_T1_dV_sb_si/T1/T1);
d_Tcen2_dV_db_di = (-(B4SOIados*1.9e-9)*d_T1_dV_db_di/T1/T1);
d_Tcen2_dV_gi_p = (-(B4SOIados*1.9e-9)*d_T1_dV_gi_p/T1/T1);
Tcen2 = ((B4SOIados*1.9e-9)/T1);

d_Ccen2_dV_b_GND = (-epssub*d_Tcen2_dV_b_GND/Tcen2/Tcen2);
d_Ccen2_dV_p_GND = (-epssub*d_Tcen2_dV_p_GND/Tcen2/Tcen2);
d_Ccen2_dTemp_t_GND = (-epssub*d_Tcen2_dTemp_t_GND/Tcen2/Tcen2);
d_Ccen2_dV_e_si = (-epssub*d_Tcen2_dV_e_si/Tcen2/Tcen2);
d_Ccen2_dV_gi_si = (-epssub*d_Tcen2_dV_gi_si/Tcen2/Tcen2);
d_Ccen2_dV_di_si = (-epssub*d_Tcen2_dV_di_si/Tcen2/Tcen2);
d_Ccen2_dV_b_si = (-epssub*d_Tcen2_dV_b_si/Tcen2/Tcen2);
d_Ccen2_dV_sb_si = (-epssub*d_Tcen2_dV_sb_si/Tcen2/Tcen2);
d_Ccen2_dV_db_di = (-epssub*d_Tcen2_dV_db_di/Tcen2/Tcen2);
d_Ccen2_dV_gi_p = (-epssub*d_Tcen2_dV_gi_p/Tcen2/Tcen2);
Ccen2 = (epssub/Tcen2);

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = (-Cox*d_Ccen2_dV_b_GND/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_p_GND = (-Cox*d_Ccen2_dV_p_GND/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dTemp_t_GND = (-Cox*d_Ccen2_dTemp_t_GND/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_e_si = (-Cox*d_Ccen2_dV_e_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_gi_si = (-Cox*d_Ccen2_dV_gi_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_di_si = (-Cox*d_Ccen2_dV_di_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_b_si = (-Cox*d_Ccen2_dV_b_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_sb_si = (-Cox*d_Ccen2_dV_sb_si/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_db_di = (-Cox*d_Ccen2_dV_db_di/(Cox+Ccen2)/(Cox+Ccen2));
d_T0_dV_gi_p = (-Cox*d_Ccen2_dV_gi_p/(Cox+Ccen2)/(Cox+Ccen2));
T0 = (Cox/(Cox+Ccen2));

d_Coxeff2_dV_b_GND = (T0*d_Ccen2_dV_b_GND+d_T0_dV_b_GND*Ccen2);
d_Coxeff2_dV_p_GND = (T0*d_Ccen2_dV_p_GND+d_T0_dV_p_GND*Ccen2);
d_Coxeff2_dTemp_t_GND = (T0*d_Ccen2_dTemp_t_GND+d_T0_dTemp_t_GND*Ccen2);
d_Coxeff2_dV_e_si = (T0*d_Ccen2_dV_e_si+d_T0_dV_e_si*Ccen2);
d_Coxeff2_dV_gi_si = (T0*d_Ccen2_dV_gi_si+d_T0_dV_gi_si*Ccen2);
d_Coxeff2_dV_di_si = (T0*d_Ccen2_dV_di_si+d_T0_dV_di_si*Ccen2);
d_Coxeff2_dV_b_si = (T0*d_Ccen2_dV_b_si+d_T0_dV_b_si*Ccen2);
d_Coxeff2_dV_sb_si = (T0*d_Ccen2_dV_sb_si+d_T0_dV_sb_si*Ccen2);
d_Coxeff2_dV_db_di = (T0*d_Ccen2_dV_db_di+d_T0_dV_db_di*Ccen2);
d_Coxeff2_dV_gi_p = (T0*d_Ccen2_dV_gi_p+d_T0_dV_gi_p*Ccen2);
Coxeff2 = (T0*Ccen2);

d_CoxWLcen2_dV_b_GND = (CoxWL2*d_Coxeff2_dV_b_GND/Cox);
d_CoxWLcen2_dV_p_GND = (CoxWL2*d_Coxeff2_dV_p_GND/Cox);
d_CoxWLcen2_dTemp_t_GND = (CoxWL2*d_Coxeff2_dTemp_t_GND/Cox);
d_CoxWLcen2_dV_gi_si = (CoxWL2*d_Coxeff2_dV_gi_si/Cox);
d_CoxWLcen2_dV_di_si = (CoxWL2*d_Coxeff2_dV_di_si/Cox);
d_CoxWLcen2_dV_e_si = (CoxWL2*d_Coxeff2_dV_e_si/Cox);
d_CoxWLcen2_dV_b_si = (CoxWL2*d_Coxeff2_dV_b_si/Cox);
d_CoxWLcen2_dV_sb_si = (CoxWL2*d_Coxeff2_dV_sb_si/Cox);
d_CoxWLcen2_dV_db_di = (CoxWL2*d_Coxeff2_dV_db_di/Cox);
d_CoxWLcen2_dV_gi_p = (CoxWL2*d_Coxeff2_dV_gi_p/Cox);
CoxWLcen2 = ((CoxWL2*Coxeff2)/Cox);

d_CoxWLcenb2_dV_b_GND = (CoxWLb2*d_Coxeff2_dV_b_GND/Cox);
d_CoxWLcenb2_dV_p_GND = (CoxWLb2*d_Coxeff2_dV_p_GND/Cox);
d_CoxWLcenb2_dTemp_t_GND = (CoxWLb2*d_Coxeff2_dTemp_t_GND/Cox);
d_CoxWLcenb2_dV_gi_si = (CoxWLb2*d_Coxeff2_dV_gi_si/Cox);
d_CoxWLcenb2_dV_di_si = (CoxWLb2*d_Coxeff2_dV_di_si/Cox);
d_CoxWLcenb2_dV_e_si = (CoxWLb2*d_Coxeff2_dV_e_si/Cox);
d_CoxWLcenb2_dV_b_si = (CoxWLb2*d_Coxeff2_dV_b_si/Cox);
d_CoxWLcenb2_dV_sb_si = (CoxWLb2*d_Coxeff2_dV_sb_si/Cox);
d_CoxWLcenb2_dV_db_di = (CoxWLb2*d_Coxeff2_dV_db_di/Cox);
d_CoxWLcenb2_dV_gi_p = (CoxWLb2*d_Coxeff2_dV_gi_p/Cox);
CoxWLcenb2 = ((CoxWLb2*Coxeff2)/Cox);
}

d_T1_dV_gm_si =  0.0;
d_T1_dV_e_si = (d_Vgsteff_dV_e_si-d_DeltaPhi_dV_e_si);
d_T1_dV_b_si = (d_Vgsteff_dV_b_si-d_DeltaPhi_dV_b_si);
d_T1_dV_b_GND = (d_Vgsteff_dV_b_GND-d_DeltaPhi_dV_b_GND);
d_T1_dV_p_GND = (d_Vgsteff_dV_p_GND-d_DeltaPhi_dV_p_GND);
d_T1_dTemp_t_GND = (d_Vgsteff_dTemp_t_GND-d_DeltaPhi_dTemp_t_GND);
d_T1_dV_gi_si = (d_Vgsteff_dV_gi_si-d_DeltaPhi_dV_gi_si);
d_T1_dV_di_si = (d_Vgsteff_dV_di_si-d_DeltaPhi_dV_di_si);
d_T1_dV_sb_si = (d_Vgsteff_dV_sb_si-d_DeltaPhi_dV_sb_si);
d_T1_dV_db_di = (d_Vgsteff_dV_db_di-d_DeltaPhi_dV_db_di);
d_T1_dV_gi_p = (d_Vgsteff_dV_gi_p-d_DeltaPhi_dV_gi_p);
T1 = (Vgsteff-DeltaPhi);

d_AbulkCV_dV_b_GND = d_Abulk0_dV_b_GND*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_p_GND = d_Abulk0_dV_p_GND*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dTemp_t_GND = d_Abulk0_dTemp_t_GND*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_gi_si = d_Abulk0_dV_gi_si*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_di_si = d_Abulk0_dV_di_si*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_e_si = d_Abulk0_dV_e_si*pParam_B4SOIabulkCVfactor;
d_AbulkCV_dV_b_si = d_Abulk0_dV_b_si*pParam_B4SOIabulkCVfactor;
AbulkCV = (Abulk0*pParam_B4SOIabulkCVfactor);

d_VdsatCV_dV_b_GND = ((AbulkCV*d_T1_dV_b_GND-T1*d_AbulkCV_dV_b_GND)/AbulkCV/AbulkCV);
d_VdsatCV_dV_p_GND = ((AbulkCV*d_T1_dV_p_GND-T1*d_AbulkCV_dV_p_GND)/AbulkCV/AbulkCV);
d_VdsatCV_dTemp_t_GND = ((AbulkCV*d_T1_dTemp_t_GND-T1*d_AbulkCV_dTemp_t_GND)/AbulkCV/AbulkCV);
d_VdsatCV_dV_gi_si = ((AbulkCV*d_T1_dV_gi_si-T1*d_AbulkCV_dV_gi_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_di_si = ((AbulkCV*d_T1_dV_di_si-T1*d_AbulkCV_dV_di_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_e_si = ((AbulkCV*d_T1_dV_e_si-T1*d_AbulkCV_dV_e_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_b_si = ((AbulkCV*d_T1_dV_b_si-T1*d_AbulkCV_dV_b_si)/AbulkCV/AbulkCV);
d_VdsatCV_dV_sb_si = (d_T1_dV_sb_si/AbulkCV);
d_VdsatCV_dV_db_di = (d_T1_dV_db_di/AbulkCV);
d_VdsatCV_dV_gi_p = (d_T1_dV_gi_p/AbulkCV);
VdsatCV = (T1/AbulkCV);

d_V4_dV_e_si = d_VdsatCV_dV_e_si;
d_V4_dV_b_si = d_VdsatCV_dV_b_si;
d_V4_dV_b_GND = d_VdsatCV_dV_b_GND;
d_V4_dV_p_GND = d_VdsatCV_dV_p_GND;
d_V4_dTemp_t_GND = d_VdsatCV_dTemp_t_GND;
d_V4_dV_gi_si = d_VdsatCV_dV_gi_si;
d_V4_dV_di_si = (d_VdsatCV_dV_di_si-d_Vds_dV_di_si);
d_V4_dV_sb_si = d_VdsatCV_dV_sb_si;
d_V4_dV_db_di = d_VdsatCV_dV_db_di;
d_V4_dV_gi_p = d_VdsatCV_dV_gi_p;
V4 = ((VdsatCV-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T0_dV_gm_si =  0.0;
d_T0_dV_e_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_e_si+d_V4_dV_e_si*V4)+(4.0*0.02)*d_VdsatCV_dV_e_si)));
d_T0_dV_b_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_si+d_V4_dV_b_si*V4)+(4.0*0.02)*d_VdsatCV_dV_b_si)));
d_T0_dV_b_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_GND+d_V4_dV_b_GND*V4)+(4.0*0.02)*d_VdsatCV_dV_b_GND)));
d_T0_dV_p_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_p_GND+d_V4_dV_p_GND*V4)+(4.0*0.02)*d_VdsatCV_dV_p_GND)));
d_T0_dTemp_t_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dTemp_t_GND+d_V4_dTemp_t_GND*V4)+(4.0*0.02)*d_VdsatCV_dTemp_t_GND)));
d_T0_dV_gi_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_si+d_V4_dV_gi_si*V4)+(4.0*0.02)*d_VdsatCV_dV_gi_si)));
d_T0_dV_di_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_di_si+d_V4_dV_di_si*V4)+(4.0*0.02)*d_VdsatCV_dV_di_si)));
d_T0_dV_sb_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_sb_si+d_V4_dV_sb_si*V4)+(4.0*0.02)*d_VdsatCV_dV_sb_si)));
d_T0_dV_db_di = (deriv_sqrt_0_d0*(((V4*d_V4_dV_db_di+d_V4_dV_db_di*V4)+(4.0*0.02)*d_VdsatCV_dV_db_di)));
d_T0_dV_gi_p = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_p+d_V4_dV_gi_p*V4)+(4.0*0.02)*d_VdsatCV_dV_gi_p)));
T0 = value_sqrt_0;
}

d_VdseffCV_dV_e_si = (d_VdsatCV_dV_e_si-0.5*(d_V4_dV_e_si+d_T0_dV_e_si));
d_VdseffCV_dV_b_si = (d_VdsatCV_dV_b_si-0.5*(d_V4_dV_b_si+d_T0_dV_b_si));
d_VdseffCV_dV_b_GND = (d_VdsatCV_dV_b_GND-0.5*(d_V4_dV_b_GND+d_T0_dV_b_GND));
d_VdseffCV_dV_p_GND = (d_VdsatCV_dV_p_GND-0.5*(d_V4_dV_p_GND+d_T0_dV_p_GND));
d_VdseffCV_dTemp_t_GND = (d_VdsatCV_dTemp_t_GND-0.5*(d_V4_dTemp_t_GND+d_T0_dTemp_t_GND));
d_VdseffCV_dV_gi_si = (d_VdsatCV_dV_gi_si-0.5*(d_V4_dV_gi_si+d_T0_dV_gi_si));
d_VdseffCV_dV_di_si = (d_VdsatCV_dV_di_si-0.5*(d_V4_dV_di_si+d_T0_dV_di_si));
d_VdseffCV_dV_sb_si = (d_VdsatCV_dV_sb_si-0.5*(d_V4_dV_sb_si+d_T0_dV_sb_si));
d_VdseffCV_dV_db_di = (d_VdsatCV_dV_db_di-0.5*(d_V4_dV_db_di+d_T0_dV_db_di));
d_VdseffCV_dV_gi_p = (d_VdsatCV_dV_gi_p-0.5*(d_V4_dV_gi_p+d_T0_dV_gi_p));
VdseffCV = (VdsatCV-(0.5*(V4+T0)));

d_T0_dV_gm_si =  0.0;
d_T0_dV_sb_si = AbulkCV*d_VdseffCV_dV_sb_si;
d_T0_dV_db_di = AbulkCV*d_VdseffCV_dV_db_di;
d_T0_dV_gi_p = AbulkCV*d_VdseffCV_dV_gi_p;
d_T0_dV_b_GND = (AbulkCV*d_VdseffCV_dV_b_GND+d_AbulkCV_dV_b_GND*VdseffCV);
d_T0_dV_p_GND = (AbulkCV*d_VdseffCV_dV_p_GND+d_AbulkCV_dV_p_GND*VdseffCV);
d_T0_dTemp_t_GND = (AbulkCV*d_VdseffCV_dTemp_t_GND+d_AbulkCV_dTemp_t_GND*VdseffCV);
d_T0_dV_gi_si = (AbulkCV*d_VdseffCV_dV_gi_si+d_AbulkCV_dV_gi_si*VdseffCV);
d_T0_dV_di_si = (AbulkCV*d_VdseffCV_dV_di_si+d_AbulkCV_dV_di_si*VdseffCV);
d_T0_dV_e_si = (AbulkCV*d_VdseffCV_dV_e_si+d_AbulkCV_dV_e_si*VdseffCV);
d_T0_dV_b_si = (AbulkCV*d_VdseffCV_dV_b_si+d_AbulkCV_dV_b_si*VdseffCV);
T0 = (AbulkCV*VdseffCV);

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = 12.0*(d_T1_dV_b_GND-0.5*d_T0_dV_b_GND);
d_T2_dV_p_GND = 12.0*(d_T1_dV_p_GND-0.5*d_T0_dV_p_GND);
d_T2_dTemp_t_GND = 12.0*(d_T1_dTemp_t_GND-0.5*d_T0_dTemp_t_GND);
d_T2_dV_gi_si = 12.0*(d_T1_dV_gi_si-0.5*d_T0_dV_gi_si);
d_T2_dV_di_si = 12.0*(d_T1_dV_di_si-0.5*d_T0_dV_di_si);
d_T2_dV_e_si = 12.0*(d_T1_dV_e_si-0.5*d_T0_dV_e_si);
d_T2_dV_b_si = 12.0*(d_T1_dV_b_si-0.5*d_T0_dV_b_si);
d_T2_dV_sb_si = 12.0*(d_T1_dV_sb_si-0.5*d_T0_dV_sb_si);
d_T2_dV_db_di = 12.0*(d_T1_dV_db_di-0.5*d_T0_dV_db_di);
d_T2_dV_gi_p = 12.0*(d_T1_dV_gi_p-0.5*d_T0_dV_gi_p);
T2 = (12.0*((T1-(0.5*T0))+1.0e-20));

d_T3_dV_b_GND = ((T2*d_T0_dV_b_GND-T0*d_T2_dV_b_GND)/T2/T2);
d_T3_dV_p_GND = ((T2*d_T0_dV_p_GND-T0*d_T2_dV_p_GND)/T2/T2);
d_T3_dTemp_t_GND = ((T2*d_T0_dTemp_t_GND-T0*d_T2_dTemp_t_GND)/T2/T2);
d_T3_dV_e_si = ((T2*d_T0_dV_e_si-T0*d_T2_dV_e_si)/T2/T2);
d_T3_dV_gi_si = ((T2*d_T0_dV_gi_si-T0*d_T2_dV_gi_si)/T2/T2);
d_T3_dV_di_si = ((T2*d_T0_dV_di_si-T0*d_T2_dV_di_si)/T2/T2);
d_T3_dV_b_si = ((T2*d_T0_dV_b_si-T0*d_T2_dV_b_si)/T2/T2);
d_T3_dV_sb_si = ((T2*d_T0_dV_sb_si-T0*d_T2_dV_sb_si)/T2/T2);
d_T3_dV_db_di = ((T2*d_T0_dV_db_di-T0*d_T2_dV_db_di)/T2/T2);
d_T3_dV_gi_p = ((T2*d_T0_dV_gi_p-T0*d_T2_dV_gi_p)/T2/T2);
T3 = (T0/T2);

d_qinv_dV_b_GND = (CoxWLcen*(d_T1_dV_b_GND-(T0*(-d_T3_dV_b_GND)+d_T0_dV_b_GND*(0.5-T3)))+d_CoxWLcen_dV_b_GND*(T1-(T0*(0.5-T3))));
d_qinv_dV_p_GND = (CoxWLcen*(d_T1_dV_p_GND-(T0*(-d_T3_dV_p_GND)+d_T0_dV_p_GND*(0.5-T3)))+d_CoxWLcen_dV_p_GND*(T1-(T0*(0.5-T3))));
d_qinv_dTemp_t_GND = (CoxWLcen*(d_T1_dTemp_t_GND-(T0*(-d_T3_dTemp_t_GND)+d_T0_dTemp_t_GND*(0.5-T3)))+d_CoxWLcen_dTemp_t_GND*(T1-(T0*(0.5-T3))));
d_qinv_dV_gi_si = (CoxWLcen*(d_T1_dV_gi_si-(T0*(-d_T3_dV_gi_si)+d_T0_dV_gi_si*(0.5-T3)))+d_CoxWLcen_dV_gi_si*(T1-(T0*(0.5-T3))));
d_qinv_dV_di_si = (CoxWLcen*(d_T1_dV_di_si-(T0*(-d_T3_dV_di_si)+d_T0_dV_di_si*(0.5-T3)))+d_CoxWLcen_dV_di_si*(T1-(T0*(0.5-T3))));
d_qinv_dV_e_si = (CoxWLcen*(d_T1_dV_e_si-(T0*(-d_T3_dV_e_si)+d_T0_dV_e_si*(0.5-T3)))+d_CoxWLcen_dV_e_si*(T1-(T0*(0.5-T3))));
d_qinv_dV_b_si = (CoxWLcen*(d_T1_dV_b_si-(T0*(-d_T3_dV_b_si)+d_T0_dV_b_si*(0.5-T3)))+d_CoxWLcen_dV_b_si*(T1-(T0*(0.5-T3))));
d_qinv_dV_sb_si = (CoxWLcen*(d_T1_dV_sb_si-(T0*(-d_T3_dV_sb_si)+d_T0_dV_sb_si*(0.5-T3)))+d_CoxWLcen_dV_sb_si*(T1-(T0*(0.5-T3))));
d_qinv_dV_db_di = (CoxWLcen*(d_T1_dV_db_di-(T0*(-d_T3_dV_db_di)+d_T0_dV_db_di*(0.5-T3)))+d_CoxWLcen_dV_db_di*(T1-(T0*(0.5-T3))));
d_qinv_dV_gi_p = (CoxWLcen*(d_T1_dV_gi_p-(T0*(-d_T3_dV_gi_p)+d_T0_dV_gi_p*(0.5-T3)))+d_CoxWLcen_dV_gi_p*(T1-(T0*(0.5-T3))));
qinv = (CoxWLcen*(T1-(T0*(0.5-T3))));
qinoi = qinv;

d_qgate_dV_gm_si =  0.0;
d_qgate_dV_e_si = d_qinv_dV_e_si;
d_qgate_dV_b_si = d_qinv_dV_b_si;
d_qgate_dV_b_GND = d_qinv_dV_b_GND;
d_qgate_dV_p_GND = d_qinv_dV_p_GND;
d_qgate_dTemp_t_GND = d_qinv_dTemp_t_GND;
d_qgate_dV_gi_si = d_qinv_dV_gi_si;
d_qgate_dV_di_si = d_qinv_dV_di_si;
d_qgate_dV_sb_si = d_qinv_dV_sb_si;
d_qgate_dV_db_di = d_qinv_dV_db_di;
d_qgate_dV_gi_p = d_qinv_dV_gi_p;
qgate = qinv;
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T12_dV_sb_si = (d_Vgsteff2_dV_sb_si-d_DeltaPhi2_dV_sb_si);
d_T12_dV_db_di = (d_Vgsteff2_dV_db_di-d_DeltaPhi2_dV_db_di);
d_T12_dV_b_GND = (d_Vgsteff2_dV_b_GND-d_DeltaPhi2_dV_b_GND);
d_T12_dV_p_GND = (d_Vgsteff2_dV_p_GND-d_DeltaPhi2_dV_p_GND);
d_T12_dTemp_t_GND = (d_Vgsteff2_dTemp_t_GND-d_DeltaPhi2_dTemp_t_GND);
d_T12_dV_gi_si = (d_Vgsteff2_dV_gi_si-d_DeltaPhi2_dV_gi_si);
d_T12_dV_di_si = (d_Vgsteff2_dV_di_si-d_DeltaPhi2_dV_di_si);
d_T12_dV_e_si = (d_Vgsteff2_dV_e_si-d_DeltaPhi2_dV_e_si);
d_T12_dV_b_si = (d_Vgsteff2_dV_b_si-d_DeltaPhi2_dV_b_si);
d_T12_dV_gi_p = (d_Vgsteff2_dV_gi_p-d_DeltaPhi2_dV_gi_p);
T12 = (Vgsteff2-DeltaPhi2);

d_VdsatCV2_dV_b_GND = ((AbulkCV*d_T12_dV_b_GND-T12*d_AbulkCV_dV_b_GND)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_p_GND = ((AbulkCV*d_T12_dV_p_GND-T12*d_AbulkCV_dV_p_GND)/AbulkCV/AbulkCV);
d_VdsatCV2_dTemp_t_GND = ((AbulkCV*d_T12_dTemp_t_GND-T12*d_AbulkCV_dTemp_t_GND)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_gi_si = ((AbulkCV*d_T12_dV_gi_si-T12*d_AbulkCV_dV_gi_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_di_si = ((AbulkCV*d_T12_dV_di_si-T12*d_AbulkCV_dV_di_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_e_si = ((AbulkCV*d_T12_dV_e_si-T12*d_AbulkCV_dV_e_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_b_si = ((AbulkCV*d_T12_dV_b_si-T12*d_AbulkCV_dV_b_si)/AbulkCV/AbulkCV);
d_VdsatCV2_dV_sb_si = (d_T12_dV_sb_si/AbulkCV);
d_VdsatCV2_dV_db_di = (d_T12_dV_db_di/AbulkCV);
d_VdsatCV2_dV_gi_p = (d_T12_dV_gi_p/AbulkCV);
VdsatCV2 = (T12/AbulkCV);

d_V4_dV_sb_si = d_VdsatCV2_dV_sb_si;
d_V4_dV_db_di = d_VdsatCV2_dV_db_di;
d_V4_dV_b_GND = d_VdsatCV2_dV_b_GND;
d_V4_dV_p_GND = d_VdsatCV2_dV_p_GND;
d_V4_dTemp_t_GND = d_VdsatCV2_dTemp_t_GND;
d_V4_dV_gi_si = d_VdsatCV2_dV_gi_si;
d_V4_dV_di_si = (d_VdsatCV2_dV_di_si-d_Vds_dV_di_si);
d_V4_dV_e_si = d_VdsatCV2_dV_e_si;
d_V4_dV_b_si = d_VdsatCV2_dV_b_si;
d_V4_dV_gi_p = d_VdsatCV2_dV_gi_p;
V4 = ((VdsatCV2-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T02_dV_e_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_e_si+d_V4_dV_e_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_e_si)));
d_T02_dV_b_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_si+d_V4_dV_b_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_b_si)));
d_T02_dV_b_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_b_GND+d_V4_dV_b_GND*V4)+(4.0*0.02)*d_VdsatCV2_dV_b_GND)));
d_T02_dV_p_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dV_p_GND+d_V4_dV_p_GND*V4)+(4.0*0.02)*d_VdsatCV2_dV_p_GND)));
d_T02_dTemp_t_GND = (deriv_sqrt_0_d0*(((V4*d_V4_dTemp_t_GND+d_V4_dTemp_t_GND*V4)+(4.0*0.02)*d_VdsatCV2_dTemp_t_GND)));
d_T02_dV_gi_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_si+d_V4_dV_gi_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_gi_si)));
d_T02_dV_di_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_di_si+d_V4_dV_di_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_di_si)));
d_T02_dV_sb_si = (deriv_sqrt_0_d0*(((V4*d_V4_dV_sb_si+d_V4_dV_sb_si*V4)+(4.0*0.02)*d_VdsatCV2_dV_sb_si)));
d_T02_dV_db_di = (deriv_sqrt_0_d0*(((V4*d_V4_dV_db_di+d_V4_dV_db_di*V4)+(4.0*0.02)*d_VdsatCV2_dV_db_di)));
d_T02_dV_gi_p = (deriv_sqrt_0_d0*(((V4*d_V4_dV_gi_p+d_V4_dV_gi_p*V4)+(4.0*0.02)*d_VdsatCV2_dV_gi_p)));
T02 = value_sqrt_0;
}

d_VdseffCV2_dV_sb_si = (d_VdsatCV2_dV_sb_si-0.5*(d_V4_dV_sb_si+d_T02_dV_sb_si));
d_VdseffCV2_dV_db_di = (d_VdsatCV2_dV_db_di-0.5*(d_V4_dV_db_di+d_T02_dV_db_di));
d_VdseffCV2_dV_b_GND = (d_VdsatCV2_dV_b_GND-0.5*(d_V4_dV_b_GND+d_T02_dV_b_GND));
d_VdseffCV2_dV_p_GND = (d_VdsatCV2_dV_p_GND-0.5*(d_V4_dV_p_GND+d_T02_dV_p_GND));
d_VdseffCV2_dTemp_t_GND = (d_VdsatCV2_dTemp_t_GND-0.5*(d_V4_dTemp_t_GND+d_T02_dTemp_t_GND));
d_VdseffCV2_dV_gi_si = (d_VdsatCV2_dV_gi_si-0.5*(d_V4_dV_gi_si+d_T02_dV_gi_si));
d_VdseffCV2_dV_di_si = (d_VdsatCV2_dV_di_si-0.5*(d_V4_dV_di_si+d_T02_dV_di_si));
d_VdseffCV2_dV_e_si = (d_VdsatCV2_dV_e_si-0.5*(d_V4_dV_e_si+d_T02_dV_e_si));
d_VdseffCV2_dV_b_si = (d_VdsatCV2_dV_b_si-0.5*(d_V4_dV_b_si+d_T02_dV_b_si));
d_VdseffCV2_dV_gi_p = (d_VdsatCV2_dV_gi_p-0.5*(d_V4_dV_gi_p+d_T02_dV_gi_p));
VdseffCV2 = (VdsatCV2-(0.5*(V4+T02)));

d_T02_dV_sb_si = AbulkCV*d_VdseffCV2_dV_sb_si;
d_T02_dV_db_di = AbulkCV*d_VdseffCV2_dV_db_di;
d_T02_dV_gi_p = AbulkCV*d_VdseffCV2_dV_gi_p;
d_T02_dV_b_GND = (AbulkCV*d_VdseffCV2_dV_b_GND+d_AbulkCV_dV_b_GND*VdseffCV2);
d_T02_dV_p_GND = (AbulkCV*d_VdseffCV2_dV_p_GND+d_AbulkCV_dV_p_GND*VdseffCV2);
d_T02_dTemp_t_GND = (AbulkCV*d_VdseffCV2_dTemp_t_GND+d_AbulkCV_dTemp_t_GND*VdseffCV2);
d_T02_dV_gi_si = (AbulkCV*d_VdseffCV2_dV_gi_si+d_AbulkCV_dV_gi_si*VdseffCV2);
d_T02_dV_di_si = (AbulkCV*d_VdseffCV2_dV_di_si+d_AbulkCV_dV_di_si*VdseffCV2);
d_T02_dV_e_si = (AbulkCV*d_VdseffCV2_dV_e_si+d_AbulkCV_dV_e_si*VdseffCV2);
d_T02_dV_b_si = (AbulkCV*d_VdseffCV2_dV_b_si+d_AbulkCV_dV_b_si*VdseffCV2);
T02 = (AbulkCV*VdseffCV2);

d_T22_dV_b_GND = 12.0*(d_T12_dV_b_GND-0.5*d_T02_dV_b_GND);
d_T22_dV_p_GND = 12.0*(d_T12_dV_p_GND-0.5*d_T02_dV_p_GND);
d_T22_dTemp_t_GND = 12.0*(d_T12_dTemp_t_GND-0.5*d_T02_dTemp_t_GND);
d_T22_dV_gi_si = 12.0*(d_T12_dV_gi_si-0.5*d_T02_dV_gi_si);
d_T22_dV_di_si = 12.0*(d_T12_dV_di_si-0.5*d_T02_dV_di_si);
d_T22_dV_e_si = 12.0*(d_T12_dV_e_si-0.5*d_T02_dV_e_si);
d_T22_dV_b_si = 12.0*(d_T12_dV_b_si-0.5*d_T02_dV_b_si);
d_T22_dV_sb_si = 12.0*(d_T12_dV_sb_si-0.5*d_T02_dV_sb_si);
d_T22_dV_db_di = 12.0*(d_T12_dV_db_di-0.5*d_T02_dV_db_di);
d_T22_dV_gi_p = 12.0*(d_T12_dV_gi_p-0.5*d_T02_dV_gi_p);
T22 = (12.0*((T12-(0.5*T02))+1.0e-20));

d_T3_dV_sb_si = ((T22*d_T02_dV_sb_si-T02*d_T22_dV_sb_si)/T22/T22);
d_T3_dV_db_di = ((T22*d_T02_dV_db_di-T02*d_T22_dV_db_di)/T22/T22);
d_T3_dV_gi_p = ((T22*d_T02_dV_gi_p-T02*d_T22_dV_gi_p)/T22/T22);
d_T3_dV_b_GND = ((T22*d_T02_dV_b_GND-T02*d_T22_dV_b_GND)/T22/T22);
d_T3_dV_p_GND = ((T22*d_T02_dV_p_GND-T02*d_T22_dV_p_GND)/T22/T22);
d_T3_dTemp_t_GND = ((T22*d_T02_dTemp_t_GND-T02*d_T22_dTemp_t_GND)/T22/T22);
d_T3_dV_gi_si = ((T22*d_T02_dV_gi_si-T02*d_T22_dV_gi_si)/T22/T22);
d_T3_dV_di_si = ((T22*d_T02_dV_di_si-T02*d_T22_dV_di_si)/T22/T22);
d_T3_dV_e_si = ((T22*d_T02_dV_e_si-T02*d_T22_dV_e_si)/T22/T22);
d_T3_dV_b_si = ((T22*d_T02_dV_b_si-T02*d_T22_dV_b_si)/T22/T22);
T3 = (T02/T22);

d_T7_dV_b_GND = (CoxWLcen2*(d_T12_dV_b_GND-(T02*(-d_T3_dV_b_GND)+d_T02_dV_b_GND*(0.5-T3)))+d_CoxWLcen2_dV_b_GND*(T12-(T02*(0.5-T3))));
d_T7_dV_p_GND = (CoxWLcen2*(d_T12_dV_p_GND-(T02*(-d_T3_dV_p_GND)+d_T02_dV_p_GND*(0.5-T3)))+d_CoxWLcen2_dV_p_GND*(T12-(T02*(0.5-T3))));
d_T7_dTemp_t_GND = (CoxWLcen2*(d_T12_dTemp_t_GND-(T02*(-d_T3_dTemp_t_GND)+d_T02_dTemp_t_GND*(0.5-T3)))+d_CoxWLcen2_dTemp_t_GND*(T12-(T02*(0.5-T3))));
d_T7_dV_gi_si = (CoxWLcen2*(d_T12_dV_gi_si-(T02*(-d_T3_dV_gi_si)+d_T02_dV_gi_si*(0.5-T3)))+d_CoxWLcen2_dV_gi_si*(T12-(T02*(0.5-T3))));
d_T7_dV_di_si = (CoxWLcen2*(d_T12_dV_di_si-(T02*(-d_T3_dV_di_si)+d_T02_dV_di_si*(0.5-T3)))+d_CoxWLcen2_dV_di_si*(T12-(T02*(0.5-T3))));
d_T7_dV_e_si = (CoxWLcen2*(d_T12_dV_e_si-(T02*(-d_T3_dV_e_si)+d_T02_dV_e_si*(0.5-T3)))+d_CoxWLcen2_dV_e_si*(T12-(T02*(0.5-T3))));
d_T7_dV_b_si = (CoxWLcen2*(d_T12_dV_b_si-(T02*(-d_T3_dV_b_si)+d_T02_dV_b_si*(0.5-T3)))+d_CoxWLcen2_dV_b_si*(T12-(T02*(0.5-T3))));
d_T7_dV_sb_si = (CoxWLcen2*(d_T12_dV_sb_si-(T02*(-d_T3_dV_sb_si)+d_T02_dV_sb_si*(0.5-T3)))+d_CoxWLcen2_dV_sb_si*(T12-(T02*(0.5-T3))));
d_T7_dV_db_di = (CoxWLcen2*(d_T12_dV_db_di-(T02*(-d_T3_dV_db_di)+d_T02_dV_db_di*(0.5-T3)))+d_CoxWLcen2_dV_db_di*(T12-(T02*(0.5-T3))));
d_T7_dV_gi_p = (CoxWLcen2*(d_T12_dV_gi_p-(T02*(-d_T3_dV_gi_p)+d_T02_dV_gi_p*(0.5-T3)))+d_CoxWLcen2_dV_gi_p*(T12-(T02*(0.5-T3))));
T7 = (CoxWLcen2*(T12-(T02*(0.5-T3))));

d_qinv_dV_e_si = (d_qinv_dV_e_si+d_T7_dV_e_si);
d_qinv_dV_b_si = (d_qinv_dV_b_si+d_T7_dV_b_si);
d_qinv_dV_b_GND = (d_qinv_dV_b_GND+d_T7_dV_b_GND);
d_qinv_dV_p_GND = (d_qinv_dV_p_GND+d_T7_dV_p_GND);
d_qinv_dTemp_t_GND = (d_qinv_dTemp_t_GND+d_T7_dTemp_t_GND);
d_qinv_dV_gi_si = (d_qinv_dV_gi_si+d_T7_dV_gi_si);
d_qinv_dV_di_si = (d_qinv_dV_di_si+d_T7_dV_di_si);
d_qinv_dV_sb_si = (d_qinv_dV_sb_si+d_T7_dV_sb_si);
d_qinv_dV_db_di = (d_qinv_dV_db_di+d_T7_dV_db_di);
d_qinv_dV_gi_p = (d_qinv_dV_gi_p+d_T7_dV_gi_p);
qinv = (qinv+T7);
qinoi = qinv;

d_qgate_dV_gm_si =  0.0;
d_qgate_dV_e_si = d_qinv_dV_e_si;
d_qgate_dV_b_si = d_qinv_dV_b_si;
d_qgate_dV_b_GND = d_qinv_dV_b_GND;
d_qgate_dV_p_GND = d_qinv_dV_p_GND;
d_qgate_dTemp_t_GND = d_qinv_dTemp_t_GND;
d_qgate_dV_gi_si = d_qinv_dV_gi_si;
d_qgate_dV_di_si = d_qinv_dV_di_si;
d_qgate_dV_sb_si = d_qinv_dV_sb_si;
d_qgate_dV_db_di = d_qinv_dV_db_di;
d_qgate_dV_gi_p = d_qinv_dV_gi_p;
qgate = qinv;
}
if ((B4SOIsoiMod==2))
{

d_qbulk_dV_b_si = d_qbulk_dV_e_si = d_qbulk_dV_di_si = d_qbulk_dV_gi_si = d_qbulk_dTemp_t_GND = d_qbulk_dV_p_GND = d_qbulk_dV_b_GND = d_qbulk_dV_gi_p = d_qbulk_dV_db_di = d_qbulk_dV_sb_si =  0.0;
qbulk = 0;
}
else
{

d_T7_dV_gi_p = d_T7_dV_db_di = d_T7_dV_sb_si =  0.0;
d_T7_dV_b_GND = (-d_AbulkCV_dV_b_GND);
d_T7_dV_p_GND = (-d_AbulkCV_dV_p_GND);
d_T7_dTemp_t_GND = (-d_AbulkCV_dTemp_t_GND);
d_T7_dV_gi_si = (-d_AbulkCV_dV_gi_si);
d_T7_dV_di_si = (-d_AbulkCV_dV_di_si);
d_T7_dV_e_si = (-d_AbulkCV_dV_e_si);
d_T7_dV_b_si = (-d_AbulkCV_dV_b_si);
T7 = (1.0-AbulkCV);

d_qbulk_dV_b_GND = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_b_GND-((T2*(T0*d_VdseffCV_dV_b_GND+d_T0_dV_b_GND*VdseffCV)-(T0*VdseffCV)*d_T2_dV_b_GND)/T2/T2))+(CoxWLcenb*d_T7_dV_b_GND+d_CoxWLcenb_dV_b_GND*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_p_GND = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_p_GND-((T2*(T0*d_VdseffCV_dV_p_GND+d_T0_dV_p_GND*VdseffCV)-(T0*VdseffCV)*d_T2_dV_p_GND)/T2/T2))+(CoxWLcenb*d_T7_dV_p_GND+d_CoxWLcenb_dV_p_GND*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dTemp_t_GND = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dTemp_t_GND-((T2*(T0*d_VdseffCV_dTemp_t_GND+d_T0_dTemp_t_GND*VdseffCV)-(T0*VdseffCV)*d_T2_dTemp_t_GND)/T2/T2))+(CoxWLcenb*d_T7_dTemp_t_GND+d_CoxWLcenb_dTemp_t_GND*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_gi_si = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_gi_si-((T2*(T0*d_VdseffCV_dV_gi_si+d_T0_dV_gi_si*VdseffCV)-(T0*VdseffCV)*d_T2_dV_gi_si)/T2/T2))+(CoxWLcenb*d_T7_dV_gi_si+d_CoxWLcenb_dV_gi_si*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_di_si = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_di_si-((T2*(T0*d_VdseffCV_dV_di_si+d_T0_dV_di_si*VdseffCV)-(T0*VdseffCV)*d_T2_dV_di_si)/T2/T2))+(CoxWLcenb*d_T7_dV_di_si+d_CoxWLcenb_dV_di_si*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_e_si = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_e_si-((T2*(T0*d_VdseffCV_dV_e_si+d_T0_dV_e_si*VdseffCV)-(T0*VdseffCV)*d_T2_dV_e_si)/T2/T2))+(CoxWLcenb*d_T7_dV_e_si+d_CoxWLcenb_dV_e_si*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_b_si = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_b_si-((T2*(T0*d_VdseffCV_dV_b_si+d_T0_dV_b_si*VdseffCV)-(T0*VdseffCV)*d_T2_dV_b_si)/T2/T2))+(CoxWLcenb*d_T7_dV_b_si+d_CoxWLcenb_dV_b_si*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_sb_si = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_sb_si-((T2*(T0*d_VdseffCV_dV_sb_si+d_T0_dV_sb_si*VdseffCV)-(T0*VdseffCV)*d_T2_dV_sb_si)/T2/T2))+(CoxWLcenb*d_T7_dV_sb_si+d_CoxWLcenb_dV_sb_si*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_db_di = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_db_di-((T2*(T0*d_VdseffCV_dV_db_di+d_T0_dV_db_di*VdseffCV)-(T0*VdseffCV)*d_T2_dV_db_di)/T2/T2))+(CoxWLcenb*d_T7_dV_db_di+d_CoxWLcenb_dV_db_di*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
d_qbulk_dV_gi_p = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dV_gi_p-((T2*(T0*d_VdseffCV_dV_gi_p+d_T0_dV_gi_p*VdseffCV)-(T0*VdseffCV)*d_T2_dV_gi_p)/T2/T2))+(CoxWLcenb*d_T7_dV_gi_p+d_CoxWLcenb_dV_gi_p*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
qbulk = ((CoxWLcenb*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_qbulk2_dV_b_GND = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_b_GND-((T22*(T02*d_VdseffCV2_dV_b_GND+d_T02_dV_b_GND*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_b_GND)/T22/T22))+(CoxWLcenb2*d_T7_dV_b_GND+d_CoxWLcenb2_dV_b_GND*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_p_GND = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_p_GND-((T22*(T02*d_VdseffCV2_dV_p_GND+d_T02_dV_p_GND*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_p_GND)/T22/T22))+(CoxWLcenb2*d_T7_dV_p_GND+d_CoxWLcenb2_dV_p_GND*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dTemp_t_GND = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dTemp_t_GND-((T22*(T02*d_VdseffCV2_dTemp_t_GND+d_T02_dTemp_t_GND*VdseffCV2)-(T02*VdseffCV2)*d_T22_dTemp_t_GND)/T22/T22))+(CoxWLcenb2*d_T7_dTemp_t_GND+d_CoxWLcenb2_dTemp_t_GND*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_gi_si = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_gi_si-((T22*(T02*d_VdseffCV2_dV_gi_si+d_T02_dV_gi_si*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_gi_si)/T22/T22))+(CoxWLcenb2*d_T7_dV_gi_si+d_CoxWLcenb2_dV_gi_si*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_di_si = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_di_si-((T22*(T02*d_VdseffCV2_dV_di_si+d_T02_dV_di_si*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_di_si)/T22/T22))+(CoxWLcenb2*d_T7_dV_di_si+d_CoxWLcenb2_dV_di_si*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_e_si = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_e_si-((T22*(T02*d_VdseffCV2_dV_e_si+d_T02_dV_e_si*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_e_si)/T22/T22))+(CoxWLcenb2*d_T7_dV_e_si+d_CoxWLcenb2_dV_e_si*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_b_si = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_b_si-((T22*(T02*d_VdseffCV2_dV_b_si+d_T02_dV_b_si*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_b_si)/T22/T22))+(CoxWLcenb2*d_T7_dV_b_si+d_CoxWLcenb2_dV_b_si*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_sb_si = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_sb_si-((T22*(T02*d_VdseffCV2_dV_sb_si+d_T02_dV_sb_si*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_sb_si)/T22/T22))+(CoxWLcenb2*d_T7_dV_sb_si+d_CoxWLcenb2_dV_sb_si*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_db_di = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_db_di-((T22*(T02*d_VdseffCV2_dV_db_di+d_T02_dV_db_di*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_db_di)/T22/T22))+(CoxWLcenb2*d_T7_dV_db_di+d_CoxWLcenb2_dV_db_di*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk2_dV_gi_p = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dV_gi_p-((T22*(T02*d_VdseffCV2_dV_gi_p+d_T02_dV_gi_p*VdseffCV2)-(T02*VdseffCV2)*d_T22_dV_gi_p)/T22/T22))+(CoxWLcenb2*d_T7_dV_gi_p+d_CoxWLcenb2_dV_gi_p*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
qbulk2 = ((CoxWLcenb2*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));

d_qbulk_dV_sb_si = (d_qbulk_dV_sb_si+d_qbulk2_dV_sb_si);
d_qbulk_dV_db_di = (d_qbulk_dV_db_di+d_qbulk2_dV_db_di);
d_qbulk_dV_gi_p = (d_qbulk_dV_gi_p+d_qbulk2_dV_gi_p);
d_qbulk_dV_b_GND = (d_qbulk_dV_b_GND+d_qbulk2_dV_b_GND);
d_qbulk_dV_p_GND = (d_qbulk_dV_p_GND+d_qbulk2_dV_p_GND);
d_qbulk_dTemp_t_GND = (d_qbulk_dTemp_t_GND+d_qbulk2_dTemp_t_GND);
d_qbulk_dV_gi_si = (d_qbulk_dV_gi_si+d_qbulk2_dV_gi_si);
d_qbulk_dV_di_si = (d_qbulk_dV_di_si+d_qbulk2_dV_di_si);
d_qbulk_dV_e_si = (d_qbulk_dV_e_si+d_qbulk2_dV_e_si);
d_qbulk_dV_b_si = (d_qbulk_dV_b_si+d_qbulk2_dV_b_si);
qbulk = (qbulk+qbulk2);
}
}
if ((B4SOIxpart>0.5))
{

d_qsrc_dV_b_GND = ((-CoxWLcen)*(((d_T1_dV_b_GND/2.0)+(d_T0_dV_b_GND/4.0))-((T2*((0.5*T0)*d_T0_dV_b_GND+0.5*d_T0_dV_b_GND*T0)-((0.5*T0)*T0)*d_T2_dV_b_GND)/T2/T2))+(-d_CoxWLcen_dV_b_GND)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_p_GND = ((-CoxWLcen)*(((d_T1_dV_p_GND/2.0)+(d_T0_dV_p_GND/4.0))-((T2*((0.5*T0)*d_T0_dV_p_GND+0.5*d_T0_dV_p_GND*T0)-((0.5*T0)*T0)*d_T2_dV_p_GND)/T2/T2))+(-d_CoxWLcen_dV_p_GND)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dTemp_t_GND = ((-CoxWLcen)*(((d_T1_dTemp_t_GND/2.0)+(d_T0_dTemp_t_GND/4.0))-((T2*((0.5*T0)*d_T0_dTemp_t_GND+0.5*d_T0_dTemp_t_GND*T0)-((0.5*T0)*T0)*d_T2_dTemp_t_GND)/T2/T2))+(-d_CoxWLcen_dTemp_t_GND)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_gi_si = ((-CoxWLcen)*(((d_T1_dV_gi_si/2.0)+(d_T0_dV_gi_si/4.0))-((T2*((0.5*T0)*d_T0_dV_gi_si+0.5*d_T0_dV_gi_si*T0)-((0.5*T0)*T0)*d_T2_dV_gi_si)/T2/T2))+(-d_CoxWLcen_dV_gi_si)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_di_si = ((-CoxWLcen)*(((d_T1_dV_di_si/2.0)+(d_T0_dV_di_si/4.0))-((T2*((0.5*T0)*d_T0_dV_di_si+0.5*d_T0_dV_di_si*T0)-((0.5*T0)*T0)*d_T2_dV_di_si)/T2/T2))+(-d_CoxWLcen_dV_di_si)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_e_si = ((-CoxWLcen)*(((d_T1_dV_e_si/2.0)+(d_T0_dV_e_si/4.0))-((T2*((0.5*T0)*d_T0_dV_e_si+0.5*d_T0_dV_e_si*T0)-((0.5*T0)*T0)*d_T2_dV_e_si)/T2/T2))+(-d_CoxWLcen_dV_e_si)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_b_si = ((-CoxWLcen)*(((d_T1_dV_b_si/2.0)+(d_T0_dV_b_si/4.0))-((T2*((0.5*T0)*d_T0_dV_b_si+0.5*d_T0_dV_b_si*T0)-((0.5*T0)*T0)*d_T2_dV_b_si)/T2/T2))+(-d_CoxWLcen_dV_b_si)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_sb_si = ((-CoxWLcen)*(((d_T1_dV_sb_si/2.0)+(d_T0_dV_sb_si/4.0))-((T2*((0.5*T0)*d_T0_dV_sb_si+0.5*d_T0_dV_sb_si*T0)-((0.5*T0)*T0)*d_T2_dV_sb_si)/T2/T2))+(-d_CoxWLcen_dV_sb_si)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_db_di = ((-CoxWLcen)*(((d_T1_dV_db_di/2.0)+(d_T0_dV_db_di/4.0))-((T2*((0.5*T0)*d_T0_dV_db_di+0.5*d_T0_dV_db_di*T0)-((0.5*T0)*T0)*d_T2_dV_db_di)/T2/T2))+(-d_CoxWLcen_dV_db_di)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
d_qsrc_dV_gi_p = ((-CoxWLcen)*(((d_T1_dV_gi_p/2.0)+(d_T0_dV_gi_p/4.0))-((T2*((0.5*T0)*d_T0_dV_gi_p+0.5*d_T0_dV_gi_p*T0)-((0.5*T0)*T0)*d_T2_dV_gi_p)/T2/T2))+(-d_CoxWLcen_dV_gi_p)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
qsrc = ((-CoxWLcen)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_qsrc2_dV_b_GND = ((-CoxWLcen2)*((((d_Vgsteff2_dV_b_GND-d_DeltaPhi2_dV_b_GND)/2.0)+(d_T02_dV_b_GND/4.0))-((T22*((0.5*T02)*d_T02_dV_b_GND+0.5*d_T02_dV_b_GND*T02)-((0.5*T02)*T02)*d_T22_dV_b_GND)/T22/T22))+(-d_CoxWLcen2_dV_b_GND)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_p_GND = ((-CoxWLcen2)*((((d_Vgsteff2_dV_p_GND-d_DeltaPhi2_dV_p_GND)/2.0)+(d_T02_dV_p_GND/4.0))-((T22*((0.5*T02)*d_T02_dV_p_GND+0.5*d_T02_dV_p_GND*T02)-((0.5*T02)*T02)*d_T22_dV_p_GND)/T22/T22))+(-d_CoxWLcen2_dV_p_GND)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dTemp_t_GND = ((-CoxWLcen2)*((((d_Vgsteff2_dTemp_t_GND-d_DeltaPhi2_dTemp_t_GND)/2.0)+(d_T02_dTemp_t_GND/4.0))-((T22*((0.5*T02)*d_T02_dTemp_t_GND+0.5*d_T02_dTemp_t_GND*T02)-((0.5*T02)*T02)*d_T22_dTemp_t_GND)/T22/T22))+(-d_CoxWLcen2_dTemp_t_GND)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_gi_si = ((-CoxWLcen2)*((((d_Vgsteff2_dV_gi_si-d_DeltaPhi2_dV_gi_si)/2.0)+(d_T02_dV_gi_si/4.0))-((T22*((0.5*T02)*d_T02_dV_gi_si+0.5*d_T02_dV_gi_si*T02)-((0.5*T02)*T02)*d_T22_dV_gi_si)/T22/T22))+(-d_CoxWLcen2_dV_gi_si)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_di_si = ((-CoxWLcen2)*((((d_Vgsteff2_dV_di_si-d_DeltaPhi2_dV_di_si)/2.0)+(d_T02_dV_di_si/4.0))-((T22*((0.5*T02)*d_T02_dV_di_si+0.5*d_T02_dV_di_si*T02)-((0.5*T02)*T02)*d_T22_dV_di_si)/T22/T22))+(-d_CoxWLcen2_dV_di_si)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_e_si = ((-CoxWLcen2)*((((d_Vgsteff2_dV_e_si-d_DeltaPhi2_dV_e_si)/2.0)+(d_T02_dV_e_si/4.0))-((T22*((0.5*T02)*d_T02_dV_e_si+0.5*d_T02_dV_e_si*T02)-((0.5*T02)*T02)*d_T22_dV_e_si)/T22/T22))+(-d_CoxWLcen2_dV_e_si)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_b_si = ((-CoxWLcen2)*((((d_Vgsteff2_dV_b_si-d_DeltaPhi2_dV_b_si)/2.0)+(d_T02_dV_b_si/4.0))-((T22*((0.5*T02)*d_T02_dV_b_si+0.5*d_T02_dV_b_si*T02)-((0.5*T02)*T02)*d_T22_dV_b_si)/T22/T22))+(-d_CoxWLcen2_dV_b_si)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_sb_si = ((-CoxWLcen2)*((((d_Vgsteff2_dV_sb_si-d_DeltaPhi2_dV_sb_si)/2.0)+(d_T02_dV_sb_si/4.0))-((T22*((0.5*T02)*d_T02_dV_sb_si+0.5*d_T02_dV_sb_si*T02)-((0.5*T02)*T02)*d_T22_dV_sb_si)/T22/T22))+(-d_CoxWLcen2_dV_sb_si)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_db_di = ((-CoxWLcen2)*((((d_Vgsteff2_dV_db_di-d_DeltaPhi2_dV_db_di)/2.0)+(d_T02_dV_db_di/4.0))-((T22*((0.5*T02)*d_T02_dV_db_di+0.5*d_T02_dV_db_di*T02)-((0.5*T02)*T02)*d_T22_dV_db_di)/T22/T22))+(-d_CoxWLcen2_dV_db_di)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc2_dV_gi_p = ((-CoxWLcen2)*((((d_Vgsteff2_dV_gi_p-d_DeltaPhi2_dV_gi_p)/2.0)+(d_T02_dV_gi_p/4.0))-((T22*((0.5*T02)*d_T02_dV_gi_p+0.5*d_T02_dV_gi_p*T02)-((0.5*T02)*T02)*d_T22_dV_gi_p)/T22/T22))+(-d_CoxWLcen2_dV_gi_p)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
qsrc2 = ((-CoxWLcen2)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));

d_qsrc_dV_e_si = (d_qsrc_dV_e_si+d_qsrc2_dV_e_si);
d_qsrc_dV_b_si = (d_qsrc_dV_b_si+d_qsrc2_dV_b_si);
d_qsrc_dV_b_GND = (d_qsrc_dV_b_GND+d_qsrc2_dV_b_GND);
d_qsrc_dV_p_GND = (d_qsrc_dV_p_GND+d_qsrc2_dV_p_GND);
d_qsrc_dTemp_t_GND = (d_qsrc_dTemp_t_GND+d_qsrc2_dTemp_t_GND);
d_qsrc_dV_gi_si = (d_qsrc_dV_gi_si+d_qsrc2_dV_gi_si);
d_qsrc_dV_di_si = (d_qsrc_dV_di_si+d_qsrc2_dV_di_si);
d_qsrc_dV_sb_si = (d_qsrc_dV_sb_si+d_qsrc2_dV_sb_si);
d_qsrc_dV_db_di = (d_qsrc_dV_db_di+d_qsrc2_dV_db_di);
d_qsrc_dV_gi_p = (d_qsrc_dV_gi_p+d_qsrc2_dV_gi_p);
qsrc = (qsrc+qsrc2);
}
}
else
{
if ((B4SOIxpart<0.5))
{

d_T2_dV_gm_si =  0.0;
d_T2_dV_b_GND = (d_T2_dV_b_GND/12.0);
d_T2_dV_p_GND = (d_T2_dV_p_GND/12.0);
d_T2_dTemp_t_GND = (d_T2_dTemp_t_GND/12.0);
d_T2_dV_gi_si = (d_T2_dV_gi_si/12.0);
d_T2_dV_di_si = (d_T2_dV_di_si/12.0);
d_T2_dV_e_si = (d_T2_dV_e_si/12.0);
d_T2_dV_b_si = (d_T2_dV_b_si/12.0);
d_T2_dV_sb_si = (d_T2_dV_sb_si/12.0);
d_T2_dV_db_di = (d_T2_dV_db_di/12.0);
d_T2_dV_gi_p = (d_T2_dV_gi_p/12.0);
T2 = (T2/12.0);

d_T3_dV_b_GND = (((T2*T2)*0.5*d_CoxWLcen_dV_b_GND-(0.5*CoxWLcen)*(T2*d_T2_dV_b_GND+d_T2_dV_b_GND*T2))/(T2*T2)/(T2*T2));
d_T3_dV_p_GND = (((T2*T2)*0.5*d_CoxWLcen_dV_p_GND-(0.5*CoxWLcen)*(T2*d_T2_dV_p_GND+d_T2_dV_p_GND*T2))/(T2*T2)/(T2*T2));
d_T3_dTemp_t_GND = (((T2*T2)*0.5*d_CoxWLcen_dTemp_t_GND-(0.5*CoxWLcen)*(T2*d_T2_dTemp_t_GND+d_T2_dTemp_t_GND*T2))/(T2*T2)/(T2*T2));
d_T3_dV_gi_si = (((T2*T2)*0.5*d_CoxWLcen_dV_gi_si-(0.5*CoxWLcen)*(T2*d_T2_dV_gi_si+d_T2_dV_gi_si*T2))/(T2*T2)/(T2*T2));
d_T3_dV_di_si = (((T2*T2)*0.5*d_CoxWLcen_dV_di_si-(0.5*CoxWLcen)*(T2*d_T2_dV_di_si+d_T2_dV_di_si*T2))/(T2*T2)/(T2*T2));
d_T3_dV_e_si = (((T2*T2)*0.5*d_CoxWLcen_dV_e_si-(0.5*CoxWLcen)*(T2*d_T2_dV_e_si+d_T2_dV_e_si*T2))/(T2*T2)/(T2*T2));
d_T3_dV_b_si = (((T2*T2)*0.5*d_CoxWLcen_dV_b_si-(0.5*CoxWLcen)*(T2*d_T2_dV_b_si+d_T2_dV_b_si*T2))/(T2*T2)/(T2*T2));
d_T3_dV_sb_si = (((T2*T2)*0.5*d_CoxWLcen_dV_sb_si-(0.5*CoxWLcen)*(T2*d_T2_dV_sb_si+d_T2_dV_sb_si*T2))/(T2*T2)/(T2*T2));
d_T3_dV_db_di = (((T2*T2)*0.5*d_CoxWLcen_dV_db_di-(0.5*CoxWLcen)*(T2*d_T2_dV_db_di+d_T2_dV_db_di*T2))/(T2*T2)/(T2*T2));
d_T3_dV_gi_p = (((T2*T2)*0.5*d_CoxWLcen_dV_gi_p-(0.5*CoxWLcen)*(T2*d_T2_dV_gi_p+d_T2_dV_gi_p*T2))/(T2*T2)/(T2*T2));
T3 = ((0.5*CoxWLcen)/(T2*T2));

d_T4_dV_gm_si =  0.0;
d_T4_dV_b_GND = ((T1*((((2.0*T0)*d_T0_dV_b_GND+2.0*d_T0_dV_b_GND*T0)/3.0)+(T1*(d_T1_dV_b_GND-(4.0*d_T0_dV_b_GND/3.0))+d_T1_dV_b_GND*(T1-((4.0*T0)/3.0))))+d_T1_dV_b_GND*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_b_GND+((2.0*T0)*d_T0_dV_b_GND+2.0*d_T0_dV_b_GND*T0)*T0)/15.0));
d_T4_dV_p_GND = ((T1*((((2.0*T0)*d_T0_dV_p_GND+2.0*d_T0_dV_p_GND*T0)/3.0)+(T1*(d_T1_dV_p_GND-(4.0*d_T0_dV_p_GND/3.0))+d_T1_dV_p_GND*(T1-((4.0*T0)/3.0))))+d_T1_dV_p_GND*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_p_GND+((2.0*T0)*d_T0_dV_p_GND+2.0*d_T0_dV_p_GND*T0)*T0)/15.0));
d_T4_dTemp_t_GND = ((T1*((((2.0*T0)*d_T0_dTemp_t_GND+2.0*d_T0_dTemp_t_GND*T0)/3.0)+(T1*(d_T1_dTemp_t_GND-(4.0*d_T0_dTemp_t_GND/3.0))+d_T1_dTemp_t_GND*(T1-((4.0*T0)/3.0))))+d_T1_dTemp_t_GND*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dTemp_t_GND+((2.0*T0)*d_T0_dTemp_t_GND+2.0*d_T0_dTemp_t_GND*T0)*T0)/15.0));
d_T4_dV_gi_si = ((T1*((((2.0*T0)*d_T0_dV_gi_si+2.0*d_T0_dV_gi_si*T0)/3.0)+(T1*(d_T1_dV_gi_si-(4.0*d_T0_dV_gi_si/3.0))+d_T1_dV_gi_si*(T1-((4.0*T0)/3.0))))+d_T1_dV_gi_si*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_gi_si+((2.0*T0)*d_T0_dV_gi_si+2.0*d_T0_dV_gi_si*T0)*T0)/15.0));
d_T4_dV_di_si = ((T1*((((2.0*T0)*d_T0_dV_di_si+2.0*d_T0_dV_di_si*T0)/3.0)+(T1*(d_T1_dV_di_si-(4.0*d_T0_dV_di_si/3.0))+d_T1_dV_di_si*(T1-((4.0*T0)/3.0))))+d_T1_dV_di_si*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_di_si+((2.0*T0)*d_T0_dV_di_si+2.0*d_T0_dV_di_si*T0)*T0)/15.0));
d_T4_dV_e_si = ((T1*((((2.0*T0)*d_T0_dV_e_si+2.0*d_T0_dV_e_si*T0)/3.0)+(T1*(d_T1_dV_e_si-(4.0*d_T0_dV_e_si/3.0))+d_T1_dV_e_si*(T1-((4.0*T0)/3.0))))+d_T1_dV_e_si*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_e_si+((2.0*T0)*d_T0_dV_e_si+2.0*d_T0_dV_e_si*T0)*T0)/15.0));
d_T4_dV_b_si = ((T1*((((2.0*T0)*d_T0_dV_b_si+2.0*d_T0_dV_b_si*T0)/3.0)+(T1*(d_T1_dV_b_si-(4.0*d_T0_dV_b_si/3.0))+d_T1_dV_b_si*(T1-((4.0*T0)/3.0))))+d_T1_dV_b_si*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_b_si+((2.0*T0)*d_T0_dV_b_si+2.0*d_T0_dV_b_si*T0)*T0)/15.0));
d_T4_dV_sb_si = ((T1*((((2.0*T0)*d_T0_dV_sb_si+2.0*d_T0_dV_sb_si*T0)/3.0)+(T1*(d_T1_dV_sb_si-(4.0*d_T0_dV_sb_si/3.0))+d_T1_dV_sb_si*(T1-((4.0*T0)/3.0))))+d_T1_dV_sb_si*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_sb_si+((2.0*T0)*d_T0_dV_sb_si+2.0*d_T0_dV_sb_si*T0)*T0)/15.0));
d_T4_dV_db_di = ((T1*((((2.0*T0)*d_T0_dV_db_di+2.0*d_T0_dV_db_di*T0)/3.0)+(T1*(d_T1_dV_db_di-(4.0*d_T0_dV_db_di/3.0))+d_T1_dV_db_di*(T1-((4.0*T0)/3.0))))+d_T1_dV_db_di*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_db_di+((2.0*T0)*d_T0_dV_db_di+2.0*d_T0_dV_db_di*T0)*T0)/15.0));
d_T4_dV_gi_p = ((T1*((((2.0*T0)*d_T0_dV_gi_p+2.0*d_T0_dV_gi_p*T0)/3.0)+(T1*(d_T1_dV_gi_p-(4.0*d_T0_dV_gi_p/3.0))+d_T1_dV_gi_p*(T1-((4.0*T0)/3.0))))+d_T1_dV_gi_p*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dV_gi_p+((2.0*T0)*d_T0_dV_gi_p+2.0*d_T0_dV_gi_p*T0)*T0)/15.0));
T4 = ((T1*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*T0)/15.0));

d_qsrc_dV_b_GND = ((-T3)*d_T4_dV_b_GND+(-d_T3_dV_b_GND)*T4);
d_qsrc_dV_p_GND = ((-T3)*d_T4_dV_p_GND+(-d_T3_dV_p_GND)*T4);
d_qsrc_dTemp_t_GND = ((-T3)*d_T4_dTemp_t_GND+(-d_T3_dTemp_t_GND)*T4);
d_qsrc_dV_gi_si = ((-T3)*d_T4_dV_gi_si+(-d_T3_dV_gi_si)*T4);
d_qsrc_dV_di_si = ((-T3)*d_T4_dV_di_si+(-d_T3_dV_di_si)*T4);
d_qsrc_dV_e_si = ((-T3)*d_T4_dV_e_si+(-d_T3_dV_e_si)*T4);
d_qsrc_dV_b_si = ((-T3)*d_T4_dV_b_si+(-d_T3_dV_b_si)*T4);
d_qsrc_dV_sb_si = ((-T3)*d_T4_dV_sb_si+(-d_T3_dV_sb_si)*T4);
d_qsrc_dV_db_di = ((-T3)*d_T4_dV_db_di+(-d_T3_dV_db_di)*T4);
d_qsrc_dV_gi_p = ((-T3)*d_T4_dV_gi_p+(-d_T3_dV_gi_p)*T4);
qsrc = ((-T3)*T4);
if ((((B4SOIsoiMod!=2)&&(B4SOIbodyMod!=0))&&(B4SOIagbcp2>0)))
{

d_T22_dV_b_GND = (d_T22_dV_b_GND/12.0);
d_T22_dV_p_GND = (d_T22_dV_p_GND/12.0);
d_T22_dTemp_t_GND = (d_T22_dTemp_t_GND/12.0);
d_T22_dV_gi_si = (d_T22_dV_gi_si/12.0);
d_T22_dV_di_si = (d_T22_dV_di_si/12.0);
d_T22_dV_e_si = (d_T22_dV_e_si/12.0);
d_T22_dV_b_si = (d_T22_dV_b_si/12.0);
d_T22_dV_sb_si = (d_T22_dV_sb_si/12.0);
d_T22_dV_db_di = (d_T22_dV_db_di/12.0);
d_T22_dV_gi_p = (d_T22_dV_gi_p/12.0);
T22 = (T22/12.0);

d_T3_dV_b_GND = (((T22*T22)*0.5*d_CoxWLcen2_dV_b_GND-(0.5*CoxWLcen2)*(T22*d_T22_dV_b_GND+d_T22_dV_b_GND*T22))/(T22*T22)/(T22*T22));
d_T3_dV_p_GND = (((T22*T22)*0.5*d_CoxWLcen2_dV_p_GND-(0.5*CoxWLcen2)*(T22*d_T22_dV_p_GND+d_T22_dV_p_GND*T22))/(T22*T22)/(T22*T22));
d_T3_dTemp_t_GND = (((T22*T22)*0.5*d_CoxWLcen2_dTemp_t_GND-(0.5*CoxWLcen2)*(T22*d_T22_dTemp_t_GND+d_T22_dTemp_t_GND*T22))/(T22*T22)/(T22*T22));
d_T3_dV_gi_si = (((T22*T22)*0.5*d_CoxWLcen2_dV_gi_si-(0.5*CoxWLcen2)*(T22*d_T22_dV_gi_si+d_T22_dV_gi_si*T22))/(T22*T22)/(T22*T22));
d_T3_dV_di_si = (((T22*T22)*0.5*d_CoxWLcen2_dV_di_si-(0.5*CoxWLcen2)*(T22*d_T22_dV_di_si+d_T22_dV_di_si*T22))/(T22*T22)/(T22*T22));
d_T3_dV_e_si = (((T22*T22)*0.5*d_CoxWLcen2_dV_e_si-(0.5*CoxWLcen2)*(T22*d_T22_dV_e_si+d_T22_dV_e_si*T22))/(T22*T22)/(T22*T22));
d_T3_dV_b_si = (((T22*T22)*0.5*d_CoxWLcen2_dV_b_si-(0.5*CoxWLcen2)*(T22*d_T22_dV_b_si+d_T22_dV_b_si*T22))/(T22*T22)/(T22*T22));
d_T3_dV_sb_si = (((T22*T22)*0.5*d_CoxWLcen2_dV_sb_si-(0.5*CoxWLcen2)*(T22*d_T22_dV_sb_si+d_T22_dV_sb_si*T22))/(T22*T22)/(T22*T22));
d_T3_dV_db_di = (((T22*T22)*0.5*d_CoxWLcen2_dV_db_di-(0.5*CoxWLcen2)*(T22*d_T22_dV_db_di+d_T22_dV_db_di*T22))/(T22*T22)/(T22*T22));
d_T3_dV_gi_p = (((T22*T22)*0.5*d_CoxWLcen2_dV_gi_p-(0.5*CoxWLcen2)*(T22*d_T22_dV_gi_p+d_T22_dV_gi_p*T22))/(T22*T22)/(T22*T22));
T3 = ((0.5*CoxWLcen2)/(T22*T22));

d_T4_dV_gm_si =  0.0;
d_T4_dV_b_GND = ((T12*((((2.0*T02)*d_T02_dV_b_GND+2.0*d_T02_dV_b_GND*T02)/3.0)+(T12*(d_T12_dV_b_GND-(4.0*d_T02_dV_b_GND/3.0))+d_T12_dV_b_GND*(T12-((4.0*T02)/3.0))))+d_T12_dV_b_GND*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_b_GND+((2.0*T02)*d_T02_dV_b_GND+2.0*d_T02_dV_b_GND*T02)*T02)/15.0));
d_T4_dV_p_GND = ((T12*((((2.0*T02)*d_T02_dV_p_GND+2.0*d_T02_dV_p_GND*T02)/3.0)+(T12*(d_T12_dV_p_GND-(4.0*d_T02_dV_p_GND/3.0))+d_T12_dV_p_GND*(T12-((4.0*T02)/3.0))))+d_T12_dV_p_GND*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_p_GND+((2.0*T02)*d_T02_dV_p_GND+2.0*d_T02_dV_p_GND*T02)*T02)/15.0));
d_T4_dTemp_t_GND = ((T12*((((2.0*T02)*d_T02_dTemp_t_GND+2.0*d_T02_dTemp_t_GND*T02)/3.0)+(T12*(d_T12_dTemp_t_GND-(4.0*d_T02_dTemp_t_GND/3.0))+d_T12_dTemp_t_GND*(T12-((4.0*T02)/3.0))))+d_T12_dTemp_t_GND*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dTemp_t_GND+((2.0*T02)*d_T02_dTemp_t_GND+2.0*d_T02_dTemp_t_GND*T02)*T02)/15.0));
d_T4_dV_gi_si = ((T12*((((2.0*T02)*d_T02_dV_gi_si+2.0*d_T02_dV_gi_si*T02)/3.0)+(T12*(d_T12_dV_gi_si-(4.0*d_T02_dV_gi_si/3.0))+d_T12_dV_gi_si*(T12-((4.0*T02)/3.0))))+d_T12_dV_gi_si*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_gi_si+((2.0*T02)*d_T02_dV_gi_si+2.0*d_T02_dV_gi_si*T02)*T02)/15.0));
d_T4_dV_di_si = ((T12*((((2.0*T02)*d_T02_dV_di_si+2.0*d_T02_dV_di_si*T02)/3.0)+(T12*(d_T12_dV_di_si-(4.0*d_T02_dV_di_si/3.0))+d_T12_dV_di_si*(T12-((4.0*T02)/3.0))))+d_T12_dV_di_si*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_di_si+((2.0*T02)*d_T02_dV_di_si+2.0*d_T02_dV_di_si*T02)*T02)/15.0));
d_T4_dV_e_si = ((T12*((((2.0*T02)*d_T02_dV_e_si+2.0*d_T02_dV_e_si*T02)/3.0)+(T12*(d_T12_dV_e_si-(4.0*d_T02_dV_e_si/3.0))+d_T12_dV_e_si*(T12-((4.0*T02)/3.0))))+d_T12_dV_e_si*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_e_si+((2.0*T02)*d_T02_dV_e_si+2.0*d_T02_dV_e_si*T02)*T02)/15.0));
d_T4_dV_b_si = ((T12*((((2.0*T02)*d_T02_dV_b_si+2.0*d_T02_dV_b_si*T02)/3.0)+(T12*(d_T12_dV_b_si-(4.0*d_T02_dV_b_si/3.0))+d_T12_dV_b_si*(T12-((4.0*T02)/3.0))))+d_T12_dV_b_si*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_b_si+((2.0*T02)*d_T02_dV_b_si+2.0*d_T02_dV_b_si*T02)*T02)/15.0));
d_T4_dV_sb_si = ((T12*((((2.0*T02)*d_T02_dV_sb_si+2.0*d_T02_dV_sb_si*T02)/3.0)+(T12*(d_T12_dV_sb_si-(4.0*d_T02_dV_sb_si/3.0))+d_T12_dV_sb_si*(T12-((4.0*T02)/3.0))))+d_T12_dV_sb_si*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_sb_si+((2.0*T02)*d_T02_dV_sb_si+2.0*d_T02_dV_sb_si*T02)*T02)/15.0));
d_T4_dV_db_di = ((T12*((((2.0*T02)*d_T02_dV_db_di+2.0*d_T02_dV_db_di*T02)/3.0)+(T12*(d_T12_dV_db_di-(4.0*d_T02_dV_db_di/3.0))+d_T12_dV_db_di*(T12-((4.0*T02)/3.0))))+d_T12_dV_db_di*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_db_di+((2.0*T02)*d_T02_dV_db_di+2.0*d_T02_dV_db_di*T02)*T02)/15.0));
d_T4_dV_gi_p = ((T12*((((2.0*T02)*d_T02_dV_gi_p+2.0*d_T02_dV_gi_p*T02)/3.0)+(T12*(d_T12_dV_gi_p-(4.0*d_T02_dV_gi_p/3.0))+d_T12_dV_gi_p*(T12-((4.0*T02)/3.0))))+d_T12_dV_gi_p*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dV_gi_p+((2.0*T02)*d_T02_dV_gi_p+2.0*d_T02_dV_gi_p*T02)*T02)/15.0));
T4 = ((T12*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*T02)/15.0));

d_qsrc2_dV_b_GND = ((-T3)*d_T4_dV_b_GND+(-d_T3_dV_b_GND)*T4);
d_qsrc2_dV_p_GND = ((-T3)*d_T4_dV_p_GND+(-d_T3_dV_p_GND)*T4);
d_qsrc2_dTemp_t_GND = ((-T3)*d_T4_dTemp_t_GND+(-d_T3_dTemp_t_GND)*T4);
d_qsrc2_dV_gi_si = ((-T3)*d_T4_dV_gi_si+(-d_T3_dV_gi_si)*T4);
d_qsrc2_dV_di_si = ((-T3)*d_T4_dV_di_si+(-d_T3_dV_di_si)*T4);
d_qsrc2_dV_e_si = ((-T3)*d_T4_dV_e_si+(-d_T3_dV_e_si)*T4);
d_qsrc2_dV_b_si = ((-T3)*d_T4_dV_b_si+(-d_T3_dV_b_si)*T4);
d_qsrc2_dV_sb_si = ((-T3)*d_T4_dV_sb_si+(-d_T3_dV_sb_si)*T4);
d_qsrc2_dV_db_di = ((-T3)*d_T4_dV_db_di+(-d_T3_dV_db_di)*T4);
d_qsrc2_dV_gi_p = ((-T3)*d_T4_dV_gi_p+(-d_T3_dV_gi_p)*T4);
qsrc2 = ((-T3)*T4);

d_qsrc_dV_e_si = (d_qsrc_dV_e_si+d_qsrc2_dV_e_si);
d_qsrc_dV_b_si = (d_qsrc_dV_b_si+d_qsrc2_dV_b_si);
d_qsrc_dV_b_GND = (d_qsrc_dV_b_GND+d_qsrc2_dV_b_GND);
d_qsrc_dV_p_GND = (d_qsrc_dV_p_GND+d_qsrc2_dV_p_GND);
d_qsrc_dTemp_t_GND = (d_qsrc_dTemp_t_GND+d_qsrc2_dTemp_t_GND);
d_qsrc_dV_gi_si = (d_qsrc_dV_gi_si+d_qsrc2_dV_gi_si);
d_qsrc_dV_di_si = (d_qsrc_dV_di_si+d_qsrc2_dV_di_si);
d_qsrc_dV_sb_si = (d_qsrc_dV_sb_si+d_qsrc2_dV_sb_si);
d_qsrc_dV_db_di = (d_qsrc_dV_db_di+d_qsrc2_dV_db_di);
d_qsrc_dV_gi_p = (d_qsrc_dV_gi_p+d_qsrc2_dV_gi_p);
qsrc = (qsrc+qsrc2);
}
}
else
{

d_qsrc_dV_e_si = (-0.5)*d_qgate_dV_e_si;
d_qsrc_dV_b_si = (-0.5)*d_qgate_dV_b_si;
d_qsrc_dV_b_GND = (-0.5)*d_qgate_dV_b_GND;
d_qsrc_dV_p_GND = (-0.5)*d_qgate_dV_p_GND;
d_qsrc_dTemp_t_GND = (-0.5)*d_qgate_dTemp_t_GND;
d_qsrc_dV_gi_si = (-0.5)*d_qgate_dV_gi_si;
d_qsrc_dV_di_si = (-0.5)*d_qgate_dV_di_si;
d_qsrc_dV_sb_si = (-0.5)*d_qgate_dV_sb_si;
d_qsrc_dV_db_di = (-0.5)*d_qgate_dV_db_di;
d_qsrc_dV_gi_p = (-0.5)*d_qgate_dV_gi_p;
qsrc = ((-0.5)*qgate);
}
}
if ((B4SOIsoiMod==2))
{

d_Qe1_dV_di_si = d_Qe1_dV_e_si = d_Qe1_dTemp_t_GND = d_Qe1_dV_p_GND = d_Qe1_dV_b_GND = d_Qe1_dV_gi_si = d_Qe1_dV_b_si =  0.0;
Qe1 = 0;
}
else
{
CboxWL = (((pParam_B4SOIkb1*B4SOIfbody)*Cbox)*((((pParam_B4SOIweffCV/B4SOInseg)*B4SOInf)*pParam_B4SOIleffCVbg)+B4SOIaebcp));

d_Qe1_dV_b_si = CboxWL*(-d_Vbs_dV_b_si);
d_Qe1_dV_gi_si = CboxWL*(-d_Vbs_dV_gi_si);
d_Qe1_dV_b_GND = CboxWL*(d_Vesfb_dV_b_GND-d_Vbs_dV_b_GND);
d_Qe1_dV_p_GND = CboxWL*(d_Vesfb_dV_p_GND-d_Vbs_dV_p_GND);
d_Qe1_dTemp_t_GND = CboxWL*(d_Vesfb_dTemp_t_GND-d_Vbs_dTemp_t_GND);
d_Qe1_dV_e_si = CboxWL*(d_Vesfb_dV_e_si-d_Vbs_dV_e_si);
d_Qe1_dV_di_si = CboxWL*(d_Vesfb_dV_di_si-d_Vbs_dV_di_si);
Qe1 = (CboxWL*(Vesfb-Vbs));
}

d_qgate_dV_gm_si =  0.0;
d_qgate_dV_e_si = (((d_qgate_dV_e_si+d_Qac0_dV_e_si)+d_Qsub0_dV_e_si)-d_qbulk_dV_e_si);
d_qgate_dV_b_si = (((d_qgate_dV_b_si+d_Qac0_dV_b_si)+d_Qsub0_dV_b_si)-d_qbulk_dV_b_si);
d_qgate_dV_b_GND = (((d_qgate_dV_b_GND+d_Qac0_dV_b_GND)+d_Qsub0_dV_b_GND)-d_qbulk_dV_b_GND);
d_qgate_dV_p_GND = (((d_qgate_dV_p_GND+d_Qac0_dV_p_GND)+d_Qsub0_dV_p_GND)-d_qbulk_dV_p_GND);
d_qgate_dTemp_t_GND = (((d_qgate_dTemp_t_GND+d_Qac0_dTemp_t_GND)+d_Qsub0_dTemp_t_GND)-d_qbulk_dTemp_t_GND);
d_qgate_dV_gi_si = (((d_qgate_dV_gi_si+d_Qac0_dV_gi_si)+d_Qsub0_dV_gi_si)-d_qbulk_dV_gi_si);
d_qgate_dV_di_si = (((d_qgate_dV_di_si+d_Qac0_dV_di_si)+d_Qsub0_dV_di_si)-d_qbulk_dV_di_si);
d_qgate_dV_sb_si = (((d_qgate_dV_sb_si+d_Qac0_dV_sb_si)+d_Qsub0_dV_sb_si)-d_qbulk_dV_sb_si);
d_qgate_dV_db_di = (((d_qgate_dV_db_di+d_Qac0_dV_db_di)+d_Qsub0_dV_db_di)-d_qbulk_dV_db_di);
d_qgate_dV_gi_p = (((d_qgate_dV_gi_p+d_Qac0_dV_gi_p)+d_Qsub0_dV_gi_p)-d_qbulk_dV_gi_p);
qgate = (((qgate+Qac0)+Qsub0)-qbulk);

d_qbody_dV_sb_si = ((d_qbulk_dV_sb_si-d_Qac0_dV_sb_si)-d_Qsub0_dV_sb_si);
d_qbody_dV_db_di = ((d_qbulk_dV_db_di-d_Qac0_dV_db_di)-d_Qsub0_dV_db_di);
d_qbody_dV_gi_p = ((d_qbulk_dV_gi_p-d_Qac0_dV_gi_p)-d_Qsub0_dV_gi_p);
d_qbody_dV_b_GND = (((d_qbulk_dV_b_GND-d_Qac0_dV_b_GND)-d_Qsub0_dV_b_GND)-d_Qe1_dV_b_GND);
d_qbody_dV_p_GND = (((d_qbulk_dV_p_GND-d_Qac0_dV_p_GND)-d_Qsub0_dV_p_GND)-d_Qe1_dV_p_GND);
d_qbody_dTemp_t_GND = (((d_qbulk_dTemp_t_GND-d_Qac0_dTemp_t_GND)-d_Qsub0_dTemp_t_GND)-d_Qe1_dTemp_t_GND);
d_qbody_dV_gi_si = (((d_qbulk_dV_gi_si-d_Qac0_dV_gi_si)-d_Qsub0_dV_gi_si)-d_Qe1_dV_gi_si);
d_qbody_dV_di_si = (((d_qbulk_dV_di_si-d_Qac0_dV_di_si)-d_Qsub0_dV_di_si)-d_Qe1_dV_di_si);
d_qbody_dV_e_si = (((d_qbulk_dV_e_si-d_Qac0_dV_e_si)-d_Qsub0_dV_e_si)-d_Qe1_dV_e_si);
d_qbody_dV_b_si = (((d_qbulk_dV_b_si-d_Qac0_dV_b_si)-d_Qsub0_dV_b_si)-d_Qe1_dV_b_si);
qbody = (((qbulk-Qac0)-Qsub0)-Qe1);

d_qsub_dV_b_si = d_Qe1_dV_b_si;
d_qsub_dV_gi_si = d_Qe1_dV_gi_si;
d_qsub_dV_b_GND = d_Qe1_dV_b_GND;
d_qsub_dV_p_GND = d_Qe1_dV_p_GND;
d_qsub_dTemp_t_GND = d_Qe1_dTemp_t_GND;
d_qsub_dV_e_si = d_Qe1_dV_e_si;
d_qsub_dV_di_si = d_Qe1_dV_di_si;
qsub = Qe1;

d_qdrn_dV_e_si = (-(((d_qgate_dV_e_si+d_qbody_dV_e_si)+d_qsub_dV_e_si)+d_qsrc_dV_e_si));
d_qdrn_dV_b_si = (-(((d_qgate_dV_b_si+d_qbody_dV_b_si)+d_qsub_dV_b_si)+d_qsrc_dV_b_si));
d_qdrn_dV_b_GND = (-(((d_qgate_dV_b_GND+d_qbody_dV_b_GND)+d_qsub_dV_b_GND)+d_qsrc_dV_b_GND));
d_qdrn_dV_p_GND = (-(((d_qgate_dV_p_GND+d_qbody_dV_p_GND)+d_qsub_dV_p_GND)+d_qsrc_dV_p_GND));
d_qdrn_dTemp_t_GND = (-(((d_qgate_dTemp_t_GND+d_qbody_dTemp_t_GND)+d_qsub_dTemp_t_GND)+d_qsrc_dTemp_t_GND));
d_qdrn_dV_gi_si = (-(((d_qgate_dV_gi_si+d_qbody_dV_gi_si)+d_qsub_dV_gi_si)+d_qsrc_dV_gi_si));
d_qdrn_dV_di_si = (-(((d_qgate_dV_di_si+d_qbody_dV_di_si)+d_qsub_dV_di_si)+d_qsrc_dV_di_si));
d_qdrn_dV_sb_si = (-((d_qgate_dV_sb_si+d_qbody_dV_sb_si)+d_qsrc_dV_sb_si));
d_qdrn_dV_db_di = (-((d_qgate_dV_db_di+d_qbody_dV_db_di)+d_qsrc_dV_db_di));
d_qdrn_dV_gi_p = (-((d_qgate_dV_gi_p+d_qbody_dV_gi_p)+d_qsrc_dV_gi_p));
qdrn = (-(((qgate+qbody)+qsub)+qsrc));
B4SOIqinv = (-qinoi);
}
else
{

d_Qac0_dV_gi_p = d_Qac0_dTemp_t_GND = d_Qac0_dV_p_GND = d_Qac0_dV_b_GND = d_Qac0_dV_db_di = d_Qac0_dV_sb_si = d_Qac0_dV_b_si = d_Qac0_dV_e_si = d_Qac0_dV_di_si = d_Qac0_dV_gi_si =  0.0;
Qac0 = 0.0;

d_Qsub0_dV_gi_p = d_Qsub0_dV_db_di = d_Qsub0_dV_sb_si = d_Qsub0_dV_b_si = d_Qsub0_dV_e_si = d_Qsub0_dV_di_si = d_Qsub0_dV_gi_si = d_Qsub0_dTemp_t_GND = d_Qsub0_dV_p_GND = d_Qsub0_dV_b_GND =  0.0;
Qsub0 = 0.0;

d_qsub_dV_di_si = d_qsub_dV_e_si = d_qsub_dTemp_t_GND = d_qsub_dV_p_GND = d_qsub_dV_b_GND = d_qsub_dV_gi_si = d_qsub_dV_b_si =  0.0;
qsub = 0.0;

d_qbody_dV_b_si = d_qbody_dV_e_si = d_qbody_dV_di_si = d_qbody_dV_gi_si = d_qbody_dTemp_t_GND = d_qbody_dV_p_GND = d_qbody_dV_b_GND = d_qbody_dV_gi_p = d_qbody_dV_db_di = d_qbody_dV_sb_si =  0.0;
qbody = 0.0;

d_qsrc_dV_gi_p = d_qsrc_dV_db_di = d_qsrc_dV_sb_si = d_qsrc_dV_di_si = d_qsrc_dV_gi_si = d_qsrc_dTemp_t_GND = d_qsrc_dV_p_GND = d_qsrc_dV_b_GND = d_qsrc_dV_b_si = d_qsrc_dV_e_si =  0.0;
qsrc = 0.0;

d_qdrn_dV_gi_p = d_qdrn_dV_db_di = d_qdrn_dV_sb_si = d_qdrn_dV_di_si = d_qdrn_dV_gi_si = d_qdrn_dTemp_t_GND = d_qdrn_dV_p_GND = d_qdrn_dV_b_GND = d_qdrn_dV_b_si = d_qdrn_dV_e_si =  0.0;
qdrn = 0.0;

d_qgate_dV_gm_si = d_qgate_dV_gi_p = d_qgate_dV_db_di = d_qgate_dV_sb_si = d_qgate_dV_di_si = d_qgate_dV_gi_si = d_qgate_dTemp_t_GND = d_qgate_dV_p_GND = d_qgate_dV_b_GND = d_qgate_dV_b_si = d_qgate_dV_e_si =  0.0;
qgate = 0.0;
}
}
if ((B4SOIsoiMod==2))
{

d_qjd_dTemp_t_GND = d_qjd_dV_p_GND = d_qjd_dV_b_GND = d_qjd_dV_gi_p = d_qjd_dV_db_di = d_qjd_dV_sb_si = d_qjd_dV_b_si = d_qjd_dV_e_si = d_qjd_dV_di_si = d_qjd_dV_gi_si =  0.0;
qjd = 0.0;

d_qjs_dTemp_t_GND = d_qjs_dV_p_GND = d_qjs_dV_b_GND = d_qjs_dV_gi_p = d_qjs_dV_db_di = d_qjs_dV_sb_si = d_qjs_dV_b_si = d_qjs_dV_e_si = d_qjs_dV_di_si = d_qjs_dV_gi_si =  0.0;
qjs = 0.0;
}
else
{

d_PhiBSWG_dTemp_t_GND = d_PhiBSWG_dV_p_GND = d_PhiBSWG_dV_b_GND =  0.0;
PhiBSWG = B4SOIGatesidewallJctSPotential;
dPhiBSWG_dT = (-B4SOItpbswg);

d_PhiBSWG_dV_b_GND = (d_PhiBSWG_dV_b_GND+dPhiBSWG_dT*d_DevTemp_dV_b_GND);
d_PhiBSWG_dV_p_GND = (d_PhiBSWG_dV_p_GND+dPhiBSWG_dT*d_DevTemp_dV_p_GND);
d_PhiBSWG_dTemp_t_GND = (d_PhiBSWG_dTemp_t_GND+dPhiBSWG_dT*d_DevTemp_dTemp_t_GND);
PhiBSWG = (PhiBSWG+(dPhiBSWG_dT*(DevTemp-B4SOItnom)));
MJSWG = B4SOIbodyJctGateSideSGradingCoeff;

d_cjsbs_dTemp_t_GND = d_cjsbs_dV_p_GND = d_cjsbs_dV_b_GND =  0.0;
cjsbs = ((((B4SOIunitLengthGateSidewallJctCapS*pParam_B4SOIwdiosCV)*B4SOItsi)*B4SOInf)/1e-7);
dcjsbs_dT = (cjsbs*B4SOItcjswg);

d_cjsbs_dV_b_GND = (d_cjsbs_dV_b_GND+dcjsbs_dT*d_DevTemp_dV_b_GND);
d_cjsbs_dV_p_GND = (d_cjsbs_dV_p_GND+dcjsbs_dT*d_DevTemp_dV_p_GND);
d_cjsbs_dTemp_t_GND = (d_cjsbs_dTemp_t_GND+dcjsbs_dT*d_DevTemp_dTemp_t_GND);
cjsbs = (cjsbs+(dcjsbs_dT*(DevTemp-B4SOItnom)));

d_cjdbs_dTemp_t_GND = d_cjdbs_dV_p_GND = d_cjdbs_dV_b_GND =  0.0;
cjdbs = ((((B4SOIunitLengthGateSidewallJctCapD*pParam_B4SOIwdiodCV)*B4SOItsi)*B4SOInf)/1e-7);
dcjdbs_dT = (cjdbs*B4SOItcjswgd);

d_cjdbs_dV_b_GND = (d_cjdbs_dV_b_GND+dcjdbs_dT*d_DevTemp_dV_b_GND);
d_cjdbs_dV_p_GND = (d_cjdbs_dV_p_GND+dcjdbs_dT*d_DevTemp_dV_p_GND);
d_cjdbs_dTemp_t_GND = (d_cjdbs_dTemp_t_GND+dcjdbs_dT*d_DevTemp_dTemp_t_GND);
cjdbs = (cjdbs+(dcjdbs_dT*(DevTemp-B4SOItnom)));

d_DioMax_dV_b_GND = 0.9*d_PhiBSWG_dV_b_GND;
d_DioMax_dV_p_GND = 0.9*d_PhiBSWG_dV_p_GND;
d_DioMax_dTemp_t_GND = 0.9*d_PhiBSWG_dTemp_t_GND;
DioMax = (0.9*PhiBSWG);

d_arg_dV_db_di =  0.0;
d_arg_dV_b_GND = (-((PhiBSWG*((vsbs>DioMax)?d_DioMax_dV_b_GND:0.0)-((vsbs>DioMax)?DioMax:vsbs)*d_PhiBSWG_dV_b_GND)/PhiBSWG/PhiBSWG));
d_arg_dV_p_GND = (-((PhiBSWG*((vsbs>DioMax)?d_DioMax_dV_p_GND:0.0)-((vsbs>DioMax)?DioMax:vsbs)*d_PhiBSWG_dV_p_GND)/PhiBSWG/PhiBSWG));
d_arg_dTemp_t_GND = (-((PhiBSWG*((vsbs>DioMax)?d_DioMax_dTemp_t_GND:0.0)-((vsbs>DioMax)?DioMax:vsbs)*d_PhiBSWG_dTemp_t_GND)/PhiBSWG/PhiBSWG));
d_arg_dV_sb_si = (-(((vsbs>DioMax)?0.0:d_vsbs_dV_sb_si)/PhiBSWG));
arg = (1.0-(((vsbs>DioMax)?DioMax:vsbs)/PhiBSWG));
if ((MJSWG==0.5))
{
{
double value_sqrt_0 = sqrt(arg);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_dT3_dVb_dV_db_di =  0.0;
d_dT3_dVb_dV_b_GND = (-(deriv_sqrt_0_d0*(d_arg_dV_b_GND))/value_sqrt_0/value_sqrt_0);
d_dT3_dVb_dV_p_GND = (-(deriv_sqrt_0_d0*(d_arg_dV_p_GND))/value_sqrt_0/value_sqrt_0);
d_dT3_dVb_dTemp_t_GND = (-(deriv_sqrt_0_d0*(d_arg_dTemp_t_GND))/value_sqrt_0/value_sqrt_0);
d_dT3_dVb_dV_sb_si = (-(deriv_sqrt_0_d0*(d_arg_dV_sb_si))/value_sqrt_0/value_sqrt_0);
dT3_dVb = (1.0/value_sqrt_0);
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(arg);
double value_exp_1 = exp(((-MJSWG)*evaluator_lln_0.getValues(arg)));
double  deriv_exp_1_d0 = value_exp_1;

d_dT3_dVb_dV_db_di =  0.0;
d_dT3_dVb_dV_b_GND = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dV_b_GND)));
d_dT3_dVb_dV_p_GND = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dV_p_GND)));
d_dT3_dVb_dTemp_t_GND = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dTemp_t_GND)));
d_dT3_dVb_dV_sb_si = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dV_sb_si)));
dT3_dVb = value_exp_1;
}
}

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = (((1.0-(arg*dT3_dVb))*d_PhiBSWG_dV_b_GND+(-(arg*d_dT3_dVb_dV_b_GND+d_arg_dV_b_GND*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
d_T3_dV_p_GND = (((1.0-(arg*dT3_dVb))*d_PhiBSWG_dV_p_GND+(-(arg*d_dT3_dVb_dV_p_GND+d_arg_dV_p_GND*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
d_T3_dTemp_t_GND = (((1.0-(arg*dT3_dVb))*d_PhiBSWG_dTemp_t_GND+(-(arg*d_dT3_dVb_dTemp_t_GND+d_arg_dTemp_t_GND*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
d_T3_dV_sb_si = ((-(arg*d_dT3_dVb_dV_sb_si+d_arg_dV_sb_si*dT3_dVb))*PhiBSWG/(1.0-MJSWG));
T3 = (((1.0-(arg*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
if ((vsbs>DioMax))
{

d_T3_dV_b_GND = (d_T3_dV_b_GND+(dT3_dVb*(-d_DioMax_dV_b_GND)+d_dT3_dVb_dV_b_GND*(vsbs-DioMax)));
d_T3_dV_p_GND = (d_T3_dV_p_GND+(dT3_dVb*(-d_DioMax_dV_p_GND)+d_dT3_dVb_dV_p_GND*(vsbs-DioMax)));
d_T3_dTemp_t_GND = (d_T3_dTemp_t_GND+(dT3_dVb*(-d_DioMax_dTemp_t_GND)+d_dT3_dVb_dTemp_t_GND*(vsbs-DioMax)));
d_T3_dV_sb_si = (d_T3_dV_sb_si+(dT3_dVb*d_vsbs_dV_sb_si+d_dT3_dVb_dV_sb_si*(vsbs-DioMax)));
d_T3_dV_db_di = (d_T3_dV_db_di+d_dT3_dVb_dV_db_di*(vsbs-DioMax));
T3 = (T3+(dT3_dVb*(vsbs-DioMax)));
}

d_qjs_dV_gi_si = (cjsbs*d_T3_dV_gi_si+B4SOItt*d_Ibsdif_dV_gi_si*B4SOInf);
d_qjs_dV_di_si = (cjsbs*d_T3_dV_di_si+B4SOItt*d_Ibsdif_dV_di_si*B4SOInf);
d_qjs_dV_e_si = (cjsbs*d_T3_dV_e_si+B4SOItt*d_Ibsdif_dV_e_si*B4SOInf);
d_qjs_dV_b_si = (cjsbs*d_T3_dV_b_si+B4SOItt*d_Ibsdif_dV_b_si*B4SOInf);
d_qjs_dV_sb_si = (cjsbs*d_T3_dV_sb_si+B4SOItt*d_Ibsdif_dV_sb_si*B4SOInf);
d_qjs_dV_db_di = cjsbs*d_T3_dV_db_di;
d_qjs_dV_gi_p = cjsbs*d_T3_dV_gi_p;
d_qjs_dV_b_GND = ((cjsbs*d_T3_dV_b_GND+d_cjsbs_dV_b_GND*T3)+B4SOItt*d_Ibsdif_dV_b_GND*B4SOInf);
d_qjs_dV_p_GND = ((cjsbs*d_T3_dV_p_GND+d_cjsbs_dV_p_GND*T3)+B4SOItt*d_Ibsdif_dV_p_GND*B4SOInf);
d_qjs_dTemp_t_GND = ((cjsbs*d_T3_dTemp_t_GND+d_cjsbs_dTemp_t_GND*T3)+B4SOItt*d_Ibsdif_dTemp_t_GND*B4SOInf);
qjs = ((cjsbs*T3)+((B4SOItt*Ibsdif)*B4SOInf));

d_PhiBSWG_dTemp_t_GND = d_PhiBSWG_dV_p_GND = d_PhiBSWG_dV_b_GND =  0.0;
PhiBSWG = B4SOIGatesidewallJctDPotential;
dPhiBSWG_dT = (-B4SOItpbswgd);

d_PhiBSWG_dV_b_GND = (d_PhiBSWG_dV_b_GND+dPhiBSWG_dT*d_DevTemp_dV_b_GND);
d_PhiBSWG_dV_p_GND = (d_PhiBSWG_dV_p_GND+dPhiBSWG_dT*d_DevTemp_dV_p_GND);
d_PhiBSWG_dTemp_t_GND = (d_PhiBSWG_dTemp_t_GND+dPhiBSWG_dT*d_DevTemp_dTemp_t_GND);
PhiBSWG = (PhiBSWG+(dPhiBSWG_dT*(DevTemp-B4SOItnom)));
MJSWG = B4SOIbodyJctGateSideDGradingCoeff;

d_DioMax_dV_b_GND = 0.9*d_PhiBSWG_dV_b_GND;
d_DioMax_dV_p_GND = 0.9*d_PhiBSWG_dV_p_GND;
d_DioMax_dTemp_t_GND = 0.9*d_PhiBSWG_dTemp_t_GND;
DioMax = (0.9*PhiBSWG);

d_arg_dV_sb_si =  0.0;
d_arg_dV_b_GND = (-((PhiBSWG*((vdbd>DioMax)?d_DioMax_dV_b_GND:0.0)-((vdbd>DioMax)?DioMax:vdbd)*d_PhiBSWG_dV_b_GND)/PhiBSWG/PhiBSWG));
d_arg_dV_p_GND = (-((PhiBSWG*((vdbd>DioMax)?d_DioMax_dV_p_GND:0.0)-((vdbd>DioMax)?DioMax:vdbd)*d_PhiBSWG_dV_p_GND)/PhiBSWG/PhiBSWG));
d_arg_dTemp_t_GND = (-((PhiBSWG*((vdbd>DioMax)?d_DioMax_dTemp_t_GND:0.0)-((vdbd>DioMax)?DioMax:vdbd)*d_PhiBSWG_dTemp_t_GND)/PhiBSWG/PhiBSWG));
d_arg_dV_db_di = (-(((vdbd>DioMax)?0.0:d_vdbd_dV_db_di)/PhiBSWG));
arg = (1.0-(((vdbd>DioMax)?DioMax:vdbd)/PhiBSWG));
if ((MJSWG==0.5))
{
{
double value_sqrt_0 = sqrt(arg);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_dT3_dVb_dV_b_GND = (-(deriv_sqrt_0_d0*(d_arg_dV_b_GND))/value_sqrt_0/value_sqrt_0);
d_dT3_dVb_dV_p_GND = (-(deriv_sqrt_0_d0*(d_arg_dV_p_GND))/value_sqrt_0/value_sqrt_0);
d_dT3_dVb_dTemp_t_GND = (-(deriv_sqrt_0_d0*(d_arg_dTemp_t_GND))/value_sqrt_0/value_sqrt_0);
d_dT3_dVb_dV_sb_si = (-(deriv_sqrt_0_d0*(d_arg_dV_sb_si))/value_sqrt_0/value_sqrt_0);
d_dT3_dVb_dV_db_di = (-(deriv_sqrt_0_d0*(d_arg_dV_db_di))/value_sqrt_0/value_sqrt_0);
dT3_dVb = (1.0/value_sqrt_0);
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(arg);
double value_exp_1 = exp(((-MJSWG)*evaluator_lln_0.getValues(arg)));
double  deriv_exp_1_d0 = value_exp_1;

d_dT3_dVb_dV_b_GND = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dV_b_GND)));
d_dT3_dVb_dV_p_GND = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dV_p_GND)));
d_dT3_dVb_dTemp_t_GND = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dTemp_t_GND)));
d_dT3_dVb_dV_sb_si = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dV_sb_si)));
d_dT3_dVb_dV_db_di = (deriv_exp_1_d0*((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dV_db_di)));
dT3_dVb = value_exp_1;
}
}

d_T3_dV_gi_p = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si =  0.0;
d_T3_dV_b_GND = (((1.0-(arg*dT3_dVb))*d_PhiBSWG_dV_b_GND+(-(arg*d_dT3_dVb_dV_b_GND+d_arg_dV_b_GND*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
d_T3_dV_p_GND = (((1.0-(arg*dT3_dVb))*d_PhiBSWG_dV_p_GND+(-(arg*d_dT3_dVb_dV_p_GND+d_arg_dV_p_GND*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
d_T3_dTemp_t_GND = (((1.0-(arg*dT3_dVb))*d_PhiBSWG_dTemp_t_GND+(-(arg*d_dT3_dVb_dTemp_t_GND+d_arg_dTemp_t_GND*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
d_T3_dV_sb_si = ((-(arg*d_dT3_dVb_dV_sb_si+d_arg_dV_sb_si*dT3_dVb))*PhiBSWG/(1.0-MJSWG));
d_T3_dV_db_di = ((-(arg*d_dT3_dVb_dV_db_di+d_arg_dV_db_di*dT3_dVb))*PhiBSWG/(1.0-MJSWG));
T3 = (((1.0-(arg*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
if ((vdbd>DioMax))
{

d_T3_dV_b_GND = (d_T3_dV_b_GND+(dT3_dVb*(-d_DioMax_dV_b_GND)+d_dT3_dVb_dV_b_GND*(vdbd-DioMax)));
d_T3_dV_p_GND = (d_T3_dV_p_GND+(dT3_dVb*(-d_DioMax_dV_p_GND)+d_dT3_dVb_dV_p_GND*(vdbd-DioMax)));
d_T3_dTemp_t_GND = (d_T3_dTemp_t_GND+(dT3_dVb*(-d_DioMax_dTemp_t_GND)+d_dT3_dVb_dTemp_t_GND*(vdbd-DioMax)));
d_T3_dV_sb_si = (d_T3_dV_sb_si+d_dT3_dVb_dV_sb_si*(vdbd-DioMax));
d_T3_dV_db_di = (d_T3_dV_db_di+(dT3_dVb*d_vdbd_dV_db_di+d_dT3_dVb_dV_db_di*(vdbd-DioMax)));
T3 = (T3+(dT3_dVb*(vdbd-DioMax)));
}

d_qjd_dV_gi_si = (cjdbs*d_T3_dV_gi_si+B4SOItt*d_Ibddif_dV_gi_si*B4SOInf);
d_qjd_dV_di_si = (cjdbs*d_T3_dV_di_si+B4SOItt*d_Ibddif_dV_di_si*B4SOInf);
d_qjd_dV_e_si = (cjdbs*d_T3_dV_e_si+B4SOItt*d_Ibddif_dV_e_si*B4SOInf);
d_qjd_dV_b_si = (cjdbs*d_T3_dV_b_si+B4SOItt*d_Ibddif_dV_b_si*B4SOInf);
d_qjd_dV_sb_si = (cjdbs*d_T3_dV_sb_si+B4SOItt*d_Ibddif_dV_sb_si*B4SOInf);
d_qjd_dV_db_di = (cjdbs*d_T3_dV_db_di+B4SOItt*d_Ibddif_dV_db_di*B4SOInf);
d_qjd_dV_gi_p = cjdbs*d_T3_dV_gi_p;
d_qjd_dV_b_GND = ((cjdbs*d_T3_dV_b_GND+d_cjdbs_dV_b_GND*T3)+B4SOItt*d_Ibddif_dV_b_GND*B4SOInf);
d_qjd_dV_p_GND = ((cjdbs*d_T3_dV_p_GND+d_cjdbs_dV_p_GND*T3)+B4SOItt*d_Ibddif_dV_p_GND*B4SOInf);
d_qjd_dTemp_t_GND = ((cjdbs*d_T3_dTemp_t_GND+d_cjdbs_dTemp_t_GND*T3)+B4SOItt*d_Ibddif_dTemp_t_GND*B4SOInf);
qjd = ((cjdbs*T3)+((B4SOItt*Ibddif)*B4SOInf));
}

d_T10_dV_db_di = d_T10_dV_sb_si = d_T10_dV_b_si = d_T10_dV_di_si = d_T10_dV_gi_si = d_T10_dTemp_t_GND = d_T10_dV_p_GND = d_T10_dV_b_GND =  0.0;
d_T10_dV_e_si = (-B4SOItype)*d_ves_dV_e_si;
T10 = ((-B4SOItype)*ves);

d_T11_dV_gi_p = d_T11_dV_db_di = d_T11_dV_sb_si = d_T11_dV_b_si = d_T11_dV_gi_si = d_T11_dTemp_t_GND = d_T11_dV_p_GND = d_T11_dV_b_GND =  0.0;
d_T11_dV_e_si = B4SOItype*(-d_ves_dV_e_si);
d_T11_dV_di_si = B4SOItype*d_vds_dV_di_si;
T11 = (B4SOItype*(vds-ves));
if ((B4SOIcsdmin!=0.0))
{
if ((((pParam_B4SOInsub>0)&&(B4SOItype>0))||((pParam_B4SOInsub<0)&&(B4SOItype<0))))
{
if ((T10<pParam_B4SOIvsdfb))
{

d_B4SOIqse_dV_gi_p =  0.0;
d_B4SOIqse_dV_b_GND = B4SOIcsbox*d_T10_dV_b_GND;
d_B4SOIqse_dV_p_GND = B4SOIcsbox*d_T10_dV_p_GND;
d_B4SOIqse_dTemp_t_GND = B4SOIcsbox*d_T10_dTemp_t_GND;
d_B4SOIqse_dV_gi_si = B4SOIcsbox*d_T10_dV_gi_si;
d_B4SOIqse_dV_di_si = B4SOIcsbox*d_T10_dV_di_si;
d_B4SOIqse_dV_e_si = B4SOIcsbox*d_T10_dV_e_si;
d_B4SOIqse_dV_b_si = B4SOIcsbox*d_T10_dV_b_si;
d_B4SOIqse_dV_sb_si = B4SOIcsbox*d_T10_dV_sb_si;
d_B4SOIqse_dV_db_di = B4SOIcsbox*d_T10_dV_db_di;
B4SOIqse = (B4SOIcsbox*(T10-pParam_B4SOIvsdfb));
}
else
{
if ((T10<pParam_B4SOIsdt1))
{

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = d_T10_dV_b_GND;
d_T0_dV_p_GND = d_T10_dV_p_GND;
d_T0_dTemp_t_GND = d_T10_dTemp_t_GND;
d_T0_dV_gi_si = d_T10_dV_gi_si;
d_T0_dV_di_si = d_T10_dV_di_si;
d_T0_dV_e_si = d_T10_dV_e_si;
d_T0_dV_b_si = d_T10_dV_b_si;
d_T0_dV_sb_si = d_T10_dV_sb_si;
d_T0_dV_db_di = d_T10_dV_db_di;
T0 = (T10-pParam_B4SOIvsdfb);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqse_dV_b_GND = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_b_GND)+d_T0_dV_b_GND*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_p_GND = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_p_GND)+d_T0_dV_p_GND*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dTemp_t_GND = (T0*(-(pParam_B4SOIst2/3)*d_T1_dTemp_t_GND)+d_T0_dTemp_t_GND*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_e_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_e_si)+d_T0_dV_e_si*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_gi_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_gi_si)+d_T0_dV_gi_si*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_di_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_di_si)+d_T0_dV_di_si*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_b_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_b_si)+d_T0_dV_b_si*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_sb_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_sb_si)+d_T0_dV_sb_si*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_db_di = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_db_di)+d_T0_dV_db_di*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_gi_p = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_gi_p)+d_T0_dV_gi_p*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
B4SOIqse = (T0*(B4SOIcsbox-((pParam_B4SOIst2/3)*T1)));
}
else
{
if ((T10<pParam_B4SOIvsdth))
{

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = d_T10_dV_b_GND;
d_T0_dV_p_GND = d_T10_dV_p_GND;
d_T0_dTemp_t_GND = d_T10_dTemp_t_GND;
d_T0_dV_gi_si = d_T10_dV_gi_si;
d_T0_dV_di_si = d_T10_dV_di_si;
d_T0_dV_e_si = d_T10_dV_e_si;
d_T0_dV_b_si = d_T10_dV_b_si;
d_T0_dV_sb_si = d_T10_dV_sb_si;
d_T0_dV_db_di = d_T10_dV_db_di;
T0 = (T10-pParam_B4SOIvsdth);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqse_dV_gi_p = (((pParam_B4SOIst3/3)*T0)*d_T1_dV_gi_p+(pParam_B4SOIst3/3)*d_T0_dV_gi_p*T1);
d_B4SOIqse_dV_b_GND = (B4SOIcsmin*d_T10_dV_b_GND+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_b_GND+(pParam_B4SOIst3/3)*d_T0_dV_b_GND*T1));
d_B4SOIqse_dV_p_GND = (B4SOIcsmin*d_T10_dV_p_GND+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_p_GND+(pParam_B4SOIst3/3)*d_T0_dV_p_GND*T1));
d_B4SOIqse_dTemp_t_GND = (B4SOIcsmin*d_T10_dTemp_t_GND+(((pParam_B4SOIst3/3)*T0)*d_T1_dTemp_t_GND+(pParam_B4SOIst3/3)*d_T0_dTemp_t_GND*T1));
d_B4SOIqse_dV_gi_si = (B4SOIcsmin*d_T10_dV_gi_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_gi_si+(pParam_B4SOIst3/3)*d_T0_dV_gi_si*T1));
d_B4SOIqse_dV_di_si = (B4SOIcsmin*d_T10_dV_di_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_di_si+(pParam_B4SOIst3/3)*d_T0_dV_di_si*T1));
d_B4SOIqse_dV_e_si = (B4SOIcsmin*d_T10_dV_e_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_e_si+(pParam_B4SOIst3/3)*d_T0_dV_e_si*T1));
d_B4SOIqse_dV_b_si = (B4SOIcsmin*d_T10_dV_b_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_b_si+(pParam_B4SOIst3/3)*d_T0_dV_b_si*T1));
d_B4SOIqse_dV_sb_si = (B4SOIcsmin*d_T10_dV_sb_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_sb_si+(pParam_B4SOIst3/3)*d_T0_dV_sb_si*T1));
d_B4SOIqse_dV_db_di = (B4SOIcsmin*d_T10_dV_db_di+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_db_di+(pParam_B4SOIst3/3)*d_T0_dV_db_di*T1));
B4SOIqse = (((B4SOIcsmin*T10)+B4SOIst4)+(((pParam_B4SOIst3/3)*T0)*T1));
}
else
{

d_B4SOIqse_dV_gi_p =  0.0;
d_B4SOIqse_dV_b_GND = B4SOIcsmin*d_T10_dV_b_GND;
d_B4SOIqse_dV_p_GND = B4SOIcsmin*d_T10_dV_p_GND;
d_B4SOIqse_dTemp_t_GND = B4SOIcsmin*d_T10_dTemp_t_GND;
d_B4SOIqse_dV_gi_si = B4SOIcsmin*d_T10_dV_gi_si;
d_B4SOIqse_dV_di_si = B4SOIcsmin*d_T10_dV_di_si;
d_B4SOIqse_dV_e_si = B4SOIcsmin*d_T10_dV_e_si;
d_B4SOIqse_dV_b_si = B4SOIcsmin*d_T10_dV_b_si;
d_B4SOIqse_dV_sb_si = B4SOIcsmin*d_T10_dV_sb_si;
d_B4SOIqse_dV_db_di = B4SOIcsmin*d_T10_dV_db_di;
B4SOIqse = ((B4SOIcsmin*T10)+B4SOIst4);
}
}
}
}
else
{
if ((T10<pParam_B4SOIvsdth))
{

d_B4SOIqse_dV_gi_p =  0.0;
d_B4SOIqse_dV_b_GND = B4SOIcsmin*d_T10_dV_b_GND;
d_B4SOIqse_dV_p_GND = B4SOIcsmin*d_T10_dV_p_GND;
d_B4SOIqse_dTemp_t_GND = B4SOIcsmin*d_T10_dTemp_t_GND;
d_B4SOIqse_dV_gi_si = B4SOIcsmin*d_T10_dV_gi_si;
d_B4SOIqse_dV_di_si = B4SOIcsmin*d_T10_dV_di_si;
d_B4SOIqse_dV_e_si = B4SOIcsmin*d_T10_dV_e_si;
d_B4SOIqse_dV_b_si = B4SOIcsmin*d_T10_dV_b_si;
d_B4SOIqse_dV_sb_si = B4SOIcsmin*d_T10_dV_sb_si;
d_B4SOIqse_dV_db_di = B4SOIcsmin*d_T10_dV_db_di;
B4SOIqse = (B4SOIcsmin*(T10-pParam_B4SOIvsdth));
}
else
{
if ((T10<pParam_B4SOIsdt1))
{

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = d_T10_dV_b_GND;
d_T0_dV_p_GND = d_T10_dV_p_GND;
d_T0_dTemp_t_GND = d_T10_dTemp_t_GND;
d_T0_dV_gi_si = d_T10_dV_gi_si;
d_T0_dV_di_si = d_T10_dV_di_si;
d_T0_dV_e_si = d_T10_dV_e_si;
d_T0_dV_b_si = d_T10_dV_b_si;
d_T0_dV_sb_si = d_T10_dV_sb_si;
d_T0_dV_db_di = d_T10_dV_db_di;
T0 = (T10-pParam_B4SOIvsdth);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqse_dV_b_GND = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_b_GND)+d_T0_dV_b_GND*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_p_GND = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_p_GND)+d_T0_dV_p_GND*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dTemp_t_GND = (T0*(-(pParam_B4SOIst2/3)*d_T1_dTemp_t_GND)+d_T0_dTemp_t_GND*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_e_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_e_si)+d_T0_dV_e_si*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_gi_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_gi_si)+d_T0_dV_gi_si*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_di_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_di_si)+d_T0_dV_di_si*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_b_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_b_si)+d_T0_dV_b_si*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_sb_si = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_sb_si)+d_T0_dV_sb_si*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_db_di = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_db_di)+d_T0_dV_db_di*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
d_B4SOIqse_dV_gi_p = (T0*(-(pParam_B4SOIst2/3)*d_T1_dV_gi_p)+d_T0_dV_gi_p*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
B4SOIqse = (T0*(B4SOIcsmin-((pParam_B4SOIst2/3)*T1)));
}
else
{
if ((T10<pParam_B4SOIvsdfb))
{

d_T0_dV_gm_si = d_T0_dV_gi_p =  0.0;
d_T0_dV_b_GND = d_T10_dV_b_GND;
d_T0_dV_p_GND = d_T10_dV_p_GND;
d_T0_dTemp_t_GND = d_T10_dTemp_t_GND;
d_T0_dV_gi_si = d_T10_dV_gi_si;
d_T0_dV_di_si = d_T10_dV_di_si;
d_T0_dV_e_si = d_T10_dV_e_si;
d_T0_dV_b_si = d_T10_dV_b_si;
d_T0_dV_sb_si = d_T10_dV_sb_si;
d_T0_dV_db_di = d_T10_dV_db_di;
T0 = (T10-pParam_B4SOIvsdfb);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqse_dV_gi_p = (((pParam_B4SOIst3/3)*T0)*d_T1_dV_gi_p+(pParam_B4SOIst3/3)*d_T0_dV_gi_p*T1);
d_B4SOIqse_dV_b_GND = (B4SOIcsbox*d_T10_dV_b_GND+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_b_GND+(pParam_B4SOIst3/3)*d_T0_dV_b_GND*T1));
d_B4SOIqse_dV_p_GND = (B4SOIcsbox*d_T10_dV_p_GND+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_p_GND+(pParam_B4SOIst3/3)*d_T0_dV_p_GND*T1));
d_B4SOIqse_dTemp_t_GND = (B4SOIcsbox*d_T10_dTemp_t_GND+(((pParam_B4SOIst3/3)*T0)*d_T1_dTemp_t_GND+(pParam_B4SOIst3/3)*d_T0_dTemp_t_GND*T1));
d_B4SOIqse_dV_gi_si = (B4SOIcsbox*d_T10_dV_gi_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_gi_si+(pParam_B4SOIst3/3)*d_T0_dV_gi_si*T1));
d_B4SOIqse_dV_di_si = (B4SOIcsbox*d_T10_dV_di_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_di_si+(pParam_B4SOIst3/3)*d_T0_dV_di_si*T1));
d_B4SOIqse_dV_e_si = (B4SOIcsbox*d_T10_dV_e_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_e_si+(pParam_B4SOIst3/3)*d_T0_dV_e_si*T1));
d_B4SOIqse_dV_b_si = (B4SOIcsbox*d_T10_dV_b_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_b_si+(pParam_B4SOIst3/3)*d_T0_dV_b_si*T1));
d_B4SOIqse_dV_sb_si = (B4SOIcsbox*d_T10_dV_sb_si+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_sb_si+(pParam_B4SOIst3/3)*d_T0_dV_sb_si*T1));
d_B4SOIqse_dV_db_di = (B4SOIcsbox*d_T10_dV_db_di+(((pParam_B4SOIst3/3)*T0)*d_T1_dV_db_di+(pParam_B4SOIst3/3)*d_T0_dV_db_di*T1));
B4SOIqse = (((B4SOIcsbox*T10)+B4SOIst4)+(((pParam_B4SOIst3/3)*T0)*T1));
}
else
{

d_B4SOIqse_dV_gi_p =  0.0;
d_B4SOIqse_dV_b_GND = B4SOIcsbox*d_T10_dV_b_GND;
d_B4SOIqse_dV_p_GND = B4SOIcsbox*d_T10_dV_p_GND;
d_B4SOIqse_dTemp_t_GND = B4SOIcsbox*d_T10_dTemp_t_GND;
d_B4SOIqse_dV_gi_si = B4SOIcsbox*d_T10_dV_gi_si;
d_B4SOIqse_dV_di_si = B4SOIcsbox*d_T10_dV_di_si;
d_B4SOIqse_dV_e_si = B4SOIcsbox*d_T10_dV_e_si;
d_B4SOIqse_dV_b_si = B4SOIcsbox*d_T10_dV_b_si;
d_B4SOIqse_dV_sb_si = B4SOIcsbox*d_T10_dV_sb_si;
d_B4SOIqse_dV_db_di = B4SOIcsbox*d_T10_dV_db_di;
B4SOIqse = ((B4SOIcsbox*T10)+B4SOIst4);
}
}
}
}
if ((((pParam_B4SOInsub>0)&&(B4SOItype>0))||((pParam_B4SOInsub<0)&&(B4SOItype<0))))
{
if ((T11<pParam_B4SOIvsdfb))
{

d_B4SOIqde_dV_b_GND = B4SOIcdbox*d_T11_dV_b_GND;
d_B4SOIqde_dV_p_GND = B4SOIcdbox*d_T11_dV_p_GND;
d_B4SOIqde_dTemp_t_GND = B4SOIcdbox*d_T11_dTemp_t_GND;
d_B4SOIqde_dV_gi_si = B4SOIcdbox*d_T11_dV_gi_si;
d_B4SOIqde_dV_di_si = B4SOIcdbox*d_T11_dV_di_si;
d_B4SOIqde_dV_e_si = B4SOIcdbox*d_T11_dV_e_si;
d_B4SOIqde_dV_b_si = B4SOIcdbox*d_T11_dV_b_si;
d_B4SOIqde_dV_sb_si = B4SOIcdbox*d_T11_dV_sb_si;
d_B4SOIqde_dV_db_di = B4SOIcdbox*d_T11_dV_db_di;
d_B4SOIqde_dV_gi_p = B4SOIcdbox*d_T11_dV_gi_p;
B4SOIqde = (B4SOIcdbox*(T11-pParam_B4SOIvsdfb));
}
else
{
if ((T11<pParam_B4SOIsdt1))
{

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = d_T11_dV_b_GND;
d_T0_dV_p_GND = d_T11_dV_p_GND;
d_T0_dTemp_t_GND = d_T11_dTemp_t_GND;
d_T0_dV_gi_si = d_T11_dV_gi_si;
d_T0_dV_di_si = d_T11_dV_di_si;
d_T0_dV_e_si = d_T11_dV_e_si;
d_T0_dV_b_si = d_T11_dV_b_si;
d_T0_dV_sb_si = d_T11_dV_sb_si;
d_T0_dV_db_di = d_T11_dV_db_di;
d_T0_dV_gi_p = d_T11_dV_gi_p;
T0 = (T11-pParam_B4SOIvsdfb);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqde_dV_b_GND = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_b_GND)+d_T0_dV_b_GND*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_p_GND = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_p_GND)+d_T0_dV_p_GND*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dTemp_t_GND = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dTemp_t_GND)+d_T0_dTemp_t_GND*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_e_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_e_si)+d_T0_dV_e_si*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_gi_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_gi_si)+d_T0_dV_gi_si*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_di_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_di_si)+d_T0_dV_di_si*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_b_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_b_si)+d_T0_dV_b_si*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_sb_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_sb_si)+d_T0_dV_sb_si*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_db_di = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_db_di)+d_T0_dV_db_di*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_gi_p = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_gi_p)+d_T0_dV_gi_p*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
B4SOIqde = (T0*(B4SOIcdbox-((pParam_B4SOIdt2/3)*T1)));
}
else
{
if ((T11<pParam_B4SOIvsdth))
{

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = d_T11_dV_b_GND;
d_T0_dV_p_GND = d_T11_dV_p_GND;
d_T0_dTemp_t_GND = d_T11_dTemp_t_GND;
d_T0_dV_gi_si = d_T11_dV_gi_si;
d_T0_dV_di_si = d_T11_dV_di_si;
d_T0_dV_e_si = d_T11_dV_e_si;
d_T0_dV_b_si = d_T11_dV_b_si;
d_T0_dV_sb_si = d_T11_dV_sb_si;
d_T0_dV_db_di = d_T11_dV_db_di;
d_T0_dV_gi_p = d_T11_dV_gi_p;
T0 = (T11-pParam_B4SOIvsdth);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqde_dV_b_GND = (B4SOIcdmin*d_T11_dV_b_GND+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_b_GND+(pParam_B4SOIdt3/3)*d_T0_dV_b_GND*T1));
d_B4SOIqde_dV_p_GND = (B4SOIcdmin*d_T11_dV_p_GND+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_p_GND+(pParam_B4SOIdt3/3)*d_T0_dV_p_GND*T1));
d_B4SOIqde_dTemp_t_GND = (B4SOIcdmin*d_T11_dTemp_t_GND+(((pParam_B4SOIdt3/3)*T0)*d_T1_dTemp_t_GND+(pParam_B4SOIdt3/3)*d_T0_dTemp_t_GND*T1));
d_B4SOIqde_dV_gi_si = (B4SOIcdmin*d_T11_dV_gi_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_gi_si+(pParam_B4SOIdt3/3)*d_T0_dV_gi_si*T1));
d_B4SOIqde_dV_di_si = (B4SOIcdmin*d_T11_dV_di_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_di_si+(pParam_B4SOIdt3/3)*d_T0_dV_di_si*T1));
d_B4SOIqde_dV_e_si = (B4SOIcdmin*d_T11_dV_e_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_e_si+(pParam_B4SOIdt3/3)*d_T0_dV_e_si*T1));
d_B4SOIqde_dV_b_si = (B4SOIcdmin*d_T11_dV_b_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_b_si+(pParam_B4SOIdt3/3)*d_T0_dV_b_si*T1));
d_B4SOIqde_dV_sb_si = (B4SOIcdmin*d_T11_dV_sb_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_sb_si+(pParam_B4SOIdt3/3)*d_T0_dV_sb_si*T1));
d_B4SOIqde_dV_db_di = (B4SOIcdmin*d_T11_dV_db_di+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_db_di+(pParam_B4SOIdt3/3)*d_T0_dV_db_di*T1));
d_B4SOIqde_dV_gi_p = (B4SOIcdmin*d_T11_dV_gi_p+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_gi_p+(pParam_B4SOIdt3/3)*d_T0_dV_gi_p*T1));
B4SOIqde = (((B4SOIcdmin*T11)+B4SOIdt4)+(((pParam_B4SOIdt3/3)*T0)*T1));
}
else
{

d_B4SOIqde_dV_b_GND = B4SOIcdmin*d_T11_dV_b_GND;
d_B4SOIqde_dV_p_GND = B4SOIcdmin*d_T11_dV_p_GND;
d_B4SOIqde_dTemp_t_GND = B4SOIcdmin*d_T11_dTemp_t_GND;
d_B4SOIqde_dV_gi_si = B4SOIcdmin*d_T11_dV_gi_si;
d_B4SOIqde_dV_di_si = B4SOIcdmin*d_T11_dV_di_si;
d_B4SOIqde_dV_e_si = B4SOIcdmin*d_T11_dV_e_si;
d_B4SOIqde_dV_b_si = B4SOIcdmin*d_T11_dV_b_si;
d_B4SOIqde_dV_sb_si = B4SOIcdmin*d_T11_dV_sb_si;
d_B4SOIqde_dV_db_di = B4SOIcdmin*d_T11_dV_db_di;
d_B4SOIqde_dV_gi_p = B4SOIcdmin*d_T11_dV_gi_p;
B4SOIqde = ((B4SOIcdmin*T11)+B4SOIdt4);
}
}
}
}
else
{
if ((T11<pParam_B4SOIvsdth))
{

d_B4SOIqde_dV_b_GND = B4SOIcdmin*d_T11_dV_b_GND;
d_B4SOIqde_dV_p_GND = B4SOIcdmin*d_T11_dV_p_GND;
d_B4SOIqde_dTemp_t_GND = B4SOIcdmin*d_T11_dTemp_t_GND;
d_B4SOIqde_dV_gi_si = B4SOIcdmin*d_T11_dV_gi_si;
d_B4SOIqde_dV_di_si = B4SOIcdmin*d_T11_dV_di_si;
d_B4SOIqde_dV_e_si = B4SOIcdmin*d_T11_dV_e_si;
d_B4SOIqde_dV_b_si = B4SOIcdmin*d_T11_dV_b_si;
d_B4SOIqde_dV_sb_si = B4SOIcdmin*d_T11_dV_sb_si;
d_B4SOIqde_dV_db_di = B4SOIcdmin*d_T11_dV_db_di;
d_B4SOIqde_dV_gi_p = B4SOIcdmin*d_T11_dV_gi_p;
B4SOIqde = (B4SOIcdmin*(T11-pParam_B4SOIvsdth));
}
else
{
if ((T11<pParam_B4SOIsdt1))
{

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = d_T11_dV_b_GND;
d_T0_dV_p_GND = d_T11_dV_p_GND;
d_T0_dTemp_t_GND = d_T11_dTemp_t_GND;
d_T0_dV_gi_si = d_T11_dV_gi_si;
d_T0_dV_di_si = d_T11_dV_di_si;
d_T0_dV_e_si = d_T11_dV_e_si;
d_T0_dV_b_si = d_T11_dV_b_si;
d_T0_dV_sb_si = d_T11_dV_sb_si;
d_T0_dV_db_di = d_T11_dV_db_di;
d_T0_dV_gi_p = d_T11_dV_gi_p;
T0 = (T11-pParam_B4SOIvsdth);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqde_dV_b_GND = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_b_GND)+d_T0_dV_b_GND*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_p_GND = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_p_GND)+d_T0_dV_p_GND*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dTemp_t_GND = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dTemp_t_GND)+d_T0_dTemp_t_GND*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_e_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_e_si)+d_T0_dV_e_si*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_gi_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_gi_si)+d_T0_dV_gi_si*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_di_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_di_si)+d_T0_dV_di_si*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_b_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_b_si)+d_T0_dV_b_si*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_sb_si = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_sb_si)+d_T0_dV_sb_si*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_db_di = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_db_di)+d_T0_dV_db_di*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
d_B4SOIqde_dV_gi_p = (T0*(-(pParam_B4SOIdt2/3)*d_T1_dV_gi_p)+d_T0_dV_gi_p*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
B4SOIqde = (T0*(B4SOIcdmin-((pParam_B4SOIdt2/3)*T1)));
}
else
{
if ((T11<pParam_B4SOIvsdfb))
{

d_T0_dV_gm_si =  0.0;
d_T0_dV_b_GND = d_T11_dV_b_GND;
d_T0_dV_p_GND = d_T11_dV_p_GND;
d_T0_dTemp_t_GND = d_T11_dTemp_t_GND;
d_T0_dV_gi_si = d_T11_dV_gi_si;
d_T0_dV_di_si = d_T11_dV_di_si;
d_T0_dV_e_si = d_T11_dV_e_si;
d_T0_dV_b_si = d_T11_dV_b_si;
d_T0_dV_sb_si = d_T11_dV_sb_si;
d_T0_dV_db_di = d_T11_dV_db_di;
d_T0_dV_gi_p = d_T11_dV_gi_p;
T0 = (T11-pParam_B4SOIvsdfb);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T1_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T1_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T1_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T1_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T1_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T1_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T1_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T1_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T1_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
T1 = (T0*T0);

d_B4SOIqde_dV_b_GND = (B4SOIcdbox*d_T11_dV_b_GND+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_b_GND+(pParam_B4SOIdt3/3)*d_T0_dV_b_GND*T1));
d_B4SOIqde_dV_p_GND = (B4SOIcdbox*d_T11_dV_p_GND+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_p_GND+(pParam_B4SOIdt3/3)*d_T0_dV_p_GND*T1));
d_B4SOIqde_dTemp_t_GND = (B4SOIcdbox*d_T11_dTemp_t_GND+(((pParam_B4SOIdt3/3)*T0)*d_T1_dTemp_t_GND+(pParam_B4SOIdt3/3)*d_T0_dTemp_t_GND*T1));
d_B4SOIqde_dV_gi_si = (B4SOIcdbox*d_T11_dV_gi_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_gi_si+(pParam_B4SOIdt3/3)*d_T0_dV_gi_si*T1));
d_B4SOIqde_dV_di_si = (B4SOIcdbox*d_T11_dV_di_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_di_si+(pParam_B4SOIdt3/3)*d_T0_dV_di_si*T1));
d_B4SOIqde_dV_e_si = (B4SOIcdbox*d_T11_dV_e_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_e_si+(pParam_B4SOIdt3/3)*d_T0_dV_e_si*T1));
d_B4SOIqde_dV_b_si = (B4SOIcdbox*d_T11_dV_b_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_b_si+(pParam_B4SOIdt3/3)*d_T0_dV_b_si*T1));
d_B4SOIqde_dV_sb_si = (B4SOIcdbox*d_T11_dV_sb_si+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_sb_si+(pParam_B4SOIdt3/3)*d_T0_dV_sb_si*T1));
d_B4SOIqde_dV_db_di = (B4SOIcdbox*d_T11_dV_db_di+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_db_di+(pParam_B4SOIdt3/3)*d_T0_dV_db_di*T1));
d_B4SOIqde_dV_gi_p = (B4SOIcdbox*d_T11_dV_gi_p+(((pParam_B4SOIdt3/3)*T0)*d_T1_dV_gi_p+(pParam_B4SOIdt3/3)*d_T0_dV_gi_p*T1));
B4SOIqde = (((B4SOIcdbox*T11)+B4SOIdt4)+(((pParam_B4SOIdt3/3)*T0)*T1));
}
else
{

d_B4SOIqde_dV_b_GND = B4SOIcdbox*d_T11_dV_b_GND;
d_B4SOIqde_dV_p_GND = B4SOIcdbox*d_T11_dV_p_GND;
d_B4SOIqde_dTemp_t_GND = B4SOIcdbox*d_T11_dTemp_t_GND;
d_B4SOIqde_dV_gi_si = B4SOIcdbox*d_T11_dV_gi_si;
d_B4SOIqde_dV_di_si = B4SOIcdbox*d_T11_dV_di_si;
d_B4SOIqde_dV_e_si = B4SOIcdbox*d_T11_dV_e_si;
d_B4SOIqde_dV_b_si = B4SOIcdbox*d_T11_dV_b_si;
d_B4SOIqde_dV_sb_si = B4SOIcdbox*d_T11_dV_sb_si;
d_B4SOIqde_dV_db_di = B4SOIcdbox*d_T11_dV_db_di;
d_B4SOIqde_dV_gi_p = B4SOIcdbox*d_T11_dV_gi_p;
B4SOIqde = ((B4SOIcdbox*T11)+B4SOIdt4);
}
}
}
}
}
else
{

d_B4SOIqse_dV_gi_p =  0.0;
d_B4SOIqse_dV_b_GND = B4SOIcsbox*d_T10_dV_b_GND;
d_B4SOIqse_dV_p_GND = B4SOIcsbox*d_T10_dV_p_GND;
d_B4SOIqse_dTemp_t_GND = B4SOIcsbox*d_T10_dTemp_t_GND;
d_B4SOIqse_dV_gi_si = B4SOIcsbox*d_T10_dV_gi_si;
d_B4SOIqse_dV_di_si = B4SOIcsbox*d_T10_dV_di_si;
d_B4SOIqse_dV_e_si = B4SOIcsbox*d_T10_dV_e_si;
d_B4SOIqse_dV_b_si = B4SOIcsbox*d_T10_dV_b_si;
d_B4SOIqse_dV_sb_si = B4SOIcsbox*d_T10_dV_sb_si;
d_B4SOIqse_dV_db_di = B4SOIcsbox*d_T10_dV_db_di;
B4SOIqse = (B4SOIcsbox*T10);

d_B4SOIqde_dV_b_GND = B4SOIcdbox*d_T11_dV_b_GND;
d_B4SOIqde_dV_p_GND = B4SOIcdbox*d_T11_dV_p_GND;
d_B4SOIqde_dTemp_t_GND = B4SOIcdbox*d_T11_dTemp_t_GND;
d_B4SOIqde_dV_gi_si = B4SOIcdbox*d_T11_dV_gi_si;
d_B4SOIqde_dV_di_si = B4SOIcdbox*d_T11_dV_di_si;
d_B4SOIqde_dV_e_si = B4SOIcdbox*d_T11_dV_e_si;
d_B4SOIqde_dV_b_si = B4SOIcdbox*d_T11_dV_b_si;
d_B4SOIqde_dV_sb_si = B4SOIcdbox*d_T11_dV_sb_si;
d_B4SOIqde_dV_db_di = B4SOIcdbox*d_T11_dV_db_di;
d_B4SOIqde_dV_gi_p = B4SOIcdbox*d_T11_dV_gi_p;
B4SOIqde = (B4SOIcdbox*T11);
}

d_B4SOIqse_dV_b_GND = (d_B4SOIqse_dV_b_GND+B4SOIcsesw*d_T10_dV_b_GND);
d_B4SOIqse_dV_p_GND = (d_B4SOIqse_dV_p_GND+B4SOIcsesw*d_T10_dV_p_GND);
d_B4SOIqse_dTemp_t_GND = (d_B4SOIqse_dTemp_t_GND+B4SOIcsesw*d_T10_dTemp_t_GND);
d_B4SOIqse_dV_gi_si = (d_B4SOIqse_dV_gi_si+B4SOIcsesw*d_T10_dV_gi_si);
d_B4SOIqse_dV_di_si = (d_B4SOIqse_dV_di_si+B4SOIcsesw*d_T10_dV_di_si);
d_B4SOIqse_dV_e_si = (d_B4SOIqse_dV_e_si+B4SOIcsesw*d_T10_dV_e_si);
d_B4SOIqse_dV_b_si = (d_B4SOIqse_dV_b_si+B4SOIcsesw*d_T10_dV_b_si);
d_B4SOIqse_dV_sb_si = (d_B4SOIqse_dV_sb_si+B4SOIcsesw*d_T10_dV_sb_si);
d_B4SOIqse_dV_db_di = (d_B4SOIqse_dV_db_di+B4SOIcsesw*d_T10_dV_db_di);
B4SOIqse = (B4SOIqse+(B4SOIcsesw*T10));

d_B4SOIqde_dV_b_GND = (d_B4SOIqde_dV_b_GND+B4SOIcdesw*d_T11_dV_b_GND);
d_B4SOIqde_dV_p_GND = (d_B4SOIqde_dV_p_GND+B4SOIcdesw*d_T11_dV_p_GND);
d_B4SOIqde_dTemp_t_GND = (d_B4SOIqde_dTemp_t_GND+B4SOIcdesw*d_T11_dTemp_t_GND);
d_B4SOIqde_dV_gi_si = (d_B4SOIqde_dV_gi_si+B4SOIcdesw*d_T11_dV_gi_si);
d_B4SOIqde_dV_di_si = (d_B4SOIqde_dV_di_si+B4SOIcdesw*d_T11_dV_di_si);
d_B4SOIqde_dV_e_si = (d_B4SOIqde_dV_e_si+B4SOIcdesw*d_T11_dV_e_si);
d_B4SOIqde_dV_b_si = (d_B4SOIqde_dV_b_si+B4SOIcdesw*d_T11_dV_b_si);
d_B4SOIqde_dV_sb_si = (d_B4SOIqde_dV_sb_si+B4SOIcdesw*d_T11_dV_sb_si);
d_B4SOIqde_dV_db_di = (d_B4SOIqde_dV_db_di+B4SOIcdesw*d_T11_dV_db_di);
d_B4SOIqde_dV_gi_p = (d_B4SOIqde_dV_gi_p+B4SOIcdesw*d_T11_dV_gi_p);
B4SOIqde = (B4SOIqde+(B4SOIcdesw*T11));
if ((B4SOIrgateMod==3))
{

d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = d_vgmd_dV_di_si;
d_T0_dV_gm_si = d_vgmd_dV_gm_si;
T0 = (vgmd+0.02);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_di_si = d_vgd_dV_di_si;
d_T0_dV_gi_si = d_vgd_dV_gi_si;
T0 = (vgd+0.02);
}
{
double value_sqrt_0 = sqrt(((T0*T0)+(4.0*0.02)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)));
d_T1_dV_gm_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gm_si+d_T0_dV_gm_si*T0)));
T1 = value_sqrt_0;
}

d_T2_dV_b_GND = 0.5*(d_T0_dV_b_GND-d_T1_dV_b_GND);
d_T2_dV_p_GND = 0.5*(d_T0_dV_p_GND-d_T1_dV_p_GND);
d_T2_dTemp_t_GND = 0.5*(d_T0_dTemp_t_GND-d_T1_dTemp_t_GND);
d_T2_dV_e_si = 0.5*(d_T0_dV_e_si-d_T1_dV_e_si);
d_T2_dV_gi_si = 0.5*(d_T0_dV_gi_si-d_T1_dV_gi_si);
d_T2_dV_di_si = 0.5*(d_T0_dV_di_si-d_T1_dV_di_si);
d_T2_dV_b_si = 0.5*(d_T0_dV_b_si-d_T1_dV_b_si);
d_T2_dV_sb_si = 0.5*(d_T0_dV_sb_si-d_T1_dV_sb_si);
d_T2_dV_db_di = 0.5*(d_T0_dV_db_di-d_T1_dV_db_di);
d_T2_dV_gi_p = 0.5*(d_T0_dV_gi_p-d_T1_dV_gi_p);
d_T2_dV_gm_si = 0.5*(d_T0_dV_gm_si-d_T1_dV_gm_si);
T2 = (0.5*(T0-T1));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = (pParam_B4SOIwdiodCV*pParam_B4SOIcgdl);
{
double value_sqrt_0 = sqrt((1.0-((4.0*T2)/pParam_B4SOIckappa)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T4_dV_b_GND = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_b_GND/pParam_B4SOIckappa))));
d_T4_dV_p_GND = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_p_GND/pParam_B4SOIckappa))));
d_T4_dTemp_t_GND = (deriv_sqrt_0_d0*((-(4.0*d_T2_dTemp_t_GND/pParam_B4SOIckappa))));
d_T4_dV_gi_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_gi_si/pParam_B4SOIckappa))));
d_T4_dV_di_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_di_si/pParam_B4SOIckappa))));
d_T4_dV_e_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_e_si/pParam_B4SOIckappa))));
d_T4_dV_b_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_b_si/pParam_B4SOIckappa))));
d_T4_dV_sb_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_sb_si/pParam_B4SOIckappa))));
d_T4_dV_db_di = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_db_di/pParam_B4SOIckappa))));
d_T4_dV_gi_p = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_gi_p/pParam_B4SOIckappa))));
d_T4_dV_gm_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_gm_si/pParam_B4SOIckappa))));
T4 = value_sqrt_0;
}
if ((B4SOIrgateMod==3))
{

d_qgdo_dV_gm_si = ((pParam_B4SOIcgdo+T3)*d_vgmd_dV_gm_si-T3*(d_T2_dV_gm_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gm_si));
d_qgdo_dV_b_GND = (d_T3_dV_b_GND*vgmd-(T3*(d_T2_dV_b_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_GND)+d_T3_dV_b_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_p_GND = (d_T3_dV_p_GND*vgmd-(T3*(d_T2_dV_p_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_p_GND)+d_T3_dV_p_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dTemp_t_GND = (d_T3_dTemp_t_GND*vgmd-(T3*(d_T2_dTemp_t_GND+(0.5*pParam_B4SOIckappa)*d_T4_dTemp_t_GND)+d_T3_dTemp_t_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_gi_si = (d_T3_dV_gi_si*vgmd-(T3*(d_T2_dV_gi_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_si)+d_T3_dV_gi_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_di_si = (((pParam_B4SOIcgdo+T3)*d_vgmd_dV_di_si+d_T3_dV_di_si*vgmd)-(T3*(d_T2_dV_di_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_di_si)+d_T3_dV_di_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_e_si = (d_T3_dV_e_si*vgmd-(T3*(d_T2_dV_e_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_e_si)+d_T3_dV_e_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_b_si = (d_T3_dV_b_si*vgmd-(T3*(d_T2_dV_b_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_si)+d_T3_dV_b_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_sb_si = (d_T3_dV_sb_si*vgmd-(T3*(d_T2_dV_sb_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_sb_si)+d_T3_dV_sb_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_db_di = (d_T3_dV_db_di*vgmd-(T3*(d_T2_dV_db_di+(0.5*pParam_B4SOIckappa)*d_T4_dV_db_di)+d_T3_dV_db_di*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_gi_p = (d_T3_dV_gi_p*vgmd-(T3*(d_T2_dV_gi_p+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_p)+d_T3_dV_gi_p*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
qgdo = (((pParam_B4SOIcgdo+T3)*vgmd)-(T3*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
}
else
{

d_qgdo_dV_gm_si = (-T3*(d_T2_dV_gm_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gm_si));
d_qgdo_dV_b_GND = (d_T3_dV_b_GND*vgd-(T3*(d_T2_dV_b_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_GND)+d_T3_dV_b_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_p_GND = (d_T3_dV_p_GND*vgd-(T3*(d_T2_dV_p_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_p_GND)+d_T3_dV_p_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dTemp_t_GND = (d_T3_dTemp_t_GND*vgd-(T3*(d_T2_dTemp_t_GND+(0.5*pParam_B4SOIckappa)*d_T4_dTemp_t_GND)+d_T3_dTemp_t_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_gi_si = (((pParam_B4SOIcgdo+T3)*d_vgd_dV_gi_si+d_T3_dV_gi_si*vgd)-(T3*(d_T2_dV_gi_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_si)+d_T3_dV_gi_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_di_si = (((pParam_B4SOIcgdo+T3)*d_vgd_dV_di_si+d_T3_dV_di_si*vgd)-(T3*(d_T2_dV_di_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_di_si)+d_T3_dV_di_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_e_si = (d_T3_dV_e_si*vgd-(T3*(d_T2_dV_e_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_e_si)+d_T3_dV_e_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_b_si = (d_T3_dV_b_si*vgd-(T3*(d_T2_dV_b_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_si)+d_T3_dV_b_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_sb_si = (d_T3_dV_sb_si*vgd-(T3*(d_T2_dV_sb_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_sb_si)+d_T3_dV_sb_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_db_di = (d_T3_dV_db_di*vgd-(T3*(d_T2_dV_db_di+(0.5*pParam_B4SOIckappa)*d_T4_dV_db_di)+d_T3_dV_db_di*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgdo_dV_gi_p = (d_T3_dV_gi_p*vgd-(T3*(d_T2_dV_gi_p+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_p)+d_T3_dV_gi_p*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
qgdo = (((pParam_B4SOIcgdo+T3)*vgd)-(T3*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
}
if ((B4SOIrgateMod==3))
{

d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_gi_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_gm_si = d_vgms_dV_gm_si;
T0 = (vgms+0.02);
}
else
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si = d_T0_dV_b_si = d_T0_dV_di_si = d_T0_dV_e_si = d_T0_dTemp_t_GND = d_T0_dV_p_GND = d_T0_dV_b_GND =  0.0;
d_T0_dV_gi_si = d_vgs_dV_gi_si;
T0 = (vgs+0.02);
}
{
double value_sqrt_0 = sqrt(((T0*T0)+(4.0*0.02)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T1_dV_b_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0)));
d_T1_dV_p_GND = (deriv_sqrt_0_d0*((T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0)));
d_T1_dTemp_t_GND = (deriv_sqrt_0_d0*((T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0)));
d_T1_dV_e_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_e_si+d_T0_dV_e_si*T0)));
d_T1_dV_gi_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0)));
d_T1_dV_di_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_di_si+d_T0_dV_di_si*T0)));
d_T1_dV_b_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_b_si+d_T0_dV_b_si*T0)));
d_T1_dV_sb_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0)));
d_T1_dV_db_di = (deriv_sqrt_0_d0*((T0*d_T0_dV_db_di+d_T0_dV_db_di*T0)));
d_T1_dV_gi_p = (deriv_sqrt_0_d0*((T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0)));
d_T1_dV_gm_si = (deriv_sqrt_0_d0*((T0*d_T0_dV_gm_si+d_T0_dV_gm_si*T0)));
T1 = value_sqrt_0;
}

d_T2_dV_b_GND = 0.5*(d_T0_dV_b_GND-d_T1_dV_b_GND);
d_T2_dV_p_GND = 0.5*(d_T0_dV_p_GND-d_T1_dV_p_GND);
d_T2_dTemp_t_GND = 0.5*(d_T0_dTemp_t_GND-d_T1_dTemp_t_GND);
d_T2_dV_e_si = 0.5*(d_T0_dV_e_si-d_T1_dV_e_si);
d_T2_dV_gi_si = 0.5*(d_T0_dV_gi_si-d_T1_dV_gi_si);
d_T2_dV_di_si = 0.5*(d_T0_dV_di_si-d_T1_dV_di_si);
d_T2_dV_b_si = 0.5*(d_T0_dV_b_si-d_T1_dV_b_si);
d_T2_dV_sb_si = 0.5*(d_T0_dV_sb_si-d_T1_dV_sb_si);
d_T2_dV_db_di = 0.5*(d_T0_dV_db_di-d_T1_dV_db_di);
d_T2_dV_gi_p = 0.5*(d_T0_dV_gi_p-d_T1_dV_gi_p);
d_T2_dV_gm_si = 0.5*(d_T0_dV_gm_si-d_T1_dV_gm_si);
T2 = (0.5*(T0-T1));

d_T3_dV_gi_p = d_T3_dV_db_di = d_T3_dV_sb_si = d_T3_dV_b_si = d_T3_dV_e_si = d_T3_dV_di_si = d_T3_dV_gi_si = d_T3_dTemp_t_GND = d_T3_dV_p_GND = d_T3_dV_b_GND =  0.0;
T3 = (pParam_B4SOIwdiosCV*pParam_B4SOIcgsl);
{
double value_sqrt_0 = sqrt((1.0-((4.0*T2)/pParam_B4SOIckappa)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);

d_T4_dV_b_GND = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_b_GND/pParam_B4SOIckappa))));
d_T4_dV_p_GND = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_p_GND/pParam_B4SOIckappa))));
d_T4_dTemp_t_GND = (deriv_sqrt_0_d0*((-(4.0*d_T2_dTemp_t_GND/pParam_B4SOIckappa))));
d_T4_dV_gi_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_gi_si/pParam_B4SOIckappa))));
d_T4_dV_di_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_di_si/pParam_B4SOIckappa))));
d_T4_dV_e_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_e_si/pParam_B4SOIckappa))));
d_T4_dV_b_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_b_si/pParam_B4SOIckappa))));
d_T4_dV_sb_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_sb_si/pParam_B4SOIckappa))));
d_T4_dV_db_di = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_db_di/pParam_B4SOIckappa))));
d_T4_dV_gi_p = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_gi_p/pParam_B4SOIckappa))));
d_T4_dV_gm_si = (deriv_sqrt_0_d0*((-(4.0*d_T2_dV_gm_si/pParam_B4SOIckappa))));
T4 = value_sqrt_0;
}
if ((B4SOIrgateMod==3))
{

d_qgso_dV_gm_si = ((pParam_B4SOIcgso+T3)*d_vgms_dV_gm_si-T3*(d_T2_dV_gm_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gm_si));
d_qgso_dV_b_GND = (d_T3_dV_b_GND*vgms-(T3*(d_T2_dV_b_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_GND)+d_T3_dV_b_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_p_GND = (d_T3_dV_p_GND*vgms-(T3*(d_T2_dV_p_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_p_GND)+d_T3_dV_p_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dTemp_t_GND = (d_T3_dTemp_t_GND*vgms-(T3*(d_T2_dTemp_t_GND+(0.5*pParam_B4SOIckappa)*d_T4_dTemp_t_GND)+d_T3_dTemp_t_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_gi_si = (d_T3_dV_gi_si*vgms-(T3*(d_T2_dV_gi_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_si)+d_T3_dV_gi_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_di_si = (d_T3_dV_di_si*vgms-(T3*(d_T2_dV_di_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_di_si)+d_T3_dV_di_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_e_si = (d_T3_dV_e_si*vgms-(T3*(d_T2_dV_e_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_e_si)+d_T3_dV_e_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_b_si = (d_T3_dV_b_si*vgms-(T3*(d_T2_dV_b_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_si)+d_T3_dV_b_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_sb_si = (d_T3_dV_sb_si*vgms-(T3*(d_T2_dV_sb_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_sb_si)+d_T3_dV_sb_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_db_di = (d_T3_dV_db_di*vgms-(T3*(d_T2_dV_db_di+(0.5*pParam_B4SOIckappa)*d_T4_dV_db_di)+d_T3_dV_db_di*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_gi_p = (d_T3_dV_gi_p*vgms-(T3*(d_T2_dV_gi_p+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_p)+d_T3_dV_gi_p*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
qgso = (((pParam_B4SOIcgso+T3)*vgms)-(T3*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
}
else
{

d_qgso_dV_gm_si = (-T3*(d_T2_dV_gm_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gm_si));
d_qgso_dV_b_GND = (d_T3_dV_b_GND*vgs-(T3*(d_T2_dV_b_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_GND)+d_T3_dV_b_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_p_GND = (d_T3_dV_p_GND*vgs-(T3*(d_T2_dV_p_GND+(0.5*pParam_B4SOIckappa)*d_T4_dV_p_GND)+d_T3_dV_p_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dTemp_t_GND = (d_T3_dTemp_t_GND*vgs-(T3*(d_T2_dTemp_t_GND+(0.5*pParam_B4SOIckappa)*d_T4_dTemp_t_GND)+d_T3_dTemp_t_GND*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_gi_si = (((pParam_B4SOIcgso+T3)*d_vgs_dV_gi_si+d_T3_dV_gi_si*vgs)-(T3*(d_T2_dV_gi_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_si)+d_T3_dV_gi_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_di_si = (d_T3_dV_di_si*vgs-(T3*(d_T2_dV_di_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_di_si)+d_T3_dV_di_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_e_si = (d_T3_dV_e_si*vgs-(T3*(d_T2_dV_e_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_e_si)+d_T3_dV_e_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_b_si = (d_T3_dV_b_si*vgs-(T3*(d_T2_dV_b_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_b_si)+d_T3_dV_b_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_sb_si = (d_T3_dV_sb_si*vgs-(T3*(d_T2_dV_sb_si+(0.5*pParam_B4SOIckappa)*d_T4_dV_sb_si)+d_T3_dV_sb_si*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_db_di = (d_T3_dV_db_di*vgs-(T3*(d_T2_dV_db_di+(0.5*pParam_B4SOIckappa)*d_T4_dV_db_di)+d_T3_dV_db_di*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
d_qgso_dV_gi_p = (d_T3_dV_gi_p*vgs-(T3*(d_T2_dV_gi_p+(0.5*pParam_B4SOIckappa)*d_T4_dV_gi_p)+d_T3_dV_gi_p*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
qgso = (((pParam_B4SOIcgso+T3)*vgs)-(T3*(T2+((0.5*pParam_B4SOIckappa)*(T4-1.0)))));
}
if ((B4SOInf!=1.0))
{

d_qgdo_dV_gm_si = d_qgdo_dV_gm_si*B4SOInf;
d_qgdo_dV_b_GND = d_qgdo_dV_b_GND*B4SOInf;
d_qgdo_dV_p_GND = d_qgdo_dV_p_GND*B4SOInf;
d_qgdo_dTemp_t_GND = d_qgdo_dTemp_t_GND*B4SOInf;
d_qgdo_dV_gi_si = d_qgdo_dV_gi_si*B4SOInf;
d_qgdo_dV_di_si = d_qgdo_dV_di_si*B4SOInf;
d_qgdo_dV_e_si = d_qgdo_dV_e_si*B4SOInf;
d_qgdo_dV_b_si = d_qgdo_dV_b_si*B4SOInf;
d_qgdo_dV_sb_si = d_qgdo_dV_sb_si*B4SOInf;
d_qgdo_dV_db_di = d_qgdo_dV_db_di*B4SOInf;
d_qgdo_dV_gi_p = d_qgdo_dV_gi_p*B4SOInf;
qgdo = (qgdo*B4SOInf);

d_qgso_dV_gm_si = d_qgso_dV_gm_si*B4SOInf;
d_qgso_dV_b_GND = d_qgso_dV_b_GND*B4SOInf;
d_qgso_dV_p_GND = d_qgso_dV_p_GND*B4SOInf;
d_qgso_dTemp_t_GND = d_qgso_dTemp_t_GND*B4SOInf;
d_qgso_dV_gi_si = d_qgso_dV_gi_si*B4SOInf;
d_qgso_dV_di_si = d_qgso_dV_di_si*B4SOInf;
d_qgso_dV_e_si = d_qgso_dV_e_si*B4SOInf;
d_qgso_dV_b_si = d_qgso_dV_b_si*B4SOInf;
d_qgso_dV_sb_si = d_qgso_dV_sb_si*B4SOInf;
d_qgso_dV_db_di = d_qgso_dV_db_di*B4SOInf;
d_qgso_dV_gi_p = d_qgso_dV_gi_p*B4SOInf;
qgso = (qgso*B4SOInf);
}

d_qgi_dV_e_si = d_qgate_dV_e_si;
d_qgi_dV_b_si = d_qgate_dV_b_si;
d_qgi_dV_b_GND = d_qgate_dV_b_GND;
d_qgi_dV_p_GND = d_qgate_dV_p_GND;
d_qgi_dTemp_t_GND = d_qgate_dTemp_t_GND;
d_qgi_dV_gi_si = d_qgate_dV_gi_si;
d_qgi_dV_di_si = d_qgate_dV_di_si;
d_qgi_dV_sb_si = d_qgate_dV_sb_si;
d_qgi_dV_db_di = d_qgate_dV_db_di;
d_qgi_dV_gi_p = d_qgate_dV_gi_p;
qgi = qgate;

d_qov_dV_gm_si = (d_qgso_dV_gm_si+d_qgdo_dV_gm_si);
d_qov_dV_b_GND = (d_qgso_dV_b_GND+d_qgdo_dV_b_GND);
d_qov_dV_p_GND = (d_qgso_dV_p_GND+d_qgdo_dV_p_GND);
d_qov_dTemp_t_GND = (d_qgso_dTemp_t_GND+d_qgdo_dTemp_t_GND);
d_qov_dV_gi_si = (d_qgso_dV_gi_si+d_qgdo_dV_gi_si);
d_qov_dV_di_si = (d_qgso_dV_di_si+d_qgdo_dV_di_si);
d_qov_dV_e_si = (d_qgso_dV_e_si+d_qgdo_dV_e_si);
d_qov_dV_b_si = (d_qgso_dV_b_si+d_qgdo_dV_b_si);
d_qov_dV_sb_si = (d_qgso_dV_sb_si+d_qgdo_dV_sb_si);
d_qov_dV_db_di = (d_qgso_dV_db_di+d_qgdo_dV_db_di);
d_qov_dV_gi_p = (d_qgso_dV_gi_p+d_qgdo_dV_gi_p);
qov = (qgso+qgdo);

d_qgate_dV_gm_si = d_qov_dV_gm_si;
d_qgate_dV_e_si = (d_qgi_dV_e_si+d_qov_dV_e_si);
d_qgate_dV_b_si = (d_qgi_dV_b_si+d_qov_dV_b_si);
d_qgate_dV_b_GND = (d_qgi_dV_b_GND+d_qov_dV_b_GND);
d_qgate_dV_p_GND = (d_qgi_dV_p_GND+d_qov_dV_p_GND);
d_qgate_dTemp_t_GND = (d_qgi_dTemp_t_GND+d_qov_dTemp_t_GND);
d_qgate_dV_gi_si = (d_qgi_dV_gi_si+d_qov_dV_gi_si);
d_qgate_dV_di_si = (d_qgi_dV_di_si+d_qov_dV_di_si);
d_qgate_dV_sb_si = (d_qgi_dV_sb_si+d_qov_dV_sb_si);
d_qgate_dV_db_di = (d_qgi_dV_db_di+d_qov_dV_db_di);
d_qgate_dV_gi_p = (d_qgi_dV_gi_p+d_qov_dV_gi_p);
qgate = (qgi+qov);
//Begin block noise
{
//Block-local variables for block noise
double cd=0.0;
double cdmin=0.0;
double logcdmin=0.0;
double wefftot=0.0;
double leffBcox=0.0;
//End of Block-local variables
//Block-local variables for block noise
double thermalNoiseContrib=0.0;
double flickerNoiseContrib=0.0;
//End of Block-local variables
//Block-local variables for block noise
double esat=0.0;
double npart_beta=0.0;
double npart_theta=0.0;
double igsquare=0.0;
//End of Block-local variables
if ((B4SOImode>0))
{
{
double value_fabs_0 = fabs(((((Ids+Ic)-Ibd)+Iii)+Igidl));
double  deriv_fabs_0_d0 = (((((((Ids+Ic)-Ibd)+Iii)+Igidl))>=0)?(+1.0):(-1.0));
cd = value_fabs_0;
}
}
else
{
{
double value_fabs_0 = fabs(((((Ids-Ic)-Ibs)+Iii)+Igidl));
double  deriv_fabs_0_d0 = (((((((Ids-Ic)-Ibs)+Iii)+Igidl))>=0)?(+1.0):(-1.0));
cd = value_fabs_0;
}
}
fourkt = ((4.0*1.3806503e-23)*B4SOITempSH);
if ((B4SOIrdsMod!=2))
{
gdnoise = (1/Rd);
gsnoise = (1/Rs);
}
else
{
gdnoise = 0.0;
gsnoise = 0.0;
}
if ( (model_.TNOIMOD) == (0))
{
{
double value_fabs_0 = fabs(B4SOIqinv);
double value_fabs_1 = fabs((B4SOIqinv/((pParam_B4SOIleff*pParam_B4SOIleff)+((B4SOIueff*value_fabs_0)*B4SOIrds))));
double  deriv_fabs_0_d0 = (((B4SOIqinv)>=0)?(+1.0):(-1.0));
double  deriv_fabs_1_d0 = ((((B4SOIqinv/((pParam_B4SOIleff*pParam_B4SOIleff)+((B4SOIueff*value_fabs_0)*B4SOIrds))))>=0)?(+1.0):(-1.0));
thermalNoiseContrib = ((B4SOIntnoi*B4SOIueff)*value_fabs_1);
}
// I(di,si) <+ (white_noise((m*abs((fourkt*thermalNoiseContrib))),"id"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*thermalNoiseContrib));
  noiseContribsPower[0]= (m*value_fabs_0);
}
}
else
if ( (model_.TNOIMOD) == (1))
{

d_T0_dV_gm_si = d_T0_dV_gi_p = d_T0_dV_db_di = d_T0_dV_sb_si =  0.0;
d_T0_dV_gi_si = 0.0;
d_T0_dV_di_si = 0.0;
d_T0_dV_e_si = 0.0;
d_T0_dV_b_si = 0.0;
d_T0_dV_b_GND = 0.0;
d_T0_dV_p_GND = 0.0;
d_T0_dTemp_t_GND = 0.0;
T0 = ((B4SOIgm+B4SOIgds)+B4SOIgmbs);

d_T0_dV_b_GND = (T0*d_T0_dV_b_GND+d_T0_dV_b_GND*T0);
d_T0_dV_p_GND = (T0*d_T0_dV_p_GND+d_T0_dV_p_GND*T0);
d_T0_dTemp_t_GND = (T0*d_T0_dTemp_t_GND+d_T0_dTemp_t_GND*T0);
d_T0_dV_e_si = (T0*d_T0_dV_e_si+d_T0_dV_e_si*T0);
d_T0_dV_gi_si = (T0*d_T0_dV_gi_si+d_T0_dV_gi_si*T0);
d_T0_dV_di_si = (T0*d_T0_dV_di_si+d_T0_dV_di_si*T0);
d_T0_dV_b_si = (T0*d_T0_dV_b_si+d_T0_dV_b_si*T0);
d_T0_dV_sb_si = (T0*d_T0_dV_sb_si+d_T0_dV_sb_si*T0);
d_T0_dV_db_di = (T0*d_T0_dV_db_di+d_T0_dV_db_di*T0);
d_T0_dV_gi_p = (T0*d_T0_dV_gi_p+d_T0_dV_gi_p*T0);
d_T0_dV_gm_si = (T0*d_T0_dV_gm_si+d_T0_dV_gm_si*T0);
T0 = (T0*T0);
esat = ((2.0*vsattemp)/B4SOIueff);

d_T5_dV_gi_p = d_T5_dV_db_di = d_T5_dV_sb_si =  0.0;
d_T5_dV_e_si = 0.0;
d_T5_dV_b_si = 0.0;
d_T5_dV_b_GND = 0.0;
d_T5_dV_p_GND = 0.0;
d_T5_dTemp_t_GND = 0.0;
d_T5_dV_gi_si = 0.0;
d_T5_dV_di_si = 0.0;
T5 = (B4SOIVgsteff/(esat*pParam_B4SOIleff));

d_T5_dV_b_GND = (T5*d_T5_dV_b_GND+d_T5_dV_b_GND*T5);
d_T5_dV_p_GND = (T5*d_T5_dV_p_GND+d_T5_dV_p_GND*T5);
d_T5_dTemp_t_GND = (T5*d_T5_dTemp_t_GND+d_T5_dTemp_t_GND*T5);
d_T5_dV_gi_si = (T5*d_T5_dV_gi_si+d_T5_dV_gi_si*T5);
d_T5_dV_di_si = (T5*d_T5_dV_di_si+d_T5_dV_di_si*T5);
d_T5_dV_e_si = (T5*d_T5_dV_e_si+d_T5_dV_e_si*T5);
d_T5_dV_b_si = (T5*d_T5_dV_b_si+d_T5_dV_b_si*T5);
d_T5_dV_sb_si = (T5*d_T5_dV_sb_si+d_T5_dV_sb_si*T5);
d_T5_dV_db_di = (T5*d_T5_dV_db_di+d_T5_dV_db_di*T5);
d_T5_dV_gi_p = (T5*d_T5_dV_gi_p+d_T5_dV_gi_p*T5);
T5 = (T5*T5);
npart_beta = (B4SOIrnoia*(1.0+((T5*B4SOItnoia)*pParam_B4SOIleff)));
npart_theta = (B4SOIrnoib*(1.0+((T5*B4SOItnoib)*pParam_B4SOIleff)));
if ((npart_theta>0.9))
{
npart_theta = 0.9;
}
if ((npart_theta>(0.9*npart_beta)))
{
npart_theta = (0.9*npart_beta);
}
igsquare = (((npart_theta*npart_theta)*T0)/B4SOIidovVds);

d_T1_dV_gm_si =  0.0;
d_T1_dV_b_GND = 0.0;
d_T1_dV_p_GND = 0.0;
d_T1_dTemp_t_GND = 0.0;
d_T1_dV_gi_si = 0.0;
d_T1_dV_di_si = 0.0;
d_T1_dV_e_si = 0.0;
d_T1_dV_b_si = 0.0;
d_T1_dV_sb_si = 0.0;
d_T1_dV_db_di = 0.0;
d_T1_dV_gi_p = 0.0;
T1 = ((npart_beta*(B4SOIgm+B4SOIgmbs))+B4SOIgds);

d_T2_dV_b_GND = ((T1*d_T1_dV_b_GND+d_T1_dV_b_GND*T1)/B4SOIidovVds);
d_T2_dV_p_GND = ((T1*d_T1_dV_p_GND+d_T1_dV_p_GND*T1)/B4SOIidovVds);
d_T2_dTemp_t_GND = ((T1*d_T1_dTemp_t_GND+d_T1_dTemp_t_GND*T1)/B4SOIidovVds);
d_T2_dV_gi_si = ((T1*d_T1_dV_gi_si+d_T1_dV_gi_si*T1)/B4SOIidovVds);
d_T2_dV_di_si = ((T1*d_T1_dV_di_si+d_T1_dV_di_si*T1)/B4SOIidovVds);
d_T2_dV_e_si = ((T1*d_T1_dV_e_si+d_T1_dV_e_si*T1)/B4SOIidovVds);
d_T2_dV_b_si = ((T1*d_T1_dV_b_si+d_T1_dV_b_si*T1)/B4SOIidovVds);
d_T2_dV_sb_si = ((T1*d_T1_dV_sb_si+d_T1_dV_sb_si*T1)/B4SOIidovVds);
d_T2_dV_db_di = ((T1*d_T1_dV_db_di+d_T1_dV_db_di*T1)/B4SOIidovVds);
d_T2_dV_gi_p = ((T1*d_T1_dV_gi_p+d_T1_dV_gi_p*T1)/B4SOIidovVds);
d_T2_dV_gm_si = ((T1*d_T1_dV_gm_si+d_T1_dV_gm_si*T1)/B4SOIidovVds);
T2 = ((T1*T1)/B4SOIidovVds);
thermalNoiseContrib = (T2-igsquare);
if ((B4SOImode>0))
{
gsnoise = (gsnoise*(1.0+(((npart_theta*npart_theta)*gsnoise)/B4SOIidovVds)));
}
else
{
gdnoise = (gdnoise*(1.0+(((npart_theta*npart_theta)*gdnoise)/B4SOIidovVds)));
}
// I(di,si) <+ (white_noise((m*abs((fourkt*thermalNoiseContrib))),"id"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*thermalNoiseContrib));
  noiseContribsPower[1]= (m*value_fabs_0);
}
}
else
if ( (model_.TNOIMOD) == (3))
{
UserInfo(*this)  << "[BSIMSOI] Although the model selector TNOIMOD is set to 3, the new correlated thermal noise model is not activated in the Verilog-A code.  Please uncomment \"`define _TNOIMOD3_\" in the beginning of the Verilog-A code." << " " <<  std::endl; 
}
else
if ( (model_.TNOIMOD) == (2))
{
{
double value_fabs_0 = fabs(((B4SOIgm+B4SOIgds)+B4SOIgmbs));
double  deriv_fabs_0_d0 = (((((B4SOIgm+B4SOIgds)+B4SOIgmbs))>=0)?(+1.0):(-1.0));
thermalNoiseContrib = (((2.0/3.0)*B4SOIntnoi)*value_fabs_0);
}
// I(di,si) <+ (white_noise((m*abs((fourkt*thermalNoiseContrib))),"id"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*thermalNoiseContrib));
  noiseContribsPower[2]= (m*value_fabs_0);
}
}
else
{
 // no default
}
wefftot = (B4SOInf*pParam_B4SOIweff);
if ((B4SOIbf==1))
{
leffBcox = (pParam_B4SOIleff*B4SOIcox);
}
else
{
if ((B4SOIbf==2))
{
leffBcox = ((pParam_B4SOIleff*pParam_B4SOIleff)*B4SOIcox);
}
else
{
{
double value_pow_0 = pow(pParam_B4SOIleff,B4SOIbf);
leffBcox = (value_pow_0*B4SOIcox);
}
}
}
if (((model_.FNOIMOD)==0))
{
if ((B4SOIw0flk>0))
{
cdmin = ((cd/wefftot)*B4SOIw0flk);
if ((cdmin<1.0e-38))
{
cdmin = 1.0e-38;
}
{
double value_log_0 = log(cdmin);
double  deriv_log_0_d0 = (1.0/cdmin);
logcdmin = value_log_0;
}
{
double value_exp_0 = exp((B4SOIaf*logcdmin));
double  deriv_exp_0_d0 = value_exp_0;
flickerNoiseContrib = ((((wefftot/B4SOIw0flk)*B4SOIkf)*value_exp_0)/leffBcox);
}
}
else
{
if ((cd<1.0e-38))
{
cdmin = 1.0e-38;
}
else
{
cdmin = cd;
}
{
double value_log_0 = log(cdmin);
double  deriv_log_0_d0 = (1.0/cdmin);
logcdmin = value_log_0;
}
{
double value_exp_0 = exp((B4SOIaf*logcdmin));
double  deriv_exp_0_d0 = value_exp_0;
flickerNoiseContrib = ((B4SOIkf*value_exp_0)/leffBcox);
}
}
}
else
//Begin block Eval1ovFNoise
{
//Block-local variables for block Eval1ovFNoise
double esat=0.0;
double DelClm=0.0;
double N0=0.0;
double Nl=0.0;
double Nratio=0.0;
double logem38=0.0;
//End of Block-local variables
//Block-local variables for block Eval1ovFNoise
double T0=0.0;
double T1=0.0;
double T2=0.0;
double T3=0.0;
double T4=0.0;
double T5=0.0;
double T6=0.0;
double T7=0.0;
double T8=0.0;
double T9=0.0;
double T10=0.0;
double T11=0.0;
//End of Block-local variables
//Block-local variables for block Eval1ovFNoise
double Ssi=0.0;
double Swi=0.0;
//End of Block-local variables
{
double value_log_0 = log(static_cast<double>(1e-38));
logem38 = value_log_0;
}
esat = ((2.0*vsattemp)/B4SOIueff);
if ((B4SOIem<=0.0))
{
DelClm = 0.0;
}
else
{
T0 = ((((Vds-B4SOIVdseff)/pParam_B4SOIlitl)+B4SOIem)/esat);
if ((T0<1e-38))
{
DelClm = (pParam_B4SOIlitl*logem38);
}
else
{
{
double value_log_0 = log(T0);
double  deriv_log_0_d0 = (1.0/T0);
DelClm = (pParam_B4SOIlitl*value_log_0);
}
}
}
T1 = (((((1.60219e-19*1.60219e-19)*1.3806503e-23)*cd)*B4SOITempSH)*B4SOIueff);
T2 = ((((1.0e10*B4SOIAbulk)*B4SOIcox)*pParam_B4SOIleff)*pParam_B4SOIleff);
N0 = ((B4SOIcox*B4SOIVgsteff)/1.60219e-19);
Nl = (((B4SOIcox*B4SOIVgsteff)*(1.0-(B4SOIAbovVgst2Vtm*B4SOIVdseff)))/1.60219e-19);
Nratio = ((N0+B4SOInstar)/(Nl+B4SOInstar));
if ((Nratio<1e-38))
{
T3 = (B4SOIoxideTrapDensityA*logem38);
}
else
{
{
double value_log_0 = log(Nratio);
double  deriv_log_0_d0 = (1.0/Nratio);
T3 = (B4SOIoxideTrapDensityA*value_log_0);
}
}
T4 = (B4SOIoxideTrapDensityB*(N0-Nl));
T5 = ((B4SOIoxideTrapDensityC*0.5)*((N0*N0)-(Nl*Nl)));
T6 = (((1.3806503e-23*B4SOITempSH)*cd)*cd);
T7 = (((1.0e10*pParam_B4SOIleff)*pParam_B4SOIleff)*wefftot);
T8 = ((B4SOIoxideTrapDensityA+(B4SOIoxideTrapDensityB*Nl))+((B4SOIoxideTrapDensityC*Nl)*Nl));
T9 = ((Nl+B4SOInstar)*(Nl+B4SOInstar));
Ssi = (((T1/T2)*((T3+T4)+T5))+((((T6/T7)*DelClm)*T8)/T9));
T10 = ((B4SOIoxideTrapDensityA*1.3806503e-23)*B4SOITempSH);
T11 = ((((wefftot*pParam_B4SOIleff)*1.0e10)*B4SOInstar)*B4SOInstar);
Swi = (((T10/T11)*cd)*cd);
T1 = (Swi+Ssi);
if ((((T1>0.0)&&(Ssi>0.0))&&(Swi>0.0)))
{
flickerNoiseContrib = ((Ssi*Swi)/T1);
}
else
{
flickerNoiseContrib = 0;
}
}
// End block Eval1ovFNoise
// I(di,si) <+ (flicker_noise((m*flickerNoiseContrib),B4SOIef,"flicker"))
if (getSolverState().noiseFlag)
{
  noiseContribsPower[3]= (m*flickerNoiseContrib);
  noiseContribsExponent[3]= B4SOIef;
}
}
// End block noise
if ((B4SOIrdsMod!=2))
{
// I(d,di) <+ (((m*V(d,di))/Rd))
staticContributions[admsNodeID_d] += ((m*(probeVars[admsProbeID_V_d_di]))/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_gi_si] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_gi_si/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_di_si] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_di_si/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_e_si] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_e_si/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_b_si] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_b_si/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_sb_si] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_sb_si/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_db_di] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_db_di/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_gi_p] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_gi_p/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_b_GND] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_b_GND/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_p_GND] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_p_GND/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_Temp_t_GND] += (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dTemp_t_GND/Rd/Rd);
d_staticContributions[admsNodeID_d][admsProbeID_V_d_di] += (m*d_probeVars[admsProbeID_V_d_di][admsProbeID_V_d_di]/Rd);
staticContributions[admsNodeID_di] -= ((m*(probeVars[admsProbeID_V_d_di]))/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_gi_si/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_di_si/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_e_si/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_b_si/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_sb_si/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_db_di] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_db_di/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_p] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_gi_p/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_b_GND/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dV_p_GND/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] -= (-(m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dTemp_t_GND/Rd/Rd);
d_staticContributions[admsNodeID_di][admsProbeID_V_d_di] -= (m*d_probeVars[admsProbeID_V_d_di][admsProbeID_V_d_di]/Rd);
// I(d,di) <+ (white_noise((m*abs((fourkt*gdnoise))),"rd"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*gdnoise));
  noiseContribsPower[4]= (m*value_fabs_0);
}
// I(s,si) <+ (((m*V(s,si))/Rs))
staticContributions[admsNodeID_s] += ((m*(probeVars[admsProbeID_V_s_si]))/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_gi_si] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_gi_si/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_di_si] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_di_si/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_e_si] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_e_si/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_b_si] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_b_si/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_sb_si] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_sb_si/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_db_di] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_db_di/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_gi_p] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_gi_p/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_b_GND] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_b_GND/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_p_GND] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_p_GND/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_Temp_t_GND] += (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dTemp_t_GND/Rs/Rs);
d_staticContributions[admsNodeID_s][admsProbeID_V_s_si] += (m*d_probeVars[admsProbeID_V_s_si][admsProbeID_V_s_si]/Rs);
staticContributions[admsNodeID_si] -= ((m*(probeVars[admsProbeID_V_s_si]))/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_gi_si/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_di_si/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_e_si/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_b_si/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_sb_si/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_db_di] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_db_di/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_p] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_gi_p/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_b_GND/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dV_p_GND/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] -= (-(m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dTemp_t_GND/Rs/Rs);
d_staticContributions[admsNodeID_si][admsProbeID_V_s_si] -= (m*d_probeVars[admsProbeID_V_s_si][admsProbeID_V_s_si]/Rs);
// I(s,si) <+ (white_noise((m*abs((fourkt*gsnoise))),"rs"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*gsnoise));
  noiseContribsPower[5]= (m*value_fabs_0);
}
}
else
{
// V(d,di) <+ (0.0)
// do nothing at all
// V(s,si) <+ (0.0)
// do nothing at all
}
if ((B4SOImode>0))
{
// I(di,si) <+ (((m*B4SOItype)*(Ids+Ic)))
staticContributions[admsNodeID_di] += ((m*B4SOItype)*(Ids+Ic));
d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si] += (m*B4SOItype)*d_Ic_dV_sb_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_db_di] += (m*B4SOItype)*d_Ic_dV_db_di;
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_Ids_dV_gi_si+d_Ic_dV_gi_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] += (m*B4SOItype)*(d_Ids_dV_di_si+d_Ic_dV_di_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] += (m*B4SOItype)*(d_Ids_dV_e_si+d_Ic_dV_e_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] += (m*B4SOItype)*(d_Ids_dV_b_si+d_Ic_dV_b_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_Ids_dV_b_GND+d_Ic_dV_b_GND);
d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_Ids_dV_p_GND+d_Ic_dV_p_GND);
d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_Ids_dTemp_t_GND+d_Ic_dTemp_t_GND);
staticContributions[admsNodeID_si] -= ((m*B4SOItype)*(Ids+Ic));
d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si] -= (m*B4SOItype)*d_Ic_dV_sb_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_db_di] -= (m*B4SOItype)*d_Ic_dV_db_di;
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_Ids_dV_gi_si+d_Ic_dV_gi_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_Ids_dV_di_si+d_Ic_dV_di_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_Ids_dV_e_si+d_Ic_dV_e_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_Ids_dV_b_si+d_Ic_dV_b_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_Ids_dV_b_GND+d_Ic_dV_b_GND);
d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_Ids_dV_p_GND+d_Ic_dV_p_GND);
d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_Ids_dTemp_t_GND+d_Ic_dTemp_t_GND);
// I(di,b) <+ (((m*B4SOItype)*Iii))
staticContributions[admsNodeID_di] += ((m*B4SOItype)*Iii);
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] += (m*B4SOItype)*d_Iii_dV_gi_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] += (m*B4SOItype)*d_Iii_dV_e_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] += (m*B4SOItype)*d_Iii_dV_b_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si] += (m*B4SOItype)*d_Iii_dV_sb_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_db_di] += (m*B4SOItype)*d_Iii_dV_db_di;
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_p] += (m*B4SOItype)*d_Iii_dV_gi_p;
d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] += (m*B4SOItype)*d_Iii_dV_b_GND;
d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] += (m*B4SOItype)*d_Iii_dV_p_GND;
d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] += (m*B4SOItype)*d_Iii_dTemp_t_GND;
d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] += (m*B4SOItype)*d_Iii_dV_di_si;
staticContributions[admsNodeID_b] -= ((m*B4SOItype)*Iii);
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] -= (m*B4SOItype)*d_Iii_dV_gi_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] -= (m*B4SOItype)*d_Iii_dV_e_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] -= (m*B4SOItype)*d_Iii_dV_b_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si] -= (m*B4SOItype)*d_Iii_dV_sb_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_db_di] -= (m*B4SOItype)*d_Iii_dV_db_di;
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p] -= (m*B4SOItype)*d_Iii_dV_gi_p;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] -= (m*B4SOItype)*d_Iii_dV_b_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] -= (m*B4SOItype)*d_Iii_dV_p_GND;
d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*d_Iii_dTemp_t_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -= (m*B4SOItype)*d_Iii_dV_di_si;

d_B4SOIigidl_dV_b_GND = B4SOItype*d_Igidl_dV_b_GND;
d_B4SOIigidl_dV_p_GND = B4SOItype*d_Igidl_dV_p_GND;
d_B4SOIigidl_dTemp_t_GND = B4SOItype*d_Igidl_dTemp_t_GND;
d_B4SOIigidl_dV_gi_si = B4SOItype*d_Igidl_dV_gi_si;
d_B4SOIigidl_dV_di_si = B4SOItype*d_Igidl_dV_di_si;
d_B4SOIigidl_dV_e_si = B4SOItype*d_Igidl_dV_e_si;
d_B4SOIigidl_dV_b_si = B4SOItype*d_Igidl_dV_b_si;
B4SOIigidl = (B4SOItype*Igidl);

d_B4SOIigisl_dV_b_GND = B4SOItype*d_Igisl_dV_b_GND;
d_B4SOIigisl_dV_p_GND = B4SOItype*d_Igisl_dV_p_GND;
d_B4SOIigisl_dTemp_t_GND = B4SOItype*d_Igisl_dTemp_t_GND;
d_B4SOIigisl_dV_gi_si = B4SOItype*d_Igisl_dV_gi_si;
d_B4SOIigisl_dV_di_si = B4SOItype*d_Igisl_dV_di_si;
d_B4SOIigisl_dV_e_si = B4SOItype*d_Igisl_dV_e_si;
d_B4SOIigisl_dV_b_si = B4SOItype*d_Igisl_dV_b_si;
B4SOIigisl = (B4SOItype*Igisl);

d_B4SOIIgcd_dV_b_GND = B4SOItype*d_Igcd_dV_b_GND;
d_B4SOIIgcd_dV_p_GND = B4SOItype*d_Igcd_dV_p_GND;
d_B4SOIIgcd_dTemp_t_GND = B4SOItype*d_Igcd_dTemp_t_GND;
d_B4SOIIgcd_dV_gi_si = B4SOItype*d_Igcd_dV_gi_si;
d_B4SOIIgcd_dV_di_si = B4SOItype*d_Igcd_dV_di_si;
d_B4SOIIgcd_dV_e_si = B4SOItype*d_Igcd_dV_e_si;
d_B4SOIIgcd_dV_b_si = B4SOItype*d_Igcd_dV_b_si;
d_B4SOIIgcd_dV_sb_si = B4SOItype*d_Igcd_dV_sb_si;
d_B4SOIIgcd_dV_db_di = B4SOItype*d_Igcd_dV_db_di;
B4SOIIgcd = (B4SOItype*Igcd);

d_B4SOIIgcs_dV_b_GND = B4SOItype*d_Igcs_dV_b_GND;
d_B4SOIIgcs_dV_p_GND = B4SOItype*d_Igcs_dV_p_GND;
d_B4SOIIgcs_dTemp_t_GND = B4SOItype*d_Igcs_dTemp_t_GND;
d_B4SOIIgcs_dV_gi_si = B4SOItype*d_Igcs_dV_gi_si;
d_B4SOIIgcs_dV_di_si = B4SOItype*d_Igcs_dV_di_si;
d_B4SOIIgcs_dV_e_si = B4SOItype*d_Igcs_dV_e_si;
d_B4SOIIgcs_dV_b_si = B4SOItype*d_Igcs_dV_b_si;
d_B4SOIIgcs_dV_sb_si = B4SOItype*d_Igcs_dV_sb_si;
d_B4SOIIgcs_dV_db_di = B4SOItype*d_Igcs_dV_db_di;
B4SOIIgcs = (B4SOItype*Igcs);

d_B4SOIqdrn_dV_e_si = B4SOItype*d_qdrn_dV_e_si;
d_B4SOIqdrn_dV_b_si = B4SOItype*d_qdrn_dV_b_si;
d_B4SOIqdrn_dV_b_GND = B4SOItype*d_qdrn_dV_b_GND;
d_B4SOIqdrn_dV_p_GND = B4SOItype*d_qdrn_dV_p_GND;
d_B4SOIqdrn_dTemp_t_GND = B4SOItype*d_qdrn_dTemp_t_GND;
d_B4SOIqdrn_dV_gi_si = B4SOItype*d_qdrn_dV_gi_si;
d_B4SOIqdrn_dV_di_si = B4SOItype*d_qdrn_dV_di_si;
d_B4SOIqdrn_dV_sb_si = B4SOItype*d_qdrn_dV_sb_si;
d_B4SOIqdrn_dV_db_di = B4SOItype*d_qdrn_dV_db_di;
d_B4SOIqdrn_dV_gi_p = B4SOItype*d_qdrn_dV_gi_p;
B4SOIqdrn = (B4SOItype*qdrn);

d_B4SOIqsrc_dV_e_si = B4SOItype*d_qsrc_dV_e_si;
d_B4SOIqsrc_dV_b_si = B4SOItype*d_qsrc_dV_b_si;
d_B4SOIqsrc_dV_b_GND = B4SOItype*d_qsrc_dV_b_GND;
d_B4SOIqsrc_dV_p_GND = B4SOItype*d_qsrc_dV_p_GND;
d_B4SOIqsrc_dTemp_t_GND = B4SOItype*d_qsrc_dTemp_t_GND;
d_B4SOIqsrc_dV_gi_si = B4SOItype*d_qsrc_dV_gi_si;
d_B4SOIqsrc_dV_di_si = B4SOItype*d_qsrc_dV_di_si;
d_B4SOIqsrc_dV_sb_si = B4SOItype*d_qsrc_dV_sb_si;
d_B4SOIqsrc_dV_db_di = B4SOItype*d_qsrc_dV_db_di;
d_B4SOIqsrc_dV_gi_p = B4SOItype*d_qsrc_dV_gi_p;
B4SOIqsrc = (B4SOItype*qsrc);
}
else
{
// I(si,di) <+ (((m*B4SOItype)*(Ids-Ic)))
staticContributions[admsNodeID_si] += ((m*B4SOItype)*(Ids-Ic));
d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si] += (m*B4SOItype)*(-d_Ic_dV_sb_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_db_di] += (m*B4SOItype)*(-d_Ic_dV_db_di);
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_Ids_dV_gi_si-d_Ic_dV_gi_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] += (m*B4SOItype)*(d_Ids_dV_di_si-d_Ic_dV_di_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] += (m*B4SOItype)*(d_Ids_dV_e_si-d_Ic_dV_e_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] += (m*B4SOItype)*(d_Ids_dV_b_si-d_Ic_dV_b_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_Ids_dV_b_GND-d_Ic_dV_b_GND);
d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_Ids_dV_p_GND-d_Ic_dV_p_GND);
d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_Ids_dTemp_t_GND-d_Ic_dTemp_t_GND);
staticContributions[admsNodeID_di] -= ((m*B4SOItype)*(Ids-Ic));
d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si] -= (m*B4SOItype)*(-d_Ic_dV_sb_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_db_di] -= (m*B4SOItype)*(-d_Ic_dV_db_di);
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_Ids_dV_gi_si-d_Ic_dV_gi_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_Ids_dV_di_si-d_Ic_dV_di_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_Ids_dV_e_si-d_Ic_dV_e_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_Ids_dV_b_si-d_Ic_dV_b_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_Ids_dV_b_GND-d_Ic_dV_b_GND);
d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_Ids_dV_p_GND-d_Ic_dV_p_GND);
d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_Ids_dTemp_t_GND-d_Ic_dTemp_t_GND);
// I(si,b) <+ (((m*B4SOItype)*Iii))
staticContributions[admsNodeID_si] += ((m*B4SOItype)*Iii);
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] += (m*B4SOItype)*d_Iii_dV_gi_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] += (m*B4SOItype)*d_Iii_dV_e_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] += (m*B4SOItype)*d_Iii_dV_b_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si] += (m*B4SOItype)*d_Iii_dV_sb_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_db_di] += (m*B4SOItype)*d_Iii_dV_db_di;
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_p] += (m*B4SOItype)*d_Iii_dV_gi_p;
d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] += (m*B4SOItype)*d_Iii_dV_b_GND;
d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] += (m*B4SOItype)*d_Iii_dV_p_GND;
d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] += (m*B4SOItype)*d_Iii_dTemp_t_GND;
d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] += (m*B4SOItype)*d_Iii_dV_di_si;
staticContributions[admsNodeID_b] -= ((m*B4SOItype)*Iii);
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] -= (m*B4SOItype)*d_Iii_dV_gi_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] -= (m*B4SOItype)*d_Iii_dV_e_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] -= (m*B4SOItype)*d_Iii_dV_b_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si] -= (m*B4SOItype)*d_Iii_dV_sb_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_db_di] -= (m*B4SOItype)*d_Iii_dV_db_di;
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p] -= (m*B4SOItype)*d_Iii_dV_gi_p;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] -= (m*B4SOItype)*d_Iii_dV_b_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] -= (m*B4SOItype)*d_Iii_dV_p_GND;
d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*d_Iii_dTemp_t_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -= (m*B4SOItype)*d_Iii_dV_di_si;

d_B4SOIigisl_dV_b_GND = B4SOItype*d_Igidl_dV_b_GND;
d_B4SOIigisl_dV_p_GND = B4SOItype*d_Igidl_dV_p_GND;
d_B4SOIigisl_dTemp_t_GND = B4SOItype*d_Igidl_dTemp_t_GND;
d_B4SOIigisl_dV_gi_si = B4SOItype*d_Igidl_dV_gi_si;
d_B4SOIigisl_dV_di_si = B4SOItype*d_Igidl_dV_di_si;
d_B4SOIigisl_dV_e_si = B4SOItype*d_Igidl_dV_e_si;
d_B4SOIigisl_dV_b_si = B4SOItype*d_Igidl_dV_b_si;
B4SOIigisl = (B4SOItype*Igidl);

d_B4SOIigidl_dV_b_GND = B4SOItype*d_Igisl_dV_b_GND;
d_B4SOIigidl_dV_p_GND = B4SOItype*d_Igisl_dV_p_GND;
d_B4SOIigidl_dTemp_t_GND = B4SOItype*d_Igisl_dTemp_t_GND;
d_B4SOIigidl_dV_gi_si = B4SOItype*d_Igisl_dV_gi_si;
d_B4SOIigidl_dV_di_si = B4SOItype*d_Igisl_dV_di_si;
d_B4SOIigidl_dV_e_si = B4SOItype*d_Igisl_dV_e_si;
d_B4SOIigidl_dV_b_si = B4SOItype*d_Igisl_dV_b_si;
B4SOIigidl = (B4SOItype*Igisl);

d_B4SOIIgcs_dV_b_GND = B4SOItype*d_Igcd_dV_b_GND;
d_B4SOIIgcs_dV_p_GND = B4SOItype*d_Igcd_dV_p_GND;
d_B4SOIIgcs_dTemp_t_GND = B4SOItype*d_Igcd_dTemp_t_GND;
d_B4SOIIgcs_dV_gi_si = B4SOItype*d_Igcd_dV_gi_si;
d_B4SOIIgcs_dV_di_si = B4SOItype*d_Igcd_dV_di_si;
d_B4SOIIgcs_dV_e_si = B4SOItype*d_Igcd_dV_e_si;
d_B4SOIIgcs_dV_b_si = B4SOItype*d_Igcd_dV_b_si;
d_B4SOIIgcs_dV_sb_si = B4SOItype*d_Igcd_dV_sb_si;
d_B4SOIIgcs_dV_db_di = B4SOItype*d_Igcd_dV_db_di;
B4SOIIgcs = (B4SOItype*Igcd);

d_B4SOIIgcd_dV_b_GND = B4SOItype*d_Igcs_dV_b_GND;
d_B4SOIIgcd_dV_p_GND = B4SOItype*d_Igcs_dV_p_GND;
d_B4SOIIgcd_dTemp_t_GND = B4SOItype*d_Igcs_dTemp_t_GND;
d_B4SOIIgcd_dV_gi_si = B4SOItype*d_Igcs_dV_gi_si;
d_B4SOIIgcd_dV_di_si = B4SOItype*d_Igcs_dV_di_si;
d_B4SOIIgcd_dV_e_si = B4SOItype*d_Igcs_dV_e_si;
d_B4SOIIgcd_dV_b_si = B4SOItype*d_Igcs_dV_b_si;
d_B4SOIIgcd_dV_sb_si = B4SOItype*d_Igcs_dV_sb_si;
d_B4SOIIgcd_dV_db_di = B4SOItype*d_Igcs_dV_db_di;
B4SOIIgcd = (B4SOItype*Igcs);

d_B4SOIqsrc_dV_e_si = B4SOItype*d_qdrn_dV_e_si;
d_B4SOIqsrc_dV_b_si = B4SOItype*d_qdrn_dV_b_si;
d_B4SOIqsrc_dV_b_GND = B4SOItype*d_qdrn_dV_b_GND;
d_B4SOIqsrc_dV_p_GND = B4SOItype*d_qdrn_dV_p_GND;
d_B4SOIqsrc_dTemp_t_GND = B4SOItype*d_qdrn_dTemp_t_GND;
d_B4SOIqsrc_dV_gi_si = B4SOItype*d_qdrn_dV_gi_si;
d_B4SOIqsrc_dV_di_si = B4SOItype*d_qdrn_dV_di_si;
d_B4SOIqsrc_dV_sb_si = B4SOItype*d_qdrn_dV_sb_si;
d_B4SOIqsrc_dV_db_di = B4SOItype*d_qdrn_dV_db_di;
d_B4SOIqsrc_dV_gi_p = B4SOItype*d_qdrn_dV_gi_p;
B4SOIqsrc = (B4SOItype*qdrn);

d_B4SOIqdrn_dV_e_si = B4SOItype*d_qsrc_dV_e_si;
d_B4SOIqdrn_dV_b_si = B4SOItype*d_qsrc_dV_b_si;
d_B4SOIqdrn_dV_b_GND = B4SOItype*d_qsrc_dV_b_GND;
d_B4SOIqdrn_dV_p_GND = B4SOItype*d_qsrc_dV_p_GND;
d_B4SOIqdrn_dTemp_t_GND = B4SOItype*d_qsrc_dTemp_t_GND;
d_B4SOIqdrn_dV_gi_si = B4SOItype*d_qsrc_dV_gi_si;
d_B4SOIqdrn_dV_di_si = B4SOItype*d_qsrc_dV_di_si;
d_B4SOIqdrn_dV_sb_si = B4SOItype*d_qsrc_dV_sb_si;
d_B4SOIqdrn_dV_db_di = B4SOItype*d_qsrc_dV_db_di;
d_B4SOIqdrn_dV_gi_p = B4SOItype*d_qsrc_dV_gi_p;
B4SOIqdrn = (B4SOItype*qsrc);
}

d_B4SOIIgd_dV_sb_si = B4SOItype*d_Igd_dV_sb_si;
d_B4SOIIgd_dV_db_di = B4SOItype*d_Igd_dV_db_di;
d_B4SOIIgd_dV_b_GND = B4SOItype*d_Igd_dV_b_GND;
d_B4SOIIgd_dV_p_GND = B4SOItype*d_Igd_dV_p_GND;
d_B4SOIIgd_dTemp_t_GND = B4SOItype*d_Igd_dTemp_t_GND;
d_B4SOIIgd_dV_e_si = B4SOItype*d_Igd_dV_e_si;
d_B4SOIIgd_dV_gi_si = B4SOItype*d_Igd_dV_gi_si;
d_B4SOIIgd_dV_di_si = B4SOItype*d_Igd_dV_di_si;
d_B4SOIIgd_dV_b_si = B4SOItype*d_Igd_dV_b_si;
B4SOIIgd = (B4SOItype*Igd);

d_B4SOIIgs_dV_sb_si = B4SOItype*d_Igs_dV_sb_si;
d_B4SOIIgs_dV_db_di = B4SOItype*d_Igs_dV_db_di;
d_B4SOIIgs_dV_b_GND = B4SOItype*d_Igs_dV_b_GND;
d_B4SOIIgs_dV_p_GND = B4SOItype*d_Igs_dV_p_GND;
d_B4SOIIgs_dTemp_t_GND = B4SOItype*d_Igs_dTemp_t_GND;
d_B4SOIIgs_dV_e_si = B4SOItype*d_Igs_dV_e_si;
d_B4SOIIgs_dV_gi_si = B4SOItype*d_Igs_dV_gi_si;
d_B4SOIIgs_dV_di_si = B4SOItype*d_Igs_dV_di_si;
d_B4SOIIgs_dV_b_si = B4SOItype*d_Igs_dV_b_si;
B4SOIIgs = (B4SOItype*Igs);
// I(di,b) <+ ((m*B4SOIigidl))
staticContributions[admsNodeID_di] += (m*B4SOIigidl);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] += m*d_B4SOIigidl_dV_b_GND;
d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] += m*d_B4SOIigidl_dV_p_GND;
d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] += m*d_B4SOIigidl_dTemp_t_GND;
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] += m*d_B4SOIigidl_dV_gi_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] += m*d_B4SOIigidl_dV_di_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] += m*d_B4SOIigidl_dV_e_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] += m*d_B4SOIigidl_dV_b_si;
staticContributions[admsNodeID_b] -= (m*B4SOIigidl);
d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] -= m*d_B4SOIigidl_dV_b_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] -= m*d_B4SOIigidl_dV_p_GND;
d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= m*d_B4SOIigidl_dTemp_t_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] -= m*d_B4SOIigidl_dV_gi_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -= m*d_B4SOIigidl_dV_di_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] -= m*d_B4SOIigidl_dV_e_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] -= m*d_B4SOIigidl_dV_b_si;
// I(si,b) <+ ((m*B4SOIigisl))
staticContributions[admsNodeID_si] += (m*B4SOIigisl);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] += m*d_B4SOIigisl_dV_b_GND;
d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] += m*d_B4SOIigisl_dV_p_GND;
d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] += m*d_B4SOIigisl_dTemp_t_GND;
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] += m*d_B4SOIigisl_dV_gi_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] += m*d_B4SOIigisl_dV_di_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] += m*d_B4SOIigisl_dV_e_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] += m*d_B4SOIigisl_dV_b_si;
staticContributions[admsNodeID_b] -= (m*B4SOIigisl);
d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] -= m*d_B4SOIigisl_dV_b_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] -= m*d_B4SOIigisl_dV_p_GND;
d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= m*d_B4SOIigisl_dTemp_t_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] -= m*d_B4SOIigisl_dV_gi_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -= m*d_B4SOIigisl_dV_di_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] -= m*d_B4SOIigisl_dV_e_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] -= m*d_B4SOIigisl_dV_b_si;
// I(db,di) <+ (((m*B4SOItype)*Ibd))
staticContributions[admsNodeID_db] += ((m*B4SOItype)*Ibd);
d_staticContributions[admsNodeID_db][admsProbeID_V_b_GND] += (m*B4SOItype)*d_Ibd_dV_b_GND;
d_staticContributions[admsNodeID_db][admsProbeID_V_p_GND] += (m*B4SOItype)*d_Ibd_dV_p_GND;
d_staticContributions[admsNodeID_db][admsProbeID_Temp_t_GND] += (m*B4SOItype)*d_Ibd_dTemp_t_GND;
d_staticContributions[admsNodeID_db][admsProbeID_V_e_si] += (m*B4SOItype)*d_Ibd_dV_e_si;
d_staticContributions[admsNodeID_db][admsProbeID_V_gi_si] += (m*B4SOItype)*d_Ibd_dV_gi_si;
d_staticContributions[admsNodeID_db][admsProbeID_V_di_si] += (m*B4SOItype)*d_Ibd_dV_di_si;
d_staticContributions[admsNodeID_db][admsProbeID_V_b_si] += (m*B4SOItype)*d_Ibd_dV_b_si;
d_staticContributions[admsNodeID_db][admsProbeID_V_sb_si] += (m*B4SOItype)*d_Ibd_dV_sb_si;
d_staticContributions[admsNodeID_db][admsProbeID_V_db_di] += (m*B4SOItype)*d_Ibd_dV_db_di;
staticContributions[admsNodeID_di] -= ((m*B4SOItype)*Ibd);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] -= (m*B4SOItype)*d_Ibd_dV_b_GND;
d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] -= (m*B4SOItype)*d_Ibd_dV_p_GND;
d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*d_Ibd_dTemp_t_GND;
d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] -= (m*B4SOItype)*d_Ibd_dV_e_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] -= (m*B4SOItype)*d_Ibd_dV_gi_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -= (m*B4SOItype)*d_Ibd_dV_di_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] -= (m*B4SOItype)*d_Ibd_dV_b_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si] -= (m*B4SOItype)*d_Ibd_dV_sb_si;
d_staticContributions[admsNodeID_di][admsProbeID_V_db_di] -= (m*B4SOItype)*d_Ibd_dV_db_di;
// I(sb,si) <+ (((m*B4SOItype)*Ibs))
staticContributions[admsNodeID_sb] += ((m*B4SOItype)*Ibs);
d_staticContributions[admsNodeID_sb][admsProbeID_V_b_GND] += (m*B4SOItype)*d_Ibs_dV_b_GND;
d_staticContributions[admsNodeID_sb][admsProbeID_V_p_GND] += (m*B4SOItype)*d_Ibs_dV_p_GND;
d_staticContributions[admsNodeID_sb][admsProbeID_Temp_t_GND] += (m*B4SOItype)*d_Ibs_dTemp_t_GND;
d_staticContributions[admsNodeID_sb][admsProbeID_V_e_si] += (m*B4SOItype)*d_Ibs_dV_e_si;
d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_si] += (m*B4SOItype)*d_Ibs_dV_gi_si;
d_staticContributions[admsNodeID_sb][admsProbeID_V_di_si] += (m*B4SOItype)*d_Ibs_dV_di_si;
d_staticContributions[admsNodeID_sb][admsProbeID_V_b_si] += (m*B4SOItype)*d_Ibs_dV_b_si;
d_staticContributions[admsNodeID_sb][admsProbeID_V_sb_si] += (m*B4SOItype)*d_Ibs_dV_sb_si;
d_staticContributions[admsNodeID_sb][admsProbeID_V_db_di] += (m*B4SOItype)*d_Ibs_dV_db_di;
staticContributions[admsNodeID_si] -= ((m*B4SOItype)*Ibs);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] -= (m*B4SOItype)*d_Ibs_dV_b_GND;
d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] -= (m*B4SOItype)*d_Ibs_dV_p_GND;
d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*d_Ibs_dTemp_t_GND;
d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] -= (m*B4SOItype)*d_Ibs_dV_e_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] -= (m*B4SOItype)*d_Ibs_dV_gi_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -= (m*B4SOItype)*d_Ibs_dV_di_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] -= (m*B4SOItype)*d_Ibs_dV_b_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si] -= (m*B4SOItype)*d_Ibs_dV_sb_si;
d_staticContributions[admsNodeID_si][admsProbeID_V_db_di] -= (m*B4SOItype)*d_Ibs_dV_db_di;
// I(gi,di) <+ ((m*(B4SOIIgd+B4SOIIgcd)))
staticContributions[admsNodeID_gi] += (m*(B4SOIIgd+B4SOIIgcd));
d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si] += m*(d_B4SOIIgd_dV_sb_si+d_B4SOIIgcd_dV_sb_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di] += m*(d_B4SOIIgd_dV_db_di+d_B4SOIIgcd_dV_db_di);
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND] += m*(d_B4SOIIgd_dV_b_GND+d_B4SOIIgcd_dV_b_GND);
d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND] += m*(d_B4SOIIgd_dV_p_GND+d_B4SOIIgcd_dV_p_GND);
d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] += m*(d_B4SOIIgd_dTemp_t_GND+d_B4SOIIgcd_dTemp_t_GND);
d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] += m*(d_B4SOIIgd_dV_e_si+d_B4SOIIgcd_dV_e_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] += m*(d_B4SOIIgd_dV_gi_si+d_B4SOIIgcd_dV_gi_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] += m*(d_B4SOIIgd_dV_di_si+d_B4SOIIgcd_dV_di_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] += m*(d_B4SOIIgd_dV_b_si+d_B4SOIIgcd_dV_b_si);
staticContributions[admsNodeID_di] -= (m*(B4SOIIgd+B4SOIIgcd));
d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si] -= m*(d_B4SOIIgd_dV_sb_si+d_B4SOIIgcd_dV_sb_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_db_di] -= m*(d_B4SOIIgd_dV_db_di+d_B4SOIIgcd_dV_db_di);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] -= m*(d_B4SOIIgd_dV_b_GND+d_B4SOIIgcd_dV_b_GND);
d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] -= m*(d_B4SOIIgd_dV_p_GND+d_B4SOIIgcd_dV_p_GND);
d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] -= m*(d_B4SOIIgd_dTemp_t_GND+d_B4SOIIgcd_dTemp_t_GND);
d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] -= m*(d_B4SOIIgd_dV_e_si+d_B4SOIIgcd_dV_e_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] -= m*(d_B4SOIIgd_dV_gi_si+d_B4SOIIgcd_dV_gi_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -= m*(d_B4SOIIgd_dV_di_si+d_B4SOIIgcd_dV_di_si);
d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] -= m*(d_B4SOIIgd_dV_b_si+d_B4SOIIgcd_dV_b_si);
// I(gi,si) <+ ((m*(B4SOIIgs+B4SOIIgcs)))
staticContributions[admsNodeID_gi] += (m*(B4SOIIgs+B4SOIIgcs));
d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si] += m*(d_B4SOIIgs_dV_sb_si+d_B4SOIIgcs_dV_sb_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di] += m*(d_B4SOIIgs_dV_db_di+d_B4SOIIgcs_dV_db_di);
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND] += m*(d_B4SOIIgs_dV_b_GND+d_B4SOIIgcs_dV_b_GND);
d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND] += m*(d_B4SOIIgs_dV_p_GND+d_B4SOIIgcs_dV_p_GND);
d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] += m*(d_B4SOIIgs_dTemp_t_GND+d_B4SOIIgcs_dTemp_t_GND);
d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] += m*(d_B4SOIIgs_dV_e_si+d_B4SOIIgcs_dV_e_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] += m*(d_B4SOIIgs_dV_gi_si+d_B4SOIIgcs_dV_gi_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] += m*(d_B4SOIIgs_dV_di_si+d_B4SOIIgcs_dV_di_si);
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] += m*(d_B4SOIIgs_dV_b_si+d_B4SOIIgcs_dV_b_si);
staticContributions[admsNodeID_si] -= (m*(B4SOIIgs+B4SOIIgcs));
d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si] -= m*(d_B4SOIIgs_dV_sb_si+d_B4SOIIgcs_dV_sb_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_db_di] -= m*(d_B4SOIIgs_dV_db_di+d_B4SOIIgcs_dV_db_di);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] -= m*(d_B4SOIIgs_dV_b_GND+d_B4SOIIgcs_dV_b_GND);
d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] -= m*(d_B4SOIIgs_dV_p_GND+d_B4SOIIgcs_dV_p_GND);
d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] -= m*(d_B4SOIIgs_dTemp_t_GND+d_B4SOIIgcs_dTemp_t_GND);
d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] -= m*(d_B4SOIIgs_dV_e_si+d_B4SOIIgcs_dV_e_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] -= m*(d_B4SOIIgs_dV_gi_si+d_B4SOIIgcs_dV_gi_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -= m*(d_B4SOIIgs_dV_di_si+d_B4SOIIgcs_dV_di_si);
d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] -= m*(d_B4SOIIgs_dV_b_si+d_B4SOIIgcs_dV_b_si);
// I(gi,b) <+ ((m*B4SOIig))
staticContributions[admsNodeID_gi] += (m*B4SOIig);
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND] += m*d_B4SOIig_dV_b_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND] += m*d_B4SOIig_dV_p_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] += m*d_B4SOIig_dTemp_t_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] += m*d_B4SOIig_dV_gi_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] += m*d_B4SOIig_dV_di_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] += m*d_B4SOIig_dV_e_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] += m*d_B4SOIig_dV_b_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si] += m*d_B4SOIig_dV_sb_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di] += m*d_B4SOIig_dV_db_di;
staticContributions[admsNodeID_b] -= (m*B4SOIig);
d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] -= m*d_B4SOIig_dV_b_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] -= m*d_B4SOIig_dV_p_GND;
d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= m*d_B4SOIig_dTemp_t_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] -= m*d_B4SOIig_dV_gi_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -= m*d_B4SOIig_dV_di_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] -= m*d_B4SOIig_dV_e_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] -= m*d_B4SOIig_dV_b_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si] -= m*d_B4SOIig_dV_sb_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_db_di] -= m*d_B4SOIig_dV_db_di;
// I(gi,p) <+ ((m*B4SOIigp))
staticContributions[admsNodeID_gi] += (m*B4SOIigp);
d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_p] += m*d_B4SOIigp_dV_gi_p;
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND] += m*d_B4SOIigp_dV_b_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND] += m*d_B4SOIigp_dV_p_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] += m*d_B4SOIigp_dTemp_t_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] += m*d_B4SOIigp_dV_gi_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] += m*d_B4SOIigp_dV_di_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] += m*d_B4SOIigp_dV_e_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] += m*d_B4SOIigp_dV_b_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si] += m*d_B4SOIigp_dV_sb_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di] += m*d_B4SOIigp_dV_db_di;
staticContributions[admsNodeID_p] -= (m*B4SOIigp);
d_staticContributions[admsNodeID_p][admsProbeID_V_gi_p] -= m*d_B4SOIigp_dV_gi_p;
d_staticContributions[admsNodeID_p][admsProbeID_V_b_GND] -= m*d_B4SOIigp_dV_b_GND;
d_staticContributions[admsNodeID_p][admsProbeID_V_p_GND] -= m*d_B4SOIigp_dV_p_GND;
d_staticContributions[admsNodeID_p][admsProbeID_Temp_t_GND] -= m*d_B4SOIigp_dTemp_t_GND;
d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si] -= m*d_B4SOIigp_dV_gi_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_di_si] -= m*d_B4SOIigp_dV_di_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_e_si] -= m*d_B4SOIigp_dV_e_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] -= m*d_B4SOIigp_dV_b_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_sb_si] -= m*d_B4SOIigp_dV_sb_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_db_di] -= m*d_B4SOIigp_dV_db_di;
if (((B4SOIbodyMod==0)||(B4SOIbodyMod==2)))
{
// V(b,p) <+ (0)
// do nothing at all
}
else
{
// I(b,p) <+ (((m*B4SOItype)*Ibp))
staticContributions[admsNodeID_b] += ((m*B4SOItype)*Ibp);
d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] += (m*B4SOItype)*d_Ibp_dV_b_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] += (m*B4SOItype)*d_Ibp_dV_p_GND;
d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] += (m*B4SOItype)*d_Ibp_dTemp_t_GND;
d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] += (m*B4SOItype)*d_Ibp_dV_e_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] += (m*B4SOItype)*d_Ibp_dV_gi_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] += (m*B4SOItype)*d_Ibp_dV_di_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] += (m*B4SOItype)*d_Ibp_dV_b_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si] += (m*B4SOItype)*d_Ibp_dV_sb_si;
d_staticContributions[admsNodeID_b][admsProbeID_V_db_di] += (m*B4SOItype)*d_Ibp_dV_db_di;
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p] += (m*B4SOItype)*d_Ibp_dV_gi_p;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_p] += (m*B4SOItype)*d_Ibp_dV_b_p;
staticContributions[admsNodeID_p] -= ((m*B4SOItype)*Ibp);
d_staticContributions[admsNodeID_p][admsProbeID_V_b_GND] -= (m*B4SOItype)*d_Ibp_dV_b_GND;
d_staticContributions[admsNodeID_p][admsProbeID_V_p_GND] -= (m*B4SOItype)*d_Ibp_dV_p_GND;
d_staticContributions[admsNodeID_p][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*d_Ibp_dTemp_t_GND;
d_staticContributions[admsNodeID_p][admsProbeID_V_e_si] -= (m*B4SOItype)*d_Ibp_dV_e_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si] -= (m*B4SOItype)*d_Ibp_dV_gi_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_di_si] -= (m*B4SOItype)*d_Ibp_dV_di_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] -= (m*B4SOItype)*d_Ibp_dV_b_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_sb_si] -= (m*B4SOItype)*d_Ibp_dV_sb_si;
d_staticContributions[admsNodeID_p][admsProbeID_V_db_di] -= (m*B4SOItype)*d_Ibp_dV_db_di;
d_staticContributions[admsNodeID_p][admsProbeID_V_gi_p] -= (m*B4SOItype)*d_Ibp_dV_gi_p;
d_staticContributions[admsNodeID_p][admsProbeID_V_b_p] -= (m*B4SOItype)*d_Ibp_dV_b_p;
// I(b,p) <+ (white_noise((((m*fourkt)*abs(Ibp))/(abs(vbp)+1.0e-9)),"rbp"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs(Ibp);
double value_fabs_1 = fabs(vbp);
  noiseContribsPower[6]= (((m*fourkt)*value_fabs_0)/(value_fabs_1+1.0e-9));
}
}
// I(db,di) <+ (white_noise(((((m*2)*1.60219e-19)*B4SOInoif)*abs(Ibd)),"ibd"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs(Ibd);
  noiseContribsPower[7]= ((((m*2)*1.60219e-19)*B4SOInoif)*value_fabs_0);
}
// I(sb,si) <+ (white_noise(((((m*2)*1.60219e-19)*B4SOInoif)*abs(Ibs)),"ibs"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs(Ibs);
  noiseContribsPower[8]= ((((m*2)*1.60219e-19)*B4SOInoif)*value_fabs_0);
}
// I(gi,di) <+ (white_noise((((m*2)*1.60219e-19)*abs((B4SOIIgd+B4SOIIgcd))),"igd"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((B4SOIIgd+B4SOIIgcd));
  noiseContribsPower[9]= (((m*2)*1.60219e-19)*value_fabs_0);
}
// I(gi,si) <+ (white_noise((((m*2)*1.60219e-19)*abs((B4SOIIgs+B4SOIIgcs))),"igs"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((B4SOIIgs+B4SOIIgcs));
  noiseContribsPower[10]= (((m*2)*1.60219e-19)*value_fabs_0);
}
// I(gi,b) <+ (white_noise((((m*2)*1.60219e-19)*abs(B4SOIig)),"igb"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs(B4SOIig);
  noiseContribsPower[11]= (((m*2)*1.60219e-19)*value_fabs_0);
}
// I(di,b) <+ ((m*ddt(B4SOIqdrn)))
dynamicContributions[admsNodeID_di] += (m*(B4SOIqdrn));
d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si] += m*(d_B4SOIqdrn_dV_e_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] += m*(d_B4SOIqdrn_dV_b_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] += m*(d_B4SOIqdrn_dV_b_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND] += m*(d_B4SOIqdrn_dV_p_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND] += m*(d_B4SOIqdrn_dTemp_t_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] += m*(d_B4SOIqdrn_dV_gi_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] += m*(d_B4SOIqdrn_dV_di_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] += m*(d_B4SOIqdrn_dV_sb_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] += m*(d_B4SOIqdrn_dV_db_di);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] += m*(d_B4SOIqdrn_dV_gi_p);
dynamicContributions[admsNodeID_b] -= (m*(B4SOIqdrn));
d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si] -= m*(d_B4SOIqdrn_dV_e_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si] -= m*(d_B4SOIqdrn_dV_b_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] -= m*(d_B4SOIqdrn_dV_b_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND] -= m*(d_B4SOIqdrn_dV_p_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= m*(d_B4SOIqdrn_dTemp_t_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si] -= m*(d_B4SOIqdrn_dV_gi_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si] -= m*(d_B4SOIqdrn_dV_di_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si] -= m*(d_B4SOIqdrn_dV_sb_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di] -= m*(d_B4SOIqdrn_dV_db_di);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] -= m*(d_B4SOIqdrn_dV_gi_p);
// I(si,b) <+ ((m*ddt(B4SOIqsrc)))
dynamicContributions[admsNodeID_si] += (m*(B4SOIqsrc));
d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si] += m*(d_B4SOIqsrc_dV_e_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] += m*(d_B4SOIqsrc_dV_b_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] += m*(d_B4SOIqsrc_dV_b_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND] += m*(d_B4SOIqsrc_dV_p_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND] += m*(d_B4SOIqsrc_dTemp_t_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] += m*(d_B4SOIqsrc_dV_gi_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] += m*(d_B4SOIqsrc_dV_di_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] += m*(d_B4SOIqsrc_dV_sb_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] += m*(d_B4SOIqsrc_dV_db_di);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] += m*(d_B4SOIqsrc_dV_gi_p);
dynamicContributions[admsNodeID_b] -= (m*(B4SOIqsrc));
d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si] -= m*(d_B4SOIqsrc_dV_e_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si] -= m*(d_B4SOIqsrc_dV_b_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] -= m*(d_B4SOIqsrc_dV_b_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND] -= m*(d_B4SOIqsrc_dV_p_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= m*(d_B4SOIqsrc_dTemp_t_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si] -= m*(d_B4SOIqsrc_dV_gi_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si] -= m*(d_B4SOIqsrc_dV_di_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si] -= m*(d_B4SOIqsrc_dV_sb_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di] -= m*(d_B4SOIqsrc_dV_db_di);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] -= m*(d_B4SOIqsrc_dV_gi_p);
// I(gi,b) <+ (((m*B4SOItype)*ddt(qgate)))
dynamicContributions[admsNodeID_gi] += ((m*B4SOItype)*(qgate));
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qgate_dV_e_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qgate_dV_b_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qgate_dV_b_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qgate_dV_p_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qgate_dTemp_t_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qgate_dV_gi_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qgate_dV_di_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si] += (m*B4SOItype)*(d_qgate_dV_sb_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di] += (m*B4SOItype)*(d_qgate_dV_db_di);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] += (m*B4SOItype)*(d_qgate_dV_gi_p);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si] += (m*B4SOItype)*(d_qgate_dV_gm_si);
dynamicContributions[admsNodeID_b] -= ((m*B4SOItype)*(qgate));
d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qgate_dV_e_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qgate_dV_b_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qgate_dV_b_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qgate_dV_p_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qgate_dTemp_t_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qgate_dV_gi_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qgate_dV_di_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si] -= (m*B4SOItype)*(d_qgate_dV_sb_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di] -= (m*B4SOItype)*(d_qgate_dV_db_di);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] -= (m*B4SOItype)*(d_qgate_dV_gi_p);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_si] -= (m*B4SOItype)*(d_qgate_dV_gm_si);
// I(e,b) <+ (((m*B4SOItype)*ddt(qsub)))
dynamicContributions[admsNodeID_e] += ((m*B4SOItype)*(qsub));
d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qsub_dV_b_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qsub_dV_gi_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qsub_dV_b_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qsub_dV_p_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qsub_dTemp_t_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qsub_dV_e_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qsub_dV_di_si);
dynamicContributions[admsNodeID_b] -= ((m*B4SOItype)*(qsub));
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qsub_dV_b_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qsub_dV_gi_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qsub_dV_b_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qsub_dV_p_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qsub_dTemp_t_GND);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qsub_dV_e_si);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qsub_dV_di_si);
// I(db,di) <+ (((m*B4SOItype)*ddt(qjd)))
dynamicContributions[admsNodeID_db] += ((m*B4SOItype)*(qjd));
d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qjd_dV_gi_si);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qjd_dV_di_si);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qjd_dV_e_si);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qjd_dV_b_si);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_sb_si] += (m*B4SOItype)*(d_qjd_dV_sb_si);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_db_di] += (m*B4SOItype)*(d_qjd_dV_db_di);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_p] += (m*B4SOItype)*(d_qjd_dV_gi_p);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qjd_dV_b_GND);
d_dynamicContributions[admsNodeID_db][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qjd_dV_p_GND);
d_dynamicContributions[admsNodeID_db][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qjd_dTemp_t_GND);
dynamicContributions[admsNodeID_di] -= ((m*B4SOItype)*(qjd));
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qjd_dV_gi_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qjd_dV_di_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qjd_dV_e_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qjd_dV_b_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] -= (m*B4SOItype)*(d_qjd_dV_sb_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] -= (m*B4SOItype)*(d_qjd_dV_db_di);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] -= (m*B4SOItype)*(d_qjd_dV_gi_p);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qjd_dV_b_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qjd_dV_p_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qjd_dTemp_t_GND);
// I(sb,si) <+ (((m*B4SOItype)*ddt(qjs)))
dynamicContributions[admsNodeID_sb] += ((m*B4SOItype)*(qjs));
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qjs_dV_gi_si);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qjs_dV_di_si);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qjs_dV_e_si);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qjs_dV_b_si);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_sb_si] += (m*B4SOItype)*(d_qjs_dV_sb_si);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_db_di] += (m*B4SOItype)*(d_qjs_dV_db_di);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_p] += (m*B4SOItype)*(d_qjs_dV_gi_p);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qjs_dV_b_GND);
d_dynamicContributions[admsNodeID_sb][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qjs_dV_p_GND);
d_dynamicContributions[admsNodeID_sb][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qjs_dTemp_t_GND);
dynamicContributions[admsNodeID_si] -= ((m*B4SOItype)*(qjs));
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qjs_dV_gi_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qjs_dV_di_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qjs_dV_e_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qjs_dV_b_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] -= (m*B4SOItype)*(d_qjs_dV_sb_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] -= (m*B4SOItype)*(d_qjs_dV_db_di);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] -= (m*B4SOItype)*(d_qjs_dV_gi_p);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qjs_dV_b_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qjs_dV_p_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qjs_dTemp_t_GND);
if (((model_.RGATEMOD)==3))
{
// I(gm,di) <+ (((m*B4SOItype)*ddt(qgdo)))
dynamicContributions[admsNodeID_gm] += ((m*B4SOItype)*(qgdo));
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_si] += (m*B4SOItype)*(d_qgdo_dV_gm_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qgdo_dV_b_GND);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qgdo_dV_p_GND);
d_dynamicContributions[admsNodeID_gm][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qgdo_dTemp_t_GND);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qgdo_dV_gi_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qgdo_dV_di_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qgdo_dV_e_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qgdo_dV_b_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_si] += (m*B4SOItype)*(d_qgdo_dV_sb_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_di] += (m*B4SOItype)*(d_qgdo_dV_db_di);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_p] += (m*B4SOItype)*(d_qgdo_dV_gi_p);
dynamicContributions[admsNodeID_di] -= ((m*B4SOItype)*(qgdo));
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_si] -= (m*B4SOItype)*(d_qgdo_dV_gm_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qgdo_dV_b_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qgdo_dV_p_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qgdo_dTemp_t_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qgdo_dV_gi_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qgdo_dV_di_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qgdo_dV_e_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qgdo_dV_b_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] -= (m*B4SOItype)*(d_qgdo_dV_sb_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] -= (m*B4SOItype)*(d_qgdo_dV_db_di);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] -= (m*B4SOItype)*(d_qgdo_dV_gi_p);
// I(gm,si) <+ (((m*B4SOItype)*ddt(qgso)))
dynamicContributions[admsNodeID_gm] += ((m*B4SOItype)*(qgso));
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_si] += (m*B4SOItype)*(d_qgso_dV_gm_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qgso_dV_b_GND);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qgso_dV_p_GND);
d_dynamicContributions[admsNodeID_gm][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qgso_dTemp_t_GND);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qgso_dV_gi_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qgso_dV_di_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qgso_dV_e_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qgso_dV_b_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_si] += (m*B4SOItype)*(d_qgso_dV_sb_si);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_di] += (m*B4SOItype)*(d_qgso_dV_db_di);
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_p] += (m*B4SOItype)*(d_qgso_dV_gi_p);
dynamicContributions[admsNodeID_si] -= ((m*B4SOItype)*(qgso));
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_si] -= (m*B4SOItype)*(d_qgso_dV_gm_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qgso_dV_b_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qgso_dV_p_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qgso_dTemp_t_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qgso_dV_gi_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qgso_dV_di_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qgso_dV_e_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qgso_dV_b_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] -= (m*B4SOItype)*(d_qgso_dV_sb_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] -= (m*B4SOItype)*(d_qgso_dV_db_di);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] -= (m*B4SOItype)*(d_qgso_dV_gi_p);
// I(gm,e) <+ ((m*ddt((V(gm,e)*pParam_B4SOIcgeo))))
dynamicContributions[admsNodeID_gm] += (m*(((probeVars[admsProbeID_V_gm_e])*pParam_B4SOIcgeo)));
d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_e] += m*(d_probeVars[admsProbeID_V_gm_e][admsProbeID_V_gm_e]*pParam_B4SOIcgeo);
dynamicContributions[admsNodeID_e] -= (m*(((probeVars[admsProbeID_V_gm_e])*pParam_B4SOIcgeo)));
d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_e] -= m*(d_probeVars[admsProbeID_V_gm_e][admsProbeID_V_gm_e]*pParam_B4SOIcgeo);
}
else
{
// I(gi,di) <+ (((m*B4SOItype)*ddt(qgdo)))
dynamicContributions[admsNodeID_gi] += ((m*B4SOItype)*(qgdo));
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si] += (m*B4SOItype)*(d_qgdo_dV_gm_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qgdo_dV_b_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qgdo_dV_p_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qgdo_dTemp_t_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qgdo_dV_gi_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qgdo_dV_di_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qgdo_dV_e_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qgdo_dV_b_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si] += (m*B4SOItype)*(d_qgdo_dV_sb_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di] += (m*B4SOItype)*(d_qgdo_dV_db_di);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] += (m*B4SOItype)*(d_qgdo_dV_gi_p);
dynamicContributions[admsNodeID_di] -= ((m*B4SOItype)*(qgdo));
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_si] -= (m*B4SOItype)*(d_qgdo_dV_gm_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qgdo_dV_b_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qgdo_dV_p_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qgdo_dTemp_t_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qgdo_dV_gi_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qgdo_dV_di_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qgdo_dV_e_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qgdo_dV_b_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] -= (m*B4SOItype)*(d_qgdo_dV_sb_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] -= (m*B4SOItype)*(d_qgdo_dV_db_di);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] -= (m*B4SOItype)*(d_qgdo_dV_gi_p);
// I(gi,si) <+ (((m*B4SOItype)*ddt(qgso)))
dynamicContributions[admsNodeID_gi] += ((m*B4SOItype)*(qgso));
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si] += (m*B4SOItype)*(d_qgso_dV_gm_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_GND] += (m*B4SOItype)*(d_qgso_dV_b_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_p_GND] += (m*B4SOItype)*(d_qgso_dV_p_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] += (m*B4SOItype)*(d_qgso_dTemp_t_GND);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si] += (m*B4SOItype)*(d_qgso_dV_gi_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si] += (m*B4SOItype)*(d_qgso_dV_di_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si] += (m*B4SOItype)*(d_qgso_dV_e_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si] += (m*B4SOItype)*(d_qgso_dV_b_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si] += (m*B4SOItype)*(d_qgso_dV_sb_si);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di] += (m*B4SOItype)*(d_qgso_dV_db_di);
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] += (m*B4SOItype)*(d_qgso_dV_gi_p);
dynamicContributions[admsNodeID_si] -= ((m*B4SOItype)*(qgso));
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_si] -= (m*B4SOItype)*(d_qgso_dV_gm_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] -= (m*B4SOItype)*(d_qgso_dV_b_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND] -= (m*B4SOItype)*(d_qgso_dV_p_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND] -= (m*B4SOItype)*(d_qgso_dTemp_t_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] -= (m*B4SOItype)*(d_qgso_dV_gi_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] -= (m*B4SOItype)*(d_qgso_dV_di_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si] -= (m*B4SOItype)*(d_qgso_dV_e_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] -= (m*B4SOItype)*(d_qgso_dV_b_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] -= (m*B4SOItype)*(d_qgso_dV_sb_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] -= (m*B4SOItype)*(d_qgso_dV_db_di);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] -= (m*B4SOItype)*(d_qgso_dV_gi_p);
// I(gi,e) <+ ((m*ddt((V(gi,e)*pParam_B4SOIcgeo))))
dynamicContributions[admsNodeID_gi] += (m*(((probeVars[admsProbeID_V_gi_e])*pParam_B4SOIcgeo)));
d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_e] += m*(d_probeVars[admsProbeID_V_gi_e][admsProbeID_V_gi_e]*pParam_B4SOIcgeo);
dynamicContributions[admsNodeID_e] -= (m*(((probeVars[admsProbeID_V_gi_e])*pParam_B4SOIcgeo)));
d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_e] -= m*(d_probeVars[admsProbeID_V_gi_e][admsProbeID_V_gi_e]*pParam_B4SOIcgeo);
}
// I(di,e) <+ ((m*ddt(B4SOIqde)))
dynamicContributions[admsNodeID_di] += (m*(B4SOIqde));
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] += m*(d_B4SOIqde_dV_b_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND] += m*(d_B4SOIqde_dV_p_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND] += m*(d_B4SOIqde_dTemp_t_GND);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] += m*(d_B4SOIqde_dV_gi_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] += m*(d_B4SOIqde_dV_di_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si] += m*(d_B4SOIqde_dV_e_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] += m*(d_B4SOIqde_dV_b_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] += m*(d_B4SOIqde_dV_sb_si);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] += m*(d_B4SOIqde_dV_db_di);
d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] += m*(d_B4SOIqde_dV_gi_p);
dynamicContributions[admsNodeID_e] -= (m*(B4SOIqde));
d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_GND] -= m*(d_B4SOIqde_dV_b_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_p_GND] -= m*(d_B4SOIqde_dV_p_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_Temp_t_GND] -= m*(d_B4SOIqde_dTemp_t_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si] -= m*(d_B4SOIqde_dV_gi_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si] -= m*(d_B4SOIqde_dV_di_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si] -= m*(d_B4SOIqde_dV_e_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si] -= m*(d_B4SOIqde_dV_b_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_si] -= m*(d_B4SOIqde_dV_sb_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_di] -= m*(d_B4SOIqde_dV_db_di);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_p] -= m*(d_B4SOIqde_dV_gi_p);
// I(si,e) <+ ((m*ddt(B4SOIqse)))
dynamicContributions[admsNodeID_si] += (m*(B4SOIqse));
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] += m*(d_B4SOIqse_dV_b_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND] += m*(d_B4SOIqse_dV_p_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND] += m*(d_B4SOIqse_dTemp_t_GND);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] += m*(d_B4SOIqse_dV_gi_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] += m*(d_B4SOIqse_dV_di_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si] += m*(d_B4SOIqse_dV_e_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] += m*(d_B4SOIqse_dV_b_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] += m*(d_B4SOIqse_dV_sb_si);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] += m*(d_B4SOIqse_dV_db_di);
d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] += m*(d_B4SOIqse_dV_gi_p);
dynamicContributions[admsNodeID_e] -= (m*(B4SOIqse));
d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_GND] -= m*(d_B4SOIqse_dV_b_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_p_GND] -= m*(d_B4SOIqse_dV_p_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_Temp_t_GND] -= m*(d_B4SOIqse_dTemp_t_GND);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si] -= m*(d_B4SOIqse_dV_gi_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si] -= m*(d_B4SOIqse_dV_di_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si] -= m*(d_B4SOIqse_dV_e_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si] -= m*(d_B4SOIqse_dV_b_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_si] -= m*(d_B4SOIqse_dV_sb_si);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_di] -= m*(d_B4SOIqse_dV_db_di);
d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_p] -= m*(d_B4SOIqse_dV_gi_p);
if ((((model_.RGATEMOD)==0)||((model_.RGATEMOD)==2)))
{
// V(g,gm) <+ (0)
// do nothing at all
}
else
{
// I(g,gm) <+ (((m*V(g,gm))*B4SOIgrgeltd))
staticContributions[admsNodeID_g] += ((m*(probeVars[admsProbeID_V_g_gm]))*B4SOIgrgeltd);
d_staticContributions[admsNodeID_g][admsProbeID_V_g_gm] += m*d_probeVars[admsProbeID_V_g_gm][admsProbeID_V_g_gm]*B4SOIgrgeltd;
staticContributions[admsNodeID_gm] -= ((m*(probeVars[admsProbeID_V_g_gm]))*B4SOIgrgeltd);
d_staticContributions[admsNodeID_gm][admsProbeID_V_g_gm] -= m*d_probeVars[admsProbeID_V_g_gm][admsProbeID_V_g_gm]*B4SOIgrgeltd;
// I(g,gm) <+ (white_noise((m*abs((fourkt*B4SOIgrgeltd))),"rg"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*B4SOIgrgeltd));
  noiseContribsPower[12]= (m*value_fabs_0);
}
}
if ((((model_.RGATEMOD)==0)||((model_.RGATEMOD)==1)))
{
// V(gm,gi) <+ (0)
// do nothing at all
}
else
{
// I(gm,gi) <+ (((m*V(gm,gi))*B4SOIgcrg))
staticContributions[admsNodeID_gm] += ((m*(probeVars[admsProbeID_V_gm_gi]))*B4SOIgcrg);
d_staticContributions[admsNodeID_gm][admsProbeID_V_b_GND] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_b_GND;
d_staticContributions[admsNodeID_gm][admsProbeID_V_p_GND] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_p_GND;
d_staticContributions[admsNodeID_gm][admsProbeID_Temp_t_GND] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dTemp_t_GND;
d_staticContributions[admsNodeID_gm][admsProbeID_V_e_si] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_e_si;
d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_si] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_gi_si;
d_staticContributions[admsNodeID_gm][admsProbeID_V_di_si] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_di_si;
d_staticContributions[admsNodeID_gm][admsProbeID_V_b_si] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_b_si;
d_staticContributions[admsNodeID_gm][admsProbeID_V_sb_si] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_sb_si;
d_staticContributions[admsNodeID_gm][admsProbeID_V_db_di] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_db_di;
d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_p] += (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_gi_p;
d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_gi] += m*d_probeVars[admsProbeID_V_gm_gi][admsProbeID_V_gm_gi]*B4SOIgcrg;
staticContributions[admsNodeID_gi] -= ((m*(probeVars[admsProbeID_V_gm_gi]))*B4SOIgcrg);
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_b_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_p_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dTemp_t_GND;
d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_e_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_gi_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_di_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_b_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_sb_si;
d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_db_di;
d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_p] -= (m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dV_gi_p;
d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_gi] -= m*d_probeVars[admsProbeID_V_gm_gi][admsProbeID_V_gm_gi]*B4SOIgcrg;
if (((model_.RGATEMOD)==2))
//Begin block rgate2
{
//Block-local variables for block rgate2
double T0=0.0;
//End of Block-local variables
T0 = (1.0+(B4SOIgrgeltd/B4SOIgcrg));
// I(gm,gi) <+ (white_noise((m*abs(((fourkt*B4SOIgrgeltd)/(T0*T0)))),"rg"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs(((fourkt*B4SOIgrgeltd)/(T0*T0)));
  noiseContribsPower[13]= (m*value_fabs_0);
}
}
// End block rgate2
}
if (B4SOIrbodyMod)
{
// I(b,db) <+ (((m*V(b,db))*B4SOIgrbdb))
staticContributions[admsNodeID_b] += ((m*(probeVars[admsProbeID_V_b_db]))*B4SOIgrbdb);
d_staticContributions[admsNodeID_b][admsProbeID_V_b_db] += m*d_probeVars[admsProbeID_V_b_db][admsProbeID_V_b_db]*B4SOIgrbdb;
staticContributions[admsNodeID_db] -= ((m*(probeVars[admsProbeID_V_b_db]))*B4SOIgrbdb);
d_staticContributions[admsNodeID_db][admsProbeID_V_b_db] -= m*d_probeVars[admsProbeID_V_b_db][admsProbeID_V_b_db]*B4SOIgrbdb;
// I(b,sb) <+ (((m*V(b,sb))*B4SOIgrbsb))
staticContributions[admsNodeID_b] += ((m*(probeVars[admsProbeID_V_b_sb]))*B4SOIgrbsb);
d_staticContributions[admsNodeID_b][admsProbeID_V_b_sb] += m*d_probeVars[admsProbeID_V_b_sb][admsProbeID_V_b_sb]*B4SOIgrbsb;
staticContributions[admsNodeID_sb] -= ((m*(probeVars[admsProbeID_V_b_sb]))*B4SOIgrbsb);
d_staticContributions[admsNodeID_sb][admsProbeID_V_b_sb] -= m*d_probeVars[admsProbeID_V_b_sb][admsProbeID_V_b_sb]*B4SOIgrbsb;
// I(b,db) <+ (white_noise((m*abs((fourkt*B4SOIgrbdb))),"rbdb"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*B4SOIgrbdb));
  noiseContribsPower[14]= (m*value_fabs_0);
}
// I(b,sb) <+ (white_noise((m*abs((fourkt*B4SOIgrbsb))),"rbsb"))
if (getSolverState().noiseFlag)
{
double value_fabs_0 = fabs((fourkt*B4SOIgrbsb));
  noiseContribsPower[15]= (m*value_fabs_0);
}
}
else
{
// V(b,db) <+ (0)
// do nothing at all
// V(b,sb) <+ (0)
// do nothing at all
}
if ((B4SOIsoiMod==2))
{
// V(b,si) <+ (0)
// do nothing at all
}
if (((SHMOD==1)&&(RTH0!=0.0)))
{
if ((TNODEOUT&&(!portsConnected_[admsNodeID_t])))
{
if (portsConnected_[admsNodeID_b])
{
// I(b,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
staticContributions[admsNodeID_b] += (m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth)));
d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] += m*(-d_Ids_dV_gi_si)*Vds;
d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] += m*((-Ids)*d_Vds_dV_di_si+(-d_Ids_dV_di_si)*Vds);
d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] += m*(-d_Ids_dV_e_si)*Vds;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] += m*(-d_Ids_dV_b_si)*Vds;
d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] += m*((-d_Ids_dV_b_GND)*Vds+(d_delTemp_dV_b_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] += m*((-d_Ids_dV_p_GND)*Vds+(d_delTemp_dV_p_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] += m*((-d_Ids_dTemp_t_GND)*Vds+(d_delTemp_dTemp_t_GND/pParam_B4SOIrth));
// I(b,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
dynamicContributions[admsNodeID_b] += (m*((delTemp*pParam_B4SOIcth)));
d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] += m*(d_delTemp_dV_b_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND] += m*(d_delTemp_dV_p_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND] += m*(d_delTemp_dTemp_t_GND*pParam_B4SOIcth);
}
else
{
if (portsConnected_[admsNodeID_p])
{
// I(p,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
staticContributions[admsNodeID_p] += (m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth)));
d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si] += m*(-d_Ids_dV_gi_si)*Vds;
d_staticContributions[admsNodeID_p][admsProbeID_V_di_si] += m*((-Ids)*d_Vds_dV_di_si+(-d_Ids_dV_di_si)*Vds);
d_staticContributions[admsNodeID_p][admsProbeID_V_e_si] += m*(-d_Ids_dV_e_si)*Vds;
d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] += m*(-d_Ids_dV_b_si)*Vds;
d_staticContributions[admsNodeID_p][admsProbeID_V_b_GND] += m*((-d_Ids_dV_b_GND)*Vds+(d_delTemp_dV_b_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_p][admsProbeID_V_p_GND] += m*((-d_Ids_dV_p_GND)*Vds+(d_delTemp_dV_p_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_p][admsProbeID_Temp_t_GND] += m*((-d_Ids_dTemp_t_GND)*Vds+(d_delTemp_dTemp_t_GND/pParam_B4SOIrth));
// I(p,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
dynamicContributions[admsNodeID_p] += (m*((delTemp*pParam_B4SOIcth)));
d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_GND] += m*(d_delTemp_dV_b_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_p][admsProbeID_V_p_GND] += m*(d_delTemp_dV_p_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_p][admsProbeID_Temp_t_GND] += m*(d_delTemp_dTemp_t_GND*pParam_B4SOIcth);
}
else
{
// Pwr(t,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
staticContributions[admsNodeID_t] += (m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth)));
d_staticContributions[admsNodeID_t][admsProbeID_V_gi_si] += m*(-d_Ids_dV_gi_si)*Vds;
d_staticContributions[admsNodeID_t][admsProbeID_V_di_si] += m*((-Ids)*d_Vds_dV_di_si+(-d_Ids_dV_di_si)*Vds);
d_staticContributions[admsNodeID_t][admsProbeID_V_e_si] += m*(-d_Ids_dV_e_si)*Vds;
d_staticContributions[admsNodeID_t][admsProbeID_V_b_si] += m*(-d_Ids_dV_b_si)*Vds;
d_staticContributions[admsNodeID_t][admsProbeID_V_b_GND] += m*((-d_Ids_dV_b_GND)*Vds+(d_delTemp_dV_b_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_t][admsProbeID_V_p_GND] += m*((-d_Ids_dV_p_GND)*Vds+(d_delTemp_dV_p_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_t][admsProbeID_Temp_t_GND] += m*((-d_Ids_dTemp_t_GND)*Vds+(d_delTemp_dTemp_t_GND/pParam_B4SOIrth));
// Pwr(t,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
dynamicContributions[admsNodeID_t] += (m*((delTemp*pParam_B4SOIcth)));
d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_GND] += m*(d_delTemp_dV_b_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_t][admsProbeID_V_p_GND] += m*(d_delTemp_dV_p_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_t][admsProbeID_Temp_t_GND] += m*(d_delTemp_dTemp_t_GND*pParam_B4SOIcth);
}
}
}
else
{
// Pwr(t,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
staticContributions[admsNodeID_t] += (m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth)));
d_staticContributions[admsNodeID_t][admsProbeID_V_gi_si] += m*(-d_Ids_dV_gi_si)*Vds;
d_staticContributions[admsNodeID_t][admsProbeID_V_di_si] += m*((-Ids)*d_Vds_dV_di_si+(-d_Ids_dV_di_si)*Vds);
d_staticContributions[admsNodeID_t][admsProbeID_V_e_si] += m*(-d_Ids_dV_e_si)*Vds;
d_staticContributions[admsNodeID_t][admsProbeID_V_b_si] += m*(-d_Ids_dV_b_si)*Vds;
d_staticContributions[admsNodeID_t][admsProbeID_V_b_GND] += m*((-d_Ids_dV_b_GND)*Vds+(d_delTemp_dV_b_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_t][admsProbeID_V_p_GND] += m*((-d_Ids_dV_p_GND)*Vds+(d_delTemp_dV_p_GND/pParam_B4SOIrth));
d_staticContributions[admsNodeID_t][admsProbeID_Temp_t_GND] += m*((-d_Ids_dTemp_t_GND)*Vds+(d_delTemp_dTemp_t_GND/pParam_B4SOIrth));
// Pwr(t,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
dynamicContributions[admsNodeID_t] += (m*((delTemp*pParam_B4SOIcth)));
d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_GND] += m*(d_delTemp_dV_b_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_t][admsProbeID_V_p_GND] += m*(d_delTemp_dV_p_GND*pParam_B4SOIcth);
d_dynamicContributions[admsNodeID_t][admsProbeID_Temp_t_GND] += m*(d_delTemp_dTemp_t_GND*pParam_B4SOIcth);
}
}
else
{
if ((TNODEOUT&&(!portsConnected_[admsNodeID_t])))
{
if (portsConnected_[admsNodeID_b])
{
UserInfo(*this)  << "Error:  you have specified a temperature node as the 6th node with TNODEOUT=0 and self-heating disabled.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 6th node." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(*this)  << "Error:  you have specified a temperature node as the 5th node with TNODEOUT=0 and self-heating disabled.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 5th node." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
else
{
UserInfo(*this)  << "Error:  you have specified a temperature node as the 7th node with TNODEOUT=0 and self-heating disabled.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 7th node." << " " <<  std::endl; 
 UserError(*this) << "$finish called." << std::endl; 
}
}
}
else
{
// Temp(t,GND) <+ (0)
// do nothing at all
}
}
IDS = (B4SOItype*Ids);
GM = B4SOIgm;
GDS = B4SOIgds;
GMBS = B4SOIgmbs;
IC = (B4SOItype*Ic);
IBS = (B4SOItype*Ibs);
IBD = (B4SOItype*Ibd);
IGIDL = B4SOIigidl;
IGISL = B4SOIigisl;
IGS = B4SOIIgs;
IGD = B4SOIIgd;
IGB = B4SOIig;
IGCS = B4SOIIgcs;
IGCD = B4SOIIgcd;

d_QG_dV_e_si = B4SOItype*d_qgate_dV_e_si;
d_QG_dV_b_si = B4SOItype*d_qgate_dV_b_si;
d_QG_dV_b_GND = B4SOItype*d_qgate_dV_b_GND;
d_QG_dV_p_GND = B4SOItype*d_qgate_dV_p_GND;
d_QG_dTemp_t_GND = B4SOItype*d_qgate_dTemp_t_GND;
d_QG_dV_gi_si = B4SOItype*d_qgate_dV_gi_si;
d_QG_dV_di_si = B4SOItype*d_qgate_dV_di_si;
d_QG_dV_sb_si = B4SOItype*d_qgate_dV_sb_si;
d_QG_dV_db_di = B4SOItype*d_qgate_dV_db_di;
d_QG_dV_gi_p = B4SOItype*d_qgate_dV_gi_p;
d_QG_dV_gm_si = B4SOItype*d_qgate_dV_gm_si;
QG = (B4SOItype*qgate);

d_QGI_dV_e_si = B4SOItype*d_qgi_dV_e_si;
d_QGI_dV_b_si = B4SOItype*d_qgi_dV_b_si;
d_QGI_dV_b_GND = B4SOItype*d_qgi_dV_b_GND;
d_QGI_dV_p_GND = B4SOItype*d_qgi_dV_p_GND;
d_QGI_dTemp_t_GND = B4SOItype*d_qgi_dTemp_t_GND;
d_QGI_dV_gi_si = B4SOItype*d_qgi_dV_gi_si;
d_QGI_dV_di_si = B4SOItype*d_qgi_dV_di_si;
d_QGI_dV_sb_si = B4SOItype*d_qgi_dV_sb_si;
d_QGI_dV_db_di = B4SOItype*d_qgi_dV_db_di;
d_QGI_dV_gi_p = B4SOItype*d_qgi_dV_gi_p;
QGI = (B4SOItype*qgi);

d_QOV_dV_gm_si = B4SOItype*d_qov_dV_gm_si;
d_QOV_dV_b_GND = B4SOItype*d_qov_dV_b_GND;
d_QOV_dV_p_GND = B4SOItype*d_qov_dV_p_GND;
d_QOV_dTemp_t_GND = B4SOItype*d_qov_dTemp_t_GND;
d_QOV_dV_gi_si = B4SOItype*d_qov_dV_gi_si;
d_QOV_dV_di_si = B4SOItype*d_qov_dV_di_si;
d_QOV_dV_e_si = B4SOItype*d_qov_dV_e_si;
d_QOV_dV_b_si = B4SOItype*d_qov_dV_b_si;
d_QOV_dV_sb_si = B4SOItype*d_qov_dV_sb_si;
d_QOV_dV_db_di = B4SOItype*d_qov_dV_db_di;
d_QOV_dV_gi_p = B4SOItype*d_qov_dV_gi_p;
QOV = (B4SOItype*qov);

d_QB_dV_sb_si = B4SOItype*d_qbody_dV_sb_si;
d_QB_dV_db_di = B4SOItype*d_qbody_dV_db_di;
d_QB_dV_gi_p = B4SOItype*d_qbody_dV_gi_p;
d_QB_dV_b_GND = B4SOItype*d_qbody_dV_b_GND;
d_QB_dV_p_GND = B4SOItype*d_qbody_dV_p_GND;
d_QB_dTemp_t_GND = B4SOItype*d_qbody_dTemp_t_GND;
d_QB_dV_gi_si = B4SOItype*d_qbody_dV_gi_si;
d_QB_dV_di_si = B4SOItype*d_qbody_dV_di_si;
d_QB_dV_e_si = B4SOItype*d_qbody_dV_e_si;
d_QB_dV_b_si = B4SOItype*d_qbody_dV_b_si;
QB = (B4SOItype*qbody);

d_QD_dV_e_si = d_B4SOIqdrn_dV_e_si;
d_QD_dV_b_si = d_B4SOIqdrn_dV_b_si;
d_QD_dV_b_GND = d_B4SOIqdrn_dV_b_GND;
d_QD_dV_p_GND = d_B4SOIqdrn_dV_p_GND;
d_QD_dTemp_t_GND = d_B4SOIqdrn_dTemp_t_GND;
d_QD_dV_gi_si = d_B4SOIqdrn_dV_gi_si;
d_QD_dV_di_si = d_B4SOIqdrn_dV_di_si;
d_QD_dV_sb_si = d_B4SOIqdrn_dV_sb_si;
d_QD_dV_db_di = d_B4SOIqdrn_dV_db_di;
d_QD_dV_gi_p = d_B4SOIqdrn_dV_gi_p;
QD = B4SOIqdrn;
QS = B4SOIqsrc;
if (((model_.RGATEMOD)==3))
{
CGG = ((+d_QGI_dV_gi_si+d_QGI_dV_gi_p)+(+d_QOV_dV_gm_si));
}
else
{
CGG = (+d_QG_dV_gi_si+d_QG_dV_gi_p);
}
CGS = (-(-d_QG_dV_e_si-d_QG_dV_b_si-d_QG_dV_gi_si-d_QG_dV_di_si-d_QG_dV_sb_si-d_QG_dV_gm_si));
CGD = (-(+d_QG_dV_di_si-d_QG_dV_db_di));
CBG = (-(+d_QB_dV_gi_p+d_QB_dV_gi_si));
CBD = (-(-d_QB_dV_db_di+d_QB_dV_di_si));
CBS = (-(-d_QB_dV_sb_si-d_QB_dV_gi_si-d_QB_dV_di_si-d_QB_dV_e_si-d_QB_dV_b_si));
CDG = (-(+d_QD_dV_gi_si+d_QD_dV_gi_p));
CDD = (+d_QD_dV_di_si-d_QD_dV_db_di);
CDS = (-(-d_QD_dV_e_si-d_QD_dV_b_si-d_QD_dV_gi_si-d_QD_dV_di_si-d_QD_dV_sb_si));

d_QJD_dV_gi_si = B4SOItype*d_qjd_dV_gi_si;
d_QJD_dV_di_si = B4SOItype*d_qjd_dV_di_si;
d_QJD_dV_e_si = B4SOItype*d_qjd_dV_e_si;
d_QJD_dV_b_si = B4SOItype*d_qjd_dV_b_si;
d_QJD_dV_sb_si = B4SOItype*d_qjd_dV_sb_si;
d_QJD_dV_db_di = B4SOItype*d_qjd_dV_db_di;
d_QJD_dV_gi_p = B4SOItype*d_qjd_dV_gi_p;
d_QJD_dV_b_GND = B4SOItype*d_qjd_dV_b_GND;
d_QJD_dV_p_GND = B4SOItype*d_qjd_dV_p_GND;
d_QJD_dTemp_t_GND = B4SOItype*d_qjd_dTemp_t_GND;
QJD = (B4SOItype*qjd);

d_QJS_dV_gi_si = B4SOItype*d_qjs_dV_gi_si;
d_QJS_dV_di_si = B4SOItype*d_qjs_dV_di_si;
d_QJS_dV_e_si = B4SOItype*d_qjs_dV_e_si;
d_QJS_dV_b_si = B4SOItype*d_qjs_dV_b_si;
d_QJS_dV_sb_si = B4SOItype*d_qjs_dV_sb_si;
d_QJS_dV_db_di = B4SOItype*d_qjs_dV_db_di;
d_QJS_dV_gi_p = B4SOItype*d_qjs_dV_gi_p;
d_QJS_dV_b_GND = B4SOItype*d_qjs_dV_b_GND;
d_QJS_dV_p_GND = B4SOItype*d_qjs_dV_p_GND;
d_QJS_dTemp_t_GND = B4SOItype*d_qjs_dTemp_t_GND;
QJS = (B4SOItype*qjs);
CAPBD = (+d_QJD_dV_db_di);
CAPBS = (+d_QJS_dV_sb_si);
}
// End block load


  // -- endcode converted from analog/code block
  if (DEBUG_DEVICE && isActive(Diag::DEVICE_PARAMETERS) && getSolverState().debugTimeFlag)
  {
    Xyce::dout() << " probeVars[admsProbeID_V_sb_GND] =  "
         <<probeVars[admsProbeID_V_sb_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_db_GND] =  "
         <<probeVars[admsProbeID_V_db_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gm_GND] =  "
         <<probeVars[admsProbeID_V_gm_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_b_sb] =  "
         <<probeVars[admsProbeID_V_b_sb] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_b_db] =  "
         <<probeVars[admsProbeID_V_b_db] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gm_gi] =  "
         <<probeVars[admsProbeID_V_gm_gi] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_g_gm] =  "
         <<probeVars[admsProbeID_V_g_gm] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gi_e] =  "
         <<probeVars[admsProbeID_V_gi_e] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gm_e] =  "
         <<probeVars[admsProbeID_V_gm_e] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_s_si] =  "
         <<probeVars[admsProbeID_V_s_si] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_d_di] =  "
         <<probeVars[admsProbeID_V_d_di] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_si_GND] =  "
         <<probeVars[admsProbeID_V_si_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_di_GND] =  "
         <<probeVars[admsProbeID_V_di_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gi_GND] =  "
         <<probeVars[admsProbeID_V_gi_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gm_si] =  "
         <<probeVars[admsProbeID_V_gm_si] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_db_di] =  "
         <<probeVars[admsProbeID_V_db_di] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_sb_si] =  "
         <<probeVars[admsProbeID_V_sb_si] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gi_p] =  "
         <<probeVars[admsProbeID_V_gi_p] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_b_p] =  "
         <<probeVars[admsProbeID_V_b_p] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_e_si] =  "
         <<probeVars[admsProbeID_V_e_si] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_gi_si] =  "
         <<probeVars[admsProbeID_V_gi_si] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_b_si] =  "
         <<probeVars[admsProbeID_V_b_si] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_di_si] =  "
         <<probeVars[admsProbeID_V_di_si] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_Temp_t_GND] =  "
         <<probeVars[admsProbeID_Temp_t_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_p_GND] =  "
         <<probeVars[admsProbeID_V_p_GND] << std::endl;
    Xyce::dout() << " probeVars[admsProbeID_V_b_GND] =  "
         <<probeVars[admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_d] =  "
         <<staticContributions[admsNodeID_d] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_d].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_d][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_d] =  "
         <<dynamicContributions[admsNodeID_d] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_d].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_d][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_g] =  "
         <<staticContributions[admsNodeID_g] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_g].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_g][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_g] =  "
         <<dynamicContributions[admsNodeID_g] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_g].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_g][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_s] =  "
         <<staticContributions[admsNodeID_s] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_s].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_s][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_s] =  "
         <<dynamicContributions[admsNodeID_s] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_s].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_s][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_e] =  "
         <<staticContributions[admsNodeID_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_e].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_e][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_e] =  "
         <<dynamicContributions[admsNodeID_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_e].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_p] =  "
         <<staticContributions[admsNodeID_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_p].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_p][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_p] =  "
         <<dynamicContributions[admsNodeID_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_p].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_b] =  "
         <<staticContributions[admsNodeID_b] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_b].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_b] =  "
         <<dynamicContributions[admsNodeID_b] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_b].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_t] =  "
         <<staticContributions[admsNodeID_t] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_t].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_t][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_t] =  "
         <<dynamicContributions[admsNodeID_t] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_t].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_di] =  "
         <<staticContributions[admsNodeID_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_di].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_di] =  "
         <<dynamicContributions[admsNodeID_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_di].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_si] =  "
         <<staticContributions[admsNodeID_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_si].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_si] =  "
         <<dynamicContributions[admsNodeID_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_si].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_gi] =  "
         <<staticContributions[admsNodeID_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gi].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_gi] =  "
         <<dynamicContributions[admsNodeID_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gi].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_gm] =  "
         <<staticContributions[admsNodeID_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_gm].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_gm][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_gm] =  "
         <<dynamicContributions[admsNodeID_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_gm].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_sb] =  "
         <<staticContributions[admsNodeID_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_sb].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_sb][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_sb] =  "
         <<dynamicContributions[admsNodeID_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_sb].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " staticContributions[admsNodeID_db] =  "
         <<staticContributions[admsNodeID_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_sb_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_db_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gm_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_b_sb) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_b_db) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gm_gi) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_g_gm) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gi_e) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gm_e) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_s_si) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_d_di) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_si_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_di_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gi_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gm_si) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_db_di) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_sb_si) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gi_p) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_b_p) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_e_si) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_gi_si) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_b_si) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_di_si) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_Temp_t_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_p_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    staticContributions[admsNodeID_db].dx(admsProbeID_V_b_GND) =  " <<d_staticContributions[admsNodeID_db][admsProbeID_V_b_GND] << std::endl;
    Xyce::dout() << " dynamicContributions[admsNodeID_db] =  "
         <<dynamicContributions[admsNodeID_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_sb_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_sb_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_db_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_db_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gm_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gm_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_b_sb) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_sb] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_b_db) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_db] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gm_gi) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gm_gi] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_g_gm) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_g_gm] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gi_e) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gm_e) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gm_e] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_s_si) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_s_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_d_di) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_d_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_si_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_si_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_di_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_di_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gi_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gm_si) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gm_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_db_di) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_db_di] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_sb_si) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_sb_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gi_p) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_b_p) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_p] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_e_si) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_e_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_gi_si) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_b_si) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_di_si) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_di_si] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_Temp_t_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_Temp_t_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_p_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_p_GND] << std::endl;
      Xyce::dout() << "    dynamicContributions[admsNodeID_db].dx(admsProbeID_V_b_GND) =  " <<d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_GND] << std::endl;

    if (!origFlag)
      Xyce::dout() << "This step was limited by this device." << std::endl;
  }


  if (loadLeadCurrent)
  {
    // If needed, we must now copy all the data from staticContributions
    // and dynamicContributionsinto the lead current F vector
    // Must make sure all collapsed node contributions get summed into the
    // external nodes properly.
    for ( int unmappedNode=0; unmappedNode < 7 ; unmappedNode++)
    {
      leadCurrentF[unmappedNode] = 0.0;
      leadCurrentQ[unmappedNode] = 0.0;
    }
    for ( int unmappedNode=0; unmappedNode < 13; unmappedNode++)
    {
      if (nodeMap[unmappedNode] < 7 && nodeMap[unmappedNode] != -1 )
      {
        leadCurrentF[nodeMap[unmappedNode]] += staticContributions[unmappedNode];
        leadCurrentQ[nodeMap[unmappedNode]] += dynamicContributions[unmappedNode];
      }
    }
  }
  return true;
}

//-----------------------------------------------------------------------------
// Function      : Instance::getNumNoiseSources
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
int Instance::getNumNoiseSources () const
{
return 16;
}

//-----------------------------------------------------------------------------
// Function      : Instance::setupNoiseSources
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::setupNoiseSources (Xyce::Analysis::NoiseData & noiseData)
{
int numSources=getNumNoiseSources();
noiseData.numSources = numSources;
noiseData.resize(numSources);

noiseData.deviceName = getName().getEncodedName();
std::string theNoiseName;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_di_si")+std::string("_")+std::string("id");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[0]=theNoiseName;
noiseData.li_Pos[0]=li_di;
noiseData.li_Neg[0]=li_si;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_di_si")+std::string("_")+std::string("id");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[1]=theNoiseName;
noiseData.li_Pos[1]=li_di;
noiseData.li_Neg[1]=li_si;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_di_si")+std::string("_")+std::string("id");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[2]=theNoiseName;
noiseData.li_Pos[2]=li_di;
noiseData.li_Neg[2]=li_si;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_flicker_di_si")+std::string("_")+std::string("flicker");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[3]=theNoiseName;
noiseData.li_Pos[3]=li_di;
noiseData.li_Neg[3]=li_si;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_d_di")+std::string("_")+std::string("rd");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[4]=theNoiseName;
noiseData.li_Pos[4]=li_d;
noiseData.li_Neg[4]=li_di;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_s_si")+std::string("_")+std::string("rs");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[5]=theNoiseName;
noiseData.li_Pos[5]=li_s;
noiseData.li_Neg[5]=li_si;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_b_p")+std::string("_")+std::string("rbp");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[6]=theNoiseName;
noiseData.li_Pos[6]=li_b;
noiseData.li_Neg[6]=li_p;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_db_di")+std::string("_")+std::string("ibd");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[7]=theNoiseName;
noiseData.li_Pos[7]=li_db;
noiseData.li_Neg[7]=li_di;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_sb_si")+std::string("_")+std::string("ibs");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[8]=theNoiseName;
noiseData.li_Pos[8]=li_sb;
noiseData.li_Neg[8]=li_si;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_gi_di")+std::string("_")+std::string("igd");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[9]=theNoiseName;
noiseData.li_Pos[9]=li_gi;
noiseData.li_Neg[9]=li_di;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_gi_si")+std::string("_")+std::string("igs");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[10]=theNoiseName;
noiseData.li_Pos[10]=li_gi;
noiseData.li_Neg[10]=li_si;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_gi_b")+std::string("_")+std::string("igb");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[11]=theNoiseName;
noiseData.li_Pos[11]=li_gi;
noiseData.li_Neg[11]=li_b;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_g_gm")+std::string("_")+std::string("rg");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[12]=theNoiseName;
noiseData.li_Pos[12]=li_g;
noiseData.li_Neg[12]=li_gm;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_gm_gi")+std::string("_")+std::string("rg");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[13]=theNoiseName;
noiseData.li_Pos[13]=li_gm;
noiseData.li_Neg[13]=li_gi;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_b_db")+std::string("_")+std::string("rbdb");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[14]=theNoiseName;
noiseData.li_Pos[14]=li_b;
noiseData.li_Neg[14]=li_db;
theNoiseName = std::string("noise_")+getName().getEncodedName()+std::string("_white_b_sb")+std::string("_")+std::string("rbsb");
std::replace(theNoiseName.begin(), theNoiseName.end(), ' ', '_');
noiseData.noiseNames[15]=theNoiseName;
noiseData.li_Pos[15]=li_b;
noiseData.li_Neg[15]=li_sb;


}
//-----------------------------------------------------------------------------
// Function      : Instance::getNoiseSources
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void Instance::getNoiseSources (Xyce::Analysis::NoiseData & noiseData)
{noiseData.noiseDens[0] = noiseContribsPower[0];
noiseData.lnNoiseDens[0] = std::log(std::max(noiseData.noiseDens[0],N_MINLOG));
noiseData.noiseDens[1] = noiseContribsPower[1];
noiseData.lnNoiseDens[1] = std::log(std::max(noiseData.noiseDens[1],N_MINLOG));
noiseData.noiseDens[2] = noiseContribsPower[2];
noiseData.lnNoiseDens[2] = std::log(std::max(noiseData.noiseDens[2],N_MINLOG));
noiseData.noiseDens[3] = noiseContribsPower[3];

      noiseData.noiseDens[3] /= pow(noiseData.freq,noiseContribsExponent[3]);noiseData.lnNoiseDens[3] = std::log(std::max(noiseData.noiseDens[3],N_MINLOG));
noiseData.noiseDens[4] = noiseContribsPower[4];
noiseData.lnNoiseDens[4] = std::log(std::max(noiseData.noiseDens[4],N_MINLOG));
noiseData.noiseDens[5] = noiseContribsPower[5];
noiseData.lnNoiseDens[5] = std::log(std::max(noiseData.noiseDens[5],N_MINLOG));
noiseData.noiseDens[6] = noiseContribsPower[6];
noiseData.lnNoiseDens[6] = std::log(std::max(noiseData.noiseDens[6],N_MINLOG));
noiseData.noiseDens[7] = noiseContribsPower[7];
noiseData.lnNoiseDens[7] = std::log(std::max(noiseData.noiseDens[7],N_MINLOG));
noiseData.noiseDens[8] = noiseContribsPower[8];
noiseData.lnNoiseDens[8] = std::log(std::max(noiseData.noiseDens[8],N_MINLOG));
noiseData.noiseDens[9] = noiseContribsPower[9];
noiseData.lnNoiseDens[9] = std::log(std::max(noiseData.noiseDens[9],N_MINLOG));
noiseData.noiseDens[10] = noiseContribsPower[10];
noiseData.lnNoiseDens[10] = std::log(std::max(noiseData.noiseDens[10],N_MINLOG));
noiseData.noiseDens[11] = noiseContribsPower[11];
noiseData.lnNoiseDens[11] = std::log(std::max(noiseData.noiseDens[11],N_MINLOG));
noiseData.noiseDens[12] = noiseContribsPower[12];
noiseData.lnNoiseDens[12] = std::log(std::max(noiseData.noiseDens[12],N_MINLOG));
noiseData.noiseDens[13] = noiseContribsPower[13];
noiseData.lnNoiseDens[13] = std::log(std::max(noiseData.noiseDens[13],N_MINLOG));
noiseData.noiseDens[14] = noiseContribsPower[14];
noiseData.lnNoiseDens[14] = std::log(std::max(noiseData.noiseDens[14],N_MINLOG));
noiseData.noiseDens[15] = noiseContribsPower[15];
noiseData.lnNoiseDens[15] = std::log(std::max(noiseData.noiseDens[15],N_MINLOG));

}

//-----------------------------------------------------------------------------
// Function      : Instance::loadDAEdFdx
// Purpose       :
// Special Notes : Load the dFdx ("static jacobian") matrix
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::loadDAEdFdx()
{
  bool bsuccess = true;
  Linear::Matrix & dFdx = *(extData.dFdxMatrixPtr);

#ifndef Xyce_NONPOINTER_MATRIX_LOAD
  (*f_di_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si];
  (*f_di_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_di][admsProbeID_V_gm_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si];
  (*f_si_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si];
  (*f_si_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_si][admsProbeID_V_s_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_gm_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si];
  (*f_di_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_db_di];
  (*f_di_Equ_di_Node_Ptr) +=  -d_staticContributions[admsNodeID_di][admsProbeID_V_d_di] +d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_db_di];
  (*f_si_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_db_di];
  (*f_si_Equ_di_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_db_di];
  (*f_di_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] +d_staticContributions[admsNodeID_di][admsProbeID_V_gi_p];
  (*f_di_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_di][admsProbeID_V_gi_p];
  (*f_si_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] +d_staticContributions[admsNodeID_si][admsProbeID_V_gi_p];
  (*f_si_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_si][admsProbeID_V_gi_p];
  (*f_di_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_e_si];
  (*f_si_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_e_si];
  (*f_di_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_di][admsProbeID_V_b_si];
  (*f_si_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_si][admsProbeID_V_b_si];
if (!collapseNode_t)
{
  (*f_di_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND];
}
if (!collapseNode_t)
{
  (*f_si_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND];
}
  (*f_di_Equ_gm_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_gm_si];
  (*f_si_Equ_gm_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_gm_si];
  (*f_d_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_d][admsProbeID_V_gi_si];
  (*f_d_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_d][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_gi_si];
  (*f_d_Equ_di_Node_Ptr) +=  -d_staticContributions[admsNodeID_d][admsProbeID_V_d_di] -d_staticContributions[admsNodeID_d][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_d][admsProbeID_V_di_si];
  (*f_d_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_e_si];
  (*f_d_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_d][admsProbeID_V_b_si];
  (*f_d_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_sb_si];
  (*f_d_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_db_di];
  (*f_d_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_d][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  (*f_d_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_Temp_t_GND];
}
  (*f_d_Equ_d_Node_Ptr) +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_d_di];
  (*f_di_Equ_d_Node_Ptr) +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_d_di];
  (*f_s_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_s][admsProbeID_V_gi_si];
  (*f_s_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_s][admsProbeID_V_s_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_gi_si];
  (*f_s_Equ_di_Node_Ptr) +=  -d_staticContributions[admsNodeID_s][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_s][admsProbeID_V_di_si];
  (*f_s_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_e_si];
  (*f_s_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_s][admsProbeID_V_b_si];
  (*f_s_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_sb_si];
  (*f_s_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_db_di];
  (*f_s_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_s][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  (*f_s_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_Temp_t_GND];
}
  (*f_s_Equ_s_Node_Ptr) +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_s_si];
  (*f_si_Equ_s_Node_Ptr) +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_s_si];
  (*f_b_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si];
  (*f_b_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si];
  (*f_b_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_e_si];
  (*f_b_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_si];
  (*f_b_Equ_sb_Node_Ptr) +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si];
  (*f_b_Equ_db_Node_Ptr) +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_b][admsProbeID_V_db_di];
  (*f_b_Equ_di_Node_Ptr) +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_db_di];
  (*f_b_Equ_p_Node_Ptr) +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  (*f_b_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND];
}
  (*f_db_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_db][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_db][admsProbeID_V_b_GND];
  (*f_db_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*f_db_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_db][admsProbeID_Temp_t_GND];
}
  (*f_db_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_e_si];
  (*f_db_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_db][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_e_si];
  (*f_db_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_gi_si];
  (*f_db_Equ_di_Node_Ptr) +=  -d_staticContributions[admsNodeID_db][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_db][admsProbeID_V_di_si];
  (*f_db_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_sb_si];
  (*f_db_Equ_db_Node_Ptr) +=  -d_staticContributions[admsNodeID_db][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_db][admsProbeID_V_db_di];
  (*f_sb_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_sb][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_sb][admsProbeID_V_b_GND];
  (*f_sb_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*f_sb_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_sb][admsProbeID_Temp_t_GND];
}
  (*f_sb_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_e_si];
  (*f_sb_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_sb][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_e_si];
  (*f_sb_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_si];
  (*f_sb_Equ_di_Node_Ptr) +=  -d_staticContributions[admsNodeID_sb][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_sb][admsProbeID_V_di_si];
  (*f_sb_Equ_sb_Node_Ptr) +=  -d_staticContributions[admsNodeID_sb][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_sb][admsProbeID_V_sb_si];
  (*f_sb_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_db_di];
  (*f_gi_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si];
  (*f_gi_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si];
  (*f_gi_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di];
  (*f_gi_Equ_di_Node_Ptr) +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di];
  (*f_gi_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND];
  (*f_gi_Equ_p_Node_Ptr) +=  -d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*f_gi_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND];
}
  (*f_gi_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si];
  (*f_gi_Equ_gi_Node_Ptr) +=  -d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_gi] +d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si];
  (*f_p_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si] +d_staticContributions[admsNodeID_p][admsProbeID_V_gi_p];
  (*f_p_Equ_p_Node_Ptr) +=  -d_staticContributions[admsNodeID_p][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_p][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_p][admsProbeID_V_gi_p];
  (*f_p_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_p][admsProbeID_V_b_GND];
if (!collapseNode_t)
{
  (*f_p_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_p][admsProbeID_Temp_t_GND];
}
  (*f_p_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_p][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si];
  (*f_p_Equ_di_Node_Ptr) +=  -d_staticContributions[admsNodeID_p][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_p][admsProbeID_V_di_si];
  (*f_p_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_e_si];
  (*f_p_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_sb_si];
  (*f_p_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_db_di];
  (*f_gi_Equ_gm_Node_Ptr) +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_gi];
  (*f_gm_Equ_gm_Node_Ptr) +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_gi] -d_staticContributions[admsNodeID_gm][admsProbeID_V_g_gm];
  (*f_gm_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_e_si];
  (*f_gm_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_gm][admsProbeID_V_b_GND];
  (*f_gm_Equ_p_Node_Ptr) +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gm][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*f_gm_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_gm][admsProbeID_Temp_t_GND];
}
  (*f_gm_Equ_gi_Node_Ptr) +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_gi] +d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_si];
  (*f_gm_Equ_di_Node_Ptr) +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_gm][admsProbeID_V_di_si];
  (*f_gm_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_e_si];
  (*f_gm_Equ_sb_Node_Ptr) +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_sb_si];
  (*f_gm_Equ_db_Node_Ptr) +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_db_di];
  (*f_g_Equ_g_Node_Ptr) +=  +d_staticContributions[admsNodeID_g][admsProbeID_V_g_gm];
  (*f_g_Equ_gm_Node_Ptr) +=  -d_staticContributions[admsNodeID_g][admsProbeID_V_g_gm];
  (*f_gm_Equ_g_Node_Ptr) +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_g_gm];
  (*f_g_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_g][admsProbeID_V_b_GND];
  (*f_g_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_g][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*f_g_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_g][admsProbeID_Temp_t_GND];
}
if (!collapseNode_t)
{
  (*f_t_Equ_gi_Node_Ptr) +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_gi_si];
}
if (!collapseNode_t)
{
  (*f_t_Equ_si_Node_Ptr) +=  -d_staticContributions[admsNodeID_t][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_t][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_t][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_t][admsProbeID_V_gi_si];
}
if (!collapseNode_t)
{
  (*f_t_Equ_di_Node_Ptr) +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_di_si];
}
if (!collapseNode_t)
{
  (*f_t_Equ_e_Node_Ptr) +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_e_si];
}
if (!collapseNode_t)
{
  (*f_t_Equ_b_Node_Ptr) +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_t][admsProbeID_V_b_si];
}
if (!collapseNode_t)
{
  (*f_t_Equ_p_Node_Ptr) +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_p_GND];
}
if (!collapseNode_t)
{
  (*f_t_Equ_t_Node_Ptr) +=  +d_staticContributions[admsNodeID_t][admsProbeID_Temp_t_GND];
}

#else
  //use the offsets instead of pointers
  dFdx[li_di][A_di_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si];
  dFdx[li_di][A_di_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_di][admsProbeID_V_gm_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_sb_si];
  dFdx[li_si][A_si_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si];
  dFdx[li_si][A_si_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_si][admsProbeID_V_s_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_gm_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_sb_si];
  dFdx[li_di][A_di_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_db_di];
  dFdx[li_di][A_di_Equ_di_NodeOffset] +=  -d_staticContributions[admsNodeID_di][admsProbeID_V_d_di] +d_staticContributions[admsNodeID_di][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_di][admsProbeID_V_db_di];
  dFdx[li_si][A_si_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_db_di];
  dFdx[li_si][A_si_Equ_di_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_si][admsProbeID_V_db_di];
  dFdx[li_di][A_di_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_gi_si] +d_staticContributions[admsNodeID_di][admsProbeID_V_gi_p];
  dFdx[li_di][A_di_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_di][admsProbeID_V_gi_p];
  dFdx[li_si][A_si_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_gi_si] +d_staticContributions[admsNodeID_si][admsProbeID_V_gi_p];
  dFdx[li_si][A_si_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_si][admsProbeID_V_gi_p];
  dFdx[li_di][A_di_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_e_si];
  dFdx[li_si][A_si_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_e_si];
  dFdx[li_di][A_di_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_di][admsProbeID_V_b_si];
  dFdx[li_si][A_si_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_si][admsProbeID_V_b_si];
if (!collapseNode_t)
{
  dFdx[li_di][A_di_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_Temp_t_GND];
}
if (!collapseNode_t)
{
  dFdx[li_si][A_si_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_Temp_t_GND];
}
  dFdx[li_di][A_di_Equ_gm_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_gm_si];
  dFdx[li_si][A_si_Equ_gm_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_gm_si];
  dFdx[li_d][A_d_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_d][admsProbeID_V_gi_si];
  dFdx[li_d][A_d_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_d][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_d][admsProbeID_V_gi_si];
  dFdx[li_d][A_d_Equ_di_NodeOffset] +=  -d_staticContributions[admsNodeID_d][admsProbeID_V_d_di] -d_staticContributions[admsNodeID_d][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_d][admsProbeID_V_di_si];
  dFdx[li_d][A_d_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_e_si];
  dFdx[li_d][A_d_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_d][admsProbeID_V_b_si];
  dFdx[li_d][A_d_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_sb_si];
  dFdx[li_d][A_d_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_db_di];
  dFdx[li_d][A_d_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_d][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  dFdx[li_d][A_d_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_Temp_t_GND];
}
  dFdx[li_d][A_d_Equ_d_NodeOffset] +=  +d_staticContributions[admsNodeID_d][admsProbeID_V_d_di];
  dFdx[li_di][A_di_Equ_d_NodeOffset] +=  +d_staticContributions[admsNodeID_di][admsProbeID_V_d_di];
  dFdx[li_s][A_s_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_s][admsProbeID_V_gi_si];
  dFdx[li_s][A_s_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_s][admsProbeID_V_s_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_s][admsProbeID_V_gi_si];
  dFdx[li_s][A_s_Equ_di_NodeOffset] +=  -d_staticContributions[admsNodeID_s][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_s][admsProbeID_V_di_si];
  dFdx[li_s][A_s_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_e_si];
  dFdx[li_s][A_s_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_s][admsProbeID_V_b_si];
  dFdx[li_s][A_s_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_sb_si];
  dFdx[li_s][A_s_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_db_di];
  dFdx[li_s][A_s_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_s][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  dFdx[li_s][A_s_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_Temp_t_GND];
}
  dFdx[li_s][A_s_Equ_s_NodeOffset] +=  +d_staticContributions[admsNodeID_s][admsProbeID_V_s_si];
  dFdx[li_si][A_si_Equ_s_NodeOffset] +=  +d_staticContributions[admsNodeID_si][admsProbeID_V_s_si];
  dFdx[li_b][A_b_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si];
  dFdx[li_b][A_b_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_gi_si];
  dFdx[li_b][A_b_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_e_si];
  dFdx[li_b][A_b_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_b][admsProbeID_V_b_si];
  dFdx[li_b][A_b_Equ_sb_NodeOffset] +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_b][admsProbeID_V_sb_si];
  dFdx[li_b][A_b_Equ_db_NodeOffset] +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_b][admsProbeID_V_db_di];
  dFdx[li_b][A_b_Equ_di_NodeOffset] +=  +d_staticContributions[admsNodeID_b][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_b][admsProbeID_V_db_di];
  dFdx[li_b][A_b_Equ_p_NodeOffset] +=  -d_staticContributions[admsNodeID_b][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_b][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_b][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  dFdx[li_b][A_b_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_b][admsProbeID_Temp_t_GND];
}
  dFdx[li_db][A_db_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_db][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_db][admsProbeID_V_b_GND];
  dFdx[li_db][A_db_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dFdx[li_db][A_db_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_db][admsProbeID_Temp_t_GND];
}
  dFdx[li_db][A_db_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_e_si];
  dFdx[li_db][A_db_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_db][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_db][admsProbeID_V_e_si];
  dFdx[li_db][A_db_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_gi_si];
  dFdx[li_db][A_db_Equ_di_NodeOffset] +=  -d_staticContributions[admsNodeID_db][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_db][admsProbeID_V_di_si];
  dFdx[li_db][A_db_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_db][admsProbeID_V_sb_si];
  dFdx[li_db][A_db_Equ_db_NodeOffset] +=  -d_staticContributions[admsNodeID_db][admsProbeID_V_b_db] +d_staticContributions[admsNodeID_db][admsProbeID_V_db_di];
  dFdx[li_sb][A_sb_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_sb][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_sb][admsProbeID_V_b_GND];
  dFdx[li_sb][A_sb_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dFdx[li_sb][A_sb_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_sb][admsProbeID_Temp_t_GND];
}
  dFdx[li_sb][A_sb_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_e_si];
  dFdx[li_sb][A_sb_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_sb][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_sb][admsProbeID_V_e_si];
  dFdx[li_sb][A_sb_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_gi_si];
  dFdx[li_sb][A_sb_Equ_di_NodeOffset] +=  -d_staticContributions[admsNodeID_sb][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_sb][admsProbeID_V_di_si];
  dFdx[li_sb][A_sb_Equ_sb_NodeOffset] +=  -d_staticContributions[admsNodeID_sb][admsProbeID_V_b_sb] +d_staticContributions[admsNodeID_sb][admsProbeID_V_sb_si];
  dFdx[li_sb][A_sb_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_sb][admsProbeID_V_db_di];
  dFdx[li_gi][A_gi_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si];
  dFdx[li_gi][A_gi_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_sb_si];
  dFdx[li_gi][A_gi_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di];
  dFdx[li_gi][A_gi_Equ_di_NodeOffset] +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_gi][admsProbeID_V_db_di];
  dFdx[li_gi][A_gi_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_gi][admsProbeID_V_b_GND];
  dFdx[li_gi][A_gi_Equ_p_NodeOffset] +=  -d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gi][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dFdx[li_gi][A_gi_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_gi][admsProbeID_Temp_t_GND];
}
  dFdx[li_gi][A_gi_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_e_si];
  dFdx[li_gi][A_gi_Equ_gi_NodeOffset] +=  -d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_gi] +d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gi][admsProbeID_V_gi_si];
  dFdx[li_p][A_p_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si] +d_staticContributions[admsNodeID_p][admsProbeID_V_gi_p];
  dFdx[li_p][A_p_Equ_p_NodeOffset] +=  -d_staticContributions[admsNodeID_p][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_p][admsProbeID_V_p_GND] -d_staticContributions[admsNodeID_p][admsProbeID_V_gi_p];
  dFdx[li_p][A_p_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_b_p] +d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_p][admsProbeID_V_b_GND];
if (!collapseNode_t)
{
  dFdx[li_p][A_p_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_p][admsProbeID_Temp_t_GND];
}
  dFdx[li_p][A_p_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_p][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_p][admsProbeID_V_gi_si];
  dFdx[li_p][A_p_Equ_di_NodeOffset] +=  -d_staticContributions[admsNodeID_p][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_p][admsProbeID_V_di_si];
  dFdx[li_p][A_p_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_e_si];
  dFdx[li_p][A_p_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_sb_si];
  dFdx[li_p][A_p_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_p][admsProbeID_V_db_di];
  dFdx[li_gi][A_gi_Equ_gm_NodeOffset] +=  +d_staticContributions[admsNodeID_gi][admsProbeID_V_gm_gi];
  dFdx[li_gm][A_gm_Equ_gm_NodeOffset] +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_gi] -d_staticContributions[admsNodeID_gm][admsProbeID_V_g_gm];
  dFdx[li_gm][A_gm_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_sb_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_si] -d_staticContributions[admsNodeID_gm][admsProbeID_V_e_si];
  dFdx[li_gm][A_gm_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_b_si] +d_staticContributions[admsNodeID_gm][admsProbeID_V_b_GND];
  dFdx[li_gm][A_gm_Equ_p_NodeOffset] +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gm][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dFdx[li_gm][A_gm_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_gm][admsProbeID_Temp_t_GND];
}
  dFdx[li_gm][A_gm_Equ_gi_NodeOffset] +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_gm_gi] +d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_staticContributions[admsNodeID_gm][admsProbeID_V_gi_si];
  dFdx[li_gm][A_gm_Equ_di_NodeOffset] +=  -d_staticContributions[admsNodeID_gm][admsProbeID_V_db_di] +d_staticContributions[admsNodeID_gm][admsProbeID_V_di_si];
  dFdx[li_gm][A_gm_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_e_si];
  dFdx[li_gm][A_gm_Equ_sb_NodeOffset] +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_sb_si];
  dFdx[li_gm][A_gm_Equ_db_NodeOffset] +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_db_di];
  dFdx[li_g][A_g_Equ_g_NodeOffset] +=  +d_staticContributions[admsNodeID_g][admsProbeID_V_g_gm];
  dFdx[li_g][A_g_Equ_gm_NodeOffset] +=  -d_staticContributions[admsNodeID_g][admsProbeID_V_g_gm];
  dFdx[li_gm][A_gm_Equ_g_NodeOffset] +=  +d_staticContributions[admsNodeID_gm][admsProbeID_V_g_gm];
  dFdx[li_g][A_g_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_g][admsProbeID_V_b_GND];
  dFdx[li_g][A_g_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_g][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dFdx[li_g][A_g_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_g][admsProbeID_Temp_t_GND];
}
if (!collapseNode_t)
{
  dFdx[li_t][A_t_Equ_gi_NodeOffset] +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_gi_si];
}
if (!collapseNode_t)
{
  dFdx[li_t][A_t_Equ_si_NodeOffset] +=  -d_staticContributions[admsNodeID_t][admsProbeID_V_b_si] -d_staticContributions[admsNodeID_t][admsProbeID_V_e_si] -d_staticContributions[admsNodeID_t][admsProbeID_V_di_si] -d_staticContributions[admsNodeID_t][admsProbeID_V_gi_si];
}
if (!collapseNode_t)
{
  dFdx[li_t][A_t_Equ_di_NodeOffset] +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_di_si];
}
if (!collapseNode_t)
{
  dFdx[li_t][A_t_Equ_e_NodeOffset] +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_e_si];
}
if (!collapseNode_t)
{
  dFdx[li_t][A_t_Equ_b_NodeOffset] +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_b_GND] +d_staticContributions[admsNodeID_t][admsProbeID_V_b_si];
}
if (!collapseNode_t)
{
  dFdx[li_t][A_t_Equ_p_NodeOffset] +=  +d_staticContributions[admsNodeID_t][admsProbeID_V_p_GND];
}
if (!collapseNode_t)
{
  dFdx[li_t][A_t_Equ_t_NodeOffset] +=  +d_staticContributions[admsNodeID_t][admsProbeID_Temp_t_GND];
}

#endif

  return bsuccess;
}


//-----------------------------------------------------------------------------
// Function      : Instance::loadDAEdQdx
// Purpose       :
// Special Notes : Load the dQdx ("dynamic jacobian") matrix
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::loadDAEdQdx()
{
  bool bsuccess = true;
  Linear::Matrix & dQdx = *(extData.dQdxMatrixPtr);

#ifndef Xyce_NONPOINTER_MATRIX_LOAD
  (*q_di_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si];
  (*q_di_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si];
  (*q_si_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si];
  (*q_si_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si];
  (*q_di_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di];
  (*q_di_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si];
  (*q_si_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di];
  (*q_si_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si];
  (*q_di_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si];
  (*q_di_Equ_p_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND];
  (*q_si_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si];
  (*q_si_Equ_p_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND];
  (*q_di_Equ_e_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si];
  (*q_si_Equ_e_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si];
  (*q_di_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si];
  (*q_si_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si];
if (!collapseNode_t)
{
  (*q_di_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND];
}
if (!collapseNode_t)
{
  (*q_si_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND];
}
  (*q_di_Equ_gm_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_si];
  (*q_si_Equ_gm_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_si];
  (*q_b_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si];
  (*q_b_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si];
  (*q_b_Equ_e_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si];
  (*q_b_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si];
  (*q_b_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si];
  (*q_b_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di];
  (*q_b_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si];
  (*q_b_Equ_p_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*q_b_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND];
}
  (*q_db_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_si];
  (*q_db_Equ_p_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_p_GND] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  (*q_db_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_Temp_t_GND];
}
  (*q_db_Equ_e_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_e_si];
  (*q_db_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_db][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_si];
  (*q_db_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_si];
  (*q_db_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_db][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_db][admsProbeID_V_di_si];
  (*q_db_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_sb_si];
  (*q_db_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_db_di];
  (*q_sb_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_si];
  (*q_sb_Equ_p_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_p_GND] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  (*q_sb_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_Temp_t_GND];
}
  (*q_sb_Equ_e_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_e_si];
  (*q_sb_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_si];
  (*q_sb_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_si];
  (*q_sb_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_di_si];
  (*q_sb_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_sb_si];
  (*q_sb_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_db_di];
  (*q_gi_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si];
  (*q_gi_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si];
  (*q_gi_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di];
  (*q_gi_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si];
  (*q_gi_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si];
  (*q_gi_Equ_p_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*q_gi_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_Temp_t_GND];
}
  (*q_gi_Equ_e_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_e] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si];
  (*q_gi_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_e] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si];
  (*q_p_Equ_p_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_p][admsProbeID_V_p_GND];
  (*q_p_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_GND];
if (!collapseNode_t)
{
  (*q_p_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_p][admsProbeID_Temp_t_GND];
}
  (*q_gi_Equ_gm_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si];
  (*q_b_Equ_gm_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_si];
  (*q_e_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si];
  (*q_e_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si];
  (*q_e_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_e] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si];
  (*q_e_Equ_p_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*q_e_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_Temp_t_GND];
}
  (*q_e_Equ_e_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_e] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_e] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si];
  (*q_e_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si];
  (*q_gm_Equ_gm_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_e] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_si];
  (*q_gm_Equ_si_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_si];
  (*q_gm_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_si] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_GND];
  (*q_gm_Equ_p_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  (*q_gm_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_Temp_t_GND];
}
  (*q_gm_Equ_gi_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_si];
  (*q_gm_Equ_di_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_si];
  (*q_gm_Equ_e_Node_Ptr) +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_e] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_e_si];
  (*q_gm_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_si];
  (*q_gm_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_di];
  (*q_e_Equ_gm_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_e];
  (*q_e_Equ_sb_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_si];
  (*q_e_Equ_db_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_di];
if (!collapseNode_t)
{
  (*q_t_Equ_b_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_GND];
}
if (!collapseNode_t)
{
  (*q_t_Equ_p_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_t][admsProbeID_V_p_GND];
}
if (!collapseNode_t)
{
  (*q_t_Equ_t_Node_Ptr) +=  +d_dynamicContributions[admsNodeID_t][admsProbeID_Temp_t_GND];
}

#else
  //use the offsets instead of pointers
  dQdx[li_di][A_di_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si];
  dQdx[li_di][A_di_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si];
  dQdx[li_si][A_si_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si];
  dQdx[li_si][A_si_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si];
  dQdx[li_di][A_di_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di];
  dQdx[li_di][A_di_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_di][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_di_si];
  dQdx[li_si][A_si_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di];
  dQdx[li_si][A_si_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_si][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_di_si];
  dQdx[li_di][A_di_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_si];
  dQdx[li_di][A_di_Equ_p_NodeOffset] +=  -d_dynamicContributions[admsNodeID_di][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_p_GND];
  dQdx[li_si][A_si_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_si];
  dQdx[li_si][A_si_Equ_p_NodeOffset] +=  -d_dynamicContributions[admsNodeID_si][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_p_GND];
  dQdx[li_di][A_di_Equ_e_NodeOffset] +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_e_si];
  dQdx[li_si][A_si_Equ_e_NodeOffset] +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_e_si];
  dQdx[li_di][A_di_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_di][admsProbeID_V_b_si];
  dQdx[li_si][A_si_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_si][admsProbeID_V_b_si];
if (!collapseNode_t)
{
  dQdx[li_di][A_di_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_Temp_t_GND];
}
if (!collapseNode_t)
{
  dQdx[li_si][A_si_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_Temp_t_GND];
}
  dQdx[li_di][A_di_Equ_gm_NodeOffset] +=  +d_dynamicContributions[admsNodeID_di][admsProbeID_V_gm_si];
  dQdx[li_si][A_si_Equ_gm_NodeOffset] +=  +d_dynamicContributions[admsNodeID_si][admsProbeID_V_gm_si];
  dQdx[li_b][A_b_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si];
  dQdx[li_b][A_b_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si];
  dQdx[li_b][A_b_Equ_e_NodeOffset] +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_e_si];
  dQdx[li_b][A_b_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_b_si];
  dQdx[li_b][A_b_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_sb_si];
  dQdx[li_b][A_b_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di];
  dQdx[li_b][A_b_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_b][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_di_si];
  dQdx[li_b][A_b_Equ_p_NodeOffset] +=  -d_dynamicContributions[admsNodeID_b][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_b][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dQdx[li_b][A_b_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_Temp_t_GND];
}
  dQdx[li_db][A_db_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_si];
  dQdx[li_db][A_db_Equ_p_NodeOffset] +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_p_GND] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  dQdx[li_db][A_db_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_Temp_t_GND];
}
  dQdx[li_db][A_db_Equ_e_NodeOffset] +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_e_si];
  dQdx[li_db][A_db_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_db][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_si];
  dQdx[li_db][A_db_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_db][admsProbeID_V_gi_si];
  dQdx[li_db][A_db_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_db][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_db][admsProbeID_V_di_si];
  dQdx[li_db][A_db_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_sb_si];
  dQdx[li_db][A_db_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_db][admsProbeID_V_db_di];
  dQdx[li_sb][A_sb_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_si];
  dQdx[li_sb][A_sb_Equ_p_NodeOffset] +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_p_GND] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_p];
if (!collapseNode_t)
{
  dQdx[li_sb][A_sb_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_Temp_t_GND];
}
  dQdx[li_sb][A_sb_Equ_e_NodeOffset] +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_e_si];
  dQdx[li_sb][A_sb_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_si];
  dQdx[li_sb][A_sb_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_gi_si];
  dQdx[li_sb][A_sb_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_sb][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_di_si];
  dQdx[li_sb][A_sb_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_sb_si];
  dQdx[li_sb][A_sb_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_sb][admsProbeID_V_db_di];
  dQdx[li_gi][A_gi_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si];
  dQdx[li_gi][A_gi_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si];
  dQdx[li_gi][A_gi_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di];
  dQdx[li_gi][A_gi_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_di_si];
  dQdx[li_gi][A_gi_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_b_si];
  dQdx[li_gi][A_gi_Equ_p_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dQdx[li_gi][A_gi_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_Temp_t_GND];
}
  dQdx[li_gi][A_gi_Equ_e_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_e] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_e_si];
  dQdx[li_gi][A_gi_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_e] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gi_si];
  dQdx[li_p][A_p_Equ_p_NodeOffset] +=  +d_dynamicContributions[admsNodeID_p][admsProbeID_V_p_GND];
  dQdx[li_p][A_p_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_p][admsProbeID_V_b_GND];
if (!collapseNode_t)
{
  dQdx[li_p][A_p_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_p][admsProbeID_Temp_t_GND];
}
  dQdx[li_gi][A_gi_Equ_gm_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gi][admsProbeID_V_gm_si];
  dQdx[li_b][A_b_Equ_gm_NodeOffset] +=  +d_dynamicContributions[admsNodeID_b][admsProbeID_V_gm_si];
  dQdx[li_e][A_e_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_GND] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si];
  dQdx[li_e][A_e_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_b_si];
  dQdx[li_e][A_e_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_e] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_si];
  dQdx[li_e][A_e_Equ_p_NodeOffset] +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dQdx[li_e][A_e_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_Temp_t_GND];
}
  dQdx[li_e][A_e_Equ_e_NodeOffset] +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gi_e] -d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_e] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_e_si];
  dQdx[li_e][A_e_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_e][admsProbeID_V_di_si];
  dQdx[li_gm][A_gm_Equ_gm_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_e] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_si];
  dQdx[li_gm][A_gm_Equ_si_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_e_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_si] -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_si];
  dQdx[li_gm][A_gm_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_si] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_b_GND];
  dQdx[li_gm][A_gm_Equ_p_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_p_GND];
if (!collapseNode_t)
{
  dQdx[li_gm][A_gm_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_Temp_t_GND];
}
  dQdx[li_gm][A_gm_Equ_gi_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_p] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gi_si];
  dQdx[li_gm][A_gm_Equ_di_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_di] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_di_si];
  dQdx[li_gm][A_gm_Equ_e_NodeOffset] +=  -d_dynamicContributions[admsNodeID_gm][admsProbeID_V_gm_e] +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_e_si];
  dQdx[li_gm][A_gm_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_sb_si];
  dQdx[li_gm][A_gm_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_gm][admsProbeID_V_db_di];
  dQdx[li_e][A_e_Equ_gm_NodeOffset] +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_gm_e];
  dQdx[li_e][A_e_Equ_sb_NodeOffset] +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_sb_si];
  dQdx[li_e][A_e_Equ_db_NodeOffset] +=  +d_dynamicContributions[admsNodeID_e][admsProbeID_V_db_di];
if (!collapseNode_t)
{
  dQdx[li_t][A_t_Equ_b_NodeOffset] +=  +d_dynamicContributions[admsNodeID_t][admsProbeID_V_b_GND];
}
if (!collapseNode_t)
{
  dQdx[li_t][A_t_Equ_p_NodeOffset] +=  +d_dynamicContributions[admsNodeID_t][admsProbeID_V_p_GND];
}
if (!collapseNode_t)
{
  dQdx[li_t][A_t_Equ_t_NodeOffset] +=  +d_dynamicContributions[admsNodeID_t][admsProbeID_Temp_t_GND];
}


#endif
  return bsuccess;
}




//-----------------------------------------------------------------------------
// Function      : Instance::updateTemperature
// Purpose       : Set temperature and update any parameters that depend on it
// Special Notes : In Xyce ADMS, we'll simply copy the temperature, which is
//                 in Kelvin, to our "admsTemperature" variable, which needs
//                 to be in Celsius.
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Instance::updateTemperature(const double & temperatureTemp)
{

  admsTemperature = temperatureTemp;
  adms_vt_nom = adms_vt(temperatureTemp);

  return true;
}

// Class Model
//-----------------------------------------------------------------------------
// Function      : Model::processParams
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
bool Model::processParams()
{

  // Set any non-constant parameter defaults:
  // Set any parameters that were not given and whose defaults depend on other
  // parameters:
  if (!given("AGBCPD"))
  {
  AGBCPD = AGBCP;
  }

// set internal model type based on model card type
if (getType() == "pmos" || getType() == "PMOS")
    TYPE = -1;
  if (!given("VGSTCVMOD"))
  {
  VGSTCVMOD = ((VERSION>=4.2)?1:0);
  }
  if (!given("VDDEOT"))
  {
  VDDEOT = ((TYPE==1)?1.5:(-1.5));
  }
  if (!given("TOXP"))
  {
  TOXP = TOX;
  }
  if (!given("TOXM"))
  {
  TOXM = TOX;
  }
  if (!given("DSUB"))
  {
  DSUB = DROUT;
  }
  if (!given("VTHO"))
  {
  VTHO = ((TYPE==1)?0.7:(-0.7));
  }
  if (!given("VTH0"))
  {
  VTH0 = VTHO;
  }
  if (!given("UC"))
  {
  UC = ((MOBMOD==3)?(-0.0465):(-0.0465e-9));
  }
  if (!given("UC1"))
  {
  UC1 = ((MOBMOD==3)?(-0.056):(-0.056e-9));
  }
  if (!given("U0"))
  {
  U0 = ((TYPE==1)?0.067:0.025);
  }
  if (!given("EU"))
  {
  EU = ((TYPE==1)?1.67:1.0);
  }
  if (!given("UCS"))
  {
  UCS = ((TYPE==1)?1.67:1.0);
  }
  if (!given("ETA0CV"))
  {
  ETA0CV = ETA0;
  }
  if (!given("ETABCV"))
  {
  ETABCV = ETAB;
  }
  if (!given("XJ"))
  {
  XJ = TSI;
  }
  if (!given("AGISL"))
  {
  AGISL = AGIDL;
  }
  if (!given("BGISL"))
  {
  BGISL = BGIDL;
  }
  if (!given("CGISL"))
  {
  CGISL = CGIDL;
  }
  if (!given("RGISL"))
  {
  RGISL = RGIDL;
  }
  if (!given("KGISL"))
  {
  KGISL = KGIDL;
  }
  if (!given("FGISL"))
  {
  FGISL = FGIDL;
  }
  if (!given("NDIODED"))
  {
  NDIODED = NDIODE;
  }
  if (!given("XDIF"))
  {
  XDIF = XBJT;
  }
  if (!given("XDIFD"))
  {
  XDIFD = XDIF;
  }
  if (!given("XRECD"))
  {
  XRECD = XREC;
  }
  if (!given("XTUND"))
  {
  XTUND = XTUN;
  }
  if (!given("PBSWGD"))
  {
  PBSWGD = PBSWG;
  }
  if (!given("MJSWGD"))
  {
  MJSWGD = MJSWG;
  }
  if (!given("CJSWGD"))
  {
  CJSWGD = CJSWG;
  }
  if (!given("DWC"))
  {
  DWC = WINT;
  }
  if (!given("DLC"))
  {
  DLC = LINT;
  }
  if (!given("NOIA"))
  {
  NOIA = ((TYPE==1)?6.25e41:6.188e40);
  }
  if (!given("NOIB"))
  {
  NOIB = ((TYPE==1)?3.125e26:1.5e25);
  }
  if (!given("STETA0CV"))
  {
  STETA0CV = STETA0;
  }
  if (!given("LODETA0CV"))
  {
  LODETA0CV = LODETA0;
  }
  if (!given("NTUND"))
  {
  NTUND = NTUN;
  }
  if (!given("NRECF0D"))
  {
  NRECF0D = NRECF0;
  }
  if (!given("NRECR0D"))
  {
  NRECR0D = NRECR0;
  }
  if (!given("IDBJT"))
  {
  IDBJT = ISBJT;
  }
  if (!given("IDDIF"))
  {
  IDDIF = ISDIF;
  }
  if (!given("IDREC"))
  {
  IDREC = ISREC;
  }
  if (!given("IDTUN"))
  {
  IDTUN = ISTUN;
  }
  if (!given("VREC0D"))
  {
  VREC0D = VREC0;
  }
  if (!given("VTUN0D"))
  {
  VTUN0D = VTUN0;
  }
  if (!given("AHLID"))
  {
  AHLID = AHLI;
  }
  if (!given("TCJSWGD"))
  {
  TCJSWGD = TCJSWG;
  }
  if (!given("TPBSWGD"))
  {
  TPBSWGD = TPBSWG;
  }
  if (!given("NOFF2"))
  {
  NOFF2 = NOFF;
  }
  if (!given("IGMOD"))
  {
  IGMOD = IGBMOD;
  }
  if (!given("TOXQM"))
  {
  TOXQM = TOX;
  }
  if (!given("AIGC"))
  {
  AIGC = ((TYPE==1)?0.43:0.31);
  }
  if (!given("BIGC"))
  {
  BIGC = ((TYPE==1)?0.054:0.024);
  }
  if (!given("CIGC"))
  {
  CIGC = ((TYPE==1)?0.075:0.03);
  }
  if (!given("AIGSD"))
  {
  AIGSD = ((TYPE==1)?0.43:0.31);
  }
  if (!given("BIGSD"))
  {
  BIGSD = ((TYPE==1)?0.054:0.024);
  }
  if (!given("CIGSD"))
  {
  CIGSD = ((TYPE==1)?0.075:0.03);
  }
  if (!given("DLCIG"))
  {
  DLCIG = LINT;
  }
  if (!given("LXDIFD"))
  {
  LXDIFD = LXDIF;
  }
  if (!given("LXRECD"))
  {
  LXRECD = LXREC;
  }
  if (!given("LXTUND"))
  {
  LXTUND = LXTUN;
  }
  if (!given("LETA0CV"))
  {
  LETA0CV = LETA0;
  }
  if (!given("LETABCV"))
  {
  LETABCV = LETAB;
  }
  if (!given("LAGISL"))
  {
  LAGISL = LAGIDL;
  }
  if (!given("LBGISL"))
  {
  LBGISL = LBGIDL;
  }
  if (!given("LCGISL"))
  {
  LCGISL = LCGIDL;
  }
  if (!given("LRGISL"))
  {
  LRGISL = LRGIDL;
  }
  if (!given("LKGISL"))
  {
  LKGISL = LKGIDL;
  }
  if (!given("LFGISL"))
  {
  LFGISL = LFGIDL;
  }
  if (!given("LNTUND"))
  {
  LNTUND = LNTUN;
  }
  if (!given("LNDIODED"))
  {
  LNDIODED = LNDIODE;
  }
  if (!given("LNRECF0D"))
  {
  LNRECF0D = LNRECF0;
  }
  if (!given("LNRECR0D"))
  {
  LNRECR0D = LNRECR0;
  }
  if (!given("LIDBJT"))
  {
  LIDBJT = LISBJT;
  }
  if (!given("LIDDIF"))
  {
  LIDDIF = LISDIF;
  }
  if (!given("LIDREC"))
  {
  LIDREC = LISREC;
  }
  if (!given("LIDTUN"))
  {
  LIDTUN = LISTUN;
  }
  if (!given("LVREC0D"))
  {
  LVREC0D = LVREC0;
  }
  if (!given("LVTUN0D"))
  {
  LVTUN0D = LVTUN0;
  }
  if (!given("LAHLID"))
  {
  LAHLID = LAHLI;
  }
  if (!given("LNOFF2"))
  {
  LNOFF2 = LNOFF;
  }
  if (!given("WXDIFD"))
  {
  WXDIFD = WXDIF;
  }
  if (!given("WXRECD"))
  {
  WXRECD = WXREC;
  }
  if (!given("WXTUND"))
  {
  WXTUND = WXTUN;
  }
  if (!given("WETA0CV"))
  {
  WETA0CV = WETA0;
  }
  if (!given("WETABCV"))
  {
  WETABCV = WETAB;
  }
  if (!given("WAGISL"))
  {
  WAGISL = WAGIDL;
  }
  if (!given("WBGISL"))
  {
  WBGISL = WBGIDL;
  }
  if (!given("WCGISL"))
  {
  WCGISL = WCGIDL;
  }
  if (!given("WRGISL"))
  {
  WRGISL = WRGIDL;
  }
  if (!given("WKGISL"))
  {
  WKGISL = WKGIDL;
  }
  if (!given("WFGISL"))
  {
  WFGISL = WFGIDL;
  }
  if (!given("WNTUND"))
  {
  WNTUND = WNTUN;
  }
  if (!given("WNDIODED"))
  {
  WNDIODED = WNDIODE;
  }
  if (!given("WNRECF0D"))
  {
  WNRECF0D = WNRECF0;
  }
  if (!given("WNRECR0D"))
  {
  WNRECR0D = WNRECR0;
  }
  if (!given("WIDBJT"))
  {
  WIDBJT = WISBJT;
  }
  if (!given("WIDDIF"))
  {
  WIDDIF = WISDIF;
  }
  if (!given("WIDREC"))
  {
  WIDREC = WISREC;
  }
  if (!given("WIDTUN"))
  {
  WIDTUN = WISTUN;
  }
  if (!given("WVREC0D"))
  {
  WVREC0D = WVREC0;
  }
  if (!given("WVTUN0D"))
  {
  WVTUN0D = WVTUN0;
  }
  if (!given("WAHLID"))
  {
  WAHLID = WAHLI;
  }
  if (!given("WNOFF2"))
  {
  WNOFF2 = WNOFF;
  }
  if (!given("PXDIFD"))
  {
  PXDIFD = PXDIF;
  }
  if (!given("PXRECD"))
  {
  PXRECD = PXREC;
  }
  if (!given("PXTUND"))
  {
  PXTUND = PXTUN;
  }
  if (!given("PETA0CV"))
  {
  PETA0CV = PETA0;
  }
  if (!given("PETABCV"))
  {
  PETABCV = PETAB;
  }
  if (!given("PAGISL"))
  {
  PAGISL = PAGIDL;
  }
  if (!given("PBGISL"))
  {
  PBGISL = PBGIDL;
  }
  if (!given("PCGISL"))
  {
  PCGISL = PCGIDL;
  }
  if (!given("PRGISL"))
  {
  PRGISL = PRGIDL;
  }
  if (!given("PKGISL"))
  {
  PKGISL = PKGIDL;
  }
  if (!given("PFGISL"))
  {
  PFGISL = PFGIDL;
  }
  if (!given("PNTUND"))
  {
  PNTUND = PNTUN;
  }
  if (!given("PNDIODED"))
  {
  PNDIODED = PNDIODE;
  }
  if (!given("PNRECF0D"))
  {
  PNRECF0D = PNRECF0;
  }
  if (!given("PNRECR0D"))
  {
  PNRECR0D = PNRECR0;
  }
  if (!given("PIDBJT"))
  {
  PIDBJT = PISBJT;
  }
  if (!given("PIDDIF"))
  {
  PIDDIF = PISDIF;
  }
  if (!given("PIDREC"))
  {
  PIDREC = PISREC;
  }
  if (!given("PIDTUN"))
  {
  PIDTUN = PISTUN;
  }
  if (!given("PVREC0D"))
  {
  PVREC0D = PVREC0;
  }
  if (!given("PVTUN0D"))
  {
  PVTUN0D = PVTUN0;
  }
  if (!given("PAHLID"))
  {
  PAHLID = PAHLI;
  }
  if (!given("PNOFF2"))
  {
  PNOFF2 = PNOFF;
  }
  if (!given("LPE0"))
  {
  LPE0 = NLX;
  }
  if (!given("EGIDL"))
  {
  EGIDL = NGIDL;
  }
  if (!given("EGISL"))
  {
  EGISL = EGIDL;
  }
  if (!given("LLPE0"))
  {
  LLPE0 = LNLX;
  }
  if (!given("LEGIDL"))
  {
  LEGIDL = LNGIDL;
  }
  if (!given("LEGISL"))
  {
  LEGISL = LEGIDL;
  }
  if (!given("WLPE0"))
  {
  WLPE0 = WNLX;
  }
  if (!given("WEGIDL"))
  {
  WEGIDL = WNGIDL;
  }
  if (!given("WEGISL"))
  {
  WEGISL = WEGIDL;
  }
  if (!given("PLPE0"))
  {
  PLPE0 = PNLX;
  }
  if (!given("PEGIDL"))
  {
  PEGIDL = PNGIDL;
  }
  if (!given("PEGISL"))
  {
  PEGISL = PEGIDL;
  }



  // Now we need to check that any parameters are within their ranges as
  // specified in the verilog:

//    Parameter L : ] 0,  (+inf) [
  if ( (!((L >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter L value " << L << " out of range ] 0,  (+inf) [";
  }

//    Parameter W : ] 0,  (+inf) [
  if ( (!((W >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter W value " << W << " out of range ] 0,  (+inf) [";
  }

//    Parameter NF : [ 1,  (+inf) [
  if ( (!((NF >=1))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NF value " << NF << " out of range [ 1,  (+inf) [";
  }

//    Parameter SA : [ 0,  (+inf) [
  if ( (!((SA >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SA value " << SA << " out of range [ 0,  (+inf) [";
  }

//    Parameter SB : [ 0,  (+inf) [
  if ( (!((SB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SB value " << SB << " out of range [ 0,  (+inf) [";
  }

//    Parameter SD : [ 0,  (+inf) [
  if ( (!((SD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SD value " << SD << " out of range [ 0,  (+inf) [";
  }

//    Parameter AD : [ 0,  (+inf) [
  if ( (!((AD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AD value " << AD << " out of range [ 0,  (+inf) [";
  }

//    Parameter AS : [ 0,  (+inf) [
  if ( (!((AS >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AS value " << AS << " out of range [ 0,  (+inf) [";
  }

//    Parameter PD : [ 0,  (+inf) [
  if ( (!((PD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PD value " << PD << " out of range [ 0,  (+inf) [";
  }

//    Parameter PS : [ 0,  (+inf) [
  if ( (!((PS >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PS value " << PS << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRD : [ 0,  (+inf) [
  if ( (!((NRD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRD value " << NRD << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRS : [ 0,  (+inf) [
  if ( (!((NRS >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRS value " << NRS << " out of range [ 0,  (+inf) [";
  }

//    Parameter OFF : [ 0, 1 ]
  if ( (!((OFF >=0 && OFF <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter OFF value " << OFF << " out of range [ 0, 1 ]";
  }

//    Parameter BJTOFF : [ 0, 1 ]
  if ( (!((BJTOFF >=0 && BJTOFF <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter BJTOFF value " << BJTOFF << " out of range [ 0, 1 ]";
  }

//    Parameter RTH0 : [ 0,  (+inf) [
  if ( (!((RTH0 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RTH0 value " << RTH0 << " out of range [ 0,  (+inf) [";
  }

//    Parameter CTH0 : [ 0,  (+inf) [
  if ( (!((CTH0 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CTH0 value " << CTH0 << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRB : [ 0,  (+inf) [
  if ( (!((NRB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRB value " << NRB << " out of range [ 0,  (+inf) [";
  }

//    Parameter RBDB : [ 0,  (+inf) [
  if ( (!((RBDB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RBDB value " << RBDB << " out of range [ 0,  (+inf) [";
  }

//    Parameter RBSB : [ 0,  (+inf) [
  if ( (!((RBSB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RBSB value " << RBSB << " out of range [ 0,  (+inf) [";
  }

//    Parameter SOIMOD : [ 0, 3 ]
  if ( (!((SOIMOD >=0 && SOIMOD <=3 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SOIMOD value " << SOIMOD << " out of range [ 0, 3 ]";
  }

//    Parameter NBC : [ 0,  (+inf) [
  if ( (!((NBC >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NBC value " << NBC << " out of range [ 0,  (+inf) [";
  }

//    Parameter NSEG : [ 1,  (+inf) [
  if ( (!((NSEG >=1))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NSEG value " << NSEG << " out of range [ 1,  (+inf) [";
  }

//    Parameter PDBCP : [ 0,  (+inf) [
  if ( (!((PDBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PDBCP value " << PDBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter PSBCP : [ 0,  (+inf) [
  if ( (!((PSBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter PSBCP value " << PSBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter AGBCP : [ 0,  (+inf) [
  if ( (!((AGBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AGBCP value " << AGBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter AGBCP2 : [ 0,  (+inf) [
  if ( (!((AGBCP2 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AGBCP2 value " << AGBCP2 << " out of range [ 0,  (+inf) [";
  }

//    Parameter AGBCPD : [ 0,  (+inf) [
  if ( (!((AGBCPD >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AGBCPD value " << AGBCPD << " out of range [ 0,  (+inf) [";
  }

//    Parameter AEBCP : [ 0,  (+inf) [
  if ( (!((AEBCP >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter AEBCP value " << AEBCP << " out of range [ 0,  (+inf) [";
  }

//    Parameter SHMOD : [ 0, 1 ]
  if ( (!((SHMOD >=0 && SHMOD <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SHMOD value " << SHMOD << " out of range [ 0, 1 ]";
  }

//    Parameter TYPE : [ (-1), 1 ]] 0, 0 [
  if ( (!((TYPE >=(-1) && TYPE <=1 )) || (TYPE >0 && TYPE <0 )) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TYPE value " << TYPE << " out of range [ (-1), 1 ] or  ] 0, 0 [";
  }

//    Parameter VERSION : [ 4.0, 5.0 [
  if ( (!((VERSION >=4.0 && VERSION <5.0 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter VERSION value " << VERSION << " out of range [ 4.0, 5.0 [";
  }

//    Parameter RGATEMOD : [ 0, 3 ]
  if ( (!((RGATEMOD >=0 && RGATEMOD <=3 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RGATEMOD value " << RGATEMOD << " out of range [ 0, 3 ]";
  }

//    Parameter RBODYMOD : [ 0, 1 ]
  if ( (!((RBODYMOD >=0 && RBODYMOD <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RBODYMOD value " << RBODYMOD << " out of range [ 0, 1 ]";
  }

//    Parameter MTRLMOD : [ 0, 1 ]
  if ( (!((MTRLMOD >=0 && MTRLMOD <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter MTRLMOD value " << MTRLMOD << " out of range [ 0, 1 ]";
  }

//    Parameter VGSTCVMOD : [ 0, 2 ]
  if ( (!((VGSTCVMOD >=0 && VGSTCVMOD <=2 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter VGSTCVMOD value " << VGSTCVMOD << " out of range [ 0, 2 ]";
  }

//    Parameter GIDLMOD : [ 0, 1 ]
  if ( (!((GIDLMOD >=0 && GIDLMOD <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter GIDLMOD value " << GIDLMOD << " out of range [ 0, 1 ]";
  }

//    Parameter IIIMOD : [ 0, 1 ]
  if ( (!((IIIMOD >=0 && IIIMOD <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter IIIMOD value " << IIIMOD << " out of range [ 0, 1 ]";
  }

//    Parameter EOT : [ 0,  (+inf) [
  if ( (!((EOT >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter EOT value " << EOT << " out of range [ 0,  (+inf) [";
  }

//    Parameter EPSROX : [ 0,  (+inf) [
  if ( (!((EPSROX >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter EPSROX value " << EPSROX << " out of range [ 0,  (+inf) [";
  }

//    Parameter EPSRSUB : ] 0,  (+inf) [
  if ( (!((EPSRSUB >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter EPSRSUB value " << EPSRSUB << " out of range ] 0,  (+inf) [";
  }

//    Parameter NI0SUB : [ 0,  (+inf) [
  if ( (!((NI0SUB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NI0SUB value " << NI0SUB << " out of range [ 0,  (+inf) [";
  }

//    Parameter BG0SUB : [ 0,  (+inf) [
  if ( (!((BG0SUB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter BG0SUB value " << BG0SUB << " out of range [ 0,  (+inf) [";
  }

//    Parameter TBGASUB : [ 0,  (+inf) [
  if ( (!((TBGASUB >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TBGASUB value " << TBGASUB << " out of range [ 0,  (+inf) [";
  }

//    Parameter LEFFEOT : ] 0,  (+inf) [
  if ( (!((LEFFEOT >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter LEFFEOT value " << LEFFEOT << " out of range ] 0,  (+inf) [";
  }

//    Parameter EPSRGATE : ] 0,  (+inf) [
  if ( (!((EPSRGATE >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter EPSRGATE value " << EPSRGATE << " out of range ] 0,  (+inf) [";
  }

//    Parameter CAPMOD : [ 2, 3 ]
  if ( (!((CAPMOD >=2 && CAPMOD <=3 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CAPMOD value " << CAPMOD << " out of range [ 2, 3 ]";
  }

//    Parameter MOBMOD : [ 1, 4 ]
  if ( (!((MOBMOD >=1 && MOBMOD <=4 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter MOBMOD value " << MOBMOD << " out of range [ 1, 4 ]";
  }

//    Parameter TOX : ] 0,  (+inf) [
  if ( (!((TOX >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TOX value " << TOX << " out of range ] 0,  (+inf) [";
  }

//    Parameter TOXP : ] 0,  (+inf) [
  if ( (!((TOXP >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TOXP value " << TOXP << " out of range ] 0,  (+inf) [";
  }

//    Parameter TOXM : ] 0,  (+inf) [
  if ( (!((TOXM >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TOXM value " << TOXM << " out of range ] 0,  (+inf) [";
  }

//    Parameter VSAT : ] 0,  (+inf) [
  if ( (!((VSAT >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter VSAT value " << VSAT << " out of range ] 0,  (+inf) [";
  }

//    Parameter NSUB : ] 0,  (+inf) [
  if ( (!((NSUB >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NSUB value " << NSUB << " out of range ] 0,  (+inf) [";
  }

//    Parameter NCH : ] 0,  (+inf) [
  if ( (!((NCH >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NCH value " << NCH << " out of range ] 0,  (+inf) [";
  }

//    Parameter NGATE : [ 0,  (+inf) [
  if ( (!((NGATE >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NGATE value " << NGATE << " out of range [ 0,  (+inf) [";
  }

//    Parameter NSD : ] 0,  (+inf) [
  if ( (!((NSD >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NSD value " << NSD << " out of range ] 0,  (+inf) [";
  }

//    Parameter CGSO : [ 0,  (+inf) [
  if ( (!((CGSO >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CGSO value " << CGSO << " out of range [ 0,  (+inf) [";
  }

//    Parameter CGDO : [ 0,  (+inf) [
  if ( (!((CGDO >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CGDO value " << CGDO << " out of range [ 0,  (+inf) [";
  }

//    Parameter RSH : [ 0,  (+inf) [
  if ( (!((RSH >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RSH value " << RSH << " out of range [ 0,  (+inf) [";
  }

//    Parameter RDSW : [ 0,  (+inf) [
  if ( (!((RDSW >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RDSW value " << RDSW << " out of range [ 0,  (+inf) [";
  }

//    Parameter RSW : [ 0,  (+inf) [
  if ( (!((RSW >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RSW value " << RSW << " out of range [ 0,  (+inf) [";
  }

//    Parameter RDW : [ 0,  (+inf) [
  if ( (!((RDW >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RDW value " << RDW << " out of range [ 0,  (+inf) [";
  }

//    Parameter RSWMIN : [ 0,  (+inf) [
  if ( (!((RSWMIN >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RSWMIN value " << RSWMIN << " out of range [ 0,  (+inf) [";
  }

//    Parameter RDWMIN : [ 0,  (+inf) [
  if ( (!((RDWMIN >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter RDWMIN value " << RDWMIN << " out of range [ 0,  (+inf) [";
  }

//    Parameter TBOX : ] 0,  (+inf) [
  if ( (!((TBOX >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TBOX value " << TBOX << " out of range ] 0,  (+inf) [";
  }

//    Parameter TSI : ] 0,  (+inf) [
  if ( (!((TSI >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TSI value " << TSI << " out of range ] 0,  (+inf) [";
  }

//    Parameter ETSI : ] 0,  (+inf) [
  if ( (!((ETSI >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter ETSI value " << ETSI << " out of range ] 0,  (+inf) [";
  }

//    Parameter XJ : ] 0,  (+inf) [
  if ( (!((XJ >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter XJ value " << XJ << " out of range ] 0,  (+inf) [";
  }

//    Parameter CJSWG : ] 0,  (+inf) [
  if ( (!((CJSWG >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CJSWG value " << CJSWG << " out of range ] 0,  (+inf) [";
  }

//    Parameter CJSWGD : ] 0,  (+inf) [
  if ( (!((CJSWGD >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CJSWGD value " << CJSWGD << " out of range ] 0,  (+inf) [";
  }

//    Parameter CKAPPA : ] 0,  (+inf) [
  if ( (!((CKAPPA >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter CKAPPA value " << CKAPPA << " out of range ] 0,  (+inf) [";
  }

//    Parameter FNOIMOD : [ 0, 1 ]
  if ( (!((FNOIMOD >=0 && FNOIMOD <=1 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter FNOIMOD value " << FNOIMOD << " out of range [ 0, 1 ]";
  }

//    Parameter TNOIMOD : [ 0, 3 ]
  if ( (!((TNOIMOD >=0 && TNOIMOD <=3 ))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TNOIMOD value " << TNOIMOD << " out of range [ 0, 3 ]";
  }

//    Parameter NTNOI : [ 0,  (+inf) [
  if ( (!((NTNOI >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NTNOI value " << NTNOI << " out of range [ 0,  (+inf) [";
  }

//    Parameter SAREF : ] 0,  (+inf) [
  if ( (!((SAREF >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SAREF value " << SAREF << " out of range ] 0,  (+inf) [";
  }

//    Parameter SBREF : ] 0,  (+inf) [
  if ( (!((SBREF >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter SBREF value " << SBREF << " out of range ] 0,  (+inf) [";
  }

//    Parameter NOIF : [ 0,  (+inf) [
  if ( (!((NOIF >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NOIF value " << NOIF << " out of range [ 0,  (+inf) [";
  }

//    Parameter NTUN : [ 0,  (+inf) [
  if ( (!((NTUN >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NTUN value " << NTUN << " out of range [ 0,  (+inf) [";
  }

//    Parameter NTUND : [ 0,  (+inf) [
  if ( (!((NTUND >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NTUND value " << NTUND << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRECF0 : [ 0,  (+inf) [
  if ( (!((NRECF0 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRECF0 value " << NRECF0 << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRECF0D : [ 0,  (+inf) [
  if ( (!((NRECF0D >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRECF0D value " << NRECF0D << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRECR0 : [ 0,  (+inf) [
  if ( (!((NRECR0 >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRECR0 value " << NRECR0 << " out of range [ 0,  (+inf) [";
  }

//    Parameter NRECR0D : [ 0,  (+inf) [
  if ( (!((NRECR0D >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NRECR0D value " << NRECR0D << " out of range [ 0,  (+inf) [";
  }

//    Parameter ISBJT : [ 0,  (+inf) [
  if ( (!((ISBJT >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter ISBJT value " << ISBJT << " out of range [ 0,  (+inf) [";
  }

//    Parameter IDBJT : [ 0,  (+inf) [
  if ( (!((IDBJT >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter IDBJT value " << IDBJT << " out of range [ 0,  (+inf) [";
  }

//    Parameter ISDIF : [ 0,  (+inf) [
  if ( (!((ISDIF >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter ISDIF value " << ISDIF << " out of range [ 0,  (+inf) [";
  }

//    Parameter IDDIF : [ 0,  (+inf) [
  if ( (!((IDDIF >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter IDDIF value " << IDDIF << " out of range [ 0,  (+inf) [";
  }

//    Parameter ISREC : [ 0,  (+inf) [
  if ( (!((ISREC >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter ISREC value " << ISREC << " out of range [ 0,  (+inf) [";
  }

//    Parameter IDREC : [ 0,  (+inf) [
  if ( (!((IDREC >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter IDREC value " << IDREC << " out of range [ 0,  (+inf) [";
  }

//    Parameter ISTUN : [ 0,  (+inf) [
  if ( (!((ISTUN >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter ISTUN value " << ISTUN << " out of range [ 0,  (+inf) [";
  }

//    Parameter IDTUN : [ 0,  (+inf) [
  if ( (!((IDTUN >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter IDTUN value " << IDTUN << " out of range [ 0,  (+inf) [";
  }

//    Parameter LN : [ 0,  (+inf) [
  if ( (!((LN >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter LN value " << LN << " out of range [ 0,  (+inf) [";
  }

//    Parameter TT : [ 0,  (+inf) [
  if ( (!((TT >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TT value " << TT << " out of range [ 0,  (+inf) [";
  }

//    Parameter ASD : ] 0,  (+inf) [
  if ( (!((ASD >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter ASD value " << ASD << " out of range ] 0,  (+inf) [";
  }

//    Parameter NOFF : ] 0,  (+inf) [
  if ( (!((NOFF >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NOFF value " << NOFF << " out of range ] 0,  (+inf) [";
  }

//    Parameter NOFF2 : ] 0,  (+inf) [
  if ( (!((NOFF2 >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NOFF2 value " << NOFF2 << " out of range ] 0,  (+inf) [";
  }

//    Parameter TOXREF : [ 0,  (+inf) [
  if ( (!((TOXREF >=0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter TOXREF value " << TOXREF << " out of range [ 0,  (+inf) [";
  }

//    Parameter VEVB : ] 0,  (+inf) [
  if ( (!((VEVB >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter VEVB value " << VEVB << " out of range ] 0,  (+inf) [";
  }

//    Parameter VECB : ] 0,  (+inf) [
  if ( (!((VECB >0))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter VECB value " << VECB << " out of range ] 0,  (+inf) [";
  }

//    Parameter NGCON : [ 1,  (+inf) [
  if ( (!((NGCON >=1))) )
  {
    UserWarning(*this) << "ADMSbsimsoi: Parameter NGCON value " << NGCON << " out of range [ 1,  (+inf) [";
  }

  // and of course, this routine is where we should put the initial_model
  // stuff


  return true;
}
//----------------------------------------------------------------------------
// Function      : Model::processInstanceParams
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       :
// Creation Date :
//----------------------------------------------------------------------------
bool Model::processInstanceParams()
{

  std::vector<Instance*>::iterator iter;
  std::vector<Instance*>::iterator first = instanceContainer.begin();
  std::vector<Instance*>::iterator last  = instanceContainer.end();

  for (iter=first; iter!=last; ++iter)
  {
    (*iter)->processParams();
  }

  return true;
}

//-----------------------------------------------------------------------------
// Function      : Model::Model
// Purpose       : model block constructor
// Special Notes :
// Scope         : public
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
Model::Model(
  const Configuration & configuration,
  const ModelBlock &    model_block,
  const FactoryBlock &  factory_block)
  : DeviceModel(model_block, configuration.getModelParameters(), factory_block),
    L(5e-6),
    W(5e-6),
    NF(1),
    SA(0.0),
    SB(0.0),
    SD(0.0),
    AD(0.0),
    AS(0.0),
    PD(0.0),
    PS(0.0),
    NRD(1.0),
    NRS(1.0),
    OFF(0),
    BJTOFF(0),
    RTH0(0.0),
    CTH0(1e-5),
    NRB(1.0),
    FRBODY(1.0),
    RBDB(50.0),
    RBSB(50.0),
    DELVTO(0.0),
    SOIMOD(0),
    NBC(0.0),
    NSEG(1),
    PDBCP(0.0),
    PSBCP(0.0),
    AGBCP(0.0),
    AGBCP2(0.0),
    AGBCPD(0.0),
    AEBCP(0.0),
    SHMOD(0),
    TYPE(1),
    VERSION(4.60),
    VBSUSR(0.0),
    RGATEMOD(0),
    RBODYMOD(0),
    MTRLMOD(0),
    VGSTCVMOD(0),
    GIDLMOD(0),
    IIIMOD(0),
    EOT(100.0e-10),
    EPSROX(3.9),
    EPSRSUB(11.7),
    NI0SUB(1.45e10),
    BG0SUB(1.16),
    TBGASUB(7.02e-4),
    TBGBSUB(1108.0),
    PHIG(4.05),
    EASUB(4.05),
    LEFFEOT(1.0),
    WEFFEOT(10.0),
    VDDEOT(0.0),
    TEMPEOT(300.15),
    ADOS(1.0),
    BDOS(1.0),
    EPSRGATE(11.7),
    CAPMOD(2),
    MOBMOD(1),
    PARAMCHK(0),
    NODECHK(1),
    BINUNIT(1),
    TOX(100.0e-10),
    TOXP(0.0),
    TOXM(0.0),
    DTOXCV(0.0),
    CDSC(2.4e-4),
    CDSCB(0.0),
    CDSCD(0.0),
    CIT(0.0),
    NFACTOR(1),
    VSAT(8.0e4),
    AT(3.3e4),
    A0(1.0),
    AGS(0.0),
    A1(0.0),
    A2(1.0),
    KETA((-0.6)),
    NSUB(6.0e16),
    NCH(1.7e17),
    NGATE(0.0),
    NSD(1.0e20),
    GAMMA1(0.0),
    GAMMA2(0.0),
    VBX(0.0),
    VBM((-3.0)),
    XT(1.55e-7),
    K1(0.53),
    KT1((-0.11)),
    KT1L(0.0),
    KT2(0.022),
    K2((-0.0186)),
    K3(0.0),
    K3B(0.0),
    W0(2.5e-6),
    LPEB(0.0),
    DVT0(2.2),
    DVT1(0.53),
    DVT2((-0.032)),
    DVT0W(0.0),
    DVT1W(5.3e6),
    DVT2W((-0.032)),
    DROUT(0.56),
    DSUB(0.0),
    VTHO(0.0),
    VTH0(0.0),
    VFB((-1.0)),
    UA(2.25e-9),
    UA1(4.31e-9),
    UB(5.87e-19),
    UB1((-7.61e-18)),
    UC(0.0),
    UC1(0.0),
    U0(0.0),
    EU(0.0),
    UTE((-1.5)),
    UCS(0.0),
    UCSTE((-4.775e-3)),
    UD(0.0),
    UD1(0.0),
    VOFF((-0.08)),
    TNOM(27.0),
    CGSO(0.0),
    CGDO(0.0),
    XPART(0.0),
    DELTA(0.01),
    RSH(0.0),
    RDSW(100),
    RSW(50),
    RDW(50),
    RSWMIN(0.0),
    RDWMIN(0.0),
    PRWG(0.0),
    PRWB(0.0),
    PRT(0.0),
    ETA0(0.08),
    ETAB((-0.07)),
    ETA0CV(0.0),
    ETABCV(0.0),
    PCLM(1.3),
    PDIBLC1(0.39),
    PDIBLC2(0.0086),
    PDIBLCB(0.0),
    PVAG(0.0),
    TBOX(3e-7),
    TSI(1e-7),
    ETSI(1e-7),
    XJ(0.0),
    AGIDL(0.0),
    BGIDL(2.3e9),
    CGIDL(0.5),
    RGIDL(1.0),
    KGIDL(0.0),
    FGIDL(0.0),
    AGISL(0.0),
    BGISL(0.0),
    CGISL(0.0),
    RGISL(0.0),
    KGISL(0.0),
    FGISL(0.0),
    NDIODE(1.0),
    NDIODED(0.0),
    XBJT(1.0),
    XDIF(0.0),
    XREC(1.0),
    XTUN(0.0),
    XDIFD(0.0),
    XRECD(0.0),
    XTUND(0.0),
    PBSWG(0.7),
    PBSWGD(0.0),
    MJSWG(0.5),
    MJSWGD(0.0),
    CJSWG(1e-10),
    CJSWGD(0.0),
    LINT(0.0),
    LL(0.0),
    LLC(0.0),
    LLN(1.0),
    LW(0.0),
    LWC(0.0),
    LWN(1.0),
    LWL(0.0),
    LWLC(0.0),
    WR(1.0),
    WINT(0.0),
    DWG(0.0),
    DWB(0.0),
    WL(0.0),
    WLC(0.0),
    WLN(1.0),
    WW(0.0),
    WWC(0.0),
    WWN(1.0),
    WWL(0.0),
    WWLC(0.0),
    B0(0.0),
    B1(0.0),
    CGSL(0.0),
    CGDL(0.0),
    CKAPPA(0.6),
    CF(0.0),
    CLC(0.1e-7),
    CLE(0.0),
    DWC(0.0),
    DLC(0.0),
    ALPHA0(0.0),
    NOIA(0.0),
    NOIB(0.0),
    NOIC(8.75e9),
    FNOIMOD(1),
    TNOIMOD(0),
    TNOIA(1.5),
    TNOIB(3.5),
    RNOIA(0.577),
    RNOIB(0.37),
    NTNOI(1.0),
    SAREF(1e-6),
    SBREF(1e-6),
    WLOD(0.0),
    KU0(0.0),
    KVSAT(0.0),
    KVTH0(0.0),
    TKU0(0.0),
    LLODKU0(0.0),
    WLODKU0(0.0),
    LLODVTH(0.0),
    WLODVTH(0.0),
    LKU0(0.0),
    WKU0(0.0),
    PKU0(0.0),
    LKVTH0(0.0),
    WKVTH0(0.0),
    PKVTH0(0.0),
    STK2(0.0),
    LODK2(1.0),
    STETA0(0.0),
    LODETA0(1.0),
    STETA0CV(0.0),
    LODETA0CV(0.0),
    GBMIN(1.0e-12),
    BF(2.0),
    W0FLK(10.0e-6),
    DVTP0(0.0),
    LDVTP0(0.0),
    WDVTP0(0.0),
    PDVTP0(0.0),
    DVTP1(0.0),
    LDVTP1(0.0),
    WDVTP1(0.0),
    PDVTP1(0.0),
    DVTP2(0.0),
    LDVTP2(0.0),
    WDVTP2(0.0),
    PDVTP2(0.0),
    DVTP3(0.0),
    LDVTP3(0.0),
    WDVTP3(0.0),
    PDVTP3(0.0),
    DVTP4(0.0),
    LDVTP4(0.0),
    WDVTP4(0.0),
    PDVTP4(0.0),
    MINV(0.0),
    LMINV(0.0),
    WMINV(0.0),
    PMINV(0.0),
    PDITS(1e-20),
    PDITSL(0.0),
    PDITSD(0.0),
    FPROUT(0.0),
    LFPROUT(0.0),
    LPDITS(0.0),
    LPDITSD(0.0),
    WFPROUT(0.0),
    WPDITS(0.0),
    WPDITSD(0.0),
    PFPROUT(0.0),
    PPDITS(0.0),
    PPDITSD(0.0),
    EM(4.1e7),
    EF(1.0),
    AF(1.0),
    KF(0.0),
    NOIF(1.0),
    K1W1(0.0),
    K1W2(0.0),
    KETAS(0.0),
    DWBC(0.0),
    BETA0(0.0),
    BETA1(0.0),
    BETA2(0.1),
    VDSATII0(0.9),
    TII(0.0),
    LII(0.0),
    SII0(0.5),
    SII1(0.1),
    SII2(0.0),
    SIID(0.0),
    FBJTII(0.0),
    EBJTII(0.0),
    CBJTII(0.0),
    VBCI(0.0),
    ABJTII(0.0),
    MBJTII(0.4),
    TVBCI(0.0),
    ESATII(1e7),
    NTUN(10.0),
    NTUND(0.0),
    NRECF0(2.0),
    NRECF0D(0.0),
    NRECR0(10.0),
    NRECR0D(0.0),
    ISBJT(1e-6),
    IDBJT(0.0),
    ISDIF(0.0),
    IDDIF(0.0),
    ISREC(1e-5),
    IDREC(0.0),
    ISTUN(0.0),
    IDTUN(0.0),
    LN(2e-6),
    VREC0(0.0),
    VREC0D(0.0),
    VTUN0(0.0),
    VTUN0D(0.0),
    NBJT(1.0),
    LBJT0(0.20e-6),
    LDIF0(1.0),
    VABJT(10.0),
    AELY(0.0),
    AHLI(0.0),
    AHLID(0.0),
    RBODY(0.0),
    RBSH(0.0),
    CGEO(0.0),
    TT(1e-12),
    NDIF((-1.0)),
    VSDFB(0.0),
    VSDTH(0.0),
    CSDMIN(0.0),
    ASD(0.3),
    CSDESW(0.0),
    NTRECF(0.0),
    NTRECR(0.0),
    DLCB(0.0),
    FBODY(1.0),
    TCJSWG(0.0),
    TPBSWG(0.0),
    TCJSWGD(0.0),
    TPBSWGD(0.0),
    ACDE(1.0),
    MOIN(15.0),
    NOFF(1.0),
    NOFF2(0.0),
    DELVT(0.0),
    KB1(1.0),
    DLBG(0.0),
    CFRCOEFF(1.0),
    IGBMOD(0),
    IGMOD(0),
    IGCMOD(0),
    TOXQM(0.0),
    WTH0(0.0),
    RHALO(1e15),
    NTOX(1),
    TOXREF(2.5e-9),
    EBG(1.2),
    VEVB(0.075),
    ALPHAGB1(0.35),
    BETAGB1(0.03),
    VGB1(300),
    VECB(0.026),
    ALPHAGB2(0.43),
    BETAGB2(0.05),
    VGB2(17),
    AIGBCP2(0.043),
    BIGBCP2(0.0054),
    CIGBCP2(0.0075),
    VOXH(5.0),
    DELTAVOX(0.005),
    AIGC(0.0),
    BIGC(0.0),
    CIGC(0.0),
    AIGSD(0.0),
    BIGSD(0.0),
    CIGSD(0.0),
    NIGC(1.0),
    PIGCD(1.0),
    POXEDGE(1.0),
    DLCIG(0.0),
    VBS0PD(0.0),
    VBS0FD(0.5),
    VBSA(0.0),
    NOFFFD(1.0),
    VOFFFD(0.0),
    K1B(1.0),
    K2B(0.0),
    DK2B(0.0),
    DVBD0(0.0),
    DVBD1(0.0),
    MOINFD(1e3),
    XRCRG1(12.0),
    XRCRG2(1.0),
    RSHG(0.1),
    NGCON(1.0),
    XGW(0.0),
    XGL(0.0),
    RDSMOD(0),
    FDMOD(0),
    VSCE(0.0),
    CDSBS(0.0),
    MINVCV(0.0),
    LMINVCV(0.0),
    WMINVCV(0.0),
    PMINVCV(0.0),
    VOFFCV(0.0),
    LVOFFCV(0.0),
    WVOFFCV(0.0),
    PVOFFCV(0.0),
    LXJ(0.0),
    LALPHAGB1(0.0),
    LBETAGB1(0.0),
    LALPHAGB2(0.0),
    LBETAGB2(0.0),
    LAIGBCP2(0.0),
    LBIGBCP2(0.0),
    LCIGBCP2(0.0),
    LCGSL(0.0),
    LCGDL(0.0),
    LCKAPPA(0.0),
    LNDIF(0.0),
    LUTE(0.0),
    LKT1(0.0),
    LKT1L(0.0),
    LKT2(0.0),
    LUA1(0.0),
    LUB1(0.0),
    LUC1(0.0),
    LAT(0.0),
    LPRT(0.0),
    LNTRECF(0.0),
    LNTRECR(0.0),
    LXBJT(0.0),
    LXDIF(0.0),
    LXREC(0.0),
    LXTUN(0.0),
    LXDIFD(0.0),
    LXRECD(0.0),
    LXTUND(0.0),
    LAIGC(0.0),
    LBIGC(0.0),
    LCIGC(0.0),
    LAIGSD(0.0),
    LBIGSD(0.0),
    LCIGSD(0.0),
    LNIGC(0.0),
    LPIGCD(0.0),
    LPOXEDGE(0.0),
    LNCH(0.0),
    LNSUB(0.0),
    LNGATE(0.0),
    LNSD(0.0),
    LVTH0(0.0),
    LVFB(0.0),
    LK1(0.0),
    LK1W1(0.0),
    LK1W2(0.0),
    LK2(0.0),
    LK3(0.0),
    LK3B(0.0),
    LKB1(0.0),
    LW0(0.0),
    LLPEB(0.0),
    LDVT0(0.0),
    LDVT1(0.0),
    LDVT2(0.0),
    LDVT0W(0.0),
    LDVT1W(0.0),
    LDVT2W(0.0),
    LU0(0.0),
    LEU(0.0),
    LUA(0.0),
    LUB(0.0),
    LUC(0.0),
    LUD(0.0),
    LUD1(0.0),
    LUCSTE(0.0),
    LUCS(0.0),
    LVSAT(0.0),
    LA0(0.0),
    LAGS(0.0),
    LB0(0.0),
    LB1(0.0),
    LKETA(0.0),
    LKETAS(0.0),
    LA1(0.0),
    LA2(0.0),
    LRDSW(0.0),
    LRSW(0.0),
    LRDW(0.0),
    LPRWB(0.0),
    LPRWG(0.0),
    LWR(0.0),
    LNFACTOR(0.0),
    LDWG(0.0),
    LDWB(0.0),
    LVOFF(0.0),
    LETA0(0.0),
    LETAB(0.0),
    LETA0CV(0.0),
    LETABCV(0.0),
    LDSUB(0.0),
    LCIT(0.0),
    LCDSC(0.0),
    LCDSCB(0.0),
    LCDSCD(0.0),
    LPCLM(0.0),
    LPDIBLC1(0.0),
    LPDIBLC2(0.0),
    LPDIBLCB(0.0),
    LDROUT(0.0),
    LPVAG(0.0),
    LDELTA(0.0),
    LALPHA0(0.0),
    LFBJTII(0.0),
    LABJTII(0.0),
    LCBJTII(0.0),
    LEBJTII(0.0),
    LMBJTII(0.0),
    LVBCI(0.0),
    LBETA0(0.0),
    LBETA1(0.0),
    LBETA2(0.0),
    LVDSATII0(0.0),
    LLII(0.0),
    LESATII(0.0),
    LSII0(0.0),
    LSII1(0.0),
    LSII2(0.0),
    LSIID(0.0),
    LAGIDL(0.0),
    LBGIDL(0.0),
    LCGIDL(0.0),
    LRGIDL(0.0),
    LKGIDL(0.0),
    LFGIDL(0.0),
    LAGISL(0.0),
    LBGISL(0.0),
    LCGISL(0.0),
    LRGISL(0.0),
    LKGISL(0.0),
    LFGISL(0.0),
    LNTUN(0.0),
    LNTUND(0.0),
    LNDIODE(0.0),
    LNDIODED(0.0),
    LNRECF0(0.0),
    LNRECF0D(0.0),
    LNRECR0(0.0),
    LNRECR0D(0.0),
    LISBJT(0.0),
    LIDBJT(0.0),
    LISDIF(0.0),
    LIDDIF(0.0),
    LISREC(0.0),
    LIDREC(0.0),
    LISTUN(0.0),
    LIDTUN(0.0),
    LVREC0(0.0),
    LVREC0D(0.0),
    LVTUN0(0.0),
    LVTUN0D(0.0),
    LNBJT(0.0),
    LLBJT0(0.0),
    LVABJT(0.0),
    LAELY(0.0),
    LAHLI(0.0),
    LAHLID(0.0),
    LVSDFB(0.0),
    LVSDTH(0.0),
    LDELVT(0.0),
    LACDE(0.0),
    LMOIN(0.0),
    LNOFF(0.0),
    LNOFF2(0.0),
    LXRCRG1(0.0),
    LXRCRG2(0.0),
    LVBSA(0.0),
    LVSCE(0.0),
    LCDSBS(0.0),
    LNOFFFD(0.0),
    LVOFFFD(0.0),
    LK1B(0.0),
    LK2B(0.0),
    LDK2B(0.0),
    LDVBD0(0.0),
    LDVBD1(0.0),
    LMOINFD(0.0),
    LVBS0PD(0.0),
    LVBS0FD(0.0),
    WXJ(0.0),
    WALPHAGB1(0.0),
    WBETAGB1(0.0),
    WALPHAGB2(0.0),
    WBETAGB2(0.0),
    WAIGBCP2(0.0),
    WBIGBCP2(0.0),
    WCIGBCP2(0.0),
    WCGSL(0.0),
    WCGDL(0.0),
    WCKAPPA(0.0),
    WNDIF(0.0),
    WUTE(0.0),
    WKT1(0.0),
    WKT1L(0.0),
    WKT2(0.0),
    WUA1(0.0),
    WUB1(0.0),
    WUC1(0.0),
    WAT(0.0),
    WPRT(0.0),
    WNTRECF(0.0),
    WNTRECR(0.0),
    WXBJT(0.0),
    WXDIF(0.0),
    WXREC(0.0),
    WXTUN(0.0),
    WXDIFD(0.0),
    WXRECD(0.0),
    WXTUND(0.0),
    WAIGC(0.0),
    WBIGC(0.0),
    WCIGC(0.0),
    WAIGSD(0.0),
    WBIGSD(0.0),
    WCIGSD(0.0),
    WNIGC(0.0),
    WPIGCD(0.0),
    WPOXEDGE(0.0),
    WNCH(0.0),
    WNSUB(0.0),
    WNGATE(0.0),
    WNSD(0.0),
    WVTH0(0.0),
    WVFB(0.0),
    WK1(0.0),
    WK1W1(0.0),
    WK1W2(0.0),
    WK2(0.0),
    WK3(0.0),
    WK3B(0.0),
    WKB1(0.0),
    WW0(0.0),
    WLPEB(0.0),
    WDVT0(0.0),
    WDVT1(0.0),
    WDVT2(0.0),
    WDVT0W(0.0),
    WDVT1W(0.0),
    WDVT2W(0.0),
    WU0(0.0),
    WEU(0.0),
    WUA(0.0),
    WUB(0.0),
    WUC(0.0),
    WUD(0.0),
    WUD1(0.0),
    WUCSTE(0.0),
    WUCS(0.0),
    WVSAT(0.0),
    WA0(0.0),
    WAGS(0.0),
    WB0(0.0),
    WB1(0.0),
    WKETA(0.0),
    WKETAS(0.0),
    WA1(0.0),
    WA2(0.0),
    WRDSW(0.0),
    WRSW(0.0),
    WRDW(0.0),
    WPRWB(0.0),
    WPRWG(0.0),
    WWR(0.0),
    WNFACTOR(0.0),
    WDWG(0.0),
    WDWB(0.0),
    WVOFF(0.0),
    WETA0(0.0),
    WETAB(0.0),
    WETA0CV(0.0),
    WETABCV(0.0),
    WDSUB(0.0),
    WCIT(0.0),
    WCDSC(0.0),
    WCDSCB(0.0),
    WCDSCD(0.0),
    WPCLM(0.0),
    WPDIBLC1(0.0),
    WPDIBLC2(0.0),
    WPDIBLCB(0.0),
    WDROUT(0.0),
    WPVAG(0.0),
    WDELTA(0.0),
    WALPHA0(0.0),
    WFBJTII(0.0),
    WABJTII(0.0),
    WCBJTII(0.0),
    WEBJTII(0.0),
    WMBJTII(0.0),
    WVBCI(0.0),
    WBETA0(0.0),
    WBETA1(0.0),
    WBETA2(0.0),
    WVDSATII0(0.0),
    WLII(0.0),
    WESATII(0.0),
    WSII0(0.0),
    WSII1(0.0),
    WSII2(0.0),
    WSIID(0.0),
    WAGIDL(0.0),
    WBGIDL(0.0),
    WCGIDL(0.0),
    WRGIDL(0.0),
    WKGIDL(0.0),
    WFGIDL(0.0),
    WAGISL(0.0),
    WBGISL(0.0),
    WCGISL(0.0),
    WRGISL(0.0),
    WKGISL(0.0),
    WFGISL(0.0),
    WNTUN(0.0),
    WNTUND(0.0),
    WNDIODE(0.0),
    WNDIODED(0.0),
    WNRECF0(0.0),
    WNRECF0D(0.0),
    WNRECR0(0.0),
    WNRECR0D(0.0),
    WISBJT(0.0),
    WIDBJT(0.0),
    WISDIF(0.0),
    WIDDIF(0.0),
    WISREC(0.0),
    WIDREC(0.0),
    WISTUN(0.0),
    WIDTUN(0.0),
    WVREC0(0.0),
    WVREC0D(0.0),
    WVTUN0(0.0),
    WVTUN0D(0.0),
    WNBJT(0.0),
    WLBJT0(0.0),
    WVABJT(0.0),
    WAELY(0.0),
    WAHLI(0.0),
    WAHLID(0.0),
    WVSDFB(0.0),
    WVSDTH(0.0),
    WDELVT(0.0),
    WACDE(0.0),
    WMOIN(0.0),
    WNOFF(0.0),
    WNOFF2(0.0),
    WXRCRG1(0.0),
    WXRCRG2(0.0),
    WVBSA(0.0),
    WVSCE(0.0),
    WCDSBS(0.0),
    WNOFFFD(0.0),
    WVOFFFD(0.0),
    WK1B(0.0),
    WK2B(0.0),
    WDK2B(0.0),
    WDVBD0(0.0),
    WDVBD1(0.0),
    WMOINFD(0.0),
    WVBS0PD(0.0),
    WVBS0FD(0.0),
    PXJ(0.0),
    PALPHAGB1(0.0),
    PBETAGB1(0.0),
    PALPHAGB2(0.0),
    PBETAGB2(0.0),
    PAIGBCP2(0.0),
    PBIGBCP2(0.0),
    PCIGBCP2(0.0),
    PCGSL(0.0),
    PCGDL(0.0),
    PCKAPPA(0.0),
    PNDIF(0.0),
    PUTE(0.0),
    PKT1(0.0),
    PKT1L(0.0),
    PKT2(0.0),
    PUA1(0.0),
    PUB1(0.0),
    PUC1(0.0),
    PAT(0.0),
    PPRT(0.0),
    PNTRECF(0.0),
    PNTRECR(0.0),
    PXBJT(0.0),
    PXDIF(0.0),
    PXREC(0.0),
    PXTUN(0.0),
    PXDIFD(0.0),
    PXRECD(0.0),
    PXTUND(0.0),
    PAIGC(0.0),
    PBIGC(0.0),
    PCIGC(0.0),
    PAIGSD(0.0),
    PBIGSD(0.0),
    PCIGSD(0.0),
    PNIGC(0.0),
    PPIGCD(0.0),
    PPOXEDGE(0.0),
    PNCH(0.0),
    PNSUB(0.0),
    PNSD(0.0),
    PNGATE(0.0),
    PVTH0(0.0),
    PVFB(0.0),
    PK1(0.0),
    PK1W1(0.0),
    PK1W2(0.0),
    PK2(0.0),
    PK3(0.0),
    PK3B(0.0),
    PKB1(0.0),
    PW0(0.0),
    PLPEB(0.0),
    PDVT0(0.0),
    PDVT1(0.0),
    PDVT2(0.0),
    PDVT0W(0.0),
    PDVT1W(0.0),
    PDVT2W(0.0),
    PU0(0.0),
    PEU(0.0),
    PUA(0.0),
    PUB(0.0),
    PUC(0.0),
    PUD(0.0),
    PUD1(0.0),
    PUCSTE(0.0),
    PUCS(0.0),
    PVSAT(0.0),
    PA0(0.0),
    PAGS(0.0),
    PB0(0.0),
    PB1(0.0),
    PKETA(0.0),
    PKETAS(0.0),
    PA1(0.0),
    PA2(0.0),
    PRDSW(0.0),
    PRSW(0.0),
    PRDW(0.0),
    PPRWB(0.0),
    PPRWG(0.0),
    PWR(0.0),
    PNFACTOR(0.0),
    PDWG(0.0),
    PDWB(0.0),
    PVOFF(0.0),
    PETA0(0.0),
    PETAB(0.0),
    PETA0CV(0.0),
    PETABCV(0.0),
    PDSUB(0.0),
    PCIT(0.0),
    PCDSC(0.0),
    PCDSCB(0.0),
    PCDSCD(0.0),
    PPCLM(0.0),
    PPDIBLC1(0.0),
    PPDIBLC2(0.0),
    PPDIBLCB(0.0),
    PDROUT(0.0),
    PPVAG(0.0),
    PDELTA(0.0),
    PALPHA0(0.0),
    PFBJTII(0.0),
    PABJTII(0.0),
    PCBJTII(0.0),
    PEBJTII(0.0),
    PMBJTII(0.0),
    PVBCI(0.0),
    PBETA0(0.0),
    PBETA1(0.0),
    PBETA2(0.0),
    PVDSATII0(0.0),
    PLII(0.0),
    PESATII(0.0),
    PSII0(0.0),
    PSII1(0.0),
    PSII2(0.0),
    PSIID(0.0),
    PAGIDL(0.0),
    PBGIDL(0.0),
    PCGIDL(0.0),
    PRGIDL(0.0),
    PKGIDL(0.0),
    PFGIDL(0.0),
    PAGISL(0.0),
    PBGISL(0.0),
    PCGISL(0.0),
    PRGISL(0.0),
    PKGISL(0.0),
    PFGISL(0.0),
    PNTUN(0.0),
    PNTUND(0.0),
    PNDIODE(0.0),
    PNDIODED(0.0),
    PNRECF0(0.0),
    PNRECF0D(0.0),
    PNRECR0(0.0),
    PNRECR0D(0.0),
    PISBJT(0.0),
    PIDBJT(0.0),
    PISDIF(0.0),
    PIDDIF(0.0),
    PISREC(0.0),
    PIDREC(0.0),
    PISTUN(0.0),
    PIDTUN(0.0),
    PVREC0(0.0),
    PVREC0D(0.0),
    PVTUN0(0.0),
    PVTUN0D(0.0),
    PNBJT(0.0),
    PLBJT0(0.0),
    PVABJT(0.0),
    PAELY(0.0),
    PAHLI(0.0),
    PAHLID(0.0),
    PVSDFB(0.0),
    PVSDTH(0.0),
    PDELVT(0.0),
    PACDE(0.0),
    PMOIN(0.0),
    PNOFF(0.0),
    PNOFF2(0.0),
    PXRCRG1(0.0),
    PXRCRG2(0.0),
    PVBSA(0.0),
    PVSCE(0.0),
    PCDSBS(0.0),
    PNOFFFD(0.0),
    PVOFFFD(0.0),
    PK1B(0.0),
    PK2B(0.0),
    PDK2B(0.0),
    PDVBD0(0.0),
    PDVBD1(0.0),
    PMOINFD(0.0),
    PVBS0PD(0.0),
    PVBS0FD(0.0),
    NLX(1.74e-7),
    LNLX(0.0),
    WNLX(0.0),
    PNLX(0.0),
    NGIDL(1.2),
    LNGIDL(0.0),
    WNGIDL(0.0),
    PNGIDL(0.0),
    LPE0(0.0),
    EGIDL(0.0),
    EGISL(0.0),
    LLPE0(0.0),
    LEGIDL(0.0),
    LEGISL(0.0),
    WLPE0(0.0),
    WEGIDL(0.0),
    WEGISL(0.0),
    PLPE0(0.0),
    PEGIDL(0.0),
    PEGISL(0.0),
    EGGBCP2(1.12),
    EGGDEP(1.12),
    AGB1(3.7622e-7),
    BGB1((-3.1051e10)),
    AGB2(4.9758e-7),
    BGB2((-2.357e10)),
    AGBC2N(3.4254e-7),
    AGBC2P(4.9723e-7),
    BGBC2N(1.1665e12),
    BGBC2P(7.4567e11),
    VTM00(0.026)
{
  // Set params to constant default values (from parTable):
  setDefaultParams();

  // Set params according to .model line and constant defaults from metadata:
  setModParams(model_block.params);

  if (!given("XYCEADMSMODTEMP"))
    admsModTemp=getDeviceOptions().temp.getImmutableValue<double>();

  // Calculate any parameters specified as expressions:

  updateDependentParameters();

  // calculate dependent (ie computed) params and check for errors:
  processParams();
}

//-----------------------------------------------------------------------------
// Function      : Model::~Model
// Purpose       : destructor
// Special Notes :
// Scope         : public
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
Model::~Model()
{
  std::vector<Instance*>::iterator iterI;
  std::vector<Instance*>::iterator firstI = instanceContainer.begin ();
  std::vector<Instance*>::iterator lastI  = instanceContainer.end ();

  // loop over instances:
  for (iterI = firstI; iterI != lastI; ++iterI)
  {
    delete (*iterI);
  }
}

//-----------------------------------------------------------------------------
// Function      : Model::printOutInstances
// Purpose       : debugging tool.
// Special Notes :
// Scope         : public
// Creator       :
// Creation Date :
//-----------------------------------------------------------------------------
std::ostream &Model::printOutInstances(std::ostream &os) const
{
  std::vector<Instance*>::const_iterator iter;
  std::vector<Instance*>::const_iterator first = instanceContainer.begin();
  std::vector<Instance*>::const_iterator last  = instanceContainer.end();

  int i;
  os << std::endl;
  os << "    name     model name  Parameters" << std::endl;
  for (i=0, iter=first; iter!=last; ++iter, ++i)
  {
    os << "  " << i << ": " << (*iter)->getName() << "      ";
    os << getName();

    os << std::endl;
      os << "DTEMP  =  " << (*iter)->DTEMP << std::endl;
      os << "L  =  " << (*iter)->L << std::endl;
      os << "W  =  " << (*iter)->W << std::endl;
      os << "NF  =  " << (*iter)->NF << std::endl;
      os << "SA  =  " << (*iter)->SA << std::endl;
      os << "SB  =  " << (*iter)->SB << std::endl;
      os << "SD  =  " << (*iter)->SD << std::endl;
      os << "AD  =  " << (*iter)->AD << std::endl;
      os << "AS  =  " << (*iter)->AS << std::endl;
      os << "PD  =  " << (*iter)->PD << std::endl;
      os << "PS  =  " << (*iter)->PS << std::endl;
      os << "NRD  =  " << (*iter)->NRD << std::endl;
      os << "NRS  =  " << (*iter)->NRS << std::endl;
      os << "OFF  =  " << (*iter)->OFF << std::endl;
      os << "BJTOFF  =  " << (*iter)->BJTOFF << std::endl;
      os << "DEBUG  =  " << (*iter)->DEBUG << std::endl;
      os << "RTH0  =  " << (*iter)->RTH0 << std::endl;
      os << "CTH0  =  " << (*iter)->CTH0 << std::endl;
      os << "NRB  =  " << (*iter)->NRB << std::endl;
      os << "FRBODY  =  " << (*iter)->FRBODY << std::endl;
      os << "RBDB  =  " << (*iter)->RBDB << std::endl;
      os << "RBSB  =  " << (*iter)->RBSB << std::endl;
      os << "DELVTO  =  " << (*iter)->DELVTO << std::endl;
      os << "SOIMOD  =  " << (*iter)->SOIMOD << std::endl;
      os << "NBC  =  " << (*iter)->NBC << std::endl;
      os << "NSEG  =  " << (*iter)->NSEG << std::endl;
      os << "PDBCP  =  " << (*iter)->PDBCP << std::endl;
      os << "PSBCP  =  " << (*iter)->PSBCP << std::endl;
      os << "AGBCP  =  " << (*iter)->AGBCP << std::endl;
      os << "AGBCP2  =  " << (*iter)->AGBCP2 << std::endl;
      os << "AGBCPD  =  " << (*iter)->AGBCPD << std::endl;
      os << "AEBCP  =  " << (*iter)->AEBCP << std::endl;
      os << "TNODEOUT  =  " << (*iter)->TNODEOUT << std::endl;
      os << "SHMOD  =  " << (*iter)->SHMOD << std::endl;
      os << "M  =  " << (*iter)->m << std::endl;
    os << std::endl;
  }

  os << std::endl;

  return os;
}

//-----------------------------------------------------------------------------
// Function      : Model::forEachInstance
// Purpose       :
// Special Notes :
// Scope         : public
// Creator       : David Baur
// Creation Date : 2/4/2014
//-----------------------------------------------------------------------------
/// Apply a device instance "op" to all instances associated with this
/// model
///
/// @param[in] op Operator to apply to all instances.
///
///
void Model::forEachInstance(DeviceInstanceOp &op) const
{
  for (std::vector<Instance *>::const_iterator it = instanceContainer.begin(); it != instanceContainer.end(); ++it)
    op(*it);
}

Device *Traits::factory(const Configuration &configuration, const FactoryBlock &factory_block)
{
  return new DeviceMaster<Traits>(configuration, factory_block, factory_block.solverState_, factory_block.deviceOptions_);
}

void
registerDevice(const DeviceCountMap& deviceMap, const std::set<int>& levelSet)
{
if (deviceMap.empty() ||
((deviceMap.find("M") != deviceMap.end() && (levelSet.find(70)!=levelSet.end()))))
{
MOSFET1::registerDevice();
  Config<Traits>::addConfiguration()
    .registerDevice("m", 70)
    .registerModelType("nmos", 70)
    .registerModelType("pmos", 70);
}
}


#ifdef Xyce_ADMS_SENSITIVITIES
//-----------------------------------------------------------------------------
// Function      : evaluateInitialInstance
// Purpose       : Evaluate the statements in the initial_instance block
// Special Notes : specific for sensitivity use
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void evaluateInitialInstance(
instanceSensStruct & instanceStruct,
modelSensStruct & modelStruct,
const int admsNodeID_d,
const int admsNodeID_g,
const int admsNodeID_s,
const int admsNodeID_e,
const int admsNodeID_p,
const int admsNodeID_b,
const int admsNodeID_t,
const int admsNodeID_di,
const int admsNodeID_si,
const int admsNodeID_gi,
const int admsNodeID_gm,
const int admsNodeID_sb,
const int admsNodeID_db,
 const std::vector<bool> & portsConnected_,
 double admsTemperature, double adms_vt_nom, double ADMSgmin_arg, const Instance & theInstance)
{
  double pParam_B4SOIdl=0.0;
double d_pParam_B4SOIdl_dX=0.0;
  double pParam_B4SOIdlc=0.0;
double d_pParam_B4SOIdlc_dX=0.0;
  double pParam_B4SOIdlcig=0.0;
double d_pParam_B4SOIdlcig_dX=0.0;
  double pParam_B4SOIdw=0.0;
double d_pParam_B4SOIdw_dX=0.0;
  double pParam_B4SOIdwc=0.0;
double d_pParam_B4SOIdwc_dX=0.0;
  double pParam_B4SOIcf=0.0;
double d_pParam_B4SOIcf_dX=0.0;
  double pParam_B4SOIcle=0.0;
double d_pParam_B4SOIcle_dX=0.0;
  double pParam_B4SOInsd=0.0;
double d_pParam_B4SOInsd_dX=0.0;
  double pParam_B4SOIvth0=0.0;
double d_pParam_B4SOIvth0_dX=0.0;
  double pParam_B4SOIwr=0.0;
double d_pParam_B4SOIwr_dX=0.0;
  double pParam_B4SOInbjt=0.0;
double d_pParam_B4SOInbjt_dX=0.0;
  double pParam_B4SOIlbjt0=0.0;
double d_pParam_B4SOIlbjt0_dX=0.0;
  double pParam_B4SOIvabjt=0.0;
double d_pParam_B4SOIvabjt_dX=0.0;
  double pParam_B4SOIaely=0.0;
double d_pParam_B4SOIaely_dX=0.0;
  double pParam_B4SOIndif=0.0;
double d_pParam_B4SOIndif_dX=0.0;
  double pParam_B4SOIvbs0pd=0.0;
double d_pParam_B4SOIvbs0pd_dX=0.0;
  double pParam_B4SOIvbs0fd=0.0;
double d_pParam_B4SOIvbs0fd_dX=0.0;
  double pParam_B4SOIdvtp2=0.0;
double d_pParam_B4SOIdvtp2_dX=0.0;
  double pParam_B4SOIdvtp3=0.0;
double d_pParam_B4SOIdvtp3_dX=0.0;
  double pParam_B4SOIminv=0.0;
double d_pParam_B4SOIminv_dX=0.0;
  double pParam_B4SOIminvcv=0.0;
double d_pParam_B4SOIminvcv_dX=0.0;
  double NchMax=0.0;
double d_NchMax_dX=0.0;
  double Qsi=0.0;
double d_Qsi_dX=0.0;
  double Vbs0t=0.0;
double d_Vbs0t_dX=0.0;
  double pParam_B4SOIsqrtXdep0=0.0;
double d_pParam_B4SOIsqrtXdep0_dX=0.0;
  double pParam_B4SOIToxRatio=0.0;
double d_pParam_B4SOIToxRatio_dX=0.0;
  double pParam_B4SOIToxRatioEdge=0.0;
double d_pParam_B4SOIToxRatioEdge_dX=0.0;
  double pParam_B4SOIkvth0=0.0;
double d_pParam_B4SOIkvth0_dX=0.0;
  double pParam_B4SOIku0temp=0.0;
double d_pParam_B4SOIku0temp_dX=0.0;
  double pParam_B4SOIrho_ref=0.0;
double d_pParam_B4SOIrho_ref_dX=0.0;
  double pParam_B4SOIvfbzb=0.0;
double d_pParam_B4SOIvfbzb_dX=0.0;
  double B4SOIcgso=0.0;
double d_B4SOIcgso_dX=0.0;
  double B4SOIcgdo=0.0;
double d_B4SOIcgdo_dX=0.0;
  double B4SOIcf=0.0;
double d_B4SOIcf_dX=0.0;
  double B4SOIVTM00=0.0;
double d_B4SOIVTM00_dX=0.0;
  double B4SOIBGBC2P=0.0;
double d_B4SOIBGBC2P_dX=0.0;
  double B4SOIBGBC2N=0.0;
double d_B4SOIBGBC2N_dX=0.0;
  double B4SOIAGBC2P=0.0;
double d_B4SOIAGBC2P_dX=0.0;
  double B4SOIAGBC2N=0.0;
double d_B4SOIAGBC2N_dX=0.0;
  double B4SOIBGB2=0.0;
double d_B4SOIBGB2_dX=0.0;
  double B4SOIAGB2=0.0;
double d_B4SOIAGB2_dX=0.0;
  double B4SOIBGB1=0.0;
double d_B4SOIBGB1_dX=0.0;
  double B4SOIAGB1=0.0;
double d_B4SOIAGB1_dX=0.0;
  double B4SOIEGGDEP=0.0;
double d_B4SOIEGGDEP_dX=0.0;
  double B4SOIEGGBCP2=0.0;
double d_B4SOIEGGBCP2_dX=0.0;
  double B4SOIvoffcv=0.0;
double d_B4SOIvoffcv_dX=0.0;
  double B4SOIminvcv=0.0;
double d_B4SOIminvcv_dX=0.0;
  double B4SOIcdsbs=0.0;
double d_B4SOIcdsbs_dX=0.0;
  double B4SOIvsce=0.0;
double d_B4SOIvsce_dX=0.0;
  double B4SOIxgl=0.0;
double d_B4SOIxgl_dX=0.0;
  double B4SOIxgw=0.0;
double d_B4SOIxgw_dX=0.0;
  double B4SOIngcon=0.0;
double d_B4SOIngcon_dX=0.0;
  double B4SOIrshg=0.0;
double d_B4SOIrshg_dX=0.0;
  double B4SOIxrcrg2=0.0;
double d_B4SOIxrcrg2_dX=0.0;
  double B4SOIxrcrg1=0.0;
double d_B4SOIxrcrg1_dX=0.0;
  double B4SOIvbs0fd=0.0;
double d_B4SOIvbs0fd_dX=0.0;
  double B4SOIvbs0pd=0.0;
double d_B4SOIvbs0pd_dX=0.0;
  double B4SOImoinFD=0.0;
double d_B4SOImoinFD_dX=0.0;
  double B4SOIdvbd1=0.0;
double d_B4SOIdvbd1_dX=0.0;
  double B4SOIdvbd0=0.0;
double d_B4SOIdvbd0_dX=0.0;
  double B4SOIdk2b=0.0;
double d_B4SOIdk2b_dX=0.0;
  double B4SOIk2b=0.0;
double d_B4SOIk2b_dX=0.0;
  double B4SOIk1b=0.0;
double d_B4SOIk1b_dX=0.0;
  double B4SOIvofffd=0.0;
double d_B4SOIvofffd_dX=0.0;
  double B4SOInofffd=0.0;
double d_B4SOInofffd_dX=0.0;
  double B4SOIvbsa=0.0;
double d_B4SOIvbsa_dX=0.0;
  double B4SOIdlcig=0.0;
double d_B4SOIdlcig_dX=0.0;
  double B4SOIpoxedge=0.0;
double d_B4SOIpoxedge_dX=0.0;
  double B4SOIpigcd=0.0;
double d_B4SOIpigcd_dX=0.0;
  double B4SOInigc=0.0;
double d_B4SOInigc_dX=0.0;
  double B4SOIcigsd=0.0;
double d_B4SOIcigsd_dX=0.0;
  double B4SOIbigsd=0.0;
double d_B4SOIbigsd_dX=0.0;
  double B4SOIaigsd=0.0;
double d_B4SOIaigsd_dX=0.0;
  double B4SOIcigc=0.0;
double d_B4SOIcigc_dX=0.0;
  double B4SOIbigc=0.0;
double d_B4SOIbigc_dX=0.0;
  double B4SOIaigc=0.0;
double d_B4SOIaigc_dX=0.0;
  double B4SOIcigbcp2=0.0;
double d_B4SOIcigbcp2_dX=0.0;
  double B4SOIbigbcp2=0.0;
double d_B4SOIbigbcp2_dX=0.0;
  double B4SOIaigbcp2=0.0;
double d_B4SOIaigbcp2_dX=0.0;
  double B4SOIbetaGB2=0.0;
double d_B4SOIbetaGB2_dX=0.0;
  double B4SOIalphaGB2=0.0;
double d_B4SOIalphaGB2_dX=0.0;
  double B4SOIbetaGB1=0.0;
double d_B4SOIbetaGB1_dX=0.0;
  double B4SOIalphaGB1=0.0;
double d_B4SOIalphaGB1_dX=0.0;
  double B4SOIcfrcoeff=0.0;
double d_B4SOIcfrcoeff_dX=0.0;
  double B4SOIkb1=0.0;
double d_B4SOIkb1_dX=0.0;
  double B4SOIdelvt=0.0;
double d_B4SOIdelvt_dX=0.0;
  double B4SOInoff2=0.0;
double d_B4SOInoff2_dX=0.0;
  double B4SOInoff=0.0;
double d_B4SOInoff_dX=0.0;
  double B4SOIdlcb=0.0;
double d_B4SOIdlcb_dX=0.0;
  double B4SOIntrecr=0.0;
double d_B4SOIntrecr_dX=0.0;
  double B4SOIntrecf=0.0;
double d_B4SOIntrecf_dX=0.0;
  double B4SOIvsdth=0.0;
double d_B4SOIvsdth_dX=0.0;
  double B4SOIvsdfb=0.0;
double d_B4SOIvsdfb_dX=0.0;
  double B4SOIndif=0.0;
double d_B4SOIndif_dX=0.0;
  double B4SOIcgeo=0.0;
double d_B4SOIcgeo_dX=0.0;
  double B4SOIvabjt=0.0;
double d_B4SOIvabjt_dX=0.0;
  double B4SOIldif0=0.0;
double d_B4SOIldif0_dX=0.0;
  double B4SOIlbjt0=0.0;
double d_B4SOIlbjt0_dX=0.0;
  double B4SOIln=0.0;
double d_B4SOIln_dX=0.0;
  double B4SOInrecr0d=0.0;
double d_B4SOInrecr0d_dX=0.0;
  double B4SOInrecr0=0.0;
double d_B4SOInrecr0_dX=0.0;
  double B4SOInrecf0d=0.0;
double d_B4SOInrecf0d_dX=0.0;
  double B4SOInrecf0=0.0;
double d_B4SOInrecf0_dX=0.0;
  double B4SOImbjtii=0.0;
double d_B4SOImbjtii_dX=0.0;
  double B4SOIabjtii=0.0;
double d_B4SOIabjtii_dX=0.0;
  double B4SOIvbci=0.0;
double d_B4SOIvbci_dX=0.0;
  double B4SOIcbjtii=0.0;
double d_B4SOIcbjtii_dX=0.0;
  double B4SOIebjtii=0.0;
double d_B4SOIebjtii_dX=0.0;
  double B4SOIsii0=0.0;
double d_B4SOIsii0_dX=0.0;
  double B4SOIvdsatii0=0.0;
double d_B4SOIvdsatii0_dX=0.0;
  double B4SOIfprout=0.0;
double d_B4SOIfprout_dX=0.0;
  double B4SOIpditsd=0.0;
double d_B4SOIpditsd_dX=0.0;
  double B4SOIpdits=0.0;
double d_B4SOIpdits_dX=0.0;
  double B4SOIminv=0.0;
double d_B4SOIminv_dX=0.0;
  double B4SOIdvtp4=0.0;
double d_B4SOIdvtp4_dX=0.0;
  double B4SOIdvtp3=0.0;
double d_B4SOIdvtp3_dX=0.0;
  double B4SOIdvtp2=0.0;
double d_B4SOIdvtp2_dX=0.0;
  double B4SOIdvtp1=0.0;
double d_B4SOIdvtp1_dX=0.0;
  double B4SOIdvtp0=0.0;
double d_B4SOIdvtp0_dX=0.0;
  double B4SOIgbmin=0.0;
double d_B4SOIgbmin_dX=0.0;
  double B4SOIsteta0cv=0.0;
double d_B4SOIsteta0cv_dX=0.0;
  double B4SOIsteta0=0.0;
double d_B4SOIsteta0_dX=0.0;
  double B4SOIstk2=0.0;
double d_B4SOIstk2_dX=0.0;
  double B4SOIwlodvth=0.0;
double d_B4SOIwlodvth_dX=0.0;
  double B4SOIllodvth=0.0;
double d_B4SOIllodvth_dX=0.0;
  double B4SOIwlodku0=0.0;
double d_B4SOIwlodku0_dX=0.0;
  double B4SOIllodku0=0.0;
double d_B4SOIllodku0_dX=0.0;
  double B4SOIkvth0=0.0;
double d_B4SOIkvth0_dX=0.0;
  double B4SOIwlod=0.0;
double d_B4SOIwlod_dX=0.0;
  double B4SOIalpha0=0.0;
double d_B4SOIalpha0_dX=0.0;
  double B4SOIdlc=0.0;
double d_B4SOIdlc_dX=0.0;
  double B4SOIdwc=0.0;
double d_B4SOIdwc_dX=0.0;
  double B4SOIcle=0.0;
double d_B4SOIcle_dX=0.0;
  double B4SOIclc=0.0;
double d_B4SOIclc_dX=0.0;
  double B4SOIckappa=0.0;
double d_B4SOIckappa_dX=0.0;
  double B4SOIcgdl=0.0;
double d_B4SOIcgdl_dX=0.0;
  double B4SOIcgsl=0.0;
double d_B4SOIcgsl_dX=0.0;
  double B4SOIb1=0.0;
double d_B4SOIb1_dX=0.0;
  double B4SOIb0=0.0;
double d_B4SOIb0_dX=0.0;
  double B4SOIWwlc=0.0;
double d_B4SOIWwlc_dX=0.0;
  double B4SOIWwl=0.0;
double d_B4SOIWwl_dX=0.0;
  double B4SOIWwn=0.0;
double d_B4SOIWwn_dX=0.0;
  double B4SOIWwc=0.0;
double d_B4SOIWwc_dX=0.0;
  double B4SOIWw=0.0;
double d_B4SOIWw_dX=0.0;
  double B4SOIWln=0.0;
double d_B4SOIWln_dX=0.0;
  double B4SOIWlc=0.0;
double d_B4SOIWlc_dX=0.0;
  double B4SOIWl=0.0;
double d_B4SOIWl_dX=0.0;
  double B4SOIdwb=0.0;
double d_B4SOIdwb_dX=0.0;
  double B4SOIdwg=0.0;
double d_B4SOIdwg_dX=0.0;
  double B4SOIWint=0.0;
double d_B4SOIWint_dX=0.0;
  double B4SOIwr=0.0;
double d_B4SOIwr_dX=0.0;
  double B4SOILwlc=0.0;
double d_B4SOILwlc_dX=0.0;
  double B4SOILwl=0.0;
double d_B4SOILwl_dX=0.0;
  double B4SOILwn=0.0;
double d_B4SOILwn_dX=0.0;
  double B4SOILwc=0.0;
double d_B4SOILwc_dX=0.0;
  double B4SOILw=0.0;
double d_B4SOILw_dX=0.0;
  double B4SOILln=0.0;
double d_B4SOILln_dX=0.0;
  double B4SOILlc=0.0;
double d_B4SOILlc_dX=0.0;
  double B4SOILl=0.0;
double d_B4SOILl_dX=0.0;
  double B4SOILint=0.0;
double d_B4SOILint_dX=0.0;
  double B4SOIxtund=0.0;
double d_B4SOIxtund_dX=0.0;
  double B4SOIxrecd=0.0;
double d_B4SOIxrecd_dX=0.0;
  double B4SOIxdifd=0.0;
double d_B4SOIxdifd_dX=0.0;
  double B4SOIxtun=0.0;
double d_B4SOIxtun_dX=0.0;
  double B4SOIxrec=0.0;
double d_B4SOIxrec_dX=0.0;
  double B4SOIxdif=0.0;
double d_B4SOIxdif_dX=0.0;
  double B4SOIxbjt=0.0;
double d_B4SOIxbjt_dX=0.0;
  double B4SOIfgisl=0.0;
double d_B4SOIfgisl_dX=0.0;
  double B4SOIkgisl=0.0;
double d_B4SOIkgisl_dX=0.0;
  double B4SOIrgisl=0.0;
double d_B4SOIrgisl_dX=0.0;
  double B4SOIfgidl=0.0;
double d_B4SOIfgidl_dX=0.0;
  double B4SOIkgidl=0.0;
double d_B4SOIkgidl_dX=0.0;
  double B4SOIrgidl=0.0;
double d_B4SOIrgidl_dX=0.0;
  double B4SOIxj=0.0;
double d_B4SOIxj_dX=0.0;
  double B4SOIetsi=0.0;
double d_B4SOIetsi_dX=0.0;
  double B4SOIpvag=0.0;
double d_B4SOIpvag_dX=0.0;
  double B4SOIpdiblb=0.0;
double d_B4SOIpdiblb_dX=0.0;
  double B4SOIpdibl2=0.0;
double d_B4SOIpdibl2_dX=0.0;
  double B4SOIpdibl1=0.0;
double d_B4SOIpdibl1_dX=0.0;
  double B4SOIpclm=0.0;
double d_B4SOIpclm_dX=0.0;
  double B4SOIetabcv=0.0;
double d_B4SOIetabcv_dX=0.0;
  double B4SOIeta0cv=0.0;
double d_B4SOIeta0cv_dX=0.0;
  double B4SOIetab=0.0;
double d_B4SOIetab_dX=0.0;
  double B4SOIeta0=0.0;
double d_B4SOIeta0_dX=0.0;
  double B4SOIprt=0.0;
double d_B4SOIprt_dX=0.0;
  double B4SOIprwb=0.0;
double d_B4SOIprwb_dX=0.0;
  double B4SOIprwg=0.0;
double d_B4SOIprwg_dX=0.0;
  double B4SOIrdw=0.0;
double d_B4SOIrdw_dX=0.0;
  double B4SOIrsw=0.0;
double d_B4SOIrsw_dX=0.0;
  double B4SOIrdsw=0.0;
double d_B4SOIrdsw_dX=0.0;
  double B4SOIsheetResistance=0.0;
double d_B4SOIsheetResistance_dX=0.0;
  double B4SOIdelta=0.0;
double d_B4SOIdelta_dX=0.0;
  double B4SOIvoff=0.0;
double d_B4SOIvoff_dX=0.0;
  double B4SOIud1=0.0;
double d_B4SOIud1_dX=0.0;
  double B4SOIud=0.0;
double d_B4SOIud_dX=0.0;
  double B4SOIucste=0.0;
double d_B4SOIucste_dX=0.0;
  double B4SOIucs=0.0;
double d_B4SOIucs_dX=0.0;
  double B4SOIute=0.0;
double d_B4SOIute_dX=0.0;
  double B4SOIeu=0.0;
double d_B4SOIeu_dX=0.0;
  double B4SOIu0=0.0;
double d_B4SOIu0_dX=0.0;
  double B4SOIuc1=0.0;
double d_B4SOIuc1_dX=0.0;
  double B4SOIuc=0.0;
double d_B4SOIuc_dX=0.0;
  double B4SOIub1=0.0;
double d_B4SOIub1_dX=0.0;
  double B4SOIub=0.0;
double d_B4SOIub_dX=0.0;
  double B4SOIua1=0.0;
double d_B4SOIua1_dX=0.0;
  double B4SOIua=0.0;
double d_B4SOIua_dX=0.0;
  double B4SOIvfb=0.0;
double d_B4SOIvfb_dX=0.0;
  double B4SOIvth0=0.0;
double d_B4SOIvth0_dX=0.0;
  double B4SOIdsub=0.0;
double d_B4SOIdsub_dX=0.0;
  double B4SOIdrout=0.0;
double d_B4SOIdrout_dX=0.0;
  double B4SOIdvt2w=0.0;
double d_B4SOIdvt2w_dX=0.0;
  double B4SOIdvt1w=0.0;
double d_B4SOIdvt1w_dX=0.0;
  double B4SOIdvt0w=0.0;
double d_B4SOIdvt0w_dX=0.0;
  double B4SOIdvt2=0.0;
double d_B4SOIdvt2_dX=0.0;
  double B4SOIdvt1=0.0;
double d_B4SOIdvt1_dX=0.0;
  double B4SOIdvt0=0.0;
double d_B4SOIdvt0_dX=0.0;
  double B4SOIlpeb=0.0;
double d_B4SOIlpeb_dX=0.0;
  double B4SOIlpe0=0.0;
double d_B4SOIlpe0_dX=0.0;
  double B4SOIw0=0.0;
double d_B4SOIw0_dX=0.0;
  double B4SOIk3b=0.0;
double d_B4SOIk3b_dX=0.0;
  double B4SOIk3=0.0;
double d_B4SOIk3_dX=0.0;
  double B4SOIk2=0.0;
double d_B4SOIk2_dX=0.0;
  double B4SOIkt2=0.0;
double d_B4SOIkt2_dX=0.0;
  double B4SOIkt1l=0.0;
double d_B4SOIkt1l_dX=0.0;
  double B4SOIkt1=0.0;
double d_B4SOIkt1_dX=0.0;
  double B4SOIk1=0.0;
double d_B4SOIk1_dX=0.0;
  double B4SOIxt=0.0;
double d_B4SOIxt_dX=0.0;
  double B4SOIvbm=0.0;
double d_B4SOIvbm_dX=0.0;
  double B4SOIvbx=0.0;
double d_B4SOIvbx_dX=0.0;
  double B4SOIgamma2=0.0;
double d_B4SOIgamma2_dX=0.0;
  double B4SOIgamma1=0.0;
double d_B4SOIgamma1_dX=0.0;
  double B4SOInpeak=0.0;
double d_B4SOInpeak_dX=0.0;
  double B4SOInsub=0.0;
double d_B4SOInsub_dX=0.0;
  double B4SOIketa=0.0;
double d_B4SOIketa_dX=0.0;
  double B4SOIa2=0.0;
double d_B4SOIa2_dX=0.0;
  double B4SOIa1=0.0;
double d_B4SOIa1_dX=0.0;
  double B4SOIags=0.0;
double d_B4SOIags_dX=0.0;
  double B4SOIa0=0.0;
double d_B4SOIa0_dX=0.0;
  double B4SOIat=0.0;
double d_B4SOIat_dX=0.0;
  double B4SOIvsat=0.0;
double d_B4SOIvsat_dX=0.0;
  double B4SOInfactor=0.0;
double d_B4SOInfactor_dX=0.0;
  double B4SOIcit=0.0;
double d_B4SOIcit_dX=0.0;
  double B4SOIcdscd=0.0;
double d_B4SOIcdscd_dX=0.0;
  double B4SOIcdscb=0.0;
double d_B4SOIcdscb_dX=0.0;
  double B4SOIcdsc=0.0;
double d_B4SOIcdsc_dX=0.0;
  int B4SOIbinUnit=0.0;
  double B4SOItempeot=0.0;
double d_B4SOItempeot_dX=0.0;
  double B4SOIvddeot=0.0;
double d_B4SOIvddeot_dX=0.0;
  double B4SOIepsrox=0.0;
double d_B4SOIepsrox_dX=0.0;
  double B4SOIpsbcp=0.0;
double d_B4SOIpsbcp_dX=0.0;
  double B4SOIpdbcp=0.0;
double d_B4SOIpdbcp_dX=0.0;
  double B4SOIdelvto=0.0;
double d_B4SOIdelvto_dX=0.0;
  double B4SOIrbsb=0.0;
double d_B4SOIrbsb_dX=0.0;
  double B4SOIrbdb=0.0;
double d_B4SOIrbdb_dX=0.0;
  double B4SOIfrbody=0.0;
double d_B4SOIfrbody_dX=0.0;
  double B4SOIbodySquares=0.0;
double d_B4SOIbodySquares_dX=0.0;
  double B4SOIsourceSquares=0.0;
double d_B4SOIsourceSquares_dX=0.0;
  double B4SOIdrainSquares=0.0;
double d_B4SOIdrainSquares_dX=0.0;
  double B4SOIsourcePerimeter=0.0;
double d_B4SOIsourcePerimeter_dX=0.0;
  double B4SOIdrainPerimeter=0.0;
double d_B4SOIdrainPerimeter_dX=0.0;
  double B4SOIsourceArea=0.0;
double d_B4SOIsourceArea_dX=0.0;
  double B4SOIdrainArea=0.0;
double d_B4SOIdrainArea_dX=0.0;
  double B4SOIw=0.0;
double d_B4SOIw_dX=0.0;
  double B4SOIl=0.0;
double d_B4SOIl_dX=0.0;
//Begin block initial_instance
{
instanceStruct.d_instanceVar_DevTemp_dX = instanceStruct.d_instancePar_DTEMP_dX;
instanceStruct.instanceVar_DevTemp = (admsTemperature+instanceStruct.instancePar_DTEMP);
instanceStruct.instanceVar_B4SOItype = modelStruct.modelPar_TYPE;
d_B4SOIl_dX = instanceStruct.d_instancePar_L_dX;
B4SOIl = instanceStruct.instancePar_L;
d_B4SOIw_dX = instanceStruct.d_instancePar_W_dX;
B4SOIw = instanceStruct.instancePar_W;
instanceStruct.d_instanceVar_B4SOInf_dX = 0.0;
instanceStruct.instanceVar_B4SOInf = instanceStruct.instancePar_NF;
instanceStruct.d_instanceVar_B4SOIsa_dX = instanceStruct.d_instancePar_SA_dX;
instanceStruct.instanceVar_B4SOIsa = instanceStruct.instancePar_SA;
instanceStruct.d_instanceVar_B4SOIsb_dX = instanceStruct.d_instancePar_SB_dX;
instanceStruct.instanceVar_B4SOIsb = instanceStruct.instancePar_SB;
instanceStruct.d_instanceVar_B4SOIsd_dX = instanceStruct.d_instancePar_SD_dX;
instanceStruct.instanceVar_B4SOIsd = instanceStruct.instancePar_SD;
d_B4SOIdrainArea_dX = instanceStruct.d_instancePar_AD_dX;
B4SOIdrainArea = instanceStruct.instancePar_AD;
d_B4SOIsourceArea_dX = instanceStruct.d_instancePar_AS_dX;
B4SOIsourceArea = instanceStruct.instancePar_AS;
d_B4SOIdrainPerimeter_dX = instanceStruct.d_instancePar_PD_dX;
B4SOIdrainPerimeter = instanceStruct.instancePar_PD;
d_B4SOIsourcePerimeter_dX = instanceStruct.d_instancePar_PS_dX;
B4SOIsourcePerimeter = instanceStruct.instancePar_PS;
d_B4SOIdrainSquares_dX = instanceStruct.d_instancePar_NRD_dX;
B4SOIdrainSquares = instanceStruct.instancePar_NRD;
d_B4SOIsourceSquares_dX = instanceStruct.d_instancePar_NRS_dX;
B4SOIsourceSquares = instanceStruct.instancePar_NRS;
instanceStruct.instanceVar_B4SOIbjtoff = instanceStruct.instancePar_BJTOFF;
instanceStruct.d_instanceVar_B4SOIrth0_dX = instanceStruct.d_instancePar_RTH0_dX;
instanceStruct.instanceVar_B4SOIrth0 = instanceStruct.instancePar_RTH0;
instanceStruct.d_instanceVar_B4SOIcth0_dX = instanceStruct.d_instancePar_CTH0_dX;
instanceStruct.instanceVar_B4SOIcth0 = instanceStruct.instancePar_CTH0;
d_B4SOIbodySquares_dX = instanceStruct.d_instancePar_NRB_dX;
B4SOIbodySquares = instanceStruct.instancePar_NRB;
d_B4SOIfrbody_dX = instanceStruct.d_instancePar_FRBODY_dX;
B4SOIfrbody = instanceStruct.instancePar_FRBODY;
d_B4SOIrbdb_dX = instanceStruct.d_instancePar_RBDB_dX;
B4SOIrbdb = instanceStruct.instancePar_RBDB;
d_B4SOIrbsb_dX = instanceStruct.d_instancePar_RBSB_dX;
B4SOIrbsb = instanceStruct.instancePar_RBSB;
d_B4SOIdelvto_dX = instanceStruct.d_instancePar_DELVTO_dX;
B4SOIdelvto = instanceStruct.instancePar_DELVTO;
instanceStruct.instanceVar_B4SOIsoiMod = instanceStruct.instancePar_SOIMOD;
instanceStruct.d_instanceVar_B4SOInbc_dX = instanceStruct.d_instancePar_NBC_dX;
instanceStruct.instanceVar_B4SOInbc = instanceStruct.instancePar_NBC;
instanceStruct.d_instanceVar_B4SOInseg_dX = instanceStruct.d_instancePar_NSEG_dX;
instanceStruct.instanceVar_B4SOInseg = instanceStruct.instancePar_NSEG;
d_B4SOIpdbcp_dX = instanceStruct.d_instancePar_PDBCP_dX;
B4SOIpdbcp = instanceStruct.instancePar_PDBCP;
d_B4SOIpsbcp_dX = instanceStruct.d_instancePar_PSBCP_dX;
B4SOIpsbcp = instanceStruct.instancePar_PSBCP;
instanceStruct.d_instanceVar_B4SOIagbcp_dX = instanceStruct.d_instancePar_AGBCP_dX;
instanceStruct.instanceVar_B4SOIagbcp = instanceStruct.instancePar_AGBCP;
instanceStruct.d_instanceVar_B4SOIagbcp2_dX = instanceStruct.d_instancePar_AGBCP2_dX;
instanceStruct.instanceVar_B4SOIagbcp2 = instanceStruct.instancePar_AGBCP2;
instanceStruct.d_instanceVar_B4SOIagbcpd_dX = instanceStruct.d_instancePar_AGBCPD_dX;
instanceStruct.instanceVar_B4SOIagbcpd = instanceStruct.instancePar_AGBCPD;
instanceStruct.d_instanceVar_B4SOIaebcp_dX = instanceStruct.d_instancePar_AEBCP_dX;
instanceStruct.instanceVar_B4SOIaebcp = instanceStruct.instancePar_AEBCP;
instanceStruct.instanceVar_B4SOIrgateMod = modelStruct.modelPar_RGATEMOD;
instanceStruct.instanceVar_B4SOIrbodyMod = modelStruct.modelPar_RBODYMOD;
instanceStruct.instanceVar_B4SOImtrlMod = modelStruct.modelPar_MTRLMOD;
instanceStruct.instanceVar_B4SOIvgstcvMod = modelStruct.modelPar_VGSTCVMOD;
instanceStruct.instanceVar_B4SOIgidlMod = modelStruct.modelPar_GIDLMOD;
instanceStruct.instanceVar_B4SOIiiiMod = modelStruct.modelPar_IIIMOD;
instanceStruct.d_instanceVar_B4SOIeot_dX = modelStruct.d_modelPar_EOT_dX;
instanceStruct.instanceVar_B4SOIeot = modelStruct.modelPar_EOT;
d_B4SOIepsrox_dX = modelStruct.d_modelPar_EPSROX_dX;
B4SOIepsrox = modelStruct.modelPar_EPSROX;
instanceStruct.d_instanceVar_B4SOIepsrsub_dX = modelStruct.d_modelPar_EPSRSUB_dX;
instanceStruct.instanceVar_B4SOIepsrsub = modelStruct.modelPar_EPSRSUB;
instanceStruct.d_instanceVar_B4SOIni0sub_dX = modelStruct.d_modelPar_NI0SUB_dX;
instanceStruct.instanceVar_B4SOIni0sub = modelStruct.modelPar_NI0SUB;
instanceStruct.d_instanceVar_B4SOIbg0sub_dX = modelStruct.d_modelPar_BG0SUB_dX;
instanceStruct.instanceVar_B4SOIbg0sub = modelStruct.modelPar_BG0SUB;
instanceStruct.d_instanceVar_B4SOItbgasub_dX = modelStruct.d_modelPar_TBGASUB_dX;
instanceStruct.instanceVar_B4SOItbgasub = modelStruct.modelPar_TBGASUB;
instanceStruct.d_instanceVar_B4SOItbgbsub_dX = modelStruct.d_modelPar_TBGBSUB_dX;
instanceStruct.instanceVar_B4SOItbgbsub = modelStruct.modelPar_TBGBSUB;
instanceStruct.d_instanceVar_B4SOIphig_dX = modelStruct.d_modelPar_PHIG_dX;
instanceStruct.instanceVar_B4SOIphig = modelStruct.modelPar_PHIG;
instanceStruct.d_instanceVar_B4SOIeasub_dX = modelStruct.d_modelPar_EASUB_dX;
instanceStruct.instanceVar_B4SOIeasub = modelStruct.modelPar_EASUB;
instanceStruct.d_instanceVar_B4SOIleffeot_dX = modelStruct.d_modelPar_LEFFEOT_dX;
instanceStruct.instanceVar_B4SOIleffeot = modelStruct.modelPar_LEFFEOT;
instanceStruct.d_instanceVar_B4SOIweffeot_dX = modelStruct.d_modelPar_WEFFEOT_dX;
instanceStruct.instanceVar_B4SOIweffeot = modelStruct.modelPar_WEFFEOT;
d_B4SOIvddeot_dX = modelStruct.d_modelPar_VDDEOT_dX;
B4SOIvddeot = modelStruct.modelPar_VDDEOT;
d_B4SOItempeot_dX = modelStruct.d_modelPar_TEMPEOT_dX;
B4SOItempeot = modelStruct.modelPar_TEMPEOT;
instanceStruct.d_instanceVar_B4SOIados_dX = modelStruct.d_modelPar_ADOS_dX;
instanceStruct.instanceVar_B4SOIados = modelStruct.modelPar_ADOS;
instanceStruct.d_instanceVar_B4SOIbdos_dX = modelStruct.d_modelPar_BDOS_dX;
instanceStruct.instanceVar_B4SOIbdos = modelStruct.modelPar_BDOS;
instanceStruct.d_instanceVar_B4SOIepsrgate_dX = modelStruct.d_modelPar_EPSRGATE_dX;
instanceStruct.instanceVar_B4SOIepsrgate = modelStruct.modelPar_EPSRGATE;
instanceStruct.instanceVar_B4SOIcapMod = modelStruct.modelPar_CAPMOD;
instanceStruct.instanceVar_B4SOImobMod = modelStruct.modelPar_MOBMOD;
B4SOIbinUnit = modelStruct.modelPar_BINUNIT;
instanceStruct.d_instanceVar_B4SOItox_dX = modelStruct.d_modelPar_TOX_dX;
instanceStruct.instanceVar_B4SOItox = modelStruct.modelPar_TOX;
instanceStruct.d_instanceVar_B4SOItoxm_dX = modelStruct.d_modelPar_TOXM_dX;
instanceStruct.instanceVar_B4SOItoxm = modelStruct.modelPar_TOXM;
instanceStruct.d_instanceVar_B4SOIdtoxcv_dX = modelStruct.d_modelPar_DTOXCV_dX;
instanceStruct.instanceVar_B4SOIdtoxcv = modelStruct.modelPar_DTOXCV;
d_B4SOIcdsc_dX = modelStruct.d_modelPar_CDSC_dX;
B4SOIcdsc = modelStruct.modelPar_CDSC;
d_B4SOIcdscb_dX = modelStruct.d_modelPar_CDSCB_dX;
B4SOIcdscb = modelStruct.modelPar_CDSCB;
d_B4SOIcdscd_dX = modelStruct.d_modelPar_CDSCD_dX;
B4SOIcdscd = modelStruct.modelPar_CDSCD;
d_B4SOIcit_dX = modelStruct.d_modelPar_CIT_dX;
B4SOIcit = modelStruct.modelPar_CIT;
d_B4SOInfactor_dX = modelStruct.d_modelPar_NFACTOR_dX;
B4SOInfactor = modelStruct.modelPar_NFACTOR;
d_B4SOIvsat_dX = modelStruct.d_modelPar_VSAT_dX;
B4SOIvsat = modelStruct.modelPar_VSAT;
d_B4SOIat_dX = modelStruct.d_modelPar_AT_dX;
B4SOIat = modelStruct.modelPar_AT;
d_B4SOIa0_dX = modelStruct.d_modelPar_A0_dX;
B4SOIa0 = modelStruct.modelPar_A0;
d_B4SOIags_dX = modelStruct.d_modelPar_AGS_dX;
B4SOIags = modelStruct.modelPar_AGS;
d_B4SOIa1_dX = modelStruct.d_modelPar_A1_dX;
B4SOIa1 = modelStruct.modelPar_A1;
d_B4SOIa2_dX = modelStruct.d_modelPar_A2_dX;
B4SOIa2 = modelStruct.modelPar_A2;
d_B4SOIketa_dX = modelStruct.d_modelPar_KETA_dX;
B4SOIketa = modelStruct.modelPar_KETA;
d_B4SOInsub_dX = modelStruct.d_modelPar_NSUB_dX;
B4SOInsub = modelStruct.modelPar_NSUB;
d_B4SOInpeak_dX = modelStruct.d_modelPar_NCH_dX;
B4SOInpeak = modelStruct.modelPar_NCH;
instanceStruct.d_instanceVar_B4SOIngate_dX = modelStruct.d_modelPar_NGATE_dX;
instanceStruct.instanceVar_B4SOIngate = modelStruct.modelPar_NGATE;
instanceStruct.d_instanceVar_B4SOInsd_dX = modelStruct.d_modelPar_NSD_dX;
instanceStruct.instanceVar_B4SOInsd = modelStruct.modelPar_NSD;
d_B4SOIgamma1_dX = modelStruct.d_modelPar_GAMMA1_dX;
B4SOIgamma1 = modelStruct.modelPar_GAMMA1;
d_B4SOIgamma2_dX = modelStruct.d_modelPar_GAMMA2_dX;
B4SOIgamma2 = modelStruct.modelPar_GAMMA2;
d_B4SOIvbx_dX = modelStruct.d_modelPar_VBX_dX;
B4SOIvbx = modelStruct.modelPar_VBX;
d_B4SOIvbm_dX = modelStruct.d_modelPar_VBM_dX;
B4SOIvbm = modelStruct.modelPar_VBM;
d_B4SOIxt_dX = modelStruct.d_modelPar_XT_dX;
B4SOIxt = modelStruct.modelPar_XT;
d_B4SOIk1_dX = modelStruct.d_modelPar_K1_dX;
B4SOIk1 = modelStruct.modelPar_K1;
d_B4SOIkt1_dX = modelStruct.d_modelPar_KT1_dX;
B4SOIkt1 = modelStruct.modelPar_KT1;
d_B4SOIkt1l_dX = modelStruct.d_modelPar_KT1L_dX;
B4SOIkt1l = modelStruct.modelPar_KT1L;
d_B4SOIkt2_dX = modelStruct.d_modelPar_KT2_dX;
B4SOIkt2 = modelStruct.modelPar_KT2;
d_B4SOIk2_dX = modelStruct.d_modelPar_K2_dX;
B4SOIk2 = modelStruct.modelPar_K2;
d_B4SOIk3_dX = modelStruct.d_modelPar_K3_dX;
B4SOIk3 = modelStruct.modelPar_K3;
d_B4SOIk3b_dX = modelStruct.d_modelPar_K3B_dX;
B4SOIk3b = modelStruct.modelPar_K3B;
d_B4SOIw0_dX = modelStruct.d_modelPar_W0_dX;
B4SOIw0 = modelStruct.modelPar_W0;
d_B4SOIlpe0_dX = modelStruct.d_modelPar_LPE0_dX;
B4SOIlpe0 = modelStruct.modelPar_LPE0;
d_B4SOIlpeb_dX = modelStruct.d_modelPar_LPEB_dX;
B4SOIlpeb = modelStruct.modelPar_LPEB;
d_B4SOIdvt0_dX = modelStruct.d_modelPar_DVT0_dX;
B4SOIdvt0 = modelStruct.modelPar_DVT0;
d_B4SOIdvt1_dX = modelStruct.d_modelPar_DVT1_dX;
B4SOIdvt1 = modelStruct.modelPar_DVT1;
d_B4SOIdvt2_dX = modelStruct.d_modelPar_DVT2_dX;
B4SOIdvt2 = modelStruct.modelPar_DVT2;
d_B4SOIdvt0w_dX = modelStruct.d_modelPar_DVT0W_dX;
B4SOIdvt0w = modelStruct.modelPar_DVT0W;
d_B4SOIdvt1w_dX = modelStruct.d_modelPar_DVT1W_dX;
B4SOIdvt1w = modelStruct.modelPar_DVT1W;
d_B4SOIdvt2w_dX = modelStruct.d_modelPar_DVT2W_dX;
B4SOIdvt2w = modelStruct.modelPar_DVT2W;
d_B4SOIdrout_dX = modelStruct.d_modelPar_DROUT_dX;
B4SOIdrout = modelStruct.modelPar_DROUT;
d_B4SOIdsub_dX = modelStruct.d_modelPar_DSUB_dX;
B4SOIdsub = modelStruct.modelPar_DSUB;
d_B4SOIvth0_dX = modelStruct.d_modelPar_VTH0_dX;
B4SOIvth0 = modelStruct.modelPar_VTH0;
d_B4SOIvfb_dX = modelStruct.d_modelPar_VFB_dX;
B4SOIvfb = modelStruct.modelPar_VFB;
d_B4SOIua_dX = modelStruct.d_modelPar_UA_dX;
B4SOIua = modelStruct.modelPar_UA;
d_B4SOIua1_dX = modelStruct.d_modelPar_UA1_dX;
B4SOIua1 = modelStruct.modelPar_UA1;
d_B4SOIub_dX = modelStruct.d_modelPar_UB_dX;
B4SOIub = modelStruct.modelPar_UB;
d_B4SOIub1_dX = modelStruct.d_modelPar_UB1_dX;
B4SOIub1 = modelStruct.modelPar_UB1;
d_B4SOIuc_dX = modelStruct.d_modelPar_UC_dX;
B4SOIuc = modelStruct.modelPar_UC;
d_B4SOIuc1_dX = modelStruct.d_modelPar_UC1_dX;
B4SOIuc1 = modelStruct.modelPar_UC1;
d_B4SOIu0_dX = modelStruct.d_modelPar_U0_dX;
B4SOIu0 = modelStruct.modelPar_U0;
d_B4SOIeu_dX = modelStruct.d_modelPar_EU_dX;
B4SOIeu = modelStruct.modelPar_EU;
d_B4SOIute_dX = modelStruct.d_modelPar_UTE_dX;
B4SOIute = modelStruct.modelPar_UTE;
d_B4SOIucs_dX = modelStruct.d_modelPar_UCS_dX;
B4SOIucs = modelStruct.modelPar_UCS;
d_B4SOIucste_dX = modelStruct.d_modelPar_UCSTE_dX;
B4SOIucste = modelStruct.modelPar_UCSTE;
d_B4SOIud_dX = modelStruct.d_modelPar_UD_dX;
B4SOIud = modelStruct.modelPar_UD;
d_B4SOIud1_dX = modelStruct.d_modelPar_UD1_dX;
B4SOIud1 = modelStruct.modelPar_UD1;
d_B4SOIvoff_dX = modelStruct.d_modelPar_VOFF_dX;
B4SOIvoff = modelStruct.modelPar_VOFF;
instanceStruct.d_instanceVar_B4SOItnom_dX = modelStruct.d_modelPar_TNOM_dX;
instanceStruct.instanceVar_B4SOItnom = (modelStruct.modelPar_TNOM+273.15);
instanceStruct.d_instanceVar_B4SOIxpart_dX = modelStruct.d_modelPar_XPART_dX;
instanceStruct.instanceVar_B4SOIxpart = modelStruct.modelPar_XPART;
d_B4SOIdelta_dX = modelStruct.d_modelPar_DELTA_dX;
B4SOIdelta = modelStruct.modelPar_DELTA;
d_B4SOIsheetResistance_dX = modelStruct.d_modelPar_RSH_dX;
B4SOIsheetResistance = modelStruct.modelPar_RSH;
d_B4SOIrdsw_dX = modelStruct.d_modelPar_RDSW_dX;
B4SOIrdsw = modelStruct.modelPar_RDSW;
d_B4SOIrsw_dX = modelStruct.d_modelPar_RSW_dX;
B4SOIrsw = modelStruct.modelPar_RSW;
d_B4SOIrdw_dX = modelStruct.d_modelPar_RDW_dX;
B4SOIrdw = modelStruct.modelPar_RDW;
instanceStruct.d_instanceVar_B4SOIrswmin_dX = modelStruct.d_modelPar_RSWMIN_dX;
instanceStruct.instanceVar_B4SOIrswmin = modelStruct.modelPar_RSWMIN;
instanceStruct.d_instanceVar_B4SOIrdwmin_dX = modelStruct.d_modelPar_RDWMIN_dX;
instanceStruct.instanceVar_B4SOIrdwmin = modelStruct.modelPar_RDWMIN;
d_B4SOIprwg_dX = modelStruct.d_modelPar_PRWG_dX;
B4SOIprwg = modelStruct.modelPar_PRWG;
d_B4SOIprwb_dX = modelStruct.d_modelPar_PRWB_dX;
B4SOIprwb = modelStruct.modelPar_PRWB;
d_B4SOIprt_dX = modelStruct.d_modelPar_PRT_dX;
B4SOIprt = modelStruct.modelPar_PRT;
d_B4SOIeta0_dX = modelStruct.d_modelPar_ETA0_dX;
B4SOIeta0 = modelStruct.modelPar_ETA0;
d_B4SOIetab_dX = modelStruct.d_modelPar_ETAB_dX;
B4SOIetab = modelStruct.modelPar_ETAB;
d_B4SOIeta0cv_dX = modelStruct.d_modelPar_ETA0CV_dX;
B4SOIeta0cv = modelStruct.modelPar_ETA0CV;
d_B4SOIetabcv_dX = modelStruct.d_modelPar_ETABCV_dX;
B4SOIetabcv = modelStruct.modelPar_ETABCV;
d_B4SOIpclm_dX = modelStruct.d_modelPar_PCLM_dX;
B4SOIpclm = modelStruct.modelPar_PCLM;
d_B4SOIpdibl1_dX = modelStruct.d_modelPar_PDIBLC1_dX;
B4SOIpdibl1 = modelStruct.modelPar_PDIBLC1;
d_B4SOIpdibl2_dX = modelStruct.d_modelPar_PDIBLC2_dX;
B4SOIpdibl2 = modelStruct.modelPar_PDIBLC2;
d_B4SOIpdiblb_dX = modelStruct.d_modelPar_PDIBLCB_dX;
B4SOIpdiblb = modelStruct.modelPar_PDIBLCB;
d_B4SOIpvag_dX = modelStruct.d_modelPar_PVAG_dX;
B4SOIpvag = modelStruct.modelPar_PVAG;
instanceStruct.d_instanceVar_B4SOItbox_dX = modelStruct.d_modelPar_TBOX_dX;
instanceStruct.instanceVar_B4SOItbox = modelStruct.modelPar_TBOX;
instanceStruct.d_instanceVar_B4SOItsi_dX = modelStruct.d_modelPar_TSI_dX;
instanceStruct.instanceVar_B4SOItsi = modelStruct.modelPar_TSI;
d_B4SOIetsi_dX = modelStruct.d_modelPar_ETSI_dX;
B4SOIetsi = modelStruct.modelPar_ETSI;
d_B4SOIxj_dX = modelStruct.d_modelPar_XJ_dX;
B4SOIxj = modelStruct.modelPar_XJ;
instanceStruct.d_instanceVar_B4SOIegidl_dX = modelStruct.d_modelPar_EGIDL_dX;
instanceStruct.instanceVar_B4SOIegidl = modelStruct.modelPar_EGIDL;
instanceStruct.d_instanceVar_B4SOIagidl_dX = modelStruct.d_modelPar_AGIDL_dX;
instanceStruct.instanceVar_B4SOIagidl = modelStruct.modelPar_AGIDL;
instanceStruct.d_instanceVar_B4SOIbgidl_dX = modelStruct.d_modelPar_BGIDL_dX;
instanceStruct.instanceVar_B4SOIbgidl = modelStruct.modelPar_BGIDL;
instanceStruct.d_instanceVar_B4SOIcgidl_dX = modelStruct.d_modelPar_CGIDL_dX;
instanceStruct.instanceVar_B4SOIcgidl = modelStruct.modelPar_CGIDL;
d_B4SOIrgidl_dX = modelStruct.d_modelPar_RGIDL_dX;
B4SOIrgidl = modelStruct.modelPar_RGIDL;
d_B4SOIkgidl_dX = modelStruct.d_modelPar_KGIDL_dX;
B4SOIkgidl = modelStruct.modelPar_KGIDL;
d_B4SOIfgidl_dX = modelStruct.d_modelPar_FGIDL_dX;
B4SOIfgidl = modelStruct.modelPar_FGIDL;
instanceStruct.d_instanceVar_B4SOIegisl_dX = modelStruct.d_modelPar_EGISL_dX;
instanceStruct.instanceVar_B4SOIegisl = modelStruct.modelPar_EGISL;
instanceStruct.d_instanceVar_B4SOIagisl_dX = modelStruct.d_modelPar_AGISL_dX;
instanceStruct.instanceVar_B4SOIagisl = modelStruct.modelPar_AGISL;
instanceStruct.d_instanceVar_B4SOIbgisl_dX = modelStruct.d_modelPar_BGISL_dX;
instanceStruct.instanceVar_B4SOIbgisl = modelStruct.modelPar_BGISL;
instanceStruct.d_instanceVar_B4SOIcgisl_dX = modelStruct.d_modelPar_CGISL_dX;
instanceStruct.instanceVar_B4SOIcgisl = modelStruct.modelPar_CGISL;
d_B4SOIrgisl_dX = modelStruct.d_modelPar_RGISL_dX;
B4SOIrgisl = modelStruct.modelPar_RGISL;
d_B4SOIkgisl_dX = modelStruct.d_modelPar_KGISL_dX;
B4SOIkgisl = modelStruct.modelPar_KGISL;
d_B4SOIfgisl_dX = modelStruct.d_modelPar_FGISL_dX;
B4SOIfgisl = modelStruct.modelPar_FGISL;
instanceStruct.d_instanceVar_B4SOIndiode_dX = modelStruct.d_modelPar_NDIODE_dX;
instanceStruct.instanceVar_B4SOIndiode = modelStruct.modelPar_NDIODE;
instanceStruct.d_instanceVar_B4SOIndioded_dX = modelStruct.d_modelPar_NDIODED_dX;
instanceStruct.instanceVar_B4SOIndioded = modelStruct.modelPar_NDIODED;
d_B4SOIxbjt_dX = modelStruct.d_modelPar_XBJT_dX;
B4SOIxbjt = modelStruct.modelPar_XBJT;
d_B4SOIxdif_dX = modelStruct.d_modelPar_XDIF_dX;
B4SOIxdif = modelStruct.modelPar_XDIF;
d_B4SOIxrec_dX = modelStruct.d_modelPar_XREC_dX;
B4SOIxrec = modelStruct.modelPar_XREC;
d_B4SOIxtun_dX = modelStruct.d_modelPar_XTUN_dX;
B4SOIxtun = modelStruct.modelPar_XTUN;
d_B4SOIxdifd_dX = modelStruct.d_modelPar_XDIFD_dX;
B4SOIxdifd = modelStruct.modelPar_XDIFD;
d_B4SOIxrecd_dX = modelStruct.d_modelPar_XRECD_dX;
B4SOIxrecd = modelStruct.modelPar_XRECD;
d_B4SOIxtund_dX = modelStruct.d_modelPar_XTUND_dX;
B4SOIxtund = modelStruct.modelPar_XTUND;
instanceStruct.d_instanceVar_B4SOIGatesidewallJctSPotential_dX = modelStruct.d_modelPar_PBSWG_dX;
instanceStruct.instanceVar_B4SOIGatesidewallJctSPotential = modelStruct.modelPar_PBSWG;
instanceStruct.d_instanceVar_B4SOIGatesidewallJctDPotential_dX = modelStruct.d_modelPar_PBSWGD_dX;
instanceStruct.instanceVar_B4SOIGatesidewallJctDPotential = modelStruct.modelPar_PBSWGD;
instanceStruct.d_instanceVar_B4SOIbodyJctGateSideSGradingCoeff_dX = modelStruct.d_modelPar_MJSWG_dX;
instanceStruct.instanceVar_B4SOIbodyJctGateSideSGradingCoeff = modelStruct.modelPar_MJSWG;
instanceStruct.d_instanceVar_B4SOIbodyJctGateSideDGradingCoeff_dX = modelStruct.d_modelPar_MJSWGD_dX;
instanceStruct.instanceVar_B4SOIbodyJctGateSideDGradingCoeff = modelStruct.modelPar_MJSWGD;
instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapS_dX = modelStruct.d_modelPar_CJSWG_dX;
instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapS = modelStruct.modelPar_CJSWG;
instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapD_dX = modelStruct.d_modelPar_CJSWGD_dX;
instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapD = modelStruct.modelPar_CJSWGD;
d_B4SOILint_dX = modelStruct.d_modelPar_LINT_dX;
B4SOILint = modelStruct.modelPar_LINT;
d_B4SOILl_dX = modelStruct.d_modelPar_LL_dX;
B4SOILl = modelStruct.modelPar_LL;
d_B4SOILlc_dX = modelStruct.d_modelPar_LLC_dX;
B4SOILlc = modelStruct.modelPar_LLC;
d_B4SOILln_dX = modelStruct.d_modelPar_LLN_dX;
B4SOILln = modelStruct.modelPar_LLN;
d_B4SOILw_dX = modelStruct.d_modelPar_LW_dX;
B4SOILw = modelStruct.modelPar_LW;
d_B4SOILwc_dX = modelStruct.d_modelPar_LWC_dX;
B4SOILwc = modelStruct.modelPar_LWC;
d_B4SOILwn_dX = modelStruct.d_modelPar_LWN_dX;
B4SOILwn = modelStruct.modelPar_LWN;
d_B4SOILwl_dX = modelStruct.d_modelPar_LWL_dX;
B4SOILwl = modelStruct.modelPar_LWL;
d_B4SOILwlc_dX = modelStruct.d_modelPar_LWLC_dX;
B4SOILwlc = modelStruct.modelPar_LWLC;
d_B4SOIwr_dX = modelStruct.d_modelPar_WR_dX;
B4SOIwr = modelStruct.modelPar_WR;
d_B4SOIWint_dX = modelStruct.d_modelPar_WINT_dX;
B4SOIWint = modelStruct.modelPar_WINT;
d_B4SOIdwg_dX = modelStruct.d_modelPar_DWG_dX;
B4SOIdwg = modelStruct.modelPar_DWG;
d_B4SOIdwb_dX = modelStruct.d_modelPar_DWB_dX;
B4SOIdwb = modelStruct.modelPar_DWB;
d_B4SOIWl_dX = modelStruct.d_modelPar_WL_dX;
B4SOIWl = modelStruct.modelPar_WL;
d_B4SOIWlc_dX = modelStruct.d_modelPar_WLC_dX;
B4SOIWlc = modelStruct.modelPar_WLC;
d_B4SOIWln_dX = modelStruct.d_modelPar_WLN_dX;
B4SOIWln = modelStruct.modelPar_WLN;
d_B4SOIWw_dX = modelStruct.d_modelPar_WW_dX;
B4SOIWw = modelStruct.modelPar_WW;
d_B4SOIWwc_dX = modelStruct.d_modelPar_WWC_dX;
B4SOIWwc = modelStruct.modelPar_WWC;
d_B4SOIWwn_dX = modelStruct.d_modelPar_WWN_dX;
B4SOIWwn = modelStruct.modelPar_WWN;
d_B4SOIWwl_dX = modelStruct.d_modelPar_WWL_dX;
B4SOIWwl = modelStruct.modelPar_WWL;
d_B4SOIWwlc_dX = modelStruct.d_modelPar_WWLC_dX;
B4SOIWwlc = modelStruct.modelPar_WWLC;
d_B4SOIb0_dX = modelStruct.d_modelPar_B0_dX;
B4SOIb0 = modelStruct.modelPar_B0;
d_B4SOIb1_dX = modelStruct.d_modelPar_B1_dX;
B4SOIb1 = modelStruct.modelPar_B1;
d_B4SOIcgsl_dX = modelStruct.d_modelPar_CGSL_dX;
B4SOIcgsl = modelStruct.modelPar_CGSL;
d_B4SOIcgdl_dX = modelStruct.d_modelPar_CGDL_dX;
B4SOIcgdl = modelStruct.modelPar_CGDL;
d_B4SOIckappa_dX = modelStruct.d_modelPar_CKAPPA_dX;
B4SOIckappa = modelStruct.modelPar_CKAPPA;
d_B4SOIclc_dX = modelStruct.d_modelPar_CLC_dX;
B4SOIclc = modelStruct.modelPar_CLC;
d_B4SOIcle_dX = modelStruct.d_modelPar_CLE_dX;
B4SOIcle = modelStruct.modelPar_CLE;
d_B4SOIdwc_dX = modelStruct.d_modelPar_DWC_dX;
B4SOIdwc = modelStruct.modelPar_DWC;
d_B4SOIdlc_dX = modelStruct.d_modelPar_DLC_dX;
B4SOIdlc = modelStruct.modelPar_DLC;
d_B4SOIalpha0_dX = modelStruct.d_modelPar_ALPHA0_dX;
B4SOIalpha0 = modelStruct.modelPar_ALPHA0;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityA_dX = modelStruct.d_modelPar_NOIA_dX;
instanceStruct.instanceVar_B4SOIoxideTrapDensityA = modelStruct.modelPar_NOIA;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityB_dX = modelStruct.d_modelPar_NOIB_dX;
instanceStruct.instanceVar_B4SOIoxideTrapDensityB = modelStruct.modelPar_NOIB;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityC_dX = modelStruct.d_modelPar_NOIC_dX;
instanceStruct.instanceVar_B4SOIoxideTrapDensityC = modelStruct.modelPar_NOIC;
instanceStruct.d_instanceVar_B4SOItnoia_dX = modelStruct.d_modelPar_TNOIA_dX;
instanceStruct.instanceVar_B4SOItnoia = modelStruct.modelPar_TNOIA;
instanceStruct.d_instanceVar_B4SOItnoib_dX = modelStruct.d_modelPar_TNOIB_dX;
instanceStruct.instanceVar_B4SOItnoib = modelStruct.modelPar_TNOIB;
instanceStruct.d_instanceVar_B4SOIrnoia_dX = modelStruct.d_modelPar_RNOIA_dX;
instanceStruct.instanceVar_B4SOIrnoia = modelStruct.modelPar_RNOIA;
instanceStruct.d_instanceVar_B4SOIrnoib_dX = modelStruct.d_modelPar_RNOIB_dX;
instanceStruct.instanceVar_B4SOIrnoib = modelStruct.modelPar_RNOIB;
instanceStruct.d_instanceVar_B4SOIntnoi_dX = modelStruct.d_modelPar_NTNOI_dX;
instanceStruct.instanceVar_B4SOIntnoi = modelStruct.modelPar_NTNOI;
instanceStruct.d_instanceVar_B4SOIsaref_dX = modelStruct.d_modelPar_SAREF_dX;
instanceStruct.instanceVar_B4SOIsaref = modelStruct.modelPar_SAREF;
instanceStruct.d_instanceVar_B4SOIsbref_dX = modelStruct.d_modelPar_SBREF_dX;
instanceStruct.instanceVar_B4SOIsbref = modelStruct.modelPar_SBREF;
d_B4SOIwlod_dX = modelStruct.d_modelPar_WLOD_dX;
B4SOIwlod = modelStruct.modelPar_WLOD;
instanceStruct.d_instanceVar_B4SOIku0_dX = modelStruct.d_modelPar_KU0_dX;
instanceStruct.instanceVar_B4SOIku0 = modelStruct.modelPar_KU0;
instanceStruct.d_instanceVar_B4SOIkvsat_dX = modelStruct.d_modelPar_KVSAT_dX;
instanceStruct.instanceVar_B4SOIkvsat = modelStruct.modelPar_KVSAT;
d_B4SOIkvth0_dX = modelStruct.d_modelPar_KVTH0_dX;
B4SOIkvth0 = modelStruct.modelPar_KVTH0;
instanceStruct.d_instanceVar_B4SOItku0_dX = modelStruct.d_modelPar_TKU0_dX;
instanceStruct.instanceVar_B4SOItku0 = modelStruct.modelPar_TKU0;
d_B4SOIllodku0_dX = modelStruct.d_modelPar_LLODKU0_dX;
B4SOIllodku0 = modelStruct.modelPar_LLODKU0;
d_B4SOIwlodku0_dX = modelStruct.d_modelPar_WLODKU0_dX;
B4SOIwlodku0 = modelStruct.modelPar_WLODKU0;
d_B4SOIllodvth_dX = modelStruct.d_modelPar_LLODVTH_dX;
B4SOIllodvth = modelStruct.modelPar_LLODVTH;
d_B4SOIwlodvth_dX = modelStruct.d_modelPar_WLODVTH_dX;
B4SOIwlodvth = modelStruct.modelPar_WLODVTH;
d_B4SOIstk2_dX = modelStruct.d_modelPar_STK2_dX;
B4SOIstk2 = modelStruct.modelPar_STK2;
instanceStruct.d_instanceVar_B4SOIlodk2_dX = modelStruct.d_modelPar_LODK2_dX;
instanceStruct.instanceVar_B4SOIlodk2 = modelStruct.modelPar_LODK2;
d_B4SOIsteta0_dX = modelStruct.d_modelPar_STETA0_dX;
B4SOIsteta0 = modelStruct.modelPar_STETA0;
instanceStruct.d_instanceVar_B4SOIlodeta0_dX = modelStruct.d_modelPar_LODETA0_dX;
instanceStruct.instanceVar_B4SOIlodeta0 = modelStruct.modelPar_LODETA0;
d_B4SOIsteta0cv_dX = modelStruct.d_modelPar_STETA0CV_dX;
B4SOIsteta0cv = modelStruct.modelPar_STETA0CV;
instanceStruct.d_instanceVar_B4SOIlodeta0cv_dX = modelStruct.d_modelPar_LODETA0CV_dX;
instanceStruct.instanceVar_B4SOIlodeta0cv = modelStruct.modelPar_LODETA0CV;
d_B4SOIgbmin_dX = modelStruct.d_modelPar_GBMIN_dX;
B4SOIgbmin = modelStruct.modelPar_GBMIN;
instanceStruct.d_instanceVar_B4SOIbf_dX = modelStruct.d_modelPar_BF_dX;
instanceStruct.instanceVar_B4SOIbf = modelStruct.modelPar_BF;
instanceStruct.d_instanceVar_B4SOIw0flk_dX = modelStruct.d_modelPar_W0FLK_dX;
instanceStruct.instanceVar_B4SOIw0flk = modelStruct.modelPar_W0FLK;
d_B4SOIdvtp0_dX = modelStruct.d_modelPar_DVTP0_dX;
B4SOIdvtp0 = modelStruct.modelPar_DVTP0;
d_B4SOIdvtp1_dX = modelStruct.d_modelPar_DVTP1_dX;
B4SOIdvtp1 = modelStruct.modelPar_DVTP1;
d_B4SOIdvtp2_dX = modelStruct.d_modelPar_DVTP2_dX;
B4SOIdvtp2 = modelStruct.modelPar_DVTP2;
d_B4SOIdvtp3_dX = modelStruct.d_modelPar_DVTP3_dX;
B4SOIdvtp3 = modelStruct.modelPar_DVTP3;
d_B4SOIdvtp4_dX = modelStruct.d_modelPar_DVTP4_dX;
B4SOIdvtp4 = modelStruct.modelPar_DVTP4;
d_B4SOIminv_dX = modelStruct.d_modelPar_MINV_dX;
B4SOIminv = modelStruct.modelPar_MINV;
d_B4SOIpdits_dX = modelStruct.d_modelPar_PDITS_dX;
B4SOIpdits = modelStruct.modelPar_PDITS;
instanceStruct.d_instanceVar_B4SOIpditsl_dX = modelStruct.d_modelPar_PDITSL_dX;
instanceStruct.instanceVar_B4SOIpditsl = modelStruct.modelPar_PDITSL;
d_B4SOIpditsd_dX = modelStruct.d_modelPar_PDITSD_dX;
B4SOIpditsd = modelStruct.modelPar_PDITSD;
d_B4SOIfprout_dX = modelStruct.d_modelPar_FPROUT_dX;
B4SOIfprout = modelStruct.modelPar_FPROUT;
instanceStruct.d_instanceVar_B4SOIem_dX = modelStruct.d_modelPar_EM_dX;
instanceStruct.instanceVar_B4SOIem = modelStruct.modelPar_EM;
instanceStruct.d_instanceVar_B4SOIef_dX = modelStruct.d_modelPar_EF_dX;
instanceStruct.instanceVar_B4SOIef = modelStruct.modelPar_EF;
instanceStruct.d_instanceVar_B4SOIaf_dX = modelStruct.d_modelPar_AF_dX;
instanceStruct.instanceVar_B4SOIaf = modelStruct.modelPar_AF;
instanceStruct.d_instanceVar_B4SOIkf_dX = modelStruct.d_modelPar_KF_dX;
instanceStruct.instanceVar_B4SOIkf = modelStruct.modelPar_KF;
instanceStruct.d_instanceVar_B4SOInoif_dX = modelStruct.d_modelPar_NOIF_dX;
instanceStruct.instanceVar_B4SOInoif = modelStruct.modelPar_NOIF;
instanceStruct.d_instanceVar_B4SOIk1w1_dX = modelStruct.d_modelPar_K1W1_dX;
instanceStruct.instanceVar_B4SOIk1w1 = modelStruct.modelPar_K1W1;
instanceStruct.d_instanceVar_B4SOIk1w2_dX = modelStruct.d_modelPar_K1W2_dX;
instanceStruct.instanceVar_B4SOIk1w2 = modelStruct.modelPar_K1W2;
instanceStruct.d_instanceVar_B4SOIketas_dX = modelStruct.d_modelPar_KETAS_dX;
instanceStruct.instanceVar_B4SOIketas = modelStruct.modelPar_KETAS;
instanceStruct.d_instanceVar_B4SOIdwbc_dX = modelStruct.d_modelPar_DWBC_dX;
instanceStruct.instanceVar_B4SOIdwbc = modelStruct.modelPar_DWBC;
instanceStruct.d_instanceVar_B4SOIbeta0_dX = modelStruct.d_modelPar_BETA0_dX;
instanceStruct.instanceVar_B4SOIbeta0 = modelStruct.modelPar_BETA0;
instanceStruct.d_instanceVar_B4SOIbeta1_dX = modelStruct.d_modelPar_BETA1_dX;
instanceStruct.instanceVar_B4SOIbeta1 = modelStruct.modelPar_BETA1;
instanceStruct.d_instanceVar_B4SOIbeta2_dX = modelStruct.d_modelPar_BETA2_dX;
instanceStruct.instanceVar_B4SOIbeta2 = modelStruct.modelPar_BETA2;
d_B4SOIvdsatii0_dX = modelStruct.d_modelPar_VDSATII0_dX;
B4SOIvdsatii0 = modelStruct.modelPar_VDSATII0;
instanceStruct.d_instanceVar_B4SOItii_dX = modelStruct.d_modelPar_TII_dX;
instanceStruct.instanceVar_B4SOItii = modelStruct.modelPar_TII;
instanceStruct.d_instanceVar_B4SOIlii_dX = modelStruct.d_modelPar_LII_dX;
instanceStruct.instanceVar_B4SOIlii = modelStruct.modelPar_LII;
d_B4SOIsii0_dX = modelStruct.d_modelPar_SII0_dX;
B4SOIsii0 = modelStruct.modelPar_SII0;
instanceStruct.d_instanceVar_B4SOIsii1_dX = modelStruct.d_modelPar_SII1_dX;
instanceStruct.instanceVar_B4SOIsii1 = modelStruct.modelPar_SII1;
instanceStruct.d_instanceVar_B4SOIsii2_dX = modelStruct.d_modelPar_SII2_dX;
instanceStruct.instanceVar_B4SOIsii2 = modelStruct.modelPar_SII2;
instanceStruct.d_instanceVar_B4SOIsiid_dX = modelStruct.d_modelPar_SIID_dX;
instanceStruct.instanceVar_B4SOIsiid = modelStruct.modelPar_SIID;
instanceStruct.d_instanceVar_B4SOIfbjtii_dX = modelStruct.d_modelPar_FBJTII_dX;
instanceStruct.instanceVar_B4SOIfbjtii = modelStruct.modelPar_FBJTII;
d_B4SOIebjtii_dX = modelStruct.d_modelPar_EBJTII_dX;
B4SOIebjtii = modelStruct.modelPar_EBJTII;
d_B4SOIcbjtii_dX = modelStruct.d_modelPar_CBJTII_dX;
B4SOIcbjtii = modelStruct.modelPar_CBJTII;
d_B4SOIvbci_dX = modelStruct.d_modelPar_VBCI_dX;
B4SOIvbci = modelStruct.modelPar_VBCI;
d_B4SOIabjtii_dX = modelStruct.d_modelPar_ABJTII_dX;
B4SOIabjtii = modelStruct.modelPar_ABJTII;
d_B4SOImbjtii_dX = modelStruct.d_modelPar_MBJTII_dX;
B4SOImbjtii = modelStruct.modelPar_MBJTII;
instanceStruct.d_instanceVar_B4SOItvbci_dX = modelStruct.d_modelPar_TVBCI_dX;
instanceStruct.instanceVar_B4SOItvbci = modelStruct.modelPar_TVBCI;
instanceStruct.d_instanceVar_B4SOIesatii_dX = modelStruct.d_modelPar_ESATII_dX;
instanceStruct.instanceVar_B4SOIesatii = modelStruct.modelPar_ESATII;
instanceStruct.d_instanceVar_B4SOIntun_dX = modelStruct.d_modelPar_NTUN_dX;
instanceStruct.instanceVar_B4SOIntun = modelStruct.modelPar_NTUN;
instanceStruct.d_instanceVar_B4SOIntund_dX = modelStruct.d_modelPar_NTUND_dX;
instanceStruct.instanceVar_B4SOIntund = modelStruct.modelPar_NTUND;
d_B4SOInrecf0_dX = modelStruct.d_modelPar_NRECF0_dX;
B4SOInrecf0 = modelStruct.modelPar_NRECF0;
d_B4SOInrecf0d_dX = modelStruct.d_modelPar_NRECF0D_dX;
B4SOInrecf0d = modelStruct.modelPar_NRECF0D;
d_B4SOInrecr0_dX = modelStruct.d_modelPar_NRECR0_dX;
B4SOInrecr0 = modelStruct.modelPar_NRECR0;
d_B4SOInrecr0d_dX = modelStruct.d_modelPar_NRECR0D_dX;
B4SOInrecr0d = modelStruct.modelPar_NRECR0D;
instanceStruct.d_instanceVar_B4SOIisbjt_dX = modelStruct.d_modelPar_ISBJT_dX;
instanceStruct.instanceVar_B4SOIisbjt = modelStruct.modelPar_ISBJT;
instanceStruct.d_instanceVar_B4SOIidbjt_dX = modelStruct.d_modelPar_IDBJT_dX;
instanceStruct.instanceVar_B4SOIidbjt = modelStruct.modelPar_IDBJT;
instanceStruct.d_instanceVar_B4SOIisdif_dX = modelStruct.d_modelPar_ISDIF_dX;
instanceStruct.instanceVar_B4SOIisdif = modelStruct.modelPar_ISDIF;
instanceStruct.d_instanceVar_B4SOIiddif_dX = modelStruct.d_modelPar_IDDIF_dX;
instanceStruct.instanceVar_B4SOIiddif = modelStruct.modelPar_IDDIF;
instanceStruct.d_instanceVar_B4SOIisrec_dX = modelStruct.d_modelPar_ISREC_dX;
instanceStruct.instanceVar_B4SOIisrec = modelStruct.modelPar_ISREC;
instanceStruct.d_instanceVar_B4SOIidrec_dX = modelStruct.d_modelPar_IDREC_dX;
instanceStruct.instanceVar_B4SOIidrec = modelStruct.modelPar_IDREC;
instanceStruct.d_instanceVar_B4SOIistun_dX = modelStruct.d_modelPar_ISTUN_dX;
instanceStruct.instanceVar_B4SOIistun = modelStruct.modelPar_ISTUN;
instanceStruct.d_instanceVar_B4SOIidtun_dX = modelStruct.d_modelPar_IDTUN_dX;
instanceStruct.instanceVar_B4SOIidtun = modelStruct.modelPar_IDTUN;
d_B4SOIln_dX = modelStruct.d_modelPar_LN_dX;
B4SOIln = modelStruct.modelPar_LN;
instanceStruct.d_instanceVar_B4SOIvrec0_dX = modelStruct.d_modelPar_VREC0_dX;
instanceStruct.instanceVar_B4SOIvrec0 = modelStruct.modelPar_VREC0;
instanceStruct.d_instanceVar_B4SOIvrec0d_dX = modelStruct.d_modelPar_VREC0D_dX;
instanceStruct.instanceVar_B4SOIvrec0d = modelStruct.modelPar_VREC0D;
instanceStruct.d_instanceVar_B4SOIvtun0_dX = modelStruct.d_modelPar_VTUN0_dX;
instanceStruct.instanceVar_B4SOIvtun0 = modelStruct.modelPar_VTUN0;
instanceStruct.d_instanceVar_B4SOIvtun0d_dX = modelStruct.d_modelPar_VTUN0D_dX;
instanceStruct.instanceVar_B4SOIvtun0d = modelStruct.modelPar_VTUN0D;
instanceStruct.d_instanceVar_B4SOInbjt_dX = modelStruct.d_modelPar_NBJT_dX;
instanceStruct.instanceVar_B4SOInbjt = modelStruct.modelPar_NBJT;
d_B4SOIlbjt0_dX = modelStruct.d_modelPar_LBJT0_dX;
B4SOIlbjt0 = modelStruct.modelPar_LBJT0;
d_B4SOIldif0_dX = modelStruct.d_modelPar_LDIF0_dX;
B4SOIldif0 = modelStruct.modelPar_LDIF0;
d_B4SOIvabjt_dX = modelStruct.d_modelPar_VABJT_dX;
B4SOIvabjt = modelStruct.modelPar_VABJT;
instanceStruct.d_instanceVar_B4SOIaely_dX = modelStruct.d_modelPar_AELY_dX;
instanceStruct.instanceVar_B4SOIaely = modelStruct.modelPar_AELY;
instanceStruct.d_instanceVar_B4SOIahli_dX = modelStruct.d_modelPar_AHLI_dX;
instanceStruct.instanceVar_B4SOIahli = modelStruct.modelPar_AHLI;
instanceStruct.d_instanceVar_B4SOIahlid_dX = modelStruct.d_modelPar_AHLID_dX;
instanceStruct.instanceVar_B4SOIahlid = modelStruct.modelPar_AHLID;
instanceStruct.d_instanceVar_B4SOIrbody_dX = modelStruct.d_modelPar_RBODY_dX;
instanceStruct.instanceVar_B4SOIrbody = modelStruct.modelPar_RBODY;
instanceStruct.d_instanceVar_B4SOIrbsh_dX = modelStruct.d_modelPar_RBSH_dX;
instanceStruct.instanceVar_B4SOIrbsh = modelStruct.modelPar_RBSH;
d_B4SOIcgeo_dX = modelStruct.d_modelPar_CGEO_dX;
B4SOIcgeo = modelStruct.modelPar_CGEO;
instanceStruct.d_instanceVar_B4SOItt_dX = modelStruct.d_modelPar_TT_dX;
instanceStruct.instanceVar_B4SOItt = modelStruct.modelPar_TT;
d_B4SOIndif_dX = modelStruct.d_modelPar_NDIF_dX;
B4SOIndif = modelStruct.modelPar_NDIF;
d_B4SOIvsdfb_dX = modelStruct.d_modelPar_VSDFB_dX;
B4SOIvsdfb = modelStruct.modelPar_VSDFB;
d_B4SOIvsdth_dX = modelStruct.d_modelPar_VSDTH_dX;
B4SOIvsdth = modelStruct.modelPar_VSDTH;
instanceStruct.d_instanceVar_B4SOIcsdmin_dX = modelStruct.d_modelPar_CSDMIN_dX;
instanceStruct.instanceVar_B4SOIcsdmin = modelStruct.modelPar_CSDMIN;
instanceStruct.d_instanceVar_B4SOIasd_dX = modelStruct.d_modelPar_ASD_dX;
instanceStruct.instanceVar_B4SOIasd = modelStruct.modelPar_ASD;
instanceStruct.d_instanceVar_B4SOIcsdesw_dX = modelStruct.d_modelPar_CSDESW_dX;
instanceStruct.instanceVar_B4SOIcsdesw = modelStruct.modelPar_CSDESW;
d_B4SOIntrecf_dX = modelStruct.d_modelPar_NTRECF_dX;
B4SOIntrecf = modelStruct.modelPar_NTRECF;
d_B4SOIntrecr_dX = modelStruct.d_modelPar_NTRECR_dX;
B4SOIntrecr = modelStruct.modelPar_NTRECR;
d_B4SOIdlcb_dX = modelStruct.d_modelPar_DLCB_dX;
B4SOIdlcb = modelStruct.modelPar_DLCB;
instanceStruct.d_instanceVar_B4SOIfbody_dX = modelStruct.d_modelPar_FBODY_dX;
instanceStruct.instanceVar_B4SOIfbody = modelStruct.modelPar_FBODY;
instanceStruct.d_instanceVar_B4SOItcjswg_dX = modelStruct.d_modelPar_TCJSWG_dX;
instanceStruct.instanceVar_B4SOItcjswg = modelStruct.modelPar_TCJSWG;
instanceStruct.d_instanceVar_B4SOItpbswg_dX = modelStruct.d_modelPar_TPBSWG_dX;
instanceStruct.instanceVar_B4SOItpbswg = modelStruct.modelPar_TPBSWG;
instanceStruct.d_instanceVar_B4SOItcjswgd_dX = modelStruct.d_modelPar_TCJSWGD_dX;
instanceStruct.instanceVar_B4SOItcjswgd = modelStruct.modelPar_TCJSWGD;
instanceStruct.d_instanceVar_B4SOItpbswgd_dX = modelStruct.d_modelPar_TPBSWGD_dX;
instanceStruct.instanceVar_B4SOItpbswgd = modelStruct.modelPar_TPBSWGD;
instanceStruct.d_instanceVar_B4SOIacde_dX = modelStruct.d_modelPar_ACDE_dX;
instanceStruct.instanceVar_B4SOIacde = modelStruct.modelPar_ACDE;
instanceStruct.d_instanceVar_B4SOImoin_dX = modelStruct.d_modelPar_MOIN_dX;
instanceStruct.instanceVar_B4SOImoin = modelStruct.modelPar_MOIN;
d_B4SOInoff_dX = modelStruct.d_modelPar_NOFF_dX;
B4SOInoff = modelStruct.modelPar_NOFF;
d_B4SOInoff2_dX = modelStruct.d_modelPar_NOFF2_dX;
B4SOInoff2 = modelStruct.modelPar_NOFF2;
d_B4SOIdelvt_dX = modelStruct.d_modelPar_DELVT_dX;
B4SOIdelvt = modelStruct.modelPar_DELVT;
d_B4SOIkb1_dX = modelStruct.d_modelPar_KB1_dX;
B4SOIkb1 = modelStruct.modelPar_KB1;
instanceStruct.d_instanceVar_B4SOIdlbg_dX = modelStruct.d_modelPar_DLBG_dX;
instanceStruct.instanceVar_B4SOIdlbg = modelStruct.modelPar_DLBG;
d_B4SOIcfrcoeff_dX = modelStruct.d_modelPar_CFRCOEFF_dX;
B4SOIcfrcoeff = modelStruct.modelPar_CFRCOEFF;
instanceStruct.instanceVar_B4SOIigbMod = modelStruct.modelPar_IGMOD;
instanceStruct.instanceVar_B4SOIigcMod = modelStruct.modelPar_IGCMOD;
instanceStruct.d_instanceVar_B4SOItoxqm_dX = modelStruct.d_modelPar_TOXQM_dX;
instanceStruct.instanceVar_B4SOItoxqm = modelStruct.modelPar_TOXQM;
instanceStruct.d_instanceVar_B4SOIwth0_dX = modelStruct.d_modelPar_WTH0_dX;
instanceStruct.instanceVar_B4SOIwth0 = modelStruct.modelPar_WTH0;
instanceStruct.d_instanceVar_B4SOIrhalo_dX = modelStruct.d_modelPar_RHALO_dX;
instanceStruct.instanceVar_B4SOIrhalo = modelStruct.modelPar_RHALO;
instanceStruct.d_instanceVar_B4SOIntox_dX = modelStruct.d_modelPar_NTOX_dX;
instanceStruct.instanceVar_B4SOIntox = modelStruct.modelPar_NTOX;
instanceStruct.d_instanceVar_B4SOItoxref_dX = modelStruct.d_modelPar_TOXREF_dX;
instanceStruct.instanceVar_B4SOItoxref = modelStruct.modelPar_TOXREF;
instanceStruct.d_instanceVar_B4SOIebg_dX = modelStruct.d_modelPar_EBG_dX;
instanceStruct.instanceVar_B4SOIebg = modelStruct.modelPar_EBG;
instanceStruct.d_instanceVar_B4SOIvevb_dX = modelStruct.d_modelPar_VEVB_dX;
instanceStruct.instanceVar_B4SOIvevb = modelStruct.modelPar_VEVB;
d_B4SOIalphaGB1_dX = modelStruct.d_modelPar_ALPHAGB1_dX;
B4SOIalphaGB1 = modelStruct.modelPar_ALPHAGB1;
d_B4SOIbetaGB1_dX = modelStruct.d_modelPar_BETAGB1_dX;
B4SOIbetaGB1 = modelStruct.modelPar_BETAGB1;
instanceStruct.d_instanceVar_B4SOIvgb1_dX = modelStruct.d_modelPar_VGB1_dX;
instanceStruct.instanceVar_B4SOIvgb1 = modelStruct.modelPar_VGB1;
instanceStruct.d_instanceVar_B4SOIvecb_dX = modelStruct.d_modelPar_VECB_dX;
instanceStruct.instanceVar_B4SOIvecb = modelStruct.modelPar_VECB;
d_B4SOIalphaGB2_dX = modelStruct.d_modelPar_ALPHAGB2_dX;
B4SOIalphaGB2 = modelStruct.modelPar_ALPHAGB2;
d_B4SOIbetaGB2_dX = modelStruct.d_modelPar_BETAGB2_dX;
B4SOIbetaGB2 = modelStruct.modelPar_BETAGB2;
instanceStruct.d_instanceVar_B4SOIvgb2_dX = modelStruct.d_modelPar_VGB2_dX;
instanceStruct.instanceVar_B4SOIvgb2 = modelStruct.modelPar_VGB2;
d_B4SOIaigbcp2_dX = modelStruct.d_modelPar_AIGBCP2_dX;
B4SOIaigbcp2 = modelStruct.modelPar_AIGBCP2;
d_B4SOIbigbcp2_dX = modelStruct.d_modelPar_BIGBCP2_dX;
B4SOIbigbcp2 = modelStruct.modelPar_BIGBCP2;
d_B4SOIcigbcp2_dX = modelStruct.d_modelPar_CIGBCP2_dX;
B4SOIcigbcp2 = modelStruct.modelPar_CIGBCP2;
instanceStruct.d_instanceVar_B4SOIvoxh_dX = modelStruct.d_modelPar_VOXH_dX;
instanceStruct.instanceVar_B4SOIvoxh = modelStruct.modelPar_VOXH;
instanceStruct.d_instanceVar_B4SOIdeltavox_dX = modelStruct.d_modelPar_DELTAVOX_dX;
instanceStruct.instanceVar_B4SOIdeltavox = modelStruct.modelPar_DELTAVOX;
d_B4SOIaigc_dX = modelStruct.d_modelPar_AIGC_dX;
B4SOIaigc = modelStruct.modelPar_AIGC;
d_B4SOIbigc_dX = modelStruct.d_modelPar_BIGC_dX;
B4SOIbigc = modelStruct.modelPar_BIGC;
d_B4SOIcigc_dX = modelStruct.d_modelPar_CIGC_dX;
B4SOIcigc = modelStruct.modelPar_CIGC;
d_B4SOIaigsd_dX = modelStruct.d_modelPar_AIGSD_dX;
B4SOIaigsd = modelStruct.modelPar_AIGSD;
d_B4SOIbigsd_dX = modelStruct.d_modelPar_BIGSD_dX;
B4SOIbigsd = modelStruct.modelPar_BIGSD;
d_B4SOIcigsd_dX = modelStruct.d_modelPar_CIGSD_dX;
B4SOIcigsd = modelStruct.modelPar_CIGSD;
d_B4SOInigc_dX = modelStruct.d_modelPar_NIGC_dX;
B4SOInigc = modelStruct.modelPar_NIGC;
d_B4SOIpigcd_dX = modelStruct.d_modelPar_PIGCD_dX;
B4SOIpigcd = modelStruct.modelPar_PIGCD;
d_B4SOIpoxedge_dX = modelStruct.d_modelPar_POXEDGE_dX;
B4SOIpoxedge = modelStruct.modelPar_POXEDGE;
d_B4SOIdlcig_dX = modelStruct.d_modelPar_DLCIG_dX;
B4SOIdlcig = modelStruct.modelPar_DLCIG;
d_B4SOIvbsa_dX = modelStruct.d_modelPar_VBSA_dX;
B4SOIvbsa = modelStruct.modelPar_VBSA;
d_B4SOInofffd_dX = modelStruct.d_modelPar_NOFFFD_dX;
B4SOInofffd = modelStruct.modelPar_NOFFFD;
d_B4SOIvofffd_dX = modelStruct.d_modelPar_VOFFFD_dX;
B4SOIvofffd = modelStruct.modelPar_VOFFFD;
d_B4SOIk1b_dX = modelStruct.d_modelPar_K1B_dX;
B4SOIk1b = modelStruct.modelPar_K1B;
d_B4SOIk2b_dX = modelStruct.d_modelPar_K2B_dX;
B4SOIk2b = modelStruct.modelPar_K2B;
d_B4SOIdk2b_dX = modelStruct.d_modelPar_DK2B_dX;
B4SOIdk2b = modelStruct.modelPar_DK2B;
d_B4SOIdvbd0_dX = modelStruct.d_modelPar_DVBD0_dX;
B4SOIdvbd0 = modelStruct.modelPar_DVBD0;
d_B4SOIdvbd1_dX = modelStruct.d_modelPar_DVBD1_dX;
B4SOIdvbd1 = modelStruct.modelPar_DVBD1;
d_B4SOImoinFD_dX = modelStruct.d_modelPar_MOINFD_dX;
B4SOImoinFD = modelStruct.modelPar_MOINFD;
d_B4SOIvbs0pd_dX = modelStruct.d_modelPar_VBS0PD_dX;
B4SOIvbs0pd = modelStruct.modelPar_VBS0PD;
d_B4SOIvbs0fd_dX = modelStruct.d_modelPar_VBS0FD_dX;
B4SOIvbs0fd = modelStruct.modelPar_VBS0FD;
d_B4SOIxrcrg1_dX = modelStruct.d_modelPar_XRCRG1_dX;
B4SOIxrcrg1 = modelStruct.modelPar_XRCRG1;
d_B4SOIxrcrg2_dX = modelStruct.d_modelPar_XRCRG2_dX;
B4SOIxrcrg2 = modelStruct.modelPar_XRCRG2;
d_B4SOIrshg_dX = modelStruct.d_modelPar_RSHG_dX;
B4SOIrshg = modelStruct.modelPar_RSHG;
d_B4SOIngcon_dX = modelStruct.d_modelPar_NGCON_dX;
B4SOIngcon = modelStruct.modelPar_NGCON;
d_B4SOIxgw_dX = modelStruct.d_modelPar_XGW_dX;
B4SOIxgw = modelStruct.modelPar_XGW;
d_B4SOIxgl_dX = modelStruct.d_modelPar_XGL_dX;
B4SOIxgl = modelStruct.modelPar_XGL;
instanceStruct.instanceVar_B4SOIrdsMod = modelStruct.modelPar_RDSMOD;
instanceStruct.instanceVar_B4SOIfdMod = modelStruct.modelPar_FDMOD;
d_B4SOIvsce_dX = modelStruct.d_modelPar_VSCE_dX;
B4SOIvsce = modelStruct.modelPar_VSCE;
d_B4SOIcdsbs_dX = modelStruct.d_modelPar_CDSBS_dX;
B4SOIcdsbs = modelStruct.modelPar_CDSBS;
d_B4SOIminvcv_dX = modelStruct.d_modelPar_MINVCV_dX;
B4SOIminvcv = modelStruct.modelPar_MINVCV;
d_B4SOIvoffcv_dX = modelStruct.d_modelPar_VOFFCV_dX;
B4SOIvoffcv = modelStruct.modelPar_VOFFCV;
d_B4SOIEGGBCP2_dX = modelStruct.d_modelPar_EGGBCP2_dX;
B4SOIEGGBCP2 = modelStruct.modelPar_EGGBCP2;
d_B4SOIEGGDEP_dX = modelStruct.d_modelPar_EGGDEP_dX;
B4SOIEGGDEP = modelStruct.modelPar_EGGDEP;
d_B4SOIAGB1_dX = modelStruct.d_modelPar_AGB1_dX;
B4SOIAGB1 = modelStruct.modelPar_AGB1;
d_B4SOIBGB1_dX = modelStruct.d_modelPar_BGB1_dX;
B4SOIBGB1 = modelStruct.modelPar_BGB1;
d_B4SOIAGB2_dX = modelStruct.d_modelPar_AGB2_dX;
B4SOIAGB2 = modelStruct.modelPar_AGB2;
d_B4SOIBGB2_dX = modelStruct.d_modelPar_BGB2_dX;
B4SOIBGB2 = modelStruct.modelPar_BGB2;
d_B4SOIAGBC2N_dX = modelStruct.d_modelPar_AGBC2N_dX;
B4SOIAGBC2N = modelStruct.modelPar_AGBC2N;
d_B4SOIAGBC2P_dX = modelStruct.d_modelPar_AGBC2P_dX;
B4SOIAGBC2P = modelStruct.modelPar_AGBC2P;
d_B4SOIBGBC2N_dX = modelStruct.d_modelPar_BGBC2N_dX;
B4SOIBGBC2N = modelStruct.modelPar_BGBC2N;
d_B4SOIBGBC2P_dX = modelStruct.d_modelPar_BGBC2P_dX;
B4SOIBGBC2P = modelStruct.modelPar_BGBC2P;
d_B4SOIVTM00_dX = modelStruct.d_modelPar_VTM00_dX;
B4SOIVTM00 = modelStruct.modelPar_VTM00;
if ((modelStruct.modelPar_given_LPE0&&modelStruct.modelPar_given_NLX))
{
UserInfo(theInstance)  << "Warning: parameter LPE0 overrides NLX." << " " <<  std::endl; 
}
if ((modelStruct.modelPar_given_LLPE0&&modelStruct.modelPar_given_LNLX))
{
UserInfo(theInstance)  << "Warning: parameter LLPE0 overrides LNLX." << " " <<  std::endl; 
}
if ((modelStruct.modelPar_given_WLPE0&&modelStruct.modelPar_given_WNLX))
{
UserInfo(theInstance)  << "Warning: parameter WLPE0 overrides WNLX." << " " <<  std::endl; 
}
if ((modelStruct.modelPar_given_PLPE0&&modelStruct.modelPar_given_PNLX))
{
UserInfo(theInstance)  << "Warning: parameter PLPE0 overrides PNLX." << " " <<  std::endl; 
}
if ((modelStruct.modelPar_given_EGIDL&&modelStruct.modelPar_given_NGIDL))
{
UserInfo(theInstance)  << "Warning: parameter EGIDL overrides NGIDL." << " " <<  std::endl; 
}
if ((modelStruct.modelPar_given_LEGIDL&&modelStruct.modelPar_given_LNGIDL))
{
UserInfo(theInstance)  << "Warning: parameter LEGIDL overrides LNGIDL." << " " <<  std::endl; 
}
if ((modelStruct.modelPar_given_WEGIDL&&modelStruct.modelPar_given_WNGIDL))
{
UserInfo(theInstance)  << "Warning: parameter WEGIDL overrides WNGIDL." << " " <<  std::endl; 
}
if ((modelStruct.modelPar_given_PEGIDL&&modelStruct.modelPar_given_PNGIDL))
{
UserInfo(theInstance)  << "Warning: parameter PEGIDL overrides PNGIDL." << " " <<  std::endl; 
}
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
instanceStruct.d_instanceVar_epsrox_dX = 0.0;
instanceStruct.instanceVar_epsrox = 3.9;
instanceStruct.d_instanceVar_toxe_dX = instanceStruct.d_instanceVar_B4SOIeot_dX;
instanceStruct.instanceVar_toxe = instanceStruct.instanceVar_B4SOIeot;
instanceStruct.d_instanceVar_epssub_dX = 8.85418e-12*instanceStruct.d_instanceVar_B4SOIepsrsub_dX;
instanceStruct.instanceVar_epssub = (8.85418e-12*instanceStruct.instanceVar_B4SOIepsrsub);
{
double value_sqrt_0 = sqrt(((2.0e6*1.60219e-19)*instanceStruct.instanceVar_epssub));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_sqrt2qeps_dX = deriv_sqrt_0_d0*((2.0e6*1.60219e-19)*instanceStruct.d_instanceVar_epssub_dX);
instanceStruct.instanceVar_sqrt2qeps = value_sqrt_0;
}
instanceStruct.d_instanceVar_B4SOIcox_dX = ((instanceStruct.instanceVar_toxe*instanceStruct.d_instanceVar_epsrox_dX*8.85418e-12-(instanceStruct.instanceVar_epsrox*8.85418e-12)*instanceStruct.d_instanceVar_toxe_dX)/instanceStruct.instanceVar_toxe/instanceStruct.instanceVar_toxe);
instanceStruct.instanceVar_B4SOIcox = ((instanceStruct.instanceVar_epsrox*8.85418e-12)/instanceStruct.instanceVar_toxe);
instanceStruct.d_instanceVar_eggbcp2_dX = d_B4SOIEGGBCP2_dX;
instanceStruct.instanceVar_eggbcp2 = B4SOIEGGBCP2;
instanceStruct.d_instanceVar_eggdep_dX = d_B4SOIEGGDEP_dX;
instanceStruct.instanceVar_eggdep = B4SOIEGGDEP;
instanceStruct.d_instanceVar_agb1_dX = d_B4SOIAGB1_dX;
instanceStruct.instanceVar_agb1 = B4SOIAGB1;
instanceStruct.d_instanceVar_bgb1_dX = d_B4SOIBGB1_dX;
instanceStruct.instanceVar_bgb1 = B4SOIBGB1;
instanceStruct.d_instanceVar_agb2_dX = d_B4SOIAGB2_dX;
instanceStruct.instanceVar_agb2 = B4SOIAGB2;
instanceStruct.d_instanceVar_bgb2_dX = d_B4SOIBGB2_dX;
instanceStruct.instanceVar_bgb2 = B4SOIBGB2;
instanceStruct.d_instanceVar_agbc2n_dX = d_B4SOIAGBC2N_dX;
instanceStruct.instanceVar_agbc2n = B4SOIAGBC2N;
instanceStruct.d_instanceVar_agbc2p_dX = d_B4SOIAGBC2P_dX;
instanceStruct.instanceVar_agbc2p = B4SOIAGBC2P;
instanceStruct.d_instanceVar_bgbc2n_dX = d_B4SOIBGBC2N_dX;
instanceStruct.instanceVar_bgbc2n = B4SOIBGBC2N;
instanceStruct.d_instanceVar_bgbc2p_dX = d_B4SOIBGBC2P_dX;
instanceStruct.instanceVar_bgbc2p = B4SOIBGBC2P;
}
else
{
instanceStruct.d_instanceVar_epsrox_dX = d_B4SOIepsrox_dX;
instanceStruct.instanceVar_epsrox = B4SOIepsrox;
instanceStruct.d_instanceVar_toxe_dX = instanceStruct.d_instanceVar_B4SOItox_dX;
instanceStruct.instanceVar_toxe = instanceStruct.instanceVar_B4SOItox;
instanceStruct.d_instanceVar_epssub_dX = 0.0;
instanceStruct.instanceVar_epssub = 1.03594e-10;
instanceStruct.d_instanceVar_sqrt2qeps_dX = 0.0;
instanceStruct.instanceVar_sqrt2qeps = 5.753e-12;
instanceStruct.d_instanceVar_B4SOIcox_dX = (-3.453133e-11*instanceStruct.d_instanceVar_B4SOItox_dX/instanceStruct.instanceVar_B4SOItox/instanceStruct.instanceVar_B4SOItox);
instanceStruct.instanceVar_B4SOIcox = (3.453133e-11/instanceStruct.instanceVar_B4SOItox);
instanceStruct.d_instanceVar_eggbcp2_dX = d_B4SOIEGGBCP2_dX;
instanceStruct.instanceVar_eggbcp2 = B4SOIEGGBCP2;
instanceStruct.d_instanceVar_eggdep_dX = d_B4SOIEGGDEP_dX;
instanceStruct.instanceVar_eggdep = B4SOIEGGDEP;
instanceStruct.d_instanceVar_agb1_dX = d_B4SOIAGB1_dX;
instanceStruct.instanceVar_agb1 = B4SOIAGB1;
instanceStruct.d_instanceVar_bgb1_dX = d_B4SOIBGB1_dX;
instanceStruct.instanceVar_bgb1 = B4SOIBGB1;
instanceStruct.d_instanceVar_agb2_dX = d_B4SOIAGB2_dX;
instanceStruct.instanceVar_agb2 = B4SOIAGB2;
instanceStruct.d_instanceVar_bgb2_dX = d_B4SOIBGB2_dX;
instanceStruct.instanceVar_bgb2 = B4SOIBGB2;
instanceStruct.d_instanceVar_agbc2n_dX = d_B4SOIAGBC2N_dX;
instanceStruct.instanceVar_agbc2n = B4SOIAGBC2N;
instanceStruct.d_instanceVar_agbc2p_dX = d_B4SOIAGBC2P_dX;
instanceStruct.instanceVar_agbc2p = B4SOIAGBC2P;
instanceStruct.d_instanceVar_bgbc2n_dX = d_B4SOIBGBC2N_dX;
instanceStruct.instanceVar_bgbc2n = B4SOIBGBC2N;
instanceStruct.d_instanceVar_bgbc2p_dX = d_B4SOIBGBC2P_dX;
instanceStruct.instanceVar_bgbc2p = B4SOIBGBC2P;
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
if ((instanceStruct.instancePar_SHMOD==0))
{
if ((instanceStruct.instancePar_TNODEOUT==0))
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(theInstance)  << "Error:  you have specified a P and B node (5th and 6th node) when soimod=2 and TNODEOUT=0.  The P and B nodes are not used in this mode and Xyce cannot deal with them being externally specified.  Please remove the 5th and 6th nodes from the instance line." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(theInstance)  << "Error:  you have specified a P node (5th node) when soimod=2 and TNODEOUT=0.  P node is not used in this mode and Xyce cannot deal with it as an externally specified node.  Please remove the 5th node." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(theInstance)  << "Error:  you have specified a P and B node (5th and 6th node) when soimod=2 and TNODEOUT=1, and a seventh node for temperature. The P and B nodes are not used in this mode and Xyce cannot deal with them being externally specified.  Please remove the P and B nodes. With TNODEOUT=1, the fifth node should be the last one and will be used for temperature." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
}
}
else
{
if ((instanceStruct.instancePar_TNODEOUT==0))
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(theInstance)  << "Error:  you have specified a P and B node (fifth and 6th nodes) when soimod=2, and a temperature node (7th) with TNODEOUT=0.  None of these nodes are used in this mode and Xyce cannot deal with them being externally specified.  Please remove the 6th node." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(theInstance)  << "Error:  you have specified P and B nodes (5th and 6th node) when soimod=2 and TNODEOUT=0.  These nodes are not used in this mode and Xyce cannot deal with them being externally specified.  Please remove the 5th and 6th nodes." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(theInstance)  << "Error:  you have specified P node (5th node) when soimod=2 and TNODEOUT=0.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 5th node." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(theInstance)  << "Error:  you have specified a B node (6th node) when soimod=2 and TNODEOUT=1, and a seventh node for temperature. The B node is not used in this mode and Xyce cannot deal with it as an externally specified node.  Please remove the B node. With TNODEOUT=1, the 6th node should be the last one and will be used for temperature." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
}
}
if (modelStruct.modelPar_NODECHK)
{
if ((instanceStruct.instancePar_SHMOD==0))
{
if ((instanceStruct.instancePar_TNODEOUT==0))
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(theInstance)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(theInstance)  << "Warning: A 'P' node was specified, however 'SOIMOD = 2' does not support a 'P' node. Internally this node is tied to global ground." << " " <<  std::endl; 
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(theInstance)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(theInstance)  << "Warning: A 'P' node was specified, however 'SOIMOD = 2' does not support a 'P' node. Internally this node is tied to global ground." << " " <<  std::endl; 
}
}
}
}
else
{
if ((instanceStruct.instancePar_TNODEOUT==0))
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(theInstance)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(theInstance)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(theInstance)  << "Warning: A 'P' node was specified however 'SOIMOD = 2' does not support a 'P' node. Internally this node is tied to global ground." << " " <<  std::endl; 
}
}
}
}
else
{
if (((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t]))
{
UserInfo(theInstance)  << "Warning: 'P' and 'B' nodes were specified, however 'SOIMOD = 2' does not support 'P' or 'B' nodes. Internally these nodes are tied to global ground." << " " <<  std::endl; 
}
else
{
if ((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b]))
{
UserInfo(theInstance)  << "Warning: Fifth and sixth nodes were specified when TNODEOUT = 1. The fifth node will be treated as the temperature node 'T', and internally the sixth node will be tied to global ground." << " " <<  std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(theInstance)  << "Warning: A fifth node has been specified with TNODEOUT = 1. The fifth node will be treated as the temperature node 'T'." << " " <<  std::endl; 
}
else
{
if ((!((portsConnected_[admsNodeID_p]&&portsConnected_[admsNodeID_b])&&portsConnected_[admsNodeID_t])))
{
UserInfo(theInstance)  << "Warning: TNODEOUT = 1 requesting an external temperature node, but a fifth node for temperature is not supplied." << " " <<  std::endl; 
}
}
}
}
}
}
}
instanceStruct.instanceVar_B4SOIbodyMod = 0;
}
else
{
if ((portsConnected_[admsNodeID_p]==0))
{
instanceStruct.instanceVar_B4SOIbodyMod = 0;
if (portsConnected_[admsNodeID_b])
{
UserInfo(theInstance)  << "Warning: cannot connect to 'b' terminal without connecting 'p' terminal." << " " <<  std::endl; 
}
}
else
{
if (((portsConnected_[admsNodeID_t]==0)&&(instanceStruct.instancePar_TNODEOUT==1)))
{
UserInfo(theInstance)  << "Xyce does not implement the BSIM-SOI 4 model with TNODEOUT and fewer than 7 nodes.  Ignore any subsequent messages regarding node collapse, they are meaningless." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
if ((portsConnected_[admsNodeID_b]==0))
{
if (((instanceStruct.instanceVar_B4SOIrbody==0.0)&&(instanceStruct.instanceVar_B4SOIrbsh==0.0)))
{
instanceStruct.instanceVar_B4SOIbodyMod = 2;
}
else
{
instanceStruct.instanceVar_B4SOIbodyMod = 1;
}
}
else
{
if (((instanceStruct.instanceVar_B4SOIrbody==0.0)&&(instanceStruct.instanceVar_B4SOIrbsh==0.0)))
{
UserInfo(theInstance)  << "Warning: model parameter rbody=0!" << " " <<  std::endl; 
instanceStruct.d_instanceVar_B4SOIrbody_dX = 0.0;
instanceStruct.instanceVar_B4SOIrbody = 1e0;
instanceStruct.instanceVar_B4SOIbodyMod = 1;
}
else
{
instanceStruct.instanceVar_B4SOIbodyMod = 1;
}
}
}
}
if (modelStruct.modelPar_given_CF)
{
d_B4SOIcf_dX = modelStruct.d_modelPar_CF_dX;
B4SOIcf = modelStruct.modelPar_CF;
}
else
{
{
double value_log_0 = log((1.0+(0.4e-6/instanceStruct.instanceVar_B4SOItox)));
double  deriv_log_0_d0 = (1.0/(1.0+(0.4e-6/instanceStruct.instanceVar_B4SOItox)));
d_B4SOIcf_dX = ((2.0*3.453133e-11)/3.14159265358979323846)*deriv_log_0_d0*((-0.4e-6*instanceStruct.d_instanceVar_B4SOItox_dX/instanceStruct.instanceVar_B4SOItox/instanceStruct.instanceVar_B4SOItox));
B4SOIcf = (((2.0*3.453133e-11)/3.14159265358979323846)*value_log_0);
}
}
if (modelStruct.modelPar_given_CGDO)
{
d_B4SOIcgdo_dX = modelStruct.d_modelPar_CGDO_dX;
B4SOIcgdo = modelStruct.modelPar_CGDO;
}
else
{
if ((modelStruct.modelPar_given_DLC&&(B4SOIdlc>0.0)))
{
d_B4SOIcgdo_dX = ((B4SOIdlc*instanceStruct.d_instanceVar_B4SOIcox_dX+d_B4SOIdlc_dX*instanceStruct.instanceVar_B4SOIcox)-d_B4SOIcgdl_dX);
B4SOIcgdo = ((B4SOIdlc*instanceStruct.instanceVar_B4SOIcox)-B4SOIcgdl);
}
else
{
d_B4SOIcgdo_dX = ((0.6*B4SOIxj)*instanceStruct.d_instanceVar_B4SOIcox_dX+0.6*d_B4SOIxj_dX*instanceStruct.instanceVar_B4SOIcox);
B4SOIcgdo = ((0.6*B4SOIxj)*instanceStruct.instanceVar_B4SOIcox);
}
}
if (modelStruct.modelPar_given_CGSO)
{
d_B4SOIcgso_dX = modelStruct.d_modelPar_CGSO_dX;
B4SOIcgso = modelStruct.modelPar_CGSO;
}
else
{
if ((modelStruct.modelPar_given_DLC&&(B4SOIdlc>0.0)))
{
d_B4SOIcgso_dX = ((B4SOIdlc*instanceStruct.d_instanceVar_B4SOIcox_dX+d_B4SOIdlc_dX*instanceStruct.instanceVar_B4SOIcox)-d_B4SOIcgsl_dX);
B4SOIcgso = ((B4SOIdlc*instanceStruct.instanceVar_B4SOIcox)-B4SOIcgsl);
}
else
{
d_B4SOIcgso_dX = ((0.6*B4SOIxj)*instanceStruct.d_instanceVar_B4SOIcox_dX+0.6*d_B4SOIxj_dX*instanceStruct.instanceVar_B4SOIcox);
B4SOIcgso = ((0.6*B4SOIxj)*instanceStruct.instanceVar_B4SOIcox);
}
}
//Begin block b4soi_temp
{
//Block-local variables for block b4soi_temp
double tmp=0.0;
double d_tmp_dX=0.0;
double tmp1=0.0;
double d_tmp1_dX=0.0;
double tmp2=0.0;
double d_tmp2_dX=0.0;
double Eg=0.0;
double d_Eg_dX=0.0;
double Eg0=0.0;
double d_Eg0_dX=0.0;
double ni=0.0;
double d_ni_dX=0.0;
double T0=0.0;
double d_T0_dX=0.0;
double T1=0.0;
double d_T1_dX=0.0;
double T2=0.0;
double d_T2_dX=0.0;
double T3=0.0;
double d_T3_dX=0.0;
double T4=0.0;
double d_T4_dX=0.0;
double T5=0.0;
double d_T5_dX=0.0;
double T6=0.0;
double d_T6_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Ldrn=0.0;
double d_Ldrn_dX=0.0;
double Wdrn=0.0;
double d_Wdrn_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double TempRatio=0.0;
double d_TempRatio_dX=0.0;
double Inv_L=0.0;
double d_Inv_L_dX=0.0;
double Inv_W=0.0;
double d_Inv_W_dX=0.0;
double Inv_LW=0.0;
double d_Inv_LW_dX=0.0;
double Vtm0=0.0;
double d_Vtm0_dX=0.0;
double Tnom=0.0;
double d_Tnom_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double SDphi=0.0;
double d_SDphi_dX=0.0;
double SDgamma=0.0;
double d_SDgamma_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Inv_saref=0.0;
double d_Inv_saref_dX=0.0;
double Inv_sbref=0.0;
double d_Inv_sbref_dX=0.0;
double Inv_sa=0.0;
double d_Inv_sa_dX=0.0;
double Inv_sb=0.0;
double d_Inv_sb_dX=0.0;
double rho=0.0;
double d_rho_dX=0.0;
double dvth0_lod=0.0;
double d_dvth0_lod_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double W_tmp=0.0;
double d_W_tmp_dX=0.0;
double Inv_ODeff=0.0;
double d_Inv_ODeff_dX=0.0;
double OD_offset=0.0;
double d_OD_offset_dX=0.0;
double dk2_lod=0.0;
double d_dk2_lod_dX=0.0;
double deta0_lod=0.0;
double d_deta0_lod_dX=0.0;
double deta0cv_lod=0.0;
double d_deta0cv_lod_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
int i=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double PowWeffWr=0.0;
double d_PowWeffWr_dX=0.0;
double T10=0.0;
double d_T10_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Vtm0eot=0.0;
double d_Vtm0eot_dX=0.0;
double Vtmeot=0.0;
double d_Vtmeot_dX=0.0;
double vbieot=0.0;
double d_vbieot_dX=0.0;
double phieot=0.0;
double d_phieot_dX=0.0;
double sqrtphieot=0.0;
double d_sqrtphieot_dX=0.0;
double vddeot=0.0;
double d_vddeot_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Vgs_eff=0.0;
double d_Vgs_eff_dX=0.0;
double Vgsteff=0.0;
double d_Vgsteff_dX=0.0;
double V0=0.0;
double d_V0_dX=0.0;
double Vth=0.0;
double d_Vth_dX=0.0;
double Vgst=0.0;
double d_Vgst_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double lt1=0.0;
double d_lt1_dX=0.0;
double ltw=0.0;
double d_ltw_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double TempRatioMinus1=0.0;
double d_TempRatioMinus1_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double n=0.0;
double d_n_dX=0.0;
double VgstNVt=0.0;
double d_VgstNVt_dX=0.0;
double ExpArg=0.0;
double d_ExpArg_dX=0.0;
double sqrtPhisExt=0.0;
double d_sqrtPhisExt_dX=0.0;
double ExpVgst=0.0;
double d_ExpVgst_dX=0.0;
double vtfbphi2eot=0.0;
double d_vtfbphi2eot_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double DIBL_Sft=0.0;
double Lpe_Vb=0.0;
double d_Lpe_Vb_dX=0.0;
double DITS_Sft=0.0;
double d_DITS_Sft_dX=0.0;
double DeltVthtemp=0.0;
double d_DeltVthtemp_dX=0.0;
double DITS_Sft2=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double Theta0=0.0;
double d_Theta0_dX=0.0;
double Delt_vth=0.0;
double d_Delt_vth_dX=0.0;
double DeltVthw=0.0;
double d_DeltVthw_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double niter=0.0;
double toxpf=0.0;
double d_toxpf_dX=0.0;
double toxpi=0.0;
double d_toxpi_dX=0.0;
double Tcen=0.0;
double d_Tcen_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double n0=0.0;
double d_n0_dX=0.0;
//End of Block-local variables
//Block-local variables for block b4soi_temp
double tmp3=0.0;
double d_tmp3_dX=0.0;
double T7=0.0;
double d_T7_dX=0.0;
//End of Block-local variables
if ((instanceStruct.instanceVar_B4SOIGatesidewallJctSPotential<0.1))
{
instanceStruct.d_instanceVar_B4SOIGatesidewallJctSPotential_dX = 0.0;
instanceStruct.instanceVar_B4SOIGatesidewallJctSPotential = 0.1;
}
if ((instanceStruct.instanceVar_B4SOIGatesidewallJctDPotential<0.1))
{
instanceStruct.d_instanceVar_B4SOIGatesidewallJctDPotential_dX = 0.0;
instanceStruct.instanceVar_B4SOIGatesidewallJctDPotential = 0.1;
}
d_Tnom_dX = instanceStruct.d_instanceVar_B4SOItnom_dX;
Tnom = instanceStruct.instanceVar_B4SOItnom;
d_TempRatio_dX = ((Tnom*instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.instanceVar_DevTemp*d_Tnom_dX)/Tnom/Tnom);
TempRatio = (instanceStruct.instanceVar_DevTemp/Tnom);
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
{
double value_sqrt_0 = sqrt(((instanceStruct.instanceVar_epssub/(instanceStruct.instanceVar_epsrox*8.85418e-12))*instanceStruct.instanceVar_toxe));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_B4SOIfactor1_dX = deriv_sqrt_0_d0*(((instanceStruct.instanceVar_epssub/(instanceStruct.instanceVar_epsrox*8.85418e-12))*instanceStruct.d_instanceVar_toxe_dX+(((instanceStruct.instanceVar_epsrox*8.85418e-12)*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*instanceStruct.d_instanceVar_epsrox_dX*8.85418e-12)/(instanceStruct.instanceVar_epsrox*8.85418e-12)/(instanceStruct.instanceVar_epsrox*8.85418e-12))*instanceStruct.instanceVar_toxe));
instanceStruct.instanceVar_B4SOIfactor1 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((1.03594e-10/3.453133e-11)*instanceStruct.instanceVar_B4SOItox));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_B4SOIfactor1_dX = deriv_sqrt_0_d0*((1.03594e-10/3.453133e-11)*instanceStruct.d_instanceVar_B4SOItox_dX);
instanceStruct.instanceVar_B4SOIfactor1 = value_sqrt_0;
}
}
instanceStruct.d_instanceVar_Vtm00_dX = d_B4SOIVTM00_dX;
instanceStruct.instanceVar_Vtm00 = B4SOIVTM00;
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_Vtm0_dX = 8.617087e-5*d_Tnom_dX;
Vtm0 = (8.617087e-5*Tnom);
d_Eg0_dX = (-(((Tnom+1108.0)*((7.02e-4*Tnom)*d_Tnom_dX+7.02e-4*d_Tnom_dX*Tnom)-((7.02e-4*Tnom)*Tnom)*d_Tnom_dX)/(Tnom+1108.0)/(Tnom+1108.0)));
Eg0 = (1.16-(((7.02e-4*Tnom)*Tnom)/(Tnom+1108.0)));
instanceStruct.d_instanceVar_B4SOIeg0_dX = d_Eg0_dX;
instanceStruct.instanceVar_B4SOIeg0 = Eg0;
instanceStruct.d_instanceVar_B4SOIvtm_dX = 8.617087e-5*instanceStruct.d_instanceVar_DevTemp_dX;
instanceStruct.instanceVar_B4SOIvtm = (8.617087e-5*instanceStruct.instanceVar_DevTemp);
d_Eg_dX = (-(((instanceStruct.instanceVar_DevTemp+1108.0)*((7.02e-4*instanceStruct.instanceVar_DevTemp)*instanceStruct.d_instanceVar_DevTemp_dX+7.02e-4*instanceStruct.d_instanceVar_DevTemp_dX*instanceStruct.instanceVar_DevTemp)-((7.02e-4*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)*instanceStruct.d_instanceVar_DevTemp_dX)/(instanceStruct.instanceVar_DevTemp+1108.0)/(instanceStruct.instanceVar_DevTemp+1108.0)));
Eg = (1.16-(((7.02e-4*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)/(instanceStruct.instanceVar_DevTemp+1108.0)));
instanceStruct.d_instanceVar_B4SOIeg_dX = d_Eg_dX;
instanceStruct.instanceVar_B4SOIeg = Eg;
{
double value_sqrt_0 = sqrt((instanceStruct.instanceVar_DevTemp/300.15));
double value_exp_1 = exp((21.5565981-(Eg/(2.0*instanceStruct.instanceVar_B4SOIvtm))));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
double  deriv_exp_1_d0 = value_exp_1;
d_ni_dX = (((1.45e10*(instanceStruct.instanceVar_DevTemp/300.15))*value_sqrt_0)*deriv_exp_1_d0*((-(((2.0*instanceStruct.instanceVar_B4SOIvtm)*d_Eg_dX-Eg*2.0*instanceStruct.d_instanceVar_B4SOIvtm_dX)/(2.0*instanceStruct.instanceVar_B4SOIvtm)/(2.0*instanceStruct.instanceVar_B4SOIvtm))))+((1.45e10*(instanceStruct.instanceVar_DevTemp/300.15))*deriv_sqrt_0_d0*((instanceStruct.d_instanceVar_DevTemp_dX/300.15))+1.45e10*(instanceStruct.d_instanceVar_DevTemp_dX/300.15)*value_sqrt_0)*value_exp_1);
ni = (((1.45e10*(instanceStruct.instanceVar_DevTemp/300.15))*value_sqrt_0)*value_exp_1);
}
}
else
{
d_Vtm0_dX = 8.617087e-5*d_Tnom_dX;
Vtm0 = (8.617087e-5*Tnom);
d_Eg0_dX = (instanceStruct.d_instanceVar_B4SOIbg0sub_dX-(((Tnom+instanceStruct.instanceVar_B4SOItbgbsub)*((instanceStruct.instanceVar_B4SOItbgasub*Tnom)*d_Tnom_dX+(instanceStruct.instanceVar_B4SOItbgasub*d_Tnom_dX+instanceStruct.d_instanceVar_B4SOItbgasub_dX*Tnom)*Tnom)-((instanceStruct.instanceVar_B4SOItbgasub*Tnom)*Tnom)*(d_Tnom_dX+instanceStruct.d_instanceVar_B4SOItbgbsub_dX))/(Tnom+instanceStruct.instanceVar_B4SOItbgbsub)/(Tnom+instanceStruct.instanceVar_B4SOItbgbsub)));
Eg0 = (instanceStruct.instanceVar_B4SOIbg0sub-(((instanceStruct.instanceVar_B4SOItbgasub*Tnom)*Tnom)/(Tnom+instanceStruct.instanceVar_B4SOItbgbsub)));
instanceStruct.d_instanceVar_B4SOIeg0_dX = d_Eg0_dX;
instanceStruct.instanceVar_B4SOIeg0 = Eg0;
instanceStruct.d_instanceVar_B4SOIvtm_dX = 8.617087e-5*instanceStruct.d_instanceVar_DevTemp_dX;
instanceStruct.instanceVar_B4SOIvtm = (8.617087e-5*instanceStruct.instanceVar_DevTemp);
d_Eg_dX = (instanceStruct.d_instanceVar_B4SOIbg0sub_dX-(((instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)*((instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.instanceVar_DevTemp)*instanceStruct.d_instanceVar_DevTemp_dX+(instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.d_instanceVar_DevTemp_dX+instanceStruct.d_instanceVar_B4SOItbgasub_dX*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)-((instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)*(instanceStruct.d_instanceVar_DevTemp_dX+instanceStruct.d_instanceVar_B4SOItbgbsub_dX))/(instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)/(instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)));
Eg = (instanceStruct.instanceVar_B4SOIbg0sub-(((instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)/(instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)));
instanceStruct.d_instanceVar_B4SOIeg_dX = d_Eg_dX;
instanceStruct.instanceVar_B4SOIeg = Eg;
{
double value_sqrt_0 = sqrt((instanceStruct.instanceVar_DevTemp/Tnom));
double value_exp_1 = exp(((Eg0/(2.0*Vtm0))-(Eg/(2.0*instanceStruct.instanceVar_B4SOIvtm))));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
double  deriv_exp_1_d0 = value_exp_1;
d_ni_dX = (((instanceStruct.instanceVar_B4SOIni0sub*(instanceStruct.instanceVar_DevTemp/Tnom))*value_sqrt_0)*deriv_exp_1_d0*(((((2.0*Vtm0)*d_Eg0_dX-Eg0*2.0*d_Vtm0_dX)/(2.0*Vtm0)/(2.0*Vtm0))-(((2.0*instanceStruct.instanceVar_B4SOIvtm)*d_Eg_dX-Eg*2.0*instanceStruct.d_instanceVar_B4SOIvtm_dX)/(2.0*instanceStruct.instanceVar_B4SOIvtm)/(2.0*instanceStruct.instanceVar_B4SOIvtm))))+((instanceStruct.instanceVar_B4SOIni0sub*(instanceStruct.instanceVar_DevTemp/Tnom))*deriv_sqrt_0_d0*(((Tnom*instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.instanceVar_DevTemp*d_Tnom_dX)/Tnom/Tnom))+(instanceStruct.instanceVar_B4SOIni0sub*((Tnom*instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.instanceVar_DevTemp*d_Tnom_dX)/Tnom/Tnom)+instanceStruct.d_instanceVar_B4SOIni0sub_dX*(instanceStruct.instanceVar_DevTemp/Tnom))*value_sqrt_0)*value_exp_1);
ni = (((instanceStruct.instanceVar_B4SOIni0sub*(instanceStruct.instanceVar_DevTemp/Tnom))*value_sqrt_0)*value_exp_1);
}
}
instanceStruct.d_instanceVar_B4SOIrbodyext_dX = (B4SOIbodySquares*instanceStruct.d_instanceVar_B4SOIrbsh_dX+d_B4SOIbodySquares_dX*instanceStruct.instanceVar_B4SOIrbsh);
instanceStruct.instanceVar_B4SOIrbodyext = (B4SOIbodySquares*instanceStruct.instanceVar_B4SOIrbsh);
d_Ldrn_dX = d_B4SOIl_dX;
Ldrn = B4SOIl;
d_Wdrn_dX = ((instanceStruct.instanceVar_B4SOInf*d_B4SOIw_dX-B4SOIw*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf);
Wdrn = (B4SOIw/instanceStruct.instanceVar_B4SOInf);
{
double value_pow_0 = pow(Ldrn,B4SOILln);
double  deriv_pow_0_d0 = ((Ldrn == 0.0)?0.0:(value_pow_0*B4SOILln/Ldrn));
double  deriv_pow_0_d1 = (Ldrn == 0.0)?0.0:(log(Ldrn)*value_pow_0);
d_T0_dX = ((deriv_pow_0_d0*(d_Ldrn_dX))+(deriv_pow_0_d1*(d_B4SOILln_dX)));
T0 = value_pow_0;
}
{
double value_pow_0 = pow(Wdrn,B4SOILwn);
double  deriv_pow_0_d0 = ((Wdrn == 0.0)?0.0:(value_pow_0*B4SOILwn/Wdrn));
double  deriv_pow_0_d1 = (Wdrn == 0.0)?0.0:(log(Wdrn)*value_pow_0);
d_T1_dX = ((deriv_pow_0_d0*(d_Wdrn_dX))+(deriv_pow_0_d1*(d_B4SOILwn_dX)));
T1 = value_pow_0;
}
d_tmp1_dX = ((((T0*d_B4SOILl_dX-B4SOILl*d_T0_dX)/T0/T0)+((T1*d_B4SOILw_dX-B4SOILw*d_T1_dX)/T1/T1))+(((T0*T1)*d_B4SOILwl_dX-B4SOILwl*(T0*d_T1_dX+d_T0_dX*T1))/(T0*T1)/(T0*T1)));
tmp1 = (((B4SOILl/T0)+(B4SOILw/T1))+(B4SOILwl/(T0*T1)));
d_pParam_B4SOIdl_dX = (d_B4SOILint_dX+d_tmp1_dX);
pParam_B4SOIdl = (B4SOILint+tmp1);
d_tmp1_dX = ((((T0*d_B4SOILlc_dX-B4SOILlc*d_T0_dX)/T0/T0)+((T1*d_B4SOILwc_dX-B4SOILwc*d_T1_dX)/T1/T1))+(((T0*T1)*d_B4SOILwlc_dX-B4SOILwlc*(T0*d_T1_dX+d_T0_dX*T1))/(T0*T1)/(T0*T1)));
tmp1 = (((B4SOILlc/T0)+(B4SOILwc/T1))+(B4SOILwlc/(T0*T1)));
d_pParam_B4SOIdlc_dX = (d_B4SOIdlc_dX+d_tmp1_dX);
pParam_B4SOIdlc = (B4SOIdlc+tmp1);
d_pParam_B4SOIdlcig_dX = (d_B4SOIdlcig_dX+d_tmp1_dX);
pParam_B4SOIdlcig = (B4SOIdlcig+tmp1);
if ((pParam_B4SOIdlcig<0.0))
{
UserInfo(theInstance)  << "Warning: pParam_B4SOIdlcig = %g is negative. Set to zero." << " " << pParam_B4SOIdlcig << " " <<  std::endl; 
d_pParam_B4SOIdlcig_dX = 0.0;
pParam_B4SOIdlcig = 0;
}
{
double value_pow_0 = pow(Ldrn,B4SOIWln);
double  deriv_pow_0_d0 = ((Ldrn == 0.0)?0.0:(value_pow_0*B4SOIWln/Ldrn));
double  deriv_pow_0_d1 = (Ldrn == 0.0)?0.0:(log(Ldrn)*value_pow_0);
d_T2_dX = ((deriv_pow_0_d0*(d_Ldrn_dX))+(deriv_pow_0_d1*(d_B4SOIWln_dX)));
T2 = value_pow_0;
}
{
double value_pow_0 = pow(Wdrn,B4SOIWwn);
double  deriv_pow_0_d0 = ((Wdrn == 0.0)?0.0:(value_pow_0*B4SOIWwn/Wdrn));
double  deriv_pow_0_d1 = (Wdrn == 0.0)?0.0:(log(Wdrn)*value_pow_0);
d_T3_dX = ((deriv_pow_0_d0*(d_Wdrn_dX))+(deriv_pow_0_d1*(d_B4SOIWwn_dX)));
T3 = value_pow_0;
}
d_tmp2_dX = ((((T2*d_B4SOIWl_dX-B4SOIWl*d_T2_dX)/T2/T2)+((T3*d_B4SOIWw_dX-B4SOIWw*d_T3_dX)/T3/T3))+(((T2*T3)*d_B4SOIWwl_dX-B4SOIWwl*(T2*d_T3_dX+d_T2_dX*T3))/(T2*T3)/(T2*T3)));
tmp2 = (((B4SOIWl/T2)+(B4SOIWw/T3))+(B4SOIWwl/(T2*T3)));
d_pParam_B4SOIdw_dX = (d_B4SOIWint_dX+d_tmp2_dX);
pParam_B4SOIdw = (B4SOIWint+tmp2);
d_tmp2_dX = ((((T2*d_B4SOIWlc_dX-B4SOIWlc*d_T2_dX)/T2/T2)+((T3*d_B4SOIWwc_dX-B4SOIWwc*d_T3_dX)/T3/T3))+(((T2*T3)*d_B4SOIWwlc_dX-B4SOIWwlc*(T2*d_T3_dX+d_T2_dX*T3))/(T2*T3)/(T2*T3)));
tmp2 = (((B4SOIWlc/T2)+(B4SOIWwc/T3))+(B4SOIWwlc/(T2*T3)));
d_pParam_B4SOIdwc_dX = (d_B4SOIdwc_dX+d_tmp2_dX);
pParam_B4SOIdwc = (B4SOIdwc+tmp2);
instanceStruct.d_instanceVar_pParam_B4SOIleff_dX = (d_B4SOIl_dX-2.0*d_pParam_B4SOIdl_dX);
instanceStruct.instanceVar_pParam_B4SOIleff = (B4SOIl-(2.0*pParam_B4SOIdl));
if ((instanceStruct.instanceVar_pParam_B4SOIleff<=0.0))
{
UserInfo(theInstance)  << "Effective channel length <= 0" << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIweff_dX = ((((instanceStruct.instanceVar_B4SOInf*d_B4SOIw_dX-B4SOIw*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf)-(instanceStruct.instanceVar_B4SOInbc*instanceStruct.d_instanceVar_B4SOIdwbc_dX+instanceStruct.d_instanceVar_B4SOInbc_dX*instanceStruct.instanceVar_B4SOIdwbc))-((2.0-instanceStruct.instanceVar_B4SOInbc)*d_pParam_B4SOIdw_dX+(-instanceStruct.d_instanceVar_B4SOInbc_dX)*pParam_B4SOIdw));
instanceStruct.instanceVar_pParam_B4SOIweff = (((B4SOIw/instanceStruct.instanceVar_B4SOInf)-(instanceStruct.instanceVar_B4SOInbc*instanceStruct.instanceVar_B4SOIdwbc))-((2.0-instanceStruct.instanceVar_B4SOInbc)*pParam_B4SOIdw));
if ((instanceStruct.instanceVar_pParam_B4SOIweff<=0.0))
{
UserInfo(theInstance)  << "Effective channel width <= 0" << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIwdiod_dX = (((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-instanceStruct.instanceVar_pParam_B4SOIweff*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+d_B4SOIpdbcp_dX);
instanceStruct.instanceVar_pParam_B4SOIwdiod = ((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpdbcp);
instanceStruct.d_instanceVar_pParam_B4SOIwdios_dX = (((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-instanceStruct.instanceVar_pParam_B4SOIweff*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+d_B4SOIpsbcp_dX);
instanceStruct.instanceVar_pParam_B4SOIwdios = ((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpsbcp);
instanceStruct.d_instanceVar_pParam_B4SOIleffCV_dX = (d_B4SOIl_dX-2.0*d_pParam_B4SOIdlc_dX);
instanceStruct.instanceVar_pParam_B4SOIleffCV = (B4SOIl-(2.0*pParam_B4SOIdlc));
if ((instanceStruct.instanceVar_pParam_B4SOIleffCV<=0.0))
{
UserInfo(theInstance)  << "Effective channel length for C-V <= 0" << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX = ((((instanceStruct.instanceVar_B4SOInf*d_B4SOIw_dX-B4SOIw*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf)-(instanceStruct.instanceVar_B4SOInbc*instanceStruct.d_instanceVar_B4SOIdwbc_dX+instanceStruct.d_instanceVar_B4SOInbc_dX*instanceStruct.instanceVar_B4SOIdwbc))-((2.0-instanceStruct.instanceVar_B4SOInbc)*d_pParam_B4SOIdwc_dX+(-instanceStruct.d_instanceVar_B4SOInbc_dX)*pParam_B4SOIdwc));
instanceStruct.instanceVar_pParam_B4SOIweffCV = (((B4SOIw/instanceStruct.instanceVar_B4SOInf)-(instanceStruct.instanceVar_B4SOInbc*instanceStruct.instanceVar_B4SOIdwbc))-((2.0-instanceStruct.instanceVar_B4SOInbc)*pParam_B4SOIdwc));
if ((instanceStruct.instanceVar_pParam_B4SOIweffCV<=0.0))
{
UserInfo(theInstance)  << "Effective channel width for C-V <= 0" << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIwdiodCV_dX = (((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX-instanceStruct.instanceVar_pParam_B4SOIweffCV*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+d_B4SOIpdbcp_dX);
instanceStruct.instanceVar_pParam_B4SOIwdiodCV = ((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)+B4SOIpdbcp);
instanceStruct.d_instanceVar_pParam_B4SOIwdiosCV_dX = (((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX-instanceStruct.instanceVar_pParam_B4SOIweffCV*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+d_B4SOIpsbcp_dX);
instanceStruct.instanceVar_pParam_B4SOIwdiosCV = ((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)+B4SOIpsbcp);
instanceStruct.d_instanceVar_pParam_B4SOIleffCVb_dX = ((d_B4SOIl_dX-2.0*d_pParam_B4SOIdlc_dX)-d_B4SOIdlcb_dX);
instanceStruct.instanceVar_pParam_B4SOIleffCVb = ((B4SOIl-(2.0*pParam_B4SOIdlc))-B4SOIdlcb);
if ((instanceStruct.instanceVar_pParam_B4SOIleffCVb<=0.0))
{
UserInfo(theInstance)  << "Effective channel length for C-V (body) <= 0" << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIleffCVbg_dX = (instanceStruct.d_instanceVar_pParam_B4SOIleffCVb_dX+2*instanceStruct.d_instanceVar_B4SOIdlbg_dX);
instanceStruct.instanceVar_pParam_B4SOIleffCVbg = (instanceStruct.instanceVar_pParam_B4SOIleffCVb+(2*instanceStruct.instanceVar_B4SOIdlbg));
if ((instanceStruct.instanceVar_pParam_B4SOIleffCVbg<=0.0))
{
UserInfo(theInstance)  << "Effective channel length for C-V (backgate) <= 0" << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX = d_B4SOIgamma1_dX;
instanceStruct.instanceVar_pParam_B4SOIgamma1 = B4SOIgamma1;
instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX = d_B4SOIgamma2_dX;
instanceStruct.instanceVar_pParam_B4SOIgamma2 = B4SOIgamma2;
instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX = d_B4SOIvbx_dX;
instanceStruct.instanceVar_pParam_B4SOIvbx = B4SOIvbx;
instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX = d_B4SOIvbm_dX;
instanceStruct.instanceVar_pParam_B4SOIvbm = B4SOIvbm;
instanceStruct.d_instanceVar_pParam_B4SOIxt_dX = d_B4SOIxt_dX;
instanceStruct.instanceVar_pParam_B4SOIxt = B4SOIxt;
d_pParam_B4SOIcf_dX = d_B4SOIcf_dX;
pParam_B4SOIcf = B4SOIcf;
instanceStruct.d_instanceVar_pParam_B4SOIclc_dX = d_B4SOIclc_dX;
instanceStruct.instanceVar_pParam_B4SOIclc = B4SOIclc;
d_pParam_B4SOIcle_dX = d_B4SOIcle_dX;
pParam_B4SOIcle = B4SOIcle;
{
double value_pow_0 = pow((instanceStruct.instanceVar_pParam_B4SOIclc/instanceStruct.instanceVar_pParam_B4SOIleff),pParam_B4SOIcle);
double  deriv_pow_0_d0 = (((instanceStruct.instanceVar_pParam_B4SOIclc/instanceStruct.instanceVar_pParam_B4SOIleff) == 0.0)?0.0:(value_pow_0*pParam_B4SOIcle/(instanceStruct.instanceVar_pParam_B4SOIclc/instanceStruct.instanceVar_pParam_B4SOIleff)));
double  deriv_pow_0_d1 = ((instanceStruct.instanceVar_pParam_B4SOIclc/instanceStruct.instanceVar_pParam_B4SOIleff) == 0.0)?0.0:(log((instanceStruct.instanceVar_pParam_B4SOIclc/instanceStruct.instanceVar_pParam_B4SOIleff))*value_pow_0);
instanceStruct.d_instanceVar_pParam_B4SOIabulkCVfactor_dX = ((deriv_pow_0_d0*(((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIclc_dX-instanceStruct.instanceVar_pParam_B4SOIclc*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX)/instanceStruct.instanceVar_pParam_B4SOIleff/instanceStruct.instanceVar_pParam_B4SOIleff)))+(deriv_pow_0_d1*(d_pParam_B4SOIcle_dX)));
instanceStruct.instanceVar_pParam_B4SOIabulkCVfactor = (1.0+value_pow_0);
}
if ((B4SOIbinUnit==1))
{
d_Inv_L_dX = (-1.0e-6*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX/instanceStruct.instanceVar_pParam_B4SOIleff/instanceStruct.instanceVar_pParam_B4SOIleff);
Inv_L = (1.0e-6/instanceStruct.instanceVar_pParam_B4SOIleff);
d_Inv_W_dX = (-1.0e-6*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX/instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_pParam_B4SOIweff);
Inv_W = (1.0e-6/instanceStruct.instanceVar_pParam_B4SOIweff);
d_Inv_LW_dX = (-1.0e-12*(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIleff_dX*instanceStruct.instanceVar_pParam_B4SOIweff)/(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIweff)/(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIweff));
Inv_LW = (1.0e-12/(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIweff));
}
else
{
d_Inv_L_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIleff_dX/instanceStruct.instanceVar_pParam_B4SOIleff/instanceStruct.instanceVar_pParam_B4SOIleff);
Inv_L = (1.0/instanceStruct.instanceVar_pParam_B4SOIleff);
d_Inv_W_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIweff_dX/instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_pParam_B4SOIweff);
Inv_W = (1.0/instanceStruct.instanceVar_pParam_B4SOIweff);
d_Inv_LW_dX = (-(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIleff_dX*instanceStruct.instanceVar_pParam_B4SOIweff)/(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIweff)/(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIweff));
Inv_LW = (1.0/(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIweff));
}
instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX = (((d_B4SOInpeak_dX+(modelStruct.modelPar_LNCH*d_Inv_L_dX+modelStruct.d_modelPar_LNCH_dX*Inv_L))+(modelStruct.modelPar_WNCH*d_Inv_W_dX+modelStruct.d_modelPar_WNCH_dX*Inv_W))+(modelStruct.modelPar_PNCH*d_Inv_LW_dX+modelStruct.d_modelPar_PNCH_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInpeak = (((B4SOInpeak+(modelStruct.modelPar_LNCH*Inv_L))+(modelStruct.modelPar_WNCH*Inv_W))+(modelStruct.modelPar_PNCH*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInsub_dX = (((d_B4SOInsub_dX+(modelStruct.modelPar_LNSUB*d_Inv_L_dX+modelStruct.d_modelPar_LNSUB_dX*Inv_L))+(modelStruct.modelPar_WNSUB*d_Inv_W_dX+modelStruct.d_modelPar_WNSUB_dX*Inv_W))+(modelStruct.modelPar_PNSUB*d_Inv_LW_dX+modelStruct.d_modelPar_PNSUB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInsub = (((B4SOInsub+(modelStruct.modelPar_LNSUB*Inv_L))+(modelStruct.modelPar_WNSUB*Inv_W))+(modelStruct.modelPar_PNSUB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIngate_dX = (((instanceStruct.d_instanceVar_B4SOIngate_dX+(modelStruct.modelPar_LNGATE*d_Inv_L_dX+modelStruct.d_modelPar_LNGATE_dX*Inv_L))+(modelStruct.modelPar_WNGATE*d_Inv_W_dX+modelStruct.d_modelPar_WNGATE_dX*Inv_W))+(modelStruct.modelPar_PNGATE*d_Inv_LW_dX+modelStruct.d_modelPar_PNGATE_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIngate = (((instanceStruct.instanceVar_B4SOIngate+(modelStruct.modelPar_LNGATE*Inv_L))+(modelStruct.modelPar_WNGATE*Inv_W))+(modelStruct.modelPar_PNGATE*Inv_LW));
d_pParam_B4SOInsd_dX = (((instanceStruct.d_instanceVar_B4SOInsd_dX+(modelStruct.modelPar_LNSD*d_Inv_L_dX+modelStruct.d_modelPar_LNSD_dX*Inv_L))+(modelStruct.modelPar_WNSD*d_Inv_W_dX+modelStruct.d_modelPar_WNSD_dX*Inv_W))+(modelStruct.modelPar_PNSD*d_Inv_LW_dX+modelStruct.d_modelPar_PNSD_dX*Inv_LW));
pParam_B4SOInsd = (((instanceStruct.instanceVar_B4SOInsd+(modelStruct.modelPar_LNSD*Inv_L))+(modelStruct.modelPar_WNSD*Inv_W))+(modelStruct.modelPar_PNSD*Inv_LW));
d_pParam_B4SOIvth0_dX = (((d_B4SOIvth0_dX+(modelStruct.modelPar_LVTH0*d_Inv_L_dX+modelStruct.d_modelPar_LVTH0_dX*Inv_L))+(modelStruct.modelPar_WVTH0*d_Inv_W_dX+modelStruct.d_modelPar_WVTH0_dX*Inv_W))+(modelStruct.modelPar_PVTH0*d_Inv_LW_dX+modelStruct.d_modelPar_PVTH0_dX*Inv_LW));
pParam_B4SOIvth0 = (((B4SOIvth0+(modelStruct.modelPar_LVTH0*Inv_L))+(modelStruct.modelPar_WVTH0*Inv_W))+(modelStruct.modelPar_PVTH0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX = (((d_B4SOIvfb_dX+(modelStruct.modelPar_LVFB*d_Inv_L_dX+modelStruct.d_modelPar_LVFB_dX*Inv_L))+(modelStruct.modelPar_WVFB*d_Inv_W_dX+modelStruct.d_modelPar_WVFB_dX*Inv_W))+(modelStruct.modelPar_PVFB*d_Inv_LW_dX+modelStruct.d_modelPar_PVFB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvfb = (((B4SOIvfb+(modelStruct.modelPar_LVFB*Inv_L))+(modelStruct.modelPar_WVFB*Inv_W))+(modelStruct.modelPar_PVFB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk1_dX = (((d_B4SOIk1_dX+(modelStruct.modelPar_LK1*d_Inv_L_dX+modelStruct.d_modelPar_LK1_dX*Inv_L))+(modelStruct.modelPar_WK1*d_Inv_W_dX+modelStruct.d_modelPar_WK1_dX*Inv_W))+(modelStruct.modelPar_PK1*d_Inv_LW_dX+modelStruct.d_modelPar_PK1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk1 = (((B4SOIk1+(modelStruct.modelPar_LK1*Inv_L))+(modelStruct.modelPar_WK1*Inv_W))+(modelStruct.modelPar_PK1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk2_dX = (((d_B4SOIk2_dX+(modelStruct.modelPar_LK2*d_Inv_L_dX+modelStruct.d_modelPar_LK2_dX*Inv_L))+(modelStruct.modelPar_WK2*d_Inv_W_dX+modelStruct.d_modelPar_WK2_dX*Inv_W))+(modelStruct.modelPar_PK2*d_Inv_LW_dX+modelStruct.d_modelPar_PK2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk2 = (((B4SOIk2+(modelStruct.modelPar_LK2*Inv_L))+(modelStruct.modelPar_WK2*Inv_W))+(modelStruct.modelPar_PK2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk1w1_dX = (((instanceStruct.d_instanceVar_B4SOIk1w1_dX+(modelStruct.modelPar_LK1W1*d_Inv_L_dX+modelStruct.d_modelPar_LK1W1_dX*Inv_L))+(modelStruct.modelPar_WK1W1*d_Inv_W_dX+modelStruct.d_modelPar_WK1W1_dX*Inv_W))+(modelStruct.modelPar_PK1W1*d_Inv_LW_dX+modelStruct.d_modelPar_PK1W1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk1w1 = (((instanceStruct.instanceVar_B4SOIk1w1+(modelStruct.modelPar_LK1W1*Inv_L))+(modelStruct.modelPar_WK1W1*Inv_W))+(modelStruct.modelPar_PK1W1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk1w2_dX = (((instanceStruct.d_instanceVar_B4SOIk1w2_dX+(modelStruct.modelPar_LK1W2*d_Inv_L_dX+modelStruct.d_modelPar_LK1W2_dX*Inv_L))+(modelStruct.modelPar_WK1W2*d_Inv_W_dX+modelStruct.d_modelPar_WK1W2_dX*Inv_W))+(modelStruct.modelPar_PK1W2*d_Inv_LW_dX+modelStruct.d_modelPar_PK1W2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk1w2 = (((instanceStruct.instanceVar_B4SOIk1w2+(modelStruct.modelPar_LK1W2*Inv_L))+(modelStruct.modelPar_WK1W2*Inv_W))+(modelStruct.modelPar_PK1W2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk3_dX = (((d_B4SOIk3_dX+(modelStruct.modelPar_LK3*d_Inv_L_dX+modelStruct.d_modelPar_LK3_dX*Inv_L))+(modelStruct.modelPar_WK3*d_Inv_W_dX+modelStruct.d_modelPar_WK3_dX*Inv_W))+(modelStruct.modelPar_PK3*d_Inv_LW_dX+modelStruct.d_modelPar_PK3_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk3 = (((B4SOIk3+(modelStruct.modelPar_LK3*Inv_L))+(modelStruct.modelPar_WK3*Inv_W))+(modelStruct.modelPar_PK3*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk3b_dX = (((d_B4SOIk3b_dX+(modelStruct.modelPar_LK3B*d_Inv_L_dX+modelStruct.d_modelPar_LK3B_dX*Inv_L))+(modelStruct.modelPar_WK3B*d_Inv_W_dX+modelStruct.d_modelPar_WK3B_dX*Inv_W))+(modelStruct.modelPar_PK3B*d_Inv_LW_dX+modelStruct.d_modelPar_PK3B_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk3b = (((B4SOIk3b+(modelStruct.modelPar_LK3B*Inv_L))+(modelStruct.modelPar_WK3B*Inv_W))+(modelStruct.modelPar_PK3B*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIkb1_dX = (((d_B4SOIkb1_dX+(modelStruct.modelPar_LKB1*d_Inv_L_dX+modelStruct.d_modelPar_LKB1_dX*Inv_L))+(modelStruct.modelPar_WKB1*d_Inv_W_dX+modelStruct.d_modelPar_WKB1_dX*Inv_W))+(modelStruct.modelPar_PKB1*d_Inv_LW_dX+modelStruct.d_modelPar_PKB1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIkb1 = (((B4SOIkb1+(modelStruct.modelPar_LKB1*Inv_L))+(modelStruct.modelPar_WKB1*Inv_W))+(modelStruct.modelPar_PKB1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIw0_dX = (((d_B4SOIw0_dX+(modelStruct.modelPar_LW0*d_Inv_L_dX+modelStruct.d_modelPar_LW0_dX*Inv_L))+(modelStruct.modelPar_WW0*d_Inv_W_dX+modelStruct.d_modelPar_WW0_dX*Inv_W))+(modelStruct.modelPar_PW0*d_Inv_LW_dX+modelStruct.d_modelPar_PW0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIw0 = (((B4SOIw0+(modelStruct.modelPar_LW0*Inv_L))+(modelStruct.modelPar_WW0*Inv_W))+(modelStruct.modelPar_PW0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX = (((d_B4SOIlpe0_dX+(modelStruct.modelPar_LLPE0*d_Inv_L_dX+modelStruct.d_modelPar_LLPE0_dX*Inv_L))+(modelStruct.modelPar_WLPE0*d_Inv_W_dX+modelStruct.d_modelPar_WLPE0_dX*Inv_W))+(modelStruct.modelPar_PLPE0*d_Inv_LW_dX+modelStruct.d_modelPar_PLPE0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIlpe0 = (((B4SOIlpe0+(modelStruct.modelPar_LLPE0*Inv_L))+(modelStruct.modelPar_WLPE0*Inv_W))+(modelStruct.modelPar_PLPE0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIlpeb_dX = (((d_B4SOIlpeb_dX+(modelStruct.modelPar_LLPEB*d_Inv_L_dX+modelStruct.d_modelPar_LLPEB_dX*Inv_L))+(modelStruct.modelPar_WLPEB*d_Inv_W_dX+modelStruct.d_modelPar_WLPEB_dX*Inv_W))+(modelStruct.modelPar_PLPEB*d_Inv_LW_dX+modelStruct.d_modelPar_PLPEB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIlpeb = (((B4SOIlpeb+(modelStruct.modelPar_LLPEB*Inv_L))+(modelStruct.modelPar_WLPEB*Inv_W))+(modelStruct.modelPar_PLPEB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX = (((d_B4SOIdvt0_dX+(modelStruct.modelPar_LDVT0*d_Inv_L_dX+modelStruct.d_modelPar_LDVT0_dX*Inv_L))+(modelStruct.modelPar_WDVT0*d_Inv_W_dX+modelStruct.d_modelPar_WDVT0_dX*Inv_W))+(modelStruct.modelPar_PDVT0*d_Inv_LW_dX+modelStruct.d_modelPar_PDVT0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvt0 = (((B4SOIdvt0+(modelStruct.modelPar_LDVT0*Inv_L))+(modelStruct.modelPar_WDVT0*Inv_W))+(modelStruct.modelPar_PDVT0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX = (((d_B4SOIdvt1_dX+(modelStruct.modelPar_LDVT1*d_Inv_L_dX+modelStruct.d_modelPar_LDVT1_dX*Inv_L))+(modelStruct.modelPar_WDVT1*d_Inv_W_dX+modelStruct.d_modelPar_WDVT1_dX*Inv_W))+(modelStruct.modelPar_PDVT1*d_Inv_LW_dX+modelStruct.d_modelPar_PDVT1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvt1 = (((B4SOIdvt1+(modelStruct.modelPar_LDVT1*Inv_L))+(modelStruct.modelPar_WDVT1*Inv_W))+(modelStruct.modelPar_PDVT1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvt2_dX = (((d_B4SOIdvt2_dX+(modelStruct.modelPar_LDVT2*d_Inv_L_dX+modelStruct.d_modelPar_LDVT2_dX*Inv_L))+(modelStruct.modelPar_WDVT2*d_Inv_W_dX+modelStruct.d_modelPar_WDVT2_dX*Inv_W))+(modelStruct.modelPar_PDVT2*d_Inv_LW_dX+modelStruct.d_modelPar_PDVT2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvt2 = (((B4SOIdvt2+(modelStruct.modelPar_LDVT2*Inv_L))+(modelStruct.modelPar_WDVT2*Inv_W))+(modelStruct.modelPar_PDVT2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX = (((d_B4SOIdvt0w_dX+(modelStruct.modelPar_LDVT0W*d_Inv_L_dX+modelStruct.d_modelPar_LDVT0W_dX*Inv_L))+(modelStruct.modelPar_WDVT0W*d_Inv_W_dX+modelStruct.d_modelPar_WDVT0W_dX*Inv_W))+(modelStruct.modelPar_PDVT0W*d_Inv_LW_dX+modelStruct.d_modelPar_PDVT0W_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvt0w = (((B4SOIdvt0w+(modelStruct.modelPar_LDVT0W*Inv_L))+(modelStruct.modelPar_WDVT0W*Inv_W))+(modelStruct.modelPar_PDVT0W*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX = (((d_B4SOIdvt1w_dX+(modelStruct.modelPar_LDVT1W*d_Inv_L_dX+modelStruct.d_modelPar_LDVT1W_dX*Inv_L))+(modelStruct.modelPar_WDVT1W*d_Inv_W_dX+modelStruct.d_modelPar_WDVT1W_dX*Inv_W))+(modelStruct.modelPar_PDVT1W*d_Inv_LW_dX+modelStruct.d_modelPar_PDVT1W_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvt1w = (((B4SOIdvt1w+(modelStruct.modelPar_LDVT1W*Inv_L))+(modelStruct.modelPar_WDVT1W*Inv_W))+(modelStruct.modelPar_PDVT1W*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvt2w_dX = (((d_B4SOIdvt2w_dX+(modelStruct.modelPar_LDVT2W*d_Inv_L_dX+modelStruct.d_modelPar_LDVT2W_dX*Inv_L))+(modelStruct.modelPar_WDVT2W*d_Inv_W_dX+modelStruct.d_modelPar_WDVT2W_dX*Inv_W))+(modelStruct.modelPar_PDVT2W*d_Inv_LW_dX+modelStruct.d_modelPar_PDVT2W_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvt2w = (((B4SOIdvt2w+(modelStruct.modelPar_LDVT2W*Inv_L))+(modelStruct.modelPar_WDVT2W*Inv_W))+(modelStruct.modelPar_PDVT2W*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIu0_dX = (((d_B4SOIu0_dX+(modelStruct.modelPar_LU0*d_Inv_L_dX+modelStruct.d_modelPar_LU0_dX*Inv_L))+(modelStruct.modelPar_WU0*d_Inv_W_dX+modelStruct.d_modelPar_WU0_dX*Inv_W))+(modelStruct.modelPar_PU0*d_Inv_LW_dX+modelStruct.d_modelPar_PU0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIu0 = (((B4SOIu0+(modelStruct.modelPar_LU0*Inv_L))+(modelStruct.modelPar_WU0*Inv_W))+(modelStruct.modelPar_PU0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIua_dX = (((d_B4SOIua_dX+(modelStruct.modelPar_LUA*d_Inv_L_dX+modelStruct.d_modelPar_LUA_dX*Inv_L))+(modelStruct.modelPar_WUA*d_Inv_W_dX+modelStruct.d_modelPar_WUA_dX*Inv_W))+(modelStruct.modelPar_PUA*d_Inv_LW_dX+modelStruct.d_modelPar_PUA_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIua = (((B4SOIua+(modelStruct.modelPar_LUA*Inv_L))+(modelStruct.modelPar_WUA*Inv_W))+(modelStruct.modelPar_PUA*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIub_dX = (((d_B4SOIub_dX+(modelStruct.modelPar_LUB*d_Inv_L_dX+modelStruct.d_modelPar_LUB_dX*Inv_L))+(modelStruct.modelPar_WUB*d_Inv_W_dX+modelStruct.d_modelPar_WUB_dX*Inv_W))+(modelStruct.modelPar_PUB*d_Inv_LW_dX+modelStruct.d_modelPar_PUB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIub = (((B4SOIub+(modelStruct.modelPar_LUB*Inv_L))+(modelStruct.modelPar_WUB*Inv_W))+(modelStruct.modelPar_PUB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIuc_dX = (((d_B4SOIuc_dX+(modelStruct.modelPar_LUC*d_Inv_L_dX+modelStruct.d_modelPar_LUC_dX*Inv_L))+(modelStruct.modelPar_WUC*d_Inv_W_dX+modelStruct.d_modelPar_WUC_dX*Inv_W))+(modelStruct.modelPar_PUC*d_Inv_LW_dX+modelStruct.d_modelPar_PUC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIuc = (((B4SOIuc+(modelStruct.modelPar_LUC*Inv_L))+(modelStruct.modelPar_WUC*Inv_W))+(modelStruct.modelPar_PUC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvsat_dX = (((d_B4SOIvsat_dX+(modelStruct.modelPar_LVSAT*d_Inv_L_dX+modelStruct.d_modelPar_LVSAT_dX*Inv_L))+(modelStruct.modelPar_WVSAT*d_Inv_W_dX+modelStruct.d_modelPar_WVSAT_dX*Inv_W))+(modelStruct.modelPar_PVSAT*d_Inv_LW_dX+modelStruct.d_modelPar_PVSAT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvsat = (((B4SOIvsat+(modelStruct.modelPar_LVSAT*Inv_L))+(modelStruct.modelPar_WVSAT*Inv_W))+(modelStruct.modelPar_PVSAT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIa0_dX = (((d_B4SOIa0_dX+(modelStruct.modelPar_LA0*d_Inv_L_dX+modelStruct.d_modelPar_LA0_dX*Inv_L))+(modelStruct.modelPar_WA0*d_Inv_W_dX+modelStruct.d_modelPar_WA0_dX*Inv_W))+(modelStruct.modelPar_PA0*d_Inv_LW_dX+modelStruct.d_modelPar_PA0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIa0 = (((B4SOIa0+(modelStruct.modelPar_LA0*Inv_L))+(modelStruct.modelPar_WA0*Inv_W))+(modelStruct.modelPar_PA0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIags_dX = (((d_B4SOIags_dX+(modelStruct.modelPar_LAGS*d_Inv_L_dX+modelStruct.d_modelPar_LAGS_dX*Inv_L))+(modelStruct.modelPar_WAGS*d_Inv_W_dX+modelStruct.d_modelPar_WAGS_dX*Inv_W))+(modelStruct.modelPar_PAGS*d_Inv_LW_dX+modelStruct.d_modelPar_PAGS_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIags = (((B4SOIags+(modelStruct.modelPar_LAGS*Inv_L))+(modelStruct.modelPar_WAGS*Inv_W))+(modelStruct.modelPar_PAGS*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIb0_dX = (((d_B4SOIb0_dX+(modelStruct.modelPar_LB0*d_Inv_L_dX+modelStruct.d_modelPar_LB0_dX*Inv_L))+(modelStruct.modelPar_WB0*d_Inv_W_dX+modelStruct.d_modelPar_WB0_dX*Inv_W))+(modelStruct.modelPar_PB0*d_Inv_LW_dX+modelStruct.d_modelPar_PB0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIb0 = (((B4SOIb0+(modelStruct.modelPar_LB0*Inv_L))+(modelStruct.modelPar_WB0*Inv_W))+(modelStruct.modelPar_PB0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIb1_dX = (((d_B4SOIb1_dX+(modelStruct.modelPar_LB1*d_Inv_L_dX+modelStruct.d_modelPar_LB1_dX*Inv_L))+(modelStruct.modelPar_WB1*d_Inv_W_dX+modelStruct.d_modelPar_WB1_dX*Inv_W))+(modelStruct.modelPar_PB1*d_Inv_LW_dX+modelStruct.d_modelPar_PB1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIb1 = (((B4SOIb1+(modelStruct.modelPar_LB1*Inv_L))+(modelStruct.modelPar_WB1*Inv_W))+(modelStruct.modelPar_PB1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIketa_dX = (((d_B4SOIketa_dX+(modelStruct.modelPar_LKETA*d_Inv_L_dX+modelStruct.d_modelPar_LKETA_dX*Inv_L))+(modelStruct.modelPar_WKETA*d_Inv_W_dX+modelStruct.d_modelPar_WKETA_dX*Inv_W))+(modelStruct.modelPar_PKETA*d_Inv_LW_dX+modelStruct.d_modelPar_PKETA_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIketa = (((B4SOIketa+(modelStruct.modelPar_LKETA*Inv_L))+(modelStruct.modelPar_WKETA*Inv_W))+(modelStruct.modelPar_PKETA*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIketas_dX = (((instanceStruct.d_instanceVar_B4SOIketas_dX+(modelStruct.modelPar_LKETAS*d_Inv_L_dX+modelStruct.d_modelPar_LKETAS_dX*Inv_L))+(modelStruct.modelPar_WKETAS*d_Inv_W_dX+modelStruct.d_modelPar_WKETAS_dX*Inv_W))+(modelStruct.modelPar_PKETAS*d_Inv_LW_dX+modelStruct.d_modelPar_PKETAS_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIketas = (((instanceStruct.instanceVar_B4SOIketas+(modelStruct.modelPar_LKETAS*Inv_L))+(modelStruct.modelPar_WKETAS*Inv_W))+(modelStruct.modelPar_PKETAS*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIa1_dX = (((d_B4SOIa1_dX+(modelStruct.modelPar_LA1*d_Inv_L_dX+modelStruct.d_modelPar_LA1_dX*Inv_L))+(modelStruct.modelPar_WA1*d_Inv_W_dX+modelStruct.d_modelPar_WA1_dX*Inv_W))+(modelStruct.modelPar_PA1*d_Inv_LW_dX+modelStruct.d_modelPar_PA1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIa1 = (((B4SOIa1+(modelStruct.modelPar_LA1*Inv_L))+(modelStruct.modelPar_WA1*Inv_W))+(modelStruct.modelPar_PA1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIa2_dX = (((d_B4SOIa2_dX+(modelStruct.modelPar_LA2*d_Inv_L_dX+modelStruct.d_modelPar_LA2_dX*Inv_L))+(modelStruct.modelPar_WA2*d_Inv_W_dX+modelStruct.d_modelPar_WA2_dX*Inv_W))+(modelStruct.modelPar_PA2*d_Inv_LW_dX+modelStruct.d_modelPar_PA2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIa2 = (((B4SOIa2+(modelStruct.modelPar_LA2*Inv_L))+(modelStruct.modelPar_WA2*Inv_W))+(modelStruct.modelPar_PA2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIrdsw_dX = (((d_B4SOIrdsw_dX+(modelStruct.modelPar_LRDSW*d_Inv_L_dX+modelStruct.d_modelPar_LRDSW_dX*Inv_L))+(modelStruct.modelPar_WRDSW*d_Inv_W_dX+modelStruct.d_modelPar_WRDSW_dX*Inv_W))+(modelStruct.modelPar_PRDSW*d_Inv_LW_dX+modelStruct.d_modelPar_PRDSW_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIrdsw = (((B4SOIrdsw+(modelStruct.modelPar_LRDSW*Inv_L))+(modelStruct.modelPar_WRDSW*Inv_W))+(modelStruct.modelPar_PRDSW*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIrsw_dX = (((d_B4SOIrsw_dX+(modelStruct.modelPar_LRSW*d_Inv_L_dX+modelStruct.d_modelPar_LRSW_dX*Inv_L))+(modelStruct.modelPar_WRSW*d_Inv_W_dX+modelStruct.d_modelPar_WRSW_dX*Inv_W))+(modelStruct.modelPar_PRSW*d_Inv_LW_dX+modelStruct.d_modelPar_PRSW_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIrsw = (((B4SOIrsw+(modelStruct.modelPar_LRSW*Inv_L))+(modelStruct.modelPar_WRSW*Inv_W))+(modelStruct.modelPar_PRSW*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIrdw_dX = (((d_B4SOIrdw_dX+(modelStruct.modelPar_LRDW*d_Inv_L_dX+modelStruct.d_modelPar_LRDW_dX*Inv_L))+(modelStruct.modelPar_WRDW*d_Inv_W_dX+modelStruct.d_modelPar_WRDW_dX*Inv_W))+(modelStruct.modelPar_PRDW*d_Inv_LW_dX+modelStruct.d_modelPar_PRDW_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIrdw = (((B4SOIrdw+(modelStruct.modelPar_LRDW*Inv_L))+(modelStruct.modelPar_WRDW*Inv_W))+(modelStruct.modelPar_PRDW*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIprwb_dX = (((d_B4SOIprwb_dX+(modelStruct.modelPar_LPRWB*d_Inv_L_dX+modelStruct.d_modelPar_LPRWB_dX*Inv_L))+(modelStruct.modelPar_WPRWB*d_Inv_W_dX+modelStruct.d_modelPar_WPRWB_dX*Inv_W))+(modelStruct.modelPar_PPRWB*d_Inv_LW_dX+modelStruct.d_modelPar_PPRWB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIprwb = (((B4SOIprwb+(modelStruct.modelPar_LPRWB*Inv_L))+(modelStruct.modelPar_WPRWB*Inv_W))+(modelStruct.modelPar_PPRWB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIprwg_dX = (((d_B4SOIprwg_dX+(modelStruct.modelPar_LPRWG*d_Inv_L_dX+modelStruct.d_modelPar_LPRWG_dX*Inv_L))+(modelStruct.modelPar_WPRWG*d_Inv_W_dX+modelStruct.d_modelPar_WPRWG_dX*Inv_W))+(modelStruct.modelPar_PPRWG*d_Inv_LW_dX+modelStruct.d_modelPar_PPRWG_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIprwg = (((B4SOIprwg+(modelStruct.modelPar_LPRWG*Inv_L))+(modelStruct.modelPar_WPRWG*Inv_W))+(modelStruct.modelPar_PPRWG*Inv_LW));
d_pParam_B4SOIwr_dX = (((d_B4SOIwr_dX+(modelStruct.modelPar_LWR*d_Inv_L_dX+modelStruct.d_modelPar_LWR_dX*Inv_L))+(modelStruct.modelPar_WWR*d_Inv_W_dX+modelStruct.d_modelPar_WWR_dX*Inv_W))+(modelStruct.modelPar_PWR*d_Inv_LW_dX+modelStruct.d_modelPar_PWR_dX*Inv_LW));
pParam_B4SOIwr = (((B4SOIwr+(modelStruct.modelPar_LWR*Inv_L))+(modelStruct.modelPar_WWR*Inv_W))+(modelStruct.modelPar_PWR*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX = (((d_B4SOInfactor_dX+(modelStruct.modelPar_LNFACTOR*d_Inv_L_dX+modelStruct.d_modelPar_LNFACTOR_dX*Inv_L))+(modelStruct.modelPar_WNFACTOR*d_Inv_W_dX+modelStruct.d_modelPar_WNFACTOR_dX*Inv_W))+(modelStruct.modelPar_PNFACTOR*d_Inv_LW_dX+modelStruct.d_modelPar_PNFACTOR_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInfactor = (((B4SOInfactor+(modelStruct.modelPar_LNFACTOR*Inv_L))+(modelStruct.modelPar_WNFACTOR*Inv_W))+(modelStruct.modelPar_PNFACTOR*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdwg_dX = (((d_B4SOIdwg_dX+(modelStruct.modelPar_LDWG*d_Inv_L_dX+modelStruct.d_modelPar_LDWG_dX*Inv_L))+(modelStruct.modelPar_WDWG*d_Inv_W_dX+modelStruct.d_modelPar_WDWG_dX*Inv_W))+(modelStruct.modelPar_PDWG*d_Inv_LW_dX+modelStruct.d_modelPar_PDWG_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdwg = (((B4SOIdwg+(modelStruct.modelPar_LDWG*Inv_L))+(modelStruct.modelPar_WDWG*Inv_W))+(modelStruct.modelPar_PDWG*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdwb_dX = (((d_B4SOIdwb_dX+(modelStruct.modelPar_LDWB*d_Inv_L_dX+modelStruct.d_modelPar_LDWB_dX*Inv_L))+(modelStruct.modelPar_WDWB*d_Inv_W_dX+modelStruct.d_modelPar_WDWB_dX*Inv_W))+(modelStruct.modelPar_PDWB*d_Inv_LW_dX+modelStruct.d_modelPar_PDWB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdwb = (((B4SOIdwb+(modelStruct.modelPar_LDWB*Inv_L))+(modelStruct.modelPar_WDWB*Inv_W))+(modelStruct.modelPar_PDWB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX = (((d_B4SOIvoff_dX+(modelStruct.modelPar_LVOFF*d_Inv_L_dX+modelStruct.d_modelPar_LVOFF_dX*Inv_L))+(modelStruct.modelPar_WVOFF*d_Inv_W_dX+modelStruct.d_modelPar_WVOFF_dX*Inv_W))+(modelStruct.modelPar_PVOFF*d_Inv_LW_dX+modelStruct.d_modelPar_PVOFF_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvoff = (((B4SOIvoff+(modelStruct.modelPar_LVOFF*Inv_L))+(modelStruct.modelPar_WVOFF*Inv_W))+(modelStruct.modelPar_PVOFF*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIeta0_dX = (((d_B4SOIeta0_dX+(modelStruct.modelPar_LETA0*d_Inv_L_dX+modelStruct.d_modelPar_LETA0_dX*Inv_L))+(modelStruct.modelPar_WETA0*d_Inv_W_dX+modelStruct.d_modelPar_WETA0_dX*Inv_W))+(modelStruct.modelPar_PETA0*d_Inv_LW_dX+modelStruct.d_modelPar_PETA0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIeta0 = (((B4SOIeta0+(modelStruct.modelPar_LETA0*Inv_L))+(modelStruct.modelPar_WETA0*Inv_W))+(modelStruct.modelPar_PETA0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIetab_dX = (((d_B4SOIetab_dX+(modelStruct.modelPar_LETAB*d_Inv_L_dX+modelStruct.d_modelPar_LETAB_dX*Inv_L))+(modelStruct.modelPar_WETAB*d_Inv_W_dX+modelStruct.d_modelPar_WETAB_dX*Inv_W))+(modelStruct.modelPar_PETAB*d_Inv_LW_dX+modelStruct.d_modelPar_PETAB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIetab = (((B4SOIetab+(modelStruct.modelPar_LETAB*Inv_L))+(modelStruct.modelPar_WETAB*Inv_W))+(modelStruct.modelPar_PETAB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIeta0cv_dX = (((d_B4SOIeta0cv_dX+(modelStruct.modelPar_LETA0CV*d_Inv_L_dX+modelStruct.d_modelPar_LETA0CV_dX*Inv_L))+(modelStruct.modelPar_WETA0CV*d_Inv_W_dX+modelStruct.d_modelPar_WETA0CV_dX*Inv_W))+(modelStruct.modelPar_PETA0CV*d_Inv_LW_dX+modelStruct.d_modelPar_PETA0CV_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIeta0cv = (((B4SOIeta0cv+(modelStruct.modelPar_LETA0CV*Inv_L))+(modelStruct.modelPar_WETA0CV*Inv_W))+(modelStruct.modelPar_PETA0CV*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIetabcv_dX = (((d_B4SOIetabcv_dX+(modelStruct.modelPar_LETABCV*d_Inv_L_dX+modelStruct.d_modelPar_LETABCV_dX*Inv_L))+(modelStruct.modelPar_WETABCV*d_Inv_W_dX+modelStruct.d_modelPar_WETABCV_dX*Inv_W))+(modelStruct.modelPar_PETABCV*d_Inv_LW_dX+modelStruct.d_modelPar_PETABCV_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIetabcv = (((B4SOIetabcv+(modelStruct.modelPar_LETABCV*Inv_L))+(modelStruct.modelPar_WETABCV*Inv_W))+(modelStruct.modelPar_PETABCV*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdsub_dX = (((d_B4SOIdsub_dX+(modelStruct.modelPar_LDSUB*d_Inv_L_dX+modelStruct.d_modelPar_LDSUB_dX*Inv_L))+(modelStruct.modelPar_WDSUB*d_Inv_W_dX+modelStruct.d_modelPar_WDSUB_dX*Inv_W))+(modelStruct.modelPar_PDSUB*d_Inv_LW_dX+modelStruct.d_modelPar_PDSUB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdsub = (((B4SOIdsub+(modelStruct.modelPar_LDSUB*Inv_L))+(modelStruct.modelPar_WDSUB*Inv_W))+(modelStruct.modelPar_PDSUB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcit_dX = (((d_B4SOIcit_dX+(modelStruct.modelPar_LCIT*d_Inv_L_dX+modelStruct.d_modelPar_LCIT_dX*Inv_L))+(modelStruct.modelPar_WCIT*d_Inv_W_dX+modelStruct.d_modelPar_WCIT_dX*Inv_W))+(modelStruct.modelPar_PCIT*d_Inv_LW_dX+modelStruct.d_modelPar_PCIT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcit = (((B4SOIcit+(modelStruct.modelPar_LCIT*Inv_L))+(modelStruct.modelPar_WCIT*Inv_W))+(modelStruct.modelPar_PCIT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX = (((d_B4SOIcdsc_dX+(modelStruct.modelPar_LCDSC*d_Inv_L_dX+modelStruct.d_modelPar_LCDSC_dX*Inv_L))+(modelStruct.modelPar_WCDSC*d_Inv_W_dX+modelStruct.d_modelPar_WCDSC_dX*Inv_W))+(modelStruct.modelPar_PCDSC*d_Inv_LW_dX+modelStruct.d_modelPar_PCDSC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcdsc = (((B4SOIcdsc+(modelStruct.modelPar_LCDSC*Inv_L))+(modelStruct.modelPar_WCDSC*Inv_W))+(modelStruct.modelPar_PCDSC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcdscb_dX = (((d_B4SOIcdscb_dX+(modelStruct.modelPar_LCDSCB*d_Inv_L_dX+modelStruct.d_modelPar_LCDSCB_dX*Inv_L))+(modelStruct.modelPar_WCDSCB*d_Inv_W_dX+modelStruct.d_modelPar_WCDSCB_dX*Inv_W))+(modelStruct.modelPar_PCDSCB*d_Inv_LW_dX+modelStruct.d_modelPar_PCDSCB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcdscb = (((B4SOIcdscb+(modelStruct.modelPar_LCDSCB*Inv_L))+(modelStruct.modelPar_WCDSCB*Inv_W))+(modelStruct.modelPar_PCDSCB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcdscd_dX = (((d_B4SOIcdscd_dX+(modelStruct.modelPar_LCDSCD*d_Inv_L_dX+modelStruct.d_modelPar_LCDSCD_dX*Inv_L))+(modelStruct.modelPar_WCDSCD*d_Inv_W_dX+modelStruct.d_modelPar_WCDSCD_dX*Inv_W))+(modelStruct.modelPar_PCDSCD*d_Inv_LW_dX+modelStruct.d_modelPar_PCDSCD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcdscd = (((B4SOIcdscd+(modelStruct.modelPar_LCDSCD*Inv_L))+(modelStruct.modelPar_WCDSCD*Inv_W))+(modelStruct.modelPar_PCDSCD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpclm_dX = (((d_B4SOIpclm_dX+(modelStruct.modelPar_LPCLM*d_Inv_L_dX+modelStruct.d_modelPar_LPCLM_dX*Inv_L))+(modelStruct.modelPar_WPCLM*d_Inv_W_dX+modelStruct.d_modelPar_WPCLM_dX*Inv_W))+(modelStruct.modelPar_PPCLM*d_Inv_LW_dX+modelStruct.d_modelPar_PPCLM_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpclm = (((B4SOIpclm+(modelStruct.modelPar_LPCLM*Inv_L))+(modelStruct.modelPar_WPCLM*Inv_W))+(modelStruct.modelPar_PPCLM*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpdibl1_dX = (((d_B4SOIpdibl1_dX+(modelStruct.modelPar_LPDIBLC1*d_Inv_L_dX+modelStruct.d_modelPar_LPDIBLC1_dX*Inv_L))+(modelStruct.modelPar_WPDIBLC1*d_Inv_W_dX+modelStruct.d_modelPar_WPDIBLC1_dX*Inv_W))+(modelStruct.modelPar_PPDIBLC1*d_Inv_LW_dX+modelStruct.d_modelPar_PPDIBLC1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpdibl1 = (((B4SOIpdibl1+(modelStruct.modelPar_LPDIBLC1*Inv_L))+(modelStruct.modelPar_WPDIBLC1*Inv_W))+(modelStruct.modelPar_PPDIBLC1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpdibl2_dX = (((d_B4SOIpdibl2_dX+(modelStruct.modelPar_LPDIBLC2*d_Inv_L_dX+modelStruct.d_modelPar_LPDIBLC2_dX*Inv_L))+(modelStruct.modelPar_WPDIBLC2*d_Inv_W_dX+modelStruct.d_modelPar_WPDIBLC2_dX*Inv_W))+(modelStruct.modelPar_PPDIBLC2*d_Inv_LW_dX+modelStruct.d_modelPar_PPDIBLC2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpdibl2 = (((B4SOIpdibl2+(modelStruct.modelPar_LPDIBLC2*Inv_L))+(modelStruct.modelPar_WPDIBLC2*Inv_W))+(modelStruct.modelPar_PPDIBLC2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpdiblb_dX = (((d_B4SOIpdiblb_dX+(modelStruct.modelPar_LPDIBLCB*d_Inv_L_dX+modelStruct.d_modelPar_LPDIBLCB_dX*Inv_L))+(modelStruct.modelPar_WPDIBLCB*d_Inv_W_dX+modelStruct.d_modelPar_WPDIBLCB_dX*Inv_W))+(modelStruct.modelPar_PPDIBLCB*d_Inv_LW_dX+modelStruct.d_modelPar_PPDIBLCB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpdiblb = (((B4SOIpdiblb+(modelStruct.modelPar_LPDIBLCB*Inv_L))+(modelStruct.modelPar_WPDIBLCB*Inv_W))+(modelStruct.modelPar_PPDIBLCB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdrout_dX = (((d_B4SOIdrout_dX+(modelStruct.modelPar_LDROUT*d_Inv_L_dX+modelStruct.d_modelPar_LDROUT_dX*Inv_L))+(modelStruct.modelPar_WDROUT*d_Inv_W_dX+modelStruct.d_modelPar_WDROUT_dX*Inv_W))+(modelStruct.modelPar_PDROUT*d_Inv_LW_dX+modelStruct.d_modelPar_PDROUT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdrout = (((B4SOIdrout+(modelStruct.modelPar_LDROUT*Inv_L))+(modelStruct.modelPar_WDROUT*Inv_W))+(modelStruct.modelPar_PDROUT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpvag_dX = (((d_B4SOIpvag_dX+(modelStruct.modelPar_LPVAG*d_Inv_L_dX+modelStruct.d_modelPar_LPVAG_dX*Inv_L))+(modelStruct.modelPar_WPVAG*d_Inv_W_dX+modelStruct.d_modelPar_WPVAG_dX*Inv_W))+(modelStruct.modelPar_PPVAG*d_Inv_LW_dX+modelStruct.d_modelPar_PPVAG_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpvag = (((B4SOIpvag+(modelStruct.modelPar_LPVAG*Inv_L))+(modelStruct.modelPar_WPVAG*Inv_W))+(modelStruct.modelPar_PPVAG*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdelta_dX = (((d_B4SOIdelta_dX+(modelStruct.modelPar_LDELTA*d_Inv_L_dX+modelStruct.d_modelPar_LDELTA_dX*Inv_L))+(modelStruct.modelPar_WDELTA*d_Inv_W_dX+modelStruct.d_modelPar_WDELTA_dX*Inv_W))+(modelStruct.modelPar_PDELTA*d_Inv_LW_dX+modelStruct.d_modelPar_PDELTA_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdelta = (((B4SOIdelta+(modelStruct.modelPar_LDELTA*Inv_L))+(modelStruct.modelPar_WDELTA*Inv_W))+(modelStruct.modelPar_PDELTA*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX = (((d_B4SOIalpha0_dX+(modelStruct.modelPar_LALPHA0*d_Inv_L_dX+modelStruct.d_modelPar_LALPHA0_dX*Inv_L))+(modelStruct.modelPar_WALPHA0*d_Inv_W_dX+modelStruct.d_modelPar_WALPHA0_dX*Inv_W))+(modelStruct.modelPar_PALPHA0*d_Inv_LW_dX+modelStruct.d_modelPar_PALPHA0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIalpha0 = (((B4SOIalpha0+(modelStruct.modelPar_LALPHA0*Inv_L))+(modelStruct.modelPar_WALPHA0*Inv_W))+(modelStruct.modelPar_PALPHA0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIfbjtii_dX = (((instanceStruct.d_instanceVar_B4SOIfbjtii_dX+(modelStruct.modelPar_LFBJTII*d_Inv_L_dX+modelStruct.d_modelPar_LFBJTII_dX*Inv_L))+(modelStruct.modelPar_WFBJTII*d_Inv_W_dX+modelStruct.d_modelPar_WFBJTII_dX*Inv_W))+(modelStruct.modelPar_PFBJTII*d_Inv_LW_dX+modelStruct.d_modelPar_PFBJTII_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIfbjtii = (((instanceStruct.instanceVar_B4SOIfbjtii+(modelStruct.modelPar_LFBJTII*Inv_L))+(modelStruct.modelPar_WFBJTII*Inv_W))+(modelStruct.modelPar_PFBJTII*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIebjtii_dX = (((d_B4SOIebjtii_dX+(modelStruct.modelPar_LEBJTII*d_Inv_L_dX+modelStruct.d_modelPar_LEBJTII_dX*Inv_L))+(modelStruct.modelPar_WEBJTII*d_Inv_W_dX+modelStruct.d_modelPar_WEBJTII_dX*Inv_W))+(modelStruct.modelPar_PEBJTII*d_Inv_LW_dX+modelStruct.d_modelPar_PEBJTII_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIebjtii = (((B4SOIebjtii+(modelStruct.modelPar_LEBJTII*Inv_L))+(modelStruct.modelPar_WEBJTII*Inv_W))+(modelStruct.modelPar_PEBJTII*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcbjtii_dX = (((d_B4SOIcbjtii_dX+(modelStruct.modelPar_LCBJTII*d_Inv_L_dX+modelStruct.d_modelPar_LCBJTII_dX*Inv_L))+(modelStruct.modelPar_WCBJTII*d_Inv_W_dX+modelStruct.d_modelPar_WCBJTII_dX*Inv_W))+(modelStruct.modelPar_PCBJTII*d_Inv_LW_dX+modelStruct.d_modelPar_PCBJTII_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcbjtii = (((B4SOIcbjtii+(modelStruct.modelPar_LCBJTII*Inv_L))+(modelStruct.modelPar_WCBJTII*Inv_W))+(modelStruct.modelPar_PCBJTII*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvbci_dX = (((d_B4SOIvbci_dX+(modelStruct.modelPar_LVBCI*d_Inv_L_dX+modelStruct.d_modelPar_LVBCI_dX*Inv_L))+(modelStruct.modelPar_WVBCI*d_Inv_W_dX+modelStruct.d_modelPar_WVBCI_dX*Inv_W))+(modelStruct.modelPar_PVBCI*d_Inv_LW_dX+modelStruct.d_modelPar_PVBCI_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvbci = (((B4SOIvbci+(modelStruct.modelPar_LVBCI*Inv_L))+(modelStruct.modelPar_WVBCI*Inv_W))+(modelStruct.modelPar_PVBCI*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIabjtii_dX = (((d_B4SOIabjtii_dX+(modelStruct.modelPar_LABJTII*d_Inv_L_dX+modelStruct.d_modelPar_LABJTII_dX*Inv_L))+(modelStruct.modelPar_WABJTII*d_Inv_W_dX+modelStruct.d_modelPar_WABJTII_dX*Inv_W))+(modelStruct.modelPar_PABJTII*d_Inv_LW_dX+modelStruct.d_modelPar_PABJTII_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIabjtii = (((B4SOIabjtii+(modelStruct.modelPar_LABJTII*Inv_L))+(modelStruct.modelPar_WABJTII*Inv_W))+(modelStruct.modelPar_PABJTII*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOImbjtii_dX = (((d_B4SOImbjtii_dX+(modelStruct.modelPar_LMBJTII*d_Inv_L_dX+modelStruct.d_modelPar_LMBJTII_dX*Inv_L))+(modelStruct.modelPar_WMBJTII*d_Inv_W_dX+modelStruct.d_modelPar_WMBJTII_dX*Inv_W))+(modelStruct.modelPar_PMBJTII*d_Inv_LW_dX+modelStruct.d_modelPar_PMBJTII_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOImbjtii = (((B4SOImbjtii+(modelStruct.modelPar_LMBJTII*Inv_L))+(modelStruct.modelPar_WMBJTII*Inv_W))+(modelStruct.modelPar_PMBJTII*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbeta0_dX = (((instanceStruct.d_instanceVar_B4SOIbeta0_dX+(modelStruct.modelPar_LBETA0*d_Inv_L_dX+modelStruct.d_modelPar_LBETA0_dX*Inv_L))+(modelStruct.modelPar_WBETA0*d_Inv_W_dX+modelStruct.d_modelPar_WBETA0_dX*Inv_W))+(modelStruct.modelPar_PBETA0*d_Inv_LW_dX+modelStruct.d_modelPar_PBETA0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbeta0 = (((instanceStruct.instanceVar_B4SOIbeta0+(modelStruct.modelPar_LBETA0*Inv_L))+(modelStruct.modelPar_WBETA0*Inv_W))+(modelStruct.modelPar_PBETA0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbeta1_dX = (((instanceStruct.d_instanceVar_B4SOIbeta1_dX+(modelStruct.modelPar_LBETA1*d_Inv_L_dX+modelStruct.d_modelPar_LBETA1_dX*Inv_L))+(modelStruct.modelPar_WBETA1*d_Inv_W_dX+modelStruct.d_modelPar_WBETA1_dX*Inv_W))+(modelStruct.modelPar_PBETA1*d_Inv_LW_dX+modelStruct.d_modelPar_PBETA1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbeta1 = (((instanceStruct.instanceVar_B4SOIbeta1+(modelStruct.modelPar_LBETA1*Inv_L))+(modelStruct.modelPar_WBETA1*Inv_W))+(modelStruct.modelPar_PBETA1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbeta2_dX = (((instanceStruct.d_instanceVar_B4SOIbeta2_dX+(modelStruct.modelPar_LBETA2*d_Inv_L_dX+modelStruct.d_modelPar_LBETA2_dX*Inv_L))+(modelStruct.modelPar_WBETA2*d_Inv_W_dX+modelStruct.d_modelPar_WBETA2_dX*Inv_W))+(modelStruct.modelPar_PBETA2*d_Inv_LW_dX+modelStruct.d_modelPar_PBETA2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbeta2 = (((instanceStruct.instanceVar_B4SOIbeta2+(modelStruct.modelPar_LBETA2*Inv_L))+(modelStruct.modelPar_WBETA2*Inv_W))+(modelStruct.modelPar_PBETA2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvdsatii0_dX = (((d_B4SOIvdsatii0_dX+(modelStruct.modelPar_LVDSATII0*d_Inv_L_dX+modelStruct.d_modelPar_LVDSATII0_dX*Inv_L))+(modelStruct.modelPar_WVDSATII0*d_Inv_W_dX+modelStruct.d_modelPar_WVDSATII0_dX*Inv_W))+(modelStruct.modelPar_PVDSATII0*d_Inv_LW_dX+modelStruct.d_modelPar_PVDSATII0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvdsatii0 = (((B4SOIvdsatii0+(modelStruct.modelPar_LVDSATII0*Inv_L))+(modelStruct.modelPar_WVDSATII0*Inv_W))+(modelStruct.modelPar_PVDSATII0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIlii_dX = (((instanceStruct.d_instanceVar_B4SOIlii_dX+(modelStruct.modelPar_LLII*d_Inv_L_dX+modelStruct.d_modelPar_LLII_dX*Inv_L))+(modelStruct.modelPar_WLII*d_Inv_W_dX+modelStruct.d_modelPar_WLII_dX*Inv_W))+(modelStruct.modelPar_PLII*d_Inv_LW_dX+modelStruct.d_modelPar_PLII_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIlii = (((instanceStruct.instanceVar_B4SOIlii+(modelStruct.modelPar_LLII*Inv_L))+(modelStruct.modelPar_WLII*Inv_W))+(modelStruct.modelPar_PLII*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIesatii_dX = (((instanceStruct.d_instanceVar_B4SOIesatii_dX+(modelStruct.modelPar_LESATII*d_Inv_L_dX+modelStruct.d_modelPar_LESATII_dX*Inv_L))+(modelStruct.modelPar_WESATII*d_Inv_W_dX+modelStruct.d_modelPar_WESATII_dX*Inv_W))+(modelStruct.modelPar_PESATII*d_Inv_LW_dX+modelStruct.d_modelPar_PESATII_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIesatii = (((instanceStruct.instanceVar_B4SOIesatii+(modelStruct.modelPar_LESATII*Inv_L))+(modelStruct.modelPar_WESATII*Inv_W))+(modelStruct.modelPar_PESATII*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIsii0_dX = (((d_B4SOIsii0_dX+(modelStruct.modelPar_LSII0*d_Inv_L_dX+modelStruct.d_modelPar_LSII0_dX*Inv_L))+(modelStruct.modelPar_WSII0*d_Inv_W_dX+modelStruct.d_modelPar_WSII0_dX*Inv_W))+(modelStruct.modelPar_PSII0*d_Inv_LW_dX+modelStruct.d_modelPar_PSII0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIsii0 = (((B4SOIsii0+(modelStruct.modelPar_LSII0*Inv_L))+(modelStruct.modelPar_WSII0*Inv_W))+(modelStruct.modelPar_PSII0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIsii1_dX = (((instanceStruct.d_instanceVar_B4SOIsii1_dX+(modelStruct.modelPar_LSII1*d_Inv_L_dX+modelStruct.d_modelPar_LSII1_dX*Inv_L))+(modelStruct.modelPar_WSII1*d_Inv_W_dX+modelStruct.d_modelPar_WSII1_dX*Inv_W))+(modelStruct.modelPar_PSII1*d_Inv_LW_dX+modelStruct.d_modelPar_PSII1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIsii1 = (((instanceStruct.instanceVar_B4SOIsii1+(modelStruct.modelPar_LSII1*Inv_L))+(modelStruct.modelPar_WSII1*Inv_W))+(modelStruct.modelPar_PSII1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIsii2_dX = (((instanceStruct.d_instanceVar_B4SOIsii2_dX+(modelStruct.modelPar_LSII2*d_Inv_L_dX+modelStruct.d_modelPar_LSII2_dX*Inv_L))+(modelStruct.modelPar_WSII2*d_Inv_W_dX+modelStruct.d_modelPar_WSII2_dX*Inv_W))+(modelStruct.modelPar_PSII2*d_Inv_LW_dX+modelStruct.d_modelPar_PSII2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIsii2 = (((instanceStruct.instanceVar_B4SOIsii2+(modelStruct.modelPar_LSII2*Inv_L))+(modelStruct.modelPar_WSII2*Inv_W))+(modelStruct.modelPar_PSII2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIsiid_dX = (((instanceStruct.d_instanceVar_B4SOIsiid_dX+(modelStruct.modelPar_LSIID*d_Inv_L_dX+modelStruct.d_modelPar_LSIID_dX*Inv_L))+(modelStruct.modelPar_WSIID*d_Inv_W_dX+modelStruct.d_modelPar_WSIID_dX*Inv_W))+(modelStruct.modelPar_PSIID*d_Inv_LW_dX+modelStruct.d_modelPar_PSIID_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIsiid = (((instanceStruct.instanceVar_B4SOIsiid+(modelStruct.modelPar_LSIID*Inv_L))+(modelStruct.modelPar_WSIID*Inv_W))+(modelStruct.modelPar_PSIID*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIagidl_dX = (((instanceStruct.d_instanceVar_B4SOIagidl_dX+(modelStruct.modelPar_LAGIDL*d_Inv_L_dX+modelStruct.d_modelPar_LAGIDL_dX*Inv_L))+(modelStruct.modelPar_WAGIDL*d_Inv_W_dX+modelStruct.d_modelPar_WAGIDL_dX*Inv_W))+(modelStruct.modelPar_PAGIDL*d_Inv_LW_dX+modelStruct.d_modelPar_PAGIDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIagidl = (((instanceStruct.instanceVar_B4SOIagidl+(modelStruct.modelPar_LAGIDL*Inv_L))+(modelStruct.modelPar_WAGIDL*Inv_W))+(modelStruct.modelPar_PAGIDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbgidl_dX = (((instanceStruct.d_instanceVar_B4SOIbgidl_dX+(modelStruct.modelPar_LBGIDL*d_Inv_L_dX+modelStruct.d_modelPar_LBGIDL_dX*Inv_L))+(modelStruct.modelPar_WBGIDL*d_Inv_W_dX+modelStruct.d_modelPar_WBGIDL_dX*Inv_W))+(modelStruct.modelPar_PBGIDL*d_Inv_LW_dX+modelStruct.d_modelPar_PBGIDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbgidl = (((instanceStruct.instanceVar_B4SOIbgidl+(modelStruct.modelPar_LBGIDL*Inv_L))+(modelStruct.modelPar_WBGIDL*Inv_W))+(modelStruct.modelPar_PBGIDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcgidl_dX = (((instanceStruct.d_instanceVar_B4SOIcgidl_dX+(modelStruct.modelPar_LCGIDL*d_Inv_L_dX+modelStruct.d_modelPar_LCGIDL_dX*Inv_L))+(modelStruct.modelPar_WCGIDL*d_Inv_W_dX+modelStruct.d_modelPar_WCGIDL_dX*Inv_W))+(modelStruct.modelPar_PCGIDL*d_Inv_LW_dX+modelStruct.d_modelPar_PCGIDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcgidl = (((instanceStruct.instanceVar_B4SOIcgidl+(modelStruct.modelPar_LCGIDL*Inv_L))+(modelStruct.modelPar_WCGIDL*Inv_W))+(modelStruct.modelPar_PCGIDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIegidl_dX = (((instanceStruct.d_instanceVar_B4SOIegidl_dX+(modelStruct.modelPar_LEGIDL*d_Inv_L_dX+modelStruct.d_modelPar_LEGIDL_dX*Inv_L))+(modelStruct.modelPar_WEGIDL*d_Inv_W_dX+modelStruct.d_modelPar_WEGIDL_dX*Inv_W))+(modelStruct.modelPar_PEGIDL*d_Inv_LW_dX+modelStruct.d_modelPar_PEGIDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIegidl = (((instanceStruct.instanceVar_B4SOIegidl+(modelStruct.modelPar_LEGIDL*Inv_L))+(modelStruct.modelPar_WEGIDL*Inv_W))+(modelStruct.modelPar_PEGIDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIrgidl_dX = (((d_B4SOIrgidl_dX+(modelStruct.modelPar_LRGIDL*d_Inv_L_dX+modelStruct.d_modelPar_LRGIDL_dX*Inv_L))+(modelStruct.modelPar_WRGIDL*d_Inv_W_dX+modelStruct.d_modelPar_WRGIDL_dX*Inv_W))+(modelStruct.modelPar_PRGIDL*d_Inv_LW_dX+modelStruct.d_modelPar_PRGIDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIrgidl = (((B4SOIrgidl+(modelStruct.modelPar_LRGIDL*Inv_L))+(modelStruct.modelPar_WRGIDL*Inv_W))+(modelStruct.modelPar_PRGIDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIkgidl_dX = (((d_B4SOIkgidl_dX+(modelStruct.modelPar_LKGIDL*d_Inv_L_dX+modelStruct.d_modelPar_LKGIDL_dX*Inv_L))+(modelStruct.modelPar_WKGIDL*d_Inv_W_dX+modelStruct.d_modelPar_WKGIDL_dX*Inv_W))+(modelStruct.modelPar_PKGIDL*d_Inv_LW_dX+modelStruct.d_modelPar_PKGIDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIkgidl = (((B4SOIkgidl+(modelStruct.modelPar_LKGIDL*Inv_L))+(modelStruct.modelPar_WKGIDL*Inv_W))+(modelStruct.modelPar_PKGIDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIfgidl_dX = (((d_B4SOIfgidl_dX+(modelStruct.modelPar_LFGIDL*d_Inv_L_dX+modelStruct.d_modelPar_LFGIDL_dX*Inv_L))+(modelStruct.modelPar_WFGIDL*d_Inv_W_dX+modelStruct.d_modelPar_WFGIDL_dX*Inv_W))+(modelStruct.modelPar_PFGIDL*d_Inv_LW_dX+modelStruct.d_modelPar_PFGIDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIfgidl = (((B4SOIfgidl+(modelStruct.modelPar_LFGIDL*Inv_L))+(modelStruct.modelPar_WFGIDL*Inv_W))+(modelStruct.modelPar_PFGIDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIagisl_dX = (((instanceStruct.d_instanceVar_B4SOIagisl_dX+(modelStruct.modelPar_LAGISL*d_Inv_L_dX+modelStruct.d_modelPar_LAGISL_dX*Inv_L))+(modelStruct.modelPar_WAGISL*d_Inv_W_dX+modelStruct.d_modelPar_WAGISL_dX*Inv_W))+(modelStruct.modelPar_PAGISL*d_Inv_LW_dX+modelStruct.d_modelPar_PAGISL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIagisl = (((instanceStruct.instanceVar_B4SOIagisl+(modelStruct.modelPar_LAGISL*Inv_L))+(modelStruct.modelPar_WAGISL*Inv_W))+(modelStruct.modelPar_PAGISL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbgisl_dX = (((instanceStruct.d_instanceVar_B4SOIbgisl_dX+(modelStruct.modelPar_LBGISL*d_Inv_L_dX+modelStruct.d_modelPar_LBGISL_dX*Inv_L))+(modelStruct.modelPar_WBGISL*d_Inv_W_dX+modelStruct.d_modelPar_WBGISL_dX*Inv_W))+(modelStruct.modelPar_PBGISL*d_Inv_LW_dX+modelStruct.d_modelPar_PBGISL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbgisl = (((instanceStruct.instanceVar_B4SOIbgisl+(modelStruct.modelPar_LBGISL*Inv_L))+(modelStruct.modelPar_WBGISL*Inv_W))+(modelStruct.modelPar_PBGISL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcgisl_dX = (((instanceStruct.d_instanceVar_B4SOIcgisl_dX+(modelStruct.modelPar_LCGISL*d_Inv_L_dX+modelStruct.d_modelPar_LCGISL_dX*Inv_L))+(modelStruct.modelPar_WCGISL*d_Inv_W_dX+modelStruct.d_modelPar_WCGISL_dX*Inv_W))+(modelStruct.modelPar_PCGISL*d_Inv_LW_dX+modelStruct.d_modelPar_PCGISL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcgisl = (((instanceStruct.instanceVar_B4SOIcgisl+(modelStruct.modelPar_LCGISL*Inv_L))+(modelStruct.modelPar_WCGISL*Inv_W))+(modelStruct.modelPar_PCGISL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIegisl_dX = (((instanceStruct.d_instanceVar_B4SOIegisl_dX+(modelStruct.modelPar_LEGISL*d_Inv_L_dX+modelStruct.d_modelPar_LEGISL_dX*Inv_L))+(modelStruct.modelPar_WEGISL*d_Inv_W_dX+modelStruct.d_modelPar_WEGISL_dX*Inv_W))+(modelStruct.modelPar_PEGISL*d_Inv_LW_dX+modelStruct.d_modelPar_PEGISL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIegisl = (((instanceStruct.instanceVar_B4SOIegisl+(modelStruct.modelPar_LEGISL*Inv_L))+(modelStruct.modelPar_WEGISL*Inv_W))+(modelStruct.modelPar_PEGISL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIrgisl_dX = (((d_B4SOIrgisl_dX+(modelStruct.modelPar_LRGISL*d_Inv_L_dX+modelStruct.d_modelPar_LRGISL_dX*Inv_L))+(modelStruct.modelPar_WRGISL*d_Inv_W_dX+modelStruct.d_modelPar_WRGISL_dX*Inv_W))+(modelStruct.modelPar_PRGISL*d_Inv_LW_dX+modelStruct.d_modelPar_PRGISL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIrgisl = (((B4SOIrgisl+(modelStruct.modelPar_LRGISL*Inv_L))+(modelStruct.modelPar_WRGISL*Inv_W))+(modelStruct.modelPar_PRGISL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIkgisl_dX = (((d_B4SOIkgisl_dX+(modelStruct.modelPar_LKGISL*d_Inv_L_dX+modelStruct.d_modelPar_LKGISL_dX*Inv_L))+(modelStruct.modelPar_WKGISL*d_Inv_W_dX+modelStruct.d_modelPar_WKGISL_dX*Inv_W))+(modelStruct.modelPar_PKGISL*d_Inv_LW_dX+modelStruct.d_modelPar_PKGISL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIkgisl = (((B4SOIkgisl+(modelStruct.modelPar_LKGISL*Inv_L))+(modelStruct.modelPar_WKGISL*Inv_W))+(modelStruct.modelPar_PKGISL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIfgisl_dX = (((d_B4SOIfgisl_dX+(modelStruct.modelPar_LFGISL*d_Inv_L_dX+modelStruct.d_modelPar_LFGISL_dX*Inv_L))+(modelStruct.modelPar_WFGISL*d_Inv_W_dX+modelStruct.d_modelPar_WFGISL_dX*Inv_W))+(modelStruct.modelPar_PFGISL*d_Inv_LW_dX+modelStruct.d_modelPar_PFGISL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIfgisl = (((B4SOIfgisl+(modelStruct.modelPar_LFGISL*Inv_L))+(modelStruct.modelPar_WFGISL*Inv_W))+(modelStruct.modelPar_PFGISL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIntun_dX = (((instanceStruct.d_instanceVar_B4SOIntun_dX+(modelStruct.modelPar_LNTUN*d_Inv_L_dX+modelStruct.d_modelPar_LNTUN_dX*Inv_L))+(modelStruct.modelPar_WNTUN*d_Inv_W_dX+modelStruct.d_modelPar_WNTUN_dX*Inv_W))+(modelStruct.modelPar_PNTUN*d_Inv_LW_dX+modelStruct.d_modelPar_PNTUN_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIntun = (((instanceStruct.instanceVar_B4SOIntun+(modelStruct.modelPar_LNTUN*Inv_L))+(modelStruct.modelPar_WNTUN*Inv_W))+(modelStruct.modelPar_PNTUN*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIntund_dX = (((instanceStruct.d_instanceVar_B4SOIntund_dX+(modelStruct.modelPar_LNTUND*d_Inv_L_dX+modelStruct.d_modelPar_LNTUND_dX*Inv_L))+(modelStruct.modelPar_WNTUND*d_Inv_W_dX+modelStruct.d_modelPar_WNTUND_dX*Inv_W))+(modelStruct.modelPar_PNTUND*d_Inv_LW_dX+modelStruct.d_modelPar_PNTUND_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIntund = (((instanceStruct.instanceVar_B4SOIntund+(modelStruct.modelPar_LNTUND*Inv_L))+(modelStruct.modelPar_WNTUND*Inv_W))+(modelStruct.modelPar_PNTUND*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX = (((instanceStruct.d_instanceVar_B4SOIndiode_dX+(modelStruct.modelPar_LNDIODE*d_Inv_L_dX+modelStruct.d_modelPar_LNDIODE_dX*Inv_L))+(modelStruct.modelPar_WNDIODE*d_Inv_W_dX+modelStruct.d_modelPar_WNDIODE_dX*Inv_W))+(modelStruct.modelPar_PNDIODE*d_Inv_LW_dX+modelStruct.d_modelPar_PNDIODE_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIndiode = (((instanceStruct.instanceVar_B4SOIndiode+(modelStruct.modelPar_LNDIODE*Inv_L))+(modelStruct.modelPar_WNDIODE*Inv_W))+(modelStruct.modelPar_PNDIODE*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX = (((instanceStruct.d_instanceVar_B4SOIndioded_dX+(modelStruct.modelPar_LNDIODED*d_Inv_L_dX+modelStruct.d_modelPar_LNDIODED_dX*Inv_L))+(modelStruct.modelPar_WNDIODED*d_Inv_W_dX+modelStruct.d_modelPar_WNDIODED_dX*Inv_W))+(modelStruct.modelPar_PNDIODED*d_Inv_LW_dX+modelStruct.d_modelPar_PNDIODED_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIndioded = (((instanceStruct.instanceVar_B4SOIndioded+(modelStruct.modelPar_LNDIODED*Inv_L))+(modelStruct.modelPar_WNDIODED*Inv_W))+(modelStruct.modelPar_PNDIODED*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInrecf0_dX = (((d_B4SOInrecf0_dX+(modelStruct.modelPar_LNRECF0*d_Inv_L_dX+modelStruct.d_modelPar_LNRECF0_dX*Inv_L))+(modelStruct.modelPar_WNRECF0*d_Inv_W_dX+modelStruct.d_modelPar_WNRECF0_dX*Inv_W))+(modelStruct.modelPar_PNRECF0*d_Inv_LW_dX+modelStruct.d_modelPar_PNRECF0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInrecf0 = (((B4SOInrecf0+(modelStruct.modelPar_LNRECF0*Inv_L))+(modelStruct.modelPar_WNRECF0*Inv_W))+(modelStruct.modelPar_PNRECF0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInrecf0d_dX = (((d_B4SOInrecf0d_dX+(modelStruct.modelPar_LNRECF0D*d_Inv_L_dX+modelStruct.d_modelPar_LNRECF0D_dX*Inv_L))+(modelStruct.modelPar_WNRECF0D*d_Inv_W_dX+modelStruct.d_modelPar_WNRECF0D_dX*Inv_W))+(modelStruct.modelPar_PNRECF0D*d_Inv_LW_dX+modelStruct.d_modelPar_PNRECF0D_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInrecf0d = (((B4SOInrecf0d+(modelStruct.modelPar_LNRECF0D*Inv_L))+(modelStruct.modelPar_WNRECF0D*Inv_W))+(modelStruct.modelPar_PNRECF0D*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInrecr0_dX = (((d_B4SOInrecr0_dX+(modelStruct.modelPar_LNRECR0*d_Inv_L_dX+modelStruct.d_modelPar_LNRECR0_dX*Inv_L))+(modelStruct.modelPar_WNRECR0*d_Inv_W_dX+modelStruct.d_modelPar_WNRECR0_dX*Inv_W))+(modelStruct.modelPar_PNRECR0*d_Inv_LW_dX+modelStruct.d_modelPar_PNRECR0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInrecr0 = (((B4SOInrecr0+(modelStruct.modelPar_LNRECR0*Inv_L))+(modelStruct.modelPar_WNRECR0*Inv_W))+(modelStruct.modelPar_PNRECR0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInrecr0d_dX = (((d_B4SOInrecr0d_dX+(modelStruct.modelPar_LNRECR0D*d_Inv_L_dX+modelStruct.d_modelPar_LNRECR0D_dX*Inv_L))+(modelStruct.modelPar_WNRECR0D*d_Inv_W_dX+modelStruct.d_modelPar_WNRECR0D_dX*Inv_W))+(modelStruct.modelPar_PNRECR0D*d_Inv_LW_dX+modelStruct.d_modelPar_PNRECR0D_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInrecr0d = (((B4SOInrecr0d+(modelStruct.modelPar_LNRECR0D*Inv_L))+(modelStruct.modelPar_WNRECR0D*Inv_W))+(modelStruct.modelPar_PNRECR0D*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIisbjt_dX = (((instanceStruct.d_instanceVar_B4SOIisbjt_dX+(modelStruct.modelPar_LISBJT*d_Inv_L_dX+modelStruct.d_modelPar_LISBJT_dX*Inv_L))+(modelStruct.modelPar_WISBJT*d_Inv_W_dX+modelStruct.d_modelPar_WISBJT_dX*Inv_W))+(modelStruct.modelPar_PISBJT*d_Inv_LW_dX+modelStruct.d_modelPar_PISBJT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIisbjt = (((instanceStruct.instanceVar_B4SOIisbjt+(modelStruct.modelPar_LISBJT*Inv_L))+(modelStruct.modelPar_WISBJT*Inv_W))+(modelStruct.modelPar_PISBJT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIidbjt_dX = (((instanceStruct.d_instanceVar_B4SOIidbjt_dX+(modelStruct.modelPar_LIDBJT*d_Inv_L_dX+modelStruct.d_modelPar_LIDBJT_dX*Inv_L))+(modelStruct.modelPar_WIDBJT*d_Inv_W_dX+modelStruct.d_modelPar_WIDBJT_dX*Inv_W))+(modelStruct.modelPar_PIDBJT*d_Inv_LW_dX+modelStruct.d_modelPar_PIDBJT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIidbjt = (((instanceStruct.instanceVar_B4SOIidbjt+(modelStruct.modelPar_LIDBJT*Inv_L))+(modelStruct.modelPar_WIDBJT*Inv_W))+(modelStruct.modelPar_PIDBJT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIisdif_dX = (((instanceStruct.d_instanceVar_B4SOIisdif_dX+(modelStruct.modelPar_LISDIF*d_Inv_L_dX+modelStruct.d_modelPar_LISDIF_dX*Inv_L))+(modelStruct.modelPar_WISDIF*d_Inv_W_dX+modelStruct.d_modelPar_WISDIF_dX*Inv_W))+(modelStruct.modelPar_PISDIF*d_Inv_LW_dX+modelStruct.d_modelPar_PISDIF_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIisdif = (((instanceStruct.instanceVar_B4SOIisdif+(modelStruct.modelPar_LISDIF*Inv_L))+(modelStruct.modelPar_WISDIF*Inv_W))+(modelStruct.modelPar_PISDIF*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIiddif_dX = (((instanceStruct.d_instanceVar_B4SOIiddif_dX+(modelStruct.modelPar_LIDDIF*d_Inv_L_dX+modelStruct.d_modelPar_LIDDIF_dX*Inv_L))+(modelStruct.modelPar_WIDDIF*d_Inv_W_dX+modelStruct.d_modelPar_WIDDIF_dX*Inv_W))+(modelStruct.modelPar_PIDDIF*d_Inv_LW_dX+modelStruct.d_modelPar_PIDDIF_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIiddif = (((instanceStruct.instanceVar_B4SOIiddif+(modelStruct.modelPar_LIDDIF*Inv_L))+(modelStruct.modelPar_WIDDIF*Inv_W))+(modelStruct.modelPar_PIDDIF*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIisrec_dX = (((instanceStruct.d_instanceVar_B4SOIisrec_dX+(modelStruct.modelPar_LISREC*d_Inv_L_dX+modelStruct.d_modelPar_LISREC_dX*Inv_L))+(modelStruct.modelPar_WISREC*d_Inv_W_dX+modelStruct.d_modelPar_WISREC_dX*Inv_W))+(modelStruct.modelPar_PISREC*d_Inv_LW_dX+modelStruct.d_modelPar_PISREC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIisrec = (((instanceStruct.instanceVar_B4SOIisrec+(modelStruct.modelPar_LISREC*Inv_L))+(modelStruct.modelPar_WISREC*Inv_W))+(modelStruct.modelPar_PISREC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIistun_dX = (((instanceStruct.d_instanceVar_B4SOIistun_dX+(modelStruct.modelPar_LISTUN*d_Inv_L_dX+modelStruct.d_modelPar_LISTUN_dX*Inv_L))+(modelStruct.modelPar_WISTUN*d_Inv_W_dX+modelStruct.d_modelPar_WISTUN_dX*Inv_W))+(modelStruct.modelPar_PISTUN*d_Inv_LW_dX+modelStruct.d_modelPar_PISTUN_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIistun = (((instanceStruct.instanceVar_B4SOIistun+(modelStruct.modelPar_LISTUN*Inv_L))+(modelStruct.modelPar_WISTUN*Inv_W))+(modelStruct.modelPar_PISTUN*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIidrec_dX = (((instanceStruct.d_instanceVar_B4SOIidrec_dX+(modelStruct.modelPar_LIDREC*d_Inv_L_dX+modelStruct.d_modelPar_LIDREC_dX*Inv_L))+(modelStruct.modelPar_WIDREC*d_Inv_W_dX+modelStruct.d_modelPar_WIDREC_dX*Inv_W))+(modelStruct.modelPar_PIDREC*d_Inv_LW_dX+modelStruct.d_modelPar_PIDREC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIidrec = (((instanceStruct.instanceVar_B4SOIidrec+(modelStruct.modelPar_LIDREC*Inv_L))+(modelStruct.modelPar_WIDREC*Inv_W))+(modelStruct.modelPar_PIDREC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIidtun_dX = (((instanceStruct.d_instanceVar_B4SOIidtun_dX+(modelStruct.modelPar_LIDTUN*d_Inv_L_dX+modelStruct.d_modelPar_LIDTUN_dX*Inv_L))+(modelStruct.modelPar_WIDTUN*d_Inv_W_dX+modelStruct.d_modelPar_WIDTUN_dX*Inv_W))+(modelStruct.modelPar_PIDTUN*d_Inv_LW_dX+modelStruct.d_modelPar_PIDTUN_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIidtun = (((instanceStruct.instanceVar_B4SOIidtun+(modelStruct.modelPar_LIDTUN*Inv_L))+(modelStruct.modelPar_WIDTUN*Inv_W))+(modelStruct.modelPar_PIDTUN*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvrec0_dX = (((instanceStruct.d_instanceVar_B4SOIvrec0_dX+(modelStruct.modelPar_LVREC0*d_Inv_L_dX+modelStruct.d_modelPar_LVREC0_dX*Inv_L))+(modelStruct.modelPar_WVREC0*d_Inv_W_dX+modelStruct.d_modelPar_WVREC0_dX*Inv_W))+(modelStruct.modelPar_PVREC0*d_Inv_LW_dX+modelStruct.d_modelPar_PVREC0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvrec0 = (((instanceStruct.instanceVar_B4SOIvrec0+(modelStruct.modelPar_LVREC0*Inv_L))+(modelStruct.modelPar_WVREC0*Inv_W))+(modelStruct.modelPar_PVREC0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvrec0d_dX = (((instanceStruct.d_instanceVar_B4SOIvrec0d_dX+(modelStruct.modelPar_LVREC0D*d_Inv_L_dX+modelStruct.d_modelPar_LVREC0D_dX*Inv_L))+(modelStruct.modelPar_WVREC0D*d_Inv_W_dX+modelStruct.d_modelPar_WVREC0D_dX*Inv_W))+(modelStruct.modelPar_PVREC0D*d_Inv_LW_dX+modelStruct.d_modelPar_PVREC0D_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvrec0d = (((instanceStruct.instanceVar_B4SOIvrec0d+(modelStruct.modelPar_LVREC0D*Inv_L))+(modelStruct.modelPar_WVREC0D*Inv_W))+(modelStruct.modelPar_PVREC0D*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvtun0_dX = (((instanceStruct.d_instanceVar_B4SOIvtun0_dX+(modelStruct.modelPar_LVTUN0*d_Inv_L_dX+modelStruct.d_modelPar_LVTUN0_dX*Inv_L))+(modelStruct.modelPar_WVTUN0*d_Inv_W_dX+modelStruct.d_modelPar_WVTUN0_dX*Inv_W))+(modelStruct.modelPar_PVTUN0*d_Inv_LW_dX+modelStruct.d_modelPar_PVTUN0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvtun0 = (((instanceStruct.instanceVar_B4SOIvtun0+(modelStruct.modelPar_LVTUN0*Inv_L))+(modelStruct.modelPar_WVTUN0*Inv_W))+(modelStruct.modelPar_PVTUN0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvtun0d_dX = (((instanceStruct.d_instanceVar_B4SOIvtun0d_dX+(modelStruct.modelPar_LVTUN0D*d_Inv_L_dX+modelStruct.d_modelPar_LVTUN0D_dX*Inv_L))+(modelStruct.modelPar_WVTUN0D*d_Inv_W_dX+modelStruct.d_modelPar_WVTUN0D_dX*Inv_W))+(modelStruct.modelPar_PVTUN0D*d_Inv_LW_dX+modelStruct.d_modelPar_PVTUN0D_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvtun0d = (((instanceStruct.instanceVar_B4SOIvtun0d+(modelStruct.modelPar_LVTUN0D*Inv_L))+(modelStruct.modelPar_WVTUN0D*Inv_W))+(modelStruct.modelPar_PVTUN0D*Inv_LW));
d_pParam_B4SOInbjt_dX = (((instanceStruct.d_instanceVar_B4SOInbjt_dX+(modelStruct.modelPar_LNBJT*d_Inv_L_dX+modelStruct.d_modelPar_LNBJT_dX*Inv_L))+(modelStruct.modelPar_WNBJT*d_Inv_W_dX+modelStruct.d_modelPar_WNBJT_dX*Inv_W))+(modelStruct.modelPar_PNBJT*d_Inv_LW_dX+modelStruct.d_modelPar_PNBJT_dX*Inv_LW));
pParam_B4SOInbjt = (((instanceStruct.instanceVar_B4SOInbjt+(modelStruct.modelPar_LNBJT*Inv_L))+(modelStruct.modelPar_WNBJT*Inv_W))+(modelStruct.modelPar_PNBJT*Inv_LW));
d_pParam_B4SOIlbjt0_dX = (((d_B4SOIlbjt0_dX+(modelStruct.modelPar_LLBJT0*d_Inv_L_dX+modelStruct.d_modelPar_LLBJT0_dX*Inv_L))+(modelStruct.modelPar_WLBJT0*d_Inv_W_dX+modelStruct.d_modelPar_WLBJT0_dX*Inv_W))+(modelStruct.modelPar_PLBJT0*d_Inv_LW_dX+modelStruct.d_modelPar_PLBJT0_dX*Inv_LW));
pParam_B4SOIlbjt0 = (((B4SOIlbjt0+(modelStruct.modelPar_LLBJT0*Inv_L))+(modelStruct.modelPar_WLBJT0*Inv_W))+(modelStruct.modelPar_PLBJT0*Inv_LW));
d_pParam_B4SOIvabjt_dX = (((d_B4SOIvabjt_dX+(modelStruct.modelPar_LVABJT*d_Inv_L_dX+modelStruct.d_modelPar_LVABJT_dX*Inv_L))+(modelStruct.modelPar_WVABJT*d_Inv_W_dX+modelStruct.d_modelPar_WVABJT_dX*Inv_W))+(modelStruct.modelPar_PVABJT*d_Inv_LW_dX+modelStruct.d_modelPar_PVABJT_dX*Inv_LW));
pParam_B4SOIvabjt = (((B4SOIvabjt+(modelStruct.modelPar_LVABJT*Inv_L))+(modelStruct.modelPar_WVABJT*Inv_W))+(modelStruct.modelPar_PVABJT*Inv_LW));
d_pParam_B4SOIaely_dX = (((instanceStruct.d_instanceVar_B4SOIaely_dX+(modelStruct.modelPar_LAELY*d_Inv_L_dX+modelStruct.d_modelPar_LAELY_dX*Inv_L))+(modelStruct.modelPar_WAELY*d_Inv_W_dX+modelStruct.d_modelPar_WAELY_dX*Inv_W))+(modelStruct.modelPar_PAELY*d_Inv_LW_dX+modelStruct.d_modelPar_PAELY_dX*Inv_LW));
pParam_B4SOIaely = (((instanceStruct.instanceVar_B4SOIaely+(modelStruct.modelPar_LAELY*Inv_L))+(modelStruct.modelPar_WAELY*Inv_W))+(modelStruct.modelPar_PAELY*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIahli_dX = (((instanceStruct.d_instanceVar_B4SOIahli_dX+(modelStruct.modelPar_LAHLI*d_Inv_L_dX+modelStruct.d_modelPar_LAHLI_dX*Inv_L))+(modelStruct.modelPar_WAHLI*d_Inv_W_dX+modelStruct.d_modelPar_WAHLI_dX*Inv_W))+(modelStruct.modelPar_PAHLI*d_Inv_LW_dX+modelStruct.d_modelPar_PAHLI_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIahli = (((instanceStruct.instanceVar_B4SOIahli+(modelStruct.modelPar_LAHLI*Inv_L))+(modelStruct.modelPar_WAHLI*Inv_W))+(modelStruct.modelPar_PAHLI*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIahlid_dX = (((instanceStruct.d_instanceVar_B4SOIahlid_dX+(modelStruct.modelPar_LAHLID*d_Inv_L_dX+modelStruct.d_modelPar_LAHLID_dX*Inv_L))+(modelStruct.modelPar_WAHLID*d_Inv_W_dX+modelStruct.d_modelPar_WAHLID_dX*Inv_W))+(modelStruct.modelPar_PAHLID*d_Inv_LW_dX+modelStruct.d_modelPar_PAHLID_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIahlid = (((instanceStruct.instanceVar_B4SOIahlid+(modelStruct.modelPar_LAHLID*Inv_L))+(modelStruct.modelPar_WAHLID*Inv_W))+(modelStruct.modelPar_PAHLID*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxj_dX = (((d_B4SOIxj_dX+(modelStruct.modelPar_LXJ*d_Inv_L_dX+modelStruct.d_modelPar_LXJ_dX*Inv_L))+(modelStruct.modelPar_WXJ*d_Inv_W_dX+modelStruct.d_modelPar_WXJ_dX*Inv_W))+(modelStruct.modelPar_PXJ*d_Inv_LW_dX+modelStruct.d_modelPar_PXJ_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxj = (((B4SOIxj+(modelStruct.modelPar_LXJ*Inv_L))+(modelStruct.modelPar_WXJ*Inv_W))+(modelStruct.modelPar_PXJ*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIalphaGB1_dX = (((d_B4SOIalphaGB1_dX+(modelStruct.modelPar_LALPHAGB1*d_Inv_L_dX+modelStruct.d_modelPar_LALPHAGB1_dX*Inv_L))+(modelStruct.modelPar_WALPHAGB1*d_Inv_W_dX+modelStruct.d_modelPar_WALPHAGB1_dX*Inv_W))+(modelStruct.modelPar_PALPHAGB1*d_Inv_LW_dX+modelStruct.d_modelPar_PALPHAGB1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIalphaGB1 = (((B4SOIalphaGB1+(modelStruct.modelPar_LALPHAGB1*Inv_L))+(modelStruct.modelPar_WALPHAGB1*Inv_W))+(modelStruct.modelPar_PALPHAGB1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIalphaGB2_dX = (((d_B4SOIalphaGB2_dX+(modelStruct.modelPar_LALPHAGB2*d_Inv_L_dX+modelStruct.d_modelPar_LALPHAGB2_dX*Inv_L))+(modelStruct.modelPar_WALPHAGB2*d_Inv_W_dX+modelStruct.d_modelPar_WALPHAGB2_dX*Inv_W))+(modelStruct.modelPar_PALPHAGB2*d_Inv_LW_dX+modelStruct.d_modelPar_PALPHAGB2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIalphaGB2 = (((B4SOIalphaGB2+(modelStruct.modelPar_LALPHAGB2*Inv_L))+(modelStruct.modelPar_WALPHAGB2*Inv_W))+(modelStruct.modelPar_PALPHAGB2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbetaGB1_dX = (((d_B4SOIbetaGB1_dX+(modelStruct.modelPar_LBETAGB1*d_Inv_L_dX+modelStruct.d_modelPar_LBETAGB1_dX*Inv_L))+(modelStruct.modelPar_WBETAGB1*d_Inv_W_dX+modelStruct.d_modelPar_WBETAGB1_dX*Inv_W))+(modelStruct.modelPar_PBETAGB1*d_Inv_LW_dX+modelStruct.d_modelPar_PBETAGB1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbetaGB1 = (((B4SOIbetaGB1+(modelStruct.modelPar_LBETAGB1*Inv_L))+(modelStruct.modelPar_WBETAGB1*Inv_W))+(modelStruct.modelPar_PBETAGB1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbetaGB2_dX = (((d_B4SOIbetaGB2_dX+(modelStruct.modelPar_LBETAGB2*d_Inv_L_dX+modelStruct.d_modelPar_LBETAGB2_dX*Inv_L))+(modelStruct.modelPar_WBETAGB2*d_Inv_W_dX+modelStruct.d_modelPar_WBETAGB2_dX*Inv_W))+(modelStruct.modelPar_PBETAGB2*d_Inv_LW_dX+modelStruct.d_modelPar_PBETAGB2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbetaGB2 = (((B4SOIbetaGB2+(modelStruct.modelPar_LBETAGB2*Inv_L))+(modelStruct.modelPar_WBETAGB2*Inv_W))+(modelStruct.modelPar_PBETAGB2*Inv_LW));
d_pParam_B4SOIndif_dX = (((d_B4SOIndif_dX+(modelStruct.modelPar_LNDIF*d_Inv_L_dX+modelStruct.d_modelPar_LNDIF_dX*Inv_L))+(modelStruct.modelPar_WNDIF*d_Inv_W_dX+modelStruct.d_modelPar_WNDIF_dX*Inv_W))+(modelStruct.modelPar_PNDIF*d_Inv_LW_dX+modelStruct.d_modelPar_PNDIF_dX*Inv_LW));
pParam_B4SOIndif = (((B4SOIndif+(modelStruct.modelPar_LNDIF*Inv_L))+(modelStruct.modelPar_WNDIF*Inv_W))+(modelStruct.modelPar_PNDIF*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIntrecf_dX = (((d_B4SOIntrecf_dX+(modelStruct.modelPar_LNTRECF*d_Inv_L_dX+modelStruct.d_modelPar_LNTRECF_dX*Inv_L))+(modelStruct.modelPar_WNTRECF*d_Inv_W_dX+modelStruct.d_modelPar_WNTRECF_dX*Inv_W))+(modelStruct.modelPar_PNTRECF*d_Inv_LW_dX+modelStruct.d_modelPar_PNTRECF_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIntrecf = (((B4SOIntrecf+(modelStruct.modelPar_LNTRECF*Inv_L))+(modelStruct.modelPar_WNTRECF*Inv_W))+(modelStruct.modelPar_PNTRECF*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIntrecr_dX = (((d_B4SOIntrecr_dX+(modelStruct.modelPar_LNTRECR*d_Inv_L_dX+modelStruct.d_modelPar_LNTRECR_dX*Inv_L))+(modelStruct.modelPar_WNTRECR*d_Inv_W_dX+modelStruct.d_modelPar_WNTRECR_dX*Inv_W))+(modelStruct.modelPar_PNTRECR*d_Inv_LW_dX+modelStruct.d_modelPar_PNTRECR_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIntrecr = (((B4SOIntrecr+(modelStruct.modelPar_LNTRECR*Inv_L))+(modelStruct.modelPar_WNTRECR*Inv_W))+(modelStruct.modelPar_PNTRECR*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxbjt_dX = (((d_B4SOIxbjt_dX+(modelStruct.modelPar_LXBJT*d_Inv_L_dX+modelStruct.d_modelPar_LXBJT_dX*Inv_L))+(modelStruct.modelPar_WXBJT*d_Inv_W_dX+modelStruct.d_modelPar_WXBJT_dX*Inv_W))+(modelStruct.modelPar_PXBJT*d_Inv_LW_dX+modelStruct.d_modelPar_PXBJT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxbjt = (((B4SOIxbjt+(modelStruct.modelPar_LXBJT*Inv_L))+(modelStruct.modelPar_WXBJT*Inv_W))+(modelStruct.modelPar_PXBJT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxdif_dX = (((d_B4SOIxdif_dX+(modelStruct.modelPar_LXDIF*d_Inv_L_dX+modelStruct.d_modelPar_LXDIF_dX*Inv_L))+(modelStruct.modelPar_WXDIF*d_Inv_W_dX+modelStruct.d_modelPar_WXDIF_dX*Inv_W))+(modelStruct.modelPar_PXDIF*d_Inv_LW_dX+modelStruct.d_modelPar_PXDIF_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxdif = (((B4SOIxdif+(modelStruct.modelPar_LXDIF*Inv_L))+(modelStruct.modelPar_WXDIF*Inv_W))+(modelStruct.modelPar_PXDIF*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxrec_dX = (((d_B4SOIxrec_dX+(modelStruct.modelPar_LXREC*d_Inv_L_dX+modelStruct.d_modelPar_LXREC_dX*Inv_L))+(modelStruct.modelPar_WXREC*d_Inv_W_dX+modelStruct.d_modelPar_WXREC_dX*Inv_W))+(modelStruct.modelPar_PXREC*d_Inv_LW_dX+modelStruct.d_modelPar_PXREC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxrec = (((B4SOIxrec+(modelStruct.modelPar_LXREC*Inv_L))+(modelStruct.modelPar_WXREC*Inv_W))+(modelStruct.modelPar_PXREC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxtun_dX = (((d_B4SOIxtun_dX+(modelStruct.modelPar_LXTUN*d_Inv_L_dX+modelStruct.d_modelPar_LXTUN_dX*Inv_L))+(modelStruct.modelPar_WXTUN*d_Inv_W_dX+modelStruct.d_modelPar_WXTUN_dX*Inv_W))+(modelStruct.modelPar_PXTUN*d_Inv_LW_dX+modelStruct.d_modelPar_PXTUN_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxtun = (((B4SOIxtun+(modelStruct.modelPar_LXTUN*Inv_L))+(modelStruct.modelPar_WXTUN*Inv_W))+(modelStruct.modelPar_PXTUN*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxdifd_dX = (((d_B4SOIxdifd_dX+(modelStruct.modelPar_LXDIFD*d_Inv_L_dX+modelStruct.d_modelPar_LXDIFD_dX*Inv_L))+(modelStruct.modelPar_WXDIFD*d_Inv_W_dX+modelStruct.d_modelPar_WXDIFD_dX*Inv_W))+(modelStruct.modelPar_PXDIFD*d_Inv_LW_dX+modelStruct.d_modelPar_PXDIFD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxdifd = (((B4SOIxdifd+(modelStruct.modelPar_LXDIFD*Inv_L))+(modelStruct.modelPar_WXDIFD*Inv_W))+(modelStruct.modelPar_PXDIFD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxrecd_dX = (((d_B4SOIxrecd_dX+(modelStruct.modelPar_LXRECD*d_Inv_L_dX+modelStruct.d_modelPar_LXRECD_dX*Inv_L))+(modelStruct.modelPar_WXRECD*d_Inv_W_dX+modelStruct.d_modelPar_WXRECD_dX*Inv_W))+(modelStruct.modelPar_PXRECD*d_Inv_LW_dX+modelStruct.d_modelPar_PXRECD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxrecd = (((B4SOIxrecd+(modelStruct.modelPar_LXRECD*Inv_L))+(modelStruct.modelPar_WXRECD*Inv_W))+(modelStruct.modelPar_PXRECD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxtund_dX = (((d_B4SOIxtund_dX+(modelStruct.modelPar_LXTUND*d_Inv_L_dX+modelStruct.d_modelPar_LXTUND_dX*Inv_L))+(modelStruct.modelPar_WXTUND*d_Inv_W_dX+modelStruct.d_modelPar_WXTUND_dX*Inv_W))+(modelStruct.modelPar_PXTUND*d_Inv_LW_dX+modelStruct.d_modelPar_PXTUND_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxtund = (((B4SOIxtund+(modelStruct.modelPar_LXTUND*Inv_L))+(modelStruct.modelPar_WXTUND*Inv_W))+(modelStruct.modelPar_PXTUND*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcgdl_dX = (((d_B4SOIcgdl_dX+(modelStruct.modelPar_LCGDL*d_Inv_L_dX+modelStruct.d_modelPar_LCGDL_dX*Inv_L))+(modelStruct.modelPar_WCGDL*d_Inv_W_dX+modelStruct.d_modelPar_WCGDL_dX*Inv_W))+(modelStruct.modelPar_PCGDL*d_Inv_LW_dX+modelStruct.d_modelPar_PCGDL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcgdl = (((B4SOIcgdl+(modelStruct.modelPar_LCGDL*Inv_L))+(modelStruct.modelPar_WCGDL*Inv_W))+(modelStruct.modelPar_PCGDL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcgsl_dX = (((d_B4SOIcgsl_dX+(modelStruct.modelPar_LCGSL*d_Inv_L_dX+modelStruct.d_modelPar_LCGSL_dX*Inv_L))+(modelStruct.modelPar_WCGSL*d_Inv_W_dX+modelStruct.d_modelPar_WCGSL_dX*Inv_W))+(modelStruct.modelPar_PCGSL*d_Inv_LW_dX+modelStruct.d_modelPar_PCGSL_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcgsl = (((B4SOIcgsl+(modelStruct.modelPar_LCGSL*Inv_L))+(modelStruct.modelPar_WCGSL*Inv_W))+(modelStruct.modelPar_PCGSL*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX = (((d_B4SOIckappa_dX+(modelStruct.modelPar_LCKAPPA*d_Inv_L_dX+modelStruct.d_modelPar_LCKAPPA_dX*Inv_L))+(modelStruct.modelPar_WCKAPPA*d_Inv_W_dX+modelStruct.d_modelPar_WCKAPPA_dX*Inv_W))+(modelStruct.modelPar_PCKAPPA*d_Inv_LW_dX+modelStruct.d_modelPar_PCKAPPA_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIckappa = (((B4SOIckappa+(modelStruct.modelPar_LCKAPPA*Inv_L))+(modelStruct.modelPar_WCKAPPA*Inv_W))+(modelStruct.modelPar_PCKAPPA*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIute_dX = (((d_B4SOIute_dX+(modelStruct.modelPar_LUTE*d_Inv_L_dX+modelStruct.d_modelPar_LUTE_dX*Inv_L))+(modelStruct.modelPar_WUTE*d_Inv_W_dX+modelStruct.d_modelPar_WUTE_dX*Inv_W))+(modelStruct.modelPar_PUTE*d_Inv_LW_dX+modelStruct.d_modelPar_PUTE_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIute = (((B4SOIute+(modelStruct.modelPar_LUTE*Inv_L))+(modelStruct.modelPar_WUTE*Inv_W))+(modelStruct.modelPar_PUTE*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIud_dX = (((d_B4SOIud_dX+(modelStruct.modelPar_LUD*d_Inv_L_dX+modelStruct.d_modelPar_LUD_dX*Inv_L))+(modelStruct.modelPar_WUD*d_Inv_W_dX+modelStruct.d_modelPar_WUD_dX*Inv_W))+(modelStruct.modelPar_PUD*d_Inv_LW_dX+modelStruct.d_modelPar_PUD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIud = (((B4SOIud+(modelStruct.modelPar_LUD*Inv_L))+(modelStruct.modelPar_WUD*Inv_W))+(modelStruct.modelPar_PUD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIud1_dX = (((d_B4SOIud1_dX+(modelStruct.modelPar_LUD1*d_Inv_L_dX+modelStruct.d_modelPar_LUD1_dX*Inv_L))+(modelStruct.modelPar_WUD1*d_Inv_W_dX+modelStruct.d_modelPar_WUD1_dX*Inv_W))+(modelStruct.modelPar_PUD1*d_Inv_LW_dX+modelStruct.d_modelPar_PUD1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIud1 = (((B4SOIud1+(modelStruct.modelPar_LUD1*Inv_L))+(modelStruct.modelPar_WUD1*Inv_W))+(modelStruct.modelPar_PUD1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIeu_dX = (((d_B4SOIeu_dX+(modelStruct.modelPar_LEU*d_Inv_L_dX+modelStruct.d_modelPar_LEU_dX*Inv_L))+(modelStruct.modelPar_WEU*d_Inv_W_dX+modelStruct.d_modelPar_WEU_dX*Inv_W))+(modelStruct.modelPar_PEU*d_Inv_LW_dX+modelStruct.d_modelPar_PEU_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIeu = (((B4SOIeu+(modelStruct.modelPar_LEU*Inv_L))+(modelStruct.modelPar_WEU*Inv_W))+(modelStruct.modelPar_PEU*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIucs_dX = (((d_B4SOIucs_dX+(modelStruct.modelPar_LUCS*d_Inv_L_dX+modelStruct.d_modelPar_LUCS_dX*Inv_L))+(modelStruct.modelPar_WUCS*d_Inv_W_dX+modelStruct.d_modelPar_WUCS_dX*Inv_W))+(modelStruct.modelPar_PUCS*d_Inv_LW_dX+modelStruct.d_modelPar_PUCS_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIucs = (((B4SOIucs+(modelStruct.modelPar_LUCS*Inv_L))+(modelStruct.modelPar_WUCS*Inv_W))+(modelStruct.modelPar_PUCS*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIucste_dX = (((d_B4SOIucste_dX+(modelStruct.modelPar_LUCSTE*d_Inv_L_dX+modelStruct.d_modelPar_LUCSTE_dX*Inv_L))+(modelStruct.modelPar_WUCSTE*d_Inv_W_dX+modelStruct.d_modelPar_WUCSTE_dX*Inv_W))+(modelStruct.modelPar_PUCSTE*d_Inv_LW_dX+modelStruct.d_modelPar_PUCSTE_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIucste = (((B4SOIucste+(modelStruct.modelPar_LUCSTE*Inv_L))+(modelStruct.modelPar_WUCSTE*Inv_W))+(modelStruct.modelPar_PUCSTE*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX = (((d_B4SOIkt1_dX+(modelStruct.modelPar_LKT1*d_Inv_L_dX+modelStruct.d_modelPar_LKT1_dX*Inv_L))+(modelStruct.modelPar_WKT1*d_Inv_W_dX+modelStruct.d_modelPar_WKT1_dX*Inv_W))+(modelStruct.modelPar_PKT1*d_Inv_LW_dX+modelStruct.d_modelPar_PKT1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIkt1 = (((B4SOIkt1+(modelStruct.modelPar_LKT1*Inv_L))+(modelStruct.modelPar_WKT1*Inv_W))+(modelStruct.modelPar_PKT1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIkt2_dX = (((d_B4SOIkt2_dX+(modelStruct.modelPar_LKT2*d_Inv_L_dX+modelStruct.d_modelPar_LKT2_dX*Inv_L))+(modelStruct.modelPar_WKT2*d_Inv_W_dX+modelStruct.d_modelPar_WKT2_dX*Inv_W))+(modelStruct.modelPar_PKT2*d_Inv_LW_dX+modelStruct.d_modelPar_PKT2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIkt2 = (((B4SOIkt2+(modelStruct.modelPar_LKT2*Inv_L))+(modelStruct.modelPar_WKT2*Inv_W))+(modelStruct.modelPar_PKT2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX = (((d_B4SOIkt1l_dX+(modelStruct.modelPar_LKT1L*d_Inv_L_dX+modelStruct.d_modelPar_LKT1L_dX*Inv_L))+(modelStruct.modelPar_WKT1L*d_Inv_W_dX+modelStruct.d_modelPar_WKT1L_dX*Inv_W))+(modelStruct.modelPar_PKT1L*d_Inv_LW_dX+modelStruct.d_modelPar_PKT1L_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIkt1l = (((B4SOIkt1l+(modelStruct.modelPar_LKT1L*Inv_L))+(modelStruct.modelPar_WKT1L*Inv_W))+(modelStruct.modelPar_PKT1L*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIua1_dX = (((d_B4SOIua1_dX+(modelStruct.modelPar_LUA1*d_Inv_L_dX+modelStruct.d_modelPar_LUA1_dX*Inv_L))+(modelStruct.modelPar_WUA1*d_Inv_W_dX+modelStruct.d_modelPar_WUA1_dX*Inv_W))+(modelStruct.modelPar_PUA1*d_Inv_LW_dX+modelStruct.d_modelPar_PUA1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIua1 = (((B4SOIua1+(modelStruct.modelPar_LUA1*Inv_L))+(modelStruct.modelPar_WUA1*Inv_W))+(modelStruct.modelPar_PUA1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIub1_dX = (((d_B4SOIub1_dX+(modelStruct.modelPar_LUB1*d_Inv_L_dX+modelStruct.d_modelPar_LUB1_dX*Inv_L))+(modelStruct.modelPar_WUB1*d_Inv_W_dX+modelStruct.d_modelPar_WUB1_dX*Inv_W))+(modelStruct.modelPar_PUB1*d_Inv_LW_dX+modelStruct.d_modelPar_PUB1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIub1 = (((B4SOIub1+(modelStruct.modelPar_LUB1*Inv_L))+(modelStruct.modelPar_WUB1*Inv_W))+(modelStruct.modelPar_PUB1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIuc1_dX = (((d_B4SOIuc1_dX+(modelStruct.modelPar_LUC1*d_Inv_L_dX+modelStruct.d_modelPar_LUC1_dX*Inv_L))+(modelStruct.modelPar_WUC1*d_Inv_W_dX+modelStruct.d_modelPar_WUC1_dX*Inv_W))+(modelStruct.modelPar_PUC1*d_Inv_LW_dX+modelStruct.d_modelPar_PUC1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIuc1 = (((B4SOIuc1+(modelStruct.modelPar_LUC1*Inv_L))+(modelStruct.modelPar_WUC1*Inv_W))+(modelStruct.modelPar_PUC1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIat_dX = (((d_B4SOIat_dX+(modelStruct.modelPar_LAT*d_Inv_L_dX+modelStruct.d_modelPar_LAT_dX*Inv_L))+(modelStruct.modelPar_WAT*d_Inv_W_dX+modelStruct.d_modelPar_WAT_dX*Inv_W))+(modelStruct.modelPar_PAT*d_Inv_LW_dX+modelStruct.d_modelPar_PAT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIat = (((B4SOIat+(modelStruct.modelPar_LAT*Inv_L))+(modelStruct.modelPar_WAT*Inv_W))+(modelStruct.modelPar_PAT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIprt_dX = (((d_B4SOIprt_dX+(modelStruct.modelPar_LPRT*d_Inv_L_dX+modelStruct.d_modelPar_LPRT_dX*Inv_L))+(modelStruct.modelPar_WPRT*d_Inv_W_dX+modelStruct.d_modelPar_WPRT_dX*Inv_W))+(modelStruct.modelPar_PPRT*d_Inv_LW_dX+modelStruct.d_modelPar_PPRT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIprt = (((B4SOIprt+(modelStruct.modelPar_LPRT*Inv_L))+(modelStruct.modelPar_WPRT*Inv_W))+(modelStruct.modelPar_PPRT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInigc_dX = (((d_B4SOInigc_dX+(modelStruct.modelPar_LNIGC*d_Inv_L_dX+modelStruct.d_modelPar_LNIGC_dX*Inv_L))+(modelStruct.modelPar_WNIGC*d_Inv_W_dX+modelStruct.d_modelPar_WNIGC_dX*Inv_W))+(modelStruct.modelPar_PNIGC*d_Inv_LW_dX+modelStruct.d_modelPar_PNIGC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInigc = (((B4SOInigc+(modelStruct.modelPar_LNIGC*Inv_L))+(modelStruct.modelPar_WNIGC*Inv_W))+(modelStruct.modelPar_PNIGC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIaigc_dX = (((d_B4SOIaigc_dX+(modelStruct.modelPar_LAIGC*d_Inv_L_dX+modelStruct.d_modelPar_LAIGC_dX*Inv_L))+(modelStruct.modelPar_WAIGC*d_Inv_W_dX+modelStruct.d_modelPar_WAIGC_dX*Inv_W))+(modelStruct.modelPar_PAIGC*d_Inv_LW_dX+modelStruct.d_modelPar_PAIGC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIaigc = (((B4SOIaigc+(modelStruct.modelPar_LAIGC*Inv_L))+(modelStruct.modelPar_WAIGC*Inv_W))+(modelStruct.modelPar_PAIGC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbigc_dX = (((d_B4SOIbigc_dX+(modelStruct.modelPar_LBIGC*d_Inv_L_dX+modelStruct.d_modelPar_LBIGC_dX*Inv_L))+(modelStruct.modelPar_WBIGC*d_Inv_W_dX+modelStruct.d_modelPar_WBIGC_dX*Inv_W))+(modelStruct.modelPar_PBIGC*d_Inv_LW_dX+modelStruct.d_modelPar_PBIGC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbigc = (((B4SOIbigc+(modelStruct.modelPar_LBIGC*Inv_L))+(modelStruct.modelPar_WBIGC*Inv_W))+(modelStruct.modelPar_PBIGC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcigc_dX = (((d_B4SOIcigc_dX+(modelStruct.modelPar_LCIGC*d_Inv_L_dX+modelStruct.d_modelPar_LCIGC_dX*Inv_L))+(modelStruct.modelPar_WCIGC*d_Inv_W_dX+modelStruct.d_modelPar_WCIGC_dX*Inv_W))+(modelStruct.modelPar_PCIGC*d_Inv_LW_dX+modelStruct.d_modelPar_PCIGC_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcigc = (((B4SOIcigc+(modelStruct.modelPar_LCIGC*Inv_L))+(modelStruct.modelPar_WCIGC*Inv_W))+(modelStruct.modelPar_PCIGC*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIaigsd_dX = (((d_B4SOIaigsd_dX+(modelStruct.modelPar_LAIGSD*d_Inv_L_dX+modelStruct.d_modelPar_LAIGSD_dX*Inv_L))+(modelStruct.modelPar_WAIGSD*d_Inv_W_dX+modelStruct.d_modelPar_WAIGSD_dX*Inv_W))+(modelStruct.modelPar_PAIGSD*d_Inv_LW_dX+modelStruct.d_modelPar_PAIGSD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIaigsd = (((B4SOIaigsd+(modelStruct.modelPar_LAIGSD*Inv_L))+(modelStruct.modelPar_WAIGSD*Inv_W))+(modelStruct.modelPar_PAIGSD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbigsd_dX = (((d_B4SOIbigsd_dX+(modelStruct.modelPar_LBIGSD*d_Inv_L_dX+modelStruct.d_modelPar_LBIGSD_dX*Inv_L))+(modelStruct.modelPar_WBIGSD*d_Inv_W_dX+modelStruct.d_modelPar_WBIGSD_dX*Inv_W))+(modelStruct.modelPar_PBIGSD*d_Inv_LW_dX+modelStruct.d_modelPar_PBIGSD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbigsd = (((B4SOIbigsd+(modelStruct.modelPar_LBIGSD*Inv_L))+(modelStruct.modelPar_WBIGSD*Inv_W))+(modelStruct.modelPar_PBIGSD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcigsd_dX = (((d_B4SOIcigsd_dX+(modelStruct.modelPar_LCIGSD*d_Inv_L_dX+modelStruct.d_modelPar_LCIGSD_dX*Inv_L))+(modelStruct.modelPar_WCIGSD*d_Inv_W_dX+modelStruct.d_modelPar_WCIGSD_dX*Inv_W))+(modelStruct.modelPar_PCIGSD*d_Inv_LW_dX+modelStruct.d_modelPar_PCIGSD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcigsd = (((B4SOIcigsd+(modelStruct.modelPar_LCIGSD*Inv_L))+(modelStruct.modelPar_WCIGSD*Inv_W))+(modelStruct.modelPar_PCIGSD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpigcd_dX = (((d_B4SOIpigcd_dX+(modelStruct.modelPar_LPIGCD*d_Inv_L_dX+modelStruct.d_modelPar_LPIGCD_dX*Inv_L))+(modelStruct.modelPar_WPIGCD*d_Inv_W_dX+modelStruct.d_modelPar_WPIGCD_dX*Inv_W))+(modelStruct.modelPar_PPIGCD*d_Inv_LW_dX+modelStruct.d_modelPar_PPIGCD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpigcd = (((B4SOIpigcd+(modelStruct.modelPar_LPIGCD*Inv_L))+(modelStruct.modelPar_WPIGCD*Inv_W))+(modelStruct.modelPar_PPIGCD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpoxedge_dX = (((d_B4SOIpoxedge_dX+(modelStruct.modelPar_LPOXEDGE*d_Inv_L_dX+modelStruct.d_modelPar_LPOXEDGE_dX*Inv_L))+(modelStruct.modelPar_WPOXEDGE*d_Inv_W_dX+modelStruct.d_modelPar_WPOXEDGE_dX*Inv_W))+(modelStruct.modelPar_PPOXEDGE*d_Inv_LW_dX+modelStruct.d_modelPar_PPOXEDGE_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpoxedge = (((B4SOIpoxedge+(modelStruct.modelPar_LPOXEDGE*Inv_L))+(modelStruct.modelPar_WPOXEDGE*Inv_W))+(modelStruct.modelPar_PPOXEDGE*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxrcrg1_dX = (((d_B4SOIxrcrg1_dX+(modelStruct.modelPar_LXRCRG1*d_Inv_L_dX+modelStruct.d_modelPar_LXRCRG1_dX*Inv_L))+(modelStruct.modelPar_WXRCRG1*d_Inv_W_dX+modelStruct.d_modelPar_WXRCRG1_dX*Inv_W))+(modelStruct.modelPar_PXRCRG1*d_Inv_LW_dX+modelStruct.d_modelPar_PXRCRG1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxrcrg1 = (((B4SOIxrcrg1+(modelStruct.modelPar_LXRCRG1*Inv_L))+(modelStruct.modelPar_WXRCRG1*Inv_W))+(modelStruct.modelPar_PXRCRG1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIxrcrg2_dX = (((d_B4SOIxrcrg2_dX+(modelStruct.modelPar_LXRCRG2*d_Inv_L_dX+modelStruct.d_modelPar_LXRCRG2_dX*Inv_L))+(modelStruct.modelPar_WXRCRG2*d_Inv_W_dX+modelStruct.d_modelPar_WXRCRG2_dX*Inv_W))+(modelStruct.modelPar_PXRCRG2*d_Inv_LW_dX+modelStruct.d_modelPar_PXRCRG2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIxrcrg2 = (((B4SOIxrcrg2+(modelStruct.modelPar_LXRCRG2*Inv_L))+(modelStruct.modelPar_WXRCRG2*Inv_W))+(modelStruct.modelPar_PXRCRG2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX = (((d_B4SOIvbsa_dX+(modelStruct.modelPar_LVBSA*d_Inv_L_dX+modelStruct.d_modelPar_LVBSA_dX*Inv_L))+(modelStruct.modelPar_WVBSA*d_Inv_W_dX+modelStruct.d_modelPar_WVBSA_dX*Inv_W))+(modelStruct.modelPar_PVBSA*d_Inv_LW_dX+modelStruct.d_modelPar_PVBSA_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvbsa = (((B4SOIvbsa+(modelStruct.modelPar_LVBSA*Inv_L))+(modelStruct.modelPar_WVBSA*Inv_W))+(modelStruct.modelPar_PVBSA*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvsce_dX = (((d_B4SOIvsce_dX+(modelStruct.modelPar_LVSCE*d_Inv_L_dX+modelStruct.d_modelPar_LVSCE_dX*Inv_L))+(modelStruct.modelPar_WVSCE*d_Inv_W_dX+modelStruct.d_modelPar_WVSCE_dX*Inv_W))+(modelStruct.modelPar_PVSCE*d_Inv_LW_dX+modelStruct.d_modelPar_PVSCE_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvsce = (((B4SOIvsce+(modelStruct.modelPar_LVSCE*Inv_L))+(modelStruct.modelPar_WVSCE*Inv_W))+(modelStruct.modelPar_PVSCE*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX = (((d_B4SOIcdsbs_dX+(modelStruct.modelPar_LCDSBS*d_Inv_L_dX+modelStruct.d_modelPar_LCDSBS_dX*Inv_L))+(modelStruct.modelPar_WCDSBS*d_Inv_W_dX+modelStruct.d_modelPar_WCDSBS_dX*Inv_W))+(modelStruct.modelPar_PCDSBS*d_Inv_LW_dX+modelStruct.d_modelPar_PCDSBS_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcdsbs = (((B4SOIcdsbs+(modelStruct.modelPar_LCDSBS*Inv_L))+(modelStruct.modelPar_WCDSBS*Inv_W))+(modelStruct.modelPar_PCDSBS*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInofffd_dX = (((d_B4SOInofffd_dX+(modelStruct.modelPar_LNOFFFD*d_Inv_L_dX+modelStruct.d_modelPar_LNOFFFD_dX*Inv_L))+(modelStruct.modelPar_WNOFFFD*d_Inv_W_dX+modelStruct.d_modelPar_WNOFFFD_dX*Inv_W))+(modelStruct.modelPar_PNOFFFD*d_Inv_LW_dX+modelStruct.d_modelPar_PNOFFFD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInofffd = (((B4SOInofffd+(modelStruct.modelPar_LNOFFFD*Inv_L))+(modelStruct.modelPar_WNOFFFD*Inv_W))+(modelStruct.modelPar_PNOFFFD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX = (((d_B4SOIvofffd_dX+(modelStruct.modelPar_LVOFFFD*d_Inv_L_dX+modelStruct.d_modelPar_LVOFFFD_dX*Inv_L))+(modelStruct.modelPar_WVOFFFD*d_Inv_W_dX+modelStruct.d_modelPar_WVOFFFD_dX*Inv_W))+(modelStruct.modelPar_PVOFFFD*d_Inv_LW_dX+modelStruct.d_modelPar_PVOFFFD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvofffd = (((B4SOIvofffd+(modelStruct.modelPar_LVOFFFD*Inv_L))+(modelStruct.modelPar_WVOFFFD*Inv_W))+(modelStruct.modelPar_PVOFFFD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk1b_dX = (((d_B4SOIk1b_dX+(modelStruct.modelPar_LK1B*d_Inv_L_dX+modelStruct.d_modelPar_LK1B_dX*Inv_L))+(modelStruct.modelPar_WK1B*d_Inv_W_dX+modelStruct.d_modelPar_WK1B_dX*Inv_W))+(modelStruct.modelPar_PK1B*d_Inv_LW_dX+modelStruct.d_modelPar_PK1B_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk1b = (((B4SOIk1b+(modelStruct.modelPar_LK1B*Inv_L))+(modelStruct.modelPar_WK1B*Inv_W))+(modelStruct.modelPar_PK1B*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIk2b_dX = (((d_B4SOIk2b_dX+(modelStruct.modelPar_LK2B*d_Inv_L_dX+modelStruct.d_modelPar_LK2B_dX*Inv_L))+(modelStruct.modelPar_WK2B*d_Inv_W_dX+modelStruct.d_modelPar_WK2B_dX*Inv_W))+(modelStruct.modelPar_PK2B*d_Inv_LW_dX+modelStruct.d_modelPar_PK2B_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIk2b = (((B4SOIk2b+(modelStruct.modelPar_LK2B*Inv_L))+(modelStruct.modelPar_WK2B*Inv_W))+(modelStruct.modelPar_PK2B*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdk2b_dX = (((d_B4SOIdk2b_dX+(modelStruct.modelPar_LDK2B*d_Inv_L_dX+modelStruct.d_modelPar_LDK2B_dX*Inv_L))+(modelStruct.modelPar_WDK2B*d_Inv_W_dX+modelStruct.d_modelPar_WDK2B_dX*Inv_W))+(modelStruct.modelPar_PDK2B*d_Inv_LW_dX+modelStruct.d_modelPar_PDK2B_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdk2b = (((B4SOIdk2b+(modelStruct.modelPar_LDK2B*Inv_L))+(modelStruct.modelPar_WDK2B*Inv_W))+(modelStruct.modelPar_PDK2B*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX = (((d_B4SOIdvbd0_dX+(modelStruct.modelPar_LDVBD0*d_Inv_L_dX+modelStruct.d_modelPar_LDVBD0_dX*Inv_L))+(modelStruct.modelPar_WDVBD0*d_Inv_W_dX+modelStruct.d_modelPar_WDVBD0_dX*Inv_W))+(modelStruct.modelPar_PDVBD0*d_Inv_LW_dX+modelStruct.d_modelPar_PDVBD0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvbd0 = (((B4SOIdvbd0+(modelStruct.modelPar_LDVBD0*Inv_L))+(modelStruct.modelPar_WDVBD0*Inv_W))+(modelStruct.modelPar_PDVBD0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX = (((d_B4SOIdvbd1_dX+(modelStruct.modelPar_LDVBD1*d_Inv_L_dX+modelStruct.d_modelPar_LDVBD1_dX*Inv_L))+(modelStruct.modelPar_WDVBD1*d_Inv_W_dX+modelStruct.d_modelPar_WDVBD1_dX*Inv_W))+(modelStruct.modelPar_PDVBD1*d_Inv_LW_dX+modelStruct.d_modelPar_PDVBD1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvbd1 = (((B4SOIdvbd1+(modelStruct.modelPar_LDVBD1*Inv_L))+(modelStruct.modelPar_WDVBD1*Inv_W))+(modelStruct.modelPar_PDVBD1*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOImoinfd_dX = (((d_B4SOImoinFD_dX+(modelStruct.modelPar_LMOINFD*d_Inv_L_dX+modelStruct.d_modelPar_LMOINFD_dX*Inv_L))+(modelStruct.modelPar_WMOINFD*d_Inv_W_dX+modelStruct.d_modelPar_WMOINFD_dX*Inv_W))+(modelStruct.modelPar_PMOINFD*d_Inv_LW_dX+modelStruct.d_modelPar_PMOINFD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOImoinfd = (((B4SOImoinFD+(modelStruct.modelPar_LMOINFD*Inv_L))+(modelStruct.modelPar_WMOINFD*Inv_W))+(modelStruct.modelPar_PMOINFD*Inv_LW));
d_pParam_B4SOIvbs0pd_dX = (((d_B4SOIvbs0pd_dX+(modelStruct.modelPar_LVBS0PD*d_Inv_L_dX+modelStruct.d_modelPar_LVBS0PD_dX*Inv_L))+(modelStruct.modelPar_WVBS0PD*d_Inv_W_dX+modelStruct.d_modelPar_WVBS0PD_dX*Inv_W))+(modelStruct.modelPar_PVBS0PD*d_Inv_LW_dX+modelStruct.d_modelPar_PVBS0PD_dX*Inv_LW));
pParam_B4SOIvbs0pd = (((B4SOIvbs0pd+(modelStruct.modelPar_LVBS0PD*Inv_L))+(modelStruct.modelPar_WVBS0PD*Inv_W))+(modelStruct.modelPar_PVBS0PD*Inv_LW));
d_pParam_B4SOIvbs0fd_dX = (((d_B4SOIvbs0fd_dX+(modelStruct.modelPar_LVBS0FD*d_Inv_L_dX+modelStruct.d_modelPar_LVBS0FD_dX*Inv_L))+(modelStruct.modelPar_WVBS0FD*d_Inv_W_dX+modelStruct.d_modelPar_WVBS0FD_dX*Inv_W))+(modelStruct.modelPar_PVBS0FD*d_Inv_LW_dX+modelStruct.d_modelPar_PVBS0FD_dX*Inv_LW));
pParam_B4SOIvbs0fd = (((B4SOIvbs0fd+(modelStruct.modelPar_LVBS0FD*Inv_L))+(modelStruct.modelPar_WVBS0FD*Inv_W))+(modelStruct.modelPar_PVBS0FD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX = (((d_B4SOIvsdfb_dX+(modelStruct.modelPar_LVSDFB*d_Inv_L_dX+modelStruct.d_modelPar_LVSDFB_dX*Inv_L))+(modelStruct.modelPar_WVSDFB*d_Inv_W_dX+modelStruct.d_modelPar_WVSDFB_dX*Inv_W))+(modelStruct.modelPar_PVSDFB*d_Inv_LW_dX+modelStruct.d_modelPar_PVSDFB_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvsdfb = (((B4SOIvsdfb+(modelStruct.modelPar_LVSDFB*Inv_L))+(modelStruct.modelPar_WVSDFB*Inv_W))+(modelStruct.modelPar_PVSDFB*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX = (((d_B4SOIvsdth_dX+(modelStruct.modelPar_LVSDTH*d_Inv_L_dX+modelStruct.d_modelPar_LVSDTH_dX*Inv_L))+(modelStruct.modelPar_WVSDTH*d_Inv_W_dX+modelStruct.d_modelPar_WVSDTH_dX*Inv_W))+(modelStruct.modelPar_PVSDTH*d_Inv_LW_dX+modelStruct.d_modelPar_PVSDTH_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvsdth = (((B4SOIvsdth+(modelStruct.modelPar_LVSDTH*Inv_L))+(modelStruct.modelPar_WVSDTH*Inv_W))+(modelStruct.modelPar_PVSDTH*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX = (((d_B4SOIdelvt_dX+(modelStruct.modelPar_LDELVT*d_Inv_L_dX+modelStruct.d_modelPar_LDELVT_dX*Inv_L))+(modelStruct.modelPar_WDELVT*d_Inv_W_dX+modelStruct.d_modelPar_WDELVT_dX*Inv_W))+(modelStruct.modelPar_PDELVT*d_Inv_LW_dX+modelStruct.d_modelPar_PDELVT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdelvt = (((B4SOIdelvt+(modelStruct.modelPar_LDELVT*Inv_L))+(modelStruct.modelPar_WDELVT*Inv_W))+(modelStruct.modelPar_PDELVT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIacde_dX = (((instanceStruct.d_instanceVar_B4SOIacde_dX+(modelStruct.modelPar_LACDE*d_Inv_L_dX+modelStruct.d_modelPar_LACDE_dX*Inv_L))+(modelStruct.modelPar_WACDE*d_Inv_W_dX+modelStruct.d_modelPar_WACDE_dX*Inv_W))+(modelStruct.modelPar_PACDE*d_Inv_LW_dX+modelStruct.d_modelPar_PACDE_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIacde = (((instanceStruct.instanceVar_B4SOIacde+(modelStruct.modelPar_LACDE*Inv_L))+(modelStruct.modelPar_WACDE*Inv_W))+(modelStruct.modelPar_PACDE*Inv_LW));
{
double value_pow_0 = pow((instanceStruct.instanceVar_pParam_B4SOInpeak/2.0e16),(-0.25));
double  deriv_pow_0_d0 = (((instanceStruct.instanceVar_pParam_B4SOInpeak/2.0e16) == 0.0)?0.0:(value_pow_0*(-0.25)/(instanceStruct.instanceVar_pParam_B4SOInpeak/2.0e16)));
instanceStruct.d_instanceVar_pParam_B4SOIacde_dX = (instanceStruct.instanceVar_pParam_B4SOIacde*(deriv_pow_0_d0*((instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX/2.0e16)))+instanceStruct.d_instanceVar_pParam_B4SOIacde_dX*value_pow_0);
instanceStruct.instanceVar_pParam_B4SOIacde = (instanceStruct.instanceVar_pParam_B4SOIacde*value_pow_0);
}
instanceStruct.d_instanceVar_pParam_B4SOImoin_dX = (((instanceStruct.d_instanceVar_B4SOImoin_dX+(modelStruct.modelPar_LMOIN*d_Inv_L_dX+modelStruct.d_modelPar_LMOIN_dX*Inv_L))+(modelStruct.modelPar_WMOIN*d_Inv_W_dX+modelStruct.d_modelPar_WMOIN_dX*Inv_W))+(modelStruct.modelPar_PMOIN*d_Inv_LW_dX+modelStruct.d_modelPar_PMOIN_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOImoin = (((instanceStruct.instanceVar_B4SOImoin+(modelStruct.modelPar_LMOIN*Inv_L))+(modelStruct.modelPar_WMOIN*Inv_W))+(modelStruct.modelPar_PMOIN*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInoff_dX = (((d_B4SOInoff_dX+(modelStruct.modelPar_LNOFF*d_Inv_L_dX+modelStruct.d_modelPar_LNOFF_dX*Inv_L))+(modelStruct.modelPar_WNOFF*d_Inv_W_dX+modelStruct.d_modelPar_WNOFF_dX*Inv_W))+(modelStruct.modelPar_PNOFF*d_Inv_LW_dX+modelStruct.d_modelPar_PNOFF_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInoff = (((B4SOInoff+(modelStruct.modelPar_LNOFF*Inv_L))+(modelStruct.modelPar_WNOFF*Inv_W))+(modelStruct.modelPar_PNOFF*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOInoff2_dX = (((d_B4SOInoff2_dX+(modelStruct.modelPar_LNOFF2*d_Inv_L_dX+modelStruct.d_modelPar_LNOFF2_dX*Inv_L))+(modelStruct.modelPar_WNOFF2*d_Inv_W_dX+modelStruct.d_modelPar_WNOFF2_dX*Inv_W))+(modelStruct.modelPar_PNOFF2*d_Inv_LW_dX+modelStruct.d_modelPar_PNOFF2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOInoff2 = (((B4SOInoff2+(modelStruct.modelPar_LNOFF2*Inv_L))+(modelStruct.modelPar_WNOFF2*Inv_W))+(modelStruct.modelPar_PNOFF2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvtp0_dX = (((d_B4SOIdvtp0_dX+(modelStruct.modelPar_LDVTP0*d_Inv_L_dX+modelStruct.d_modelPar_LDVTP0_dX*Inv_L))+(modelStruct.modelPar_WDVTP0*d_Inv_W_dX+modelStruct.d_modelPar_WDVTP0_dX*Inv_W))+(modelStruct.modelPar_PDVTP0*d_Inv_LW_dX+modelStruct.d_modelPar_PDVTP0_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvtp0 = (((B4SOIdvtp0+(modelStruct.modelPar_LDVTP0*Inv_L))+(modelStruct.modelPar_WDVTP0*Inv_W))+(modelStruct.modelPar_PDVTP0*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvtp1_dX = (((d_B4SOIdvtp1_dX+(modelStruct.modelPar_LDVTP1*d_Inv_L_dX+modelStruct.d_modelPar_LDVTP1_dX*Inv_L))+(modelStruct.modelPar_WDVTP1*d_Inv_W_dX+modelStruct.d_modelPar_WDVTP1_dX*Inv_W))+(modelStruct.modelPar_PDVTP1*d_Inv_LW_dX+modelStruct.d_modelPar_PDVTP1_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvtp1 = (((B4SOIdvtp1+(modelStruct.modelPar_LDVTP1*Inv_L))+(modelStruct.modelPar_WDVTP1*Inv_W))+(modelStruct.modelPar_PDVTP1*Inv_LW));
d_pParam_B4SOIdvtp2_dX = (((d_B4SOIdvtp2_dX+(modelStruct.modelPar_LDVTP2*d_Inv_L_dX+modelStruct.d_modelPar_LDVTP2_dX*Inv_L))+(modelStruct.modelPar_WDVTP2*d_Inv_W_dX+modelStruct.d_modelPar_WDVTP2_dX*Inv_W))+(modelStruct.modelPar_PDVTP2*d_Inv_LW_dX+modelStruct.d_modelPar_PDVTP2_dX*Inv_LW));
pParam_B4SOIdvtp2 = (((B4SOIdvtp2+(modelStruct.modelPar_LDVTP2*Inv_L))+(modelStruct.modelPar_WDVTP2*Inv_W))+(modelStruct.modelPar_PDVTP2*Inv_LW));
d_pParam_B4SOIdvtp3_dX = (((d_B4SOIdvtp3_dX+(modelStruct.modelPar_LDVTP3*d_Inv_L_dX+modelStruct.d_modelPar_LDVTP3_dX*Inv_L))+(modelStruct.modelPar_WDVTP3*d_Inv_W_dX+modelStruct.d_modelPar_WDVTP3_dX*Inv_W))+(modelStruct.modelPar_PDVTP3*d_Inv_LW_dX+modelStruct.d_modelPar_PDVTP3_dX*Inv_LW));
pParam_B4SOIdvtp3 = (((B4SOIdvtp3+(modelStruct.modelPar_LDVTP3*Inv_L))+(modelStruct.modelPar_WDVTP3*Inv_W))+(modelStruct.modelPar_PDVTP3*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIdvtp4_dX = (((d_B4SOIdvtp4_dX+(modelStruct.modelPar_LDVTP4*d_Inv_L_dX+modelStruct.d_modelPar_LDVTP4_dX*Inv_L))+(modelStruct.modelPar_WDVTP4*d_Inv_W_dX+modelStruct.d_modelPar_WDVTP4_dX*Inv_W))+(modelStruct.modelPar_PDVTP4*d_Inv_LW_dX+modelStruct.d_modelPar_PDVTP4_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIdvtp4 = (((B4SOIdvtp4+(modelStruct.modelPar_LDVTP4*Inv_L))+(modelStruct.modelPar_WDVTP4*Inv_W))+(modelStruct.modelPar_PDVTP4*Inv_LW));
d_pParam_B4SOIminv_dX = (((d_B4SOIminv_dX+(modelStruct.modelPar_LMINV*d_Inv_L_dX+modelStruct.d_modelPar_LMINV_dX*Inv_L))+(modelStruct.modelPar_WMINV*d_Inv_W_dX+modelStruct.d_modelPar_WMINV_dX*Inv_W))+(modelStruct.modelPar_PMINV*d_Inv_LW_dX+modelStruct.d_modelPar_PMINV_dX*Inv_LW));
pParam_B4SOIminv = (((B4SOIminv+(modelStruct.modelPar_LMINV*Inv_L))+(modelStruct.modelPar_WMINV*Inv_W))+(modelStruct.modelPar_PMINV*Inv_LW));
d_pParam_B4SOIminvcv_dX = (((d_B4SOIminvcv_dX+(modelStruct.modelPar_LMINVCV*d_Inv_L_dX+modelStruct.d_modelPar_LMINVCV_dX*Inv_L))+(modelStruct.modelPar_WMINVCV*d_Inv_W_dX+modelStruct.d_modelPar_WMINVCV_dX*Inv_W))+(modelStruct.modelPar_PMINVCV*d_Inv_LW_dX+modelStruct.d_modelPar_PMINVCV_dX*Inv_LW));
pParam_B4SOIminvcv = (((B4SOIminvcv+(modelStruct.modelPar_LMINVCV*Inv_L))+(modelStruct.modelPar_WMINVCV*Inv_W))+(modelStruct.modelPar_PMINVCV*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIvoffcv_dX = (((d_B4SOIvoffcv_dX+(modelStruct.modelPar_LVOFFCV*d_Inv_L_dX+modelStruct.d_modelPar_LVOFFCV_dX*Inv_L))+(modelStruct.modelPar_WVOFFCV*d_Inv_W_dX+modelStruct.d_modelPar_WVOFFCV_dX*Inv_W))+(modelStruct.modelPar_PVOFFCV*d_Inv_LW_dX+modelStruct.d_modelPar_PVOFFCV_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIvoffcv = (((B4SOIvoffcv+(modelStruct.modelPar_LVOFFCV*Inv_L))+(modelStruct.modelPar_WVOFFCV*Inv_W))+(modelStruct.modelPar_PVOFFCV*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIfprout_dX = (((d_B4SOIfprout_dX+(modelStruct.modelPar_LFPROUT*d_Inv_L_dX+modelStruct.d_modelPar_LFPROUT_dX*Inv_L))+(modelStruct.modelPar_WFPROUT*d_Inv_W_dX+modelStruct.d_modelPar_WFPROUT_dX*Inv_W))+(modelStruct.modelPar_PFPROUT*d_Inv_LW_dX+modelStruct.d_modelPar_PFPROUT_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIfprout = (((B4SOIfprout+(modelStruct.modelPar_LFPROUT*Inv_L))+(modelStruct.modelPar_WFPROUT*Inv_W))+(modelStruct.modelPar_PFPROUT*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpdits_dX = (((d_B4SOIpdits_dX+(modelStruct.modelPar_LPDITS*d_Inv_L_dX+modelStruct.d_modelPar_LPDITS_dX*Inv_L))+(modelStruct.modelPar_WPDITS*d_Inv_W_dX+modelStruct.d_modelPar_WPDITS_dX*Inv_W))+(modelStruct.modelPar_PPDITS*d_Inv_LW_dX+modelStruct.d_modelPar_PPDITS_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpdits = (((B4SOIpdits+(modelStruct.modelPar_LPDITS*Inv_L))+(modelStruct.modelPar_WPDITS*Inv_W))+(modelStruct.modelPar_PPDITS*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIpditsd_dX = (((d_B4SOIpditsd_dX+(modelStruct.modelPar_LPDITSD*d_Inv_L_dX+modelStruct.d_modelPar_LPDITSD_dX*Inv_L))+(modelStruct.modelPar_WPDITSD*d_Inv_W_dX+modelStruct.d_modelPar_WPDITSD_dX*Inv_W))+(modelStruct.modelPar_PPDITSD*d_Inv_LW_dX+modelStruct.d_modelPar_PPDITSD_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIpditsd = (((B4SOIpditsd+(modelStruct.modelPar_LPDITSD*Inv_L))+(modelStruct.modelPar_WPDITSD*Inv_W))+(modelStruct.modelPar_PPDITSD*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIaigbcp2_dX = (((d_B4SOIaigbcp2_dX+(modelStruct.modelPar_LAIGBCP2*d_Inv_L_dX+modelStruct.d_modelPar_LAIGBCP2_dX*Inv_L))+(modelStruct.modelPar_WAIGBCP2*d_Inv_W_dX+modelStruct.d_modelPar_WAIGBCP2_dX*Inv_W))+(modelStruct.modelPar_PAIGBCP2*d_Inv_LW_dX+modelStruct.d_modelPar_PAIGBCP2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIaigbcp2 = (((B4SOIaigbcp2+(modelStruct.modelPar_LAIGBCP2*Inv_L))+(modelStruct.modelPar_WAIGBCP2*Inv_W))+(modelStruct.modelPar_PAIGBCP2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIbigbcp2_dX = (((d_B4SOIbigbcp2_dX+(modelStruct.modelPar_LBIGBCP2*d_Inv_L_dX+modelStruct.d_modelPar_LBIGBCP2_dX*Inv_L))+(modelStruct.modelPar_WBIGBCP2*d_Inv_W_dX+modelStruct.d_modelPar_WBIGBCP2_dX*Inv_W))+(modelStruct.modelPar_PBIGBCP2*d_Inv_LW_dX+modelStruct.d_modelPar_PBIGBCP2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIbigbcp2 = (((B4SOIbigbcp2+(modelStruct.modelPar_LBIGBCP2*Inv_L))+(modelStruct.modelPar_WBIGBCP2*Inv_W))+(modelStruct.modelPar_PBIGBCP2*Inv_LW));
instanceStruct.d_instanceVar_pParam_B4SOIcigbcp2_dX = (((d_B4SOIcigbcp2_dX+(modelStruct.modelPar_LCIGBCP2*d_Inv_L_dX+modelStruct.d_modelPar_LCIGBCP2_dX*Inv_L))+(modelStruct.modelPar_WCIGBCP2*d_Inv_W_dX+modelStruct.d_modelPar_WCIGBCP2_dX*Inv_W))+(modelStruct.modelPar_PCIGBCP2*d_Inv_LW_dX+modelStruct.d_modelPar_PCIGBCP2_dX*Inv_LW));
instanceStruct.instanceVar_pParam_B4SOIcigbcp2 = (((B4SOIcigbcp2+(modelStruct.modelPar_LCIGBCP2*Inv_L))+(modelStruct.modelPar_WCIGBCP2*Inv_W))+(modelStruct.modelPar_PCIGBCP2*Inv_LW));
{
double value_atan_0 = atan(pParam_B4SOIminv);
double  deriv_atan_0_d0 = (+1.0/(1+pParam_B4SOIminv*pParam_B4SOIminv));
instanceStruct.d_instanceVar_pParam_B4SOImstar_dX = (deriv_atan_0_d0*(d_pParam_B4SOIminv_dX)/3.14159265358979323846);
instanceStruct.instanceVar_pParam_B4SOImstar = (0.5+(value_atan_0/3.14159265358979323846));
}
if (((instanceStruct.instanceVar_B4SOIvgstcvMod==0)&&(modelStruct.modelPar_VERSION>=4.1)))
{
UserInfo(theInstance)  << "vgstcvMod = 0, users are suggested to use vgstcvMod = 1 or 2." << " " <<  std::endl; 
}
{
double value_atan_0 = atan(pParam_B4SOIminvcv);
double  deriv_atan_0_d0 = (+1.0/(1+pParam_B4SOIminvcv*pParam_B4SOIminvcv));
instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX = (deriv_atan_0_d0*(d_pParam_B4SOIminvcv_dX)/3.14159265358979323846);
instanceStruct.instanceVar_pParam_B4SOImstarcv = (0.5+(value_atan_0/3.14159265358979323846));
}
d_T0_dX = d_TempRatio_dX;
T0 = (TempRatio-1.0);
instanceStruct.d_instanceVar_pParam_B4SOIuatemp_dX = instanceStruct.d_instanceVar_pParam_B4SOIua_dX;
instanceStruct.instanceVar_pParam_B4SOIuatemp = instanceStruct.instanceVar_pParam_B4SOIua;
instanceStruct.d_instanceVar_pParam_B4SOIubtemp_dX = instanceStruct.d_instanceVar_pParam_B4SOIub_dX;
instanceStruct.instanceVar_pParam_B4SOIubtemp = instanceStruct.instanceVar_pParam_B4SOIub;
instanceStruct.d_instanceVar_pParam_B4SOIuctemp_dX = instanceStruct.d_instanceVar_pParam_B4SOIuc_dX;
instanceStruct.instanceVar_pParam_B4SOIuctemp = instanceStruct.instanceVar_pParam_B4SOIuc;
{
double value_pow_0 = pow((instanceStruct.instanceVar_pParam_B4SOIweff*1E6),pParam_B4SOIwr);
double  deriv_pow_0_d0 = (((instanceStruct.instanceVar_pParam_B4SOIweff*1E6) == 0.0)?0.0:(value_pow_0*pParam_B4SOIwr/(instanceStruct.instanceVar_pParam_B4SOIweff*1E6)));
double  deriv_pow_0_d1 = ((instanceStruct.instanceVar_pParam_B4SOIweff*1E6) == 0.0)?0.0:(log((instanceStruct.instanceVar_pParam_B4SOIweff*1E6))*value_pow_0);
instanceStruct.d_instanceVar_pParam_B4SOIrds0denom_dX = ((deriv_pow_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOIweff_dX*1E6))+(deriv_pow_0_d1*(d_pParam_B4SOIwr_dX)));
instanceStruct.instanceVar_pParam_B4SOIrds0denom = value_pow_0;
}
instanceStruct.d_instanceVar_pParam_B4SOIrth_dX = ((instanceStruct.instanceVar_B4SOIrth0/(instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0)))*instanceStruct.d_instanceVar_B4SOInseg_dX+(((instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0))*instanceStruct.d_instanceVar_B4SOIrth0_dX-instanceStruct.instanceVar_B4SOIrth0*(instanceStruct.instanceVar_B4SOInf*(instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_B4SOIwth0_dX)+instanceStruct.d_instanceVar_B4SOInf_dX*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0)))/(instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0))/(instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0)))*instanceStruct.instanceVar_B4SOInseg);
instanceStruct.instanceVar_pParam_B4SOIrth = ((instanceStruct.instanceVar_B4SOIrth0/(instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0)))*instanceStruct.instanceVar_B4SOInseg);
instanceStruct.d_instanceVar_pParam_B4SOIcth_dX = ((instanceStruct.instanceVar_B4SOInseg*(instanceStruct.instanceVar_B4SOIcth0*(instanceStruct.instanceVar_B4SOInf*(instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_B4SOIwth0_dX)+instanceStruct.d_instanceVar_B4SOInf_dX*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0))+instanceStruct.d_instanceVar_B4SOIcth0_dX*(instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0)))-(instanceStruct.instanceVar_B4SOIcth0*(instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0)))*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg);
instanceStruct.instanceVar_pParam_B4SOIcth = ((instanceStruct.instanceVar_B4SOIcth0*(instanceStruct.instanceVar_B4SOInf*(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_B4SOIwth0)))/instanceStruct.instanceVar_B4SOInseg);
if ((instanceStruct.instanceVar_B4SOIrbody==0))
{
instanceStruct.d_instanceVar_pParam_B4SOIrbody_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrbody = 0.0;
}
else
{
instanceStruct.d_instanceVar_pParam_B4SOIrbody_dX = ((instanceStruct.instanceVar_B4SOInf*((instanceStruct.instanceVar_B4SOInseg*((((B4SOIfrbody*instanceStruct.instanceVar_B4SOIrbody)*instanceStruct.instanceVar_B4SOIrhalo)/((2*instanceStruct.instanceVar_B4SOIrbody)+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.instanceVar_pParam_B4SOIleff)))*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+((((2*instanceStruct.instanceVar_B4SOIrbody)+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.instanceVar_pParam_B4SOIleff))*((B4SOIfrbody*instanceStruct.instanceVar_B4SOIrbody)*instanceStruct.d_instanceVar_B4SOIrhalo_dX+(B4SOIfrbody*instanceStruct.d_instanceVar_B4SOIrbody_dX+d_B4SOIfrbody_dX*instanceStruct.instanceVar_B4SOIrbody)*instanceStruct.instanceVar_B4SOIrhalo)-((B4SOIfrbody*instanceStruct.instanceVar_B4SOIrbody)*instanceStruct.instanceVar_B4SOIrhalo)*(2*instanceStruct.d_instanceVar_B4SOIrbody_dX+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+instanceStruct.d_instanceVar_B4SOIrhalo_dX*instanceStruct.instanceVar_pParam_B4SOIleff)))/((2*instanceStruct.instanceVar_B4SOIrbody)+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.instanceVar_pParam_B4SOIleff))/((2*instanceStruct.instanceVar_B4SOIrbody)+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.instanceVar_pParam_B4SOIleff)))*instanceStruct.instanceVar_pParam_B4SOIweff)-((((B4SOIfrbody*instanceStruct.instanceVar_B4SOIrbody)*instanceStruct.instanceVar_B4SOIrhalo)/((2*instanceStruct.instanceVar_B4SOIrbody)+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.instanceVar_pParam_B4SOIleff)))*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)-(((((B4SOIfrbody*instanceStruct.instanceVar_B4SOIrbody)*instanceStruct.instanceVar_B4SOIrhalo)/((2*instanceStruct.instanceVar_B4SOIrbody)+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.instanceVar_pParam_B4SOIleff)))*instanceStruct.instanceVar_pParam_B4SOIweff)/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf);
instanceStruct.instanceVar_pParam_B4SOIrbody = ((((((B4SOIfrbody*instanceStruct.instanceVar_B4SOIrbody)*instanceStruct.instanceVar_B4SOIrhalo)/((2*instanceStruct.instanceVar_B4SOIrbody)+(instanceStruct.instanceVar_B4SOIrhalo*instanceStruct.instanceVar_pParam_B4SOIleff)))*instanceStruct.instanceVar_pParam_B4SOIweff)/instanceStruct.instanceVar_B4SOInseg)/instanceStruct.instanceVar_B4SOInf);
}
{
double value_pow_0 = pow((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm),instanceStruct.instanceVar_B4SOIntox);
double  deriv_pow_0_d0 = (((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm) == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_B4SOIntox/(instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm)));
double  deriv_pow_0_d1 = ((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm) == 0.0)?0.0:(log((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm))*value_pow_0);
instanceStruct.d_instanceVar_pParam_B4SOIoxideRatio_dX = ((instanceStruct.instanceVar_B4SOItoxqm*((instanceStruct.instanceVar_B4SOItoxqm*((deriv_pow_0_d0*(((instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.d_instanceVar_B4SOItoxref_dX-instanceStruct.instanceVar_B4SOItoxref*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm)))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_B4SOIntox_dX)))-value_pow_0*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm)-(value_pow_0/instanceStruct.instanceVar_B4SOItoxqm)*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm);
instanceStruct.instanceVar_pParam_B4SOIoxideRatio = ((value_pow_0/instanceStruct.instanceVar_B4SOItoxqm)/instanceStruct.instanceVar_B4SOItoxqm);
}
instanceStruct.d_instanceVar_pParam_B4SOIua_dX = (instanceStruct.d_instanceVar_pParam_B4SOIua_dX+(instanceStruct.instanceVar_pParam_B4SOIua1*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIua1_dX*T0));
instanceStruct.instanceVar_pParam_B4SOIua = (instanceStruct.instanceVar_pParam_B4SOIua+(instanceStruct.instanceVar_pParam_B4SOIua1*T0));
instanceStruct.d_instanceVar_pParam_B4SOIub_dX = (instanceStruct.d_instanceVar_pParam_B4SOIub_dX+(instanceStruct.instanceVar_pParam_B4SOIub1*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIub1_dX*T0));
instanceStruct.instanceVar_pParam_B4SOIub = (instanceStruct.instanceVar_pParam_B4SOIub+(instanceStruct.instanceVar_pParam_B4SOIub1*T0));
instanceStruct.d_instanceVar_pParam_B4SOIuc_dX = (instanceStruct.d_instanceVar_pParam_B4SOIuc_dX+(instanceStruct.instanceVar_pParam_B4SOIuc1*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIuc1_dX*T0));
instanceStruct.instanceVar_pParam_B4SOIuc = (instanceStruct.instanceVar_pParam_B4SOIuc+(instanceStruct.instanceVar_pParam_B4SOIuc1*T0));
if ((instanceStruct.instanceVar_pParam_B4SOIu0>1.0))
{
instanceStruct.d_instanceVar_pParam_B4SOIu0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIu0_dX/1.0e4);
instanceStruct.instanceVar_pParam_B4SOIu0 = (instanceStruct.instanceVar_pParam_B4SOIu0/1.0e4);
}
{
double value_pow_0 = pow(TempRatio,instanceStruct.instanceVar_pParam_B4SOIute);
double  deriv_pow_0_d0 = ((TempRatio == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_pParam_B4SOIute/TempRatio));
double  deriv_pow_0_d1 = (TempRatio == 0.0)?0.0:(log(TempRatio)*value_pow_0);
instanceStruct.d_instanceVar_pParam_B4SOIu0temp_dX = (instanceStruct.instanceVar_pParam_B4SOIu0*((deriv_pow_0_d0*(d_TempRatio_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_pParam_B4SOIute_dX)))+instanceStruct.d_instanceVar_pParam_B4SOIu0_dX*value_pow_0);
instanceStruct.instanceVar_pParam_B4SOIu0temp = (instanceStruct.instanceVar_pParam_B4SOIu0*value_pow_0);
}
instanceStruct.d_instanceVar_pParam_B4SOIvsattemp_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvsat_dX-(instanceStruct.instanceVar_pParam_B4SOIat*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIat_dX*T0));
instanceStruct.instanceVar_pParam_B4SOIvsattemp = (instanceStruct.instanceVar_pParam_B4SOIvsat-(instanceStruct.instanceVar_pParam_B4SOIat*T0));
instanceStruct.d_instanceVar_pParam_B4SOIrds0_dX = ((instanceStruct.instanceVar_pParam_B4SOIrds0denom*(instanceStruct.d_instanceVar_pParam_B4SOIrdsw_dX+(instanceStruct.instanceVar_pParam_B4SOIprt*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIprt_dX*T0))-(instanceStruct.instanceVar_pParam_B4SOIrdsw+(instanceStruct.instanceVar_pParam_B4SOIprt*T0))*instanceStruct.d_instanceVar_pParam_B4SOIrds0denom_dX)/instanceStruct.instanceVar_pParam_B4SOIrds0denom/instanceStruct.instanceVar_pParam_B4SOIrds0denom);
instanceStruct.instanceVar_pParam_B4SOIrds0 = ((instanceStruct.instanceVar_pParam_B4SOIrdsw+(instanceStruct.instanceVar_pParam_B4SOIprt*T0))/instanceStruct.instanceVar_pParam_B4SOIrds0denom);
if ((instanceStruct.instanceVar_B4SOIrdsMod==1))
{
d_PowWeffWr_dX = (instanceStruct.instanceVar_pParam_B4SOIrds0denom*instanceStruct.d_instanceVar_B4SOInf_dX+instanceStruct.d_instanceVar_pParam_B4SOIrds0denom_dX*instanceStruct.instanceVar_B4SOInf);
PowWeffWr = (instanceStruct.instanceVar_pParam_B4SOIrds0denom*instanceStruct.instanceVar_B4SOInf);
d_T10_dX = (instanceStruct.instanceVar_pParam_B4SOIprt*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIprt_dX*T0);
T10 = (instanceStruct.instanceVar_pParam_B4SOIprt*T0);
d_T1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIrdw_dX+d_T10_dX);
T1 = (instanceStruct.instanceVar_pParam_B4SOIrdw+T10);
d_T2_dX = (instanceStruct.d_instanceVar_B4SOIrdwmin_dX+d_T10_dX);
T2 = (instanceStruct.instanceVar_B4SOIrdwmin+T10);
if ((T1<0.0))
{
d_T1_dX = 0.0;
T1 = 0.0;
UserInfo(theInstance)  << "Warning: Rdw at current temperature is negative; set to 0." << " " <<  std::endl; 
}
if ((T2<0.0))
{
d_T2_dX = 0.0;
T2 = 0.0;
UserInfo(theInstance)  << "Warning: Rdwmin at current temperature is negative; set to 0." << " " <<  std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIrd0_dX = ((PowWeffWr*d_T1_dX-T1*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
instanceStruct.instanceVar_pParam_B4SOIrd0 = (T1/PowWeffWr);
instanceStruct.d_instanceVar_pParam_B4SOIrdwmin_dX = ((PowWeffWr*d_T2_dX-T2*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
instanceStruct.instanceVar_pParam_B4SOIrdwmin = (T2/PowWeffWr);
d_T3_dX = (instanceStruct.d_instanceVar_pParam_B4SOIrsw_dX+d_T10_dX);
T3 = (instanceStruct.instanceVar_pParam_B4SOIrsw+T10);
d_T4_dX = (instanceStruct.d_instanceVar_B4SOIrswmin_dX+d_T10_dX);
T4 = (instanceStruct.instanceVar_B4SOIrswmin+T10);
if ((T3<0.0))
{
d_T3_dX = 0.0;
T3 = 0.0;
UserInfo(theInstance)  << "Warning: Rsw at current temperature is negative; set to 0." << " " <<  std::endl; 
}
if ((T4<0.0))
{
d_T4_dX = 0.0;
T4 = 0.0;
UserInfo(theInstance)  << "Warning: Rswmin at current temperature is negative; set to 0." << " " <<  std::endl; 
}
instanceStruct.d_instanceVar_pParam_B4SOIrs0_dX = ((PowWeffWr*d_T3_dX-T3*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
instanceStruct.instanceVar_pParam_B4SOIrs0 = (T3/PowWeffWr);
instanceStruct.d_instanceVar_pParam_B4SOIrswmin_dX = ((PowWeffWr*d_T4_dX-T4*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
instanceStruct.instanceVar_pParam_B4SOIrswmin = (T4/PowWeffWr);
}
else
{
instanceStruct.d_instanceVar_pParam_B4SOIrd0_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrd0 = 0;
instanceStruct.d_instanceVar_pParam_B4SOIrdwmin_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrdwmin = 0;
instanceStruct.d_instanceVar_pParam_B4SOIrs0_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrs0 = 0;
instanceStruct.d_instanceVar_pParam_B4SOIrswmin_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrswmin = 0;
}
if ((B4SOIcgdo<0.0))
{
UserInfo(theInstance)  << "Warning: cgdo = %g is negative. Set to zero." << " " << B4SOIcgdo << " " <<  std::endl; 
d_B4SOIcgdo_dX = 0.0;
B4SOIcgdo = 0.0;
}
if ((B4SOIcgso<0.0))
{
UserInfo(theInstance)  << "Warning: cgso = %g is negative. Set to zero." << " " << B4SOIcgso << " " <<  std::endl; 
d_B4SOIcgso_dX = 0.0;
B4SOIcgso = 0.0;
}
if ((B4SOIcgeo<0.0))
{
UserInfo(theInstance)  << "Warning: cgeo = %g is negative. Set to zero." << " " << B4SOIcgeo << " " <<  std::endl; 
d_B4SOIcgeo_dX = 0.0;
B4SOIcgeo = 0.0;
}
instanceStruct.d_instanceVar_pParam_B4SOIcgdo_dX = ((B4SOIcgdo+pParam_B4SOIcf)*instanceStruct.d_instanceVar_pParam_B4SOIwdiodCV_dX+(d_B4SOIcgdo_dX+d_pParam_B4SOIcf_dX)*instanceStruct.instanceVar_pParam_B4SOIwdiodCV);
instanceStruct.instanceVar_pParam_B4SOIcgdo = ((B4SOIcgdo+pParam_B4SOIcf)*instanceStruct.instanceVar_pParam_B4SOIwdiodCV);
instanceStruct.d_instanceVar_pParam_B4SOIcgso_dX = ((B4SOIcgso+pParam_B4SOIcf)*instanceStruct.d_instanceVar_pParam_B4SOIwdiosCV_dX+(d_B4SOIcgso_dX+d_pParam_B4SOIcf_dX)*instanceStruct.instanceVar_pParam_B4SOIwdiosCV);
instanceStruct.instanceVar_pParam_B4SOIcgso = ((B4SOIcgso+pParam_B4SOIcf)*instanceStruct.instanceVar_pParam_B4SOIwdiosCV);
instanceStruct.d_instanceVar_pParam_B4SOIcgeo_dX = ((B4SOIcgeo*instanceStruct.instanceVar_pParam_B4SOIleffCV)*instanceStruct.d_instanceVar_B4SOInf_dX+(B4SOIcgeo*instanceStruct.d_instanceVar_pParam_B4SOIleffCV_dX+d_B4SOIcgeo_dX*instanceStruct.instanceVar_pParam_B4SOIleffCV)*instanceStruct.instanceVar_B4SOInf);
instanceStruct.instanceVar_pParam_B4SOIcgeo = ((B4SOIcgeo*instanceStruct.instanceVar_pParam_B4SOIleffCV)*instanceStruct.instanceVar_B4SOInf);
if (((!modelStruct.modelPar_given_NCH)&&modelStruct.modelPar_given_GAMMA1))
{
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIgamma1*instanceStruct.d_instanceVar_B4SOIcox_dX+instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX*instanceStruct.instanceVar_B4SOIcox);
T0 = (instanceStruct.instanceVar_pParam_B4SOIgamma1*instanceStruct.instanceVar_B4SOIcox);
instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX = ((3.021E22*T0)*d_T0_dX+3.021E22*d_T0_dX*T0);
instanceStruct.instanceVar_pParam_B4SOInpeak = ((3.021E22*T0)*T0);
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
d_NchMax_dX = (((B4SOIetsi*B4SOIetsi)*((((instanceStruct.instanceVar_B4SOIbg0sub-0.1)/1.60219e-19)*2e-6)*instanceStruct.d_instanceVar_epssub_dX+(instanceStruct.d_instanceVar_B4SOIbg0sub_dX/1.60219e-19)*2e-6*instanceStruct.instanceVar_epssub)-((((instanceStruct.instanceVar_B4SOIbg0sub-0.1)/1.60219e-19)*2e-6)*instanceStruct.instanceVar_epssub)*(B4SOIetsi*d_B4SOIetsi_dX+d_B4SOIetsi_dX*B4SOIetsi))/(B4SOIetsi*B4SOIetsi)/(B4SOIetsi*B4SOIetsi));
NchMax = (((((instanceStruct.instanceVar_B4SOIbg0sub-0.1)/1.60219e-19)*2e-6)*instanceStruct.instanceVar_epssub)/(B4SOIetsi*B4SOIetsi));
if ((instanceStruct.instanceVar_pParam_B4SOInpeak>NchMax))
{
UserInfo(theInstance)  << "SOIMOD=2 cannot support given Nch=%g cm^-3 and Etsi=%g m " << " " << instanceStruct.instanceVar_pParam_B4SOInpeak << " " << B4SOIetsi << " " <<  std::endl; 
UserInfo(theInstance)  << "Exceeds maximum allowed band bending of (Eg-0.1)eV" << " " <<  std::endl; 
UserInfo(theInstance)  << "Nch is set to  %g cm^-3" << " " << NchMax << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX = d_NchMax_dX;
instanceStruct.instanceVar_pParam_B4SOInpeak = NchMax;
}
}
else
{
d_NchMax_dX = (((instanceStruct.instanceVar_B4SOItsi*instanceStruct.instanceVar_B4SOItsi)*(((1.12-0.1)/1.60219e-19)*2e-6)*instanceStruct.d_instanceVar_epssub_dX-((((1.12-0.1)/1.60219e-19)*2e-6)*instanceStruct.instanceVar_epssub)*(instanceStruct.instanceVar_B4SOItsi*instanceStruct.d_instanceVar_B4SOItsi_dX+instanceStruct.d_instanceVar_B4SOItsi_dX*instanceStruct.instanceVar_B4SOItsi))/(instanceStruct.instanceVar_B4SOItsi*instanceStruct.instanceVar_B4SOItsi)/(instanceStruct.instanceVar_B4SOItsi*instanceStruct.instanceVar_B4SOItsi));
NchMax = (((((1.12-0.1)/1.60219e-19)*2e-6)*instanceStruct.instanceVar_epssub)/(instanceStruct.instanceVar_B4SOItsi*instanceStruct.instanceVar_B4SOItsi));
if ((instanceStruct.instanceVar_pParam_B4SOInpeak>NchMax))
{
UserInfo(theInstance)  << "SOIMOD=2 cannot support given Nch=%g cm^-3 and Tsi=%g m " << " " << instanceStruct.instanceVar_pParam_B4SOInpeak << " " << instanceStruct.instanceVar_B4SOItsi << " " <<  std::endl; 
UserInfo(theInstance)  << "Exceeds maximum allowed band bending of (Eg-0.1)eV" << " " <<  std::endl; 
UserInfo(theInstance)  << "Nch is set to  %g cm^-3" << " " << NchMax << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX = d_NchMax_dX;
instanceStruct.instanceVar_pParam_B4SOInpeak = NchMax;
}
}
}
instanceStruct.d_instanceVar_B4SOIcbox_dX = (-3.453133e-11*instanceStruct.d_instanceVar_B4SOItbox_dX/instanceStruct.instanceVar_B4SOItbox/instanceStruct.instanceVar_B4SOItbox);
instanceStruct.instanceVar_B4SOIcbox = (3.453133e-11/instanceStruct.instanceVar_B4SOItbox);
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
instanceStruct.d_instanceVar_B4SOIcsi_dX = (-1.03594e-10*d_B4SOIetsi_dX/B4SOIetsi/B4SOIetsi);
instanceStruct.instanceVar_B4SOIcsi = (1.03594e-10/B4SOIetsi);
}
else
{
instanceStruct.d_instanceVar_B4SOIcsi_dX = (-1.03594e-10*instanceStruct.d_instanceVar_B4SOItsi_dX/instanceStruct.instanceVar_B4SOItsi/instanceStruct.instanceVar_B4SOItsi);
instanceStruct.instanceVar_B4SOIcsi = (1.03594e-10/instanceStruct.instanceVar_B4SOItsi);
}
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
d_Qsi_dX = ((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*(1.0+(B4SOIlpe0/B4SOIl)))*1e6)*d_B4SOIetsi_dX+((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*((B4SOIl*d_B4SOIlpe0_dX-B4SOIlpe0*d_B4SOIl_dX)/B4SOIl/B4SOIl)+1.60219e-19*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX*(1.0+(B4SOIlpe0/B4SOIl)))*1e6*B4SOIetsi);
Qsi = ((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*(1.0+(B4SOIlpe0/B4SOIl)))*1e6)*B4SOIetsi);
}
else
{
d_Qsi_dX = ((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*(1.0+(B4SOIlpe0/B4SOIl)))*1e6)*instanceStruct.d_instanceVar_B4SOItsi_dX+((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*((B4SOIl*d_B4SOIlpe0_dX-B4SOIlpe0*d_B4SOIl_dX)/B4SOIl/B4SOIl)+1.60219e-19*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX*(1.0+(B4SOIlpe0/B4SOIl)))*1e6*instanceStruct.instanceVar_B4SOItsi);
Qsi = ((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*(1.0+(B4SOIlpe0/B4SOIl)))*1e6)*instanceStruct.instanceVar_B4SOItsi);
}
d_Vbs0t_dX = ((-((instanceStruct.instanceVar_B4SOIcsi*0.5*d_Qsi_dX-(0.5*Qsi)*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi))+instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX);
Vbs0t = ((0.8-((0.5*Qsi)/instanceStruct.instanceVar_B4SOIcsi))+instanceStruct.instanceVar_pParam_B4SOIvbsa);
if ((instanceStruct.instanceVar_B4SOIsoiMod==3))
{
if ((Vbs0t>pParam_B4SOIvbs0fd))
{
instanceStruct.instanceVar_B4SOIsoiMod = 2;
}
else
{
if ((Vbs0t<pParam_B4SOIvbs0pd))
{
instanceStruct.instanceVar_B4SOIsoiMod = 0;
}
else
{
instanceStruct.instanceVar_B4SOIsoiMod = 1;
}
}
}
d_T4_dX = ((1.115/instanceStruct.instanceVar_B4SOIvtm)*d_TempRatio_dX+(-1.115*instanceStruct.d_instanceVar_B4SOIvtm_dX/instanceStruct.instanceVar_B4SOIvtm/instanceStruct.instanceVar_B4SOIvtm)*(TempRatio-1.0));
T4 = ((1.115/instanceStruct.instanceVar_B4SOIvtm)*(TempRatio-1.0));
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndiode*(instanceStruct.instanceVar_pParam_B4SOIxbjt*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxbjt_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX)/instanceStruct.instanceVar_pParam_B4SOIndiode/instanceStruct.instanceVar_pParam_B4SOIndiode);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)/instanceStruct.instanceVar_pParam_B4SOIndiode);
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndiode*(instanceStruct.instanceVar_pParam_B4SOIxdif*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxdif_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxdif*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX)/instanceStruct.instanceVar_pParam_B4SOIndiode/instanceStruct.instanceVar_pParam_B4SOIndiode);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxdif*T4)/instanceStruct.instanceVar_pParam_B4SOIndiode);
if ((T7>100.0))
{
d_T1_dX = 2.688117142e+43*d_T7_dX;
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T7_dX);
T1 = value_exp_0;
}
}
}
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOInrecf0*(instanceStruct.instanceVar_pParam_B4SOIxrec*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxrec_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxrec*T4)*instanceStruct.d_instanceVar_pParam_B4SOInrecf0_dX)/instanceStruct.instanceVar_pParam_B4SOInrecf0/instanceStruct.instanceVar_pParam_B4SOInrecf0);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxrec*T4)/instanceStruct.instanceVar_pParam_B4SOInrecf0);
if ((T7>100.0))
{
d_T2_dX = 2.688117142e+43*d_T7_dX;
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T2_dX = 0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T2_dX = deriv_exp_0_d0*(d_T7_dX);
T2 = value_exp_0;
}
}
}
instanceStruct.d_instanceVar_pParam_B4SOIahli0s_dX = (instanceStruct.instanceVar_pParam_B4SOIahli*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIahli_dX*T0);
instanceStruct.instanceVar_pParam_B4SOIahli0s = (instanceStruct.instanceVar_pParam_B4SOIahli*T0);
instanceStruct.d_instanceVar_pParam_B4SOIjbjts_dX = (instanceStruct.instanceVar_pParam_B4SOIisbjt*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIisbjt_dX*T0);
instanceStruct.instanceVar_pParam_B4SOIjbjts = (instanceStruct.instanceVar_pParam_B4SOIisbjt*T0);
instanceStruct.d_instanceVar_pParam_B4SOIjdifs_dX = (instanceStruct.instanceVar_pParam_B4SOIisdif*d_T1_dX+instanceStruct.d_instanceVar_pParam_B4SOIisdif_dX*T1);
instanceStruct.instanceVar_pParam_B4SOIjdifs = (instanceStruct.instanceVar_pParam_B4SOIisdif*T1);
instanceStruct.d_instanceVar_pParam_B4SOIjrecs_dX = (instanceStruct.instanceVar_pParam_B4SOIisrec*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIisrec_dX*T2);
instanceStruct.instanceVar_pParam_B4SOIjrecs = (instanceStruct.instanceVar_pParam_B4SOIisrec*T2);
d_T7_dX = (instanceStruct.instanceVar_pParam_B4SOIxtun*d_TempRatio_dX+instanceStruct.d_instanceVar_pParam_B4SOIxtun_dX*(TempRatio-1));
T7 = (instanceStruct.instanceVar_pParam_B4SOIxtun*(TempRatio-1));
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
instanceStruct.d_instanceVar_pParam_B4SOIjtuns_dX = (instanceStruct.instanceVar_pParam_B4SOIistun*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIistun_dX*T0);
instanceStruct.instanceVar_pParam_B4SOIjtuns = (instanceStruct.instanceVar_pParam_B4SOIistun*T0);
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndioded*(instanceStruct.instanceVar_pParam_B4SOIxbjt*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxbjt_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX)/instanceStruct.instanceVar_pParam_B4SOIndioded/instanceStruct.instanceVar_pParam_B4SOIndioded);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)/instanceStruct.instanceVar_pParam_B4SOIndioded);
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndioded*(instanceStruct.instanceVar_pParam_B4SOIxdifd*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxdifd_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxdifd*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX)/instanceStruct.instanceVar_pParam_B4SOIndioded/instanceStruct.instanceVar_pParam_B4SOIndioded);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxdifd*T4)/instanceStruct.instanceVar_pParam_B4SOIndioded);
if ((T7>100.0))
{
d_T1_dX = 2.688117142e+43*d_T7_dX;
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T7_dX);
T1 = value_exp_0;
}
}
}
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOInrecf0d*(instanceStruct.instanceVar_pParam_B4SOIxrecd*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxrecd_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxrecd*T4)*instanceStruct.d_instanceVar_pParam_B4SOInrecf0d_dX)/instanceStruct.instanceVar_pParam_B4SOInrecf0d/instanceStruct.instanceVar_pParam_B4SOInrecf0d);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxrecd*T4)/instanceStruct.instanceVar_pParam_B4SOInrecf0d);
if ((T7>100.0))
{
d_T2_dX = 2.688117142e+43*d_T7_dX;
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T2_dX = 0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T2_dX = deriv_exp_0_d0*(d_T7_dX);
T2 = value_exp_0;
}
}
}
instanceStruct.d_instanceVar_pParam_B4SOIahli0d_dX = (instanceStruct.instanceVar_pParam_B4SOIahlid*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIahlid_dX*T0);
instanceStruct.instanceVar_pParam_B4SOIahli0d = (instanceStruct.instanceVar_pParam_B4SOIahlid*T0);
instanceStruct.d_instanceVar_pParam_B4SOIjbjtd_dX = (instanceStruct.instanceVar_pParam_B4SOIidbjt*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIidbjt_dX*T0);
instanceStruct.instanceVar_pParam_B4SOIjbjtd = (instanceStruct.instanceVar_pParam_B4SOIidbjt*T0);
instanceStruct.d_instanceVar_pParam_B4SOIjdifd_dX = (instanceStruct.instanceVar_pParam_B4SOIiddif*d_T1_dX+instanceStruct.d_instanceVar_pParam_B4SOIiddif_dX*T1);
instanceStruct.instanceVar_pParam_B4SOIjdifd = (instanceStruct.instanceVar_pParam_B4SOIiddif*T1);
instanceStruct.d_instanceVar_pParam_B4SOIjrecd_dX = (instanceStruct.instanceVar_pParam_B4SOIidrec*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIidrec_dX*T2);
instanceStruct.instanceVar_pParam_B4SOIjrecd = (instanceStruct.instanceVar_pParam_B4SOIidrec*T2);
d_T7_dX = (instanceStruct.instanceVar_pParam_B4SOIxtund*d_TempRatio_dX+instanceStruct.d_instanceVar_pParam_B4SOIxtund_dX*(TempRatio-1));
T7 = (instanceStruct.instanceVar_pParam_B4SOIxtund*(TempRatio-1));
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
instanceStruct.d_instanceVar_pParam_B4SOIjtund_dX = (instanceStruct.instanceVar_pParam_B4SOIidtun*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIidtun_dX*T0);
instanceStruct.instanceVar_pParam_B4SOIjtund = (instanceStruct.instanceVar_pParam_B4SOIidtun*T0);
if ((instanceStruct.instanceVar_pParam_B4SOInsub>0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_pParam_B4SOInpeak/instanceStruct.instanceVar_pParam_B4SOInsub));
instanceStruct.d_instanceVar_pParam_B4SOIvfbb_dX = (((-instanceStruct.instanceVar_B4SOItype)*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_pParam_B4SOInpeak/instanceStruct.instanceVar_pParam_B4SOInsub),((instanceStruct.instanceVar_pParam_B4SOInsub*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-instanceStruct.instanceVar_pParam_B4SOInpeak*instanceStruct.d_instanceVar_pParam_B4SOInsub_dX)/instanceStruct.instanceVar_pParam_B4SOInsub/instanceStruct.instanceVar_pParam_B4SOInsub))+(-instanceStruct.instanceVar_B4SOItype)*instanceStruct.d_instanceVar_B4SOIvtm_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/instanceStruct.instanceVar_pParam_B4SOInsub)));
instanceStruct.instanceVar_pParam_B4SOIvfbb = (((-instanceStruct.instanceVar_B4SOItype)*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/instanceStruct.instanceVar_pParam_B4SOInsub)));
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni));
instanceStruct.d_instanceVar_pParam_B4SOIvfbb_dX = (((-instanceStruct.instanceVar_B4SOItype)*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_0.getDerivs(((((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni),((ni*((ni*((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.d_instanceVar_pParam_B4SOInsub_dX+(-instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX)*instanceStruct.instanceVar_pParam_B4SOInsub)-((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)*d_ni_dX)/ni/ni)-(((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)*d_ni_dX)/ni/ni))+(-instanceStruct.instanceVar_B4SOItype)*instanceStruct.d_instanceVar_B4SOIvtm_dX*evaluator_lln_0.getValues(((((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni)));
instanceStruct.instanceVar_pParam_B4SOIvfbb = (((-instanceStruct.instanceVar_B4SOItype)*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_0.getValues(((((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni)));
}
}
if ((!modelStruct.modelPar_given_VSDFB))
{
if ((instanceStruct.instanceVar_pParam_B4SOInsub>0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((((1e20*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni));
instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX = (-instanceStruct.instanceVar_B4SOItype)*(instanceStruct.instanceVar_B4SOIvtm*evaluator_lln_0.getDerivs((((1e20*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni),((ni*((ni*1e20*instanceStruct.d_instanceVar_pParam_B4SOInsub_dX-(1e20*instanceStruct.instanceVar_pParam_B4SOInsub)*d_ni_dX)/ni/ni)-((1e20*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)*d_ni_dX)/ni/ni))+instanceStruct.d_instanceVar_B4SOIvtm_dX*evaluator_lln_0.getValues((((1e20*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni)));
instanceStruct.instanceVar_pParam_B4SOIvsdfb = ((-instanceStruct.instanceVar_B4SOItype)*((instanceStruct.instanceVar_B4SOIvtm*evaluator_lln_0.getValues((((1e20*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni)))-0.3));
}
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOInsub<0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((-1e20)/instanceStruct.instanceVar_pParam_B4SOInsub));
instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX = (-instanceStruct.instanceVar_B4SOItype)*(instanceStruct.instanceVar_B4SOIvtm*evaluator_lln_0.getDerivs(((-1e20)/instanceStruct.instanceVar_pParam_B4SOInsub),(-(-1e20)*instanceStruct.d_instanceVar_pParam_B4SOInsub_dX/instanceStruct.instanceVar_pParam_B4SOInsub/instanceStruct.instanceVar_pParam_B4SOInsub))+instanceStruct.d_instanceVar_B4SOIvtm_dX*evaluator_lln_0.getValues(((-1e20)/instanceStruct.instanceVar_pParam_B4SOInsub)));
instanceStruct.instanceVar_pParam_B4SOIvsdfb = ((-instanceStruct.instanceVar_B4SOItype)*((instanceStruct.instanceVar_B4SOIvtm*evaluator_lln_0.getValues(((-1e20)/instanceStruct.instanceVar_pParam_B4SOInsub)))+0.3));
}
}
}
}
{
double value_fabs_0 = fabs(instanceStruct.instanceVar_pParam_B4SOInsub);
AnalogFunctions::llnEvaluator evaluator_lln_1((value_fabs_0/ni));
double  deriv_fabs_0_d0 = (((instanceStruct.instanceVar_pParam_B4SOInsub)>=0)?(+1.0):(-1.0));
d_SDphi_dX = ((2.0*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_1.getDerivs((value_fabs_0/ni),((ni*deriv_fabs_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOInsub_dX)-value_fabs_0*d_ni_dX)/ni/ni))+2.0*instanceStruct.d_instanceVar_B4SOIvtm_dX*evaluator_lln_1.getValues((value_fabs_0/ni)));
SDphi = ((2.0*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_1.getValues((value_fabs_0/ni)));
}
{
double value_fabs_0 = fabs(instanceStruct.instanceVar_pParam_B4SOInsub);
double value_sqrt_1 = sqrt(value_fabs_0);
double  deriv_fabs_0_d0 = (((instanceStruct.instanceVar_pParam_B4SOInsub)>=0)?(+1.0):(-1.0));
double  deriv_sqrt_1_d0 = (0.5/value_sqrt_1);
d_SDgamma_dX = ((instanceStruct.instanceVar_B4SOIcbox*(instanceStruct.instanceVar_sqrt2qeps*deriv_sqrt_1_d0*(deriv_fabs_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOInsub_dX))+instanceStruct.d_instanceVar_sqrt2qeps_dX*value_sqrt_1)-(instanceStruct.instanceVar_sqrt2qeps*value_sqrt_1)*instanceStruct.d_instanceVar_B4SOIcbox_dX)/instanceStruct.instanceVar_B4SOIcbox/instanceStruct.instanceVar_B4SOIcbox);
SDgamma = ((instanceStruct.instanceVar_sqrt2qeps*value_sqrt_1)/instanceStruct.instanceVar_B4SOIcbox);
}
if ((!modelStruct.modelPar_given_VSDTH))
{
if ((((instanceStruct.instanceVar_pParam_B4SOInsub>0)&&(instanceStruct.instanceVar_B4SOItype>0))||((instanceStruct.instanceVar_pParam_B4SOInsub<0)&&(instanceStruct.instanceVar_B4SOItype<0))))
{
{
double value_sqrt_0 = sqrt(SDphi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX+d_SDphi_dX)+(SDgamma*deriv_sqrt_0_d0*(d_SDphi_dX)+d_SDgamma_dX*value_sqrt_0));
instanceStruct.instanceVar_pParam_B4SOIvsdth = ((instanceStruct.instanceVar_pParam_B4SOIvsdfb+SDphi)+(SDgamma*value_sqrt_0));
}
}
else
{
{
double value_sqrt_0 = sqrt(SDphi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX-d_SDphi_dX)-(SDgamma*deriv_sqrt_0_d0*(d_SDphi_dX)+d_SDgamma_dX*value_sqrt_0));
instanceStruct.instanceVar_pParam_B4SOIvsdth = ((instanceStruct.instanceVar_pParam_B4SOIvsdfb-SDphi)-(SDgamma*value_sqrt_0));
}
}
}
if ((!modelStruct.modelPar_given_CSDMIN))
{
{
double value_fabs_0 = fabs(instanceStruct.instanceVar_pParam_B4SOInsub);
double value_sqrt_1 = sqrt((((2.0*instanceStruct.instanceVar_epssub)*SDphi)/((1.60219e-19*value_fabs_0)*1.0e6)));
double  deriv_fabs_0_d0 = (((instanceStruct.instanceVar_pParam_B4SOInsub)>=0)?(+1.0):(-1.0));
double  deriv_sqrt_1_d0 = (0.5/value_sqrt_1);
d_tmp_dX = deriv_sqrt_1_d0*(((((1.60219e-19*value_fabs_0)*1.0e6)*((2.0*instanceStruct.instanceVar_epssub)*d_SDphi_dX+2.0*instanceStruct.d_instanceVar_epssub_dX*SDphi)-((2.0*instanceStruct.instanceVar_epssub)*SDphi)*1.60219e-19*deriv_fabs_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOInsub_dX)*1.0e6)/((1.60219e-19*value_fabs_0)*1.0e6)/((1.60219e-19*value_fabs_0)*1.0e6)));
tmp = value_sqrt_1;
}
d_tmp1_dX = ((tmp*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*d_tmp_dX)/tmp/tmp);
tmp1 = (instanceStruct.instanceVar_epssub/tmp);
instanceStruct.d_instanceVar_B4SOIcsdmin_dX = (((tmp1+instanceStruct.instanceVar_B4SOIcbox)*(tmp1*instanceStruct.d_instanceVar_B4SOIcbox_dX+d_tmp1_dX*instanceStruct.instanceVar_B4SOIcbox)-(tmp1*instanceStruct.instanceVar_B4SOIcbox)*(d_tmp1_dX+instanceStruct.d_instanceVar_B4SOIcbox_dX))/(tmp1+instanceStruct.instanceVar_B4SOIcbox)/(tmp1+instanceStruct.instanceVar_B4SOIcbox));
instanceStruct.instanceVar_B4SOIcsdmin = ((tmp1*instanceStruct.instanceVar_B4SOIcbox)/(tmp1+instanceStruct.instanceVar_B4SOIcbox));
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_pParam_B4SOInpeak/ni));
instanceStruct.d_instanceVar_pParam_B4SOIphi_dX = ((2.0*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_pParam_B4SOInpeak/ni),((ni*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-instanceStruct.instanceVar_pParam_B4SOInpeak*d_ni_dX)/ni/ni))+2.0*instanceStruct.d_instanceVar_B4SOIvtm_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/ni)));
instanceStruct.instanceVar_pParam_B4SOIphi = ((2.0*instanceStruct.instanceVar_B4SOIvtm)*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/ni)));
}
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_pParam_B4SOIphi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX = deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOIphi_dX);
instanceStruct.instanceVar_pParam_B4SOIsqrtPhi = value_sqrt_0;
}
{
double value_sqrt_0 = sqrt(((2.0*instanceStruct.instanceVar_epssub)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIXdep0_dX = (value_sqrt_0*instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX+deriv_sqrt_0_d0*(((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)*2.0*instanceStruct.d_instanceVar_epssub_dX-(2.0*instanceStruct.instanceVar_epssub)*1.60219e-19*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX*1.0e6)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)))*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi);
instanceStruct.instanceVar_pParam_B4SOIXdep0 = (value_sqrt_0*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi);
}
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_pParam_B4SOIXdep0);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_pParam_B4SOIsqrtXdep0_dX = deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOIXdep0_dX);
pParam_B4SOIsqrtXdep0 = value_sqrt_0;
}
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
{
double value_sqrt_0 = sqrt(((((3.0*3.9)/instanceStruct.instanceVar_epsrox)*instanceStruct.instanceVar_pParam_B4SOIxj)*instanceStruct.instanceVar_B4SOItox));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX = deriv_sqrt_0_d0*(((((3.0*3.9)/instanceStruct.instanceVar_epsrox)*instanceStruct.instanceVar_pParam_B4SOIxj)*instanceStruct.d_instanceVar_B4SOItox_dX+(((3.0*3.9)/instanceStruct.instanceVar_epsrox)*instanceStruct.d_instanceVar_pParam_B4SOIxj_dX+(-(3.0*3.9)*instanceStruct.d_instanceVar_epsrox_dX/instanceStruct.instanceVar_epsrox/instanceStruct.instanceVar_epsrox)*instanceStruct.instanceVar_pParam_B4SOIxj)*instanceStruct.instanceVar_B4SOItox));
instanceStruct.instanceVar_pParam_B4SOIlitl = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt((((instanceStruct.instanceVar_epssub*instanceStruct.instanceVar_pParam_B4SOIxj)*instanceStruct.instanceVar_toxe)/(instanceStruct.instanceVar_epsrox*8.85418e-12)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX = deriv_sqrt_0_d0*((((instanceStruct.instanceVar_epsrox*8.85418e-12)*((instanceStruct.instanceVar_epssub*instanceStruct.instanceVar_pParam_B4SOIxj)*instanceStruct.d_instanceVar_toxe_dX+(instanceStruct.instanceVar_epssub*instanceStruct.d_instanceVar_pParam_B4SOIxj_dX+instanceStruct.d_instanceVar_epssub_dX*instanceStruct.instanceVar_pParam_B4SOIxj)*instanceStruct.instanceVar_toxe)-((instanceStruct.instanceVar_epssub*instanceStruct.instanceVar_pParam_B4SOIxj)*instanceStruct.instanceVar_toxe)*instanceStruct.d_instanceVar_epsrox_dX*8.85418e-12)/(instanceStruct.instanceVar_epsrox*8.85418e-12)/(instanceStruct.instanceVar_epsrox*8.85418e-12)));
instanceStruct.instanceVar_pParam_B4SOIlitl = value_sqrt_0;
}
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)));
instanceStruct.d_instanceVar_pParam_B4SOIvbi_dX = (instanceStruct.instanceVar_B4SOIvtm*evaluator_lln_0.getDerivs(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)),(((ni*ni)*1.0e20*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-(1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)*(ni*d_ni_dX+d_ni_dX*ni))/(ni*ni)/(ni*ni)))+instanceStruct.d_instanceVar_B4SOIvtm_dX*evaluator_lln_0.getValues(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni))));
instanceStruct.instanceVar_pParam_B4SOIvbi = (instanceStruct.instanceVar_B4SOIvtm*evaluator_lln_0.getValues(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni))));
}
{
double value_sqrt_0 = sqrt((((((1.60219e-19*instanceStruct.instanceVar_epssub)*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)/2.0)/instanceStruct.instanceVar_pParam_B4SOIphi));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX = deriv_sqrt_0_d0*(((instanceStruct.instanceVar_pParam_B4SOIphi*(((1.60219e-19*instanceStruct.instanceVar_epssub)*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX+1.60219e-19*instanceStruct.d_instanceVar_epssub_dX*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6/2.0)-((((1.60219e-19*instanceStruct.instanceVar_epssub)*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)/2.0)*instanceStruct.d_instanceVar_pParam_B4SOIphi_dX)/instanceStruct.instanceVar_pParam_B4SOIphi/instanceStruct.instanceVar_pParam_B4SOIphi));
instanceStruct.instanceVar_pParam_B4SOIcdep0 = value_sqrt_0;
}
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
if ((instanceStruct.instanceVar_pParam_B4SOIngate>0.0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_pParam_B4SOIngate/1.0e20));
instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX = (Vtm0*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_pParam_B4SOIngate/1.0e20),(instanceStruct.d_instanceVar_pParam_B4SOIngate_dX/1.0e20))+d_Vtm0_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOIngate/1.0e20)));
instanceStruct.instanceVar_pParam_B4SOIvfbsd = (Vtm0*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOIngate/1.0e20)));
}
}
else
{
instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIvfbsd = 0.0;
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((pParam_B4SOInsd/ni));
d_T0_dX = (Vtm0*evaluator_lln_0.getDerivs((pParam_B4SOInsd/ni),((ni*d_pParam_B4SOInsd_dX-pParam_B4SOInsd*d_ni_dX)/ni/ni))+d_Vtm0_dX*evaluator_lln_0.getValues((pParam_B4SOInsd/ni)));
T0 = (Vtm0*evaluator_lln_0.getValues((pParam_B4SOInsd/ni)));
}
d_T1_dX = 0.5*d_Eg0_dX;
T1 = (0.5*Eg0);
if ((T0>T1))
{
d_T0_dX = d_T1_dX;
T0 = T1;
}
d_T2_dX = ((instanceStruct.d_instanceVar_B4SOIeasub_dX+d_T1_dX)-instanceStruct.instanceVar_B4SOItype*d_T0_dX);
T2 = ((instanceStruct.instanceVar_B4SOIeasub+T1)-(instanceStruct.instanceVar_B4SOItype*T0));
instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX = (instanceStruct.d_instanceVar_B4SOIphig_dX-d_T2_dX);
instanceStruct.instanceVar_pParam_B4SOIvfbsd = (instanceStruct.instanceVar_B4SOIphig-T2);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm));
double value_exp_1 = exp((instanceStruct.instanceVar_B4SOIntox*evaluator_lln_0.getValues((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm))));
double  deriv_exp_1_d0 = value_exp_1;
d_pParam_B4SOIToxRatio_dX = ((instanceStruct.instanceVar_B4SOItoxqm*((instanceStruct.instanceVar_B4SOItoxqm*deriv_exp_1_d0*((instanceStruct.instanceVar_B4SOIntox*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm),((instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.d_instanceVar_B4SOItoxref_dX-instanceStruct.instanceVar_B4SOItoxref*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm))+instanceStruct.d_instanceVar_B4SOIntox_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_B4SOItoxref/instanceStruct.instanceVar_B4SOItoxqm))))-value_exp_1*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm)-(value_exp_1/instanceStruct.instanceVar_B4SOItoxqm)*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm);
pParam_B4SOIToxRatio = ((value_exp_1/instanceStruct.instanceVar_B4SOItoxqm)/instanceStruct.instanceVar_B4SOItoxqm);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_B4SOItoxref/(instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.instanceVar_pParam_B4SOIpoxedge)));
double value_exp_1 = exp((instanceStruct.instanceVar_B4SOIntox*evaluator_lln_0.getValues((instanceStruct.instanceVar_B4SOItoxref/(instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.instanceVar_pParam_B4SOIpoxedge)))));
double  deriv_exp_1_d0 = value_exp_1;
d_pParam_B4SOIToxRatioEdge_dX = ((instanceStruct.instanceVar_pParam_B4SOIpoxedge*((instanceStruct.instanceVar_pParam_B4SOIpoxedge*((instanceStruct.instanceVar_B4SOItoxqm*((instanceStruct.instanceVar_B4SOItoxqm*deriv_exp_1_d0*((instanceStruct.instanceVar_B4SOIntox*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_B4SOItoxref/(instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.instanceVar_pParam_B4SOIpoxedge)),(((instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.instanceVar_pParam_B4SOIpoxedge)*instanceStruct.d_instanceVar_B4SOItoxref_dX-instanceStruct.instanceVar_B4SOItoxref*(instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.d_instanceVar_pParam_B4SOIpoxedge_dX+instanceStruct.d_instanceVar_B4SOItoxqm_dX*instanceStruct.instanceVar_pParam_B4SOIpoxedge))/(instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.instanceVar_pParam_B4SOIpoxedge)/(instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.instanceVar_pParam_B4SOIpoxedge)))+instanceStruct.d_instanceVar_B4SOIntox_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_B4SOItoxref/(instanceStruct.instanceVar_B4SOItoxqm*instanceStruct.instanceVar_pParam_B4SOIpoxedge)))))-value_exp_1*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm)-(value_exp_1/instanceStruct.instanceVar_B4SOItoxqm)*instanceStruct.d_instanceVar_B4SOItoxqm_dX)/instanceStruct.instanceVar_B4SOItoxqm/instanceStruct.instanceVar_B4SOItoxqm)-((value_exp_1/instanceStruct.instanceVar_B4SOItoxqm)/instanceStruct.instanceVar_B4SOItoxqm)*instanceStruct.d_instanceVar_pParam_B4SOIpoxedge_dX)/instanceStruct.instanceVar_pParam_B4SOIpoxedge/instanceStruct.instanceVar_pParam_B4SOIpoxedge)-(((value_exp_1/instanceStruct.instanceVar_B4SOItoxqm)/instanceStruct.instanceVar_B4SOItoxqm)/instanceStruct.instanceVar_pParam_B4SOIpoxedge)*instanceStruct.d_instanceVar_pParam_B4SOIpoxedge_dX)/instanceStruct.instanceVar_pParam_B4SOIpoxedge/instanceStruct.instanceVar_pParam_B4SOIpoxedge);
pParam_B4SOIToxRatioEdge = ((((value_exp_1/instanceStruct.instanceVar_B4SOItoxqm)/instanceStruct.instanceVar_B4SOItoxqm)/instanceStruct.instanceVar_pParam_B4SOIpoxedge)/instanceStruct.instanceVar_pParam_B4SOIpoxedge);
}
instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.d_instanceVar_agbc2p_dX:instanceStruct.d_instanceVar_agbc2n_dX);
instanceStruct.instanceVar_pParam_B4SOIAechvb = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.instanceVar_agbc2p:instanceStruct.instanceVar_agbc2n);
instanceStruct.d_instanceVar_pParam_B4SOIBechvb_dX = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.d_instanceVar_bgbc2p_dX:instanceStruct.d_instanceVar_bgbc2n_dX);
instanceStruct.instanceVar_pParam_B4SOIBechvb = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.instanceVar_bgbc2p:instanceStruct.instanceVar_bgbc2n);
instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdges_dX = (((instanceStruct.instanceVar_pParam_B4SOIAechvb*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpsbcp))*pParam_B4SOIdlcig)*d_pParam_B4SOIToxRatioEdge_dX+((instanceStruct.instanceVar_pParam_B4SOIAechvb*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpsbcp))*d_pParam_B4SOIdlcig_dX+(instanceStruct.instanceVar_pParam_B4SOIAechvb*(((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-instanceStruct.instanceVar_pParam_B4SOIweff*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+d_B4SOIpsbcp_dX)+instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpsbcp))*pParam_B4SOIdlcig)*pParam_B4SOIToxRatioEdge);
instanceStruct.instanceVar_pParam_B4SOIAechvbEdges = (((instanceStruct.instanceVar_pParam_B4SOIAechvb*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpsbcp))*pParam_B4SOIdlcig)*pParam_B4SOIToxRatioEdge);
instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdged_dX = (((instanceStruct.instanceVar_pParam_B4SOIAechvb*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpdbcp))*pParam_B4SOIdlcig)*d_pParam_B4SOIToxRatioEdge_dX+((instanceStruct.instanceVar_pParam_B4SOIAechvb*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpdbcp))*d_pParam_B4SOIdlcig_dX+(instanceStruct.instanceVar_pParam_B4SOIAechvb*(((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-instanceStruct.instanceVar_pParam_B4SOIweff*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+d_B4SOIpdbcp_dX)+instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpdbcp))*pParam_B4SOIdlcig)*pParam_B4SOIToxRatioEdge);
instanceStruct.instanceVar_pParam_B4SOIAechvbEdged = (((instanceStruct.instanceVar_pParam_B4SOIAechvb*((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)+B4SOIpdbcp))*pParam_B4SOIdlcig)*pParam_B4SOIToxRatioEdge);
instanceStruct.d_instanceVar_pParam_B4SOIBechvbEdge_dX = (((-instanceStruct.instanceVar_pParam_B4SOIBechvb)*instanceStruct.instanceVar_B4SOItoxqm)*instanceStruct.d_instanceVar_pParam_B4SOIpoxedge_dX+((-instanceStruct.instanceVar_pParam_B4SOIBechvb)*instanceStruct.d_instanceVar_B4SOItoxqm_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIBechvb_dX)*instanceStruct.instanceVar_B4SOItoxqm)*instanceStruct.instanceVar_pParam_B4SOIpoxedge);
instanceStruct.instanceVar_pParam_B4SOIBechvbEdge = (((-instanceStruct.instanceVar_pParam_B4SOIBechvb)*instanceStruct.instanceVar_B4SOItoxqm)*instanceStruct.instanceVar_pParam_B4SOIpoxedge);
instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX = ((instanceStruct.instanceVar_pParam_B4SOIAechvb*pParam_B4SOIToxRatio)*(((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-instanceStruct.instanceVar_pParam_B4SOIweff*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_pParam_B4SOIleff)+((instanceStruct.instanceVar_B4SOInf*instanceStruct.d_instanceVar_B4SOIagbcpd_dX-instanceStruct.instanceVar_B4SOIagbcpd*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf))+(instanceStruct.instanceVar_pParam_B4SOIAechvb*d_pParam_B4SOIToxRatio_dX+instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX*pParam_B4SOIToxRatio)*(((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_pParam_B4SOIleff)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf)));
instanceStruct.instanceVar_pParam_B4SOIAechvb = ((instanceStruct.instanceVar_pParam_B4SOIAechvb*pParam_B4SOIToxRatio)*(((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_pParam_B4SOIleff)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf)));
instanceStruct.d_instanceVar_pParam_B4SOIBechvb_dX = (instanceStruct.instanceVar_pParam_B4SOIBechvb*(-instanceStruct.d_instanceVar_B4SOItoxqm_dX)+instanceStruct.d_instanceVar_pParam_B4SOIBechvb_dX*(-instanceStruct.instanceVar_B4SOItoxqm));
instanceStruct.instanceVar_pParam_B4SOIBechvb = (instanceStruct.instanceVar_pParam_B4SOIBechvb*(-instanceStruct.instanceVar_B4SOItoxqm));
if ((modelStruct.modelPar_given_K1||modelStruct.modelPar_given_K2))
{
if ((!modelStruct.modelPar_given_K1))
{
UserInfo(theInstance)  << "Warning: k1 should be specified with k2." << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIk1_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIk1 = 0.53;
}
if ((!modelStruct.modelPar_given_K2))
{
UserInfo(theInstance)  << "Warning: k2 should be specified with k1." << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIk2_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIk2 = (-0.0186);
}
if (modelStruct.modelPar_given_XT)
{
UserInfo(theInstance)  << "Warning: xt is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_VBX)
{
UserInfo(theInstance)  << "Warning: vbx is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_VBM)
{
UserInfo(theInstance)  << "Warning: vbm is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_GAMMA1)
{
UserInfo(theInstance)  << "Warning: gamma1 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_GAMMA2)
{
UserInfo(theInstance)  << "Warning: gamma2 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
}
else
{
if ((!modelStruct.modelPar_given_VBX))
{
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
d_T0_dX = (-1.60219e-19*2.0*instanceStruct.d_instanceVar_epssub_dX/(2.0*instanceStruct.instanceVar_epssub)/(2.0*instanceStruct.instanceVar_epssub))*1.0e6;
T0 = ((1.60219e-19/(2.0*instanceStruct.instanceVar_epssub))*1.0e6);
}
else
{
d_T0_dX = 0.0;
T0 = 7.7348e-4;
}
instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX = (instanceStruct.d_instanceVar_pParam_B4SOIphi_dX-(((T0*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOIxt)*instanceStruct.d_instanceVar_pParam_B4SOIxt_dX+((T0*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.d_instanceVar_pParam_B4SOIxt_dX+(T0*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX+d_T0_dX*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOIxt)*instanceStruct.instanceVar_pParam_B4SOIxt));
instanceStruct.instanceVar_pParam_B4SOIvbx = (instanceStruct.instanceVar_pParam_B4SOIphi-(((T0*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOIxt)*instanceStruct.instanceVar_pParam_B4SOIxt));
}
if ((instanceStruct.instanceVar_pParam_B4SOIvbx>0.0))
{
instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX);
instanceStruct.instanceVar_pParam_B4SOIvbx = (-instanceStruct.instanceVar_pParam_B4SOIvbx);
}
if ((instanceStruct.instanceVar_pParam_B4SOIvbm>0.0))
{
instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX);
instanceStruct.instanceVar_pParam_B4SOIvbm = (-instanceStruct.instanceVar_pParam_B4SOIvbm);
}
if ((!modelStruct.modelPar_given_GAMMA1))
{
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_pParam_B4SOInpeak);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX = ((instanceStruct.instanceVar_B4SOIcox*(instanceStruct.instanceVar_sqrt2qeps*deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX)+instanceStruct.d_instanceVar_sqrt2qeps_dX*value_sqrt_0)-(instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
instanceStruct.instanceVar_pParam_B4SOIgamma1 = ((instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)/instanceStruct.instanceVar_B4SOIcox);
}
}
if ((!modelStruct.modelPar_given_GAMMA2))
{
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_pParam_B4SOInsub);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX = ((instanceStruct.instanceVar_B4SOIcox*(instanceStruct.instanceVar_sqrt2qeps*deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOInsub_dX)+instanceStruct.d_instanceVar_sqrt2qeps_dX*value_sqrt_0)-(instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
instanceStruct.instanceVar_pParam_B4SOIgamma2 = ((instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)/instanceStruct.instanceVar_B4SOIcox);
}
}
d_T0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX-instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX);
T0 = (instanceStruct.instanceVar_pParam_B4SOIgamma1-instanceStruct.instanceVar_pParam_B4SOIgamma2);
{
double value_sqrt_0 = sqrt((instanceStruct.instanceVar_pParam_B4SOIphi-instanceStruct.instanceVar_pParam_B4SOIvbx));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = (deriv_sqrt_0_d0*((instanceStruct.d_instanceVar_pParam_B4SOIphi_dX-instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX))-instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX);
T1 = (value_sqrt_0-instanceStruct.instanceVar_pParam_B4SOIsqrtPhi);
}
{
double value_sqrt_0 = sqrt((instanceStruct.instanceVar_pParam_B4SOIphi-instanceStruct.instanceVar_pParam_B4SOIvbm));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = (instanceStruct.instanceVar_pParam_B4SOIsqrtPhi*(deriv_sqrt_0_d0*((instanceStruct.d_instanceVar_pParam_B4SOIphi_dX-instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX))-instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX)+instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX*(value_sqrt_0-instanceStruct.instanceVar_pParam_B4SOIsqrtPhi));
T2 = (instanceStruct.instanceVar_pParam_B4SOIsqrtPhi*(value_sqrt_0-instanceStruct.instanceVar_pParam_B4SOIsqrtPhi));
}
instanceStruct.d_instanceVar_pParam_B4SOIk2_dX = ((((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm)*(T0*d_T1_dX+d_T0_dX*T1)-(T0*T1)*(2.0*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX))/((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm)/((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm));
instanceStruct.instanceVar_pParam_B4SOIk2 = ((T0*T1)/((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm));
{
double value_sqrt_0 = sqrt((instanceStruct.instanceVar_pParam_B4SOIphi-instanceStruct.instanceVar_pParam_B4SOIvbm));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIk1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX-((2.0*instanceStruct.instanceVar_pParam_B4SOIk2)*deriv_sqrt_0_d0*((instanceStruct.d_instanceVar_pParam_B4SOIphi_dX-instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX))+2.0*instanceStruct.d_instanceVar_pParam_B4SOIk2_dX*value_sqrt_0));
instanceStruct.instanceVar_pParam_B4SOIk1 = (instanceStruct.instanceVar_pParam_B4SOIgamma2-((2.0*instanceStruct.instanceVar_pParam_B4SOIk2)*value_sqrt_0));
}
}
d_T0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1w2_dX);
T0 = (instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIk1w2);
if ((T0<1e-8))
{
d_T0_dX = 0.0;
T0 = 1e-8;
}
instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX = (instanceStruct.instanceVar_pParam_B4SOIk1*((T0*instanceStruct.d_instanceVar_pParam_B4SOIk1w1_dX-instanceStruct.instanceVar_pParam_B4SOIk1w1*d_T0_dX)/T0/T0)+instanceStruct.d_instanceVar_pParam_B4SOIk1_dX*(1+(instanceStruct.instanceVar_pParam_B4SOIk1w1/T0)));
instanceStruct.instanceVar_pParam_B4SOIk1eff = (instanceStruct.instanceVar_pParam_B4SOIk1*(1+(instanceStruct.instanceVar_pParam_B4SOIk1w1/T0)));
if ((!modelStruct.modelPar_given_VFB))
{
if ((modelStruct.modelPar_given_VTH0||modelStruct.modelPar_given_VTHO))
{
instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX = ((instanceStruct.instanceVar_B4SOItype*d_pParam_B4SOIvth0_dX-instanceStruct.d_instanceVar_pParam_B4SOIphi_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi));
instanceStruct.instanceVar_pParam_B4SOIvfb = (((instanceStruct.instanceVar_B4SOItype*pParam_B4SOIvth0)-instanceStruct.instanceVar_pParam_B4SOIphi)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi));
}
else
{
instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIvfb = (-1.0);
}
}
if ((!modelStruct.modelPar_given_VTH0))
{
d_pParam_B4SOIvth0_dX = instanceStruct.instanceVar_B4SOItype*((instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX+instanceStruct.d_instanceVar_pParam_B4SOIphi_dX)+(instanceStruct.instanceVar_pParam_B4SOIk1eff*instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi));
pParam_B4SOIvth0 = (instanceStruct.instanceVar_B4SOItype*((instanceStruct.instanceVar_pParam_B4SOIvfb+instanceStruct.instanceVar_pParam_B4SOIphi)+(instanceStruct.instanceVar_pParam_B4SOIk1eff*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi)));
}
instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX = ((instanceStruct.instanceVar_B4SOItoxm*(instanceStruct.instanceVar_pParam_B4SOIk1eff*instanceStruct.d_instanceVar_B4SOItox_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*instanceStruct.instanceVar_B4SOItox)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*instanceStruct.instanceVar_B4SOItox)*instanceStruct.d_instanceVar_B4SOItoxm_dX)/instanceStruct.instanceVar_B4SOItoxm/instanceStruct.instanceVar_B4SOItoxm);
instanceStruct.instanceVar_pParam_B4SOIk1ox = ((instanceStruct.instanceVar_pParam_B4SOIk1eff*instanceStruct.instanceVar_B4SOItox)/instanceStruct.instanceVar_B4SOItoxm);
d_T1_dX = (instanceStruct.instanceVar_B4SOIfactor1*d_pParam_B4SOIsqrtXdep0_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*pParam_B4SOIsqrtXdep0);
T1 = (instanceStruct.instanceVar_B4SOIfactor1*pParam_B4SOIsqrtXdep0);
{
double value_exp_0 = exp(((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdsub)*instanceStruct.instanceVar_pParam_B4SOIleff)/T1));
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(((T1*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdsub)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdsub_dX*instanceStruct.instanceVar_pParam_B4SOIleff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdsub)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_T1_dX)/T1/T1));
T0 = value_exp_0;
}
instanceStruct.d_instanceVar_pParam_B4SOItheta0vb0_dX = (d_T0_dX+((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0));
instanceStruct.instanceVar_pParam_B4SOItheta0vb0 = (T0+((2.0*T0)*T0));
{
double value_exp_0 = exp(((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdrout)*instanceStruct.instanceVar_pParam_B4SOIleff)/T1));
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(((T1*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdrout)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdrout_dX*instanceStruct.instanceVar_pParam_B4SOIleff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdrout)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_T1_dX)/T1/T1));
T0 = value_exp_0;
}
d_T2_dX = (d_T0_dX+((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0));
T2 = (T0+((2.0*T0)*T0));
instanceStruct.d_instanceVar_pParam_B4SOIthetaRout_dX = ((instanceStruct.instanceVar_pParam_B4SOIpdibl1*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIpdibl1_dX*T2)+instanceStruct.d_instanceVar_pParam_B4SOIpdibl2_dX);
instanceStruct.instanceVar_pParam_B4SOIthetaRout = ((instanceStruct.instanceVar_pParam_B4SOIpdibl1*T2)+instanceStruct.instanceVar_pParam_B4SOIpdibl2);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(instanceStruct.instanceVar_pParam_B4SOIleff);
double value_exp_1 = exp((pParam_B4SOIdvtp3*evaluator_lln_0.getValues(instanceStruct.instanceVar_pParam_B4SOIleff)));
double  deriv_exp_1_d0 = value_exp_1;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp2factor_dX = ((value_exp_1*d_pParam_B4SOIdvtp2_dX-pParam_B4SOIdvtp2*deriv_exp_1_d0*((pParam_B4SOIdvtp3*evaluator_lln_0.getDerivs(instanceStruct.instanceVar_pParam_B4SOIleff,instanceStruct.d_instanceVar_pParam_B4SOIleff_dX)+d_pParam_B4SOIdvtp3_dX*evaluator_lln_0.getValues(instanceStruct.instanceVar_pParam_B4SOIleff))))/value_exp_1/value_exp_1);
instanceStruct.instanceVar_pParam_B4SOIdvtp2factor = (pParam_B4SOIdvtp2/value_exp_1);
}
if ((B4SOIwlod<0.0))
{
UserInfo(theInstance)  << "Warning: WLOD = %g is less than 0. Set to 0.0" << " " << B4SOIwlod << " " <<  std::endl; 
d_B4SOIwlod_dX = 0.0;
B4SOIwlod = 0.0;
}
{
double value_pow_0 = pow(Ldrn,B4SOIllodku0);
double  deriv_pow_0_d0 = ((Ldrn == 0.0)?0.0:(value_pow_0*B4SOIllodku0/Ldrn));
double  deriv_pow_0_d1 = (Ldrn == 0.0)?0.0:(log(Ldrn)*value_pow_0);
d_T0_dX = ((deriv_pow_0_d0*(d_Ldrn_dX))+(deriv_pow_0_d1*(d_B4SOIllodku0_dX)));
T0 = value_pow_0;
}
d_W_tmp_dX = (d_Wdrn_dX+d_B4SOIwlod_dX);
W_tmp = (Wdrn+B4SOIwlod);
{
double value_pow_0 = pow(W_tmp,B4SOIwlodku0);
double  deriv_pow_0_d0 = ((W_tmp == 0.0)?0.0:(value_pow_0*B4SOIwlodku0/W_tmp));
double  deriv_pow_0_d1 = (W_tmp == 0.0)?0.0:(log(W_tmp)*value_pow_0);
d_T1_dX = ((deriv_pow_0_d0*(d_W_tmp_dX))+(deriv_pow_0_d1*(d_B4SOIwlodku0_dX)));
T1 = value_pow_0;
}
d_tmp1_dX = ((((T0*modelStruct.d_modelPar_LKU0_dX-modelStruct.modelPar_LKU0*d_T0_dX)/T0/T0)+((T1*modelStruct.d_modelPar_WKU0_dX-modelStruct.modelPar_WKU0*d_T1_dX)/T1/T1))+(((T0*T1)*modelStruct.d_modelPar_PKU0_dX-modelStruct.modelPar_PKU0*(T0*d_T1_dX+d_T0_dX*T1))/(T0*T1)/(T0*T1)));
tmp1 = (((modelStruct.modelPar_LKU0/T0)+(modelStruct.modelPar_WKU0/T1))+(modelStruct.modelPar_PKU0/(T0*T1)));
instanceStruct.d_instanceVar_pParam_B4SOIku0_dX = d_tmp1_dX;
instanceStruct.instanceVar_pParam_B4SOIku0 = (1.0+tmp1);
{
double value_pow_0 = pow(Ldrn,B4SOIllodvth);
double  deriv_pow_0_d0 = ((Ldrn == 0.0)?0.0:(value_pow_0*B4SOIllodvth/Ldrn));
double  deriv_pow_0_d1 = (Ldrn == 0.0)?0.0:(log(Ldrn)*value_pow_0);
d_T0_dX = ((deriv_pow_0_d0*(d_Ldrn_dX))+(deriv_pow_0_d1*(d_B4SOIllodvth_dX)));
T0 = value_pow_0;
}
{
double value_pow_0 = pow(W_tmp,B4SOIwlodvth);
double  deriv_pow_0_d0 = ((W_tmp == 0.0)?0.0:(value_pow_0*B4SOIwlodvth/W_tmp));
double  deriv_pow_0_d1 = (W_tmp == 0.0)?0.0:(log(W_tmp)*value_pow_0);
d_T1_dX = ((deriv_pow_0_d0*(d_W_tmp_dX))+(deriv_pow_0_d1*(d_B4SOIwlodvth_dX)));
T1 = value_pow_0;
}
d_tmp1_dX = ((((T0*modelStruct.d_modelPar_LKVTH0_dX-modelStruct.modelPar_LKVTH0*d_T0_dX)/T0/T0)+((T1*modelStruct.d_modelPar_WKVTH0_dX-modelStruct.modelPar_WKVTH0*d_T1_dX)/T1/T1))+(((T0*T1)*modelStruct.d_modelPar_PKVTH0_dX-modelStruct.modelPar_PKVTH0*(T0*d_T1_dX+d_T0_dX*T1))/(T0*T1)/(T0*T1)));
tmp1 = (((modelStruct.modelPar_LKVTH0/T0)+(modelStruct.modelPar_WKVTH0/T1))+(modelStruct.modelPar_PKVTH0/(T0*T1)));
d_pParam_B4SOIkvth0_dX = d_tmp1_dX;
pParam_B4SOIkvth0 = (1.0+tmp1);
{
double value_sqrt_0 = sqrt(((pParam_B4SOIkvth0*pParam_B4SOIkvth0)+1.0E-9));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_pParam_B4SOIkvth0_dX = deriv_sqrt_0_d0*((pParam_B4SOIkvth0*d_pParam_B4SOIkvth0_dX+d_pParam_B4SOIkvth0_dX*pParam_B4SOIkvth0));
pParam_B4SOIkvth0 = value_sqrt_0;
}
d_T0_dX = d_TempRatio_dX;
T0 = (TempRatio-1.0);
d_pParam_B4SOIku0temp_dX = (instanceStruct.instanceVar_pParam_B4SOIku0*(instanceStruct.instanceVar_B4SOItku0*d_T0_dX+instanceStruct.d_instanceVar_B4SOItku0_dX*T0)+instanceStruct.d_instanceVar_pParam_B4SOIku0_dX*(1.0+(instanceStruct.instanceVar_B4SOItku0*T0)));
pParam_B4SOIku0temp = ((instanceStruct.instanceVar_pParam_B4SOIku0*(1.0+(instanceStruct.instanceVar_B4SOItku0*T0)))+1.0E-9);
d_Inv_saref_dX = (-(instanceStruct.d_instanceVar_B4SOIsaref_dX+0.5*d_Ldrn_dX)/(instanceStruct.instanceVar_B4SOIsaref+(0.5*Ldrn))/(instanceStruct.instanceVar_B4SOIsaref+(0.5*Ldrn)));
Inv_saref = (1.0/(instanceStruct.instanceVar_B4SOIsaref+(0.5*Ldrn)));
d_Inv_sbref_dX = (-(instanceStruct.d_instanceVar_B4SOIsbref_dX+0.5*d_Ldrn_dX)/(instanceStruct.instanceVar_B4SOIsbref+(0.5*Ldrn))/(instanceStruct.instanceVar_B4SOIsbref+(0.5*Ldrn)));
Inv_sbref = (1.0/(instanceStruct.instanceVar_B4SOIsbref+(0.5*Ldrn)));
instanceStruct.d_instanceVar_pParam_B4SOIinv_od_ref_dX = (d_Inv_saref_dX+d_Inv_sbref_dX);
instanceStruct.instanceVar_pParam_B4SOIinv_od_ref = (Inv_saref+Inv_sbref);
d_pParam_B4SOIrho_ref_dX = ((instanceStruct.instanceVar_B4SOIku0/pParam_B4SOIku0temp)*instanceStruct.d_instanceVar_pParam_B4SOIinv_od_ref_dX+((pParam_B4SOIku0temp*instanceStruct.d_instanceVar_B4SOIku0_dX-instanceStruct.instanceVar_B4SOIku0*d_pParam_B4SOIku0temp_dX)/pParam_B4SOIku0temp/pParam_B4SOIku0temp)*instanceStruct.instanceVar_pParam_B4SOIinv_od_ref);
pParam_B4SOIrho_ref = ((instanceStruct.instanceVar_B4SOIku0/pParam_B4SOIku0temp)*instanceStruct.instanceVar_pParam_B4SOIinv_od_ref);
if ((((instanceStruct.instanceVar_B4SOIsa>0.0)&&(instanceStruct.instanceVar_B4SOIsb>0.0))&&((instanceStruct.instanceVar_B4SOInf==1.0)||((instanceStruct.instanceVar_B4SOInf>1.0)&&(instanceStruct.instanceVar_B4SOIsd>0.0)))))
{
d_Inv_sa_dX = 0.0;
Inv_sa = 0;
d_Inv_sb_dX = 0.0;
Inv_sb = 0;
if ((instanceStruct.instanceVar_B4SOIkvsat<(-1.0)))
{
UserInfo(theInstance)  << "Warning: KVSAT = %g is too small; Reset to -1.0." << " " << instanceStruct.instanceVar_B4SOIkvsat << " " <<  std::endl; 
instanceStruct.d_instanceVar_B4SOIkvsat_dX = 0.0;
instanceStruct.instanceVar_B4SOIkvsat = (-1.0);
}
else
{
if ((instanceStruct.instanceVar_B4SOIkvsat>1.0))
{
UserInfo(theInstance)  << "Warning: KVSAT = %g is too big; Reset to 1.0." << " " << instanceStruct.instanceVar_B4SOIkvsat << " " <<  std::endl; 
instanceStruct.d_instanceVar_B4SOIkvsat_dX = 0.0;
instanceStruct.instanceVar_B4SOIkvsat = 1.0;
}
else
{
}
}
for (i=0;(i<instanceStruct.instanceVar_B4SOInf);i=(i+1) )//Begin block forloop
{
//Block-local variables for block forloop
double T0=0.0;
double d_T0_dX=0.0;
double T1=0.0;
double d_T1_dX=0.0;
//End of Block-local variables
d_T0_dX = ((((instanceStruct.instanceVar_B4SOIsa+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn)))*(-instanceStruct.d_instanceVar_B4SOInf_dX/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf)-(1.0/instanceStruct.instanceVar_B4SOInf)*((instanceStruct.d_instanceVar_B4SOIsa_dX+0.5*d_Ldrn_dX)+i*(instanceStruct.d_instanceVar_B4SOIsd_dX+d_Ldrn_dX)))/((instanceStruct.instanceVar_B4SOIsa+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn)))/((instanceStruct.instanceVar_B4SOIsa+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn))));
T0 = ((1.0/instanceStruct.instanceVar_B4SOInf)/((instanceStruct.instanceVar_B4SOIsa+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn))));
d_T1_dX = ((((instanceStruct.instanceVar_B4SOIsb+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn)))*(-instanceStruct.d_instanceVar_B4SOInf_dX/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf)-(1.0/instanceStruct.instanceVar_B4SOInf)*((instanceStruct.d_instanceVar_B4SOIsb_dX+0.5*d_Ldrn_dX)+i*(instanceStruct.d_instanceVar_B4SOIsd_dX+d_Ldrn_dX)))/((instanceStruct.instanceVar_B4SOIsb+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn)))/((instanceStruct.instanceVar_B4SOIsb+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn))));
T1 = ((1.0/instanceStruct.instanceVar_B4SOInf)/((instanceStruct.instanceVar_B4SOIsb+(0.5*Ldrn))+(i*(instanceStruct.instanceVar_B4SOIsd+Ldrn))));
d_Inv_sa_dX = (d_Inv_sa_dX+d_T0_dX);
Inv_sa = (Inv_sa+T0);
d_Inv_sb_dX = (d_Inv_sb_dX+d_T1_dX);
Inv_sb = (Inv_sb+T1);
}
// End block forloop
d_Inv_ODeff_dX = (d_Inv_sa_dX+d_Inv_sb_dX);
Inv_ODeff = (Inv_sa+Inv_sb);
instanceStruct.d_instanceVar_B4SOIInv_ODeff_dX = d_Inv_ODeff_dX;
instanceStruct.instanceVar_B4SOIInv_ODeff = Inv_ODeff;
d_rho_dX = ((instanceStruct.instanceVar_B4SOIku0/pParam_B4SOIku0temp)*d_Inv_ODeff_dX+((pParam_B4SOIku0temp*instanceStruct.d_instanceVar_B4SOIku0_dX-instanceStruct.instanceVar_B4SOIku0*d_pParam_B4SOIku0temp_dX)/pParam_B4SOIku0temp/pParam_B4SOIku0temp)*Inv_ODeff);
rho = ((instanceStruct.instanceVar_B4SOIku0/pParam_B4SOIku0temp)*Inv_ODeff);
d_T0_dX = (((1.0+pParam_B4SOIrho_ref)*d_rho_dX-(1.0+rho)*d_pParam_B4SOIrho_ref_dX)/(1.0+pParam_B4SOIrho_ref)/(1.0+pParam_B4SOIrho_ref));
T0 = ((1.0+rho)/(1.0+pParam_B4SOIrho_ref));
instanceStruct.d_instanceVar_here_B4SOIu0temp_dX = (instanceStruct.instanceVar_pParam_B4SOIu0temp*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIu0temp_dX*T0);
instanceStruct.instanceVar_here_B4SOIu0temp = (instanceStruct.instanceVar_pParam_B4SOIu0temp*T0);
d_T1_dX = (((1.0+(instanceStruct.instanceVar_B4SOIkvsat*pParam_B4SOIrho_ref))*(instanceStruct.instanceVar_B4SOIkvsat*d_rho_dX+instanceStruct.d_instanceVar_B4SOIkvsat_dX*rho)-(1.0+(instanceStruct.instanceVar_B4SOIkvsat*rho))*(instanceStruct.instanceVar_B4SOIkvsat*d_pParam_B4SOIrho_ref_dX+instanceStruct.d_instanceVar_B4SOIkvsat_dX*pParam_B4SOIrho_ref))/(1.0+(instanceStruct.instanceVar_B4SOIkvsat*pParam_B4SOIrho_ref))/(1.0+(instanceStruct.instanceVar_B4SOIkvsat*pParam_B4SOIrho_ref)));
T1 = ((1.0+(instanceStruct.instanceVar_B4SOIkvsat*rho))/(1.0+(instanceStruct.instanceVar_B4SOIkvsat*pParam_B4SOIrho_ref)));
instanceStruct.d_instanceVar_here_B4SOIvsattemp_dX = (instanceStruct.instanceVar_pParam_B4SOIvsattemp*d_T1_dX+instanceStruct.d_instanceVar_pParam_B4SOIvsattemp_dX*T1);
instanceStruct.instanceVar_here_B4SOIvsattemp = (instanceStruct.instanceVar_pParam_B4SOIvsattemp*T1);
d_OD_offset_dX = (d_Inv_ODeff_dX-instanceStruct.d_instanceVar_pParam_B4SOIinv_od_ref_dX);
OD_offset = (Inv_ODeff-instanceStruct.instanceVar_pParam_B4SOIinv_od_ref);
d_dvth0_lod_dX = ((B4SOIkvth0/pParam_B4SOIkvth0)*d_OD_offset_dX+((pParam_B4SOIkvth0*d_B4SOIkvth0_dX-B4SOIkvth0*d_pParam_B4SOIkvth0_dX)/pParam_B4SOIkvth0/pParam_B4SOIkvth0)*OD_offset);
dvth0_lod = ((B4SOIkvth0/pParam_B4SOIkvth0)*OD_offset);
{
double value_pow_0 = pow(pParam_B4SOIkvth0,instanceStruct.instanceVar_B4SOIlodk2);
double  deriv_pow_0_d0 = ((pParam_B4SOIkvth0 == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_B4SOIlodk2/pParam_B4SOIkvth0));
double  deriv_pow_0_d1 = (pParam_B4SOIkvth0 == 0.0)?0.0:(log(pParam_B4SOIkvth0)*value_pow_0);
d_dk2_lod_dX = ((B4SOIstk2/value_pow_0)*d_OD_offset_dX+((value_pow_0*d_B4SOIstk2_dX-B4SOIstk2*((deriv_pow_0_d0*(d_pParam_B4SOIkvth0_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_B4SOIlodk2_dX))))/value_pow_0/value_pow_0)*OD_offset);
dk2_lod = ((B4SOIstk2/value_pow_0)*OD_offset);
}
{
double value_pow_0 = pow(pParam_B4SOIkvth0,instanceStruct.instanceVar_B4SOIlodeta0);
double  deriv_pow_0_d0 = ((pParam_B4SOIkvth0 == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_B4SOIlodeta0/pParam_B4SOIkvth0));
double  deriv_pow_0_d1 = (pParam_B4SOIkvth0 == 0.0)?0.0:(log(pParam_B4SOIkvth0)*value_pow_0);
d_deta0_lod_dX = ((B4SOIsteta0/value_pow_0)*d_OD_offset_dX+((value_pow_0*d_B4SOIsteta0_dX-B4SOIsteta0*((deriv_pow_0_d0*(d_pParam_B4SOIkvth0_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_B4SOIlodeta0_dX))))/value_pow_0/value_pow_0)*OD_offset);
deta0_lod = ((B4SOIsteta0/value_pow_0)*OD_offset);
}
{
double value_pow_0 = pow(pParam_B4SOIkvth0,instanceStruct.instanceVar_B4SOIlodeta0cv);
double  deriv_pow_0_d0 = ((pParam_B4SOIkvth0 == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_B4SOIlodeta0cv/pParam_B4SOIkvth0));
double  deriv_pow_0_d1 = (pParam_B4SOIkvth0 == 0.0)?0.0:(log(pParam_B4SOIkvth0)*value_pow_0);
d_deta0cv_lod_dX = ((B4SOIsteta0cv/value_pow_0)*d_OD_offset_dX+((value_pow_0*d_B4SOIsteta0cv_dX-B4SOIsteta0cv*((deriv_pow_0_d0*(d_pParam_B4SOIkvth0_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_B4SOIlodeta0cv_dX))))/value_pow_0/value_pow_0)*OD_offset);
deta0cv_lod = ((B4SOIsteta0cv/value_pow_0)*OD_offset);
}
instanceStruct.d_instanceVar_here_B4SOIvth0_dX = (d_pParam_B4SOIvth0_dX+d_dvth0_lod_dX);
instanceStruct.instanceVar_here_B4SOIvth0 = (pParam_B4SOIvth0+dvth0_lod);
instanceStruct.d_instanceVar_here_B4SOIk2_dX = (instanceStruct.d_instanceVar_pParam_B4SOIk2_dX+d_dk2_lod_dX);
instanceStruct.instanceVar_here_B4SOIk2 = (instanceStruct.instanceVar_pParam_B4SOIk2+dk2_lod);
instanceStruct.d_instanceVar_here_B4SOIeta0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIeta0_dX+d_deta0_lod_dX);
instanceStruct.instanceVar_here_B4SOIeta0 = (instanceStruct.instanceVar_pParam_B4SOIeta0+deta0_lod);
instanceStruct.d_instanceVar_here_B4SOIeta0cv_dX = (instanceStruct.d_instanceVar_pParam_B4SOIeta0cv_dX+d_deta0cv_lod_dX);
instanceStruct.instanceVar_here_B4SOIeta0cv = (instanceStruct.instanceVar_pParam_B4SOIeta0cv+deta0cv_lod);
}
else
{
instanceStruct.d_instanceVar_here_B4SOIu0temp_dX = instanceStruct.d_instanceVar_pParam_B4SOIu0temp_dX;
instanceStruct.instanceVar_here_B4SOIu0temp = instanceStruct.instanceVar_pParam_B4SOIu0temp;
instanceStruct.d_instanceVar_here_B4SOIvth0_dX = d_pParam_B4SOIvth0_dX;
instanceStruct.instanceVar_here_B4SOIvth0 = pParam_B4SOIvth0;
instanceStruct.d_instanceVar_here_B4SOIvsattemp_dX = instanceStruct.d_instanceVar_pParam_B4SOIvsattemp_dX;
instanceStruct.instanceVar_here_B4SOIvsattemp = instanceStruct.instanceVar_pParam_B4SOIvsattemp;
instanceStruct.d_instanceVar_here_B4SOIk2_dX = instanceStruct.d_instanceVar_pParam_B4SOIk2_dX;
instanceStruct.instanceVar_here_B4SOIk2 = instanceStruct.instanceVar_pParam_B4SOIk2;
instanceStruct.d_instanceVar_here_B4SOIeta0_dX = instanceStruct.d_instanceVar_pParam_B4SOIeta0_dX;
instanceStruct.instanceVar_here_B4SOIeta0 = instanceStruct.instanceVar_pParam_B4SOIeta0;
instanceStruct.d_instanceVar_here_B4SOIeta0cv_dX = instanceStruct.d_instanceVar_pParam_B4SOIeta0cv_dX;
instanceStruct.instanceVar_here_B4SOIeta0cv = instanceStruct.instanceVar_pParam_B4SOIeta0cv;
instanceStruct.d_instanceVar_B4SOIInv_ODeff_dX = 0.0;
instanceStruct.instanceVar_B4SOIInv_ODeff = 0;
instanceStruct.d_instanceVar_pParam_B4SOIinv_od_ref_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIinv_od_ref = 0;
instanceStruct.d_instanceVar_B4SOIkvsat_dX = 0.0;
instanceStruct.instanceVar_B4SOIkvsat = 0;
}
instanceStruct.d_instanceVar_here_B4SOIk2ox_dX = ((instanceStruct.instanceVar_B4SOItoxm*(instanceStruct.instanceVar_here_B4SOIk2*instanceStruct.d_instanceVar_B4SOItox_dX+instanceStruct.d_instanceVar_here_B4SOIk2_dX*instanceStruct.instanceVar_B4SOItox)-(instanceStruct.instanceVar_here_B4SOIk2*instanceStruct.instanceVar_B4SOItox)*instanceStruct.d_instanceVar_B4SOItoxm_dX)/instanceStruct.instanceVar_B4SOItoxm/instanceStruct.instanceVar_B4SOItoxm);
instanceStruct.instanceVar_here_B4SOIk2ox = ((instanceStruct.instanceVar_here_B4SOIk2*instanceStruct.instanceVar_B4SOItox)/instanceStruct.instanceVar_B4SOItoxm);
instanceStruct.d_instanceVar_here_B4SOIvth0_dX = (instanceStruct.d_instanceVar_here_B4SOIvth0_dX+d_B4SOIdelvto_dX);
instanceStruct.instanceVar_here_B4SOIvth0 = (instanceStruct.instanceVar_here_B4SOIvth0+B4SOIdelvto);
instanceStruct.d_instanceVar_here_B4SOIvfb_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX+instanceStruct.instanceVar_B4SOItype*d_B4SOIdelvto_dX);
instanceStruct.instanceVar_here_B4SOIvfb = (instanceStruct.instanceVar_pParam_B4SOIvfb+(instanceStruct.instanceVar_B4SOItype*B4SOIdelvto));
instanceStruct.d_instanceVar_B4SOIcsbox_dX = (instanceStruct.instanceVar_B4SOIcbox*d_B4SOIsourceArea_dX+instanceStruct.d_instanceVar_B4SOIcbox_dX*B4SOIsourceArea);
instanceStruct.instanceVar_B4SOIcsbox = (instanceStruct.instanceVar_B4SOIcbox*B4SOIsourceArea);
instanceStruct.d_instanceVar_B4SOIcsmin_dX = (instanceStruct.instanceVar_B4SOIcsdmin*d_B4SOIsourceArea_dX+instanceStruct.d_instanceVar_B4SOIcsdmin_dX*B4SOIsourceArea);
instanceStruct.instanceVar_B4SOIcsmin = (instanceStruct.instanceVar_B4SOIcsdmin*B4SOIsourceArea);
instanceStruct.d_instanceVar_B4SOIcdbox_dX = (instanceStruct.instanceVar_B4SOIcbox*d_B4SOIdrainArea_dX+instanceStruct.d_instanceVar_B4SOIcbox_dX*B4SOIdrainArea);
instanceStruct.instanceVar_B4SOIcdbox = (instanceStruct.instanceVar_B4SOIcbox*B4SOIdrainArea);
instanceStruct.d_instanceVar_B4SOIcdmin_dX = (instanceStruct.instanceVar_B4SOIcsdmin*d_B4SOIdrainArea_dX+instanceStruct.d_instanceVar_B4SOIcsdmin_dX*B4SOIdrainArea);
instanceStruct.instanceVar_B4SOIcdmin = (instanceStruct.instanceVar_B4SOIcsdmin*B4SOIdrainArea);
if ((instanceStruct.instanceVar_B4SOIcsdmin>0))
{
if ((((instanceStruct.instanceVar_pParam_B4SOInsub>0)&&(instanceStruct.instanceVar_B4SOItype>0))||((instanceStruct.instanceVar_pParam_B4SOInsub<0)&&(instanceStruct.instanceVar_B4SOItype<0))))
{
d_T0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX);
T0 = (instanceStruct.instanceVar_pParam_B4SOIvsdth-instanceStruct.instanceVar_pParam_B4SOIvsdfb);
instanceStruct.d_instanceVar_pParam_B4SOIsdt1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX+(instanceStruct.instanceVar_B4SOIasd*d_T0_dX+instanceStruct.d_instanceVar_B4SOIasd_dX*T0));
instanceStruct.instanceVar_pParam_B4SOIsdt1 = (instanceStruct.instanceVar_pParam_B4SOIvsdfb+(instanceStruct.instanceVar_B4SOIasd*T0));
d_T1_dX = (instanceStruct.d_instanceVar_B4SOIcsbox_dX-instanceStruct.d_instanceVar_B4SOIcsmin_dX);
T1 = (instanceStruct.instanceVar_B4SOIcsbox-instanceStruct.instanceVar_B4SOIcsmin);
d_T2_dX = ((T0*((T0*d_T1_dX-T1*d_T0_dX)/T0/T0)-(T1/T0)*d_T0_dX)/T0/T0);
T2 = ((T1/T0)/T0);
instanceStruct.d_instanceVar_pParam_B4SOIst2_dX = ((instanceStruct.instanceVar_B4SOIasd*d_T2_dX-T2*instanceStruct.d_instanceVar_B4SOIasd_dX)/instanceStruct.instanceVar_B4SOIasd/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.instanceVar_pParam_B4SOIst2 = (T2/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.d_instanceVar_pParam_B4SOIst3_dX = (((1-instanceStruct.instanceVar_B4SOIasd)*d_T2_dX-T2*(-instanceStruct.d_instanceVar_B4SOIasd_dX))/(1-instanceStruct.instanceVar_B4SOIasd)/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.instanceVar_pParam_B4SOIst3 = (T2/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.d_instanceVar_B4SOIst4_dX = ((((T0*T1)*instanceStruct.d_instanceVar_B4SOIasd_dX+(T0*d_T1_dX+d_T0_dX*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcsmin*instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX+instanceStruct.d_instanceVar_B4SOIcsmin_dX*instanceStruct.instanceVar_pParam_B4SOIvsdfb));
instanceStruct.instanceVar_B4SOIst4 = ((((T0*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcsmin*instanceStruct.instanceVar_pParam_B4SOIvsdfb));
d_T1_dX = (instanceStruct.d_instanceVar_B4SOIcdbox_dX-instanceStruct.d_instanceVar_B4SOIcdmin_dX);
T1 = (instanceStruct.instanceVar_B4SOIcdbox-instanceStruct.instanceVar_B4SOIcdmin);
d_T2_dX = ((T0*((T0*d_T1_dX-T1*d_T0_dX)/T0/T0)-(T1/T0)*d_T0_dX)/T0/T0);
T2 = ((T1/T0)/T0);
instanceStruct.d_instanceVar_pParam_B4SOIdt2_dX = ((instanceStruct.instanceVar_B4SOIasd*d_T2_dX-T2*instanceStruct.d_instanceVar_B4SOIasd_dX)/instanceStruct.instanceVar_B4SOIasd/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.instanceVar_pParam_B4SOIdt2 = (T2/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.d_instanceVar_pParam_B4SOIdt3_dX = (((1-instanceStruct.instanceVar_B4SOIasd)*d_T2_dX-T2*(-instanceStruct.d_instanceVar_B4SOIasd_dX))/(1-instanceStruct.instanceVar_B4SOIasd)/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.instanceVar_pParam_B4SOIdt3 = (T2/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.d_instanceVar_B4SOIdt4_dX = ((((T0*T1)*instanceStruct.d_instanceVar_B4SOIasd_dX+(T0*d_T1_dX+d_T0_dX*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcdmin*instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX+instanceStruct.d_instanceVar_B4SOIcdmin_dX*instanceStruct.instanceVar_pParam_B4SOIvsdfb));
instanceStruct.instanceVar_B4SOIdt4 = ((((T0*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcdmin*instanceStruct.instanceVar_pParam_B4SOIvsdfb));
}
else
{
d_T0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX);
T0 = (instanceStruct.instanceVar_pParam_B4SOIvsdfb-instanceStruct.instanceVar_pParam_B4SOIvsdth);
instanceStruct.d_instanceVar_pParam_B4SOIsdt1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX+(instanceStruct.instanceVar_B4SOIasd*d_T0_dX+instanceStruct.d_instanceVar_B4SOIasd_dX*T0));
instanceStruct.instanceVar_pParam_B4SOIsdt1 = (instanceStruct.instanceVar_pParam_B4SOIvsdth+(instanceStruct.instanceVar_B4SOIasd*T0));
d_T1_dX = (instanceStruct.d_instanceVar_B4SOIcsmin_dX-instanceStruct.d_instanceVar_B4SOIcsbox_dX);
T1 = (instanceStruct.instanceVar_B4SOIcsmin-instanceStruct.instanceVar_B4SOIcsbox);
d_T2_dX = ((T0*((T0*d_T1_dX-T1*d_T0_dX)/T0/T0)-(T1/T0)*d_T0_dX)/T0/T0);
T2 = ((T1/T0)/T0);
instanceStruct.d_instanceVar_pParam_B4SOIst2_dX = ((instanceStruct.instanceVar_B4SOIasd*d_T2_dX-T2*instanceStruct.d_instanceVar_B4SOIasd_dX)/instanceStruct.instanceVar_B4SOIasd/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.instanceVar_pParam_B4SOIst2 = (T2/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.d_instanceVar_pParam_B4SOIst3_dX = (((1-instanceStruct.instanceVar_B4SOIasd)*d_T2_dX-T2*(-instanceStruct.d_instanceVar_B4SOIasd_dX))/(1-instanceStruct.instanceVar_B4SOIasd)/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.instanceVar_pParam_B4SOIst3 = (T2/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.d_instanceVar_B4SOIst4_dX = ((((T0*T1)*instanceStruct.d_instanceVar_B4SOIasd_dX+(T0*d_T1_dX+d_T0_dX*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcsbox*instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX+instanceStruct.d_instanceVar_B4SOIcsbox_dX*instanceStruct.instanceVar_pParam_B4SOIvsdth));
instanceStruct.instanceVar_B4SOIst4 = ((((T0*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcsbox*instanceStruct.instanceVar_pParam_B4SOIvsdth));
d_T1_dX = (instanceStruct.d_instanceVar_B4SOIcdmin_dX-instanceStruct.d_instanceVar_B4SOIcdbox_dX);
T1 = (instanceStruct.instanceVar_B4SOIcdmin-instanceStruct.instanceVar_B4SOIcdbox);
d_T2_dX = ((T0*((T0*d_T1_dX-T1*d_T0_dX)/T0/T0)-(T1/T0)*d_T0_dX)/T0/T0);
T2 = ((T1/T0)/T0);
instanceStruct.d_instanceVar_pParam_B4SOIdt2_dX = ((instanceStruct.instanceVar_B4SOIasd*d_T2_dX-T2*instanceStruct.d_instanceVar_B4SOIasd_dX)/instanceStruct.instanceVar_B4SOIasd/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.instanceVar_pParam_B4SOIdt2 = (T2/instanceStruct.instanceVar_B4SOIasd);
instanceStruct.d_instanceVar_pParam_B4SOIdt3_dX = (((1-instanceStruct.instanceVar_B4SOIasd)*d_T2_dX-T2*(-instanceStruct.d_instanceVar_B4SOIasd_dX))/(1-instanceStruct.instanceVar_B4SOIasd)/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.instanceVar_pParam_B4SOIdt3 = (T2/(1-instanceStruct.instanceVar_B4SOIasd));
instanceStruct.d_instanceVar_B4SOIdt4_dX = ((((T0*T1)*instanceStruct.d_instanceVar_B4SOIasd_dX+(T0*d_T1_dX+d_T0_dX*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcdbox*instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX+instanceStruct.d_instanceVar_B4SOIcdbox_dX*instanceStruct.instanceVar_pParam_B4SOIvsdth));
instanceStruct.instanceVar_B4SOIdt4 = ((((T0*T1)*(1+instanceStruct.instanceVar_B4SOIasd))/3)-(instanceStruct.instanceVar_B4SOIcdbox*instanceStruct.instanceVar_pParam_B4SOIvsdth));
}
}
else
{
instanceStruct.d_instanceVar_pParam_B4SOIsdt1_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIsdt1 = 0;
instanceStruct.d_instanceVar_pParam_B4SOIst2_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIst2 = 0;
instanceStruct.d_instanceVar_pParam_B4SOIst3_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIst3 = 0;
instanceStruct.d_instanceVar_B4SOIst4_dX = 0.0;
instanceStruct.instanceVar_B4SOIst4 = 0;
instanceStruct.d_instanceVar_pParam_B4SOIdt2_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIdt2 = 0;
instanceStruct.d_instanceVar_pParam_B4SOIdt3_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIdt3 = 0;
instanceStruct.d_instanceVar_B4SOIdt4_dX = 0.0;
instanceStruct.instanceVar_B4SOIdt4 = 0;
}
if (((B4SOIcfrcoeff<1.0)||(B4SOIcfrcoeff>2.0)))
{
UserInfo(theInstance)  << "Warning: CfrCoeff = %g is outside valid range [1,2], set to 1. \n" << " " << B4SOIcfrcoeff << " " <<  std::endl; 
d_B4SOIcfrcoeff_dX = 0.0;
B4SOIcfrcoeff = 1.0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((B4SOIcfrcoeff*(1+(instanceStruct.instanceVar_B4SOItsi/instanceStruct.instanceVar_B4SOItbox))));
d_T0_dX = (instanceStruct.instanceVar_B4SOIcsdesw*evaluator_lln_0.getDerivs((B4SOIcfrcoeff*(1+(instanceStruct.instanceVar_B4SOItsi/instanceStruct.instanceVar_B4SOItbox))),(B4SOIcfrcoeff*((instanceStruct.instanceVar_B4SOItbox*instanceStruct.d_instanceVar_B4SOItsi_dX-instanceStruct.instanceVar_B4SOItsi*instanceStruct.d_instanceVar_B4SOItbox_dX)/instanceStruct.instanceVar_B4SOItbox/instanceStruct.instanceVar_B4SOItbox)+d_B4SOIcfrcoeff_dX*(1+(instanceStruct.instanceVar_B4SOItsi/instanceStruct.instanceVar_B4SOItbox))))+instanceStruct.d_instanceVar_B4SOIcsdesw_dX*evaluator_lln_0.getValues((B4SOIcfrcoeff*(1+(instanceStruct.instanceVar_B4SOItsi/instanceStruct.instanceVar_B4SOItbox)))));
T0 = (instanceStruct.instanceVar_B4SOIcsdesw*evaluator_lln_0.getValues((B4SOIcfrcoeff*(1+(instanceStruct.instanceVar_B4SOItsi/instanceStruct.instanceVar_B4SOItbox)))));
}
d_T1_dX = (d_B4SOIsourcePerimeter_dX-d_B4SOIw_dX);
T1 = (B4SOIsourcePerimeter-B4SOIw);
if ((T1>0.0))
{
instanceStruct.d_instanceVar_B4SOIcsesw_dX = (T0*d_T1_dX+d_T0_dX*T1);
instanceStruct.instanceVar_B4SOIcsesw = (T0*T1);
}
else
{
instanceStruct.d_instanceVar_B4SOIcsesw_dX = 0.0;
instanceStruct.instanceVar_B4SOIcsesw = 0.0;
}
d_T1_dX = (d_B4SOIdrainPerimeter_dX-d_B4SOIw_dX);
T1 = (B4SOIdrainPerimeter-B4SOIw);
if ((T1>0.0))
{
instanceStruct.d_instanceVar_B4SOIcdesw_dX = (T0*d_T1_dX+d_T0_dX*T1);
instanceStruct.instanceVar_B4SOIcdesw = (T0*T1);
}
else
{
instanceStruct.d_instanceVar_B4SOIcdesw_dX = 0.0;
instanceStruct.instanceVar_B4SOIcdesw = 0.0;
}
instanceStruct.d_instanceVar_B4SOIdrainResistance_dX = (B4SOIsheetResistance*d_B4SOIdrainSquares_dX+d_B4SOIsheetResistance_dX*B4SOIdrainSquares);
instanceStruct.instanceVar_B4SOIdrainResistance = (B4SOIsheetResistance*B4SOIdrainSquares);
if ((instanceStruct.instanceVar_B4SOIdrainResistance<=1.0e-3))
{
instanceStruct.d_instanceVar_B4SOIdrainResistance_dX = 0.0;
instanceStruct.instanceVar_B4SOIdrainResistance = 1.0e-3;
}
instanceStruct.d_instanceVar_B4SOIsourceResistance_dX = (B4SOIsheetResistance*d_B4SOIsourceSquares_dX+d_B4SOIsheetResistance_dX*B4SOIsourceSquares);
instanceStruct.instanceVar_B4SOIsourceResistance = (B4SOIsheetResistance*B4SOIsourceSquares);
if ((instanceStruct.instanceVar_B4SOIsourceResistance<=1.0e-3))
{
instanceStruct.d_instanceVar_B4SOIsourceResistance_dX = 0.0;
instanceStruct.instanceVar_B4SOIsourceResistance = 1.0e-3;
}
if ((B4SOIln<1e-15))
{
d_B4SOIln_dX = 0.0;
B4SOIln = 1e-15;
}
d_T0_dX = ((B4SOIln*((B4SOIln*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX*instanceStruct.instanceVar_pParam_B4SOIleff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_B4SOIln_dX)/B4SOIln/B4SOIln)-((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_pParam_B4SOIleff)/B4SOIln)*d_B4SOIln_dX)/B4SOIln/B4SOIln);
T0 = (((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_pParam_B4SOIleff)/B4SOIln)/B4SOIln);
if ((T0>100.0))
{
d_T1_dX = 2.688117142e+43*d_T0_dX;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
}
instanceStruct.d_instanceVar_pParam_B4SOIarfabjt_dX = d_T1_dX;
instanceStruct.instanceVar_pParam_B4SOIarfabjt = T1;
d_T0_dX = (pParam_B4SOIlbjt0*((-instanceStruct.d_instanceVar_pParam_B4SOIleff_dX/instanceStruct.instanceVar_pParam_B4SOIleff/instanceStruct.instanceVar_pParam_B4SOIleff)+(-d_B4SOIln_dX/B4SOIln/B4SOIln))+d_pParam_B4SOIlbjt0_dX*((1.0/instanceStruct.instanceVar_pParam_B4SOIleff)+(1.0/B4SOIln)));
T0 = (pParam_B4SOIlbjt0*((1.0/instanceStruct.instanceVar_pParam_B4SOIleff)+(1.0/B4SOIln)));
{
double value_pow_0 = pow(T0,pParam_B4SOInbjt);
double  deriv_pow_0_d0 = ((T0 == 0.0)?0.0:(value_pow_0*pParam_B4SOInbjt/T0));
double  deriv_pow_0_d1 = (T0 == 0.0)?0.0:(log(T0)*value_pow_0);
instanceStruct.d_instanceVar_pParam_B4SOIlratio_dX = ((deriv_pow_0_d0*(d_T0_dX))+(deriv_pow_0_d1*(d_pParam_B4SOInbjt_dX)));
instanceStruct.instanceVar_pParam_B4SOIlratio = value_pow_0;
}
{
double value_pow_0 = pow(T0,pParam_B4SOIndif);
double  deriv_pow_0_d0 = ((T0 == 0.0)?0.0:(value_pow_0*pParam_B4SOIndif/T0));
double  deriv_pow_0_d1 = (T0 == 0.0)?0.0:(log(T0)*value_pow_0);
instanceStruct.d_instanceVar_pParam_B4SOIlratiodif_dX = (B4SOIldif0*((deriv_pow_0_d0*(d_T0_dX))+(deriv_pow_0_d1*(d_pParam_B4SOIndif_dX)))+d_B4SOIldif0_dX*value_pow_0);
instanceStruct.instanceVar_pParam_B4SOIlratiodif = (1.0+(B4SOIldif0*value_pow_0));
}
instanceStruct.d_instanceVar_pParam_B4SOIvearly_dX = (d_pParam_B4SOIvabjt_dX+(pParam_B4SOIaely*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+d_pParam_B4SOIaely_dX*instanceStruct.instanceVar_pParam_B4SOIleff));
instanceStruct.instanceVar_pParam_B4SOIvearly = (pParam_B4SOIvabjt+(pParam_B4SOIaely*instanceStruct.instanceVar_pParam_B4SOIleff));
if ((instanceStruct.instanceVar_pParam_B4SOIvearly<1))
{
instanceStruct.d_instanceVar_pParam_B4SOIvearly_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIvearly = 1;
}
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
instanceStruct.d_instanceVar_B4SOItoxp_dX = (instanceStruct.d_instanceVar_B4SOItox_dX-instanceStruct.d_instanceVar_B4SOIdtoxcv_dX);
instanceStruct.instanceVar_B4SOItoxp = (instanceStruct.instanceVar_B4SOItox-instanceStruct.instanceVar_B4SOIdtoxcv);
}
else
{
d_Vtm0eot_dX = 8.617087e-5*d_B4SOItempeot_dX;
Vtm0eot = (8.617087e-5*B4SOItempeot);
d_Vtmeot_dX = d_Vtm0eot_dX;
Vtmeot = Vtm0eot;
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)));
d_vbieot_dX = (Vtm0eot*evaluator_lln_0.getDerivs(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)),(((ni*ni)*1.0e20*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-(1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)*(ni*d_ni_dX+d_ni_dX*ni))/(ni*ni)/(ni*ni)))+d_Vtm0eot_dX*evaluator_lln_0.getValues(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni))));
vbieot = (Vtm0eot*evaluator_lln_0.getValues(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni))));
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_pParam_B4SOInpeak/ni));
d_phieot_dX = ((2.0*Vtm0eot)*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_pParam_B4SOInpeak/ni),((ni*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-instanceStruct.instanceVar_pParam_B4SOInpeak*d_ni_dX)/ni/ni))+2.0*d_Vtm0eot_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/ni)));
phieot = ((2.0*Vtm0eot)*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/ni)));
}
{
double value_sqrt_0 = sqrt(phieot);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_sqrtphieot_dX = deriv_sqrt_0_d0*(d_phieot_dX);
sqrtphieot = value_sqrt_0;
}
d_tmp2_dX = (instanceStruct.d_instanceVar_here_B4SOIvfb_dX+d_phieot_dX);
tmp2 = (instanceStruct.instanceVar_here_B4SOIvfb+phieot);
d_vddeot_dX = instanceStruct.instanceVar_B4SOItype*d_B4SOIvddeot_dX;
vddeot = (instanceStruct.instanceVar_B4SOItype*B4SOIvddeot);
d_T0_dX = instanceStruct.d_instanceVar_B4SOIepsrgate_dX*8.85418e-12;
T0 = (instanceStruct.instanceVar_B4SOIepsrgate*8.85418e-12);
if (((((instanceStruct.instanceVar_pParam_B4SOIngate>1.0e18)&&(instanceStruct.instanceVar_pParam_B4SOIngate<1.0e25))&&(vddeot>tmp2))&&(T0!=0)))
{
d_T1_dX = (((instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox)*(((1.0e6*1.60219e-19)*instanceStruct.instanceVar_epssub)*instanceStruct.d_instanceVar_pParam_B4SOIngate_dX+(1.0e6*1.60219e-19)*instanceStruct.d_instanceVar_epssub_dX*instanceStruct.instanceVar_pParam_B4SOIngate)-(((1.0e6*1.60219e-19)*instanceStruct.instanceVar_epssub)*instanceStruct.instanceVar_pParam_B4SOIngate)*(instanceStruct.instanceVar_B4SOIcox*instanceStruct.d_instanceVar_B4SOIcox_dX+instanceStruct.d_instanceVar_B4SOIcox_dX*instanceStruct.instanceVar_B4SOIcox))/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox)/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox));
T1 = ((((1.0e6*1.60219e-19)*instanceStruct.instanceVar_epssub)*instanceStruct.instanceVar_pParam_B4SOIngate)/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox));
{
double value_sqrt_0 = sqrt((1.0+((2.0*(vddeot-T0))/T1)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T4_dX = deriv_sqrt_0_d0*(((T1*2.0*(d_vddeot_dX-d_T0_dX)-(2.0*(vddeot-T0))*d_T1_dX)/T1/T1));
T4 = value_sqrt_0;
}
d_T2_dX = (T1*d_T4_dX+d_T1_dX*(T4-1.0));
T2 = (T1*(T4-1.0));
d_T3_dX = ((T1*((0.5*T2)*d_T2_dX+0.5*d_T2_dX*T2)-((0.5*T2)*T2)*d_T1_dX)/T1/T1);
T3 = (((0.5*T2)*T2)/T1);
d_T7_dX = (instanceStruct.d_instanceVar_eggdep_dX-d_T3_dX);
T7 = ((instanceStruct.instanceVar_eggdep-T3)-0.05);
{
double value_sqrt_0 = sqrt(((T7*T7)+0.224));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T6_dX = deriv_sqrt_0_d0*((T7*d_T7_dX+d_T7_dX*T7));
T6 = value_sqrt_0;
}
d_T5_dX = (instanceStruct.d_instanceVar_eggdep_dX-0.5*(d_T7_dX+d_T6_dX));
T5 = (instanceStruct.instanceVar_eggdep-(0.5*(T7+T6)));
d_Vgs_eff_dX = (d_vddeot_dX-d_T5_dX);
Vgs_eff = (vddeot-T5);
}
else
{
d_Vgs_eff_dX = d_vddeot_dX;
Vgs_eff = vddeot;
}
d_V0_dX = (d_vbieot_dX-d_phieot_dX);
V0 = (vbieot-phieot);
d_T3_dX = d_pParam_B4SOIsqrtXdep0_dX;
T3 = pParam_B4SOIsqrtXdep0;
d_lt1_dX = (instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3);
lt1 = (instanceStruct.instanceVar_B4SOIfactor1*T3);
d_ltw_dX = (instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3);
ltw = (instanceStruct.instanceVar_B4SOIfactor1*T3);
d_T0_dX = ((lt1*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*instanceStruct.d_instanceVar_B4SOIleffeot_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX*instanceStruct.instanceVar_B4SOIleffeot)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*instanceStruct.instanceVar_B4SOIleffeot)*d_lt1_dX)/lt1/lt1);
T0 = ((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*instanceStruct.instanceVar_B4SOIleffeot)/lt1);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_Theta0_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_Theta0_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
d_T2_dX = ((instanceStruct.instanceVar_pParam_B4SOIXdep0*(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.d_instanceVar_epssub_dX+instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX*instanceStruct.instanceVar_epssub)-(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)*instanceStruct.d_instanceVar_pParam_B4SOIXdep0_dX)/instanceStruct.instanceVar_pParam_B4SOIXdep0/instanceStruct.instanceVar_pParam_B4SOIXdep0);
T2 = ((instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)/instanceStruct.instanceVar_pParam_B4SOIXdep0);
d_T3_dX = instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX;
T3 = instanceStruct.instanceVar_pParam_B4SOIcdsc;
d_T4_dX = ((instanceStruct.instanceVar_B4SOIcox*((d_T2_dX+(T3*d_Theta0_dX+d_T3_dX*Theta0))+instanceStruct.d_instanceVar_pParam_B4SOIcit_dX)-((T2+(T3*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
T4 = (((T2+(T3*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)/instanceStruct.instanceVar_B4SOIcox);
if ((T4>=(-0.5)))
{
d_n_dX = d_T4_dX;
n = (1.0+T4);
}
else
{
d_T0_dX = (-8.0*d_T4_dX/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
T0 = (1.0/(3.0+(8.0*T4)));
d_n_dX = ((1.0+(3.0*T4))*d_T0_dX+3.0*d_T4_dX*T0);
n = ((1.0+(3.0*T4))*T0);
}
if ((instanceStruct.instanceVar_pParam_B4SOIdvtp0>0.0))
{
d_T3_dX = (instanceStruct.d_instanceVar_B4SOIleffeot_dX+2.0*instanceStruct.d_instanceVar_pParam_B4SOIdvtp0_dX);
T3 = (instanceStruct.instanceVar_B4SOIleffeot+(2.0*instanceStruct.instanceVar_pParam_B4SOIdvtp0));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_B4SOIleffeot/T3));
d_T4_dX = (Vtmeot*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_B4SOIleffeot/T3),((T3*instanceStruct.d_instanceVar_B4SOIleffeot_dX-instanceStruct.instanceVar_B4SOIleffeot*d_T3_dX)/T3/T3))+d_Vtmeot_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_B4SOIleffeot/T3)));
T4 = (Vtmeot*evaluator_lln_0.getValues((instanceStruct.instanceVar_B4SOIleffeot/T3)));
}
d_DITS_Sft_dX = (n*d_T4_dX+d_n_dX*T4);
DITS_Sft = (n*T4);
}
else
{
d_DITS_Sft_dX = 0.0;
DITS_Sft = 0.0;
}
instanceStruct.d_instanceVar_B4SOIthetavth_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0*d_Theta0_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX*Theta0);
instanceStruct.instanceVar_B4SOIthetavth = (instanceStruct.instanceVar_pParam_B4SOIdvt0*Theta0);
d_Delt_vth_dX = (instanceStruct.instanceVar_B4SOIthetavth*d_V0_dX+instanceStruct.d_instanceVar_B4SOIthetavth_dX*V0);
Delt_vth = (instanceStruct.instanceVar_B4SOIthetavth*V0);
d_T0_dX = ((ltw*((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_B4SOIweffeot)*instanceStruct.d_instanceVar_B4SOIleffeot_dX+(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.d_instanceVar_B4SOIweffeot_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX*instanceStruct.instanceVar_B4SOIweffeot)*instanceStruct.instanceVar_B4SOIleffeot)-((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_B4SOIweffeot)*instanceStruct.instanceVar_B4SOIleffeot)*d_ltw_dX)/ltw/ltw);
T0 = (((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_B4SOIweffeot)*instanceStruct.instanceVar_B4SOIleffeot)/ltw);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX*T2);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*T2);
d_DeltVthw_dX = (T0*d_V0_dX+d_T0_dX*V0);
DeltVthw = (T0*V0);
d_TempRatioMinus1_dX = ((instanceStruct.instanceVar_B4SOItnom*d_B4SOItempeot_dX-B4SOItempeot*instanceStruct.d_instanceVar_B4SOItnom_dX)/instanceStruct.instanceVar_B4SOItnom/instanceStruct.instanceVar_B4SOItnom);
TempRatioMinus1 = ((B4SOItempeot/instanceStruct.instanceVar_B4SOItnom)-1.0);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/instanceStruct.instanceVar_B4SOIleffeot)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((instanceStruct.instanceVar_B4SOIleffeot*instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX-instanceStruct.instanceVar_pParam_B4SOIlpe0*instanceStruct.d_instanceVar_B4SOIleffeot_dX)/instanceStruct.instanceVar_B4SOIleffeot/instanceStruct.instanceVar_B4SOIleffeot));
T0 = value_sqrt_0;
}
d_T1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX+((instanceStruct.instanceVar_B4SOIleffeot*instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX-instanceStruct.instanceVar_pParam_B4SOIkt1l*instanceStruct.d_instanceVar_B4SOIleffeot_dX)/instanceStruct.instanceVar_B4SOIleffeot/instanceStruct.instanceVar_B4SOIleffeot));
T1 = (instanceStruct.instanceVar_pParam_B4SOIkt1+(instanceStruct.instanceVar_pParam_B4SOIkt1l/instanceStruct.instanceVar_B4SOIleffeot));
d_DeltVthtemp_dX = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*d_sqrtphieot_dX+(instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*(T0-1.0))*sqrtphieot)+(T1*d_TempRatioMinus1_dX+d_T1_dX*TempRatioMinus1));
DeltVthtemp = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*sqrtphieot)+(T1*TempRatioMinus1));
d_tmp2_dX = (((instanceStruct.instanceVar_B4SOIweffeot+instanceStruct.instanceVar_pParam_B4SOIw0)*(instanceStruct.instanceVar_toxe*d_phieot_dX+instanceStruct.d_instanceVar_toxe_dX*phieot)-(instanceStruct.instanceVar_toxe*phieot)*(instanceStruct.d_instanceVar_B4SOIweffeot_dX+instanceStruct.d_instanceVar_pParam_B4SOIw0_dX))/(instanceStruct.instanceVar_B4SOIweffeot+instanceStruct.instanceVar_pParam_B4SOIw0)/(instanceStruct.instanceVar_B4SOIweffeot+instanceStruct.instanceVar_pParam_B4SOIw0));
tmp2 = ((instanceStruct.instanceVar_toxe*phieot)/(instanceStruct.instanceVar_B4SOIweffeot+instanceStruct.instanceVar_pParam_B4SOIw0));
DIBL_Sft = 0.0;
DITS_Sft2 = 0.0;
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpeb/instanceStruct.instanceVar_B4SOIleffeot)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_Lpe_Vb_dX = deriv_sqrt_0_d0*(((instanceStruct.instanceVar_B4SOIleffeot*instanceStruct.d_instanceVar_pParam_B4SOIlpeb_dX-instanceStruct.instanceVar_pParam_B4SOIlpeb*instanceStruct.d_instanceVar_B4SOIleffeot_dX)/instanceStruct.instanceVar_B4SOIleffeot/instanceStruct.instanceVar_B4SOIleffeot));
Lpe_Vb = value_sqrt_0;
}
d_sqrtPhisExt_dX = d_sqrtphieot_dX;
sqrtPhisExt = sqrtphieot;
d_Vth_dX = ((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhisExt)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtphieot))*d_Lpe_Vb_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_sqrtPhisExt_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*sqrtPhisExt)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtphieot_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtphieot))*Lpe_Vb))-d_Delt_vth_dX)-d_DeltVthw_dX)+(instanceStruct.instanceVar_pParam_B4SOIk3*d_tmp2_dX+instanceStruct.d_instanceVar_pParam_B4SOIk3_dX*tmp2))+d_DeltVthtemp_dX)-d_DITS_Sft_dX);
Vth = (((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhisExt)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtphieot))*Lpe_Vb))-Delt_vth)-DeltVthw)+(instanceStruct.instanceVar_pParam_B4SOIk3*tmp2))+DeltVthtemp)-DIBL_Sft)-DITS_Sft)-DITS_Sft2);
d_Vgst_dX = (d_Vgs_eff_dX-d_Vth_dX);
Vgst = (Vgs_eff-Vth);
d_T10_dX = (n*d_Vtmeot_dX+d_n_dX*Vtmeot);
T10 = (n*Vtmeot);
d_VgstNVt_dX = ((T10*(instanceStruct.instanceVar_pParam_B4SOImstar*d_Vgst_dX+instanceStruct.d_instanceVar_pParam_B4SOImstar_dX*Vgst)-(instanceStruct.instanceVar_pParam_B4SOImstar*Vgst)*d_T10_dX)/T10/T10);
VgstNVt = ((instanceStruct.instanceVar_pParam_B4SOImstar*Vgst)/T10);
d_ExpArg_dX = ((T10*(instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX-((1-instanceStruct.instanceVar_pParam_B4SOImstar)*d_Vgst_dX+(-instanceStruct.d_instanceVar_pParam_B4SOImstar_dX)*Vgst))-(instanceStruct.instanceVar_pParam_B4SOIvoff-((1-instanceStruct.instanceVar_pParam_B4SOImstar)*Vgst))*d_T10_dX)/T10/T10);
ExpArg = ((instanceStruct.instanceVar_pParam_B4SOIvoff-((1-instanceStruct.instanceVar_pParam_B4SOImstar)*Vgst))/T10);
if ((VgstNVt>100.0))
{
d_Vgsteff_dX = d_Vgst_dX;
Vgsteff = Vgst;
}
else
{
if ((ExpArg>100.0))
{
d_T0_dX = (((n*Vtmeot)*(d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX)-(Vgst-instanceStruct.instanceVar_pParam_B4SOIvoff)*(n*d_Vtmeot_dX+d_n_dX*Vtmeot))/(n*Vtmeot)/(n*Vtmeot));
T0 = ((Vgst-instanceStruct.instanceVar_pParam_B4SOIvoff)/(n*Vtmeot));
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = deriv_exp_0_d0*(d_T0_dX);
ExpVgst = value_exp_0;
}
d_Vgsteff_dX = (((Vtmeot*instanceStruct.instanceVar_pParam_B4SOIcdep0)/instanceStruct.instanceVar_B4SOIcox)*d_ExpVgst_dX+((instanceStruct.instanceVar_B4SOIcox*(Vtmeot*instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX+d_Vtmeot_dX*instanceStruct.instanceVar_pParam_B4SOIcdep0)-(Vtmeot*instanceStruct.instanceVar_pParam_B4SOIcdep0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox)*ExpVgst);
Vgsteff = (((Vtmeot*instanceStruct.instanceVar_pParam_B4SOIcdep0)/instanceStruct.instanceVar_B4SOIcox)*ExpVgst);
}
else
{
{
double value_exp_0 = exp(VgstNVt);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = deriv_exp_0_d0*(d_VgstNVt_dX);
ExpVgst = value_exp_0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));
d_T1_dX = (T10*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dX)+d_T10_dX*evaluator_lln_0.getValues((1.0+ExpVgst)));
T1 = (T10*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
{
double value_exp_0 = exp(ExpArg);
double  deriv_exp_0_d0 = value_exp_0;
d_T4_dX = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm0eot*instanceStruct.instanceVar_pParam_B4SOIcdep0))*value_exp_0)*(-instanceStruct.d_instanceVar_pParam_B4SOImstar_dX)+(((-instanceStruct.instanceVar_B4SOIcox)/(Vtm0eot*instanceStruct.instanceVar_pParam_B4SOIcdep0))*deriv_exp_0_d0*(d_ExpArg_dX)+(((Vtm0eot*instanceStruct.instanceVar_pParam_B4SOIcdep0)*(-instanceStruct.d_instanceVar_B4SOIcox_dX)-(-instanceStruct.instanceVar_B4SOIcox)*(Vtm0eot*instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX+d_Vtm0eot_dX*instanceStruct.instanceVar_pParam_B4SOIcdep0))/(Vtm0eot*instanceStruct.instanceVar_pParam_B4SOIcdep0)/(Vtm0eot*instanceStruct.instanceVar_pParam_B4SOIcdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstar));
T4 = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm0eot*instanceStruct.instanceVar_pParam_B4SOIcdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstar));
}
d_T2_dX = (instanceStruct.d_instanceVar_pParam_B4SOImstar_dX-(((1.0-instanceStruct.instanceVar_pParam_B4SOImstar)*(T10*d_T4_dX+d_T10_dX*T4)-(T10*T4)*(-instanceStruct.d_instanceVar_pParam_B4SOImstar_dX))/(1.0-instanceStruct.instanceVar_pParam_B4SOImstar)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstar)));
T2 = (instanceStruct.instanceVar_pParam_B4SOImstar-((T10*T4)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstar)));
d_Vgsteff_dX = ((T2*d_T1_dX-T1*d_T2_dX)/T2/T2);
Vgsteff = (T1/T2);
}
}
d_T3_dX = ((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX-instanceStruct.d_instanceVar_here_B4SOIvfb_dX)-d_phieot_dX);
T3 = (((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)-instanceStruct.instanceVar_here_B4SOIvfb)-phieot);
d_vtfbphi2eot_dX = 4.0*d_T3_dX;
vtfbphi2eot = (4.0*T3);
if ((vtfbphi2eot<0.0))
{
d_vtfbphi2eot_dX = 0.0;
vtfbphi2eot = 0.0;
}
niter = 0;
d_toxpf_dX = instanceStruct.d_instanceVar_toxe_dX;
toxpf = instanceStruct.instanceVar_toxe;
d_toxpi_dX = 0.0;
toxpi = 1e6;
while (((niter<=4)&&(fabs((toxpf-toxpi))>1e-12)))
//Begin block whileloop
{
//Block-local variables for block whileloop
double T0=0.0;
double d_T0_dX=0.0;
double T1=0.0;
double d_T1_dX=0.0;
//End of Block-local variables
d_toxpi_dX = d_toxpf_dX;
toxpi = toxpf;
d_tmp2_dX = 2.0e8*d_toxpf_dX;
tmp2 = (2.0e8*toxpf);
d_T0_dX = ((tmp2*(d_Vgsteff_dX+d_vtfbphi2eot_dX)-(Vgsteff+vtfbphi2eot)*d_tmp2_dX)/tmp2/tmp2);
T0 = ((Vgsteff+vtfbphi2eot)/tmp2);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp(((instanceStruct.instanceVar_B4SOIbdos*0.7)*evaluator_lln_0.getValues(T0)));
double  deriv_exp_1_d0 = value_exp_1;
d_T1_dX = deriv_exp_1_d0*(((instanceStruct.instanceVar_B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dX)+instanceStruct.d_instanceVar_B4SOIbdos_dX*0.7*evaluator_lln_0.getValues(T0)));
T1 = (1.0+value_exp_1);
}
d_Tcen_dX = ((T1*instanceStruct.d_instanceVar_B4SOIados_dX*1.9e-9-(instanceStruct.instanceVar_B4SOIados*1.9e-9)*d_T1_dX)/T1/T1);
Tcen = ((instanceStruct.instanceVar_B4SOIados*1.9e-9)/T1);
d_toxpf_dX = (instanceStruct.d_instanceVar_toxe_dX-((instanceStruct.instanceVar_epsrox/instanceStruct.instanceVar_B4SOIepsrsub)*d_Tcen_dX+((instanceStruct.instanceVar_B4SOIepsrsub*instanceStruct.d_instanceVar_epsrox_dX-instanceStruct.instanceVar_epsrox*instanceStruct.d_instanceVar_B4SOIepsrsub_dX)/instanceStruct.instanceVar_B4SOIepsrsub/instanceStruct.instanceVar_B4SOIepsrsub)*Tcen));
toxpf = (instanceStruct.instanceVar_toxe-((instanceStruct.instanceVar_epsrox/instanceStruct.instanceVar_B4SOIepsrsub)*Tcen));
niter = (niter+1);
}
// End block whileloop
instanceStruct.d_instanceVar_B4SOItoxp_dX = d_toxpf_dX;
instanceStruct.instanceVar_B4SOItoxp = toxpf;
}
d_tmp_dX = d_pParam_B4SOIsqrtXdep0_dX;
tmp = pParam_B4SOIsqrtXdep0;
d_tmp1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvbi_dX-instanceStruct.d_instanceVar_pParam_B4SOIphi_dX);
tmp1 = (instanceStruct.instanceVar_pParam_B4SOIvbi-instanceStruct.instanceVar_pParam_B4SOIphi);
d_tmp2_dX = (instanceStruct.instanceVar_B4SOIfactor1*d_tmp_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*tmp);
tmp2 = (instanceStruct.instanceVar_B4SOIfactor1*tmp);
d_T0_dX = ((tmp2*((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.instanceVar_pParam_B4SOIleff)-((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_tmp2_dX)/tmp2/tmp2);
T0 = (((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.instanceVar_pParam_B4SOIleff)/tmp2);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX*T2);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*T2);
d_T2_dX = (T0*d_tmp1_dX+d_T0_dX*tmp1);
T2 = (T0*tmp1);
d_T0_dX = ((tmp2*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX*instanceStruct.instanceVar_pParam_B4SOIleff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_tmp2_dX)/tmp2/tmp2);
T0 = ((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*instanceStruct.instanceVar_pParam_B4SOIleff)/tmp2);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T3_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T3 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_T3_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T3 = (T1*(1.0+(2.0*T1)));
}
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIdvt0*T3)*d_tmp1_dX+(instanceStruct.instanceVar_pParam_B4SOIdvt0*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX*T3)*tmp1);
T3 = ((instanceStruct.instanceVar_pParam_B4SOIdvt0*T3)*tmp1);
d_T4_dX = (((instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)*(instanceStruct.instanceVar_B4SOItoxp*instanceStruct.d_instanceVar_pParam_B4SOIphi_dX+instanceStruct.d_instanceVar_B4SOItoxp_dX*instanceStruct.instanceVar_pParam_B4SOIphi)-(instanceStruct.instanceVar_B4SOItoxp*instanceStruct.instanceVar_pParam_B4SOIphi)*(instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIw0_dX))/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
T4 = ((instanceStruct.instanceVar_B4SOItoxp*instanceStruct.instanceVar_pParam_B4SOIphi)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/instanceStruct.instanceVar_pParam_B4SOIleff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX-instanceStruct.instanceVar_pParam_B4SOIlpe0*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX)/instanceStruct.instanceVar_pParam_B4SOIleff/instanceStruct.instanceVar_pParam_B4SOIleff));
T0 = value_sqrt_0;
}
d_T5_dX = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX+(instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*(T0-1.0))*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi)+((instanceStruct.instanceVar_pParam_B4SOIkt1+(instanceStruct.instanceVar_pParam_B4SOIkt1l/instanceStruct.instanceVar_pParam_B4SOIleff))*d_TempRatio_dX+(instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX+((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX-instanceStruct.instanceVar_pParam_B4SOIkt1l*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX)/instanceStruct.instanceVar_pParam_B4SOIleff/instanceStruct.instanceVar_pParam_B4SOIleff))*(TempRatio-1.0)));
T5 = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi)+((instanceStruct.instanceVar_pParam_B4SOIkt1+(instanceStruct.instanceVar_pParam_B4SOIkt1l/instanceStruct.instanceVar_pParam_B4SOIleff))*(TempRatio-1.0)));
d_tmp3_dX = ((((instanceStruct.instanceVar_B4SOItype*d_pParam_B4SOIvth0_dX-d_T2_dX)-d_T3_dX)+(instanceStruct.instanceVar_pParam_B4SOIk3*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIk3_dX*T4))+d_T5_dX);
tmp3 = (((((instanceStruct.instanceVar_B4SOItype*pParam_B4SOIvth0)-T2)-T3)+(instanceStruct.instanceVar_pParam_B4SOIk3*T4))+T5);
d_pParam_B4SOIvfbzb_dX = ((d_tmp3_dX-instanceStruct.d_instanceVar_pParam_B4SOIphi_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1*instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1_dX*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi));
pParam_B4SOIvfbzb = ((tmp3-instanceStruct.instanceVar_pParam_B4SOIphi)-(instanceStruct.instanceVar_pParam_B4SOIk1*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi));
instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX = ((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*(1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/instanceStruct.instanceVar_pParam_B4SOIleff)))*1e6)*instanceStruct.d_instanceVar_B4SOItsi_dX+((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX-instanceStruct.instanceVar_pParam_B4SOIlpe0*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX)/instanceStruct.instanceVar_pParam_B4SOIleff/instanceStruct.instanceVar_pParam_B4SOIleff)+1.60219e-19*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX*(1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/instanceStruct.instanceVar_pParam_B4SOIleff)))*1e6*instanceStruct.instanceVar_B4SOItsi);
instanceStruct.instanceVar_pParam_B4SOIqsi = ((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*(1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/instanceStruct.instanceVar_pParam_B4SOIleff)))*1e6)*instanceStruct.instanceVar_B4SOItsi);
instanceStruct.d_instanceVar_B4SOIgrgeltd_dX = ((((B4SOIngcon*instanceStruct.instanceVar_B4SOInf)*(B4SOIl-B4SOIxgl))*(B4SOIrshg*(d_B4SOIxgw_dX+((B4SOIngcon*(((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-instanceStruct.instanceVar_pParam_B4SOIweff*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)/3.0)-((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)/3.0)*d_B4SOIngcon_dX)/B4SOIngcon/B4SOIngcon))+d_B4SOIrshg_dX*(B4SOIxgw+(((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)/3.0)/B4SOIngcon)))-(B4SOIrshg*(B4SOIxgw+(((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)/3.0)/B4SOIngcon)))*((B4SOIngcon*instanceStruct.instanceVar_B4SOInf)*(d_B4SOIl_dX-d_B4SOIxgl_dX)+(B4SOIngcon*instanceStruct.d_instanceVar_B4SOInf_dX+d_B4SOIngcon_dX*instanceStruct.instanceVar_B4SOInf)*(B4SOIl-B4SOIxgl)))/((B4SOIngcon*instanceStruct.instanceVar_B4SOInf)*(B4SOIl-B4SOIxgl))/((B4SOIngcon*instanceStruct.instanceVar_B4SOInf)*(B4SOIl-B4SOIxgl)));
instanceStruct.instanceVar_B4SOIgrgeltd = ((B4SOIrshg*(B4SOIxgw+(((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)/3.0)/B4SOIngcon)))/((B4SOIngcon*instanceStruct.instanceVar_B4SOInf)*(B4SOIl-B4SOIxgl)));
if ((instanceStruct.instanceVar_B4SOIgrgeltd>0.0))
{
instanceStruct.d_instanceVar_B4SOIgrgeltd_dX = (-instanceStruct.d_instanceVar_B4SOIgrgeltd_dX/instanceStruct.instanceVar_B4SOIgrgeltd/instanceStruct.instanceVar_B4SOIgrgeltd);
instanceStruct.instanceVar_B4SOIgrgeltd = (1.0/instanceStruct.instanceVar_B4SOIgrgeltd);
}
else
{
instanceStruct.d_instanceVar_B4SOIgrgeltd_dX = 0.0;
instanceStruct.instanceVar_B4SOIgrgeltd = 1.0e3;
if ((instanceStruct.instanceVar_B4SOIrgateMod!=0))
{
UserInfo(theInstance)  << "Warning: The gate conductance reset to 1.0e3 mho." << " " <<  std::endl; 
}
}
if (instanceStruct.instanceVar_B4SOIrbodyMod)
{
if ((B4SOIrbdb<1.0e-3))
{
instanceStruct.d_instanceVar_B4SOIgrbdb_dX = 0.0;
instanceStruct.instanceVar_B4SOIgrbdb = 1.0e3;
}
else
{
instanceStruct.d_instanceVar_B4SOIgrbdb_dX = (d_B4SOIgbmin_dX+(-d_B4SOIrbdb_dX/B4SOIrbdb/B4SOIrbdb));
instanceStruct.instanceVar_B4SOIgrbdb = (B4SOIgbmin+(1.0/B4SOIrbdb));
}
if ((B4SOIrbsb<1.0e-3))
{
instanceStruct.d_instanceVar_B4SOIgrbsb_dX = 0.0;
instanceStruct.instanceVar_B4SOIgrbsb = 1.0e3;
}
else
{
instanceStruct.d_instanceVar_B4SOIgrbsb_dX = (d_B4SOIgbmin_dX+(-d_B4SOIrbsb_dX/B4SOIrbsb/B4SOIrbsb));
instanceStruct.instanceVar_B4SOIgrbsb = (B4SOIgbmin+(1.0/B4SOIrbsb));
}
}
else
{
instanceStruct.d_instanceVar_B4SOIgrbdb_dX = 0.0;
instanceStruct.instanceVar_B4SOIgrbdb = 0;
instanceStruct.d_instanceVar_B4SOIgrbsb_dX = 0.0;
instanceStruct.instanceVar_B4SOIgrbsb = 0;
}
instanceStruct.d_instanceVar_B4SOIvfbzb_dX = (d_pParam_B4SOIvfbzb_dX+instanceStruct.instanceVar_B4SOItype*d_B4SOIdelvto_dX);
instanceStruct.instanceVar_B4SOIvfbzb = (pParam_B4SOIvfbzb+(instanceStruct.instanceVar_B4SOItype*B4SOIdelvto));
{
double value_sqrt_0 = sqrt(((instanceStruct.instanceVar_epssub*Vtm0)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX = (deriv_sqrt_0_d0*(((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)*(instanceStruct.instanceVar_epssub*d_Vtm0_dX+instanceStruct.d_instanceVar_epssub_dX*Vtm0)-(instanceStruct.instanceVar_epssub*Vtm0)*1.60219e-19*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX*1.0e6)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)))/3.0);
instanceStruct.instanceVar_pParam_B4SOIldeb = (value_sqrt_0/3.0);
}
d_T1_dX = ((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX-instanceStruct.d_instanceVar_here_B4SOIvfb_dX)-instanceStruct.d_instanceVar_pParam_B4SOIphi_dX);
T1 = (((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)-instanceStruct.instanceVar_here_B4SOIvfb)-instanceStruct.instanceVar_pParam_B4SOIphi);
d_T2_dX = (d_T1_dX+d_T1_dX);
T2 = (T1+T1);
d_T3_dX = 2.5*d_T1_dX;
T3 = (2.5*T1);
instanceStruct.d_instanceVar_B4SOIvtfbphi1_dX = ((instanceStruct.instanceVar_B4SOItype==1)?d_T2_dX:d_T3_dX);
instanceStruct.instanceVar_B4SOIvtfbphi1 = ((instanceStruct.instanceVar_B4SOItype==1)?T2:T3);
if ((instanceStruct.instanceVar_B4SOIvtfbphi1<0.0))
{
instanceStruct.d_instanceVar_B4SOIvtfbphi1_dX = 0.0;
instanceStruct.instanceVar_B4SOIvtfbphi1 = 0.0;
}
if ((instanceStruct.instanceVar_B4SOImobMod==4))
{
d_lt1_dX = (instanceStruct.instanceVar_B4SOIfactor1*d_pParam_B4SOIsqrtXdep0_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*pParam_B4SOIsqrtXdep0);
lt1 = (instanceStruct.instanceVar_B4SOIfactor1*pParam_B4SOIsqrtXdep0);
d_T0_dX = ((lt1*(instanceStruct.instanceVar_pParam_B4SOIdvt1*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX*instanceStruct.instanceVar_pParam_B4SOIleff)-(instanceStruct.instanceVar_pParam_B4SOIdvt1*instanceStruct.instanceVar_pParam_B4SOIleff)*d_lt1_dX)/lt1/lt1);
T0 = ((instanceStruct.instanceVar_pParam_B4SOIdvt1*instanceStruct.instanceVar_pParam_B4SOIleff)/lt1);
if ((T0<100.0))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T2_dX = d_T1_dX;
T2 = (T1-1.0);
d_T3_dX = (T2*d_T2_dX+d_T2_dX*T2);
T3 = (T2*T2);
d_T4_dX = (d_T3_dX+2.0*d_T1_dX*3.720075976e-44);
T4 = (T3+((2.0*T1)*3.720075976e-44));
d_Theta0_dX = ((T4*d_T1_dX-T1*d_T4_dX)/T4/T4);
Theta0 = (T1/T4);
}
else
{
d_Theta0_dX = 0.0;
Theta0 = (1.0/(2.688117142e+43-2.0));
}
d_tmp1_dX = ((instanceStruct.instanceVar_pParam_B4SOIXdep0*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*instanceStruct.d_instanceVar_pParam_B4SOIXdep0_dX)/instanceStruct.instanceVar_pParam_B4SOIXdep0/instanceStruct.instanceVar_pParam_B4SOIXdep0);
tmp1 = (instanceStruct.instanceVar_epssub/instanceStruct.instanceVar_pParam_B4SOIXdep0);
d_tmp2_dX = (instanceStruct.instanceVar_pParam_B4SOInfactor*d_tmp1_dX+instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX*tmp1);
tmp2 = (instanceStruct.instanceVar_pParam_B4SOInfactor*tmp1);
d_tmp3_dX = ((instanceStruct.instanceVar_B4SOIcox*((d_tmp2_dX+(instanceStruct.instanceVar_pParam_B4SOIcdsc*d_Theta0_dX+instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX*Theta0))+instanceStruct.d_instanceVar_pParam_B4SOIcit_dX)-((tmp2+(instanceStruct.instanceVar_pParam_B4SOIcdsc*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
tmp3 = (((tmp2+(instanceStruct.instanceVar_pParam_B4SOIcdsc*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)/instanceStruct.instanceVar_B4SOIcox);
if ((tmp3>=(-0.5)))
{
d_n0_dX = d_tmp3_dX;
n0 = (1.0+tmp3);
}
else
{
d_T0_dX = (-8.0*d_tmp3_dX/(3.0+(8.0*tmp3))/(3.0+(8.0*tmp3)));
T0 = (1.0/(3.0+(8.0*tmp3)));
d_n0_dX = ((1.0+(3.0*tmp3))*d_T0_dX+3.0*d_tmp3_dX*T0);
n0 = ((1.0+(3.0*tmp3))*T0);
}
d_T0_dX = (n0*d_Vtm0_dX+d_n0_dX*Vtm0);
T0 = (n0*Vtm0);
d_T1_dX = instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX;
T1 = instanceStruct.instanceVar_pParam_B4SOIvoff;
d_T2_dX = ((T0*d_T1_dX-T1*d_T0_dX)/T0/T0);
T2 = (T1/T0);
if ((T2<(-100.0)))
{
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIcdep0*instanceStruct.d_instanceVar_B4SOIcox_dX*3.720075976e-44-(instanceStruct.instanceVar_B4SOIcox*3.720075976e-44)*instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX)/instanceStruct.instanceVar_pParam_B4SOIcdep0/instanceStruct.instanceVar_pParam_B4SOIcdep0);
T3 = ((instanceStruct.instanceVar_B4SOIcox*3.720075976e-44)/instanceStruct.instanceVar_pParam_B4SOIcdep0);
d_T4_dX = (instanceStruct.d_instanceVar_pParam_B4SOImstar_dX+(T3*d_n0_dX+d_T3_dX*n0));
T4 = (instanceStruct.instanceVar_pParam_B4SOImstar+(T3*n0));
}
else
{
if ((T2>100.0))
{
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIcdep0*instanceStruct.d_instanceVar_B4SOIcox_dX*2.688117142e+43-(instanceStruct.instanceVar_B4SOIcox*2.688117142e+43)*instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX)/instanceStruct.instanceVar_pParam_B4SOIcdep0/instanceStruct.instanceVar_pParam_B4SOIcdep0);
T3 = ((instanceStruct.instanceVar_B4SOIcox*2.688117142e+43)/instanceStruct.instanceVar_pParam_B4SOIcdep0);
d_T4_dX = (instanceStruct.d_instanceVar_pParam_B4SOImstar_dX+(T3*d_n0_dX+d_T3_dX*n0));
T4 = (instanceStruct.instanceVar_pParam_B4SOImstar+(T3*n0));
}
else
{
{
double value_exp_0 = exp(T2);
double  deriv_exp_0_d0 = value_exp_0;
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIcdep0*(value_exp_0*instanceStruct.d_instanceVar_B4SOIcox_dX+deriv_exp_0_d0*(d_T2_dX)*instanceStruct.instanceVar_B4SOIcox)-(value_exp_0*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX)/instanceStruct.instanceVar_pParam_B4SOIcdep0/instanceStruct.instanceVar_pParam_B4SOIcdep0);
T3 = ((value_exp_0*instanceStruct.instanceVar_B4SOIcox)/instanceStruct.instanceVar_pParam_B4SOIcdep0);
}
d_T4_dX = (instanceStruct.d_instanceVar_pParam_B4SOImstar_dX+(T3*d_n0_dX+d_T3_dX*n0));
T4 = (instanceStruct.instanceVar_pParam_B4SOImstar+(T3*n0));
}
}
instanceStruct.d_instanceVar_B4SOIvgsteffvth_dX = ((T4*d_T0_dX*.69314718055994530941-(T0*.69314718055994530941)*d_T4_dX)/T4/T4);
instanceStruct.instanceVar_B4SOIvgsteffvth = ((T0*.69314718055994530941)/T4);
}
else
{
instanceStruct.d_instanceVar_B4SOIvgsteffvth_dX = 0.0;
instanceStruct.instanceVar_B4SOIvgsteffvth = 0;
}
}
// End block b4soi_temp
}
// End block initial_instance
}



//-----------------------------------------------------------------------------
// Function      : evaluateInitialModel
// Purpose       : Evaluate the statements in the initial_model block
// Special Notes : specific for sensitivity use
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void evaluateInitialModel(
modelSensStruct & modelStruct,
 double admsTemperature, double ADMSgmin_arg, const Instance & theInstance)
{
}



//-----------------------------------------------------------------------------
// Function      : evaluateModelEquations
// Purpose       : Evaluate the main module block.  Similar to
//                 updateIntermediateVars, but takes all instance and model
//                 parameters and variables as arguments instead of using
//                 the ones stored in the objects.
// Special Notes : specific for sensitivity use
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
void evaluateModelEquations(
std::vector <double> & probeVars,
// probe constants
const int admsProbeID_V_sb_GND,
const int admsProbeID_V_db_GND,
const int admsProbeID_V_gm_GND,
const int admsProbeID_V_b_sb,
const int admsProbeID_V_b_db,
const int admsProbeID_V_gm_gi,
const int admsProbeID_V_g_gm,
const int admsProbeID_V_gi_e,
const int admsProbeID_V_gm_e,
const int admsProbeID_V_s_si,
const int admsProbeID_V_d_di,
const int admsProbeID_V_si_GND,
const int admsProbeID_V_di_GND,
const int admsProbeID_V_gi_GND,
const int admsProbeID_V_gm_si,
const int admsProbeID_V_db_di,
const int admsProbeID_V_sb_si,
const int admsProbeID_V_gi_p,
const int admsProbeID_V_b_p,
const int admsProbeID_V_e_si,
const int admsProbeID_V_gi_si,
const int admsProbeID_V_b_si,
const int admsProbeID_V_di_si,
const int admsProbeID_Temp_t_GND,
const int admsProbeID_V_p_GND,
const int admsProbeID_V_b_GND,
// node constants
const int admsNodeID_d,
const int admsNodeID_g,
const int admsNodeID_s,
const int admsNodeID_e,
const int admsNodeID_p,
const int admsNodeID_b,
const int admsNodeID_t,
const int admsNodeID_di,
const int admsNodeID_si,
const int admsNodeID_gi,
const int admsNodeID_gm,
const int admsNodeID_sb,
const int admsNodeID_db,
instanceSensStruct & instanceStruct,
modelSensStruct & modelStruct,
const std::vector<bool> & portsConnected_,
// basic variables
 double admsTemperature, double adms_vt_nom, double ADMSgmin_arg, std::vector <double> & d_staticContributions_dX, std::vector <double> & d_dynamicContributions_dX, const Instance & theInstance)
{

  // Local variables
  double epsgate=0.0;
double d_epsgate_dX=0.0;
  double B4SOIig=0.0;
double d_B4SOIig_dX=0.0;
  double B4SOIigp=0.0;
double d_B4SOIigp_dX=0.0;
  double B4SOIgcrg=0.0;
double d_B4SOIgcrg_dX=0.0;
  double B4SOIqse=0.0;
double d_B4SOIqse_dX=0.0;
  double B4SOIqde=0.0;
double d_B4SOIqde_dX=0.0;
  double qgi=0.0;
double d_qgi_dX=0.0;
  double qov=0.0;
double d_qov_dX=0.0;
  double B4SOIigidl=0.0;
double d_B4SOIigidl_dX=0.0;
  double B4SOIigisl=0.0;
double d_B4SOIigisl_dX=0.0;
  double B4SOIIgcd=0.0;
double d_B4SOIIgcd_dX=0.0;
  double B4SOIIgcs=0.0;
double d_B4SOIIgcs_dX=0.0;
  double B4SOIqdrn=0.0;
double d_B4SOIqdrn_dX=0.0;
  double B4SOIqsrc=0.0;
double d_B4SOIqsrc_dX=0.0;
  double B4SOIIgd=0.0;
double d_B4SOIIgd_dX=0.0;
  double B4SOIIgs=0.0;
double d_B4SOIIgs_dX=0.0;
  double QGI=0.0;
double d_QGI_dX=0.0;
  double QOV=0.0;
double d_QOV_dX=0.0;
  double TempRatioMinus1=0.0;
double d_TempRatioMinus1_dX=0.0;
  double TempRatio=0.0;
double d_TempRatio_dX=0.0;
  double delTemp=0.0;


  // -- code converted from analog/code block
//Begin block b4soi_check
{
//Block-local variables for block b4soi_check
int Fatal_Flag=0.0;
//End of Block-local variables
Fatal_Flag = 0;
if ((instanceStruct.instanceVar_pParam_B4SOIlpe0<(-instanceStruct.instanceVar_pParam_B4SOIleff)))
{
UserInfo(theInstance)  << "Fatal: Lpe0 = %g is less than -Leff." << " " << instanceStruct.instanceVar_pParam_B4SOIlpe0 << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((((instanceStruct.instanceVar_B4SOIsa>0.0)&&(instanceStruct.instanceVar_B4SOIsb>0.0))&&((instanceStruct.instanceVar_B4SOInf==1.0)||((instanceStruct.instanceVar_B4SOInf>1.0)&&(instanceStruct.instanceVar_B4SOIsd>0.0)))))
{
if ((instanceStruct.instanceVar_B4SOIsaref<=0.0))
{
UserInfo(theInstance)  << "Fatal: SAref = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIsaref << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOIsbref<=0.0))
{
UserInfo(theInstance)  << "Fatal: SBref = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIsbref << " " <<  std::endl; 
Fatal_Flag = 1;
}
}
if ((instanceStruct.instanceVar_pParam_B4SOIlpeb<(-instanceStruct.instanceVar_pParam_B4SOIleff)))
{
UserInfo(theInstance)  << "Fatal: Lpeb = %g is less than -Leff." << " " << instanceStruct.instanceVar_pParam_B4SOIlpeb << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIfprout<0.0))
{
UserInfo(theInstance)  << "Fatal: fprout = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIfprout << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIpdits<0.0))
{
UserInfo(theInstance)  << "Fatal: pdits = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIpdits << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOIpditsl<0.0))
{
UserInfo(theInstance)  << "Fatal: pditsl = %g is negative." << " " << instanceStruct.instanceVar_B4SOIpditsl << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOItox<=0.0))
{
UserInfo(theInstance)  << "Fatal: Tox = %g is not positive." << " " << instanceStruct.instanceVar_B4SOItox << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOIleffeot<=0.0))
{
UserInfo(theInstance)  << "Fatal: Leffeot = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIleffeot << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOIweffeot<=0.0))
{
UserInfo(theInstance)  << "Fatal: Weffeot = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIweffeot << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOItoxp<=0.0))
{
UserInfo(theInstance)  << "Fatal: Toxp = %g is not positive." << " " << instanceStruct.instanceVar_B4SOItoxp << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOIepsrgate<0.0))
{
UserInfo(theInstance)  << "Fatal: Epsrgate = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIepsrgate << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOItoxm<=0.0))
{
UserInfo(theInstance)  << "Fatal: Toxm = %g is not positive." << " " << instanceStruct.instanceVar_B4SOItoxm << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOInf<1.0))
{
UserInfo(theInstance)  << "Fatal: Number of finger = %g is smaller than one." << " " << instanceStruct.instanceVar_B4SOInf << " " <<  std::endl; 
Fatal_Flag = 1;
}
if (((instanceStruct.instanceVar_B4SOItox-instanceStruct.instanceVar_B4SOIdtoxcv)<=0.0))
{
UserInfo(theInstance)  << "Fatal: Tox - dtoxcv = %g is not positive." << " " << (instanceStruct.instanceVar_B4SOItox-instanceStruct.instanceVar_B4SOIdtoxcv) << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOItbox<=0.0))
{
UserInfo(theInstance)  << "Fatal: Tbox = %g is not positive." << " " << instanceStruct.instanceVar_B4SOItbox << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOInpeak<=0.0))
{
UserInfo(theInstance)  << "Fatal: Nch = %g is not positive." << " " << instanceStruct.instanceVar_pParam_B4SOInpeak << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIngate<0.0))
{
UserInfo(theInstance)  << "Fatal: Ngate = %g Ngate is not positive." << " " << instanceStruct.instanceVar_pParam_B4SOIngate << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIngate>1.0e25))
{
UserInfo(theInstance)  << "Fatal: Ngate = %g Ngate is too high" << " " << instanceStruct.instanceVar_pParam_B4SOIngate << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIdvt1<0.0))
{
UserInfo(theInstance)  << "Fatal: Dvt1 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIdvt1 << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIdvt1w<0.0))
{
UserInfo(theInstance)  << "Fatal: Dvt1w = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIdvt1w << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIw0==(-instanceStruct.instanceVar_pParam_B4SOIweff)))
{
UserInfo(theInstance)  << "Fatal: (W0 + Weff) = 0 causing divided-by-zero." << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIdsub<0.0))
{
UserInfo(theInstance)  << "Fatal: Dsub = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIdsub << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIb1==(-instanceStruct.instanceVar_pParam_B4SOIweff)))
{
UserInfo(theInstance)  << "Fatal: (B1 + Weff) = 0 causing divided-by-zero." << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIu0temp<=0.0))
{
UserInfo(theInstance)  << "Fatal: u0 at current temperature = %g is not positive." << " " << instanceStruct.instanceVar_pParam_B4SOIu0temp << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIdelta<0.0))
{
UserInfo(theInstance)  << "Fatal: Delta = %g is less than zero." << " " << instanceStruct.instanceVar_pParam_B4SOIdelta << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIvsattemp<=0.0))
{
UserInfo(theInstance)  << "Fatal: Vsat at current temperature = %g is not positive." << " " << instanceStruct.instanceVar_pParam_B4SOIvsattemp << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIpclm<=0.0))
{
UserInfo(theInstance)  << "Fatal: Pclm = %g is not positive." << " " << instanceStruct.instanceVar_pParam_B4SOIpclm << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIdrout<0.0))
{
UserInfo(theInstance)  << "Fatal: Drout = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIdrout << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIclc<0.0))
{
UserInfo(theInstance)  << "Fatal: Clc = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIclc << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOInoff<0.1))
{
UserInfo(theInstance)  << "Warning: Noff = %g is too small." << " " << instanceStruct.instanceVar_pParam_B4SOInoff << " " <<  std::endl; 
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOInoff>4.0))
{
UserInfo(theInstance)  << "Warning: Noff = %g is too large." << " " << instanceStruct.instanceVar_pParam_B4SOInoff << " " <<  std::endl; 
}
}
if ((instanceStruct.instanceVar_pParam_B4SOInoff2<0.1))
{
UserInfo(theInstance)  << "Warning: Noff2 = %g is too small." << " " << instanceStruct.instanceVar_pParam_B4SOInoff2 << " " <<  std::endl; 
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOInoff2>4.0))
{
UserInfo(theInstance)  << "Warning: Noff2 = %g is too large." << " " << instanceStruct.instanceVar_pParam_B4SOInoff2 << " " <<  std::endl; 
}
}
if ((((instanceStruct.instanceVar_B4SOIsa>0.0)&&(instanceStruct.instanceVar_B4SOIsb>0.0))&&((instanceStruct.instanceVar_B4SOInf==1.0)||((instanceStruct.instanceVar_B4SOInf>1.0)&&(instanceStruct.instanceVar_B4SOIsd>0.0)))))
{
if ((instanceStruct.instanceVar_B4SOIlodk2<=0.0))
{
UserInfo(theInstance)  << "Warning: LODK2 = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIlodk2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIlodeta0<=0.0))
{
UserInfo(theInstance)  << "Warning: LODETA0 = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIlodeta0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIlodeta0cv<=0.0))
{
UserInfo(theInstance)  << "Warning: LODETA0CV = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIlodeta0cv << " " <<  std::endl; 
}
}
if ((instanceStruct.instanceVar_pParam_B4SOImoin<5.0))
{
UserInfo(theInstance)  << "Warning: Moin = %g is too small." << " " << instanceStruct.instanceVar_pParam_B4SOImoin << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOImoin>25.0))
{
UserInfo(theInstance)  << "Warning: Moin = %g is too large." << " " << instanceStruct.instanceVar_pParam_B4SOImoin << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOImoinfd<5.0))
{
UserInfo(theInstance)  << "Warning: MoinFD = %g is too small." << " " << instanceStruct.instanceVar_pParam_B4SOImoinfd << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIcapMod==3))
{
if ((instanceStruct.instanceVar_pParam_B4SOIacde<0.1))
{
UserInfo(theInstance)  << "Warning: Acde = %g is too small." << " " << instanceStruct.instanceVar_pParam_B4SOIacde << " " <<  std::endl; 
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOIacde>1.6))
{
UserInfo(theInstance)  << "Warning: Acde = %g is too large." << " " << instanceStruct.instanceVar_pParam_B4SOIacde << " " <<  std::endl; 
}
}
}
if (((instanceStruct.instanceVar_B4SOIcsdmin>0)&&((instanceStruct.instanceVar_B4SOIasd<=0.0)||(instanceStruct.instanceVar_B4SOIasd>=1.0))))
{
UserInfo(theInstance)  << "Fatal: Asd = %g should be within (0, 1)." << " " << instanceStruct.instanceVar_B4SOIasd << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOInigc<=0.0))
{
UserInfo(theInstance)  << "Fatal: nigc = %g is non-positive." << " " << instanceStruct.instanceVar_pParam_B4SOInigc << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIpoxedge<=0.0))
{
UserInfo(theInstance)  << "Fatal: poxedge = %g is non-positive." << " " << instanceStruct.instanceVar_pParam_B4SOIpoxedge << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_pParam_B4SOIpigcd<=0.0))
{
UserInfo(theInstance)  << "Fatal: pigcd = %g is non-positive." << " " << instanceStruct.instanceVar_pParam_B4SOIpigcd << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOItoxref<0.0))
{
UserInfo(theInstance)  << "Warning:  Toxref = %g is negative." << " " << instanceStruct.instanceVar_B4SOItoxref << " " <<  std::endl; 
Fatal_Flag = 1;
}
if ((instanceStruct.instanceVar_B4SOItoxqm<=0.0))
{
UserInfo(theInstance)  << "Fatal: Toxqm = %g is not positive." << " " << instanceStruct.instanceVar_B4SOItoxqm << " " <<  std::endl; 
Fatal_Flag = 1;
}
if (((modelStruct.modelPar_VERSION>=4.4)||modelStruct.modelPar_PARAMCHK))
{
if ((instanceStruct.instanceVar_pParam_B4SOIa2<0.01))
{
UserInfo(theInstance)  << "Warning: A2 = %g is too small. Set to 0.01." << " " << instanceStruct.instanceVar_pParam_B4SOIa2 << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIa2_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIa2 = 0.01;
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOIa2>1.0))
{
UserInfo(theInstance)  << "Warning: A2 = %g is larger than 1. A2 is set to 1 and A1 is set to 0." << " " << instanceStruct.instanceVar_pParam_B4SOIa2 << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIa2_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIa2 = 1.0;
instanceStruct.d_instanceVar_pParam_B4SOIa1_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIa1 = 0.0;
}
}
}
if ((instanceStruct.instanceVar_pParam_B4SOIrdsw<0.0))
{
UserInfo(theInstance)  << "Warning: Rdsw = %g is negative. Set to zero." << " " << instanceStruct.instanceVar_pParam_B4SOIrdsw << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIrdsw_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrdsw = 0.0;
instanceStruct.d_instanceVar_pParam_B4SOIrds0_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrds0 = 0.0;
}
else
{
if (((instanceStruct.instanceVar_pParam_B4SOIrds0<0.001)&&(instanceStruct.instanceVar_pParam_B4SOIrds0!=0.0)))
{
UserInfo(theInstance)  << "Warning: Rds at current temperature = %g is less than 0.001 ohm. Set to zero." << " " << instanceStruct.instanceVar_pParam_B4SOIrds0 << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIrds0_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIrds0 = 0.0;
}
}
if (modelStruct.modelPar_PARAMCHK)
{
if ((instanceStruct.instanceVar_pParam_B4SOIleff<=5.0e-8))
{
UserInfo(theInstance)  << "Warning: Leff = %g may be too small." << " " << instanceStruct.instanceVar_pParam_B4SOIleff << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIleffCV<=5.0e-8))
{
UserInfo(theInstance)  << "Warning: Leff for CV = %g may be too small." << " " << instanceStruct.instanceVar_pParam_B4SOIleffCV << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIweff<=1.0e-7))
{
UserInfo(theInstance)  << "Warning: Weff = %g may be too small." << " " << instanceStruct.instanceVar_pParam_B4SOIweff << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIweffCV<=1.0e-7))
{
UserInfo(theInstance)  << "Warning: Weff for CV = %g may be too small." << " " << instanceStruct.instanceVar_pParam_B4SOIweffCV << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIlpe0<0.0))
{
UserInfo(theInstance)  << "Warning: Lpe0 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIlpe0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOItox<1.0e-9))
{
UserInfo(theInstance)  << "Warning: Tox = %g is less than 10A." << " " << instanceStruct.instanceVar_B4SOItox << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOInpeak<=1.0e15))
{
UserInfo(theInstance)  << "Warning: Nch = %g may be too small." << " " << instanceStruct.instanceVar_pParam_B4SOInpeak << " " <<  std::endl; 
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOInpeak>=1.0e21))
{
UserInfo(theInstance)  << "Warning: Nch = %g may be too large." << " " << instanceStruct.instanceVar_pParam_B4SOInpeak << " " <<  std::endl; 
}
}
if ((fabs(instanceStruct.instanceVar_pParam_B4SOInsub)>=1.0e21))
{
UserInfo(theInstance)  << "Warning: Nsub = %g may be too large." << " " << instanceStruct.instanceVar_pParam_B4SOInsub << " " <<  std::endl; 
}
if (((instanceStruct.instanceVar_pParam_B4SOIngate>0.0)&&(instanceStruct.instanceVar_pParam_B4SOIngate<=1.0e18)))
{
UserInfo(theInstance)  << "Warning: Ngate = %g is less than 1.E18cm^-3." << " " << instanceStruct.instanceVar_pParam_B4SOIngate << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIdvt0<0.0))
{
UserInfo(theInstance)  << "Warning: Dvt0 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIdvt0 << " " <<  std::endl; 
}
if ((fabs((1.0e-6/(instanceStruct.instanceVar_pParam_B4SOIw0+instanceStruct.instanceVar_pParam_B4SOIweff)))>10.0))
{
UserInfo(theInstance)  << "Warning: (W0 + Weff) may be too small." << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOInsd>1.0e23))
{
UserInfo(theInstance)  << "Warning: Nsd = %g is too large, should be specified in cm^-3." << " " << instanceStruct.instanceVar_B4SOInsd << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIngate>1.0e23))
{
UserInfo(theInstance)  << "Warning: Ngate = %g is too large, should be specified in cm^-3." << " " << instanceStruct.instanceVar_B4SOIngate << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOInfactor<0.0))
{
UserInfo(theInstance)  << "Warning: Nfactor = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOInfactor << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIcdsc<0.0))
{
UserInfo(theInstance)  << "Warning: Cdsc = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIcdsc << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIcdscd<0.0))
{
UserInfo(theInstance)  << "Warning: Cdscd = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIcdscd << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIeta0<0.0))
{
UserInfo(theInstance)  << "Warning: Eta0 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIeta0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIeta0cv<0.0))
{
UserInfo(theInstance)  << "Warning: Eta0CV = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIeta0cv << " " <<  std::endl; 
}
if ((fabs((1.0e-6/(instanceStruct.instanceVar_pParam_B4SOIb1+instanceStruct.instanceVar_pParam_B4SOIweff)))>10.0))
{
UserInfo(theInstance)  << "Warning: (B1 + Weff) may be too small." << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIvsattemp<1.0e3))
{
UserInfo(theInstance)  << "Warning: Vsat at current temperature = %g may be too small." << " " << instanceStruct.instanceVar_pParam_B4SOIvsattemp << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIpdibl1<0.0))
{
UserInfo(theInstance)  << "Warning: Pdibl1 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIpdibl1 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIpdibl2<0.0))
{
UserInfo(theInstance)  << "Warning: Pdibl2 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIpdibl2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIntun<0.0))
{
UserInfo(theInstance)  << "Warning: Ntuns = %g is negative." << " " << instanceStruct.instanceVar_B4SOIntun << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIntund<0.0))
{
UserInfo(theInstance)  << "Warning: Ntund = %g is negative." << " " << instanceStruct.instanceVar_B4SOIntund << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIndiode<0.0))
{
UserInfo(theInstance)  << "Warning: Ndiode = %g is negative." << " " << instanceStruct.instanceVar_B4SOIndiode << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIndioded<0.0))
{
UserInfo(theInstance)  << "Warning: Ndioded = %g is negative." << " " << instanceStruct.instanceVar_B4SOIndioded << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIisbjt<0.0))
{
UserInfo(theInstance)  << "Warning: Isbjt = %g is negative." << " " << instanceStruct.instanceVar_B4SOIisbjt << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIidbjt<0.0))
{
UserInfo(theInstance)  << "Warning: Idbjt = %g is negative." << " " << instanceStruct.instanceVar_B4SOIidbjt << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIisdif<0.0))
{
UserInfo(theInstance)  << "Warning: Isdif = %g is negative." << " " << instanceStruct.instanceVar_B4SOIisdif << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIiddif<0.0))
{
UserInfo(theInstance)  << "Warning: Iddif = %g is negative." << " " << instanceStruct.instanceVar_B4SOIiddif << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIisrec<0.0))
{
UserInfo(theInstance)  << "Warning: Isrec = %g is negative." << " " << instanceStruct.instanceVar_B4SOIisrec << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIidrec<0.0))
{
UserInfo(theInstance)  << "Warning: Idrec = %g is negative." << " " << instanceStruct.instanceVar_B4SOIidrec << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIistun<0.0))
{
UserInfo(theInstance)  << "Warning: Istun = %g is negative." << " " << instanceStruct.instanceVar_B4SOIistun << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIidtun<0.0))
{
UserInfo(theInstance)  << "Warning: Idtun = %g is negative." << " " << instanceStruct.instanceVar_B4SOIidtun << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOItt<0.0))
{
UserInfo(theInstance)  << "Warning: Tt = %g is negative." << " " << instanceStruct.instanceVar_B4SOItt << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIcsdmin<0.0))
{
UserInfo(theInstance)  << "Warning: Csdmin = %g is negative." << " " << instanceStruct.instanceVar_B4SOIcsdmin << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIcsdesw<0.0))
{
UserInfo(theInstance)  << "Warning: Csdesw = %g is negative." << " " << instanceStruct.instanceVar_B4SOIcsdesw << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIrth0<0.0))
{
UserInfo(theInstance)  << "Warning: Rth0 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIrth0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIcth0<0.0))
{
UserInfo(theInstance)  << "Warning: Cth0 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIcth0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIwth0<0.0))
{
UserInfo(theInstance)  << "Warning:  Wth0 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIwth0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIrbody<0.0))
{
UserInfo(theInstance)  << "Warning: Rbody = %g is negative." << " " << instanceStruct.instanceVar_B4SOIrbody << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIrbsh<0.0))
{
UserInfo(theInstance)  << "Warning: Rbsh = %g is negative." << " " << instanceStruct.instanceVar_B4SOIrbsh << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIrhalo<0.0))
{
UserInfo(theInstance)  << "Warning:  Rhalo = %g is negative." << " " << instanceStruct.instanceVar_B4SOIrhalo << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIntox<0.0))
{
UserInfo(theInstance)  << "Warning:  Ntox = %g is negative." << " " << instanceStruct.instanceVar_B4SOIntox << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIebg<0.0))
{
UserInfo(theInstance)  << "Warning:  Ebg = %g is negative." << " " << instanceStruct.instanceVar_B4SOIebg << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvevb<0.0))
{
UserInfo(theInstance)  << "Warning:  Vevb = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvevb << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIalphaGB1<0.0))
{
UserInfo(theInstance)  << "Warning:  AlphaGB1 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIalphaGB1 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIbetaGB1<0.0))
{
UserInfo(theInstance)  << "Warning:  BetaGB1 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIbetaGB1 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvgb1<0.0))
{
UserInfo(theInstance)  << "Warning:  Vgb1 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvgb1 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvecb<0.0))
{
UserInfo(theInstance)  << "Warning:  Vecb = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvecb << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIalphaGB2<0.0))
{
UserInfo(theInstance)  << "Warning:  AlphaGB2 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIalphaGB2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIbetaGB2<0.0))
{
UserInfo(theInstance)  << "Warning:  BetaGB2 = %g is negative." << " " << instanceStruct.instanceVar_pParam_B4SOIbetaGB2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvgb2<0.0))
{
UserInfo(theInstance)  << "Warning:  Vgb2 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvgb2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvoxh<0.0))
{
UserInfo(theInstance)  << "Warning:  Voxh = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvoxh << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIdeltavox<=0.0))
{
UserInfo(theInstance)  << "Fatal: Deltavox = %g is not positive." << " " << instanceStruct.instanceVar_B4SOIdeltavox << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIk1w1<0.0))
{
UserInfo(theInstance)  << "Warning:  K1w1 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIk1w1 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIk1w2<0.0))
{
UserInfo(theInstance)  << "Warning:  K1w2 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIk1w2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIketas<0.0))
{
UserInfo(theInstance)  << "Warning:  Ketas = %g is negative." << " " << instanceStruct.instanceVar_B4SOIketas << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIdwbc<0.0))
{
UserInfo(theInstance)  << "Warning:  Dwbc = %g is negative." << " " << instanceStruct.instanceVar_B4SOIdwbc << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIbeta0<0.0))
{
UserInfo(theInstance)  << "Warning:  Beta0 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIbeta0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIbeta1<0.0))
{
UserInfo(theInstance)  << "Warning:  Beta1 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIbeta1 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIbeta2<0.0))
{
UserInfo(theInstance)  << "Warning:  Beta2 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIbeta2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIlii<0.0))
{
UserInfo(theInstance)  << "Warning:  Lii = %g is negative." << " " << instanceStruct.instanceVar_B4SOIlii << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIsii1<0.0))
{
UserInfo(theInstance)  << "Warning:  Sii1 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIsii1 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIsii2<0.0))
{
UserInfo(theInstance)  << "Warning:  Sii2 = %g is negative." << " " << instanceStruct.instanceVar_B4SOIsii2 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIsiid<0.0))
{
UserInfo(theInstance)  << "Warning:  Siid = %g is negative." << " " << instanceStruct.instanceVar_B4SOIsiid << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIfbjtii<0.0))
{
UserInfo(theInstance)  << "Warning:  fbjtii = %g is negative." << " " << instanceStruct.instanceVar_B4SOIfbjtii << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvrec0<0.0))
{
UserInfo(theInstance)  << "Warning:  Vrec0s = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvrec0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvrec0d<0.0))
{
UserInfo(theInstance)  << "Warning:  Vrec0d = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvrec0d << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvtun0<0.0))
{
UserInfo(theInstance)  << "Warning:  Vtun0s = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvtun0 << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIvtun0d<0.0))
{
UserInfo(theInstance)  << "Warning:  Vtun0d = %g is negative." << " " << instanceStruct.instanceVar_B4SOIvtun0d << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOInbjt<0.0))
{
UserInfo(theInstance)  << "Warning:  Nbjt = %g is negative." << " " << instanceStruct.instanceVar_B4SOInbjt << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIaely<0.0))
{
UserInfo(theInstance)  << "Warning:  Aely = %g is negative." << " " << instanceStruct.instanceVar_B4SOIaely << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIahli<0.0))
{
UserInfo(theInstance)  << "Warning:  Ahlis = %g is negative." << " " << instanceStruct.instanceVar_B4SOIahli << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIahlid<0.0))
{
UserInfo(theInstance)  << "Warning:  Ahlid = %g is negative." << " " << instanceStruct.instanceVar_B4SOIahlid << " " <<  std::endl; 
}
if (((instanceStruct.instanceVar_B4SOIacde<0.1)||(instanceStruct.instanceVar_B4SOIacde>1.6)))
{
UserInfo(theInstance)  << "Warning:  Acde = %g is out of range." << " " << instanceStruct.instanceVar_B4SOIacde << " " <<  std::endl; 
}
if (((instanceStruct.instanceVar_B4SOImoin<5.0)||(instanceStruct.instanceVar_B4SOImoin>25.0)))
{
UserInfo(theInstance)  << "Warning:  Moin = %g is out of range." << " " << instanceStruct.instanceVar_B4SOImoin << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIdlbg<0.0))
{
UserInfo(theInstance)  << "Warning:  dlbg = %g is negative." << " " << instanceStruct.instanceVar_B4SOIdlbg << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIagidl<0.0))
{
UserInfo(theInstance)  << "Warning:  Agidl = %g is negative." << " " << instanceStruct.instanceVar_B4SOIagidl << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIbgidl<0.0))
{
UserInfo(theInstance)  << "Warning:  Bgidl = %g is negative." << " " << instanceStruct.instanceVar_B4SOIbgidl << " " <<  std::endl; 
}
if ((fabs(instanceStruct.instanceVar_B4SOIcgidl)<1e-9))
{
UserInfo(theInstance)  << "Warning:  Cgidl = %g is smaller than 1e-9." << " " << instanceStruct.instanceVar_B4SOIcgidl << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIegidl<0.0))
{
UserInfo(theInstance)  << "Warning:  Egidl = %g is negative." << " " << instanceStruct.instanceVar_B4SOIegidl << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIagisl<0.0))
{
UserInfo(theInstance)  << "Warning:  Agidl = %g is negative." << " " << instanceStruct.instanceVar_B4SOIagidl << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIbgisl<0.0))
{
UserInfo(theInstance)  << "Warning:  Bgisl = %g is negative." << " " << instanceStruct.instanceVar_B4SOIbgisl << " " <<  std::endl; 
}
if ((fabs(instanceStruct.instanceVar_B4SOIcgisl)<1e-9))
{
UserInfo(theInstance)  << "Warning:  Cgisl = %g is smaller than 1e-9." << " " << instanceStruct.instanceVar_B4SOIcgisl << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIegisl<0.0))
{
UserInfo(theInstance)  << "Warning:  Egisl = %g is negative." << " " << instanceStruct.instanceVar_B4SOIegisl << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_B4SOIesatii<0.0))
{
UserInfo(theInstance)  << "Warning: Esatii = %g is negative." << " " << instanceStruct.instanceVar_B4SOIesatii << " " <<  std::endl; 
}
if ((instanceStruct.instanceVar_pParam_B4SOIxj>instanceStruct.instanceVar_B4SOItsi))
{
UserInfo(theInstance)  << "Warning: Xj = %g is thicker than Tsi = %g." << " " << instanceStruct.instanceVar_pParam_B4SOIxj << " " << instanceStruct.instanceVar_B4SOItsi << " " <<  std::endl; 
}
}
if (Fatal_Flag)
{
 UserError(theInstance) << "$finish called." << std::endl; 
}
}
// End block b4soi_check
if (((instanceStruct.instancePar_SHMOD==1)&&(instanceStruct.instancePar_RTH0!=0.0)))
{
if ((instanceStruct.instancePar_TNODEOUT&&(!portsConnected_[admsNodeID_t])))
{
if (portsConnected_[admsNodeID_b])
{
delTemp = (probeVars[admsProbeID_V_b_GND]);
}
else
{
if (portsConnected_[admsNodeID_p])
{
delTemp = (probeVars[admsProbeID_V_p_GND]);
}
else
{
delTemp = (probeVars[admsProbeID_Temp_t_GND]);
}
}
}
else
{
delTemp = (probeVars[admsProbeID_Temp_t_GND]);
}
}
else
{
delTemp = 0;
}
instanceStruct.d_instanceVar_DevTemp_dX = instanceStruct.d_instancePar_DTEMP_dX;
instanceStruct.instanceVar_DevTemp = (admsTemperature+instanceStruct.instancePar_DTEMP);
instanceStruct.instanceVar_DevTemp = (delTemp+instanceStruct.instanceVar_DevTemp);
d_TempRatio_dX = ((instanceStruct.instanceVar_B4SOItnom*instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.instanceVar_DevTemp*instanceStruct.d_instanceVar_B4SOItnom_dX)/instanceStruct.instanceVar_B4SOItnom/instanceStruct.instanceVar_B4SOItnom);
TempRatio = (instanceStruct.instanceVar_DevTemp/instanceStruct.instanceVar_B4SOItnom);
d_TempRatioMinus1_dX = ((instanceStruct.instanceVar_B4SOItnom*instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.instanceVar_DevTemp*instanceStruct.d_instanceVar_B4SOItnom_dX)/instanceStruct.instanceVar_B4SOItnom/instanceStruct.instanceVar_B4SOItnom);
TempRatioMinus1 = ((instanceStruct.instanceVar_DevTemp/instanceStruct.instanceVar_B4SOItnom)-1.0);
instanceStruct.d_instanceVar_B4SOITempSH_dX = instanceStruct.d_instanceVar_DevTemp_dX;
instanceStruct.instanceVar_B4SOITempSH = instanceStruct.instanceVar_DevTemp;
//Begin block load
{
//Block-local variables for block load
double VgstNVt=0.0;
double d_VgstNVt_dX=0.0;
double ExpVgst=0.0;
double d_ExpVgst_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double arg=0.0;
double d_arg_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vfbeff=0.0;
double d_Vfbeff_dX=0.0;
double V3=0.0;
double d_V3_dX=0.0;
double V4=0.0;
double d_V4_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double PhiBSWG=0.0;
double d_PhiBSWG_dX=0.0;
double MJSWG=0.0;
double d_MJSWG_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double vbd=0.0;
double vbs=0.0;
double vds=0.0;
double vgd=0.0;
double vgs=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vds=0.0;
double Vgs=0.0;
double Vbs=0.0;
double d_Vbs_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgs_eff=0.0;
double d_Vgs_eff_dX=0.0;
double Vfb=0.0;
double d_Vfb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Phis=0.0;
double d_Phis_dX=0.0;
double sqrtPhis=0.0;
double d_sqrtPhis_dX=0.0;
double Vth=0.0;
double d_Vth_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgst=0.0;
double d_Vgst_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double n=0.0;
double d_n_dX=0.0;
double Vtm=0.0;
double d_Vtm_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double ExpArg=0.0;
double d_ExpArg_dX=0.0;
double V0=0.0;
double d_V0_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double ueff=0.0;
double d_ueff_dX=0.0;
double Esat=0.0;
double d_Esat_dX=0.0;
double Vdsat=0.0;
double d_Vdsat_dX=0.0;
double EsatL=0.0;
double d_EsatL_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Va=0.0;
double d_Va_dX=0.0;
double Vasat=0.0;
double d_Vasat_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbseff=0.0;
double d_Vbseff_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWL=0.0;
double d_CoxWL_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double T0=0.0;
double d_T0_dX=0.0;
double T1=0.0;
double d_T1_dX=0.0;
double T2=0.0;
double d_T2_dX=0.0;
double T3=0.0;
double d_T3_dX=0.0;
double T4=0.0;
double d_T4_dX=0.0;
double T5=0.0;
double d_T5_dX=0.0;
double T6=0.0;
double d_T6_dX=0.0;
double T7=0.0;
double d_T7_dX=0.0;
double T8=0.0;
double d_T8_dX=0.0;
double T9=0.0;
double d_T9_dX=0.0;
double T10=0.0;
double d_T10_dX=0.0;
double T11=0.0;
double d_T11_dX=0.0;
double T12=0.0;
double d_T12_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double dT2_dVg=0.0;
double d_dT2_dVg_dX=0.0;
double dT3_dVb=0.0;
double d_dT3_dVb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double tmp=0.0;
double d_tmp_dX=0.0;
double Abulk=0.0;
double d_Abulk_dX=0.0;
double Abulk0=0.0;
double d_Abulk0_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VACLM=0.0;
double d_VACLM_dX=0.0;
double VADIBL=0.0;
double d_VADIBL_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Xdep=0.0;
double d_Xdep_dX=0.0;
double lt1=0.0;
double d_lt1_dX=0.0;
double ltw=0.0;
double d_ltw_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Delt_vth=0.0;
double d_Delt_vth_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Theta0=0.0;
double d_Theta0_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double tmp1=0.0;
double d_tmp1_dX=0.0;
double tmp2=0.0;
double d_tmp2_dX=0.0;
double tmp3=0.0;
double d_tmp3_dX=0.0;
double tmp4=0.0;
double d_tmp4_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DIBL_Sft=0.0;
double d_DIBL_Sft_dX=0.0;
double Lambda=0.0;
double d_Lambda_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgsteff=0.0;
double d_Vgsteff_dX=0.0;
double Vdseff=0.0;
double d_Vdseff_dX=0.0;
double VdseffCV=0.0;
double d_VdseffCV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double diffVds=0.0;
double d_diffVds_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double dAbulk_dVg=0.0;
double d_dAbulk_dVg_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double beta=0.0;
double d_beta_dX=0.0;
double gche=0.0;
double d_gche_dX=0.0;
double fgche1=0.0;
double d_fgche1_dX=0.0;
double fgche2=0.0;
double d_fgche2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Idl=0.0;
double d_Idl_dX=0.0;
double Ids=0.0;
double d_Ids_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWovL=0.0;
double d_CoxWovL_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Rds=0.0;
double d_Rds_dX=0.0;
double WVCox=0.0;
double d_WVCox_dX=0.0;
double WVCoxRds=0.0;
double d_WVCoxRds_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgst2Vtm=0.0;
double d_Vgst2Vtm_dX=0.0;
double VdsatCV=0.0;
double d_VdsatCV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Leff=0.0;
double d_Leff_dX=0.0;
double Weff=0.0;
double d_Weff_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double AbulkCV=0.0;
double d_AbulkCV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double qgdo=0.0;
double d_qgdo_dX=0.0;
double qgso=0.0;
double d_qgso_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double ves=0.0;
double ved=0.0;
double vbp=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbd=0.0;
double Ves=0.0;
double Vesfb=0.0;
double d_Vesfb_dX=0.0;
double DeltVthtemp=0.0;
double d_DeltVthtemp_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltVthw=0.0;
double d_DeltVthw_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igidl=0.0;
double d_Igidl_dX=0.0;
double Igisl=0.0;
double d_Igisl_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibp=0.0;
double d_Ibp_dX=0.0;
double Iii=0.0;
double d_Iii_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double qjd=0.0;
double d_qjd_dX=0.0;
double qjs=0.0;
double d_qjs_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double ni=0.0;
double d_ni_dX=0.0;
double Eg=0.0;
double d_Eg_dX=0.0;
double Cbox=0.0;
double d_Cbox_dX=0.0;
double CboxWL=0.0;
double d_CboxWL_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double qinv=0.0;
double d_qinv_dX=0.0;
double qgate=0.0;
double d_qgate_dX=0.0;
double qbody=0.0;
double d_qbody_dX=0.0;
double qdrn=0.0;
double d_qdrn_dX=0.0;
double qsrc=0.0;
double d_qsrc_dX=0.0;
double qsub=0.0;
double d_qsub_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vdsatii=0.0;
double d_Vdsatii_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibd1=0.0;
double d_Ibd1_dX=0.0;
double Ibd2=0.0;
double d_Ibd2_dX=0.0;
double Ibd3=0.0;
double d_Ibd3_dX=0.0;
double Ibd4=0.0;
double d_Ibd4_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibs1=0.0;
double d_Ibs1_dX=0.0;
double Ibs2=0.0;
double d_Ibs2_dX=0.0;
double Ibs3=0.0;
double d_Ibs3_dX=0.0;
double Ibs4=0.0;
double d_Ibs4_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double WTsi=0.0;
double d_WTsi_dX=0.0;
double NVtm1=0.0;
double d_NVtm1_dX=0.0;
double NVtm2=0.0;
double d_NVtm2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ic=0.0;
double d_Ic_dX=0.0;
double Ibs=0.0;
double d_Ibs_dX=0.0;
double Ibd=0.0;
double d_Ibd_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Denomi=0.0;
double d_Denomi_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Qsub0=0.0;
double d_Qsub0_dX=0.0;
double Qac0=0.0;
double d_Qac0_dX=0.0;
double Qe1=0.0;
double d_Qe1_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double vbi=0.0;
double d_vbi_dX=0.0;
double vfbb=0.0;
double d_vfbb_dX=0.0;
double phi=0.0;
double d_phi_dX=0.0;
double sqrtPhi=0.0;
double d_sqrtPhi_dX=0.0;
double Xdep0=0.0;
double d_Xdep0_dX=0.0;
double u0temp=0.0;
double d_u0temp_dX=0.0;
double vsattemp=0.0;
double d_vsattemp_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double jdifs=0.0;
double d_jdifs_dX=0.0;
double jdifd=0.0;
double d_jdifd_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double jbjts=0.0;
double d_jbjts_dX=0.0;
double jbjtd=0.0;
double d_jbjtd_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double jrecs=0.0;
double d_jrecs_dX=0.0;
double jrecd=0.0;
double d_jrecd_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double jtuns=0.0;
double d_jtuns_dX=0.0;
double jtund=0.0;
double d_jtund_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double rds0=0.0;
double d_rds0_dX=0.0;
double ua=0.0;
double d_ua_dX=0.0;
double ub=0.0;
double d_ub_dX=0.0;
double uc=0.0;
double d_uc_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double rho=0.0;
double d_rho_dX=0.0;
double rho_ref=0.0;
double d_rho_ref_dX=0.0;
double ku0temp=0.0;
double d_ku0temp_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsh=0.0;
double d_Vbsh_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double sqrtPhisExt=0.0;
double d_sqrtPhisExt_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double T13=0.0;
double d_T13_dX=0.0;
double T14=0.0;
double d_T14_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vdsatii0=0.0;
double d_Vdsatii0_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VgsStep=0.0;
double d_VgsStep_dX=0.0;
double Ratio=0.0;
double d_Ratio_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vdiff=0.0;
double d_Vdiff_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double NVtmf=0.0;
double d_NVtmf_dX=0.0;
double NVtmr=0.0;
double d_NVtmr_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ahlis=0.0;
double d_Ahlis_dX=0.0;
double Ahlid=0.0;
double d_Ahlid_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double WsTsi=0.0;
double d_WsTsi_dX=0.0;
double WdTsi=0.0;
double d_WdTsi_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double cjsbs=0.0;
double d_cjsbs_dX=0.0;
double cjdbs=0.0;
double d_cjdbs_dX=0.0;
double dcjdbs_dT=0.0;
double d_dcjdbs_dT_dX=0.0;
double dcjsbs_dT=0.0;
double d_dcjsbs_dT_dX=0.0;
double dPhiBSWG_dT=0.0;
double d_dPhiBSWG_dT_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWLb=0.0;
double d_CoxWLb_dX=0.0;
double CoxWLcenb=0.0;
double d_CoxWLcenb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double ExpVbsNVtm=0.0;
double d_ExpVbsNVtm_dX=0.0;
double ExpVbdNVtm=0.0;
double d_ExpVbdNVtm_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ien=0.0;
double d_Ien_dX=0.0;
double Iendif=0.0;
double d_Iendif_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ibsdif=0.0;
double d_Ibsdif_dX=0.0;
double Ibddif=0.0;
double d_Ibddif_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ehlis=0.0;
double d_Ehlis_dX=0.0;
double Ehlid=0.0;
double d_Ehlid_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double EhlisFactor=0.0;
double d_EhlisFactor_dX=0.0;
double EhlidFactor=0.0;
double d_EhlidFactor_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double E2ndFactor=0.0;
double d_E2ndFactor_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DioMax=0.0;
double d_DioMax_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double wdios=0.0;
double d_wdios_dX=0.0;
double wdiod=0.0;
double d_wdiod_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Cox=0.0;
double d_Cox_dX=0.0;
double Tox=0.0;
double d_Tox_dX=0.0;
double Tcen=0.0;
double d_Tcen_dX=0.0;
double LINK=0.0;
double d_LINK_dX=0.0;
double Ccen=0.0;
double d_Ccen_dX=0.0;
double Coxeff=0.0;
double d_Coxeff_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWLcen=0.0;
double d_CoxWLcen_dX=0.0;
double DeltaPhi=0.0;
double d_DeltaPhi_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double qinoi=0.0;
double d_qinoi_dX=0.0;
double qbulk=0.0;
double d_qbulk_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double T3zb=0.0;
double d_T3zb_dX=0.0;
double lt1zb=0.0;
double d_lt1zb_dX=0.0;
double ltwzb=0.0;
double d_ltwzb_dX=0.0;
double Theta0zb=0.0;
double d_Theta0zb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Delt_vthzb=0.0;
double d_Delt_vthzb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltVthwzb=0.0;
double d_DeltVthwzb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltVthtempzb=0.0;
double d_DeltVthtempzb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vthzb=0.0;
double d_Vthzb_dX=0.0;
double Vfbzb=0.0;
double d_Vfbzb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double noff=0.0;
double d_noff_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double noff2=0.0;
double d_noff2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double vgms=0.0;
double vgmd=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igc=0.0;
double d_Igc_dX=0.0;
double Igs=0.0;
double d_Igs_dX=0.0;
double Igd=0.0;
double d_Igd_dX=0.0;
double Igcs=0.0;
double d_Igcs_dX=0.0;
double Igcd=0.0;
double d_Igcd_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double vgs_eff=0.0;
double d_vgs_eff_dX=0.0;
double vgd_eff=0.0;
double d_vgd_eff_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VxNVt=0.0;
double d_VxNVt_dX=0.0;
double ExpVxNVt=0.0;
double d_ExpVxNVt_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsitf=0.0;
double d_Vbsitf_dX=0.0;
double Vbs0=0.0;
double d_Vbs0_dX=0.0;
double Vbs0mos=0.0;
double d_Vbs0mos_dX=0.0;
double Vbsmos=0.0;
double d_Vbsmos_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double PhiON=0.0;
double d_PhiON_dX=0.0;
double PhiFD=0.0;
double d_PhiFD_dX=0.0;
double Vbs0t=0.0;
double d_Vbs0t_dX=0.0;
double VthFD=0.0;
double d_VthFD_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgsFD=0.0;
double d_VtgsFD_dX=0.0;
double ExpVtgsFD=0.0;
double d_ExpVtgsFD_dX=0.0;
double VgstFD=0.0;
double d_VgstFD_dX=0.0;
double ExpVgstFD=0.0;
double d_ExpVgstFD_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgseffFD=0.0;
double d_VtgseffFD_dX=0.0;
double VgsteffFD=0.0;
double d_VgsteffFD_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbseff_CV=0.0;
double d_Vbseff_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsh_CV=0.0;
double d_Vbsh_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbsitf_CV=0.0;
double d_Vbsitf_CV_dX=0.0;
double Vbs0_CV=0.0;
double d_Vbs0_CV_dX=0.0;
double Vbsmos_CV=0.0;
double d_Vbsmos_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double PhiON_CV=0.0;
double d_PhiON_CV_dX=0.0;
double PhiFD_CV=0.0;
double d_PhiFD_CV_dX=0.0;
double Vbs0t_CV=0.0;
double d_Vbs0t_CV_dX=0.0;
double VthFD_CV=0.0;
double d_VthFD_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Phis_CV=0.0;
double d_Phis_CV_dX=0.0;
double sqrtPhis_CV=0.0;
double d_sqrtPhis_CV_dX=0.0;
double Xdep_CV=0.0;
double d_Xdep_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double lt1_CV=0.0;
double d_lt1_CV_dX=0.0;
double ltw_CV=0.0;
double d_ltw_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Theta0_CV=0.0;
double d_Theta0_CV_dX=0.0;
double n_CV=0.0;
double d_n_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgsFD_CV=0.0;
double d_VtgsFD_CV_dX=0.0;
double ExpVtgsFD_CV=0.0;
double d_ExpVtgsFD_CV_dX=0.0;
double VgstFD_CV=0.0;
double d_VgstFD_CV_dX=0.0;
double ExpVgstFD_CV=0.0;
double d_ExpVgstFD_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VtgseffFD_CV=0.0;
double d_VtgseffFD_CV_dX=0.0;
double VgsteffFD_CV=0.0;
double d_VgsteffFD_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double tmp2_CV=0.0;
double d_tmp2_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Delt_vth_CV=0.0;
double d_Delt_vth_CV_dX=0.0;
double DeltVthw_CV=0.0;
double d_DeltVthw_CV_dX=0.0;
double DeltVthtemp_CV=0.0;
double d_DeltVthtemp_CV_dX=0.0;
double DIBL_Sft_CV=0.0;
double d_DIBL_Sft_CV_dX=0.0;
double DITS_Sft_CV=0.0;
double d_DITS_Sft_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double sqrtPhisExt_CV=0.0;
double d_sqrtPhisExt_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vth_CV=0.0;
double d_Vth_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Abulk0_CV=0.0;
double d_Abulk0_CV_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgb=0.0;
double d_Vgb_dX=0.0;
double Vox=0.0;
double d_Vox_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double OxideRatio=0.0;
double d_OxideRatio_dX=0.0;
double Vaux=0.0;
double d_Vaux_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igb=0.0;
double d_Igb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Voxeff=0.0;
double d_Voxeff_dX=0.0;
double Voxacc=0.0;
double d_Voxacc_dX=0.0;
double Voxdepinv=0.0;
double d_Voxdepinv_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Igb1=0.0;
double d_Igb1_dX=0.0;
double Igb2=0.0;
double d_Igb2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double IdlovVdseff=0.0;
double d_IdlovVdseff_dX=0.0;
double IdovVds=0.0;
double d_IdovVds_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double vsbs=0.0;
double vdbd=0.0;
//End of Block-local variables
//Block-local variables for block load
double Lpe_Vb=0.0;
double d_Lpe_Vb_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DITS_Sft=0.0;
double d_DITS_Sft_dX=0.0;
double DITS_Sft2=0.0;
double d_DITS_Sft2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double FP=0.0;
double d_FP_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VADITS=0.0;
double d_VADITS_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double PowWeffWr=0.0;
double d_PowWeffWr_dX=0.0;
double rd0=0.0;
double d_rd0_dX=0.0;
double rs0=0.0;
double d_rs0_dX=0.0;
double rdwmin=0.0;
double d_rdwmin_dX=0.0;
double rswmin=0.0;
double d_rswmin_dX=0.0;
double Rd=0.0;
double d_Rd_dX=0.0;
double Rs=0.0;
double d_Rs_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Tnom=0.0;
double d_Tnom_dX=0.0;
double Eg0=0.0;
double d_Eg0_dX=0.0;
double Vtm0=0.0;
double d_Vtm0_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vbci=0.0;
double d_Vbci_dX=0.0;
double Idsmosfet=0.0;
double d_Idsmosfet_dX=0.0;
double Iiibjt=0.0;
double d_Iiibjt_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double VgsteffVth=0.0;
double d_VgsteffVth_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double toxe_mob=0.0;
double d_toxe_mob_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgd=0.0;
double Vgd_eff=0.0;
double d_Vgd_eff_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Ig_agbcp2=0.0;
double d_Ig_agbcp2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double vgp_eff=0.0;
double d_vgp_eff_dX=0.0;
double vgp=0.0;
//End of Block-local variables
//Block-local variables for block load
double CoxWL2=0.0;
double d_CoxWL2_dX=0.0;
double CoxWLb2=0.0;
double d_CoxWLb2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double ExpVgst2=0.0;
double d_ExpVgst2_dX=0.0;
double Vgsteff2=0.0;
double d_Vgsteff2_dX=0.0;
double VgstNVt2=0.0;
double d_VgstNVt2_dX=0.0;
double ExpArg2=0.0;
double d_ExpArg2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double T02=0.0;
double d_T02_dX=0.0;
double T22=0.0;
double d_T22_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Qac02=0.0;
double d_Qac02_dX=0.0;
double Qsub02=0.0;
double d_Qsub02_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vgs_eff2=0.0;
//End of Block-local variables
//Block-local variables for block load
double Vfbzb2=0.0;
double d_Vfbzb2_dX=0.0;
double Vfb2=0.0;
double d_Vfb2_dX=0.0;
double Vfbeff2=0.0;
double d_Vfbeff2_dX=0.0;
double VdsatCV2=0.0;
double d_VdsatCV2_dX=0.0;
double VdseffCV2=0.0;
double d_VdseffCV2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double Tcen2=0.0;
double d_Tcen2_dX=0.0;
double Ccen2=0.0;
double d_Ccen2_dX=0.0;
double Coxeff2=0.0;
double d_Coxeff2_dX=0.0;
double CoxWLcen2=0.0;
double d_CoxWLcen2_dX=0.0;
double CoxWLcenb2=0.0;
double d_CoxWLcenb2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DeltaPhi2=0.0;
double d_DeltaPhi2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double qsrc2=0.0;
double d_qsrc2_dX=0.0;
double qbulk2=0.0;
double d_qbulk2_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double DELTA_3_SOI2=0.0;
//End of Block-local variables
//Block-local variables for block load
double cdep0=0.0;
double d_cdep0_dX=0.0;
double theta0vb0=0.0;
double d_theta0vb0_dX=0.0;
double thetaRout=0.0;
double d_thetaRout_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double agidl=0.0;
double d_agidl_dX=0.0;
double bgidl=0.0;
double d_bgidl_dX=0.0;
double cgidl=0.0;
double d_cgidl_dX=0.0;
double egidl=0.0;
double d_egidl_dX=0.0;
double rgidl=0.0;
double d_rgidl_dX=0.0;
double kgidl=0.0;
double d_kgidl_dX=0.0;
double fgidl=0.0;
double d_fgidl_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double agisl=0.0;
double d_agisl_dX=0.0;
double bgisl=0.0;
double d_bgisl_dX=0.0;
double cgisl=0.0;
double d_cgisl_dX=0.0;
double egisl=0.0;
double d_egisl_dX=0.0;
double rgisl=0.0;
double d_rgisl_dX=0.0;
double kgisl=0.0;
double d_kgisl_dX=0.0;
double fgisl=0.0;
double d_fgisl_dX=0.0;
//End of Block-local variables
//Block-local variables for block load
double ucs=0.0;
double d_ucs_dX=0.0;
double ud=0.0;
double d_ud_dX=0.0;
//End of Block-local variables
d_Coxeff2_dX = 0.0;
Coxeff2 = 0;
d_CoxWLcen2_dX = 0.0;
CoxWLcen2 = 0;
d_CoxWLcenb2_dX = 0.0;
CoxWLcenb2 = 0;
d_DeltaPhi2_dX = 0.0;
DeltaPhi2 = 0;
d_Tcen2_dX = 0.0;
Tcen2 = 0;
d_T02_dX = 0.0;
T02 = 0;
d_T12_dX = 0.0;
T12 = 0;
d_T22_dX = 0.0;
T22 = 0;
d_VdseffCV2_dX = 0.0;
VdseffCV2 = 0;
d_Vfb2_dX = 0.0;
Vfb2 = 0;
d_Vfbeff2_dX = 0.0;
Vfbeff2 = 0;
d_Vfbzb2_dX = 0.0;
Vfbzb2 = 0;
d_Vgsteff2_dX = 0.0;
Vgsteff2 = 0;
d_rds0_dX = instanceStruct.d_instanceVar_pParam_B4SOIrds0_dX;
rds0 = instanceStruct.instanceVar_pParam_B4SOIrds0;
d_rd0_dX = instanceStruct.d_instanceVar_pParam_B4SOIrd0_dX;
rd0 = instanceStruct.instanceVar_pParam_B4SOIrd0;
d_rs0_dX = instanceStruct.d_instanceVar_pParam_B4SOIrs0_dX;
rs0 = instanceStruct.instanceVar_pParam_B4SOIrs0;
d_rdwmin_dX = instanceStruct.d_instanceVar_pParam_B4SOIrdwmin_dX;
rdwmin = instanceStruct.instanceVar_pParam_B4SOIrdwmin;
d_rswmin_dX = instanceStruct.d_instanceVar_pParam_B4SOIrswmin_dX;
rswmin = instanceStruct.instanceVar_pParam_B4SOIrswmin;
if (((instanceStruct.instancePar_SHMOD==1)&&(instanceStruct.instancePar_RTH0!=0.0)))
{
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_Vtm_dX = 8.617087e-5*instanceStruct.d_instanceVar_DevTemp_dX;
Vtm = (8.617087e-5*instanceStruct.instanceVar_DevTemp);
d_T0_dX = instanceStruct.d_instanceVar_DevTemp_dX;
T0 = (1108.0+instanceStruct.instanceVar_DevTemp);
d_T5_dX = (instanceStruct.instanceVar_DevTemp*instanceStruct.d_instanceVar_DevTemp_dX+instanceStruct.d_instanceVar_DevTemp_dX*instanceStruct.instanceVar_DevTemp);
T5 = (instanceStruct.instanceVar_DevTemp*instanceStruct.instanceVar_DevTemp);
d_Eg_dX = (-((T0*7.02e-4*d_T5_dX-(7.02e-4*T5)*d_T0_dX)/T0/T0));
Eg = (1.16-((7.02e-4*T5)/T0));
d_T2_dX = 0.0;
T2 = 1.9230584e-4;
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_DevTemp);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T5_dX = deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_DevTemp_dX);
T5 = value_sqrt_0;
}
d_T3_dX = (((1.45e10*instanceStruct.instanceVar_DevTemp)*T5)*d_T2_dX+((1.45e10*instanceStruct.instanceVar_DevTemp)*d_T5_dX+1.45e10*instanceStruct.d_instanceVar_DevTemp_dX*T5)*T2);
T3 = (((1.45e10*instanceStruct.instanceVar_DevTemp)*T5)*T2);
d_T6_dX = (-(((2.0*Vtm)*d_Eg_dX-Eg*2.0*d_Vtm_dX)/(2.0*Vtm)/(2.0*Vtm)));
T6 = (21.5565981-(Eg/(2.0*Vtm)));
if ((T6>(-100.0)))
{
{
double value_exp_0 = exp(T6);
double  deriv_exp_0_d0 = value_exp_0;
d_T4_dX = deriv_exp_0_d0*(d_T6_dX);
T4 = value_exp_0;
}
}
else
{
{
double value_exp_0 = exp((-100.0));
d_T4_dX = 0.0;
T4 = value_exp_0;
}
}
d_ni_dX = (T3*d_T4_dX+d_T3_dX*T4);
ni = (T3*T4);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)));
d_T0_dX = evaluator_lln_0.getDerivs(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)),(((ni*ni)*1.0e20*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-(1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)*(ni*d_ni_dX+d_ni_dX*ni))/(ni*ni)/(ni*ni)));
T0 = evaluator_lln_0.getValues(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)));
}
d_vbi_dX = (Vtm*d_T0_dX+d_Vtm_dX*T0);
vbi = (Vtm*T0);
}
else
{
d_Tnom_dX = instanceStruct.d_instanceVar_B4SOItnom_dX;
Tnom = instanceStruct.instanceVar_B4SOItnom;
d_Vtm_dX = 8.617087e-5*instanceStruct.d_instanceVar_DevTemp_dX;
Vtm = (8.617087e-5*instanceStruct.instanceVar_DevTemp);
d_Vtm0_dX = 8.617087e-5*d_Tnom_dX;
Vtm0 = (8.617087e-5*Tnom);
d_Eg0_dX = instanceStruct.d_instanceVar_B4SOIeg0_dX;
Eg0 = instanceStruct.instanceVar_B4SOIeg0;
d_Eg_dX = (instanceStruct.d_instanceVar_B4SOIbg0sub_dX-(((instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)*((instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.instanceVar_DevTemp)*instanceStruct.d_instanceVar_DevTemp_dX+(instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.d_instanceVar_DevTemp_dX+instanceStruct.d_instanceVar_B4SOItbgasub_dX*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)-((instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)*(instanceStruct.d_instanceVar_DevTemp_dX+instanceStruct.d_instanceVar_B4SOItbgbsub_dX))/(instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)/(instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)));
Eg = (instanceStruct.instanceVar_B4SOIbg0sub-(((instanceStruct.instanceVar_B4SOItbgasub*instanceStruct.instanceVar_DevTemp)*instanceStruct.instanceVar_DevTemp)/(instanceStruct.instanceVar_DevTemp+instanceStruct.instanceVar_B4SOItbgbsub)));
{
double value_sqrt_0 = sqrt(((Tnom*Tnom)*Tnom));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = (-1*deriv_sqrt_0_d0*(((Tnom*Tnom)*d_Tnom_dX+(Tnom*d_Tnom_dX+d_Tnom_dX*Tnom)*Tnom))/value_sqrt_0/value_sqrt_0);
T2 = (1/value_sqrt_0);
}
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_DevTemp);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T5_dX = deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_DevTemp_dX);
T5 = value_sqrt_0;
}
d_T3_dX = (((instanceStruct.instanceVar_B4SOIni0sub*instanceStruct.instanceVar_DevTemp)*T5)*d_T2_dX+((instanceStruct.instanceVar_B4SOIni0sub*instanceStruct.instanceVar_DevTemp)*d_T5_dX+(instanceStruct.instanceVar_B4SOIni0sub*instanceStruct.d_instanceVar_DevTemp_dX+instanceStruct.d_instanceVar_B4SOIni0sub_dX*instanceStruct.instanceVar_DevTemp)*T5)*T2);
T3 = (((instanceStruct.instanceVar_B4SOIni0sub*instanceStruct.instanceVar_DevTemp)*T5)*T2);
{
double value_exp_0 = exp(((Eg0/(2.0*Vtm0))-(Eg/(2.0*Vtm))));
double  deriv_exp_0_d0 = value_exp_0;
d_T4_dX = deriv_exp_0_d0*(((((2.0*Vtm0)*d_Eg0_dX-Eg0*2.0*d_Vtm0_dX)/(2.0*Vtm0)/(2.0*Vtm0))-(((2.0*Vtm)*d_Eg_dX-Eg*2.0*d_Vtm_dX)/(2.0*Vtm)/(2.0*Vtm))));
T4 = value_exp_0;
}
d_ni_dX = (T3*d_T4_dX+d_T3_dX*T4);
ni = (T3*T4);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)));
d_T0_dX = evaluator_lln_0.getDerivs(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)),(((ni*ni)*1.0e20*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-(1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)*(ni*d_ni_dX+d_ni_dX*ni))/(ni*ni)/(ni*ni)));
T0 = evaluator_lln_0.getValues(((1.0e20*instanceStruct.instanceVar_pParam_B4SOInpeak)/(ni*ni)));
}
d_vbi_dX = (Vtm*d_T0_dX+d_Vtm_dX*T0);
vbi = (Vtm*T0);
}
if ((instanceStruct.instanceVar_pParam_B4SOInsub>0))
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_pParam_B4SOInpeak/instanceStruct.instanceVar_pParam_B4SOInsub));
d_T0_dX = evaluator_lln_0.getDerivs((instanceStruct.instanceVar_pParam_B4SOInpeak/instanceStruct.instanceVar_pParam_B4SOInsub),((instanceStruct.instanceVar_pParam_B4SOInsub*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-instanceStruct.instanceVar_pParam_B4SOInpeak*instanceStruct.d_instanceVar_pParam_B4SOInsub_dX)/instanceStruct.instanceVar_pParam_B4SOInsub/instanceStruct.instanceVar_pParam_B4SOInsub));
T0 = evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/instanceStruct.instanceVar_pParam_B4SOInsub));
}
d_vfbb_dX = (((-instanceStruct.instanceVar_B4SOItype)*Vtm)*d_T0_dX+(-instanceStruct.instanceVar_B4SOItype)*d_Vtm_dX*T0);
vfbb = (((-instanceStruct.instanceVar_B4SOItype)*Vtm)*T0);
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(((((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni));
d_T0_dX = evaluator_lln_0.getDerivs(((((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni),((ni*((ni*((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.d_instanceVar_pParam_B4SOInsub_dX+(-instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX)*instanceStruct.instanceVar_pParam_B4SOInsub)-((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)*d_ni_dX)/ni/ni)-(((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)*d_ni_dX)/ni/ni));
T0 = evaluator_lln_0.getValues(((((-instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOInsub)/ni)/ni));
}
d_vfbb_dX = (((-instanceStruct.instanceVar_B4SOItype)*Vtm)*d_T0_dX+(-instanceStruct.instanceVar_B4SOItype)*d_Vtm_dX*T0);
vfbb = (((-instanceStruct.instanceVar_B4SOItype)*Vtm)*T0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((instanceStruct.instanceVar_pParam_B4SOInpeak/ni));
d_phi_dX = ((2.0*Vtm)*evaluator_lln_0.getDerivs((instanceStruct.instanceVar_pParam_B4SOInpeak/ni),((ni*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX-instanceStruct.instanceVar_pParam_B4SOInpeak*d_ni_dX)/ni/ni))+2.0*d_Vtm_dX*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/ni)));
phi = ((2.0*Vtm)*evaluator_lln_0.getValues((instanceStruct.instanceVar_pParam_B4SOInpeak/ni)));
}
{
double value_sqrt_0 = sqrt(phi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_sqrtPhi_dX = deriv_sqrt_0_d0*(d_phi_dX);
sqrtPhi = value_sqrt_0;
}
{
double value_sqrt_0 = sqrt(((2.0*instanceStruct.instanceVar_epssub)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_Xdep0_dX = (value_sqrt_0*d_sqrtPhi_dX+deriv_sqrt_0_d0*(((((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)*2.0*instanceStruct.d_instanceVar_epssub_dX-(2.0*instanceStruct.instanceVar_epssub)*1.60219e-19*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX*1.0e6)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)/((1.60219e-19*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)))*sqrtPhi);
Xdep0 = (value_sqrt_0*sqrtPhi);
}
{
double value_sqrt_0 = sqrt(((((1.60219e-19*instanceStruct.instanceVar_epssub)*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6)/2.0));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_cdep0_dX = ((sqrtPhi*deriv_sqrt_0_d0*((((1.60219e-19*instanceStruct.instanceVar_epssub)*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX+1.60219e-19*instanceStruct.d_instanceVar_epssub_dX*instanceStruct.instanceVar_pParam_B4SOInpeak)*1.0e6/2.0))-value_sqrt_0*d_sqrtPhi_dX)/sqrtPhi/sqrtPhi);
cdep0 = (value_sqrt_0/sqrtPhi);
}
{
double value_sqrt_0 = sqrt((((instanceStruct.instanceVar_epssub/(instanceStruct.instanceVar_epsrox*8.85418e-12))*instanceStruct.instanceVar_toxe)*Xdep0));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((((instanceStruct.instanceVar_epssub/(instanceStruct.instanceVar_epsrox*8.85418e-12))*instanceStruct.instanceVar_toxe)*d_Xdep0_dX+((instanceStruct.instanceVar_epssub/(instanceStruct.instanceVar_epsrox*8.85418e-12))*instanceStruct.d_instanceVar_toxe_dX+(((instanceStruct.instanceVar_epsrox*8.85418e-12)*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*instanceStruct.d_instanceVar_epsrox_dX*8.85418e-12)/(instanceStruct.instanceVar_epsrox*8.85418e-12)/(instanceStruct.instanceVar_epsrox*8.85418e-12))*instanceStruct.instanceVar_toxe)*Xdep0));
T1 = value_sqrt_0;
}
{
double value_exp_0 = exp(((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdsub)*instanceStruct.instanceVar_pParam_B4SOIleff)/T1));
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(((T1*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdsub)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdsub_dX*instanceStruct.instanceVar_pParam_B4SOIleff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdsub)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_T1_dX)/T1/T1));
T0 = value_exp_0;
}
d_theta0vb0_dX = (d_T0_dX+((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0));
theta0vb0 = (T0+((2.0*T0)*T0));
{
double value_exp_0 = exp(((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdrout)*instanceStruct.instanceVar_pParam_B4SOIleff)/T1));
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(((T1*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdrout)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdrout_dX*instanceStruct.instanceVar_pParam_B4SOIleff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdrout)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_T1_dX)/T1/T1));
T0 = value_exp_0;
}
d_T2_dX = (d_T0_dX+((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0));
T2 = (T0+((2.0*T0)*T0));
d_thetaRout_dX = ((instanceStruct.instanceVar_pParam_B4SOIpdibl1*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIpdibl1_dX*T2)+instanceStruct.d_instanceVar_pParam_B4SOIpdibl2_dX);
thetaRout = ((instanceStruct.instanceVar_pParam_B4SOIpdibl1*T2)+instanceStruct.instanceVar_pParam_B4SOIpdibl2);
instanceStruct.d_instanceVar_B4SOIvtm_dX = d_Vtm_dX;
instanceStruct.instanceVar_B4SOIvtm = Vtm;
d_T3_dX = d_TempRatio_dX;
T3 = (TempRatio-1.0);
d_T4_dX = ((1.115/Vtm)*d_T3_dX+(-1.115*d_Vtm_dX/Vtm/Vtm)*T3);
T4 = ((1.115/Vtm)*T3);
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndiode*(instanceStruct.instanceVar_pParam_B4SOIxbjt*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxbjt_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX)/instanceStruct.instanceVar_pParam_B4SOIndiode/instanceStruct.instanceVar_pParam_B4SOIndiode);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)/instanceStruct.instanceVar_pParam_B4SOIndiode);
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
if ((instanceStruct.instanceVar_pParam_B4SOIxbjt==instanceStruct.instanceVar_pParam_B4SOIxdif))
{
d_T1_dX = d_T0_dX;
T1 = T0;
}
else
{
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndiode*(instanceStruct.instanceVar_pParam_B4SOIxdif*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxdif_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxdif*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX)/instanceStruct.instanceVar_pParam_B4SOIndiode/instanceStruct.instanceVar_pParam_B4SOIndiode);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxdif*T4)/instanceStruct.instanceVar_pParam_B4SOIndiode);
if ((T7>100.0))
{
d_T1_dX = 2.688117142e+43*d_T7_dX;
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T7_dX);
T1 = value_exp_0;
}
}
}
}
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOInrecf0*(instanceStruct.instanceVar_pParam_B4SOIxrec*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxrec_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxrec*T4)*instanceStruct.d_instanceVar_pParam_B4SOInrecf0_dX)/instanceStruct.instanceVar_pParam_B4SOInrecf0/instanceStruct.instanceVar_pParam_B4SOInrecf0);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxrec*T4)/instanceStruct.instanceVar_pParam_B4SOInrecf0);
if ((T7>100.0))
{
d_T2_dX = 2.688117142e+43*d_T7_dX;
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T2_dX = 0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T2_dX = deriv_exp_0_d0*(d_T7_dX);
T2 = value_exp_0;
}
}
}
d_Ahlis_dX = (instanceStruct.instanceVar_pParam_B4SOIahli*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIahli_dX*T0);
Ahlis = (instanceStruct.instanceVar_pParam_B4SOIahli*T0);
d_jbjts_dX = (instanceStruct.instanceVar_pParam_B4SOIisbjt*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIisbjt_dX*T0);
jbjts = (instanceStruct.instanceVar_pParam_B4SOIisbjt*T0);
d_jdifs_dX = (instanceStruct.instanceVar_pParam_B4SOIisdif*d_T1_dX+instanceStruct.d_instanceVar_pParam_B4SOIisdif_dX*T1);
jdifs = (instanceStruct.instanceVar_pParam_B4SOIisdif*T1);
d_jrecs_dX = (instanceStruct.instanceVar_pParam_B4SOIisrec*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIisrec_dX*T2);
jrecs = (instanceStruct.instanceVar_pParam_B4SOIisrec*T2);
d_T7_dX = (instanceStruct.instanceVar_pParam_B4SOIxtun*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIxtun_dX*T3);
T7 = (instanceStruct.instanceVar_pParam_B4SOIxtun*T3);
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
d_jtuns_dX = (instanceStruct.instanceVar_pParam_B4SOIistun*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIistun_dX*T0);
jtuns = (instanceStruct.instanceVar_pParam_B4SOIistun*T0);
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndioded*(instanceStruct.instanceVar_pParam_B4SOIxbjt*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxbjt_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX)/instanceStruct.instanceVar_pParam_B4SOIndioded/instanceStruct.instanceVar_pParam_B4SOIndioded);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxbjt*T4)/instanceStruct.instanceVar_pParam_B4SOIndioded);
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
if ((instanceStruct.instanceVar_pParam_B4SOIxbjt==instanceStruct.instanceVar_pParam_B4SOIxdifd))
{
d_T1_dX = d_T0_dX;
T1 = T0;
}
else
{
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOIndioded*(instanceStruct.instanceVar_pParam_B4SOIxdifd*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxdifd_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxdifd*T4)*instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX)/instanceStruct.instanceVar_pParam_B4SOIndioded/instanceStruct.instanceVar_pParam_B4SOIndioded);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxdifd*T4)/instanceStruct.instanceVar_pParam_B4SOIndioded);
if ((T7>100.0))
{
d_T1_dX = 2.688117142e+43*d_T7_dX;
T1 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T7_dX);
T1 = value_exp_0;
}
}
}
}
d_T7_dX = ((instanceStruct.instanceVar_pParam_B4SOInrecf0d*(instanceStruct.instanceVar_pParam_B4SOIxrecd*d_T4_dX+instanceStruct.d_instanceVar_pParam_B4SOIxrecd_dX*T4)-(instanceStruct.instanceVar_pParam_B4SOIxrecd*T4)*instanceStruct.d_instanceVar_pParam_B4SOInrecf0d_dX)/instanceStruct.instanceVar_pParam_B4SOInrecf0d/instanceStruct.instanceVar_pParam_B4SOInrecf0d);
T7 = ((instanceStruct.instanceVar_pParam_B4SOIxrecd*T4)/instanceStruct.instanceVar_pParam_B4SOInrecf0d);
if ((T7>100.0))
{
d_T2_dX = 2.688117142e+43*d_T7_dX;
T2 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T2_dX = 0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T2_dX = deriv_exp_0_d0*(d_T7_dX);
T2 = value_exp_0;
}
}
}
d_Ahlid_dX = (instanceStruct.instanceVar_pParam_B4SOIahlid*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIahlid_dX*T0);
Ahlid = (instanceStruct.instanceVar_pParam_B4SOIahlid*T0);
d_jbjtd_dX = (instanceStruct.instanceVar_pParam_B4SOIidbjt*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIidbjt_dX*T0);
jbjtd = (instanceStruct.instanceVar_pParam_B4SOIidbjt*T0);
d_jdifd_dX = (instanceStruct.instanceVar_pParam_B4SOIiddif*d_T1_dX+instanceStruct.d_instanceVar_pParam_B4SOIiddif_dX*T1);
jdifd = (instanceStruct.instanceVar_pParam_B4SOIiddif*T1);
d_jrecd_dX = (instanceStruct.instanceVar_pParam_B4SOIidrec*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIidrec_dX*T2);
jrecd = (instanceStruct.instanceVar_pParam_B4SOIidrec*T2);
d_T7_dX = (instanceStruct.instanceVar_pParam_B4SOIxtund*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIxtund_dX*T3);
T7 = (instanceStruct.instanceVar_pParam_B4SOIxtund*T3);
if ((T7>100.0))
{
d_T0_dX = 2.688117142e+43*d_T7_dX;
T0 = (2.688117142e+43*((1.0+T7)-100.0));
}
else
{
if ((T7<(-100.0)))
{
d_T0_dX = 0.0;
T0 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(d_T7_dX);
T0 = value_exp_0;
}
}
}
d_jtund_dX = (instanceStruct.instanceVar_pParam_B4SOIidtun*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIidtun_dX*T0);
jtund = (instanceStruct.instanceVar_pParam_B4SOIidtun*T0);
{
double value_pow_0 = pow(TempRatio,instanceStruct.instanceVar_pParam_B4SOIute);
double  deriv_pow_0_d0 = ((TempRatio == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_pParam_B4SOIute/TempRatio));
double  deriv_pow_0_d1 = (TempRatio == 0.0)?0.0:(log(TempRatio)*value_pow_0);
d_u0temp_dX = (instanceStruct.instanceVar_pParam_B4SOIu0*((deriv_pow_0_d0*(d_TempRatio_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_pParam_B4SOIute_dX)))+instanceStruct.d_instanceVar_pParam_B4SOIu0_dX*value_pow_0);
u0temp = (instanceStruct.instanceVar_pParam_B4SOIu0*value_pow_0);
}
if ((modelStruct.modelPar_VERSION<4.2))
{
d_ku0temp_dX = (instanceStruct.instanceVar_pParam_B4SOIku0*(instanceStruct.instanceVar_B4SOItku0*d_TempRatio_dX+instanceStruct.d_instanceVar_B4SOItku0_dX*TempRatio)+instanceStruct.d_instanceVar_pParam_B4SOIku0_dX*(1.0+(instanceStruct.instanceVar_B4SOItku0*TempRatio)));
ku0temp = ((instanceStruct.instanceVar_pParam_B4SOIku0*(1.0+(instanceStruct.instanceVar_B4SOItku0*TempRatio)))+1.0E-9);
}
else
{
d_ku0temp_dX = (instanceStruct.instanceVar_pParam_B4SOIku0*(instanceStruct.instanceVar_B4SOItku0*d_T3_dX+instanceStruct.d_instanceVar_B4SOItku0_dX*T3)+instanceStruct.d_instanceVar_pParam_B4SOIku0_dX*(1.0+(instanceStruct.instanceVar_B4SOItku0*T3)));
ku0temp = ((instanceStruct.instanceVar_pParam_B4SOIku0*(1.0+(instanceStruct.instanceVar_B4SOItku0*T3)))+1.0E-9);
}
d_T7_dX = (instanceStruct.instanceVar_B4SOIku0*instanceStruct.d_instanceVar_pParam_B4SOIinv_od_ref_dX+instanceStruct.d_instanceVar_B4SOIku0_dX*instanceStruct.instanceVar_pParam_B4SOIinv_od_ref);
T7 = (instanceStruct.instanceVar_B4SOIku0*instanceStruct.instanceVar_pParam_B4SOIinv_od_ref);
d_rho_ref_dX = ((ku0temp*d_T7_dX-T7*d_ku0temp_dX)/ku0temp/ku0temp);
rho_ref = (T7/ku0temp);
d_T4_dX = (instanceStruct.instanceVar_B4SOIku0*instanceStruct.d_instanceVar_B4SOIInv_ODeff_dX+instanceStruct.d_instanceVar_B4SOIku0_dX*instanceStruct.instanceVar_B4SOIInv_ODeff);
T4 = (instanceStruct.instanceVar_B4SOIku0*instanceStruct.instanceVar_B4SOIInv_ODeff);
d_rho_dX = ((ku0temp*d_T4_dX-T4*d_ku0temp_dX)/ku0temp/ku0temp);
rho = (T4/ku0temp);
d_T2_dX = d_rho_dX;
T2 = (1.0+rho);
d_T7_dX = d_rho_ref_dX;
T7 = (1.0+rho_ref);
d_T0_dX = ((T7*d_T2_dX-T2*d_T7_dX)/T7/T7);
T0 = (T2/T7);
d_u0temp_dX = (u0temp*d_T0_dX+d_u0temp_dX*T0);
u0temp = (u0temp*T0);
d_vsattemp_dX = (instanceStruct.d_instanceVar_pParam_B4SOIvsat_dX-(instanceStruct.instanceVar_pParam_B4SOIat*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIat_dX*T3));
vsattemp = (instanceStruct.instanceVar_pParam_B4SOIvsat-(instanceStruct.instanceVar_pParam_B4SOIat*T3));
d_T2_dX = (instanceStruct.instanceVar_B4SOIkvsat*d_rho_dX+instanceStruct.d_instanceVar_B4SOIkvsat_dX*rho);
T2 = (1.0+(instanceStruct.instanceVar_B4SOIkvsat*rho));
d_T7_dX = (instanceStruct.instanceVar_B4SOIkvsat*d_rho_ref_dX+instanceStruct.d_instanceVar_B4SOIkvsat_dX*rho_ref);
T7 = (1.0+(instanceStruct.instanceVar_B4SOIkvsat*rho_ref));
d_T0_dX = ((T7*d_T2_dX-T2*d_T7_dX)/T7/T7);
T0 = (T2/T7);
d_vsattemp_dX = (vsattemp*d_T0_dX+d_vsattemp_dX*T0);
vsattemp = (vsattemp*T0);
if ((instanceStruct.instanceVar_B4SOIrdsMod!=1))
{
d_rds0_dX = ((instanceStruct.instanceVar_pParam_B4SOIrds0denom*(instanceStruct.d_instanceVar_pParam_B4SOIrdsw_dX+(instanceStruct.instanceVar_pParam_B4SOIprt*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIprt_dX*T3))-(instanceStruct.instanceVar_pParam_B4SOIrdsw+(instanceStruct.instanceVar_pParam_B4SOIprt*T3))*instanceStruct.d_instanceVar_pParam_B4SOIrds0denom_dX)/instanceStruct.instanceVar_pParam_B4SOIrds0denom/instanceStruct.instanceVar_pParam_B4SOIrds0denom);
rds0 = ((instanceStruct.instanceVar_pParam_B4SOIrdsw+(instanceStruct.instanceVar_pParam_B4SOIprt*T3))/instanceStruct.instanceVar_pParam_B4SOIrds0denom);
d_rd0_dX = 0.0;
rd0 = 0;
d_rs0_dX = 0.0;
rs0 = 0;
}
else
{
d_rds0_dX = 0.0;
rds0 = 0;
d_PowWeffWr_dX = (instanceStruct.instanceVar_pParam_B4SOIrds0denom*instanceStruct.d_instanceVar_B4SOInf_dX+instanceStruct.d_instanceVar_pParam_B4SOIrds0denom_dX*instanceStruct.instanceVar_B4SOInf);
PowWeffWr = (instanceStruct.instanceVar_pParam_B4SOIrds0denom*instanceStruct.instanceVar_B4SOInf);
d_T10_dX = (instanceStruct.instanceVar_pParam_B4SOIprt*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIprt_dX*T3);
T10 = (instanceStruct.instanceVar_pParam_B4SOIprt*T3);
d_T1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIrdw_dX+d_T10_dX);
T1 = (instanceStruct.instanceVar_pParam_B4SOIrdw+T10);
d_T2_dX = (instanceStruct.d_instanceVar_B4SOIrdwmin_dX+d_T10_dX);
T2 = (instanceStruct.instanceVar_B4SOIrdwmin+T10);
d_rd0_dX = ((PowWeffWr*d_T1_dX-T1*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
rd0 = (T1/PowWeffWr);
d_rdwmin_dX = ((PowWeffWr*d_T2_dX-T2*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
rdwmin = (T2/PowWeffWr);
d_T7_dX = (instanceStruct.d_instanceVar_pParam_B4SOIrsw_dX+d_T10_dX);
T7 = (instanceStruct.instanceVar_pParam_B4SOIrsw+T10);
d_T4_dX = (instanceStruct.d_instanceVar_B4SOIrswmin_dX+d_T10_dX);
T4 = (instanceStruct.instanceVar_B4SOIrswmin+T10);
d_rs0_dX = ((PowWeffWr*d_T7_dX-T7*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
rs0 = (T7/PowWeffWr);
d_rswmin_dX = ((PowWeffWr*d_T4_dX-T4*d_PowWeffWr_dX)/PowWeffWr/PowWeffWr);
rswmin = (T4/PowWeffWr);
}
d_ua_dX = (instanceStruct.d_instanceVar_pParam_B4SOIuatemp_dX+(instanceStruct.instanceVar_pParam_B4SOIua1*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIua1_dX*T3));
ua = (instanceStruct.instanceVar_pParam_B4SOIuatemp+(instanceStruct.instanceVar_pParam_B4SOIua1*T3));
d_ub_dX = (instanceStruct.d_instanceVar_pParam_B4SOIubtemp_dX+(instanceStruct.instanceVar_pParam_B4SOIub1*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIub1_dX*T3));
ub = (instanceStruct.instanceVar_pParam_B4SOIubtemp+(instanceStruct.instanceVar_pParam_B4SOIub1*T3));
d_uc_dX = (instanceStruct.d_instanceVar_pParam_B4SOIuctemp_dX+(instanceStruct.instanceVar_pParam_B4SOIuc1*d_T3_dX+instanceStruct.d_instanceVar_pParam_B4SOIuc1_dX*T3));
uc = (instanceStruct.instanceVar_pParam_B4SOIuctemp+(instanceStruct.instanceVar_pParam_B4SOIuc1*T3));
}
else
{
d_vbi_dX = instanceStruct.d_instanceVar_pParam_B4SOIvbi_dX;
vbi = instanceStruct.instanceVar_pParam_B4SOIvbi;
d_vfbb_dX = instanceStruct.d_instanceVar_pParam_B4SOIvfbb_dX;
vfbb = instanceStruct.instanceVar_pParam_B4SOIvfbb;
d_phi_dX = instanceStruct.d_instanceVar_pParam_B4SOIphi_dX;
phi = instanceStruct.instanceVar_pParam_B4SOIphi;
d_sqrtPhi_dX = instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX;
sqrtPhi = instanceStruct.instanceVar_pParam_B4SOIsqrtPhi;
d_Xdep0_dX = instanceStruct.d_instanceVar_pParam_B4SOIXdep0_dX;
Xdep0 = instanceStruct.instanceVar_pParam_B4SOIXdep0;
d_Eg_dX = instanceStruct.d_instanceVar_B4SOIeg_dX;
Eg = instanceStruct.instanceVar_B4SOIeg;
d_cdep0_dX = instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX;
cdep0 = instanceStruct.instanceVar_pParam_B4SOIcdep0;
d_theta0vb0_dX = instanceStruct.d_instanceVar_pParam_B4SOItheta0vb0_dX;
theta0vb0 = instanceStruct.instanceVar_pParam_B4SOItheta0vb0;
d_thetaRout_dX = instanceStruct.d_instanceVar_pParam_B4SOIthetaRout_dX;
thetaRout = instanceStruct.instanceVar_pParam_B4SOIthetaRout;
d_jbjts_dX = instanceStruct.d_instanceVar_pParam_B4SOIjbjts_dX;
jbjts = instanceStruct.instanceVar_pParam_B4SOIjbjts;
d_jbjtd_dX = instanceStruct.d_instanceVar_pParam_B4SOIjbjtd_dX;
jbjtd = instanceStruct.instanceVar_pParam_B4SOIjbjtd;
d_jdifs_dX = instanceStruct.d_instanceVar_pParam_B4SOIjdifs_dX;
jdifs = instanceStruct.instanceVar_pParam_B4SOIjdifs;
d_jdifd_dX = instanceStruct.d_instanceVar_pParam_B4SOIjdifd_dX;
jdifd = instanceStruct.instanceVar_pParam_B4SOIjdifd;
d_jrecs_dX = instanceStruct.d_instanceVar_pParam_B4SOIjrecs_dX;
jrecs = instanceStruct.instanceVar_pParam_B4SOIjrecs;
d_jrecd_dX = instanceStruct.d_instanceVar_pParam_B4SOIjrecd_dX;
jrecd = instanceStruct.instanceVar_pParam_B4SOIjrecd;
d_jtuns_dX = instanceStruct.d_instanceVar_pParam_B4SOIjtuns_dX;
jtuns = instanceStruct.instanceVar_pParam_B4SOIjtuns;
d_jtund_dX = instanceStruct.d_instanceVar_pParam_B4SOIjtund_dX;
jtund = instanceStruct.instanceVar_pParam_B4SOIjtund;
d_Ahlis_dX = instanceStruct.d_instanceVar_pParam_B4SOIahli0s_dX;
Ahlis = instanceStruct.instanceVar_pParam_B4SOIahli0s;
d_Ahlid_dX = instanceStruct.d_instanceVar_pParam_B4SOIahli0d_dX;
Ahlid = instanceStruct.instanceVar_pParam_B4SOIahli0d;
d_u0temp_dX = instanceStruct.d_instanceVar_here_B4SOIu0temp_dX;
u0temp = instanceStruct.instanceVar_here_B4SOIu0temp;
d_vsattemp_dX = instanceStruct.d_instanceVar_here_B4SOIvsattemp_dX;
vsattemp = instanceStruct.instanceVar_here_B4SOIvsattemp;
d_ua_dX = instanceStruct.d_instanceVar_pParam_B4SOIua_dX;
ua = instanceStruct.instanceVar_pParam_B4SOIua;
d_ub_dX = instanceStruct.d_instanceVar_pParam_B4SOIub_dX;
ub = instanceStruct.instanceVar_pParam_B4SOIub;
d_uc_dX = instanceStruct.d_instanceVar_pParam_B4SOIuc_dX;
uc = instanceStruct.instanceVar_pParam_B4SOIuc;
}
if ((modelStruct.modelPar_given_K1||modelStruct.modelPar_given_K2))
{
if ((!modelStruct.modelPar_given_K1))
{
UserInfo(theInstance)  << "Warning: k1 should be specified with k2." << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIk1_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIk1 = 0.53;
}
if ((!modelStruct.modelPar_given_K2))
{
UserInfo(theInstance)  << "Warning: k2 should be specified with k1." << " " <<  std::endl; 
instanceStruct.d_instanceVar_pParam_B4SOIk2_dX = 0.0;
instanceStruct.instanceVar_pParam_B4SOIk2 = (-0.0186);
}
if (modelStruct.modelPar_given_XT)
{
UserInfo(theInstance)  << "Warning: xt is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_VBX)
{
UserInfo(theInstance)  << "Warning: vbx is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_VBM)
{
UserInfo(theInstance)  << "Warning: vbm is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_GAMMA1)
{
UserInfo(theInstance)  << "Warning: gamma1 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
if (modelStruct.modelPar_given_GAMMA2)
{
UserInfo(theInstance)  << "Warning: gamma2 is ignored because k1 or k2 is given." << " " <<  std::endl; 
}
}
else
{
if ((!modelStruct.modelPar_given_VBX))
{
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
d_T0_dX = (-1.60219e-19*2.0*instanceStruct.d_instanceVar_epssub_dX/(2.0*instanceStruct.instanceVar_epssub)/(2.0*instanceStruct.instanceVar_epssub))*1.0e6;
T0 = ((1.60219e-19/(2.0*instanceStruct.instanceVar_epssub))*1.0e6);
}
else
{
d_T0_dX = 0.0;
T0 = 7.7348e-4;
}
instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX = (d_phi_dX-(((T0*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOIxt)*instanceStruct.d_instanceVar_pParam_B4SOIxt_dX+((T0*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.d_instanceVar_pParam_B4SOIxt_dX+(T0*instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX+d_T0_dX*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOIxt)*instanceStruct.instanceVar_pParam_B4SOIxt));
instanceStruct.instanceVar_pParam_B4SOIvbx = (phi-(((T0*instanceStruct.instanceVar_pParam_B4SOInpeak)*instanceStruct.instanceVar_pParam_B4SOIxt)*instanceStruct.instanceVar_pParam_B4SOIxt));
}
if ((instanceStruct.instanceVar_pParam_B4SOIvbx>0.0))
{
instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX);
instanceStruct.instanceVar_pParam_B4SOIvbx = (-instanceStruct.instanceVar_pParam_B4SOIvbx);
}
if ((instanceStruct.instanceVar_pParam_B4SOIvbm>0.0))
{
instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX);
instanceStruct.instanceVar_pParam_B4SOIvbm = (-instanceStruct.instanceVar_pParam_B4SOIvbm);
}
if ((!modelStruct.modelPar_given_GAMMA1))
{
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_pParam_B4SOInpeak);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX = ((instanceStruct.instanceVar_B4SOIcox*(instanceStruct.instanceVar_sqrt2qeps*deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX)+instanceStruct.d_instanceVar_sqrt2qeps_dX*value_sqrt_0)-(instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
instanceStruct.instanceVar_pParam_B4SOIgamma1 = ((instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)/instanceStruct.instanceVar_B4SOIcox);
}
}
if ((!modelStruct.modelPar_given_GAMMA2))
{
{
double value_sqrt_0 = sqrt(instanceStruct.instanceVar_pParam_B4SOInsub);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX = ((instanceStruct.instanceVar_B4SOIcox*(instanceStruct.instanceVar_sqrt2qeps*deriv_sqrt_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOInsub_dX)+instanceStruct.d_instanceVar_sqrt2qeps_dX*value_sqrt_0)-(instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
instanceStruct.instanceVar_pParam_B4SOIgamma2 = ((instanceStruct.instanceVar_sqrt2qeps*value_sqrt_0)/instanceStruct.instanceVar_B4SOIcox);
}
}
d_T0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX-instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX);
T0 = (instanceStruct.instanceVar_pParam_B4SOIgamma1-instanceStruct.instanceVar_pParam_B4SOIgamma2);
{
double value_sqrt_0 = sqrt((phi-instanceStruct.instanceVar_pParam_B4SOIvbx));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = (deriv_sqrt_0_d0*((d_phi_dX-instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX))-d_sqrtPhi_dX);
T1 = (value_sqrt_0-sqrtPhi);
}
{
double value_sqrt_0 = sqrt((phi-instanceStruct.instanceVar_pParam_B4SOIvbm));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = (sqrtPhi*(deriv_sqrt_0_d0*((d_phi_dX-instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX))-d_sqrtPhi_dX)+d_sqrtPhi_dX*(value_sqrt_0-sqrtPhi));
T2 = (sqrtPhi*(value_sqrt_0-sqrtPhi));
}
d_T3_dX = ((((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm)*(T0*d_T1_dX+d_T0_dX*T1)-(T0*T1)*(2.0*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX))/((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm)/((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm));
T3 = ((T0*T1)/((2.0*T2)+instanceStruct.instanceVar_pParam_B4SOIvbm));
instanceStruct.d_instanceVar_here_B4SOIk2_dX = ((instanceStruct.d_instanceVar_here_B4SOIk2_dX-instanceStruct.d_instanceVar_pParam_B4SOIk2_dX)+d_T3_dX);
instanceStruct.instanceVar_here_B4SOIk2 = ((instanceStruct.instanceVar_here_B4SOIk2-instanceStruct.instanceVar_pParam_B4SOIk2)+T3);
{
double value_sqrt_0 = sqrt((phi-instanceStruct.instanceVar_pParam_B4SOIvbm));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
instanceStruct.d_instanceVar_pParam_B4SOIk1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX-((2.0*instanceStruct.instanceVar_here_B4SOIk2)*deriv_sqrt_0_d0*((d_phi_dX-instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX))+2.0*instanceStruct.d_instanceVar_here_B4SOIk2_dX*value_sqrt_0));
instanceStruct.instanceVar_pParam_B4SOIk1 = (instanceStruct.instanceVar_pParam_B4SOIgamma2-((2.0*instanceStruct.instanceVar_here_B4SOIk2)*value_sqrt_0));
}
}
d_T0_dX = (instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1w2_dX);
T0 = (instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIk1w2);
if ((T0<1e-8))
{
d_T0_dX = 0.0;
T0 = 1e-8;
}
instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX = (instanceStruct.instanceVar_pParam_B4SOIk1*((T0*instanceStruct.d_instanceVar_pParam_B4SOIk1w1_dX-instanceStruct.instanceVar_pParam_B4SOIk1w1*d_T0_dX)/T0/T0)+instanceStruct.d_instanceVar_pParam_B4SOIk1_dX*(1+(instanceStruct.instanceVar_pParam_B4SOIk1w1/T0)));
instanceStruct.instanceVar_pParam_B4SOIk1eff = (instanceStruct.instanceVar_pParam_B4SOIk1*(1+(instanceStruct.instanceVar_pParam_B4SOIk1w1/T0)));
if ((!modelStruct.modelPar_given_VFB))
{
if ((modelStruct.modelPar_given_VTH0||modelStruct.modelPar_given_VTHO))
{
instanceStruct.d_instanceVar_here_B4SOIvfb_dX = ((((instanceStruct.d_instanceVar_here_B4SOIvfb_dX-instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX)+instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX)-d_phi_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi));
instanceStruct.instanceVar_here_B4SOIvfb = ((((instanceStruct.instanceVar_here_B4SOIvfb-instanceStruct.instanceVar_pParam_B4SOIvfb)+(instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0))-phi)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi));
}
else
{
}
}
if ((!modelStruct.modelPar_given_VTH0))
{
instanceStruct.d_instanceVar_here_B4SOIvth0_dX = instanceStruct.instanceVar_B4SOItype*((instanceStruct.d_instanceVar_here_B4SOIvfb_dX+d_phi_dX)+(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi));
instanceStruct.instanceVar_here_B4SOIvth0 = (instanceStruct.instanceVar_B4SOItype*((instanceStruct.instanceVar_here_B4SOIvfb+phi)+(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi)));
}
if ((modelStruct.modelPar_VERSION<4.2))
{
d_rd0_dX = instanceStruct.d_instanceVar_pParam_B4SOIrd0_dX;
rd0 = instanceStruct.instanceVar_pParam_B4SOIrd0;
d_rdwmin_dX = instanceStruct.d_instanceVar_pParam_B4SOIrdwmin_dX;
rdwmin = instanceStruct.instanceVar_pParam_B4SOIrdwmin;
d_cdep0_dX = instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX;
cdep0 = instanceStruct.instanceVar_pParam_B4SOIcdep0;
d_theta0vb0_dX = instanceStruct.d_instanceVar_pParam_B4SOItheta0vb0_dX;
theta0vb0 = instanceStruct.instanceVar_pParam_B4SOItheta0vb0;
d_thetaRout_dX = instanceStruct.d_instanceVar_pParam_B4SOIthetaRout_dX;
thetaRout = instanceStruct.instanceVar_pParam_B4SOIthetaRout;
if ((instanceStruct.instanceVar_B4SOImobMod==4))
{
d_ua_dX = instanceStruct.d_instanceVar_pParam_B4SOIua_dX;
ua = instanceStruct.instanceVar_pParam_B4SOIua;
d_uc_dX = instanceStruct.d_instanceVar_pParam_B4SOIuc_dX;
uc = instanceStruct.instanceVar_pParam_B4SOIuc;
}
}
vds = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_di_si]));
vbs = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_b_si]));
vgs = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_gi_si]));
ves = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_e_si]));
vbp = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_b_p]));
vgp = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_gi_p]));
vsbs = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_sb_si]));
vdbd = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_db_di]));
vgms = (instanceStruct.instanceVar_B4SOItype*(probeVars[admsProbeID_V_gm_si]));
vbd = (vbs-vds);
vgd = (vgs-vds);
ved = (ves-vds);
vgmd = (vgms-vds);
if ((vds>=0.0))
{
instanceStruct.instanceVar_B4SOImode = 1;
Vds = vds;
Vgs = vgs;
d_Vbs_dX = 0.0;
Vbs = vbs;
Vbd = vbd;
Ves = ves;
Vgd = vgd;
d_wdios_dX = instanceStruct.d_instanceVar_pParam_B4SOIwdios_dX;
wdios = instanceStruct.instanceVar_pParam_B4SOIwdios;
d_wdiod_dX = instanceStruct.d_instanceVar_pParam_B4SOIwdiod_dX;
wdiod = instanceStruct.instanceVar_pParam_B4SOIwdiod;
d_agidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIagidl_dX;
agidl = instanceStruct.instanceVar_pParam_B4SOIagidl;
d_bgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIbgidl_dX;
bgidl = instanceStruct.instanceVar_pParam_B4SOIbgidl;
d_cgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIcgidl_dX;
cgidl = instanceStruct.instanceVar_pParam_B4SOIcgidl;
d_egidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIegidl_dX;
egidl = instanceStruct.instanceVar_pParam_B4SOIegidl;
d_rgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIrgidl_dX;
rgidl = instanceStruct.instanceVar_pParam_B4SOIrgidl;
d_kgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIkgidl_dX;
kgidl = instanceStruct.instanceVar_pParam_B4SOIkgidl;
d_fgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIfgidl_dX;
fgidl = instanceStruct.instanceVar_pParam_B4SOIfgidl;
d_agisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIagisl_dX;
agisl = instanceStruct.instanceVar_pParam_B4SOIagisl;
d_bgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIbgisl_dX;
bgisl = instanceStruct.instanceVar_pParam_B4SOIbgisl;
d_cgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIcgisl_dX;
cgisl = instanceStruct.instanceVar_pParam_B4SOIcgisl;
d_egisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIegisl_dX;
egisl = instanceStruct.instanceVar_pParam_B4SOIegisl;
d_rgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIrgisl_dX;
rgisl = instanceStruct.instanceVar_pParam_B4SOIrgisl;
d_kgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIkgisl_dX;
kgisl = instanceStruct.instanceVar_pParam_B4SOIkgisl;
d_fgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIfgisl_dX;
fgisl = instanceStruct.instanceVar_pParam_B4SOIfgisl;
}
else
{
instanceStruct.instanceVar_B4SOImode = (-1);
Vds = (-vds);
Vgs = vgd;
d_Vbs_dX = 0.0;
Vbs = vbd;
Vbd = vbs;
Ves = ved;
Vgd = vgs;
d_wdios_dX = instanceStruct.d_instanceVar_pParam_B4SOIwdiod_dX;
wdios = instanceStruct.instanceVar_pParam_B4SOIwdiod;
d_wdiod_dX = instanceStruct.d_instanceVar_pParam_B4SOIwdios_dX;
wdiod = instanceStruct.instanceVar_pParam_B4SOIwdios;
d_agidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIagisl_dX;
agidl = instanceStruct.instanceVar_pParam_B4SOIagisl;
d_bgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIbgisl_dX;
bgidl = instanceStruct.instanceVar_pParam_B4SOIbgisl;
d_cgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIcgisl_dX;
cgidl = instanceStruct.instanceVar_pParam_B4SOIcgisl;
d_egidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIegisl_dX;
egidl = instanceStruct.instanceVar_pParam_B4SOIegisl;
d_rgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIrgisl_dX;
rgidl = instanceStruct.instanceVar_pParam_B4SOIrgisl;
d_kgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIkgisl_dX;
kgidl = instanceStruct.instanceVar_pParam_B4SOIkgisl;
d_fgidl_dX = instanceStruct.d_instanceVar_pParam_B4SOIfgisl_dX;
fgidl = instanceStruct.instanceVar_pParam_B4SOIfgisl;
d_agisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIagidl_dX;
agisl = instanceStruct.instanceVar_pParam_B4SOIagidl;
d_bgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIbgidl_dX;
bgisl = instanceStruct.instanceVar_pParam_B4SOIbgidl;
d_cgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIcgidl_dX;
cgisl = instanceStruct.instanceVar_pParam_B4SOIcgidl;
d_egisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIegidl_dX;
egisl = instanceStruct.instanceVar_pParam_B4SOIegidl;
d_rgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIrgidl_dX;
rgisl = instanceStruct.instanceVar_pParam_B4SOIrgidl;
d_kgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIkgidl_dX;
kgisl = instanceStruct.instanceVar_pParam_B4SOIkgidl;
d_fgisl_dX = instanceStruct.d_instanceVar_pParam_B4SOIfgidl_dX;
fgisl = instanceStruct.instanceVar_pParam_B4SOIfgidl;
}
d_Vesfb_dX = (-d_vfbb_dX);
Vesfb = (Ves-vfbb);
d_Cbox_dX = instanceStruct.d_instanceVar_B4SOIcbox_dX;
Cbox = instanceStruct.instanceVar_B4SOIcbox;
d_T0_dX = (instanceStruct.d_instanceVar_here_B4SOIvfb_dX+d_phi_dX);
T0 = (instanceStruct.instanceVar_here_B4SOIvfb+phi);
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_epsgate_dX = instanceStruct.d_instanceVar_epssub_dX;
epsgate = instanceStruct.instanceVar_epssub;
}
else
{
d_epsgate_dX = instanceStruct.d_instanceVar_B4SOIepsrgate_dX*8.85418e-12;
epsgate = (instanceStruct.instanceVar_B4SOIepsrgate*8.85418e-12);
}
if (((((instanceStruct.instanceVar_pParam_B4SOIngate>1.0e18)&&(instanceStruct.instanceVar_pParam_B4SOIngate<1.0e25))&&(Vgs>T0))&&(epsgate!=0)))
{
d_T1_dX = (((instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox)*(((1.0e6*1.60219e-19)*epsgate)*instanceStruct.d_instanceVar_pParam_B4SOIngate_dX+(1.0e6*1.60219e-19)*d_epsgate_dX*instanceStruct.instanceVar_pParam_B4SOIngate)-(((1.0e6*1.60219e-19)*epsgate)*instanceStruct.instanceVar_pParam_B4SOIngate)*(instanceStruct.instanceVar_B4SOIcox*instanceStruct.d_instanceVar_B4SOIcox_dX+instanceStruct.d_instanceVar_B4SOIcox_dX*instanceStruct.instanceVar_B4SOIcox))/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox)/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox));
T1 = ((((1.0e6*1.60219e-19)*epsgate)*instanceStruct.instanceVar_pParam_B4SOIngate)/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox));
{
double value_sqrt_0 = sqrt((1.0+((2.0*(Vgs-T0))/T1)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T4_dX = deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dX)-(2.0*(Vgs-T0))*d_T1_dX)/T1/T1));
T4 = value_sqrt_0;
}
d_T2_dX = (T1*d_T4_dX+d_T1_dX*(T4-1.0));
T2 = (T1*(T4-1.0));
d_T3_dX = ((T1*((0.5*T2)*d_T2_dX+0.5*d_T2_dX*T2)-((0.5*T2)*T2)*d_T1_dX)/T1/T1);
T3 = (((0.5*T2)*T2)/T1);
d_T7_dX = (instanceStruct.d_instanceVar_eggdep_dX-d_T3_dX);
T7 = ((instanceStruct.instanceVar_eggdep-T3)-0.05);
{
double value_sqrt_0 = sqrt(((T7*T7)+0.224));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T6_dX = deriv_sqrt_0_d0*((T7*d_T7_dX+d_T7_dX*T7));
T6 = value_sqrt_0;
}
d_T5_dX = (instanceStruct.d_instanceVar_eggdep_dX-0.5*(d_T7_dX+d_T6_dX));
T5 = (instanceStruct.instanceVar_eggdep-(0.5*(T7+T6)));
d_Vgs_eff_dX = (-d_T5_dX);
Vgs_eff = (Vgs-T5);
}
else
{
d_Vgs_eff_dX = 0.0;
Vgs_eff = Vgs;
}
if (((((instanceStruct.instanceVar_pParam_B4SOIngate>1.0e18)&&(instanceStruct.instanceVar_pParam_B4SOIngate<1.0e25))&&(Vgd>T0))&&(epsgate!=0)))
{
d_T1_dX = (((instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox)*(((1.0e6*1.60219e-19)*epsgate)*instanceStruct.d_instanceVar_pParam_B4SOIngate_dX+(1.0e6*1.60219e-19)*d_epsgate_dX*instanceStruct.instanceVar_pParam_B4SOIngate)-(((1.0e6*1.60219e-19)*epsgate)*instanceStruct.instanceVar_pParam_B4SOIngate)*(instanceStruct.instanceVar_B4SOIcox*instanceStruct.d_instanceVar_B4SOIcox_dX+instanceStruct.d_instanceVar_B4SOIcox_dX*instanceStruct.instanceVar_B4SOIcox))/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox)/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox));
T1 = ((((1.0e6*1.60219e-19)*epsgate)*instanceStruct.instanceVar_pParam_B4SOIngate)/(instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIcox));
{
double value_sqrt_0 = sqrt((1.0+((2.0*(Vgd-T0))/T1)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T4_dX = deriv_sqrt_0_d0*(((T1*2.0*(-d_T0_dX)-(2.0*(Vgd-T0))*d_T1_dX)/T1/T1));
T4 = value_sqrt_0;
}
d_T2_dX = (T1*d_T4_dX+d_T1_dX*(T4-1.0));
T2 = (T1*(T4-1.0));
d_T3_dX = ((T1*((0.5*T2)*d_T2_dX+0.5*d_T2_dX*T2)-((0.5*T2)*T2)*d_T1_dX)/T1/T1);
T3 = (((0.5*T2)*T2)/T1);
d_T7_dX = (instanceStruct.d_instanceVar_eggdep_dX-d_T3_dX);
T7 = ((instanceStruct.instanceVar_eggdep-T3)-0.05);
{
double value_sqrt_0 = sqrt(((T7*T7)+0.224));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T6_dX = deriv_sqrt_0_d0*((T7*d_T7_dX+d_T7_dX*T7));
T6 = value_sqrt_0;
}
d_T5_dX = (instanceStruct.d_instanceVar_eggdep_dX-0.5*(d_T7_dX+d_T6_dX));
T5 = (instanceStruct.instanceVar_eggdep-(0.5*(T7+T6)));
d_Vgd_eff_dX = (-d_T5_dX);
Vgd_eff = (Vgd-T5);
}
else
{
d_Vgd_eff_dX = 0.0;
Vgd_eff = Vgd;
}
Vgs_eff2 = Vgs;
d_Leff_dX = instanceStruct.d_instanceVar_pParam_B4SOIleff_dX;
Leff = instanceStruct.instanceVar_pParam_B4SOIleff;
if (((instanceStruct.instancePar_SHMOD==1)&&(instanceStruct.instancePar_RTH0!=0.0)))
{
d_Vtm_dX = 8.617087e-5*instanceStruct.d_instanceVar_DevTemp_dX;
Vtm = (8.617087e-5*instanceStruct.instanceVar_DevTemp);
}
else
{
d_Vtm_dX = instanceStruct.d_instanceVar_B4SOIvtm_dX;
Vtm = instanceStruct.instanceVar_B4SOIvtm;
}
d_V0_dX = (d_vbi_dX-d_phi_dX);
V0 = (vbi-phi);
if ((instanceStruct.instanceVar_B4SOIsoiMod==0))
{
d_Vbsmos_dX = d_Vbs_dX;
Vbsmos = Vbs;
d_Vbsmos_CV_dX = d_Vbs_dX;
Vbsmos_CV = Vbs;
}
else
{
if ((instanceStruct.instanceVar_B4SOIfdMod==0))
{
d_T0_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T0 = (((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T0));
double value_exp_1 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T1_dX = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(deriv_exp_0_d0*(0.5*d_T0_dX)+2*deriv_exp_1_d0*(d_T0_dX))+instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX*(value_exp_0+(2*value_exp_1)));
T1 = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}
d_T2_dX = (T1*(d_vbi_dX-d_phi_dX)+d_T1_dX*(vbi-phi));
T2 = (T1*(vbi-phi));
d_T3_dX = ((instanceStruct.instanceVar_B4SOIcsi*0.5*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX-(0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi);
T3 = ((0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)/instanceStruct.instanceVar_B4SOIcsi);
d_Vbs0t_dX = (((d_phi_dX-d_T3_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX)+d_T2_dX);
Vbs0t = (((phi-T3)+instanceStruct.instanceVar_pParam_B4SOIvbsa)+T2);
d_T0_dX = ((Cbox*instanceStruct.d_instanceVar_B4SOIcsi_dX-instanceStruct.instanceVar_B4SOIcsi*d_Cbox_dX)/Cbox/Cbox);
T0 = (1+(instanceStruct.instanceVar_B4SOIcsi/Cbox));
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdk2b_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T3 = (((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T3));
double value_exp_1 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T5_dX = (instanceStruct.instanceVar_pParam_B4SOIk2b*(deriv_exp_0_d0*(0.5*d_T3_dX)+2*deriv_exp_1_d0*(d_T3_dX))+instanceStruct.d_instanceVar_pParam_B4SOIk2b_dX*(value_exp_0+(2*value_exp_1)));
T5 = (instanceStruct.instanceVar_pParam_B4SOIk2b*(value_exp_0+(2*value_exp_1)));
}
d_T1_dX = ((T0*(instanceStruct.d_instanceVar_pParam_B4SOIk1b_dX-d_T5_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1b-T5)*d_T0_dX)/T0/T0);
T1 = ((instanceStruct.instanceVar_pParam_B4SOIk1b-T5)/T0);
d_T2_dX = (T1*d_Vesfb_dX+d_T1_dX*Vesfb);
T2 = (T1*Vesfb);
d_T4_dX = (-((instanceStruct.instanceVar_B4SOIcsi*d_Cbox_dX-Cbox*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi)/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi))/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi)));
T4 = (1.0/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi)));
d_Vbs0_dX = ((T4*d_Vbs0t_dX+d_T4_dX*Vbs0t)+d_T2_dX);
Vbs0 = ((T4*Vbs0t)+T2);
}
else
{
d_T0_dX = (-((instanceStruct.d_instanceVar_B4SOIcsi_dX+d_Cbox_dX)+instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX)/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs)/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs));
T0 = (1.0/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs));
d_T1_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T1 = (((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T1));
double value_exp_1 = exp(T1);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T2_dX = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(deriv_exp_0_d0*(0.5*d_T1_dX)+2*deriv_exp_1_d0*(d_T1_dX))+instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX*(value_exp_0+(2*value_exp_1)));
T2 = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}
d_T3_dX = (T2*instanceStruct.d_instanceVar_pParam_B4SOIvsce_dX+d_T2_dX*(Vds+instanceStruct.instanceVar_pParam_B4SOIvsce));
T3 = (T2*(Vds+instanceStruct.instanceVar_pParam_B4SOIvsce));
d_T4_dX = ((instanceStruct.instanceVar_B4SOIcsi*0.5*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX-(0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi);
T4 = ((0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)/instanceStruct.instanceVar_B4SOIcsi);
d_T5_dX = ((instanceStruct.instanceVar_B4SOIcsi*T0)*((d_phi_dX-d_T4_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX)+(instanceStruct.instanceVar_B4SOIcsi*d_T0_dX+instanceStruct.d_instanceVar_B4SOIcsi_dX*T0)*((phi-T4)+instanceStruct.instanceVar_pParam_B4SOIvbsa));
T5 = ((instanceStruct.instanceVar_B4SOIcsi*T0)*((phi-T4)+instanceStruct.instanceVar_pParam_B4SOIvbsa));
d_T6_dX = ((instanceStruct.instanceVar_pParam_B4SOIcdsbs*T0)*d_T3_dX+(instanceStruct.instanceVar_pParam_B4SOIcdsbs*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX*T0)*T3);
T6 = ((instanceStruct.instanceVar_pParam_B4SOIcdsbs*T0)*T3);
d_Vbs0t_dX = (d_T5_dX+d_T6_dX);
Vbs0t = (T5+T6);
d_T7_dX = ((Cbox*T0)*d_Vesfb_dX+(Cbox*d_T0_dX+d_Cbox_dX*T0)*Vesfb);
T7 = ((Cbox*T0)*Vesfb);
d_Vbs0_dX = (d_Vbs0t_dX+d_T7_dX);
Vbs0 = (Vbs0t+T7);
}
d_T1_dX = (d_Vbs0t_dX-d_Vbs0_dX);
T1 = ((Vbs0t-Vbs0)-0.005);
{
double value_sqrt_0 = sqrt(((T1*T1)+2.5e-5));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*((T1*d_T1_dX+d_T1_dX*T1));
T2 = value_sqrt_0;
}
d_T3_dX = 0.5*(d_T1_dX+d_T2_dX);
T3 = (0.5*(T1+T2));
d_T4_dX = ((instanceStruct.instanceVar_pParam_B4SOIqsi*(T3*instanceStruct.d_instanceVar_B4SOIcsi_dX+d_T3_dX*instanceStruct.instanceVar_B4SOIcsi)-(T3*instanceStruct.instanceVar_B4SOIcsi)*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX)/instanceStruct.instanceVar_pParam_B4SOIqsi/instanceStruct.instanceVar_pParam_B4SOIqsi);
T4 = ((T3*instanceStruct.instanceVar_B4SOIcsi)/instanceStruct.instanceVar_pParam_B4SOIqsi);
d_Vbs0mos_dX = (d_Vbs0_dX-((0.5*T3)*d_T4_dX+0.5*d_T3_dX*T4));
Vbs0mos = (Vbs0-((0.5*T3)*T4));
d_T1_dX = d_phi_dX;
T1 = (phi-0.02);
d_T2_dX = (d_T1_dX-d_Vbs0mos_dX);
T2 = ((T1-Vbs0mos)-0.005);
{
double value_sqrt_0 = sqrt(((T2*T2)+(4.0*0.005)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*((T2*d_T2_dX+d_T2_dX*T2));
T3 = value_sqrt_0;
}
d_Vbs0mos_dX = (d_T1_dX-0.5*(d_T2_dX+d_T3_dX));
Vbs0mos = (T1-(0.5*(T2+T3)));
d_Phis_dX = (d_phi_dX-d_Vbs0mos_dX);
Phis = (phi-Vbs0mos);
{
double value_sqrt_0 = sqrt(Phis);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_sqrtPhis_dX = deriv_sqrt_0_d0*(d_Phis_dX);
sqrtPhis = value_sqrt_0;
}
d_Xdep_dX = ((sqrtPhi*(Xdep0*d_sqrtPhis_dX+d_Xdep0_dX*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dX)/sqrtPhi/sqrtPhi);
Xdep = ((Xdep0*sqrtPhis)/sqrtPhi);
{
double value_sqrt_0 = sqrt(Xdep);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(d_Xdep_dX);
T3 = value_sqrt_0;
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt2*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt2_dX*Vbs0mos);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt2*Vbs0mos);
if ((T0>=(-0.5)))
{
d_T1_dX = d_T0_dX;
T1 = (1.0+T0);
}
else
{
d_T4_dX = (-8.0*d_T0_dX/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));
d_T1_dX = ((1.0+(3.0*T0))*d_T4_dX+3.0*d_T0_dX*T4);
T1 = ((1.0+(3.0*T0))*T4);
}
d_lt1_dX = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*d_T1_dX+(instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3)*T1);
lt1 = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*T1);
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt2w*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt2w_dX*Vbs0mos);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt2w*Vbs0mos);
if ((T0>=(-0.5)))
{
d_T1_dX = d_T0_dX;
T1 = (1.0+T0);
}
else
{
d_T4_dX = (-8.0*d_T0_dX/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));
d_T1_dX = ((1.0+(3.0*T0))*d_T4_dX+3.0*d_T0_dX*T4);
T1 = ((1.0+(3.0*T0))*T4);
}
d_ltw_dX = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*d_T1_dX+(instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3)*T1);
ltw = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*T1);
d_T0_dX = ((lt1*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*d_Leff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX*Leff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)*d_lt1_dX)/lt1/lt1);
T0 = ((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)/lt1);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_Theta0_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_Theta0_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
d_T2_dX = ((Xdep*(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.d_instanceVar_epssub_dX+instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX*instanceStruct.instanceVar_epssub)-(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)*d_Xdep_dX)/Xdep/Xdep);
T2 = ((instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)/Xdep);
d_T3_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX+(instanceStruct.instanceVar_pParam_B4SOIcdscb*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIcdscb_dX*Vbs0mos))+instanceStruct.d_instanceVar_pParam_B4SOIcdscd_dX*Vds);
T3 = ((instanceStruct.instanceVar_pParam_B4SOIcdsc+(instanceStruct.instanceVar_pParam_B4SOIcdscb*Vbs0mos))+(instanceStruct.instanceVar_pParam_B4SOIcdscd*Vds));
d_T4_dX = ((instanceStruct.instanceVar_B4SOIcox*((d_T2_dX+(T3*d_Theta0_dX+d_T3_dX*Theta0))+instanceStruct.d_instanceVar_pParam_B4SOIcit_dX)-((T2+(T3*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
T4 = (((T2+(T3*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)/instanceStruct.instanceVar_B4SOIcox);
if ((T4>=(-0.5)))
{
d_n_dX = d_T4_dX;
n = (1.0+T4);
}
else
{
d_T0_dX = (-8.0*d_T4_dX/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
T0 = (1.0/(3.0+(8.0*T4)));
d_n_dX = ((1.0+(3.0*T4))*d_T0_dX+3.0*d_T4_dX*T0);
n = ((1.0+(3.0*T4))*T0);
}
if ((instanceStruct.instanceVar_pParam_B4SOIdvtp0>0.0))
{
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIdvtp1_dX)*Vds;
T0 = ((-instanceStruct.instanceVar_pParam_B4SOIdvtp1)*Vds);
if ((T0<(-100.0)))
{
d_T2_dX = 0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T2_dX = deriv_exp_0_d0*(d_T0_dX);
T2 = value_exp_0;
}
}
d_T3_dX = (d_Leff_dX+(instanceStruct.instanceVar_pParam_B4SOIdvtp0*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvtp0_dX*(1.0+T2)));
T3 = (Leff+(instanceStruct.instanceVar_pParam_B4SOIdvtp0*(1.0+T2)));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((Leff/T3));
d_T4_dX = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),((T3*d_Leff_dX-Leff*d_T3_dX)/T3/T3))+d_Vtm_dX*evaluator_lln_0.getValues((Leff/T3)));
T4 = (Vtm*evaluator_lln_0.getValues((Leff/T3)));
}
d_DITS_Sft_dX = (n*d_T4_dX+d_n_dX*T4);
DITS_Sft = (n*T4);
}
else
{
d_DITS_Sft_dX = 0.0;
DITS_Sft = 0.0;
}
instanceStruct.d_instanceVar_B4SOIthetavth_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0*d_Theta0_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX*Theta0);
instanceStruct.instanceVar_B4SOIthetavth = (instanceStruct.instanceVar_pParam_B4SOIdvt0*Theta0);
d_Delt_vth_dX = (instanceStruct.instanceVar_B4SOIthetavth*d_V0_dX+instanceStruct.d_instanceVar_B4SOIthetavth_dX*V0);
Delt_vth = (instanceStruct.instanceVar_B4SOIthetavth*V0);
d_T0_dX = ((ltw*((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*d_Leff_dX+(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)-((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)*d_ltw_dX)/ltw/ltw);
T0 = (((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)/ltw);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX*T2);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*T2);
d_DeltVthw_dX = (T0*d_V0_dX+d_T0_dX*V0);
DeltVthw = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/Leff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX-instanceStruct.instanceVar_pParam_B4SOIlpe0*d_Leff_dX)/Leff/Leff));
T0 = value_sqrt_0;
}
d_T1_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX+((Leff*instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX-instanceStruct.instanceVar_pParam_B4SOIkt1l*d_Leff_dX)/Leff/Leff))+(instanceStruct.instanceVar_pParam_B4SOIkt2*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIkt2_dX*Vbs0mos));
T1 = ((instanceStruct.instanceVar_pParam_B4SOIkt1+(instanceStruct.instanceVar_pParam_B4SOIkt1l/Leff))+(instanceStruct.instanceVar_pParam_B4SOIkt2*Vbs0mos));
d_DeltVthtemp_dX = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dX+(instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*(T0-1.0))*sqrtPhi)+(T1*d_TempRatioMinus1_dX+d_T1_dX*TempRatioMinus1));
DeltVthtemp = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));
d_tmp2_dX = (((instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)*(instanceStruct.instanceVar_toxe*d_phi_dX+instanceStruct.d_instanceVar_toxe_dX*phi)-(instanceStruct.instanceVar_toxe*phi)*(instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIw0_dX))/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
tmp2 = ((instanceStruct.instanceVar_toxe*phi)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
d_T3_dX = (instanceStruct.d_instanceVar_here_B4SOIeta0_dX+(instanceStruct.instanceVar_pParam_B4SOIetab*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIetab_dX*Vbs0mos));
T3 = (instanceStruct.instanceVar_here_B4SOIeta0+(instanceStruct.instanceVar_pParam_B4SOIetab*Vbs0mos));
if ((T3<1.0e-4))
{
d_T9_dX = (-(-2.0e4*d_T3_dX)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));
d_T3_dX = ((2.0e-4-T3)*d_T9_dX+(-d_T3_dX)*T9);
T3 = ((2.0e-4-T3)*T9);
}
d_DIBL_Sft_dX = (T3*d_theta0vb0_dX+d_T3_dX*theta0vb0)*Vds;
DIBL_Sft = ((T3*theta0vb0)*Vds);
d_T3_dX = (instanceStruct.d_instanceVar_here_B4SOIeta0cv_dX+(instanceStruct.instanceVar_pParam_B4SOIetabcv*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIetabcv_dX*Vbs0mos));
T3 = (instanceStruct.instanceVar_here_B4SOIeta0cv+(instanceStruct.instanceVar_pParam_B4SOIetabcv*Vbs0mos));
if ((T3<1.0e-4))
{
d_T9_dX = (-(-2.0e4*d_T3_dX)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));
d_T3_dX = ((2.0e-4-T3)*d_T9_dX+(-d_T3_dX)*T9);
T3 = ((2.0e-4-T3)*T9);
}
d_DIBL_Sft_CV_dX = (T3*d_theta0vb0_dX+d_T3_dX*theta0vb0)*Vds;
DIBL_Sft_CV = ((T3*theta0vb0)*Vds);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpeb/Leff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_Lpe_Vb_dX = deriv_sqrt_0_d0*(((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlpeb_dX-instanceStruct.instanceVar_pParam_B4SOIlpeb*d_Leff_dX)/Leff/Leff));
Lpe_Vb = value_sqrt_0;
}
{
double value_exp_0 = exp(((2.0*instanceStruct.instanceVar_pParam_B4SOIdvtp4)*Vds));
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(2.0*instanceStruct.d_instanceVar_pParam_B4SOIdvtp4_dX*Vds);
T0 = value_exp_0;
}
d_DITS_Sft2_dX = (((T0+1)*(instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvtp2factor_dX*(T0-1))-(instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dX)/(T0+1)/(T0+1));
DITS_Sft2 = ((instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*(T0-1))/(T0+1));
d_VthFD_dX = (((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhis)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*d_Lpe_Vb_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_sqrtPhis_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*sqrtPhis)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*d_Vbs0mos_dX+instanceStruct.d_instanceVar_here_B4SOIk2ox_dX*Vbs0mos))-d_Delt_vth_dX)-d_DeltVthw_dX)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dX+(instanceStruct.d_instanceVar_pParam_B4SOIk3_dX+(instanceStruct.instanceVar_pParam_B4SOIk3b*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIk3b_dX*Vbs0mos))*tmp2))+d_DeltVthtemp_dX)-d_DIBL_Sft_dX)-d_DITS_Sft_dX)-d_DITS_Sft2_dX);
VthFD = ((((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhis)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*Vbs0mos))-Delt_vth)-DeltVthw)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbs0mos))*tmp2))+DeltVthtemp)-DIBL_Sft)-DITS_Sft)-DITS_Sft2);
d_VthFD_CV_dX = (((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhis)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*d_Lpe_Vb_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_sqrtPhis_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*sqrtPhis)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*d_Vbs0mos_dX+instanceStruct.d_instanceVar_here_B4SOIk2ox_dX*Vbs0mos))-d_Delt_vth_dX)-d_DeltVthw_dX)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbs0mos))*d_tmp2_dX+(instanceStruct.d_instanceVar_pParam_B4SOIk3_dX+(instanceStruct.instanceVar_pParam_B4SOIk3b*d_Vbs0mos_dX+instanceStruct.d_instanceVar_pParam_B4SOIk3b_dX*Vbs0mos))*tmp2))+d_DeltVthtemp_dX)-d_DIBL_Sft_CV_dX)-d_DITS_Sft_dX)-d_DITS_Sft2_dX);
VthFD_CV = ((((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhis)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*Vbs0mos))-Delt_vth)-DeltVthw)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbs0mos))*tmp2))+DeltVthtemp)-DIBL_Sft_CV)-DITS_Sft)-DITS_Sft2);
d_VtgsFD_dX = (d_VthFD_dX-d_Vgs_eff_dX);
VtgsFD = (VthFD-Vgs_eff);
d_T10_dX = (instanceStruct.instanceVar_pParam_B4SOInofffd*d_Vtm_dX+instanceStruct.d_instanceVar_pParam_B4SOInofffd_dX*Vtm);
T10 = (instanceStruct.instanceVar_pParam_B4SOInofffd*Vtm);
if ((((VtgsFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)>100.0))
{
d_ExpVtgsFD_dX = 2.688117142e+43*((T10*(d_VtgsFD_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VtgsFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10);
ExpVtgsFD = (2.688117142e+43*((1.0+((VtgsFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VtgsFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)<(-100.0)))
{
d_ExpVtgsFD_dX = 0.0;
ExpVtgsFD = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VtgsFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVtgsFD_dX = deriv_exp_0_d0*(((T10*(d_VtgsFD_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VtgsFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10));
ExpVtgsFD = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVtgsFD));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVtgsFD));
d_VtgseffFD_dX = (T10*deriv_log_0_d0*(d_ExpVtgsFD_dX)+d_T10_dX*value_log_0);
VtgseffFD = (T10*value_log_0);
}
d_VgstFD_dX = (d_Vgs_eff_dX-d_VthFD_dX);
VgstFD = (Vgs_eff-VthFD);
if ((((VgstFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)>100.0))
{
d_ExpVgstFD_dX = 2.688117142e+43*((T10*(d_VgstFD_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VgstFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10);
ExpVgstFD = (2.688117142e+43*((1.0+((VgstFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VgstFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)<(-100.0)))
{
d_ExpVgstFD_dX = 0.0;
ExpVgstFD = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VgstFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgstFD_dX = deriv_exp_0_d0*(((T10*(d_VgstFD_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VgstFD-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10));
ExpVgstFD = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVgstFD));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVgstFD));
d_VgsteffFD_dX = (T10*deriv_log_0_d0*(d_ExpVgstFD_dX)+d_T10_dX*value_log_0);
VgsteffFD = (T10*value_log_0);
}
d_T1_dX = (((instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Vtm)*d_Vtm_dX+((instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.instanceVar_pParam_B4SOIk1ox)*d_Vtm_dX+(instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+instanceStruct.d_instanceVar_pParam_B4SOImoinfd_dX*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Vtm)*Vtm);
T1 = (((instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Vtm)*Vtm);
{
double value_sqrt_0 = sqrt(phi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = (d_VgsteffFD_dX+((2*instanceStruct.instanceVar_pParam_B4SOIk1eff)*deriv_sqrt_0_d0*(d_phi_dX)+2*instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*value_sqrt_0));
T2 = (VgsteffFD+((2*instanceStruct.instanceVar_pParam_B4SOIk1eff)*value_sqrt_0));
}
d_T0_dX = ((T1*(VgsteffFD*d_T2_dX+d_VgsteffFD_dX*T2)-(VgsteffFD*T2)*d_T1_dX)/T1/T1);
T0 = (1+((VgsteffFD*T2)/T1));
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
d_PhiON_dX = (d_phi_dX+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dX)+d_Vtm_dX*evaluator_lln_0.getValues(T0)));
PhiON = (phi+(Vtm*evaluator_lln_0.getValues(T0)));
}
d_T0_dX = (((instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox))))*instanceStruct.d_instanceVar_B4SOIcox_dX-instanceStruct.instanceVar_B4SOIcox*(instanceStruct.d_instanceVar_B4SOIcox_dX+(-((-instanceStruct.d_instanceVar_B4SOIcsi_dX/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi)+(-d_Cbox_dX/Cbox/Cbox))/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox))/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox)))))/(instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox))))/(instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox)))));
T0 = (instanceStruct.instanceVar_B4SOIcox/(instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox)))));
d_PhiFD_dX = (d_PhiON_dX-(T0*d_VtgseffFD_dX+d_T0_dX*VtgseffFD));
PhiFD = (PhiON-(T0*VtgseffFD));
if ((instanceStruct.instanceVar_B4SOIfdMod==0))
{
d_T0_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T0 = (((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T0));
double value_exp_1 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T1_dX = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(deriv_exp_0_d0*(0.5*d_T0_dX)+2*deriv_exp_1_d0*(d_T0_dX))+instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX*(value_exp_0+(2*value_exp_1)));
T1 = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}
d_T2_dX = (T1*(d_vbi_dX-d_phi_dX)+d_T1_dX*(vbi-phi));
T2 = (T1*(vbi-phi));
d_T3_dX = ((instanceStruct.instanceVar_B4SOIcsi*0.5*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX-(0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi);
T3 = ((0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)/instanceStruct.instanceVar_B4SOIcsi);
d_Vbs0t_dX = (((d_PhiFD_dX-d_T3_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX)+d_T2_dX);
Vbs0t = (((PhiFD-T3)+instanceStruct.instanceVar_pParam_B4SOIvbsa)+T2);
d_T0_dX = ((Cbox*instanceStruct.d_instanceVar_B4SOIcsi_dX-instanceStruct.instanceVar_B4SOIcsi*d_Cbox_dX)/Cbox/Cbox);
T0 = (1+(instanceStruct.instanceVar_B4SOIcsi/Cbox));
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdk2b_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T3 = (((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T3));
double value_exp_1 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T5_dX = (instanceStruct.instanceVar_pParam_B4SOIk2b*(deriv_exp_0_d0*(0.5*d_T3_dX)+2*deriv_exp_1_d0*(d_T3_dX))+instanceStruct.d_instanceVar_pParam_B4SOIk2b_dX*(value_exp_0+(2*value_exp_1)));
T5 = (instanceStruct.instanceVar_pParam_B4SOIk2b*(value_exp_0+(2*value_exp_1)));
}
d_T1_dX = ((T0*(instanceStruct.d_instanceVar_pParam_B4SOIk1b_dX-d_T5_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1b-T5)*d_T0_dX)/T0/T0);
T1 = ((instanceStruct.instanceVar_pParam_B4SOIk1b-T5)/T0);
d_T2_dX = (T1*d_Vesfb_dX+d_T1_dX*Vesfb);
T2 = (T1*Vesfb);
d_T0_dX = (-((instanceStruct.instanceVar_B4SOIcsi*d_Cbox_dX-Cbox*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi)/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi))/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi)));
T0 = (1.0/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi)));
d_Vbs0_dX = ((T0*d_Vbs0t_dX+d_T0_dX*Vbs0t)+d_T2_dX);
Vbs0 = ((T0*Vbs0t)+T2);
}
else
{
d_T0_dX = (-((instanceStruct.d_instanceVar_B4SOIcsi_dX+d_Cbox_dX)+instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX)/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs)/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs));
T0 = (1.0/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs));
d_T1_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T1 = (((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T1));
double value_exp_1 = exp(T1);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T2_dX = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(deriv_exp_0_d0*(0.5*d_T1_dX)+2*deriv_exp_1_d0*(d_T1_dX))+instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX*(value_exp_0+(2*value_exp_1)));
T2 = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}
d_T3_dX = (T2*instanceStruct.d_instanceVar_pParam_B4SOIvsce_dX+d_T2_dX*(Vds+instanceStruct.instanceVar_pParam_B4SOIvsce));
T3 = (T2*(Vds+instanceStruct.instanceVar_pParam_B4SOIvsce));
d_T4_dX = ((instanceStruct.instanceVar_B4SOIcsi*0.5*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX-(0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi);
T4 = ((0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)/instanceStruct.instanceVar_B4SOIcsi);
d_T5_dX = ((instanceStruct.instanceVar_B4SOIcsi*T0)*((d_PhiFD_dX-d_T4_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX)+(instanceStruct.instanceVar_B4SOIcsi*d_T0_dX+instanceStruct.d_instanceVar_B4SOIcsi_dX*T0)*((PhiFD-T4)+instanceStruct.instanceVar_pParam_B4SOIvbsa));
T5 = ((instanceStruct.instanceVar_B4SOIcsi*T0)*((PhiFD-T4)+instanceStruct.instanceVar_pParam_B4SOIvbsa));
d_T6_dX = ((instanceStruct.instanceVar_pParam_B4SOIcdsbs*T0)*d_T3_dX+(instanceStruct.instanceVar_pParam_B4SOIcdsbs*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX*T0)*T3);
T6 = ((instanceStruct.instanceVar_pParam_B4SOIcdsbs*T0)*T3);
d_Vbs0t_dX = (d_T5_dX+d_T6_dX);
Vbs0t = (T5+T6);
d_T7_dX = ((Cbox*T0)*d_Vesfb_dX+(Cbox*d_T0_dX+d_Cbox_dX*T0)*Vesfb);
T7 = ((Cbox*T0)*Vesfb);
d_Vbs0_dX = (d_Vbs0t_dX+d_T7_dX);
Vbs0 = (Vbs0t+T7);
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_Vbsitf_dX = d_Vbs0_dX;
Vbsitf = (Vbs0+0.02);
d_Vbs_dX = d_Vbs0_dX;
Vbs = (Vbs0+0.02);
}
else
{
d_T1_dX = (d_Vbs_dX-d_Vbs0_dX);
T1 = ((Vbs-(Vbs0+0.02))-0.01);
{
double value_sqrt_0 = sqrt(((T1*T1)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*((T1*d_T1_dX+d_T1_dX*T1));
T2 = value_sqrt_0;
}
d_Vbsitf_dX = (d_Vbs0_dX+0.5*(d_T1_dX+d_T2_dX));
Vbsitf = ((Vbs0+0.02)+(0.5*(T1+T2)));
}
d_T1_dX = (d_Vbs0t_dX-d_Vbsitf_dX);
T1 = ((Vbs0t-Vbsitf)-0.005);
{
double value_sqrt_0 = sqrt(((T1*T1)+2.5e-5));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*((T1*d_T1_dX+d_T1_dX*T1));
T2 = value_sqrt_0;
}
d_T3_dX = 0.5*(d_T1_dX+d_T2_dX);
T3 = (0.5*(T1+T2));
d_T4_dX = ((instanceStruct.instanceVar_pParam_B4SOIqsi*(T3*instanceStruct.d_instanceVar_B4SOIcsi_dX+d_T3_dX*instanceStruct.instanceVar_B4SOIcsi)-(T3*instanceStruct.instanceVar_B4SOIcsi)*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX)/instanceStruct.instanceVar_pParam_B4SOIqsi/instanceStruct.instanceVar_pParam_B4SOIqsi);
T4 = ((T3*instanceStruct.instanceVar_B4SOIcsi)/instanceStruct.instanceVar_pParam_B4SOIqsi);
d_Vbsmos_dX = (d_Vbsitf_dX-((0.5*T3)*d_T4_dX+0.5*d_T3_dX*T4));
Vbsmos = (Vbsitf-((0.5*T3)*T4));
d_VtgsFD_CV_dX = (d_VthFD_CV_dX-d_Vgs_eff_dX);
VtgsFD_CV = (VthFD_CV-Vgs_eff);
d_T10_dX = (instanceStruct.instanceVar_pParam_B4SOInofffd*d_Vtm_dX+instanceStruct.d_instanceVar_pParam_B4SOInofffd_dX*Vtm);
T10 = (instanceStruct.instanceVar_pParam_B4SOInofffd*Vtm);
if ((((VtgsFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)>100.0))
{
d_ExpVtgsFD_CV_dX = 2.688117142e+43*((T10*(d_VtgsFD_CV_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VtgsFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10);
ExpVtgsFD_CV = (2.688117142e+43*((1.0+((VtgsFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VtgsFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)<(-100.0)))
{
d_ExpVtgsFD_CV_dX = 0.0;
ExpVtgsFD_CV = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VtgsFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVtgsFD_CV_dX = deriv_exp_0_d0*(((T10*(d_VtgsFD_CV_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VtgsFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10));
ExpVtgsFD_CV = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVtgsFD_CV));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVtgsFD_CV));
d_VtgseffFD_CV_dX = (T10*deriv_log_0_d0*(d_ExpVtgsFD_CV_dX)+d_T10_dX*value_log_0);
VtgseffFD_CV = (T10*value_log_0);
}
d_VgstFD_CV_dX = (d_Vgs_eff_dX-d_VthFD_CV_dX);
VgstFD_CV = (Vgs_eff-VthFD_CV);
if ((((VgstFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)>100.0))
{
d_ExpVgstFD_CV_dX = 2.688117142e+43*((T10*(d_VgstFD_CV_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VgstFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10);
ExpVgstFD_CV = (2.688117142e+43*((1.0+((VgstFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10))-100.0));
}
else
{
if ((((VgstFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10)<(-100.0)))
{
d_ExpVgstFD_CV_dX = 0.0;
ExpVgstFD_CV = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(((VgstFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)/T10));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgstFD_CV_dX = deriv_exp_0_d0*(((T10*(d_VgstFD_CV_dX-instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX)-(VgstFD_CV-instanceStruct.instanceVar_pParam_B4SOIvofffd)*d_T10_dX)/T10/T10));
ExpVgstFD_CV = value_exp_0;
}
}
}
{
double value_log_0 = log((1.0+ExpVgstFD_CV));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVgstFD_CV));
d_VgsteffFD_CV_dX = (T10*deriv_log_0_d0*(d_ExpVgstFD_CV_dX)+d_T10_dX*value_log_0);
VgsteffFD_CV = (T10*value_log_0);
}
d_T1_dX = (((instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Vtm)*d_Vtm_dX+((instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.instanceVar_pParam_B4SOIk1ox)*d_Vtm_dX+(instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+instanceStruct.d_instanceVar_pParam_B4SOImoinfd_dX*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Vtm)*Vtm);
T1 = (((instanceStruct.instanceVar_pParam_B4SOImoinfd*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Vtm)*Vtm);
{
double value_sqrt_0 = sqrt(phi);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = (d_VgsteffFD_CV_dX+((2*instanceStruct.instanceVar_pParam_B4SOIk1eff)*deriv_sqrt_0_d0*(d_phi_dX)+2*instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*value_sqrt_0));
T2 = (VgsteffFD_CV+((2*instanceStruct.instanceVar_pParam_B4SOIk1eff)*value_sqrt_0));
}
d_T0_dX = ((T1*(VgsteffFD_CV*d_T2_dX+d_VgsteffFD_CV_dX*T2)-(VgsteffFD_CV*T2)*d_T1_dX)/T1/T1);
T0 = (1+((VgsteffFD_CV*T2)/T1));
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
d_PhiON_CV_dX = (d_phi_dX+(Vtm*evaluator_lln_0.getDerivs(T0,d_T0_dX)+d_Vtm_dX*evaluator_lln_0.getValues(T0)));
PhiON_CV = (phi+(Vtm*evaluator_lln_0.getValues(T0)));
}
d_T0_dX = (((instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox))))*instanceStruct.d_instanceVar_B4SOIcox_dX-instanceStruct.instanceVar_B4SOIcox*(instanceStruct.d_instanceVar_B4SOIcox_dX+(-((-instanceStruct.d_instanceVar_B4SOIcsi_dX/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi)+(-d_Cbox_dX/Cbox/Cbox))/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox))/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox)))))/(instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox))))/(instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox)))));
T0 = (instanceStruct.instanceVar_B4SOIcox/(instanceStruct.instanceVar_B4SOIcox+(1.0/((1.0/instanceStruct.instanceVar_B4SOIcsi)+(1.0/Cbox)))));
d_PhiFD_CV_dX = (d_PhiON_CV_dX-(T0*d_VtgseffFD_CV_dX+d_T0_dX*VtgseffFD_CV));
PhiFD_CV = (PhiON_CV-(T0*VtgseffFD_CV));
if ((instanceStruct.instanceVar_B4SOIfdMod==0))
{
d_T0_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T0 = (((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T0));
double value_exp_1 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T1_dX = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(deriv_exp_0_d0*(0.5*d_T0_dX)+2*deriv_exp_1_d0*(d_T0_dX))+instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX*(value_exp_0+(2*value_exp_1)));
T1 = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}
d_T2_dX = (T1*(d_vbi_dX-d_phi_dX)+d_T1_dX*(vbi-phi));
T2 = (T1*(vbi-phi));
d_T3_dX = ((instanceStruct.instanceVar_B4SOIcsi*0.5*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX-(0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi);
T3 = ((0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)/instanceStruct.instanceVar_B4SOIcsi);
d_Vbs0t_CV_dX = (((d_PhiFD_CV_dX-d_T3_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX)+d_T2_dX);
Vbs0t_CV = (((PhiFD_CV-T3)+instanceStruct.instanceVar_pParam_B4SOIvbsa)+T2);
d_T0_dX = ((Cbox*instanceStruct.d_instanceVar_B4SOIcsi_dX-instanceStruct.instanceVar_B4SOIcsi*d_Cbox_dX)/Cbox/Cbox);
T0 = (1+(instanceStruct.instanceVar_B4SOIcsi/Cbox));
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdk2b_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T3 = (((-instanceStruct.instanceVar_pParam_B4SOIdk2b)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T3));
double value_exp_1 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T5_dX = (instanceStruct.instanceVar_pParam_B4SOIk2b*(deriv_exp_0_d0*(0.5*d_T3_dX)+2*deriv_exp_1_d0*(d_T3_dX))+instanceStruct.d_instanceVar_pParam_B4SOIk2b_dX*(value_exp_0+(2*value_exp_1)));
T5 = (instanceStruct.instanceVar_pParam_B4SOIk2b*(value_exp_0+(2*value_exp_1)));
}
d_T1_dX = ((T0*(instanceStruct.d_instanceVar_pParam_B4SOIk1b_dX-d_T5_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1b-T5)*d_T0_dX)/T0/T0);
T1 = ((instanceStruct.instanceVar_pParam_B4SOIk1b-T5)/T0);
d_T2_dX = (T1*d_Vesfb_dX+d_T1_dX*Vesfb);
T2 = (T1*Vesfb);
d_T0_dX = (-((instanceStruct.instanceVar_B4SOIcsi*d_Cbox_dX-Cbox*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi)/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi))/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi)));
T0 = (1.0/(1+(Cbox/instanceStruct.instanceVar_B4SOIcsi)));
d_Vbs0_CV_dX = ((T0*d_Vbs0t_CV_dX+d_T0_dX*Vbs0t_CV)+d_T2_dX);
Vbs0_CV = ((T0*Vbs0t_CV)+T2);
}
else
{
d_T0_dX = (-((instanceStruct.d_instanceVar_B4SOIcsi_dX+d_Cbox_dX)+instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX)/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs)/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs));
T0 = (1.0/((instanceStruct.instanceVar_B4SOIcsi+Cbox)+instanceStruct.instanceVar_pParam_B4SOIcdsbs));
d_T1_dX = ((instanceStruct.instanceVar_pParam_B4SOIlitl*((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(-instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX)*instanceStruct.instanceVar_pParam_B4SOIleff)-((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl);
T1 = (((-instanceStruct.instanceVar_pParam_B4SOIdvbd1)*instanceStruct.instanceVar_pParam_B4SOIleff)/instanceStruct.instanceVar_pParam_B4SOIlitl);
{
double value_exp_0 = exp((0.5*T1));
double value_exp_1 = exp(T1);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_T2_dX = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(deriv_exp_0_d0*(0.5*d_T1_dX)+2*deriv_exp_1_d0*(d_T1_dX))+instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX*(value_exp_0+(2*value_exp_1)));
T2 = (instanceStruct.instanceVar_pParam_B4SOIdvbd0*(value_exp_0+(2*value_exp_1)));
}
d_T3_dX = (T2*instanceStruct.d_instanceVar_pParam_B4SOIvsce_dX+d_T2_dX*(Vds+instanceStruct.instanceVar_pParam_B4SOIvsce));
T3 = (T2*(Vds+instanceStruct.instanceVar_pParam_B4SOIvsce));
d_T4_dX = ((instanceStruct.instanceVar_B4SOIcsi*0.5*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX-(0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)*instanceStruct.d_instanceVar_B4SOIcsi_dX)/instanceStruct.instanceVar_B4SOIcsi/instanceStruct.instanceVar_B4SOIcsi);
T4 = ((0.5*instanceStruct.instanceVar_pParam_B4SOIqsi)/instanceStruct.instanceVar_B4SOIcsi);
d_T5_dX = ((instanceStruct.instanceVar_B4SOIcsi*T0)*((d_PhiFD_CV_dX-d_T4_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX)+(instanceStruct.instanceVar_B4SOIcsi*d_T0_dX+instanceStruct.d_instanceVar_B4SOIcsi_dX*T0)*((PhiFD_CV-T4)+instanceStruct.instanceVar_pParam_B4SOIvbsa));
T5 = ((instanceStruct.instanceVar_B4SOIcsi*T0)*((PhiFD_CV-T4)+instanceStruct.instanceVar_pParam_B4SOIvbsa));
d_T6_dX = ((instanceStruct.instanceVar_pParam_B4SOIcdsbs*T0)*d_T3_dX+(instanceStruct.instanceVar_pParam_B4SOIcdsbs*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX*T0)*T3);
T6 = ((instanceStruct.instanceVar_pParam_B4SOIcdsbs*T0)*T3);
d_Vbs0t_CV_dX = (d_T5_dX+d_T6_dX);
Vbs0t_CV = (T5+T6);
d_T7_dX = ((Cbox*T0)*d_Vesfb_dX+(Cbox*d_T0_dX+d_Cbox_dX*T0)*Vesfb);
T7 = ((Cbox*T0)*Vesfb);
d_Vbs0_CV_dX = (d_Vbs0t_CV_dX+d_T7_dX);
Vbs0_CV = (Vbs0t_CV+T7);
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_Vbsitf_CV_dX = d_Vbs0_CV_dX;
Vbsitf_CV = (Vbs0_CV+0.02);
d_Vbs_dX = d_Vbs0_CV_dX;
Vbs = (Vbs0_CV+0.02);
}
else
{
d_T1_dX = (d_Vbs_dX-d_Vbs0_CV_dX);
T1 = ((Vbs-(Vbs0_CV+0.02))-0.01);
{
double value_sqrt_0 = sqrt(((T1*T1)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*((T1*d_T1_dX+d_T1_dX*T1));
T2 = value_sqrt_0;
}
d_Vbsitf_CV_dX = (d_Vbs0_CV_dX+0.5*(d_T1_dX+d_T2_dX));
Vbsitf_CV = ((Vbs0_CV+0.02)+(0.5*(T1+T2)));
}
d_T1_dX = (d_Vbs0t_CV_dX-d_Vbsitf_CV_dX);
T1 = ((Vbs0t_CV-Vbsitf_CV)-0.005);
{
double value_sqrt_0 = sqrt(((T1*T1)+2.5e-5));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*((T1*d_T1_dX+d_T1_dX*T1));
T2 = value_sqrt_0;
}
d_T3_dX = 0.5*(d_T1_dX+d_T2_dX);
T3 = (0.5*(T1+T2));
d_T4_dX = ((instanceStruct.instanceVar_pParam_B4SOIqsi*(T3*instanceStruct.d_instanceVar_B4SOIcsi_dX+d_T3_dX*instanceStruct.instanceVar_B4SOIcsi)-(T3*instanceStruct.instanceVar_B4SOIcsi)*instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX)/instanceStruct.instanceVar_pParam_B4SOIqsi/instanceStruct.instanceVar_pParam_B4SOIqsi);
T4 = ((T3*instanceStruct.instanceVar_B4SOIcsi)/instanceStruct.instanceVar_pParam_B4SOIqsi);
d_Vbsmos_CV_dX = (d_Vbsitf_CV_dX-((0.5*T3)*d_T4_dX+0.5*d_T3_dX*T4));
Vbsmos_CV = (Vbsitf_CV-((0.5*T3)*T4));
}
d_T0_dX = d_Vbsmos_dX;
T0 = ((Vbsmos+5)-0.001);
{
double value_sqrt_0 = sqrt(((T0*T0)-(0.004*(-5))));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
T1 = value_sqrt_0;
}
d_T2_dX = 0.5*(d_T0_dX+d_T1_dX);
T2 = ((-5)+(0.5*(T0+T1)));
d_T0_dX = 0.0;
T0 = 1.5;
d_T1_dX = (d_T0_dX-d_T2_dX);
T1 = ((T0-T2)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+0.008*d_T0_dX));
T3 = value_sqrt_0;
}
d_Vbsh_dX = (d_T0_dX-0.5*(d_T1_dX+d_T3_dX));
Vbsh = (T0-(0.5*(T1+T3)));
d_T0_dX = 0.95*d_phi_dX;
T0 = (0.95*phi);
d_T1_dX = (d_T0_dX-d_Vbsh_dX);
T1 = ((T0-Vbsh)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+0.008*d_T0_dX));
T2 = value_sqrt_0;
}
d_Vbseff_dX = (d_T0_dX-0.5*(d_T1_dX+d_T2_dX));
Vbseff = (T0-(0.5*(T1+T2)));
d_T0_dX = d_Vbsmos_CV_dX;
T0 = ((Vbsmos_CV+5)-0.001);
{
double value_sqrt_0 = sqrt(((T0*T0)-(0.004*(-5))));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
T1 = value_sqrt_0;
}
d_T2_dX = 0.5*(d_T0_dX+d_T1_dX);
T2 = ((-5)+(0.5*(T0+T1)));
d_T0_dX = 0.0;
T0 = 1.5;
d_T1_dX = (d_T0_dX-d_T2_dX);
T1 = ((T0-T2)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+0.008*d_T0_dX));
T3 = value_sqrt_0;
}
d_Vbsh_CV_dX = (d_T0_dX-0.5*(d_T1_dX+d_T3_dX));
Vbsh_CV = (T0-(0.5*(T1+T3)));
d_T0_dX = 0.95*d_phi_dX;
T0 = (0.95*phi);
d_T1_dX = (d_T0_dX-d_Vbsh_CV_dX);
T1 = ((T0-Vbsh_CV)-0.002);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.008*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+0.008*d_T0_dX));
T2 = value_sqrt_0;
}
d_Vbseff_CV_dX = (d_T0_dX-0.5*(d_T1_dX+d_T2_dX));
Vbseff_CV = (T0-(0.5*(T1+T2)));
d_Phis_dX = (d_phi_dX-d_Vbseff_dX);
Phis = (phi-Vbseff);
{
double value_sqrt_0 = sqrt(Phis);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_sqrtPhis_dX = deriv_sqrt_0_d0*(d_Phis_dX);
sqrtPhis = value_sqrt_0;
}
d_Xdep_dX = ((sqrtPhi*(Xdep0*d_sqrtPhis_dX+d_Xdep0_dX*sqrtPhis)-(Xdep0*sqrtPhis)*d_sqrtPhi_dX)/sqrtPhi/sqrtPhi);
Xdep = ((Xdep0*sqrtPhis)/sqrtPhi);
instanceStruct.d_instanceVar_B4SOInstar_dX = ((instanceStruct.instanceVar_B4SOIvtm/1.60219e-19)*((instanceStruct.d_instanceVar_B4SOIcox_dX+((Xdep*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*d_Xdep_dX)/Xdep/Xdep))+instanceStruct.d_instanceVar_pParam_B4SOIcit_dX)+(instanceStruct.d_instanceVar_B4SOIvtm_dX/1.60219e-19)*((instanceStruct.instanceVar_B4SOIcox+(instanceStruct.instanceVar_epssub/Xdep))+instanceStruct.instanceVar_pParam_B4SOIcit));
instanceStruct.instanceVar_B4SOInstar = ((instanceStruct.instanceVar_B4SOIvtm/1.60219e-19)*((instanceStruct.instanceVar_B4SOIcox+(instanceStruct.instanceVar_epssub/Xdep))+instanceStruct.instanceVar_pParam_B4SOIcit));
{
double value_sqrt_0 = sqrt(Xdep);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(d_Xdep_dX);
T3 = value_sqrt_0;
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt2*d_Vbseff_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt2_dX*Vbseff);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt2*Vbseff);
if ((T0>=(-0.5)))
{
d_T1_dX = d_T0_dX;
T1 = (1.0+T0);
}
else
{
d_T4_dX = (-8.0*d_T0_dX/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));
d_T1_dX = ((1.0+(3.0*T0))*d_T4_dX+3.0*d_T0_dX*T4);
T1 = ((1.0+(3.0*T0))*T4);
}
d_lt1_dX = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*d_T1_dX+(instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3)*T1);
lt1 = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*T1);
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt2w*d_Vbseff_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt2w_dX*Vbseff);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt2w*Vbseff);
if ((T0>=(-0.5)))
{
d_T1_dX = d_T0_dX;
T1 = (1.0+T0);
}
else
{
d_T4_dX = (-8.0*d_T0_dX/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));
d_T1_dX = ((1.0+(3.0*T0))*d_T4_dX+3.0*d_T0_dX*T4);
T1 = ((1.0+(3.0*T0))*T4);
}
d_ltw_dX = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*d_T1_dX+(instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3)*T1);
ltw = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*T1);
d_T0_dX = ((lt1*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*d_Leff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX*Leff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)*d_lt1_dX)/lt1/lt1);
T0 = ((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)/lt1);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_Theta0_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_Theta0_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0 = (T1*(1.0+(2.0*T1)));
}
d_T2_dX = ((Xdep*(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.d_instanceVar_epssub_dX+instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX*instanceStruct.instanceVar_epssub)-(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)*d_Xdep_dX)/Xdep/Xdep);
T2 = ((instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)/Xdep);
d_T3_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX+(instanceStruct.instanceVar_pParam_B4SOIcdscb*d_Vbseff_dX+instanceStruct.d_instanceVar_pParam_B4SOIcdscb_dX*Vbseff))+instanceStruct.d_instanceVar_pParam_B4SOIcdscd_dX*Vds);
T3 = ((instanceStruct.instanceVar_pParam_B4SOIcdsc+(instanceStruct.instanceVar_pParam_B4SOIcdscb*Vbseff))+(instanceStruct.instanceVar_pParam_B4SOIcdscd*Vds));
d_T4_dX = ((instanceStruct.instanceVar_B4SOIcox*((d_T2_dX+(T3*d_Theta0_dX+d_T3_dX*Theta0))+instanceStruct.d_instanceVar_pParam_B4SOIcit_dX)-((T2+(T3*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
T4 = (((T2+(T3*Theta0))+instanceStruct.instanceVar_pParam_B4SOIcit)/instanceStruct.instanceVar_B4SOIcox);
if ((T4>=(-0.5)))
{
d_n_dX = d_T4_dX;
n = (1.0+T4);
}
else
{
d_T0_dX = (-8.0*d_T4_dX/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
T0 = (1.0/(3.0+(8.0*T4)));
d_n_dX = ((1.0+(3.0*T4))*d_T0_dX+3.0*d_T4_dX*T0);
n = ((1.0+(3.0*T4))*T0);
}
if ((instanceStruct.instanceVar_pParam_B4SOIdvtp0>0.0))
{
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIdvtp1_dX)*Vds;
T0 = ((-instanceStruct.instanceVar_pParam_B4SOIdvtp1)*Vds);
if ((T0<(-100.0)))
{
d_T2_dX = 0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T2_dX = deriv_exp_0_d0*(d_T0_dX);
T2 = value_exp_0;
}
}
d_T3_dX = (d_Leff_dX+(instanceStruct.instanceVar_pParam_B4SOIdvtp0*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvtp0_dX*(1.0+T2)));
T3 = (Leff+(instanceStruct.instanceVar_pParam_B4SOIdvtp0*(1.0+T2)));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((Leff/T3));
d_T4_dX = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),((T3*d_Leff_dX-Leff*d_T3_dX)/T3/T3))+d_Vtm_dX*evaluator_lln_0.getValues((Leff/T3)));
T4 = (Vtm*evaluator_lln_0.getValues((Leff/T3)));
}
d_DITS_Sft_dX = (n*d_T4_dX+d_n_dX*T4);
DITS_Sft = (n*T4);
}
else
{
d_DITS_Sft_dX = 0.0;
DITS_Sft = 0.0;
}
instanceStruct.d_instanceVar_B4SOIthetavth_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0*d_Theta0_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX*Theta0);
instanceStruct.instanceVar_B4SOIthetavth = (instanceStruct.instanceVar_pParam_B4SOIdvt0*Theta0);
d_Delt_vth_dX = (instanceStruct.instanceVar_B4SOIthetavth*d_V0_dX+instanceStruct.d_instanceVar_B4SOIthetavth_dX*V0);
Delt_vth = (instanceStruct.instanceVar_B4SOIthetavth*V0);
d_T0_dX = ((ltw*((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*d_Leff_dX+(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)-((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)*d_ltw_dX)/ltw/ltw);
T0 = (((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)/ltw);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX*T2);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*T2);
d_DeltVthw_dX = (T0*d_V0_dX+d_T0_dX*V0);
DeltVthw = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/Leff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX-instanceStruct.instanceVar_pParam_B4SOIlpe0*d_Leff_dX)/Leff/Leff));
T0 = value_sqrt_0;
}
d_T1_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX+((Leff*instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX-instanceStruct.instanceVar_pParam_B4SOIkt1l*d_Leff_dX)/Leff/Leff))+(instanceStruct.instanceVar_pParam_B4SOIkt2*d_Vbseff_dX+instanceStruct.d_instanceVar_pParam_B4SOIkt2_dX*Vbseff));
T1 = ((instanceStruct.instanceVar_pParam_B4SOIkt1+(instanceStruct.instanceVar_pParam_B4SOIkt1l/Leff))+(instanceStruct.instanceVar_pParam_B4SOIkt2*Vbseff));
d_DeltVthtemp_dX = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dX+(instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*(T0-1.0))*sqrtPhi)+(T1*d_TempRatioMinus1_dX+d_T1_dX*TempRatioMinus1));
DeltVthtemp = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));
d_tmp2_dX = (((instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)*(instanceStruct.instanceVar_toxe*d_phi_dX+instanceStruct.d_instanceVar_toxe_dX*phi)-(instanceStruct.instanceVar_toxe*phi)*(instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIw0_dX))/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
tmp2 = ((instanceStruct.instanceVar_toxe*phi)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
d_T3_dX = (instanceStruct.d_instanceVar_here_B4SOIeta0_dX+(instanceStruct.instanceVar_pParam_B4SOIetab*d_Vbseff_dX+instanceStruct.d_instanceVar_pParam_B4SOIetab_dX*Vbseff));
T3 = (instanceStruct.instanceVar_here_B4SOIeta0+(instanceStruct.instanceVar_pParam_B4SOIetab*Vbseff));
if ((T3<1.0e-4))
{
d_T9_dX = (-(-2.0e4*d_T3_dX)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));
d_T3_dX = ((2.0e-4-T3)*d_T9_dX+(-d_T3_dX)*T9);
T3 = ((2.0e-4-T3)*T9);
}
d_DIBL_Sft_dX = (T3*d_theta0vb0_dX+d_T3_dX*theta0vb0)*Vds;
DIBL_Sft = ((T3*theta0vb0)*Vds);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpeb/Leff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_Lpe_Vb_dX = deriv_sqrt_0_d0*(((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlpeb_dX-instanceStruct.instanceVar_pParam_B4SOIlpeb*d_Leff_dX)/Leff/Leff));
Lpe_Vb = value_sqrt_0;
}
d_T9_dX = (-2.2361*d_sqrtPhi_dX/sqrtPhi/sqrtPhi);
T9 = (2.2361/sqrtPhi);
d_sqrtPhisExt_dX = (d_sqrtPhis_dX-(T9*(d_Vbsh_dX-d_Vbseff_dX)+d_T9_dX*(Vbsh-Vbseff)));
sqrtPhisExt = (sqrtPhis-(T9*(Vbsh-Vbseff)));
{
double value_exp_0 = exp(((2.0*instanceStruct.instanceVar_pParam_B4SOIdvtp4)*Vds));
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(2.0*instanceStruct.d_instanceVar_pParam_B4SOIdvtp4_dX*Vds);
T0 = value_exp_0;
}
d_DITS_Sft2_dX = (((T0+1)*(instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvtp2factor_dX*(T0-1))-(instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dX)/(T0+1)/(T0+1));
DITS_Sft2 = ((instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*(T0-1))/(T0+1));
d_Vth_dX = (((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhisExt)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*d_Lpe_Vb_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_sqrtPhisExt_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*sqrtPhisExt)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*d_Vbseff_dX+instanceStruct.d_instanceVar_here_B4SOIk2ox_dX*Vbseff))-d_Delt_vth_dX)-d_DeltVthw_dX)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbseff))*d_tmp2_dX+(instanceStruct.d_instanceVar_pParam_B4SOIk3_dX+(instanceStruct.instanceVar_pParam_B4SOIk3b*d_Vbseff_dX+instanceStruct.d_instanceVar_pParam_B4SOIk3b_dX*Vbseff))*tmp2))+d_DeltVthtemp_dX)-d_DIBL_Sft_dX)-d_DITS_Sft_dX)-d_DITS_Sft2_dX);
Vth = ((((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhisExt)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*Vbseff))-Delt_vth)-DeltVthw)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbseff))*tmp2))+DeltVthtemp)-DIBL_Sft)-DITS_Sft)-DITS_Sft2);
d_Phis_CV_dX = (d_phi_dX-d_Vbseff_CV_dX);
Phis_CV = (phi-Vbseff_CV);
{
double value_sqrt_0 = sqrt(Phis_CV);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_sqrtPhis_CV_dX = deriv_sqrt_0_d0*(d_Phis_CV_dX);
sqrtPhis_CV = value_sqrt_0;
}
d_Xdep_CV_dX = ((sqrtPhi*(Xdep0*d_sqrtPhis_CV_dX+d_Xdep0_dX*sqrtPhis_CV)-(Xdep0*sqrtPhis_CV)*d_sqrtPhi_dX)/sqrtPhi/sqrtPhi);
Xdep_CV = ((Xdep0*sqrtPhis_CV)/sqrtPhi);
instanceStruct.d_instanceVar_B4SOInstar_dX = ((instanceStruct.instanceVar_B4SOIvtm/1.60219e-19)*((instanceStruct.d_instanceVar_B4SOIcox_dX+((Xdep_CV*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*d_Xdep_CV_dX)/Xdep_CV/Xdep_CV))+instanceStruct.d_instanceVar_pParam_B4SOIcit_dX)+(instanceStruct.d_instanceVar_B4SOIvtm_dX/1.60219e-19)*((instanceStruct.instanceVar_B4SOIcox+(instanceStruct.instanceVar_epssub/Xdep_CV))+instanceStruct.instanceVar_pParam_B4SOIcit));
instanceStruct.instanceVar_B4SOInstar = ((instanceStruct.instanceVar_B4SOIvtm/1.60219e-19)*((instanceStruct.instanceVar_B4SOIcox+(instanceStruct.instanceVar_epssub/Xdep_CV))+instanceStruct.instanceVar_pParam_B4SOIcit));
{
double value_sqrt_0 = sqrt(Xdep_CV);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(d_Xdep_CV_dX);
T3 = value_sqrt_0;
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt2*d_Vbseff_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt2_dX*Vbseff_CV);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt2*Vbseff_CV);
if ((T0>=(-0.5)))
{
d_T1_dX = d_T0_dX;
T1 = (1.0+T0);
}
else
{
d_T4_dX = (-8.0*d_T0_dX/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));
d_T1_dX = ((1.0+(3.0*T0))*d_T4_dX+3.0*d_T0_dX*T4);
T1 = ((1.0+(3.0*T0))*T4);
}
d_lt1_CV_dX = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*d_T1_dX+(instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3)*T1);
lt1_CV = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*T1);
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt2w*d_Vbseff_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt2w_dX*Vbseff_CV);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt2w*Vbseff_CV);
if ((T0>=(-0.5)))
{
d_T1_dX = d_T0_dX;
T1 = (1.0+T0);
}
else
{
d_T4_dX = (-8.0*d_T0_dX/(3.0+(8.0*T0))/(3.0+(8.0*T0)));
T4 = (1.0/(3.0+(8.0*T0)));
d_T1_dX = ((1.0+(3.0*T0))*d_T4_dX+3.0*d_T0_dX*T4);
T1 = ((1.0+(3.0*T0))*T4);
}
d_ltw_CV_dX = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*d_T1_dX+(instanceStruct.instanceVar_B4SOIfactor1*d_T3_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3)*T1);
ltw_CV = ((instanceStruct.instanceVar_B4SOIfactor1*T3)*T1);
d_T0_dX = ((lt1_CV*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*d_Leff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX*Leff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)*d_lt1_CV_dX)/lt1_CV/lt1_CV);
T0 = ((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)/lt1_CV);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_Theta0_CV_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0_CV = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_Theta0_CV_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0_CV = (T1*(1.0+(2.0*T1)));
}
d_T2_dX = ((Xdep_CV*(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.d_instanceVar_epssub_dX+instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX*instanceStruct.instanceVar_epssub)-(instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)*d_Xdep_CV_dX)/Xdep_CV/Xdep_CV);
T2 = ((instanceStruct.instanceVar_pParam_B4SOInfactor*instanceStruct.instanceVar_epssub)/Xdep_CV);
d_T3_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX+(instanceStruct.instanceVar_pParam_B4SOIcdscb*d_Vbseff_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIcdscb_dX*Vbseff_CV))+instanceStruct.d_instanceVar_pParam_B4SOIcdscd_dX*Vds);
T3 = ((instanceStruct.instanceVar_pParam_B4SOIcdsc+(instanceStruct.instanceVar_pParam_B4SOIcdscb*Vbseff_CV))+(instanceStruct.instanceVar_pParam_B4SOIcdscd*Vds));
d_T4_dX = ((instanceStruct.instanceVar_B4SOIcox*((d_T2_dX+(T3*d_Theta0_CV_dX+d_T3_dX*Theta0_CV))+instanceStruct.d_instanceVar_pParam_B4SOIcit_dX)-((T2+(T3*Theta0_CV))+instanceStruct.instanceVar_pParam_B4SOIcit)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox);
T4 = (((T2+(T3*Theta0_CV))+instanceStruct.instanceVar_pParam_B4SOIcit)/instanceStruct.instanceVar_B4SOIcox);
if ((T4>=(-0.5)))
{
d_n_CV_dX = d_T4_dX;
n_CV = (1.0+T4);
}
else
{
d_T0_dX = (-8.0*d_T4_dX/(3.0+(8.0*T4))/(3.0+(8.0*T4)));
T0 = (1.0/(3.0+(8.0*T4)));
d_n_CV_dX = ((1.0+(3.0*T4))*d_T0_dX+3.0*d_T4_dX*T0);
n_CV = ((1.0+(3.0*T4))*T0);
}
if ((instanceStruct.instanceVar_pParam_B4SOIdvtp0>0.0))
{
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIdvtp1_dX)*Vds;
T0 = ((-instanceStruct.instanceVar_pParam_B4SOIdvtp1)*Vds);
if ((T0<(-100.0)))
{
d_T2_dX = 0.0;
T2 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T2_dX = deriv_exp_0_d0*(d_T0_dX);
T2 = value_exp_0;
}
}
d_T3_dX = (d_Leff_dX+(instanceStruct.instanceVar_pParam_B4SOIdvtp0*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvtp0_dX*(1.0+T2)));
T3 = (Leff+(instanceStruct.instanceVar_pParam_B4SOIdvtp0*(1.0+T2)));
{
AnalogFunctions::llnEvaluator evaluator_lln_0((Leff/T3));
d_T4_dX = (Vtm*evaluator_lln_0.getDerivs((Leff/T3),((T3*d_Leff_dX-Leff*d_T3_dX)/T3/T3))+d_Vtm_dX*evaluator_lln_0.getValues((Leff/T3)));
T4 = (Vtm*evaluator_lln_0.getValues((Leff/T3)));
}
d_DITS_Sft_CV_dX = (n_CV*d_T4_dX+d_n_CV_dX*T4);
DITS_Sft_CV = (n_CV*T4);
}
else
{
d_DITS_Sft_CV_dX = 0.0;
DITS_Sft_CV = 0.0;
}
instanceStruct.d_instanceVar_B4SOIthetavth_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0*d_Theta0_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX*Theta0_CV);
instanceStruct.instanceVar_B4SOIthetavth = (instanceStruct.instanceVar_pParam_B4SOIdvt0*Theta0_CV);
d_Delt_vth_CV_dX = (instanceStruct.instanceVar_B4SOIthetavth*d_V0_dX+instanceStruct.d_instanceVar_B4SOIthetavth_dX*V0);
Delt_vth_CV = (instanceStruct.instanceVar_B4SOIthetavth*V0);
d_T0_dX = ((ltw_CV*((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*d_Leff_dX+(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)-((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)*d_ltw_CV_dX)/ltw_CV/ltw_CV);
T0 = (((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)/ltw_CV);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX*T2);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*T2);
d_DeltVthw_CV_dX = (T0*d_V0_dX+d_T0_dX*V0);
DeltVthw_CV = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/Leff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX-instanceStruct.instanceVar_pParam_B4SOIlpe0*d_Leff_dX)/Leff/Leff));
T0 = value_sqrt_0;
}
d_T1_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX+((Leff*instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX-instanceStruct.instanceVar_pParam_B4SOIkt1l*d_Leff_dX)/Leff/Leff))+(instanceStruct.instanceVar_pParam_B4SOIkt2*d_Vbseff_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIkt2_dX*Vbseff_CV));
T1 = ((instanceStruct.instanceVar_pParam_B4SOIkt1+(instanceStruct.instanceVar_pParam_B4SOIkt1l/Leff))+(instanceStruct.instanceVar_pParam_B4SOIkt2*Vbseff_CV));
d_DeltVthtemp_CV_dX = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dX+(instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*(T0-1.0))*sqrtPhi)+(T1*d_TempRatioMinus1_dX+d_T1_dX*TempRatioMinus1));
DeltVthtemp_CV = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));
d_tmp2_CV_dX = (((instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)*(instanceStruct.instanceVar_toxe*d_phi_dX+instanceStruct.d_instanceVar_toxe_dX*phi)-(instanceStruct.instanceVar_toxe*phi)*(instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIw0_dX))/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
tmp2_CV = ((instanceStruct.instanceVar_toxe*phi)/(instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIw0));
d_T3_dX = (instanceStruct.d_instanceVar_here_B4SOIeta0cv_dX+(instanceStruct.instanceVar_pParam_B4SOIetabcv*d_Vbseff_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIetabcv_dX*Vbseff_CV));
T3 = (instanceStruct.instanceVar_here_B4SOIeta0cv+(instanceStruct.instanceVar_pParam_B4SOIetabcv*Vbseff_CV));
if ((T3<1.0e-4))
{
d_T9_dX = (-(-2.0e4*d_T3_dX)/(3.0-(2.0e4*T3))/(3.0-(2.0e4*T3)));
T9 = (1.0/(3.0-(2.0e4*T3)));
d_T3_dX = ((2.0e-4-T3)*d_T9_dX+(-d_T3_dX)*T9);
T3 = ((2.0e-4-T3)*T9);
}
d_DIBL_Sft_CV_dX = (T3*d_theta0vb0_dX+d_T3_dX*theta0vb0)*Vds;
DIBL_Sft_CV = ((T3*theta0vb0)*Vds);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpeb/Leff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_Lpe_Vb_dX = deriv_sqrt_0_d0*(((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlpeb_dX-instanceStruct.instanceVar_pParam_B4SOIlpeb*d_Leff_dX)/Leff/Leff));
Lpe_Vb = value_sqrt_0;
}
d_T9_dX = (-2.2361*d_sqrtPhi_dX/sqrtPhi/sqrtPhi);
T9 = (2.2361/sqrtPhi);
d_sqrtPhisExt_CV_dX = (d_sqrtPhis_CV_dX-(T9*(d_Vbsh_CV_dX-d_Vbseff_CV_dX)+d_T9_dX*(Vbsh_CV-Vbseff_CV)));
sqrtPhisExt_CV = (sqrtPhis_CV-(T9*(Vbsh_CV-Vbseff_CV)));
{
double value_exp_0 = exp(((2.0*instanceStruct.instanceVar_pParam_B4SOIdvtp4)*Vds));
double  deriv_exp_0_d0 = value_exp_0;
d_T0_dX = deriv_exp_0_d0*(2.0*instanceStruct.d_instanceVar_pParam_B4SOIdvtp4_dX*Vds);
T0 = value_exp_0;
}
d_DITS_Sft2_dX = (((T0+1)*(instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvtp2factor_dX*(T0-1))-(instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*(T0-1))*d_T0_dX)/(T0+1)/(T0+1));
DITS_Sft2 = ((instanceStruct.instanceVar_pParam_B4SOIdvtp2factor*(T0-1))/(T0+1));
d_Vth_CV_dX = (((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhisExt_CV)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*d_Lpe_Vb_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_sqrtPhisExt_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*sqrtPhisExt_CV)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*d_Vbseff_CV_dX+instanceStruct.d_instanceVar_here_B4SOIk2ox_dX*Vbseff_CV))-d_Delt_vth_CV_dX)-d_DeltVthw_CV_dX)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbseff_CV))*d_tmp2_CV_dX+(instanceStruct.d_instanceVar_pParam_B4SOIk3_dX+(instanceStruct.instanceVar_pParam_B4SOIk3b*d_Vbseff_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIk3b_dX*Vbseff_CV))*tmp2_CV))+d_DeltVthtemp_CV_dX)-d_DIBL_Sft_CV_dX)-d_DITS_Sft_CV_dX)-d_DITS_Sft2_dX);
Vth_CV = ((((((((((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)+(((instanceStruct.instanceVar_pParam_B4SOIk1ox*sqrtPhisExt_CV)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))*Lpe_Vb))-(instanceStruct.instanceVar_here_B4SOIk2ox*Vbseff_CV))-Delt_vth_CV)-DeltVthw_CV)+((instanceStruct.instanceVar_pParam_B4SOIk3+(instanceStruct.instanceVar_pParam_B4SOIk3b*Vbseff_CV))*tmp2_CV))+DeltVthtemp_CV)-DIBL_Sft_CV)-DITS_Sft_CV)-DITS_Sft2);
if ((((instanceStruct.instanceVar_B4SOIcapMod==3)&&(instanceStruct.instancePar_SHMOD==1))&&(instanceStruct.instancePar_RTH0!=0.0)))
{
{
double value_sqrt_0 = sqrt(Xdep0);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3zb_dX = deriv_sqrt_0_d0*(d_Xdep0_dX);
T3zb = value_sqrt_0;
}
d_lt1zb_dX = (instanceStruct.instanceVar_B4SOIfactor1*d_T3zb_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3zb);
lt1zb = (instanceStruct.instanceVar_B4SOIfactor1*T3zb);
d_ltwzb_dX = (instanceStruct.instanceVar_B4SOIfactor1*d_T3zb_dX+instanceStruct.d_instanceVar_B4SOIfactor1_dX*T3zb);
ltwzb = (instanceStruct.instanceVar_B4SOIfactor1*T3zb);
d_T0_dX = ((lt1zb*(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*d_Leff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX*Leff)-(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)*d_lt1zb_dX)/lt1zb/lt1zb);
T0 = ((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1)*Leff)/lt1zb);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_Theta0zb_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0zb = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_Theta0zb_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
Theta0zb = (T1*(1.0+(2.0*T1)));
}
d_Delt_vthzb_dX = ((instanceStruct.instanceVar_pParam_B4SOIdvt0*Theta0zb)*d_V0_dX+(instanceStruct.instanceVar_pParam_B4SOIdvt0*d_Theta0zb_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX*Theta0zb)*V0);
Delt_vthzb = ((instanceStruct.instanceVar_pParam_B4SOIdvt0*Theta0zb)*V0);
d_T0_dX = ((ltwzb*((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*d_Leff_dX+(((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+(-0.5)*instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)-((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)*d_ltwzb_dX)/ltwzb/ltwzb);
T0 = (((((-0.5)*instanceStruct.instanceVar_pParam_B4SOIdvt1w)*instanceStruct.instanceVar_pParam_B4SOIweff)*Leff)/ltwzb);
if ((T0>(-100.0)))
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
else
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
d_T2_dX = (T1*2.0*d_T1_dX+d_T1_dX*(1.0+(2.0*T1)));
T2 = (T1*(1.0+(2.0*T1)));
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*d_T2_dX+instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX*T2);
T0 = (instanceStruct.instanceVar_pParam_B4SOIdvt0w*T2);
d_DeltVthwzb_dX = (T0*d_V0_dX+d_T0_dX*V0);
DeltVthwzb = (T0*V0);
{
double value_sqrt_0 = sqrt((1.0+(instanceStruct.instanceVar_pParam_B4SOIlpe0/Leff)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX-instanceStruct.instanceVar_pParam_B4SOIlpe0*d_Leff_dX)/Leff/Leff));
T0 = value_sqrt_0;
}
d_T1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX+((Leff*instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX-instanceStruct.instanceVar_pParam_B4SOIkt1l*d_Leff_dX)/Leff/Leff));
T1 = (instanceStruct.instanceVar_pParam_B4SOIkt1+(instanceStruct.instanceVar_pParam_B4SOIkt1l/Leff));
d_DeltVthtempzb_dX = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*d_sqrtPhi_dX+(instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*(T0-1.0))*sqrtPhi)+(T1*d_TempRatioMinus1_dX+d_T1_dX*TempRatioMinus1));
DeltVthtempzb = (((instanceStruct.instanceVar_pParam_B4SOIk1ox*(T0-1.0))*sqrtPhi)+(T1*TempRatioMinus1));
d_Vthzb_dX = ((((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX-d_Delt_vthzb_dX)-d_DeltVthwzb_dX)+(instanceStruct.instanceVar_pParam_B4SOIk3*d_tmp2_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIk3_dX*tmp2_CV))+d_DeltVthtempzb_dX);
Vthzb = (((((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)-Delt_vthzb)-DeltVthwzb)+(instanceStruct.instanceVar_pParam_B4SOIk3*tmp2_CV))+DeltVthtempzb);
}
else
{
d_Vthzb_dX = 0.0;
Vthzb = 0;
}
d_Vgst_dX = (d_Vgs_eff_dX-d_Vth_dX);
Vgst = (Vgs_eff-Vth);
d_T10_dX = (n*d_Vtm_dX+d_n_dX*Vtm);
T10 = (n*Vtm);
d_VgstNVt_dX = ((T10*(instanceStruct.instanceVar_pParam_B4SOImstar*d_Vgst_dX+instanceStruct.d_instanceVar_pParam_B4SOImstar_dX*Vgst)-(instanceStruct.instanceVar_pParam_B4SOImstar*Vgst)*d_T10_dX)/T10/T10);
VgstNVt = ((instanceStruct.instanceVar_pParam_B4SOImstar*Vgst)/T10);
d_ExpArg_dX = ((T10*(instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX-((1-instanceStruct.instanceVar_pParam_B4SOImstar)*d_Vgst_dX+(-instanceStruct.d_instanceVar_pParam_B4SOImstar_dX)*Vgst))-(instanceStruct.instanceVar_pParam_B4SOIvoff-((1-instanceStruct.instanceVar_pParam_B4SOImstar)*Vgst))*d_T10_dX)/T10/T10);
ExpArg = ((instanceStruct.instanceVar_pParam_B4SOIvoff-((1-instanceStruct.instanceVar_pParam_B4SOImstar)*Vgst))/T10);
if ((VgstNVt>100.0))
{
d_Vgsteff_dX = d_Vgst_dX;
Vgsteff = Vgst;
d_ExpVgst_dX = 0.0;
ExpVgst = 0;
}
else
{
if ((ExpArg>100.0))
{
d_T0_dX = (((n*Vtm)*(d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX)-(Vgst-instanceStruct.instanceVar_pParam_B4SOIvoff)*(n*d_Vtm_dX+d_n_dX*Vtm))/(n*Vtm)/(n*Vtm));
T0 = ((Vgst-instanceStruct.instanceVar_pParam_B4SOIvoff)/(n*Vtm));
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = deriv_exp_0_d0*(d_T0_dX);
ExpVgst = value_exp_0;
}
d_Vgsteff_dX = (((Vtm*cdep0)/instanceStruct.instanceVar_B4SOIcox)*d_ExpVgst_dX+((instanceStruct.instanceVar_B4SOIcox*(Vtm*d_cdep0_dX+d_Vtm_dX*cdep0)-(Vtm*cdep0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox)*ExpVgst);
Vgsteff = (((Vtm*cdep0)/instanceStruct.instanceVar_B4SOIcox)*ExpVgst);
}
else
{
{
double value_exp_0 = exp(VgstNVt);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = deriv_exp_0_d0*(d_VgstNVt_dX);
ExpVgst = value_exp_0;
}
{
double value_log_0 = log((1.0+ExpVgst));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVgst));
d_T1_dX = (T10*deriv_log_0_d0*(d_ExpVgst_dX)+d_T10_dX*value_log_0);
T1 = (T10*value_log_0);
}
{
double value_exp_0 = exp(ExpArg);
double  deriv_exp_0_d0 = value_exp_0;
d_dT2_dVg_dX = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(-instanceStruct.d_instanceVar_pParam_B4SOImstar_dX)+(((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*deriv_exp_0_d0*(d_ExpArg_dX)+(((Vtm*cdep0)*(-instanceStruct.d_instanceVar_B4SOIcox_dX)-(-instanceStruct.instanceVar_B4SOIcox)*(Vtm*d_cdep0_dX+d_Vtm_dX*cdep0))/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstar));
dT2_dVg = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstar));
}
d_T2_dX = (instanceStruct.d_instanceVar_pParam_B4SOImstar_dX-(((1.0-instanceStruct.instanceVar_pParam_B4SOImstar)*(T10*d_dT2_dVg_dX+d_T10_dX*dT2_dVg)-(T10*dT2_dVg)*(-instanceStruct.d_instanceVar_pParam_B4SOImstar_dX))/(1.0-instanceStruct.instanceVar_pParam_B4SOImstar)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstar)));
T2 = (instanceStruct.instanceVar_pParam_B4SOImstar-((T10*dT2_dVg)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstar)));
d_Vgsteff_dX = ((T2*d_T1_dX-T1*d_T2_dX)/T2/T2);
Vgsteff = (T1/T2);
}
}
d_Vgst2Vtm_dX = (d_Vgsteff_dX+2.0*d_Vtm_dX);
Vgst2Vtm = (Vgsteff+(2.0*Vtm));
instanceStruct.d_instanceVar_B4SOIVgsteff_dX = d_Vgsteff_dX;
instanceStruct.instanceVar_B4SOIVgsteff = Vgsteff;
if ((instanceStruct.instanceVar_pParam_B4SOIfprout<=0.0))
{
d_FP_dX = 0.0;
FP = 1.0;
}
else
{
{
double value_sqrt_0 = sqrt(Leff);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T9_dX = ((Vgst2Vtm*(instanceStruct.instanceVar_pParam_B4SOIfprout*deriv_sqrt_0_d0*(d_Leff_dX)+instanceStruct.d_instanceVar_pParam_B4SOIfprout_dX*value_sqrt_0)-(instanceStruct.instanceVar_pParam_B4SOIfprout*value_sqrt_0)*d_Vgst2Vtm_dX)/Vgst2Vtm/Vgst2Vtm);
T9 = ((instanceStruct.instanceVar_pParam_B4SOIfprout*value_sqrt_0)/Vgst2Vtm);
}
d_FP_dX = (-d_T9_dX/(1.0+T9)/(1.0+T9));
FP = (1.0/(1.0+T9));
}
d_T9_dX = (d_sqrtPhis_dX-d_sqrtPhi_dX);
T9 = (sqrtPhis-sqrtPhi);
d_Weff_dX = (instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-((2.0-instanceStruct.instanceVar_B4SOInbc)*((instanceStruct.instanceVar_pParam_B4SOIdwg*d_Vgsteff_dX+instanceStruct.d_instanceVar_pParam_B4SOIdwg_dX*Vgsteff)+(instanceStruct.instanceVar_pParam_B4SOIdwb*d_T9_dX+instanceStruct.d_instanceVar_pParam_B4SOIdwb_dX*T9))+(-instanceStruct.d_instanceVar_B4SOInbc_dX)*((instanceStruct.instanceVar_pParam_B4SOIdwg*Vgsteff)+(instanceStruct.instanceVar_pParam_B4SOIdwb*T9))));
Weff = (instanceStruct.instanceVar_pParam_B4SOIweff-((2.0-instanceStruct.instanceVar_B4SOInbc)*((instanceStruct.instanceVar_pParam_B4SOIdwg*Vgsteff)+(instanceStruct.instanceVar_pParam_B4SOIdwb*T9))));
if ((Weff<2.0e-8))
{
d_T0_dX = (-(-2.0*d_Weff_dX)/(6.0e-8-(2.0*Weff))/(6.0e-8-(2.0*Weff)));
T0 = (1.0/(6.0e-8-(2.0*Weff)));
d_Weff_dX = ((2.0e-8*(4.0e-8-Weff))*d_T0_dX+2.0e-8*(-d_Weff_dX)*T0);
Weff = ((2.0e-8*(4.0e-8-Weff))*T0);
}
if ((instanceStruct.instanceVar_B4SOIrdsMod==1))
{
d_Rds_dX = 0.0;
Rds = 0.0;
}
else
{
d_T0_dX = ((instanceStruct.instanceVar_pParam_B4SOIprwg*d_Vgsteff_dX+instanceStruct.d_instanceVar_pParam_B4SOIprwg_dX*Vgsteff)+(instanceStruct.instanceVar_pParam_B4SOIprwb*d_T9_dX+instanceStruct.d_instanceVar_pParam_B4SOIprwb_dX*T9));
T0 = ((instanceStruct.instanceVar_pParam_B4SOIprwg*Vgsteff)+(instanceStruct.instanceVar_pParam_B4SOIprwb*T9));
if ((T0>=(-0.9)))
{
d_Rds_dX = (rds0*d_T0_dX+d_rds0_dX*(1.0+T0));
Rds = (rds0*(1.0+T0));
}
else
{
d_T1_dX = (-20.0*d_T0_dX/(17.0+(20.0*T0))/(17.0+(20.0*T0)));
T1 = (1.0/(17.0+(20.0*T0)));
d_Rds_dX = ((rds0*(0.8+T0))*d_T1_dX+(rds0*d_T0_dX+d_rds0_dX*(0.8+T0))*T1);
Rds = ((rds0*(0.8+T0))*T1);
}
}
if ((instanceStruct.instanceVar_B4SOIrdsMod==2))
{
d_Rds_dX = ((instanceStruct.d_instanceVar_B4SOIdrainResistance_dX+d_Rds_dX)+instanceStruct.d_instanceVar_B4SOIsourceResistance_dX);
Rds = ((instanceStruct.instanceVar_B4SOIdrainResistance+Rds)+instanceStruct.instanceVar_B4SOIsourceResistance);
}
instanceStruct.d_instanceVar_B4SOIrds_dX = ((instanceStruct.instanceVar_B4SOInf*d_Rds_dX-Rds*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf);
instanceStruct.instanceVar_B4SOIrds = (Rds/instanceStruct.instanceVar_B4SOInf);
if ((instanceStruct.instanceVar_pParam_B4SOIa0==0.0))
{
d_Abulk_dX = 0.0;
Abulk = 1.0;
d_Abulk0_dX = 0.0;
Abulk0 = 1.0;
}
else
{
d_T10_dX = (instanceStruct.instanceVar_pParam_B4SOIketa*d_Vbsh_dX+instanceStruct.d_instanceVar_pParam_B4SOIketa_dX*Vbsh);
T10 = (instanceStruct.instanceVar_pParam_B4SOIketa*Vbsh);
if ((T10>=(-0.5)))
{
d_T11_dX = (-d_T10_dX/(1.0+T10)/(1.0+T10));
T11 = (1.0/(1.0+T10));
}
else
{
d_T12_dX = 0.0;
T12 = ((-1.0)/((1.0-0.5)*(1.0-0.5)));
d_T13_dX = d_T12_dX*0.5;
T13 = ((1.0/(1.0-0.5))+(T12*0.5));
d_T11_dX = ((T12*d_T10_dX+d_T12_dX*T10)+d_T13_dX);
T11 = ((T12*T10)+T13);
}
d_T10_dX = (d_phi_dX+instanceStruct.d_instanceVar_pParam_B4SOIketas_dX);
T10 = (phi+instanceStruct.instanceVar_pParam_B4SOIketas);
d_T13_dX = ((T10*(Vbsh*d_T11_dX+d_Vbsh_dX*T11)-(Vbsh*T11)*d_T10_dX)/T10/T10);
T13 = ((Vbsh*T11)/T10);
if ((T13<0.50))
{
{
double value_sqrt_0 = sqrt((1.0-T13));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T14_dX = (-deriv_sqrt_0_d0*((-d_T13_dX))/value_sqrt_0/value_sqrt_0);
T14 = (1.0/value_sqrt_0);
}
}
else
{
{
double value_sqrt_0 = sqrt((1-0.50));
d_T11_dX = 0.0;
T11 = (1.0/((2*(1-0.50))*value_sqrt_0));
}
{
double value_sqrt_0 = sqrt((1.0-0.50));
d_T12_dX = (-d_T11_dX*0.50);
T12 = ((1/value_sqrt_0)-(T11*0.50));
}
d_T14_dX = ((T11*d_T13_dX+d_T11_dX*T13)+d_T12_dX);
T14 = ((T11*T13)+T12);
}
{
double value_sqrt_0 = sqrt((phi+instanceStruct.instanceVar_pParam_B4SOIketas));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T10_dX = ((value_sqrt_0*((0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox)*d_Lpe_Vb_dX+0.5*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*Lpe_Vb)-((0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Lpe_Vb)*deriv_sqrt_0_d0*((d_phi_dX+instanceStruct.d_instanceVar_pParam_B4SOIketas_dX)))/value_sqrt_0/value_sqrt_0);
T10 = (((0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Lpe_Vb)/value_sqrt_0);
}
d_T1_dX = (T10*d_T14_dX+d_T10_dX*T14);
T1 = (T10*T14);
{
double value_sqrt_0 = sqrt((instanceStruct.instanceVar_pParam_B4SOIxj*Xdep));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T9_dX = deriv_sqrt_0_d0*((instanceStruct.instanceVar_pParam_B4SOIxj*d_Xdep_dX+instanceStruct.d_instanceVar_pParam_B4SOIxj_dX*Xdep));
T9 = value_sqrt_0;
}
d_tmp1_dX = (d_Leff_dX+2.0*d_T9_dX);
tmp1 = (Leff+(2.0*T9));
d_T5_dX = ((tmp1*d_Leff_dX-Leff*d_tmp1_dX)/tmp1/tmp1);
T5 = (Leff/tmp1);
d_tmp2_dX = (instanceStruct.instanceVar_pParam_B4SOIa0*d_T5_dX+instanceStruct.d_instanceVar_pParam_B4SOIa0_dX*T5);
tmp2 = (instanceStruct.instanceVar_pParam_B4SOIa0*T5);
d_tmp3_dX = (instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIb1_dX);
tmp3 = (instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIb1);
d_tmp4_dX = ((tmp3*instanceStruct.d_instanceVar_pParam_B4SOIb0_dX-instanceStruct.instanceVar_pParam_B4SOIb0*d_tmp3_dX)/tmp3/tmp3);
tmp4 = (instanceStruct.instanceVar_pParam_B4SOIb0/tmp3);
d_T2_dX = (d_tmp2_dX+d_tmp4_dX);
T2 = (tmp2+tmp4);
d_T6_dX = (T5*d_T5_dX+d_T5_dX*T5);
T6 = (T5*T5);
d_T7_dX = (T5*d_T6_dX+d_T5_dX*T6);
T7 = (T5*T6);
d_Abulk0_dX = (T1*d_T2_dX+d_T1_dX*T2);
Abulk0 = (1+(T1*T2));
d_T8_dX = ((instanceStruct.instanceVar_pParam_B4SOIags*instanceStruct.instanceVar_pParam_B4SOIa0)*d_T7_dX+(instanceStruct.instanceVar_pParam_B4SOIags*instanceStruct.d_instanceVar_pParam_B4SOIa0_dX+instanceStruct.d_instanceVar_pParam_B4SOIags_dX*instanceStruct.instanceVar_pParam_B4SOIa0)*T7);
T8 = ((instanceStruct.instanceVar_pParam_B4SOIags*instanceStruct.instanceVar_pParam_B4SOIa0)*T7);
d_dAbulk_dVg_dX = ((-T1)*d_T8_dX+(-d_T1_dX)*T8);
dAbulk_dVg = ((-T1)*T8);
d_Abulk_dX = (d_Abulk0_dX+(dAbulk_dVg*d_Vgsteff_dX+d_dAbulk_dVg_dX*Vgsteff));
Abulk = (Abulk0+(dAbulk_dVg*Vgsteff));
}
if ((Abulk0<0.01))
{
d_T9_dX = (-(-200.0*d_Abulk0_dX)/(3.0-(200.0*Abulk0))/(3.0-(200.0*Abulk0)));
T9 = (1.0/(3.0-(200.0*Abulk0)));
d_Abulk0_dX = ((0.02-Abulk0)*d_T9_dX+(-d_Abulk0_dX)*T9);
Abulk0 = ((0.02-Abulk0)*T9);
}
if ((Abulk<0.01))
{
d_T9_dX = (-(-200.0*d_Abulk_dX)/(3.0-(200.0*Abulk))/(3.0-(200.0*Abulk)));
T9 = (1.0/(3.0-(200.0*Abulk)));
d_Abulk_dX = ((0.02-Abulk)*d_T9_dX+(-d_Abulk_dX)*T9);
Abulk = ((0.02-Abulk)*T9);
}
instanceStruct.d_instanceVar_B4SOIAbulk_dX = d_Abulk_dX;
instanceStruct.instanceVar_B4SOIAbulk = Abulk;
if ((instanceStruct.instanceVar_pParam_B4SOIa0==0.0))
{
d_Abulk0_CV_dX = 0.0;
Abulk0_CV = 1.0;
}
else
{
d_T10_dX = (instanceStruct.instanceVar_pParam_B4SOIketa*d_Vbsh_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIketa_dX*Vbsh_CV);
T10 = (instanceStruct.instanceVar_pParam_B4SOIketa*Vbsh_CV);
if ((T10>=(-0.5)))
{
d_T11_dX = (-d_T10_dX/(1.0+T10)/(1.0+T10));
T11 = (1.0/(1.0+T10));
}
else
{
d_T12_dX = 0.0;
T12 = ((-1.0)/((1.0-0.5)*(1.0-0.5)));
d_T13_dX = d_T12_dX*0.5;
T13 = ((1.0/(1.0-0.5))+(T12*0.5));
d_T11_dX = ((T12*d_T10_dX+d_T12_dX*T10)+d_T13_dX);
T11 = ((T12*T10)+T13);
}
d_T10_dX = (d_phi_dX+instanceStruct.d_instanceVar_pParam_B4SOIketas_dX);
T10 = (phi+instanceStruct.instanceVar_pParam_B4SOIketas);
d_T13_dX = ((T10*(Vbsh_CV*d_T11_dX+d_Vbsh_CV_dX*T11)-(Vbsh_CV*T11)*d_T10_dX)/T10/T10);
T13 = ((Vbsh_CV*T11)/T10);
if ((T13<0.50))
{
{
double value_sqrt_0 = sqrt((1.0-T13));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T14_dX = (-deriv_sqrt_0_d0*((-d_T13_dX))/value_sqrt_0/value_sqrt_0);
T14 = (1.0/value_sqrt_0);
}
}
else
{
{
double value_sqrt_0 = sqrt((1-0.50));
d_T11_dX = 0.0;
T11 = (1.0/((2*(1-0.50))*value_sqrt_0));
}
{
double value_sqrt_0 = sqrt((1.0-0.50));
d_T12_dX = (-d_T11_dX*0.50);
T12 = ((1/value_sqrt_0)-(T11*0.50));
}
d_T14_dX = ((T11*d_T13_dX+d_T11_dX*T13)+d_T12_dX);
T14 = ((T11*T13)+T12);
}
{
double value_sqrt_0 = sqrt((phi+instanceStruct.instanceVar_pParam_B4SOIketas));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T10_dX = ((value_sqrt_0*((0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox)*d_Lpe_Vb_dX+0.5*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*Lpe_Vb)-((0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Lpe_Vb)*deriv_sqrt_0_d0*((d_phi_dX+instanceStruct.d_instanceVar_pParam_B4SOIketas_dX)))/value_sqrt_0/value_sqrt_0);
T10 = (((0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox)*Lpe_Vb)/value_sqrt_0);
}
d_T1_dX = (T10*d_T14_dX+d_T10_dX*T14);
T1 = (T10*T14);
{
double value_sqrt_0 = sqrt((instanceStruct.instanceVar_pParam_B4SOIxj*Xdep_CV));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T9_dX = deriv_sqrt_0_d0*((instanceStruct.instanceVar_pParam_B4SOIxj*d_Xdep_CV_dX+instanceStruct.d_instanceVar_pParam_B4SOIxj_dX*Xdep_CV));
T9 = value_sqrt_0;
}
d_tmp1_dX = (d_Leff_dX+2.0*d_T9_dX);
tmp1 = (Leff+(2.0*T9));
d_T5_dX = ((tmp1*d_Leff_dX-Leff*d_tmp1_dX)/tmp1/tmp1);
T5 = (Leff/tmp1);
d_tmp2_dX = (instanceStruct.instanceVar_pParam_B4SOIa0*d_T5_dX+instanceStruct.d_instanceVar_pParam_B4SOIa0_dX*T5);
tmp2 = (instanceStruct.instanceVar_pParam_B4SOIa0*T5);
d_tmp3_dX = (instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_pParam_B4SOIb1_dX);
tmp3 = (instanceStruct.instanceVar_pParam_B4SOIweff+instanceStruct.instanceVar_pParam_B4SOIb1);
d_tmp4_dX = ((tmp3*instanceStruct.d_instanceVar_pParam_B4SOIb0_dX-instanceStruct.instanceVar_pParam_B4SOIb0*d_tmp3_dX)/tmp3/tmp3);
tmp4 = (instanceStruct.instanceVar_pParam_B4SOIb0/tmp3);
d_T2_dX = (d_tmp2_dX+d_tmp4_dX);
T2 = (tmp2+tmp4);
d_T6_dX = (T5*d_T5_dX+d_T5_dX*T5);
T6 = (T5*T5);
d_T7_dX = (T5*d_T6_dX+d_T5_dX*T6);
T7 = (T5*T6);
d_Abulk0_CV_dX = (T1*d_T2_dX+d_T1_dX*T2);
Abulk0_CV = (1+(T1*T2));
}
if ((Abulk0_CV<0.01))
{
d_T9_dX = (-(-200.0*d_Abulk0_CV_dX)/(3.0-(200.0*Abulk0_CV))/(3.0-(200.0*Abulk0_CV)));
T9 = (1.0/(3.0-(200.0*Abulk0_CV)));
d_Abulk0_CV_dX = ((0.02-Abulk0_CV)*d_T9_dX+(-d_Abulk0_CV_dX)*T9);
Abulk0_CV = ((0.02-Abulk0_CV)*T9);
}
if (instanceStruct.instanceVar_B4SOImtrlMod)
{
d_T14_dX = (2.0*instanceStruct.instanceVar_B4SOItype)*((instanceStruct.d_instanceVar_B4SOIphig_dX-instanceStruct.d_instanceVar_B4SOIeasub_dX)-0.5*d_Eg_dX);
T14 = ((2.0*instanceStruct.instanceVar_B4SOItype)*(((instanceStruct.instanceVar_B4SOIphig-instanceStruct.instanceVar_B4SOIeasub)-(0.5*Eg))+0.45));
d_toxe_mob_dX = ((instanceStruct.instanceVar_B4SOIeot*instanceStruct.d_instanceVar_B4SOIepsrsub_dX+instanceStruct.d_instanceVar_B4SOIeot_dX*instanceStruct.instanceVar_B4SOIepsrsub)/3.9);
toxe_mob = ((instanceStruct.instanceVar_B4SOIeot*instanceStruct.instanceVar_B4SOIepsrsub)/3.9);
}
else
{
d_T14_dX = 0.0;
T14 = 0.0;
d_toxe_mob_dX = instanceStruct.d_instanceVar_B4SOItox_dX;
toxe_mob = instanceStruct.instanceVar_B4SOItox;
}
if ((instanceStruct.instanceVar_B4SOImobMod==1))
{
d_T0_dX = (((d_Vgsteff_dX+d_Vth_dX)+d_Vth_dX)-d_T14_dX);
T0 = (((Vgsteff+Vth)+Vth)-T14);
d_T2_dX = (d_ua_dX+(uc*d_Vbseff_dX+d_uc_dX*Vbseff));
T2 = (ua+(uc*Vbseff));
d_T3_dX = ((toxe_mob*d_T0_dX-T0*d_toxe_mob_dX)/toxe_mob/toxe_mob);
T3 = (T0/toxe_mob);
d_T5_dX = (T3*(d_T2_dX+(ub*d_T3_dX+d_ub_dX*T3))+d_T3_dX*(T2+(ub*T3)));
T5 = (T3*(T2+(ub*T3)));
}
else
{
if ((instanceStruct.instanceVar_B4SOImobMod==2))
{
d_T5_dX = (((Vgsteff-T14)/instanceStruct.instanceVar_toxe)*((d_ua_dX+(uc*d_Vbseff_dX+d_uc_dX*Vbseff))+((instanceStruct.instanceVar_toxe*(ub*(d_Vgsteff_dX-d_T14_dX)+d_ub_dX*(Vgsteff-T14))-(ub*(Vgsteff-T14))*instanceStruct.d_instanceVar_toxe_dX)/instanceStruct.instanceVar_toxe/instanceStruct.instanceVar_toxe))+((instanceStruct.instanceVar_toxe*(d_Vgsteff_dX-d_T14_dX)-(Vgsteff-T14)*instanceStruct.d_instanceVar_toxe_dX)/instanceStruct.instanceVar_toxe/instanceStruct.instanceVar_toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/instanceStruct.instanceVar_toxe)));
T5 = (((Vgsteff-T14)/instanceStruct.instanceVar_toxe)*((ua+(uc*Vbseff))+((ub*(Vgsteff-T14))/instanceStruct.instanceVar_toxe)));
}
else
{
if ((instanceStruct.instanceVar_B4SOImobMod==3))
{
d_T0_dX = (((d_Vgsteff_dX+d_Vth_dX)+d_Vth_dX)-d_T14_dX);
T0 = (((Vgsteff+Vth)+Vth)-T14);
d_T2_dX = (uc*d_Vbseff_dX+d_uc_dX*Vbseff);
T2 = (1.0+(uc*Vbseff));
d_T3_dX = ((toxe_mob*d_T0_dX-T0*d_toxe_mob_dX)/toxe_mob/toxe_mob);
T3 = (T0/toxe_mob);
d_T4_dX = (T3*(d_ua_dX+(ub*d_T3_dX+d_ub_dX*T3))+d_T3_dX*(ua+(ub*T3)));
T4 = (T3*(ua+(ub*T3)));
d_T5_dX = (T4*d_T2_dX+d_T4_dX*T2);
T5 = (T4*T2);
}
else
{
d_T0_dX = (((instanceStruct.instanceVar_toxe*(d_Vgsteff_dX+instanceStruct.d_instanceVar_B4SOIvtfbphi1_dX)*1.0e-8-((Vgsteff+instanceStruct.instanceVar_B4SOIvtfbphi1)*1.0e-8)*instanceStruct.d_instanceVar_toxe_dX)/instanceStruct.instanceVar_toxe/instanceStruct.instanceVar_toxe)/6.0);
T0 = ((((Vgsteff+instanceStruct.instanceVar_B4SOIvtfbphi1)*1.0e-8)/instanceStruct.instanceVar_toxe)/6.0);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp((instanceStruct.instanceVar_pParam_B4SOIeu*evaluator_lln_0.getValues(T0)));
double  deriv_exp_1_d0 = value_exp_1;
d_T1_dX = deriv_exp_1_d0*((instanceStruct.instanceVar_pParam_B4SOIeu*evaluator_lln_0.getDerivs(T0,d_T0_dX)+instanceStruct.d_instanceVar_pParam_B4SOIeu_dX*evaluator_lln_0.getValues(T0)));
T1 = value_exp_1;
}
d_T2_dX = (d_ua_dX+(uc*d_Vbseff_dX+d_uc_dX*Vbseff));
T2 = (ua+(uc*Vbseff));
{
double value_pow_0 = pow(TempRatio,instanceStruct.instanceVar_pParam_B4SOIucste);
double  deriv_pow_0_d0 = ((TempRatio == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_pParam_B4SOIucste/TempRatio));
double  deriv_pow_0_d1 = (TempRatio == 0.0)?0.0:(log(TempRatio)*value_pow_0);
d_ucs_dX = (instanceStruct.instanceVar_pParam_B4SOIucs*((deriv_pow_0_d0*(d_TempRatio_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_pParam_B4SOIucste_dX)))+instanceStruct.d_instanceVar_pParam_B4SOIucs_dX*value_pow_0);
ucs = (instanceStruct.instanceVar_pParam_B4SOIucs*value_pow_0);
}
{
double value_pow_0 = pow(TempRatio,instanceStruct.instanceVar_pParam_B4SOIud1);
double  deriv_pow_0_d0 = ((TempRatio == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_pParam_B4SOIud1/TempRatio));
double  deriv_pow_0_d1 = (TempRatio == 0.0)?0.0:(log(TempRatio)*value_pow_0);
d_ud_dX = (instanceStruct.instanceVar_pParam_B4SOIud*((deriv_pow_0_d0*(d_TempRatio_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_pParam_B4SOIud1_dX)))+instanceStruct.d_instanceVar_pParam_B4SOIud_dX*value_pow_0);
ud = (instanceStruct.instanceVar_pParam_B4SOIud*value_pow_0);
}
d_VgsteffVth_dX = instanceStruct.d_instanceVar_B4SOIvgsteffvth_dX;
VgsteffVth = instanceStruct.instanceVar_B4SOIvgsteffvth;
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+(Vgsteff/VgsteffVth)));
double value_exp_1 = exp((ucs*evaluator_lln_0.getValues((1.0+(Vgsteff/VgsteffVth)))));
double  deriv_exp_1_d0 = value_exp_1;
d_T10_dX = deriv_exp_1_d0*((ucs*evaluator_lln_0.getDerivs((1.0+(Vgsteff/VgsteffVth)),((VgsteffVth*d_Vgsteff_dX-Vgsteff*d_VgsteffVth_dX)/VgsteffVth/VgsteffVth))+d_ucs_dX*evaluator_lln_0.getValues((1.0+(Vgsteff/VgsteffVth)))));
T10 = value_exp_1;
}
d_T11_dX = ((T10*d_ud_dX-ud*d_T10_dX)/T10/T10);
T11 = (ud/T10);
d_T5_dX = ((T1*d_T2_dX+d_T1_dX*T2)+d_T11_dX);
T5 = ((T1*T2)+T11);
}
}
}
if ((T5>=(-0.8)))
{
d_Denomi_dX = d_T5_dX;
Denomi = (1.0+T5);
}
else
{
d_T9_dX = (-10.0*d_T5_dX/(7.0+(10.0*T5))/(7.0+(10.0*T5)));
T9 = (1.0/(7.0+(10.0*T5)));
d_Denomi_dX = ((0.6+T5)*d_T9_dX+d_T5_dX*T9);
Denomi = ((0.6+T5)*T9);
}
d_ueff_dX = ((Denomi*d_u0temp_dX-u0temp*d_Denomi_dX)/Denomi/Denomi);
ueff = (u0temp/Denomi);
instanceStruct.d_instanceVar_B4SOIueff_dX = d_ueff_dX;
instanceStruct.instanceVar_B4SOIueff = ueff;
d_WVCox_dX = ((Weff*vsattemp)*instanceStruct.d_instanceVar_B4SOIcox_dX+(Weff*d_vsattemp_dX+d_Weff_dX*vsattemp)*instanceStruct.instanceVar_B4SOIcox);
WVCox = ((Weff*vsattemp)*instanceStruct.instanceVar_B4SOIcox);
d_WVCoxRds_dX = (WVCox*d_Rds_dX+d_WVCox_dX*Rds);
WVCoxRds = (WVCox*Rds);
d_Esat_dX = ((ueff*2.0*d_vsattemp_dX-(2.0*vsattemp)*d_ueff_dX)/ueff/ueff);
Esat = ((2.0*vsattemp)/ueff);
d_EsatL_dX = (Esat*d_Leff_dX+d_Esat_dX*Leff);
EsatL = (Esat*Leff);
if ((instanceStruct.instanceVar_pParam_B4SOIa1==0.0))
{
d_Lambda_dX = instanceStruct.d_instanceVar_pParam_B4SOIa2_dX;
Lambda = instanceStruct.instanceVar_pParam_B4SOIa2;
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOIa1>0.0))
{
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIa2_dX);
T0 = (1.0-instanceStruct.instanceVar_pParam_B4SOIa2);
d_T1_dX = (d_T0_dX-(instanceStruct.instanceVar_pParam_B4SOIa1*d_Vgsteff_dX+instanceStruct.d_instanceVar_pParam_B4SOIa1_dX*Vgsteff));
T1 = ((T0-(instanceStruct.instanceVar_pParam_B4SOIa1*Vgsteff))-0.0001);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.0004*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+0.0004*d_T0_dX));
T2 = value_sqrt_0;
}
d_Lambda_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIa2_dX+d_T0_dX)-0.5*(d_T1_dX+d_T2_dX));
Lambda = ((instanceStruct.instanceVar_pParam_B4SOIa2+T0)-(0.5*(T1+T2)));
}
else
{
d_T1_dX = (instanceStruct.d_instanceVar_pParam_B4SOIa2_dX+(instanceStruct.instanceVar_pParam_B4SOIa1*d_Vgsteff_dX+instanceStruct.d_instanceVar_pParam_B4SOIa1_dX*Vgsteff));
T1 = ((instanceStruct.instanceVar_pParam_B4SOIa2+(instanceStruct.instanceVar_pParam_B4SOIa1*Vgsteff))-0.0001);
{
double value_sqrt_0 = sqrt(((T1*T1)+(0.0004*instanceStruct.instanceVar_pParam_B4SOIa2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+0.0004*instanceStruct.d_instanceVar_pParam_B4SOIa2_dX));
T2 = value_sqrt_0;
}
d_Lambda_dX = 0.5*(d_T1_dX+d_T2_dX);
Lambda = (0.5*(T1+T2));
}
}
instanceStruct.d_instanceVar_B4SOIAbovVgst2Vtm_dX = ((Vgst2Vtm*d_Abulk_dX-Abulk*d_Vgst2Vtm_dX)/Vgst2Vtm/Vgst2Vtm);
instanceStruct.instanceVar_B4SOIAbovVgst2Vtm = (Abulk/Vgst2Vtm);
if (((Rds==0.0)&&(Lambda==1.0)))
{
d_T0_dX = (-((Abulk*d_EsatL_dX+d_Abulk_dX*EsatL)+d_Vgst2Vtm_dX)/((Abulk*EsatL)+Vgst2Vtm)/((Abulk*EsatL)+Vgst2Vtm));
T0 = (1.0/((Abulk*EsatL)+Vgst2Vtm));
d_T3_dX = (EsatL*d_Vgst2Vtm_dX+d_EsatL_dX*Vgst2Vtm);
T3 = (EsatL*Vgst2Vtm);
d_Vdsat_dX = (T3*d_T0_dX+d_T3_dX*T0);
Vdsat = (T3*T0);
}
else
{
d_T9_dX = (Abulk*d_WVCoxRds_dX+d_Abulk_dX*WVCoxRds);
T9 = (Abulk*WVCoxRds);
d_T7_dX = (Vgst2Vtm*d_T9_dX+d_Vgst2Vtm_dX*T9);
T7 = (Vgst2Vtm*T9);
d_T6_dX = (Vgst2Vtm*d_WVCoxRds_dX+d_Vgst2Vtm_dX*WVCoxRds);
T6 = (Vgst2Vtm*WVCoxRds);
d_T0_dX = ((2.0*Abulk)*(d_T9_dX+(-d_Lambda_dX/Lambda/Lambda))+2.0*d_Abulk_dX*((T9-1.0)+(1.0/Lambda)));
T0 = ((2.0*Abulk)*((T9-1.0)+(1.0/Lambda)));
d_T1_dX = (((Vgst2Vtm*(-2.0*d_Lambda_dX/Lambda/Lambda)+d_Vgst2Vtm_dX*((2.0/Lambda)-1.0))+(Abulk*d_EsatL_dX+d_Abulk_dX*EsatL))+3.0*d_T7_dX);
T1 = (((Vgst2Vtm*((2.0/Lambda)-1.0))+(Abulk*EsatL))+(3.0*T7));
d_T2_dX = (Vgst2Vtm*(d_EsatL_dX+2.0*d_T6_dX)+d_Vgst2Vtm_dX*(EsatL+(2.0*T6)));
T2 = (Vgst2Vtm*(EsatL+(2.0*T6)));
{
double value_sqrt_0 = sqrt(((T1*T1)-((2.0*T0)*T2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)-((2.0*T0)*d_T2_dX+2.0*d_T0_dX*T2)));
T3 = value_sqrt_0;
}
d_Vdsat_dX = ((T0*(d_T1_dX-d_T3_dX)-(T1-T3)*d_T0_dX)/T0/T0);
Vdsat = ((T1-T3)/T0);
}
d_T1_dX = (d_Vdsat_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelta_dX);
T1 = ((Vdsat-Vds)-instanceStruct.instanceVar_pParam_B4SOIdelta);
{
double value_sqrt_0 = sqrt(((T1*T1)+((4.0*instanceStruct.instanceVar_pParam_B4SOIdelta)*Vdsat)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+((4.0*instanceStruct.instanceVar_pParam_B4SOIdelta)*d_Vdsat_dX+4.0*instanceStruct.d_instanceVar_pParam_B4SOIdelta_dX*Vdsat)));
T2 = value_sqrt_0;
}
d_Vdseff_dX = (d_Vdsat_dX-0.5*(d_T1_dX+d_T2_dX));
Vdseff = (Vdsat-(0.5*(T1+T2)));
if ((Vdseff>Vds))
{
d_Vdseff_dX = 0.0;
Vdseff = Vds;
}
d_diffVds_dX = (-d_Vdseff_dX);
diffVds = (Vds-Vdseff);
instanceStruct.d_instanceVar_B4SOIVdseff_dX = d_Vdseff_dX;
instanceStruct.instanceVar_B4SOIVdseff = Vdseff;
d_tmp4_dX = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdsat_dX+0.5*d_Abulk_dX*Vdsat)-((0.5*Abulk)*Vdsat)*d_Vgst2Vtm_dX)/Vgst2Vtm/Vgst2Vtm));
tmp4 = (1.0-(((0.5*Abulk)*Vdsat)/Vgst2Vtm));
d_T9_dX = (WVCoxRds*d_Vgsteff_dX+d_WVCoxRds_dX*Vgsteff);
T9 = (WVCoxRds*Vgsteff);
d_T0_dX = ((d_EsatL_dX+d_Vdsat_dX)+((2.0*T9)*d_tmp4_dX+2.0*d_T9_dX*tmp4));
T0 = ((EsatL+Vdsat)+((2.0*T9)*tmp4));
d_T9_dX = (WVCoxRds*d_Abulk_dX+d_WVCoxRds_dX*Abulk);
T9 = (WVCoxRds*Abulk);
d_T1_dX = ((-2.0*d_Lambda_dX/Lambda/Lambda)+d_T9_dX);
T1 = (((2.0/Lambda)-1.0)+T9);
d_Vasat_dX = ((T1*d_T0_dX-T0*d_T1_dX)/T1/T1);
Vasat = (T0/T1);
if (((instanceStruct.instanceVar_pParam_B4SOIpclm>0.0)&&(diffVds>1.0e-10)))
{
d_T0_dX = (-((instanceStruct.instanceVar_pParam_B4SOIpclm*Abulk)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX+(instanceStruct.instanceVar_pParam_B4SOIpclm*d_Abulk_dX+instanceStruct.d_instanceVar_pParam_B4SOIpclm_dX*Abulk)*instanceStruct.instanceVar_pParam_B4SOIlitl)/((instanceStruct.instanceVar_pParam_B4SOIpclm*Abulk)*instanceStruct.instanceVar_pParam_B4SOIlitl)/((instanceStruct.instanceVar_pParam_B4SOIpclm*Abulk)*instanceStruct.instanceVar_pParam_B4SOIlitl));
T0 = (1.0/((instanceStruct.instanceVar_pParam_B4SOIpclm*Abulk)*instanceStruct.instanceVar_pParam_B4SOIlitl));
d_T2_dX = ((EsatL*d_Vgsteff_dX-Vgsteff*d_EsatL_dX)/EsatL/EsatL);
T2 = (Vgsteff/EsatL);
d_T1_dX = (Leff*(d_Abulk_dX+d_T2_dX)+d_Leff_dX*(Abulk+T2));
T1 = (Leff*(Abulk+T2));
d_T9_dX = (T0*d_T1_dX+d_T0_dX*T1);
T9 = (T0*T1);
d_VACLM_dX = (T9*d_diffVds_dX+d_T9_dX*diffVds);
VACLM = (T9*diffVds);
}
else
{
d_VACLM_dX = 0.0;
VACLM = 2.688117142e+43;
}
if ((thetaRout>0.0))
{
d_T8_dX = (Abulk*d_Vdsat_dX+d_Abulk_dX*Vdsat);
T8 = (Abulk*Vdsat);
d_T0_dX = (Vgst2Vtm*d_T8_dX+d_Vgst2Vtm_dX*T8);
T0 = (Vgst2Vtm*T8);
d_T1_dX = (d_Vgst2Vtm_dX+d_T8_dX);
T1 = (Vgst2Vtm+T8);
d_T2_dX = d_thetaRout_dX;
T2 = thetaRout;
d_VADIBL_dX = ((T2*(d_Vgst2Vtm_dX-((T1*d_T0_dX-T0*d_T1_dX)/T1/T1))-(Vgst2Vtm-(T0/T1))*d_T2_dX)/T2/T2);
VADIBL = ((Vgst2Vtm-(T0/T1))/T2);
d_T7_dX = (instanceStruct.instanceVar_pParam_B4SOIpdiblb*d_Vbseff_dX+instanceStruct.d_instanceVar_pParam_B4SOIpdiblb_dX*Vbseff);
T7 = (instanceStruct.instanceVar_pParam_B4SOIpdiblb*Vbseff);
if ((T7>=(-0.9)))
{
d_T3_dX = (-d_T7_dX/(1.0+T7)/(1.0+T7));
T3 = (1.0/(1.0+T7));
d_VADIBL_dX = (VADIBL*d_T3_dX+d_VADIBL_dX*T3);
VADIBL = (VADIBL*T3);
}
else
{
d_T4_dX = (-d_T7_dX/(0.8+T7)/(0.8+T7));
T4 = (1.0/(0.8+T7));
d_T3_dX = ((17.0+(20.0*T7))*d_T4_dX+20.0*d_T7_dX*T4);
T3 = ((17.0+(20.0*T7))*T4);
d_VADIBL_dX = (VADIBL*d_T3_dX+d_VADIBL_dX*T3);
VADIBL = (VADIBL*T3);
}
}
else
{
d_VADIBL_dX = 0.0;
VADIBL = 2.688117142e+43;
}
d_T0_dX = instanceStruct.d_instanceVar_pParam_B4SOIpditsd_dX*Vds;
T0 = (instanceStruct.instanceVar_pParam_B4SOIpditsd*Vds);
if ((T0>100.0))
{
d_T1_dX = 0.0;
T1 = 2.688117142e+43;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
if ((instanceStruct.instanceVar_pParam_B4SOIpdits>3.720075976e-44))
{
d_T2_dX = (instanceStruct.instanceVar_B4SOIpditsl*d_Leff_dX+instanceStruct.d_instanceVar_B4SOIpditsl_dX*Leff);
T2 = (1.0+(instanceStruct.instanceVar_B4SOIpditsl*Leff));
d_VADITS_dX = ((instanceStruct.instanceVar_pParam_B4SOIpdits*(T2*d_T1_dX+d_T2_dX*T1)-(1.0+(T2*T1))*instanceStruct.d_instanceVar_pParam_B4SOIpdits_dX)/instanceStruct.instanceVar_pParam_B4SOIpdits/instanceStruct.instanceVar_pParam_B4SOIpdits);
VADITS = ((1.0+(T2*T1))/instanceStruct.instanceVar_pParam_B4SOIpdits);
d_VADITS_dX = (VADITS*d_FP_dX+d_VADITS_dX*FP);
VADITS = (VADITS*FP);
}
else
{
d_VADITS_dX = 0.0;
VADITS = 2.688117142e+43;
}
d_T8_dX = ((EsatL*instanceStruct.d_instanceVar_pParam_B4SOIpvag_dX-instanceStruct.instanceVar_pParam_B4SOIpvag*d_EsatL_dX)/EsatL/EsatL);
T8 = (instanceStruct.instanceVar_pParam_B4SOIpvag/EsatL);
d_T9_dX = (T8*d_Vgsteff_dX+d_T8_dX*Vgsteff);
T9 = (T8*Vgsteff);
if ((T9>(-0.9)))
{
d_T0_dX = d_T9_dX;
T0 = (1.0+T9);
}
else
{
d_T1_dX = (-20.0*d_T9_dX/(17.0+(20.0*T9))/(17.0+(20.0*T9)));
T1 = (1.0/(17.0+(20.0*T9)));
d_T0_dX = ((0.8+T9)*d_T1_dX+d_T9_dX*T1);
T0 = ((0.8+T9)*T1);
}
d_tmp3_dX = (d_VACLM_dX+d_VADIBL_dX);
tmp3 = (VACLM+VADIBL);
d_T1_dX = ((tmp3*(VACLM*d_VADIBL_dX+d_VACLM_dX*VADIBL)-(VACLM*VADIBL)*d_tmp3_dX)/tmp3/tmp3);
T1 = ((VACLM*VADIBL)/tmp3);
d_tmp3_dX = (d_T1_dX+d_VADITS_dX);
tmp3 = (T1+VADITS);
d_T2_dX = ((tmp3*(T1*d_VADITS_dX+d_T1_dX*VADITS)-(T1*VADITS)*d_tmp3_dX)/tmp3/tmp3);
T2 = ((T1*VADITS)/tmp3);
d_Va_dX = (d_Vasat_dX+(T0*d_T2_dX+d_T0_dX*T2));
Va = (Vasat+(T0*T2));
d_CoxWovL_dX = ((Leff*(instanceStruct.instanceVar_B4SOIcox*d_Weff_dX+instanceStruct.d_instanceVar_B4SOIcox_dX*Weff)-(instanceStruct.instanceVar_B4SOIcox*Weff)*d_Leff_dX)/Leff/Leff);
CoxWovL = ((instanceStruct.instanceVar_B4SOIcox*Weff)/Leff);
d_beta_dX = (ueff*d_CoxWovL_dX+d_ueff_dX*CoxWovL);
beta = (ueff*CoxWovL);
d_T0_dX = (-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dX+0.5*d_Abulk_dX*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dX)/Vgst2Vtm/Vgst2Vtm));
T0 = (1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm));
d_fgche1_dX = (Vgsteff*d_T0_dX+d_Vgsteff_dX*T0);
fgche1 = (Vgsteff*T0);
d_T9_dX = ((EsatL*d_Vdseff_dX-Vdseff*d_EsatL_dX)/EsatL/EsatL);
T9 = (Vdseff/EsatL);
d_fgche2_dX = d_T9_dX;
fgche2 = (1.0+T9);
d_gche_dX = ((fgche2*(beta*d_fgche1_dX+d_beta_dX*fgche1)-(beta*fgche1)*d_fgche2_dX)/fgche2/fgche2);
gche = ((beta*fgche1)/fgche2);
d_T0_dX = (gche*d_Rds_dX+d_gche_dX*Rds);
T0 = (1.0+(gche*Rds));
d_T9_dX = ((T0*d_Vdseff_dX-Vdseff*d_T0_dX)/T0/T0);
T9 = (Vdseff/T0);
d_Idl_dX = (gche*d_T9_dX+d_gche_dX*T9);
Idl = (gche*T9);
d_IdlovVdseff_dX = ((T0*d_gche_dX-gche*d_T0_dX)/T0/T0);
IdlovVdseff = (gche/T0);
d_T9_dX = ((Va*d_diffVds_dX-diffVds*d_Va_dX)/Va/Va);
T9 = (diffVds/Va);
d_T0_dX = d_T9_dX;
T0 = (1.0+T9);
d_Ids_dX = ((instanceStruct.instanceVar_B4SOInseg*(Idl*d_T0_dX+d_Idl_dX*T0)-(Idl*T0)*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg);
Ids = ((Idl*T0)/instanceStruct.instanceVar_B4SOInseg);
instanceStruct.d_instanceVar_B4SOIidovVds_dX = ((instanceStruct.instanceVar_B4SOInseg*(IdlovVdseff*d_T0_dX+d_IdlovVdseff_dX*T0)-(IdlovVdseff*T0)*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg);
instanceStruct.instanceVar_B4SOIidovVds = ((IdlovVdseff*T0)/instanceStruct.instanceVar_B4SOInseg);
if ((instanceStruct.instanceVar_B4SOIidovVds<1e-9))
{
instanceStruct.d_instanceVar_B4SOIidovVds_dX = 0.0;
instanceStruct.instanceVar_B4SOIidovVds = 1e-9;
}
d_IdovVds_dX = ((instanceStruct.instanceVar_B4SOInseg*(IdlovVdseff*d_T0_dX+d_IdlovVdseff_dX*T0)-(IdlovVdseff*T0)*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg);
IdovVds = ((IdlovVdseff*T0)/instanceStruct.instanceVar_B4SOInseg);
if ((instanceStruct.instanceVar_B4SOIsoiMod!=2))
{
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_T0_dX = (((3.0*3.9)/instanceStruct.instanceVar_epsrox)*instanceStruct.d_instanceVar_toxe_dX+(-(3.0*3.9)*instanceStruct.d_instanceVar_epsrox_dX/instanceStruct.instanceVar_epsrox/instanceStruct.instanceVar_epsrox)*instanceStruct.instanceVar_toxe);
T0 = (((3.0*3.9)/instanceStruct.instanceVar_epsrox)*instanceStruct.instanceVar_toxe);
}
else
{
d_T0_dX = ((instanceStruct.instanceVar_epsrox*(instanceStruct.instanceVar_B4SOIepsrsub*instanceStruct.d_instanceVar_toxe_dX+instanceStruct.d_instanceVar_B4SOIepsrsub_dX*instanceStruct.instanceVar_toxe)-(instanceStruct.instanceVar_B4SOIepsrsub*instanceStruct.instanceVar_toxe)*instanceStruct.d_instanceVar_epsrox_dX)/instanceStruct.instanceVar_epsrox/instanceStruct.instanceVar_epsrox);
T0 = ((instanceStruct.instanceVar_B4SOIepsrsub*instanceStruct.instanceVar_toxe)/instanceStruct.instanceVar_epsrox);
}
if ((instanceStruct.instanceVar_B4SOIgidlMod==0))
{
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_T1_dX = ((T0*((-d_Vgd_eff_dX)-d_egisl_dX)-(((-Vds)-Vgd_eff)-egisl)*d_T0_dX)/T0/T0);
T1 = ((((-Vds)-Vgd_eff)-egisl)/T0);
}
else
{
d_T1_dX = ((T0*(((-d_Vgd_eff_dX)-d_egisl_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX)-((((-Vds)-Vgd_eff)-egisl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)*d_T0_dX)/T0/T0);
T1 = (((((-Vds)-Vgd_eff)-egisl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)/T0);
}
if ((((agisl<=0.0)||(bgisl<=0.0))||(cgisl<0.0)))
{
d_Igisl_dX = 0.0;
Igisl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);
d_T1_dX = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dX,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}
d_T2_dX = (((T1+1.0E-3)*d_bgisl_dX-bgisl*d_T1_dX)/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgisl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;
d_Igisl_dX = (((wdios*agisl)*T1)*deriv_exp_0_d0*((-d_T2_dX))+((wdios*agisl)*d_T1_dX+(wdios*d_agisl_dX+d_wdios_dX*agisl)*T1)*value_exp_0);
Igisl = (((wdios*agisl)*T1)*value_exp_0);
}
d_T4_dX = (Vbs*d_Vbs_dX+d_Vbs_dX*Vbs);
T4 = (Vbs*Vbs);
d_T5_dX = ((-Vbs)*d_T4_dX+(-d_Vbs_dX)*T4);
T5 = ((-Vbs)*T4);
{
double value_fabs_0 = fabs(T5);
double  deriv_fabs_0_d0 = (((T5)>=0)?(+1.0):(-1.0));
d_T6_dX = (d_cgisl_dX+deriv_fabs_0_d0*(d_T5_dX));
T6 = ((cgisl+value_fabs_0)+1.0E-9);
}
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0((T5/T6),1.0E-6);
d_T7_dX = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dX-T5*d_T6_dX)/T6/T6),0.0);
T7 = (evaluator_hypsmooth_0.getValues((T5/T6),1.0E-6)-1.0E-6);
}
d_Igisl_dX = (Igisl*d_T7_dX+d_Igisl_dX*T7);
Igisl = (Igisl*T7);
}
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_T1_dX = ((T0*((-d_Vgs_eff_dX)-d_egidl_dX)-((Vds-Vgs_eff)-egidl)*d_T0_dX)/T0/T0);
T1 = (((Vds-Vgs_eff)-egidl)/T0);
}
else
{
d_T1_dX = ((T0*(((-d_Vgs_eff_dX)-d_egidl_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX)-(((Vds-Vgs_eff)-egidl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)*d_T0_dX)/T0/T0);
T1 = ((((Vds-Vgs_eff)-egidl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)/T0);
}
if ((((agidl<=0.0)||(bgidl<=0.0))||(cgidl<0.0)))
{
d_Igidl_dX = 0.0;
Igidl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);
d_T1_dX = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dX,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}
d_T2_dX = (((T1+1.0E-3)*d_bgidl_dX-bgidl*d_T1_dX)/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgidl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;
d_Igidl_dX = (((wdiod*agidl)*T1)*deriv_exp_0_d0*((-d_T2_dX))+((wdiod*agidl)*d_T1_dX+(wdiod*d_agidl_dX+d_wdiod_dX*agidl)*T1)*value_exp_0);
Igidl = (((wdiod*agidl)*T1)*value_exp_0);
}
d_T4_dX = 0.0;
T4 = (Vbd*Vbd);
d_T5_dX = (-Vbd)*d_T4_dX;
T5 = ((-Vbd)*T4);
{
double value_fabs_0 = fabs(T5);
double  deriv_fabs_0_d0 = (((T5)>=0)?(+1.0):(-1.0));
d_T6_dX = (d_cgidl_dX+deriv_fabs_0_d0*(d_T5_dX));
T6 = ((cgidl+value_fabs_0)+1.0E-9);
}
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0((T5/T6),1.0E-6);
d_T7_dX = evaluator_hypsmooth_0.getDerivs((T5/T6),1.0E-6,((T6*d_T5_dX-T5*d_T6_dX)/T6/T6),0.0);
T7 = (evaluator_hypsmooth_0.getValues((T5/T6),1.0E-6)-1.0E-6);
}
d_Igidl_dX = (Igidl*d_T7_dX+d_Igidl_dX*T7);
Igidl = (Igidl*T7);
}
}
else
{
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_T1_dX = ((T0*((-(rgisl*d_Vgd_eff_dX+d_rgisl_dX*Vgd_eff))-d_egisl_dX)-(((-Vds)-(rgisl*Vgd_eff))-egisl)*d_T0_dX)/T0/T0);
T1 = ((((-Vds)-(rgisl*Vgd_eff))-egisl)/T0);
}
else
{
d_T1_dX = ((T0*(((-(rgisl*d_Vgd_eff_dX+d_rgisl_dX*Vgd_eff))-d_egisl_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX)-((((-Vds)-(rgisl*Vgd_eff))-egisl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)*d_T0_dX)/T0/T0);
T1 = (((((-Vds)-(rgisl*Vgd_eff))-egisl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)/T0);
}
if ((((agisl<=0.0)||(bgisl<=0.0))||(cgisl<0.0)))
{
d_Igisl_dX = 0.0;
Igisl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);
d_T1_dX = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dX,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}
d_T2_dX = (((T1+1.0E-3)*d_bgisl_dX-bgisl*d_T1_dX)/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgisl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;
d_Igisl_dX = (((wdios*agisl)*T1)*deriv_exp_0_d0*((-d_T2_dX))+((wdios*agisl)*d_T1_dX+(wdios*d_agisl_dX+d_wdios_dX*agisl)*T1)*value_exp_0);
Igisl = (((wdios*agisl)*T1)*value_exp_0);
}
d_T4_dX = (d_Vbs_dX-d_fgisl_dX);
T4 = (Vbs-fgisl);
if ((T4>=((-1.0)/100.0)))
{
d_T5_dX = (-d_kgisl_dX)*100.0;
T5 = ((-kgisl)*100.0);
}
else
{
d_T5_dX = ((T4*d_kgisl_dX-kgisl*d_T4_dX)/T4/T4);
T5 = (kgisl/T4);
}
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;
d_T6_dX = deriv_exp_0_d0*(d_T5_dX);
T6 = value_exp_0;
}
d_Igisl_dX = (Igisl*d_T6_dX+d_Igisl_dX*T6);
Igisl = (Igisl*T6);
}
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_T1_dX = ((T0*((-(rgidl*d_Vgs_eff_dX+d_rgidl_dX*Vgs_eff))-d_egidl_dX)-((Vds-(rgidl*Vgs_eff))-egidl)*d_T0_dX)/T0/T0);
T1 = (((Vds-(rgidl*Vgs_eff))-egidl)/T0);
}
else
{
d_T1_dX = ((T0*(((-(rgidl*d_Vgs_eff_dX+d_rgidl_dX*Vgs_eff))-d_egidl_dX)+instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX)-(((Vds-(rgidl*Vgs_eff))-egidl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)*d_T0_dX)/T0/T0);
T1 = ((((Vds-(rgidl*Vgs_eff))-egidl)+instanceStruct.instanceVar_pParam_B4SOIvfbsd)/T0);
}
if ((((agidl<=0.0)||(bgidl<=0.0))||(cgidl<0.0)))
{
d_Igidl_dX = 0.0;
Igidl = 0.0;
}
else
{
{
AnalogFunctions::hypsmoothEvaluator evaluator_hypsmooth_0(T1,1.0E-2);
d_T1_dX = evaluator_hypsmooth_0.getDerivs(T1,1.0E-2,d_T1_dX,0.0);
T1 = evaluator_hypsmooth_0.getValues(T1,1.0E-2);
}
d_T2_dX = (((T1+1.0E-3)*d_bgidl_dX-bgidl*d_T1_dX)/(T1+1.0E-3)/(T1+1.0E-3));
T2 = (bgidl/(T1+1.0E-3));
{
double value_exp_0 = exp((-T2));
double  deriv_exp_0_d0 = value_exp_0;
d_Igidl_dX = (((wdiod*agidl)*T1)*deriv_exp_0_d0*((-d_T2_dX))+((wdiod*agidl)*d_T1_dX+(wdiod*d_agidl_dX+d_wdiod_dX*agidl)*T1)*value_exp_0);
Igidl = (((wdiod*agidl)*T1)*value_exp_0);
}
d_T4_dX = (-d_fgidl_dX);
T4 = (Vbd-fgidl);
if ((T4>=((-1.0)/100.0)))
{
d_T5_dX = (-d_kgidl_dX)*100.0;
T5 = ((-kgidl)*100.0);
}
else
{
d_T5_dX = ((T4*d_kgidl_dX-kgidl*d_T4_dX)/T4/T4);
T5 = (kgidl/T4);
}
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;
d_T6_dX = deriv_exp_0_d0*(d_T5_dX);
T6 = value_exp_0;
}
d_Igidl_dX = (Igidl*d_T6_dX+d_Igidl_dX*T6);
Igidl = (Igidl*T6);
}
}
d_WsTsi_dX = (instanceStruct.instanceVar_pParam_B4SOIwdios*instanceStruct.d_instanceVar_B4SOItsi_dX+instanceStruct.d_instanceVar_pParam_B4SOIwdios_dX*instanceStruct.instanceVar_B4SOItsi);
WsTsi = (instanceStruct.instanceVar_pParam_B4SOIwdios*instanceStruct.instanceVar_B4SOItsi);
d_WdTsi_dX = (instanceStruct.instanceVar_pParam_B4SOIwdiod*instanceStruct.d_instanceVar_B4SOItsi_dX+instanceStruct.d_instanceVar_pParam_B4SOIwdiod_dX*instanceStruct.instanceVar_B4SOItsi);
WdTsi = (instanceStruct.instanceVar_pParam_B4SOIwdiod*instanceStruct.instanceVar_B4SOItsi);
d_NVtm1_dX = (Vtm*instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX+d_Vtm_dX*instanceStruct.instanceVar_pParam_B4SOIndiode);
NVtm1 = (Vtm*instanceStruct.instanceVar_pParam_B4SOIndiode);
d_T0_dX = (-vsbs*d_NVtm1_dX/NVtm1/NVtm1);
T0 = (vsbs/NVtm1);
if ((T0>100.0))
{
d_ExpVbsNVtm_dX = 2.688117142e+43*d_T0_dX;
ExpVbsNVtm = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_ExpVbsNVtm_dX = 0.0;
ExpVbsNVtm = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVbsNVtm_dX = deriv_exp_0_d0*(d_T0_dX);
ExpVbsNVtm = value_exp_0;
}
}
}
d_NVtm1_dX = (Vtm*instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX+d_Vtm_dX*instanceStruct.instanceVar_pParam_B4SOIndioded);
NVtm1 = (Vtm*instanceStruct.instanceVar_pParam_B4SOIndioded);
d_T0_dX = (-vdbd*d_NVtm1_dX/NVtm1/NVtm1);
T0 = (vdbd/NVtm1);
if ((T0>100.0))
{
d_ExpVbdNVtm_dX = 2.688117142e+43*d_T0_dX;
ExpVbdNVtm = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_ExpVbdNVtm_dX = 0.0;
ExpVbdNVtm = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVbdNVtm_dX = deriv_exp_0_d0*(d_T0_dX);
ExpVbdNVtm = value_exp_0;
}
}
}
if ((jdifs==0))
{
d_Ibs1_dX = 0.0;
Ibs1 = 0;
}
else
{
d_T0_dX = (WsTsi*d_jdifs_dX+d_WsTsi_dX*jdifs);
T0 = (WsTsi*jdifs);
d_Ibs1_dX = (T0*d_ExpVbsNVtm_dX+d_T0_dX*(ExpVbsNVtm-1));
Ibs1 = (T0*(ExpVbsNVtm-1));
}
if ((jdifd==0))
{
d_Ibd1_dX = 0.0;
Ibd1 = 0;
}
else
{
d_T0_dX = (WdTsi*d_jdifd_dX+d_WdTsi_dX*jdifd);
T0 = (WdTsi*jdifd);
d_Ibd1_dX = (T0*d_ExpVbdNVtm_dX+d_T0_dX*(ExpVbdNVtm-1));
Ibd1 = (T0*(ExpVbdNVtm-1));
}
if ((jrecs==0))
{
d_Ibs2_dX = 0.0;
Ibs2 = 0;
}
else
{
d_NVtmf_dX = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecf0)*(instanceStruct.instanceVar_pParam_B4SOIntrecf*d_TempRatio_dX+instanceStruct.d_instanceVar_pParam_B4SOIntrecf_dX*(TempRatio-1))+(instanceStruct.instanceVar_Vtm00*instanceStruct.d_instanceVar_pParam_B4SOInrecf0_dX+instanceStruct.d_instanceVar_Vtm00_dX*instanceStruct.instanceVar_pParam_B4SOInrecf0)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecf*(TempRatio-1))));
NVtmf = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecf0)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecf*(TempRatio-1))));
d_NVtmr_dX = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecr0)*(instanceStruct.instanceVar_pParam_B4SOIntrecr*d_TempRatio_dX+instanceStruct.d_instanceVar_pParam_B4SOIntrecr_dX*(TempRatio-1))+(instanceStruct.instanceVar_Vtm00*instanceStruct.d_instanceVar_pParam_B4SOInrecr0_dX+instanceStruct.d_instanceVar_Vtm00_dX*instanceStruct.instanceVar_pParam_B4SOInrecr0)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecr*(TempRatio-1))));
NVtmr = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecr0)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecr*(TempRatio-1))));
d_T0_dX = (-vsbs*d_NVtmf_dX/NVtmf/NVtmf);
T0 = (vsbs/NVtmf);
if ((T0>100.0))
{
d_T10_dX = 2.688117142e+43*d_T0_dX;
T10 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T10_dX = 0.0;
T10 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T10_dX = deriv_exp_0_d0*(d_T0_dX);
T10 = value_exp_0;
}
}
}
if (((instanceStruct.instanceVar_pParam_B4SOIvrec0-vsbs)<1e-3))
{
d_T1_dX = 0.0;
T1 = 1e3;
d_T0_dX = ((((-vsbs)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0)*d_T1_dX+(((-vsbs)/NVtmr)*instanceStruct.d_instanceVar_pParam_B4SOIvrec0_dX+(-(-vsbs)*d_NVtmr_dX/NVtmr/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0)*T1);
T0 = ((((-vsbs)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0)*T1);
if ((T0>100.0))
{
d_T11_dX = 2.688117142e+43*d_T0_dX;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T11_dX = 0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T11_dX = deriv_exp_0_d0*(d_T0_dX);
T11 = value_exp_0;
}
}
}
d_T11_dX = (-d_T11_dX);
T11 = (-T11);
}
else
{
d_T1_dX = (-1*instanceStruct.d_instanceVar_pParam_B4SOIvrec0_dX/(instanceStruct.instanceVar_pParam_B4SOIvrec0-vsbs)/(instanceStruct.instanceVar_pParam_B4SOIvrec0-vsbs));
T1 = (1/(instanceStruct.instanceVar_pParam_B4SOIvrec0-vsbs));
d_T0_dX = ((((-vsbs)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0)*d_T1_dX+(((-vsbs)/NVtmr)*instanceStruct.d_instanceVar_pParam_B4SOIvrec0_dX+(-(-vsbs)*d_NVtmr_dX/NVtmr/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0)*T1);
T0 = ((((-vsbs)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0)*T1);
if ((T0>100.0))
{
d_T11_dX = 2.688117142e+43*d_T0_dX;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T11_dX = 0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T11_dX = deriv_exp_0_d0*(d_T0_dX);
T11 = value_exp_0;
}
}
}
d_T11_dX = (-d_T11_dX);
T11 = (-T11);
}
d_T3_dX = (WsTsi*d_jrecs_dX+d_WsTsi_dX*jrecs);
T3 = (WsTsi*jrecs);
d_Ibs2_dX = (T3*(d_T10_dX+d_T11_dX)+d_T3_dX*(T10+T11));
Ibs2 = (T3*(T10+T11));
}
if ((jrecd==0))
{
d_Ibd2_dX = 0.0;
Ibd2 = 0;
}
else
{
d_NVtmf_dX = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecf0d)*(instanceStruct.instanceVar_pParam_B4SOIntrecf*d_TempRatio_dX+instanceStruct.d_instanceVar_pParam_B4SOIntrecf_dX*(TempRatio-1))+(instanceStruct.instanceVar_Vtm00*instanceStruct.d_instanceVar_pParam_B4SOInrecf0d_dX+instanceStruct.d_instanceVar_Vtm00_dX*instanceStruct.instanceVar_pParam_B4SOInrecf0d)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecf*(TempRatio-1))));
NVtmf = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecf0d)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecf*(TempRatio-1))));
d_NVtmr_dX = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecr0d)*(instanceStruct.instanceVar_pParam_B4SOIntrecr*d_TempRatio_dX+instanceStruct.d_instanceVar_pParam_B4SOIntrecr_dX*(TempRatio-1))+(instanceStruct.instanceVar_Vtm00*instanceStruct.d_instanceVar_pParam_B4SOInrecr0d_dX+instanceStruct.d_instanceVar_Vtm00_dX*instanceStruct.instanceVar_pParam_B4SOInrecr0d)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecr*(TempRatio-1))));
NVtmr = ((instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOInrecr0d)*(1+(instanceStruct.instanceVar_pParam_B4SOIntrecr*(TempRatio-1))));
d_T0_dX = (-vdbd*d_NVtmf_dX/NVtmf/NVtmf);
T0 = (vdbd/NVtmf);
if ((T0>100.0))
{
d_T10_dX = 2.688117142e+43*d_T0_dX;
T10 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T10_dX = 0.0;
T10 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T10_dX = deriv_exp_0_d0*(d_T0_dX);
T10 = value_exp_0;
}
}
}
if (((instanceStruct.instanceVar_pParam_B4SOIvrec0d-vdbd)<1e-3))
{
d_T1_dX = 0.0;
T1 = 1e3;
d_T0_dX = ((((-vdbd)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0d)*d_T1_dX+(((-vdbd)/NVtmr)*instanceStruct.d_instanceVar_pParam_B4SOIvrec0d_dX+(-(-vdbd)*d_NVtmr_dX/NVtmr/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0d)*T1);
T0 = ((((-vdbd)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0d)*T1);
if ((T0>100.0))
{
d_T11_dX = 2.688117142e+43*d_T0_dX;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T11_dX = 0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T11_dX = deriv_exp_0_d0*(d_T0_dX);
T11 = value_exp_0;
}
}
}
d_T11_dX = (-d_T11_dX);
T11 = (-T11);
}
else
{
d_T1_dX = (-1*instanceStruct.d_instanceVar_pParam_B4SOIvrec0d_dX/(instanceStruct.instanceVar_pParam_B4SOIvrec0d-vdbd)/(instanceStruct.instanceVar_pParam_B4SOIvrec0d-vdbd));
T1 = (1/(instanceStruct.instanceVar_pParam_B4SOIvrec0d-vdbd));
d_T0_dX = ((((-vdbd)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0d)*d_T1_dX+(((-vdbd)/NVtmr)*instanceStruct.d_instanceVar_pParam_B4SOIvrec0d_dX+(-(-vdbd)*d_NVtmr_dX/NVtmr/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0d)*T1);
T0 = ((((-vdbd)/NVtmr)*instanceStruct.instanceVar_pParam_B4SOIvrec0d)*T1);
if ((T0>100.0))
{
d_T11_dX = 2.688117142e+43*d_T0_dX;
T11 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T11_dX = 0.0;
T11 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T11_dX = deriv_exp_0_d0*(d_T0_dX);
T11 = value_exp_0;
}
}
}
d_T11_dX = (-d_T11_dX);
T11 = (-T11);
}
d_T3_dX = (WdTsi*d_jrecd_dX+d_WdTsi_dX*jrecd);
T3 = (WdTsi*jrecd);
d_Ibd2_dX = (T3*(d_T10_dX+d_T11_dX)+d_T3_dX*(T10+T11));
Ibd2 = (T3*(T10+T11));
}
d_WTsi_dX = ((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.d_instanceVar_B4SOItsi_dX+((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX-instanceStruct.instanceVar_pParam_B4SOIweff*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOItsi);
WTsi = ((instanceStruct.instanceVar_pParam_B4SOIweff/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOItsi);
if (((jbjts==0.0)&&(jbjtd==0.0)))
{
d_Ibs3_dX = 0.0;
Ibs3 = 0.0;
d_Ibd3_dX = 0.0;
Ibd3 = 0.0;
d_Ibsdif_dX = 0.0;
Ibsdif = 0;
d_Ibddif_dX = 0.0;
Ibddif = 0;
d_Ic_dX = 0.0;
Ic = 0.0;
}
else
{
d_Ehlis_dX = (Ahlis*d_ExpVbsNVtm_dX+d_Ahlis_dX*(ExpVbsNVtm-1));
Ehlis = (Ahlis*(ExpVbsNVtm-1));
if ((Ehlis<1e-5))
{
d_Ehlis_dX = 0.0;
Ehlis = 0;
d_EhlisFactor_dX = 0.0;
EhlisFactor = 1;
}
else
{
{
double value_sqrt_0 = sqrt((1+Ehlis));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_EhlisFactor_dX = (-deriv_sqrt_0_d0*(d_Ehlis_dX)/value_sqrt_0/value_sqrt_0);
EhlisFactor = (1.0/value_sqrt_0);
}
}
d_Ehlid_dX = (Ahlid*d_ExpVbdNVtm_dX+d_Ahlid_dX*(ExpVbdNVtm-1));
Ehlid = (Ahlid*(ExpVbdNVtm-1));
if ((Ehlid<1e-5))
{
d_Ehlid_dX = 0.0;
Ehlid = 0;
d_EhlidFactor_dX = 0.0;
EhlidFactor = 1;
}
else
{
{
double value_sqrt_0 = sqrt((1+Ehlid));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_EhlidFactor_dX = (-deriv_sqrt_0_d0*(d_Ehlid_dX)/value_sqrt_0/value_sqrt_0);
EhlidFactor = (1.0/value_sqrt_0);
}
}
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIarfabjt_dX);
T0 = (1-instanceStruct.instanceVar_pParam_B4SOIarfabjt);
d_Ien_dX = ((WTsi*jbjts)*instanceStruct.d_instanceVar_pParam_B4SOIlratio_dX+(WTsi*d_jbjts_dX+d_WTsi_dX*jbjts)*instanceStruct.instanceVar_pParam_B4SOIlratio);
Ien = ((WTsi*jbjts)*instanceStruct.instanceVar_pParam_B4SOIlratio);
d_T1_dX = (T0*d_Ien_dX+d_T0_dX*Ien);
T1 = (T0*Ien);
d_Ibs3_dX = ((T1*(ExpVbsNVtm-1))*d_EhlisFactor_dX+(T1*d_ExpVbsNVtm_dX+d_T1_dX*(ExpVbsNVtm-1))*EhlisFactor);
Ibs3 = ((T1*(ExpVbsNVtm-1))*EhlisFactor);
d_Ien_dX = ((WTsi*jbjtd)*instanceStruct.d_instanceVar_pParam_B4SOIlratio_dX+(WTsi*d_jbjtd_dX+d_WTsi_dX*jbjtd)*instanceStruct.instanceVar_pParam_B4SOIlratio);
Ien = ((WTsi*jbjtd)*instanceStruct.instanceVar_pParam_B4SOIlratio);
d_T1_dX = (T0*d_Ien_dX+d_T0_dX*Ien);
T1 = (T0*Ien);
d_Ibd3_dX = ((T1*(ExpVbdNVtm-1))*d_EhlidFactor_dX+(T1*d_ExpVbdNVtm_dX+d_T1_dX*(ExpVbdNVtm-1))*EhlidFactor);
Ibd3 = ((T1*(ExpVbdNVtm-1))*EhlidFactor);
d_Iendif_dX = ((WTsi*jbjts)*instanceStruct.d_instanceVar_pParam_B4SOIlratiodif_dX+(WTsi*d_jbjts_dX+d_WTsi_dX*jbjts)*instanceStruct.instanceVar_pParam_B4SOIlratiodif);
Iendif = ((WTsi*jbjts)*instanceStruct.instanceVar_pParam_B4SOIlratiodif);
d_Ibsdif_dX = ((Iendif*(ExpVbsNVtm-1))*d_EhlisFactor_dX+(Iendif*d_ExpVbsNVtm_dX+d_Iendif_dX*(ExpVbsNVtm-1))*EhlisFactor);
Ibsdif = ((Iendif*(ExpVbsNVtm-1))*EhlisFactor);
d_Iendif_dX = ((WTsi*jbjtd)*instanceStruct.d_instanceVar_pParam_B4SOIlratiodif_dX+(WTsi*d_jbjtd_dX+d_WTsi_dX*jbjtd)*instanceStruct.instanceVar_pParam_B4SOIlratiodif);
Iendif = ((WTsi*jbjtd)*instanceStruct.instanceVar_pParam_B4SOIlratiodif);
d_Ibddif_dX = ((Iendif*(ExpVbdNVtm-1))*d_EhlidFactor_dX+(Iendif*d_ExpVbdNVtm_dX+d_Iendif_dX*(ExpVbdNVtm-1))*EhlidFactor);
Ibddif = ((Iendif*(ExpVbdNVtm-1))*EhlidFactor);
if ((instanceStruct.instanceVar_B4SOIbjtoff==1))
{
d_Ic_dX = 0.0;
Ic = 0.0;
}
else
{
d_T0_dX = (-(vsbs+vdbd)*instanceStruct.d_instanceVar_pParam_B4SOIvearly_dX/instanceStruct.instanceVar_pParam_B4SOIvearly/instanceStruct.instanceVar_pParam_B4SOIvearly);
T0 = (1+((vsbs+vdbd)/instanceStruct.instanceVar_pParam_B4SOIvearly));
d_T1_dX = (d_Ehlis_dX+d_Ehlid_dX);
T1 = (Ehlis+Ehlid);
{
double value_sqrt_0 = sqrt(((T0*T0)+(4*T1)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(((T0*d_T0_dX+d_T0_dX*T0)+4*d_T1_dX));
T3 = value_sqrt_0;
}
d_T2_dX = ((d_T0_dX+d_T3_dX)/2.0);
T2 = ((T0+T3)/2.0);
if ((T2<0.1))
{
d_E2ndFactor_dX = 0.0;
E2ndFactor = 10.0;
}
else
{
d_E2ndFactor_dX = (-d_T2_dX/T2/T2);
E2ndFactor = (1.0/T2);
}
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIarfabjt*d_Ien_dX+instanceStruct.d_instanceVar_pParam_B4SOIarfabjt_dX*Ien);
T0 = (instanceStruct.instanceVar_pParam_B4SOIarfabjt*Ien);
d_Ic_dX = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*d_E2ndFactor_dX+(T0*(d_ExpVbsNVtm_dX-d_ExpVbdNVtm_dX)+d_T0_dX*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
Ic = ((T0*(ExpVbsNVtm-ExpVbdNVtm))*E2ndFactor);
}
}
if (((jtuns==0)&&(jtund==0)))
{
d_Ibd4_dX = 0.0;
Ibd4 = 0;
d_Ibs4_dX = 0.0;
Ibs4 = 0;
}
else
{
d_NVtm2_dX = (instanceStruct.instanceVar_Vtm00*instanceStruct.d_instanceVar_pParam_B4SOIntun_dX+instanceStruct.d_instanceVar_Vtm00_dX*instanceStruct.instanceVar_pParam_B4SOIntun);
NVtm2 = (instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOIntun);
if (((instanceStruct.instanceVar_pParam_B4SOIvtun0-vsbs)<1e-3))
{
d_T1_dX = 0.0;
T1 = 1e3;
d_T0_dX = ((((-vsbs)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0)*d_T1_dX+(((-vsbs)/NVtm2)*instanceStruct.d_instanceVar_pParam_B4SOIvtun0_dX+(-(-vsbs)*d_NVtm2_dX/NVtm2/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0)*T1);
T0 = ((((-vsbs)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0)*T1);
if ((T0>100.0))
{
d_T1_dX = 2.688117142e+43*d_T0_dX;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
}
d_T3_dX = (WsTsi*d_jtuns_dX+d_WsTsi_dX*jtuns);
T3 = (WsTsi*jtuns);
d_Ibs4_dX = (T3*(-d_T1_dX)+d_T3_dX*(1-T1));
Ibs4 = (T3*(1-T1));
}
else
{
d_T1_dX = (-1*instanceStruct.d_instanceVar_pParam_B4SOIvtun0_dX/(instanceStruct.instanceVar_pParam_B4SOIvtun0-vsbs)/(instanceStruct.instanceVar_pParam_B4SOIvtun0-vsbs));
T1 = (1/(instanceStruct.instanceVar_pParam_B4SOIvtun0-vsbs));
d_T0_dX = ((((-vsbs)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0)*d_T1_dX+(((-vsbs)/NVtm2)*instanceStruct.d_instanceVar_pParam_B4SOIvtun0_dX+(-(-vsbs)*d_NVtm2_dX/NVtm2/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0)*T1);
T0 = ((((-vsbs)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0)*T1);
if ((T0>100.0))
{
d_T1_dX = 2.688117142e+43*d_T0_dX;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
}
d_T3_dX = (WsTsi*d_jtuns_dX+d_WsTsi_dX*jtuns);
T3 = (WsTsi*jtuns);
d_Ibs4_dX = (T3*(-d_T1_dX)+d_T3_dX*(1-T1));
Ibs4 = (T3*(1-T1));
}
d_NVtm2_dX = (instanceStruct.instanceVar_Vtm00*instanceStruct.d_instanceVar_pParam_B4SOIntund_dX+instanceStruct.d_instanceVar_Vtm00_dX*instanceStruct.instanceVar_pParam_B4SOIntund);
NVtm2 = (instanceStruct.instanceVar_Vtm00*instanceStruct.instanceVar_pParam_B4SOIntund);
if (((instanceStruct.instanceVar_pParam_B4SOIvtun0d-vdbd)<1e-3))
{
d_T1_dX = 0.0;
T1 = 1e3;
d_T0_dX = ((((-vdbd)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0d)*d_T1_dX+(((-vdbd)/NVtm2)*instanceStruct.d_instanceVar_pParam_B4SOIvtun0d_dX+(-(-vdbd)*d_NVtm2_dX/NVtm2/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0d)*T1);
T0 = ((((-vdbd)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0d)*T1);
if ((T0>100.0))
{
d_T1_dX = 2.688117142e+43*d_T0_dX;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
}
d_T3_dX = (WdTsi*d_jtund_dX+d_WdTsi_dX*jtund);
T3 = (WdTsi*jtund);
d_Ibd4_dX = (T3*(-d_T1_dX)+d_T3_dX*(1-T1));
Ibd4 = (T3*(1-T1));
}
else
{
d_T1_dX = (-1*instanceStruct.d_instanceVar_pParam_B4SOIvtun0d_dX/(instanceStruct.instanceVar_pParam_B4SOIvtun0d-vdbd)/(instanceStruct.instanceVar_pParam_B4SOIvtun0d-vdbd));
T1 = (1/(instanceStruct.instanceVar_pParam_B4SOIvtun0d-vdbd));
d_T0_dX = ((((-vdbd)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0d)*d_T1_dX+(((-vdbd)/NVtm2)*instanceStruct.d_instanceVar_pParam_B4SOIvtun0d_dX+(-(-vdbd)*d_NVtm2_dX/NVtm2/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0d)*T1);
T0 = ((((-vdbd)/NVtm2)*instanceStruct.instanceVar_pParam_B4SOIvtun0d)*T1);
if ((T0>100.0))
{
d_T1_dX = 2.688117142e+43*d_T0_dX;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
}
d_T3_dX = (WdTsi*d_jtund_dX+d_WdTsi_dX*jtund);
T3 = (WdTsi*jtund);
d_Ibd4_dX = (T3*(-d_T1_dX)+d_T3_dX*(1-T1));
Ibd4 = (T3*(1-T1));
}
}
d_Ibs_dX = (((d_Ibs1_dX+d_Ibs2_dX)+d_Ibs3_dX)+d_Ibs4_dX);
Ibs = (((Ibs1+Ibs2)+Ibs3)+Ibs4);
d_Ibd_dX = (((d_Ibd1_dX+d_Ibd2_dX)+d_Ibd3_dX)+d_Ibd4_dX);
Ibd = (((Ibd1+Ibd2)+Ibd3)+Ibd4);
}
else
{
d_Igidl_dX = 0.0;
Igidl = 0.0;
d_Igisl_dX = 0.0;
Igisl = 0.0;
d_Ibs_dX = 0.0;
Ibs = 0;
d_Ibd_dX = 0.0;
Ibd = 0;
d_Ibsdif_dX = 0.0;
Ibsdif = 0;
d_Ibddif_dX = 0.0;
Ibddif = 0;
d_Ic_dX = 0.0;
Ic = 0.0;
}
if (((instanceStruct.instanceVar_B4SOIigbMod!=0)||(instanceStruct.instanceVar_B4SOIigcMod!=0)))
{
d_Vgb_dX = (d_Vgs_eff_dX-d_Vbs_dX);
Vgb = (Vgs_eff-Vbs);
d_Vfb_dX = ((instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX-d_phi_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi));
Vfb = (((instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0)-phi)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi));
d_T3_dX = ((d_Vfb_dX-d_Vgs_eff_dX)+d_Vbs_dX);
T3 = (((Vfb-Vgs_eff)+Vbs)-0.02);
if ((Vfb<=0.0))
{
{
double value_sqrt_0 = sqrt(((T3*T3)-((4.0*0.02)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((T3*d_T3_dX+d_T3_dX*T3)-(4.0*0.02)*d_Vfb_dX));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((T3*T3)+((4.0*0.02)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((T3*d_T3_dX+d_T3_dX*T3)+(4.0*0.02)*d_Vfb_dX));
T0 = value_sqrt_0;
}
}
d_Vfbeff_dX = (d_Vfb_dX-0.5*(d_T3_dX+d_T0_dX));
Vfbeff = (Vfb-(0.5*(T3+T0)));
d_Voxacc_dX = (d_Vfb_dX-d_Vfbeff_dX);
Voxacc = (Vfb-Vfbeff);
if ((Voxacc<0.0))
{
d_Voxacc_dX = 0.0;
Voxacc = 0.0;
}
if ((instanceStruct.instanceVar_pParam_B4SOIk1ox==0.0))
{
d_Voxdepinv_dX = 0.0;
Voxdepinv = 0.0;
}
else
{
d_T0_dX = (((d_Vgs_eff_dX-d_Vgsteff_dX)-d_Vfbeff_dX)-d_Vbseff_dX);
T0 = (((Vgs_eff-Vgsteff)-Vfbeff)-Vbseff);
if ((T0<0.0))
{
d_T1_dX = ((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T0_dX-T0*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX)/instanceStruct.instanceVar_pParam_B4SOIk1ox/instanceStruct.instanceVar_pParam_B4SOIk1ox);
T1 = (T0/instanceStruct.instanceVar_pParam_B4SOIk1ox);
}
else
{
{
double value_sqrt_0 = sqrt((1+(((4*T0)/instanceStruct.instanceVar_pParam_B4SOIk1ox)/instanceStruct.instanceVar_pParam_B4SOIk1ox)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = ((instanceStruct.instanceVar_pParam_B4SOIk1ox/2)*deriv_sqrt_0_d0*(((instanceStruct.instanceVar_pParam_B4SOIk1ox*((instanceStruct.instanceVar_pParam_B4SOIk1ox*4*d_T0_dX-(4*T0)*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX)/instanceStruct.instanceVar_pParam_B4SOIk1ox/instanceStruct.instanceVar_pParam_B4SOIk1ox)-((4*T0)/instanceStruct.instanceVar_pParam_B4SOIk1ox)*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX)/instanceStruct.instanceVar_pParam_B4SOIk1ox/instanceStruct.instanceVar_pParam_B4SOIk1ox))+(instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX/2)*((-1)+value_sqrt_0));
T1 = ((instanceStruct.instanceVar_pParam_B4SOIk1ox/2)*((-1)+value_sqrt_0));
}
}
d_Voxdepinv_dX = ((d_Vgs_eff_dX-((T1*d_T1_dX+d_T1_dX*T1)+d_Vbs_dX))-d_Vfb_dX);
Voxdepinv = ((Vgs_eff-((T1*T1)+Vbs))-Vfb);
}
}
else
{
d_Vfb_dX = 0.0;
Vfb = 0;
d_Vgb_dX = 0.0;
Vgb = 0;
d_Voxacc_dX = 0.0;
Voxacc = 0;
d_Voxdepinv_dX = 0.0;
Voxdepinv = 0;
}
if (instanceStruct.instanceVar_B4SOIigcMod)
{
d_T0_dX = (Vtm*instanceStruct.d_instanceVar_pParam_B4SOInigc_dX+d_Vtm_dX*instanceStruct.instanceVar_pParam_B4SOInigc);
T0 = (Vtm*instanceStruct.instanceVar_pParam_B4SOInigc);
d_VxNVt_dX = ((T0*(d_Vgs_eff_dX-instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX)-(Vgs_eff-(instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0))*d_T0_dX)/T0/T0);
VxNVt = ((Vgs_eff-(instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0))/T0);
if ((VxNVt>100.0))
{
d_Vaux_dX = (d_Vgs_eff_dX-instanceStruct.instanceVar_B4SOItype*instanceStruct.d_instanceVar_here_B4SOIvth0_dX);
Vaux = (Vgs_eff-(instanceStruct.instanceVar_B4SOItype*instanceStruct.instanceVar_here_B4SOIvth0));
}
else
{
if ((VxNVt<(-100.0)))
{
{
double value_log_0 = log((1.0+3.720075976e-44));
d_Vaux_dX = d_T0_dX*value_log_0;
Vaux = (T0*value_log_0);
}
}
else
{
{
double value_exp_0 = exp(VxNVt);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVxNVt_dX = deriv_exp_0_d0*(d_VxNVt_dX);
ExpVxNVt = value_exp_0;
}
{
double value_log_0 = log((1.0+ExpVxNVt));
double  deriv_log_0_d0 = (1.0/(1.0+ExpVxNVt));
d_Vaux_dX = (T0*deriv_log_0_d0*(d_ExpVxNVt_dX)+d_T0_dX*value_log_0);
Vaux = (T0*value_log_0);
}
}
}
d_T2_dX = (Vgs_eff*d_Vaux_dX+d_Vgs_eff_dX*Vaux);
T2 = (Vgs_eff*Vaux);
d_T11_dX = instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX;
T11 = instanceStruct.instanceVar_pParam_B4SOIAechvb;
d_T12_dX = instanceStruct.d_instanceVar_pParam_B4SOIBechvb_dX;
T12 = instanceStruct.instanceVar_pParam_B4SOIBechvb;
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIaigc*instanceStruct.d_instanceVar_pParam_B4SOIcigc_dX+instanceStruct.d_instanceVar_pParam_B4SOIaigc_dX*instanceStruct.instanceVar_pParam_B4SOIcigc)-instanceStruct.d_instanceVar_pParam_B4SOIbigc_dX);
T3 = ((instanceStruct.instanceVar_pParam_B4SOIaigc*instanceStruct.instanceVar_pParam_B4SOIcigc)-instanceStruct.instanceVar_pParam_B4SOIbigc);
d_T4_dX = (instanceStruct.instanceVar_pParam_B4SOIbigc*instanceStruct.d_instanceVar_pParam_B4SOIcigc_dX+instanceStruct.d_instanceVar_pParam_B4SOIbigc_dX*instanceStruct.instanceVar_pParam_B4SOIcigc);
T4 = (instanceStruct.instanceVar_pParam_B4SOIbigc*instanceStruct.instanceVar_pParam_B4SOIcigc);
d_T5_dX = (T12*((instanceStruct.d_instanceVar_pParam_B4SOIaigc_dX+(T3*d_Voxdepinv_dX+d_T3_dX*Voxdepinv))-((T4*Voxdepinv)*d_Voxdepinv_dX+(T4*d_Voxdepinv_dX+d_T4_dX*Voxdepinv)*Voxdepinv))+d_T12_dX*((instanceStruct.instanceVar_pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
T5 = (T12*((instanceStruct.instanceVar_pParam_B4SOIaigc+(T3*Voxdepinv))-((T4*Voxdepinv)*Voxdepinv)));
if ((T5>100.0))
{
d_T6_dX = 0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{
d_T6_dX = 0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;
d_T6_dX = deriv_exp_0_d0*(d_T5_dX);
T6 = value_exp_0;
}
}
}
d_Igc_dX = ((T11*T2)*d_T6_dX+(T11*d_T2_dX+d_T11_dX*T2)*T6);
Igc = ((T11*T2)*T6);
d_T7_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIpigcd_dX)*Vds;
T7 = ((-instanceStruct.instanceVar_pParam_B4SOIpigcd)*Vds);
d_T8_dX = (T7*d_T7_dX+d_T7_dX*T7);
T8 = ((T7*T7)+2.0e-4);
if ((T7>100.0))
{
d_T9_dX = 0.0;
T9 = 2.688117142e+43;
}
else
{
if ((T7<(-100.0)))
{
d_T9_dX = 0.0;
T9 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T7);
double  deriv_exp_0_d0 = value_exp_0;
d_T9_dX = deriv_exp_0_d0*(d_T7_dX);
T9 = value_exp_0;
}
}
}
d_T1_dX = d_T9_dX;
T1 = ((T9-1.0)+1.0e-4);
d_T10_dX = ((T8*(d_T1_dX-d_T7_dX)-(T1-T7)*d_T8_dX)/T8/T8);
T10 = ((T1-T7)/T8);
d_Igcs_dX = (Igc*d_T10_dX+d_Igc_dX*T10);
Igcs = (Igc*T10);
d_T1_dX = d_T9_dX;
T1 = ((T9-1.0)-1.0e-4);
d_T10_dX = ((T8*((T7*d_T9_dX+d_T7_dX*T9)-d_T1_dX)-((T7*T9)-T1)*d_T8_dX)/T8/T8);
T10 = (((T7*T9)-T1)/T8);
d_Igcd_dX = (Igc*d_T10_dX+d_Igc_dX*T10);
Igcd = (Igc*T10);
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX);
T0 = (vgs-instanceStruct.instanceVar_pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_vgs_eff_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
vgs_eff = value_sqrt_0;
}
d_T2_dX = vgs*d_vgs_eff_dX;
T2 = (vgs*vgs_eff);
d_T13_dX = instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdges_dX;
T13 = instanceStruct.instanceVar_pParam_B4SOIAechvbEdges;
d_T14_dX = instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdged_dX;
T14 = instanceStruct.instanceVar_pParam_B4SOIAechvbEdged;
d_T12_dX = instanceStruct.d_instanceVar_pParam_B4SOIBechvbEdge_dX;
T12 = instanceStruct.instanceVar_pParam_B4SOIBechvbEdge;
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIaigsd*instanceStruct.d_instanceVar_pParam_B4SOIcigsd_dX+instanceStruct.d_instanceVar_pParam_B4SOIaigsd_dX*instanceStruct.instanceVar_pParam_B4SOIcigsd)-instanceStruct.d_instanceVar_pParam_B4SOIbigsd_dX);
T3 = ((instanceStruct.instanceVar_pParam_B4SOIaigsd*instanceStruct.instanceVar_pParam_B4SOIcigsd)-instanceStruct.instanceVar_pParam_B4SOIbigsd);
d_T4_dX = (instanceStruct.instanceVar_pParam_B4SOIbigsd*instanceStruct.d_instanceVar_pParam_B4SOIcigsd_dX+instanceStruct.d_instanceVar_pParam_B4SOIbigsd_dX*instanceStruct.instanceVar_pParam_B4SOIcigsd);
T4 = (instanceStruct.instanceVar_pParam_B4SOIbigsd*instanceStruct.instanceVar_pParam_B4SOIcigsd);
d_T5_dX = (T12*((instanceStruct.d_instanceVar_pParam_B4SOIaigsd_dX+(T3*d_vgs_eff_dX+d_T3_dX*vgs_eff))-((T4*vgs_eff)*d_vgs_eff_dX+(T4*d_vgs_eff_dX+d_T4_dX*vgs_eff)*vgs_eff))+d_T12_dX*((instanceStruct.instanceVar_pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
T5 = (T12*((instanceStruct.instanceVar_pParam_B4SOIaigsd+(T3*vgs_eff))-((T4*vgs_eff)*vgs_eff)));
if ((T5>100.0))
{
d_T6_dX = 0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{
d_T6_dX = 0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;
d_T6_dX = deriv_exp_0_d0*(d_T5_dX);
T6 = value_exp_0;
}
}
}
d_Igs_dX = ((T13*T2)*d_T6_dX+(T13*d_T2_dX+d_T13_dX*T2)*T6);
Igs = ((T13*T2)*T6);
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX);
T0 = (vgd-instanceStruct.instanceVar_pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_vgd_eff_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
vgd_eff = value_sqrt_0;
}
d_T2_dX = vgd*d_vgd_eff_dX;
T2 = (vgd*vgd_eff);
d_T5_dX = (T12*((instanceStruct.d_instanceVar_pParam_B4SOIaigsd_dX+(T3*d_vgd_eff_dX+d_T3_dX*vgd_eff))-((T4*vgd_eff)*d_vgd_eff_dX+(T4*d_vgd_eff_dX+d_T4_dX*vgd_eff)*vgd_eff))+d_T12_dX*((instanceStruct.instanceVar_pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
T5 = (T12*((instanceStruct.instanceVar_pParam_B4SOIaigsd+(T3*vgd_eff))-((T4*vgd_eff)*vgd_eff)));
if ((T5>100.0))
{
d_T6_dX = 0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{
d_T6_dX = 0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;
d_T6_dX = deriv_exp_0_d0*(d_T5_dX);
T6 = value_exp_0;
}
}
}
d_Igd_dX = ((T14*T2)*d_T6_dX+(T14*d_T2_dX+d_T14_dX*T2)*T6);
Igd = ((T14*T2)*T6);
}
else
{
d_Igd_dX = 0.0;
Igd = 0.0;
d_Igs_dX = 0.0;
Igs = 0.0;
d_Igcd_dX = 0.0;
Igcd = 0.0;
d_Igcs_dX = 0.0;
Igcs = 0.0;
}
if (((instanceStruct.instanceVar_B4SOIigbMod!=0)&&(instanceStruct.instanceVar_B4SOIsoiMod!=2)))
{
d_OxideRatio_dX = instanceStruct.d_instanceVar_pParam_B4SOIoxideRatio_dX;
OxideRatio = instanceStruct.instanceVar_pParam_B4SOIoxideRatio;
d_Vox_dX = d_Voxdepinv_dX;
Vox = Voxdepinv;
d_T0_dX = instanceStruct.d_instanceVar_B4SOIvoxh_dX;
T0 = instanceStruct.instanceVar_B4SOIvoxh;
d_T1_dX = ((d_T0_dX-d_Vox_dX)-instanceStruct.d_instanceVar_B4SOIdeltavox_dX);
T1 = ((T0-Vox)-instanceStruct.instanceVar_B4SOIdeltavox);
{
double value_sqrt_0 = sqrt(((T1*T1)+((4*instanceStruct.instanceVar_B4SOIdeltavox)*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+((4*instanceStruct.instanceVar_B4SOIdeltavox)*d_T0_dX+4*instanceStruct.d_instanceVar_B4SOIdeltavox_dX*T0)));
T3 = value_sqrt_0;
}
d_Voxeff_dX = (d_T0_dX-0.5*(d_T1_dX+d_T3_dX));
Voxeff = (T0-(0.5*(T1+T3)));
d_Vox_dX = d_Voxeff_dX;
Vox = Voxeff;
d_T0_dX = ((instanceStruct.instanceVar_B4SOIvevb*(d_Vox_dX-instanceStruct.d_instanceVar_B4SOIebg_dX)-(Vox-instanceStruct.instanceVar_B4SOIebg)*instanceStruct.d_instanceVar_B4SOIvevb_dX)/instanceStruct.instanceVar_B4SOIvevb/instanceStruct.instanceVar_B4SOIvevb);
T0 = ((Vox-instanceStruct.instanceVar_B4SOIebg)/instanceStruct.instanceVar_B4SOIvevb);
if ((T0>100.0))
{
d_T1_dX = 2.688117142e+43*d_T0_dX;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
}
{
double value_log_0 = log((1+T1));
double  deriv_log_0_d0 = (1.0/(1+T1));
d_Vaux_dX = (instanceStruct.instanceVar_B4SOIvevb*deriv_log_0_d0*(d_T1_dX)+instanceStruct.d_instanceVar_B4SOIvevb_dX*value_log_0);
Vaux = (instanceStruct.instanceVar_B4SOIvevb*value_log_0);
}
if ((instanceStruct.instanceVar_B4SOIvgb1!=0))
{
d_T0_dX = (-((instanceStruct.instanceVar_B4SOIvgb1*d_Vox_dX-Vox*instanceStruct.d_instanceVar_B4SOIvgb1_dX)/instanceStruct.instanceVar_B4SOIvgb1/instanceStruct.instanceVar_B4SOIvgb1));
T0 = (1-(Vox/instanceStruct.instanceVar_B4SOIvgb1));
}
else
{
d_T0_dX = 0.0;
T0 = 1;
}
if ((T0<0.01))
{
d_T0_dX = 0.0;
T0 = 0.01;
}
d_T1_dX = (((((Leff*Weff)/instanceStruct.instanceVar_B4SOInseg)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf))*instanceStruct.instanceVar_agb1)*d_OxideRatio_dX+((((Leff*Weff)/instanceStruct.instanceVar_B4SOInseg)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf))*instanceStruct.d_instanceVar_agb1_dX+(((instanceStruct.instanceVar_B4SOInseg*(Leff*d_Weff_dX+d_Leff_dX*Weff)-(Leff*Weff)*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+((instanceStruct.instanceVar_B4SOInf*instanceStruct.d_instanceVar_B4SOIagbcpd_dX-instanceStruct.instanceVar_B4SOIagbcpd*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf))*instanceStruct.instanceVar_agb1)*OxideRatio);
T1 = (((((Leff*Weff)/instanceStruct.instanceVar_B4SOInseg)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf))*instanceStruct.instanceVar_agb1)*OxideRatio);
d_T2_dX = (instanceStruct.instanceVar_bgb1*instanceStruct.d_instanceVar_B4SOItoxqm_dX+instanceStruct.d_instanceVar_bgb1_dX*instanceStruct.instanceVar_B4SOItoxqm);
T2 = (instanceStruct.instanceVar_bgb1*instanceStruct.instanceVar_B4SOItoxqm);
d_T3_dX = instanceStruct.d_instanceVar_pParam_B4SOIalphaGB1_dX;
T3 = instanceStruct.instanceVar_pParam_B4SOIalphaGB1;
d_T4_dX = instanceStruct.d_instanceVar_pParam_B4SOIbetaGB1_dX;
T4 = instanceStruct.instanceVar_pParam_B4SOIbetaGB1;
d_T6_dX = ((T0*(T2*(d_T3_dX-(T4*d_Vox_dX+d_T4_dX*Vox))+d_T2_dX*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dX)/T0/T0);
T6 = ((T2*(T3-(T4*Vox)))/T0);
if ((T6>100.0))
{
d_T5_dX = 2.688117142e+43*d_T6_dX;
T5 = (2.688117142e+43*((1.0+T6)-100.0));
}
else
{
if ((T6<(-100.0)))
{
d_T5_dX = 0.0;
T5 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T6);
double  deriv_exp_0_d0 = value_exp_0;
d_T5_dX = deriv_exp_0_d0*(d_T6_dX);
T5 = value_exp_0;
}
}
}
d_Igb1_dX = (((T1*Vgb)*Vaux)*d_T5_dX+((T1*Vgb)*d_Vaux_dX+(T1*d_Vgb_dX+d_T1_dX*Vgb)*Vaux)*T5);
Igb1 = (((T1*Vgb)*Vaux)*T5);
d_Vox_dX = d_Voxacc_dX;
Vox = Voxacc;
d_T0_dX = instanceStruct.d_instanceVar_B4SOIvoxh_dX;
T0 = instanceStruct.instanceVar_B4SOIvoxh;
d_T1_dX = ((d_T0_dX-d_Vox_dX)-instanceStruct.d_instanceVar_B4SOIdeltavox_dX);
T1 = ((T0-Vox)-instanceStruct.instanceVar_B4SOIdeltavox);
{
double value_sqrt_0 = sqrt(((T1*T1)+((4*instanceStruct.instanceVar_B4SOIdeltavox)*T0)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = deriv_sqrt_0_d0*(((T1*d_T1_dX+d_T1_dX*T1)+((4*instanceStruct.instanceVar_B4SOIdeltavox)*d_T0_dX+4*instanceStruct.d_instanceVar_B4SOIdeltavox_dX*T0)));
T3 = value_sqrt_0;
}
d_Voxeff_dX = (d_T0_dX-0.5*(d_T1_dX+d_T3_dX));
Voxeff = (T0-(0.5*(T1+T3)));
d_Vox_dX = d_Voxeff_dX;
Vox = Voxeff;
d_T0_dX = ((instanceStruct.instanceVar_B4SOIvecb*((-d_Vgb_dX)+d_Vfb_dX)-((-Vgb)+Vfb)*instanceStruct.d_instanceVar_B4SOIvecb_dX)/instanceStruct.instanceVar_B4SOIvecb/instanceStruct.instanceVar_B4SOIvecb);
T0 = (((-Vgb)+Vfb)/instanceStruct.instanceVar_B4SOIvecb);
if ((T0>100.0))
{
d_T1_dX = 2.688117142e+43*d_T0_dX;
T1 = (2.688117142e+43*((1.0+T0)-100.0));
}
else
{
if ((T0<(-100.0)))
{
d_T1_dX = 0.0;
T1 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_T1_dX = deriv_exp_0_d0*(d_T0_dX);
T1 = value_exp_0;
}
}
}
{
double value_log_0 = log((1+T1));
double  deriv_log_0_d0 = (1.0/(1+T1));
d_Vaux_dX = (instanceStruct.instanceVar_B4SOIvecb*deriv_log_0_d0*(d_T1_dX)+instanceStruct.d_instanceVar_B4SOIvecb_dX*value_log_0);
Vaux = (instanceStruct.instanceVar_B4SOIvecb*value_log_0);
}
if ((instanceStruct.instanceVar_B4SOIvgb2!=0))
{
d_T0_dX = (-((instanceStruct.instanceVar_B4SOIvgb2*d_Vox_dX-Vox*instanceStruct.d_instanceVar_B4SOIvgb2_dX)/instanceStruct.instanceVar_B4SOIvgb2/instanceStruct.instanceVar_B4SOIvgb2));
T0 = (1-(Vox/instanceStruct.instanceVar_B4SOIvgb2));
}
else
{
d_T0_dX = 0.0;
T0 = 1;
}
if ((T0<0.01))
{
d_T0_dX = 0.0;
T0 = 0.01;
}
d_T1_dX = (((((Leff*Weff)/instanceStruct.instanceVar_B4SOInseg)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf))*instanceStruct.instanceVar_agb2)*d_OxideRatio_dX+((((Leff*Weff)/instanceStruct.instanceVar_B4SOInseg)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf))*instanceStruct.d_instanceVar_agb2_dX+(((instanceStruct.instanceVar_B4SOInseg*(Leff*d_Weff_dX+d_Leff_dX*Weff)-(Leff*Weff)*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)+((instanceStruct.instanceVar_B4SOInf*instanceStruct.d_instanceVar_B4SOIagbcpd_dX-instanceStruct.instanceVar_B4SOIagbcpd*instanceStruct.d_instanceVar_B4SOInf_dX)/instanceStruct.instanceVar_B4SOInf/instanceStruct.instanceVar_B4SOInf))*instanceStruct.instanceVar_agb2)*OxideRatio);
T1 = (((((Leff*Weff)/instanceStruct.instanceVar_B4SOInseg)+(instanceStruct.instanceVar_B4SOIagbcpd/instanceStruct.instanceVar_B4SOInf))*instanceStruct.instanceVar_agb2)*OxideRatio);
d_T2_dX = (instanceStruct.instanceVar_bgb2*instanceStruct.d_instanceVar_B4SOItoxqm_dX+instanceStruct.d_instanceVar_bgb2_dX*instanceStruct.instanceVar_B4SOItoxqm);
T2 = (instanceStruct.instanceVar_bgb2*instanceStruct.instanceVar_B4SOItoxqm);
d_T3_dX = instanceStruct.d_instanceVar_pParam_B4SOIalphaGB2_dX;
T3 = instanceStruct.instanceVar_pParam_B4SOIalphaGB2;
d_T4_dX = instanceStruct.d_instanceVar_pParam_B4SOIbetaGB2_dX;
T4 = instanceStruct.instanceVar_pParam_B4SOIbetaGB2;
d_T6_dX = ((T0*(T2*(d_T3_dX-(T4*d_Vox_dX+d_T4_dX*Vox))+d_T2_dX*(T3-(T4*Vox)))-(T2*(T3-(T4*Vox)))*d_T0_dX)/T0/T0);
T6 = ((T2*(T3-(T4*Vox)))/T0);
if ((T6>100.0))
{
d_T5_dX = 2.688117142e+43*d_T6_dX;
T5 = (2.688117142e+43*((1.0+T6)-100.0));
}
else
{
if ((T6<(-100.0)))
{
d_T5_dX = 0.0;
T5 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T6);
double  deriv_exp_0_d0 = value_exp_0;
d_T5_dX = deriv_exp_0_d0*(d_T6_dX);
T5 = value_exp_0;
}
}
}
d_Igb2_dX = (((T1*Vgb)*Vaux)*d_T5_dX+((T1*Vgb)*d_Vaux_dX+(T1*d_Vgb_dX+d_T1_dX*Vgb)*Vaux)*T5);
Igb2 = (((T1*Vgb)*Vaux)*T5);
if ((Vgb>=0))
{
d_Igb_dX = d_Igb1_dX;
Igb = Igb1;
}
else
{
d_Igb_dX = d_Igb2_dX;
Igb = Igb2;
}
d_Vfb2_dX = (d_Vfb_dX+instanceStruct.d_instanceVar_eggbcp2_dX);
Vfb2 = (Vfb+instanceStruct.instanceVar_eggbcp2);
}
else
{
d_Igb_dX = 0.0;
Igb = 0.0;
}
d_B4SOIig_dX = instanceStruct.instanceVar_B4SOItype*d_Igb_dX;
B4SOIig = (instanceStruct.instanceVar_B4SOItype*Igb);
if ((((((instanceStruct.instanceVar_B4SOIigbMod!=0)&&(instanceStruct.instanceVar_B4SOIsoiMod!=2))&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0))&&(vgp<Vfb2)))
{
d_T0_dX = (-d_Vfb2_dX);
T0 = (vgp-Vfb2);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
T1 = value_sqrt_0;
}
d_vgp_eff_dX = 0.5*((-d_T0_dX)+d_T1_dX);
vgp_eff = (0.5*(((-T0)+T1)-1.0e-2));
d_T11_dX = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.d_instanceVar_agbc2n_dX:instanceStruct.d_instanceVar_agbc2p_dX);
T11 = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.instanceVar_agbc2n:instanceStruct.instanceVar_agbc2p);
d_T12_dX = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.d_instanceVar_bgbc2n_dX:instanceStruct.d_instanceVar_bgbc2p_dX);
T12 = ((instanceStruct.instanceVar_B4SOItype==1)?instanceStruct.instanceVar_bgbc2n:instanceStruct.instanceVar_bgbc2p);
d_T2_dX = vgp*d_vgp_eff_dX;
T2 = (vgp*vgp_eff);
d_T3_dX = ((instanceStruct.instanceVar_pParam_B4SOIaigbcp2*instanceStruct.d_instanceVar_pParam_B4SOIcigbcp2_dX+instanceStruct.d_instanceVar_pParam_B4SOIaigbcp2_dX*instanceStruct.instanceVar_pParam_B4SOIcigbcp2)-instanceStruct.d_instanceVar_pParam_B4SOIbigbcp2_dX);
T3 = ((instanceStruct.instanceVar_pParam_B4SOIaigbcp2*instanceStruct.instanceVar_pParam_B4SOIcigbcp2)-instanceStruct.instanceVar_pParam_B4SOIbigbcp2);
d_T4_dX = (instanceStruct.instanceVar_pParam_B4SOIbigbcp2*instanceStruct.d_instanceVar_pParam_B4SOIcigbcp2_dX+instanceStruct.d_instanceVar_pParam_B4SOIbigbcp2_dX*instanceStruct.instanceVar_pParam_B4SOIcigbcp2);
T4 = (instanceStruct.instanceVar_pParam_B4SOIbigbcp2*instanceStruct.instanceVar_pParam_B4SOIcigbcp2);
d_T5_dX = (((-T12)*instanceStruct.instanceVar_B4SOItoxqm)*((instanceStruct.d_instanceVar_pParam_B4SOIaigbcp2_dX+(T3*d_vgp_eff_dX+d_T3_dX*vgp_eff))-((T4*vgp_eff)*d_vgp_eff_dX+(T4*d_vgp_eff_dX+d_T4_dX*vgp_eff)*vgp_eff))+((-T12)*instanceStruct.d_instanceVar_B4SOItoxqm_dX+(-d_T12_dX)*instanceStruct.instanceVar_B4SOItoxqm)*((instanceStruct.instanceVar_pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
T5 = (((-T12)*instanceStruct.instanceVar_B4SOItoxqm)*((instanceStruct.instanceVar_pParam_B4SOIaigbcp2+(T3*vgp_eff))-((T4*vgp_eff)*vgp_eff)));
if ((T5>100.0))
{
d_T6_dX = 0.0;
T6 = 2.688117142e+43;
}
else
{
if ((T5<(-100.0)))
{
d_T6_dX = 0.0;
T6 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T5);
double  deriv_exp_0_d0 = value_exp_0;
d_T6_dX = deriv_exp_0_d0*(d_T5_dX);
T6 = value_exp_0;
}
}
}
d_T11_dX = ((T11*instanceStruct.instanceVar_B4SOIagbcp2)*instanceStruct.d_instanceVar_pParam_B4SOIoxideRatio_dX+(T11*instanceStruct.d_instanceVar_B4SOIagbcp2_dX+d_T11_dX*instanceStruct.instanceVar_B4SOIagbcp2)*instanceStruct.instanceVar_pParam_B4SOIoxideRatio);
T11 = ((T11*instanceStruct.instanceVar_B4SOIagbcp2)*instanceStruct.instanceVar_pParam_B4SOIoxideRatio);
d_Ig_agbcp2_dX = ((T11*T2)*d_T6_dX+(T11*d_T2_dX+d_T11_dX*T2)*T6);
Ig_agbcp2 = ((T11*T2)*T6);
}
else
{
d_Ig_agbcp2_dX = 0.0;
Ig_agbcp2 = 0.0;
}
d_B4SOIigp_dX = instanceStruct.instanceVar_B4SOItype*d_Ig_agbcp2_dX;
B4SOIigp = (instanceStruct.instanceVar_B4SOItype*Ig_agbcp2);
if ((instanceStruct.instanceVar_B4SOIsoiMod!=2))
{
if ((instanceStruct.instanceVar_B4SOIiiiMod==0))
{
if ((instanceStruct.instanceVar_pParam_B4SOIalpha0<=0.0))
{
d_Iii_dX = 0.0;
Iii = 0.0;
}
else
{
d_Vdsatii0_dX = ((instanceStruct.instanceVar_pParam_B4SOIvdsatii0*(instanceStruct.instanceVar_B4SOItii*d_TempRatio_dX+instanceStruct.d_instanceVar_B4SOItii_dX*(TempRatio-1.0))+instanceStruct.d_instanceVar_pParam_B4SOIvdsatii0_dX*(1+(instanceStruct.instanceVar_B4SOItii*(TempRatio-1.0))))-((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlii_dX-instanceStruct.instanceVar_pParam_B4SOIlii*d_Leff_dX)/Leff/Leff));
Vdsatii0 = ((instanceStruct.instanceVar_pParam_B4SOIvdsatii0*(1+(instanceStruct.instanceVar_B4SOItii*(TempRatio-1.0))))-(instanceStruct.instanceVar_pParam_B4SOIlii/Leff));
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIesatii*d_Leff_dX+instanceStruct.d_instanceVar_pParam_B4SOIesatii_dX*Leff);
T0 = (instanceStruct.instanceVar_pParam_B4SOIesatii*Leff);
d_T1_dX = (((1.0+T0)*(instanceStruct.instanceVar_pParam_B4SOIsii0*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIsii0_dX*T0)-(instanceStruct.instanceVar_pParam_B4SOIsii0*T0)*d_T0_dX)/(1.0+T0)/(1.0+T0));
T1 = ((instanceStruct.instanceVar_pParam_B4SOIsii0*T0)/(1.0+T0));
d_T0_dX = (-1*(instanceStruct.instanceVar_pParam_B4SOIsii1*d_Vgsteff_dX+instanceStruct.d_instanceVar_pParam_B4SOIsii1_dX*Vgsteff)/(1+(instanceStruct.instanceVar_pParam_B4SOIsii1*Vgsteff))/(1+(instanceStruct.instanceVar_pParam_B4SOIsii1*Vgsteff)));
T0 = (1/(1+(instanceStruct.instanceVar_pParam_B4SOIsii1*Vgsteff)));
d_T3_dX = (d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIsii2_dX);
T3 = (T0+instanceStruct.instanceVar_pParam_B4SOIsii2);
d_T2_dX = (Vgst*d_T3_dX+d_Vgst_dX*T3);
T2 = (Vgst*T3);
d_T3_dX = (-1*instanceStruct.d_instanceVar_pParam_B4SOIsiid_dX*Vds/(1+(instanceStruct.instanceVar_pParam_B4SOIsiid*Vds))/(1+(instanceStruct.instanceVar_pParam_B4SOIsiid*Vds)));
T3 = (1/(1+(instanceStruct.instanceVar_pParam_B4SOIsiid*Vds)));
d_VgsStep_dX = ((T1*T2)*d_T3_dX+(T1*d_T2_dX+d_T1_dX*T2)*T3);
VgsStep = ((T1*T2)*T3);
d_Vdsatii_dX = (d_Vdsatii0_dX+d_VgsStep_dX);
Vdsatii = (Vdsatii0+VgsStep);
d_Vdiff_dX = (-d_Vdsatii_dX);
Vdiff = (Vds-Vdsatii);
d_T0_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIbeta2_dX+(instanceStruct.instanceVar_pParam_B4SOIbeta1*d_Vdiff_dX+instanceStruct.d_instanceVar_pParam_B4SOIbeta1_dX*Vdiff))+((instanceStruct.instanceVar_pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dX+(instanceStruct.instanceVar_pParam_B4SOIbeta0*d_Vdiff_dX+instanceStruct.d_instanceVar_pParam_B4SOIbeta0_dX*Vdiff)*Vdiff));
T0 = ((instanceStruct.instanceVar_pParam_B4SOIbeta2+(instanceStruct.instanceVar_pParam_B4SOIbeta1*Vdiff))+((instanceStruct.instanceVar_pParam_B4SOIbeta0*Vdiff)*Vdiff));
if ((T0<1e-5))
{
d_T0_dX = 0.0;
T0 = 1e-5;
}
if (((T0<(Vdiff/100.0))&&(Vdiff>0.0)))
{
d_Ratio_dX = instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX*2.688117142e+43;
Ratio = (instanceStruct.instanceVar_pParam_B4SOIalpha0*2.688117142e+43);
}
else
{
if (((T0<((-Vdiff)/100.0))&&(Vdiff<0.0)))
{
d_Ratio_dX = instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX*3.720075976e-44;
Ratio = (instanceStruct.instanceVar_pParam_B4SOIalpha0*3.720075976e-44);
}
else
{
{
double value_exp_0 = exp((Vdiff/T0));
double  deriv_exp_0_d0 = value_exp_0;
d_Ratio_dX = (instanceStruct.instanceVar_pParam_B4SOIalpha0*deriv_exp_0_d0*(((T0*d_Vdiff_dX-Vdiff*d_T0_dX)/T0/T0))+instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX*value_exp_0);
Ratio = (instanceStruct.instanceVar_pParam_B4SOIalpha0*value_exp_0);
}
}
}
if ((Ratio>10.0))
{
d_Ratio_dX = 0.0;
Ratio = 10.0;
}
d_T0_dX = (d_Ids_dX+((instanceStruct.instanceVar_pParam_B4SOIfbjtii*instanceStruct.instanceVar_B4SOImode)*d_Ic_dX+instanceStruct.d_instanceVar_pParam_B4SOIfbjtii_dX*instanceStruct.instanceVar_B4SOImode*Ic));
T0 = (Ids+((instanceStruct.instanceVar_pParam_B4SOIfbjtii*instanceStruct.instanceVar_B4SOImode)*Ic));
d_Iii_dX = (Ratio*d_T0_dX+d_Ratio_dX*T0);
Iii = (Ratio*T0);
}
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOIalpha0<=0.0))
{
d_Idsmosfet_dX = 0.0;
Idsmosfet = 0.0;
}
else
{
d_Vdsatii0_dX = ((instanceStruct.instanceVar_pParam_B4SOIvdsatii0*(instanceStruct.instanceVar_B4SOItii*d_TempRatio_dX+instanceStruct.d_instanceVar_B4SOItii_dX*(TempRatio-1.0))+instanceStruct.d_instanceVar_pParam_B4SOIvdsatii0_dX*(1+(instanceStruct.instanceVar_B4SOItii*(TempRatio-1.0))))-((Leff*instanceStruct.d_instanceVar_pParam_B4SOIlii_dX-instanceStruct.instanceVar_pParam_B4SOIlii*d_Leff_dX)/Leff/Leff));
Vdsatii0 = ((instanceStruct.instanceVar_pParam_B4SOIvdsatii0*(1+(instanceStruct.instanceVar_B4SOItii*(TempRatio-1.0))))-(instanceStruct.instanceVar_pParam_B4SOIlii/Leff));
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIesatii*d_Leff_dX+instanceStruct.d_instanceVar_pParam_B4SOIesatii_dX*Leff);
T0 = (instanceStruct.instanceVar_pParam_B4SOIesatii*Leff);
d_T1_dX = (((1.0+T0)*(instanceStruct.instanceVar_pParam_B4SOIsii0*d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIsii0_dX*T0)-(instanceStruct.instanceVar_pParam_B4SOIsii0*T0)*d_T0_dX)/(1.0+T0)/(1.0+T0));
T1 = ((instanceStruct.instanceVar_pParam_B4SOIsii0*T0)/(1.0+T0));
d_T0_dX = (-1*(instanceStruct.instanceVar_pParam_B4SOIsii1*d_Vgsteff_dX+instanceStruct.d_instanceVar_pParam_B4SOIsii1_dX*Vgsteff)/(1+(instanceStruct.instanceVar_pParam_B4SOIsii1*Vgsteff))/(1+(instanceStruct.instanceVar_pParam_B4SOIsii1*Vgsteff)));
T0 = (1/(1+(instanceStruct.instanceVar_pParam_B4SOIsii1*Vgsteff)));
d_T3_dX = (d_T0_dX+instanceStruct.d_instanceVar_pParam_B4SOIsii2_dX);
T3 = (T0+instanceStruct.instanceVar_pParam_B4SOIsii2);
d_T2_dX = (Vgst*d_T3_dX+d_Vgst_dX*T3);
T2 = (Vgst*T3);
d_T3_dX = (-1*instanceStruct.d_instanceVar_pParam_B4SOIsiid_dX*Vds/(1+(instanceStruct.instanceVar_pParam_B4SOIsiid*Vds))/(1+(instanceStruct.instanceVar_pParam_B4SOIsiid*Vds)));
T3 = (1/(1+(instanceStruct.instanceVar_pParam_B4SOIsiid*Vds)));
d_VgsStep_dX = ((T1*T2)*d_T3_dX+(T1*d_T2_dX+d_T1_dX*T2)*T3);
VgsStep = ((T1*T2)*T3);
d_Vdsatii_dX = (d_Vdsatii0_dX+d_VgsStep_dX);
Vdsatii = (Vdsatii0+VgsStep);
d_Vdiff_dX = (-d_Vdsatii_dX);
Vdiff = (Vds-Vdsatii);
d_T0_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIbeta2_dX+(instanceStruct.instanceVar_pParam_B4SOIbeta1*d_Vdiff_dX+instanceStruct.d_instanceVar_pParam_B4SOIbeta1_dX*Vdiff))+((instanceStruct.instanceVar_pParam_B4SOIbeta0*Vdiff)*d_Vdiff_dX+(instanceStruct.instanceVar_pParam_B4SOIbeta0*d_Vdiff_dX+instanceStruct.d_instanceVar_pParam_B4SOIbeta0_dX*Vdiff)*Vdiff));
T0 = ((instanceStruct.instanceVar_pParam_B4SOIbeta2+(instanceStruct.instanceVar_pParam_B4SOIbeta1*Vdiff))+((instanceStruct.instanceVar_pParam_B4SOIbeta0*Vdiff)*Vdiff));
if ((T0<1e-5))
{
d_T0_dX = 0.0;
T0 = 1e-5;
}
if (((T0<(Vdiff/100.0))&&(Vdiff>0.0)))
{
d_Ratio_dX = instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX*2.688117142e+43;
Ratio = (instanceStruct.instanceVar_pParam_B4SOIalpha0*2.688117142e+43);
}
else
{
if (((T0<((-Vdiff)/100.0))&&(Vdiff<0.0)))
{
d_Ratio_dX = instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX*3.720075976e-44;
Ratio = (instanceStruct.instanceVar_pParam_B4SOIalpha0*3.720075976e-44);
}
else
{
{
double value_exp_0 = exp((Vdiff/T0));
double  deriv_exp_0_d0 = value_exp_0;
d_Ratio_dX = (instanceStruct.instanceVar_pParam_B4SOIalpha0*deriv_exp_0_d0*(((T0*d_Vdiff_dX-Vdiff*d_T0_dX)/T0/T0))+instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX*value_exp_0);
Ratio = (instanceStruct.instanceVar_pParam_B4SOIalpha0*value_exp_0);
}
}
}
if ((Ratio>10.0))
{
d_Ratio_dX = 0.0;
Ratio = 10.0;
}
d_T0_dX = d_Ids_dX;
T0 = Ids;
d_Idsmosfet_dX = (Ratio*d_T0_dX+d_Ratio_dX*T0);
Idsmosfet = (Ratio*T0);
}
d_T0_dX = ((Leff*(instanceStruct.d_instanceVar_pParam_B4SOIcbjtii_dX+(instanceStruct.instanceVar_pParam_B4SOIebjtii*d_Leff_dX+instanceStruct.d_instanceVar_pParam_B4SOIebjtii_dX*Leff))-(instanceStruct.instanceVar_pParam_B4SOIcbjtii+(instanceStruct.instanceVar_pParam_B4SOIebjtii*Leff))*d_Leff_dX)/Leff/Leff);
T0 = ((instanceStruct.instanceVar_pParam_B4SOIcbjtii+(instanceStruct.instanceVar_pParam_B4SOIebjtii*Leff))/Leff);
d_Vbci_dX = (instanceStruct.instanceVar_pParam_B4SOIvbci*(instanceStruct.instanceVar_B4SOItvbci*d_TempRatio_dX+instanceStruct.d_instanceVar_B4SOItvbci_dX*(TempRatio-1.0))+instanceStruct.d_instanceVar_pParam_B4SOIvbci_dX*(1.0+(instanceStruct.instanceVar_B4SOItvbci*(TempRatio-1.0))));
Vbci = (instanceStruct.instanceVar_pParam_B4SOIvbci*(1.0+(instanceStruct.instanceVar_B4SOItvbci*(TempRatio-1.0))));
if ((instanceStruct.instanceVar_B4SOImode>0))
{
d_T1_dX = d_Vbci_dX;
T1 = (Vbci-vdbd);
}
else
{
d_T1_dX = d_Vbci_dX;
T1 = (Vbci-vsbs);
}
d_T2_dX = instanceStruct.d_instanceVar_pParam_B4SOImbjtii_dX;
T2 = (instanceStruct.instanceVar_pParam_B4SOImbjtii-1.0);
if ((T1<=0.0))
{
d_T3_dX = 0.0;
T3 = 0.0;
}
else
{
{
double value_pow_0 = pow(T1,T2);
double  deriv_pow_0_d0 = ((T1 == 0.0)?0.0:(value_pow_0*T2/T1));
double  deriv_pow_0_d1 = (T1 == 0.0)?0.0:(log(T1)*value_pow_0);
d_T3_dX = ((-instanceStruct.instanceVar_pParam_B4SOIabjtii)*((deriv_pow_0_d0*(d_T1_dX))+(deriv_pow_0_d1*(d_T2_dX)))+(-instanceStruct.d_instanceVar_pParam_B4SOIabjtii_dX)*value_pow_0);
T3 = ((-instanceStruct.instanceVar_pParam_B4SOIabjtii)*value_pow_0);
}
}
if ((T3>100.0))
{
d_T4_dX = 0.0;
T4 = 2.688117142e+43;
}
else
{
if ((T3<(-100.0)))
{
d_T4_dX = 0.0;
T4 = 3.720075976e-44;
}
else
{
{
double value_exp_0 = exp(T3);
double  deriv_exp_0_d0 = value_exp_0;
d_T4_dX = deriv_exp_0_d0*(d_T3_dX);
T4 = value_exp_0;
}
}
}
d_Iiibjt_dX = ((((T0*instanceStruct.instanceVar_B4SOImode)*Ic)*T1)*d_T4_dX+(((T0*instanceStruct.instanceVar_B4SOImode)*Ic)*d_T1_dX+((T0*instanceStruct.instanceVar_B4SOImode)*d_Ic_dX+d_T0_dX*instanceStruct.instanceVar_B4SOImode*Ic)*T1)*T4);
Iiibjt = ((((T0*instanceStruct.instanceVar_B4SOImode)*Ic)*T1)*T4);
d_Iii_dX = (d_Idsmosfet_dX+d_Iiibjt_dX);
Iii = (Idsmosfet+Iiibjt);
}
if (((instanceStruct.instanceVar_B4SOIbodyMod==0)||(instanceStruct.instanceVar_B4SOIbodyMod==2)))
{
d_Ibp_dX = 0.0;
Ibp = 0;
}
else
{
if ((instanceStruct.instanceVar_pParam_B4SOIrbody<1e-3))
{
if ((instanceStruct.instanceVar_B4SOIrbodyext<=1e-3))
{
d_T0_dX = 0.0;
T0 = (1.0/1e-3);
}
else
{
d_T0_dX = (-instanceStruct.d_instanceVar_B4SOIrbodyext_dX/instanceStruct.instanceVar_B4SOIrbodyext/instanceStruct.instanceVar_B4SOIrbodyext);
T0 = (1.0/instanceStruct.instanceVar_B4SOIrbodyext);
}
d_Ibp_dX = vbp*d_T0_dX;
Ibp = (vbp*T0);
}
else
{
d_Ibp_dX = (-vbp*(instanceStruct.d_instanceVar_pParam_B4SOIrbody_dX+instanceStruct.d_instanceVar_B4SOIrbodyext_dX)/(instanceStruct.instanceVar_pParam_B4SOIrbody+instanceStruct.instanceVar_B4SOIrbodyext)/(instanceStruct.instanceVar_pParam_B4SOIrbody+instanceStruct.instanceVar_B4SOIrbodyext));
Ibp = (vbp/(instanceStruct.instanceVar_pParam_B4SOIrbody+instanceStruct.instanceVar_B4SOIrbodyext));
}
}
}
else
{
d_Iii_dX = 0.0;
Iii = 0.0;
d_Ibp_dX = 0.0;
Ibp = 0.0;
}
if ((instanceStruct.instanceVar_B4SOIrgateMod>1))
{
d_T9_dX = (instanceStruct.instanceVar_pParam_B4SOIxrcrg2*instanceStruct.d_instanceVar_B4SOIvtm_dX+instanceStruct.d_instanceVar_pParam_B4SOIxrcrg2_dX*instanceStruct.instanceVar_B4SOIvtm);
T9 = (instanceStruct.instanceVar_pParam_B4SOIxrcrg2*instanceStruct.instanceVar_B4SOIvtm);
d_T0_dX = (T9*d_beta_dX+d_T9_dX*beta);
T0 = (T9*beta);
d_B4SOIgcrg_dX = (instanceStruct.instanceVar_pParam_B4SOIxrcrg1*(d_T0_dX+d_IdovVds_dX)+instanceStruct.d_instanceVar_pParam_B4SOIxrcrg1_dX*(T0+IdovVds));
B4SOIgcrg = (instanceStruct.instanceVar_pParam_B4SOIxrcrg1*(T0+IdovVds));
if ((instanceStruct.instanceVar_B4SOInf!=1.0))
{
d_B4SOIgcrg_dX = (B4SOIgcrg*instanceStruct.d_instanceVar_B4SOInf_dX+d_B4SOIgcrg_dX*instanceStruct.instanceVar_B4SOInf);
B4SOIgcrg = (B4SOIgcrg*instanceStruct.instanceVar_B4SOInf);
}
if ((instanceStruct.instanceVar_B4SOIrgateMod==2))
{
d_T11_dX = (instanceStruct.d_instanceVar_B4SOIgrgeltd_dX+d_B4SOIgcrg_dX);
T11 = (instanceStruct.instanceVar_B4SOIgrgeltd+B4SOIgcrg);
d_B4SOIgcrg_dX = ((T11*(instanceStruct.instanceVar_B4SOIgrgeltd*d_B4SOIgcrg_dX+instanceStruct.d_instanceVar_B4SOIgrgeltd_dX*B4SOIgcrg)-(instanceStruct.instanceVar_B4SOIgrgeltd*B4SOIgcrg)*d_T11_dX)/T11/T11);
B4SOIgcrg = ((instanceStruct.instanceVar_B4SOIgrgeltd*B4SOIgcrg)/T11);
}
}
else
{
d_B4SOIgcrg_dX = 0.0;
B4SOIgcrg = 0;
}
if ((instanceStruct.instanceVar_B4SOIrdsMod==1))
{
d_Rds_dX = 0.0;
Rds = 0.0;
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX);
T0 = (vgs-instanceStruct.instanceVar_pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
T1 = value_sqrt_0;
}
d_vgs_eff_dX = 0.5*(d_T0_dX+d_T1_dX);
vgs_eff = (0.5*(T0+T1));
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIprwg*d_vgs_eff_dX+instanceStruct.d_instanceVar_pParam_B4SOIprwg_dX*vgs_eff);
T0 = (1.0+(instanceStruct.instanceVar_pParam_B4SOIprwg*vgs_eff));
d_T1_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIprwb_dX)*vbs;
T1 = ((-instanceStruct.instanceVar_pParam_B4SOIprwb)*vbs);
d_T2_dX = ((-d_T0_dX/T0/T0)+d_T1_dX);
T2 = ((1.0/T0)+T1);
{
double value_sqrt_0 = sqrt(((T2*T2)+0.01));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = (d_T2_dX+deriv_sqrt_0_d0*((T2*d_T2_dX+d_T2_dX*T2)));
T3 = (T2+value_sqrt_0);
}
d_T4_dX = d_rs0_dX*0.5;
T4 = (rs0*0.5);
d_Rs_dX = ((d_rswmin_dX+(T3*d_T4_dX+d_T3_dX*T4))+instanceStruct.d_instanceVar_B4SOIsourceResistance_dX);
Rs = ((rswmin+(T3*T4))+instanceStruct.instanceVar_B4SOIsourceResistance);
d_T0_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX);
T0 = (vgd-instanceStruct.instanceVar_pParam_B4SOIvfbsd);
{
double value_sqrt_0 = sqrt(((T0*T0)+1.0e-4));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
T1 = value_sqrt_0;
}
d_vgd_eff_dX = 0.5*(d_T0_dX+d_T1_dX);
vgd_eff = (0.5*(T0+T1));
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIprwg*d_vgd_eff_dX+instanceStruct.d_instanceVar_pParam_B4SOIprwg_dX*vgd_eff);
T0 = (1.0+(instanceStruct.instanceVar_pParam_B4SOIprwg*vgd_eff));
d_T1_dX = (-instanceStruct.d_instanceVar_pParam_B4SOIprwb_dX)*vbd;
T1 = ((-instanceStruct.instanceVar_pParam_B4SOIprwb)*vbd);
d_T2_dX = ((-d_T0_dX/T0/T0)+d_T1_dX);
T2 = ((1.0/T0)+T1);
{
double value_sqrt_0 = sqrt(((T2*T2)+0.01));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T3_dX = (d_T2_dX+deriv_sqrt_0_d0*((T2*d_T2_dX+d_T2_dX*T2)));
T3 = (T2+value_sqrt_0);
}
d_T4_dX = d_rd0_dX*0.5;
T4 = (rd0*0.5);
d_Rd_dX = ((d_rdwmin_dX+(T3*d_T4_dX+d_T3_dX*T4))+instanceStruct.d_instanceVar_B4SOIdrainResistance_dX);
Rd = ((rdwmin+(T3*T4))+instanceStruct.instanceVar_B4SOIdrainResistance);
}
else
{
d_Rs_dX = instanceStruct.d_instanceVar_B4SOIsourceResistance_dX;
Rs = instanceStruct.instanceVar_B4SOIsourceResistance;
d_Rd_dX = instanceStruct.d_instanceVar_B4SOIdrainResistance_dX;
Rd = instanceStruct.instanceVar_B4SOIdrainResistance;
}
if ((instanceStruct.instanceVar_B4SOIrdsMod==2))
{
d_Rs_dX = 0.0;
Rs = 0;
d_Rd_dX = 0.0;
Rd = 0;
}
d_T1_dX = (Vgsteff*(-((Vgst2Vtm*((0.5*Abulk)*d_Vdseff_dX+0.5*d_Abulk_dX*Vdseff)-((0.5*Abulk)*Vdseff)*d_Vgst2Vtm_dX)/Vgst2Vtm/Vgst2Vtm))+d_Vgsteff_dX*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
T1 = (Vgsteff*(1.0-(((0.5*Abulk)*Vdseff)/Vgst2Vtm)));
instanceStruct.d_instanceVar_B4SOIqinv_dX = (((((-instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.instanceVar_B4SOInf)*Leff)*d_T1_dX+((((-instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.instanceVar_B4SOInf)*d_Leff_dX+(((-instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.d_instanceVar_B4SOInf_dX+((-instanceStruct.instanceVar_B4SOIcox)*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+(-instanceStruct.d_instanceVar_B4SOIcox_dX)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.instanceVar_B4SOInf)*Leff)*T1);
instanceStruct.instanceVar_B4SOIqinv = (((((-instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_pParam_B4SOIweff)*instanceStruct.instanceVar_B4SOInf)*Leff)*T1);
if ((instanceStruct.instanceVar_B4SOInf!=1))
{
d_Ids_dX = (Ids*instanceStruct.d_instanceVar_B4SOInf_dX+d_Ids_dX*instanceStruct.instanceVar_B4SOInf);
Ids = (Ids*instanceStruct.instanceVar_B4SOInf);
d_Ic_dX = (Ic*instanceStruct.d_instanceVar_B4SOInf_dX+d_Ic_dX*instanceStruct.instanceVar_B4SOInf);
Ic = (Ic*instanceStruct.instanceVar_B4SOInf);
instanceStruct.d_instanceVar_B4SOIidovVds_dX = (instanceStruct.instanceVar_B4SOIidovVds*instanceStruct.d_instanceVar_B4SOInf_dX+instanceStruct.d_instanceVar_B4SOIidovVds_dX*instanceStruct.instanceVar_B4SOInf);
instanceStruct.instanceVar_B4SOIidovVds = (instanceStruct.instanceVar_B4SOIidovVds*instanceStruct.instanceVar_B4SOInf);
d_Ibs_dX = (Ibs*instanceStruct.d_instanceVar_B4SOInf_dX+d_Ibs_dX*instanceStruct.instanceVar_B4SOInf);
Ibs = (Ibs*instanceStruct.instanceVar_B4SOInf);
d_Ibd_dX = (Ibd*instanceStruct.d_instanceVar_B4SOInf_dX+d_Ibd_dX*instanceStruct.instanceVar_B4SOInf);
Ibd = (Ibd*instanceStruct.instanceVar_B4SOInf);
d_Igcs_dX = (Igcs*instanceStruct.d_instanceVar_B4SOInf_dX+d_Igcs_dX*instanceStruct.instanceVar_B4SOInf);
Igcs = (Igcs*instanceStruct.instanceVar_B4SOInf);
d_Igcd_dX = (Igcd*instanceStruct.d_instanceVar_B4SOInf_dX+d_Igcd_dX*instanceStruct.instanceVar_B4SOInf);
Igcd = (Igcd*instanceStruct.instanceVar_B4SOInf);
d_Igs_dX = (Igs*instanceStruct.d_instanceVar_B4SOInf_dX+d_Igs_dX*instanceStruct.instanceVar_B4SOInf);
Igs = (Igs*instanceStruct.instanceVar_B4SOInf);
d_Igd_dX = (Igd*instanceStruct.d_instanceVar_B4SOInf_dX+d_Igd_dX*instanceStruct.instanceVar_B4SOInf);
Igd = (Igd*instanceStruct.instanceVar_B4SOInf);
d_Iii_dX = (Iii*instanceStruct.d_instanceVar_B4SOInf_dX+d_Iii_dX*instanceStruct.instanceVar_B4SOInf);
Iii = (Iii*instanceStruct.instanceVar_B4SOInf);
d_B4SOIig_dX = (B4SOIig*instanceStruct.d_instanceVar_B4SOInf_dX+d_B4SOIig_dX*instanceStruct.instanceVar_B4SOInf);
B4SOIig = (B4SOIig*instanceStruct.instanceVar_B4SOInf);
d_Igidl_dX = (Igidl*instanceStruct.d_instanceVar_B4SOInf_dX+d_Igidl_dX*instanceStruct.instanceVar_B4SOInf);
Igidl = (Igidl*instanceStruct.instanceVar_B4SOInf);
d_Igisl_dX = (Igisl*instanceStruct.d_instanceVar_B4SOInf_dX+d_Igisl_dX*instanceStruct.instanceVar_B4SOInf);
Igisl = (Igisl*instanceStruct.instanceVar_B4SOInf);
}
instanceStruct.d_instanceVar_B4SOIgm_dX = 0.0;
instanceStruct.instanceVar_B4SOIgm = (instanceStruct.instanceVar_B4SOItype*0.0 /*FIXME: ddx(Ids,V(gi,GND)) not implemented in sensitivity context */);
if ((instanceStruct.instanceVar_B4SOImode>0))
{
instanceStruct.d_instanceVar_B4SOIgds_dX = 0.0;
instanceStruct.instanceVar_B4SOIgds = (instanceStruct.instanceVar_B4SOItype*0.0 /*FIXME: ddx(Ids,V(di,GND)) not implemented in sensitivity context */);
}
else
{
instanceStruct.d_instanceVar_B4SOIgds_dX = 0.0;
instanceStruct.instanceVar_B4SOIgds = (instanceStruct.instanceVar_B4SOItype*0.0 /*FIXME: ddx(Ids,V(si,GND)) not implemented in sensitivity context */);
}
instanceStruct.d_instanceVar_B4SOIgmbs_dX = 0.0;
instanceStruct.instanceVar_B4SOIgmbs = (instanceStruct.instanceVar_B4SOItype*0.0 /*FIXME: ddx(Ids,V(b,GND)) not implemented in sensitivity context */);
d_CoxWL_dX = (instanceStruct.instanceVar_B4SOIcox*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.d_instanceVar_pParam_B4SOIleffCV_dX+((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.d_instanceVar_B4SOInf_dX+((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX-instanceStruct.instanceVar_pParam_B4SOIweffCV*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCV)+instanceStruct.d_instanceVar_B4SOIagbcp_dX)+instanceStruct.d_instanceVar_B4SOIcox_dX*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCV)+instanceStruct.instanceVar_B4SOIagbcp));
CoxWL = (instanceStruct.instanceVar_B4SOIcox*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCV)+instanceStruct.instanceVar_B4SOIagbcp));
d_CoxWLb_dX = ((instanceStruct.instanceVar_B4SOIfbody*instanceStruct.instanceVar_B4SOIcox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.d_instanceVar_pParam_B4SOIleffCVb_dX+((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.d_instanceVar_B4SOInf_dX+((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX-instanceStruct.instanceVar_pParam_B4SOIweffCV*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVb)+instanceStruct.d_instanceVar_B4SOIagbcp_dX)+(instanceStruct.instanceVar_B4SOIfbody*instanceStruct.d_instanceVar_B4SOIcox_dX+instanceStruct.d_instanceVar_B4SOIfbody_dX*instanceStruct.instanceVar_B4SOIcox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVb)+instanceStruct.instanceVar_B4SOIagbcp));
CoxWLb = ((instanceStruct.instanceVar_B4SOIfbody*instanceStruct.instanceVar_B4SOIcox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVb)+instanceStruct.instanceVar_B4SOIagbcp));
d_CoxWL2_dX = (instanceStruct.instanceVar_B4SOIcox*instanceStruct.d_instanceVar_B4SOIagbcp2_dX+instanceStruct.d_instanceVar_B4SOIcox_dX*instanceStruct.instanceVar_B4SOIagbcp2);
CoxWL2 = (instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIagbcp2);
d_CoxWLb2_dX = ((instanceStruct.instanceVar_B4SOIfbody*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.d_instanceVar_B4SOIagbcp2_dX+(instanceStruct.instanceVar_B4SOIfbody*instanceStruct.d_instanceVar_B4SOIcox_dX+instanceStruct.d_instanceVar_B4SOIfbody_dX*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_B4SOIagbcp2);
CoxWLb2 = ((instanceStruct.instanceVar_B4SOIfbody*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_B4SOIagbcp2);
d_Vgst_dX = (d_Vgs_eff_dX-d_Vth_CV_dX);
Vgst = (Vgs_eff-Vth_CV);
d_T10_dX = (n_CV*d_Vtm_dX+d_n_CV_dX*Vtm);
T10 = (n_CV*Vtm);
d_VgstNVt_dX = ((T10*(instanceStruct.instanceVar_pParam_B4SOImstar*d_Vgst_dX+instanceStruct.d_instanceVar_pParam_B4SOImstar_dX*Vgst)-(instanceStruct.instanceVar_pParam_B4SOImstar*Vgst)*d_T10_dX)/T10/T10);
VgstNVt = ((instanceStruct.instanceVar_pParam_B4SOImstar*Vgst)/T10);
d_noff_dX = ((n_CV*instanceStruct.instanceVar_pParam_B4SOInoff)*d_Vtm_dX+(n_CV*instanceStruct.d_instanceVar_pParam_B4SOInoff_dX+d_n_CV_dX*instanceStruct.instanceVar_pParam_B4SOInoff)*Vtm);
noff = ((n_CV*instanceStruct.instanceVar_pParam_B4SOInoff)*Vtm);
d_noff2_dX = ((n_CV*instanceStruct.instanceVar_pParam_B4SOInoff2)*d_Vtm_dX+(n_CV*instanceStruct.d_instanceVar_pParam_B4SOInoff2_dX+d_n_CV_dX*instanceStruct.instanceVar_pParam_B4SOInoff2)*Vtm);
noff2 = ((n_CV*instanceStruct.instanceVar_pParam_B4SOInoff2)*Vtm);
if ((instanceStruct.instanceVar_B4SOIvgstcvMod==0))
{
if (((VgstNVt>(-100.0))&&(VgstNVt<100.0)))
{
{
double value_exp_0 = exp(VgstNVt);
double value_exp_1 = exp(VgstNVt);
double  deriv_exp_0_d0 = value_exp_0;
double  deriv_exp_1_d0 = value_exp_1;
d_ExpVgst_dX = (value_exp_0*deriv_exp_1_d0*(d_VgstNVt_dX)+deriv_exp_0_d0*(d_VgstNVt_dX)*value_exp_1);
ExpVgst = (value_exp_0*value_exp_1);
}
{
double value_exp_0 = exp((-(instanceStruct.instanceVar_pParam_B4SOIdelvt/noff)));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = (ExpVgst*deriv_exp_0_d0*((-((noff*instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX-instanceStruct.instanceVar_pParam_B4SOIdelvt*d_noff_dX)/noff/noff)))+d_ExpVgst_dX*value_exp_0);
ExpVgst = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));
d_Vgsteff_dX = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dX)+d_noff_dX*evaluator_lln_0.getValues((1.0+ExpVgst)));
Vgsteff = (noff*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
{
double value_exp_0 = exp((((-instanceStruct.instanceVar_eggbcp2)/noff2)/(Vtm*Vtm)));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst2_dX = (ExpVgst*deriv_exp_0_d0*((((Vtm*Vtm)*((noff2*(-instanceStruct.d_instanceVar_eggbcp2_dX)-(-instanceStruct.instanceVar_eggbcp2)*d_noff2_dX)/noff2/noff2)-((-instanceStruct.instanceVar_eggbcp2)/noff2)*(Vtm*d_Vtm_dX+d_Vtm_dX*Vtm))/(Vtm*Vtm)/(Vtm*Vtm)))+d_ExpVgst_dX*value_exp_0);
ExpVgst2 = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst2));
d_Vgsteff2_dX = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dX)+d_noff2_dX*evaluator_lln_0.getValues((1.0+ExpVgst2)));
Vgsteff2 = (noff2*evaluator_lln_0.getValues((1.0+ExpVgst2)));
}
}
}
}
else
{
if ((instanceStruct.instanceVar_B4SOIvgstcvMod==1))
{
if (((VgstNVt>(-100.0))&&(VgstNVt<100.0)))
{
{
double value_exp_0 = exp((VgstNVt/(instanceStruct.instanceVar_pParam_B4SOImstar*instanceStruct.instanceVar_pParam_B4SOInoff)));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = deriv_exp_0_d0*((((instanceStruct.instanceVar_pParam_B4SOImstar*instanceStruct.instanceVar_pParam_B4SOInoff)*d_VgstNVt_dX-VgstNVt*(instanceStruct.instanceVar_pParam_B4SOImstar*instanceStruct.d_instanceVar_pParam_B4SOInoff_dX+instanceStruct.d_instanceVar_pParam_B4SOImstar_dX*instanceStruct.instanceVar_pParam_B4SOInoff))/(instanceStruct.instanceVar_pParam_B4SOImstar*instanceStruct.instanceVar_pParam_B4SOInoff)/(instanceStruct.instanceVar_pParam_B4SOImstar*instanceStruct.instanceVar_pParam_B4SOInoff)));
ExpVgst = value_exp_0;
}
{
double value_exp_0 = exp((-(instanceStruct.instanceVar_pParam_B4SOIdelvt/noff)));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = (ExpVgst*deriv_exp_0_d0*((-((noff*instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX-instanceStruct.instanceVar_pParam_B4SOIdelvt*d_noff_dX)/noff/noff)))+d_ExpVgst_dX*value_exp_0);
ExpVgst = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));
d_Vgsteff_dX = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dX)+d_noff_dX*evaluator_lln_0.getValues((1.0+ExpVgst)));
Vgsteff = (noff*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
{
double value_exp_0 = exp((((-instanceStruct.instanceVar_eggbcp2)/noff2)/(Vtm*Vtm)));
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst2_dX = (ExpVgst*deriv_exp_0_d0*((((Vtm*Vtm)*((noff2*(-instanceStruct.d_instanceVar_eggbcp2_dX)-(-instanceStruct.instanceVar_eggbcp2)*d_noff2_dX)/noff2/noff2)-((-instanceStruct.instanceVar_eggbcp2)/noff2)*(Vtm*d_Vtm_dX+d_Vtm_dX*Vtm))/(Vtm*Vtm)/(Vtm*Vtm)))+d_ExpVgst_dX*value_exp_0);
ExpVgst2 = (ExpVgst*value_exp_0);
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst2));
d_Vgsteff2_dX = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dX)+d_noff2_dX*evaluator_lln_0.getValues((1.0+ExpVgst2)));
Vgsteff2 = (noff2*evaluator_lln_0.getValues((1.0+ExpVgst2)));
}
}
}
}
else
{
d_VgstNVt_dX = ((noff*(instanceStruct.instanceVar_pParam_B4SOImstarcv*(d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX)+instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX*(Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt))-(instanceStruct.instanceVar_pParam_B4SOImstarcv*(Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt))*d_noff_dX)/noff/noff);
VgstNVt = ((instanceStruct.instanceVar_pParam_B4SOImstarcv*(Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt))/noff);
d_ExpArg_dX = ((noff*(instanceStruct.d_instanceVar_pParam_B4SOIvoffcv_dX-((1-instanceStruct.instanceVar_pParam_B4SOImstarcv)*(d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX)+(-instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX)*(Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)))-(instanceStruct.instanceVar_pParam_B4SOIvoffcv-((1-instanceStruct.instanceVar_pParam_B4SOImstarcv)*(Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)))*d_noff_dX)/noff/noff);
ExpArg = ((instanceStruct.instanceVar_pParam_B4SOIvoffcv-((1-instanceStruct.instanceVar_pParam_B4SOImstarcv)*(Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)))/noff);
if ((VgstNVt>100.0))
{
d_Vgsteff_dX = (d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX);
Vgsteff = (Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt);
}
else
{
if ((ExpArg>100.0))
{
d_T0_dX = ((noff*((d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX)-instanceStruct.d_instanceVar_pParam_B4SOIvoffcv_dX)-((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_pParam_B4SOIvoffcv)*d_noff_dX)/noff/noff);
T0 = (((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_pParam_B4SOIvoffcv)/noff);
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = deriv_exp_0_d0*(d_T0_dX);
ExpVgst = value_exp_0;
}
d_Vgsteff_dX = (((Vtm*cdep0)/instanceStruct.instanceVar_B4SOIcox)*d_ExpVgst_dX+((instanceStruct.instanceVar_B4SOIcox*(Vtm*d_cdep0_dX+d_Vtm_dX*cdep0)-(Vtm*cdep0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox)*ExpVgst);
Vgsteff = (((Vtm*cdep0)/instanceStruct.instanceVar_B4SOIcox)*ExpVgst);
}
else
{
{
double value_exp_0 = exp(VgstNVt);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst_dX = deriv_exp_0_d0*(d_VgstNVt_dX);
ExpVgst = value_exp_0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst));
d_T1_dX = (noff*evaluator_lln_0.getDerivs((1.0+ExpVgst),d_ExpVgst_dX)+d_noff_dX*evaluator_lln_0.getValues((1.0+ExpVgst)));
T1 = (noff*evaluator_lln_0.getValues((1.0+ExpVgst)));
}
{
double value_exp_0 = exp(ExpArg);
double  deriv_exp_0_d0 = value_exp_0;
d_dT2_dVg_dX = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(-instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX)+(((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*deriv_exp_0_d0*(d_ExpArg_dX)+(((Vtm*cdep0)*(-instanceStruct.d_instanceVar_B4SOIcox_dX)-(-instanceStruct.instanceVar_B4SOIcox)*(Vtm*d_cdep0_dX+d_Vtm_dX*cdep0))/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstarcv));
dT2_dVg = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstarcv));
}
d_T2_dX = (instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX-(((1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)*(noff*d_dT2_dVg_dX+d_noff_dX*dT2_dVg)-(noff*dT2_dVg)*(-instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX))/(1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)));
T2 = (instanceStruct.instanceVar_pParam_B4SOImstarcv-((noff*dT2_dVg)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)));
d_Vgsteff_dX = ((T2*d_T1_dX-T1*d_T2_dX)/T2/T2);
Vgsteff = (T1/T2);
}
}
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
d_VgstNVt2_dX = ((noff2*(instanceStruct.instanceVar_pParam_B4SOImstarcv*((d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX)-instanceStruct.d_instanceVar_eggbcp2_dX)+instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX*((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_eggbcp2))-(instanceStruct.instanceVar_pParam_B4SOImstarcv*((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_eggbcp2))*d_noff2_dX)/noff2/noff2);
VgstNVt2 = ((instanceStruct.instanceVar_pParam_B4SOImstarcv*((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_eggbcp2))/noff2);
d_ExpArg2_dX = ((noff2*(instanceStruct.d_instanceVar_pParam_B4SOIvoffcv_dX-((1-instanceStruct.instanceVar_pParam_B4SOImstarcv)*((d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX)-instanceStruct.d_instanceVar_eggbcp2_dX)+(-instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX)*((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_eggbcp2)))-(instanceStruct.instanceVar_pParam_B4SOIvoffcv-((1-instanceStruct.instanceVar_pParam_B4SOImstarcv)*((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_eggbcp2)))*d_noff2_dX)/noff2/noff2);
ExpArg2 = ((instanceStruct.instanceVar_pParam_B4SOIvoffcv-((1-instanceStruct.instanceVar_pParam_B4SOImstarcv)*((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_eggbcp2)))/noff2);
if ((VgstNVt2>100.0))
{
d_Vgsteff2_dX = ((d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX)-instanceStruct.d_instanceVar_eggbcp2_dX);
Vgsteff2 = ((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_eggbcp2);
}
else
{
if ((ExpArg2>100.0))
{
d_T0_dX = ((noff2*(((d_Vgst_dX-instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX)-instanceStruct.d_instanceVar_pParam_B4SOIvoffcv_dX)-instanceStruct.d_instanceVar_eggbcp2_dX)-(((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_pParam_B4SOIvoffcv)-instanceStruct.instanceVar_eggbcp2)*d_noff2_dX)/noff2/noff2);
T0 = ((((Vgst-instanceStruct.instanceVar_pParam_B4SOIdelvt)-instanceStruct.instanceVar_pParam_B4SOIvoffcv)-instanceStruct.instanceVar_eggbcp2)/noff2);
{
double value_exp_0 = exp(T0);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst2_dX = deriv_exp_0_d0*(d_T0_dX);
ExpVgst2 = value_exp_0;
}
d_Vgsteff2_dX = (((Vtm*cdep0)/instanceStruct.instanceVar_B4SOIcox)*d_ExpVgst2_dX+((instanceStruct.instanceVar_B4SOIcox*(Vtm*d_cdep0_dX+d_Vtm_dX*cdep0)-(Vtm*cdep0)*instanceStruct.d_instanceVar_B4SOIcox_dX)/instanceStruct.instanceVar_B4SOIcox/instanceStruct.instanceVar_B4SOIcox)*ExpVgst2);
Vgsteff2 = (((Vtm*cdep0)/instanceStruct.instanceVar_B4SOIcox)*ExpVgst2);
}
else
{
{
double value_exp_0 = exp(VgstNVt2);
double  deriv_exp_0_d0 = value_exp_0;
d_ExpVgst2_dX = deriv_exp_0_d0*(d_VgstNVt2_dX);
ExpVgst2 = value_exp_0;
}
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+ExpVgst2));
d_T1_dX = (noff2*evaluator_lln_0.getDerivs((1.0+ExpVgst2),d_ExpVgst2_dX)+d_noff2_dX*evaluator_lln_0.getValues((1.0+ExpVgst2)));
T1 = (noff2*evaluator_lln_0.getValues((1.0+ExpVgst2)));
}
{
double value_exp_0 = exp(ExpArg2);
double  deriv_exp_0_d0 = value_exp_0;
d_dT2_dVg_dX = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(-instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX)+(((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*deriv_exp_0_d0*(d_ExpArg2_dX)+(((Vtm*cdep0)*(-instanceStruct.d_instanceVar_B4SOIcox_dX)-(-instanceStruct.instanceVar_B4SOIcox)*(Vtm*d_cdep0_dX+d_Vtm_dX*cdep0))/(Vtm*cdep0)/(Vtm*cdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstarcv));
dT2_dVg = ((((-instanceStruct.instanceVar_B4SOIcox)/(Vtm*cdep0))*value_exp_0)*(1-instanceStruct.instanceVar_pParam_B4SOImstarcv));
}
d_T2_dX = (instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX-(((1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)*(noff2*d_dT2_dVg_dX+d_noff2_dX*dT2_dVg)-(noff2*dT2_dVg)*(-instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX))/(1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)));
T2 = (instanceStruct.instanceVar_pParam_B4SOImstarcv-((noff2*dT2_dVg)/(1.0-instanceStruct.instanceVar_pParam_B4SOImstarcv)));
d_Vgsteff2_dX = ((T2*d_T1_dX-T1*d_T2_dX)/T2/T2);
Vgsteff2 = (T1/T2);
}
}
}
}
}
d_Vth_dX = d_Vth_CV_dX;
Vth = Vth_CV;
d_sqrtPhis_dX = d_sqrtPhis_CV_dX;
sqrtPhis = sqrtPhis_CV;
d_Vbseff_dX = d_Vbseff_CV_dX;
Vbseff = Vbseff_CV;
if ((instanceStruct.instanceVar_B4SOIcapMod==2))
{
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_Qac0_dX = 0.0;
Qac0 = 0.0;
d_Qsub0_dX = 0.0;
Qsub0 = 0.0;
}
else
{
d_Vfb_dX = (((d_Vth_dX-d_phi_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhis_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhis))+instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX);
Vfb = (((Vth-phi)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhis))+instanceStruct.instanceVar_pParam_B4SOIdelvt);
d_V3_dX = ((d_Vfb_dX-d_Vgs_eff_dX)+d_Vbseff_dX);
V3 = (((Vfb-Vgs_eff)+Vbseff)-0.08);
if ((Vfb<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((4.0*0.08)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)-(4.0*0.08)*d_Vfb_dX));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*0.08)*Vfb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)+(4.0*0.08)*d_Vfb_dX));
T0 = value_sqrt_0;
}
}
d_Vfbeff_dX = (d_Vfb_dX-0.5*(d_V3_dX+d_T0_dX));
Vfbeff = (Vfb-(0.5*(V3+T0)));
d_Qac0_dX = (CoxWLb*(d_Vfbeff_dX-d_Vfb_dX)+d_CoxWLb_dX*(Vfbeff-Vfb));
Qac0 = (CoxWLb*(Vfbeff-Vfb));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_Vfb2_dX = (d_Vfb_dX+instanceStruct.d_instanceVar_eggbcp2_dX);
Vfb2 = (Vfb+instanceStruct.instanceVar_eggbcp2);
DELTA_3_SOI2 = 0.08;
d_V3_dX = (d_Vfb2_dX+d_Vbseff_dX);
V3 = (((Vfb2-Vgs_eff2)+Vbseff)-DELTA_3_SOI2);
if ((Vfb2<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((100.0*DELTA_3_SOI2)*Vfb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)-(100.0*DELTA_3_SOI2)*d_Vfb2_dX));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((100.0*DELTA_3_SOI2)*Vfb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)+(100.0*DELTA_3_SOI2)*d_Vfb2_dX));
T0 = value_sqrt_0;
}
}
d_Vfbeff2_dX = (d_Vfb2_dX-0.5*(d_V3_dX+d_T0_dX));
Vfbeff2 = (Vfb2-(0.5*(V3+T0)));
d_Qac0_dX = (d_Qac0_dX+(CoxWLb2*(d_Vfbeff2_dX-d_Vfb2_dX)+d_CoxWLb2_dX*(Vfbeff2-Vfb2)));
Qac0 = (Qac0+(CoxWLb2*(Vfbeff2-Vfb2)));
}
d_T0_dX = 0.5*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX;
T0 = (0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox);
d_T3_dX = (((d_Vgs_eff_dX-d_Vfbeff_dX)-d_Vbseff_dX)-d_Vgsteff_dX);
T3 = (((Vgs_eff-Vfbeff)-Vbseff)-Vgsteff);
if ((instanceStruct.instanceVar_pParam_B4SOIk1ox==0.0))
{
d_T1_dX = 0.0;
T1 = 0.0;
}
else
{
if ((T3<0.0))
{
d_T1_dX = (d_T0_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T3_dX-T3*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX)/instanceStruct.instanceVar_pParam_B4SOIk1ox/instanceStruct.instanceVar_pParam_B4SOIk1ox));
T1 = (T0+(T3/instanceStruct.instanceVar_pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*(((T0*d_T0_dX+d_T0_dX*T0)+d_T3_dX));
T1 = value_sqrt_0;
}
}
}
d_Qsub0_dX = ((CoxWLb*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(d_T1_dX-d_T0_dX)+(CoxWLb*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+d_CoxWLb_dX*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0));
Qsub0 = ((CoxWLb*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T3_dX = (((-d_Vfbeff2_dX)-d_Vbseff_dX)-d_Vgsteff2_dX);
T3 = (((Vgs_eff2-Vfbeff2)-Vbseff)-Vgsteff2);
if ((T3<0.0))
{
d_T1_dX = (d_T0_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T3_dX-T3*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX)/instanceStruct.instanceVar_pParam_B4SOIk1ox/instanceStruct.instanceVar_pParam_B4SOIk1ox));
T1 = (T0+(T3/instanceStruct.instanceVar_pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*(((T0*d_T0_dX+d_T0_dX*T0)+d_T3_dX));
T1 = value_sqrt_0;
}
}
d_Qsub0_dX = (d_Qsub0_dX+((CoxWLb2*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(d_T1_dX-d_T0_dX)+(CoxWLb2*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+d_CoxWLb2_dX*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0)));
Qsub0 = (Qsub0+((CoxWLb2*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0)));
}
}
d_AbulkCV_dX = (Abulk0*instanceStruct.d_instanceVar_pParam_B4SOIabulkCVfactor_dX+d_Abulk0_dX*instanceStruct.instanceVar_pParam_B4SOIabulkCVfactor);
AbulkCV = (Abulk0*instanceStruct.instanceVar_pParam_B4SOIabulkCVfactor);
d_VdsatCV_dX = ((AbulkCV*d_Vgsteff_dX-Vgsteff*d_AbulkCV_dX)/AbulkCV/AbulkCV);
VdsatCV = (Vgsteff/AbulkCV);
d_V4_dX = d_VdsatCV_dX;
V4 = ((VdsatCV-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V4*d_V4_dX+d_V4_dX*V4)+(4.0*0.02)*d_VdsatCV_dX));
T0 = value_sqrt_0;
}
d_VdseffCV_dX = (d_VdsatCV_dX-0.5*(d_V4_dX+d_T0_dX));
VdseffCV = (VdsatCV-(0.5*(V4+T0)));
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
d_VdsatCV2_dX = ((AbulkCV*d_Vgsteff2_dX-Vgsteff2*d_AbulkCV_dX)/AbulkCV/AbulkCV);
VdsatCV2 = (Vgsteff2/AbulkCV);
d_V4_dX = d_VdsatCV2_dX;
V4 = ((VdsatCV2-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V4*d_V4_dX+d_V4_dX*V4)+(4.0*0.02)*d_VdsatCV2_dX));
T0 = value_sqrt_0;
}
d_VdseffCV2_dX = (d_VdsatCV2_dX-0.5*(d_V4_dX+d_T0_dX));
VdseffCV2 = (VdsatCV2-(0.5*(V4+T0)));
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_qbulk_dX = 0.0;
qbulk = 0;
}
else
{
d_T0_dX = (AbulkCV*d_VdseffCV_dX+d_AbulkCV_dX*VdseffCV);
T0 = (AbulkCV*VdseffCV);
d_T1_dX = 12.0*(d_Vgsteff_dX-0.5*d_T0_dX);
T1 = (12.0*((Vgsteff-(0.5*T0))+1e-20));
d_T2_dX = ((T1*d_VdseffCV_dX-VdseffCV*d_T1_dX)/T1/T1);
T2 = (VdseffCV/T1);
d_T3_dX = (T0*d_T2_dX+d_T0_dX*T2);
T3 = (T0*T2);
d_T7_dX = (-d_AbulkCV_dX);
T7 = (1.0-AbulkCV);
d_qbulk_dX = ((CoxWLb*T7)*(0.5*d_VdseffCV_dX-d_T3_dX)+(CoxWLb*d_T7_dX+d_CoxWLb_dX*T7)*((0.5*VdseffCV)-T3));
qbulk = ((CoxWLb*T7)*((0.5*VdseffCV)-T3));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T0_dX = (AbulkCV*d_VdseffCV2_dX+d_AbulkCV_dX*VdseffCV2);
T0 = (AbulkCV*VdseffCV2);
d_T1_dX = 12.0*(d_Vgsteff2_dX-0.5*d_T0_dX);
T1 = (12.0*((Vgsteff2-(0.5*T0))+1e-20));
d_T2_dX = ((T1*d_VdseffCV2_dX-VdseffCV2*d_T1_dX)/T1/T1);
T2 = (VdseffCV2/T1);
d_T3_dX = (T0*d_T2_dX+d_T0_dX*T2);
T3 = (T0*T2);
d_T7_dX = (-d_AbulkCV_dX);
T7 = (1.0-AbulkCV);
d_qbulk_dX = (d_qbulk_dX+((CoxWLb2*T7)*(0.5*d_VdseffCV2_dX-d_T3_dX)+(CoxWLb2*d_T7_dX+d_CoxWLb2_dX*T7)*((0.5*VdseffCV2)-T3)));
qbulk = (qbulk+((CoxWLb2*T7)*((0.5*VdseffCV2)-T3)));
}
}
d_T0_dX = (AbulkCV*d_VdseffCV_dX+d_AbulkCV_dX*VdseffCV);
T0 = (AbulkCV*VdseffCV);
d_T1_dX = 12.0*(d_Vgsteff_dX-0.5*d_T0_dX);
T1 = (12.0*((Vgsteff-(0.5*T0))+1e-20));
d_T2_dX = ((T1*d_T0_dX-T0*d_T1_dX)/T1/T1);
T2 = (T0/T1);
d_T3_dX = (T0*d_T2_dX+d_T0_dX*T2);
T3 = (T0*T2);
d_qinv_dX = (CoxWL*((d_Vgsteff_dX-0.5*d_T0_dX)+d_T3_dX)+d_CoxWL_dX*((Vgsteff-(0.5*T0))+T3));
qinv = (CoxWL*((Vgsteff-(0.5*T0))+T3));
instanceStruct.d_instanceVar_B4SOIqinv_dX = (-d_qinv_dX);
instanceStruct.instanceVar_B4SOIqinv = (-qinv);
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T02_dX = (AbulkCV*d_VdseffCV2_dX+d_AbulkCV_dX*VdseffCV2);
T02 = (AbulkCV*VdseffCV2);
d_T12_dX = 12.0*(d_Vgsteff2_dX-0.5*d_T02_dX);
T12 = (12.0*((Vgsteff2-(0.5*T02))+1e-20));
d_T2_dX = ((T12*d_T02_dX-T02*d_T12_dX)/T12/T12);
T2 = (T02/T12);
d_T3_dX = (T02*d_T2_dX+d_T02_dX*T2);
T3 = (T02*T2);
d_qinv_dX = (d_qinv_dX+(CoxWL2*((d_Vgsteff2_dX-0.5*d_T02_dX)+d_T3_dX)+d_CoxWL2_dX*((Vgsteff2-(0.5*T02))+T3)));
qinv = (qinv+(CoxWL2*((Vgsteff2-(0.5*T02))+T3)));
instanceStruct.d_instanceVar_B4SOIqinv_dX = (-d_qinv_dX);
instanceStruct.instanceVar_B4SOIqinv = (-qinv);
}
if ((instanceStruct.instanceVar_B4SOIxpart>0.5))
{
d_T1_dX = (d_T1_dX+d_T1_dX);
T1 = (T1+T1);
d_qsrc_dX = ((-CoxWL)*((0.5*d_Vgsteff_dX+0.25*d_T0_dX)-((T1*(T0*d_T0_dX+d_T0_dX*T0)-(T0*T0)*d_T1_dX)/T1/T1))+(-d_CoxWL_dX)*(((0.5*Vgsteff)+(0.25*T0))-((T0*T0)/T1)));
qsrc = ((-CoxWL)*(((0.5*Vgsteff)+(0.25*T0))-((T0*T0)/T1)));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T12_dX = (d_T12_dX+d_T12_dX);
T12 = (T12+T12);
d_qsrc_dX = (d_qsrc_dX-(CoxWL2*((0.5*d_Vgsteff2_dX+0.25*d_T02_dX)-((T12*(T02*d_T02_dX+d_T02_dX*T02)-(T02*T02)*d_T12_dX)/T12/T12))+d_CoxWL2_dX*(((0.5*Vgsteff2)+(0.25*T02))-((T02*T02)/T12))));
qsrc = (qsrc-(CoxWL2*(((0.5*Vgsteff2)+(0.25*T02))-((T02*T02)/T12))));
}
}
else
{
if ((instanceStruct.instanceVar_B4SOIxpart<0.5))
{
d_T1_dX = (d_T1_dX/12.0);
T1 = (T1/12.0);
d_T2_dX = (((T1*T1)*0.5*d_CoxWL_dX-(0.5*CoxWL)*(T1*d_T1_dX+d_T1_dX*T1))/(T1*T1)/(T1*T1));
T2 = ((0.5*CoxWL)/(T1*T1));
d_T3_dX = ((Vgsteff*((((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0)/3.0)+(Vgsteff*(d_Vgsteff_dX-(4.0*d_T0_dX/3.0))+d_Vgsteff_dX*(Vgsteff-((4.0*T0)/3.0))))+d_Vgsteff_dX*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dX+((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0)*T0)/15.0));
T3 = ((Vgsteff*((((2.0*T0)*T0)/3.0)+(Vgsteff*(Vgsteff-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*T0)/15.0));
d_qsrc_dX = ((-T2)*d_T3_dX+(-d_T2_dX)*T3);
qsrc = ((-T2)*T3);
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T12_dX = (d_T12_dX/12.0);
T12 = (T12/12.0);
d_T2_dX = (((T12*T12)*0.5*d_CoxWL2_dX-(0.5*CoxWL2)*(T12*d_T12_dX+d_T12_dX*T12))/(T12*T12)/(T12*T12));
T2 = ((0.5*CoxWL2)/(T12*T12));
d_T3_dX = ((Vgsteff2*((((2.0*T02)*d_T02_dX+2.0*d_T02_dX*T02)/3.0)+(Vgsteff2*(d_Vgsteff2_dX-(4.0*d_T02_dX/3.0))+d_Vgsteff2_dX*(Vgsteff2-((4.0*T02)/3.0))))+d_Vgsteff2_dX*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dX+((2.0*T02)*d_T02_dX+2.0*d_T02_dX*T02)*T02)/15.0));
T3 = ((Vgsteff2*((((2.0*T02)*T02)/3.0)+(Vgsteff2*(Vgsteff2-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*T02)/15.0));
d_qsrc2_dX = ((-T2)*d_T3_dX+(-d_T2_dX)*T3);
qsrc2 = ((-T2)*T3);
d_qsrc_dX = (d_qsrc_dX+d_qsrc2_dX);
qsrc = (qsrc+qsrc2);
}
}
else
{
d_qsrc_dX = (-0.5)*(d_qinv_dX+d_qbulk_dX);
qsrc = ((-0.5)*(qinv+qbulk));
}
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_Qe1_dX = 0.0;
Qe1 = 0;
}
else
{
d_CboxWL_dX = (((instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.instanceVar_B4SOIfbody)*Cbox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.d_instanceVar_pParam_B4SOIleffCVbg_dX+((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.d_instanceVar_B4SOInf_dX+((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX-instanceStruct.instanceVar_pParam_B4SOIweffCV*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVbg)+instanceStruct.d_instanceVar_B4SOIaebcp_dX)+((instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.instanceVar_B4SOIfbody)*d_Cbox_dX+(instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.d_instanceVar_B4SOIfbody_dX+instanceStruct.d_instanceVar_pParam_B4SOIkb1_dX*instanceStruct.instanceVar_B4SOIfbody)*Cbox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVbg)+instanceStruct.instanceVar_B4SOIaebcp));
CboxWL = (((instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.instanceVar_B4SOIfbody)*Cbox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVbg)+instanceStruct.instanceVar_B4SOIaebcp));
d_Qe1_dX = (CboxWL*(d_Vesfb_dX-d_Vbs_dX)+d_CboxWL_dX*(Vesfb-Vbs));
Qe1 = (CboxWL*(Vesfb-Vbs));
}
d_qgate_dX = ((d_qinv_dX+d_Qac0_dX)+d_Qsub0_dX);
qgate = ((qinv+Qac0)+Qsub0);
d_qbody_dX = (((d_qbulk_dX-d_Qac0_dX)-d_Qsub0_dX)-d_Qe1_dX);
qbody = (((qbulk-Qac0)-Qsub0)-Qe1);
d_qsub_dX = d_Qe1_dX;
qsub = Qe1;
d_qdrn_dX = (-(((d_qgate_dX+d_qsrc_dX)+d_qbody_dX)+d_qsub_dX));
qdrn = (-(((qgate+qsrc)+qbody)+qsub));
}
else
{
if ((instanceStruct.instanceVar_B4SOIcapMod==3))
{
if ((instanceStruct.instanceVar_B4SOImtrlMod==0))
{
d_Cox_dX = (-3.453133e-11*instanceStruct.d_instanceVar_B4SOItoxp_dX/instanceStruct.instanceVar_B4SOItoxp/instanceStruct.instanceVar_B4SOItoxp);
Cox = (3.453133e-11/instanceStruct.instanceVar_B4SOItoxp);
}
else
{
d_Cox_dX = ((instanceStruct.instanceVar_B4SOItoxp*instanceStruct.d_instanceVar_epsrox_dX*8.85418e-12-(instanceStruct.instanceVar_epsrox*8.85418e-12)*instanceStruct.d_instanceVar_B4SOItoxp_dX)/instanceStruct.instanceVar_B4SOItoxp/instanceStruct.instanceVar_B4SOItoxp);
Cox = ((instanceStruct.instanceVar_epsrox*8.85418e-12)/instanceStruct.instanceVar_B4SOItoxp);
}
d_CoxWL_dX = ((instanceStruct.instanceVar_B4SOItoxp*(CoxWL*instanceStruct.d_instanceVar_toxe_dX+d_CoxWL_dX*instanceStruct.instanceVar_toxe)-(CoxWL*instanceStruct.instanceVar_toxe)*instanceStruct.d_instanceVar_B4SOItoxp_dX)/instanceStruct.instanceVar_B4SOItoxp/instanceStruct.instanceVar_B4SOItoxp);
CoxWL = ((CoxWL*instanceStruct.instanceVar_toxe)/instanceStruct.instanceVar_B4SOItoxp);
d_CoxWLb_dX = ((instanceStruct.instanceVar_B4SOItoxp*(CoxWLb*instanceStruct.d_instanceVar_B4SOItox_dX+d_CoxWLb_dX*instanceStruct.instanceVar_B4SOItox)-(CoxWLb*instanceStruct.instanceVar_B4SOItox)*instanceStruct.d_instanceVar_B4SOItoxp_dX)/instanceStruct.instanceVar_B4SOItoxp/instanceStruct.instanceVar_B4SOItoxp);
CoxWLb = ((CoxWLb*instanceStruct.instanceVar_B4SOItox)/instanceStruct.instanceVar_B4SOItoxp);
d_Tox_dX = 1.0e8*instanceStruct.d_instanceVar_B4SOItoxp_dX;
Tox = (1.0e8*instanceStruct.instanceVar_B4SOItoxp);
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
d_CoxWL2_dX = ((instanceStruct.instanceVar_B4SOItoxp*(CoxWL2*instanceStruct.d_instanceVar_B4SOItox_dX+d_CoxWL2_dX*instanceStruct.instanceVar_B4SOItox)-(CoxWL2*instanceStruct.instanceVar_B4SOItox)*instanceStruct.d_instanceVar_B4SOItoxp_dX)/instanceStruct.instanceVar_B4SOItoxp/instanceStruct.instanceVar_B4SOItoxp);
CoxWL2 = ((CoxWL2*instanceStruct.instanceVar_B4SOItox)/instanceStruct.instanceVar_B4SOItoxp);
d_CoxWLb2_dX = ((instanceStruct.instanceVar_B4SOItoxp*(CoxWLb2*instanceStruct.d_instanceVar_B4SOItox_dX+d_CoxWLb2_dX*instanceStruct.instanceVar_B4SOItox)-(CoxWLb2*instanceStruct.instanceVar_B4SOItox)*instanceStruct.d_instanceVar_B4SOItoxp_dX)/instanceStruct.instanceVar_B4SOItoxp/instanceStruct.instanceVar_B4SOItoxp);
CoxWLb2 = ((CoxWLb2*instanceStruct.instanceVar_B4SOItox)/instanceStruct.instanceVar_B4SOItoxp);
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_Qac0_dX = 0.0;
Qac0 = 0.0;
d_Qsub0_dX = 0.0;
Qsub0 = 0.0;
d_Vfbzb_dX = 0.0;
Vfbzb = 0;
}
else
{
if (((instanceStruct.instancePar_SHMOD==1)&&(instanceStruct.instancePar_RTH0!=0.0)))
{
d_Vfbzb_dX = (((d_Vthzb_dX-d_phi_dX)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*d_sqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX*sqrtPhi))+instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX);
Vfbzb = (((Vthzb-phi)-(instanceStruct.instanceVar_pParam_B4SOIk1eff*sqrtPhi))+instanceStruct.instanceVar_pParam_B4SOIdelvt);
}
else
{
d_Vfbzb_dX = (instanceStruct.d_instanceVar_B4SOIvfbzb_dX+instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX);
Vfbzb = (instanceStruct.instanceVar_B4SOIvfbzb+instanceStruct.instanceVar_pParam_B4SOIdelvt);
}
d_V3_dX = ((d_Vfbzb_dX-d_Vgs_eff_dX)+d_Vbseff_dX);
V3 = (((Vfbzb-Vgs_eff)+Vbseff)-0.02);
if ((Vfbzb<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((4.0*0.02)*Vfbzb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)-(4.0*0.02)*d_Vfbzb_dX));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*0.02)*Vfbzb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)+(4.0*0.02)*d_Vfbzb_dX));
T0 = value_sqrt_0;
}
}
d_Vfbeff_dX = (d_Vfbzb_dX-0.5*(d_V3_dX+d_T0_dX));
Vfbeff = (Vfbzb-(0.5*(V3+T0)));
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
d_Vfbzb2_dX = (d_Vfbzb_dX+instanceStruct.d_instanceVar_eggbcp2_dX);
Vfbzb2 = (Vfbzb+instanceStruct.instanceVar_eggbcp2);
d_V3_dX = (d_Vfbzb2_dX+d_Vbseff_dX);
V3 = (((Vfbzb2-Vgs_eff2)+Vbseff)-0.02);
if ((Vfbzb2<=0.0))
{
{
double value_sqrt_0 = sqrt(((V3*V3)-((100.0*0.02)*Vfbzb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)-(100.0*0.02)*d_Vfbzb2_dX));
T0 = value_sqrt_0;
}
}
else
{
{
double value_sqrt_0 = sqrt(((V3*V3)+((100.0*0.02)*Vfbzb2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)+(100.0*0.02)*d_Vfbzb2_dX));
T0 = value_sqrt_0;
}
}
d_Vfbeff2_dX = (d_Vfbzb2_dX-0.5*(d_V3_dX+d_T0_dX));
Vfbeff2 = (Vfbzb2-(0.5*(V3+T0)));
}
d_T0_dX = ((Tox*((d_Vgs_eff_dX-d_Vbseff_dX)-d_Vfbzb_dX)-((Vgs_eff-Vbseff)-Vfbzb)*d_Tox_dX)/Tox/Tox);
T0 = (((Vgs_eff-Vbseff)-Vfbzb)/Tox);
d_tmp_dX = (T0*instanceStruct.d_instanceVar_pParam_B4SOIacde_dX+d_T0_dX*instanceStruct.instanceVar_pParam_B4SOIacde);
tmp = (T0*instanceStruct.instanceVar_pParam_B4SOIacde);
if ((((-100.0)<tmp)&&(tmp<100.0)))
{
{
double value_exp_0 = exp(tmp);
double  deriv_exp_0_d0 = value_exp_0;
d_Tcen_dX = (instanceStruct.instanceVar_pParam_B4SOIldeb*deriv_exp_0_d0*(d_tmp_dX)+instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX*value_exp_0);
Tcen = (instanceStruct.instanceVar_pParam_B4SOIldeb*value_exp_0);
}
}
else
{
if ((tmp<=(-100.0)))
{
d_Tcen_dX = instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX*3.720075976e-44;
Tcen = (instanceStruct.instanceVar_pParam_B4SOIldeb*3.720075976e-44);
}
else
{
d_Tcen_dX = instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX*2.688117142e+43;
Tcen = (instanceStruct.instanceVar_pParam_B4SOIldeb*2.688117142e+43);
}
}
d_LINK_dX = 1.0e-3*instanceStruct.d_instanceVar_B4SOItoxp_dX;
LINK = (1.0e-3*instanceStruct.instanceVar_B4SOItoxp);
d_V3_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX-d_Tcen_dX)-d_LINK_dX);
V3 = ((instanceStruct.instanceVar_pParam_B4SOIldeb-Tcen)-LINK);
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*LINK)*instanceStruct.instanceVar_pParam_B4SOIldeb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_V4_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)+((4.0*LINK)*instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX+4.0*d_LINK_dX*instanceStruct.instanceVar_pParam_B4SOIldeb)));
V4 = value_sqrt_0;
}
d_Tcen_dX = (instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX-0.5*(d_V3_dX+d_V4_dX));
Tcen = (instanceStruct.instanceVar_pParam_B4SOIldeb-(0.5*(V3+V4)));
if ((Tcen<1e-15))
{
d_Tcen_dX = 0.0;
Tcen = 1e-15;
}
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
d_T0_dX = ((Tox*((-d_Vbseff_dX)-d_Vfbzb2_dX)-((Vgs_eff2-Vbseff)-Vfbzb2)*d_Tox_dX)/Tox/Tox);
T0 = (((Vgs_eff2-Vbseff)-Vfbzb2)/Tox);
d_tmp_dX = (T0*instanceStruct.d_instanceVar_pParam_B4SOIacde_dX+d_T0_dX*instanceStruct.instanceVar_pParam_B4SOIacde);
tmp = (T0*instanceStruct.instanceVar_pParam_B4SOIacde);
if ((((-100.0)<tmp)&&(tmp<100.0)))
{
{
double value_exp_0 = exp(tmp);
double  deriv_exp_0_d0 = value_exp_0;
d_Tcen2_dX = (instanceStruct.instanceVar_pParam_B4SOIldeb*deriv_exp_0_d0*(d_tmp_dX)+instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX*value_exp_0);
Tcen2 = (instanceStruct.instanceVar_pParam_B4SOIldeb*value_exp_0);
}
}
else
{
if ((tmp<=(-100.0)))
{
d_Tcen2_dX = instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX*3.720075976e-44;
Tcen2 = (instanceStruct.instanceVar_pParam_B4SOIldeb*3.720075976e-44);
}
else
{
d_Tcen2_dX = instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX*2.688117142e+43;
Tcen2 = (instanceStruct.instanceVar_pParam_B4SOIldeb*2.688117142e+43);
}
}
d_V3_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX-d_Tcen2_dX)-d_LINK_dX);
V3 = ((instanceStruct.instanceVar_pParam_B4SOIldeb-Tcen2)-LINK);
{
double value_sqrt_0 = sqrt(((V3*V3)+((4.0*LINK)*instanceStruct.instanceVar_pParam_B4SOIldeb)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_V4_dX = deriv_sqrt_0_d0*(((V3*d_V3_dX+d_V3_dX*V3)+((4.0*LINK)*instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX+4.0*d_LINK_dX*instanceStruct.instanceVar_pParam_B4SOIldeb)));
V4 = value_sqrt_0;
}
d_Tcen2_dX = (instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX-0.5*(d_V3_dX+d_V4_dX));
Tcen2 = (instanceStruct.instanceVar_pParam_B4SOIldeb-(0.5*(V3+V4)));
if ((Tcen2<1e-15))
{
d_Tcen2_dX = 0.0;
Tcen2 = 1e-15;
}
}
d_Ccen_dX = ((Tcen*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*d_Tcen_dX)/Tcen/Tcen);
Ccen = (instanceStruct.instanceVar_epssub/Tcen);
d_T2_dX = (((Cox+Ccen)*d_Cox_dX-Cox*(d_Cox_dX+d_Ccen_dX))/(Cox+Ccen)/(Cox+Ccen));
T2 = (Cox/(Cox+Ccen));
d_Coxeff_dX = (T2*d_Ccen_dX+d_T2_dX*Ccen);
Coxeff = (T2*Ccen);
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_Ccen2_dX = ((Tcen2*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*d_Tcen2_dX)/Tcen2/Tcen2);
Ccen2 = (instanceStruct.instanceVar_epssub/Tcen2);
d_T2_dX = (((Cox+Ccen2)*d_Cox_dX-Cox*(d_Cox_dX+d_Ccen2_dX))/(Cox+Ccen2)/(Cox+Ccen2));
T2 = (Cox/(Cox+Ccen2));
d_Coxeff2_dX = (T2*d_Ccen2_dX+d_T2_dX*Ccen2);
Coxeff2 = (T2*Ccen2);
}
d_CoxWLcenb_dX = ((Cox*(CoxWLb*d_Coxeff_dX+d_CoxWLb_dX*Coxeff)-(CoxWLb*Coxeff)*d_Cox_dX)/Cox/Cox);
CoxWLcenb = ((CoxWLb*Coxeff)/Cox);
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
d_CoxWLcenb2_dX = ((Cox*(CoxWLb2*d_Coxeff2_dX+d_CoxWLb2_dX*Coxeff2)-(CoxWLb2*Coxeff2)*d_Cox_dX)/Cox/Cox);
CoxWLcenb2 = ((CoxWLb2*Coxeff2)/Cox);
}
d_Qac0_dX = (CoxWLcenb*(d_Vfbeff_dX-d_Vfbzb_dX)+d_CoxWLcenb_dX*(Vfbeff-Vfbzb));
Qac0 = (CoxWLcenb*(Vfbeff-Vfbzb));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_Qac02_dX = (CoxWLcenb2*(d_Vfbeff2_dX-d_Vfbzb2_dX)+d_CoxWLcenb2_dX*(Vfbeff2-Vfbzb2));
Qac02 = (CoxWLcenb2*(Vfbeff2-Vfbzb2));
d_Qac0_dX = (d_Qac0_dX+d_Qac02_dX);
Qac0 = (Qac0+Qac02);
}
d_T0_dX = 0.5*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX;
T0 = (0.5*instanceStruct.instanceVar_pParam_B4SOIk1ox);
d_T3_dX = (((d_Vgs_eff_dX-d_Vfbeff_dX)-d_Vbseff_dX)-d_Vgsteff_dX);
T3 = (((Vgs_eff-Vfbeff)-Vbseff)-Vgsteff);
if ((instanceStruct.instanceVar_pParam_B4SOIk1ox==0.0))
{
d_T1_dX = 0.0;
T1 = 0.0;
}
else
{
if ((T3<0.0))
{
d_T1_dX = (d_T0_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T3_dX-T3*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX)/instanceStruct.instanceVar_pParam_B4SOIk1ox/instanceStruct.instanceVar_pParam_B4SOIk1ox));
T1 = (T0+(T3/instanceStruct.instanceVar_pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*(((T0*d_T0_dX+d_T0_dX*T0)+d_T3_dX));
T1 = value_sqrt_0;
}
}
}
d_Qsub0_dX = ((CoxWLcenb*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(d_T1_dX-d_T0_dX)+(CoxWLcenb*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+d_CoxWLcenb_dX*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0));
Qsub0 = ((CoxWLcenb*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T3_dX = (((-d_Vfbeff2_dX)-d_Vbseff_dX)-d_Vgsteff2_dX);
T3 = (((Vgs_eff2-Vfbeff2)-Vbseff)-Vgsteff2);
if ((instanceStruct.instanceVar_pParam_B4SOIk1ox==0.0))
{
d_T1_dX = 0.0;
T1 = 0.0;
}
else
{
if ((T3<0.0))
{
d_T1_dX = (d_T0_dX+((instanceStruct.instanceVar_pParam_B4SOIk1ox*d_T3_dX-T3*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX)/instanceStruct.instanceVar_pParam_B4SOIk1ox/instanceStruct.instanceVar_pParam_B4SOIk1ox));
T1 = (T0+(T3/instanceStruct.instanceVar_pParam_B4SOIk1ox));
}
else
{
{
double value_sqrt_0 = sqrt(((T0*T0)+T3));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*(((T0*d_T0_dX+d_T0_dX*T0)+d_T3_dX));
T1 = value_sqrt_0;
}
}
}
d_Qsub02_dX = ((CoxWLcenb2*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(d_T1_dX-d_T0_dX)+(CoxWLcenb2*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+d_CoxWLcenb2_dX*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0));
Qsub02 = ((CoxWLcenb2*instanceStruct.instanceVar_pParam_B4SOIk1ox)*(T1-T0));
d_Qsub0_dX = (d_Qsub0_dX+d_Qsub02_dX);
Qsub0 = (Qsub0+Qsub02);
}
}
if ((instanceStruct.instanceVar_pParam_B4SOIk1ox<=0.0))
{
d_Denomi_dX = ((0.25*instanceStruct.instanceVar_pParam_B4SOImoin)*d_Vtm_dX+0.25*instanceStruct.d_instanceVar_pParam_B4SOImoin_dX*Vtm);
Denomi = ((0.25*instanceStruct.instanceVar_pParam_B4SOImoin)*Vtm);
d_T0_dX = 0.5*instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX;
T0 = (0.5*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi);
}
else
{
d_Denomi_dX = (((instanceStruct.instanceVar_pParam_B4SOImoin*Vtm)*instanceStruct.instanceVar_pParam_B4SOIk1ox)*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+((instanceStruct.instanceVar_pParam_B4SOImoin*Vtm)*instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX+(instanceStruct.instanceVar_pParam_B4SOImoin*d_Vtm_dX+instanceStruct.d_instanceVar_pParam_B4SOImoin_dX*Vtm)*instanceStruct.instanceVar_pParam_B4SOIk1ox)*instanceStruct.instanceVar_pParam_B4SOIk1ox);
Denomi = (((instanceStruct.instanceVar_pParam_B4SOImoin*Vtm)*instanceStruct.instanceVar_pParam_B4SOIk1ox)*instanceStruct.instanceVar_pParam_B4SOIk1ox);
d_T0_dX = (instanceStruct.instanceVar_pParam_B4SOIk1ox*instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX+instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi);
T0 = (instanceStruct.instanceVar_pParam_B4SOIk1ox*instanceStruct.instanceVar_pParam_B4SOIsqrtPhi);
}
d_T1_dX = (2.0*d_T0_dX+d_Vgsteff_dX);
T1 = ((2.0*T0)+Vgsteff);
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+((T1*Vgsteff)/Denomi)));
d_DeltaPhi_dX = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff)/Denomi)),((Denomi*(T1*d_Vgsteff_dX+d_T1_dX*Vgsteff)-(T1*Vgsteff)*d_Denomi_dX)/Denomi/Denomi))+d_Vtm_dX*evaluator_lln_0.getValues((1.0+((T1*Vgsteff)/Denomi))));
DeltaPhi = (Vtm*evaluator_lln_0.getValues((1.0+((T1*Vgsteff)/Denomi))));
}
if ((instanceStruct.instanceVar_B4SOIagbcp2>0))
{
d_T1_dX = (2.0*d_T0_dX+d_Vgsteff2_dX);
T1 = ((2.0*T0)+Vgsteff2);
{
AnalogFunctions::llnEvaluator evaluator_lln_0((1.0+((T1*Vgsteff2)/Denomi)));
d_DeltaPhi2_dX = (Vtm*evaluator_lln_0.getDerivs((1.0+((T1*Vgsteff2)/Denomi)),((Denomi*(T1*d_Vgsteff2_dX+d_T1_dX*Vgsteff2)-(T1*Vgsteff2)*d_Denomi_dX)/Denomi/Denomi))+d_Vtm_dX*evaluator_lln_0.getValues((1.0+((T1*Vgsteff2)/Denomi))));
DeltaPhi2 = (Vtm*evaluator_lln_0.getValues((1.0+((T1*Vgsteff2)/Denomi))));
}
}
d_T3_dX = 4.0*((d_Vth_dX-d_Vfbzb_dX)-d_phi_dX);
T3 = (4.0*((Vth-Vfbzb)-phi));
{
double value_sqrt_0 = sqrt(((T3*T3)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*((T3*d_T3_dX+d_T3_dX*T3));
T2 = value_sqrt_0;
}
d_T4_dX = 0.5*(d_T3_dX+d_T2_dX);
T4 = (0.5*(T3+T2));
d_Tox_dX = (d_Tox_dX+d_Tox_dX);
Tox = (Tox+Tox);
d_T0_dX = ((Tox*(d_Vgsteff_dX+d_T4_dX)-(Vgsteff+T4)*d_Tox_dX)/Tox/Tox);
T0 = ((Vgsteff+T4)/Tox);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp(((instanceStruct.instanceVar_B4SOIbdos*0.7)*evaluator_lln_0.getValues(T0)));
double  deriv_exp_1_d0 = value_exp_1;
d_tmp_dX = deriv_exp_1_d0*(((instanceStruct.instanceVar_B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dX)+instanceStruct.d_instanceVar_B4SOIbdos_dX*0.7*evaluator_lln_0.getValues(T0)));
tmp = value_exp_1;
}
d_T1_dX = d_tmp_dX;
T1 = (1.0+tmp);
d_Tcen_dX = ((T1*instanceStruct.d_instanceVar_B4SOIados_dX*1.9e-9-(instanceStruct.instanceVar_B4SOIados*1.9e-9)*d_T1_dX)/T1/T1);
Tcen = ((instanceStruct.instanceVar_B4SOIados*1.9e-9)/T1);
d_Ccen_dX = ((Tcen*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*d_Tcen_dX)/Tcen/Tcen);
Ccen = (instanceStruct.instanceVar_epssub/Tcen);
d_T0_dX = (((Cox+Ccen)*d_Cox_dX-Cox*(d_Cox_dX+d_Ccen_dX))/(Cox+Ccen)/(Cox+Ccen));
T0 = (Cox/(Cox+Ccen));
d_Coxeff_dX = (T0*d_Ccen_dX+d_T0_dX*Ccen);
Coxeff = (T0*Ccen);
d_CoxWLcen_dX = ((Cox*(CoxWL*d_Coxeff_dX+d_CoxWL_dX*Coxeff)-(CoxWL*Coxeff)*d_Cox_dX)/Cox/Cox);
CoxWLcen = ((CoxWL*Coxeff)/Cox);
d_CoxWLcenb_dX = ((Cox*(CoxWLb*d_Coxeff_dX+d_CoxWLb_dX*Coxeff)-(CoxWLb*Coxeff)*d_Cox_dX)/Cox/Cox);
CoxWLcenb = ((CoxWLb*Coxeff)/Cox);
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T3_dX = 4.0*(((d_Vth_dX+instanceStruct.d_instanceVar_eggbcp2_dX)-d_Vfbzb2_dX)-d_phi_dX);
T3 = (4.0*(((Vth+instanceStruct.instanceVar_eggbcp2)-Vfbzb2)-phi));
{
double value_sqrt_0 = sqrt(((T3*T3)+0.0001));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T2_dX = deriv_sqrt_0_d0*((T3*d_T3_dX+d_T3_dX*T3));
T2 = value_sqrt_0;
}
d_T4_dX = 0.5*(d_T3_dX+d_T2_dX);
T4 = (0.5*(T3+T2));
d_T0_dX = ((Tox*(d_Vgsteff2_dX+d_T4_dX)-(Vgsteff2+T4)*d_Tox_dX)/Tox/Tox);
T0 = ((Vgsteff2+T4)/Tox);
{
AnalogFunctions::llnEvaluator evaluator_lln_0(T0);
double value_exp_1 = exp(((instanceStruct.instanceVar_B4SOIbdos*0.7)*evaluator_lln_0.getValues(T0)));
double  deriv_exp_1_d0 = value_exp_1;
d_tmp_dX = deriv_exp_1_d0*(((instanceStruct.instanceVar_B4SOIbdos*0.7)*evaluator_lln_0.getDerivs(T0,d_T0_dX)+instanceStruct.d_instanceVar_B4SOIbdos_dX*0.7*evaluator_lln_0.getValues(T0)));
tmp = value_exp_1;
}
d_T1_dX = d_tmp_dX;
T1 = (1.0+tmp);
d_Tcen2_dX = ((T1*instanceStruct.d_instanceVar_B4SOIados_dX*1.9e-9-(instanceStruct.instanceVar_B4SOIados*1.9e-9)*d_T1_dX)/T1/T1);
Tcen2 = ((instanceStruct.instanceVar_B4SOIados*1.9e-9)/T1);
d_Ccen2_dX = ((Tcen2*instanceStruct.d_instanceVar_epssub_dX-instanceStruct.instanceVar_epssub*d_Tcen2_dX)/Tcen2/Tcen2);
Ccen2 = (instanceStruct.instanceVar_epssub/Tcen2);
d_T0_dX = (((Cox+Ccen2)*d_Cox_dX-Cox*(d_Cox_dX+d_Ccen2_dX))/(Cox+Ccen2)/(Cox+Ccen2));
T0 = (Cox/(Cox+Ccen2));
d_Coxeff2_dX = (T0*d_Ccen2_dX+d_T0_dX*Ccen2);
Coxeff2 = (T0*Ccen2);
d_CoxWLcen2_dX = ((Cox*(CoxWL2*d_Coxeff2_dX+d_CoxWL2_dX*Coxeff2)-(CoxWL2*Coxeff2)*d_Cox_dX)/Cox/Cox);
CoxWLcen2 = ((CoxWL2*Coxeff2)/Cox);
d_CoxWLcenb2_dX = ((Cox*(CoxWLb2*d_Coxeff2_dX+d_CoxWLb2_dX*Coxeff2)-(CoxWLb2*Coxeff2)*d_Cox_dX)/Cox/Cox);
CoxWLcenb2 = ((CoxWLb2*Coxeff2)/Cox);
}
d_T1_dX = (d_Vgsteff_dX-d_DeltaPhi_dX);
T1 = (Vgsteff-DeltaPhi);
d_AbulkCV_dX = (Abulk0*instanceStruct.d_instanceVar_pParam_B4SOIabulkCVfactor_dX+d_Abulk0_dX*instanceStruct.instanceVar_pParam_B4SOIabulkCVfactor);
AbulkCV = (Abulk0*instanceStruct.instanceVar_pParam_B4SOIabulkCVfactor);
d_VdsatCV_dX = ((AbulkCV*d_T1_dX-T1*d_AbulkCV_dX)/AbulkCV/AbulkCV);
VdsatCV = (T1/AbulkCV);
d_V4_dX = d_VdsatCV_dX;
V4 = ((VdsatCV-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T0_dX = deriv_sqrt_0_d0*(((V4*d_V4_dX+d_V4_dX*V4)+(4.0*0.02)*d_VdsatCV_dX));
T0 = value_sqrt_0;
}
d_VdseffCV_dX = (d_VdsatCV_dX-0.5*(d_V4_dX+d_T0_dX));
VdseffCV = (VdsatCV-(0.5*(V4+T0)));
d_T0_dX = (AbulkCV*d_VdseffCV_dX+d_AbulkCV_dX*VdseffCV);
T0 = (AbulkCV*VdseffCV);
d_T2_dX = 12.0*(d_T1_dX-0.5*d_T0_dX);
T2 = (12.0*((T1-(0.5*T0))+1.0e-20));
d_T3_dX = ((T2*d_T0_dX-T0*d_T2_dX)/T2/T2);
T3 = (T0/T2);
d_qinv_dX = (CoxWLcen*(d_T1_dX-(T0*(-d_T3_dX)+d_T0_dX*(0.5-T3)))+d_CoxWLcen_dX*(T1-(T0*(0.5-T3))));
qinv = (CoxWLcen*(T1-(T0*(0.5-T3))));
d_qinoi_dX = d_qinv_dX;
qinoi = qinv;
d_qgate_dX = d_qinv_dX;
qgate = qinv;
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T12_dX = (d_Vgsteff2_dX-d_DeltaPhi2_dX);
T12 = (Vgsteff2-DeltaPhi2);
d_VdsatCV2_dX = ((AbulkCV*d_T12_dX-T12*d_AbulkCV_dX)/AbulkCV/AbulkCV);
VdsatCV2 = (T12/AbulkCV);
d_V4_dX = d_VdsatCV2_dX;
V4 = ((VdsatCV2-Vds)-0.02);
{
double value_sqrt_0 = sqrt(((V4*V4)+((4.0*0.02)*VdsatCV2)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T02_dX = deriv_sqrt_0_d0*(((V4*d_V4_dX+d_V4_dX*V4)+(4.0*0.02)*d_VdsatCV2_dX));
T02 = value_sqrt_0;
}
d_VdseffCV2_dX = (d_VdsatCV2_dX-0.5*(d_V4_dX+d_T02_dX));
VdseffCV2 = (VdsatCV2-(0.5*(V4+T02)));
d_T02_dX = (AbulkCV*d_VdseffCV2_dX+d_AbulkCV_dX*VdseffCV2);
T02 = (AbulkCV*VdseffCV2);
d_T22_dX = 12.0*(d_T12_dX-0.5*d_T02_dX);
T22 = (12.0*((T12-(0.5*T02))+1.0e-20));
d_T3_dX = ((T22*d_T02_dX-T02*d_T22_dX)/T22/T22);
T3 = (T02/T22);
d_T7_dX = (CoxWLcen2*(d_T12_dX-(T02*(-d_T3_dX)+d_T02_dX*(0.5-T3)))+d_CoxWLcen2_dX*(T12-(T02*(0.5-T3))));
T7 = (CoxWLcen2*(T12-(T02*(0.5-T3))));
d_qinv_dX = (d_qinv_dX+d_T7_dX);
qinv = (qinv+T7);
d_qinoi_dX = d_qinv_dX;
qinoi = qinv;
d_qgate_dX = d_qinv_dX;
qgate = qinv;
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_qbulk_dX = 0.0;
qbulk = 0;
}
else
{
d_T7_dX = (-d_AbulkCV_dX);
T7 = (1.0-AbulkCV);
d_qbulk_dX = ((CoxWLcenb*T7)*(0.5*d_VdseffCV_dX-((T2*(T0*d_VdseffCV_dX+d_T0_dX*VdseffCV)-(T0*VdseffCV)*d_T2_dX)/T2/T2))+(CoxWLcenb*d_T7_dX+d_CoxWLcenb_dX*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
qbulk = ((CoxWLcenb*T7)*((0.5*VdseffCV)-((T0*VdseffCV)/T2)));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_qbulk2_dX = ((CoxWLcenb2*T7)*(0.5*d_VdseffCV2_dX-((T22*(T02*d_VdseffCV2_dX+d_T02_dX*VdseffCV2)-(T02*VdseffCV2)*d_T22_dX)/T22/T22))+(CoxWLcenb2*d_T7_dX+d_CoxWLcenb2_dX*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
qbulk2 = ((CoxWLcenb2*T7)*((0.5*VdseffCV2)-((T02*VdseffCV2)/T22)));
d_qbulk_dX = (d_qbulk_dX+d_qbulk2_dX);
qbulk = (qbulk+qbulk2);
}
}
if ((instanceStruct.instanceVar_B4SOIxpart>0.5))
{
d_qsrc_dX = ((-CoxWLcen)*(((d_T1_dX/2.0)+(d_T0_dX/4.0))-((T2*((0.5*T0)*d_T0_dX+0.5*d_T0_dX*T0)-((0.5*T0)*T0)*d_T2_dX)/T2/T2))+(-d_CoxWLcen_dX)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
qsrc = ((-CoxWLcen)*(((T1/2.0)+(T0/4.0))-(((0.5*T0)*T0)/T2)));
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_qsrc2_dX = ((-CoxWLcen2)*((((d_Vgsteff2_dX-d_DeltaPhi2_dX)/2.0)+(d_T02_dX/4.0))-((T22*((0.5*T02)*d_T02_dX+0.5*d_T02_dX*T02)-((0.5*T02)*T02)*d_T22_dX)/T22/T22))+(-d_CoxWLcen2_dX)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
qsrc2 = ((-CoxWLcen2)*((((Vgsteff2-DeltaPhi2)/2.0)+(T02/4.0))-(((0.5*T02)*T02)/T22)));
d_qsrc_dX = (d_qsrc_dX+d_qsrc2_dX);
qsrc = (qsrc+qsrc2);
}
}
else
{
if ((instanceStruct.instanceVar_B4SOIxpart<0.5))
{
d_T2_dX = (d_T2_dX/12.0);
T2 = (T2/12.0);
d_T3_dX = (((T2*T2)*0.5*d_CoxWLcen_dX-(0.5*CoxWLcen)*(T2*d_T2_dX+d_T2_dX*T2))/(T2*T2)/(T2*T2));
T3 = ((0.5*CoxWLcen)/(T2*T2));
d_T4_dX = ((T1*((((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0)/3.0)+(T1*(d_T1_dX-(4.0*d_T0_dX/3.0))+d_T1_dX*(T1-((4.0*T0)/3.0))))+d_T1_dX*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*d_T0_dX+((2.0*T0)*d_T0_dX+2.0*d_T0_dX*T0)*T0)/15.0));
T4 = ((T1*((((2.0*T0)*T0)/3.0)+(T1*(T1-((4.0*T0)/3.0)))))-((((2.0*T0)*T0)*T0)/15.0));
d_qsrc_dX = ((-T3)*d_T4_dX+(-d_T3_dX)*T4);
qsrc = ((-T3)*T4);
if ((((instanceStruct.instanceVar_B4SOIsoiMod!=2)&&(instanceStruct.instanceVar_B4SOIbodyMod!=0))&&(instanceStruct.instanceVar_B4SOIagbcp2>0)))
{
d_T22_dX = (d_T22_dX/12.0);
T22 = (T22/12.0);
d_T3_dX = (((T22*T22)*0.5*d_CoxWLcen2_dX-(0.5*CoxWLcen2)*(T22*d_T22_dX+d_T22_dX*T22))/(T22*T22)/(T22*T22));
T3 = ((0.5*CoxWLcen2)/(T22*T22));
d_T4_dX = ((T12*((((2.0*T02)*d_T02_dX+2.0*d_T02_dX*T02)/3.0)+(T12*(d_T12_dX-(4.0*d_T02_dX/3.0))+d_T12_dX*(T12-((4.0*T02)/3.0))))+d_T12_dX*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*d_T02_dX+((2.0*T02)*d_T02_dX+2.0*d_T02_dX*T02)*T02)/15.0));
T4 = ((T12*((((2.0*T02)*T02)/3.0)+(T12*(T12-((4.0*T02)/3.0)))))-((((2.0*T02)*T02)*T02)/15.0));
d_qsrc2_dX = ((-T3)*d_T4_dX+(-d_T3_dX)*T4);
qsrc2 = ((-T3)*T4);
d_qsrc_dX = (d_qsrc_dX+d_qsrc2_dX);
qsrc = (qsrc+qsrc2);
}
}
else
{
d_qsrc_dX = (-0.5)*d_qgate_dX;
qsrc = ((-0.5)*qgate);
}
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_Qe1_dX = 0.0;
Qe1 = 0;
}
else
{
d_CboxWL_dX = (((instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.instanceVar_B4SOIfbody)*Cbox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.d_instanceVar_pParam_B4SOIleffCVbg_dX+((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.d_instanceVar_B4SOInf_dX+((instanceStruct.instanceVar_B4SOInseg*instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX-instanceStruct.instanceVar_pParam_B4SOIweffCV*instanceStruct.d_instanceVar_B4SOInseg_dX)/instanceStruct.instanceVar_B4SOInseg/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVbg)+instanceStruct.d_instanceVar_B4SOIaebcp_dX)+((instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.instanceVar_B4SOIfbody)*d_Cbox_dX+(instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.d_instanceVar_B4SOIfbody_dX+instanceStruct.d_instanceVar_pParam_B4SOIkb1_dX*instanceStruct.instanceVar_B4SOIfbody)*Cbox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVbg)+instanceStruct.instanceVar_B4SOIaebcp));
CboxWL = (((instanceStruct.instanceVar_pParam_B4SOIkb1*instanceStruct.instanceVar_B4SOIfbody)*Cbox)*((((instanceStruct.instanceVar_pParam_B4SOIweffCV/instanceStruct.instanceVar_B4SOInseg)*instanceStruct.instanceVar_B4SOInf)*instanceStruct.instanceVar_pParam_B4SOIleffCVbg)+instanceStruct.instanceVar_B4SOIaebcp));
d_Qe1_dX = (CboxWL*(d_Vesfb_dX-d_Vbs_dX)+d_CboxWL_dX*(Vesfb-Vbs));
Qe1 = (CboxWL*(Vesfb-Vbs));
}
d_qgate_dX = (((d_qgate_dX+d_Qac0_dX)+d_Qsub0_dX)-d_qbulk_dX);
qgate = (((qgate+Qac0)+Qsub0)-qbulk);
d_qbody_dX = (((d_qbulk_dX-d_Qac0_dX)-d_Qsub0_dX)-d_Qe1_dX);
qbody = (((qbulk-Qac0)-Qsub0)-Qe1);
d_qsub_dX = d_Qe1_dX;
qsub = Qe1;
d_qdrn_dX = (-(((d_qgate_dX+d_qbody_dX)+d_qsub_dX)+d_qsrc_dX));
qdrn = (-(((qgate+qbody)+qsub)+qsrc));
instanceStruct.d_instanceVar_B4SOIqinv_dX = (-d_qinoi_dX);
instanceStruct.instanceVar_B4SOIqinv = (-qinoi);
}
else
{
d_Qac0_dX = 0.0;
Qac0 = 0.0;
d_Qsub0_dX = 0.0;
Qsub0 = 0.0;
d_qsub_dX = 0.0;
qsub = 0.0;
d_qbody_dX = 0.0;
qbody = 0.0;
d_qsrc_dX = 0.0;
qsrc = 0.0;
d_qdrn_dX = 0.0;
qdrn = 0.0;
d_qgate_dX = 0.0;
qgate = 0.0;
}
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
d_qjd_dX = 0.0;
qjd = 0.0;
d_qjs_dX = 0.0;
qjs = 0.0;
}
else
{
d_PhiBSWG_dX = instanceStruct.d_instanceVar_B4SOIGatesidewallJctSPotential_dX;
PhiBSWG = instanceStruct.instanceVar_B4SOIGatesidewallJctSPotential;
d_dPhiBSWG_dT_dX = (-instanceStruct.d_instanceVar_B4SOItpbswg_dX);
dPhiBSWG_dT = (-instanceStruct.instanceVar_B4SOItpbswg);
d_PhiBSWG_dX = (d_PhiBSWG_dX+(dPhiBSWG_dT*(instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.d_instanceVar_B4SOItnom_dX)+d_dPhiBSWG_dT_dX*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
PhiBSWG = (PhiBSWG+(dPhiBSWG_dT*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
d_MJSWG_dX = instanceStruct.d_instanceVar_B4SOIbodyJctGateSideSGradingCoeff_dX;
MJSWG = instanceStruct.instanceVar_B4SOIbodyJctGateSideSGradingCoeff;
d_cjsbs_dX = ((((instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapS*instanceStruct.instanceVar_pParam_B4SOIwdiosCV)*instanceStruct.instanceVar_B4SOItsi)*instanceStruct.d_instanceVar_B4SOInf_dX+((instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapS*instanceStruct.instanceVar_pParam_B4SOIwdiosCV)*instanceStruct.d_instanceVar_B4SOItsi_dX+(instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapS*instanceStruct.d_instanceVar_pParam_B4SOIwdiosCV_dX+instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapS_dX*instanceStruct.instanceVar_pParam_B4SOIwdiosCV)*instanceStruct.instanceVar_B4SOItsi)*instanceStruct.instanceVar_B4SOInf)/1e-7);
cjsbs = ((((instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapS*instanceStruct.instanceVar_pParam_B4SOIwdiosCV)*instanceStruct.instanceVar_B4SOItsi)*instanceStruct.instanceVar_B4SOInf)/1e-7);
d_dcjsbs_dT_dX = (cjsbs*instanceStruct.d_instanceVar_B4SOItcjswg_dX+d_cjsbs_dX*instanceStruct.instanceVar_B4SOItcjswg);
dcjsbs_dT = (cjsbs*instanceStruct.instanceVar_B4SOItcjswg);
d_cjsbs_dX = (d_cjsbs_dX+(dcjsbs_dT*(instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.d_instanceVar_B4SOItnom_dX)+d_dcjsbs_dT_dX*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
cjsbs = (cjsbs+(dcjsbs_dT*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
d_cjdbs_dX = ((((instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapD*instanceStruct.instanceVar_pParam_B4SOIwdiodCV)*instanceStruct.instanceVar_B4SOItsi)*instanceStruct.d_instanceVar_B4SOInf_dX+((instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapD*instanceStruct.instanceVar_pParam_B4SOIwdiodCV)*instanceStruct.d_instanceVar_B4SOItsi_dX+(instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapD*instanceStruct.d_instanceVar_pParam_B4SOIwdiodCV_dX+instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapD_dX*instanceStruct.instanceVar_pParam_B4SOIwdiodCV)*instanceStruct.instanceVar_B4SOItsi)*instanceStruct.instanceVar_B4SOInf)/1e-7);
cjdbs = ((((instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapD*instanceStruct.instanceVar_pParam_B4SOIwdiodCV)*instanceStruct.instanceVar_B4SOItsi)*instanceStruct.instanceVar_B4SOInf)/1e-7);
d_dcjdbs_dT_dX = (cjdbs*instanceStruct.d_instanceVar_B4SOItcjswgd_dX+d_cjdbs_dX*instanceStruct.instanceVar_B4SOItcjswgd);
dcjdbs_dT = (cjdbs*instanceStruct.instanceVar_B4SOItcjswgd);
d_cjdbs_dX = (d_cjdbs_dX+(dcjdbs_dT*(instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.d_instanceVar_B4SOItnom_dX)+d_dcjdbs_dT_dX*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
cjdbs = (cjdbs+(dcjdbs_dT*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
d_DioMax_dX = 0.9*d_PhiBSWG_dX;
DioMax = (0.9*PhiBSWG);
d_arg_dX = (-((PhiBSWG*((vsbs>DioMax)?d_DioMax_dX:0.0)-((vsbs>DioMax)?DioMax:vsbs)*d_PhiBSWG_dX)/PhiBSWG/PhiBSWG));
arg = (1.0-(((vsbs>DioMax)?DioMax:vsbs)/PhiBSWG));
if ((MJSWG==0.5))
{
{
double value_sqrt_0 = sqrt(arg);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_dT3_dVb_dX = (-deriv_sqrt_0_d0*(d_arg_dX)/value_sqrt_0/value_sqrt_0);
dT3_dVb = (1.0/value_sqrt_0);
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(arg);
double value_exp_1 = exp(((-MJSWG)*evaluator_lln_0.getValues(arg)));
double  deriv_exp_1_d0 = value_exp_1;
d_dT3_dVb_dX = deriv_exp_1_d0*(((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dX)+(-d_MJSWG_dX)*evaluator_lln_0.getValues(arg)));
dT3_dVb = value_exp_1;
}
}
d_T3_dX = (((1.0-MJSWG)*((1.0-(arg*dT3_dVb))*d_PhiBSWG_dX+(-(arg*d_dT3_dVb_dX+d_arg_dX*dT3_dVb))*PhiBSWG)-((1.0-(arg*dT3_dVb))*PhiBSWG)*(-d_MJSWG_dX))/(1.0-MJSWG)/(1.0-MJSWG));
T3 = (((1.0-(arg*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
if ((vsbs>DioMax))
{
d_T3_dX = (d_T3_dX+(dT3_dVb*(-d_DioMax_dX)+d_dT3_dVb_dX*(vsbs-DioMax)));
T3 = (T3+(dT3_dVb*(vsbs-DioMax)));
}
d_qjs_dX = ((cjsbs*d_T3_dX+d_cjsbs_dX*T3)+((instanceStruct.instanceVar_B4SOItt*Ibsdif)*instanceStruct.d_instanceVar_B4SOInf_dX+(instanceStruct.instanceVar_B4SOItt*d_Ibsdif_dX+instanceStruct.d_instanceVar_B4SOItt_dX*Ibsdif)*instanceStruct.instanceVar_B4SOInf));
qjs = ((cjsbs*T3)+((instanceStruct.instanceVar_B4SOItt*Ibsdif)*instanceStruct.instanceVar_B4SOInf));
d_PhiBSWG_dX = instanceStruct.d_instanceVar_B4SOIGatesidewallJctDPotential_dX;
PhiBSWG = instanceStruct.instanceVar_B4SOIGatesidewallJctDPotential;
d_dPhiBSWG_dT_dX = (-instanceStruct.d_instanceVar_B4SOItpbswgd_dX);
dPhiBSWG_dT = (-instanceStruct.instanceVar_B4SOItpbswgd);
d_PhiBSWG_dX = (d_PhiBSWG_dX+(dPhiBSWG_dT*(instanceStruct.d_instanceVar_DevTemp_dX-instanceStruct.d_instanceVar_B4SOItnom_dX)+d_dPhiBSWG_dT_dX*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
PhiBSWG = (PhiBSWG+(dPhiBSWG_dT*(instanceStruct.instanceVar_DevTemp-instanceStruct.instanceVar_B4SOItnom)));
d_MJSWG_dX = instanceStruct.d_instanceVar_B4SOIbodyJctGateSideDGradingCoeff_dX;
MJSWG = instanceStruct.instanceVar_B4SOIbodyJctGateSideDGradingCoeff;
d_DioMax_dX = 0.9*d_PhiBSWG_dX;
DioMax = (0.9*PhiBSWG);
d_arg_dX = (-((PhiBSWG*((vdbd>DioMax)?d_DioMax_dX:0.0)-((vdbd>DioMax)?DioMax:vdbd)*d_PhiBSWG_dX)/PhiBSWG/PhiBSWG));
arg = (1.0-(((vdbd>DioMax)?DioMax:vdbd)/PhiBSWG));
if ((MJSWG==0.5))
{
{
double value_sqrt_0 = sqrt(arg);
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_dT3_dVb_dX = (-deriv_sqrt_0_d0*(d_arg_dX)/value_sqrt_0/value_sqrt_0);
dT3_dVb = (1.0/value_sqrt_0);
}
}
else
{
{
AnalogFunctions::llnEvaluator evaluator_lln_0(arg);
double value_exp_1 = exp(((-MJSWG)*evaluator_lln_0.getValues(arg)));
double  deriv_exp_1_d0 = value_exp_1;
d_dT3_dVb_dX = deriv_exp_1_d0*(((-MJSWG)*evaluator_lln_0.getDerivs(arg,d_arg_dX)+(-d_MJSWG_dX)*evaluator_lln_0.getValues(arg)));
dT3_dVb = value_exp_1;
}
}
d_T3_dX = (((1.0-MJSWG)*((1.0-(arg*dT3_dVb))*d_PhiBSWG_dX+(-(arg*d_dT3_dVb_dX+d_arg_dX*dT3_dVb))*PhiBSWG)-((1.0-(arg*dT3_dVb))*PhiBSWG)*(-d_MJSWG_dX))/(1.0-MJSWG)/(1.0-MJSWG));
T3 = (((1.0-(arg*dT3_dVb))*PhiBSWG)/(1.0-MJSWG));
if ((vdbd>DioMax))
{
d_T3_dX = (d_T3_dX+(dT3_dVb*(-d_DioMax_dX)+d_dT3_dVb_dX*(vdbd-DioMax)));
T3 = (T3+(dT3_dVb*(vdbd-DioMax)));
}
d_qjd_dX = ((cjdbs*d_T3_dX+d_cjdbs_dX*T3)+((instanceStruct.instanceVar_B4SOItt*Ibddif)*instanceStruct.d_instanceVar_B4SOInf_dX+(instanceStruct.instanceVar_B4SOItt*d_Ibddif_dX+instanceStruct.d_instanceVar_B4SOItt_dX*Ibddif)*instanceStruct.instanceVar_B4SOInf));
qjd = ((cjdbs*T3)+((instanceStruct.instanceVar_B4SOItt*Ibddif)*instanceStruct.instanceVar_B4SOInf));
}
d_T10_dX = 0.0;
T10 = ((-instanceStruct.instanceVar_B4SOItype)*ves);
d_T11_dX = 0.0;
T11 = (instanceStruct.instanceVar_B4SOItype*(vds-ves));
if ((instanceStruct.instanceVar_B4SOIcsdmin!=0.0))
{
if ((((instanceStruct.instanceVar_pParam_B4SOInsub>0)&&(instanceStruct.instanceVar_B4SOItype>0))||((instanceStruct.instanceVar_pParam_B4SOInsub<0)&&(instanceStruct.instanceVar_B4SOItype<0))))
{
if ((T10<instanceStruct.instanceVar_pParam_B4SOIvsdfb))
{
d_B4SOIqse_dX = (instanceStruct.instanceVar_B4SOIcsbox*(d_T10_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX)+instanceStruct.d_instanceVar_B4SOIcsbox_dX*(T10-instanceStruct.instanceVar_pParam_B4SOIvsdfb));
B4SOIqse = (instanceStruct.instanceVar_B4SOIcsbox*(T10-instanceStruct.instanceVar_pParam_B4SOIvsdfb));
}
else
{
if ((T10<instanceStruct.instanceVar_pParam_B4SOIsdt1))
{
d_T0_dX = (d_T10_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX);
T0 = (T10-instanceStruct.instanceVar_pParam_B4SOIvsdfb);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqse_dX = (T0*(instanceStruct.d_instanceVar_B4SOIcsbox_dX-((instanceStruct.instanceVar_pParam_B4SOIst2/3)*d_T1_dX+(instanceStruct.d_instanceVar_pParam_B4SOIst2_dX/3)*T1))+d_T0_dX*(instanceStruct.instanceVar_B4SOIcsbox-((instanceStruct.instanceVar_pParam_B4SOIst2/3)*T1)));
B4SOIqse = (T0*(instanceStruct.instanceVar_B4SOIcsbox-((instanceStruct.instanceVar_pParam_B4SOIst2/3)*T1)));
}
else
{
if ((T10<instanceStruct.instanceVar_pParam_B4SOIvsdth))
{
d_T0_dX = (d_T10_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX);
T0 = (T10-instanceStruct.instanceVar_pParam_B4SOIvsdth);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqse_dX = (((instanceStruct.instanceVar_B4SOIcsmin*d_T10_dX+instanceStruct.d_instanceVar_B4SOIcsmin_dX*T10)+instanceStruct.d_instanceVar_B4SOIst4_dX)+(((instanceStruct.instanceVar_pParam_B4SOIst3/3)*T0)*d_T1_dX+((instanceStruct.instanceVar_pParam_B4SOIst3/3)*d_T0_dX+(instanceStruct.d_instanceVar_pParam_B4SOIst3_dX/3)*T0)*T1));
B4SOIqse = (((instanceStruct.instanceVar_B4SOIcsmin*T10)+instanceStruct.instanceVar_B4SOIst4)+(((instanceStruct.instanceVar_pParam_B4SOIst3/3)*T0)*T1));
}
else
{
d_B4SOIqse_dX = ((instanceStruct.instanceVar_B4SOIcsmin*d_T10_dX+instanceStruct.d_instanceVar_B4SOIcsmin_dX*T10)+instanceStruct.d_instanceVar_B4SOIst4_dX);
B4SOIqse = ((instanceStruct.instanceVar_B4SOIcsmin*T10)+instanceStruct.instanceVar_B4SOIst4);
}
}
}
}
else
{
if ((T10<instanceStruct.instanceVar_pParam_B4SOIvsdth))
{
d_B4SOIqse_dX = (instanceStruct.instanceVar_B4SOIcsmin*(d_T10_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX)+instanceStruct.d_instanceVar_B4SOIcsmin_dX*(T10-instanceStruct.instanceVar_pParam_B4SOIvsdth));
B4SOIqse = (instanceStruct.instanceVar_B4SOIcsmin*(T10-instanceStruct.instanceVar_pParam_B4SOIvsdth));
}
else
{
if ((T10<instanceStruct.instanceVar_pParam_B4SOIsdt1))
{
d_T0_dX = (d_T10_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX);
T0 = (T10-instanceStruct.instanceVar_pParam_B4SOIvsdth);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqse_dX = (T0*(instanceStruct.d_instanceVar_B4SOIcsmin_dX-((instanceStruct.instanceVar_pParam_B4SOIst2/3)*d_T1_dX+(instanceStruct.d_instanceVar_pParam_B4SOIst2_dX/3)*T1))+d_T0_dX*(instanceStruct.instanceVar_B4SOIcsmin-((instanceStruct.instanceVar_pParam_B4SOIst2/3)*T1)));
B4SOIqse = (T0*(instanceStruct.instanceVar_B4SOIcsmin-((instanceStruct.instanceVar_pParam_B4SOIst2/3)*T1)));
}
else
{
if ((T10<instanceStruct.instanceVar_pParam_B4SOIvsdfb))
{
d_T0_dX = (d_T10_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX);
T0 = (T10-instanceStruct.instanceVar_pParam_B4SOIvsdfb);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqse_dX = (((instanceStruct.instanceVar_B4SOIcsbox*d_T10_dX+instanceStruct.d_instanceVar_B4SOIcsbox_dX*T10)+instanceStruct.d_instanceVar_B4SOIst4_dX)+(((instanceStruct.instanceVar_pParam_B4SOIst3/3)*T0)*d_T1_dX+((instanceStruct.instanceVar_pParam_B4SOIst3/3)*d_T0_dX+(instanceStruct.d_instanceVar_pParam_B4SOIst3_dX/3)*T0)*T1));
B4SOIqse = (((instanceStruct.instanceVar_B4SOIcsbox*T10)+instanceStruct.instanceVar_B4SOIst4)+(((instanceStruct.instanceVar_pParam_B4SOIst3/3)*T0)*T1));
}
else
{
d_B4SOIqse_dX = ((instanceStruct.instanceVar_B4SOIcsbox*d_T10_dX+instanceStruct.d_instanceVar_B4SOIcsbox_dX*T10)+instanceStruct.d_instanceVar_B4SOIst4_dX);
B4SOIqse = ((instanceStruct.instanceVar_B4SOIcsbox*T10)+instanceStruct.instanceVar_B4SOIst4);
}
}
}
}
if ((((instanceStruct.instanceVar_pParam_B4SOInsub>0)&&(instanceStruct.instanceVar_B4SOItype>0))||((instanceStruct.instanceVar_pParam_B4SOInsub<0)&&(instanceStruct.instanceVar_B4SOItype<0))))
{
if ((T11<instanceStruct.instanceVar_pParam_B4SOIvsdfb))
{
d_B4SOIqde_dX = (instanceStruct.instanceVar_B4SOIcdbox*(d_T11_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX)+instanceStruct.d_instanceVar_B4SOIcdbox_dX*(T11-instanceStruct.instanceVar_pParam_B4SOIvsdfb));
B4SOIqde = (instanceStruct.instanceVar_B4SOIcdbox*(T11-instanceStruct.instanceVar_pParam_B4SOIvsdfb));
}
else
{
if ((T11<instanceStruct.instanceVar_pParam_B4SOIsdt1))
{
d_T0_dX = (d_T11_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX);
T0 = (T11-instanceStruct.instanceVar_pParam_B4SOIvsdfb);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqde_dX = (T0*(instanceStruct.d_instanceVar_B4SOIcdbox_dX-((instanceStruct.instanceVar_pParam_B4SOIdt2/3)*d_T1_dX+(instanceStruct.d_instanceVar_pParam_B4SOIdt2_dX/3)*T1))+d_T0_dX*(instanceStruct.instanceVar_B4SOIcdbox-((instanceStruct.instanceVar_pParam_B4SOIdt2/3)*T1)));
B4SOIqde = (T0*(instanceStruct.instanceVar_B4SOIcdbox-((instanceStruct.instanceVar_pParam_B4SOIdt2/3)*T1)));
}
else
{
if ((T11<instanceStruct.instanceVar_pParam_B4SOIvsdth))
{
d_T0_dX = (d_T11_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX);
T0 = (T11-instanceStruct.instanceVar_pParam_B4SOIvsdth);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqde_dX = (((instanceStruct.instanceVar_B4SOIcdmin*d_T11_dX+instanceStruct.d_instanceVar_B4SOIcdmin_dX*T11)+instanceStruct.d_instanceVar_B4SOIdt4_dX)+(((instanceStruct.instanceVar_pParam_B4SOIdt3/3)*T0)*d_T1_dX+((instanceStruct.instanceVar_pParam_B4SOIdt3/3)*d_T0_dX+(instanceStruct.d_instanceVar_pParam_B4SOIdt3_dX/3)*T0)*T1));
B4SOIqde = (((instanceStruct.instanceVar_B4SOIcdmin*T11)+instanceStruct.instanceVar_B4SOIdt4)+(((instanceStruct.instanceVar_pParam_B4SOIdt3/3)*T0)*T1));
}
else
{
d_B4SOIqde_dX = ((instanceStruct.instanceVar_B4SOIcdmin*d_T11_dX+instanceStruct.d_instanceVar_B4SOIcdmin_dX*T11)+instanceStruct.d_instanceVar_B4SOIdt4_dX);
B4SOIqde = ((instanceStruct.instanceVar_B4SOIcdmin*T11)+instanceStruct.instanceVar_B4SOIdt4);
}
}
}
}
else
{
if ((T11<instanceStruct.instanceVar_pParam_B4SOIvsdth))
{
d_B4SOIqde_dX = (instanceStruct.instanceVar_B4SOIcdmin*(d_T11_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX)+instanceStruct.d_instanceVar_B4SOIcdmin_dX*(T11-instanceStruct.instanceVar_pParam_B4SOIvsdth));
B4SOIqde = (instanceStruct.instanceVar_B4SOIcdmin*(T11-instanceStruct.instanceVar_pParam_B4SOIvsdth));
}
else
{
if ((T11<instanceStruct.instanceVar_pParam_B4SOIsdt1))
{
d_T0_dX = (d_T11_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX);
T0 = (T11-instanceStruct.instanceVar_pParam_B4SOIvsdth);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqde_dX = (T0*(instanceStruct.d_instanceVar_B4SOIcdmin_dX-((instanceStruct.instanceVar_pParam_B4SOIdt2/3)*d_T1_dX+(instanceStruct.d_instanceVar_pParam_B4SOIdt2_dX/3)*T1))+d_T0_dX*(instanceStruct.instanceVar_B4SOIcdmin-((instanceStruct.instanceVar_pParam_B4SOIdt2/3)*T1)));
B4SOIqde = (T0*(instanceStruct.instanceVar_B4SOIcdmin-((instanceStruct.instanceVar_pParam_B4SOIdt2/3)*T1)));
}
else
{
if ((T11<instanceStruct.instanceVar_pParam_B4SOIvsdfb))
{
d_T0_dX = (d_T11_dX-instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX);
T0 = (T11-instanceStruct.instanceVar_pParam_B4SOIvsdfb);
d_T1_dX = (T0*d_T0_dX+d_T0_dX*T0);
T1 = (T0*T0);
d_B4SOIqde_dX = (((instanceStruct.instanceVar_B4SOIcdbox*d_T11_dX+instanceStruct.d_instanceVar_B4SOIcdbox_dX*T11)+instanceStruct.d_instanceVar_B4SOIdt4_dX)+(((instanceStruct.instanceVar_pParam_B4SOIdt3/3)*T0)*d_T1_dX+((instanceStruct.instanceVar_pParam_B4SOIdt3/3)*d_T0_dX+(instanceStruct.d_instanceVar_pParam_B4SOIdt3_dX/3)*T0)*T1));
B4SOIqde = (((instanceStruct.instanceVar_B4SOIcdbox*T11)+instanceStruct.instanceVar_B4SOIdt4)+(((instanceStruct.instanceVar_pParam_B4SOIdt3/3)*T0)*T1));
}
else
{
d_B4SOIqde_dX = ((instanceStruct.instanceVar_B4SOIcdbox*d_T11_dX+instanceStruct.d_instanceVar_B4SOIcdbox_dX*T11)+instanceStruct.d_instanceVar_B4SOIdt4_dX);
B4SOIqde = ((instanceStruct.instanceVar_B4SOIcdbox*T11)+instanceStruct.instanceVar_B4SOIdt4);
}
}
}
}
}
else
{
d_B4SOIqse_dX = (instanceStruct.instanceVar_B4SOIcsbox*d_T10_dX+instanceStruct.d_instanceVar_B4SOIcsbox_dX*T10);
B4SOIqse = (instanceStruct.instanceVar_B4SOIcsbox*T10);
d_B4SOIqde_dX = (instanceStruct.instanceVar_B4SOIcdbox*d_T11_dX+instanceStruct.d_instanceVar_B4SOIcdbox_dX*T11);
B4SOIqde = (instanceStruct.instanceVar_B4SOIcdbox*T11);
}
d_B4SOIqse_dX = (d_B4SOIqse_dX+(instanceStruct.instanceVar_B4SOIcsesw*d_T10_dX+instanceStruct.d_instanceVar_B4SOIcsesw_dX*T10));
B4SOIqse = (B4SOIqse+(instanceStruct.instanceVar_B4SOIcsesw*T10));
d_B4SOIqde_dX = (d_B4SOIqde_dX+(instanceStruct.instanceVar_B4SOIcdesw*d_T11_dX+instanceStruct.d_instanceVar_B4SOIcdesw_dX*T11));
B4SOIqde = (B4SOIqde+(instanceStruct.instanceVar_B4SOIcdesw*T11));
if ((instanceStruct.instanceVar_B4SOIrgateMod==3))
{
d_T0_dX = 0.0;
T0 = (vgmd+0.02);
}
else
{
d_T0_dX = 0.0;
T0 = (vgd+0.02);
}
{
double value_sqrt_0 = sqrt(((T0*T0)+(4.0*0.02)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
T1 = value_sqrt_0;
}
d_T2_dX = 0.5*(d_T0_dX-d_T1_dX);
T2 = (0.5*(T0-T1));
d_T3_dX = (instanceStruct.instanceVar_pParam_B4SOIwdiodCV*instanceStruct.d_instanceVar_pParam_B4SOIcgdl_dX+instanceStruct.d_instanceVar_pParam_B4SOIwdiodCV_dX*instanceStruct.instanceVar_pParam_B4SOIcgdl);
T3 = (instanceStruct.instanceVar_pParam_B4SOIwdiodCV*instanceStruct.instanceVar_pParam_B4SOIcgdl);
{
double value_sqrt_0 = sqrt((1.0-((4.0*T2)/instanceStruct.instanceVar_pParam_B4SOIckappa)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T4_dX = deriv_sqrt_0_d0*((-((instanceStruct.instanceVar_pParam_B4SOIckappa*4.0*d_T2_dX-(4.0*T2)*instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX)/instanceStruct.instanceVar_pParam_B4SOIckappa/instanceStruct.instanceVar_pParam_B4SOIckappa)));
T4 = value_sqrt_0;
}
if ((instanceStruct.instanceVar_B4SOIrgateMod==3))
{
d_qgdo_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIcgdo_dX+d_T3_dX)*vgmd-(T3*(d_T2_dX+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*d_T4_dX+0.5*instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX*(T4-1.0)))+d_T3_dX*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
qgdo = (((instanceStruct.instanceVar_pParam_B4SOIcgdo+T3)*vgmd)-(T3*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
}
else
{
d_qgdo_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIcgdo_dX+d_T3_dX)*vgd-(T3*(d_T2_dX+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*d_T4_dX+0.5*instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX*(T4-1.0)))+d_T3_dX*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
qgdo = (((instanceStruct.instanceVar_pParam_B4SOIcgdo+T3)*vgd)-(T3*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
}
if ((instanceStruct.instanceVar_B4SOIrgateMod==3))
{
d_T0_dX = 0.0;
T0 = (vgms+0.02);
}
else
{
d_T0_dX = 0.0;
T0 = (vgs+0.02);
}
{
double value_sqrt_0 = sqrt(((T0*T0)+(4.0*0.02)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T1_dX = deriv_sqrt_0_d0*((T0*d_T0_dX+d_T0_dX*T0));
T1 = value_sqrt_0;
}
d_T2_dX = 0.5*(d_T0_dX-d_T1_dX);
T2 = (0.5*(T0-T1));
d_T3_dX = (instanceStruct.instanceVar_pParam_B4SOIwdiosCV*instanceStruct.d_instanceVar_pParam_B4SOIcgsl_dX+instanceStruct.d_instanceVar_pParam_B4SOIwdiosCV_dX*instanceStruct.instanceVar_pParam_B4SOIcgsl);
T3 = (instanceStruct.instanceVar_pParam_B4SOIwdiosCV*instanceStruct.instanceVar_pParam_B4SOIcgsl);
{
double value_sqrt_0 = sqrt((1.0-((4.0*T2)/instanceStruct.instanceVar_pParam_B4SOIckappa)));
double  deriv_sqrt_0_d0 = (0.5/value_sqrt_0);
d_T4_dX = deriv_sqrt_0_d0*((-((instanceStruct.instanceVar_pParam_B4SOIckappa*4.0*d_T2_dX-(4.0*T2)*instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX)/instanceStruct.instanceVar_pParam_B4SOIckappa/instanceStruct.instanceVar_pParam_B4SOIckappa)));
T4 = value_sqrt_0;
}
if ((instanceStruct.instanceVar_B4SOIrgateMod==3))
{
d_qgso_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIcgso_dX+d_T3_dX)*vgms-(T3*(d_T2_dX+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*d_T4_dX+0.5*instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX*(T4-1.0)))+d_T3_dX*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
qgso = (((instanceStruct.instanceVar_pParam_B4SOIcgso+T3)*vgms)-(T3*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
}
else
{
d_qgso_dX = ((instanceStruct.d_instanceVar_pParam_B4SOIcgso_dX+d_T3_dX)*vgs-(T3*(d_T2_dX+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*d_T4_dX+0.5*instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX*(T4-1.0)))+d_T3_dX*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
qgso = (((instanceStruct.instanceVar_pParam_B4SOIcgso+T3)*vgs)-(T3*(T2+((0.5*instanceStruct.instanceVar_pParam_B4SOIckappa)*(T4-1.0)))));
}
if ((instanceStruct.instanceVar_B4SOInf!=1.0))
{
d_qgdo_dX = (qgdo*instanceStruct.d_instanceVar_B4SOInf_dX+d_qgdo_dX*instanceStruct.instanceVar_B4SOInf);
qgdo = (qgdo*instanceStruct.instanceVar_B4SOInf);
d_qgso_dX = (qgso*instanceStruct.d_instanceVar_B4SOInf_dX+d_qgso_dX*instanceStruct.instanceVar_B4SOInf);
qgso = (qgso*instanceStruct.instanceVar_B4SOInf);
}
d_qgi_dX = d_qgate_dX;
qgi = qgate;
d_qov_dX = (d_qgso_dX+d_qgdo_dX);
qov = (qgso+qgdo);
d_qgate_dX = (d_qgi_dX+d_qov_dX);
qgate = (qgi+qov);
//Begin block noise
{
//Block-local variables for block noise
double cd=0.0;
double d_cd_dX=0.0;
double cdmin=0.0;
double d_cdmin_dX=0.0;
double logcdmin=0.0;
double d_logcdmin_dX=0.0;
double wefftot=0.0;
double d_wefftot_dX=0.0;
double leffBcox=0.0;
double d_leffBcox_dX=0.0;
//End of Block-local variables
//Block-local variables for block noise
double thermalNoiseContrib=0.0;
double d_thermalNoiseContrib_dX=0.0;
double flickerNoiseContrib=0.0;
double d_flickerNoiseContrib_dX=0.0;
//End of Block-local variables
//Block-local variables for block noise
double esat=0.0;
double d_esat_dX=0.0;
double npart_beta=0.0;
double d_npart_beta_dX=0.0;
double npart_theta=0.0;
double d_npart_theta_dX=0.0;
double igsquare=0.0;
double d_igsquare_dX=0.0;
//End of Block-local variables
if ((instanceStruct.instanceVar_B4SOImode>0))
{
{
double value_fabs_0 = fabs(((((Ids+Ic)-Ibd)+Iii)+Igidl));
double  deriv_fabs_0_d0 = (((((((Ids+Ic)-Ibd)+Iii)+Igidl))>=0)?(+1.0):(-1.0));
d_cd_dX = deriv_fabs_0_d0*(((((d_Ids_dX+d_Ic_dX)-d_Ibd_dX)+d_Iii_dX)+d_Igidl_dX));
cd = value_fabs_0;
}
}
else
{
{
double value_fabs_0 = fabs(((((Ids-Ic)-Ibs)+Iii)+Igidl));
double  deriv_fabs_0_d0 = (((((((Ids-Ic)-Ibs)+Iii)+Igidl))>=0)?(+1.0):(-1.0));
d_cd_dX = deriv_fabs_0_d0*(((((d_Ids_dX-d_Ic_dX)-d_Ibs_dX)+d_Iii_dX)+d_Igidl_dX));
cd = value_fabs_0;
}
}
instanceStruct.d_instanceVar_fourkt_dX = (4.0*1.3806503e-23)*instanceStruct.d_instanceVar_B4SOITempSH_dX;
instanceStruct.instanceVar_fourkt = ((4.0*1.3806503e-23)*instanceStruct.instanceVar_B4SOITempSH);
if ((instanceStruct.instanceVar_B4SOIrdsMod!=2))
{
instanceStruct.d_instanceVar_gdnoise_dX = (-1*d_Rd_dX/Rd/Rd);
instanceStruct.instanceVar_gdnoise = (1/Rd);
instanceStruct.d_instanceVar_gsnoise_dX = (-1*d_Rs_dX/Rs/Rs);
instanceStruct.instanceVar_gsnoise = (1/Rs);
}
else
{
instanceStruct.d_instanceVar_gdnoise_dX = 0.0;
instanceStruct.instanceVar_gdnoise = 0.0;
instanceStruct.d_instanceVar_gsnoise_dX = 0.0;
instanceStruct.instanceVar_gsnoise = 0.0;
}
if ( modelStruct.modelPar_TNOIMOD == (0))
{
{
double value_fabs_0 = fabs(instanceStruct.instanceVar_B4SOIqinv);
double value_fabs_1 = fabs((instanceStruct.instanceVar_B4SOIqinv/((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIleff)+((instanceStruct.instanceVar_B4SOIueff*value_fabs_0)*instanceStruct.instanceVar_B4SOIrds))));
double  deriv_fabs_0_d0 = (((instanceStruct.instanceVar_B4SOIqinv)>=0)?(+1.0):(-1.0));
double  deriv_fabs_1_d0 = ((((instanceStruct.instanceVar_B4SOIqinv/((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIleff)+((instanceStruct.instanceVar_B4SOIueff*value_fabs_0)*instanceStruct.instanceVar_B4SOIrds))))>=0)?(+1.0):(-1.0));
d_thermalNoiseContrib_dX = ((instanceStruct.instanceVar_B4SOIntnoi*instanceStruct.instanceVar_B4SOIueff)*deriv_fabs_1_d0*(((((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIleff)+((instanceStruct.instanceVar_B4SOIueff*value_fabs_0)*instanceStruct.instanceVar_B4SOIrds))*instanceStruct.d_instanceVar_B4SOIqinv_dX-instanceStruct.instanceVar_B4SOIqinv*((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+instanceStruct.d_instanceVar_pParam_B4SOIleff_dX*instanceStruct.instanceVar_pParam_B4SOIleff)+((instanceStruct.instanceVar_B4SOIueff*value_fabs_0)*instanceStruct.d_instanceVar_B4SOIrds_dX+(instanceStruct.instanceVar_B4SOIueff*deriv_fabs_0_d0*(instanceStruct.d_instanceVar_B4SOIqinv_dX)+instanceStruct.d_instanceVar_B4SOIueff_dX*value_fabs_0)*instanceStruct.instanceVar_B4SOIrds)))/((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIleff)+((instanceStruct.instanceVar_B4SOIueff*value_fabs_0)*instanceStruct.instanceVar_B4SOIrds))/((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIleff)+((instanceStruct.instanceVar_B4SOIueff*value_fabs_0)*instanceStruct.instanceVar_B4SOIrds))))+(instanceStruct.instanceVar_B4SOIntnoi*instanceStruct.d_instanceVar_B4SOIueff_dX+instanceStruct.d_instanceVar_B4SOIntnoi_dX*instanceStruct.instanceVar_B4SOIueff)*value_fabs_1);
thermalNoiseContrib = ((instanceStruct.instanceVar_B4SOIntnoi*instanceStruct.instanceVar_B4SOIueff)*value_fabs_1);
}
// I(di,si) <+ (white_noise((m*abs((fourkt*thermalNoiseContrib))),"id"))
}
else
if ( modelStruct.modelPar_TNOIMOD == (1))
{
d_T0_dX = ((instanceStruct.d_instanceVar_B4SOIgm_dX+instanceStruct.d_instanceVar_B4SOIgds_dX)+instanceStruct.d_instanceVar_B4SOIgmbs_dX);
T0 = ((instanceStruct.instanceVar_B4SOIgm+instanceStruct.instanceVar_B4SOIgds)+instanceStruct.instanceVar_B4SOIgmbs);
d_T0_dX = (T0*d_T0_dX+d_T0_dX*T0);
T0 = (T0*T0);
d_esat_dX = ((instanceStruct.instanceVar_B4SOIueff*2.0*d_vsattemp_dX-(2.0*vsattemp)*instanceStruct.d_instanceVar_B4SOIueff_dX)/instanceStruct.instanceVar_B4SOIueff/instanceStruct.instanceVar_B4SOIueff);
esat = ((2.0*vsattemp)/instanceStruct.instanceVar_B4SOIueff);
d_T5_dX = (((esat*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_B4SOIVgsteff_dX-instanceStruct.instanceVar_B4SOIVgsteff*(esat*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+d_esat_dX*instanceStruct.instanceVar_pParam_B4SOIleff))/(esat*instanceStruct.instanceVar_pParam_B4SOIleff)/(esat*instanceStruct.instanceVar_pParam_B4SOIleff));
T5 = (instanceStruct.instanceVar_B4SOIVgsteff/(esat*instanceStruct.instanceVar_pParam_B4SOIleff));
d_T5_dX = (T5*d_T5_dX+d_T5_dX*T5);
T5 = (T5*T5);
d_npart_beta_dX = (instanceStruct.instanceVar_B4SOIrnoia*((T5*instanceStruct.instanceVar_B4SOItnoia)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(T5*instanceStruct.d_instanceVar_B4SOItnoia_dX+d_T5_dX*instanceStruct.instanceVar_B4SOItnoia)*instanceStruct.instanceVar_pParam_B4SOIleff)+instanceStruct.d_instanceVar_B4SOIrnoia_dX*(1.0+((T5*instanceStruct.instanceVar_B4SOItnoia)*instanceStruct.instanceVar_pParam_B4SOIleff)));
npart_beta = (instanceStruct.instanceVar_B4SOIrnoia*(1.0+((T5*instanceStruct.instanceVar_B4SOItnoia)*instanceStruct.instanceVar_pParam_B4SOIleff)));
d_npart_theta_dX = (instanceStruct.instanceVar_B4SOIrnoib*((T5*instanceStruct.instanceVar_B4SOItnoib)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(T5*instanceStruct.d_instanceVar_B4SOItnoib_dX+d_T5_dX*instanceStruct.instanceVar_B4SOItnoib)*instanceStruct.instanceVar_pParam_B4SOIleff)+instanceStruct.d_instanceVar_B4SOIrnoib_dX*(1.0+((T5*instanceStruct.instanceVar_B4SOItnoib)*instanceStruct.instanceVar_pParam_B4SOIleff)));
npart_theta = (instanceStruct.instanceVar_B4SOIrnoib*(1.0+((T5*instanceStruct.instanceVar_B4SOItnoib)*instanceStruct.instanceVar_pParam_B4SOIleff)));
if ((npart_theta>0.9))
{
d_npart_theta_dX = 0.0;
npart_theta = 0.9;
}
if ((npart_theta>(0.9*npart_beta)))
{
d_npart_theta_dX = 0.9*d_npart_beta_dX;
npart_theta = (0.9*npart_beta);
}
d_igsquare_dX = ((instanceStruct.instanceVar_B4SOIidovVds*((npart_theta*npart_theta)*d_T0_dX+(npart_theta*d_npart_theta_dX+d_npart_theta_dX*npart_theta)*T0)-((npart_theta*npart_theta)*T0)*instanceStruct.d_instanceVar_B4SOIidovVds_dX)/instanceStruct.instanceVar_B4SOIidovVds/instanceStruct.instanceVar_B4SOIidovVds);
igsquare = (((npart_theta*npart_theta)*T0)/instanceStruct.instanceVar_B4SOIidovVds);
d_T1_dX = ((npart_beta*(instanceStruct.d_instanceVar_B4SOIgm_dX+instanceStruct.d_instanceVar_B4SOIgmbs_dX)+d_npart_beta_dX*(instanceStruct.instanceVar_B4SOIgm+instanceStruct.instanceVar_B4SOIgmbs))+instanceStruct.d_instanceVar_B4SOIgds_dX);
T1 = ((npart_beta*(instanceStruct.instanceVar_B4SOIgm+instanceStruct.instanceVar_B4SOIgmbs))+instanceStruct.instanceVar_B4SOIgds);
d_T2_dX = ((instanceStruct.instanceVar_B4SOIidovVds*(T1*d_T1_dX+d_T1_dX*T1)-(T1*T1)*instanceStruct.d_instanceVar_B4SOIidovVds_dX)/instanceStruct.instanceVar_B4SOIidovVds/instanceStruct.instanceVar_B4SOIidovVds);
T2 = ((T1*T1)/instanceStruct.instanceVar_B4SOIidovVds);
d_thermalNoiseContrib_dX = (d_T2_dX-d_igsquare_dX);
thermalNoiseContrib = (T2-igsquare);
if ((instanceStruct.instanceVar_B4SOImode>0))
{
instanceStruct.d_instanceVar_gsnoise_dX = (instanceStruct.instanceVar_gsnoise*((instanceStruct.instanceVar_B4SOIidovVds*((npart_theta*npart_theta)*instanceStruct.d_instanceVar_gsnoise_dX+(npart_theta*d_npart_theta_dX+d_npart_theta_dX*npart_theta)*instanceStruct.instanceVar_gsnoise)-((npart_theta*npart_theta)*instanceStruct.instanceVar_gsnoise)*instanceStruct.d_instanceVar_B4SOIidovVds_dX)/instanceStruct.instanceVar_B4SOIidovVds/instanceStruct.instanceVar_B4SOIidovVds)+instanceStruct.d_instanceVar_gsnoise_dX*(1.0+(((npart_theta*npart_theta)*instanceStruct.instanceVar_gsnoise)/instanceStruct.instanceVar_B4SOIidovVds)));
instanceStruct.instanceVar_gsnoise = (instanceStruct.instanceVar_gsnoise*(1.0+(((npart_theta*npart_theta)*instanceStruct.instanceVar_gsnoise)/instanceStruct.instanceVar_B4SOIidovVds)));
}
else
{
instanceStruct.d_instanceVar_gdnoise_dX = (instanceStruct.instanceVar_gdnoise*((instanceStruct.instanceVar_B4SOIidovVds*((npart_theta*npart_theta)*instanceStruct.d_instanceVar_gdnoise_dX+(npart_theta*d_npart_theta_dX+d_npart_theta_dX*npart_theta)*instanceStruct.instanceVar_gdnoise)-((npart_theta*npart_theta)*instanceStruct.instanceVar_gdnoise)*instanceStruct.d_instanceVar_B4SOIidovVds_dX)/instanceStruct.instanceVar_B4SOIidovVds/instanceStruct.instanceVar_B4SOIidovVds)+instanceStruct.d_instanceVar_gdnoise_dX*(1.0+(((npart_theta*npart_theta)*instanceStruct.instanceVar_gdnoise)/instanceStruct.instanceVar_B4SOIidovVds)));
instanceStruct.instanceVar_gdnoise = (instanceStruct.instanceVar_gdnoise*(1.0+(((npart_theta*npart_theta)*instanceStruct.instanceVar_gdnoise)/instanceStruct.instanceVar_B4SOIidovVds)));
}
// I(di,si) <+ (white_noise((m*abs((fourkt*thermalNoiseContrib))),"id"))
}
else
if ( modelStruct.modelPar_TNOIMOD == (3))
{
UserInfo(theInstance)  << "[BSIMSOI] Although the model selector TNOIMOD is set to 3, the new correlated thermal noise model is not activated in the Verilog-A code.  Please uncomment \"`define _TNOIMOD3_\" in the beginning of the Verilog-A code." << " " <<  std::endl; 
}
else
if ( modelStruct.modelPar_TNOIMOD == (2))
{
{
double value_fabs_0 = fabs(((instanceStruct.instanceVar_B4SOIgm+instanceStruct.instanceVar_B4SOIgds)+instanceStruct.instanceVar_B4SOIgmbs));
double  deriv_fabs_0_d0 = (((((instanceStruct.instanceVar_B4SOIgm+instanceStruct.instanceVar_B4SOIgds)+instanceStruct.instanceVar_B4SOIgmbs))>=0)?(+1.0):(-1.0));
d_thermalNoiseContrib_dX = (((2.0/3.0)*instanceStruct.instanceVar_B4SOIntnoi)*deriv_fabs_0_d0*(((instanceStruct.d_instanceVar_B4SOIgm_dX+instanceStruct.d_instanceVar_B4SOIgds_dX)+instanceStruct.d_instanceVar_B4SOIgmbs_dX))+(2.0/3.0)*instanceStruct.d_instanceVar_B4SOIntnoi_dX*value_fabs_0);
thermalNoiseContrib = (((2.0/3.0)*instanceStruct.instanceVar_B4SOIntnoi)*value_fabs_0);
}
// I(di,si) <+ (white_noise((m*abs((fourkt*thermalNoiseContrib))),"id"))
}
else
{
 // no default
}
d_wefftot_dX = (instanceStruct.instanceVar_B4SOInf*instanceStruct.d_instanceVar_pParam_B4SOIweff_dX+instanceStruct.d_instanceVar_B4SOInf_dX*instanceStruct.instanceVar_pParam_B4SOIweff);
wefftot = (instanceStruct.instanceVar_B4SOInf*instanceStruct.instanceVar_pParam_B4SOIweff);
if ((instanceStruct.instanceVar_B4SOIbf==1))
{
d_leffBcox_dX = (instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_B4SOIcox_dX+instanceStruct.d_instanceVar_pParam_B4SOIleff_dX*instanceStruct.instanceVar_B4SOIcox);
leffBcox = (instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_B4SOIcox);
}
else
{
if ((instanceStruct.instanceVar_B4SOIbf==2))
{
d_leffBcox_dX = ((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_B4SOIcox_dX+(instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+instanceStruct.d_instanceVar_pParam_B4SOIleff_dX*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_B4SOIcox);
leffBcox = ((instanceStruct.instanceVar_pParam_B4SOIleff*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_B4SOIcox);
}
else
{
{
double value_pow_0 = pow(instanceStruct.instanceVar_pParam_B4SOIleff,instanceStruct.instanceVar_B4SOIbf);
double  deriv_pow_0_d0 = ((instanceStruct.instanceVar_pParam_B4SOIleff == 0.0)?0.0:(value_pow_0*instanceStruct.instanceVar_B4SOIbf/instanceStruct.instanceVar_pParam_B4SOIleff));
double  deriv_pow_0_d1 = (instanceStruct.instanceVar_pParam_B4SOIleff == 0.0)?0.0:(log(instanceStruct.instanceVar_pParam_B4SOIleff)*value_pow_0);
d_leffBcox_dX = (value_pow_0*instanceStruct.d_instanceVar_B4SOIcox_dX+((deriv_pow_0_d0*(instanceStruct.d_instanceVar_pParam_B4SOIleff_dX))+(deriv_pow_0_d1*(instanceStruct.d_instanceVar_B4SOIbf_dX)))*instanceStruct.instanceVar_B4SOIcox);
leffBcox = (value_pow_0*instanceStruct.instanceVar_B4SOIcox);
}
}
}
if ((modelStruct.modelPar_FNOIMOD==0))
{
if ((instanceStruct.instanceVar_B4SOIw0flk>0))
{
d_cdmin_dX = ((cd/wefftot)*instanceStruct.d_instanceVar_B4SOIw0flk_dX+((wefftot*d_cd_dX-cd*d_wefftot_dX)/wefftot/wefftot)*instanceStruct.instanceVar_B4SOIw0flk);
cdmin = ((cd/wefftot)*instanceStruct.instanceVar_B4SOIw0flk);
if ((cdmin<1.0e-38))
{
d_cdmin_dX = 0.0;
cdmin = 1.0e-38;
}
{
double value_log_0 = log(cdmin);
double  deriv_log_0_d0 = (1.0/cdmin);
d_logcdmin_dX = deriv_log_0_d0*(d_cdmin_dX);
logcdmin = value_log_0;
}
{
double value_exp_0 = exp((instanceStruct.instanceVar_B4SOIaf*logcdmin));
double  deriv_exp_0_d0 = value_exp_0;
d_flickerNoiseContrib_dX = ((leffBcox*(((wefftot/instanceStruct.instanceVar_B4SOIw0flk)*instanceStruct.instanceVar_B4SOIkf)*deriv_exp_0_d0*((instanceStruct.instanceVar_B4SOIaf*d_logcdmin_dX+instanceStruct.d_instanceVar_B4SOIaf_dX*logcdmin))+((wefftot/instanceStruct.instanceVar_B4SOIw0flk)*instanceStruct.d_instanceVar_B4SOIkf_dX+((instanceStruct.instanceVar_B4SOIw0flk*d_wefftot_dX-wefftot*instanceStruct.d_instanceVar_B4SOIw0flk_dX)/instanceStruct.instanceVar_B4SOIw0flk/instanceStruct.instanceVar_B4SOIw0flk)*instanceStruct.instanceVar_B4SOIkf)*value_exp_0)-(((wefftot/instanceStruct.instanceVar_B4SOIw0flk)*instanceStruct.instanceVar_B4SOIkf)*value_exp_0)*d_leffBcox_dX)/leffBcox/leffBcox);
flickerNoiseContrib = ((((wefftot/instanceStruct.instanceVar_B4SOIw0flk)*instanceStruct.instanceVar_B4SOIkf)*value_exp_0)/leffBcox);
}
}
else
{
if ((cd<1.0e-38))
{
d_cdmin_dX = 0.0;
cdmin = 1.0e-38;
}
else
{
d_cdmin_dX = d_cd_dX;
cdmin = cd;
}
{
double value_log_0 = log(cdmin);
double  deriv_log_0_d0 = (1.0/cdmin);
d_logcdmin_dX = deriv_log_0_d0*(d_cdmin_dX);
logcdmin = value_log_0;
}
{
double value_exp_0 = exp((instanceStruct.instanceVar_B4SOIaf*logcdmin));
double  deriv_exp_0_d0 = value_exp_0;
d_flickerNoiseContrib_dX = ((leffBcox*(instanceStruct.instanceVar_B4SOIkf*deriv_exp_0_d0*((instanceStruct.instanceVar_B4SOIaf*d_logcdmin_dX+instanceStruct.d_instanceVar_B4SOIaf_dX*logcdmin))+instanceStruct.d_instanceVar_B4SOIkf_dX*value_exp_0)-(instanceStruct.instanceVar_B4SOIkf*value_exp_0)*d_leffBcox_dX)/leffBcox/leffBcox);
flickerNoiseContrib = ((instanceStruct.instanceVar_B4SOIkf*value_exp_0)/leffBcox);
}
}
}
else
//Begin block Eval1ovFNoise
{
//Block-local variables for block Eval1ovFNoise
double esat=0.0;
double d_esat_dX=0.0;
double DelClm=0.0;
double d_DelClm_dX=0.0;
double N0=0.0;
double d_N0_dX=0.0;
double Nl=0.0;
double d_Nl_dX=0.0;
double Nratio=0.0;
double d_Nratio_dX=0.0;
double logem38=0.0;
//End of Block-local variables
//Block-local variables for block Eval1ovFNoise
double T0=0.0;
double d_T0_dX=0.0;
double T1=0.0;
double d_T1_dX=0.0;
double T2=0.0;
double d_T2_dX=0.0;
double T3=0.0;
double d_T3_dX=0.0;
double T4=0.0;
double d_T4_dX=0.0;
double T5=0.0;
double d_T5_dX=0.0;
double T6=0.0;
double d_T6_dX=0.0;
double T7=0.0;
double d_T7_dX=0.0;
double T8=0.0;
double d_T8_dX=0.0;
double T9=0.0;
double d_T9_dX=0.0;
double T10=0.0;
double d_T10_dX=0.0;
double T11=0.0;
double d_T11_dX=0.0;
//End of Block-local variables
//Block-local variables for block Eval1ovFNoise
double Ssi=0.0;
double d_Ssi_dX=0.0;
double Swi=0.0;
double d_Swi_dX=0.0;
//End of Block-local variables
{
double value_log_0 = log(static_cast<double>(1e-38));
logem38 = value_log_0;
}
d_esat_dX = ((instanceStruct.instanceVar_B4SOIueff*2.0*d_vsattemp_dX-(2.0*vsattemp)*instanceStruct.d_instanceVar_B4SOIueff_dX)/instanceStruct.instanceVar_B4SOIueff/instanceStruct.instanceVar_B4SOIueff);
esat = ((2.0*vsattemp)/instanceStruct.instanceVar_B4SOIueff);
if ((instanceStruct.instanceVar_B4SOIem<=0.0))
{
d_DelClm_dX = 0.0;
DelClm = 0.0;
}
else
{
d_T0_dX = ((esat*(((instanceStruct.instanceVar_pParam_B4SOIlitl*(-instanceStruct.d_instanceVar_B4SOIVdseff_dX)-(Vds-instanceStruct.instanceVar_B4SOIVdseff)*instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX)/instanceStruct.instanceVar_pParam_B4SOIlitl/instanceStruct.instanceVar_pParam_B4SOIlitl)+instanceStruct.d_instanceVar_B4SOIem_dX)-(((Vds-instanceStruct.instanceVar_B4SOIVdseff)/instanceStruct.instanceVar_pParam_B4SOIlitl)+instanceStruct.instanceVar_B4SOIem)*d_esat_dX)/esat/esat);
T0 = ((((Vds-instanceStruct.instanceVar_B4SOIVdseff)/instanceStruct.instanceVar_pParam_B4SOIlitl)+instanceStruct.instanceVar_B4SOIem)/esat);
if ((T0<1e-38))
{
d_DelClm_dX = instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX*logem38;
DelClm = (instanceStruct.instanceVar_pParam_B4SOIlitl*logem38);
}
else
{
{
double value_log_0 = log(T0);
double  deriv_log_0_d0 = (1.0/T0);
d_DelClm_dX = (instanceStruct.instanceVar_pParam_B4SOIlitl*deriv_log_0_d0*(d_T0_dX)+instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX*value_log_0);
DelClm = (instanceStruct.instanceVar_pParam_B4SOIlitl*value_log_0);
}
}
}
d_T1_dX = (((((1.60219e-19*1.60219e-19)*1.3806503e-23)*cd)*instanceStruct.instanceVar_B4SOITempSH)*instanceStruct.d_instanceVar_B4SOIueff_dX+((((1.60219e-19*1.60219e-19)*1.3806503e-23)*cd)*instanceStruct.d_instanceVar_B4SOITempSH_dX+((1.60219e-19*1.60219e-19)*1.3806503e-23)*d_cd_dX*instanceStruct.instanceVar_B4SOITempSH)*instanceStruct.instanceVar_B4SOIueff);
T1 = (((((1.60219e-19*1.60219e-19)*1.3806503e-23)*cd)*instanceStruct.instanceVar_B4SOITempSH)*instanceStruct.instanceVar_B4SOIueff);
d_T2_dX = ((((1.0e10*instanceStruct.instanceVar_B4SOIAbulk)*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+(((1.0e10*instanceStruct.instanceVar_B4SOIAbulk)*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+((1.0e10*instanceStruct.instanceVar_B4SOIAbulk)*instanceStruct.d_instanceVar_B4SOIcox_dX+1.0e10*instanceStruct.d_instanceVar_B4SOIAbulk_dX*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_pParam_B4SOIleff);
T2 = ((((1.0e10*instanceStruct.instanceVar_B4SOIAbulk)*instanceStruct.instanceVar_B4SOIcox)*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_pParam_B4SOIleff);
d_N0_dX = ((instanceStruct.instanceVar_B4SOIcox*instanceStruct.d_instanceVar_B4SOIVgsteff_dX+instanceStruct.d_instanceVar_B4SOIcox_dX*instanceStruct.instanceVar_B4SOIVgsteff)/1.60219e-19);
N0 = ((instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIVgsteff)/1.60219e-19);
d_Nl_dX = (((instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIVgsteff)*(-(instanceStruct.instanceVar_B4SOIAbovVgst2Vtm*instanceStruct.d_instanceVar_B4SOIVdseff_dX+instanceStruct.d_instanceVar_B4SOIAbovVgst2Vtm_dX*instanceStruct.instanceVar_B4SOIVdseff))+(instanceStruct.instanceVar_B4SOIcox*instanceStruct.d_instanceVar_B4SOIVgsteff_dX+instanceStruct.d_instanceVar_B4SOIcox_dX*instanceStruct.instanceVar_B4SOIVgsteff)*(1.0-(instanceStruct.instanceVar_B4SOIAbovVgst2Vtm*instanceStruct.instanceVar_B4SOIVdseff)))/1.60219e-19);
Nl = (((instanceStruct.instanceVar_B4SOIcox*instanceStruct.instanceVar_B4SOIVgsteff)*(1.0-(instanceStruct.instanceVar_B4SOIAbovVgst2Vtm*instanceStruct.instanceVar_B4SOIVdseff)))/1.60219e-19);
d_Nratio_dX = (((Nl+instanceStruct.instanceVar_B4SOInstar)*(d_N0_dX+instanceStruct.d_instanceVar_B4SOInstar_dX)-(N0+instanceStruct.instanceVar_B4SOInstar)*(d_Nl_dX+instanceStruct.d_instanceVar_B4SOInstar_dX))/(Nl+instanceStruct.instanceVar_B4SOInstar)/(Nl+instanceStruct.instanceVar_B4SOInstar));
Nratio = ((N0+instanceStruct.instanceVar_B4SOInstar)/(Nl+instanceStruct.instanceVar_B4SOInstar));
if ((Nratio<1e-38))
{
d_T3_dX = instanceStruct.d_instanceVar_B4SOIoxideTrapDensityA_dX*logem38;
T3 = (instanceStruct.instanceVar_B4SOIoxideTrapDensityA*logem38);
}
else
{
{
double value_log_0 = log(Nratio);
double  deriv_log_0_d0 = (1.0/Nratio);
d_T3_dX = (instanceStruct.instanceVar_B4SOIoxideTrapDensityA*deriv_log_0_d0*(d_Nratio_dX)+instanceStruct.d_instanceVar_B4SOIoxideTrapDensityA_dX*value_log_0);
T3 = (instanceStruct.instanceVar_B4SOIoxideTrapDensityA*value_log_0);
}
}
d_T4_dX = (instanceStruct.instanceVar_B4SOIoxideTrapDensityB*(d_N0_dX-d_Nl_dX)+instanceStruct.d_instanceVar_B4SOIoxideTrapDensityB_dX*(N0-Nl));
T4 = (instanceStruct.instanceVar_B4SOIoxideTrapDensityB*(N0-Nl));
d_T5_dX = ((instanceStruct.instanceVar_B4SOIoxideTrapDensityC*0.5)*((N0*d_N0_dX+d_N0_dX*N0)-(Nl*d_Nl_dX+d_Nl_dX*Nl))+instanceStruct.d_instanceVar_B4SOIoxideTrapDensityC_dX*0.5*((N0*N0)-(Nl*Nl)));
T5 = ((instanceStruct.instanceVar_B4SOIoxideTrapDensityC*0.5)*((N0*N0)-(Nl*Nl)));
d_T6_dX = (((1.3806503e-23*instanceStruct.instanceVar_B4SOITempSH)*cd)*d_cd_dX+((1.3806503e-23*instanceStruct.instanceVar_B4SOITempSH)*d_cd_dX+1.3806503e-23*instanceStruct.d_instanceVar_B4SOITempSH_dX*cd)*cd);
T6 = (((1.3806503e-23*instanceStruct.instanceVar_B4SOITempSH)*cd)*cd);
d_T7_dX = (((1.0e10*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_pParam_B4SOIleff)*d_wefftot_dX+((1.0e10*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+1.0e10*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX*instanceStruct.instanceVar_pParam_B4SOIleff)*wefftot);
T7 = (((1.0e10*instanceStruct.instanceVar_pParam_B4SOIleff)*instanceStruct.instanceVar_pParam_B4SOIleff)*wefftot);
d_T8_dX = ((instanceStruct.d_instanceVar_B4SOIoxideTrapDensityA_dX+(instanceStruct.instanceVar_B4SOIoxideTrapDensityB*d_Nl_dX+instanceStruct.d_instanceVar_B4SOIoxideTrapDensityB_dX*Nl))+((instanceStruct.instanceVar_B4SOIoxideTrapDensityC*Nl)*d_Nl_dX+(instanceStruct.instanceVar_B4SOIoxideTrapDensityC*d_Nl_dX+instanceStruct.d_instanceVar_B4SOIoxideTrapDensityC_dX*Nl)*Nl));
T8 = ((instanceStruct.instanceVar_B4SOIoxideTrapDensityA+(instanceStruct.instanceVar_B4SOIoxideTrapDensityB*Nl))+((instanceStruct.instanceVar_B4SOIoxideTrapDensityC*Nl)*Nl));
d_T9_dX = ((Nl+instanceStruct.instanceVar_B4SOInstar)*(d_Nl_dX+instanceStruct.d_instanceVar_B4SOInstar_dX)+(d_Nl_dX+instanceStruct.d_instanceVar_B4SOInstar_dX)*(Nl+instanceStruct.instanceVar_B4SOInstar));
T9 = ((Nl+instanceStruct.instanceVar_B4SOInstar)*(Nl+instanceStruct.instanceVar_B4SOInstar));
d_Ssi_dX = (((T1/T2)*((d_T3_dX+d_T4_dX)+d_T5_dX)+((T2*d_T1_dX-T1*d_T2_dX)/T2/T2)*((T3+T4)+T5))+((T9*(((T6/T7)*DelClm)*d_T8_dX+((T6/T7)*d_DelClm_dX+((T7*d_T6_dX-T6*d_T7_dX)/T7/T7)*DelClm)*T8)-(((T6/T7)*DelClm)*T8)*d_T9_dX)/T9/T9));
Ssi = (((T1/T2)*((T3+T4)+T5))+((((T6/T7)*DelClm)*T8)/T9));
d_T10_dX = ((instanceStruct.instanceVar_B4SOIoxideTrapDensityA*1.3806503e-23)*instanceStruct.d_instanceVar_B4SOITempSH_dX+instanceStruct.d_instanceVar_B4SOIoxideTrapDensityA_dX*1.3806503e-23*instanceStruct.instanceVar_B4SOITempSH);
T10 = ((instanceStruct.instanceVar_B4SOIoxideTrapDensityA*1.3806503e-23)*instanceStruct.instanceVar_B4SOITempSH);
d_T11_dX = ((((wefftot*instanceStruct.instanceVar_pParam_B4SOIleff)*1.0e10)*instanceStruct.instanceVar_B4SOInstar)*instanceStruct.d_instanceVar_B4SOInstar_dX+(((wefftot*instanceStruct.instanceVar_pParam_B4SOIleff)*1.0e10)*instanceStruct.d_instanceVar_B4SOInstar_dX+(wefftot*instanceStruct.d_instanceVar_pParam_B4SOIleff_dX+d_wefftot_dX*instanceStruct.instanceVar_pParam_B4SOIleff)*1.0e10*instanceStruct.instanceVar_B4SOInstar)*instanceStruct.instanceVar_B4SOInstar);
T11 = ((((wefftot*instanceStruct.instanceVar_pParam_B4SOIleff)*1.0e10)*instanceStruct.instanceVar_B4SOInstar)*instanceStruct.instanceVar_B4SOInstar);
d_Swi_dX = (((T10/T11)*cd)*d_cd_dX+((T10/T11)*d_cd_dX+((T11*d_T10_dX-T10*d_T11_dX)/T11/T11)*cd)*cd);
Swi = (((T10/T11)*cd)*cd);
d_T1_dX = (d_Swi_dX+d_Ssi_dX);
T1 = (Swi+Ssi);
if ((((T1>0.0)&&(Ssi>0.0))&&(Swi>0.0)))
{
d_flickerNoiseContrib_dX = ((T1*(Ssi*d_Swi_dX+d_Ssi_dX*Swi)-(Ssi*Swi)*d_T1_dX)/T1/T1);
flickerNoiseContrib = ((Ssi*Swi)/T1);
}
else
{
d_flickerNoiseContrib_dX = 0.0;
flickerNoiseContrib = 0;
}
}
// End block Eval1ovFNoise
// I(di,si) <+ (flicker_noise((m*flickerNoiseContrib),B4SOIef,"flicker"))
}
// End block noise
if ((instanceStruct.instanceVar_B4SOIrdsMod!=2))
{
// I(d,di) <+ (((m*V(d,di))/Rd))
d_staticContributions_dX[admsNodeID_d]+= ((Rd*instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_d_di])-(instanceStruct.instancePar_m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dX)/Rd/Rd);
d_staticContributions_dX[(admsNodeID_di)]-= ((Rd*instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_d_di])-(instanceStruct.instancePar_m*(probeVars[admsProbeID_V_d_di]))*d_Rd_dX)/Rd/Rd);
// I(d,di) <+ (white_noise((m*abs((fourkt*gdnoise))),"rd"))
// I(s,si) <+ (((m*V(s,si))/Rs))
d_staticContributions_dX[admsNodeID_s]+= ((Rs*instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_s_si])-(instanceStruct.instancePar_m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dX)/Rs/Rs);
d_staticContributions_dX[(admsNodeID_si)]-= ((Rs*instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_s_si])-(instanceStruct.instancePar_m*(probeVars[admsProbeID_V_s_si]))*d_Rs_dX)/Rs/Rs);
// I(s,si) <+ (white_noise((m*abs((fourkt*gsnoise))),"rs"))
}
else
{
// V(d,di) <+ (0.0)
// do nothing at all
// V(s,si) <+ (0.0)
// do nothing at all
}
if ((instanceStruct.instanceVar_B4SOImode>0))
{
// I(di,si) <+ (((m*B4SOItype)*(Ids+Ic)))
d_staticContributions_dX[admsNodeID_di]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_Ids_dX+d_Ic_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(Ids+Ic));
d_staticContributions_dX[(admsNodeID_si)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_Ids_dX+d_Ic_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(Ids+Ic));
// I(di,b) <+ (((m*B4SOItype)*Iii))
d_staticContributions_dX[admsNodeID_di]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Iii_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Iii);
d_staticContributions_dX[(admsNodeID_b)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Iii_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Iii);
d_B4SOIigidl_dX = instanceStruct.instanceVar_B4SOItype*d_Igidl_dX;
B4SOIigidl = (instanceStruct.instanceVar_B4SOItype*Igidl);
d_B4SOIigisl_dX = instanceStruct.instanceVar_B4SOItype*d_Igisl_dX;
B4SOIigisl = (instanceStruct.instanceVar_B4SOItype*Igisl);
d_B4SOIIgcd_dX = instanceStruct.instanceVar_B4SOItype*d_Igcd_dX;
B4SOIIgcd = (instanceStruct.instanceVar_B4SOItype*Igcd);
d_B4SOIIgcs_dX = instanceStruct.instanceVar_B4SOItype*d_Igcs_dX;
B4SOIIgcs = (instanceStruct.instanceVar_B4SOItype*Igcs);
d_B4SOIqdrn_dX = instanceStruct.instanceVar_B4SOItype*d_qdrn_dX;
B4SOIqdrn = (instanceStruct.instanceVar_B4SOItype*qdrn);
d_B4SOIqsrc_dX = instanceStruct.instanceVar_B4SOItype*d_qsrc_dX;
B4SOIqsrc = (instanceStruct.instanceVar_B4SOItype*qsrc);
}
else
{
// I(si,di) <+ (((m*B4SOItype)*(Ids-Ic)))
d_staticContributions_dX[admsNodeID_si]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_Ids_dX-d_Ic_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(Ids-Ic));
d_staticContributions_dX[(admsNodeID_di)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_Ids_dX-d_Ic_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(Ids-Ic));
// I(si,b) <+ (((m*B4SOItype)*Iii))
d_staticContributions_dX[admsNodeID_si]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Iii_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Iii);
d_staticContributions_dX[(admsNodeID_b)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Iii_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Iii);
d_B4SOIigisl_dX = instanceStruct.instanceVar_B4SOItype*d_Igidl_dX;
B4SOIigisl = (instanceStruct.instanceVar_B4SOItype*Igidl);
d_B4SOIigidl_dX = instanceStruct.instanceVar_B4SOItype*d_Igisl_dX;
B4SOIigidl = (instanceStruct.instanceVar_B4SOItype*Igisl);
d_B4SOIIgcs_dX = instanceStruct.instanceVar_B4SOItype*d_Igcd_dX;
B4SOIIgcs = (instanceStruct.instanceVar_B4SOItype*Igcd);
d_B4SOIIgcd_dX = instanceStruct.instanceVar_B4SOItype*d_Igcs_dX;
B4SOIIgcd = (instanceStruct.instanceVar_B4SOItype*Igcs);
d_B4SOIqsrc_dX = instanceStruct.instanceVar_B4SOItype*d_qdrn_dX;
B4SOIqsrc = (instanceStruct.instanceVar_B4SOItype*qdrn);
d_B4SOIqdrn_dX = instanceStruct.instanceVar_B4SOItype*d_qsrc_dX;
B4SOIqdrn = (instanceStruct.instanceVar_B4SOItype*qsrc);
}
d_B4SOIIgd_dX = instanceStruct.instanceVar_B4SOItype*d_Igd_dX;
B4SOIIgd = (instanceStruct.instanceVar_B4SOItype*Igd);
d_B4SOIIgs_dX = instanceStruct.instanceVar_B4SOItype*d_Igs_dX;
B4SOIIgs = (instanceStruct.instanceVar_B4SOItype*Igs);
// I(di,b) <+ ((m*B4SOIigidl))
d_staticContributions_dX[admsNodeID_di]+= (instanceStruct.instancePar_m*d_B4SOIigidl_dX+instanceStruct.d_instancePar_m_dX*B4SOIigidl);
d_staticContributions_dX[(admsNodeID_b)]-= (instanceStruct.instancePar_m*d_B4SOIigidl_dX+instanceStruct.d_instancePar_m_dX*B4SOIigidl);
// I(si,b) <+ ((m*B4SOIigisl))
d_staticContributions_dX[admsNodeID_si]+= (instanceStruct.instancePar_m*d_B4SOIigisl_dX+instanceStruct.d_instancePar_m_dX*B4SOIigisl);
d_staticContributions_dX[(admsNodeID_b)]-= (instanceStruct.instancePar_m*d_B4SOIigisl_dX+instanceStruct.d_instancePar_m_dX*B4SOIigisl);
// I(db,di) <+ (((m*B4SOItype)*Ibd))
d_staticContributions_dX[admsNodeID_db]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Ibd_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Ibd);
d_staticContributions_dX[(admsNodeID_di)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Ibd_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Ibd);
// I(sb,si) <+ (((m*B4SOItype)*Ibs))
d_staticContributions_dX[admsNodeID_sb]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Ibs_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Ibs);
d_staticContributions_dX[(admsNodeID_si)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Ibs_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Ibs);
// I(gi,di) <+ ((m*(B4SOIIgd+B4SOIIgcd)))
d_staticContributions_dX[admsNodeID_gi]+= (instanceStruct.instancePar_m*(d_B4SOIIgd_dX+d_B4SOIIgcd_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIIgd+B4SOIIgcd));
d_staticContributions_dX[(admsNodeID_di)]-= (instanceStruct.instancePar_m*(d_B4SOIIgd_dX+d_B4SOIIgcd_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIIgd+B4SOIIgcd));
// I(gi,si) <+ ((m*(B4SOIIgs+B4SOIIgcs)))
d_staticContributions_dX[admsNodeID_gi]+= (instanceStruct.instancePar_m*(d_B4SOIIgs_dX+d_B4SOIIgcs_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIIgs+B4SOIIgcs));
d_staticContributions_dX[(admsNodeID_si)]-= (instanceStruct.instancePar_m*(d_B4SOIIgs_dX+d_B4SOIIgcs_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIIgs+B4SOIIgcs));
// I(gi,b) <+ ((m*B4SOIig))
d_staticContributions_dX[admsNodeID_gi]+= (instanceStruct.instancePar_m*d_B4SOIig_dX+instanceStruct.d_instancePar_m_dX*B4SOIig);
d_staticContributions_dX[(admsNodeID_b)]-= (instanceStruct.instancePar_m*d_B4SOIig_dX+instanceStruct.d_instancePar_m_dX*B4SOIig);
// I(gi,p) <+ ((m*B4SOIigp))
d_staticContributions_dX[admsNodeID_gi]+= (instanceStruct.instancePar_m*d_B4SOIigp_dX+instanceStruct.d_instancePar_m_dX*B4SOIigp);
d_staticContributions_dX[(admsNodeID_p)]-= (instanceStruct.instancePar_m*d_B4SOIigp_dX+instanceStruct.d_instancePar_m_dX*B4SOIigp);
if (((instanceStruct.instanceVar_B4SOIbodyMod==0)||(instanceStruct.instanceVar_B4SOIbodyMod==2)))
{
// V(b,p) <+ (0)
// do nothing at all
}
else
{
// I(b,p) <+ (((m*B4SOItype)*Ibp))
d_staticContributions_dX[admsNodeID_b]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Ibp_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Ibp);
d_staticContributions_dX[(admsNodeID_p)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*d_Ibp_dX+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*Ibp);
// I(b,p) <+ (white_noise((((m*fourkt)*abs(Ibp))/(abs(vbp)+1.0e-9)),"rbp"))
}
// I(db,di) <+ (white_noise(((((m*2)*1.60219e-19)*B4SOInoif)*abs(Ibd)),"ibd"))
// I(sb,si) <+ (white_noise(((((m*2)*1.60219e-19)*B4SOInoif)*abs(Ibs)),"ibs"))
// I(gi,di) <+ (white_noise((((m*2)*1.60219e-19)*abs((B4SOIIgd+B4SOIIgcd))),"igd"))
// I(gi,si) <+ (white_noise((((m*2)*1.60219e-19)*abs((B4SOIIgs+B4SOIIgcs))),"igs"))
// I(gi,b) <+ (white_noise((((m*2)*1.60219e-19)*abs(B4SOIig)),"igb"))
// I(di,b) <+ ((m*ddt(B4SOIqdrn)))
d_dynamicContributions_dX[admsNodeID_di]+= (instanceStruct.instancePar_m*(d_B4SOIqdrn_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqdrn));
d_dynamicContributions_dX[(admsNodeID_b)]-= (instanceStruct.instancePar_m*(d_B4SOIqdrn_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqdrn));
// I(si,b) <+ ((m*ddt(B4SOIqsrc)))
d_dynamicContributions_dX[admsNodeID_si]+= (instanceStruct.instancePar_m*(d_B4SOIqsrc_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqsrc));
d_dynamicContributions_dX[(admsNodeID_b)]-= (instanceStruct.instancePar_m*(d_B4SOIqsrc_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqsrc));
// I(gi,b) <+ (((m*B4SOItype)*ddt(qgate)))
d_dynamicContributions_dX[admsNodeID_gi]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgate_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgate));
d_dynamicContributions_dX[(admsNodeID_b)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgate_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgate));
// I(e,b) <+ (((m*B4SOItype)*ddt(qsub)))
d_dynamicContributions_dX[admsNodeID_e]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qsub_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qsub));
d_dynamicContributions_dX[(admsNodeID_b)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qsub_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qsub));
// I(db,di) <+ (((m*B4SOItype)*ddt(qjd)))
d_dynamicContributions_dX[admsNodeID_db]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qjd_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qjd));
d_dynamicContributions_dX[(admsNodeID_di)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qjd_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qjd));
// I(sb,si) <+ (((m*B4SOItype)*ddt(qjs)))
d_dynamicContributions_dX[admsNodeID_sb]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qjs_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qjs));
d_dynamicContributions_dX[(admsNodeID_si)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qjs_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qjs));
if ((modelStruct.modelPar_RGATEMOD==3))
{
// I(gm,di) <+ (((m*B4SOItype)*ddt(qgdo)))
d_dynamicContributions_dX[admsNodeID_gm]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgdo_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgdo));
d_dynamicContributions_dX[(admsNodeID_di)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgdo_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgdo));
// I(gm,si) <+ (((m*B4SOItype)*ddt(qgso)))
d_dynamicContributions_dX[admsNodeID_gm]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgso_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgso));
d_dynamicContributions_dX[(admsNodeID_si)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgso_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgso));
// I(gm,e) <+ ((m*ddt((V(gm,e)*pParam_B4SOIcgeo))))
d_dynamicContributions_dX[admsNodeID_gm]+= (instanceStruct.instancePar_m*((probeVars[admsProbeID_V_gm_e])*instanceStruct.d_instanceVar_pParam_B4SOIcgeo_dX)+instanceStruct.d_instancePar_m_dX*(((probeVars[admsProbeID_V_gm_e])*instanceStruct.instanceVar_pParam_B4SOIcgeo)));
d_dynamicContributions_dX[(admsNodeID_e)]-= (instanceStruct.instancePar_m*((probeVars[admsProbeID_V_gm_e])*instanceStruct.d_instanceVar_pParam_B4SOIcgeo_dX)+instanceStruct.d_instancePar_m_dX*(((probeVars[admsProbeID_V_gm_e])*instanceStruct.instanceVar_pParam_B4SOIcgeo)));
}
else
{
// I(gi,di) <+ (((m*B4SOItype)*ddt(qgdo)))
d_dynamicContributions_dX[admsNodeID_gi]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgdo_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgdo));
d_dynamicContributions_dX[(admsNodeID_di)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgdo_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgdo));
// I(gi,si) <+ (((m*B4SOItype)*ddt(qgso)))
d_dynamicContributions_dX[admsNodeID_gi]+= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgso_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgso));
d_dynamicContributions_dX[(admsNodeID_si)]-= ((instanceStruct.instancePar_m*instanceStruct.instanceVar_B4SOItype)*(d_qgso_dX)+instanceStruct.d_instancePar_m_dX*instanceStruct.instanceVar_B4SOItype*(qgso));
// I(gi,e) <+ ((m*ddt((V(gi,e)*pParam_B4SOIcgeo))))
d_dynamicContributions_dX[admsNodeID_gi]+= (instanceStruct.instancePar_m*((probeVars[admsProbeID_V_gi_e])*instanceStruct.d_instanceVar_pParam_B4SOIcgeo_dX)+instanceStruct.d_instancePar_m_dX*(((probeVars[admsProbeID_V_gi_e])*instanceStruct.instanceVar_pParam_B4SOIcgeo)));
d_dynamicContributions_dX[(admsNodeID_e)]-= (instanceStruct.instancePar_m*((probeVars[admsProbeID_V_gi_e])*instanceStruct.d_instanceVar_pParam_B4SOIcgeo_dX)+instanceStruct.d_instancePar_m_dX*(((probeVars[admsProbeID_V_gi_e])*instanceStruct.instanceVar_pParam_B4SOIcgeo)));
}
// I(di,e) <+ ((m*ddt(B4SOIqde)))
d_dynamicContributions_dX[admsNodeID_di]+= (instanceStruct.instancePar_m*(d_B4SOIqde_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqde));
d_dynamicContributions_dX[(admsNodeID_e)]-= (instanceStruct.instancePar_m*(d_B4SOIqde_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqde));
// I(si,e) <+ ((m*ddt(B4SOIqse)))
d_dynamicContributions_dX[admsNodeID_si]+= (instanceStruct.instancePar_m*(d_B4SOIqse_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqse));
d_dynamicContributions_dX[(admsNodeID_e)]-= (instanceStruct.instancePar_m*(d_B4SOIqse_dX)+instanceStruct.d_instancePar_m_dX*(B4SOIqse));
if (((modelStruct.modelPar_RGATEMOD==0)||(modelStruct.modelPar_RGATEMOD==2)))
{
// V(g,gm) <+ (0)
// do nothing at all
}
else
{
// I(g,gm) <+ (((m*V(g,gm))*B4SOIgrgeltd))
d_staticContributions_dX[admsNodeID_g]+= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_g_gm]))*instanceStruct.d_instanceVar_B4SOIgrgeltd_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_g_gm])*instanceStruct.instanceVar_B4SOIgrgeltd);
d_staticContributions_dX[(admsNodeID_gm)]-= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_g_gm]))*instanceStruct.d_instanceVar_B4SOIgrgeltd_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_g_gm])*instanceStruct.instanceVar_B4SOIgrgeltd);
// I(g,gm) <+ (white_noise((m*abs((fourkt*B4SOIgrgeltd))),"rg"))
}
if (((modelStruct.modelPar_RGATEMOD==0)||(modelStruct.modelPar_RGATEMOD==1)))
{
// V(gm,gi) <+ (0)
// do nothing at all
}
else
{
// I(gm,gi) <+ (((m*V(gm,gi))*B4SOIgcrg))
d_staticContributions_dX[admsNodeID_gm]+= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_gm_gi])*B4SOIgcrg);
d_staticContributions_dX[(admsNodeID_gi)]-= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_gm_gi]))*d_B4SOIgcrg_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_gm_gi])*B4SOIgcrg);
if ((modelStruct.modelPar_RGATEMOD==2))
//Begin block rgate2
{
//Block-local variables for block rgate2
double T0=0.0;
double d_T0_dX=0.0;
//End of Block-local variables
d_T0_dX = ((B4SOIgcrg*instanceStruct.d_instanceVar_B4SOIgrgeltd_dX-instanceStruct.instanceVar_B4SOIgrgeltd*d_B4SOIgcrg_dX)/B4SOIgcrg/B4SOIgcrg);
T0 = (1.0+(instanceStruct.instanceVar_B4SOIgrgeltd/B4SOIgcrg));
// I(gm,gi) <+ (white_noise((m*abs(((fourkt*B4SOIgrgeltd)/(T0*T0)))),"rg"))
}
// End block rgate2
}
if (instanceStruct.instanceVar_B4SOIrbodyMod)
{
// I(b,db) <+ (((m*V(b,db))*B4SOIgrbdb))
d_staticContributions_dX[admsNodeID_b]+= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_b_db]))*instanceStruct.d_instanceVar_B4SOIgrbdb_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_b_db])*instanceStruct.instanceVar_B4SOIgrbdb);
d_staticContributions_dX[(admsNodeID_db)]-= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_b_db]))*instanceStruct.d_instanceVar_B4SOIgrbdb_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_b_db])*instanceStruct.instanceVar_B4SOIgrbdb);
// I(b,sb) <+ (((m*V(b,sb))*B4SOIgrbsb))
d_staticContributions_dX[admsNodeID_b]+= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_b_sb]))*instanceStruct.d_instanceVar_B4SOIgrbsb_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_b_sb])*instanceStruct.instanceVar_B4SOIgrbsb);
d_staticContributions_dX[(admsNodeID_sb)]-= ((instanceStruct.instancePar_m*(probeVars[admsProbeID_V_b_sb]))*instanceStruct.d_instanceVar_B4SOIgrbsb_dX+instanceStruct.d_instancePar_m_dX*(probeVars[admsProbeID_V_b_sb])*instanceStruct.instanceVar_B4SOIgrbsb);
// I(b,db) <+ (white_noise((m*abs((fourkt*B4SOIgrbdb))),"rbdb"))
// I(b,sb) <+ (white_noise((m*abs((fourkt*B4SOIgrbsb))),"rbsb"))
}
else
{
// V(b,db) <+ (0)
// do nothing at all
// V(b,sb) <+ (0)
// do nothing at all
}
if ((instanceStruct.instanceVar_B4SOIsoiMod==2))
{
// V(b,si) <+ (0)
// do nothing at all
}
if (((instanceStruct.instancePar_SHMOD==1)&&(instanceStruct.instancePar_RTH0!=0.0)))
{
if ((instanceStruct.instancePar_TNODEOUT&&(!portsConnected_[admsNodeID_t])))
{
if (portsConnected_[admsNodeID_b])
{
// I(b,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
d_staticContributions_dX[admsNodeID_b]+= (instanceStruct.instancePar_m*((-d_Ids_dX)*Vds+(-delTemp*instanceStruct.d_instanceVar_pParam_B4SOIrth_dX/instanceStruct.instanceVar_pParam_B4SOIrth/instanceStruct.instanceVar_pParam_B4SOIrth))+instanceStruct.d_instancePar_m_dX*(((-Ids)*Vds)+(delTemp/instanceStruct.instanceVar_pParam_B4SOIrth)));
// I(b,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
d_dynamicContributions_dX[admsNodeID_b]+= (instanceStruct.instancePar_m*(delTemp*instanceStruct.d_instanceVar_pParam_B4SOIcth_dX)+instanceStruct.d_instancePar_m_dX*((delTemp*instanceStruct.instanceVar_pParam_B4SOIcth)));
}
else
{
if (portsConnected_[admsNodeID_p])
{
// I(p,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
d_staticContributions_dX[admsNodeID_p]+= (instanceStruct.instancePar_m*((-d_Ids_dX)*Vds+(-delTemp*instanceStruct.d_instanceVar_pParam_B4SOIrth_dX/instanceStruct.instanceVar_pParam_B4SOIrth/instanceStruct.instanceVar_pParam_B4SOIrth))+instanceStruct.d_instancePar_m_dX*(((-Ids)*Vds)+(delTemp/instanceStruct.instanceVar_pParam_B4SOIrth)));
// I(p,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
d_dynamicContributions_dX[admsNodeID_p]+= (instanceStruct.instancePar_m*(delTemp*instanceStruct.d_instanceVar_pParam_B4SOIcth_dX)+instanceStruct.d_instancePar_m_dX*((delTemp*instanceStruct.instanceVar_pParam_B4SOIcth)));
}
else
{
// Pwr(t,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
d_staticContributions_dX[admsNodeID_t]+= (instanceStruct.instancePar_m*((-d_Ids_dX)*Vds+(-delTemp*instanceStruct.d_instanceVar_pParam_B4SOIrth_dX/instanceStruct.instanceVar_pParam_B4SOIrth/instanceStruct.instanceVar_pParam_B4SOIrth))+instanceStruct.d_instancePar_m_dX*(((-Ids)*Vds)+(delTemp/instanceStruct.instanceVar_pParam_B4SOIrth)));
// Pwr(t,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
d_dynamicContributions_dX[admsNodeID_t]+= (instanceStruct.instancePar_m*(delTemp*instanceStruct.d_instanceVar_pParam_B4SOIcth_dX)+instanceStruct.d_instancePar_m_dX*((delTemp*instanceStruct.instanceVar_pParam_B4SOIcth)));
}
}
}
else
{
// Pwr(t,GND) <+ ((m*(((-Ids)*Vds)+(delTemp/pParam_B4SOIrth))))
d_staticContributions_dX[admsNodeID_t]+= (instanceStruct.instancePar_m*((-d_Ids_dX)*Vds+(-delTemp*instanceStruct.d_instanceVar_pParam_B4SOIrth_dX/instanceStruct.instanceVar_pParam_B4SOIrth/instanceStruct.instanceVar_pParam_B4SOIrth))+instanceStruct.d_instancePar_m_dX*(((-Ids)*Vds)+(delTemp/instanceStruct.instanceVar_pParam_B4SOIrth)));
// Pwr(t,GND) <+ ((m*ddt((delTemp*pParam_B4SOIcth))))
d_dynamicContributions_dX[admsNodeID_t]+= (instanceStruct.instancePar_m*(delTemp*instanceStruct.d_instanceVar_pParam_B4SOIcth_dX)+instanceStruct.d_instancePar_m_dX*((delTemp*instanceStruct.instanceVar_pParam_B4SOIcth)));
}
}
else
{
if ((instanceStruct.instancePar_TNODEOUT&&(!portsConnected_[admsNodeID_t])))
{
if (portsConnected_[admsNodeID_b])
{
UserInfo(theInstance)  << "Error:  you have specified a temperature node as the 6th node with TNODEOUT=0 and self-heating disabled.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 6th node." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
else
{
if (portsConnected_[admsNodeID_p])
{
UserInfo(theInstance)  << "Error:  you have specified a temperature node as the 5th node with TNODEOUT=0 and self-heating disabled.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 5th node." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
else
{
UserInfo(theInstance)  << "Error:  you have specified a temperature node as the 7th node with TNODEOUT=0 and self-heating disabled.  This node is not used in this mode and Xyce cannot deal with it being externally specified.  Please remove the 7th node." << " " <<  std::endl; 
 UserError(theInstance) << "$finish called." << std::endl; 
}
}
}
else
{
// Temp(t,GND) <+ (0)
// do nothing at all
}
}
d_QGI_dX = instanceStruct.instanceVar_B4SOItype*d_qgi_dX;
QGI = (instanceStruct.instanceVar_B4SOItype*qgi);
d_QOV_dX = instanceStruct.instanceVar_B4SOItype*d_qov_dX;
QOV = (instanceStruct.instanceVar_B4SOItype*qov);
if ((modelStruct.modelPar_RGATEMOD==3))
{
}
else
{
}
}
// End block load
}



//-----------------------------------------------------------------------------
// Function      : InstanceSensitivity::operator()
// Purpose       : return sensitivity for a single instance parameter
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
/// Compute sensitivity of a device instance's outputs to a specified instance
/// parameter
///
///
/// @param[in] entity  The device entity that owns the parameter
/// @param[in] name    The unadorned parameter name specific to the entity
/// @param[out] dfdp   The derivative of the F vector with respect to the parameter
/// @param[out] dqdp   The derivative of the Q vector with respect to the parameter
/// @param[out] dbdp   The derivative of the B vector with respect to the parameter
/// @param[out] Findices   The indices into the F vector corresponding to the elements of dfdp
/// @param[out] Qindices   The indices into the Q vector corresponding to the elements of dfdp
/// @param[out] Bindices   The indices into the B vector corresponding to the elements of dfdp
///
///
void InstanceSensitivity::operator()
(
const ParameterBase &entity,
const std::string &name,
std::vector<double> & dfdp,
std::vector<double> & dqdp,
std::vector<double> & dbdp,
std::vector<int> & Findices,
std::vector<int> & Qindices,
std::vector<int> & Bindices
) const
{
const ParameterBase * e1 = &entity;
const Instance & in = *(dynamic_cast<const Instance *> (e1));
const Model & mod =in.model_;

dfdp.resize(13+0);
dqdp.resize(13+0);
Findices.resize(13+0);
Qindices.resize(13+0);

std::vector <double> probeVars(26);
std::vector <double> d_staticContributions_dX(13+0);
std::vector <double> d_dynamicContributions_dX(13+0);


// initialize contributions to zero (automatically sets derivatives to zero)
for (int i=0; i < 13+0 ; ++i)
{
   d_staticContributions_dX[i]=0;
   d_dynamicContributions_dX[i]=0;
}

// Copy out all the model parameters (doubles) into local structure
modelSensStruct modelStruct;

//reals
modelStruct.modelPar_L=mod.L;
modelStruct.d_modelPar_L_dX=0.0;
modelStruct.modelPar_given_L=mod.given("L");
modelStruct.modelPar_W=mod.W;
modelStruct.d_modelPar_W_dX=0.0;
modelStruct.modelPar_given_W=mod.given("W");
modelStruct.modelPar_SA=mod.SA;
modelStruct.d_modelPar_SA_dX=0.0;
modelStruct.modelPar_given_SA=mod.given("SA");
modelStruct.modelPar_SB=mod.SB;
modelStruct.d_modelPar_SB_dX=0.0;
modelStruct.modelPar_given_SB=mod.given("SB");
modelStruct.modelPar_SD=mod.SD;
modelStruct.d_modelPar_SD_dX=0.0;
modelStruct.modelPar_given_SD=mod.given("SD");
modelStruct.modelPar_AD=mod.AD;
modelStruct.d_modelPar_AD_dX=0.0;
modelStruct.modelPar_given_AD=mod.given("AD");
modelStruct.modelPar_AS=mod.AS;
modelStruct.d_modelPar_AS_dX=0.0;
modelStruct.modelPar_given_AS=mod.given("AS");
modelStruct.modelPar_PD=mod.PD;
modelStruct.d_modelPar_PD_dX=0.0;
modelStruct.modelPar_given_PD=mod.given("PD");
modelStruct.modelPar_PS=mod.PS;
modelStruct.d_modelPar_PS_dX=0.0;
modelStruct.modelPar_given_PS=mod.given("PS");
modelStruct.modelPar_NRD=mod.NRD;
modelStruct.d_modelPar_NRD_dX=0.0;
modelStruct.modelPar_given_NRD=mod.given("NRD");
modelStruct.modelPar_NRS=mod.NRS;
modelStruct.d_modelPar_NRS_dX=0.0;
modelStruct.modelPar_given_NRS=mod.given("NRS");
modelStruct.modelPar_RTH0=mod.RTH0;
modelStruct.d_modelPar_RTH0_dX=0.0;
modelStruct.modelPar_given_RTH0=mod.given("RTH0");
modelStruct.modelPar_CTH0=mod.CTH0;
modelStruct.d_modelPar_CTH0_dX=0.0;
modelStruct.modelPar_given_CTH0=mod.given("CTH0");
modelStruct.modelPar_NRB=mod.NRB;
modelStruct.d_modelPar_NRB_dX=0.0;
modelStruct.modelPar_given_NRB=mod.given("NRB");
modelStruct.modelPar_FRBODY=mod.FRBODY;
modelStruct.d_modelPar_FRBODY_dX=0.0;
modelStruct.modelPar_given_FRBODY=mod.given("FRBODY");
modelStruct.modelPar_RBDB=mod.RBDB;
modelStruct.d_modelPar_RBDB_dX=0.0;
modelStruct.modelPar_given_RBDB=mod.given("RBDB");
modelStruct.modelPar_RBSB=mod.RBSB;
modelStruct.d_modelPar_RBSB_dX=0.0;
modelStruct.modelPar_given_RBSB=mod.given("RBSB");
modelStruct.modelPar_DELVTO=mod.DELVTO;
modelStruct.d_modelPar_DELVTO_dX=0.0;
modelStruct.modelPar_given_DELVTO=mod.given("DELVTO");
modelStruct.modelPar_NBC=mod.NBC;
modelStruct.d_modelPar_NBC_dX=0.0;
modelStruct.modelPar_given_NBC=mod.given("NBC");
modelStruct.modelPar_NSEG=mod.NSEG;
modelStruct.d_modelPar_NSEG_dX=0.0;
modelStruct.modelPar_given_NSEG=mod.given("NSEG");
modelStruct.modelPar_PDBCP=mod.PDBCP;
modelStruct.d_modelPar_PDBCP_dX=0.0;
modelStruct.modelPar_given_PDBCP=mod.given("PDBCP");
modelStruct.modelPar_PSBCP=mod.PSBCP;
modelStruct.d_modelPar_PSBCP_dX=0.0;
modelStruct.modelPar_given_PSBCP=mod.given("PSBCP");
modelStruct.modelPar_AGBCP=mod.AGBCP;
modelStruct.d_modelPar_AGBCP_dX=0.0;
modelStruct.modelPar_given_AGBCP=mod.given("AGBCP");
modelStruct.modelPar_AGBCP2=mod.AGBCP2;
modelStruct.d_modelPar_AGBCP2_dX=0.0;
modelStruct.modelPar_given_AGBCP2=mod.given("AGBCP2");
modelStruct.modelPar_AGBCPD=mod.AGBCPD;
modelStruct.d_modelPar_AGBCPD_dX=0.0;
modelStruct.modelPar_given_AGBCPD=mod.given("AGBCPD");
modelStruct.modelPar_AEBCP=mod.AEBCP;
modelStruct.d_modelPar_AEBCP_dX=0.0;
modelStruct.modelPar_given_AEBCP=mod.given("AEBCP");
modelStruct.modelPar_VERSION=mod.VERSION;
modelStruct.d_modelPar_VERSION_dX=0.0;
modelStruct.modelPar_given_VERSION=mod.given("VERSION");
modelStruct.modelPar_VBSUSR=mod.VBSUSR;
modelStruct.d_modelPar_VBSUSR_dX=0.0;
modelStruct.modelPar_given_VBSUSR=mod.given("VBSUSR");
modelStruct.modelPar_EOT=mod.EOT;
modelStruct.d_modelPar_EOT_dX=0.0;
modelStruct.modelPar_given_EOT=mod.given("EOT");
modelStruct.modelPar_EPSROX=mod.EPSROX;
modelStruct.d_modelPar_EPSROX_dX=0.0;
modelStruct.modelPar_given_EPSROX=mod.given("EPSROX");
modelStruct.modelPar_EPSRSUB=mod.EPSRSUB;
modelStruct.d_modelPar_EPSRSUB_dX=0.0;
modelStruct.modelPar_given_EPSRSUB=mod.given("EPSRSUB");
modelStruct.modelPar_NI0SUB=mod.NI0SUB;
modelStruct.d_modelPar_NI0SUB_dX=0.0;
modelStruct.modelPar_given_NI0SUB=mod.given("NI0SUB");
modelStruct.modelPar_BG0SUB=mod.BG0SUB;
modelStruct.d_modelPar_BG0SUB_dX=0.0;
modelStruct.modelPar_given_BG0SUB=mod.given("BG0SUB");
modelStruct.modelPar_TBGASUB=mod.TBGASUB;
modelStruct.d_modelPar_TBGASUB_dX=0.0;
modelStruct.modelPar_given_TBGASUB=mod.given("TBGASUB");
modelStruct.modelPar_TBGBSUB=mod.TBGBSUB;
modelStruct.d_modelPar_TBGBSUB_dX=0.0;
modelStruct.modelPar_given_TBGBSUB=mod.given("TBGBSUB");
modelStruct.modelPar_PHIG=mod.PHIG;
modelStruct.d_modelPar_PHIG_dX=0.0;
modelStruct.modelPar_given_PHIG=mod.given("PHIG");
modelStruct.modelPar_EASUB=mod.EASUB;
modelStruct.d_modelPar_EASUB_dX=0.0;
modelStruct.modelPar_given_EASUB=mod.given("EASUB");
modelStruct.modelPar_LEFFEOT=mod.LEFFEOT;
modelStruct.d_modelPar_LEFFEOT_dX=0.0;
modelStruct.modelPar_given_LEFFEOT=mod.given("LEFFEOT");
modelStruct.modelPar_WEFFEOT=mod.WEFFEOT;
modelStruct.d_modelPar_WEFFEOT_dX=0.0;
modelStruct.modelPar_given_WEFFEOT=mod.given("WEFFEOT");
modelStruct.modelPar_VDDEOT=mod.VDDEOT;
modelStruct.d_modelPar_VDDEOT_dX=0.0;
modelStruct.modelPar_given_VDDEOT=mod.given("VDDEOT");
modelStruct.modelPar_TEMPEOT=mod.TEMPEOT;
modelStruct.d_modelPar_TEMPEOT_dX=0.0;
modelStruct.modelPar_given_TEMPEOT=mod.given("TEMPEOT");
modelStruct.modelPar_ADOS=mod.ADOS;
modelStruct.d_modelPar_ADOS_dX=0.0;
modelStruct.modelPar_given_ADOS=mod.given("ADOS");
modelStruct.modelPar_BDOS=mod.BDOS;
modelStruct.d_modelPar_BDOS_dX=0.0;
modelStruct.modelPar_given_BDOS=mod.given("BDOS");
modelStruct.modelPar_EPSRGATE=mod.EPSRGATE;
modelStruct.d_modelPar_EPSRGATE_dX=0.0;
modelStruct.modelPar_given_EPSRGATE=mod.given("EPSRGATE");
modelStruct.modelPar_TOX=mod.TOX;
modelStruct.d_modelPar_TOX_dX=0.0;
modelStruct.modelPar_given_TOX=mod.given("TOX");
modelStruct.modelPar_TOXP=mod.TOXP;
modelStruct.d_modelPar_TOXP_dX=0.0;
modelStruct.modelPar_given_TOXP=mod.given("TOXP");
modelStruct.modelPar_TOXM=mod.TOXM;
modelStruct.d_modelPar_TOXM_dX=0.0;
modelStruct.modelPar_given_TOXM=mod.given("TOXM");
modelStruct.modelPar_DTOXCV=mod.DTOXCV;
modelStruct.d_modelPar_DTOXCV_dX=0.0;
modelStruct.modelPar_given_DTOXCV=mod.given("DTOXCV");
modelStruct.modelPar_CDSC=mod.CDSC;
modelStruct.d_modelPar_CDSC_dX=0.0;
modelStruct.modelPar_given_CDSC=mod.given("CDSC");
modelStruct.modelPar_CDSCB=mod.CDSCB;
modelStruct.d_modelPar_CDSCB_dX=0.0;
modelStruct.modelPar_given_CDSCB=mod.given("CDSCB");
modelStruct.modelPar_CDSCD=mod.CDSCD;
modelStruct.d_modelPar_CDSCD_dX=0.0;
modelStruct.modelPar_given_CDSCD=mod.given("CDSCD");
modelStruct.modelPar_CIT=mod.CIT;
modelStruct.d_modelPar_CIT_dX=0.0;
modelStruct.modelPar_given_CIT=mod.given("CIT");
modelStruct.modelPar_NFACTOR=mod.NFACTOR;
modelStruct.d_modelPar_NFACTOR_dX=0.0;
modelStruct.modelPar_given_NFACTOR=mod.given("NFACTOR");
modelStruct.modelPar_VSAT=mod.VSAT;
modelStruct.d_modelPar_VSAT_dX=0.0;
modelStruct.modelPar_given_VSAT=mod.given("VSAT");
modelStruct.modelPar_AT=mod.AT;
modelStruct.d_modelPar_AT_dX=0.0;
modelStruct.modelPar_given_AT=mod.given("AT");
modelStruct.modelPar_A0=mod.A0;
modelStruct.d_modelPar_A0_dX=0.0;
modelStruct.modelPar_given_A0=mod.given("A0");
modelStruct.modelPar_AGS=mod.AGS;
modelStruct.d_modelPar_AGS_dX=0.0;
modelStruct.modelPar_given_AGS=mod.given("AGS");
modelStruct.modelPar_A1=mod.A1;
modelStruct.d_modelPar_A1_dX=0.0;
modelStruct.modelPar_given_A1=mod.given("A1");
modelStruct.modelPar_A2=mod.A2;
modelStruct.d_modelPar_A2_dX=0.0;
modelStruct.modelPar_given_A2=mod.given("A2");
modelStruct.modelPar_KETA=mod.KETA;
modelStruct.d_modelPar_KETA_dX=0.0;
modelStruct.modelPar_given_KETA=mod.given("KETA");
modelStruct.modelPar_NSUB=mod.NSUB;
modelStruct.d_modelPar_NSUB_dX=0.0;
modelStruct.modelPar_given_NSUB=mod.given("NSUB");
modelStruct.modelPar_NCH=mod.NCH;
modelStruct.d_modelPar_NCH_dX=0.0;
modelStruct.modelPar_given_NCH=mod.given("NCH");
modelStruct.modelPar_NGATE=mod.NGATE;
modelStruct.d_modelPar_NGATE_dX=0.0;
modelStruct.modelPar_given_NGATE=mod.given("NGATE");
modelStruct.modelPar_NSD=mod.NSD;
modelStruct.d_modelPar_NSD_dX=0.0;
modelStruct.modelPar_given_NSD=mod.given("NSD");
modelStruct.modelPar_GAMMA1=mod.GAMMA1;
modelStruct.d_modelPar_GAMMA1_dX=0.0;
modelStruct.modelPar_given_GAMMA1=mod.given("GAMMA1");
modelStruct.modelPar_GAMMA2=mod.GAMMA2;
modelStruct.d_modelPar_GAMMA2_dX=0.0;
modelStruct.modelPar_given_GAMMA2=mod.given("GAMMA2");
modelStruct.modelPar_VBX=mod.VBX;
modelStruct.d_modelPar_VBX_dX=0.0;
modelStruct.modelPar_given_VBX=mod.given("VBX");
modelStruct.modelPar_VBM=mod.VBM;
modelStruct.d_modelPar_VBM_dX=0.0;
modelStruct.modelPar_given_VBM=mod.given("VBM");
modelStruct.modelPar_XT=mod.XT;
modelStruct.d_modelPar_XT_dX=0.0;
modelStruct.modelPar_given_XT=mod.given("XT");
modelStruct.modelPar_K1=mod.K1;
modelStruct.d_modelPar_K1_dX=0.0;
modelStruct.modelPar_given_K1=mod.given("K1");
modelStruct.modelPar_KT1=mod.KT1;
modelStruct.d_modelPar_KT1_dX=0.0;
modelStruct.modelPar_given_KT1=mod.given("KT1");
modelStruct.modelPar_KT1L=mod.KT1L;
modelStruct.d_modelPar_KT1L_dX=0.0;
modelStruct.modelPar_given_KT1L=mod.given("KT1L");
modelStruct.modelPar_KT2=mod.KT2;
modelStruct.d_modelPar_KT2_dX=0.0;
modelStruct.modelPar_given_KT2=mod.given("KT2");
modelStruct.modelPar_K2=mod.K2;
modelStruct.d_modelPar_K2_dX=0.0;
modelStruct.modelPar_given_K2=mod.given("K2");
modelStruct.modelPar_K3=mod.K3;
modelStruct.d_modelPar_K3_dX=0.0;
modelStruct.modelPar_given_K3=mod.given("K3");
modelStruct.modelPar_K3B=mod.K3B;
modelStruct.d_modelPar_K3B_dX=0.0;
modelStruct.modelPar_given_K3B=mod.given("K3B");
modelStruct.modelPar_W0=mod.W0;
modelStruct.d_modelPar_W0_dX=0.0;
modelStruct.modelPar_given_W0=mod.given("W0");
modelStruct.modelPar_LPEB=mod.LPEB;
modelStruct.d_modelPar_LPEB_dX=0.0;
modelStruct.modelPar_given_LPEB=mod.given("LPEB");
modelStruct.modelPar_DVT0=mod.DVT0;
modelStruct.d_modelPar_DVT0_dX=0.0;
modelStruct.modelPar_given_DVT0=mod.given("DVT0");
modelStruct.modelPar_DVT1=mod.DVT1;
modelStruct.d_modelPar_DVT1_dX=0.0;
modelStruct.modelPar_given_DVT1=mod.given("DVT1");
modelStruct.modelPar_DVT2=mod.DVT2;
modelStruct.d_modelPar_DVT2_dX=0.0;
modelStruct.modelPar_given_DVT2=mod.given("DVT2");
modelStruct.modelPar_DVT0W=mod.DVT0W;
modelStruct.d_modelPar_DVT0W_dX=0.0;
modelStruct.modelPar_given_DVT0W=mod.given("DVT0W");
modelStruct.modelPar_DVT1W=mod.DVT1W;
modelStruct.d_modelPar_DVT1W_dX=0.0;
modelStruct.modelPar_given_DVT1W=mod.given("DVT1W");
modelStruct.modelPar_DVT2W=mod.DVT2W;
modelStruct.d_modelPar_DVT2W_dX=0.0;
modelStruct.modelPar_given_DVT2W=mod.given("DVT2W");
modelStruct.modelPar_DROUT=mod.DROUT;
modelStruct.d_modelPar_DROUT_dX=0.0;
modelStruct.modelPar_given_DROUT=mod.given("DROUT");
modelStruct.modelPar_DSUB=mod.DSUB;
modelStruct.d_modelPar_DSUB_dX=0.0;
modelStruct.modelPar_given_DSUB=mod.given("DSUB");
modelStruct.modelPar_VTHO=mod.VTHO;
modelStruct.d_modelPar_VTHO_dX=0.0;
modelStruct.modelPar_given_VTHO=mod.given("VTHO");
modelStruct.modelPar_VTH0=mod.VTH0;
modelStruct.d_modelPar_VTH0_dX=0.0;
modelStruct.modelPar_given_VTH0=mod.given("VTH0");
modelStruct.modelPar_VFB=mod.VFB;
modelStruct.d_modelPar_VFB_dX=0.0;
modelStruct.modelPar_given_VFB=mod.given("VFB");
modelStruct.modelPar_UA=mod.UA;
modelStruct.d_modelPar_UA_dX=0.0;
modelStruct.modelPar_given_UA=mod.given("UA");
modelStruct.modelPar_UA1=mod.UA1;
modelStruct.d_modelPar_UA1_dX=0.0;
modelStruct.modelPar_given_UA1=mod.given("UA1");
modelStruct.modelPar_UB=mod.UB;
modelStruct.d_modelPar_UB_dX=0.0;
modelStruct.modelPar_given_UB=mod.given("UB");
modelStruct.modelPar_UB1=mod.UB1;
modelStruct.d_modelPar_UB1_dX=0.0;
modelStruct.modelPar_given_UB1=mod.given("UB1");
modelStruct.modelPar_UC=mod.UC;
modelStruct.d_modelPar_UC_dX=0.0;
modelStruct.modelPar_given_UC=mod.given("UC");
modelStruct.modelPar_UC1=mod.UC1;
modelStruct.d_modelPar_UC1_dX=0.0;
modelStruct.modelPar_given_UC1=mod.given("UC1");
modelStruct.modelPar_U0=mod.U0;
modelStruct.d_modelPar_U0_dX=0.0;
modelStruct.modelPar_given_U0=mod.given("U0");
modelStruct.modelPar_EU=mod.EU;
modelStruct.d_modelPar_EU_dX=0.0;
modelStruct.modelPar_given_EU=mod.given("EU");
modelStruct.modelPar_UTE=mod.UTE;
modelStruct.d_modelPar_UTE_dX=0.0;
modelStruct.modelPar_given_UTE=mod.given("UTE");
modelStruct.modelPar_UCS=mod.UCS;
modelStruct.d_modelPar_UCS_dX=0.0;
modelStruct.modelPar_given_UCS=mod.given("UCS");
modelStruct.modelPar_UCSTE=mod.UCSTE;
modelStruct.d_modelPar_UCSTE_dX=0.0;
modelStruct.modelPar_given_UCSTE=mod.given("UCSTE");
modelStruct.modelPar_UD=mod.UD;
modelStruct.d_modelPar_UD_dX=0.0;
modelStruct.modelPar_given_UD=mod.given("UD");
modelStruct.modelPar_UD1=mod.UD1;
modelStruct.d_modelPar_UD1_dX=0.0;
modelStruct.modelPar_given_UD1=mod.given("UD1");
modelStruct.modelPar_VOFF=mod.VOFF;
modelStruct.d_modelPar_VOFF_dX=0.0;
modelStruct.modelPar_given_VOFF=mod.given("VOFF");
modelStruct.modelPar_TNOM=mod.TNOM;
modelStruct.d_modelPar_TNOM_dX=0.0;
modelStruct.modelPar_given_TNOM=mod.given("TNOM");
modelStruct.modelPar_CGSO=mod.CGSO;
modelStruct.d_modelPar_CGSO_dX=0.0;
modelStruct.modelPar_given_CGSO=mod.given("CGSO");
modelStruct.modelPar_CGDO=mod.CGDO;
modelStruct.d_modelPar_CGDO_dX=0.0;
modelStruct.modelPar_given_CGDO=mod.given("CGDO");
modelStruct.modelPar_XPART=mod.XPART;
modelStruct.d_modelPar_XPART_dX=0.0;
modelStruct.modelPar_given_XPART=mod.given("XPART");
modelStruct.modelPar_DELTA=mod.DELTA;
modelStruct.d_modelPar_DELTA_dX=0.0;
modelStruct.modelPar_given_DELTA=mod.given("DELTA");
modelStruct.modelPar_RSH=mod.RSH;
modelStruct.d_modelPar_RSH_dX=0.0;
modelStruct.modelPar_given_RSH=mod.given("RSH");
modelStruct.modelPar_RDSW=mod.RDSW;
modelStruct.d_modelPar_RDSW_dX=0.0;
modelStruct.modelPar_given_RDSW=mod.given("RDSW");
modelStruct.modelPar_RSW=mod.RSW;
modelStruct.d_modelPar_RSW_dX=0.0;
modelStruct.modelPar_given_RSW=mod.given("RSW");
modelStruct.modelPar_RDW=mod.RDW;
modelStruct.d_modelPar_RDW_dX=0.0;
modelStruct.modelPar_given_RDW=mod.given("RDW");
modelStruct.modelPar_RSWMIN=mod.RSWMIN;
modelStruct.d_modelPar_RSWMIN_dX=0.0;
modelStruct.modelPar_given_RSWMIN=mod.given("RSWMIN");
modelStruct.modelPar_RDWMIN=mod.RDWMIN;
modelStruct.d_modelPar_RDWMIN_dX=0.0;
modelStruct.modelPar_given_RDWMIN=mod.given("RDWMIN");
modelStruct.modelPar_PRWG=mod.PRWG;
modelStruct.d_modelPar_PRWG_dX=0.0;
modelStruct.modelPar_given_PRWG=mod.given("PRWG");
modelStruct.modelPar_PRWB=mod.PRWB;
modelStruct.d_modelPar_PRWB_dX=0.0;
modelStruct.modelPar_given_PRWB=mod.given("PRWB");
modelStruct.modelPar_PRT=mod.PRT;
modelStruct.d_modelPar_PRT_dX=0.0;
modelStruct.modelPar_given_PRT=mod.given("PRT");
modelStruct.modelPar_ETA0=mod.ETA0;
modelStruct.d_modelPar_ETA0_dX=0.0;
modelStruct.modelPar_given_ETA0=mod.given("ETA0");
modelStruct.modelPar_ETAB=mod.ETAB;
modelStruct.d_modelPar_ETAB_dX=0.0;
modelStruct.modelPar_given_ETAB=mod.given("ETAB");
modelStruct.modelPar_ETA0CV=mod.ETA0CV;
modelStruct.d_modelPar_ETA0CV_dX=0.0;
modelStruct.modelPar_given_ETA0CV=mod.given("ETA0CV");
modelStruct.modelPar_ETABCV=mod.ETABCV;
modelStruct.d_modelPar_ETABCV_dX=0.0;
modelStruct.modelPar_given_ETABCV=mod.given("ETABCV");
modelStruct.modelPar_PCLM=mod.PCLM;
modelStruct.d_modelPar_PCLM_dX=0.0;
modelStruct.modelPar_given_PCLM=mod.given("PCLM");
modelStruct.modelPar_PDIBLC1=mod.PDIBLC1;
modelStruct.d_modelPar_PDIBLC1_dX=0.0;
modelStruct.modelPar_given_PDIBLC1=mod.given("PDIBLC1");
modelStruct.modelPar_PDIBLC2=mod.PDIBLC2;
modelStruct.d_modelPar_PDIBLC2_dX=0.0;
modelStruct.modelPar_given_PDIBLC2=mod.given("PDIBLC2");
modelStruct.modelPar_PDIBLCB=mod.PDIBLCB;
modelStruct.d_modelPar_PDIBLCB_dX=0.0;
modelStruct.modelPar_given_PDIBLCB=mod.given("PDIBLCB");
modelStruct.modelPar_PVAG=mod.PVAG;
modelStruct.d_modelPar_PVAG_dX=0.0;
modelStruct.modelPar_given_PVAG=mod.given("PVAG");
modelStruct.modelPar_TBOX=mod.TBOX;
modelStruct.d_modelPar_TBOX_dX=0.0;
modelStruct.modelPar_given_TBOX=mod.given("TBOX");
modelStruct.modelPar_TSI=mod.TSI;
modelStruct.d_modelPar_TSI_dX=0.0;
modelStruct.modelPar_given_TSI=mod.given("TSI");
modelStruct.modelPar_ETSI=mod.ETSI;
modelStruct.d_modelPar_ETSI_dX=0.0;
modelStruct.modelPar_given_ETSI=mod.given("ETSI");
modelStruct.modelPar_XJ=mod.XJ;
modelStruct.d_modelPar_XJ_dX=0.0;
modelStruct.modelPar_given_XJ=mod.given("XJ");
modelStruct.modelPar_AGIDL=mod.AGIDL;
modelStruct.d_modelPar_AGIDL_dX=0.0;
modelStruct.modelPar_given_AGIDL=mod.given("AGIDL");
modelStruct.modelPar_BGIDL=mod.BGIDL;
modelStruct.d_modelPar_BGIDL_dX=0.0;
modelStruct.modelPar_given_BGIDL=mod.given("BGIDL");
modelStruct.modelPar_CGIDL=mod.CGIDL;
modelStruct.d_modelPar_CGIDL_dX=0.0;
modelStruct.modelPar_given_CGIDL=mod.given("CGIDL");
modelStruct.modelPar_RGIDL=mod.RGIDL;
modelStruct.d_modelPar_RGIDL_dX=0.0;
modelStruct.modelPar_given_RGIDL=mod.given("RGIDL");
modelStruct.modelPar_KGIDL=mod.KGIDL;
modelStruct.d_modelPar_KGIDL_dX=0.0;
modelStruct.modelPar_given_KGIDL=mod.given("KGIDL");
modelStruct.modelPar_FGIDL=mod.FGIDL;
modelStruct.d_modelPar_FGIDL_dX=0.0;
modelStruct.modelPar_given_FGIDL=mod.given("FGIDL");
modelStruct.modelPar_AGISL=mod.AGISL;
modelStruct.d_modelPar_AGISL_dX=0.0;
modelStruct.modelPar_given_AGISL=mod.given("AGISL");
modelStruct.modelPar_BGISL=mod.BGISL;
modelStruct.d_modelPar_BGISL_dX=0.0;
modelStruct.modelPar_given_BGISL=mod.given("BGISL");
modelStruct.modelPar_CGISL=mod.CGISL;
modelStruct.d_modelPar_CGISL_dX=0.0;
modelStruct.modelPar_given_CGISL=mod.given("CGISL");
modelStruct.modelPar_RGISL=mod.RGISL;
modelStruct.d_modelPar_RGISL_dX=0.0;
modelStruct.modelPar_given_RGISL=mod.given("RGISL");
modelStruct.modelPar_KGISL=mod.KGISL;
modelStruct.d_modelPar_KGISL_dX=0.0;
modelStruct.modelPar_given_KGISL=mod.given("KGISL");
modelStruct.modelPar_FGISL=mod.FGISL;
modelStruct.d_modelPar_FGISL_dX=0.0;
modelStruct.modelPar_given_FGISL=mod.given("FGISL");
modelStruct.modelPar_NDIODE=mod.NDIODE;
modelStruct.d_modelPar_NDIODE_dX=0.0;
modelStruct.modelPar_given_NDIODE=mod.given("NDIODE");
modelStruct.modelPar_NDIODED=mod.NDIODED;
modelStruct.d_modelPar_NDIODED_dX=0.0;
modelStruct.modelPar_given_NDIODED=mod.given("NDIODED");
modelStruct.modelPar_XBJT=mod.XBJT;
modelStruct.d_modelPar_XBJT_dX=0.0;
modelStruct.modelPar_given_XBJT=mod.given("XBJT");
modelStruct.modelPar_XDIF=mod.XDIF;
modelStruct.d_modelPar_XDIF_dX=0.0;
modelStruct.modelPar_given_XDIF=mod.given("XDIF");
modelStruct.modelPar_XREC=mod.XREC;
modelStruct.d_modelPar_XREC_dX=0.0;
modelStruct.modelPar_given_XREC=mod.given("XREC");
modelStruct.modelPar_XTUN=mod.XTUN;
modelStruct.d_modelPar_XTUN_dX=0.0;
modelStruct.modelPar_given_XTUN=mod.given("XTUN");
modelStruct.modelPar_XDIFD=mod.XDIFD;
modelStruct.d_modelPar_XDIFD_dX=0.0;
modelStruct.modelPar_given_XDIFD=mod.given("XDIFD");
modelStruct.modelPar_XRECD=mod.XRECD;
modelStruct.d_modelPar_XRECD_dX=0.0;
modelStruct.modelPar_given_XRECD=mod.given("XRECD");
modelStruct.modelPar_XTUND=mod.XTUND;
modelStruct.d_modelPar_XTUND_dX=0.0;
modelStruct.modelPar_given_XTUND=mod.given("XTUND");
modelStruct.modelPar_PBSWG=mod.PBSWG;
modelStruct.d_modelPar_PBSWG_dX=0.0;
modelStruct.modelPar_given_PBSWG=mod.given("PBSWG");
modelStruct.modelPar_PBSWGD=mod.PBSWGD;
modelStruct.d_modelPar_PBSWGD_dX=0.0;
modelStruct.modelPar_given_PBSWGD=mod.given("PBSWGD");
modelStruct.modelPar_MJSWG=mod.MJSWG;
modelStruct.d_modelPar_MJSWG_dX=0.0;
modelStruct.modelPar_given_MJSWG=mod.given("MJSWG");
modelStruct.modelPar_MJSWGD=mod.MJSWGD;
modelStruct.d_modelPar_MJSWGD_dX=0.0;
modelStruct.modelPar_given_MJSWGD=mod.given("MJSWGD");
modelStruct.modelPar_CJSWG=mod.CJSWG;
modelStruct.d_modelPar_CJSWG_dX=0.0;
modelStruct.modelPar_given_CJSWG=mod.given("CJSWG");
modelStruct.modelPar_CJSWGD=mod.CJSWGD;
modelStruct.d_modelPar_CJSWGD_dX=0.0;
modelStruct.modelPar_given_CJSWGD=mod.given("CJSWGD");
modelStruct.modelPar_LINT=mod.LINT;
modelStruct.d_modelPar_LINT_dX=0.0;
modelStruct.modelPar_given_LINT=mod.given("LINT");
modelStruct.modelPar_LL=mod.LL;
modelStruct.d_modelPar_LL_dX=0.0;
modelStruct.modelPar_given_LL=mod.given("LL");
modelStruct.modelPar_LLC=mod.LLC;
modelStruct.d_modelPar_LLC_dX=0.0;
modelStruct.modelPar_given_LLC=mod.given("LLC");
modelStruct.modelPar_LLN=mod.LLN;
modelStruct.d_modelPar_LLN_dX=0.0;
modelStruct.modelPar_given_LLN=mod.given("LLN");
modelStruct.modelPar_LW=mod.LW;
modelStruct.d_modelPar_LW_dX=0.0;
modelStruct.modelPar_given_LW=mod.given("LW");
modelStruct.modelPar_LWC=mod.LWC;
modelStruct.d_modelPar_LWC_dX=0.0;
modelStruct.modelPar_given_LWC=mod.given("LWC");
modelStruct.modelPar_LWN=mod.LWN;
modelStruct.d_modelPar_LWN_dX=0.0;
modelStruct.modelPar_given_LWN=mod.given("LWN");
modelStruct.modelPar_LWL=mod.LWL;
modelStruct.d_modelPar_LWL_dX=0.0;
modelStruct.modelPar_given_LWL=mod.given("LWL");
modelStruct.modelPar_LWLC=mod.LWLC;
modelStruct.d_modelPar_LWLC_dX=0.0;
modelStruct.modelPar_given_LWLC=mod.given("LWLC");
modelStruct.modelPar_WR=mod.WR;
modelStruct.d_modelPar_WR_dX=0.0;
modelStruct.modelPar_given_WR=mod.given("WR");
modelStruct.modelPar_WINT=mod.WINT;
modelStruct.d_modelPar_WINT_dX=0.0;
modelStruct.modelPar_given_WINT=mod.given("WINT");
modelStruct.modelPar_DWG=mod.DWG;
modelStruct.d_modelPar_DWG_dX=0.0;
modelStruct.modelPar_given_DWG=mod.given("DWG");
modelStruct.modelPar_DWB=mod.DWB;
modelStruct.d_modelPar_DWB_dX=0.0;
modelStruct.modelPar_given_DWB=mod.given("DWB");
modelStruct.modelPar_WL=mod.WL;
modelStruct.d_modelPar_WL_dX=0.0;
modelStruct.modelPar_given_WL=mod.given("WL");
modelStruct.modelPar_WLC=mod.WLC;
modelStruct.d_modelPar_WLC_dX=0.0;
modelStruct.modelPar_given_WLC=mod.given("WLC");
modelStruct.modelPar_WLN=mod.WLN;
modelStruct.d_modelPar_WLN_dX=0.0;
modelStruct.modelPar_given_WLN=mod.given("WLN");
modelStruct.modelPar_WW=mod.WW;
modelStruct.d_modelPar_WW_dX=0.0;
modelStruct.modelPar_given_WW=mod.given("WW");
modelStruct.modelPar_WWC=mod.WWC;
modelStruct.d_modelPar_WWC_dX=0.0;
modelStruct.modelPar_given_WWC=mod.given("WWC");
modelStruct.modelPar_WWN=mod.WWN;
modelStruct.d_modelPar_WWN_dX=0.0;
modelStruct.modelPar_given_WWN=mod.given("WWN");
modelStruct.modelPar_WWL=mod.WWL;
modelStruct.d_modelPar_WWL_dX=0.0;
modelStruct.modelPar_given_WWL=mod.given("WWL");
modelStruct.modelPar_WWLC=mod.WWLC;
modelStruct.d_modelPar_WWLC_dX=0.0;
modelStruct.modelPar_given_WWLC=mod.given("WWLC");
modelStruct.modelPar_B0=mod.B0;
modelStruct.d_modelPar_B0_dX=0.0;
modelStruct.modelPar_given_B0=mod.given("B0");
modelStruct.modelPar_B1=mod.B1;
modelStruct.d_modelPar_B1_dX=0.0;
modelStruct.modelPar_given_B1=mod.given("B1");
modelStruct.modelPar_CGSL=mod.CGSL;
modelStruct.d_modelPar_CGSL_dX=0.0;
modelStruct.modelPar_given_CGSL=mod.given("CGSL");
modelStruct.modelPar_CGDL=mod.CGDL;
modelStruct.d_modelPar_CGDL_dX=0.0;
modelStruct.modelPar_given_CGDL=mod.given("CGDL");
modelStruct.modelPar_CKAPPA=mod.CKAPPA;
modelStruct.d_modelPar_CKAPPA_dX=0.0;
modelStruct.modelPar_given_CKAPPA=mod.given("CKAPPA");
modelStruct.modelPar_CF=mod.CF;
modelStruct.d_modelPar_CF_dX=0.0;
modelStruct.modelPar_given_CF=mod.given("CF");
modelStruct.modelPar_CLC=mod.CLC;
modelStruct.d_modelPar_CLC_dX=0.0;
modelStruct.modelPar_given_CLC=mod.given("CLC");
modelStruct.modelPar_CLE=mod.CLE;
modelStruct.d_modelPar_CLE_dX=0.0;
modelStruct.modelPar_given_CLE=mod.given("CLE");
modelStruct.modelPar_DWC=mod.DWC;
modelStruct.d_modelPar_DWC_dX=0.0;
modelStruct.modelPar_given_DWC=mod.given("DWC");
modelStruct.modelPar_DLC=mod.DLC;
modelStruct.d_modelPar_DLC_dX=0.0;
modelStruct.modelPar_given_DLC=mod.given("DLC");
modelStruct.modelPar_ALPHA0=mod.ALPHA0;
modelStruct.d_modelPar_ALPHA0_dX=0.0;
modelStruct.modelPar_given_ALPHA0=mod.given("ALPHA0");
modelStruct.modelPar_NOIA=mod.NOIA;
modelStruct.d_modelPar_NOIA_dX=0.0;
modelStruct.modelPar_given_NOIA=mod.given("NOIA");
modelStruct.modelPar_NOIB=mod.NOIB;
modelStruct.d_modelPar_NOIB_dX=0.0;
modelStruct.modelPar_given_NOIB=mod.given("NOIB");
modelStruct.modelPar_NOIC=mod.NOIC;
modelStruct.d_modelPar_NOIC_dX=0.0;
modelStruct.modelPar_given_NOIC=mod.given("NOIC");
modelStruct.modelPar_TNOIA=mod.TNOIA;
modelStruct.d_modelPar_TNOIA_dX=0.0;
modelStruct.modelPar_given_TNOIA=mod.given("TNOIA");
modelStruct.modelPar_TNOIB=mod.TNOIB;
modelStruct.d_modelPar_TNOIB_dX=0.0;
modelStruct.modelPar_given_TNOIB=mod.given("TNOIB");
modelStruct.modelPar_RNOIA=mod.RNOIA;
modelStruct.d_modelPar_RNOIA_dX=0.0;
modelStruct.modelPar_given_RNOIA=mod.given("RNOIA");
modelStruct.modelPar_RNOIB=mod.RNOIB;
modelStruct.d_modelPar_RNOIB_dX=0.0;
modelStruct.modelPar_given_RNOIB=mod.given("RNOIB");
modelStruct.modelPar_NTNOI=mod.NTNOI;
modelStruct.d_modelPar_NTNOI_dX=0.0;
modelStruct.modelPar_given_NTNOI=mod.given("NTNOI");
modelStruct.modelPar_SAREF=mod.SAREF;
modelStruct.d_modelPar_SAREF_dX=0.0;
modelStruct.modelPar_given_SAREF=mod.given("SAREF");
modelStruct.modelPar_SBREF=mod.SBREF;
modelStruct.d_modelPar_SBREF_dX=0.0;
modelStruct.modelPar_given_SBREF=mod.given("SBREF");
modelStruct.modelPar_WLOD=mod.WLOD;
modelStruct.d_modelPar_WLOD_dX=0.0;
modelStruct.modelPar_given_WLOD=mod.given("WLOD");
modelStruct.modelPar_KU0=mod.KU0;
modelStruct.d_modelPar_KU0_dX=0.0;
modelStruct.modelPar_given_KU0=mod.given("KU0");
modelStruct.modelPar_KVSAT=mod.KVSAT;
modelStruct.d_modelPar_KVSAT_dX=0.0;
modelStruct.modelPar_given_KVSAT=mod.given("KVSAT");
modelStruct.modelPar_KVTH0=mod.KVTH0;
modelStruct.d_modelPar_KVTH0_dX=0.0;
modelStruct.modelPar_given_KVTH0=mod.given("KVTH0");
modelStruct.modelPar_TKU0=mod.TKU0;
modelStruct.d_modelPar_TKU0_dX=0.0;
modelStruct.modelPar_given_TKU0=mod.given("TKU0");
modelStruct.modelPar_LLODKU0=mod.LLODKU0;
modelStruct.d_modelPar_LLODKU0_dX=0.0;
modelStruct.modelPar_given_LLODKU0=mod.given("LLODKU0");
modelStruct.modelPar_WLODKU0=mod.WLODKU0;
modelStruct.d_modelPar_WLODKU0_dX=0.0;
modelStruct.modelPar_given_WLODKU0=mod.given("WLODKU0");
modelStruct.modelPar_LLODVTH=mod.LLODVTH;
modelStruct.d_modelPar_LLODVTH_dX=0.0;
modelStruct.modelPar_given_LLODVTH=mod.given("LLODVTH");
modelStruct.modelPar_WLODVTH=mod.WLODVTH;
modelStruct.d_modelPar_WLODVTH_dX=0.0;
modelStruct.modelPar_given_WLODVTH=mod.given("WLODVTH");
modelStruct.modelPar_LKU0=mod.LKU0;
modelStruct.d_modelPar_LKU0_dX=0.0;
modelStruct.modelPar_given_LKU0=mod.given("LKU0");
modelStruct.modelPar_WKU0=mod.WKU0;
modelStruct.d_modelPar_WKU0_dX=0.0;
modelStruct.modelPar_given_WKU0=mod.given("WKU0");
modelStruct.modelPar_PKU0=mod.PKU0;
modelStruct.d_modelPar_PKU0_dX=0.0;
modelStruct.modelPar_given_PKU0=mod.given("PKU0");
modelStruct.modelPar_LKVTH0=mod.LKVTH0;
modelStruct.d_modelPar_LKVTH0_dX=0.0;
modelStruct.modelPar_given_LKVTH0=mod.given("LKVTH0");
modelStruct.modelPar_WKVTH0=mod.WKVTH0;
modelStruct.d_modelPar_WKVTH0_dX=0.0;
modelStruct.modelPar_given_WKVTH0=mod.given("WKVTH0");
modelStruct.modelPar_PKVTH0=mod.PKVTH0;
modelStruct.d_modelPar_PKVTH0_dX=0.0;
modelStruct.modelPar_given_PKVTH0=mod.given("PKVTH0");
modelStruct.modelPar_STK2=mod.STK2;
modelStruct.d_modelPar_STK2_dX=0.0;
modelStruct.modelPar_given_STK2=mod.given("STK2");
modelStruct.modelPar_LODK2=mod.LODK2;
modelStruct.d_modelPar_LODK2_dX=0.0;
modelStruct.modelPar_given_LODK2=mod.given("LODK2");
modelStruct.modelPar_STETA0=mod.STETA0;
modelStruct.d_modelPar_STETA0_dX=0.0;
modelStruct.modelPar_given_STETA0=mod.given("STETA0");
modelStruct.modelPar_LODETA0=mod.LODETA0;
modelStruct.d_modelPar_LODETA0_dX=0.0;
modelStruct.modelPar_given_LODETA0=mod.given("LODETA0");
modelStruct.modelPar_STETA0CV=mod.STETA0CV;
modelStruct.d_modelPar_STETA0CV_dX=0.0;
modelStruct.modelPar_given_STETA0CV=mod.given("STETA0CV");
modelStruct.modelPar_LODETA0CV=mod.LODETA0CV;
modelStruct.d_modelPar_LODETA0CV_dX=0.0;
modelStruct.modelPar_given_LODETA0CV=mod.given("LODETA0CV");
modelStruct.modelPar_GBMIN=mod.GBMIN;
modelStruct.d_modelPar_GBMIN_dX=0.0;
modelStruct.modelPar_given_GBMIN=mod.given("GBMIN");
modelStruct.modelPar_BF=mod.BF;
modelStruct.d_modelPar_BF_dX=0.0;
modelStruct.modelPar_given_BF=mod.given("BF");
modelStruct.modelPar_W0FLK=mod.W0FLK;
modelStruct.d_modelPar_W0FLK_dX=0.0;
modelStruct.modelPar_given_W0FLK=mod.given("W0FLK");
modelStruct.modelPar_DVTP0=mod.DVTP0;
modelStruct.d_modelPar_DVTP0_dX=0.0;
modelStruct.modelPar_given_DVTP0=mod.given("DVTP0");
modelStruct.modelPar_LDVTP0=mod.LDVTP0;
modelStruct.d_modelPar_LDVTP0_dX=0.0;
modelStruct.modelPar_given_LDVTP0=mod.given("LDVTP0");
modelStruct.modelPar_WDVTP0=mod.WDVTP0;
modelStruct.d_modelPar_WDVTP0_dX=0.0;
modelStruct.modelPar_given_WDVTP0=mod.given("WDVTP0");
modelStruct.modelPar_PDVTP0=mod.PDVTP0;
modelStruct.d_modelPar_PDVTP0_dX=0.0;
modelStruct.modelPar_given_PDVTP0=mod.given("PDVTP0");
modelStruct.modelPar_DVTP1=mod.DVTP1;
modelStruct.d_modelPar_DVTP1_dX=0.0;
modelStruct.modelPar_given_DVTP1=mod.given("DVTP1");
modelStruct.modelPar_LDVTP1=mod.LDVTP1;
modelStruct.d_modelPar_LDVTP1_dX=0.0;
modelStruct.modelPar_given_LDVTP1=mod.given("LDVTP1");
modelStruct.modelPar_WDVTP1=mod.WDVTP1;
modelStruct.d_modelPar_WDVTP1_dX=0.0;
modelStruct.modelPar_given_WDVTP1=mod.given("WDVTP1");
modelStruct.modelPar_PDVTP1=mod.PDVTP1;
modelStruct.d_modelPar_PDVTP1_dX=0.0;
modelStruct.modelPar_given_PDVTP1=mod.given("PDVTP1");
modelStruct.modelPar_DVTP2=mod.DVTP2;
modelStruct.d_modelPar_DVTP2_dX=0.0;
modelStruct.modelPar_given_DVTP2=mod.given("DVTP2");
modelStruct.modelPar_LDVTP2=mod.LDVTP2;
modelStruct.d_modelPar_LDVTP2_dX=0.0;
modelStruct.modelPar_given_LDVTP2=mod.given("LDVTP2");
modelStruct.modelPar_WDVTP2=mod.WDVTP2;
modelStruct.d_modelPar_WDVTP2_dX=0.0;
modelStruct.modelPar_given_WDVTP2=mod.given("WDVTP2");
modelStruct.modelPar_PDVTP2=mod.PDVTP2;
modelStruct.d_modelPar_PDVTP2_dX=0.0;
modelStruct.modelPar_given_PDVTP2=mod.given("PDVTP2");
modelStruct.modelPar_DVTP3=mod.DVTP3;
modelStruct.d_modelPar_DVTP3_dX=0.0;
modelStruct.modelPar_given_DVTP3=mod.given("DVTP3");
modelStruct.modelPar_LDVTP3=mod.LDVTP3;
modelStruct.d_modelPar_LDVTP3_dX=0.0;
modelStruct.modelPar_given_LDVTP3=mod.given("LDVTP3");
modelStruct.modelPar_WDVTP3=mod.WDVTP3;
modelStruct.d_modelPar_WDVTP3_dX=0.0;
modelStruct.modelPar_given_WDVTP3=mod.given("WDVTP3");
modelStruct.modelPar_PDVTP3=mod.PDVTP3;
modelStruct.d_modelPar_PDVTP3_dX=0.0;
modelStruct.modelPar_given_PDVTP3=mod.given("PDVTP3");
modelStruct.modelPar_DVTP4=mod.DVTP4;
modelStruct.d_modelPar_DVTP4_dX=0.0;
modelStruct.modelPar_given_DVTP4=mod.given("DVTP4");
modelStruct.modelPar_LDVTP4=mod.LDVTP4;
modelStruct.d_modelPar_LDVTP4_dX=0.0;
modelStruct.modelPar_given_LDVTP4=mod.given("LDVTP4");
modelStruct.modelPar_WDVTP4=mod.WDVTP4;
modelStruct.d_modelPar_WDVTP4_dX=0.0;
modelStruct.modelPar_given_WDVTP4=mod.given("WDVTP4");
modelStruct.modelPar_PDVTP4=mod.PDVTP4;
modelStruct.d_modelPar_PDVTP4_dX=0.0;
modelStruct.modelPar_given_PDVTP4=mod.given("PDVTP4");
modelStruct.modelPar_MINV=mod.MINV;
modelStruct.d_modelPar_MINV_dX=0.0;
modelStruct.modelPar_given_MINV=mod.given("MINV");
modelStruct.modelPar_LMINV=mod.LMINV;
modelStruct.d_modelPar_LMINV_dX=0.0;
modelStruct.modelPar_given_LMINV=mod.given("LMINV");
modelStruct.modelPar_WMINV=mod.WMINV;
modelStruct.d_modelPar_WMINV_dX=0.0;
modelStruct.modelPar_given_WMINV=mod.given("WMINV");
modelStruct.modelPar_PMINV=mod.PMINV;
modelStruct.d_modelPar_PMINV_dX=0.0;
modelStruct.modelPar_given_PMINV=mod.given("PMINV");
modelStruct.modelPar_PDITS=mod.PDITS;
modelStruct.d_modelPar_PDITS_dX=0.0;
modelStruct.modelPar_given_PDITS=mod.given("PDITS");
modelStruct.modelPar_PDITSL=mod.PDITSL;
modelStruct.d_modelPar_PDITSL_dX=0.0;
modelStruct.modelPar_given_PDITSL=mod.given("PDITSL");
modelStruct.modelPar_PDITSD=mod.PDITSD;
modelStruct.d_modelPar_PDITSD_dX=0.0;
modelStruct.modelPar_given_PDITSD=mod.given("PDITSD");
modelStruct.modelPar_FPROUT=mod.FPROUT;
modelStruct.d_modelPar_FPROUT_dX=0.0;
modelStruct.modelPar_given_FPROUT=mod.given("FPROUT");
modelStruct.modelPar_LFPROUT=mod.LFPROUT;
modelStruct.d_modelPar_LFPROUT_dX=0.0;
modelStruct.modelPar_given_LFPROUT=mod.given("LFPROUT");
modelStruct.modelPar_LPDITS=mod.LPDITS;
modelStruct.d_modelPar_LPDITS_dX=0.0;
modelStruct.modelPar_given_LPDITS=mod.given("LPDITS");
modelStruct.modelPar_LPDITSD=mod.LPDITSD;
modelStruct.d_modelPar_LPDITSD_dX=0.0;
modelStruct.modelPar_given_LPDITSD=mod.given("LPDITSD");
modelStruct.modelPar_WFPROUT=mod.WFPROUT;
modelStruct.d_modelPar_WFPROUT_dX=0.0;
modelStruct.modelPar_given_WFPROUT=mod.given("WFPROUT");
modelStruct.modelPar_WPDITS=mod.WPDITS;
modelStruct.d_modelPar_WPDITS_dX=0.0;
modelStruct.modelPar_given_WPDITS=mod.given("WPDITS");
modelStruct.modelPar_WPDITSD=mod.WPDITSD;
modelStruct.d_modelPar_WPDITSD_dX=0.0;
modelStruct.modelPar_given_WPDITSD=mod.given("WPDITSD");
modelStruct.modelPar_PFPROUT=mod.PFPROUT;
modelStruct.d_modelPar_PFPROUT_dX=0.0;
modelStruct.modelPar_given_PFPROUT=mod.given("PFPROUT");
modelStruct.modelPar_PPDITS=mod.PPDITS;
modelStruct.d_modelPar_PPDITS_dX=0.0;
modelStruct.modelPar_given_PPDITS=mod.given("PPDITS");
modelStruct.modelPar_PPDITSD=mod.PPDITSD;
modelStruct.d_modelPar_PPDITSD_dX=0.0;
modelStruct.modelPar_given_PPDITSD=mod.given("PPDITSD");
modelStruct.modelPar_EM=mod.EM;
modelStruct.d_modelPar_EM_dX=0.0;
modelStruct.modelPar_given_EM=mod.given("EM");
modelStruct.modelPar_EF=mod.EF;
modelStruct.d_modelPar_EF_dX=0.0;
modelStruct.modelPar_given_EF=mod.given("EF");
modelStruct.modelPar_AF=mod.AF;
modelStruct.d_modelPar_AF_dX=0.0;
modelStruct.modelPar_given_AF=mod.given("AF");
modelStruct.modelPar_KF=mod.KF;
modelStruct.d_modelPar_KF_dX=0.0;
modelStruct.modelPar_given_KF=mod.given("KF");
modelStruct.modelPar_NOIF=mod.NOIF;
modelStruct.d_modelPar_NOIF_dX=0.0;
modelStruct.modelPar_given_NOIF=mod.given("NOIF");
modelStruct.modelPar_K1W1=mod.K1W1;
modelStruct.d_modelPar_K1W1_dX=0.0;
modelStruct.modelPar_given_K1W1=mod.given("K1W1");
modelStruct.modelPar_K1W2=mod.K1W2;
modelStruct.d_modelPar_K1W2_dX=0.0;
modelStruct.modelPar_given_K1W2=mod.given("K1W2");
modelStruct.modelPar_KETAS=mod.KETAS;
modelStruct.d_modelPar_KETAS_dX=0.0;
modelStruct.modelPar_given_KETAS=mod.given("KETAS");
modelStruct.modelPar_DWBC=mod.DWBC;
modelStruct.d_modelPar_DWBC_dX=0.0;
modelStruct.modelPar_given_DWBC=mod.given("DWBC");
modelStruct.modelPar_BETA0=mod.BETA0;
modelStruct.d_modelPar_BETA0_dX=0.0;
modelStruct.modelPar_given_BETA0=mod.given("BETA0");
modelStruct.modelPar_BETA1=mod.BETA1;
modelStruct.d_modelPar_BETA1_dX=0.0;
modelStruct.modelPar_given_BETA1=mod.given("BETA1");
modelStruct.modelPar_BETA2=mod.BETA2;
modelStruct.d_modelPar_BETA2_dX=0.0;
modelStruct.modelPar_given_BETA2=mod.given("BETA2");
modelStruct.modelPar_VDSATII0=mod.VDSATII0;
modelStruct.d_modelPar_VDSATII0_dX=0.0;
modelStruct.modelPar_given_VDSATII0=mod.given("VDSATII0");
modelStruct.modelPar_TII=mod.TII;
modelStruct.d_modelPar_TII_dX=0.0;
modelStruct.modelPar_given_TII=mod.given("TII");
modelStruct.modelPar_LII=mod.LII;
modelStruct.d_modelPar_LII_dX=0.0;
modelStruct.modelPar_given_LII=mod.given("LII");
modelStruct.modelPar_SII0=mod.SII0;
modelStruct.d_modelPar_SII0_dX=0.0;
modelStruct.modelPar_given_SII0=mod.given("SII0");
modelStruct.modelPar_SII1=mod.SII1;
modelStruct.d_modelPar_SII1_dX=0.0;
modelStruct.modelPar_given_SII1=mod.given("SII1");
modelStruct.modelPar_SII2=mod.SII2;
modelStruct.d_modelPar_SII2_dX=0.0;
modelStruct.modelPar_given_SII2=mod.given("SII2");
modelStruct.modelPar_SIID=mod.SIID;
modelStruct.d_modelPar_SIID_dX=0.0;
modelStruct.modelPar_given_SIID=mod.given("SIID");
modelStruct.modelPar_FBJTII=mod.FBJTII;
modelStruct.d_modelPar_FBJTII_dX=0.0;
modelStruct.modelPar_given_FBJTII=mod.given("FBJTII");
modelStruct.modelPar_EBJTII=mod.EBJTII;
modelStruct.d_modelPar_EBJTII_dX=0.0;
modelStruct.modelPar_given_EBJTII=mod.given("EBJTII");
modelStruct.modelPar_CBJTII=mod.CBJTII;
modelStruct.d_modelPar_CBJTII_dX=0.0;
modelStruct.modelPar_given_CBJTII=mod.given("CBJTII");
modelStruct.modelPar_VBCI=mod.VBCI;
modelStruct.d_modelPar_VBCI_dX=0.0;
modelStruct.modelPar_given_VBCI=mod.given("VBCI");
modelStruct.modelPar_ABJTII=mod.ABJTII;
modelStruct.d_modelPar_ABJTII_dX=0.0;
modelStruct.modelPar_given_ABJTII=mod.given("ABJTII");
modelStruct.modelPar_MBJTII=mod.MBJTII;
modelStruct.d_modelPar_MBJTII_dX=0.0;
modelStruct.modelPar_given_MBJTII=mod.given("MBJTII");
modelStruct.modelPar_TVBCI=mod.TVBCI;
modelStruct.d_modelPar_TVBCI_dX=0.0;
modelStruct.modelPar_given_TVBCI=mod.given("TVBCI");
modelStruct.modelPar_ESATII=mod.ESATII;
modelStruct.d_modelPar_ESATII_dX=0.0;
modelStruct.modelPar_given_ESATII=mod.given("ESATII");
modelStruct.modelPar_NTUN=mod.NTUN;
modelStruct.d_modelPar_NTUN_dX=0.0;
modelStruct.modelPar_given_NTUN=mod.given("NTUN");
modelStruct.modelPar_NTUND=mod.NTUND;
modelStruct.d_modelPar_NTUND_dX=0.0;
modelStruct.modelPar_given_NTUND=mod.given("NTUND");
modelStruct.modelPar_NRECF0=mod.NRECF0;
modelStruct.d_modelPar_NRECF0_dX=0.0;
modelStruct.modelPar_given_NRECF0=mod.given("NRECF0");
modelStruct.modelPar_NRECF0D=mod.NRECF0D;
modelStruct.d_modelPar_NRECF0D_dX=0.0;
modelStruct.modelPar_given_NRECF0D=mod.given("NRECF0D");
modelStruct.modelPar_NRECR0=mod.NRECR0;
modelStruct.d_modelPar_NRECR0_dX=0.0;
modelStruct.modelPar_given_NRECR0=mod.given("NRECR0");
modelStruct.modelPar_NRECR0D=mod.NRECR0D;
modelStruct.d_modelPar_NRECR0D_dX=0.0;
modelStruct.modelPar_given_NRECR0D=mod.given("NRECR0D");
modelStruct.modelPar_ISBJT=mod.ISBJT;
modelStruct.d_modelPar_ISBJT_dX=0.0;
modelStruct.modelPar_given_ISBJT=mod.given("ISBJT");
modelStruct.modelPar_IDBJT=mod.IDBJT;
modelStruct.d_modelPar_IDBJT_dX=0.0;
modelStruct.modelPar_given_IDBJT=mod.given("IDBJT");
modelStruct.modelPar_ISDIF=mod.ISDIF;
modelStruct.d_modelPar_ISDIF_dX=0.0;
modelStruct.modelPar_given_ISDIF=mod.given("ISDIF");
modelStruct.modelPar_IDDIF=mod.IDDIF;
modelStruct.d_modelPar_IDDIF_dX=0.0;
modelStruct.modelPar_given_IDDIF=mod.given("IDDIF");
modelStruct.modelPar_ISREC=mod.ISREC;
modelStruct.d_modelPar_ISREC_dX=0.0;
modelStruct.modelPar_given_ISREC=mod.given("ISREC");
modelStruct.modelPar_IDREC=mod.IDREC;
modelStruct.d_modelPar_IDREC_dX=0.0;
modelStruct.modelPar_given_IDREC=mod.given("IDREC");
modelStruct.modelPar_ISTUN=mod.ISTUN;
modelStruct.d_modelPar_ISTUN_dX=0.0;
modelStruct.modelPar_given_ISTUN=mod.given("ISTUN");
modelStruct.modelPar_IDTUN=mod.IDTUN;
modelStruct.d_modelPar_IDTUN_dX=0.0;
modelStruct.modelPar_given_IDTUN=mod.given("IDTUN");
modelStruct.modelPar_LN=mod.LN;
modelStruct.d_modelPar_LN_dX=0.0;
modelStruct.modelPar_given_LN=mod.given("LN");
modelStruct.modelPar_VREC0=mod.VREC0;
modelStruct.d_modelPar_VREC0_dX=0.0;
modelStruct.modelPar_given_VREC0=mod.given("VREC0");
modelStruct.modelPar_VREC0D=mod.VREC0D;
modelStruct.d_modelPar_VREC0D_dX=0.0;
modelStruct.modelPar_given_VREC0D=mod.given("VREC0D");
modelStruct.modelPar_VTUN0=mod.VTUN0;
modelStruct.d_modelPar_VTUN0_dX=0.0;
modelStruct.modelPar_given_VTUN0=mod.given("VTUN0");
modelStruct.modelPar_VTUN0D=mod.VTUN0D;
modelStruct.d_modelPar_VTUN0D_dX=0.0;
modelStruct.modelPar_given_VTUN0D=mod.given("VTUN0D");
modelStruct.modelPar_NBJT=mod.NBJT;
modelStruct.d_modelPar_NBJT_dX=0.0;
modelStruct.modelPar_given_NBJT=mod.given("NBJT");
modelStruct.modelPar_LBJT0=mod.LBJT0;
modelStruct.d_modelPar_LBJT0_dX=0.0;
modelStruct.modelPar_given_LBJT0=mod.given("LBJT0");
modelStruct.modelPar_LDIF0=mod.LDIF0;
modelStruct.d_modelPar_LDIF0_dX=0.0;
modelStruct.modelPar_given_LDIF0=mod.given("LDIF0");
modelStruct.modelPar_VABJT=mod.VABJT;
modelStruct.d_modelPar_VABJT_dX=0.0;
modelStruct.modelPar_given_VABJT=mod.given("VABJT");
modelStruct.modelPar_AELY=mod.AELY;
modelStruct.d_modelPar_AELY_dX=0.0;
modelStruct.modelPar_given_AELY=mod.given("AELY");
modelStruct.modelPar_AHLI=mod.AHLI;
modelStruct.d_modelPar_AHLI_dX=0.0;
modelStruct.modelPar_given_AHLI=mod.given("AHLI");
modelStruct.modelPar_AHLID=mod.AHLID;
modelStruct.d_modelPar_AHLID_dX=0.0;
modelStruct.modelPar_given_AHLID=mod.given("AHLID");
modelStruct.modelPar_RBODY=mod.RBODY;
modelStruct.d_modelPar_RBODY_dX=0.0;
modelStruct.modelPar_given_RBODY=mod.given("RBODY");
modelStruct.modelPar_RBSH=mod.RBSH;
modelStruct.d_modelPar_RBSH_dX=0.0;
modelStruct.modelPar_given_RBSH=mod.given("RBSH");
modelStruct.modelPar_CGEO=mod.CGEO;
modelStruct.d_modelPar_CGEO_dX=0.0;
modelStruct.modelPar_given_CGEO=mod.given("CGEO");
modelStruct.modelPar_TT=mod.TT;
modelStruct.d_modelPar_TT_dX=0.0;
modelStruct.modelPar_given_TT=mod.given("TT");
modelStruct.modelPar_NDIF=mod.NDIF;
modelStruct.d_modelPar_NDIF_dX=0.0;
modelStruct.modelPar_given_NDIF=mod.given("NDIF");
modelStruct.modelPar_VSDFB=mod.VSDFB;
modelStruct.d_modelPar_VSDFB_dX=0.0;
modelStruct.modelPar_given_VSDFB=mod.given("VSDFB");
modelStruct.modelPar_VSDTH=mod.VSDTH;
modelStruct.d_modelPar_VSDTH_dX=0.0;
modelStruct.modelPar_given_VSDTH=mod.given("VSDTH");
modelStruct.modelPar_CSDMIN=mod.CSDMIN;
modelStruct.d_modelPar_CSDMIN_dX=0.0;
modelStruct.modelPar_given_CSDMIN=mod.given("CSDMIN");
modelStruct.modelPar_ASD=mod.ASD;
modelStruct.d_modelPar_ASD_dX=0.0;
modelStruct.modelPar_given_ASD=mod.given("ASD");
modelStruct.modelPar_CSDESW=mod.CSDESW;
modelStruct.d_modelPar_CSDESW_dX=0.0;
modelStruct.modelPar_given_CSDESW=mod.given("CSDESW");
modelStruct.modelPar_NTRECF=mod.NTRECF;
modelStruct.d_modelPar_NTRECF_dX=0.0;
modelStruct.modelPar_given_NTRECF=mod.given("NTRECF");
modelStruct.modelPar_NTRECR=mod.NTRECR;
modelStruct.d_modelPar_NTRECR_dX=0.0;
modelStruct.modelPar_given_NTRECR=mod.given("NTRECR");
modelStruct.modelPar_DLCB=mod.DLCB;
modelStruct.d_modelPar_DLCB_dX=0.0;
modelStruct.modelPar_given_DLCB=mod.given("DLCB");
modelStruct.modelPar_FBODY=mod.FBODY;
modelStruct.d_modelPar_FBODY_dX=0.0;
modelStruct.modelPar_given_FBODY=mod.given("FBODY");
modelStruct.modelPar_TCJSWG=mod.TCJSWG;
modelStruct.d_modelPar_TCJSWG_dX=0.0;
modelStruct.modelPar_given_TCJSWG=mod.given("TCJSWG");
modelStruct.modelPar_TPBSWG=mod.TPBSWG;
modelStruct.d_modelPar_TPBSWG_dX=0.0;
modelStruct.modelPar_given_TPBSWG=mod.given("TPBSWG");
modelStruct.modelPar_TCJSWGD=mod.TCJSWGD;
modelStruct.d_modelPar_TCJSWGD_dX=0.0;
modelStruct.modelPar_given_TCJSWGD=mod.given("TCJSWGD");
modelStruct.modelPar_TPBSWGD=mod.TPBSWGD;
modelStruct.d_modelPar_TPBSWGD_dX=0.0;
modelStruct.modelPar_given_TPBSWGD=mod.given("TPBSWGD");
modelStruct.modelPar_ACDE=mod.ACDE;
modelStruct.d_modelPar_ACDE_dX=0.0;
modelStruct.modelPar_given_ACDE=mod.given("ACDE");
modelStruct.modelPar_MOIN=mod.MOIN;
modelStruct.d_modelPar_MOIN_dX=0.0;
modelStruct.modelPar_given_MOIN=mod.given("MOIN");
modelStruct.modelPar_NOFF=mod.NOFF;
modelStruct.d_modelPar_NOFF_dX=0.0;
modelStruct.modelPar_given_NOFF=mod.given("NOFF");
modelStruct.modelPar_NOFF2=mod.NOFF2;
modelStruct.d_modelPar_NOFF2_dX=0.0;
modelStruct.modelPar_given_NOFF2=mod.given("NOFF2");
modelStruct.modelPar_DELVT=mod.DELVT;
modelStruct.d_modelPar_DELVT_dX=0.0;
modelStruct.modelPar_given_DELVT=mod.given("DELVT");
modelStruct.modelPar_KB1=mod.KB1;
modelStruct.d_modelPar_KB1_dX=0.0;
modelStruct.modelPar_given_KB1=mod.given("KB1");
modelStruct.modelPar_DLBG=mod.DLBG;
modelStruct.d_modelPar_DLBG_dX=0.0;
modelStruct.modelPar_given_DLBG=mod.given("DLBG");
modelStruct.modelPar_CFRCOEFF=mod.CFRCOEFF;
modelStruct.d_modelPar_CFRCOEFF_dX=0.0;
modelStruct.modelPar_given_CFRCOEFF=mod.given("CFRCOEFF");
modelStruct.modelPar_TOXQM=mod.TOXQM;
modelStruct.d_modelPar_TOXQM_dX=0.0;
modelStruct.modelPar_given_TOXQM=mod.given("TOXQM");
modelStruct.modelPar_WTH0=mod.WTH0;
modelStruct.d_modelPar_WTH0_dX=0.0;
modelStruct.modelPar_given_WTH0=mod.given("WTH0");
modelStruct.modelPar_RHALO=mod.RHALO;
modelStruct.d_modelPar_RHALO_dX=0.0;
modelStruct.modelPar_given_RHALO=mod.given("RHALO");
modelStruct.modelPar_NTOX=mod.NTOX;
modelStruct.d_modelPar_NTOX_dX=0.0;
modelStruct.modelPar_given_NTOX=mod.given("NTOX");
modelStruct.modelPar_TOXREF=mod.TOXREF;
modelStruct.d_modelPar_TOXREF_dX=0.0;
modelStruct.modelPar_given_TOXREF=mod.given("TOXREF");
modelStruct.modelPar_EBG=mod.EBG;
modelStruct.d_modelPar_EBG_dX=0.0;
modelStruct.modelPar_given_EBG=mod.given("EBG");
modelStruct.modelPar_VEVB=mod.VEVB;
modelStruct.d_modelPar_VEVB_dX=0.0;
modelStruct.modelPar_given_VEVB=mod.given("VEVB");
modelStruct.modelPar_ALPHAGB1=mod.ALPHAGB1;
modelStruct.d_modelPar_ALPHAGB1_dX=0.0;
modelStruct.modelPar_given_ALPHAGB1=mod.given("ALPHAGB1");
modelStruct.modelPar_BETAGB1=mod.BETAGB1;
modelStruct.d_modelPar_BETAGB1_dX=0.0;
modelStruct.modelPar_given_BETAGB1=mod.given("BETAGB1");
modelStruct.modelPar_VGB1=mod.VGB1;
modelStruct.d_modelPar_VGB1_dX=0.0;
modelStruct.modelPar_given_VGB1=mod.given("VGB1");
modelStruct.modelPar_VECB=mod.VECB;
modelStruct.d_modelPar_VECB_dX=0.0;
modelStruct.modelPar_given_VECB=mod.given("VECB");
modelStruct.modelPar_ALPHAGB2=mod.ALPHAGB2;
modelStruct.d_modelPar_ALPHAGB2_dX=0.0;
modelStruct.modelPar_given_ALPHAGB2=mod.given("ALPHAGB2");
modelStruct.modelPar_BETAGB2=mod.BETAGB2;
modelStruct.d_modelPar_BETAGB2_dX=0.0;
modelStruct.modelPar_given_BETAGB2=mod.given("BETAGB2");
modelStruct.modelPar_VGB2=mod.VGB2;
modelStruct.d_modelPar_VGB2_dX=0.0;
modelStruct.modelPar_given_VGB2=mod.given("VGB2");
modelStruct.modelPar_AIGBCP2=mod.AIGBCP2;
modelStruct.d_modelPar_AIGBCP2_dX=0.0;
modelStruct.modelPar_given_AIGBCP2=mod.given("AIGBCP2");
modelStruct.modelPar_BIGBCP2=mod.BIGBCP2;
modelStruct.d_modelPar_BIGBCP2_dX=0.0;
modelStruct.modelPar_given_BIGBCP2=mod.given("BIGBCP2");
modelStruct.modelPar_CIGBCP2=mod.CIGBCP2;
modelStruct.d_modelPar_CIGBCP2_dX=0.0;
modelStruct.modelPar_given_CIGBCP2=mod.given("CIGBCP2");
modelStruct.modelPar_VOXH=mod.VOXH;
modelStruct.d_modelPar_VOXH_dX=0.0;
modelStruct.modelPar_given_VOXH=mod.given("VOXH");
modelStruct.modelPar_DELTAVOX=mod.DELTAVOX;
modelStruct.d_modelPar_DELTAVOX_dX=0.0;
modelStruct.modelPar_given_DELTAVOX=mod.given("DELTAVOX");
modelStruct.modelPar_AIGC=mod.AIGC;
modelStruct.d_modelPar_AIGC_dX=0.0;
modelStruct.modelPar_given_AIGC=mod.given("AIGC");
modelStruct.modelPar_BIGC=mod.BIGC;
modelStruct.d_modelPar_BIGC_dX=0.0;
modelStruct.modelPar_given_BIGC=mod.given("BIGC");
modelStruct.modelPar_CIGC=mod.CIGC;
modelStruct.d_modelPar_CIGC_dX=0.0;
modelStruct.modelPar_given_CIGC=mod.given("CIGC");
modelStruct.modelPar_AIGSD=mod.AIGSD;
modelStruct.d_modelPar_AIGSD_dX=0.0;
modelStruct.modelPar_given_AIGSD=mod.given("AIGSD");
modelStruct.modelPar_BIGSD=mod.BIGSD;
modelStruct.d_modelPar_BIGSD_dX=0.0;
modelStruct.modelPar_given_BIGSD=mod.given("BIGSD");
modelStruct.modelPar_CIGSD=mod.CIGSD;
modelStruct.d_modelPar_CIGSD_dX=0.0;
modelStruct.modelPar_given_CIGSD=mod.given("CIGSD");
modelStruct.modelPar_NIGC=mod.NIGC;
modelStruct.d_modelPar_NIGC_dX=0.0;
modelStruct.modelPar_given_NIGC=mod.given("NIGC");
modelStruct.modelPar_PIGCD=mod.PIGCD;
modelStruct.d_modelPar_PIGCD_dX=0.0;
modelStruct.modelPar_given_PIGCD=mod.given("PIGCD");
modelStruct.modelPar_POXEDGE=mod.POXEDGE;
modelStruct.d_modelPar_POXEDGE_dX=0.0;
modelStruct.modelPar_given_POXEDGE=mod.given("POXEDGE");
modelStruct.modelPar_DLCIG=mod.DLCIG;
modelStruct.d_modelPar_DLCIG_dX=0.0;
modelStruct.modelPar_given_DLCIG=mod.given("DLCIG");
modelStruct.modelPar_VBS0PD=mod.VBS0PD;
modelStruct.d_modelPar_VBS0PD_dX=0.0;
modelStruct.modelPar_given_VBS0PD=mod.given("VBS0PD");
modelStruct.modelPar_VBS0FD=mod.VBS0FD;
modelStruct.d_modelPar_VBS0FD_dX=0.0;
modelStruct.modelPar_given_VBS0FD=mod.given("VBS0FD");
modelStruct.modelPar_VBSA=mod.VBSA;
modelStruct.d_modelPar_VBSA_dX=0.0;
modelStruct.modelPar_given_VBSA=mod.given("VBSA");
modelStruct.modelPar_NOFFFD=mod.NOFFFD;
modelStruct.d_modelPar_NOFFFD_dX=0.0;
modelStruct.modelPar_given_NOFFFD=mod.given("NOFFFD");
modelStruct.modelPar_VOFFFD=mod.VOFFFD;
modelStruct.d_modelPar_VOFFFD_dX=0.0;
modelStruct.modelPar_given_VOFFFD=mod.given("VOFFFD");
modelStruct.modelPar_K1B=mod.K1B;
modelStruct.d_modelPar_K1B_dX=0.0;
modelStruct.modelPar_given_K1B=mod.given("K1B");
modelStruct.modelPar_K2B=mod.K2B;
modelStruct.d_modelPar_K2B_dX=0.0;
modelStruct.modelPar_given_K2B=mod.given("K2B");
modelStruct.modelPar_DK2B=mod.DK2B;
modelStruct.d_modelPar_DK2B_dX=0.0;
modelStruct.modelPar_given_DK2B=mod.given("DK2B");
modelStruct.modelPar_DVBD0=mod.DVBD0;
modelStruct.d_modelPar_DVBD0_dX=0.0;
modelStruct.modelPar_given_DVBD0=mod.given("DVBD0");
modelStruct.modelPar_DVBD1=mod.DVBD1;
modelStruct.d_modelPar_DVBD1_dX=0.0;
modelStruct.modelPar_given_DVBD1=mod.given("DVBD1");
modelStruct.modelPar_MOINFD=mod.MOINFD;
modelStruct.d_modelPar_MOINFD_dX=0.0;
modelStruct.modelPar_given_MOINFD=mod.given("MOINFD");
modelStruct.modelPar_XRCRG1=mod.XRCRG1;
modelStruct.d_modelPar_XRCRG1_dX=0.0;
modelStruct.modelPar_given_XRCRG1=mod.given("XRCRG1");
modelStruct.modelPar_XRCRG2=mod.XRCRG2;
modelStruct.d_modelPar_XRCRG2_dX=0.0;
modelStruct.modelPar_given_XRCRG2=mod.given("XRCRG2");
modelStruct.modelPar_RSHG=mod.RSHG;
modelStruct.d_modelPar_RSHG_dX=0.0;
modelStruct.modelPar_given_RSHG=mod.given("RSHG");
modelStruct.modelPar_NGCON=mod.NGCON;
modelStruct.d_modelPar_NGCON_dX=0.0;
modelStruct.modelPar_given_NGCON=mod.given("NGCON");
modelStruct.modelPar_XGW=mod.XGW;
modelStruct.d_modelPar_XGW_dX=0.0;
modelStruct.modelPar_given_XGW=mod.given("XGW");
modelStruct.modelPar_XGL=mod.XGL;
modelStruct.d_modelPar_XGL_dX=0.0;
modelStruct.modelPar_given_XGL=mod.given("XGL");
modelStruct.modelPar_VSCE=mod.VSCE;
modelStruct.d_modelPar_VSCE_dX=0.0;
modelStruct.modelPar_given_VSCE=mod.given("VSCE");
modelStruct.modelPar_CDSBS=mod.CDSBS;
modelStruct.d_modelPar_CDSBS_dX=0.0;
modelStruct.modelPar_given_CDSBS=mod.given("CDSBS");
modelStruct.modelPar_MINVCV=mod.MINVCV;
modelStruct.d_modelPar_MINVCV_dX=0.0;
modelStruct.modelPar_given_MINVCV=mod.given("MINVCV");
modelStruct.modelPar_LMINVCV=mod.LMINVCV;
modelStruct.d_modelPar_LMINVCV_dX=0.0;
modelStruct.modelPar_given_LMINVCV=mod.given("LMINVCV");
modelStruct.modelPar_WMINVCV=mod.WMINVCV;
modelStruct.d_modelPar_WMINVCV_dX=0.0;
modelStruct.modelPar_given_WMINVCV=mod.given("WMINVCV");
modelStruct.modelPar_PMINVCV=mod.PMINVCV;
modelStruct.d_modelPar_PMINVCV_dX=0.0;
modelStruct.modelPar_given_PMINVCV=mod.given("PMINVCV");
modelStruct.modelPar_VOFFCV=mod.VOFFCV;
modelStruct.d_modelPar_VOFFCV_dX=0.0;
modelStruct.modelPar_given_VOFFCV=mod.given("VOFFCV");
modelStruct.modelPar_LVOFFCV=mod.LVOFFCV;
modelStruct.d_modelPar_LVOFFCV_dX=0.0;
modelStruct.modelPar_given_LVOFFCV=mod.given("LVOFFCV");
modelStruct.modelPar_WVOFFCV=mod.WVOFFCV;
modelStruct.d_modelPar_WVOFFCV_dX=0.0;
modelStruct.modelPar_given_WVOFFCV=mod.given("WVOFFCV");
modelStruct.modelPar_PVOFFCV=mod.PVOFFCV;
modelStruct.d_modelPar_PVOFFCV_dX=0.0;
modelStruct.modelPar_given_PVOFFCV=mod.given("PVOFFCV");
modelStruct.modelPar_LXJ=mod.LXJ;
modelStruct.d_modelPar_LXJ_dX=0.0;
modelStruct.modelPar_given_LXJ=mod.given("LXJ");
modelStruct.modelPar_LALPHAGB1=mod.LALPHAGB1;
modelStruct.d_modelPar_LALPHAGB1_dX=0.0;
modelStruct.modelPar_given_LALPHAGB1=mod.given("LALPHAGB1");
modelStruct.modelPar_LBETAGB1=mod.LBETAGB1;
modelStruct.d_modelPar_LBETAGB1_dX=0.0;
modelStruct.modelPar_given_LBETAGB1=mod.given("LBETAGB1");
modelStruct.modelPar_LALPHAGB2=mod.LALPHAGB2;
modelStruct.d_modelPar_LALPHAGB2_dX=0.0;
modelStruct.modelPar_given_LALPHAGB2=mod.given("LALPHAGB2");
modelStruct.modelPar_LBETAGB2=mod.LBETAGB2;
modelStruct.d_modelPar_LBETAGB2_dX=0.0;
modelStruct.modelPar_given_LBETAGB2=mod.given("LBETAGB2");
modelStruct.modelPar_LAIGBCP2=mod.LAIGBCP2;
modelStruct.d_modelPar_LAIGBCP2_dX=0.0;
modelStruct.modelPar_given_LAIGBCP2=mod.given("LAIGBCP2");
modelStruct.modelPar_LBIGBCP2=mod.LBIGBCP2;
modelStruct.d_modelPar_LBIGBCP2_dX=0.0;
modelStruct.modelPar_given_LBIGBCP2=mod.given("LBIGBCP2");
modelStruct.modelPar_LCIGBCP2=mod.LCIGBCP2;
modelStruct.d_modelPar_LCIGBCP2_dX=0.0;
modelStruct.modelPar_given_LCIGBCP2=mod.given("LCIGBCP2");
modelStruct.modelPar_LCGSL=mod.LCGSL;
modelStruct.d_modelPar_LCGSL_dX=0.0;
modelStruct.modelPar_given_LCGSL=mod.given("LCGSL");
modelStruct.modelPar_LCGDL=mod.LCGDL;
modelStruct.d_modelPar_LCGDL_dX=0.0;
modelStruct.modelPar_given_LCGDL=mod.given("LCGDL");
modelStruct.modelPar_LCKAPPA=mod.LCKAPPA;
modelStruct.d_modelPar_LCKAPPA_dX=0.0;
modelStruct.modelPar_given_LCKAPPA=mod.given("LCKAPPA");
modelStruct.modelPar_LNDIF=mod.LNDIF;
modelStruct.d_modelPar_LNDIF_dX=0.0;
modelStruct.modelPar_given_LNDIF=mod.given("LNDIF");
modelStruct.modelPar_LUTE=mod.LUTE;
modelStruct.d_modelPar_LUTE_dX=0.0;
modelStruct.modelPar_given_LUTE=mod.given("LUTE");
modelStruct.modelPar_LKT1=mod.LKT1;
modelStruct.d_modelPar_LKT1_dX=0.0;
modelStruct.modelPar_given_LKT1=mod.given("LKT1");
modelStruct.modelPar_LKT1L=mod.LKT1L;
modelStruct.d_modelPar_LKT1L_dX=0.0;
modelStruct.modelPar_given_LKT1L=mod.given("LKT1L");
modelStruct.modelPar_LKT2=mod.LKT2;
modelStruct.d_modelPar_LKT2_dX=0.0;
modelStruct.modelPar_given_LKT2=mod.given("LKT2");
modelStruct.modelPar_LUA1=mod.LUA1;
modelStruct.d_modelPar_LUA1_dX=0.0;
modelStruct.modelPar_given_LUA1=mod.given("LUA1");
modelStruct.modelPar_LUB1=mod.LUB1;
modelStruct.d_modelPar_LUB1_dX=0.0;
modelStruct.modelPar_given_LUB1=mod.given("LUB1");
modelStruct.modelPar_LUC1=mod.LUC1;
modelStruct.d_modelPar_LUC1_dX=0.0;
modelStruct.modelPar_given_LUC1=mod.given("LUC1");
modelStruct.modelPar_LAT=mod.LAT;
modelStruct.d_modelPar_LAT_dX=0.0;
modelStruct.modelPar_given_LAT=mod.given("LAT");
modelStruct.modelPar_LPRT=mod.LPRT;
modelStruct.d_modelPar_LPRT_dX=0.0;
modelStruct.modelPar_given_LPRT=mod.given("LPRT");
modelStruct.modelPar_LNTRECF=mod.LNTRECF;
modelStruct.d_modelPar_LNTRECF_dX=0.0;
modelStruct.modelPar_given_LNTRECF=mod.given("LNTRECF");
modelStruct.modelPar_LNTRECR=mod.LNTRECR;
modelStruct.d_modelPar_LNTRECR_dX=0.0;
modelStruct.modelPar_given_LNTRECR=mod.given("LNTRECR");
modelStruct.modelPar_LXBJT=mod.LXBJT;
modelStruct.d_modelPar_LXBJT_dX=0.0;
modelStruct.modelPar_given_LXBJT=mod.given("LXBJT");
modelStruct.modelPar_LXDIF=mod.LXDIF;
modelStruct.d_modelPar_LXDIF_dX=0.0;
modelStruct.modelPar_given_LXDIF=mod.given("LXDIF");
modelStruct.modelPar_LXREC=mod.LXREC;
modelStruct.d_modelPar_LXREC_dX=0.0;
modelStruct.modelPar_given_LXREC=mod.given("LXREC");
modelStruct.modelPar_LXTUN=mod.LXTUN;
modelStruct.d_modelPar_LXTUN_dX=0.0;
modelStruct.modelPar_given_LXTUN=mod.given("LXTUN");
modelStruct.modelPar_LXDIFD=mod.LXDIFD;
modelStruct.d_modelPar_LXDIFD_dX=0.0;
modelStruct.modelPar_given_LXDIFD=mod.given("LXDIFD");
modelStruct.modelPar_LXRECD=mod.LXRECD;
modelStruct.d_modelPar_LXRECD_dX=0.0;
modelStruct.modelPar_given_LXRECD=mod.given("LXRECD");
modelStruct.modelPar_LXTUND=mod.LXTUND;
modelStruct.d_modelPar_LXTUND_dX=0.0;
modelStruct.modelPar_given_LXTUND=mod.given("LXTUND");
modelStruct.modelPar_LAIGC=mod.LAIGC;
modelStruct.d_modelPar_LAIGC_dX=0.0;
modelStruct.modelPar_given_LAIGC=mod.given("LAIGC");
modelStruct.modelPar_LBIGC=mod.LBIGC;
modelStruct.d_modelPar_LBIGC_dX=0.0;
modelStruct.modelPar_given_LBIGC=mod.given("LBIGC");
modelStruct.modelPar_LCIGC=mod.LCIGC;
modelStruct.d_modelPar_LCIGC_dX=0.0;
modelStruct.modelPar_given_LCIGC=mod.given("LCIGC");
modelStruct.modelPar_LAIGSD=mod.LAIGSD;
modelStruct.d_modelPar_LAIGSD_dX=0.0;
modelStruct.modelPar_given_LAIGSD=mod.given("LAIGSD");
modelStruct.modelPar_LBIGSD=mod.LBIGSD;
modelStruct.d_modelPar_LBIGSD_dX=0.0;
modelStruct.modelPar_given_LBIGSD=mod.given("LBIGSD");
modelStruct.modelPar_LCIGSD=mod.LCIGSD;
modelStruct.d_modelPar_LCIGSD_dX=0.0;
modelStruct.modelPar_given_LCIGSD=mod.given("LCIGSD");
modelStruct.modelPar_LNIGC=mod.LNIGC;
modelStruct.d_modelPar_LNIGC_dX=0.0;
modelStruct.modelPar_given_LNIGC=mod.given("LNIGC");
modelStruct.modelPar_LPIGCD=mod.LPIGCD;
modelStruct.d_modelPar_LPIGCD_dX=0.0;
modelStruct.modelPar_given_LPIGCD=mod.given("LPIGCD");
modelStruct.modelPar_LPOXEDGE=mod.LPOXEDGE;
modelStruct.d_modelPar_LPOXEDGE_dX=0.0;
modelStruct.modelPar_given_LPOXEDGE=mod.given("LPOXEDGE");
modelStruct.modelPar_LNCH=mod.LNCH;
modelStruct.d_modelPar_LNCH_dX=0.0;
modelStruct.modelPar_given_LNCH=mod.given("LNCH");
modelStruct.modelPar_LNSUB=mod.LNSUB;
modelStruct.d_modelPar_LNSUB_dX=0.0;
modelStruct.modelPar_given_LNSUB=mod.given("LNSUB");
modelStruct.modelPar_LNGATE=mod.LNGATE;
modelStruct.d_modelPar_LNGATE_dX=0.0;
modelStruct.modelPar_given_LNGATE=mod.given("LNGATE");
modelStruct.modelPar_LNSD=mod.LNSD;
modelStruct.d_modelPar_LNSD_dX=0.0;
modelStruct.modelPar_given_LNSD=mod.given("LNSD");
modelStruct.modelPar_LVTH0=mod.LVTH0;
modelStruct.d_modelPar_LVTH0_dX=0.0;
modelStruct.modelPar_given_LVTH0=mod.given("LVTH0");
modelStruct.modelPar_LVFB=mod.LVFB;
modelStruct.d_modelPar_LVFB_dX=0.0;
modelStruct.modelPar_given_LVFB=mod.given("LVFB");
modelStruct.modelPar_LK1=mod.LK1;
modelStruct.d_modelPar_LK1_dX=0.0;
modelStruct.modelPar_given_LK1=mod.given("LK1");
modelStruct.modelPar_LK1W1=mod.LK1W1;
modelStruct.d_modelPar_LK1W1_dX=0.0;
modelStruct.modelPar_given_LK1W1=mod.given("LK1W1");
modelStruct.modelPar_LK1W2=mod.LK1W2;
modelStruct.d_modelPar_LK1W2_dX=0.0;
modelStruct.modelPar_given_LK1W2=mod.given("LK1W2");
modelStruct.modelPar_LK2=mod.LK2;
modelStruct.d_modelPar_LK2_dX=0.0;
modelStruct.modelPar_given_LK2=mod.given("LK2");
modelStruct.modelPar_LK3=mod.LK3;
modelStruct.d_modelPar_LK3_dX=0.0;
modelStruct.modelPar_given_LK3=mod.given("LK3");
modelStruct.modelPar_LK3B=mod.LK3B;
modelStruct.d_modelPar_LK3B_dX=0.0;
modelStruct.modelPar_given_LK3B=mod.given("LK3B");
modelStruct.modelPar_LKB1=mod.LKB1;
modelStruct.d_modelPar_LKB1_dX=0.0;
modelStruct.modelPar_given_LKB1=mod.given("LKB1");
modelStruct.modelPar_LW0=mod.LW0;
modelStruct.d_modelPar_LW0_dX=0.0;
modelStruct.modelPar_given_LW0=mod.given("LW0");
modelStruct.modelPar_LLPEB=mod.LLPEB;
modelStruct.d_modelPar_LLPEB_dX=0.0;
modelStruct.modelPar_given_LLPEB=mod.given("LLPEB");
modelStruct.modelPar_LDVT0=mod.LDVT0;
modelStruct.d_modelPar_LDVT0_dX=0.0;
modelStruct.modelPar_given_LDVT0=mod.given("LDVT0");
modelStruct.modelPar_LDVT1=mod.LDVT1;
modelStruct.d_modelPar_LDVT1_dX=0.0;
modelStruct.modelPar_given_LDVT1=mod.given("LDVT1");
modelStruct.modelPar_LDVT2=mod.LDVT2;
modelStruct.d_modelPar_LDVT2_dX=0.0;
modelStruct.modelPar_given_LDVT2=mod.given("LDVT2");
modelStruct.modelPar_LDVT0W=mod.LDVT0W;
modelStruct.d_modelPar_LDVT0W_dX=0.0;
modelStruct.modelPar_given_LDVT0W=mod.given("LDVT0W");
modelStruct.modelPar_LDVT1W=mod.LDVT1W;
modelStruct.d_modelPar_LDVT1W_dX=0.0;
modelStruct.modelPar_given_LDVT1W=mod.given("LDVT1W");
modelStruct.modelPar_LDVT2W=mod.LDVT2W;
modelStruct.d_modelPar_LDVT2W_dX=0.0;
modelStruct.modelPar_given_LDVT2W=mod.given("LDVT2W");
modelStruct.modelPar_LU0=mod.LU0;
modelStruct.d_modelPar_LU0_dX=0.0;
modelStruct.modelPar_given_LU0=mod.given("LU0");
modelStruct.modelPar_LEU=mod.LEU;
modelStruct.d_modelPar_LEU_dX=0.0;
modelStruct.modelPar_given_LEU=mod.given("LEU");
modelStruct.modelPar_LUA=mod.LUA;
modelStruct.d_modelPar_LUA_dX=0.0;
modelStruct.modelPar_given_LUA=mod.given("LUA");
modelStruct.modelPar_LUB=mod.LUB;
modelStruct.d_modelPar_LUB_dX=0.0;
modelStruct.modelPar_given_LUB=mod.given("LUB");
modelStruct.modelPar_LUC=mod.LUC;
modelStruct.d_modelPar_LUC_dX=0.0;
modelStruct.modelPar_given_LUC=mod.given("LUC");
modelStruct.modelPar_LUD=mod.LUD;
modelStruct.d_modelPar_LUD_dX=0.0;
modelStruct.modelPar_given_LUD=mod.given("LUD");
modelStruct.modelPar_LUD1=mod.LUD1;
modelStruct.d_modelPar_LUD1_dX=0.0;
modelStruct.modelPar_given_LUD1=mod.given("LUD1");
modelStruct.modelPar_LUCSTE=mod.LUCSTE;
modelStruct.d_modelPar_LUCSTE_dX=0.0;
modelStruct.modelPar_given_LUCSTE=mod.given("LUCSTE");
modelStruct.modelPar_LUCS=mod.LUCS;
modelStruct.d_modelPar_LUCS_dX=0.0;
modelStruct.modelPar_given_LUCS=mod.given("LUCS");
modelStruct.modelPar_LVSAT=mod.LVSAT;
modelStruct.d_modelPar_LVSAT_dX=0.0;
modelStruct.modelPar_given_LVSAT=mod.given("LVSAT");
modelStruct.modelPar_LA0=mod.LA0;
modelStruct.d_modelPar_LA0_dX=0.0;
modelStruct.modelPar_given_LA0=mod.given("LA0");
modelStruct.modelPar_LAGS=mod.LAGS;
modelStruct.d_modelPar_LAGS_dX=0.0;
modelStruct.modelPar_given_LAGS=mod.given("LAGS");
modelStruct.modelPar_LB0=mod.LB0;
modelStruct.d_modelPar_LB0_dX=0.0;
modelStruct.modelPar_given_LB0=mod.given("LB0");
modelStruct.modelPar_LB1=mod.LB1;
modelStruct.d_modelPar_LB1_dX=0.0;
modelStruct.modelPar_given_LB1=mod.given("LB1");
modelStruct.modelPar_LKETA=mod.LKETA;
modelStruct.d_modelPar_LKETA_dX=0.0;
modelStruct.modelPar_given_LKETA=mod.given("LKETA");
modelStruct.modelPar_LKETAS=mod.LKETAS;
modelStruct.d_modelPar_LKETAS_dX=0.0;
modelStruct.modelPar_given_LKETAS=mod.given("LKETAS");
modelStruct.modelPar_LA1=mod.LA1;
modelStruct.d_modelPar_LA1_dX=0.0;
modelStruct.modelPar_given_LA1=mod.given("LA1");
modelStruct.modelPar_LA2=mod.LA2;
modelStruct.d_modelPar_LA2_dX=0.0;
modelStruct.modelPar_given_LA2=mod.given("LA2");
modelStruct.modelPar_LRDSW=mod.LRDSW;
modelStruct.d_modelPar_LRDSW_dX=0.0;
modelStruct.modelPar_given_LRDSW=mod.given("LRDSW");
modelStruct.modelPar_LRSW=mod.LRSW;
modelStruct.d_modelPar_LRSW_dX=0.0;
modelStruct.modelPar_given_LRSW=mod.given("LRSW");
modelStruct.modelPar_LRDW=mod.LRDW;
modelStruct.d_modelPar_LRDW_dX=0.0;
modelStruct.modelPar_given_LRDW=mod.given("LRDW");
modelStruct.modelPar_LPRWB=mod.LPRWB;
modelStruct.d_modelPar_LPRWB_dX=0.0;
modelStruct.modelPar_given_LPRWB=mod.given("LPRWB");
modelStruct.modelPar_LPRWG=mod.LPRWG;
modelStruct.d_modelPar_LPRWG_dX=0.0;
modelStruct.modelPar_given_LPRWG=mod.given("LPRWG");
modelStruct.modelPar_LWR=mod.LWR;
modelStruct.d_modelPar_LWR_dX=0.0;
modelStruct.modelPar_given_LWR=mod.given("LWR");
modelStruct.modelPar_LNFACTOR=mod.LNFACTOR;
modelStruct.d_modelPar_LNFACTOR_dX=0.0;
modelStruct.modelPar_given_LNFACTOR=mod.given("LNFACTOR");
modelStruct.modelPar_LDWG=mod.LDWG;
modelStruct.d_modelPar_LDWG_dX=0.0;
modelStruct.modelPar_given_LDWG=mod.given("LDWG");
modelStruct.modelPar_LDWB=mod.LDWB;
modelStruct.d_modelPar_LDWB_dX=0.0;
modelStruct.modelPar_given_LDWB=mod.given("LDWB");
modelStruct.modelPar_LVOFF=mod.LVOFF;
modelStruct.d_modelPar_LVOFF_dX=0.0;
modelStruct.modelPar_given_LVOFF=mod.given("LVOFF");
modelStruct.modelPar_LETA0=mod.LETA0;
modelStruct.d_modelPar_LETA0_dX=0.0;
modelStruct.modelPar_given_LETA0=mod.given("LETA0");
modelStruct.modelPar_LETAB=mod.LETAB;
modelStruct.d_modelPar_LETAB_dX=0.0;
modelStruct.modelPar_given_LETAB=mod.given("LETAB");
modelStruct.modelPar_LETA0CV=mod.LETA0CV;
modelStruct.d_modelPar_LETA0CV_dX=0.0;
modelStruct.modelPar_given_LETA0CV=mod.given("LETA0CV");
modelStruct.modelPar_LETABCV=mod.LETABCV;
modelStruct.d_modelPar_LETABCV_dX=0.0;
modelStruct.modelPar_given_LETABCV=mod.given("LETABCV");
modelStruct.modelPar_LDSUB=mod.LDSUB;
modelStruct.d_modelPar_LDSUB_dX=0.0;
modelStruct.modelPar_given_LDSUB=mod.given("LDSUB");
modelStruct.modelPar_LCIT=mod.LCIT;
modelStruct.d_modelPar_LCIT_dX=0.0;
modelStruct.modelPar_given_LCIT=mod.given("LCIT");
modelStruct.modelPar_LCDSC=mod.LCDSC;
modelStruct.d_modelPar_LCDSC_dX=0.0;
modelStruct.modelPar_given_LCDSC=mod.given("LCDSC");
modelStruct.modelPar_LCDSCB=mod.LCDSCB;
modelStruct.d_modelPar_LCDSCB_dX=0.0;
modelStruct.modelPar_given_LCDSCB=mod.given("LCDSCB");
modelStruct.modelPar_LCDSCD=mod.LCDSCD;
modelStruct.d_modelPar_LCDSCD_dX=0.0;
modelStruct.modelPar_given_LCDSCD=mod.given("LCDSCD");
modelStruct.modelPar_LPCLM=mod.LPCLM;
modelStruct.d_modelPar_LPCLM_dX=0.0;
modelStruct.modelPar_given_LPCLM=mod.given("LPCLM");
modelStruct.modelPar_LPDIBLC1=mod.LPDIBLC1;
modelStruct.d_modelPar_LPDIBLC1_dX=0.0;
modelStruct.modelPar_given_LPDIBLC1=mod.given("LPDIBLC1");
modelStruct.modelPar_LPDIBLC2=mod.LPDIBLC2;
modelStruct.d_modelPar_LPDIBLC2_dX=0.0;
modelStruct.modelPar_given_LPDIBLC2=mod.given("LPDIBLC2");
modelStruct.modelPar_LPDIBLCB=mod.LPDIBLCB;
modelStruct.d_modelPar_LPDIBLCB_dX=0.0;
modelStruct.modelPar_given_LPDIBLCB=mod.given("LPDIBLCB");
modelStruct.modelPar_LDROUT=mod.LDROUT;
modelStruct.d_modelPar_LDROUT_dX=0.0;
modelStruct.modelPar_given_LDROUT=mod.given("LDROUT");
modelStruct.modelPar_LPVAG=mod.LPVAG;
modelStruct.d_modelPar_LPVAG_dX=0.0;
modelStruct.modelPar_given_LPVAG=mod.given("LPVAG");
modelStruct.modelPar_LDELTA=mod.LDELTA;
modelStruct.d_modelPar_LDELTA_dX=0.0;
modelStruct.modelPar_given_LDELTA=mod.given("LDELTA");
modelStruct.modelPar_LALPHA0=mod.LALPHA0;
modelStruct.d_modelPar_LALPHA0_dX=0.0;
modelStruct.modelPar_given_LALPHA0=mod.given("LALPHA0");
modelStruct.modelPar_LFBJTII=mod.LFBJTII;
modelStruct.d_modelPar_LFBJTII_dX=0.0;
modelStruct.modelPar_given_LFBJTII=mod.given("LFBJTII");
modelStruct.modelPar_LABJTII=mod.LABJTII;
modelStruct.d_modelPar_LABJTII_dX=0.0;
modelStruct.modelPar_given_LABJTII=mod.given("LABJTII");
modelStruct.modelPar_LCBJTII=mod.LCBJTII;
modelStruct.d_modelPar_LCBJTII_dX=0.0;
modelStruct.modelPar_given_LCBJTII=mod.given("LCBJTII");
modelStruct.modelPar_LEBJTII=mod.LEBJTII;
modelStruct.d_modelPar_LEBJTII_dX=0.0;
modelStruct.modelPar_given_LEBJTII=mod.given("LEBJTII");
modelStruct.modelPar_LMBJTII=mod.LMBJTII;
modelStruct.d_modelPar_LMBJTII_dX=0.0;
modelStruct.modelPar_given_LMBJTII=mod.given("LMBJTII");
modelStruct.modelPar_LVBCI=mod.LVBCI;
modelStruct.d_modelPar_LVBCI_dX=0.0;
modelStruct.modelPar_given_LVBCI=mod.given("LVBCI");
modelStruct.modelPar_LBETA0=mod.LBETA0;
modelStruct.d_modelPar_LBETA0_dX=0.0;
modelStruct.modelPar_given_LBETA0=mod.given("LBETA0");
modelStruct.modelPar_LBETA1=mod.LBETA1;
modelStruct.d_modelPar_LBETA1_dX=0.0;
modelStruct.modelPar_given_LBETA1=mod.given("LBETA1");
modelStruct.modelPar_LBETA2=mod.LBETA2;
modelStruct.d_modelPar_LBETA2_dX=0.0;
modelStruct.modelPar_given_LBETA2=mod.given("LBETA2");
modelStruct.modelPar_LVDSATII0=mod.LVDSATII0;
modelStruct.d_modelPar_LVDSATII0_dX=0.0;
modelStruct.modelPar_given_LVDSATII0=mod.given("LVDSATII0");
modelStruct.modelPar_LLII=mod.LLII;
modelStruct.d_modelPar_LLII_dX=0.0;
modelStruct.modelPar_given_LLII=mod.given("LLII");
modelStruct.modelPar_LESATII=mod.LESATII;
modelStruct.d_modelPar_LESATII_dX=0.0;
modelStruct.modelPar_given_LESATII=mod.given("LESATII");
modelStruct.modelPar_LSII0=mod.LSII0;
modelStruct.d_modelPar_LSII0_dX=0.0;
modelStruct.modelPar_given_LSII0=mod.given("LSII0");
modelStruct.modelPar_LSII1=mod.LSII1;
modelStruct.d_modelPar_LSII1_dX=0.0;
modelStruct.modelPar_given_LSII1=mod.given("LSII1");
modelStruct.modelPar_LSII2=mod.LSII2;
modelStruct.d_modelPar_LSII2_dX=0.0;
modelStruct.modelPar_given_LSII2=mod.given("LSII2");
modelStruct.modelPar_LSIID=mod.LSIID;
modelStruct.d_modelPar_LSIID_dX=0.0;
modelStruct.modelPar_given_LSIID=mod.given("LSIID");
modelStruct.modelPar_LAGIDL=mod.LAGIDL;
modelStruct.d_modelPar_LAGIDL_dX=0.0;
modelStruct.modelPar_given_LAGIDL=mod.given("LAGIDL");
modelStruct.modelPar_LBGIDL=mod.LBGIDL;
modelStruct.d_modelPar_LBGIDL_dX=0.0;
modelStruct.modelPar_given_LBGIDL=mod.given("LBGIDL");
modelStruct.modelPar_LCGIDL=mod.LCGIDL;
modelStruct.d_modelPar_LCGIDL_dX=0.0;
modelStruct.modelPar_given_LCGIDL=mod.given("LCGIDL");
modelStruct.modelPar_LRGIDL=mod.LRGIDL;
modelStruct.d_modelPar_LRGIDL_dX=0.0;
modelStruct.modelPar_given_LRGIDL=mod.given("LRGIDL");
modelStruct.modelPar_LKGIDL=mod.LKGIDL;
modelStruct.d_modelPar_LKGIDL_dX=0.0;
modelStruct.modelPar_given_LKGIDL=mod.given("LKGIDL");
modelStruct.modelPar_LFGIDL=mod.LFGIDL;
modelStruct.d_modelPar_LFGIDL_dX=0.0;
modelStruct.modelPar_given_LFGIDL=mod.given("LFGIDL");
modelStruct.modelPar_LAGISL=mod.LAGISL;
modelStruct.d_modelPar_LAGISL_dX=0.0;
modelStruct.modelPar_given_LAGISL=mod.given("LAGISL");
modelStruct.modelPar_LBGISL=mod.LBGISL;
modelStruct.d_modelPar_LBGISL_dX=0.0;
modelStruct.modelPar_given_LBGISL=mod.given("LBGISL");
modelStruct.modelPar_LCGISL=mod.LCGISL;
modelStruct.d_modelPar_LCGISL_dX=0.0;
modelStruct.modelPar_given_LCGISL=mod.given("LCGISL");
modelStruct.modelPar_LRGISL=mod.LRGISL;
modelStruct.d_modelPar_LRGISL_dX=0.0;
modelStruct.modelPar_given_LRGISL=mod.given("LRGISL");
modelStruct.modelPar_LKGISL=mod.LKGISL;
modelStruct.d_modelPar_LKGISL_dX=0.0;
modelStruct.modelPar_given_LKGISL=mod.given("LKGISL");
modelStruct.modelPar_LFGISL=mod.LFGISL;
modelStruct.d_modelPar_LFGISL_dX=0.0;
modelStruct.modelPar_given_LFGISL=mod.given("LFGISL");
modelStruct.modelPar_LNTUN=mod.LNTUN;
modelStruct.d_modelPar_LNTUN_dX=0.0;
modelStruct.modelPar_given_LNTUN=mod.given("LNTUN");
modelStruct.modelPar_LNTUND=mod.LNTUND;
modelStruct.d_modelPar_LNTUND_dX=0.0;
modelStruct.modelPar_given_LNTUND=mod.given("LNTUND");
modelStruct.modelPar_LNDIODE=mod.LNDIODE;
modelStruct.d_modelPar_LNDIODE_dX=0.0;
modelStruct.modelPar_given_LNDIODE=mod.given("LNDIODE");
modelStruct.modelPar_LNDIODED=mod.LNDIODED;
modelStruct.d_modelPar_LNDIODED_dX=0.0;
modelStruct.modelPar_given_LNDIODED=mod.given("LNDIODED");
modelStruct.modelPar_LNRECF0=mod.LNRECF0;
modelStruct.d_modelPar_LNRECF0_dX=0.0;
modelStruct.modelPar_given_LNRECF0=mod.given("LNRECF0");
modelStruct.modelPar_LNRECF0D=mod.LNRECF0D;
modelStruct.d_modelPar_LNRECF0D_dX=0.0;
modelStruct.modelPar_given_LNRECF0D=mod.given("LNRECF0D");
modelStruct.modelPar_LNRECR0=mod.LNRECR0;
modelStruct.d_modelPar_LNRECR0_dX=0.0;
modelStruct.modelPar_given_LNRECR0=mod.given("LNRECR0");
modelStruct.modelPar_LNRECR0D=mod.LNRECR0D;
modelStruct.d_modelPar_LNRECR0D_dX=0.0;
modelStruct.modelPar_given_LNRECR0D=mod.given("LNRECR0D");
modelStruct.modelPar_LISBJT=mod.LISBJT;
modelStruct.d_modelPar_LISBJT_dX=0.0;
modelStruct.modelPar_given_LISBJT=mod.given("LISBJT");
modelStruct.modelPar_LIDBJT=mod.LIDBJT;
modelStruct.d_modelPar_LIDBJT_dX=0.0;
modelStruct.modelPar_given_LIDBJT=mod.given("LIDBJT");
modelStruct.modelPar_LISDIF=mod.LISDIF;
modelStruct.d_modelPar_LISDIF_dX=0.0;
modelStruct.modelPar_given_LISDIF=mod.given("LISDIF");
modelStruct.modelPar_LIDDIF=mod.LIDDIF;
modelStruct.d_modelPar_LIDDIF_dX=0.0;
modelStruct.modelPar_given_LIDDIF=mod.given("LIDDIF");
modelStruct.modelPar_LISREC=mod.LISREC;
modelStruct.d_modelPar_LISREC_dX=0.0;
modelStruct.modelPar_given_LISREC=mod.given("LISREC");
modelStruct.modelPar_LIDREC=mod.LIDREC;
modelStruct.d_modelPar_LIDREC_dX=0.0;
modelStruct.modelPar_given_LIDREC=mod.given("LIDREC");
modelStruct.modelPar_LISTUN=mod.LISTUN;
modelStruct.d_modelPar_LISTUN_dX=0.0;
modelStruct.modelPar_given_LISTUN=mod.given("LISTUN");
modelStruct.modelPar_LIDTUN=mod.LIDTUN;
modelStruct.d_modelPar_LIDTUN_dX=0.0;
modelStruct.modelPar_given_LIDTUN=mod.given("LIDTUN");
modelStruct.modelPar_LVREC0=mod.LVREC0;
modelStruct.d_modelPar_LVREC0_dX=0.0;
modelStruct.modelPar_given_LVREC0=mod.given("LVREC0");
modelStruct.modelPar_LVREC0D=mod.LVREC0D;
modelStruct.d_modelPar_LVREC0D_dX=0.0;
modelStruct.modelPar_given_LVREC0D=mod.given("LVREC0D");
modelStruct.modelPar_LVTUN0=mod.LVTUN0;
modelStruct.d_modelPar_LVTUN0_dX=0.0;
modelStruct.modelPar_given_LVTUN0=mod.given("LVTUN0");
modelStruct.modelPar_LVTUN0D=mod.LVTUN0D;
modelStruct.d_modelPar_LVTUN0D_dX=0.0;
modelStruct.modelPar_given_LVTUN0D=mod.given("LVTUN0D");
modelStruct.modelPar_LNBJT=mod.LNBJT;
modelStruct.d_modelPar_LNBJT_dX=0.0;
modelStruct.modelPar_given_LNBJT=mod.given("LNBJT");
modelStruct.modelPar_LLBJT0=mod.LLBJT0;
modelStruct.d_modelPar_LLBJT0_dX=0.0;
modelStruct.modelPar_given_LLBJT0=mod.given("LLBJT0");
modelStruct.modelPar_LVABJT=mod.LVABJT;
modelStruct.d_modelPar_LVABJT_dX=0.0;
modelStruct.modelPar_given_LVABJT=mod.given("LVABJT");
modelStruct.modelPar_LAELY=mod.LAELY;
modelStruct.d_modelPar_LAELY_dX=0.0;
modelStruct.modelPar_given_LAELY=mod.given("LAELY");
modelStruct.modelPar_LAHLI=mod.LAHLI;
modelStruct.d_modelPar_LAHLI_dX=0.0;
modelStruct.modelPar_given_LAHLI=mod.given("LAHLI");
modelStruct.modelPar_LAHLID=mod.LAHLID;
modelStruct.d_modelPar_LAHLID_dX=0.0;
modelStruct.modelPar_given_LAHLID=mod.given("LAHLID");
modelStruct.modelPar_LVSDFB=mod.LVSDFB;
modelStruct.d_modelPar_LVSDFB_dX=0.0;
modelStruct.modelPar_given_LVSDFB=mod.given("LVSDFB");
modelStruct.modelPar_LVSDTH=mod.LVSDTH;
modelStruct.d_modelPar_LVSDTH_dX=0.0;
modelStruct.modelPar_given_LVSDTH=mod.given("LVSDTH");
modelStruct.modelPar_LDELVT=mod.LDELVT;
modelStruct.d_modelPar_LDELVT_dX=0.0;
modelStruct.modelPar_given_LDELVT=mod.given("LDELVT");
modelStruct.modelPar_LACDE=mod.LACDE;
modelStruct.d_modelPar_LACDE_dX=0.0;
modelStruct.modelPar_given_LACDE=mod.given("LACDE");
modelStruct.modelPar_LMOIN=mod.LMOIN;
modelStruct.d_modelPar_LMOIN_dX=0.0;
modelStruct.modelPar_given_LMOIN=mod.given("LMOIN");
modelStruct.modelPar_LNOFF=mod.LNOFF;
modelStruct.d_modelPar_LNOFF_dX=0.0;
modelStruct.modelPar_given_LNOFF=mod.given("LNOFF");
modelStruct.modelPar_LNOFF2=mod.LNOFF2;
modelStruct.d_modelPar_LNOFF2_dX=0.0;
modelStruct.modelPar_given_LNOFF2=mod.given("LNOFF2");
modelStruct.modelPar_LXRCRG1=mod.LXRCRG1;
modelStruct.d_modelPar_LXRCRG1_dX=0.0;
modelStruct.modelPar_given_LXRCRG1=mod.given("LXRCRG1");
modelStruct.modelPar_LXRCRG2=mod.LXRCRG2;
modelStruct.d_modelPar_LXRCRG2_dX=0.0;
modelStruct.modelPar_given_LXRCRG2=mod.given("LXRCRG2");
modelStruct.modelPar_LVBSA=mod.LVBSA;
modelStruct.d_modelPar_LVBSA_dX=0.0;
modelStruct.modelPar_given_LVBSA=mod.given("LVBSA");
modelStruct.modelPar_LVSCE=mod.LVSCE;
modelStruct.d_modelPar_LVSCE_dX=0.0;
modelStruct.modelPar_given_LVSCE=mod.given("LVSCE");
modelStruct.modelPar_LCDSBS=mod.LCDSBS;
modelStruct.d_modelPar_LCDSBS_dX=0.0;
modelStruct.modelPar_given_LCDSBS=mod.given("LCDSBS");
modelStruct.modelPar_LNOFFFD=mod.LNOFFFD;
modelStruct.d_modelPar_LNOFFFD_dX=0.0;
modelStruct.modelPar_given_LNOFFFD=mod.given("LNOFFFD");
modelStruct.modelPar_LVOFFFD=mod.LVOFFFD;
modelStruct.d_modelPar_LVOFFFD_dX=0.0;
modelStruct.modelPar_given_LVOFFFD=mod.given("LVOFFFD");
modelStruct.modelPar_LK1B=mod.LK1B;
modelStruct.d_modelPar_LK1B_dX=0.0;
modelStruct.modelPar_given_LK1B=mod.given("LK1B");
modelStruct.modelPar_LK2B=mod.LK2B;
modelStruct.d_modelPar_LK2B_dX=0.0;
modelStruct.modelPar_given_LK2B=mod.given("LK2B");
modelStruct.modelPar_LDK2B=mod.LDK2B;
modelStruct.d_modelPar_LDK2B_dX=0.0;
modelStruct.modelPar_given_LDK2B=mod.given("LDK2B");
modelStruct.modelPar_LDVBD0=mod.LDVBD0;
modelStruct.d_modelPar_LDVBD0_dX=0.0;
modelStruct.modelPar_given_LDVBD0=mod.given("LDVBD0");
modelStruct.modelPar_LDVBD1=mod.LDVBD1;
modelStruct.d_modelPar_LDVBD1_dX=0.0;
modelStruct.modelPar_given_LDVBD1=mod.given("LDVBD1");
modelStruct.modelPar_LMOINFD=mod.LMOINFD;
modelStruct.d_modelPar_LMOINFD_dX=0.0;
modelStruct.modelPar_given_LMOINFD=mod.given("LMOINFD");
modelStruct.modelPar_LVBS0PD=mod.LVBS0PD;
modelStruct.d_modelPar_LVBS0PD_dX=0.0;
modelStruct.modelPar_given_LVBS0PD=mod.given("LVBS0PD");
modelStruct.modelPar_LVBS0FD=mod.LVBS0FD;
modelStruct.d_modelPar_LVBS0FD_dX=0.0;
modelStruct.modelPar_given_LVBS0FD=mod.given("LVBS0FD");
modelStruct.modelPar_WXJ=mod.WXJ;
modelStruct.d_modelPar_WXJ_dX=0.0;
modelStruct.modelPar_given_WXJ=mod.given("WXJ");
modelStruct.modelPar_WALPHAGB1=mod.WALPHAGB1;
modelStruct.d_modelPar_WALPHAGB1_dX=0.0;
modelStruct.modelPar_given_WALPHAGB1=mod.given("WALPHAGB1");
modelStruct.modelPar_WBETAGB1=mod.WBETAGB1;
modelStruct.d_modelPar_WBETAGB1_dX=0.0;
modelStruct.modelPar_given_WBETAGB1=mod.given("WBETAGB1");
modelStruct.modelPar_WALPHAGB2=mod.WALPHAGB2;
modelStruct.d_modelPar_WALPHAGB2_dX=0.0;
modelStruct.modelPar_given_WALPHAGB2=mod.given("WALPHAGB2");
modelStruct.modelPar_WBETAGB2=mod.WBETAGB2;
modelStruct.d_modelPar_WBETAGB2_dX=0.0;
modelStruct.modelPar_given_WBETAGB2=mod.given("WBETAGB2");
modelStruct.modelPar_WAIGBCP2=mod.WAIGBCP2;
modelStruct.d_modelPar_WAIGBCP2_dX=0.0;
modelStruct.modelPar_given_WAIGBCP2=mod.given("WAIGBCP2");
modelStruct.modelPar_WBIGBCP2=mod.WBIGBCP2;
modelStruct.d_modelPar_WBIGBCP2_dX=0.0;
modelStruct.modelPar_given_WBIGBCP2=mod.given("WBIGBCP2");
modelStruct.modelPar_WCIGBCP2=mod.WCIGBCP2;
modelStruct.d_modelPar_WCIGBCP2_dX=0.0;
modelStruct.modelPar_given_WCIGBCP2=mod.given("WCIGBCP2");
modelStruct.modelPar_WCGSL=mod.WCGSL;
modelStruct.d_modelPar_WCGSL_dX=0.0;
modelStruct.modelPar_given_WCGSL=mod.given("WCGSL");
modelStruct.modelPar_WCGDL=mod.WCGDL;
modelStruct.d_modelPar_WCGDL_dX=0.0;
modelStruct.modelPar_given_WCGDL=mod.given("WCGDL");
modelStruct.modelPar_WCKAPPA=mod.WCKAPPA;
modelStruct.d_modelPar_WCKAPPA_dX=0.0;
modelStruct.modelPar_given_WCKAPPA=mod.given("WCKAPPA");
modelStruct.modelPar_WNDIF=mod.WNDIF;
modelStruct.d_modelPar_WNDIF_dX=0.0;
modelStruct.modelPar_given_WNDIF=mod.given("WNDIF");
modelStruct.modelPar_WUTE=mod.WUTE;
modelStruct.d_modelPar_WUTE_dX=0.0;
modelStruct.modelPar_given_WUTE=mod.given("WUTE");
modelStruct.modelPar_WKT1=mod.WKT1;
modelStruct.d_modelPar_WKT1_dX=0.0;
modelStruct.modelPar_given_WKT1=mod.given("WKT1");
modelStruct.modelPar_WKT1L=mod.WKT1L;
modelStruct.d_modelPar_WKT1L_dX=0.0;
modelStruct.modelPar_given_WKT1L=mod.given("WKT1L");
modelStruct.modelPar_WKT2=mod.WKT2;
modelStruct.d_modelPar_WKT2_dX=0.0;
modelStruct.modelPar_given_WKT2=mod.given("WKT2");
modelStruct.modelPar_WUA1=mod.WUA1;
modelStruct.d_modelPar_WUA1_dX=0.0;
modelStruct.modelPar_given_WUA1=mod.given("WUA1");
modelStruct.modelPar_WUB1=mod.WUB1;
modelStruct.d_modelPar_WUB1_dX=0.0;
modelStruct.modelPar_given_WUB1=mod.given("WUB1");
modelStruct.modelPar_WUC1=mod.WUC1;
modelStruct.d_modelPar_WUC1_dX=0.0;
modelStruct.modelPar_given_WUC1=mod.given("WUC1");
modelStruct.modelPar_WAT=mod.WAT;
modelStruct.d_modelPar_WAT_dX=0.0;
modelStruct.modelPar_given_WAT=mod.given("WAT");
modelStruct.modelPar_WPRT=mod.WPRT;
modelStruct.d_modelPar_WPRT_dX=0.0;
modelStruct.modelPar_given_WPRT=mod.given("WPRT");
modelStruct.modelPar_WNTRECF=mod.WNTRECF;
modelStruct.d_modelPar_WNTRECF_dX=0.0;
modelStruct.modelPar_given_WNTRECF=mod.given("WNTRECF");
modelStruct.modelPar_WNTRECR=mod.WNTRECR;
modelStruct.d_modelPar_WNTRECR_dX=0.0;
modelStruct.modelPar_given_WNTRECR=mod.given("WNTRECR");
modelStruct.modelPar_WXBJT=mod.WXBJT;
modelStruct.d_modelPar_WXBJT_dX=0.0;
modelStruct.modelPar_given_WXBJT=mod.given("WXBJT");
modelStruct.modelPar_WXDIF=mod.WXDIF;
modelStruct.d_modelPar_WXDIF_dX=0.0;
modelStruct.modelPar_given_WXDIF=mod.given("WXDIF");
modelStruct.modelPar_WXREC=mod.WXREC;
modelStruct.d_modelPar_WXREC_dX=0.0;
modelStruct.modelPar_given_WXREC=mod.given("WXREC");
modelStruct.modelPar_WXTUN=mod.WXTUN;
modelStruct.d_modelPar_WXTUN_dX=0.0;
modelStruct.modelPar_given_WXTUN=mod.given("WXTUN");
modelStruct.modelPar_WXDIFD=mod.WXDIFD;
modelStruct.d_modelPar_WXDIFD_dX=0.0;
modelStruct.modelPar_given_WXDIFD=mod.given("WXDIFD");
modelStruct.modelPar_WXRECD=mod.WXRECD;
modelStruct.d_modelPar_WXRECD_dX=0.0;
modelStruct.modelPar_given_WXRECD=mod.given("WXRECD");
modelStruct.modelPar_WXTUND=mod.WXTUND;
modelStruct.d_modelPar_WXTUND_dX=0.0;
modelStruct.modelPar_given_WXTUND=mod.given("WXTUND");
modelStruct.modelPar_WAIGC=mod.WAIGC;
modelStruct.d_modelPar_WAIGC_dX=0.0;
modelStruct.modelPar_given_WAIGC=mod.given("WAIGC");
modelStruct.modelPar_WBIGC=mod.WBIGC;
modelStruct.d_modelPar_WBIGC_dX=0.0;
modelStruct.modelPar_given_WBIGC=mod.given("WBIGC");
modelStruct.modelPar_WCIGC=mod.WCIGC;
modelStruct.d_modelPar_WCIGC_dX=0.0;
modelStruct.modelPar_given_WCIGC=mod.given("WCIGC");
modelStruct.modelPar_WAIGSD=mod.WAIGSD;
modelStruct.d_modelPar_WAIGSD_dX=0.0;
modelStruct.modelPar_given_WAIGSD=mod.given("WAIGSD");
modelStruct.modelPar_WBIGSD=mod.WBIGSD;
modelStruct.d_modelPar_WBIGSD_dX=0.0;
modelStruct.modelPar_given_WBIGSD=mod.given("WBIGSD");
modelStruct.modelPar_WCIGSD=mod.WCIGSD;
modelStruct.d_modelPar_WCIGSD_dX=0.0;
modelStruct.modelPar_given_WCIGSD=mod.given("WCIGSD");
modelStruct.modelPar_WNIGC=mod.WNIGC;
modelStruct.d_modelPar_WNIGC_dX=0.0;
modelStruct.modelPar_given_WNIGC=mod.given("WNIGC");
modelStruct.modelPar_WPIGCD=mod.WPIGCD;
modelStruct.d_modelPar_WPIGCD_dX=0.0;
modelStruct.modelPar_given_WPIGCD=mod.given("WPIGCD");
modelStruct.modelPar_WPOXEDGE=mod.WPOXEDGE;
modelStruct.d_modelPar_WPOXEDGE_dX=0.0;
modelStruct.modelPar_given_WPOXEDGE=mod.given("WPOXEDGE");
modelStruct.modelPar_WNCH=mod.WNCH;
modelStruct.d_modelPar_WNCH_dX=0.0;
modelStruct.modelPar_given_WNCH=mod.given("WNCH");
modelStruct.modelPar_WNSUB=mod.WNSUB;
modelStruct.d_modelPar_WNSUB_dX=0.0;
modelStruct.modelPar_given_WNSUB=mod.given("WNSUB");
modelStruct.modelPar_WNGATE=mod.WNGATE;
modelStruct.d_modelPar_WNGATE_dX=0.0;
modelStruct.modelPar_given_WNGATE=mod.given("WNGATE");
modelStruct.modelPar_WNSD=mod.WNSD;
modelStruct.d_modelPar_WNSD_dX=0.0;
modelStruct.modelPar_given_WNSD=mod.given("WNSD");
modelStruct.modelPar_WVTH0=mod.WVTH0;
modelStruct.d_modelPar_WVTH0_dX=0.0;
modelStruct.modelPar_given_WVTH0=mod.given("WVTH0");
modelStruct.modelPar_WVFB=mod.WVFB;
modelStruct.d_modelPar_WVFB_dX=0.0;
modelStruct.modelPar_given_WVFB=mod.given("WVFB");
modelStruct.modelPar_WK1=mod.WK1;
modelStruct.d_modelPar_WK1_dX=0.0;
modelStruct.modelPar_given_WK1=mod.given("WK1");
modelStruct.modelPar_WK1W1=mod.WK1W1;
modelStruct.d_modelPar_WK1W1_dX=0.0;
modelStruct.modelPar_given_WK1W1=mod.given("WK1W1");
modelStruct.modelPar_WK1W2=mod.WK1W2;
modelStruct.d_modelPar_WK1W2_dX=0.0;
modelStruct.modelPar_given_WK1W2=mod.given("WK1W2");
modelStruct.modelPar_WK2=mod.WK2;
modelStruct.d_modelPar_WK2_dX=0.0;
modelStruct.modelPar_given_WK2=mod.given("WK2");
modelStruct.modelPar_WK3=mod.WK3;
modelStruct.d_modelPar_WK3_dX=0.0;
modelStruct.modelPar_given_WK3=mod.given("WK3");
modelStruct.modelPar_WK3B=mod.WK3B;
modelStruct.d_modelPar_WK3B_dX=0.0;
modelStruct.modelPar_given_WK3B=mod.given("WK3B");
modelStruct.modelPar_WKB1=mod.WKB1;
modelStruct.d_modelPar_WKB1_dX=0.0;
modelStruct.modelPar_given_WKB1=mod.given("WKB1");
modelStruct.modelPar_WW0=mod.WW0;
modelStruct.d_modelPar_WW0_dX=0.0;
modelStruct.modelPar_given_WW0=mod.given("WW0");
modelStruct.modelPar_WLPEB=mod.WLPEB;
modelStruct.d_modelPar_WLPEB_dX=0.0;
modelStruct.modelPar_given_WLPEB=mod.given("WLPEB");
modelStruct.modelPar_WDVT0=mod.WDVT0;
modelStruct.d_modelPar_WDVT0_dX=0.0;
modelStruct.modelPar_given_WDVT0=mod.given("WDVT0");
modelStruct.modelPar_WDVT1=mod.WDVT1;
modelStruct.d_modelPar_WDVT1_dX=0.0;
modelStruct.modelPar_given_WDVT1=mod.given("WDVT1");
modelStruct.modelPar_WDVT2=mod.WDVT2;
modelStruct.d_modelPar_WDVT2_dX=0.0;
modelStruct.modelPar_given_WDVT2=mod.given("WDVT2");
modelStruct.modelPar_WDVT0W=mod.WDVT0W;
modelStruct.d_modelPar_WDVT0W_dX=0.0;
modelStruct.modelPar_given_WDVT0W=mod.given("WDVT0W");
modelStruct.modelPar_WDVT1W=mod.WDVT1W;
modelStruct.d_modelPar_WDVT1W_dX=0.0;
modelStruct.modelPar_given_WDVT1W=mod.given("WDVT1W");
modelStruct.modelPar_WDVT2W=mod.WDVT2W;
modelStruct.d_modelPar_WDVT2W_dX=0.0;
modelStruct.modelPar_given_WDVT2W=mod.given("WDVT2W");
modelStruct.modelPar_WU0=mod.WU0;
modelStruct.d_modelPar_WU0_dX=0.0;
modelStruct.modelPar_given_WU0=mod.given("WU0");
modelStruct.modelPar_WEU=mod.WEU;
modelStruct.d_modelPar_WEU_dX=0.0;
modelStruct.modelPar_given_WEU=mod.given("WEU");
modelStruct.modelPar_WUA=mod.WUA;
modelStruct.d_modelPar_WUA_dX=0.0;
modelStruct.modelPar_given_WUA=mod.given("WUA");
modelStruct.modelPar_WUB=mod.WUB;
modelStruct.d_modelPar_WUB_dX=0.0;
modelStruct.modelPar_given_WUB=mod.given("WUB");
modelStruct.modelPar_WUC=mod.WUC;
modelStruct.d_modelPar_WUC_dX=0.0;
modelStruct.modelPar_given_WUC=mod.given("WUC");
modelStruct.modelPar_WUD=mod.WUD;
modelStruct.d_modelPar_WUD_dX=0.0;
modelStruct.modelPar_given_WUD=mod.given("WUD");
modelStruct.modelPar_WUD1=mod.WUD1;
modelStruct.d_modelPar_WUD1_dX=0.0;
modelStruct.modelPar_given_WUD1=mod.given("WUD1");
modelStruct.modelPar_WUCSTE=mod.WUCSTE;
modelStruct.d_modelPar_WUCSTE_dX=0.0;
modelStruct.modelPar_given_WUCSTE=mod.given("WUCSTE");
modelStruct.modelPar_WUCS=mod.WUCS;
modelStruct.d_modelPar_WUCS_dX=0.0;
modelStruct.modelPar_given_WUCS=mod.given("WUCS");
modelStruct.modelPar_WVSAT=mod.WVSAT;
modelStruct.d_modelPar_WVSAT_dX=0.0;
modelStruct.modelPar_given_WVSAT=mod.given("WVSAT");
modelStruct.modelPar_WA0=mod.WA0;
modelStruct.d_modelPar_WA0_dX=0.0;
modelStruct.modelPar_given_WA0=mod.given("WA0");
modelStruct.modelPar_WAGS=mod.WAGS;
modelStruct.d_modelPar_WAGS_dX=0.0;
modelStruct.modelPar_given_WAGS=mod.given("WAGS");
modelStruct.modelPar_WB0=mod.WB0;
modelStruct.d_modelPar_WB0_dX=0.0;
modelStruct.modelPar_given_WB0=mod.given("WB0");
modelStruct.modelPar_WB1=mod.WB1;
modelStruct.d_modelPar_WB1_dX=0.0;
modelStruct.modelPar_given_WB1=mod.given("WB1");
modelStruct.modelPar_WKETA=mod.WKETA;
modelStruct.d_modelPar_WKETA_dX=0.0;
modelStruct.modelPar_given_WKETA=mod.given("WKETA");
modelStruct.modelPar_WKETAS=mod.WKETAS;
modelStruct.d_modelPar_WKETAS_dX=0.0;
modelStruct.modelPar_given_WKETAS=mod.given("WKETAS");
modelStruct.modelPar_WA1=mod.WA1;
modelStruct.d_modelPar_WA1_dX=0.0;
modelStruct.modelPar_given_WA1=mod.given("WA1");
modelStruct.modelPar_WA2=mod.WA2;
modelStruct.d_modelPar_WA2_dX=0.0;
modelStruct.modelPar_given_WA2=mod.given("WA2");
modelStruct.modelPar_WRDSW=mod.WRDSW;
modelStruct.d_modelPar_WRDSW_dX=0.0;
modelStruct.modelPar_given_WRDSW=mod.given("WRDSW");
modelStruct.modelPar_WRSW=mod.WRSW;
modelStruct.d_modelPar_WRSW_dX=0.0;
modelStruct.modelPar_given_WRSW=mod.given("WRSW");
modelStruct.modelPar_WRDW=mod.WRDW;
modelStruct.d_modelPar_WRDW_dX=0.0;
modelStruct.modelPar_given_WRDW=mod.given("WRDW");
modelStruct.modelPar_WPRWB=mod.WPRWB;
modelStruct.d_modelPar_WPRWB_dX=0.0;
modelStruct.modelPar_given_WPRWB=mod.given("WPRWB");
modelStruct.modelPar_WPRWG=mod.WPRWG;
modelStruct.d_modelPar_WPRWG_dX=0.0;
modelStruct.modelPar_given_WPRWG=mod.given("WPRWG");
modelStruct.modelPar_WWR=mod.WWR;
modelStruct.d_modelPar_WWR_dX=0.0;
modelStruct.modelPar_given_WWR=mod.given("WWR");
modelStruct.modelPar_WNFACTOR=mod.WNFACTOR;
modelStruct.d_modelPar_WNFACTOR_dX=0.0;
modelStruct.modelPar_given_WNFACTOR=mod.given("WNFACTOR");
modelStruct.modelPar_WDWG=mod.WDWG;
modelStruct.d_modelPar_WDWG_dX=0.0;
modelStruct.modelPar_given_WDWG=mod.given("WDWG");
modelStruct.modelPar_WDWB=mod.WDWB;
modelStruct.d_modelPar_WDWB_dX=0.0;
modelStruct.modelPar_given_WDWB=mod.given("WDWB");
modelStruct.modelPar_WVOFF=mod.WVOFF;
modelStruct.d_modelPar_WVOFF_dX=0.0;
modelStruct.modelPar_given_WVOFF=mod.given("WVOFF");
modelStruct.modelPar_WETA0=mod.WETA0;
modelStruct.d_modelPar_WETA0_dX=0.0;
modelStruct.modelPar_given_WETA0=mod.given("WETA0");
modelStruct.modelPar_WETAB=mod.WETAB;
modelStruct.d_modelPar_WETAB_dX=0.0;
modelStruct.modelPar_given_WETAB=mod.given("WETAB");
modelStruct.modelPar_WETA0CV=mod.WETA0CV;
modelStruct.d_modelPar_WETA0CV_dX=0.0;
modelStruct.modelPar_given_WETA0CV=mod.given("WETA0CV");
modelStruct.modelPar_WETABCV=mod.WETABCV;
modelStruct.d_modelPar_WETABCV_dX=0.0;
modelStruct.modelPar_given_WETABCV=mod.given("WETABCV");
modelStruct.modelPar_WDSUB=mod.WDSUB;
modelStruct.d_modelPar_WDSUB_dX=0.0;
modelStruct.modelPar_given_WDSUB=mod.given("WDSUB");
modelStruct.modelPar_WCIT=mod.WCIT;
modelStruct.d_modelPar_WCIT_dX=0.0;
modelStruct.modelPar_given_WCIT=mod.given("WCIT");
modelStruct.modelPar_WCDSC=mod.WCDSC;
modelStruct.d_modelPar_WCDSC_dX=0.0;
modelStruct.modelPar_given_WCDSC=mod.given("WCDSC");
modelStruct.modelPar_WCDSCB=mod.WCDSCB;
modelStruct.d_modelPar_WCDSCB_dX=0.0;
modelStruct.modelPar_given_WCDSCB=mod.given("WCDSCB");
modelStruct.modelPar_WCDSCD=mod.WCDSCD;
modelStruct.d_modelPar_WCDSCD_dX=0.0;
modelStruct.modelPar_given_WCDSCD=mod.given("WCDSCD");
modelStruct.modelPar_WPCLM=mod.WPCLM;
modelStruct.d_modelPar_WPCLM_dX=0.0;
modelStruct.modelPar_given_WPCLM=mod.given("WPCLM");
modelStruct.modelPar_WPDIBLC1=mod.WPDIBLC1;
modelStruct.d_modelPar_WPDIBLC1_dX=0.0;
modelStruct.modelPar_given_WPDIBLC1=mod.given("WPDIBLC1");
modelStruct.modelPar_WPDIBLC2=mod.WPDIBLC2;
modelStruct.d_modelPar_WPDIBLC2_dX=0.0;
modelStruct.modelPar_given_WPDIBLC2=mod.given("WPDIBLC2");
modelStruct.modelPar_WPDIBLCB=mod.WPDIBLCB;
modelStruct.d_modelPar_WPDIBLCB_dX=0.0;
modelStruct.modelPar_given_WPDIBLCB=mod.given("WPDIBLCB");
modelStruct.modelPar_WDROUT=mod.WDROUT;
modelStruct.d_modelPar_WDROUT_dX=0.0;
modelStruct.modelPar_given_WDROUT=mod.given("WDROUT");
modelStruct.modelPar_WPVAG=mod.WPVAG;
modelStruct.d_modelPar_WPVAG_dX=0.0;
modelStruct.modelPar_given_WPVAG=mod.given("WPVAG");
modelStruct.modelPar_WDELTA=mod.WDELTA;
modelStruct.d_modelPar_WDELTA_dX=0.0;
modelStruct.modelPar_given_WDELTA=mod.given("WDELTA");
modelStruct.modelPar_WALPHA0=mod.WALPHA0;
modelStruct.d_modelPar_WALPHA0_dX=0.0;
modelStruct.modelPar_given_WALPHA0=mod.given("WALPHA0");
modelStruct.modelPar_WFBJTII=mod.WFBJTII;
modelStruct.d_modelPar_WFBJTII_dX=0.0;
modelStruct.modelPar_given_WFBJTII=mod.given("WFBJTII");
modelStruct.modelPar_WABJTII=mod.WABJTII;
modelStruct.d_modelPar_WABJTII_dX=0.0;
modelStruct.modelPar_given_WABJTII=mod.given("WABJTII");
modelStruct.modelPar_WCBJTII=mod.WCBJTII;
modelStruct.d_modelPar_WCBJTII_dX=0.0;
modelStruct.modelPar_given_WCBJTII=mod.given("WCBJTII");
modelStruct.modelPar_WEBJTII=mod.WEBJTII;
modelStruct.d_modelPar_WEBJTII_dX=0.0;
modelStruct.modelPar_given_WEBJTII=mod.given("WEBJTII");
modelStruct.modelPar_WMBJTII=mod.WMBJTII;
modelStruct.d_modelPar_WMBJTII_dX=0.0;
modelStruct.modelPar_given_WMBJTII=mod.given("WMBJTII");
modelStruct.modelPar_WVBCI=mod.WVBCI;
modelStruct.d_modelPar_WVBCI_dX=0.0;
modelStruct.modelPar_given_WVBCI=mod.given("WVBCI");
modelStruct.modelPar_WBETA0=mod.WBETA0;
modelStruct.d_modelPar_WBETA0_dX=0.0;
modelStruct.modelPar_given_WBETA0=mod.given("WBETA0");
modelStruct.modelPar_WBETA1=mod.WBETA1;
modelStruct.d_modelPar_WBETA1_dX=0.0;
modelStruct.modelPar_given_WBETA1=mod.given("WBETA1");
modelStruct.modelPar_WBETA2=mod.WBETA2;
modelStruct.d_modelPar_WBETA2_dX=0.0;
modelStruct.modelPar_given_WBETA2=mod.given("WBETA2");
modelStruct.modelPar_WVDSATII0=mod.WVDSATII0;
modelStruct.d_modelPar_WVDSATII0_dX=0.0;
modelStruct.modelPar_given_WVDSATII0=mod.given("WVDSATII0");
modelStruct.modelPar_WLII=mod.WLII;
modelStruct.d_modelPar_WLII_dX=0.0;
modelStruct.modelPar_given_WLII=mod.given("WLII");
modelStruct.modelPar_WESATII=mod.WESATII;
modelStruct.d_modelPar_WESATII_dX=0.0;
modelStruct.modelPar_given_WESATII=mod.given("WESATII");
modelStruct.modelPar_WSII0=mod.WSII0;
modelStruct.d_modelPar_WSII0_dX=0.0;
modelStruct.modelPar_given_WSII0=mod.given("WSII0");
modelStruct.modelPar_WSII1=mod.WSII1;
modelStruct.d_modelPar_WSII1_dX=0.0;
modelStruct.modelPar_given_WSII1=mod.given("WSII1");
modelStruct.modelPar_WSII2=mod.WSII2;
modelStruct.d_modelPar_WSII2_dX=0.0;
modelStruct.modelPar_given_WSII2=mod.given("WSII2");
modelStruct.modelPar_WSIID=mod.WSIID;
modelStruct.d_modelPar_WSIID_dX=0.0;
modelStruct.modelPar_given_WSIID=mod.given("WSIID");
modelStruct.modelPar_WAGIDL=mod.WAGIDL;
modelStruct.d_modelPar_WAGIDL_dX=0.0;
modelStruct.modelPar_given_WAGIDL=mod.given("WAGIDL");
modelStruct.modelPar_WBGIDL=mod.WBGIDL;
modelStruct.d_modelPar_WBGIDL_dX=0.0;
modelStruct.modelPar_given_WBGIDL=mod.given("WBGIDL");
modelStruct.modelPar_WCGIDL=mod.WCGIDL;
modelStruct.d_modelPar_WCGIDL_dX=0.0;
modelStruct.modelPar_given_WCGIDL=mod.given("WCGIDL");
modelStruct.modelPar_WRGIDL=mod.WRGIDL;
modelStruct.d_modelPar_WRGIDL_dX=0.0;
modelStruct.modelPar_given_WRGIDL=mod.given("WRGIDL");
modelStruct.modelPar_WKGIDL=mod.WKGIDL;
modelStruct.d_modelPar_WKGIDL_dX=0.0;
modelStruct.modelPar_given_WKGIDL=mod.given("WKGIDL");
modelStruct.modelPar_WFGIDL=mod.WFGIDL;
modelStruct.d_modelPar_WFGIDL_dX=0.0;
modelStruct.modelPar_given_WFGIDL=mod.given("WFGIDL");
modelStruct.modelPar_WAGISL=mod.WAGISL;
modelStruct.d_modelPar_WAGISL_dX=0.0;
modelStruct.modelPar_given_WAGISL=mod.given("WAGISL");
modelStruct.modelPar_WBGISL=mod.WBGISL;
modelStruct.d_modelPar_WBGISL_dX=0.0;
modelStruct.modelPar_given_WBGISL=mod.given("WBGISL");
modelStruct.modelPar_WCGISL=mod.WCGISL;
modelStruct.d_modelPar_WCGISL_dX=0.0;
modelStruct.modelPar_given_WCGISL=mod.given("WCGISL");
modelStruct.modelPar_WRGISL=mod.WRGISL;
modelStruct.d_modelPar_WRGISL_dX=0.0;
modelStruct.modelPar_given_WRGISL=mod.given("WRGISL");
modelStruct.modelPar_WKGISL=mod.WKGISL;
modelStruct.d_modelPar_WKGISL_dX=0.0;
modelStruct.modelPar_given_WKGISL=mod.given("WKGISL");
modelStruct.modelPar_WFGISL=mod.WFGISL;
modelStruct.d_modelPar_WFGISL_dX=0.0;
modelStruct.modelPar_given_WFGISL=mod.given("WFGISL");
modelStruct.modelPar_WNTUN=mod.WNTUN;
modelStruct.d_modelPar_WNTUN_dX=0.0;
modelStruct.modelPar_given_WNTUN=mod.given("WNTUN");
modelStruct.modelPar_WNTUND=mod.WNTUND;
modelStruct.d_modelPar_WNTUND_dX=0.0;
modelStruct.modelPar_given_WNTUND=mod.given("WNTUND");
modelStruct.modelPar_WNDIODE=mod.WNDIODE;
modelStruct.d_modelPar_WNDIODE_dX=0.0;
modelStruct.modelPar_given_WNDIODE=mod.given("WNDIODE");
modelStruct.modelPar_WNDIODED=mod.WNDIODED;
modelStruct.d_modelPar_WNDIODED_dX=0.0;
modelStruct.modelPar_given_WNDIODED=mod.given("WNDIODED");
modelStruct.modelPar_WNRECF0=mod.WNRECF0;
modelStruct.d_modelPar_WNRECF0_dX=0.0;
modelStruct.modelPar_given_WNRECF0=mod.given("WNRECF0");
modelStruct.modelPar_WNRECF0D=mod.WNRECF0D;
modelStruct.d_modelPar_WNRECF0D_dX=0.0;
modelStruct.modelPar_given_WNRECF0D=mod.given("WNRECF0D");
modelStruct.modelPar_WNRECR0=mod.WNRECR0;
modelStruct.d_modelPar_WNRECR0_dX=0.0;
modelStruct.modelPar_given_WNRECR0=mod.given("WNRECR0");
modelStruct.modelPar_WNRECR0D=mod.WNRECR0D;
modelStruct.d_modelPar_WNRECR0D_dX=0.0;
modelStruct.modelPar_given_WNRECR0D=mod.given("WNRECR0D");
modelStruct.modelPar_WISBJT=mod.WISBJT;
modelStruct.d_modelPar_WISBJT_dX=0.0;
modelStruct.modelPar_given_WISBJT=mod.given("WISBJT");
modelStruct.modelPar_WIDBJT=mod.WIDBJT;
modelStruct.d_modelPar_WIDBJT_dX=0.0;
modelStruct.modelPar_given_WIDBJT=mod.given("WIDBJT");
modelStruct.modelPar_WISDIF=mod.WISDIF;
modelStruct.d_modelPar_WISDIF_dX=0.0;
modelStruct.modelPar_given_WISDIF=mod.given("WISDIF");
modelStruct.modelPar_WIDDIF=mod.WIDDIF;
modelStruct.d_modelPar_WIDDIF_dX=0.0;
modelStruct.modelPar_given_WIDDIF=mod.given("WIDDIF");
modelStruct.modelPar_WISREC=mod.WISREC;
modelStruct.d_modelPar_WISREC_dX=0.0;
modelStruct.modelPar_given_WISREC=mod.given("WISREC");
modelStruct.modelPar_WIDREC=mod.WIDREC;
modelStruct.d_modelPar_WIDREC_dX=0.0;
modelStruct.modelPar_given_WIDREC=mod.given("WIDREC");
modelStruct.modelPar_WISTUN=mod.WISTUN;
modelStruct.d_modelPar_WISTUN_dX=0.0;
modelStruct.modelPar_given_WISTUN=mod.given("WISTUN");
modelStruct.modelPar_WIDTUN=mod.WIDTUN;
modelStruct.d_modelPar_WIDTUN_dX=0.0;
modelStruct.modelPar_given_WIDTUN=mod.given("WIDTUN");
modelStruct.modelPar_WVREC0=mod.WVREC0;
modelStruct.d_modelPar_WVREC0_dX=0.0;
modelStruct.modelPar_given_WVREC0=mod.given("WVREC0");
modelStruct.modelPar_WVREC0D=mod.WVREC0D;
modelStruct.d_modelPar_WVREC0D_dX=0.0;
modelStruct.modelPar_given_WVREC0D=mod.given("WVREC0D");
modelStruct.modelPar_WVTUN0=mod.WVTUN0;
modelStruct.d_modelPar_WVTUN0_dX=0.0;
modelStruct.modelPar_given_WVTUN0=mod.given("WVTUN0");
modelStruct.modelPar_WVTUN0D=mod.WVTUN0D;
modelStruct.d_modelPar_WVTUN0D_dX=0.0;
modelStruct.modelPar_given_WVTUN0D=mod.given("WVTUN0D");
modelStruct.modelPar_WNBJT=mod.WNBJT;
modelStruct.d_modelPar_WNBJT_dX=0.0;
modelStruct.modelPar_given_WNBJT=mod.given("WNBJT");
modelStruct.modelPar_WLBJT0=mod.WLBJT0;
modelStruct.d_modelPar_WLBJT0_dX=0.0;
modelStruct.modelPar_given_WLBJT0=mod.given("WLBJT0");
modelStruct.modelPar_WVABJT=mod.WVABJT;
modelStruct.d_modelPar_WVABJT_dX=0.0;
modelStruct.modelPar_given_WVABJT=mod.given("WVABJT");
modelStruct.modelPar_WAELY=mod.WAELY;
modelStruct.d_modelPar_WAELY_dX=0.0;
modelStruct.modelPar_given_WAELY=mod.given("WAELY");
modelStruct.modelPar_WAHLI=mod.WAHLI;
modelStruct.d_modelPar_WAHLI_dX=0.0;
modelStruct.modelPar_given_WAHLI=mod.given("WAHLI");
modelStruct.modelPar_WAHLID=mod.WAHLID;
modelStruct.d_modelPar_WAHLID_dX=0.0;
modelStruct.modelPar_given_WAHLID=mod.given("WAHLID");
modelStruct.modelPar_WVSDFB=mod.WVSDFB;
modelStruct.d_modelPar_WVSDFB_dX=0.0;
modelStruct.modelPar_given_WVSDFB=mod.given("WVSDFB");
modelStruct.modelPar_WVSDTH=mod.WVSDTH;
modelStruct.d_modelPar_WVSDTH_dX=0.0;
modelStruct.modelPar_given_WVSDTH=mod.given("WVSDTH");
modelStruct.modelPar_WDELVT=mod.WDELVT;
modelStruct.d_modelPar_WDELVT_dX=0.0;
modelStruct.modelPar_given_WDELVT=mod.given("WDELVT");
modelStruct.modelPar_WACDE=mod.WACDE;
modelStruct.d_modelPar_WACDE_dX=0.0;
modelStruct.modelPar_given_WACDE=mod.given("WACDE");
modelStruct.modelPar_WMOIN=mod.WMOIN;
modelStruct.d_modelPar_WMOIN_dX=0.0;
modelStruct.modelPar_given_WMOIN=mod.given("WMOIN");
modelStruct.modelPar_WNOFF=mod.WNOFF;
modelStruct.d_modelPar_WNOFF_dX=0.0;
modelStruct.modelPar_given_WNOFF=mod.given("WNOFF");
modelStruct.modelPar_WNOFF2=mod.WNOFF2;
modelStruct.d_modelPar_WNOFF2_dX=0.0;
modelStruct.modelPar_given_WNOFF2=mod.given("WNOFF2");
modelStruct.modelPar_WXRCRG1=mod.WXRCRG1;
modelStruct.d_modelPar_WXRCRG1_dX=0.0;
modelStruct.modelPar_given_WXRCRG1=mod.given("WXRCRG1");
modelStruct.modelPar_WXRCRG2=mod.WXRCRG2;
modelStruct.d_modelPar_WXRCRG2_dX=0.0;
modelStruct.modelPar_given_WXRCRG2=mod.given("WXRCRG2");
modelStruct.modelPar_WVBSA=mod.WVBSA;
modelStruct.d_modelPar_WVBSA_dX=0.0;
modelStruct.modelPar_given_WVBSA=mod.given("WVBSA");
modelStruct.modelPar_WVSCE=mod.WVSCE;
modelStruct.d_modelPar_WVSCE_dX=0.0;
modelStruct.modelPar_given_WVSCE=mod.given("WVSCE");
modelStruct.modelPar_WCDSBS=mod.WCDSBS;
modelStruct.d_modelPar_WCDSBS_dX=0.0;
modelStruct.modelPar_given_WCDSBS=mod.given("WCDSBS");
modelStruct.modelPar_WNOFFFD=mod.WNOFFFD;
modelStruct.d_modelPar_WNOFFFD_dX=0.0;
modelStruct.modelPar_given_WNOFFFD=mod.given("WNOFFFD");
modelStruct.modelPar_WVOFFFD=mod.WVOFFFD;
modelStruct.d_modelPar_WVOFFFD_dX=0.0;
modelStruct.modelPar_given_WVOFFFD=mod.given("WVOFFFD");
modelStruct.modelPar_WK1B=mod.WK1B;
modelStruct.d_modelPar_WK1B_dX=0.0;
modelStruct.modelPar_given_WK1B=mod.given("WK1B");
modelStruct.modelPar_WK2B=mod.WK2B;
modelStruct.d_modelPar_WK2B_dX=0.0;
modelStruct.modelPar_given_WK2B=mod.given("WK2B");
modelStruct.modelPar_WDK2B=mod.WDK2B;
modelStruct.d_modelPar_WDK2B_dX=0.0;
modelStruct.modelPar_given_WDK2B=mod.given("WDK2B");
modelStruct.modelPar_WDVBD0=mod.WDVBD0;
modelStruct.d_modelPar_WDVBD0_dX=0.0;
modelStruct.modelPar_given_WDVBD0=mod.given("WDVBD0");
modelStruct.modelPar_WDVBD1=mod.WDVBD1;
modelStruct.d_modelPar_WDVBD1_dX=0.0;
modelStruct.modelPar_given_WDVBD1=mod.given("WDVBD1");
modelStruct.modelPar_WMOINFD=mod.WMOINFD;
modelStruct.d_modelPar_WMOINFD_dX=0.0;
modelStruct.modelPar_given_WMOINFD=mod.given("WMOINFD");
modelStruct.modelPar_WVBS0PD=mod.WVBS0PD;
modelStruct.d_modelPar_WVBS0PD_dX=0.0;
modelStruct.modelPar_given_WVBS0PD=mod.given("WVBS0PD");
modelStruct.modelPar_WVBS0FD=mod.WVBS0FD;
modelStruct.d_modelPar_WVBS0FD_dX=0.0;
modelStruct.modelPar_given_WVBS0FD=mod.given("WVBS0FD");
modelStruct.modelPar_PXJ=mod.PXJ;
modelStruct.d_modelPar_PXJ_dX=0.0;
modelStruct.modelPar_given_PXJ=mod.given("PXJ");
modelStruct.modelPar_PALPHAGB1=mod.PALPHAGB1;
modelStruct.d_modelPar_PALPHAGB1_dX=0.0;
modelStruct.modelPar_given_PALPHAGB1=mod.given("PALPHAGB1");
modelStruct.modelPar_PBETAGB1=mod.PBETAGB1;
modelStruct.d_modelPar_PBETAGB1_dX=0.0;
modelStruct.modelPar_given_PBETAGB1=mod.given("PBETAGB1");
modelStruct.modelPar_PALPHAGB2=mod.PALPHAGB2;
modelStruct.d_modelPar_PALPHAGB2_dX=0.0;
modelStruct.modelPar_given_PALPHAGB2=mod.given("PALPHAGB2");
modelStruct.modelPar_PBETAGB2=mod.PBETAGB2;
modelStruct.d_modelPar_PBETAGB2_dX=0.0;
modelStruct.modelPar_given_PBETAGB2=mod.given("PBETAGB2");
modelStruct.modelPar_PAIGBCP2=mod.PAIGBCP2;
modelStruct.d_modelPar_PAIGBCP2_dX=0.0;
modelStruct.modelPar_given_PAIGBCP2=mod.given("PAIGBCP2");
modelStruct.modelPar_PBIGBCP2=mod.PBIGBCP2;
modelStruct.d_modelPar_PBIGBCP2_dX=0.0;
modelStruct.modelPar_given_PBIGBCP2=mod.given("PBIGBCP2");
modelStruct.modelPar_PCIGBCP2=mod.PCIGBCP2;
modelStruct.d_modelPar_PCIGBCP2_dX=0.0;
modelStruct.modelPar_given_PCIGBCP2=mod.given("PCIGBCP2");
modelStruct.modelPar_PCGSL=mod.PCGSL;
modelStruct.d_modelPar_PCGSL_dX=0.0;
modelStruct.modelPar_given_PCGSL=mod.given("PCGSL");
modelStruct.modelPar_PCGDL=mod.PCGDL;
modelStruct.d_modelPar_PCGDL_dX=0.0;
modelStruct.modelPar_given_PCGDL=mod.given("PCGDL");
modelStruct.modelPar_PCKAPPA=mod.PCKAPPA;
modelStruct.d_modelPar_PCKAPPA_dX=0.0;
modelStruct.modelPar_given_PCKAPPA=mod.given("PCKAPPA");
modelStruct.modelPar_PNDIF=mod.PNDIF;
modelStruct.d_modelPar_PNDIF_dX=0.0;
modelStruct.modelPar_given_PNDIF=mod.given("PNDIF");
modelStruct.modelPar_PUTE=mod.PUTE;
modelStruct.d_modelPar_PUTE_dX=0.0;
modelStruct.modelPar_given_PUTE=mod.given("PUTE");
modelStruct.modelPar_PKT1=mod.PKT1;
modelStruct.d_modelPar_PKT1_dX=0.0;
modelStruct.modelPar_given_PKT1=mod.given("PKT1");
modelStruct.modelPar_PKT1L=mod.PKT1L;
modelStruct.d_modelPar_PKT1L_dX=0.0;
modelStruct.modelPar_given_PKT1L=mod.given("PKT1L");
modelStruct.modelPar_PKT2=mod.PKT2;
modelStruct.d_modelPar_PKT2_dX=0.0;
modelStruct.modelPar_given_PKT2=mod.given("PKT2");
modelStruct.modelPar_PUA1=mod.PUA1;
modelStruct.d_modelPar_PUA1_dX=0.0;
modelStruct.modelPar_given_PUA1=mod.given("PUA1");
modelStruct.modelPar_PUB1=mod.PUB1;
modelStruct.d_modelPar_PUB1_dX=0.0;
modelStruct.modelPar_given_PUB1=mod.given("PUB1");
modelStruct.modelPar_PUC1=mod.PUC1;
modelStruct.d_modelPar_PUC1_dX=0.0;
modelStruct.modelPar_given_PUC1=mod.given("PUC1");
modelStruct.modelPar_PAT=mod.PAT;
modelStruct.d_modelPar_PAT_dX=0.0;
modelStruct.modelPar_given_PAT=mod.given("PAT");
modelStruct.modelPar_PPRT=mod.PPRT;
modelStruct.d_modelPar_PPRT_dX=0.0;
modelStruct.modelPar_given_PPRT=mod.given("PPRT");
modelStruct.modelPar_PNTRECF=mod.PNTRECF;
modelStruct.d_modelPar_PNTRECF_dX=0.0;
modelStruct.modelPar_given_PNTRECF=mod.given("PNTRECF");
modelStruct.modelPar_PNTRECR=mod.PNTRECR;
modelStruct.d_modelPar_PNTRECR_dX=0.0;
modelStruct.modelPar_given_PNTRECR=mod.given("PNTRECR");
modelStruct.modelPar_PXBJT=mod.PXBJT;
modelStruct.d_modelPar_PXBJT_dX=0.0;
modelStruct.modelPar_given_PXBJT=mod.given("PXBJT");
modelStruct.modelPar_PXDIF=mod.PXDIF;
modelStruct.d_modelPar_PXDIF_dX=0.0;
modelStruct.modelPar_given_PXDIF=mod.given("PXDIF");
modelStruct.modelPar_PXREC=mod.PXREC;
modelStruct.d_modelPar_PXREC_dX=0.0;
modelStruct.modelPar_given_PXREC=mod.given("PXREC");
modelStruct.modelPar_PXTUN=mod.PXTUN;
modelStruct.d_modelPar_PXTUN_dX=0.0;
modelStruct.modelPar_given_PXTUN=mod.given("PXTUN");
modelStruct.modelPar_PXDIFD=mod.PXDIFD;
modelStruct.d_modelPar_PXDIFD_dX=0.0;
modelStruct.modelPar_given_PXDIFD=mod.given("PXDIFD");
modelStruct.modelPar_PXRECD=mod.PXRECD;
modelStruct.d_modelPar_PXRECD_dX=0.0;
modelStruct.modelPar_given_PXRECD=mod.given("PXRECD");
modelStruct.modelPar_PXTUND=mod.PXTUND;
modelStruct.d_modelPar_PXTUND_dX=0.0;
modelStruct.modelPar_given_PXTUND=mod.given("PXTUND");
modelStruct.modelPar_PAIGC=mod.PAIGC;
modelStruct.d_modelPar_PAIGC_dX=0.0;
modelStruct.modelPar_given_PAIGC=mod.given("PAIGC");
modelStruct.modelPar_PBIGC=mod.PBIGC;
modelStruct.d_modelPar_PBIGC_dX=0.0;
modelStruct.modelPar_given_PBIGC=mod.given("PBIGC");
modelStruct.modelPar_PCIGC=mod.PCIGC;
modelStruct.d_modelPar_PCIGC_dX=0.0;
modelStruct.modelPar_given_PCIGC=mod.given("PCIGC");
modelStruct.modelPar_PAIGSD=mod.PAIGSD;
modelStruct.d_modelPar_PAIGSD_dX=0.0;
modelStruct.modelPar_given_PAIGSD=mod.given("PAIGSD");
modelStruct.modelPar_PBIGSD=mod.PBIGSD;
modelStruct.d_modelPar_PBIGSD_dX=0.0;
modelStruct.modelPar_given_PBIGSD=mod.given("PBIGSD");
modelStruct.modelPar_PCIGSD=mod.PCIGSD;
modelStruct.d_modelPar_PCIGSD_dX=0.0;
modelStruct.modelPar_given_PCIGSD=mod.given("PCIGSD");
modelStruct.modelPar_PNIGC=mod.PNIGC;
modelStruct.d_modelPar_PNIGC_dX=0.0;
modelStruct.modelPar_given_PNIGC=mod.given("PNIGC");
modelStruct.modelPar_PPIGCD=mod.PPIGCD;
modelStruct.d_modelPar_PPIGCD_dX=0.0;
modelStruct.modelPar_given_PPIGCD=mod.given("PPIGCD");
modelStruct.modelPar_PPOXEDGE=mod.PPOXEDGE;
modelStruct.d_modelPar_PPOXEDGE_dX=0.0;
modelStruct.modelPar_given_PPOXEDGE=mod.given("PPOXEDGE");
modelStruct.modelPar_PNCH=mod.PNCH;
modelStruct.d_modelPar_PNCH_dX=0.0;
modelStruct.modelPar_given_PNCH=mod.given("PNCH");
modelStruct.modelPar_PNSUB=mod.PNSUB;
modelStruct.d_modelPar_PNSUB_dX=0.0;
modelStruct.modelPar_given_PNSUB=mod.given("PNSUB");
modelStruct.modelPar_PNSD=mod.PNSD;
modelStruct.d_modelPar_PNSD_dX=0.0;
modelStruct.modelPar_given_PNSD=mod.given("PNSD");
modelStruct.modelPar_PNGATE=mod.PNGATE;
modelStruct.d_modelPar_PNGATE_dX=0.0;
modelStruct.modelPar_given_PNGATE=mod.given("PNGATE");
modelStruct.modelPar_PVTH0=mod.PVTH0;
modelStruct.d_modelPar_PVTH0_dX=0.0;
modelStruct.modelPar_given_PVTH0=mod.given("PVTH0");
modelStruct.modelPar_PVFB=mod.PVFB;
modelStruct.d_modelPar_PVFB_dX=0.0;
modelStruct.modelPar_given_PVFB=mod.given("PVFB");
modelStruct.modelPar_PK1=mod.PK1;
modelStruct.d_modelPar_PK1_dX=0.0;
modelStruct.modelPar_given_PK1=mod.given("PK1");
modelStruct.modelPar_PK1W1=mod.PK1W1;
modelStruct.d_modelPar_PK1W1_dX=0.0;
modelStruct.modelPar_given_PK1W1=mod.given("PK1W1");
modelStruct.modelPar_PK1W2=mod.PK1W2;
modelStruct.d_modelPar_PK1W2_dX=0.0;
modelStruct.modelPar_given_PK1W2=mod.given("PK1W2");
modelStruct.modelPar_PK2=mod.PK2;
modelStruct.d_modelPar_PK2_dX=0.0;
modelStruct.modelPar_given_PK2=mod.given("PK2");
modelStruct.modelPar_PK3=mod.PK3;
modelStruct.d_modelPar_PK3_dX=0.0;
modelStruct.modelPar_given_PK3=mod.given("PK3");
modelStruct.modelPar_PK3B=mod.PK3B;
modelStruct.d_modelPar_PK3B_dX=0.0;
modelStruct.modelPar_given_PK3B=mod.given("PK3B");
modelStruct.modelPar_PKB1=mod.PKB1;
modelStruct.d_modelPar_PKB1_dX=0.0;
modelStruct.modelPar_given_PKB1=mod.given("PKB1");
modelStruct.modelPar_PW0=mod.PW0;
modelStruct.d_modelPar_PW0_dX=0.0;
modelStruct.modelPar_given_PW0=mod.given("PW0");
modelStruct.modelPar_PLPEB=mod.PLPEB;
modelStruct.d_modelPar_PLPEB_dX=0.0;
modelStruct.modelPar_given_PLPEB=mod.given("PLPEB");
modelStruct.modelPar_PDVT0=mod.PDVT0;
modelStruct.d_modelPar_PDVT0_dX=0.0;
modelStruct.modelPar_given_PDVT0=mod.given("PDVT0");
modelStruct.modelPar_PDVT1=mod.PDVT1;
modelStruct.d_modelPar_PDVT1_dX=0.0;
modelStruct.modelPar_given_PDVT1=mod.given("PDVT1");
modelStruct.modelPar_PDVT2=mod.PDVT2;
modelStruct.d_modelPar_PDVT2_dX=0.0;
modelStruct.modelPar_given_PDVT2=mod.given("PDVT2");
modelStruct.modelPar_PDVT0W=mod.PDVT0W;
modelStruct.d_modelPar_PDVT0W_dX=0.0;
modelStruct.modelPar_given_PDVT0W=mod.given("PDVT0W");
modelStruct.modelPar_PDVT1W=mod.PDVT1W;
modelStruct.d_modelPar_PDVT1W_dX=0.0;
modelStruct.modelPar_given_PDVT1W=mod.given("PDVT1W");
modelStruct.modelPar_PDVT2W=mod.PDVT2W;
modelStruct.d_modelPar_PDVT2W_dX=0.0;
modelStruct.modelPar_given_PDVT2W=mod.given("PDVT2W");
modelStruct.modelPar_PU0=mod.PU0;
modelStruct.d_modelPar_PU0_dX=0.0;
modelStruct.modelPar_given_PU0=mod.given("PU0");
modelStruct.modelPar_PEU=mod.PEU;
modelStruct.d_modelPar_PEU_dX=0.0;
modelStruct.modelPar_given_PEU=mod.given("PEU");
modelStruct.modelPar_PUA=mod.PUA;
modelStruct.d_modelPar_PUA_dX=0.0;
modelStruct.modelPar_given_PUA=mod.given("PUA");
modelStruct.modelPar_PUB=mod.PUB;
modelStruct.d_modelPar_PUB_dX=0.0;
modelStruct.modelPar_given_PUB=mod.given("PUB");
modelStruct.modelPar_PUC=mod.PUC;
modelStruct.d_modelPar_PUC_dX=0.0;
modelStruct.modelPar_given_PUC=mod.given("PUC");
modelStruct.modelPar_PUD=mod.PUD;
modelStruct.d_modelPar_PUD_dX=0.0;
modelStruct.modelPar_given_PUD=mod.given("PUD");
modelStruct.modelPar_PUD1=mod.PUD1;
modelStruct.d_modelPar_PUD1_dX=0.0;
modelStruct.modelPar_given_PUD1=mod.given("PUD1");
modelStruct.modelPar_PUCSTE=mod.PUCSTE;
modelStruct.d_modelPar_PUCSTE_dX=0.0;
modelStruct.modelPar_given_PUCSTE=mod.given("PUCSTE");
modelStruct.modelPar_PUCS=mod.PUCS;
modelStruct.d_modelPar_PUCS_dX=0.0;
modelStruct.modelPar_given_PUCS=mod.given("PUCS");
modelStruct.modelPar_PVSAT=mod.PVSAT;
modelStruct.d_modelPar_PVSAT_dX=0.0;
modelStruct.modelPar_given_PVSAT=mod.given("PVSAT");
modelStruct.modelPar_PA0=mod.PA0;
modelStruct.d_modelPar_PA0_dX=0.0;
modelStruct.modelPar_given_PA0=mod.given("PA0");
modelStruct.modelPar_PAGS=mod.PAGS;
modelStruct.d_modelPar_PAGS_dX=0.0;
modelStruct.modelPar_given_PAGS=mod.given("PAGS");
modelStruct.modelPar_PB0=mod.PB0;
modelStruct.d_modelPar_PB0_dX=0.0;
modelStruct.modelPar_given_PB0=mod.given("PB0");
modelStruct.modelPar_PB1=mod.PB1;
modelStruct.d_modelPar_PB1_dX=0.0;
modelStruct.modelPar_given_PB1=mod.given("PB1");
modelStruct.modelPar_PKETA=mod.PKETA;
modelStruct.d_modelPar_PKETA_dX=0.0;
modelStruct.modelPar_given_PKETA=mod.given("PKETA");
modelStruct.modelPar_PKETAS=mod.PKETAS;
modelStruct.d_modelPar_PKETAS_dX=0.0;
modelStruct.modelPar_given_PKETAS=mod.given("PKETAS");
modelStruct.modelPar_PA1=mod.PA1;
modelStruct.d_modelPar_PA1_dX=0.0;
modelStruct.modelPar_given_PA1=mod.given("PA1");
modelStruct.modelPar_PA2=mod.PA2;
modelStruct.d_modelPar_PA2_dX=0.0;
modelStruct.modelPar_given_PA2=mod.given("PA2");
modelStruct.modelPar_PRDSW=mod.PRDSW;
modelStruct.d_modelPar_PRDSW_dX=0.0;
modelStruct.modelPar_given_PRDSW=mod.given("PRDSW");
modelStruct.modelPar_PRSW=mod.PRSW;
modelStruct.d_modelPar_PRSW_dX=0.0;
modelStruct.modelPar_given_PRSW=mod.given("PRSW");
modelStruct.modelPar_PRDW=mod.PRDW;
modelStruct.d_modelPar_PRDW_dX=0.0;
modelStruct.modelPar_given_PRDW=mod.given("PRDW");
modelStruct.modelPar_PPRWB=mod.PPRWB;
modelStruct.d_modelPar_PPRWB_dX=0.0;
modelStruct.modelPar_given_PPRWB=mod.given("PPRWB");
modelStruct.modelPar_PPRWG=mod.PPRWG;
modelStruct.d_modelPar_PPRWG_dX=0.0;
modelStruct.modelPar_given_PPRWG=mod.given("PPRWG");
modelStruct.modelPar_PWR=mod.PWR;
modelStruct.d_modelPar_PWR_dX=0.0;
modelStruct.modelPar_given_PWR=mod.given("PWR");
modelStruct.modelPar_PNFACTOR=mod.PNFACTOR;
modelStruct.d_modelPar_PNFACTOR_dX=0.0;
modelStruct.modelPar_given_PNFACTOR=mod.given("PNFACTOR");
modelStruct.modelPar_PDWG=mod.PDWG;
modelStruct.d_modelPar_PDWG_dX=0.0;
modelStruct.modelPar_given_PDWG=mod.given("PDWG");
modelStruct.modelPar_PDWB=mod.PDWB;
modelStruct.d_modelPar_PDWB_dX=0.0;
modelStruct.modelPar_given_PDWB=mod.given("PDWB");
modelStruct.modelPar_PVOFF=mod.PVOFF;
modelStruct.d_modelPar_PVOFF_dX=0.0;
modelStruct.modelPar_given_PVOFF=mod.given("PVOFF");
modelStruct.modelPar_PETA0=mod.PETA0;
modelStruct.d_modelPar_PETA0_dX=0.0;
modelStruct.modelPar_given_PETA0=mod.given("PETA0");
modelStruct.modelPar_PETAB=mod.PETAB;
modelStruct.d_modelPar_PETAB_dX=0.0;
modelStruct.modelPar_given_PETAB=mod.given("PETAB");
modelStruct.modelPar_PETA0CV=mod.PETA0CV;
modelStruct.d_modelPar_PETA0CV_dX=0.0;
modelStruct.modelPar_given_PETA0CV=mod.given("PETA0CV");
modelStruct.modelPar_PETABCV=mod.PETABCV;
modelStruct.d_modelPar_PETABCV_dX=0.0;
modelStruct.modelPar_given_PETABCV=mod.given("PETABCV");
modelStruct.modelPar_PDSUB=mod.PDSUB;
modelStruct.d_modelPar_PDSUB_dX=0.0;
modelStruct.modelPar_given_PDSUB=mod.given("PDSUB");
modelStruct.modelPar_PCIT=mod.PCIT;
modelStruct.d_modelPar_PCIT_dX=0.0;
modelStruct.modelPar_given_PCIT=mod.given("PCIT");
modelStruct.modelPar_PCDSC=mod.PCDSC;
modelStruct.d_modelPar_PCDSC_dX=0.0;
modelStruct.modelPar_given_PCDSC=mod.given("PCDSC");
modelStruct.modelPar_PCDSCB=mod.PCDSCB;
modelStruct.d_modelPar_PCDSCB_dX=0.0;
modelStruct.modelPar_given_PCDSCB=mod.given("PCDSCB");
modelStruct.modelPar_PCDSCD=mod.PCDSCD;
modelStruct.d_modelPar_PCDSCD_dX=0.0;
modelStruct.modelPar_given_PCDSCD=mod.given("PCDSCD");
modelStruct.modelPar_PPCLM=mod.PPCLM;
modelStruct.d_modelPar_PPCLM_dX=0.0;
modelStruct.modelPar_given_PPCLM=mod.given("PPCLM");
modelStruct.modelPar_PPDIBLC1=mod.PPDIBLC1;
modelStruct.d_modelPar_PPDIBLC1_dX=0.0;
modelStruct.modelPar_given_PPDIBLC1=mod.given("PPDIBLC1");
modelStruct.modelPar_PPDIBLC2=mod.PPDIBLC2;
modelStruct.d_modelPar_PPDIBLC2_dX=0.0;
modelStruct.modelPar_given_PPDIBLC2=mod.given("PPDIBLC2");
modelStruct.modelPar_PPDIBLCB=mod.PPDIBLCB;
modelStruct.d_modelPar_PPDIBLCB_dX=0.0;
modelStruct.modelPar_given_PPDIBLCB=mod.given("PPDIBLCB");
modelStruct.modelPar_PDROUT=mod.PDROUT;
modelStruct.d_modelPar_PDROUT_dX=0.0;
modelStruct.modelPar_given_PDROUT=mod.given("PDROUT");
modelStruct.modelPar_PPVAG=mod.PPVAG;
modelStruct.d_modelPar_PPVAG_dX=0.0;
modelStruct.modelPar_given_PPVAG=mod.given("PPVAG");
modelStruct.modelPar_PDELTA=mod.PDELTA;
modelStruct.d_modelPar_PDELTA_dX=0.0;
modelStruct.modelPar_given_PDELTA=mod.given("PDELTA");
modelStruct.modelPar_PALPHA0=mod.PALPHA0;
modelStruct.d_modelPar_PALPHA0_dX=0.0;
modelStruct.modelPar_given_PALPHA0=mod.given("PALPHA0");
modelStruct.modelPar_PFBJTII=mod.PFBJTII;
modelStruct.d_modelPar_PFBJTII_dX=0.0;
modelStruct.modelPar_given_PFBJTII=mod.given("PFBJTII");
modelStruct.modelPar_PABJTII=mod.PABJTII;
modelStruct.d_modelPar_PABJTII_dX=0.0;
modelStruct.modelPar_given_PABJTII=mod.given("PABJTII");
modelStruct.modelPar_PCBJTII=mod.PCBJTII;
modelStruct.d_modelPar_PCBJTII_dX=0.0;
modelStruct.modelPar_given_PCBJTII=mod.given("PCBJTII");
modelStruct.modelPar_PEBJTII=mod.PEBJTII;
modelStruct.d_modelPar_PEBJTII_dX=0.0;
modelStruct.modelPar_given_PEBJTII=mod.given("PEBJTII");
modelStruct.modelPar_PMBJTII=mod.PMBJTII;
modelStruct.d_modelPar_PMBJTII_dX=0.0;
modelStruct.modelPar_given_PMBJTII=mod.given("PMBJTII");
modelStruct.modelPar_PVBCI=mod.PVBCI;
modelStruct.d_modelPar_PVBCI_dX=0.0;
modelStruct.modelPar_given_PVBCI=mod.given("PVBCI");
modelStruct.modelPar_PBETA0=mod.PBETA0;
modelStruct.d_modelPar_PBETA0_dX=0.0;
modelStruct.modelPar_given_PBETA0=mod.given("PBETA0");
modelStruct.modelPar_PBETA1=mod.PBETA1;
modelStruct.d_modelPar_PBETA1_dX=0.0;
modelStruct.modelPar_given_PBETA1=mod.given("PBETA1");
modelStruct.modelPar_PBETA2=mod.PBETA2;
modelStruct.d_modelPar_PBETA2_dX=0.0;
modelStruct.modelPar_given_PBETA2=mod.given("PBETA2");
modelStruct.modelPar_PVDSATII0=mod.PVDSATII0;
modelStruct.d_modelPar_PVDSATII0_dX=0.0;
modelStruct.modelPar_given_PVDSATII0=mod.given("PVDSATII0");
modelStruct.modelPar_PLII=mod.PLII;
modelStruct.d_modelPar_PLII_dX=0.0;
modelStruct.modelPar_given_PLII=mod.given("PLII");
modelStruct.modelPar_PESATII=mod.PESATII;
modelStruct.d_modelPar_PESATII_dX=0.0;
modelStruct.modelPar_given_PESATII=mod.given("PESATII");
modelStruct.modelPar_PSII0=mod.PSII0;
modelStruct.d_modelPar_PSII0_dX=0.0;
modelStruct.modelPar_given_PSII0=mod.given("PSII0");
modelStruct.modelPar_PSII1=mod.PSII1;
modelStruct.d_modelPar_PSII1_dX=0.0;
modelStruct.modelPar_given_PSII1=mod.given("PSII1");
modelStruct.modelPar_PSII2=mod.PSII2;
modelStruct.d_modelPar_PSII2_dX=0.0;
modelStruct.modelPar_given_PSII2=mod.given("PSII2");
modelStruct.modelPar_PSIID=mod.PSIID;
modelStruct.d_modelPar_PSIID_dX=0.0;
modelStruct.modelPar_given_PSIID=mod.given("PSIID");
modelStruct.modelPar_PAGIDL=mod.PAGIDL;
modelStruct.d_modelPar_PAGIDL_dX=0.0;
modelStruct.modelPar_given_PAGIDL=mod.given("PAGIDL");
modelStruct.modelPar_PBGIDL=mod.PBGIDL;
modelStruct.d_modelPar_PBGIDL_dX=0.0;
modelStruct.modelPar_given_PBGIDL=mod.given("PBGIDL");
modelStruct.modelPar_PCGIDL=mod.PCGIDL;
modelStruct.d_modelPar_PCGIDL_dX=0.0;
modelStruct.modelPar_given_PCGIDL=mod.given("PCGIDL");
modelStruct.modelPar_PRGIDL=mod.PRGIDL;
modelStruct.d_modelPar_PRGIDL_dX=0.0;
modelStruct.modelPar_given_PRGIDL=mod.given("PRGIDL");
modelStruct.modelPar_PKGIDL=mod.PKGIDL;
modelStruct.d_modelPar_PKGIDL_dX=0.0;
modelStruct.modelPar_given_PKGIDL=mod.given("PKGIDL");
modelStruct.modelPar_PFGIDL=mod.PFGIDL;
modelStruct.d_modelPar_PFGIDL_dX=0.0;
modelStruct.modelPar_given_PFGIDL=mod.given("PFGIDL");
modelStruct.modelPar_PAGISL=mod.PAGISL;
modelStruct.d_modelPar_PAGISL_dX=0.0;
modelStruct.modelPar_given_PAGISL=mod.given("PAGISL");
modelStruct.modelPar_PBGISL=mod.PBGISL;
modelStruct.d_modelPar_PBGISL_dX=0.0;
modelStruct.modelPar_given_PBGISL=mod.given("PBGISL");
modelStruct.modelPar_PCGISL=mod.PCGISL;
modelStruct.d_modelPar_PCGISL_dX=0.0;
modelStruct.modelPar_given_PCGISL=mod.given("PCGISL");
modelStruct.modelPar_PRGISL=mod.PRGISL;
modelStruct.d_modelPar_PRGISL_dX=0.0;
modelStruct.modelPar_given_PRGISL=mod.given("PRGISL");
modelStruct.modelPar_PKGISL=mod.PKGISL;
modelStruct.d_modelPar_PKGISL_dX=0.0;
modelStruct.modelPar_given_PKGISL=mod.given("PKGISL");
modelStruct.modelPar_PFGISL=mod.PFGISL;
modelStruct.d_modelPar_PFGISL_dX=0.0;
modelStruct.modelPar_given_PFGISL=mod.given("PFGISL");
modelStruct.modelPar_PNTUN=mod.PNTUN;
modelStruct.d_modelPar_PNTUN_dX=0.0;
modelStruct.modelPar_given_PNTUN=mod.given("PNTUN");
modelStruct.modelPar_PNTUND=mod.PNTUND;
modelStruct.d_modelPar_PNTUND_dX=0.0;
modelStruct.modelPar_given_PNTUND=mod.given("PNTUND");
modelStruct.modelPar_PNDIODE=mod.PNDIODE;
modelStruct.d_modelPar_PNDIODE_dX=0.0;
modelStruct.modelPar_given_PNDIODE=mod.given("PNDIODE");
modelStruct.modelPar_PNDIODED=mod.PNDIODED;
modelStruct.d_modelPar_PNDIODED_dX=0.0;
modelStruct.modelPar_given_PNDIODED=mod.given("PNDIODED");
modelStruct.modelPar_PNRECF0=mod.PNRECF0;
modelStruct.d_modelPar_PNRECF0_dX=0.0;
modelStruct.modelPar_given_PNRECF0=mod.given("PNRECF0");
modelStruct.modelPar_PNRECF0D=mod.PNRECF0D;
modelStruct.d_modelPar_PNRECF0D_dX=0.0;
modelStruct.modelPar_given_PNRECF0D=mod.given("PNRECF0D");
modelStruct.modelPar_PNRECR0=mod.PNRECR0;
modelStruct.d_modelPar_PNRECR0_dX=0.0;
modelStruct.modelPar_given_PNRECR0=mod.given("PNRECR0");
modelStruct.modelPar_PNRECR0D=mod.PNRECR0D;
modelStruct.d_modelPar_PNRECR0D_dX=0.0;
modelStruct.modelPar_given_PNRECR0D=mod.given("PNRECR0D");
modelStruct.modelPar_PISBJT=mod.PISBJT;
modelStruct.d_modelPar_PISBJT_dX=0.0;
modelStruct.modelPar_given_PISBJT=mod.given("PISBJT");
modelStruct.modelPar_PIDBJT=mod.PIDBJT;
modelStruct.d_modelPar_PIDBJT_dX=0.0;
modelStruct.modelPar_given_PIDBJT=mod.given("PIDBJT");
modelStruct.modelPar_PISDIF=mod.PISDIF;
modelStruct.d_modelPar_PISDIF_dX=0.0;
modelStruct.modelPar_given_PISDIF=mod.given("PISDIF");
modelStruct.modelPar_PIDDIF=mod.PIDDIF;
modelStruct.d_modelPar_PIDDIF_dX=0.0;
modelStruct.modelPar_given_PIDDIF=mod.given("PIDDIF");
modelStruct.modelPar_PISREC=mod.PISREC;
modelStruct.d_modelPar_PISREC_dX=0.0;
modelStruct.modelPar_given_PISREC=mod.given("PISREC");
modelStruct.modelPar_PIDREC=mod.PIDREC;
modelStruct.d_modelPar_PIDREC_dX=0.0;
modelStruct.modelPar_given_PIDREC=mod.given("PIDREC");
modelStruct.modelPar_PISTUN=mod.PISTUN;
modelStruct.d_modelPar_PISTUN_dX=0.0;
modelStruct.modelPar_given_PISTUN=mod.given("PISTUN");
modelStruct.modelPar_PIDTUN=mod.PIDTUN;
modelStruct.d_modelPar_PIDTUN_dX=0.0;
modelStruct.modelPar_given_PIDTUN=mod.given("PIDTUN");
modelStruct.modelPar_PVREC0=mod.PVREC0;
modelStruct.d_modelPar_PVREC0_dX=0.0;
modelStruct.modelPar_given_PVREC0=mod.given("PVREC0");
modelStruct.modelPar_PVREC0D=mod.PVREC0D;
modelStruct.d_modelPar_PVREC0D_dX=0.0;
modelStruct.modelPar_given_PVREC0D=mod.given("PVREC0D");
modelStruct.modelPar_PVTUN0=mod.PVTUN0;
modelStruct.d_modelPar_PVTUN0_dX=0.0;
modelStruct.modelPar_given_PVTUN0=mod.given("PVTUN0");
modelStruct.modelPar_PVTUN0D=mod.PVTUN0D;
modelStruct.d_modelPar_PVTUN0D_dX=0.0;
modelStruct.modelPar_given_PVTUN0D=mod.given("PVTUN0D");
modelStruct.modelPar_PNBJT=mod.PNBJT;
modelStruct.d_modelPar_PNBJT_dX=0.0;
modelStruct.modelPar_given_PNBJT=mod.given("PNBJT");
modelStruct.modelPar_PLBJT0=mod.PLBJT0;
modelStruct.d_modelPar_PLBJT0_dX=0.0;
modelStruct.modelPar_given_PLBJT0=mod.given("PLBJT0");
modelStruct.modelPar_PVABJT=mod.PVABJT;
modelStruct.d_modelPar_PVABJT_dX=0.0;
modelStruct.modelPar_given_PVABJT=mod.given("PVABJT");
modelStruct.modelPar_PAELY=mod.PAELY;
modelStruct.d_modelPar_PAELY_dX=0.0;
modelStruct.modelPar_given_PAELY=mod.given("PAELY");
modelStruct.modelPar_PAHLI=mod.PAHLI;
modelStruct.d_modelPar_PAHLI_dX=0.0;
modelStruct.modelPar_given_PAHLI=mod.given("PAHLI");
modelStruct.modelPar_PAHLID=mod.PAHLID;
modelStruct.d_modelPar_PAHLID_dX=0.0;
modelStruct.modelPar_given_PAHLID=mod.given("PAHLID");
modelStruct.modelPar_PVSDFB=mod.PVSDFB;
modelStruct.d_modelPar_PVSDFB_dX=0.0;
modelStruct.modelPar_given_PVSDFB=mod.given("PVSDFB");
modelStruct.modelPar_PVSDTH=mod.PVSDTH;
modelStruct.d_modelPar_PVSDTH_dX=0.0;
modelStruct.modelPar_given_PVSDTH=mod.given("PVSDTH");
modelStruct.modelPar_PDELVT=mod.PDELVT;
modelStruct.d_modelPar_PDELVT_dX=0.0;
modelStruct.modelPar_given_PDELVT=mod.given("PDELVT");
modelStruct.modelPar_PACDE=mod.PACDE;
modelStruct.d_modelPar_PACDE_dX=0.0;
modelStruct.modelPar_given_PACDE=mod.given("PACDE");
modelStruct.modelPar_PMOIN=mod.PMOIN;
modelStruct.d_modelPar_PMOIN_dX=0.0;
modelStruct.modelPar_given_PMOIN=mod.given("PMOIN");
modelStruct.modelPar_PNOFF=mod.PNOFF;
modelStruct.d_modelPar_PNOFF_dX=0.0;
modelStruct.modelPar_given_PNOFF=mod.given("PNOFF");
modelStruct.modelPar_PNOFF2=mod.PNOFF2;
modelStruct.d_modelPar_PNOFF2_dX=0.0;
modelStruct.modelPar_given_PNOFF2=mod.given("PNOFF2");
modelStruct.modelPar_PXRCRG1=mod.PXRCRG1;
modelStruct.d_modelPar_PXRCRG1_dX=0.0;
modelStruct.modelPar_given_PXRCRG1=mod.given("PXRCRG1");
modelStruct.modelPar_PXRCRG2=mod.PXRCRG2;
modelStruct.d_modelPar_PXRCRG2_dX=0.0;
modelStruct.modelPar_given_PXRCRG2=mod.given("PXRCRG2");
modelStruct.modelPar_PVBSA=mod.PVBSA;
modelStruct.d_modelPar_PVBSA_dX=0.0;
modelStruct.modelPar_given_PVBSA=mod.given("PVBSA");
modelStruct.modelPar_PVSCE=mod.PVSCE;
modelStruct.d_modelPar_PVSCE_dX=0.0;
modelStruct.modelPar_given_PVSCE=mod.given("PVSCE");
modelStruct.modelPar_PCDSBS=mod.PCDSBS;
modelStruct.d_modelPar_PCDSBS_dX=0.0;
modelStruct.modelPar_given_PCDSBS=mod.given("PCDSBS");
modelStruct.modelPar_PNOFFFD=mod.PNOFFFD;
modelStruct.d_modelPar_PNOFFFD_dX=0.0;
modelStruct.modelPar_given_PNOFFFD=mod.given("PNOFFFD");
modelStruct.modelPar_PVOFFFD=mod.PVOFFFD;
modelStruct.d_modelPar_PVOFFFD_dX=0.0;
modelStruct.modelPar_given_PVOFFFD=mod.given("PVOFFFD");
modelStruct.modelPar_PK1B=mod.PK1B;
modelStruct.d_modelPar_PK1B_dX=0.0;
modelStruct.modelPar_given_PK1B=mod.given("PK1B");
modelStruct.modelPar_PK2B=mod.PK2B;
modelStruct.d_modelPar_PK2B_dX=0.0;
modelStruct.modelPar_given_PK2B=mod.given("PK2B");
modelStruct.modelPar_PDK2B=mod.PDK2B;
modelStruct.d_modelPar_PDK2B_dX=0.0;
modelStruct.modelPar_given_PDK2B=mod.given("PDK2B");
modelStruct.modelPar_PDVBD0=mod.PDVBD0;
modelStruct.d_modelPar_PDVBD0_dX=0.0;
modelStruct.modelPar_given_PDVBD0=mod.given("PDVBD0");
modelStruct.modelPar_PDVBD1=mod.PDVBD1;
modelStruct.d_modelPar_PDVBD1_dX=0.0;
modelStruct.modelPar_given_PDVBD1=mod.given("PDVBD1");
modelStruct.modelPar_PMOINFD=mod.PMOINFD;
modelStruct.d_modelPar_PMOINFD_dX=0.0;
modelStruct.modelPar_given_PMOINFD=mod.given("PMOINFD");
modelStruct.modelPar_PVBS0PD=mod.PVBS0PD;
modelStruct.d_modelPar_PVBS0PD_dX=0.0;
modelStruct.modelPar_given_PVBS0PD=mod.given("PVBS0PD");
modelStruct.modelPar_PVBS0FD=mod.PVBS0FD;
modelStruct.d_modelPar_PVBS0FD_dX=0.0;
modelStruct.modelPar_given_PVBS0FD=mod.given("PVBS0FD");
modelStruct.modelPar_NLX=mod.NLX;
modelStruct.d_modelPar_NLX_dX=0.0;
modelStruct.modelPar_given_NLX=mod.given("NLX");
modelStruct.modelPar_LNLX=mod.LNLX;
modelStruct.d_modelPar_LNLX_dX=0.0;
modelStruct.modelPar_given_LNLX=mod.given("LNLX");
modelStruct.modelPar_WNLX=mod.WNLX;
modelStruct.d_modelPar_WNLX_dX=0.0;
modelStruct.modelPar_given_WNLX=mod.given("WNLX");
modelStruct.modelPar_PNLX=mod.PNLX;
modelStruct.d_modelPar_PNLX_dX=0.0;
modelStruct.modelPar_given_PNLX=mod.given("PNLX");
modelStruct.modelPar_NGIDL=mod.NGIDL;
modelStruct.d_modelPar_NGIDL_dX=0.0;
modelStruct.modelPar_given_NGIDL=mod.given("NGIDL");
modelStruct.modelPar_LNGIDL=mod.LNGIDL;
modelStruct.d_modelPar_LNGIDL_dX=0.0;
modelStruct.modelPar_given_LNGIDL=mod.given("LNGIDL");
modelStruct.modelPar_WNGIDL=mod.WNGIDL;
modelStruct.d_modelPar_WNGIDL_dX=0.0;
modelStruct.modelPar_given_WNGIDL=mod.given("WNGIDL");
modelStruct.modelPar_PNGIDL=mod.PNGIDL;
modelStruct.d_modelPar_PNGIDL_dX=0.0;
modelStruct.modelPar_given_PNGIDL=mod.given("PNGIDL");
modelStruct.modelPar_LPE0=mod.LPE0;
modelStruct.d_modelPar_LPE0_dX=0.0;
modelStruct.modelPar_given_LPE0=mod.given("LPE0");
modelStruct.modelPar_EGIDL=mod.EGIDL;
modelStruct.d_modelPar_EGIDL_dX=0.0;
modelStruct.modelPar_given_EGIDL=mod.given("EGIDL");
modelStruct.modelPar_EGISL=mod.EGISL;
modelStruct.d_modelPar_EGISL_dX=0.0;
modelStruct.modelPar_given_EGISL=mod.given("EGISL");
modelStruct.modelPar_LLPE0=mod.LLPE0;
modelStruct.d_modelPar_LLPE0_dX=0.0;
modelStruct.modelPar_given_LLPE0=mod.given("LLPE0");
modelStruct.modelPar_LEGIDL=mod.LEGIDL;
modelStruct.d_modelPar_LEGIDL_dX=0.0;
modelStruct.modelPar_given_LEGIDL=mod.given("LEGIDL");
modelStruct.modelPar_LEGISL=mod.LEGISL;
modelStruct.d_modelPar_LEGISL_dX=0.0;
modelStruct.modelPar_given_LEGISL=mod.given("LEGISL");
modelStruct.modelPar_WLPE0=mod.WLPE0;
modelStruct.d_modelPar_WLPE0_dX=0.0;
modelStruct.modelPar_given_WLPE0=mod.given("WLPE0");
modelStruct.modelPar_WEGIDL=mod.WEGIDL;
modelStruct.d_modelPar_WEGIDL_dX=0.0;
modelStruct.modelPar_given_WEGIDL=mod.given("WEGIDL");
modelStruct.modelPar_WEGISL=mod.WEGISL;
modelStruct.d_modelPar_WEGISL_dX=0.0;
modelStruct.modelPar_given_WEGISL=mod.given("WEGISL");
modelStruct.modelPar_PLPE0=mod.PLPE0;
modelStruct.d_modelPar_PLPE0_dX=0.0;
modelStruct.modelPar_given_PLPE0=mod.given("PLPE0");
modelStruct.modelPar_PEGIDL=mod.PEGIDL;
modelStruct.d_modelPar_PEGIDL_dX=0.0;
modelStruct.modelPar_given_PEGIDL=mod.given("PEGIDL");
modelStruct.modelPar_PEGISL=mod.PEGISL;
modelStruct.d_modelPar_PEGISL_dX=0.0;
modelStruct.modelPar_given_PEGISL=mod.given("PEGISL");
modelStruct.modelPar_EGGBCP2=mod.EGGBCP2;
modelStruct.d_modelPar_EGGBCP2_dX=0.0;
modelStruct.modelPar_given_EGGBCP2=mod.given("EGGBCP2");
modelStruct.modelPar_EGGDEP=mod.EGGDEP;
modelStruct.d_modelPar_EGGDEP_dX=0.0;
modelStruct.modelPar_given_EGGDEP=mod.given("EGGDEP");
modelStruct.modelPar_AGB1=mod.AGB1;
modelStruct.d_modelPar_AGB1_dX=0.0;
modelStruct.modelPar_given_AGB1=mod.given("AGB1");
modelStruct.modelPar_BGB1=mod.BGB1;
modelStruct.d_modelPar_BGB1_dX=0.0;
modelStruct.modelPar_given_BGB1=mod.given("BGB1");
modelStruct.modelPar_AGB2=mod.AGB2;
modelStruct.d_modelPar_AGB2_dX=0.0;
modelStruct.modelPar_given_AGB2=mod.given("AGB2");
modelStruct.modelPar_BGB2=mod.BGB2;
modelStruct.d_modelPar_BGB2_dX=0.0;
modelStruct.modelPar_given_BGB2=mod.given("BGB2");
modelStruct.modelPar_AGBC2N=mod.AGBC2N;
modelStruct.d_modelPar_AGBC2N_dX=0.0;
modelStruct.modelPar_given_AGBC2N=mod.given("AGBC2N");
modelStruct.modelPar_AGBC2P=mod.AGBC2P;
modelStruct.d_modelPar_AGBC2P_dX=0.0;
modelStruct.modelPar_given_AGBC2P=mod.given("AGBC2P");
modelStruct.modelPar_BGBC2N=mod.BGBC2N;
modelStruct.d_modelPar_BGBC2N_dX=0.0;
modelStruct.modelPar_given_BGBC2N=mod.given("BGBC2N");
modelStruct.modelPar_BGBC2P=mod.BGBC2P;
modelStruct.d_modelPar_BGBC2P_dX=0.0;
modelStruct.modelPar_given_BGBC2P=mod.given("BGBC2P");
modelStruct.modelPar_VTM00=mod.VTM00;
modelStruct.d_modelPar_VTM00_dX=0.0;
modelStruct.modelPar_given_VTM00=mod.given("VTM00");


// hidden reals


// non-reals (including hiddens)
modelStruct.modelPar_NF=mod.NF;
modelStruct.modelPar_given_NF=mod.given("NF");
modelStruct.modelPar_OFF=mod.OFF;
modelStruct.modelPar_given_OFF=mod.given("OFF");
modelStruct.modelPar_BJTOFF=mod.BJTOFF;
modelStruct.modelPar_given_BJTOFF=mod.given("BJTOFF");
modelStruct.modelPar_SOIMOD=mod.SOIMOD;
modelStruct.modelPar_given_SOIMOD=mod.given("SOIMOD");
modelStruct.modelPar_SHMOD=mod.SHMOD;
modelStruct.modelPar_given_SHMOD=mod.given("SHMOD");
modelStruct.modelPar_TYPE=mod.TYPE;
modelStruct.modelPar_given_TYPE=mod.given("TYPE");
modelStruct.modelPar_RGATEMOD=mod.RGATEMOD;
modelStruct.modelPar_given_RGATEMOD=mod.given("RGATEMOD");
modelStruct.modelPar_RBODYMOD=mod.RBODYMOD;
modelStruct.modelPar_given_RBODYMOD=mod.given("RBODYMOD");
modelStruct.modelPar_MTRLMOD=mod.MTRLMOD;
modelStruct.modelPar_given_MTRLMOD=mod.given("MTRLMOD");
modelStruct.modelPar_VGSTCVMOD=mod.VGSTCVMOD;
modelStruct.modelPar_given_VGSTCVMOD=mod.given("VGSTCVMOD");
modelStruct.modelPar_GIDLMOD=mod.GIDLMOD;
modelStruct.modelPar_given_GIDLMOD=mod.given("GIDLMOD");
modelStruct.modelPar_IIIMOD=mod.IIIMOD;
modelStruct.modelPar_given_IIIMOD=mod.given("IIIMOD");
modelStruct.modelPar_CAPMOD=mod.CAPMOD;
modelStruct.modelPar_given_CAPMOD=mod.given("CAPMOD");
modelStruct.modelPar_MOBMOD=mod.MOBMOD;
modelStruct.modelPar_given_MOBMOD=mod.given("MOBMOD");
modelStruct.modelPar_PARAMCHK=mod.PARAMCHK;
modelStruct.modelPar_given_PARAMCHK=mod.given("PARAMCHK");
modelStruct.modelPar_NODECHK=mod.NODECHK;
modelStruct.modelPar_given_NODECHK=mod.given("NODECHK");
modelStruct.modelPar_BINUNIT=mod.BINUNIT;
modelStruct.modelPar_given_BINUNIT=mod.given("BINUNIT");
modelStruct.modelPar_FNOIMOD=mod.FNOIMOD;
modelStruct.modelPar_given_FNOIMOD=mod.given("FNOIMOD");
modelStruct.modelPar_TNOIMOD=mod.TNOIMOD;
modelStruct.modelPar_given_TNOIMOD=mod.given("TNOIMOD");
modelStruct.modelPar_IGBMOD=mod.IGBMOD;
modelStruct.modelPar_given_IGBMOD=mod.given("IGBMOD");
modelStruct.modelPar_IGMOD=mod.IGMOD;
modelStruct.modelPar_given_IGMOD=mod.given("IGMOD");
modelStruct.modelPar_IGCMOD=mod.IGCMOD;
modelStruct.modelPar_given_IGCMOD=mod.given("IGCMOD");
modelStruct.modelPar_RDSMOD=mod.RDSMOD;
modelStruct.modelPar_given_RDSMOD=mod.given("RDSMOD");
modelStruct.modelPar_FDMOD=mod.FDMOD;
modelStruct.modelPar_given_FDMOD=mod.given("FDMOD");



// Copy out all the instance parameters (doubles) into local struct
// Keep a map so we can set the right one to the independent variable
// We do this solely to avoid a big ugly "if/else" block just to find the
// one parameter we're doing sensitivities on.
unordered_map <std::string,double*,HashNoCase,EqualNoCase> inParamMap;
instanceSensStruct instanceStruct;
// reals
instanceStruct.instancePar_DTEMP=in.DTEMP;
instanceStruct.d_instancePar_DTEMP_dX=0.0;
instanceStruct.instancePar_given_DTEMP=in.given("DTEMP");
inParamMap["DTEMP"] = &(instanceStruct.d_instancePar_DTEMP_dX);
instanceStruct.instancePar_L=in.L;
instanceStruct.d_instancePar_L_dX=0.0;
instanceStruct.instancePar_given_L=in.given("L");
inParamMap["L"] = &(instanceStruct.d_instancePar_L_dX);
instanceStruct.instancePar_W=in.W;
instanceStruct.d_instancePar_W_dX=0.0;
instanceStruct.instancePar_given_W=in.given("W");
inParamMap["W"] = &(instanceStruct.d_instancePar_W_dX);
instanceStruct.instancePar_SA=in.SA;
instanceStruct.d_instancePar_SA_dX=0.0;
instanceStruct.instancePar_given_SA=in.given("SA");
inParamMap["SA"] = &(instanceStruct.d_instancePar_SA_dX);
instanceStruct.instancePar_SB=in.SB;
instanceStruct.d_instancePar_SB_dX=0.0;
instanceStruct.instancePar_given_SB=in.given("SB");
inParamMap["SB"] = &(instanceStruct.d_instancePar_SB_dX);
instanceStruct.instancePar_SD=in.SD;
instanceStruct.d_instancePar_SD_dX=0.0;
instanceStruct.instancePar_given_SD=in.given("SD");
inParamMap["SD"] = &(instanceStruct.d_instancePar_SD_dX);
instanceStruct.instancePar_AD=in.AD;
instanceStruct.d_instancePar_AD_dX=0.0;
instanceStruct.instancePar_given_AD=in.given("AD");
inParamMap["AD"] = &(instanceStruct.d_instancePar_AD_dX);
instanceStruct.instancePar_AS=in.AS;
instanceStruct.d_instancePar_AS_dX=0.0;
instanceStruct.instancePar_given_AS=in.given("AS");
inParamMap["AS"] = &(instanceStruct.d_instancePar_AS_dX);
instanceStruct.instancePar_PD=in.PD;
instanceStruct.d_instancePar_PD_dX=0.0;
instanceStruct.instancePar_given_PD=in.given("PD");
inParamMap["PD"] = &(instanceStruct.d_instancePar_PD_dX);
instanceStruct.instancePar_PS=in.PS;
instanceStruct.d_instancePar_PS_dX=0.0;
instanceStruct.instancePar_given_PS=in.given("PS");
inParamMap["PS"] = &(instanceStruct.d_instancePar_PS_dX);
instanceStruct.instancePar_NRD=in.NRD;
instanceStruct.d_instancePar_NRD_dX=0.0;
instanceStruct.instancePar_given_NRD=in.given("NRD");
inParamMap["NRD"] = &(instanceStruct.d_instancePar_NRD_dX);
instanceStruct.instancePar_NRS=in.NRS;
instanceStruct.d_instancePar_NRS_dX=0.0;
instanceStruct.instancePar_given_NRS=in.given("NRS");
inParamMap["NRS"] = &(instanceStruct.d_instancePar_NRS_dX);
instanceStruct.instancePar_RTH0=in.RTH0;
instanceStruct.d_instancePar_RTH0_dX=0.0;
instanceStruct.instancePar_given_RTH0=in.given("RTH0");
inParamMap["RTH0"] = &(instanceStruct.d_instancePar_RTH0_dX);
instanceStruct.instancePar_CTH0=in.CTH0;
instanceStruct.d_instancePar_CTH0_dX=0.0;
instanceStruct.instancePar_given_CTH0=in.given("CTH0");
inParamMap["CTH0"] = &(instanceStruct.d_instancePar_CTH0_dX);
instanceStruct.instancePar_NRB=in.NRB;
instanceStruct.d_instancePar_NRB_dX=0.0;
instanceStruct.instancePar_given_NRB=in.given("NRB");
inParamMap["NRB"] = &(instanceStruct.d_instancePar_NRB_dX);
instanceStruct.instancePar_FRBODY=in.FRBODY;
instanceStruct.d_instancePar_FRBODY_dX=0.0;
instanceStruct.instancePar_given_FRBODY=in.given("FRBODY");
inParamMap["FRBODY"] = &(instanceStruct.d_instancePar_FRBODY_dX);
instanceStruct.instancePar_RBDB=in.RBDB;
instanceStruct.d_instancePar_RBDB_dX=0.0;
instanceStruct.instancePar_given_RBDB=in.given("RBDB");
inParamMap["RBDB"] = &(instanceStruct.d_instancePar_RBDB_dX);
instanceStruct.instancePar_RBSB=in.RBSB;
instanceStruct.d_instancePar_RBSB_dX=0.0;
instanceStruct.instancePar_given_RBSB=in.given("RBSB");
inParamMap["RBSB"] = &(instanceStruct.d_instancePar_RBSB_dX);
instanceStruct.instancePar_DELVTO=in.DELVTO;
instanceStruct.d_instancePar_DELVTO_dX=0.0;
instanceStruct.instancePar_given_DELVTO=in.given("DELVTO");
inParamMap["DELVTO"] = &(instanceStruct.d_instancePar_DELVTO_dX);
instanceStruct.instancePar_NBC=in.NBC;
instanceStruct.d_instancePar_NBC_dX=0.0;
instanceStruct.instancePar_given_NBC=in.given("NBC");
inParamMap["NBC"] = &(instanceStruct.d_instancePar_NBC_dX);
instanceStruct.instancePar_NSEG=in.NSEG;
instanceStruct.d_instancePar_NSEG_dX=0.0;
instanceStruct.instancePar_given_NSEG=in.given("NSEG");
inParamMap["NSEG"] = &(instanceStruct.d_instancePar_NSEG_dX);
instanceStruct.instancePar_PDBCP=in.PDBCP;
instanceStruct.d_instancePar_PDBCP_dX=0.0;
instanceStruct.instancePar_given_PDBCP=in.given("PDBCP");
inParamMap["PDBCP"] = &(instanceStruct.d_instancePar_PDBCP_dX);
instanceStruct.instancePar_PSBCP=in.PSBCP;
instanceStruct.d_instancePar_PSBCP_dX=0.0;
instanceStruct.instancePar_given_PSBCP=in.given("PSBCP");
inParamMap["PSBCP"] = &(instanceStruct.d_instancePar_PSBCP_dX);
instanceStruct.instancePar_AGBCP=in.AGBCP;
instanceStruct.d_instancePar_AGBCP_dX=0.0;
instanceStruct.instancePar_given_AGBCP=in.given("AGBCP");
inParamMap["AGBCP"] = &(instanceStruct.d_instancePar_AGBCP_dX);
instanceStruct.instancePar_AGBCP2=in.AGBCP2;
instanceStruct.d_instancePar_AGBCP2_dX=0.0;
instanceStruct.instancePar_given_AGBCP2=in.given("AGBCP2");
inParamMap["AGBCP2"] = &(instanceStruct.d_instancePar_AGBCP2_dX);
instanceStruct.instancePar_AGBCPD=in.AGBCPD;
instanceStruct.d_instancePar_AGBCPD_dX=0.0;
instanceStruct.instancePar_given_AGBCPD=in.given("AGBCPD");
inParamMap["AGBCPD"] = &(instanceStruct.d_instancePar_AGBCPD_dX);
instanceStruct.instancePar_AEBCP=in.AEBCP;
instanceStruct.d_instancePar_AEBCP_dX=0.0;
instanceStruct.instancePar_given_AEBCP=in.given("AEBCP");
inParamMap["AEBCP"] = &(instanceStruct.d_instancePar_AEBCP_dX);
instanceStruct.instancePar_m=in.m;
instanceStruct.d_instancePar_m_dX=0.0;
instanceStruct.instancePar_given_m=in.given("m");
inParamMap["m"] = &(instanceStruct.d_instancePar_m_dX);


// Copy all the real hidden instance params into local struct



// Copy all the non-real instance params into vars of their appropriate type:
instanceStruct.instancePar_NF=in.NF;
instanceStruct.instancePar_given_NF=in.given("NF");
instanceStruct.instancePar_OFF=in.OFF;
instanceStruct.instancePar_given_OFF=in.given("OFF");
instanceStruct.instancePar_BJTOFF=in.BJTOFF;
instanceStruct.instancePar_given_BJTOFF=in.given("BJTOFF");
instanceStruct.instancePar_DEBUG=in.DEBUG;
instanceStruct.instancePar_given_DEBUG=in.given("DEBUG");
instanceStruct.instancePar_SOIMOD=in.SOIMOD;
instanceStruct.instancePar_given_SOIMOD=in.given("SOIMOD");
instanceStruct.instancePar_TNODEOUT=in.TNODEOUT;
instanceStruct.instancePar_given_TNODEOUT=in.given("TNODEOUT");
instanceStruct.instancePar_SHMOD=in.SHMOD;
instanceStruct.instancePar_given_SHMOD=in.given("SHMOD");


// Set the one parameter whose name was passed in to be the independent
// variable for differentiation purposes.  Since we stored variable pointers, this
// makes sure that that ONE variable gets set right.
// FIXME: make this check the name first, otherwise segfault on invalid name!
*(inParamMap[name])=1.0;

//make local copies of all instance vars
//reals
instanceStruct.instanceVar_B4SOInf=in.B4SOInf;
instanceStruct.d_instanceVar_B4SOInf_dX=0.0;
instanceStruct.instanceVar_B4SOIsa=in.B4SOIsa;
instanceStruct.d_instanceVar_B4SOIsa_dX=0.0;
instanceStruct.instanceVar_B4SOIsb=in.B4SOIsb;
instanceStruct.d_instanceVar_B4SOIsb_dX=0.0;
instanceStruct.instanceVar_B4SOIsd=in.B4SOIsd;
instanceStruct.d_instanceVar_B4SOIsd_dX=0.0;
instanceStruct.instanceVar_B4SOIrth0=in.B4SOIrth0;
instanceStruct.d_instanceVar_B4SOIrth0_dX=0.0;
instanceStruct.instanceVar_B4SOIcth0=in.B4SOIcth0;
instanceStruct.d_instanceVar_B4SOIcth0_dX=0.0;
instanceStruct.instanceVar_B4SOInbc=in.B4SOInbc;
instanceStruct.d_instanceVar_B4SOInbc_dX=0.0;
instanceStruct.instanceVar_B4SOInseg=in.B4SOInseg;
instanceStruct.d_instanceVar_B4SOInseg_dX=0.0;
instanceStruct.instanceVar_B4SOIagbcp=in.B4SOIagbcp;
instanceStruct.d_instanceVar_B4SOIagbcp_dX=0.0;
instanceStruct.instanceVar_B4SOIagbcp2=in.B4SOIagbcp2;
instanceStruct.d_instanceVar_B4SOIagbcp2_dX=0.0;
instanceStruct.instanceVar_B4SOIagbcpd=in.B4SOIagbcpd;
instanceStruct.d_instanceVar_B4SOIagbcpd_dX=0.0;
instanceStruct.instanceVar_B4SOIaebcp=in.B4SOIaebcp;
instanceStruct.d_instanceVar_B4SOIaebcp_dX=0.0;
instanceStruct.instanceVar_B4SOIeot=in.B4SOIeot;
instanceStruct.d_instanceVar_B4SOIeot_dX=0.0;
instanceStruct.instanceVar_B4SOIepsrsub=in.B4SOIepsrsub;
instanceStruct.d_instanceVar_B4SOIepsrsub_dX=0.0;
instanceStruct.instanceVar_B4SOIni0sub=in.B4SOIni0sub;
instanceStruct.d_instanceVar_B4SOIni0sub_dX=0.0;
instanceStruct.instanceVar_B4SOIbg0sub=in.B4SOIbg0sub;
instanceStruct.d_instanceVar_B4SOIbg0sub_dX=0.0;
instanceStruct.instanceVar_B4SOItbgasub=in.B4SOItbgasub;
instanceStruct.d_instanceVar_B4SOItbgasub_dX=0.0;
instanceStruct.instanceVar_B4SOItbgbsub=in.B4SOItbgbsub;
instanceStruct.d_instanceVar_B4SOItbgbsub_dX=0.0;
instanceStruct.instanceVar_B4SOIphig=in.B4SOIphig;
instanceStruct.d_instanceVar_B4SOIphig_dX=0.0;
instanceStruct.instanceVar_B4SOIeasub=in.B4SOIeasub;
instanceStruct.d_instanceVar_B4SOIeasub_dX=0.0;
instanceStruct.instanceVar_B4SOIleffeot=in.B4SOIleffeot;
instanceStruct.d_instanceVar_B4SOIleffeot_dX=0.0;
instanceStruct.instanceVar_B4SOIweffeot=in.B4SOIweffeot;
instanceStruct.d_instanceVar_B4SOIweffeot_dX=0.0;
instanceStruct.instanceVar_B4SOIados=in.B4SOIados;
instanceStruct.d_instanceVar_B4SOIados_dX=0.0;
instanceStruct.instanceVar_B4SOIbdos=in.B4SOIbdos;
instanceStruct.d_instanceVar_B4SOIbdos_dX=0.0;
instanceStruct.instanceVar_B4SOIepsrgate=in.B4SOIepsrgate;
instanceStruct.d_instanceVar_B4SOIepsrgate_dX=0.0;
instanceStruct.instanceVar_B4SOItox=in.B4SOItox;
instanceStruct.d_instanceVar_B4SOItox_dX=0.0;
instanceStruct.instanceVar_B4SOItoxp=in.B4SOItoxp;
instanceStruct.d_instanceVar_B4SOItoxp_dX=0.0;
instanceStruct.instanceVar_B4SOItoxm=in.B4SOItoxm;
instanceStruct.d_instanceVar_B4SOItoxm_dX=0.0;
instanceStruct.instanceVar_B4SOIdtoxcv=in.B4SOIdtoxcv;
instanceStruct.d_instanceVar_B4SOIdtoxcv_dX=0.0;
instanceStruct.instanceVar_B4SOIngate=in.B4SOIngate;
instanceStruct.d_instanceVar_B4SOIngate_dX=0.0;
instanceStruct.instanceVar_B4SOInsd=in.B4SOInsd;
instanceStruct.d_instanceVar_B4SOInsd_dX=0.0;
instanceStruct.instanceVar_B4SOItnom=in.B4SOItnom;
instanceStruct.d_instanceVar_B4SOItnom_dX=0.0;
instanceStruct.instanceVar_B4SOIxpart=in.B4SOIxpart;
instanceStruct.d_instanceVar_B4SOIxpart_dX=0.0;
instanceStruct.instanceVar_B4SOIrswmin=in.B4SOIrswmin;
instanceStruct.d_instanceVar_B4SOIrswmin_dX=0.0;
instanceStruct.instanceVar_B4SOIrdwmin=in.B4SOIrdwmin;
instanceStruct.d_instanceVar_B4SOIrdwmin_dX=0.0;
instanceStruct.instanceVar_B4SOItbox=in.B4SOItbox;
instanceStruct.d_instanceVar_B4SOItbox_dX=0.0;
instanceStruct.instanceVar_B4SOItsi=in.B4SOItsi;
instanceStruct.d_instanceVar_B4SOItsi_dX=0.0;
instanceStruct.instanceVar_B4SOIegidl=in.B4SOIegidl;
instanceStruct.d_instanceVar_B4SOIegidl_dX=0.0;
instanceStruct.instanceVar_B4SOIagidl=in.B4SOIagidl;
instanceStruct.d_instanceVar_B4SOIagidl_dX=0.0;
instanceStruct.instanceVar_B4SOIbgidl=in.B4SOIbgidl;
instanceStruct.d_instanceVar_B4SOIbgidl_dX=0.0;
instanceStruct.instanceVar_B4SOIcgidl=in.B4SOIcgidl;
instanceStruct.d_instanceVar_B4SOIcgidl_dX=0.0;
instanceStruct.instanceVar_B4SOIegisl=in.B4SOIegisl;
instanceStruct.d_instanceVar_B4SOIegisl_dX=0.0;
instanceStruct.instanceVar_B4SOIagisl=in.B4SOIagisl;
instanceStruct.d_instanceVar_B4SOIagisl_dX=0.0;
instanceStruct.instanceVar_B4SOIbgisl=in.B4SOIbgisl;
instanceStruct.d_instanceVar_B4SOIbgisl_dX=0.0;
instanceStruct.instanceVar_B4SOIcgisl=in.B4SOIcgisl;
instanceStruct.d_instanceVar_B4SOIcgisl_dX=0.0;
instanceStruct.instanceVar_B4SOIndiode=in.B4SOIndiode;
instanceStruct.d_instanceVar_B4SOIndiode_dX=0.0;
instanceStruct.instanceVar_B4SOIndioded=in.B4SOIndioded;
instanceStruct.d_instanceVar_B4SOIndioded_dX=0.0;
instanceStruct.instanceVar_B4SOIGatesidewallJctSPotential=in.B4SOIGatesidewallJctSPotential;
instanceStruct.d_instanceVar_B4SOIGatesidewallJctSPotential_dX=0.0;
instanceStruct.instanceVar_B4SOIGatesidewallJctDPotential=in.B4SOIGatesidewallJctDPotential;
instanceStruct.d_instanceVar_B4SOIGatesidewallJctDPotential_dX=0.0;
instanceStruct.instanceVar_B4SOIbodyJctGateSideSGradingCoeff=in.B4SOIbodyJctGateSideSGradingCoeff;
instanceStruct.d_instanceVar_B4SOIbodyJctGateSideSGradingCoeff_dX=0.0;
instanceStruct.instanceVar_B4SOIbodyJctGateSideDGradingCoeff=in.B4SOIbodyJctGateSideDGradingCoeff;
instanceStruct.d_instanceVar_B4SOIbodyJctGateSideDGradingCoeff_dX=0.0;
instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapS=in.B4SOIunitLengthGateSidewallJctCapS;
instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapS_dX=0.0;
instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapD=in.B4SOIunitLengthGateSidewallJctCapD;
instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapD_dX=0.0;
instanceStruct.instanceVar_B4SOIoxideTrapDensityA=in.B4SOIoxideTrapDensityA;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityA_dX=0.0;
instanceStruct.instanceVar_B4SOIoxideTrapDensityB=in.B4SOIoxideTrapDensityB;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityB_dX=0.0;
instanceStruct.instanceVar_B4SOIoxideTrapDensityC=in.B4SOIoxideTrapDensityC;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityC_dX=0.0;
instanceStruct.instanceVar_B4SOItnoia=in.B4SOItnoia;
instanceStruct.d_instanceVar_B4SOItnoia_dX=0.0;
instanceStruct.instanceVar_B4SOItnoib=in.B4SOItnoib;
instanceStruct.d_instanceVar_B4SOItnoib_dX=0.0;
instanceStruct.instanceVar_B4SOIrnoia=in.B4SOIrnoia;
instanceStruct.d_instanceVar_B4SOIrnoia_dX=0.0;
instanceStruct.instanceVar_B4SOIrnoib=in.B4SOIrnoib;
instanceStruct.d_instanceVar_B4SOIrnoib_dX=0.0;
instanceStruct.instanceVar_B4SOIntnoi=in.B4SOIntnoi;
instanceStruct.d_instanceVar_B4SOIntnoi_dX=0.0;
instanceStruct.instanceVar_B4SOIsaref=in.B4SOIsaref;
instanceStruct.d_instanceVar_B4SOIsaref_dX=0.0;
instanceStruct.instanceVar_B4SOIsbref=in.B4SOIsbref;
instanceStruct.d_instanceVar_B4SOIsbref_dX=0.0;
instanceStruct.instanceVar_B4SOIku0=in.B4SOIku0;
instanceStruct.d_instanceVar_B4SOIku0_dX=0.0;
instanceStruct.instanceVar_B4SOIkvsat=in.B4SOIkvsat;
instanceStruct.d_instanceVar_B4SOIkvsat_dX=0.0;
instanceStruct.instanceVar_B4SOItku0=in.B4SOItku0;
instanceStruct.d_instanceVar_B4SOItku0_dX=0.0;
instanceStruct.instanceVar_B4SOIlodk2=in.B4SOIlodk2;
instanceStruct.d_instanceVar_B4SOIlodk2_dX=0.0;
instanceStruct.instanceVar_B4SOIlodeta0=in.B4SOIlodeta0;
instanceStruct.d_instanceVar_B4SOIlodeta0_dX=0.0;
instanceStruct.instanceVar_B4SOIlodeta0cv=in.B4SOIlodeta0cv;
instanceStruct.d_instanceVar_B4SOIlodeta0cv_dX=0.0;
instanceStruct.instanceVar_B4SOIbf=in.B4SOIbf;
instanceStruct.d_instanceVar_B4SOIbf_dX=0.0;
instanceStruct.instanceVar_B4SOIw0flk=in.B4SOIw0flk;
instanceStruct.d_instanceVar_B4SOIw0flk_dX=0.0;
instanceStruct.instanceVar_B4SOIpditsl=in.B4SOIpditsl;
instanceStruct.d_instanceVar_B4SOIpditsl_dX=0.0;
instanceStruct.instanceVar_B4SOIem=in.B4SOIem;
instanceStruct.d_instanceVar_B4SOIem_dX=0.0;
instanceStruct.instanceVar_B4SOIef=in.B4SOIef;
instanceStruct.d_instanceVar_B4SOIef_dX=0.0;
instanceStruct.instanceVar_B4SOIaf=in.B4SOIaf;
instanceStruct.d_instanceVar_B4SOIaf_dX=0.0;
instanceStruct.instanceVar_B4SOIkf=in.B4SOIkf;
instanceStruct.d_instanceVar_B4SOIkf_dX=0.0;
instanceStruct.instanceVar_B4SOInoif=in.B4SOInoif;
instanceStruct.d_instanceVar_B4SOInoif_dX=0.0;
instanceStruct.instanceVar_B4SOIk1w1=in.B4SOIk1w1;
instanceStruct.d_instanceVar_B4SOIk1w1_dX=0.0;
instanceStruct.instanceVar_B4SOIk1w2=in.B4SOIk1w2;
instanceStruct.d_instanceVar_B4SOIk1w2_dX=0.0;
instanceStruct.instanceVar_B4SOIketas=in.B4SOIketas;
instanceStruct.d_instanceVar_B4SOIketas_dX=0.0;
instanceStruct.instanceVar_B4SOIdwbc=in.B4SOIdwbc;
instanceStruct.d_instanceVar_B4SOIdwbc_dX=0.0;
instanceStruct.instanceVar_B4SOIbeta0=in.B4SOIbeta0;
instanceStruct.d_instanceVar_B4SOIbeta0_dX=0.0;
instanceStruct.instanceVar_B4SOIbeta1=in.B4SOIbeta1;
instanceStruct.d_instanceVar_B4SOIbeta1_dX=0.0;
instanceStruct.instanceVar_B4SOIbeta2=in.B4SOIbeta2;
instanceStruct.d_instanceVar_B4SOIbeta2_dX=0.0;
instanceStruct.instanceVar_B4SOItii=in.B4SOItii;
instanceStruct.d_instanceVar_B4SOItii_dX=0.0;
instanceStruct.instanceVar_B4SOIlii=in.B4SOIlii;
instanceStruct.d_instanceVar_B4SOIlii_dX=0.0;
instanceStruct.instanceVar_B4SOIsii1=in.B4SOIsii1;
instanceStruct.d_instanceVar_B4SOIsii1_dX=0.0;
instanceStruct.instanceVar_B4SOIsii2=in.B4SOIsii2;
instanceStruct.d_instanceVar_B4SOIsii2_dX=0.0;
instanceStruct.instanceVar_B4SOIsiid=in.B4SOIsiid;
instanceStruct.d_instanceVar_B4SOIsiid_dX=0.0;
instanceStruct.instanceVar_B4SOIfbjtii=in.B4SOIfbjtii;
instanceStruct.d_instanceVar_B4SOIfbjtii_dX=0.0;
instanceStruct.instanceVar_B4SOItvbci=in.B4SOItvbci;
instanceStruct.d_instanceVar_B4SOItvbci_dX=0.0;
instanceStruct.instanceVar_B4SOIesatii=in.B4SOIesatii;
instanceStruct.d_instanceVar_B4SOIesatii_dX=0.0;
instanceStruct.instanceVar_B4SOIntun=in.B4SOIntun;
instanceStruct.d_instanceVar_B4SOIntun_dX=0.0;
instanceStruct.instanceVar_B4SOIntund=in.B4SOIntund;
instanceStruct.d_instanceVar_B4SOIntund_dX=0.0;
instanceStruct.instanceVar_B4SOIisbjt=in.B4SOIisbjt;
instanceStruct.d_instanceVar_B4SOIisbjt_dX=0.0;
instanceStruct.instanceVar_B4SOIidbjt=in.B4SOIidbjt;
instanceStruct.d_instanceVar_B4SOIidbjt_dX=0.0;
instanceStruct.instanceVar_B4SOIisdif=in.B4SOIisdif;
instanceStruct.d_instanceVar_B4SOIisdif_dX=0.0;
instanceStruct.instanceVar_B4SOIiddif=in.B4SOIiddif;
instanceStruct.d_instanceVar_B4SOIiddif_dX=0.0;
instanceStruct.instanceVar_B4SOIisrec=in.B4SOIisrec;
instanceStruct.d_instanceVar_B4SOIisrec_dX=0.0;
instanceStruct.instanceVar_B4SOIidrec=in.B4SOIidrec;
instanceStruct.d_instanceVar_B4SOIidrec_dX=0.0;
instanceStruct.instanceVar_B4SOIistun=in.B4SOIistun;
instanceStruct.d_instanceVar_B4SOIistun_dX=0.0;
instanceStruct.instanceVar_B4SOIidtun=in.B4SOIidtun;
instanceStruct.d_instanceVar_B4SOIidtun_dX=0.0;
instanceStruct.instanceVar_B4SOIvrec0=in.B4SOIvrec0;
instanceStruct.d_instanceVar_B4SOIvrec0_dX=0.0;
instanceStruct.instanceVar_B4SOIvrec0d=in.B4SOIvrec0d;
instanceStruct.d_instanceVar_B4SOIvrec0d_dX=0.0;
instanceStruct.instanceVar_B4SOIvtun0=in.B4SOIvtun0;
instanceStruct.d_instanceVar_B4SOIvtun0_dX=0.0;
instanceStruct.instanceVar_B4SOIvtun0d=in.B4SOIvtun0d;
instanceStruct.d_instanceVar_B4SOIvtun0d_dX=0.0;
instanceStruct.instanceVar_B4SOInbjt=in.B4SOInbjt;
instanceStruct.d_instanceVar_B4SOInbjt_dX=0.0;
instanceStruct.instanceVar_B4SOIaely=in.B4SOIaely;
instanceStruct.d_instanceVar_B4SOIaely_dX=0.0;
instanceStruct.instanceVar_B4SOIahli=in.B4SOIahli;
instanceStruct.d_instanceVar_B4SOIahli_dX=0.0;
instanceStruct.instanceVar_B4SOIahlid=in.B4SOIahlid;
instanceStruct.d_instanceVar_B4SOIahlid_dX=0.0;
instanceStruct.instanceVar_B4SOIrbody=in.B4SOIrbody;
instanceStruct.d_instanceVar_B4SOIrbody_dX=0.0;
instanceStruct.instanceVar_B4SOIrbsh=in.B4SOIrbsh;
instanceStruct.d_instanceVar_B4SOIrbsh_dX=0.0;
instanceStruct.instanceVar_B4SOItt=in.B4SOItt;
instanceStruct.d_instanceVar_B4SOItt_dX=0.0;
instanceStruct.instanceVar_B4SOIcsdmin=in.B4SOIcsdmin;
instanceStruct.d_instanceVar_B4SOIcsdmin_dX=0.0;
instanceStruct.instanceVar_B4SOIasd=in.B4SOIasd;
instanceStruct.d_instanceVar_B4SOIasd_dX=0.0;
instanceStruct.instanceVar_B4SOIcsdesw=in.B4SOIcsdesw;
instanceStruct.d_instanceVar_B4SOIcsdesw_dX=0.0;
instanceStruct.instanceVar_B4SOIfbody=in.B4SOIfbody;
instanceStruct.d_instanceVar_B4SOIfbody_dX=0.0;
instanceStruct.instanceVar_B4SOItcjswg=in.B4SOItcjswg;
instanceStruct.d_instanceVar_B4SOItcjswg_dX=0.0;
instanceStruct.instanceVar_B4SOItpbswg=in.B4SOItpbswg;
instanceStruct.d_instanceVar_B4SOItpbswg_dX=0.0;
instanceStruct.instanceVar_B4SOItcjswgd=in.B4SOItcjswgd;
instanceStruct.d_instanceVar_B4SOItcjswgd_dX=0.0;
instanceStruct.instanceVar_B4SOItpbswgd=in.B4SOItpbswgd;
instanceStruct.d_instanceVar_B4SOItpbswgd_dX=0.0;
instanceStruct.instanceVar_B4SOIacde=in.B4SOIacde;
instanceStruct.d_instanceVar_B4SOIacde_dX=0.0;
instanceStruct.instanceVar_B4SOImoin=in.B4SOImoin;
instanceStruct.d_instanceVar_B4SOImoin_dX=0.0;
instanceStruct.instanceVar_B4SOIdlbg=in.B4SOIdlbg;
instanceStruct.d_instanceVar_B4SOIdlbg_dX=0.0;
instanceStruct.instanceVar_B4SOItoxqm=in.B4SOItoxqm;
instanceStruct.d_instanceVar_B4SOItoxqm_dX=0.0;
instanceStruct.instanceVar_B4SOIwth0=in.B4SOIwth0;
instanceStruct.d_instanceVar_B4SOIwth0_dX=0.0;
instanceStruct.instanceVar_B4SOIrhalo=in.B4SOIrhalo;
instanceStruct.d_instanceVar_B4SOIrhalo_dX=0.0;
instanceStruct.instanceVar_B4SOIntox=in.B4SOIntox;
instanceStruct.d_instanceVar_B4SOIntox_dX=0.0;
instanceStruct.instanceVar_B4SOItoxref=in.B4SOItoxref;
instanceStruct.d_instanceVar_B4SOItoxref_dX=0.0;
instanceStruct.instanceVar_B4SOIebg=in.B4SOIebg;
instanceStruct.d_instanceVar_B4SOIebg_dX=0.0;
instanceStruct.instanceVar_B4SOIvevb=in.B4SOIvevb;
instanceStruct.d_instanceVar_B4SOIvevb_dX=0.0;
instanceStruct.instanceVar_B4SOIvgb1=in.B4SOIvgb1;
instanceStruct.d_instanceVar_B4SOIvgb1_dX=0.0;
instanceStruct.instanceVar_B4SOIvecb=in.B4SOIvecb;
instanceStruct.d_instanceVar_B4SOIvecb_dX=0.0;
instanceStruct.instanceVar_B4SOIvgb2=in.B4SOIvgb2;
instanceStruct.d_instanceVar_B4SOIvgb2_dX=0.0;
instanceStruct.instanceVar_B4SOIvoxh=in.B4SOIvoxh;
instanceStruct.d_instanceVar_B4SOIvoxh_dX=0.0;
instanceStruct.instanceVar_B4SOIdeltavox=in.B4SOIdeltavox;
instanceStruct.d_instanceVar_B4SOIdeltavox_dX=0.0;
instanceStruct.instanceVar_B4SOIvtm=in.B4SOIvtm;
instanceStruct.d_instanceVar_B4SOIvtm_dX=0.0;
instanceStruct.instanceVar_B4SOIueff=in.B4SOIueff;
instanceStruct.d_instanceVar_B4SOIueff_dX=0.0;
instanceStruct.instanceVar_B4SOIthetavth=in.B4SOIthetavth;
instanceStruct.d_instanceVar_B4SOIthetavth_dX=0.0;
instanceStruct.instanceVar_B4SOIInv_ODeff=in.B4SOIInv_ODeff;
instanceStruct.d_instanceVar_B4SOIInv_ODeff_dX=0.0;
instanceStruct.instanceVar_B4SOIgrbsb=in.B4SOIgrbsb;
instanceStruct.d_instanceVar_B4SOIgrbsb_dX=0.0;
instanceStruct.instanceVar_B4SOIgrbdb=in.B4SOIgrbdb;
instanceStruct.d_instanceVar_B4SOIgrbdb_dX=0.0;
instanceStruct.instanceVar_B4SOITempSH=in.B4SOITempSH;
instanceStruct.d_instanceVar_B4SOITempSH_dX=0.0;
instanceStruct.instanceVar_B4SOIgrgeltd=in.B4SOIgrgeltd;
instanceStruct.d_instanceVar_B4SOIgrgeltd_dX=0.0;
instanceStruct.instanceVar_B4SOIsourceResistance=in.B4SOIsourceResistance;
instanceStruct.d_instanceVar_B4SOIsourceResistance_dX=0.0;
instanceStruct.instanceVar_B4SOIdrainResistance=in.B4SOIdrainResistance;
instanceStruct.d_instanceVar_B4SOIdrainResistance_dX=0.0;
instanceStruct.instanceVar_B4SOIvfbzb=in.B4SOIvfbzb;
instanceStruct.d_instanceVar_B4SOIvfbzb_dX=0.0;
instanceStruct.instanceVar_B4SOIvtfbphi1=in.B4SOIvtfbphi1;
instanceStruct.d_instanceVar_B4SOIvtfbphi1_dX=0.0;
instanceStruct.instanceVar_B4SOIvgsteffvth=in.B4SOIvgsteffvth;
instanceStruct.d_instanceVar_B4SOIvgsteffvth_dX=0.0;
instanceStruct.instanceVar_B4SOIrbodyext=in.B4SOIrbodyext;
instanceStruct.d_instanceVar_B4SOIrbodyext_dX=0.0;
instanceStruct.instanceVar_B4SOIcsesw=in.B4SOIcsesw;
instanceStruct.d_instanceVar_B4SOIcsesw_dX=0.0;
instanceStruct.instanceVar_B4SOIcdesw=in.B4SOIcdesw;
instanceStruct.d_instanceVar_B4SOIcdesw_dX=0.0;
instanceStruct.instanceVar_B4SOIcsbox=in.B4SOIcsbox;
instanceStruct.d_instanceVar_B4SOIcsbox_dX=0.0;
instanceStruct.instanceVar_B4SOIcdbox=in.B4SOIcdbox;
instanceStruct.d_instanceVar_B4SOIcdbox_dX=0.0;
instanceStruct.instanceVar_B4SOIcsmin=in.B4SOIcsmin;
instanceStruct.d_instanceVar_B4SOIcsmin_dX=0.0;
instanceStruct.instanceVar_B4SOIcdmin=in.B4SOIcdmin;
instanceStruct.d_instanceVar_B4SOIcdmin_dX=0.0;
instanceStruct.instanceVar_B4SOIst4=in.B4SOIst4;
instanceStruct.d_instanceVar_B4SOIst4_dX=0.0;
instanceStruct.instanceVar_B4SOIdt4=in.B4SOIdt4;
instanceStruct.d_instanceVar_B4SOIdt4_dX=0.0;
instanceStruct.instanceVar_B4SOInstar=in.B4SOInstar;
instanceStruct.d_instanceVar_B4SOInstar_dX=0.0;
instanceStruct.instanceVar_B4SOIAbulk=in.B4SOIAbulk;
instanceStruct.d_instanceVar_B4SOIAbulk_dX=0.0;
instanceStruct.instanceVar_B4SOIqinv=in.B4SOIqinv;
instanceStruct.d_instanceVar_B4SOIqinv_dX=0.0;
instanceStruct.instanceVar_B4SOIgm=in.B4SOIgm;
instanceStruct.d_instanceVar_B4SOIgm_dX=0.0;
instanceStruct.instanceVar_B4SOIgds=in.B4SOIgds;
instanceStruct.d_instanceVar_B4SOIgds_dX=0.0;
instanceStruct.instanceVar_B4SOIgmbs=in.B4SOIgmbs;
instanceStruct.d_instanceVar_B4SOIgmbs_dX=0.0;
instanceStruct.instanceVar_B4SOIrds=in.B4SOIrds;
instanceStruct.d_instanceVar_B4SOIrds_dX=0.0;
instanceStruct.instanceVar_B4SOIVgsteff=in.B4SOIVgsteff;
instanceStruct.d_instanceVar_B4SOIVgsteff_dX=0.0;
instanceStruct.instanceVar_B4SOIVdseff=in.B4SOIVdseff;
instanceStruct.d_instanceVar_B4SOIVdseff_dX=0.0;
instanceStruct.instanceVar_B4SOIAbovVgst2Vtm=in.B4SOIAbovVgst2Vtm;
instanceStruct.d_instanceVar_B4SOIAbovVgst2Vtm_dX=0.0;
instanceStruct.instanceVar_B4SOIidovVds=in.B4SOIidovVds;
instanceStruct.d_instanceVar_B4SOIidovVds_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdsc=in.pParam_B4SOIcdsc;
instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdscb=in.pParam_B4SOIcdscb;
instanceStruct.d_instanceVar_pParam_B4SOIcdscb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdscd=in.pParam_B4SOIcdscd;
instanceStruct.d_instanceVar_pParam_B4SOIcdscd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcit=in.pParam_B4SOIcit;
instanceStruct.d_instanceVar_pParam_B4SOIcit_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInfactor=in.pParam_B4SOInfactor;
instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsat=in.pParam_B4SOIvsat;
instanceStruct.d_instanceVar_pParam_B4SOIvsat_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIat=in.pParam_B4SOIat;
instanceStruct.d_instanceVar_pParam_B4SOIat_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIa0=in.pParam_B4SOIa0;
instanceStruct.d_instanceVar_pParam_B4SOIa0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIags=in.pParam_B4SOIags;
instanceStruct.d_instanceVar_pParam_B4SOIags_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIa1=in.pParam_B4SOIa1;
instanceStruct.d_instanceVar_pParam_B4SOIa1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIa2=in.pParam_B4SOIa2;
instanceStruct.d_instanceVar_pParam_B4SOIa2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIketa=in.pParam_B4SOIketa;
instanceStruct.d_instanceVar_pParam_B4SOIketa_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInpeak=in.pParam_B4SOInpeak;
instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInsub=in.pParam_B4SOInsub;
instanceStruct.d_instanceVar_pParam_B4SOInsub_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIngate=in.pParam_B4SOIngate;
instanceStruct.d_instanceVar_pParam_B4SOIngate_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIgamma1=in.pParam_B4SOIgamma1;
instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIgamma2=in.pParam_B4SOIgamma2;
instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbx=in.pParam_B4SOIvbx;
instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbi=in.pParam_B4SOIvbi;
instanceStruct.d_instanceVar_pParam_B4SOIvbi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbm=in.pParam_B4SOIvbm;
instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxt=in.pParam_B4SOIxt;
instanceStruct.d_instanceVar_pParam_B4SOIxt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIphi=in.pParam_B4SOIphi;
instanceStruct.d_instanceVar_pParam_B4SOIphi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlitl=in.pParam_B4SOIlitl;
instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1=in.pParam_B4SOIk1;
instanceStruct.d_instanceVar_pParam_B4SOIk1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkt1=in.pParam_B4SOIkt1;
instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkt1l=in.pParam_B4SOIkt1l;
instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkt2=in.pParam_B4SOIkt2;
instanceStruct.d_instanceVar_pParam_B4SOIkt2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk2=in.pParam_B4SOIk2;
instanceStruct.d_instanceVar_pParam_B4SOIk2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk3=in.pParam_B4SOIk3;
instanceStruct.d_instanceVar_pParam_B4SOIk3_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk3b=in.pParam_B4SOIk3b;
instanceStruct.d_instanceVar_pParam_B4SOIk3b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIw0=in.pParam_B4SOIw0;
instanceStruct.d_instanceVar_pParam_B4SOIw0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlpe0=in.pParam_B4SOIlpe0;
instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt0=in.pParam_B4SOIdvt0;
instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt1=in.pParam_B4SOIdvt1;
instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt2=in.pParam_B4SOIdvt2;
instanceStruct.d_instanceVar_pParam_B4SOIdvt2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt0w=in.pParam_B4SOIdvt0w;
instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt1w=in.pParam_B4SOIdvt1w;
instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt2w=in.pParam_B4SOIdvt2w;
instanceStruct.d_instanceVar_pParam_B4SOIdvt2w_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdrout=in.pParam_B4SOIdrout;
instanceStruct.d_instanceVar_pParam_B4SOIdrout_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdsub=in.pParam_B4SOIdsub;
instanceStruct.d_instanceVar_pParam_B4SOIdsub_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIua=in.pParam_B4SOIua;
instanceStruct.d_instanceVar_pParam_B4SOIua_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIua1=in.pParam_B4SOIua1;
instanceStruct.d_instanceVar_pParam_B4SOIua1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIub=in.pParam_B4SOIub;
instanceStruct.d_instanceVar_pParam_B4SOIub_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIub1=in.pParam_B4SOIub1;
instanceStruct.d_instanceVar_pParam_B4SOIub1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuc=in.pParam_B4SOIuc;
instanceStruct.d_instanceVar_pParam_B4SOIuc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuc1=in.pParam_B4SOIuc1;
instanceStruct.d_instanceVar_pParam_B4SOIuc1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIu0=in.pParam_B4SOIu0;
instanceStruct.d_instanceVar_pParam_B4SOIu0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIute=in.pParam_B4SOIute;
instanceStruct.d_instanceVar_pParam_B4SOIute_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIud=in.pParam_B4SOIud;
instanceStruct.d_instanceVar_pParam_B4SOIud_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIud1=in.pParam_B4SOIud1;
instanceStruct.d_instanceVar_pParam_B4SOIud1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIeu=in.pParam_B4SOIeu;
instanceStruct.d_instanceVar_pParam_B4SOIeu_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIucs=in.pParam_B4SOIucs;
instanceStruct.d_instanceVar_pParam_B4SOIucs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIucste=in.pParam_B4SOIucste;
instanceStruct.d_instanceVar_pParam_B4SOIucste_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvoff=in.pParam_B4SOIvoff;
instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvfb=in.pParam_B4SOIvfb;
instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuatemp=in.pParam_B4SOIuatemp;
instanceStruct.d_instanceVar_pParam_B4SOIuatemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIubtemp=in.pParam_B4SOIubtemp;
instanceStruct.d_instanceVar_pParam_B4SOIubtemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuctemp=in.pParam_B4SOIuctemp;
instanceStruct.d_instanceVar_pParam_B4SOIuctemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrbody=in.pParam_B4SOIrbody;
instanceStruct.d_instanceVar_pParam_B4SOIrbody_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrth=in.pParam_B4SOIrth;
instanceStruct.d_instanceVar_pParam_B4SOIrth_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcth=in.pParam_B4SOIcth;
instanceStruct.d_instanceVar_pParam_B4SOIcth_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrds0denom=in.pParam_B4SOIrds0denom;
instanceStruct.d_instanceVar_pParam_B4SOIrds0denom_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvfbb=in.pParam_B4SOIvfbb;
instanceStruct.d_instanceVar_pParam_B4SOIvfbb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjbjts=in.pParam_B4SOIjbjts;
instanceStruct.d_instanceVar_pParam_B4SOIjbjts_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjbjtd=in.pParam_B4SOIjbjtd;
instanceStruct.d_instanceVar_pParam_B4SOIjbjtd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjdifs=in.pParam_B4SOIjdifs;
instanceStruct.d_instanceVar_pParam_B4SOIjdifs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjdifd=in.pParam_B4SOIjdifd;
instanceStruct.d_instanceVar_pParam_B4SOIjdifd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjrecs=in.pParam_B4SOIjrecs;
instanceStruct.d_instanceVar_pParam_B4SOIjrecs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjrecd=in.pParam_B4SOIjrecd;
instanceStruct.d_instanceVar_pParam_B4SOIjrecd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjtuns=in.pParam_B4SOIjtuns;
instanceStruct.d_instanceVar_pParam_B4SOIjtuns_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjtund=in.pParam_B4SOIjtund;
instanceStruct.d_instanceVar_pParam_B4SOIjtund_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrdw=in.pParam_B4SOIrdw;
instanceStruct.d_instanceVar_pParam_B4SOIrdw_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrsw=in.pParam_B4SOIrsw;
instanceStruct.d_instanceVar_pParam_B4SOIrsw_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrdwmin=in.pParam_B4SOIrdwmin;
instanceStruct.d_instanceVar_pParam_B4SOIrdwmin_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrswmin=in.pParam_B4SOIrswmin;
instanceStruct.d_instanceVar_pParam_B4SOIrswmin_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrd0=in.pParam_B4SOIrd0;
instanceStruct.d_instanceVar_pParam_B4SOIrd0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrs0=in.pParam_B4SOIrs0;
instanceStruct.d_instanceVar_pParam_B4SOIrs0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsdt1=in.pParam_B4SOIsdt1;
instanceStruct.d_instanceVar_pParam_B4SOIsdt1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIst2=in.pParam_B4SOIst2;
instanceStruct.d_instanceVar_pParam_B4SOIst2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIst3=in.pParam_B4SOIst3;
instanceStruct.d_instanceVar_pParam_B4SOIst3_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdt2=in.pParam_B4SOIdt2;
instanceStruct.d_instanceVar_pParam_B4SOIdt2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdt3=in.pParam_B4SOIdt3;
instanceStruct.d_instanceVar_pParam_B4SOIdt3_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdelta=in.pParam_B4SOIdelta;
instanceStruct.d_instanceVar_pParam_B4SOIdelta_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrdsw=in.pParam_B4SOIrdsw;
instanceStruct.d_instanceVar_pParam_B4SOIrdsw_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrds0=in.pParam_B4SOIrds0;
instanceStruct.d_instanceVar_pParam_B4SOIrds0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIprwg=in.pParam_B4SOIprwg;
instanceStruct.d_instanceVar_pParam_B4SOIprwg_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIprwb=in.pParam_B4SOIprwb;
instanceStruct.d_instanceVar_pParam_B4SOIprwb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIprt=in.pParam_B4SOIprt;
instanceStruct.d_instanceVar_pParam_B4SOIprt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIeta0=in.pParam_B4SOIeta0;
instanceStruct.d_instanceVar_pParam_B4SOIeta0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIetab=in.pParam_B4SOIetab;
instanceStruct.d_instanceVar_pParam_B4SOIetab_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIeta0cv=in.pParam_B4SOIeta0cv;
instanceStruct.d_instanceVar_pParam_B4SOIeta0cv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIetabcv=in.pParam_B4SOIetabcv;
instanceStruct.d_instanceVar_pParam_B4SOIetabcv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpclm=in.pParam_B4SOIpclm;
instanceStruct.d_instanceVar_pParam_B4SOIpclm_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdibl1=in.pParam_B4SOIpdibl1;
instanceStruct.d_instanceVar_pParam_B4SOIpdibl1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdibl2=in.pParam_B4SOIpdibl2;
instanceStruct.d_instanceVar_pParam_B4SOIpdibl2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdiblb=in.pParam_B4SOIpdiblb;
instanceStruct.d_instanceVar_pParam_B4SOIpdiblb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpvag=in.pParam_B4SOIpvag;
instanceStruct.d_instanceVar_pParam_B4SOIpvag_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdwg=in.pParam_B4SOIdwg;
instanceStruct.d_instanceVar_pParam_B4SOIdwg_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdwb=in.pParam_B4SOIdwb;
instanceStruct.d_instanceVar_pParam_B4SOIdwb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIb0=in.pParam_B4SOIb0;
instanceStruct.d_instanceVar_pParam_B4SOIb0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIb1=in.pParam_B4SOIb1;
instanceStruct.d_instanceVar_pParam_B4SOIb1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIalpha0=in.pParam_B4SOIalpha0;
instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbeta0=in.pParam_B4SOIbeta0;
instanceStruct.d_instanceVar_pParam_B4SOIbeta0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIaigc=in.pParam_B4SOIaigc;
instanceStruct.d_instanceVar_pParam_B4SOIaigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbigc=in.pParam_B4SOIbigc;
instanceStruct.d_instanceVar_pParam_B4SOIbigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcigc=in.pParam_B4SOIcigc;
instanceStruct.d_instanceVar_pParam_B4SOIcigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIaigsd=in.pParam_B4SOIaigsd;
instanceStruct.d_instanceVar_pParam_B4SOIaigsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbigsd=in.pParam_B4SOIbigsd;
instanceStruct.d_instanceVar_pParam_B4SOIbigsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcigsd=in.pParam_B4SOIcigsd;
instanceStruct.d_instanceVar_pParam_B4SOIcigsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInigc=in.pParam_B4SOInigc;
instanceStruct.d_instanceVar_pParam_B4SOInigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpigcd=in.pParam_B4SOIpigcd;
instanceStruct.d_instanceVar_pParam_B4SOIpigcd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpoxedge=in.pParam_B4SOIpoxedge;
instanceStruct.d_instanceVar_pParam_B4SOIpoxedge_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbsa=in.pParam_B4SOIvbsa;
instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsce=in.pParam_B4SOIvsce;
instanceStruct.d_instanceVar_pParam_B4SOIvsce_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdsbs=in.pParam_B4SOIcdsbs;
instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInofffd=in.pParam_B4SOInofffd;
instanceStruct.d_instanceVar_pParam_B4SOInofffd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvofffd=in.pParam_B4SOIvofffd;
instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1b=in.pParam_B4SOIk1b;
instanceStruct.d_instanceVar_pParam_B4SOIk1b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk2b=in.pParam_B4SOIk2b;
instanceStruct.d_instanceVar_pParam_B4SOIk2b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdk2b=in.pParam_B4SOIdk2b;
instanceStruct.d_instanceVar_pParam_B4SOIdk2b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvbd0=in.pParam_B4SOIdvbd0;
instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvbd1=in.pParam_B4SOIdvbd1;
instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImoinfd=in.pParam_B4SOImoinfd;
instanceStruct.d_instanceVar_pParam_B4SOImoinfd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrcrg1=in.pParam_B4SOIxrcrg1;
instanceStruct.d_instanceVar_pParam_B4SOIxrcrg1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrcrg2=in.pParam_B4SOIxrcrg2;
instanceStruct.d_instanceVar_pParam_B4SOIxrcrg2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIku0=in.pParam_B4SOIku0;
instanceStruct.d_instanceVar_pParam_B4SOIku0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIinv_od_ref=in.pParam_B4SOIinv_od_ref;
instanceStruct.d_instanceVar_pParam_B4SOIinv_od_ref_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgsl=in.pParam_B4SOIcgsl;
instanceStruct.d_instanceVar_pParam_B4SOIcgsl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgdl=in.pParam_B4SOIcgdl;
instanceStruct.d_instanceVar_pParam_B4SOIcgdl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIckappa=in.pParam_B4SOIckappa;
instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIclc=in.pParam_B4SOIclc;
instanceStruct.d_instanceVar_pParam_B4SOIclc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxj=in.pParam_B4SOIxj;
instanceStruct.d_instanceVar_pParam_B4SOIxj_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIalphaGB1=in.pParam_B4SOIalphaGB1;
instanceStruct.d_instanceVar_pParam_B4SOIalphaGB1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbetaGB1=in.pParam_B4SOIbetaGB1;
instanceStruct.d_instanceVar_pParam_B4SOIbetaGB1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIalphaGB2=in.pParam_B4SOIalphaGB2;
instanceStruct.d_instanceVar_pParam_B4SOIalphaGB2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbetaGB2=in.pParam_B4SOIbetaGB2;
instanceStruct.d_instanceVar_pParam_B4SOIbetaGB2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIaigbcp2=in.pParam_B4SOIaigbcp2;
instanceStruct.d_instanceVar_pParam_B4SOIaigbcp2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbigbcp2=in.pParam_B4SOIbigbcp2;
instanceStruct.d_instanceVar_pParam_B4SOIbigbcp2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcigbcp2=in.pParam_B4SOIcigbcp2;
instanceStruct.d_instanceVar_pParam_B4SOIcigbcp2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntrecf=in.pParam_B4SOIntrecf;
instanceStruct.d_instanceVar_pParam_B4SOIntrecf_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntrecr=in.pParam_B4SOIntrecr;
instanceStruct.d_instanceVar_pParam_B4SOIntrecr_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxbjt=in.pParam_B4SOIxbjt;
instanceStruct.d_instanceVar_pParam_B4SOIxbjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxdif=in.pParam_B4SOIxdif;
instanceStruct.d_instanceVar_pParam_B4SOIxdif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrec=in.pParam_B4SOIxrec;
instanceStruct.d_instanceVar_pParam_B4SOIxrec_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxtun=in.pParam_B4SOIxtun;
instanceStruct.d_instanceVar_pParam_B4SOIxtun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxdifd=in.pParam_B4SOIxdifd;
instanceStruct.d_instanceVar_pParam_B4SOIxdifd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrecd=in.pParam_B4SOIxrecd;
instanceStruct.d_instanceVar_pParam_B4SOIxrecd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxtund=in.pParam_B4SOIxtund;
instanceStruct.d_instanceVar_pParam_B4SOIxtund_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkb1=in.pParam_B4SOIkb1;
instanceStruct.d_instanceVar_pParam_B4SOIkb1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1w1=in.pParam_B4SOIk1w1;
instanceStruct.d_instanceVar_pParam_B4SOIk1w1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1w2=in.pParam_B4SOIk1w2;
instanceStruct.d_instanceVar_pParam_B4SOIk1w2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIketas=in.pParam_B4SOIketas;
instanceStruct.d_instanceVar_pParam_B4SOIketas_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfbjtii=in.pParam_B4SOIfbjtii;
instanceStruct.d_instanceVar_pParam_B4SOIfbjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIebjtii=in.pParam_B4SOIebjtii;
instanceStruct.d_instanceVar_pParam_B4SOIebjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcbjtii=in.pParam_B4SOIcbjtii;
instanceStruct.d_instanceVar_pParam_B4SOIcbjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbci=in.pParam_B4SOIvbci;
instanceStruct.d_instanceVar_pParam_B4SOIvbci_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIabjtii=in.pParam_B4SOIabjtii;
instanceStruct.d_instanceVar_pParam_B4SOIabjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImbjtii=in.pParam_B4SOImbjtii;
instanceStruct.d_instanceVar_pParam_B4SOImbjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbeta1=in.pParam_B4SOIbeta1;
instanceStruct.d_instanceVar_pParam_B4SOIbeta1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbeta2=in.pParam_B4SOIbeta2;
instanceStruct.d_instanceVar_pParam_B4SOIbeta2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvdsatii0=in.pParam_B4SOIvdsatii0;
instanceStruct.d_instanceVar_pParam_B4SOIvdsatii0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlii=in.pParam_B4SOIlii;
instanceStruct.d_instanceVar_pParam_B4SOIlii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIesatii=in.pParam_B4SOIesatii;
instanceStruct.d_instanceVar_pParam_B4SOIesatii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsii0=in.pParam_B4SOIsii0;
instanceStruct.d_instanceVar_pParam_B4SOIsii0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsii1=in.pParam_B4SOIsii1;
instanceStruct.d_instanceVar_pParam_B4SOIsii1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsii2=in.pParam_B4SOIsii2;
instanceStruct.d_instanceVar_pParam_B4SOIsii2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsiid=in.pParam_B4SOIsiid;
instanceStruct.d_instanceVar_pParam_B4SOIsiid_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIagidl=in.pParam_B4SOIagidl;
instanceStruct.d_instanceVar_pParam_B4SOIagidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbgidl=in.pParam_B4SOIbgidl;
instanceStruct.d_instanceVar_pParam_B4SOIbgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgidl=in.pParam_B4SOIcgidl;
instanceStruct.d_instanceVar_pParam_B4SOIcgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIegidl=in.pParam_B4SOIegidl;
instanceStruct.d_instanceVar_pParam_B4SOIegidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrgidl=in.pParam_B4SOIrgidl;
instanceStruct.d_instanceVar_pParam_B4SOIrgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkgidl=in.pParam_B4SOIkgidl;
instanceStruct.d_instanceVar_pParam_B4SOIkgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfgidl=in.pParam_B4SOIfgidl;
instanceStruct.d_instanceVar_pParam_B4SOIfgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIagisl=in.pParam_B4SOIagisl;
instanceStruct.d_instanceVar_pParam_B4SOIagisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbgisl=in.pParam_B4SOIbgisl;
instanceStruct.d_instanceVar_pParam_B4SOIbgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgisl=in.pParam_B4SOIcgisl;
instanceStruct.d_instanceVar_pParam_B4SOIcgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIegisl=in.pParam_B4SOIegisl;
instanceStruct.d_instanceVar_pParam_B4SOIegisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrgisl=in.pParam_B4SOIrgisl;
instanceStruct.d_instanceVar_pParam_B4SOIrgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkgisl=in.pParam_B4SOIkgisl;
instanceStruct.d_instanceVar_pParam_B4SOIkgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfgisl=in.pParam_B4SOIfgisl;
instanceStruct.d_instanceVar_pParam_B4SOIfgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntun=in.pParam_B4SOIntun;
instanceStruct.d_instanceVar_pParam_B4SOIntun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntund=in.pParam_B4SOIntund;
instanceStruct.d_instanceVar_pParam_B4SOIntund_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIndiode=in.pParam_B4SOIndiode;
instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIndioded=in.pParam_B4SOIndioded;
instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecf0=in.pParam_B4SOInrecf0;
instanceStruct.d_instanceVar_pParam_B4SOInrecf0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecf0d=in.pParam_B4SOInrecf0d;
instanceStruct.d_instanceVar_pParam_B4SOInrecf0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecr0=in.pParam_B4SOInrecr0;
instanceStruct.d_instanceVar_pParam_B4SOInrecr0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecr0d=in.pParam_B4SOInrecr0d;
instanceStruct.d_instanceVar_pParam_B4SOInrecr0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIisbjt=in.pParam_B4SOIisbjt;
instanceStruct.d_instanceVar_pParam_B4SOIisbjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIidbjt=in.pParam_B4SOIidbjt;
instanceStruct.d_instanceVar_pParam_B4SOIidbjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIisdif=in.pParam_B4SOIisdif;
instanceStruct.d_instanceVar_pParam_B4SOIisdif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIiddif=in.pParam_B4SOIiddif;
instanceStruct.d_instanceVar_pParam_B4SOIiddif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIisrec=in.pParam_B4SOIisrec;
instanceStruct.d_instanceVar_pParam_B4SOIisrec_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIidrec=in.pParam_B4SOIidrec;
instanceStruct.d_instanceVar_pParam_B4SOIidrec_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIistun=in.pParam_B4SOIistun;
instanceStruct.d_instanceVar_pParam_B4SOIistun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIidtun=in.pParam_B4SOIidtun;
instanceStruct.d_instanceVar_pParam_B4SOIidtun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvrec0=in.pParam_B4SOIvrec0;
instanceStruct.d_instanceVar_pParam_B4SOIvrec0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvrec0d=in.pParam_B4SOIvrec0d;
instanceStruct.d_instanceVar_pParam_B4SOIvrec0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvtun0=in.pParam_B4SOIvtun0;
instanceStruct.d_instanceVar_pParam_B4SOIvtun0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvtun0d=in.pParam_B4SOIvtun0d;
instanceStruct.d_instanceVar_pParam_B4SOIvtun0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsdfb=in.pParam_B4SOIvsdfb;
instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsdth=in.pParam_B4SOIvsdth;
instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdelvt=in.pParam_B4SOIdelvt;
instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleff=in.pParam_B4SOIleff;
instanceStruct.d_instanceVar_pParam_B4SOIleff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIweff=in.pParam_B4SOIweff;
instanceStruct.d_instanceVar_pParam_B4SOIweff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleffCV=in.pParam_B4SOIleffCV;
instanceStruct.d_instanceVar_pParam_B4SOIleffCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIweffCV=in.pParam_B4SOIweffCV;
instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIabulkCVfactor=in.pParam_B4SOIabulkCVfactor;
instanceStruct.d_instanceVar_pParam_B4SOIabulkCVfactor_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgso=in.pParam_B4SOIcgso;
instanceStruct.d_instanceVar_pParam_B4SOIcgso_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgdo=in.pParam_B4SOIcgdo;
instanceStruct.d_instanceVar_pParam_B4SOIcgdo_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgeo=in.pParam_B4SOIcgeo;
instanceStruct.d_instanceVar_pParam_B4SOIcgeo_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIu0temp=in.pParam_B4SOIu0temp;
instanceStruct.d_instanceVar_pParam_B4SOIu0temp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsattemp=in.pParam_B4SOIvsattemp;
instanceStruct.d_instanceVar_pParam_B4SOIvsattemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsqrtPhi=in.pParam_B4SOIsqrtPhi;
instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIXdep0=in.pParam_B4SOIXdep0;
instanceStruct.d_instanceVar_pParam_B4SOIXdep0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOItheta0vb0=in.pParam_B4SOItheta0vb0;
instanceStruct.d_instanceVar_pParam_B4SOItheta0vb0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIthetaRout=in.pParam_B4SOIthetaRout;
instanceStruct.d_instanceVar_pParam_B4SOIthetaRout_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIqsi=in.pParam_B4SOIqsi;
instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIoxideRatio=in.pParam_B4SOIoxideRatio;
instanceStruct.d_instanceVar_pParam_B4SOIoxideRatio_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1eff=in.pParam_B4SOIk1eff;
instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdios=in.pParam_B4SOIwdios;
instanceStruct.d_instanceVar_pParam_B4SOIwdios_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdiod=in.pParam_B4SOIwdiod;
instanceStruct.d_instanceVar_pParam_B4SOIwdiod_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdiodCV=in.pParam_B4SOIwdiodCV;
instanceStruct.d_instanceVar_pParam_B4SOIwdiodCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdiosCV=in.pParam_B4SOIwdiosCV;
instanceStruct.d_instanceVar_pParam_B4SOIwdiosCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIarfabjt=in.pParam_B4SOIarfabjt;
instanceStruct.d_instanceVar_pParam_B4SOIarfabjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlratio=in.pParam_B4SOIlratio;
instanceStruct.d_instanceVar_pParam_B4SOIlratio_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlratiodif=in.pParam_B4SOIlratiodif;
instanceStruct.d_instanceVar_pParam_B4SOIlratiodif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvearly=in.pParam_B4SOIvearly;
instanceStruct.d_instanceVar_pParam_B4SOIvearly_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahli=in.pParam_B4SOIahli;
instanceStruct.d_instanceVar_pParam_B4SOIahli_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahlid=in.pParam_B4SOIahlid;
instanceStruct.d_instanceVar_pParam_B4SOIahlid_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahli0s=in.pParam_B4SOIahli0s;
instanceStruct.d_instanceVar_pParam_B4SOIahli0s_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahli0d=in.pParam_B4SOIahli0d;
instanceStruct.d_instanceVar_pParam_B4SOIahli0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIldeb=in.pParam_B4SOIldeb;
instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIacde=in.pParam_B4SOIacde;
instanceStruct.d_instanceVar_pParam_B4SOIacde_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImoin=in.pParam_B4SOImoin;
instanceStruct.d_instanceVar_pParam_B4SOImoin_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInoff=in.pParam_B4SOInoff;
instanceStruct.d_instanceVar_pParam_B4SOInoff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInoff2=in.pParam_B4SOInoff2;
instanceStruct.d_instanceVar_pParam_B4SOInoff2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleffCVb=in.pParam_B4SOIleffCVb;
instanceStruct.d_instanceVar_pParam_B4SOIleffCVb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleffCVbg=in.pParam_B4SOIleffCVbg;
instanceStruct.d_instanceVar_pParam_B4SOIleffCVbg_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdep0=in.pParam_B4SOIcdep0;
instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIAechvb=in.pParam_B4SOIAechvb;
instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIBechvb=in.pParam_B4SOIBechvb;
instanceStruct.d_instanceVar_pParam_B4SOIBechvb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIAechvbEdges=in.pParam_B4SOIAechvbEdges;
instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdges_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIAechvbEdged=in.pParam_B4SOIAechvbEdged;
instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdged_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIBechvbEdge=in.pParam_B4SOIBechvbEdge;
instanceStruct.d_instanceVar_pParam_B4SOIBechvbEdge_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvfbsd=in.pParam_B4SOIvfbsd;
instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1ox=in.pParam_B4SOIk1ox;
instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlpeb=in.pParam_B4SOIlpeb;
instanceStruct.d_instanceVar_pParam_B4SOIlpeb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp0=in.pParam_B4SOIdvtp0;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp1=in.pParam_B4SOIdvtp1;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp4=in.pParam_B4SOIdvtp4;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp4_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImstar=in.pParam_B4SOImstar;
instanceStruct.d_instanceVar_pParam_B4SOImstar_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfprout=in.pParam_B4SOIfprout;
instanceStruct.d_instanceVar_pParam_B4SOIfprout_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdits=in.pParam_B4SOIpdits;
instanceStruct.d_instanceVar_pParam_B4SOIpdits_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpditsd=in.pParam_B4SOIpditsd;
instanceStruct.d_instanceVar_pParam_B4SOIpditsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImstarcv=in.pParam_B4SOImstarcv;
instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvoffcv=in.pParam_B4SOIvoffcv;
instanceStruct.d_instanceVar_pParam_B4SOIvoffcv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp2factor=in.pParam_B4SOIdvtp2factor;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp2factor_dX=0.0;
instanceStruct.instanceVar_B4SOIcbox=in.B4SOIcbox;
instanceStruct.d_instanceVar_B4SOIcbox_dX=0.0;
instanceStruct.instanceVar_B4SOIcsi=in.B4SOIcsi;
instanceStruct.d_instanceVar_B4SOIcsi_dX=0.0;
instanceStruct.instanceVar_B4SOIeg0=in.B4SOIeg0;
instanceStruct.d_instanceVar_B4SOIeg0_dX=0.0;
instanceStruct.instanceVar_B4SOIeg=in.B4SOIeg;
instanceStruct.d_instanceVar_B4SOIeg_dX=0.0;
instanceStruct.instanceVar_B4SOIcox=in.B4SOIcox;
instanceStruct.d_instanceVar_B4SOIcox_dX=0.0;
instanceStruct.instanceVar_B4SOIfactor1=in.B4SOIfactor1;
instanceStruct.d_instanceVar_B4SOIfactor1_dX=0.0;
instanceStruct.instanceVar_here_B4SOIeta0=in.here_B4SOIeta0;
instanceStruct.d_instanceVar_here_B4SOIeta0_dX=0.0;
instanceStruct.instanceVar_here_B4SOIeta0cv=in.here_B4SOIeta0cv;
instanceStruct.d_instanceVar_here_B4SOIeta0cv_dX=0.0;
instanceStruct.instanceVar_here_B4SOIk2=in.here_B4SOIk2;
instanceStruct.d_instanceVar_here_B4SOIk2_dX=0.0;
instanceStruct.instanceVar_here_B4SOIk2ox=in.here_B4SOIk2ox;
instanceStruct.d_instanceVar_here_B4SOIk2ox_dX=0.0;
instanceStruct.instanceVar_here_B4SOIu0temp=in.here_B4SOIu0temp;
instanceStruct.d_instanceVar_here_B4SOIu0temp_dX=0.0;
instanceStruct.instanceVar_here_B4SOIvfb=in.here_B4SOIvfb;
instanceStruct.d_instanceVar_here_B4SOIvfb_dX=0.0;
instanceStruct.instanceVar_here_B4SOIvsattemp=in.here_B4SOIvsattemp;
instanceStruct.d_instanceVar_here_B4SOIvsattemp_dX=0.0;
instanceStruct.instanceVar_here_B4SOIvth0=in.here_B4SOIvth0;
instanceStruct.d_instanceVar_here_B4SOIvth0_dX=0.0;
instanceStruct.instanceVar_DevTemp=in.DevTemp;
instanceStruct.d_instanceVar_DevTemp_dX=0.0;
instanceStruct.instanceVar_fourkt=in.fourkt;
instanceStruct.d_instanceVar_fourkt_dX=0.0;
instanceStruct.instanceVar_gdnoise=in.gdnoise;
instanceStruct.d_instanceVar_gdnoise_dX=0.0;
instanceStruct.instanceVar_gsnoise=in.gsnoise;
instanceStruct.d_instanceVar_gsnoise_dX=0.0;
instanceStruct.instanceVar_toxe=in.toxe;
instanceStruct.d_instanceVar_toxe_dX=0.0;
instanceStruct.instanceVar_epsrox=in.epsrox;
instanceStruct.d_instanceVar_epsrox_dX=0.0;
instanceStruct.instanceVar_epssub=in.epssub;
instanceStruct.d_instanceVar_epssub_dX=0.0;
instanceStruct.instanceVar_sqrt2qeps=in.sqrt2qeps;
instanceStruct.d_instanceVar_sqrt2qeps_dX=0.0;
instanceStruct.instanceVar_eggbcp2=in.eggbcp2;
instanceStruct.d_instanceVar_eggbcp2_dX=0.0;
instanceStruct.instanceVar_eggdep=in.eggdep;
instanceStruct.d_instanceVar_eggdep_dX=0.0;
instanceStruct.instanceVar_Vtm00=in.Vtm00;
instanceStruct.d_instanceVar_Vtm00_dX=0.0;
instanceStruct.instanceVar_agb1=in.agb1;
instanceStruct.d_instanceVar_agb1_dX=0.0;
instanceStruct.instanceVar_bgb1=in.bgb1;
instanceStruct.d_instanceVar_bgb1_dX=0.0;
instanceStruct.instanceVar_agb2=in.agb2;
instanceStruct.d_instanceVar_agb2_dX=0.0;
instanceStruct.instanceVar_bgb2=in.bgb2;
instanceStruct.d_instanceVar_bgb2_dX=0.0;
instanceStruct.instanceVar_agbc2n=in.agbc2n;
instanceStruct.d_instanceVar_agbc2n_dX=0.0;
instanceStruct.instanceVar_agbc2p=in.agbc2p;
instanceStruct.d_instanceVar_agbc2p_dX=0.0;
instanceStruct.instanceVar_bgbc2n=in.bgbc2n;
instanceStruct.d_instanceVar_bgbc2n_dX=0.0;
instanceStruct.instanceVar_bgbc2p=in.bgbc2p;
instanceStruct.d_instanceVar_bgbc2p_dX=0.0;


//non-reals
instanceStruct.instanceVar_B4SOItype=in.B4SOItype;
instanceStruct.instanceVar_B4SOIbjtoff=in.B4SOIbjtoff;
instanceStruct.instanceVar_B4SOIsoiMod=in.B4SOIsoiMod;
instanceStruct.instanceVar_B4SOIrgateMod=in.B4SOIrgateMod;
instanceStruct.instanceVar_B4SOIrbodyMod=in.B4SOIrbodyMod;
instanceStruct.instanceVar_B4SOImtrlMod=in.B4SOImtrlMod;
instanceStruct.instanceVar_B4SOIvgstcvMod=in.B4SOIvgstcvMod;
instanceStruct.instanceVar_B4SOIgidlMod=in.B4SOIgidlMod;
instanceStruct.instanceVar_B4SOIiiiMod=in.B4SOIiiiMod;
instanceStruct.instanceVar_B4SOIcapMod=in.B4SOIcapMod;
instanceStruct.instanceVar_B4SOImobMod=in.B4SOImobMod;
instanceStruct.instanceVar_B4SOIigbMod=in.B4SOIigbMod;
instanceStruct.instanceVar_B4SOIigcMod=in.B4SOIigcMod;
instanceStruct.instanceVar_B4SOIrdsMod=in.B4SOIrdsMod;
instanceStruct.instanceVar_B4SOIfdMod=in.B4SOIfdMod;
instanceStruct.instanceVar_B4SOImode=in.B4SOImode;
instanceStruct.instanceVar_B4SOIbodyMod=in.B4SOIbodyMod;


//make local copies of all model vars
//reals


// non-reals



Linear::Vector * solVectorPtr = in.extData.nextSolVectorPtr;

// extract solution variables
probeVars[in.admsProbeID_V_sb_GND] = (*solVectorPtr)[in.li_sb];
probeVars[in.admsProbeID_V_db_GND] = (*solVectorPtr)[in.li_db];
probeVars[in.admsProbeID_V_gm_GND] = (*solVectorPtr)[in.li_gm];
probeVars[in.admsProbeID_V_b_sb] = (*solVectorPtr)[in.li_b]- (*solVectorPtr)[in.li_sb];
probeVars[in.admsProbeID_V_b_db] = (*solVectorPtr)[in.li_b]- (*solVectorPtr)[in.li_db];
probeVars[in.admsProbeID_V_gm_gi] = (*solVectorPtr)[in.li_gm]- (*solVectorPtr)[in.li_gi];
probeVars[in.admsProbeID_V_g_gm] = (*solVectorPtr)[in.li_g]- (*solVectorPtr)[in.li_gm];
probeVars[in.admsProbeID_V_gi_e] = (*solVectorPtr)[in.li_gi]- (*solVectorPtr)[in.li_e];
probeVars[in.admsProbeID_V_gm_e] = (*solVectorPtr)[in.li_gm]- (*solVectorPtr)[in.li_e];
probeVars[in.admsProbeID_V_s_si] = (*solVectorPtr)[in.li_s]- (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_d_di] = (*solVectorPtr)[in.li_d]- (*solVectorPtr)[in.li_di];
probeVars[in.admsProbeID_V_si_GND] = (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_di_GND] = (*solVectorPtr)[in.li_di];
probeVars[in.admsProbeID_V_gi_GND] = (*solVectorPtr)[in.li_gi];
probeVars[in.admsProbeID_V_gm_si] = (*solVectorPtr)[in.li_gm]- (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_db_di] = (*solVectorPtr)[in.li_db]- (*solVectorPtr)[in.li_di];
probeVars[in.admsProbeID_V_sb_si] = (*solVectorPtr)[in.li_sb]- (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_gi_p] = (*solVectorPtr)[in.li_gi]- (*solVectorPtr)[in.li_p];
probeVars[in.admsProbeID_V_b_p] = (*solVectorPtr)[in.li_b]- (*solVectorPtr)[in.li_p];
probeVars[in.admsProbeID_V_e_si] = (*solVectorPtr)[in.li_e]- (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_gi_si] = (*solVectorPtr)[in.li_gi]- (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_b_si] = (*solVectorPtr)[in.li_b]- (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_di_si] = (*solVectorPtr)[in.li_di]- (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_Temp_t_GND] = (in.collapseNode_t)?(0):((*solVectorPtr)[in.li_t]);
probeVars[in.admsProbeID_V_p_GND] = (*solVectorPtr)[in.li_p];
probeVars[in.admsProbeID_V_b_GND] = (*solVectorPtr)[in.li_b];


evaluateInitialInstance(
instanceStruct,
modelStruct,
in.admsNodeID_d,
in.admsNodeID_g,
in.admsNodeID_s,
in.admsNodeID_e,
in.admsNodeID_p,
in.admsNodeID_b,
in.admsNodeID_t,
in.admsNodeID_di,
in.admsNodeID_si,
in.admsNodeID_gi,
in.admsNodeID_gm,
in.admsNodeID_sb,
in.admsNodeID_db,
 in.portsConnected_,
in.admsTemperature,in.adms_vt_nom,in.getDeviceOptions().gmin,in);


// Now call  the function that does the heavy lifting.
evaluateModelEquations(
probeVars,
// probe constants
in.admsProbeID_V_sb_GND,
in.admsProbeID_V_db_GND,
in.admsProbeID_V_gm_GND,
in.admsProbeID_V_b_sb,
in.admsProbeID_V_b_db,
in.admsProbeID_V_gm_gi,
in.admsProbeID_V_g_gm,
in.admsProbeID_V_gi_e,
in.admsProbeID_V_gm_e,
in.admsProbeID_V_s_si,
in.admsProbeID_V_d_di,
in.admsProbeID_V_si_GND,
in.admsProbeID_V_di_GND,
in.admsProbeID_V_gi_GND,
in.admsProbeID_V_gm_si,
in.admsProbeID_V_db_di,
in.admsProbeID_V_sb_si,
in.admsProbeID_V_gi_p,
in.admsProbeID_V_b_p,
in.admsProbeID_V_e_si,
in.admsProbeID_V_gi_si,
in.admsProbeID_V_b_si,
in.admsProbeID_V_di_si,
in.admsProbeID_Temp_t_GND,
in.admsProbeID_V_p_GND,
in.admsProbeID_V_b_GND,

// node constants
in.admsNodeID_d,
in.admsNodeID_g,
in.admsNodeID_s,
in.admsNodeID_e,
in.admsNodeID_p,
in.admsNodeID_b,
in.admsNodeID_t,
in.admsNodeID_di,
in.admsNodeID_si,
in.admsNodeID_gi,
in.admsNodeID_gm,
in.admsNodeID_sb,
in.admsNodeID_db,
instanceStruct,
modelStruct,
in.portsConnected_,
 in.admsTemperature,
 in.adms_vt_nom,
 in.getDeviceOptions().gmin,
 d_staticContributions_dX,
 d_dynamicContributions_dX,
 in);


// We now have the F and Q vector stuff, populate the dependencies:

    dfdp[in.admsNodeID_d] += d_staticContributions_dX[in.admsNodeID_d];
    dqdp[in.admsNodeID_d] += d_dynamicContributions_dX[in.admsNodeID_d];
    Findices[in.admsNodeID_d] = in.li_d;
    Qindices[in.admsNodeID_d] = in.li_d;
    dfdp[in.admsNodeID_g] += d_staticContributions_dX[in.admsNodeID_g];
    dqdp[in.admsNodeID_g] += d_dynamicContributions_dX[in.admsNodeID_g];
    Findices[in.admsNodeID_g] = in.li_g;
    Qindices[in.admsNodeID_g] = in.li_g;
    dfdp[in.admsNodeID_s] += d_staticContributions_dX[in.admsNodeID_s];
    dqdp[in.admsNodeID_s] += d_dynamicContributions_dX[in.admsNodeID_s];
    Findices[in.admsNodeID_s] = in.li_s;
    Qindices[in.admsNodeID_s] = in.li_s;
    dfdp[in.admsNodeID_e] += d_staticContributions_dX[in.admsNodeID_e];
    dqdp[in.admsNodeID_e] += d_dynamicContributions_dX[in.admsNodeID_e];
    Findices[in.admsNodeID_e] = in.li_e;
    Qindices[in.admsNodeID_e] = in.li_e;
    dfdp[in.admsNodeID_p] += d_staticContributions_dX[in.admsNodeID_p];
    dqdp[in.admsNodeID_p] += d_dynamicContributions_dX[in.admsNodeID_p];
    Findices[in.admsNodeID_p] = in.li_p;
    Qindices[in.admsNodeID_p] = in.li_p;
    dfdp[in.admsNodeID_b] += d_staticContributions_dX[in.admsNodeID_b];
    dqdp[in.admsNodeID_b] += d_dynamicContributions_dX[in.admsNodeID_b];
    Findices[in.admsNodeID_b] = in.li_b;
    Qindices[in.admsNodeID_b] = in.li_b;
    dfdp[in.admsNodeID_t] += d_staticContributions_dX[in.admsNodeID_t];
    dqdp[in.admsNodeID_t] += d_dynamicContributions_dX[in.admsNodeID_t];
    Findices[in.admsNodeID_t] = in.li_t;
    Qindices[in.admsNodeID_t] = in.li_t;
    dfdp[in.admsNodeID_di] += d_staticContributions_dX[in.admsNodeID_di];
    dqdp[in.admsNodeID_di] += d_dynamicContributions_dX[in.admsNodeID_di];
    Findices[in.admsNodeID_di] = in.li_di;
    Qindices[in.admsNodeID_di] = in.li_di;
    dfdp[in.admsNodeID_si] += d_staticContributions_dX[in.admsNodeID_si];
    dqdp[in.admsNodeID_si] += d_dynamicContributions_dX[in.admsNodeID_si];
    Findices[in.admsNodeID_si] = in.li_si;
    Qindices[in.admsNodeID_si] = in.li_si;
    dfdp[in.admsNodeID_gi] += d_staticContributions_dX[in.admsNodeID_gi];
    dqdp[in.admsNodeID_gi] += d_dynamicContributions_dX[in.admsNodeID_gi];
    Findices[in.admsNodeID_gi] = in.li_gi;
    Qindices[in.admsNodeID_gi] = in.li_gi;
    dfdp[in.admsNodeID_gm] += d_staticContributions_dX[in.admsNodeID_gm];
    dqdp[in.admsNodeID_gm] += d_dynamicContributions_dX[in.admsNodeID_gm];
    Findices[in.admsNodeID_gm] = in.li_gm;
    Qindices[in.admsNodeID_gm] = in.li_gm;
    dfdp[in.admsNodeID_sb] += d_staticContributions_dX[in.admsNodeID_sb];
    dqdp[in.admsNodeID_sb] += d_dynamicContributions_dX[in.admsNodeID_sb];
    Findices[in.admsNodeID_sb] = in.li_sb;
    Qindices[in.admsNodeID_sb] = in.li_sb;
    dfdp[in.admsNodeID_db] += d_staticContributions_dX[in.admsNodeID_db];
    dqdp[in.admsNodeID_db] += d_dynamicContributions_dX[in.admsNodeID_db];
    Findices[in.admsNodeID_db] = in.li_db;
    Qindices[in.admsNodeID_db] = in.li_db;


}

//-----------------------------------------------------------------------------
// Function      : ModelSensitivity::operator()
// Purpose       : return sensitivity for a single model parameter
// Special Notes :
// Scope         : public
// Creator       : admsXml
// Creation Date :
//-----------------------------------------------------------------------------
/// Compute sensitivity to a specified model
/// parameter of all device instances of that model's outputs
///
///
/// @param[in] entity  The device entity that owns the parameter
/// @param[in] name    The unadorned parameter name specific to the entity
/// @param[out] dfdp   The derivative of the F vector with respect to the parameter
/// @param[out] dqdp   The derivative of the Q vector with respect to the parameter
/// @param[out] dbdp   The derivative of the B vector with respect to the parameter
/// @param[out] Findices   The indices into the F vector corresponding to the elements of dfdp
/// @param[out] Qindices   The indices into the Q vector corresponding to the elements of dfdp
/// @param[out] Bindices   The indices into the B vector corresponding to the elements of dfdp
///
///
void ModelSensitivity::operator()
(
const ParameterBase &entity,
const std::string &name,
std::vector<double> & dfdp,
std::vector<double> & dqdp,
std::vector<double> & dbdp,
std::vector<int> & Findices,
std::vector<int> & Qindices,
std::vector<int> & Bindices
) const
{
const ParameterBase * e1 = &entity;
const Model & mod = *(dynamic_cast<const Model *> (e1));
int sizeInstance = mod.instanceContainer.size();

dfdp.resize((13+0)*sizeInstance);
dqdp.resize((13+0)*sizeInstance);
Findices.resize((13+0)*sizeInstance);
Qindices.resize((13+0)*sizeInstance);

std::vector <double> probeVars(26);
std::vector <double> d_staticContributions_dX(13+0);
std::vector <double> d_dynamicContributions_dX(13+0);


// Copy out all the model parameters (doubles) into local structure
modelSensStruct modelStruct;

// Keep a map so we can set the right one to the independent variable
// We do this solely to avoid a big ugly "if/else" block just to find the
// one parameter we're doing sensitivities on.
unordered_map <std::string,double*,HashNoCase,EqualNoCase> modParamMap;

// reals
modelStruct.modelPar_L=mod.L;
modelStruct.d_modelPar_L_dX=0.0;
modelStruct.modelPar_given_L=mod.given("L");
modParamMap["L"] = &(modelStruct.d_modelPar_L_dX);
modelStruct.modelPar_W=mod.W;
modelStruct.d_modelPar_W_dX=0.0;
modelStruct.modelPar_given_W=mod.given("W");
modParamMap["W"] = &(modelStruct.d_modelPar_W_dX);
modelStruct.modelPar_SA=mod.SA;
modelStruct.d_modelPar_SA_dX=0.0;
modelStruct.modelPar_given_SA=mod.given("SA");
modParamMap["SA"] = &(modelStruct.d_modelPar_SA_dX);
modelStruct.modelPar_SB=mod.SB;
modelStruct.d_modelPar_SB_dX=0.0;
modelStruct.modelPar_given_SB=mod.given("SB");
modParamMap["SB"] = &(modelStruct.d_modelPar_SB_dX);
modelStruct.modelPar_SD=mod.SD;
modelStruct.d_modelPar_SD_dX=0.0;
modelStruct.modelPar_given_SD=mod.given("SD");
modParamMap["SD"] = &(modelStruct.d_modelPar_SD_dX);
modelStruct.modelPar_AD=mod.AD;
modelStruct.d_modelPar_AD_dX=0.0;
modelStruct.modelPar_given_AD=mod.given("AD");
modParamMap["AD"] = &(modelStruct.d_modelPar_AD_dX);
modelStruct.modelPar_AS=mod.AS;
modelStruct.d_modelPar_AS_dX=0.0;
modelStruct.modelPar_given_AS=mod.given("AS");
modParamMap["AS"] = &(modelStruct.d_modelPar_AS_dX);
modelStruct.modelPar_PD=mod.PD;
modelStruct.d_modelPar_PD_dX=0.0;
modelStruct.modelPar_given_PD=mod.given("PD");
modParamMap["PD"] = &(modelStruct.d_modelPar_PD_dX);
modelStruct.modelPar_PS=mod.PS;
modelStruct.d_modelPar_PS_dX=0.0;
modelStruct.modelPar_given_PS=mod.given("PS");
modParamMap["PS"] = &(modelStruct.d_modelPar_PS_dX);
modelStruct.modelPar_NRD=mod.NRD;
modelStruct.d_modelPar_NRD_dX=0.0;
modelStruct.modelPar_given_NRD=mod.given("NRD");
modParamMap["NRD"] = &(modelStruct.d_modelPar_NRD_dX);
modelStruct.modelPar_NRS=mod.NRS;
modelStruct.d_modelPar_NRS_dX=0.0;
modelStruct.modelPar_given_NRS=mod.given("NRS");
modParamMap["NRS"] = &(modelStruct.d_modelPar_NRS_dX);
modelStruct.modelPar_RTH0=mod.RTH0;
modelStruct.d_modelPar_RTH0_dX=0.0;
modelStruct.modelPar_given_RTH0=mod.given("RTH0");
modParamMap["RTH0"] = &(modelStruct.d_modelPar_RTH0_dX);
modelStruct.modelPar_CTH0=mod.CTH0;
modelStruct.d_modelPar_CTH0_dX=0.0;
modelStruct.modelPar_given_CTH0=mod.given("CTH0");
modParamMap["CTH0"] = &(modelStruct.d_modelPar_CTH0_dX);
modelStruct.modelPar_NRB=mod.NRB;
modelStruct.d_modelPar_NRB_dX=0.0;
modelStruct.modelPar_given_NRB=mod.given("NRB");
modParamMap["NRB"] = &(modelStruct.d_modelPar_NRB_dX);
modelStruct.modelPar_FRBODY=mod.FRBODY;
modelStruct.d_modelPar_FRBODY_dX=0.0;
modelStruct.modelPar_given_FRBODY=mod.given("FRBODY");
modParamMap["FRBODY"] = &(modelStruct.d_modelPar_FRBODY_dX);
modelStruct.modelPar_RBDB=mod.RBDB;
modelStruct.d_modelPar_RBDB_dX=0.0;
modelStruct.modelPar_given_RBDB=mod.given("RBDB");
modParamMap["RBDB"] = &(modelStruct.d_modelPar_RBDB_dX);
modelStruct.modelPar_RBSB=mod.RBSB;
modelStruct.d_modelPar_RBSB_dX=0.0;
modelStruct.modelPar_given_RBSB=mod.given("RBSB");
modParamMap["RBSB"] = &(modelStruct.d_modelPar_RBSB_dX);
modelStruct.modelPar_DELVTO=mod.DELVTO;
modelStruct.d_modelPar_DELVTO_dX=0.0;
modelStruct.modelPar_given_DELVTO=mod.given("DELVTO");
modParamMap["DELVTO"] = &(modelStruct.d_modelPar_DELVTO_dX);
modelStruct.modelPar_NBC=mod.NBC;
modelStruct.d_modelPar_NBC_dX=0.0;
modelStruct.modelPar_given_NBC=mod.given("NBC");
modParamMap["NBC"] = &(modelStruct.d_modelPar_NBC_dX);
modelStruct.modelPar_NSEG=mod.NSEG;
modelStruct.d_modelPar_NSEG_dX=0.0;
modelStruct.modelPar_given_NSEG=mod.given("NSEG");
modParamMap["NSEG"] = &(modelStruct.d_modelPar_NSEG_dX);
modelStruct.modelPar_PDBCP=mod.PDBCP;
modelStruct.d_modelPar_PDBCP_dX=0.0;
modelStruct.modelPar_given_PDBCP=mod.given("PDBCP");
modParamMap["PDBCP"] = &(modelStruct.d_modelPar_PDBCP_dX);
modelStruct.modelPar_PSBCP=mod.PSBCP;
modelStruct.d_modelPar_PSBCP_dX=0.0;
modelStruct.modelPar_given_PSBCP=mod.given("PSBCP");
modParamMap["PSBCP"] = &(modelStruct.d_modelPar_PSBCP_dX);
modelStruct.modelPar_AGBCP=mod.AGBCP;
modelStruct.d_modelPar_AGBCP_dX=0.0;
modelStruct.modelPar_given_AGBCP=mod.given("AGBCP");
modParamMap["AGBCP"] = &(modelStruct.d_modelPar_AGBCP_dX);
modelStruct.modelPar_AGBCP2=mod.AGBCP2;
modelStruct.d_modelPar_AGBCP2_dX=0.0;
modelStruct.modelPar_given_AGBCP2=mod.given("AGBCP2");
modParamMap["AGBCP2"] = &(modelStruct.d_modelPar_AGBCP2_dX);
modelStruct.modelPar_AGBCPD=mod.AGBCPD;
modelStruct.d_modelPar_AGBCPD_dX=0.0;
modelStruct.modelPar_given_AGBCPD=mod.given("AGBCPD");
modParamMap["AGBCPD"] = &(modelStruct.d_modelPar_AGBCPD_dX);
modelStruct.modelPar_AEBCP=mod.AEBCP;
modelStruct.d_modelPar_AEBCP_dX=0.0;
modelStruct.modelPar_given_AEBCP=mod.given("AEBCP");
modParamMap["AEBCP"] = &(modelStruct.d_modelPar_AEBCP_dX);
modelStruct.modelPar_VERSION=mod.VERSION;
modelStruct.d_modelPar_VERSION_dX=0.0;
modelStruct.modelPar_given_VERSION=mod.given("VERSION");
modParamMap["VERSION"] = &(modelStruct.d_modelPar_VERSION_dX);
modelStruct.modelPar_VBSUSR=mod.VBSUSR;
modelStruct.d_modelPar_VBSUSR_dX=0.0;
modelStruct.modelPar_given_VBSUSR=mod.given("VBSUSR");
modParamMap["VBSUSR"] = &(modelStruct.d_modelPar_VBSUSR_dX);
modelStruct.modelPar_EOT=mod.EOT;
modelStruct.d_modelPar_EOT_dX=0.0;
modelStruct.modelPar_given_EOT=mod.given("EOT");
modParamMap["EOT"] = &(modelStruct.d_modelPar_EOT_dX);
modelStruct.modelPar_EPSROX=mod.EPSROX;
modelStruct.d_modelPar_EPSROX_dX=0.0;
modelStruct.modelPar_given_EPSROX=mod.given("EPSROX");
modParamMap["EPSROX"] = &(modelStruct.d_modelPar_EPSROX_dX);
modelStruct.modelPar_EPSRSUB=mod.EPSRSUB;
modelStruct.d_modelPar_EPSRSUB_dX=0.0;
modelStruct.modelPar_given_EPSRSUB=mod.given("EPSRSUB");
modParamMap["EPSRSUB"] = &(modelStruct.d_modelPar_EPSRSUB_dX);
modelStruct.modelPar_NI0SUB=mod.NI0SUB;
modelStruct.d_modelPar_NI0SUB_dX=0.0;
modelStruct.modelPar_given_NI0SUB=mod.given("NI0SUB");
modParamMap["NI0SUB"] = &(modelStruct.d_modelPar_NI0SUB_dX);
modelStruct.modelPar_BG0SUB=mod.BG0SUB;
modelStruct.d_modelPar_BG0SUB_dX=0.0;
modelStruct.modelPar_given_BG0SUB=mod.given("BG0SUB");
modParamMap["BG0SUB"] = &(modelStruct.d_modelPar_BG0SUB_dX);
modelStruct.modelPar_TBGASUB=mod.TBGASUB;
modelStruct.d_modelPar_TBGASUB_dX=0.0;
modelStruct.modelPar_given_TBGASUB=mod.given("TBGASUB");
modParamMap["TBGASUB"] = &(modelStruct.d_modelPar_TBGASUB_dX);
modelStruct.modelPar_TBGBSUB=mod.TBGBSUB;
modelStruct.d_modelPar_TBGBSUB_dX=0.0;
modelStruct.modelPar_given_TBGBSUB=mod.given("TBGBSUB");
modParamMap["TBGBSUB"] = &(modelStruct.d_modelPar_TBGBSUB_dX);
modelStruct.modelPar_PHIG=mod.PHIG;
modelStruct.d_modelPar_PHIG_dX=0.0;
modelStruct.modelPar_given_PHIG=mod.given("PHIG");
modParamMap["PHIG"] = &(modelStruct.d_modelPar_PHIG_dX);
modelStruct.modelPar_EASUB=mod.EASUB;
modelStruct.d_modelPar_EASUB_dX=0.0;
modelStruct.modelPar_given_EASUB=mod.given("EASUB");
modParamMap["EASUB"] = &(modelStruct.d_modelPar_EASUB_dX);
modelStruct.modelPar_LEFFEOT=mod.LEFFEOT;
modelStruct.d_modelPar_LEFFEOT_dX=0.0;
modelStruct.modelPar_given_LEFFEOT=mod.given("LEFFEOT");
modParamMap["LEFFEOT"] = &(modelStruct.d_modelPar_LEFFEOT_dX);
modelStruct.modelPar_WEFFEOT=mod.WEFFEOT;
modelStruct.d_modelPar_WEFFEOT_dX=0.0;
modelStruct.modelPar_given_WEFFEOT=mod.given("WEFFEOT");
modParamMap["WEFFEOT"] = &(modelStruct.d_modelPar_WEFFEOT_dX);
modelStruct.modelPar_VDDEOT=mod.VDDEOT;
modelStruct.d_modelPar_VDDEOT_dX=0.0;
modelStruct.modelPar_given_VDDEOT=mod.given("VDDEOT");
modParamMap["VDDEOT"] = &(modelStruct.d_modelPar_VDDEOT_dX);
modelStruct.modelPar_TEMPEOT=mod.TEMPEOT;
modelStruct.d_modelPar_TEMPEOT_dX=0.0;
modelStruct.modelPar_given_TEMPEOT=mod.given("TEMPEOT");
modParamMap["TEMPEOT"] = &(modelStruct.d_modelPar_TEMPEOT_dX);
modelStruct.modelPar_ADOS=mod.ADOS;
modelStruct.d_modelPar_ADOS_dX=0.0;
modelStruct.modelPar_given_ADOS=mod.given("ADOS");
modParamMap["ADOS"] = &(modelStruct.d_modelPar_ADOS_dX);
modelStruct.modelPar_BDOS=mod.BDOS;
modelStruct.d_modelPar_BDOS_dX=0.0;
modelStruct.modelPar_given_BDOS=mod.given("BDOS");
modParamMap["BDOS"] = &(modelStruct.d_modelPar_BDOS_dX);
modelStruct.modelPar_EPSRGATE=mod.EPSRGATE;
modelStruct.d_modelPar_EPSRGATE_dX=0.0;
modelStruct.modelPar_given_EPSRGATE=mod.given("EPSRGATE");
modParamMap["EPSRGATE"] = &(modelStruct.d_modelPar_EPSRGATE_dX);
modelStruct.modelPar_TOX=mod.TOX;
modelStruct.d_modelPar_TOX_dX=0.0;
modelStruct.modelPar_given_TOX=mod.given("TOX");
modParamMap["TOX"] = &(modelStruct.d_modelPar_TOX_dX);
modelStruct.modelPar_TOXP=mod.TOXP;
modelStruct.d_modelPar_TOXP_dX=0.0;
modelStruct.modelPar_given_TOXP=mod.given("TOXP");
modParamMap["TOXP"] = &(modelStruct.d_modelPar_TOXP_dX);
modelStruct.modelPar_TOXM=mod.TOXM;
modelStruct.d_modelPar_TOXM_dX=0.0;
modelStruct.modelPar_given_TOXM=mod.given("TOXM");
modParamMap["TOXM"] = &(modelStruct.d_modelPar_TOXM_dX);
modelStruct.modelPar_DTOXCV=mod.DTOXCV;
modelStruct.d_modelPar_DTOXCV_dX=0.0;
modelStruct.modelPar_given_DTOXCV=mod.given("DTOXCV");
modParamMap["DTOXCV"] = &(modelStruct.d_modelPar_DTOXCV_dX);
modelStruct.modelPar_CDSC=mod.CDSC;
modelStruct.d_modelPar_CDSC_dX=0.0;
modelStruct.modelPar_given_CDSC=mod.given("CDSC");
modParamMap["CDSC"] = &(modelStruct.d_modelPar_CDSC_dX);
modelStruct.modelPar_CDSCB=mod.CDSCB;
modelStruct.d_modelPar_CDSCB_dX=0.0;
modelStruct.modelPar_given_CDSCB=mod.given("CDSCB");
modParamMap["CDSCB"] = &(modelStruct.d_modelPar_CDSCB_dX);
modelStruct.modelPar_CDSCD=mod.CDSCD;
modelStruct.d_modelPar_CDSCD_dX=0.0;
modelStruct.modelPar_given_CDSCD=mod.given("CDSCD");
modParamMap["CDSCD"] = &(modelStruct.d_modelPar_CDSCD_dX);
modelStruct.modelPar_CIT=mod.CIT;
modelStruct.d_modelPar_CIT_dX=0.0;
modelStruct.modelPar_given_CIT=mod.given("CIT");
modParamMap["CIT"] = &(modelStruct.d_modelPar_CIT_dX);
modelStruct.modelPar_NFACTOR=mod.NFACTOR;
modelStruct.d_modelPar_NFACTOR_dX=0.0;
modelStruct.modelPar_given_NFACTOR=mod.given("NFACTOR");
modParamMap["NFACTOR"] = &(modelStruct.d_modelPar_NFACTOR_dX);
modelStruct.modelPar_VSAT=mod.VSAT;
modelStruct.d_modelPar_VSAT_dX=0.0;
modelStruct.modelPar_given_VSAT=mod.given("VSAT");
modParamMap["VSAT"] = &(modelStruct.d_modelPar_VSAT_dX);
modelStruct.modelPar_AT=mod.AT;
modelStruct.d_modelPar_AT_dX=0.0;
modelStruct.modelPar_given_AT=mod.given("AT");
modParamMap["AT"] = &(modelStruct.d_modelPar_AT_dX);
modelStruct.modelPar_A0=mod.A0;
modelStruct.d_modelPar_A0_dX=0.0;
modelStruct.modelPar_given_A0=mod.given("A0");
modParamMap["A0"] = &(modelStruct.d_modelPar_A0_dX);
modelStruct.modelPar_AGS=mod.AGS;
modelStruct.d_modelPar_AGS_dX=0.0;
modelStruct.modelPar_given_AGS=mod.given("AGS");
modParamMap["AGS"] = &(modelStruct.d_modelPar_AGS_dX);
modelStruct.modelPar_A1=mod.A1;
modelStruct.d_modelPar_A1_dX=0.0;
modelStruct.modelPar_given_A1=mod.given("A1");
modParamMap["A1"] = &(modelStruct.d_modelPar_A1_dX);
modelStruct.modelPar_A2=mod.A2;
modelStruct.d_modelPar_A2_dX=0.0;
modelStruct.modelPar_given_A2=mod.given("A2");
modParamMap["A2"] = &(modelStruct.d_modelPar_A2_dX);
modelStruct.modelPar_KETA=mod.KETA;
modelStruct.d_modelPar_KETA_dX=0.0;
modelStruct.modelPar_given_KETA=mod.given("KETA");
modParamMap["KETA"] = &(modelStruct.d_modelPar_KETA_dX);
modelStruct.modelPar_NSUB=mod.NSUB;
modelStruct.d_modelPar_NSUB_dX=0.0;
modelStruct.modelPar_given_NSUB=mod.given("NSUB");
modParamMap["NSUB"] = &(modelStruct.d_modelPar_NSUB_dX);
modelStruct.modelPar_NCH=mod.NCH;
modelStruct.d_modelPar_NCH_dX=0.0;
modelStruct.modelPar_given_NCH=mod.given("NCH");
modParamMap["NCH"] = &(modelStruct.d_modelPar_NCH_dX);
modelStruct.modelPar_NGATE=mod.NGATE;
modelStruct.d_modelPar_NGATE_dX=0.0;
modelStruct.modelPar_given_NGATE=mod.given("NGATE");
modParamMap["NGATE"] = &(modelStruct.d_modelPar_NGATE_dX);
modelStruct.modelPar_NSD=mod.NSD;
modelStruct.d_modelPar_NSD_dX=0.0;
modelStruct.modelPar_given_NSD=mod.given("NSD");
modParamMap["NSD"] = &(modelStruct.d_modelPar_NSD_dX);
modelStruct.modelPar_GAMMA1=mod.GAMMA1;
modelStruct.d_modelPar_GAMMA1_dX=0.0;
modelStruct.modelPar_given_GAMMA1=mod.given("GAMMA1");
modParamMap["GAMMA1"] = &(modelStruct.d_modelPar_GAMMA1_dX);
modelStruct.modelPar_GAMMA2=mod.GAMMA2;
modelStruct.d_modelPar_GAMMA2_dX=0.0;
modelStruct.modelPar_given_GAMMA2=mod.given("GAMMA2");
modParamMap["GAMMA2"] = &(modelStruct.d_modelPar_GAMMA2_dX);
modelStruct.modelPar_VBX=mod.VBX;
modelStruct.d_modelPar_VBX_dX=0.0;
modelStruct.modelPar_given_VBX=mod.given("VBX");
modParamMap["VBX"] = &(modelStruct.d_modelPar_VBX_dX);
modelStruct.modelPar_VBM=mod.VBM;
modelStruct.d_modelPar_VBM_dX=0.0;
modelStruct.modelPar_given_VBM=mod.given("VBM");
modParamMap["VBM"] = &(modelStruct.d_modelPar_VBM_dX);
modelStruct.modelPar_XT=mod.XT;
modelStruct.d_modelPar_XT_dX=0.0;
modelStruct.modelPar_given_XT=mod.given("XT");
modParamMap["XT"] = &(modelStruct.d_modelPar_XT_dX);
modelStruct.modelPar_K1=mod.K1;
modelStruct.d_modelPar_K1_dX=0.0;
modelStruct.modelPar_given_K1=mod.given("K1");
modParamMap["K1"] = &(modelStruct.d_modelPar_K1_dX);
modelStruct.modelPar_KT1=mod.KT1;
modelStruct.d_modelPar_KT1_dX=0.0;
modelStruct.modelPar_given_KT1=mod.given("KT1");
modParamMap["KT1"] = &(modelStruct.d_modelPar_KT1_dX);
modelStruct.modelPar_KT1L=mod.KT1L;
modelStruct.d_modelPar_KT1L_dX=0.0;
modelStruct.modelPar_given_KT1L=mod.given("KT1L");
modParamMap["KT1L"] = &(modelStruct.d_modelPar_KT1L_dX);
modelStruct.modelPar_KT2=mod.KT2;
modelStruct.d_modelPar_KT2_dX=0.0;
modelStruct.modelPar_given_KT2=mod.given("KT2");
modParamMap["KT2"] = &(modelStruct.d_modelPar_KT2_dX);
modelStruct.modelPar_K2=mod.K2;
modelStruct.d_modelPar_K2_dX=0.0;
modelStruct.modelPar_given_K2=mod.given("K2");
modParamMap["K2"] = &(modelStruct.d_modelPar_K2_dX);
modelStruct.modelPar_K3=mod.K3;
modelStruct.d_modelPar_K3_dX=0.0;
modelStruct.modelPar_given_K3=mod.given("K3");
modParamMap["K3"] = &(modelStruct.d_modelPar_K3_dX);
modelStruct.modelPar_K3B=mod.K3B;
modelStruct.d_modelPar_K3B_dX=0.0;
modelStruct.modelPar_given_K3B=mod.given("K3B");
modParamMap["K3B"] = &(modelStruct.d_modelPar_K3B_dX);
modelStruct.modelPar_W0=mod.W0;
modelStruct.d_modelPar_W0_dX=0.0;
modelStruct.modelPar_given_W0=mod.given("W0");
modParamMap["W0"] = &(modelStruct.d_modelPar_W0_dX);
modelStruct.modelPar_LPEB=mod.LPEB;
modelStruct.d_modelPar_LPEB_dX=0.0;
modelStruct.modelPar_given_LPEB=mod.given("LPEB");
modParamMap["LPEB"] = &(modelStruct.d_modelPar_LPEB_dX);
modelStruct.modelPar_DVT0=mod.DVT0;
modelStruct.d_modelPar_DVT0_dX=0.0;
modelStruct.modelPar_given_DVT0=mod.given("DVT0");
modParamMap["DVT0"] = &(modelStruct.d_modelPar_DVT0_dX);
modelStruct.modelPar_DVT1=mod.DVT1;
modelStruct.d_modelPar_DVT1_dX=0.0;
modelStruct.modelPar_given_DVT1=mod.given("DVT1");
modParamMap["DVT1"] = &(modelStruct.d_modelPar_DVT1_dX);
modelStruct.modelPar_DVT2=mod.DVT2;
modelStruct.d_modelPar_DVT2_dX=0.0;
modelStruct.modelPar_given_DVT2=mod.given("DVT2");
modParamMap["DVT2"] = &(modelStruct.d_modelPar_DVT2_dX);
modelStruct.modelPar_DVT0W=mod.DVT0W;
modelStruct.d_modelPar_DVT0W_dX=0.0;
modelStruct.modelPar_given_DVT0W=mod.given("DVT0W");
modParamMap["DVT0W"] = &(modelStruct.d_modelPar_DVT0W_dX);
modelStruct.modelPar_DVT1W=mod.DVT1W;
modelStruct.d_modelPar_DVT1W_dX=0.0;
modelStruct.modelPar_given_DVT1W=mod.given("DVT1W");
modParamMap["DVT1W"] = &(modelStruct.d_modelPar_DVT1W_dX);
modelStruct.modelPar_DVT2W=mod.DVT2W;
modelStruct.d_modelPar_DVT2W_dX=0.0;
modelStruct.modelPar_given_DVT2W=mod.given("DVT2W");
modParamMap["DVT2W"] = &(modelStruct.d_modelPar_DVT2W_dX);
modelStruct.modelPar_DROUT=mod.DROUT;
modelStruct.d_modelPar_DROUT_dX=0.0;
modelStruct.modelPar_given_DROUT=mod.given("DROUT");
modParamMap["DROUT"] = &(modelStruct.d_modelPar_DROUT_dX);
modelStruct.modelPar_DSUB=mod.DSUB;
modelStruct.d_modelPar_DSUB_dX=0.0;
modelStruct.modelPar_given_DSUB=mod.given("DSUB");
modParamMap["DSUB"] = &(modelStruct.d_modelPar_DSUB_dX);
modelStruct.modelPar_VTHO=mod.VTHO;
modelStruct.d_modelPar_VTHO_dX=0.0;
modelStruct.modelPar_given_VTHO=mod.given("VTHO");
modParamMap["VTHO"] = &(modelStruct.d_modelPar_VTHO_dX);
modelStruct.modelPar_VTH0=mod.VTH0;
modelStruct.d_modelPar_VTH0_dX=0.0;
modelStruct.modelPar_given_VTH0=mod.given("VTH0");
modParamMap["VTH0"] = &(modelStruct.d_modelPar_VTH0_dX);
modelStruct.modelPar_VFB=mod.VFB;
modelStruct.d_modelPar_VFB_dX=0.0;
modelStruct.modelPar_given_VFB=mod.given("VFB");
modParamMap["VFB"] = &(modelStruct.d_modelPar_VFB_dX);
modelStruct.modelPar_UA=mod.UA;
modelStruct.d_modelPar_UA_dX=0.0;
modelStruct.modelPar_given_UA=mod.given("UA");
modParamMap["UA"] = &(modelStruct.d_modelPar_UA_dX);
modelStruct.modelPar_UA1=mod.UA1;
modelStruct.d_modelPar_UA1_dX=0.0;
modelStruct.modelPar_given_UA1=mod.given("UA1");
modParamMap["UA1"] = &(modelStruct.d_modelPar_UA1_dX);
modelStruct.modelPar_UB=mod.UB;
modelStruct.d_modelPar_UB_dX=0.0;
modelStruct.modelPar_given_UB=mod.given("UB");
modParamMap["UB"] = &(modelStruct.d_modelPar_UB_dX);
modelStruct.modelPar_UB1=mod.UB1;
modelStruct.d_modelPar_UB1_dX=0.0;
modelStruct.modelPar_given_UB1=mod.given("UB1");
modParamMap["UB1"] = &(modelStruct.d_modelPar_UB1_dX);
modelStruct.modelPar_UC=mod.UC;
modelStruct.d_modelPar_UC_dX=0.0;
modelStruct.modelPar_given_UC=mod.given("UC");
modParamMap["UC"] = &(modelStruct.d_modelPar_UC_dX);
modelStruct.modelPar_UC1=mod.UC1;
modelStruct.d_modelPar_UC1_dX=0.0;
modelStruct.modelPar_given_UC1=mod.given("UC1");
modParamMap["UC1"] = &(modelStruct.d_modelPar_UC1_dX);
modelStruct.modelPar_U0=mod.U0;
modelStruct.d_modelPar_U0_dX=0.0;
modelStruct.modelPar_given_U0=mod.given("U0");
modParamMap["U0"] = &(modelStruct.d_modelPar_U0_dX);
modelStruct.modelPar_EU=mod.EU;
modelStruct.d_modelPar_EU_dX=0.0;
modelStruct.modelPar_given_EU=mod.given("EU");
modParamMap["EU"] = &(modelStruct.d_modelPar_EU_dX);
modelStruct.modelPar_UTE=mod.UTE;
modelStruct.d_modelPar_UTE_dX=0.0;
modelStruct.modelPar_given_UTE=mod.given("UTE");
modParamMap["UTE"] = &(modelStruct.d_modelPar_UTE_dX);
modelStruct.modelPar_UCS=mod.UCS;
modelStruct.d_modelPar_UCS_dX=0.0;
modelStruct.modelPar_given_UCS=mod.given("UCS");
modParamMap["UCS"] = &(modelStruct.d_modelPar_UCS_dX);
modelStruct.modelPar_UCSTE=mod.UCSTE;
modelStruct.d_modelPar_UCSTE_dX=0.0;
modelStruct.modelPar_given_UCSTE=mod.given("UCSTE");
modParamMap["UCSTE"] = &(modelStruct.d_modelPar_UCSTE_dX);
modelStruct.modelPar_UD=mod.UD;
modelStruct.d_modelPar_UD_dX=0.0;
modelStruct.modelPar_given_UD=mod.given("UD");
modParamMap["UD"] = &(modelStruct.d_modelPar_UD_dX);
modelStruct.modelPar_UD1=mod.UD1;
modelStruct.d_modelPar_UD1_dX=0.0;
modelStruct.modelPar_given_UD1=mod.given("UD1");
modParamMap["UD1"] = &(modelStruct.d_modelPar_UD1_dX);
modelStruct.modelPar_VOFF=mod.VOFF;
modelStruct.d_modelPar_VOFF_dX=0.0;
modelStruct.modelPar_given_VOFF=mod.given("VOFF");
modParamMap["VOFF"] = &(modelStruct.d_modelPar_VOFF_dX);
modelStruct.modelPar_TNOM=mod.TNOM;
modelStruct.d_modelPar_TNOM_dX=0.0;
modelStruct.modelPar_given_TNOM=mod.given("TNOM");
modParamMap["TNOM"] = &(modelStruct.d_modelPar_TNOM_dX);
modelStruct.modelPar_CGSO=mod.CGSO;
modelStruct.d_modelPar_CGSO_dX=0.0;
modelStruct.modelPar_given_CGSO=mod.given("CGSO");
modParamMap["CGSO"] = &(modelStruct.d_modelPar_CGSO_dX);
modelStruct.modelPar_CGDO=mod.CGDO;
modelStruct.d_modelPar_CGDO_dX=0.0;
modelStruct.modelPar_given_CGDO=mod.given("CGDO");
modParamMap["CGDO"] = &(modelStruct.d_modelPar_CGDO_dX);
modelStruct.modelPar_XPART=mod.XPART;
modelStruct.d_modelPar_XPART_dX=0.0;
modelStruct.modelPar_given_XPART=mod.given("XPART");
modParamMap["XPART"] = &(modelStruct.d_modelPar_XPART_dX);
modelStruct.modelPar_DELTA=mod.DELTA;
modelStruct.d_modelPar_DELTA_dX=0.0;
modelStruct.modelPar_given_DELTA=mod.given("DELTA");
modParamMap["DELTA"] = &(modelStruct.d_modelPar_DELTA_dX);
modelStruct.modelPar_RSH=mod.RSH;
modelStruct.d_modelPar_RSH_dX=0.0;
modelStruct.modelPar_given_RSH=mod.given("RSH");
modParamMap["RSH"] = &(modelStruct.d_modelPar_RSH_dX);
modelStruct.modelPar_RDSW=mod.RDSW;
modelStruct.d_modelPar_RDSW_dX=0.0;
modelStruct.modelPar_given_RDSW=mod.given("RDSW");
modParamMap["RDSW"] = &(modelStruct.d_modelPar_RDSW_dX);
modelStruct.modelPar_RSW=mod.RSW;
modelStruct.d_modelPar_RSW_dX=0.0;
modelStruct.modelPar_given_RSW=mod.given("RSW");
modParamMap["RSW"] = &(modelStruct.d_modelPar_RSW_dX);
modelStruct.modelPar_RDW=mod.RDW;
modelStruct.d_modelPar_RDW_dX=0.0;
modelStruct.modelPar_given_RDW=mod.given("RDW");
modParamMap["RDW"] = &(modelStruct.d_modelPar_RDW_dX);
modelStruct.modelPar_RSWMIN=mod.RSWMIN;
modelStruct.d_modelPar_RSWMIN_dX=0.0;
modelStruct.modelPar_given_RSWMIN=mod.given("RSWMIN");
modParamMap["RSWMIN"] = &(modelStruct.d_modelPar_RSWMIN_dX);
modelStruct.modelPar_RDWMIN=mod.RDWMIN;
modelStruct.d_modelPar_RDWMIN_dX=0.0;
modelStruct.modelPar_given_RDWMIN=mod.given("RDWMIN");
modParamMap["RDWMIN"] = &(modelStruct.d_modelPar_RDWMIN_dX);
modelStruct.modelPar_PRWG=mod.PRWG;
modelStruct.d_modelPar_PRWG_dX=0.0;
modelStruct.modelPar_given_PRWG=mod.given("PRWG");
modParamMap["PRWG"] = &(modelStruct.d_modelPar_PRWG_dX);
modelStruct.modelPar_PRWB=mod.PRWB;
modelStruct.d_modelPar_PRWB_dX=0.0;
modelStruct.modelPar_given_PRWB=mod.given("PRWB");
modParamMap["PRWB"] = &(modelStruct.d_modelPar_PRWB_dX);
modelStruct.modelPar_PRT=mod.PRT;
modelStruct.d_modelPar_PRT_dX=0.0;
modelStruct.modelPar_given_PRT=mod.given("PRT");
modParamMap["PRT"] = &(modelStruct.d_modelPar_PRT_dX);
modelStruct.modelPar_ETA0=mod.ETA0;
modelStruct.d_modelPar_ETA0_dX=0.0;
modelStruct.modelPar_given_ETA0=mod.given("ETA0");
modParamMap["ETA0"] = &(modelStruct.d_modelPar_ETA0_dX);
modelStruct.modelPar_ETAB=mod.ETAB;
modelStruct.d_modelPar_ETAB_dX=0.0;
modelStruct.modelPar_given_ETAB=mod.given("ETAB");
modParamMap["ETAB"] = &(modelStruct.d_modelPar_ETAB_dX);
modelStruct.modelPar_ETA0CV=mod.ETA0CV;
modelStruct.d_modelPar_ETA0CV_dX=0.0;
modelStruct.modelPar_given_ETA0CV=mod.given("ETA0CV");
modParamMap["ETA0CV"] = &(modelStruct.d_modelPar_ETA0CV_dX);
modelStruct.modelPar_ETABCV=mod.ETABCV;
modelStruct.d_modelPar_ETABCV_dX=0.0;
modelStruct.modelPar_given_ETABCV=mod.given("ETABCV");
modParamMap["ETABCV"] = &(modelStruct.d_modelPar_ETABCV_dX);
modelStruct.modelPar_PCLM=mod.PCLM;
modelStruct.d_modelPar_PCLM_dX=0.0;
modelStruct.modelPar_given_PCLM=mod.given("PCLM");
modParamMap["PCLM"] = &(modelStruct.d_modelPar_PCLM_dX);
modelStruct.modelPar_PDIBLC1=mod.PDIBLC1;
modelStruct.d_modelPar_PDIBLC1_dX=0.0;
modelStruct.modelPar_given_PDIBLC1=mod.given("PDIBLC1");
modParamMap["PDIBLC1"] = &(modelStruct.d_modelPar_PDIBLC1_dX);
modelStruct.modelPar_PDIBLC2=mod.PDIBLC2;
modelStruct.d_modelPar_PDIBLC2_dX=0.0;
modelStruct.modelPar_given_PDIBLC2=mod.given("PDIBLC2");
modParamMap["PDIBLC2"] = &(modelStruct.d_modelPar_PDIBLC2_dX);
modelStruct.modelPar_PDIBLCB=mod.PDIBLCB;
modelStruct.d_modelPar_PDIBLCB_dX=0.0;
modelStruct.modelPar_given_PDIBLCB=mod.given("PDIBLCB");
modParamMap["PDIBLCB"] = &(modelStruct.d_modelPar_PDIBLCB_dX);
modelStruct.modelPar_PVAG=mod.PVAG;
modelStruct.d_modelPar_PVAG_dX=0.0;
modelStruct.modelPar_given_PVAG=mod.given("PVAG");
modParamMap["PVAG"] = &(modelStruct.d_modelPar_PVAG_dX);
modelStruct.modelPar_TBOX=mod.TBOX;
modelStruct.d_modelPar_TBOX_dX=0.0;
modelStruct.modelPar_given_TBOX=mod.given("TBOX");
modParamMap["TBOX"] = &(modelStruct.d_modelPar_TBOX_dX);
modelStruct.modelPar_TSI=mod.TSI;
modelStruct.d_modelPar_TSI_dX=0.0;
modelStruct.modelPar_given_TSI=mod.given("TSI");
modParamMap["TSI"] = &(modelStruct.d_modelPar_TSI_dX);
modelStruct.modelPar_ETSI=mod.ETSI;
modelStruct.d_modelPar_ETSI_dX=0.0;
modelStruct.modelPar_given_ETSI=mod.given("ETSI");
modParamMap["ETSI"] = &(modelStruct.d_modelPar_ETSI_dX);
modelStruct.modelPar_XJ=mod.XJ;
modelStruct.d_modelPar_XJ_dX=0.0;
modelStruct.modelPar_given_XJ=mod.given("XJ");
modParamMap["XJ"] = &(modelStruct.d_modelPar_XJ_dX);
modelStruct.modelPar_AGIDL=mod.AGIDL;
modelStruct.d_modelPar_AGIDL_dX=0.0;
modelStruct.modelPar_given_AGIDL=mod.given("AGIDL");
modParamMap["AGIDL"] = &(modelStruct.d_modelPar_AGIDL_dX);
modelStruct.modelPar_BGIDL=mod.BGIDL;
modelStruct.d_modelPar_BGIDL_dX=0.0;
modelStruct.modelPar_given_BGIDL=mod.given("BGIDL");
modParamMap["BGIDL"] = &(modelStruct.d_modelPar_BGIDL_dX);
modelStruct.modelPar_CGIDL=mod.CGIDL;
modelStruct.d_modelPar_CGIDL_dX=0.0;
modelStruct.modelPar_given_CGIDL=mod.given("CGIDL");
modParamMap["CGIDL"] = &(modelStruct.d_modelPar_CGIDL_dX);
modelStruct.modelPar_RGIDL=mod.RGIDL;
modelStruct.d_modelPar_RGIDL_dX=0.0;
modelStruct.modelPar_given_RGIDL=mod.given("RGIDL");
modParamMap["RGIDL"] = &(modelStruct.d_modelPar_RGIDL_dX);
modelStruct.modelPar_KGIDL=mod.KGIDL;
modelStruct.d_modelPar_KGIDL_dX=0.0;
modelStruct.modelPar_given_KGIDL=mod.given("KGIDL");
modParamMap["KGIDL"] = &(modelStruct.d_modelPar_KGIDL_dX);
modelStruct.modelPar_FGIDL=mod.FGIDL;
modelStruct.d_modelPar_FGIDL_dX=0.0;
modelStruct.modelPar_given_FGIDL=mod.given("FGIDL");
modParamMap["FGIDL"] = &(modelStruct.d_modelPar_FGIDL_dX);
modelStruct.modelPar_AGISL=mod.AGISL;
modelStruct.d_modelPar_AGISL_dX=0.0;
modelStruct.modelPar_given_AGISL=mod.given("AGISL");
modParamMap["AGISL"] = &(modelStruct.d_modelPar_AGISL_dX);
modelStruct.modelPar_BGISL=mod.BGISL;
modelStruct.d_modelPar_BGISL_dX=0.0;
modelStruct.modelPar_given_BGISL=mod.given("BGISL");
modParamMap["BGISL"] = &(modelStruct.d_modelPar_BGISL_dX);
modelStruct.modelPar_CGISL=mod.CGISL;
modelStruct.d_modelPar_CGISL_dX=0.0;
modelStruct.modelPar_given_CGISL=mod.given("CGISL");
modParamMap["CGISL"] = &(modelStruct.d_modelPar_CGISL_dX);
modelStruct.modelPar_RGISL=mod.RGISL;
modelStruct.d_modelPar_RGISL_dX=0.0;
modelStruct.modelPar_given_RGISL=mod.given("RGISL");
modParamMap["RGISL"] = &(modelStruct.d_modelPar_RGISL_dX);
modelStruct.modelPar_KGISL=mod.KGISL;
modelStruct.d_modelPar_KGISL_dX=0.0;
modelStruct.modelPar_given_KGISL=mod.given("KGISL");
modParamMap["KGISL"] = &(modelStruct.d_modelPar_KGISL_dX);
modelStruct.modelPar_FGISL=mod.FGISL;
modelStruct.d_modelPar_FGISL_dX=0.0;
modelStruct.modelPar_given_FGISL=mod.given("FGISL");
modParamMap["FGISL"] = &(modelStruct.d_modelPar_FGISL_dX);
modelStruct.modelPar_NDIODE=mod.NDIODE;
modelStruct.d_modelPar_NDIODE_dX=0.0;
modelStruct.modelPar_given_NDIODE=mod.given("NDIODE");
modParamMap["NDIODE"] = &(modelStruct.d_modelPar_NDIODE_dX);
modelStruct.modelPar_NDIODED=mod.NDIODED;
modelStruct.d_modelPar_NDIODED_dX=0.0;
modelStruct.modelPar_given_NDIODED=mod.given("NDIODED");
modParamMap["NDIODED"] = &(modelStruct.d_modelPar_NDIODED_dX);
modelStruct.modelPar_XBJT=mod.XBJT;
modelStruct.d_modelPar_XBJT_dX=0.0;
modelStruct.modelPar_given_XBJT=mod.given("XBJT");
modParamMap["XBJT"] = &(modelStruct.d_modelPar_XBJT_dX);
modelStruct.modelPar_XDIF=mod.XDIF;
modelStruct.d_modelPar_XDIF_dX=0.0;
modelStruct.modelPar_given_XDIF=mod.given("XDIF");
modParamMap["XDIF"] = &(modelStruct.d_modelPar_XDIF_dX);
modelStruct.modelPar_XREC=mod.XREC;
modelStruct.d_modelPar_XREC_dX=0.0;
modelStruct.modelPar_given_XREC=mod.given("XREC");
modParamMap["XREC"] = &(modelStruct.d_modelPar_XREC_dX);
modelStruct.modelPar_XTUN=mod.XTUN;
modelStruct.d_modelPar_XTUN_dX=0.0;
modelStruct.modelPar_given_XTUN=mod.given("XTUN");
modParamMap["XTUN"] = &(modelStruct.d_modelPar_XTUN_dX);
modelStruct.modelPar_XDIFD=mod.XDIFD;
modelStruct.d_modelPar_XDIFD_dX=0.0;
modelStruct.modelPar_given_XDIFD=mod.given("XDIFD");
modParamMap["XDIFD"] = &(modelStruct.d_modelPar_XDIFD_dX);
modelStruct.modelPar_XRECD=mod.XRECD;
modelStruct.d_modelPar_XRECD_dX=0.0;
modelStruct.modelPar_given_XRECD=mod.given("XRECD");
modParamMap["XRECD"] = &(modelStruct.d_modelPar_XRECD_dX);
modelStruct.modelPar_XTUND=mod.XTUND;
modelStruct.d_modelPar_XTUND_dX=0.0;
modelStruct.modelPar_given_XTUND=mod.given("XTUND");
modParamMap["XTUND"] = &(modelStruct.d_modelPar_XTUND_dX);
modelStruct.modelPar_PBSWG=mod.PBSWG;
modelStruct.d_modelPar_PBSWG_dX=0.0;
modelStruct.modelPar_given_PBSWG=mod.given("PBSWG");
modParamMap["PBSWG"] = &(modelStruct.d_modelPar_PBSWG_dX);
modelStruct.modelPar_PBSWGD=mod.PBSWGD;
modelStruct.d_modelPar_PBSWGD_dX=0.0;
modelStruct.modelPar_given_PBSWGD=mod.given("PBSWGD");
modParamMap["PBSWGD"] = &(modelStruct.d_modelPar_PBSWGD_dX);
modelStruct.modelPar_MJSWG=mod.MJSWG;
modelStruct.d_modelPar_MJSWG_dX=0.0;
modelStruct.modelPar_given_MJSWG=mod.given("MJSWG");
modParamMap["MJSWG"] = &(modelStruct.d_modelPar_MJSWG_dX);
modelStruct.modelPar_MJSWGD=mod.MJSWGD;
modelStruct.d_modelPar_MJSWGD_dX=0.0;
modelStruct.modelPar_given_MJSWGD=mod.given("MJSWGD");
modParamMap["MJSWGD"] = &(modelStruct.d_modelPar_MJSWGD_dX);
modelStruct.modelPar_CJSWG=mod.CJSWG;
modelStruct.d_modelPar_CJSWG_dX=0.0;
modelStruct.modelPar_given_CJSWG=mod.given("CJSWG");
modParamMap["CJSWG"] = &(modelStruct.d_modelPar_CJSWG_dX);
modelStruct.modelPar_CJSWGD=mod.CJSWGD;
modelStruct.d_modelPar_CJSWGD_dX=0.0;
modelStruct.modelPar_given_CJSWGD=mod.given("CJSWGD");
modParamMap["CJSWGD"] = &(modelStruct.d_modelPar_CJSWGD_dX);
modelStruct.modelPar_LINT=mod.LINT;
modelStruct.d_modelPar_LINT_dX=0.0;
modelStruct.modelPar_given_LINT=mod.given("LINT");
modParamMap["LINT"] = &(modelStruct.d_modelPar_LINT_dX);
modelStruct.modelPar_LL=mod.LL;
modelStruct.d_modelPar_LL_dX=0.0;
modelStruct.modelPar_given_LL=mod.given("LL");
modParamMap["LL"] = &(modelStruct.d_modelPar_LL_dX);
modelStruct.modelPar_LLC=mod.LLC;
modelStruct.d_modelPar_LLC_dX=0.0;
modelStruct.modelPar_given_LLC=mod.given("LLC");
modParamMap["LLC"] = &(modelStruct.d_modelPar_LLC_dX);
modelStruct.modelPar_LLN=mod.LLN;
modelStruct.d_modelPar_LLN_dX=0.0;
modelStruct.modelPar_given_LLN=mod.given("LLN");
modParamMap["LLN"] = &(modelStruct.d_modelPar_LLN_dX);
modelStruct.modelPar_LW=mod.LW;
modelStruct.d_modelPar_LW_dX=0.0;
modelStruct.modelPar_given_LW=mod.given("LW");
modParamMap["LW"] = &(modelStruct.d_modelPar_LW_dX);
modelStruct.modelPar_LWC=mod.LWC;
modelStruct.d_modelPar_LWC_dX=0.0;
modelStruct.modelPar_given_LWC=mod.given("LWC");
modParamMap["LWC"] = &(modelStruct.d_modelPar_LWC_dX);
modelStruct.modelPar_LWN=mod.LWN;
modelStruct.d_modelPar_LWN_dX=0.0;
modelStruct.modelPar_given_LWN=mod.given("LWN");
modParamMap["LWN"] = &(modelStruct.d_modelPar_LWN_dX);
modelStruct.modelPar_LWL=mod.LWL;
modelStruct.d_modelPar_LWL_dX=0.0;
modelStruct.modelPar_given_LWL=mod.given("LWL");
modParamMap["LWL"] = &(modelStruct.d_modelPar_LWL_dX);
modelStruct.modelPar_LWLC=mod.LWLC;
modelStruct.d_modelPar_LWLC_dX=0.0;
modelStruct.modelPar_given_LWLC=mod.given("LWLC");
modParamMap["LWLC"] = &(modelStruct.d_modelPar_LWLC_dX);
modelStruct.modelPar_WR=mod.WR;
modelStruct.d_modelPar_WR_dX=0.0;
modelStruct.modelPar_given_WR=mod.given("WR");
modParamMap["WR"] = &(modelStruct.d_modelPar_WR_dX);
modelStruct.modelPar_WINT=mod.WINT;
modelStruct.d_modelPar_WINT_dX=0.0;
modelStruct.modelPar_given_WINT=mod.given("WINT");
modParamMap["WINT"] = &(modelStruct.d_modelPar_WINT_dX);
modelStruct.modelPar_DWG=mod.DWG;
modelStruct.d_modelPar_DWG_dX=0.0;
modelStruct.modelPar_given_DWG=mod.given("DWG");
modParamMap["DWG"] = &(modelStruct.d_modelPar_DWG_dX);
modelStruct.modelPar_DWB=mod.DWB;
modelStruct.d_modelPar_DWB_dX=0.0;
modelStruct.modelPar_given_DWB=mod.given("DWB");
modParamMap["DWB"] = &(modelStruct.d_modelPar_DWB_dX);
modelStruct.modelPar_WL=mod.WL;
modelStruct.d_modelPar_WL_dX=0.0;
modelStruct.modelPar_given_WL=mod.given("WL");
modParamMap["WL"] = &(modelStruct.d_modelPar_WL_dX);
modelStruct.modelPar_WLC=mod.WLC;
modelStruct.d_modelPar_WLC_dX=0.0;
modelStruct.modelPar_given_WLC=mod.given("WLC");
modParamMap["WLC"] = &(modelStruct.d_modelPar_WLC_dX);
modelStruct.modelPar_WLN=mod.WLN;
modelStruct.d_modelPar_WLN_dX=0.0;
modelStruct.modelPar_given_WLN=mod.given("WLN");
modParamMap["WLN"] = &(modelStruct.d_modelPar_WLN_dX);
modelStruct.modelPar_WW=mod.WW;
modelStruct.d_modelPar_WW_dX=0.0;
modelStruct.modelPar_given_WW=mod.given("WW");
modParamMap["WW"] = &(modelStruct.d_modelPar_WW_dX);
modelStruct.modelPar_WWC=mod.WWC;
modelStruct.d_modelPar_WWC_dX=0.0;
modelStruct.modelPar_given_WWC=mod.given("WWC");
modParamMap["WWC"] = &(modelStruct.d_modelPar_WWC_dX);
modelStruct.modelPar_WWN=mod.WWN;
modelStruct.d_modelPar_WWN_dX=0.0;
modelStruct.modelPar_given_WWN=mod.given("WWN");
modParamMap["WWN"] = &(modelStruct.d_modelPar_WWN_dX);
modelStruct.modelPar_WWL=mod.WWL;
modelStruct.d_modelPar_WWL_dX=0.0;
modelStruct.modelPar_given_WWL=mod.given("WWL");
modParamMap["WWL"] = &(modelStruct.d_modelPar_WWL_dX);
modelStruct.modelPar_WWLC=mod.WWLC;
modelStruct.d_modelPar_WWLC_dX=0.0;
modelStruct.modelPar_given_WWLC=mod.given("WWLC");
modParamMap["WWLC"] = &(modelStruct.d_modelPar_WWLC_dX);
modelStruct.modelPar_B0=mod.B0;
modelStruct.d_modelPar_B0_dX=0.0;
modelStruct.modelPar_given_B0=mod.given("B0");
modParamMap["B0"] = &(modelStruct.d_modelPar_B0_dX);
modelStruct.modelPar_B1=mod.B1;
modelStruct.d_modelPar_B1_dX=0.0;
modelStruct.modelPar_given_B1=mod.given("B1");
modParamMap["B1"] = &(modelStruct.d_modelPar_B1_dX);
modelStruct.modelPar_CGSL=mod.CGSL;
modelStruct.d_modelPar_CGSL_dX=0.0;
modelStruct.modelPar_given_CGSL=mod.given("CGSL");
modParamMap["CGSL"] = &(modelStruct.d_modelPar_CGSL_dX);
modelStruct.modelPar_CGDL=mod.CGDL;
modelStruct.d_modelPar_CGDL_dX=0.0;
modelStruct.modelPar_given_CGDL=mod.given("CGDL");
modParamMap["CGDL"] = &(modelStruct.d_modelPar_CGDL_dX);
modelStruct.modelPar_CKAPPA=mod.CKAPPA;
modelStruct.d_modelPar_CKAPPA_dX=0.0;
modelStruct.modelPar_given_CKAPPA=mod.given("CKAPPA");
modParamMap["CKAPPA"] = &(modelStruct.d_modelPar_CKAPPA_dX);
modelStruct.modelPar_CF=mod.CF;
modelStruct.d_modelPar_CF_dX=0.0;
modelStruct.modelPar_given_CF=mod.given("CF");
modParamMap["CF"] = &(modelStruct.d_modelPar_CF_dX);
modelStruct.modelPar_CLC=mod.CLC;
modelStruct.d_modelPar_CLC_dX=0.0;
modelStruct.modelPar_given_CLC=mod.given("CLC");
modParamMap["CLC"] = &(modelStruct.d_modelPar_CLC_dX);
modelStruct.modelPar_CLE=mod.CLE;
modelStruct.d_modelPar_CLE_dX=0.0;
modelStruct.modelPar_given_CLE=mod.given("CLE");
modParamMap["CLE"] = &(modelStruct.d_modelPar_CLE_dX);
modelStruct.modelPar_DWC=mod.DWC;
modelStruct.d_modelPar_DWC_dX=0.0;
modelStruct.modelPar_given_DWC=mod.given("DWC");
modParamMap["DWC"] = &(modelStruct.d_modelPar_DWC_dX);
modelStruct.modelPar_DLC=mod.DLC;
modelStruct.d_modelPar_DLC_dX=0.0;
modelStruct.modelPar_given_DLC=mod.given("DLC");
modParamMap["DLC"] = &(modelStruct.d_modelPar_DLC_dX);
modelStruct.modelPar_ALPHA0=mod.ALPHA0;
modelStruct.d_modelPar_ALPHA0_dX=0.0;
modelStruct.modelPar_given_ALPHA0=mod.given("ALPHA0");
modParamMap["ALPHA0"] = &(modelStruct.d_modelPar_ALPHA0_dX);
modelStruct.modelPar_NOIA=mod.NOIA;
modelStruct.d_modelPar_NOIA_dX=0.0;
modelStruct.modelPar_given_NOIA=mod.given("NOIA");
modParamMap["NOIA"] = &(modelStruct.d_modelPar_NOIA_dX);
modelStruct.modelPar_NOIB=mod.NOIB;
modelStruct.d_modelPar_NOIB_dX=0.0;
modelStruct.modelPar_given_NOIB=mod.given("NOIB");
modParamMap["NOIB"] = &(modelStruct.d_modelPar_NOIB_dX);
modelStruct.modelPar_NOIC=mod.NOIC;
modelStruct.d_modelPar_NOIC_dX=0.0;
modelStruct.modelPar_given_NOIC=mod.given("NOIC");
modParamMap["NOIC"] = &(modelStruct.d_modelPar_NOIC_dX);
modelStruct.modelPar_TNOIA=mod.TNOIA;
modelStruct.d_modelPar_TNOIA_dX=0.0;
modelStruct.modelPar_given_TNOIA=mod.given("TNOIA");
modParamMap["TNOIA"] = &(modelStruct.d_modelPar_TNOIA_dX);
modelStruct.modelPar_TNOIB=mod.TNOIB;
modelStruct.d_modelPar_TNOIB_dX=0.0;
modelStruct.modelPar_given_TNOIB=mod.given("TNOIB");
modParamMap["TNOIB"] = &(modelStruct.d_modelPar_TNOIB_dX);
modelStruct.modelPar_RNOIA=mod.RNOIA;
modelStruct.d_modelPar_RNOIA_dX=0.0;
modelStruct.modelPar_given_RNOIA=mod.given("RNOIA");
modParamMap["RNOIA"] = &(modelStruct.d_modelPar_RNOIA_dX);
modelStruct.modelPar_RNOIB=mod.RNOIB;
modelStruct.d_modelPar_RNOIB_dX=0.0;
modelStruct.modelPar_given_RNOIB=mod.given("RNOIB");
modParamMap["RNOIB"] = &(modelStruct.d_modelPar_RNOIB_dX);
modelStruct.modelPar_NTNOI=mod.NTNOI;
modelStruct.d_modelPar_NTNOI_dX=0.0;
modelStruct.modelPar_given_NTNOI=mod.given("NTNOI");
modParamMap["NTNOI"] = &(modelStruct.d_modelPar_NTNOI_dX);
modelStruct.modelPar_SAREF=mod.SAREF;
modelStruct.d_modelPar_SAREF_dX=0.0;
modelStruct.modelPar_given_SAREF=mod.given("SAREF");
modParamMap["SAREF"] = &(modelStruct.d_modelPar_SAREF_dX);
modelStruct.modelPar_SBREF=mod.SBREF;
modelStruct.d_modelPar_SBREF_dX=0.0;
modelStruct.modelPar_given_SBREF=mod.given("SBREF");
modParamMap["SBREF"] = &(modelStruct.d_modelPar_SBREF_dX);
modelStruct.modelPar_WLOD=mod.WLOD;
modelStruct.d_modelPar_WLOD_dX=0.0;
modelStruct.modelPar_given_WLOD=mod.given("WLOD");
modParamMap["WLOD"] = &(modelStruct.d_modelPar_WLOD_dX);
modelStruct.modelPar_KU0=mod.KU0;
modelStruct.d_modelPar_KU0_dX=0.0;
modelStruct.modelPar_given_KU0=mod.given("KU0");
modParamMap["KU0"] = &(modelStruct.d_modelPar_KU0_dX);
modelStruct.modelPar_KVSAT=mod.KVSAT;
modelStruct.d_modelPar_KVSAT_dX=0.0;
modelStruct.modelPar_given_KVSAT=mod.given("KVSAT");
modParamMap["KVSAT"] = &(modelStruct.d_modelPar_KVSAT_dX);
modelStruct.modelPar_KVTH0=mod.KVTH0;
modelStruct.d_modelPar_KVTH0_dX=0.0;
modelStruct.modelPar_given_KVTH0=mod.given("KVTH0");
modParamMap["KVTH0"] = &(modelStruct.d_modelPar_KVTH0_dX);
modelStruct.modelPar_TKU0=mod.TKU0;
modelStruct.d_modelPar_TKU0_dX=0.0;
modelStruct.modelPar_given_TKU0=mod.given("TKU0");
modParamMap["TKU0"] = &(modelStruct.d_modelPar_TKU0_dX);
modelStruct.modelPar_LLODKU0=mod.LLODKU0;
modelStruct.d_modelPar_LLODKU0_dX=0.0;
modelStruct.modelPar_given_LLODKU0=mod.given("LLODKU0");
modParamMap["LLODKU0"] = &(modelStruct.d_modelPar_LLODKU0_dX);
modelStruct.modelPar_WLODKU0=mod.WLODKU0;
modelStruct.d_modelPar_WLODKU0_dX=0.0;
modelStruct.modelPar_given_WLODKU0=mod.given("WLODKU0");
modParamMap["WLODKU0"] = &(modelStruct.d_modelPar_WLODKU0_dX);
modelStruct.modelPar_LLODVTH=mod.LLODVTH;
modelStruct.d_modelPar_LLODVTH_dX=0.0;
modelStruct.modelPar_given_LLODVTH=mod.given("LLODVTH");
modParamMap["LLODVTH"] = &(modelStruct.d_modelPar_LLODVTH_dX);
modelStruct.modelPar_WLODVTH=mod.WLODVTH;
modelStruct.d_modelPar_WLODVTH_dX=0.0;
modelStruct.modelPar_given_WLODVTH=mod.given("WLODVTH");
modParamMap["WLODVTH"] = &(modelStruct.d_modelPar_WLODVTH_dX);
modelStruct.modelPar_LKU0=mod.LKU0;
modelStruct.d_modelPar_LKU0_dX=0.0;
modelStruct.modelPar_given_LKU0=mod.given("LKU0");
modParamMap["LKU0"] = &(modelStruct.d_modelPar_LKU0_dX);
modelStruct.modelPar_WKU0=mod.WKU0;
modelStruct.d_modelPar_WKU0_dX=0.0;
modelStruct.modelPar_given_WKU0=mod.given("WKU0");
modParamMap["WKU0"] = &(modelStruct.d_modelPar_WKU0_dX);
modelStruct.modelPar_PKU0=mod.PKU0;
modelStruct.d_modelPar_PKU0_dX=0.0;
modelStruct.modelPar_given_PKU0=mod.given("PKU0");
modParamMap["PKU0"] = &(modelStruct.d_modelPar_PKU0_dX);
modelStruct.modelPar_LKVTH0=mod.LKVTH0;
modelStruct.d_modelPar_LKVTH0_dX=0.0;
modelStruct.modelPar_given_LKVTH0=mod.given("LKVTH0");
modParamMap["LKVTH0"] = &(modelStruct.d_modelPar_LKVTH0_dX);
modelStruct.modelPar_WKVTH0=mod.WKVTH0;
modelStruct.d_modelPar_WKVTH0_dX=0.0;
modelStruct.modelPar_given_WKVTH0=mod.given("WKVTH0");
modParamMap["WKVTH0"] = &(modelStruct.d_modelPar_WKVTH0_dX);
modelStruct.modelPar_PKVTH0=mod.PKVTH0;
modelStruct.d_modelPar_PKVTH0_dX=0.0;
modelStruct.modelPar_given_PKVTH0=mod.given("PKVTH0");
modParamMap["PKVTH0"] = &(modelStruct.d_modelPar_PKVTH0_dX);
modelStruct.modelPar_STK2=mod.STK2;
modelStruct.d_modelPar_STK2_dX=0.0;
modelStruct.modelPar_given_STK2=mod.given("STK2");
modParamMap["STK2"] = &(modelStruct.d_modelPar_STK2_dX);
modelStruct.modelPar_LODK2=mod.LODK2;
modelStruct.d_modelPar_LODK2_dX=0.0;
modelStruct.modelPar_given_LODK2=mod.given("LODK2");
modParamMap["LODK2"] = &(modelStruct.d_modelPar_LODK2_dX);
modelStruct.modelPar_STETA0=mod.STETA0;
modelStruct.d_modelPar_STETA0_dX=0.0;
modelStruct.modelPar_given_STETA0=mod.given("STETA0");
modParamMap["STETA0"] = &(modelStruct.d_modelPar_STETA0_dX);
modelStruct.modelPar_LODETA0=mod.LODETA0;
modelStruct.d_modelPar_LODETA0_dX=0.0;
modelStruct.modelPar_given_LODETA0=mod.given("LODETA0");
modParamMap["LODETA0"] = &(modelStruct.d_modelPar_LODETA0_dX);
modelStruct.modelPar_STETA0CV=mod.STETA0CV;
modelStruct.d_modelPar_STETA0CV_dX=0.0;
modelStruct.modelPar_given_STETA0CV=mod.given("STETA0CV");
modParamMap["STETA0CV"] = &(modelStruct.d_modelPar_STETA0CV_dX);
modelStruct.modelPar_LODETA0CV=mod.LODETA0CV;
modelStruct.d_modelPar_LODETA0CV_dX=0.0;
modelStruct.modelPar_given_LODETA0CV=mod.given("LODETA0CV");
modParamMap["LODETA0CV"] = &(modelStruct.d_modelPar_LODETA0CV_dX);
modelStruct.modelPar_GBMIN=mod.GBMIN;
modelStruct.d_modelPar_GBMIN_dX=0.0;
modelStruct.modelPar_given_GBMIN=mod.given("GBMIN");
modParamMap["GBMIN"] = &(modelStruct.d_modelPar_GBMIN_dX);
modelStruct.modelPar_BF=mod.BF;
modelStruct.d_modelPar_BF_dX=0.0;
modelStruct.modelPar_given_BF=mod.given("BF");
modParamMap["BF"] = &(modelStruct.d_modelPar_BF_dX);
modelStruct.modelPar_W0FLK=mod.W0FLK;
modelStruct.d_modelPar_W0FLK_dX=0.0;
modelStruct.modelPar_given_W0FLK=mod.given("W0FLK");
modParamMap["W0FLK"] = &(modelStruct.d_modelPar_W0FLK_dX);
modelStruct.modelPar_DVTP0=mod.DVTP0;
modelStruct.d_modelPar_DVTP0_dX=0.0;
modelStruct.modelPar_given_DVTP0=mod.given("DVTP0");
modParamMap["DVTP0"] = &(modelStruct.d_modelPar_DVTP0_dX);
modelStruct.modelPar_LDVTP0=mod.LDVTP0;
modelStruct.d_modelPar_LDVTP0_dX=0.0;
modelStruct.modelPar_given_LDVTP0=mod.given("LDVTP0");
modParamMap["LDVTP0"] = &(modelStruct.d_modelPar_LDVTP0_dX);
modelStruct.modelPar_WDVTP0=mod.WDVTP0;
modelStruct.d_modelPar_WDVTP0_dX=0.0;
modelStruct.modelPar_given_WDVTP0=mod.given("WDVTP0");
modParamMap["WDVTP0"] = &(modelStruct.d_modelPar_WDVTP0_dX);
modelStruct.modelPar_PDVTP0=mod.PDVTP0;
modelStruct.d_modelPar_PDVTP0_dX=0.0;
modelStruct.modelPar_given_PDVTP0=mod.given("PDVTP0");
modParamMap["PDVTP0"] = &(modelStruct.d_modelPar_PDVTP0_dX);
modelStruct.modelPar_DVTP1=mod.DVTP1;
modelStruct.d_modelPar_DVTP1_dX=0.0;
modelStruct.modelPar_given_DVTP1=mod.given("DVTP1");
modParamMap["DVTP1"] = &(modelStruct.d_modelPar_DVTP1_dX);
modelStruct.modelPar_LDVTP1=mod.LDVTP1;
modelStruct.d_modelPar_LDVTP1_dX=0.0;
modelStruct.modelPar_given_LDVTP1=mod.given("LDVTP1");
modParamMap["LDVTP1"] = &(modelStruct.d_modelPar_LDVTP1_dX);
modelStruct.modelPar_WDVTP1=mod.WDVTP1;
modelStruct.d_modelPar_WDVTP1_dX=0.0;
modelStruct.modelPar_given_WDVTP1=mod.given("WDVTP1");
modParamMap["WDVTP1"] = &(modelStruct.d_modelPar_WDVTP1_dX);
modelStruct.modelPar_PDVTP1=mod.PDVTP1;
modelStruct.d_modelPar_PDVTP1_dX=0.0;
modelStruct.modelPar_given_PDVTP1=mod.given("PDVTP1");
modParamMap["PDVTP1"] = &(modelStruct.d_modelPar_PDVTP1_dX);
modelStruct.modelPar_DVTP2=mod.DVTP2;
modelStruct.d_modelPar_DVTP2_dX=0.0;
modelStruct.modelPar_given_DVTP2=mod.given("DVTP2");
modParamMap["DVTP2"] = &(modelStruct.d_modelPar_DVTP2_dX);
modelStruct.modelPar_LDVTP2=mod.LDVTP2;
modelStruct.d_modelPar_LDVTP2_dX=0.0;
modelStruct.modelPar_given_LDVTP2=mod.given("LDVTP2");
modParamMap["LDVTP2"] = &(modelStruct.d_modelPar_LDVTP2_dX);
modelStruct.modelPar_WDVTP2=mod.WDVTP2;
modelStruct.d_modelPar_WDVTP2_dX=0.0;
modelStruct.modelPar_given_WDVTP2=mod.given("WDVTP2");
modParamMap["WDVTP2"] = &(modelStruct.d_modelPar_WDVTP2_dX);
modelStruct.modelPar_PDVTP2=mod.PDVTP2;
modelStruct.d_modelPar_PDVTP2_dX=0.0;
modelStruct.modelPar_given_PDVTP2=mod.given("PDVTP2");
modParamMap["PDVTP2"] = &(modelStruct.d_modelPar_PDVTP2_dX);
modelStruct.modelPar_DVTP3=mod.DVTP3;
modelStruct.d_modelPar_DVTP3_dX=0.0;
modelStruct.modelPar_given_DVTP3=mod.given("DVTP3");
modParamMap["DVTP3"] = &(modelStruct.d_modelPar_DVTP3_dX);
modelStruct.modelPar_LDVTP3=mod.LDVTP3;
modelStruct.d_modelPar_LDVTP3_dX=0.0;
modelStruct.modelPar_given_LDVTP3=mod.given("LDVTP3");
modParamMap["LDVTP3"] = &(modelStruct.d_modelPar_LDVTP3_dX);
modelStruct.modelPar_WDVTP3=mod.WDVTP3;
modelStruct.d_modelPar_WDVTP3_dX=0.0;
modelStruct.modelPar_given_WDVTP3=mod.given("WDVTP3");
modParamMap["WDVTP3"] = &(modelStruct.d_modelPar_WDVTP3_dX);
modelStruct.modelPar_PDVTP3=mod.PDVTP3;
modelStruct.d_modelPar_PDVTP3_dX=0.0;
modelStruct.modelPar_given_PDVTP3=mod.given("PDVTP3");
modParamMap["PDVTP3"] = &(modelStruct.d_modelPar_PDVTP3_dX);
modelStruct.modelPar_DVTP4=mod.DVTP4;
modelStruct.d_modelPar_DVTP4_dX=0.0;
modelStruct.modelPar_given_DVTP4=mod.given("DVTP4");
modParamMap["DVTP4"] = &(modelStruct.d_modelPar_DVTP4_dX);
modelStruct.modelPar_LDVTP4=mod.LDVTP4;
modelStruct.d_modelPar_LDVTP4_dX=0.0;
modelStruct.modelPar_given_LDVTP4=mod.given("LDVTP4");
modParamMap["LDVTP4"] = &(modelStruct.d_modelPar_LDVTP4_dX);
modelStruct.modelPar_WDVTP4=mod.WDVTP4;
modelStruct.d_modelPar_WDVTP4_dX=0.0;
modelStruct.modelPar_given_WDVTP4=mod.given("WDVTP4");
modParamMap["WDVTP4"] = &(modelStruct.d_modelPar_WDVTP4_dX);
modelStruct.modelPar_PDVTP4=mod.PDVTP4;
modelStruct.d_modelPar_PDVTP4_dX=0.0;
modelStruct.modelPar_given_PDVTP4=mod.given("PDVTP4");
modParamMap["PDVTP4"] = &(modelStruct.d_modelPar_PDVTP4_dX);
modelStruct.modelPar_MINV=mod.MINV;
modelStruct.d_modelPar_MINV_dX=0.0;
modelStruct.modelPar_given_MINV=mod.given("MINV");
modParamMap["MINV"] = &(modelStruct.d_modelPar_MINV_dX);
modelStruct.modelPar_LMINV=mod.LMINV;
modelStruct.d_modelPar_LMINV_dX=0.0;
modelStruct.modelPar_given_LMINV=mod.given("LMINV");
modParamMap["LMINV"] = &(modelStruct.d_modelPar_LMINV_dX);
modelStruct.modelPar_WMINV=mod.WMINV;
modelStruct.d_modelPar_WMINV_dX=0.0;
modelStruct.modelPar_given_WMINV=mod.given("WMINV");
modParamMap["WMINV"] = &(modelStruct.d_modelPar_WMINV_dX);
modelStruct.modelPar_PMINV=mod.PMINV;
modelStruct.d_modelPar_PMINV_dX=0.0;
modelStruct.modelPar_given_PMINV=mod.given("PMINV");
modParamMap["PMINV"] = &(modelStruct.d_modelPar_PMINV_dX);
modelStruct.modelPar_PDITS=mod.PDITS;
modelStruct.d_modelPar_PDITS_dX=0.0;
modelStruct.modelPar_given_PDITS=mod.given("PDITS");
modParamMap["PDITS"] = &(modelStruct.d_modelPar_PDITS_dX);
modelStruct.modelPar_PDITSL=mod.PDITSL;
modelStruct.d_modelPar_PDITSL_dX=0.0;
modelStruct.modelPar_given_PDITSL=mod.given("PDITSL");
modParamMap["PDITSL"] = &(modelStruct.d_modelPar_PDITSL_dX);
modelStruct.modelPar_PDITSD=mod.PDITSD;
modelStruct.d_modelPar_PDITSD_dX=0.0;
modelStruct.modelPar_given_PDITSD=mod.given("PDITSD");
modParamMap["PDITSD"] = &(modelStruct.d_modelPar_PDITSD_dX);
modelStruct.modelPar_FPROUT=mod.FPROUT;
modelStruct.d_modelPar_FPROUT_dX=0.0;
modelStruct.modelPar_given_FPROUT=mod.given("FPROUT");
modParamMap["FPROUT"] = &(modelStruct.d_modelPar_FPROUT_dX);
modelStruct.modelPar_LFPROUT=mod.LFPROUT;
modelStruct.d_modelPar_LFPROUT_dX=0.0;
modelStruct.modelPar_given_LFPROUT=mod.given("LFPROUT");
modParamMap["LFPROUT"] = &(modelStruct.d_modelPar_LFPROUT_dX);
modelStruct.modelPar_LPDITS=mod.LPDITS;
modelStruct.d_modelPar_LPDITS_dX=0.0;
modelStruct.modelPar_given_LPDITS=mod.given("LPDITS");
modParamMap["LPDITS"] = &(modelStruct.d_modelPar_LPDITS_dX);
modelStruct.modelPar_LPDITSD=mod.LPDITSD;
modelStruct.d_modelPar_LPDITSD_dX=0.0;
modelStruct.modelPar_given_LPDITSD=mod.given("LPDITSD");
modParamMap["LPDITSD"] = &(modelStruct.d_modelPar_LPDITSD_dX);
modelStruct.modelPar_WFPROUT=mod.WFPROUT;
modelStruct.d_modelPar_WFPROUT_dX=0.0;
modelStruct.modelPar_given_WFPROUT=mod.given("WFPROUT");
modParamMap["WFPROUT"] = &(modelStruct.d_modelPar_WFPROUT_dX);
modelStruct.modelPar_WPDITS=mod.WPDITS;
modelStruct.d_modelPar_WPDITS_dX=0.0;
modelStruct.modelPar_given_WPDITS=mod.given("WPDITS");
modParamMap["WPDITS"] = &(modelStruct.d_modelPar_WPDITS_dX);
modelStruct.modelPar_WPDITSD=mod.WPDITSD;
modelStruct.d_modelPar_WPDITSD_dX=0.0;
modelStruct.modelPar_given_WPDITSD=mod.given("WPDITSD");
modParamMap["WPDITSD"] = &(modelStruct.d_modelPar_WPDITSD_dX);
modelStruct.modelPar_PFPROUT=mod.PFPROUT;
modelStruct.d_modelPar_PFPROUT_dX=0.0;
modelStruct.modelPar_given_PFPROUT=mod.given("PFPROUT");
modParamMap["PFPROUT"] = &(modelStruct.d_modelPar_PFPROUT_dX);
modelStruct.modelPar_PPDITS=mod.PPDITS;
modelStruct.d_modelPar_PPDITS_dX=0.0;
modelStruct.modelPar_given_PPDITS=mod.given("PPDITS");
modParamMap["PPDITS"] = &(modelStruct.d_modelPar_PPDITS_dX);
modelStruct.modelPar_PPDITSD=mod.PPDITSD;
modelStruct.d_modelPar_PPDITSD_dX=0.0;
modelStruct.modelPar_given_PPDITSD=mod.given("PPDITSD");
modParamMap["PPDITSD"] = &(modelStruct.d_modelPar_PPDITSD_dX);
modelStruct.modelPar_EM=mod.EM;
modelStruct.d_modelPar_EM_dX=0.0;
modelStruct.modelPar_given_EM=mod.given("EM");
modParamMap["EM"] = &(modelStruct.d_modelPar_EM_dX);
modelStruct.modelPar_EF=mod.EF;
modelStruct.d_modelPar_EF_dX=0.0;
modelStruct.modelPar_given_EF=mod.given("EF");
modParamMap["EF"] = &(modelStruct.d_modelPar_EF_dX);
modelStruct.modelPar_AF=mod.AF;
modelStruct.d_modelPar_AF_dX=0.0;
modelStruct.modelPar_given_AF=mod.given("AF");
modParamMap["AF"] = &(modelStruct.d_modelPar_AF_dX);
modelStruct.modelPar_KF=mod.KF;
modelStruct.d_modelPar_KF_dX=0.0;
modelStruct.modelPar_given_KF=mod.given("KF");
modParamMap["KF"] = &(modelStruct.d_modelPar_KF_dX);
modelStruct.modelPar_NOIF=mod.NOIF;
modelStruct.d_modelPar_NOIF_dX=0.0;
modelStruct.modelPar_given_NOIF=mod.given("NOIF");
modParamMap["NOIF"] = &(modelStruct.d_modelPar_NOIF_dX);
modelStruct.modelPar_K1W1=mod.K1W1;
modelStruct.d_modelPar_K1W1_dX=0.0;
modelStruct.modelPar_given_K1W1=mod.given("K1W1");
modParamMap["K1W1"] = &(modelStruct.d_modelPar_K1W1_dX);
modelStruct.modelPar_K1W2=mod.K1W2;
modelStruct.d_modelPar_K1W2_dX=0.0;
modelStruct.modelPar_given_K1W2=mod.given("K1W2");
modParamMap["K1W2"] = &(modelStruct.d_modelPar_K1W2_dX);
modelStruct.modelPar_KETAS=mod.KETAS;
modelStruct.d_modelPar_KETAS_dX=0.0;
modelStruct.modelPar_given_KETAS=mod.given("KETAS");
modParamMap["KETAS"] = &(modelStruct.d_modelPar_KETAS_dX);
modelStruct.modelPar_DWBC=mod.DWBC;
modelStruct.d_modelPar_DWBC_dX=0.0;
modelStruct.modelPar_given_DWBC=mod.given("DWBC");
modParamMap["DWBC"] = &(modelStruct.d_modelPar_DWBC_dX);
modelStruct.modelPar_BETA0=mod.BETA0;
modelStruct.d_modelPar_BETA0_dX=0.0;
modelStruct.modelPar_given_BETA0=mod.given("BETA0");
modParamMap["BETA0"] = &(modelStruct.d_modelPar_BETA0_dX);
modelStruct.modelPar_BETA1=mod.BETA1;
modelStruct.d_modelPar_BETA1_dX=0.0;
modelStruct.modelPar_given_BETA1=mod.given("BETA1");
modParamMap["BETA1"] = &(modelStruct.d_modelPar_BETA1_dX);
modelStruct.modelPar_BETA2=mod.BETA2;
modelStruct.d_modelPar_BETA2_dX=0.0;
modelStruct.modelPar_given_BETA2=mod.given("BETA2");
modParamMap["BETA2"] = &(modelStruct.d_modelPar_BETA2_dX);
modelStruct.modelPar_VDSATII0=mod.VDSATII0;
modelStruct.d_modelPar_VDSATII0_dX=0.0;
modelStruct.modelPar_given_VDSATII0=mod.given("VDSATII0");
modParamMap["VDSATII0"] = &(modelStruct.d_modelPar_VDSATII0_dX);
modelStruct.modelPar_TII=mod.TII;
modelStruct.d_modelPar_TII_dX=0.0;
modelStruct.modelPar_given_TII=mod.given("TII");
modParamMap["TII"] = &(modelStruct.d_modelPar_TII_dX);
modelStruct.modelPar_LII=mod.LII;
modelStruct.d_modelPar_LII_dX=0.0;
modelStruct.modelPar_given_LII=mod.given("LII");
modParamMap["LII"] = &(modelStruct.d_modelPar_LII_dX);
modelStruct.modelPar_SII0=mod.SII0;
modelStruct.d_modelPar_SII0_dX=0.0;
modelStruct.modelPar_given_SII0=mod.given("SII0");
modParamMap["SII0"] = &(modelStruct.d_modelPar_SII0_dX);
modelStruct.modelPar_SII1=mod.SII1;
modelStruct.d_modelPar_SII1_dX=0.0;
modelStruct.modelPar_given_SII1=mod.given("SII1");
modParamMap["SII1"] = &(modelStruct.d_modelPar_SII1_dX);
modelStruct.modelPar_SII2=mod.SII2;
modelStruct.d_modelPar_SII2_dX=0.0;
modelStruct.modelPar_given_SII2=mod.given("SII2");
modParamMap["SII2"] = &(modelStruct.d_modelPar_SII2_dX);
modelStruct.modelPar_SIID=mod.SIID;
modelStruct.d_modelPar_SIID_dX=0.0;
modelStruct.modelPar_given_SIID=mod.given("SIID");
modParamMap["SIID"] = &(modelStruct.d_modelPar_SIID_dX);
modelStruct.modelPar_FBJTII=mod.FBJTII;
modelStruct.d_modelPar_FBJTII_dX=0.0;
modelStruct.modelPar_given_FBJTII=mod.given("FBJTII");
modParamMap["FBJTII"] = &(modelStruct.d_modelPar_FBJTII_dX);
modelStruct.modelPar_EBJTII=mod.EBJTII;
modelStruct.d_modelPar_EBJTII_dX=0.0;
modelStruct.modelPar_given_EBJTII=mod.given("EBJTII");
modParamMap["EBJTII"] = &(modelStruct.d_modelPar_EBJTII_dX);
modelStruct.modelPar_CBJTII=mod.CBJTII;
modelStruct.d_modelPar_CBJTII_dX=0.0;
modelStruct.modelPar_given_CBJTII=mod.given("CBJTII");
modParamMap["CBJTII"] = &(modelStruct.d_modelPar_CBJTII_dX);
modelStruct.modelPar_VBCI=mod.VBCI;
modelStruct.d_modelPar_VBCI_dX=0.0;
modelStruct.modelPar_given_VBCI=mod.given("VBCI");
modParamMap["VBCI"] = &(modelStruct.d_modelPar_VBCI_dX);
modelStruct.modelPar_ABJTII=mod.ABJTII;
modelStruct.d_modelPar_ABJTII_dX=0.0;
modelStruct.modelPar_given_ABJTII=mod.given("ABJTII");
modParamMap["ABJTII"] = &(modelStruct.d_modelPar_ABJTII_dX);
modelStruct.modelPar_MBJTII=mod.MBJTII;
modelStruct.d_modelPar_MBJTII_dX=0.0;
modelStruct.modelPar_given_MBJTII=mod.given("MBJTII");
modParamMap["MBJTII"] = &(modelStruct.d_modelPar_MBJTII_dX);
modelStruct.modelPar_TVBCI=mod.TVBCI;
modelStruct.d_modelPar_TVBCI_dX=0.0;
modelStruct.modelPar_given_TVBCI=mod.given("TVBCI");
modParamMap["TVBCI"] = &(modelStruct.d_modelPar_TVBCI_dX);
modelStruct.modelPar_ESATII=mod.ESATII;
modelStruct.d_modelPar_ESATII_dX=0.0;
modelStruct.modelPar_given_ESATII=mod.given("ESATII");
modParamMap["ESATII"] = &(modelStruct.d_modelPar_ESATII_dX);
modelStruct.modelPar_NTUN=mod.NTUN;
modelStruct.d_modelPar_NTUN_dX=0.0;
modelStruct.modelPar_given_NTUN=mod.given("NTUN");
modParamMap["NTUN"] = &(modelStruct.d_modelPar_NTUN_dX);
modelStruct.modelPar_NTUND=mod.NTUND;
modelStruct.d_modelPar_NTUND_dX=0.0;
modelStruct.modelPar_given_NTUND=mod.given("NTUND");
modParamMap["NTUND"] = &(modelStruct.d_modelPar_NTUND_dX);
modelStruct.modelPar_NRECF0=mod.NRECF0;
modelStruct.d_modelPar_NRECF0_dX=0.0;
modelStruct.modelPar_given_NRECF0=mod.given("NRECF0");
modParamMap["NRECF0"] = &(modelStruct.d_modelPar_NRECF0_dX);
modelStruct.modelPar_NRECF0D=mod.NRECF0D;
modelStruct.d_modelPar_NRECF0D_dX=0.0;
modelStruct.modelPar_given_NRECF0D=mod.given("NRECF0D");
modParamMap["NRECF0D"] = &(modelStruct.d_modelPar_NRECF0D_dX);
modelStruct.modelPar_NRECR0=mod.NRECR0;
modelStruct.d_modelPar_NRECR0_dX=0.0;
modelStruct.modelPar_given_NRECR0=mod.given("NRECR0");
modParamMap["NRECR0"] = &(modelStruct.d_modelPar_NRECR0_dX);
modelStruct.modelPar_NRECR0D=mod.NRECR0D;
modelStruct.d_modelPar_NRECR0D_dX=0.0;
modelStruct.modelPar_given_NRECR0D=mod.given("NRECR0D");
modParamMap["NRECR0D"] = &(modelStruct.d_modelPar_NRECR0D_dX);
modelStruct.modelPar_ISBJT=mod.ISBJT;
modelStruct.d_modelPar_ISBJT_dX=0.0;
modelStruct.modelPar_given_ISBJT=mod.given("ISBJT");
modParamMap["ISBJT"] = &(modelStruct.d_modelPar_ISBJT_dX);
modelStruct.modelPar_IDBJT=mod.IDBJT;
modelStruct.d_modelPar_IDBJT_dX=0.0;
modelStruct.modelPar_given_IDBJT=mod.given("IDBJT");
modParamMap["IDBJT"] = &(modelStruct.d_modelPar_IDBJT_dX);
modelStruct.modelPar_ISDIF=mod.ISDIF;
modelStruct.d_modelPar_ISDIF_dX=0.0;
modelStruct.modelPar_given_ISDIF=mod.given("ISDIF");
modParamMap["ISDIF"] = &(modelStruct.d_modelPar_ISDIF_dX);
modelStruct.modelPar_IDDIF=mod.IDDIF;
modelStruct.d_modelPar_IDDIF_dX=0.0;
modelStruct.modelPar_given_IDDIF=mod.given("IDDIF");
modParamMap["IDDIF"] = &(modelStruct.d_modelPar_IDDIF_dX);
modelStruct.modelPar_ISREC=mod.ISREC;
modelStruct.d_modelPar_ISREC_dX=0.0;
modelStruct.modelPar_given_ISREC=mod.given("ISREC");
modParamMap["ISREC"] = &(modelStruct.d_modelPar_ISREC_dX);
modelStruct.modelPar_IDREC=mod.IDREC;
modelStruct.d_modelPar_IDREC_dX=0.0;
modelStruct.modelPar_given_IDREC=mod.given("IDREC");
modParamMap["IDREC"] = &(modelStruct.d_modelPar_IDREC_dX);
modelStruct.modelPar_ISTUN=mod.ISTUN;
modelStruct.d_modelPar_ISTUN_dX=0.0;
modelStruct.modelPar_given_ISTUN=mod.given("ISTUN");
modParamMap["ISTUN"] = &(modelStruct.d_modelPar_ISTUN_dX);
modelStruct.modelPar_IDTUN=mod.IDTUN;
modelStruct.d_modelPar_IDTUN_dX=0.0;
modelStruct.modelPar_given_IDTUN=mod.given("IDTUN");
modParamMap["IDTUN"] = &(modelStruct.d_modelPar_IDTUN_dX);
modelStruct.modelPar_LN=mod.LN;
modelStruct.d_modelPar_LN_dX=0.0;
modelStruct.modelPar_given_LN=mod.given("LN");
modParamMap["LN"] = &(modelStruct.d_modelPar_LN_dX);
modelStruct.modelPar_VREC0=mod.VREC0;
modelStruct.d_modelPar_VREC0_dX=0.0;
modelStruct.modelPar_given_VREC0=mod.given("VREC0");
modParamMap["VREC0"] = &(modelStruct.d_modelPar_VREC0_dX);
modelStruct.modelPar_VREC0D=mod.VREC0D;
modelStruct.d_modelPar_VREC0D_dX=0.0;
modelStruct.modelPar_given_VREC0D=mod.given("VREC0D");
modParamMap["VREC0D"] = &(modelStruct.d_modelPar_VREC0D_dX);
modelStruct.modelPar_VTUN0=mod.VTUN0;
modelStruct.d_modelPar_VTUN0_dX=0.0;
modelStruct.modelPar_given_VTUN0=mod.given("VTUN0");
modParamMap["VTUN0"] = &(modelStruct.d_modelPar_VTUN0_dX);
modelStruct.modelPar_VTUN0D=mod.VTUN0D;
modelStruct.d_modelPar_VTUN0D_dX=0.0;
modelStruct.modelPar_given_VTUN0D=mod.given("VTUN0D");
modParamMap["VTUN0D"] = &(modelStruct.d_modelPar_VTUN0D_dX);
modelStruct.modelPar_NBJT=mod.NBJT;
modelStruct.d_modelPar_NBJT_dX=0.0;
modelStruct.modelPar_given_NBJT=mod.given("NBJT");
modParamMap["NBJT"] = &(modelStruct.d_modelPar_NBJT_dX);
modelStruct.modelPar_LBJT0=mod.LBJT0;
modelStruct.d_modelPar_LBJT0_dX=0.0;
modelStruct.modelPar_given_LBJT0=mod.given("LBJT0");
modParamMap["LBJT0"] = &(modelStruct.d_modelPar_LBJT0_dX);
modelStruct.modelPar_LDIF0=mod.LDIF0;
modelStruct.d_modelPar_LDIF0_dX=0.0;
modelStruct.modelPar_given_LDIF0=mod.given("LDIF0");
modParamMap["LDIF0"] = &(modelStruct.d_modelPar_LDIF0_dX);
modelStruct.modelPar_VABJT=mod.VABJT;
modelStruct.d_modelPar_VABJT_dX=0.0;
modelStruct.modelPar_given_VABJT=mod.given("VABJT");
modParamMap["VABJT"] = &(modelStruct.d_modelPar_VABJT_dX);
modelStruct.modelPar_AELY=mod.AELY;
modelStruct.d_modelPar_AELY_dX=0.0;
modelStruct.modelPar_given_AELY=mod.given("AELY");
modParamMap["AELY"] = &(modelStruct.d_modelPar_AELY_dX);
modelStruct.modelPar_AHLI=mod.AHLI;
modelStruct.d_modelPar_AHLI_dX=0.0;
modelStruct.modelPar_given_AHLI=mod.given("AHLI");
modParamMap["AHLI"] = &(modelStruct.d_modelPar_AHLI_dX);
modelStruct.modelPar_AHLID=mod.AHLID;
modelStruct.d_modelPar_AHLID_dX=0.0;
modelStruct.modelPar_given_AHLID=mod.given("AHLID");
modParamMap["AHLID"] = &(modelStruct.d_modelPar_AHLID_dX);
modelStruct.modelPar_RBODY=mod.RBODY;
modelStruct.d_modelPar_RBODY_dX=0.0;
modelStruct.modelPar_given_RBODY=mod.given("RBODY");
modParamMap["RBODY"] = &(modelStruct.d_modelPar_RBODY_dX);
modelStruct.modelPar_RBSH=mod.RBSH;
modelStruct.d_modelPar_RBSH_dX=0.0;
modelStruct.modelPar_given_RBSH=mod.given("RBSH");
modParamMap["RBSH"] = &(modelStruct.d_modelPar_RBSH_dX);
modelStruct.modelPar_CGEO=mod.CGEO;
modelStruct.d_modelPar_CGEO_dX=0.0;
modelStruct.modelPar_given_CGEO=mod.given("CGEO");
modParamMap["CGEO"] = &(modelStruct.d_modelPar_CGEO_dX);
modelStruct.modelPar_TT=mod.TT;
modelStruct.d_modelPar_TT_dX=0.0;
modelStruct.modelPar_given_TT=mod.given("TT");
modParamMap["TT"] = &(modelStruct.d_modelPar_TT_dX);
modelStruct.modelPar_NDIF=mod.NDIF;
modelStruct.d_modelPar_NDIF_dX=0.0;
modelStruct.modelPar_given_NDIF=mod.given("NDIF");
modParamMap["NDIF"] = &(modelStruct.d_modelPar_NDIF_dX);
modelStruct.modelPar_VSDFB=mod.VSDFB;
modelStruct.d_modelPar_VSDFB_dX=0.0;
modelStruct.modelPar_given_VSDFB=mod.given("VSDFB");
modParamMap["VSDFB"] = &(modelStruct.d_modelPar_VSDFB_dX);
modelStruct.modelPar_VSDTH=mod.VSDTH;
modelStruct.d_modelPar_VSDTH_dX=0.0;
modelStruct.modelPar_given_VSDTH=mod.given("VSDTH");
modParamMap["VSDTH"] = &(modelStruct.d_modelPar_VSDTH_dX);
modelStruct.modelPar_CSDMIN=mod.CSDMIN;
modelStruct.d_modelPar_CSDMIN_dX=0.0;
modelStruct.modelPar_given_CSDMIN=mod.given("CSDMIN");
modParamMap["CSDMIN"] = &(modelStruct.d_modelPar_CSDMIN_dX);
modelStruct.modelPar_ASD=mod.ASD;
modelStruct.d_modelPar_ASD_dX=0.0;
modelStruct.modelPar_given_ASD=mod.given("ASD");
modParamMap["ASD"] = &(modelStruct.d_modelPar_ASD_dX);
modelStruct.modelPar_CSDESW=mod.CSDESW;
modelStruct.d_modelPar_CSDESW_dX=0.0;
modelStruct.modelPar_given_CSDESW=mod.given("CSDESW");
modParamMap["CSDESW"] = &(modelStruct.d_modelPar_CSDESW_dX);
modelStruct.modelPar_NTRECF=mod.NTRECF;
modelStruct.d_modelPar_NTRECF_dX=0.0;
modelStruct.modelPar_given_NTRECF=mod.given("NTRECF");
modParamMap["NTRECF"] = &(modelStruct.d_modelPar_NTRECF_dX);
modelStruct.modelPar_NTRECR=mod.NTRECR;
modelStruct.d_modelPar_NTRECR_dX=0.0;
modelStruct.modelPar_given_NTRECR=mod.given("NTRECR");
modParamMap["NTRECR"] = &(modelStruct.d_modelPar_NTRECR_dX);
modelStruct.modelPar_DLCB=mod.DLCB;
modelStruct.d_modelPar_DLCB_dX=0.0;
modelStruct.modelPar_given_DLCB=mod.given("DLCB");
modParamMap["DLCB"] = &(modelStruct.d_modelPar_DLCB_dX);
modelStruct.modelPar_FBODY=mod.FBODY;
modelStruct.d_modelPar_FBODY_dX=0.0;
modelStruct.modelPar_given_FBODY=mod.given("FBODY");
modParamMap["FBODY"] = &(modelStruct.d_modelPar_FBODY_dX);
modelStruct.modelPar_TCJSWG=mod.TCJSWG;
modelStruct.d_modelPar_TCJSWG_dX=0.0;
modelStruct.modelPar_given_TCJSWG=mod.given("TCJSWG");
modParamMap["TCJSWG"] = &(modelStruct.d_modelPar_TCJSWG_dX);
modelStruct.modelPar_TPBSWG=mod.TPBSWG;
modelStruct.d_modelPar_TPBSWG_dX=0.0;
modelStruct.modelPar_given_TPBSWG=mod.given("TPBSWG");
modParamMap["TPBSWG"] = &(modelStruct.d_modelPar_TPBSWG_dX);
modelStruct.modelPar_TCJSWGD=mod.TCJSWGD;
modelStruct.d_modelPar_TCJSWGD_dX=0.0;
modelStruct.modelPar_given_TCJSWGD=mod.given("TCJSWGD");
modParamMap["TCJSWGD"] = &(modelStruct.d_modelPar_TCJSWGD_dX);
modelStruct.modelPar_TPBSWGD=mod.TPBSWGD;
modelStruct.d_modelPar_TPBSWGD_dX=0.0;
modelStruct.modelPar_given_TPBSWGD=mod.given("TPBSWGD");
modParamMap["TPBSWGD"] = &(modelStruct.d_modelPar_TPBSWGD_dX);
modelStruct.modelPar_ACDE=mod.ACDE;
modelStruct.d_modelPar_ACDE_dX=0.0;
modelStruct.modelPar_given_ACDE=mod.given("ACDE");
modParamMap["ACDE"] = &(modelStruct.d_modelPar_ACDE_dX);
modelStruct.modelPar_MOIN=mod.MOIN;
modelStruct.d_modelPar_MOIN_dX=0.0;
modelStruct.modelPar_given_MOIN=mod.given("MOIN");
modParamMap["MOIN"] = &(modelStruct.d_modelPar_MOIN_dX);
modelStruct.modelPar_NOFF=mod.NOFF;
modelStruct.d_modelPar_NOFF_dX=0.0;
modelStruct.modelPar_given_NOFF=mod.given("NOFF");
modParamMap["NOFF"] = &(modelStruct.d_modelPar_NOFF_dX);
modelStruct.modelPar_NOFF2=mod.NOFF2;
modelStruct.d_modelPar_NOFF2_dX=0.0;
modelStruct.modelPar_given_NOFF2=mod.given("NOFF2");
modParamMap["NOFF2"] = &(modelStruct.d_modelPar_NOFF2_dX);
modelStruct.modelPar_DELVT=mod.DELVT;
modelStruct.d_modelPar_DELVT_dX=0.0;
modelStruct.modelPar_given_DELVT=mod.given("DELVT");
modParamMap["DELVT"] = &(modelStruct.d_modelPar_DELVT_dX);
modelStruct.modelPar_KB1=mod.KB1;
modelStruct.d_modelPar_KB1_dX=0.0;
modelStruct.modelPar_given_KB1=mod.given("KB1");
modParamMap["KB1"] = &(modelStruct.d_modelPar_KB1_dX);
modelStruct.modelPar_DLBG=mod.DLBG;
modelStruct.d_modelPar_DLBG_dX=0.0;
modelStruct.modelPar_given_DLBG=mod.given("DLBG");
modParamMap["DLBG"] = &(modelStruct.d_modelPar_DLBG_dX);
modelStruct.modelPar_CFRCOEFF=mod.CFRCOEFF;
modelStruct.d_modelPar_CFRCOEFF_dX=0.0;
modelStruct.modelPar_given_CFRCOEFF=mod.given("CFRCOEFF");
modParamMap["CFRCOEFF"] = &(modelStruct.d_modelPar_CFRCOEFF_dX);
modelStruct.modelPar_TOXQM=mod.TOXQM;
modelStruct.d_modelPar_TOXQM_dX=0.0;
modelStruct.modelPar_given_TOXQM=mod.given("TOXQM");
modParamMap["TOXQM"] = &(modelStruct.d_modelPar_TOXQM_dX);
modelStruct.modelPar_WTH0=mod.WTH0;
modelStruct.d_modelPar_WTH0_dX=0.0;
modelStruct.modelPar_given_WTH0=mod.given("WTH0");
modParamMap["WTH0"] = &(modelStruct.d_modelPar_WTH0_dX);
modelStruct.modelPar_RHALO=mod.RHALO;
modelStruct.d_modelPar_RHALO_dX=0.0;
modelStruct.modelPar_given_RHALO=mod.given("RHALO");
modParamMap["RHALO"] = &(modelStruct.d_modelPar_RHALO_dX);
modelStruct.modelPar_NTOX=mod.NTOX;
modelStruct.d_modelPar_NTOX_dX=0.0;
modelStruct.modelPar_given_NTOX=mod.given("NTOX");
modParamMap["NTOX"] = &(modelStruct.d_modelPar_NTOX_dX);
modelStruct.modelPar_TOXREF=mod.TOXREF;
modelStruct.d_modelPar_TOXREF_dX=0.0;
modelStruct.modelPar_given_TOXREF=mod.given("TOXREF");
modParamMap["TOXREF"] = &(modelStruct.d_modelPar_TOXREF_dX);
modelStruct.modelPar_EBG=mod.EBG;
modelStruct.d_modelPar_EBG_dX=0.0;
modelStruct.modelPar_given_EBG=mod.given("EBG");
modParamMap["EBG"] = &(modelStruct.d_modelPar_EBG_dX);
modelStruct.modelPar_VEVB=mod.VEVB;
modelStruct.d_modelPar_VEVB_dX=0.0;
modelStruct.modelPar_given_VEVB=mod.given("VEVB");
modParamMap["VEVB"] = &(modelStruct.d_modelPar_VEVB_dX);
modelStruct.modelPar_ALPHAGB1=mod.ALPHAGB1;
modelStruct.d_modelPar_ALPHAGB1_dX=0.0;
modelStruct.modelPar_given_ALPHAGB1=mod.given("ALPHAGB1");
modParamMap["ALPHAGB1"] = &(modelStruct.d_modelPar_ALPHAGB1_dX);
modelStruct.modelPar_BETAGB1=mod.BETAGB1;
modelStruct.d_modelPar_BETAGB1_dX=0.0;
modelStruct.modelPar_given_BETAGB1=mod.given("BETAGB1");
modParamMap["BETAGB1"] = &(modelStruct.d_modelPar_BETAGB1_dX);
modelStruct.modelPar_VGB1=mod.VGB1;
modelStruct.d_modelPar_VGB1_dX=0.0;
modelStruct.modelPar_given_VGB1=mod.given("VGB1");
modParamMap["VGB1"] = &(modelStruct.d_modelPar_VGB1_dX);
modelStruct.modelPar_VECB=mod.VECB;
modelStruct.d_modelPar_VECB_dX=0.0;
modelStruct.modelPar_given_VECB=mod.given("VECB");
modParamMap["VECB"] = &(modelStruct.d_modelPar_VECB_dX);
modelStruct.modelPar_ALPHAGB2=mod.ALPHAGB2;
modelStruct.d_modelPar_ALPHAGB2_dX=0.0;
modelStruct.modelPar_given_ALPHAGB2=mod.given("ALPHAGB2");
modParamMap["ALPHAGB2"] = &(modelStruct.d_modelPar_ALPHAGB2_dX);
modelStruct.modelPar_BETAGB2=mod.BETAGB2;
modelStruct.d_modelPar_BETAGB2_dX=0.0;
modelStruct.modelPar_given_BETAGB2=mod.given("BETAGB2");
modParamMap["BETAGB2"] = &(modelStruct.d_modelPar_BETAGB2_dX);
modelStruct.modelPar_VGB2=mod.VGB2;
modelStruct.d_modelPar_VGB2_dX=0.0;
modelStruct.modelPar_given_VGB2=mod.given("VGB2");
modParamMap["VGB2"] = &(modelStruct.d_modelPar_VGB2_dX);
modelStruct.modelPar_AIGBCP2=mod.AIGBCP2;
modelStruct.d_modelPar_AIGBCP2_dX=0.0;
modelStruct.modelPar_given_AIGBCP2=mod.given("AIGBCP2");
modParamMap["AIGBCP2"] = &(modelStruct.d_modelPar_AIGBCP2_dX);
modelStruct.modelPar_BIGBCP2=mod.BIGBCP2;
modelStruct.d_modelPar_BIGBCP2_dX=0.0;
modelStruct.modelPar_given_BIGBCP2=mod.given("BIGBCP2");
modParamMap["BIGBCP2"] = &(modelStruct.d_modelPar_BIGBCP2_dX);
modelStruct.modelPar_CIGBCP2=mod.CIGBCP2;
modelStruct.d_modelPar_CIGBCP2_dX=0.0;
modelStruct.modelPar_given_CIGBCP2=mod.given("CIGBCP2");
modParamMap["CIGBCP2"] = &(modelStruct.d_modelPar_CIGBCP2_dX);
modelStruct.modelPar_VOXH=mod.VOXH;
modelStruct.d_modelPar_VOXH_dX=0.0;
modelStruct.modelPar_given_VOXH=mod.given("VOXH");
modParamMap["VOXH"] = &(modelStruct.d_modelPar_VOXH_dX);
modelStruct.modelPar_DELTAVOX=mod.DELTAVOX;
modelStruct.d_modelPar_DELTAVOX_dX=0.0;
modelStruct.modelPar_given_DELTAVOX=mod.given("DELTAVOX");
modParamMap["DELTAVOX"] = &(modelStruct.d_modelPar_DELTAVOX_dX);
modelStruct.modelPar_AIGC=mod.AIGC;
modelStruct.d_modelPar_AIGC_dX=0.0;
modelStruct.modelPar_given_AIGC=mod.given("AIGC");
modParamMap["AIGC"] = &(modelStruct.d_modelPar_AIGC_dX);
modelStruct.modelPar_BIGC=mod.BIGC;
modelStruct.d_modelPar_BIGC_dX=0.0;
modelStruct.modelPar_given_BIGC=mod.given("BIGC");
modParamMap["BIGC"] = &(modelStruct.d_modelPar_BIGC_dX);
modelStruct.modelPar_CIGC=mod.CIGC;
modelStruct.d_modelPar_CIGC_dX=0.0;
modelStruct.modelPar_given_CIGC=mod.given("CIGC");
modParamMap["CIGC"] = &(modelStruct.d_modelPar_CIGC_dX);
modelStruct.modelPar_AIGSD=mod.AIGSD;
modelStruct.d_modelPar_AIGSD_dX=0.0;
modelStruct.modelPar_given_AIGSD=mod.given("AIGSD");
modParamMap["AIGSD"] = &(modelStruct.d_modelPar_AIGSD_dX);
modelStruct.modelPar_BIGSD=mod.BIGSD;
modelStruct.d_modelPar_BIGSD_dX=0.0;
modelStruct.modelPar_given_BIGSD=mod.given("BIGSD");
modParamMap["BIGSD"] = &(modelStruct.d_modelPar_BIGSD_dX);
modelStruct.modelPar_CIGSD=mod.CIGSD;
modelStruct.d_modelPar_CIGSD_dX=0.0;
modelStruct.modelPar_given_CIGSD=mod.given("CIGSD");
modParamMap["CIGSD"] = &(modelStruct.d_modelPar_CIGSD_dX);
modelStruct.modelPar_NIGC=mod.NIGC;
modelStruct.d_modelPar_NIGC_dX=0.0;
modelStruct.modelPar_given_NIGC=mod.given("NIGC");
modParamMap["NIGC"] = &(modelStruct.d_modelPar_NIGC_dX);
modelStruct.modelPar_PIGCD=mod.PIGCD;
modelStruct.d_modelPar_PIGCD_dX=0.0;
modelStruct.modelPar_given_PIGCD=mod.given("PIGCD");
modParamMap["PIGCD"] = &(modelStruct.d_modelPar_PIGCD_dX);
modelStruct.modelPar_POXEDGE=mod.POXEDGE;
modelStruct.d_modelPar_POXEDGE_dX=0.0;
modelStruct.modelPar_given_POXEDGE=mod.given("POXEDGE");
modParamMap["POXEDGE"] = &(modelStruct.d_modelPar_POXEDGE_dX);
modelStruct.modelPar_DLCIG=mod.DLCIG;
modelStruct.d_modelPar_DLCIG_dX=0.0;
modelStruct.modelPar_given_DLCIG=mod.given("DLCIG");
modParamMap["DLCIG"] = &(modelStruct.d_modelPar_DLCIG_dX);
modelStruct.modelPar_VBS0PD=mod.VBS0PD;
modelStruct.d_modelPar_VBS0PD_dX=0.0;
modelStruct.modelPar_given_VBS0PD=mod.given("VBS0PD");
modParamMap["VBS0PD"] = &(modelStruct.d_modelPar_VBS0PD_dX);
modelStruct.modelPar_VBS0FD=mod.VBS0FD;
modelStruct.d_modelPar_VBS0FD_dX=0.0;
modelStruct.modelPar_given_VBS0FD=mod.given("VBS0FD");
modParamMap["VBS0FD"] = &(modelStruct.d_modelPar_VBS0FD_dX);
modelStruct.modelPar_VBSA=mod.VBSA;
modelStruct.d_modelPar_VBSA_dX=0.0;
modelStruct.modelPar_given_VBSA=mod.given("VBSA");
modParamMap["VBSA"] = &(modelStruct.d_modelPar_VBSA_dX);
modelStruct.modelPar_NOFFFD=mod.NOFFFD;
modelStruct.d_modelPar_NOFFFD_dX=0.0;
modelStruct.modelPar_given_NOFFFD=mod.given("NOFFFD");
modParamMap["NOFFFD"] = &(modelStruct.d_modelPar_NOFFFD_dX);
modelStruct.modelPar_VOFFFD=mod.VOFFFD;
modelStruct.d_modelPar_VOFFFD_dX=0.0;
modelStruct.modelPar_given_VOFFFD=mod.given("VOFFFD");
modParamMap["VOFFFD"] = &(modelStruct.d_modelPar_VOFFFD_dX);
modelStruct.modelPar_K1B=mod.K1B;
modelStruct.d_modelPar_K1B_dX=0.0;
modelStruct.modelPar_given_K1B=mod.given("K1B");
modParamMap["K1B"] = &(modelStruct.d_modelPar_K1B_dX);
modelStruct.modelPar_K2B=mod.K2B;
modelStruct.d_modelPar_K2B_dX=0.0;
modelStruct.modelPar_given_K2B=mod.given("K2B");
modParamMap["K2B"] = &(modelStruct.d_modelPar_K2B_dX);
modelStruct.modelPar_DK2B=mod.DK2B;
modelStruct.d_modelPar_DK2B_dX=0.0;
modelStruct.modelPar_given_DK2B=mod.given("DK2B");
modParamMap["DK2B"] = &(modelStruct.d_modelPar_DK2B_dX);
modelStruct.modelPar_DVBD0=mod.DVBD0;
modelStruct.d_modelPar_DVBD0_dX=0.0;
modelStruct.modelPar_given_DVBD0=mod.given("DVBD0");
modParamMap["DVBD0"] = &(modelStruct.d_modelPar_DVBD0_dX);
modelStruct.modelPar_DVBD1=mod.DVBD1;
modelStruct.d_modelPar_DVBD1_dX=0.0;
modelStruct.modelPar_given_DVBD1=mod.given("DVBD1");
modParamMap["DVBD1"] = &(modelStruct.d_modelPar_DVBD1_dX);
modelStruct.modelPar_MOINFD=mod.MOINFD;
modelStruct.d_modelPar_MOINFD_dX=0.0;
modelStruct.modelPar_given_MOINFD=mod.given("MOINFD");
modParamMap["MOINFD"] = &(modelStruct.d_modelPar_MOINFD_dX);
modelStruct.modelPar_XRCRG1=mod.XRCRG1;
modelStruct.d_modelPar_XRCRG1_dX=0.0;
modelStruct.modelPar_given_XRCRG1=mod.given("XRCRG1");
modParamMap["XRCRG1"] = &(modelStruct.d_modelPar_XRCRG1_dX);
modelStruct.modelPar_XRCRG2=mod.XRCRG2;
modelStruct.d_modelPar_XRCRG2_dX=0.0;
modelStruct.modelPar_given_XRCRG2=mod.given("XRCRG2");
modParamMap["XRCRG2"] = &(modelStruct.d_modelPar_XRCRG2_dX);
modelStruct.modelPar_RSHG=mod.RSHG;
modelStruct.d_modelPar_RSHG_dX=0.0;
modelStruct.modelPar_given_RSHG=mod.given("RSHG");
modParamMap["RSHG"] = &(modelStruct.d_modelPar_RSHG_dX);
modelStruct.modelPar_NGCON=mod.NGCON;
modelStruct.d_modelPar_NGCON_dX=0.0;
modelStruct.modelPar_given_NGCON=mod.given("NGCON");
modParamMap["NGCON"] = &(modelStruct.d_modelPar_NGCON_dX);
modelStruct.modelPar_XGW=mod.XGW;
modelStruct.d_modelPar_XGW_dX=0.0;
modelStruct.modelPar_given_XGW=mod.given("XGW");
modParamMap["XGW"] = &(modelStruct.d_modelPar_XGW_dX);
modelStruct.modelPar_XGL=mod.XGL;
modelStruct.d_modelPar_XGL_dX=0.0;
modelStruct.modelPar_given_XGL=mod.given("XGL");
modParamMap["XGL"] = &(modelStruct.d_modelPar_XGL_dX);
modelStruct.modelPar_VSCE=mod.VSCE;
modelStruct.d_modelPar_VSCE_dX=0.0;
modelStruct.modelPar_given_VSCE=mod.given("VSCE");
modParamMap["VSCE"] = &(modelStruct.d_modelPar_VSCE_dX);
modelStruct.modelPar_CDSBS=mod.CDSBS;
modelStruct.d_modelPar_CDSBS_dX=0.0;
modelStruct.modelPar_given_CDSBS=mod.given("CDSBS");
modParamMap["CDSBS"] = &(modelStruct.d_modelPar_CDSBS_dX);
modelStruct.modelPar_MINVCV=mod.MINVCV;
modelStruct.d_modelPar_MINVCV_dX=0.0;
modelStruct.modelPar_given_MINVCV=mod.given("MINVCV");
modParamMap["MINVCV"] = &(modelStruct.d_modelPar_MINVCV_dX);
modelStruct.modelPar_LMINVCV=mod.LMINVCV;
modelStruct.d_modelPar_LMINVCV_dX=0.0;
modelStruct.modelPar_given_LMINVCV=mod.given("LMINVCV");
modParamMap["LMINVCV"] = &(modelStruct.d_modelPar_LMINVCV_dX);
modelStruct.modelPar_WMINVCV=mod.WMINVCV;
modelStruct.d_modelPar_WMINVCV_dX=0.0;
modelStruct.modelPar_given_WMINVCV=mod.given("WMINVCV");
modParamMap["WMINVCV"] = &(modelStruct.d_modelPar_WMINVCV_dX);
modelStruct.modelPar_PMINVCV=mod.PMINVCV;
modelStruct.d_modelPar_PMINVCV_dX=0.0;
modelStruct.modelPar_given_PMINVCV=mod.given("PMINVCV");
modParamMap["PMINVCV"] = &(modelStruct.d_modelPar_PMINVCV_dX);
modelStruct.modelPar_VOFFCV=mod.VOFFCV;
modelStruct.d_modelPar_VOFFCV_dX=0.0;
modelStruct.modelPar_given_VOFFCV=mod.given("VOFFCV");
modParamMap["VOFFCV"] = &(modelStruct.d_modelPar_VOFFCV_dX);
modelStruct.modelPar_LVOFFCV=mod.LVOFFCV;
modelStruct.d_modelPar_LVOFFCV_dX=0.0;
modelStruct.modelPar_given_LVOFFCV=mod.given("LVOFFCV");
modParamMap["LVOFFCV"] = &(modelStruct.d_modelPar_LVOFFCV_dX);
modelStruct.modelPar_WVOFFCV=mod.WVOFFCV;
modelStruct.d_modelPar_WVOFFCV_dX=0.0;
modelStruct.modelPar_given_WVOFFCV=mod.given("WVOFFCV");
modParamMap["WVOFFCV"] = &(modelStruct.d_modelPar_WVOFFCV_dX);
modelStruct.modelPar_PVOFFCV=mod.PVOFFCV;
modelStruct.d_modelPar_PVOFFCV_dX=0.0;
modelStruct.modelPar_given_PVOFFCV=mod.given("PVOFFCV");
modParamMap["PVOFFCV"] = &(modelStruct.d_modelPar_PVOFFCV_dX);
modelStruct.modelPar_LXJ=mod.LXJ;
modelStruct.d_modelPar_LXJ_dX=0.0;
modelStruct.modelPar_given_LXJ=mod.given("LXJ");
modParamMap["LXJ"] = &(modelStruct.d_modelPar_LXJ_dX);
modelStruct.modelPar_LALPHAGB1=mod.LALPHAGB1;
modelStruct.d_modelPar_LALPHAGB1_dX=0.0;
modelStruct.modelPar_given_LALPHAGB1=mod.given("LALPHAGB1");
modParamMap["LALPHAGB1"] = &(modelStruct.d_modelPar_LALPHAGB1_dX);
modelStruct.modelPar_LBETAGB1=mod.LBETAGB1;
modelStruct.d_modelPar_LBETAGB1_dX=0.0;
modelStruct.modelPar_given_LBETAGB1=mod.given("LBETAGB1");
modParamMap["LBETAGB1"] = &(modelStruct.d_modelPar_LBETAGB1_dX);
modelStruct.modelPar_LALPHAGB2=mod.LALPHAGB2;
modelStruct.d_modelPar_LALPHAGB2_dX=0.0;
modelStruct.modelPar_given_LALPHAGB2=mod.given("LALPHAGB2");
modParamMap["LALPHAGB2"] = &(modelStruct.d_modelPar_LALPHAGB2_dX);
modelStruct.modelPar_LBETAGB2=mod.LBETAGB2;
modelStruct.d_modelPar_LBETAGB2_dX=0.0;
modelStruct.modelPar_given_LBETAGB2=mod.given("LBETAGB2");
modParamMap["LBETAGB2"] = &(modelStruct.d_modelPar_LBETAGB2_dX);
modelStruct.modelPar_LAIGBCP2=mod.LAIGBCP2;
modelStruct.d_modelPar_LAIGBCP2_dX=0.0;
modelStruct.modelPar_given_LAIGBCP2=mod.given("LAIGBCP2");
modParamMap["LAIGBCP2"] = &(modelStruct.d_modelPar_LAIGBCP2_dX);
modelStruct.modelPar_LBIGBCP2=mod.LBIGBCP2;
modelStruct.d_modelPar_LBIGBCP2_dX=0.0;
modelStruct.modelPar_given_LBIGBCP2=mod.given("LBIGBCP2");
modParamMap["LBIGBCP2"] = &(modelStruct.d_modelPar_LBIGBCP2_dX);
modelStruct.modelPar_LCIGBCP2=mod.LCIGBCP2;
modelStruct.d_modelPar_LCIGBCP2_dX=0.0;
modelStruct.modelPar_given_LCIGBCP2=mod.given("LCIGBCP2");
modParamMap["LCIGBCP2"] = &(modelStruct.d_modelPar_LCIGBCP2_dX);
modelStruct.modelPar_LCGSL=mod.LCGSL;
modelStruct.d_modelPar_LCGSL_dX=0.0;
modelStruct.modelPar_given_LCGSL=mod.given("LCGSL");
modParamMap["LCGSL"] = &(modelStruct.d_modelPar_LCGSL_dX);
modelStruct.modelPar_LCGDL=mod.LCGDL;
modelStruct.d_modelPar_LCGDL_dX=0.0;
modelStruct.modelPar_given_LCGDL=mod.given("LCGDL");
modParamMap["LCGDL"] = &(modelStruct.d_modelPar_LCGDL_dX);
modelStruct.modelPar_LCKAPPA=mod.LCKAPPA;
modelStruct.d_modelPar_LCKAPPA_dX=0.0;
modelStruct.modelPar_given_LCKAPPA=mod.given("LCKAPPA");
modParamMap["LCKAPPA"] = &(modelStruct.d_modelPar_LCKAPPA_dX);
modelStruct.modelPar_LNDIF=mod.LNDIF;
modelStruct.d_modelPar_LNDIF_dX=0.0;
modelStruct.modelPar_given_LNDIF=mod.given("LNDIF");
modParamMap["LNDIF"] = &(modelStruct.d_modelPar_LNDIF_dX);
modelStruct.modelPar_LUTE=mod.LUTE;
modelStruct.d_modelPar_LUTE_dX=0.0;
modelStruct.modelPar_given_LUTE=mod.given("LUTE");
modParamMap["LUTE"] = &(modelStruct.d_modelPar_LUTE_dX);
modelStruct.modelPar_LKT1=mod.LKT1;
modelStruct.d_modelPar_LKT1_dX=0.0;
modelStruct.modelPar_given_LKT1=mod.given("LKT1");
modParamMap["LKT1"] = &(modelStruct.d_modelPar_LKT1_dX);
modelStruct.modelPar_LKT1L=mod.LKT1L;
modelStruct.d_modelPar_LKT1L_dX=0.0;
modelStruct.modelPar_given_LKT1L=mod.given("LKT1L");
modParamMap["LKT1L"] = &(modelStruct.d_modelPar_LKT1L_dX);
modelStruct.modelPar_LKT2=mod.LKT2;
modelStruct.d_modelPar_LKT2_dX=0.0;
modelStruct.modelPar_given_LKT2=mod.given("LKT2");
modParamMap["LKT2"] = &(modelStruct.d_modelPar_LKT2_dX);
modelStruct.modelPar_LUA1=mod.LUA1;
modelStruct.d_modelPar_LUA1_dX=0.0;
modelStruct.modelPar_given_LUA1=mod.given("LUA1");
modParamMap["LUA1"] = &(modelStruct.d_modelPar_LUA1_dX);
modelStruct.modelPar_LUB1=mod.LUB1;
modelStruct.d_modelPar_LUB1_dX=0.0;
modelStruct.modelPar_given_LUB1=mod.given("LUB1");
modParamMap["LUB1"] = &(modelStruct.d_modelPar_LUB1_dX);
modelStruct.modelPar_LUC1=mod.LUC1;
modelStruct.d_modelPar_LUC1_dX=0.0;
modelStruct.modelPar_given_LUC1=mod.given("LUC1");
modParamMap["LUC1"] = &(modelStruct.d_modelPar_LUC1_dX);
modelStruct.modelPar_LAT=mod.LAT;
modelStruct.d_modelPar_LAT_dX=0.0;
modelStruct.modelPar_given_LAT=mod.given("LAT");
modParamMap["LAT"] = &(modelStruct.d_modelPar_LAT_dX);
modelStruct.modelPar_LPRT=mod.LPRT;
modelStruct.d_modelPar_LPRT_dX=0.0;
modelStruct.modelPar_given_LPRT=mod.given("LPRT");
modParamMap["LPRT"] = &(modelStruct.d_modelPar_LPRT_dX);
modelStruct.modelPar_LNTRECF=mod.LNTRECF;
modelStruct.d_modelPar_LNTRECF_dX=0.0;
modelStruct.modelPar_given_LNTRECF=mod.given("LNTRECF");
modParamMap["LNTRECF"] = &(modelStruct.d_modelPar_LNTRECF_dX);
modelStruct.modelPar_LNTRECR=mod.LNTRECR;
modelStruct.d_modelPar_LNTRECR_dX=0.0;
modelStruct.modelPar_given_LNTRECR=mod.given("LNTRECR");
modParamMap["LNTRECR"] = &(modelStruct.d_modelPar_LNTRECR_dX);
modelStruct.modelPar_LXBJT=mod.LXBJT;
modelStruct.d_modelPar_LXBJT_dX=0.0;
modelStruct.modelPar_given_LXBJT=mod.given("LXBJT");
modParamMap["LXBJT"] = &(modelStruct.d_modelPar_LXBJT_dX);
modelStruct.modelPar_LXDIF=mod.LXDIF;
modelStruct.d_modelPar_LXDIF_dX=0.0;
modelStruct.modelPar_given_LXDIF=mod.given("LXDIF");
modParamMap["LXDIF"] = &(modelStruct.d_modelPar_LXDIF_dX);
modelStruct.modelPar_LXREC=mod.LXREC;
modelStruct.d_modelPar_LXREC_dX=0.0;
modelStruct.modelPar_given_LXREC=mod.given("LXREC");
modParamMap["LXREC"] = &(modelStruct.d_modelPar_LXREC_dX);
modelStruct.modelPar_LXTUN=mod.LXTUN;
modelStruct.d_modelPar_LXTUN_dX=0.0;
modelStruct.modelPar_given_LXTUN=mod.given("LXTUN");
modParamMap["LXTUN"] = &(modelStruct.d_modelPar_LXTUN_dX);
modelStruct.modelPar_LXDIFD=mod.LXDIFD;
modelStruct.d_modelPar_LXDIFD_dX=0.0;
modelStruct.modelPar_given_LXDIFD=mod.given("LXDIFD");
modParamMap["LXDIFD"] = &(modelStruct.d_modelPar_LXDIFD_dX);
modelStruct.modelPar_LXRECD=mod.LXRECD;
modelStruct.d_modelPar_LXRECD_dX=0.0;
modelStruct.modelPar_given_LXRECD=mod.given("LXRECD");
modParamMap["LXRECD"] = &(modelStruct.d_modelPar_LXRECD_dX);
modelStruct.modelPar_LXTUND=mod.LXTUND;
modelStruct.d_modelPar_LXTUND_dX=0.0;
modelStruct.modelPar_given_LXTUND=mod.given("LXTUND");
modParamMap["LXTUND"] = &(modelStruct.d_modelPar_LXTUND_dX);
modelStruct.modelPar_LAIGC=mod.LAIGC;
modelStruct.d_modelPar_LAIGC_dX=0.0;
modelStruct.modelPar_given_LAIGC=mod.given("LAIGC");
modParamMap["LAIGC"] = &(modelStruct.d_modelPar_LAIGC_dX);
modelStruct.modelPar_LBIGC=mod.LBIGC;
modelStruct.d_modelPar_LBIGC_dX=0.0;
modelStruct.modelPar_given_LBIGC=mod.given("LBIGC");
modParamMap["LBIGC"] = &(modelStruct.d_modelPar_LBIGC_dX);
modelStruct.modelPar_LCIGC=mod.LCIGC;
modelStruct.d_modelPar_LCIGC_dX=0.0;
modelStruct.modelPar_given_LCIGC=mod.given("LCIGC");
modParamMap["LCIGC"] = &(modelStruct.d_modelPar_LCIGC_dX);
modelStruct.modelPar_LAIGSD=mod.LAIGSD;
modelStruct.d_modelPar_LAIGSD_dX=0.0;
modelStruct.modelPar_given_LAIGSD=mod.given("LAIGSD");
modParamMap["LAIGSD"] = &(modelStruct.d_modelPar_LAIGSD_dX);
modelStruct.modelPar_LBIGSD=mod.LBIGSD;
modelStruct.d_modelPar_LBIGSD_dX=0.0;
modelStruct.modelPar_given_LBIGSD=mod.given("LBIGSD");
modParamMap["LBIGSD"] = &(modelStruct.d_modelPar_LBIGSD_dX);
modelStruct.modelPar_LCIGSD=mod.LCIGSD;
modelStruct.d_modelPar_LCIGSD_dX=0.0;
modelStruct.modelPar_given_LCIGSD=mod.given("LCIGSD");
modParamMap["LCIGSD"] = &(modelStruct.d_modelPar_LCIGSD_dX);
modelStruct.modelPar_LNIGC=mod.LNIGC;
modelStruct.d_modelPar_LNIGC_dX=0.0;
modelStruct.modelPar_given_LNIGC=mod.given("LNIGC");
modParamMap["LNIGC"] = &(modelStruct.d_modelPar_LNIGC_dX);
modelStruct.modelPar_LPIGCD=mod.LPIGCD;
modelStruct.d_modelPar_LPIGCD_dX=0.0;
modelStruct.modelPar_given_LPIGCD=mod.given("LPIGCD");
modParamMap["LPIGCD"] = &(modelStruct.d_modelPar_LPIGCD_dX);
modelStruct.modelPar_LPOXEDGE=mod.LPOXEDGE;
modelStruct.d_modelPar_LPOXEDGE_dX=0.0;
modelStruct.modelPar_given_LPOXEDGE=mod.given("LPOXEDGE");
modParamMap["LPOXEDGE"] = &(modelStruct.d_modelPar_LPOXEDGE_dX);
modelStruct.modelPar_LNCH=mod.LNCH;
modelStruct.d_modelPar_LNCH_dX=0.0;
modelStruct.modelPar_given_LNCH=mod.given("LNCH");
modParamMap["LNCH"] = &(modelStruct.d_modelPar_LNCH_dX);
modelStruct.modelPar_LNSUB=mod.LNSUB;
modelStruct.d_modelPar_LNSUB_dX=0.0;
modelStruct.modelPar_given_LNSUB=mod.given("LNSUB");
modParamMap["LNSUB"] = &(modelStruct.d_modelPar_LNSUB_dX);
modelStruct.modelPar_LNGATE=mod.LNGATE;
modelStruct.d_modelPar_LNGATE_dX=0.0;
modelStruct.modelPar_given_LNGATE=mod.given("LNGATE");
modParamMap["LNGATE"] = &(modelStruct.d_modelPar_LNGATE_dX);
modelStruct.modelPar_LNSD=mod.LNSD;
modelStruct.d_modelPar_LNSD_dX=0.0;
modelStruct.modelPar_given_LNSD=mod.given("LNSD");
modParamMap["LNSD"] = &(modelStruct.d_modelPar_LNSD_dX);
modelStruct.modelPar_LVTH0=mod.LVTH0;
modelStruct.d_modelPar_LVTH0_dX=0.0;
modelStruct.modelPar_given_LVTH0=mod.given("LVTH0");
modParamMap["LVTH0"] = &(modelStruct.d_modelPar_LVTH0_dX);
modelStruct.modelPar_LVFB=mod.LVFB;
modelStruct.d_modelPar_LVFB_dX=0.0;
modelStruct.modelPar_given_LVFB=mod.given("LVFB");
modParamMap["LVFB"] = &(modelStruct.d_modelPar_LVFB_dX);
modelStruct.modelPar_LK1=mod.LK1;
modelStruct.d_modelPar_LK1_dX=0.0;
modelStruct.modelPar_given_LK1=mod.given("LK1");
modParamMap["LK1"] = &(modelStruct.d_modelPar_LK1_dX);
modelStruct.modelPar_LK1W1=mod.LK1W1;
modelStruct.d_modelPar_LK1W1_dX=0.0;
modelStruct.modelPar_given_LK1W1=mod.given("LK1W1");
modParamMap["LK1W1"] = &(modelStruct.d_modelPar_LK1W1_dX);
modelStruct.modelPar_LK1W2=mod.LK1W2;
modelStruct.d_modelPar_LK1W2_dX=0.0;
modelStruct.modelPar_given_LK1W2=mod.given("LK1W2");
modParamMap["LK1W2"] = &(modelStruct.d_modelPar_LK1W2_dX);
modelStruct.modelPar_LK2=mod.LK2;
modelStruct.d_modelPar_LK2_dX=0.0;
modelStruct.modelPar_given_LK2=mod.given("LK2");
modParamMap["LK2"] = &(modelStruct.d_modelPar_LK2_dX);
modelStruct.modelPar_LK3=mod.LK3;
modelStruct.d_modelPar_LK3_dX=0.0;
modelStruct.modelPar_given_LK3=mod.given("LK3");
modParamMap["LK3"] = &(modelStruct.d_modelPar_LK3_dX);
modelStruct.modelPar_LK3B=mod.LK3B;
modelStruct.d_modelPar_LK3B_dX=0.0;
modelStruct.modelPar_given_LK3B=mod.given("LK3B");
modParamMap["LK3B"] = &(modelStruct.d_modelPar_LK3B_dX);
modelStruct.modelPar_LKB1=mod.LKB1;
modelStruct.d_modelPar_LKB1_dX=0.0;
modelStruct.modelPar_given_LKB1=mod.given("LKB1");
modParamMap["LKB1"] = &(modelStruct.d_modelPar_LKB1_dX);
modelStruct.modelPar_LW0=mod.LW0;
modelStruct.d_modelPar_LW0_dX=0.0;
modelStruct.modelPar_given_LW0=mod.given("LW0");
modParamMap["LW0"] = &(modelStruct.d_modelPar_LW0_dX);
modelStruct.modelPar_LLPEB=mod.LLPEB;
modelStruct.d_modelPar_LLPEB_dX=0.0;
modelStruct.modelPar_given_LLPEB=mod.given("LLPEB");
modParamMap["LLPEB"] = &(modelStruct.d_modelPar_LLPEB_dX);
modelStruct.modelPar_LDVT0=mod.LDVT0;
modelStruct.d_modelPar_LDVT0_dX=0.0;
modelStruct.modelPar_given_LDVT0=mod.given("LDVT0");
modParamMap["LDVT0"] = &(modelStruct.d_modelPar_LDVT0_dX);
modelStruct.modelPar_LDVT1=mod.LDVT1;
modelStruct.d_modelPar_LDVT1_dX=0.0;
modelStruct.modelPar_given_LDVT1=mod.given("LDVT1");
modParamMap["LDVT1"] = &(modelStruct.d_modelPar_LDVT1_dX);
modelStruct.modelPar_LDVT2=mod.LDVT2;
modelStruct.d_modelPar_LDVT2_dX=0.0;
modelStruct.modelPar_given_LDVT2=mod.given("LDVT2");
modParamMap["LDVT2"] = &(modelStruct.d_modelPar_LDVT2_dX);
modelStruct.modelPar_LDVT0W=mod.LDVT0W;
modelStruct.d_modelPar_LDVT0W_dX=0.0;
modelStruct.modelPar_given_LDVT0W=mod.given("LDVT0W");
modParamMap["LDVT0W"] = &(modelStruct.d_modelPar_LDVT0W_dX);
modelStruct.modelPar_LDVT1W=mod.LDVT1W;
modelStruct.d_modelPar_LDVT1W_dX=0.0;
modelStruct.modelPar_given_LDVT1W=mod.given("LDVT1W");
modParamMap["LDVT1W"] = &(modelStruct.d_modelPar_LDVT1W_dX);
modelStruct.modelPar_LDVT2W=mod.LDVT2W;
modelStruct.d_modelPar_LDVT2W_dX=0.0;
modelStruct.modelPar_given_LDVT2W=mod.given("LDVT2W");
modParamMap["LDVT2W"] = &(modelStruct.d_modelPar_LDVT2W_dX);
modelStruct.modelPar_LU0=mod.LU0;
modelStruct.d_modelPar_LU0_dX=0.0;
modelStruct.modelPar_given_LU0=mod.given("LU0");
modParamMap["LU0"] = &(modelStruct.d_modelPar_LU0_dX);
modelStruct.modelPar_LEU=mod.LEU;
modelStruct.d_modelPar_LEU_dX=0.0;
modelStruct.modelPar_given_LEU=mod.given("LEU");
modParamMap["LEU"] = &(modelStruct.d_modelPar_LEU_dX);
modelStruct.modelPar_LUA=mod.LUA;
modelStruct.d_modelPar_LUA_dX=0.0;
modelStruct.modelPar_given_LUA=mod.given("LUA");
modParamMap["LUA"] = &(modelStruct.d_modelPar_LUA_dX);
modelStruct.modelPar_LUB=mod.LUB;
modelStruct.d_modelPar_LUB_dX=0.0;
modelStruct.modelPar_given_LUB=mod.given("LUB");
modParamMap["LUB"] = &(modelStruct.d_modelPar_LUB_dX);
modelStruct.modelPar_LUC=mod.LUC;
modelStruct.d_modelPar_LUC_dX=0.0;
modelStruct.modelPar_given_LUC=mod.given("LUC");
modParamMap["LUC"] = &(modelStruct.d_modelPar_LUC_dX);
modelStruct.modelPar_LUD=mod.LUD;
modelStruct.d_modelPar_LUD_dX=0.0;
modelStruct.modelPar_given_LUD=mod.given("LUD");
modParamMap["LUD"] = &(modelStruct.d_modelPar_LUD_dX);
modelStruct.modelPar_LUD1=mod.LUD1;
modelStruct.d_modelPar_LUD1_dX=0.0;
modelStruct.modelPar_given_LUD1=mod.given("LUD1");
modParamMap["LUD1"] = &(modelStruct.d_modelPar_LUD1_dX);
modelStruct.modelPar_LUCSTE=mod.LUCSTE;
modelStruct.d_modelPar_LUCSTE_dX=0.0;
modelStruct.modelPar_given_LUCSTE=mod.given("LUCSTE");
modParamMap["LUCSTE"] = &(modelStruct.d_modelPar_LUCSTE_dX);
modelStruct.modelPar_LUCS=mod.LUCS;
modelStruct.d_modelPar_LUCS_dX=0.0;
modelStruct.modelPar_given_LUCS=mod.given("LUCS");
modParamMap["LUCS"] = &(modelStruct.d_modelPar_LUCS_dX);
modelStruct.modelPar_LVSAT=mod.LVSAT;
modelStruct.d_modelPar_LVSAT_dX=0.0;
modelStruct.modelPar_given_LVSAT=mod.given("LVSAT");
modParamMap["LVSAT"] = &(modelStruct.d_modelPar_LVSAT_dX);
modelStruct.modelPar_LA0=mod.LA0;
modelStruct.d_modelPar_LA0_dX=0.0;
modelStruct.modelPar_given_LA0=mod.given("LA0");
modParamMap["LA0"] = &(modelStruct.d_modelPar_LA0_dX);
modelStruct.modelPar_LAGS=mod.LAGS;
modelStruct.d_modelPar_LAGS_dX=0.0;
modelStruct.modelPar_given_LAGS=mod.given("LAGS");
modParamMap["LAGS"] = &(modelStruct.d_modelPar_LAGS_dX);
modelStruct.modelPar_LB0=mod.LB0;
modelStruct.d_modelPar_LB0_dX=0.0;
modelStruct.modelPar_given_LB0=mod.given("LB0");
modParamMap["LB0"] = &(modelStruct.d_modelPar_LB0_dX);
modelStruct.modelPar_LB1=mod.LB1;
modelStruct.d_modelPar_LB1_dX=0.0;
modelStruct.modelPar_given_LB1=mod.given("LB1");
modParamMap["LB1"] = &(modelStruct.d_modelPar_LB1_dX);
modelStruct.modelPar_LKETA=mod.LKETA;
modelStruct.d_modelPar_LKETA_dX=0.0;
modelStruct.modelPar_given_LKETA=mod.given("LKETA");
modParamMap["LKETA"] = &(modelStruct.d_modelPar_LKETA_dX);
modelStruct.modelPar_LKETAS=mod.LKETAS;
modelStruct.d_modelPar_LKETAS_dX=0.0;
modelStruct.modelPar_given_LKETAS=mod.given("LKETAS");
modParamMap["LKETAS"] = &(modelStruct.d_modelPar_LKETAS_dX);
modelStruct.modelPar_LA1=mod.LA1;
modelStruct.d_modelPar_LA1_dX=0.0;
modelStruct.modelPar_given_LA1=mod.given("LA1");
modParamMap["LA1"] = &(modelStruct.d_modelPar_LA1_dX);
modelStruct.modelPar_LA2=mod.LA2;
modelStruct.d_modelPar_LA2_dX=0.0;
modelStruct.modelPar_given_LA2=mod.given("LA2");
modParamMap["LA2"] = &(modelStruct.d_modelPar_LA2_dX);
modelStruct.modelPar_LRDSW=mod.LRDSW;
modelStruct.d_modelPar_LRDSW_dX=0.0;
modelStruct.modelPar_given_LRDSW=mod.given("LRDSW");
modParamMap["LRDSW"] = &(modelStruct.d_modelPar_LRDSW_dX);
modelStruct.modelPar_LRSW=mod.LRSW;
modelStruct.d_modelPar_LRSW_dX=0.0;
modelStruct.modelPar_given_LRSW=mod.given("LRSW");
modParamMap["LRSW"] = &(modelStruct.d_modelPar_LRSW_dX);
modelStruct.modelPar_LRDW=mod.LRDW;
modelStruct.d_modelPar_LRDW_dX=0.0;
modelStruct.modelPar_given_LRDW=mod.given("LRDW");
modParamMap["LRDW"] = &(modelStruct.d_modelPar_LRDW_dX);
modelStruct.modelPar_LPRWB=mod.LPRWB;
modelStruct.d_modelPar_LPRWB_dX=0.0;
modelStruct.modelPar_given_LPRWB=mod.given("LPRWB");
modParamMap["LPRWB"] = &(modelStruct.d_modelPar_LPRWB_dX);
modelStruct.modelPar_LPRWG=mod.LPRWG;
modelStruct.d_modelPar_LPRWG_dX=0.0;
modelStruct.modelPar_given_LPRWG=mod.given("LPRWG");
modParamMap["LPRWG"] = &(modelStruct.d_modelPar_LPRWG_dX);
modelStruct.modelPar_LWR=mod.LWR;
modelStruct.d_modelPar_LWR_dX=0.0;
modelStruct.modelPar_given_LWR=mod.given("LWR");
modParamMap["LWR"] = &(modelStruct.d_modelPar_LWR_dX);
modelStruct.modelPar_LNFACTOR=mod.LNFACTOR;
modelStruct.d_modelPar_LNFACTOR_dX=0.0;
modelStruct.modelPar_given_LNFACTOR=mod.given("LNFACTOR");
modParamMap["LNFACTOR"] = &(modelStruct.d_modelPar_LNFACTOR_dX);
modelStruct.modelPar_LDWG=mod.LDWG;
modelStruct.d_modelPar_LDWG_dX=0.0;
modelStruct.modelPar_given_LDWG=mod.given("LDWG");
modParamMap["LDWG"] = &(modelStruct.d_modelPar_LDWG_dX);
modelStruct.modelPar_LDWB=mod.LDWB;
modelStruct.d_modelPar_LDWB_dX=0.0;
modelStruct.modelPar_given_LDWB=mod.given("LDWB");
modParamMap["LDWB"] = &(modelStruct.d_modelPar_LDWB_dX);
modelStruct.modelPar_LVOFF=mod.LVOFF;
modelStruct.d_modelPar_LVOFF_dX=0.0;
modelStruct.modelPar_given_LVOFF=mod.given("LVOFF");
modParamMap["LVOFF"] = &(modelStruct.d_modelPar_LVOFF_dX);
modelStruct.modelPar_LETA0=mod.LETA0;
modelStruct.d_modelPar_LETA0_dX=0.0;
modelStruct.modelPar_given_LETA0=mod.given("LETA0");
modParamMap["LETA0"] = &(modelStruct.d_modelPar_LETA0_dX);
modelStruct.modelPar_LETAB=mod.LETAB;
modelStruct.d_modelPar_LETAB_dX=0.0;
modelStruct.modelPar_given_LETAB=mod.given("LETAB");
modParamMap["LETAB"] = &(modelStruct.d_modelPar_LETAB_dX);
modelStruct.modelPar_LETA0CV=mod.LETA0CV;
modelStruct.d_modelPar_LETA0CV_dX=0.0;
modelStruct.modelPar_given_LETA0CV=mod.given("LETA0CV");
modParamMap["LETA0CV"] = &(modelStruct.d_modelPar_LETA0CV_dX);
modelStruct.modelPar_LETABCV=mod.LETABCV;
modelStruct.d_modelPar_LETABCV_dX=0.0;
modelStruct.modelPar_given_LETABCV=mod.given("LETABCV");
modParamMap["LETABCV"] = &(modelStruct.d_modelPar_LETABCV_dX);
modelStruct.modelPar_LDSUB=mod.LDSUB;
modelStruct.d_modelPar_LDSUB_dX=0.0;
modelStruct.modelPar_given_LDSUB=mod.given("LDSUB");
modParamMap["LDSUB"] = &(modelStruct.d_modelPar_LDSUB_dX);
modelStruct.modelPar_LCIT=mod.LCIT;
modelStruct.d_modelPar_LCIT_dX=0.0;
modelStruct.modelPar_given_LCIT=mod.given("LCIT");
modParamMap["LCIT"] = &(modelStruct.d_modelPar_LCIT_dX);
modelStruct.modelPar_LCDSC=mod.LCDSC;
modelStruct.d_modelPar_LCDSC_dX=0.0;
modelStruct.modelPar_given_LCDSC=mod.given("LCDSC");
modParamMap["LCDSC"] = &(modelStruct.d_modelPar_LCDSC_dX);
modelStruct.modelPar_LCDSCB=mod.LCDSCB;
modelStruct.d_modelPar_LCDSCB_dX=0.0;
modelStruct.modelPar_given_LCDSCB=mod.given("LCDSCB");
modParamMap["LCDSCB"] = &(modelStruct.d_modelPar_LCDSCB_dX);
modelStruct.modelPar_LCDSCD=mod.LCDSCD;
modelStruct.d_modelPar_LCDSCD_dX=0.0;
modelStruct.modelPar_given_LCDSCD=mod.given("LCDSCD");
modParamMap["LCDSCD"] = &(modelStruct.d_modelPar_LCDSCD_dX);
modelStruct.modelPar_LPCLM=mod.LPCLM;
modelStruct.d_modelPar_LPCLM_dX=0.0;
modelStruct.modelPar_given_LPCLM=mod.given("LPCLM");
modParamMap["LPCLM"] = &(modelStruct.d_modelPar_LPCLM_dX);
modelStruct.modelPar_LPDIBLC1=mod.LPDIBLC1;
modelStruct.d_modelPar_LPDIBLC1_dX=0.0;
modelStruct.modelPar_given_LPDIBLC1=mod.given("LPDIBLC1");
modParamMap["LPDIBLC1"] = &(modelStruct.d_modelPar_LPDIBLC1_dX);
modelStruct.modelPar_LPDIBLC2=mod.LPDIBLC2;
modelStruct.d_modelPar_LPDIBLC2_dX=0.0;
modelStruct.modelPar_given_LPDIBLC2=mod.given("LPDIBLC2");
modParamMap["LPDIBLC2"] = &(modelStruct.d_modelPar_LPDIBLC2_dX);
modelStruct.modelPar_LPDIBLCB=mod.LPDIBLCB;
modelStruct.d_modelPar_LPDIBLCB_dX=0.0;
modelStruct.modelPar_given_LPDIBLCB=mod.given("LPDIBLCB");
modParamMap["LPDIBLCB"] = &(modelStruct.d_modelPar_LPDIBLCB_dX);
modelStruct.modelPar_LDROUT=mod.LDROUT;
modelStruct.d_modelPar_LDROUT_dX=0.0;
modelStruct.modelPar_given_LDROUT=mod.given("LDROUT");
modParamMap["LDROUT"] = &(modelStruct.d_modelPar_LDROUT_dX);
modelStruct.modelPar_LPVAG=mod.LPVAG;
modelStruct.d_modelPar_LPVAG_dX=0.0;
modelStruct.modelPar_given_LPVAG=mod.given("LPVAG");
modParamMap["LPVAG"] = &(modelStruct.d_modelPar_LPVAG_dX);
modelStruct.modelPar_LDELTA=mod.LDELTA;
modelStruct.d_modelPar_LDELTA_dX=0.0;
modelStruct.modelPar_given_LDELTA=mod.given("LDELTA");
modParamMap["LDELTA"] = &(modelStruct.d_modelPar_LDELTA_dX);
modelStruct.modelPar_LALPHA0=mod.LALPHA0;
modelStruct.d_modelPar_LALPHA0_dX=0.0;
modelStruct.modelPar_given_LALPHA0=mod.given("LALPHA0");
modParamMap["LALPHA0"] = &(modelStruct.d_modelPar_LALPHA0_dX);
modelStruct.modelPar_LFBJTII=mod.LFBJTII;
modelStruct.d_modelPar_LFBJTII_dX=0.0;
modelStruct.modelPar_given_LFBJTII=mod.given("LFBJTII");
modParamMap["LFBJTII"] = &(modelStruct.d_modelPar_LFBJTII_dX);
modelStruct.modelPar_LABJTII=mod.LABJTII;
modelStruct.d_modelPar_LABJTII_dX=0.0;
modelStruct.modelPar_given_LABJTII=mod.given("LABJTII");
modParamMap["LABJTII"] = &(modelStruct.d_modelPar_LABJTII_dX);
modelStruct.modelPar_LCBJTII=mod.LCBJTII;
modelStruct.d_modelPar_LCBJTII_dX=0.0;
modelStruct.modelPar_given_LCBJTII=mod.given("LCBJTII");
modParamMap["LCBJTII"] = &(modelStruct.d_modelPar_LCBJTII_dX);
modelStruct.modelPar_LEBJTII=mod.LEBJTII;
modelStruct.d_modelPar_LEBJTII_dX=0.0;
modelStruct.modelPar_given_LEBJTII=mod.given("LEBJTII");
modParamMap["LEBJTII"] = &(modelStruct.d_modelPar_LEBJTII_dX);
modelStruct.modelPar_LMBJTII=mod.LMBJTII;
modelStruct.d_modelPar_LMBJTII_dX=0.0;
modelStruct.modelPar_given_LMBJTII=mod.given("LMBJTII");
modParamMap["LMBJTII"] = &(modelStruct.d_modelPar_LMBJTII_dX);
modelStruct.modelPar_LVBCI=mod.LVBCI;
modelStruct.d_modelPar_LVBCI_dX=0.0;
modelStruct.modelPar_given_LVBCI=mod.given("LVBCI");
modParamMap["LVBCI"] = &(modelStruct.d_modelPar_LVBCI_dX);
modelStruct.modelPar_LBETA0=mod.LBETA0;
modelStruct.d_modelPar_LBETA0_dX=0.0;
modelStruct.modelPar_given_LBETA0=mod.given("LBETA0");
modParamMap["LBETA0"] = &(modelStruct.d_modelPar_LBETA0_dX);
modelStruct.modelPar_LBETA1=mod.LBETA1;
modelStruct.d_modelPar_LBETA1_dX=0.0;
modelStruct.modelPar_given_LBETA1=mod.given("LBETA1");
modParamMap["LBETA1"] = &(modelStruct.d_modelPar_LBETA1_dX);
modelStruct.modelPar_LBETA2=mod.LBETA2;
modelStruct.d_modelPar_LBETA2_dX=0.0;
modelStruct.modelPar_given_LBETA2=mod.given("LBETA2");
modParamMap["LBETA2"] = &(modelStruct.d_modelPar_LBETA2_dX);
modelStruct.modelPar_LVDSATII0=mod.LVDSATII0;
modelStruct.d_modelPar_LVDSATII0_dX=0.0;
modelStruct.modelPar_given_LVDSATII0=mod.given("LVDSATII0");
modParamMap["LVDSATII0"] = &(modelStruct.d_modelPar_LVDSATII0_dX);
modelStruct.modelPar_LLII=mod.LLII;
modelStruct.d_modelPar_LLII_dX=0.0;
modelStruct.modelPar_given_LLII=mod.given("LLII");
modParamMap["LLII"] = &(modelStruct.d_modelPar_LLII_dX);
modelStruct.modelPar_LESATII=mod.LESATII;
modelStruct.d_modelPar_LESATII_dX=0.0;
modelStruct.modelPar_given_LESATII=mod.given("LESATII");
modParamMap["LESATII"] = &(modelStruct.d_modelPar_LESATII_dX);
modelStruct.modelPar_LSII0=mod.LSII0;
modelStruct.d_modelPar_LSII0_dX=0.0;
modelStruct.modelPar_given_LSII0=mod.given("LSII0");
modParamMap["LSII0"] = &(modelStruct.d_modelPar_LSII0_dX);
modelStruct.modelPar_LSII1=mod.LSII1;
modelStruct.d_modelPar_LSII1_dX=0.0;
modelStruct.modelPar_given_LSII1=mod.given("LSII1");
modParamMap["LSII1"] = &(modelStruct.d_modelPar_LSII1_dX);
modelStruct.modelPar_LSII2=mod.LSII2;
modelStruct.d_modelPar_LSII2_dX=0.0;
modelStruct.modelPar_given_LSII2=mod.given("LSII2");
modParamMap["LSII2"] = &(modelStruct.d_modelPar_LSII2_dX);
modelStruct.modelPar_LSIID=mod.LSIID;
modelStruct.d_modelPar_LSIID_dX=0.0;
modelStruct.modelPar_given_LSIID=mod.given("LSIID");
modParamMap["LSIID"] = &(modelStruct.d_modelPar_LSIID_dX);
modelStruct.modelPar_LAGIDL=mod.LAGIDL;
modelStruct.d_modelPar_LAGIDL_dX=0.0;
modelStruct.modelPar_given_LAGIDL=mod.given("LAGIDL");
modParamMap["LAGIDL"] = &(modelStruct.d_modelPar_LAGIDL_dX);
modelStruct.modelPar_LBGIDL=mod.LBGIDL;
modelStruct.d_modelPar_LBGIDL_dX=0.0;
modelStruct.modelPar_given_LBGIDL=mod.given("LBGIDL");
modParamMap["LBGIDL"] = &(modelStruct.d_modelPar_LBGIDL_dX);
modelStruct.modelPar_LCGIDL=mod.LCGIDL;
modelStruct.d_modelPar_LCGIDL_dX=0.0;
modelStruct.modelPar_given_LCGIDL=mod.given("LCGIDL");
modParamMap["LCGIDL"] = &(modelStruct.d_modelPar_LCGIDL_dX);
modelStruct.modelPar_LRGIDL=mod.LRGIDL;
modelStruct.d_modelPar_LRGIDL_dX=0.0;
modelStruct.modelPar_given_LRGIDL=mod.given("LRGIDL");
modParamMap["LRGIDL"] = &(modelStruct.d_modelPar_LRGIDL_dX);
modelStruct.modelPar_LKGIDL=mod.LKGIDL;
modelStruct.d_modelPar_LKGIDL_dX=0.0;
modelStruct.modelPar_given_LKGIDL=mod.given("LKGIDL");
modParamMap["LKGIDL"] = &(modelStruct.d_modelPar_LKGIDL_dX);
modelStruct.modelPar_LFGIDL=mod.LFGIDL;
modelStruct.d_modelPar_LFGIDL_dX=0.0;
modelStruct.modelPar_given_LFGIDL=mod.given("LFGIDL");
modParamMap["LFGIDL"] = &(modelStruct.d_modelPar_LFGIDL_dX);
modelStruct.modelPar_LAGISL=mod.LAGISL;
modelStruct.d_modelPar_LAGISL_dX=0.0;
modelStruct.modelPar_given_LAGISL=mod.given("LAGISL");
modParamMap["LAGISL"] = &(modelStruct.d_modelPar_LAGISL_dX);
modelStruct.modelPar_LBGISL=mod.LBGISL;
modelStruct.d_modelPar_LBGISL_dX=0.0;
modelStruct.modelPar_given_LBGISL=mod.given("LBGISL");
modParamMap["LBGISL"] = &(modelStruct.d_modelPar_LBGISL_dX);
modelStruct.modelPar_LCGISL=mod.LCGISL;
modelStruct.d_modelPar_LCGISL_dX=0.0;
modelStruct.modelPar_given_LCGISL=mod.given("LCGISL");
modParamMap["LCGISL"] = &(modelStruct.d_modelPar_LCGISL_dX);
modelStruct.modelPar_LRGISL=mod.LRGISL;
modelStruct.d_modelPar_LRGISL_dX=0.0;
modelStruct.modelPar_given_LRGISL=mod.given("LRGISL");
modParamMap["LRGISL"] = &(modelStruct.d_modelPar_LRGISL_dX);
modelStruct.modelPar_LKGISL=mod.LKGISL;
modelStruct.d_modelPar_LKGISL_dX=0.0;
modelStruct.modelPar_given_LKGISL=mod.given("LKGISL");
modParamMap["LKGISL"] = &(modelStruct.d_modelPar_LKGISL_dX);
modelStruct.modelPar_LFGISL=mod.LFGISL;
modelStruct.d_modelPar_LFGISL_dX=0.0;
modelStruct.modelPar_given_LFGISL=mod.given("LFGISL");
modParamMap["LFGISL"] = &(modelStruct.d_modelPar_LFGISL_dX);
modelStruct.modelPar_LNTUN=mod.LNTUN;
modelStruct.d_modelPar_LNTUN_dX=0.0;
modelStruct.modelPar_given_LNTUN=mod.given("LNTUN");
modParamMap["LNTUN"] = &(modelStruct.d_modelPar_LNTUN_dX);
modelStruct.modelPar_LNTUND=mod.LNTUND;
modelStruct.d_modelPar_LNTUND_dX=0.0;
modelStruct.modelPar_given_LNTUND=mod.given("LNTUND");
modParamMap["LNTUND"] = &(modelStruct.d_modelPar_LNTUND_dX);
modelStruct.modelPar_LNDIODE=mod.LNDIODE;
modelStruct.d_modelPar_LNDIODE_dX=0.0;
modelStruct.modelPar_given_LNDIODE=mod.given("LNDIODE");
modParamMap["LNDIODE"] = &(modelStruct.d_modelPar_LNDIODE_dX);
modelStruct.modelPar_LNDIODED=mod.LNDIODED;
modelStruct.d_modelPar_LNDIODED_dX=0.0;
modelStruct.modelPar_given_LNDIODED=mod.given("LNDIODED");
modParamMap["LNDIODED"] = &(modelStruct.d_modelPar_LNDIODED_dX);
modelStruct.modelPar_LNRECF0=mod.LNRECF0;
modelStruct.d_modelPar_LNRECF0_dX=0.0;
modelStruct.modelPar_given_LNRECF0=mod.given("LNRECF0");
modParamMap["LNRECF0"] = &(modelStruct.d_modelPar_LNRECF0_dX);
modelStruct.modelPar_LNRECF0D=mod.LNRECF0D;
modelStruct.d_modelPar_LNRECF0D_dX=0.0;
modelStruct.modelPar_given_LNRECF0D=mod.given("LNRECF0D");
modParamMap["LNRECF0D"] = &(modelStruct.d_modelPar_LNRECF0D_dX);
modelStruct.modelPar_LNRECR0=mod.LNRECR0;
modelStruct.d_modelPar_LNRECR0_dX=0.0;
modelStruct.modelPar_given_LNRECR0=mod.given("LNRECR0");
modParamMap["LNRECR0"] = &(modelStruct.d_modelPar_LNRECR0_dX);
modelStruct.modelPar_LNRECR0D=mod.LNRECR0D;
modelStruct.d_modelPar_LNRECR0D_dX=0.0;
modelStruct.modelPar_given_LNRECR0D=mod.given("LNRECR0D");
modParamMap["LNRECR0D"] = &(modelStruct.d_modelPar_LNRECR0D_dX);
modelStruct.modelPar_LISBJT=mod.LISBJT;
modelStruct.d_modelPar_LISBJT_dX=0.0;
modelStruct.modelPar_given_LISBJT=mod.given("LISBJT");
modParamMap["LISBJT"] = &(modelStruct.d_modelPar_LISBJT_dX);
modelStruct.modelPar_LIDBJT=mod.LIDBJT;
modelStruct.d_modelPar_LIDBJT_dX=0.0;
modelStruct.modelPar_given_LIDBJT=mod.given("LIDBJT");
modParamMap["LIDBJT"] = &(modelStruct.d_modelPar_LIDBJT_dX);
modelStruct.modelPar_LISDIF=mod.LISDIF;
modelStruct.d_modelPar_LISDIF_dX=0.0;
modelStruct.modelPar_given_LISDIF=mod.given("LISDIF");
modParamMap["LISDIF"] = &(modelStruct.d_modelPar_LISDIF_dX);
modelStruct.modelPar_LIDDIF=mod.LIDDIF;
modelStruct.d_modelPar_LIDDIF_dX=0.0;
modelStruct.modelPar_given_LIDDIF=mod.given("LIDDIF");
modParamMap["LIDDIF"] = &(modelStruct.d_modelPar_LIDDIF_dX);
modelStruct.modelPar_LISREC=mod.LISREC;
modelStruct.d_modelPar_LISREC_dX=0.0;
modelStruct.modelPar_given_LISREC=mod.given("LISREC");
modParamMap["LISREC"] = &(modelStruct.d_modelPar_LISREC_dX);
modelStruct.modelPar_LIDREC=mod.LIDREC;
modelStruct.d_modelPar_LIDREC_dX=0.0;
modelStruct.modelPar_given_LIDREC=mod.given("LIDREC");
modParamMap["LIDREC"] = &(modelStruct.d_modelPar_LIDREC_dX);
modelStruct.modelPar_LISTUN=mod.LISTUN;
modelStruct.d_modelPar_LISTUN_dX=0.0;
modelStruct.modelPar_given_LISTUN=mod.given("LISTUN");
modParamMap["LISTUN"] = &(modelStruct.d_modelPar_LISTUN_dX);
modelStruct.modelPar_LIDTUN=mod.LIDTUN;
modelStruct.d_modelPar_LIDTUN_dX=0.0;
modelStruct.modelPar_given_LIDTUN=mod.given("LIDTUN");
modParamMap["LIDTUN"] = &(modelStruct.d_modelPar_LIDTUN_dX);
modelStruct.modelPar_LVREC0=mod.LVREC0;
modelStruct.d_modelPar_LVREC0_dX=0.0;
modelStruct.modelPar_given_LVREC0=mod.given("LVREC0");
modParamMap["LVREC0"] = &(modelStruct.d_modelPar_LVREC0_dX);
modelStruct.modelPar_LVREC0D=mod.LVREC0D;
modelStruct.d_modelPar_LVREC0D_dX=0.0;
modelStruct.modelPar_given_LVREC0D=mod.given("LVREC0D");
modParamMap["LVREC0D"] = &(modelStruct.d_modelPar_LVREC0D_dX);
modelStruct.modelPar_LVTUN0=mod.LVTUN0;
modelStruct.d_modelPar_LVTUN0_dX=0.0;
modelStruct.modelPar_given_LVTUN0=mod.given("LVTUN0");
modParamMap["LVTUN0"] = &(modelStruct.d_modelPar_LVTUN0_dX);
modelStruct.modelPar_LVTUN0D=mod.LVTUN0D;
modelStruct.d_modelPar_LVTUN0D_dX=0.0;
modelStruct.modelPar_given_LVTUN0D=mod.given("LVTUN0D");
modParamMap["LVTUN0D"] = &(modelStruct.d_modelPar_LVTUN0D_dX);
modelStruct.modelPar_LNBJT=mod.LNBJT;
modelStruct.d_modelPar_LNBJT_dX=0.0;
modelStruct.modelPar_given_LNBJT=mod.given("LNBJT");
modParamMap["LNBJT"] = &(modelStruct.d_modelPar_LNBJT_dX);
modelStruct.modelPar_LLBJT0=mod.LLBJT0;
modelStruct.d_modelPar_LLBJT0_dX=0.0;
modelStruct.modelPar_given_LLBJT0=mod.given("LLBJT0");
modParamMap["LLBJT0"] = &(modelStruct.d_modelPar_LLBJT0_dX);
modelStruct.modelPar_LVABJT=mod.LVABJT;
modelStruct.d_modelPar_LVABJT_dX=0.0;
modelStruct.modelPar_given_LVABJT=mod.given("LVABJT");
modParamMap["LVABJT"] = &(modelStruct.d_modelPar_LVABJT_dX);
modelStruct.modelPar_LAELY=mod.LAELY;
modelStruct.d_modelPar_LAELY_dX=0.0;
modelStruct.modelPar_given_LAELY=mod.given("LAELY");
modParamMap["LAELY"] = &(modelStruct.d_modelPar_LAELY_dX);
modelStruct.modelPar_LAHLI=mod.LAHLI;
modelStruct.d_modelPar_LAHLI_dX=0.0;
modelStruct.modelPar_given_LAHLI=mod.given("LAHLI");
modParamMap["LAHLI"] = &(modelStruct.d_modelPar_LAHLI_dX);
modelStruct.modelPar_LAHLID=mod.LAHLID;
modelStruct.d_modelPar_LAHLID_dX=0.0;
modelStruct.modelPar_given_LAHLID=mod.given("LAHLID");
modParamMap["LAHLID"] = &(modelStruct.d_modelPar_LAHLID_dX);
modelStruct.modelPar_LVSDFB=mod.LVSDFB;
modelStruct.d_modelPar_LVSDFB_dX=0.0;
modelStruct.modelPar_given_LVSDFB=mod.given("LVSDFB");
modParamMap["LVSDFB"] = &(modelStruct.d_modelPar_LVSDFB_dX);
modelStruct.modelPar_LVSDTH=mod.LVSDTH;
modelStruct.d_modelPar_LVSDTH_dX=0.0;
modelStruct.modelPar_given_LVSDTH=mod.given("LVSDTH");
modParamMap["LVSDTH"] = &(modelStruct.d_modelPar_LVSDTH_dX);
modelStruct.modelPar_LDELVT=mod.LDELVT;
modelStruct.d_modelPar_LDELVT_dX=0.0;
modelStruct.modelPar_given_LDELVT=mod.given("LDELVT");
modParamMap["LDELVT"] = &(modelStruct.d_modelPar_LDELVT_dX);
modelStruct.modelPar_LACDE=mod.LACDE;
modelStruct.d_modelPar_LACDE_dX=0.0;
modelStruct.modelPar_given_LACDE=mod.given("LACDE");
modParamMap["LACDE"] = &(modelStruct.d_modelPar_LACDE_dX);
modelStruct.modelPar_LMOIN=mod.LMOIN;
modelStruct.d_modelPar_LMOIN_dX=0.0;
modelStruct.modelPar_given_LMOIN=mod.given("LMOIN");
modParamMap["LMOIN"] = &(modelStruct.d_modelPar_LMOIN_dX);
modelStruct.modelPar_LNOFF=mod.LNOFF;
modelStruct.d_modelPar_LNOFF_dX=0.0;
modelStruct.modelPar_given_LNOFF=mod.given("LNOFF");
modParamMap["LNOFF"] = &(modelStruct.d_modelPar_LNOFF_dX);
modelStruct.modelPar_LNOFF2=mod.LNOFF2;
modelStruct.d_modelPar_LNOFF2_dX=0.0;
modelStruct.modelPar_given_LNOFF2=mod.given("LNOFF2");
modParamMap["LNOFF2"] = &(modelStruct.d_modelPar_LNOFF2_dX);
modelStruct.modelPar_LXRCRG1=mod.LXRCRG1;
modelStruct.d_modelPar_LXRCRG1_dX=0.0;
modelStruct.modelPar_given_LXRCRG1=mod.given("LXRCRG1");
modParamMap["LXRCRG1"] = &(modelStruct.d_modelPar_LXRCRG1_dX);
modelStruct.modelPar_LXRCRG2=mod.LXRCRG2;
modelStruct.d_modelPar_LXRCRG2_dX=0.0;
modelStruct.modelPar_given_LXRCRG2=mod.given("LXRCRG2");
modParamMap["LXRCRG2"] = &(modelStruct.d_modelPar_LXRCRG2_dX);
modelStruct.modelPar_LVBSA=mod.LVBSA;
modelStruct.d_modelPar_LVBSA_dX=0.0;
modelStruct.modelPar_given_LVBSA=mod.given("LVBSA");
modParamMap["LVBSA"] = &(modelStruct.d_modelPar_LVBSA_dX);
modelStruct.modelPar_LVSCE=mod.LVSCE;
modelStruct.d_modelPar_LVSCE_dX=0.0;
modelStruct.modelPar_given_LVSCE=mod.given("LVSCE");
modParamMap["LVSCE"] = &(modelStruct.d_modelPar_LVSCE_dX);
modelStruct.modelPar_LCDSBS=mod.LCDSBS;
modelStruct.d_modelPar_LCDSBS_dX=0.0;
modelStruct.modelPar_given_LCDSBS=mod.given("LCDSBS");
modParamMap["LCDSBS"] = &(modelStruct.d_modelPar_LCDSBS_dX);
modelStruct.modelPar_LNOFFFD=mod.LNOFFFD;
modelStruct.d_modelPar_LNOFFFD_dX=0.0;
modelStruct.modelPar_given_LNOFFFD=mod.given("LNOFFFD");
modParamMap["LNOFFFD"] = &(modelStruct.d_modelPar_LNOFFFD_dX);
modelStruct.modelPar_LVOFFFD=mod.LVOFFFD;
modelStruct.d_modelPar_LVOFFFD_dX=0.0;
modelStruct.modelPar_given_LVOFFFD=mod.given("LVOFFFD");
modParamMap["LVOFFFD"] = &(modelStruct.d_modelPar_LVOFFFD_dX);
modelStruct.modelPar_LK1B=mod.LK1B;
modelStruct.d_modelPar_LK1B_dX=0.0;
modelStruct.modelPar_given_LK1B=mod.given("LK1B");
modParamMap["LK1B"] = &(modelStruct.d_modelPar_LK1B_dX);
modelStruct.modelPar_LK2B=mod.LK2B;
modelStruct.d_modelPar_LK2B_dX=0.0;
modelStruct.modelPar_given_LK2B=mod.given("LK2B");
modParamMap["LK2B"] = &(modelStruct.d_modelPar_LK2B_dX);
modelStruct.modelPar_LDK2B=mod.LDK2B;
modelStruct.d_modelPar_LDK2B_dX=0.0;
modelStruct.modelPar_given_LDK2B=mod.given("LDK2B");
modParamMap["LDK2B"] = &(modelStruct.d_modelPar_LDK2B_dX);
modelStruct.modelPar_LDVBD0=mod.LDVBD0;
modelStruct.d_modelPar_LDVBD0_dX=0.0;
modelStruct.modelPar_given_LDVBD0=mod.given("LDVBD0");
modParamMap["LDVBD0"] = &(modelStruct.d_modelPar_LDVBD0_dX);
modelStruct.modelPar_LDVBD1=mod.LDVBD1;
modelStruct.d_modelPar_LDVBD1_dX=0.0;
modelStruct.modelPar_given_LDVBD1=mod.given("LDVBD1");
modParamMap["LDVBD1"] = &(modelStruct.d_modelPar_LDVBD1_dX);
modelStruct.modelPar_LMOINFD=mod.LMOINFD;
modelStruct.d_modelPar_LMOINFD_dX=0.0;
modelStruct.modelPar_given_LMOINFD=mod.given("LMOINFD");
modParamMap["LMOINFD"] = &(modelStruct.d_modelPar_LMOINFD_dX);
modelStruct.modelPar_LVBS0PD=mod.LVBS0PD;
modelStruct.d_modelPar_LVBS0PD_dX=0.0;
modelStruct.modelPar_given_LVBS0PD=mod.given("LVBS0PD");
modParamMap["LVBS0PD"] = &(modelStruct.d_modelPar_LVBS0PD_dX);
modelStruct.modelPar_LVBS0FD=mod.LVBS0FD;
modelStruct.d_modelPar_LVBS0FD_dX=0.0;
modelStruct.modelPar_given_LVBS0FD=mod.given("LVBS0FD");
modParamMap["LVBS0FD"] = &(modelStruct.d_modelPar_LVBS0FD_dX);
modelStruct.modelPar_WXJ=mod.WXJ;
modelStruct.d_modelPar_WXJ_dX=0.0;
modelStruct.modelPar_given_WXJ=mod.given("WXJ");
modParamMap["WXJ"] = &(modelStruct.d_modelPar_WXJ_dX);
modelStruct.modelPar_WALPHAGB1=mod.WALPHAGB1;
modelStruct.d_modelPar_WALPHAGB1_dX=0.0;
modelStruct.modelPar_given_WALPHAGB1=mod.given("WALPHAGB1");
modParamMap["WALPHAGB1"] = &(modelStruct.d_modelPar_WALPHAGB1_dX);
modelStruct.modelPar_WBETAGB1=mod.WBETAGB1;
modelStruct.d_modelPar_WBETAGB1_dX=0.0;
modelStruct.modelPar_given_WBETAGB1=mod.given("WBETAGB1");
modParamMap["WBETAGB1"] = &(modelStruct.d_modelPar_WBETAGB1_dX);
modelStruct.modelPar_WALPHAGB2=mod.WALPHAGB2;
modelStruct.d_modelPar_WALPHAGB2_dX=0.0;
modelStruct.modelPar_given_WALPHAGB2=mod.given("WALPHAGB2");
modParamMap["WALPHAGB2"] = &(modelStruct.d_modelPar_WALPHAGB2_dX);
modelStruct.modelPar_WBETAGB2=mod.WBETAGB2;
modelStruct.d_modelPar_WBETAGB2_dX=0.0;
modelStruct.modelPar_given_WBETAGB2=mod.given("WBETAGB2");
modParamMap["WBETAGB2"] = &(modelStruct.d_modelPar_WBETAGB2_dX);
modelStruct.modelPar_WAIGBCP2=mod.WAIGBCP2;
modelStruct.d_modelPar_WAIGBCP2_dX=0.0;
modelStruct.modelPar_given_WAIGBCP2=mod.given("WAIGBCP2");
modParamMap["WAIGBCP2"] = &(modelStruct.d_modelPar_WAIGBCP2_dX);
modelStruct.modelPar_WBIGBCP2=mod.WBIGBCP2;
modelStruct.d_modelPar_WBIGBCP2_dX=0.0;
modelStruct.modelPar_given_WBIGBCP2=mod.given("WBIGBCP2");
modParamMap["WBIGBCP2"] = &(modelStruct.d_modelPar_WBIGBCP2_dX);
modelStruct.modelPar_WCIGBCP2=mod.WCIGBCP2;
modelStruct.d_modelPar_WCIGBCP2_dX=0.0;
modelStruct.modelPar_given_WCIGBCP2=mod.given("WCIGBCP2");
modParamMap["WCIGBCP2"] = &(modelStruct.d_modelPar_WCIGBCP2_dX);
modelStruct.modelPar_WCGSL=mod.WCGSL;
modelStruct.d_modelPar_WCGSL_dX=0.0;
modelStruct.modelPar_given_WCGSL=mod.given("WCGSL");
modParamMap["WCGSL"] = &(modelStruct.d_modelPar_WCGSL_dX);
modelStruct.modelPar_WCGDL=mod.WCGDL;
modelStruct.d_modelPar_WCGDL_dX=0.0;
modelStruct.modelPar_given_WCGDL=mod.given("WCGDL");
modParamMap["WCGDL"] = &(modelStruct.d_modelPar_WCGDL_dX);
modelStruct.modelPar_WCKAPPA=mod.WCKAPPA;
modelStruct.d_modelPar_WCKAPPA_dX=0.0;
modelStruct.modelPar_given_WCKAPPA=mod.given("WCKAPPA");
modParamMap["WCKAPPA"] = &(modelStruct.d_modelPar_WCKAPPA_dX);
modelStruct.modelPar_WNDIF=mod.WNDIF;
modelStruct.d_modelPar_WNDIF_dX=0.0;
modelStruct.modelPar_given_WNDIF=mod.given("WNDIF");
modParamMap["WNDIF"] = &(modelStruct.d_modelPar_WNDIF_dX);
modelStruct.modelPar_WUTE=mod.WUTE;
modelStruct.d_modelPar_WUTE_dX=0.0;
modelStruct.modelPar_given_WUTE=mod.given("WUTE");
modParamMap["WUTE"] = &(modelStruct.d_modelPar_WUTE_dX);
modelStruct.modelPar_WKT1=mod.WKT1;
modelStruct.d_modelPar_WKT1_dX=0.0;
modelStruct.modelPar_given_WKT1=mod.given("WKT1");
modParamMap["WKT1"] = &(modelStruct.d_modelPar_WKT1_dX);
modelStruct.modelPar_WKT1L=mod.WKT1L;
modelStruct.d_modelPar_WKT1L_dX=0.0;
modelStruct.modelPar_given_WKT1L=mod.given("WKT1L");
modParamMap["WKT1L"] = &(modelStruct.d_modelPar_WKT1L_dX);
modelStruct.modelPar_WKT2=mod.WKT2;
modelStruct.d_modelPar_WKT2_dX=0.0;
modelStruct.modelPar_given_WKT2=mod.given("WKT2");
modParamMap["WKT2"] = &(modelStruct.d_modelPar_WKT2_dX);
modelStruct.modelPar_WUA1=mod.WUA1;
modelStruct.d_modelPar_WUA1_dX=0.0;
modelStruct.modelPar_given_WUA1=mod.given("WUA1");
modParamMap["WUA1"] = &(modelStruct.d_modelPar_WUA1_dX);
modelStruct.modelPar_WUB1=mod.WUB1;
modelStruct.d_modelPar_WUB1_dX=0.0;
modelStruct.modelPar_given_WUB1=mod.given("WUB1");
modParamMap["WUB1"] = &(modelStruct.d_modelPar_WUB1_dX);
modelStruct.modelPar_WUC1=mod.WUC1;
modelStruct.d_modelPar_WUC1_dX=0.0;
modelStruct.modelPar_given_WUC1=mod.given("WUC1");
modParamMap["WUC1"] = &(modelStruct.d_modelPar_WUC1_dX);
modelStruct.modelPar_WAT=mod.WAT;
modelStruct.d_modelPar_WAT_dX=0.0;
modelStruct.modelPar_given_WAT=mod.given("WAT");
modParamMap["WAT"] = &(modelStruct.d_modelPar_WAT_dX);
modelStruct.modelPar_WPRT=mod.WPRT;
modelStruct.d_modelPar_WPRT_dX=0.0;
modelStruct.modelPar_given_WPRT=mod.given("WPRT");
modParamMap["WPRT"] = &(modelStruct.d_modelPar_WPRT_dX);
modelStruct.modelPar_WNTRECF=mod.WNTRECF;
modelStruct.d_modelPar_WNTRECF_dX=0.0;
modelStruct.modelPar_given_WNTRECF=mod.given("WNTRECF");
modParamMap["WNTRECF"] = &(modelStruct.d_modelPar_WNTRECF_dX);
modelStruct.modelPar_WNTRECR=mod.WNTRECR;
modelStruct.d_modelPar_WNTRECR_dX=0.0;
modelStruct.modelPar_given_WNTRECR=mod.given("WNTRECR");
modParamMap["WNTRECR"] = &(modelStruct.d_modelPar_WNTRECR_dX);
modelStruct.modelPar_WXBJT=mod.WXBJT;
modelStruct.d_modelPar_WXBJT_dX=0.0;
modelStruct.modelPar_given_WXBJT=mod.given("WXBJT");
modParamMap["WXBJT"] = &(modelStruct.d_modelPar_WXBJT_dX);
modelStruct.modelPar_WXDIF=mod.WXDIF;
modelStruct.d_modelPar_WXDIF_dX=0.0;
modelStruct.modelPar_given_WXDIF=mod.given("WXDIF");
modParamMap["WXDIF"] = &(modelStruct.d_modelPar_WXDIF_dX);
modelStruct.modelPar_WXREC=mod.WXREC;
modelStruct.d_modelPar_WXREC_dX=0.0;
modelStruct.modelPar_given_WXREC=mod.given("WXREC");
modParamMap["WXREC"] = &(modelStruct.d_modelPar_WXREC_dX);
modelStruct.modelPar_WXTUN=mod.WXTUN;
modelStruct.d_modelPar_WXTUN_dX=0.0;
modelStruct.modelPar_given_WXTUN=mod.given("WXTUN");
modParamMap["WXTUN"] = &(modelStruct.d_modelPar_WXTUN_dX);
modelStruct.modelPar_WXDIFD=mod.WXDIFD;
modelStruct.d_modelPar_WXDIFD_dX=0.0;
modelStruct.modelPar_given_WXDIFD=mod.given("WXDIFD");
modParamMap["WXDIFD"] = &(modelStruct.d_modelPar_WXDIFD_dX);
modelStruct.modelPar_WXRECD=mod.WXRECD;
modelStruct.d_modelPar_WXRECD_dX=0.0;
modelStruct.modelPar_given_WXRECD=mod.given("WXRECD");
modParamMap["WXRECD"] = &(modelStruct.d_modelPar_WXRECD_dX);
modelStruct.modelPar_WXTUND=mod.WXTUND;
modelStruct.d_modelPar_WXTUND_dX=0.0;
modelStruct.modelPar_given_WXTUND=mod.given("WXTUND");
modParamMap["WXTUND"] = &(modelStruct.d_modelPar_WXTUND_dX);
modelStruct.modelPar_WAIGC=mod.WAIGC;
modelStruct.d_modelPar_WAIGC_dX=0.0;
modelStruct.modelPar_given_WAIGC=mod.given("WAIGC");
modParamMap["WAIGC"] = &(modelStruct.d_modelPar_WAIGC_dX);
modelStruct.modelPar_WBIGC=mod.WBIGC;
modelStruct.d_modelPar_WBIGC_dX=0.0;
modelStruct.modelPar_given_WBIGC=mod.given("WBIGC");
modParamMap["WBIGC"] = &(modelStruct.d_modelPar_WBIGC_dX);
modelStruct.modelPar_WCIGC=mod.WCIGC;
modelStruct.d_modelPar_WCIGC_dX=0.0;
modelStruct.modelPar_given_WCIGC=mod.given("WCIGC");
modParamMap["WCIGC"] = &(modelStruct.d_modelPar_WCIGC_dX);
modelStruct.modelPar_WAIGSD=mod.WAIGSD;
modelStruct.d_modelPar_WAIGSD_dX=0.0;
modelStruct.modelPar_given_WAIGSD=mod.given("WAIGSD");
modParamMap["WAIGSD"] = &(modelStruct.d_modelPar_WAIGSD_dX);
modelStruct.modelPar_WBIGSD=mod.WBIGSD;
modelStruct.d_modelPar_WBIGSD_dX=0.0;
modelStruct.modelPar_given_WBIGSD=mod.given("WBIGSD");
modParamMap["WBIGSD"] = &(modelStruct.d_modelPar_WBIGSD_dX);
modelStruct.modelPar_WCIGSD=mod.WCIGSD;
modelStruct.d_modelPar_WCIGSD_dX=0.0;
modelStruct.modelPar_given_WCIGSD=mod.given("WCIGSD");
modParamMap["WCIGSD"] = &(modelStruct.d_modelPar_WCIGSD_dX);
modelStruct.modelPar_WNIGC=mod.WNIGC;
modelStruct.d_modelPar_WNIGC_dX=0.0;
modelStruct.modelPar_given_WNIGC=mod.given("WNIGC");
modParamMap["WNIGC"] = &(modelStruct.d_modelPar_WNIGC_dX);
modelStruct.modelPar_WPIGCD=mod.WPIGCD;
modelStruct.d_modelPar_WPIGCD_dX=0.0;
modelStruct.modelPar_given_WPIGCD=mod.given("WPIGCD");
modParamMap["WPIGCD"] = &(modelStruct.d_modelPar_WPIGCD_dX);
modelStruct.modelPar_WPOXEDGE=mod.WPOXEDGE;
modelStruct.d_modelPar_WPOXEDGE_dX=0.0;
modelStruct.modelPar_given_WPOXEDGE=mod.given("WPOXEDGE");
modParamMap["WPOXEDGE"] = &(modelStruct.d_modelPar_WPOXEDGE_dX);
modelStruct.modelPar_WNCH=mod.WNCH;
modelStruct.d_modelPar_WNCH_dX=0.0;
modelStruct.modelPar_given_WNCH=mod.given("WNCH");
modParamMap["WNCH"] = &(modelStruct.d_modelPar_WNCH_dX);
modelStruct.modelPar_WNSUB=mod.WNSUB;
modelStruct.d_modelPar_WNSUB_dX=0.0;
modelStruct.modelPar_given_WNSUB=mod.given("WNSUB");
modParamMap["WNSUB"] = &(modelStruct.d_modelPar_WNSUB_dX);
modelStruct.modelPar_WNGATE=mod.WNGATE;
modelStruct.d_modelPar_WNGATE_dX=0.0;
modelStruct.modelPar_given_WNGATE=mod.given("WNGATE");
modParamMap["WNGATE"] = &(modelStruct.d_modelPar_WNGATE_dX);
modelStruct.modelPar_WNSD=mod.WNSD;
modelStruct.d_modelPar_WNSD_dX=0.0;
modelStruct.modelPar_given_WNSD=mod.given("WNSD");
modParamMap["WNSD"] = &(modelStruct.d_modelPar_WNSD_dX);
modelStruct.modelPar_WVTH0=mod.WVTH0;
modelStruct.d_modelPar_WVTH0_dX=0.0;
modelStruct.modelPar_given_WVTH0=mod.given("WVTH0");
modParamMap["WVTH0"] = &(modelStruct.d_modelPar_WVTH0_dX);
modelStruct.modelPar_WVFB=mod.WVFB;
modelStruct.d_modelPar_WVFB_dX=0.0;
modelStruct.modelPar_given_WVFB=mod.given("WVFB");
modParamMap["WVFB"] = &(modelStruct.d_modelPar_WVFB_dX);
modelStruct.modelPar_WK1=mod.WK1;
modelStruct.d_modelPar_WK1_dX=0.0;
modelStruct.modelPar_given_WK1=mod.given("WK1");
modParamMap["WK1"] = &(modelStruct.d_modelPar_WK1_dX);
modelStruct.modelPar_WK1W1=mod.WK1W1;
modelStruct.d_modelPar_WK1W1_dX=0.0;
modelStruct.modelPar_given_WK1W1=mod.given("WK1W1");
modParamMap["WK1W1"] = &(modelStruct.d_modelPar_WK1W1_dX);
modelStruct.modelPar_WK1W2=mod.WK1W2;
modelStruct.d_modelPar_WK1W2_dX=0.0;
modelStruct.modelPar_given_WK1W2=mod.given("WK1W2");
modParamMap["WK1W2"] = &(modelStruct.d_modelPar_WK1W2_dX);
modelStruct.modelPar_WK2=mod.WK2;
modelStruct.d_modelPar_WK2_dX=0.0;
modelStruct.modelPar_given_WK2=mod.given("WK2");
modParamMap["WK2"] = &(modelStruct.d_modelPar_WK2_dX);
modelStruct.modelPar_WK3=mod.WK3;
modelStruct.d_modelPar_WK3_dX=0.0;
modelStruct.modelPar_given_WK3=mod.given("WK3");
modParamMap["WK3"] = &(modelStruct.d_modelPar_WK3_dX);
modelStruct.modelPar_WK3B=mod.WK3B;
modelStruct.d_modelPar_WK3B_dX=0.0;
modelStruct.modelPar_given_WK3B=mod.given("WK3B");
modParamMap["WK3B"] = &(modelStruct.d_modelPar_WK3B_dX);
modelStruct.modelPar_WKB1=mod.WKB1;
modelStruct.d_modelPar_WKB1_dX=0.0;
modelStruct.modelPar_given_WKB1=mod.given("WKB1");
modParamMap["WKB1"] = &(modelStruct.d_modelPar_WKB1_dX);
modelStruct.modelPar_WW0=mod.WW0;
modelStruct.d_modelPar_WW0_dX=0.0;
modelStruct.modelPar_given_WW0=mod.given("WW0");
modParamMap["WW0"] = &(modelStruct.d_modelPar_WW0_dX);
modelStruct.modelPar_WLPEB=mod.WLPEB;
modelStruct.d_modelPar_WLPEB_dX=0.0;
modelStruct.modelPar_given_WLPEB=mod.given("WLPEB");
modParamMap["WLPEB"] = &(modelStruct.d_modelPar_WLPEB_dX);
modelStruct.modelPar_WDVT0=mod.WDVT0;
modelStruct.d_modelPar_WDVT0_dX=0.0;
modelStruct.modelPar_given_WDVT0=mod.given("WDVT0");
modParamMap["WDVT0"] = &(modelStruct.d_modelPar_WDVT0_dX);
modelStruct.modelPar_WDVT1=mod.WDVT1;
modelStruct.d_modelPar_WDVT1_dX=0.0;
modelStruct.modelPar_given_WDVT1=mod.given("WDVT1");
modParamMap["WDVT1"] = &(modelStruct.d_modelPar_WDVT1_dX);
modelStruct.modelPar_WDVT2=mod.WDVT2;
modelStruct.d_modelPar_WDVT2_dX=0.0;
modelStruct.modelPar_given_WDVT2=mod.given("WDVT2");
modParamMap["WDVT2"] = &(modelStruct.d_modelPar_WDVT2_dX);
modelStruct.modelPar_WDVT0W=mod.WDVT0W;
modelStruct.d_modelPar_WDVT0W_dX=0.0;
modelStruct.modelPar_given_WDVT0W=mod.given("WDVT0W");
modParamMap["WDVT0W"] = &(modelStruct.d_modelPar_WDVT0W_dX);
modelStruct.modelPar_WDVT1W=mod.WDVT1W;
modelStruct.d_modelPar_WDVT1W_dX=0.0;
modelStruct.modelPar_given_WDVT1W=mod.given("WDVT1W");
modParamMap["WDVT1W"] = &(modelStruct.d_modelPar_WDVT1W_dX);
modelStruct.modelPar_WDVT2W=mod.WDVT2W;
modelStruct.d_modelPar_WDVT2W_dX=0.0;
modelStruct.modelPar_given_WDVT2W=mod.given("WDVT2W");
modParamMap["WDVT2W"] = &(modelStruct.d_modelPar_WDVT2W_dX);
modelStruct.modelPar_WU0=mod.WU0;
modelStruct.d_modelPar_WU0_dX=0.0;
modelStruct.modelPar_given_WU0=mod.given("WU0");
modParamMap["WU0"] = &(modelStruct.d_modelPar_WU0_dX);
modelStruct.modelPar_WEU=mod.WEU;
modelStruct.d_modelPar_WEU_dX=0.0;
modelStruct.modelPar_given_WEU=mod.given("WEU");
modParamMap["WEU"] = &(modelStruct.d_modelPar_WEU_dX);
modelStruct.modelPar_WUA=mod.WUA;
modelStruct.d_modelPar_WUA_dX=0.0;
modelStruct.modelPar_given_WUA=mod.given("WUA");
modParamMap["WUA"] = &(modelStruct.d_modelPar_WUA_dX);
modelStruct.modelPar_WUB=mod.WUB;
modelStruct.d_modelPar_WUB_dX=0.0;
modelStruct.modelPar_given_WUB=mod.given("WUB");
modParamMap["WUB"] = &(modelStruct.d_modelPar_WUB_dX);
modelStruct.modelPar_WUC=mod.WUC;
modelStruct.d_modelPar_WUC_dX=0.0;
modelStruct.modelPar_given_WUC=mod.given("WUC");
modParamMap["WUC"] = &(modelStruct.d_modelPar_WUC_dX);
modelStruct.modelPar_WUD=mod.WUD;
modelStruct.d_modelPar_WUD_dX=0.0;
modelStruct.modelPar_given_WUD=mod.given("WUD");
modParamMap["WUD"] = &(modelStruct.d_modelPar_WUD_dX);
modelStruct.modelPar_WUD1=mod.WUD1;
modelStruct.d_modelPar_WUD1_dX=0.0;
modelStruct.modelPar_given_WUD1=mod.given("WUD1");
modParamMap["WUD1"] = &(modelStruct.d_modelPar_WUD1_dX);
modelStruct.modelPar_WUCSTE=mod.WUCSTE;
modelStruct.d_modelPar_WUCSTE_dX=0.0;
modelStruct.modelPar_given_WUCSTE=mod.given("WUCSTE");
modParamMap["WUCSTE"] = &(modelStruct.d_modelPar_WUCSTE_dX);
modelStruct.modelPar_WUCS=mod.WUCS;
modelStruct.d_modelPar_WUCS_dX=0.0;
modelStruct.modelPar_given_WUCS=mod.given("WUCS");
modParamMap["WUCS"] = &(modelStruct.d_modelPar_WUCS_dX);
modelStruct.modelPar_WVSAT=mod.WVSAT;
modelStruct.d_modelPar_WVSAT_dX=0.0;
modelStruct.modelPar_given_WVSAT=mod.given("WVSAT");
modParamMap["WVSAT"] = &(modelStruct.d_modelPar_WVSAT_dX);
modelStruct.modelPar_WA0=mod.WA0;
modelStruct.d_modelPar_WA0_dX=0.0;
modelStruct.modelPar_given_WA0=mod.given("WA0");
modParamMap["WA0"] = &(modelStruct.d_modelPar_WA0_dX);
modelStruct.modelPar_WAGS=mod.WAGS;
modelStruct.d_modelPar_WAGS_dX=0.0;
modelStruct.modelPar_given_WAGS=mod.given("WAGS");
modParamMap["WAGS"] = &(modelStruct.d_modelPar_WAGS_dX);
modelStruct.modelPar_WB0=mod.WB0;
modelStruct.d_modelPar_WB0_dX=0.0;
modelStruct.modelPar_given_WB0=mod.given("WB0");
modParamMap["WB0"] = &(modelStruct.d_modelPar_WB0_dX);
modelStruct.modelPar_WB1=mod.WB1;
modelStruct.d_modelPar_WB1_dX=0.0;
modelStruct.modelPar_given_WB1=mod.given("WB1");
modParamMap["WB1"] = &(modelStruct.d_modelPar_WB1_dX);
modelStruct.modelPar_WKETA=mod.WKETA;
modelStruct.d_modelPar_WKETA_dX=0.0;
modelStruct.modelPar_given_WKETA=mod.given("WKETA");
modParamMap["WKETA"] = &(modelStruct.d_modelPar_WKETA_dX);
modelStruct.modelPar_WKETAS=mod.WKETAS;
modelStruct.d_modelPar_WKETAS_dX=0.0;
modelStruct.modelPar_given_WKETAS=mod.given("WKETAS");
modParamMap["WKETAS"] = &(modelStruct.d_modelPar_WKETAS_dX);
modelStruct.modelPar_WA1=mod.WA1;
modelStruct.d_modelPar_WA1_dX=0.0;
modelStruct.modelPar_given_WA1=mod.given("WA1");
modParamMap["WA1"] = &(modelStruct.d_modelPar_WA1_dX);
modelStruct.modelPar_WA2=mod.WA2;
modelStruct.d_modelPar_WA2_dX=0.0;
modelStruct.modelPar_given_WA2=mod.given("WA2");
modParamMap["WA2"] = &(modelStruct.d_modelPar_WA2_dX);
modelStruct.modelPar_WRDSW=mod.WRDSW;
modelStruct.d_modelPar_WRDSW_dX=0.0;
modelStruct.modelPar_given_WRDSW=mod.given("WRDSW");
modParamMap["WRDSW"] = &(modelStruct.d_modelPar_WRDSW_dX);
modelStruct.modelPar_WRSW=mod.WRSW;
modelStruct.d_modelPar_WRSW_dX=0.0;
modelStruct.modelPar_given_WRSW=mod.given("WRSW");
modParamMap["WRSW"] = &(modelStruct.d_modelPar_WRSW_dX);
modelStruct.modelPar_WRDW=mod.WRDW;
modelStruct.d_modelPar_WRDW_dX=0.0;
modelStruct.modelPar_given_WRDW=mod.given("WRDW");
modParamMap["WRDW"] = &(modelStruct.d_modelPar_WRDW_dX);
modelStruct.modelPar_WPRWB=mod.WPRWB;
modelStruct.d_modelPar_WPRWB_dX=0.0;
modelStruct.modelPar_given_WPRWB=mod.given("WPRWB");
modParamMap["WPRWB"] = &(modelStruct.d_modelPar_WPRWB_dX);
modelStruct.modelPar_WPRWG=mod.WPRWG;
modelStruct.d_modelPar_WPRWG_dX=0.0;
modelStruct.modelPar_given_WPRWG=mod.given("WPRWG");
modParamMap["WPRWG"] = &(modelStruct.d_modelPar_WPRWG_dX);
modelStruct.modelPar_WWR=mod.WWR;
modelStruct.d_modelPar_WWR_dX=0.0;
modelStruct.modelPar_given_WWR=mod.given("WWR");
modParamMap["WWR"] = &(modelStruct.d_modelPar_WWR_dX);
modelStruct.modelPar_WNFACTOR=mod.WNFACTOR;
modelStruct.d_modelPar_WNFACTOR_dX=0.0;
modelStruct.modelPar_given_WNFACTOR=mod.given("WNFACTOR");
modParamMap["WNFACTOR"] = &(modelStruct.d_modelPar_WNFACTOR_dX);
modelStruct.modelPar_WDWG=mod.WDWG;
modelStruct.d_modelPar_WDWG_dX=0.0;
modelStruct.modelPar_given_WDWG=mod.given("WDWG");
modParamMap["WDWG"] = &(modelStruct.d_modelPar_WDWG_dX);
modelStruct.modelPar_WDWB=mod.WDWB;
modelStruct.d_modelPar_WDWB_dX=0.0;
modelStruct.modelPar_given_WDWB=mod.given("WDWB");
modParamMap["WDWB"] = &(modelStruct.d_modelPar_WDWB_dX);
modelStruct.modelPar_WVOFF=mod.WVOFF;
modelStruct.d_modelPar_WVOFF_dX=0.0;
modelStruct.modelPar_given_WVOFF=mod.given("WVOFF");
modParamMap["WVOFF"] = &(modelStruct.d_modelPar_WVOFF_dX);
modelStruct.modelPar_WETA0=mod.WETA0;
modelStruct.d_modelPar_WETA0_dX=0.0;
modelStruct.modelPar_given_WETA0=mod.given("WETA0");
modParamMap["WETA0"] = &(modelStruct.d_modelPar_WETA0_dX);
modelStruct.modelPar_WETAB=mod.WETAB;
modelStruct.d_modelPar_WETAB_dX=0.0;
modelStruct.modelPar_given_WETAB=mod.given("WETAB");
modParamMap["WETAB"] = &(modelStruct.d_modelPar_WETAB_dX);
modelStruct.modelPar_WETA0CV=mod.WETA0CV;
modelStruct.d_modelPar_WETA0CV_dX=0.0;
modelStruct.modelPar_given_WETA0CV=mod.given("WETA0CV");
modParamMap["WETA0CV"] = &(modelStruct.d_modelPar_WETA0CV_dX);
modelStruct.modelPar_WETABCV=mod.WETABCV;
modelStruct.d_modelPar_WETABCV_dX=0.0;
modelStruct.modelPar_given_WETABCV=mod.given("WETABCV");
modParamMap["WETABCV"] = &(modelStruct.d_modelPar_WETABCV_dX);
modelStruct.modelPar_WDSUB=mod.WDSUB;
modelStruct.d_modelPar_WDSUB_dX=0.0;
modelStruct.modelPar_given_WDSUB=mod.given("WDSUB");
modParamMap["WDSUB"] = &(modelStruct.d_modelPar_WDSUB_dX);
modelStruct.modelPar_WCIT=mod.WCIT;
modelStruct.d_modelPar_WCIT_dX=0.0;
modelStruct.modelPar_given_WCIT=mod.given("WCIT");
modParamMap["WCIT"] = &(modelStruct.d_modelPar_WCIT_dX);
modelStruct.modelPar_WCDSC=mod.WCDSC;
modelStruct.d_modelPar_WCDSC_dX=0.0;
modelStruct.modelPar_given_WCDSC=mod.given("WCDSC");
modParamMap["WCDSC"] = &(modelStruct.d_modelPar_WCDSC_dX);
modelStruct.modelPar_WCDSCB=mod.WCDSCB;
modelStruct.d_modelPar_WCDSCB_dX=0.0;
modelStruct.modelPar_given_WCDSCB=mod.given("WCDSCB");
modParamMap["WCDSCB"] = &(modelStruct.d_modelPar_WCDSCB_dX);
modelStruct.modelPar_WCDSCD=mod.WCDSCD;
modelStruct.d_modelPar_WCDSCD_dX=0.0;
modelStruct.modelPar_given_WCDSCD=mod.given("WCDSCD");
modParamMap["WCDSCD"] = &(modelStruct.d_modelPar_WCDSCD_dX);
modelStruct.modelPar_WPCLM=mod.WPCLM;
modelStruct.d_modelPar_WPCLM_dX=0.0;
modelStruct.modelPar_given_WPCLM=mod.given("WPCLM");
modParamMap["WPCLM"] = &(modelStruct.d_modelPar_WPCLM_dX);
modelStruct.modelPar_WPDIBLC1=mod.WPDIBLC1;
modelStruct.d_modelPar_WPDIBLC1_dX=0.0;
modelStruct.modelPar_given_WPDIBLC1=mod.given("WPDIBLC1");
modParamMap["WPDIBLC1"] = &(modelStruct.d_modelPar_WPDIBLC1_dX);
modelStruct.modelPar_WPDIBLC2=mod.WPDIBLC2;
modelStruct.d_modelPar_WPDIBLC2_dX=0.0;
modelStruct.modelPar_given_WPDIBLC2=mod.given("WPDIBLC2");
modParamMap["WPDIBLC2"] = &(modelStruct.d_modelPar_WPDIBLC2_dX);
modelStruct.modelPar_WPDIBLCB=mod.WPDIBLCB;
modelStruct.d_modelPar_WPDIBLCB_dX=0.0;
modelStruct.modelPar_given_WPDIBLCB=mod.given("WPDIBLCB");
modParamMap["WPDIBLCB"] = &(modelStruct.d_modelPar_WPDIBLCB_dX);
modelStruct.modelPar_WDROUT=mod.WDROUT;
modelStruct.d_modelPar_WDROUT_dX=0.0;
modelStruct.modelPar_given_WDROUT=mod.given("WDROUT");
modParamMap["WDROUT"] = &(modelStruct.d_modelPar_WDROUT_dX);
modelStruct.modelPar_WPVAG=mod.WPVAG;
modelStruct.d_modelPar_WPVAG_dX=0.0;
modelStruct.modelPar_given_WPVAG=mod.given("WPVAG");
modParamMap["WPVAG"] = &(modelStruct.d_modelPar_WPVAG_dX);
modelStruct.modelPar_WDELTA=mod.WDELTA;
modelStruct.d_modelPar_WDELTA_dX=0.0;
modelStruct.modelPar_given_WDELTA=mod.given("WDELTA");
modParamMap["WDELTA"] = &(modelStruct.d_modelPar_WDELTA_dX);
modelStruct.modelPar_WALPHA0=mod.WALPHA0;
modelStruct.d_modelPar_WALPHA0_dX=0.0;
modelStruct.modelPar_given_WALPHA0=mod.given("WALPHA0");
modParamMap["WALPHA0"] = &(modelStruct.d_modelPar_WALPHA0_dX);
modelStruct.modelPar_WFBJTII=mod.WFBJTII;
modelStruct.d_modelPar_WFBJTII_dX=0.0;
modelStruct.modelPar_given_WFBJTII=mod.given("WFBJTII");
modParamMap["WFBJTII"] = &(modelStruct.d_modelPar_WFBJTII_dX);
modelStruct.modelPar_WABJTII=mod.WABJTII;
modelStruct.d_modelPar_WABJTII_dX=0.0;
modelStruct.modelPar_given_WABJTII=mod.given("WABJTII");
modParamMap["WABJTII"] = &(modelStruct.d_modelPar_WABJTII_dX);
modelStruct.modelPar_WCBJTII=mod.WCBJTII;
modelStruct.d_modelPar_WCBJTII_dX=0.0;
modelStruct.modelPar_given_WCBJTII=mod.given("WCBJTII");
modParamMap["WCBJTII"] = &(modelStruct.d_modelPar_WCBJTII_dX);
modelStruct.modelPar_WEBJTII=mod.WEBJTII;
modelStruct.d_modelPar_WEBJTII_dX=0.0;
modelStruct.modelPar_given_WEBJTII=mod.given("WEBJTII");
modParamMap["WEBJTII"] = &(modelStruct.d_modelPar_WEBJTII_dX);
modelStruct.modelPar_WMBJTII=mod.WMBJTII;
modelStruct.d_modelPar_WMBJTII_dX=0.0;
modelStruct.modelPar_given_WMBJTII=mod.given("WMBJTII");
modParamMap["WMBJTII"] = &(modelStruct.d_modelPar_WMBJTII_dX);
modelStruct.modelPar_WVBCI=mod.WVBCI;
modelStruct.d_modelPar_WVBCI_dX=0.0;
modelStruct.modelPar_given_WVBCI=mod.given("WVBCI");
modParamMap["WVBCI"] = &(modelStruct.d_modelPar_WVBCI_dX);
modelStruct.modelPar_WBETA0=mod.WBETA0;
modelStruct.d_modelPar_WBETA0_dX=0.0;
modelStruct.modelPar_given_WBETA0=mod.given("WBETA0");
modParamMap["WBETA0"] = &(modelStruct.d_modelPar_WBETA0_dX);
modelStruct.modelPar_WBETA1=mod.WBETA1;
modelStruct.d_modelPar_WBETA1_dX=0.0;
modelStruct.modelPar_given_WBETA1=mod.given("WBETA1");
modParamMap["WBETA1"] = &(modelStruct.d_modelPar_WBETA1_dX);
modelStruct.modelPar_WBETA2=mod.WBETA2;
modelStruct.d_modelPar_WBETA2_dX=0.0;
modelStruct.modelPar_given_WBETA2=mod.given("WBETA2");
modParamMap["WBETA2"] = &(modelStruct.d_modelPar_WBETA2_dX);
modelStruct.modelPar_WVDSATII0=mod.WVDSATII0;
modelStruct.d_modelPar_WVDSATII0_dX=0.0;
modelStruct.modelPar_given_WVDSATII0=mod.given("WVDSATII0");
modParamMap["WVDSATII0"] = &(modelStruct.d_modelPar_WVDSATII0_dX);
modelStruct.modelPar_WLII=mod.WLII;
modelStruct.d_modelPar_WLII_dX=0.0;
modelStruct.modelPar_given_WLII=mod.given("WLII");
modParamMap["WLII"] = &(modelStruct.d_modelPar_WLII_dX);
modelStruct.modelPar_WESATII=mod.WESATII;
modelStruct.d_modelPar_WESATII_dX=0.0;
modelStruct.modelPar_given_WESATII=mod.given("WESATII");
modParamMap["WESATII"] = &(modelStruct.d_modelPar_WESATII_dX);
modelStruct.modelPar_WSII0=mod.WSII0;
modelStruct.d_modelPar_WSII0_dX=0.0;
modelStruct.modelPar_given_WSII0=mod.given("WSII0");
modParamMap["WSII0"] = &(modelStruct.d_modelPar_WSII0_dX);
modelStruct.modelPar_WSII1=mod.WSII1;
modelStruct.d_modelPar_WSII1_dX=0.0;
modelStruct.modelPar_given_WSII1=mod.given("WSII1");
modParamMap["WSII1"] = &(modelStruct.d_modelPar_WSII1_dX);
modelStruct.modelPar_WSII2=mod.WSII2;
modelStruct.d_modelPar_WSII2_dX=0.0;
modelStruct.modelPar_given_WSII2=mod.given("WSII2");
modParamMap["WSII2"] = &(modelStruct.d_modelPar_WSII2_dX);
modelStruct.modelPar_WSIID=mod.WSIID;
modelStruct.d_modelPar_WSIID_dX=0.0;
modelStruct.modelPar_given_WSIID=mod.given("WSIID");
modParamMap["WSIID"] = &(modelStruct.d_modelPar_WSIID_dX);
modelStruct.modelPar_WAGIDL=mod.WAGIDL;
modelStruct.d_modelPar_WAGIDL_dX=0.0;
modelStruct.modelPar_given_WAGIDL=mod.given("WAGIDL");
modParamMap["WAGIDL"] = &(modelStruct.d_modelPar_WAGIDL_dX);
modelStruct.modelPar_WBGIDL=mod.WBGIDL;
modelStruct.d_modelPar_WBGIDL_dX=0.0;
modelStruct.modelPar_given_WBGIDL=mod.given("WBGIDL");
modParamMap["WBGIDL"] = &(modelStruct.d_modelPar_WBGIDL_dX);
modelStruct.modelPar_WCGIDL=mod.WCGIDL;
modelStruct.d_modelPar_WCGIDL_dX=0.0;
modelStruct.modelPar_given_WCGIDL=mod.given("WCGIDL");
modParamMap["WCGIDL"] = &(modelStruct.d_modelPar_WCGIDL_dX);
modelStruct.modelPar_WRGIDL=mod.WRGIDL;
modelStruct.d_modelPar_WRGIDL_dX=0.0;
modelStruct.modelPar_given_WRGIDL=mod.given("WRGIDL");
modParamMap["WRGIDL"] = &(modelStruct.d_modelPar_WRGIDL_dX);
modelStruct.modelPar_WKGIDL=mod.WKGIDL;
modelStruct.d_modelPar_WKGIDL_dX=0.0;
modelStruct.modelPar_given_WKGIDL=mod.given("WKGIDL");
modParamMap["WKGIDL"] = &(modelStruct.d_modelPar_WKGIDL_dX);
modelStruct.modelPar_WFGIDL=mod.WFGIDL;
modelStruct.d_modelPar_WFGIDL_dX=0.0;
modelStruct.modelPar_given_WFGIDL=mod.given("WFGIDL");
modParamMap["WFGIDL"] = &(modelStruct.d_modelPar_WFGIDL_dX);
modelStruct.modelPar_WAGISL=mod.WAGISL;
modelStruct.d_modelPar_WAGISL_dX=0.0;
modelStruct.modelPar_given_WAGISL=mod.given("WAGISL");
modParamMap["WAGISL"] = &(modelStruct.d_modelPar_WAGISL_dX);
modelStruct.modelPar_WBGISL=mod.WBGISL;
modelStruct.d_modelPar_WBGISL_dX=0.0;
modelStruct.modelPar_given_WBGISL=mod.given("WBGISL");
modParamMap["WBGISL"] = &(modelStruct.d_modelPar_WBGISL_dX);
modelStruct.modelPar_WCGISL=mod.WCGISL;
modelStruct.d_modelPar_WCGISL_dX=0.0;
modelStruct.modelPar_given_WCGISL=mod.given("WCGISL");
modParamMap["WCGISL"] = &(modelStruct.d_modelPar_WCGISL_dX);
modelStruct.modelPar_WRGISL=mod.WRGISL;
modelStruct.d_modelPar_WRGISL_dX=0.0;
modelStruct.modelPar_given_WRGISL=mod.given("WRGISL");
modParamMap["WRGISL"] = &(modelStruct.d_modelPar_WRGISL_dX);
modelStruct.modelPar_WKGISL=mod.WKGISL;
modelStruct.d_modelPar_WKGISL_dX=0.0;
modelStruct.modelPar_given_WKGISL=mod.given("WKGISL");
modParamMap["WKGISL"] = &(modelStruct.d_modelPar_WKGISL_dX);
modelStruct.modelPar_WFGISL=mod.WFGISL;
modelStruct.d_modelPar_WFGISL_dX=0.0;
modelStruct.modelPar_given_WFGISL=mod.given("WFGISL");
modParamMap["WFGISL"] = &(modelStruct.d_modelPar_WFGISL_dX);
modelStruct.modelPar_WNTUN=mod.WNTUN;
modelStruct.d_modelPar_WNTUN_dX=0.0;
modelStruct.modelPar_given_WNTUN=mod.given("WNTUN");
modParamMap["WNTUN"] = &(modelStruct.d_modelPar_WNTUN_dX);
modelStruct.modelPar_WNTUND=mod.WNTUND;
modelStruct.d_modelPar_WNTUND_dX=0.0;
modelStruct.modelPar_given_WNTUND=mod.given("WNTUND");
modParamMap["WNTUND"] = &(modelStruct.d_modelPar_WNTUND_dX);
modelStruct.modelPar_WNDIODE=mod.WNDIODE;
modelStruct.d_modelPar_WNDIODE_dX=0.0;
modelStruct.modelPar_given_WNDIODE=mod.given("WNDIODE");
modParamMap["WNDIODE"] = &(modelStruct.d_modelPar_WNDIODE_dX);
modelStruct.modelPar_WNDIODED=mod.WNDIODED;
modelStruct.d_modelPar_WNDIODED_dX=0.0;
modelStruct.modelPar_given_WNDIODED=mod.given("WNDIODED");
modParamMap["WNDIODED"] = &(modelStruct.d_modelPar_WNDIODED_dX);
modelStruct.modelPar_WNRECF0=mod.WNRECF0;
modelStruct.d_modelPar_WNRECF0_dX=0.0;
modelStruct.modelPar_given_WNRECF0=mod.given("WNRECF0");
modParamMap["WNRECF0"] = &(modelStruct.d_modelPar_WNRECF0_dX);
modelStruct.modelPar_WNRECF0D=mod.WNRECF0D;
modelStruct.d_modelPar_WNRECF0D_dX=0.0;
modelStruct.modelPar_given_WNRECF0D=mod.given("WNRECF0D");
modParamMap["WNRECF0D"] = &(modelStruct.d_modelPar_WNRECF0D_dX);
modelStruct.modelPar_WNRECR0=mod.WNRECR0;
modelStruct.d_modelPar_WNRECR0_dX=0.0;
modelStruct.modelPar_given_WNRECR0=mod.given("WNRECR0");
modParamMap["WNRECR0"] = &(modelStruct.d_modelPar_WNRECR0_dX);
modelStruct.modelPar_WNRECR0D=mod.WNRECR0D;
modelStruct.d_modelPar_WNRECR0D_dX=0.0;
modelStruct.modelPar_given_WNRECR0D=mod.given("WNRECR0D");
modParamMap["WNRECR0D"] = &(modelStruct.d_modelPar_WNRECR0D_dX);
modelStruct.modelPar_WISBJT=mod.WISBJT;
modelStruct.d_modelPar_WISBJT_dX=0.0;
modelStruct.modelPar_given_WISBJT=mod.given("WISBJT");
modParamMap["WISBJT"] = &(modelStruct.d_modelPar_WISBJT_dX);
modelStruct.modelPar_WIDBJT=mod.WIDBJT;
modelStruct.d_modelPar_WIDBJT_dX=0.0;
modelStruct.modelPar_given_WIDBJT=mod.given("WIDBJT");
modParamMap["WIDBJT"] = &(modelStruct.d_modelPar_WIDBJT_dX);
modelStruct.modelPar_WISDIF=mod.WISDIF;
modelStruct.d_modelPar_WISDIF_dX=0.0;
modelStruct.modelPar_given_WISDIF=mod.given("WISDIF");
modParamMap["WISDIF"] = &(modelStruct.d_modelPar_WISDIF_dX);
modelStruct.modelPar_WIDDIF=mod.WIDDIF;
modelStruct.d_modelPar_WIDDIF_dX=0.0;
modelStruct.modelPar_given_WIDDIF=mod.given("WIDDIF");
modParamMap["WIDDIF"] = &(modelStruct.d_modelPar_WIDDIF_dX);
modelStruct.modelPar_WISREC=mod.WISREC;
modelStruct.d_modelPar_WISREC_dX=0.0;
modelStruct.modelPar_given_WISREC=mod.given("WISREC");
modParamMap["WISREC"] = &(modelStruct.d_modelPar_WISREC_dX);
modelStruct.modelPar_WIDREC=mod.WIDREC;
modelStruct.d_modelPar_WIDREC_dX=0.0;
modelStruct.modelPar_given_WIDREC=mod.given("WIDREC");
modParamMap["WIDREC"] = &(modelStruct.d_modelPar_WIDREC_dX);
modelStruct.modelPar_WISTUN=mod.WISTUN;
modelStruct.d_modelPar_WISTUN_dX=0.0;
modelStruct.modelPar_given_WISTUN=mod.given("WISTUN");
modParamMap["WISTUN"] = &(modelStruct.d_modelPar_WISTUN_dX);
modelStruct.modelPar_WIDTUN=mod.WIDTUN;
modelStruct.d_modelPar_WIDTUN_dX=0.0;
modelStruct.modelPar_given_WIDTUN=mod.given("WIDTUN");
modParamMap["WIDTUN"] = &(modelStruct.d_modelPar_WIDTUN_dX);
modelStruct.modelPar_WVREC0=mod.WVREC0;
modelStruct.d_modelPar_WVREC0_dX=0.0;
modelStruct.modelPar_given_WVREC0=mod.given("WVREC0");
modParamMap["WVREC0"] = &(modelStruct.d_modelPar_WVREC0_dX);
modelStruct.modelPar_WVREC0D=mod.WVREC0D;
modelStruct.d_modelPar_WVREC0D_dX=0.0;
modelStruct.modelPar_given_WVREC0D=mod.given("WVREC0D");
modParamMap["WVREC0D"] = &(modelStruct.d_modelPar_WVREC0D_dX);
modelStruct.modelPar_WVTUN0=mod.WVTUN0;
modelStruct.d_modelPar_WVTUN0_dX=0.0;
modelStruct.modelPar_given_WVTUN0=mod.given("WVTUN0");
modParamMap["WVTUN0"] = &(modelStruct.d_modelPar_WVTUN0_dX);
modelStruct.modelPar_WVTUN0D=mod.WVTUN0D;
modelStruct.d_modelPar_WVTUN0D_dX=0.0;
modelStruct.modelPar_given_WVTUN0D=mod.given("WVTUN0D");
modParamMap["WVTUN0D"] = &(modelStruct.d_modelPar_WVTUN0D_dX);
modelStruct.modelPar_WNBJT=mod.WNBJT;
modelStruct.d_modelPar_WNBJT_dX=0.0;
modelStruct.modelPar_given_WNBJT=mod.given("WNBJT");
modParamMap["WNBJT"] = &(modelStruct.d_modelPar_WNBJT_dX);
modelStruct.modelPar_WLBJT0=mod.WLBJT0;
modelStruct.d_modelPar_WLBJT0_dX=0.0;
modelStruct.modelPar_given_WLBJT0=mod.given("WLBJT0");
modParamMap["WLBJT0"] = &(modelStruct.d_modelPar_WLBJT0_dX);
modelStruct.modelPar_WVABJT=mod.WVABJT;
modelStruct.d_modelPar_WVABJT_dX=0.0;
modelStruct.modelPar_given_WVABJT=mod.given("WVABJT");
modParamMap["WVABJT"] = &(modelStruct.d_modelPar_WVABJT_dX);
modelStruct.modelPar_WAELY=mod.WAELY;
modelStruct.d_modelPar_WAELY_dX=0.0;
modelStruct.modelPar_given_WAELY=mod.given("WAELY");
modParamMap["WAELY"] = &(modelStruct.d_modelPar_WAELY_dX);
modelStruct.modelPar_WAHLI=mod.WAHLI;
modelStruct.d_modelPar_WAHLI_dX=0.0;
modelStruct.modelPar_given_WAHLI=mod.given("WAHLI");
modParamMap["WAHLI"] = &(modelStruct.d_modelPar_WAHLI_dX);
modelStruct.modelPar_WAHLID=mod.WAHLID;
modelStruct.d_modelPar_WAHLID_dX=0.0;
modelStruct.modelPar_given_WAHLID=mod.given("WAHLID");
modParamMap["WAHLID"] = &(modelStruct.d_modelPar_WAHLID_dX);
modelStruct.modelPar_WVSDFB=mod.WVSDFB;
modelStruct.d_modelPar_WVSDFB_dX=0.0;
modelStruct.modelPar_given_WVSDFB=mod.given("WVSDFB");
modParamMap["WVSDFB"] = &(modelStruct.d_modelPar_WVSDFB_dX);
modelStruct.modelPar_WVSDTH=mod.WVSDTH;
modelStruct.d_modelPar_WVSDTH_dX=0.0;
modelStruct.modelPar_given_WVSDTH=mod.given("WVSDTH");
modParamMap["WVSDTH"] = &(modelStruct.d_modelPar_WVSDTH_dX);
modelStruct.modelPar_WDELVT=mod.WDELVT;
modelStruct.d_modelPar_WDELVT_dX=0.0;
modelStruct.modelPar_given_WDELVT=mod.given("WDELVT");
modParamMap["WDELVT"] = &(modelStruct.d_modelPar_WDELVT_dX);
modelStruct.modelPar_WACDE=mod.WACDE;
modelStruct.d_modelPar_WACDE_dX=0.0;
modelStruct.modelPar_given_WACDE=mod.given("WACDE");
modParamMap["WACDE"] = &(modelStruct.d_modelPar_WACDE_dX);
modelStruct.modelPar_WMOIN=mod.WMOIN;
modelStruct.d_modelPar_WMOIN_dX=0.0;
modelStruct.modelPar_given_WMOIN=mod.given("WMOIN");
modParamMap["WMOIN"] = &(modelStruct.d_modelPar_WMOIN_dX);
modelStruct.modelPar_WNOFF=mod.WNOFF;
modelStruct.d_modelPar_WNOFF_dX=0.0;
modelStruct.modelPar_given_WNOFF=mod.given("WNOFF");
modParamMap["WNOFF"] = &(modelStruct.d_modelPar_WNOFF_dX);
modelStruct.modelPar_WNOFF2=mod.WNOFF2;
modelStruct.d_modelPar_WNOFF2_dX=0.0;
modelStruct.modelPar_given_WNOFF2=mod.given("WNOFF2");
modParamMap["WNOFF2"] = &(modelStruct.d_modelPar_WNOFF2_dX);
modelStruct.modelPar_WXRCRG1=mod.WXRCRG1;
modelStruct.d_modelPar_WXRCRG1_dX=0.0;
modelStruct.modelPar_given_WXRCRG1=mod.given("WXRCRG1");
modParamMap["WXRCRG1"] = &(modelStruct.d_modelPar_WXRCRG1_dX);
modelStruct.modelPar_WXRCRG2=mod.WXRCRG2;
modelStruct.d_modelPar_WXRCRG2_dX=0.0;
modelStruct.modelPar_given_WXRCRG2=mod.given("WXRCRG2");
modParamMap["WXRCRG2"] = &(modelStruct.d_modelPar_WXRCRG2_dX);
modelStruct.modelPar_WVBSA=mod.WVBSA;
modelStruct.d_modelPar_WVBSA_dX=0.0;
modelStruct.modelPar_given_WVBSA=mod.given("WVBSA");
modParamMap["WVBSA"] = &(modelStruct.d_modelPar_WVBSA_dX);
modelStruct.modelPar_WVSCE=mod.WVSCE;
modelStruct.d_modelPar_WVSCE_dX=0.0;
modelStruct.modelPar_given_WVSCE=mod.given("WVSCE");
modParamMap["WVSCE"] = &(modelStruct.d_modelPar_WVSCE_dX);
modelStruct.modelPar_WCDSBS=mod.WCDSBS;
modelStruct.d_modelPar_WCDSBS_dX=0.0;
modelStruct.modelPar_given_WCDSBS=mod.given("WCDSBS");
modParamMap["WCDSBS"] = &(modelStruct.d_modelPar_WCDSBS_dX);
modelStruct.modelPar_WNOFFFD=mod.WNOFFFD;
modelStruct.d_modelPar_WNOFFFD_dX=0.0;
modelStruct.modelPar_given_WNOFFFD=mod.given("WNOFFFD");
modParamMap["WNOFFFD"] = &(modelStruct.d_modelPar_WNOFFFD_dX);
modelStruct.modelPar_WVOFFFD=mod.WVOFFFD;
modelStruct.d_modelPar_WVOFFFD_dX=0.0;
modelStruct.modelPar_given_WVOFFFD=mod.given("WVOFFFD");
modParamMap["WVOFFFD"] = &(modelStruct.d_modelPar_WVOFFFD_dX);
modelStruct.modelPar_WK1B=mod.WK1B;
modelStruct.d_modelPar_WK1B_dX=0.0;
modelStruct.modelPar_given_WK1B=mod.given("WK1B");
modParamMap["WK1B"] = &(modelStruct.d_modelPar_WK1B_dX);
modelStruct.modelPar_WK2B=mod.WK2B;
modelStruct.d_modelPar_WK2B_dX=0.0;
modelStruct.modelPar_given_WK2B=mod.given("WK2B");
modParamMap["WK2B"] = &(modelStruct.d_modelPar_WK2B_dX);
modelStruct.modelPar_WDK2B=mod.WDK2B;
modelStruct.d_modelPar_WDK2B_dX=0.0;
modelStruct.modelPar_given_WDK2B=mod.given("WDK2B");
modParamMap["WDK2B"] = &(modelStruct.d_modelPar_WDK2B_dX);
modelStruct.modelPar_WDVBD0=mod.WDVBD0;
modelStruct.d_modelPar_WDVBD0_dX=0.0;
modelStruct.modelPar_given_WDVBD0=mod.given("WDVBD0");
modParamMap["WDVBD0"] = &(modelStruct.d_modelPar_WDVBD0_dX);
modelStruct.modelPar_WDVBD1=mod.WDVBD1;
modelStruct.d_modelPar_WDVBD1_dX=0.0;
modelStruct.modelPar_given_WDVBD1=mod.given("WDVBD1");
modParamMap["WDVBD1"] = &(modelStruct.d_modelPar_WDVBD1_dX);
modelStruct.modelPar_WMOINFD=mod.WMOINFD;
modelStruct.d_modelPar_WMOINFD_dX=0.0;
modelStruct.modelPar_given_WMOINFD=mod.given("WMOINFD");
modParamMap["WMOINFD"] = &(modelStruct.d_modelPar_WMOINFD_dX);
modelStruct.modelPar_WVBS0PD=mod.WVBS0PD;
modelStruct.d_modelPar_WVBS0PD_dX=0.0;
modelStruct.modelPar_given_WVBS0PD=mod.given("WVBS0PD");
modParamMap["WVBS0PD"] = &(modelStruct.d_modelPar_WVBS0PD_dX);
modelStruct.modelPar_WVBS0FD=mod.WVBS0FD;
modelStruct.d_modelPar_WVBS0FD_dX=0.0;
modelStruct.modelPar_given_WVBS0FD=mod.given("WVBS0FD");
modParamMap["WVBS0FD"] = &(modelStruct.d_modelPar_WVBS0FD_dX);
modelStruct.modelPar_PXJ=mod.PXJ;
modelStruct.d_modelPar_PXJ_dX=0.0;
modelStruct.modelPar_given_PXJ=mod.given("PXJ");
modParamMap["PXJ"] = &(modelStruct.d_modelPar_PXJ_dX);
modelStruct.modelPar_PALPHAGB1=mod.PALPHAGB1;
modelStruct.d_modelPar_PALPHAGB1_dX=0.0;
modelStruct.modelPar_given_PALPHAGB1=mod.given("PALPHAGB1");
modParamMap["PALPHAGB1"] = &(modelStruct.d_modelPar_PALPHAGB1_dX);
modelStruct.modelPar_PBETAGB1=mod.PBETAGB1;
modelStruct.d_modelPar_PBETAGB1_dX=0.0;
modelStruct.modelPar_given_PBETAGB1=mod.given("PBETAGB1");
modParamMap["PBETAGB1"] = &(modelStruct.d_modelPar_PBETAGB1_dX);
modelStruct.modelPar_PALPHAGB2=mod.PALPHAGB2;
modelStruct.d_modelPar_PALPHAGB2_dX=0.0;
modelStruct.modelPar_given_PALPHAGB2=mod.given("PALPHAGB2");
modParamMap["PALPHAGB2"] = &(modelStruct.d_modelPar_PALPHAGB2_dX);
modelStruct.modelPar_PBETAGB2=mod.PBETAGB2;
modelStruct.d_modelPar_PBETAGB2_dX=0.0;
modelStruct.modelPar_given_PBETAGB2=mod.given("PBETAGB2");
modParamMap["PBETAGB2"] = &(modelStruct.d_modelPar_PBETAGB2_dX);
modelStruct.modelPar_PAIGBCP2=mod.PAIGBCP2;
modelStruct.d_modelPar_PAIGBCP2_dX=0.0;
modelStruct.modelPar_given_PAIGBCP2=mod.given("PAIGBCP2");
modParamMap["PAIGBCP2"] = &(modelStruct.d_modelPar_PAIGBCP2_dX);
modelStruct.modelPar_PBIGBCP2=mod.PBIGBCP2;
modelStruct.d_modelPar_PBIGBCP2_dX=0.0;
modelStruct.modelPar_given_PBIGBCP2=mod.given("PBIGBCP2");
modParamMap["PBIGBCP2"] = &(modelStruct.d_modelPar_PBIGBCP2_dX);
modelStruct.modelPar_PCIGBCP2=mod.PCIGBCP2;
modelStruct.d_modelPar_PCIGBCP2_dX=0.0;
modelStruct.modelPar_given_PCIGBCP2=mod.given("PCIGBCP2");
modParamMap["PCIGBCP2"] = &(modelStruct.d_modelPar_PCIGBCP2_dX);
modelStruct.modelPar_PCGSL=mod.PCGSL;
modelStruct.d_modelPar_PCGSL_dX=0.0;
modelStruct.modelPar_given_PCGSL=mod.given("PCGSL");
modParamMap["PCGSL"] = &(modelStruct.d_modelPar_PCGSL_dX);
modelStruct.modelPar_PCGDL=mod.PCGDL;
modelStruct.d_modelPar_PCGDL_dX=0.0;
modelStruct.modelPar_given_PCGDL=mod.given("PCGDL");
modParamMap["PCGDL"] = &(modelStruct.d_modelPar_PCGDL_dX);
modelStruct.modelPar_PCKAPPA=mod.PCKAPPA;
modelStruct.d_modelPar_PCKAPPA_dX=0.0;
modelStruct.modelPar_given_PCKAPPA=mod.given("PCKAPPA");
modParamMap["PCKAPPA"] = &(modelStruct.d_modelPar_PCKAPPA_dX);
modelStruct.modelPar_PNDIF=mod.PNDIF;
modelStruct.d_modelPar_PNDIF_dX=0.0;
modelStruct.modelPar_given_PNDIF=mod.given("PNDIF");
modParamMap["PNDIF"] = &(modelStruct.d_modelPar_PNDIF_dX);
modelStruct.modelPar_PUTE=mod.PUTE;
modelStruct.d_modelPar_PUTE_dX=0.0;
modelStruct.modelPar_given_PUTE=mod.given("PUTE");
modParamMap["PUTE"] = &(modelStruct.d_modelPar_PUTE_dX);
modelStruct.modelPar_PKT1=mod.PKT1;
modelStruct.d_modelPar_PKT1_dX=0.0;
modelStruct.modelPar_given_PKT1=mod.given("PKT1");
modParamMap["PKT1"] = &(modelStruct.d_modelPar_PKT1_dX);
modelStruct.modelPar_PKT1L=mod.PKT1L;
modelStruct.d_modelPar_PKT1L_dX=0.0;
modelStruct.modelPar_given_PKT1L=mod.given("PKT1L");
modParamMap["PKT1L"] = &(modelStruct.d_modelPar_PKT1L_dX);
modelStruct.modelPar_PKT2=mod.PKT2;
modelStruct.d_modelPar_PKT2_dX=0.0;
modelStruct.modelPar_given_PKT2=mod.given("PKT2");
modParamMap["PKT2"] = &(modelStruct.d_modelPar_PKT2_dX);
modelStruct.modelPar_PUA1=mod.PUA1;
modelStruct.d_modelPar_PUA1_dX=0.0;
modelStruct.modelPar_given_PUA1=mod.given("PUA1");
modParamMap["PUA1"] = &(modelStruct.d_modelPar_PUA1_dX);
modelStruct.modelPar_PUB1=mod.PUB1;
modelStruct.d_modelPar_PUB1_dX=0.0;
modelStruct.modelPar_given_PUB1=mod.given("PUB1");
modParamMap["PUB1"] = &(modelStruct.d_modelPar_PUB1_dX);
modelStruct.modelPar_PUC1=mod.PUC1;
modelStruct.d_modelPar_PUC1_dX=0.0;
modelStruct.modelPar_given_PUC1=mod.given("PUC1");
modParamMap["PUC1"] = &(modelStruct.d_modelPar_PUC1_dX);
modelStruct.modelPar_PAT=mod.PAT;
modelStruct.d_modelPar_PAT_dX=0.0;
modelStruct.modelPar_given_PAT=mod.given("PAT");
modParamMap["PAT"] = &(modelStruct.d_modelPar_PAT_dX);
modelStruct.modelPar_PPRT=mod.PPRT;
modelStruct.d_modelPar_PPRT_dX=0.0;
modelStruct.modelPar_given_PPRT=mod.given("PPRT");
modParamMap["PPRT"] = &(modelStruct.d_modelPar_PPRT_dX);
modelStruct.modelPar_PNTRECF=mod.PNTRECF;
modelStruct.d_modelPar_PNTRECF_dX=0.0;
modelStruct.modelPar_given_PNTRECF=mod.given("PNTRECF");
modParamMap["PNTRECF"] = &(modelStruct.d_modelPar_PNTRECF_dX);
modelStruct.modelPar_PNTRECR=mod.PNTRECR;
modelStruct.d_modelPar_PNTRECR_dX=0.0;
modelStruct.modelPar_given_PNTRECR=mod.given("PNTRECR");
modParamMap["PNTRECR"] = &(modelStruct.d_modelPar_PNTRECR_dX);
modelStruct.modelPar_PXBJT=mod.PXBJT;
modelStruct.d_modelPar_PXBJT_dX=0.0;
modelStruct.modelPar_given_PXBJT=mod.given("PXBJT");
modParamMap["PXBJT"] = &(modelStruct.d_modelPar_PXBJT_dX);
modelStruct.modelPar_PXDIF=mod.PXDIF;
modelStruct.d_modelPar_PXDIF_dX=0.0;
modelStruct.modelPar_given_PXDIF=mod.given("PXDIF");
modParamMap["PXDIF"] = &(modelStruct.d_modelPar_PXDIF_dX);
modelStruct.modelPar_PXREC=mod.PXREC;
modelStruct.d_modelPar_PXREC_dX=0.0;
modelStruct.modelPar_given_PXREC=mod.given("PXREC");
modParamMap["PXREC"] = &(modelStruct.d_modelPar_PXREC_dX);
modelStruct.modelPar_PXTUN=mod.PXTUN;
modelStruct.d_modelPar_PXTUN_dX=0.0;
modelStruct.modelPar_given_PXTUN=mod.given("PXTUN");
modParamMap["PXTUN"] = &(modelStruct.d_modelPar_PXTUN_dX);
modelStruct.modelPar_PXDIFD=mod.PXDIFD;
modelStruct.d_modelPar_PXDIFD_dX=0.0;
modelStruct.modelPar_given_PXDIFD=mod.given("PXDIFD");
modParamMap["PXDIFD"] = &(modelStruct.d_modelPar_PXDIFD_dX);
modelStruct.modelPar_PXRECD=mod.PXRECD;
modelStruct.d_modelPar_PXRECD_dX=0.0;
modelStruct.modelPar_given_PXRECD=mod.given("PXRECD");
modParamMap["PXRECD"] = &(modelStruct.d_modelPar_PXRECD_dX);
modelStruct.modelPar_PXTUND=mod.PXTUND;
modelStruct.d_modelPar_PXTUND_dX=0.0;
modelStruct.modelPar_given_PXTUND=mod.given("PXTUND");
modParamMap["PXTUND"] = &(modelStruct.d_modelPar_PXTUND_dX);
modelStruct.modelPar_PAIGC=mod.PAIGC;
modelStruct.d_modelPar_PAIGC_dX=0.0;
modelStruct.modelPar_given_PAIGC=mod.given("PAIGC");
modParamMap["PAIGC"] = &(modelStruct.d_modelPar_PAIGC_dX);
modelStruct.modelPar_PBIGC=mod.PBIGC;
modelStruct.d_modelPar_PBIGC_dX=0.0;
modelStruct.modelPar_given_PBIGC=mod.given("PBIGC");
modParamMap["PBIGC"] = &(modelStruct.d_modelPar_PBIGC_dX);
modelStruct.modelPar_PCIGC=mod.PCIGC;
modelStruct.d_modelPar_PCIGC_dX=0.0;
modelStruct.modelPar_given_PCIGC=mod.given("PCIGC");
modParamMap["PCIGC"] = &(modelStruct.d_modelPar_PCIGC_dX);
modelStruct.modelPar_PAIGSD=mod.PAIGSD;
modelStruct.d_modelPar_PAIGSD_dX=0.0;
modelStruct.modelPar_given_PAIGSD=mod.given("PAIGSD");
modParamMap["PAIGSD"] = &(modelStruct.d_modelPar_PAIGSD_dX);
modelStruct.modelPar_PBIGSD=mod.PBIGSD;
modelStruct.d_modelPar_PBIGSD_dX=0.0;
modelStruct.modelPar_given_PBIGSD=mod.given("PBIGSD");
modParamMap["PBIGSD"] = &(modelStruct.d_modelPar_PBIGSD_dX);
modelStruct.modelPar_PCIGSD=mod.PCIGSD;
modelStruct.d_modelPar_PCIGSD_dX=0.0;
modelStruct.modelPar_given_PCIGSD=mod.given("PCIGSD");
modParamMap["PCIGSD"] = &(modelStruct.d_modelPar_PCIGSD_dX);
modelStruct.modelPar_PNIGC=mod.PNIGC;
modelStruct.d_modelPar_PNIGC_dX=0.0;
modelStruct.modelPar_given_PNIGC=mod.given("PNIGC");
modParamMap["PNIGC"] = &(modelStruct.d_modelPar_PNIGC_dX);
modelStruct.modelPar_PPIGCD=mod.PPIGCD;
modelStruct.d_modelPar_PPIGCD_dX=0.0;
modelStruct.modelPar_given_PPIGCD=mod.given("PPIGCD");
modParamMap["PPIGCD"] = &(modelStruct.d_modelPar_PPIGCD_dX);
modelStruct.modelPar_PPOXEDGE=mod.PPOXEDGE;
modelStruct.d_modelPar_PPOXEDGE_dX=0.0;
modelStruct.modelPar_given_PPOXEDGE=mod.given("PPOXEDGE");
modParamMap["PPOXEDGE"] = &(modelStruct.d_modelPar_PPOXEDGE_dX);
modelStruct.modelPar_PNCH=mod.PNCH;
modelStruct.d_modelPar_PNCH_dX=0.0;
modelStruct.modelPar_given_PNCH=mod.given("PNCH");
modParamMap["PNCH"] = &(modelStruct.d_modelPar_PNCH_dX);
modelStruct.modelPar_PNSUB=mod.PNSUB;
modelStruct.d_modelPar_PNSUB_dX=0.0;
modelStruct.modelPar_given_PNSUB=mod.given("PNSUB");
modParamMap["PNSUB"] = &(modelStruct.d_modelPar_PNSUB_dX);
modelStruct.modelPar_PNSD=mod.PNSD;
modelStruct.d_modelPar_PNSD_dX=0.0;
modelStruct.modelPar_given_PNSD=mod.given("PNSD");
modParamMap["PNSD"] = &(modelStruct.d_modelPar_PNSD_dX);
modelStruct.modelPar_PNGATE=mod.PNGATE;
modelStruct.d_modelPar_PNGATE_dX=0.0;
modelStruct.modelPar_given_PNGATE=mod.given("PNGATE");
modParamMap["PNGATE"] = &(modelStruct.d_modelPar_PNGATE_dX);
modelStruct.modelPar_PVTH0=mod.PVTH0;
modelStruct.d_modelPar_PVTH0_dX=0.0;
modelStruct.modelPar_given_PVTH0=mod.given("PVTH0");
modParamMap["PVTH0"] = &(modelStruct.d_modelPar_PVTH0_dX);
modelStruct.modelPar_PVFB=mod.PVFB;
modelStruct.d_modelPar_PVFB_dX=0.0;
modelStruct.modelPar_given_PVFB=mod.given("PVFB");
modParamMap["PVFB"] = &(modelStruct.d_modelPar_PVFB_dX);
modelStruct.modelPar_PK1=mod.PK1;
modelStruct.d_modelPar_PK1_dX=0.0;
modelStruct.modelPar_given_PK1=mod.given("PK1");
modParamMap["PK1"] = &(modelStruct.d_modelPar_PK1_dX);
modelStruct.modelPar_PK1W1=mod.PK1W1;
modelStruct.d_modelPar_PK1W1_dX=0.0;
modelStruct.modelPar_given_PK1W1=mod.given("PK1W1");
modParamMap["PK1W1"] = &(modelStruct.d_modelPar_PK1W1_dX);
modelStruct.modelPar_PK1W2=mod.PK1W2;
modelStruct.d_modelPar_PK1W2_dX=0.0;
modelStruct.modelPar_given_PK1W2=mod.given("PK1W2");
modParamMap["PK1W2"] = &(modelStruct.d_modelPar_PK1W2_dX);
modelStruct.modelPar_PK2=mod.PK2;
modelStruct.d_modelPar_PK2_dX=0.0;
modelStruct.modelPar_given_PK2=mod.given("PK2");
modParamMap["PK2"] = &(modelStruct.d_modelPar_PK2_dX);
modelStruct.modelPar_PK3=mod.PK3;
modelStruct.d_modelPar_PK3_dX=0.0;
modelStruct.modelPar_given_PK3=mod.given("PK3");
modParamMap["PK3"] = &(modelStruct.d_modelPar_PK3_dX);
modelStruct.modelPar_PK3B=mod.PK3B;
modelStruct.d_modelPar_PK3B_dX=0.0;
modelStruct.modelPar_given_PK3B=mod.given("PK3B");
modParamMap["PK3B"] = &(modelStruct.d_modelPar_PK3B_dX);
modelStruct.modelPar_PKB1=mod.PKB1;
modelStruct.d_modelPar_PKB1_dX=0.0;
modelStruct.modelPar_given_PKB1=mod.given("PKB1");
modParamMap["PKB1"] = &(modelStruct.d_modelPar_PKB1_dX);
modelStruct.modelPar_PW0=mod.PW0;
modelStruct.d_modelPar_PW0_dX=0.0;
modelStruct.modelPar_given_PW0=mod.given("PW0");
modParamMap["PW0"] = &(modelStruct.d_modelPar_PW0_dX);
modelStruct.modelPar_PLPEB=mod.PLPEB;
modelStruct.d_modelPar_PLPEB_dX=0.0;
modelStruct.modelPar_given_PLPEB=mod.given("PLPEB");
modParamMap["PLPEB"] = &(modelStruct.d_modelPar_PLPEB_dX);
modelStruct.modelPar_PDVT0=mod.PDVT0;
modelStruct.d_modelPar_PDVT0_dX=0.0;
modelStruct.modelPar_given_PDVT0=mod.given("PDVT0");
modParamMap["PDVT0"] = &(modelStruct.d_modelPar_PDVT0_dX);
modelStruct.modelPar_PDVT1=mod.PDVT1;
modelStruct.d_modelPar_PDVT1_dX=0.0;
modelStruct.modelPar_given_PDVT1=mod.given("PDVT1");
modParamMap["PDVT1"] = &(modelStruct.d_modelPar_PDVT1_dX);
modelStruct.modelPar_PDVT2=mod.PDVT2;
modelStruct.d_modelPar_PDVT2_dX=0.0;
modelStruct.modelPar_given_PDVT2=mod.given("PDVT2");
modParamMap["PDVT2"] = &(modelStruct.d_modelPar_PDVT2_dX);
modelStruct.modelPar_PDVT0W=mod.PDVT0W;
modelStruct.d_modelPar_PDVT0W_dX=0.0;
modelStruct.modelPar_given_PDVT0W=mod.given("PDVT0W");
modParamMap["PDVT0W"] = &(modelStruct.d_modelPar_PDVT0W_dX);
modelStruct.modelPar_PDVT1W=mod.PDVT1W;
modelStruct.d_modelPar_PDVT1W_dX=0.0;
modelStruct.modelPar_given_PDVT1W=mod.given("PDVT1W");
modParamMap["PDVT1W"] = &(modelStruct.d_modelPar_PDVT1W_dX);
modelStruct.modelPar_PDVT2W=mod.PDVT2W;
modelStruct.d_modelPar_PDVT2W_dX=0.0;
modelStruct.modelPar_given_PDVT2W=mod.given("PDVT2W");
modParamMap["PDVT2W"] = &(modelStruct.d_modelPar_PDVT2W_dX);
modelStruct.modelPar_PU0=mod.PU0;
modelStruct.d_modelPar_PU0_dX=0.0;
modelStruct.modelPar_given_PU0=mod.given("PU0");
modParamMap["PU0"] = &(modelStruct.d_modelPar_PU0_dX);
modelStruct.modelPar_PEU=mod.PEU;
modelStruct.d_modelPar_PEU_dX=0.0;
modelStruct.modelPar_given_PEU=mod.given("PEU");
modParamMap["PEU"] = &(modelStruct.d_modelPar_PEU_dX);
modelStruct.modelPar_PUA=mod.PUA;
modelStruct.d_modelPar_PUA_dX=0.0;
modelStruct.modelPar_given_PUA=mod.given("PUA");
modParamMap["PUA"] = &(modelStruct.d_modelPar_PUA_dX);
modelStruct.modelPar_PUB=mod.PUB;
modelStruct.d_modelPar_PUB_dX=0.0;
modelStruct.modelPar_given_PUB=mod.given("PUB");
modParamMap["PUB"] = &(modelStruct.d_modelPar_PUB_dX);
modelStruct.modelPar_PUC=mod.PUC;
modelStruct.d_modelPar_PUC_dX=0.0;
modelStruct.modelPar_given_PUC=mod.given("PUC");
modParamMap["PUC"] = &(modelStruct.d_modelPar_PUC_dX);
modelStruct.modelPar_PUD=mod.PUD;
modelStruct.d_modelPar_PUD_dX=0.0;
modelStruct.modelPar_given_PUD=mod.given("PUD");
modParamMap["PUD"] = &(modelStruct.d_modelPar_PUD_dX);
modelStruct.modelPar_PUD1=mod.PUD1;
modelStruct.d_modelPar_PUD1_dX=0.0;
modelStruct.modelPar_given_PUD1=mod.given("PUD1");
modParamMap["PUD1"] = &(modelStruct.d_modelPar_PUD1_dX);
modelStruct.modelPar_PUCSTE=mod.PUCSTE;
modelStruct.d_modelPar_PUCSTE_dX=0.0;
modelStruct.modelPar_given_PUCSTE=mod.given("PUCSTE");
modParamMap["PUCSTE"] = &(modelStruct.d_modelPar_PUCSTE_dX);
modelStruct.modelPar_PUCS=mod.PUCS;
modelStruct.d_modelPar_PUCS_dX=0.0;
modelStruct.modelPar_given_PUCS=mod.given("PUCS");
modParamMap["PUCS"] = &(modelStruct.d_modelPar_PUCS_dX);
modelStruct.modelPar_PVSAT=mod.PVSAT;
modelStruct.d_modelPar_PVSAT_dX=0.0;
modelStruct.modelPar_given_PVSAT=mod.given("PVSAT");
modParamMap["PVSAT"] = &(modelStruct.d_modelPar_PVSAT_dX);
modelStruct.modelPar_PA0=mod.PA0;
modelStruct.d_modelPar_PA0_dX=0.0;
modelStruct.modelPar_given_PA0=mod.given("PA0");
modParamMap["PA0"] = &(modelStruct.d_modelPar_PA0_dX);
modelStruct.modelPar_PAGS=mod.PAGS;
modelStruct.d_modelPar_PAGS_dX=0.0;
modelStruct.modelPar_given_PAGS=mod.given("PAGS");
modParamMap["PAGS"] = &(modelStruct.d_modelPar_PAGS_dX);
modelStruct.modelPar_PB0=mod.PB0;
modelStruct.d_modelPar_PB0_dX=0.0;
modelStruct.modelPar_given_PB0=mod.given("PB0");
modParamMap["PB0"] = &(modelStruct.d_modelPar_PB0_dX);
modelStruct.modelPar_PB1=mod.PB1;
modelStruct.d_modelPar_PB1_dX=0.0;
modelStruct.modelPar_given_PB1=mod.given("PB1");
modParamMap["PB1"] = &(modelStruct.d_modelPar_PB1_dX);
modelStruct.modelPar_PKETA=mod.PKETA;
modelStruct.d_modelPar_PKETA_dX=0.0;
modelStruct.modelPar_given_PKETA=mod.given("PKETA");
modParamMap["PKETA"] = &(modelStruct.d_modelPar_PKETA_dX);
modelStruct.modelPar_PKETAS=mod.PKETAS;
modelStruct.d_modelPar_PKETAS_dX=0.0;
modelStruct.modelPar_given_PKETAS=mod.given("PKETAS");
modParamMap["PKETAS"] = &(modelStruct.d_modelPar_PKETAS_dX);
modelStruct.modelPar_PA1=mod.PA1;
modelStruct.d_modelPar_PA1_dX=0.0;
modelStruct.modelPar_given_PA1=mod.given("PA1");
modParamMap["PA1"] = &(modelStruct.d_modelPar_PA1_dX);
modelStruct.modelPar_PA2=mod.PA2;
modelStruct.d_modelPar_PA2_dX=0.0;
modelStruct.modelPar_given_PA2=mod.given("PA2");
modParamMap["PA2"] = &(modelStruct.d_modelPar_PA2_dX);
modelStruct.modelPar_PRDSW=mod.PRDSW;
modelStruct.d_modelPar_PRDSW_dX=0.0;
modelStruct.modelPar_given_PRDSW=mod.given("PRDSW");
modParamMap["PRDSW"] = &(modelStruct.d_modelPar_PRDSW_dX);
modelStruct.modelPar_PRSW=mod.PRSW;
modelStruct.d_modelPar_PRSW_dX=0.0;
modelStruct.modelPar_given_PRSW=mod.given("PRSW");
modParamMap["PRSW"] = &(modelStruct.d_modelPar_PRSW_dX);
modelStruct.modelPar_PRDW=mod.PRDW;
modelStruct.d_modelPar_PRDW_dX=0.0;
modelStruct.modelPar_given_PRDW=mod.given("PRDW");
modParamMap["PRDW"] = &(modelStruct.d_modelPar_PRDW_dX);
modelStruct.modelPar_PPRWB=mod.PPRWB;
modelStruct.d_modelPar_PPRWB_dX=0.0;
modelStruct.modelPar_given_PPRWB=mod.given("PPRWB");
modParamMap["PPRWB"] = &(modelStruct.d_modelPar_PPRWB_dX);
modelStruct.modelPar_PPRWG=mod.PPRWG;
modelStruct.d_modelPar_PPRWG_dX=0.0;
modelStruct.modelPar_given_PPRWG=mod.given("PPRWG");
modParamMap["PPRWG"] = &(modelStruct.d_modelPar_PPRWG_dX);
modelStruct.modelPar_PWR=mod.PWR;
modelStruct.d_modelPar_PWR_dX=0.0;
modelStruct.modelPar_given_PWR=mod.given("PWR");
modParamMap["PWR"] = &(modelStruct.d_modelPar_PWR_dX);
modelStruct.modelPar_PNFACTOR=mod.PNFACTOR;
modelStruct.d_modelPar_PNFACTOR_dX=0.0;
modelStruct.modelPar_given_PNFACTOR=mod.given("PNFACTOR");
modParamMap["PNFACTOR"] = &(modelStruct.d_modelPar_PNFACTOR_dX);
modelStruct.modelPar_PDWG=mod.PDWG;
modelStruct.d_modelPar_PDWG_dX=0.0;
modelStruct.modelPar_given_PDWG=mod.given("PDWG");
modParamMap["PDWG"] = &(modelStruct.d_modelPar_PDWG_dX);
modelStruct.modelPar_PDWB=mod.PDWB;
modelStruct.d_modelPar_PDWB_dX=0.0;
modelStruct.modelPar_given_PDWB=mod.given("PDWB");
modParamMap["PDWB"] = &(modelStruct.d_modelPar_PDWB_dX);
modelStruct.modelPar_PVOFF=mod.PVOFF;
modelStruct.d_modelPar_PVOFF_dX=0.0;
modelStruct.modelPar_given_PVOFF=mod.given("PVOFF");
modParamMap["PVOFF"] = &(modelStruct.d_modelPar_PVOFF_dX);
modelStruct.modelPar_PETA0=mod.PETA0;
modelStruct.d_modelPar_PETA0_dX=0.0;
modelStruct.modelPar_given_PETA0=mod.given("PETA0");
modParamMap["PETA0"] = &(modelStruct.d_modelPar_PETA0_dX);
modelStruct.modelPar_PETAB=mod.PETAB;
modelStruct.d_modelPar_PETAB_dX=0.0;
modelStruct.modelPar_given_PETAB=mod.given("PETAB");
modParamMap["PETAB"] = &(modelStruct.d_modelPar_PETAB_dX);
modelStruct.modelPar_PETA0CV=mod.PETA0CV;
modelStruct.d_modelPar_PETA0CV_dX=0.0;
modelStruct.modelPar_given_PETA0CV=mod.given("PETA0CV");
modParamMap["PETA0CV"] = &(modelStruct.d_modelPar_PETA0CV_dX);
modelStruct.modelPar_PETABCV=mod.PETABCV;
modelStruct.d_modelPar_PETABCV_dX=0.0;
modelStruct.modelPar_given_PETABCV=mod.given("PETABCV");
modParamMap["PETABCV"] = &(modelStruct.d_modelPar_PETABCV_dX);
modelStruct.modelPar_PDSUB=mod.PDSUB;
modelStruct.d_modelPar_PDSUB_dX=0.0;
modelStruct.modelPar_given_PDSUB=mod.given("PDSUB");
modParamMap["PDSUB"] = &(modelStruct.d_modelPar_PDSUB_dX);
modelStruct.modelPar_PCIT=mod.PCIT;
modelStruct.d_modelPar_PCIT_dX=0.0;
modelStruct.modelPar_given_PCIT=mod.given("PCIT");
modParamMap["PCIT"] = &(modelStruct.d_modelPar_PCIT_dX);
modelStruct.modelPar_PCDSC=mod.PCDSC;
modelStruct.d_modelPar_PCDSC_dX=0.0;
modelStruct.modelPar_given_PCDSC=mod.given("PCDSC");
modParamMap["PCDSC"] = &(modelStruct.d_modelPar_PCDSC_dX);
modelStruct.modelPar_PCDSCB=mod.PCDSCB;
modelStruct.d_modelPar_PCDSCB_dX=0.0;
modelStruct.modelPar_given_PCDSCB=mod.given("PCDSCB");
modParamMap["PCDSCB"] = &(modelStruct.d_modelPar_PCDSCB_dX);
modelStruct.modelPar_PCDSCD=mod.PCDSCD;
modelStruct.d_modelPar_PCDSCD_dX=0.0;
modelStruct.modelPar_given_PCDSCD=mod.given("PCDSCD");
modParamMap["PCDSCD"] = &(modelStruct.d_modelPar_PCDSCD_dX);
modelStruct.modelPar_PPCLM=mod.PPCLM;
modelStruct.d_modelPar_PPCLM_dX=0.0;
modelStruct.modelPar_given_PPCLM=mod.given("PPCLM");
modParamMap["PPCLM"] = &(modelStruct.d_modelPar_PPCLM_dX);
modelStruct.modelPar_PPDIBLC1=mod.PPDIBLC1;
modelStruct.d_modelPar_PPDIBLC1_dX=0.0;
modelStruct.modelPar_given_PPDIBLC1=mod.given("PPDIBLC1");
modParamMap["PPDIBLC1"] = &(modelStruct.d_modelPar_PPDIBLC1_dX);
modelStruct.modelPar_PPDIBLC2=mod.PPDIBLC2;
modelStruct.d_modelPar_PPDIBLC2_dX=0.0;
modelStruct.modelPar_given_PPDIBLC2=mod.given("PPDIBLC2");
modParamMap["PPDIBLC2"] = &(modelStruct.d_modelPar_PPDIBLC2_dX);
modelStruct.modelPar_PPDIBLCB=mod.PPDIBLCB;
modelStruct.d_modelPar_PPDIBLCB_dX=0.0;
modelStruct.modelPar_given_PPDIBLCB=mod.given("PPDIBLCB");
modParamMap["PPDIBLCB"] = &(modelStruct.d_modelPar_PPDIBLCB_dX);
modelStruct.modelPar_PDROUT=mod.PDROUT;
modelStruct.d_modelPar_PDROUT_dX=0.0;
modelStruct.modelPar_given_PDROUT=mod.given("PDROUT");
modParamMap["PDROUT"] = &(modelStruct.d_modelPar_PDROUT_dX);
modelStruct.modelPar_PPVAG=mod.PPVAG;
modelStruct.d_modelPar_PPVAG_dX=0.0;
modelStruct.modelPar_given_PPVAG=mod.given("PPVAG");
modParamMap["PPVAG"] = &(modelStruct.d_modelPar_PPVAG_dX);
modelStruct.modelPar_PDELTA=mod.PDELTA;
modelStruct.d_modelPar_PDELTA_dX=0.0;
modelStruct.modelPar_given_PDELTA=mod.given("PDELTA");
modParamMap["PDELTA"] = &(modelStruct.d_modelPar_PDELTA_dX);
modelStruct.modelPar_PALPHA0=mod.PALPHA0;
modelStruct.d_modelPar_PALPHA0_dX=0.0;
modelStruct.modelPar_given_PALPHA0=mod.given("PALPHA0");
modParamMap["PALPHA0"] = &(modelStruct.d_modelPar_PALPHA0_dX);
modelStruct.modelPar_PFBJTII=mod.PFBJTII;
modelStruct.d_modelPar_PFBJTII_dX=0.0;
modelStruct.modelPar_given_PFBJTII=mod.given("PFBJTII");
modParamMap["PFBJTII"] = &(modelStruct.d_modelPar_PFBJTII_dX);
modelStruct.modelPar_PABJTII=mod.PABJTII;
modelStruct.d_modelPar_PABJTII_dX=0.0;
modelStruct.modelPar_given_PABJTII=mod.given("PABJTII");
modParamMap["PABJTII"] = &(modelStruct.d_modelPar_PABJTII_dX);
modelStruct.modelPar_PCBJTII=mod.PCBJTII;
modelStruct.d_modelPar_PCBJTII_dX=0.0;
modelStruct.modelPar_given_PCBJTII=mod.given("PCBJTII");
modParamMap["PCBJTII"] = &(modelStruct.d_modelPar_PCBJTII_dX);
modelStruct.modelPar_PEBJTII=mod.PEBJTII;
modelStruct.d_modelPar_PEBJTII_dX=0.0;
modelStruct.modelPar_given_PEBJTII=mod.given("PEBJTII");
modParamMap["PEBJTII"] = &(modelStruct.d_modelPar_PEBJTII_dX);
modelStruct.modelPar_PMBJTII=mod.PMBJTII;
modelStruct.d_modelPar_PMBJTII_dX=0.0;
modelStruct.modelPar_given_PMBJTII=mod.given("PMBJTII");
modParamMap["PMBJTII"] = &(modelStruct.d_modelPar_PMBJTII_dX);
modelStruct.modelPar_PVBCI=mod.PVBCI;
modelStruct.d_modelPar_PVBCI_dX=0.0;
modelStruct.modelPar_given_PVBCI=mod.given("PVBCI");
modParamMap["PVBCI"] = &(modelStruct.d_modelPar_PVBCI_dX);
modelStruct.modelPar_PBETA0=mod.PBETA0;
modelStruct.d_modelPar_PBETA0_dX=0.0;
modelStruct.modelPar_given_PBETA0=mod.given("PBETA0");
modParamMap["PBETA0"] = &(modelStruct.d_modelPar_PBETA0_dX);
modelStruct.modelPar_PBETA1=mod.PBETA1;
modelStruct.d_modelPar_PBETA1_dX=0.0;
modelStruct.modelPar_given_PBETA1=mod.given("PBETA1");
modParamMap["PBETA1"] = &(modelStruct.d_modelPar_PBETA1_dX);
modelStruct.modelPar_PBETA2=mod.PBETA2;
modelStruct.d_modelPar_PBETA2_dX=0.0;
modelStruct.modelPar_given_PBETA2=mod.given("PBETA2");
modParamMap["PBETA2"] = &(modelStruct.d_modelPar_PBETA2_dX);
modelStruct.modelPar_PVDSATII0=mod.PVDSATII0;
modelStruct.d_modelPar_PVDSATII0_dX=0.0;
modelStruct.modelPar_given_PVDSATII0=mod.given("PVDSATII0");
modParamMap["PVDSATII0"] = &(modelStruct.d_modelPar_PVDSATII0_dX);
modelStruct.modelPar_PLII=mod.PLII;
modelStruct.d_modelPar_PLII_dX=0.0;
modelStruct.modelPar_given_PLII=mod.given("PLII");
modParamMap["PLII"] = &(modelStruct.d_modelPar_PLII_dX);
modelStruct.modelPar_PESATII=mod.PESATII;
modelStruct.d_modelPar_PESATII_dX=0.0;
modelStruct.modelPar_given_PESATII=mod.given("PESATII");
modParamMap["PESATII"] = &(modelStruct.d_modelPar_PESATII_dX);
modelStruct.modelPar_PSII0=mod.PSII0;
modelStruct.d_modelPar_PSII0_dX=0.0;
modelStruct.modelPar_given_PSII0=mod.given("PSII0");
modParamMap["PSII0"] = &(modelStruct.d_modelPar_PSII0_dX);
modelStruct.modelPar_PSII1=mod.PSII1;
modelStruct.d_modelPar_PSII1_dX=0.0;
modelStruct.modelPar_given_PSII1=mod.given("PSII1");
modParamMap["PSII1"] = &(modelStruct.d_modelPar_PSII1_dX);
modelStruct.modelPar_PSII2=mod.PSII2;
modelStruct.d_modelPar_PSII2_dX=0.0;
modelStruct.modelPar_given_PSII2=mod.given("PSII2");
modParamMap["PSII2"] = &(modelStruct.d_modelPar_PSII2_dX);
modelStruct.modelPar_PSIID=mod.PSIID;
modelStruct.d_modelPar_PSIID_dX=0.0;
modelStruct.modelPar_given_PSIID=mod.given("PSIID");
modParamMap["PSIID"] = &(modelStruct.d_modelPar_PSIID_dX);
modelStruct.modelPar_PAGIDL=mod.PAGIDL;
modelStruct.d_modelPar_PAGIDL_dX=0.0;
modelStruct.modelPar_given_PAGIDL=mod.given("PAGIDL");
modParamMap["PAGIDL"] = &(modelStruct.d_modelPar_PAGIDL_dX);
modelStruct.modelPar_PBGIDL=mod.PBGIDL;
modelStruct.d_modelPar_PBGIDL_dX=0.0;
modelStruct.modelPar_given_PBGIDL=mod.given("PBGIDL");
modParamMap["PBGIDL"] = &(modelStruct.d_modelPar_PBGIDL_dX);
modelStruct.modelPar_PCGIDL=mod.PCGIDL;
modelStruct.d_modelPar_PCGIDL_dX=0.0;
modelStruct.modelPar_given_PCGIDL=mod.given("PCGIDL");
modParamMap["PCGIDL"] = &(modelStruct.d_modelPar_PCGIDL_dX);
modelStruct.modelPar_PRGIDL=mod.PRGIDL;
modelStruct.d_modelPar_PRGIDL_dX=0.0;
modelStruct.modelPar_given_PRGIDL=mod.given("PRGIDL");
modParamMap["PRGIDL"] = &(modelStruct.d_modelPar_PRGIDL_dX);
modelStruct.modelPar_PKGIDL=mod.PKGIDL;
modelStruct.d_modelPar_PKGIDL_dX=0.0;
modelStruct.modelPar_given_PKGIDL=mod.given("PKGIDL");
modParamMap["PKGIDL"] = &(modelStruct.d_modelPar_PKGIDL_dX);
modelStruct.modelPar_PFGIDL=mod.PFGIDL;
modelStruct.d_modelPar_PFGIDL_dX=0.0;
modelStruct.modelPar_given_PFGIDL=mod.given("PFGIDL");
modParamMap["PFGIDL"] = &(modelStruct.d_modelPar_PFGIDL_dX);
modelStruct.modelPar_PAGISL=mod.PAGISL;
modelStruct.d_modelPar_PAGISL_dX=0.0;
modelStruct.modelPar_given_PAGISL=mod.given("PAGISL");
modParamMap["PAGISL"] = &(modelStruct.d_modelPar_PAGISL_dX);
modelStruct.modelPar_PBGISL=mod.PBGISL;
modelStruct.d_modelPar_PBGISL_dX=0.0;
modelStruct.modelPar_given_PBGISL=mod.given("PBGISL");
modParamMap["PBGISL"] = &(modelStruct.d_modelPar_PBGISL_dX);
modelStruct.modelPar_PCGISL=mod.PCGISL;
modelStruct.d_modelPar_PCGISL_dX=0.0;
modelStruct.modelPar_given_PCGISL=mod.given("PCGISL");
modParamMap["PCGISL"] = &(modelStruct.d_modelPar_PCGISL_dX);
modelStruct.modelPar_PRGISL=mod.PRGISL;
modelStruct.d_modelPar_PRGISL_dX=0.0;
modelStruct.modelPar_given_PRGISL=mod.given("PRGISL");
modParamMap["PRGISL"] = &(modelStruct.d_modelPar_PRGISL_dX);
modelStruct.modelPar_PKGISL=mod.PKGISL;
modelStruct.d_modelPar_PKGISL_dX=0.0;
modelStruct.modelPar_given_PKGISL=mod.given("PKGISL");
modParamMap["PKGISL"] = &(modelStruct.d_modelPar_PKGISL_dX);
modelStruct.modelPar_PFGISL=mod.PFGISL;
modelStruct.d_modelPar_PFGISL_dX=0.0;
modelStruct.modelPar_given_PFGISL=mod.given("PFGISL");
modParamMap["PFGISL"] = &(modelStruct.d_modelPar_PFGISL_dX);
modelStruct.modelPar_PNTUN=mod.PNTUN;
modelStruct.d_modelPar_PNTUN_dX=0.0;
modelStruct.modelPar_given_PNTUN=mod.given("PNTUN");
modParamMap["PNTUN"] = &(modelStruct.d_modelPar_PNTUN_dX);
modelStruct.modelPar_PNTUND=mod.PNTUND;
modelStruct.d_modelPar_PNTUND_dX=0.0;
modelStruct.modelPar_given_PNTUND=mod.given("PNTUND");
modParamMap["PNTUND"] = &(modelStruct.d_modelPar_PNTUND_dX);
modelStruct.modelPar_PNDIODE=mod.PNDIODE;
modelStruct.d_modelPar_PNDIODE_dX=0.0;
modelStruct.modelPar_given_PNDIODE=mod.given("PNDIODE");
modParamMap["PNDIODE"] = &(modelStruct.d_modelPar_PNDIODE_dX);
modelStruct.modelPar_PNDIODED=mod.PNDIODED;
modelStruct.d_modelPar_PNDIODED_dX=0.0;
modelStruct.modelPar_given_PNDIODED=mod.given("PNDIODED");
modParamMap["PNDIODED"] = &(modelStruct.d_modelPar_PNDIODED_dX);
modelStruct.modelPar_PNRECF0=mod.PNRECF0;
modelStruct.d_modelPar_PNRECF0_dX=0.0;
modelStruct.modelPar_given_PNRECF0=mod.given("PNRECF0");
modParamMap["PNRECF0"] = &(modelStruct.d_modelPar_PNRECF0_dX);
modelStruct.modelPar_PNRECF0D=mod.PNRECF0D;
modelStruct.d_modelPar_PNRECF0D_dX=0.0;
modelStruct.modelPar_given_PNRECF0D=mod.given("PNRECF0D");
modParamMap["PNRECF0D"] = &(modelStruct.d_modelPar_PNRECF0D_dX);
modelStruct.modelPar_PNRECR0=mod.PNRECR0;
modelStruct.d_modelPar_PNRECR0_dX=0.0;
modelStruct.modelPar_given_PNRECR0=mod.given("PNRECR0");
modParamMap["PNRECR0"] = &(modelStruct.d_modelPar_PNRECR0_dX);
modelStruct.modelPar_PNRECR0D=mod.PNRECR0D;
modelStruct.d_modelPar_PNRECR0D_dX=0.0;
modelStruct.modelPar_given_PNRECR0D=mod.given("PNRECR0D");
modParamMap["PNRECR0D"] = &(modelStruct.d_modelPar_PNRECR0D_dX);
modelStruct.modelPar_PISBJT=mod.PISBJT;
modelStruct.d_modelPar_PISBJT_dX=0.0;
modelStruct.modelPar_given_PISBJT=mod.given("PISBJT");
modParamMap["PISBJT"] = &(modelStruct.d_modelPar_PISBJT_dX);
modelStruct.modelPar_PIDBJT=mod.PIDBJT;
modelStruct.d_modelPar_PIDBJT_dX=0.0;
modelStruct.modelPar_given_PIDBJT=mod.given("PIDBJT");
modParamMap["PIDBJT"] = &(modelStruct.d_modelPar_PIDBJT_dX);
modelStruct.modelPar_PISDIF=mod.PISDIF;
modelStruct.d_modelPar_PISDIF_dX=0.0;
modelStruct.modelPar_given_PISDIF=mod.given("PISDIF");
modParamMap["PISDIF"] = &(modelStruct.d_modelPar_PISDIF_dX);
modelStruct.modelPar_PIDDIF=mod.PIDDIF;
modelStruct.d_modelPar_PIDDIF_dX=0.0;
modelStruct.modelPar_given_PIDDIF=mod.given("PIDDIF");
modParamMap["PIDDIF"] = &(modelStruct.d_modelPar_PIDDIF_dX);
modelStruct.modelPar_PISREC=mod.PISREC;
modelStruct.d_modelPar_PISREC_dX=0.0;
modelStruct.modelPar_given_PISREC=mod.given("PISREC");
modParamMap["PISREC"] = &(modelStruct.d_modelPar_PISREC_dX);
modelStruct.modelPar_PIDREC=mod.PIDREC;
modelStruct.d_modelPar_PIDREC_dX=0.0;
modelStruct.modelPar_given_PIDREC=mod.given("PIDREC");
modParamMap["PIDREC"] = &(modelStruct.d_modelPar_PIDREC_dX);
modelStruct.modelPar_PISTUN=mod.PISTUN;
modelStruct.d_modelPar_PISTUN_dX=0.0;
modelStruct.modelPar_given_PISTUN=mod.given("PISTUN");
modParamMap["PISTUN"] = &(modelStruct.d_modelPar_PISTUN_dX);
modelStruct.modelPar_PIDTUN=mod.PIDTUN;
modelStruct.d_modelPar_PIDTUN_dX=0.0;
modelStruct.modelPar_given_PIDTUN=mod.given("PIDTUN");
modParamMap["PIDTUN"] = &(modelStruct.d_modelPar_PIDTUN_dX);
modelStruct.modelPar_PVREC0=mod.PVREC0;
modelStruct.d_modelPar_PVREC0_dX=0.0;
modelStruct.modelPar_given_PVREC0=mod.given("PVREC0");
modParamMap["PVREC0"] = &(modelStruct.d_modelPar_PVREC0_dX);
modelStruct.modelPar_PVREC0D=mod.PVREC0D;
modelStruct.d_modelPar_PVREC0D_dX=0.0;
modelStruct.modelPar_given_PVREC0D=mod.given("PVREC0D");
modParamMap["PVREC0D"] = &(modelStruct.d_modelPar_PVREC0D_dX);
modelStruct.modelPar_PVTUN0=mod.PVTUN0;
modelStruct.d_modelPar_PVTUN0_dX=0.0;
modelStruct.modelPar_given_PVTUN0=mod.given("PVTUN0");
modParamMap["PVTUN0"] = &(modelStruct.d_modelPar_PVTUN0_dX);
modelStruct.modelPar_PVTUN0D=mod.PVTUN0D;
modelStruct.d_modelPar_PVTUN0D_dX=0.0;
modelStruct.modelPar_given_PVTUN0D=mod.given("PVTUN0D");
modParamMap["PVTUN0D"] = &(modelStruct.d_modelPar_PVTUN0D_dX);
modelStruct.modelPar_PNBJT=mod.PNBJT;
modelStruct.d_modelPar_PNBJT_dX=0.0;
modelStruct.modelPar_given_PNBJT=mod.given("PNBJT");
modParamMap["PNBJT"] = &(modelStruct.d_modelPar_PNBJT_dX);
modelStruct.modelPar_PLBJT0=mod.PLBJT0;
modelStruct.d_modelPar_PLBJT0_dX=0.0;
modelStruct.modelPar_given_PLBJT0=mod.given("PLBJT0");
modParamMap["PLBJT0"] = &(modelStruct.d_modelPar_PLBJT0_dX);
modelStruct.modelPar_PVABJT=mod.PVABJT;
modelStruct.d_modelPar_PVABJT_dX=0.0;
modelStruct.modelPar_given_PVABJT=mod.given("PVABJT");
modParamMap["PVABJT"] = &(modelStruct.d_modelPar_PVABJT_dX);
modelStruct.modelPar_PAELY=mod.PAELY;
modelStruct.d_modelPar_PAELY_dX=0.0;
modelStruct.modelPar_given_PAELY=mod.given("PAELY");
modParamMap["PAELY"] = &(modelStruct.d_modelPar_PAELY_dX);
modelStruct.modelPar_PAHLI=mod.PAHLI;
modelStruct.d_modelPar_PAHLI_dX=0.0;
modelStruct.modelPar_given_PAHLI=mod.given("PAHLI");
modParamMap["PAHLI"] = &(modelStruct.d_modelPar_PAHLI_dX);
modelStruct.modelPar_PAHLID=mod.PAHLID;
modelStruct.d_modelPar_PAHLID_dX=0.0;
modelStruct.modelPar_given_PAHLID=mod.given("PAHLID");
modParamMap["PAHLID"] = &(modelStruct.d_modelPar_PAHLID_dX);
modelStruct.modelPar_PVSDFB=mod.PVSDFB;
modelStruct.d_modelPar_PVSDFB_dX=0.0;
modelStruct.modelPar_given_PVSDFB=mod.given("PVSDFB");
modParamMap["PVSDFB"] = &(modelStruct.d_modelPar_PVSDFB_dX);
modelStruct.modelPar_PVSDTH=mod.PVSDTH;
modelStruct.d_modelPar_PVSDTH_dX=0.0;
modelStruct.modelPar_given_PVSDTH=mod.given("PVSDTH");
modParamMap["PVSDTH"] = &(modelStruct.d_modelPar_PVSDTH_dX);
modelStruct.modelPar_PDELVT=mod.PDELVT;
modelStruct.d_modelPar_PDELVT_dX=0.0;
modelStruct.modelPar_given_PDELVT=mod.given("PDELVT");
modParamMap["PDELVT"] = &(modelStruct.d_modelPar_PDELVT_dX);
modelStruct.modelPar_PACDE=mod.PACDE;
modelStruct.d_modelPar_PACDE_dX=0.0;
modelStruct.modelPar_given_PACDE=mod.given("PACDE");
modParamMap["PACDE"] = &(modelStruct.d_modelPar_PACDE_dX);
modelStruct.modelPar_PMOIN=mod.PMOIN;
modelStruct.d_modelPar_PMOIN_dX=0.0;
modelStruct.modelPar_given_PMOIN=mod.given("PMOIN");
modParamMap["PMOIN"] = &(modelStruct.d_modelPar_PMOIN_dX);
modelStruct.modelPar_PNOFF=mod.PNOFF;
modelStruct.d_modelPar_PNOFF_dX=0.0;
modelStruct.modelPar_given_PNOFF=mod.given("PNOFF");
modParamMap["PNOFF"] = &(modelStruct.d_modelPar_PNOFF_dX);
modelStruct.modelPar_PNOFF2=mod.PNOFF2;
modelStruct.d_modelPar_PNOFF2_dX=0.0;
modelStruct.modelPar_given_PNOFF2=mod.given("PNOFF2");
modParamMap["PNOFF2"] = &(modelStruct.d_modelPar_PNOFF2_dX);
modelStruct.modelPar_PXRCRG1=mod.PXRCRG1;
modelStruct.d_modelPar_PXRCRG1_dX=0.0;
modelStruct.modelPar_given_PXRCRG1=mod.given("PXRCRG1");
modParamMap["PXRCRG1"] = &(modelStruct.d_modelPar_PXRCRG1_dX);
modelStruct.modelPar_PXRCRG2=mod.PXRCRG2;
modelStruct.d_modelPar_PXRCRG2_dX=0.0;
modelStruct.modelPar_given_PXRCRG2=mod.given("PXRCRG2");
modParamMap["PXRCRG2"] = &(modelStruct.d_modelPar_PXRCRG2_dX);
modelStruct.modelPar_PVBSA=mod.PVBSA;
modelStruct.d_modelPar_PVBSA_dX=0.0;
modelStruct.modelPar_given_PVBSA=mod.given("PVBSA");
modParamMap["PVBSA"] = &(modelStruct.d_modelPar_PVBSA_dX);
modelStruct.modelPar_PVSCE=mod.PVSCE;
modelStruct.d_modelPar_PVSCE_dX=0.0;
modelStruct.modelPar_given_PVSCE=mod.given("PVSCE");
modParamMap["PVSCE"] = &(modelStruct.d_modelPar_PVSCE_dX);
modelStruct.modelPar_PCDSBS=mod.PCDSBS;
modelStruct.d_modelPar_PCDSBS_dX=0.0;
modelStruct.modelPar_given_PCDSBS=mod.given("PCDSBS");
modParamMap["PCDSBS"] = &(modelStruct.d_modelPar_PCDSBS_dX);
modelStruct.modelPar_PNOFFFD=mod.PNOFFFD;
modelStruct.d_modelPar_PNOFFFD_dX=0.0;
modelStruct.modelPar_given_PNOFFFD=mod.given("PNOFFFD");
modParamMap["PNOFFFD"] = &(modelStruct.d_modelPar_PNOFFFD_dX);
modelStruct.modelPar_PVOFFFD=mod.PVOFFFD;
modelStruct.d_modelPar_PVOFFFD_dX=0.0;
modelStruct.modelPar_given_PVOFFFD=mod.given("PVOFFFD");
modParamMap["PVOFFFD"] = &(modelStruct.d_modelPar_PVOFFFD_dX);
modelStruct.modelPar_PK1B=mod.PK1B;
modelStruct.d_modelPar_PK1B_dX=0.0;
modelStruct.modelPar_given_PK1B=mod.given("PK1B");
modParamMap["PK1B"] = &(modelStruct.d_modelPar_PK1B_dX);
modelStruct.modelPar_PK2B=mod.PK2B;
modelStruct.d_modelPar_PK2B_dX=0.0;
modelStruct.modelPar_given_PK2B=mod.given("PK2B");
modParamMap["PK2B"] = &(modelStruct.d_modelPar_PK2B_dX);
modelStruct.modelPar_PDK2B=mod.PDK2B;
modelStruct.d_modelPar_PDK2B_dX=0.0;
modelStruct.modelPar_given_PDK2B=mod.given("PDK2B");
modParamMap["PDK2B"] = &(modelStruct.d_modelPar_PDK2B_dX);
modelStruct.modelPar_PDVBD0=mod.PDVBD0;
modelStruct.d_modelPar_PDVBD0_dX=0.0;
modelStruct.modelPar_given_PDVBD0=mod.given("PDVBD0");
modParamMap["PDVBD0"] = &(modelStruct.d_modelPar_PDVBD0_dX);
modelStruct.modelPar_PDVBD1=mod.PDVBD1;
modelStruct.d_modelPar_PDVBD1_dX=0.0;
modelStruct.modelPar_given_PDVBD1=mod.given("PDVBD1");
modParamMap["PDVBD1"] = &(modelStruct.d_modelPar_PDVBD1_dX);
modelStruct.modelPar_PMOINFD=mod.PMOINFD;
modelStruct.d_modelPar_PMOINFD_dX=0.0;
modelStruct.modelPar_given_PMOINFD=mod.given("PMOINFD");
modParamMap["PMOINFD"] = &(modelStruct.d_modelPar_PMOINFD_dX);
modelStruct.modelPar_PVBS0PD=mod.PVBS0PD;
modelStruct.d_modelPar_PVBS0PD_dX=0.0;
modelStruct.modelPar_given_PVBS0PD=mod.given("PVBS0PD");
modParamMap["PVBS0PD"] = &(modelStruct.d_modelPar_PVBS0PD_dX);
modelStruct.modelPar_PVBS0FD=mod.PVBS0FD;
modelStruct.d_modelPar_PVBS0FD_dX=0.0;
modelStruct.modelPar_given_PVBS0FD=mod.given("PVBS0FD");
modParamMap["PVBS0FD"] = &(modelStruct.d_modelPar_PVBS0FD_dX);
modelStruct.modelPar_NLX=mod.NLX;
modelStruct.d_modelPar_NLX_dX=0.0;
modelStruct.modelPar_given_NLX=mod.given("NLX");
modParamMap["NLX"] = &(modelStruct.d_modelPar_NLX_dX);
modelStruct.modelPar_LNLX=mod.LNLX;
modelStruct.d_modelPar_LNLX_dX=0.0;
modelStruct.modelPar_given_LNLX=mod.given("LNLX");
modParamMap["LNLX"] = &(modelStruct.d_modelPar_LNLX_dX);
modelStruct.modelPar_WNLX=mod.WNLX;
modelStruct.d_modelPar_WNLX_dX=0.0;
modelStruct.modelPar_given_WNLX=mod.given("WNLX");
modParamMap["WNLX"] = &(modelStruct.d_modelPar_WNLX_dX);
modelStruct.modelPar_PNLX=mod.PNLX;
modelStruct.d_modelPar_PNLX_dX=0.0;
modelStruct.modelPar_given_PNLX=mod.given("PNLX");
modParamMap["PNLX"] = &(modelStruct.d_modelPar_PNLX_dX);
modelStruct.modelPar_NGIDL=mod.NGIDL;
modelStruct.d_modelPar_NGIDL_dX=0.0;
modelStruct.modelPar_given_NGIDL=mod.given("NGIDL");
modParamMap["NGIDL"] = &(modelStruct.d_modelPar_NGIDL_dX);
modelStruct.modelPar_LNGIDL=mod.LNGIDL;
modelStruct.d_modelPar_LNGIDL_dX=0.0;
modelStruct.modelPar_given_LNGIDL=mod.given("LNGIDL");
modParamMap["LNGIDL"] = &(modelStruct.d_modelPar_LNGIDL_dX);
modelStruct.modelPar_WNGIDL=mod.WNGIDL;
modelStruct.d_modelPar_WNGIDL_dX=0.0;
modelStruct.modelPar_given_WNGIDL=mod.given("WNGIDL");
modParamMap["WNGIDL"] = &(modelStruct.d_modelPar_WNGIDL_dX);
modelStruct.modelPar_PNGIDL=mod.PNGIDL;
modelStruct.d_modelPar_PNGIDL_dX=0.0;
modelStruct.modelPar_given_PNGIDL=mod.given("PNGIDL");
modParamMap["PNGIDL"] = &(modelStruct.d_modelPar_PNGIDL_dX);
modelStruct.modelPar_LPE0=mod.LPE0;
modelStruct.d_modelPar_LPE0_dX=0.0;
modelStruct.modelPar_given_LPE0=mod.given("LPE0");
modParamMap["LPE0"] = &(modelStruct.d_modelPar_LPE0_dX);
modelStruct.modelPar_EGIDL=mod.EGIDL;
modelStruct.d_modelPar_EGIDL_dX=0.0;
modelStruct.modelPar_given_EGIDL=mod.given("EGIDL");
modParamMap["EGIDL"] = &(modelStruct.d_modelPar_EGIDL_dX);
modelStruct.modelPar_EGISL=mod.EGISL;
modelStruct.d_modelPar_EGISL_dX=0.0;
modelStruct.modelPar_given_EGISL=mod.given("EGISL");
modParamMap["EGISL"] = &(modelStruct.d_modelPar_EGISL_dX);
modelStruct.modelPar_LLPE0=mod.LLPE0;
modelStruct.d_modelPar_LLPE0_dX=0.0;
modelStruct.modelPar_given_LLPE0=mod.given("LLPE0");
modParamMap["LLPE0"] = &(modelStruct.d_modelPar_LLPE0_dX);
modelStruct.modelPar_LEGIDL=mod.LEGIDL;
modelStruct.d_modelPar_LEGIDL_dX=0.0;
modelStruct.modelPar_given_LEGIDL=mod.given("LEGIDL");
modParamMap["LEGIDL"] = &(modelStruct.d_modelPar_LEGIDL_dX);
modelStruct.modelPar_LEGISL=mod.LEGISL;
modelStruct.d_modelPar_LEGISL_dX=0.0;
modelStruct.modelPar_given_LEGISL=mod.given("LEGISL");
modParamMap["LEGISL"] = &(modelStruct.d_modelPar_LEGISL_dX);
modelStruct.modelPar_WLPE0=mod.WLPE0;
modelStruct.d_modelPar_WLPE0_dX=0.0;
modelStruct.modelPar_given_WLPE0=mod.given("WLPE0");
modParamMap["WLPE0"] = &(modelStruct.d_modelPar_WLPE0_dX);
modelStruct.modelPar_WEGIDL=mod.WEGIDL;
modelStruct.d_modelPar_WEGIDL_dX=0.0;
modelStruct.modelPar_given_WEGIDL=mod.given("WEGIDL");
modParamMap["WEGIDL"] = &(modelStruct.d_modelPar_WEGIDL_dX);
modelStruct.modelPar_WEGISL=mod.WEGISL;
modelStruct.d_modelPar_WEGISL_dX=0.0;
modelStruct.modelPar_given_WEGISL=mod.given("WEGISL");
modParamMap["WEGISL"] = &(modelStruct.d_modelPar_WEGISL_dX);
modelStruct.modelPar_PLPE0=mod.PLPE0;
modelStruct.d_modelPar_PLPE0_dX=0.0;
modelStruct.modelPar_given_PLPE0=mod.given("PLPE0");
modParamMap["PLPE0"] = &(modelStruct.d_modelPar_PLPE0_dX);
modelStruct.modelPar_PEGIDL=mod.PEGIDL;
modelStruct.d_modelPar_PEGIDL_dX=0.0;
modelStruct.modelPar_given_PEGIDL=mod.given("PEGIDL");
modParamMap["PEGIDL"] = &(modelStruct.d_modelPar_PEGIDL_dX);
modelStruct.modelPar_PEGISL=mod.PEGISL;
modelStruct.d_modelPar_PEGISL_dX=0.0;
modelStruct.modelPar_given_PEGISL=mod.given("PEGISL");
modParamMap["PEGISL"] = &(modelStruct.d_modelPar_PEGISL_dX);
modelStruct.modelPar_EGGBCP2=mod.EGGBCP2;
modelStruct.d_modelPar_EGGBCP2_dX=0.0;
modelStruct.modelPar_given_EGGBCP2=mod.given("EGGBCP2");
modParamMap["EGGBCP2"] = &(modelStruct.d_modelPar_EGGBCP2_dX);
modelStruct.modelPar_EGGDEP=mod.EGGDEP;
modelStruct.d_modelPar_EGGDEP_dX=0.0;
modelStruct.modelPar_given_EGGDEP=mod.given("EGGDEP");
modParamMap["EGGDEP"] = &(modelStruct.d_modelPar_EGGDEP_dX);
modelStruct.modelPar_AGB1=mod.AGB1;
modelStruct.d_modelPar_AGB1_dX=0.0;
modelStruct.modelPar_given_AGB1=mod.given("AGB1");
modParamMap["AGB1"] = &(modelStruct.d_modelPar_AGB1_dX);
modelStruct.modelPar_BGB1=mod.BGB1;
modelStruct.d_modelPar_BGB1_dX=0.0;
modelStruct.modelPar_given_BGB1=mod.given("BGB1");
modParamMap["BGB1"] = &(modelStruct.d_modelPar_BGB1_dX);
modelStruct.modelPar_AGB2=mod.AGB2;
modelStruct.d_modelPar_AGB2_dX=0.0;
modelStruct.modelPar_given_AGB2=mod.given("AGB2");
modParamMap["AGB2"] = &(modelStruct.d_modelPar_AGB2_dX);
modelStruct.modelPar_BGB2=mod.BGB2;
modelStruct.d_modelPar_BGB2_dX=0.0;
modelStruct.modelPar_given_BGB2=mod.given("BGB2");
modParamMap["BGB2"] = &(modelStruct.d_modelPar_BGB2_dX);
modelStruct.modelPar_AGBC2N=mod.AGBC2N;
modelStruct.d_modelPar_AGBC2N_dX=0.0;
modelStruct.modelPar_given_AGBC2N=mod.given("AGBC2N");
modParamMap["AGBC2N"] = &(modelStruct.d_modelPar_AGBC2N_dX);
modelStruct.modelPar_AGBC2P=mod.AGBC2P;
modelStruct.d_modelPar_AGBC2P_dX=0.0;
modelStruct.modelPar_given_AGBC2P=mod.given("AGBC2P");
modParamMap["AGBC2P"] = &(modelStruct.d_modelPar_AGBC2P_dX);
modelStruct.modelPar_BGBC2N=mod.BGBC2N;
modelStruct.d_modelPar_BGBC2N_dX=0.0;
modelStruct.modelPar_given_BGBC2N=mod.given("BGBC2N");
modParamMap["BGBC2N"] = &(modelStruct.d_modelPar_BGBC2N_dX);
modelStruct.modelPar_BGBC2P=mod.BGBC2P;
modelStruct.d_modelPar_BGBC2P_dX=0.0;
modelStruct.modelPar_given_BGBC2P=mod.given("BGBC2P");
modParamMap["BGBC2P"] = &(modelStruct.d_modelPar_BGBC2P_dX);
modelStruct.modelPar_VTM00=mod.VTM00;
modelStruct.d_modelPar_VTM00_dX=0.0;
modelStruct.modelPar_given_VTM00=mod.given("VTM00");
modParamMap["VTM00"] = &(modelStruct.d_modelPar_VTM00_dX);


// hidden reals


// non-reals (including hiddens)
modelStruct.modelPar_NF=mod.NF;
modelStruct.modelPar_given_NF=mod.given("NF");
modelStruct.modelPar_OFF=mod.OFF;
modelStruct.modelPar_given_OFF=mod.given("OFF");
modelStruct.modelPar_BJTOFF=mod.BJTOFF;
modelStruct.modelPar_given_BJTOFF=mod.given("BJTOFF");
modelStruct.modelPar_SOIMOD=mod.SOIMOD;
modelStruct.modelPar_given_SOIMOD=mod.given("SOIMOD");
modelStruct.modelPar_SHMOD=mod.SHMOD;
modelStruct.modelPar_given_SHMOD=mod.given("SHMOD");
modelStruct.modelPar_TYPE=mod.TYPE;
modelStruct.modelPar_given_TYPE=mod.given("TYPE");
modelStruct.modelPar_RGATEMOD=mod.RGATEMOD;
modelStruct.modelPar_given_RGATEMOD=mod.given("RGATEMOD");
modelStruct.modelPar_RBODYMOD=mod.RBODYMOD;
modelStruct.modelPar_given_RBODYMOD=mod.given("RBODYMOD");
modelStruct.modelPar_MTRLMOD=mod.MTRLMOD;
modelStruct.modelPar_given_MTRLMOD=mod.given("MTRLMOD");
modelStruct.modelPar_VGSTCVMOD=mod.VGSTCVMOD;
modelStruct.modelPar_given_VGSTCVMOD=mod.given("VGSTCVMOD");
modelStruct.modelPar_GIDLMOD=mod.GIDLMOD;
modelStruct.modelPar_given_GIDLMOD=mod.given("GIDLMOD");
modelStruct.modelPar_IIIMOD=mod.IIIMOD;
modelStruct.modelPar_given_IIIMOD=mod.given("IIIMOD");
modelStruct.modelPar_CAPMOD=mod.CAPMOD;
modelStruct.modelPar_given_CAPMOD=mod.given("CAPMOD");
modelStruct.modelPar_MOBMOD=mod.MOBMOD;
modelStruct.modelPar_given_MOBMOD=mod.given("MOBMOD");
modelStruct.modelPar_PARAMCHK=mod.PARAMCHK;
modelStruct.modelPar_given_PARAMCHK=mod.given("PARAMCHK");
modelStruct.modelPar_NODECHK=mod.NODECHK;
modelStruct.modelPar_given_NODECHK=mod.given("NODECHK");
modelStruct.modelPar_BINUNIT=mod.BINUNIT;
modelStruct.modelPar_given_BINUNIT=mod.given("BINUNIT");
modelStruct.modelPar_FNOIMOD=mod.FNOIMOD;
modelStruct.modelPar_given_FNOIMOD=mod.given("FNOIMOD");
modelStruct.modelPar_TNOIMOD=mod.TNOIMOD;
modelStruct.modelPar_given_TNOIMOD=mod.given("TNOIMOD");
modelStruct.modelPar_IGBMOD=mod.IGBMOD;
modelStruct.modelPar_given_IGBMOD=mod.given("IGBMOD");
modelStruct.modelPar_IGMOD=mod.IGMOD;
modelStruct.modelPar_given_IGMOD=mod.given("IGMOD");
modelStruct.modelPar_IGCMOD=mod.IGCMOD;
modelStruct.modelPar_given_IGCMOD=mod.given("IGCMOD");
modelStruct.modelPar_RDSMOD=mod.RDSMOD;
modelStruct.modelPar_given_RDSMOD=mod.given("RDSMOD");
modelStruct.modelPar_FDMOD=mod.FDMOD;
modelStruct.modelPar_given_FDMOD=mod.given("FDMOD");


// Set the one parameter whose name was passed in to be the independent
// variable for differentiation purposes.  Since we stored variable pointers, this
// makes sure that that ONE variable gets set right.
// FIXME: make this check name for presence first!  Segfault on invalid.
*(modParamMap[name])=1.0;

//make local copies of all model vars
//reals


// non-reals



// Now loop over all instances and do the deed
int inst=0;
for (std::vector<Instance*>::const_iterator in_it=mod.instanceContainer.begin(); in_it != mod.instanceContainer.end(); ++in_it,++inst)
{

  Instance & in=*(*in_it);
  // initialize contributions to zero (automatically sets derivatives to zero)
  for (int i=0; i < 13+0 ; ++i)
  {
  d_staticContributions_dX[i]=0;
  d_dynamicContributions_dX[i]=0;
  }



  // Copy out all the instance parameters (doubles) into local struct
 instanceSensStruct instanceStruct;

 // reals
instanceStruct.instancePar_DTEMP=in.DTEMP;
instanceStruct.d_instancePar_DTEMP_dX=0.0;
instanceStruct.instancePar_given_DTEMP=in.given("DTEMP");
instanceStruct.instancePar_L=in.L;
instanceStruct.d_instancePar_L_dX=0.0;
instanceStruct.instancePar_given_L=in.given("L");
instanceStruct.instancePar_W=in.W;
instanceStruct.d_instancePar_W_dX=0.0;
instanceStruct.instancePar_given_W=in.given("W");
instanceStruct.instancePar_SA=in.SA;
instanceStruct.d_instancePar_SA_dX=0.0;
instanceStruct.instancePar_given_SA=in.given("SA");
instanceStruct.instancePar_SB=in.SB;
instanceStruct.d_instancePar_SB_dX=0.0;
instanceStruct.instancePar_given_SB=in.given("SB");
instanceStruct.instancePar_SD=in.SD;
instanceStruct.d_instancePar_SD_dX=0.0;
instanceStruct.instancePar_given_SD=in.given("SD");
instanceStruct.instancePar_AD=in.AD;
instanceStruct.d_instancePar_AD_dX=0.0;
instanceStruct.instancePar_given_AD=in.given("AD");
instanceStruct.instancePar_AS=in.AS;
instanceStruct.d_instancePar_AS_dX=0.0;
instanceStruct.instancePar_given_AS=in.given("AS");
instanceStruct.instancePar_PD=in.PD;
instanceStruct.d_instancePar_PD_dX=0.0;
instanceStruct.instancePar_given_PD=in.given("PD");
instanceStruct.instancePar_PS=in.PS;
instanceStruct.d_instancePar_PS_dX=0.0;
instanceStruct.instancePar_given_PS=in.given("PS");
instanceStruct.instancePar_NRD=in.NRD;
instanceStruct.d_instancePar_NRD_dX=0.0;
instanceStruct.instancePar_given_NRD=in.given("NRD");
instanceStruct.instancePar_NRS=in.NRS;
instanceStruct.d_instancePar_NRS_dX=0.0;
instanceStruct.instancePar_given_NRS=in.given("NRS");
instanceStruct.instancePar_RTH0=in.RTH0;
instanceStruct.d_instancePar_RTH0_dX=0.0;
instanceStruct.instancePar_given_RTH0=in.given("RTH0");
instanceStruct.instancePar_CTH0=in.CTH0;
instanceStruct.d_instancePar_CTH0_dX=0.0;
instanceStruct.instancePar_given_CTH0=in.given("CTH0");
instanceStruct.instancePar_NRB=in.NRB;
instanceStruct.d_instancePar_NRB_dX=0.0;
instanceStruct.instancePar_given_NRB=in.given("NRB");
instanceStruct.instancePar_FRBODY=in.FRBODY;
instanceStruct.d_instancePar_FRBODY_dX=0.0;
instanceStruct.instancePar_given_FRBODY=in.given("FRBODY");
instanceStruct.instancePar_RBDB=in.RBDB;
instanceStruct.d_instancePar_RBDB_dX=0.0;
instanceStruct.instancePar_given_RBDB=in.given("RBDB");
instanceStruct.instancePar_RBSB=in.RBSB;
instanceStruct.d_instancePar_RBSB_dX=0.0;
instanceStruct.instancePar_given_RBSB=in.given("RBSB");
instanceStruct.instancePar_DELVTO=in.DELVTO;
instanceStruct.d_instancePar_DELVTO_dX=0.0;
instanceStruct.instancePar_given_DELVTO=in.given("DELVTO");
instanceStruct.instancePar_NBC=in.NBC;
instanceStruct.d_instancePar_NBC_dX=0.0;
instanceStruct.instancePar_given_NBC=in.given("NBC");
instanceStruct.instancePar_NSEG=in.NSEG;
instanceStruct.d_instancePar_NSEG_dX=0.0;
instanceStruct.instancePar_given_NSEG=in.given("NSEG");
instanceStruct.instancePar_PDBCP=in.PDBCP;
instanceStruct.d_instancePar_PDBCP_dX=0.0;
instanceStruct.instancePar_given_PDBCP=in.given("PDBCP");
instanceStruct.instancePar_PSBCP=in.PSBCP;
instanceStruct.d_instancePar_PSBCP_dX=0.0;
instanceStruct.instancePar_given_PSBCP=in.given("PSBCP");
instanceStruct.instancePar_AGBCP=in.AGBCP;
instanceStruct.d_instancePar_AGBCP_dX=0.0;
instanceStruct.instancePar_given_AGBCP=in.given("AGBCP");
instanceStruct.instancePar_AGBCP2=in.AGBCP2;
instanceStruct.d_instancePar_AGBCP2_dX=0.0;
instanceStruct.instancePar_given_AGBCP2=in.given("AGBCP2");
instanceStruct.instancePar_AGBCPD=in.AGBCPD;
instanceStruct.d_instancePar_AGBCPD_dX=0.0;
instanceStruct.instancePar_given_AGBCPD=in.given("AGBCPD");
instanceStruct.instancePar_AEBCP=in.AEBCP;
instanceStruct.d_instancePar_AEBCP_dX=0.0;
instanceStruct.instancePar_given_AEBCP=in.given("AEBCP");
instanceStruct.instancePar_m=in.m;
instanceStruct.d_instancePar_m_dX=0.0;
instanceStruct.instancePar_given_m=in.given("m");


  // real hidden instance


  // Copy all the non-real instance params into vars of their appropriate type:
instanceStruct.instancePar_NF=in.NF;
instanceStruct.instancePar_given_NF=in.given("NF");
instanceStruct.instancePar_OFF=in.OFF;
instanceStruct.instancePar_given_OFF=in.given("OFF");
instanceStruct.instancePar_BJTOFF=in.BJTOFF;
instanceStruct.instancePar_given_BJTOFF=in.given("BJTOFF");
instanceStruct.instancePar_DEBUG=in.DEBUG;
instanceStruct.instancePar_given_DEBUG=in.given("DEBUG");
instanceStruct.instancePar_SOIMOD=in.SOIMOD;
instanceStruct.instancePar_given_SOIMOD=in.given("SOIMOD");
instanceStruct.instancePar_TNODEOUT=in.TNODEOUT;
instanceStruct.instancePar_given_TNODEOUT=in.given("TNODEOUT");
instanceStruct.instancePar_SHMOD=in.SHMOD;
instanceStruct.instancePar_given_SHMOD=in.given("SHMOD");




  // If there are any instance parameters that were not given, and have
  // corresponding model params, copy the model param into the instance.
  // This was already done by the instance constructor, but we do it again
  // because now we're propagating derivatives, and the user could be trying
  // to get sensitivity to the model parameter.
   if (!(in.given("L")))
   {
      instanceStruct.instancePar_L = modelStruct.modelPar_L;
      instanceStruct.d_instancePar_L_dX = modelStruct.d_modelPar_L_dX;
   }
   if (!(in.given("W")))
   {
      instanceStruct.instancePar_W = modelStruct.modelPar_W;
      instanceStruct.d_instancePar_W_dX = modelStruct.d_modelPar_W_dX;
   }
   if (!(in.given("NF")))
   {
      instanceStruct.instancePar_NF = modelStruct.modelPar_NF;
   }
   if (!(in.given("SA")))
   {
      instanceStruct.instancePar_SA = modelStruct.modelPar_SA;
      instanceStruct.d_instancePar_SA_dX = modelStruct.d_modelPar_SA_dX;
   }
   if (!(in.given("SB")))
   {
      instanceStruct.instancePar_SB = modelStruct.modelPar_SB;
      instanceStruct.d_instancePar_SB_dX = modelStruct.d_modelPar_SB_dX;
   }
   if (!(in.given("SD")))
   {
      instanceStruct.instancePar_SD = modelStruct.modelPar_SD;
      instanceStruct.d_instancePar_SD_dX = modelStruct.d_modelPar_SD_dX;
   }
   if (!(in.given("AD")))
   {
      instanceStruct.instancePar_AD = modelStruct.modelPar_AD;
      instanceStruct.d_instancePar_AD_dX = modelStruct.d_modelPar_AD_dX;
   }
   if (!(in.given("AS")))
   {
      instanceStruct.instancePar_AS = modelStruct.modelPar_AS;
      instanceStruct.d_instancePar_AS_dX = modelStruct.d_modelPar_AS_dX;
   }
   if (!(in.given("PD")))
   {
      instanceStruct.instancePar_PD = modelStruct.modelPar_PD;
      instanceStruct.d_instancePar_PD_dX = modelStruct.d_modelPar_PD_dX;
   }
   if (!(in.given("PS")))
   {
      instanceStruct.instancePar_PS = modelStruct.modelPar_PS;
      instanceStruct.d_instancePar_PS_dX = modelStruct.d_modelPar_PS_dX;
   }
   if (!(in.given("NRD")))
   {
      instanceStruct.instancePar_NRD = modelStruct.modelPar_NRD;
      instanceStruct.d_instancePar_NRD_dX = modelStruct.d_modelPar_NRD_dX;
   }
   if (!(in.given("NRS")))
   {
      instanceStruct.instancePar_NRS = modelStruct.modelPar_NRS;
      instanceStruct.d_instancePar_NRS_dX = modelStruct.d_modelPar_NRS_dX;
   }
   if (!(in.given("OFF")))
   {
      instanceStruct.instancePar_OFF = modelStruct.modelPar_OFF;
   }
   if (!(in.given("BJTOFF")))
   {
      instanceStruct.instancePar_BJTOFF = modelStruct.modelPar_BJTOFF;
   }
   if (!(in.given("RTH0")))
   {
      instanceStruct.instancePar_RTH0 = modelStruct.modelPar_RTH0;
      instanceStruct.d_instancePar_RTH0_dX = modelStruct.d_modelPar_RTH0_dX;
   }
   if (!(in.given("CTH0")))
   {
      instanceStruct.instancePar_CTH0 = modelStruct.modelPar_CTH0;
      instanceStruct.d_instancePar_CTH0_dX = modelStruct.d_modelPar_CTH0_dX;
   }
   if (!(in.given("NRB")))
   {
      instanceStruct.instancePar_NRB = modelStruct.modelPar_NRB;
      instanceStruct.d_instancePar_NRB_dX = modelStruct.d_modelPar_NRB_dX;
   }
   if (!(in.given("FRBODY")))
   {
      instanceStruct.instancePar_FRBODY = modelStruct.modelPar_FRBODY;
      instanceStruct.d_instancePar_FRBODY_dX = modelStruct.d_modelPar_FRBODY_dX;
   }
   if (!(in.given("RBDB")))
   {
      instanceStruct.instancePar_RBDB = modelStruct.modelPar_RBDB;
      instanceStruct.d_instancePar_RBDB_dX = modelStruct.d_modelPar_RBDB_dX;
   }
   if (!(in.given("RBSB")))
   {
      instanceStruct.instancePar_RBSB = modelStruct.modelPar_RBSB;
      instanceStruct.d_instancePar_RBSB_dX = modelStruct.d_modelPar_RBSB_dX;
   }
   if (!(in.given("DELVTO")))
   {
      instanceStruct.instancePar_DELVTO = modelStruct.modelPar_DELVTO;
      instanceStruct.d_instancePar_DELVTO_dX = modelStruct.d_modelPar_DELVTO_dX;
   }
   if (!(in.given("SOIMOD")))
   {
      instanceStruct.instancePar_SOIMOD = modelStruct.modelPar_SOIMOD;
   }
   if (!(in.given("NBC")))
   {
      instanceStruct.instancePar_NBC = modelStruct.modelPar_NBC;
      instanceStruct.d_instancePar_NBC_dX = modelStruct.d_modelPar_NBC_dX;
   }
   if (!(in.given("NSEG")))
   {
      instanceStruct.instancePar_NSEG = modelStruct.modelPar_NSEG;
      instanceStruct.d_instancePar_NSEG_dX = modelStruct.d_modelPar_NSEG_dX;
   }
   if (!(in.given("PDBCP")))
   {
      instanceStruct.instancePar_PDBCP = modelStruct.modelPar_PDBCP;
      instanceStruct.d_instancePar_PDBCP_dX = modelStruct.d_modelPar_PDBCP_dX;
   }
   if (!(in.given("PSBCP")))
   {
      instanceStruct.instancePar_PSBCP = modelStruct.modelPar_PSBCP;
      instanceStruct.d_instancePar_PSBCP_dX = modelStruct.d_modelPar_PSBCP_dX;
   }
   if (!(in.given("AGBCP")))
   {
      instanceStruct.instancePar_AGBCP = modelStruct.modelPar_AGBCP;
      instanceStruct.d_instancePar_AGBCP_dX = modelStruct.d_modelPar_AGBCP_dX;
   }
   if (!(in.given("AGBCP2")))
   {
      instanceStruct.instancePar_AGBCP2 = modelStruct.modelPar_AGBCP2;
      instanceStruct.d_instancePar_AGBCP2_dX = modelStruct.d_modelPar_AGBCP2_dX;
   }
   if (!(in.given("AGBCPD")))
   {
      instanceStruct.instancePar_AGBCPD = modelStruct.modelPar_AGBCPD;
      instanceStruct.d_instancePar_AGBCPD_dX = modelStruct.d_modelPar_AGBCPD_dX;
   }
   if (!(in.given("AEBCP")))
   {
      instanceStruct.instancePar_AEBCP = modelStruct.modelPar_AEBCP;
      instanceStruct.d_instancePar_AEBCP_dX = modelStruct.d_modelPar_AEBCP_dX;
   }
   if (!(in.given("SHMOD")))
   {
      instanceStruct.instancePar_SHMOD = modelStruct.modelPar_SHMOD;
   }


  //make local copies of all instance vars
  //reals
instanceStruct.instanceVar_B4SOInf=in.B4SOInf;
instanceStruct.d_instanceVar_B4SOInf_dX=0.0;
instanceStruct.instanceVar_B4SOIsa=in.B4SOIsa;
instanceStruct.d_instanceVar_B4SOIsa_dX=0.0;
instanceStruct.instanceVar_B4SOIsb=in.B4SOIsb;
instanceStruct.d_instanceVar_B4SOIsb_dX=0.0;
instanceStruct.instanceVar_B4SOIsd=in.B4SOIsd;
instanceStruct.d_instanceVar_B4SOIsd_dX=0.0;
instanceStruct.instanceVar_B4SOIrth0=in.B4SOIrth0;
instanceStruct.d_instanceVar_B4SOIrth0_dX=0.0;
instanceStruct.instanceVar_B4SOIcth0=in.B4SOIcth0;
instanceStruct.d_instanceVar_B4SOIcth0_dX=0.0;
instanceStruct.instanceVar_B4SOInbc=in.B4SOInbc;
instanceStruct.d_instanceVar_B4SOInbc_dX=0.0;
instanceStruct.instanceVar_B4SOInseg=in.B4SOInseg;
instanceStruct.d_instanceVar_B4SOInseg_dX=0.0;
instanceStruct.instanceVar_B4SOIagbcp=in.B4SOIagbcp;
instanceStruct.d_instanceVar_B4SOIagbcp_dX=0.0;
instanceStruct.instanceVar_B4SOIagbcp2=in.B4SOIagbcp2;
instanceStruct.d_instanceVar_B4SOIagbcp2_dX=0.0;
instanceStruct.instanceVar_B4SOIagbcpd=in.B4SOIagbcpd;
instanceStruct.d_instanceVar_B4SOIagbcpd_dX=0.0;
instanceStruct.instanceVar_B4SOIaebcp=in.B4SOIaebcp;
instanceStruct.d_instanceVar_B4SOIaebcp_dX=0.0;
instanceStruct.instanceVar_B4SOIeot=in.B4SOIeot;
instanceStruct.d_instanceVar_B4SOIeot_dX=0.0;
instanceStruct.instanceVar_B4SOIepsrsub=in.B4SOIepsrsub;
instanceStruct.d_instanceVar_B4SOIepsrsub_dX=0.0;
instanceStruct.instanceVar_B4SOIni0sub=in.B4SOIni0sub;
instanceStruct.d_instanceVar_B4SOIni0sub_dX=0.0;
instanceStruct.instanceVar_B4SOIbg0sub=in.B4SOIbg0sub;
instanceStruct.d_instanceVar_B4SOIbg0sub_dX=0.0;
instanceStruct.instanceVar_B4SOItbgasub=in.B4SOItbgasub;
instanceStruct.d_instanceVar_B4SOItbgasub_dX=0.0;
instanceStruct.instanceVar_B4SOItbgbsub=in.B4SOItbgbsub;
instanceStruct.d_instanceVar_B4SOItbgbsub_dX=0.0;
instanceStruct.instanceVar_B4SOIphig=in.B4SOIphig;
instanceStruct.d_instanceVar_B4SOIphig_dX=0.0;
instanceStruct.instanceVar_B4SOIeasub=in.B4SOIeasub;
instanceStruct.d_instanceVar_B4SOIeasub_dX=0.0;
instanceStruct.instanceVar_B4SOIleffeot=in.B4SOIleffeot;
instanceStruct.d_instanceVar_B4SOIleffeot_dX=0.0;
instanceStruct.instanceVar_B4SOIweffeot=in.B4SOIweffeot;
instanceStruct.d_instanceVar_B4SOIweffeot_dX=0.0;
instanceStruct.instanceVar_B4SOIados=in.B4SOIados;
instanceStruct.d_instanceVar_B4SOIados_dX=0.0;
instanceStruct.instanceVar_B4SOIbdos=in.B4SOIbdos;
instanceStruct.d_instanceVar_B4SOIbdos_dX=0.0;
instanceStruct.instanceVar_B4SOIepsrgate=in.B4SOIepsrgate;
instanceStruct.d_instanceVar_B4SOIepsrgate_dX=0.0;
instanceStruct.instanceVar_B4SOItox=in.B4SOItox;
instanceStruct.d_instanceVar_B4SOItox_dX=0.0;
instanceStruct.instanceVar_B4SOItoxp=in.B4SOItoxp;
instanceStruct.d_instanceVar_B4SOItoxp_dX=0.0;
instanceStruct.instanceVar_B4SOItoxm=in.B4SOItoxm;
instanceStruct.d_instanceVar_B4SOItoxm_dX=0.0;
instanceStruct.instanceVar_B4SOIdtoxcv=in.B4SOIdtoxcv;
instanceStruct.d_instanceVar_B4SOIdtoxcv_dX=0.0;
instanceStruct.instanceVar_B4SOIngate=in.B4SOIngate;
instanceStruct.d_instanceVar_B4SOIngate_dX=0.0;
instanceStruct.instanceVar_B4SOInsd=in.B4SOInsd;
instanceStruct.d_instanceVar_B4SOInsd_dX=0.0;
instanceStruct.instanceVar_B4SOItnom=in.B4SOItnom;
instanceStruct.d_instanceVar_B4SOItnom_dX=0.0;
instanceStruct.instanceVar_B4SOIxpart=in.B4SOIxpart;
instanceStruct.d_instanceVar_B4SOIxpart_dX=0.0;
instanceStruct.instanceVar_B4SOIrswmin=in.B4SOIrswmin;
instanceStruct.d_instanceVar_B4SOIrswmin_dX=0.0;
instanceStruct.instanceVar_B4SOIrdwmin=in.B4SOIrdwmin;
instanceStruct.d_instanceVar_B4SOIrdwmin_dX=0.0;
instanceStruct.instanceVar_B4SOItbox=in.B4SOItbox;
instanceStruct.d_instanceVar_B4SOItbox_dX=0.0;
instanceStruct.instanceVar_B4SOItsi=in.B4SOItsi;
instanceStruct.d_instanceVar_B4SOItsi_dX=0.0;
instanceStruct.instanceVar_B4SOIegidl=in.B4SOIegidl;
instanceStruct.d_instanceVar_B4SOIegidl_dX=0.0;
instanceStruct.instanceVar_B4SOIagidl=in.B4SOIagidl;
instanceStruct.d_instanceVar_B4SOIagidl_dX=0.0;
instanceStruct.instanceVar_B4SOIbgidl=in.B4SOIbgidl;
instanceStruct.d_instanceVar_B4SOIbgidl_dX=0.0;
instanceStruct.instanceVar_B4SOIcgidl=in.B4SOIcgidl;
instanceStruct.d_instanceVar_B4SOIcgidl_dX=0.0;
instanceStruct.instanceVar_B4SOIegisl=in.B4SOIegisl;
instanceStruct.d_instanceVar_B4SOIegisl_dX=0.0;
instanceStruct.instanceVar_B4SOIagisl=in.B4SOIagisl;
instanceStruct.d_instanceVar_B4SOIagisl_dX=0.0;
instanceStruct.instanceVar_B4SOIbgisl=in.B4SOIbgisl;
instanceStruct.d_instanceVar_B4SOIbgisl_dX=0.0;
instanceStruct.instanceVar_B4SOIcgisl=in.B4SOIcgisl;
instanceStruct.d_instanceVar_B4SOIcgisl_dX=0.0;
instanceStruct.instanceVar_B4SOIndiode=in.B4SOIndiode;
instanceStruct.d_instanceVar_B4SOIndiode_dX=0.0;
instanceStruct.instanceVar_B4SOIndioded=in.B4SOIndioded;
instanceStruct.d_instanceVar_B4SOIndioded_dX=0.0;
instanceStruct.instanceVar_B4SOIGatesidewallJctSPotential=in.B4SOIGatesidewallJctSPotential;
instanceStruct.d_instanceVar_B4SOIGatesidewallJctSPotential_dX=0.0;
instanceStruct.instanceVar_B4SOIGatesidewallJctDPotential=in.B4SOIGatesidewallJctDPotential;
instanceStruct.d_instanceVar_B4SOIGatesidewallJctDPotential_dX=0.0;
instanceStruct.instanceVar_B4SOIbodyJctGateSideSGradingCoeff=in.B4SOIbodyJctGateSideSGradingCoeff;
instanceStruct.d_instanceVar_B4SOIbodyJctGateSideSGradingCoeff_dX=0.0;
instanceStruct.instanceVar_B4SOIbodyJctGateSideDGradingCoeff=in.B4SOIbodyJctGateSideDGradingCoeff;
instanceStruct.d_instanceVar_B4SOIbodyJctGateSideDGradingCoeff_dX=0.0;
instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapS=in.B4SOIunitLengthGateSidewallJctCapS;
instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapS_dX=0.0;
instanceStruct.instanceVar_B4SOIunitLengthGateSidewallJctCapD=in.B4SOIunitLengthGateSidewallJctCapD;
instanceStruct.d_instanceVar_B4SOIunitLengthGateSidewallJctCapD_dX=0.0;
instanceStruct.instanceVar_B4SOIoxideTrapDensityA=in.B4SOIoxideTrapDensityA;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityA_dX=0.0;
instanceStruct.instanceVar_B4SOIoxideTrapDensityB=in.B4SOIoxideTrapDensityB;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityB_dX=0.0;
instanceStruct.instanceVar_B4SOIoxideTrapDensityC=in.B4SOIoxideTrapDensityC;
instanceStruct.d_instanceVar_B4SOIoxideTrapDensityC_dX=0.0;
instanceStruct.instanceVar_B4SOItnoia=in.B4SOItnoia;
instanceStruct.d_instanceVar_B4SOItnoia_dX=0.0;
instanceStruct.instanceVar_B4SOItnoib=in.B4SOItnoib;
instanceStruct.d_instanceVar_B4SOItnoib_dX=0.0;
instanceStruct.instanceVar_B4SOIrnoia=in.B4SOIrnoia;
instanceStruct.d_instanceVar_B4SOIrnoia_dX=0.0;
instanceStruct.instanceVar_B4SOIrnoib=in.B4SOIrnoib;
instanceStruct.d_instanceVar_B4SOIrnoib_dX=0.0;
instanceStruct.instanceVar_B4SOIntnoi=in.B4SOIntnoi;
instanceStruct.d_instanceVar_B4SOIntnoi_dX=0.0;
instanceStruct.instanceVar_B4SOIsaref=in.B4SOIsaref;
instanceStruct.d_instanceVar_B4SOIsaref_dX=0.0;
instanceStruct.instanceVar_B4SOIsbref=in.B4SOIsbref;
instanceStruct.d_instanceVar_B4SOIsbref_dX=0.0;
instanceStruct.instanceVar_B4SOIku0=in.B4SOIku0;
instanceStruct.d_instanceVar_B4SOIku0_dX=0.0;
instanceStruct.instanceVar_B4SOIkvsat=in.B4SOIkvsat;
instanceStruct.d_instanceVar_B4SOIkvsat_dX=0.0;
instanceStruct.instanceVar_B4SOItku0=in.B4SOItku0;
instanceStruct.d_instanceVar_B4SOItku0_dX=0.0;
instanceStruct.instanceVar_B4SOIlodk2=in.B4SOIlodk2;
instanceStruct.d_instanceVar_B4SOIlodk2_dX=0.0;
instanceStruct.instanceVar_B4SOIlodeta0=in.B4SOIlodeta0;
instanceStruct.d_instanceVar_B4SOIlodeta0_dX=0.0;
instanceStruct.instanceVar_B4SOIlodeta0cv=in.B4SOIlodeta0cv;
instanceStruct.d_instanceVar_B4SOIlodeta0cv_dX=0.0;
instanceStruct.instanceVar_B4SOIbf=in.B4SOIbf;
instanceStruct.d_instanceVar_B4SOIbf_dX=0.0;
instanceStruct.instanceVar_B4SOIw0flk=in.B4SOIw0flk;
instanceStruct.d_instanceVar_B4SOIw0flk_dX=0.0;
instanceStruct.instanceVar_B4SOIpditsl=in.B4SOIpditsl;
instanceStruct.d_instanceVar_B4SOIpditsl_dX=0.0;
instanceStruct.instanceVar_B4SOIem=in.B4SOIem;
instanceStruct.d_instanceVar_B4SOIem_dX=0.0;
instanceStruct.instanceVar_B4SOIef=in.B4SOIef;
instanceStruct.d_instanceVar_B4SOIef_dX=0.0;
instanceStruct.instanceVar_B4SOIaf=in.B4SOIaf;
instanceStruct.d_instanceVar_B4SOIaf_dX=0.0;
instanceStruct.instanceVar_B4SOIkf=in.B4SOIkf;
instanceStruct.d_instanceVar_B4SOIkf_dX=0.0;
instanceStruct.instanceVar_B4SOInoif=in.B4SOInoif;
instanceStruct.d_instanceVar_B4SOInoif_dX=0.0;
instanceStruct.instanceVar_B4SOIk1w1=in.B4SOIk1w1;
instanceStruct.d_instanceVar_B4SOIk1w1_dX=0.0;
instanceStruct.instanceVar_B4SOIk1w2=in.B4SOIk1w2;
instanceStruct.d_instanceVar_B4SOIk1w2_dX=0.0;
instanceStruct.instanceVar_B4SOIketas=in.B4SOIketas;
instanceStruct.d_instanceVar_B4SOIketas_dX=0.0;
instanceStruct.instanceVar_B4SOIdwbc=in.B4SOIdwbc;
instanceStruct.d_instanceVar_B4SOIdwbc_dX=0.0;
instanceStruct.instanceVar_B4SOIbeta0=in.B4SOIbeta0;
instanceStruct.d_instanceVar_B4SOIbeta0_dX=0.0;
instanceStruct.instanceVar_B4SOIbeta1=in.B4SOIbeta1;
instanceStruct.d_instanceVar_B4SOIbeta1_dX=0.0;
instanceStruct.instanceVar_B4SOIbeta2=in.B4SOIbeta2;
instanceStruct.d_instanceVar_B4SOIbeta2_dX=0.0;
instanceStruct.instanceVar_B4SOItii=in.B4SOItii;
instanceStruct.d_instanceVar_B4SOItii_dX=0.0;
instanceStruct.instanceVar_B4SOIlii=in.B4SOIlii;
instanceStruct.d_instanceVar_B4SOIlii_dX=0.0;
instanceStruct.instanceVar_B4SOIsii1=in.B4SOIsii1;
instanceStruct.d_instanceVar_B4SOIsii1_dX=0.0;
instanceStruct.instanceVar_B4SOIsii2=in.B4SOIsii2;
instanceStruct.d_instanceVar_B4SOIsii2_dX=0.0;
instanceStruct.instanceVar_B4SOIsiid=in.B4SOIsiid;
instanceStruct.d_instanceVar_B4SOIsiid_dX=0.0;
instanceStruct.instanceVar_B4SOIfbjtii=in.B4SOIfbjtii;
instanceStruct.d_instanceVar_B4SOIfbjtii_dX=0.0;
instanceStruct.instanceVar_B4SOItvbci=in.B4SOItvbci;
instanceStruct.d_instanceVar_B4SOItvbci_dX=0.0;
instanceStruct.instanceVar_B4SOIesatii=in.B4SOIesatii;
instanceStruct.d_instanceVar_B4SOIesatii_dX=0.0;
instanceStruct.instanceVar_B4SOIntun=in.B4SOIntun;
instanceStruct.d_instanceVar_B4SOIntun_dX=0.0;
instanceStruct.instanceVar_B4SOIntund=in.B4SOIntund;
instanceStruct.d_instanceVar_B4SOIntund_dX=0.0;
instanceStruct.instanceVar_B4SOIisbjt=in.B4SOIisbjt;
instanceStruct.d_instanceVar_B4SOIisbjt_dX=0.0;
instanceStruct.instanceVar_B4SOIidbjt=in.B4SOIidbjt;
instanceStruct.d_instanceVar_B4SOIidbjt_dX=0.0;
instanceStruct.instanceVar_B4SOIisdif=in.B4SOIisdif;
instanceStruct.d_instanceVar_B4SOIisdif_dX=0.0;
instanceStruct.instanceVar_B4SOIiddif=in.B4SOIiddif;
instanceStruct.d_instanceVar_B4SOIiddif_dX=0.0;
instanceStruct.instanceVar_B4SOIisrec=in.B4SOIisrec;
instanceStruct.d_instanceVar_B4SOIisrec_dX=0.0;
instanceStruct.instanceVar_B4SOIidrec=in.B4SOIidrec;
instanceStruct.d_instanceVar_B4SOIidrec_dX=0.0;
instanceStruct.instanceVar_B4SOIistun=in.B4SOIistun;
instanceStruct.d_instanceVar_B4SOIistun_dX=0.0;
instanceStruct.instanceVar_B4SOIidtun=in.B4SOIidtun;
instanceStruct.d_instanceVar_B4SOIidtun_dX=0.0;
instanceStruct.instanceVar_B4SOIvrec0=in.B4SOIvrec0;
instanceStruct.d_instanceVar_B4SOIvrec0_dX=0.0;
instanceStruct.instanceVar_B4SOIvrec0d=in.B4SOIvrec0d;
instanceStruct.d_instanceVar_B4SOIvrec0d_dX=0.0;
instanceStruct.instanceVar_B4SOIvtun0=in.B4SOIvtun0;
instanceStruct.d_instanceVar_B4SOIvtun0_dX=0.0;
instanceStruct.instanceVar_B4SOIvtun0d=in.B4SOIvtun0d;
instanceStruct.d_instanceVar_B4SOIvtun0d_dX=0.0;
instanceStruct.instanceVar_B4SOInbjt=in.B4SOInbjt;
instanceStruct.d_instanceVar_B4SOInbjt_dX=0.0;
instanceStruct.instanceVar_B4SOIaely=in.B4SOIaely;
instanceStruct.d_instanceVar_B4SOIaely_dX=0.0;
instanceStruct.instanceVar_B4SOIahli=in.B4SOIahli;
instanceStruct.d_instanceVar_B4SOIahli_dX=0.0;
instanceStruct.instanceVar_B4SOIahlid=in.B4SOIahlid;
instanceStruct.d_instanceVar_B4SOIahlid_dX=0.0;
instanceStruct.instanceVar_B4SOIrbody=in.B4SOIrbody;
instanceStruct.d_instanceVar_B4SOIrbody_dX=0.0;
instanceStruct.instanceVar_B4SOIrbsh=in.B4SOIrbsh;
instanceStruct.d_instanceVar_B4SOIrbsh_dX=0.0;
instanceStruct.instanceVar_B4SOItt=in.B4SOItt;
instanceStruct.d_instanceVar_B4SOItt_dX=0.0;
instanceStruct.instanceVar_B4SOIcsdmin=in.B4SOIcsdmin;
instanceStruct.d_instanceVar_B4SOIcsdmin_dX=0.0;
instanceStruct.instanceVar_B4SOIasd=in.B4SOIasd;
instanceStruct.d_instanceVar_B4SOIasd_dX=0.0;
instanceStruct.instanceVar_B4SOIcsdesw=in.B4SOIcsdesw;
instanceStruct.d_instanceVar_B4SOIcsdesw_dX=0.0;
instanceStruct.instanceVar_B4SOIfbody=in.B4SOIfbody;
instanceStruct.d_instanceVar_B4SOIfbody_dX=0.0;
instanceStruct.instanceVar_B4SOItcjswg=in.B4SOItcjswg;
instanceStruct.d_instanceVar_B4SOItcjswg_dX=0.0;
instanceStruct.instanceVar_B4SOItpbswg=in.B4SOItpbswg;
instanceStruct.d_instanceVar_B4SOItpbswg_dX=0.0;
instanceStruct.instanceVar_B4SOItcjswgd=in.B4SOItcjswgd;
instanceStruct.d_instanceVar_B4SOItcjswgd_dX=0.0;
instanceStruct.instanceVar_B4SOItpbswgd=in.B4SOItpbswgd;
instanceStruct.d_instanceVar_B4SOItpbswgd_dX=0.0;
instanceStruct.instanceVar_B4SOIacde=in.B4SOIacde;
instanceStruct.d_instanceVar_B4SOIacde_dX=0.0;
instanceStruct.instanceVar_B4SOImoin=in.B4SOImoin;
instanceStruct.d_instanceVar_B4SOImoin_dX=0.0;
instanceStruct.instanceVar_B4SOIdlbg=in.B4SOIdlbg;
instanceStruct.d_instanceVar_B4SOIdlbg_dX=0.0;
instanceStruct.instanceVar_B4SOItoxqm=in.B4SOItoxqm;
instanceStruct.d_instanceVar_B4SOItoxqm_dX=0.0;
instanceStruct.instanceVar_B4SOIwth0=in.B4SOIwth0;
instanceStruct.d_instanceVar_B4SOIwth0_dX=0.0;
instanceStruct.instanceVar_B4SOIrhalo=in.B4SOIrhalo;
instanceStruct.d_instanceVar_B4SOIrhalo_dX=0.0;
instanceStruct.instanceVar_B4SOIntox=in.B4SOIntox;
instanceStruct.d_instanceVar_B4SOIntox_dX=0.0;
instanceStruct.instanceVar_B4SOItoxref=in.B4SOItoxref;
instanceStruct.d_instanceVar_B4SOItoxref_dX=0.0;
instanceStruct.instanceVar_B4SOIebg=in.B4SOIebg;
instanceStruct.d_instanceVar_B4SOIebg_dX=0.0;
instanceStruct.instanceVar_B4SOIvevb=in.B4SOIvevb;
instanceStruct.d_instanceVar_B4SOIvevb_dX=0.0;
instanceStruct.instanceVar_B4SOIvgb1=in.B4SOIvgb1;
instanceStruct.d_instanceVar_B4SOIvgb1_dX=0.0;
instanceStruct.instanceVar_B4SOIvecb=in.B4SOIvecb;
instanceStruct.d_instanceVar_B4SOIvecb_dX=0.0;
instanceStruct.instanceVar_B4SOIvgb2=in.B4SOIvgb2;
instanceStruct.d_instanceVar_B4SOIvgb2_dX=0.0;
instanceStruct.instanceVar_B4SOIvoxh=in.B4SOIvoxh;
instanceStruct.d_instanceVar_B4SOIvoxh_dX=0.0;
instanceStruct.instanceVar_B4SOIdeltavox=in.B4SOIdeltavox;
instanceStruct.d_instanceVar_B4SOIdeltavox_dX=0.0;
instanceStruct.instanceVar_B4SOIvtm=in.B4SOIvtm;
instanceStruct.d_instanceVar_B4SOIvtm_dX=0.0;
instanceStruct.instanceVar_B4SOIueff=in.B4SOIueff;
instanceStruct.d_instanceVar_B4SOIueff_dX=0.0;
instanceStruct.instanceVar_B4SOIthetavth=in.B4SOIthetavth;
instanceStruct.d_instanceVar_B4SOIthetavth_dX=0.0;
instanceStruct.instanceVar_B4SOIInv_ODeff=in.B4SOIInv_ODeff;
instanceStruct.d_instanceVar_B4SOIInv_ODeff_dX=0.0;
instanceStruct.instanceVar_B4SOIgrbsb=in.B4SOIgrbsb;
instanceStruct.d_instanceVar_B4SOIgrbsb_dX=0.0;
instanceStruct.instanceVar_B4SOIgrbdb=in.B4SOIgrbdb;
instanceStruct.d_instanceVar_B4SOIgrbdb_dX=0.0;
instanceStruct.instanceVar_B4SOITempSH=in.B4SOITempSH;
instanceStruct.d_instanceVar_B4SOITempSH_dX=0.0;
instanceStruct.instanceVar_B4SOIgrgeltd=in.B4SOIgrgeltd;
instanceStruct.d_instanceVar_B4SOIgrgeltd_dX=0.0;
instanceStruct.instanceVar_B4SOIsourceResistance=in.B4SOIsourceResistance;
instanceStruct.d_instanceVar_B4SOIsourceResistance_dX=0.0;
instanceStruct.instanceVar_B4SOIdrainResistance=in.B4SOIdrainResistance;
instanceStruct.d_instanceVar_B4SOIdrainResistance_dX=0.0;
instanceStruct.instanceVar_B4SOIvfbzb=in.B4SOIvfbzb;
instanceStruct.d_instanceVar_B4SOIvfbzb_dX=0.0;
instanceStruct.instanceVar_B4SOIvtfbphi1=in.B4SOIvtfbphi1;
instanceStruct.d_instanceVar_B4SOIvtfbphi1_dX=0.0;
instanceStruct.instanceVar_B4SOIvgsteffvth=in.B4SOIvgsteffvth;
instanceStruct.d_instanceVar_B4SOIvgsteffvth_dX=0.0;
instanceStruct.instanceVar_B4SOIrbodyext=in.B4SOIrbodyext;
instanceStruct.d_instanceVar_B4SOIrbodyext_dX=0.0;
instanceStruct.instanceVar_B4SOIcsesw=in.B4SOIcsesw;
instanceStruct.d_instanceVar_B4SOIcsesw_dX=0.0;
instanceStruct.instanceVar_B4SOIcdesw=in.B4SOIcdesw;
instanceStruct.d_instanceVar_B4SOIcdesw_dX=0.0;
instanceStruct.instanceVar_B4SOIcsbox=in.B4SOIcsbox;
instanceStruct.d_instanceVar_B4SOIcsbox_dX=0.0;
instanceStruct.instanceVar_B4SOIcdbox=in.B4SOIcdbox;
instanceStruct.d_instanceVar_B4SOIcdbox_dX=0.0;
instanceStruct.instanceVar_B4SOIcsmin=in.B4SOIcsmin;
instanceStruct.d_instanceVar_B4SOIcsmin_dX=0.0;
instanceStruct.instanceVar_B4SOIcdmin=in.B4SOIcdmin;
instanceStruct.d_instanceVar_B4SOIcdmin_dX=0.0;
instanceStruct.instanceVar_B4SOIst4=in.B4SOIst4;
instanceStruct.d_instanceVar_B4SOIst4_dX=0.0;
instanceStruct.instanceVar_B4SOIdt4=in.B4SOIdt4;
instanceStruct.d_instanceVar_B4SOIdt4_dX=0.0;
instanceStruct.instanceVar_B4SOInstar=in.B4SOInstar;
instanceStruct.d_instanceVar_B4SOInstar_dX=0.0;
instanceStruct.instanceVar_B4SOIAbulk=in.B4SOIAbulk;
instanceStruct.d_instanceVar_B4SOIAbulk_dX=0.0;
instanceStruct.instanceVar_B4SOIqinv=in.B4SOIqinv;
instanceStruct.d_instanceVar_B4SOIqinv_dX=0.0;
instanceStruct.instanceVar_B4SOIgm=in.B4SOIgm;
instanceStruct.d_instanceVar_B4SOIgm_dX=0.0;
instanceStruct.instanceVar_B4SOIgds=in.B4SOIgds;
instanceStruct.d_instanceVar_B4SOIgds_dX=0.0;
instanceStruct.instanceVar_B4SOIgmbs=in.B4SOIgmbs;
instanceStruct.d_instanceVar_B4SOIgmbs_dX=0.0;
instanceStruct.instanceVar_B4SOIrds=in.B4SOIrds;
instanceStruct.d_instanceVar_B4SOIrds_dX=0.0;
instanceStruct.instanceVar_B4SOIVgsteff=in.B4SOIVgsteff;
instanceStruct.d_instanceVar_B4SOIVgsteff_dX=0.0;
instanceStruct.instanceVar_B4SOIVdseff=in.B4SOIVdseff;
instanceStruct.d_instanceVar_B4SOIVdseff_dX=0.0;
instanceStruct.instanceVar_B4SOIAbovVgst2Vtm=in.B4SOIAbovVgst2Vtm;
instanceStruct.d_instanceVar_B4SOIAbovVgst2Vtm_dX=0.0;
instanceStruct.instanceVar_B4SOIidovVds=in.B4SOIidovVds;
instanceStruct.d_instanceVar_B4SOIidovVds_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdsc=in.pParam_B4SOIcdsc;
instanceStruct.d_instanceVar_pParam_B4SOIcdsc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdscb=in.pParam_B4SOIcdscb;
instanceStruct.d_instanceVar_pParam_B4SOIcdscb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdscd=in.pParam_B4SOIcdscd;
instanceStruct.d_instanceVar_pParam_B4SOIcdscd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcit=in.pParam_B4SOIcit;
instanceStruct.d_instanceVar_pParam_B4SOIcit_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInfactor=in.pParam_B4SOInfactor;
instanceStruct.d_instanceVar_pParam_B4SOInfactor_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsat=in.pParam_B4SOIvsat;
instanceStruct.d_instanceVar_pParam_B4SOIvsat_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIat=in.pParam_B4SOIat;
instanceStruct.d_instanceVar_pParam_B4SOIat_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIa0=in.pParam_B4SOIa0;
instanceStruct.d_instanceVar_pParam_B4SOIa0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIags=in.pParam_B4SOIags;
instanceStruct.d_instanceVar_pParam_B4SOIags_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIa1=in.pParam_B4SOIa1;
instanceStruct.d_instanceVar_pParam_B4SOIa1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIa2=in.pParam_B4SOIa2;
instanceStruct.d_instanceVar_pParam_B4SOIa2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIketa=in.pParam_B4SOIketa;
instanceStruct.d_instanceVar_pParam_B4SOIketa_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInpeak=in.pParam_B4SOInpeak;
instanceStruct.d_instanceVar_pParam_B4SOInpeak_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInsub=in.pParam_B4SOInsub;
instanceStruct.d_instanceVar_pParam_B4SOInsub_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIngate=in.pParam_B4SOIngate;
instanceStruct.d_instanceVar_pParam_B4SOIngate_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIgamma1=in.pParam_B4SOIgamma1;
instanceStruct.d_instanceVar_pParam_B4SOIgamma1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIgamma2=in.pParam_B4SOIgamma2;
instanceStruct.d_instanceVar_pParam_B4SOIgamma2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbx=in.pParam_B4SOIvbx;
instanceStruct.d_instanceVar_pParam_B4SOIvbx_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbi=in.pParam_B4SOIvbi;
instanceStruct.d_instanceVar_pParam_B4SOIvbi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbm=in.pParam_B4SOIvbm;
instanceStruct.d_instanceVar_pParam_B4SOIvbm_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxt=in.pParam_B4SOIxt;
instanceStruct.d_instanceVar_pParam_B4SOIxt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIphi=in.pParam_B4SOIphi;
instanceStruct.d_instanceVar_pParam_B4SOIphi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlitl=in.pParam_B4SOIlitl;
instanceStruct.d_instanceVar_pParam_B4SOIlitl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1=in.pParam_B4SOIk1;
instanceStruct.d_instanceVar_pParam_B4SOIk1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkt1=in.pParam_B4SOIkt1;
instanceStruct.d_instanceVar_pParam_B4SOIkt1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkt1l=in.pParam_B4SOIkt1l;
instanceStruct.d_instanceVar_pParam_B4SOIkt1l_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkt2=in.pParam_B4SOIkt2;
instanceStruct.d_instanceVar_pParam_B4SOIkt2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk2=in.pParam_B4SOIk2;
instanceStruct.d_instanceVar_pParam_B4SOIk2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk3=in.pParam_B4SOIk3;
instanceStruct.d_instanceVar_pParam_B4SOIk3_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk3b=in.pParam_B4SOIk3b;
instanceStruct.d_instanceVar_pParam_B4SOIk3b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIw0=in.pParam_B4SOIw0;
instanceStruct.d_instanceVar_pParam_B4SOIw0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlpe0=in.pParam_B4SOIlpe0;
instanceStruct.d_instanceVar_pParam_B4SOIlpe0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt0=in.pParam_B4SOIdvt0;
instanceStruct.d_instanceVar_pParam_B4SOIdvt0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt1=in.pParam_B4SOIdvt1;
instanceStruct.d_instanceVar_pParam_B4SOIdvt1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt2=in.pParam_B4SOIdvt2;
instanceStruct.d_instanceVar_pParam_B4SOIdvt2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt0w=in.pParam_B4SOIdvt0w;
instanceStruct.d_instanceVar_pParam_B4SOIdvt0w_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt1w=in.pParam_B4SOIdvt1w;
instanceStruct.d_instanceVar_pParam_B4SOIdvt1w_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvt2w=in.pParam_B4SOIdvt2w;
instanceStruct.d_instanceVar_pParam_B4SOIdvt2w_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdrout=in.pParam_B4SOIdrout;
instanceStruct.d_instanceVar_pParam_B4SOIdrout_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdsub=in.pParam_B4SOIdsub;
instanceStruct.d_instanceVar_pParam_B4SOIdsub_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIua=in.pParam_B4SOIua;
instanceStruct.d_instanceVar_pParam_B4SOIua_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIua1=in.pParam_B4SOIua1;
instanceStruct.d_instanceVar_pParam_B4SOIua1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIub=in.pParam_B4SOIub;
instanceStruct.d_instanceVar_pParam_B4SOIub_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIub1=in.pParam_B4SOIub1;
instanceStruct.d_instanceVar_pParam_B4SOIub1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuc=in.pParam_B4SOIuc;
instanceStruct.d_instanceVar_pParam_B4SOIuc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuc1=in.pParam_B4SOIuc1;
instanceStruct.d_instanceVar_pParam_B4SOIuc1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIu0=in.pParam_B4SOIu0;
instanceStruct.d_instanceVar_pParam_B4SOIu0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIute=in.pParam_B4SOIute;
instanceStruct.d_instanceVar_pParam_B4SOIute_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIud=in.pParam_B4SOIud;
instanceStruct.d_instanceVar_pParam_B4SOIud_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIud1=in.pParam_B4SOIud1;
instanceStruct.d_instanceVar_pParam_B4SOIud1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIeu=in.pParam_B4SOIeu;
instanceStruct.d_instanceVar_pParam_B4SOIeu_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIucs=in.pParam_B4SOIucs;
instanceStruct.d_instanceVar_pParam_B4SOIucs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIucste=in.pParam_B4SOIucste;
instanceStruct.d_instanceVar_pParam_B4SOIucste_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvoff=in.pParam_B4SOIvoff;
instanceStruct.d_instanceVar_pParam_B4SOIvoff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvfb=in.pParam_B4SOIvfb;
instanceStruct.d_instanceVar_pParam_B4SOIvfb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuatemp=in.pParam_B4SOIuatemp;
instanceStruct.d_instanceVar_pParam_B4SOIuatemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIubtemp=in.pParam_B4SOIubtemp;
instanceStruct.d_instanceVar_pParam_B4SOIubtemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIuctemp=in.pParam_B4SOIuctemp;
instanceStruct.d_instanceVar_pParam_B4SOIuctemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrbody=in.pParam_B4SOIrbody;
instanceStruct.d_instanceVar_pParam_B4SOIrbody_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrth=in.pParam_B4SOIrth;
instanceStruct.d_instanceVar_pParam_B4SOIrth_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcth=in.pParam_B4SOIcth;
instanceStruct.d_instanceVar_pParam_B4SOIcth_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrds0denom=in.pParam_B4SOIrds0denom;
instanceStruct.d_instanceVar_pParam_B4SOIrds0denom_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvfbb=in.pParam_B4SOIvfbb;
instanceStruct.d_instanceVar_pParam_B4SOIvfbb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjbjts=in.pParam_B4SOIjbjts;
instanceStruct.d_instanceVar_pParam_B4SOIjbjts_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjbjtd=in.pParam_B4SOIjbjtd;
instanceStruct.d_instanceVar_pParam_B4SOIjbjtd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjdifs=in.pParam_B4SOIjdifs;
instanceStruct.d_instanceVar_pParam_B4SOIjdifs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjdifd=in.pParam_B4SOIjdifd;
instanceStruct.d_instanceVar_pParam_B4SOIjdifd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjrecs=in.pParam_B4SOIjrecs;
instanceStruct.d_instanceVar_pParam_B4SOIjrecs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjrecd=in.pParam_B4SOIjrecd;
instanceStruct.d_instanceVar_pParam_B4SOIjrecd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjtuns=in.pParam_B4SOIjtuns;
instanceStruct.d_instanceVar_pParam_B4SOIjtuns_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIjtund=in.pParam_B4SOIjtund;
instanceStruct.d_instanceVar_pParam_B4SOIjtund_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrdw=in.pParam_B4SOIrdw;
instanceStruct.d_instanceVar_pParam_B4SOIrdw_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrsw=in.pParam_B4SOIrsw;
instanceStruct.d_instanceVar_pParam_B4SOIrsw_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrdwmin=in.pParam_B4SOIrdwmin;
instanceStruct.d_instanceVar_pParam_B4SOIrdwmin_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrswmin=in.pParam_B4SOIrswmin;
instanceStruct.d_instanceVar_pParam_B4SOIrswmin_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrd0=in.pParam_B4SOIrd0;
instanceStruct.d_instanceVar_pParam_B4SOIrd0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrs0=in.pParam_B4SOIrs0;
instanceStruct.d_instanceVar_pParam_B4SOIrs0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsdt1=in.pParam_B4SOIsdt1;
instanceStruct.d_instanceVar_pParam_B4SOIsdt1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIst2=in.pParam_B4SOIst2;
instanceStruct.d_instanceVar_pParam_B4SOIst2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIst3=in.pParam_B4SOIst3;
instanceStruct.d_instanceVar_pParam_B4SOIst3_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdt2=in.pParam_B4SOIdt2;
instanceStruct.d_instanceVar_pParam_B4SOIdt2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdt3=in.pParam_B4SOIdt3;
instanceStruct.d_instanceVar_pParam_B4SOIdt3_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdelta=in.pParam_B4SOIdelta;
instanceStruct.d_instanceVar_pParam_B4SOIdelta_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrdsw=in.pParam_B4SOIrdsw;
instanceStruct.d_instanceVar_pParam_B4SOIrdsw_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrds0=in.pParam_B4SOIrds0;
instanceStruct.d_instanceVar_pParam_B4SOIrds0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIprwg=in.pParam_B4SOIprwg;
instanceStruct.d_instanceVar_pParam_B4SOIprwg_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIprwb=in.pParam_B4SOIprwb;
instanceStruct.d_instanceVar_pParam_B4SOIprwb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIprt=in.pParam_B4SOIprt;
instanceStruct.d_instanceVar_pParam_B4SOIprt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIeta0=in.pParam_B4SOIeta0;
instanceStruct.d_instanceVar_pParam_B4SOIeta0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIetab=in.pParam_B4SOIetab;
instanceStruct.d_instanceVar_pParam_B4SOIetab_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIeta0cv=in.pParam_B4SOIeta0cv;
instanceStruct.d_instanceVar_pParam_B4SOIeta0cv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIetabcv=in.pParam_B4SOIetabcv;
instanceStruct.d_instanceVar_pParam_B4SOIetabcv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpclm=in.pParam_B4SOIpclm;
instanceStruct.d_instanceVar_pParam_B4SOIpclm_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdibl1=in.pParam_B4SOIpdibl1;
instanceStruct.d_instanceVar_pParam_B4SOIpdibl1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdibl2=in.pParam_B4SOIpdibl2;
instanceStruct.d_instanceVar_pParam_B4SOIpdibl2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdiblb=in.pParam_B4SOIpdiblb;
instanceStruct.d_instanceVar_pParam_B4SOIpdiblb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpvag=in.pParam_B4SOIpvag;
instanceStruct.d_instanceVar_pParam_B4SOIpvag_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdwg=in.pParam_B4SOIdwg;
instanceStruct.d_instanceVar_pParam_B4SOIdwg_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdwb=in.pParam_B4SOIdwb;
instanceStruct.d_instanceVar_pParam_B4SOIdwb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIb0=in.pParam_B4SOIb0;
instanceStruct.d_instanceVar_pParam_B4SOIb0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIb1=in.pParam_B4SOIb1;
instanceStruct.d_instanceVar_pParam_B4SOIb1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIalpha0=in.pParam_B4SOIalpha0;
instanceStruct.d_instanceVar_pParam_B4SOIalpha0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbeta0=in.pParam_B4SOIbeta0;
instanceStruct.d_instanceVar_pParam_B4SOIbeta0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIaigc=in.pParam_B4SOIaigc;
instanceStruct.d_instanceVar_pParam_B4SOIaigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbigc=in.pParam_B4SOIbigc;
instanceStruct.d_instanceVar_pParam_B4SOIbigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcigc=in.pParam_B4SOIcigc;
instanceStruct.d_instanceVar_pParam_B4SOIcigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIaigsd=in.pParam_B4SOIaigsd;
instanceStruct.d_instanceVar_pParam_B4SOIaigsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbigsd=in.pParam_B4SOIbigsd;
instanceStruct.d_instanceVar_pParam_B4SOIbigsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcigsd=in.pParam_B4SOIcigsd;
instanceStruct.d_instanceVar_pParam_B4SOIcigsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInigc=in.pParam_B4SOInigc;
instanceStruct.d_instanceVar_pParam_B4SOInigc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpigcd=in.pParam_B4SOIpigcd;
instanceStruct.d_instanceVar_pParam_B4SOIpigcd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpoxedge=in.pParam_B4SOIpoxedge;
instanceStruct.d_instanceVar_pParam_B4SOIpoxedge_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbsa=in.pParam_B4SOIvbsa;
instanceStruct.d_instanceVar_pParam_B4SOIvbsa_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsce=in.pParam_B4SOIvsce;
instanceStruct.d_instanceVar_pParam_B4SOIvsce_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdsbs=in.pParam_B4SOIcdsbs;
instanceStruct.d_instanceVar_pParam_B4SOIcdsbs_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInofffd=in.pParam_B4SOInofffd;
instanceStruct.d_instanceVar_pParam_B4SOInofffd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvofffd=in.pParam_B4SOIvofffd;
instanceStruct.d_instanceVar_pParam_B4SOIvofffd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1b=in.pParam_B4SOIk1b;
instanceStruct.d_instanceVar_pParam_B4SOIk1b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk2b=in.pParam_B4SOIk2b;
instanceStruct.d_instanceVar_pParam_B4SOIk2b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdk2b=in.pParam_B4SOIdk2b;
instanceStruct.d_instanceVar_pParam_B4SOIdk2b_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvbd0=in.pParam_B4SOIdvbd0;
instanceStruct.d_instanceVar_pParam_B4SOIdvbd0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvbd1=in.pParam_B4SOIdvbd1;
instanceStruct.d_instanceVar_pParam_B4SOIdvbd1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImoinfd=in.pParam_B4SOImoinfd;
instanceStruct.d_instanceVar_pParam_B4SOImoinfd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrcrg1=in.pParam_B4SOIxrcrg1;
instanceStruct.d_instanceVar_pParam_B4SOIxrcrg1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrcrg2=in.pParam_B4SOIxrcrg2;
instanceStruct.d_instanceVar_pParam_B4SOIxrcrg2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIku0=in.pParam_B4SOIku0;
instanceStruct.d_instanceVar_pParam_B4SOIku0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIinv_od_ref=in.pParam_B4SOIinv_od_ref;
instanceStruct.d_instanceVar_pParam_B4SOIinv_od_ref_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgsl=in.pParam_B4SOIcgsl;
instanceStruct.d_instanceVar_pParam_B4SOIcgsl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgdl=in.pParam_B4SOIcgdl;
instanceStruct.d_instanceVar_pParam_B4SOIcgdl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIckappa=in.pParam_B4SOIckappa;
instanceStruct.d_instanceVar_pParam_B4SOIckappa_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIclc=in.pParam_B4SOIclc;
instanceStruct.d_instanceVar_pParam_B4SOIclc_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxj=in.pParam_B4SOIxj;
instanceStruct.d_instanceVar_pParam_B4SOIxj_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIalphaGB1=in.pParam_B4SOIalphaGB1;
instanceStruct.d_instanceVar_pParam_B4SOIalphaGB1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbetaGB1=in.pParam_B4SOIbetaGB1;
instanceStruct.d_instanceVar_pParam_B4SOIbetaGB1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIalphaGB2=in.pParam_B4SOIalphaGB2;
instanceStruct.d_instanceVar_pParam_B4SOIalphaGB2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbetaGB2=in.pParam_B4SOIbetaGB2;
instanceStruct.d_instanceVar_pParam_B4SOIbetaGB2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIaigbcp2=in.pParam_B4SOIaigbcp2;
instanceStruct.d_instanceVar_pParam_B4SOIaigbcp2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbigbcp2=in.pParam_B4SOIbigbcp2;
instanceStruct.d_instanceVar_pParam_B4SOIbigbcp2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcigbcp2=in.pParam_B4SOIcigbcp2;
instanceStruct.d_instanceVar_pParam_B4SOIcigbcp2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntrecf=in.pParam_B4SOIntrecf;
instanceStruct.d_instanceVar_pParam_B4SOIntrecf_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntrecr=in.pParam_B4SOIntrecr;
instanceStruct.d_instanceVar_pParam_B4SOIntrecr_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxbjt=in.pParam_B4SOIxbjt;
instanceStruct.d_instanceVar_pParam_B4SOIxbjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxdif=in.pParam_B4SOIxdif;
instanceStruct.d_instanceVar_pParam_B4SOIxdif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrec=in.pParam_B4SOIxrec;
instanceStruct.d_instanceVar_pParam_B4SOIxrec_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxtun=in.pParam_B4SOIxtun;
instanceStruct.d_instanceVar_pParam_B4SOIxtun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxdifd=in.pParam_B4SOIxdifd;
instanceStruct.d_instanceVar_pParam_B4SOIxdifd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxrecd=in.pParam_B4SOIxrecd;
instanceStruct.d_instanceVar_pParam_B4SOIxrecd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIxtund=in.pParam_B4SOIxtund;
instanceStruct.d_instanceVar_pParam_B4SOIxtund_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkb1=in.pParam_B4SOIkb1;
instanceStruct.d_instanceVar_pParam_B4SOIkb1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1w1=in.pParam_B4SOIk1w1;
instanceStruct.d_instanceVar_pParam_B4SOIk1w1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1w2=in.pParam_B4SOIk1w2;
instanceStruct.d_instanceVar_pParam_B4SOIk1w2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIketas=in.pParam_B4SOIketas;
instanceStruct.d_instanceVar_pParam_B4SOIketas_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfbjtii=in.pParam_B4SOIfbjtii;
instanceStruct.d_instanceVar_pParam_B4SOIfbjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIebjtii=in.pParam_B4SOIebjtii;
instanceStruct.d_instanceVar_pParam_B4SOIebjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcbjtii=in.pParam_B4SOIcbjtii;
instanceStruct.d_instanceVar_pParam_B4SOIcbjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvbci=in.pParam_B4SOIvbci;
instanceStruct.d_instanceVar_pParam_B4SOIvbci_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIabjtii=in.pParam_B4SOIabjtii;
instanceStruct.d_instanceVar_pParam_B4SOIabjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImbjtii=in.pParam_B4SOImbjtii;
instanceStruct.d_instanceVar_pParam_B4SOImbjtii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbeta1=in.pParam_B4SOIbeta1;
instanceStruct.d_instanceVar_pParam_B4SOIbeta1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbeta2=in.pParam_B4SOIbeta2;
instanceStruct.d_instanceVar_pParam_B4SOIbeta2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvdsatii0=in.pParam_B4SOIvdsatii0;
instanceStruct.d_instanceVar_pParam_B4SOIvdsatii0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlii=in.pParam_B4SOIlii;
instanceStruct.d_instanceVar_pParam_B4SOIlii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIesatii=in.pParam_B4SOIesatii;
instanceStruct.d_instanceVar_pParam_B4SOIesatii_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsii0=in.pParam_B4SOIsii0;
instanceStruct.d_instanceVar_pParam_B4SOIsii0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsii1=in.pParam_B4SOIsii1;
instanceStruct.d_instanceVar_pParam_B4SOIsii1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsii2=in.pParam_B4SOIsii2;
instanceStruct.d_instanceVar_pParam_B4SOIsii2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsiid=in.pParam_B4SOIsiid;
instanceStruct.d_instanceVar_pParam_B4SOIsiid_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIagidl=in.pParam_B4SOIagidl;
instanceStruct.d_instanceVar_pParam_B4SOIagidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbgidl=in.pParam_B4SOIbgidl;
instanceStruct.d_instanceVar_pParam_B4SOIbgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgidl=in.pParam_B4SOIcgidl;
instanceStruct.d_instanceVar_pParam_B4SOIcgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIegidl=in.pParam_B4SOIegidl;
instanceStruct.d_instanceVar_pParam_B4SOIegidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrgidl=in.pParam_B4SOIrgidl;
instanceStruct.d_instanceVar_pParam_B4SOIrgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkgidl=in.pParam_B4SOIkgidl;
instanceStruct.d_instanceVar_pParam_B4SOIkgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfgidl=in.pParam_B4SOIfgidl;
instanceStruct.d_instanceVar_pParam_B4SOIfgidl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIagisl=in.pParam_B4SOIagisl;
instanceStruct.d_instanceVar_pParam_B4SOIagisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIbgisl=in.pParam_B4SOIbgisl;
instanceStruct.d_instanceVar_pParam_B4SOIbgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgisl=in.pParam_B4SOIcgisl;
instanceStruct.d_instanceVar_pParam_B4SOIcgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIegisl=in.pParam_B4SOIegisl;
instanceStruct.d_instanceVar_pParam_B4SOIegisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIrgisl=in.pParam_B4SOIrgisl;
instanceStruct.d_instanceVar_pParam_B4SOIrgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIkgisl=in.pParam_B4SOIkgisl;
instanceStruct.d_instanceVar_pParam_B4SOIkgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfgisl=in.pParam_B4SOIfgisl;
instanceStruct.d_instanceVar_pParam_B4SOIfgisl_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntun=in.pParam_B4SOIntun;
instanceStruct.d_instanceVar_pParam_B4SOIntun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIntund=in.pParam_B4SOIntund;
instanceStruct.d_instanceVar_pParam_B4SOIntund_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIndiode=in.pParam_B4SOIndiode;
instanceStruct.d_instanceVar_pParam_B4SOIndiode_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIndioded=in.pParam_B4SOIndioded;
instanceStruct.d_instanceVar_pParam_B4SOIndioded_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecf0=in.pParam_B4SOInrecf0;
instanceStruct.d_instanceVar_pParam_B4SOInrecf0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecf0d=in.pParam_B4SOInrecf0d;
instanceStruct.d_instanceVar_pParam_B4SOInrecf0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecr0=in.pParam_B4SOInrecr0;
instanceStruct.d_instanceVar_pParam_B4SOInrecr0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInrecr0d=in.pParam_B4SOInrecr0d;
instanceStruct.d_instanceVar_pParam_B4SOInrecr0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIisbjt=in.pParam_B4SOIisbjt;
instanceStruct.d_instanceVar_pParam_B4SOIisbjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIidbjt=in.pParam_B4SOIidbjt;
instanceStruct.d_instanceVar_pParam_B4SOIidbjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIisdif=in.pParam_B4SOIisdif;
instanceStruct.d_instanceVar_pParam_B4SOIisdif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIiddif=in.pParam_B4SOIiddif;
instanceStruct.d_instanceVar_pParam_B4SOIiddif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIisrec=in.pParam_B4SOIisrec;
instanceStruct.d_instanceVar_pParam_B4SOIisrec_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIidrec=in.pParam_B4SOIidrec;
instanceStruct.d_instanceVar_pParam_B4SOIidrec_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIistun=in.pParam_B4SOIistun;
instanceStruct.d_instanceVar_pParam_B4SOIistun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIidtun=in.pParam_B4SOIidtun;
instanceStruct.d_instanceVar_pParam_B4SOIidtun_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvrec0=in.pParam_B4SOIvrec0;
instanceStruct.d_instanceVar_pParam_B4SOIvrec0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvrec0d=in.pParam_B4SOIvrec0d;
instanceStruct.d_instanceVar_pParam_B4SOIvrec0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvtun0=in.pParam_B4SOIvtun0;
instanceStruct.d_instanceVar_pParam_B4SOIvtun0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvtun0d=in.pParam_B4SOIvtun0d;
instanceStruct.d_instanceVar_pParam_B4SOIvtun0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsdfb=in.pParam_B4SOIvsdfb;
instanceStruct.d_instanceVar_pParam_B4SOIvsdfb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsdth=in.pParam_B4SOIvsdth;
instanceStruct.d_instanceVar_pParam_B4SOIvsdth_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdelvt=in.pParam_B4SOIdelvt;
instanceStruct.d_instanceVar_pParam_B4SOIdelvt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleff=in.pParam_B4SOIleff;
instanceStruct.d_instanceVar_pParam_B4SOIleff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIweff=in.pParam_B4SOIweff;
instanceStruct.d_instanceVar_pParam_B4SOIweff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleffCV=in.pParam_B4SOIleffCV;
instanceStruct.d_instanceVar_pParam_B4SOIleffCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIweffCV=in.pParam_B4SOIweffCV;
instanceStruct.d_instanceVar_pParam_B4SOIweffCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIabulkCVfactor=in.pParam_B4SOIabulkCVfactor;
instanceStruct.d_instanceVar_pParam_B4SOIabulkCVfactor_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgso=in.pParam_B4SOIcgso;
instanceStruct.d_instanceVar_pParam_B4SOIcgso_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgdo=in.pParam_B4SOIcgdo;
instanceStruct.d_instanceVar_pParam_B4SOIcgdo_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcgeo=in.pParam_B4SOIcgeo;
instanceStruct.d_instanceVar_pParam_B4SOIcgeo_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIu0temp=in.pParam_B4SOIu0temp;
instanceStruct.d_instanceVar_pParam_B4SOIu0temp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvsattemp=in.pParam_B4SOIvsattemp;
instanceStruct.d_instanceVar_pParam_B4SOIvsattemp_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIsqrtPhi=in.pParam_B4SOIsqrtPhi;
instanceStruct.d_instanceVar_pParam_B4SOIsqrtPhi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIXdep0=in.pParam_B4SOIXdep0;
instanceStruct.d_instanceVar_pParam_B4SOIXdep0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOItheta0vb0=in.pParam_B4SOItheta0vb0;
instanceStruct.d_instanceVar_pParam_B4SOItheta0vb0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIthetaRout=in.pParam_B4SOIthetaRout;
instanceStruct.d_instanceVar_pParam_B4SOIthetaRout_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIqsi=in.pParam_B4SOIqsi;
instanceStruct.d_instanceVar_pParam_B4SOIqsi_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIoxideRatio=in.pParam_B4SOIoxideRatio;
instanceStruct.d_instanceVar_pParam_B4SOIoxideRatio_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1eff=in.pParam_B4SOIk1eff;
instanceStruct.d_instanceVar_pParam_B4SOIk1eff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdios=in.pParam_B4SOIwdios;
instanceStruct.d_instanceVar_pParam_B4SOIwdios_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdiod=in.pParam_B4SOIwdiod;
instanceStruct.d_instanceVar_pParam_B4SOIwdiod_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdiodCV=in.pParam_B4SOIwdiodCV;
instanceStruct.d_instanceVar_pParam_B4SOIwdiodCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIwdiosCV=in.pParam_B4SOIwdiosCV;
instanceStruct.d_instanceVar_pParam_B4SOIwdiosCV_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIarfabjt=in.pParam_B4SOIarfabjt;
instanceStruct.d_instanceVar_pParam_B4SOIarfabjt_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlratio=in.pParam_B4SOIlratio;
instanceStruct.d_instanceVar_pParam_B4SOIlratio_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlratiodif=in.pParam_B4SOIlratiodif;
instanceStruct.d_instanceVar_pParam_B4SOIlratiodif_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvearly=in.pParam_B4SOIvearly;
instanceStruct.d_instanceVar_pParam_B4SOIvearly_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahli=in.pParam_B4SOIahli;
instanceStruct.d_instanceVar_pParam_B4SOIahli_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahlid=in.pParam_B4SOIahlid;
instanceStruct.d_instanceVar_pParam_B4SOIahlid_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahli0s=in.pParam_B4SOIahli0s;
instanceStruct.d_instanceVar_pParam_B4SOIahli0s_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIahli0d=in.pParam_B4SOIahli0d;
instanceStruct.d_instanceVar_pParam_B4SOIahli0d_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIldeb=in.pParam_B4SOIldeb;
instanceStruct.d_instanceVar_pParam_B4SOIldeb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIacde=in.pParam_B4SOIacde;
instanceStruct.d_instanceVar_pParam_B4SOIacde_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImoin=in.pParam_B4SOImoin;
instanceStruct.d_instanceVar_pParam_B4SOImoin_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInoff=in.pParam_B4SOInoff;
instanceStruct.d_instanceVar_pParam_B4SOInoff_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOInoff2=in.pParam_B4SOInoff2;
instanceStruct.d_instanceVar_pParam_B4SOInoff2_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleffCVb=in.pParam_B4SOIleffCVb;
instanceStruct.d_instanceVar_pParam_B4SOIleffCVb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIleffCVbg=in.pParam_B4SOIleffCVbg;
instanceStruct.d_instanceVar_pParam_B4SOIleffCVbg_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIcdep0=in.pParam_B4SOIcdep0;
instanceStruct.d_instanceVar_pParam_B4SOIcdep0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIAechvb=in.pParam_B4SOIAechvb;
instanceStruct.d_instanceVar_pParam_B4SOIAechvb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIBechvb=in.pParam_B4SOIBechvb;
instanceStruct.d_instanceVar_pParam_B4SOIBechvb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIAechvbEdges=in.pParam_B4SOIAechvbEdges;
instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdges_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIAechvbEdged=in.pParam_B4SOIAechvbEdged;
instanceStruct.d_instanceVar_pParam_B4SOIAechvbEdged_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIBechvbEdge=in.pParam_B4SOIBechvbEdge;
instanceStruct.d_instanceVar_pParam_B4SOIBechvbEdge_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvfbsd=in.pParam_B4SOIvfbsd;
instanceStruct.d_instanceVar_pParam_B4SOIvfbsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIk1ox=in.pParam_B4SOIk1ox;
instanceStruct.d_instanceVar_pParam_B4SOIk1ox_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIlpeb=in.pParam_B4SOIlpeb;
instanceStruct.d_instanceVar_pParam_B4SOIlpeb_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp0=in.pParam_B4SOIdvtp0;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp0_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp1=in.pParam_B4SOIdvtp1;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp1_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp4=in.pParam_B4SOIdvtp4;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp4_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImstar=in.pParam_B4SOImstar;
instanceStruct.d_instanceVar_pParam_B4SOImstar_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIfprout=in.pParam_B4SOIfprout;
instanceStruct.d_instanceVar_pParam_B4SOIfprout_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpdits=in.pParam_B4SOIpdits;
instanceStruct.d_instanceVar_pParam_B4SOIpdits_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIpditsd=in.pParam_B4SOIpditsd;
instanceStruct.d_instanceVar_pParam_B4SOIpditsd_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOImstarcv=in.pParam_B4SOImstarcv;
instanceStruct.d_instanceVar_pParam_B4SOImstarcv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIvoffcv=in.pParam_B4SOIvoffcv;
instanceStruct.d_instanceVar_pParam_B4SOIvoffcv_dX=0.0;
instanceStruct.instanceVar_pParam_B4SOIdvtp2factor=in.pParam_B4SOIdvtp2factor;
instanceStruct.d_instanceVar_pParam_B4SOIdvtp2factor_dX=0.0;
instanceStruct.instanceVar_B4SOIcbox=in.B4SOIcbox;
instanceStruct.d_instanceVar_B4SOIcbox_dX=0.0;
instanceStruct.instanceVar_B4SOIcsi=in.B4SOIcsi;
instanceStruct.d_instanceVar_B4SOIcsi_dX=0.0;
instanceStruct.instanceVar_B4SOIeg0=in.B4SOIeg0;
instanceStruct.d_instanceVar_B4SOIeg0_dX=0.0;
instanceStruct.instanceVar_B4SOIeg=in.B4SOIeg;
instanceStruct.d_instanceVar_B4SOIeg_dX=0.0;
instanceStruct.instanceVar_B4SOIcox=in.B4SOIcox;
instanceStruct.d_instanceVar_B4SOIcox_dX=0.0;
instanceStruct.instanceVar_B4SOIfactor1=in.B4SOIfactor1;
instanceStruct.d_instanceVar_B4SOIfactor1_dX=0.0;
instanceStruct.instanceVar_here_B4SOIeta0=in.here_B4SOIeta0;
instanceStruct.d_instanceVar_here_B4SOIeta0_dX=0.0;
instanceStruct.instanceVar_here_B4SOIeta0cv=in.here_B4SOIeta0cv;
instanceStruct.d_instanceVar_here_B4SOIeta0cv_dX=0.0;
instanceStruct.instanceVar_here_B4SOIk2=in.here_B4SOIk2;
instanceStruct.d_instanceVar_here_B4SOIk2_dX=0.0;
instanceStruct.instanceVar_here_B4SOIk2ox=in.here_B4SOIk2ox;
instanceStruct.d_instanceVar_here_B4SOIk2ox_dX=0.0;
instanceStruct.instanceVar_here_B4SOIu0temp=in.here_B4SOIu0temp;
instanceStruct.d_instanceVar_here_B4SOIu0temp_dX=0.0;
instanceStruct.instanceVar_here_B4SOIvfb=in.here_B4SOIvfb;
instanceStruct.d_instanceVar_here_B4SOIvfb_dX=0.0;
instanceStruct.instanceVar_here_B4SOIvsattemp=in.here_B4SOIvsattemp;
instanceStruct.d_instanceVar_here_B4SOIvsattemp_dX=0.0;
instanceStruct.instanceVar_here_B4SOIvth0=in.here_B4SOIvth0;
instanceStruct.d_instanceVar_here_B4SOIvth0_dX=0.0;
instanceStruct.instanceVar_DevTemp=in.DevTemp;
instanceStruct.d_instanceVar_DevTemp_dX=0.0;
instanceStruct.instanceVar_fourkt=in.fourkt;
instanceStruct.d_instanceVar_fourkt_dX=0.0;
instanceStruct.instanceVar_gdnoise=in.gdnoise;
instanceStruct.d_instanceVar_gdnoise_dX=0.0;
instanceStruct.instanceVar_gsnoise=in.gsnoise;
instanceStruct.d_instanceVar_gsnoise_dX=0.0;
instanceStruct.instanceVar_toxe=in.toxe;
instanceStruct.d_instanceVar_toxe_dX=0.0;
instanceStruct.instanceVar_epsrox=in.epsrox;
instanceStruct.d_instanceVar_epsrox_dX=0.0;
instanceStruct.instanceVar_epssub=in.epssub;
instanceStruct.d_instanceVar_epssub_dX=0.0;
instanceStruct.instanceVar_sqrt2qeps=in.sqrt2qeps;
instanceStruct.d_instanceVar_sqrt2qeps_dX=0.0;
instanceStruct.instanceVar_eggbcp2=in.eggbcp2;
instanceStruct.d_instanceVar_eggbcp2_dX=0.0;
instanceStruct.instanceVar_eggdep=in.eggdep;
instanceStruct.d_instanceVar_eggdep_dX=0.0;
instanceStruct.instanceVar_Vtm00=in.Vtm00;
instanceStruct.d_instanceVar_Vtm00_dX=0.0;
instanceStruct.instanceVar_agb1=in.agb1;
instanceStruct.d_instanceVar_agb1_dX=0.0;
instanceStruct.instanceVar_bgb1=in.bgb1;
instanceStruct.d_instanceVar_bgb1_dX=0.0;
instanceStruct.instanceVar_agb2=in.agb2;
instanceStruct.d_instanceVar_agb2_dX=0.0;
instanceStruct.instanceVar_bgb2=in.bgb2;
instanceStruct.d_instanceVar_bgb2_dX=0.0;
instanceStruct.instanceVar_agbc2n=in.agbc2n;
instanceStruct.d_instanceVar_agbc2n_dX=0.0;
instanceStruct.instanceVar_agbc2p=in.agbc2p;
instanceStruct.d_instanceVar_agbc2p_dX=0.0;
instanceStruct.instanceVar_bgbc2n=in.bgbc2n;
instanceStruct.d_instanceVar_bgbc2n_dX=0.0;
instanceStruct.instanceVar_bgbc2p=in.bgbc2p;
instanceStruct.d_instanceVar_bgbc2p_dX=0.0;


  //non-reals
instanceStruct.instanceVar_B4SOItype=in.B4SOItype;
instanceStruct.instanceVar_B4SOIbjtoff=in.B4SOIbjtoff;
instanceStruct.instanceVar_B4SOIsoiMod=in.B4SOIsoiMod;
instanceStruct.instanceVar_B4SOIrgateMod=in.B4SOIrgateMod;
instanceStruct.instanceVar_B4SOIrbodyMod=in.B4SOIrbodyMod;
instanceStruct.instanceVar_B4SOImtrlMod=in.B4SOImtrlMod;
instanceStruct.instanceVar_B4SOIvgstcvMod=in.B4SOIvgstcvMod;
instanceStruct.instanceVar_B4SOIgidlMod=in.B4SOIgidlMod;
instanceStruct.instanceVar_B4SOIiiiMod=in.B4SOIiiiMod;
instanceStruct.instanceVar_B4SOIcapMod=in.B4SOIcapMod;
instanceStruct.instanceVar_B4SOImobMod=in.B4SOImobMod;
instanceStruct.instanceVar_B4SOIigbMod=in.B4SOIigbMod;
instanceStruct.instanceVar_B4SOIigcMod=in.B4SOIigcMod;
instanceStruct.instanceVar_B4SOIrdsMod=in.B4SOIrdsMod;
instanceStruct.instanceVar_B4SOIfdMod=in.B4SOIfdMod;
instanceStruct.instanceVar_B4SOImode=in.B4SOImode;
instanceStruct.instanceVar_B4SOIbodyMod=in.B4SOIbodyMod;



  Linear::Vector * solVectorPtr = in.extData.nextSolVectorPtr;

  // extract solution variables
probeVars[in.admsProbeID_V_sb_GND] = (*solVectorPtr)[in.li_sb];
probeVars[in.admsProbeID_V_db_GND] = (*solVectorPtr)[in.li_db];
probeVars[in.admsProbeID_V_gm_GND] = (*solVectorPtr)[in.li_gm];
probeVars[in.admsProbeID_V_b_sb] = (*solVectorPtr)[in.li_b] - (*solVectorPtr)[in.li_sb];
probeVars[in.admsProbeID_V_b_db] = (*solVectorPtr)[in.li_b] - (*solVectorPtr)[in.li_db];
probeVars[in.admsProbeID_V_gm_gi] = (*solVectorPtr)[in.li_gm] - (*solVectorPtr)[in.li_gi];
probeVars[in.admsProbeID_V_g_gm] = (*solVectorPtr)[in.li_g] - (*solVectorPtr)[in.li_gm];
probeVars[in.admsProbeID_V_gi_e] = (*solVectorPtr)[in.li_gi] - (*solVectorPtr)[in.li_e];
probeVars[in.admsProbeID_V_gm_e] = (*solVectorPtr)[in.li_gm] - (*solVectorPtr)[in.li_e];
probeVars[in.admsProbeID_V_s_si] = (*solVectorPtr)[in.li_s] - (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_d_di] = (*solVectorPtr)[in.li_d] - (*solVectorPtr)[in.li_di];
probeVars[in.admsProbeID_V_si_GND] = (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_di_GND] = (*solVectorPtr)[in.li_di];
probeVars[in.admsProbeID_V_gi_GND] = (*solVectorPtr)[in.li_gi];
probeVars[in.admsProbeID_V_gm_si] = (*solVectorPtr)[in.li_gm] - (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_db_di] = (*solVectorPtr)[in.li_db] - (*solVectorPtr)[in.li_di];
probeVars[in.admsProbeID_V_sb_si] = (*solVectorPtr)[in.li_sb] - (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_gi_p] = (*solVectorPtr)[in.li_gi] - (*solVectorPtr)[in.li_p];
probeVars[in.admsProbeID_V_b_p] = (*solVectorPtr)[in.li_b] - (*solVectorPtr)[in.li_p];
probeVars[in.admsProbeID_V_e_si] = (*solVectorPtr)[in.li_e] - (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_gi_si] = (*solVectorPtr)[in.li_gi] - (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_b_si] = (*solVectorPtr)[in.li_b] - (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_V_di_si] = (*solVectorPtr)[in.li_di] - (*solVectorPtr)[in.li_si];
probeVars[in.admsProbeID_Temp_t_GND] = (in.collapseNode_t)?(0):((*solVectorPtr)[in.li_t]);
probeVars[in.admsProbeID_V_p_GND] = (*solVectorPtr)[in.li_p];
probeVars[in.admsProbeID_V_b_GND] = (*solVectorPtr)[in.li_b];


evaluateInitialInstance(
instanceStruct,
modelStruct,
in.admsNodeID_d,
in.admsNodeID_g,
in.admsNodeID_s,
in.admsNodeID_e,
in.admsNodeID_p,
in.admsNodeID_b,
in.admsNodeID_t,
in.admsNodeID_di,
in.admsNodeID_si,
in.admsNodeID_gi,
in.admsNodeID_gm,
in.admsNodeID_sb,
in.admsNodeID_db,
 in.portsConnected_,
in.admsTemperature, in.adms_vt_nom,in.getDeviceOptions().gmin,in);


// Now call  the function that does the heavy lifting.

evaluateModelEquations(
probeVars,
// probe constants
in.admsProbeID_V_sb_GND,
in.admsProbeID_V_db_GND,
in.admsProbeID_V_gm_GND,
in.admsProbeID_V_b_sb,
in.admsProbeID_V_b_db,
in.admsProbeID_V_gm_gi,
in.admsProbeID_V_g_gm,
in.admsProbeID_V_gi_e,
in.admsProbeID_V_gm_e,
in.admsProbeID_V_s_si,
in.admsProbeID_V_d_di,
in.admsProbeID_V_si_GND,
in.admsProbeID_V_di_GND,
in.admsProbeID_V_gi_GND,
in.admsProbeID_V_gm_si,
in.admsProbeID_V_db_di,
in.admsProbeID_V_sb_si,
in.admsProbeID_V_gi_p,
in.admsProbeID_V_b_p,
in.admsProbeID_V_e_si,
in.admsProbeID_V_gi_si,
in.admsProbeID_V_b_si,
in.admsProbeID_V_di_si,
in.admsProbeID_Temp_t_GND,
in.admsProbeID_V_p_GND,
in.admsProbeID_V_b_GND,

// node constants
in.admsNodeID_d,
in.admsNodeID_g,
in.admsNodeID_s,
in.admsNodeID_e,
in.admsNodeID_p,
in.admsNodeID_b,
in.admsNodeID_t,
in.admsNodeID_di,
in.admsNodeID_si,
in.admsNodeID_gi,
in.admsNodeID_gm,
in.admsNodeID_sb,
in.admsNodeID_db,
instanceStruct,
modelStruct,
 in.portsConnected_,
 in.admsTemperature,
 in.adms_vt_nom,
 in.getDeviceOptions().gmin,
 d_staticContributions_dX,
 d_dynamicContributions_dX,
 in);


  // We now have the F and Q vector stuff, populate the dependencies:

    dfdp[in.admsNodeID_d+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_d];
    dqdp[in.admsNodeID_d+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_d];
    Findices[in.admsNodeID_d+inst*(13+0)] = in.li_d;
    Qindices[in.admsNodeID_d+inst*(13+0)] = in.li_d;
    dfdp[in.admsNodeID_g+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_g];
    dqdp[in.admsNodeID_g+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_g];
    Findices[in.admsNodeID_g+inst*(13+0)] = in.li_g;
    Qindices[in.admsNodeID_g+inst*(13+0)] = in.li_g;
    dfdp[in.admsNodeID_s+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_s];
    dqdp[in.admsNodeID_s+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_s];
    Findices[in.admsNodeID_s+inst*(13+0)] = in.li_s;
    Qindices[in.admsNodeID_s+inst*(13+0)] = in.li_s;
    dfdp[in.admsNodeID_e+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_e];
    dqdp[in.admsNodeID_e+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_e];
    Findices[in.admsNodeID_e+inst*(13+0)] = in.li_e;
    Qindices[in.admsNodeID_e+inst*(13+0)] = in.li_e;
    dfdp[in.admsNodeID_p+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_p];
    dqdp[in.admsNodeID_p+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_p];
    Findices[in.admsNodeID_p+inst*(13+0)] = in.li_p;
    Qindices[in.admsNodeID_p+inst*(13+0)] = in.li_p;
    dfdp[in.admsNodeID_b+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_b];
    dqdp[in.admsNodeID_b+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_b];
    Findices[in.admsNodeID_b+inst*(13+0)] = in.li_b;
    Qindices[in.admsNodeID_b+inst*(13+0)] = in.li_b;
    dfdp[in.admsNodeID_t+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_t];
    dqdp[in.admsNodeID_t+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_t];
    Findices[in.admsNodeID_t+inst*(13+0)] = in.li_t;
    Qindices[in.admsNodeID_t+inst*(13+0)] = in.li_t;
    dfdp[in.admsNodeID_di+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_di];
    dqdp[in.admsNodeID_di+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_di];
    Findices[in.admsNodeID_di+inst*(13+0)] = in.li_di;
    Qindices[in.admsNodeID_di+inst*(13+0)] = in.li_di;
    dfdp[in.admsNodeID_si+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_si];
    dqdp[in.admsNodeID_si+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_si];
    Findices[in.admsNodeID_si+inst*(13+0)] = in.li_si;
    Qindices[in.admsNodeID_si+inst*(13+0)] = in.li_si;
    dfdp[in.admsNodeID_gi+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_gi];
    dqdp[in.admsNodeID_gi+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_gi];
    Findices[in.admsNodeID_gi+inst*(13+0)] = in.li_gi;
    Qindices[in.admsNodeID_gi+inst*(13+0)] = in.li_gi;
    dfdp[in.admsNodeID_gm+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_gm];
    dqdp[in.admsNodeID_gm+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_gm];
    Findices[in.admsNodeID_gm+inst*(13+0)] = in.li_gm;
    Qindices[in.admsNodeID_gm+inst*(13+0)] = in.li_gm;
    dfdp[in.admsNodeID_sb+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_sb];
    dqdp[in.admsNodeID_sb+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_sb];
    Findices[in.admsNodeID_sb+inst*(13+0)] = in.li_sb;
    Qindices[in.admsNodeID_sb+inst*(13+0)] = in.li_sb;
    dfdp[in.admsNodeID_db+inst*(13+0)] += d_staticContributions_dX[in.admsNodeID_db];
    dqdp[in.admsNodeID_db+inst*(13+0)] += d_dynamicContributions_dX[in.admsNodeID_db];
    Findices[in.admsNodeID_db+inst*(13+0)] = in.li_db;
    Qindices[in.admsNodeID_db+inst*(13+0)] = in.li_db;
}

}

#endif // Xyce_ADMS_SENSITIVITIES

} // namespace ADMSbsimsoi
} // namespace Device
} // namespace Xyce
